/** \file mssm_twoloop_as.cpp
    Project: SOFTSUSY 
    Author: Ben Allanach and Alex Voigt
    Manual: B.C. Allanach, A. Bednyakov and R. Ruiz de Austri, 
    Comput. Phys. Commun. (2015) 192, arXiv:1407.6130.
    Webpage: http://hepforge.cedar.ac.uk/softsusy/
*/

#include "mssm_twoloop_as.h"
#include "numerics.h"
#include <algorithm>
#include <array>
#include <cmath>
#include <complex>
#include <limits>
#include <ostream>

namespace flexiblesusy {
namespace mssm_twoloop_as {

namespace {
   const Real Pi = 3.1415926535897932384626433832795l;

   template <typename T> T power2(T x)  { return x*x; }
   template <typename T> T power3(T x)  { return x*x*x; }
   template <typename T> T power4(T x)  { return x*x*x*x; }
   template <typename T> T power5(T x)  { return x*x*x*x*x; }
   template <typename T> T power6(T x)  { return x*x*x*x*x*x; }
   template <typename T> T power7(T x)  { return x*x*x*x*x*x*x; }
   template <typename T> T power8(T x)  { return x*x*x*x*x*x*x*x; }
   template <typename T> T power10(T x) { return x*x*x*x*x * x*x*x*x*x; }
   template <typename T> T power12(T x) { return x*x*x*x*x * x*x*x*x*x * x*x; }
   template <typename T> T power14(T x) { return x*x*x*x*x * x*x*x*x*x * x*x*x*x; }

   const Real oneLoop = 1.l/power2(4*Pi);
   const Real twoLoop = power2(oneLoop);

   template <typename T>
   bool is_zero(T a, T prec = std::numeric_limits<T>::epsilon())
   {
      return std::fabs(a) < prec;
   }

   template <typename T>
   bool is_equal(T a, T b, T prec = std::numeric_limits<T>::epsilon())
   {
      return is_zero(a - b, prec);
   }

   template <typename T>
   bool is_equal_rel(T a, T b, T prec = std::numeric_limits<T>::epsilon())
   {
      if (is_equal(a, b, std::numeric_limits<T>::epsilon()))
         return true;

      if (std::abs(a) < std::numeric_limits<T>::epsilon() ||
          std::abs(b) < std::numeric_limits<T>::epsilon())
         return false;

      return std::abs((a - b)/a) < prec;
   }

   Real LambdaSquared(Real x, Real y)
   {
      return power2(1 - x - y) - 4*x*y;
   }

   /// ClausenCl[2,x]
   Real ClausenCl2(Real x)
   {
      using std::exp;
      //      using gm2calc::dilog;
      const std::complex<Real> img(0.l,1.l);

      return std::imag(dilogcl(exp(img*x)));
   }

   /// x < 1 && y < 1, LambdaSquared(x,y) > 0
   Real PhiPos(Real x, Real y)
   {
     //      using gm2calc::dilog;
      const Real lambda = std::sqrt(LambdaSquared(x,y));

      return ((-(log(x)*log(y))
              + 2.l*log((1 - lambda + x - y)/2.)*log((1 - lambda - x + y)/2.)
              - 2.l*dilogcl((1 - lambda + x - y)/2.)
              - 2.l*dilogcl((1 - lambda - x + y)/2.)
	       + power2(Pi)/3.)/lambda).real();
   }

   /// LambdaSquared(x,y) < 0
   Real PhiNeg(Real x, Real y)
   {
      using std::acos;
      using std::sqrt;
      const Real lambda = std::sqrt(-LambdaSquared(x,y));

      return 2*(+ ClausenCl2(2*acos((1 + x - y)/(2.*sqrt(x))))
                + ClausenCl2(2*acos((1 - x + y)/(2.*sqrt(y))))
                + ClausenCl2(2*acos((-1 + x + y)/(2.*sqrt(x*y)))))/lambda;
   }

   Real Phi(Real x, Real y)
   {
      const Real lambda = LambdaSquared(x,y);

      if (lambda > 0.)
         return PhiPos(x,y);

      return PhiNeg(x,y);
   }

   /**
    * Fin3[] function from twoloopbubble.m .
    *
    * @param mm1 squared mass \f$m_1^2\f$
    * @param mm2 squared mass \f$m_2^2\f$
    * @param mm3 squared mass \f$m_3^2\f$
    * @param mmu squared renormalization scale
    *
    * @return Fin3(m12, m22, m32, mmu)
    */
   Real Fin3(Real mm1, Real mm2, Real mm3, Real mmu)
   {
      using std::log;

      std::array<Real,3> masses = { mm1, mm2, mm3 };
      std::sort(masses.begin(), masses.end());

      const Real mm = masses[2];
      const Real x = masses[0]/mm;
      const Real y = masses[1]/mm;

      const Real lambda = LambdaSquared(x,y);

      if (is_zero(lambda, 1e-10l)) {
         return -(mm*(2*y*(-3 + 2*log(mm/mmu))*log(y)
                      + log(x)*(2*x*(-3 + 2*log(mm/mmu)) + (-1 + x + y)*log(y))
                      + (1 + x + y)*(7 - 6*log(mm/mmu) + power2(Pi)/6. + 2*power2(log(mm/mmu)))
                      + x*power2(log(x)) + y*power2(log(y))))/2.;
      }

      return mm*((-7 + 6*log(mm/mmu) + log(x)*log(y)
                  - lambda*Phi(x,y) - power2(Pi)/6. - 2*power2(log(mm/mmu)))/2.
                 - (x*(7 - 6*log(mm/mmu) + log(x)*(-6 + 4*log(mm/mmu) + log(y))
                       + power2(Pi)/6. + 2*power2(log(mm/mmu)) + power2(log(x))))/2.
                 - (y*(7 - 6*log(mm/mmu) + (
                     -6 + 4*log(mm/mmu) + log(x))*log(y) + power2(Pi)/6.
                       + 2*power2(log(mm/mmu)) + power2(log(y))))/2.);
   }

   /// Delta[m1,m2,m3,-1]
   Real DeltaInv(Real m1, Real m2, Real m3)
   {
      return 1./(power2(m1) + power2(m2) + power2(m3) - 2*(m1*m2 + m1*m3 + m2*m3));
   }

   /// calculates sin(theta)
   Real calc_sin_theta(Real mf, Real xf, Real msf12, Real msf22)
   {
      if (is_zero(mf, 1e-10l) || is_zero(xf, 1e-10l))
         return 0.;

      const Real sin_2theta = 2.0*mf*xf / (msf12 - msf22);
      const Real theta = 0.5*std::asin(sin_2theta);

      return std::sin(theta);
   }

   /// calculates Higgs mixing angle from squarde Higgs masses and tan(beta)
   Real calc_alpha(Real mh2, Real mH2, Real tb)
   {
      const Real beta = std::atan(tb);
      const Real sin_2alpha = -(mH2 + mh2)/(mH2 - mh2) * std::sin(2*beta);

      return 0.5*std::asin(sin_2alpha);
   }

   /// shift gluino mass away from mst1 and mst2 if too close
   Real shift_mg(Real mg, Real mst1, Real mst2)
   {
      if (is_equal_rel(std::min(mst1, mst2), mg, 0.01l))
         return mg * 0.98l;

      if (is_equal_rel(std::max(mst1, mst2), mg, 0.01l))
         return mg * 1.02l;

      return mg;
   }

} // anonymous namespace

/// 2-loop O(alpha_s^2) contributions to Delta alpha_s [hep-ph/0509048,arXiv:0810.5101]
Real delta_alpha_s_2loop_as_as(const Parameters& pars)
{
   using std::log;
   const Real g3    = pars.g3;
   const Real xt    = pars.xt;
   const Real xb    = pars.xb;
   const Real mt    = pars.mt;
   const Real mt2   = power2(pars.mt);
   const Real mb    = pars.mb;
   const Real mg    = shift_mg(pars.mg, pars.mst1, pars.mst2);
   const Real mg2   = power2(mg);
   //   const Real mst1  = pars.mst1;
   const Real mst12 = power2(pars.mst1);
   const Real mst14 = power4(pars.mst1);
   //   const Real mst2  = pars.mst2;
   const Real mst22 = power2(pars.mst2);
   const Real mst24 = power4(pars.mst2);
   const Real msb12 = power2(pars.msb1);
   const Real msb22 = power2(pars.msb2);
   //   const Real msd1  = pars.msd1;
   const Real msd12 = power2(pars.msd1);
   const Real msd14 = power4(pars.msd1);
   //   const Real msd2  = pars.msd2;
   const Real msd22 = power2(pars.msd2);
   const Real msd24 = power4(pars.msd2);
   const Real Q2    = power2(pars.Q);
   const Real snt   = calc_sin_theta(mt, xt, mst12, mst22);
   const Real snb   = calc_sin_theta(mb, xb, msb12, msb22); 
   const Real tb    = pars.tb;  
   const Real sb    = tb / std::sqrt(1. + power2(tb));
   const Real cb    = 1. / std::sqrt(1. + power2(tb));
   const Real mu    = pars.mu;
   
   const Real result =
   -1.375 + (9*log(mg2/Q2))/2. + ((3*mg2 + msb12)*log(msb12/Q2))/(8.*(mg2 -
     msb12)) - (2*(-5 + (2*msb12)/(-mg2 + msb12))*log(msb12/Q2))/9. + (log(mg2/
     Q2)*log(msb12/Q2))/4. + ((3*mg2 + msb22)*log(msb22/Q2))/(8.*(mg2 - msb22))
     - (2*(-5 + (2*msb22)/(-mg2 + msb22))*log(msb22/Q2))/9. + (log(mg2/Q2)*log(
     msb22/Q2))/4. + (log(msb12/Q2)*log(msb22/Q2))/48. + log(mg2/Q2)*log(msd12/
     Q2) + (log(msb12/Q2)*log(msd12/Q2))/12. + (log(msb22/Q2)*log(msd12/Q2))/
     12. + log(mg2/Q2)*log(msd22/Q2) + (log(msb12/Q2)*log(msd22/Q2))/12. + (
     log(msb22/Q2)*log(msd22/Q2))/12. + (log(msd12/Q2)*log(msd22/Q2))/3. + (
     log(msb12/Q2)*log(mst12/Q2))/48. + (log(msb22/Q2)*log(mst12/Q2))/48. + (
     log(msd12/Q2)*log(mst12/Q2))/12. + (log(msd22/Q2)*log(mst12/Q2))/12. + (
     log(msb12/Q2)*log(mst22/Q2))/48. + (log(msb22/Q2)*log(mst22/Q2))/48. + (
     log(msd12/Q2)*log(mst22/Q2))/12. + (log(msd22/Q2)*log(mst22/Q2))/12. + (
     log(mst12/Q2)*log(mst22/Q2))/48. + (log(msb12/Q2)*log(mt2/Q2))/12. + (log(
     msb22/Q2)*log(mt2/Q2))/12. + (log(msd12/Q2)*log(mt2/Q2))/3. + (log(msd22/
     Q2)*log(mt2/Q2))/3. + (log(mst12/Q2)*log(mt2/Q2))/12. + (log(mst22/Q2)*
     log(mt2/Q2))/12. + (log(msd12/Q2)*(-6*mg2*msd12 + 10*msd14 + 9*power2(mg2)
     - 13*power2(msd12)))/(6.*power2(mg2 - msd12)) + (8*log(msd12/Q2)*(-8*mg2*
     msd12 - msd14 + 5*power2(mg2) + 4*power2(msd12)))/(9.*power2(mg2 - msd12))
     + (log(msd22/Q2)*(-6*mg2*msd22 + 10*msd24 + 9*power2(mg2) - 13*power2(
     msd22)))/(6.*power2(mg2 - msd22)) + (8*log(msd22/Q2)*(-8*mg2*msd22 - msd24
     + 5*power2(mg2) + 4*power2(msd22)))/(9.*power2(mg2 - msd22)) - (4*(-2 +
     msb12/msb22 + msb22/msb12)*power2(snb))/9. + (4*(-1 + msb12/msb22)*log(
     msb12/Q2)*power2(snb))/9. + (4*(-1 + msb22/msb12)*log(msb22/Q2)*power2(
     snb))/9. - (4*(-2 + mst12/mst22 + mst22/mst12)*power2(snt))/9. + (4*(-1 +
     mst12/mst22)*log(mst12/Q2)*power2(snt))/9. + (4*(-1 + mst22/mst12)*log(
     mst22/Q2)*power2(snt))/9. + (3*power2(log(mg2/Q2)))/2. + power2(log(msb12/
     Q2))/96. + power2(log(msb22/Q2))/96. + power2(log(msd12/Q2))/6. + power2(
     log(msd22/Q2))/6. + power2(log(mst12/Q2))/96. + power2(log(mst22/Q2))/96.
      + power2(log(mt2/Q2))/
     6. - (8*DeltaInv(mg2,mt2,mst12)*log(mg2/Q2)*log(mst12/Q2)*power2(mg2)*(
     mst12 + DeltaInv(mg2,mt2,mst12)*(8*mg2*mst14 + 4*mg2*mst12*mt2 + 2*mst14*
     mt2 + mst12*power2(mg2) - power3(mst12)) - 2*mg2*power2(DeltaInv(mg2,mt2,
     mst12))*((mst14 - mst12*mt2)*power2(mg2) + mst12*power3(mg2) + (3*mst12 -
     5*mt2)*power3(mst12) - 5*mg2*(2*mst14*mt2 + power3(mst12)))))/(3.*mt2) + (
     4*cb*mu*log(mt2/Q2)*(mst14 - mst24 + mst12*mt2 - mst22*mt2 + (-mst12 +
     mt2)*DeltaInv(mg2,mt2,mst12)*power3(mst12) + (mst22 - mt2)*DeltaInv(mg2,
     mst22,mt2)*power3(mst22)))/(mg*(mst12 - mst22)*mt2*sb) + (4*cb*mu*log(mt2/
     Q2)*(-mst14 + mst24 - mst12*mt2 + mst22*mt2 + (mst12 - mt2)*DeltaInv(mg2,
     mt2,mst12)*power3(mst12) + (-mst22 + mt2)*DeltaInv(mg2,mst22,mt2)*power3(
     mst22)))/(mg*(mst12 - mst22)*mt2*sb) - (8*DeltaInv(mg2,mst22,mt2)*log(mg2/
     Q2)*log(mst22/Q2)*power2(mg2)*(mst22 + DeltaInv(mg2,mst22,mt2)*(8*mg2*
     mst24 + 4*mg2*mst22*mt2 + 2*mst24*mt2 + mst22*power2(mg2) - power3(mst22))
     - 2*mg2*power2(DeltaInv(mg2,mst22,mt2))*((mst24 - mst22*mt2)*power2(mg2) +
     mst22*power3(mg2) + (3*mst22 - 5*mt2)*power3(mst22) - 5*mg2*(2*mst24*mt2 +
     power3(mst22)))))/(3.*mt2) + (8*power2(log(mg2/Q2))*power3(mg2)*((3*mg2*
     mst22 + mst24 + mst22*mt2)*power2(DeltaInv(mg2,mst22,mt2)) + power2(
     DeltaInv(mg2,mt2,mst12))*(3*mg2*mst12 + mst14 + mst12*mt2 + DeltaInv(mg2,
     mt2,mst12)*(5*(mst14 + mst12*mt2)*power2(mg2) - 3*mst12*power3(mg2) + (-
     mst12 + mt2)*power3(mst12) - mg2*(-10*mst14*mt2 + power3(mst12)))) + (5*(
     mst24 + mst22*mt2)*power2(mg2) - 3*mst22*power3(mg2) + (-mst22 + mt2)*
     power3(mst22) - mg2*(-10*mst24*mt2 + power3(mst22)))*power3(DeltaInv(mg2,
     mst22,mt2))))/(3.*mt2) + (16*cb*mg*mu*power2(mg2)*power2(log(mg2/Q2))*(-(
     mst22*DeltaInv(mg2,mst22,mt2)) + DeltaInv(mg2,mt2,mst12)*(mst12 +
     DeltaInv(mg2,mt2,mst12)*(6*mg2*mst14 + 4*mg2*mst12*mt2 + 2*mst14*mt2 + 3*
     mst12*power2(mg2) - power3(mst12)) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))
     *((2*mst14 + 3*mst12*mt2)*power2(mg2) - 2*mst12*power3(mg2) + (-2*mst12 +
     3*mt2)*power3(mst12) + 2*mg2*(5*mst14*mt2 + power3(mst12)))) + power2(
     DeltaInv(mg2,mst22,mt2))*(-6*mg2*mst24 - 4*mg2*mst22*mt2 - 2*mst24*mt2 -
     3*mst22*power2(mg2) + power3(mst22)) + 2*mg2*(-((2*mst24 + 3*mst22*mt2)*
     power2(mg2)) + 2*mst22*power3(mg2) + (2*mst22 - 3*mt2)*power3(mst22) - 2*
     mg2*(5*mst24*mt2 + power3(mst22)))*power3(DeltaInv(mg2,mst22,mt2))))/(3.*(
     mst12 - mst22)*mt2*sb) + (16*cb*mg*mu*power2(mg2)*power2(log(mg2/Q2))*(
     mst22*DeltaInv(mg2,mst22,mt2) + DeltaInv(mg2,mt2,mst12)*(-mst12 +
     DeltaInv(mg2,mt2,mst12)*(-6*mg2*mst14 - 4*mg2*mst12*mt2 - 2*mst14*mt2 - 3*
     mst12*power2(mg2) + power3(mst12)) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))
     *(-((2*mst14 + 3*mst12*mt2)*power2(mg2)) + 2*mst12*power3(mg2) + (2*mst12
     - 3*mt2)*power3(mst12) - 2*mg2*(5*mst14*mt2 + power3(mst12)))) + power2(
     DeltaInv(mg2,mst22,mt2))*(6*mg2*mst24 + 4*mg2*mst22*mt2 + 2*mst24*mt2 + 3*
     mst22*power2(mg2) - power3(mst22)) + 2*mg2*((2*mst24 + 3*mst22*mt2)*
     power2(mg2) - 2*mst22*power3(mg2) + (-2*mst22 + 3*mt2)*power3(mst22) + 2*
     mg2*(5*mst24*mt2 + power3(mst22)))*power3(DeltaInv(mg2,mst22,mt2))))/(3.*(
     mst12 - mst22)*mt2*sb) + (8*log(mg2/Q2)*log(mt2/Q2)*power2(mg2)*(mst22*
     DeltaInv(mg2,mst22,mt2) + DeltaInv(mg2,mt2,mst12)*(mst12 + DeltaInv(mg2,
     mt2,mst12)*(10*mg2*mst14 + 6*mg2*mst12*mt2 + 2*mst14*mt2 + 7*mst12*power2(
     mg2) - power3(mst12)) + 4*mg2*power2(DeltaInv(mg2,mt2,mst12))*((2*mst14 +
     3*mst12*mt2)*power2(mg2) - 2*mst12*power3(mg2) + (-2*mst12 + 3*mt2)*
     power3(mst12) + 2*mg2*(5*mst14*mt2 + power3(mst12)))) + power2(DeltaInv(
     mg2,mst22,mt2))*(10*mg2*mst24 + 6*mg2*mst22*mt2 + 2*mst24*mt2 + 7*mst22*
     power2(mg2) - power3(mst22)) + 4*mg2*((2*mst24 + 3*mst22*mt2)*power2(mg2)
     - 2*mst22*power3(mg2) + (-2*mst22 + 3*mt2)*power3(mst22) + 2*mg2*(5*mst24*
     mt2 + power3(mst22)))*power3(DeltaInv(mg2,mst22,mt2))))/(3.*mt2) - (log(
     mg2/Q2)*log(mst12/Q2)*(-mt2 + 12*mg2*(3*mg2*mst12 + 3*mst14 + mst12*mt2)*
     DeltaInv(mg2,mt2,mst12) - 12*mg2*power2(DeltaInv(mg2,mt2,mst12))*(-((19*
     mst14 + 5*mst12*mt2)*power2(mg2)) + 3*mst12*power3(mg2) + (3*mst12 - 5*
     mt2)*power3(mst12) - mg2*(22*mst14*mt2 + 19*power3(mst12))) - 96*power2(
     mg2)*power3(DeltaInv(mg2,mt2,mst12))*(2*mst14*power3(mg2) - 2*mg2*(mst12 +
     5*mt2)*power3(mst12) - power2(mg2)*(3*mst14*mt2 + 2*power3(mst12)) + (2*
     mst12 - 3*mt2)*power4(mst12))))/(4.*mt2) - (2*log(mst12/Q2)*(-5*mt2 + 2*
     mg2*mst12*(12*mg2 + mt2)*DeltaInv(mg2,mt2,mst12) - 24*power2(mg2)*power2(
     DeltaInv(mg2,mt2,mst12))*(-11*mg2*mst14 - 2*mg2*mst12*mt2 - 5*mst14*mt2 +
     mst12*power2(mg2) - 2*power3(mst12)) - 72*power2(mg2)*power3(DeltaInv(mg2,
     mt2,mst12))*(3*mst14*power3(mg2) + mg2*(mst12 - 10*mt2)*power3(mst12) - 5*
     power2(mg2)*(mst14*mt2 + power3(mst12)) + (mst12 - mt2)*power4(mst12))))/(
     9.*mt2) + (8*DeltaInv(mg2,mt2,mst12)*log(mst12/Q2)*log(mt2/Q2)*power2(mg2)
     *(mst12 + DeltaInv(mg2,mt2,mst12)*(14*mg2*mst14 + 4*mg2*mst12*mt2 + 4*
     mst14*mt2 + mst12*power2(mg2) + power3(mst12)) - 2*power2(DeltaInv(mg2,
     mt2,mst12))*((4*mst14 - mst12*mt2)*power3(mg2) + mg2*(4*mst12 - 15*mt2)*
     power3(mst12) - 5*power2(mg2)*(3*mst14*mt2 + 2*power3(mst12)) + mst12*
     power4(mg2) + (mst12 - mt2)*power4(mst12))))/(3.*mt2) + (8*power2(mg2)*
     power2(DeltaInv(mg2,mt2,mst12))*power2(log(mst12/Q2))*(mst14*(3*mg2 + mt2)
     + power3(mst12) + DeltaInv(mg2,mt2,mst12)*(-3*mst14*power3(mg2) - mg2*(
     mst12 - 10*mt2)*power3(mst12) + 5*power2(mg2)*(mst14*mt2 + power3(mst12))
     + (-mst12 + mt2)*power4(mst12))))/(3.*mt2) + Fin3(mg2,mt2,mst12,Q2)*((16*
     power2(mg2)*power2(DeltaInv(mg2,mt2,mst12))*(3*mg2*mst12 + mst14 + mst12*
     mt2 + DeltaInv(mg2,mt2,mst12)*(5*(mst14 + mst12*mt2)*power2(mg2) - 3*
     mst12*power3(mg2) + (-mst12 + mt2)*power3(mst12) - mg2*(-10*mst14*mt2 +
     power3(mst12)))))/(3.*mt2) + (6*mg2*DeltaInv(mg2,mt2,mst12)*(mst12 +
     DeltaInv(mg2,mt2,mst12)*(14*mg2*mst14 + 4*mg2*mst12*mt2 + 4*mst14*mt2 +
     mst12*power2(mg2) + power3(mst12)) - 2*power2(DeltaInv(mg2,mt2,mst12))*((
     4*mst14 - mst12*mt2)*power3(mg2) + mg2*(4*mst12 - 15*mt2)*power3(mst12) -
     5*power2(mg2)*(3*mst14*mt2 + 2*power3(mst12)) + mst12*power4(mg2) + (mst12
     - mt2)*power4(mst12))))/mt2) + (6*cb*mg*mg2*mu*power2(log(mg2/Q2))*((3*
     mg2*mst22 + 3*mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2) + power2(
     DeltaInv(mg2,mst22,mt2))*((23*mst24 + 7*mst22*mt2)*power2(mg2) - mst22*
     power3(mg2) + (-3*mst22 + 5*mt2)*power3(mst22) + mg2*(20*mst24*mt2 + 13*
     power3(mst22))) + DeltaInv(mg2,mt2,mst12)*(-3*mg2*mst12 - 3*mst14 - mst12*
     mt2 + DeltaInv(mg2,mt2,mst12)*(-((23*mst14 + 7*mst12*mt2)*power2(mg2)) +
     mst12*power3(mg2) + (3*mst12 - 5*mt2)*power3(mst12) - mg2*(20*mst14*mt2 +
     13*power3(mst12))) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*((8*mst14 -
     mst12*mt2)*power3(mg2) - 35*mg2*mt2*power3(mst12) - 7*power2(mg2)*(3*
     mst14*mt2 + 2*power3(mst12)) + mst12*power4(mg2) + (5*mst12 - 7*mt2)*
     power4(mst12))) - 2*mg2*power3(DeltaInv(mg2,mst22,mt2))*((8*mst24 - mst22*
     mt2)*power3(mg2) - 35*mg2*mt2*power3(mst22) - 7*power2(mg2)*(3*mst24*mt2 +
     2*power3(mst22)) + mst22*power4(mg2) + (5*mst22 - 7*mt2)*power4(mst22))))/
     ((mst12 - mst22)*mt2*sb) + (6*cb*mg*mg2*mu*power2(log(mg2/Q2))*(-((3*mg2*
     mst22 + 3*mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2)) + power2(DeltaInv(
     mg2,mst22,mt2))*(-((23*mst24 + 7*mst22*mt2)*power2(mg2)) + mst22*power3(
     mg2) + (3*mst22 - 5*mt2)*power3(mst22) - mg2*(20*mst24*mt2 + 13*power3(
     mst22))) + DeltaInv(mg2,mt2,mst12)*(3*mg2*mst12 + 3*mst14 + mst12*mt2 +
     DeltaInv(mg2,mt2,mst12)*((23*mst14 + 7*mst12*mt2)*power2(mg2) - mst12*
     power3(mg2) + (-3*mst12 + 5*mt2)*power3(mst12) + mg2*(20*mst14*mt2 + 13*
     power3(mst12))) - 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*((8*mst14 - mst12*
     mt2)*power3(mg2) - 35*mg2*mt2*power3(mst12) - 7*power2(mg2)*(3*mst14*mt2 +
     2*power3(mst12)) + mst12*power4(mg2) + (5*mst12 - 7*mt2)*power4(mst12))) +
     2*mg2*power3(DeltaInv(mg2,mst22,mt2))*((8*mst24 - mst22*mt2)*power3(mg2) -
     35*mg2*mt2*power3(mst22) - 7*power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) +
     mst22*power4(mg2) + (5*mst22 - 7*mt2)*power4(mst22))))/((mst12 - mst22)*
     mt2*sb) + (log(mg2/Q2)*log(mt2/Q2)*(mt2 + 3*mg2*(5*mg2*mst22 + 3*mst24 +
     mst22*mt2)*DeltaInv(mg2,mst22,mt2) + 3*mg2*(5*mg2*mst12 + 3*mst14 + mst12*
     mt2)*DeltaInv(mg2,mt2,mst12) + 3*mg2*power2(DeltaInv(mg2,mt2,mst12))*((47*
     mst14 + 13*mst12*mt2)*power2(mg2) - mst12*power3(mg2) + (-3*mst12 + 5*mt2)
     *power3(mst12) + 3*mg2*(10*mst14*mt2 + 7*power3(mst12))) + 3*mg2*power2(
     DeltaInv(mg2,mst22,mt2))*((47*mst24 + 13*mst22*mt2)*power2(mg2) - mst22*
     power3(mg2) + (-3*mst22 + 5*mt2)*power3(mst22) + 3*mg2*(10*mst24*mt2 + 7*
     power3(mst22))) - 12*power2(mg2)*power3(DeltaInv(mg2,mt2,mst12))*((8*mst14
     - mst12*mt2)*power3(mg2) - 35*mg2*mt2*power3(mst12) - 7*power2(mg2)*(3*
     mst14*mt2 + 2*power3(mst12)) + mst12*power4(mg2) + (5*mst12 - 7*mt2)*
     power4(mst12)) - 12*power2(mg2)*power3(DeltaInv(mg2,mst22,mt2))*((8*mst24
     - mst22*mt2)*power3(mg2) - 35*mg2*mt2*power3(mst22) - 7*power2(mg2)*(3*
     mst24*mt2 + 2*power3(mst22)) + mst22*power4(mg2) + (5*mst22 - 7*mt2)*
     power4(mst22))))/mt2 + (8*cb*mg*mg2*mu*power2(Pi)*((5*mg2*mst22 + 3*mst24
     + mst22*mt2)*DeltaInv(mg2,mst22,mt2) + power2(DeltaInv(mg2,mst22,mt2))*(5*
     (7*mst24 + 3*mst22*mt2)*power2(mg2) + 5*mst22*power3(mg2) + (-3*mst22 + 5*
     mt2)*power3(mst22) + mg2*(24*mst24*mt2 + 11*power3(mst22))) + DeltaInv(
     mg2,mt2,mst12)*(-5*mg2*mst12 - 3*mst14 - mst12*mt2 - DeltaInv(mg2,mt2,
     mst12)*(5*(7*mst14 + 3*mst12*mt2)*power2(mg2) + 5*mst12*power3(mg2) + (-3*
     mst12 + 5*mt2)*power3(mst12) + mg2*(24*mst14*mt2 + 11*power3(mst12))) + 2*
     mg2*power2(DeltaInv(mg2,mt2,mst12))*((4*mst14 - 7*mst12*mt2)*power3(mg2) +
     mg2*(4*mst12 - 41*mt2)*power3(mst12) - power2(mg2)*(41*mst14*mt2 + 18*
     power3(mst12)) + 5*mst12*power4(mg2) + (5*mst12 - 7*mt2)*power4(mst12))) -
     2*mg2*power3(DeltaInv(mg2,mst22,mt2))*((4*mst24 - 7*mst22*mt2)*power3(mg2)
     + mg2*(4*mst22 - 41*mt2)*power3(mst22) - power2(mg2)*(41*mst24*mt2 + 18*
     power3(mst22)) + 5*mst22*power4(mg2) + (5*mst22 - 7*mt2)*power4(mst22))))/
     (9.*(mst12 - mst22)*mt2*sb) + (8*cb*mg*mg2*mu*power2(Pi)*(-((5*mg2*mst22 +
     3*mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2)) - power2(DeltaInv(mg2,mst22,
     mt2))*(5*(7*mst24 + 3*mst22*mt2)*power2(mg2) + 5*mst22*power3(mg2) + (-3*
     mst22 + 5*mt2)*power3(mst22) + mg2*(24*mst24*mt2 + 11*power3(mst22))) +
     DeltaInv(mg2,mt2,mst12)*(5*mg2*mst12 + 3*mst14 + mst12*mt2 + DeltaInv(mg2,
     mt2,mst12)*(5*(7*mst14 + 3*mst12*mt2)*power2(mg2) + 5*mst12*power3(mg2) +
     (-3*mst12 + 5*mt2)*power3(mst12) + mg2*(24*mst14*mt2 + 11*power3(mst12)))
     - 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*((4*mst14 - 7*mst12*mt2)*power3(
     mg2) + mg2*(4*mst12 - 41*mt2)*power3(mst12) - power2(mg2)*(41*mst14*mt2 +
     18*power3(mst12)) + 5*mst12*power4(mg2) + (5*mst12 - 7*mt2)*power4(mst12))
     ) + 2*mg2*power3(DeltaInv(mg2,mst22,mt2))*((4*mst24 - 7*mst22*mt2)*power3(
     mg2) + mg2*(4*mst22 - 41*mt2)*power3(mst22) - power2(mg2)*(41*mst24*mt2 +
     18*power3(mst22)) + 5*mst22*power4(mg2) + (5*mst22 - 7*mt2)*power4(mst22))
     ))/(9.*(mst12 - mst22)*mt2*sb) + (16*cb*mg*mg2*mu*power2(log(mt2/Q2))*(-((
     4*mg2*mst22 + 2*mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2)) - power2(
     DeltaInv(mg2,mst22,mt2))*((26*mst24 + 11*mst22*mt2)*power2(mg2) + 2*mst22*
     power3(mg2) + (-2*mst22 + 3*mt2)*power3(mst22) + 6*mg2*(3*mst24*mt2 +
     power3(mst22))) + DeltaInv(mg2,mt2,mst12)*(4*mg2*mst12 + 2*mst14 + mst12*
     mt2 + DeltaInv(mg2,mt2,mst12)*((26*mst14 + 11*mst12*mt2)*power2(mg2) + 2*
     mst12*power3(mg2) + (-2*mst12 + 3*mt2)*power3(mst12) + 6*mg2*(3*mst14*mt2
     + power3(mst12))) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*(-4*(mst14 -
     mst12*mt2)*power3(mg2) - 4*mg2*(mst12 - 7*mt2)*power3(mst12) + 14*power2(
     mg2)*(2*mst14*mt2 + power3(mst12)) - 3*mst12*power4(mg2) + (-3*mst12 + 4*
     mt2)*power4(mst12))) + 2*mg2*power3(DeltaInv(mg2,mst22,mt2))*(4*(mst24 -
     mst22*mt2)*power3(mg2) + 4*mg2*(mst22 - 7*mt2)*power3(mst22) - 14*power2(
     mg2)*(2*mst24*mt2 + power3(mst22)) + 3*mst22*power4(mg2) + (3*mst22 - 4*
     mt2)*power4(mst22))))/(3.*(mst12 - mst22)*mt2*sb) - (log(mg2/Q2)*log(
     mst22/Q2)*(-mt2 + 12*mg2*(3*mg2*mst22 + 3*mst24 + mst22*mt2)*DeltaInv(mg2,
     mst22,mt2) - 12*mg2*power2(DeltaInv(mg2,mst22,mt2))*(-((19*mst24 + 5*
     mst22*mt2)*power2(mg2)) + 3*mst22*power3(mg2) + (3*mst22 - 5*mt2)*power3(
     mst22) - mg2*(22*mst24*mt2 + 19*power3(mst22))) - 96*power2(mg2)*power3(
     DeltaInv(mg2,mst22,mt2))*(2*mst24*power3(mg2) - 2*mg2*(mst22 + 5*mt2)*
     power3(mst22) - power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) + (2*mst22 -
     3*mt2)*power4(mst22))))/(4.*mt2) - (2*log(mst22/Q2)*(-5*mt2 + 2*mg2*mst22*
     (12*mg2 + mt2)*DeltaInv(mg2,mst22,mt2) - 24*power2(mg2)*power2(DeltaInv(
     mg2,mst22,mt2))*(-11*mg2*mst24 - 2*mg2*mst22*mt2 - 5*mst24*mt2 + mst22*
     power2(mg2) - 2*power3(mst22)) - 72*power2(mg2)*power3(DeltaInv(mg2,mst22,
     mt2))*(3*mst24*power3(mg2) + mg2*(mst22 - 10*mt2)*power3(mst22) - 5*
     power2(mg2)*(mst24*mt2 + power3(mst22)) + (mst22 - mt2)*power4(mst22))))/(
     9.*mt2) + (8*DeltaInv(mg2,mst22,mt2)*log(mst22/Q2)*log(mt2/Q2)*power2(mg2)
     *(mst22 + DeltaInv(mg2,mst22,mt2)*(14*mg2*mst24 + 4*mg2*mst22*mt2 + 4*
     mst24*mt2 + mst22*power2(mg2) + power3(mst22)) - 2*power2(DeltaInv(mg2,
     mst22,mt2))*((4*mst24 - mst22*mt2)*power3(mg2) + mg2*(4*mst22 - 15*mt2)*
     power3(mst22) - 5*power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) + mst22*
     power4(mg2) + (mst22 - mt2)*power4(mst22))))/(3.*mt2) + (3*power2(mg2)*
     power2(log(mg2/Q2))*(mst22*DeltaInv(mg2,mst22,mt2) + power2(DeltaInv(mg2,
     mst22,mt2))*(14*mg2*mst24 + 4*mg2*mst22*mt2 + 4*mst24*mt2 + mst22*power2(
     mg2) + power3(mst22)) + DeltaInv(mg2,mt2,mst12)*(mst12 + DeltaInv(mg2,mt2,
     mst12)*(14*mg2*mst14 + 4*mg2*mst12*mt2 + 4*mst14*mt2 + mst12*power2(mg2) +
     power3(mst12)) - 2*power2(DeltaInv(mg2,mt2,mst12))*((4*mst14 - mst12*mt2)*
     power3(mg2) + mg2*(4*mst12 - 15*mt2)*power3(mst12) - 5*power2(mg2)*(3*
     mst14*mt2 + 2*power3(mst12)) + mst12*power4(mg2) + (mst12 - mt2)*power4(
     mst12))) - 2*power3(DeltaInv(mg2,mst22,mt2))*((4*mst24 - mst22*mt2)*
     power3(mg2) + mg2*(4*mst22 - 15*mt2)*power3(mst22) - 5*power2(mg2)*(3*
     mst24*mt2 + 2*power3(mst22)) + mst22*power4(mg2) + (mst22 - mt2)*power4(
     mst22))))/mt2 + (4*power2(mg2)*power2(Pi)*(mst22*DeltaInv(mg2,mst22,mt2) +
     power2(DeltaInv(mg2,mst22,mt2))*(16*mg2*mst24 + 6*mg2*mst22*mt2 + 4*mst24*
     mt2 + 7*mst22*power2(mg2) + power3(mst22)) + DeltaInv(mg2,mt2,mst12)*(
     mst12 + DeltaInv(mg2,mt2,mst12)*(16*mg2*mst14 + 6*mg2*mst12*mt2 + 4*mst14*
     mt2 + 7*mst12*power2(mg2) + power3(mst12)) - 2*power2(DeltaInv(mg2,mt2,
     mst12))*(-((mst14 + 6*mst12*mt2)*power3(mg2)) + mg2*(5*mst12 - 16*mt2)*
     power3(mst12) - power2(mg2)*(25*mst14*mt2 + 9*power3(mst12)) + 4*mst12*
     power4(mg2) + (mst12 - mt2)*power4(mst12))) - 2*power3(DeltaInv(mg2,mst22,
     mt2))*(-((mst24 + 6*mst22*mt2)*power3(mg2)) + mg2*(5*mst22 - 16*mt2)*
     power3(mst22) - power2(mg2)*(25*mst24*mt2 + 9*power3(mst22)) + 4*mst22*
     power4(mg2) + (mst22 - mt2)*power4(mst22))))/(9.*mt2) + (16*cb*mg*mg2*mu*
     log(mg2/Q2)*log(mt2/Q2)*((5*mg2*mst22 + mst24 + mst22*mt2)*DeltaInv(mg2,
     mst22,mt2) + power2(DeltaInv(mg2,mst22,mt2))*((29*mst24 + 15*mst22*mt2)*
     power2(mg2) + 5*mst22*power3(mg2) + (-mst22 + mt2)*power3(mst22) - mg2*(-
     16*mst24*mt2 + power3(mst22))) + DeltaInv(mg2,mt2,mst12)*(-5*mg2*mst12 -
     mst14 - mst12*mt2 + DeltaInv(mg2,mt2,mst12)*(-((29*mst14 + 15*mst12*mt2)*
     power2(mg2)) - 5*mst12*power3(mg2) + (mst12 - mt2)*power3(mst12) + mg2*(-
     16*mst14*mt2 + power3(mst12))) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*(-
     7*mst12*mt2*power3(mg2) + mg2*(8*mst12 - 21*mt2)*power3(mst12) - 7*power2(
     mg2)*(5*mst14*mt2 + 2*power3(mst12)) + 5*mst12*power4(mg2) + (mst12 - mt2)
     *power4(mst12))) - 2*mg2*power3(DeltaInv(mg2,mst22,mt2))*(-7*mst22*mt2*
     power3(mg2) + mg2*(8*mst22 - 21*mt2)*power3(mst22) - 7*power2(mg2)*(5*
     mst24*mt2 + 2*power3(mst22)) + 5*mst22*power4(mg2) + (mst22 - mt2)*power4(
     mst22))))/(3.*(mst12 - mst22)*mt2*sb) + (16*cb*mg*mg2*mu*log(mg2/Q2)*log(
     mt2/Q2)*(-((5*mg2*mst22 + mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2)) +
     power2(DeltaInv(mg2,mst22,mt2))*(-((29*mst24 + 15*mst22*mt2)*power2(mg2))
     - 5*mst22*power3(mg2) + (mst22 - mt2)*power3(mst22) + mg2*(-16*mst24*mt2 +
     power3(mst22))) + DeltaInv(mg2,mt2,mst12)*(5*mg2*mst12 + mst14 + mst12*mt2
     + DeltaInv(mg2,mt2,mst12)*((29*mst14 + 15*mst12*mt2)*power2(mg2) + 5*
     mst12*power3(mg2) + (-mst12 + mt2)*power3(mst12) - mg2*(-16*mst14*mt2 +
     power3(mst12))) - 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*(-7*mst12*mt2*
     power3(mg2) + mg2*(8*mst12 - 21*mt2)*power3(mst12) - 7*power2(mg2)*(5*
     mst14*mt2 + 2*power3(mst12)) + 5*mst12*power4(mg2) + (mst12 - mt2)*power4(
     mst12))) + 2*mg2*power3(DeltaInv(mg2,mst22,mt2))*(-7*mst22*mt2*power3(mg2)
     + mg2*(8*mst22 - 21*mt2)*power3(mst22) - 7*power2(mg2)*(5*mst24*mt2 + 2*
     power3(mst22)) + 5*mst22*power4(mg2) + (mst22 - mt2)*power4(mst22))))/(3.*
     (mst12 - mst22)*mt2*sb) + (4*log(mt2/Q2)*(-(mg2*mst12*mt2) - mg2*mst22*mt2
     - 3*mst12*mst22*mt2 - mst12*power2(mg2) - mst22*power2(mg2) - mst12*
     power2(mt2) - mst22*power2(mt2) + mg2*mst22*DeltaInv(mg2,mt2,mst12)*(mg2*
     mst12*(-17*mst12 + 2*mt2) + (4*mst12 - mt2)*power2(mg2) + mt2*power2(
     mst12) + power3(mg2)) + mg2*mst12*DeltaInv(mg2,mst22,mt2)*(mg2*mst22*(-17*
     mst22 + 2*mt2) + (4*mst22 - mt2)*power2(mg2) + mt2*power2(mst22) + power3(
     mg2)) - 6*mst12*mst22*power2(mg2)*power2(DeltaInv(mg2,mt2,mst12))*(13*
     mst14*mt2 + mg2*(67*mst14 + 20*mst12*mt2) + (25*mst12 - mt2)*power2(mg2) +
     power3(mg2) + 3*power3(mst12)) - 6*mst12*mst22*power2(mg2)*power2(
     DeltaInv(mg2,mst22,mt2))*(13*mst24*mt2 + mg2*(67*mst24 + 20*mst22*mt2) + (
     25*mst22 - mt2)*power2(mg2) + power3(mg2) + 3*power3(mst22)) + 36*mst12*
     mst22*power2(mg2)*power3(DeltaInv(mg2,mt2,mst12))*(-7*mst12*mt2*power3(
     mg2) + mg2*(8*mst12 - 21*mt2)*power3(mst12) - 7*power2(mg2)*(5*mst14*mt2 +
     2*power3(mst12)) + 5*mst12*power4(mg2) + (mst12 - mt2)*power4(mst12)) +
     36*mst12*mst22*power2(mg2)*power3(DeltaInv(mg2,mst22,mt2))*(-7*mst22*mt2*
     power3(mg2) + mg2*(8*mst22 - 21*mt2)*power3(mst22) - 7*power2(mg2)*(5*
     mst24*mt2 + 2*power3(mst22)) + 5*mst22*power4(mg2) + (mst22 - mt2)*power4(
     mst22))))/(9.*mst12*mst22*mt2) + (8*power2(mg2)*power2(DeltaInv(mg2,mst22,
     mt2))*power2(log(mst22/Q2))*(mst24*(3*mg2 + mt2) + power3(mst22) +
     DeltaInv(mg2,mst22,mt2)*(-3*mst24*power3(mg2) - mg2*(mst22 - 10*mt2)*
     power3(mst22) + 5*power2(mg2)*(mst24*mt2 + power3(mst22)) + (-mst22 + mt2)
     *power4(mst22))))/(3.*mt2) + (8*power2(mg2)*power2(log(mt2/Q2))*(mst22*
     DeltaInv(mg2,mst22,mt2) + (12*mg2*mst24 + 5*mg2*mst22*mt2 + 3*mst24*mt2 +
     4*mst22*power2(mg2))*power2(DeltaInv(mg2,mst22,mt2)) + DeltaInv(mg2,mt2,
     mst12)*(mst12 + DeltaInv(mg2,mt2,mst12)*(12*mg2*mst14 + 5*mg2*mst12*mt2 +
     3*mst14*mt2 + 4*mst12*power2(mg2)) + power2(DeltaInv(mg2,mt2,mst12))*(7*
     mst12*mt2*power3(mg2) + mg2*(-8*mst12 + 21*mt2)*power3(mst12) + 7*power2(
     mg2)*(5*mst14*mt2 + 2*power3(mst12)) - 5*mst12*power4(mg2) + (-mst12 +
     mt2)*power4(mst12))) + power3(DeltaInv(mg2,mst22,mt2))*(7*mst22*mt2*
     power3(mg2) + mg2*(-8*mst22 + 21*mt2)*power3(mst22) + 7*power2(mg2)*(5*
     mst24*mt2 + 2*power3(mst22)) - 5*mst22*power4(mg2) + (-mst22 + mt2)*
     power4(mst22))))/(3.*mt2) + (16*cb*mg*mg2*mu*power2(log(mt2/Q2))*((4*mg2*
     mst22 + 2*mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2) + power2(DeltaInv(
     mg2,mst22,mt2))*((26*mst24 + 11*mst22*mt2)*power2(mg2) + 2*mst22*power3(
     mg2) + (-2*mst22 + 3*mt2)*power3(mst22) + 6*mg2*(3*mst24*mt2 + power3(
     mst22))) + DeltaInv(mg2,mt2,mst12)*(-4*mg2*mst12 - 2*mst14 - mst12*mt2 -
     DeltaInv(mg2,mt2,mst12)*((26*mst14 + 11*mst12*mt2)*power2(mg2) + 2*mst12*
     power3(mg2) + (-2*mst12 + 3*mt2)*power3(mst12) + 6*mg2*(3*mst14*mt2 +
     power3(mst12))) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*(4*(mst14 - mst12*
     mt2)*power3(mg2) + 4*mg2*(mst12 - 7*mt2)*power3(mst12) - 14*power2(mg2)*(
     2*mst14*mt2 + power3(mst12)) + 3*mst12*power4(mg2) + (3*mst12 - 4*mt2)*
     power4(mst12))) + 2*mg2*power3(DeltaInv(mg2,mst22,mt2))*(-4*(mst24 -
     mst22*mt2)*power3(mg2) - 4*mg2*(mst22 - 7*mt2)*power3(mst22) + 14*power2(
     mg2)*(2*mst24*mt2 + power3(mst22)) - 3*mst22*power4(mg2) + (-3*mst22 + 4*
     mt2)*power4(mst22))))/(3.*(mst12 - mst22)*mt2*sb) + Fin3(mg2,mst22,mt2,Q2)
     *((16*power2(mg2)*power2(DeltaInv(mg2,mst22,mt2))*(3*mg2*mst22 + mst24 +
     mst22*mt2 + DeltaInv(mg2,mst22,mt2)*(5*(mst24 + mst22*mt2)*power2(mg2) -
     3*mst22*power3(mg2) + (-mst22 + mt2)*power3(mst22) - mg2*(-10*mst24*mt2 +
     power3(mst22)))))/(3.*mt2) + (6*mg2*DeltaInv(mg2,mst22,mt2)*(mst22 +
     DeltaInv(mg2,mst22,mt2)*(14*mg2*mst24 + 4*mg2*mst22*mt2 + 4*mst24*mt2 +
     mst22*power2(mg2) + power3(mst22)) - 2*power2(DeltaInv(mg2,mst22,mt2))*((
     4*mst24 - mst22*mt2)*power3(mg2) + mg2*(4*mst22 - 15*mt2)*power3(mst22) -
     5*power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) + mst22*power4(mg2) + (mst22
     - mt2)*power4(mst22))))/mt2) + (4*(-2 + msb12/msb22 + msb22/msb12)*power4(
     snb))/9. - (4*(-1 + msb12/msb22)*log(msb12/Q2)*power4(snb))/9. + (4*(msb12
     - msb22)*log(msb22/Q2)*power4(snb))/(9.*msb12) + (4*(-2 + mst12/mst22 +
     mst22/mst12)*power4(snt))/9. - (4*(-1 + mst12/mst22)*log(mst12/Q2)*power4(
     snt))/9. + (4*(mst12 - mst22)*log(mst22/Q2)*power4(snt))/(9.*mst12) + (3*
     mg2*DeltaInv(mg2,mt2,mst12)*log(mst12/Q2)*log(mt2/Q2)*(3*mg2*mst12 + 5*
     mst14 + mst12*mt2 + DeltaInv(mg2,mt2,mst12)*((21*mst14 + 5*mst12*mt2)*
     power2(mg2) - 3*mst12*power3(mg2) - (mst12 - 13*mt2)*power3(mst12) + mg2*(
     30*mst14*mt2 + 47*power3(mst12))) - 4*power2(DeltaInv(mg2,mt2,mst12))*(-7*
     mst14*mt2*power3(mg2) - 7*(2*mst12 + 5*mt2)*power2(mg2)*power3(mst12) + 5*
     mst14*power4(mg2) + mg2*(8*mst12 - 21*mt2)*power4(mst12) + (mst12 - mt2)*
     power5(mst12))))/mt2 + (3*mg2*DeltaInv(mg2,mt2,mst12)*power2(log(mst12/Q2)
     )*(mst14 + DeltaInv(mg2,mt2,mst12)*(4*mg2*mst14*mt2 + mst14*power2(mg2) +
     14*mg2*power3(mst12) + 4*mt2*power3(mst12) + power4(mst12)) - 2*power2(
     DeltaInv(mg2,mt2,mst12))*(-5*(2*mst12 + 3*mt2)*power2(mg2)*power3(mst12) +
     power3(mg2)*(-(mst14*mt2) + 4*power3(mst12)) + mst14*power4(mg2) + mg2*(4*
     mst12 - 15*mt2)*power4(mst12) + (mst12 - mt2)*power5(mst12))))/mt2 - (log(
     mst12/Q2)*(4*mst12 - 3*mt2 + 4*DeltaInv(mg2,mt2,mst12)*(74*mg2*mst14 + 13*
     mg2*mst12*mt2 + 3*mst14*mt2 + 11*mst12*power2(mg2) - power3(mst12)) - 48*
     mg2*power2(DeltaInv(mg2,mt2,mst12))*(-((2*mst14 + mst12*mt2)*power2(mg2))
     + mst12*power3(mg2) - 17*mt2*power3(mst12) - mg2*(22*mst14*mt2 + 47*
     power3(mst12))) - 288*mg2*power3(DeltaInv(mg2,mt2,mst12))*(-5*(2*mst12 +
     3*mt2)*power2(mg2)*power3(mst12) + power3(mg2)*(-(mst14*mt2) + 4*power3(
     mst12)) + mst14*power4(mg2) + mg2*(4*mst12 - 15*mt2)*power4(mst12) + (
     mst12 - mt2)*power5(mst12))))/(8.*mt2) + (3*mg2*DeltaInv(mg2,mst22,mt2)*
     log(mst22/Q2)*log(mt2/Q2)*(3*mg2*mst22 + 5*mst24 + mst22*mt2 + DeltaInv(
     mg2,mst22,mt2)*((21*mst24 + 5*mst22*mt2)*power2(mg2) - 3*mst22*power3(mg2)
     - (mst22 - 13*mt2)*power3(mst22) + mg2*(30*mst24*mt2 + 47*power3(mst22)))
     - 4*power2(DeltaInv(mg2,mst22,mt2))*(-7*mst24*mt2*power3(mg2) - 7*(2*mst22
     + 5*mt2)*power2(mg2)*power3(mst22) + 5*mst24*power4(mg2) + mg2*(8*mst22 -
     21*mt2)*power4(mst22) + (mst22 - mt2)*power5(mst22))))/mt2 + (3*mg2*
     DeltaInv(mg2,mst22,mt2)*power2(log(mst22/Q2))*(mst24 + DeltaInv(mg2,mst22,
     mt2)*(4*mg2*mst24*mt2 + mst24*power2(mg2) + 14*mg2*power3(mst22) + 4*mt2*
     power3(mst22) + power4(mst22)) - 2*power2(DeltaInv(mg2,mst22,mt2))*(-5*(2*
     mst22 + 3*mt2)*power2(mg2)*power3(mst22) + power3(mg2)*(-(mst24*mt2) + 4*
     power3(mst22)) + mst24*power4(mg2) + mg2*(4*mst22 - 15*mt2)*power4(mst22)
     + (mst22 - mt2)*power5(mst22))))/mt2 - (log(mst22/Q2)*(4*mst22 - 3*mt2 +
     4*DeltaInv(mg2,mst22,mt2)*(74*mg2*mst24 + 13*mg2*mst22*mt2 + 3*mst24*mt2 +
     11*mst22*power2(mg2) - power3(mst22)) - 48*mg2*power2(DeltaInv(mg2,mst22,
     mt2))*(-((2*mst24 + mst22*mt2)*power2(mg2)) + mst22*power3(mg2) - 17*mt2*
     power3(mst22) - mg2*(22*mst24*mt2 + 47*power3(mst22))) - 288*mg2*power3(
     DeltaInv(mg2,mst22,mt2))*(-5*(2*mst22 + 3*mt2)*power2(mg2)*power3(mst22) +
     power3(mg2)*(-(mst24*mt2) + 4*power3(mst22)) + mst24*power4(mg2) + mg2*(4*
     mst22 - 15*mt2)*power4(mst22) + (mst22 - mt2)*power5(mst22))))/(8.*mt2) -
     (log(mt2/Q2)*(-2*mg2 - mst12 - mst22 - 7*mt2 + DeltaInv(mg2,mt2,mst12)*(-
     3*mst14*mt2 + 5*mg2*(19*mst14 + 2*mst12*mt2) + (95*mst12 - 3*mt2)*power2(
     mg2) + power3(mg2) + power3(mst12)) - 12*mg2*power2(DeltaInv(mg2,mt2,
     mst12))*(-7*(14*mst14 + 3*mst12*mt2)*power2(mg2) + 2*mst12*power3(mg2) + (
     2*mst12 - 21*mt2)*power3(mst12) - 14*mg2*(5*mst14*mt2 + 7*power3(mst12)))
     + DeltaInv(mg2,mst22,mt2)*(-3*mst24*mt2 + 5*mg2*(19*mst24 + 2*mst22*mt2) +
     (95*mst22 - 3*mt2)*power2(mg2) + power3(mg2) + power3(mst22)) - 12*mg2*
     power2(DeltaInv(mg2,mst22,mt2))*(-7*(14*mst24 + 3*mst22*mt2)*power2(mg2) +
     2*mst22*power3(mg2) + (2*mst22 - 21*mt2)*power3(mst22) - 14*mg2*(5*mst24*
     mt2 + 7*power3(mst22))) - 72*mg2*power3(DeltaInv(mg2,mt2,mst12))*(-14*(
     mst12 + 5*mt2)*power2(mg2)*power3(mst12) - 14*power3(mg2)*(2*mst14*mt2 +
     power3(mst12)) + (13*mst14 - mst12*mt2)*power4(mg2) + mg2*(13*mst12 - 28*
     mt2)*power4(mst12) + mst12*power5(mg2) + (mst12 - mt2)*power5(mst12)) -
     72*mg2*power3(DeltaInv(mg2,mst22,mt2))*(-14*(mst22 + 5*mt2)*power2(mg2)*
     power3(mst22) - 14*power3(mg2)*(2*mst24*mt2 + power3(mst22)) + (13*mst24 -
     mst22*mt2)*power4(mg2) + mg2*(13*mst22 - 28*mt2)*power4(mst22) + mst22*
     power5(mg2) + (mst22 - mt2)*power5(mst22))))/(2.*mt2) + (mg2*power2(Pi)*((
     5*mg2*mst22 + 5*mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2) + power2(
     DeltaInv(mg2,mst22,mt2))*((49*mst24 + 13*mst22*mt2)*power2(mg2) - mst22*
     power3(mg2) - (mst22 - 13*mt2)*power3(mst22) + mg2*(38*mst24*mt2 + 49*
     power3(mst22))) + DeltaInv(mg2,mt2,mst12)*(5*mg2*mst12 + 5*mst14 + mst12*
     mt2 + DeltaInv(mg2,mt2,mst12)*((49*mst14 + 13*mst12*mt2)*power2(mg2) -
     mst12*power3(mg2) - (mst12 - 13*mt2)*power3(mst12) + mg2*(38*mst14*mt2 +
     49*power3(mst12))) - 4*power2(DeltaInv(mg2,mt2,mst12))*(-10*(mst12 + 5*
     mt2)*power2(mg2)*power3(mst12) - 2*power3(mg2)*(11*mst14*mt2 + 5*power3(
     mst12)) + (9*mst14 - mst12*mt2)*power4(mg2) + mg2*(9*mst12 - 22*mt2)*
     power4(mst12) + mst12*power5(mg2) + (mst12 - mt2)*power5(mst12))) - 4*
     power3(DeltaInv(mg2,mst22,mt2))*(-10*(mst22 + 5*mt2)*power2(mg2)*power3(
     mst22) - 2*power3(mg2)*(11*mst24*mt2 + 5*power3(mst22)) + (9*mst24 -
     mst22*mt2)*power4(mg2) + mg2*(9*mst22 - 22*mt2)*power4(mst22) + mst22*
     power5(mg2) + (mst22 - mt2)*power5(mst22))))/(2.*mt2) + (18*DeltaInv(mg2,
     mt2,mst12)*(-(mst14*mt2) + mg2*(104*mst14 + 15*mst12*mt2) + (104*mst12 -
     mt2)*power2(mg2) + power3(mg2) + power3(mst12)) + 378*mg2*power2(DeltaInv(
     mg2,mt2,mst12))*((49*mst14 + 13*mst12*mt2)*power2(mg2) - mst12*power3(mg2)
     - (mst12 - 13*mt2)*power3(mst12) + mg2*(38*mst14*mt2 + 49*power3(mst12)))
     + 18*DeltaInv(mg2,mst22,mt2)*(-(mst24*mt2) + mg2*(104*mst24 + 15*mst22*
     mt2) + (104*mst22 - mt2)*power2(mg2) + power3(mg2) + power3(mst22)) + 378*
     mg2*power2(DeltaInv(mg2,mst22,mt2))*((49*mst24 + 13*mst22*mt2)*power2(mg2)
     - mst22*power3(mg2) - (mst22 - 13*mt2)*power3(mst22) + mg2*(38*mst24*mt2 +
     49*power3(mst22))) + (-77*msd14*mt2*power2(msd22) - power2(msd12)*(77*
     msd24*mt2 + 2*(9*mst12 + 9*mst22 - 128*mt2)*power2(msd22)) + power2(mg2)*(
     -77*(msd14 + msd24)*mt2 + 24*msd12*msd22*(3*msd22 - 3*mst12 - 3*mst22 +
     17*mt2) + (72*msd22 - 18*mst12 - 18*mst22 + 179*mt2)*power2(msd12) + (-18*
     mst12 - 18*mst22 + 179*mt2)*power2(msd22)) + 2*mg2*(77*msd14*msd22*mt2 +
     msd22*(-18*msd22 + 18*mst12 + 18*mst22 - 179*mt2)*power2(msd12) + msd12*(
     77*msd24*mt2 + (18*mst12 + 18*mst22 - 179*mt2)*power2(msd22))) - 12*(
     msd22*(3*msd22 - 3*mst12 - 3*mst22 + 17*mt2) + msd12*(12*msd22 - 3*mst12 -
     3*mst22 + 17*mt2) + 3*power2(msd12))*power3(mg2) + 6*(12*msd12 + 12*msd22
     - 3*mst12 - 3*mst22 + 17*mt2)*power4(mg2) - 36*power5(mg2))/(power2(mg2 -
     msd12)*power2(mg2 - msd22)) - 1512*mg2*power3(DeltaInv(mg2,mt2,mst12))*(-
     10*(mst12 + 5*mt2)*power2(mg2)*power3(mst12) - 2*power3(mg2)*(11*mst14*mt2
     + 5*power3(mst12)) + (9*mst14 - mst12*mt2)*power4(mg2) + mg2*(9*mst12 -
     22*mt2)*power4(mst12) + mst12*power5(mg2) + (mst12 - mt2)*power5(mst12)) +
     1512*mg2*power3(DeltaInv(mg2,mst22,mt2))*(10*(mst22 + 5*mt2)*power2(mg2)*
     power3(mst22) + 2*power3(mg2)*(11*mst24*mt2 + 5*power3(mst22)) + (-9*mst24
     + mst22*mt2)*power4(mg2) + mg2*(-9*mst22 + 22*mt2)*power4(mst22) - mst22*
     power5(mg2) + (-mst22 + mt2)*power5(mst22)))/(18.*mt2) + (3*mg2*power2(
     log(mt2/Q2))*((4*mg2*mst22 + 4*mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2)
     + power2(DeltaInv(mg2,mst22,mt2))*((34*mst24 + 9*mst22*mt2)*power2(mg2) -
     2*mst22*power3(mg2) + (-2*mst22 + 9*mt2)*power3(mst22) + mg2*(30*mst24*mt2
     + 34*power3(mst22))) + DeltaInv(mg2,mt2,mst12)*(4*mg2*mst12 + 4*mst14 +
     mst12*mt2 + DeltaInv(mg2,mt2,mst12)*((34*mst14 + 9*mst12*mt2)*power2(mg2)
     - 2*mst12*power3(mg2) + (-2*mst12 + 9*mt2)*power3(mst12) + mg2*(30*mst14*
     mt2 + 34*power3(mst12))) + 2*power2(DeltaInv(mg2,mt2,mst12))*(14*(mst12 +
     5*mt2)*power2(mg2)*power3(mst12) + 14*power3(mg2)*(2*mst14*mt2 + power3(
     mst12)) + (-13*mst14 + mst12*mt2)*power4(mg2) + mg2*(-13*mst12 + 28*mt2)*
     power4(mst12) - mst12*power5(mg2) + (-mst12 + mt2)*power5(mst12))) + 2*
     power3(DeltaInv(mg2,mst22,mt2))*(14*(mst22 + 5*mt2)*power2(mg2)*power3(
     mst22) + 14*power3(mg2)*(2*mst24*mt2 + power3(mst22)) + (-13*mst24 +
     mst22*mt2)*power4(mg2) + mg2*(-13*mst22 + 28*mt2)*power4(mst22) - mst22*
     power5(mg2) + (-mst22 + mt2)*power5(mst22))))/mt2 + xt*((4*log(mst12/Q2)*(
     mst14 + mst12*mt2 + (-mst12 + mt2)*DeltaInv(mg2,mt2,mst12)*power3(mst12)))
     /(mg*(mst12 - mst22)*mt2) - (4*log(mst22/Q2)*(mst24 + mst22*mt2 + (-mst22
     + mt2)*DeltaInv(mg2,mst22,mt2)*power3(mst22)))/(mg*(mst12 - mst22)*mt2) +
     (4*log(mt2/Q2)*(-mst14 + mst24 - mst12*mt2 + mst22*mt2 + (mst12 - mt2)*
     DeltaInv(mg2,mt2,mst12)*power3(mst12) + (-mst22 + mt2)*DeltaInv(mg2,mst22,
     mt2)*power3(mst22)))/(mg*(mst12 - mst22)*mt2) + (16*mg*power2(mg2)*power2(
     log(mg2/Q2))*(mst22*DeltaInv(mg2,mst22,mt2) + DeltaInv(mg2,mt2,mst12)*(-
     mst12 + DeltaInv(mg2,mt2,mst12)*(-6*mg2*mst14 - 4*mg2*mst12*mt2 - 2*mst14*
     mt2 - 3*mst12*power2(mg2) + power3(mst12)) + 2*mg2*power2(DeltaInv(mg2,
     mt2,mst12))*(-((2*mst14 + 3*mst12*mt2)*power2(mg2)) + 2*mst12*power3(mg2)
     + (2*mst12 - 3*mt2)*power3(mst12) - 2*mg2*(5*mst14*mt2 + power3(mst12))))
     + power2(DeltaInv(mg2,mst22,mt2))*(6*mg2*mst24 + 4*mg2*mst22*mt2 + 2*
     mst24*mt2 + 3*mst22*power2(mg2) - power3(mst22)) + 2*mg2*((2*mst24 + 3*
     mst22*mt2)*power2(mg2) - 2*mst22*power3(mg2) + (-2*mst22 + 3*mt2)*power3(
     mst22) + 2*mg2*(5*mst24*mt2 + power3(mst22)))*power3(DeltaInv(mg2,mst22,
     mt2))))/(3.*(mst12 - mst22)*mt2) - (8*mg*mg2*log(mg2/Q2)*(-((mst12 -
     mst22)*(mg2 + mt2)) - mst22*DeltaInv(mg2,mt2,mst12)*(mg2*mst12*(61*mst12 +
     6*mt2) + 3*mst12*(-mst14 + 3*mst12*mt2) + (mst12 - mt2)*power2(mg2) +
     power3(mg2)) + mst12*DeltaInv(mg2,mst22,mt2)*(mg2*mst22*(61*mst22 + 6*mt2)
     + 3*mst22*(-mst24 + 3*mst22*mt2) + (mst22 - mt2)*power2(mg2) + power3(mg2)
     ) + 6*mg2*mst12*mst22*power2(DeltaInv(mg2,mt2,mst12))*(-17*mst14*mt2 -
     mg2*(41*mst14 + 34*mst12*mt2) - (17*mst12 + mt2)*power2(mg2) + power3(mg2)
     + 9*power3(mst12)) - 6*mg2*mst12*mst22*power2(DeltaInv(mg2,mst22,mt2))*(-
     17*mst24*mt2 - mg2*(41*mst24 + 34*mst22*mt2) - (17*mst22 + mt2)*power2(
     mg2) + power3(mg2) + 9*power3(mst22)) - 72*mst12*mst22*power2(mg2)*(-((2*
     mst24 + 3*mst22*mt2)*power2(mg2)) + 2*mst22*power3(mg2) + (2*mst22 - 3*
     mt2)*power3(mst22) - 2*mg2*(5*mst24*mt2 + power3(mst22)))*power3(DeltaInv(
     mg2,mst22,mt2)) + 72*mst12*mst22*power2(mg2)*(-((2*mst14 + 3*mst12*mt2)*
     power2(mg2)) + 2*mst12*power3(mg2) + (2*mst12 - 3*mt2)*power3(mst12) - 2*
     mg2*(5*mst14*mt2 + power3(mst12)))*power3(DeltaInv(mg2,mt2,mst12))))/(9.*
     mst12*(mst12 - mst22)*mst22*mt2) + (16*mg*mg2*DeltaInv(mg2,mt2,mst12)*log(
     mst12/Q2)*log(mt2/Q2)*(-3*mg2*mst12 - 3*mst14 - mst12*mt2 + DeltaInv(mg2,
     mt2,mst12)*(-((23*mst14 + 7*mst12*mt2)*power2(mg2)) + mst12*power3(mg2) +
     (3*mst12 - 5*mt2)*power3(mst12) - mg2*(20*mst14*mt2 + 13*power3(mst12))) +
     2*mg2*power2(DeltaInv(mg2,mt2,mst12))*((8*mst14 - mst12*mt2)*power3(mg2) -
     35*mg2*mt2*power3(mst12) - 7*power2(mg2)*(3*mst14*mt2 + 2*power3(mst12)) +
     mst12*power4(mg2) + (5*mst12 - 7*mt2)*power4(mst12))))/(3.*(mst12 - mst22)
     *mt2) + (16*mg*mg2*DeltaInv(mg2,mt2,mst12)*power2(log(mst12/Q2))*(-mst14 +
     DeltaInv(mg2,mt2,mst12)*(-4*mg2*mst14*mt2 - 3*mst14*power2(mg2) - 6*mg2*
     power3(mst12) - 2*mt2*power3(mst12) + power4(mst12)) + 2*mg2*power2(
     DeltaInv(mg2,mt2,mst12))*(2*mst14*power3(mg2) - 2*mg2*(mst12 + 5*mt2)*
     power3(mst12) - power2(mg2)*(3*mst14*mt2 + 2*power3(mst12)) + (2*mst12 -
     3*mt2)*power4(mst12))))/(3.*(mst12 - mst22)*mt2) - (16*mg*log(mst12/Q2)*(-
     mst12 - DeltaInv(mg2,mt2,mst12)*(23*mg2*mst14 + 6*mg2*mst12*mt2 + mst14*
     mt2 + 8*mst12*power2(mg2) - power3(mst12)) + 3*mg2*power2(DeltaInv(mg2,
     mt2,mst12))*(-((23*mst14 + 5*mst12*mt2)*power2(mg2)) + 3*mst12*power3(mg2)
     + (7*mst12 - 13*mt2)*power3(mst12) - mg2*(34*mst14*mt2 + 35*power3(mst12))
     ) + 36*power2(mg2)*power3(DeltaInv(mg2,mt2,mst12))*(2*mst14*power3(mg2) -
     2*mg2*(mst12 + 5*mt2)*power3(mst12) - power2(mg2)*(3*mst14*mt2 + 2*power3(
     mst12)) + (2*mst12 - 3*mt2)*power4(mst12))))/(9.*(mst12 - mst22)*mt2) - (
     16*mg*mg2*DeltaInv(mg2,mt2,mst12)*log(mg2/Q2)*log(mst12/Q2)*(-3*mg2*mst12
     - mst14 - mst12*mt2 + DeltaInv(mg2,mt2,mst12)*(-((17*mst14 + 7*mst12*mt2)*
     power2(mg2)) + mst12*power3(mg2) + (mst12 - mt2)*power3(mst12) - mg2*(12*
     mst14*mt2 + power3(mst12))) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*((4*
     mst14 - mst12*mt2)*power3(mg2) + mg2*(4*mst12 - 15*mt2)*power3(mst12) - 5*
     power2(mg2)*(3*mst14*mt2 + 2*power3(mst12)) + mst12*power4(mg2) + (mst12 -
     mt2)*power4(mst12))))/(3.*(mst12 - mst22)*mt2) + Fin3(mg2,mt2,mst12,Q2)*((
     32*mg*mg2*DeltaInv(mg2,mt2,mst12)*(-mst12 + DeltaInv(mg2,mt2,mst12)*(-6*
     mg2*mst14 - 4*mg2*mst12*mt2 - 2*mst14*mt2 - 3*mst12*power2(mg2) + power3(
     mst12)) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*(-((2*mst14 + 3*mst12*mt2)
     *power2(mg2)) + 2*mst12*power3(mg2) + (2*mst12 - 3*mt2)*power3(mst12) - 2*
     mg2*(5*mst14*mt2 + power3(mst12)))))/(3.*(mst12 - mst22)*mt2) + (12*mg*
     DeltaInv(mg2,mt2,mst12)*(-3*mg2*mst12 - 3*mst14 - mst12*mt2 + DeltaInv(
     mg2,mt2,mst12)*(-((23*mst14 + 7*mst12*mt2)*power2(mg2)) + mst12*power3(
     mg2) + (3*mst12 - 5*mt2)*power3(mst12) - mg2*(20*mst14*mt2 + 13*power3(
     mst12))) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*((8*mst14 - mst12*mt2)*
     power3(mg2) - 35*mg2*mt2*power3(mst12) - 7*power2(mg2)*(3*mst14*mt2 + 2*
     power3(mst12)) + mst12*power4(mg2) + (5*mst12 - 7*mt2)*power4(mst12))))/((
     mst12 - mst22)*mt2)) - (16*mg*mg2*DeltaInv(mg2,mst22,mt2)*log(mst22/Q2)*
     log(mt2/Q2)*(-3*mg2*mst22 - 3*mst24 - mst22*mt2 + DeltaInv(mg2,mst22,mt2)*
     (-((23*mst24 + 7*mst22*mt2)*power2(mg2)) + mst22*power3(mg2) + (3*mst22 -
     5*mt2)*power3(mst22) - mg2*(20*mst24*mt2 + 13*power3(mst22))) + 2*mg2*
     power2(DeltaInv(mg2,mst22,mt2))*((8*mst24 - mst22*mt2)*power3(mg2) - 35*
     mg2*mt2*power3(mst22) - 7*power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) +
     mst22*power4(mg2) + (5*mst22 - 7*mt2)*power4(mst22))))/(3.*(mst12 - mst22)
     *mt2) + (6*mg*mg2*power2(log(mg2/Q2))*((3*mg2*mst22 + 3*mst24 + mst22*mt2)
     *DeltaInv(mg2,mst22,mt2) + power2(DeltaInv(mg2,mst22,mt2))*((23*mst24 + 7*
     mst22*mt2)*power2(mg2) - mst22*power3(mg2) + (-3*mst22 + 5*mt2)*power3(
     mst22) + mg2*(20*mst24*mt2 + 13*power3(mst22))) + DeltaInv(mg2,mt2,mst12)*
     (-3*mg2*mst12 - 3*mst14 - mst12*mt2 + DeltaInv(mg2,mt2,mst12)*(-((23*mst14
     + 7*mst12*mt2)*power2(mg2)) + mst12*power3(mg2) + (3*mst12 - 5*mt2)*
     power3(mst12) - mg2*(20*mst14*mt2 + 13*power3(mst12))) + 2*mg2*power2(
     DeltaInv(mg2,mt2,mst12))*((8*mst14 - mst12*mt2)*power3(mg2) - 35*mg2*mt2*
     power3(mst12) - 7*power2(mg2)*(3*mst14*mt2 + 2*power3(mst12)) + mst12*
     power4(mg2) + (5*mst12 - 7*mt2)*power4(mst12))) - 2*mg2*power3(DeltaInv(
     mg2,mst22,mt2))*((8*mst24 - mst22*mt2)*power3(mg2) - 35*mg2*mt2*power3(
     mst22) - 7*power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) + mst22*power4(mg2)
     + (5*mst22 - 7*mt2)*power4(mst22))))/((mst12 - mst22)*mt2) - (2*mg*log(
     mg2/Q2)*(-6*mst12 + 6*mst22 + DeltaInv(mg2,mt2,mst12)*(-(mg2*(89*mst14 +
     44*mst12*mt2)) - 2*(37*mst12 + mt2)*power2(mg2) + power3(mg2) + 6*(-2*
     mst14*mt2 + power3(mst12))) + 6*mg2*power2(DeltaInv(mg2,mt2,mst12))*(-((
     73*mst14 + 27*mst12*mt2)*power2(mg2)) + 7*mst12*power3(mg2) + (13*mst12 -
     21*mt2)*power3(mst12) - mg2*(80*mst14*mt2 + 43*power3(mst12))) + DeltaInv(
     mg2,mst22,mt2)*(mg2*(89*mst24 + 44*mst22*mt2) + 2*(37*mst22 + mt2)*power2(
     mg2) - power3(mg2) - 6*(-2*mst24*mt2 + power3(mst22))) - 6*mg2*power2(
     DeltaInv(mg2,mst22,mt2))*(-((73*mst24 + 27*mst22*mt2)*power2(mg2)) + 7*
     mst22*power3(mg2) + (13*mst22 - 21*mt2)*power3(mst22) - mg2*(80*mst24*mt2
     + 43*power3(mst22))) + 36*power2(mg2)*power3(DeltaInv(mg2,mt2,mst12))*((8*
     mst14 - mst12*mt2)*power3(mg2) - 35*mg2*mt2*power3(mst12) - 7*power2(mg2)*
     (3*mst14*mt2 + 2*power3(mst12)) + mst12*power4(mg2) + (5*mst12 - 7*mt2)*
     power4(mst12)) - 36*power2(mg2)*power3(DeltaInv(mg2,mst22,mt2))*((8*mst24
     - mst22*mt2)*power3(mg2) - 35*mg2*mt2*power3(mst22) - 7*power2(mg2)*(3*
     mst24*mt2 + 2*power3(mst22)) + mst22*power4(mg2) + (5*mst22 - 7*mt2)*
     power4(mst22))))/((mst12 - mst22)*mt2) + (8*mg*mg2*power2(Pi)*((5*mg2*
     mst22 + 3*mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2) + power2(DeltaInv(
     mg2,mst22,mt2))*(5*(7*mst24 + 3*mst22*mt2)*power2(mg2) + 5*mst22*power3(
     mg2) + (-3*mst22 + 5*mt2)*power3(mst22) + mg2*(24*mst24*mt2 + 11*power3(
     mst22))) + DeltaInv(mg2,mt2,mst12)*(-5*mg2*mst12 - 3*mst14 - mst12*mt2 -
     DeltaInv(mg2,mt2,mst12)*(5*(7*mst14 + 3*mst12*mt2)*power2(mg2) + 5*mst12*
     power3(mg2) + (-3*mst12 + 5*mt2)*power3(mst12) + mg2*(24*mst14*mt2 + 11*
     power3(mst12))) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*((4*mst14 - 7*
     mst12*mt2)*power3(mg2) + mg2*(4*mst12 - 41*mt2)*power3(mst12) - power2(
     mg2)*(41*mst14*mt2 + 18*power3(mst12)) + 5*mst12*power4(mg2) + (5*mst12 -
     7*mt2)*power4(mst12))) - 2*mg2*power3(DeltaInv(mg2,mst22,mt2))*((4*mst24 -
     7*mst22*mt2)*power3(mg2) + mg2*(4*mst22 - 41*mt2)*power3(mst22) - power2(
     mg2)*(41*mst24*mt2 + 18*power3(mst22)) + 5*mst22*power4(mg2) + (5*mst22 -
     7*mt2)*power4(mst22))))/(9.*(mst12 - mst22)*mt2) + (8*mg*(-2*mg2*mst12*
     mst22*DeltaInv(mg2,mt2,mst12)*(103*mg2*mst12 + 64*mst14 - 2*mg2*mt2 + 19*
     mst12*mt2 + power2(mg2)) + 2*mg2*mst12*mst22*DeltaInv(mg2,mst22,mt2)*(103*
     mg2*mst22 + 64*mst24 - 2*mg2*mt2 + 19*mst22*mt2 + power2(mg2)) + (mst12 -
     mst22)*power2(mt2) - 42*mg2*mst12*mst22*power2(DeltaInv(mg2,mt2,mst12))*(
     5*(7*mst14 + 3*mst12*mt2)*power2(mg2) + 5*mst12*power3(mg2) + (-3*mst12 +
     5*mt2)*power3(mst12) + mg2*(24*mst14*mt2 + 11*power3(mst12))) + 42*mg2*
     mst12*mst22*power2(DeltaInv(mg2,mst22,mt2))*(5*(7*mst24 + 3*mst22*mt2)*
     power2(mg2) + 5*mst22*power3(mg2) + (-3*mst22 + 5*mt2)*power3(mst22) +
     mg2*(24*mst24*mt2 + 11*power3(mst22))) + 84*mst12*mst22*power2(mg2)*
     power3(DeltaInv(mg2,mt2,mst12))*((4*mst14 - 7*mst12*mt2)*power3(mg2) +
     mg2*(4*mst12 - 41*mt2)*power3(mst12) - power2(mg2)*(41*mst14*mt2 + 18*
     power3(mst12)) + 5*mst12*power4(mg2) + (5*mst12 - 7*mt2)*power4(mst12)) -
     84*mst12*mst22*power2(mg2)*power3(DeltaInv(mg2,mst22,mt2))*((4*mst24 - 7*
     mst22*mt2)*power3(mg2) + mg2*(4*mst22 - 41*mt2)*power3(mst22) - power2(
     mg2)*(41*mst24*mt2 + 18*power3(mst22)) + 5*mst22*power4(mg2) + (5*mst22 -
     7*mt2)*power4(mst22))))/(9.*mst12*(mst12 - mst22)*mst22*mt2) - (8*mg*log(
     mt2/Q2)*((mst12 - mst22)*(2*mst12*mst22 + mg2*mt2 + power2(mg2) + power2(
     mt2)) + mst12*DeltaInv(mg2,mst22,mt2)*(5*mg2*mst22*(13*mst24 + 3*mst22*
     mt2) + mst22*(103*mst22 - 6*mt2)*power2(mg2) + (-mst22 + mt2)*power3(mg2)
     + 2*mst22*(-(mst24*mt2) + power3(mst22)) - power4(mg2)) - 6*mg2*mst12*
     mst22*power2(DeltaInv(mg2,mt2,mst12))*((146*mst14 + 51*mst12*mt2)*power2(
     mg2) + (16*mst12 - mt2)*power3(mg2) + (-11*mst12 + 17*mt2)*power3(mst12) +
     mg2*(93*mst14*mt2 + 40*power3(mst12)) + power4(mg2)) + 6*mg2*mst12*mst22*
     power2(DeltaInv(mg2,mst22,mt2))*((146*mst24 + 51*mst22*mt2)*power2(mg2) +
     (16*mst22 - mt2)*power3(mg2) + (-11*mst22 + 17*mt2)*power3(mst22) + mg2*(
     93*mst24*mt2 + 40*power3(mst22)) + power4(mg2)) + mst22*DeltaInv(mg2,mt2,
     mst12)*(2*mst12*mst14*mt2 - 5*mg2*mst12*(13*mst14 + 3*mst12*mt2) + mst12*(
     -103*mst12 + 6*mt2)*power2(mg2) + (mst12 - mt2)*power3(mg2) + power4(mg2)
     - 2*power4(mst12)) + 72*mst12*mst22*power2(mg2)*power3(DeltaInv(mg2,mt2,
     mst12))*(4*(mst14 - mst12*mt2)*power3(mg2) + 4*mg2*(mst12 - 7*mt2)*power3(
     mst12) - 14*power2(mg2)*(2*mst14*mt2 + power3(mst12)) + 3*mst12*power4(
     mg2) + (3*mst12 - 4*mt2)*power4(mst12)) - 72*mst12*mst22*power2(mg2)*
     power3(DeltaInv(mg2,mst22,mt2))*(4*(mst24 - mst22*mt2)*power3(mg2) + 4*
     mg2*(mst22 - 7*mt2)*power3(mst22) - 14*power2(mg2)*(2*mst24*mt2 + power3(
     mst22)) + 3*mst22*power4(mg2) + (3*mst22 - 4*mt2)*power4(mst22))))/(9.*
     mst12*(mst12 - mst22)*mst22*mt2) - (16*mg*mg2*DeltaInv(mg2,mst22,mt2)*
     power2(log(mst22/Q2))*(-mst24 + DeltaInv(mg2,mst22,mt2)*(-4*mg2*mst24*mt2
     - 3*mst24*power2(mg2) - 6*mg2*power3(mst22) - 2*mt2*power3(mst22) +
     power4(mst22)) + 2*mg2*power2(DeltaInv(mg2,mst22,mt2))*(2*mst24*power3(
     mg2) - 2*mg2*(mst22 + 5*mt2)*power3(mst22) - power2(mg2)*(3*mst24*mt2 + 2*
     power3(mst22)) + (2*mst22 - 3*mt2)*power4(mst22))))/(3.*(mst12 - mst22)*
     mt2) + (16*mg*log(mst22/Q2)*(-mst22 - DeltaInv(mg2,mst22,mt2)*(23*mg2*
     mst24 + 6*mg2*mst22*mt2 + mst24*mt2 + 8*mst22*power2(mg2) - power3(mst22))
     + 3*mg2*power2(DeltaInv(mg2,mst22,mt2))*(-((23*mst24 + 5*mst22*mt2)*
     power2(mg2)) + 3*mst22*power3(mg2) + (7*mst22 - 13*mt2)*power3(mst22) -
     mg2*(34*mst24*mt2 + 35*power3(mst22))) + 36*power2(mg2)*power3(DeltaInv(
     mg2,mst22,mt2))*(2*mst24*power3(mg2) - 2*mg2*(mst22 + 5*mt2)*power3(mst22)
     - power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) + (2*mst22 - 3*mt2)*power4(
     mst22))))/(9.*(mst12 - mst22)*mt2) + (16*mg*mg2*DeltaInv(mg2,mst22,mt2)*
     log(mg2/Q2)*log(mst22/Q2)*(-3*mg2*mst22 - mst24 - mst22*mt2 + DeltaInv(
     mg2,mst22,mt2)*(-((17*mst24 + 7*mst22*mt2)*power2(mg2)) + mst22*power3(
     mg2) + (mst22 - mt2)*power3(mst22) - mg2*(12*mst24*mt2 + power3(mst22))) +
     2*mg2*power2(DeltaInv(mg2,mst22,mt2))*((4*mst24 - mst22*mt2)*power3(mg2) +
     mg2*(4*mst22 - 15*mt2)*power3(mst22) - 5*power2(mg2)*(3*mst24*mt2 + 2*
     power3(mst22)) + mst22*power4(mg2) + (mst22 - mt2)*power4(mst22))))/(3.*(
     mst12 - mst22)*mt2) + (16*mg*mg2*log(mg2/Q2)*log(mt2/Q2)*((5*mg2*mst22 +
     mst24 + mst22*mt2)*DeltaInv(mg2,mst22,mt2) + power2(DeltaInv(mg2,mst22,
     mt2))*((29*mst24 + 15*mst22*mt2)*power2(mg2) + 5*mst22*power3(mg2) + (-
     mst22 + mt2)*power3(mst22) - mg2*(-16*mst24*mt2 + power3(mst22))) +
     DeltaInv(mg2,mt2,mst12)*(-5*mg2*mst12 - mst14 - mst12*mt2 + DeltaInv(mg2,
     mt2,mst12)*(-((29*mst14 + 15*mst12*mt2)*power2(mg2)) - 5*mst12*power3(mg2)
     + (mst12 - mt2)*power3(mst12) + mg2*(-16*mst14*mt2 + power3(mst12))) + 2*
     mg2*power2(DeltaInv(mg2,mt2,mst12))*(-7*mst12*mt2*power3(mg2) + mg2*(8*
     mst12 - 21*mt2)*power3(mst12) - 7*power2(mg2)*(5*mst14*mt2 + 2*power3(
     mst12)) + 5*mst12*power4(mg2) + (mst12 - mt2)*power4(mst12))) - 2*mg2*
     power3(DeltaInv(mg2,mst22,mt2))*(-7*mst22*mt2*power3(mg2) + mg2*(8*mst22 -
     21*mt2)*power3(mst22) - 7*power2(mg2)*(5*mst24*mt2 + 2*power3(mst22)) + 5*
     mst22*power4(mg2) + (mst22 - mt2)*power4(mst22))))/(3.*(mst12 - mst22)*
     mt2) + (16*mg*mg2*power2(log(mt2/Q2))*((4*mg2*mst22 + 2*mst24 + mst22*mt2)
     *DeltaInv(mg2,mst22,mt2) + power2(DeltaInv(mg2,mst22,mt2))*((26*mst24 +
     11*mst22*mt2)*power2(mg2) + 2*mst22*power3(mg2) + (-2*mst22 + 3*mt2)*
     power3(mst22) + 6*mg2*(3*mst24*mt2 + power3(mst22))) + DeltaInv(mg2,mt2,
     mst12)*(-4*mg2*mst12 - 2*mst14 - mst12*mt2 - DeltaInv(mg2,mt2,mst12)*((26*
     mst14 + 11*mst12*mt2)*power2(mg2) + 2*mst12*power3(mg2) + (-2*mst12 + 3*
     mt2)*power3(mst12) + 6*mg2*(3*mst14*mt2 + power3(mst12))) + 2*mg2*power2(
     DeltaInv(mg2,mt2,mst12))*(4*(mst14 - mst12*mt2)*power3(mg2) + 4*mg2*(mst12
     - 7*mt2)*power3(mst12) - 14*power2(mg2)*(2*mst14*mt2 + power3(mst12)) + 3*
     mst12*power4(mg2) + (3*mst12 - 4*mt2)*power4(mst12))) + 2*mg2*power3(
     DeltaInv(mg2,mst22,mt2))*(-4*(mst24 - mst22*mt2)*power3(mg2) - 4*mg2*(
     mst22 - 7*mt2)*power3(mst22) + 14*power2(mg2)*(2*mst24*mt2 + power3(mst22)
     ) - 3*mst22*power4(mg2) + (-3*mst22 + 4*mt2)*power4(mst22))))/(3.*(mst12 -
     mst22)*mt2) + Fin3(mg2,mst22,mt2,Q2)*((-32*mg*mg2*DeltaInv(mg2,mst22,mt2)*
     (-mst22 + DeltaInv(mg2,mst22,mt2)*(-6*mg2*mst24 - 4*mg2*mst22*mt2 - 2*
     mst24*mt2 - 3*mst22*power2(mg2) + power3(mst22)) + 2*mg2*power2(DeltaInv(
     mg2,mst22,mt2))*(-((2*mst24 + 3*mst22*mt2)*power2(mg2)) + 2*mst22*power3(
     mg2) + (2*mst22 - 3*mt2)*power3(mst22) - 2*mg2*(5*mst24*mt2 + power3(
     mst22)))))/(3.*(mst12 - mst22)*mt2) - (12*mg*DeltaInv(mg2,mst22,mt2)*(-3*
     mg2*mst22 - 3*mst24 - mst22*mt2 + DeltaInv(mg2,mst22,mt2)*(-((23*mst24 +
     7*mst22*mt2)*power2(mg2)) + mst22*power3(mg2) + (3*mst22 - 5*mt2)*power3(
     mst22) - mg2*(20*mst24*mt2 + 13*power3(mst22))) + 2*mg2*power2(DeltaInv(
     mg2,mst22,mt2))*((8*mst24 - mst22*mt2)*power3(mg2) - 35*mg2*mt2*power3(
     mst22) - 7*power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) + mst22*power4(mg2)
     + (5*mst22 - 7*mt2)*power4(mst22))))/((mst12 - mst22)*mt2)) + (6*mg*
     DeltaInv(mg2,mt2,mst12)*power2(log(mst12/Q2))*(-(mst14*(3*mg2 + mt2)) - 3*
     power3(mst12) + DeltaInv(mg2,mt2,mst12)*(mst14*power3(mg2) - mg2*(13*mst12
     + 20*mt2)*power3(mst12) - power2(mg2)*(7*mst14*mt2 + 23*power3(mst12)) + (
     3*mst12 - 5*mt2)*power4(mst12)) + 2*mg2*power2(DeltaInv(mg2,mt2,mst12))*(-
     7*(2*mst12 + 3*mt2)*power2(mg2)*power3(mst12) + power3(mg2)*(-(mst14*mt2)
     + 8*power3(mst12)) + mst14*power4(mg2) - 35*mg2*mt2*power4(mst12) + (5*
     mst12 - 7*mt2)*power5(mst12))))/((mst12 - mst22)*mt2) - (2*mg*log(mst12/
     Q2)*(-5*mst12 - DeltaInv(mg2,mt2,mst12)*(42*mst14*mt2 + 14*mg2*(7*mst14 +
     mst12*mt2) + mst12*power2(mg2) + 59*power3(mst12)) + 6*power2(DeltaInv(
     mg2,mt2,mst12))*((7*mst14 - mst12*mt2)*power3(mg2) - 5*mg2*(7*mst12 + 15*
     mt2)*power3(mst12) - power2(mg2)*(36*mst14*mt2 + 79*power3(mst12)) +
     mst12*power4(mg2) + 2*(5*mst12 - 8*mt2)*power4(mst12)) + 36*mg2*power3(
     DeltaInv(mg2,mt2,mst12))*(-7*(2*mst12 + 3*mt2)*power2(mg2)*power3(mst12) +
     power3(mg2)*(-(mst14*mt2) + 8*power3(mst12)) + mst14*power4(mg2) - 35*mg2*
     mt2*power4(mst12) + (5*mst12 - 7*mt2)*power5(mst12))))/((mst12 - mst22)*
     mt2) + (6*mg*log(mst12/Q2)*log(mt2/Q2)*(-mst12 - DeltaInv(mg2,mt2,mst12)*(
     22*mg2*mst14 + 4*mg2*mst12*mt2 + 6*mst14*mt2 + 3*mst12*power2(mg2) + 7*
     power3(mst12)) + 2*power2(DeltaInv(mg2,mt2,mst12))*(-3*(2*mst14 + mst12*
     mt2)*power3(mg2) - mg2*(10*mst12 + 39*mt2)*power3(mst12) - 2*power2(mg2)*(
     16*mst14*mt2 + 27*power3(mst12)) + 2*mst12*power4(mg2) + 2*(2*mst12 - 3*
     mt2)*power4(mst12)) + 8*mg2*power3(DeltaInv(mg2,mt2,mst12))*(-14*(mst12 +
     2*mt2)*power2(mg2)*power3(mst12) + 4*power3(mg2)*(-(mst14*mt2) + power3(
     mst12)) + 3*mst14*power4(mg2) + 4*mg2*(mst12 - 7*mt2)*power4(mst12) + (3*
     mst12 - 4*mt2)*power5(mst12))))/((mst12 - mst22)*mt2) - (6*mg*log(mg2/Q2)*
     log(mst12/Q2)*(-mst12 - DeltaInv(mg2,mt2,mst12)*(4*mst14*mt2 + 4*mg2*(4*
     mst14 + mst12*mt2) + 3*mst12*power2(mg2) + power3(mst12)) + 2*power2(
     DeltaInv(mg2,mt2,mst12))*(-((7*mst14 + 3*mst12*mt2)*power3(mg2)) + mg2*(3*
     mst12 - 19*mt2)*power3(mst12) - power2(mg2)*(25*mst14*mt2 + 31*power3(
     mst12)) + 2*mst12*power4(mg2) + (mst12 - mt2)*power4(mst12)) + 4*mg2*
     power3(DeltaInv(mg2,mt2,mst12))*(-7*mst14*mt2*power3(mg2) - 7*(2*mst12 +
     5*mt2)*power2(mg2)*power3(mst12) + 5*mst14*power4(mg2) + mg2*(8*mst12 -
     21*mt2)*power4(mst12) + (mst12 - mt2)*power5(mst12))))/((mst12 - mst22)*
     mt2) - (2*mg*log(mt2/Q2)*(-7*mst12 + 7*mst22 - DeltaInv(mg2,mt2,mst12)*(
     54*mst14*mt2 + mg2*(317*mst14 + 50*mst12*mt2) + (87*mst12 - 2*mt2)*power2(
     mg2) + power3(mg2) + 73*power3(mst12)) + DeltaInv(mg2,mst22,mt2)*(54*
     mst24*mt2 + mg2*(317*mst24 + 50*mst22*mt2) + (87*mst22 - 2*mt2)*power2(
     mg2) + power3(mg2) + 73*power3(mst22)) + 12*power2(DeltaInv(mg2,mt2,mst12)
     )*(-2*(27*mst14 + 8*mst12*mt2)*power3(mg2) - 2*mg2*(5*mst12 + 42*mt2)*
     power3(mst12) - 14*power2(mg2)*(7*mst14*mt2 + 10*power3(mst12)) + 5*mst12*
     power4(mg2) + (7*mst12 - 10*mt2)*power4(mst12)) - 12*power2(DeltaInv(mg2,
     mst22,mt2))*(-2*(27*mst24 + 8*mst22*mt2)*power3(mg2) - 2*mg2*(5*mst22 +
     42*mt2)*power3(mst22) - 14*power2(mg2)*(7*mst24*mt2 + 10*power3(mst22)) +
     5*mst22*power4(mg2) + (7*mst22 - 10*mt2)*power4(mst22)) + 36*mg2*power3(
     DeltaInv(mg2,mt2,mst12))*(-42*(mst12 + 3*mt2)*power2(mg2)*power3(mst12) -
     6*power3(mg2)*(6*mst14*mt2 + power3(mst12)) + (19*mst14 - mst12*mt2)*
     power4(mg2) + 21*mg2*(mst12 - 4*mt2)*power4(mst12) + mst12*power5(mg2) + (
     7*mst12 - 9*mt2)*power5(mst12)) - 36*mg2*power3(DeltaInv(mg2,mst22,mt2))*(
     -42*(mst22 + 3*mt2)*power2(mg2)*power3(mst22) - 6*power3(mg2)*(6*mst24*mt2
     + power3(mst22)) + (19*mst24 - mst22*mt2)*power4(mg2) + 21*mg2*(mst22 - 4*
     mt2)*power4(mst22) + mst22*power5(mg2) + (7*mst22 - 9*mt2)*power5(mst22)))
     )/((mst12 - mst22)*mt2) + (6*mg*power2(log(mt2/Q2))*(-mst12 + mst22 -
     DeltaInv(mg2,mt2,mst12)*(22*mg2*mst14 + 5*mg2*mst12*mt2 + 5*mst14*mt2 + 6*
     mst12*power2(mg2) + 4*power3(mst12)) + DeltaInv(mg2,mst22,mt2)*(22*mg2*
     mst24 + 5*mg2*mst22*mt2 + 5*mst24*mt2 + 6*mst22*power2(mg2) + 4*power3(
     mst22)) + power2(DeltaInv(mg2,mt2,mst12))*(-((36*mst14 + 13*mst12*mt2)*
     power3(mg2)) - mg2*(4*mst12 + 63*mt2)*power3(mst12) - 7*power2(mg2)*(11*
     mst14*mt2 + 14*power3(mst12)) + 5*mst12*power4(mg2) + (5*mst12 - 7*mt2)*
     power4(mst12)) + power2(DeltaInv(mg2,mst22,mt2))*((36*mst24 + 13*mst22*
     mt2)*power3(mg2) + mg2*(4*mst22 + 63*mt2)*power3(mst22) + power2(mg2)*(77*
     mst24*mt2 + 98*power3(mst22)) - 5*mst22*power4(mg2) + (-5*mst22 + 7*mt2)*
     power4(mst22)) + 2*mg2*power3(DeltaInv(mg2,mt2,mst12))*(-42*(mst12 + 3*
     mt2)*power2(mg2)*power3(mst12) - 6*power3(mg2)*(6*mst14*mt2 + power3(
     mst12)) + (19*mst14 - mst12*mt2)*power4(mg2) + 21*mg2*(mst12 - 4*mt2)*
     power4(mst12) + mst12*power5(mg2) + (7*mst12 - 9*mt2)*power5(mst12)) - 2*
     mg2*power3(DeltaInv(mg2,mst22,mt2))*(-42*(mst22 + 3*mt2)*power2(mg2)*
     power3(mst22) - 6*power3(mg2)*(6*mst24*mt2 + power3(mst22)) + (19*mst24 -
     mst22*mt2)*power4(mg2) + 21*mg2*(mst22 - 4*mt2)*power4(mst22) + mst22*
     power5(mg2) + (7*mst22 - 9*mt2)*power5(mst22))))/((mst12 - mst22)*mt2) - (
     6*mg*DeltaInv(mg2,mst22,mt2)*power2(log(mst22/Q2))*(-(mst24*(3*mg2 + mt2))
     - 3*power3(mst22) + DeltaInv(mg2,mst22,mt2)*(mst24*power3(mg2) - mg2*(13*
     mst22 + 20*mt2)*power3(mst22) - power2(mg2)*(7*mst24*mt2 + 23*power3(
     mst22)) + (3*mst22 - 5*mt2)*power4(mst22)) + 2*mg2*power2(DeltaInv(mg2,
     mst22,mt2))*(-7*(2*mst22 + 3*mt2)*power2(mg2)*power3(mst22) + power3(mg2)*
     (-(mst24*mt2) + 8*power3(mst22)) + mst24*power4(mg2) - 35*mg2*mt2*power4(
     mst22) + (5*mst22 - 7*mt2)*power5(mst22))))/((mst12 - mst22)*mt2) + (2*mg*
     log(mst22/Q2)*(-5*mst22 - DeltaInv(mg2,mst22,mt2)*(42*mst24*mt2 + 14*mg2*(
     7*mst24 + mst22*mt2) + mst22*power2(mg2) + 59*power3(mst22)) + 6*power2(
     DeltaInv(mg2,mst22,mt2))*((7*mst24 - mst22*mt2)*power3(mg2) - 5*mg2*(7*
     mst22 + 15*mt2)*power3(mst22) - power2(mg2)*(36*mst24*mt2 + 79*power3(
     mst22)) + mst22*power4(mg2) + 2*(5*mst22 - 8*mt2)*power4(mst22)) + 36*mg2*
     power3(DeltaInv(mg2,mst22,mt2))*(-7*(2*mst22 + 3*mt2)*power2(mg2)*power3(
     mst22) + power3(mg2)*(-(mst24*mt2) + 8*power3(mst22)) + mst24*power4(mg2)
     - 35*mg2*mt2*power4(mst22) + (5*mst22 - 7*mt2)*power5(mst22))))/((mst12 -
     mst22)*mt2) - (6*mg*log(mst22/Q2)*log(mt2/Q2)*(-mst22 - DeltaInv(mg2,
     mst22,mt2)*(22*mg2*mst24 + 4*mg2*mst22*mt2 + 6*mst24*mt2 + 3*mst22*power2(
     mg2) + 7*power3(mst22)) + 2*power2(DeltaInv(mg2,mst22,mt2))*(-3*(2*mst24 +
     mst22*mt2)*power3(mg2) - mg2*(10*mst22 + 39*mt2)*power3(mst22) - 2*power2(
     mg2)*(16*mst24*mt2 + 27*power3(mst22)) + 2*mst22*power4(mg2) + 2*(2*mst22
     - 3*mt2)*power4(mst22)) + 8*mg2*power3(DeltaInv(mg2,mst22,mt2))*(-14*(
     mst22 + 2*mt2)*power2(mg2)*power3(mst22) + 4*power3(mg2)*(-(mst24*mt2) +
     power3(mst22)) + 3*mst24*power4(mg2) + 4*mg2*(mst22 - 7*mt2)*power4(mst22)
     + (3*mst22 - 4*mt2)*power5(mst22))))/((mst12 - mst22)*mt2) + (2*mg*(18*(-
     mst12 + mst22) - DeltaInv(mg2,mt2,mst12)*(121*mst14*mt2 + mg2*(583*mst14 +
     116*mst12*mt2) + (190*mst12 - mt2)*power2(mg2) + power3(mg2) + 150*power3(
     mst12)) + DeltaInv(mg2,mst22,mt2)*(121*mst24*mt2 + mg2*(583*mst24 + 116*
     mst22*mt2) + (190*mst22 - mt2)*power2(mg2) + power3(mg2) + 150*power3(
     mst22)) + 42*power2(DeltaInv(mg2,mt2,mst12))*(-((29*mst14 + 10*mst12*mt2)*
     power3(mg2)) - mg2*(7*mst12 + 44*mt2)*power3(mst12) - power2(mg2)*(52*
     mst14*mt2 + 67*power3(mst12)) + 3*mst12*power4(mg2) + 2*(2*mst12 - 3*mt2)*
     power4(mst12)) - 42*power2(DeltaInv(mg2,mst22,mt2))*(-((29*mst24 + 10*
     mst22*mt2)*power3(mg2)) - mg2*(7*mst22 + 44*mt2)*power3(mst22) - power2(
     mg2)*(52*mst24*mt2 + 67*power3(mst22)) + 3*mst22*power4(mg2) + 2*(2*mst22
     - 3*mt2)*power4(mst22)) + 84*mg2*power3(DeltaInv(mg2,mt2,mst12))*(-7*(4*
     mst12 + 13*mt2)*power2(mg2)*power3(mst12) - power3(mg2)*(29*mst14*mt2 + 6*
     power3(mst12)) + (14*mst14 - mst12*mt2)*power4(mg2) + mg2*(13*mst12 - 63*
     mt2)*power4(mst12) + mst12*power5(mg2) + 2*(3*mst12 - 4*mt2)*power5(mst12)
     ) - 84*mg2*power3(DeltaInv(mg2,mst22,mt2))*(-7*(4*mst22 + 13*mt2)*power2(
     mg2)*power3(mst22) - power3(mg2)*(29*mst24*mt2 + 6*power3(mst22)) + (14*
     mst24 - mst22*mt2)*power4(mg2) + mg2*(13*mst22 - 63*mt2)*power4(mst22) +
     mst22*power5(mg2) + 2*(3*mst22 - 4*mt2)*power5(mst22))))/((mst12 - mst22)*
     mt2) + (mg*power2(Pi)*(-mst12 + mst22 - DeltaInv(mg2,mt2,mst12)*(28*mg2*
     mst14 + 6*mg2*mst12*mt2 + 6*mst14*mt2 + 9*mst12*power2(mg2) + 7*power3(
     mst12)) + DeltaInv(mg2,mst22,mt2)*(28*mg2*mst24 + 6*mg2*mst22*mt2 + 6*
     mst24*mt2 + 9*mst22*power2(mg2) + 7*power3(mst22)) + 2*power2(DeltaInv(
     mg2,mt2,mst12))*(-((29*mst14 + 10*mst12*mt2)*power3(mg2)) - mg2*(7*mst12 +
     44*mt2)*power3(mst12) - power2(mg2)*(52*mst14*mt2 + 67*power3(mst12)) + 3*
     mst12*power4(mg2) + 2*(2*mst12 - 3*mt2)*power4(mst12)) + 4*mg2*power3(
     DeltaInv(mg2,mt2,mst12))*(-7*(4*mst12 + 13*mt2)*power2(mg2)*power3(mst12)
     - power3(mg2)*(29*mst14*mt2 + 6*power3(mst12)) + (14*mst14 - mst12*mt2)*
     power4(mg2) + mg2*(13*mst12 - 63*mt2)*power4(mst12) + mst12*power5(mg2) +
     2*(3*mst12 - 4*mt2)*power5(mst12)) + 2*power2(DeltaInv(mg2,mst22,mt2))*((
     29*mst24 + 10*mst22*mt2)*power3(mg2) + mg2*(7*mst22 + 44*mt2)*power3(
     mst22) + power2(mg2)*(52*mst24*mt2 + 67*power3(mst22)) - 3*mst22*power4(
     mg2) + 6*mt2*power4(mst22) - 4*power5(mst22)) - 4*mg2*power3(DeltaInv(mg2,
     mst22,mt2))*(-7*(4*mst22 + 13*mt2)*power2(mg2)*power3(mst22) - power3(mg2)
     *(29*mst24*mt2 + 6*power3(mst22)) + (14*mst24 - mst22*mt2)*power4(mg2) +
     mg2*(13*mst22 - 63*mt2)*power4(mst22) + mst22*power5(mg2) + 2*(3*mst22 -
     4*mt2)*power5(mst22))))/((mst12 - mst22)*mt2) + (6*mg*log(mg2/Q2)*log(
     mst22/Q2)*(-mst22 - DeltaInv(mg2,mst22,mt2)*(4*mst24*mt2 + 4*mg2*(4*mst24
     + mst22*mt2) + 3*mst22*power2(mg2) + power3(mst22)) + 2*power2(DeltaInv(
     mg2,mst22,mt2))*(-((7*mst24 + 3*mst22*mt2)*power3(mg2)) + mg2*(3*mst22 -
     19*mt2)*power3(mst22) - power2(mg2)*(25*mst24*mt2 + 31*power3(mst22)) + 2*
     mst22*power4(mg2) + (mst22 - mt2)*power4(mst22)) + 4*mg2*power3(DeltaInv(
     mg2,mst22,mt2))*(-7*mst24*mt2*power3(mg2) - 7*(2*mst22 + 5*mt2)*power2(
     mg2)*power3(mst22) + 5*mst24*power4(mg2) + mg2*(8*mst22 - 21*mt2)*power4(
     mst22) + (mst22 - mt2)*power5(mst22))))/((mst12 - mst22)*mt2) + (6*mg*log(
     mg2/Q2)*log(mt2/Q2)*(-mst12 + mst22 - DeltaInv(mg2,mt2,mst12)*(22*mg2*
     mst14 + 6*mg2*mst12*mt2 + 4*mst14*mt2 + 9*mst12*power2(mg2) + power3(
     mst12)) + DeltaInv(mg2,mst22,mt2)*(22*mg2*mst24 + 6*mg2*mst22*mt2 + 4*
     mst24*mt2 + 9*mst22*power2(mg2) + power3(mst22)) + 2*power2(DeltaInv(mg2,
     mt2,mst12))*(-10*(3*mst14 + mst12*mt2)*power3(mg2) + 6*mg2*(mst12 - 4*mt2)
     *power3(mst12) - power2(mg2)*(45*mst14*mt2 + 44*power3(mst12)) + 3*mst12*
     power4(mg2) + (mst12 - mt2)*power4(mst12)) + power2(DeltaInv(mg2,mst22,
     mt2))*(20*(3*mst24 + mst22*mt2)*power3(mg2) - 12*mg2*(mst22 - 4*mt2)*
     power3(mst22) + power2(mg2)*(90*mst24*mt2 + 88*power3(mst22)) - 6*mst22*
     power4(mg2) + 2*(-mst22 + mt2)*power4(mst22)) + 4*mg2*power3(DeltaInv(mg2,
     mt2,mst12))*(-14*(mst12 + 5*mt2)*power2(mg2)*power3(mst12) - 14*power3(
     mg2)*(2*mst14*mt2 + power3(mst12)) + (13*mst14 - mst12*mt2)*power4(mg2) +
     mg2*(13*mst12 - 28*mt2)*power4(mst12) + mst12*power5(mg2) + (mst12 - mt2)*
     power5(mst12)) - 4*mg2*power3(DeltaInv(mg2,mst22,mt2))*(-14*(mst22 + 5*
     mt2)*power2(mg2)*power3(mst22) - 14*power3(mg2)*(2*mst24*mt2 + power3(
     mst22)) + (13*mst24 - mst22*mt2)*power4(mg2) + mg2*(13*mst22 - 28*mt2)*
     power4(mst22) + mst22*power5(mg2) + (mst22 - mt2)*power5(mst22))))/((mst12
     - mst22)*mt2)) + (207 + (12*mg2)/msb12 + (12*mg2)/msb22 + (48*mg2)/msd12 +
     (40*msd12)/mg2 + (40*msd12)/(-mg2 + msd12) + (48*mg2)/msd22 + (40*msd22)/
     mg2 + (40*msd22)/(-mg2 + msd22) + (12*mg2)/mst12 + (12*mg2)/mst22 - (24*
     mg2)/mt2 + (12*mt2)/mst12 + (12*mt2)/mst22 + (12*mg2*DeltaInv(mg2,mt2,
     mst12)*(40*mg2*mst12 + mst14 - 3*mg2*mt2 - mst12*mt2 + power2(mg2)))/mt2 +
     (12*mg2*DeltaInv(mg2,mst22,mt2)*(40*mg2*mst22 + mst24 - 3*mg2*mt2 - mst22*
     mt2 + power2(mg2)))/mt2 + (40*msd14)/power2(mg2 - msd12) + (40*msd24)/
     power2(mg2 - msd22) - (92*power3(msd12))/(mg2*power2(mg2 - msd12)) + (52*
     power3(msd12))/power3(mg2 - msd12) - (92*power3(msd22))/(mg2*power2(mg2 -
     msd22)) + (52*power3(msd22))/power3(mg2 - msd22) + (504*power2(mg2)*
     power2(DeltaInv(mg2,mt2,mst12))*(16*mg2*mst14 + 6*mg2*mst12*mt2 + 4*mst14*
     mt2 + 7*mst12*power2(mg2) + power3(mst12)))/mt2 + (504*power2(mg2)*power2(
     DeltaInv(mg2,mst22,mt2))*(16*mg2*mst24 + 6*mg2*mst22*mt2 + 4*mst24*mt2 +
     7*mst22*power2(mg2) + power3(mst22)))/mt2 - (52*power4(msd12))/power4(mg2
     - msd12) - (52*power4(msd22))/power4(mg2 - msd22) - (1008*power2(mg2)*
     power3(DeltaInv(mg2,mt2,mst12))*(-((mst14 + 6*mst12*mt2)*power3(mg2)) +
     mg2*(5*mst12 - 16*mt2)*power3(mst12) - power2(mg2)*(25*mst14*mt2 + 9*
     power3(mst12)) + 4*mst12*power4(mg2) + (mst12 - mt2)*power4(mst12)))/mt2 +
     (1008*power2(mg2)*power3(DeltaInv(mg2,mst22,mt2))*((mst24 + 6*mst22*mt2)*
     power3(mg2) + mg2*(-5*mst22 + 16*mt2)*power3(mst22) + power2(mg2)*(25*
     mst24*mt2 + 9*power3(mst22)) - 4*mst22*power4(mg2) + (-mst22 + mt2)*
     power4(mst22)))/mt2 + (72*power5(msd12))/(mg2*power4(mg2 - msd12)) + (20*
     power5(msd12))/power5(-mg2 + msd12) + (72*power5(msd22))/(mg2*power4(mg2 -
     msd22)) + (20*power5(msd22))/power5(-mg2 + msd22) + (20*power6(msd12))/
     power6(mg2 - msd12) + (20*power6(msd22))/power6(mg2 - msd22) - (20*power7(
     msd12))/(mg2*power6(mg2 - msd12)) - (20*power7(msd22))/(mg2*power6(mg2 -
     msd22)))/27. - (4*log(mg2/Q2)*(30 + (3*mg2)/msb12 + (3*msb12)/(mg2 -
     msb12) + (3*mg2)/msb22 + (3*msb22)/(mg2 - msb22) + (12*mg2)/msd12 + (6*
     msd12)/mg2 + (6*msd12)/(mg2 - msd12) + (12*mg2)/msd22 + (6*msd22)/mg2 + (
     6*msd22)/(mg2 - msd22) - (3*power2(mg2))/(mst12*mt2) - (3*power2(mg2))/(
     mst22*mt2) + (6*msd14)/power2(mg2 - msd12) + (6*msd24)/power2(mg2 - msd22)
     + (3*DeltaInv(mg2,mt2,mst12)*power2(mg2)*(4*mg2*mst12 - mg2*mt2 + 2*mst12*
     mt2 + power2(mg2) + 7*power2(mst12)))/(mst12*mt2) + (3*DeltaInv(mg2,mst22,
     mt2)*power2(mg2)*(4*mg2*mst22 - mg2*mt2 + 2*mst22*mt2 + power2(mg2) + 7*
     power2(mst22)))/(mst22*mt2) - (12*power3(msd12))/(mg2*power2(mg2 - msd12))
     + (6*power3(msd12))/power3(mg2 - msd12) - (12*power3(msd22))/(mg2*power2(
     mg2 - msd22)) + (6*power3(msd22))/power3(mg2 - msd22) + (18*power2(mg2)*
     power2(DeltaInv(mg2,mt2,mst12))*(mst14*mt2 + mg2*(7*mst14 + 12*mst12*mt2)
     + (19*mst12 + mt2)*power2(mg2) - power3(mg2) - power3(mst12)))/mt2 + (18*
     power2(mg2)*power2(DeltaInv(mg2,mst22,mt2))*(mst24*mt2 + mg2*(7*mst24 +
     12*mst22*mt2) + (19*mst22 + mt2)*power2(mg2) - power3(mg2) - power3(mst22)
     ))/mt2 - (108*power3(mg2)*(-5*(mst24 + mst22*mt2)*power2(mg2) + 3*mst22*
     power3(mg2) + (mst22 - mt2)*power3(mst22) + mg2*(-10*mst24*mt2 + power3(
     mst22)))*power3(DeltaInv(mg2,mst22,mt2)))/mt2 - (108*power3(mg2)*(-5*(
     mst14 + mst12*mt2)*power2(mg2) + 3*mst12*power3(mg2) + (mst12 - mt2)*
     power3(mst12) + mg2*(-10*mst14*mt2 + power3(mst12)))*power3(DeltaInv(mg2,
     mt2,mst12)))/mt2 - (6*power4(msd12))/power4(mg2 - msd12) - (6*power4(
     msd22))/power4(mg2 - msd22) + (8*power5(msd12))/(mg2*power4(mg2 - msd12))
     + (2*power5(msd12))/power5(-mg2 + msd12) + (8*power5(msd22))/(mg2*power4(
     mg2 - msd22)) + (2*power5(msd22))/power5(-mg2 + msd22) + (2*power6(msd12))
     /power6(mg2 - msd12) + (2*power6(msd22))/power6(mg2 - msd22) - (2*power7(
     msd12))/(mg2*power6(mg2 - msd12)) - (2*power7(msd22))/(mg2*power6(mg2 -
     msd22))))/27. - (log(mg2/Q2)*(30 + (3*msb12)/(mg2 - msb12) + (3*msb22)/(
     mg2 - msb22) - (10*msd12)/mg2 + (22*msd12)/(mg2 - msd12) - (10*msd22)/mg2
     + (22*msd22)/(mg2 - msd22) + (6*mg2)/mt2 + (3*mg2*DeltaInv(mg2,mt2,mst12)*
     (74*mg2*mst12 + 11*mst14 + 3*mg2*mt2 + 13*mst12*mt2 - power2(mg2)))/mt2 +
     (3*mg2*DeltaInv(mg2,mst22,mt2)*(74*mg2*mst22 + 11*mst24 + 3*mg2*mt2 + 13*
     mst22*mt2 - power2(mg2)))/mt2 - (10*msd14)/power2(mg2 - msd12) - (10*
     msd24)/power2(mg2 - msd22) - (10*power3(msd12))/(mg2*power2(mg2 - msd12))
     + (20*power3(msd12))/power3(mg2 - msd12) - (10*power3(msd22))/(mg2*power2(
     mg2 - msd22)) + (20*power3(msd22))/power3(mg2 - msd22) + (36*mg2*power2(
     DeltaInv(mg2,mt2,mst12))*((47*mst14 + 17*mst12*mt2)*power2(mg2) + (-mst12
     + mt2)*power3(mst12) + 2*mg2*(11*mst14*mt2 + power3(mst12))))/mt2 + (36*
     mg2*power2(DeltaInv(mg2,mst22,mt2))*((47*mst24 + 17*mst22*mt2)*power2(mg2)
     + (-mst22 + mt2)*power3(mst22) + 2*mg2*(11*mst24*mt2 + power3(mst22))))/
     mt2 - (20*power4(msd12))/power4(mg2 - msd12) - (20*power4(msd22))/power4(
     mg2 - msd22) - (216*power2(mg2)*power3(DeltaInv(mg2,mt2,mst12))*((4*mst14
     - mst12*mt2)*power3(mg2) + mg2*(4*mst12 - 15*mt2)*power3(mst12) - 5*
     power2(mg2)*(3*mst14*mt2 + 2*power3(mst12)) + mst12*power4(mg2) + (mst12 -
     mt2)*power4(mst12)))/mt2 + (216*power2(mg2)*power3(DeltaInv(mg2,mst22,mt2)
     )*((-4*mst24 + mst22*mt2)*power3(mg2) + mg2*(-4*mst22 + 15*mt2)*power3(
     mst22) + 5*power2(mg2)*(3*mst24*mt2 + 2*power3(mst22)) - mst22*power4(mg2)
     + (-mst22 + mt2)*power4(mst22)))/mt2 - (4*power5(msd12))/(mg2*power4(mg2 -
     msd12)) + (24*power5(msd12))/power5(mg2 - msd12) - (4*power5(msd22))/(mg2*
     power4(mg2 - msd22)) + (24*power5(msd22))/power5(mg2 - msd22) - (24*
     power6(msd12))/power6(mg2 - msd12) - (24*power6(msd22))/power6(mg2 -
     msd22) + (24*power7(msd12))/(mg2*power6(mg2 - msd12)) + (24*power7(msd22))
     /(mg2*power6(mg2 - msd22))))/6.;

   return power4(g3) * result * twoLoop;
}

/// 2-loop O(alpha_t*alpha_s) contributions to Delta alpha_s [arXiv:1009.5455]
Real delta_alpha_s_2loop_at_as(const Parameters& pars)
{
   using std::log;
   const Real g3    = pars.g3;
   const Real yt    = pars.yt;
   const Real xt    = pars.xt;
   const Real xb    = pars.xb;
   const Real mt    = pars.mt;
   const Real mt2   = power2(pars.mt);
   const Real mb    = pars.mb;
   const Real mst1  = pars.mst1;
   const Real mst12 = power2(pars.mst1);
   const Real mst14 = power4(pars.mst1);
   const Real mst2  = pars.mst2;
   const Real mst22 = power2(pars.mst2);
   const Real mst24 = power4(pars.mst2);
   //   const Real msb1  = pars.msb1;
   const Real msb12 = power2(pars.msb1);
   const Real msb14 = power4(pars.msb1);
   //   const Real msb2  = pars.msb2;
   const Real msb22 = power2(pars.msb2);
   const Real msb24 = power4(pars.msb2);
   const Real mw2   = power2(pars.mw);
   const Real mz2   = power2(pars.mz);
   const Real mh2   = power2(pars.mh);
   const Real mH2   = power2(pars.mH);
   const Real mC2   = power2(pars.mC);
   const Real mA2   = power2(pars.mA);
   const Real mu    = pars.mu;
   const Real mu2   = power2(pars.mu);
   const Real tb    = pars.tb;
   const Real sb    = tb / std::sqrt(1. + power2(tb));
   const Real cb    = 1. / std::sqrt(1. + power2(tb));
   const Real Q2    = power2(pars.Q);
   const Real snt   = calc_sin_theta(mt, xt, mst12, mst22);
   const Real snb   = calc_sin_theta(mb, xb, msb12, msb22);
   const Real alpha = calc_alpha(mh2, mH2, tb);
   const Real sa    = std::sin(alpha);
   const Real ca    = std::cos(alpha);

   const Real result =
   -0.16666666666666666 - mh2/(24.*mst12) - mh2/(24.*mst22) - msb12/(12.*mst22)
     - mst12/(12.*mst22) - mst22/(12.*msb12) - mst22/(12.*mst12) - mt2/(6.*mh2)
     + mt2/(12.*msb12) - mt2/(12.*mst12) + mu2/(12.*msb12) + mu2/(12.*mst12) +
     mu2/(6.*mst22) - mu2/(2.*mt2) - mw2/(12.*msb12) - mw2/(12.*mst22) - (35*
     mw2)/(216.*mt2) + (35*mw2)/(216.*(mt2 - mw2)) - mz2/(24.*mst12) - mz2/(24.
     *mst22) + (mh2*mt2*DeltaInv(mh2,mst12,mst12))/6. + (mt2*(22*mh2 + 101*mt2)
     *DeltaInv(mh2,mt2,mt2))/6. + (mH2*mu2*DeltaInv(mH2,mst22,mst12))/12. - (
     msb12*mu2*DeltaInv(msb12,mt2,mu2))/6. + (msb14*mu2*DeltaInv(msb12,mt2,mu2)
     )/(6.*mt2) + (mA2*mu2*DeltaInv(mst22,mA2,mst12))/12. + (mC2*mu2*DeltaInv(
     mst22,msb12,mC2))/6. + (mh2*mt2*DeltaInv(mst22,mst22,mh2))/6. - (mst22*
     mu2*DeltaInv(mst22,mt2,mu2))/6. + (mst24*mu2*DeltaInv(mst22,mt2,mu2))/(6.*
     mt2) - (19*mt2*mz2*DeltaInv(mt2,mt2,mz2))/6. - (mst12*mu2*DeltaInv(mt2,
     mu2,mst12))/6. + (mst14*mu2*DeltaInv(mt2,mu2,mst12))/(6.*mt2) + (mt2*mw2*
     DeltaInv(mw2,msb12,mst12))/6. - (49*DeltaInv(mt2,mt2,mz2)*power2(mt2))/2.
      + mt2*DeltaInv(mh2,mt2,mt2)*log(mh2/Q2)*log(mt2/Q2)*(mh2 + 3*mt2 + 12*
     mh2*DeltaInv(mh2,mt2,mt2)*power2(mt2)) - (DeltaInv(msb12,mt2,mu2)*power2(
     mu2))/2. + (20*msb12*DeltaInv(msb12,mt2,mu2)*power2(mu2))/(3.*mt2) - (
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/2. + (20*mst22*DeltaInv(mst22,mt2,
     mu2)*power2(mu2))/(3.*mt2) - (DeltaInv(mt2,mu2,mst12)*power2(mu2))/2. + (
     20*mst12*DeltaInv(mt2,mu2,mst12)*power2(mu2))/(3.*mt2) - (35*power2(mw2))/
     (216.*power2(mt2 - mw2)) + mt2*DeltaInv(mH2,mt2,mt2)*(1 + 3*mH2*mt2*
     DeltaInv(mH2,mt2,mt2))*Fin3(mH2,mt2,mt2,Q2)*power2(sa) - mt2*DeltaInv(mh2,
     mt2,mt2)*log(mh2/Q2)*log(mt2/Q2)*(mh2 + 3*mt2 + 12*mh2*DeltaInv(mh2,mt2,
     mt2)*power2(mt2))*power2(sa) + mt2*DeltaInv(mH2,mt2,mt2)*log(mH2/Q2)*log(
     mt2/Q2)*(mH2 + 3*mt2 + 12*mH2*DeltaInv(mH2,mt2,mt2)*power2(mt2))*power2(
     sa) + Fin3(mh2,mt2,mt2,Q2)*(mt2*DeltaInv(mh2,mt2,mt2)*(1 + 3*mh2*mt2*
     DeltaInv(mh2,mt2,mt2)) - mt2*DeltaInv(mh2,mt2,mt2)*(1 + 3*mh2*mt2*
     DeltaInv(mh2,mt2,mt2))*power2(sa)) + ((mst12/(mst12 - mu2) - mu2/mst12 +
     mu2/mst22 + mst22/(-mst22 + mu2))*log(mu2/Q2)*power2(snt))/6. - (7*mst22*
     mu2*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)))/2. - (21*mst24*mu2*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)))/(2.*mst12) - (mt2*log(mt2/
     Q2)*power2(sa)*(2/mh2 - 2/mH2 - (29*mh2 + 100*mt2)*DeltaInv(mh2,mt2,mt2) +
     (29*mH2 + 100*mt2)*DeltaInv(mH2,mt2,mt2) - 324*mh2*power2(mt2)*power2(
     DeltaInv(mh2,mt2,mt2)) + 324*mH2*power2(mt2)*power2(DeltaInv(mH2,mt2,mt2))
     ))/18. + 28*msb14*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2)) - (7*mst22*
     mu2*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)))/2. - (21*mst24*mu2*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)))/(2.*mst12) - 7*mst22*mu2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - (21*mst24*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)))/msb12 - 7*mh2*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) + 28*mst24*power2(mu2)*power2(DeltaInv(mst22,
     mt2,mu2)) + 28*mst14*power2(mu2)*power2(DeltaInv(mt2,mu2,mst12)) - 7*
     mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - (21*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/msb12 - (2*ca*cb*mst14*mt2*
     mu2*sa*(mh2*power2(DeltaInv(mh2,mst12,mst12)) - mH2*power2(DeltaInv(mH2,
     mst12,mst12)))*power2(log(mst12/Q2)))/((mst12 - mst22)*sb) + (2*mst14*mt2*
     mu2*power2(cb)*power2(sa)*(mh2*power2(DeltaInv(mh2,mst12,mst12)) - mH2*
     power2(DeltaInv(mH2,mst12,mst12)))*power2(log(mst12/Q2)))/((mst12 - mst22)
     *power2(sb)) - (2*ca*cb*mst24*mt2*mu2*sa*(mH2*power2(DeltaInv(mH2,mst22,
     mst22)) - mh2*power2(DeltaInv(mst22,mst22,mh2)))*power2(log(mst22/Q2)))/((
     mst12 - mst22)*sb) + (2*mst24*mt2*mu2*power2(cb)*power2(sa)*(mH2*power2(
     DeltaInv(mH2,mst22,mst22)) - mh2*power2(DeltaInv(mst22,mst22,mh2)))*
     power2(log(mst22/Q2)))/((mst12 - mst22)*power2(sb)) + (mt2*((mh2 - mt2)*
     DeltaInv(mh2,mt2,mt2) + (-mH2 + mt2)*DeltaInv(mH2,mt2,mt2))*power2(sa)*
     power2(log(mt2/Q2)))/2. + (mt2*power2(cb)*((mA2 + mt2)*DeltaInv(mt2,mt2,
     mA2) - DeltaInv(mt2,mt2,mz2)*(mt2 + mz2 + 4*mz2*DeltaInv(mt2,mt2,mz2)*
     power2(mt2)) + 4*mA2*power2(mt2)*power2(DeltaInv(mt2,mt2,mA2)))*power2(
     log(mt2/Q2)))/2. - (21*mst22*mu2*power2(DeltaInv(mst22,mA2,mst12))*power3(
     mA2))/(2.*mst12) - (21*mst22*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(
     mC2))/msb12 - (21*mst22*mu2*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2))
     /(2.*mst12) + (7*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*power3(msb12)
     )/mt2 + (7*power2(mu2)*power2(DeltaInv(mt2,mu2,mst12))*power3(mst12))/mt2
     - 7*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) -
     (mt2*(mst12 - mst22 - 2*mu2)*log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(
     mC2)*power2(DeltaInv(msb12,mst12,mC2))*(-(mC2*mst12) - msb12*mst12 - mst14
     + DeltaInv(msb12,mst12,mC2)*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*
     power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 +
     power3(mst12)))))/(msb12*(mst12 - mst22)) + ((mt2 + mu2)*log(mC2/Q2)*log(
     mst12/Q2)*power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(msb12,mst12,
     mC2))*(-(mC2*mst12) - msb12*mst12 - mst14 + DeltaInv(msb12,mst12,mC2)*(-((
     msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*
     power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)))))/msb12 + (2*mt2*mu2*
     log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))*(mC2*mst12 + msb12*mst12 + mst14 + DeltaInv(msb12,mst12,mC2)*(
     (msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*
     power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))))/(msb12*(mst12 -
     mst22)) + (mu2*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 + mst14 + DeltaInv(
     msb12,mst12,mC2)*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) +
     (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))))/
     msb12 + (2*mu2*log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(
     snt)*power2(DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 + mst14 +
     DeltaInv(msb12,mst12,mC2)*((msb12*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(
     mst12)))))/msb12 - 42*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22)
     + (7*power2(mu2)*power2(DeltaInv(mst22,mt2,mu2))*power3(mst22))/mt2 - (
     mu2*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)
     )*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(
     msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))))/msb12 + (mu2*log(
     mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,
     mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(
     msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))))/msb12 - (mu2*log(
     msb12/Q2)*log(mst22/Q2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*(
     mC2*mst22 + msb12*mst22 + 3*mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*
     mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*
     power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))))/msb12 + (mu2*log(
     msb12/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,
     msb12,mC2))*(mC2*mst22 + msb12*mst22 + 3*mst24 + DeltaInv(mst22,msb12,mC2)
     *((msb12*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*
     mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))))/msb12 + ((
     mst12 - mst22 - 2*mt2)*mu2*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2))*(-(mC2*mst22) - msb12*mst22 - mst24
     + DeltaInv(mst22,msb12,mC2)*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*
     power3(mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 +
     power3(mst22)))))/(msb12*(mst12 - mst22)) + (2*(mst12 - mst22 - mt2)*mu2*
     log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2))*(-(mC2*mst22) - msb12*mst22 - mst24 + DeltaInv(mst22,msb12,
     mC2)*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12 +
     mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)))))/(msb12*(-
     mst12 + mst22)) - ((mt2 + mu2)*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*
     power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-(mC2*mst22) -
     msb12*mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-((msb12*mst22 + mst24)*
     power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(
     6*msb12*mst24 + power3(mst22)))))/msb12 + (mu2*log(mC2/Q2)*log(mst22/Q2)*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*mst22 + msb12*mst22 +
     mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 + mst24)*power2(mC2) -
     mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 +
     power3(mst22)))))/msb12 - (mu2*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*
     power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*mst22 + msb12*mst22 +
     mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 + mst24)*power2(mC2) -
     mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 +
     power3(mst22)))))/msb12 - (2*mu2*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*
     power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*mst22 +
     msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 + mst24)*
     power2(mC2) - mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*
     msb12*mst24 + power3(mst22)))))/msb12 - (mu2*log(mC2/Q2)*log(msb22/Q2)*
     power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 +
     msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*
     power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-
     10*msb22*mst24 + power3(mst22)))))/msb22 - (mu2*log(msb22/Q2)*log(mst22/
     Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 +
     msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 - mst24)*
     power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*
     mC2*(2*msb22*mst24 + power3(mst22)))))/msb22 - (mu2*log(mA2/Q2)*log(mst12/
     Q2)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*(3*mA2*mst22 + mst12*
     mst22 + mst24 + DeltaInv(mst22,mA2,mst12)*(5*(mst12*mst22 + mst24)*power2(
     mA2) - 3*mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) - mA2*(-10*
     mst12*mst24 + power3(mst22)))))/(2.*mst12) - (mu2*log(mA2/Q2)*log(mst12/
     Q2)*power2(cb)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*(3*mA2*mst22
     + mst12*mst22 + mst24 + DeltaInv(mst22,mA2,mst12)*(5*(mst12*mst22 + mst24)
     *power2(mA2) - 3*mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) - mA2*(
     -10*mst12*mst24 + power3(mst22)))))/(2.*mst12) - (mu2*log(mH2/Q2)*log(
     mst12/Q2)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(3*mH2*mst22 +
     mst12*mst22 + mst24 + DeltaInv(mH2,mst22,mst12)*(5*(mst12*mst22 + mst24)*
     power2(mH2) - 3*mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) - mH2*(-
     10*mst12*mst24 + power3(mst22)))))/(2.*mst12) + (2*mu2*log(mst12/Q2)*log(
     mst22/Q2)*power2(cb)*power2(mh2)*power2(snt)*power2(DeltaInv(mst22,mh2,
     mst12))*(mh2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)*((
     mst12*mst22 - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)
     *power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22)))))/(mst12*power2(
     sb)) + (2*mu2*log(mst12/Q2)*log(mst22/Q2)*power2(mH2)*power2(snt)*power2(
     DeltaInv(mH2,mst22,mst12))*(mH2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(
     mH2,mst22,mst12)*((mst12*mst22 - mst24)*power2(mH2) - mst22*power3(mH2) +
     (5*mst12 - 3*mst22)*power3(mst22) + 5*mH2*(2*mst12*mst24 + power3(mst22)))
     ))/mst12 + (mu2*log(mA2/Q2)*log(mst22/Q2)*power2(mA2)*power2(DeltaInv(
     mst22,mA2,mst12))*(mA2*mst22 + mst12*mst22 + mst24 + DeltaInv(mst22,mA2,
     mst12)*((mst12*mst22 + mst24)*power2(mA2) - mst22*power3(mA2) + (mst12 -
     mst22)*power3(mst22) + mA2*(6*mst12*mst24 + power3(mst22)))))/(2.*mst12) +
     (mu2*log(mA2/Q2)*log(mst22/Q2)*power2(cb)*power2(mA2)*power2(DeltaInv(
     mst22,mA2,mst12))*(mA2*mst22 + mst12*mst22 + mst24 + DeltaInv(mst22,mA2,
     mst12)*((mst12*mst22 + mst24)*power2(mA2) - mst22*power3(mA2) + (mst12 -
     mst22)*power3(mst22) + mA2*(6*mst12*mst24 + power3(mst22)))))/(2.*mst12) +
     (mu2*log(mH2/Q2)*log(mst22/Q2)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12))*(mH2*mst22 + mst12*mst22 + mst24 + DeltaInv(mH2,mst22,mst12)*((
     mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (mst12 - mst22)*
     power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))))/(2.*mst12) + 63*
     mh2*power2(DeltaInv(mh2,mt2,mt2))*power3(mt2) - 119*mz2*power2(DeltaInv(
     mt2,mt2,mz2))*power3(mt2) - mt2*DeltaInv(mt2,mt2,mA2)*Fin3(mt2,mt2,mA2,Q2)
     *power2(cb)*(1 + mt2*(5*mA2 + 4*mt2)*DeltaInv(mt2,mt2,mA2) + 16*mA2*
     power2(DeltaInv(mt2,mt2,mA2))*power3(mt2)) + Fin3(mt2,mt2,mz2,Q2)*(-(mt2*
     DeltaInv(mt2,mt2,mz2)*(1 + mt2*(4*mt2 + 5*mz2)*DeltaInv(mt2,mt2,mz2) + 16*
     mz2*power2(DeltaInv(mt2,mt2,mz2))*power3(mt2))) + mt2*DeltaInv(mt2,mt2,
     mz2)*power2(cb)*(1 + mt2*(4*mt2 + 5*mz2)*DeltaInv(mt2,mt2,mz2) + 16*mz2*
     power2(DeltaInv(mt2,mt2,mz2))*power3(mt2))) + (DeltaInv(msb12,mt2,mu2)*
     power3(mu2))/(6.*mt2) + (DeltaInv(mst22,mt2,mu2)*power3(mu2))/(6.*mt2) + (
     DeltaInv(mt2,mu2,mst12)*power3(mu2))/(6.*mt2) + 42*msb12*power2(DeltaInv(
     msb12,mt2,mu2))*power3(mu2) + (112*msb14*power2(DeltaInv(msb12,mt2,mu2))*
     power3(mu2))/mt2 + 42*mst22*power2(DeltaInv(mst22,mt2,mu2))*power3(mu2) +
     (112*mst24*power2(DeltaInv(mst22,mt2,mu2))*power3(mu2))/mt2 + 42*mst12*
     power2(DeltaInv(mt2,mu2,mst12))*power3(mu2) + (112*mst14*power2(DeltaInv(
     mt2,mu2,mst12))*power3(mu2))/mt2 + (5*power3(mw2))/(24.*mt2*power2(mt2 -
     mw2)) - (21*mst12*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2))/msb12
     + (5*power3(mw2))/(108.*power3(-mt2 + mw2)) - 49*mst24*mu2*power3(mH2)*
     power3(DeltaInv(mH2,mst22,mst12)) - (35*mu2*power2(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)))/2. - (21*mu2*power3(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)))/(2.*mst12) + (mu2*log(mst12/Q2)*log(
     mst22/Q2)*power2(cb)*power2(sa)*(-((mH2*mst22 + mst12*mst22 + 3*mst24)*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))) + power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*(mh2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(
     mst22,mh2,mst12)*((mst12*mst22 - mst24)*power2(mh2) - mst22*power3(mh2) +
     (5*mst12 - 3*mst22)*power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22)))
     ) + power2(mH2)*((-(mst12*mst22) + mst24)*power2(mH2) + mst22*power3(mH2)
     + (-5*mst12 + 3*mst22)*power3(mst22) - 5*mH2*(2*mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mH2,mst22,mst12))))/(2.*mst12*power2(sb)) + (4*
     ca*cb*mu2*sa*log(mst12/Q2)*log(mst22/Q2)*power2(snt)*(-((mH2*mst22 +
     mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))) +
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(mh2*mst22 + mst12*mst22 +
     3*mst24 + DeltaInv(mst22,mh2,mst12)*((mst12*mst22 - mst24)*power2(mh2) -
     mst22*power3(mh2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*mh2*(2*mst12*
     mst24 + power3(mst22)))) + power2(mH2)*((-(mst12*mst22) + mst24)*power2(
     mH2) + mst22*power3(mH2) + (-5*mst12 + 3*mst22)*power3(mst22) - 5*mH2*(2*
     mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12))))/(mst12*
     sb) + (2*mu2*log(mst12/Q2)*log(mst22/Q2)*power2(sa)*power2(snt)*(-((mH2*
     mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12))) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(mh2*mst22 +
     mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)*((mst12*mst22 - mst24)*
     power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*
     mh2*(2*mst12*mst24 + power3(mst22)))) + power2(mH2)*((-(mst12*mst22) +
     mst24)*power2(mH2) + mst22*power3(mH2) + (-5*mst12 + 3*mst22)*power3(
     mst22) - 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,
     mst12))))/mst12 - (mu2*log(mst12/Q2)*log(mst22/Q2)*((mH2*mst22 + mst12*
     mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(
     mA2)*power2(DeltaInv(mst22,mA2,mst12))*(mA2*mst22 + mst12*mst22 + 3*mst24
     + DeltaInv(mst22,mA2,mst12)*((mst12*mst22 - mst24)*power2(mA2) - mst22*
     power3(mA2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*mA2*(2*mst12*mst24 +
     power3(mst22)))) + power2(mH2)*((mst12*mst22 - mst24)*power2(mH2) - mst22*
     power3(mH2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*mH2*(2*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12))))/(2.*mst12) + (ca*cb*
     mu2*sa*log(mst12/Q2)*log(mst22/Q2)*((mH2*mst22 + mst12*mst22 + 3*mst24)*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*(-(mh2*mst22) - mst12*mst22 - 3*mst24 +
     DeltaInv(mst22,mh2,mst12)*((-(mst12*mst22) + mst24)*power2(mh2) + mst22*
     power3(mh2) + (-5*mst12 + 3*mst22)*power3(mst22) - 5*mh2*(2*mst12*mst24 +
     power3(mst22)))) + power2(mH2)*((mst12*mst22 - mst24)*power2(mH2) - mst22*
     power3(mH2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*mH2*(2*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12))))/(mst12*sb) + (mu2*log(
     mst12/Q2)*log(mst22/Q2)*power2(sa)*((mH2*mst22 + mst12*mst22 + 3*mst24)*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*(-(mh2*mst22) - mst12*mst22 - 3*mst24 +
     DeltaInv(mst22,mh2,mst12)*((-(mst12*mst22) + mst24)*power2(mh2) + mst22*
     power3(mh2) + (-5*mst12 + 3*mst22)*power3(mst22) - 5*mh2*(2*mst12*mst24 +
     power3(mst22)))) + power2(mH2)*((mst12*mst22 - mst24)*power2(mH2) - mst22*
     power3(mH2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*mH2*(2*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12))))/(2.*mst12) + (2*mu2*
     log(mst12/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)*power2(snt)*((mH2*mst22
     + mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-(mh2*mst22) - mst12*mst22
     - 3*mst24 + DeltaInv(mst22,mh2,mst12)*((-(mst12*mst22) + mst24)*power2(
     mh2) + mst22*power3(mh2) + (-5*mst12 + 3*mst22)*power3(mst22) - 5*mh2*(2*
     mst12*mst24 + power3(mst22)))) + power2(mH2)*((mst12*mst22 - mst24)*
     power2(mH2) - mst22*power3(mH2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*
     mH2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12))))/(
     mst12*power2(sb)) - (mu2*power2(cb)*power2(sa)*power2(log(mst12/Q2))*(4*
     mh2*mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)) - 4*mH2*mst12*mst14*
     mt2*power2(DeltaInv(mH2,mst12,mst12)) + (mst12 - mst22)*((2*mH2*mst22 +
     mst12*mst22 + 2*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-2*mh2*mst22 - mst12*mst22
     - 2*mst24 + DeltaInv(mst22,mh2,mst12)*(-((3*mst12*mst22 + 2*mst24)*power2(
     mh2)) + 2*mst22*power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(
     5*mst12*mst24 + power3(mst22)))) + power2(mH2)*((3*mst12*mst22 + 2*mst24)*
     power2(mH2) - 2*mst22*power3(mH2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*
     mH2*(5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)))))/
     (2.*mst12*(mst12 - mst22)*power2(sb)) + (ca*cb*mu2*sa*power2(log(mst12/Q2)
     )*(2*mh2*mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)) - 2*mH2*mst12*
     mst14*mt2*power2(DeltaInv(mH2,mst12,mst12)) + (mst12 - mst22)*((2*mH2*
     mst22 + mst12*mst22 + 2*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-2*mh2*mst22 -
     mst12*mst22 - 2*mst24 + DeltaInv(mst22,mh2,mst12)*(-((3*mst12*mst22 + 2*
     mst24)*power2(mh2)) + 2*mst22*power3(mh2) + (-3*mst12 + 2*mst22)*power3(
     mst22) - 2*mh2*(5*mst12*mst24 + power3(mst22)))) + power2(mH2)*((3*mst12*
     mst22 + 2*mst24)*power2(mH2) - 2*mst22*power3(mH2) + (3*mst12 - 2*mst22)*
     power3(mst22) + 2*mH2*(5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mH2,mst22,mst12)))))/(mst12*(mst12 - mst22)*sb) + (power2(sa)*power2(log(
     mst12/Q2))*(-2*mh2*mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)) + 2*
     mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,mst12,mst12)) + mu2*((2*mH2*mst22
     + mst12*mst22 + 2*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-2*mh2*mst22 - mst12*mst22
     - 2*mst24 + DeltaInv(mst22,mh2,mst12)*(-((3*mst12*mst22 + 2*mst24)*power2(
     mh2)) + 2*mst22*power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(
     5*mst12*mst24 + power3(mst22)))) + power2(mH2)*((3*mst12*mst22 + 2*mst24)*
     power2(mH2) - 2*mst22*power3(mH2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*
     mH2*(5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)))))/
     (2.*mst12) + (2*mu2*log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*((3*mC2*mst12 + msb12*mst12 + mst14)*mt2*power2(DeltaInv(
     msb12,mst12,mC2)) - (mst12 - mst22 - mt2)*power2(DeltaInv(mst22,msb12,mC2)
     )*(-3*mC2*mst22 - msb12*mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-5*(
     msb12*mst22 + mst24)*power2(mC2) + 3*mst22*power3(mC2) + (-msb12 + mst22)*
     power3(mst22) + mC2*(-10*msb12*mst24 + power3(mst22)))) + mt2*(5*(msb12*
     mst12 + mst14)*power2(mC2) - 3*mst12*power3(mC2) + (msb12 - mst12)*power3(
     mst12) - mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2))))/(msb12*(mst12 - mst22)) + (mu2*log(mC2/Q2)*log(msb12/Q2)*
     power2(mC2)*power2(snt)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*
     mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) -
     mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/
     msb12 + (2*mu2*log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*power2(
     snt)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,
     mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 +
     mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) -
     3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24
     + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*
     power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/msb12 - ((mt2 + mu2)*
     log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*power2(snt)*(-((3*mC2*
     mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(
     DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(
     mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(
     mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24 + power3(
     mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) +
     (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2))))/msb12 - (mu2*log(mC2/Q2)*log(msb12/Q2)
     *power2(mC2)*power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,
     mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(
     msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12
     + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2))))/msb12 - (2*mu2*log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(
     mC2)*power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*
     power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*
     mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(
     mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12)
     + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     ))/msb12 + ((mt2 + mu2)*log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*
     mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) -
     mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/
     msb12 + (2*mu2*log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*(-((3*mC2*
     mst12 + msb12*mst12 + mst14)*mt2*power2(DeltaInv(msb12,mst12,mC2))) + (
     mst12 - mst22 - mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 -
     msb12*mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*
     power2(mC2) + 3*mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(
     -10*msb12*mst24 + power3(mst22)))) + mt2*(-5*(msb12*mst12 + mst14)*power2(
     mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(
     mst12 - mst22)) + (log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*(-((3*
     mC2*mst12 + msb12*mst12 + mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(
     DeltaInv(msb12,mst12,mC2))) - (mst12 - mst22 - 2*mt2)*mu2*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*mst22 - msb12*mst22 - mst24 + DeltaInv(mst22,
     msb12,mC2)*(-5*(msb12*mst22 + mst24)*power2(mC2) + 3*mst22*power3(mC2) + (
     -msb12 + mst22)*power3(mst22) + mC2*(-10*msb12*mst24 + power3(mst22)))) +
     mt2*(mst12 - mst22 - 2*mu2)*(-5*(msb12*mst12 + mst14)*power2(mC2) + 3*
     mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12 -
     mst22)) - (log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-
     ((3*mC2*mst12 + msb12*mst12 + mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(
     DeltaInv(msb12,mst12,mC2))) - (mst12 - mst22 - 2*mt2)*mu2*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*mst22 - msb12*mst22 - mst24 + DeltaInv(mst22,
     msb12,mC2)*(-5*(msb12*mst22 + mst24)*power2(mC2) + 3*mst22*power3(mC2) + (
     -msb12 + mst22)*power3(mst22) + mC2*(-10*msb12*mst24 + power3(mst22)))) +
     mt2*(mst12 - mst22 - 2*mu2)*(-5*(msb12*mst12 + mst14)*power2(mC2) + 3*
     mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12 -
     mst22)) + (mt2*(mst12 - mst22 - 2*mu2)*log(mC2/Q2)*log(mst12/Q2)*power2(
     cb)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*
     power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,
     mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((
     msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*mt2*mu2*log(mC2/Q2)*log(mst12/
     Q2)*power2(cb)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22*(mst12 - mst22)) + (mu2*log(mC2/Q2)*log(mst12/Q2)*
     power2(mC2)*power2(snb)*power2(snt)*(-(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22) + (2*mu2*log(mC2/Q2)*log(mst12/Q2)*power2(cb)*
     power2(mC2)*power2(snb)*power2(snt)*(-(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22) - ((mt2 + mu2)*log(mC2/Q2)*log(mst12/Q2)*power2(cb)*
     power2(mC2)*power2(snb)*power2(snt)*(-(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22) + 224*power3(msb12)*power3(mu2)*power3(DeltaInv(
     msb12,mt2,mu2)) + (mu2*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*
     mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(
     mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*
     msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) +
     3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 + (2*
     mu2*log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(
     snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,mst12,
     mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 +
     mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(mC2) -
     3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*msb22*mst24
     + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*
     power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 - ((mt2 + mu2)*
     log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(
     -((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*msb22*mst24 +
     power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 + (2*mu2*log(mC2/Q2)*
     log(msb22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-((3*mC2*mst12 + msb22*
     mst12 + mst14)*mt2*power2(DeltaInv(msb22,mst12,mC2))) + (mst12 - mst22 -
     mt2)*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*mst22 - msb22*mst22 - mst24
     + DeltaInv(mst22,msb22,mC2)*(-5*(msb22*mst22 + mst24)*power2(mC2) + 3*
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) + mC2*(-10*msb22*mst24
     + power3(mst22)))) + mt2*(-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*
     power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12 - mst22)
     ) + (log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-((3*
     mC2*mst12 + msb22*mst12 + mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(
     DeltaInv(msb22,mst12,mC2))) - (mst12 - mst22 - 2*mt2)*mu2*power2(DeltaInv(
     mst22,msb22,mC2))*(-3*mC2*mst22 - msb22*mst22 - mst24 + DeltaInv(mst22,
     msb22,mC2)*(-5*(msb22*mst22 + mst24)*power2(mC2) + 3*mst22*power3(mC2) + (
     -msb22 + mst22)*power3(mst22) + mC2*(-10*msb22*mst24 + power3(mst22)))) +
     mt2*(mst12 - mst22 - 2*mu2)*(-5*(msb22*mst12 + mst14)*power2(mC2) + 3*
     mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12 -
     mst22)) - 49*mst24*mu2*power3(mA2)*power3(DeltaInv(mst22,mA2,mst12)) - (
     35*mu2*power2(mA2)*power3(mst22)*power3(DeltaInv(mst22,mA2,mst12)))/2. - (
     21*mu2*power3(mA2)*power3(mst22)*power3(DeltaInv(mst22,mA2,mst12)))/(2.*
     mst12) + (2*ca*cb*mu2*sa*power2(snt)*power2(log(mst22/Q2))*(-2*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) + 2*power2(mH2)*(mst24*power2(mH2) + (-2*mst12
     + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)) + mh2*(2*mh2*mst24*power2(DeltaInv(mst22,mh2,
     mst12)) + mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 2*mh2*(mst24*
     power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))))/(mst12*sb) + (mu2*
     power2(sa)*power2(snt)*power2(log(mst22/Q2))*(-2*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,
     mst22)) + 2*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,
     mst12)) + mh2*(2*mh2*mst24*power2(DeltaInv(mst22,mh2,mst12)) + mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 2*mh2*(mst24*power2(mh2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)))))/mst12 + (mu2*power2(cb)*power2(sa)*
     power2(snt)*power2(log(mst22/Q2))*(2*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) + mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*
     power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mh2*(-
     2*mh2*mst24*power2(DeltaInv(mst22,mh2,mst12)) - mst12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) + 2*mh2*(mst24*power2(mh2) + (-2*mst12 + mst22)
     *power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)))))/(mst12*power2(sb)) - (ca*cb*mu2*sa*power2(log(mst22/
     Q2))*((-mst12 + mst22)*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))
     - 2*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) + (mst12 -
     mst22)*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) -
     2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) +
     mh2*(mh2*(mst12 - mst22)*mst24*power2(DeltaInv(mst22,mh2,mst12)) + 2*
     mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) - mh2*(mst12 - mst22)*(
     mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))))/(mst12*(mst12 -
     mst22)*sb) + (mu2*power2(cb)*power2(sa)*power2(log(mst22/Q2))*((-mst12 +
     mst22)*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 4*mH2*mst12*
     mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) + (mst12 - mst22)*power2(mH2)*
     (mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mh2*(mh2*(mst12 -
     mst22)*mst24*power2(DeltaInv(mst22,mh2,mst12)) + 4*mst12*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) - mh2*(mst12 - mst22)*(mst24*power2(mh2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)))))/(2.*mst12*(mst12 - mst22)*power2(sb))
     + (power2(sa)*power2(log(mst22/Q2))*(mst24*mu2*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 2*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,
     mst22,mst22)) - mu2*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,
     mst22,mst12)) + mh2*(-(mh2*mst24*mu2*power2(DeltaInv(mst22,mh2,mst12))) -
     2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) + mh2*mu2*(mst24*
     power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))))/(2.*mst12) - 98*
     mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 35*mu2*power2(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - (21*mu2*power3(mC2)
     *power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)))/msb12 + (mu2*power2(mC2)
     *power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(-mst24 + DeltaInv(
     mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))) - msb22*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (mu2*log(mC2/Q2)*log(
     mst22/Q2)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst22 + msb12*mst22 +
     mst24)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(
     (msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*
     power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*(-((msb12*
     mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(
     mst22) - mC2*(6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(msb12*msb22) + ((mst12 - mst22 - 2*mt2)*mu2*log(mC2/Q2)*log(
     mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst22 + msb12*
     mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(
     mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,
     mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 -
     mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*(-((
     msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*
     power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(msb12*msb22*(-mst12 + mst22)) + (2*(mst12 - mst22 -
     mt2)*mu2*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24
     + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + ((
     mt2 + mu2)*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*
     power2(snt)*(-(msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(
     mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 +
     msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*
     power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*
     msb22*mst24 + power3(mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(
     mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22)
     + (mu2*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(snt)*(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) +
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (2*mu2*log(
     mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) +
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) - (log(mst22/
     Q2)*power2(snb)*((-msb12 + msb22)*mst22 + msb22*(5*mC2*mst22 - msb12*mst22
     + mst24)*mu2*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*mst22 - msb22*mst22
     + mst24)*mu2*DeltaInv(mst22,msb22,mC2) - 6*mC2*msb22*mu2*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) + 6*mC2*msb12*mu2*power2(DeltaInv(
     mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 +
     mst22*power2(mC2) + power3(mst22)) - 72*msb22*mu2*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 72*msb12*mu2*power2(
     mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(12.*msb12*
     msb22) + 224*power3(mst22)*power3(mu2)*power3(DeltaInv(mst22,mt2,mu2)) +
     224*power3(mst12)*power3(mu2)*power3(DeltaInv(mt2,mu2,mst12)) - 35*mt2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 98*mst14*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - (21*mt2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/msb12 + (mu2*log(mC2/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(
     mC2*mst12 + msb12*mst12 + mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) +
     mC2*(6*msb12*mst14 + power3(mst12))))*power4(cb))/(msb12*power2(sb)) + ((
     mst12 - mst22 - 2*mt2)*mu2*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))*(-(mC2*mst22) - msb12*mst22 - mst24 + DeltaInv(
     mst22,msb12,mC2)*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2)
     + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22))))*
     power4(cb))/(msb12*(-mst12 + mst22)*power2(sb)) + (2*mt2*mu2*log(mC2/Q2)*
     log(mst22/Q2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*mst22 +
     msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 + mst24)*
     power2(mC2) - mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*
     msb12*mst24 + power3(mst22))))*power4(cb))/(msb12*(mst12 - mst22)*power2(
     sb)) - (mu2*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snt)*power2(
     DeltaInv(mst22,msb12,mC2))*(mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(
     mst22,msb12,mC2)*((msb12*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) +
     (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(mst22))))*
     power4(cb))/(msb12*power2(sb)) - (mu2*log(mA2/Q2)*log(mst12/Q2)*power2(
     mA2)*power2(DeltaInv(mst22,mA2,mst12))*(3*mA2*mst22 + mst12*mst22 + mst24
     + DeltaInv(mst22,mA2,mst12)*(5*(mst12*mst22 + mst24)*power2(mA2) - 3*
     mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) - mA2*(-10*mst12*mst24 +
     power3(mst22))))*power4(cb))/(2.*mst12*power2(sb)) + (mu2*log(mA2/Q2)*log(
     mst22/Q2)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*(mA2*mst22 +
     mst12*mst22 + mst24 + DeltaInv(mst22,mA2,mst12)*((mst12*mst22 + mst24)*
     power2(mA2) - mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) + mA2*(6*
     mst12*mst24 + power3(mst22))))*power4(cb))/(2.*mst12*power2(sb)) - (2*mt2*
     mu2*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*(-((3*mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,
     mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(
     msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12
     + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)))*power4(cb))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mt2*mu2*
     log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*(-((3*mC2*mst12 + msb12*
     mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,
     msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)
     *(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 -
     mst22)*power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(
     msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*
     power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)))*power4(cb))/(msb12*(mst12 - mst22)*power2(sb)) + (mu2*
     log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snt)*(-((3*mC2*mst12 + msb12*
     mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,
     msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)
     *(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 -
     mst22)*power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(
     msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*
     power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)))*power4(cb))/(msb12*power2(sb)) - (mu2*log(mC2/Q2)*log(
     msb12/Q2)*power2(mC2)*power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb12*
     mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,
     msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)
     *(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 -
     mst22)*power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(
     msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*
     power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)))*power4(cb))/(msb12*power2(sb)) + (mu2*log(mC2/Q2)*log(
     msb12/Q2)*power2(mC2)*power2(snb)*(2*(3*mC2*mst12 + msb12*mst12 + mst14)*
     mt2*power2(DeltaInv(msb12,mst12,mC2)) - (mst12 - mst22 - 2*mt2)*power2(
     DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 - msb12*mst22 - mst24 + DeltaInv(
     mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*power2(mC2) + 3*mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(-10*msb12*mst24 + power3(
     mst22)))) - 2*mt2*(-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)))*power4(cb))/(msb12*(mst12 -
     mst22)*power2(sb)) + (mu2*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*(-2*(3*
     mC2*mst12 + msb12*mst12 + mst14)*mt2*power2(DeltaInv(msb12,mst12,mC2)) + (
     mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 -
     msb12*mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*
     power2(mC2) + 3*mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(
     -10*msb12*mst24 + power3(mst22)))) + 2*mt2*(-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)))*
     power4(cb))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mt2*mu2*log(mC2/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2)))*power4(cb))/(msb12*msb22*(mst12 - mst22)*power2(sb)) + (mu2*log(
     mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*power2(snt)*(-(msb22*(mC2*
     mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*
     power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb12,mst12,mC2)))*power4(cb))/(msb12*msb22*power2(sb))
     + (2*mt2*mu2*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*(msb22*(
     mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 -
     mst14 + DeltaInv(msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) +
     mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 +
     power3(mst12)))) + msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)))*power4(cb))/(msb12*msb22*(
     mst12 - mst22)*power2(sb)) - (2*mt2*mu2*log(mC2/Q2)*log(msb22/Q2)*power2(
     mC2)*power2(snb)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(
     msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 +
     msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*
     power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-
     10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2)
     + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)))*power4(cb))/(
     msb22*(mst12 - mst22)*power2(sb)) + (mu2*log(mC2/Q2)*log(msb22/Q2)*power2(
     mC2)*power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*
     power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*
     mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12)
     + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     )*power4(cb))/(msb22*power2(sb)) + (mu2*log(mC2/Q2)*log(msb22/Q2)*power2(
     mC2)*power2(snb)*(-2*(3*mC2*mst12 + msb22*mst12 + mst14)*mt2*power2(
     DeltaInv(msb22,mst12,mC2)) + (mst12 - mst22 - 2*mt2)*power2(DeltaInv(
     mst22,msb22,mC2))*(-3*mC2*mst22 - msb22*mst22 - mst24 + DeltaInv(mst22,
     msb22,mC2)*(-5*(msb22*mst22 + mst24)*power2(mC2) + 3*mst22*power3(mC2) + (
     -msb22 + mst22)*power3(mst22) + mC2*(-10*msb22*mst24 + power3(mst22)))) +
     2*mt2*(-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-
     msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)))*power4(cb))/(msb22*(mst12 - mst22)*
     power2(sb)) + ((mst12 - mst22 - 2*mt2)*mu2*log(mC2/Q2)*log(mst22/Q2)*
     power2(mC2)*power2(snb)*(-(msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(
     DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(
     mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22
     + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) +
     mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*(-((msb12*mst22 + mst24)*
     power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(
     6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)))*power4(
     cb))/(msb12*msb22*(mst12 - mst22)*power2(sb)) + (2*mt2*mu2*log(mC2/Q2)*
     log(mst22/Q2)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst22 + msb12*mst22 +
     mst24)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(
     (msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*
     power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*(-((msb12*
     mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(
     mst22) - mC2*(6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2)))*power4(cb))/(msb12*msb22*(mst12 - mst22)*power2(sb)) + (mu2*log(
     mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(snt)*(msb22*(mC2*
     mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 +
     DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*
     power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)))*power4(cb))/(msb12*msb22*
     power2(sb)) - (35*mst22*mu2*power3(DeltaInv(mst22,mA2,mst12))*power4(mA2))
     /2. - (21*mst24*mu2*power3(DeltaInv(mst22,mA2,mst12))*power4(mA2))/(2.*
     mst12) - 35*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - (21*
     mst24*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2))/msb12 - (35*
     mst22*mu2*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2))/2. - (21*mst24*
     mu2*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2))/(2.*mst12) + Fin3(
     msb12,mt2,mu2,Q2)*((2*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*(msb14 +
     msb12*(mt2 + 3*mu2) + DeltaInv(msb12,mt2,mu2)*(5*msb14*mu2*(2*mt2 + mu2) +
     msb12*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*power3(msb12) - power4(
     msb12))))/mt2 - (2*power2(mu2)*power2(snb)*power2(DeltaInv(msb12,mt2,mu2))
     *(msb14 + msb12*(mt2 + 3*mu2) + DeltaInv(msb12,mt2,mu2)*(5*msb14*mu2*(2*
     mt2 + mu2) + msb12*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*power3(msb12)
     - power4(msb12))))/mt2) + 14*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*
     power4(msb12) - (70*power3(mu2)*power3(DeltaInv(msb12,mt2,mu2))*power4(
     msb12))/mt2 - (DeltaInv(msb12,mt2,mu2)*log(msb12/Q2)*log(mu2/Q2)*power2(
     mu2)*(msb12 + DeltaInv(msb12,mt2,mu2)*(msb12*mu2*(4*mt2 + mu2) + 2*msb14*(
     mt2 + 4*mu2) - power3(msb12)) - 2*mu2*power2(DeltaInv(msb12,mt2,mu2))*(
     msb14*mu2*(-10*mt2 + mu2) + msb12*(-mt2 + mu2)*power2(mu2) - 5*(mt2 + mu2)
     *power3(msb12) + 3*power4(msb12))))/mt2 + (DeltaInv(msb12,mt2,mu2)*log(
     msb12/Q2)*log(mu2/Q2)*power2(mu2)*power2(snb)*(msb12 + DeltaInv(msb12,mt2,
     mu2)*(msb12*mu2*(4*mt2 + mu2) + 2*msb14*(mt2 + 4*mu2) - power3(msb12)) -
     2*mu2*power2(DeltaInv(msb12,mt2,mu2))*(msb14*mu2*(-10*mt2 + mu2) + msb12*(
     -mt2 + mu2)*power2(mu2) - 5*(mt2 + mu2)*power3(msb12) + 3*power4(msb12))))
     /mt2 + (log(mt2/Q2)*log(mu2/Q2)*power2(mu2)*power2(snb)*(-(msb12*DeltaInv(
     msb12,mt2,mu2)) + power2(DeltaInv(msb12,mt2,mu2))*(-2*msb14*(mt2 + 5*mu2)
     - msb12*mu2*(6*mt2 + 7*mu2) + power3(msb12)) + 4*mu2*power3(DeltaInv(
     msb12,mt2,mu2))*(-2*msb14*mu2*(5*mt2 + mu2) + msb12*(-3*mt2 + 2*mu2)*
     power2(mu2) - (3*mt2 + 2*mu2)*power3(msb12) + 2*power4(msb12)) + DeltaInv(
     mt2,mu2,msb22)*(msb22 + DeltaInv(mt2,mu2,msb22)*(2*msb24*(mt2 + 5*mu2) +
     msb22*mu2*(6*mt2 + 7*mu2) - power3(msb22)) + 4*mu2*power2(DeltaInv(mt2,
     mu2,msb22))*(2*msb24*mu2*(5*mt2 + mu2) + msb22*(3*mt2 - 2*mu2)*power2(mu2)
     + (3*mt2 + 2*mu2)*power3(msb22) - 2*power4(msb22)))))/mt2 + (power2(snb)*
     power2(log(mu2/Q2))*power3(mu2)*(-((msb14 + msb12*(mt2 + 3*mu2))*power2(
     DeltaInv(msb12,mt2,mu2))) + power3(DeltaInv(msb12,mt2,mu2))*(-5*msb14*mu2*
     (2*mt2 + mu2) + msb12*(-5*mt2 + 3*mu2)*power2(mu2) + (-mt2 + mu2)*power3(
     msb12) + power4(msb12)) + power2(DeltaInv(mt2,mu2,msb22))*(msb24 + msb22*
     mt2 + 3*msb22*mu2 + DeltaInv(mt2,mu2,msb22)*(5*msb24*mu2*(2*mt2 + mu2) +
     msb22*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*power3(msb22) - power4(
     msb22)))))/mt2 + (2*Fin3(mt2,mu2,msb22,Q2)*power2(mu2)*power2(snb)*power2(
     DeltaInv(mt2,mu2,msb22))*(msb24 + msb22*(mt2 + 3*mu2) + DeltaInv(mt2,mu2,
     msb22)*(5*msb24*mu2*(2*mt2 + mu2) + msb22*(5*mt2 - 3*mu2)*power2(mu2) + (
     mt2 - mu2)*power3(msb22) - power4(msb22))))/mt2 - (log(mu2/Q2)*power2(mu2)
     *power2(snb)*(-msb12 + msb22 - msb22*DeltaInv(msb12,mt2,mu2)*(mu2*(-mt2 +
     mu2) + 2*msb12*(mt2 + 2*mu2) + 7*power2(msb12)) + msb12*DeltaInv(mt2,mu2,
     msb22)*(mu2*(-mt2 + mu2) + 2*msb22*(mt2 + 2*mu2) + 7*power2(msb22)) + 6*
     msb12*msb22*power2(DeltaInv(msb12,mt2,mu2))*(-(msb14*(mt2 + 7*mu2)) -
     msb12*mu2*(12*mt2 + 19*mu2) + (-mt2 + mu2)*power2(mu2) + power3(msb12)) +
     6*msb12*msb22*power2(DeltaInv(mt2,mu2,msb22))*(msb24*(mt2 + 7*mu2) +
     msb22*mu2*(12*mt2 + 19*mu2) + (mt2 - mu2)*power2(mu2) - power3(msb22)) +
     36*msb12*msb22*mu2*power3(DeltaInv(msb12,mt2,mu2))*(-5*msb14*mu2*(2*mt2 +
     mu2) + msb12*(-5*mt2 + 3*mu2)*power2(mu2) + (-mt2 + mu2)*power3(msb12) +
     power4(msb12)) - 36*msb12*msb22*mu2*power3(DeltaInv(mt2,mu2,msb22))*(-5*
     msb24*mu2*(2*mt2 + mu2) + msb22*(-5*mt2 + 3*mu2)*power2(mu2) + (-mt2 +
     mu2)*power3(msb22) + power4(msb22))))/(6.*msb12*msb22*mt2) - (DeltaInv(
     mt2,mu2,msb22)*log(msb22/Q2)*log(mu2/Q2)*power2(mu2)*power2(snb)*(msb22 +
     DeltaInv(mt2,mu2,msb22)*(msb22*mu2*(4*mt2 + mu2) + 2*msb24*(mt2 + 4*mu2) -
     power3(msb22)) - 2*mu2*power2(DeltaInv(mt2,mu2,msb22))*(msb24*mu2*(-10*mt2
     + mu2) + msb22*(-mt2 + mu2)*power2(mu2) - 5*(mt2 + mu2)*power3(msb22) + 3*
     power4(msb22))))/mt2 + (mt2*(mst12 - mst22 - 2*mu2)*log(msb12/Q2)*log(
     mst12/Q2)*power2(cb)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)
     *power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb12*(mst12 - mst22)) +
     (2*mt2*mu2*log(msb12/Q2)*log(mst12/Q2)*power2(cb)*(-((mC2*mst12 + msb12*
     mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(
     mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     *(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*
     mst14*mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(
     mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(
     msb12*(mst12 - mst22)) - (mt2*(mst12 - mst22 - 2*mu2)*log(msb12/Q2)*log(
     mst12/Q2)*power2(cb)*power2(snb)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*
     mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*
     power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12)
     + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb12*(
     mst12 - mst22)) + (2*mu2*log(msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(
     snt)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) +
     mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) -
     mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*
     power3(mw2) - 3*power4(mst12)))))/msb12 - ((mt2 + mu2)*log(msb12/Q2)*log(
     mst12/Q2)*power2(cb)*power2(snt)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*
     mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*
     power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12)
     + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/msb12 + ((
     mt2 + mu2)*log(msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*power2(snt)*
     (-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) - mst14*
     power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(
     mw2) - 3*power4(mst12)))))/msb12 + (2*mt2*mu2*log(msb12/Q2)*log(mst12/Q2)*
     power4(cb)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)*
     power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (2*mt2*mu2*log(msb12/Q2)*log(mst12/Q2)*power2(snb)*power4(
     cb)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) +
     mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) -
     mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*
     power3(mw2) - 3*power4(mst12)))))/(msb12*(mst12 - mst22)*power2(sb)) + (
     mu2*log(msb12/Q2)*log(mst12/Q2)*power2(snt)*power4(cb)*(-((mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-
     5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(
     10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*
     msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(
     mst12)))))/(msb12*power2(sb)) + (log(msb12/Q2)*log(mst12/Q2)*power2(snt)*(
     -((mC2*mst12 + msb12*mst12 + 3*mst14)*mu2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + mu2*power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2)
     + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2
     + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*power2(mw2)
     - mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*
     power3(mw2) - 3*power4(mst12)))))/msb12 + (mt2*(mst12 - mst22 - 2*mu2)*
     log(msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*(-((mC2*mst12 + msb22*
     mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(
     mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb22
     + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     *(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(
     mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(
     msb22*(mst12 - mst22)) + (2*mt2*mu2*log(msb22/Q2)*log(mst12/Q2)*power2(cb)
     *power2(snb)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*
     power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb22*(mst12 - mst22)) +
     (2*mu2*log(msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*power2(snt)*(-((
     mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*
     power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(
     mw2) - 3*power4(mst12)))))/msb22 - ((mt2 + mu2)*log(msb22/Q2)*log(mst12/
     Q2)*power2(cb)*power2(snb)*power2(snt)*(-((mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(
     msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     *(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(
     mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/
     msb22 + (2*mt2*mu2*log(msb22/Q2)*log(mst12/Q2)*power2(snb)*power4(cb)*(-((
     mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*
     power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(
     mw2) - 3*power4(mst12)))))/(msb22*(mst12 - mst22)*power2(sb)) + (mu2*log(
     msb22/Q2)*log(mst12/Q2)*power2(snb)*power2(snt)*power4(cb)*(-((mC2*mst12 +
     msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-
     5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*
     msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(
     mst12)))))/(msb22*power2(sb)) + (log(msb22/Q2)*log(mst12/Q2)*power2(snb)*
     power2(snt)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*mu2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))) + mu2*power2(mC2)*((-(msb22*mst12) + mst14)*
     power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/msb22 + (2*mu2*power2(cb)
     *power2(snt)*power2(log(mst12/Q2))*(-(mst14*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*
     power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14 +
     DeltaInv(mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*
     msb12*(mst14*mw2 + power3(mst12)) - power4(mst12)))))/msb12 + (mu2*power2(
     snt)*power2(log(mst12/Q2))*power4(cb)*(-(mst14*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     *(mst14 + DeltaInv(mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12)))))/(msb12*
     power2(sb)) - (mu2*power2(cb)*power2(snt)*power2(log(mst12/Q2))*(-(mh2*
     msb12*mst12*mst14*power2(DeltaInv(mh2,mst12,mst12))) - 2*msb12*(2*mh2*
     mst22 + mst12*mst22 + 2*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + 2*msb12*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) +
     2*mst22*power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14 + DeltaInv(mw2,msb12,mst12)*
     (-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(
     mst12)) - power4(mst12)))))/(msb12*mst12*power2(sb)) + ((mt2 + mu2)*
     power2(cb)*power2(snb)*power2(snt)*power2(log(mst12/Q2))*(-(msb22*mst14*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst14*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) + msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - msb12*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12)))))/(msb12*
     msb22) + (2*mt2*mu2*power2(snb)*power2(log(mst12/Q2))*power4(cb)*(-(msb22*
     mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst14*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb22*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - msb12*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(msb22*
     mst14*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*
     power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) +
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*
     power3(mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12)))))/(
     msb12*msb22*(mst12 - mst22)*power2(sb)) - (mt2*log(msb12/Q2)*log(mst12/Q2)
     *power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*mw2*
     power3(mst12) + msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12))
     - mst12*power3(mw2) - 3*power4(mst12))))/msb12 + (mt2*log(msb12/Q2)*log(
     mst12/Q2)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(-(mst14*
     power2(mw2)) + 5*mw2*power3(mst12) + msb12*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/msb12 - (
     mu2*log(msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(mw2)*power2(snt)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*
     power4(mst12))))/(msb12*power2(sb)) + (mu2*log(msb12/Q2)*log(mst12/Q2)*
     power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(-(
     mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb12*(10*mst14*mw2 + mst12*
     power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/(
     msb12*power2(sb)) - (mt2*log(msb22/Q2)*log(mst12/Q2)*power2(mw2)*power2(
     snb)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2
     + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(
     mw2) - 3*power4(mst12))))/msb22 - (mu2*log(msb22/Q2)*log(mst12/Q2)*power2(
     cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*
     (msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 5*mw2*power3(mst12) + msb22*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/(msb22*
     power2(sb)) + (2*Fin3(mt2,mu2,mst12,Q2)*power2(mu2)*power2(DeltaInv(mt2,
     mu2,mst12))*(mst14 + mst12*(mt2 + 3*mu2) + DeltaInv(mt2,mu2,mst12)*(5*
     mst14*mu2*(2*mt2 + mu2) + mst12*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*
     power3(mst12) - power4(mst12))))/mt2 + (mt2*power2(mw2)*power2(snb)*
     power2(log(mst12/Q2))*(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12)) -
     msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)
     *(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 +
     power3(mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))
     *(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 +
     power3(mst12)) - power4(mst12))))/(msb12*msb22) + (2*mt2*mu2*power2(cb)*
     power2(mw2)*power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12))))/(msb12*
     msb22*(mst12 - mst22)*power2(sb)) + (mu2*power2(cb)*power2(mw2)*power2(
     snb)*power2(snt)*power2(log(mst12/Q2))*(msb22*mst14*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(
     mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(
     mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,
     msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(
     mst14*mw2 + power3(mst12)) - power4(mst12))))/(msb12*msb22*power2(sb)) - (
     mt2*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     )*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))))/msb12 + (
     mt2*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     )/msb12 - (mt2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))))/msb22 + (mt2*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(
     6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))))/msb12 - (mt2*(mst12 - mst22 - 2*mu2)*log(mst12/Q2)*log(
     mw2/Q2)*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) +
     mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)
     ) - mst12*power3(mw2) - power4(mst12))))/(msb12*(mst12 - mst22)) - (2*mt2*
     mu2*log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)
     *(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*
     power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(
     msb12*(mst12 - mst22)) - (mt2*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 +
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*power3(
     mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*
     power3(mw2) - power4(mst12))))/msb12 - (2*mu2*log(mst12/Q2)*log(mw2/Q2)*
     power2(cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/msb12 + ((mt2 + mu2)
     *log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snt)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*
     mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(
     mst12))))/msb12 + (mu2*log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 +
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*power3(
     mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*
     power3(mw2) - power4(mst12))))/(msb12*power2(sb)) - (mu2*log(mst12/Q2)*
     log(mw2/Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*
     power4(cb)*(msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(
     mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(
     mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*
     power2(sb)) - (mt2*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*(
     msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12))
     - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2
     + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) +
     msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(
     mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*
     power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*msb22) + (
     mt2*(mst12 - mst22 - 2*mu2)*log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(
     mw2)*power2(snb)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/
     (msb12*msb22*(mst12 - mst22)) + (2*mt2*mu2*log(mst12/Q2)*log(mw2/Q2)*
     power2(cb)*power2(mw2)*power2(snb)*(msb22*(msb12*mst12 + mst14 + mst12*
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     mst14*power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(
     mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) +
     msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(
     mw2) - power4(mst12))))/(msb12*msb22*(mst12 - mst22)) + (mt2*log(mst12/Q2)
     *log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*(msb22*(msb12*mst12 +
     mst14 + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*
     mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) +
     mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)
     ) - mst12*power3(mw2) - power4(mst12))))/(msb12*msb22) + (2*mu2*log(mst12/
     Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*(msb22*(
     msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12)) -
     msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb22*(
     6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*msb22) - ((
     mt2 + mu2)*log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*
     power2(snt)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/
     (msb12*msb22) - (mu2*log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snb)*power2(snt)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*
     (6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))))/(msb12*msb22*power2(sb)) + (mu2*log(mst12/Q2)*log(mw2/Q2)
     *power2(mw2)*power2(snb)*power2(snt)*power4(cb)*(msb22*(msb12*mst12 +
     mst14 + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*
     mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) +
     mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)
     ) - mst12*power3(mw2) - power4(mst12))))/(msb12*msb22*power2(sb)) - 168*
     mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) + 14*power2(mu2)*
     power3(DeltaInv(mt2,mu2,mst12))*power4(mst12) - (70*power3(mu2)*power3(
     DeltaInv(mt2,mu2,mst12))*power4(mst12))/mt2 + (21*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12))/msb12 - 2*mt2*log(mh2/Q2)*log(
     mst12/Q2)*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12) +
     16*mh2*DeltaInv(mh2,mst12,mst12)*power4(mst12)) + (4*ca*cb*mt2*mu2*sa*log(
     mh2/Q2)*log(mst12/Q2)*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*
     power3(mst12) + 16*mh2*DeltaInv(mh2,mst12,mst12)*power4(mst12)))/((mst12 -
     mst22)*sb) + (4*mt2*mu2*log(mh2/Q2)*log(mst12/Q2)*power2(cb)*power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12) + 16*mh2*DeltaInv(
     mh2,mst12,mst12)*power4(mst12)))/((mst12 - mst22)*power2(sb)) - (4*mt2*
     mu2*log(mh2/Q2)*log(mst12/Q2)*power2(cb)*power2(sa)*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 4*power3(mst12) + 16*mh2*DeltaInv(mh2,mst12,
     mst12)*power4(mst12)))/((mst12 - mst22)*power2(sb)) - (4*ca*cb*mt2*mu2*sa*
     log(mH2/Q2)*log(mst12/Q2)*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 +
     4*power3(mst12) + 16*mH2*DeltaInv(mH2,mst12,mst12)*power4(mst12)))/((mst12
     - mst22)*sb) + (4*mt2*mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*power2(sa)*
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12) + 16*mH2*
     DeltaInv(mH2,mst12,mst12)*power4(mst12)))/((mst12 - mst22)*power2(sb)) + (
     2*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(log(mst12/Q2))*(-(
     msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2))
     + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*
     mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))
     /(msb12*msb22*(mst12 - mst22)*power2(sb)) - (DeltaInv(mt2,mu2,mst12)*log(
     mst12/Q2)*log(mu2/Q2)*power2(mu2)*(mst12 + DeltaInv(mt2,mu2,mst12)*(mst12*
     mu2*(4*mt2 + mu2) + 2*mst14*(mt2 + 4*mu2) - power3(mst12)) - 2*mu2*power2(
     DeltaInv(mt2,mu2,mst12))*(mst14*mu2*(-10*mt2 + mu2) + mst12*(-mt2 + mu2)*
     power2(mu2) - 5*(mt2 + mu2)*power3(mst12) + 3*power4(mst12))))/mt2 - (ca*
     cb*mt2*mu2*sa*log(mst12/Q2)*(-((mh2 - 6*mst12)*DeltaInv(mh2,mst12,mst12))
     + 24*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) +
     288*mh2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) + DeltaInv(mH2,
     mst12,mst12)*(mH2 - 6*mst12 - 24*DeltaInv(mH2,mst12,mst12)*(mH2*mst14 + 3*
     power3(mst12)) - 288*mH2*power2(DeltaInv(mH2,mst12,mst12))*power4(mst12)))
     )/(3.*(mst12 - mst22)*sb) + (mt2*mu2*log(mst12/Q2)*power2(cb)*power2(sa)*(
     -((mh2 - 6*mst12)*DeltaInv(mh2,mst12,mst12)) + 24*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) + 288*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power4(mst12) + DeltaInv(mH2,mst12,mst12)*(mH2 - 6*mst12 -
     24*DeltaInv(mH2,mst12,mst12)*(mH2*mst14 + 3*power3(mst12)) - 288*mH2*
     power2(DeltaInv(mH2,mst12,mst12))*power4(mst12))))/(3.*(mst12 - mst22)*
     power2(sb)) - (2*mt2*mu2*power2(cb)*power2(log(mst12/Q2))*(mh2*msb12*
     mst14*power2(DeltaInv(mh2,mst12,mst12)) + power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12))*(-mst14 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 2*
     mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))
     )/(msb12*(mst12 - mst22)*power2(sb)) + ((mt2 + mu2)*power2(cb)*power2(snt)
     *power2(log(mst12/Q2))*(mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)
     ) - power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*
     mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-mst14 + DeltaInv(mw2,
     msb12,mst12)*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2
     + power3(mst12)) + power4(mst12)))))/msb12 + (2*mt2*mu2*power2(log(mst12/
     Q2))*power4(cb)*(mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-mst14 + DeltaInv(mw2,msb12,mst12)
     *(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(msb12*(mst12 - mst22)*power2(sb)) + (mu2*log(
     mst12/Q2)*power2(cb)*power2(snt)*((5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2) - 6*mC2*power2(DeltaInv(msb12,mst12,mC2))*(-3*
     mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 72*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(
     mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2)) - DeltaInv(mw2,msb12,mst12)*(-(msb12*mst12) + mst14 + 5*mst12*mw2 -
     6*mw2*DeltaInv(mw2,msb12,mst12)*(mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12
     + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 +
     power3(mst12)) + power4(mst12)))))/(6.*msb12) + (mu2*log(mst12/Q2)*power2(
     snt)*power4(cb)*((5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,
     mC2) - 6*mC2*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*
     mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(
     mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - DeltaInv(mw2,
     msb12,mst12)*(-(msb12*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,
     msb12,mst12)*(mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) +
     power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14*
     power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) +
     power4(mst12)))))/(12.*msb12*power2(sb)) + ((mt2 + mu2)*log(mst12/Q2)*
     power2(cb)*power2(snt)*(-((5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(
     msb12,mst12,mC2)) + 6*mC2*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*
     mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) +
     72*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*
     mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     DeltaInv(mw2,msb12,mst12)*(-(msb12*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*
     DeltaInv(mw2,msb12,mst12)*(mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*
     mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(12.*msb12) + (mt2*mu2*log(mst12/Q2)*power4(
     cb)*(-((5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + 6*
     mC2*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 -
     msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + 72*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + DeltaInv(mw2,msb12,
     mst12)*(-(msb12*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb12,
     mst12)*(mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) + power3(
     mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14*power2(
     mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(
     mst12)))))/(6.*msb12*(mst12 - mst22)*power2(sb)) - (mt2*mu2*log(mst12/Q2)*
     power2(cb)*(-2*msb12*(mh2 - 6*mst12)*DeltaInv(mh2,mst12,mst12) + 48*msb12*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) + 576*mh2*
     msb12*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) + DeltaInv(mw2,
     msb12,mst12)*(-(msb12*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,
     msb12,mst12)*(mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) +
     power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14*
     power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) +
     power4(mst12)))))/(6.*msb12*(mst12 - mst22)*power2(sb)) + (mt2*(mst12 -
     mst22 - 2*mu2)*power2(cb)*power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12)
     - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(msb12*msb22*(
     mst12 - mst22)) + (2*mt2*mu2*power2(cb)*power2(snb)*power2(log(mst12/Q2))*
     (msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(
     msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2))
     + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*
     mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))
     )/(msb12*msb22*(mst12 - mst22)) + (2*mu2*power2(cb)*power2(snb)*power2(
     snt)*power2(log(mst12/Q2))*(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) -
     2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(-(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(
     mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(msb12*msb22) + (2*mt2*mu2*power2(snb)*power2(
     log(mst12/Q2))*power4(cb)*(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) -
     2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(-(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(
     mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(msb12*msb22*(mst12 - mst22)*power2(sb)) + (
     mu2*power2(snb)*power2(snt)*power2(log(mst12/Q2))*power4(cb)*(msb22*mst14*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12)
     - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(msb12*msb22*
     power2(sb)) + (power2(snb)*power2(snt)*power2(log(mst12/Q2))*(msb22*mst14*
     mu2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*mu2*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mu2*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + msb12*mu2*power2(
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + mt2*power2(
     mw2)*(-(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(
     mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(
     mst12)))))/(msb12*msb22) - (mt2*log(mst12/Q2)*power2(snb)*(msb22*(-(msb12*
     mst12) + mst14 + 5*mst12*mw2)*DeltaInv(mw2,msb12,mst12) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(mst12*mw2*(-3*msb12 + mw2) - mst14*(
     msb12 + 14*mw2) + power3(mst12)) - 72*msb22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(
     mst14*mw2 + power3(mst12)) + power4(mst12)) - msb12*DeltaInv(mw2,msb22,
     mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,
     mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) + power3(
     mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*power2(
     mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(
     mst12)))))/(12.*msb12*msb22) - (mu2*log(mst12/Q2)*power2(cb)*power2(snb)*
     power2(snt)*(msb22*(-(msb12*mst12) + mst14 + 5*mst12*mw2)*DeltaInv(mw2,
     msb12,mst12) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(mst12*mw2*(-
     3*msb12 + mw2) - mst14*(msb12 + 14*mw2) + power3(mst12)) - 72*msb22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)) -
     msb12*DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*
     mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 +
     14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     )*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(12.*msb12*msb22*power2(sb)) + (2*mt2*mu2*log(
     msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*((mC2*mst12 + msb12*mst12 +
     3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((
     msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)
     *power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(
     msb12*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*
     power2(mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/(msb12*(
     mst12 - mst22)) + (2*mu2*log(msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(
     snb)*power2(snt)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb12 + (2*mt2*mu2*log(msb12/Q2)*log(mst12/Q2)*
     power4(cb)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mt2*
     mu2*log(msb12/Q2)*log(mst12/Q2)*power2(snb)*power4(cb)*((mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) +
     power2(mC2)*((msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*
     msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(-(msb12*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,mst12)*
     (mst14*power2(mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 + mst12*
     power2(mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/(
     msb12*(mst12 - mst22)*power2(sb)) + (mu2*log(msb12/Q2)*log(mst12/Q2)*
     power2(snb)*power2(snt)*power4(cb)*((mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12
     - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(
     mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*
     mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(
     mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*
     power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/(msb12*power2(sb)
     ) + (log(msb12/Q2)*log(mst12/Q2)*power2(snb)*power2(snt)*((mC2*mst12 +
     msb12*mst12 + 3*mst14)*mu2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) +
     mu2*power2(mC2)*((msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (
     5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 +
     mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)
     ))))/msb12 + (2*mt2*mu2*log(msb22/Q2)*log(mst12/Q2)*power2(snb)*power4(cb)
     *((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) + power2(mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*mst14 - mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/(msb22*(mst12 - mst22)*power2(sb)) - (power2(
     snt)*power2(log(mst12/Q2))*(-(mH2*msb12*mst12*mst14*mu2*power2(DeltaInv(
     mH2,mst12,mst12))) - 2*msb12*(2*mH2*mst22 + mst12*mst22 + 2*mst24)*mu2*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 2*msb12*mu2*power2(mH2)*(-
     ((3*mst12*mst22 + 2*mst24)*power2(mH2)) + 2*mst22*power3(mH2) + (-3*mst12
     + 2*mst22)*power3(mst22) - 2*mH2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)) + mst12*(mst14*mu2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)) - mu2*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(-mst14 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))))/(
     msb12*mst12) + (mu2*power2(cb)*power2(snt)*power2(log(mst22/Q2))*(2*msb12*
     mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 2*msb12*power2(mh2)*
     (mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*(mh2*msb12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) + power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(mst24 + DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2))
     + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(
     mst22))))))/(msb12*mst12*power2(sb)) + (mu2*power2(cb)*power2(Pi)*power2(
     snt)*(-((3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + (3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*(-((5*msb12*mst12 + 3*
     mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(
     mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mC2)*((5*msb12*mst22 + 3*mst24)*power2(mC2) - 3*
     mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + mC2*(14*msb12*
     mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*
     (-((msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))
     ) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + 3*mst12*mw2
     + DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(
     mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) -
     3*mst12*power3(mw2) - 3*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12)
     )*(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) +
     5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)))))/(3.*msb12) + (mu2*power2(Pi)*power2(snt)*power4(cb)*(-((
     3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + (3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     + power2(mC2)*((5*msb12*mst22 + 3*mst24)*power2(mC2) - 3*mst22*power3(mC2)
     + (5*msb12 - 3*mst22)*power3(mst22) + mC2*(14*msb12*mst24 + 3*power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-((msb12*mst22 +
     3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*
     (14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)))))/(6.*msb12*power2(sb)) + ((mt2 + mu2)*power2(cb)*power2(
     Pi)*power2(snt)*((3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - (3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((5*msb12*mst12 + 3*
     mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(
     mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mC2)*(-((5*msb12*mst22 + 3*mst24)*power2(mC2)) + 3*
     mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - mC2*(14*msb12*
     mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*
     ((msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) -
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*
     (14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)))))/(6.*msb12) + (mt2*mu2*power2(Pi)*power4(cb)*((3*mC2*
     mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2)) - (3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + power2(mC2)*((5*msb12*mst12 + 3*mst14)*power2(mC2) -
     3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*
     (-((5*msb12*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-5*
     msb12 + 3*mst22)*power3(mst22) - mC2*(14*msb12*mst24 + 3*power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((msb12*mst22 + 3*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(
     14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*
     msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 +
     5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) +
     3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(3.*msb12*
     (mst12 - mst22)*power2(sb)) + (mu2*power2(cb)*power2(snt)*(mC2*msb12*
     DeltaInv(msb12,mst12,mC2) - mC2*msb12*DeltaInv(mst22,msb12,mC2) - 42*(3*
     mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) + 42*(3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 42*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     - 42*power2(mC2)*(-((5*msb12*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*
     power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - mC2*(14*msb12*mst24 +
     3*power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + mw2*(msb12*DeltaInv(
     mst22,mw2,msb12) - 42*mw2*(msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + DeltaInv(mw2,msb12,mst12)*(-msb12 + 42*mw2*(
     msb12*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb12,mst12) + 42*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*
     power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(
     mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) - 42*mw2*power3(DeltaInv(
     mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*
     mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*
     power3(mw2) - 3*power4(mst22)))))/(3.*msb12) + (mu2*power2(snt)*power4(cb)
     *(mC2*msb12*DeltaInv(msb12,mst12,mC2) - mC2*msb12*DeltaInv(mst22,msb12,
     mC2) - 42*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + 42*(3*mC2*mst22 + msb12*mst22 + 3*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*power2(mC2)*(-((5*
     msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*
     mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 42*power2(mC2)*(-((5*msb12*mst22 + 3*mst24)*
     power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) -
     mC2*(14*msb12*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))
     + mw2*(msb12*DeltaInv(mst22,mw2,msb12) - 42*mw2*(msb12*mst22 + 3*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) + DeltaInv(mw2,msb12,mst12)*
     (-msb12 + 42*mw2*(msb12*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb12,
     mst12) + 42*mw2*power2(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*mw2 + 5*
     msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*
     mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) - 42*mw2*
     power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(6.*msb12*power2(sb)) +
     (mt2*mu2*power4(cb)*(-(mC2*msb12*DeltaInv(msb12,mst12,mC2)) + mC2*msb12*
     DeltaInv(mst22,msb12,mC2) + 42*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - 42*(3*mC2*mst22 + msb12*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 42*
     power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(
     mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 42*power2(mC2)*(-((5*
     msb12*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb12 + 3*
     mst22)*power3(mst22) - mC2*(14*msb12*mst24 + 3*power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + mw2*(-(msb12*DeltaInv(mst22,mw2,msb12)) + 42*
     mw2*(msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)
     ) - DeltaInv(mw2,msb12,mst12)*(-msb12 + 42*mw2*(msb12*mst12 + 3*mst14 + 3*
     mst12*mw2)*DeltaInv(mw2,msb12,mst12) + 42*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(
     mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) -
     3*power4(mst12))) + 42*mw2*power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*
     mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*
     power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(
     mst22)))))/(3.*msb12*(mst12 - mst22)*power2(sb)) + ((mst12 - mst22 - 2*
     mt2)*mu2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*(-((mC2*mst22 + msb12*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + power2(
     mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb12
     + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*
     mst24*mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(
     mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(
     msb12*(-mst12 + mst22)) + (2*(mst12 - mst22 - mt2)*mu2*log(msb12/Q2)*log(
     mst22/Q2)*power2(cb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)
     *power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb12*(mst12 - mst22)) +
     ((mst12 - mst22 - 2*mt2)*mu2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*
     power2(snb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*
     power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb12*(mst12 - mst22)) +
     (2*(mst12 - mst22 - mt2)*mu2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*
     power2(snb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*
     power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb12*(-mst12 + mst22))
     + ((mt2 + mu2)*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(snt)*(-((mC2*
     mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*power3(
     mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,
     msb12)*(10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) +
     5*msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*
     power4(mst22)))))/msb12 + (2*mu2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*
     power2(snb)*power2(snt)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)
     *power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) +
     mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(
     mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 +
     3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 +
     msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*
     mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/msb12 - ((mt2
     + mu2)*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*power2(snt)*(-((
     mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*
     power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) - mst24*
     power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(
     mw2) - 3*power4(mst22)))))/msb12 + ((mst12 - mst22 - 2*mt2)*mu2*log(msb12/
     Q2)*log(mst22/Q2)*power4(cb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*
     mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*
     power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*
     mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*
     mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22)
     + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb12*(
     mst12 - mst22)*power2(sb)) + (2*mt2*mu2*log(msb12/Q2)*log(mst22/Q2)*
     power4(cb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*
     power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb12*(mst12 - mst22)*
     power2(sb)) + ((mst12 - mst22 - 2*mt2)*mu2*log(msb12/Q2)*log(mst22/Q2)*
     power2(snb)*power4(cb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)
     *power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb12*(-mst12 + mst22)*
     power2(sb)) + (mu2*log(msb12/Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*
     power4(cb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*
     power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb12*power2(sb)) + (
     log(msb12/Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*(-((mC2*mst22 + msb12*
     mst22 + 3*mst24)*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + mu2*
     power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-
     5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)
     *(10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(
     mst22)))))/msb12 + ((mst12 - mst22 - 2*mt2)*mu2*log(msb22/Q2)*log(mst22/
     Q2)*power2(cb)*power2(snb)*(-((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*((-(msb22*mst22) +
     mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(
     mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*mst24*mw2 +
     msb22*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(mst22) + 5*
     mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb22*(-mst12
     + mst22)) + (2*(mst12 - mst22 - mt2)*mu2*log(msb22/Q2)*log(mst22/Q2)*
     power2(cb)*power2(snb)*(-((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*((-(msb22*mst22) + mst24)
     *power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*mst24*mw2 + msb22*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb22*(mst12 - mst22)) +
     ((mt2 + mu2)*log(msb22/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*power2(
     snt)*(-((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2))) + power2(mC2)*((-(msb22*mst22) + mst24)*power2(mC2) +
     mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2 +
     DeltaInv(mst22,mw2,msb22)*(10*msb22*mst24*mw2 + msb22*mst22*power2(mw2) -
     mst24*power2(mw2) + 5*msb22*power3(mst22) + 5*mw2*power3(mst22) - mst22*
     power3(mw2) - 3*power4(mst22)))))/msb22 + ((mst12 - mst22 - 2*mt2)*mu2*
     log(msb22/Q2)*log(mst22/Q2)*power2(snb)*power4(cb)*(-((mC2*mst22 + msb22*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))) + power2(
     mC2)*((-(msb22*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22
     + 3*mst22)*power3(mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))
     *(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*
     mst24*mw2 + msb22*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(
     mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(
     msb22*(mst12 - mst22)*power2(sb)) + (2*mt2*mu2*log(msb22/Q2)*log(mst22/Q2)
     *power2(snb)*power4(cb)*(-((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)
     *power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*((-(msb22*mst22) +
     mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(
     mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*mst24*mw2 +
     msb22*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(mst22) + 5*
     mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb22*(mst12
     - mst22)*power2(sb)) + (mu2*log(mst12/Q2)*log(mst22/Q2)*power2(cb)*(-((
     mA2*mst22 + mst12*mst22 + 3*mst24)*power2(mA2)*power2(DeltaInv(mst22,mA2,
     mst12))) + power2(mA2)*((-(mst12*mst22) + mst24)*power2(mA2) + mst22*
     power3(mA2) + (-5*mst12 + 3*mst22)*power3(mst22) - 5*mA2*(2*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) + power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + 3*mst24 + mst22*mz2 + DeltaInv(
     mst22,mz2,mst12)*(10*mst12*mst24*mz2 + mst12*mst22*power2(mz2) - mst24*
     power2(mz2) + 5*mst12*power3(mst22) + 5*mz2*power3(mst22) - mst22*power3(
     mz2) - 3*power4(mst22)))))/mst12 + (mu2*log(mst12/Q2)*log(mst22/Q2)*
     power4(cb)*(-((mA2*mst22 + mst12*mst22 + 3*mst24)*power2(mA2)*power2(
     DeltaInv(mst22,mA2,mst12))) + power2(mA2)*((-(mst12*mst22) + mst24)*
     power2(mA2) + mst22*power3(mA2) + (-5*mst12 + 3*mst22)*power3(mst22) - 5*
     mA2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) +
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + 3*mst24 +
     mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(10*mst12*mst24*mz2 + mst12*mst22*
     power2(mz2) - mst24*power2(mz2) + 5*mst12*power3(mst22) + 5*mz2*power3(
     mst22) - mst22*power3(mz2) - 3*power4(mst22)))))/(2.*mst12*power2(sb)) - (
     mu2*log(mst12/Q2)*log(mst22/Q2)*power2(cb)*((mh2*mst22 + mst12*mst22 + 3*
     mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((
     mst12*mst22 - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)
     *power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)) + power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*
     mst22 + 3*mst24 + mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(10*mst12*mst24*
     mz2 + mst12*mst22*power2(mz2) - mst24*power2(mz2) + 5*mst12*power3(mst22)
     + 5*mz2*power3(mst22) - mst22*power3(mz2) - 3*power4(mst22)))))/(2.*mst12*
     power2(sb)) + (2*mu2*power2(cb)*power2(snt)*power2(log(msb12/Q2))*(-((2*
     mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     power2(mC2)*((3*msb12*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) +
     (3*msb12 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))
     *power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-((msb12*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,
     msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(10*
     msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb12 + ((mt2 + mu2)*power2(cb)*power2(snb)*power2(snt)*
     power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)
     *power2(DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*
     mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((3*msb12*mst22 + 2*mst24)*
     power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) + 2*
     mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*
     mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22)))))/msb12 + (2*mt2*mu2*power2(snb)*
     power2(log(msb12/Q2))*power4(cb)*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*
     mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(
     mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-
     3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((3*msb12*mst22 + 2*mst24)
     *power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) +
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*
     mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22)*power2(sb))
     + (mu2*power2(snt)*power2(log(msb12/Q2))*power4(cb)*(-((2*mC2*mst12 +
     msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (
     2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((
     3*msb12*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*
     mst22)*power3(mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-((msb12*mst22 + 2*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(
     mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) -
     2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2
     + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22)
     + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*
     power2(sb)) + (power2(snt)*power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*
     mst12 + 2*mst14)*mu2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (2*
     mC2*mst22 + msb12*mst22 + 2*mst24)*mu2*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + mu2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + mu2*power2(
     mC2)*((3*msb12*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*
     msb12 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + mt2*power2(mw2)*(-((msb12*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,
     msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(10*
     msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb12 + ((mt2 + mu2)*power2(cb)*power2(snt)*power2(log(
     msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - (2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((3*msb12*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((
     msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) -
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb12 + (2*mu2*power2(cb)*power2(snb)*power2(snt)*
     power2(log(msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) - (2*mC2*mst22 + msb12*mst22 + 2*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((3*msb12*
     mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*
     power3(mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2))
     + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*
     msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(
     mw2)*((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,
     mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*
     power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/msb12 + (2*mt2*mu2*power2(log(msb12/Q2))
     *power4(cb)*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - (2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((3*msb12*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((
     msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) -
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb12*(mst12 - mst22)*power2(sb)) + (mu2*power2(snb)*
     power2(snt)*power2(log(msb12/Q2))*power4(cb)*((2*mC2*mst12 + msb12*mst12 +
     2*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - (2*mC2*mst22 +
     msb12*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) +
     (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))
     *power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*power2(sb)) + (
     power2(snb)*power2(snt)*power2(log(msb12/Q2))*((2*mC2*mst12 + msb12*mst12
     + 2*mst14)*mu2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - (2*mC2*
     mst22 + msb12*mst22 + 2*mst24)*mu2*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + mu2*power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*
     mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + mu2*power2(
     mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + mt2*power2(mw2)*((msb12*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,
     msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(10*
     msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb12 + (mu2*power2(snb)*power2(log(msb12/Q2))*power4(
     cb)*(2*(2*mC2*mst12 + msb12*mst12 + 2*mst14)*mt2*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*(mst12
     - mst22 - 2*mt2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 2*mt2*
     power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) +
     (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))
     *power3(DeltaInv(msb12,mst12,mC2)) + (-mst12 + mst22 + 2*mt2)*power2(mC2)*
     (-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*
     msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-((mst12 - mst22 - 2*mt2)
     *(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)))
     - 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*
     mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - (mst12 - mst22 - 2*mt2)
     *power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (mu2*power2(log(msb12/Q2))*power4(cb)*(-2*(2*mC2*mst12 +
     msb12*mst12 + 2*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) +
     (2*mC2*mst22 + msb12*mst22 + 2*mst24)*(-mst12 + mst22 + 2*mt2)*power2(mC2)
     *power2(DeltaInv(mst22,msb12,mC2)) + 2*mt2*power2(mC2)*(-((3*msb12*mst12 +
     2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + (mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + 2*(mst24
     + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,
     msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(
     mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*
     mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*
     mu2*power2(cb)*power2(snb)*power2(log(msb12/Q2))*((2*mC2*mst12 + msb12*
     mst12 + 2*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - (2*
     mC2*mst22 + msb12*mst22 + 2*mst24)*(-mst12 + mst22 + mt2)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + mt2*power2(mC2)*((3*msb12*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + (-mst12 + mst22 + mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*(-((mst12 - mst22 - mt2)*(msb12*mst22 + 2*(mst24
     + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) - mt2*power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,
     msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) - (mst12 - mst22 - mt2)*power3(DeltaInv(
     mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*
     mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22)) + (2*mu2*power2(
     cb)*power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*mt2*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*
     mst22 + 2*mst24)*(-mst12 + mst22 + mt2)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + mt2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - (-mst12 +
     mst22 + mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((
     mst12 - mst22 - mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + (
     mst12 - mst22 - mt2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2
     + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22)
     + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(
     mst12 - mst22)) + (power2(cb)*power2(log(msb12/Q2))*(-((2*mC2*mst12 +
     msb12*mst12 + 2*mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*(mst12
     - mst22 - 2*mt2)*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + mt2*(
     mst12 - mst22 - 2*mu2)*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(
     mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(
     5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + (-
     mst12 + mst22 + 2*mt2)*mu2*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*(-((mst12 - mst22 - 2*mt2)*mu2*(msb12*mst22 + 2*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + mt2*(mst12 - mst22 - 2*
     mu2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*
     mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - (mst12 - mst22 - 2*mt2)
     *mu2*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*
     mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*
     power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 -
     mst22)) - (power2(cb)*power2(snb)*power2(log(msb12/Q2))*(-((2*mC2*mst12 +
     msb12*mst12 + 2*mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*(mst12
     - mst22 - 2*mt2)*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + mt2*(
     mst12 - mst22 - 2*mu2)*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(
     mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(
     5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + (-
     mst12 + mst22 + 2*mt2)*mu2*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*(-((mst12 - mst22 - 2*mt2)*mu2*(msb12*mst22 + 2*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + mt2*(mst12 - mst22 - 2*
     mu2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*
     mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - (mst12 - mst22 - 2*mt2)
     *mu2*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*
     mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*
     power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 -
     mst22)) + (2*mu2*power2(cb)*power2(snb)*power2(snt)*power2(log(msb22/Q2))*
     (-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-
     ((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)))
     + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (mu2*power2(snb)*power2(snt)*power2(log(msb22/
     Q2))*power4(cb)*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)
     *power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) -
     2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-
     ((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)))
     + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb22*power2(sb)) + (power2(snb)*power2(snt)*power2(
     log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*mu2*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)
     *mu2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + mu2*power2(mC2)*(-((
     3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + mu2*power2(mC2)*((3*msb22*mst22 + 2*mst24)*
     power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*
     mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     mt2*power2(mw2)*(-((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*
     mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*
     msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - power3(
     DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22)))))/msb22 + ((mt2 + mu2)*power2(cb)*
     power2(snb)*power2(snt)*power2(log(msb22/Q2))*((2*mC2*mst12 + msb22*mst12
     + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 +
     msb22*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) +
     power2(mC2)*((3*msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) +
     (3*msb22 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))
     *power3(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*
     mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/msb22 + (2*mt2*mu2*
     power2(snb)*power2(log(msb22/Q2))*power4(cb)*((2*mC2*mst12 + msb22*mst12 +
     2*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 +
     msb22*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) +
     power2(mC2)*((3*msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) +
     (3*msb22 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))
     *power3(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*
     mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb22*(mst12 - mst22)*
     power2(sb)) + (mu2*power2(snb)*power2(log(msb22/Q2))*power4(cb)*(-2*(2*
     mC2*mst12 + msb22*mst12 + 2*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*(-mst12 + mst22 + 2*
     mt2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 2*mt2*power2(mC2)*(-(
     (3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + (mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + power2(mw2)*((mst12 - mst22 - 2*mt2)*(msb22*
     mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*(mst14 + mst12*mw2) +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(
     DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22)))))/(msb22*(mst12 - mst22)*power2(sb))
     + (2*mu2*power2(cb)*power2(snb)*power2(log(msb22/Q2))*(-((2*mC2*mst12 +
     msb22*mst12 + 2*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)))
     + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*(-mst12 + mst22 + mt2)*power2(mC2)
     *power2(DeltaInv(mst22,msb22,mC2)) + mt2*power2(mC2)*(-((3*msb22*mst12 +
     2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) - (-mst12 + mst22 + mt2)*power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*((mst12 - mst22 - mt2)*(msb22*mst22 + 2*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) + mt2*power2(DeltaInv(mw2,
     msb22,mst12))*(msb22*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb22,
     mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(
     mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) -
     2*power4(mst12))) + (mst12 - mst22 - mt2)*power3(DeltaInv(mst22,mw2,msb22)
     )*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) +
     3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb22*(mst12 - mst22)) + (power2(cb)*power2(snb)*
     power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*mt2*(mst12
     - mst22 - 2*mu2)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*
     mst22 + msb22*mst22 + 2*mst24)*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + mt2*(mst12 - mst22 - 2*mu2)*power2(
     mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-
     3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + (-mst12 + mst22 + 2*mt2)*mu2*power2(
     mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-((mst12 - mst22 - 2*mt2)
     *mu2*(msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb22))) + mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) - (mst12 - mst22 - 2*mt2)*mu2*power3(DeltaInv(mst22,mw2,msb22))*(
     10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb22*(mst12 - mst22)) + ((mt2 + mu2)*power2(cb)*
     power2(snt)*power2(log(mst22/Q2))*(-(mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2))) + power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*
     power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24 +
     DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*
     msb12*(mst24*mw2 + power3(mst22)) - power4(mst22)))))/msb12 + (2*mt2*mu2*
     power2(log(mst22/Q2))*power4(cb)*(-(mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2))) + power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*
     power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24 +
     DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*
     msb12*(mst24*mw2 + power3(mst22)) - power4(mst22)))))/(msb12*(mst12 -
     mst22)*power2(sb)) + ((mst12 - mst22 - 2*mt2)*mu2*power2(cb)*power2(snb)*
     power2(log(mst22/Q2))*(-(msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2))) + msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) +
     msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) -
     2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) -
     msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     power2(mw2)*(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(
     mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) -
     power4(mst22)))))/(msb12*msb22*(mst12 - mst22)) + (2*mu2*power2(cb)*
     power2(snb)*power2(snt)*power2(log(mst22/Q2))*(-(msb22*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2))) + msb12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + msb22*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - msb12*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(msb22*mst24*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*
     power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(mst22)))))/(msb12*
     msb22) + (mu2*power2(snb)*power2(snt)*power2(log(mst22/Q2))*power4(cb)*(-(
     msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb22*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - msb12*power2(mC2)*(
     mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(msb22*
     mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,
     msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*
     power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) +
     msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(mw2)) + 2*mw2*
     power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(mst22)))))/(
     msb12*msb22*power2(sb)) + (power2(snb)*power2(snt)*power2(log(mst22/Q2))*(
     -(msb22*mst24*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*
     mst24*mu2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb22*mu2*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - msb12*
     mu2*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*
     mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     mt2*power2(mw2)*(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*
     power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(
     mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(
     mst22)) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(
     mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(
     mst22)) - power4(mst22)))))/(msb12*msb22) + (mt2*mu2*power2(cb)*power2(
     mw2)*power2(Pi)*power2(snb)*(msb22*(msb12*mst22 + 3*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb12)) - msb12*(msb22*mst22 + 3*mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*power2(DeltaInv(mw2,msb12,
     mst12)) - 3*msb22*mst12*mw2*power2(DeltaInv(mw2,msb12,mst12)) + msb12*
     msb22*mst12*power2(DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst12*mw2*power2(DeltaInv(mw2,msb22,
     mst12)) - 14*msb12*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,mst12)) - 5*
     msb12*msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*
     mst14*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*msb22*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mw2*power3(
     mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst12*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 14*msb12*msb22*mst14*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) + 5*msb12*msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12)) + 5*msb12*msb22*power3(mst12)*power3(DeltaInv(mw2,msb22,
     mst12)) + 3*msb12*mw2*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 3*
     msb12*mst12*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 3*msb12*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + msb22*power3(DeltaInv(mst22,mw2,msb12))*
     (14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*
     mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(
     mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22))))/(
     3.*msb12*msb22*(mst12 - mst22)*power2(sb)) + (mu2*power2(cb)*power2(mw2)*
     power2(Pi)*power2(snb)*power2(snt)*(-(msb22*(msb12*mst22 + 3*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + msb12*(msb22*mst22 + 3*
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) + msb12*msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*power2(DeltaInv(
     mw2,msb12,mst12)) + 3*msb22*mst12*mw2*power2(DeltaInv(mw2,msb12,mst12)) -
     msb12*msb22*mst12*power2(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*
     power2(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst12*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) + 14*msb12*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,mst12))
     + 5*msb12*msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*
     msb22*mst14*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mw2*power3(
     mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst12*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 14*msb12*msb22*mst14*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12)) - 5*msb12*msb22*power3(mst12)*power3(DeltaInv(mw2,msb22,
     mst12)) - 3*msb12*mw2*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 3*
     msb12*mst12*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb12*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - msb22*power3(DeltaInv(mst22,mw2,msb12))*
     (14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)) + msb12*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*
     mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(
     mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22))))/(
     6.*msb12*msb22*power2(sb)) + (mu2*power2(cb)*power2(mw2)*power2(Pi)*
     power2(snb)*(msb22*(mst12 - mst22 - 2*mt2)*(msb12*mst22 + 3*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - msb12*(mst12 - mst22 - 2*
     mt2)*(msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb22)) + 2*mt2*(msb22*(msb12*mst12 + 3*(mst14 + mst12*mw2))*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*
     mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*
     power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*mw2 +
     5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) +
     3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) + msb22*(
     mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*
     mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(
     mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) -
     msb12*(mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*
     mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*
     power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(
     mst22))))/(6.*msb12*msb22*(mst12 - mst22)*power2(sb)) + ((mst12 - mst22 -
     2*mt2)*mu2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*
     power4(mst22))))/(msb12*(-mst12 + mst22)*power2(sb)) - (2*mt2*mu2*log(
     msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(-(
     mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*mst24*mw2 + mst22*
     power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/(
     msb12*(mst12 - mst22)*power2(sb)) + ((mst12 - mst22 - 2*mt2)*mu2*log(
     msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*
     power4(mst22))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mt2*mu2*log(
     msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*
     power4(mst22))))/(msb12*(mst12 - mst22)*power2(sb)) + (mu2*log(msb12/Q2)*
     log(mst22/Q2)*power2(cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,
     mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)
     *(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*mst24*mw2 +
     mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22)
     )))/(msb12*power2(sb)) - (mu2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(
     msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(-(mst24*
     power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/(msb12*
     power2(sb)) + ((mst12 - mst22 - 2*mt2)*mu2*log(msb22/Q2)*log(mst22/Q2)*
     power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 5*mw2*power3(mst22) + msb22*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/(msb22*(-
     mst12 + mst22)*power2(sb)) - (2*mt2*mu2*log(msb22/Q2)*log(mst22/Q2)*
     power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 5*mw2*power3(mst22) + msb22*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/(msb22*(
     mst12 - mst22)*power2(sb)) + (mu2*log(msb22/Q2)*log(mst22/Q2)*power2(cb)*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 5*mw2*power3(mst22) + msb22*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/(msb22*
     power2(sb)) + (log(mt2/Q2)*log(mu2/Q2)*power2(mu2)*(msb12*DeltaInv(msb12,
     mt2,mu2) + mst22*DeltaInv(mst22,mt2,mu2) + power2(DeltaInv(msb12,mt2,mu2))
     *(2*msb14*(mt2 + 5*mu2) + msb12*mu2*(6*mt2 + 7*mu2) - power3(msb12)) +
     power2(DeltaInv(mst22,mt2,mu2))*(2*mst24*(mt2 + 5*mu2) + mst22*mu2*(6*mt2
     + 7*mu2) - power3(mst22)) + 4*mu2*power3(DeltaInv(msb12,mt2,mu2))*(2*
     msb14*mu2*(5*mt2 + mu2) + msb12*(3*mt2 - 2*mu2)*power2(mu2) + (3*mt2 + 2*
     mu2)*power3(msb12) - 2*power4(msb12)) + DeltaInv(mt2,mu2,mst12)*(mst12 +
     DeltaInv(mt2,mu2,mst12)*(2*mst14*(mt2 + 5*mu2) + mst12*mu2*(6*mt2 + 7*mu2)
     - power3(mst12)) + 4*mu2*power2(DeltaInv(mt2,mu2,mst12))*(2*mst14*mu2*(5*
     mt2 + mu2) + mst12*(3*mt2 - 2*mu2)*power2(mu2) + (3*mt2 + 2*mu2)*power3(
     mst12) - 2*power4(mst12))) + 4*mu2*power3(DeltaInv(mst22,mt2,mu2))*(2*
     mst24*mu2*(5*mt2 + mu2) + mst22*(3*mt2 - 2*mu2)*power2(mu2) + (3*mt2 + 2*
     mu2)*power3(mst22) - 2*power4(mst22))))/mt2 + (2*mt2*mu2*power2(cb)*
     power2(mw2)*power2(log(msb12/Q2))*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))
     *power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*
     mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*
     msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*
     mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb12*(mst12
     - mst22)*power2(sb)) + (2*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)*
     power2(log(msb12/Q2))*((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (mu2*power2(cb)*power2(mw2)*power2(snt)*power2(log(msb12/Q2)
     )*((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))
     - power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(msb12*power2(sb)) - (mu2*power2(cb)*power2(mw2)*power2(
     snb)*power2(snt)*power2(log(msb12/Q2))*((msb12*mst22 + 2*(mst24 + mst22*
     mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12)
     )*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*
     msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 +
     3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb12*
     power2(sb)) - (mu2*power2(cb)*power2(mw2)*power2(log(msb12/Q2))*((mst12 -
     mst22 - 2*mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*
     msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*
     mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb12*(mst12
     - mst22)*power2(sb)) + (mu2*power2(cb)*power2(mw2)*power2(snb)*power2(log(
     msb12/Q2))*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12)
     )*(msb12*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*
     msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12)
     )*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) +
     3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(msb12*(mst12 - mst22)*power2(sb)) - (mt2*mu2*log(msb12/
     Q2)*power2(cb)*(-((mst12 - mst22)*(msb12 + mw2)) - mst22*DeltaInv(mw2,
     msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)) +
     mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + mst12*DeltaInv(mst22,mw2,
     msb12)*(5*mw2*power2(mst22) - msb12*(power2(mst22) + power2(mw2)) + mst22*
     (mst24 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*mw2) -
     msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) - 6*mst12*
     mst22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2 + mst22*mw2*(6*
     msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) + power3(mst22) + power3(
     mw2)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(10*
     msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(
     mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) -
     2*power4(mst22))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*power2(sb)) + (
     mt2*mu2*log(msb12/Q2)*power2(cb)*power2(snb)*(-((mst12 - mst22)*(msb12 +
     mw2)) - mst22*DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(
     power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)
     ) + mst12*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(
     mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 6*
     mst12*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*
     mw2*(6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) +
     power3(mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*
     mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) +
     power3(mst22) + power3(mw2)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*
     mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)) - 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(6.*msb12*mst12*(mst12 - mst22)*
     mst22*power2(sb)) + (mu2*log(msb12/Q2)*power2(cb)*power2(snt)*(-((mst12 -
     mst22)*(msb12 + mw2)) - mst22*DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(
     mst12) - msb12*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(
     mw2)) + power3(mw2)) + mst12*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(
     mst22) - msb12*(power2(mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(
     mw2)) + power3(mw2)) + 6*mst12*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))
     *(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)
     ) + power3(mst12) + power3(mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(
     mst22,mw2,msb12))*(23*mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(
     mst24 + power2(mw2)) + power3(mst22) + power3(mw2)) + 72*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) - 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 +
     3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb12*
     mst12*mst22*power2(sb)) + (mu2*log(msb12/Q2)*power2(cb)*power2(snb)*
     power2(snt)*((mst12 - mst22)*(msb12 + mw2) + mst22*DeltaInv(mw2,msb12,
     mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)) + mst12*
     (mst14 + 5*power2(mw2)) + power3(mw2)) - mst12*DeltaInv(mst22,mw2,msb12)*(
     5*mw2*power2(mst22) - msb12*(power2(mst22) + power2(mw2)) + mst22*(mst24 +
     5*power2(mw2)) + power3(mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14
     + power2(mw2)) + power3(mst12) + power3(mw2)) + 6*mst12*mst22*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) -
     msb12*(mst24 + power2(mw2)) + power3(mst22) + power3(mw2)) - 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*mw2 +
     3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) +
     2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) + 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 +
     3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb12*
     mst12*mst22*power2(sb)) + (mu2*log(msb12/Q2)*power2(cb)*power2(snb)*(-((
     mst12 - mst22)*(msb12*(mst12 - 2*mt2) - 2*mt2*mw2 + mst12*(mst22 + mw2)))
     + 2*mst22*mt2*DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(
     power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)
     ) + mst12*(mst12 - mst22 - 2*mt2)*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(
     mst22) - msb12*(power2(mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(
     mw2)) + power3(mw2)) - 12*mst12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14 +
     power2(mw2)) + power3(mst12) + power3(mw2)) - 6*mst12*mst22*(mst12 - mst22
     - 2*mt2)*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2 + mst22*mw2*(
     6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) + power3(mst22) + power3(
     mw2)) - 144*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     (10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb12*mst12*(mst12
     - mst22)*mst22*power2(sb)) + (mu2*log(msb12/Q2)*power2(cb)*((mst12 -
     mst22)*(msb12*(mst12 - 2*mt2) - 2*mt2*mw2 + mst12*(mst22 + mw2)) - 2*
     mst22*mt2*DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(
     mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) -
     mst12*(mst12 - mst22 - 2*mt2)*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(
     mst22) - msb12*(power2(mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(
     mw2)) + power3(mw2)) + 12*mst12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14 +
     power2(mw2)) + power3(mst12) + power3(mw2)) + 6*mst12*mst22*(mst12 - mst22
     - 2*mt2)*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2 + mst22*mw2*(
     6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) + power3(mst22) + power3(
     mw2)) + 144*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     (10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12)) + 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb12*mst12*(mst12
     - mst22)*mst22*power2(sb)) + (2*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)
     *power2(log(msb22/Q2))*(-((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*
     mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) -
     power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb22*(mst12 - mst22)*
     power2(sb)) + (mu2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(
     log(msb22/Q2))*((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*
     mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*
     msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(msb22*power2(sb)) - (mu2*power2(
     cb)*power2(mw2)*power2(snb)*power2(log(msb22/Q2))*((mst12 - mst22 - 2*mt2)
     *(msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) +
     2*mt2*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*(mst14 + mst12*
     mw2) + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 - 2*mt2)
     *power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb22*(mst12 - mst22)*
     power2(sb)) - (mt2*mu2*log(msb22/Q2)*power2(cb)*power2(snb)*(-((mst12 -
     mst22)*(msb22 + mw2)) - mst22*DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(
     mst12) - msb22*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(
     mw2)) + power3(mw2)) + mst12*DeltaInv(mst22,mw2,msb22)*(5*mw2*power2(
     mst22) - msb22*(power2(mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(
     mw2)) + power3(mw2)) + 6*mst12*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12))
     *(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)
     ) + power3(mst12) + power3(mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(
     mst22,mw2,msb22))*(23*mst24*mw2 + mst22*mw2*(6*msb22 + 23*mw2) - msb22*(
     mst24 + power2(mw2)) + power3(mst22) + power3(mw2)) + 72*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*
     msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) - 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 +
     3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*log(msb22/Q2)*power2(cb)*
     power2(snb)*power2(snt)*(-((mst12 - mst22)*(msb22 + mw2)) - mst22*
     DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + mst12*
     DeltaInv(mst22,mw2,msb22)*(5*mw2*power2(mst22) - msb22*(power2(mst22) +
     power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*
     mst22*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*
     msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(
     mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(mst22,mw2,msb22))*(23*mst24*mw2
     + mst22*mw2*(6*msb22 + 23*mw2) - msb22*(mst24 + power2(mw2)) + power3(
     mst22) + power3(mw2)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)) - 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb22*mst12*mst22*power2(sb))
     + (mu2*log(msb22/Q2)*power2(cb)*power2(snb)*((mst12 - mst22)*(msb22*(mst12
     - 2*mt2) - 2*mt2*mw2 + mst12*(mst22 + mw2)) - 2*mst22*mt2*DeltaInv(mw2,
     msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) + power2(mw2)) +
     mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) - mst12*(mst12 - mst22 - 2*
     mt2)*DeltaInv(mst22,mw2,msb22)*(5*mw2*power2(mst22) - msb22*(power2(mst22)
     + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 12*mst12*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*
     (6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(
     mw2)) + 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*power2(DeltaInv(mst22,
     mw2,msb22))*(23*mst24*mw2 + mst22*mw2*(6*msb22 + 23*mw2) - msb22*(mst24 +
     power2(mw2)) + power3(mst22) + power3(mw2)) + 144*mst12*mst22*mt2*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) + 72*mst12*mst22*(
     mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(10*
     msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(12.*msb22*mst12*(mst12 - mst22)*mst22*power2(sb)) + (2*
     Fin3(mst22,mt2,mu2,Q2)*power2(mu2)*power2(DeltaInv(mst22,mt2,mu2))*(mst24
     + mst22*(mt2 + 3*mu2) + DeltaInv(mst22,mt2,mu2)*(5*mst24*mu2*(2*mt2 + mu2)
     + mst22*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*power3(mst22) - power4(
     mst22))))/mt2 + (power2(log(mu2/Q2))*power3(mu2)*((msb14 + msb12*(mt2 + 3*
     mu2))*power2(DeltaInv(msb12,mt2,mu2)) + (mst24 + mst22*(mt2 + 3*mu2))*
     power2(DeltaInv(mst22,mt2,mu2)) + power3(DeltaInv(msb12,mt2,mu2))*(5*
     msb14*mu2*(2*mt2 + mu2) + msb12*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*
     power3(msb12) - power4(msb12)) + power2(DeltaInv(mt2,mu2,mst12))*(mst14 +
     mst12*mt2 + 3*mst12*mu2 + DeltaInv(mt2,mu2,mst12)*(5*mst14*mu2*(2*mt2 +
     mu2) + mst12*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*power3(mst12) -
     power4(mst12))) + power3(DeltaInv(mst22,mt2,mu2))*(5*mst24*mu2*(2*mt2 +
     mu2) + mst22*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*power3(mst22) -
     power4(mst22))))/mt2 + ((mst12 - mst22 - 2*mt2)*mu2*power2(cb)*power2(mw2)
     *power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*power2(DeltaInv(mst22,mw2,
     msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,
     mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*
     mw2 + power3(mst22)) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 +
     power3(mst22)) - power4(mst22))))/(msb12*msb22*(mst12 - mst22)*power2(sb))
     + (2*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(log(mst22/Q2))*(
     msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(
     mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2))
     + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(
     mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(mw2)) +
     2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(mst22))
     ))/(msb12*msb22*(mst12 - mst22)*power2(sb)) + ((mt2 + mu2)*log(msb12/Q2)*
     log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snt)*((msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22)
     + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/msb12 + (mu2*log(msb12/Q2)*log(mw2/Q2)*
     power2(cb)*power2(mw2)*power2(snt)*((msb12*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22) +
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/(msb12*power2(sb)) + (mt2*log(msb12/Q2)*
     log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*((msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22)
     + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/msb12 + (2*mu2*log(msb12/Q2)*log(mw2/Q2)*
     power2(cb)*power2(mw2)*power2(snb)*power2(snt)*((msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22)
     + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/msb12 + (mu2*log(msb12/Q2)*log(mw2/Q2)*
     power2(mw2)*power2(snb)*power2(snt)*power4(cb)*((msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22)
     + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/(msb12*power2(sb)) - (mu2*log(msb12/Q2)*
     log(mw2/Q2)*power2(cb)*power2(mw2)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 +
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12)
     )*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*
     mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22)))
     )/(msb12*(mst12 - mst22)*power2(sb)) + (mu2*log(msb12/Q2)*log(mw2/Q2)*
     power2(cb)*power2(mw2)*power2(snb)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 +
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12)
     )*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*
     mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22)))
     )/(msb12*(mst12 - mst22)*power2(sb)) + (mu2*log(msb12/Q2)*log(mw2/Q2)*
     power2(mw2)*power4(cb)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*
     power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(
     mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*(
     mst12 - mst22)*power2(sb)) - (mu2*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snb)*power4(cb)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*
     power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(
     mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*(
     mst12 - mst22)*power2(sb)) + (2*mu2*log(msb12/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*((mst12 - mst22 - mt2)*(msb12*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + mt2*power2(DeltaInv(mw2,msb12,mst12))*
     (msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - mt2)*power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*
     power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22))
     - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)) - (2*mu2*
     log(msb12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*((mst12 -
     mst22 - mt2)*(msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 +
     3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + (mst12 - mst22 - mt2)*power3(
     DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22) +
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)) + (log(msb12/Q2)*
     log(mw2/Q2)*power2(cb)*power2(mw2)*((mst12 - mst22 - 2*mt2)*mu2*(msb12*
     mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - mt2*(
     mst12 - mst22 - 2*mu2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) -
     mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(
     mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 - mst22 - 2*mt2)*
     mu2*power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(
     mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*
     mst22*power3(mw2) - power4(mst22))))/(msb12*(-mst12 + mst22)) + ((mt2 +
     mu2)*log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(
     snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)
     ) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(5*
     mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb22 + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*
     power2(snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(
     5*mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(
     msb22*power2(sb)) - (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snb)*((mst12 - mst22 - 2*mt2)*(msb22*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,mst12)
     )*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb22*(mst12 - mst22)*
     power2(sb)) + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power4(cb)*((mst12 - mst22 - 2*mt2)*(msb22*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,mst12)
     )*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb22*(mst12 - mst22)*
     power2(sb)) + (2*mu2*log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snb)*((mst12 - mst22 - mt2)*(msb22*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + mt2*power2(DeltaInv(mw2,msb22,mst12))*
     (msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - mt2)*power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(mw2) - mw2*
     power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22))
     - 3*mst22*power3(mw2) - power4(mst22))))/(msb22*(mst12 - mst22)) + (log(
     msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*((mst12 - mst22 -
     2*mt2)*mu2*(msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) - mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*mu2*power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(mw2)
     - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb22*(-mst12 + mst22))
     + ((mst12 - mst22 - 2*mt2)*mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)) + (2*(mst12 - mst22
     - mt2)*mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*
     mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(
     mst22))))/(msb12*(-mst12 + mst22)) + ((mst12 - mst22 - 2*mt2)*mu2*log(
     mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(
     mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(
     mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*(mst12
     - mst22)*power2(sb)) + (2*mt2*mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)*power2(sb)) + (mt2*
     log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,
     mw2,msb12))*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(
     mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(
     mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/msb12 + (2*
     mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snt)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*
     mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(
     mst22))))/msb12 - ((mt2 + mu2)*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 +
     mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*
     power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) -
     mst22*power3(mw2) - power4(mst22))))/msb12 - (mu2*log(mst22/Q2)*log(mw2/
     Q2)*power2(cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(
     msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*power2(sb)) +
     ((mst12 - mst22 - 2*mt2)*mu2*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*power4(cb)*(msb12*mst22 + mst24 + mst22*mw2 +
     DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(
     6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*(-mst12 + mst22)*power2(sb)) - (2*mt2*mu2*log(
     mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(cb)*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(
     mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(
     mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*(mst12
     - mst22)*power2(sb)) + (mu2*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     snt)*power2(DeltaInv(mst22,mw2,msb12))*power4(cb)*(msb12*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/(msb12*power2(sb)) + ((mst12 - mst22 - 2*
     mt2)*mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*(
     msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2
     + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) +
     msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(
     mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*
     power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22*(-mst12
     + mst22)) + (2*(mst12 - mst22 - mt2)*mu2*log(mst22/Q2)*log(mw2/Q2)*power2(
     cb)*power2(mw2)*power2(snb)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22)
     )*(msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*
     power2(mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*
     (6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*msb22*(mst12 - mst22)) + ((mst12 - mst22 - 2*mt2)*
     mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*(msb22*(
     msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2 +
     DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) + msb22*(
     6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22*(-mst12
     + mst22)*power2(sb)) - (2*mt2*mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*power2(snb)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*
     (6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*msb22*(mst12 - mst22)*power2(sb)) - (mt2*log(
     mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*(msb22*(msb12*
     mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*
     power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2 +
     DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) + msb22*(
     6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22) - (2*
     mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(
     snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(
     mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(
     mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22)
     + ((mt2 + mu2)*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(
     snb)*power2(snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*
     power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) -
     mst22*power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*msb22) + (mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snb)*power2(snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*
     (6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*msb22*power2(sb)) + ((mst12 - mst22 - 2*mt2)*mu2*
     log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power4(cb)*(msb22*(
     msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2 +
     DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) + msb22*(
     6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22*(mst12
     - mst22)*power2(sb)) + (2*mt2*mu2*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snb)*power4(cb)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*
     (6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*msb22*(mst12 - mst22)*power2(sb)) - (mu2*log(
     mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*power4(cb)*(
     msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2
     + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) +
     msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(
     mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*
     power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22*power2(
     sb)) + (mu2*log(mst12/Q2)*log(mz2/Q2)*power2(cb)*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + mst24 + 3*mst22*mz2 + DeltaInv(
     mst22,mz2,mst12)*(5*mst24*power2(mz2) - mz2*power3(mst22) + mst12*(10*
     mst24*mz2 + 5*mst22*power2(mz2) + power3(mst22)) - 3*mst22*power3(mz2) -
     power4(mst22))))/(2.*mst12) - (mu2*log(mst12/Q2)*log(mz2/Q2)*power2(cb)*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + mst24 + 3*
     mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(5*mst24*power2(mz2) - mz2*power3(
     mst22) + mst12*(10*mst24*mz2 + 5*mst22*power2(mz2) + power3(mst22)) - 3*
     mst22*power3(mz2) - power4(mst22))))/(2.*mst12*power2(sb)) + (mu2*log(
     mst12/Q2)*log(mz2/Q2)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*
     power4(cb)*(mst12*mst22 + mst24 + 3*mst22*mz2 + DeltaInv(mst22,mz2,mst12)*
     (5*mst24*power2(mz2) - mz2*power3(mst22) + mst12*(10*mst24*mz2 + 5*mst22*
     power2(mz2) + power3(mst22)) - 3*mst22*power3(mz2) - power4(mst22))))/(2.*
     mst12*power2(sb)) - (mu2*log(mst22/Q2)*log(mz2/Q2)*power2(cb)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + mst24 + mst22*mz2 +
     DeltaInv(mst22,mz2,mst12)*(mst24*power2(mz2) + mz2*power3(mst22) + mst12*(
     6*mst24*mz2 + mst22*power2(mz2) + power3(mst22)) - mst22*power3(mz2) -
     power4(mst22))))/(2.*mst12) + (mu2*log(mst22/Q2)*log(mz2/Q2)*power2(cb)*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + mst24 +
     mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(mst24*power2(mz2) + mz2*power3(
     mst22) + mst12*(6*mst24*mz2 + mst22*power2(mz2) + power3(mst22)) - mst22*
     power3(mz2) - power4(mst22))))/(2.*mst12*power2(sb)) - (mu2*log(mst22/Q2)*
     log(mz2/Q2)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*power4(cb)*(
     mst12*mst22 + mst24 + mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(mst24*power2(
     mz2) + mz2*power3(mst22) + mst12*(6*mst24*mz2 + mst22*power2(mz2) +
     power3(mst22)) - mst22*power3(mz2) - power4(mst22))))/(2.*mst12*power2(sb)
     ) + (21*mu2*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22))/(
     2.*mst12) + (21*mu2*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*power4(
     mst22))/(2.*mst12) + (21*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     *power4(mst22))/msb12 - 168*mh2*mt2*power3(DeltaInv(mst22,mst22,mh2))*
     power4(mst22) + 14*power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*power4(
     mst22) - (70*power3(mu2)*power3(DeltaInv(mst22,mt2,mu2))*power4(mst22))/
     mt2 + (4*ca*cb*mt2*mu2*sa*log(mH2/Q2)*log(mst22/Q2)*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,
     mst22)*power4(mst22)))/((mst12 - mst22)*sb) - (4*mt2*mu2*log(mH2/Q2)*log(
     mst22/Q2)*power2(cb)*power2(sa)*power2(DeltaInv(mH2,mst22,mst22))*(mH2*
     mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)))
     /((mst12 - mst22)*power2(sb)) - 2*mt2*log(mh2/Q2)*log(mst22/Q2)*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22)) - (4*ca*cb*mt2*mu2*sa*log(mh2/Q2)*log(
     mst22/Q2)*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) +
     16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)))/((mst12 - mst22)*sb) - (
     4*mt2*mu2*log(mh2/Q2)*log(mst22/Q2)*power2(cb)*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22)))/((mst12 - mst22)*power2(sb)) + (4*mt2*mu2*log(mh2/Q2)
     *log(mst22/Q2)*power2(cb)*power2(sa)*power2(DeltaInv(mst22,mst22,mh2))*(
     mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(
     mst22)))/((mst12 - mst22)*power2(sb)) + (ca*cb*mt2*mu2*sa*(-(mh2*DeltaInv(
     mh2,mst12,mst12)) + mH2*DeltaInv(mH2,mst12,mst12) - mH2*DeltaInv(mH2,
     mst22,mst22) + mh2*DeltaInv(mst22,mst22,mh2) + 42*mH2*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) - 42*mh2*mst24*power2(DeltaInv(mst22,mst22,mh2)
     ) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) -
     42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 6*power3(mst12)) + 252*
     power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) - 252*power2(DeltaInv(
     mst22,mst22,mh2))*power3(mst22) + 1008*mh2*power3(DeltaInv(mh2,mst12,
     mst12))*power4(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power4(
     mst12) + 1008*mH2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) - 1008*
     mh2*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(3.*(mst12 - mst22)*
     sb) - (mt2*mu2*power2(cb)*power2(sa)*(-(mh2*DeltaInv(mh2,mst12,mst12)) +
     mH2*DeltaInv(mH2,mst12,mst12) - mH2*DeltaInv(mH2,mst22,mst22) + mh2*
     DeltaInv(mst22,mst22,mh2) + 42*mH2*mst24*power2(DeltaInv(mH2,mst22,mst22))
     - 42*mh2*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 42*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) - 42*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 6*power3(mst12)) + 252*power2(DeltaInv(mH2,
     mst22,mst22))*power3(mst22) - 252*power2(DeltaInv(mst22,mst22,mh2))*
     power3(mst22) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) -
     1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power4(mst12) + 1008*mH2*
     power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) - 1008*mh2*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(3.*(mst12 - mst22)*power2(sb))
     + (ca*cb*mt2*mu2*sa*power2(Pi)*(mH2*mst24*power2(DeltaInv(mH2,mst22,mst22)
     ) - mh2*mst24*power2(DeltaInv(mst22,mst22,mh2)) + power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) - power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst14 + 6*power3(mst12)) + 6*power2(DeltaInv(mH2,mst22,mst22)
     )*power3(mst22) - 6*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 24*
     mh2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) - 24*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power4(mst12) + 24*mH2*power3(DeltaInv(mH2,
     mst22,mst22))*power4(mst22) - 24*mh2*power3(DeltaInv(mst22,mst22,mh2))*
     power4(mst22)))/(3.*(mst12 - mst22)*sb) - (mt2*mu2*power2(cb)*power2(Pi)*
     power2(sa)*(mH2*mst24*power2(DeltaInv(mH2,mst22,mst22)) - mh2*mst24*
     power2(DeltaInv(mst22,mst22,mh2)) + power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 6*power3(mst12)) - power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst14 + 6*power3(mst12)) + 6*power2(DeltaInv(mH2,mst22,mst22))*power3(
     mst22) - 6*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 24*mh2*
     power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) - 24*mH2*power3(DeltaInv(
     mH2,mst12,mst12))*power4(mst12) + 24*mH2*power3(DeltaInv(mH2,mst22,mst22))
     *power4(mst22) - 24*mh2*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)))/
     (3.*(mst12 - mst22)*power2(sb)) + (mt2*mu2*power2(cb)*power2(Pi)*(msb12*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) - msb12*
     power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 6*power3(mst22)) + 24*mh2*
     msb12*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) + power2(mw2)*(-((
     msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) +
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*
     (14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22))) - 24*mh2*msb12*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22)))/(3.*msb12*(mst12 - mst22)*power2(sb)) - (ca*cb*mu2*sa*log(mst22/
     Q2)*power2(snt)*(-((5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(mH2,mst22,
     mst12)) - mst12*(mH2 - 6*mst22)*DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*
     DeltaInv(mst22,mh2,mst12) - mst12*mst22*DeltaInv(mst22,mh2,mst12) + mst24*
     DeltaInv(mst22,mh2,mst12) + mh2*mst12*DeltaInv(mst22,mst22,mh2) - 6*mst12*
     mst22*DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*mst24*power2(DeltaInv(mst22,
     mh2,mst12)) + 18*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + 84*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 24*mh2*mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*power3(mh2) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mst22) - 72*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*
     power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 -
     mst12*mst24 + mst22*power2(mH2) + power3(mst22)) + 24*mst12*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 72*power2(mH2)*
     (mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 144*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 288*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) - 288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(
     3.*mst12*sb) - (mu2*log(mst22/Q2)*power2(sa)*power2(snt)*(-((5*mH2*mst22 -
     mst12*mst22 + mst24)*DeltaInv(mH2,mst22,mst12)) - mst12*(mH2 - 6*mst22)*
     DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*DeltaInv(mst22,mh2,mst12) - mst12*
     mst22*DeltaInv(mst22,mh2,mst12) + mst24*DeltaInv(mst22,mh2,mst12) + mh2*
     mst12*DeltaInv(mst22,mst22,mh2) - 6*mst12*mst22*DeltaInv(mst22,mst22,mh2)
     + 6*mh2*mst12*mst24*power2(DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 24*mh2*mst12*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2)
     - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power3(mst22) - 72*mst12*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*power2(DeltaInv(mH2,
     mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*
     power2(mH2) + power3(mst22)) + 24*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     (mH2*mst24 + 3*power3(mst22)) + 72*power2(mH2)*(mst24*power2(mH2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 144*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 288*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(6.*mst12)
     + (mu2*log(mst22/Q2)*power2(cb)*power2(sa)*power2(snt)*(-((5*mH2*mst22 -
     mst12*mst22 + mst24)*DeltaInv(mH2,mst22,mst12)) - mst12*(mH2 - 6*mst22)*
     DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*DeltaInv(mst22,mh2,mst12) - mst12*
     mst22*DeltaInv(mst22,mh2,mst12) + mst24*DeltaInv(mst22,mh2,mst12) + mh2*
     mst12*DeltaInv(mst22,mst22,mh2) - 6*mst12*mst22*DeltaInv(mst22,mst22,mh2)
     + 6*mh2*mst12*mst24*power2(DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 24*mh2*mst12*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2)
     - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power3(mst22) - 72*mst12*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*power2(DeltaInv(mH2,
     mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*
     power2(mH2) + power3(mst22)) + 24*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     (mH2*mst24 + 3*power3(mst22)) + 72*power2(mH2)*(mst24*power2(mH2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 144*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 288*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(6.*mst12*
     power2(sb)) + (log(mh2/Q2)*(18 + (3*mh2)/mst12 + (3*mh2)/mst22 + (8*mt2)/
     mh2 + (12*mt2)/mst12 + (12*mt2)/mst22 + 12*(mh2 - 6*mst12)*mt2*DeltaInv(
     mh2,mst12,mst12) - 4*mt2*(29*mh2 + 154*mt2)*DeltaInv(mh2,mt2,mt2) + 12*
     mh2*mt2*DeltaInv(mst22,mst22,mh2) - 72*mst22*mt2*DeltaInv(mst22,mst22,mh2)
     + 144*mh2*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) + 144*mt2*power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 1728*mt2*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) - 2592*mh2*power2(
     DeltaInv(mh2,mt2,mt2))*power3(mt2) + 6912*mh2*mt2*power3(DeltaInv(mh2,
     mst12,mst12))*power4(mst12) + 6912*mh2*mt2*power3(DeltaInv(mst22,mst22,
     mh2))*power4(mst22)))/72. + (log(mst22/Q2)*power2(sa)*(-((5*mH2*mst22 -
     mst12*mst22 + mst24)*mu2*DeltaInv(mH2,mst22,mst12)) - 4*mst12*(mH2 - 6*
     mst22)*mt2*DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*mu2*DeltaInv(mst22,mh2,
     mst12) - mst12*mst22*mu2*DeltaInv(mst22,mh2,mst12) + mst24*mu2*DeltaInv(
     mst22,mh2,mst12) + 4*mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) - 24*mst12*
     mst22*mt2*DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*mst24*mu2*power2(
     DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*mu2*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + 84*mst24*mu2*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 96*mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,
     mh2)) - 6*mst22*mu2*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 6*mh2*
     mu2*power2(DeltaInv(mst22,mh2,mst12))*power3(mst22) - 288*mst12*mt2*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*mu2*power2(
     DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*
     mst24 + mst22*power2(mH2) + power3(mst22)) + 96*mst12*mt2*power2(DeltaInv(
     mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 72*mu2*power2(mH2)*(
     mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*mu2*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*mu2*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mu2*power3(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 72*mst24*mu2*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 1152*mH2*mst12*mt2*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) - 72*mu2*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) - 1152*mh2*mst12*mt2*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(24.*mst12) + (mu2*power2(cb)*
     power2(mw2)*power2(snb)*power2(snt)*power2(log(mst22/Q2))*(-(msb22*mst24*
     power2(DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(mst22,mw2,
     msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*
     power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) +
     msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22))))/(msb12*
     msb22*power2(sb)) + (2*mt2*mu2*log(msb12/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(-
     5*mst24*power2(mw2) + mw2*power3(mst22) - msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/(
     msb12*(mst12 - mst22)*power2(sb)) - (2*mt2*mu2*log(msb12/Q2)*log(mw2/Q2)*
     power2(cb)*power2(mw2)*power2(snb)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*
     power3(mw2) + power4(mst22))))/(msb12*(mst12 - mst22)*power2(sb)) + (mt2*
     log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*(-((msb12*mst22 + mst24
     + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/msb12 + (2*mu2*log(msb12/Q2)*log(mw2/
     Q2)*power2(cb)*power2(mw2)*power2(snt)*(-((msb12*mst22 + mst24 + 3*mst22*
     mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12)
     )*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*
     power3(mw2) + power4(mst22))))/msb12 + ((mt2 + mu2)*log(msb12/Q2)*log(mw2/
     Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*(-((msb12*mst22 + mst24
     + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/msb12 + (mu2*log(msb12/Q2)*log(mw2/
     Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*(-((msb12*mst22 + mst24
     + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/(msb12*power2(sb)) - (2*mt2*mu2*log(
     msb12/Q2)*log(mw2/Q2)*power2(mw2)*power4(cb)*(-((msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/(msb12*(mst12 - mst22)*power2(sb)) +
     (2*mt2*mu2*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power4(cb)*(-
     ((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) +
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(-5*
     mst24*power2(mw2) + mw2*power3(mst22) - msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/(
     msb12*(mst12 - mst22)*power2(sb)) + (mu2*log(msb12/Q2)*log(mw2/Q2)*power2(
     mw2)*power2(snt)*power4(cb)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(
     mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) +
     power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) - msb12*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) +
     power4(mst22))))/(msb12*power2(sb)) + (log(msb12/Q2)*log(mw2/Q2)*power2(
     cb)*power2(mw2)*power2(snb)*(-((mst12 - mst22 - 2*mt2)*mu2*(msb12*mst22 +
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + mt2*(mst12 -
     mst22 - 2*mu2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + (mst12 - mst22 - 2*mt2)*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*
     power3(mw2) + power4(mst22))))/(msb12*(-mst12 + mst22)) + (2*mt2*mu2*log(
     msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*(-((msb22*mst22 +
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/(msb22*(mst12 - mst22)*power2(sb)) +
     (mt2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*(-((
     msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22))) +
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(-5*
     mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb22 + (2*mu2*log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(
     snb)*power2(snt)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) -
     mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(
     mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,
     mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*
     mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(
     mst22))))/msb22 - (2*mt2*mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     snb)*power4(cb)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) -
     mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(
     mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,
     mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*
     mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(
     mst22))))/(msb22*(mst12 - mst22)*power2(sb)) + (mu2*log(msb22/Q2)*log(mw2/
     Q2)*power2(mw2)*power2(snb)*power2(snt)*power4(cb)*(-((msb22*mst22 + mst24
     + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,
     msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/(msb22*power2(sb)) - (DeltaInv(mst22,
     mt2,mu2)*log(mst22/Q2)*log(mu2/Q2)*power2(mu2)*(mst22 + DeltaInv(mst22,
     mt2,mu2)*(mst22*mu2*(4*mt2 + mu2) + 2*mst24*(mt2 + 4*mu2) - power3(mst22))
     - 2*mu2*power2(DeltaInv(mst22,mt2,mu2))*(mst24*mu2*(-10*mt2 + mu2) +
     mst22*(-mt2 + mu2)*power2(mu2) - 5*(mt2 + mu2)*power3(mst22) + 3*power4(
     mst22))))/mt2 - (2*ca*cb*mt2*mu2*sa*power2(log(mH2/Q2))*(power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12)) + 16*mH2*power3(DeltaInv(
     mH2,mst12,mst12))*power4(mst12) - power2(DeltaInv(mH2,mst22,mst22))*(mH2*
     mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)))
     )/((mst12 - mst22)*sb) + (2*mt2*mu2*power2(cb)*power2(sa)*power2(log(mH2/
     Q2))*(power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12)) +
     16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power4(mst12) - power2(DeltaInv(
     mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,
     mst22,mst22)*power4(mst22))))/((mst12 - mst22)*power2(sb)) - (2*mu2*log(
     mH2/Q2)*log(mst22/Q2)*power2(snt)*((mH2*mst22 + mst12*mst22 + mst24)*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(mH2)*((mst12*mst22
     + mst24)*power2(mH2) - mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) +
     mH2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) +
     mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*
     mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22))))/mst12 + (4*ca*cb*mu2*sa*
     log(mH2/Q2)*log(mst22/Q2)*power2(snt)*((mH2*mst22 + mst12*mst22 + mst24)*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(mH2)*((mst12*mst22
     + mst24)*power2(mH2) - mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) +
     mH2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) +
     mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*
     mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22))))/(mst12*sb) + (2*mu2*log(
     mH2/Q2)*log(mst22/Q2)*power2(sa)*power2(snt)*((mH2*mst22 + mst12*mst22 +
     mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(mH2)*((
     mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (mst12 - mst22)*
     power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,
     mst22,mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*
     power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22))))/mst12 -
     (2*mu2*log(mH2/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)*power2(snt)*((mH2*
     mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))
     + power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (
     mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(
     mst22))))/(mst12*power2(sb)) + (ca*cb*mu2*sa*log(mH2/Q2)*log(mst22/Q2)*((
     mst12 - mst22)*(mH2*mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + (mst12 - mst22)*power2(mH2)*((mst12*mst22 +
     mst24)*power2(mH2) - mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) +
     mH2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) +
     4*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22)
     + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22))))/(mst12*(-mst12 +
     mst22)*sb) - (log(mH2/Q2)*log(mst22/Q2)*power2(sa)*((mH2*mst22 + mst12*
     mst22 + mst24)*mu2*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + mu2*
     power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (
     mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + 4*mst12*mt2*power2(DeltaInv(mH2,mst22,
     mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*
     power4(mst22))))/(2.*mst12) - (mu2*log(mH2/Q2)*log(mst22/Q2)*power2(cb)*
     power2(sa)*((mst12 - mst22)*(mH2*mst22 + mst12*mst22 + mst24)*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + (mst12 - mst22)*power2(mH2)*((mst12*
     mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (mst12 - mst22)*power3(
     mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,
     mst12)) + 8*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*
     power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22))))/(2.*
     mst12*(-mst12 + mst22)*power2(sb)) + (mt2*power2(log(mh2/Q2))*((mh2 + 3*
     mt2)*DeltaInv(mh2,mt2,mt2) + 12*mh2*power2(mt2)*power2(DeltaInv(mh2,mt2,
     mt2)) - 2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12))
     - 32*mh2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) - 2*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22))))/2. + (2*ca*cb*mt2*mu2*sa*power2(log(mh2/
     Q2))*(power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12)) +
     16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) - power2(DeltaInv(
     mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,
     mst22,mh2)*power4(mst22))))/((mst12 - mst22)*sb) + (2*mt2*mu2*power2(cb)*
     power2(log(mh2/Q2))*(power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*
     power3(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) -
     power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*
     DeltaInv(mst22,mst22,mh2)*power4(mst22))))/((mst12 - mst22)*power2(sb)) -
     (2*mt2*mu2*power2(cb)*power2(sa)*power2(log(mh2/Q2))*(power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 4*power3(mst12)) + 16*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power4(mst12) - power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24
     + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/((
     mst12 - mst22)*power2(sb)) - (4*ca*cb*mu2*sa*log(mh2/Q2)*log(mst22/Q2)*
     power2(snt)*((mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22 + mst24)*power2(
     mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22))))/(mst12*sb) - (2*mu2*log(mh2/Q2)*log(
     mst22/Q2)*power2(sa)*power2(snt)*((mh2*mst22 + mst12*mst22 + mst24)*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22
     + mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/mst12 - (2*mu2*log(mh2/Q2)*
     log(mst22/Q2)*power2(cb)*power2(snt)*((mh2*mst22 + mst12*mst22 + mst24)*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22
     + mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/(mst12*power2(sb)) + (2*
     mu2*log(mh2/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)*power2(snt)*((mh2*
     mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (
     mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(DeltaInv(mst22,mst22,mh2)
     )*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(
     mst22))))/(mst12*power2(sb)) - (ca*cb*mu2*sa*log(mh2/Q2)*log(mst22/Q2)*((
     mst12 - mst22)*(mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + (mst12 - mst22)*power2(mh2)*((mst12*mst22 +
     mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     4*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22)
     + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/(mst12*(-mst12 +
     mst22)*sb) + (log(mh2/Q2)*log(mst22/Q2)*power2(sa)*((mh2*mst22 + mst12*
     mst22 + mst24)*mu2*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + mu2*
     power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (
     mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)) + 4*mst12*mt2*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22))))/(2.*mst12) - (mu2*log(mh2/Q2)*log(mst22/Q2)*power2(
     cb)*((mst12 - mst22)*(mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + (mst12 - mst22)*power2(mh2)*((mst12*mst22 +
     mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     8*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22)
     + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/(2.*mst12*(-mst12 +
     mst22)*power2(sb)) + (mu2*log(mh2/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)*
     ((mst12 - mst22)*(mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + (mst12 - mst22)*power2(mh2)*((mst12*mst22 +
     mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     8*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22)
     + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/(2.*mst12*(-mst12 +
     mst22)*power2(sb)) - (ca*cb*mt2*mu2*sa*log(mst22/Q2)*(-((mH2 - 6*mst22)*
     DeltaInv(mH2,mst22,mst22)) + 24*power2(DeltaInv(mH2,mst22,mst22))*(mH2*
     mst24 + 3*power3(mst22)) + 288*mH2*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) + DeltaInv(mst22,mst22,mh2)*(mh2 - 6*mst22 - 24*DeltaInv(
     mst22,mst22,mh2)*(mh2*mst24 + 3*power3(mst22)) - 288*mh2*power2(DeltaInv(
     mst22,mst22,mh2))*power4(mst22))))/(3.*(mst12 - mst22)*sb) + (mt2*mu2*log(
     mst22/Q2)*power2(cb)*power2(sa)*(-((mH2 - 6*mst22)*DeltaInv(mH2,mst22,
     mst22)) + 24*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(
     mst22)) + 288*mH2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) +
     DeltaInv(mst22,mst22,mh2)*(mh2 - 6*mst22 - 24*DeltaInv(mst22,mst22,mh2)*(
     mh2*mst24 + 3*power3(mst22)) - 288*mh2*power2(DeltaInv(mst22,mst22,mh2))*
     power4(mst22))))/(3.*(mst12 - mst22)*power2(sb)) + (2*mt2*mu2*power2(cb)*
     power2(log(mst22/Q2))*(mh2*msb12*mst24*power2(DeltaInv(mst22,mst22,mh2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-mst24 + DeltaInv(mst22,
     mw2,msb12)*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 +
     power3(mst22)) + power4(mst22)))))/(msb12*(mst12 - mst22)*power2(sb)) + (
     2*mu2*power2(cb)*power2(snt)*power2(log(mst22/Q2))*(mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(-mst24 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 2*mw2*
     power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/
     msb12 + (mu2*power2(snt)*power2(log(mst22/Q2))*power4(cb)*(mst24*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) - power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(-mst24 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 2*mw2*
     power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(
     msb12*power2(sb)) + (mt2*mu2*log(mst22/Q2)*power4(cb)*((5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - 6*mC2*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) - 72*power2(mC2)*(mst24*power2(mC2) + (
     -2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - DeltaInv(mst22,mw2,msb12)*(-(msb12*
     mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb12)*(mst22*mw2*
     (-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(6.*
     msb12*(mst12 - mst22)*power2(sb)) + (mu2*log(mst22/Q2)*power2(cb)*power2(
     snt)*(-((5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) +
     6*mC2*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24
     - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*(
     mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + DeltaInv(mst22,mw2,
     msb12)*(-(msb12*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,
     msb12)*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(6.*msb12) - ((mt2 + mu2)*log(mst22/Q2)*power2(cb)*power2(snt)*
     (-((5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + 6*mC2*
     power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 -
     msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + DeltaInv(mst22,mw2,
     msb12)*(-(msb12*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,
     msb12)*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(12.*msb12) + (mu2*log(mst22/Q2)*power2(snt)*power4(cb)*(-((5*
     mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + 6*mC2*
     power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 -
     msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + DeltaInv(mst22,mw2,
     msb12)*(-(msb12*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,
     msb12)*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(12.*msb12*power2(sb)) + (mt2*mu2*log(mst22/Q2)*power2(cb)*(-2*
     msb12*(mh2 - 6*mst22)*DeltaInv(mst22,mst22,mh2) + 48*msb12*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 3*power3(mst22)) + 576*mh2*msb12*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + DeltaInv(mst22,mw2,
     msb12)*(-(msb12*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,
     msb12)*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(6.*msb12*(mst12 - mst22)*power2(sb)) + (2*(mst12 - mst22 -
     mt2)*mu2*power2(cb)*power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - msb22*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + msb12*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-(msb22*mst24*power2(
     DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*
     power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22)
     - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(msb12*msb22*(
     mst12 - mst22)) + ((mt2 + mu2)*power2(cb)*power2(snb)*power2(snt)*power2(
     log(mst22/Q2))*(msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     - msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb22*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + msb12*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(
     mw2)*(-(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12))) + msb12*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(msb12*msb22) + ((mst12 - mst22 - 2*mt2)*mu2*power2(snb)*
     power2(log(mst22/Q2))*power4(cb)*(msb22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2)) - msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2)) + msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(
     mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,
     mC2)) + power2(mw2)*(-(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12))) +
     msb12*power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)
     *(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 +
     power3(mst22)) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))
     *(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(
     mst22)) + power4(mst22)))))/(msb12*msb22*(mst12 - mst22)*power2(sb)) + (2*
     mt2*mu2*power2(snb)*power2(log(mst22/Q2))*power4(cb)*(msb22*mst24*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - msb22*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + msb12*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-(msb22*mst24*power2(
     DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*
     power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22)
     - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(msb12*msb22*(
     mst12 - mst22)*power2(sb)) + ((mst12 - mst22 - 2*mt2)*mu2*log(mst22/Q2)*
     power2(cb)*power2(snb)*(msb22*(-(msb12*mst22) + mst24 + 5*mst22*mw2)*
     DeltaInv(mst22,mw2,msb12) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     (mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) -
     72*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2)
     - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(
     mst22)) - msb12*DeltaInv(mst22,mw2,msb22)*(-(msb22*mst22) + mst24 + 5*
     mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2) -
     mst24*(msb22 + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(
     mst24*mw2 + power3(mst22)) + power4(mst22)))))/(12.*msb12*msb22*(-mst12 +
     mst22)*power2(sb)) - (mt2*mu2*log(mst22/Q2)*power2(cb)*power2(snb)*(msb22*
     (-(msb12*mst22) + mst24 + 5*mst22*mw2)*DeltaInv(mst22,mw2,msb12) - 6*
     msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(mst22*mw2*(-3*msb12 + mw2) -
     mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*
     msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)) - msb12*DeltaInv(mst22,
     mw2,msb22)*(-(msb22*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,
     mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2) - mst24*(msb22 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(6.*msb12*msb22*(mst12 - mst22)*power2(sb)) + (mu2*log(mst22/
     Q2)*power2(cb)*power2(snb)*power2(snt)*(msb22*(-(msb12*mst22) + mst24 + 5*
     mst22*mw2)*DeltaInv(mst22,mw2,msb12) - 6*msb22*mw2*power2(DeltaInv(mst22,
     mw2,msb12))*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(
     mst22)) - 72*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mst24*
     power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) +
     power4(mst22)) - msb12*DeltaInv(mst22,mw2,msb22)*(-(msb22*mst22) + mst24 +
     5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2)
     - mst24*(msb22 + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*
     msb22*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(12.*msb12*msb22*
     power2(sb)) + (2*mu2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(snt)*((
     mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + power2(mC2)*((msb12*mst22 - mst24)*power2(mC2) - mst22*
     power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(-(msb12*mst22) - 3*mst24 - mst22*mw2 +
     DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 5*mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(
     mw2) + 3*power4(mst22)))))/msb12 + (2*mt2*mu2*log(msb12/Q2)*log(mst22/Q2)*
     power2(snb)*power4(cb)*((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((msb12*mst22 - mst24)*
     power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-(msb12*mst22) - 3*mst24 -
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 5*mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) +
     mst22*power3(mw2) + 3*power4(mst22)))))/(msb12*(mst12 - mst22)*power2(sb))
     + (mu2*log(msb12/Q2)*log(mst22/Q2)*power2(snt)*power4(cb)*((mC2*mst22 +
     msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     power2(mC2)*((msb12*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*
     msb12 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(-(msb12*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb12)*
     (mst24*power2(mw2) - 5*mw2*power3(mst22) - msb12*(10*mst24*mw2 + mst22*
     power2(mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/(
     msb12*power2(sb)) + (log(msb12/Q2)*log(mst22/Q2)*power2(snt)*((mC2*mst22 +
     msb12*mst22 + 3*mst24)*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     mu2*power2(mC2)*((msb12*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (
     5*msb12 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12))*(-(msb12*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,
     msb12)*(mst24*power2(mw2) - 5*mw2*power3(mst22) - msb12*(10*mst24*mw2 +
     mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)
     ))))/msb12 + (2*mu2*log(msb22/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*
     power2(snt)*((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((msb22*mst22 - mst24)*power2(
     mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*
     msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-(msb22*mst22) - 3*mst24 - mst22*
     mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) - 5*mw2*power3(mst22) -
     msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(
     mw2) + 3*power4(mst22)))))/msb22 + (mu2*log(msb22/Q2)*log(mst22/Q2)*
     power2(snb)*power2(snt)*power4(cb)*((mC2*mst22 + msb22*mst22 + 3*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((msb22*mst22
     - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(
     mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-(msb22*
     mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) - 5*mw2*power3(mst22) - msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*
     power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/(msb22*power2(sb)
     ) + (log(msb22/Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*((mC2*mst22 +
     msb22*mst22 + 3*mst24)*mu2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) +
     mu2*power2(mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (
     5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22))*(-(msb22*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,
     msb22)*(mst24*power2(mw2) - 5*mw2*power3(mst22) - msb22*(10*mst24*mw2 +
     mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)
     ))))/msb22 + (mu2*log(mst12/Q2)*log(mst22/Q2)*power2(cb)*((mA2*mst22 +
     mst12*mst22 + 3*mst24)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) +
     power2(mA2)*((mst12*mst22 - mst24)*power2(mA2) - mst22*power3(mA2) + (5*
     mst12 - 3*mst22)*power3(mst22) + 5*mA2*(2*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mA2,mst12)) + power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12))*(-(mst12*mst22) - 3*mst24 - mst22*mz2 + DeltaInv(mst22,mz2,mst12)*
     (mst24*power2(mz2) - 5*mz2*power3(mst22) - mst12*(10*mst24*mz2 + mst22*
     power2(mz2) + 5*power3(mst22)) + mst22*power3(mz2) + 3*power4(mst22)))))/(
     2.*mst12) + (power2(snt)*power2(log(mst22/Q2))*(2*msb12*mst24*mu2*power2(
     mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 2*msb12*mu2*power2(mH2)*(mst24*
     power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mst12*(mH2*msb12*
     mst24*mu2*power2(DeltaInv(mH2,mst22,mst22)) + mst24*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - mu2*power2(mC2)*(mst24*power2(mC2) + (
     -2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12))*(-mst24 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 2*
     mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22))))
     ))/(msb12*mst12) + (mu2*log(mst22/Q2)*power2(cb)*power2(snt)*(-2*msb12*(5*
     mh2*mst22 - mst12*mst22 + mst24)*DeltaInv(mst22,mh2,mst12) + 12*mh2*msb12*
     power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst12*mst22 - 14*mh2*mst24 -
     mst12*mst24 + mst22*power2(mh2) + power3(mst22)) + 144*msb12*power2(mh2)*(
     mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*(-2*msb12*(mh2
     - 6*mst22)*DeltaInv(mst22,mst22,mh2) + 48*msb12*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 3*power3(mst22)) + 576*mh2*msb12*power3(DeltaInv(
     mst22,mst22,mh2))*power4(mst22) - DeltaInv(mst22,mw2,msb12)*(-(msb12*
     mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb12)*(mst22*mw2*
     (-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22))))))/(12.*
     msb12*mst12*power2(sb)) - 84*power2(DeltaInv(mt2,mt2,mz2))*power4(mt2) -
     mt2*DeltaInv(mt2,mt2,mA2)*log(mA2/Q2)*log(mt2/Q2)*power2(cb)*(mA2 + 5*mt2
     + 8*(3*mA2 + 2*mt2)*DeltaInv(mt2,mt2,mA2)*power2(mt2) + 64*mA2*power2(
     DeltaInv(mt2,mt2,mA2))*power4(mt2)) - mt2*DeltaInv(mt2,mt2,mz2)*log(mt2/
     Q2)*log(mz2/Q2)*(5*mt2 + mz2 + 8*(2*mt2 + 3*mz2)*DeltaInv(mt2,mt2,mz2)*
     power2(mt2) + 64*mz2*power2(DeltaInv(mt2,mt2,mz2))*power4(mt2)) + mt2*
     DeltaInv(mt2,mt2,mz2)*log(mt2/Q2)*log(mz2/Q2)*power2(cb)*(5*mt2 + mz2 + 8*
     (2*mt2 + 3*mz2)*DeltaInv(mt2,mt2,mz2)*power2(mt2) + 64*mz2*power2(
     DeltaInv(mt2,mt2,mz2))*power4(mt2)) - (mt2*DeltaInv(mt2,mt2,mz2)*power2(
     log(mz2/Q2))*(5*mt2 + mz2 + 8*(2*mt2 + 3*mz2)*DeltaInv(mt2,mt2,mz2)*
     power2(mt2) + 64*mz2*power2(DeltaInv(mt2,mt2,mz2))*power4(mt2)))/2. + (
     mt2*log(mt2/Q2)*power2(cb)*(2*(mC2 - mw2) + (mC2 - mt2)*(13*mA2 + 48*mt2)*
     (mt2 - mw2)*DeltaInv(mt2,mt2,mA2) - (mC2 - mt2)*(mt2 - mw2)*(48*mt2 + 13*
     mz2)*DeltaInv(mt2,mt2,mz2) + 12*(mC2 - mt2)*(19*mA2 + 12*mt2)*(mt2 - mw2)*
     power2(mt2)*power2(DeltaInv(mt2,mt2,mA2)) - 12*(mC2 - mt2)*(mt2 - mw2)*(
     12*mt2 + 19*mz2)*power2(mt2)*power2(DeltaInv(mt2,mt2,mz2)) + 576*mA2*(mC2
     - mt2)*(mt2 - mw2)*power3(DeltaInv(mt2,mt2,mA2))*power4(mt2) - 576*(mC2 -
     mt2)*(mt2 - mw2)*mz2*power3(DeltaInv(mt2,mt2,mz2))*power4(mt2)))/(6.*(mC2
     - mt2)*(mt2 - mw2)) + (49*msb12*power2(DeltaInv(msb12,mt2,mu2))*power4(
     mu2))/mt2 + (49*mst22*power2(DeltaInv(mst22,mt2,mu2))*power4(mu2))/mt2 + (
     49*mst12*power2(DeltaInv(mt2,mu2,mst12))*power4(mu2))/mt2 + 350*msb14*
     power3(DeltaInv(msb12,mt2,mu2))*power4(mu2) + (126*power3(msb12)*power3(
     DeltaInv(msb12,mt2,mu2))*power4(mu2))/mt2 + 350*mst24*power3(DeltaInv(
     mst22,mt2,mu2))*power4(mu2) + (126*power3(mst22)*power3(DeltaInv(mst22,
     mt2,mu2))*power4(mu2))/mt2 + 350*mst14*power3(DeltaInv(mt2,mu2,mst12))*
     power4(mu2) + (126*power3(mst12)*power3(DeltaInv(mt2,mu2,mst12))*power4(
     mu2))/mt2 - 35*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - (
     21*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2))/msb12 + (5*
     power4(mw2))/(108.*power4(mt2 - mw2)) + (log(mst22/Q2)*power2(snt)*(8 -
     mst22/msb12 - (8*mst22)/mst12 + (2*mst22)/(mst22 - mu2) - (2*(5*mH2*mst22
     - mst12*mst22 + mst24)*mu2*DeltaInv(mH2,mst22,mst12))/mst12 - 2*(mH2 - 6*
     mst22)*mu2*DeltaInv(mH2,mst22,mst22) + mst22*mu2*DeltaInv(mst22,msb12,mC2)
     - (5*mC2*mst22*mu2*DeltaInv(mst22,msb12,mC2))/msb12 - (mst24*mu2*DeltaInv(
     mst22,msb12,mC2))/msb12 - mst22*mt2*DeltaInv(mst22,mw2,msb12) + (mst24*
     mt2*DeltaInv(mst22,mw2,msb12))/msb12 + (5*mst22*mt2*mw2*DeltaInv(mst22,
     mw2,msb12))/msb12 - 6*mC2*mst24*mu2*power2(DeltaInv(mst22,msb12,mC2)) -
     18*mst22*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - (84*mst24*
     mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)))/msb12 + 6*mst24*mt2*
     mw2*power2(DeltaInv(mst22,mw2,msb12)) + 18*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + (84*mst24*mt2*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)))/msb12 + (6*mst22*mu2*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2))/msb12 + (6*mC2*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(
     mst22))/msb12 - (6*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22)
     )/msb12 + (12*mH2*mu2*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*
     mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*power2(mH2) + power3(mst22)))/
     mst12 + 48*mu2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(
     mst22)) - (6*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/
     msb12 + (144*mu2*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,
     mst22,mst12)))/mst12 - 144*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 144*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) - (144*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)))/msb12 + 144*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12)) + 144*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + (144*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)))/msb12 + (72*mst24*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2))/msb12 + 576*mH2*mu2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22)
     + (72*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22))/
     msb12 - (72*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22))/msb12 - (72*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2))/msb12))/12. + (mu2*log(mst22/Q2)*power2(cb)*power2(snb)*power2(snt)*
     (msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) -
     msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) +
     msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) - msb22*mst24*DeltaInv(mst22,
     mw2,msb12) - 5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*
     mst22*DeltaInv(mst22,mw2,msb22) + msb12*mst24*DeltaInv(mst22,mw2,msb22) +
     5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) - 6*msb12*msb22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) - 18*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 84*msb22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb22)) + 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 84*msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) + 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*
     msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) - 6*mC2*msb22*
     power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 -
     msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + 6*mC2*msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*
     mst24 + mst22*power2(mC2) + power3(mst22)) + 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) - 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(6.*msb12*msb22) + (mu2*log(
     mst22/Q2)*power2(snb)*power2(snt)*power4(cb)*(msb22*(5*mC2*mst22 - msb12*
     mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*mst22 - msb22*
     mst22 + mst24)*DeltaInv(mst22,msb22,mC2) + msb12*msb22*mst22*DeltaInv(
     mst22,mw2,msb12) - msb22*mst24*DeltaInv(mst22,mw2,msb12) - 5*msb22*mst22*
     mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*DeltaInv(mst22,mw2,
     msb22) + msb12*mst24*DeltaInv(mst22,mw2,msb22) + 5*msb12*mst22*mw2*
     DeltaInv(mst22,mw2,msb22) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) - 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 84*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 18*msb12*
     msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 84*msb12*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*msb12*mw2*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mst22) - 6*mC2*msb22*power2(DeltaInv(mst22,msb12,
     mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2)
     + power3(mst22)) + 6*mC2*msb12*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*
     msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(
     mst22)) + 6*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*
     msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 72*msb22*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 72*
     msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) -
     144*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)
     ) - 144*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     144*msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     144*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)
     ) + 144*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) +
     144*msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) +
     72*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 72*
     msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 72*
     msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 72*msb12*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(12.*msb12*msb22*
     power2(sb)) + ((mst12 - mst22 - 2*mt2)*mu2*log(mst22/Q2)*power2(cb)*
     power2(snb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,
     msb12,mC2)) + msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,
     msb22,mC2) - msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) + msb22*mst24*
     DeltaInv(mst22,mw2,msb12) + 5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) +
     msb12*msb22*mst22*DeltaInv(mst22,mw2,msb22) - msb12*mst24*DeltaInv(mst22,
     mw2,msb22) - 5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) + 6*msb12*msb22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 18*msb12*msb22*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 84*msb22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*mst24*mw2*power2(
     DeltaInv(mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(
     mst22) + 6*msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*
     mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*
     mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*mC2*msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 -
     msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(12.*msb12*msb22*(-mst12 + mst22)
     ) + ((mst12 - mst22 - mt2)*mu2*log(mst22/Q2)*power2(cb)*power2(snb)*(-(
     msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) +
     msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) -
     msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) + msb22*mst24*DeltaInv(mst22,
     mw2,msb12) + 5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*
     mst22*DeltaInv(mst22,mw2,msb22) - msb12*mst24*DeltaInv(mst22,mw2,msb22) -
     5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) + 6*msb12*msb22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) + 18*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 84*msb22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*
     msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*msb22*
     power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 -
     msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*mC2*msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*
     mst24 + mst22*power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22))
     + ((mt2 + mu2)*log(mst22/Q2)*power2(cb)*power2(snb)*power2(snt)*(-(msb22*(
     5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + msb12*(5*
     mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) - msb12*msb22*
     mst22*DeltaInv(mst22,mw2,msb12) + msb22*mst24*DeltaInv(mst22,mw2,msb12) +
     5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*mst22*DeltaInv(
     mst22,mw2,msb22) - msb12*mst24*DeltaInv(mst22,mw2,msb22) - 5*msb12*mst22*
     mw2*DeltaInv(mst22,mw2,msb22) + 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) + 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 84*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) - 18*
     msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mw2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*msb12*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*msb22*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) - 6*mC2*msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)
     *power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2)))/(12.*msb12*msb22) + ((mst12 - mst22 - 2*mt2)*mu2*
     log(mst22/Q2)*power2(snb)*power4(cb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 +
     mst24)*DeltaInv(mst22,msb12,mC2)) + msb12*(5*mC2*mst22 - msb22*mst22 +
     mst24)*DeltaInv(mst22,msb22,mC2) - msb12*msb22*mst22*DeltaInv(mst22,mw2,
     msb12) + msb22*mst24*DeltaInv(mst22,mw2,msb12) + 5*msb22*mst22*mw2*
     DeltaInv(mst22,mw2,msb12) + msb12*msb22*mst22*DeltaInv(mst22,mw2,msb22) -
     msb12*mst24*DeltaInv(mst22,mw2,msb22) - 5*msb12*mst22*mw2*DeltaInv(mst22,
     mw2,msb22) + 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) +
     18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 84*
     msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mst22) + 6*msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mst22) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*
     msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) - 6*mC2*msb12*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*
     mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) -
     6*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)
     *(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     144*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) -
     144*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*
     msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*
     msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*
     msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(12.*msb12*msb22*(
     mst12 - mst22)*power2(sb)) + (mt2*mu2*log(mst22/Q2)*power2(snb)*power4(cb)
     *(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) +
     msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) -
     msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) + msb22*mst24*DeltaInv(mst22,
     mw2,msb12) + 5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*
     mst22*DeltaInv(mst22,mw2,msb22) - msb12*mst24*DeltaInv(mst22,mw2,msb22) -
     5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) + 6*msb12*msb22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) + 18*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 84*msb22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*
     msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*msb22*
     power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 -
     msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*mC2*msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*
     mst24 + mst22*power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22)*
     power2(sb)) + (log(mst22/Q2)*power2(snb)*power2(snt)*(-(msb12*mst22) +
     msb22*mst22 + msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*mu2*DeltaInv(
     mst22,msb12,mC2) - msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*mu2*DeltaInv(
     mst22,msb22,mC2) + msb12*msb22*mst22*mt2*DeltaInv(mst22,mw2,msb12) -
     msb22*mst24*mt2*DeltaInv(mst22,mw2,msb12) - 5*msb22*mst22*mt2*mw2*
     DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*mt2*DeltaInv(mst22,mw2,
     msb22) + msb12*mst24*mt2*DeltaInv(mst22,mw2,msb22) + 5*msb12*mst22*mt2*
     mw2*DeltaInv(mst22,mw2,msb22) - 6*msb12*msb22*mst24*mt2*mw2*power2(
     DeltaInv(mst22,mw2,msb12)) - 18*msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 84*msb22*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*msb22*mst24*mt2*mw2*power2(DeltaInv(
     mst22,mw2,msb22)) + 18*msb12*msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 84*msb12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) + 6*msb22*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(
     mst22) - 6*msb12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) -
     6*mC2*msb22*mu2*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 -
     14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + 6*mC2*
     msb12*mu2*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*
     mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) + 6*msb22*mst22*
     mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 72*msb22*mu2*power2(mC2)*(
     mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 72*msb12*mu2*power2(
     mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*
     msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) -
     144*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     144*msb22*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     + 144*msb12*msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22)) + 144*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) + 144*msb12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22)) + 72*msb22*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 72*msb12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))
     *power4(mst22) + 72*msb22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 72*msb12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2)))/(12.*msb12*msb22) - (log(mst12/Q2)*power2(snt)*(-8 - mst12/msb12 +
     (8*mst12)/mst22 + (2*mst12)/(mst12 - mu2) + (2*mu2)/mst12 + (2*mu2)/mst22
     + (2*mH2*mu2)/(mst12*mst22) + 2*(mH2 - 6*mst12)*mu2*DeltaInv(mH2,mst12,
     mst12) + mst12*mu2*DeltaInv(msb12,mst12,mC2) - (5*mC2*mst12*mu2*DeltaInv(
     msb12,mst12,mC2))/msb12 - (mst14*mu2*DeltaInv(msb12,mst12,mC2))/msb12 -
     mst12*mt2*DeltaInv(mw2,msb12,mst12) + (mst14*mt2*DeltaInv(mw2,msb12,mst12)
     )/msb12 + (5*mst12*mt2*mw2*DeltaInv(mw2,msb12,mst12))/msb12 - 6*mC2*mst14*
     mu2*power2(DeltaInv(msb12,mst12,mC2)) - 18*mst12*mu2*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - (84*mst14*mu2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)))/msb12 + 6*mst14*mt2*mw2*power2(DeltaInv(mw2,msb12,
     mst12)) + 18*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + (
     84*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/msb12 + (6*
     mst12*mu2*power2(DeltaInv(msb12,mst12,mC2))*power3(mC2))/msb12 - (2*mu2*
     DeltaInv(mH2,mst22,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*
     mst22)*power2(mH2) + 5*mH2*power2(mst22) + power3(mH2)))/(mst12*mst22) + (
     6*mC2*mu2*power2(DeltaInv(msb12,mst12,mC2))*power3(mst12))/msb12 - (6*mt2*
     mw2*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12))/msb12 - 48*mu2*
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(mst12)) + (12*mH2*
     mu2*power2(DeltaInv(mH2,mst22,mst12))*(-(mst12*mst24) + mH2*(6*mst12*mst22
     + 23*mst24) - (mst12 - 23*mst22)*power2(mH2) + power3(mH2) + power3(mst22)
     ))/mst12 - (6*mst12*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2))/
     msb12 + (144*mu2*power2(mH2)*((3*mst12*mst22 + 2*mst24)*power2(mH2) - 2*
     mst22*power3(mH2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mH2*(5*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)))/mst12 - 144*
     mst14*mu2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) - 144*mu2*power2(
     mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) - (144*mu2*power3(
     mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)))/msb12 + 144*mt2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 144*mst14*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + (144*mt2*power3(mst12)
     *power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/msb12 + (72*mst14*mu2*
     power3(DeltaInv(msb12,mst12,mC2))*power4(mC2))/msb12 - 576*mH2*mu2*power3(
     DeltaInv(mH2,mst12,mst12))*power4(mst12) + (72*mu2*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*power4(mst12))/msb12 - (72*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12))/msb12 - (72*mst14*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2))/msb12))/12. + (mt2*mu2*mw2*
     power2(cb)*power2(snb)*(-(msb12*msb22*DeltaInv(mst22,mw2,msb12)) + msb12*
     msb22*DeltaInv(mst22,mw2,msb22) + msb12*msb22*DeltaInv(mw2,msb12,mst12) -
     msb12*msb22*DeltaInv(mw2,msb22,mst12) + 42*msb22*mw2*(msb12*mst22 + 3*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*mw2*(
     msb22*mst22 + 3*mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     42*msb12*msb22*mst12*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*mst12*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) + 126*msb12*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) - 588*msb12*msb22*mst14*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 210*msb12*msb22*mw2*power3(mst12)*power3(DeltaInv(mw2,msb12,
     mst12)) - 126*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,
     mst12)) - 210*msb12*msb22*mst12*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 126*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     588*msb12*msb22*mst14*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 210*
     msb12*msb22*mw2*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 210*
     msb12*msb22*mst12*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mw2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*mw2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb22*mw2*power3(DeltaInv(
     mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*
     mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*
     power3(mw2) - 3*power4(mst22)) - 42*msb12*mw2*power3(DeltaInv(mst22,mw2,
     msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(
     mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) -
     3*power4(mst22)) + 126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 126*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2)))/(3.*msb12*msb22*(mst12 - mst22)*power2(sb)) - (mu2*mw2*power2(cb)*
     power2(snb)*power2(snt)*(-(msb12*msb22*DeltaInv(mst22,mw2,msb12)) + msb12*
     msb22*DeltaInv(mst22,mw2,msb22) + msb12*msb22*DeltaInv(mw2,msb12,mst12) -
     msb12*msb22*DeltaInv(mw2,msb22,mst12) + 42*msb22*mw2*(msb12*mst22 + 3*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*mw2*(
     msb22*mst22 + 3*mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     42*msb12*msb22*mst12*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*mst12*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) + 126*msb12*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) - 588*msb12*msb22*mst14*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 210*msb12*msb22*mw2*power3(mst12)*power3(DeltaInv(mw2,msb12,
     mst12)) - 126*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,
     mst12)) - 210*msb12*msb22*mst12*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 126*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     588*msb12*msb22*mst14*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 210*
     msb12*msb22*mw2*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 210*
     msb12*msb22*mst12*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mw2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*mw2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb22*mw2*power3(DeltaInv(
     mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*
     mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*
     power3(mw2) - 3*power4(mst22)) - 42*msb12*mw2*power3(DeltaInv(mst22,mw2,
     msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(
     mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) -
     3*power4(mst22)) + 126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 126*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2)))/(6.*msb12*msb22*power2(sb)) + ((mt2 + mu2)*log(mst12/Q2)*power2(cb)
     *power2(snb)*power2(snt)*(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12 - msb22*mst12 + mst14)*
     DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*DeltaInv(mw2,msb12,mst12) -
     msb22*mst14*DeltaInv(mw2,msb12,mst12) - 5*msb22*mst12*mw2*DeltaInv(mw2,
     msb12,mst12) - msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) + msb12*mst14*
     DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*mw2*DeltaInv(mw2,msb22,mst12) -
     6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 18*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb22*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst14*
     mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 84*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))
     *power3(mst12) - 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mst12) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12
     - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + 6*mC2*
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14
     - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 6*msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 6*msb12*mst12*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) - 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + 72*msb12*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 144*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*msb22*
     mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 72*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb12*mst14*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(12.*msb12*msb22) + (mt2*mu2*log(
     mst12/Q2)*power2(snb)*power4(cb)*(msb22*(5*mC2*mst12 - msb12*mst12 +
     mst14)*DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*DeltaInv(mw2,msb12,
     mst12) - msb22*mst14*DeltaInv(mw2,msb12,mst12) - 5*msb22*mst12*mw2*
     DeltaInv(mw2,msb12,mst12) - msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) +
     msb12*mst14*DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*mw2*DeltaInv(mw2,
     msb22,mst12) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) -
     18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 84*msb12*mst14*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 6*msb22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) - 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) + 6*mC2*msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) +
     6*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 6*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*msb22*power2(mC2)
     *(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*msb12*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 72*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb12*
     mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(6.*msb12*msb22*(
     mst12 - mst22)*power2(sb)) + (mt2*(mst12 - mst22 - 2*mu2)*log(mst12/Q2)*
     power2(cb)*power2(snb)*(-(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*mst12 + mst14)*
     DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*DeltaInv(mw2,msb12,mst12) +
     msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mw2*DeltaInv(mw2,
     msb12,mst12) + msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) - msb12*mst14*
     DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mw2*DeltaInv(mw2,msb22,mst12) +
     6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*mst14*
     mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))
     *power3(mst12) + 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mst12) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12
     - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) - 6*mC2*
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14
     - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*
     mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(12.*msb12*msb22*(mst12 - mst22))
     + (mt2*mu2*log(mst12/Q2)*power2(cb)*power2(snb)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 -
     msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*
     DeltaInv(mw2,msb12,mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*
     msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22)) + (mu2*log(mst12/
     Q2)*power2(cb)*power2(snb)*power2(snt)*(-(msb22*(5*mC2*mst12 - msb12*mst12
     + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*DeltaInv(mw2,msb12,
     mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) -
     msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mw2*DeltaInv(mw2,
     msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) +
     18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst14*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) + 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) -
     6*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*msb22*power2(mC2)
     *(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) -
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*
     mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(6.*msb12*msb22) + (
     mt2*mu2*log(mst12/Q2)*power2(snb)*power4(cb)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 -
     msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*
     DeltaInv(mw2,msb12,mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*
     msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22)*power2(sb)) + (mu2*
     log(mst12/Q2)*power2(snb)*power2(snt)*power4(cb)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 -
     msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*
     DeltaInv(mw2,msb12,mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*
     msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(12.*msb12*msb22*power2(sb)) - (log(mst12/Q2)*
     power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb22*(5*mC2*mst12
     - msb12*mst12 + mst14)*mu2*DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12
     - msb22*mst12 + mst14)*mu2*DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*
     mt2*DeltaInv(mw2,msb12,mst12) - msb22*mst14*mt2*DeltaInv(mw2,msb12,mst12)
     - 5*msb22*mst12*mt2*mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*mst12*mt2*
     DeltaInv(mw2,msb22,mst12) + msb12*mst14*mt2*DeltaInv(mw2,msb22,mst12) + 5*
     msb12*mst12*mt2*mw2*DeltaInv(mw2,msb22,mst12) - 6*msb12*msb22*mst14*mt2*
     mw2*power2(DeltaInv(mw2,msb12,mst12)) - 18*msb12*msb22*mst12*mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb22*mst14*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst14*mt2*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 84*msb12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 6*msb22*mt2*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) - 6*msb12*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) - 6*mC2*msb22*mu2*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) + 6*mC2*msb12*mu2*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) +
     6*msb22*mst12*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 6*msb12*
     mst12*mt2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*msb22*mu2*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*
     msb12*mu2*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(
     mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,
     mC2)) - 144*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)) - 144*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 144*msb22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) + 144*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(
     DeltaInv(mw2,msb22,mst12)) + 144*msb12*msb22*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 144*msb12*mt2*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) + 72*msb22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*msb12*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*msb22*mst14*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb12*mst14*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(12.*msb12*msb22) - (mu2*power2(
     cb)*power2(log(mst22/Q2))*((mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)))/mst12 + (4*mh2*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)))/(
     mst12 - mst22) + (2*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/
     msb12 - (4*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(
     msb12*mst12 - msb12*mst22) + (mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)))/mst12 - (power2(mh2)*(mst24*power2(mh2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)))/mst12 + 4*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12)) - (8*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12)))/(mst12 - mst22) + 4*mst24*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) - (8*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)))/(mst12 - mst22) + (4*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)))/msb12 - (8*mt2*power3(mst22)*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)))/(msb12*mst12 - msb12*mst22) + 2*power2(mz2)*
     power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 2*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) + (2*power3(mst22)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)))/mst12 - (2*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22))/msb12 + (4*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mst22))/(msb12*mst12 - msb12*mst22) - (power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 - (2*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/msb12 + (4*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(msb12*mst12 - msb12*mst22)
     - (mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12))/(2.*
     power2(sb)) - (mu2*log(mst22/Q2)*power2(cb)*(((-5*mh2*mst22 + mst12*mst22
     - mst24)*DeltaInv(mst22,mh2,mst12))/mst12 + (8*(mh2 - 6*mst22)*mt2*
     DeltaInv(mst22,mst22,mh2))/(-mst12 + mst22) + 2*mst22*DeltaInv(mst22,mw2,
     msb12) - (2*mst24*DeltaInv(mst22,mw2,msb12))/msb12 - (4*mst22*mt2*
     DeltaInv(mst22,mw2,msb12))/(mst12 - mst22) + (4*mst24*mt2*DeltaInv(mst22,
     mw2,msb12))/(msb12*mst12 - msb12*mst22) - (10*mst22*mw2*DeltaInv(mst22,
     mw2,msb12))/msb12 + (20*mst22*mt2*mw2*DeltaInv(mst22,mw2,msb12))/(msb12*
     mst12 - msb12*mst22) + mst22*DeltaInv(mst22,mz2,mst12) - (mst24*DeltaInv(
     mst22,mz2,mst12))/mst12 - (5*mst22*mz2*DeltaInv(mst22,mz2,mst12))/mst12 -
     12*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + (24*mst24*mt2*mw2*power2(
     DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - 36*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - (168*mst24*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)))/msb12 + (72*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)))/(mst12 - mst22) + (336*mst24*mt2*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)))/(msb12*mst12 - msb12*mst22) - 6*mst24*mz2*power2(
     DeltaInv(mst22,mz2,mst12)) - 18*mst22*power2(mz2)*power2(DeltaInv(mst22,
     mz2,mst12)) - (84*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/
     mst12 + (12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22))/msb12 - (
     24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22))/(msb12*mst12 -
     msb12*mst22) + (6*mz2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22))/
     mst12 + (6*mh2*power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst12*mst22 - 14*
     mh2*mst24 - mst12*mst24 + mst22*power2(mh2) + power3(mst22)))/mst12 + (
     192*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 3*power3(mst22)))/(
     mst12 - mst22) + (12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/
     msb12 + (24*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/(
     msb12*(-mst12 + mst22)) + (6*mst22*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mz2))/mst12 + (72*power2(mh2)*(mst24*power2(mh2) + (-2*mst12 +
     mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12)))/mst12 - 288*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) + (576*mt2*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - 288*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + (576*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - (288*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 + (576*mt2*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(msb12*mst12 - msb12*mst22)
     - 144*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 144*
     mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - (144*power3(mst22)*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)))/mst12 + (2304*mh2*mt2*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22))/(mst12 - mst22) + (144*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/msb12 - (288*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/(msb12*
     mst12 - msb12*mst22) + (72*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22))/mst12 + (144*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2))/msb12 - (288*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2))/(msb12*mst12 - msb12*mst22) + (72*mst24*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mz2))/mst12))/(24.*power2(sb)) - (2*mu2*log(
     mst12/Q2)*log(mst22/Q2)*power2(cb)*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12))*(mh2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)*((
     mst12*mst22 - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)
     *power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22))))*power4(snt))/(
     mst12*power2(sb)) - (2*mu2*log(mst12/Q2)*log(mst22/Q2)*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12))*(mH2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(
     mH2,mst22,mst12)*((mst12*mst22 - mst24)*power2(mH2) - mst22*power3(mH2) +
     (5*mst12 - 3*mst22)*power3(mst22) + 5*mH2*(2*mst12*mst24 + power3(mst22)))
     )*power4(snt))/mst12 - (mh2*mu2*power2(cb)*power2(log(mst12/Q2))*(mst12*
     mst14*power2(DeltaInv(mh2,mst12,mst12)) + 2*mh2*power2(DeltaInv(mst22,mh2,
     mst12))*(2*mh2*mst22 + mst12*mst22 + 2*mst24 + DeltaInv(mst22,mh2,mst12)*(
     (3*mst12*mst22 + 2*mst24)*power2(mh2) - 2*mst22*power3(mh2) + (3*mst12 -
     2*mst22)*power3(mst22) + 2*mh2*(5*mst12*mst24 + power3(mst22)))))*power4(
     snt))/(mst12*power2(sb)) - (mH2*mu2*power2(log(mst12/Q2))*(mst12*mst14*
     power2(DeltaInv(mH2,mst12,mst12)) + 2*mH2*power2(DeltaInv(mH2,mst22,mst12)
     )*(2*mH2*mst22 + mst12*mst22 + 2*mst24 + DeltaInv(mH2,mst22,mst12)*((3*
     mst12*mst22 + 2*mst24)*power2(mH2) - 2*mst22*power3(mH2) + (3*mst12 - 2*
     mst22)*power3(mst22) + 2*mH2*(5*mst12*mst24 + power3(mst22)))))*power4(
     snt))/mst12 - (mH2*mu2*power2(log(mst22/Q2))*(2*mH2*mst24*power2(DeltaInv(
     mH2,mst22,mst12)) + mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*mH2*
     (mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)))*power4(snt))/mst12 +
     (2*mu2*log(mst12/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)*(-((mH2*mst22 +
     mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))) +
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(mh2*mst22 + mst12*mst22 +
     3*mst24 + DeltaInv(mst22,mh2,mst12)*((mst12*mst22 - mst24)*power2(mh2) -
     mst22*power3(mh2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*mh2*(2*mst12*
     mst24 + power3(mst22)))) + power2(mH2)*((-(mst12*mst22) + mst24)*power2(
     mH2) + mst22*power3(mH2) + (-5*mst12 + 3*mst22)*power3(mst22) - 5*mH2*(2*
     mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)))*power4(
     snt))/(mst12*power2(sb)) + (4*ca*cb*mu2*sa*log(mst12/Q2)*log(mst22/Q2)*((
     mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-(mh2*mst22) -
     mst12*mst22 - 3*mst24 + DeltaInv(mst22,mh2,mst12)*((-(mst12*mst22) +
     mst24)*power2(mh2) + mst22*power3(mh2) + (-5*mst12 + 3*mst22)*power3(
     mst22) - 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(mH2)*((mst12*
     mst22 - mst24)*power2(mH2) - mst22*power3(mH2) + (5*mst12 - 3*mst22)*
     power3(mst22) + 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mH2,mst22,mst12)))*power4(snt))/(mst12*sb) + (2*mu2*log(mst12/Q2)*log(
     mst22/Q2)*power2(sa)*((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12))*(-(mh2*mst22) - mst12*mst22 - 3*mst24 + DeltaInv(mst22,mh2,mst12)*
     ((-(mst12*mst22) + mst24)*power2(mh2) + mst22*power3(mh2) + (-5*mst12 + 3*
     mst22)*power3(mst22) - 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(
     mH2)*((mst12*mst22 - mst24)*power2(mH2) - mst22*power3(mH2) + (5*mst12 -
     3*mst22)*power3(mst22) + 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)))*power4(snt))/mst12 - (mh2*mu2*power2(cb)*
     power2(log(mst22/Q2))*(2*mh2*mst24*power2(DeltaInv(mst22,mh2,mst12)) +
     mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 2*mh2*(mst24*power2(mh2) +
     (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)))*power4(snt))/(mst12*power2(sb)) + (mu2*
     power2(cb)*power2(sa)*power2(log(mst22/Q2))*(-2*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,
     mst22)) + 2*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,
     mst12)) + mh2*(2*mh2*mst24*power2(DeltaInv(mst22,mh2,mst12)) + mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 2*mh2*(mst24*power2(mh2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12))))*power4(snt))/(mst12*power2(sb)) + (2*
     ca*cb*mu2*sa*power2(log(mst22/Q2))*(2*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) + mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*
     power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mh2*(-
     2*mh2*mst24*power2(DeltaInv(mst22,mh2,mst12)) - mst12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) + 2*mh2*(mst24*power2(mh2) + (-2*mst12 + mst22)
     *power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12))))*power4(snt))/(mst12*sb) + (mu2*power2(sa)*power2(log(
     mst22/Q2))*(2*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + mH2*
     mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*power2(mH2)*(mst24*
     power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mh2*(-2*mh2*mst24*
     power2(DeltaInv(mst22,mh2,mst12)) - mst12*mst24*power2(DeltaInv(mst22,
     mst22,mh2)) + 2*mh2*(mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22)
     - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12))))
     *power4(snt))/mst12 - (log(mst22/Q2)*(4*(mst12 - mst22) - (5*mH2*mst22 -
     mst12*mst22 + mst24)*mu2*DeltaInv(mH2,mst22,mst12) - mst12*(mH2 - 6*mst22)
     *mu2*DeltaInv(mH2,mst22,mst22) + 6*mH2*mu2*power2(DeltaInv(mH2,mst22,
     mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*power2(
     mH2) + power3(mst22)) + 24*mst12*mu2*power2(DeltaInv(mH2,mst22,mst22))*(
     mH2*mst24 + 3*power3(mst22)) + 72*mu2*power2(mH2)*(mst24*power2(mH2) + (-
     2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + 288*mH2*mst12*mu2*power3(DeltaInv(mH2,
     mst22,mst22))*power4(mst22))*power4(snt))/(6.*mst12) + (ca*cb*mu2*sa*log(
     mst22/Q2)*(-((5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(mH2,mst22,mst12)
     ) - mst12*(mH2 - 6*mst22)*DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*
     DeltaInv(mst22,mh2,mst12) - mst12*mst22*DeltaInv(mst22,mh2,mst12) + mst24*
     DeltaInv(mst22,mh2,mst12) + mh2*mst12*DeltaInv(mst22,mst22,mh2) - 6*mst12*
     mst22*DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*mst24*power2(DeltaInv(mst22,
     mh2,mst12)) + 18*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + 84*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 24*mh2*mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*power3(mh2) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mst22) - 72*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*
     power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 -
     mst12*mst24 + mst22*power2(mH2) + power3(mst22)) + 24*mst12*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 72*power2(mH2)*
     (mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 144*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 288*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) - 288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22))*
     power4(snt))/(3.*mst12*sb) + (mu2*log(mst22/Q2)*power2(sa)*(-((5*mH2*mst22
     - mst12*mst22 + mst24)*DeltaInv(mH2,mst22,mst12)) - mst12*(mH2 - 6*mst22)*
     DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*DeltaInv(mst22,mh2,mst12) - mst12*
     mst22*DeltaInv(mst22,mh2,mst12) + mst24*DeltaInv(mst22,mh2,mst12) + mh2*
     mst12*DeltaInv(mst22,mst22,mh2) - 6*mst12*mst22*DeltaInv(mst22,mst22,mh2)
     + 6*mh2*mst12*mst24*power2(DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 24*mh2*mst12*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2)
     - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power3(mst22) - 72*mst12*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*power2(DeltaInv(mH2,
     mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*
     power2(mH2) + power3(mst22)) + 24*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     (mH2*mst24 + 3*power3(mst22)) + 72*power2(mH2)*(mst24*power2(mH2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 144*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 288*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22))*power4(snt)
     )/(6.*mst12) - (mu2*log(mst22/Q2)*power2(cb)*power2(sa)*(-((5*mH2*mst22 -
     mst12*mst22 + mst24)*DeltaInv(mH2,mst22,mst12)) - mst12*(mH2 - 6*mst22)*
     DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*DeltaInv(mst22,mh2,mst12) - mst12*
     mst22*DeltaInv(mst22,mh2,mst12) + mst24*DeltaInv(mst22,mh2,mst12) + mh2*
     mst12*DeltaInv(mst22,mst22,mh2) - 6*mst12*mst22*DeltaInv(mst22,mst22,mh2)
     + 6*mh2*mst12*mst24*power2(DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 24*mh2*mst12*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2)
     - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power3(mst22) - 72*mst12*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*power2(DeltaInv(mH2,
     mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*
     power2(mH2) + power3(mst22)) + 24*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     (mH2*mst24 + 3*power3(mst22)) + 72*power2(mH2)*(mst24*power2(mH2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 144*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 288*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22))*power4(snt)
     )/(6.*mst12*power2(sb)) + (2*mu2*log(mH2/Q2)*log(mst22/Q2)*((mH2*mst22 +
     mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (
     mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(
     mst22)))*power4(snt))/mst12 - (4*ca*cb*mu2*sa*log(mH2/Q2)*log(mst22/Q2)*((
     mH2*mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*power3(
     mH2) + (mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22))
     )*power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(DeltaInv(mH2,mst22,
     mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*
     power4(mst22)))*power4(snt))/(mst12*sb) - (2*mu2*log(mH2/Q2)*log(mst22/Q2)
     *power2(sa)*((mH2*mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + power2(mH2)*((mst12*mst22 + mst24)*power2(
     mH2) - mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) + mH2*(6*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(
     mH2,mst22,mst22)*power4(mst22)))*power4(snt))/mst12 + (2*mu2*log(mH2/Q2)*
     log(mst22/Q2)*power2(cb)*power2(sa)*((mH2*mst22 + mst12*mst22 + mst24)*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(mH2)*((mst12*mst22
     + mst24)*power2(mH2) - mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) +
     mH2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) +
     mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*
     mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)))*power4(snt))/(mst12*power2(
     sb)) + (4*ca*cb*mu2*sa*log(mh2/Q2)*log(mst22/Q2)*((mh2*mst22 + mst12*mst22
     + mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((
     mst12*mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*
     power3(mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)) + mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 +
     4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)))*power4(
     snt))/(mst12*sb) + (2*mu2*log(mh2/Q2)*log(mst22/Q2)*power2(sa)*((mh2*mst22
     + mst12*mst22 + mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) +
     power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (
     mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(DeltaInv(mst22,mst22,mh2)
     )*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(
     mst22)))*power4(snt))/mst12 + (2*mu2*log(mh2/Q2)*log(mst22/Q2)*power2(cb)*
     ((mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) - mst22*power3(
     mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 + power3(mst22))
     )*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(DeltaInv(mst22,mst22,
     mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*
     power4(mst22)))*power4(snt))/(mst12*power2(sb)) - (2*mu2*log(mh2/Q2)*log(
     mst22/Q2)*power2(cb)*power2(sa)*((mh2*mst22 + mst12*mst22 + mst24)*power2(
     mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22 +
     mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)))*power4(snt))/(mst12*power2(
     sb)) - (mu2*log(mst22/Q2)*power2(cb)*(-((5*mh2*mst22 - mst12*mst22 +
     mst24)*DeltaInv(mst22,mh2,mst12)) + 6*mh2*power2(DeltaInv(mst22,mh2,mst12)
     )*(-3*mh2*mst12*mst22 - 14*mh2*mst24 - mst12*mst24 + mst22*power2(mh2) +
     power3(mst22)) + 72*power2(mh2)*(mst24*power2(mh2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)) + mst12*DeltaInv(mst22,mst22,mh2)*(-mh2 + 6*mst22 + 24*
     DeltaInv(mst22,mst22,mh2)*(mh2*mst24 + 3*power3(mst22)) + 288*mh2*power2(
     DeltaInv(mst22,mst22,mh2))*power4(mst22)))*power4(snt))/(6.*mst12*power2(
     sb)) + (21*mst22*mu2*power3(DeltaInv(mst22,mA2,mst12))*power5(mA2))/(2.*
     mst12) + (21*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2))/
     msb12 + (21*mst22*mu2*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2))/(2.*
     mst12) + (mu2*power2(cb)*power2(sa)*power2(snt)*power2(log(mst12/Q2))*(-(
     mh2*mst12*mst14*power2(DeltaInv(mh2,mst12,mst12))) + mH2*mst12*mst14*
     power2(DeltaInv(mH2,mst12,mst12)) + 2*mst12*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 4*mst24*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) - 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) - 4*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 4*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 4*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) + 20*mst12*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) + 6*mst12*power2(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) + 4*power3(mH2)*power3(mst22)*power3(DeltaInv(
     mH2,mst22,mst12)) - 20*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 4*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) -
     6*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 4*mst24*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst12*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*mst24*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) - 4*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mst22) + 4*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 4*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 4*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power5(mH2)))/(mst12*power2(sb)) + (2*
     ca*cb*mu2*sa*power2(log(mst12/Q2))*power4(snt)*(-(mh2*mst12*mst14*power2(
     DeltaInv(mh2,mst12,mst12))) + mH2*mst12*mst14*power2(DeltaInv(mH2,mst12,
     mst12)) + 2*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 4*
     mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 2*mst12*mst22*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 4*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 4*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 4*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(
     mH2) + 20*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 6*
     mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 4*
     power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 20*mst12*
     mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 6*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 4*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 6*mst12*mst22*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) - 4*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 6*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2)
     + 4*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) - 4*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) + 4*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) + 4*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) - 4*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2)))/(mst12*sb) + (mu2*power2(sa)*power2(log(mst12/Q2))*power4(
     snt)*(-(mh2*mst12*mst14*power2(DeltaInv(mh2,mst12,mst12))) + mH2*mst12*
     mst14*power2(DeltaInv(mH2,mst12,mst12)) + 2*mst12*mst22*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + 4*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) - 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 4*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 4*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 4*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) + 20*mst12*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) + 6*mst12*power2(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) + 4*power3(mH2)*power3(mst22)*power3(DeltaInv(
     mH2,mst22,mst12)) - 20*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 4*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) -
     6*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 4*mst24*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst12*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*mst24*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) - 4*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mst22) + 4*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 4*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 4*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power5(mH2)))/mst12 + (2*ca*cb*mu2*sa*
     power2(snt)*power2(log(mst12/Q2))*(mh2*mst12*mst14*power2(DeltaInv(mh2,
     mst12,mst12)) - mH2*mst12*mst14*power2(DeltaInv(mH2,mst12,mst12)) - 2*
     mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 4*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 2*mst12*mst22*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + 4*mst24*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 4*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2)
     - 4*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 20*mst12*mst24*
     power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) - 6*mst12*power2(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 4*power3(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 20*mst12*mst24*power3(mh2)*
     power3(DeltaInv(mst22,mh2,mst12)) + 6*mst12*power2(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) + 4*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 6*mst12*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) + 4*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) - 6*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) - 4*
     mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 4*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) - 4*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) + 4*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2)))/(mst12*sb) + (mu2*power2(sa)*power2(snt)*power2(log(mst12/
     Q2))*(mh2*mst12*mst14*power2(DeltaInv(mh2,mst12,mst12)) - mH2*mst12*mst14*
     power2(DeltaInv(mH2,mst12,mst12)) - 2*mst12*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 4*mst24*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) + 4*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 4*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 4*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 20*mst12*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) - 6*mst12*power2(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) - 4*power3(mH2)*power3(mst22)*power3(DeltaInv(
     mH2,mst22,mst12)) + 20*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) + 6*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) + 4*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) +
     6*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 4*mst24*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 6*mst12*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) - 4*mst24*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) + 4*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mst22) - 4*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) - 4*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) + 4*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power5(mH2)))/mst12 + (mu2*power2(cb)*
     power2(sa)*power2(log(mst12/Q2))*power4(snt)*(mh2*mst12*mst14*power2(
     DeltaInv(mh2,mst12,mst12)) - mH2*mst12*mst14*power2(DeltaInv(mH2,mst12,
     mst12)) - 2*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 4*
     mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 2*mst12*mst22*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 4*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + 4*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) - 4*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(
     mH2) - 20*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) - 6*
     mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 4*
     power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 20*mst12*
     mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 6*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 4*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 6*mst12*mst22*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) + 4*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) - 6*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2)
     - 4*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 4*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) - 4*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) + 4*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2)))/(mst12*power2(sb)) + (power2(sa)*(mh2/mst12 - mH2/mst12 +
     mh2/mst22 - mH2/mst22 + (4*mt2)/mh2 - (4*mt2)/mH2 - 4*mh2*mt2*DeltaInv(
     mh2,mst12,mst12) - 4*mt2*(22*mh2 + 101*mt2)*DeltaInv(mh2,mt2,mt2) + 4*mH2*
     mt2*DeltaInv(mH2,mst12,mst12) - 2*mH2*mu2*DeltaInv(mH2,mst22,mst12) + 4*
     mH2*mt2*DeltaInv(mH2,mst22,mst22) + 88*mH2*mt2*DeltaInv(mH2,mt2,mt2) + 2*
     mh2*mu2*DeltaInv(mst22,mh2,mst12) - 4*mh2*mt2*DeltaInv(mst22,mst22,mh2) +
     404*DeltaInv(mH2,mt2,mt2)*power2(mt2) - 168*mH2*mst14*mt2*power2(DeltaInv(
     mH2,mst12,mst12)) + 84*mst22*mu2*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + (252*mst24*mu2*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)))/
     mst12 - 168*mH2*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) - 84*mst22*
     mu2*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - (252*mst24*mu2*power2(
     mh2)*power2(DeltaInv(mst22,mh2,mst12)))/mst12 + 168*mh2*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) - (252*mst22*mu2*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2))/mst12 + (252*mst22*mu2*power2(DeltaInv(mH2,mst22,
     mst12))*power3(mH2))/mst12 - 1008*mt2*power2(DeltaInv(mH2,mst12,mst12))*
     power3(mst12) + 168*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*
     power3(mst12)) - 1008*mt2*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22)
     + 1008*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) - 1512*mh2*
     power2(DeltaInv(mh2,mt2,mt2))*power3(mt2) + 1512*mH2*power2(DeltaInv(mH2,
     mt2,mt2))*power3(mt2) + 1176*mst24*mu2*power3(mH2)*power3(DeltaInv(mH2,
     mst22,mst12)) + 420*mu2*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,
     mst22,mst12)) + (252*mu2*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,
     mst22,mst12)))/mst12 - 1176*mst24*mu2*power3(mh2)*power3(DeltaInv(mst22,
     mh2,mst12)) - 420*mu2*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - (252*mu2*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)))/mst12 - 420*mst22*mu2*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) - (252*mst24*mu2*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2))/mst12
     + 420*mst22*mu2*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + (252*
     mst24*mu2*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2))/mst12 + 4032*mh2*
     mt2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) - 4032*mH2*mt2*power3(
     DeltaInv(mH2,mst12,mst12))*power4(mst12) - (252*mu2*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mst22))/mst12 - 4032*mH2*mt2*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + (252*mu2*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22))/mst12 + 4032*mh2*mt2*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22) + (252*mst22*mu2*power3(DeltaInv(
     mst22,mh2,mst12))*power5(mh2))/mst12 - (252*mst22*mu2*power3(DeltaInv(mH2,
     mst22,mst12))*power5(mH2))/mst12))/24. - (14*power2(mu2)*power3(DeltaInv(
     msb12,mt2,mu2))*power5(msb12))/mt2 + (DeltaInv(msb12,mt2,mu2)*log(msb12/
     Q2)*log(mt2/Q2)*power2(mu2)*(msb12 + DeltaInv(msb12,mt2,mu2)*(msb12*mu2*(
     4*mt2 + mu2) + 2*msb14*(2*mt2 + 7*mu2) + power3(msb12)) - 2*power2(
     DeltaInv(msb12,mt2,mu2))*(msb14*(-15*mt2 + 4*mu2)*power2(mu2) - 5*mu2*(3*
     mt2 + 2*mu2)*power3(msb12) + msb12*(-mt2 + mu2)*power3(mu2) - (mt2 - 4*
     mu2)*power4(msb12) + power5(msb12))))/mt2 - (DeltaInv(msb12,mt2,mu2)*log(
     msb12/Q2)*log(mt2/Q2)*power2(mu2)*power2(snb)*(msb12 + DeltaInv(msb12,mt2,
     mu2)*(msb12*mu2*(4*mt2 + mu2) + 2*msb14*(2*mt2 + 7*mu2) + power3(msb12)) -
     2*power2(DeltaInv(msb12,mt2,mu2))*(msb14*(-15*mt2 + 4*mu2)*power2(mu2) -
     5*mu2*(3*mt2 + 2*mu2)*power3(msb12) + msb12*(-mt2 + mu2)*power3(mu2) - (
     mt2 - 4*mu2)*power4(msb12) + power5(msb12))))/mt2 - (power2(log(msb12/Q2))
     *(-(msb12*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*(msb14*(mt2 + 3*mu2)
     + power3(msb12))) + mt2*((2*mC2*mst22 + msb12*mst22 + 2*mst24)*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + mu2*power2(mC2)*((3*msb12*mst22 +
     2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))) +
     msb12*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*(msb14*(-5*mt2 + 3*mu2)*
     power2(mu2) - 5*mu2*(2*mt2 + mu2)*power3(msb12) + (-mt2 + mu2)*power4(
     msb12) + power5(msb12))))/(msb12*mt2) + (power2(snb)*power2(log(msb12/Q2))
     *(-(msb12*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*(msb14*(mt2 + 3*mu2)
     + power3(msb12))) + mt2*((2*mC2*mst22 + msb12*mst22 + 2*mst24)*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + mu2*power2(mC2)*((3*msb12*mst22 +
     2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))) +
     msb12*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*(msb14*(-5*mt2 + 3*mu2)*
     power2(mu2) - 5*mu2*(2*mt2 + mu2)*power3(msb12) + (-mt2 + mu2)*power4(
     msb12) + power5(msb12))))/(msb12*mt2) + (log(msb12/Q2)*(-2*mst12*mst22*
     mu2*(mt2 + 12*mu2)*DeltaInv(msb12,mt2,mu2)*power2(msb12) - 24*msb12*mst12*
     mst22*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*(msb12*(2*mt2 - mu2)*mu2
     + msb14*(5*mt2 + 11*mu2) + 2*power3(msb12)) + mt2*(msb12*mst22*mt2 +
     mst12*mst22*mt2 + mC2*mst12*mu2 + msb12*mst12*mu2 + mst12*mst22*mu2 +
     mst22*mt2*mw2 + mst12*power2(msb12) - mst12*mu2*DeltaInv(mst22,msb12,mC2)*
     (mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*
     power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*mu2*power2(DeltaInv(
     mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (
     msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) - mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*
     (6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(
     mw2)) - 72*mst12*mst22*mu2*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     72*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(10*
     msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))) + 72*msb12*mst12*mst22*power2(mu2)*power3(DeltaInv(msb12,
     mt2,mu2))*(msb14*(-5*mt2 + 3*mu2)*power2(mu2) - 5*mu2*(2*mt2 + mu2)*
     power3(msb12) + (-mt2 + mu2)*power4(msb12) + power5(msb12))))/(12.*msb12*
     mst12*mst22*mt2) - (log(msb12/Q2)*power2(snb)*(-2*mst12*mst22*mu2*(mt2 +
     12*mu2)*DeltaInv(msb12,mt2,mu2)*power2(msb12) - 24*msb12*mst12*mst22*
     power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*(msb12*(2*mt2 - mu2)*mu2 +
     msb14*(5*mt2 + 11*mu2) + 2*power3(msb12)) + mt2*(msb12*mst22*mt2 + mst12*
     mst22*mt2 + mC2*mst12*mu2 + msb12*mst12*mu2 + mst12*mst22*mu2 + mst22*mt2*
     mw2 + mst12*power2(msb12) - mst12*mu2*DeltaInv(mst22,msb12,mC2)*(mst22*(-(
     msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*mu2*power2(DeltaInv(mst22,msb12,
     mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*
     mst22)*power2(mC2) + power3(mC2) + power3(mst22)) - mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)
     ) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mst22*mt2*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*
     mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) - 72*
     mst12*mst22*mu2*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 72*mst12*
     mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) +
     72*msb12*mst12*mst22*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*(msb14*(-
     5*mt2 + 3*mu2)*power2(mu2) - 5*mu2*(2*mt2 + mu2)*power3(msb12) + (-mt2 +
     mu2)*power4(msb12) + power5(msb12))))/(12.*msb12*mst12*mst22*mt2) + (
     power2(mu2)*power2(snb)*power2(log(mt2/Q2))*(-(msb12*DeltaInv(msb12,mt2,
     mu2)) - (3*msb14*(mt2 + 4*mu2) + msb12*mu2*(5*mt2 + 4*mu2))*power2(
     DeltaInv(msb12,mt2,mu2)) + power3(DeltaInv(msb12,mt2,mu2))*(-35*msb14*mt2*
     power2(mu2) - 7*mu2*(3*mt2 + 2*mu2)*power3(msb12) + msb12*(-7*mt2 + 5*mu2)
     *power3(mu2) - (mt2 - 8*mu2)*power4(msb12) + power5(msb12)) + DeltaInv(
     mt2,mu2,msb22)*(msb22 + (3*msb24*(mt2 + 4*mu2) + msb22*mu2*(5*mt2 + 4*mu2)
     )*DeltaInv(mt2,mu2,msb22) + power2(DeltaInv(mt2,mu2,msb22))*(35*msb24*mt2*
     power2(mu2) + 7*mu2*(3*mt2 + 2*mu2)*power3(msb22) + msb22*(7*mt2 - 5*mu2)*
     power3(mu2) + (mt2 - 8*mu2)*power4(msb22) - power5(msb22)))))/mt2 - (log(
     mt2/Q2)*power2(snb)*((msb12 - msb22)*(mt2*mu2 + power2(mt2) + power2(mu2))
     + msb22*mu2*DeltaInv(msb12,mt2,mu2)*(2*msb12*mu2*(mt2 + 2*mu2) + (mt2 -
     17*mu2)*power2(msb12) + (-mt2 + mu2)*power2(mu2)) - msb12*mu2*DeltaInv(
     mt2,mu2,msb22)*(2*msb22*mu2*(mt2 + 2*mu2) + (mt2 - 17*mu2)*power2(msb22) +
     (-mt2 + mu2)*power2(mu2)) - 6*msb12*msb22*power2(mu2)*power2(DeltaInv(
     msb12,mt2,mu2))*(13*msb14*mt2 + 67*msb14*mu2 + 5*msb12*mu2*(4*mt2 + 5*mu2)
     - mt2*power2(mu2) + 3*power3(msb12) + power3(mu2)) + 6*msb12*msb22*power2(
     mu2)*power2(DeltaInv(mt2,mu2,msb22))*(13*msb24*mt2 + 67*msb24*mu2 + 5*
     msb22*mu2*(4*mt2 + 5*mu2) - mt2*power2(mu2) + 3*power3(msb22) + power3(
     mu2)) + 36*msb12*msb22*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*(-35*
     msb14*mt2*power2(mu2) - 7*mu2*(3*mt2 + 2*mu2)*power3(msb12) + msb12*(-7*
     mt2 + 5*mu2)*power3(mu2) - (mt2 - 8*mu2)*power4(msb12) + power5(msb12)) -
     36*msb12*msb22*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*(-35*msb24*mt2*
     power2(mu2) - 7*mu2*(3*mt2 + 2*mu2)*power3(msb22) + msb22*(-7*mt2 + 5*mu2)
     *power3(mu2) - (mt2 - 8*mu2)*power4(msb22) + power5(msb22))))/(6.*msb12*
     msb22*mt2) + (DeltaInv(mt2,mu2,msb22)*log(msb22/Q2)*log(mt2/Q2)*power2(
     mu2)*power2(snb)*(msb22 + DeltaInv(mt2,mu2,msb22)*(msb22*mu2*(4*mt2 + mu2)
     + 2*msb24*(2*mt2 + 7*mu2) + power3(msb22)) - 2*power2(DeltaInv(mt2,mu2,
     msb22))*(msb24*(-15*mt2 + 4*mu2)*power2(mu2) - 5*mu2*(3*mt2 + 2*mu2)*
     power3(msb22) + msb22*(-mt2 + mu2)*power3(mu2) - (mt2 - 4*mu2)*power4(
     msb22) + power5(msb22))))/mt2 - (power2(snb)*power2(log(msb22/Q2))*((2*
     mC2*mst22 + msb22*mst22 + 2*mst24)*mt2*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) - msb22*power2(mu2)*power2(DeltaInv(mt2,mu2,msb22))*(
     msb24*(mt2 + 3*mu2) + power3(msb22)) + mt2*mu2*power2(mC2)*((3*msb22*mst22
     + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mt2)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) + msb22*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*(msb24*(-5*
     mt2 + 3*mu2)*power2(mu2) - 5*mu2*(2*mt2 + mu2)*power3(msb22) + (-mt2 +
     mu2)*power4(msb22) + power5(msb22))))/(msb22*mt2) + (log(msb22/Q2)*power2(
     snb)*(-2*mst12*mst22*mu2*(mt2 + 12*mu2)*DeltaInv(mt2,mu2,msb22)*power2(
     msb22) - mst12*mt2*mu2*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) +
     mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)
     ) - 24*msb22*mst12*mst22*power2(mu2)*power2(DeltaInv(mt2,mu2,msb22))*(
     msb22*(2*mt2 - mu2)*mu2 + msb24*(5*mt2 + 11*mu2) + 2*power3(msb22)) + 6*
     mC2*mst12*mst22*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24)
     + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - 72*mst12*mst22*mt2*mu2*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + mt2*(msb22*mst22*mt2 + mst12*mst22*mt2 + mC2*
     mst12*mu2 + msb22*mst12*mu2 + mst12*mst22*mu2 + mst22*mt2*mw2 + mst12*
     power2(msb22) - mst22*mt2*DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) -
     msb22*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) +
     power3(mw2)) + 6*mst12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*(
     23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2))
     + power3(mst12) + power3(mw2)) + 72*mst12*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + 72*msb22*mst12*mst22*power2(mu2)*
     power3(DeltaInv(mt2,mu2,msb22))*(msb24*(-5*mt2 + 3*mu2)*power2(mu2) - 5*
     mu2*(2*mt2 + mu2)*power3(msb22) + (-mt2 + mu2)*power4(msb22) + power5(
     msb22))))/(12.*msb22*mst12*mst22*mt2) - (14*power2(mu2)*power3(DeltaInv(
     mt2,mu2,mst12))*power5(mst12))/mt2 - (4*ca*cb*mu2*sa*log(mh2/Q2)*log(
     mst12/Q2)*power2(snt)*(power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-3*
     mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(mst22,mh2,mst12)*(-5*(mst12*
     mst22 + mst24)*power2(mh2) + 3*mst22*power3(mh2) + (-mst12 + mst22)*
     power3(mst22) + mh2*(-10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + 16*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(mst12*sb) - (2*mu2*log(mh2/Q2)
     *log(mst12/Q2)*power2(sa)*power2(snt)*(power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(mst22,mh2,
     mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(mh2) + (-
     mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(mst22)))) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/mst12 - (2*mu2*
     log(mh2/Q2)*log(mst12/Q2)*power2(cb)*power2(snt)*(power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(
     mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(
     mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(
     mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(
     mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(mst12*
     power2(sb)) + (2*mu2*log(mh2/Q2)*log(mst12/Q2)*power2(cb)*power2(sa)*
     power2(snt)*(power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 -
     mst12*mst22 - mst24 + DeltaInv(mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*
     power2(mh2) + 3*mst22*power3(mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(
     -10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst12*mst14 + 4*power4(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12)))/(mst12*power2(sb)) + (4*ca*cb*mu2*sa*log(mh2/Q2)*
     log(mst12/Q2)*power4(snt)*(power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(
     -3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(mst22,mh2,mst12)*(-5*(mst12*
     mst22 + mst24)*power2(mh2) + 3*mst22*power3(mh2) + (-mst12 + mst22)*
     power3(mst22) + mh2*(-10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + 16*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(mst12*sb) + (2*mu2*log(mh2/Q2)
     *log(mst12/Q2)*power2(sa)*power4(snt)*(power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(mst22,mh2,
     mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(mh2) + (-
     mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(mst22)))) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/mst12 + (2*mu2*
     log(mh2/Q2)*log(mst12/Q2)*power2(cb)*power4(snt)*(power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(
     mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(
     mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(
     mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(
     mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(mst12*
     power2(sb)) - (2*mu2*log(mh2/Q2)*log(mst12/Q2)*power2(cb)*power2(sa)*
     power4(snt)*(power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 -
     mst12*mst22 - mst24 + DeltaInv(mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*
     power2(mh2) + 3*mst22*power3(mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(
     -10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst12*mst14 + 4*power4(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12)))/(mst12*power2(sb)) - (2*ca*cb*mu2*sa*power2(snt)*
     power2(log(mh2/Q2))*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mh2) + 2*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12)))/(mst12*sb) - (mu2*power2(sa)*power2(snt)*
     power2(log(mh2/Q2))*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mh2) + 2*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12)))/mst12 - (mu2*power2(cb)*power2(snt)*power2(
     log(mh2/Q2))*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*
     power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(
     mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))
     *power5(mst12)))/(mst12*power2(sb)) + (mu2*power2(cb)*power2(sa)*power2(
     snt)*power2(log(mh2/Q2))*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 2*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24)
     + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12)))/(mst12*power2(sb)) + (2*ca*cb*mu2*sa*power2(
     log(mh2/Q2))*power4(snt)*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 2*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24)
     + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12)))/(mst12*sb) + (mu2*power2(sa)*power2(log(mh2/
     Q2))*power4(snt)*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) +
     2*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*
     power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) + power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + mst12*
     power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*
     DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12)))/mst12 + (mu2*power2(cb)*power2(log(mh2/Q2))*
     power4(snt)*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*
     power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(
     mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))
     *power5(mst12)))/(mst12*power2(sb)) - (mu2*power2(cb)*power2(sa)*power2(
     log(mh2/Q2))*power4(snt)*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 2*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24)
     + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12)))/(mst12*power2(sb)) + (mu2*power2(cb)*power2(
     Pi)*power4(snt)*(-2*(3*mh2*mst22 + mst12*mst22 + 3*mst24)*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst12*mst14 + 6*power4(mst12)) + 2*power2(mh2)*power3(DeltaInv(mst22,
     mh2,mst12))*(-14*mh2*mst12*mst24 - 5*mst12*mst22*power2(mh2) - 3*mst24*
     power2(mh2) + 3*mst22*power3(mh2) - 3*mh2*power3(mst22) - 5*mst12*power3(
     mst22) + 3*power4(mst22)) + mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*
     mst24 + 6*power3(mst22) + 24*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))
     + 24*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(6.*mst12*
     power2(sb)) + (mu2*power2(cb)*power4(snt)*(-(mh2*mst12*DeltaInv(mh2,mst12,
     mst12)) + 2*mh2*mst12*DeltaInv(mst22,mh2,mst12) - 84*(3*mh2*mst22 + mst12*
     mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*
     power2(mh2)*(-((5*mst12*mst22 + 3*mst24)*power2(mh2)) + 3*mst22*power3(
     mh2) + (-5*mst12 + 3*mst22)*power3(mst22) - mh2*(14*mst12*mst24 + 3*
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + 42*power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) + mst12*DeltaInv(
     mst22,mst22,mh2)*(-mh2 + 42*DeltaInv(mst22,mst22,mh2)*(mh2*mst24 + 6*
     power3(mst22)) + 1008*mh2*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22))
     + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(6.*mst12*
     power2(sb)) - (mu2*power2(cb)*power2(Pi)*power2(snt)*(-2*msb12*(3*mh2*
     mst22 + mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + msb12*mst12*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*
     power3(mst12)) + 2*msb12*power2(mh2)*(-((5*mst12*mst22 + 3*mst24)*power2(
     mh2)) + 3*mst22*power3(mh2) + (-5*mst12 + 3*mst22)*power3(mst22) - mh2*(
     14*mst12*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     mst12*(msb12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 6*power3(
     mst22)) + power2(mw2)*(-((msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*
     mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(
     mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) -
     power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22))) + 24*mh2*msb12*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22)) + 24*mh2*msb12*power3(DeltaInv(
     mh2,mst12,mst12))*power5(mst12)))/(6.*msb12*mst12*power2(sb)) - (mu2*log(
     mst12/Q2)*power2(cb)*power4(snt)*(-mh2 - mst12 - mst22 + mst12*(-mh2 + 6*
     mst12)*mst22*DeltaInv(mh2,mst12,mst12) + DeltaInv(mst22,mh2,mst12)*(mst22*
     (-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(
     mst22) + power3(mh2)) + 24*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 3*power3(mst12)) - 6*mh2*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 + 23*mst24) - (mst12 - 23*
     mst22)*power2(mh2) + power3(mh2) + power3(mst22)) + 72*mst22*power2(mh2)*(
     -((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*power3(mh2) + (-3*mst12
     + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12)) + 288*mh2*mst22*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12)))/(6.*mst12*mst22*power2(sb)) - (ca*cb*mu2*sa*
     power2(log(mh2/Q2))*((mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 2*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst14 + 4*power3(mst12)) - (mst12 - mst22)*power3(mh2)*(-2*mst12*mst24 -
     2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*power3(
     DeltaInv(mst22,mh2,mst12)) - 2*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))
     *(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(
     mst22)) + 32*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(
     mst12*(mst12 - mst22)*sb) + (power2(sa)*power2(log(mh2/Q2))*(-(mst12*mt2*(
     mh2 + 3*mt2)*DeltaInv(mh2,mt2,mt2)) + 2*mh2*mst12*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) - mst22*mu2*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 2*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 +
     4*power3(mst12)) + 8*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) - 12*mh2*mst12*power2(DeltaInv(mh2,mt2,mt2))*power3(mt2) - 2*mst12*
     mst24*mu2*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + mu2*power3(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 2*mst12*mst22*mu2*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 2*mst24*mu2*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 32*mh2*mst12*mt2*power3(DeltaInv(
     mst22,mst22,mh2))*power4(mst22) + mst22*mu2*power3(DeltaInv(mst22,mh2,
     mst12))*power5(mh2) + 32*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/(2.*mst12) - (ca*cb*mu2*sa*log(mh2/Q2)*log(mst12/Q2)*(4*mst12*
     mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12)) + (
     mst12 - mst22)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(3*mh2*mst22
     + mst12*mst22 + mst24 + DeltaInv(mst22,mh2,mst12)*(5*(mst12*mst22 + mst24)
     *power2(mh2) - 3*mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) - mh2*(
     -10*mst12*mst24 + power3(mst22)))) + 64*mh2*mt2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12)))/(mst12*(mst12 - mst22)*sb) + (log(mh2/Q2)*log(
     mst12/Q2)*power2(sa)*(4*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst14 + 4*power3(mst12)) + mu2*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(mst22,mh2,mst12)*(-
     5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(mh2) + (-mst12 +
     mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(mst22)))) + 64*mh2*
     mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(2.*mst12) - (mu2*
     power2(cb)*power2(log(mh2/Q2))*((mst12 - mst22)*mst22*power2(DeltaInv(
     mst22,mh2,mst12))*power3(mh2) + 4*mst12*mt2*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 4*power3(mst12)) - (mst12 - mst22)*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) - 4*mst12*mt2*power2(DeltaInv(
     mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,
     mst22,mh2)*power4(mst22)) + 64*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12)))/(2.*mst12*(mst12 - mst22)*power2(sb)) + (mu2*power2(cb)*
     power2(sa)*power2(log(mh2/Q2))*((mst12 - mst22)*mst22*power2(DeltaInv(
     mst22,mh2,mst12))*power3(mh2) + 4*mst12*mt2*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 4*power3(mst12)) - (mst12 - mst22)*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) - 4*mst12*mt2*power2(DeltaInv(
     mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,
     mst22,mh2)*power4(mst22)) + 64*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12)))/(2.*mst12*(mst12 - mst22)*power2(sb)) - (mu2*log(mh2/Q2)*
     log(mst12/Q2)*power2(cb)*(8*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 4*power3(mst12)) + (mst12 - mst22)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*(3*mh2*mst22 + mst12*mst22 + mst24 + DeltaInv(
     mst22,mh2,mst12)*(5*(mst12*mst22 + mst24)*power2(mh2) - 3*mst22*power3(
     mh2) + (mst12 - mst22)*power3(mst22) - mh2*(-10*mst12*mst24 + power3(
     mst22)))) + 128*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/
     (2.*mst12*(mst12 - mst22)*power2(sb)) + (mu2*log(mh2/Q2)*log(mst12/Q2)*
     power2(cb)*power2(sa)*(8*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst14 + 4*power3(mst12)) + (mst12 - mst22)*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12))*(3*mh2*mst22 + mst12*mst22 + mst24 + DeltaInv(mst22,mh2,
     mst12)*(5*(mst12*mst22 + mst24)*power2(mh2) - 3*mst22*power3(mh2) + (mst12
     - mst22)*power3(mst22) - mh2*(-10*mst12*mst24 + power3(mst22)))) + 128*
     mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(2.*mst12*(mst12
     - mst22)*power2(sb)) - (ca*cb*mu2*sa*power2(snt)*(-(mh2*mst12*DeltaInv(
     mh2,mst12,mst12)) + mH2*mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*
     DeltaInv(mH2,mst22,mst12) + mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*
     mst12*DeltaInv(mst22,mh2,mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) +
     84*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) - 84*mst12*mst22*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 252*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) + 42*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 252*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 252*mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*power3(mst22) + 252*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 1176*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) +
     420*mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) +
     252*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 1176*
     mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 252*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mH2) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(
     mst12)) - 42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*
     power4(mst12)) - 252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) - 1008*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) +
     252*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*
     mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(
     mH2,mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))
     *power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12))
     )/(3.*mst12*sb) - (mu2*power2(sa)*power2(snt)*(-(mh2*mst12*DeltaInv(mh2,
     mst12,mst12)) + mH2*mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*
     DeltaInv(mH2,mst22,mst12) + mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*
     mst12*DeltaInv(mst22,mh2,mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) +
     84*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) - 84*mst12*mst22*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 252*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) + 42*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 252*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 252*mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*power3(mst22) + 252*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 1176*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) +
     420*mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) +
     252*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 1176*
     mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 252*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mH2) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(
     mst12)) - 42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*
     power4(mst12)) - 252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) - 1008*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) +
     252*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*
     mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(
     mH2,mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))
     *power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12))
     )/(6.*mst12) + (mu2*power2(cb)*power2(sa)*power2(snt)*(-(mh2*mst12*
     DeltaInv(mh2,mst12,mst12)) + mH2*mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*
     mst12*DeltaInv(mH2,mst22,mst12) + mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*
     mh2*mst12*DeltaInv(mst22,mh2,mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2)
     + 84*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 252*
     mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) - 84*mst12*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 252*mst24*power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12)) + 42*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*
     mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 252*mst22*power2(
     DeltaInv(mH2,mst22,mst12))*power3(mH2) - 252*mst12*power2(DeltaInv(mH2,
     mst22,mst22))*power3(mst22) + 252*mst12*power2(DeltaInv(mst22,mst22,mh2))*
     power3(mst22) + 1176*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) + 420*mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,
     mst12)) + 252*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12))
     - 1176*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 420*
     mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 252*
     power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*
     mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 420*mst12*mst22*power3(DeltaInv(
     mH2,mst22,mst12))*power4(mH2) + 252*mst24*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mH2) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*
     power4(mst12)) - 42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 +
     6*power4(mst12)) - 252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*
     power4(mst22) - 1008*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) + 252*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) +
     1008*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*
     mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*
     power5(mst12)))/(6.*mst12*power2(sb)) + (ca*cb*mu2*sa*power4(snt)*(-(mh2*
     mst12*DeltaInv(mh2,mst12,mst12)) + mH2*mst12*DeltaInv(mH2,mst12,mst12) -
     2*mH2*mst12*DeltaInv(mH2,mst22,mst12) + mH2*mst12*DeltaInv(mH2,mst22,
     mst22) + 2*mh2*mst12*DeltaInv(mst22,mh2,mst12) - mh2*mst12*DeltaInv(mst22,
     mst22,mh2) + 84*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))
     + 252*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*
     mst24*power2(DeltaInv(mH2,mst22,mst22)) - 84*mst12*mst22*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 252*mst24*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 42*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) -
     252*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 252*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 252*mst12*power2(DeltaInv(
     mH2,mst22,mst22))*power3(mst22) + 252*mst12*power2(DeltaInv(mst22,mst22,
     mh2))*power3(mst22) + 1176*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,
     mst22,mst12)) + 420*mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,
     mst22,mst12)) + 252*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,
     mst12)) - 1176*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) -
     420*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) -
     252*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 420*
     mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mst24*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 420*mst12*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 252*mst24*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst12*mst14 + 6*power4(mst12)) - 42*power2(DeltaInv(mH2,mst12,mst12))*(
     mH2*mst12*mst14 + 6*power4(mst12)) - 252*power2(mH2)*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mst22) - 1008*mH2*mst12*power3(DeltaInv(mH2,mst22,
     mst22))*power4(mst22) + 252*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mst22) + 1008*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) + 252*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*
     mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 1008*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(3.*mst12*sb) + (mu2*power2(sa)*power4(snt)*
     (-(mh2*mst12*DeltaInv(mh2,mst12,mst12)) + mH2*mst12*DeltaInv(mH2,mst12,
     mst12) - 2*mH2*mst12*DeltaInv(mH2,mst22,mst12) + mH2*mst12*DeltaInv(mH2,
     mst22,mst22) + 2*mh2*mst12*DeltaInv(mst22,mh2,mst12) - mh2*mst12*DeltaInv(
     mst22,mst22,mh2) + 84*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + 252*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 42*
     mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 84*mst12*mst22*power2(
     mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 252*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + 42*mh2*mst12*mst24*power2(DeltaInv(mst22,
     mst22,mh2)) - 252*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) +
     252*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 252*mst12*
     power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 252*mst12*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 1176*mst12*mst24*power3(mH2)*
     power3(DeltaInv(mH2,mst22,mst12)) + 420*mst12*power2(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) + 252*power3(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) - 1176*mst12*mst24*power3(mh2)*power3(DeltaInv(
     mst22,mh2,mst12)) - 420*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 252*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,
     mh2,mst12)) - 420*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) - 252*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 420*
     mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 252*mst24*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 42*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - 42*power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) - 252*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 1008*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + 252*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*mh2*mst12*power3(DeltaInv(
     mst22,mst22,mh2))*power4(mst22) + 252*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(
     mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 1008*
     mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*mst12) - (mu2*
     power2(cb)*power2(sa)*power4(snt)*(-(mh2*mst12*DeltaInv(mh2,mst12,mst12))
     + mH2*mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*DeltaInv(mH2,mst22,
     mst12) + mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*mst12*DeltaInv(mst22,
     mh2,mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) + 84*mst12*mst22*power2(
     mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,
     mst22)) - 84*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) -
     252*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 42*mh2*mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*power3(mh2) + 252*mst22*power2(DeltaInv(mH2,mst22,mst12))*
     power3(mH2) - 252*mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) +
     252*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 1176*mst12*
     mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 420*mst12*power2(
     mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 252*power3(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 1176*mst12*mst24*power3(
     mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*power2(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 252*power3(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mH2) + 252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 42*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) -
     42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) -
     252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 1008*
     mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 252*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(mH2,
     mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))
     /(6.*mst12*power2(sb)) - (ca*cb*mu2*sa*power2(Pi)*power2(snt)*(2*mst12*
     mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 6*mst24*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,
     mst22,mst22)) - 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) - 6*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + mh2*mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*power3(mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*
     power3(mH2) - 6*mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 6*
     mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 28*mst12*mst24*
     power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*power2(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*power3(mh2)*
     power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*power2(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) - 6*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 10*mst12*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) - 6*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) + 10*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*
     mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mst22) - 24*mH2*mst12*power3(DeltaInv(
     mH2,mst22,mst22))*power4(mst22) + 6*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*
     power4(mst22) + 6*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 6*
     mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*mH2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(3.*mst12*sb) - (mu2*power2(Pi)*power2(sa)*
     power2(snt)*(2*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     6*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) - 2*mst12*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 6*mst24*power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12)) + mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 6*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 6*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 6*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) +
     28*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*
     power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(
     mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 6*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*mst22*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) - 6*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 10*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2)
     + 6*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 24*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + 6*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) + 6*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 6*mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*
     mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*mst12) + (mu2*power2(cb)*
     power2(Pi)*power2(sa)*power2(snt)*(2*mst12*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mst24*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*
     mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*mst24*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + mh2*mst12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 6*
     mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 6*mst12*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 28*mst12*mst24*power3(mH2)*
     power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*power2(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*power3(mh2)*power3(DeltaInv(
     mst22,mh2,mst12)) - 10*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 6*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 10*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) -
     6*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 10*mst12*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mst24*power3(DeltaInv(
     mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 24*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) + 6*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 6*
     mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 6*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) - 24*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/(6.*mst12*power2(sb)) + (ca*cb*mu2*sa*power2(Pi)*power4(snt)*(2*
     mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 6*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) - 2*mst12*mst22*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 6*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) + 6*mst22*power2(DeltaInv(mH2,
     mst22,mst12))*power3(mH2) - 6*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 6*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) +
     28*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*
     power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(
     mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 6*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*mst22*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) - 6*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 10*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2)
     + 6*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 24*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + 6*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) + 6*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 6*mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*
     mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(3.*mst12*sb) + (mu2*power2(Pi)
     *power2(sa)*power4(snt)*(2*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + 6*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) -
     mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*mst12*mst22*power2(
     mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,
     mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 6*mst12*power2(DeltaInv(
     mH2,mst22,mst22))*power3(mst22) + 6*mst12*power2(DeltaInv(mst22,mst22,mh2)
     )*power3(mst22) + 28*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) + 10*mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,
     mst12)) + 6*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) -
     28*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 6*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 6*mst24*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) + 10*mst12*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 6*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mH2) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(
     mst12)) - power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(
     mst12)) - 6*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     24*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 6*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 24*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 6*mst22*power3(DeltaInv(
     mst22,mh2,mst12))*power5(mh2) - 6*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2) + 24*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*
     mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*mst12) - (mu2*
     power2(cb)*power2(Pi)*power2(sa)*power4(snt)*(2*mst12*mst22*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + 6*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*
     mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*mst24*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + mh2*mst12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 6*
     mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 6*mst12*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 28*mst12*mst24*power3(mH2)*
     power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*power2(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*power3(mh2)*power3(DeltaInv(
     mst22,mh2,mst12)) - 10*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 6*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 10*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) -
     6*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 10*mst12*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mst24*power3(DeltaInv(
     mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 24*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) + 6*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 6*
     mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 6*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) - 24*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/(6.*mst12*power2(sb)) - (2*mu2*log(mH2/Q2)*log(mst12/Q2)*power2(
     snt)*(power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*
     mst22 - mst24 + DeltaInv(mH2,mst22,mst12)*(-5*(mst12*mst22 + mst24)*
     power2(mH2) + 3*mst22*power3(mH2) + (-mst12 + mst22)*power3(mst22) + mH2*(
     -10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(mH2,mst12,mst12))*(
     mH2*mst12*mst14 + 4*power4(mst12)) + 16*mH2*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12)))/mst12 + (4*ca*cb*mu2*sa*log(mH2/Q2)*log(mst12/Q2)*
     power2(snt)*(power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst22 -
     mst12*mst22 - mst24 + DeltaInv(mH2,mst22,mst12)*(-5*(mst12*mst22 + mst24)*
     power2(mH2) + 3*mst22*power3(mH2) + (-mst12 + mst22)*power3(mst22) + mH2*(
     -10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(mH2,mst12,mst12))*(
     mH2*mst12*mst14 + 4*power4(mst12)) + 16*mH2*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12)))/(mst12*sb) + (2*mu2*log(mH2/Q2)*log(mst12/Q2)*
     power2(sa)*power2(snt)*(power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(-3*
     mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(mH2,mst22,mst12)*(-5*(mst12*
     mst22 + mst24)*power2(mH2) + 3*mst22*power3(mH2) + (-mst12 + mst22)*
     power3(mst22) + mH2*(-10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + 16*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/mst12 - (2*mu2*log(mH2/Q2)*log(
     mst12/Q2)*power2(cb)*power2(sa)*power2(snt)*(power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(mH2,
     mst22,mst12)*(-5*(mst12*mst22 + mst24)*power2(mH2) + 3*mst22*power3(mH2) +
     (-mst12 + mst22)*power3(mst22) + mH2*(-10*mst12*mst24 + power3(mst22)))) +
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) +
     16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*power2(sb)
     ) + (2*mu2*log(mH2/Q2)*log(mst12/Q2)*power4(snt)*(power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(
     mH2,mst22,mst12)*(-5*(mst12*mst22 + mst24)*power2(mH2) + 3*mst22*power3(
     mH2) + (-mst12 + mst22)*power3(mst22) + mH2*(-10*mst12*mst24 + power3(
     mst22)))) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(
     mst12)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/mst12 -
     (4*ca*cb*mu2*sa*log(mH2/Q2)*log(mst12/Q2)*power4(snt)*(power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(
     mH2,mst22,mst12)*(-5*(mst12*mst22 + mst24)*power2(mH2) + 3*mst22*power3(
     mH2) + (-mst12 + mst22)*power3(mst22) + mH2*(-10*mst12*mst24 + power3(
     mst22)))) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(
     mst12)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*
     sb) - (2*mu2*log(mH2/Q2)*log(mst12/Q2)*power2(sa)*power4(snt)*(power2(mH2)
     *power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*mst22 - mst24 +
     DeltaInv(mH2,mst22,mst12)*(-5*(mst12*mst22 + mst24)*power2(mH2) + 3*mst22*
     power3(mH2) + (-mst12 + mst22)*power3(mst22) + mH2*(-10*mst12*mst24 +
     power3(mst22)))) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*
     power4(mst12)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/
     mst12 + (2*mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*power2(sa)*power4(snt)
     *(power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*
     mst22 - mst24 + DeltaInv(mH2,mst22,mst12)*(-5*(mst12*mst22 + mst24)*
     power2(mH2) + 3*mst22*power3(mH2) + (-mst12 + mst22)*power3(mst22) + mH2*(
     -10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(mH2,mst12,mst12))*(
     mH2*mst12*mst14 + 4*power4(mst12)) + 16*mH2*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12)))/(mst12*power2(sb)) - (mu2*power2(snt)*power2(log(
     mH2/Q2))*(-2*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*
     power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(
     mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) + power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(
     mH2,mst22,mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))
     *power5(mst12)))/mst12 + (2*ca*cb*mu2*sa*power2(snt)*power2(log(mH2/Q2))*(
     -2*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*power3(mH2)*(-
     2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,
     mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/(mst12*sb) + (mu2*power2(sa)*power2(snt)*power2(log(mH2/Q2))*(-2*
     mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*power3(mH2)*(-2*
     mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,
     mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/mst12 - (mu2*power2(cb)*power2(sa)*power2(snt)*power2(log(mH2/Q2)
     )*(-2*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*power3(mH2)*
     (-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) +
     power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) + power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(
     mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,
     mst22,mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*
     power5(mst12)))/(mst12*power2(sb)) + (mu2*power2(log(mH2/Q2))*power4(snt)*
     (-2*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*power3(mH2)*(-
     2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,
     mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/mst12 - (2*ca*cb*mu2*sa*power2(log(mH2/Q2))*power4(snt)*(-2*
     mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*power3(mH2)*(-2*
     mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,
     mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/(mst12*sb) - (mu2*power2(sa)*power2(log(mH2/Q2))*power4(snt)*(-2*
     mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*power3(mH2)*(-2*
     mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,
     mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/mst12 + (mu2*power2(cb)*power2(sa)*power2(log(mH2/Q2))*power4(
     snt)*(-2*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*power3(
     mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) +
     power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) + power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(
     mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,
     mst22,mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*
     power5(mst12)))/(mst12*power2(sb)) + (mu2*power2(Pi)*power4(snt)*(-2*(3*
     mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(
     mst12)) + 2*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*(-14*mH2*mst12*
     mst24 - 5*mst12*mst22*power2(mH2) - 3*mst24*power2(mH2) + 3*mst22*power3(
     mH2) - 3*mH2*power3(mst22) - 5*mst12*power3(mst22) + 3*power4(mst22)) +
     mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 6*power3(mst22) + 24*
     mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 24*mH2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(6.*mst12) + (power2(Pi)*power2(sa)*(-(
     mst12*mt2*(mh2 + 5*mt2)*DeltaInv(mh2,mt2,mt2)) + mH2*mst12*mt2*DeltaInv(
     mH2,mt2,mt2) + 5*mst12*DeltaInv(mH2,mt2,mt2)*power2(mt2) - 2*mH2*mst12*
     mst14*mt2*power2(DeltaInv(mH2,mst12,mst12)) + mst12*mst22*mu2*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + 3*mst24*mu2*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 2*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,
     mst22,mst22)) - mst12*mst22*mu2*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 3*mst24*mu2*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 2*
     mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) - 3*mst22*mu2*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 3*mst22*mu2*power2(
     DeltaInv(mH2,mst22,mst12))*power3(mH2) + 2*mst12*mt2*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) - 12*mst12*mt2*power2(
     DeltaInv(mH2,mst22,mst22))*power3(mst22) + 12*mst12*mt2*power2(DeltaInv(
     mst22,mst22,mh2))*power3(mst22) - 18*mh2*mst12*power2(DeltaInv(mh2,mt2,
     mt2))*power3(mt2) + 18*mH2*mst12*power2(DeltaInv(mH2,mt2,mt2))*power3(mt2)
     + 14*mst12*mst24*mu2*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 5*
     mst12*mu2*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 3*
     mu2*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 14*
     mst12*mst24*mu2*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 5*mst12*
     mu2*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 3*mu2*
     power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 5*mst12*
     mst22*mu2*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 3*mst24*mu2*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 5*mst12*mst22*mu2*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 3*mst24*mu2*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) - 12*mt2*power2(DeltaInv(mH2,mst12,mst12))*
     power4(mst12) - 3*mu2*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*
     power4(mst22) - 48*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) + 3*mu2*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22)
     + 48*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 3*
     mst22*mu2*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 3*mst22*mu2*
     power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 48*mh2*mt2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) - 48*mH2*mt2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(12.*mst12) + (ca*cb*mu2*sa*power2(log(mH2/
     Q2))*((mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2)
     + 2*mst12*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(
     mst12)) - (mst12 - mst22)*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22
     + mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) - 2*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*
     power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 32*mH2*
     mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*(mst12 -
     mst22)*sb) - (power2(sa)*power2(log(mH2/Q2))*(-(mst22*mu2*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2)) + 2*mst12*mt2*power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst14 + 4*power3(mst12)) + mu2*power3(mH2)*(-2*mst12*mst24 -
     2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*power3(
     DeltaInv(mH2,mst22,mst12)) + mst12*mt2*(-(DeltaInv(mH2,mt2,mt2)*(mH2 + 3*
     mt2 + 12*mH2*DeltaInv(mH2,mt2,mt2)*power2(mt2))) + 2*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22)) + 32*mH2*power3(DeltaInv(mH2,
     mst22,mst22))*power4(mst22)) + 32*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12)
     )*power5(mst12)))/(2.*mst12) + (ca*cb*mu2*sa*power2(Pi)*(-2*mh2*mst12*
     mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)) + 2*mH2*mst12*mst14*mt2*
     power2(DeltaInv(mH2,mst12,mst12)) + mst12*(mst12 - mst22)*mst22*power2(
     mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 3*(mst12 - mst22)*mst24*power2(
     mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 2*mH2*mst12*mst24*mt2*power2(
     DeltaInv(mH2,mst22,mst22)) - mst12*(mst12 - mst22)*mst22*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 3*(mst12 - mst22)*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + 2*mh2*mst12*mst24*mt2*power2(DeltaInv(
     mst22,mst22,mh2)) - 3*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 3*(mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,
     mst12))*power3(mH2) - 12*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 12*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 14*mst12*(mst12 - mst22)*mst24*power3(mH2)*power3(DeltaInv(mH2,
     mst22,mst12)) + 5*mst12*(mst12 - mst22)*power2(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) + 3*(mst12 - mst22)*power3(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) - 14*mst12*(mst12 - mst22)*mst24*power3(
     mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 5*mst12*(mst12 - mst22)*power2(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 3*(mst12 - mst22)*
     power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 5*mst12*(
     mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 3*(
     mst12 - mst22)*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 5*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2)
     + 3*(mst12 - mst22)*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) -
     12*mt2*power2(DeltaInv(mh2,mst12,mst12))*power4(mst12) + 12*mt2*power2(
     DeltaInv(mH2,mst12,mst12))*power4(mst12) - 3*(mst12 - mst22)*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 48*mH2*mst12*mt2*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + 3*(mst12 - mst22)*power2(mh2)*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 48*mh2*mst12*mt2*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22) + 3*(mst12 - mst22)*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mh2) - 3*(mst12 - mst22)*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mH2) - 48*mh2*mt2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12) + 48*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))
     *power5(mst12)))/(6.*mst12*(mst12 - mst22)*sb) + (ca*cb*mu2*sa*log(mH2/Q2)
     *log(mst12/Q2)*(4*mst12*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 +
     4*power3(mst12)) + (mst12 - mst22)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12))*(3*mH2*mst22 + mst12*mst22 + mst24 + DeltaInv(mH2,mst22,mst12)*(5*
     (mst12*mst22 + mst24)*power2(mH2) - 3*mst22*power3(mH2) + (mst12 - mst22)*
     power3(mst22) - mH2*(-10*mst12*mst24 + power3(mst22)))) + 64*mH2*mt2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*(mst12 - mst22)*
     sb) - (log(mH2/Q2)*log(mst12/Q2)*power2(sa)*(4*mst12*mt2*power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12)) + mu2*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(
     mH2,mst22,mst12)*(-5*(mst12*mst22 + mst24)*power2(mH2) + 3*mst22*power3(
     mH2) + (-mst12 + mst22)*power3(mst22) + mH2*(-10*mst12*mst24 + power3(
     mst22)))) + 64*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(
     2.*mst12) - (mu2*power2(cb)*power2(sa)*power2(log(mH2/Q2))*((mst12 -
     mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 4*mst12*mt2*
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12)) - (mst12 -
     mst22)*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*
     power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) - 4*mst12*
     mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*
     mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 64*mH2*mt2*power3(DeltaInv(
     mH2,mst12,mst12))*power5(mst12)))/(2.*mst12*(mst12 - mst22)*power2(sb)) -
     (mu2*power2(cb)*power2(Pi)*power2(sa)*(-4*mh2*mst12*mst14*mt2*power2(
     DeltaInv(mh2,mst12,mst12)) + 4*mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,
     mst12,mst12)) + mst12*(mst12 - mst22)*mst22*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) + 3*(mst12 - mst22)*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) - 4*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)
     ) - mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 3*(mst12 - mst22)*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + 4*mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) - 3*(
     mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 3*(
     mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 24*
     mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 24*mst12*mt2*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 14*mst12*(mst12 - mst22)
     *mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 5*mst12*(mst12 -
     mst22)*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 3*(
     mst12 - mst22)*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12))
     - 14*mst12*(mst12 - mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 5*mst12*(mst12 - mst22)*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 3*(mst12 - mst22)*power3(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) - 5*mst12*(mst12 - mst22)*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 3*(mst12 - mst22)*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 5*mst12*(mst12 - mst22)*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 3*(mst12 - mst22)*mst24*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) - 24*mt2*power2(DeltaInv(
     mh2,mst12,mst12))*power4(mst12) + 24*mt2*power2(DeltaInv(mH2,mst12,mst12))
     *power4(mst12) - 3*(mst12 - mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 96*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))
     *power4(mst22) + 3*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) + 96*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))
     *power4(mst22) + 3*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))
     *power5(mh2) - 3*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2) - 96*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) +
     96*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(12.*mst12*(
     mst12 - mst22)*power2(sb)) - (mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*
     power2(sa)*(8*mst12*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*
     power3(mst12)) + (mst12 - mst22)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12))*(3*mH2*mst22 + mst12*mst22 + mst24 + DeltaInv(mH2,mst22,mst12)*(5*
     (mst12*mst22 + mst24)*power2(mH2) - 3*mst22*power3(mH2) + (mst12 - mst22)*
     power3(mst22) - mH2*(-10*mst12*mst24 + power3(mst22)))) + 128*mH2*mt2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(2.*mst12*(mst12 -
     mst22)*power2(sb)) + (ca*cb*mu2*sa*(2*mh2*mst12*mt2*DeltaInv(mh2,mst12,
     mst12) - 2*mH2*mst12*mt2*DeltaInv(mH2,mst12,mst12) - mH2*mst12*(mst12 -
     mst22)*DeltaInv(mH2,mst22,mst12) + 2*mH2*mst12*mt2*DeltaInv(mH2,mst22,
     mst22) + mh2*mst12*(mst12 - mst22)*DeltaInv(mst22,mh2,mst12) - 2*mh2*
     mst12*mt2*DeltaInv(mst22,mst22,mh2) - 84*mh2*mst12*mst14*mt2*power2(
     DeltaInv(mh2,mst12,mst12)) + 84*mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,
     mst12,mst12)) + 42*mst12*(mst12 - mst22)*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 84*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,
     mst22,mst22)) - 42*mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 126*(mst12 - mst22)*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + 84*mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,
     mst22,mh2)) - 126*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 126*(mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*
     power3(mH2) - 504*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*power3(
     mst22) + 504*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) +
     588*mst12*(mst12 - mst22)*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) + 210*mst12*(mst12 - mst22)*power2(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*power3(mH2)*power3(mst22)
     *power3(DeltaInv(mH2,mst22,mst12)) - 588*mst12*(mst12 - mst22)*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 210*mst12*(mst12 - mst22)*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 126*(mst12 -
     mst22)*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 210*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2)
     - 126*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2)
     + 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power4(mH2) + 126*(mst12 - mst22)*mst24*power3(DeltaInv(mH2,mst22,mst12))*
     power4(mH2) - 504*mt2*power2(DeltaInv(mh2,mst12,mst12))*power4(mst12) +
     504*mt2*power2(DeltaInv(mH2,mst12,mst12))*power4(mst12) - 126*(mst12 -
     mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 2016*
     mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 126*(mst12
     - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) +
     2016*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 126*(
     mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 126*(
     mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) - 2016*
     mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 2016*mH2*mt2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*mst12*(mst12 -
     mst22)*sb) - (mu2*power2(cb)*power2(sa)*(4*mh2*mst12*mt2*DeltaInv(mh2,
     mst12,mst12) - 4*mH2*mst12*mt2*DeltaInv(mH2,mst12,mst12) - mH2*mst12*(
     mst12 - mst22)*DeltaInv(mH2,mst22,mst12) + 4*mH2*mst12*mt2*DeltaInv(mH2,
     mst22,mst22) + mh2*mst12*(mst12 - mst22)*DeltaInv(mst22,mh2,mst12) - 4*
     mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) - 168*mh2*mst12*mst14*mt2*power2(
     DeltaInv(mh2,mst12,mst12)) + 168*mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,
     mst12,mst12)) + 42*mst12*(mst12 - mst22)*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 168*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,
     mst22,mst22)) - 42*mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 126*(mst12 - mst22)*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + 168*mh2*mst12*mst24*mt2*power2(DeltaInv(
     mst22,mst22,mh2)) - 126*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 126*(mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,
     mst12))*power3(mH2) - 1008*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 1008*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 588*mst12*(mst12 - mst22)*mst24*power3(mH2)*power3(DeltaInv(mH2,
     mst22,mst12)) + 210*mst12*(mst12 - mst22)*power2(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*power3(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 588*mst12*(mst12 -
     mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 210*mst12*(
     mst12 - mst22)*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12))
     - 126*(mst12 - mst22)*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12)
     )*power4(mh2) - 126*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mh2,mst12)
     )*power4(mh2) + 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 126*(mst12 - mst22)*mst24*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) - 1008*mt2*power2(DeltaInv(mh2,mst12,mst12))*power4(
     mst12) + 1008*mt2*power2(DeltaInv(mH2,mst12,mst12))*power4(mst12) - 126*(
     mst12 - mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22)
     - 4032*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) +
     126*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 4032*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) + 126*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 126*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2) - 4032*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)
     + 4032*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(12.*
     mst12*(mst12 - mst22)*power2(sb)) - (log(mst12/Q2)*power2(sa)*(-4*(mh2 -
     6*mst12)*mst12*mst22*mt2*DeltaInv(mh2,mst12,mst12) + 4*(mH2 - 6*mst12)*
     mst12*mst22*mt2*DeltaInv(mH2,mst12,mst12) + 96*mst12*mst22*mt2*power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) - 96*mst12*mst22*
     mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(mst12)) + mu2*
     (-mh2 + mH2 + DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) -
     (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)) -
     DeltaInv(mH2,mst22,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*
     mst22)*power2(mH2) + 5*mH2*power2(mst22) + power3(mH2)) - 6*mh2*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 +
     23*mst24) - (mst12 - 23*mst22)*power2(mh2) + power3(mh2) + power3(mst22))
     + 6*mH2*mst22*power2(DeltaInv(mH2,mst22,mst12))*(-(mst12*mst24) + mH2*(6*
     mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mH2) + power3(mH2) +
     power3(mst22)) - 72*mst22*power2(mH2)*(-((3*mst12*mst22 + 2*mst24)*power2(
     mH2)) + 2*mst22*power3(mH2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mH2*(
     5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 72*
     mst22*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*
     power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12))) + 1152*mh2*mst22*mt2*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 1152*mH2*mst22*mt2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(24.*mst12*mst22) - (ca*
     cb*mu2*sa*log(mst12/Q2)*(4*(mh2 - 6*mst12)*mst12*mst22*mt2*DeltaInv(mh2,
     mst12,mst12) - 4*(mH2 - 6*mst12)*mst12*mst22*mt2*DeltaInv(mH2,mst12,mst12)
     - 96*mst12*mst22*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*
     power3(mst12)) + 96*mst12*mst22*mt2*power2(DeltaInv(mH2,mst12,mst12))*(
     mH2*mst14 + 3*power3(mst12)) + (mst12 - mst22)*(-mh2 + mH2 + DeltaInv(
     mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*
     power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)) - DeltaInv(mH2,mst22,
     mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mH2) +
     5*mH2*power2(mst22) + power3(mH2)) - 6*mh2*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 + 23*mst24) - (mst12 -
     23*mst22)*power2(mh2) + power3(mh2) + power3(mst22)) + 6*mH2*mst22*power2(
     DeltaInv(mH2,mst22,mst12))*(-(mst12*mst24) + mH2*(6*mst12*mst22 + 23*
     mst24) - (mst12 - 23*mst22)*power2(mH2) + power3(mH2) + power3(mst22)) -
     72*mst22*power2(mH2)*(-((3*mst12*mst22 + 2*mst24)*power2(mH2)) + 2*mst22*
     power3(mH2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mH2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 72*mst22*power2(mh2)*(
     -((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*power3(mh2) + (-3*mst12
     + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12))) - 1152*mh2*mst22*mt2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12) + 1152*mH2*mst22*mt2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(12.*mst12*(mst12 - mst22)*mst22*sb) + (mu2*
     log(mst12/Q2)*power2(cb)*power2(sa)*(8*(mh2 - 6*mst12)*mst12*mst22*mt2*
     DeltaInv(mh2,mst12,mst12) - 8*(mH2 - 6*mst12)*mst12*mst22*mt2*DeltaInv(
     mH2,mst12,mst12) - 192*mst12*mst22*mt2*power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 3*power3(mst12)) + 192*mst12*mst22*mt2*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 3*power3(mst12)) + (mst12 - mst22)*(-mh2 + mH2
     + DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*
     mst22)*power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)) - DeltaInv(mH2,
     mst22,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(
     mH2) + 5*mH2*power2(mst22) + power3(mH2)) - 6*mh2*mst22*power2(DeltaInv(
     mst22,mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 + 23*mst24) - (
     mst12 - 23*mst22)*power2(mh2) + power3(mh2) + power3(mst22)) + 6*mH2*
     mst22*power2(DeltaInv(mH2,mst22,mst12))*(-(mst12*mst24) + mH2*(6*mst12*
     mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mH2) + power3(mH2) + power3(
     mst22)) - 72*mst22*power2(mH2)*(-((3*mst12*mst22 + 2*mst24)*power2(mH2)) +
     2*mst22*power3(mH2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mH2*(5*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 72*mst22*
     power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*power3(
     mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,mh2,mst12))) - 2304*mh2*mst22*mt2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) + 2304*mH2*mst22*mt2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(24.*mst12*(mst12 - mst22)*
     mst22*power2(sb)) - (log(mst12/Q2)*power4(snt)*(4*mst12*mst22 - mH2*mu2 -
     mst12*mu2 - mst22*mu2 + mst12*(-mH2 + 6*mst12)*mst22*mu2*DeltaInv(mH2,
     mst12,mst12) - 4*power2(mst12) + mu2*DeltaInv(mH2,mst22,mst12)*(mst22*(-(
     mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mH2) + 5*mH2*power2(
     mst22) + power3(mH2)) + 24*mst12*mst22*mu2*power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst14 + 3*power3(mst12)) - 6*mH2*mst22*mu2*power2(DeltaInv(
     mH2,mst22,mst12))*(-(mst12*mst24) + mH2*(6*mst12*mst22 + 23*mst24) - (
     mst12 - 23*mst22)*power2(mH2) + power3(mH2) + power3(mst22)) + 72*mst22*
     mu2*power2(mH2)*(-((3*mst12*mst22 + 2*mst24)*power2(mH2)) + 2*mst22*
     power3(mH2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mH2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 288*mH2*mst22*mu2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*mst12*mst22) + (mu2*
     log(mst12/Q2)*power2(cb)*power2(snt)*(-2*mh2*msb12 - 2*msb12*mst12 - 2*
     msb12*mst22 - 2*msb12*(mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,
     mst12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) - msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) + 6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 18*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) + 2*msb12*DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*
     mst22) + mst24) - (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(mst22) +
     power3(mh2)) + 48*msb12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 3*power3(mst12)) - 12*mh2*msb12*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 + 23*mst24) - (mst12 -
     23*mst22)*power2(mh2) + power3(mh2) + power3(mst22)) - 6*mst22*power2(
     mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 144*msb12*mst22*
     power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*power3(
     mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + 144*msb12*mst12*mst14*mst22*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*msb12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*mst12*mst14*mst22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 576*mh2*msb12*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 72*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)))/(12.*msb12*mst12*mst22*
     power2(sb)) + (DeltaInv(mt2,mu2,mst12)*log(mst12/Q2)*log(mt2/Q2)*power2(
     mu2)*(mst12 + DeltaInv(mt2,mu2,mst12)*(mst12*mu2*(4*mt2 + mu2) + 2*mst14*(
     2*mt2 + 7*mu2) + power3(mst12)) - 2*power2(DeltaInv(mt2,mu2,mst12))*(
     mst14*(-15*mt2 + 4*mu2)*power2(mu2) - 5*mu2*(3*mt2 + 2*mu2)*power3(mst12)
     + mst12*(-mt2 + mu2)*power3(mu2) - (mt2 - 4*mu2)*power4(mst12) + power5(
     mst12))))/mt2 + (power2(log(mst22/Q2))*(-(msb12*mst24*mt2*mu2*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12))) - msb12*mst24*mt2*mu2*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12)) + msb12*mt2*mu2*power2(mH2)*(mst24*
     power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + msb12*mt2*mu2*power2(
     mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(mst22) - 2*mA2*(mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) + 2*mst12*(-(
     mst24*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + mt2*mu2*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + msb12*(
     mh2*mst24*power2(mt2)*power2(DeltaInv(mst22,mst22,mh2)) + power2(mu2)*
     power2(DeltaInv(mst22,mt2,mu2))*(mst24*mt2 + 3*mst24*mu2 + power3(mst22) +
     DeltaInv(mst22,mt2,mu2)*(mst24*(5*mt2 - 3*mu2)*power2(mu2) + 5*mu2*(2*mt2
     + mu2)*power3(mst22) + (mt2 - mu2)*power4(mst22) - power5(mst22)))))))/(2.
     *msb12*mst12*mt2) - (14*power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*
     power5(mst22))/mt2 + (ca*cb*mt2*mu2*sa*log(mH2/Q2)*(-mst12 + mst22 + (mH2
     - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mst12*mst22*(-mH2 + 6*
     mst22)*DeltaInv(mH2,mst22,mst22) + 12*mst12*mst22*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) - 12*mst12*mst22*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 12*power3(mst22)) + 576*mH2*mst22*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) - 576*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(3.*mst12*(mst12 - mst22)*
     mst22*sb) - (mt2*mu2*log(mH2/Q2)*power2(cb)*power2(sa)*(-mst12 + mst22 + (
     mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mst12*mst22*(-mH2 +
     6*mst22)*DeltaInv(mH2,mst22,mst22) + 12*mst12*mst22*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) - 12*mst12*mst22*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 12*power3(mst22)) + 576*mH2*mst22*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) - 576*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(3.*mst12*(mst12 - mst22)*
     mst22*power2(sb)) + (mu2*log(mH2/Q2)*power2(snt)*(mH2 + mst12 + mst22 + (
     mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*
     DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*
     mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 +
     power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*
     mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,
     mst22,mst22))*power4(mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12) + 576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power5(mst22)))/(6.*mst12*mst22) - (ca*cb*mu2*sa*log(mH2/Q2)*power2(snt)*(
     mH2 + mst12 + mst22 + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,
     mst12) + mH2*mst12*mst22*DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 + 5*
     mst22)*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,mst22,
     mst22)*power2(mst22) + 6*mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 +
     14*mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 12*
     mH2*mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*
     mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*
     power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) + 144*
     mst12*power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 576*mH2*mst22*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + 576*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(3.*mst12*mst22*sb) - (mu2*log(
     mH2/Q2)*power2(sa)*power2(snt)*(mH2 + mst12 + mst22 + (mH2 - 6*mst12)*
     mst12*mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*DeltaInv(mH2,
     mst22,mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,mst12)*power2(
     mH2) - 6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*mst22*power2(
     mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 + power2(mH2))*
     power2(DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*mst12*
     mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*
     power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,mst22,
     mst22))*power4(mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,mst12))*
     power5(mst12) + 576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power5(
     mst22)))/(6.*mst12*mst22) + (mu2*log(mH2/Q2)*power2(cb)*power2(sa)*power2(
     snt)*(mH2 + mst12 + mst22 + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,
     mst12,mst12) + mH2*mst12*mst22*DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 +
     5*mst22)*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,
     mst22,mst22)*power2(mst22) + 6*mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(
     mst12 + 14*mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12))
     + 12*mH2*mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*
     mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) +
     72*mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) +
     mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) +
     144*mst12*power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 576*mH2*mst22*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + 576*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(6.*mst12*mst22*power2(sb)) - (
     mu2*log(mH2/Q2)*power4(snt)*(mH2 + mst12 + mst22 + (mH2 - 6*mst12)*mst12*
     mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*DeltaInv(mH2,mst22,
     mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,mst12)*power2(mH2) -
     6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*mst22*power2(mH2)*(-3*
     mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 + power2(mH2))*power2(
     DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*power2(DeltaInv(mH2,
     mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(
     mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(
     mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) +
     576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(6.*mst12*
     mst22) + (ca*cb*mu2*sa*log(mH2/Q2)*power4(snt)*(mH2 + mst12 + mst22 + (mH2
     - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*
     DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*
     mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 +
     power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*
     mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,
     mst22,mst22))*power4(mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12) + 576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power5(mst22)))/(3.*mst12*mst22*sb) + (mu2*log(mH2/Q2)*power2(sa)*power4(
     snt)*(mH2 + mst12 + mst22 + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,
     mst12,mst12) + mH2*mst12*mst22*DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 +
     5*mst22)*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,
     mst22,mst22)*power2(mst22) + 6*mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(
     mst12 + 14*mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12))
     + 12*mH2*mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*
     mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) +
     72*mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) +
     mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) +
     144*mst12*power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 576*mH2*mst22*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + 576*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(6.*mst12*mst22) - (mu2*log(
     mH2/Q2)*power2(cb)*power2(sa)*power4(snt)*(mH2 + mst12 + mst22 + (mH2 - 6*
     mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*DeltaInv(
     mH2,mst22,mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,mst12)*
     power2(mH2) - 6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*mst22*
     power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 + power2(mH2)
     )*power2(DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*mst12*
     mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*
     power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,mst22,
     mst22))*power4(mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,mst12))*
     power5(mst12) + 576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power5(
     mst22)))/(6.*mst12*mst22*power2(sb)) + (ca*cb*mu2*sa*log(mH2/Q2)*(mH2*
     mst12 - mH2*mst22 + 4*mst12*mt2 - 4*mst22*mt2 - 4*(mH2 - 6*mst12)*mst12*
     mst22*mt2*DeltaInv(mH2,mst12,mst12) + 4*mH2*mst12*mst22*mt2*DeltaInv(mH2,
     mst22,mst22) + (-mH2 + mst12 - 5*mst22)*(mst12 - mst22)*DeltaInv(mH2,
     mst22,mst12)*power2(mH2) - 24*mst12*mt2*DeltaInv(mH2,mst22,mst22)*power2(
     mst22) + 6*(mst12 - mst22)*mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12
     + 14*mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 48*
     mH2*mst12*mst22*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) - 48*mst12*
     mst22*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12))
     + 72*(mst12 - mst22)*mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*
     mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,
     mst22,mst12)) + 576*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) - 2304*mH2*mst22*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12) + 2304*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power5(
     mst22)))/(12.*mst12*(mst12 - mst22)*mst22*sb) - (mu2*log(mH2/Q2)*power2(
     cb)*power2(sa)*(mH2*mst12 - mH2*mst22 + 8*mst12*mt2 - 8*mst22*mt2 - 8*(mH2
     - 6*mst12)*mst12*mst22*mt2*DeltaInv(mH2,mst12,mst12) + 8*mH2*mst12*mst22*
     mt2*DeltaInv(mH2,mst22,mst22) + (-mH2 + mst12 - 5*mst22)*(mst12 - mst22)*
     DeltaInv(mH2,mst22,mst12)*power2(mH2) - 48*mst12*mt2*DeltaInv(mH2,mst22,
     mst22)*power2(mst22) + 6*(mst12 - mst22)*mst22*power2(mH2)*(-3*mst12*mst22
     - mH2*(mst12 + 14*mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,
     mst12)) + 96*mH2*mst12*mst22*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) -
     96*mst12*mst22*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*
     power3(mst12)) + 72*(mst12 - mst22)*mst22*power3(mH2)*(-2*mst12*mst24 - 2*
     mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*power3(
     DeltaInv(mH2,mst22,mst12)) + 1152*mst12*mt2*power2(DeltaInv(mH2,mst22,
     mst22))*power4(mst22) - 4608*mH2*mst22*mt2*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12) + 4608*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,
     mst22))*power5(mst22)))/(24.*mst12*(mst12 - mst22)*mst22*power2(sb)) + (
     power4(snt)*(8*mst12*mst22 - mH2*mst12*mst22*mu2*DeltaInv(mH2,mst12,mst12)
     + 2*mH2*mst12*mst22*mu2*DeltaInv(mH2,mst22,mst12) - mH2*mst12*mst22*mu2*
     DeltaInv(mH2,mst22,mst22) - 4*power2(mst12) - 4*power2(mst22) - 84*mst22*(
     3*mH2*mst22 + mst12*mst22 + 3*mst24)*mu2*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + 42*mH2*mst12*mst22*mst24*mu2*power2(DeltaInv(mH2,mst22,
     mst22)) + 42*mst12*mst22*mu2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14
     + 6*power3(mst12)) + 252*mst12*mu2*power2(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) + 84*mst22*mu2*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))
     *(-14*mH2*mst12*mst24 - 5*mst12*mst22*power2(mH2) - 3*mst24*power2(mH2) +
     3*mst22*power3(mH2) - 3*mH2*power3(mst22) - 5*mst12*power3(mst22) + 3*
     power4(mst22)) + 1008*mH2*mst22*mu2*power3(DeltaInv(mH2,mst12,mst12))*
     power5(mst12) + 1008*mH2*mst12*mu2*power3(DeltaInv(mH2,mst22,mst22))*
     power5(mst22)))/(6.*mst12*mst22) + (log(mH2/Q2)*power2(sa)*(18*mH2*mst12*
     mst22 + 12*mH2*mst12*mt2 + 12*mH2*mst22*mt2 + 8*mst12*mst22*mt2 + 12*mH2*(
     mH2 - 6*mst12)*mst12*mst22*mt2*DeltaInv(mH2,mst12,mst12) + 3*mst12*power2(
     mH2) + 3*mst22*power2(mH2) + 3*mu2*power2(mH2) + 12*mst12*mst22*mt2*
     DeltaInv(mH2,mst22,mst22)*power2(mH2) - 116*mst12*mst22*mt2*DeltaInv(mH2,
     mt2,mt2)*power2(mH2) - 72*mH2*mst12*mt2*DeltaInv(mH2,mst22,mst22)*power2(
     mst22) - 616*mH2*mst12*mst22*DeltaInv(mH2,mt2,mt2)*power2(mt2) + 144*
     mst12*mst22*mst24*mt2*power2(mH2)*power2(DeltaInv(mH2,mst22,mst22)) - 3*(
     mH2 - mst12 + 5*mst22)*mu2*DeltaInv(mH2,mst22,mst12)*power3(mH2) + 18*
     mst22*mu2*(-3*mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 + power2(mH2))*
     power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 144*mH2*mst12*mst22*mt2*
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) - 2592*
     mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mt2,mt2))*power3(mt2) + 216*
     mst22*mu2*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(
     mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) +
     1728*mH2*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 6912*
     mst22*mt2*power2(mH2)*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) +
     6912*mst12*mt2*power2(mH2)*power3(DeltaInv(mH2,mst22,mst22))*power5(mst22)
     ))/(72.*mH2*mst12*mst22) + (ca*cb*mu2*sa*log(mst22/Q2)*((mst12 - mst22)*(
     5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(mH2,mst22,mst12) + 4*mst12*(
     mH2 - 6*mst22)*mt2*DeltaInv(mH2,mst22,mst22) - 5*mh2*mst12*mst22*DeltaInv(
     mst22,mh2,mst12) - mst12*mst24*DeltaInv(mst22,mh2,mst12) + mst22*mst24*
     DeltaInv(mst22,mh2,mst12) - 4*mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) +
     24*mst12*mst22*mt2*DeltaInv(mst22,mst22,mh2) + mst22*DeltaInv(mst22,mh2,
     mst12)*power2(mst12) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) + 6*mh2*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mh2,mst12)) - 84*mst12*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + 84*mst22*mst24*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 6*mh2*mst24*power2(mst12)*power2(DeltaInv(mst22,mh2,
     mst12)) - 18*mst22*power2(mh2)*power2(mst12)*power2(DeltaInv(mst22,mh2,
     mst12)) + 18*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + 96*mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 6*power2(
     mst22)*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 6*mh2*mst12*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mst22) + 288*mst12*mt2*power2(DeltaInv(
     mst22,mst22,mh2))*power3(mst22) - 6*mH2*(mst12 - mst22)*power2(DeltaInv(
     mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 +
     mst22*power2(mH2) + power3(mst22)) - 96*mst12*mt2*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) - 72*(mst12 - mst22)*power2(
     mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*
     mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 144*mst24*
     power2(mst12)*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 144*power2(
     mh2)*power2(mst12)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 144*
     mst12*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 72*
     mst12*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 72*mst22*
     mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 6*mh2*power2(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) - 1152*mH2*mst12*mt2*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + 216*mst12*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1152*mh2*mst12*mt2*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22) - 72*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*power5(mst22)))/(12.*mst12*(-mst12 + mst22)*sb) + (mu2*
     log(mst22/Q2)*power2(cb)*power2(sa)*((mst12 - mst22)*(5*mH2*mst22 - mst12*
     mst22 + mst24)*DeltaInv(mH2,mst22,mst12) + 8*mst12*(mH2 - 6*mst22)*mt2*
     DeltaInv(mH2,mst22,mst22) - 5*mh2*mst12*mst22*DeltaInv(mst22,mh2,mst12) -
     mst12*mst24*DeltaInv(mst22,mh2,mst12) + mst22*mst24*DeltaInv(mst22,mh2,
     mst12) - 8*mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) + 48*mst12*mst22*mt2*
     DeltaInv(mst22,mst22,mh2) + mst22*DeltaInv(mst22,mh2,mst12)*power2(mst12)
     + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) - mst12*DeltaInv(mst22,
     mh2,mst12)*power2(mst22) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,
     mh2,mst12)) - 84*mst12*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + 84*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*mh2*
     mst24*power2(mst12)*power2(DeltaInv(mst22,mh2,mst12)) - 18*mst22*power2(
     mh2)*power2(mst12)*power2(DeltaInv(mst22,mh2,mst12)) + 18*mst12*power2(
     mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,mst12)) + 192*mh2*mst12*
     mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 6*power2(mst22)*power2(DeltaInv(
     mst22,mh2,mst12))*power3(mh2) + 6*mh2*mst12*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mst22) + 576*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*
     power3(mst22) - 6*mH2*(mst12 - mst22)*power2(DeltaInv(mH2,mst22,mst12))*(-
     3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*power2(mH2) +
     power3(mst22)) - 192*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*
     mst24 + 3*power3(mst22)) - 72*(mst12 - mst22)*power2(mH2)*(mst24*power2(
     mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst22*mst24*power3(
     mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 144*mst24*power2(mst12)*power3(
     mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 144*power2(mh2)*power2(mst12)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 144*mst12*power3(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 72*mst12*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 72*mst22*mst24*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*
     power4(mst22) - 2304*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) + 216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mst22) + 144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 2304*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power5(mst22)))/
     (24.*mst12*(mst12 - mst22)*power2(sb)) + (ca*cb*mu2*sa*log(mst12/Q2)*
     power2(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*mst12)*mst22*DeltaInv(mh2,mst12,
     mst12) + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) - mst22*
     mst24*DeltaInv(mH2,mst22,mst12) + mst22*mst24*DeltaInv(mst22,mh2,mst12) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mh2) + 5*mst22*DeltaInv(mst22,mh2,
     mst12)*power2(mh2) + mst12*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*
     mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*mH2*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,mst12)*power2(mst22) + 5*
     mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) - mst12*DeltaInv(mst22,mh2,
     mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,
     mst12)) + 138*mst22*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     36*mst12*power2(mH2)*power2(mst22)*power2(DeltaInv(mH2,mst22,mst12)) + 6*
     mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mh2,mst12)) - 138*mst22*mst24*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 36*mst12*power2(mh2)*
     power2(mst22)*power2(DeltaInv(mst22,mh2,mst12)) + DeltaInv(mst22,mh2,
     mst12)*power3(mh2) + 6*mst12*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) - 138*power2(mst22)*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mh2) - DeltaInv(mH2,mst22,mst12)*power3(mH2) - 6*mst12*mst22*power2(
     DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*mst22*power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) - 24*mst12*mst22*power2(
     DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(mst12)) + 720*mst12*
     mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) - 720*mst12*
     mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 6*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*mst22*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*power2(mst22)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*power2(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 216*
     mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) + 144*
     power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 216*mst12*
     power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) - 144*power3(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*power2(mst22)*
     power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*power3(DeltaInv(
     mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12) - 144*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mst22) + 144*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power5(
     mst22)))/(3.*mst12*mst22*sb) + (mu2*log(mst12/Q2)*power2(sa)*power2(snt)*(
     -mh2 + mH2 + mst12*(-mh2 + 6*mst12)*mst22*DeltaInv(mh2,mst12,mst12) + (mH2
     - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) - mst22*mst24*DeltaInv(
     mH2,mst22,mst12) + mst22*mst24*DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(
     mst22,mh2,mst12)*power2(mh2) + 5*mst22*DeltaInv(mst22,mh2,mst12)*power2(
     mh2) + mst12*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,
     mst22,mst12)*power2(mH2) - 5*mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) +
     mst12*DeltaInv(mH2,mst22,mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,
     mst12)*power2(mst22) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*
     mH2*mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(
     mst22)*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,
     mst12)*power3(mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*
     power3(mH2) + 138*power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(
     mH2) + 24*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*
     power3(mst12)) - 24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst14 + 3*power3(mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) - 144*mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) - 216*mst12*power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*
     mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(
     DeltaInv(mH2,mst22,mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) - 144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mst22) + 144*power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 144*power2(mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(
     mH2) + 288*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) -
     288*mH2*mst22*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*
     power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(6.*mst12*mst22) -
     (mu2*log(mst12/Q2)*power2(cb)*power2(sa)*power2(snt)*(-mh2 + mH2 + mst12*(
     -mh2 + 6*mst12)*mst22*DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*
     mst22*DeltaInv(mH2,mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) +
     mst22*mst24*DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*
     power2(mh2) + 5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*
     DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*
     power2(mH2) - 5*mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*
     DeltaInv(mH2,mst22,mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*
     power2(mst22) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*
     mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(
     mst22)*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,
     mst12)*power3(mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*
     power3(mH2) + 138*power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(
     mH2) + 24*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*
     power3(mst12)) - 24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst14 + 3*power3(mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) - 144*mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) - 216*mst12*power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*
     mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(
     DeltaInv(mH2,mst22,mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) - 144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mst22) + 144*power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 144*power2(mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(
     mH2) + 288*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) -
     288*mH2*mst22*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*
     power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(6.*mst12*mst22*
     power2(sb)) - (ca*cb*mu2*sa*log(mst12/Q2)*power4(snt)*(-mh2 + mH2 + mst12*
     (-mh2 + 6*mst12)*mst22*DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*
     mst22*DeltaInv(mH2,mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) +
     mst22*mst24*DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*
     power2(mh2) + 5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*
     DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*
     power2(mH2) - 5*mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*
     DeltaInv(mH2,mst22,mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*
     power2(mst22) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*
     mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(
     mst22)*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,
     mst12)*power3(mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*
     power3(mH2) + 138*power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(
     mH2) + 24*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*
     power3(mst12)) - 24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst14 + 3*power3(mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) - 144*mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) - 216*mst12*power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*
     mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(
     DeltaInv(mH2,mst22,mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) - 144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mst22) + 144*power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 144*power2(mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(
     mH2) + 288*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) -
     288*mH2*mst22*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*
     power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(3.*mst12*mst22*sb)
     - (mu2*log(mst12/Q2)*power2(sa)*power4(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*
     mst12)*mst22*DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*mst22*
     DeltaInv(mH2,mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) + mst22*
     mst24*DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*power2(
     mh2) + 5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*DeltaInv(mH2,
     mst22,mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) -
     5*mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,mst12)*power3(
     mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) -
     24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(
     mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*
     mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(6.*mst12*mst22) + (mu2*log(
     mst12/Q2)*power2(cb)*power2(sa)*power4(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*
     mst12)*mst22*DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*mst22*
     DeltaInv(mH2,mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) + mst22*
     mst24*DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*power2(
     mh2) + 5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*DeltaInv(mH2,
     mst22,mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) -
     5*mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,mst12)*power3(
     mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) -
     24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(
     mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*
     mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(6.*mst12*mst22*power2(sb)) - (
     ca*cb*mt2*mu2*sa*log(mh2/Q2)*(-mst12 + mst22 + (mh2 - 6*mst12)*mst12*
     mst22*DeltaInv(mh2,mst12,mst12) + mst12*mst22*(-mh2 + 6*mst22)*DeltaInv(
     mst22,mst22,mh2) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst14 + 12*power3(mst12)) - 12*mst12*mst22*power2(DeltaInv(mst22,mst22,
     mh2))*(mh2*mst24 + 12*power3(mst22)) + 576*mh2*mst22*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12) - 576*mh2*mst12*power3(DeltaInv(mst22,mst22,
     mh2))*power5(mst22)))/(3.*mst12*(mst12 - mst22)*mst22*sb) - (mt2*mu2*log(
     mh2/Q2)*power2(cb)*(-mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(
     mh2,mst12,mst12) + mst12*mst22*(-mh2 + 6*mst22)*DeltaInv(mst22,mst22,mh2)
     + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(
     mst12)) - 12*mst12*mst22*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 +
     12*power3(mst22)) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) - 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(
     mst22)))/(3.*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mt2*mu2*log(mh2/
     Q2)*power2(cb)*power2(sa)*(-mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*
     DeltaInv(mh2,mst12,mst12) + mst12*mst22*(-mh2 + 6*mst22)*DeltaInv(mst22,
     mst22,mh2) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 +
     12*power3(mst12)) - 12*mst12*mst22*power2(DeltaInv(mst22,mst22,mh2))*(mh2*
     mst24 + 12*power3(mst22)) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12)
     )*power5(mst12) - 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(
     mst22)))/(3.*mst12*(mst12 - mst22)*mst22*power2(sb)) + (ca*cb*mu2*sa*log(
     mh2/Q2)*power2(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*
     DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(mst22,mst22,mh2) - (
     mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*mst12*
     DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-3*mst12*
     mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(DeltaInv(
     mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,
     mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*
     power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22
     + mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,
     mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 576*
     mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 576*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(3.*mst12*mst22*sb) + (
     mu2*log(mh2/Q2)*power2(sa)*power2(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*
     mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(
     mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*
     power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*
     power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2)
     )*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*
     mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*
     power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,
     mh2))*power4(mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) + 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(
     mst22)))/(6.*mst12*mst22) + (mu2*log(mh2/Q2)*power2(cb)*power2(snt)*(mh2 +
     mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) +
     mh2*mst12*mst22*DeltaInv(mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*
     DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*
     power2(mst22) + 6*mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*
     mst22) + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*
     mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*
     mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*
     power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) + 144*
     mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 576*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 576*mh2*mst12*power3(
     DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(6.*mst12*mst22*power2(sb)) - (
     mu2*log(mh2/Q2)*power2(cb)*power2(sa)*power2(snt)*(mh2 + mst12 + mst22 + (
     mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*
     DeltaInv(mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,
     mst12)*power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*
     mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 +
     power2(mh2))*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(
     mst22,mst22,mh2))*power4(mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) + 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*
     power5(mst22)))/(6.*mst12*mst22*power2(sb)) - (ca*cb*mu2*sa*log(mh2/Q2)*
     power4(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(
     mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(mst22,mst22,mh2) - (mh2 -
     mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*mst12*DeltaInv(
     mst22,mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-3*mst12*mst22 -
     mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,
     mst12)) + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 12*
     mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(
     mst12)) + 72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,
     mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 576*
     mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 576*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(3.*mst12*mst22*sb) - (
     mu2*log(mh2/Q2)*power2(sa)*power4(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*
     mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(
     mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*
     power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*
     power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2)
     )*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*
     mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*
     power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,
     mh2))*power4(mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) + 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(
     mst22)))/(6.*mst12*mst22) - (mu2*log(mh2/Q2)*power2(cb)*power4(snt)*(mh2 +
     mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) +
     mh2*mst12*mst22*DeltaInv(mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*
     DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*
     power2(mst22) + 6*mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*
     mst22) + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*
     mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*
     mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*
     power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) + 144*
     mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 576*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 576*mh2*mst12*power3(
     DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(6.*mst12*mst22*power2(sb)) + (
     mu2*log(mh2/Q2)*power2(cb)*power2(sa)*power4(snt)*(mh2 + mst12 + mst22 + (
     mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*
     DeltaInv(mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,
     mst12)*power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*
     mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 +
     power2(mh2))*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(
     mst22,mst22,mh2))*power4(mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) + 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*
     power5(mst22)))/(6.*mst12*mst22*power2(sb)) - (ca*cb*mu2*sa*log(mh2/Q2)*(
     mh2*mst12 - mh2*mst22 + 4*mst12*mt2 - 4*mst22*mt2 - 4*(mh2 - 6*mst12)*
     mst12*mst22*mt2*DeltaInv(mh2,mst12,mst12) + 4*mh2*mst12*mst22*mt2*
     DeltaInv(mst22,mst22,mh2) + (-mh2 + mst12 - 5*mst22)*(mst12 - mst22)*
     DeltaInv(mst22,mh2,mst12)*power2(mh2) - 24*mst12*mt2*DeltaInv(mst22,mst22,
     mh2)*power2(mst22) + 6*(mst12 - mst22)*mst22*power2(mh2)*(-3*mst12*mst22 -
     mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,
     mst12)) + 48*mh2*mst12*mst22*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) -
     48*mst12*mst22*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*
     power3(mst12)) + 72*(mst12 - mst22)*mst22*power3(mh2)*(-2*mst12*mst24 - 2*
     mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*power3(
     DeltaInv(mst22,mh2,mst12)) + 576*mst12*mt2*power2(DeltaInv(mst22,mst22,
     mh2))*power4(mst22) - 2304*mh2*mst22*mt2*power3(DeltaInv(mh2,mst12,mst12))
     *power5(mst12) + 2304*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*
     power5(mst22)))/(12.*mst12*(mst12 - mst22)*mst22*sb) - (mu2*log(mh2/Q2)*
     power2(cb)*(mh2*mst12 - mh2*mst22 + 8*mst12*mt2 - 8*mst22*mt2 - 8*(mh2 -
     6*mst12)*mst12*mst22*mt2*DeltaInv(mh2,mst12,mst12) + 8*mh2*mst12*mst22*
     mt2*DeltaInv(mst22,mst22,mh2) + (-mh2 + mst12 - 5*mst22)*(mst12 - mst22)*
     DeltaInv(mst22,mh2,mst12)*power2(mh2) - 48*mst12*mt2*DeltaInv(mst22,mst22,
     mh2)*power2(mst22) + 6*(mst12 - mst22)*mst22*power2(mh2)*(-3*mst12*mst22 -
     mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,
     mst12)) + 96*mh2*mst12*mst22*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) -
     96*mst12*mst22*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*
     power3(mst12)) + 72*(mst12 - mst22)*mst22*power3(mh2)*(-2*mst12*mst24 - 2*
     mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*power3(
     DeltaInv(mst22,mh2,mst12)) + 1152*mst12*mt2*power2(DeltaInv(mst22,mst22,
     mh2))*power4(mst22) - 4608*mh2*mst22*mt2*power3(DeltaInv(mh2,mst12,mst12))
     *power5(mst12) + 4608*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*
     power5(mst22)))/(24.*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*log(
     mh2/Q2)*power2(cb)*power2(sa)*(mh2*mst12 - mh2*mst22 + 8*mst12*mt2 - 8*
     mst22*mt2 - 8*(mh2 - 6*mst12)*mst12*mst22*mt2*DeltaInv(mh2,mst12,mst12) +
     8*mh2*mst12*mst22*mt2*DeltaInv(mst22,mst22,mh2) + (-mh2 + mst12 - 5*mst22)
     *(mst12 - mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 48*mst12*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*(mst12 - mst22)*mst22*power2(
     mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*
     power2(DeltaInv(mst22,mh2,mst12)) + 96*mh2*mst12*mst22*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) - 96*mst12*mst22*mt2*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*(mst12 - mst22)*mst22*power3(
     mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) + 1152*mst12*mt2*power2(
     DeltaInv(mst22,mst22,mh2))*power4(mst22) - 4608*mh2*mst22*mt2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) + 4608*mh2*mst12*mt2*power3(
     DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(24.*mst12*(mst12 - mst22)*
     mst22*power2(sb)) + (mu2*log(mst22/Q2)*power2(cb)*(msb12*(mst12 - mst22)*(
     5*mA2*mst22 - mst12*mst22 + mst24)*DeltaInv(mst22,mA2,mst12) + 2*mst12*(5*
     mC2*mst22 - msb12*mst22 + mst24)*(mst12 - mst22 - 2*mt2)*DeltaInv(mst22,
     msb12,mC2) + 2*mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) - 4*msb12*
     mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12) + 4*mst12*mst24*mt2*DeltaInv(
     mst22,mw2,msb12) + 20*mst12*mst22*mt2*mw2*DeltaInv(mst22,mw2,msb12) -
     msb12*mst12*mst24*DeltaInv(mst22,mz2,mst12) + msb12*mst22*mst24*DeltaInv(
     mst22,mz2,mst12) - 5*msb12*mst12*mst22*mz2*DeltaInv(mst22,mz2,mst12) + 2*
     msb12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12) - 2*mst24*DeltaInv(
     mst22,mw2,msb12)*power2(mst12) - 10*mst22*mw2*DeltaInv(mst22,mw2,msb12)*
     power2(mst12) + msb12*mst22*DeltaInv(mst22,mz2,mst12)*power2(mst12) - 2*
     msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 10*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*DeltaInv(mst22,mz2,
     mst12)*power2(mst22) + 5*msb12*mz2*DeltaInv(mst22,mz2,mst12)*power2(mst22)
     + 12*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 24*
     msb12*mst12*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 12*msb12*
     mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) + 168*mst12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 72*msb12*
     mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 336*mst12*
     mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst22*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 168*mst24*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*
     mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*
     msb12*mst12*mst22*mst24*mz2*power2(DeltaInv(mst22,mz2,mst12)) - 6*msb12*
     mst24*mz2*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12)) - 84*msb12*
     mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 84*msb12*
     mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 18*msb12*
     mst22*power2(mst12)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 18*
     msb12*mst12*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) -
     24*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 12*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*msb12*
     mst12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22) - 6*mA2*msb12*(
     mst12 - mst22)*power2(DeltaInv(mst22,mA2,mst12))*(-3*mA2*mst12*mst22 - 14*
     mA2*mst24 - mst12*mst24 + mst22*power2(mA2) + power3(mst22)) - 12*mC2*
     mst12*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*
     msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) - 24*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     + 12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) -
     12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 6*msb12*
     power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 72*msb12*(
     mst12 - mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,
     mst12)) - 144*mst12*(mst12 - mst22 - 2*mt2)*power2(mC2)*(mst24*power2(mC2)
     + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 576*msb12*mst12*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 288*msb12*power2(mst12)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 288*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 576*
     msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 288*
     msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     576*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 288*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 144*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) + 144*msb12*mst12*mst22*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst24*power2(mst12)*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst12*power3(mst22)*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 12*mst12*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*msb12*mz2*power2(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) + 288*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 288*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*power2(mst12)*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 288*mst12*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 144*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 144*
     mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 288*
     mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst24*
     power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 72*msb12*
     mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 72*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(24.*msb12*
     mst12*(-mst12 + mst22)) + (mu2*log(mst22/Q2)*power4(cb)*(msb12*(mst12 -
     mst22)*(5*mA2*mst22 - mst12*mst22 + mst24)*DeltaInv(mst22,mA2,mst12) + 2*
     mst12*(5*mC2*mst22 - msb12*mst22 + mst24)*(mst12 - mst22 - 2*mt2)*
     DeltaInv(mst22,msb12,mC2) + 2*mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12)
     - 4*msb12*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12) + 4*mst12*mst24*mt2*
     DeltaInv(mst22,mw2,msb12) + 20*mst12*mst22*mt2*mw2*DeltaInv(mst22,mw2,
     msb12) - msb12*mst12*mst24*DeltaInv(mst22,mz2,mst12) + msb12*mst22*mst24*
     DeltaInv(mst22,mz2,mst12) - 5*msb12*mst12*mst22*mz2*DeltaInv(mst22,mz2,
     mst12) + 2*msb12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12) - 2*mst24*
     DeltaInv(mst22,mw2,msb12)*power2(mst12) - 10*mst22*mw2*DeltaInv(mst22,mw2,
     msb12)*power2(mst12) + msb12*mst22*DeltaInv(mst22,mz2,mst12)*power2(mst12)
     - 2*msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 10*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*DeltaInv(mst22,mz2,
     mst12)*power2(mst22) + 5*msb12*mz2*DeltaInv(mst22,mz2,mst12)*power2(mst22)
     + 12*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 24*
     msb12*mst12*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 12*msb12*
     mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) + 168*mst12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 72*msb12*
     mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 336*mst12*
     mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst22*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 168*mst24*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*
     mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*
     msb12*mst12*mst22*mst24*mz2*power2(DeltaInv(mst22,mz2,mst12)) - 6*msb12*
     mst24*mz2*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12)) - 84*msb12*
     mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 84*msb12*
     mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 18*msb12*
     mst22*power2(mst12)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 18*
     msb12*mst12*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) -
     24*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 12*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*msb12*
     mst12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22) - 6*mA2*msb12*(
     mst12 - mst22)*power2(DeltaInv(mst22,mA2,mst12))*(-3*mA2*mst12*mst22 - 14*
     mA2*mst24 - mst12*mst24 + mst22*power2(mA2) + power3(mst22)) - 12*mC2*
     mst12*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*
     msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) - 24*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     + 12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) -
     12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 6*msb12*
     power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 72*msb12*(
     mst12 - mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,
     mst12)) - 144*mst12*(mst12 - mst22 - 2*mt2)*power2(mC2)*(mst24*power2(mC2)
     + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 576*msb12*mst12*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 288*msb12*power2(mst12)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 288*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 576*
     msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 288*
     msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     576*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 288*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 144*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) + 144*msb12*mst12*mst22*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst24*power2(mst12)*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst12*power3(mst22)*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 12*mst12*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*msb12*mz2*power2(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) + 288*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 288*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*power2(mst12)*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 288*mst12*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 144*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 144*
     mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 288*
     mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst24*
     power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 72*msb12*
     mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 72*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(24.*msb12*
     mst12*(mst12 - mst22)*power2(sb)) + (mu2*log(mst22/Q2)*power2(cb)*(msb12*(
     mst12 - mst22)*(5*mA2*mst22 - mst12*mst22 + mst24)*DeltaInv(mst22,mA2,
     mst12) - 2*mst12*(5*mC2*mst22 - msb12*mst22 + mst24)*(-mst12 + mst22 +
     mt2)*DeltaInv(mst22,msb12,mC2) + 2*mst12*mst22*mst24*DeltaInv(mst22,mw2,
     msb12) - 2*msb12*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12) + 2*mst12*
     mst24*mt2*DeltaInv(mst22,mw2,msb12) + 10*mst12*mst22*mt2*mw2*DeltaInv(
     mst22,mw2,msb12) - msb12*mst12*mst24*DeltaInv(mst22,mz2,mst12) + msb12*
     mst22*mst24*DeltaInv(mst22,mz2,mst12) - 5*msb12*mst12*mst22*mz2*DeltaInv(
     mst22,mz2,mst12) + 2*msb12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12) -
     2*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst12) - 10*mst22*mw2*DeltaInv(
     mst22,mw2,msb12)*power2(mst12) + msb12*mst22*DeltaInv(mst22,mz2,mst12)*
     power2(mst12) - 2*msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) +
     10*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*
     DeltaInv(mst22,mz2,mst12)*power2(mst22) + 5*msb12*mz2*DeltaInv(mst22,mz2,
     mst12)*power2(mst22) + 12*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) + 12*msb12*mst12*mst24*mt2*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) - 12*msb12*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12)) + 168*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 36*msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 168*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - 36*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 168*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 36*msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*msb12*mst12*mst22*mst24*mz2*power2(DeltaInv(mst22,mz2,
     mst12)) - 6*msb12*mst24*mz2*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12)
     ) - 84*msb12*mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) +
     84*msb12*mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 18*
     msb12*mst22*power2(mst12)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) +
     18*msb12*mst12*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))
     - 12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 12*
     mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*
     msb12*mst12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22) - 6*mA2*
     msb12*(mst12 - mst22)*power2(DeltaInv(mst22,mA2,mst12))*(-3*mA2*mst12*
     mst22 - 14*mA2*mst24 - mst12*mst24 + mst22*power2(mA2) + power3(mst22)) +
     12*mC2*mst12*(-mst12 + mst22 + mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*
     mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) - 12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     + 12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) -
     12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 6*msb12*
     power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 72*msb12*(
     mst12 - mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,
     mst12)) + 144*mst12*(-mst12 + mst22 + mt2)*power2(mC2)*(mst24*power2(mC2)
     + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 288*msb12*mst12*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 288*msb12*power2(mst12)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 288*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 288*
     msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 288*
     msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     288*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 288*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 144*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) + 144*msb12*mst12*mst22*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst24*power2(mst12)*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst12*power3(mst22)*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 12*mst12*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*msb12*mz2*power2(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) + 288*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*power2(mst12)*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 288*mst12*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 144*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 144*
     mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*
     mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst24*
     power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 72*msb12*
     mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 72*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(12.*msb12*
     mst12*(mst12 - mst22)) + (mu2*power2(cb)*power2(log(mst22/Q2))*(msb12*(-
     mst12 + mst22)*mst24*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 2*
     mst12*mst24*(-mst12 + mst22 + mt2)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 2*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 2*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     + 2*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     msb12*mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - msb12*
     mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*(mst12 -
     mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(mst22) -
     2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) -
     2*mst12*(-mst12 + mst22 + mt2)*power2(mC2)*(mst24*power2(mC2) + (-2*msb12
     + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) - 4*msb12*mst12*mt2*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) + 4*msb12*power2(mst12)*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 4*msb12*mst24*power2(
     mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 4*mst12*mt2*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 4*power2(mst12)*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 2*msb12*
     power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12))
     - 2*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     + 2*msb12*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)
     ) + 2*msb12*mst12*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)) - 4*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 2*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 2*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 4*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 3*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mst22) - 2*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mst22) + 2*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 2*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 2*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(
     mz2) + msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) +
     2*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) +
     msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(
     msb12*mst12*(mst12 - mst22)) - (mu2*power2(cb)*power2(log(mst22/Q2))*(
     msb12*(-mst12 + mst22)*mst24*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))
     + 2*mst12*mst24*(-mst12 + mst22 + 2*mt2)*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - 2*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 4*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 2*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + msb12*mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))
     - msb12*mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*
     (mst12 - mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mA2,mst12)) + 2*mst12*(mst12 - mst22 - 2*mt2)*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 8*msb12*mst12*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 4*msb12*
     power2(mst12)*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12))
     - 4*msb12*mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 8*msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     4*msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 8*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     + 4*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + 2*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(
     mst22,mz2,mst12)) - 2*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + 2*msb12*mst24*power2(mst12)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)) + 2*msb12*mst12*power3(mst22)*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - 4*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 4*mst12*mt2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 2*power2(mst12)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 4*mst12*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*mst12*power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 2*msb12*power3(mz2)
     *power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 2*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 4*mst12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 2*mst24*power2(mst12)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - msb12*mst12*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) + msb12*mst22*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) + 2*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + msb12*power2(mz2)*power3(
     DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(2.*msb12*mst12*(mst12 - mst22)
     ) + (mu2*power2(log(mst22/Q2))*power4(cb)*(msb12*(-mst12 + mst22)*mst24*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 2*mst12*mst24*(-mst12 +
     mst22 + 2*mt2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 2*mst12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 4*mst12*mst24*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mst24*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*mst12*mst24*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst22*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + msb12*(mst12 - mst22)*power2(mA2)*(
     mst24*power2(mA2) + (-2*mst12 + mst22)*power3(mst22) - 2*mA2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) + 2*mst12*(mst12 -
     mst22 - 2*mt2)*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2)) - 8*msb12*mst12*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,
     mw2,msb12)) + 4*msb12*power2(mst12)*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 8*msb12*mst12*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 4*msb12*mst24*power2(mst12)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 8*mst12*mt2*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 4*power2(mst12)*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 2*msb12*power2(mst12)*power2(mz2)
     *power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 2*msb12*mst12*mst22*
     mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 2*msb12*mst24*
     power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 2*msb12*
     mst12*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 4*
     msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     4*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) -
     2*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 4*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 3*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 2*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) + 2*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 4*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 2*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     - msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + msb12*
     mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 2*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(2.*msb12*
     mst12*(mst12 - mst22)*power2(sb)) + (DeltaInv(mst22,mt2,mu2)*log(mst22/Q2)
     *log(mt2/Q2)*power2(mu2)*(mst22 + DeltaInv(mst22,mt2,mu2)*(mst22*mu2*(4*
     mt2 + mu2) + 2*mst24*(2*mt2 + 7*mu2) + power3(mst22)) - 2*power2(DeltaInv(
     mst22,mt2,mu2))*(mst24*(-15*mt2 + 4*mu2)*power2(mu2) - 5*mu2*(3*mt2 + 2*
     mu2)*power3(mst22) + mst22*(-mt2 + mu2)*power3(mu2) - (mt2 - 4*mu2)*
     power4(mst22) + power5(mst22))))/mt2 - 336*mz2*power3(DeltaInv(mt2,mt2,
     mz2))*power5(mt2) + (log(mz2/Q2)*(mst22*mz2 + mst12*(-10*mst22 + mz2) + 4*
     mst12*mst22*mt2*(78*mt2 + 5*mz2)*DeltaInv(mt2,mt2,mz2) + 384*mst12*mst22*(
     3*mt2 + 4*mz2)*power2(DeltaInv(mt2,mt2,mz2))*power3(mt2) + 4608*mst12*
     mst22*mz2*power3(DeltaInv(mt2,mt2,mz2))*power5(mt2)))/(24.*mst12*mst22) +
     84*msb12*power3(DeltaInv(msb12,mt2,mu2))*power5(mu2) + (14*msb14*power3(
     DeltaInv(msb12,mt2,mu2))*power5(mu2))/mt2 + 84*mst22*power3(DeltaInv(
     mst22,mt2,mu2))*power5(mu2) + (14*mst24*power3(DeltaInv(mst22,mt2,mu2))*
     power5(mu2))/mt2 + 84*mst12*power3(DeltaInv(mt2,mu2,mst12))*power5(mu2) +
     (14*mst14*power3(DeltaInv(mt2,mu2,mst12))*power5(mu2))/mt2 - (log(mst22/
     Q2)*(((-5*mH2*mst22 + mst12*mst22 - mst24)*mu2*DeltaInv(mH2,mst22,mst12))/
     mst12 + ((-5*mA2*mst22 + mst12*mst22 - mst24)*mu2*DeltaInv(mst22,mA2,
     mst12))/mst12 + (6*mA2*mu2*power2(DeltaInv(mst22,mA2,mst12))*(-3*mA2*
     mst12*mst22 - 14*mA2*mst24 - mst12*mst24 + mst22*power2(mA2) + power3(
     mst22)))/mst12 + (6*mH2*mu2*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*
     mst12*mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*power2(mH2) + power3(
     mst22)))/mst12 + (72*mu2*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 +
     mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)))/mst12 + (72*mu2*power2(mA2)*(mst24*power2(mA2)
     + (-2*mst12 + mst22)*power3(mst22) - 2*mA2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mA2,mst12)))/mst12 + 2*(-(mst22/msb12) - mst22/mst12
     + (2*mst22)/(mst22 - mu2) + ((-5*mC2*mst22 + msb12*mst22 - mst24)*mu2*
     DeltaInv(mst22,msb12,mC2))/msb12 + 2*(mh2 - 6*mst22)*mt2*DeltaInv(mst22,
     mst22,mh2) + 2*mst22*mu2*DeltaInv(mst22,mt2,mu2) + (24*mst22*DeltaInv(
     mst22,mt2,mu2)*power2(mu2))/mt2 + 120*mst24*power2(mu2)*power2(DeltaInv(
     mst22,mt2,mu2)) + (48*power2(mu2)*power2(DeltaInv(mst22,mt2,mu2))*power3(
     mst22))/mt2 + (6*mC2*mu2*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*
     mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)))/
     msb12 - 48*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 3*power3(
     mst22)) + 48*mst22*power2(DeltaInv(mst22,mt2,mu2))*power3(mu2) + (264*
     mst24*power2(DeltaInv(mst22,mt2,mu2))*power3(mu2))/mt2 + (72*mu2*power2(
     mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)))/msb12 + 720*
     power3(mst22)*power3(mu2)*power3(DeltaInv(mst22,mt2,mu2)) - 576*mh2*mt2*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 72*power2(mu2)*power3(
     DeltaInv(mst22,mt2,mu2))*power4(mst22) - (72*power3(mu2)*power3(DeltaInv(
     mst22,mt2,mu2))*power4(mst22))/mt2 - (24*mst22*power2(DeltaInv(mst22,mt2,
     mu2))*power4(mu2))/mt2 + 360*mst24*power3(DeltaInv(mst22,mt2,mu2))*power4(
     mu2) + (360*power3(mst22)*power3(DeltaInv(mst22,mt2,mu2))*power4(mu2))/mt2
     - (72*power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*power5(mst22))/mt2 - (
     216*mst24*power3(DeltaInv(mst22,mt2,mu2))*power5(mu2))/mt2)))/24. + (21*
     mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2))/msb12 - (223*
     power5(mw2))/(216.*mt2*power4(mt2 - mw2)) + (71*power5(mw2))/(72.*power5(
     mt2 - mw2)) + (mt2*mu2*power2(cb)*(-3*msb12*mst12 + 3*msb12*mst22 - 2*mh2*
     msb12*mst12*mst22*DeltaInv(mh2,mst12,mst12) + 2*mh2*msb12*mst12*mst22*
     DeltaInv(mst22,mst22,mh2) + 2*msb12*mst12*mst22*mw2*DeltaInv(mst22,mw2,
     msb12) - 2*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) - 252*mst12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*msb12*
     mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 252*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     84*msb12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*
     power3(mst12)) - 84*msb12*mst12*mst22*power2(DeltaInv(mst22,mst22,mh2))*(
     mh2*mst24 + 6*power3(mst22)) - 252*mst12*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) + 252*mst22*power2(mst12)*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) - 1176*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 1176*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 420*msb12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 252*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 420*msb12*mst12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*mst12*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*mst12*mst22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 420*msb12*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 252*mst12*
     mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 420*msb12*
     mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2016*
     mh2*msb12*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 252*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 2016*
     mh2*msb12*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22) + 252*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 252*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 252*
     mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(6.*
     msb12*mst12*(mst12 - mst22)*mst22*power2(sb)) - (mu2*power2(cb)*power2(
     snt)*(-(msb12*mst12) + msb12*mst22 - 2*mh2*msb12*mst12*mst22*DeltaInv(mh2,
     mst12,mst12) + 4*mh2*msb12*mst12*mst22*DeltaInv(mst22,mh2,mst12) - 2*mh2*
     msb12*mst12*mst22*DeltaInv(mst22,mst22,mh2) + 2*msb12*mst12*mst22*mw2*
     DeltaInv(mst22,mw2,msb12) - 2*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb12,
     mst12) - 168*msb12*mst22*(3*mh2*mst22 + mst12*mst22 + 3*mst24)*power2(mh2)
     *power2(DeltaInv(mst22,mh2,mst12)) + 84*mh2*msb12*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mst22,mh2)) - 252*mst12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 84*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 252*mst12*mst14*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*mst12*mst22*power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) - 252*mst12*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 252*mst22*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 1176*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 1176*
     msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     420*msb12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12) + 252*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12) + 504*msb12*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) -
     420*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 252*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) + 168*msb12*mst22*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*(-
     14*mh2*mst12*mst24 - 5*mst12*mst22*power2(mh2) - 3*mst24*power2(mh2) + 3*
     mst22*power3(mh2) - 3*mh2*power3(mst22) - 5*mst12*power3(mst22) + 3*
     power4(mst22)) - 252*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 420*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 252*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 420*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) + 2016*mh2*msb12*mst22*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12) - 252*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mst12) + 2016*mh2*msb12*mst12*power3(DeltaInv(mst22,
     mst22,mh2))*power5(mst22) + 252*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mst22) + 252*mst12*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mw2) - 252*mst22*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2)))/(12.*msb12*mst12*mst22*power2(sb)) + ((mt2 +
     mu2)*log(msb12/Q2)*power2(cb)*power2(snt)*(mC2*mst12 - mC2*mst22 - mst12*
     mw2 + mst22*mw2 + mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) - mst12*
     mst14*mst22*DeltaInv(mw2,msb12,mst12) + msb12*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12)
     - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) +
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*msb12*mst12*mst22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(
     msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2)
     )*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*
     mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) +
     mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*DeltaInv(mw2,msb12,
     mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) - 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*
     msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 216*msb12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 216*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(12.*msb12*
     mst12*mst22) + (mu2*log(msb12/Q2)*power2(cb)*power2(snb)*power2(snt)*(mC2*
     mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*mst24*DeltaInv(
     mst22,mw2,msb12) - mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) + msb12*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*
     mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(
     mst22,mw2,msb12)*power2(mw2) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*
     power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*
     msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*
     mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*
     msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*
     mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*
     mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) +
     mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) -
     mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 216*msb12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*mst22) + (mt2*mu2*log(msb12/Q2)*power4(cb)*(mC2*
     mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*mst24*DeltaInv(
     mst22,mw2,msb12) - mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) + msb12*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*
     mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(
     mst22,mw2,msb12)*power2(mw2) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*
     power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*
     msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*
     mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*
     msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*
     mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*
     mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) +
     mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) -
     mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 216*msb12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*log(msb12/
     Q2)*power2(snb)*power2(snt)*power4(cb)*(mC2*mst12 - mC2*mst22 - mst12*mw2
     + mst22*mw2 + mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) - mst12*mst14*
     mst22*DeltaInv(mw2,msb12,mst12) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) -
     msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) +
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*msb12*mst12*mst22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(
     msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2)
     )*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*
     mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) +
     mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*DeltaInv(mw2,msb12,
     mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) - 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*
     msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 216*msb12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 216*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(12.*msb12*
     mst12*mst22*power2(sb)) + (mu2*log(msb12/Q2)*power2(cb)*power2(snt)*(-(
     mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) -
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22)
     + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*
     msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 36*
     msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) -
     mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2)
     + mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 216*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*mst22) + ((mt2 + mu2)*log(msb12/Q2)*power2(cb)*
     power2(snb)*power2(snt)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2
     - mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*
     DeltaInv(mw2,msb12,mst12) - msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(
     mst12) + 5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*
     mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) - 5*mst12*mw2*DeltaInv(
     mst22,mw2,msb12)*power2(mst22) + msb12*mst12*DeltaInv(mst22,mw2,msb12)*
     power2(mw2) - 5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,
     msb12,mst12)*power2(mw2) - 6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) + 138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 36*msb12*mst12*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) - 138*mst12*mst14*mst22*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 36*msb12*mst22*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - mst22*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(
     msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2)
     )*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*
     mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) -
     mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) + mst22*DeltaInv(mw2,msb12,
     mst12)*power3(mw2) - 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) + 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 720*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*
     msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 216*msb12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 216*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(12.*msb12*
     mst12*mst22) + (mt2*mu2*log(msb12/Q2)*power2(snb)*power4(cb)*(-(mC2*mst12)
     + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*DeltaInv(mst22,
     mw2,msb12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) - msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)
     - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + msb12*mst12*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) +
     5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 6*msb12*mst12*mst22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 138*mst12*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*mst12*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*
     mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 138*mst12*mst14*mst22*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 36*msb12*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - mst22*DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,
     msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(
     msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (
     msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*
     msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) + mst22*
     DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 216*msb12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*log(msb12/
     Q2)*power2(snt)*power4(cb)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*
     mw2 - mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*
     DeltaInv(mw2,msb12,mst12) - msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(
     mst12) + 5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*
     mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) - 5*mst12*mw2*DeltaInv(
     mst22,mw2,msb12)*power2(mst22) + msb12*mst12*DeltaInv(mst22,mw2,msb12)*
     power2(mw2) - 5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,
     msb12,mst12)*power2(mw2) - 6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) + 138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 36*msb12*mst12*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) - 138*mst12*mst14*mst22*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 36*msb12*mst22*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - mst22*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(
     msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2)
     )*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*
     mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) -
     mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) + mst22*DeltaInv(mw2,msb12,
     mst12)*power3(mw2) - 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) + 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 720*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*
     msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 216*msb12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 216*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(12.*msb12*
     mst12*mst22*power2(sb)) + (power2(snt)*(-16*msb12*mst12*mst22 - msb12*
     mst12*mt2 + msb12*mst22*mt2 - msb12*mst12*mu2 + msb12*mst22*mu2 + msb12*
     mst12*mw2 - msb12*mst22*mw2 + 2*mH2*msb12*mst12*mst22*mu2*DeltaInv(mH2,
     mst12,mst12) - 4*mH2*msb12*mst12*mst22*mu2*DeltaInv(mH2,mst22,mst12) + 2*
     mH2*msb12*mst12*mst22*mu2*DeltaInv(mH2,mst22,mst22) + 2*mC2*msb12*mst12*
     mst22*mu2*DeltaInv(msb12,mst12,mC2) - 2*mC2*msb12*mst12*mst22*mu2*
     DeltaInv(mst22,msb12,mC2) + 2*msb12*mst12*mst22*mt2*mw2*DeltaInv(mst22,
     mw2,msb12) - 2*msb12*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) +
     mst12*power2(msb12) - mst22*power2(msb12) + 8*msb12*power2(mst12) - mst22*
     power2(mst12) + 8*msb12*power2(mst22) + mst12*power2(mst22) + 168*msb12*
     mst22*(3*mH2*mst22 + mst12*mst22 + 3*mst24)*mu2*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 84*mH2*msb12*mst12*mst22*mst24*mu2*power2(
     DeltaInv(mH2,mst22,mst22)) - 252*mst12*mst14*mst22*mu2*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - 84*msb12*mst22*mu2*power2(mC2)*power2(mst12)*
     power2(DeltaInv(msb12,mst12,mC2)) + 252*mst12*mst22*mst24*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 84*msb12*mst12*mu2*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 252*mst12*mst22*mst24*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*msb12*mst12*mt2*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 252*mst12*mst14*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*mst22*
     mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 252*
     mst22*mu2*power2(mst12)*power2(DeltaInv(msb12,mst12,mC2))*power3(mC2) +
     252*mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2)
     - 84*msb12*mst12*mst22*mu2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 +
     6*power3(mst12)) - 252*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 252*mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,
     msb12,mst12))*power3(mw2) - 1176*msb12*mst12*mst14*mst22*mu2*power3(mC2)*
     power3(DeltaInv(msb12,mst12,mC2)) + 1176*msb12*mst12*mst22*mst24*mu2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 1176*msb12*mst12*mst22*
     mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 1176*msb12*
     mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 252*
     mst12*mst14*mst22*mu2*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) - 420*
     msb12*mst22*mu2*power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power4(
     mC2) + 252*mst12*mst22*mst24*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) + 420*msb12*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     *power4(mC2) - 420*msb12*mst22*mu2*power2(mC2)*power3(DeltaInv(msb12,
     mst12,mC2))*power4(mst12) - 252*mst22*mu2*power3(mC2)*power3(DeltaInv(
     msb12,mst12,mC2))*power4(mst12) + 420*msb12*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 252*mst22*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 504*msb12*mst12*mu2*
     power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 420*msb12*mst12*mu2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 252*mst12*
     mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 420*
     msb12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 252*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 168*msb12*mst22*mu2*power2(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*(-14*mH2*mst12*mst24 - 5*mst12*mst22*power2(mH2) - 3*mst24*power2(
     mH2) + 3*mst22*power3(mH2) - 3*mH2*power3(mst22) - 5*mst12*power3(mst22) +
     3*power4(mst22)) - 252*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 420*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 252*mst12*mst14*mst22*mt2*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 420*msb12*mst22*mt2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 252*mst22*mu2*power2(mst12)*
     power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) - 252*mst12*mu2*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 2016*mH2*msb12*
     mst22*mu2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + 252*mst22*mu2*
     power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power5(mst12) - 252*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 2016*
     mH2*msb12*mst12*mu2*power3(DeltaInv(mH2,mst22,mst22))*power5(mst22) - 252*
     mst12*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power5(mst22) +
     252*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22)
     + 252*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) - 252*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2)))/(12.*msb12*mst12*mst22) - (log(msb12/Q2)*power2(snt)*(-(
     msb12*mst12*mt2) + msb12*mst22*mt2 + mC2*mst12*mu2 + msb12*mst12*mu2 -
     mC2*mst22*mu2 - msb12*mst22*mu2 - mst12*mt2*mw2 + mst22*mt2*mw2 + mst12*
     mst22*mst24*mt2*DeltaInv(mst22,mw2,msb12) - mst12*mst14*mst22*mt2*
     DeltaInv(mw2,msb12,mst12) + mst12*power2(msb12) - mst22*power2(msb12) +
     msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst12) - 5*mst22*mt2*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) - msb12*mst12*mt2*DeltaInv(mst22,
     mw2,msb12)*power2(mst22) + 5*mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12)*
     power2(mst22) - msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 5*
     mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + msb12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*mst12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mw2) + 6*msb12*mst12*mst22*mst24*mt2*mw2*power2(
     DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*mt2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst14*mst22*mt2*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*mt2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*mu2*DeltaInv(msb12,
     mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2)
     + 5*mC2*power2(mst12) + power3(mC2)) - mst12*mu2*DeltaInv(mst22,msb12,mC2)
     *(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*
     power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*mu2*power2(DeltaInv(
     msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (
     msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*mu2*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(
     6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*
     mt2*DeltaInv(mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*mst12*mt2*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*
     mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*mt2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*mst12*
     mst22*mu2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*mu2*
     power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst14*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*mt2*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 216*msb12*mst22*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*mst22*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*mst12*
     mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*mst12*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*
     mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*
     mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*
     mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst22*
     mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mst12*
     mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*
     mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 216*
     msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 144*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mst22) + 144*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 144*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2)))/(12.*msb12*mst12*mst22) + (log(msb12/Q2)*
     power2(snb)*power2(snt)*(-(msb12*mst12*mt2) + msb12*mst22*mt2 + mC2*mst12*
     mu2 + msb12*mst12*mu2 - mC2*mst22*mu2 - msb12*mst22*mu2 - mst12*mt2*mw2 +
     mst22*mt2*mw2 + mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,msb12) - mst12*
     mst14*mst22*mt2*DeltaInv(mw2,msb12,mst12) + mst12*power2(msb12) - mst22*
     power2(msb12) + msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst12) -
     5*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) - msb12*mst12*mt2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mt2*mw2*DeltaInv(mst22,
     mw2,msb12)*power2(mst22) - msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*
     power2(mw2) + 5*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) +
     msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*mst12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*msb12*mst12*mst22*mst24*mt2*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*mt2*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst14*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*mt2*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*mu2*DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*mu2*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*mu2*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*mu2*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24)
     + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(
     mw2) - mst22*mt2*DeltaInv(mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*mst12*mt2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*
     mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*
     mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*
     mst12*mst22*mu2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*mst12*
     mst22*mu2*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*mst12*mst22*
     mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*
     mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 216*msb12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*
     mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) -
     144*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     - 6*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*
     mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*
     mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     144*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) +
     216*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 144*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mst12) + 144*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power5(mst22) + 144*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 144*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2)))/(12.*msb12*mst12*mst22) + (power2(cb)*power2(
     snt)*(mC2*msb12*mst12 - mC2*msb12*mst22 - msb12*mst12*mw2 + msb12*mst22*
     mw2 - 2*mC2*msb12*mst12*mst22*(mt2 + mu2)*DeltaInv(msb12,mst12,mC2) + 2*
     mC2*msb12*mst12*mst22*(mt2 + mu2)*DeltaInv(mst22,msb12,mC2) - 2*msb12*
     mst12*mst22*mt2*mw2*DeltaInv(mst22,mw2,msb12) - 2*msb12*mst12*mst22*mu2*
     mw2*DeltaInv(mst22,mw2,msb12) + 2*msb12*mst12*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12) + 2*msb12*mst12*mst22*mu2*mw2*DeltaInv(mw2,msb12,mst12) + 84*
     mst12*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*mst22*(mt2 + mu2)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) - 84*mst12*mst22*(3*mC2*mst22 + msb12*
     mst22 + 3*mst24)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     + 252*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     + 252*mst12*mst22*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     + 84*msb12*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 84*msb12*mst12*mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 252*mst12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) - 252*mst12*mst14*mst22*mu2*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) - 84*msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 84*msb12*mst22*mu2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 252*mst12*mt2*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 252*mst12*mu2*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 252*mst22*mt2*power2(
     mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 252*mst22*mu2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 84*mst12*
     mst22*(mt2 + mu2)*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) +
     3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 1176*msb12*
     mst12*mst22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     1176*msb12*mst12*mst22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 1176*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) - 1176*msb12*mst12*mst14*mst22*mu2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 420*msb12*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 420*msb12*mst22*mu2*power2(mw2)
     *power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 252*mst22*mt2*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 252*mst22*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 420*msb12*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     420*msb12*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) + 252*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 252*mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) + 84*mst12*mst22*(mt2 + mu2)*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*(-14*mC2*msb12*mst24 - 5*msb12*mst22*power2(mC2) - 3*
     mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*mC2*power3(mst22) - 5*msb12*
     power3(mst22) + 3*power4(mst22)) + 252*mst12*mst22*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 252*mst12*mst22*mst24*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 420*msb12*mst12*mt2*power2(mst22)
     *power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 420*msb12*mst12*mu2*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 252*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 252*mst12*
     mst14*mst22*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 420*msb12*
     mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) -
     420*msb12*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 252*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mst12) + 252*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power5(mst12) - 252*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mst22) - 252*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mst22) - 252*mst12*mt2*power2(mst22)*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) - 252*mst12*mu2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) + 252*mst22*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 252*mst22*mu2*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(12.*msb12*mst12*
     mst22) + (mu2*power2(cb)*power2(snb)*power2(snt)*(-(mC2*msb12*msb22*
     DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) +
     mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*DeltaInv(
     mst22,msb22,mC2) - msb12*msb22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*
     msb22*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mw2*DeltaInv(mw2,msb12,
     mst12) - msb12*msb22*mw2*DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*mC2*mst12
     + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     42*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - 42*msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 42*msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + 126*msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) + 42*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 126*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     42*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 126*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) + 126*msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2))*power3(mC2) + 126*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 126*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 126*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2) + 126*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 42*
     msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*
     mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*
     msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 126*
     msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 210*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 126*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) -
     588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 210*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 210*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb12*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*
     mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*
     power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22) - (
     (mt2 + mu2)*power2(cb)*power2(snb)*power2(snt)*(-(mC2*msb12*msb22*
     DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) +
     mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*DeltaInv(
     mst22,msb22,mC2) - msb12*msb22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*
     msb22*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mw2*DeltaInv(mw2,msb12,
     mst12) - msb12*msb22*mw2*DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*mC2*mst12
     + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     42*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - 42*msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 42*msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + 126*msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) + 42*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 126*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     42*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 126*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) + 126*msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2))*power3(mC2) + 126*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 126*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 126*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2) + 126*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 42*
     msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*
     mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*
     msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 126*
     msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 210*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 126*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) -
     588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 210*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 210*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb12*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*
     mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*
     power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) - (
     mt2*mu2*power2(snb)*power4(cb)*(-(mC2*msb12*msb22*DeltaInv(msb12,mst12,
     mC2)) + mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*
     DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*DeltaInv(mst22,msb22,mC2) -
     msb12*msb22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*mw2*DeltaInv(
     mst22,mw2,msb22) + msb12*msb22*mw2*DeltaInv(mw2,msb12,mst12) - msb12*
     msb22*mw2*DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*mC2*mst12 + msb12*mst12
     + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*
     mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - 42*msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 126*msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)
     ) + 42*msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) +
     126*msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*
     msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*msb12*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*msb22*mst12*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst14*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 126*msb22*mst22*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) + 126*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 126*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(
     mw2) - 126*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) -
     126*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 126*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 42*msb22*power2(mC2)
     *(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*
     msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 588*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 126*msb22*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*msb12*msb22*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 588*msb12*
     msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 126*msb12*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 210*msb12*
     msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 588*
     msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 210*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb12*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*
     mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*
     power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*(
     mst12 - mst22)*power2(sb)) + (mu2*power2(snb)*power2(snt)*power4(cb)*(-(
     mC2*msb12*msb22*DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(
     msb22,mst12,mC2) + mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*
     msb22*DeltaInv(mst22,msb22,mC2) - msb12*msb22*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*msb22*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mw2*
     DeltaInv(mw2,msb12,mst12) - msb12*msb22*mw2*DeltaInv(mw2,msb22,mst12) +
     42*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*msb12*msb22*
     mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*msb12*msb22*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 42*msb12*msb22*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 126*msb12*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 42*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 126*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 42*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) + 126*msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     126*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 126*msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb22*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 126*msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb22*mst12*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) + 126*msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 42*msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     - 588*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     210*msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 126*msb22*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     + 588*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     210*msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)
     ) + 126*msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2))
     + 210*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) + 588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + 126*msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 210*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,
     mw2,msb22)) - 588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 126*msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 210*msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)) - 588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 126*msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 210*msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(
     mw2,msb22,mst12)) + 588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 126*msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 210*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 126*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) + 210*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2)
     + 126*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*
     msb12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 -
     5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*
     mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22*
     power2(sb)) + (mu2*power2(cb)*power2(Pi)*power2(snb)*power2(snt)*(msb22*(
     3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) + 3*msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))
     + msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*
     msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) - 3*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) + 3*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     + 3*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 3*msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb22*mst12*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*power2(mC2)*((5*msb12*mst12
     + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(
     mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) - 14*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 5*msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)) - 3*msb22*power3(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)) + 14*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 5*msb12*msb22*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 3*msb22*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 5*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 14*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 3*msb12*power3(mst22)*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb22)) - 5*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 14*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*power3(mst12)*power3(mw2)
     *power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 14*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb12*power3(mst12)*power3(mw2)
     *power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst24*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) + 5*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2)
     + 3*msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 3*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + msb12*
     power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*
     msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*
     mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 3*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 5*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 5*msb12*msb22*mst12*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mw2) + 3*msb12*mst14*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mw2) + 3*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mC2) - 3*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 3*msb12*mst22*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb22*mst12*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*mst12*power3(DeltaInv(
     mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22) + (mu2*power2(Pi)*power2(
     snb)*power2(snt)*power4(cb)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*msb22*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) - 3*msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))
     - msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 3*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + msb12*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 3*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) + 3*msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2))*power3(mC2) + 3*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 3*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 3*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2)
     + 3*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*
     power2(mC2)*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) +
     (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12))
     )*power3(DeltaInv(msb12,mst12,mC2)) - 14*msb12*msb22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 14*msb12*msb22*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 3*msb12*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb12*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 3*msb22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 3*msb12*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + msb12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*
     power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(
     mst12) + 3*power4(mst12)) + 3*msb22*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 3*msb12*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22) - 3*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 3*msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 3*msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 3*msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*msb12*msb22*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb12*mst14*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst22*power3(DeltaInv(
     mst22,msb22,mC2))*power5(mC2) - 3*msb22*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 3*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)
     - 3*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*
     msb22*power2(sb)) + ((mt2 + mu2)*power2(cb)*power2(Pi)*power2(snb)*power2(
     snt)*(-(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb12*msb22*mst22*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) - msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 3*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     + msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 3*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + msb12*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst22*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 3*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) - 3*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     + 3*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 3*msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 3*msb12*mst12*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*power2(mC2)*(-((5*
     msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*
     mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 14*msb12*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 3*msb12*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + msb12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2) + 3*mst14*
     power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*msb22*power3(
     mst12) - 3*power4(mst12)) - 3*msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 3*msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) - 3*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) + 3*msb12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     )*power4(mst22) + 3*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 3*msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 3*msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*
     msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 5*msb12*msb22*
     mst12*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 3*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 5*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb12*mst14*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst22*power3(DeltaInv(mst22,msb22,
     mC2))*power5(mC2) + 3*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 3*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2)
     - 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 3*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) + (
     mt2*mu2*power2(Pi)*power2(snb)*power4(cb)*(-(msb22*(3*mC2*mst12 + msb12*
     mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*(
     3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) + msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) + 3*msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) -
     msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst22*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 3*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) - msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 3*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 3*msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) - 3*msb22*mst22*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) + 3*msb12*mst22*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mw2) + 3*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 3*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2)
     + msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 14*msb12*msb22*
     mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*
     msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*
     msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 14*
     msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*
     msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 14*
     msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 3*msb12*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 5*msb12*
     msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 14*
     msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*
     msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 14*
     msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + msb12*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2)
     + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*
     msb22*power3(mst12) - 3*power4(mst12)) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*mst22*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) + 3*msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 3*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*
     msb12*mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst22*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) - 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) + 3*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2))
     )/(3.*msb12*msb22*(mst12 - mst22)*power2(sb)) + (mu2*power2(snb)*power4(
     cb)*(-2*mC2*msb12*msb22*mt2*DeltaInv(msb12,mst12,mC2) + 2*mC2*msb12*msb22*
     mt2*DeltaInv(msb22,mst12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,
     msb12,mC2) + mC2*msb12*msb22*mst22*DeltaInv(mst22,msb12,mC2) + 2*mC2*
     msb12*msb22*mt2*DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb22,mC2) - mC2*msb12*msb22*mst22*DeltaInv(mst22,msb22,
     mC2) - 2*mC2*msb12*msb22*mt2*DeltaInv(mst22,msb22,mC2) + msb12*msb22*
     mst12*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*mw2*DeltaInv(
     mst22,mw2,msb12) - 2*msb12*msb22*mt2*mw2*DeltaInv(mst22,mw2,msb12) -
     msb12*msb22*mst12*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mst22*mw2*
     DeltaInv(mst22,mw2,msb22) + 2*msb12*msb22*mt2*mw2*DeltaInv(mst22,mw2,
     msb22) + 2*msb12*msb22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 2*msb12*msb22*
     mt2*mw2*DeltaInv(mw2,msb22,mst12) + 84*msb22*(3*mC2*mst12 + msb12*mst12 +
     3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - 84*msb12*(3*
     mC2*mst12 + msb22*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) + 42*msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 126*msb22*mst12*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - 126*msb22*mst22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - 84*msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - 252*msb22*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 42*msb12*msb22*power2(mC2)*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) - 42*msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 126*msb12*mst12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 84*msb12*msb22*mst22*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 252*msb12*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*power2(mC2)*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2)) - 42*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mst22,mw2,msb12)) - 126*msb22*mst12*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 84*msb12*msb22*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 252*msb22*mst24*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 42*msb12*msb22*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*msb12*msb22*mst12*mst22*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 126*msb12*mst12*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*msb12*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*msb22*mst22*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 252*msb12*mst24*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*msb22*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*msb22*
     mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 252*msb22*mst14*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*msb22*mst12*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 252*msb12*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mst12*mst22*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 252*msb22*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 252*msb12*mst22*mt2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 126*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 252*msb22*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 252*msb12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 252*msb22*mst12*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 252*msb12*mst12*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 84*msb22*mt2*power2(mC2)*(
     -((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 588*msb12*msb22*mst12*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 588*msb12*msb22*mst22*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 1176*msb12*msb22*mst24*mt2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 210*msb12*msb22*mst12*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 420*msb12*
     msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     126*msb22*mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 252*msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) - 588*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 588*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 1176*msb12*msb22*mst24*mt2*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 210*msb12*msb22*mst12*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 420*msb12*msb22*mt2*power2(mC2)
     *power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 126*msb12*mst12*power3(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 252*msb12*mt2*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 210*msb12*
     msb22*mst12*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     420*msb12*msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) - 588*msb12*msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 588*msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 1176*msb12*msb22*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 126*msb22*mst12*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 252*msb22*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 210*msb12*msb22*mst12*power2(mw2)
     *power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 420*msb12*msb22*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 588*msb12*
     msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 588*
     msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     1176*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) +
     126*msb12*mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)
     ) - 252*msb12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 420*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(
     mw2,msb12,mst12)) - 1176*msb12*msb22*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 252*msb22*mt2*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 420*msb12*msb22*mt2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 1176*msb12*msb22*mst14*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 252*msb12*mt2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*
     mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 126*msb22*
     mst12*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*msb22*
     mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 420*msb12*
     msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 252*msb22*
     mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 210*msb12*msb22*
     power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 210*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 126*
     msb12*mst12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*
     msb12*mst22*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 420*
     msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 252*
     msb12*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 210*msb12*
     msb22*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 252*
     msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     252*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     + 84*msb12*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) - 210*msb12*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mst22) - 126*msb22*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) + 252*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 126*msb22*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 210*msb12*msb22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mst22) + 126*msb12*mst12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 252*msb12*mt2*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 126*msb12*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 210*msb12*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb22*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*msb22*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*
     msb22*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 210*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) -
     126*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mst22) + 252*msb12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 126*msb12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 126*msb22*mst12*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 126*msb22*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 420*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 252*msb22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) + 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) + 126*msb12*mst12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 126*msb12*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 420*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 252*msb12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 420*msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 252*msb22*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 420*msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mw2) + 252*msb12*mst14*mt2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) - 126*msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 252*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 126*msb22*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 126*msb12*mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 252*msb12*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 126*msb12*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) + 126*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mst22) - 126*msb12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mst22) - 126*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mst22) + 126*msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mst22) + 126*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 252*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 126*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 126*msb12*mst12*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 252*msb12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 126*msb12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 252*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 252*msb12*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mw2)))/(6.*msb12*msb22*(mst12 - mst22)*power2(sb)) + (mu2*power2(
     cb)*power2(snb)*(-(mC2*msb12*msb22*mt2*DeltaInv(msb12,mst12,mC2)) + mC2*
     msb12*msb22*mt2*DeltaInv(msb22,mst12,mC2) - mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*mst22*DeltaInv(mst22,msb12,
     mC2) + mC2*msb12*msb22*mt2*DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*
     mst12*DeltaInv(mst22,msb22,mC2) - mC2*msb12*msb22*mst22*DeltaInv(mst22,
     msb22,mC2) - mC2*msb12*msb22*mt2*DeltaInv(mst22,msb22,mC2) + msb12*msb22*
     mst12*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*mw2*DeltaInv(
     mst22,mw2,msb12) - msb12*msb22*mt2*mw2*DeltaInv(mst22,mw2,msb12) - msb12*
     msb22*mst12*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mst22*mw2*
     DeltaInv(mst22,mw2,msb22) + msb12*msb22*mt2*mw2*DeltaInv(mst22,mw2,msb22)
     + msb12*msb22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*mt2*mw2*
     DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)
     *mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*mC2*mst12
     + msb22*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))
     + 42*msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     + 126*msb22*mst12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) -
     126*msb22*mst22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 42*
     msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 42*msb12*
     msb22*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 42*
     msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) -
     126*msb12*mst12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*
     msb12*mst22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*
     msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*
     msb12*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*
     msb22*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 42*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     126*msb22*mst12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*
     msb22*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*
     msb12*msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*
     msb22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*msb12*
     msb22*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     126*msb12*mst12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*
     msb12*msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*
     msb22*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*
     msb12*msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 126*
     msb22*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*
     msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*
     mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb22*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb12*mst22*mt2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 126*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb22*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb22*mst12*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 126*msb12*mst12*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 42*msb22*mt2*power2(mC2)*(
     -((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 588*msb12*msb22*mst12*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 588*msb12*msb22*mst22*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 588*msb12*msb22*mst24*mt2*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 210*msb12*msb22*mst12*power2(mC2)
     *power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 126*msb22*
     mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) -
     588*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     + 588*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     ) + 588*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     ) - 210*msb12*msb22*mst12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) + 210*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) - 126*msb12*mst12*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mt2*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 210*msb12*msb22*mst12*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 210*msb12*msb22*
     mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 588*
     msb12*msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     588*msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     + 588*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 126*msb22*mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + 126*msb22*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 210*msb12*msb22*mst12*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb22)) - 210*msb12*msb22*mt2*power2(mw2)*power3(mst22)
     *power3(DeltaInv(mst22,mw2,msb22)) + 588*msb12*msb22*mst12*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 588*msb12*msb22*mst22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 588*msb12*msb22*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 126*msb12*mst12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 126*msb12*mt2*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 210*msb12*
     msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     588*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     126*msb22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     + 210*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,
     mst12)) + 588*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + 126*msb12*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) + 126*msb22*mst12*mst24*power3(DeltaInv(mst22,msb12,mC2)
     )*power4(mC2) - 126*msb22*mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 210*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 126*msb22*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) - 126*msb12*mst12*mst24*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 126*msb12*mst22*mst24*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 210*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 126*msb12*mst24*mt2*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mC2) + 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 126*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 126*msb12*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 42*msb12*mt2*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(
     mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) -
     5*msb22*power3(mst12) + 3*power4(mst12)) - 210*msb12*msb22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb22*mst12*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 126*msb22*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb22*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 210*msb12*
     msb22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 126*
     msb12*mst12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) -
     126*msb12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)
     + 126*msb12*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) +
     210*msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) + 126*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 126*msb22*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) + 126*msb22*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 210*msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 126*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 126*msb12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power4(mst22) - 126*msb12*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power4(mst22) - 210*msb12*msb22*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 126*msb22*mst12*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*mst22*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 210*msb12*msb22*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 210*msb12*msb22*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 210*msb12*msb22*mst12*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 126*msb12*mst12*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*
     mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*
     msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*
     mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*
     mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*
     mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 126*msb22*mst12*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 126*msb22*mst22*mt2*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 126*msb22*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 126*msb12*mst12*mst22*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) - 126*msb12*mst22*mt2*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) - 126*msb12*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 126*msb22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mst22) - 126*msb12*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mst22) - 126*msb22*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 126*msb12*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mst22) + 126*msb22*mst12*mst22*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 126*msb22*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 126*msb22*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 126*msb12*mst12*mst22*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 126*msb12*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 126*msb12*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 126*msb22*mst12*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*mst12*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*(mst12 -
     mst22)) + (mu2*power2(Pi)*power2(snb)*power4(cb)*(2*msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     2*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mt2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst22*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*msb22*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*msb22*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst22*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + msb12*msb22*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst12*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb22*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst12*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 2*msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 6*msb12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 2*msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 6*msb22*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 6*msb12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst12*mst22*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 6*msb22*mst22*mt2*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 3*msb22*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)
     )*power3(mC2) - 3*msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 6*msb12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2) + 3*msb12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 3*msb22*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*
     msb22*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb22*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb12*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb12*mst22*
     mt2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mt2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 2*msb22*mt2*power2(mC2)*((5*
     msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*
     mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 14*msb12*msb22*mst12*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst22*mst24*power3(mC2)
     *power3(DeltaInv(mst22,msb12,mC2)) - 28*msb12*msb22*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*mst12*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 10*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*
     mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 6*
     msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) -
     14*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     14*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     28*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     5*msb12*msb22*mst12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) + 10*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) - 3*msb12*mst12*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*mt2*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*mst12*power2(mw2)*power3(mst22)
     *power3(DeltaInv(mst22,mw2,msb12)) + 10*msb12*msb22*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst12*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*
     mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 28*msb12*
     msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*
     mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 6*
     msb22*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*
     msb12*msb22*mst12*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22)) - 10*msb12*msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb22)) + 14*msb12*msb22*mst12*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) - 14*msb12*msb22*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 28*msb12*msb22*mst24*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 6*msb12*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 10*msb12*msb22*mt2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 28*msb12*msb22*mst14*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*msb22*mt2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 10*msb12*msb22*mt2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 28*msb12*
     msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*msb12*
     mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*
     mst12*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst22*
     mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 10*msb12*msb22*
     mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 6*msb22*mst24*
     mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*
     mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*mst12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst22*mst24*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 10*msb12*msb22*mst22*mt2*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 6*msb12*mst24*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 5*msb12*msb22*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 6*msb22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*msb12*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 2*msb12*mt2*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*
     power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(
     mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) - 5*msb12*msb22*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb22*mst12*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 6*msb22*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb22*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 5*msb12*
     msb22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*
     msb12*mst12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) -
     6*msb12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) +
     3*msb12*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 5*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     3*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     - 6*msb22*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     + 3*msb22*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 5*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) -
     3*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22)
     + 6*msb12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22)
     - 3*msb12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*
     msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*
     msb22*mst12*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*
     mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 10*msb12*
     msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*msb22*
     mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 10*msb12*msb22*
     mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*msb12*mst24*
     mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 10*msb12*msb22*
     mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 6*msb22*mst14*
     mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 10*msb12*msb22*mst12*
     mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 6*msb12*mst14*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst12*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 6*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 6*msb12*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mst22) - 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mst22) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 3*msb22*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 6*msb22*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 6*msb12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 6*msb22*mst12*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 6*msb12*mst12*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22*(mst12 - mst22)*
     power2(sb)) + (power2(Pi)*power2(snb)*power2(snt)*(msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*mu2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mu2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - msb12*msb22*mst22*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + msb12*msb22*mst22*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 3*msb22*mst22*mu2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) + 3*msb12*mst22*mu2*power2(DeltaInv(mst22,msb22,
     mC2))*power3(mC2) + 3*msb22*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 3*msb12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*power3(
     mw2) - 3*msb22*mst12*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) +
     3*msb12*mst12*mt2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*
     mu2*power2(mC2)*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(
     mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*msb12*msb22*mst24*mu2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mu2*power2(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mu2*power3(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 14*msb12*msb22*
     mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*
     mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*
     mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*
     msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     14*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     3*msb22*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     5*msb12*msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22)) - 14*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 3*msb12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 5*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)) - 14*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 3*msb22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 5*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(
     DeltaInv(mw2,msb22,mst12)) + 14*msb12*msb22*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mt2*power3(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*mst22*mu2*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst24*mu2*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mC2) + 5*msb12*msb22*mst22*mu2*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 3*msb12*mst24*mu2*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mC2) + 3*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 3*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + msb12*mu2*power2(mC2)*power3(DeltaInv(msb22,mst12,
     mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(
     mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) +
     3*power4(mst12)) + 3*msb22*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) - 3*msb12*mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mst22) - 3*msb22*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 3*msb12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 5*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 3*msb22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 5*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 3*msb12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 5*msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 3*msb22*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) +
     5*msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*
     msb12*mst14*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*
     mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst22*
     mu2*power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb22*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 3*msb12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb22*mst12*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*mst12*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) + (mu2*power2(
     cb)*power2(Pi)*power2(snb)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*
     mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) +
     msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 3*
     msb22*mst12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*
     mst22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*
     mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*
     mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst12*
     mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst12*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*mt2*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst12*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst12*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst12*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst12*mst22*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 3*msb22*mst22*mt2*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 3*msb22*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)
     )*power3(mC2) - 3*msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 3*msb12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2) + 3*msb12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 3*msb22*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*
     msb22*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb22*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb12*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb12*mst22*
     mt2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb22*mst12*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*mt2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*mt2*power2(mC2)*((5*msb12*
     mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*
     power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 14*msb12*msb22*mst12*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*mst12*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*
     mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) -
     14*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     14*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     14*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     5*msb12*msb22*mst12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) + 5*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) - 3*msb12*mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) + 3*msb12*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) - 5*msb12*msb22*mst12*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) + 5*msb12*msb22*mt2*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst12*mst24*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*mst22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*mst24*mt2*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst12*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mt2*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*msb22*mst12*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 5*msb12*
     msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) +
     14*msb12*msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     14*msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     14*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) +
     3*msb12*mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))
     - 3*msb12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))
     - 5*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,
     mst12)) - 14*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 3*msb22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 5*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb22,mst12)) + 14*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 3*msb12*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 5*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,mC2)
     )*power4(mC2) + 3*msb22*mst12*mst24*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 3*msb22*mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 5*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 3*msb22*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) - 5*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 5*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 3*msb12*mst12*mst24*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb12*mst22*mst24*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 5*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb12*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mC2) + 5*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 3*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) + msb12*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-
     14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*
     mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*
     power4(mst12)) - 5*msb12*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) - 3*msb22*mst12*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 3*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 3*msb22*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 5*msb12*msb22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mst22) + 3*msb12*mst12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb12*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb12*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) + 5*msb12*msb22*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb22*mst12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb22*mt2*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb22*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 5*msb12*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 3*msb12*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 3*msb12*mt2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 3*msb12*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*msb22*
     mst12*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*mst22*
     mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*
     mst12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*msb12*mst12*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*mst12*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*msb22*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*msb12*msb22*mst12*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb12*mst14*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst12*mst22*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb12*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mst22) - 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mst22) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 3*msb22*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb22*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb22*mst12*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*mst12*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*(mst12 - mst22))
     + (mu2*power2(cb)*power2(snb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22
     - msb22*mst22 + 2*msb12*msb22*(-mst12 + mst22 + 2*mt2)*mw2*DeltaInv(mst22,
     mw2,msb12) + 2*msb12*msb22*(mst12 - mst22 - 2*mt2)*mw2*DeltaInv(mst22,mw2,
     msb22) - 4*msb12*msb22*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 4*msb12*msb22*
     mt2*mw2*DeltaInv(mw2,msb22,mst12) + 84*msb22*(mst12 - mst22 - 2*mt2)*(
     msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 84*msb12*(mst12 - mst22 - 2*mt2)*(msb22*mst22 + 3*(mst24 +
     mst22*mw2))*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 168*msb12*
     msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 504*msb22*
     mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 168*msb12*msb22*
     mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 504*msb12*mst14*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 504*msb22*mst12*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 504*msb12*mst12*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 840*msb12*msb22*mt2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 2352*msb12*
     msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 504*msb22*
     mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 840*
     msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)
     ) - 2352*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 504*msb12*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 504*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) + 504*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 84*msb22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) - 84*msb12*(mst12 - mst22
     - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2
     + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22)
     + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) + 840*
     msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 504*
     msb22*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 840*msb12*
     msb22*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 504*msb12*
     mst14*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 504*msb22*mst12*
     mt2*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 504*msb12*mst12*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(12.*msb12*msb22*(mst12 -
     mst22)*power2(sb)) + ((mt2 + mu2)*log(msb22/Q2)*power2(cb)*power2(snb)*
     power2(snt)*(mC2*mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*
     mst24*DeltaInv(mst22,mw2,msb22) - mst12*mst14*mst22*DeltaInv(mw2,msb22,
     mst12) + msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*
     mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) - msb22*mst12*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(
     mst22) - msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*
     mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(
     mw2) + 6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) -
     138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     - mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(12.*msb22*mst12*mst22) + (mt2*mu2*log(msb22/Q2)*power2(snb)*
     power4(cb)*(mC2*mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*
     mst24*DeltaInv(mst22,mw2,msb22) - mst12*mst14*mst22*DeltaInv(mw2,msb22,
     mst12) + msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*
     mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) - msb22*mst12*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(
     mst22) - msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*
     mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(
     mw2) + 6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) -
     138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     - mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*log(msb22/
     Q2)*power2(cb)*power2(snb)*power2(snt)*(-(mC2*mst12) + mC2*mst22 + mst12*
     mw2 - mst22*mw2 - mst12*mst22*mst24*DeltaInv(mst22,mw2,msb22) + mst12*
     mst14*mst22*DeltaInv(mw2,msb22,mst12) - msb22*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mst12) + 5*mst22*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12)
     + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22) - 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb22)*power2(mst22) + msb22*mst12*DeltaInv(mst22,mw2,
     msb22)*power2(mw2) - 5*mst12*mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) -
     msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 5*mst12*mst22*
     DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb22)) + 138*mst12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 36*msb22*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 138*mst12*mst14*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 36*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - mst22*DeltaInv(msb22,mst12,mC2)*(
     mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(
     msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2)
     )*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) -
     mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2) + mst22*DeltaInv(mw2,msb22,
     mst12)*power3(mw2) - 6*msb22*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))
     *power3(mw2) + 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) + 6*msb22*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) - 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 720*msb22*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 720*
     msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*
     mst22*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 216*msb22*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 216*msb22*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 216*msb22*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 216*msb22*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mst12) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(mst22) - 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb22*
     mst12*mst22) + (mu2*log(msb22/Q2)*power2(snb)*power2(snt)*power4(cb)*(-(
     mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) -
     msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*DeltaInv(mst22,mw2,
     msb22)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     + mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     + 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(12.*msb22*mst12*mst22*power2(sb)) + (power2(cb)*power2(snb)*(-(
     mC2*msb12*mst12) + mC2*msb22*mst12 + mC2*msb12*mst22 - mC2*msb22*mst22 +
     msb12*mst12*mw2 - msb22*mst12*mw2 - msb12*mst22*mw2 + msb22*mst22*mw2 + 2*
     mC2*msb12*msb22*mt2*(-mst12 + mst22 + 2*mu2)*DeltaInv(msb12,mst12,mC2) +
     2*mC2*msb12*msb22*mt2*(mst12 - mst22 - 2*mu2)*DeltaInv(msb22,mst12,mC2) +
     2*mC2*msb12*msb22*mst12*mu2*DeltaInv(mst22,msb12,mC2) - 2*mC2*msb12*msb22*
     mst22*mu2*DeltaInv(mst22,msb12,mC2) - 4*mC2*msb12*msb22*mt2*mu2*DeltaInv(
     mst22,msb12,mC2) - 2*mC2*msb12*msb22*mst12*mu2*DeltaInv(mst22,msb22,mC2) +
     2*mC2*msb12*msb22*mst22*mu2*DeltaInv(mst22,msb22,mC2) + 4*mC2*msb12*msb22*
     mt2*mu2*DeltaInv(mst22,msb22,mC2) - 2*msb12*msb22*mst12*mu2*mw2*DeltaInv(
     mst22,mw2,msb12) + 2*msb12*msb22*mst22*mu2*mw2*DeltaInv(mst22,mw2,msb12) +
     4*msb12*msb22*mt2*mu2*mw2*DeltaInv(mst22,mw2,msb12) + 2*msb12*msb22*mst12*
     mu2*mw2*DeltaInv(mst22,mw2,msb22) - 2*msb12*msb22*mst22*mu2*mw2*DeltaInv(
     mst22,mw2,msb22) - 4*msb12*msb22*mt2*mu2*mw2*DeltaInv(mst22,mw2,msb22) +
     2*msb12*msb22*mst12*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 2*msb12*msb22*
     mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 4*msb12*msb22*mt2*mu2*mw2*
     DeltaInv(mw2,msb12,mst12) - 2*msb12*msb22*mst12*mt2*mw2*DeltaInv(mw2,
     msb22,mst12) + 2*msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) + 4*
     msb12*msb22*mt2*mu2*mw2*DeltaInv(mw2,msb22,mst12) + 84*msb22*(3*mC2*mst12
     + msb12*mst12 + 3*mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - 84*msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - 84*msb12*msb22*mst12*mst22*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - 252*msb22*mst12*mst24*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 252*msb22*mst22*mst24*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 168*msb12*msb22*mst22*mt2*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 504*msb22*mst24*mt2*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 84*msb12*msb22*mu2*power2(mC2)*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2)) + 84*msb12*msb22*mst12*mst22*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 252*msb12*mst12*mst24*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 252*msb12*mst22*mst24*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 168*msb12*msb22*mst22*mt2*mu2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 504*msb12*mst24*mt2*mu2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 84*msb12*msb22*mu2*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*msb22*
     mst12*mst22*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 252*msb22*
     mst12*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 252*msb22*
     mst22*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 168*msb12*
     msb22*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 504*
     msb22*mst24*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*
     msb12*msb22*mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - 84*msb12*msb22*mst12*mst22*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) - 252*msb12*mst12*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) + 252*msb12*mst22*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) + 168*msb12*msb22*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) + 504*msb12*mst24*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) + 84*msb12*msb22*mu2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 252*msb22*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 84*msb12*msb22*mst12*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 252*msb22*mst14*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 168*msb12*msb22*mst12*mt2*mu2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 504*msb22*mst14*mt2*mu2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb12*msb22*mt2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 252*msb12*mst12*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*msb22*mst12*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 252*msb12*mst14*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 168*msb12*msb22*
     mst12*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 504*msb12*
     mst14*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 84*msb12*
     msb22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     252*msb22*mst12*mst22*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) +
     504*msb22*mst22*mt2*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) +
     252*msb22*mu2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2)
     + 252*msb12*mst12*mst22*mu2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 504*msb12*mst22*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) -
     252*msb12*mu2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     + 252*msb22*mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     - 504*msb22*mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) -
     252*msb22*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     - 252*msb12*mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2)
     + 504*msb12*mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) +
     252*msb12*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2)
     + 252*msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2)
     + 504*msb22*mst12*mt2*mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) -
     252*msb22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2)
     - 252*msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2)
     - 504*msb12*mst12*mt2*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) +
     252*msb12*mt2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2)
     - 84*msb22*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*(-((5*msb12*mst12 + 3*
     mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(
     mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) - 1176*msb12*msb22*mst12*mst24*mu2*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 1176*msb12*msb22*mst22*mst24*mu2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 2352*msb12*msb22*mst24*mt2*mu2*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 420*msb12*msb22*mst12*mu2*power2(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 840*msb12*msb22*
     mt2*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 252*
     msb22*mst12*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) + 504*msb22*mt2*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) + 1176*msb12*msb22*mst12*mst24*mu2*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 1176*msb12*msb22*mst22*mst24*mu2*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) - 2352*msb12*msb22*mst24*mt2*mu2*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 420*msb12*msb22*mst12*mu2*power2(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 840*msb12*msb22*
     mt2*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 252*
     msb12*mst12*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)
     ) - 504*msb12*mt2*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) + 420*msb12*msb22*mst12*mu2*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) - 840*msb12*msb22*mt2*mu2*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 1176*msb12*msb22*mst12*mst24*
     mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 1176*msb12*msb22*
     mst22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 2352*
     msb12*msb22*mst24*mt2*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     252*msb22*mst12*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 504*msb22*mt2*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) - 420*msb12*msb22*mst12*mu2*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb22)) + 840*msb12*msb22*mt2*mu2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 1176*msb12*msb22*mst12*
     mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 1176*msb12*
     msb22*mst22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) +
     2352*msb12*msb22*mst24*mt2*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 252*msb12*mst12*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb22)) + 504*msb12*mt2*mu2*power3(mst22)*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) + 420*msb12*msb22*mst22*mt2*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 840*msb12*msb22*mt2*mu2*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 1176*msb12*msb22*
     mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 1176*
     msb12*msb22*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     + 2352*msb12*msb22*mst14*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 252*msb22*mst22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) + 504*msb22*mt2*mu2*power3(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 420*msb12*msb22*mst22*mt2*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 840*msb12*msb22*mt2*mu2*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 1176*msb12*msb22*
     mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 1176*
     msb12*msb22*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     - 2352*msb12*msb22*mst14*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 252*msb12*mst22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12)) - 504*msb12*mt2*mu2*power3(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 420*msb12*msb22*mst12*mst22*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) - 252*msb22*mst12*mst24*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 252*msb22*mst22*mst24*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 840*msb12*msb22*mst22*mt2*mu2*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 504*msb22*mst24*mt2*mu2*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 420*msb12*msb22*mu2*
     power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 420*msb12*
     msb22*mst12*mst22*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 252*
     msb12*mst12*mst24*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 252*
     msb12*mst22*mst24*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 840*
     msb12*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) -
     504*msb12*mst24*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) -
     420*msb12*msb22*mu2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 420*msb12*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 252*msb22*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 504*msb22*mt2*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 252*msb22*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 420*msb12*msb22*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 252*msb12*
     mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) +
     504*msb12*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mst12) + 252*msb12*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) + 84*msb12*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(
     mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) -
     5*msb22*power3(mst12) + 3*power4(mst12)) + 420*msb12*msb22*mu2*power2(mC2)
     *power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 252*msb22*mst12*mu2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 504*msb22*
     mt2*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 252*
     msb22*mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     420*msb12*msb22*mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mst22) - 252*msb12*mst12*mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power4(mst22) + 504*msb12*mt2*mu2*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22) - 252*msb12*mu2*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mst22) - 420*msb12*msb22*mu2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*msb22*mst12*mu2*power2(mw2)
     *power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 504*msb22*mt2*mu2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*msb22*
     mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 420*
     msb12*msb22*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mst22) + 252*msb12*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))
     *power4(mst22) - 504*msb12*mt2*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 252*msb12*mu2*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 420*msb12*msb22*mst12*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 252*msb22*mst12*mst24*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 252*msb22*mst22*mst24*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 840*msb12*msb22*mst22*mt2*mu2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 504*msb22*mst24*mt2*mu2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 420*msb12*msb22*mu2*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 420*msb12*
     msb22*mst12*mst22*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 252*
     msb12*mst12*mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 252*
     msb12*mst22*mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 840*
     msb12*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) +
     504*msb12*mst24*mt2*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) +
     420*msb12*msb22*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 252*msb22*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 420*msb12*msb22*mst12*mst22*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 252*msb22*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 840*msb12*msb22*mst12*mt2*mu2*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) + 504*msb22*mst14*mt2*mu2*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) - 420*msb12*msb22*mt2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 252*msb12*mst12*mst14*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 420*msb12*msb22*mst12*mst22*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 252*msb12*mst14*mst22*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 840*msb12*msb22*mst12*mt2*
     mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 504*msb12*mst14*mt2*
     mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 420*msb12*msb22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 252*msb22*
     mst12*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 504*msb22*
     mst22*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 252*msb22*
     mu2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 252*
     msb12*mst12*mst22*mu2*power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 504*
     msb12*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 252*
     msb12*mu2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) +
     252*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)
     - 252*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 252*msb22*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mst22) + 252*msb12*mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mst22) + 252*msb22*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mst22) - 252*msb12*mu2*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power5(mst22) - 252*msb22*mst12*mst22*mu2*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 504*msb22*mst22*mt2*mu2*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 252*msb22*mu2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) + 252*msb12*mst12*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) - 504*msb12*mst22*mt2*mu2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) - 252*msb12*mu2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 252*msb22*mst12*mst22*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 504*msb22*mst12*mt2*mu2*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 252*msb22*mt2*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 252*msb12*mst12*
     mst22*mt2*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) + 504*msb12*mst12*
     mt2*mu2*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) - 252*msb12*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(12.*msb12*
     msb22*(mst12 - mst22)) + (power2(cb)*power2(Pi)*power2(snb)*(msb22*(3*mC2*
     mst12 + msb12*mst12 + 3*mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - msb12*msb22*mst12*mst22*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - 3*msb22*mst12*mst24*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 3*msb22*mst22*mst24*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 2*msb12*msb22*mst22*mt2*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 6*msb22*mst24*mt2*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + msb12*msb22*mu2*power2(mC2)*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2)) + msb12*msb22*mst12*mst22*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst12*mst24*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst22*mst24*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 2*msb12*msb22*mst22*mt2*mu2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 6*msb12*mst24*mt2*mu2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mu2*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst12*
     mst22*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst12*
     mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst22*
     mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 2*msb12*msb22*
     mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb22*
     mst24*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*msb22*
     mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*
     msb22*mst12*mst22*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 3*
     msb12*mst12*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 3*
     msb12*mst22*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 2*
     msb12*msb22*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     6*msb12*mst24*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     msb12*msb22*mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)
     ) - 3*msb22*mst12*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     + msb12*msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) + 3*msb22*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     + 2*msb12*msb22*mst12*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) + 6*msb22*mst14*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) -
     msb12*msb22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) + 3*msb12*mst12*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - msb12*msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) - 3*msb12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - 2*msb12*msb22*mst12*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) - 6*msb12*mst14*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     msb12*msb22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) - 3*msb22*mst12*mst22*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2)
     + 6*msb22*mst22*mt2*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 3*
     msb22*mu2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 3*
     msb12*mst12*mst22*mu2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 6*
     msb12*mst22*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 3*
     msb12*mu2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 3*
     msb22*mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*
     msb22*mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 3*
     msb22*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 3*
     msb12*mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*
     msb12*mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 3*
     msb12*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 3*
     msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*
     msb22*mst12*mt2*mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 3*
     msb22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 3*
     msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 6*
     msb12*mst12*mt2*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 3*
     msb12*mt2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) -
     msb22*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)
     *power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     - 14*msb12*msb22*mst12*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 14*msb12*msb22*mst22*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) + 28*msb12*msb22*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 5*msb12*msb22*mst12*mu2*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) + 10*msb12*msb22*mt2*mu2*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst12*mu2*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 6*msb22*mt2*
     mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 14*
     msb12*msb22*mst12*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     - 14*msb12*msb22*mst22*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) - 28*msb12*msb22*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 5*msb12*msb22*mst12*mu2*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) - 10*msb12*msb22*mt2*mu2*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst12*mu2*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 6*msb12*mt2*mu2*power3(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*
     mst12*mu2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) -
     10*msb12*msb22*mt2*mu2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,
     mw2,msb12)) + 14*msb12*msb22*mst12*mst24*mu2*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) - 14*msb12*msb22*mst22*mst24*mu2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 28*msb12*msb22*mst24*mt2*mu2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst12*mu2*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 6*msb22*mt2*mu2*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 5*msb12*msb22*
     mst12*mu2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) +
     10*msb12*msb22*mt2*mu2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,
     mw2,msb22)) - 14*msb12*msb22*mst12*mst24*mu2*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb22)) + 14*msb12*msb22*mst22*mst24*mu2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) + 28*msb12*msb22*mst24*mt2*mu2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst12*mu2*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 6*msb12*mt2*mu2*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 5*msb12*msb22*
     mst22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     10*msb12*msb22*mt2*mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)) - 14*msb12*msb22*mst12*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 14*msb12*msb22*mst14*mst22*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 28*msb12*msb22*mst14*mt2*mu2*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst22*mt2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*msb22*mt2*mu2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*msb22*
     mst22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) -
     10*msb12*msb22*mt2*mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb22,mst12)) + 14*msb12*msb22*mst12*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst14*mst22*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 28*msb12*msb22*mst14*mt2*mu2*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst22*mt2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*msb12*mt2*mu2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*
     mst12*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*
     mst12*mst24*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*
     mst22*mst24*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 10*msb12*
     msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 6*
     msb22*mst24*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 5*
     msb12*msb22*mu2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) + 5*msb12*msb22*mst12*mst22*mu2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb12*mst12*mst24*mu2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 3*msb12*mst22*mst24*mu2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 10*msb12*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mC2) - 6*msb12*mst24*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2)
     )*power4(mC2) - 5*msb12*msb22*mu2*power2(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) - 5*msb12*msb22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 3*msb22*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*msb22*mt2*mu2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 3*msb22*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 5*msb12*msb22*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 3*msb12*
     mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 6*
     msb12*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     + 3*msb12*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     + msb12*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*
     power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(
     mst12) + 3*power4(mst12)) + 5*msb12*msb22*mu2*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) + 3*msb22*mst12*mu2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) - 6*msb22*mt2*mu2*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb22*mu2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 5*msb12*msb22*mu2*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb12*
     mst12*mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 6*
     msb12*mt2*mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)
     - 3*msb12*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)
     - 5*msb12*msb22*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 3*msb22*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 6*msb22*mt2*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 3*msb22*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 5*msb12*msb22*mu2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power4(mst22) + 3*msb12*mst12*mu2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 6*msb12*mt2*mu2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 3*msb12*mu2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 5*msb12*msb22*mst12*
     mst22*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst12*
     mst24*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*msb22*mst22*
     mst24*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 10*msb12*msb22*
     mst22*mt2*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*msb22*
     mst24*mt2*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 5*msb12*
     msb22*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 5*
     msb12*msb22*mst12*mst22*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 3*msb12*mst12*mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) +
     3*msb12*mst22*mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) +
     10*msb12*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     + 6*msb12*mst24*mt2*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 5*
     msb12*msb22*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 3*msb22*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 5*msb12*msb22*mst12*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 3*msb22*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 10*msb12*msb22*mst12*mt2*mu2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 6*msb22*mst14*mt2*mu2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 5*msb12*msb22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) + 3*msb12*mst12*mst14*mt2*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mw2) - 5*msb12*msb22*mst12*mst22*mt2*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) - 3*msb12*mst14*mst22*mt2*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) - 10*msb12*msb22*mst12*mt2*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 6*msb12*mst14*mt2*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 5*msb12*msb22*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*mst12*mst22*mu2*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 6*msb22*mst22*mt2*mu2*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb22*mu2*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst12*mst22*mu2*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 6*msb12*mst22*mt2*mu2*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb12*mu2*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 3*msb12*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mst12) - 3*msb22*mu2*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mst22) + 3*msb12*mu2*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mst22) + 3*msb22*mu2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 3*msb12*mu2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mst22) - 3*msb22*mst12*mst22*mu2*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 6*msb22*mst22*mt2*mu2*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 3*msb22*mu2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 3*msb12*mst12*mst22*mu2*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 6*msb12*mst22*mt2*mu2*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 3*msb12*mu2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 3*msb22*mst12*mst22*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 6*msb22*mst12*mt2*mu2*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 3*msb22*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 3*msb12*mst12*mst22*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) + 6*msb12*mst12*mt2*mu2*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) - 3*msb12*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22*(mst12 -
     mst22)) - (log(msb22/Q2)*power2(snb)*power2(snt)*(-(msb22*mst12*mt2) +
     msb22*mst22*mt2 + mC2*mst12*mu2 + msb22*mst12*mu2 - mC2*mst22*mu2 - msb22*
     mst22*mu2 - mst12*mt2*mw2 + mst22*mt2*mw2 + mst12*mst22*mst24*mt2*
     DeltaInv(mst22,mw2,msb22) - mst12*mst14*mst22*mt2*DeltaInv(mw2,msb22,
     mst12) + mst12*power2(msb22) - mst22*power2(msb22) + msb22*mst22*mt2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*mt2*mw2*DeltaInv(mw2,
     msb22,mst12)*power2(mst12) - msb22*mst12*mt2*DeltaInv(mst22,mw2,msb22)*
     power2(mst22) + 5*mst12*mt2*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22) -
     msb22*mst12*mt2*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*mst22*mt2*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*mst22*mt2*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 5*mst12*mst22*mt2*DeltaInv(mw2,msb22,mst12)*
     power2(mw2) + 6*msb22*mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,
     msb22)) - 138*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) - 36*msb22*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) + 138*mst12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) + 36*msb22*mst22*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + mst22*mu2*DeltaInv(msb22,mst12,mC2)*(mst12*(-
     (msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) - mst12*mu2*DeltaInv(mst22,msb22,mC2)*(mst22*(-(
     msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) - 6*mC2*mst12*mst22*mu2*power2(DeltaInv(msb22,mst12,
     mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*
     mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*mu2*
     power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 +
     23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     + mst12*mt2*DeltaInv(mst22,mw2,msb22)*power3(mw2) - mst22*mt2*DeltaInv(
     mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*mt2*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) - 138*mst12*mt2*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*mt2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*mt2*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*mu2*power2(
     mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-
     3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*mu2*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + 216*msb22*mst22*mt2*power2(mw2)
     *power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 144*mst22*mt2*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 6*mst12*mt2*mw2*
     power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 216*msb22*mst12*mt2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 144*mst12*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 6*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 6*mst12*mst22*
     mt2*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 144*mst12*mst22*mst24*
     mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 216*msb22*mst12*mt2*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 144*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 216*msb22*
     mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) -
     144*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mst12)
     + 144*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) - 144*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12)
     )*power5(mw2)))/(12.*msb22*mst12*mst22) - (power2(snb)*power2(snt)*(2*mC2*
     msb12*msb22*mst12*mst22*mu2*DeltaInv(msb12,mst12,mC2) - 2*mC2*msb12*msb22*
     mst12*mst22*mu2*DeltaInv(msb22,mst12,mC2) - 2*mC2*msb12*msb22*mst12*mst22*
     mu2*DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*msb22*mst12*mst22*mu2*
     DeltaInv(mst22,msb22,mC2) + 2*msb12*msb22*mst12*mst22*mt2*mw2*DeltaInv(
     mst22,mw2,msb12) - 2*msb12*msb22*mst12*mst22*mt2*mw2*DeltaInv(mst22,mw2,
     msb22) - 2*msb12*msb22*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 2*
     msb12*msb22*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst12*
     power2(msb12) - msb22*mst22*power2(msb12) - msb12*mst12*power2(msb22) +
     msb12*mst22*power2(msb22) + msb12*mst22*power2(mst12) - msb22*mst22*
     power2(mst12) - msb12*mst12*power2(mst22) + msb22*mst12*power2(mst22) -
     84*msb22*mst12*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*mst22*mu2*power2(mC2)
     *power2(DeltaInv(msb12,mst12,mC2)) + 84*msb12*mst12*(3*mC2*mst12 + msb22*
     mst12 + 3*mst14)*mst22*mu2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) +
     252*msb22*mst12*mst22*mst24*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) + 84*msb12*msb22*mst12*mu2*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - 252*msb12*mst12*mst22*mst24*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) - 84*msb12*msb22*mst12*mu2*power2(mC2)*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 252*msb22*mst12*mst22*
     mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*msb12*msb22*
     mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     252*msb12*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) + 84*msb12*msb22*mst12*mt2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 252*msb22*mst12*mst14*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*msb22*mst22*mt2*power2(mst12)
     *power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 252*msb12*mst12*mst14*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*msb22*
     mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     252*msb22*mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) - 252*msb12*mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,
     msb22,mC2))*power3(mC2) - 252*msb22*mst12*mt2*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 252*msb12*mst12*mt2*power2(mst22)
     *power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 252*msb22*mst22*mt2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 252*msb12*
     mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) +
     84*msb22*mst12*mst22*mu2*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(
     mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(
     14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     1176*msb12*msb22*mst12*mst22*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 1176*msb12*msb22*mst12*mst22*mst24*mu2*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 1176*msb12*msb22*mst12*mst22*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 1176*msb12*msb22*mst12*
     mst22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 1176*
     msb12*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 1176*msb12*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 252*msb22*mst12*mst22*mst24*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 420*msb12*msb22*mst12*mu2*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 252*msb12*mst12*
     mst22*mst24*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 420*msb12*
     msb22*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mC2) + 420*msb12*msb22*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 252*msb22*mst22*mt2*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 420*msb12*msb22*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 252*msb12*mst22*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 84*msb12*
     mst12*mst22*mu2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) + 420*msb12*msb22*mst12*mu2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 252*msb22*mst12*mu2*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) - 420*msb12*msb22*mst12*mu2*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 252*msb12*
     mst12*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) -
     420*msb12*msb22*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 252*msb22*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 420*msb12*msb22*mst12*mt2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 252*msb12*mst12*mt2*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 252*msb22*mst12*mst22*
     mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 420*msb12*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     252*msb12*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 420*msb12*msb22*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) + 252*msb22*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) + 420*msb12*msb22*mst22*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 252*msb12*mst12*mst14*
     mst22*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 420*msb12*msb22*
     mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) -
     252*msb22*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 252*msb12*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power5(mC2) - 252*msb22*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) + 252*msb12*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mst12) - 252*msb22*mst12*mu2*power2(mC2)
     *power3(DeltaInv(mst22,msb12,mC2))*power5(mst22) + 252*msb12*mst12*mu2*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power5(mst22) + 252*msb22*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) -
     252*msb12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 252*msb22*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 252*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(
     mst22,mw2,msb22))*power5(mw2) - 252*msb22*mst22*mt2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) + 252*msb12*mst22*mt2*power2(mst12)
     *power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(12.*msb12*msb22*mst12*
     mst22) - (power2(Pi)*power2(snt)*(-2*msb12*(3*mH2*mst22 + mst12*mst22 + 3*
     mst24)*mu2*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + msb12*mst12*
     mu2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 6*power3(mst12)) + 2*
     msb12*mu2*power2(mH2)*(-((5*mst12*mst22 + 3*mst24)*power2(mH2)) + 3*mst22*
     power3(mH2) + (-5*mst12 + 3*mst22)*power3(mst22) - mH2*(14*mst12*mst24 +
     3*power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 24*mH2*msb12*mu2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + mst12*((3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*mu2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     msb12*mst22*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 3*mst24*
     mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst22*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*mst24*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - msb12*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 3*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 3*mst22*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2)
     + msb12*mu2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 6*power3(mst22)
     ) + 3*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 3*mst12*
     mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 14*msb12*mst24*mu2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*mu2*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*mu2*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*mt2*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 5*msb12*mt2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 14*msb12*mst14*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*mt2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*mst22*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*mst24*mu2*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + mu2*power2(mC2)*power3(DeltaInv(msb12,
     mst12,mC2))*(14*mC2*msb12*mst14 + 5*msb12*mst12*power2(mC2) + 3*mst14*
     power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*msb12*power3(
     mst12) - 3*power4(mst12)) + 3*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 24*mH2*msb12*mu2*power3(DeltaInv(mH2,mst22,mst22))
     *power4(mst22) + 3*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mst22) - 3*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 5*msb12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 3*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     5*msb12*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*mst14*
     mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 3*mst22*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*mst22*mt2*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 3*mst12*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mw2))))/(6.*msb12*mst12) + (mu2*log(mst12/Q2)*power2(cb)*(
     2/mst12 + 2/mst22 + mh2/(mst12*mst22) + mz2/(mst12*mst22) - (8*(mh2 - 6*
     mst12)*mt2*DeltaInv(mh2,mst12,mst12))/(mst12 - mst22) + mst22*DeltaInv(
     mst22,mz2,mst12) - (mst24*DeltaInv(mst22,mz2,mst12))/mst12 - (5*mst22*mz2*
     DeltaInv(mst22,mz2,mst12))/mst12 - (4*mst12*mt2*DeltaInv(mw2,msb12,mst12))
     /(mst12 - mst22) + (4*mst14*mt2*DeltaInv(mw2,msb12,mst12))/(msb12*mst12 -
     msb12*mst22) + (20*mst12*mt2*mw2*DeltaInv(mw2,msb12,mst12))/(msb12*mst12 -
     msb12*mst22) - (5*DeltaInv(mst22,mz2,mst12)*power2(mz2))/mst12 + (
     DeltaInv(mst22,mz2,mst12)*power2(mz2))/mst22 - 6*mst24*mz2*power2(
     DeltaInv(mst22,mz2,mst12)) + 36*mst22*power2(mz2)*power2(DeltaInv(mst22,
     mz2,mst12)) + (138*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/
     mst12 + (24*mst14*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)))/(mst12 -
     mst22) + (72*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(
     mst12 - mst22) + (336*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)))/(msb12*mst12 - msb12*mst22) - (DeltaInv(mst22,mh2,mst12)*(mst22*(
     -(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(
     mst22) + power3(mh2)))/(mst12*mst22) - (24*mt2*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*power3(mst12))/(msb12*mst12 - msb12*mst22) + (192*mt2*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)))/(mst12 -
     mst22) + (6*mz2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22))/mst12 + (
     6*mh2*power2(DeltaInv(mst22,mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*
     mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mh2) + power3(mh2) + power3(
     mst22)))/mst12 - (24*mst12*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2))/(msb12*mst12 - msb12*mst22) - (DeltaInv(mst22,mz2,mst12)*power3(mz2)
     )/(mst12*mst22) - 6*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + (138*
     mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/mst12 + (72*power2(
     mh2)*((3*mst12*mst22 + 2*mst24)*power2(mh2) - 2*mst22*power3(mh2) + (3*
     mst12 - 2*mst22)*power3(mst22) + 2*mh2*(5*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)))/mst12 + 216*power2(mz2)*power3(mst22)*
     power3(DeltaInv(mst22,mz2,mst12)) + 720*mst24*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + (144*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,
     mz2,mst12)))/mst12 + (576*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(
     mw2,msb12,mst12)))/(mst12 - mst22) + (576*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (576*mt2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 - msb12*mst22)
     + (2304*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12))/(mst12 -
     mst22) - (288*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12))/(msb12*mst12 - msb12*mst22) - (144*power2(mz2)*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mst22))/mst12 - (288*mst14*mt2*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2))/(msb12*mst12 - msb12*mst22) + (6*power2(
     DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 + 216*mst22*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mz2) + (144*mst24*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2))/mst12 - (144*mst22*power3(DeltaInv(mst22,mz2,mst12))*
     power5(mz2))/mst12))/(24.*power2(sb)) - (mu2*power2(cb)*power2(log(mst12/
     Q2))*((-4*mh2*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)))/(mst12 - mst22)
     + ((2*mh2*mst22 + mst12*mst22 + 2*mst24)*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)))/mst12 + mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) + (2*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/mst12 +
     (4*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 -
     msb12*mst22) + (2*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/
     mst12 + (power2(mh2)*((3*mst12*mst22 + 2*mst24)*power2(mh2) - 2*mst22*
     power3(mh2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))/mst12 + 3*power2(mz2)*
     power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 10*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) + (2*power3(mst22)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)))/mst12 + (8*mt2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (8*mst14*mt2*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (8*mt2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 -
     msb12*mst22) - (4*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12))/(msb12*mst12 - msb12*mst22) - (2*power2(mz2)*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 - (4*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2))/(msb12*mst12 - msb12*mst22) + 3*
     mst22*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + (2*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 - (2*mst22*power3(DeltaInv(
     mst22,mz2,mst12))*power5(mz2))/mst12))/(2.*power2(sb)) + (mu2*power2(cb)*(
     -(1/msb12) - 1/mst12 - 2/mst22 + (6*mt2)/((mst12 - mst22)*mst22) + (4*mh2*
     mt2*DeltaInv(mh2,mst12,mst12))/(mst12 - mst22) + mh2*DeltaInv(mst22,mh2,
     mst12) - (4*mh2*mt2*DeltaInv(mst22,mst22,mh2))/(mst12 - mst22) + 2*mw2*
     DeltaInv(mst22,mw2,msb12) - (4*mt2*mw2*DeltaInv(mst22,mw2,msb12))/(mst12 -
     mst22) + mz2*DeltaInv(mst22,mz2,mst12) + (4*mt2*mw2*DeltaInv(mw2,msb12,
     mst12))/(mst12 - mst22) - (6*mt2)/(-(mst12*mst22) + power2(mst12)) - (42*(
     3*mh2*mst22 + mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12)))/mst12 + (168*mh2*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)))
     /(mst12 - mst22) - 84*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - (252*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/msb12 + (168*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) +
     (504*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(msb12*mst12
     - msb12*mst22) - 42*mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) -
     (126*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/mst12 - (168*
     mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) -
     (504*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12
     - msb12*mst22) - (168*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 +
     6*power3(mst12)))/(mst12 - mst22) + (1008*mt2*power2(DeltaInv(mst22,mst22,
     mh2))*power3(mst22))/(mst12 - mst22) - (252*mst22*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2))/msb12 + (504*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (504*mst12*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (
     126*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/mst12 - 420*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + (840*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 -
     mst22) - 1176*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + (2352*
     mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) -
     (252*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 +
     (504*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(
     msb12*mst12 - msb12*mst22) - 210*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) - 588*mst24*power3(mz2)*power3(DeltaInv(mst22,
     mz2,mst12)) - (126*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)))/mst12 - (840*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)))/(mst12 - mst22) - (2352*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) - (504*mt2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 - msb12*mst22)
     - (4032*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12))/(mst12 -
     mst22) + (504*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12))/(msb12*mst12 - msb12*mst22) - (42*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*(14*mh2*mst12*mst24 + 5*mst12*mst22*power2(mh2) + 3*
     mst24*power2(mh2) - 3*mst22*power3(mh2) + 3*mh2*power3(mst22) + 5*mst12*
     power3(mst22) - 3*power4(mst22)))/mst12 + (4032*mh2*mt2*power3(DeltaInv(
     mst22,mst22,mh2))*power4(mst22))/(mst12 - mst22) + (252*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/msb12 - (504*mt2*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/(msb12*mst12 -
     msb12*mst22) + (126*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(
     mst22))/mst12 - 420*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     (252*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/msb12 + (840*
     mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(mst12 - mst22) +
     (504*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(msb12*mst12
     - msb12*mst22) - (840*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2))/(mst12 - mst22) - (504*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2))/(msb12*mst12 - msb12*mst22) - 210*mst22*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mz2) - (126*mst24*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2))/mst12 + (252*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2))/msb12 + (504*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2))/(msb12*(-mst12 + mst22)) + (504*mst12*mt2*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mw2))/(msb12*mst12 - msb12*mst22) + (126*mst22*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))/mst12))/(12.*power2(sb)) -
     (mu2*power2(cb)*power2(Pi)*(4*mh2*msb12*mst12*mst14*mt2*power2(DeltaInv(
     mh2,mst12,mst12)) + msb12*mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + 3*msb12*(mst12 - mst22)*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 4*mh2*msb12*mst12*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) + 2*msb12*mst12*(mst12 - mst22)*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*mst12*(mst12 - mst22)*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst22*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 12*mst12*mst24*mt2*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*mst12*(mst12 - mst22)*
     mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 3*msb12*(mst12 -
     mst22)*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 12*mst12*
     mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 4*msb12*mt2*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb12*(
     mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 24*
     msb12*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mst12*
     (mst12 - mst22)*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 12*
     mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 12*mt2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*(
     mst12 - mst22)*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 14*
     msb12*mst12*(mst12 - mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) + 5*msb12*mst12*(mst12 - mst22)*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 3*msb12*(mst12 - mst22)*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 10*msb12*mst12*(mst12 - mst22)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 20*msb12*
     mst12*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     28*msb12*mst12*(mst12 - mst22)*mst24*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) - 56*msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 6*mst12*(mst12 - mst22)*power3(mst22)*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 12*mst12*mt2*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*mst12*(mst12 - mst22)*power2(
     mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 14*msb12*mst12*(
     mst12 - mst22)*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 3*
     msb12*(mst12 - mst22)*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)) + 56*msb12*mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 5*msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) + 3*msb12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 24*msb12*mt2*power2(DeltaInv(mh2,mst12,mst12))*
     power4(mst12) + 20*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mst12) + 12*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 3*msb12*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) - 96*mh2*msb12*mst12*mt2*power3(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) - 6*mst12*(mst12 - mst22)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 12*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*(mst12 - mst22)*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 10*msb12*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 6*mst12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 20*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 12*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     12*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 20*
     msb12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*
     msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 3*msb12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 3*msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) + 96*mh2*msb12*mt2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) - 12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mst12) - 6*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 12*mst12*mst22*mt2*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mw2) - 12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mw2) - 3*msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,
     mz2,mst12))*power5(mz2)))/(12.*msb12*mst12*(mst12 - mst22)*power2(sb)) - (
     mu2*power2(cb)*(-2*mC2*msb12*mst12*mt2*DeltaInv(msb12,mst12,mC2) - mA2*
     msb12*mst12*(mst12 - mst22)*DeltaInv(mst22,mA2,mst12) - 2*mC2*msb12*mst12*
     (mst12 - mst22)*DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*mst12*mt2*
     DeltaInv(mst22,msb12,mC2) + 2*msb12*mst12*(mst12 - mst22)*mw2*DeltaInv(
     mst22,mw2,msb12) - 2*msb12*mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12) +
     msb12*mst12*(mst12 - mst22)*mz2*DeltaInv(mst22,mz2,mst12) + 2*msb12*mst12*
     mt2*mw2*DeltaInv(mw2,msb12,mst12) + 252*mst12*mst14*mt2*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) + 84*msb12*mt2*power2(mC2)*power2(mst12)
     *power2(DeltaInv(msb12,mst12,mC2)) + 42*msb12*mst12*(mst12 - mst22)*mst22*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 126*msb12*(mst12 - mst22)*
     mst24*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 84*msb12*mst12*(
     mst12 - mst22)*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 252*
     mst12*(mst12 - mst22)*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     - 84*msb12*mst12*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) -
     252*mst12*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb12*mst12*(mst12 - mst22)*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 252*mst12*(mst12 - mst22)*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 84*msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 252*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 42*msb12*mst12*(mst12 - mst22)*mst22*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) - 126*msb12*(mst12 - mst22)*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - 252*mst12*mst14*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - 84*msb12*mt2*power2(mst12)*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) + 126*msb12*(mst12 - mst22)*mst22*
     power2(DeltaInv(mst22,mA2,mst12))*power3(mA2) + 252*mt2*power2(mst12)*
     power2(DeltaInv(msb12,mst12,mC2))*power3(mC2) + 252*mst12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 252*mst12*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 252*mst12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 252*mst12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 252*mt2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 126*msb12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 1176*msb12*mst12*
     mst14*mt2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 588*msb12*mst12*
     (mst12 - mst22)*mst24*power3(mA2)*power3(DeltaInv(mst22,mA2,mst12)) + 210*
     msb12*mst12*(mst12 - mst22)*power2(mA2)*power3(mst22)*power3(DeltaInv(
     mst22,mA2,mst12)) + 126*msb12*(mst12 - mst22)*power3(mA2)*power3(mst22)*
     power3(DeltaInv(mst22,mA2,mst12)) + 1176*msb12*mst12*(mst12 - mst22)*
     mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 1176*msb12*mst12*
     mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 420*msb12*mst12*
     (mst12 - mst22)*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 420*msb12*mst12*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) + 252*mst12*(mst12 - mst22)*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) - 252*mst12*mt2*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 420*msb12*mst12*(mst12 - mst22)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 420*msb12*
     mst12*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) -
     1176*msb12*mst12*(mst12 - mst22)*mst24*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 1176*msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) - 252*mst12*(mst12 - mst22)*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 252*mst12*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*msb12*mst12*(mst12 - mst22)*
     power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 588*msb12*
     mst12*(mst12 - mst22)*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     - 126*msb12*(mst12 - mst22)*power3(mst22)*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) - 1176*msb12*mst12*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 210*msb12*mst12*(mst12 - mst22)*mst22*power3(
     DeltaInv(mst22,mA2,mst12))*power4(mA2) + 126*msb12*(mst12 - mst22)*mst24*
     power3(DeltaInv(mst22,mA2,mst12))*power4(mA2) + 252*mst12*mst14*mt2*
     power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 420*msb12*mt2*power2(
     mst12)*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 420*msb12*mst12*(
     mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 252*
     mst12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     - 420*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     - 252*mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 420*
     msb12*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) +
     252*mt2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) - 420*
     msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     252*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*
     msb12*(mst12 - mst22)*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*
     power4(mst22) - 252*mst12*(mst12 - mst22)*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) + 252*mst12*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 252*mst12*(mst12 - mst22)*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*mst12*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*
     msb12*(mst12 - mst22)*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 420*msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 252*mst12*(mst12 - mst22)*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 420*msb12*mst12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 252*mst12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 252*mst12*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 420*msb12*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 210*msb12*mst12*(mst12 -
     mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*msb12*(
     mst12 - mst22)*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mA2,mst12))*power5(mA2)
     - 252*mt2*power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) -
     252*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mC2) + 252*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) -
     252*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power5(mst12) + 252*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 252*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2)
     - 252*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 252*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))
     )/(6.*msb12*mst12*(mst12 - mst22)) - (mu2*power4(cb)*(-4*mC2*msb12*mst12*
     mt2*DeltaInv(msb12,mst12,mC2) - mA2*msb12*mst12*(mst12 - mst22)*DeltaInv(
     mst22,mA2,mst12) - 2*mC2*msb12*mst12*(mst12 - mst22)*DeltaInv(mst22,msb12,
     mC2) + 4*mC2*msb12*mst12*mt2*DeltaInv(mst22,msb12,mC2) + 2*msb12*mst12*(
     mst12 - mst22)*mw2*DeltaInv(mst22,mw2,msb12) - 4*msb12*mst12*mt2*mw2*
     DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 - mst22)*mz2*DeltaInv(
     mst22,mz2,mst12) + 4*msb12*mst12*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 504*
     mst12*mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + 168*msb12*
     mt2*power2(mC2)*power2(mst12)*power2(DeltaInv(msb12,mst12,mC2)) + 42*
     msb12*mst12*(mst12 - mst22)*mst22*power2(mA2)*power2(DeltaInv(mst22,mA2,
     mst12)) + 126*msb12*(mst12 - mst22)*mst24*power2(mA2)*power2(DeltaInv(
     mst22,mA2,mst12)) + 84*msb12*mst12*(mst12 - mst22)*mst22*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 252*mst12*(mst12 - mst22)*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 168*msb12*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 504*mst12*mst24*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb12*mst12*(mst12 -
     mst22)*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 252*mst12*(
     mst12 - mst22)*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 168*
     msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 504*
     mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*
     mst12*(mst12 - mst22)*mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))
     - 126*msb12*(mst12 - mst22)*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) - 504*mst12*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) - 168*msb12*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 126*msb12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mA2,mst12)
     )*power3(mA2) + 504*mt2*power2(mst12)*power2(DeltaInv(msb12,mst12,mC2))*
     power3(mC2) + 252*mst12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 504*mst12*mst22*mt2*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) - 252*mst12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 504*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12)
     )*power3(mw2) - 504*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 126*msb12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mz2,
     mst12))*power3(mz2) + 2352*msb12*mst12*mst14*mt2*power3(mC2)*power3(
     DeltaInv(msb12,mst12,mC2)) + 588*msb12*mst12*(mst12 - mst22)*mst24*power3(
     mA2)*power3(DeltaInv(mst22,mA2,mst12)) + 210*msb12*mst12*(mst12 - mst22)*
     power2(mA2)*power3(mst22)*power3(DeltaInv(mst22,mA2,mst12)) + 126*msb12*(
     mst12 - mst22)*power3(mA2)*power3(mst22)*power3(DeltaInv(mst22,mA2,mst12))
     + 1176*msb12*mst12*(mst12 - mst22)*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 2352*msb12*mst12*mst24*mt2*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 420*msb12*mst12*(mst12 - mst22)*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 840*msb12*mst12*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 252*mst12*(
     mst12 - mst22)*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     - 504*mst12*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 420*msb12*mst12*(mst12 - mst22)*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) + 840*msb12*mst12*mt2*power2(mw2)*power3(mst22)
     *power3(DeltaInv(mst22,mw2,msb12)) - 1176*msb12*mst12*(mst12 - mst22)*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 2352*msb12*mst12*
     mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 252*mst12*(mst12
     - mst22)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     504*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 210*msb12*mst12*(mst12 - mst22)*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) - 588*msb12*mst12*(mst12 - mst22)*mst24*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 126*msb12*(mst12 - mst22)*power3(
     mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 2352*msb12*mst12*
     mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*msb12*mst12*
     (mst12 - mst22)*mst22*power3(DeltaInv(mst22,mA2,mst12))*power4(mA2) + 126*
     msb12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mA2,mst12))*power4(mA2)
     + 504*mst12*mst14*mt2*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 840*
     msb12*mt2*power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) +
     420*msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) + 252*mst12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 840*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 504*mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) + 840*msb12*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*
     power4(mst12) + 504*mt2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2))*
     power4(mst12) - 840*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) - 504*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 126*msb12*(mst12 - mst22)*power2(mA2)*power3(DeltaInv(
     mst22,mA2,mst12))*power4(mst22) - 252*mst12*(mst12 - mst22)*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 504*mst12*mt2*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 252*mst12*(mst12 -
     mst22)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 504*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     126*msb12*(mst12 - mst22)*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 420*msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 252*mst12*(mst12 - mst22)*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 840*msb12*mst12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 504*mst12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 504*mst12*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 840*msb12*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 210*msb12*mst12*(mst12 -
     mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*msb12*(
     mst12 - mst22)*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mA2,mst12))*power5(mA2)
     - 504*mt2*power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) -
     252*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mC2) + 504*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) -
     504*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power5(mst12) + 504*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 252*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2)
     - 504*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 504*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))
     )/(12.*msb12*mst12*(mst12 - mst22)*power2(sb)) + (power2(cb)*power2(Pi)*(-
     (mA2*mt2*DeltaInv(mt2,mt2,mA2)) + mt2*mz2*DeltaInv(mt2,mt2,mz2) - 7*
     DeltaInv(mt2,mt2,mA2)*power2(mt2) + 7*DeltaInv(mt2,mt2,mz2)*power2(mt2) -
     (2*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*mt2*(mst12 - mst22 - 2*mu2)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)))/(msb12*(mst12 - mst22)) + (
     (3*mA2*mst22 + mst12*mst22 + 3*mst24)*mu2*power2(mA2)*power2(DeltaInv(
     mst22,mA2,mst12)))/mst12 + 2*mst22*mu2*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + (6*mst24*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)))/
     msb12 - (4*mst22*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)))/(
     mst12 - mst22) - (12*mst24*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)))/(msb12*mst12 - msb12*mst22) - 2*mst22*mu2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - (6*mst24*mu2*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)))/msb12 + (4*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)))/(mst12 - mst22) + (12*mst24*mt2*mu2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)))/(msb12*mst12 - msb12*mst22) - mst22*mu2*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - (3*mst24*mu2*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)))/mst12 + 2*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + (6*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)))/msb12 - (4*mst12*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)))/(mst12 - mst22) - (12*mst14*mt2*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 - msb12*mst22) + (6*mst22*mu2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2))/msb12 + (12*mst22*mt2*mu2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2))/(msb12*(-mst12 + mst22)) -
     34*mA2*power2(DeltaInv(mt2,mt2,mA2))*power3(mt2) + 34*mz2*power2(DeltaInv(
     mt2,mt2,mz2))*power3(mt2) - (6*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2))/msb12 + (12*mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2))/(msb12*mst12 - msb12*mst22) + (6*mst12*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2))/msb12 - (12*mst12*mt2*mu2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (3*mst22*mu2*
     power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/mst12 + (2*mt2*(mst12 -
     mst22 - 2*mu2)*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*
     mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)))/(msb12*(mst12
     - mst22)) + 28*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - (
     56*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)))/(mst12 -
     mst22) + 10*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - (20*mt2*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ))/(mst12 - mst22) + (6*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)))/msb12 - (12*mt2*mu2*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)))/(msb12*mst12 - msb12*mst22) - 10*mu2*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + (20*mt2*mu2*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) -
     28*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + (56*mst24*
     mt2*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - (
     6*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 +
     (12*mt2*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(
     msb12*mst12 - msb12*mst22) - 5*mu2*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) - 14*mst24*mu2*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) - (3*mu2*power3(mst22)*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)))/mst12 + 10*mt2*power2(mw2)*power3(mst12)*power3(
     DeltaInv(mw2,msb12,mst12)) - (20*mt2*mu2*power2(mw2)*power3(mst12)*power3(
     DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + 28*mst14*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - (56*mst14*mt2*mu2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (6*mt2*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)))/msb12 - (12*mt2*mu2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 - msb12*mst22)
     + 10*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + (6*mst24*
     mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2))/msb12 - (20*mst22*mt2*
     mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2))/(mst12 - mst22) - (12*
     mst24*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2))/(msb12*mst12
     - msb12*mst22) - (6*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12))/msb12 + (12*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12))/(msb12*mst12 - msb12*mst22) + (mu2*power2(mA2)*
     power3(DeltaInv(mst22,mA2,mst12))*(14*mA2*mst12*mst24 + 5*mst12*mst22*
     power2(mA2) + 3*mst24*power2(mA2) - 3*mst22*power3(mA2) + 3*mA2*power3(
     mst22) + 5*mst12*power3(mst22) - 3*power4(mst22)))/mst12 - (6*mu2*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22))/msb12 + (12*mt2*mu2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22))/(msb12*mst12
     - msb12*mst22) + (6*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22))/msb12 - (12*mt2*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22))/(msb12*mst12 - msb12*mst22) + (3*mu2*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 - 24*power2(
     DeltaInv(mt2,mt2,mA2))*power4(mt2) + 24*power2(DeltaInv(mt2,mt2,mz2))*
     power4(mt2) - 10*mst22*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     (6*mst24*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/msb12 + (20*
     mst22*mt2*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(mst12 -
     mst22) + (12*mst24*mt2*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/
     (msb12*mst12 - msb12*mst22) + 10*mst12*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + (6*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2))/msb12 - (20*mst12*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2))/(mst12 - mst22) - (12*mst14*mt2*mu2*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2))/(msb12*mst12 - msb12*mst22) - 5*mst22*mu2*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - (3*mst24*mu2*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 - (6*mst22*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2))/msb12 + (12*mst22*mt2*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2))/(msb12*mst12 - msb12*mst22) - 96*
     mA2*power3(DeltaInv(mt2,mt2,mA2))*power5(mt2) + 96*mz2*power3(DeltaInv(
     mt2,mt2,mz2))*power5(mt2) + (6*mst22*mu2*power3(DeltaInv(mst22,mw2,msb12))
     *power5(mw2))/msb12 + (12*mst22*mt2*mu2*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2))/(msb12*(-mst12 + mst22)) - (6*mst12*mt2*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2))/msb12 + (12*mst12*mt2*mu2*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2))/(msb12*mst12 - msb12*mst22) + (3*mst22*mu2*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))/mst12))/12. + (mu2*power2(
     log(mst12/Q2))*power4(cb)*(-4*mst12*mst14*mt2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)) - msb12*(mst12 - mst22)*(2*mA2*mst22 + mst12*mst22 + 2*
     mst24)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 2*msb12*mst12*
     mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 2*msb12*mst22*mst24*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(mst12)*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(mst22)*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 4*mst12*mst14*mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 2*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mz2) - 2*msb12*power2(mst22)*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mz2) + 4*mst12*mt2*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - msb12*(mst12 - mst22)*
     power2(mA2)*((3*mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*power3(mA2) +
     (3*mst12 - 2*mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 + power3(mst22)))
     *power3(DeltaInv(mst22,mA2,mst12)) + 3*msb12*power2(mst12)*power2(mz2)*
     power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 10*msb12*mst12*mst22*
     mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 10*msb12*mst24*
     power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 2*msb12*
     mst12*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 8*
     msb12*mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 8*
     msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 8*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 5*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 2*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 4*
     mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 2*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 3*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 3*msb12*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 4*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 2*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 2*msb12*
     mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 2*msb12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(2.*msb12*
     mst12*(mst12 - mst22)*power2(sb)) + (mu2*power2(cb)*power2(log(mst12/Q2))*
     (-2*mst12*mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*
     (mst12 - mst22)*(2*mA2*mst22 + mst12*mst22 + 2*mst24)*power2(mA2)*power2(
     DeltaInv(mst22,mA2,mst12)) + 2*msb12*mst12*mst24*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) - 2*msb12*mst22*mst24*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(mst12)*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(mst22)*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) + 2*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*mst12*mst22*power2(DeltaInv(mst22,
     mz2,mst12))*power3(mz2) - 2*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,
     mst12))*power3(mz2) + 2*mst12*mt2*power2(mC2)*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - msb12*(mst12 - mst22)*power2(mA2)*((3*
     mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*
     mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mA2,mst12)) + 3*msb12*power2(mst12)*power2(mz2)*power3(
     mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 10*msb12*mst12*mst22*mst24*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 10*msb12*mst24*power2(
     mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 2*msb12*mst12*
     power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 4*msb12*
     mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 4*msb12*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 4*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 5*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 2*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 2*
     mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 2*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 3*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 3*msb12*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 2*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 2*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 2*msb12*
     mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 2*msb12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(msb12*
     mst12*(mst12 - mst22)) + (mu2*power2(Pi)*power4(cb)*(-4*mst12*(3*mC2*mst12
     + msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))
     - msb12*(mst12 - mst22)*(3*mA2*mst22 + mst12*mst22 + 3*mst24)*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12)) + 6*mst12*mst22*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 4*msb12*mst12*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 12*mst12*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 2*msb12*mst22*power2(mC2)*power2(mst12)*
     power2(DeltaInv(mst22,msb12,mC2)) - 6*mst24*power2(mC2)*power2(mst12)*
     power2(DeltaInv(mst22,msb12,mC2)) + 2*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mst12*mst22*mst24*power2(mw2)
     *power2(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 12*mst12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*msb12*mst22*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 6*mst24*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 2*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb12*mst12*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - 3*msb12*mst22*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(mst12)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(mst22)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + 12*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 4*msb12*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 12*mst12*mst22*mt2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 6*mst22*power2(mst12)*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) + 6*mst12*power2(mst22)*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 12*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 3*msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12)
     )*power3(mz2) - 3*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mz2) + 4*mst12*mt2*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(
     mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(
     14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 28*
     msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     56*msb12*mst12*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     28*msb12*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     + 20*msb12*mst12*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) - 10*msb12*power2(mC2)*power2(mst12)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 12*mst12*mt2*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 6*power2(mst12)*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 20*msb12*mst12*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 10*msb12*power2(mst12)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 28*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 56*
     msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 28*
     msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     12*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     6*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     ) + 5*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,
     mz2,mst12)) - 14*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + 14*msb12*mst24*power2(mst12)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)) + 3*msb12*mst12*power3(mst22)*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) + 56*msb12*mst12*mst14*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*mst12*mst22*mst24*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 20*msb12*mst12*mst22*mt2*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 12*mst12*mst24*mt2*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mC2) - 10*msb12*mst22*power2(mst12)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 6*mst24*power2(mst12)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 10*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 20*msb12*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 12*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - msb12*(mst12 - mst22)*power2(
     mA2)*power3(DeltaInv(mst22,mA2,mst12))*(14*mA2*mst12*mst24 + 5*mst12*
     mst22*power2(mA2) + 3*mst24*power2(mA2) - 3*mst22*power3(mA2) + 3*mA2*
     power3(mst22) + 5*mst12*power3(mst22) - 3*power4(mst22)) + 10*msb12*mst12*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 12*mst12*
     mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 6*
     power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22)
     + 6*mst12*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     10*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     + 12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     - 6*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 6*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 8*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 3*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 6*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 20*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 12*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 10*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 10*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 12*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) + 20*msb12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 3*msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 3*msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 5*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mz2,mst12)
     )*power4(mz2) - 5*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))
     *power5(mC2) + 6*mst22*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 6*mst12*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 6*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mst22) + 6*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 3*msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(
     mst22) + 12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2)
     - 6*mst22*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 6*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 12*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*
     mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 3*msb12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(12.*msb12*
     mst12*(mst12 - mst22)*power2(sb)) + (mu2*power2(cb)*power2(Pi)*(-2*mst12*(
     3*mC2*mst12 + msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)) - msb12*(mst12 - mst22)*(3*mA2*mst22 + mst12*mst22 + 3*
     mst24)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 6*mst12*mst22*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 2*msb12*mst12*mst22*
     mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 6*mst12*mst24*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 2*msb12*mst22*power2(mC2)*
     power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mst24*power2(mC2)*
     power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 2*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mst12*mst22*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 2*msb12*mst12*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*mst12*mst24*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*msb12*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*mst24*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 2*msb12*mst12*
     power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb12*
     mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 3*msb12*mst22*
     mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(
     mst12)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(
     mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 6*mst12*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 2*msb12*mt2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*mst12*mst22*mt2*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*mst22*power2(mst12)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) + 6*mst12*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*mst22*power2(mst12)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mz2) - 3*msb12*power2(mst22)*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mz2) + 2*mst12*mt2*power2(mC2)*(-((5*
     msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*
     mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 28*msb12*mst12*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 28*msb12*mst12*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 28*msb12*mst24*power2(mst12)*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 10*msb12*mst12*mt2*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 10*msb12*power2(mC2)*
     power2(mst12)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 6*mst12*
     mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 6*
     power2(mst12)*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     - 10*msb12*mst12*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) + 10*msb12*power2(mst12)*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) - 28*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 28*msb12*mst12*mst24*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 28*msb12*mst24*power2(mst12)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 6*mst12*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 6*power2(mst12)*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*power2(mst12)*
     power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 14*msb12*
     mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 14*
     msb12*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) +
     3*msb12*mst12*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     + 28*msb12*mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     6*mst12*mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 10*
     msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 6*
     mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 10*msb12*
     mst22*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 6*
     mst24*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 10*
     msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) +
     10*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) +
     6*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - msb12*
     (mst12 - mst22)*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*(14*mA2*
     mst12*mst24 + 5*mst12*mst22*power2(mA2) + 3*mst24*power2(mA2) - 3*mst22*
     power3(mA2) + 3*mA2*power3(mst22) + 5*mst12*power3(mst22) - 3*power4(
     mst22)) + 10*msb12*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mst22) - 6*mst12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mst22) + 6*power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) + 6*mst12*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     )*power4(mst22) - 10*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 6*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 6*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) - 6*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 8*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mst22) - 3*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mst22) - 6*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 10*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 6*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 10*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 6*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 10*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mw2) + 6*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 10*msb12*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) + 3*msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 3*msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12)
     )*power4(mz2) + 5*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 5*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mz2) - 6*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,
     mC2))*power5(mC2) + 6*mst22*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2)
     )*power5(mC2) - 6*mst12*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 6*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 6*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mst22) + 6*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 3*msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(
     mst22) + 6*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) -
     6*mst22*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 6*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 6*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*
     mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 3*msb12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(6.*msb12*
     mst12*(mst12 - mst22)) - (log(mh2/Q2)*power2(sa)*(18*mh2*mst12*mst22 + 12*
     mh2*mst12*mt2 + 12*mh2*mst22*mt2 + 8*mst12*mst22*mt2 + 12*mh2*(mh2 - 6*
     mst12)*mst12*mst22*mt2*DeltaInv(mh2,mst12,mst12) - 4*mh2*mst12*mst22*mt2*(
     29*mh2 + 154*mt2)*DeltaInv(mh2,mt2,mt2) + 3*mst12*power2(mh2) + 3*mst22*
     power2(mh2) + 3*mu2*power2(mh2) + 12*mst12*mst22*mt2*DeltaInv(mst22,mst22,
     mh2)*power2(mh2) - 72*mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2)*power2(
     mst22) + 144*mst12*mst22*mst24*mt2*power2(mh2)*power2(DeltaInv(mst22,
     mst22,mh2)) + 3*mst12*mu2*DeltaInv(mst22,mh2,mst12)*power3(mh2) - 15*
     mst22*mu2*DeltaInv(mst22,mh2,mst12)*power3(mh2) + 18*mst22*mst24*mu2*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 54*mst12*mu2*power2(mst22)
     *power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 144*mh2*mst12*mst22*mt2*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) - 2592*
     mst12*mst22*power2(mh2)*power2(DeltaInv(mh2,mt2,mt2))*power3(mt2) - 3*mu2*
     DeltaInv(mst22,mh2,mst12)*power4(mh2) - 18*mst12*mst22*mu2*power2(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mu2*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 432*mst12*mst22*mst24*mu2*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 1728*mh2*mst12*mt2*power2(
     DeltaInv(mst22,mst22,mh2))*power4(mst22) + 216*mu2*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2)*power4(mst22) + 18*mst22*mu2*power2(DeltaInv(
     mst22,mh2,mst12))*power5(mh2) - 432*mst22*mst24*mu2*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) - 432*mst12*mu2*power2(mst22)*power3(DeltaInv(
     mst22,mh2,mst12))*power5(mh2) + 6912*mst22*mt2*power2(mh2)*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) + 6912*mst12*mt2*power2(mh2)*
     power3(DeltaInv(mst22,mst22,mh2))*power5(mst22) + 216*mu2*power2(mst22)*
     power3(DeltaInv(mst22,mh2,mst12))*power6(mh2)))/(72.*mh2*mst12*mst22) + (
     mt2*power2(DeltaInv(mw2,msb12,mst12))*power2(log(mw2/Q2))*power3(mw2)*(-
     mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1))))/msb12 + (mw2*log(mw2/Q2)*(-(mst22*(
     mst12 - mt2)*(mt2 - mw2)) + msb12*mst12*(-4*mst22 - mt2 + mw2) + mst22*
     mt2*(msb12 - 5*mst12 - mw2)*(mt2 - mw2)*mw2*DeltaInv(mw2,msb12,mst12) + 6*
     mst12*mst22*mt2*(mt2 - mw2)*mw2*(mst14 + mw2*(-14*mst12 + mw2) - msb12*(3*
     mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) + 72*mst12*mst22*mt2*(mt2
     - mw2)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(12.*msb12*
     mst12*mst22*(-mt2 + mw2)) - (mw2*log(mw2/Q2)*power2(snb)*(-((msb12 -
     msb22)*(mst12 - mt2)) + msb22*mt2*mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(
     mw2,msb12,mst12) + msb12*mt2*(msb22 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,
     msb22,mst12) + 6*msb22*mst12*mt2*mw2*(-mst14 + (14*mst12 - mw2)*mw2 +
     msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*mst12*
     mt2*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst12*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1)) + 72*msb12*mst12*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/(12.*msb12*msb22*mst12) + Fin3(mh2,mst12,
     mst12,Q2)*(-2*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mst14 + 4*mh2*
     DeltaInv(mh2,mst12,mst12)*power6(mst1)) + 2*mt2*power2(sa)*power2(
     DeltaInv(mh2,mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*
     power6(mst1)) - (4*ca*cb*mu2*sa*power2(snt)*power2(DeltaInv(mh2,mst12,
     mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)))/sb - 2*
     mu2*power2(sa)*power2(snt)*power2(DeltaInv(mh2,mst12,mst12))*(mst14 + 4*
     mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)) - (2*mu2*power2(cb)*power2(
     snt)*power2(DeltaInv(mh2,mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,
     mst12)*power6(mst1)))/power2(sb) + (2*mu2*power2(cb)*power2(sa)*power2(
     snt)*power2(DeltaInv(mh2,mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,
     mst12)*power6(mst1)))/power2(sb) + (4*ca*cb*mu2*sa*power2(DeltaInv(mh2,
     mst12,mst12))*power4(snt)*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(
     mst1)))/sb + 2*mu2*power2(sa)*power2(DeltaInv(mh2,mst12,mst12))*power4(
     snt)*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)) + (2*mu2*
     power2(cb)*power2(DeltaInv(mh2,mst12,mst12))*power4(snt)*(mst14 + 4*mh2*
     DeltaInv(mh2,mst12,mst12)*power6(mst1)))/power2(sb) - (2*mu2*power2(cb)*
     power2(sa)*power2(DeltaInv(mh2,mst12,mst12))*power4(snt)*(mst14 + 4*mh2*
     DeltaInv(mh2,mst12,mst12)*power6(mst1)))/power2(sb)) + Fin3(mH2,mst12,
     mst12,Q2)*(-2*mt2*power2(sa)*power2(DeltaInv(mH2,mst12,mst12))*(mst14 + 4*
     mH2*DeltaInv(mH2,mst12,mst12)*power6(mst1)) - 2*mu2*power2(snt)*power2(
     DeltaInv(mH2,mst12,mst12))*(mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*
     power6(mst1)) + (4*ca*cb*mu2*sa*power2(snt)*power2(DeltaInv(mH2,mst12,
     mst12))*(mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*power6(mst1)))/sb + 2*
     mu2*power2(sa)*power2(snt)*power2(DeltaInv(mH2,mst12,mst12))*(mst14 + 4*
     mH2*DeltaInv(mH2,mst12,mst12)*power6(mst1)) - (2*mu2*power2(cb)*power2(sa)
     *power2(snt)*power2(DeltaInv(mH2,mst12,mst12))*(mst14 + 4*mH2*DeltaInv(
     mH2,mst12,mst12)*power6(mst1)))/power2(sb) + 2*mu2*power2(DeltaInv(mH2,
     mst12,mst12))*power4(snt)*(mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*power6(
     mst1)) - (4*ca*cb*mu2*sa*power2(DeltaInv(mH2,mst12,mst12))*power4(snt)*(
     mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*power6(mst1)))/sb - 2*mu2*power2(
     sa)*power2(DeltaInv(mH2,mst12,mst12))*power4(snt)*(mst14 + 4*mH2*DeltaInv(
     mH2,mst12,mst12)*power6(mst1)) + (2*mu2*power2(cb)*power2(sa)*power2(
     DeltaInv(mH2,mst12,mst12))*power4(snt)*(mst14 + 4*mH2*DeltaInv(mH2,mst12,
     mst12)*power6(mst1)))/power2(sb)) + Fin3(mw2,msb12,mst12,Q2)*((2*mt2*(
     mst12 - mst22 - 2*mu2)*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (
     4*mt2*mu2*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst12
     - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (4*mt2*mu2*power2(
     cb)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,
     msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mt2*power2(mw2)*
     power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))))/msb12 - (2*mt2*(mst12 - mst22 - 2*mu2)*power2(cb)*power2(mw2)*
     power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))))/(msb12*(mst12 - mst22)) + (4*mt2*mu2*power2(cb)*power2(mw2)*
     power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mt2*power2(mw2)*power2(
     snt)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*
     (mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)))
     )/msb12 + (4*mu2*power2(cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,
     msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 - (2*(mt2 +
     mu2)*power2(cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*
     (mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1))))/msb12 + (2*(mt2 + mu2)*power2(cb)*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(
     mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1))))/msb12 + (2*mu2*power2(cb)*power2(mw2)
     *power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*power2(sb)) + (4*mt2*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12))*power4(cb)*(mst12 - DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))))/(msb12*(mst12 - mst22)*power2(sb)) + (4*mt2*mu2*power2(mw2)*
     power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*power4(cb)*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mu2*
     power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*power4(cb)*(
     mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1))))/(msb12*power2(sb)) + (2*mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)
     *(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))
     ))/msb12 + (4*mt2*mu2*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (4*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*
     (mst12 - mst22)) + (4*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*
     (mst12 - mst22)*power2(sb)) + (2*mu2*power2(cb)*power2(mw2)*power2(snt)*
     power2(DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))
     /(msb12*power2(sb)) + (2*mt2*power2(mw2)*power2(snb)*power2(snt)*power2(
     DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 +
     (4*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,
     msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 + (4*mt2*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*power4(cb)*(-mst12 +
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)*power2(sb)) + (4*mt2*
     mu2*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*power4(cb)*(
     -mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)*power2(sb)) +
     (2*mu2*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,
     mst12))*power4(cb)*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*power2(
     sb))) + (mt2*power2(snb)*power2(log(mw2/Q2))*power3(mw2)*(msb22*mst12*
     power2(DeltaInv(mw2,msb12,mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12)
     )*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)
     ) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(
     mst1)))))/(msb12*msb22) + Fin3(mw2,msb22,mst12,Q2)*((2*mt2*(mst12 - mst22
     - 2*mu2)*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,
     mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (
     4*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,
     mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (
     4*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,
     mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(msb22*(mst12 - mst22)*
     power2(sb)) + (2*mt2*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (4*mu2*
     power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,
     mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 - (2*(mt2 + mu2)*
     power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,
     mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (4*mt2*mu2*power2(
     mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*power4(cb)*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb22*(mst12 - mst22)*power2(sb)) + (2*mu2*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*
     power4(cb)*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2)
     - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(msb22*power2(sb)) + (2*
     mt2*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 +
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/msb22 + (4*mt2*mu2*power2(cb)*power2(mw2)*
     power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,
     msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) +
     power6(mst1))))/(msb22*(mst12 - mst22)*power2(sb)) + (2*mu2*power2(cb)*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(-
     mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1))))/(msb22*power2(sb)) + (4*mt2*mu2*
     power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*power4(cb)*(-
     mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1))))/(msb22*(mst12 - mst22)*power2(sb))) +
     Fin3(msb12,mst12,mC2,Q2)*((4*mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*
     power2(DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(msb12*(mst12 - mst22)) + (2*mu2*power2(mC2)*power2(snb)*power2(
     snt)*power2(DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(msb12,mst12,mC2)*
     (-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/msb12 + (4*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(
     snt)*power2(DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(msb12,mst12,mC2)*
     (-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/msb12 + (4*mt2*mu2*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))*power4(cb)*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14
     - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(
     msb12*(mst12 - mst22)*power2(sb)) + (4*mt2*mu2*power2(mC2)*power2(snb)*
     power2(DeltaInv(msb12,mst12,mC2))*power4(cb)*(mst12 - DeltaInv(msb12,
     mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mu2*power2(
     mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*power4(cb)*
     (mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*power2(sb)) + (2*mt2*(
     mst12 - mst22 - 2*mu2)*power2(cb)*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 -
     mst22)) + (4*mt2*mu2*power2(cb)*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 -
     mst22)) - (2*mt2*(mst12 - mst22 - 2*mu2)*power2(cb)*power2(mC2)*power2(
     snb)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)
     *(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/(msb12*(mst12 - mst22)) + (2*mu2*power2(mC2)*power2(snt)*
     power2(DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb12 + (4*mu2*power2(cb)*power2(mC2)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     msb12 - (2*(mt2 + mu2)*power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(
     msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb12 + (
     2*(mt2 + mu2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     msb12 + (4*mt2*mu2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))*power4(
     cb)*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 -
     mst22)*power2(sb)) + (4*mt2*mu2*power2(mC2)*power2(snb)*power2(DeltaInv(
     msb12,mst12,mC2))*power4(cb)*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mu2*power2(mC2)*power2(
     snt)*power2(DeltaInv(msb12,mst12,mC2))*power4(cb)*(-mst12 + DeltaInv(
     msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(msb12*power2(sb))) + Fin3(msb22,mst12,mC2,
     Q2)*((4*mt2*mu2*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*
     power4(cb)*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22)*power2(sb)) + (2*mt2*(mst12 - mst22 - 2*mu2)*power2(cb)*power2(mC2)
     *power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (4*mt2*mu2*power2(cb)*
     power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (2*mu2*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-
     mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (4*mu2*power2(cb)*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-
     mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 - (2*(mt2 + mu2)*
     power2(cb)*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb22,
     mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (4*mt2*
     mu2*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*power4(cb)*(
     -mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)*
     power2(sb)) + (2*mu2*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(
     msb22,mst12,mC2))*power4(cb)*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(msb22*power2(sb))) - (power2(log(mst12/Q2))*(-2*mh2*msb12*mst12*
     mst14*power2(mt2)*power2(DeltaInv(mh2,mst12,mst12)) + msb12*(2*mH2*mst22 +
     mst12*mst22 + 2*mst24)*mt2*mu2*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + msb12*mst12*mst22*mt2*mu2*power2(mA2)*power2(DeltaInv(mst22,mA2,
     mst12)) + 2*msb12*mst24*mt2*mu2*power2(mA2)*power2(DeltaInv(mst22,mA2,
     mst12)) - 2*msb12*mst12*mst14*mt2*power2(mu2)*power2(DeltaInv(mt2,mu2,
     mst12)) + 2*mst12*mst14*power2(mt2)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 2*msb12*mst22*mt2*mu2*power2(DeltaInv(mst22,mA2,mst12))*power3(
     mA2) - 6*msb12*mst12*mst14*power2(DeltaInv(mt2,mu2,mst12))*power3(mu2) +
     msb12*mt2*mu2*power2(mH2)*((3*mst12*mst22 + 2*mst24)*power2(mH2) - 2*
     mst22*power3(mH2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mH2*(5*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 10*msb12*
     mst12*mst24*mt2*mu2*power3(mA2)*power3(DeltaInv(mst22,mA2,mst12)) + 3*
     msb12*mst12*mt2*mu2*power2(mA2)*power3(mst22)*power3(DeltaInv(mst22,mA2,
     mst12)) + 2*msb12*mt2*mu2*power3(mA2)*power3(mst22)*power3(DeltaInv(mst22,
     mA2,mst12)) + 4*msb12*mst12*mst14*power2(mt2)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) + 3*msb12*mst12*mst22*mt2*mu2*power3(DeltaInv(mst22,mA2,
     mst12))*power4(mA2) + 2*msb12*mst24*mt2*mu2*power3(DeltaInv(mst22,mA2,
     mst12))*power4(mA2) - 2*msb12*power2(mu2)*power2(DeltaInv(mt2,mu2,mst12))*
     power4(mst12) - 20*msb12*mt2*power3(mu2)*power3(DeltaInv(mt2,mu2,mst12))*
     power4(mst12) + 4*msb12*power2(mt2)*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 4*power2(mt2)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 2*msb12*mt2*mu2*power2(mA2)*power3(DeltaInv(
     mst22,mA2,mst12))*power4(mst22) - 10*msb12*mst12*mst14*mt2*power3(
     DeltaInv(mt2,mu2,mst12))*power4(mu2) - 10*msb12*power3(DeltaInv(mt2,mu2,
     mst12))*power4(mst12)*power4(mu2) - 2*mst12*mst14*power2(mt2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 2*msb12*mst22*mt2*mu2*power3(
     DeltaInv(mst22,mA2,mst12))*power5(mA2) - 2*msb12*mt2*power2(mu2)*power3(
     DeltaInv(mt2,mu2,mst12))*power5(mst12) + 2*msb12*power3(mu2)*power3(
     DeltaInv(mt2,mu2,mst12))*power5(mst12) - 2*power2(mt2)*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mst12) + 6*msb12*mst12*mst14*power3(
     DeltaInv(mt2,mu2,mst12))*power5(mu2) + 2*msb12*power2(mu2)*power3(
     DeltaInv(mt2,mu2,mst12))*power6(mst12)))/(2.*msb12*mst12*mt2) + (log(
     mst12/Q2)*(mA2*msb12*mt2*mu2 + mH2*msb12*mt2*mu2 + 2*msb12*mst12*mt2*mu2 +
     2*msb12*mst22*mt2*mu2 - msb12*mst22*mst24*mt2*mu2*DeltaInv(mst22,mA2,
     mst12) + msb12*mst12*mt2*mu2*DeltaInv(mst22,mA2,mst12)*power2(mA2) - 5*
     msb12*mst22*mt2*mu2*DeltaInv(mst22,mA2,mst12)*power2(mA2) + 2*msb12*mt2*
     power2(mst12) - 4*msb12*mst22*mt2*mu2*DeltaInv(mt2,mu2,mst12)*power2(
     mst12) - 5*mA2*msb12*mt2*mu2*DeltaInv(mst22,mA2,mst12)*power2(mst22) +
     msb12*mst12*mt2*mu2*DeltaInv(mst22,mA2,mst12)*power2(mst22) - 4*msb12*(mh2
     - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12)*power2(mt2) + 2*mst12*
     mst14*mst22*DeltaInv(mw2,msb12,mst12)*power2(mt2) - 2*msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mt2) + 10*mst22*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mt2) - 48*msb12*mst22*
     DeltaInv(mt2,mu2,mst12)*power2(mst12)*power2(mu2) - 6*mA2*msb12*mst12*
     mst22*mst24*mt2*mu2*power2(DeltaInv(mst22,mA2,mst12)) + 138*msb12*mst22*
     mst24*mt2*mu2*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 36*msb12*
     mst12*mt2*mu2*power2(mA2)*power2(mst22)*power2(DeltaInv(mst22,mA2,mst12))
     - 240*msb12*mst12*mst14*mst22*mt2*power2(mu2)*power2(DeltaInv(mt2,mu2,
     mst12)) + 12*msb12*mst12*mst14*mst22*mw2*power2(mt2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 168*mst12*mst14*mst22*power2(mt2)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*power2(mst12)*power2(mt2)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*mt2*mu2*DeltaInv(
     mst22,mA2,mst12)*power3(mA2) - 6*msb12*mst12*mst22*mt2*mu2*power2(
     DeltaInv(mst22,mA2,mst12))*power3(mA2) + 138*msb12*mt2*mu2*power2(mst22)*
     power2(DeltaInv(mst22,mA2,mst12))*power3(mA2) - msb12*mt2*mu2*DeltaInv(
     mH2,mst22,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*
     power2(mH2) + 5*mH2*power2(mst22) + power3(mH2)) + 96*msb12*mst12*mst22*
     power2(mt2)*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)
     ) + 6*mH2*msb12*mst22*mt2*mu2*power2(DeltaInv(mH2,mst22,mst12))*(-(mst12*
     mst24) + mH2*(6*mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mH2) +
     power3(mH2) + power3(mst22)) - 528*msb12*mst12*mst14*mst22*power2(
     DeltaInv(mt2,mu2,mst12))*power3(mu2) - 96*msb12*mst22*mt2*power2(mst12)*
     power2(DeltaInv(mt2,mu2,mst12))*power3(mu2) - 12*mst22*power2(mst12)*
     power2(mt2)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*msb12*
     mst22*mt2*mu2*power2(mH2)*(-((3*mst12*mst22 + 2*mst24)*power2(mH2)) + 2*
     mst22*power3(mH2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mH2*(5*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 720*msb12*
     mst12*mst22*mst24*mt2*mu2*power3(mA2)*power3(DeltaInv(mst22,mA2,mst12)) +
     288*msb12*mst12*mst14*mst22*power2(mt2)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 6*msb12*mst22*mt2*mu2*power2(DeltaInv(mst22,mA2,mst12))*
     power4(mA2) + 144*msb12*mst22*mst24*mt2*mu2*power3(DeltaInv(mst22,mA2,
     mst12))*power4(mA2) + 216*msb12*mst12*mt2*mu2*power2(mst22)*power3(
     DeltaInv(mst22,mA2,mst12))*power4(mA2) - 96*msb12*mst22*power2(mu2)*
     power2(DeltaInv(mt2,mu2,mst12))*power4(mst12) - 12*mst22*mw2*power2(mt2)*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 1440*msb12*mst22*mt2*
     power3(mu2)*power3(DeltaInv(mt2,mu2,mst12))*power4(mst12) + 288*msb12*
     mst22*power2(mt2)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12) + 288*mst22*power2(mt2)*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 6*mA2*msb12*mt2*mu2*power2(DeltaInv(mst22,mA2,
     mst12))*power4(mst22) + 216*msb12*mst12*mt2*mu2*power2(mA2)*power3(
     DeltaInv(mst22,mA2,mst12))*power4(mst22) + 144*msb12*mt2*mu2*power3(mA2)*
     power3(DeltaInv(mst22,mA2,mst12))*power4(mst22) + 48*msb12*mst22*power2(
     mst12)*power2(DeltaInv(mt2,mu2,mst12))*power4(mu2) - 720*msb12*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mt2,mu2,mst12))*power4(mu2) - 720*msb12*
     mst22*power3(DeltaInv(mt2,mu2,mst12))*power4(mst12)*power4(mu2) - 144*
     mst12*mst14*mst22*power2(mt2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*msb12*mt2*mu2*power2(mst22)*power3(DeltaInv(mst22,mA2,mst12))*
     power5(mA2) + 1152*mh2*msb12*mst22*power2(mt2)*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) - 144*msb12*mst22*mt2*power2(mu2)*power3(DeltaInv(
     mt2,mu2,mst12))*power5(mst12) + 144*msb12*mst22*power3(mu2)*power3(
     DeltaInv(mt2,mu2,mst12))*power5(mst12) - 144*mst22*power2(mt2)*power2(mw2)
     *power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 144*msb12*mt2*mu2*
     power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*power5(mst22) + 432*msb12*
     mst12*mst14*mst22*power3(DeltaInv(mt2,mu2,mst12))*power5(mu2) + 144*msb12*
     mst22*power2(mu2)*power3(DeltaInv(mt2,mu2,mst12))*power6(mst12)))/(24.*
     msb12*mst12*mst22*mt2) + (power2(cb)*power2(log(mst12/Q2))*(2*mst12*mst14*
     mt2*(-mst12 + mst22 + 2*mu2)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))
     + msb12*(mst12 - mst22)*(2*mA2*mst22 + mst12*mst22 + 2*mst24)*mu2*power2(
     mA2)*power2(DeltaInv(mst22,mA2,mst12)) - 2*msb12*mst12*mst24*mu2*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 2*msb12*mst22*mst24*mu2*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst22*mu2*power2(mst12)*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*mst12*mu2*power2(
     mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 2*mst12*mst14*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 4*mst12*mst14*
     mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 2*mst14*mt2*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 2*msb12*
     mst12*mst22*mu2*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 2*msb12*
     mu2*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 2*mst12*
     mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + msb12*(mst12 - mst22)*mu2*power2(mA2)*((3*
     mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*
     mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mA2,mst12)) - 3*msb12*mu2*power2(mst12)*power2(mz2)*power3(
     mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 10*msb12*mst12*mst22*mst24*mu2*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 10*msb12*mst24*mu2*power2(
     mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 2*msb12*mst12*mu2*
     power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 4*msb12*
     mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 8*
     msb12*mst12*mst14*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     4*msb12*mst14*mt2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 4*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 8*msb12*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 4*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 8*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 5*msb12*mst12*mu2*power2(mz2)*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) + 2*msb12*mu2*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) + 2*mst12*mst14*mst22*mt2*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 4*mst12*mst14*mt2*mu2*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) - 2*mst14*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) - 2*msb12*mst12*mst24*mu2*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mz2) + 2*msb12*mst22*mst24*mu2*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mz2) - 3*msb12*mst22*mu2*power2(mst12)*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mz2) + 3*msb12*mst12*mu2*power2(mst22)*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) + 4*msb12*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mst12) + 2*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mst12) + 4*mt2*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mst12) + 4*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mst12) - 2*msb12*mu2*power2(mz2)*power3(
     DeltaInv(mst22,mz2,mst12))*power5(mst22) + 2*msb12*mst12*mst22*mu2*power3(
     DeltaInv(mst22,mz2,mst12))*power5(mz2) - 2*msb12*mu2*power2(mst22)*power3(
     DeltaInv(mst22,mz2,mst12))*power5(mz2) - 2*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power6(mst12)))/(2.*msb12*mst12*(mst12 - mst22)
     ) + (mu2*power2(log(mw2/Q2))*power3(mw2)*power4(cb)*(mst22*(mst12 - mst22
     - 2*mt2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,
     msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + (mst12 - mst22 - 2*
     mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*mst24 - mst22*mw2) + 2*
     msb12*(mst24 + mst22*mw2) - power6(mst2))))/(msb12*(mst12 - mst22)*power2(
     sb)) + (2*mu2*power2(cb)*power2(log(mw2/Q2))*power3(mw2)*(-(mst22*(-mst12
     + mst22 + mt2)*power2(DeltaInv(mst22,mw2,msb12))) + mt2*power2(DeltaInv(
     mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + (mst12 - mst22
     - mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*mst24 - mst22*mw2) + 2*
     msb12*(mst24 + mst22*mw2) - power6(mst2))))/(msb12*(mst12 - mst22)) + (
     power2(cb)*power2(log(mw2/Q2))*power3(mw2)*(mst22*(mst12 - mst22 - 2*mt2)*
     mu2*power2(DeltaInv(mst22,mw2,msb12)) - mt2*(mst12 - mst22 - 2*mu2)*
     power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*
     (-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + (
     mst12 - mst22 - 2*mt2)*mu2*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*mst24
     - mst22*mw2) + 2*msb12*(mst24 + mst22*mw2) - power6(mst2))))/(msb12*(-
     mst12 + mst22)) + (mu2*power2(snb)*power2(snt)*power2(log(mC2/Q2))*power3(
     mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*power2(
     DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))
     + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))
     + msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) - msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22) + (2*mu2*power2(cb)*power2(snb)*power2(snt)*power2(log(mC2/
     Q2))*power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*
     mst12*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2)) + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) + msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*
     msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb22,mC2)) - msb12*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) - msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(msb12*msb22) + (mu2*power2(snb)*power2(snt)*
     power2(log(mC2/Q2))*power3(mC2)*power4(cb)*(msb22*mst12*power2(DeltaInv(
     msb12,mst12,mC2)) - msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) - msb22*
     mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,
     mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - 2*msb12*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) -
     msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - msb22*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb12*power3(DeltaInv(msb22,mst12,mC2)
     )*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) -
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22*
     power2(sb)) + ((mt2 + mu2)*power2(cb)*power2(snb)*power2(snt)*power2(log(
     mC2/Q2))*power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) + msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))
     + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb22*
     mst24*power3(DeltaInv(mst22,msb12,mC2)) + 2*msb12*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2)) - msb22*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,
     mC2)) - 2*mC2*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) - 2*msb12*
     msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) + msb12*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + msb22*power3(DeltaInv(msb12,mst12,mC2)
     )*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) - msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 -
     2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - msb22*
     power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) + msb12*power3(DeltaInv(
     mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22) + (2*mt2*mu2*power2(snb)*
     power2(log(mC2/Q2))*power3(mC2)*power4(cb)*(-(msb22*mst12*power2(DeltaInv(
     msb12,mst12,mC2))) + msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) +
     msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - msb22*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2))
     + msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + msb22*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - msb12*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) + msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*
     msb22*(mst12 - mst22)*power2(sb)) - (mC2*mu2*log(mC2/Q2)*power2(snb)*
     power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*
     msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*
     msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*
     DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,
     mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2) - msb22*mst12*
     DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*DeltaInv(mst22,msb22,
     mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*
     mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*
     mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*
     mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*
     mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*
     mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14
     - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(12.*msb12*msb22*mst12*mst22) - (mC2*mu2*log(mC2/Q2)*
     power2(cb)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*
     mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(
     msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(
     msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*
     mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*
     DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*
     mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*
     mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) -
     6*mC2*msb12*mst12*mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*
     msb12*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) +
     6*msb22*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*
     msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 144*msb22*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*
     power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14
     - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(6.*msb12*msb22*mst12*mst22) + (mC2*(mt2 + mu2)*log(mC2/
     Q2)*power2(cb)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 +
     msb12*mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*
     DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*
     DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,
     mC2) - 5*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*
     msb22*mst12*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(
     mst22,msb22,mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) +
     msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*
     mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*
     mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) - 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 18*mC2*msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) - 84*msb22*mst12*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*mst22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*
     mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*
     msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     12.*msb12*msb22*mst12*mst22) + (mC2*mt2*mu2*log(mC2/Q2)*power2(snb)*
     power4(cb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*
     msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*
     msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*
     DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,
     mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2) - msb22*mst12*
     DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*DeltaInv(mst22,msb22,
     mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*
     mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*
     mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*
     mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*
     mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*
     mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14
     - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(6.*msb12*msb22*mst12*(mst12 - mst22)*mst22*power2(sb)) -
     (mC2*mu2*log(mC2/Q2)*power2(snb)*power2(snt)*power4(cb)*(-(msb12*mst12) +
     msb22*mst12 + msb12*mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*
     mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*
     mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(
     mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) -
     mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*
     DeltaInv(mst22,msb22,mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(
     mC2) + msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*
     mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2)
     )*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*
     mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) - 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 18*mC2*msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) - 84*msb22*mst12*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*mst22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*
     mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*
     msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     12.*msb12*msb22*mst12*mst22*power2(sb)) + (mC2*mu2*log(mC2/Q2)*power2(snb)
     *power4(cb)*(-(msb12*mst12*mst22) + msb22*mst12*mst22 - 2*msb12*mst12*mt2
     + 2*msb22*mst12*mt2 + 2*msb12*mst22*mt2 - 2*msb22*mst22*mt2 + 2*mC2*msb22*
     (mC2 - msb12 + 5*mst12)*mst22*mt2*DeltaInv(msb12,mst12,mC2) - 2*mC2*msb12*
     (mC2 - msb22 + 5*mst12)*mst22*mt2*DeltaInv(msb22,mst12,mC2) + mC2*msb12*
     msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*msb22*mst12*mt2*
     DeltaInv(mst22,msb12,mC2) - 10*mC2*msb22*mst12*mst22*mt2*DeltaInv(mst22,
     msb12,mC2) - mC2*msb12*msb22*mst12*mst22*DeltaInv(mst22,msb22,mC2) - 2*
     mC2*msb12*msb22*mst12*mt2*DeltaInv(mst22,msb22,mC2) + 10*mC2*msb12*mst12*
     mst22*mt2*DeltaInv(mst22,msb22,mC2) - msb22*mst12*mst22*DeltaInv(mst22,
     msb12,mC2)*power2(mC2) - 2*msb22*mst12*mt2*DeltaInv(mst22,msb12,mC2)*
     power2(mC2) + msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2)*power2(mC2) + 2*
     msb12*mst12*mt2*DeltaInv(mst22,msb22,mC2)*power2(mC2) + msb12*power2(
     mst12) - msb22*power2(mst12) - mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2)*
     power2(mst12) + 5*mC2*msb22*mst22*DeltaInv(mst22,msb12,mC2)*power2(mst12)
     + mC2*msb12*msb22*DeltaInv(mst22,msb22,mC2)*power2(mst12) - 5*mC2*msb12*
     mst22*DeltaInv(mst22,msb22,mC2)*power2(mst12) + msb22*DeltaInv(mst22,
     msb12,mC2)*power2(mC2)*power2(mst12) - msb12*DeltaInv(mst22,msb22,mC2)*
     power2(mC2)*power2(mst12) - 5*mC2*msb22*mst12*DeltaInv(mst22,msb12,mC2)*
     power2(mst22) + 5*mC2*msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mst22)
     - 12*mC2*msb22*mst12*mst22*mt2*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) +
     mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 12*mC2*msb12*
     mst12*mst22*mt2*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 12*mC2*msb22*mst12*mst22*mst24*
     mt2*power2(DeltaInv(mst22,msb12,mC2)) - 12*msb12*msb22*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb22*mst22*mst24*
     power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 6*msb12*msb22*mst22*
     power2(mC2)*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*
     mst12*mst24*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 36*mC2*
     msb12*msb22*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*
     msb12*msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,
     mC2)) - 168*msb22*mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) + 18*mC2*msb12*msb22*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) + 84*msb22*power2(mC2)*power2(mst12)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 12*mC2*msb12*mst12*mst22*mst24*
     mt2*power2(DeltaInv(mst22,msb22,mC2)) + 12*msb12*msb22*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 6*mC2*msb12*mst22*mst24*
     power2(mst12)*power2(DeltaInv(mst22,msb22,mC2)) - 6*msb12*msb22*mst22*
     power2(mC2)*power2(mst12)*power2(DeltaInv(mst22,msb22,mC2)) - 6*mC2*msb12*
     mst12*mst24*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 36*mC2*
     msb12*msb22*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*
     msb12*msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,
     mC2)) + 168*msb12*mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(
     mst22,msb22,mC2)) - 18*mC2*msb12*msb22*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) - 84*msb12*power2(mC2)*power2(mst12)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 12*msb22*mst12*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb22*mst22*power2(
     mst12)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 6*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 12*msb12*
     mst12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 6*msb12*
     mst22*power2(mst12)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 6*
     msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) -
     18*mC2*msb12*msb22*mst12*power2(DeltaInv(mst22,msb12,mC2))*power3(mst22) -
     84*msb22*mst12*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*power3(mst22)
     + 18*mC2*msb12*msb22*mst12*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22)
     + 84*msb12*mst12*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mst22) - 288*msb12*msb22*mst12*mst22*mst24*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*mst22*mst24*power2(mC2)*
     power2(mst12)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*msb22*mst12*
     mst24*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 288*
     msb22*mst12*mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     + 144*msb22*mst22*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 144*msb22*mst12*mst24*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 288*msb12*msb22*mst12*mt2*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*power2(
     mst12)*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) + 288*msb12*msb22*mst12*mst22*mst24*mt2*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - 144*msb12*msb22*mst22*mst24*power2(mC2)*power2(mst12)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*mst24*power2(
     mC2)*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 288*msb12*mst12*
     mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*
     mst22*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     144*msb12*mst12*mst24*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 288*msb12*msb22*mst12*mt2*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*power2(mst12)*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) -
     72*msb22*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) + 72*msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 144*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 72*msb12*power2(mst12)*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb12*mst12*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 144*msb22*mst12*
     mst22*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 -
     2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 144*
     msb12*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + 144*msb22*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power6(mst2) - 72*msb22*mst22*power2(mC2)*power2(mst12)*
     power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) + 72*msb22*mst12*power2(
     mC2)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 144*
     msb12*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*
     power6(mst2) + 72*msb12*mst22*power2(mC2)*power2(mst12)*power3(DeltaInv(
     mst22,msb22,mC2))*power6(mst2) - 72*msb12*mst12*power2(mC2)*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(12.*msb12*msb22*mst12*
     mst22*(-mst12 + mst22)*power2(sb)) + (mC2*mu2*log(mC2/Q2)*power2(cb)*
     power2(snb)*(-(msb12*mst12*mst22) + msb22*mst12*mst22 - msb12*mst12*mt2 +
     msb22*mst12*mt2 + msb12*mst22*mt2 - msb22*mst22*mt2 + mC2*msb22*(mC2 -
     msb12 + 5*mst12)*mst22*mt2*DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 +
     msb22 - 5*mst12)*mst22*mt2*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*
     mst12*mst22*DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*mst12*mt2*
     DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*mt2*DeltaInv(mst22,
     msb12,mC2) - mC2*msb12*msb22*mst12*mst22*DeltaInv(mst22,msb22,mC2) - mC2*
     msb12*msb22*mst12*mt2*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*
     mt2*DeltaInv(mst22,msb22,mC2) - msb22*mst12*mst22*DeltaInv(mst22,msb12,
     mC2)*power2(mC2) - msb22*mst12*mt2*DeltaInv(mst22,msb12,mC2)*power2(mC2) +
     msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2)*power2(mC2) + msb12*mst12*mt2*
     DeltaInv(mst22,msb22,mC2)*power2(mC2) + msb12*power2(mst12) - msb22*
     power2(mst12) - mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2)*power2(mst12) +
     5*mC2*msb22*mst22*DeltaInv(mst22,msb12,mC2)*power2(mst12) + mC2*msb12*
     msb22*DeltaInv(mst22,msb22,mC2)*power2(mst12) - 5*mC2*msb12*mst22*
     DeltaInv(mst22,msb22,mC2)*power2(mst12) + msb22*DeltaInv(mst22,msb12,mC2)*
     power2(mC2)*power2(mst12) - msb12*DeltaInv(mst22,msb22,mC2)*power2(mC2)*
     power2(mst12) - 5*mC2*msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mst22)
     + 5*mC2*msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mst22) - 6*mC2*msb22*
     mst12*mst22*mt2*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*mt2*(-3*
     msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*mt2*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb22*mst22*mst24*power2(mst12)*
     power2(DeltaInv(mst22,msb12,mC2)) + 6*msb12*msb22*mst22*power2(mC2)*
     power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*mst12*mst24*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*
     mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*
     mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb22*mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)
     ) + 18*mC2*msb12*msb22*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,
     msb12,mC2)) + 84*msb22*power2(mC2)*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*mt2*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 6*mC2*msb12*mst22*mst24*power2(mst12)*
     power2(DeltaInv(mst22,msb22,mC2)) - 6*msb12*msb22*mst22*power2(mC2)*
     power2(mst12)*power2(DeltaInv(mst22,msb22,mC2)) - 6*mC2*msb12*mst12*mst24*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*
     mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*
     mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*
     msb12*mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)
     ) - 18*mC2*msb12*msb22*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,
     msb22,mC2)) - 84*msb12*power2(mC2)*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*mst22*mt2*power2(DeltaInv(
     mst22,msb12,mC2))*power3(mC2) - 6*msb22*mst22*power2(mst12)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) + 6*msb22*mst12*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*mt2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 6*msb12*mst22*power2(
     mst12)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 6*msb12*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 18*mC2*
     msb12*msb22*mst12*power2(DeltaInv(mst22,msb12,mC2))*power3(mst22) - 84*
     msb22*mst12*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*power3(mst22) +
     18*mC2*msb12*msb22*mst12*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22) +
     84*msb12*mst12*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22)
     - 144*msb12*msb22*mst12*mst22*mst24*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) + 144*msb12*msb22*mst22*mst24*power2(mC2)*power2(mst12)*
     power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*msb22*mst12*mst24*power2(
     mC2)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*
     mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb22*
     mst22*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     144*msb22*mst12*mst24*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 144*msb12*msb22*mst12*mt2*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*power2(mst12)*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*msb22*mst12*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb12*
     msb22*mst12*mst22*mst24*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     - 144*msb12*msb22*mst22*mst24*power2(mC2)*power2(mst12)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*msb22*mst12*mst24*power2(mC2)*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*mt2*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*mst22*mst24*
     power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*
     mst12*mst24*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     144*msb12*msb22*mst12*mt2*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) - 144*msb12*msb22*power2(mst12)*power2(mst22)*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*mt2*
     power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb22*
     power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     + 72*msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) - 72*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 72*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) - 72*msb12*mst12*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*mt2*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*mt2*
     power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*
     mst22*mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*
     msb22*mst22*power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*
     power6(mst2) + 72*msb22*mst12*power2(mC2)*power2(mst22)*power3(DeltaInv(
     mst22,msb12,mC2))*power6(mst2) - 72*msb12*mst12*mst22*mt2*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power6(mst2) + 72*msb12*mst22*power2(
     mC2)*power2(mst12)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2) - 72*
     msb12*mst12*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power6(mst2)))/(6.*msb12*msb22*mst12*mst22*(-mst12 + mst22)) - (mC2*log(
     mC2/Q2)*power2(cb)*power2(snb)*(msb12*mst12*mst22*mt2 - msb22*mst12*mst22*
     mt2 + msb12*mst12*mst22*mu2 - msb22*mst12*mst22*mu2 + 2*msb12*mst12*mt2*
     mu2 - 2*msb22*mst12*mt2*mu2 - 2*msb12*mst22*mt2*mu2 + 2*msb22*mst22*mt2*
     mu2 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*mt2*(mst12 - mst22 - 2*mu2)*
     DeltaInv(msb12,mst12,mC2) - mC2*msb12*(mC2 - msb22 + 5*mst12)*mst22*mt2*(
     mst12 - mst22 - 2*mu2)*DeltaInv(msb22,mst12,mC2) - mC2*msb12*msb22*mst12*
     mst22*mu2*DeltaInv(mst22,msb12,mC2) - 2*mC2*msb12*msb22*mst12*mt2*mu2*
     DeltaInv(mst22,msb12,mC2) + 10*mC2*msb22*mst12*mst22*mt2*mu2*DeltaInv(
     mst22,msb12,mC2) + mC2*msb12*msb22*mst12*mst22*mu2*DeltaInv(mst22,msb22,
     mC2) + 2*mC2*msb12*msb22*mst12*mt2*mu2*DeltaInv(mst22,msb22,mC2) - 10*mC2*
     msb12*mst12*mst22*mt2*mu2*DeltaInv(mst22,msb22,mC2) + msb22*mst12*mst22*
     mu2*DeltaInv(mst22,msb12,mC2)*power2(mC2) + 2*msb22*mst12*mt2*mu2*
     DeltaInv(mst22,msb12,mC2)*power2(mC2) - msb12*mst12*mst22*mu2*DeltaInv(
     mst22,msb22,mC2)*power2(mC2) - 2*msb12*mst12*mt2*mu2*DeltaInv(mst22,msb22,
     mC2)*power2(mC2) - msb12*mst22*power2(mst12) + msb22*mst22*power2(mst12) -
     msb12*mu2*power2(mst12) + msb22*mu2*power2(mst12) + mC2*msb12*msb22*mu2*
     DeltaInv(mst22,msb12,mC2)*power2(mst12) - 5*mC2*msb22*mst22*mu2*DeltaInv(
     mst22,msb12,mC2)*power2(mst12) - mC2*msb12*msb22*mu2*DeltaInv(mst22,msb22,
     mC2)*power2(mst12) + 5*mC2*msb12*mst22*mu2*DeltaInv(mst22,msb22,mC2)*
     power2(mst12) - msb22*mu2*DeltaInv(mst22,msb12,mC2)*power2(mC2)*power2(
     mst12) + msb12*mu2*DeltaInv(mst22,msb22,mC2)*power2(mC2)*power2(mst12) +
     msb12*mst12*power2(mst22) - msb22*mst12*power2(mst22) - msb12*mt2*power2(
     mst22) + msb22*mt2*power2(mst22) + 5*mC2*msb22*mst12*mu2*DeltaInv(mst22,
     msb12,mC2)*power2(mst22) - 5*mC2*msb12*mst12*mu2*DeltaInv(mst22,msb22,mC2)
     *power2(mst22) - 6*mC2*msb22*mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*(-3*
     msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*mt2*(mst12 - mst22 -
     2*mu2)*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb22,mst12,mC2)) - 12*mC2*msb22*mst12*mst22*mst24*mt2*
     mu2*power2(DeltaInv(mst22,msb12,mC2)) + 12*msb12*msb22*mst12*mst22*mt2*
     mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*mst22*
     mst24*mu2*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*
     mst22*mu2*power2(mC2)*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) - 6*
     mC2*msb22*mst12*mst24*mu2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))
     + 36*mC2*msb12*msb22*mst12*mt2*mu2*power2(mst22)*power2(DeltaInv(mst22,
     msb12,mC2)) + 6*msb12*msb22*mst12*mu2*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) + 168*msb22*mst12*mt2*mu2*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mu2*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mu2*
     power2(mC2)*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))
     + 12*mC2*msb12*mst12*mst22*mst24*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))
     - 12*msb12*msb22*mst12*mst22*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) - 6*mC2*msb12*mst22*mst24*mu2*power2(mst12)*power2(DeltaInv(
     mst22,msb22,mC2)) + 6*msb12*msb22*mst22*mu2*power2(mC2)*power2(mst12)*
     power2(DeltaInv(mst22,msb22,mC2)) + 6*mC2*msb12*mst12*mst24*mu2*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 36*mC2*msb12*msb22*mst12*mt2*
     mu2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 6*msb12*msb22*mst12*
     mu2*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 168*
     msb12*mst12*mt2*mu2*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,
     mC2)) + 18*mC2*msb12*msb22*mu2*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) + 84*msb12*mu2*power2(mC2)*power2(mst12)*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 12*msb22*mst12*mst22*
     mt2*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 6*msb22*mst22*mu2*
     power2(mst12)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb22*
     mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) +
     12*msb12*mst12*mst22*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 6*msb12*mst22*mu2*power2(mst12)*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 6*msb12*mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,msb22,
     mC2))*power3(mC2) + 18*mC2*msb12*msb22*mst12*mu2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mst22) + 84*msb22*mst12*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mst22) - 18*mC2*msb12*msb22*mst12*mu2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mst22) - 84*msb12*mst12*mu2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22) + 288*msb12*
     msb22*mst12*mst22*mst24*mt2*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 144*msb12*msb22*mst22*mst24*mu2*power2(mC2)*power2(mst12)*power3(
     DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*mst12*mst24*mu2*power2(mC2)*
     power2(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 288*msb22*mst12*mst22*
     mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst22*mst24*mu2*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     ) + 144*msb22*mst12*mst24*mu2*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 288*msb12*msb22*mst12*mt2*mu2*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*msb22*mu2*power2(mst12)
     *power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb12*
     msb22*mst12*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 288*msb12*msb22*mst12*mst22*mst24*mt2*mu2*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*msb22*mst22*mst24*mu2*power2(mC2)*power2(
     mst12)*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*mst12*mst24*
     mu2*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 288*
     msb12*mst12*mst22*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst22*mst24*mu2*power2(mst12)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 144*msb12*mst12*mst24*mu2*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 288*msb12*msb22*mst12*mt2*
     mu2*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 144*
     msb12*msb22*mu2*power2(mst12)*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - 144*msb12*msb22*mst12*mu2*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) - 144*msb22*mst12*mt2*mu2*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 72*msb22*mu2*power2(mst12)
     *power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb22*
     mst12*mu2*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) +
     144*msb12*mst12*mt2*mu2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 72*msb12*mu2*power2(mst12)*power2(mst22)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 72*msb12*mst12*mu2*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*mt2*(mst12 -
     mst22 - 2*mu2)*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*msb12*mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - 144*msb22*mst12*mst22*mt2*mu2*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) + 72*msb22*mst22*mu2*
     power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) -
     72*msb22*mst12*mu2*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb12,
     mC2))*power6(mst2) + 144*msb12*mst12*mst22*mt2*mu2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power6(mst2) - 72*msb12*mst22*mu2*power2(mC2)*
     power2(mst12)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2) + 72*msb12*
     mst12*mu2*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power6(mst2)))/(12.*msb12*msb22*mst12*(mst12 - mst22)*mst22) + ((mt2 +
     mu2)*power2(cb)*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(mst22*power2(
     DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))) - power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 +
     (mu2*power2(cb)*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(mst22*power2(
     DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))) - power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     power2(sb)) + (2*mt2*mu2*power2(cb)*power2(log(mw2/Q2))*power3(mw2)*(-(
     mst22*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,
     mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(msb12*(mst12 - mst22)*power2(sb)) + (mt2*power2(snt)*
     power2(log(mw2/Q2))*power3(mw2)*(-(mst22*power2(DeltaInv(mst22,mw2,msb12))
     ) + power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)))
     + power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*
     (mst24 + mst22*mw2) + power6(mst2))))/msb12 + (2*mu2*power2(cb)*power2(
     snt)*power2(log(mw2/Q2))*power3(mw2)*(-(mst22*power2(DeltaInv(mst22,mw2,
     msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))) + power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 - (2*mt2*mu2*power2(
     log(mw2/Q2))*power3(mw2)*power4(cb)*(-(mst22*power2(DeltaInv(mst22,mw2,
     msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))) + power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (mu2*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*power4(cb)*
     (-(mst22*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,
     mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(msb12*power2(sb)) + (mu2*power2(cb)*power2(log(mw2/Q2))*
     power3(mw2)*(mst22*(-mst12 + mst22 + 2*mt2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,
     msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))
     *(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))
     ))/(msb12*(mst12 - mst22)*power2(sb)) + (mu2*power2(snb)*power2(log(mw2/
     Q2))*power3(mw2)*power4(cb)*(msb22*mst22*(-mst12 + mst22 + 2*mt2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*mst22*(mst12 - mst22 - 2*mt2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 2*mt2*(-(msb22*mst12*power2(DeltaInv(mw2,
     msb12,mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) + msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))) +
     msb12*(mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(2*
     mst24 - mst22*mw2) + 2*msb22*(mst24 + mst22*mw2) - power6(mst2)) + msb22*(
     mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22*(
     mst12 - mst22)*power2(sb)) + (2*mu2*power2(cb)*power2(snb)*power2(log(mw2/
     Q2))*power3(mw2)*(msb22*mst22*(-mst12 + mst22 + mt2)*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*mst22*(-mst12 + mst22 + mt2)*power2(DeltaInv(
     mst22,mw2,msb22)) + mt2*(-(msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)))
     + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1)) + msb12*power2(DeltaInv(mw2,
     msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))) + msb12*(mst12 -
     mst22 - mt2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(2*mst24 - mst22*mw2)
     + 2*msb22*(mst24 + mst22*mw2) - power6(mst2)) + msb22*(mst12 - mst22 -
     mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*
     msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22*(mst12 - mst22))
     + (power2(cb)*power2(snb)*power2(log(mw2/Q2))*power3(mw2)*(msb22*mst22*(-
     mst12 + mst22 + 2*mt2)*mu2*power2(DeltaInv(mst22,mw2,msb12)) + msb12*
     mst22*(mst12 - mst22 - 2*mt2)*mu2*power2(DeltaInv(mst22,mw2,msb22)) + mt2*
     (mst12 - mst22 - 2*mu2)*(msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) -
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1)) - msb12*power2(DeltaInv(mw2,
     msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))) + msb12*(mst12 -
     mst22 - 2*mt2)*mu2*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(2*mst24 -
     mst22*mw2) + 2*msb22*(mst24 + mst22*mw2) - power6(mst2)) + msb22*(mst12 -
     mst22 - 2*mt2)*mu2*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22*(-
     mst12 + mst22)) + (mt2*mu2*mw2*log(mw2/Q2)*power2(cb)*(-mst12 + mst22 +
     mst12*mw2*(-msb12 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb12) + mst22*(
     msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,mst12) + 6*mst12*mst22*mw2*(
     -mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(
     mst22,mw2,msb12)) - 6*mst12*mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 +
     msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) + 72*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2)
     - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*power2(mw2)
     *power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(
     mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*mst12*(mst12 - mst22)*
     mst22*power2(sb)) + (mu2*mw2*log(mw2/Q2)*power2(cb)*power2(snt)*(-mst12 +
     mst22 + mst12*mw2*(-msb12 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb12) +
     mst22*(msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,mst12) + 6*mst12*
     mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) - 6*mst12*mst22*mw2*(-mst14 + (14*mst12 - mw2)*
     mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) + 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2)
     - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*mst12*mst22) + (
     mu2*mw2*log(mw2/Q2)*power2(snt)*power4(cb)*(-mst12 + mst22 + mst12*mw2*(-
     msb12 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb12) + mst22*(msb12 - 5*mst12
     - mw2)*mw2*DeltaInv(mw2,msb12,mst12) + 6*mst12*mst22*mw2*(-mst24 + (14*
     mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)
     ) - 6*mst12*mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 +
     mw2))*power2(DeltaInv(mw2,msb12,mst12)) + 72*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(12.*msb12*mst12*mst22*power2(sb)) + (mu2*
     mw2*log(mw2/Q2)*power2(cb)*power2(snt)*(mst12 - mst22 + mst12*(msb12 - 5*
     mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + mst22*mw2*(-msb12 + 5*mst12 +
     mw2)*DeltaInv(mw2,msb12,mst12) - 6*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*
     mst12*mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*
     power2(DeltaInv(mw2,msb12,mst12)) - 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2))))/(12.*msb12*mst12*mst22*power2(sb)) + (mt2*mu2*mw2*
     log(mw2/Q2)*power4(cb)*(mst12 - mst22 + mst12*(msb12 - 5*mst22 - mw2)*mw2*
     DeltaInv(mst22,mw2,msb12) + mst22*mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(
     mw2,msb12,mst12) - 6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 +
     msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*mst12*mst22*
     mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb12,mst12)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*
     mw2) + power6(mst1)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*
     mw2*log(mw2/Q2)*power2(cb)*(-((mst12 - mst22)*(mst12 - 2*mt2)) + mst12*(
     mst12 - mst22 - 2*mt2)*mw2*(-msb12 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,
     msb12) + 2*mst22*mt2*mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(mw2,msb12,
     mst12) + 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 12*
     mst12*mst22*mt2*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2)
     )*power2(DeltaInv(mw2,msb12,mst12)) - 144*mst12*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*(mst12 - mst22 - 2*
     mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*
     mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*mst12*(
     mst12 - mst22)*mst22*power2(sb)) + (mu2*mw2*log(mw2/Q2)*power4(cb)*((mst12
     - mst22)*(mst12 - 2*mt2) + mst12*(mst12 - mst22 - 2*mt2)*(msb12 - 5*mst22
     - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + 2*mst22*mt2*(msb12 - 5*mst12 - mw2)
     *mw2*DeltaInv(mw2,msb12,mst12) - 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*
     mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) - 12*mst12*mst22*mt2*mw2*(-mst14 + (14*mst12 -
     mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) + 144*
     mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) + 72*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(12.*msb12*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*
     mw2*log(mw2/Q2)*power2(cb)*((mst12 - mst22)*(mst12 - mt2) + mst12*(mst12 -
     mst22 - mt2)*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) +
     mst22*mt2*(msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,mst12) - 6*mst12*
     mst22*(mst12 - mst22 - mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*
     mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 6*mst12*mst22*mt2*mw2*(-
     mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,
     msb12,mst12)) + 72*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1)) + 72*mst12*mst22*(mst12 - mst22 - mt2)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22) - (
     mw2*log(mw2/Q2)*power2(snt)*((mst12 - mst22)*(msb12 + mt2) + mst12*mt2*(
     msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + mst22*mt2*mw2*(-
     msb12 + 5*mst12 + mw2)*DeltaInv(mw2,msb12,mst12) - 6*mst12*mst22*mt2*mw2*(
     -mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(
     mst22,mw2,msb12)) + 6*mst12*mst22*mt2*mw2*(-mst14 + (14*mst12 - mw2)*mw2 +
     msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) - 72*mst12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) + 72*mst12*mst22*mt2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2)
     - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*mst12*mst22) +
     (mw2*log(mw2/Q2)*power2(cb)*power2(snt)*((mst12 - mst22)*(msb12 + mt2 +
     mu2) + mst12*(mt2 + mu2)*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,
     msb12) - mst22*(mt2 + mu2)*(msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,
     mst12) + 6*mst12*mst22*(mt2 + mu2)*mw2*(mst24 + mw2*(-14*mst22 + mw2) -
     msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*mst12*mst22*(
     mt2 + mu2)*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*
     power2(DeltaInv(mw2,msb12,mst12)) - 72*mst12*mst22*(mt2 + mu2)*power2(mw2)
     *power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1)) + 72*mst12*mst22*(mt2 + mu2)*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*
     msb12*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*mst12*mst22) - (
     mw2*log(mw2/Q2)*power2(cb)*((mst12 - mst22)*((mst12*(mst22 + mu2) - mt2*(
     mst22 + 2*mu2))*(mt2 - mw2) + msb12*mst12*(4*mst22 + mt2 - mw2)) + mst12*(
     mst12 - mst22 - 2*mt2)*mu2*(msb12 - 5*mst22 - mw2)*(mt2 - mw2)*mw2*
     DeltaInv(mst22,mw2,msb12) - mst22*mt2*(-mst12 + mst22 + 2*mu2)*(mt2 - mw2)
     *mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(mw2,msb12,mst12) + 6*mst12*mst22*(
     mst12 - mst22 - 2*mt2)*mu2*(mt2 - mw2)*mw2*(mst24 + mw2*(-14*mst22 + mw2)
     - msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 6*mst12*
     mst22*mt2*(mst12 - mst22 - 2*mu2)*(mt2 - mw2)*mw2*(mst14 + mw2*(-14*mst12
     + mw2) - msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) - 72*
     mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*(mt2 - mw2)*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1)) + 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*(
     mt2 - mw2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     mst12*(mst12 - mst22)*mst22*(mt2 - mw2)) - (2*mt2*mu2*power2(cb)*power2(
     snb)*power2(log(mw2/Q2))*power3(mw2)*(-(msb22*mst22*power2(DeltaInv(mst22,
     mw2,msb12))) + msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)) + msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12)) - msb12*mst12*power2(DeltaInv(mw2,
     msb22,mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) + 2*
     msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*
     mw2*power3(DeltaInv(mw2,msb12,mst12)) - msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb22,mst12)) - 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb22,mst12))
     - 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,mst12)) + msb12*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - msb22*power3(DeltaInv(mw2,
     msb12,mst12))*power6(mst1) + msb12*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - msb12*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*msb22*(mst12 - mst22)*power2(sb)) + ((
     mt2 + mu2)*power2(cb)*power2(snb)*power2(snt)*power2(log(mw2/Q2))*power3(
     mw2)*(-(msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))) + msb12*mst22*
     power2(DeltaInv(mst22,mw2,msb22)) + msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12)) - msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*
     mst14*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,
     mst12)) - msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 2*
     msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst12*
     mw2*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*mw2*power3(DeltaInv(
     mw2,msb22,mst12)) + msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + msb12*
     power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + msb22*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     msb22) + (mu2*power2(cb)*power2(snb)*power2(snt)*power2(log(mw2/Q2))*
     power3(mw2)*(-(msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))) + msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22)) + msb22*mst12*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) + 2*msb12*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mw2*
     power3(DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*mw2*power3(DeltaInv(mw2,
     msb12,mst12)) - msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*
     mst12*mw2*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) + msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     msb12*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(
     msb12*msb22*power2(sb)) + (2*mt2*mu2*power2(snb)*power2(log(mw2/Q2))*
     power3(mw2)*power4(cb)*(-(msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))) +
     msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)) + msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))
     + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*
     mst12*mw2*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*mw2*power3(
     DeltaInv(mw2,msb12,mst12)) - msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) - 2*
     msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*
     mw2*power3(DeltaInv(mw2,msb22,mst12)) + msb12*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*
     power6(mst1) + msb12*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) +
     msb22*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*
     msb12*(mst24 + mst22*mw2) + power6(mst2)) - msb12*power3(DeltaInv(mst22,
     mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) +
     power6(mst2))))/(msb12*msb22*(mst12 - mst22)*power2(sb)) + (mt2*power2(
     snb)*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))
     - msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) + msb12*mst12*power2(
     DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb12,mst12)) - 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12))
     - 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,mst12)) + msb22*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst14*
     power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) + 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,
     mst12)) - msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) +
     msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - msb12*power3(
     DeltaInv(mw2,msb22,mst12))*power6(mst1) - msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) + msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22) +
     (2*mu2*power2(cb)*power2(snb)*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*
     (msb22*mst22*power2(DeltaInv(mst22,mw2,msb12)) - msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22)) - msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))
     + msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb12,mst12)) - 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,
     mst12)) + msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 2*
     msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst12*
     mw2*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*mst14*mw2*power3(DeltaInv(
     mw2,msb22,mst12)) - msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - msb12*
     power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) - msb22*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2)) + msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     msb22) + (mu2*power2(snb)*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*
     power4(cb)*(msb22*mst22*power2(DeltaInv(mst22,mw2,msb12)) - msb12*mst22*
     power2(DeltaInv(mst22,mw2,msb22)) - msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12)) + msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*
     mst14*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb12,mst12)) - 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,
     mst12)) + msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 2*
     msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst12*
     mw2*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*mst14*mw2*power3(DeltaInv(
     mw2,msb22,mst12)) - msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - msb12*
     power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) - msb22*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2)) + msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     msb22*power2(sb)) + (mu2*power2(cb)*power2(snb)*power2(log(mw2/Q2))*
     power3(mw2)*(msb22*mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,
     mw2,msb12)) + msb12*mst22*(-mst12 + mst22 + 2*mt2)*power2(DeltaInv(mst22,
     mw2,msb22)) + 2*mt2*(msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) -
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1)) - msb12*power2(DeltaInv(mw2,
     msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))) + msb22*(mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*mst24 - mst22*
     mw2) + 2*msb12*(mst24 + mst22*mw2) - power6(mst2)) + msb12*(mst12 - mst22
     - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22*(mst12 - mst22)
     *power2(sb)) + (mt2*mu2*mw2*log(mw2/Q2)*power2(cb)*power2(snb)*(-(msb12*
     mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + msb22*mst12*(msb12 - 5*
     mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*mw2*(-msb22 + 5*
     mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*msb22*mst22*mw2*DeltaInv(
     mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) +
     msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mst22*mw2*
     DeltaInv(mw2,msb22,mst12) + msb22*mst22*DeltaInv(mw2,msb12,mst12)*power2(
     mw2) - msb12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*
     mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*
     msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*
     msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*
     msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) - 6*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     84*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - 6*msb22*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*
     msb12*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*
     msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     ) + 144*msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(
     mst1) + 72*msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     *(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))
     - 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.
     *msb12*msb22*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mt2*mw2*log(mw2/
     Q2)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 -
     msb22*mst22 + msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,
     msb22) - msb12*msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*
     mst22*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,
     msb22,mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*
     msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 +
     mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     msb22*mst12*mst22) + (mu2*mw2*log(mw2/Q2)*power2(cb)*power2(snb)*power2(
     snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + msb22*
     mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*
     mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*msb22*
     mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*DeltaInv(
     mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,mst12) - 5*
     msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst22*DeltaInv(
     mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb22,mst12)*
     power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 +
     msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*
     mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*
     msb22*mst12*mst22) - ((mt2 + mu2)*mw2*log(mw2/Q2)*power2(cb)*power2(snb)*
     power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*
     mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*
     msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,
     mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     msb22*mst12*mst22) - (mu2*mw2*log(mw2/Q2)*power2(cb)*power2(snb)*power2(
     snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + msb22*
     mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*
     mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*msb22*
     mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*DeltaInv(
     mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,mst12) - 5*
     msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst22*DeltaInv(
     mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb22,mst12)*
     power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 +
     msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*
     mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     msb22*mst12*mst22*power2(sb)) - (mt2*mu2*mw2*log(mw2/Q2)*power2(snb)*
     power4(cb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*
     mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*
     msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,
     mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*
     msb22*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*mw2*log(mw2/Q2)*
     power2(snb)*power2(snt)*power4(cb)*(-(msb12*mst12) + msb22*mst12 + msb12*
     mst22 - msb22*mst22 + msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(
     mst22,mw2,msb12) + msb12*mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(
     mst22,mw2,msb22) - msb12*msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*
     msb22*mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*
     DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,
     mst12) + msb22*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*
     DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 +
     (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,
     msb12)) + 6*msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(
     3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*
     mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*
     mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*
     mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*
     mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*
     mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*
     msb22*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*
     mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*
     msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     144*msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     + 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(
     mst1) + 72*msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     *(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))
     - 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(
     12.*msb12*msb22*mst12*mst22*power2(sb)) + (mu2*mw2*log(mw2/Q2)*power2(snb)
     *power4(cb)*(msb12*mst12*mst22 - msb22*mst12*mst22 + 2*msb12*mst12*mt2 -
     2*msb22*mst12*mt2 - 2*msb12*mst22*mt2 + 2*msb22*mst22*mt2 + msb22*mst12*(
     mst12 - mst22 - 2*mt2)*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*mst12*(mst12 - mst22 - 2*mt2)*mw2*(-msb22 + 5*mst22 + mw2)*
     DeltaInv(mst22,mw2,msb22) + 2*msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12) - 10*msb22*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 2*
     msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) + 10*msb12*mst12*
     mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) - msb12*power2(mst12) + msb22*
     power2(mst12) - 2*msb22*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) +
     2*msb12*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*
     mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(
     3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*(
     mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22
     + mw2))*power2(DeltaInv(mst22,mw2,msb22)) + 12*msb22*mst12*mst14*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 36*msb12*msb22*mst22*mt2*mw2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) - 12*msb12*msb22*mst12*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 168*msb22*mst22*
     mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 12*
     msb12*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     msb12*msb22*mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))
     + 12*msb12*msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) + 168*msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) + 12*msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) - 12*msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 288*msb12*msb22*mst12*mst14*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 288*msb22*mst12*mst14*mst22*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 288*msb12*msb22*mst22*mt2*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*msb12*
     msb22*mst12*mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     + 288*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + 288*msb12*msb22*mst22*mt2*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 144*msb22*mst22*mt2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*msb12*mst22*mt2*power2(mst12)
     *power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*msb22*mst12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - 144*
     msb12*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) + 72*msb22*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(
     mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*mst22*(mst12 - mst22 -
     2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     msb22*mst12*mst22*(-mst12 + mst22)*power2(sb)) + (mu2*mw2*log(mw2/Q2)*
     power2(cb)*power2(snb)*(-(msb12*mst12*mst22) + msb22*mst12*mst22 - 2*
     msb12*mst12*mt2 + 2*msb22*mst12*mt2 + 2*msb12*mst22*mt2 - 2*msb22*mst22*
     mt2 + msb22*mst12*(mst12 - mst22 - 2*mt2)*mw2*(-msb12 + 5*mst22 + mw2)*
     DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 - mst22 - 2*mt2)*(msb22 -
     5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb22) - 2*msb12*msb22*mst22*mt2*
     mw2*DeltaInv(mw2,msb12,mst12) + 10*msb22*mst12*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12) + 2*msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) - 10*
     msb12*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) + msb12*power2(mst12)
     - msb22*power2(mst12) + 2*msb22*mst22*mt2*DeltaInv(mw2,msb12,mst12)*
     power2(mw2) - 2*msb12*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 6*
     msb22*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 +
     msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 12*msb22*mst12*
     mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*msb22*
     mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 12*msb12*
     msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 168*
     msb22*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) + 12*msb12*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12)) -
     36*msb12*msb22*mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,
     mst12)) - 12*msb12*msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) - 168*msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 12*msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,
     msb12,mst12))*power3(mw2) + 12*msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,
     msb22,mst12))*power3(mw2) + 288*msb12*msb22*mst12*mst14*mst22*mt2*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*msb22*mst12*mst14*mst22*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*msb12*msb22*mst22*mt2*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 288*msb12*
     msb22*mst12*mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     - 288*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 288*msb12*msb22*mst22*mt2*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 144*msb22*mst22*mt2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*msb12*mst22*mt2*power2(mst12)
     *power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 144*msb22*mst12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 144*
     msb12*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) - 72*msb22*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(
     mst24 + mst22*mw2) + power6(mst2)) + 72*msb12*mst12*mst22*(mst12 - mst22 -
     2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     msb22*mst12*mst22*(-mst12 + mst22)*power2(sb)) + (mu2*mw2*log(mw2/Q2)*
     power2(cb)*power2(snb)*(msb12*mst12*mst22 - msb22*mst12*mst22 + msb12*
     mst12*mt2 - msb22*mst12*mt2 - msb12*mst22*mt2 + msb22*mst22*mt2 + msb22*
     mst12*(mst12 - mst22 - mt2)*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,
     mw2,msb12) + msb12*mst12*(mst12 - mst22 - mt2)*mw2*(-msb22 + 5*mst22 +
     mw2)*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12) - 5*msb22*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) -
     msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*mst22*
     mt2*mw2*DeltaInv(mw2,msb22,mst12) - msb12*power2(mst12) + msb22*power2(
     mst12) - msb22*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) + msb12*
     mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*(
     mst12 - mst22 - mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 +
     mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*(mst12 -
     mst22 - mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mt2*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) - 18*msb12*msb22*mst22*mt2*mw2*power2(
     mst12)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*mst12*mst22*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb22*mst22*mt2*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*mst12*
     mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*
     mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) + 6*msb12*
     msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 84*
     msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) + 6*msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2)
     - 6*msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) -
     144*msb12*msb22*mst12*mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 144*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst22*mt2*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst12*
     mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*
     mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*
     msb12*msb22*mst22*mt2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + 72*msb22*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 72*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 72*msb22*mst12*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - 72*msb12*mst12*mst22*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*(mst12 - mst22 - mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) - 72*msb12*mst12*mst22*(mst12 - mst22 - mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*msb22*mst12*mst22*(-mst12 +
     mst22)) + (mw2*log(mw2/Q2)*power2(cb)*power2(snb)*(msb12*mst12*mst22*mt2 -
     msb22*mst12*mst22*mt2 + msb12*mst12*mst22*mu2 - msb22*mst12*mst22*mu2 + 2*
     msb12*mst12*mt2*mu2 - 2*msb22*mst12*mt2*mu2 - 2*msb12*mst22*mt2*mu2 + 2*
     msb22*mst22*mt2*mu2 + msb22*mst12*(mst12 - mst22 - 2*mt2)*mu2*(msb12 - 5*
     mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 - mst22 -
     2*mt2)*mu2*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*
     msb22*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 2*msb12*msb22*mst22*
     mt2*mu2*mw2*DeltaInv(mw2,msb12,mst12) - 10*msb22*mst12*mst22*mt2*mu2*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*mst22*mt2*mw2*DeltaInv(mw2,
     msb22,mst12) - 2*msb12*msb22*mst22*mt2*mu2*mw2*DeltaInv(mw2,msb22,mst12) +
     10*msb12*mst12*mst22*mt2*mu2*mw2*DeltaInv(mw2,msb22,mst12) - msb12*mst22*
     power2(mst12) + msb22*mst22*power2(mst12) - msb12*mu2*power2(mst12) +
     msb22*mu2*power2(mst12) + 5*msb22*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) - 5*msb12*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12)*power2(
     mst12) + msb12*mst12*power2(mst22) - msb22*mst12*power2(mst22) - msb12*
     mt2*power2(mst22) + msb22*mt2*power2(mst22) + msb12*msb22*mt2*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst22) - 5*msb22*mst12*mt2*mw2*DeltaInv(
     mw2,msb12,mst12)*power2(mst22) - msb12*msb22*mt2*mw2*DeltaInv(mw2,msb22,
     mst12)*power2(mst22) + 5*msb12*mst12*mt2*mw2*DeltaInv(mw2,msb22,mst12)*
     power2(mst22) + msb22*mst12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(
     mw2) - 2*msb22*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     msb12*mst12*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 2*msb12*
     mst22*mt2*mu2*DeltaInv(mw2,msb22,mst12)*power2(mw2) - msb22*mt2*DeltaInv(
     mw2,msb12,mst12)*power2(mst22)*power2(mw2) + msb12*mt2*DeltaInv(mw2,msb22,
     mst12)*power2(mst22)*power2(mw2) - 6*msb22*mst12*mst22*(mst12 - mst22 - 2*
     mt2)*mu2*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*(mst12 - mst22 -
     2*mt2)*mu2*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb22)) + 12*msb22*mst12*mst14*mst22*mt2*mu2*
     mw2*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb22*mst14*mst22*mt2*mw2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) - 36*msb12*msb22*mst22*
     mt2*mu2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb22*
     mst12*mst14*mt2*mw2*power2(mst22)*power2(DeltaInv(mw2,msb12,mst12)) - 18*
     msb12*msb22*mt2*mw2*power2(mst12)*power2(mst22)*power2(DeltaInv(mw2,msb12,
     mst12)) - 12*msb12*msb22*mst12*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) + 6*msb12*msb22*mst22*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - 168*msb22*mst22*mt2*mu2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*mst12*mt2*
     power2(mst22)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb22*
     mt2*power2(mst12)*power2(mst22)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 12*msb12*mst12*mst14*mst22*mt2*mu2*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) + 6*msb12*mst14*mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(
     mw2,msb22,mst12)) + 36*msb12*msb22*mst22*mt2*mu2*mw2*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12)) - 6*msb12*mst12*mst14*mt2*mw2*power2(mst22)*
     power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mt2*mw2*power2(mst12)*
     power2(mst22)*power2(DeltaInv(mw2,msb22,mst12)) + 12*msb12*msb22*mst12*
     mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*
     msb22*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) + 168*msb12*mst22*mt2*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) + 6*msb12*msb22*mst12*mt2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 84*msb12*mt2*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mst22*mt2*
     mw2*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 84*msb22*mst22*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) - 18*msb12*
     msb22*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(mst12) - 84*
     msb12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mst12) + 12*msb22*mst12*mst22*mt2*mu2*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 6*msb22*mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb22*mst12*mt2*power2(mst22)*power2(DeltaInv(mw2,
     msb12,mst12))*power3(mw2) - 12*msb12*mst12*mst22*mt2*mu2*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) + 6*msb12*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) - 6*msb12*mst12*mt2*power2(mst22)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 288*msb12*msb22*mst12*
     mst14*mst22*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb12*msb22*mst14*mst22*mt2*power2(mst12)*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mt2*power2(mst22)*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 288*msb22*mst12*mst14*mst22*mt2*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst14*mst22*mt2*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 288*msb12*
     msb22*mst22*mt2*mu2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 144*msb22*mst12*mst14*mt2*power2(mst22)*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mt2*power2(mst12)*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*
     mst22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     288*msb12*msb22*mst12*mst14*mst22*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst14*mst22*mt2*power2(mst12)*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*msb22*mst12*mst14*mt2*
     power2(mst22)*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 288*msb12*
     mst12*mst14*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) -
     144*msb12*mst14*mst22*mt2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 288*msb12*msb22*mst22*mt2*mu2*power2(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*mst12*mst14*mt2*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*msb22*
     mt2*power2(mst12)*power2(mst22)*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*msb22*mst22*mt2*power3(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 144*msb22*mst22*mt2*mu2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb22*mt2*power2(mst12)*
     power2(mst22)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb22*
     mst22*mt2*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) -
     144*msb12*mst22*mt2*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) - 72*msb12*mt2*power2(mst12)*power2(mst22)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 72*msb12*mst22*mt2*power3(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*msb22*mst12*mst22*mt2*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - 72*msb22*
     mst22*mt2*power2(mst12)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power6(mst1) + 72*msb22*mst12*mt2*power2(mst22)*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power6(mst1) - 144*msb12*mst12*mst22*mt2*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb12*
     mst22*mt2*power2(mst12)*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) - 72*msb12*mst12*mt2*power2(mst22)*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*mst12*mst22*(mst12 -
     mst22 - 2*mt2)*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*
     msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*
     mw2) + power6(mst2))))/(12.*msb12*msb22*mst12*(mst12 - mst22)*mst22) + (
     mu2*power2(cb)*power2(DeltaInv(mst22,mz2,mst12))*power2(log(mz2/Q2))*
     power3(mz2)*(mst22 - DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2)
     - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/mst12 + (mu2*power2(
     DeltaInv(mst22,mz2,mst12))*power2(log(mz2/Q2))*power3(mz2)*power4(cb)*(
     mst22 - DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(
     mst24 + mst22*mz2) + power6(mst2))))/(2.*mst12*power2(sb)) + (mu2*power2(
     cb)*power2(DeltaInv(mst22,mz2,mst12))*power2(log(mz2/Q2))*power3(mz2)*(-
     mst22 + DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(
     mst24 + mst22*mz2) + power6(mst2))))/(2.*mst12*power2(sb)) + (mu2*mz2*log(
     mz2/Q2)*power2(cb)*(-1 + mz2*(-mst12 + 5*mst22 + mz2)*DeltaInv(mst22,mz2,
     mst12) + 6*mst22*mz2*(-mst24 + (14*mst22 - mz2)*mz2 + mst12*(3*mst22 +
     mz2))*power2(DeltaInv(mst22,mz2,mst12)) - 72*mst22*power2(mz2)*power3(
     DeltaInv(mst22,mz2,mst12))*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 +
     mst22*mz2) + power6(mst2))))/(24.*mst12*mst22*power2(sb)) + (mu2*mz2*log(
     mz2/Q2)*power2(cb)*(1 + (mst12 - 5*mst22 - mz2)*mz2*DeltaInv(mst22,mz2,
     mst12) - 6*mst22*mz2*(-mst24 + (14*mst22 - mz2)*mz2 + mst12*(3*mst22 +
     mz2))*power2(DeltaInv(mst22,mz2,mst12)) + 72*mst22*power2(mz2)*power3(
     DeltaInv(mst22,mz2,mst12))*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 +
     mst22*mz2) + power6(mst2))))/(12.*mst12*mst22) + (mu2*mz2*log(mz2/Q2)*
     power4(cb)*(1 + (mst12 - 5*mst22 - mz2)*mz2*DeltaInv(mst22,mz2,mst12) - 6*
     mst22*mz2*(-mst24 + (14*mst22 - mz2)*mz2 + mst12*(3*mst22 + mz2))*power2(
     DeltaInv(mst22,mz2,mst12)) + 72*mst22*power2(mz2)*power3(DeltaInv(mst22,
     mz2,mst12))*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) +
     power6(mst2))))/(24.*mst12*mst22*power2(sb)) + (power2(cb)*power2(log(mz2/
     Q2))*(-(mst22*mu2*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2)) + mst12*
     mt2*DeltaInv(mt2,mt2,mz2)*(5*mt2 + mz2 + 8*(2*mt2 + 3*mz2)*DeltaInv(mt2,
     mt2,mz2)*power2(mt2) + 64*mz2*power2(DeltaInv(mt2,mt2,mz2))*power4(mt2)) +
     mu2*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*(mz2*(-2*mst24 + mst22*
     mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/(2.*mst12) + (log(
     mz2/Q2)*power2(cb)*(10*mst12*mst22 - mst12*mz2 - mst22*mz2 - mu2*mz2 - 4*
     mst12*mst22*mt2*(78*mt2 + 5*mz2)*DeltaInv(mt2,mt2,mz2) + mu2*(-mst12 + 5*
     mst22 + mz2)*DeltaInv(mst22,mz2,mst12)*power2(mz2) + 6*mst22*mu2*(-mst24 +
     (14*mst22 - mz2)*mz2 + mst12*(3*mst22 + mz2))*power2(mz2)*power2(DeltaInv(
     mst22,mz2,mst12)) - 384*mst12*mst22*(3*mt2 + 4*mz2)*power2(DeltaInv(mt2,
     mt2,mz2))*power3(mt2) - 4608*mst12*mst22*mz2*power3(DeltaInv(mt2,mt2,mz2))
     *power5(mt2) - 72*mst22*mu2*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     (mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(mst2)))
     )/(24.*mst12*mst22) + (mu2*power2(DeltaInv(mst22,mA2,mst12))*power2(log(
     mA2/Q2))*power3(mA2)*(-mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 -
     2*mA2*(mst12*mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/(2.*
     mst12) + (mu2*power2(cb)*power2(DeltaInv(mst22,mA2,mst12))*power2(log(mA2/
     Q2))*power3(mA2)*(-mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*
     mA2*(mst12*mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/mst12 + (
     mu2*power2(DeltaInv(mst22,mA2,mst12))*power2(log(mA2/Q2))*power3(mA2)*
     power4(cb)*(-mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(
     mst12*mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/(2.*mst12*
     power2(sb)) + (mA2*mu2*log(mA2/Q2)*(-1 + mA2*(mA2 - mst12 + 5*mst22)*
     DeltaInv(mst22,mA2,mst12) - 6*mA2*mst22*(-3*mst12*mst22 - mA2*(mst12 + 14*
     mst22) + mst24 + power2(mA2))*power2(DeltaInv(mst22,mA2,mst12)) - 72*
     mst22*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*(-2*mst12*mst24 - 2*
     mA2*(mst12*mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/(24.*
     mst12*mst22) + (mA2*mu2*log(mA2/Q2)*power2(cb)*(-1 + mA2*(mA2 - mst12 + 5*
     mst22)*DeltaInv(mst22,mA2,mst12) - 6*mA2*mst22*(-3*mst12*mst22 - mA2*(
     mst12 + 14*mst22) + mst24 + power2(mA2))*power2(DeltaInv(mst22,mA2,mst12))
     - 72*mst22*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*(-2*mst12*mst24 -
     2*mA2*(mst12*mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/(12.*
     mst12*mst22) + (mA2*mu2*log(mA2/Q2)*power4(cb)*(-1 + mA2*(mA2 - mst12 + 5*
     mst22)*DeltaInv(mst22,mA2,mst12) - 6*mA2*mst22*(-3*mst12*mst22 - mA2*(
     mst12 + 14*mst22) + mst24 + power2(mA2))*power2(DeltaInv(mst22,mA2,mst12))
     - 72*mst22*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*(-2*mst12*mst24 -
     2*mA2*(mst12*mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/(24.*
     mst12*mst22*power2(sb)) - (power2(cb)*power2(log(mA2/Q2))*(-(mst22*mu2*
     power2(DeltaInv(mst22,mA2,mst12))*power3(mA2)) + mst12*mt2*DeltaInv(mt2,
     mt2,mA2)*(mA2 + 5*mt2 + 8*(3*mA2 + 2*mt2)*DeltaInv(mt2,mt2,mA2)*power2(
     mt2) + 64*mA2*power2(DeltaInv(mt2,mt2,mA2))*power4(mt2)) + mu2*power3(mA2)
     *power3(DeltaInv(mst22,mA2,mst12))*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 +
     mst24) + mst22*power2(mA2) + power6(mst2))))/(2.*mst12) + (log(mA2/Q2)*
     power2(cb)*(mA2*mst12 + mA2*mst22 - 10*mst12*mst22 + mA2*mu2 + 4*mst12*
     mst22*mt2*(5*mA2 + 78*mt2)*DeltaInv(mt2,mt2,mA2) + (-mA2 + mst12 - 5*
     mst22)*mu2*DeltaInv(mst22,mA2,mst12)*power2(mA2) + 6*mst22*mu2*power2(mA2)
     *(-3*mst12*mst22 - mA2*(mst12 + 14*mst22) + mst24 + power2(mA2))*power2(
     DeltaInv(mst22,mA2,mst12)) + 384*mst12*mst22*(4*mA2 + 3*mt2)*power2(
     DeltaInv(mt2,mt2,mA2))*power3(mt2) + 4608*mA2*mst12*mst22*power3(DeltaInv(
     mt2,mt2,mA2))*power5(mt2) + 72*mst22*mu2*power3(mA2)*power3(DeltaInv(
     mst22,mA2,mst12))*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 + mst24) + mst22*
     power2(mA2) + power6(mst2))))/(24.*mst12*mst22) + (mu2*power2(DeltaInv(
     mst22,msb12,mC2))*power2(log(mC2/Q2))*power3(mC2)*(-mst22 + DeltaInv(
     mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/msb12 + (mC2*mu2*log(mC2/Q2)*(-1 + mC2*(mC2
     - msb12 + 5*mst22)*DeltaInv(mst22,msb12,mC2) - 6*mC2*mst22*(-3*msb12*mst22
     - mC2*(msb12 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,
     msb12,mC2)) - 72*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*
     msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(12.*msb12*mst22) + (mC2*mt2*mu2*log(mC2/Q2)*power4(cb)*(-mst12 +
     mst22 + mC2*(-mC2 + msb12 - 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) +
     mC2*mst12*(mC2 - msb12 + 5*mst22)*DeltaInv(mst22,msb12,mC2) + 6*mC2*mst12*
     mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb12,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*msb12*mst22 -
     mC2*(msb12 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb12,
     mC2)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*
     msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*power2(sb)) + (mC2*mu2*log(
     mC2/Q2)*power2(snt)*(mst12 - mst22 + mC2*(mC2 - msb12 + 5*mst12)*mst22*
     DeltaInv(msb12,mst12,mC2) + mC2*mst12*(-mC2 + msb12 - 5*mst22)*DeltaInv(
     mst22,msb12,mC2) - 6*mC2*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*
     mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*
     mst12*mst22*(-3*msb12*mst22 - mC2*(msb12 + 14*mst22) + mst24 + power2(mC2)
     )*power2(DeltaInv(mst22,msb12,mC2)) - 72*mst12*mst22*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(12.*msb12*mst12*mst22) + (mC2*mu2*
     log(mC2/Q2)*power2(cb)*power2(snt)*(mst12 - mst22 + mC2*(mC2 - msb12 + 5*
     mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*mst12*(-mC2 + msb12 - 5*
     mst22)*DeltaInv(mst22,msb12,mC2) - 6*mC2*mst12*mst22*(-3*msb12*mst12 -
     mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,
     mC2)) + 6*mC2*mst12*mst22*(-3*msb12*mst22 - mC2*(msb12 + 14*mst22) + mst24
     + power2(mC2))*power2(DeltaInv(mst22,msb12,mC2)) - 72*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb12*
     mst12*mst22) + (mC2*mu2*log(mC2/Q2)*power2(snt)*power4(cb)*(mst12 - mst22
     + mC2*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*mst12*
     (-mC2 + msb12 - 5*mst22)*DeltaInv(mst22,msb12,mC2) - 6*mC2*mst12*mst22*(-
     3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb12,mst12,mC2)) + 6*mC2*mst12*mst22*(-3*msb12*mst22 - mC2*(
     msb12 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb12,mC2))
     - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     (12.*msb12*mst12*mst22*power2(sb)) + (mC2*mu2*log(mC2/Q2)*power4(cb)*(-((
     mst12 - mst22)*(mst12 - 2*mt2)) + 2*mC2*(mC2 - msb12 + 5*mst12)*mst22*mt2*
     DeltaInv(msb12,mst12,mC2) + mC2*mst12*(mC2 - msb12 + 5*mst22)*(mst12 -
     mst22 - 2*mt2)*DeltaInv(mst22,msb12,mC2) - 12*mC2*mst12*mst22*mt2*(-3*
     msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb12,mst12,mC2)) - 6*mC2*mst12*mst22*(mst12 - mst22 - 2*mt2)*(-
     3*msb12*mst22 - mC2*(msb12 + 14*mst22) + mst24 + power2(mC2))*power2(
     DeltaInv(mst22,msb12,mC2)) - 144*mst12*mst22*mt2*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)
     *power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(12.*msb12*
     mst12*(mst12 - mst22)*mst22*power2(sb)) + (mC2*mu2*log(mC2/Q2)*power2(cb)*
     (-((mst12 - mst22)*(mst12 - mt2)) + mC2*(mC2 - msb12 + 5*mst12)*mst22*mt2*
     DeltaInv(msb12,mst12,mC2) - mC2*mst12*(mC2 - msb12 + 5*mst22)*(-mst12 +
     mst22 + mt2)*DeltaInv(mst22,msb12,mC2) - 6*mC2*mst12*mst22*mt2*(-3*msb12*
     mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb12,mst12,mC2)) + 6*mC2*mst12*mst22*(-mst12 + mst22 + mt2)*(-3*msb12*
     mst22 - mC2*(msb12 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(
     mst22,msb12,mC2)) - 72*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(msb12,
     mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) + 72*mst12*mst22*(-mst12 + mst22 + mt2)*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb12*mst12*(mst12 -
     mst22)*mst22) - (mC2*log(mC2/Q2)*power2(cb)*((mst12 - mst22)*(msb12*mst12*
     (4*mst22 + mt2) + mt2*(mst12*(mst22 + mu2) - mt2*(mst22 + 2*mu2)) - mC2*(
     msb12*mst12 + mst12*(mst22 + mu2) - mt2*(mst22 + 2*mu2))) - mC2*(mC2 -
     msb12 + 5*mst12)*mst22*(mC2 - mt2)*mt2*(mst12 - mst22 - 2*mu2)*DeltaInv(
     msb12,mst12,mC2) + mC2*mst12*(mC2 - msb12 + 5*mst22)*(mst12 - mst22 - 2*
     mt2)*(mC2 - mt2)*mu2*DeltaInv(mst22,msb12,mC2) + 6*mC2*mst12*mst22*(mC2 -
     mt2)*mt2*(mst12 - mst22 - 2*mu2)*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12)
     + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) - 6*mC2*mst12*
     mst22*(mst12 - mst22 - 2*mt2)*(mC2 - mt2)*mu2*(-3*msb12*mst22 - mC2*(msb12
     + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb12,mC2)) + 72*
     mst12*mst22*(mC2 - mt2)*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)
     *(mC2 - mt2)*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     (12.*msb12*mst12*(mst12 - mst22)*mst22*(mC2 - mt2)) + (mC2*log(mC2/Q2)*
     power2(cb)*power2(snt)*(-((mst12 - mst22)*(msb12 + mt2 + mu2)) - mC2*(mC2
     - msb12 + 5*mst12)*mst22*(mt2 + mu2)*DeltaInv(msb12,mst12,mC2) + mC2*
     mst12*(mC2 - msb12 + 5*mst22)*(mt2 + mu2)*DeltaInv(mst22,msb12,mC2) + 6*
     mC2*mst12*mst22*(mt2 + mu2)*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) +
     mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) - 6*mC2*mst12*
     mst22*(mt2 + mu2)*(-3*msb12*mst22 - mC2*(msb12 + 14*mst22) + mst24 +
     power2(mC2))*power2(DeltaInv(mst22,msb12,mC2)) + 72*mst12*mst22*(mt2 +
     mu2)*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - 72*mst12*
     mst22*(mt2 + mu2)*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     (12.*msb12*mst12*mst22) - (mC2*mu2*log(mC2/Q2)*power2(snb)*(msb12 - msb22
     + mC2*msb22*(mC2 - msb12 + 5*mst22)*DeltaInv(mst22,msb12,mC2) + mC2*msb12*
     (-mC2 + msb22 - 5*mst22)*DeltaInv(mst22,msb22,mC2) - 6*mC2*msb22*mst22*(-
     3*msb12*mst22 - mC2*(msb12 + 14*mst22) + mst24 + power2(mC2))*power2(
     DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb12*mst22*(-3*msb22*mst22 - mC2*(
     msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb22,mC2))
     - 72*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)) +
     72*msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     (12.*msb12*msb22*mst22) + (mu2*power2(DeltaInv(mH2,mst22,mst12))*power2(
     log(mH2/Q2))*power3(mH2)*(-mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*
     mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/
     (2.*mst12) + (mH2*mu2*log(mH2/Q2)*(-1 + mH2*(mH2 - mst12 + 5*mst22)*
     DeltaInv(mH2,mst22,mst12) - 6*mH2*mst22*(-3*mst12*mst22 - mH2*(mst12 + 14*
     mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) - 72*
     mst22*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*(-2*mst12*mst24 - 2*
     mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/(24.*
     mst12*mst22) + Fin3(mH2,mst22,mst22,Q2)*(-2*mt2*power2(sa)*power2(
     DeltaInv(mH2,mst22,mst22))*(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*
     power6(mst2)) - 2*mu2*power2(snt)*power2(DeltaInv(mH2,mst22,mst22))*(mst24
     + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)) + (4*ca*cb*mu2*sa*power2(
     snt)*power2(DeltaInv(mH2,mst22,mst22))*(mst24 + 4*mH2*DeltaInv(mH2,mst22,
     mst22)*power6(mst2)))/sb + 2*mu2*power2(sa)*power2(snt)*power2(DeltaInv(
     mH2,mst22,mst22))*(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)) -
     (2*mu2*power2(cb)*power2(sa)*power2(snt)*power2(DeltaInv(mH2,mst22,mst22))
     *(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)))/power2(sb) + 2*
     mu2*power2(DeltaInv(mH2,mst22,mst22))*power4(snt)*(mst24 + 4*mH2*DeltaInv(
     mH2,mst22,mst22)*power6(mst2)) - (4*ca*cb*mu2*sa*power2(DeltaInv(mH2,
     mst22,mst22))*power4(snt)*(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(
     mst2)))/sb - 2*mu2*power2(sa)*power2(DeltaInv(mH2,mst22,mst22))*power4(
     snt)*(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)) + (2*mu2*
     power2(cb)*power2(sa)*power2(DeltaInv(mH2,mst22,mst22))*power4(snt)*(mst24
     + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)))/power2(sb)) + Fin3(mst22,
     mst22,mh2,Q2)*(-2*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)) + 2*mt2*power2(sa)*power2(
     DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)) - (4*ca*cb*mu2*sa*power2(snt)*power2(DeltaInv(mst22,mst22,
     mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/sb - 2*mu2*
     power2(sa)*power2(snt)*power2(DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)) - (2*mu2*power2(cb)*power2(snt)*
     power2(DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)
     *power6(mst2)))/power2(sb) + (2*mu2*power2(cb)*power2(sa)*power2(snt)*
     power2(DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)
     *power6(mst2)))/power2(sb) + (4*ca*cb*mu2*sa*power2(DeltaInv(mst22,mst22,
     mh2))*power4(snt)*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/
     sb + 2*mu2*power2(sa)*power2(DeltaInv(mst22,mst22,mh2))*power4(snt)*(mst24
     + 4*mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)) + (2*mu2*power2(cb)*
     power2(DeltaInv(mst22,mst22,mh2))*power4(snt)*(mst24 + 4*mh2*DeltaInv(
     mst22,mst22,mh2)*power6(mst2)))/power2(sb) - (2*mu2*power2(cb)*power2(sa)*
     power2(DeltaInv(mst22,mst22,mh2))*power4(snt)*(mst24 + 4*mh2*DeltaInv(
     mst22,mst22,mh2)*power6(mst2)))/power2(sb)) + Fin3(mst22,mw2,msb12,Q2)*((
     2*(mst12 - mst22 - 2*mt2)*mu2*power2(cb)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(-mst12
     + mst22)) + (4*(mst12 - mst22 - mt2)*mu2*power2(cb)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     (mst12 - mst22)) + (2*(mst12 - mst22 - 2*mt2)*mu2*power2(cb)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(
     msb12*(-mst12 + mst22)*power2(sb)) + (2*(mst12 - mst22 - 2*mt2)*mu2*
     power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb12))*(
     mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(
     mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)) + (4*(mst12 -
     mst22 - mt2)*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,
     mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2)
     - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(-mst12 + mst22)) +
     (2*(mst12 - mst22 - 2*mt2)*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     (mst12 - mst22)*power2(sb)) + (4*mt2*mu2*power2(cb)*power2(mw2)*power2(
     snb)*power2(DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*
     (mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)))
     )/(msb12*(mst12 - mst22)*power2(sb)) + (2*(mt2 + mu2)*power2(cb)*power2(
     mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(
     mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2)
     + power6(mst2))))/msb12 + (2*mu2*power2(cb)*power2(mw2)*power2(snt)*
     power2(DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(
     msb12*power2(sb)) + (2*mt2*power2(mw2)*power2(snb)*power2(snt)*power2(
     DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 +
     (4*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 - (2*(mt2
     + mu2)*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 + (2*(
     mst12 - mst22 - 2*mt2)*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(cb)*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2)
     - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (4*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(cb)*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2)
     - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (2*(mst12 - mst22 - 2*mt2)*mu2*power2(mw2)*power2(snb)*
     power2(DeltaInv(mst22,mw2,msb12))*power4(cb)*(mst22 - DeltaInv(mst22,mw2,
     msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(
     mst2))))/(msb12*(-mst12 + mst22)*power2(sb)) + (2*mu2*power2(mw2)*power2(
     snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*power4(cb)*(mst22 -
     DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*power2(sb)) + (4*mt2*mu2*power2(cb)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 + DeltaInv(mst22,
     mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(msb12*(mst12 - mst22)*power2(sb)) + (2*mt2*power2(mw2)*
     power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 + DeltaInv(mst22,
     mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/msb12 + (4*mu2*power2(cb)*power2(mw2)*power2(snt)*power2(
     DeltaInv(mst22,mw2,msb12))*(-mst22 + DeltaInv(mst22,mw2,msb12)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 +
     (2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb12))*(-mst22 + DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*power2(
     sb)) + (4*mt2*mu2*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(cb)*(-mst22 + DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*
     mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 -
     mst22)*power2(sb)) + (2*mu2*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,
     mw2,msb12))*power4(cb)*(-mst22 + DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24
     + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     power2(sb))) + Fin3(mst22,mw2,msb22,Q2)*((2*(mst12 - mst22 - 2*mt2)*mu2*
     power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22))*(
     mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/(msb22*(-mst12 + mst22)) + (4*(mst12
     - mst22 - mt2)*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(
     mst22,mw2,msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*(mst12 -
     mst22)) + (2*(mst12 - mst22 - 2*mt2)*mu2*power2(cb)*power2(mw2)*power2(
     snb)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*
     (mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2)))
     )/(msb22*(-mst12 + mst22)*power2(sb)) + (2*(mt2 + mu2)*power2(cb)*power2(
     mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 -
     DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/msb22 + (2*mu2*power2(cb)*power2(mw2)*power2(
     snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 - DeltaInv(
     mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2)
     + power6(mst2))))/(msb22*power2(sb)) + (2*(mst12 - mst22 - 2*mt2)*mu2*
     power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22))*power4(cb)*(
     mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/(msb22*(mst12 - mst22)*power2(sb)) +
     (4*mt2*mu2*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22))*
     power4(cb)*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2)
     - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*(mst12 - mst22)*
     power2(sb)) + (4*mt2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mst22,mw2,msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*
     (mst12 - mst22)*power2(sb)) + (2*mt2*power2(mw2)*power2(snb)*power2(snt)*
     power2(DeltaInv(mst22,mw2,msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(
     mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))
     /msb22 + (4*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(
     DeltaInv(mst22,mw2,msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 +
     (2*mu2*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,
     msb22))*power4(cb)*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*power2(
     sb))) + Fin3(mst22,mz2,mst12,Q2)*((2*mu2*power2(cb)*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12))*(mst22 - DeltaInv(mst22,mz2,mst12)*(mz2*(-2*
     mst24 + mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/mst12 +
     (mu2*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*power4(cb)*(mst22 -
     DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 +
     mst22*mz2) + power6(mst2))))/(mst12*power2(sb)) + (mu2*power2(cb)*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12))*(-mst22 + DeltaInv(mst22,mz2,mst12)
     *(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))
     ))/mst12 + (mu2*power2(cb)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(
     -mst22 + DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(
     mst24 + mst22*mz2) + power6(mst2))))/(mst12*power2(sb))) + Fin3(mst22,mA2,
     mst12,Q2)*((mu2*power2(cb)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*(
     mst22 - DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 +
     mst24) + mst22*power2(mA2) + power6(mst2))))/mst12 + (mu2*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12))*(-mst22 + DeltaInv(mst22,mA2,mst12)*(-2*
     mst12*mst24 - 2*mA2*(mst12*mst22 + mst24) + mst22*power2(mA2) + power6(
     mst2))))/mst12 + (2*mu2*power2(cb)*power2(mA2)*power2(DeltaInv(mst22,mA2,
     mst12))*(-mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(
     mst12*mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/mst12 + (mu2*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*power4(cb)*(-mst22 +
     DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 + mst24) +
     mst22*power2(mA2) + power6(mst2))))/(mst12*power2(sb))) + (mu2*power2(snt)
     *power2(log(mC2/Q2))*power3(mC2)*(-(mst12*power2(DeltaInv(msb12,mst12,mC2)
     )) + power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(
     mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*
     mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/msb12 + (
     2*mu2*power2(cb)*power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(-(mst12*
     power2(DeltaInv(msb12,mst12,mC2))) + power3(DeltaInv(msb12,mst12,mC2))*(-
     2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + power2(DeltaInv(mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,
     mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2)))))/msb12 - ((mt2 + mu2)*power2(cb)*power2(snt)*power2(log(
     mC2/Q2))*power3(mC2)*(-(mst12*power2(DeltaInv(msb12,mst12,mC2))) + power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb12,mC2))*(
     mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2)))))/msb12 + (mu2*power2(snt)*
     power2(log(mC2/Q2))*power3(mC2)*power4(cb)*(-(mst12*power2(DeltaInv(msb12,
     mst12,mC2))) + power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(
     msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(
     DeltaInv(mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     /(msb12*power2(sb)) + (2*mt2*mu2*power2(log(mC2/Q2))*power3(mC2)*power4(
     cb)*(mst12*power2(DeltaInv(msb12,mst12,mC2)) - power3(DeltaInv(msb12,
     mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/(msb12*(mst12 - mst22)*power2(sb)) +
     (mu2*power2(log(mC2/Q2))*power3(mC2)*power4(cb)*(-2*mst12*mt2*power2(
     DeltaInv(msb12,mst12,mC2)) + 2*mt2*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + (mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22
     + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24)
     + mst22*power2(mC2) + power6(mst2)))))/(msb12*(mst12 - mst22)*power2(sb))
     + (2*mu2*power2(cb)*power2(log(mC2/Q2))*power3(mC2)*(-(mst12*mt2*power2(
     DeltaInv(msb12,mst12,mC2))) + mt2*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + (mst12 - mst22 - mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/(msb12*(mst12 - mst22)) + (power2(cb)
     *power2(log(mC2/Q2))*power3(mC2)*(mst12*mt2*(-mst12 + mst22 + 2*mu2)*
     power2(DeltaInv(msb12,mst12,mC2)) + mt2*(mst12 - mst22 - 2*mu2)*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - (mst12 - mst22 - 2*mt2)*mu2*power2(
     DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     /(msb12*(mst12 - mst22)) + Fin3(mst22,msb12,mC2,Q2)*((2*mu2*power2(mC2)*
     power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,
     msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/msb12 + (2*mu2*power2(mC2)*power2(snt)*power2(
     DeltaInv(mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     msb12 + (4*mu2*power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,
     msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (4*mt2*
     mu2*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*power4(cb)*(
     mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (2*mu2*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb12,
     mC2))*power4(cb)*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*
     mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*
     power2(sb)) + (2*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22
     + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24)
     + mst22*power2(mC2) + power6(mst2))))/msb12 + (2*(mst12 - mst22 - 2*mt2)*
     mu2*power2(cb)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb12*(-mst12 + mst22)) + (4*(mst12 -
     mst22 - mt2)*mu2*power2(cb)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*
     (-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)) + (2*
     (mst12 - mst22 - 2*mt2)*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(
     DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     (msb12*(mst12 - mst22)) + (4*(mst12 - mst22 - mt2)*mu2*power2(cb)*power2(
     mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(
     mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(msb12*(-mst12 + mst22)) + (2*(mt2 + mu2)*
     power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-
     mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (2*mu2*power2(mC2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb12 + (4*mu2*power2(cb)*power2(mC2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb12 - (2*(mt2 + mu2)*power2(cb)*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-
     mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (2*(mst12 - mst22 -
     2*mt2)*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*power4(cb)*(-
     mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)*
     power2(sb)) + (4*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*
     power4(cb)*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 -
     mst22)*power2(sb)) + (2*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*power2(
     snb)*power2(DeltaInv(mst22,msb12,mC2))*power4(cb)*(-mst22 + DeltaInv(
     mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(msb12*(-mst12 + mst22)*power2(sb)) + (2*
     mu2*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*
     power4(cb)*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*power2(
     sb))) + (mu2*power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2)) - msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2)) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/(msb12*msb22) + Fin3(mst22,msb22,mC2,
     Q2)*((2*mu2*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (4*mu2*
     power2(cb)*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (2*mu2*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*
     power4(cb)*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*power2(
     sb)) + (2*mu2*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(-
     mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (2*(mst12 - mst22 -
     2*mt2)*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,
     mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*
     mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(-mst12 +
     mst22)) + (4*(mst12 - mst22 - mt2)*mu2*power2(cb)*power2(mC2)*power2(snb)*
     power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(msb22*(mst12 - mst22)) + (2*(mt2 + mu2)*power2(cb)*power2(mC2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb22 + (2*(mst12 - mst22 - 2*mt2)*
     mu2*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*power4(cb)*(
     -mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22)*
     power2(sb)) + (4*mt2*mu2*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,
     msb22,mC2))*power4(cb)*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24
     - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(
     msb22*(mst12 - mst22)*power2(sb))) + Fin3(mst22,mh2,mst12,Q2)*((mu2*
     power2(cb)*power2(mh2)*power2(sa)*power2(DeltaInv(mst22,mh2,mst12))*(mst22
     - DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24)
     + mst22*power2(mh2) + power6(mst2))))/(mst12*power2(sb)) + (8*ca*cb*mu2*
     sa*power2(mh2)*power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(mst22 -
     DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power6(mst2))))/(mst12*sb) + (4*mu2*power2(mh2)*
     power2(sa)*power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(mst22 -
     DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power6(mst2))))/mst12 + (4*mu2*power2(cb)*power2(mh2)*
     power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(mst22 - DeltaInv(mst22,mh2,
     mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power6(mst2))))/(mst12*power2(sb)) + (4*mu2*power2(cb)*power2(mh2)*power2(
     sa)*power2(DeltaInv(mst22,mh2,mst12))*power4(snt)*(mst22 - DeltaInv(mst22,
     mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(
     mh2) + power6(mst2))))/(mst12*power2(sb)) + (2*ca*cb*mu2*sa*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12))*(-mst22 + DeltaInv(mst22,mh2,mst12)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power6(
     mst2))))/(mst12*sb) + (mu2*power2(mh2)*power2(sa)*power2(DeltaInv(mst22,
     mh2,mst12))*(-mst22 + DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(
     mst12*mst22 + mst24) + mst22*power2(mh2) + power6(mst2))))/mst12 + (mu2*
     power2(cb)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-mst22 +
     DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power6(mst2))))/(mst12*power2(sb)) + (4*mu2*power2(cb)
     *power2(mh2)*power2(sa)*power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(-
     mst22 + DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power6(mst2))))/(mst12*power2(sb)) + (8*ca*
     cb*mu2*sa*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*power4(snt)*(-
     mst22 + DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power6(mst2))))/(mst12*sb) + (4*mu2*power2(
     mh2)*power2(sa)*power2(DeltaInv(mst22,mh2,mst12))*power4(snt)*(-mst22 +
     DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power6(mst2))))/mst12 + (4*mu2*power2(cb)*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12))*power4(snt)*(-mst22 + DeltaInv(mst22,
     mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(
     mh2) + power6(mst2))))/(mst12*power2(sb))) + Fin3(mH2,mst22,mst12,Q2)*((2*
     ca*cb*mu2*sa*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(mst22 -
     DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) +
     mst22*power2(mH2) + power6(mst2))))/(mst12*sb) + (mu2*power2(mH2)*power2(
     sa)*power2(DeltaInv(mH2,mst22,mst12))*(mst22 - DeltaInv(mH2,mst22,mst12)*(
     -2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power6(
     mst2))))/mst12 + (4*mu2*power2(mH2)*power2(snt)*power2(DeltaInv(mH2,mst22,
     mst12))*(mst22 - DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*
     mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/mst12 + (4*mu2*
     power2(cb)*power2(mH2)*power2(sa)*power2(snt)*power2(DeltaInv(mH2,mst22,
     mst12))*(mst22 - DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*
     mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/(mst12*power2(sb)) +
     (8*ca*cb*mu2*sa*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*power4(snt)*
     (mst22 - DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power6(mst2))))/(mst12*sb) + (4*mu2*power2(
     mH2)*power2(sa)*power2(DeltaInv(mH2,mst22,mst12))*power4(snt)*(mst22 -
     DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) +
     mst22*power2(mH2) + power6(mst2))))/mst12 + (mu2*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12))*(-mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*
     mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/
     mst12 + (mu2*power2(cb)*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,mst22,
     mst12))*(-mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(
     mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/(mst12*power2(
     sb)) + (8*ca*cb*mu2*sa*power2(mH2)*power2(snt)*power2(DeltaInv(mH2,mst22,
     mst12))*(-mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(
     mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/(mst12*sb) + (
     4*mu2*power2(mH2)*power2(sa)*power2(snt)*power2(DeltaInv(mH2,mst22,mst12))
     *(-mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22
     + mst24) + mst22*power2(mH2) + power6(mst2))))/mst12 + (4*mu2*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12))*power4(snt)*(-mst22 + DeltaInv(mH2,
     mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(
     mH2) + power6(mst2))))/mst12 + (4*mu2*power2(cb)*power2(mH2)*power2(sa)*
     power2(DeltaInv(mH2,mst22,mst12))*power4(snt)*(-mst22 + DeltaInv(mH2,
     mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(
     mH2) + power6(mst2))))/(mst12*power2(sb))) + (mu2*power2(snb)*power2(log(
     mC2/Q2))*power3(mC2)*power4(cb)*(2*msb22*mst12*mt2*power2(DeltaInv(msb12,
     mst12,mC2)) - 2*msb12*mst12*mt2*power2(DeltaInv(msb22,mst12,mC2)) - 2*
     msb22*mt2*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(
     msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 2*msb12*mt2*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - (mst12 - mst22 - 2*mt2)*(-(
     msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))) + msb22*power3(DeltaInv(
     mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2)) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(
     mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))))/(msb12*msb22*(mst12 -
     mst22)*power2(sb)) + (2*mu2*power2(cb)*power2(snb)*power2(log(mC2/Q2))*
     power3(mC2)*(msb22*mst12*mt2*power2(DeltaInv(msb12,mst12,mC2)) - msb12*
     mst12*mt2*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mt2*power3(DeltaInv(
     msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + msb12*mt2*power3(DeltaInv(msb22,mst12,mC2))*
     (-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) - (mst12 - mst22 - mt2)*(-(msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2))) + msb22*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)
     *(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2))))))/(msb12*msb22*(mst12 - mst22)) + (power2(cb)*power2(snb)*
     power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*mt2*(mst12 - mst22 - 2*mu2)*
     power2(DeltaInv(msb12,mst12,mC2)) + msb12*mst12*mt2*(-mst12 + mst22 + 2*
     mu2)*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mt2*(mst12 - mst22 - 2*mu2)
     *power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*mt2*(mst12 - mst22 - 2*
     mu2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + (mst12 - mst22 - 2*
     mt2)*mu2*(-(msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))) + msb22*power3(
     DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)) + msb12*power2(DeltaInv(mst22,msb22,mC2)
     )*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22
     + mst24) + mst22*power2(mC2) + power6(mst2))))))/(msb12*msb22*(mst12 -
     mst22)) + (mu2*log(msb12/Q2)*power2(cb)*(-(mC2*mst12*mst22) - mC2*mst12*
     mt2 + mC2*mst22*mt2 + mst12*mst22*mw2 + mst12*mt2*mw2 - mst22*mt2*mw2 -
     mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*mt2*
     DeltaInv(mw2,msb12,mst12) + mC2*power2(mst12) - mw2*power2(mst12) + mst22*
     mst24*DeltaInv(mst22,mw2,msb12)*power2(mst12) - msb12*mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*power2(mst12) + 5*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)
     *power2(mst12) - mst12*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst22) +
     msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - 5*mst12*mt2*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*DeltaInv(mst22,mw2,msb12)*
     power2(mst12)*power2(mst22) + 5*mw2*DeltaInv(mst22,mw2,msb12)*power2(
     mst12)*power2(mst22) + msb12*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(
     mw2) + msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*mst12*
     mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + 5*mst12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mw2) - msb12*DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power2(mw2) + 5*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2)
     - 5*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)*power2(mw2) - 6*msb12*
     mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*
     mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12)) + 138*
     mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 138*
     mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     138*mst12*mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) + 36*msb12*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 36*msb12*power2(mst12)*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) - 138*mst12*mst14*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 36*msb12*mst22*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - mst22*mt2*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*(-mst12 + mst22 + mt2)*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*mt2*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power3(mst22) - 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power3(mst22) + 36*msb12*mst12*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*mC2*mst12*mst22*(-
     mst12 + mst22 + mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) +
     mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,msb12)*power3(mw2)
     - mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) + mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*power3(mw2) + DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power3(mw2) - 6*msb12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)
     )*power3(mw2) - 6*msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 138*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2) - 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) - 138*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst12*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mst22)*power3(mw2) - 72*mst12*mst22*mt2*power2(mC2)*(-(
     (3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*(-mst12 + mst22 + mt2)*power2(
     mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     720*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 6*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12)
     - 216*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 144*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) + 6*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 216*msb12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 216*msb12*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 6*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     216*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 216*msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 144*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 216*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 144*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power5(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mst22) - 144*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mst22) + 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) - 144*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power6(mst22)))/(6.*msb12*
     mst12*(mst12 - mst22)*mst22) + (mu2*log(msb12/Q2)*power4(cb)*(-(mC2*mst12*
     mst22) - 2*mC2*mst12*mt2 + 2*mC2*mst22*mt2 + mst12*mst22*mw2 + 2*mst12*
     mt2*mw2 - 2*mst22*mt2*mw2 - 2*mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,
     msb12) + 2*mst12*mst14*mst22*mt2*DeltaInv(mw2,msb12,mst12) + mC2*power2(
     mst12) - mw2*power2(mst12) + mst22*mst24*DeltaInv(mst22,mw2,msb12)*power2(
     mst12) - 2*msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 10*
     mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) - mst12*mst24*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) + 2*msb12*mst12*mt2*DeltaInv(
     mst22,mw2,msb12)*power2(mst22) - 10*mst12*mt2*mw2*DeltaInv(mst22,mw2,
     msb12)*power2(mst22) - msb12*DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power2(mst22) + 5*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(
     mst22) + msb12*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 2*
     msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 10*mst12*mst22*
     mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 2*msb12*mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*power2(mw2) + 10*mst12*mst22*mt2*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) - msb12*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(
     mw2) + 5*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2) - 5*
     mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)*power2(mw2) - 12*msb12*
     mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*
     mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12)) + 276*
     mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 138*
     mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     138*mst12*mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) + 72*msb12*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 36*msb12*power2(mst12)*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 12*msb12*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) - 276*mst12*mst14*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 72*msb12*mst22*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - 2*mst22*mt2*DeltaInv(msb12,mst12,mC2)*
     (mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*(-mst12 + mst22 + 2*mt2)*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 12*mC2*mst12*mst22*mt2*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power3(mst22) - 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power3(mst22) + 36*msb12*mst12*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*mC2*mst12*mst22*(mst12
     - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(
     6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,msb12)*power3(mw2) - 2*
     mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) + 2*mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*power3(mw2) + DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power3(mw2) - 12*msb12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)
     )*power3(mw2) - 6*msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 276*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2) - 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) + 12*msb12*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) - 276*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst12*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mst22)*power3(mw2) - 144*mst12*mst22*mt2*power2(mC2)*(-
     ((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12
     + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*
     power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 1440*msb12*mst12*mst22*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     1440*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12)
     - 432*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 288*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) + 12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 432*msb12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 216*msb12*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 288*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 12*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 12*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 288*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 432*msb12*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     216*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 216*msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 288*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 432*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 288*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power5(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mst22) - 288*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mst22) + 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) - 288*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 288*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power6(mst22)))/(12.*msb12*
     mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*log(msb12/Q2)*power2(snb)*
     power4(cb)*(mC2*mst12*mst22 + 2*mC2*mst12*mt2 - 2*mC2*mst22*mt2 - mst12*
     mst22*mw2 - 2*mst12*mt2*mw2 + 2*mst22*mt2*mw2 + 2*mst12*mst22*mst24*mt2*
     DeltaInv(mst22,mw2,msb12) - 2*mst12*mst14*mst22*mt2*DeltaInv(mw2,msb12,
     mst12) - mC2*power2(mst12) + mw2*power2(mst12) - mst22*mst24*DeltaInv(
     mst22,mw2,msb12)*power2(mst12) + 2*msb12*mst22*mt2*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) - 10*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power2(
     mst12) + mst12*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst22) - 2*msb12*
     mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 10*mst12*mt2*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) + msb12*DeltaInv(mst22,mw2,msb12)*
     power2(mst12)*power2(mst22) - 5*mw2*DeltaInv(mst22,mw2,msb12)*power2(
     mst12)*power2(mst22) - msb12*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(
     mw2) - 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 10*mst12*
     mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 2*msb12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - 10*mst12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mw2) + msb12*DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power2(mw2) - 5*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2)
     + 5*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)*power2(mw2) + 12*msb12*
     mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*
     mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12)) - 276*
     mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 138*
     mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     138*mst12*mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - 72*msb12*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 36*msb12*power2(mst12)*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 12*msb12*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) + 276*mst12*mst14*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 72*msb12*mst22*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 2*mst22*mt2*DeltaInv(msb12,mst12,mC2)*
     (mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*(mst12 - mst22 - 2*mt2)*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 12*mC2*mst12*mst22*mt2*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power3(mst22) + 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power3(mst22) - 36*msb12*mst12*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*mC2*mst12*mst22*(mst12
     - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(
     6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + mst12*mst22*DeltaInv(mst22,mw2,msb12)*power3(mw2) + 2*
     mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) - 2*mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*power3(mw2) - DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power3(mw2) + 12*msb12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)
     )*power3(mw2) + 6*msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 276*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2) + 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) - 12*msb12*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) + 276*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst12*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mst22)*power3(mw2) + 144*mst12*mst22*mt2*power2(mC2)*(-
     ((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12
     + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*
     power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 1440*msb12*mst12*mst22*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     1440*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12)
     + 432*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) + 288*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) - 12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 432*msb12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 288*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 12*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 12*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 288*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 432*msb12*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     216*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 216*msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 288*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 432*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 288*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power5(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mst22) + 288*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mst22) - 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) - 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 288*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 288*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power6(mst22)))/(12.*msb12*
     mst12*(mst12 - mst22)*mst22*power2(sb)) + (mu2*log(msb12/Q2)*power2(cb)*
     power2(snb)*(mC2*mst12*mst22 + mC2*mst12*mt2 - mC2*mst22*mt2 - mst12*
     mst22*mw2 - mst12*mt2*mw2 + mst22*mt2*mw2 + mst12*mst22*mst24*mt2*
     DeltaInv(mst22,mw2,msb12) - mst12*mst14*mst22*mt2*DeltaInv(mw2,msb12,
     mst12) - mC2*power2(mst12) + mw2*power2(mst12) - mst22*mst24*DeltaInv(
     mst22,mw2,msb12)*power2(mst12) + msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)
     *power2(mst12) - 5*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) +
     mst12*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*mt2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mt2*mw2*DeltaInv(mst22,
     mw2,msb12)*power2(mst22) + msb12*DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power2(mst22) - 5*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(
     mst22) - msb12*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*
     mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 5*mst12*mst22*mt2*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) + msb12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mw2) - 5*mst12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*
     power2(mw2) + msb12*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2) -
     5*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2) + 5*mst12*
     DeltaInv(mst22,mw2,msb12)*power2(mst22)*power2(mw2) + 6*msb12*mst12*mst22*
     mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst22*mst24*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst24*mw2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 138*mst22*mst24*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*
     mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*
     msb12*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) + 36*msb12*power2(mst12)*power2(mst22)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 6*msb12*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,
     msb12,mst12)) + 138*mst12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 36*msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + mst22*mt2*DeltaInv(msb12,mst12,mC2)*(mst12*(-
     (msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) - mst12*(-mst12 + mst22 + mt2)*DeltaInv(mst22,msb12,
     mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*
     mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*mt2*power2(DeltaInv(
     msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (
     msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - msb12*
     mst12*DeltaInv(mst22,mw2,msb12)*power3(mst22) + 5*mst12*mw2*DeltaInv(
     mst22,mw2,msb12)*power3(mst22) - 36*msb12*mst12*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*mC2*mst12*mst22*(-mst12 +
     mst22 + mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*
     msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + mst12*mst22*DeltaInv(mst22,mw2,msb12)*power3(mw2) +
     mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power3(mw2) - DeltaInv(mst22,mw2,msb12)*power2(mst12)*power3(
     mw2) + 6*msb12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(
     mw2) - 6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 6*msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12)
     )*power3(mw2) - 138*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) + 138*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst12*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mst22)*power3(mw2) + 72*mst12*mst22*mt2*power2(mC2)*(-(
     (3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*(-mst12 + mst22 + mt2)*power2(
     mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     720*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 6*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12)
     + 216*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) + 144*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) - 6*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 216*msb12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     216*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 216*msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 144*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 216*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 144*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power5(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mst22) + 144*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mst22) - 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) - 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power6(mst22)))/(6.*msb12*
     mst12*(mst12 - mst22)*mst22) - (log(msb12/Q2)*power2(cb)*(-(mC2*mst12*
     mst22*mt2) - mC2*mst12*mst22*mu2 - 2*mC2*mst12*mt2*mu2 + 2*mC2*mst22*mt2*
     mu2 + mst12*mst22*mt2*mw2 + mst12*mst22*mu2*mw2 + 2*mst12*mt2*mu2*mw2 - 2*
     mst22*mt2*mu2*mw2 - 2*mst12*mst22*mst24*mt2*mu2*DeltaInv(mst22,mw2,msb12)
     + 2*mst12*mst14*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12) + mC2*mu2*power2(
     mst12) - mu2*mw2*power2(mst12) + mst22*mst24*mu2*DeltaInv(mst22,mw2,msb12)
     *power2(mst12) - mst14*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst12) -
     2*msb12*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 10*mst22*
     mt2*mu2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + mC2*mt2*power2(
     mst22) - mt2*mw2*power2(mst22) - mst12*mst24*mu2*DeltaInv(mst22,mw2,msb12)
     *power2(mst22) + 2*msb12*mst12*mt2*mu2*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) - 10*mst12*mt2*mu2*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) +
     mst12*mst14*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst22) - msb12*mu2*
     DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mst22) + 5*mu2*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mst22) - msb12*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mst22) + 5*mt2*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mst22) + msb12*mst12*mst22*
     mu2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 2*msb12*mst12*mt2*mu2*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - 10*mst12*mst22*mt2*mu2*DeltaInv(
     mst22,mw2,msb12)*power2(mw2) + msb12*mst12*mst22*mt2*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) - 2*msb12*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*
     power2(mw2) + 10*mst12*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*power2(mw2)
     - msb12*mu2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2) + 5*mst22*
     mu2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2) - 5*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mw2) - 5*mst12*mu2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22)*power2(mw2) - msb12*mt2*DeltaInv(
     mw2,msb12,mst12)*power2(mst22)*power2(mw2) + 5*mst12*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mst22)*power2(mw2) - 12*msb12*mst12*mst22*mst24*mt2*
     mu2*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst22*mst24*mu2*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst24*mu2*
     mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12)) + 276*mst12*mst22*
     mst24*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst22*
     mst24*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     138*mst12*mst24*mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 72*msb12*mst12*mt2*mu2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 36*msb12*mu2*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 12*msb12*mst12*mst14*
     mst22*mt2*mu2*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*mst14*mst22*
     mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*
     mst14*mt2*mw2*power2(mst22)*power2(DeltaInv(mw2,msb12,mst12)) - 276*mst12*
     mst14*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 138*
     mst14*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) - 72*msb12*mst22*mt2*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 138*mst12*mst14*mt2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 36*msb12*mt2*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*mt2*(mst12 - mst22 -
     2*mu2)*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12
     - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*(-
     mst12 + mst22 + 2*mt2)*mu2*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*
     mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) + msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power3(mst12) -
     5*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power3(mst12) + 36*msb12*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) - 6*mC2*
     mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(msb12,mst12,mC2))*
     (-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + msb12*mst12*mu2*DeltaInv(
     mst22,mw2,msb12)*power3(mst22) - 5*mst12*mu2*mw2*DeltaInv(mst22,mw2,msb12)
     *power3(mst22) + 36*msb12*mst12*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mst22) + 6*mC2*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*
     power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 +
     23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     - mst12*mst22*mu2*DeltaInv(mst22,mw2,msb12)*power3(mw2) - 2*mst12*mt2*mu2*
     DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power3(mw2) + 2*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*
     power3(mw2) + mu2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power3(mw2) +
     mt2*DeltaInv(mw2,msb12,mst12)*power2(mst22)*power3(mw2) - 12*msb12*mst12*
     mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*
     mst22*mu2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*
     msb12*mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(
     mw2) + 276*mst12*mt2*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 138*mu2*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2) + 12*msb12*mst12*mst22*mt2*mu2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) - 6*msb12*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 276*mst22*mt2*mu2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mt2*power2(
     mst22)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mt2*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*
     mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12)*power3(mw2) +
     138*mst12*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22)*power3(mw2)
     + 72*mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*(-((3*msb12*mst12
     + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*
     power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)
     *(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*
     msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 1440*msb12*mst12*mst22*mst24*mt2*mu2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst22*mst24*mu2*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*
     mst12*mst24*mu2*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     ) - 1440*msb12*mst12*mst14*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 720*msb12*mst14*mst22*mt2*power2(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 720*msb12*mst12*mst14*mt2*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 12*mst22*mt2*mu2*
     mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*mt2*mw2*power2(
     mst22)*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 432*msb12*mst22*
     mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 216*
     msb12*mt2*power2(mst22)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 288*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 144*mt2*power2(mst22)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 12*mst12*mt2*mu2*mw2*power2(DeltaInv(
     mst22,mw2,msb12))*power4(mst22) - 6*mu2*mw2*power2(mst12)*power2(DeltaInv(
     mst22,mw2,msb12))*power4(mst22) + 432*msb12*mst12*mt2*mu2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*mu2*power2(
     mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 288*
     mst12*mt2*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     - 144*mu2*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 12*mst12*mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 6*mst22*mu2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 12*mst12*mst22*mt2*mu2*power2(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 6*mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 6*mst12*mt2*power2(mst22)*power2(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 288*mst12*mst22*mst24*mt2*mu2*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 144*mst22*mst24*mu2*power2(mst12)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 144*mst12*mst24*mu2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 432*msb12*mst12*mt2*mu2*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mu2*
     power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 216*msb12*mst12*mu2*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 288*mst12*mst14*mst22*mt2*mu2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 144*mst14*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 432*msb12*mst22*mt2*mu2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst14*mt2*
     power2(mst22)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 216*msb12*
     mt2*power2(mst12)*power2(mst22)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 216*msb12*mst22*mt2*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) + 6*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 216*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     *power5(mst12) + 288*mst22*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mst12) + 144*mt2*power2(mst22)*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) + 144*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mst12) + 6*mst12*mu2*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 216*msb12*mst12*mu2*power2(mw2)
     *power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 288*mst12*mt2*mu2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mu2*
     power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22)
     + 144*mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 288*mst12*mt2*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))
     *power5(mw2) + 144*mu2*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mw2) - 144*mst12*mu2*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 288*mst22*mt2*mu2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) + 144*mt2*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 144*mst22*mt2*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 144*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst12) - 144*
     mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power6(mst22)))/(
     12.*msb12*mst12*(mst12 - mst22)*mst22) + (log(msb12/Q2)*power2(cb)*power2(
     snb)*(-(mC2*mst12*mst22*mt2) - mC2*mst12*mst22*mu2 - 2*mC2*mst12*mt2*mu2 +
     2*mC2*mst22*mt2*mu2 + mst12*mst22*mt2*mw2 + mst12*mst22*mu2*mw2 + 2*mst12*
     mt2*mu2*mw2 - 2*mst22*mt2*mu2*mw2 - 2*mst12*mst22*mst24*mt2*mu2*DeltaInv(
     mst22,mw2,msb12) + 2*mst12*mst14*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12) +
     mC2*mu2*power2(mst12) - mu2*mw2*power2(mst12) + mst22*mst24*mu2*DeltaInv(
     mst22,mw2,msb12)*power2(mst12) - mst14*mst22*mt2*DeltaInv(mw2,msb12,mst12)
     *power2(mst12) - 2*msb12*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*power2(
     mst12) + 10*mst22*mt2*mu2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) +
     mC2*mt2*power2(mst22) - mt2*mw2*power2(mst22) - mst12*mst24*mu2*DeltaInv(
     mst22,mw2,msb12)*power2(mst22) + 2*msb12*mst12*mt2*mu2*DeltaInv(mst22,mw2,
     msb12)*power2(mst22) - 10*mst12*mt2*mu2*mw2*DeltaInv(mst22,mw2,msb12)*
     power2(mst22) + mst12*mst14*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst22) -
     msb12*mu2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mst22) + 5*mu2*
     mw2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mst22) - msb12*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mst22) + 5*mt2*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mst22) + msb12*mst12*mst22*
     mu2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 2*msb12*mst12*mt2*mu2*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - 10*mst12*mst22*mt2*mu2*DeltaInv(
     mst22,mw2,msb12)*power2(mw2) + msb12*mst12*mst22*mt2*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) - 2*msb12*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*
     power2(mw2) + 10*mst12*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*power2(mw2)
     - msb12*mu2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2) + 5*mst22*
     mu2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2) - 5*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mw2) - 5*mst12*mu2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22)*power2(mw2) - msb12*mt2*DeltaInv(
     mw2,msb12,mst12)*power2(mst22)*power2(mw2) + 5*mst12*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mst22)*power2(mw2) - 12*msb12*mst12*mst22*mst24*mt2*
     mu2*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst22*mst24*mu2*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst24*mu2*
     mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12)) + 276*mst12*mst22*
     mst24*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst22*
     mst24*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     138*mst12*mst24*mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 72*msb12*mst12*mt2*mu2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 36*msb12*mu2*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 12*msb12*mst12*mst14*
     mst22*mt2*mu2*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*mst14*mst22*
     mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*
     mst14*mt2*mw2*power2(mst22)*power2(DeltaInv(mw2,msb12,mst12)) - 276*mst12*
     mst14*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 138*
     mst14*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) - 72*msb12*mst22*mt2*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 138*mst12*mst14*mt2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 36*msb12*mt2*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*mt2*(mst12 - mst22 -
     2*mu2)*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12
     - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*(-
     mst12 + mst22 + 2*mt2)*mu2*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*
     mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) + msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power3(mst12) -
     5*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power3(mst12) + 36*msb12*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) - 6*mC2*
     mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(msb12,mst12,mC2))*
     (-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + msb12*mst12*mu2*DeltaInv(
     mst22,mw2,msb12)*power3(mst22) - 5*mst12*mu2*mw2*DeltaInv(mst22,mw2,msb12)
     *power3(mst22) + 36*msb12*mst12*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mst22) + 6*mC2*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*
     power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 +
     23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     - mst12*mst22*mu2*DeltaInv(mst22,mw2,msb12)*power3(mw2) - 2*mst12*mt2*mu2*
     DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power3(mw2) + 2*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*
     power3(mw2) + mu2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power3(mw2) +
     mt2*DeltaInv(mw2,msb12,mst12)*power2(mst22)*power3(mw2) - 12*msb12*mst12*
     mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*
     mst22*mu2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*
     msb12*mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(
     mw2) + 276*mst12*mt2*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 138*mu2*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2) + 12*msb12*mst12*mst22*mt2*mu2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) - 6*msb12*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 276*mst22*mt2*mu2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mt2*power2(
     mst22)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mt2*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*
     mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12)*power3(mw2) +
     138*mst12*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22)*power3(mw2)
     + 72*mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*(-((3*msb12*mst12
     + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*
     power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)
     *(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*
     msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 1440*msb12*mst12*mst22*mst24*mt2*mu2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst22*mst24*mu2*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*
     mst12*mst24*mu2*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     ) - 1440*msb12*mst12*mst14*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 720*msb12*mst14*mst22*mt2*power2(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 720*msb12*mst12*mst14*mt2*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 12*mst22*mt2*mu2*
     mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*mt2*mw2*power2(
     mst22)*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 432*msb12*mst22*
     mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 216*
     msb12*mt2*power2(mst22)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 288*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 144*mt2*power2(mst22)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 12*mst12*mt2*mu2*mw2*power2(DeltaInv(
     mst22,mw2,msb12))*power4(mst22) - 6*mu2*mw2*power2(mst12)*power2(DeltaInv(
     mst22,mw2,msb12))*power4(mst22) + 432*msb12*mst12*mt2*mu2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*mu2*power2(
     mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 288*
     mst12*mt2*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     - 144*mu2*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 12*mst12*mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 6*mst22*mu2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 12*mst12*mst22*mt2*mu2*power2(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 6*mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 6*mst12*mt2*power2(mst22)*power2(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 288*mst12*mst22*mst24*mt2*mu2*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 144*mst22*mst24*mu2*power2(mst12)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 144*mst12*mst24*mu2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 432*msb12*mst12*mt2*mu2*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mu2*
     power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 216*msb12*mst12*mu2*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 288*mst12*mst14*mst22*mt2*mu2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 144*mst14*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 432*msb12*mst22*mt2*mu2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst14*mt2*
     power2(mst22)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 216*msb12*
     mt2*power2(mst12)*power2(mst22)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 216*msb12*mst22*mt2*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) + 6*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 216*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     *power5(mst12) + 288*mst22*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mst12) + 144*mt2*power2(mst22)*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) + 144*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mst12) + 6*mst12*mu2*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 216*msb12*mst12*mu2*power2(mw2)
     *power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 288*mst12*mt2*mu2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mu2*
     power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22)
     + 144*mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 288*mst12*mt2*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))
     *power5(mw2) + 144*mu2*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mw2) - 144*mst12*mu2*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 288*mst22*mt2*mu2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) + 144*mt2*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 144*mst22*mt2*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 144*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst12) - 144*
     mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power6(mst22)))/(
     12.*msb12*mst12*(mst12 - mst22)*mst22) + (mu2*log(msb22/Q2)*power2(cb)*
     power2(snb)*(-(mC2*mst12*mst22) - mC2*mst12*mt2 + mC2*mst22*mt2 + mst12*
     mst22*mw2 + mst12*mt2*mw2 - mst22*mt2*mw2 - mst12*mst22*mst24*mt2*
     DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*mt2*DeltaInv(mw2,msb22,
     mst12) + mC2*power2(mst12) - mw2*power2(mst12) + mst22*mst24*DeltaInv(
     mst22,mw2,msb22)*power2(mst12) - msb22*mst22*mt2*DeltaInv(mw2,msb22,mst12)
     *power2(mst12) + 5*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) -
     mst12*mst24*DeltaInv(mst22,mw2,msb22)*power2(mst22) + msb22*mst12*mt2*
     DeltaInv(mst22,mw2,msb22)*power2(mst22) - 5*mst12*mt2*mw2*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) - msb22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*
     power2(mst22) + 5*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(
     mst22) + msb22*mst12*mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*
     mst12*mt2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*mst12*mst22*mt2*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*mt2*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) + 5*mst12*mst22*mt2*DeltaInv(mw2,msb22,mst12)*
     power2(mw2) - msb22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(mw2) +
     5*mst22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(mw2) - 5*mst12*
     DeltaInv(mst22,mw2,msb22)*power2(mst22)*power2(mw2) - 6*msb22*mst12*mst22*
     mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst22*mst24*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst24*mw2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb22)) + 138*mst12*mst22*mst24*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 138*mst22*mst24*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 138*mst12*
     mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 36*
     msb22*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)
     ) - 36*msb22*power2(mst12)*power2(mst22)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) - 138*mst12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) - 36*msb22*mst22*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - mst22*mt2*DeltaInv(msb22,mst12,mC2)*(mst12*(-
     (msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) + mst12*(-mst12 + mst22 + mt2)*DeltaInv(mst22,msb22,
     mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*
     mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*mt2*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + msb22*
     mst12*DeltaInv(mst22,mw2,msb22)*power3(mst22) - 5*mst12*mw2*DeltaInv(
     mst22,mw2,msb22)*power3(mst22) + 36*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mst22) - 6*mC2*mst12*mst22*(-mst12 +
     mst22 + mt2)*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,msb22)*power3(mw2) -
     mst12*mt2*DeltaInv(mst22,mw2,msb22)*power3(mw2) + mst22*mt2*DeltaInv(mw2,
     msb22,mst12)*power3(mw2) + DeltaInv(mst22,mw2,msb22)*power2(mst12)*power3(
     mw2) - 6*msb22*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*power3(
     mw2) + 6*msb22*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 6*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22)
     )*power3(mw2) + 138*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mw2) - 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) - 138*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst12*power2(DeltaInv(mst22,
     mw2,msb22))*power3(mst22)*power3(mw2) - 72*mst12*mst22*mt2*power2(mC2)*(-(
     (3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*(-mst12 + mst22 + mt2)*power2(
     mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     720*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 6*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     - 216*msb22*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) - 144*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mst12) + 6*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) + 216*msb22*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 144*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 6*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 216*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) -
     216*msb22*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 216*msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 144*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2) - 216*msb22*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 144*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power5(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power5(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power5(mst22) - 144*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power5(mst22) + 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) - 144*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power6(mst22)))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + (mu2*log(msb22/Q2)*power2(snb)*power4(cb)*(
     -(mC2*mst12*mst22) - 2*mC2*mst12*mt2 + 2*mC2*mst22*mt2 + mst12*mst22*mw2 +
     2*mst12*mt2*mw2 - 2*mst22*mt2*mw2 - 2*mst12*mst22*mst24*mt2*DeltaInv(
     mst22,mw2,msb22) + 2*mst12*mst14*mst22*mt2*DeltaInv(mw2,msb22,mst12) +
     mC2*power2(mst12) - mw2*power2(mst12) + mst22*mst24*DeltaInv(mst22,mw2,
     msb22)*power2(mst12) - 2*msb22*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(
     mst12) + 10*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) - mst12*
     mst24*DeltaInv(mst22,mw2,msb22)*power2(mst22) + 2*msb22*mst12*mt2*
     DeltaInv(mst22,mw2,msb22)*power2(mst22) - 10*mst12*mt2*mw2*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) - msb22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*
     power2(mst22) + 5*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(
     mst22) + msb22*mst12*mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 2*
     msb22*mst12*mt2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 10*mst12*mst22*
     mt2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 2*msb22*mst22*mt2*DeltaInv(
     mw2,msb22,mst12)*power2(mw2) + 10*mst12*mst22*mt2*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) - msb22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(
     mw2) + 5*mst22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(mw2) - 5*
     mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22)*power2(mw2) - 12*msb22*
     mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*
     mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*
     mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22)) + 276*
     mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 138*
     mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     138*mst12*mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)
     ) + 72*msb22*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) - 36*msb22*power2(mst12)*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 12*msb22*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) - 276*mst12*mst14*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 2*mst22*mt2*DeltaInv(msb22,mst12,mC2)*
     (mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*(-mst12 + mst22 + 2*mt2)*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 12*mC2*mst12*mst22*mt2*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power3(mst22) - 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb22)*power3(mst22) + 36*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*mst12*mst22*(mst12
     - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(
     6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,msb22)*power3(mw2) - 2*
     mst12*mt2*DeltaInv(mst22,mw2,msb22)*power3(mw2) + 2*mst22*mt2*DeltaInv(
     mw2,msb22,mst12)*power3(mw2) + DeltaInv(mst22,mw2,msb22)*power2(mst12)*
     power3(mw2) - 12*msb22*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) + 6*msb22*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22)
     )*power3(mw2) - 6*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mw2) + 276*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb22))*power3(mw2) - 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 12*msb22*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) - 276*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst12*power2(DeltaInv(mst22,
     mw2,msb22))*power3(mst22)*power3(mw2) - 144*mst12*mst22*mt2*power2(mC2)*(-
     ((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22
     + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*
     power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 1440*msb22*mst12*mst22*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     1440*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     - 432*msb22*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) - 288*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mst12) + 12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) + 432*msb22*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 288*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 12*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 12*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 288*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 432*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) -
     216*msb22*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 216*msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 288*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2) - 432*msb22*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 288*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power5(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power5(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power5(mst22) - 288*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power5(mst22) + 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) - 288*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 288*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power6(mst22)))/(12.*msb22*
     mst12*(mst12 - mst22)*mst22*power2(sb)) - (log(msb22/Q2)*power2(cb)*
     power2(snb)*(-(mC2*mst12*mst22*mt2) - mC2*mst12*mst22*mu2 - 2*mC2*mst12*
     mt2*mu2 + 2*mC2*mst22*mt2*mu2 + mst12*mst22*mt2*mw2 + mst12*mst22*mu2*mw2
     + 2*mst12*mt2*mu2*mw2 - 2*mst22*mt2*mu2*mw2 - 2*mst12*mst22*mst24*mt2*mu2*
     DeltaInv(mst22,mw2,msb22) + 2*mst12*mst14*mst22*mt2*mu2*DeltaInv(mw2,
     msb22,mst12) + mC2*mu2*power2(mst12) - mu2*mw2*power2(mst12) + mst22*
     mst24*mu2*DeltaInv(mst22,mw2,msb22)*power2(mst12) - mst14*mst22*mt2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) - 2*msb22*mst22*mt2*mu2*DeltaInv(
     mw2,msb22,mst12)*power2(mst12) + 10*mst22*mt2*mu2*mw2*DeltaInv(mw2,msb22,
     mst12)*power2(mst12) + mC2*mt2*power2(mst22) - mt2*mw2*power2(mst22) -
     mst12*mst24*mu2*DeltaInv(mst22,mw2,msb22)*power2(mst22) + 2*msb22*mst12*
     mt2*mu2*DeltaInv(mst22,mw2,msb22)*power2(mst22) - 10*mst12*mt2*mu2*mw2*
     DeltaInv(mst22,mw2,msb22)*power2(mst22) + mst12*mst14*mt2*DeltaInv(mw2,
     msb22,mst12)*power2(mst22) - msb22*mu2*DeltaInv(mst22,mw2,msb22)*power2(
     mst12)*power2(mst22) + 5*mu2*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst12)*
     power2(mst22) - msb22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mst12)*power2(
     mst22) + 5*mt2*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12)*power2(mst22) +
     msb22*mst12*mst22*mu2*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 2*msb22*
     mst12*mt2*mu2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 10*mst12*mst22*mt2*
     mu2*DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*mst12*mst22*mt2*
     DeltaInv(mw2,msb22,mst12)*power2(mw2) - 2*msb22*mst22*mt2*mu2*DeltaInv(
     mw2,msb22,mst12)*power2(mw2) + 10*mst12*mst22*mt2*mu2*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) - msb22*mu2*DeltaInv(mst22,mw2,msb22)*power2(mst12)*
     power2(mw2) + 5*mst22*mu2*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(
     mw2) - 5*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mst12)*power2(mw2) -
     5*mst12*mu2*DeltaInv(mst22,mw2,msb22)*power2(mst22)*power2(mw2) - msb22*
     mt2*DeltaInv(mw2,msb22,mst12)*power2(mst22)*power2(mw2) + 5*mst12*mt2*
     DeltaInv(mw2,msb22,mst12)*power2(mst22)*power2(mw2) - 12*msb22*mst12*
     mst22*mst24*mt2*mu2*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst22*
     mst24*mu2*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*
     mst12*mst24*mu2*mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22)) + 276*
     mst12*mst22*mst24*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     138*mst22*mst24*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) + 138*mst12*mst24*mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 72*msb22*mst12*mt2*mu2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 36*msb22*mu2*power2(mst12)*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 12*msb22*mst12*
     mst14*mst22*mt2*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst14*
     mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) + 6*msb22*
     mst12*mst14*mt2*mw2*power2(mst22)*power2(DeltaInv(mw2,msb22,mst12)) - 276*
     mst12*mst14*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     138*mst14*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) - 72*msb22*mst22*mt2*mu2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 138*mst12*mst14*mt2*power2(mst22)*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 36*msb22*mt2*power2(mst12)*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + mst22*mt2*(mst12 -
     mst22 - 2*mu2)*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) -
     (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) +
     mst12*(-mst12 + mst22 + 2*mt2)*mu2*DeltaInv(mst22,msb22,mC2)*(mst22*(-(
     msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + msb22*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power3(
     mst12) - 5*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12)*power3(mst12) + 36*
     msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mst12) - 6*mC2*mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + msb22*
     mst12*mu2*DeltaInv(mst22,mw2,msb22)*power3(mst22) - 5*mst12*mu2*mw2*
     DeltaInv(mst22,mw2,msb22)*power3(mst22) + 36*msb22*mst12*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*mst12*mst22*(mst12
     - mst22 - 2*mt2)*mu2*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) +
     mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) - mst12*mst22*mu2*DeltaInv(mst22,mw2,msb22)*power3(
     mw2) - 2*mst12*mt2*mu2*DeltaInv(mst22,mw2,msb22)*power3(mw2) - mst12*
     mst22*mt2*DeltaInv(mw2,msb22,mst12)*power3(mw2) + 2*mst22*mt2*mu2*
     DeltaInv(mw2,msb22,mst12)*power3(mw2) + mu2*DeltaInv(mst22,mw2,msb22)*
     power2(mst12)*power3(mw2) + mt2*DeltaInv(mw2,msb22,mst12)*power2(mst22)*
     power3(mw2) - 12*msb22*mst12*mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mw2) + 6*msb22*mst22*mu2*power2(mst12)*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mu2*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) + 276*mst12*mt2*mu2*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mu2*power2(mst12)*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 12*msb22*
     mst12*mst22*mt2*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 6*
     msb22*mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mw2) - 276*mst22*mt2*mu2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 6*msb22*mst12*mt2*power2(mst22)*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 138*mt2*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) + 138*mst22*mt2*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mst12)*power3(mw2) + 138*mst12*mu2*power2(DeltaInv(mst22,
     mw2,msb22))*power3(mst22)*power3(mw2) + 72*mst12*mst22*mt2*(mst12 - mst22
     - 2*mu2)*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*(mst12
     - mst22 - 2*mt2)*mu2*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2))
     + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*
     msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 1440*
     msb22*mst12*mst22*mst24*mt2*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 720*msb22*mst22*mst24*mu2*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst24*mu2*power2(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 1440*msb22*mst12*mst14*
     mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 720*msb22*
     mst14*mst22*mt2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     ) - 720*msb22*mst12*mst14*mt2*power2(mst22)*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12)) - 12*mst22*mt2*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12))
     *power4(mst12) - 6*mt2*mw2*power2(mst22)*power2(DeltaInv(mw2,msb22,mst12))
     *power4(mst12) - 432*msb22*mst22*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 216*msb22*mt2*power2(mst22)*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 288*mst22*mt2*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 144*mt2*
     power2(mst22)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     + 12*mst12*mt2*mu2*mw2*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) -
     6*mu2*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) +
     432*msb22*mst12*mt2*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 216*msb22*mu2*power2(mst12)*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power4(mst22) + 288*mst12*mt2*mu2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 144*mu2*power2(mst12)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 12*mst12*mst22*mt2*
     mu2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst22*mu2*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 6*mst12*mu2*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 12*mst12*mst22*mt2*
     mu2*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 6*mst22*mt2*power2(
     mst12)*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 6*mst12*mt2*power2(
     mst22)*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 288*mst12*mst22*
     mst24*mt2*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 144*mst22*
     mst24*mu2*power2(mst12)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) +
     144*mst12*mst24*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 432*msb22*mst12*mt2*mu2*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) - 216*msb22*mu2*power2(mst12)*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 216*msb22*mst12*mu2*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 288*mst12*
     mst14*mst22*mt2*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*
     mst14*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 432*msb22*mst22*mt2*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2) - 144*mst12*mst14*mt2*power2(mst22)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) - 216*msb22*mt2*power2(mst12)*power2(mst22)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 216*msb22*mst22*mt2*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 6*mst22*mt2*
     mw2*power2(DeltaInv(mw2,msb22,mst12))*power5(mst12) + 216*msb22*mst22*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mst12) + 288*mst22*
     mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mst12) + 144*
     mt2*power2(mst22)*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mst12) + 6*mst12*mu2*mw2*power2(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 216*msb22*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))
     *power5(mst22) - 288*mst12*mt2*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mst22) + 144*mu2*power2(mst12)*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power5(mst22) + 144*mst12*mu2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) - 288*mst12*mt2*mu2*power2(mst22)
     *power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*mu2*power2(mst12)*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 144*mst12*
     mu2*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 288*
     mst22*mt2*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)
     + 144*mt2*power2(mst12)*power2(mst22)*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mw2) - 144*mst22*mt2*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)
     )*power5(mw2) - 144*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power6(mst12) - 144*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power6(mst22)))/(12.*msb22*mst12*(mst12 - mst22)*mst22) - (mu2*
     log(mst12/Q2)*power4(cb)*(-(mA2*msb12*mst12) + mA2*msb12*mst22 + msb12*
     mst12*mz2 - msb12*mst22*mz2 - 4*mst12*(5*mC2*mst12 - msb12*mst12 + mst14)*
     mst22*mt2*DeltaInv(msb12,mst12,mC2) - msb12*mst12*mst22*mst24*DeltaInv(
     mst22,mz2,mst12) + 4*mst12*mst14*mst22*mt2*DeltaInv(mw2,msb12,mst12) - 4*
     msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 20*mst22*mt2*
     mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst24*DeltaInv(mst22,
     mz2,mst12)*power2(mst22) - 5*msb12*mst12*mz2*DeltaInv(mst22,mz2,mst12)*
     power2(mst22) + msb12*DeltaInv(mst22,mz2,mst12)*power2(mst12)*power2(
     mst22) - 6*msb12*mst12*mst22*DeltaInv(mst22,mz2,mst12)*power2(mz2) +
     msb12*DeltaInv(mst22,mz2,mst12)*power2(mst12)*power2(mz2) + 5*msb12*
     DeltaInv(mst22,mz2,mst12)*power2(mst22)*power2(mz2) - 6*msb12*mst22*mst24*
     mz2*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12)) + 6*msb12*mst12*mst24*
     mz2*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12)) + 138*msb12*mst12*
     mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 138*msb12*
     mst24*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 36*
     msb12*power2(mst12)*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) + 24*msb12*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,
     mst12)) + 336*mst12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 72*msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) + msb12*(mst12 - mst22)*DeltaInv(mst22,mA2,mst12)*(
     mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mA2) + 5*mA2*
     power2(mst22) + power3(mA2)) + 24*mC2*mst12*mst22*mt2*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - msb12*mst12*DeltaInv(mst22,mz2,mst12)
     *power3(mst22) + 5*msb12*mz2*DeltaInv(mst22,mz2,mst12)*power3(mst22) - 36*
     msb12*mst12*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22) -
     6*mA2*msb12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mA2,mst12))*(-(
     mst12*mst24) + mA2*(6*mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(
     mA2) + power3(mA2) + power3(mst22)) - 24*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - msb12*mst12*DeltaInv(mst22,mz2,
     mst12)*power3(mz2) + msb12*mst22*DeltaInv(mst22,mz2,mst12)*power3(mz2) -
     6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2)
     + 144*msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(
     mz2) - 138*msb12*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22)*power3(
     mz2) + 288*mst12*mst22*mt2*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*msb12*(mst12 - mst22)*mst22*power2(mA2)*((
     3*mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*
     mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mA2,mst12)) + 720*msb12*mst22*mst24*power2(mst12)*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 720*msb12*mst12*mst24*power2(
     mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 576*msb12*mst12*
     mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 24*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 576*msb12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 576*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*
     msb12*mst12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 216*
     msb12*power2(mst12)*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(
     mst22) + 144*msb12*mst12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 288*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 6*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 144*msb12*mst12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12)
     )*power4(mz2) - 144*msb12*mst24*power2(mst22)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) + 216*msb12*power2(mst12)*power2(mst22)*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) - 216*msb12*mst12*power3(mst22)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 288*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 6*msb12*mz2*power2(
     DeltaInv(mst22,mz2,mst12))*power5(mst22) - 360*msb12*mst12*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 144*msb12*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 144*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 144*msb12*power3(
     mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 144*msb12*power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power6(mst22)))/(24.*msb12*mst12*(
     mst12 - mst22)*mst22*power2(sb)) - (mu2*log(mst12/Q2)*power2(cb)*(-(mA2*
     msb12*mst12) + mA2*msb12*mst22 + msb12*mst12*mz2 - msb12*mst22*mz2 - 2*
     mst12*(5*mC2*mst12 - msb12*mst12 + mst14)*mst22*mt2*DeltaInv(msb12,mst12,
     mC2) - msb12*mst12*mst22*mst24*DeltaInv(mst22,mz2,mst12) + 2*mst12*mst14*
     mst22*mt2*DeltaInv(mw2,msb12,mst12) - 2*msb12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) + 10*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) + msb12*mst24*DeltaInv(mst22,mz2,mst12)*power2(mst22) - 5*
     msb12*mst12*mz2*DeltaInv(mst22,mz2,mst12)*power2(mst22) + msb12*DeltaInv(
     mst22,mz2,mst12)*power2(mst12)*power2(mst22) - 6*msb12*mst12*mst22*
     DeltaInv(mst22,mz2,mst12)*power2(mz2) + msb12*DeltaInv(mst22,mz2,mst12)*
     power2(mst12)*power2(mz2) + 5*msb12*DeltaInv(mst22,mz2,mst12)*power2(
     mst22)*power2(mz2) - 6*msb12*mst22*mst24*mz2*power2(mst12)*power2(
     DeltaInv(mst22,mz2,mst12)) + 6*msb12*mst12*mst24*mz2*power2(mst22)*power2(
     DeltaInv(mst22,mz2,mst12)) + 138*msb12*mst12*mst22*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - 138*msb12*mst24*power2(mst22)*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 36*msb12*power2(mst12)*power2(
     mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 12*msb12*mst12*
     mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 168*mst12*mst14*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*
     mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + msb12*(
     mst12 - mst22)*DeltaInv(mst22,mA2,mst12)*(mst22*(-(mst12*mst22) + mst24) -
     (mst12 - 5*mst22)*power2(mA2) + 5*mA2*power2(mst22) + power3(mA2)) + 12*
     mC2*mst12*mst22*mt2*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12
     - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) - msb12*
     mst12*DeltaInv(mst22,mz2,mst12)*power3(mst22) + 5*msb12*mz2*DeltaInv(
     mst22,mz2,mst12)*power3(mst22) - 36*msb12*mst12*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mst22) - 6*mA2*msb12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mA2,mst12))*(-(mst12*mst24) + mA2*(6*mst12*
     mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mA2) + power3(mA2) + power3(
     mst22)) - 12*mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - msb12*mst12*DeltaInv(mst22,mz2,mst12)*power3(mz2) + msb12*
     mst22*DeltaInv(mst22,mz2,mst12)*power3(mz2) - 6*msb12*mst22*power2(mst12)*
     power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 144*msb12*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 138*msb12*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mst22)*power3(mz2) + 144*mst12*mst22*
     mt2*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*
     mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*
     msb12*(mst12 - mst22)*mst22*power2(mA2)*((3*mst12*mst22 + 2*mst24)*power2(
     mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mA2*(5*
     mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) + 720*
     msb12*mst22*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)) - 720*msb12*mst12*mst24*power2(mst22)*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + 288*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 288*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 288*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*msb12*mst12*mz2*power2(
     DeltaInv(mst22,mz2,mst12))*power4(mst22) + 216*msb12*power2(mst12)*power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 144*msb12*mst12*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 144*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 6*msb12*
     mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 6*msb12*
     power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 144*msb12*
     mst12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 144*
     msb12*mst24*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) +
     216*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 216*msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 144*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mst12) - 6*msb12*mz2*power2(DeltaInv(mst22,mz2,mst12))*
     power5(mst22) - 360*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power5(mst22) - 144*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power5(mst22) - 144*msb12*mst12*power2(mst22)*power3(DeltaInv(
     mst22,mz2,mst12))*power5(mz2) + 144*msb12*power3(mst22)*power3(DeltaInv(
     mst22,mz2,mst12))*power5(mz2) + 144*msb12*power2(mz2)*power3(DeltaInv(
     mst22,mz2,mst12))*power6(mst22)))/(12.*msb12*mst12*(mst12 - mst22)*mst22)
     + (log(mst12/Q2)*power2(cb)*(-(mA2*msb12*mst12*mu2) + mA2*msb12*mst22*mu2
     + msb12*mst12*mu2*mz2 - msb12*mst22*mu2*mz2 + 2*mst12*(5*mC2*mst12 -
     msb12*mst12 + mst14)*mst22*mt2*(mst12 - mst22 - 2*mu2)*DeltaInv(msb12,
     mst12,mC2) - msb12*mst12*mst22*mst24*mu2*DeltaInv(mst22,mz2,mst12) + 4*
     mst12*mst14*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12) - 2*mst14*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) - 4*msb12*mst22*mt2*mu2*DeltaInv(
     mw2,msb12,mst12)*power2(mst12) + 20*mst22*mt2*mu2*mw2*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) + msb12*mst24*mu2*DeltaInv(mst22,mz2,mst12)*power2(
     mst22) - 5*msb12*mst12*mu2*mz2*DeltaInv(mst22,mz2,mst12)*power2(mst22) +
     2*mst12*mst14*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst22) + msb12*mu2*
     DeltaInv(mst22,mz2,mst12)*power2(mst12)*power2(mst22) - 2*msb12*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mst22) + 10*mt2*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12)*power2(mst22) - 6*msb12*mst12*
     mst22*mu2*DeltaInv(mst22,mz2,mst12)*power2(mz2) + msb12*mu2*DeltaInv(
     mst22,mz2,mst12)*power2(mst12)*power2(mz2) + 5*msb12*mu2*DeltaInv(mst22,
     mz2,mst12)*power2(mst22)*power2(mz2) - 6*msb12*mst22*mst24*mu2*mz2*power2(
     mst12)*power2(DeltaInv(mst22,mz2,mst12)) + 6*msb12*mst12*mst24*mu2*mz2*
     power2(mst22)*power2(DeltaInv(mst22,mz2,mst12)) + 138*msb12*mst12*mst22*
     mst24*mu2*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 138*msb12*mst24*
     mu2*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 36*
     msb12*mu2*power2(mst12)*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,
     mz2,mst12)) + 24*msb12*mst12*mst14*mst22*mt2*mu2*mw2*power2(DeltaInv(mw2,
     msb12,mst12)) - 12*msb12*mst14*mst22*mt2*mw2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12)) + 12*msb12*mst12*mst14*mt2*mw2*power2(mst22)*
     power2(DeltaInv(mw2,msb12,mst12)) + 336*mst12*mst14*mst22*mt2*mu2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 168*mst14*mst22*mt2*power2(mst12)
     *power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 72*msb12*mst22*mt2*mu2*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 168*mst12*
     mst14*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     36*msb12*mt2*power2(mst12)*power2(mst22)*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + msb12*(mst12 - mst22)*mu2*DeltaInv(mst22,mA2,mst12)*(
     mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mA2) + 5*mA2*
     power2(mst22) + power3(mA2)) + 2*msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)
     *power3(mst12) - 10*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power3(mst12)
     - 36*msb12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mst12) - 12*mC2*mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - msb12*mst12*mu2*DeltaInv(mst22,mz2,
     mst12)*power3(mst22) + 5*msb12*mu2*mz2*DeltaInv(mst22,mz2,mst12)*power3(
     mst22) - 36*msb12*mst12*mu2*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mst22) - 6*mA2*msb12*(mst12 - mst22)*mst22*mu2*power2(DeltaInv(
     mst22,mA2,mst12))*(-(mst12*mst24) + mA2*(6*mst12*mst22 + 23*mst24) - (
     mst12 - 23*mst22)*power2(mA2) + power3(mA2) + power3(mst22)) - 24*mst22*
     mt2*mu2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 12*
     mt2*power2(mst12)*power2(mst22)*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2) + 12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12)*
     power3(mw2) - msb12*mst12*mu2*DeltaInv(mst22,mz2,mst12)*power3(mz2) +
     msb12*mst22*mu2*DeltaInv(mst22,mz2,mst12)*power3(mz2) - 6*msb12*mst22*mu2*
     power2(mst12)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 144*msb12*
     mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) -
     138*msb12*mu2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22)*power3(mz2)
     - 144*mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*(mst12 - mst22)*
     mst22*mu2*power2(mA2)*((3*mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*
     power3(mA2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) + 720*msb12*mst22*mst24*
     mu2*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 720*
     msb12*mst12*mst24*mu2*power2(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)) + 576*msb12*mst12*mst14*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) - 288*msb12*mst14*mst22*mt2*power2(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 288*msb12*mst12*mst14*mt2*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 24*mst22*mt2*mu2*
     mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 12*mt2*mw2*power2(
     mst22)*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 576*msb12*mst22*
     mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 288*
     msb12*mt2*power2(mst22)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) + 576*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 288*mt2*power2(mst22)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 6*msb12*mst12*mu2*mz2*power2(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) + 216*msb12*mu2*power2(mst12)*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 144*msb12*mst12*mu2*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 288*mst12*
     mst14*mst22*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*
     mst14*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst12*mst14*mt2*power2(mst22)*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) + 6*msb12*mst12*mst22*mu2*power2(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 6*msb12*mu2*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 144*msb12*mst12*mst22*mst24*mu2*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 144*msb12*mst24*mu2*power2(mst22)*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mz2) + 216*msb12*mu2*power2(mst12)*power2(mst22)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 216*msb12*mst12*mu2*
     power3(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 12*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 288*msb12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 288*
     mst22*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)
     - 144*mt2*power2(mst22)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mst12) - 288*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power5(mst12) - 6*msb12*mu2*mz2*power2(DeltaInv(mst22,mz2,mst12))*
     power5(mst22) - 360*msb12*mst12*mu2*power2(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power5(mst22) - 144*msb12*mu2*power3(mz2)*power3(DeltaInv(mst22,
     mz2,mst12))*power5(mst22) - 144*msb12*mst12*mu2*power2(mst22)*power3(
     DeltaInv(mst22,mz2,mst12))*power5(mz2) + 144*msb12*mu2*power3(mst22)*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 144*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst12) + 144*msb12*mu2*power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power6(mst22)))/(24.*msb12*mst12*(
     mst12 - mst22)*mst22) + power2(xt)*((2*mt2*log(mC2/Q2)*log(mst12/Q2)*
     power2(cb)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))*(mC2*mst12 +
     msb12*mst12 + mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*
     msb12*mst14 + power3(mst12)))))/(msb12*(mst12 - mst22)) + (log(mC2/Q2)*
     log(mst12/Q2)*power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(msb12,
     mst12,mC2))*(mC2*mst12 + msb12*mst12 + mst14 + DeltaInv(msb12,mst12,mC2)*(
     (msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*
     power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))))/msb12 + ((mst12 -
     mst22 - 2*mt2)*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))*(-(mC2*mst22) - msb12*mst22 - mst24 + DeltaInv(
     mst22,msb12,mC2)*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2)
     + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)))))/
     (msb12*(-mst12 + mst22)) - (log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(
     mC2)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*mst22 + msb12*
     mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 + mst24)*power2(
     mC2) - mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*
     mst24 + power3(mst22)))))/msb12 - (log(mA2/Q2)*log(mst12/Q2)*power2(cb)*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*(3*mA2*mst22 + mst12*mst22 +
     mst24 + DeltaInv(mst22,mA2,mst12)*(5*(mst12*mst22 + mst24)*power2(mA2) -
     3*mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) - mA2*(-10*mst12*mst24
     + power3(mst22)))))/(2.*mst12) + (2*log(mst12/Q2)*log(mst22/Q2)*power2(
     mh2)*power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(mh2*mst22 + mst12*
     mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)*((mst12*mst22 - mst24)*power2(
     mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*mh2*(2*
     mst12*mst24 + power3(mst22)))))/mst12 + (log(mA2/Q2)*log(mst22/Q2)*power2(
     cb)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*(mA2*mst22 + mst12*mst22
     + mst24 + DeltaInv(mst22,mA2,mst12)*((mst12*mst22 + mst24)*power2(mA2) -
     mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) + mA2*(6*mst12*mst24 +
     power3(mst22)))))/(2.*mst12) + (log(mst12/Q2)*log(mst22/Q2)*power2(sa)*(-(
     (mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12))) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(mh2*mst22 +
     mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)*((mst12*mst22 - mst24)*
     power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*
     mh2*(2*mst12*mst24 + power3(mst22)))) + power2(mH2)*((-(mst12*mst22) +
     mst24)*power2(mH2) + mst22*power3(mH2) + (-5*mst12 + 3*mst22)*power3(
     mst22) - 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,
     mst12))))/(2.*mst12) + (2*log(mst12/Q2)*log(mst22/Q2)*power2(sa)*power2(
     snt)*((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-(mh2*
     mst22) - mst12*mst22 - 3*mst24 + DeltaInv(mst22,mh2,mst12)*((-(mst12*
     mst22) + mst24)*power2(mh2) + mst22*power3(mh2) + (-5*mst12 + 3*mst22)*
     power3(mst22) - 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(mH2)*((
     mst12*mst22 - mst24)*power2(mH2) - mst22*power3(mH2) + (5*mst12 - 3*mst22)
     *power3(mst22) + 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mH2,mst22,mst12))))/mst12 - (power2(sa)*power2(log(mst12/Q2))*(4*mh2*
     mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)) - 4*mH2*mst12*mst14*mt2*
     power2(DeltaInv(mH2,mst12,mst12)) + (mst12 - mst22)*((2*mH2*mst22 + mst12*
     mst22 + 2*mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(
     mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-2*mh2*mst22 - mst12*mst22 - 2*
     mst24 + DeltaInv(mst22,mh2,mst12)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)
     ) + 2*mst22*power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*
     mst12*mst24 + power3(mst22)))) + power2(mH2)*((3*mst12*mst22 + 2*mst24)*
     power2(mH2) - 2*mst22*power3(mH2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*
     mH2*(5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)))))/
     (2.*mst12*(mst12 - mst22)) + (log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(
     mC2)*power2(snt)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(
     msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 +
     msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*
     power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-
     10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2)
     + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/msb12 - (log(
     mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(-((
     3*mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 +
     DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24 +
     power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/msb12 + (log(mC2/Q2)*log(
     msb12/Q2)*power2(cb)*power2(mC2)*power2(snb)*(2*(3*mC2*mst12 + msb12*mst12
     + mst14)*mt2*power2(DeltaInv(msb12,mst12,mC2)) - (mst12 - mst22 - 2*mt2)*
     power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 - msb12*mst22 - mst24 +
     DeltaInv(mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*power2(mC2) + 3*mst22*
     power3(mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(-10*msb12*mst24 +
     power3(mst22)))) - 2*mt2*(-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*
     power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12 - mst22)
     ) + (log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*(-2*(3*mC2*mst12 +
     msb12*mst12 + mst14)*mt2*power2(DeltaInv(msb12,mst12,mC2)) + (mst12 -
     mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 - msb12*
     mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*
     power2(mC2) + 3*mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(
     -10*msb12*mst24 + power3(mst22)))) + 2*mt2*(-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*(mst12 - mst22)) + (2*mt2*log(mC2/Q2)*log(mst12/Q2)*power2(cb)*
     power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*
     power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(
     6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*msb22*(mst12 - mst22)) + (log(mC2/Q2)*log(mst12/Q2)*power2(cb)*
     power2(mC2)*power2(snb)*power2(snt)*(-(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22) + (log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*
     mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) -
     mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(
     -10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/
     msb22 + (log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-2*
     (3*mC2*mst12 + msb22*mst12 + mst14)*mt2*power2(DeltaInv(msb22,mst12,mC2))
     + (mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*mst22
     - msb22*mst22 - mst24 + DeltaInv(mst22,msb22,mC2)*(-5*(msb22*mst22 +
     mst24)*power2(mC2) + 3*mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22)
     + mC2*(-10*msb22*mst24 + power3(mst22)))) + 2*mt2*(-5*(msb22*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12)
     + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     ))/(msb22*(mst12 - mst22)) + (power2(sa)*power2(snt)*power2(log(mst22/Q2))
     *(2*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + mH2*mst12*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) - 2*power2(mH2)*(mst24*power2(mH2) + (-
     2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mh2*(-2*mh2*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 2*mh2*
     (mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))))/mst12 + (power2(sa)
     *power2(log(mst22/Q2))*((-mst12 + mst22)*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 4*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,
     mst22,mst22)) + (mst12 - mst22)*power2(mH2)*(mst24*power2(mH2) + (-2*mst12
     + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)) + mh2*(mh2*(mst12 - mst22)*mst24*power2(
     DeltaInv(mst22,mh2,mst12)) + 4*mst12*mst24*mt2*power2(DeltaInv(mst22,
     mst22,mh2)) - mh2*(mst12 - mst22)*(mst24*power2(mh2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)))))/(2.*mst12*(mst12 - mst22)) + ((mst12 - mst22 - 2*mt2)
     *log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-(msb22*(
     mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (
     log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) +
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (2*mt2*log(
     msb12/Q2)*log(mst12/Q2)*power2(cb)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*
     mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*
     power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12)
     + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb12*(
     mst12 - mst22)) + (log(msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(snt)*(-((
     mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) - mst14*
     power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(
     mw2) - 3*power4(mst12)))))/msb12 + (2*mt2*log(msb22/Q2)*log(mst12/Q2)*
     power2(cb)*power2(snb)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)
     *power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb22*(mst12 - mst22)) +
     (log(msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*power2(snt)*(-((mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(
     mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) +
     5*msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*
     power4(mst12)))))/msb22 + (power2(cb)*power2(snt)*power2(log(mst12/Q2))*(-
     (mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*(
     mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12))*(mst14 + DeltaInv(mw2,msb12,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/msb12 - (power2(snt)*power2(log(mst12/Q2))*(-(mh2*msb12*
     mst12*mst14*power2(DeltaInv(mh2,mst12,mst12))) - 2*msb12*(2*mh2*mst22 +
     mst12*mst22 + 2*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 2*
     msb12*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*
     power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12))*(mst14 + DeltaInv(mw2,msb12,mst12)*(-(
     mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(
     mst12)) - power4(mst12)))))/(msb12*mst12) - (2*mt2*log(msb12/Q2)*log(
     mst12/Q2)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*
     mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*
     mw2*power3(mst12) + msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(
     mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/(msb12*(mst12 - mst22)) +
     (2*mt2*log(msb12/Q2)*log(mst12/Q2)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*
     power4(mst12))))/(msb12*(mst12 - mst22)) - (log(msb12/Q2)*log(mst12/Q2)*
     power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*
     mw2*power3(mst12) + msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(
     mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/msb12 + (log(msb12/Q2)*
     log(mst12/Q2)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb12*(10*mst14*mw2 +
     mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12)
     )))/msb12 - (2*mt2*log(msb22/Q2)*log(mst12/Q2)*power2(mw2)*power2(snb)*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(
     mw2) - 3*power4(mst12))))/(msb22*(mst12 - mst22)) - (log(msb22/Q2)*log(
     mst12/Q2)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb22*(10*mst14*mw2 + mst12*
     power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/
     msb22 + (2*mt2*power2(mw2)*power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*
     power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12)
     )*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12))))/(msb12*
     msb22*(mst12 - mst22)) + (power2(mw2)*power2(snb)*power2(snt)*power2(log(
     mst12/Q2))*(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(
     mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) -
     power4(mst12))))/(msb12*msb22) + (2*mt2*log(mst12/Q2)*log(mw2/Q2)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(
     6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))))/(msb12*(mst12 - mst22)) - (2*mt2*log(mst12/Q2)*log(mw2/
     Q2)*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12
     + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))))/(msb12*(mst12 - mst22)) + (log(mst12/
     Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*
     (msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*
     power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/msb12 - (log(mst12/
     Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)
     *(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*
     power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/msb12
     - (2*mt2*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*(msb22*(msb12*
     mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb22*(
     6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*msb22*(mst12
     - mst22)) + (2*mt2*log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snb)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/
     (msb12*msb22*(mst12 - mst22)) - (log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snb)*power2(snt)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*
     (6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))))/(msb12*msb22) + (log(mst12/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*power2(snb)*power2(snt)*(msb22*(msb12*mst12 + mst14 + mst12*
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     mst14*power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(
     mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) +
     msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(
     mw2) - power4(mst12))))/(msb12*msb22) + (log(mst12/Q2)*power2(cb)*power2(
     snt)*((5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - 6*
     mC2*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 -
     msb12*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - DeltaInv(mw2,msb12,
     mst12)*(-(msb12*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb12,
     mst12)*(mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) + power3(
     mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14*power2(
     mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(
     mst12)))))/(12.*msb12) + (2*mt2*power2(cb)*power2(snb)*power2(log(mst12/
     Q2))*(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*
     mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(
     msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2))
     + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*
     mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))
     )/(msb12*msb22*(mst12 - mst22)) + (power2(cb)*power2(snb)*power2(snt)*
     power2(log(mst12/Q2))*(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) -
     2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(-(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(
     mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(msb12*msb22) - (mt2*log(mst12/Q2)*power2(snb)
     *(msb22*(-(msb12*mst12) + mst14 + 5*mst12*mw2)*DeltaInv(mw2,msb12,mst12) -
     6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(mst12*mw2*(-3*msb12 + mw2)
     - mst14*(msb12 + 14*mw2) + power3(mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*
     msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)) - msb12*DeltaInv(mw2,
     msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,
     msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) +
     power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*
     power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) +
     power4(mst12)))))/(6.*msb12*msb22*(mst12 - mst22)) - (log(mst12/Q2)*
     power2(snb)*power2(snt)*(msb22*(-(msb12*mst12) + mst14 + 5*mst12*mw2)*
     DeltaInv(mw2,msb12,mst12) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))*
     (mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) + power3(mst12)) -
     72*msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2)
     - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(
     mst12)) - msb12*DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*
     mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) -
     mst14*(msb22 + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(
     mst14*mw2 + power3(mst12)) + power4(mst12)))))/(12.*msb12*msb22) + (2*mt2*
     log(msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*((mC2*mst12 + msb12*
     mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + power2(
     mC2)*((msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb12 -
     3*mst12)*power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     *(-(msb12*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*
     power2(mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/(msb12*(
     mst12 - mst22)) + (log(msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*
     power2(snt)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb12 + (power2(snt)*power2(log(mst22/Q2))*(2*
     msb12*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 2*msb12*
     power2(mh2)*(mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*(
     mh2*msb12*mst24*power2(DeltaInv(mst22,mst22,mh2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(mst24 + DeltaInv(mst22,mw2,msb12)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) -
     power4(mst22))))))/(msb12*mst12) + (power2(cb)*power2(Pi)*power2(snt)*(-((
     3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + (3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     + power2(mC2)*((5*msb12*mst22 + 3*mst24)*power2(mC2) - 3*mst22*power3(mC2)
     + (5*msb12 - 3*mst22)*power3(mst22) + mC2*(14*msb12*mst24 + 3*power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-((msb12*mst22 +
     3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*
     (14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)))))/(6.*msb12) + (power2(cb)*power2(snt)*(mC2*msb12*
     DeltaInv(msb12,mst12,mC2) - mC2*msb12*DeltaInv(mst22,msb12,mC2) - 42*(3*
     mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) + 42*(3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 42*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     - 42*power2(mC2)*(-((5*msb12*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*
     power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - mC2*(14*msb12*mst24 +
     3*power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + mw2*(msb12*DeltaInv(
     mst22,mw2,msb12) - 42*mw2*(msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + DeltaInv(mw2,msb12,mst12)*(-msb12 + 42*mw2*(
     msb12*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb12,mst12) + 42*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*
     power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(
     mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) - 42*mw2*power3(DeltaInv(
     mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*
     mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*
     power3(mw2) - 3*power4(mst22)))))/(6.*msb12) + ((mst12 - mst22 - 2*mt2)*
     log(msb12/Q2)*log(mst22/Q2)*power2(cb)*(-((mC2*mst22 + msb12*mst22 + 3*
     mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(
     msb12*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*
     mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*
     mst24*mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(
     mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(
     msb12*(mst12 - mst22)) + ((mst12 - mst22 - 2*mt2)*log(msb12/Q2)*log(mst22/
     Q2)*power2(cb)*power2(snb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) +
     mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(
     mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 +
     3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 +
     msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*
     mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb12*(-mst12
     + mst22)) + (log(msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*power2(
     snt)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) +
     mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 +
     DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) -
     mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*
     power3(mw2) - 3*power4(mst22)))))/msb12 + ((mst12 - mst22 - 2*mt2)*log(
     msb22/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*(-((mC2*mst22 + msb22*mst22
     + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*((
     -(msb22*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*
     mst22)*power3(mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))
     *(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*
     mst24*mw2 + msb22*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(
     mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(
     msb22*(mst12 - mst22)) + (log(mst12/Q2)*log(mst22/Q2)*power2(cb)*(-((mA2*
     mst22 + mst12*mst22 + 3*mst24)*power2(mA2)*power2(DeltaInv(mst22,mA2,
     mst12))) + power2(mA2)*((-(mst12*mst22) + mst24)*power2(mA2) + mst22*
     power3(mA2) + (-5*mst12 + 3*mst22)*power3(mst22) - 5*mA2*(2*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) + power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + 3*mst24 + mst22*mz2 + DeltaInv(
     mst22,mz2,mst12)*(10*mst12*mst24*mz2 + mst12*mst22*power2(mz2) - mst24*
     power2(mz2) + 5*mst12*power3(mst22) + 5*mz2*power3(mst22) - mst22*power3(
     mz2) - 3*power4(mst22)))))/(2.*mst12) - (log(mst12/Q2)*log(mst22/Q2)*((
     mh2*mst22 + mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + power2(mh2)*((mst12*mst22 - mst24)*power2(mh2) - mst22*power3(
     mh2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + power2(mz2)*power2(DeltaInv(
     mst22,mz2,mst12))*(mst12*mst22 + 3*mst24 + mst22*mz2 + DeltaInv(mst22,mz2,
     mst12)*(10*mst12*mst24*mz2 + mst12*mst22*power2(mz2) - mst24*power2(mz2) +
     5*mst12*power3(mst22) + 5*mz2*power3(mst22) - mst22*power3(mz2) - 3*
     power4(mst22)))))/(2.*mst12) + (power2(cb)*power2(snt)*power2(log(msb12/
     Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*(-((3*msb12*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mC2)*((3*msb12*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-
     ((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)))
     + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb12 + (power2(cb)*power2(snb)*power2(snt)*power2(log(
     msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - (2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((3*msb12*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((
     msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) -
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb12 + (power2(cb)*power2(snb)*power2(log(msb12/Q2))*(
     2*(2*mC2*mst12 + msb12*mst12 + 2*mst14)*mt2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*(mst12 - mst22 -
     2*mt2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 2*mt2*power2(mC2)*(
     (3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 -
     2*mst12)*power3(mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + (-mst12 + mst22 + 2*mt2)*power2(mC2)*(-((3*
     msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-((mst12 - mst22 - 2*mt2)*(
     msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) -
     2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*
     mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - (mst12 - mst22 - 2*mt2)
     *power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22))
     + (power2(cb)*power2(log(msb12/Q2))*(-2*(2*mC2*mst12 + msb12*mst12 + 2*
     mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + (2*mC2*mst22 +
     msb12*mst22 + 2*mst24)*(-mst12 + mst22 + 2*mt2)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 2*mt2*power2(mC2)*(-((3*msb12*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + (mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + 2*(mst24
     + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,
     msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(
     mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*
     mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22)) + (power2(cb)*
     power2(snb)*power2(snt)*power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*
     mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*
     mst22 + msb22*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2)) + power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((3*msb22*
     mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*
     power3(mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + power2(mw2)*(-((msb22*mst22 + 2*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) - power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*
     msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*
     mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/msb22 + (
     power2(cb)*power2(snb)*power2(log(msb22/Q2))*(-2*(2*mC2*mst12 + msb22*
     mst12 + 2*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + (2*
     mC2*mst22 + msb22*mst22 + 2*mst24)*(-mst12 + mst22 + 2*mt2)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 2*mt2*power2(mC2)*(-((3*msb22*mst12 +
     2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + (mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*((mst12 - mst22 - 2*mt2)*(msb22*mst22 + 2*(mst24
     + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,
     msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(
     mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*
     mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/(msb22*(mst12 - mst22)) + (power2(cb)*
     power2(snb)*power2(snt)*power2(log(mst22/Q2))*(-(msb22*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2))) + msb12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + msb22*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - msb12*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(msb22*mst24*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*
     power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(mst22)))))/(msb12*
     msb22) + (power2(mw2)*power2(Pi)*power2(snb)*power2(snt)*(-(msb22*(msb12*
     mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + msb12*
     (msb22*mst22 + 3*mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     msb12*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*
     power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst12*mw2*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*msb22*mst12*power2(DeltaInv(mw2,msb22,mst12)) - 3*
     msb12*mst14*power2(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst12*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) + 14*msb12*msb22*mst14*mw2*power3(DeltaInv(mw2,
     msb12,mst12)) + 5*msb12*msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 3*msb22*mst14*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*
     msb12*msb22*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mw2*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst12*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 14*msb12*msb22*mst14*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12)) - 5*msb12*msb22*power3(mst12)*power3(DeltaInv(mw2,msb22,
     mst12)) - 3*msb12*mw2*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 3*
     msb12*mst12*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb12*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - msb22*power3(DeltaInv(mst22,mw2,msb12))*
     (14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)) + msb12*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*
     mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(
     mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22))))/(
     6.*msb12*msb22) + (power2(mw2)*power2(Pi)*power2(snb)*(msb22*(mst12 -
     mst22 - 2*mt2)*(msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*(mst12 - mst22 - 2*mt2)*(msb22*mst22 + 3*(mst24
     + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*(msb22*(msb12*
     mst12 + 3*(mst14 + mst12*mw2))*power2(DeltaInv(mw2,msb12,mst12)) - msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(
     mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) -
     3*power4(mst12))) + msb22*(mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,
     mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*
     power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*
     power3(mw2) - 3*power4(mst22)) - msb12*(mst12 - mst22 - 2*mt2)*power3(
     DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2)
     + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*
     mst22*power3(mw2) - 3*power4(mst22))))/(6.*msb12*msb22*(mst12 - mst22)) +
     ((mst12 - mst22 - 2*mt2)*log(msb12/Q2)*log(mst22/Q2)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*
     power4(mst22))))/(msb12*(-mst12 + mst22)) + ((mst12 - mst22 - 2*mt2)*log(
     msb12/Q2)*log(mst22/Q2)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,
     msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(-(
     mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*mst24*mw2 + mst22*
     power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/(
     msb12*(mst12 - mst22)) + (log(msb12/Q2)*log(mst22/Q2)*power2(mw2)*power2(
     snt)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2
     + DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) +
     msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(
     mw2) - 3*power4(mst22))))/msb12 - (log(msb12/Q2)*log(mst22/Q2)*power2(mw2)
     *power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 +
     3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*
     mw2*power3(mst22) + msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(
     mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/msb12 + ((mst12 - mst22 -
     2*mt2)*log(msb22/Q2)*log(mst22/Q2)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb22*(10*
     mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*
     power4(mst22))))/(msb22*(-mst12 + mst22)) + (log(msb22/Q2)*log(mst22/Q2)*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 5*mw2*power3(mst22) + msb22*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/msb22 + (
     power2(mw2)*power2(snt)*power2(log(msb12/Q2))*((msb12*mst22 + 2*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(
     10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 +
     3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/msb12 - (
     power2(mw2)*power2(snb)*power2(snt)*power2(log(msb12/Q2))*((msb12*mst22 +
     2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/msb12 - (power2(mw2)*power2(log(msb12/Q2))*((mst12 -
     mst22 - 2*mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*
     msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*
     mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb12*(mst12
     - mst22)) + (power2(mw2)*power2(snb)*power2(log(msb12/Q2))*((mst12 - mst22
     - 2*mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14
     + mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 -
     2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*
     mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*
     power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb12*(mst12 -
     mst22)) + (log(msb12/Q2)*power2(snt)*(-((mst12 - mst22)*(msb12 + mw2)) -
     mst22*DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(
     mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) +
     mst12*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(
     mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 6*
     mst12*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*
     mw2*(6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) +
     power3(mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*
     mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) +
     power3(mst22) + power3(mw2)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*
     mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)) - 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb12*mst12*mst22) + (log(
     msb12/Q2)*power2(snb)*power2(snt)*((mst12 - mst22)*(msb12 + mw2) + mst22*
     DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) - mst12*
     DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(mst22) +
     power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) - 6*mst12*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*
     msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(
     mw2)) + 6*mst12*mst22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2
     + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) + power3(
     mst22) + power3(mw2)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)) + 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb12*mst12*mst22) + (log(
     msb12/Q2)*power2(snb)*(-((mst12 - mst22)*(msb12*(mst12 - 2*mt2) - 2*mt2*
     mw2 + mst12*(mst22 + mw2))) + 2*mst22*mt2*DeltaInv(mw2,msb12,mst12)*(5*
     mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)) + mst12*(mst14 +
     5*power2(mw2)) + power3(mw2)) + mst12*(mst12 - mst22 - 2*mt2)*DeltaInv(
     mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(mst22) + power2(mw2)
     ) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) - 12*mst12*mst22*mt2*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*
     mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) - 6*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     (23*mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2))
     + power3(mst22) + power3(mw2)) - 144*mst12*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12)) - 72*mst12*mst22*(mst12 - mst22 - 2*
     mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 +
     3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb12*
     mst12*(mst12 - mst22)*mst22) + (log(msb12/Q2)*((mst12 - mst22)*(msb12*(
     mst12 - 2*mt2) - 2*mt2*mw2 + mst12*(mst22 + mw2)) - 2*mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)
     ) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) - mst12*(mst12 - mst22 -
     2*mt2)*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(
     mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 12*
     mst12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 +
     mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12)
     + power3(mw2)) + 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) -
     msb12*(mst24 + power2(mw2)) + power3(mst22) + power3(mw2)) + 144*mst12*
     mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) +
     72*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*
     power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22))))/(12.*msb12*mst12*(mst12 - mst22)*mst22) +
     (power2(mw2)*power2(snb)*power2(snt)*power2(log(msb22/Q2))*((msb22*mst22 +
     2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) - power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/msb22 - (power2(mw2)*power2(snb)*power2(log(msb22/Q2))*((
     mst12 - mst22 - 2*mt2)*(msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(10*
     msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(msb22*(mst12 - mst22)) + (log(msb22/Q2)*power2(snb)*
     power2(snt)*(-((mst12 - mst22)*(msb22 + mw2)) - mst22*DeltaInv(mw2,msb22,
     mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) + power2(mw2)) + mst12*
     (mst14 + 5*power2(mw2)) + power3(mw2)) + mst12*DeltaInv(mst22,mw2,msb22)*(
     5*mw2*power2(mst22) - msb22*(power2(mst22) + power2(mw2)) + mst22*(mst24 +
     5*power2(mw2)) + power3(mw2)) + 6*mst12*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14
     + power2(mw2)) + power3(mst12) + power3(mw2)) - 6*mst12*mst22*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*(23*mst24*mw2 + mst22*mw2*(6*msb22 + 23*mw2) -
     msb22*(mst24 + power2(mw2)) + power3(mst22) + power3(mw2)) + 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 +
     3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) +
     2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) - 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 +
     3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(12.*msb22*
     mst12*mst22) + (log(msb22/Q2)*power2(snb)*((mst12 - mst22)*(msb22*(mst12 -
     2*mt2) - 2*mt2*mw2 + mst12*(mst22 + mw2)) - 2*mst22*mt2*DeltaInv(mw2,
     msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) + power2(mw2)) +
     mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) - mst12*(mst12 - mst22 - 2*
     mt2)*DeltaInv(mst22,mw2,msb22)*(5*mw2*power2(mst22) - msb22*(power2(mst22)
     + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 12*mst12*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*
     (6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(
     mw2)) + 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*power2(DeltaInv(mst22,
     mw2,msb22))*(23*mst24*mw2 + mst22*mw2*(6*msb22 + 23*mw2) - msb22*(mst24 +
     power2(mw2)) + power3(mst22) + power3(mw2)) + 144*mst12*mst22*mt2*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) + 72*mst12*mst22*(
     mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(10*
     msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(12.*msb22*mst12*(mst12 - mst22)*mst22) + ((mst12 - mst22
     - 2*mt2)*power2(mw2)*power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*
     power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22)
     )*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*
     power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(mst22))))/(msb12*
     msb22*(mst12 - mst22)) + (log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     snt)*((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(5*
     mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb12 + (log(msb12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*
     power2(snt)*((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(
     5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb12 - (log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*((mst12 - mst22 - 2*mt2)*(
     msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*
     mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(
     mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) -
     power4(mst22))))/(msb12*(mst12 - mst22)) + (log(msb12/Q2)*log(mw2/Q2)*
     power2(cb)*power2(mw2)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*
     power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(
     mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*(
     mst12 - mst22)) + (log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*((
     mst12 - mst22 - 2*mt2)*(msb12*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)) -
     (log(msb12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*((mst12 -
     mst22 - 2*mt2)*(msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14
     + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*
     power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12))
     - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(
     DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22) +
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)) + (log(msb22/Q2)*
     log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*((msb22*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(mw2) - mw2*power3(mst22)
     + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/msb22 - (log(msb22/Q2)*log(mw2/Q2)*power2(
     mw2)*power2(snb)*((mst12 - mst22 - 2*mt2)*(msb22*mst22 + mst24 + 3*mst22*
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (
     mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(
     mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) +
     power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb22*(mst12 -
     mst22)) + (log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*((
     mst12 - mst22 - 2*mt2)*(msb22*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb22*(mst12 - mst22)) +
     ((mst12 - mst22 - 2*mt2)*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*
     mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(
     mst22))))/(msb12*(mst12 - mst22)) + ((mst12 - mst22 - 2*mt2)*log(mst22/Q2)
     *log(mw2/Q2)*power2(cb)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(
     msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*(-mst12 +
     mst22)) - (log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*
     mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(
     mst22))))/msb12 + (log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/msb12 + ((mst12 - mst22 - 2*mt2)*log(mst22/
     Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*(msb22*(msb12*mst22 + mst24 +
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(
     mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,
     msb22)*(mst24*power2(mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))) +
     msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/(msb12*msb22*(-mst12 + mst22)) + ((mst12 -
     mst22 - 2*mt2)*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(
     snb)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(
     mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(
     mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22*
     (mst12 - mst22)) + (log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power2(snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*
     power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) -
     mst22*power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*msb22) - (log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(
     snb)*power2(snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*
     power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) -
     mst22*power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*msb22) - (log(mst12/Q2)*log(mz2/Q2)*power2(mz2)*power2(DeltaInv(
     mst22,mz2,mst12))*(mst12*mst22 + mst24 + 3*mst22*mz2 + DeltaInv(mst22,mz2,
     mst12)*(5*mst24*power2(mz2) - mz2*power3(mst22) + mst12*(10*mst24*mz2 + 5*
     mst22*power2(mz2) + power3(mst22)) - 3*mst22*power3(mz2) - power4(mst22)))
     )/(2.*mst12) + (log(mst12/Q2)*log(mz2/Q2)*power2(cb)*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + mst24 + 3*mst22*mz2 + DeltaInv(
     mst22,mz2,mst12)*(5*mst24*power2(mz2) - mz2*power3(mst22) + mst12*(10*
     mst24*mz2 + 5*mst22*power2(mz2) + power3(mst22)) - 3*mst22*power3(mz2) -
     power4(mst22))))/(2.*mst12) + (log(mst22/Q2)*log(mz2/Q2)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + mst24 + mst22*mz2 +
     DeltaInv(mst22,mz2,mst12)*(mst24*power2(mz2) + mz2*power3(mst22) + mst12*(
     6*mst24*mz2 + mst22*power2(mz2) + power3(mst22)) - mst22*power3(mz2) -
     power4(mst22))))/(2.*mst12) - (log(mst22/Q2)*log(mz2/Q2)*power2(cb)*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + mst24 +
     mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(mst24*power2(mz2) + mz2*power3(
     mst22) + mst12*(6*mst24*mz2 + mst22*power2(mz2) + power3(mst22)) - mst22*
     power3(mz2) - power4(mst22))))/(2.*mst12) + (log(mst22/Q2)*power2(sa)*
     power2(snt)*(-((5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(mH2,mst22,
     mst12)) - mst12*(mH2 - 6*mst22)*DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*
     DeltaInv(mst22,mh2,mst12) - mst12*mst22*DeltaInv(mst22,mh2,mst12) + mst24*
     DeltaInv(mst22,mh2,mst12) + mh2*mst12*DeltaInv(mst22,mst22,mh2) - 6*mst12*
     mst22*DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*mst24*power2(DeltaInv(mst22,
     mh2,mst12)) + 18*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + 84*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 24*mh2*mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*power3(mh2) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mst22) - 72*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*
     power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 -
     mst12*mst24 + mst22*power2(mH2) + power3(mst22)) + 24*mst12*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 72*power2(mH2)*
     (mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 144*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 288*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) - 288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(
     6.*mst12) + (power2(mw2)*power2(snb)*power2(snt)*power2(log(mst22/Q2))*(-(
     msb22*mst24*power2(DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(
     mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2))
     + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(
     mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*
     mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22))))
     /(msb12*msb22) + (log(msb12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(
     snt)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,
     msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(-
     5*mst24*power2(mw2) + mw2*power3(mst22) - msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb12 + (log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*(-(
     (msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) +
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(-5*
     mst24*power2(mw2) + mw2*power3(mst22) - msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb12 + (log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*
     power2(snt)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(-
     5*mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb22 - (2*log(mH2/Q2)*log(mst22/Q2)*power2(sa)*power2(snt)*((mH2*mst22 +
     mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (
     mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(
     mst22))))/mst12 - (log(mH2/Q2)*log(mst22/Q2)*power2(sa)*((mst12 - mst22)*(
     mH2*mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + (mst12 - mst22)*power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) -
     mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 8*mst12*mt2*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(
     mH2,mst22,mst22)*power4(mst22))))/(2.*mst12*(-mst12 + mst22)) - (2*log(
     mh2/Q2)*log(mst22/Q2)*power2(snt)*((mh2*mst22 + mst12*mst22 + mst24)*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22
     + mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/mst12 + (2*log(mh2/Q2)*log(
     mst22/Q2)*power2(sa)*power2(snt)*((mh2*mst22 + mst12*mst22 + mst24)*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22
     + mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/mst12 - (log(mh2/Q2)*log(
     mst22/Q2)*((mst12 - mst22)*(mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + (mst12 - mst22)*power2(mh2)*((mst12*
     mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(
     mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,
     mst12)) + 8*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*
     power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/(2.*
     mst12*(-mst12 + mst22)) + (log(mh2/Q2)*log(mst22/Q2)*power2(sa)*((mst12 -
     mst22)*(mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + (mst12 - mst22)*power2(mh2)*((mst12*mst22 + mst24)*
     power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*
     mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + 8*mst12*
     mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/(2.*mst12*(-mst12 + mst22))
     + (power2(cb)*power2(snt)*power2(log(mst22/Q2))*(mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - power2(mC2)*(mst24*power2(mC2) + (-2*msb12 +
     mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(-mst24 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/msb12 + (
     log(mst22/Q2)*power2(cb)*power2(snt)*(-((5*mC2*mst22 - msb12*mst22 +
     mst24)*DeltaInv(mst22,msb12,mC2)) + 6*mC2*power2(DeltaInv(mst22,msb12,mC2)
     )*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) +
     power3(mst22)) + 72*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*
     power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + DeltaInv(mst22,mw2,msb12)*(-(msb12*mst22) + mst24 + 5*
     mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb12)*(mst22*mw2*(-3*msb12 + mw2) -
     mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(
     mst24*mw2 + power3(mst22)) + power4(mst22)))))/(12.*msb12) + ((mst12 -
     mst22 - 2*mt2)*power2(cb)*power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) - msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + msb12*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-(msb22*mst24*power2(
     DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*
     power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22)
     - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(msb12*msb22*(
     mst12 - mst22)) + ((mst12 - mst22 - 2*mt2)*log(mst22/Q2)*power2(snb)*(
     msb22*(-(msb12*mst22) + mst24 + 5*mst22*mw2)*DeltaInv(mst22,mw2,msb12) -
     6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(mst22*mw2*(-3*msb12 + mw2)
     - mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*
     msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)) - msb12*DeltaInv(mst22,
     mw2,msb22)*(-(msb22*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,
     mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2) - mst24*(msb22 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(12.*msb12*msb22*(-mst12 + mst22)) + (log(mst22/Q2)*power2(snb)
     *power2(snt)*(msb22*(-(msb12*mst22) + mst24 + 5*mst22*mw2)*DeltaInv(mst22,
     mw2,msb12) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(mst22*mw2*(-3*
     msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)) - msb12*
     DeltaInv(mst22,mw2,msb22)*(-(msb22*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*
     DeltaInv(mst22,mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2) - mst24*(msb22 + 14*
     mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(
     mst22)) + power4(mst22)))))/(12.*msb12*msb22) + (log(msb12/Q2)*log(mst22/
     Q2)*power2(cb)*power2(snt)*((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((msb12*mst22 -
     mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22)
     + 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))
     + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-(msb12*mst22) - 3*mst24
     - mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 5*mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) +
     mst22*power3(mw2) + 3*power4(mst22)))))/msb12 + (log(msb22/Q2)*log(mst22/
     Q2)*power2(cb)*power2(snb)*power2(snt)*((mC2*mst22 + msb22*mst22 + 3*
     mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((
     msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)
     *power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-(
     msb22*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*
     power2(mw2) - 5*mw2*power3(mst22) - msb22*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/msb22 +
     (log(mst22/Q2)*power2(snt)*(-2*msb12*(5*mh2*mst22 - mst12*mst22 + mst24)*
     DeltaInv(mst22,mh2,mst12) + 12*mh2*msb12*power2(DeltaInv(mst22,mh2,mst12))
     *(-3*mh2*mst12*mst22 - 14*mh2*mst24 - mst12*mst24 + mst22*power2(mh2) +
     power3(mst22)) + 144*msb12*power2(mh2)*(mst24*power2(mh2) + (-2*mst12 +
     mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12)) + mst12*(-2*msb12*(mh2 - 6*mst22)*DeltaInv(
     mst22,mst22,mh2) + 48*msb12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 +
     3*power3(mst22)) + 576*mh2*msb12*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) - DeltaInv(mst22,mw2,msb12)*(-(msb12*mst22) + mst24 + 5*mst22*mw2 -
     6*mw2*DeltaInv(mst22,mw2,msb12)*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12
     + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 +
     power3(mst22)) + power4(mst22))))))/(12.*msb12*mst12) + (log(mst22/Q2)*
     power2(cb)*power2(snb)*power2(snt)*(msb22*(5*mC2*mst22 - msb12*mst22 +
     mst24)*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*mst22 - msb22*mst22 +
     mst24)*DeltaInv(mst22,msb22,mC2) + msb12*msb22*mst22*DeltaInv(mst22,mw2,
     msb12) - msb22*mst24*DeltaInv(mst22,mw2,msb12) - 5*msb22*mst22*mw2*
     DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*DeltaInv(mst22,mw2,msb22) +
     msb12*mst24*DeltaInv(mst22,mw2,msb22) + 5*msb12*mst22*mw2*DeltaInv(mst22,
     mw2,msb22) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) -
     18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*
     msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*msb22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 18*msb12*msb22*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 84*msb12*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mst22) - 6*msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mst22) - 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*
     msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) + 6*mC2*msb12*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*
     mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) +
     6*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 72*msb22*power2(mC2)
     *(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 72*msb12*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) - 144*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) -
     144*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) +
     144*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 144*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 72*
     msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 72*
     msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 72*
     msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 72*msb12*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(12.*msb12*msb22) +
     ((mst12 - mst22 - 2*mt2)*log(mst22/Q2)*power2(cb)*power2(snb)*(-(msb22*(5*
     mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + msb12*(5*
     mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) - msb12*msb22*
     mst22*DeltaInv(mst22,mw2,msb12) + msb22*mst24*DeltaInv(mst22,mw2,msb12) +
     5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*mst22*DeltaInv(
     mst22,mw2,msb22) - msb12*mst24*DeltaInv(mst22,mw2,msb22) - 5*msb12*mst22*
     mw2*DeltaInv(mst22,mw2,msb22) + 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) + 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 84*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) - 18*
     msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mw2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*msb12*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*msb22*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) - 6*mC2*msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)
     *power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2)))/(12.*msb12*msb22*(mst12 - mst22)) - (mw2*power2(snb)
     *power2(snt)*(-(msb12*msb22*DeltaInv(mst22,mw2,msb12)) + msb12*msb22*
     DeltaInv(mst22,mw2,msb22) + msb12*msb22*DeltaInv(mw2,msb12,mst12) - msb12*
     msb22*DeltaInv(mw2,msb22,mst12) + 42*msb22*mw2*(msb12*mst22 + 3*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*mw2*(msb22*mst22
     + 3*mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*
     msb22*mst12*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst14*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*mst12*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) + 126*msb12*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) +
     126*msb12*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 588*msb12*
     msb22*mst14*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 210*msb12*
     msb22*mw2*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 210*msb12*
     msb22*mst12*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 588*msb12*msb22*
     mst14*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*mw2*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 126*msb12*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*mst12*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mw2*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 126*msb12*mw2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 42*msb22*mw2*power3(DeltaInv(mst22,mw2,msb12))*(
     14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)) - 42*msb12*mw2*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*
     mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*
     power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(
     mst22)) + 126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) -
     126*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(6.*msb12*
     msb22) + (mt2*log(mst12/Q2)*power2(cb)*power2(snb)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 -
     msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*
     DeltaInv(mw2,msb12,mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*
     msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22)) + (log(mst12/Q2)*
     power2(cb)*power2(snb)*power2(snt)*(-(msb22*(5*mC2*mst12 - msb12*mst12 +
     mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*DeltaInv(mw2,msb12,
     mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) -
     msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mw2*DeltaInv(mw2,
     msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) +
     18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst14*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) + 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) -
     6*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*msb22*power2(mC2)
     *(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) -
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*
     mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(12.*msb12*msb22) +
     (log(mst22/Q2)*(-(((-5*mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst22,
     mh2,mst12))/mst12) - (8*(mh2 - 6*mst22)*mt2*DeltaInv(mst22,mst22,mh2))/(-
     mst12 + mst22) - 2*mst22*DeltaInv(mst22,mw2,msb12) + (2*mst24*DeltaInv(
     mst22,mw2,msb12))/msb12 + (4*mst22*mt2*DeltaInv(mst22,mw2,msb12))/(mst12 -
     mst22) - (4*mst24*mt2*DeltaInv(mst22,mw2,msb12))/(msb12*mst12 - msb12*
     mst22) + (10*mst22*mw2*DeltaInv(mst22,mw2,msb12))/msb12 - (20*mst22*mt2*
     mw2*DeltaInv(mst22,mw2,msb12))/(msb12*mst12 - msb12*mst22) - mst22*
     DeltaInv(mst22,mz2,mst12) + (mst24*DeltaInv(mst22,mz2,mst12))/mst12 + (5*
     mst22*mz2*DeltaInv(mst22,mz2,mst12))/mst12 + 12*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) - (24*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)))/(
     mst12 - mst22) + 36*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     (168*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/msb12 - (72*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) -
     (336*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(msb12*mst12
     - msb12*mst22) + 6*mst24*mz2*power2(DeltaInv(mst22,mz2,mst12)) + 18*mst22*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + (84*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)))/mst12 - (12*mw2*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mst22))/msb12 + (24*mt2*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mst22))/(msb12*mst12 - msb12*mst22) - (6*mz2*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mst22))/mst12 - (6*mh2*power2(DeltaInv(
     mst22,mh2,mst12))*(-3*mh2*mst12*mst22 - 14*mh2*mst24 - mst12*mst24 +
     mst22*power2(mh2) + power3(mst22)))/mst12 - (192*mt2*power2(DeltaInv(
     mst22,mst22,mh2))*(mh2*mst24 + 3*power3(mst22)))/(mst12 - mst22) - (12*
     mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/msb12 - (24*mst22*
     mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/(msb12*(-mst12 + mst22)
     ) - (6*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/mst12 - (72*
     power2(mh2)*(mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))/mst12 +
     288*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - (576*
     mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 -
     mst22) + 288*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - (576*
     mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) +
     (288*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 -
     (576*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(
     msb12*mst12 - msb12*mst22) + 144*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) + 144*mst24*power3(mz2)*power3(DeltaInv(mst22,
     mz2,mst12)) + (144*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)))/mst12 - (2304*mh2*mt2*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22))/(mst12 - mst22) - (144*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22))/msb12 + (288*mt2*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22))/(msb12*mst12 - msb12*mst22) - (72*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 - (144*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/msb12 + (288*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(msb12*mst12 - msb12*mst22)
     - (72*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12))/24. + (
     power2(log(mst22/Q2))*(-((mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)))/mst12) - (4*mh2*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)))/(
     mst12 - mst22) - (2*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/
     msb12 + (4*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(
     msb12*mst12 - msb12*mst22) - (mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)))/mst12 + (power2(mh2)*(mst24*power2(mh2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)))/mst12 - 4*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12)) + (8*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12)))/(mst12 - mst22) - 4*mst24*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + (8*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)))/(mst12 - mst22) - (4*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)))/msb12 + (8*mt2*power3(mst22)*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)))/(msb12*mst12 - msb12*mst22) - 2*power2(mz2)*
     power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 2*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - (2*power3(mst22)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)))/mst12 + (2*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22))/msb12 - (4*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mst22))/(msb12*mst12 - msb12*mst22) + (power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 + (2*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/msb12 - (4*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(msb12*mst12 - msb12*mst22)
     + (mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12))/2. - (2*
     log(mst12/Q2)*log(mst22/Q2)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*
     (mh2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)*((mst12*
     mst22 - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)*
     power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22))))*power4(snt))/mst12
     - (mh2*power2(log(mst12/Q2))*(mst12*mst14*power2(DeltaInv(mh2,mst12,mst12)
     ) + 2*mh2*power2(DeltaInv(mst22,mh2,mst12))*(2*mh2*mst22 + mst12*mst22 +
     2*mst24 + DeltaInv(mst22,mh2,mst12)*((3*mst12*mst22 + 2*mst24)*power2(mh2)
     - 2*mst22*power3(mh2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mh2*(5*
     mst12*mst24 + power3(mst22)))))*power4(snt))/mst12 + (2*log(mst12/Q2)*log(
     mst22/Q2)*power2(sa)*(-((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12))) + power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12))*(mh2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)
     *((mst12*mst22 - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*
     mst22)*power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(
     mH2)*((-(mst12*mst22) + mst24)*power2(mH2) + mst22*power3(mH2) + (-5*mst12
     + 3*mst22)*power3(mst22) - 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)))*power4(snt))/mst12 - (mh2*power2(log(mst22/Q2)
     )*(2*mh2*mst24*power2(DeltaInv(mst22,mh2,mst12)) + mst12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) - 2*mh2*(mst24*power2(mh2) + (-2*mst12 + mst22)
     *power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)))*power4(snt))/mst12 + (power2(sa)*power2(log(mst22/Q2))*
     (-2*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) + 2*power2(mH2)*(mst24*power2(mH2) + (-
     2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mh2*(2*mh2*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) + mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 2*mh2*
     (mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12))))*power4(snt))/mst12 -
     (log(mst22/Q2)*power2(sa)*(-((5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(
     mH2,mst22,mst12)) - mst12*(mH2 - 6*mst22)*DeltaInv(mH2,mst22,mst22) + 5*
     mh2*mst22*DeltaInv(mst22,mh2,mst12) - mst12*mst22*DeltaInv(mst22,mh2,
     mst12) + mst24*DeltaInv(mst22,mh2,mst12) + mh2*mst12*DeltaInv(mst22,mst22,
     mh2) - 6*mst12*mst22*DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*mst24*power2(
     DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 84*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     - 24*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 6*mh2*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mst22) - 72*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 6*mH2*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*
     mH2*mst24 - mst12*mst24 + mst22*power2(mH2) + power3(mst22)) + 24*mst12*
     power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 72*
     power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*
     mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 144*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 288*mH2*mst12*power3(DeltaInv(
     mH2,mst22,mst22))*power4(mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) - 288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2)
     )*power4(mst22))*power4(snt))/(6.*mst12) + (2*log(mH2/Q2)*log(mst22/Q2)*
     power2(sa)*((mH2*mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) + power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) -
     mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(DeltaInv(
     mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,
     mst22,mst22)*power4(mst22)))*power4(snt))/mst12 + (2*log(mh2/Q2)*log(
     mst22/Q2)*((mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) -
     mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(DeltaInv(
     mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,
     mst22,mh2)*power4(mst22)))*power4(snt))/mst12 - (2*log(mh2/Q2)*log(mst22/
     Q2)*power2(sa)*((mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22 + mst24)*power2(
     mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22)))*power4(snt))/mst12 - (log(mst22/Q2)*(-((
     5*mh2*mst22 - mst12*mst22 + mst24)*DeltaInv(mst22,mh2,mst12)) + 6*mh2*
     power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst12*mst22 - 14*mh2*mst24 -
     mst12*mst24 + mst22*power2(mh2) + power3(mst22)) + 72*power2(mh2)*(mst24*
     power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*DeltaInv(mst22,
     mst22,mh2)*(-mh2 + 6*mst22 + 24*DeltaInv(mst22,mst22,mh2)*(mh2*mst24 + 3*
     power3(mst22)) + 288*mh2*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22)))
     *power4(snt))/(6.*mst12) + (power2(sa)*power2(snt)*power2(log(mst12/Q2))*(
     -(mh2*mst12*mst14*power2(DeltaInv(mh2,mst12,mst12))) + mH2*mst12*mst14*
     power2(DeltaInv(mH2,mst12,mst12)) + 2*mst12*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 4*mst24*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) - 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) - 4*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 4*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 4*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) + 20*mst12*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) + 6*mst12*power2(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) + 4*power3(mH2)*power3(mst22)*power3(DeltaInv(
     mH2,mst22,mst12)) - 20*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 4*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) -
     6*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 4*mst24*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst12*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*mst24*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) - 4*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mst22) + 4*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 4*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 4*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power5(mH2)))/mst12 + (power2(sa)*
     power2(log(mst12/Q2))*power4(snt)*(mh2*mst12*mst14*power2(DeltaInv(mh2,
     mst12,mst12)) - mH2*mst12*mst14*power2(DeltaInv(mH2,mst12,mst12)) - 2*
     mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 4*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 2*mst12*mst22*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + 4*mst24*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 4*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2)
     - 4*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 20*mst12*mst24*
     power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) - 6*mst12*power2(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 4*power3(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 20*mst12*mst24*power3(mh2)*
     power3(DeltaInv(mst22,mh2,mst12)) + 6*mst12*power2(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) + 4*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 6*mst12*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) + 4*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) - 6*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) - 4*
     mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 4*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) - 4*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) + 4*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2)))/mst12 - (2*log(mh2/Q2)*log(mst12/Q2)*power2(snt)*(power2(
     mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24
     + DeltaInv(mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*
     mst22*power3(mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24
     + power3(mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 +
     4*power4(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12))
     )/mst12 + (2*log(mh2/Q2)*log(mst12/Q2)*power2(sa)*power2(snt)*(power2(mh2)
     *power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 +
     DeltaInv(mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*
     power3(mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 +
     power3(mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*
     power4(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/
     mst12 + (2*log(mh2/Q2)*log(mst12/Q2)*power4(snt)*(power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(
     mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(
     mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(
     mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(
     mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/mst12 -
     (2*log(mh2/Q2)*log(mst12/Q2)*power2(sa)*power4(snt)*(power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(
     mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(
     mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(
     mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(
     mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/mst12 -
     (power2(snt)*power2(log(mh2/Q2))*(-2*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 2*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,
     mst12)) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(
     mst12)) + mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(
     mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12)))/mst12 + (power2(sa)*power2(snt)
     *power2(log(mh2/Q2))*(-2*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mh2) + 2*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12)))/mst12 + (power2(log(mh2/Q2))*power4(snt)*(-
     2*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/mst12 - (power2(sa)*power2(log(mh2/Q2))*power4(snt)*(-2*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*power3(mh2)*(-2*mst12*
     mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*
     power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mst22,mst22,
     mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*
     power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/
     mst12 + (power2(Pi)*power4(snt)*(-2*(3*mh2*mst22 + mst12*mst22 + 3*mst24)*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) + 2*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*(-14*mh2*mst12*mst24 - 5*mst12*mst22*power2(
     mh2) - 3*mst24*power2(mh2) + 3*mst22*power3(mh2) - 3*mh2*power3(mst22) -
     5*mst12*power3(mst22) + 3*power4(mst22)) + mst12*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 6*power3(mst22) + 24*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22)) + 24*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/(6.*mst12) + (power4(snt)*(-(mh2*mst12*DeltaInv(mh2,mst12,mst12))
     + 2*mh2*mst12*DeltaInv(mst22,mh2,mst12) - 84*(3*mh2*mst22 + mst12*mst22 +
     3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*power2(mh2)*(-
     ((5*mst12*mst22 + 3*mst24)*power2(mh2)) + 3*mst22*power3(mh2) + (-5*mst12
     + 3*mst22)*power3(mst22) - mh2*(14*mst12*mst24 + 3*power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12)) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst12*mst14 + 6*power4(mst12)) + mst12*DeltaInv(mst22,mst22,mh2)*(-mh2 +
     42*DeltaInv(mst22,mst22,mh2)*(mh2*mst24 + 6*power3(mst22)) + 1008*mh2*
     power2(DeltaInv(mst22,mst22,mh2))*power4(mst22)) + 1008*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(6.*mst12) - (power2(Pi)*
     power2(snt)*(-2*msb12*(3*mh2*mst22 + mst12*mst22 + 3*mst24)*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + msb12*mst12*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 6*power3(mst12)) + 2*msb12*power2(mh2)*(-((5*mst12*
     mst22 + 3*mst24)*power2(mh2)) + 3*mst22*power3(mh2) + (-5*mst12 + 3*mst22)
     *power3(mst22) - mh2*(14*mst12*mst24 + 3*power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)) + mst12*(msb12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*
     mst24 + 6*power3(mst22)) + power2(mw2)*(-((msb12*mst22 + 3*(mst24 + mst22*
     mw2))*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(
     14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*
     msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*
     power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 +
     5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) +
     3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22))) + 24*mh2*
     msb12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)) + 24*mh2*msb12*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(6.*msb12*mst12) - (log(
     mst12/Q2)*power4(snt)*(-mh2 - mst12 - mst22 + mst12*(-mh2 + 6*mst12)*
     mst22*DeltaInv(mh2,mst12,mst12) + DeltaInv(mst22,mh2,mst12)*(mst22*(-(
     mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(
     mst22) + power3(mh2)) + 24*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 3*power3(mst12)) - 6*mh2*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 + 23*mst24) - (mst12 - 23*
     mst22)*power2(mh2) + power3(mh2) + power3(mst22)) + 72*mst22*power2(mh2)*(
     -((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*power3(mh2) + (-3*mst12
     + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12)) + 288*mh2*mst22*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12)))/(6.*mst12*mst22) - (power2(log(mh2/Q2))*((mst12 -
     mst22)*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 4*mst12*mt2*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12)) - (mst12 -
     mst22)*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*
     power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) - 4*mst12*
     mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 64*mh2*mt2*power3(DeltaInv(
     mh2,mst12,mst12))*power5(mst12)))/(2.*mst12*(mst12 - mst22)) + (power2(sa)
     *power2(log(mh2/Q2))*((mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 4*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst14 + 4*power3(mst12)) - (mst12 - mst22)*power3(mh2)*(-2*mst12*mst24 -
     2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*power3(
     DeltaInv(mst22,mh2,mst12)) - 4*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))
     *(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(
     mst22)) + 64*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(2.
     *mst12*(mst12 - mst22)) - (log(mh2/Q2)*log(mst12/Q2)*(8*mst12*mt2*power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12)) + (mst12 - mst22)
     *power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(3*mh2*mst22 + mst12*mst22
     + mst24 + DeltaInv(mst22,mh2,mst12)*(5*(mst12*mst22 + mst24)*power2(mh2) -
     3*mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) - mh2*(-10*mst12*mst24
     + power3(mst22)))) + 128*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/(2.*mst12*(mst12 - mst22)) + (log(mh2/Q2)*log(mst12/Q2)*power2(
     sa)*(8*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(
     mst12)) + (mst12 - mst22)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(
     3*mh2*mst22 + mst12*mst22 + mst24 + DeltaInv(mst22,mh2,mst12)*(5*(mst12*
     mst22 + mst24)*power2(mh2) - 3*mst22*power3(mh2) + (mst12 - mst22)*power3(
     mst22) - mh2*(-10*mst12*mst24 + power3(mst22)))) + 128*mh2*mt2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(2.*mst12*(mst12 - mst22)) + (
     power2(sa)*power2(snt)*(-(mh2*mst12*DeltaInv(mh2,mst12,mst12)) + mH2*
     mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*DeltaInv(mH2,mst22,mst12) +
     mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*mst12*DeltaInv(mst22,mh2,
     mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) + 84*mst12*mst22*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) -
     84*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 252*mst24*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 42*mh2*mst12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) - 252*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 252*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) -
     252*mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 252*mst12*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 1176*mst12*mst24*power3(
     mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 420*mst12*power2(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 252*power3(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) - 1176*mst12*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) - 420*mst12*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 252*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) +
     252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 42*power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - 42*
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) -
     252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 1008*
     mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 252*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(mH2,
     mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))
     /(6.*mst12) - (power2(sa)*power4(snt)*(-(mh2*mst12*DeltaInv(mh2,mst12,
     mst12)) + mH2*mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*DeltaInv(mH2,
     mst22,mst12) + mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*mst12*DeltaInv(
     mst22,mh2,mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) + 84*mst12*mst22*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(DeltaInv(
     mH2,mst22,mst22)) - 84*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 252*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 42*
     mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) + 252*mst22*power2(DeltaInv(mH2,
     mst22,mst12))*power3(mH2) - 252*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 252*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22)
     + 1176*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 420*
     mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 252*
     power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 1176*mst12*
     mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*power2(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 252*power3(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mH2) + 252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) +
     42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) -
     42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) -
     252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 1008*
     mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 252*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(mH2,
     mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))
     /(6.*mst12) + (power2(Pi)*power2(sa)*power2(snt)*(2*mst12*mst22*power2(
     mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 6*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,
     mst22)) - 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*
     mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + mh2*mst12*mst24*
     power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(
     mH2) - 6*mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 6*mst12*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 28*mst12*mst24*power3(
     mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*power2(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) - 10*mst12*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 6*power3(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 10*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) - 6*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 10*
     mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mst24*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mst22) - 24*mH2*mst12*power3(DeltaInv(
     mH2,mst22,mst22))*power4(mst22) + 6*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*
     power4(mst22) + 6*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 6*
     mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*mH2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(6.*mst12) - (power2(Pi)*power2(sa)*power4(
     snt)*(2*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 6*
     mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) - 2*mst12*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 6*mst24*power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12)) + mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 6*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 6*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 6*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) +
     28*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*
     power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(
     mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 6*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*mst22*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) - 6*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 10*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2)
     + 6*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 24*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + 6*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) + 6*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 6*mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*
     mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*mst12) - (2*log(mH2/Q2)*
     log(mst12/Q2)*power2(sa)*power2(snt)*(power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12))*(-3*mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(mH2,mst22,
     mst12)*(-5*(mst12*mst22 + mst24)*power2(mH2) + 3*mst22*power3(mH2) + (-
     mst12 + mst22)*power3(mst22) + mH2*(-10*mst12*mst24 + power3(mst22)))) +
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) +
     16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/mst12 + (2*log(
     mH2/Q2)*log(mst12/Q2)*power2(sa)*power4(snt)*(power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(mH2,
     mst22,mst12)*(-5*(mst12*mst22 + mst24)*power2(mH2) + 3*mst22*power3(mH2) +
     (-mst12 + mst22)*power3(mst22) + mH2*(-10*mst12*mst24 + power3(mst22)))) +
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) +
     16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/mst12 - (power2(
     sa)*power2(snt)*power2(log(mH2/Q2))*(-2*mst22*power2(DeltaInv(mH2,mst22,
     mst12))*power3(mH2) + 2*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(
     mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(
     mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 16*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/mst12 + (power2(sa)*power2(log(
     mH2/Q2))*power4(snt)*(-2*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(
     mH2) + 2*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) +
     mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) +
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) +
     mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*
     mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 16*mH2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/mst12 - (power2(sa)*power2(log(mH2/Q2))*((
     mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 4*
     mst12*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12))
     - (mst12 - mst22)*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) - 4*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*
     power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 64*mH2*
     mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(2.*mst12*(mst12 -
     mst22)) - (power2(Pi)*power2(sa)*(-4*mh2*mst12*mst14*mt2*power2(DeltaInv(
     mh2,mst12,mst12)) + 4*mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,mst12,mst12)
     ) + mst12*(mst12 - mst22)*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + 3*(mst12 - mst22)*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) - 4*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) - mst12*
     (mst12 - mst22)*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 3*(
     mst12 - mst22)*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 4*
     mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) - 3*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 3*(mst12 - mst22)*
     mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 24*mst12*mt2*power2(
     DeltaInv(mH2,mst22,mst22))*power3(mst22) + 24*mst12*mt2*power2(DeltaInv(
     mst22,mst22,mh2))*power3(mst22) + 14*mst12*(mst12 - mst22)*mst24*power3(
     mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 5*mst12*(mst12 - mst22)*power2(
     mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 3*(mst12 - mst22)*
     power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 14*mst12*(
     mst12 - mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 5*
     mst12*(mst12 - mst22)*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 3*(mst12 - mst22)*power3(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 5*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) - 3*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 5*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mH2,mst22,mst12))*power4(mH2) + 3*(mst12 - mst22)*mst24*power3(DeltaInv(
     mH2,mst22,mst12))*power4(mH2) - 24*mt2*power2(DeltaInv(mh2,mst12,mst12))*
     power4(mst12) + 24*mt2*power2(DeltaInv(mH2,mst12,mst12))*power4(mst12) -
     3*(mst12 - mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) - 96*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22)
     + 3*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 96*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)
     + 3*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) -
     3*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) -
     96*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 96*mH2*mt2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(12.*mst12*(mst12 -
     mst22)) - (log(mH2/Q2)*log(mst12/Q2)*power2(sa)*(8*mst12*mt2*power2(
     DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12)) + (mst12 - mst22)
     *power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(3*mH2*mst22 + mst12*mst22
     + mst24 + DeltaInv(mH2,mst22,mst12)*(5*(mst12*mst22 + mst24)*power2(mH2) -
     3*mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) - mH2*(-10*mst12*mst24
     + power3(mst22)))) + 128*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/(2.*mst12*(mst12 - mst22)) - (power2(sa)*(4*mh2*mst12*mt2*
     DeltaInv(mh2,mst12,mst12) - 4*mH2*mst12*mt2*DeltaInv(mH2,mst12,mst12) -
     mH2*mst12*(mst12 - mst22)*DeltaInv(mH2,mst22,mst12) + 4*mH2*mst12*mt2*
     DeltaInv(mH2,mst22,mst22) + mh2*mst12*(mst12 - mst22)*DeltaInv(mst22,mh2,
     mst12) - 4*mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) - 168*mh2*mst12*mst14*
     mt2*power2(DeltaInv(mh2,mst12,mst12)) + 168*mH2*mst12*mst14*mt2*power2(
     DeltaInv(mH2,mst12,mst12)) + 42*mst12*(mst12 - mst22)*mst22*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*mst24*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) - 168*mH2*mst12*mst24*mt2*power2(
     DeltaInv(mH2,mst22,mst22)) - 42*mst12*(mst12 - mst22)*mst22*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) - 126*(mst12 - mst22)*mst24*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + 168*mh2*mst12*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) - 126*(mst12 - mst22)*mst22*power2(DeltaInv(
     mst22,mh2,mst12))*power3(mh2) + 126*(mst12 - mst22)*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 1008*mst12*mt2*power2(DeltaInv(mH2,mst22,
     mst22))*power3(mst22) + 1008*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*
     power3(mst22) + 588*mst12*(mst12 - mst22)*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) + 210*mst12*(mst12 - mst22)*power2(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*power3(mH2)
     *power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 588*mst12*(mst12 -
     mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 210*mst12*(
     mst12 - mst22)*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12))
     - 126*(mst12 - mst22)*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12)
     )*power4(mh2) - 126*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mh2,mst12)
     )*power4(mh2) + 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 126*(mst12 - mst22)*mst24*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) - 1008*mt2*power2(DeltaInv(mh2,mst12,mst12))*power4(
     mst12) + 1008*mt2*power2(DeltaInv(mH2,mst12,mst12))*power4(mst12) - 126*(
     mst12 - mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22)
     - 4032*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) +
     126*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 4032*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) + 126*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 126*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2) - 4032*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)
     + 4032*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(12.*
     mst12*(mst12 - mst22)) + (log(mst12/Q2)*power2(sa)*(8*(mh2 - 6*mst12)*
     mst12*mst22*mt2*DeltaInv(mh2,mst12,mst12) - 8*(mH2 - 6*mst12)*mst12*mst22*
     mt2*DeltaInv(mH2,mst12,mst12) - 192*mst12*mst22*mt2*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) + 192*mst12*mst22*mt2*power2(
     DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(mst12)) + (mst12 - mst22)
     *(-mh2 + mH2 + DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) -
     (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)) -
     DeltaInv(mH2,mst22,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*
     mst22)*power2(mH2) + 5*mH2*power2(mst22) + power3(mH2)) - 6*mh2*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 +
     23*mst24) - (mst12 - 23*mst22)*power2(mh2) + power3(mh2) + power3(mst22))
     + 6*mH2*mst22*power2(DeltaInv(mH2,mst22,mst12))*(-(mst12*mst24) + mH2*(6*
     mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mH2) + power3(mH2) +
     power3(mst22)) - 72*mst22*power2(mH2)*(-((3*mst12*mst22 + 2*mst24)*power2(
     mH2)) + 2*mst22*power3(mH2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mH2*(
     5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 72*
     mst22*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*
     power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12))) - 2304*mh2*mst22*mt2*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 2304*mH2*mst22*mt2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(24.*mst12*(mst12 -
     mst22)*mst22) + (log(mst12/Q2)*power2(snt)*(-2*mh2*msb12 - 2*msb12*mst12 -
     2*msb12*mst22 - 2*msb12*(mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,
     mst12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) - msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) + 6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 18*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) + 2*msb12*DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*
     mst22) + mst24) - (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(mst22) +
     power3(mh2)) + 48*msb12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 3*power3(mst12)) - 12*mh2*msb12*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 + 23*mst24) - (mst12 -
     23*mst22)*power2(mh2) + power3(mh2) + power3(mst22)) - 6*mst22*power2(
     mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 144*msb12*mst22*
     power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*power3(
     mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + 144*msb12*mst12*mst14*mst22*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*msb12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*mst12*mst14*mst22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 576*mh2*msb12*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 72*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)))/(12.*msb12*mst12*mst22)
     + (log(mH2/Q2)*power2(sa)*power2(snt)*(mH2 + mst12 + mst22 + (mH2 - 6*
     mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*DeltaInv(
     mH2,mst22,mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,mst12)*
     power2(mH2) - 6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*mst22*
     power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 + power2(mH2)
     )*power2(DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*mst12*
     mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*
     power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,mst22,
     mst22))*power4(mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,mst12))*
     power5(mst12) + 576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power5(
     mst22)))/(6.*mst12*mst22) - (log(mH2/Q2)*power2(sa)*power4(snt)*(mH2 +
     mst12 + mst22 + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) +
     mH2*mst12*mst22*DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 + 5*mst22)*
     DeltaInv(mH2,mst22,mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,mst22,mst22)*
     power2(mst22) + 6*mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*
     mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 12*mH2*
     mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*
     mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*
     power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) + 144*
     mst12*power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 576*mH2*mst22*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + 576*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(6.*mst12*mst22) - (log(mH2/Q2)
     *power2(sa)*(mH2*mst12 - mH2*mst22 + 8*mst12*mt2 - 8*mst22*mt2 - 8*(mH2 -
     6*mst12)*mst12*mst22*mt2*DeltaInv(mH2,mst12,mst12) + 8*mH2*mst12*mst22*
     mt2*DeltaInv(mH2,mst22,mst22) + (-mH2 + mst12 - 5*mst22)*(mst12 - mst22)*
     DeltaInv(mH2,mst22,mst12)*power2(mH2) - 48*mst12*mt2*DeltaInv(mH2,mst22,
     mst22)*power2(mst22) + 6*(mst12 - mst22)*mst22*power2(mH2)*(-3*mst12*mst22
     - mH2*(mst12 + 14*mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,
     mst12)) + 96*mH2*mst12*mst22*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) -
     96*mst12*mst22*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*
     power3(mst12)) + 72*(mst12 - mst22)*mst22*power3(mH2)*(-2*mst12*mst24 - 2*
     mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*power3(
     DeltaInv(mH2,mst22,mst12)) + 1152*mst12*mt2*power2(DeltaInv(mH2,mst22,
     mst22))*power4(mst22) - 4608*mH2*mst22*mt2*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12) + 4608*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,
     mst22))*power5(mst22)))/(24.*mst12*(mst12 - mst22)*mst22) + (log(mst22/Q2)
     *power2(sa)*((mst12 - mst22)*(5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(
     mH2,mst22,mst12) + 8*mst12*(mH2 - 6*mst22)*mt2*DeltaInv(mH2,mst22,mst22) -
     5*mh2*mst12*mst22*DeltaInv(mst22,mh2,mst12) - mst12*mst24*DeltaInv(mst22,
     mh2,mst12) + mst22*mst24*DeltaInv(mst22,mh2,mst12) - 8*mh2*mst12*mt2*
     DeltaInv(mst22,mst22,mh2) + 48*mst12*mst22*mt2*DeltaInv(mst22,mst22,mh2) +
     mst22*DeltaInv(mst22,mh2,mst12)*power2(mst12) + 5*mh2*DeltaInv(mst22,mh2,
     mst12)*power2(mst22) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) + 6*
     mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mh2,mst12)) - 84*mst12*mst24*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*mst22*mst24*power2(mh2)
     *power2(DeltaInv(mst22,mh2,mst12)) - 6*mh2*mst24*power2(mst12)*power2(
     DeltaInv(mst22,mh2,mst12)) - 18*mst22*power2(mh2)*power2(mst12)*power2(
     DeltaInv(mst22,mh2,mst12)) + 18*mst12*power2(mh2)*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12)) + 192*mh2*mst12*mst24*mt2*power2(DeltaInv(
     mst22,mst22,mh2)) + 6*mst12*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) - 6*power2(mst22)*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mh2) + 6*mh2*mst12*power2(DeltaInv(mst22,mh2,mst12))*power3(mst22) + 576*
     mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) - 6*mH2*(mst12 -
     mst22)*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*
     mst24 - mst12*mst24 + mst22*power2(mH2) + power3(mst22)) - 192*mst12*mt2*
     power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) - 72*(
     mst12 - mst22)*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,
     mst12)) + 144*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 144*mst24*power2(mst12)*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 144*power2(mh2)*power2(mst12)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 144*mst12*power3(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) + 72*mst12*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) - 72*mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2)
     - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(mst22) - 2304*mH2*mst12*
     mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 216*mst12*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*power3(mh2)*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 2304*mh2*mst12*mt2*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) - 72*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(24.*mst12*(mst12 - mst22)) - (
     log(mst12/Q2)*power2(sa)*power2(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*mst12)*
     mst22*DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(
     mH2,mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) + mst22*mst24*
     DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mh2) +
     5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*
     mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,mst12)*power3(
     mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) -
     24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(
     mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*
     mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(6.*mst12*mst22) + (log(mst12/
     Q2)*power2(sa)*power4(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*mst12)*mst22*
     DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,
     mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) + mst22*mst24*
     DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mh2) +
     5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*
     mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,mst12)*power3(
     mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) -
     24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(
     mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*
     mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(6.*mst12*mst22) + (log(mh2/Q2)
     *power2(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(
     mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(mst22,mst22,mh2) - (mh2 -
     mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*mst12*DeltaInv(
     mst22,mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-3*mst12*mst22 -
     mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,
     mst12)) + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 12*
     mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(
     mst12)) + 72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,
     mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 576*
     mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 576*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(6.*mst12*mst22) - (log(
     mh2/Q2)*power2(sa)*power2(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*mst12)*
     mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(mst22,
     mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2)
     - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-
     3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(
     DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(
     mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(
     mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) +
     576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(6.*mst12*
     mst22) - (log(mh2/Q2)*power4(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*mst12)*
     mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(mst22,
     mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2)
     - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-
     3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(
     DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(
     mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(
     mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) +
     576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(6.*mst12*
     mst22) + (log(mh2/Q2)*power2(sa)*power4(snt)*(mh2 + mst12 + mst22 + (mh2 -
     6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(
     mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*
     power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*
     power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2)
     )*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*
     mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*
     power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,
     mh2))*power4(mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) + 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(
     mst22)))/(6.*mst12*mst22) - (log(mh2/Q2)*(mh2*mst12 - mh2*mst22 + 8*mst12*
     mt2 - 8*mst22*mt2 - 8*(mh2 - 6*mst12)*mst12*mst22*mt2*DeltaInv(mh2,mst12,
     mst12) + 8*mh2*mst12*mst22*mt2*DeltaInv(mst22,mst22,mh2) + (-mh2 + mst12 -
     5*mst22)*(mst12 - mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 48*mst12*
     mt2*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*(mst12 - mst22)*mst22*
     power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2)
     )*power2(DeltaInv(mst22,mh2,mst12)) + 96*mh2*mst12*mst22*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) - 96*mst12*mst22*mt2*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*(mst12 - mst22)*mst22*power3(
     mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) + 1152*mst12*mt2*power2(
     DeltaInv(mst22,mst22,mh2))*power4(mst22) - 4608*mh2*mst22*mt2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) + 4608*mh2*mst12*mt2*power3(
     DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(24.*mst12*(mst12 - mst22)*
     mst22) + (log(mh2/Q2)*power2(sa)*(mh2*mst12 - mh2*mst22 + 8*mst12*mt2 - 8*
     mst22*mt2 - 8*(mh2 - 6*mst12)*mst12*mst22*mt2*DeltaInv(mh2,mst12,mst12) +
     8*mh2*mst12*mst22*mt2*DeltaInv(mst22,mst22,mh2) + (-mh2 + mst12 - 5*mst22)
     *(mst12 - mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 48*mst12*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*(mst12 - mst22)*mst22*power2(
     mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*
     power2(DeltaInv(mst22,mh2,mst12)) + 96*mh2*mst12*mst22*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) - 96*mst12*mst22*mt2*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*(mst12 - mst22)*mst22*power3(
     mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) + 1152*mst12*mt2*power2(
     DeltaInv(mst22,mst22,mh2))*power4(mst22) - 4608*mh2*mst22*mt2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) + 4608*mh2*mst12*mt2*power3(
     DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(24.*mst12*(mst12 - mst22)*
     mst22) + (log(mst22/Q2)*power2(cb)*(msb12*(mst12 - mst22)*(5*mA2*mst22 -
     mst12*mst22 + mst24)*DeltaInv(mst22,mA2,mst12) + 2*mst12*(5*mC2*mst22 -
     msb12*mst22 + mst24)*(mst12 - mst22 - 2*mt2)*DeltaInv(mst22,msb12,mC2) +
     2*mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) - 4*msb12*mst12*mst22*mt2*
     DeltaInv(mst22,mw2,msb12) + 4*mst12*mst24*mt2*DeltaInv(mst22,mw2,msb12) +
     20*mst12*mst22*mt2*mw2*DeltaInv(mst22,mw2,msb12) - msb12*mst12*mst24*
     DeltaInv(mst22,mz2,mst12) + msb12*mst22*mst24*DeltaInv(mst22,mz2,mst12) -
     5*msb12*mst12*mst22*mz2*DeltaInv(mst22,mz2,mst12) + 2*msb12*mst22*
     DeltaInv(mst22,mw2,msb12)*power2(mst12) - 2*mst24*DeltaInv(mst22,mw2,
     msb12)*power2(mst12) - 10*mst22*mw2*DeltaInv(mst22,mw2,msb12)*power2(
     mst12) + msb12*mst22*DeltaInv(mst22,mz2,mst12)*power2(mst12) - 2*msb12*
     mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 10*mst12*mw2*DeltaInv(
     mst22,mw2,msb12)*power2(mst22) - msb12*mst12*DeltaInv(mst22,mz2,mst12)*
     power2(mst22) + 5*msb12*mz2*DeltaInv(mst22,mz2,mst12)*power2(mst22) + 12*
     msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 24*msb12*
     mst12*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 12*msb12*mst24*
     mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) + 168*mst12*mst22*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 72*msb12*mst12*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 336*mst12*mst24*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 168*mst24*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*mst12*
     power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst12*mst22*mst24*mz2*power2(DeltaInv(mst22,mz2,mst12)) - 6*msb12*mst24*
     mz2*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12)) - 84*msb12*mst12*
     mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 84*msb12*mst22*
     mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 18*msb12*mst22*
     power2(mst12)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 18*msb12*
     mst12*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 24*
     mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 12*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*msb12*
     mst12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22) - 6*mA2*msb12*(
     mst12 - mst22)*power2(DeltaInv(mst22,mA2,mst12))*(-3*mA2*mst12*mst22 - 14*
     mA2*mst24 - mst12*mst24 + mst22*power2(mA2) + power3(mst22)) - 12*mC2*
     mst12*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*
     msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) - 24*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     + 12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) -
     12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 6*msb12*
     power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 72*msb12*(
     mst12 - mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,
     mst12)) - 144*mst12*(mst12 - mst22 - 2*mt2)*power2(mC2)*(mst24*power2(mC2)
     + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 576*msb12*mst12*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 288*msb12*power2(mst12)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 288*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 576*
     msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 288*
     msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     576*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 288*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 144*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) + 144*msb12*mst12*mst22*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst24*power2(mst12)*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst12*power3(mst22)*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 12*mst12*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*msb12*mz2*power2(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) + 288*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 288*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*power2(mst12)*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 288*mst12*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 144*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 144*
     mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 288*
     mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst24*
     power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 72*msb12*
     mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 72*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(24.*msb12*
     mst12*(mst12 - mst22)) + (power2(cb)*power2(log(mst22/Q2))*(msb12*(-mst12
     + mst22)*mst24*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 2*mst12*
     mst24*(-mst12 + mst22 + 2*mt2)*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) - 2*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - 4*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*
     mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*
     mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst22*
     mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*(mst12 -
     mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(mst22) -
     2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) +
     2*mst12*(mst12 - mst22 - 2*mt2)*power2(mC2)*(mst24*power2(mC2) + (-2*msb12
     + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) - 8*msb12*mst12*mt2*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) + 4*msb12*power2(mst12)*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 8*msb12*mst12*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 4*msb12*mst24*power2(
     mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 8*mst12*mt2*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 4*power2(mst12)*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 2*msb12*
     power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12))
     - 2*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     + 2*msb12*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)
     ) + 2*msb12*mst12*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)) - 4*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 4*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 2*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 4*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 3*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mst22) - 2*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mst22) + 2*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 4*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 2*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(
     mz2) + msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) +
     2*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) +
     msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(2.*
     msb12*mst12*(mst12 - mst22)) - (power2(snt)*(-(msb12*mst12) + msb12*mst22
     - 2*mh2*msb12*mst12*mst22*DeltaInv(mh2,mst12,mst12) + 4*mh2*msb12*mst12*
     mst22*DeltaInv(mst22,mh2,mst12) - 2*mh2*msb12*mst12*mst22*DeltaInv(mst22,
     mst22,mh2) + 2*msb12*mst12*mst22*mw2*DeltaInv(mst22,mw2,msb12) - 2*msb12*
     mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) - 168*msb12*mst22*(3*mh2*mst22 +
     mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*
     mh2*msb12*mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*mst12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*msb12*
     mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 252*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     84*msb12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*
     power3(mst12)) - 252*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) + 252*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 1176*msb12*mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 1176*msb12*mst12*mst14*mst22*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 420*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 252*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 504*msb12*mst12*power2(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) - 420*msb12*mst12*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mst22) - 252*mst12*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mst22) + 168*msb12*mst22*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*(-14*mh2*mst12*mst24 - 5*mst12*mst22*power2(
     mh2) - 3*mst24*power2(mh2) + 3*mst22*power3(mh2) - 3*mh2*power3(mst22) -
     5*mst12*power3(mst22) + 3*power4(mst22)) - 252*mst12*mst22*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 420*msb12*mst12*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 252*mst12*mst14*mst22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 420*msb12*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2016*mh2*msb12*
     mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 252*mst22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 2016*mh2*msb12*
     mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22) + 252*mst12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 252*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 252*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(12.*msb12*mst12*
     mst22) + (log(msb12/Q2)*power2(cb)*power2(snb)*power2(snt)*(mC2*mst12 -
     mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*mst24*DeltaInv(mst22,mw2,
     msb12) - mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) + msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)
     + 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*msb12*mst12*mst22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*
     mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,
     msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (
     msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*
     msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*
     DeltaInv(mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 216*msb12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(12.*msb12*mst12*mst22) + (log(msb12/Q2)*power2(cb)*power2(snt)*(-(
     mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) -
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22)
     + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*
     msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 36*
     msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) -
     mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2)
     + mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 216*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(12.*msb12*mst12*mst22) + (power2(cb)*power2(snb)*power2(snt)*(-(
     mC2*msb12*msb22*DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(
     msb22,mst12,mC2) + mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*
     msb22*DeltaInv(mst22,msb22,mC2) - msb12*msb22*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*msb22*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mw2*
     DeltaInv(mw2,msb12,mst12) - msb12*msb22*mw2*DeltaInv(mw2,msb22,mst12) +
     42*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*msb12*msb22*
     mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*msb12*msb22*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 42*msb12*msb22*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 126*msb12*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 42*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 126*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 42*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) + 126*msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     126*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 126*msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb22*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 126*msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb22*mst12*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) + 126*msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 42*msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     - 588*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     210*msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 126*msb22*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     + 588*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     210*msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)
     ) + 126*msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2))
     + 210*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) + 588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + 126*msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 210*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,
     mw2,msb22)) - 588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 126*msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 210*msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)) - 588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 126*msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 210*msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(
     mw2,msb22,mst12)) + 588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 126*msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 210*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 126*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) + 210*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2)
     + 126*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*
     msb12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 -
     5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*
     mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) + (
     power2(cb)*power2(Pi)*power2(snb)*power2(snt)*(msb22*(3*mC2*mst12 + msb12*
     mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*
     mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) - 3*msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - msb12*msb22*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 3*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) + msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 3*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 3*msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 3*msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) + 3*msb22*mst22*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) - 3*msb12*mst22*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mw2) - 3*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 3*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2)
     + msb22*power2(mC2)*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*
     power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*msb12*msb22*mst24*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 14*msb12*msb22*mst24*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*msb22*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 5*msb12*msb22*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 14*msb12*
     msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 3*msb12*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 5*msb12*
     msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 14*
     msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*
     msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 14*
     msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb12*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 3*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + msb12*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(
     mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) -
     5*msb22*power3(mst12) + 3*power4(mst12)) + 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) - 3*msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 3*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*
     msb12*mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb22*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 3*msb12*mst22*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) + 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 3*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2))
     )/(6.*msb12*msb22) + (power2(cb)*power2(snb)*(-2*mC2*msb12*msb22*mt2*
     DeltaInv(msb12,mst12,mC2) + 2*mC2*msb12*msb22*mt2*DeltaInv(msb22,mst12,
     mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*
     mst22*DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*msb22*mt2*DeltaInv(mst22,
     msb12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,mC2) - mC2*msb12*
     msb22*mst22*DeltaInv(mst22,msb22,mC2) - 2*mC2*msb12*msb22*mt2*DeltaInv(
     mst22,msb22,mC2) + msb12*msb22*mst12*mw2*DeltaInv(mst22,mw2,msb12) -
     msb12*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) - 2*msb12*msb22*mt2*mw2*
     DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst12*mw2*DeltaInv(mst22,mw2,
     msb22) + msb12*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb22) + 2*msb12*msb22*
     mt2*mw2*DeltaInv(mst22,mw2,msb22) + 2*msb12*msb22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12) - 2*msb12*msb22*mt2*mw2*DeltaInv(mw2,msb22,mst12) + 84*msb22*
     (3*mC2*mst12 + msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)) - 84*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mt2*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 42*msb12*msb22*mst12*
     mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 126*msb22*mst12*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst22*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb12*msb22*
     mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 252*msb22*mst24*
     mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 42*msb12*msb22*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 42*msb12*msb22*
     mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 126*msb12*
     mst12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*
     mst22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*
     msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 252*msb12*
     mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*
     power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 42*msb12*
     msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 126*
     msb22*mst12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*
     msb22*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 84*
     msb12*msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 252*
     msb22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*msb12*
     msb22*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     126*msb12*mst12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*
     msb12*msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 252*
     msb12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*
     msb22*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*
     msb12*msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 252*
     msb22*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*
     msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 252*msb12*
     mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 252*msb22*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 252*msb12*mst22*mt2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 126*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 252*msb22*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 252*msb12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 252*msb22*mst12*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 252*msb12*mst12*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 84*msb22*mt2*power2(mC2)*(
     -((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 588*msb12*msb22*mst12*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 588*msb12*msb22*mst22*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 1176*msb12*msb22*mst24*mt2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 210*msb12*msb22*mst12*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 420*msb12*
     msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     126*msb22*mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 252*msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) - 588*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 588*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 1176*msb12*msb22*mst24*mt2*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 210*msb12*msb22*mst12*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 420*msb12*msb22*mt2*power2(mC2)
     *power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 126*msb12*mst12*power3(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 252*msb12*mt2*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 210*msb12*
     msb22*mst12*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     420*msb12*msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) - 588*msb12*msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 588*msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 1176*msb12*msb22*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 126*msb22*mst12*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 252*msb22*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 210*msb12*msb22*mst12*power2(mw2)
     *power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 420*msb12*msb22*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 588*msb12*
     msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 588*
     msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     1176*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) +
     126*msb12*mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)
     ) - 252*msb12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 420*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(
     mw2,msb12,mst12)) - 1176*msb12*msb22*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 252*msb22*mt2*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 420*msb12*msb22*mt2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 1176*msb12*msb22*mst14*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 252*msb12*mt2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*
     mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 126*msb22*
     mst12*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*msb22*
     mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 420*msb12*
     msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 252*msb22*
     mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 210*msb12*msb22*
     power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 210*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 126*
     msb12*mst12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*
     msb12*mst22*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 420*
     msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 252*
     msb12*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 210*msb12*
     msb22*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 252*
     msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     252*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     + 84*msb12*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) - 210*msb12*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mst22) - 126*msb22*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) + 252*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 126*msb22*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 210*msb12*msb22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mst22) + 126*msb12*mst12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 252*msb12*mt2*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 126*msb12*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 210*msb12*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb22*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*msb22*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*
     msb22*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 210*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) -
     126*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mst22) + 252*msb12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 126*msb12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 126*msb22*mst12*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 126*msb22*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 420*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 252*msb22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) + 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) + 126*msb12*mst12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 126*msb12*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 420*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 252*msb12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 420*msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 252*msb22*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 420*msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mw2) + 252*msb12*mst14*mt2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) - 126*msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 252*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 126*msb22*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 126*msb12*mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 252*msb12*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 126*msb12*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) + 126*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mst22) - 126*msb12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mst22) - 126*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mst22) + 126*msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mst22) + 126*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 252*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 126*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 126*msb12*mst12*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 252*msb12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 126*msb12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 252*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 252*msb12*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mw2)))/(6.*msb12*msb22*(mst12 - mst22)) + (power2(cb)*power2(Pi)*
     power2(snb)*(2*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*mt2*power2(mC2)
     *power2(DeltaInv(msb12,mst12,mC2)) - 2*msb12*(3*mC2*mst12 + msb22*mst12 +
     3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb12*msb22*
     mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst12*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst22*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb22*mst24*mt2*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst12*mst22*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst12*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + msb12*msb22*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst12*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb22*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst12*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 2*msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 6*msb12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 2*msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 6*msb22*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 6*msb12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst12*mst22*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 6*msb22*mst22*mt2*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 3*msb22*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)
     )*power3(mC2) - 3*msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 6*msb12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2) + 3*msb12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 3*msb22*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*
     msb22*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb22*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb12*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb12*mst22*
     mt2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mt2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 2*msb22*mt2*power2(mC2)*((5*
     msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*
     mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 14*msb12*msb22*mst12*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst22*mst24*power3(mC2)
     *power3(DeltaInv(mst22,msb12,mC2)) - 28*msb12*msb22*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*mst12*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 10*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*
     mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 6*
     msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) -
     14*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     14*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     28*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     5*msb12*msb22*mst12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) + 10*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) - 3*msb12*mst12*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*mt2*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*mst12*power2(mw2)*power3(mst22)
     *power3(DeltaInv(mst22,mw2,msb12)) + 10*msb12*msb22*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst12*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*
     mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 28*msb12*
     msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*
     mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 6*
     msb22*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*
     msb12*msb22*mst12*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22)) - 10*msb12*msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb22)) + 14*msb12*msb22*mst12*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) - 14*msb12*msb22*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 28*msb12*msb22*mst24*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 6*msb12*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 10*msb12*msb22*mt2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 28*msb12*msb22*mst14*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*msb22*mt2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 10*msb12*msb22*mt2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 28*msb12*
     msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*msb12*
     mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*
     mst12*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst22*
     mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 10*msb12*msb22*
     mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 6*msb22*mst24*
     mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*
     mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*mst12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst22*mst24*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 10*msb12*msb22*mst22*mt2*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 6*msb12*mst24*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 5*msb12*msb22*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 6*msb22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*msb12*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 2*msb12*mt2*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*
     power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(
     mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) - 5*msb12*msb22*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb22*mst12*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 6*msb22*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb22*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 5*msb12*
     msb22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*
     msb12*mst12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) -
     6*msb12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) +
     3*msb12*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 5*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     3*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     - 6*msb22*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     + 3*msb22*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 5*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) -
     3*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22)
     + 6*msb12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22)
     - 3*msb12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*
     msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*
     msb22*mst12*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*
     mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 10*msb12*
     msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*msb22*
     mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 10*msb12*msb22*
     mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*msb12*mst24*
     mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 10*msb12*msb22*
     mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 6*msb22*mst14*
     mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 10*msb12*msb22*mst12*
     mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 6*msb12*mst14*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst12*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 6*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 6*msb12*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mst22) - 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mst22) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 3*msb22*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 6*msb22*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 6*msb12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 6*msb22*mst12*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 6*msb12*mst12*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22*(mst12 - mst22))
     + (power2(snb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     2*msb12*msb22*(-mst12 + mst22 + 2*mt2)*mw2*DeltaInv(mst22,mw2,msb12) + 2*
     msb12*msb22*(mst12 - mst22 - 2*mt2)*mw2*DeltaInv(mst22,mw2,msb22) - 4*
     msb12*msb22*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 4*msb12*msb22*mt2*mw2*
     DeltaInv(mw2,msb22,mst12) + 84*msb22*(mst12 - mst22 - 2*mt2)*(msb12*mst22
     + 3*(mst24 + mst22*mw2))*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     84*msb12*(mst12 - mst22 - 2*mt2)*(msb22*mst22 + 3*(mst24 + mst22*mw2))*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 168*msb12*msb22*mst12*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 504*msb22*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 168*msb12*msb22*mst12*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 504*msb12*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 504*msb22*mst12*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 504*msb12*mst12*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 840*msb12*msb22*mt2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 2352*msb12*
     msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 504*msb22*
     mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 840*
     msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)
     ) - 2352*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 504*msb12*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 504*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) + 504*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 84*msb22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) - 84*msb12*(mst12 - mst22
     - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2
     + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22)
     + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) + 840*
     msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 504*
     msb22*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 840*msb12*
     msb22*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 504*msb12*
     mst14*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 504*msb22*mst12*
     mt2*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 504*msb12*mst12*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(12.*msb12*msb22*(mst12 -
     mst22)) + (log(msb22/Q2)*power2(cb)*power2(snb)*power2(snt)*(-(mC2*mst12)
     + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*DeltaInv(mst22,
     mw2,msb22) + mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) - msb22*mst22*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mw2*DeltaInv(mw2,msb22,
     mst12)*power2(mst12) + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22) + msb22*mst12*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*mst12*mst22*DeltaInv(mst22,mw2,
     msb22)*power2(mw2) - msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) +
     5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 138*mst12*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 36*msb22*mst12*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst12*
     mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 138*mst12*mst14*mst22*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 36*msb22*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2) + mst22*
     DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) - 216*msb22*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) +
     216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(12.*msb22*mst12*mst22) + (log(mst12/Q2)*(2/mst12 + 2/mst22 + mh2/(
     mst12*mst22) + mz2/(mst12*mst22) - (8*(mh2 - 6*mst12)*mt2*DeltaInv(mh2,
     mst12,mst12))/(mst12 - mst22) + mst22*DeltaInv(mst22,mz2,mst12) - (mst24*
     DeltaInv(mst22,mz2,mst12))/mst12 - (5*mst22*mz2*DeltaInv(mst22,mz2,mst12))
     /mst12 - (4*mst12*mt2*DeltaInv(mw2,msb12,mst12))/(mst12 - mst22) + (4*
     mst14*mt2*DeltaInv(mw2,msb12,mst12))/(msb12*mst12 - msb12*mst22) + (20*
     mst12*mt2*mw2*DeltaInv(mw2,msb12,mst12))/(msb12*mst12 - msb12*mst22) - (5*
     DeltaInv(mst22,mz2,mst12)*power2(mz2))/mst12 + (DeltaInv(mst22,mz2,mst12)*
     power2(mz2))/mst22 - 6*mst24*mz2*power2(DeltaInv(mst22,mz2,mst12)) + 36*
     mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + (138*mst24*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12)))/mst12 + (24*mst14*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (72*mst12*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (336*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 - msb12*mst22)
     - (DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*
     mst22)*power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)))/(mst12*mst22) - (
     24*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12))/(msb12*mst12 -
     msb12*mst22) + (192*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*
     power3(mst12)))/(mst12 - mst22) + (6*mz2*power2(DeltaInv(mst22,mz2,mst12))
     *power3(mst22))/mst12 + (6*mh2*power2(DeltaInv(mst22,mh2,mst12))*(-(mst12*
     mst24) + mh2*(6*mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mh2) +
     power3(mh2) + power3(mst22)))/mst12 - (24*mst12*mt2*power2(DeltaInv(mw2,
     msb12,mst12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (DeltaInv(mst22,
     mz2,mst12)*power3(mz2))/(mst12*mst22) - 6*power2(DeltaInv(mst22,mz2,mst12)
     )*power3(mz2) + (138*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/
     mst12 + (72*power2(mh2)*((3*mst12*mst22 + 2*mst24)*power2(mh2) - 2*mst22*
     power3(mh2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))/mst12 + 216*power2(mz2)
     *power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 720*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) + (144*power3(mst22)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)))/mst12 + (576*mt2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (576*mst14*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (576*mt2*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12
     - msb12*mst22) + (2304*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power4(
     mst12))/(mst12 - mst22) - (288*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12))/(msb12*mst12 - msb12*mst22) - (144*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 - (288*mst14*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2))/(msb12*mst12 - msb12*mst22)
     + (6*power2(DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 + 216*mst22*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + (144*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 - (144*mst22*power3(
     DeltaInv(mst22,mz2,mst12))*power5(mz2))/mst12))/24. + (power2(log(mst12/
     Q2))*((4*mh2*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)))/(mst12 - mst22)
     - ((2*mh2*mst22 + mst12*mst22 + 2*mst24)*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)))/mst12 - mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) - (2*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/mst12 -
     (4*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 -
     msb12*mst22) - (2*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/
     mst12 - (power2(mh2)*((3*mst12*mst22 + 2*mst24)*power2(mh2) - 2*mst22*
     power3(mh2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))/mst12 - 3*power2(mz2)*
     power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 10*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - (2*power3(mst22)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)))/mst12 - (8*mt2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) - (8*mst14*mt2*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) - (8*mt2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 -
     msb12*mst22) + (4*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12))/(msb12*mst12 - msb12*mst22) + (2*power2(mz2)*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 + (4*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2))/(msb12*mst12 - msb12*mst22) - 3*
     mst22*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - (2*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 + (2*mst22*power3(DeltaInv(
     mst22,mz2,mst12))*power5(mz2))/mst12))/2. + (-(1/msb12) - 1/mst12 - 2/
     mst22 + (6*mt2)/((mst12 - mst22)*mst22) + (4*mh2*mt2*DeltaInv(mh2,mst12,
     mst12))/(mst12 - mst22) + mh2*DeltaInv(mst22,mh2,mst12) - (4*mh2*mt2*
     DeltaInv(mst22,mst22,mh2))/(mst12 - mst22) + 2*mw2*DeltaInv(mst22,mw2,
     msb12) - (4*mt2*mw2*DeltaInv(mst22,mw2,msb12))/(mst12 - mst22) + mz2*
     DeltaInv(mst22,mz2,mst12) + (4*mt2*mw2*DeltaInv(mw2,msb12,mst12))/(mst12 -
     mst22) - (6*mt2)/(-(mst12*mst22) + power2(mst12)) - (42*(3*mh2*mst22 +
     mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)))/
     mst12 + (168*mh2*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)))/(mst12 -
     mst22) - 84*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - (252*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/msb12 + (168*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) + (504*
     mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(msb12*mst12 -
     msb12*mst22) - 42*mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - (
     126*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/mst12 - (168*
     mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) -
     (504*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12
     - msb12*mst22) - (168*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 +
     6*power3(mst12)))/(mst12 - mst22) + (1008*mt2*power2(DeltaInv(mst22,mst22,
     mh2))*power3(mst22))/(mst12 - mst22) - (252*mst22*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2))/msb12 + (504*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (504*mst12*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (
     126*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/mst12 - 420*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + (840*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 -
     mst22) - 1176*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + (2352*
     mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) -
     (252*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 +
     (504*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(
     msb12*mst12 - msb12*mst22) - 210*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) - 588*mst24*power3(mz2)*power3(DeltaInv(mst22,
     mz2,mst12)) - (126*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)))/mst12 - (840*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)))/(mst12 - mst22) - (2352*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) - (504*mt2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 - msb12*mst22)
     - (4032*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12))/(mst12 -
     mst22) + (504*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12))/(msb12*mst12 - msb12*mst22) - (42*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*(14*mh2*mst12*mst24 + 5*mst12*mst22*power2(mh2) + 3*
     mst24*power2(mh2) - 3*mst22*power3(mh2) + 3*mh2*power3(mst22) + 5*mst12*
     power3(mst22) - 3*power4(mst22)))/mst12 + (4032*mh2*mt2*power3(DeltaInv(
     mst22,mst22,mh2))*power4(mst22))/(mst12 - mst22) + (252*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/msb12 - (504*mt2*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/(msb12*mst12 -
     msb12*mst22) + (126*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(
     mst22))/mst12 - 420*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     (252*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/msb12 + (840*
     mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(mst12 - mst22) +
     (504*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(msb12*mst12
     - msb12*mst22) - (840*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2))/(mst12 - mst22) - (504*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2))/(msb12*mst12 - msb12*mst22) - 210*mst22*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mz2) - (126*mst24*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2))/mst12 + (252*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2))/msb12 + (504*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2))/(msb12*(-mst12 + mst22)) + (504*mst12*mt2*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mw2))/(msb12*mst12 - msb12*mst22) + (126*mst22*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))/mst12)/12. - (power2(Pi)*(
     4*mh2*msb12*mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)) + msb12*
     mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + 3*msb12*(mst12 - mst22)*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 4*mh2*msb12*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) +
     2*msb12*mst12*(mst12 - mst22)*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 6*mst12*(mst12 - mst22)*mst24*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 4*msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 12*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + msb12*mst12*(mst12 - mst22)*mst22*power2(mz2)*power2(DeltaInv(
     mst22,mz2,mst12)) + 3*msb12*(mst12 - mst22)*mst24*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) + 12*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 4*msb12*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 3*msb12*(mst12 - mst22)*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 24*msb12*mst12*mt2*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mst12*(mst12 - mst22)*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 12*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 12*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*(mst12 - mst22)*mst22*
     power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 14*msb12*mst12*(mst12 -
     mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 5*msb12*
     mst12*(mst12 - mst22)*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) + 3*msb12*(mst12 - mst22)*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 10*msb12*mst12*(mst12 - mst22)*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 20*msb12*mst12*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 28*msb12*
     mst12*(mst12 - mst22)*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 56*msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     6*mst12*(mst12 - mst22)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) - 12*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 5*msb12*mst12*(mst12 - mst22)*power2(mz2)*power3(
     mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 14*msb12*mst12*(mst12 - mst22)*
     mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 3*msb12*(mst12 -
     mst22)*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 56*
     msb12*mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*
     msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 3*msb12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) + 24*msb12*mt2*power2(DeltaInv(mh2,mst12,mst12))*
     power4(mst12) + 20*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mst12) + 12*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 3*msb12*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) - 96*mh2*msb12*mst12*mt2*power3(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) - 6*mst12*(mst12 - mst22)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 12*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*(mst12 - mst22)*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 10*msb12*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 6*mst12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 20*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 12*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     12*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 20*
     msb12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*
     msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 3*msb12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 3*msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) + 96*mh2*msb12*mt2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) - 12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mst12) - 6*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 12*mst12*mst22*mt2*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mw2) - 12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mw2) - 3*msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,
     mz2,mst12))*power5(mz2)))/(12.*msb12*mst12*(mst12 - mst22)) - (power2(cb)*
     (-4*mC2*msb12*mst12*mt2*DeltaInv(msb12,mst12,mC2) - mA2*msb12*mst12*(mst12
     - mst22)*DeltaInv(mst22,mA2,mst12) - 2*mC2*msb12*mst12*(mst12 - mst22)*
     DeltaInv(mst22,msb12,mC2) + 4*mC2*msb12*mst12*mt2*DeltaInv(mst22,msb12,
     mC2) + 2*msb12*mst12*(mst12 - mst22)*mw2*DeltaInv(mst22,mw2,msb12) - 4*
     msb12*mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 -
     mst22)*mz2*DeltaInv(mst22,mz2,mst12) + 4*msb12*mst12*mt2*mw2*DeltaInv(mw2,
     msb12,mst12) + 504*mst12*mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) + 168*msb12*mt2*power2(mC2)*power2(mst12)*power2(DeltaInv(
     msb12,mst12,mC2)) + 42*msb12*mst12*(mst12 - mst22)*mst22*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12)) + 126*msb12*(mst12 - mst22)*mst24*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 84*msb12*mst12*(mst12 -
     mst22)*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 252*mst12*(
     mst12 - mst22)*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 168*
     msb12*mst12*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 504*
     mst12*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb12*
     mst12*(mst12 - mst22)*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - 252*mst12*(mst12 - mst22)*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 168*msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 504*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - 42*msb12*mst12*(mst12 - mst22)*mst22*power2(mz2)*power2(DeltaInv(
     mst22,mz2,mst12)) - 126*msb12*(mst12 - mst22)*mst24*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) - 504*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 168*msb12*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 126*msb12*(mst12 - mst22)*mst22*
     power2(DeltaInv(mst22,mA2,mst12))*power3(mA2) + 504*mt2*power2(mst12)*
     power2(DeltaInv(msb12,mst12,mC2))*power3(mC2) + 252*mst12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 504*mst12*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 252*mst12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 504*mst12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 504*mt2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 126*msb12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 2352*msb12*mst12*
     mst14*mt2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 588*msb12*mst12*
     (mst12 - mst22)*mst24*power3(mA2)*power3(DeltaInv(mst22,mA2,mst12)) + 210*
     msb12*mst12*(mst12 - mst22)*power2(mA2)*power3(mst22)*power3(DeltaInv(
     mst22,mA2,mst12)) + 126*msb12*(mst12 - mst22)*power3(mA2)*power3(mst22)*
     power3(DeltaInv(mst22,mA2,mst12)) + 1176*msb12*mst12*(mst12 - mst22)*
     mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 2352*msb12*mst12*
     mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 420*msb12*mst12*
     (mst12 - mst22)*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 840*msb12*mst12*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) + 252*mst12*(mst12 - mst22)*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) - 504*mst12*mt2*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 420*msb12*mst12*(mst12 - mst22)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 840*msb12*
     mst12*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) -
     1176*msb12*mst12*(mst12 - mst22)*mst24*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 2352*msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) - 252*mst12*(mst12 - mst22)*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 504*mst12*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*msb12*mst12*(mst12 - mst22)*
     power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 588*msb12*
     mst12*(mst12 - mst22)*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     - 126*msb12*(mst12 - mst22)*power3(mst22)*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) - 2352*msb12*mst12*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 210*msb12*mst12*(mst12 - mst22)*mst22*power3(
     DeltaInv(mst22,mA2,mst12))*power4(mA2) + 126*msb12*(mst12 - mst22)*mst24*
     power3(DeltaInv(mst22,mA2,mst12))*power4(mA2) + 504*mst12*mst14*mt2*
     power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 840*msb12*mt2*power2(
     mst12)*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 420*msb12*mst12*(
     mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 252*
     mst12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     - 840*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     - 504*mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 840*
     msb12*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) +
     504*mt2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) - 840*
     msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     504*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*
     msb12*(mst12 - mst22)*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*
     power4(mst22) - 252*mst12*(mst12 - mst22)*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) + 504*mst12*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 252*mst12*(mst12 - mst22)*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 504*mst12*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*
     msb12*(mst12 - mst22)*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 420*msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 252*mst12*(mst12 - mst22)*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 840*msb12*mst12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 504*mst12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 504*mst12*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 840*msb12*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 210*msb12*mst12*(mst12 -
     mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*msb12*(
     mst12 - mst22)*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mA2,mst12))*power5(mA2)
     - 504*mt2*power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) -
     252*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mC2) + 504*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) -
     504*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power5(mst12) + 504*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 252*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2)
     - 504*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 504*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))
     )/(12.*msb12*mst12*(mst12 - mst22)) + (power2(cb)*power2(log(mst12/Q2))*(-
     4*mst12*mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(
     mst12 - mst22)*(2*mA2*mst22 + mst12*mst22 + 2*mst24)*power2(mA2)*power2(
     DeltaInv(mst22,mA2,mst12)) + 2*msb12*mst12*mst24*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) - 2*msb12*mst22*mst24*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(mst12)*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(mst22)*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) + 4*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*mst12*mst22*power2(DeltaInv(mst22,
     mz2,mst12))*power3(mz2) - 2*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,
     mst12))*power3(mz2) + 4*mst12*mt2*power2(mC2)*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - msb12*(mst12 - mst22)*power2(mA2)*((3*
     mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*
     mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mA2,mst12)) + 3*msb12*power2(mst12)*power2(mz2)*power3(
     mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 10*msb12*mst12*mst22*mst24*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 10*msb12*mst24*power2(
     mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 2*msb12*mst12*
     power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 8*msb12*
     mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 8*msb12*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 8*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 5*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 2*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 4*
     mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 2*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 3*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 3*msb12*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 4*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 2*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 2*msb12*
     mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 2*msb12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(2.*msb12*
     mst12*(mst12 - mst22)) + (power2(cb)*power2(Pi)*(-4*mst12*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     msb12*(mst12 - mst22)*(3*mA2*mst22 + mst12*mst22 + 3*mst24)*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12)) + 6*mst12*mst22*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 4*msb12*mst12*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 12*mst12*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 2*msb12*mst22*power2(mC2)*power2(mst12)*
     power2(DeltaInv(mst22,msb12,mC2)) - 6*mst24*power2(mC2)*power2(mst12)*
     power2(DeltaInv(mst22,msb12,mC2)) + 2*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mst12*mst22*mst24*power2(mw2)
     *power2(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 12*mst12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*msb12*mst22*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 6*mst24*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 2*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb12*mst12*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - 3*msb12*mst22*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(mst12)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(mst22)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + 12*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 4*msb12*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 12*mst12*mst22*mt2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 6*mst22*power2(mst12)*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) + 6*mst12*power2(mst22)*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 12*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 3*msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12)
     )*power3(mz2) - 3*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mz2) + 4*mst12*mt2*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(
     mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(
     14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 28*
     msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     56*msb12*mst12*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     28*msb12*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     + 20*msb12*mst12*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) - 10*msb12*power2(mC2)*power2(mst12)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 12*mst12*mt2*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 6*power2(mst12)*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 20*msb12*mst12*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 10*msb12*power2(mst12)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 28*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 56*
     msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 28*
     msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     12*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     6*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     ) + 5*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,
     mz2,mst12)) - 14*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + 14*msb12*mst24*power2(mst12)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)) + 3*msb12*mst12*power3(mst22)*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) + 56*msb12*mst12*mst14*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*mst12*mst22*mst24*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 20*msb12*mst12*mst22*mt2*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 12*mst12*mst24*mt2*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mC2) - 10*msb12*mst22*power2(mst12)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 6*mst24*power2(mst12)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 10*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 20*msb12*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 12*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - msb12*(mst12 - mst22)*power2(
     mA2)*power3(DeltaInv(mst22,mA2,mst12))*(14*mA2*mst12*mst24 + 5*mst12*
     mst22*power2(mA2) + 3*mst24*power2(mA2) - 3*mst22*power3(mA2) + 3*mA2*
     power3(mst22) + 5*mst12*power3(mst22) - 3*power4(mst22)) + 10*msb12*mst12*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 12*mst12*
     mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 6*
     power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22)
     + 6*mst12*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     10*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     + 12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     - 6*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 6*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 8*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 3*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 6*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 20*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 12*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 10*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 10*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 12*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) + 20*msb12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 3*msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 3*msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 5*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mz2,mst12)
     )*power4(mz2) - 5*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))
     *power5(mC2) + 6*mst22*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 6*mst12*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 6*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mst22) + 6*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 3*msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(
     mst22) + 12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2)
     - 6*mst22*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 6*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 12*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*
     mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 3*msb12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(12.*msb12*
     mst12*(mst12 - mst22)) + Fin3(mh2,mst12,mst12,Q2)*((-4*mt2*power2(
     DeltaInv(mh2,mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*
     power6(mst1)))/(mst12 - mst22) + (4*mt2*power2(sa)*power2(DeltaInv(mh2,
     mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)))/(
     mst12 - mst22) - 2*power2(snt)*power2(DeltaInv(mh2,mst12,mst12))*(mst14 +
     4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)) + 2*power2(sa)*power2(snt)*
     power2(DeltaInv(mh2,mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)
     *power6(mst1)) + 2*power2(DeltaInv(mh2,mst12,mst12))*power4(snt)*(mst14 +
     4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)) - 2*power2(sa)*power2(
     DeltaInv(mh2,mst12,mst12))*power4(snt)*(mst14 + 4*mh2*DeltaInv(mh2,mst12,
     mst12)*power6(mst1))) + Fin3(mH2,mst12,mst12,Q2)*((-4*mt2*power2(sa)*
     power2(DeltaInv(mH2,mst12,mst12))*(mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)
     *power6(mst1)))/(mst12 - mst22) - 2*power2(sa)*power2(snt)*power2(
     DeltaInv(mH2,mst12,mst12))*(mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*
     power6(mst1)) + 2*power2(sa)*power2(DeltaInv(mH2,mst12,mst12))*power4(snt)
     *(mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*power6(mst1))) + Fin3(mw2,msb12,
     mst12,Q2)*((4*mt2*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     *(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*
     (mst14 + mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (4*mt2*
     power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (2*power2(cb)*
     power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/msb12 + (2*power2(mw2)*power2(snb)*power2(
     snt)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*
     (mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)))
     )/msb12 + (4*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-mst12 +
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (4*mt2*power2(cb)*
     power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(-mst12 +
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (2*power2(mw2)*
     power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,
     msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1))))/msb12 + (2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*
     power2(DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))
     /msb12) + Fin3(mw2,msb22,mst12,Q2)*((4*mt2*power2(cb)*power2(mw2)*power2(
     snb)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*
     (mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))
     )/(msb22*(mst12 - mst22)) + (2*power2(cb)*power2(mw2)*power2(snb)*power2(
     snt)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*
     (mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))
     )/msb22 + (4*mt2*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))
     *(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb22*(mst14 + mst12*mw2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (2*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(-
     mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1))))/msb22) + Fin3(msb12,mst12,mC2,Q2)*((
     4*mt2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))
     *(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (2*
     power2(cb)*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb12,
     mst12,mC2))*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(
     msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb12 + (4*mt2*
     power2(cb)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (2*power2(
     cb)*power2(mC2)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb12) + Fin3(msb22,mst12,mC2,Q2)*((4*
     mt2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(
     -mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (2*
     power2(cb)*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb22,
     mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb22) + (
     power2(cb)*power2(log(mw2/Q2))*power3(mw2)*(mst22*(mst12 - mst22 - 2*mt2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12)
     )*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1))) + (mst12 - mst22 - 2*mt2)*
     power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*mst24 - mst22*mw2) + 2*msb12*(
     mst24 + mst22*mw2) - power6(mst2))))/(msb12*(mst12 - mst22)) + (power2(cb)
     *power2(snb)*power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*power2(DeltaInv(msb22,
     mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst22*
     power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) - msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22) - (mC2*log(mC2/Q2)*power2(cb)*power2(snb)*power2(snt)*(-(
     msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + mC2*msb22*(mC2 -
     msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22
     - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,
     mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*
     mst12*mst22*DeltaInv(mst22,msb22,mC2) - msb22*mst12*DeltaInv(mst22,msb12,
     mC2)*power2(mC2) + msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*
     mC2*msb22*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(
     -3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*
     power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*power2(mst22)
     *power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*mst22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*
     mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*
     msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     12.*msb12*msb22*mst12*mst22) + (mC2*log(mC2/Q2)*power2(cb)*power2(snb)*(-(
     msb12*mst12*mst22) + msb22*mst12*mst22 - 2*msb12*mst12*mt2 + 2*msb22*
     mst12*mt2 + 2*msb12*mst22*mt2 - 2*msb22*mst22*mt2 + 2*mC2*msb22*(mC2 -
     msb12 + 5*mst12)*mst22*mt2*DeltaInv(msb12,mst12,mC2) - 2*mC2*msb12*(mC2 -
     msb22 + 5*mst12)*mst22*mt2*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*
     mst12*mst22*DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*msb22*mst12*mt2*
     DeltaInv(mst22,msb12,mC2) - 10*mC2*msb22*mst12*mst22*mt2*DeltaInv(mst22,
     msb12,mC2) - mC2*msb12*msb22*mst12*mst22*DeltaInv(mst22,msb22,mC2) - 2*
     mC2*msb12*msb22*mst12*mt2*DeltaInv(mst22,msb22,mC2) + 10*mC2*msb12*mst12*
     mst22*mt2*DeltaInv(mst22,msb22,mC2) - msb22*mst12*mst22*DeltaInv(mst22,
     msb12,mC2)*power2(mC2) - 2*msb22*mst12*mt2*DeltaInv(mst22,msb12,mC2)*
     power2(mC2) + msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2)*power2(mC2) + 2*
     msb12*mst12*mt2*DeltaInv(mst22,msb22,mC2)*power2(mC2) + msb12*power2(
     mst12) - msb22*power2(mst12) - mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2)*
     power2(mst12) + 5*mC2*msb22*mst22*DeltaInv(mst22,msb12,mC2)*power2(mst12)
     + mC2*msb12*msb22*DeltaInv(mst22,msb22,mC2)*power2(mst12) - 5*mC2*msb12*
     mst22*DeltaInv(mst22,msb22,mC2)*power2(mst12) + msb22*DeltaInv(mst22,
     msb12,mC2)*power2(mC2)*power2(mst12) - msb12*DeltaInv(mst22,msb22,mC2)*
     power2(mC2)*power2(mst12) - 5*mC2*msb22*mst12*DeltaInv(mst22,msb12,mC2)*
     power2(mst22) + 5*mC2*msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mst22)
     - 12*mC2*msb22*mst12*mst22*mt2*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) +
     mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 12*mC2*msb12*
     mst12*mst22*mt2*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 12*mC2*msb22*mst12*mst22*mst24*
     mt2*power2(DeltaInv(mst22,msb12,mC2)) - 12*msb12*msb22*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb22*mst22*mst24*
     power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 6*msb12*msb22*mst22*
     power2(mC2)*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*
     mst12*mst24*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 36*mC2*
     msb12*msb22*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*
     msb12*msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,
     mC2)) - 168*msb22*mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) + 18*mC2*msb12*msb22*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) + 84*msb22*power2(mC2)*power2(mst12)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 12*mC2*msb12*mst12*mst22*mst24*
     mt2*power2(DeltaInv(mst22,msb22,mC2)) + 12*msb12*msb22*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 6*mC2*msb12*mst22*mst24*
     power2(mst12)*power2(DeltaInv(mst22,msb22,mC2)) - 6*msb12*msb22*mst22*
     power2(mC2)*power2(mst12)*power2(DeltaInv(mst22,msb22,mC2)) - 6*mC2*msb12*
     mst12*mst24*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 36*mC2*
     msb12*msb22*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*
     msb12*msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,
     mC2)) + 168*msb12*mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(
     mst22,msb22,mC2)) - 18*mC2*msb12*msb22*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) - 84*msb12*power2(mC2)*power2(mst12)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 12*msb22*mst12*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb22*mst22*power2(
     mst12)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 6*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 12*msb12*
     mst12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 6*msb12*
     mst22*power2(mst12)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 6*
     msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) -
     18*mC2*msb12*msb22*mst12*power2(DeltaInv(mst22,msb12,mC2))*power3(mst22) -
     84*msb22*mst12*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*power3(mst22)
     + 18*mC2*msb12*msb22*mst12*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22)
     + 84*msb12*mst12*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mst22) - 288*msb12*msb22*mst12*mst22*mst24*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*mst22*mst24*power2(mC2)*
     power2(mst12)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*msb22*mst12*
     mst24*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 288*
     msb22*mst12*mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     + 144*msb22*mst22*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 144*msb22*mst12*mst24*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 288*msb12*msb22*mst12*mt2*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*power2(
     mst12)*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) + 288*msb12*msb22*mst12*mst22*mst24*mt2*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - 144*msb12*msb22*mst22*mst24*power2(mC2)*power2(mst12)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*mst24*power2(
     mC2)*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 288*msb12*mst12*
     mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*
     mst22*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     144*msb12*mst12*mst24*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 288*msb12*msb22*mst12*mt2*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*power2(mst12)*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) -
     72*msb22*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) + 72*msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 144*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 72*msb12*power2(mst12)*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb12*mst12*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 144*msb22*mst12*
     mst22*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 -
     2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 144*
     msb12*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + 144*msb22*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power6(mst2) - 72*msb22*mst22*power2(mC2)*power2(mst12)*
     power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) + 72*msb22*mst12*power2(
     mC2)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 144*
     msb12*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*
     power6(mst2) + 72*msb12*mst22*power2(mC2)*power2(mst12)*power3(DeltaInv(
     mst22,msb22,mC2))*power6(mst2) - 72*msb12*mst12*power2(mC2)*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(12.*msb12*msb22*mst12*
     mst22*(-mst12 + mst22)) + (power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(
     mst22*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12)
     )*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1))) - power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/msb12 + (power2(cb)*power2(snt)*power2(log(mw2/Q2))*
     power3(mw2)*(-(mst22*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(
     mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/msb12 + (power2(log(mw2/Q2))*power3(mw2)*(
     mst22*(-mst12 + mst22 + 2*mt2)*power2(DeltaInv(mst22,mw2,msb12)) - 2*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*
     (-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + (
     mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 -
     mst22)) + (power2(cb)*power2(snb)*power2(log(mw2/Q2))*power3(mw2)*(msb22*
     mst22*(-mst12 + mst22 + 2*mt2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*
     mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*(-
     (msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))) + msb22*power3(DeltaInv(
     mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*
     mw2) + power6(mst1)) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1)))) + msb12*(mst12 - mst22 - 2*mt2)*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(2*mst24 - mst22*mw2) + 2*msb22*(mst24 +
     mst22*mw2) - power6(mst2)) + msb22*(mst12 - mst22 - 2*mt2)*power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*msb22*(mst12 - mst22)) + (mw2*log(mw2/
     Q2)*power2(cb)*power2(snt)*(-mst12 + mst22 + mst12*mw2*(-msb12 + 5*mst22 +
     mw2)*DeltaInv(mst22,mw2,msb12) + mst22*(msb12 - 5*mst12 - mw2)*mw2*
     DeltaInv(mw2,msb12,mst12) + 6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 6*mst12*
     mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb12,mst12)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*
     mw2) + power6(mst1)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(12.*msb12*mst12*mst22) + (mw2*log(mw2/Q2)*power2(snt)*(
     mst12 - mst22 + mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,
     msb12) + mst22*mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(mw2,msb12,mst12) - 6*
     mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb12)) + 6*mst12*mst22*mw2*(-mst14 + (14*mst12
     - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) -
     72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) + 72*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     mst12*mst22) + (mw2*log(mw2/Q2)*(-((mst12 - mst22)*(mst12 - 2*mt2)) +
     mst12*(mst12 - mst22 - 2*mt2)*mw2*(-msb12 + 5*mst22 + mw2)*DeltaInv(mst22,
     mw2,msb12) + 2*mst22*mt2*mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(mw2,msb12,
     mst12) + 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 12*
     mst12*mst22*mt2*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2)
     )*power2(DeltaInv(mw2,msb12,mst12)) - 144*mst12*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*(mst12 - mst22 - 2*
     mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*
     mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*mst12*(
     mst12 - mst22)*mst22) + (mw2*log(mw2/Q2)*power2(cb)*((mst12 - mst22)*(
     mst12 - 2*mt2) + mst12*(mst12 - mst22 - 2*mt2)*(msb12 - 5*mst22 - mw2)*
     mw2*DeltaInv(mst22,mw2,msb12) + 2*mst22*mt2*(msb12 - 5*mst12 - mw2)*mw2*
     DeltaInv(mw2,msb12,mst12) - 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-
     mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(
     mst22,mw2,msb12)) - 12*mst12*mst22*mt2*mw2*(-mst14 + (14*mst12 - mw2)*mw2
     + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) + 144*mst12*
     mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) + 72*mst12*mst22*
     (mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(
     mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))
     /(12.*msb12*mst12*(mst12 - mst22)*mst22) + (power2(snb)*power2(snt)*
     power2(log(mw2/Q2))*power3(mw2)*(-(msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))) + msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)) + msb22*mst12*
     power2(DeltaInv(mw2,msb12,mst12)) - msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*
     msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*mw2*
     power3(DeltaInv(mw2,msb12,mst12)) - msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb22,mst12)) - 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb22,mst12))
     - 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,mst12)) + msb12*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - msb22*power3(DeltaInv(mw2,
     msb12,mst12))*power6(mst1) + msb12*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - msb12*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*msb22) + (power2(cb)*power2(snb)*
     power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(msb22*mst22*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)) - msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12)) + msb12*mst12*power2(DeltaInv(mw2,
     msb22,mst12)) - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) - 2*
     msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb22*mst14*
     mw2*power3(DeltaInv(mw2,msb12,mst12)) + msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb22,mst12)) + 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb22,mst12))
     + 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,mst12)) - msb12*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + msb22*power3(DeltaInv(mw2,
     msb12,mst12))*power6(mst1) - msb12*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) - msb22*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) + msb12*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*msb22) + (power2(snb)*power2(log(mw2/
     Q2))*power3(mw2)*(msb22*mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(
     mst22,mw2,msb12)) + msb12*mst22*(-mst12 + mst22 + 2*mt2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 2*mt2*(msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) -
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1)) - msb12*power2(DeltaInv(mw2,
     msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))) + msb22*(mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*mst24 - mst22*
     mw2) + 2*msb12*(mst24 + mst22*mw2) - power6(mst2)) + msb12*(mst12 - mst22
     - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22*(mst12 - mst22)
     ) - (mw2*log(mw2/Q2)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12
     + msb12*mst22 - msb22*mst22 + msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*
     DeltaInv(mst22,mw2,msb12) + msb12*mst12*mw2*(-msb22 + 5*mst22 + mw2)*
     DeltaInv(mst22,mw2,msb22) - msb12*msb22*mst22*mw2*DeltaInv(mw2,msb12,
     mst12) + 5*msb22*mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*
     mst22*mw2*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(
     mw2,msb22,mst12) + msb22*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     msb12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*
     mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*
     msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*
     msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*
     msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) - 6*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     84*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - 6*msb22*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*
     msb12*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*
     msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     ) + 144*msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(
     mst1) + 72*msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     *(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))
     - 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(
     12.*msb12*msb22*mst12*mst22) + (mw2*log(mw2/Q2)*power2(cb)*power2(snb)*
     power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*
     mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*
     msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,
     mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     msb22*mst12*mst22) + (mw2*log(mw2/Q2)*power2(cb)*power2(snb)*(msb12*mst12*
     mst22 - msb22*mst12*mst22 + 2*msb12*mst12*mt2 - 2*msb22*mst12*mt2 - 2*
     msb12*mst22*mt2 + 2*msb22*mst22*mt2 + msb22*mst12*(mst12 - mst22 - 2*mt2)*
     (msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12
     - mst22 - 2*mt2)*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) +
     2*msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 10*msb22*mst12*
     mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 2*msb12*msb22*mst22*mt2*mw2*
     DeltaInv(mw2,msb22,mst12) + 10*msb12*mst12*mst22*mt2*mw2*DeltaInv(mw2,
     msb22,mst12) - msb12*power2(mst12) + msb22*power2(mst12) - 2*msb22*mst22*
     mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 2*msb12*mst22*mt2*DeltaInv(
     mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*(mst12 - mst22 - 2*mt2)
     *mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*
     mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) + 12*msb22*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) - 36*msb12*msb22*mst22*mt2*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12)) - 12*msb12*msb22*mst12*mst22*mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 168*msb22*mst22*mt2*power2(mst12)
     *power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 12*msb12*mst12*mst14*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 36*msb12*msb22*mst22*
     mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) + 12*msb12*msb22*
     mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 168*msb12*
     mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     12*msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) -
     12*msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) -
     288*msb12*msb22*mst12*mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 288*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 288*msb12*msb22*mst22*mt2*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*msb12*msb22*mst12*
     mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 288*msb12*
     mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 288*
     msb12*msb22*mst22*mt2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + 144*msb22*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 144*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 144*msb22*mst12*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - 144*msb12*mst12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) - 72*msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*msb22*mst12*mst22*(-mst12
     + mst22)) + (mw2*log(mw2/Q2)*power2(snb)*(-(msb12*mst12*mst22) + msb22*
     mst12*mst22 - 2*msb12*mst12*mt2 + 2*msb22*mst12*mt2 + 2*msb12*mst22*mt2 -
     2*msb22*mst22*mt2 + msb22*mst12*(mst12 - mst22 - 2*mt2)*mw2*(-msb12 + 5*
     mst22 + mw2)*DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 - mst22 - 2*
     mt2)*(msb22 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb22) - 2*msb12*
     msb22*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 10*msb22*mst12*mst22*mt2*
     mw2*DeltaInv(mw2,msb12,mst12) + 2*msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,
     msb22,mst12) - 10*msb12*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) +
     msb12*power2(mst12) - msb22*power2(mst12) + 2*msb22*mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*power2(mw2) - 2*msb12*mst22*mt2*DeltaInv(mw2,msb22,mst12)
     *power2(mw2) + 6*msb22*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (
     14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,
     msb12)) - 6*msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*
     mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)
     ) - 12*msb22*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) +
     36*msb12*msb22*mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,
     mst12)) + 12*msb12*msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 168*msb22*mst22*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 12*msb12*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) - 36*msb12*msb22*mst22*mt2*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12)) - 12*msb12*msb22*mst12*mst22*mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 168*msb12*mst22*mt2*power2(mst12)
     *power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 12*msb22*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 12*msb12*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 288*msb12*msb22*mst12*
     mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*msb22*
     mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*
     msb12*msb22*mst22*mt2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 288*msb12*msb22*mst12*mst14*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 288*msb12*mst12*mst14*mst22*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 288*msb12*msb22*mst22*mt2*power2(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb22*mst22*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*
     msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*msb22*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power6(mst1) + 144*msb12*mst12*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power6(mst1) - 72*msb22*mst12*mst22*(mst12 -
     mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) + 72*
     msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*
     mw2) + power6(mst2))))/(12.*msb12*msb22*mst12*mst22*(-mst12 + mst22)) + (
     power2(cb)*power2(DeltaInv(mst22,mz2,mst12))*power2(log(mz2/Q2))*power3(
     mz2)*(mst22 - DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*
     mst12*(mst24 + mst22*mz2) + power6(mst2))))/(2.*mst12) + (power2(DeltaInv(
     mst22,mz2,mst12))*power2(log(mz2/Q2))*power3(mz2)*(-mst22 + DeltaInv(
     mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 + mst22*mz2)
     + power6(mst2))))/(2.*mst12) + (mz2*log(mz2/Q2)*(-1 + mz2*(-mst12 + 5*
     mst22 + mz2)*DeltaInv(mst22,mz2,mst12) + 6*mst22*mz2*(-mst24 + (14*mst22 -
     mz2)*mz2 + mst12*(3*mst22 + mz2))*power2(DeltaInv(mst22,mz2,mst12)) - 72*
     mst22*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*(mz2*(-2*mst24 +
     mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/(24.*mst12*
     mst22) + (mz2*log(mz2/Q2)*power2(cb)*(1 + (mst12 - 5*mst22 - mz2)*mz2*
     DeltaInv(mst22,mz2,mst12) - 6*mst22*mz2*(-mst24 + (14*mst22 - mz2)*mz2 +
     mst12*(3*mst22 + mz2))*power2(DeltaInv(mst22,mz2,mst12)) + 72*mst22*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*(mz2*(-2*mst24 + mst22*mz2)
     - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/(24.*mst12*mst22) + (Fin3(
     mst22,mA2,mst12,Q2)*power2(cb)*power2(mA2)*power2(DeltaInv(mst22,mA2,
     mst12))*(-mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(
     mst12*mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/mst12 + (
     power2(cb)*power2(DeltaInv(mst22,mA2,mst12))*power2(log(mA2/Q2))*power3(
     mA2)*(-mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(mst12*
     mst22 + mst24) + mst22*power2(mA2) + power6(mst2))))/(2.*mst12) + (mA2*
     log(mA2/Q2)*power2(cb)*(-1 + mA2*(mA2 - mst12 + 5*mst22)*DeltaInv(mst22,
     mA2,mst12) - 6*mA2*mst22*(-3*mst12*mst22 - mA2*(mst12 + 14*mst22) + mst24
     + power2(mA2))*power2(DeltaInv(mst22,mA2,mst12)) - 72*mst22*power2(mA2)*
     power3(DeltaInv(mst22,mA2,mst12))*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 +
     mst24) + mst22*power2(mA2) + power6(mst2))))/(24.*mst12*mst22) + (mC2*log(
     mC2/Q2)*power2(cb)*power2(snt)*(mst12 - mst22 + mC2*(mC2 - msb12 + 5*
     mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*mst12*(-mC2 + msb12 - 5*
     mst22)*DeltaInv(mst22,msb12,mC2) - 6*mC2*mst12*mst22*(-3*msb12*mst12 -
     mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,
     mC2)) + 6*mC2*mst12*mst22*(-3*msb12*mst22 - mC2*(msb12 + 14*mst22) + mst24
     + power2(mC2))*power2(DeltaInv(mst22,msb12,mC2)) - 72*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(12.*msb12*
     mst12*mst22) + (mC2*log(mC2/Q2)*power2(cb)*(-((mst12 - mst22)*(mst12 - 2*
     mt2)) + 2*mC2*(mC2 - msb12 + 5*mst12)*mst22*mt2*DeltaInv(msb12,mst12,mC2)
     + mC2*mst12*(mC2 - msb12 + 5*mst22)*(mst12 - mst22 - 2*mt2)*DeltaInv(
     mst22,msb12,mC2) - 12*mC2*mst12*mst22*mt2*(-3*msb12*mst12 - mC2*(msb12 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) - 6*
     mC2*mst12*mst22*(mst12 - mst22 - 2*mt2)*(-3*msb12*mst22 - mC2*(msb12 + 14*
     mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb12,mC2)) - 144*
     mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) -
     72*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(12.*msb12*mst12*(mst12 - mst22)*mst22) + Fin3(mH2,
     mst22,mst22,Q2)*((4*mt2*power2(sa)*power2(DeltaInv(mH2,mst22,mst22))*(
     mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)))/(mst12 - mst22) -
     2*power2(sa)*power2(snt)*power2(DeltaInv(mH2,mst22,mst22))*(mst24 + 4*mH2*
     DeltaInv(mH2,mst22,mst22)*power6(mst2)) + 2*power2(sa)*power2(DeltaInv(
     mH2,mst22,mst22))*power4(snt)*(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*
     power6(mst2))) + Fin3(mst22,mst22,mh2,Q2)*((4*mt2*power2(DeltaInv(mst22,
     mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/(mst12
     - mst22) - (4*mt2*power2(sa)*power2(DeltaInv(mst22,mst22,mh2))*(mst24 + 4*
     mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/(mst12 - mst22) - 2*power2(
     snt)*power2(DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,
     mst22,mh2)*power6(mst2)) + 2*power2(sa)*power2(snt)*power2(DeltaInv(mst22,
     mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)) + 2*
     power2(DeltaInv(mst22,mst22,mh2))*power4(snt)*(mst24 + 4*mh2*DeltaInv(
     mst22,mst22,mh2)*power6(mst2)) - 2*power2(sa)*power2(DeltaInv(mst22,mst22,
     mh2))*power4(snt)*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))
     + Fin3(mst22,mw2,msb12,Q2)*((2*(mst12 - mst22 - 2*mt2)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     (-mst12 + mst22)) + (2*(mst12 - mst22 - 2*mt2)*power2(cb)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(
     msb12*(mst12 - mst22)) + (2*(mst12 - mst22 - 2*mt2)*power2(mw2)*power2(
     snb)*power2(DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*
     (mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)))
     )/(msb12*(mst12 - mst22)) + (2*(mst12 - mst22 - 2*mt2)*power2(cb)*power2(
     mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(
     mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2)
     + power6(mst2))))/(msb12*(-mst12 + mst22)) + (2*power2(mw2)*power2(snt)*
     power2(DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/
     msb12 + (2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 + (2*
     power2(cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(-
     mst22 + DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(
     mst24 + mst22*mw2) + power6(mst2))))/msb12 + (2*power2(mw2)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 + DeltaInv(mst22,
     mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/msb12) + Fin3(mst22,mw2,msb22,Q2)*((2*(mst12 - mst22 - 2*
     mt2)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 -
     DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb22*(-mst12 + mst22)) + (2*(mst12 - mst22
     - 2*mt2)*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,
     msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*(mst12 - mst22)) + (
     2*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(
     mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/msb22 + (2*power2(cb)*power2(mw2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(-mst22 +
     DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/msb22) + Fin3(mst22,mz2,mst12,Q2)*((power2(
     cb)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst22 - DeltaInv(mst22,
     mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) +
     power6(mst2))))/mst12 + (power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(-
     mst22 + DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(
     mst24 + mst22*mz2) + power6(mst2))))/mst12) + (power2(cb)*power2(snt)*
     power2(log(mC2/Q2))*power3(mC2)*(-(mst12*power2(DeltaInv(msb12,mst12,mC2))
     ) + power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12
     + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,
     msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/msb12 + (
     power2(cb)*power2(log(mC2/Q2))*power3(mC2)*(-2*mst12*mt2*power2(DeltaInv(
     msb12,mst12,mC2)) + 2*mt2*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     (mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/(msb12*(mst12 - mst22)) + Fin3(mst22,
     msb12,mC2,Q2)*((2*power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(
     mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*
     mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (
     2*(mst12 - mst22 - 2*mt2)*power2(cb)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 -
     mst22)) + (2*(mst12 - mst22 - 2*mt2)*power2(cb)*power2(mC2)*power2(snb)*
     power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*
     msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(msb12*(-mst12 + mst22)) + (2*power2(cb)*power2(mC2)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,
     msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/msb12) + Fin3(mst22,msb22,mC2,Q2)*((2*power2(cb)*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(
     mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (2*(mst12 - mst22 -
     2*mt2)*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2)
     )*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22
     + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22))) +
     Fin3(mst22,mh2,mst12,Q2)*((power2(mh2)*power2(sa)*power2(DeltaInv(mst22,
     mh2,mst12))*(mst22 - DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(
     mst12*mst22 + mst24) + mst22*power2(mh2) + power6(mst2))))/mst12 + (4*
     power2(mh2)*power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(mst22 -
     DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power6(mst2))))/mst12 + (4*power2(mh2)*power2(sa)*
     power2(DeltaInv(mst22,mh2,mst12))*power4(snt)*(mst22 - DeltaInv(mst22,mh2,
     mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power6(mst2))))/mst12 + (power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-
     mst22 + DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power6(mst2))))/mst12 + (4*power2(mh2)*
     power2(sa)*power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(-mst22 +
     DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power6(mst2))))/mst12 + (4*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*power4(snt)*(-mst22 + DeltaInv(mst22,mh2,mst12)
     *(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power6(mst2))))/mst12) + Fin3(mH2,mst22,mst12,Q2)*((4*power2(mH2)*power2(
     sa)*power2(snt)*power2(DeltaInv(mH2,mst22,mst12))*(mst22 - DeltaInv(mH2,
     mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(
     mH2) + power6(mst2))))/mst12 + (power2(mH2)*power2(sa)*power2(DeltaInv(
     mH2,mst22,mst12))*(-mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*
     mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/mst12 + (
     4*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,mst22,mst12))*power4(snt)*(-
     mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power6(mst2))))/mst12) + (power2(cb)*power2(
     snb)*power2(log(mC2/Q2))*power3(mC2)*(2*msb22*mst12*mt2*power2(DeltaInv(
     msb12,mst12,mC2)) - 2*msb12*mst12*mt2*power2(DeltaInv(msb22,mst12,mC2)) -
     2*msb22*mt2*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(
     msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 2*msb12*mt2*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - (mst12 - mst22 - 2*mt2)*(-(
     msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))) + msb22*power3(DeltaInv(
     mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2)) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(
     mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))))/(msb12*msb22*(mst12 -
     mst22)) + (log(msb12/Q2)*power2(cb)*(-(mC2*mst12*mst22) - 2*mC2*mst12*mt2
     + 2*mC2*mst22*mt2 + mst12*mst22*mw2 + 2*mst12*mt2*mw2 - 2*mst22*mt2*mw2 -
     2*mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,msb12) + 2*mst12*mst14*mst22*
     mt2*DeltaInv(mw2,msb12,mst12) + mC2*power2(mst12) - mw2*power2(mst12) +
     mst22*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst12) - 2*msb12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + 10*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) - mst12*mst24*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) + 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - 10*
     mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*DeltaInv(
     mst22,mw2,msb12)*power2(mst12)*power2(mst22) + 5*mw2*DeltaInv(mst22,mw2,
     msb12)*power2(mst12)*power2(mst22) + msb12*mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) + 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(
     mw2) - 10*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 2*msb12*
     mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 10*mst12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*DeltaInv(mst22,mw2,msb12)*
     power2(mst12)*power2(mw2) + 5*mst22*DeltaInv(mst22,mw2,msb12)*power2(
     mst12)*power2(mw2) - 5*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)*
     power2(mw2) - 12*msb12*mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*msb12*mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,
     mw2,msb12)) - 6*msb12*mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12)) + 276*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 138*mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 138*mst12*mst24*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 72*msb12*mst12*mt2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 12*msb12*mst12*mst14*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 276*mst12*mst14*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 72*msb12*mst22*mt2*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 2*mst22*mt2*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*(-mst12 +
     mst22 + 2*mt2)*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) -
     (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 12*
     mC2*mst12*mst22*mt2*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) +
     mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(
     mC2) + power3(mst12)) + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power3(
     mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power3(mst22) + 36*msb12*
     mst12*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*mC2*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power3(mw2) - 2*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) +
     2*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power3(mw2) + DeltaInv(mst22,mw2,
     msb12)*power2(mst12)*power3(mw2) - 12*msb12*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(mst12)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 276*mst12*mt2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 12*
     msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 276*
     mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) +
     138*mst12*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22)*power3(mw2) -
     144*mst12*mst22*mt2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2))
     + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + 1440*msb12*mst12*mst22*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst22*mst24*power2(mst12)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst24*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 1440*msb12*mst12*
     mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 12*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 432*msb12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 288*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) +
     12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 432*msb12*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) -
     216*msb12*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 288*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) - 144*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) + 288*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))
     *power4(mw2) - 144*mst22*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 144*mst12*mst24*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mw2) + 432*msb12*mst12*mt2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*mst12*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 288*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 432*msb12*
     mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) +
     288*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)
     + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 288*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) +
     144*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 288*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) + 144*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 144*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 288*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power6(mst22)))/(12.*msb12*mst12*(mst12 - mst22)*mst22) + (
     log(msb12/Q2)*power2(cb)*power2(snb)*(mC2*mst12*mst22 + 2*mC2*mst12*mt2 -
     2*mC2*mst22*mt2 - mst12*mst22*mw2 - 2*mst12*mt2*mw2 + 2*mst22*mt2*mw2 + 2*
     mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,msb12) - 2*mst12*mst14*mst22*mt2*
     DeltaInv(mw2,msb12,mst12) - mC2*power2(mst12) + mw2*power2(mst12) - mst22*
     mst24*DeltaInv(mst22,mw2,msb12)*power2(mst12) + 2*msb12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) - 10*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) + mst12*mst24*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) - 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 10*
     mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + msb12*DeltaInv(
     mst22,mw2,msb12)*power2(mst12)*power2(mst22) - 5*mw2*DeltaInv(mst22,mw2,
     msb12)*power2(mst12)*power2(mst22) - msb12*mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) - 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(
     mw2) + 10*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 2*msb12*
     mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 10*mst12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + msb12*DeltaInv(mst22,mw2,msb12)*
     power2(mst12)*power2(mw2) - 5*mst22*DeltaInv(mst22,mw2,msb12)*power2(
     mst12)*power2(mw2) + 5*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)*
     power2(mw2) + 12*msb12*mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) - 6*msb12*mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*msb12*mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12)) - 276*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 138*mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 138*mst12*mst24*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 72*msb12*mst12*mt2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 12*msb12*mst12*mst14*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 276*mst12*mst14*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 72*msb12*mst22*mt2*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 2*mst22*mt2*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*(mst12 -
     mst22 - 2*mt2)*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) -
     (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 12*
     mC2*mst12*mst22*mt2*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) +
     mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(
     mC2) + power3(mst12)) - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power3(
     mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power3(mst22) - 36*msb12*
     mst12*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*mC2*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) + mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power3(mw2) + 2*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) -
     2*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power3(mw2) - DeltaInv(mst22,mw2,
     msb12)*power2(mst12)*power3(mw2) + 12*msb12*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst22*power2(mst12)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 276*mst12*mt2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 12*
     msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 276*
     mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) -
     138*mst12*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22)*power3(mw2) +
     144*mst12*mst22*mt2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2))
     + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) - 1440*msb12*mst12*mst22*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst22*mst24*power2(mst12)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst24*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 1440*msb12*mst12*
     mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 12*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 432*msb12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 288*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 6*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 432*msb12*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     216*msb12*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 288*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) + 144*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) - 288*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))
     *power4(mw2) + 144*mst22*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 144*mst12*mst24*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mw2) - 432*msb12*mst12*mt2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mst12*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 288*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 432*msb12*
     mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) -
     288*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)
     - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 288*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) -
     144*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 288*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 144*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 144*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 288*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power6(mst22)))/(12.*msb12*mst12*(mst12 - mst22)*mst22) + (
     log(msb22/Q2)*power2(cb)*power2(snb)*(-(mC2*mst12*mst22) - 2*mC2*mst12*mt2
     + 2*mC2*mst22*mt2 + mst12*mst22*mw2 + 2*mst12*mt2*mw2 - 2*mst22*mt2*mw2 -
     2*mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,msb22) + 2*mst12*mst14*mst22*
     mt2*DeltaInv(mw2,msb22,mst12) + mC2*power2(mst12) - mw2*power2(mst12) +
     mst22*mst24*DeltaInv(mst22,mw2,msb22)*power2(mst12) - 2*msb22*mst22*mt2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) + 10*mst22*mt2*mw2*DeltaInv(mw2,
     msb22,mst12)*power2(mst12) - mst12*mst24*DeltaInv(mst22,mw2,msb22)*power2(
     mst22) + 2*msb22*mst12*mt2*DeltaInv(mst22,mw2,msb22)*power2(mst22) - 10*
     mst12*mt2*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22) - msb22*DeltaInv(
     mst22,mw2,msb22)*power2(mst12)*power2(mst22) + 5*mw2*DeltaInv(mst22,mw2,
     msb22)*power2(mst12)*power2(mst22) + msb22*mst12*mst22*DeltaInv(mst22,mw2,
     msb22)*power2(mw2) + 2*msb22*mst12*mt2*DeltaInv(mst22,mw2,msb22)*power2(
     mw2) - 10*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 2*msb22*
     mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 10*mst12*mst22*mt2*
     DeltaInv(mw2,msb22,mst12)*power2(mw2) - msb22*DeltaInv(mst22,mw2,msb22)*
     power2(mst12)*power2(mw2) + 5*mst22*DeltaInv(mst22,mw2,msb22)*power2(
     mst12)*power2(mw2) - 5*mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22)*
     power2(mw2) - 12*msb22*mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,
     mw2,msb22)) + 6*msb22*mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,
     mw2,msb22)) - 6*msb22*mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb22)) + 276*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) - 138*mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 138*mst12*mst24*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 72*msb22*mst12*mt2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 36*msb22*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 12*msb22*mst12*mst14*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 276*mst12*mst14*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*mt2*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 2*mst22*mt2*
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*(-mst12 +
     mst22 + 2*mt2)*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) -
     (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 12*
     mC2*mst12*mst22*mt2*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) +
     mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(
     mC2) + power3(mst12)) + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power3(
     mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power3(mst22) + 36*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb22,mC2))*(-(
     msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,
     msb22)*power3(mw2) - 2*mst12*mt2*DeltaInv(mst22,mw2,msb22)*power3(mw2) +
     2*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power3(mw2) + DeltaInv(mst22,mw2,
     msb22)*power2(mst12)*power3(mw2) - 12*msb22*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst22*power2(mst12)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 276*mst12*mt2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 12*
     msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 276*
     mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) +
     138*mst12*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22)*power3(mw2) -
     144*mst12*mst22*mt2*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2))
     + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*
     msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + 1440*msb22*mst12*mst22*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst22*mst24*power2(mst12)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst24*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 1440*msb22*mst12*
     mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 12*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 432*msb22*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 288*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) +
     12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 6*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 432*msb22*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) -
     216*msb22*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) + 288*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)
     )*power4(mst22) - 144*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) + 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,mst12))
     *power4(mw2) + 288*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb22))
     *power4(mw2) - 144*mst22*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) + 144*mst12*mst24*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) + 432*msb22*mst12*mt2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2) - 216*msb22*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 216*msb22*mst12*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 288*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 432*msb22*
     mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) +
     288*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mst12)
     + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power5(mst22) + 216*msb22*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(mst22) - 288*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(mst22) +
     144*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 288*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 144*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) - 144*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) + 288*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb22,mst12))*power5(mw2) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power6(mst22)))/(12.*msb22*mst12*(mst12 - mst22)*mst22) - (
     log(mst12/Q2)*power2(cb)*(-(mA2*msb12*mst12) + mA2*msb12*mst22 + msb12*
     mst12*mz2 - msb12*mst22*mz2 - 4*mst12*(5*mC2*mst12 - msb12*mst12 + mst14)*
     mst22*mt2*DeltaInv(msb12,mst12,mC2) - msb12*mst12*mst22*mst24*DeltaInv(
     mst22,mz2,mst12) + 4*mst12*mst14*mst22*mt2*DeltaInv(mw2,msb12,mst12) - 4*
     msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 20*mst22*mt2*
     mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst24*DeltaInv(mst22,
     mz2,mst12)*power2(mst22) - 5*msb12*mst12*mz2*DeltaInv(mst22,mz2,mst12)*
     power2(mst22) + msb12*DeltaInv(mst22,mz2,mst12)*power2(mst12)*power2(
     mst22) - 6*msb12*mst12*mst22*DeltaInv(mst22,mz2,mst12)*power2(mz2) +
     msb12*DeltaInv(mst22,mz2,mst12)*power2(mst12)*power2(mz2) + 5*msb12*
     DeltaInv(mst22,mz2,mst12)*power2(mst22)*power2(mz2) - 6*msb12*mst22*mst24*
     mz2*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12)) + 6*msb12*mst12*mst24*
     mz2*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12)) + 138*msb12*mst12*
     mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 138*msb12*
     mst24*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 36*
     msb12*power2(mst12)*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) + 24*msb12*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,
     mst12)) + 336*mst12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 72*msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) + msb12*(mst12 - mst22)*DeltaInv(mst22,mA2,mst12)*(
     mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mA2) + 5*mA2*
     power2(mst22) + power3(mA2)) + 24*mC2*mst12*mst22*mt2*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - msb12*mst12*DeltaInv(mst22,mz2,mst12)
     *power3(mst22) + 5*msb12*mz2*DeltaInv(mst22,mz2,mst12)*power3(mst22) - 36*
     msb12*mst12*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22) -
     6*mA2*msb12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mA2,mst12))*(-(
     mst12*mst24) + mA2*(6*mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(
     mA2) + power3(mA2) + power3(mst22)) - 24*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - msb12*mst12*DeltaInv(mst22,mz2,
     mst12)*power3(mz2) + msb12*mst22*DeltaInv(mst22,mz2,mst12)*power3(mz2) -
     6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2)
     + 144*msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(
     mz2) - 138*msb12*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22)*power3(
     mz2) + 288*mst12*mst22*mt2*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*msb12*(mst12 - mst22)*mst22*power2(mA2)*((
     3*mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*
     mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mA2,mst12)) + 720*msb12*mst22*mst24*power2(mst12)*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 720*msb12*mst12*mst24*power2(
     mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 576*msb12*mst12*
     mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 24*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 576*msb12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 576*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*
     msb12*mst12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 216*
     msb12*power2(mst12)*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(
     mst22) + 144*msb12*mst12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 288*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 6*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 144*msb12*mst12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12)
     )*power4(mz2) - 144*msb12*mst24*power2(mst22)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) + 216*msb12*power2(mst12)*power2(mst22)*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) - 216*msb12*mst12*power3(mst22)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 288*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 6*msb12*mz2*power2(
     DeltaInv(mst22,mz2,mst12))*power5(mst22) - 360*msb12*mst12*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 144*msb12*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 144*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 144*msb12*power3(
     mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 144*msb12*power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power6(mst22)))/(24.*msb12*mst12*(
     mst12 - mst22)*mst22)) + xt*((2*cb*mst14*mt2*mu*power2(sa)*(mh2*power2(
     DeltaInv(mh2,mst12,mst12)) - mH2*power2(DeltaInv(mH2,mst12,mst12)))*
     power2(log(mst12/Q2)))/((mst12 - mst22)*sb) + (2*cb*mst24*mt2*mu*power2(
     sa)*(mH2*power2(DeltaInv(mH2,mst22,mst22)) - mh2*power2(DeltaInv(mst22,
     mst22,mh2)))*power2(log(mst22/Q2)))/((mst12 - mst22)*sb) + (2*cb*mt2*mu*
     sb*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))
     *(mC2*mst12 + msb12*mst12 + mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*
     mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*power3(
     mst12) + mC2*(6*msb12*mst14 + power3(mst12)))))/(msb12*(mst12 - mst22)) +
     (2*cb*mu*sb*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 + mst14 + DeltaInv(
     msb12,mst12,mC2)*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) +
     (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))))/
     msb12 + (2*mt2*mu*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))*power3(cb)*(mC2*mst12 + msb12*mst12 + mst14 + DeltaInv(
     msb12,mst12,mC2)*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) +
     (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))))/(
     msb12*(mst12 - mst22)*sb) + (2*mu*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*
     power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*power3(cb)*(mC2*mst12 +
     msb12*mst12 + mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*
     msb12*mst14 + power3(mst12)))))/(msb12*sb) + (2*cb*(mst12 - mst22 - mt2)*
     mu*sb*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2))*(-(mC2*mst22) - msb12*mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-((
     msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*
     power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)))))/(msb12*(-mst12 +
     mst22)) + (2*(mst12 - mst22 - 2*mt2)*mu*log(mC2/Q2)*log(mst22/Q2)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2))*power3(cb)*(-(mC2*mst22) - msb12*
     mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-((msb12*mst22 + mst24)*power2(
     mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*
     mst24 + power3(mst22)))))/(msb12*(-mst12 + mst22)*sb) - (2*cb*mu*sb*log(
     mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb12,
     mC2))*(mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*((
     msb12*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) + mC2*(6*msb12*mst24 + power3(mst22)))))/msb12 + (2*mt2*mu*
     log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*
     power3(cb)*(mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*((
     msb12*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) + mC2*(6*msb12*mst24 + power3(mst22)))))/(msb12*(mst12 -
     mst22)*sb) - (2*mu*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snt)*
     power2(DeltaInv(mst22,msb12,mC2))*power3(cb)*(mC2*mst22 + msb12*mst22 +
     mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 + mst24)*power2(mC2) -
     mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 +
     power3(mst22)))))/(msb12*sb) - (cb*mu*sb*log(mA2/Q2)*log(mst12/Q2)*power2(
     mA2)*power2(DeltaInv(mst22,mA2,mst12))*(3*mA2*mst22 + mst12*mst22 + mst24
     + DeltaInv(mst22,mA2,mst12)*(5*(mst12*mst22 + mst24)*power2(mA2) - 3*
     mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) - mA2*(-10*mst12*mst24 +
     power3(mst22)))))/mst12 - (mu*log(mA2/Q2)*log(mst12/Q2)*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12))*power3(cb)*(3*mA2*mst22 + mst12*mst22 +
     mst24 + DeltaInv(mst22,mA2,mst12)*(5*(mst12*mst22 + mst24)*power2(mA2) -
     3*mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) - mA2*(-10*mst12*mst24
     + power3(mst22)))))/(mst12*sb) + (4*cb*mu*log(mst12/Q2)*log(mst22/Q2)*
     power2(mh2)*power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(mh2*mst22 +
     mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)*((mst12*mst22 - mst24)*
     power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*
     mh2*(2*mst12*mst24 + power3(mst22)))))/(mst12*sb) + (cb*mu*sb*log(mA2/Q2)*
     log(mst22/Q2)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*(mA2*mst22 +
     mst12*mst22 + mst24 + DeltaInv(mst22,mA2,mst12)*((mst12*mst22 + mst24)*
     power2(mA2) - mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) + mA2*(6*
     mst12*mst24 + power3(mst22)))))/mst12 + (mu*log(mA2/Q2)*log(mst22/Q2)*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*power3(cb)*(mA2*mst22 +
     mst12*mst22 + mst24 + DeltaInv(mst22,mA2,mst12)*((mst12*mst22 + mst24)*
     power2(mA2) - mst22*power3(mA2) + (mst12 - mst22)*power3(mst22) + mA2*(6*
     mst12*mst24 + power3(mst22)))))/(mst12*sb) + (cb*mu*log(mst12/Q2)*log(
     mst22/Q2)*power2(sa)*(-((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12))) + power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12))*(mh2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)
     *((mst12*mst22 - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*
     mst22)*power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(
     mH2)*((-(mst12*mst22) + mst24)*power2(mH2) + mst22*power3(mH2) + (-5*mst12
     + 3*mst22)*power3(mst22) - 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12))))/(mst12*sb) + (4*ca*mu*sa*log(mst12/Q2)*log(
     mst22/Q2)*power2(snt)*(-((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12))) + power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12))*(mh2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)
     *((mst12*mst22 - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*
     mst22)*power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(
     mH2)*((-(mst12*mst22) + mst24)*power2(mH2) + mst22*power3(mH2) + (-5*mst12
     + 3*mst22)*power3(mst22) - 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12))))/mst12 + (ca*mu*sa*log(mst12/Q2)*log(mst22/Q2)
     *((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-(mh2*
     mst22) - mst12*mst22 - 3*mst24 + DeltaInv(mst22,mh2,mst12)*((-(mst12*
     mst22) + mst24)*power2(mh2) + mst22*power3(mh2) + (-5*mst12 + 3*mst22)*
     power3(mst22) - 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(mH2)*((
     mst12*mst22 - mst24)*power2(mH2) - mst22*power3(mH2) + (5*mst12 - 3*mst22)
     *power3(mst22) + 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mH2,mst22,mst12))))/mst12 + (4*cb*mu*log(mst12/Q2)*log(mst22/Q2)*power2(
     sa)*power2(snt)*((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     *(-(mh2*mst22) - mst12*mst22 - 3*mst24 + DeltaInv(mst22,mh2,mst12)*((-(
     mst12*mst22) + mst24)*power2(mh2) + mst22*power3(mh2) + (-5*mst12 + 3*
     mst22)*power3(mst22) - 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(
     mH2)*((mst12*mst22 - mst24)*power2(mH2) - mst22*power3(mH2) + (5*mst12 -
     3*mst22)*power3(mst22) + 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12))))/(mst12*sb) - (cb*mu*power2(sa)*power2(log(
     mst12/Q2))*(4*mh2*mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)) - 4*
     mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,mst12,mst12)) + (mst12 - mst22)*((
     2*mH2*mst22 + mst12*mst22 + 2*mst24)*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-2*mh2*
     mst22 - mst12*mst22 - 2*mst24 + DeltaInv(mst22,mh2,mst12)*(-((3*mst12*
     mst22 + 2*mst24)*power2(mh2)) + 2*mst22*power3(mh2) + (-3*mst12 + 2*mst22)
     *power3(mst22) - 2*mh2*(5*mst12*mst24 + power3(mst22)))) + power2(mH2)*((
     3*mst12*mst22 + 2*mst24)*power2(mH2) - 2*mst22*power3(mH2) + (3*mst12 - 2*
     mst22)*power3(mst22) + 2*mH2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)))))/(mst12*(mst12 - mst22)*sb) + (ca*mu*sa*
     power2(log(mst12/Q2))*(2*mh2*mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,
     mst12)) - 2*mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,mst12,mst12)) + (mst12
     - mst22)*((2*mH2*mst22 + mst12*mst22 + 2*mst24)*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     *(-2*mh2*mst22 - mst12*mst22 - 2*mst24 + DeltaInv(mst22,mh2,mst12)*(-((3*
     mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*power3(mh2) + (-3*mst12 + 2*
     mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 + power3(mst22)))) + power2(
     mH2)*((3*mst12*mst22 + 2*mst24)*power2(mH2) - 2*mst22*power3(mH2) + (3*
     mst12 - 2*mst22)*power3(mst22) + 2*mH2*(5*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)))))/(mst12*(mst12 - mst22)) + (2*cb*mu*
     sb*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*((3*mC2*mst12 +
     msb12*mst12 + mst14)*mt2*power2(DeltaInv(msb12,mst12,mC2)) - (mst12 -
     mst22 - mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 - msb12*mst22
     - mst24 + DeltaInv(mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*power2(mC2)
     + 3*mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(-10*msb12*
     mst24 + power3(mst22)))) + mt2*(5*(msb12*mst12 + mst14)*power2(mC2) - 3*
     mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) - mC2*(-10*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12 - mst22)
     ) + (2*cb*mu*sb*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snt)*(-((3*
     mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 +
     DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24 +
     power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/msb12 - (2*cb*mu*sb*log(mC2/
     Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*power2(snt)*(-((3*mC2*mst12 +
     msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(
     mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,
     msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (
     msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (
     -5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 +
     mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/msb12 - (2*mt2*mu*log(mC2/Q2)*log(msb12/Q2)*
     power2(mC2)*power3(cb)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*
     mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) -
     mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*(mst12 - mst22)*sb) + (2*mt2*mu*log(mC2/Q2)*log(msb12/Q2)*power2(
     mC2)*power2(snb)*power3(cb)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*
     mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) -
     mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*(mst12 - mst22)*sb) + (2*mu*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*
     power2(snt)*power3(cb)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*
     mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) -
     mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*sb) - (2*mu*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*power3(cb)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*
     mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) -
     mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*sb) + (2*mu*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*
     power3(cb)*(2*(3*mC2*mst12 + msb12*mst12 + mst14)*mt2*power2(DeltaInv(
     msb12,mst12,mC2)) - (mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,
     mC2))*(-3*mC2*mst22 - msb12*mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-5*
     (msb12*mst22 + mst24)*power2(mC2) + 3*mst22*power3(mC2) + (-msb12 + mst22)
     *power3(mst22) + mC2*(-10*msb12*mst24 + power3(mst22)))) - 2*mt2*(-5*(
     msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*
     power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(msb12*(mst12 - mst22)*sb) + (2*cb*mu*sb*log(mC2/Q2)*
     log(msb12/Q2)*power2(mC2)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*mt2*
     power2(DeltaInv(msb12,mst12,mC2))) + (mst12 - mst22 - mt2)*power2(
     DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 - msb12*mst22 - mst24 + DeltaInv(
     mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*power2(mC2) + 3*mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(-10*msb12*mst24 + power3(
     mst22)))) + mt2*(-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12 - mst22)) + (2*
     mu*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power3(cb)*(-2*(3*mC2*mst12 +
     msb12*mst12 + mst14)*mt2*power2(DeltaInv(msb12,mst12,mC2)) + (mst12 -
     mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 - msb12*
     mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*
     power2(mC2) + 3*mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(
     -10*msb12*mst24 + power3(mst22)))) + 2*mt2*(-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*(mst12 - mst22)*sb) + (2*cb*mt2*mu*sb*log(mC2/Q2)*log(mst12/Q2)*
     power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*
     power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(
     6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*msb22*(mst12 - mst22)) + (2*cb*mu*sb*log(mC2/Q2)*log(mst12/Q2)*
     power2(mC2)*power2(snb)*power2(snt)*(-(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22) + (4*mt2*mu*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*
     power2(snb)*power3(cb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*
     power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(
     6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*msb22*(mst12 - mst22)*sb) + (2*mu*log(mC2/Q2)*log(mst12/Q2)*power2(
     mC2)*power2(snb)*power2(snt)*power3(cb)*(-(msb22*(mC2*mst12 + msb12*mst12
     + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22*sb) + (2*mt2*mu*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)
     *power2(snb)*power3(cb)*(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-(
     mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(msb22,mst12,mC2)*(-((msb22*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb22 + mst12)*power3(
     mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*((msb12*mst12 +
     mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) +
     mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mu*sb*log(mC2/Q2)*log(msb22/Q2)*
     power2(mC2)*power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)
     *power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(
     3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12)
     + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     ))/msb22 - (2*mt2*mu*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(snb)*
     power3(cb)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*
     mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(
     mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*
     msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) +
     3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12
     - mst22)*sb) + (2*mu*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*power3(cb)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*
     mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) -
     mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(
     -10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(
     msb22*sb) + (2*cb*mu*sb*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(snb)*
     (-((3*mC2*mst12 + msb22*mst12 + mst14)*mt2*power2(DeltaInv(msb22,mst12,
     mC2))) + (mst12 - mst22 - mt2)*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*
     mst22 - msb22*mst22 - mst24 + DeltaInv(mst22,msb22,mC2)*(-5*(msb22*mst22 +
     mst24)*power2(mC2) + 3*mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22)
     + mC2*(-10*msb22*mst24 + power3(mst22)))) + mt2*(-5*(msb22*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(
     -10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(
     msb22*(mst12 - mst22)) + (2*mu*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*
     power2(snb)*power3(cb)*(-2*(3*mC2*mst12 + msb22*mst12 + mst14)*mt2*power2(
     DeltaInv(msb22,mst12,mC2)) + (mst12 - mst22 - 2*mt2)*power2(DeltaInv(
     mst22,msb22,mC2))*(-3*mC2*mst22 - msb22*mst22 - mst24 + DeltaInv(mst22,
     msb22,mC2)*(-5*(msb22*mst22 + mst24)*power2(mC2) + 3*mst22*power3(mC2) + (
     -msb22 + mst22)*power3(mst22) + mC2*(-10*msb22*mst24 + power3(mst22)))) +
     2*mt2*(-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-
     msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12 - mst22)*sb) + (2*ca*mu*
     sa*power2(snt)*power2(log(mst22/Q2))*(-2*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,
     mst22)) + 2*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,
     mst12)) + mh2*(2*mh2*mst24*power2(DeltaInv(mst22,mh2,mst12)) + mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 2*mh2*(mst24*power2(mh2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)))))/mst12 + (2*cb*mu*power2(sa)*power2(
     snt)*power2(log(mst22/Q2))*(2*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) + mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*power2(
     mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mh2*(-2*mh2*
     mst24*power2(DeltaInv(mst22,mh2,mst12)) - mst12*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) + 2*mh2*(mst24*power2(mh2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,
     mst12)))))/(mst12*sb) - (ca*mu*sa*power2(log(mst22/Q2))*((-mst12 + mst22)*
     mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 2*mH2*mst12*mst24*
     mt2*power2(DeltaInv(mH2,mst22,mst22)) + (mst12 - mst22)*power2(mH2)*(
     mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mh2*(mh2*(mst12 -
     mst22)*mst24*power2(DeltaInv(mst22,mh2,mst12)) + 2*mst12*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) - mh2*(mst12 - mst22)*(mst24*power2(mh2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)))))/(mst12*(mst12 - mst22)) + (cb*mu*
     power2(sa)*power2(log(mst22/Q2))*((-mst12 + mst22)*mst24*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) - 4*mH2*mst12*mst24*mt2*power2(DeltaInv(
     mH2,mst22,mst22)) + (mst12 - mst22)*power2(mH2)*(mst24*power2(mH2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mh2*(mh2*(mst12 - mst22)*mst24*power2(
     DeltaInv(mst22,mh2,mst12)) + 4*mst12*mst24*mt2*power2(DeltaInv(mst22,
     mst22,mh2)) - mh2*(mst12 - mst22)*(mst24*power2(mh2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mh2,mst12)))))/(mst12*(mst12 - mst22)*sb) + (2*cb*(mst12 - mst22 -
     mt2)*mu*sb*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*(-(msb22*(
     mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*
     (mst12 - mst22 - 2*mt2)*mu*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(
     snb)*power3(cb)*(-(msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(
     DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(
     mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22
     + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) +
     mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*(-((msb12*mst22 + mst24)*
     power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(
     6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(
     msb12*msb22*(mst12 - mst22)*sb) + (2*mt2*mu*log(mC2/Q2)*log(mst22/Q2)*
     power2(mC2)*power2(snb)*power3(cb)*(-(msb22*(mC2*mst22 + msb12*mst22 +
     mst24)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(
     (msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*
     power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*(-((msb12*
     mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(
     mst22) - mC2*(6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mu*sb*log(mC2/Q2)*log(
     mst22/Q2)*power2(mC2)*power2(snb)*power2(snt)*(msb22*(mC2*mst22 + msb12*
     mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(DeltaInv(
     mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 + DeltaInv(mst22,
     msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-
     msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 + power3(mst22)))) +
     msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb12 -
     mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (2*mu*log(mC2/Q2)*log(mst22/
     Q2)*power2(mC2)*power2(snb)*power2(snt)*power3(cb)*(msb22*(mC2*mst22 +
     msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 + DeltaInv(
     mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2)
     + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 + power3(mst22)))) +
     msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb12 -
     mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*sb) + (2*cb*mt2*mu*sb*log(msb12/
     Q2)*log(mst12/Q2)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)
     *power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb12*(mst12 - mst22)) +
     (2*cb*mu*sb*log(msb12/Q2)*log(mst12/Q2)*power2(snt)*(-((mC2*mst12 + msb12*
     mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(
     mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     *(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*
     mst14*mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(
     mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/
     msb12 + (4*mt2*mu*log(msb12/Q2)*log(mst12/Q2)*power3(cb)*(-((mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-
     5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(
     10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*
     msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(
     mst12)))))/(msb12*(mst12 - mst22)*sb) + (2*mt2*mu*log(msb12/Q2)*log(mst12/
     Q2)*power2(snb)*power3(cb)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(
     mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*mst12) +
     mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(
     mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 +
     msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*
     mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb12*(mst12
     - mst22)*sb) + (2*mu*log(msb12/Q2)*log(mst12/Q2)*power2(snt)*power3(cb)*(-
     ((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) - mst14*
     power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(
     mw2) - 3*power4(mst12)))))/(msb12*sb) + (2*cb*mt2*mu*sb*log(msb22/Q2)*log(
     mst12/Q2)*power2(snb)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)
     *power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb22*(mst12 - mst22)) +
     (2*cb*mu*sb*log(msb22/Q2)*log(mst12/Q2)*power2(snb)*power2(snt)*(-((mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(
     mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) +
     5*msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*
     power4(mst12)))))/msb22 + (4*mt2*mu*log(msb22/Q2)*log(mst12/Q2)*power2(
     snb)*power3(cb)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*
     power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/(msb22*(mst12 - mst22)*
     sb) + (2*mu*log(msb22/Q2)*log(mst12/Q2)*power2(snb)*power2(snt)*power3(cb)
     *(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*
     power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(
     mw2) - 3*power4(mst12)))))/(msb22*sb) + (2*cb*mu*sb*power2(snt)*power2(
     log(mst12/Q2))*(-(mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14 + DeltaInv(mw2,msb12,mst12)*
     (-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(
     mst12)) - power4(mst12)))))/msb12 + (2*mu*power2(snt)*power2(log(mst12/Q2)
     )*power3(cb)*(-(mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14 + DeltaInv(mw2,msb12,mst12)*
     (-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(
     mst12)) - power4(mst12)))))/(msb12*sb) - (2*cb*mu*power2(snt)*power2(log(
     mst12/Q2))*(-(mh2*msb12*mst12*mst14*power2(DeltaInv(mh2,mst12,mst12))) -
     2*msb12*(2*mh2*mst22 + mst12*mst22 + 2*mst24)*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 2*msb12*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*
     power2(mh2)) + 2*mst22*power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) -
     2*mh2*(5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14 + DeltaInv(mw2,
     msb12,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*
     mw2 + power3(mst12)) - power4(mst12)))))/(msb12*mst12*sb) + (2*mt2*mu*
     power2(snb)*power2(log(mst12/Q2))*power3(cb)*(-(msb22*mst14*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst14*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + msb22*power2(mC2)*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - msb12*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12)))))/(msb12*
     msb22*(mst12 - mst22)*sb) - (2*cb*mt2*mu*log(msb12/Q2)*log(mst12/Q2)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*mw2*
     power3(mst12) + msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12))
     - mst12*power3(mw2) - 3*power4(mst12))))/(msb12*(mst12 - mst22)*sb) + (2*
     cb*mt2*mu*log(msb12/Q2)*log(mst12/Q2)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*
     power4(mst12))))/(msb12*(mst12 - mst22)*sb) - (2*cb*mu*log(msb12/Q2)*log(
     mst12/Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(-(mst14*
     power2(mw2)) + 5*mw2*power3(mst12) + msb12*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/(msb12*
     sb) + (2*cb*mu*log(msb12/Q2)*log(mst12/Q2)*power2(mw2)*power2(snb)*power2(
     snt)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2
     + DeltaInv(mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) +
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(
     mw2) - 3*power4(mst12))))/(msb12*sb) - (2*cb*mt2*mu*log(msb22/Q2)*log(
     mst12/Q2)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 5*mw2*power3(mst12) + msb22*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/(msb22*(
     mst12 - mst22)*sb) - (2*cb*mu*log(msb22/Q2)*log(mst12/Q2)*power2(mw2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 5*
     mw2*power3(mst12) + msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(
     mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/(msb22*sb) + (4*cb*mt2*
     mu*power2(mw2)*power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12))))/(msb12*
     msb22*(mst12 - mst22)*sb) + (2*cb*mu*power2(mw2)*power2(snb)*power2(snt)*
     power2(log(mst12/Q2))*(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12)) -
     msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)
     *(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 +
     power3(mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))
     *(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 +
     power3(mst12)) - power4(mst12))))/(msb12*msb22*sb) + (2*cb*mt2*mu*log(
     mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*(mst12 -
     mst22)*sb) - (2*cb*mt2*mu*sb*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*
     mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(
     mst12))))/(msb12*(mst12 - mst22)) + (2*cb*mu*log(mst12/Q2)*log(mw2/Q2)*
     power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))))/(msb12*sb) - (2*cb*mu*sb*log(mst12/
     Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*
     (msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*
     power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/msb12 - (2*mt2*mu*
     log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(cb)*(msb12*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(
     mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(
     mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*(mst12
     - mst22)*sb) - (2*mu*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(cb)*(msb12*mst12 + mst14 + mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) +
     msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(
     mw2) - power4(mst12))))/(msb12*sb) - (2*cb*mt2*mu*log(mst12/Q2)*log(mw2/
     Q2)*power2(mw2)*power2(snb)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12)
     )*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*
     power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*
     (6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mt2*mu*sb*log(
     mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*(msb22*(msb12*mst12 + mst14
     + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))) +
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(
     mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*
     power3(mw2) - power4(mst12))))/(msb12*msb22*(mst12 - mst22)) - (2*cb*mu*
     log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*(msb22*(
     msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12)) -
     msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb22*(
     6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*msb22*sb) + (
     2*cb*mu*sb*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*(
     msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12))
     - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2
     + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) +
     msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(
     mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*
     power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*msb22) + (2*
     mt2*mu*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power3(cb)*(
     msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12))
     - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2
     + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(mst12) +
     msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(
     mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*
     power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/(msb12*msb22*(mst12
     - mst22)*sb) + (2*mu*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power2(snt)*power3(cb)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*
     (6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))))/(msb12*msb22*sb) + (4*cb*mt2*mu*log(mh2/Q2)*log(mst12/Q2)
     *power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12) + 16*mh2*
     DeltaInv(mh2,mst12,mst12)*power4(mst12)))/((mst12 - mst22)*sb) - (4*cb*
     mt2*mu*log(mh2/Q2)*log(mst12/Q2)*power2(sa)*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 4*power3(mst12) + 16*mh2*DeltaInv(mh2,mst12,mst12)*
     power4(mst12)))/((mst12 - mst22)*sb) + (4*cb*mt2*mu*log(mH2/Q2)*log(mst12/
     Q2)*power2(sa)*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(
     mst12) + 16*mH2*DeltaInv(mH2,mst12,mst12)*power4(mst12)))/((mst12 - mst22)
     *sb) + (2*cb*mt2*mu*power2(mw2)*power2(snb)*power2(log(mst12/Q2))*(-(
     msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2))
     + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*
     mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))
     /(msb12*msb22*(mst12 - mst22)*sb) + (cb*mt2*mu*log(mst12/Q2)*power2(sa)*(-
     ((mh2 - 6*mst12)*DeltaInv(mh2,mst12,mst12)) + 24*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) + 288*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power4(mst12) + DeltaInv(mH2,mst12,mst12)*(mH2 - 6*mst12 -
     24*DeltaInv(mH2,mst12,mst12)*(mH2*mst14 + 3*power3(mst12)) - 288*mH2*
     power2(DeltaInv(mH2,mst12,mst12))*power4(mst12))))/(3.*(mst12 - mst22)*sb)
     - (2*cb*mt2*mu*power2(log(mst12/Q2))*(mh2*msb12*mst14*power2(DeltaInv(mh2,
     mst12,mst12)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-mst14 +
     DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*
     msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(msb12*(mst12 -
     mst22)*sb) + (2*mt2*mu*power2(log(mst12/Q2))*power3(cb)*(mst14*power2(mC2)
     *power2(DeltaInv(msb12,mst12,mC2)) - power2(mC2)*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(-mst14 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(
     msb12*(mst12 - mst22)*sb) + (cb*mu*sb*log(mst12/Q2)*power2(snt)*((5*mC2*
     mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - 6*mC2*power2(
     DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*
     mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - DeltaInv(mw2,msb12,mst12)*(-(
     msb12*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb12,mst12)*(
     mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) + power3(mst12)) - 72*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(
     6.*msb12) + (mu*log(mst12/Q2)*power2(snt)*power3(cb)*((5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - 6*mC2*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*(mst14*power2(mC2) + (
     -2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - DeltaInv(mw2,msb12,mst12)*(-(msb12*
     mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb12,mst12)*(mst12*mw2*
     (-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) + power3(mst12)) - 72*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(
     mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(6.*
     msb12*sb) + (mt2*mu*log(mst12/Q2)*power3(cb)*(-((5*mC2*mst12 - msb12*mst12
     + mst14)*DeltaInv(msb12,mst12,mC2)) + 6*mC2*power2(DeltaInv(msb12,mst12,
     mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2)
     + power3(mst12)) + 72*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*
     power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + DeltaInv(mw2,msb12,mst12)*(-(msb12*mst12) + mst14 + 5*
     mst12*mw2 - 6*mw2*DeltaInv(mw2,msb12,mst12)*(mst12*mw2*(-3*msb12 + mw2) -
     mst14*(msb12 + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(
     mst14*mw2 + power3(mst12)) + power4(mst12)))))/(6.*msb12*(mst12 - mst22)*
     sb) - (cb*mt2*mu*log(mst12/Q2)*(-2*msb12*(mh2 - 6*mst12)*DeltaInv(mh2,
     mst12,mst12) + 48*msb12*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*
     power3(mst12)) + 576*mh2*msb12*power3(DeltaInv(mh2,mst12,mst12))*power4(
     mst12) + DeltaInv(mw2,msb12,mst12)*(-(msb12*mst12) + mst14 + 5*mst12*mw2 -
     6*mw2*DeltaInv(mw2,msb12,mst12)*(mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12
     + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 +
     power3(mst12)) + power4(mst12)))))/(6.*msb12*(mst12 - mst22)*sb) + (2*cb*
     mt2*mu*sb*power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12)
     - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(msb12*msb22*(
     mst12 - mst22)) + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(mst12/Q2)
     )*(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*
     mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(
     msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2))
     + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*
     mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))
     )/(msb12*msb22) + (4*mt2*mu*power2(snb)*power2(log(mst12/Q2))*power3(cb)*(
     msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(
     msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2))
     + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(
     mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*
     mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))
     )/(msb12*msb22*(mst12 - mst22)*sb) + (2*mu*power2(snb)*power2(snt)*power2(
     log(mst12/Q2))*power3(cb)*(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) -
     2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(-(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(
     mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(msb12*msb22*sb) - (cb*mt2*mu*log(mst12/Q2)*
     power2(snb)*(msb22*(-(msb12*mst12) + mst14 + 5*mst12*mw2)*DeltaInv(mw2,
     msb12,mst12) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(mst12*mw2*(-
     3*msb12 + mw2) - mst14*(msb12 + 14*mw2) + power3(mst12)) - 72*msb22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)) -
     msb12*DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*
     mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 +
     14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     )*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(6.*msb12*msb22*(mst12 - mst22)*sb) - (cb*mu*
     log(mst12/Q2)*power2(snb)*power2(snt)*(msb22*(-(msb12*mst12) + mst14 + 5*
     mst12*mw2)*DeltaInv(mw2,msb12,mst12) - 6*msb22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*(mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) +
     power3(mst12)) - 72*msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)) - msb12*DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12)
     + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*
     msb22 + mw2) - mst14*(msb22 + 14*mw2) + power3(mst12)) - 72*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12)
     - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(6.*msb12*msb22*
     sb) + (2*cb*mt2*mu*sb*log(msb12/Q2)*log(mst12/Q2)*power2(snb)*((mC2*mst12
     + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) +
     power2(mC2)*((msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*
     msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(-(msb12*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,mst12)*
     (mst14*power2(mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 + mst12*
     power2(mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/(
     msb12*(mst12 - mst22)) + (2*cb*mu*sb*log(msb12/Q2)*log(mst12/Q2)*power2(
     snb)*power2(snt)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb12 + (2*mt2*mu*log(msb12/Q2)*log(mst12/Q2)*
     power3(cb)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/(msb12*(mst12 - mst22)*sb) + (4*mt2*mu*log(
     msb12/Q2)*log(mst12/Q2)*power2(snb)*power3(cb)*((mC2*mst12 + msb12*mst12 +
     3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((
     msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)
     *power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(
     msb12*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*
     power2(mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/(msb12*(
     mst12 - mst22)*sb) + (2*mu*log(msb12/Q2)*log(mst12/Q2)*power2(snb)*power2(
     snt)*power3(cb)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/(msb12*sb) + (2*mt2*mu*log(msb22/Q2)*log(mst12/
     Q2)*power2(snb)*power3(cb)*((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((msb22*mst12 -
     mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12)
     + 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*mst14
     - mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*power3(
     mst12) - msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) +
     mst12*power3(mw2) + 3*power4(mst12)))))/(msb22*(mst12 - mst22)*sb) + (2*
     cb*mu*power2(snt)*power2(log(mst22/Q2))*(2*msb12*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 2*msb12*power2(mh2)*(mst24*power2(mh2) + (-2*
     mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)) + mst12*(mh2*msb12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(mst24 + DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(mst22))))))/(msb12*
     mst12*sb) + (cb*mu*sb*power2(Pi)*power2(snt)*(-((3*mC2*mst12 + msb12*mst12
     + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (3*mC2*mst22 +
     msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(
     mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((5*msb12*
     mst22 + 3*mst24)*power2(mC2) - 3*mst22*power3(mC2) + (5*msb12 - 3*mst22)*
     power3(mst22) + mC2*(14*msb12*mst24 + 3*power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + power2(mw2)*(-((msb12*mst22 + 3*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(14*msb12*
     mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*
     power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(
     mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*
     msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*
     mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(3.*msb12) +
     (mu*power2(Pi)*power2(snt)*power3(cb)*(-((3*mC2*mst12 + msb12*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (3*mC2*mst22 +
     msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(
     mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((5*msb12*
     mst22 + 3*mst24)*power2(mC2) - 3*mst22*power3(mC2) + (5*msb12 - 3*mst22)*
     power3(mst22) + mC2*(14*msb12*mst24 + 3*power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + power2(mw2)*(-((msb12*mst22 + 3*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(14*msb12*
     mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*
     power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(
     mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*
     msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*
     mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(3.*msb12*
     sb) + (mt2*mu*power2(Pi)*power3(cb)*((3*mC2*mst12 + msb12*mst12 + 3*mst14)
     *power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - (3*mC2*mst22 + msb12*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(
     mC2)*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*
     msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*(-((5*msb12*mst22 + 3*
     mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(
     mst22) - mC2*(14*msb12*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*((msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*
     mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(
     mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(3.*msb12*(mst12 -
     mst22)*sb) + (cb*mu*sb*power2(snt)*(mC2*msb12*DeltaInv(msb12,mst12,mC2) -
     mC2*msb12*DeltaInv(mst22,msb12,mC2) - 42*(3*mC2*mst12 + msb12*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + 42*(3*mC2*mst22 +
     msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*
     power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(
     mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 42*power2(mC2)*(-((5*
     msb12*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb12 + 3*
     mst22)*power3(mst22) - mC2*(14*msb12*mst24 + 3*power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + mw2*(msb12*DeltaInv(mst22,mw2,msb12) - 42*
     mw2*(msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)
     ) + DeltaInv(mw2,msb12,mst12)*(-msb12 + 42*mw2*(msb12*mst12 + 3*mst14 + 3*
     mst12*mw2)*DeltaInv(mw2,msb12,mst12) + 42*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(
     mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) -
     3*power4(mst12))) - 42*mw2*power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*
     mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*
     power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(
     mst22)))))/(3.*msb12) + (mu*power2(snt)*power3(cb)*(mC2*msb12*DeltaInv(
     msb12,mst12,mC2) - mC2*msb12*DeltaInv(mst22,msb12,mC2) - 42*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + 42*
     (3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + 42*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) +
     3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 42*power2(
     mC2)*(-((5*msb12*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-
     5*msb12 + 3*mst22)*power3(mst22) - mC2*(14*msb12*mst24 + 3*power3(mst22)))
     *power3(DeltaInv(mst22,msb12,mC2)) + mw2*(msb12*DeltaInv(mst22,mw2,msb12)
     - 42*mw2*(msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb12)) + DeltaInv(mw2,msb12,mst12)*(-msb12 + 42*mw2*(msb12*mst12 + 3*
     mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb12,mst12) + 42*mw2*power2(DeltaInv(
     mw2,msb12,mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*
     mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*
     power3(mw2) - 3*power4(mst12))) - 42*mw2*power3(DeltaInv(mst22,mw2,msb12))
     *(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) +
     5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)))))/(3.*msb12*sb) + (mt2*mu*power3(cb)*(-(mC2*msb12*
     DeltaInv(msb12,mst12,mC2)) + mC2*msb12*DeltaInv(mst22,msb12,mC2) + 42*(3*
     mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - 42*(3*mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 42*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     + 42*power2(mC2)*(-((5*msb12*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*
     power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - mC2*(14*msb12*mst24 +
     3*power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + mw2*(-(msb12*
     DeltaInv(mst22,mw2,msb12)) + 42*mw2*(msb12*mst22 + 3*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb12)) - DeltaInv(mw2,msb12,mst12)*(-msb12 +
     42*mw2*(msb12*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb12,mst12) +
     42*mw2*power2(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*mw2 + 5*msb12*
     mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*
     power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) + 42*mw2*power3(
     DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2)
     + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*
     mst22*power3(mw2) - 3*power4(mst22)))))/(3.*msb12*(mst12 - mst22)*sb) + (
     2*cb*(mst12 - mst22 - mt2)*mu*sb*log(msb12/Q2)*log(mst22/Q2)*(-((mC2*mst22
     + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) +
     power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-
     5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(
     10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(
     mst22)))))/(msb12*(mst12 - mst22)) + (2*cb*(mst12 - mst22 - mt2)*mu*sb*
     log(msb12/Q2)*log(mst22/Q2)*power2(snb)*(-((mC2*mst22 + msb12*mst22 + 3*
     mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(
     msb12*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*
     mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*
     mst24*mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(
     mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(
     msb12*(-mst12 + mst22)) + (2*cb*mu*sb*log(msb12/Q2)*log(mst22/Q2)*power2(
     snb)*power2(snt)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)
     *power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/msb12 + (2*(mst12 - mst22
     - 2*mt2)*mu*log(msb12/Q2)*log(mst22/Q2)*power3(cb)*(-((mC2*mst22 + msb12*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) + power2(
     mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb12
     + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*
     mst24*mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(
     mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(
     msb12*(mst12 - mst22)*sb) + (2*mt2*mu*log(msb12/Q2)*log(mst22/Q2)*power3(
     cb)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) +
     mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 +
     DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) -
     mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*
     power3(mw2) - 3*power4(mst22)))))/(msb12*(mst12 - mst22)*sb) + (2*(mst12 -
     mst22 - 2*mt2)*mu*log(msb12/Q2)*log(mst22/Q2)*power2(snb)*power3(cb)*(-((
     mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*
     power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) - mst24*
     power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(
     mw2) - 3*power4(mst22)))))/(msb12*(-mst12 + mst22)*sb) + (2*mu*log(msb12/
     Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*power3(cb)*(-((mC2*mst22 +
     msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) +
     power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-
     5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(
     10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(
     mst22)))))/(msb12*sb) + (2*cb*(mst12 - mst22 - mt2)*mu*sb*log(msb22/Q2)*
     log(mst22/Q2)*power2(snb)*(-((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*((-(msb22*mst22) +
     mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(
     mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*mst24*mw2 +
     msb22*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(mst22) + 5*
     mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb22*(mst12
     - mst22)) + (2*(mst12 - mst22 - 2*mt2)*mu*log(msb22/Q2)*log(mst22/Q2)*
     power2(snb)*power3(cb)*(-((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*((-(msb22*mst22) + mst24)
     *power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*mst24*mw2 + msb22*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/(msb22*(mst12 - mst22)*
     sb) + (2*mt2*mu*log(msb22/Q2)*log(mst22/Q2)*power2(snb)*power3(cb)*(-((
     mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2))) + power2(mC2)*((-(msb22*mst22) + mst24)*power2(mC2) + mst22*
     power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb22)*(10*msb22*mst24*mw2 + msb22*mst22*power2(mw2) - mst24*
     power2(mw2) + 5*msb22*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(
     mw2) - 3*power4(mst22)))))/(msb22*(mst12 - mst22)*sb) + (cb*mu*sb*log(
     mst12/Q2)*log(mst22/Q2)*(-((mA2*mst22 + mst12*mst22 + 3*mst24)*power2(mA2)
     *power2(DeltaInv(mst22,mA2,mst12))) + power2(mA2)*((-(mst12*mst22) +
     mst24)*power2(mA2) + mst22*power3(mA2) + (-5*mst12 + 3*mst22)*power3(
     mst22) - 5*mA2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,
     mst12)) + power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + 3*
     mst24 + mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(10*mst12*mst24*mz2 + mst12*
     mst22*power2(mz2) - mst24*power2(mz2) + 5*mst12*power3(mst22) + 5*mz2*
     power3(mst22) - mst22*power3(mz2) - 3*power4(mst22)))))/mst12 + (mu*log(
     mst12/Q2)*log(mst22/Q2)*power3(cb)*(-((mA2*mst22 + mst12*mst22 + 3*mst24)*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))) + power2(mA2)*((-(mst12*
     mst22) + mst24)*power2(mA2) + mst22*power3(mA2) + (-5*mst12 + 3*mst22)*
     power3(mst22) - 5*mA2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mA2,mst12)) + power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*
     mst22 + 3*mst24 + mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(10*mst12*mst24*
     mz2 + mst12*mst22*power2(mz2) - mst24*power2(mz2) + 5*mst12*power3(mst22)
     + 5*mz2*power3(mst22) - mst22*power3(mz2) - 3*power4(mst22)))))/(mst12*sb)
     - (cb*mu*log(mst12/Q2)*log(mst22/Q2)*((mh2*mst22 + mst12*mst22 + 3*mst24)*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22
     - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)*power3(
     mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,
     mst12)) + power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + 3*
     mst24 + mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(10*mst12*mst24*mz2 + mst12*
     mst22*power2(mz2) - mst24*power2(mz2) + 5*mst12*power3(mst22) + 5*mz2*
     power3(mst22) - mst22*power3(mz2) - 3*power4(mst22)))))/(mst12*sb) + (2*
     cb*mu*sb*power2(snt)*power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 +
     2*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 +
     msb12*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((3*msb12*mst22 +
     2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) -
     power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/msb12 + (2*mt2*mu*
     power2(snb)*power2(log(msb12/Q2))*power3(cb)*(-((2*mC2*mst12 + msb12*mst12
     + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 +
     msb12*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((3*msb12*mst22 +
     2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) -
     power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22)*
     sb) + (2*mu*power2(snt)*power2(log(msb12/Q2))*power3(cb)*(-((2*mC2*mst12 +
     msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (
     2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((
     3*msb12*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*
     mst22)*power3(mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-((msb12*mst22 + 2*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(
     mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) -
     2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2
     + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22)
     + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*
     sb) + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(msb12/Q2))*((2*mC2*
     mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2)) - (2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) -
     2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*(-
     ((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12
     + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((msb12*mst22 + 2*(mst24 + mst22*
     mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12)
     )*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*
     msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 +
     3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/msb12 + (
     2*mt2*mu*power2(log(msb12/Q2))*power3(cb)*((2*mC2*mst12 + msb12*mst12 + 2*
     mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - (2*mC2*mst22 +
     msb12*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) +
     (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))
     *power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22)*
     sb) + (2*mu*power2(snb)*power2(snt)*power2(log(msb12/Q2))*power3(cb)*((2*
     mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - (2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((3*msb12*mst12 + 2*mst14)*
     power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(mst12) + 2*
     mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((msb12*mst22 +
     2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb12*sb) + (2*mu*power2(snb)*power2(log(msb12/Q2))*
     power3(cb)*(2*(2*mC2*mst12 + msb12*mst12 + 2*mst14)*mt2*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*
     (mst12 - mst22 - 2*mt2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 2*
     mt2*power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(
     mC2) + (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + (-mst12 + mst22 + 2*mt2)*
     power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-((mst12 - mst22
     - 2*mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb12))) - 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14
     + mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - (mst12 - mst22 -
     2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*
     mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*
     power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 -
     mst22)*sb) + (2*mu*power2(log(msb12/Q2))*power3(cb)*(-2*(2*mC2*mst12 +
     msb12*mst12 + 2*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) +
     (2*mC2*mst22 + msb12*mst22 + 2*mst24)*(-mst12 + mst22 + 2*mt2)*power2(mC2)
     *power2(DeltaInv(mst22,msb12,mC2)) + 2*mt2*power2(mC2)*(-((3*msb12*mst12 +
     2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + (mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + 2*(mst24
     + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,
     msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(
     mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*
     mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 - mst22)*sb) + (2*cb*mu*
     sb*power2(snb)*power2(log(msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*
     mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - (2*mC2*mst22 +
     msb12*mst22 + 2*mst24)*(-mst12 + mst22 + mt2)*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + mt2*power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2)
     - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + (-mst12
     + mst22 + mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-
     ((mst12 - mst22 - mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12))) - mt2*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*
     mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) - (mst12 - mst22 - mt2)*power3(DeltaInv(mst22,mw2,msb12))*(10*
     msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb12*(mst12 - mst22)) + (2*cb*mu*sb*power2(log(msb12/
     Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*mt2*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*(-
     mst12 + mst22 + mt2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + mt2*
     power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - (-mst12 + mst22 + mt2)*
     power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((mst12 - mst22 -
     mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb12)) + mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 +
     mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 -
     mt2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*
     mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*
     power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb12*(mst12 -
     mst22)) + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(msb22/Q2))*(-((2*
     mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) +
     (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))
     *power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-((msb22*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,
     msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*(10*
     msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (2*mu*power2(snb)*power2(snt)*power2(log(msb22/
     Q2))*power3(cb)*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)
     *power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) -
     2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-
     ((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)))
     + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb22*sb) + (2*mt2*mu*power2(snb)*power2(log(msb22/Q2))
     *power3(cb)*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((3*msb22*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*((
     msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) -
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb22*(mst12 - mst22)*sb) + (2*mu*power2(snb)*power2(
     log(msb22/Q2))*power3(cb)*(-2*(2*mC2*mst12 + msb22*mst12 + 2*mst14)*mt2*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + (2*mC2*mst22 + msb22*mst22
     + 2*mst24)*(-mst12 + mst22 + 2*mt2)*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) + 2*mt2*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2))
     + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*
     msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + (mst12 -
     mst22 - 2*mt2)*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*((
     mst12 - mst22 - 2*mt2)*(msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(10*
     msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/(msb22*(mst12 - mst22)*sb) + (2*cb*mu*sb*power2(snb)*
     power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*mt2*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*
     mst24)*(-mst12 + mst22 + mt2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)
     ) + mt2*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - (-mst12 + mst22 + mt2)
     *power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*((mst12 - mst22 -
     mt2)*(msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb22)) + mt2*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*(mst14 +
     mst12*mw2) + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 -
     mt2)*power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*
     mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*
     power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/(msb22*(mst12 -
     mst22)) + (2*mt2*mu*power2(log(mst22/Q2))*power3(cb)*(-(mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(mst24 + DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 2*mw2*
     power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(mst22)))))/(
     msb12*(mst12 - mst22)*sb) + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(
     log(mst22/Q2))*(-(msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)
     )) + msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb22*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - msb12*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(
     mw2)*(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(
     mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) -
     power4(mst22)))))/(msb12*msb22) + (2*mu*power2(snb)*power2(snt)*power2(
     log(mst22/Q2))*power3(cb)*(-(msb22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2))) + msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2)) + msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2)) - msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(
     mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,
     mC2)) + power2(mw2)*(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) -
     msb12*power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)
     *(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 +
     power3(mst22)) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))
     *(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 +
     power3(mst22)) - power4(mst22)))))/(msb12*msb22*sb) + (cb*mt2*mu*power2(
     mw2)*power2(Pi)*power2(snb)*(msb22*(msb12*mst22 + 3*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb12)) - msb12*(msb22*mst22 + 3*mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*power2(DeltaInv(mw2,msb12,
     mst12)) - 3*msb22*mst12*mw2*power2(DeltaInv(mw2,msb12,mst12)) + msb12*
     msb22*mst12*power2(DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst12*mw2*power2(DeltaInv(mw2,msb22,
     mst12)) - 14*msb12*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,mst12)) - 5*
     msb12*msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*
     mst14*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*msb22*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mw2*power3(
     mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst12*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 14*msb12*msb22*mst14*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) + 5*msb12*msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12)) + 5*msb12*msb22*power3(mst12)*power3(DeltaInv(mw2,msb22,
     mst12)) + 3*msb12*mw2*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 3*
     msb12*mst12*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 3*msb12*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + msb22*power3(DeltaInv(mst22,mw2,msb12))*
     (14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*
     mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(
     mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22))))/(
     3.*msb12*msb22*(mst12 - mst22)*sb) + (cb*mu*power2(mw2)*power2(Pi)*power2(
     snb)*power2(snt)*(-(msb22*(msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12))) + msb12*(msb22*mst22 + 3*mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + msb12*msb22*mst12*power2(DeltaInv(mw2,
     msb12,mst12)) + 3*msb22*mst14*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*
     mst12*mw2*power2(DeltaInv(mw2,msb12,mst12)) - msb12*msb22*mst12*power2(
     DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*power2(DeltaInv(mw2,msb22,
     mst12)) - 3*msb12*mst12*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 14*msb12*
     msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*power3(mst12)*power3(
     DeltaInv(mw2,msb12,mst12)) + 3*msb22*mw2*power3(mst12)*power3(DeltaInv(
     mw2,msb12,mst12)) - 3*msb22*mst12*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 14*msb12*msb22*mst14*mw2*power3(DeltaInv(mw2,msb22,mst12)) - 5*
     msb12*msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*
     mst14*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mw2*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst12*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 3*msb22*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 3*msb12*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mst12) - msb22*power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*
     msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*
     mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) + msb12*power3(
     DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2)
     + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*
     mst22*power3(mw2) - 3*power4(mst22))))/(3.*msb12*msb22*sb) + (cb*mu*
     power2(mw2)*power2(Pi)*power2(snb)*(msb22*(mst12 - mst22 - 2*mt2)*(msb12*
     mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - msb12*(
     mst12 - mst22 - 2*mt2)*(msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) + 2*mt2*(msb22*(msb12*mst12 + 3*(mst14 + mst12*
     mw2))*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*
     msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*
     mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(
     mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) +
     msb22*(mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*
     mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*
     power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(
     mst22)) - msb12*(mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*
     (14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22))))/(3.*msb12*msb22*(mst12 - mst22)*sb) + (2*cb*(mst12 -
     mst22 - 2*mt2)*mu*log(msb12/Q2)*log(mst22/Q2)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,
     msb12)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*mst24*mw2 +
     mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22)
     )))/(msb12*(-mst12 + mst22)*sb) - (2*cb*mt2*mu*log(msb12/Q2)*log(mst22/Q2)
     *power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*
     power3(mst22) + msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22))
     - mst22*power3(mw2) - 3*power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*
     cb*(mst12 - mst22 - 2*mt2)*mu*log(msb12/Q2)*log(mst22/Q2)*power2(mw2)*
     power2(snb)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*
     power3(mst22) + msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22))
     - mst22*power3(mw2) - 3*power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*
     cb*mt2*mu*log(msb12/Q2)*log(mst22/Q2)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*
     power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*cb*mu*log(msb12/Q2)*log(
     mst22/Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(
     msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(-(mst24*
     power2(mw2)) + 5*mw2*power3(mst22) + msb12*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/(msb12*
     sb) - (2*cb*mu*log(msb12/Q2)*log(mst22/Q2)*power2(mw2)*power2(snb)*power2(
     snt)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2
     + DeltaInv(mst22,mw2,msb12)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) +
     msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(
     mw2) - 3*power4(mst22))))/(msb12*sb) + (2*cb*(mst12 - mst22 - 2*mt2)*mu*
     log(msb22/Q2)*log(mst22/Q2)*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,
     mw2,msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)
     *(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) + msb22*(10*mst24*mw2 +
     mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(mw2) - 3*power4(mst22)
     )))/(msb22*(-mst12 + mst22)*sb) - (2*cb*mt2*mu*log(msb22/Q2)*log(mst22/Q2)
     *power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 5*
     mw2*power3(mst22) + msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(
     mst22)) - mst22*power3(mw2) - 3*power4(mst22))))/(msb22*(mst12 - mst22)*
     sb) + (2*cb*mu*log(msb22/Q2)*log(mst22/Q2)*power2(mw2)*power2(snb)*power2(
     snt)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2
     + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 5*mw2*power3(mst22) +
     msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) - mst22*power3(
     mw2) - 3*power4(mst22))))/(msb22*sb) + (2*cb*mt2*mu*power2(mw2)*power2(
     log(msb12/Q2))*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*
     mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*cb*
     mt2*mu*power2(mw2)*power2(snb)*power2(log(msb12/Q2))*((msb12*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,
     msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(10*
     msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*cb*mu*power2(mw2)*power2(
     snt)*power2(log(msb12/Q2))*((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*
     power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(
     mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb12*sb) - (2*cb*mu*
     power2(mw2)*power2(snb)*power2(snt)*power2(log(msb12/Q2))*((msb12*mst22 +
     2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(msb12*sb) - (2*cb*mu*power2(mw2)*power2(log(msb12/Q2))*(
     (mst12 - mst22 - 2*mt2)*(msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*
     mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(10*
     msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*cb*mu*power2(mw2)*power2(
     snb)*power2(log(msb12/Q2))*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 + mst12*mw2) +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(msb12*(mst12 - mst22)*sb) - (cb*
     mt2*mu*log(msb12/Q2)*(-((mst12 - mst22)*(msb12 + mw2)) - mst22*DeltaInv(
     mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)
     ) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + mst12*DeltaInv(mst22,
     mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(mst22) + power2(mw2)) +
     mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*mw2) -
     msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) - 6*mst12*
     mst22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2 + mst22*mw2*(6*
     msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) + power3(mst22) + power3(
     mw2)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(10*
     msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(
     mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) -
     2*power4(mst22))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (cb*mt2*mu*
     log(msb12/Q2)*power2(snb)*(-((mst12 - mst22)*(msb12 + mw2)) - mst22*
     DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + mst12*
     DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(mst22) +
     power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*
     msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(
     mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2
     + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) + power3(
     mst22) + power3(mw2)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)) - 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(6.*msb12*mst12*(mst12 - mst22)*
     mst22*sb) + (cb*mu*log(msb12/Q2)*power2(snt)*(-((mst12 - mst22)*(msb12 +
     mw2)) - mst22*DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(
     power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)
     ) + mst12*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(
     mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) + 6*
     mst12*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*
     mw2*(6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) +
     power3(mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*
     mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) +
     power3(mst22) + power3(mw2)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*
     mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)) - 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(6.*msb12*mst12*mst22*sb) + (cb*mu*
     log(msb12/Q2)*power2(snb)*power2(snt)*((mst12 - mst22)*(msb12 + mw2) +
     mst22*DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(
     mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) -
     mst12*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(
     mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) - 6*
     mst12*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*
     mw2*(6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) +
     power3(mw2)) + 6*mst12*mst22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(23*
     mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2)) +
     power3(mst22) + power3(mw2)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*
     mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)) + 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22))))/(6.*msb12*mst12*mst22*sb) + (cb*mu*
     log(msb12/Q2)*power2(snb)*(-((mst12 - mst22)*(msb12*(mst12 - 2*mt2) - 2*
     mt2*mw2 + mst12*(mst22 + mw2))) + 2*mst22*mt2*DeltaInv(mw2,msb12,mst12)*(
     5*mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)) + mst12*(mst14 +
     5*power2(mw2)) + power3(mw2)) + mst12*(mst12 - mst22 - 2*mt2)*DeltaInv(
     mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(power2(mst22) + power2(mw2)
     ) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)) - 12*mst12*mst22*mt2*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*
     mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) - 6*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     (23*mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) - msb12*(mst24 + power2(mw2))
     + power3(mst22) + power3(mw2)) - 144*mst12*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12)) - 72*mst12*mst22*(mst12 - mst22 - 2*
     mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 +
     3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(6.*msb12*
     mst12*(mst12 - mst22)*mst22*sb) + (cb*mu*log(msb12/Q2)*((mst12 - mst22)*(
     msb12*(mst12 - 2*mt2) - 2*mt2*mw2 + mst12*(mst22 + mw2)) - 2*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) - mst12*(mst12
     - mst22 - 2*mt2)*DeltaInv(mst22,mw2,msb12)*(5*mw2*power2(mst22) - msb12*(
     power2(mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)
     ) + 12*mst12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2
     + mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(
     mst12) + power3(mw2)) + 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*(23*mst24*mw2 + mst22*mw2*(6*msb12 + 23*mw2) -
     msb12*(mst24 + power2(mw2)) + power3(mst22) + power3(mw2)) + 144*mst12*
     mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*
     mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) +
     72*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*
     power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb)
     + (2*cb*mt2*mu*power2(mw2)*power2(snb)*power2(log(msb22/Q2))*(-((msb22*
     mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22))) +
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22))))/(msb22*(mst12 - mst22)*sb) + (2*cb*mu*power2(mw2)*power2(
     snb)*power2(snt)*power2(log(msb22/Q2))*((msb22*mst22 + 2*(mst24 + mst22*
     mw2))*power2(DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12)
     )*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*
     msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 +
     3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb22*sb)
     - (2*cb*mu*power2(mw2)*power2(snb)*power2(log(msb22/Q2))*((mst12 - mst22 -
     2*mt2)*(msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*(mst14
     + mst12*mw2) + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + (mst12 - mst22 -
     2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*
     mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*
     power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(msb22*(mst12 -
     mst22)*sb) - (cb*mt2*mu*log(msb22/Q2)*power2(snb)*(-((mst12 - mst22)*(
     msb22 + mw2)) - mst22*DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) -
     msb22*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) +
     power3(mw2)) + mst12*DeltaInv(mst22,mw2,msb22)*(5*mw2*power2(mst22) -
     msb22*(power2(mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) +
     power3(mw2)) + 6*mst12*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*
     mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) +
     power3(mst12) + power3(mw2)) - 6*mst12*mst22*mw2*power2(DeltaInv(mst22,
     mw2,msb22))*(23*mst24*mw2 + mst22*mw2*(6*msb22 + 23*mw2) - msb22*(mst24 +
     power2(mw2)) + power3(mst22) + power3(mw2)) + 72*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) - 72*mst12*mst22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*
     mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*
     power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(6.*msb22*mst12*(
     mst12 - mst22)*mst22*sb) + (cb*mu*log(msb22/Q2)*power2(snb)*power2(snt)*(-
     ((mst12 - mst22)*(msb22 + mw2)) - mst22*DeltaInv(mw2,msb22,mst12)*(5*mw2*
     power2(mst12) - msb22*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*
     power2(mw2)) + power3(mw2)) + mst12*DeltaInv(mst22,mw2,msb22)*(5*mw2*
     power2(mst22) - msb22*(power2(mst22) + power2(mw2)) + mst22*(mst24 + 5*
     power2(mw2)) + power3(mw2)) + 6*mst12*mst22*mw2*power2(DeltaInv(mw2,msb22,
     mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 +
     power2(mw2)) + power3(mst12) + power3(mw2)) - 6*mst12*mst22*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*(23*mst24*mw2 + mst22*mw2*(6*msb22 + 23*mw2) -
     msb22*(mst24 + power2(mw2)) + power3(mst22) + power3(mw2)) + 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 +
     3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) +
     2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) - 72*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 +
     3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22))))/(6.*msb22*
     mst12*mst22*sb) + (cb*mu*log(msb22/Q2)*power2(snb)*((mst12 - mst22)*(
     msb22*(mst12 - 2*mt2) - 2*mt2*mw2 + mst12*(mst22 + mw2)) - 2*mst22*mt2*
     DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) - mst12*(mst12
     - mst22 - 2*mt2)*DeltaInv(mst22,mw2,msb22)*(5*mw2*power2(mst22) - msb22*(
     power2(mst22) + power2(mw2)) + mst22*(mst24 + 5*power2(mw2)) + power3(mw2)
     ) + 12*mst12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2
     + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) + power3(
     mst12) + power3(mw2)) + 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*(23*mst24*mw2 + mst22*mw2*(6*msb22 + 23*mw2) -
     msb22*(mst24 + power2(mw2)) + power3(mst22) + power3(mw2)) + 144*mst12*
     mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*
     mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(
     mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)) +
     72*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*
     power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22))))/(6.*msb22*mst12*(mst12 - mst22)*mst22*sb)
     + (2*cb*(mst12 - mst22 - 2*mt2)*mu*power2(mw2)*power2(snb)*power2(log(
     mst22/Q2))*(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(
     mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) -
     power4(mst22))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mt2*mu*power2(
     mw2)*power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*power2(DeltaInv(mst22,
     mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(
     mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(
     mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*power3(DeltaInv(
     mst22,mw2,msb12))*(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb12*(
     mst24*mw2 + power3(mst22)) - power4(mst22))))/(msb12*msb22*(mst12 - mst22)
     *sb) + (2*cb*mu*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*((msb12*
     mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*sb) + (2*cb*mu*sb*
     log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*((msb12*
     mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/msb12 + (2*mu*log(msb12/
     Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*power3(cb)*((msb12*
     mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*sb) - (2*cb*mu*
     log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*((mst12 - mst22 - 2*mt2)*(msb12*
     mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(
     mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) -
     power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*cb*mu*log(msb12/Q2)*log(
     mw2/Q2)*power2(mw2)*power2(snb)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 +
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12)
     )*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*
     mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22)))
     )/(msb12*(mst12 - mst22)*sb) + (2*mu*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)
     *power3(cb)*((mst12 - mst22 - 2*mt2)*(msb12*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12)
     )*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)*
     sb) - (2*mu*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power3(cb)*(
     (mst12 - mst22 - 2*mt2)*(msb12*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)*
     sb) + (2*cb*mu*sb*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*((mst12 - mst22 -
     mt2)*(msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))
     + mt2*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2
     + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + (mst12 - mst22 - mt2)*power3(DeltaInv(
     mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) -
     power4(mst22))))/(msb12*(mst12 - mst22)) - (2*cb*mu*sb*log(msb12/Q2)*log(
     mw2/Q2)*power2(mw2)*power2(snb)*((mst12 - mst22 - mt2)*(msb12*mst22 +
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) + mt2*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))) + (mst12 - mst22 - mt2)*power3(DeltaInv(mst22,mw2,msb12))*
     (5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(
     msb12*(mst12 - mst22)) + (2*cb*mu*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snb)*power2(snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(
     mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) +
     power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb22))*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) -
     power4(mst22))))/(msb22*sb) - (2*cb*mu*log(msb22/Q2)*log(mw2/Q2)*power2(
     mw2)*power2(snb)*((mst12 - mst22 - 2*mt2)*(msb22*mst22 + mst24 + 3*mst22*
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (
     mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(
     mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) +
     power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb22*(mst12 -
     mst22)*sb) + (2*mu*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power3(cb)*((mst12 - mst22 - 2*mt2)*(msb22*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*power2(DeltaInv(mw2,msb22,mst12)
     )*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 -
     mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(mw2) -
     mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/(msb22*(mst12 - mst22)*
     sb) + (2*cb*mu*sb*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*((
     mst12 - mst22 - mt2)*(msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + mt2*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) -
     mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(
     mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + (mst12 - mst22 - mt2)*
     power3(DeltaInv(mst22,mw2,msb22))*(5*mst24*power2(mw2) - mw2*power3(mst22)
     + msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*
     power3(mw2) - power4(mst22))))/(msb22*(mst12 - mst22)) + (2*cb*(mst12 -
     mst22 - 2*mt2)*mu*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,
     msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*(mst12 - mst22)*sb) + (2*cb*mt2*mu*log(mst22/Q2)*log(mw2/Q2)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*cb*(mst12 -
     mst22 - mt2)*mu*sb*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,
     msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*(-mst12 + mst22)) - (2*cb*mu*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/(msb12*sb) + (2*cb*mu*sb*log(mst22/Q2)*log(
     mw2/Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*
     mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) +
     mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)
     ) - mst22*power3(mw2) - power4(mst22))))/msb12 + (2*(mst12 - mst22 - 2*
     mt2)*mu*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(cb)*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,
     msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*(-mst12 + mst22)*sb) - (2*mt2*mu*log(mst22/Q2)*log(mw2/Q2)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12))*power3(cb)*(msb12*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*mu*log(
     mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(cb)*(msb12*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,
     msb12)*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*sb) + (2*cb*(mst12 - mst22 - 2*mt2)*mu*log(mst22/Q2)*log(mw2/Q2)*
     power2(mw2)*power2(snb)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*
     (6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*msb22*(-mst12 + mst22)*sb) - (2*cb*mt2*mu*log(
     mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*(msb22*(msb12*mst22 + mst24
     + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(
     mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,
     msb22)*(mst24*power2(mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))) +
     msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*cb*(
     mst12 - mst22 - mt2)*mu*sb*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     snb)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(
     mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(
     mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22*
     (mst12 - mst22)) + (2*cb*mu*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     snb)*power2(snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*
     power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) -
     mst22*power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*msb22*sb) - (2*cb*mu*sb*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snb)*power2(snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*
     (6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*msb22) + (2*(mst12 - mst22 - 2*mt2)*mu*log(mst22/
     Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power3(cb)*(msb22*(msb12*mst22 +
     mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(
     DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) + msb22*(6*mst24*
     mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(
     mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) +
     mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)
     ) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22*(mst12 - mst22)*sb)
     + (2*mt2*mu*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power3(cb)*(
     msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2
     + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) +
     msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(
     mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*
     power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22*(mst12
     - mst22)*sb) - (2*mu*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power2(snt)*power3(cb)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*
     (6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*msb22*sb) - (cb*mu*log(mst12/Q2)*log(mz2/Q2)*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 + mst24 + 3*
     mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(5*mst24*power2(mz2) - mz2*power3(
     mst22) + mst12*(10*mst24*mz2 + 5*mst22*power2(mz2) + power3(mst22)) - 3*
     mst22*power3(mz2) - power4(mst22))))/(mst12*sb) + (cb*mu*sb*log(mst12/Q2)*
     log(mz2/Q2)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*(mst12*mst22 +
     mst24 + 3*mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(5*mst24*power2(mz2) -
     mz2*power3(mst22) + mst12*(10*mst24*mz2 + 5*mst22*power2(mz2) + power3(
     mst22)) - 3*mst22*power3(mz2) - power4(mst22))))/mst12 + (mu*log(mst12/Q2)
     *log(mz2/Q2)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*power3(cb)*(
     mst12*mst22 + mst24 + 3*mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(5*mst24*
     power2(mz2) - mz2*power3(mst22) + mst12*(10*mst24*mz2 + 5*mst22*power2(
     mz2) + power3(mst22)) - 3*mst22*power3(mz2) - power4(mst22))))/(mst12*sb)
     + (cb*mu*log(mst22/Q2)*log(mz2/Q2)*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12))*(mst12*mst22 + mst24 + mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(
     mst24*power2(mz2) + mz2*power3(mst22) + mst12*(6*mst24*mz2 + mst22*power2(
     mz2) + power3(mst22)) - mst22*power3(mz2) - power4(mst22))))/(mst12*sb) -
     (cb*mu*sb*log(mst22/Q2)*log(mz2/Q2)*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12))*(mst12*mst22 + mst24 + mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(
     mst24*power2(mz2) + mz2*power3(mst22) + mst12*(6*mst24*mz2 + mst22*power2(
     mz2) + power3(mst22)) - mst22*power3(mz2) - power4(mst22))))/mst12 - (mu*
     log(mst22/Q2)*log(mz2/Q2)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*
     power3(cb)*(mst12*mst22 + mst24 + mst22*mz2 + DeltaInv(mst22,mz2,mst12)*(
     mst24*power2(mz2) + mz2*power3(mst22) + mst12*(6*mst24*mz2 + mst22*power2(
     mz2) + power3(mst22)) - mst22*power3(mz2) - power4(mst22))))/(mst12*sb) -
     (4*cb*mt2*mu*log(mH2/Q2)*log(mst22/Q2)*power2(sa)*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,
     mst22)*power4(mst22)))/((mst12 - mst22)*sb) - (4*cb*mt2*mu*log(mh2/Q2)*
     log(mst22/Q2)*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(
     mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)))/((mst12 - mst22)
     *sb) + (4*cb*mt2*mu*log(mh2/Q2)*log(mst22/Q2)*power2(sa)*power2(DeltaInv(
     mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,
     mst22,mh2)*power4(mst22)))/((mst12 - mst22)*sb) - (cb*mt2*mu*power2(sa)*(-
     (mh2*DeltaInv(mh2,mst12,mst12)) + mH2*DeltaInv(mH2,mst12,mst12) - mH2*
     DeltaInv(mH2,mst22,mst22) + mh2*DeltaInv(mst22,mst22,mh2) + 42*mH2*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) - 42*mh2*mst24*power2(DeltaInv(mst22,
     mst22,mh2)) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*power3(
     mst12)) - 42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 6*power3(
     mst12)) + 252*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) - 252*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 1008*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power4(mst12) - 1008*mH2*power3(DeltaInv(mH2,
     mst12,mst12))*power4(mst12) + 1008*mH2*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) - 1008*mh2*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22)))
     /(3.*(mst12 - mst22)*sb) - (cb*mt2*mu*power2(Pi)*power2(sa)*(mH2*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) - mh2*mst24*power2(DeltaInv(mst22,mst22,
     mh2)) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) -
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 6*power3(mst12)) + 6*
     power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) - 6*power2(DeltaInv(mst22,
     mst22,mh2))*power3(mst22) + 24*mh2*power3(DeltaInv(mh2,mst12,mst12))*
     power4(mst12) - 24*mH2*power3(DeltaInv(mH2,mst12,mst12))*power4(mst12) +
     24*mH2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) - 24*mh2*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(3.*(mst12 - mst22)*sb) + (cb*
     mt2*mu*power2(Pi)*(msb12*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*
     power3(mst12)) - msb12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 6*
     power3(mst22)) + 24*mh2*msb12*power3(DeltaInv(mh2,mst12,mst12))*power4(
     mst12) + power2(mw2)*(-((msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*
     mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(
     mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) -
     power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22))) - 24*mh2*msb12*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22)))/(3.*msb12*(mst12 - mst22)*sb) -
     (ca*mu*sa*log(mst22/Q2)*power2(snt)*(-((5*mH2*mst22 - mst12*mst22 + mst24)
     *DeltaInv(mH2,mst22,mst12)) - mst12*(mH2 - 6*mst22)*DeltaInv(mH2,mst22,
     mst22) + 5*mh2*mst22*DeltaInv(mst22,mh2,mst12) - mst12*mst22*DeltaInv(
     mst22,mh2,mst12) + mst24*DeltaInv(mst22,mh2,mst12) + mh2*mst12*DeltaInv(
     mst22,mst22,mh2) - 6*mst12*mst22*DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*
     mst24*power2(DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + 84*mst24*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 24*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) -
     6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 6*mh2*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mst22) - 72*mst12*power2(DeltaInv(mst22,
     mst22,mh2))*power3(mst22) + 6*mH2*power2(DeltaInv(mH2,mst22,mst12))*(-3*
     mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 + mst22*power2(mH2) + power3(
     mst22)) + 24*mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*
     power3(mst22)) + 72*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,
     mst22,mst12)) + 144*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) + 144*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) + 144*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12))
     - 72*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 288*mH2*mst12*
     power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) - 72*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) - 288*mh2*mst12*power3(DeltaInv(
     mst22,mst22,mh2))*power4(mst22)))/(3.*mst12) + (cb*mu*log(mst22/Q2)*
     power2(sa)*power2(snt)*(-((5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(
     mH2,mst22,mst12)) - mst12*(mH2 - 6*mst22)*DeltaInv(mH2,mst22,mst22) + 5*
     mh2*mst22*DeltaInv(mst22,mh2,mst12) - mst12*mst22*DeltaInv(mst22,mh2,
     mst12) + mst24*DeltaInv(mst22,mh2,mst12) + mh2*mst12*DeltaInv(mst22,mst22,
     mh2) - 6*mst12*mst22*DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*mst24*power2(
     DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 84*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     - 24*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 6*mh2*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mst22) - 72*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 6*mH2*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*
     mH2*mst24 - mst12*mst24 + mst22*power2(mH2) + power3(mst22)) + 24*mst12*
     power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 72*
     power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*
     mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 144*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 288*mH2*mst12*power3(DeltaInv(
     mH2,mst22,mst22))*power4(mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) - 288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2)
     )*power4(mst22)))/(3.*mst12*sb) + (2*cb*mu*power2(mw2)*power2(snb)*power2(
     snt)*power2(log(mst22/Q2))*(-(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)
     )) + msb12*power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,
     msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 +
     power3(mst22)) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))
     *(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(
     mst22)) + power4(mst22))))/(msb12*msb22*sb) + (2*cb*mt2*mu*log(msb12/Q2)*
     log(mw2/Q2)*power2(mw2)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(
     mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) +
     power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) - msb12*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) +
     power4(mst22))))/(msb12*(mst12 - mst22)*sb) - (2*cb*mt2*mu*log(msb12/Q2)*
     log(mw2/Q2)*power2(mw2)*power2(snb)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)
     *power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*
     power3(mw2) + power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*cb*mu*sb*
     log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*(-((msb12*mst22 + mst24
     + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/msb12 + (2*cb*mu*log(msb12/Q2)*log(
     mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*(-((msb12*mst22 + mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) +
     power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/(msb12*sb) - (2*mt2*mu*log(msb12/Q2)*
     log(mw2/Q2)*power2(mw2)*power3(cb)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*
     power3(mw2) + power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*mt2*mu*log(
     msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power3(cb)*(-((msb12*mst22 +
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/(msb12*(mst12 - mst22)*sb) + (2*mu*
     log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*power3(cb)*(-((msb12*
     mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2)
     + mw2*power3(mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(
     mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/(msb12*sb) + (2*cb*mt2*
     mu*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*(-((msb22*mst22 +
     mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/(msb22*(mst12 - mst22)*sb) + (2*cb*
     mu*sb*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)*(-((
     msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22))) +
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(-5*
     mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb22 - (2*mt2*mu*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power3(cb)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(-
     5*mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/(
     msb22*(mst12 - mst22)*sb) + (2*mu*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snb)*power2(snt)*power3(cb)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(mst22) -
     msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*
     power3(mw2) + power4(mst22))))/(msb22*sb) + (2*cb*mt2*mu*power2(sa)*
     power2(log(mH2/Q2))*(power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*
     power3(mst12)) + 16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power4(mst12) -
     power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*
     DeltaInv(mH2,mst22,mst22)*power4(mst22))))/((mst12 - mst22)*sb) + (4*ca*
     mu*sa*log(mH2/Q2)*log(mst22/Q2)*power2(snt)*((mH2*mst22 + mst12*mst22 +
     mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + power2(mH2)*((
     mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (mst12 - mst22)*
     power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,
     mst22,mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*
     power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22))))/mst12 -
     (4*cb*mu*log(mH2/Q2)*log(mst22/Q2)*power2(sa)*power2(snt)*((mH2*mst22 +
     mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (
     mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(
     mst22))))/(mst12*sb) + (ca*mu*sa*log(mH2/Q2)*log(mst22/Q2)*((mst12 -
     mst22)*(mH2*mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + (mst12 - mst22)*power2(mH2)*((mst12*mst22 + mst24)*power2(
     mH2) - mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) + mH2*(6*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 4*mst12*mt2*
     power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*
     DeltaInv(mH2,mst22,mst22)*power4(mst22))))/(mst12*(-mst12 + mst22)) - (cb*
     mu*log(mH2/Q2)*log(mst22/Q2)*power2(sa)*((mst12 - mst22)*(mH2*mst22 +
     mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + (
     mst12 - mst22)*power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*
     power3(mH2) + (mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 8*mst12*mt2*power2(DeltaInv(
     mH2,mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,
     mst22,mst22)*power4(mst22))))/(mst12*(-mst12 + mst22)*sb) + (2*cb*mt2*mu*
     power2(log(mh2/Q2))*(power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*
     power3(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12) -
     power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*
     DeltaInv(mst22,mst22,mh2)*power4(mst22))))/((mst12 - mst22)*sb) - (2*cb*
     mt2*mu*power2(sa)*power2(log(mh2/Q2))*(power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 4*power3(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*
     power4(mst12) - power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(
     mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/((mst12 -
     mst22)*sb) - (4*ca*mu*sa*log(mh2/Q2)*log(mst22/Q2)*power2(snt)*((mh2*mst22
     + mst12*mst22 + mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) +
     power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (
     mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(DeltaInv(mst22,mst22,mh2)
     )*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(
     mst22))))/mst12 - (4*cb*mu*log(mh2/Q2)*log(mst22/Q2)*power2(snt)*((mh2*
     mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (
     mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(DeltaInv(mst22,mst22,mh2)
     )*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(
     mst22))))/(mst12*sb) + (4*cb*mu*log(mh2/Q2)*log(mst22/Q2)*power2(sa)*
     power2(snt)*((mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22 + mst24)*power2(
     mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22))))/(mst12*sb) - (ca*mu*sa*log(mh2/Q2)*log(
     mst22/Q2)*((mst12 - mst22)*(mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + (mst12 - mst22)*power2(mh2)*((mst12*
     mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(
     mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,
     mst12)) + 4*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*
     power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22))))/(mst12*(
     -mst12 + mst22)) - (cb*mu*log(mh2/Q2)*log(mst22/Q2)*((mst12 - mst22)*(mh2*
     mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     + (mst12 - mst22)*power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) - mst22*
     power3(mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + 8*mst12*mt2*power2(DeltaInv(
     mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,
     mst22,mh2)*power4(mst22))))/(mst12*(-mst12 + mst22)*sb) + (cb*mu*log(mh2/
     Q2)*log(mst22/Q2)*power2(sa)*((mst12 - mst22)*(mh2*mst22 + mst12*mst22 +
     mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + (mst12 - mst22)*
     power2(mh2)*((mst12*mst22 + mst24)*power2(mh2) - mst22*power3(mh2) + (
     mst12 - mst22)*power3(mst22) + mh2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mh2,mst12)) + 8*mst12*mt2*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22))))/(mst12*(-mst12 + mst22)*sb) + (cb*mt2*mu*log(mst22/
     Q2)*power2(sa)*(-((mH2 - 6*mst22)*DeltaInv(mH2,mst22,mst22)) + 24*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 288*mH2*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + DeltaInv(mst22,mst22,mh2)*(mh2
     - 6*mst22 - 24*DeltaInv(mst22,mst22,mh2)*(mh2*mst24 + 3*power3(mst22)) -
     288*mh2*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22))))/(3.*(mst12 -
     mst22)*sb) + (2*cb*mt2*mu*power2(log(mst22/Q2))*(mh2*msb12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(-mst24 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(msb12*(
     mst12 - mst22)*sb) + (2*cb*mu*sb*power2(snt)*power2(log(mst22/Q2))*(mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - power2(mC2)*(mst24*power2(
     mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(-mst24 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) -
     2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22))
     )))/msb12 + (2*mu*power2(snt)*power2(log(mst22/Q2))*power3(cb)*(mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - power2(mC2)*(mst24*power2(
     mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12))*(-mst24 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) -
     2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22))
     )))/(msb12*sb) + (mt2*mu*log(mst22/Q2)*power3(cb)*((5*mC2*mst22 - msb12*
     mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - 6*mC2*power2(DeltaInv(mst22,
     msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*
     power2(mC2) + power3(mst22)) - 72*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - DeltaInv(mst22,mw2,msb12)*(-(msb12*
     mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb12)*(mst22*mw2*
     (-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(6.*
     msb12*(mst12 - mst22)*sb) + (cb*mu*sb*log(mst22/Q2)*power2(snt)*(-((5*mC2*
     mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + 6*mC2*power2(
     DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*
     mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + DeltaInv(mst22,mw2,msb12)*(-(
     msb12*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb12)*(
     mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*
     power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(
     6.*msb12) + (mu*log(mst22/Q2)*power2(snt)*power3(cb)*(-((5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + 6*mC2*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*(mst24*power2(mC2) + (
     -2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + DeltaInv(mst22,mw2,msb12)*(-(msb12*
     mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb12)*(mst22*mw2*
     (-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(6.*
     msb12*sb) + (cb*mt2*mu*log(mst22/Q2)*(-2*msb12*(mh2 - 6*mst22)*DeltaInv(
     mst22,mst22,mh2) + 48*msb12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 +
     3*power3(mst22)) + 576*mh2*msb12*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) + DeltaInv(mst22,mw2,msb12)*(-(msb12*mst22) + mst24 + 5*mst22*mw2 -
     6*mw2*DeltaInv(mst22,mw2,msb12)*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12
     + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 +
     power3(mst22)) + power4(mst22)))))/(6.*msb12*(mst12 - mst22)*sb) + (2*cb*(
     mst12 - mst22 - mt2)*mu*sb*power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) - msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + msb12*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-(msb22*mst24*power2(
     DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*
     power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22)
     - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(msb12*msb22*(
     mst12 - mst22)) + (2*(mst12 - mst22 - 2*mt2)*mu*power2(snb)*power2(log(
     mst22/Q2))*power3(cb)*(msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) -
     msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) -
     2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     power2(mw2)*(-(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12))) + msb12*
     power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(
     mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(
     mst22)) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(
     mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(
     mst22)) + power4(mst22)))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*mt2*mu*
     power2(snb)*power2(log(mst22/Q2))*power3(cb)*(msb22*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - msb22*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + msb12*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-(msb22*mst24*power2(
     DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22))) + msb22*
     power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22)
     - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(msb12*msb22*(
     mst12 - mst22)*sb) + (cb*(mst12 - mst22 - 2*mt2)*mu*log(mst22/Q2)*power2(
     snb)*(msb22*(-(msb12*mst22) + mst24 + 5*mst22*mw2)*DeltaInv(mst22,mw2,
     msb12) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*(mst22*mw2*(-3*
     msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22)) - msb12*
     DeltaInv(mst22,mw2,msb22)*(-(msb22*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*
     DeltaInv(mst22,mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2) - mst24*(msb22 + 14*
     mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(
     mst22)) + power4(mst22)))))/(6.*msb12*msb22*(-mst12 + mst22)*sb) - (cb*
     mt2*mu*log(mst22/Q2)*power2(snb)*(msb22*(-(msb12*mst22) + mst24 + 5*mst22*
     mw2)*DeltaInv(mst22,mw2,msb12) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(
     mst22)) - 72*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mst24*
     power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) +
     power4(mst22)) - msb12*DeltaInv(mst22,mw2,msb22)*(-(msb22*mst22) + mst24 +
     5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2)
     - mst24*(msb22 + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*
     msb22*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(6.*msb12*msb22*(
     mst12 - mst22)*sb) + (cb*mu*log(mst22/Q2)*power2(snb)*power2(snt)*(msb22*(
     -(msb12*mst22) + mst24 + 5*mst22*mw2)*DeltaInv(mst22,mw2,msb12) - 6*msb22*
     mw2*power2(DeltaInv(mst22,mw2,msb12))*(mst22*mw2*(-3*msb12 + mw2) - mst24*
     (msb12 + 14*mw2) + power3(mst22)) - 72*msb22*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(
     mst24*mw2 + power3(mst22)) + power4(mst22)) - msb12*DeltaInv(mst22,mw2,
     msb22)*(-(msb22*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,
     msb22)*(mst22*mw2*(-3*msb22 + mw2) - mst24*(msb22 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(6.*msb12*msb22*sb) + (2*cb*mu*sb*log(msb12/Q2)*log(mst22/Q2)*
     power2(snt)*((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((msb12*mst22 - mst24)*power2(
     mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + 5*mC2*(2*
     msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-(msb12*mst22) - 3*mst24 - mst22*
     mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 5*mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(
     mw2) + 3*power4(mst22)))))/msb12 + (2*mt2*mu*log(msb12/Q2)*log(mst22/Q2)*
     power2(snb)*power3(cb)*((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((msb12*mst22 - mst24)*
     power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + 5*
     mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-(msb12*mst22) - 3*mst24 -
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 5*mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) +
     mst22*power3(mw2) + 3*power4(mst22)))))/(msb12*(mst12 - mst22)*sb) + (2*
     mu*log(msb12/Q2)*log(mst22/Q2)*power2(snt)*power3(cb)*((mC2*mst22 + msb12*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(
     mC2)*((msb12*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 -
     3*mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(-(msb12*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*
     power2(mw2) - 5*mw2*power3(mst22) - msb12*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/(msb12*
     sb) + (2*cb*mu*sb*log(msb22/Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*((
     mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) + power2(mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*
     power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22))*(-(msb22*mst22) - 3*mst24 - mst22*mw2 +
     DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) - 5*mw2*power3(mst22) -
     msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(
     mw2) + 3*power4(mst22)))))/msb22 + (2*mu*log(msb22/Q2)*log(mst22/Q2)*
     power2(snb)*power2(snt)*power3(cb)*((mC2*mst22 + msb22*mst22 + 3*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((msb22*mst22
     - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(
     mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-(msb22*
     mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) - 5*mw2*power3(mst22) - msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*
     power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/(msb22*sb) + (cb*
     mu*log(mst22/Q2)*power2(snt)*(-2*msb12*(5*mh2*mst22 - mst12*mst22 + mst24)
     *DeltaInv(mst22,mh2,mst12) + 12*mh2*msb12*power2(DeltaInv(mst22,mh2,mst12)
     )*(-3*mh2*mst12*mst22 - 14*mh2*mst24 - mst12*mst24 + mst22*power2(mh2) +
     power3(mst22)) + 144*msb12*power2(mh2)*(mst24*power2(mh2) + (-2*mst12 +
     mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12)) + mst12*(-2*msb12*(mh2 - 6*mst22)*DeltaInv(
     mst22,mst22,mh2) + 48*msb12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 +
     3*power3(mst22)) + 576*mh2*msb12*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) - DeltaInv(mst22,mw2,msb12)*(-(msb12*mst22) + mst24 + 5*mst22*mw2 -
     6*mw2*DeltaInv(mst22,mw2,msb12)*(mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12
     + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 +
     power3(mst22)) + power4(mst22))))))/(6.*msb12*mst12*sb) + (cb*mu*sb*log(
     mst22/Q2)*power2(snb)*power2(snt)*(msb22*(5*mC2*mst22 - msb12*mst22 +
     mst24)*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*mst22 - msb22*mst22 +
     mst24)*DeltaInv(mst22,msb22,mC2) + msb12*msb22*mst22*DeltaInv(mst22,mw2,
     msb12) - msb22*mst24*DeltaInv(mst22,mw2,msb12) - 5*msb22*mst22*mw2*
     DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*DeltaInv(mst22,mw2,msb22) +
     msb12*mst24*DeltaInv(mst22,mw2,msb22) + 5*msb12*mst22*mw2*DeltaInv(mst22,
     mw2,msb22) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) -
     18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*
     msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*msb22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 18*msb12*msb22*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 84*msb12*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mst22) - 6*msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mst22) - 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*
     msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) + 6*mC2*msb12*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*
     mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) +
     6*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 72*msb22*power2(mC2)
     *(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 72*msb12*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) - 144*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) -
     144*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) +
     144*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 144*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 72*
     msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 72*
     msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 72*
     msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 72*msb12*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(6.*msb12*msb22) + (
     mu*log(mst22/Q2)*power2(snb)*power2(snt)*power3(cb)*(msb22*(5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*mst22 -
     msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) + msb12*msb22*mst22*
     DeltaInv(mst22,mw2,msb12) - msb22*mst24*DeltaInv(mst22,mw2,msb12) - 5*
     msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*DeltaInv(
     mst22,mw2,msb22) + msb12*mst24*DeltaInv(mst22,mw2,msb22) + 5*msb12*mst22*
     mw2*DeltaInv(mst22,mw2,msb22) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) - 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 84*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 18*
     msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 84*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mw2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*msb12*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mst22) - 6*mC2*msb22*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) + 6*mC2*msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) + 6*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 6*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)
     *power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + 72*msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb22*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 72*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 72*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 72*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 72*msb12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2)))/(6.*msb12*msb22*sb) + (cb*(mst12 - mst22 - mt2)*mu*
     sb*log(mst22/Q2)*power2(snb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*
     DeltaInv(mst22,msb12,mC2)) + msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*
     DeltaInv(mst22,msb22,mC2) - msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) +
     msb22*mst24*DeltaInv(mst22,mw2,msb12) + 5*msb22*mst22*mw2*DeltaInv(mst22,
     mw2,msb12) + msb12*msb22*mst22*DeltaInv(mst22,mw2,msb22) - msb12*mst24*
     DeltaInv(mst22,mw2,msb22) - 5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) +
     6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 18*msb12*
     msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 84*msb22*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*mst24*
     mw2*power2(DeltaInv(mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mst22) + 6*msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(
     mst22) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22
     - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*mC2*
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24
     - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22))
     + ((mst12 - mst22 - 2*mt2)*mu*log(mst22/Q2)*power2(snb)*power3(cb)*(-(
     msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) +
     msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) -
     msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) + msb22*mst24*DeltaInv(mst22,
     mw2,msb12) + 5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*
     mst22*DeltaInv(mst22,mw2,msb22) - msb12*mst24*DeltaInv(mst22,mw2,msb22) -
     5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) + 6*msb12*msb22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) + 18*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 84*msb22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*
     msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*msb22*
     power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 -
     msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*mC2*msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*
     mst24 + mst22*power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22)*
     sb) + (mt2*mu*log(mst22/Q2)*power2(snb)*power3(cb)*(-(msb22*(5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + msb12*(5*mC2*mst22 -
     msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) - msb12*msb22*mst22*
     DeltaInv(mst22,mw2,msb12) + msb22*mst24*DeltaInv(mst22,mw2,msb12) + 5*
     msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*mst22*DeltaInv(
     mst22,mw2,msb22) - msb12*mst24*DeltaInv(mst22,mw2,msb22) - 5*msb12*mst22*
     mw2*DeltaInv(mst22,mw2,msb22) + 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) + 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 84*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) - 18*
     msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mw2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*msb12*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*msb22*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) - 6*mC2*msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)
     *power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22)*sb) + (cb*mt2*mu*
     mw2*power2(snb)*(-(msb12*msb22*DeltaInv(mst22,mw2,msb12)) + msb12*msb22*
     DeltaInv(mst22,mw2,msb22) + msb12*msb22*DeltaInv(mw2,msb12,mst12) - msb12*
     msb22*DeltaInv(mw2,msb22,mst12) + 42*msb22*mw2*(msb12*mst22 + 3*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*mw2*(msb22*mst22
     + 3*mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*
     msb22*mst12*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst14*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*mst12*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) + 126*msb12*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) +
     126*msb12*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 588*msb12*
     msb22*mst14*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 210*msb12*
     msb22*mw2*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 210*msb12*
     msb22*mst12*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 588*msb12*msb22*
     mst14*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*mw2*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 126*msb12*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*mst12*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mw2*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 126*msb12*mw2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 42*msb22*mw2*power3(DeltaInv(mst22,mw2,msb12))*(
     14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)) - 42*msb12*mw2*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*
     mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*
     power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(
     mst22)) + 126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) -
     126*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(3.*msb12*
     msb22*(mst12 - mst22)*sb) - (cb*mu*mw2*power2(snb)*power2(snt)*(-(msb12*
     msb22*DeltaInv(mst22,mw2,msb12)) + msb12*msb22*DeltaInv(mst22,mw2,msb22) +
     msb12*msb22*DeltaInv(mw2,msb12,mst12) - msb12*msb22*DeltaInv(mw2,msb22,
     mst12) + 42*msb22*mw2*(msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) - 42*msb12*mw2*(msb22*mst22 + 3*mst24 + 3*
     mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*msb22*mst12*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) - 126*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 42*msb12*msb22*mst12*mw2*power2(DeltaInv(mw2,msb22,mst12)) +
     126*msb12*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst12*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 588*msb12*msb22*mst14*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 210*msb12*msb22*mw2*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 210*msb12*msb22*mst12*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 588*msb12*msb22*mst14*power2(mw2)
     *power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*mw2*power3(mst12)*
     power3(DeltaInv(mw2,msb22,mst12)) + 126*msb12*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*mst12*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 126*msb22*mw2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 126*msb12*mw2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) + 42*msb22*mw2*power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*
     mst24*mw2 + 5*msb12*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*
     power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(
     mst22)) - 42*msb12*mw2*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*
     mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(
     mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(3.*msb12*msb22*sb)
     + (mt2*mu*log(mst12/Q2)*power2(snb)*power3(cb)*(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12 -
     msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*
     DeltaInv(mw2,msb12,mst12) - msb22*mst14*DeltaInv(mw2,msb12,mst12) - 5*
     msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) + msb12*mst14*DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) - 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) - 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) - 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) + 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) + 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) - 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) - 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22)*sb) + (cb*mt2*mu*sb*
     log(mst12/Q2)*power2(snb)*(-(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*mst12 + mst14)*
     DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*DeltaInv(mw2,msb12,mst12) +
     msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mw2*DeltaInv(mw2,
     msb12,mst12) + msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) - msb12*mst14*
     DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mw2*DeltaInv(mw2,msb22,mst12) +
     6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*mst14*
     mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))
     *power3(mst12) + 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mst12) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12
     - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) - 6*mC2*
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14
     - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*
     mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(6.*msb12*msb22*(mst12 - mst22))
     + (cb*mu*sb*log(mst12/Q2)*power2(snb)*power2(snt)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 -
     msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*
     DeltaInv(mw2,msb12,mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*
     msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22) + (mt2*mu*log(mst12/Q2)*power2(snb)
     *power3(cb)*(-(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,
     mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,
     mst12,mC2) - msb12*msb22*mst12*DeltaInv(mw2,msb12,mst12) + msb22*mst14*
     DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) +
     msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,
     msb22,mst12) - 5*msb12*mst12*mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*
     msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst12*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst14*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*mst14*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mst12) + 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*
     mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*
     mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*
     power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 -
     msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*
     mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(3.*msb12*msb22*(mst12 - mst22)*
     sb) + (mu*log(mst12/Q2)*power2(snb)*power2(snt)*power3(cb)*(-(msb22*(5*
     mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*
     mC2*mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*
     mst12*DeltaInv(mw2,msb12,mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) +
     5*msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22*sb) - (cb*mu*power2(log(mst22/Q2))*(
     (mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)))/mst12 + (4*mh2*
     mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)))/(mst12 - mst22) + (2*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/msb12 - (4*mst24*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(msb12*mst12 - msb12*mst22)
     + (mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/mst12 - (power2(
     mh2)*(mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))/mst12 + 4*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - (8*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 -
     mst22) + 4*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - (8*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) + (4*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 - (8*
     mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(msb12*
     mst12 - msb12*mst22) + 2*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,
     mz2,mst12)) + 2*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + (2*
     power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)))/mst12 - (2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/msb12 + (4*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/(msb12*
     mst12 - msb12*mst22) - (power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22))/mst12 - (2*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2))/msb12 + (4*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/
     (msb12*mst12 - msb12*mst22) - (mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2))/mst12))/sb - (cb*mu*log(mst22/Q2)*(((-5*mh2*mst22 + mst12*
     mst22 - mst24)*DeltaInv(mst22,mh2,mst12))/mst12 + (8*(mh2 - 6*mst22)*mt2*
     DeltaInv(mst22,mst22,mh2))/(-mst12 + mst22) + 2*mst22*DeltaInv(mst22,mw2,
     msb12) - (2*mst24*DeltaInv(mst22,mw2,msb12))/msb12 - (4*mst22*mt2*
     DeltaInv(mst22,mw2,msb12))/(mst12 - mst22) + (4*mst24*mt2*DeltaInv(mst22,
     mw2,msb12))/(msb12*mst12 - msb12*mst22) - (10*mst22*mw2*DeltaInv(mst22,
     mw2,msb12))/msb12 + (20*mst22*mt2*mw2*DeltaInv(mst22,mw2,msb12))/(msb12*
     mst12 - msb12*mst22) + mst22*DeltaInv(mst22,mz2,mst12) - (mst24*DeltaInv(
     mst22,mz2,mst12))/mst12 - (5*mst22*mz2*DeltaInv(mst22,mz2,mst12))/mst12 -
     12*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + (24*mst24*mt2*mw2*power2(
     DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - 36*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - (168*mst24*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)))/msb12 + (72*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)))/(mst12 - mst22) + (336*mst24*mt2*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)))/(msb12*mst12 - msb12*mst22) - 6*mst24*mz2*power2(
     DeltaInv(mst22,mz2,mst12)) - 18*mst22*power2(mz2)*power2(DeltaInv(mst22,
     mz2,mst12)) - (84*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/
     mst12 + (12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22))/msb12 - (
     24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22))/(msb12*mst12 -
     msb12*mst22) + (6*mz2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22))/
     mst12 + (6*mh2*power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst12*mst22 - 14*
     mh2*mst24 - mst12*mst24 + mst22*power2(mh2) + power3(mst22)))/mst12 + (
     192*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 3*power3(mst22)))/(
     mst12 - mst22) + (12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/
     msb12 + (24*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/(
     msb12*(-mst12 + mst22)) + (6*mst22*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mz2))/mst12 + (72*power2(mh2)*(mst24*power2(mh2) + (-2*mst12 +
     mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12)))/mst12 - 288*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) + (576*mt2*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - 288*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + (576*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - (288*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 + (576*mt2*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(msb12*mst12 - msb12*mst22)
     - 144*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 144*
     mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - (144*power3(mst22)*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)))/mst12 + (2304*mh2*mt2*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22))/(mst12 - mst22) + (144*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/msb12 - (288*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/(msb12*
     mst12 - msb12*mst22) + (72*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22))/mst12 + (144*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2))/msb12 - (288*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2))/(msb12*mst12 - msb12*mst22) + (72*mst24*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mz2))/mst12))/(12.*sb) - (4*cb*mu*log(mst12/Q2)*
     log(mst22/Q2)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(mh2*mst22 +
     mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,mst12)*((mst12*mst22 - mst24)*
     power2(mh2) - mst22*power3(mh2) + (5*mst12 - 3*mst22)*power3(mst22) + 5*
     mh2*(2*mst12*mst24 + power3(mst22))))*power4(snt))/(mst12*sb) - (2*cb*mh2*
     mu*power2(log(mst12/Q2))*(mst12*mst14*power2(DeltaInv(mh2,mst12,mst12)) +
     2*mh2*power2(DeltaInv(mst22,mh2,mst12))*(2*mh2*mst22 + mst12*mst22 + 2*
     mst24 + DeltaInv(mst22,mh2,mst12)*((3*mst12*mst22 + 2*mst24)*power2(mh2) -
     2*mst22*power3(mh2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mh2*(5*mst12*
     mst24 + power3(mst22)))))*power4(snt))/(mst12*sb) + (4*cb*mu*log(mst12/Q2)
     *log(mst22/Q2)*power2(sa)*(-((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(
     mH2)*power2(DeltaInv(mH2,mst22,mst12))) + power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12))*(mh2*mst22 + mst12*mst22 + 3*mst24 + DeltaInv(mst22,mh2,
     mst12)*((mst12*mst22 - mst24)*power2(mh2) - mst22*power3(mh2) + (5*mst12 -
     3*mst22)*power3(mst22) + 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(
     mH2)*((-(mst12*mst22) + mst24)*power2(mH2) + mst22*power3(mH2) + (-5*mst12
     + 3*mst22)*power3(mst22) - 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)))*power4(snt))/(mst12*sb) + (4*ca*mu*sa*log(
     mst12/Q2)*log(mst22/Q2)*((mH2*mst22 + mst12*mst22 + 3*mst24)*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12))*(-(mh2*mst22) - mst12*mst22 - 3*mst24 + DeltaInv(mst22,mh2,mst12)*
     ((-(mst12*mst22) + mst24)*power2(mh2) + mst22*power3(mh2) + (-5*mst12 + 3*
     mst22)*power3(mst22) - 5*mh2*(2*mst12*mst24 + power3(mst22)))) + power2(
     mH2)*((mst12*mst22 - mst24)*power2(mH2) - mst22*power3(mH2) + (5*mst12 -
     3*mst22)*power3(mst22) + 5*mH2*(2*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mH2,mst22,mst12)))*power4(snt))/mst12 - (2*cb*mh2*mu*power2(log(
     mst22/Q2))*(2*mh2*mst24*power2(DeltaInv(mst22,mh2,mst12)) + mst12*mst24*
     power2(DeltaInv(mst22,mst22,mh2)) - 2*mh2*(mst24*power2(mh2) + (-2*mst12 +
     mst22)*power3(mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mh2,mst12)))*power4(snt))/(mst12*sb) + (2*cb*mu*power2(sa)*
     power2(log(mst22/Q2))*(-2*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,
     mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) + 2*power2(
     mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mh2*(2*mh2*
     mst24*power2(DeltaInv(mst22,mh2,mst12)) + mst12*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) - 2*mh2*(mst24*power2(mh2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mh2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,
     mst12))))*power4(snt))/(mst12*sb) + (2*ca*mu*sa*power2(log(mst22/Q2))*(2*
     mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + mH2*mst12*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) - 2*power2(mH2)*(mst24*power2(mH2) + (-
     2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mh2*(-2*mh2*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 2*mh2*
     (mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12))))*power4(snt))/mst12 +
     (ca*mu*sa*log(mst22/Q2)*(-((5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(
     mH2,mst22,mst12)) - mst12*(mH2 - 6*mst22)*DeltaInv(mH2,mst22,mst22) + 5*
     mh2*mst22*DeltaInv(mst22,mh2,mst12) - mst12*mst22*DeltaInv(mst22,mh2,
     mst12) + mst24*DeltaInv(mst22,mh2,mst12) + mh2*mst12*DeltaInv(mst22,mst22,
     mh2) - 6*mst12*mst22*DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*mst24*power2(
     DeltaInv(mst22,mh2,mst12)) + 18*mst12*mst22*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 84*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))
     - 24*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 6*mh2*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mst22) - 72*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 6*mH2*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*
     mH2*mst24 - mst12*mst24 + mst22*power2(mH2) + power3(mst22)) + 24*mst12*
     power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 72*
     power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*
     mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 144*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 288*mH2*mst12*power3(DeltaInv(
     mH2,mst22,mst22))*power4(mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) - 288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2)
     )*power4(mst22))*power4(snt))/(3.*mst12) - (cb*mu*log(mst22/Q2)*power2(sa)
     *(-((5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(mH2,mst22,mst12)) -
     mst12*(mH2 - 6*mst22)*DeltaInv(mH2,mst22,mst22) + 5*mh2*mst22*DeltaInv(
     mst22,mh2,mst12) - mst12*mst22*DeltaInv(mst22,mh2,mst12) + mst24*DeltaInv(
     mst22,mh2,mst12) + mh2*mst12*DeltaInv(mst22,mst22,mh2) - 6*mst12*mst22*
     DeltaInv(mst22,mst22,mh2) + 6*mh2*mst12*mst24*power2(DeltaInv(mst22,mh2,
     mst12)) + 18*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) +
     84*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 24*mh2*mst12*
     mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*power3(mh2) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power3(
     mst22) - 72*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mH2*
     power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 -
     mst12*mst24 + mst22*power2(mH2) + power3(mst22)) + 24*mst12*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) + 72*power2(mH2)*
     (mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*mst24
     + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 144*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 72*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 288*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) - 72*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) - 288*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22))*
     power4(snt))/(3.*mst12*sb) - (4*ca*mu*sa*log(mH2/Q2)*log(mst22/Q2)*((mH2*
     mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))
     + power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*power3(mH2) + (
     mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(
     mst22)))*power4(snt))/mst12 + (4*cb*mu*log(mH2/Q2)*log(mst22/Q2)*power2(
     sa)*((mH2*mst22 + mst12*mst22 + mst24)*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + power2(mH2)*((mst12*mst22 + mst24)*power2(mH2) - mst22*
     power3(mH2) + (mst12 - mst22)*power3(mst22) + mH2*(6*mst12*mst24 + power3(
     mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + mst12*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,
     mst22)*power4(mst22)))*power4(snt))/(mst12*sb) + (4*ca*mu*sa*log(mh2/Q2)*
     log(mst22/Q2)*((mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22 + mst24)*power2(
     mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22)))*power4(snt))/mst12 + (4*cb*mu*log(mh2/Q2)
     *log(mst22/Q2)*((mh2*mst22 + mst12*mst22 + mst24)*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22 + mst24)*power2(
     mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) + mh2*(6*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22)))*power4(snt))/(mst12*sb) - (4*cb*mu*log(
     mh2/Q2)*log(mst22/Q2)*power2(sa)*((mh2*mst22 + mst12*mst22 + mst24)*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + power2(mh2)*((mst12*mst22
     + mst24)*power2(mh2) - mst22*power3(mh2) + (mst12 - mst22)*power3(mst22) +
     mh2*(6*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) +
     mst12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*
     mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)))*power4(snt))/(mst12*sb) - (
     cb*mu*log(mst22/Q2)*(-((5*mh2*mst22 - mst12*mst22 + mst24)*DeltaInv(mst22,
     mh2,mst12)) + 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst12*mst22
     - 14*mh2*mst24 - mst12*mst24 + mst22*power2(mh2) + power3(mst22)) + 72*
     power2(mh2)*(mst24*power2(mh2) + (-2*mst12 + mst22)*power3(mst22) - 2*mh2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*
     DeltaInv(mst22,mst22,mh2)*(-mh2 + 6*mst22 + 24*DeltaInv(mst22,mst22,mh2)*(
     mh2*mst24 + 3*power3(mst22)) + 288*mh2*power2(DeltaInv(mst22,mst22,mh2))*
     power4(mst22)))*power4(snt))/(3.*mst12*sb) + (2*cb*mu*power2(sa)*power2(
     snt)*power2(log(mst12/Q2))*(-(mh2*mst12*mst14*power2(DeltaInv(mh2,mst12,
     mst12))) + mH2*mst12*mst14*power2(DeltaInv(mH2,mst12,mst12)) + 2*mst12*
     mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 4*mst24*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) - 2*mst12*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 4*mst24*power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12)) - 4*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 4*
     mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 20*mst12*mst24*
     power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 6*mst12*power2(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 4*power3(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 20*mst12*mst24*power3(mh2)*
     power3(DeltaInv(mst22,mh2,mst12)) - 6*mst12*power2(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) - 4*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 6*mst12*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) - 4*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) + 6*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*
     mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) - 4*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) + 4*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) + 4*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) - 4*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2)))/(mst12*sb) + (2*ca*mu*sa*power2(log(mst12/Q2))*power4(snt)*(
     -(mh2*mst12*mst14*power2(DeltaInv(mh2,mst12,mst12))) + mH2*mst12*mst14*
     power2(DeltaInv(mH2,mst12,mst12)) + 2*mst12*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 4*mst24*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) - 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) - 4*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 4*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 4*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) + 20*mst12*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) + 6*mst12*power2(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) + 4*power3(mH2)*power3(mst22)*power3(DeltaInv(
     mH2,mst22,mst12)) - 20*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 4*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) -
     6*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 4*mst24*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst12*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*mst24*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) - 4*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mst22) + 4*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 4*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 4*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power5(mH2)))/mst12 + (2*ca*mu*sa*
     power2(snt)*power2(log(mst12/Q2))*(mh2*mst12*mst14*power2(DeltaInv(mh2,
     mst12,mst12)) - mH2*mst12*mst14*power2(DeltaInv(mH2,mst12,mst12)) - 2*
     mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 4*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 2*mst12*mst22*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)) + 4*mst24*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 4*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2)
     - 4*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 20*mst12*mst24*
     power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) - 6*mst12*power2(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 4*power3(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 20*mst12*mst24*power3(mh2)*
     power3(DeltaInv(mst22,mh2,mst12)) + 6*mst12*power2(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) + 4*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 6*mst12*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) + 4*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) - 6*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) - 4*
     mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 4*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 4*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) - 4*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) + 4*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2)))/mst12 + (2*cb*mu*power2(sa)*power2(log(mst12/Q2))*power4(
     snt)*(mh2*mst12*mst14*power2(DeltaInv(mh2,mst12,mst12)) - mH2*mst12*mst14*
     power2(DeltaInv(mH2,mst12,mst12)) - 2*mst12*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 4*mst24*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) + 4*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 4*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 4*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 20*mst12*mst24*power3(mH2)*power3(
     DeltaInv(mH2,mst22,mst12)) - 6*mst12*power2(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) - 4*power3(mH2)*power3(mst22)*power3(DeltaInv(
     mH2,mst22,mst12)) + 20*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) + 6*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) + 4*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) +
     6*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 4*mst24*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 6*mst12*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) - 4*mst24*power3(DeltaInv(mH2,
     mst22,mst12))*power4(mH2) + 4*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mst22) - 4*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) - 4*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) + 4*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power5(mH2)))/(mst12*sb) - (4*ca*mu*sa*
     log(mh2/Q2)*log(mst12/Q2)*power2(snt)*(power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(mst22,mh2,
     mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(mh2) + (-
     mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(mst22)))) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/mst12 - (4*cb*mu*
     log(mh2/Q2)*log(mst12/Q2)*power2(snt)*(power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 + DeltaInv(mst22,mh2,
     mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*power3(mh2) + (-
     mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 + power3(mst22)))) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) +
     16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(mst12*sb) + (4*
     cb*mu*log(mh2/Q2)*log(mst12/Q2)*power2(sa)*power2(snt)*(power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 +
     DeltaInv(mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*
     power3(mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 +
     power3(mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*
     power4(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/
     (mst12*sb) + (4*ca*mu*sa*log(mh2/Q2)*log(mst12/Q2)*power4(snt)*(power2(
     mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24
     + DeltaInv(mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*
     mst22*power3(mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24
     + power3(mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 +
     4*power4(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12))
     )/mst12 + (4*cb*mu*log(mh2/Q2)*log(mst12/Q2)*power4(snt)*(power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22 - mst24 +
     DeltaInv(mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2) + 3*mst22*
     power3(mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*mst24 +
     power3(mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 4*
     power4(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/
     (mst12*sb) - (4*cb*mu*log(mh2/Q2)*log(mst12/Q2)*power2(sa)*power4(snt)*(
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-3*mh2*mst22 - mst12*mst22
     - mst24 + DeltaInv(mst22,mh2,mst12)*(-5*(mst12*mst22 + mst24)*power2(mh2)
     + 3*mst22*power3(mh2) + (-mst12 + mst22)*power3(mst22) + mh2*(-10*mst12*
     mst24 + power3(mst22)))) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*
     mst14 + 4*power4(mst12)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12)))/(mst12*sb) - (2*ca*mu*sa*power2(snt)*power2(log(mh2/Q2))*(
     -2*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*power3(mh2)*(-
     2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/mst12 - (2*cb*mu*power2(snt)*power2(log(mh2/Q2))*(-2*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*power3(mh2)*(-2*mst12*
     mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*
     power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mst22,mst22,
     mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*
     power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/
     (mst12*sb) + (2*cb*mu*power2(sa)*power2(snt)*power2(log(mh2/Q2))*(-2*
     mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/(mst12*sb) + (2*ca*mu*sa*power2(log(mh2/Q2))*power4(snt)*(-2*
     mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/mst12 + (2*cb*mu*power2(log(mh2/Q2))*power4(snt)*(-2*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*power3(mh2)*(-2*mst12*
     mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*
     power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mst22,mst22,
     mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*
     power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/
     (mst12*sb) - (2*cb*mu*power2(sa)*power2(log(mh2/Q2))*power4(snt)*(-2*
     mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) + power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst12*mst14 + 4*power4(mst12)) + mst12*power2(DeltaInv(mst22,
     mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,
     mh2)*power4(mst22)) + 16*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/(mst12*sb) + (cb*mu*power2(Pi)*power4(snt)*(-2*(3*mh2*mst22 +
     mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) +
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) + 2*
     power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*(-14*mh2*mst12*mst24 - 5*
     mst12*mst22*power2(mh2) - 3*mst24*power2(mh2) + 3*mst22*power3(mh2) - 3*
     mh2*power3(mst22) - 5*mst12*power3(mst22) + 3*power4(mst22)) + mst12*
     power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 6*power3(mst22) + 24*mh2*
     DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 24*mh2*power3(DeltaInv(mh2,
     mst12,mst12))*power5(mst12)))/(3.*mst12*sb) + (cb*mu*power4(snt)*(-(mh2*
     mst12*DeltaInv(mh2,mst12,mst12)) + 2*mh2*mst12*DeltaInv(mst22,mh2,mst12) -
     84*(3*mh2*mst22 + mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) + 84*power2(mh2)*(-((5*mst12*mst22 + 3*mst24)*power2(
     mh2)) + 3*mst22*power3(mh2) + (-5*mst12 + 3*mst22)*power3(mst22) - mh2*(
     14*mst12*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + 42*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) +
     mst12*DeltaInv(mst22,mst22,mh2)*(-mh2 + 42*DeltaInv(mst22,mst22,mh2)*(mh2*
     mst24 + 6*power3(mst22)) + 1008*mh2*power2(DeltaInv(mst22,mst22,mh2))*
     power4(mst22)) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12))
     )/(3.*mst12*sb) - (cb*mu*power2(Pi)*power2(snt)*(-2*msb12*(3*mh2*mst22 +
     mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) +
     msb12*mst12*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*power3(mst12)
     ) + 2*msb12*power2(mh2)*(-((5*mst12*mst22 + 3*mst24)*power2(mh2)) + 3*
     mst22*power3(mh2) + (-5*mst12 + 3*mst22)*power3(mst22) - mh2*(14*mst12*
     mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + mst12*(
     msb12*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 6*power3(mst22)) +
     power2(mw2)*(-((msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*
     mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(14*msb12*mst14*mw2 + 5*
     msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*
     mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) - power3(
     DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*power2(mw2)
     + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(mst22) - 3*
     mst22*power3(mw2) - 3*power4(mst22))) + 24*mh2*msb12*power3(DeltaInv(
     mst22,mst22,mh2))*power4(mst22)) + 24*mh2*msb12*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12)))/(3.*msb12*mst12*sb) - (cb*mu*log(mst12/Q2)*power4(
     snt)*(-mh2 - mst12 - mst22 + mst12*(-mh2 + 6*mst12)*mst22*DeltaInv(mh2,
     mst12,mst12) + DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) -
     (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)) + 24*
     mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)
     ) - 6*mh2*mst22*power2(DeltaInv(mst22,mh2,mst12))*(-(mst12*mst24) + mh2*(
     6*mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mh2) + power3(mh2) +
     power3(mst22)) + 72*mst22*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(
     mh2)) + 2*mst22*power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(
     5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + 288*
     mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(3.*mst12*
     mst22*sb) - (ca*mu*sa*power2(log(mh2/Q2))*((mst12 - mst22)*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) + 2*mst12*mt2*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 4*power3(mst12)) - (mst12 - mst22)*power3(mh2)*
     (-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) - 2*mst12*mt2*power2(
     DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power4(mst22)) + 32*mh2*mt2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12)))/(mst12*(mst12 - mst22)) - (ca*mu*sa*log(mh2/Q2)*
     log(mst12/Q2)*(4*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 +
     4*power3(mst12)) + (mst12 - mst22)*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12))*(3*mh2*mst22 + mst12*mst22 + mst24 + DeltaInv(mst22,mh2,mst12)*(5*
     (mst12*mst22 + mst24)*power2(mh2) - 3*mst22*power3(mh2) + (mst12 - mst22)*
     power3(mst22) - mh2*(-10*mst12*mst24 + power3(mst22)))) + 64*mh2*mt2*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(mst12*(mst12 - mst22))
     - (cb*mu*power2(log(mh2/Q2))*((mst12 - mst22)*mst22*power2(DeltaInv(mst22,
     mh2,mst12))*power3(mh2) + 4*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(
     mh2*mst14 + 4*power3(mst12)) - (mst12 - mst22)*power3(mh2)*(-2*mst12*mst24
     - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*power3(
     DeltaInv(mst22,mh2,mst12)) - 4*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))
     *(mh2*mst24 + 4*power3(mst22) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power4(
     mst22)) + 64*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(
     mst12*(mst12 - mst22)*sb) + (cb*mu*power2(sa)*power2(log(mh2/Q2))*((mst12
     - mst22)*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 4*mst12*
     mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*power3(mst12)) - (
     mst12 - mst22)*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) - 4*
     mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 4*power3(mst22) +
     16*mh2*DeltaInv(mst22,mst22,mh2)*power4(mst22)) + 64*mh2*mt2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(mst12*(mst12 - mst22)*sb) - (
     cb*mu*log(mh2/Q2)*log(mst12/Q2)*(8*mst12*mt2*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 4*power3(mst12)) + (mst12 - mst22)*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12))*(3*mh2*mst22 + mst12*mst22 + mst24 +
     DeltaInv(mst22,mh2,mst12)*(5*(mst12*mst22 + mst24)*power2(mh2) - 3*mst22*
     power3(mh2) + (mst12 - mst22)*power3(mst22) - mh2*(-10*mst12*mst24 +
     power3(mst22)))) + 128*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12)))/(mst12*(mst12 - mst22)*sb) + (cb*mu*log(mh2/Q2)*log(mst12/Q2)*
     power2(sa)*(8*mst12*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 4*
     power3(mst12)) + (mst12 - mst22)*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12))*(3*mh2*mst22 + mst12*mst22 + mst24 + DeltaInv(mst22,mh2,mst12)*(5*
     (mst12*mst22 + mst24)*power2(mh2) - 3*mst22*power3(mh2) + (mst12 - mst22)*
     power3(mst22) - mh2*(-10*mst12*mst24 + power3(mst22)))) + 128*mh2*mt2*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)))/(mst12*(mst12 - mst22)*
     sb) - (ca*mu*sa*power2(snt)*(-(mh2*mst12*DeltaInv(mh2,mst12,mst12)) + mH2*
     mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*DeltaInv(mH2,mst22,mst12) +
     mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*mst12*DeltaInv(mst22,mh2,
     mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) + 84*mst12*mst22*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) -
     84*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 252*mst24*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 42*mh2*mst12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) - 252*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 252*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) -
     252*mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 252*mst12*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 1176*mst12*mst24*power3(
     mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 420*mst12*power2(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 252*power3(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) - 1176*mst12*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) - 420*mst12*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 252*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) +
     252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 42*power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - 42*
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) -
     252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 1008*
     mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 252*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(mH2,
     mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))
     /(3.*mst12) + (cb*mu*power2(sa)*power2(snt)*(-(mh2*mst12*DeltaInv(mh2,
     mst12,mst12)) + mH2*mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*
     DeltaInv(mH2,mst22,mst12) + mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*
     mst12*DeltaInv(mst22,mh2,mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) +
     84*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) - 84*mst12*mst22*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 252*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) + 42*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 252*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 252*mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*power3(mst22) + 252*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 1176*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) +
     420*mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) +
     252*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 1176*
     mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 252*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mH2) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(
     mst12)) - 42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*
     power4(mst12)) - 252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) - 1008*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) +
     252*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*
     mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(
     mH2,mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))
     *power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12))
     )/(3.*mst12*sb) + (ca*mu*sa*power4(snt)*(-(mh2*mst12*DeltaInv(mh2,mst12,
     mst12)) + mH2*mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*DeltaInv(mH2,
     mst22,mst12) + mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*mst12*DeltaInv(
     mst22,mh2,mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) + 84*mst12*mst22*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*power2(mH2)*
     power2(DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(DeltaInv(
     mH2,mst22,mst22)) - 84*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 252*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 42*
     mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) + 252*mst22*power2(DeltaInv(mH2,
     mst22,mst12))*power3(mH2) - 252*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 252*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22)
     + 1176*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 420*
     mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 252*
     power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 1176*mst12*
     mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*power2(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 252*power3(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12)
     )*power4(mH2) + 252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) +
     42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) -
     42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) -
     252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 1008*
     mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 252*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(mH2,
     mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))
     /(3.*mst12) - (cb*mu*power2(sa)*power4(snt)*(-(mh2*mst12*DeltaInv(mh2,
     mst12,mst12)) + mH2*mst12*DeltaInv(mH2,mst12,mst12) - 2*mH2*mst12*
     DeltaInv(mH2,mst22,mst12) + mH2*mst12*DeltaInv(mH2,mst22,mst22) + 2*mh2*
     mst12*DeltaInv(mst22,mh2,mst12) - mh2*mst12*DeltaInv(mst22,mst22,mh2) +
     84*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 252*mst24*
     power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 42*mH2*mst12*mst24*power2(
     DeltaInv(mH2,mst22,mst22)) - 84*mst12*mst22*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 252*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)
     ) + 42*mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 252*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 252*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 252*mst12*power2(DeltaInv(mH2,mst22,mst22)
     )*power3(mst22) + 252*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 1176*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) +
     420*mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) +
     252*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 1176*
     mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 252*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 420*mst12*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 252*mst24*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) + 420*mst12*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 252*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mH2) + 42*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(
     mst12)) - 42*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*
     power4(mst12)) - 252*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) - 1008*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) +
     252*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1008*
     mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 252*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 252*mst22*power3(DeltaInv(
     mH2,mst22,mst12))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,mst12,mst12))
     *power5(mst12) - 1008*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12))
     )/(3.*mst12*sb) - (ca*mu*sa*power2(Pi)*power2(snt)*(2*mst12*mst22*power2(
     mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 6*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,
     mst22)) - 2*mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*
     mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + mh2*mst12*mst24*
     power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(
     mH2) - 6*mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 6*mst12*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 28*mst12*mst24*power3(
     mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*power2(mH2)*power3(
     mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12)) - 10*mst12*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 6*power3(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 10*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) - 6*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 10*
     mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mst24*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mst22) - 24*mH2*mst12*power3(DeltaInv(
     mH2,mst22,mst22))*power4(mst22) + 6*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*
     power4(mst22) + 6*mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 6*
     mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*mh2*power3(
     DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*mH2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(3.*mst12) + (cb*mu*power2(Pi)*power2(sa)*
     power2(snt)*(2*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) +
     6*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - mH2*mst12*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) - 2*mst12*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 6*mst24*power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12)) + mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,mh2)) - 6*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 6*mst22*power2(DeltaInv(
     mH2,mst22,mst12))*power3(mH2) - 6*mst12*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 6*mst12*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) +
     28*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*
     power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(
     mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*
     power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*power2(mh2)*
     power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 6*power3(mh2)*power3(
     mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*mst22*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) - 6*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 10*mst12*mst22*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2)
     + 6*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(
     mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) - 24*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + 6*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) + 6*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 6*mst22*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*
     mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(3.*mst12*sb) + (ca*mu*sa*
     power2(Pi)*power4(snt)*(2*mst12*mst22*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + 6*mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) -
     mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*mst12*mst22*power2(
     mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + mh2*mst12*mst24*power2(DeltaInv(mst22,mst22,
     mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 6*mst12*power2(DeltaInv(
     mH2,mst22,mst22))*power3(mst22) + 6*mst12*power2(DeltaInv(mst22,mst22,mh2)
     )*power3(mst22) + 28*mst12*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) + 10*mst12*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,
     mst12)) + 6*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) -
     28*mst12*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*
     power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 6*power3(
     mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 10*mst12*mst22*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 6*mst24*power3(DeltaInv(
     mst22,mh2,mst12))*power4(mh2) + 10*mst12*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 6*mst24*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mH2) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst12*mst14 + 6*power4(
     mst12)) - power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 6*power4(
     mst12)) - 6*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     24*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 6*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 24*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 6*mst22*power3(DeltaInv(
     mst22,mh2,mst12))*power5(mh2) - 6*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2) + 24*mh2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 24*
     mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(3.*mst12) - (cb*mu*
     power2(Pi)*power2(sa)*power4(snt)*(2*mst12*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mst24*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) - mH2*mst12*mst24*power2(DeltaInv(mH2,mst22,mst22)) - 2*
     mst12*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*mst24*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + mh2*mst12*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 6*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 6*
     mst12*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 6*mst12*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 28*mst12*mst24*power3(mH2)*
     power3(DeltaInv(mH2,mst22,mst12)) + 10*mst12*power2(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) + 6*power3(mH2)*power3(mst22)*power3(
     DeltaInv(mH2,mst22,mst12)) - 28*mst12*mst24*power3(mh2)*power3(DeltaInv(
     mst22,mh2,mst12)) - 10*mst12*power2(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 6*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 10*mst12*mst22*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) -
     6*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 10*mst12*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mst24*power3(DeltaInv(
     mH2,mst22,mst12))*power4(mH2) + power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst12*mst14 + 6*power4(mst12)) - power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst12*mst14 + 6*power4(mst12)) - 6*power2(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 24*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power4(mst22) + 6*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 24*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 6*
     mst22*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 6*mst22*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) - 24*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/(3.*mst12*sb) + (4*ca*mu*sa*log(mH2/Q2)*log(mst12/Q2)*power2(snt)
     *(power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst22 - mst12*
     mst22 - mst24 + DeltaInv(mH2,mst22,mst12)*(-5*(mst12*mst22 + mst24)*
     power2(mH2) + 3*mst22*power3(mH2) + (-mst12 + mst22)*power3(mst22) + mH2*(
     -10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(mH2,mst12,mst12))*(
     mH2*mst12*mst14 + 4*power4(mst12)) + 16*mH2*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12)))/mst12 - (4*cb*mu*log(mH2/Q2)*log(mst12/Q2)*power2(
     sa)*power2(snt)*(power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*
     mst22 - mst12*mst22 - mst24 + DeltaInv(mH2,mst22,mst12)*(-5*(mst12*mst22 +
     mst24)*power2(mH2) + 3*mst22*power3(mH2) + (-mst12 + mst22)*power3(mst22)
     + mH2*(-10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(mH2,mst12,
     mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + 16*mH2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(mst12*sb) - (4*ca*mu*sa*log(mH2/Q2)*log(
     mst12/Q2)*power4(snt)*(power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(-3*
     mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(mH2,mst22,mst12)*(-5*(mst12*
     mst22 + mst24)*power2(mH2) + 3*mst22*power3(mH2) + (-mst12 + mst22)*
     power3(mst22) + mH2*(-10*mst12*mst24 + power3(mst22)))) + power2(DeltaInv(
     mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) + 16*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/mst12 + (4*cb*mu*log(mH2/Q2)*
     log(mst12/Q2)*power2(sa)*power4(snt)*(power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12))*(-3*mH2*mst22 - mst12*mst22 - mst24 + DeltaInv(mH2,mst22,
     mst12)*(-5*(mst12*mst22 + mst24)*power2(mH2) + 3*mst22*power3(mH2) + (-
     mst12 + mst22)*power3(mst22) + mH2*(-10*mst12*mst24 + power3(mst22)))) +
     power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(mst12)) +
     16*mH2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*sb) + (2*
     ca*mu*sa*power2(snt)*power2(log(mH2/Q2))*(-2*mst22*power2(DeltaInv(mH2,
     mst22,mst12))*power3(mH2) + 2*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*
     mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,
     mst22,mst12)) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*
     power4(mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*
     power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 16*mH2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/mst12 - (2*cb*mu*power2(
     sa)*power2(snt)*power2(log(mH2/Q2))*(-2*mst22*power2(DeltaInv(mH2,mst22,
     mst12))*power3(mH2) + 2*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(
     mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(
     mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 16*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*sb) - (2*ca*mu*sa*
     power2(log(mH2/Q2))*power4(snt)*(-2*mst22*power2(DeltaInv(mH2,mst22,mst12)
     )*power3(mH2) + 2*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(
     mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(
     mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 16*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/mst12 + (2*cb*mu*power2(sa)*
     power2(log(mH2/Q2))*power4(snt)*(-2*mst22*power2(DeltaInv(mH2,mst22,mst12)
     )*power3(mH2) + 2*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) + power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst12*mst14 + 4*power4(
     mst12)) + mst12*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*power3(
     mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 16*mH2*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*sb) + (ca*mu*sa*power2(
     log(mH2/Q2))*((mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*
     power3(mH2) + 2*mst12*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 +
     4*power3(mst12)) - (mst12 - mst22)*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(
     mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(
     mH2,mst22,mst12)) - 2*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*
     mst24 + 4*power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22))
     + 32*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*(
     mst12 - mst22)) + (ca*mu*sa*power2(Pi)*(-2*mh2*mst12*mst14*mt2*power2(
     DeltaInv(mh2,mst12,mst12)) + 2*mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,
     mst12,mst12)) + mst12*(mst12 - mst22)*mst22*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) + 3*(mst12 - mst22)*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) - 2*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)
     ) - mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 3*(mst12 - mst22)*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + 2*mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) - 3*(
     mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 3*(
     mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 12*
     mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 12*mst12*mt2*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 14*mst12*(mst12 - mst22)
     *mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 5*mst12*(mst12 -
     mst22)*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 3*(
     mst12 - mst22)*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12))
     - 14*mst12*(mst12 - mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 5*mst12*(mst12 - mst22)*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 3*(mst12 - mst22)*power3(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) - 5*mst12*(mst12 - mst22)*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 3*(mst12 - mst22)*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 5*mst12*(mst12 - mst22)*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 3*(mst12 - mst22)*mst24*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) - 12*mt2*power2(DeltaInv(
     mh2,mst12,mst12))*power4(mst12) + 12*mt2*power2(DeltaInv(mH2,mst12,mst12))
     *power4(mst12) - 3*(mst12 - mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 48*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))
     *power4(mst22) + 3*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) + 48*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))
     *power4(mst22) + 3*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))
     *power5(mh2) - 3*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2) - 48*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) +
     48*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*mst12*(
     mst12 - mst22)) + (ca*mu*sa*log(mH2/Q2)*log(mst12/Q2)*(4*mst12*mt2*power2(
     DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12)) + (mst12 - mst22)
     *power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(3*mH2*mst22 + mst12*mst22
     + mst24 + DeltaInv(mH2,mst22,mst12)*(5*(mst12*mst22 + mst24)*power2(mH2) -
     3*mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) - mH2*(-10*mst12*mst24
     + power3(mst22)))) + 64*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(
     mst12)))/(mst12*(mst12 - mst22)) - (cb*mu*power2(sa)*power2(log(mH2/Q2))*(
     (mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 4*
     mst12*mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12))
     - (mst12 - mst22)*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) - 4*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 4*
     power3(mst22) + 16*mH2*DeltaInv(mH2,mst22,mst22)*power4(mst22)) + 64*mH2*
     mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(mst12*(mst12 -
     mst22)*sb) - (cb*mu*power2(Pi)*power2(sa)*(-4*mh2*mst12*mst14*mt2*power2(
     DeltaInv(mh2,mst12,mst12)) + 4*mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,
     mst12,mst12)) + mst12*(mst12 - mst22)*mst22*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) + 3*(mst12 - mst22)*mst24*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12)) - 4*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)
     ) - mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 3*(mst12 - mst22)*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + 4*mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) - 3*(
     mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 3*(
     mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) - 24*
     mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*power3(mst22) + 24*mst12*mt2*
     power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) + 14*mst12*(mst12 - mst22)
     *mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) + 5*mst12*(mst12 -
     mst22)*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 3*(
     mst12 - mst22)*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12))
     - 14*mst12*(mst12 - mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 5*mst12*(mst12 - mst22)*power2(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) - 3*(mst12 - mst22)*power3(mh2)*power3(mst22)*
     power3(DeltaInv(mst22,mh2,mst12)) - 5*mst12*(mst12 - mst22)*mst22*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) - 3*(mst12 - mst22)*mst24*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mh2) + 5*mst12*(mst12 - mst22)*mst22*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 3*(mst12 - mst22)*mst24*
     power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) - 24*mt2*power2(DeltaInv(
     mh2,mst12,mst12))*power4(mst12) + 24*mt2*power2(DeltaInv(mH2,mst12,mst12))
     *power4(mst12) - 3*(mst12 - mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 96*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))
     *power4(mst22) + 3*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mst22) + 96*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))
     *power4(mst22) + 3*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))
     *power5(mh2) - 3*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2) - 96*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) +
     96*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*mst12*(
     mst12 - mst22)*sb) - (cb*mu*log(mH2/Q2)*log(mst12/Q2)*power2(sa)*(8*mst12*
     mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 4*power3(mst12)) + (
     mst12 - mst22)*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*(3*mH2*mst22
     + mst12*mst22 + mst24 + DeltaInv(mH2,mst22,mst12)*(5*(mst12*mst22 + mst24)
     *power2(mH2) - 3*mst22*power3(mH2) + (mst12 - mst22)*power3(mst22) - mH2*(
     -10*mst12*mst24 + power3(mst22)))) + 128*mH2*mt2*power3(DeltaInv(mH2,
     mst12,mst12))*power5(mst12)))/(mst12*(mst12 - mst22)*sb) + (ca*mu*sa*(2*
     mh2*mst12*mt2*DeltaInv(mh2,mst12,mst12) - 2*mH2*mst12*mt2*DeltaInv(mH2,
     mst12,mst12) - mH2*mst12*(mst12 - mst22)*DeltaInv(mH2,mst22,mst12) + 2*
     mH2*mst12*mt2*DeltaInv(mH2,mst22,mst22) + mh2*mst12*(mst12 - mst22)*
     DeltaInv(mst22,mh2,mst12) - 2*mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) -
     84*mh2*mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)) + 84*mH2*mst12*
     mst14*mt2*power2(DeltaInv(mH2,mst12,mst12)) + 42*mst12*(mst12 - mst22)*
     mst22*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*
     mst24*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12)) - 84*mH2*mst12*mst24*
     mt2*power2(DeltaInv(mH2,mst22,mst22)) - 42*mst12*(mst12 - mst22)*mst22*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 126*(mst12 - mst22)*mst24*
     power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*mh2*mst12*mst24*mt2*
     power2(DeltaInv(mst22,mst22,mh2)) - 126*(mst12 - mst22)*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) + 126*(mst12 - mst22)*mst22*power2(
     DeltaInv(mH2,mst22,mst12))*power3(mH2) - 504*mst12*mt2*power2(DeltaInv(
     mH2,mst22,mst22))*power3(mst22) + 504*mst12*mt2*power2(DeltaInv(mst22,
     mst22,mh2))*power3(mst22) + 588*mst12*(mst12 - mst22)*mst24*power3(mH2)*
     power3(DeltaInv(mH2,mst22,mst12)) + 210*mst12*(mst12 - mst22)*power2(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*
     power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 588*mst12*(
     mst12 - mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 210*
     mst12*(mst12 - mst22)*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 126*(mst12 - mst22)*power3(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) - 126*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mh2) + 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mH2,mst22,mst12))*power4(mH2) + 126*(mst12 - mst22)*mst24*power3(DeltaInv(
     mH2,mst22,mst12))*power4(mH2) - 504*mt2*power2(DeltaInv(mh2,mst12,mst12))*
     power4(mst12) + 504*mt2*power2(DeltaInv(mH2,mst12,mst12))*power4(mst12) -
     126*(mst12 - mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) - 2016*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) + 126*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))
     *power4(mst22) + 2016*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*
     power4(mst22) + 126*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12)
     )*power5(mh2) - 126*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12)
     )*power5(mH2) - 2016*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(
     mst12) + 2016*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(
     6.*mst12*(mst12 - mst22)) - (cb*mu*power2(sa)*(4*mh2*mst12*mt2*DeltaInv(
     mh2,mst12,mst12) - 4*mH2*mst12*mt2*DeltaInv(mH2,mst12,mst12) - mH2*mst12*(
     mst12 - mst22)*DeltaInv(mH2,mst22,mst12) + 4*mH2*mst12*mt2*DeltaInv(mH2,
     mst22,mst22) + mh2*mst12*(mst12 - mst22)*DeltaInv(mst22,mh2,mst12) - 4*
     mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) - 168*mh2*mst12*mst14*mt2*power2(
     DeltaInv(mh2,mst12,mst12)) + 168*mH2*mst12*mst14*mt2*power2(DeltaInv(mH2,
     mst12,mst12)) + 42*mst12*(mst12 - mst22)*mst22*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) - 168*mH2*mst12*mst24*mt2*power2(DeltaInv(mH2,
     mst22,mst22)) - 42*mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) - 126*(mst12 - mst22)*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + 168*mh2*mst12*mst24*mt2*power2(DeltaInv(
     mst22,mst22,mh2)) - 126*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) + 126*(mst12 - mst22)*mst22*power2(DeltaInv(mH2,mst22,
     mst12))*power3(mH2) - 1008*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*
     power3(mst22) + 1008*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(
     mst22) + 588*mst12*(mst12 - mst22)*mst24*power3(mH2)*power3(DeltaInv(mH2,
     mst22,mst12)) + 210*mst12*(mst12 - mst22)*power2(mH2)*power3(mst22)*
     power3(DeltaInv(mH2,mst22,mst12)) + 126*(mst12 - mst22)*power3(mH2)*
     power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) - 588*mst12*(mst12 -
     mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 210*mst12*(
     mst12 - mst22)*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12))
     - 126*(mst12 - mst22)*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) - 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12)
     )*power4(mh2) - 126*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mh2,mst12)
     )*power4(mh2) + 210*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) + 126*(mst12 - mst22)*mst24*power3(DeltaInv(mH2,mst22,
     mst12))*power4(mH2) - 1008*mt2*power2(DeltaInv(mh2,mst12,mst12))*power4(
     mst12) + 1008*mt2*power2(DeltaInv(mH2,mst12,mst12))*power4(mst12) - 126*(
     mst12 - mst22)*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22)
     - 4032*mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) +
     126*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 4032*mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) + 126*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power5(mh2) - 126*(mst12 - mst22)*mst22*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2) - 4032*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12)
     + 4032*mH2*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(6.*
     mst12*(mst12 - mst22)*sb) - (ca*mu*sa*log(mst12/Q2)*(4*(mh2 - 6*mst12)*
     mst12*mst22*mt2*DeltaInv(mh2,mst12,mst12) - 4*(mH2 - 6*mst12)*mst12*mst22*
     mt2*DeltaInv(mH2,mst12,mst12) - 96*mst12*mst22*mt2*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) + 96*mst12*mst22*mt2*power2(
     DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(mst12)) + (mst12 - mst22)
     *(-mh2 + mH2 + DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) -
     (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)) -
     DeltaInv(mH2,mst22,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*
     mst22)*power2(mH2) + 5*mH2*power2(mst22) + power3(mH2)) - 6*mh2*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 +
     23*mst24) - (mst12 - 23*mst22)*power2(mh2) + power3(mh2) + power3(mst22))
     + 6*mH2*mst22*power2(DeltaInv(mH2,mst22,mst12))*(-(mst12*mst24) + mH2*(6*
     mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mH2) + power3(mH2) +
     power3(mst22)) - 72*mst22*power2(mH2)*(-((3*mst12*mst22 + 2*mst24)*power2(
     mH2)) + 2*mst22*power3(mH2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mH2*(
     5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 72*
     mst22*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*
     power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12))) - 1152*mh2*mst22*mt2*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 1152*mH2*mst22*mt2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(12.*mst12*(mst12 -
     mst22)*mst22) + (cb*mu*log(mst12/Q2)*power2(sa)*(8*(mh2 - 6*mst12)*mst12*
     mst22*mt2*DeltaInv(mh2,mst12,mst12) - 8*(mH2 - 6*mst12)*mst12*mst22*mt2*
     DeltaInv(mH2,mst12,mst12) - 192*mst12*mst22*mt2*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 3*power3(mst12)) + 192*mst12*mst22*mt2*power2(
     DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(mst12)) + (mst12 - mst22)
     *(-mh2 + mH2 + DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) -
     (mst12 - 5*mst22)*power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)) -
     DeltaInv(mH2,mst22,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*
     mst22)*power2(mH2) + 5*mH2*power2(mst22) + power3(mH2)) - 6*mh2*mst22*
     power2(DeltaInv(mst22,mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 +
     23*mst24) - (mst12 - 23*mst22)*power2(mh2) + power3(mh2) + power3(mst22))
     + 6*mH2*mst22*power2(DeltaInv(mH2,mst22,mst12))*(-(mst12*mst24) + mH2*(6*
     mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mH2) + power3(mH2) +
     power3(mst22)) - 72*mst22*power2(mH2)*(-((3*mst12*mst22 + 2*mst24)*power2(
     mH2)) + 2*mst22*power3(mH2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mH2*(
     5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 72*
     mst22*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*mst22*
     power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12))) - 2304*mh2*mst22*mt2*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 2304*mH2*mst22*mt2*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12)))/(12.*mst12*(mst12 -
     mst22)*mst22*sb) + (cb*mu*log(mst12/Q2)*power2(snt)*(-2*mh2*msb12 - 2*
     msb12*mst12 - 2*msb12*mst22 - 2*msb12*(mh2 - 6*mst12)*mst12*mst22*
     DeltaInv(mh2,mst12,mst12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) -
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 84*mst12*mst14*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 2*msb12*DeltaInv(mst22,mh2,mst12)
     *(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mh2) + 5*mh2*
     power2(mst22) + power3(mh2)) + 48*msb12*mst12*mst22*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) - 12*mh2*msb12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*(-(mst12*mst24) + mh2*(6*mst12*mst22 + 23*
     mst24) - (mst12 - 23*mst22)*power2(mh2) + power3(mh2) + power3(mst22)) -
     6*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 144*
     msb12*mst22*power2(mh2)*(-((3*mst12*mst22 + 2*mst24)*power2(mh2)) + 2*
     mst22*power3(mh2) + (-3*mst12 + 2*mst22)*power3(mst22) - 2*mh2*(5*mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)) + 144*msb12*
     mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*
     mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*msb12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*mst22*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*mst12*
     mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 576*mh2*msb12*
     mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 72*mst22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)))/(6.*msb12*mst12*
     mst22*sb) - (cb*mt2*mu*log(mH2/Q2)*power2(sa)*(-mst12 + mst22 + (mH2 - 6*
     mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mst12*mst22*(-mH2 + 6*
     mst22)*DeltaInv(mH2,mst22,mst22) + 12*mst12*mst22*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) - 12*mst12*mst22*power2(
     DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 12*power3(mst22)) + 576*mH2*mst22*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) - 576*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(3.*mst12*(mst12 - mst22)*
     mst22*sb) - (ca*mu*sa*log(mH2/Q2)*power2(snt)*(mH2 + mst12 + mst22 + (mH2
     - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*
     DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*
     mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 +
     power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*
     mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,
     mst22,mst22))*power4(mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12) + 576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power5(mst22)))/(3.*mst12*mst22) + (cb*mu*log(mH2/Q2)*power2(sa)*power2(
     snt)*(mH2 + mst12 + mst22 + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,
     mst12,mst12) + mH2*mst12*mst22*DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 +
     5*mst22)*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,
     mst22,mst22)*power2(mst22) + 6*mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(
     mst12 + 14*mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12))
     + 12*mH2*mst12*mst22*mst24*power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*
     mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) +
     72*mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) +
     mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,mst12)) +
     144*mst12*power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 576*mH2*mst22*
     power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + 576*mH2*mst12*power3(
     DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(3.*mst12*mst22*sb) + (ca*mu*
     sa*log(mH2/Q2)*power4(snt)*(mH2 + mst12 + mst22 + (mH2 - 6*mst12)*mst12*
     mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*DeltaInv(mH2,mst22,
     mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,mst12)*power2(mH2) -
     6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*mst22*power2(mH2)*(-3*
     mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 + power2(mH2))*power2(
     DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*power2(DeltaInv(mH2,
     mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*
     mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(
     mst12*mst22 + mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(
     mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,mst22,mst22))*power4(
     mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) +
     576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(3.*mst12*
     mst22) - (cb*mu*log(mH2/Q2)*power2(sa)*power4(snt)*(mH2 + mst12 + mst22 +
     (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,mst12,mst12) + mH2*mst12*mst22*
     DeltaInv(mH2,mst22,mst22) - (mH2 - mst12 + 5*mst22)*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 6*mst12*DeltaInv(mH2,mst22,mst22)*power2(mst22) + 6*
     mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*mst22) + mst24 +
     power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 12*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst22)) + 12*mst12*mst22*power2(DeltaInv(mH2,
     mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mH2)*(-2*
     mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power3(
     mst22))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*power2(DeltaInv(mH2,
     mst22,mst22))*power4(mst22) + 576*mH2*mst22*power3(DeltaInv(mH2,mst12,
     mst12))*power5(mst12) + 576*mH2*mst12*power3(DeltaInv(mH2,mst22,mst22))*
     power5(mst22)))/(3.*mst12*mst22*sb) + (ca*mu*sa*log(mH2/Q2)*(mH2*mst12 -
     mH2*mst22 + 4*mst12*mt2 - 4*mst22*mt2 - 4*(mH2 - 6*mst12)*mst12*mst22*mt2*
     DeltaInv(mH2,mst12,mst12) + 4*mH2*mst12*mst22*mt2*DeltaInv(mH2,mst22,
     mst22) + (-mH2 + mst12 - 5*mst22)*(mst12 - mst22)*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 24*mst12*mt2*DeltaInv(mH2,mst22,mst22)*power2(mst22)
     + 6*(mst12 - mst22)*mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*
     mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 48*mH2*
     mst12*mst22*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) - 48*mst12*mst22*
     mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*
     (mst12 - mst22)*mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) + 576*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) -
     2304*mH2*mst22*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + 2304*
     mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(12.*
     mst12*(mst12 - mst22)*mst22) - (cb*mu*log(mH2/Q2)*power2(sa)*(mH2*mst12 -
     mH2*mst22 + 8*mst12*mt2 - 8*mst22*mt2 - 8*(mH2 - 6*mst12)*mst12*mst22*mt2*
     DeltaInv(mH2,mst12,mst12) + 8*mH2*mst12*mst22*mt2*DeltaInv(mH2,mst22,
     mst22) + (-mH2 + mst12 - 5*mst22)*(mst12 - mst22)*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 48*mst12*mt2*DeltaInv(mH2,mst22,mst22)*power2(mst22)
     + 6*(mst12 - mst22)*mst22*power2(mH2)*(-3*mst12*mst22 - mH2*(mst12 + 14*
     mst22) + mst24 + power2(mH2))*power2(DeltaInv(mH2,mst22,mst12)) + 96*mH2*
     mst12*mst22*mst24*mt2*power2(DeltaInv(mH2,mst22,mst22)) - 96*mst12*mst22*
     mt2*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 12*power3(mst12)) + 72*
     (mst12 - mst22)*mst22*power3(mH2)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 +
     mst24) + mst22*power2(mH2) + power3(mst22))*power3(DeltaInv(mH2,mst22,
     mst12)) + 1152*mst12*mt2*power2(DeltaInv(mH2,mst22,mst22))*power4(mst22) -
     4608*mH2*mst22*mt2*power3(DeltaInv(mH2,mst12,mst12))*power5(mst12) + 4608*
     mH2*mst12*mt2*power3(DeltaInv(mH2,mst22,mst22))*power5(mst22)))/(12.*
     mst12*(mst12 - mst22)*mst22*sb) + (ca*mu*sa*log(mst22/Q2)*((mst12 - mst22)
     *(5*mH2*mst22 - mst12*mst22 + mst24)*DeltaInv(mH2,mst22,mst12) + 4*mst12*(
     mH2 - 6*mst22)*mt2*DeltaInv(mH2,mst22,mst22) - 5*mh2*mst12*mst22*DeltaInv(
     mst22,mh2,mst12) - mst12*mst24*DeltaInv(mst22,mh2,mst12) + mst22*mst24*
     DeltaInv(mst22,mh2,mst12) - 4*mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) +
     24*mst12*mst22*mt2*DeltaInv(mst22,mst22,mh2) + mst22*DeltaInv(mst22,mh2,
     mst12)*power2(mst12) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) + 6*mh2*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mh2,mst12)) - 84*mst12*mst24*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12)) + 84*mst22*mst24*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12)) - 6*mh2*mst24*power2(mst12)*power2(DeltaInv(mst22,mh2,
     mst12)) - 18*mst22*power2(mh2)*power2(mst12)*power2(DeltaInv(mst22,mh2,
     mst12)) + 18*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + 96*mh2*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) - 6*power2(
     mst22)*power2(DeltaInv(mst22,mh2,mst12))*power3(mh2) + 6*mh2*mst12*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mst22) + 288*mst12*mt2*power2(DeltaInv(
     mst22,mst22,mh2))*power3(mst22) - 6*mH2*(mst12 - mst22)*power2(DeltaInv(
     mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*mH2*mst24 - mst12*mst24 +
     mst22*power2(mH2) + power3(mst22)) - 96*mst12*mt2*power2(DeltaInv(mH2,
     mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) - 72*(mst12 - mst22)*power2(
     mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(mst22) - 2*mH2*(mst12*
     mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,mst12)) + 144*mst12*
     mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 144*mst24*
     power2(mst12)*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) - 144*power2(
     mh2)*power2(mst12)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) - 144*
     mst12*power3(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,mst12)) + 72*
     mst12*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 72*mst22*
     mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 6*mh2*power2(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) - 1152*mH2*mst12*mt2*power3(
     DeltaInv(mH2,mst22,mst22))*power4(mst22) + 216*mst12*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*power3(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power4(mst22) + 1152*mh2*mst12*mt2*power3(
     DeltaInv(mst22,mst22,mh2))*power4(mst22) - 72*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*power5(mst22)))/(12.*mst12*(-mst12 + mst22)) + (cb*mu*
     log(mst22/Q2)*power2(sa)*((mst12 - mst22)*(5*mH2*mst22 - mst12*mst22 +
     mst24)*DeltaInv(mH2,mst22,mst12) + 8*mst12*(mH2 - 6*mst22)*mt2*DeltaInv(
     mH2,mst22,mst22) - 5*mh2*mst12*mst22*DeltaInv(mst22,mh2,mst12) - mst12*
     mst24*DeltaInv(mst22,mh2,mst12) + mst22*mst24*DeltaInv(mst22,mh2,mst12) -
     8*mh2*mst12*mt2*DeltaInv(mst22,mst22,mh2) + 48*mst12*mst22*mt2*DeltaInv(
     mst22,mst22,mh2) + mst22*DeltaInv(mst22,mh2,mst12)*power2(mst12) + 5*mh2*
     DeltaInv(mst22,mh2,mst12)*power2(mst22) - mst12*DeltaInv(mst22,mh2,mst12)*
     power2(mst22) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mh2,mst12))
     - 84*mst12*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) + 84*mst22*
     mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)) - 6*mh2*mst24*power2(
     mst12)*power2(DeltaInv(mst22,mh2,mst12)) - 18*mst22*power2(mh2)*power2(
     mst12)*power2(DeltaInv(mst22,mh2,mst12)) + 18*mst12*power2(mh2)*power2(
     mst22)*power2(DeltaInv(mst22,mh2,mst12)) + 192*mh2*mst12*mst24*mt2*power2(
     DeltaInv(mst22,mst22,mh2)) + 6*mst12*mst22*power2(DeltaInv(mst22,mh2,
     mst12))*power3(mh2) - 6*power2(mst22)*power2(DeltaInv(mst22,mh2,mst12))*
     power3(mh2) + 6*mh2*mst12*power2(DeltaInv(mst22,mh2,mst12))*power3(mst22)
     + 576*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22) - 6*mH2*(
     mst12 - mst22)*power2(DeltaInv(mH2,mst22,mst12))*(-3*mH2*mst12*mst22 - 14*
     mH2*mst24 - mst12*mst24 + mst22*power2(mH2) + power3(mst22)) - 192*mst12*
     mt2*power2(DeltaInv(mH2,mst22,mst22))*(mH2*mst24 + 3*power3(mst22)) - 72*(
     mst12 - mst22)*power2(mH2)*(mst24*power2(mH2) + (-2*mst12 + mst22)*power3(
     mst22) - 2*mH2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mH2,mst22,
     mst12)) + 144*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 144*mst24*power2(mst12)*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 144*power2(mh2)*power2(mst12)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) - 144*mst12*power3(mh2)*power3(mst22)*power3(DeltaInv(
     mst22,mh2,mst12)) + 72*mst12*mst24*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) - 72*mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2)
     - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(mst22) - 2304*mH2*mst12*
     mt2*power3(DeltaInv(mH2,mst22,mst22))*power4(mst22) + 216*mst12*power2(
     mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*power3(mh2)*
     power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 2304*mh2*mst12*mt2*
     power3(DeltaInv(mst22,mst22,mh2))*power4(mst22) - 72*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(12.*mst12*(mst12 - mst22)*sb)
     + (ca*mu*sa*log(mst12/Q2)*power2(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*mst12)
     *mst22*DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(
     mH2,mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) + mst22*mst24*
     DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mh2) +
     5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*
     mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,mst12)*power3(
     mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) -
     24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(
     mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*
     mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(3.*mst12*mst22) - (cb*mu*log(
     mst12/Q2)*power2(sa)*power2(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*mst12)*
     mst22*DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(
     mH2,mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) + mst22*mst24*
     DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mh2) +
     5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*
     mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,mst12)*power3(
     mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) -
     24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(
     mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*
     mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(3.*mst12*mst22*sb) - (ca*mu*
     sa*log(mst12/Q2)*power4(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*mst12)*mst22*
     DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(mH2,
     mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) + mst22*mst24*
     DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mh2) +
     5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*
     mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,mst12)*power3(
     mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) -
     24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(
     mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*
     mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(3.*mst12*mst22) + (cb*mu*log(
     mst12/Q2)*power2(sa)*power4(snt)*(-mh2 + mH2 + mst12*(-mh2 + 6*mst12)*
     mst22*DeltaInv(mh2,mst12,mst12) + (mH2 - 6*mst12)*mst12*mst22*DeltaInv(
     mH2,mst12,mst12) - mst22*mst24*DeltaInv(mH2,mst22,mst12) + mst22*mst24*
     DeltaInv(mst22,mh2,mst12) - mst12*DeltaInv(mst22,mh2,mst12)*power2(mh2) +
     5*mst22*DeltaInv(mst22,mh2,mst12)*power2(mh2) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mH2) - 5*mst22*DeltaInv(mH2,mst22,mst12)*power2(mH2) - 5*
     mH2*DeltaInv(mH2,mst22,mst12)*power2(mst22) + mst12*DeltaInv(mH2,mst22,
     mst12)*power2(mst22) + 5*mh2*DeltaInv(mst22,mh2,mst12)*power2(mst22) -
     mst12*DeltaInv(mst22,mh2,mst12)*power2(mst22) - 6*mH2*mst12*mst22*mst24*
     power2(DeltaInv(mH2,mst22,mst12)) + 138*mst22*mst24*power2(mH2)*power2(
     DeltaInv(mH2,mst22,mst12)) + 36*mst12*power2(mH2)*power2(mst22)*power2(
     DeltaInv(mH2,mst22,mst12)) + 6*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mh2,mst12)) - 138*mst22*mst24*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) - 36*mst12*power2(mh2)*power2(mst22)*power2(DeltaInv(mst22,mh2,
     mst12)) + DeltaInv(mst22,mh2,mst12)*power3(mh2) + 6*mst12*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 138*power2(mst22)*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - DeltaInv(mH2,mst22,mst12)*power3(
     mH2) - 6*mst12*mst22*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 138*
     power2(mst22)*power2(DeltaInv(mH2,mst22,mst12))*power3(mH2) + 24*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*power3(mst12)) -
     24*mst12*mst22*power2(DeltaInv(mH2,mst12,mst12))*(mH2*mst14 + 3*power3(
     mst12)) + 720*mst12*mst22*mst24*power3(mH2)*power3(DeltaInv(mH2,mst22,
     mst12)) - 720*mst12*mst22*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,
     mst12)) - 6*mst22*power2(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 144*
     mst22*mst24*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) - 216*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power4(mh2) + 6*mst22*
     power2(DeltaInv(mH2,mst22,mst12))*power4(mH2) + 144*mst22*mst24*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 216*mst12*power2(mst22)*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2) + 6*mH2*power2(DeltaInv(mH2,mst22,
     mst12))*power4(mst22) - 6*mh2*power2(DeltaInv(mst22,mh2,mst12))*power4(
     mst22) + 216*mst12*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(
     mst22) + 144*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12))*power4(mst22) -
     216*mst12*power2(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) -
     144*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12))*power4(mst22) + 144*
     power2(mst22)*power3(DeltaInv(mst22,mh2,mst12))*power5(mh2) - 144*power2(
     mst22)*power3(DeltaInv(mH2,mst22,mst12))*power5(mH2) + 288*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 288*mH2*mst22*power3(
     DeltaInv(mH2,mst12,mst12))*power5(mst12) - 144*power2(mH2)*power3(
     DeltaInv(mH2,mst22,mst12))*power5(mst22) + 144*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*power5(mst22)))/(3.*mst12*mst22*sb) - (cb*mt2*
     mu*log(mh2/Q2)*(-mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,
     mst12,mst12) + mst12*mst22*(-mh2 + 6*mst22)*DeltaInv(mst22,mst22,mh2) +
     12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(
     mst12)) - 12*mst12*mst22*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 +
     12*power3(mst22)) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) - 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(
     mst22)))/(3.*mst12*(mst12 - mst22)*mst22*sb) + (cb*mt2*mu*log(mh2/Q2)*
     power2(sa)*(-mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,
     mst12,mst12) + mst12*mst22*(-mh2 + 6*mst22)*DeltaInv(mst22,mst22,mh2) +
     12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(
     mst12)) - 12*mst12*mst22*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 +
     12*power3(mst22)) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) - 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(
     mst22)))/(3.*mst12*(mst12 - mst22)*mst22*sb) + (ca*mu*sa*log(mh2/Q2)*
     power2(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(
     mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(mst22,mst22,mh2) - (mh2 -
     mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*mst12*DeltaInv(
     mst22,mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-3*mst12*mst22 -
     mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,
     mst12)) + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 12*
     mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(
     mst12)) + 72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,
     mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 576*
     mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 576*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(3.*mst12*mst22) + (cb*
     mu*log(mh2/Q2)*power2(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*mst12)*mst12*
     mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(mst22,mst22,
     mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*
     mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-3*
     mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(
     DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(
     mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*
     mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(
     mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(
     mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(
     mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) +
     576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(3.*mst12*
     mst22*sb) - (cb*mu*log(mh2/Q2)*power2(sa)*power2(snt)*(mh2 + mst12 + mst22
     + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*
     DeltaInv(mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,
     mst12)*power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*
     mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 +
     power2(mh2))*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*
     power2(DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,
     mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*
     mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(
     mst22))*power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(
     mst22,mst22,mh2))*power4(mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) + 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*
     power5(mst22)))/(3.*mst12*mst22*sb) - (ca*mu*sa*log(mh2/Q2)*power4(snt)*(
     mh2 + mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*DeltaInv(mh2,mst12,
     mst12) + mh2*mst12*mst22*DeltaInv(mst22,mst22,mh2) - (mh2 - mst12 + 5*
     mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*mst12*DeltaInv(mst22,
     mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(
     mst12 + 14*mst22) + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,mst12))
     + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,mh2)) + 12*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) +
     72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) +
     144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 576*mh2*mst22*
     power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 576*mh2*mst12*power3(
     DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(3.*mst12*mst22) - (cb*mu*log(
     mh2/Q2)*power4(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*mst12)*mst12*mst22*
     DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(mst22,mst22,mh2) - (
     mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) - 6*mst12*
     DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*power2(mh2)*(-3*mst12*
     mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2))*power2(DeltaInv(
     mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,
     mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*
     power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22
     + mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,
     mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) + 576*
     mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 576*mh2*mst12*
     power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(3.*mst12*mst22*sb) + (
     cb*mu*log(mh2/Q2)*power2(sa)*power4(snt)*(mh2 + mst12 + mst22 + (mh2 - 6*
     mst12)*mst12*mst22*DeltaInv(mh2,mst12,mst12) + mh2*mst12*mst22*DeltaInv(
     mst22,mst22,mh2) - (mh2 - mst12 + 5*mst22)*DeltaInv(mst22,mh2,mst12)*
     power2(mh2) - 6*mst12*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*mst22*
     power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 + power2(mh2)
     )*power2(DeltaInv(mst22,mh2,mst12)) + 12*mh2*mst12*mst22*mst24*power2(
     DeltaInv(mst22,mst22,mh2)) + 12*mst12*mst22*power2(DeltaInv(mh2,mst12,
     mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*mst22*power3(mh2)*(-2*mst12*
     mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power3(mst22))*
     power3(DeltaInv(mst22,mh2,mst12)) + 144*mst12*power2(DeltaInv(mst22,mst22,
     mh2))*power4(mst22) + 576*mh2*mst22*power3(DeltaInv(mh2,mst12,mst12))*
     power5(mst12) + 576*mh2*mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(
     mst22)))/(3.*mst12*mst22*sb) - (ca*mu*sa*log(mh2/Q2)*(mh2*mst12 - mh2*
     mst22 + 4*mst12*mt2 - 4*mst22*mt2 - 4*(mh2 - 6*mst12)*mst12*mst22*mt2*
     DeltaInv(mh2,mst12,mst12) + 4*mh2*mst12*mst22*mt2*DeltaInv(mst22,mst22,
     mh2) + (-mh2 + mst12 - 5*mst22)*(mst12 - mst22)*DeltaInv(mst22,mh2,mst12)*
     power2(mh2) - 24*mst12*mt2*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*(
     mst12 - mst22)*mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22)
     + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,mst12)) + 48*mh2*mst12*
     mst22*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) - 48*mst12*mst22*mt2*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*(
     mst12 - mst22)*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,
     mst12)) + 576*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) -
     2304*mh2*mst22*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 2304*
     mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(12.*
     mst12*(mst12 - mst22)*mst22) - (cb*mu*log(mh2/Q2)*(mh2*mst12 - mh2*mst22 +
     8*mst12*mt2 - 8*mst22*mt2 - 8*(mh2 - 6*mst12)*mst12*mst22*mt2*DeltaInv(
     mh2,mst12,mst12) + 8*mh2*mst12*mst22*mt2*DeltaInv(mst22,mst22,mh2) + (-mh2
     + mst12 - 5*mst22)*(mst12 - mst22)*DeltaInv(mst22,mh2,mst12)*power2(mh2) -
     48*mst12*mt2*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*(mst12 - mst22)*
     mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22) + mst24 +
     power2(mh2))*power2(DeltaInv(mst22,mh2,mst12)) + 96*mh2*mst12*mst22*mst24*
     mt2*power2(DeltaInv(mst22,mst22,mh2)) - 96*mst12*mst22*mt2*power2(
     DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*(mst12 -
     mst22)*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) +
     mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,mst12)) +
     1152*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) - 4608*mh2*
     mst22*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 4608*mh2*
     mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(12.*mst12*(
     mst12 - mst22)*mst22*sb) + (cb*mu*log(mh2/Q2)*power2(sa)*(mh2*mst12 - mh2*
     mst22 + 8*mst12*mt2 - 8*mst22*mt2 - 8*(mh2 - 6*mst12)*mst12*mst22*mt2*
     DeltaInv(mh2,mst12,mst12) + 8*mh2*mst12*mst22*mt2*DeltaInv(mst22,mst22,
     mh2) + (-mh2 + mst12 - 5*mst22)*(mst12 - mst22)*DeltaInv(mst22,mh2,mst12)*
     power2(mh2) - 48*mst12*mt2*DeltaInv(mst22,mst22,mh2)*power2(mst22) + 6*(
     mst12 - mst22)*mst22*power2(mh2)*(-3*mst12*mst22 - mh2*(mst12 + 14*mst22)
     + mst24 + power2(mh2))*power2(DeltaInv(mst22,mh2,mst12)) + 96*mh2*mst12*
     mst22*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)) - 96*mst12*mst22*mt2*
     power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 12*power3(mst12)) + 72*(
     mst12 - mst22)*mst22*power3(mh2)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power3(mst22))*power3(DeltaInv(mst22,mh2,
     mst12)) + 1152*mst12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power4(mst22) -
     4608*mh2*mst22*mt2*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) + 4608*
     mh2*mst12*mt2*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22)))/(12.*
     mst12*(mst12 - mst22)*mst22*sb) + (mu*log(mst22/Q2)*power3(cb)*(msb12*(
     mst12 - mst22)*(5*mA2*mst22 - mst12*mst22 + mst24)*DeltaInv(mst22,mA2,
     mst12) + 2*mst12*(5*mC2*mst22 - msb12*mst22 + mst24)*(mst12 - mst22 - 2*
     mt2)*DeltaInv(mst22,msb12,mC2) + 2*mst12*mst22*mst24*DeltaInv(mst22,mw2,
     msb12) - 4*msb12*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12) + 4*mst12*
     mst24*mt2*DeltaInv(mst22,mw2,msb12) + 20*mst12*mst22*mt2*mw2*DeltaInv(
     mst22,mw2,msb12) - msb12*mst12*mst24*DeltaInv(mst22,mz2,mst12) + msb12*
     mst22*mst24*DeltaInv(mst22,mz2,mst12) - 5*msb12*mst12*mst22*mz2*DeltaInv(
     mst22,mz2,mst12) + 2*msb12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12) -
     2*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst12) - 10*mst22*mw2*DeltaInv(
     mst22,mw2,msb12)*power2(mst12) + msb12*mst22*DeltaInv(mst22,mz2,mst12)*
     power2(mst12) - 2*msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) +
     10*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*
     DeltaInv(mst22,mz2,mst12)*power2(mst22) + 5*msb12*mz2*DeltaInv(mst22,mz2,
     mst12)*power2(mst22) + 12*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) + 24*msb12*mst12*mst24*mt2*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) - 12*msb12*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12)) + 168*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 72*msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 336*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - 36*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 168*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 36*msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*msb12*mst12*mst22*mst24*mz2*power2(DeltaInv(mst22,mz2,
     mst12)) - 6*msb12*mst24*mz2*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12)
     ) - 84*msb12*mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) +
     84*msb12*mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 18*
     msb12*mst22*power2(mst12)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) +
     18*msb12*mst12*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))
     - 24*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 12*
     mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*
     msb12*mst12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power3(mst22) - 6*mA2*
     msb12*(mst12 - mst22)*power2(DeltaInv(mst22,mA2,mst12))*(-3*mA2*mst12*
     mst22 - 14*mA2*mst24 - mst12*mst24 + mst22*power2(mA2) + power3(mst22)) -
     12*mC2*mst12*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-
     3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) +
     power3(mst22)) - 24*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mz2) - 6*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mz2) - 72*msb12*(mst12 - mst22)*power2(mA2)*(mst24*power2(mA2) + (-
     2*mst12 + mst22)*power3(mst22) - 2*mA2*(mst12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,mA2,mst12)) - 144*mst12*(mst12 - mst22 - 2*mt2)*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 576*
     msb12*mst12*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)
     ) - 288*msb12*power2(mst12)*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12)) + 288*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 576*msb12*mst12*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 288*msb12*mst24*power2(mst12)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 576*mst12*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 288*power2(mst12)*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*power2(mst12)*
     power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 144*msb12*
     mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 144*
     msb12*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) -
     144*msb12*mst12*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)
     ) - 12*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*
     msb12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 288*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 288*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     144*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) + 288*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) + 216*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) + 144*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 288*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 144*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 72*msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 72*msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mst22) - 72*msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power5(mst22)))/(12.*msb12*mst12*(mst12 - mst22)*sb) + (cb*mu*sb*log(
     mst22/Q2)*(msb12*(mst12 - mst22)*(5*mA2*mst22 - mst12*mst22 + mst24)*
     DeltaInv(mst22,mA2,mst12) - 2*mst12*(5*mC2*mst22 - msb12*mst22 + mst24)*(-
     mst12 + mst22 + mt2)*DeltaInv(mst22,msb12,mC2) + 2*mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb12) - 2*msb12*mst12*mst22*mt2*DeltaInv(mst22,mw2,
     msb12) + 2*mst12*mst24*mt2*DeltaInv(mst22,mw2,msb12) + 10*mst12*mst22*mt2*
     mw2*DeltaInv(mst22,mw2,msb12) - msb12*mst12*mst24*DeltaInv(mst22,mz2,
     mst12) + msb12*mst22*mst24*DeltaInv(mst22,mz2,mst12) - 5*msb12*mst12*
     mst22*mz2*DeltaInv(mst22,mz2,mst12) + 2*msb12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mst12) - 2*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst12) -
     10*mst22*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst12) + msb12*mst22*
     DeltaInv(mst22,mz2,mst12)*power2(mst12) - 2*msb12*mst12*DeltaInv(mst22,
     mw2,msb12)*power2(mst22) + 10*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) - msb12*mst12*DeltaInv(mst22,mz2,mst12)*power2(mst22) + 5*msb12*
     mz2*DeltaInv(mst22,mz2,mst12)*power2(mst22) + 12*msb12*mst12*mst22*mst24*
     mw2*power2(DeltaInv(mst22,mw2,msb12)) + 12*msb12*mst12*mst24*mt2*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) - 12*msb12*mst24*mw2*power2(mst12)*
     power2(DeltaInv(mst22,mw2,msb12)) + 168*mst12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*mst12*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 168*mst12*mst24*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 168*mst24*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*mst12*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*mst24*
     mz2*power2(DeltaInv(mst22,mz2,mst12)) - 6*msb12*mst24*mz2*power2(mst12)*
     power2(DeltaInv(mst22,mz2,mst12)) - 84*msb12*mst12*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + 84*msb12*mst22*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - 18*msb12*mst22*power2(mst12)*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 18*msb12*mst12*power2(mst22)*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 12*mst12*mt2*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mst22) + 12*mw2*power2(mst12)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*msb12*mst12*mz2*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mst22) - 6*mA2*msb12*(mst12 - mst22)*
     power2(DeltaInv(mst22,mA2,mst12))*(-3*mA2*mst12*mst22 - 14*mA2*mst24 -
     mst12*mst24 + mst22*power2(mA2) + power3(mst22)) + 12*mC2*mst12*(-mst12 +
     mst22 + mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*
     mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 12*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 12*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 12*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*
     mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 6*msb12*power2(
     mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 72*msb12*(mst12 -
     mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(mst22) -
     2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) +
     144*mst12*(-mst12 + mst22 + mt2)*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) + 288*msb12*mst12*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 288*msb12*power2(mst12)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 288*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 288*
     msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 288*
     msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     288*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 288*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 144*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) + 144*msb12*mst12*mst22*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst24*power2(mst12)*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 144*msb12*mst12*power3(mst22)*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 12*mst12*mw2*power2(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*msb12*mz2*power2(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) + 288*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*power2(mst12)*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 288*mst12*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 144*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 144*
     mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*
     mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst24*
     power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 72*msb12*
     mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 72*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(12.*msb12*
     mst12*(mst12 - mst22)) + (cb*mu*sb*power2(log(mst22/Q2))*(msb12*(-mst12 +
     mst22)*mst24*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 2*mst12*
     mst24*(-mst12 + mst22 + mt2)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     - 2*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 2*
     mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mst24*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*mst12*
     mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst22*mst24*
     power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*(mst12 - mst22)*
     power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*power3(mst22) - 2*mA2*
     (mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) - 2*
     mst12*(-mst12 + mst22 + mt2)*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 +
     mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) - 4*msb12*mst12*mt2*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) + 4*msb12*power2(mst12)*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 4*msb12*mst24*power2(
     mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 4*mst12*mt2*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 4*power2(mst12)*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 2*msb12*
     power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12))
     - 2*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     + 2*msb12*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)
     ) + 2*msb12*mst12*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)) - 4*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 2*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 2*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 4*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 3*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mst22) - 2*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mst22) + 2*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 2*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 2*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(
     mz2) + msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) +
     2*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) +
     msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(
     msb12*mst12*(mst12 - mst22)) + (mu*power2(log(mst22/Q2))*power3(cb)*(
     msb12*(-mst12 + mst22)*mst24*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))
     + 2*mst12*mst24*(-mst12 + mst22 + 2*mt2)*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - 2*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 4*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 2*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + msb12*mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))
     - msb12*mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*
     (mst12 - mst22)*power2(mA2)*(mst24*power2(mA2) + (-2*mst12 + mst22)*
     power3(mst22) - 2*mA2*(mst12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,mA2,mst12)) + 2*mst12*(mst12 - mst22 - 2*mt2)*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 8*msb12*mst12*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 4*msb12*
     power2(mst12)*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12))
     - 4*msb12*mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 8*msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     4*msb12*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 8*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     + 4*power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + 2*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(
     mst22,mz2,mst12)) - 2*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + 2*msb12*mst24*power2(mst12)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)) + 2*msb12*mst12*power3(mst22)*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) - 4*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 4*mst12*mt2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 2*power2(mst12)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 4*mst12*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*mst12*power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 2*msb12*power3(mz2)
     *power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 2*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 4*mst12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 2*mst24*power2(mst12)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - msb12*mst12*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) + msb12*mst22*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) + 2*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + msb12*power2(mz2)*power3(
     DeltaInv(mst22,mz2,mst12))*power5(mst22)))/(msb12*mst12*(mst12 - mst22)*
     sb) + (cb*mt2*mu*(-3*msb12*mst12 + 3*msb12*mst22 - 2*mh2*msb12*mst12*
     mst22*DeltaInv(mh2,mst12,mst12) + 2*mh2*msb12*mst12*mst22*DeltaInv(mst22,
     mst22,mh2) + 2*msb12*mst12*mst22*mw2*DeltaInv(mst22,mw2,msb12) - 2*msb12*
     mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) - 252*mst12*mst22*mst24*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*msb12*mst12*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 252*mst12*mst14*mst22*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*mst12*
     mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*power3(mst12)) -
     84*msb12*mst12*mst22*power2(DeltaInv(mst22,mst22,mh2))*(mh2*mst24 + 6*
     power3(mst22)) - 252*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) + 252*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 1176*msb12*mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 1176*msb12*mst12*mst14*mst22*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 420*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 252*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 420*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*mst12*mst22*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 420*msb12*mst12*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 252*mst12*mst14*mst22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 420*msb12*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2016*mh2*msb12*
     mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 252*mst22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 2016*mh2*msb12*
     mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22) + 252*mst12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 252*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 252*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(6.*msb12*mst12*(
     mst12 - mst22)*mst22*sb) - (cb*mu*power2(snt)*(-(msb12*mst12) + msb12*
     mst22 - 2*mh2*msb12*mst12*mst22*DeltaInv(mh2,mst12,mst12) + 4*mh2*msb12*
     mst12*mst22*DeltaInv(mst22,mh2,mst12) - 2*mh2*msb12*mst12*mst22*DeltaInv(
     mst22,mst22,mh2) + 2*msb12*mst12*mst22*mw2*DeltaInv(mst22,mw2,msb12) - 2*
     msb12*mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) - 168*msb12*mst22*(3*mh2*
     mst22 + mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + 84*mh2*msb12*mst12*mst22*mst24*power2(DeltaInv(mst22,mst22,mh2))
     - 252*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     84*msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     + 252*mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     84*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     + 84*msb12*mst12*mst22*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 6*
     power3(mst12)) - 252*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) + 252*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 1176*msb12*mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 1176*msb12*mst12*mst14*mst22*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 420*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 252*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 504*msb12*mst12*power2(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) - 420*msb12*mst12*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mst22) - 252*mst12*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mst22) + 168*msb12*mst22*power2(mh2)*power3(
     DeltaInv(mst22,mh2,mst12))*(-14*mh2*mst12*mst24 - 5*mst12*mst22*power2(
     mh2) - 3*mst24*power2(mh2) + 3*mst22*power3(mh2) - 3*mh2*power3(mst22) -
     5*mst12*power3(mst22) + 3*power4(mst22)) - 252*mst12*mst22*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 420*msb12*mst12*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 252*mst12*mst14*mst22*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 420*msb12*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2016*mh2*msb12*
     mst22*power3(DeltaInv(mh2,mst12,mst12))*power5(mst12) - 252*mst22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 2016*mh2*msb12*
     mst12*power3(DeltaInv(mst22,mst22,mh2))*power5(mst22) + 252*mst12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 252*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 252*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(6.*msb12*mst12*
     mst22*sb) + (cb*mu*sb*log(msb12/Q2)*power2(snb)*power2(snt)*(mC2*mst12 -
     mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*mst24*DeltaInv(mst22,mw2,
     msb12) - mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) + msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)
     + 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*msb12*mst12*mst22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*
     mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,
     msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (
     msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*
     msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*
     DeltaInv(mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 216*msb12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*mst22) + (mt2*mu*log(msb12/Q2)*power3(cb)*(mC2*
     mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*mst24*DeltaInv(
     mst22,mw2,msb12) - mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) + msb12*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*
     mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(
     mst22,mw2,msb12)*power2(mw2) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*
     power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*
     msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*
     mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*
     msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*
     mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*
     mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) +
     mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) -
     mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 216*msb12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (mu*log(msb12/Q2)*
     power2(snb)*power2(snt)*power3(cb)*(mC2*mst12 - mC2*mst22 - mst12*mw2 +
     mst22*mw2 + mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) - mst12*mst14*
     mst22*DeltaInv(mw2,msb12,mst12) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) -
     msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) +
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*msb12*mst12*mst22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(
     msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2)
     )*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*
     mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) +
     mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*DeltaInv(mw2,msb12,
     mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) - 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*
     msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 216*msb12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 216*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(6.*msb12*
     mst12*mst22*sb) + (cb*mu*sb*log(msb12/Q2)*power2(snt)*(-(mC2*mst12) + mC2*
     mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*DeltaInv(mst22,mw2,
     msb12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) - msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*DeltaInv(mw2,msb12,
     mst12)*power2(mst12) + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)
     - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + msb12*mst12*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) +
     5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 6*msb12*mst12*mst22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 138*mst12*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*mst12*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*
     mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 138*mst12*mst14*mst22*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 36*msb12*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - mst22*DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,
     msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(
     msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (
     msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*
     msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) + mst22*
     DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 216*msb12*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*mst22) + (mt2*mu*log(msb12/Q2)*power2(snb)*power3(
     cb)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) -
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22)
     + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*
     msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 36*
     msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) -
     mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2)
     + mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 216*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (mu*log(msb12/Q2)*
     power2(snt)*power3(cb)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 -
     mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*DeltaInv(
     mw2,msb12,mst12) - msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) +
     5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst12*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,
     msb12)*power2(mst22) + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) -
     5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) - 6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) + 138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 36*msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12)) - 138*mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 36*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) +
     mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (
     msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*
     msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb12)*
     power3(mw2) + mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*
     mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*
     mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*power2(
     mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*mst12*mst22*
     power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-
     ((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12
     + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 216*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*mst22*sb) + (cb*mu*sb*power2(snb)*power2(snt)*(-(
     mC2*msb12*msb22*DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(
     msb22,mst12,mC2) + mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*
     msb22*DeltaInv(mst22,msb22,mC2) - msb12*msb22*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*msb22*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mw2*
     DeltaInv(mw2,msb12,mst12) - msb12*msb22*mw2*DeltaInv(mw2,msb22,mst12) +
     42*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*msb12*msb22*
     mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*msb12*msb22*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 42*msb12*msb22*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 126*msb12*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 42*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 126*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 42*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) + 126*msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     126*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 126*msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb22*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 126*msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb22*mst12*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) + 126*msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 42*msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     - 588*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     210*msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 126*msb22*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     + 588*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     210*msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)
     ) + 126*msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2))
     + 210*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) + 588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + 126*msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 210*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,
     mw2,msb22)) - 588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 126*msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 210*msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)) - 588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 126*msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 210*msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(
     mw2,msb22,mst12)) + 588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 126*msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 210*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 126*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) + 210*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2)
     + 126*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*
     msb12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 -
     5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*
     mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22) - (
     mt2*mu*power2(snb)*power3(cb)*(-(mC2*msb12*msb22*DeltaInv(msb12,mst12,mC2)
     ) + mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*DeltaInv(
     mst22,msb12,mC2) - mC2*msb12*msb22*DeltaInv(mst22,msb22,mC2) - msb12*
     msb22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*mw2*DeltaInv(mst22,mw2,
     msb22) + msb12*msb22*mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*mw2*
     DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)
     *power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*
     msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*msb12*
     msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*msb22*mst22*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*msb12*mst24*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) - 126*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) + 126*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2) + 126*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) -
     126*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 126*msb12*mst12*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 42*msb22*power2(mC2)*(-((
     5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 +
     3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 588*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 126*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 588*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 126*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 210*msb12*msb22*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 588*msb12*
     msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*msb12*
     msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 588*
     msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 210*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb12*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*
     mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*
     power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*(
     mst12 - mst22)*sb) + (mu*power2(snb)*power2(snt)*power3(cb)*(-(mC2*msb12*
     msb22*DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(msb22,mst12,
     mC2) + mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*
     DeltaInv(mst22,msb22,mC2) - msb12*msb22*mw2*DeltaInv(mst22,mw2,msb12) +
     msb12*msb22*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mw2*DeltaInv(mw2,
     msb12,mst12) - msb12*msb22*mw2*DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*
     mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - 42*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - 42*msb12*msb22*mst22*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 42*msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + 42*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 126*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 42*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) - 126*msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     42*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 126*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 126*msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 126*msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb22*mst22*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) - 126*msb12*mst22*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mw2) - 126*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 126*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mw2) - 42*msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*
     mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*
     msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*
     msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*
     msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     126*msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     210*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)
     ) + 588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     126*msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     210*msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)
     ) - 588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     126*msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     210*msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)
     ) - 588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     126*msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     210*msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)
     ) + 588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) +
     126*msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) -
     210*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb12*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*
     mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*
     power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*sb)
     + (cb*mu*sb*power2(Pi)*power2(snb)*power2(snt)*(msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) - msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     + msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*
     msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*msb22*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) + msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) + 3*msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) - 3*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 3*msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 3*msb22*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 3*msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb22*mst12*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) + msb22*power2(mC2)*((5*msb12*mst12 + 3*mst14)*power2(
     mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*
     msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*
     msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 14*
     msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 3*
     msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     14*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 5*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) -
     14*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 3*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 5*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     14*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     14*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*
     mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 3*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + msb12*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(
     mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) -
     5*msb22*power3(mst12) + 3*power4(mst12)) + 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) - 3*msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 3*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*
     msb12*mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb22*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 3*msb12*mst22*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) + 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 3*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2))
     )/(3.*msb12*msb22) + (mu*power2(Pi)*power2(snb)*power2(snt)*power3(cb)*(
     msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb12*msb22*mst22*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 3*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 3*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - msb12*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 3*msb22*mst22*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) + 3*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 3*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     - 3*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*power2(mC2)*((5*
     msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*
     mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 14*msb12*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) + 14*msb12*msb22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 3*msb12*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb12*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 3*msb22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 3*msb12*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + msb12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*
     power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(
     mst12) + 3*power4(mst12)) + 3*msb22*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 3*msb12*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22) - 3*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 3*msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 3*msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 3*msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*msb12*msb22*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb12*mst14*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst22*power3(DeltaInv(
     mst22,msb22,mC2))*power5(mC2) - 3*msb22*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 3*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)
     - 3*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*
     msb22*sb) + (mt2*mu*power2(Pi)*power2(snb)*power3(cb)*(-(msb22*(3*mC2*
     mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2))) + msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) - 3*msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))
     - msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 3*
     msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) + 3*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) - 3*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 3*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 3*msb22*mst12*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 3*msb12*mst12*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*power2(mC2)*(-((5*msb12*
     mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)
     *power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 14*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 5*msb12*msb22*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*power3(mst22)*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 14*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 3*msb12*power3(mst22)*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb22)) + 5*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 14*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*power3(mst12)*power3(mw2)
     *power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*power3(mst12)*power3(mw2)
     *power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 3*msb22*mst24*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 3*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2)
     + msb12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(14*mC2*msb22*mst14
     + 5*msb22*mst12*power2(mC2) + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) +
     3*mC2*power3(mst12) + 5*msb22*power3(mst12) - 3*power4(mst12)) - 3*msb22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 3*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2) + 5*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 3*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mw2) - 3*msb12*mst14*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mw2) - 3*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mC2) + 3*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst22*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 3*msb22*mst12*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) + 3*msb12*mst12*power3(DeltaInv(
     mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*(mst12 - mst22)*sb) + (mu*
     power2(snb)*power3(cb)*(-2*mC2*msb12*msb22*mt2*DeltaInv(msb12,mst12,mC2) +
     2*mC2*msb12*msb22*mt2*DeltaInv(msb22,mst12,mC2) - mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*mst22*DeltaInv(mst22,msb12,
     mC2) + 2*mC2*msb12*msb22*mt2*DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*
     mst12*DeltaInv(mst22,msb22,mC2) - mC2*msb12*msb22*mst22*DeltaInv(mst22,
     msb22,mC2) - 2*mC2*msb12*msb22*mt2*DeltaInv(mst22,msb22,mC2) + msb12*
     msb22*mst12*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*mw2*
     DeltaInv(mst22,mw2,msb12) - 2*msb12*msb22*mt2*mw2*DeltaInv(mst22,mw2,
     msb12) - msb12*msb22*mst12*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*
     mst22*mw2*DeltaInv(mst22,mw2,msb22) + 2*msb12*msb22*mt2*mw2*DeltaInv(
     mst22,mw2,msb22) + 2*msb12*msb22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 2*
     msb12*msb22*mt2*mw2*DeltaInv(mw2,msb22,mst12) + 84*msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     84*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mt2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + 42*msb12*msb22*mst12*mst22*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 126*msb22*mst12*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst22*mst24*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 84*msb12*msb22*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 252*msb22*mst24*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 42*msb12*msb22*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 42*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 126*msb12*mst12*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst22*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*msb22*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 252*msb12*mst24*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*power2(mC2)
     *power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 42*msb12*msb22*mst12*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 126*msb22*mst12*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*mst22*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 84*msb12*msb22*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 252*msb22*mst24*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*msb12*msb22*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*msb12*msb22*
     mst12*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 126*msb12*
     mst12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*msb12*
     mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*
     msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 252*msb12*
     mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*msb22*
     power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*
     msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 252*msb22*
     mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*msb22*
     mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 252*msb12*mst14*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mst12*mst22*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 252*msb22*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 252*msb12*mst22*mt2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 126*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 252*msb22*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 252*msb12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 252*msb22*mst12*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 252*msb12*mst12*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 84*msb22*mt2*power2(mC2)*(
     -((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 588*msb12*msb22*mst12*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 588*msb12*msb22*mst22*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 1176*msb12*msb22*mst24*mt2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 210*msb12*msb22*mst12*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 420*msb12*
     msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     126*msb22*mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 252*msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) - 588*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 588*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 1176*msb12*msb22*mst24*mt2*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 210*msb12*msb22*mst12*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 420*msb12*msb22*mt2*power2(mC2)
     *power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 126*msb12*mst12*power3(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 252*msb12*mt2*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 210*msb12*
     msb22*mst12*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     420*msb12*msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) - 588*msb12*msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 588*msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 1176*msb12*msb22*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 126*msb22*mst12*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 252*msb22*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 210*msb12*msb22*mst12*power2(mw2)
     *power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 420*msb12*msb22*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 588*msb12*
     msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 588*
     msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     1176*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) +
     126*msb12*mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)
     ) - 252*msb12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22)) - 420*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(
     mw2,msb12,mst12)) - 1176*msb12*msb22*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 252*msb22*mt2*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 420*msb12*msb22*mt2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 1176*msb12*msb22*mst14*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 252*msb12*mt2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 210*msb12*msb22*
     mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 126*msb22*
     mst12*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*msb22*
     mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 420*msb12*
     msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 252*msb22*
     mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 210*msb12*msb22*
     power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 210*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 126*
     msb12*mst12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*
     msb12*mst22*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 420*
     msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 252*
     msb12*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 210*msb12*
     msb22*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 252*
     msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     252*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     + 84*msb12*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) - 210*msb12*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mst22) - 126*msb22*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) + 252*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 126*msb22*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 210*msb12*msb22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mst22) + 126*msb12*mst12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 252*msb12*mt2*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 126*msb12*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 210*msb12*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb22*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*msb22*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*
     msb22*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 210*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) -
     126*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mst22) + 252*msb12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 126*msb12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 126*msb22*mst12*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 126*msb22*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 420*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 252*msb22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) + 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) + 126*msb12*mst12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 126*msb12*mst22*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 420*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 252*msb12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 420*msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 252*msb22*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 420*msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mw2) + 252*msb12*mst14*mt2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) - 126*msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 252*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 126*msb22*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 126*msb12*mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 252*msb12*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 126*msb12*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) + 126*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mst22) - 126*msb12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mst22) - 126*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mst22) + 126*msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mst22) + 126*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 252*msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 126*msb22*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 126*msb12*mst12*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 252*msb12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 126*msb12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 252*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 252*msb12*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mw2)))/(3.*msb12*msb22*(mst12 - mst22)*sb) + (cb*mu*sb*power2(snb)*
     (-(mC2*msb12*msb22*mt2*DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*mt2*
     DeltaInv(msb22,mst12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,
     mC2) + mC2*msb12*msb22*mst22*DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*
     mt2*DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,
     msb22,mC2) - mC2*msb12*msb22*mst22*DeltaInv(mst22,msb22,mC2) - mC2*msb12*
     msb22*mt2*DeltaInv(mst22,msb22,mC2) + msb12*msb22*mst12*mw2*DeltaInv(
     mst22,mw2,msb12) - msb12*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) -
     msb12*msb22*mt2*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst12*mw2*
     DeltaInv(mst22,mw2,msb22) + msb12*msb22*mst22*mw2*DeltaInv(mst22,mw2,
     msb22) + msb12*msb22*mt2*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mt2*
     mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*mt2*mw2*DeltaInv(mw2,msb22,
     mst12) + 42*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*mt2*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*mC2*mst12 + msb22*mst12 +
     3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 42*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 126*
     msb22*mst12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*mst22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 42*
     msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 42*msb12*
     msb22*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 42*
     msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) -
     126*msb12*mst12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*
     msb12*mst22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*
     msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*
     msb12*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*
     msb22*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 42*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     126*msb22*mst12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*
     msb22*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*
     msb12*msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*
     msb22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*msb12*
     msb22*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 42*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     126*msb12*mst12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*
     msb12*msb22*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*
     msb22*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*
     msb12*msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 126*
     msb22*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*
     msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*
     mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb22*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb12*mst22*mt2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 126*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb22*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb22*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 126*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb22*mst12*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 126*msb12*mst12*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 42*msb22*mt2*power2(mC2)*(
     -((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 588*msb12*msb22*mst12*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 588*msb12*msb22*mst22*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 588*msb12*msb22*mst24*mt2*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 210*msb12*msb22*mst12*power2(mC2)
     *power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 126*msb22*
     mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) -
     588*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     + 588*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     ) + 588*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     ) - 210*msb12*msb22*mst12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) + 210*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) - 126*msb12*mst12*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mt2*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 210*msb12*msb22*mst12*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 210*msb12*msb22*
     mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 588*
     msb12*msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     588*msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     + 588*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 126*msb22*mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + 126*msb22*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 210*msb12*msb22*mst12*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb22)) - 210*msb12*msb22*mt2*power2(mw2)*power3(mst22)
     *power3(DeltaInv(mst22,mw2,msb22)) + 588*msb12*msb22*mst12*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 588*msb12*msb22*mst22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 588*msb12*msb22*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 126*msb12*mst12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 126*msb12*mt2*
     power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 210*msb12*
     msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     588*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     126*msb22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     + 210*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,
     mst12)) + 588*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + 126*msb12*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) + 126*msb22*mst12*mst24*power3(DeltaInv(mst22,msb12,mC2)
     )*power4(mC2) - 126*msb22*mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 210*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 126*msb22*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 210*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) - 126*msb12*mst12*mst24*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 126*msb12*mst22*mst24*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 210*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 126*msb12*mst24*mt2*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mC2) + 210*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 126*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 126*msb12*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 42*msb12*mt2*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(
     mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) -
     5*msb22*power3(mst12) + 3*power4(mst12)) - 210*msb12*msb22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb22*mst12*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 126*msb22*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb22*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 210*msb12*
     msb22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 126*
     msb12*mst12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) -
     126*msb12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)
     + 126*msb12*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) +
     210*msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) + 126*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 126*msb22*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) + 126*msb22*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 210*msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 126*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 126*msb12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power4(mst22) - 126*msb12*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power4(mst22) - 210*msb12*msb22*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 126*msb22*mst12*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*mst22*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 210*msb12*msb22*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 210*msb12*msb22*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 210*msb12*msb22*mst12*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 126*msb12*mst12*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*
     mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*
     msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*
     mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*
     mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*
     mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 126*msb22*mst12*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 126*msb22*mst22*mt2*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 126*msb22*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 126*msb12*mst12*mst22*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) - 126*msb12*mst22*mt2*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) - 126*msb12*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 126*msb22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mst22) - 126*msb12*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power5(mst22) - 126*msb22*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 126*msb12*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mst22) + 126*msb22*mst12*mst22*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 126*msb22*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 126*msb22*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 126*msb12*mst12*mst22*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 126*msb12*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 126*msb12*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 126*msb22*mst12*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*mst12*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*(mst12 -
     mst22)) + (mu*power2(Pi)*power2(snb)*power3(cb)*(2*msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     2*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mt2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst22*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*msb22*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*msb22*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst12*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst22*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + msb12*msb22*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst12*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb22*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst12*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst22*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 2*msb12*msb22*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 6*msb12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 2*msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 6*msb22*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 6*msb12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst12*mst22*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 6*msb22*mst22*mt2*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 3*msb22*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)
     )*power3(mC2) - 3*msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 6*msb12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2) + 3*msb12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 3*msb22*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*
     msb22*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb22*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb12*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb12*mst22*
     mt2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mt2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 2*msb22*mt2*power2(mC2)*((5*
     msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*
     mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 14*msb12*msb22*mst12*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst22*mst24*power3(mC2)
     *power3(DeltaInv(mst22,msb12,mC2)) - 28*msb12*msb22*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*mst12*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 10*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*
     mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 6*
     msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) -
     14*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     14*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     28*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     5*msb12*msb22*mst12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) + 10*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) - 3*msb12*mst12*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*mt2*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*mst12*power2(mw2)*power3(mst22)
     *power3(DeltaInv(mst22,mw2,msb12)) + 10*msb12*msb22*mt2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst12*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*
     mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 28*msb12*
     msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*
     mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 6*
     msb22*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*
     msb12*msb22*mst12*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22)) - 10*msb12*msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb22)) + 14*msb12*msb22*mst12*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) - 14*msb12*msb22*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 28*msb12*msb22*mst24*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 6*msb12*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 10*msb12*msb22*mt2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 28*msb12*msb22*mst14*
     mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*msb22*mt2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 10*msb12*msb22*mt2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 28*msb12*
     msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*msb12*
     mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*
     mst12*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst22*
     mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 10*msb12*msb22*
     mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 6*msb22*mst24*
     mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*
     mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*mst12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst22*mst24*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 10*msb12*msb22*mst22*mt2*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 6*msb12*mst24*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 5*msb12*msb22*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 6*msb22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*msb12*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 2*msb12*mt2*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*
     power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(
     mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) - 5*msb12*msb22*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb22*mst12*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 6*msb22*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb22*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 5*msb12*
     msb22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*
     msb12*mst12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) -
     6*msb12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) +
     3*msb12*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 5*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     3*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     - 6*msb22*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     + 3*msb22*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 5*
     msb12*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) -
     3*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22)
     + 6*msb12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22)
     - 3*msb12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*
     msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*
     msb22*mst12*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*
     mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 10*msb12*
     msb22*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*msb22*
     mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 10*msb12*msb22*
     mst22*mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*msb12*mst24*
     mt2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 10*msb12*msb22*
     mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 6*msb22*mst14*
     mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 10*msb12*msb22*mst12*
     mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 6*msb12*mst14*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst12*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 6*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 6*msb12*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mst22) - 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mst22) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 3*msb22*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 6*msb22*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 6*msb12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 6*msb22*mst12*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 6*msb12*mst12*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*(mst12 - mst22)*
     sb) + (cb*mu*sb*power2(Pi)*power2(snb)*(msb22*(3*mC2*mst12 + msb12*mst12 +
     3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*
     mst12 + msb22*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) + msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + 3*msb22*mst12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) - 3*msb22*mst22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     - msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*
     msb22*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*
     mst12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*
     mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*power2(mC2)*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst12*mst22*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst12*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst22*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst24*mt2*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst12*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst12*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + msb12*msb22*mst12*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst12*mst22*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 3*msb22*mst22*mt2*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 3*msb22*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)
     )*power3(mC2) - 3*msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 3*msb12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2) + 3*msb12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 3*msb22*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*
     msb22*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb22*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb12*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb12*mst22*
     mt2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb22*mst12*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*mt2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*mt2*power2(mC2)*((5*msb12*
     mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*
     power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 14*msb12*msb22*mst12*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*mst12*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*
     mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) -
     14*msb12*msb22*mst12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     14*msb12*msb22*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     14*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     5*msb12*msb22*mst12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) + 5*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) - 3*msb12*mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) + 3*msb12*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) - 5*msb12*msb22*mst12*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) + 5*msb12*msb22*mt2*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst12*mst24*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*mst22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*mst24*mt2*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst12*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mt2*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*msb22*mst12*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 5*msb12*
     msb22*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) +
     14*msb12*msb22*mst12*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     14*msb12*msb22*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     14*msb12*msb22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) +
     3*msb12*mst12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))
     - 3*msb12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))
     - 5*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,
     mst12)) - 14*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 3*msb22*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 5*msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb22,mst12)) + 14*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 3*msb12*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 5*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,msb12,mC2)
     )*power4(mC2) + 3*msb22*mst12*mst24*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 3*msb22*mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 5*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 3*msb22*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) - 5*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 5*msb12*msb22*mst12*mst22*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 3*msb12*mst12*mst24*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb12*mst22*mst24*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 5*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb12*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mC2) + 5*msb12*msb22*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 3*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) + msb12*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-
     14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*
     mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*
     power4(mst12)) - 5*msb12*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) - 3*msb22*mst12*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 3*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 3*msb22*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 5*msb12*msb22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mst22) + 3*msb12*mst12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb12*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb12*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) + 5*msb12*msb22*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb22*mst12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb22*mt2*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb22*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 5*msb12*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 3*msb12*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 3*msb12*mt2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 3*msb12*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*msb12*
     msb22*mst12*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*msb22*
     mst12*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*mst22*
     mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*
     mst12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*msb12*mst12*
     mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*mst12*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*msb22*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*msb12*msb22*mst12*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb12*mst14*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst12*mst22*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb12*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mst22) - 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mst22) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 3*msb22*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb22*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb22*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst12*mst22*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb12*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb22*mst12*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*mst12*mt2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22*(mst12 - mst22))
     + (cb*mu*power2(snb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*
     mst22 + 2*msb12*msb22*(-mst12 + mst22 + 2*mt2)*mw2*DeltaInv(mst22,mw2,
     msb12) + 2*msb12*msb22*(mst12 - mst22 - 2*mt2)*mw2*DeltaInv(mst22,mw2,
     msb22) - 4*msb12*msb22*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 4*msb12*msb22*
     mt2*mw2*DeltaInv(mw2,msb22,mst12) + 84*msb22*(mst12 - mst22 - 2*mt2)*(
     msb12*mst22 + 3*(mst24 + mst22*mw2))*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 84*msb12*(mst12 - mst22 - 2*mt2)*(msb22*mst22 + 3*(mst24 +
     mst22*mw2))*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 168*msb12*
     msb22*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 504*msb22*
     mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 168*msb12*msb22*
     mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 504*msb12*mst14*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 504*msb22*mst12*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 504*msb12*mst12*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 840*msb12*msb22*mt2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 2352*msb12*
     msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 504*msb22*
     mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 840*
     msb12*msb22*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)
     ) - 2352*msb12*msb22*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 504*msb12*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 504*msb22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) + 504*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 84*msb22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*(14*msb12*mst24*mw2 + 5*msb12*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb12*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) - 84*msb12*(mst12 - mst22
     - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2
     + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22)
     + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)) + 840*
     msb12*msb22*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 504*
     msb22*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 840*msb12*
     msb22*mst12*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 504*msb12*
     mst14*mt2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 504*msb22*mst12*
     mt2*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 504*msb12*mst12*mt2*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22*(mst12 -
     mst22)*sb) + (mt2*mu*log(msb22/Q2)*power2(snb)*power3(cb)*(mC2*mst12 -
     mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*mst24*DeltaInv(mst22,mw2,
     msb22) - mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) + msb22*mst22*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,msb22,
     mst12)*power2(mst12) - msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     + 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22) - msb22*mst12*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*mst22*DeltaInv(mst22,mw2,
     msb22)*power2(mw2) + msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 6*msb22*mst12*mst22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) - 138*mst12*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*msb22*mst12*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*
     mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*mst12*mst14*mst22*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*msb22*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2) - mst22*
     DeltaInv(mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 216*msb22*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mst22) - 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) -
     216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*(mst12 - mst22)*mst22*sb) + (cb*mu*sb*log(msb22/Q2)
     *power2(snb)*power2(snt)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2
     - mst12*mst22*mst24*DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*
     DeltaInv(mw2,msb22,mst12) - msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(
     mst12) + 5*mst22*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*
     mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22) - 5*mst12*mw2*DeltaInv(
     mst22,mw2,msb22)*power2(mst22) + msb22*mst12*DeltaInv(mst22,mw2,msb22)*
     power2(mw2) - 5*mst12*mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*
     mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb22)) + 138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 36*msb22*mst12*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) - 138*mst12*mst14*mst22*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 36*msb22*mst22*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - mst22*DeltaInv(msb22,mst12,mC2)*(
     mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(
     msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2)
     )*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) -
     mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2) + mst22*DeltaInv(mw2,msb22,
     mst12)*power3(mw2) - 6*msb22*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))
     *power3(mw2) + 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) + 6*msb22*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) - 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 720*msb22*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 720*
     msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*
     mst22*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 216*msb22*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 216*msb22*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 216*msb22*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 216*msb22*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mst12) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(mst22) - 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb22*
     mst12*mst22) + (mu*log(msb22/Q2)*power2(snb)*power2(snt)*power3(cb)*(-(
     mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) -
     msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*DeltaInv(mst22,mw2,
     msb22)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     + mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     + 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*mst22*sb) + (cb*mu*log(mst12/Q2)*(2/mst12 + 2/mst22
     + mh2/(mst12*mst22) + mz2/(mst12*mst22) - (8*(mh2 - 6*mst12)*mt2*DeltaInv(
     mh2,mst12,mst12))/(mst12 - mst22) + mst22*DeltaInv(mst22,mz2,mst12) - (
     mst24*DeltaInv(mst22,mz2,mst12))/mst12 - (5*mst22*mz2*DeltaInv(mst22,mz2,
     mst12))/mst12 - (4*mst12*mt2*DeltaInv(mw2,msb12,mst12))/(mst12 - mst22) +
     (4*mst14*mt2*DeltaInv(mw2,msb12,mst12))/(msb12*mst12 - msb12*mst22) + (20*
     mst12*mt2*mw2*DeltaInv(mw2,msb12,mst12))/(msb12*mst12 - msb12*mst22) - (5*
     DeltaInv(mst22,mz2,mst12)*power2(mz2))/mst12 + (DeltaInv(mst22,mz2,mst12)*
     power2(mz2))/mst22 - 6*mst24*mz2*power2(DeltaInv(mst22,mz2,mst12)) + 36*
     mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + (138*mst24*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12)))/mst12 + (24*mst14*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (72*mst12*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (336*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 - msb12*mst22)
     - (DeltaInv(mst22,mh2,mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*
     mst22)*power2(mh2) + 5*mh2*power2(mst22) + power3(mh2)))/(mst12*mst22) - (
     24*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power3(mst12))/(msb12*mst12 -
     msb12*mst22) + (192*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 + 3*
     power3(mst12)))/(mst12 - mst22) + (6*mz2*power2(DeltaInv(mst22,mz2,mst12))
     *power3(mst22))/mst12 + (6*mh2*power2(DeltaInv(mst22,mh2,mst12))*(-(mst12*
     mst24) + mh2*(6*mst12*mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mh2) +
     power3(mh2) + power3(mst22)))/mst12 - (24*mst12*mt2*power2(DeltaInv(mw2,
     msb12,mst12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (DeltaInv(mst22,
     mz2,mst12)*power3(mz2))/(mst12*mst22) - 6*power2(DeltaInv(mst22,mz2,mst12)
     )*power3(mz2) + (138*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/
     mst12 + (72*power2(mh2)*((3*mst12*mst22 + 2*mst24)*power2(mh2) - 2*mst22*
     power3(mh2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mh2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))/mst12 + 216*power2(mz2)
     *power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 720*mst24*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) + (144*power3(mst22)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)))/mst12 + (576*mt2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (576*mst14*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (576*mt2*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12
     - msb12*mst22) + (2304*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power4(
     mst12))/(mst12 - mst22) - (288*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12))/(msb12*mst12 - msb12*mst22) - (144*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 - (288*mst14*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2))/(msb12*mst12 - msb12*mst22)
     + (6*power2(DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 + 216*mst22*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + (144*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 - (144*mst22*power3(
     DeltaInv(mst22,mz2,mst12))*power5(mz2))/mst12))/(12.*sb) - (cb*mu*power2(
     log(mst12/Q2))*((-4*mh2*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12)))/(
     mst12 - mst22) + ((2*mh2*mst22 + mst12*mst22 + 2*mst24)*power2(mh2)*
     power2(DeltaInv(mst22,mh2,mst12)))/mst12 + mst22*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) + (2*mst24*power2(mz2)*power2(DeltaInv(mst22,
     mz2,mst12)))/mst12 + (4*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)))/(msb12*mst12 - msb12*mst22) + (2*mst22*power2(DeltaInv(mst22,mz2,
     mst12))*power3(mz2))/mst12 + (power2(mh2)*((3*mst12*mst22 + 2*mst24)*
     power2(mh2) - 2*mst22*power3(mh2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*
     mh2*(5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mh2,mst12)))/
     mst12 + 3*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) +
     10*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + (2*power3(mst22)*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)))/mst12 + (8*mt2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (8*
     mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) +
     (8*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(
     msb12*mst12 - msb12*mst22) - (4*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12))/(msb12*mst12 - msb12*mst22) - (2*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 - (4*mst14*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2))/(msb12*mst12 - msb12*mst22)
     + 3*mst22*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + (2*mst24*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 - (2*mst22*power3(DeltaInv(
     mst22,mz2,mst12))*power5(mz2))/mst12))/sb + (cb*mu*(-(1/msb12) - 1/mst12 -
     2/mst22 + (6*mt2)/((mst12 - mst22)*mst22) + (4*mh2*mt2*DeltaInv(mh2,mst12,
     mst12))/(mst12 - mst22) + mh2*DeltaInv(mst22,mh2,mst12) - (4*mh2*mt2*
     DeltaInv(mst22,mst22,mh2))/(mst12 - mst22) + 2*mw2*DeltaInv(mst22,mw2,
     msb12) - (4*mt2*mw2*DeltaInv(mst22,mw2,msb12))/(mst12 - mst22) + mz2*
     DeltaInv(mst22,mz2,mst12) + (4*mt2*mw2*DeltaInv(mw2,msb12,mst12))/(mst12 -
     mst22) - (6*mt2)/(-(mst12*mst22) + power2(mst12)) - (42*(3*mh2*mst22 +
     mst12*mst22 + 3*mst24)*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12)))/
     mst12 + (168*mh2*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)))/(mst12 -
     mst22) - 84*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - (252*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/msb12 + (168*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) + (504*
     mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(msb12*mst12 -
     msb12*mst22) - 42*mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - (
     126*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)))/mst12 - (168*
     mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) -
     (504*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12
     - msb12*mst22) - (168*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 +
     6*power3(mst12)))/(mst12 - mst22) + (1008*mt2*power2(DeltaInv(mst22,mst22,
     mh2))*power3(mst22))/(mst12 - mst22) - (252*mst22*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2))/msb12 + (504*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (504*mst12*mt2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2))/(msb12*mst12 - msb12*mst22) - (
     126*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/mst12 - 420*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + (840*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 -
     mst22) - 1176*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + (2352*
     mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) -
     (252*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 +
     (504*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(
     msb12*mst12 - msb12*mst22) - 210*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) - 588*mst24*power3(mz2)*power3(DeltaInv(mst22,
     mz2,mst12)) - (126*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)))/mst12 - (840*mt2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,
     msb12,mst12)))/(mst12 - mst22) - (2352*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) - (504*mt2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(msb12*mst12 - msb12*mst22)
     - (4032*mh2*mt2*power3(DeltaInv(mh2,mst12,mst12))*power4(mst12))/(mst12 -
     mst22) + (504*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12))/(msb12*mst12 - msb12*mst22) - (42*power2(mh2)*power3(DeltaInv(
     mst22,mh2,mst12))*(14*mh2*mst12*mst24 + 5*mst12*mst22*power2(mh2) + 3*
     mst24*power2(mh2) - 3*mst22*power3(mh2) + 3*mh2*power3(mst22) + 5*mst12*
     power3(mst22) - 3*power4(mst22)))/mst12 + (4032*mh2*mt2*power3(DeltaInv(
     mst22,mst22,mh2))*power4(mst22))/(mst12 - mst22) + (252*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/msb12 - (504*mt2*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/(msb12*mst12 -
     msb12*mst22) + (126*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(
     mst22))/mst12 - 420*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     (252*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/msb12 + (840*
     mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(mst12 - mst22) +
     (504*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(msb12*mst12
     - msb12*mst22) - (840*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2))/(mst12 - mst22) - (504*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2))/(msb12*mst12 - msb12*mst22) - 210*mst22*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mz2) - (126*mst24*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2))/mst12 + (252*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2))/msb12 + (504*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2))/(msb12*(-mst12 + mst22)) + (504*mst12*mt2*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mw2))/(msb12*mst12 - msb12*mst22) + (126*mst22*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))/mst12))/(6.*sb) - (cb*mu*
     power2(Pi)*(4*mh2*msb12*mst12*mst14*mt2*power2(DeltaInv(mh2,mst12,mst12))
     + msb12*mst12*(mst12 - mst22)*mst22*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12)) + 3*msb12*(mst12 - mst22)*mst24*power2(mh2)*power2(DeltaInv(mst22,
     mh2,mst12)) - 4*mh2*msb12*mst12*mst24*mt2*power2(DeltaInv(mst22,mst22,mh2)
     ) + 2*msb12*mst12*(mst12 - mst22)*mst22*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*mst12*(mst12 - mst22)*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 4*msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 12*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + msb12*mst12*(mst12 - mst22)*mst22*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) + 3*msb12*(mst12 - mst22)*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + 12*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 4*msb12*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 3*msb12*(mst12 - mst22)*mst22*power2(
     DeltaInv(mst22,mh2,mst12))*power3(mh2) - 24*msb12*mst12*mt2*power2(
     DeltaInv(mst22,mst22,mh2))*power3(mst22) + 6*mst12*(mst12 - mst22)*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 12*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 12*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*(mst12 - mst22)*mst22*
     power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 14*msb12*mst12*(mst12 -
     mst22)*mst24*power3(mh2)*power3(DeltaInv(mst22,mh2,mst12)) + 5*msb12*
     mst12*(mst12 - mst22)*power2(mh2)*power3(mst22)*power3(DeltaInv(mst22,mh2,
     mst12)) + 3*msb12*(mst12 - mst22)*power3(mh2)*power3(mst22)*power3(
     DeltaInv(mst22,mh2,mst12)) + 10*msb12*mst12*(mst12 - mst22)*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 20*msb12*mst12*mt2*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 28*msb12*
     mst12*(mst12 - mst22)*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 56*msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     6*mst12*(mst12 - mst22)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) - 12*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 5*msb12*mst12*(mst12 - mst22)*power2(mz2)*power3(
     mst22)*power3(DeltaInv(mst22,mz2,mst12)) + 14*msb12*mst12*(mst12 - mst22)*
     mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 3*msb12*(mst12 -
     mst22)*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 56*
     msb12*mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*
     msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mh2,mst12))*
     power4(mh2) + 3*msb12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mh2,
     mst12))*power4(mh2) + 24*msb12*mt2*power2(DeltaInv(mh2,mst12,mst12))*
     power4(mst12) + 20*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mst12) + 12*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 3*msb12*(mst12 - mst22)*power2(mh2)*power3(DeltaInv(mst22,
     mh2,mst12))*power4(mst22) - 96*mh2*msb12*mst12*mt2*power3(DeltaInv(mst22,
     mst22,mh2))*power4(mst22) - 6*mst12*(mst12 - mst22)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 12*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*(mst12 - mst22)*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 10*msb12*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 6*mst12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 20*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 12*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     12*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 20*
     msb12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*
     msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 3*msb12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 3*msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,
     mh2,mst12))*power5(mh2) + 96*mh2*msb12*mt2*power3(DeltaInv(mh2,mst12,
     mst12))*power5(mst12) - 12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mst12) - 6*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 12*mst12*mst22*mt2*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mw2) - 12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mw2) - 3*msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,
     mz2,mst12))*power5(mz2)))/(6.*msb12*mst12*(mst12 - mst22)*sb) - (cb*mu*sb*
     (-2*mC2*msb12*mst12*mt2*DeltaInv(msb12,mst12,mC2) - mA2*msb12*mst12*(mst12
     - mst22)*DeltaInv(mst22,mA2,mst12) - 2*mC2*msb12*mst12*(mst12 - mst22)*
     DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*mst12*mt2*DeltaInv(mst22,msb12,
     mC2) + 2*msb12*mst12*(mst12 - mst22)*mw2*DeltaInv(mst22,mw2,msb12) - 2*
     msb12*mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 -
     mst22)*mz2*DeltaInv(mst22,mz2,mst12) + 2*msb12*mst12*mt2*mw2*DeltaInv(mw2,
     msb12,mst12) + 252*mst12*mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) + 84*msb12*mt2*power2(mC2)*power2(mst12)*power2(DeltaInv(
     msb12,mst12,mC2)) + 42*msb12*mst12*(mst12 - mst22)*mst22*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12)) + 126*msb12*(mst12 - mst22)*mst24*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 84*msb12*mst12*(mst12 -
     mst22)*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 252*mst12*(
     mst12 - mst22)*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb12*mst12*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 252*
     mst12*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb12*
     mst12*(mst12 - mst22)*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - 252*mst12*(mst12 - mst22)*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 84*msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 252*mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - 42*msb12*mst12*(mst12 - mst22)*mst22*power2(mz2)*power2(DeltaInv(
     mst22,mz2,mst12)) - 126*msb12*(mst12 - mst22)*mst24*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12)) - 252*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 84*msb12*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 126*msb12*(mst12 - mst22)*mst22*
     power2(DeltaInv(mst22,mA2,mst12))*power3(mA2) + 252*mt2*power2(mst12)*
     power2(DeltaInv(msb12,mst12,mC2))*power3(mC2) + 252*mst12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 252*mst12*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 252*mst12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 252*mst12*mst22*mt2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 252*mt2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 126*msb12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 1176*msb12*mst12*
     mst14*mt2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 588*msb12*mst12*
     (mst12 - mst22)*mst24*power3(mA2)*power3(DeltaInv(mst22,mA2,mst12)) + 210*
     msb12*mst12*(mst12 - mst22)*power2(mA2)*power3(mst22)*power3(DeltaInv(
     mst22,mA2,mst12)) + 126*msb12*(mst12 - mst22)*power3(mA2)*power3(mst22)*
     power3(DeltaInv(mst22,mA2,mst12)) + 1176*msb12*mst12*(mst12 - mst22)*
     mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 1176*msb12*mst12*
     mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 420*msb12*mst12*
     (mst12 - mst22)*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 420*msb12*mst12*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) + 252*mst12*(mst12 - mst22)*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) - 252*mst12*mt2*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 420*msb12*mst12*(mst12 - mst22)*
     power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 420*msb12*
     mst12*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) -
     1176*msb12*mst12*(mst12 - mst22)*mst24*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 1176*msb12*mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) - 252*mst12*(mst12 - mst22)*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 252*mst12*mt2*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*msb12*mst12*(mst12 - mst22)*
     power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 588*msb12*
     mst12*(mst12 - mst22)*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     - 126*msb12*(mst12 - mst22)*power3(mst22)*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) - 1176*msb12*mst12*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 210*msb12*mst12*(mst12 - mst22)*mst22*power3(
     DeltaInv(mst22,mA2,mst12))*power4(mA2) + 126*msb12*(mst12 - mst22)*mst24*
     power3(DeltaInv(mst22,mA2,mst12))*power4(mA2) + 252*mst12*mst14*mt2*
     power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 420*msb12*mt2*power2(
     mst12)*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 420*msb12*mst12*(
     mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 252*
     mst12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     - 420*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     - 252*mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 420*
     msb12*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) +
     252*mt2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) - 420*
     msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     252*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*
     msb12*(mst12 - mst22)*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*
     power4(mst22) - 252*mst12*(mst12 - mst22)*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) + 252*mst12*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 252*mst12*(mst12 - mst22)*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 252*mst12*
     mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*
     msb12*(mst12 - mst22)*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 420*msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 252*mst12*(mst12 - mst22)*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 420*msb12*mst12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 252*mst12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 252*mst12*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 420*msb12*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 210*msb12*mst12*(mst12 -
     mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*msb12*(
     mst12 - mst22)*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mA2,mst12))*power5(mA2)
     - 252*mt2*power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) -
     252*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mC2) + 252*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) -
     252*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power5(mst12) + 252*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 252*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2)
     - 252*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 252*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))
     )/(6.*msb12*mst12*(mst12 - mst22)) - (mu*power3(cb)*(-4*mC2*msb12*mst12*
     mt2*DeltaInv(msb12,mst12,mC2) - mA2*msb12*mst12*(mst12 - mst22)*DeltaInv(
     mst22,mA2,mst12) - 2*mC2*msb12*mst12*(mst12 - mst22)*DeltaInv(mst22,msb12,
     mC2) + 4*mC2*msb12*mst12*mt2*DeltaInv(mst22,msb12,mC2) + 2*msb12*mst12*(
     mst12 - mst22)*mw2*DeltaInv(mst22,mw2,msb12) - 4*msb12*mst12*mt2*mw2*
     DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 - mst22)*mz2*DeltaInv(
     mst22,mz2,mst12) + 4*msb12*mst12*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 504*
     mst12*mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + 168*msb12*
     mt2*power2(mC2)*power2(mst12)*power2(DeltaInv(msb12,mst12,mC2)) + 42*
     msb12*mst12*(mst12 - mst22)*mst22*power2(mA2)*power2(DeltaInv(mst22,mA2,
     mst12)) + 126*msb12*(mst12 - mst22)*mst24*power2(mA2)*power2(DeltaInv(
     mst22,mA2,mst12)) + 84*msb12*mst12*(mst12 - mst22)*mst22*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 252*mst12*(mst12 - mst22)*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 168*msb12*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 504*mst12*mst24*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb12*mst12*(mst12 -
     mst22)*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 252*mst12*(
     mst12 - mst22)*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 168*
     msb12*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 504*
     mst12*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*
     mst12*(mst12 - mst22)*mst22*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))
     - 126*msb12*(mst12 - mst22)*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) - 504*mst12*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) - 168*msb12*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 126*msb12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mA2,mst12)
     )*power3(mA2) + 504*mt2*power2(mst12)*power2(DeltaInv(msb12,mst12,mC2))*
     power3(mC2) + 252*mst12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 504*mst12*mst22*mt2*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) - 252*mst12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 504*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12)
     )*power3(mw2) - 504*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 126*msb12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,mz2,
     mst12))*power3(mz2) + 2352*msb12*mst12*mst14*mt2*power3(mC2)*power3(
     DeltaInv(msb12,mst12,mC2)) + 588*msb12*mst12*(mst12 - mst22)*mst24*power3(
     mA2)*power3(DeltaInv(mst22,mA2,mst12)) + 210*msb12*mst12*(mst12 - mst22)*
     power2(mA2)*power3(mst22)*power3(DeltaInv(mst22,mA2,mst12)) + 126*msb12*(
     mst12 - mst22)*power3(mA2)*power3(mst22)*power3(DeltaInv(mst22,mA2,mst12))
     + 1176*msb12*mst12*(mst12 - mst22)*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 2352*msb12*mst12*mst24*mt2*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 420*msb12*mst12*(mst12 - mst22)*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 840*msb12*mst12*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 252*mst12*(
     mst12 - mst22)*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     - 504*mst12*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 420*msb12*mst12*(mst12 - mst22)*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) + 840*msb12*mst12*mt2*power2(mw2)*power3(mst22)
     *power3(DeltaInv(mst22,mw2,msb12)) - 1176*msb12*mst12*(mst12 - mst22)*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 2352*msb12*mst12*
     mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 252*mst12*(mst12
     - mst22)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     504*mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     - 210*msb12*mst12*(mst12 - mst22)*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) - 588*msb12*mst12*(mst12 - mst22)*mst24*power3(
     mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 126*msb12*(mst12 - mst22)*power3(
     mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 2352*msb12*mst12*
     mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*msb12*mst12*
     (mst12 - mst22)*mst22*power3(DeltaInv(mst22,mA2,mst12))*power4(mA2) + 126*
     msb12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,mA2,mst12))*power4(mA2)
     + 504*mst12*mst14*mt2*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 840*
     msb12*mt2*power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) +
     420*msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) + 252*mst12*(mst12 - mst22)*mst24*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 840*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 504*mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) + 840*msb12*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*
     power4(mst12) + 504*mt2*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2))*
     power4(mst12) - 840*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) - 504*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) - 126*msb12*(mst12 - mst22)*power2(mA2)*power3(DeltaInv(
     mst22,mA2,mst12))*power4(mst22) - 252*mst12*(mst12 - mst22)*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 504*mst12*mt2*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 252*mst12*(mst12 -
     mst22)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 504*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     126*msb12*(mst12 - mst22)*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 420*msb12*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 252*mst12*(mst12 - mst22)*mst24*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 840*msb12*mst12*mst22*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 504*mst12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 504*mst12*mst14*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 840*msb12*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 210*msb12*mst12*(mst12 -
     mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*msb12*(
     mst12 - mst22)*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mA2,mst12))*power5(mA2)
     - 504*mt2*power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) -
     252*mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mC2) + 504*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) -
     504*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power5(mst12) + 504*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 252*
     mst12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2)
     - 504*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 504*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 126*
     msb12*(mst12 - mst22)*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2))
     )/(6.*msb12*mst12*(mst12 - mst22)*sb) + (mu*power2(log(mst12/Q2))*power3(
     cb)*(-4*mst12*mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     msb12*(mst12 - mst22)*(2*mA2*mst22 + mst12*mst22 + 2*mst24)*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12)) + 2*msb12*mst12*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - 2*msb12*mst22*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(mst12)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(mst22)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + 4*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*mst12*mst22*power2(DeltaInv(mst22,
     mz2,mst12))*power3(mz2) - 2*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,
     mst12))*power3(mz2) + 4*mst12*mt2*power2(mC2)*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - msb12*(mst12 - mst22)*power2(mA2)*((3*
     mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*
     mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,mA2,mst12)) + 3*msb12*power2(mst12)*power2(mz2)*power3(
     mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 10*msb12*mst12*mst22*mst24*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 10*msb12*mst24*power2(
     mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 2*msb12*mst12*
     power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) + 8*msb12*
     mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 8*msb12*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 8*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 5*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 2*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 4*
     mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2*msb12*
     mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 2*msb12*mst22*
     mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 3*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 3*msb12*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 4*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 2*msb12*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 2*msb12*
     mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 2*msb12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(msb12*
     mst12*(mst12 - mst22)*sb) + (cb*mu*sb*power2(log(mst12/Q2))*(-2*mst12*
     mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(mst12 -
     mst22)*(2*mA2*mst22 + mst12*mst22 + 2*mst24)*power2(mA2)*power2(DeltaInv(
     mst22,mA2,mst12)) + 2*msb12*mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,
     mz2,mst12)) - 2*msb12*mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) + msb12*mst22*power2(mst12)*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) - msb12*mst12*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12)) + 2*mst12*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     + 2*msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 2*
     msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 2*
     mst12*mt2*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(
     mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2)) - msb12*(mst12 - mst22)*power2(mA2)*((3*mst12*mst22 + 2*mst24)*
     power2(mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*
     mA2*(5*mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) +
     3*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,
     mst12)) - 10*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(mst22,
     mz2,mst12)) + 10*msb12*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + 2*msb12*mst12*power3(mst22)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)) + 4*msb12*mst12*mst14*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 4*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 4*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) - 5*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mst22) - 2*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mst22) - 2*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 2*msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 2*msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 3*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mz2,mst12)
     )*power4(mz2) - 3*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 2*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mst12) + 2*msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power5(mst22) - 2*msb12*mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*
     power5(mz2) + 2*msb12*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*
     power5(mz2)))/(msb12*mst12*(mst12 - mst22)) + (mu*power2(Pi)*power3(cb)*(-
     4*mst12*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - msb12*(mst12 - mst22)*(3*mA2*mst22 + mst12*
     mst22 + 3*mst24)*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + 6*mst12*
     mst22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 4*msb12*mst12*
     mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 12*mst12*mst24*
     mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 2*msb12*mst22*power2(
     mC2)*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mst24*power2(mC2)
     *power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 2*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mst12*mst22*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 4*msb12*mst12*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 12*mst12*mst24*mt2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*msb12*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*mst24*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 2*msb12*mst12*
     power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb12*
     mst12*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - 3*msb12*mst22*
     mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(
     mst12)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(
     mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 12*mst12*mst14*mt2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 4*msb12*mt2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 12*mst12*mst22*mt2*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*mst22*power2(mst12)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) + 6*mst12*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 12*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*mst22*power2(mst12)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 12*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mz2) - 3*msb12*power2(mst22)*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mz2) + 4*mst12*mt2*power2(mC2)*(-((5*
     msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*
     mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 28*msb12*mst12*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 56*msb12*mst12*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 28*msb12*mst24*power2(mst12)*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 20*msb12*mst12*mt2*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 10*msb12*power2(mC2)*
     power2(mst12)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 12*mst12*
     mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 6*
     power2(mst12)*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     - 20*msb12*mst12*mt2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) + 10*msb12*power2(mst12)*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) - 28*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 56*msb12*mst12*mst24*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 28*msb12*mst24*power2(mst12)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 12*mst12*mt2*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 6*power2(mst12)*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*power2(
     mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12)) - 14*
     msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) +
     14*msb12*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     + 3*msb12*mst12*power3(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)
     ) + 56*msb12*mst12*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     + 6*mst12*mst22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 20*
     msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 12*
     mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 10*msb12*
     mst22*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 6*
     mst24*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 10*
     msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) +
     20*msb12*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) +
     12*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     msb12*(mst12 - mst22)*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12))*(14*
     mA2*mst12*mst24 + 5*mst12*mst22*power2(mA2) + 3*mst24*power2(mA2) - 3*
     mst22*power3(mA2) + 3*mA2*power3(mst22) + 5*mst12*power3(mst22) - 3*
     power4(mst22)) + 10*msb12*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) - 12*mst12*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) + 6*power2(mC2)*power2(mst12)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) + 6*mst12*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) - 10*msb12*mst12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 12*mst12*mt2*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*power2(mst12)*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*mst12*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 8*msb12*mst12*power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 3*msb12*power3(mz2)
     *power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 6*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 20*msb12*mst12*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 12*mst12*mst24*mt2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 10*msb12*mst22*power2(mst12)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst24*power2(mst12)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 10*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 12*mst12*mst14*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 20*msb12*mt2*power2(mst12)
     *power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 3*msb12*mst12*mst24*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 3*msb12*mst22*mst24*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 5*msb12*mst22*power2(
     mst12)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 5*msb12*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 12*mst12*
     mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 6*mst22*power2(
     mst12)*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 6*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 12*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 6*mst12*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mst22) + 6*mst12*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 3*msb12*power2(mz2)*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) + 12*mst12*mst22*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 6*mst22*power2(mst12)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 6*mst12*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 12*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*mst12*mst22*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 3*msb12*power2(mst22)*
     power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(6.*msb12*mst12*(mst12 -
     mst22)*sb) + (cb*mu*sb*power2(Pi)*(-2*mst12*(3*mC2*mst12 + msb12*mst12 +
     3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(mst12
     - mst22)*(3*mA2*mst22 + mst12*mst22 + 3*mst24)*power2(mA2)*power2(
     DeltaInv(mst22,mA2,mst12)) + 6*mst12*mst22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 2*msb12*mst12*mst22*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 6*mst12*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 2*msb12*mst22*power2(mC2)*power2(mst12)*
     power2(DeltaInv(mst22,msb12,mC2)) - 6*mst24*power2(mC2)*power2(mst12)*
     power2(DeltaInv(mst22,msb12,mC2)) + 2*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mst12*mst22*mst24*power2(mw2)
     *power2(DeltaInv(mst22,mw2,msb12)) - 2*msb12*mst12*mst22*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 6*mst12*mst24*mt2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 2*msb12*mst22*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 6*mst24*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 2*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb12*mst12*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - 3*msb12*mst22*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + msb12*mst22*power2(mst12)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - msb12*mst12*power2(mst22)*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) + 6*mst12*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 6*mst12*mst22*mt2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 6*mst22*power2(mst12)*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) + 6*mst12*power2(mst22)*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 6*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) + 6*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)
     )*power3(mw2) + 3*msb12*mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mz2) - 3*msb12*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mz2) + 2*mst12*mt2*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(
     mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(
     14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 28*
     msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     28*msb12*mst12*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     28*msb12*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     + 10*msb12*mst12*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) - 10*msb12*power2(mC2)*power2(mst12)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 6*mst12*mt2*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) - 6*power2(mst12)*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 10*msb12*mst12*mt2*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 10*msb12*power2(mst12)*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 28*msb12*mst12*
     mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 28*msb12*
     mst12*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 28*msb12*
     mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 6*
     mst12*mt2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 6*
     power2(mst12)*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     + 5*msb12*power2(mst12)*power2(mz2)*power3(mst22)*power3(DeltaInv(mst22,
     mz2,mst12)) - 14*msb12*mst12*mst22*mst24*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + 14*msb12*mst24*power2(mst12)*power3(mz2)*power3(
     DeltaInv(mst22,mz2,mst12)) + 3*msb12*mst12*power3(mst22)*power3(mz2)*
     power3(DeltaInv(mst22,mz2,mst12)) + 28*msb12*mst12*mst14*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*mst12*mst22*mst24*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 10*msb12*mst12*mst22*mt2*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 6*mst12*mst24*mt2*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mC2) - 10*msb12*mst22*power2(mst12)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 6*mst24*power2(mst12)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 10*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 10*msb12*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - msb12*(mst12 - mst22)*power2(
     mA2)*power3(DeltaInv(mst22,mA2,mst12))*(14*mA2*mst12*mst24 + 5*mst12*
     mst22*power2(mA2) + 3*mst24*power2(mA2) - 3*mst22*power3(mA2) + 3*mA2*
     power3(mst22) + 5*mst12*power3(mst22) - 3*power4(mst22)) + 10*msb12*mst12*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 6*mst12*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 6*power2(
     mC2)*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 6*
     mst12*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 10*
     msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     6*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) -
     6*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 6*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 8*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 3*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) - 6*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 10*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 6*mst12*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 10*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 10*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 6*mst12*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 10*msb12*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) + 3*msb12*mst12*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 3*msb12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) + 5*msb12*mst22*power2(mst12)*power3(DeltaInv(mst22,mz2,mst12)
     )*power4(mz2) - 5*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 6*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 6*mst22*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 6*mst12*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 6*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 6*mst12*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mst22) + 6*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 3*msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(
     mst22) + 6*mst12*mst22*mt2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) -
     6*mst22*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 6*
     mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 6*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*
     mst12*mst22*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) + 3*msb12*
     power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2)))/(6.*msb12*
     mst12*(mst12 - mst22)) + Fin3(mh2,mst12,mst12,Q2)*((-4*ca*mt2*mu*sa*
     power2(DeltaInv(mh2,mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)
     *power6(mst1)))/(mst12 - mst22) - (4*cb*mt2*mu*power2(DeltaInv(mh2,mst12,
     mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)))/((mst12 -
     mst22)*sb) + (4*cb*mt2*mu*power2(sa)*power2(DeltaInv(mh2,mst12,mst12))*(
     mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)))/((mst12 - mst22)*
     sb) - 4*ca*mu*sa*power2(snt)*power2(DeltaInv(mh2,mst12,mst12))*(mst14 + 4*
     mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)) - (4*cb*mu*power2(snt)*power2(
     DeltaInv(mh2,mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*
     power6(mst1)))/sb + (4*cb*mu*power2(sa)*power2(snt)*power2(DeltaInv(mh2,
     mst12,mst12))*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)))/sb +
     4*ca*mu*sa*power2(DeltaInv(mh2,mst12,mst12))*power4(snt)*(mst14 + 4*mh2*
     DeltaInv(mh2,mst12,mst12)*power6(mst1)) + (4*cb*mu*power2(DeltaInv(mh2,
     mst12,mst12))*power4(snt)*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(
     mst1)))/sb - (4*cb*mu*power2(sa)*power2(DeltaInv(mh2,mst12,mst12))*power4(
     snt)*(mst14 + 4*mh2*DeltaInv(mh2,mst12,mst12)*power6(mst1)))/sb) + Fin3(
     mH2,mst12,mst12,Q2)*((4*ca*mt2*mu*sa*power2(DeltaInv(mH2,mst12,mst12))*(
     mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*power6(mst1)))/(mst12 - mst22) - (
     4*cb*mt2*mu*power2(sa)*power2(DeltaInv(mH2,mst12,mst12))*(mst14 + 4*mH2*
     DeltaInv(mH2,mst12,mst12)*power6(mst1)))/((mst12 - mst22)*sb) + 4*ca*mu*
     sa*power2(snt)*power2(DeltaInv(mH2,mst12,mst12))*(mst14 + 4*mH2*DeltaInv(
     mH2,mst12,mst12)*power6(mst1)) - (4*cb*mu*power2(sa)*power2(snt)*power2(
     DeltaInv(mH2,mst12,mst12))*(mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*
     power6(mst1)))/sb - 4*ca*mu*sa*power2(DeltaInv(mH2,mst12,mst12))*power4(
     snt)*(mst14 + 4*mH2*DeltaInv(mH2,mst12,mst12)*power6(mst1)) + (4*cb*mu*
     power2(sa)*power2(DeltaInv(mH2,mst12,mst12))*power4(snt)*(mst14 + 4*mH2*
     DeltaInv(mH2,mst12,mst12)*power6(mst1)))/sb) + Fin3(mw2,msb12,mst12,Q2)*((
     4*cb*mt2*mu*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)*sb) + (4*cb*mt2*mu*sb*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))))/(msb12*(mst12 - mst22)) + (8*cb*mt2*mu*power2(mw2)*power2(snb)*
     power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*
     (-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(
     msb12*(mst12 - mst22)*sb) + (4*cb*mu*sb*power2(mw2)*power2(snt)*power2(
     DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 +
     (4*cb*mu*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,
     mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*sb) + (8*mt2*mu*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*power3(cb)*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)*sb) + (4*mt2*mu*
     power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*power3(cb)*(
     mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1))))/(msb12*(mst12 - mst22)*sb) + (4*mu*
     power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*power3(cb)*(
     mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1))))/(msb12*sb) + (8*cb*mt2*mu*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))
     /(msb12*(mst12 - mst22)*sb) + (4*cb*mt2*mu*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*
     (mst12 - mst22)*sb) + (4*cb*mt2*mu*sb*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*
     (mst12 - mst22)) + (4*cb*mu*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,
     msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/(msb12*sb) + (4*cb*
     mu*sb*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12)
     )*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 + (4*mt2*mu*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(cb)*(-mst12 + DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))))/(msb12*(mst12 - mst22)*sb) + (8*mt2*mu*power2(mw2)*power2(snb)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(cb)*(-mst12 + DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))))/(msb12*(mst12 - mst22)*sb) + (4*mu*power2(mw2)*power2(snb)*
     power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*power3(cb)*(-mst12 +
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb12*sb)) + Fin3(mw2,msb22,mst12,Q2)*((4*
     cb*mt2*mu*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(mst12
     - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb22*(mst12 - mst22)*sb) + (4*cb*mt2*mu*sb*
     power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (4*cb*mu*sb*power2(
     mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/msb22 + (8*mt2*mu*power2(mw2)*power2(snb)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(cb)*(mst12 - DeltaInv(mw2,msb22,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(
     mst1))))/(msb22*(mst12 - mst22)*sb) + (4*mu*power2(mw2)*power2(snb)*
     power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*power3(cb)*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/(msb22*sb) + (8*cb*mt2*mu*power2(mw2)*power2(
     snb)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,msb22,mst12)
     *(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))
     ))/(msb22*(mst12 - mst22)*sb) + (4*cb*mu*power2(mw2)*power2(snb)*power2(
     snt)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,msb22,mst12)
     *(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))
     ))/(msb22*sb) + (4*mt2*mu*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,
     msb22,mst12))*power3(cb)*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(msb22*
     (mst12 - mst22)*sb)) + Fin3(msb12,mst12,mC2,Q2)*((4*cb*mt2*mu*sb*power2(
     mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(
     msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (4*cb*mu*sb*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(
     mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb12 + (4*mt2*mu*power2(mC2)
     *power2(DeltaInv(msb12,mst12,mC2))*power3(cb)*(mst12 - DeltaInv(msb12,
     mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(msb12*(mst12 - mst22)*sb) + (8*mt2*mu*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*power3(cb)*(mst12 -
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)*sb) + (4*mu*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*
     power3(cb)*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(
     msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*sb) + (
     4*cb*mt2*mu*sb*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (4*cb*mu*sb*
     power2(mC2)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb12 + (8*mt2*mu*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2))*power3(cb)*(-mst12 + DeltaInv(msb12,mst12,mC2)*
     (-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/(msb12*(mst12 - mst22)*sb) + (4*mt2*mu*power2(mC2)*power2(
     snb)*power2(DeltaInv(msb12,mst12,mC2))*power3(cb)*(-mst12 + DeltaInv(
     msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)*sb) + (4*mu*power2(
     mC2)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*power3(cb)*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*sb)) + Fin3(msb22,mst12,mC2,Q2)
     *((4*mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*
     power3(cb)*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22)*sb) + (4*cb*mt2*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(
     msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(
     mst12 - mst22)) + (4*cb*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     msb22 + (8*mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2)
     )*power3(cb)*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22)*sb) + (4*mu*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(
     msb22,mst12,mC2))*power3(cb)*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(msb22*sb)) + (2*mu*power2(log(mw2/Q2))*power3(cb)*power3(mw2)*(
     mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,mw2,msb12)) + 2*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*
     (-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + (
     mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*mst24 -
     mst22*mw2) + 2*msb12*(mst24 + mst22*mw2) - power6(mst2))))/(msb12*(mst12 -
     mst22)*sb) + (2*cb*mu*sb*power2(log(mw2/Q2))*power3(mw2)*(-(mst22*(-mst12
     + mst22 + mt2)*power2(DeltaInv(mst22,mw2,msb12))) + mt2*power2(DeltaInv(
     mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + (mst12 - mst22
     - mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*mst24 - mst22*mw2) + 2*
     msb12*(mst24 + mst22*mw2) - power6(mst2))))/(msb12*(mst12 - mst22)) + (2*
     cb*mu*sb*power2(snb)*power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*power2(DeltaInv(
     msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) - msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22) + (2*mu*power2(snb)*power2(snt)*power2(log(mC2/Q2))*power3(
     cb)*power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*
     mst12*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2)) + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) + msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*
     msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb22,mC2)) - msb12*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) - msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(msb12*msb22*sb) + (2*mt2*mu*power2(snb)*
     power2(log(mC2/Q2))*power3(cb)*power3(mC2)*(-(msb22*mst12*power2(DeltaInv(
     msb12,mst12,mC2))) + msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) +
     msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - msb22*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2))
     + msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + msb22*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - msb12*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) + msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*
     msb22*(mst12 - mst22)*sb) - (cb*mC2*mu*sb*log(mC2/Q2)*power2(snb)*power2(
     snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + mC2*
     msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*msb12*
     (-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*msb12*
     msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*DeltaInv(
     mst22,msb12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,mC2) + 5*
     mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2) - msb22*mst12*DeltaInv(
     mst22,msb12,mC2)*power2(mC2) + msb12*mst12*DeltaInv(mst22,msb22,mC2)*
     power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*
     mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*
     msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*
     mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*
     mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*
     mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*
     mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14
     - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(6.*msb12*msb22*mst12*mst22) + (mC2*mt2*mu*log(mC2/Q2)*
     power2(snb)*power3(cb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 -
     msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,
     mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,
     mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*
     msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*
     DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*
     mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*
     mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) -
     6*mC2*msb12*mst12*mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*
     msb12*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) +
     6*msb22*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*
     msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 144*msb22*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*
     power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14
     - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(6.*msb12*msb22*mst12*(mst12 - mst22)*mst22*sb) - (mC2*mu*
     log(mC2/Q2)*power2(snb)*power2(snt)*power3(cb)*(-(msb12*mst12) + msb22*
     mst12 + msb12*mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*
     mst22*DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*
     mst22*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,
     msb12,mC2) - 5*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*
     msb12*msb22*mst12*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*
     DeltaInv(mst22,msb22,mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(
     mC2) + msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*
     mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2)
     )*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*
     mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) - 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 18*mC2*msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) - 84*msb22*mst12*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*mst22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*
     mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*
     msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     6.*msb12*msb22*mst12*mst22*sb) + (mC2*mu*log(mC2/Q2)*power2(snb)*power3(
     cb)*(-(msb12*mst12*mst22) + msb22*mst12*mst22 - 2*msb12*mst12*mt2 + 2*
     msb22*mst12*mt2 + 2*msb12*mst22*mt2 - 2*msb22*mst22*mt2 + 2*mC2*msb22*(mC2
     - msb12 + 5*mst12)*mst22*mt2*DeltaInv(msb12,mst12,mC2) - 2*mC2*msb12*(mC2
     - msb22 + 5*mst12)*mst22*mt2*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*
     mst12*mst22*DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*msb22*mst12*mt2*
     DeltaInv(mst22,msb12,mC2) - 10*mC2*msb22*mst12*mst22*mt2*DeltaInv(mst22,
     msb12,mC2) - mC2*msb12*msb22*mst12*mst22*DeltaInv(mst22,msb22,mC2) - 2*
     mC2*msb12*msb22*mst12*mt2*DeltaInv(mst22,msb22,mC2) + 10*mC2*msb12*mst12*
     mst22*mt2*DeltaInv(mst22,msb22,mC2) - msb22*mst12*mst22*DeltaInv(mst22,
     msb12,mC2)*power2(mC2) - 2*msb22*mst12*mt2*DeltaInv(mst22,msb12,mC2)*
     power2(mC2) + msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2)*power2(mC2) + 2*
     msb12*mst12*mt2*DeltaInv(mst22,msb22,mC2)*power2(mC2) + msb12*power2(
     mst12) - msb22*power2(mst12) - mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2)*
     power2(mst12) + 5*mC2*msb22*mst22*DeltaInv(mst22,msb12,mC2)*power2(mst12)
     + mC2*msb12*msb22*DeltaInv(mst22,msb22,mC2)*power2(mst12) - 5*mC2*msb12*
     mst22*DeltaInv(mst22,msb22,mC2)*power2(mst12) + msb22*DeltaInv(mst22,
     msb12,mC2)*power2(mC2)*power2(mst12) - msb12*DeltaInv(mst22,msb22,mC2)*
     power2(mC2)*power2(mst12) - 5*mC2*msb22*mst12*DeltaInv(mst22,msb12,mC2)*
     power2(mst22) + 5*mC2*msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mst22)
     - 12*mC2*msb22*mst12*mst22*mt2*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) +
     mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 12*mC2*msb12*
     mst12*mst22*mt2*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 12*mC2*msb22*mst12*mst22*mst24*
     mt2*power2(DeltaInv(mst22,msb12,mC2)) - 12*msb12*msb22*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb22*mst22*mst24*
     power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 6*msb12*msb22*mst22*
     power2(mC2)*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*
     mst12*mst24*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 36*mC2*
     msb12*msb22*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*
     msb12*msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,
     mC2)) - 168*msb22*mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) + 18*mC2*msb12*msb22*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) + 84*msb22*power2(mC2)*power2(mst12)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 12*mC2*msb12*mst12*mst22*mst24*
     mt2*power2(DeltaInv(mst22,msb22,mC2)) + 12*msb12*msb22*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 6*mC2*msb12*mst22*mst24*
     power2(mst12)*power2(DeltaInv(mst22,msb22,mC2)) - 6*msb12*msb22*mst22*
     power2(mC2)*power2(mst12)*power2(DeltaInv(mst22,msb22,mC2)) - 6*mC2*msb12*
     mst12*mst24*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 36*mC2*
     msb12*msb22*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*
     msb12*msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,
     mC2)) + 168*msb12*mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(
     mst22,msb22,mC2)) - 18*mC2*msb12*msb22*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) - 84*msb12*power2(mC2)*power2(mst12)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 12*msb22*mst12*mst22*mt2*
     power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb22*mst22*power2(
     mst12)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 6*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 12*msb12*
     mst12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 6*msb12*
     mst22*power2(mst12)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 6*
     msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) -
     18*mC2*msb12*msb22*mst12*power2(DeltaInv(mst22,msb12,mC2))*power3(mst22) -
     84*msb22*mst12*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*power3(mst22)
     + 18*mC2*msb12*msb22*mst12*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22)
     + 84*msb12*mst12*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mst22) - 288*msb12*msb22*mst12*mst22*mst24*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*mst22*mst24*power2(mC2)*
     power2(mst12)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*msb22*mst12*
     mst24*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 288*
     msb22*mst12*mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     + 144*msb22*mst22*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 144*msb22*mst12*mst24*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 288*msb12*msb22*mst12*mt2*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*power2(
     mst12)*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) + 288*msb12*msb22*mst12*mst22*mst24*mt2*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - 144*msb12*msb22*mst22*mst24*power2(mC2)*power2(mst12)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*mst24*power2(
     mC2)*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 288*msb12*mst12*
     mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*
     mst22*mst24*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     144*msb12*mst12*mst24*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 288*msb12*msb22*mst12*mt2*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*power2(mst12)*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) -
     72*msb22*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) + 72*msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 144*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 72*msb12*power2(mst12)*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb12*mst12*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 144*msb22*mst12*
     mst22*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 -
     2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 144*
     msb12*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + 144*msb22*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power6(mst2) - 72*msb22*mst22*power2(mC2)*power2(mst12)*
     power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) + 72*msb22*mst12*power2(
     mC2)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 144*
     msb12*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*
     power6(mst2) + 72*msb12*mst22*power2(mC2)*power2(mst12)*power3(DeltaInv(
     mst22,msb22,mC2))*power6(mst2) - 72*msb12*mst12*power2(mC2)*power2(mst22)*
     power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(6.*msb12*msb22*mst12*
     mst22*(-mst12 + mst22)*sb) + (cb*mC2*mu*sb*log(mC2/Q2)*power2(snb)*(-(
     msb12*mst12*mst22) + msb22*mst12*mst22 - msb12*mst12*mt2 + msb22*mst12*mt2
     + msb12*mst22*mt2 - msb22*mst22*mt2 + mC2*msb22*(mC2 - msb12 + 5*mst12)*
     mst22*mt2*DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*
     mst22*mt2*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*mst12*mst22*
     DeltaInv(mst22,msb12,mC2) + mC2*msb12*msb22*mst12*mt2*DeltaInv(mst22,
     msb12,mC2) - 5*mC2*msb22*mst12*mst22*mt2*DeltaInv(mst22,msb12,mC2) - mC2*
     msb12*msb22*mst12*mst22*DeltaInv(mst22,msb22,mC2) - mC2*msb12*msb22*mst12*
     mt2*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*mt2*DeltaInv(
     mst22,msb22,mC2) - msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2)*power2(mC2)
     - msb22*mst12*mt2*DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*
     mst22*DeltaInv(mst22,msb22,mC2)*power2(mC2) + msb12*mst12*mt2*DeltaInv(
     mst22,msb22,mC2)*power2(mC2) + msb12*power2(mst12) - msb22*power2(mst12) -
     mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2)*power2(mst12) + 5*mC2*msb22*
     mst22*DeltaInv(mst22,msb12,mC2)*power2(mst12) + mC2*msb12*msb22*DeltaInv(
     mst22,msb22,mC2)*power2(mst12) - 5*mC2*msb12*mst22*DeltaInv(mst22,msb22,
     mC2)*power2(mst12) + msb22*DeltaInv(mst22,msb12,mC2)*power2(mC2)*power2(
     mst12) - msb12*DeltaInv(mst22,msb22,mC2)*power2(mC2)*power2(mst12) - 5*
     mC2*msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mst22) + 5*mC2*msb12*
     mst12*DeltaInv(mst22,msb22,mC2)*power2(mst22) - 6*mC2*msb22*mst12*mst22*
     mt2*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*mt2*(-3*msb22*
     mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*mt2*power2(DeltaInv(
     mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb22*mst22*mst24*power2(mst12)*power2(
     DeltaInv(mst22,msb12,mC2)) + 6*msb12*msb22*mst22*power2(mC2)*power2(mst12)
     *power2(DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*mst12*mst24*power2(mst22)
     *power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*mt2*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*power2(mC2)
     *power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*mt2*
     power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) + 18*mC2*
     msb12*msb22*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))
     + 84*msb22*power2(mC2)*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,
     msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*mt2*power2(DeltaInv(mst22,
     msb22,mC2)) + 6*msb12*msb22*mst12*mst22*mt2*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + 6*mC2*msb12*mst22*mst24*power2(mst12)*power2(DeltaInv(
     mst22,msb22,mC2)) - 6*msb12*msb22*mst22*power2(mC2)*power2(mst12)*power2(
     DeltaInv(mst22,msb22,mC2)) - 6*mC2*msb12*mst12*mst24*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*mt2*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*power2(mC2)*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*mt2*
     power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 18*mC2*
     msb12*msb22*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))
     - 84*msb12*power2(mC2)*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,
     msb22,mC2)) + 6*msb22*mst12*mst22*mt2*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) - 6*msb22*mst22*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)
     )*power3(mC2) + 6*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 6*msb12*mst12*mst22*mt2*power2(DeltaInv(mst22,msb22,
     mC2))*power3(mC2) + 6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,
     msb22,mC2))*power3(mC2) - 6*msb12*mst12*power2(mst22)*power2(DeltaInv(
     mst22,msb22,mC2))*power3(mC2) - 18*mC2*msb12*msb22*mst12*power2(DeltaInv(
     mst22,msb12,mC2))*power3(mst22) - 84*msb22*mst12*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mst22) + 18*mC2*msb12*msb22*mst12*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mst22) + 84*msb12*mst12*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22) - 144*msb12*msb22*
     mst12*mst22*mst24*mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 144*
     msb12*msb22*mst22*mst24*power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,
     msb12,mC2)) - 144*msb12*msb22*mst12*mst24*power2(mC2)*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*mst22*mst24*mt2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb22*mst22*mst24*
     power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst12*mst24*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     144*msb12*msb22*mst12*mt2*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) + 144*msb12*msb22*power2(mst12)*power2(mst22)*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*msb22*mst12*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*mst12*
     mst22*mst24*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*
     msb22*mst22*mst24*power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*msb22*mst12*mst24*power2(mC2)*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*mst22*mst24*power2(mst12)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst24*
     power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*
     msb22*mst12*mt2*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     ) - 144*msb12*msb22*power2(mst12)*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*mt2*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb22*power2(
     mst12)*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 72*
     msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) -
     72*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mC2) + 72*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mC2) - 72*msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*mt2*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*mt2*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*mst22*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*msb22*
     mst22*power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) + 72*msb22*mst12*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,
     msb12,mC2))*power6(mst2) - 72*msb12*mst12*mst22*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power6(mst2) + 72*msb12*mst22*power2(mC2)*
     power2(mst12)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2) - 72*msb12*
     mst12*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power6(
     mst2)))/(6.*msb12*msb22*mst12*mst22*(-mst12 + mst22)) + (2*cb*mu*power2(
     snt)*power2(log(mw2/Q2))*power3(mw2)*(mst22*power2(DeltaInv(mst22,mw2,
     msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(
     mst1))) - power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*sb) + (2*cb*mt2*mu*
     power2(log(mw2/Q2))*power3(mw2)*(-(mst22*power2(DeltaInv(mst22,mw2,msb12))
     ) + power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)))
     + power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*
     (mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb) + (2*cb*
     mu*sb*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(-(mst22*power2(
     DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))) + power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 -
     (2*mt2*mu*power2(log(mw2/Q2))*power3(cb)*power3(mw2)*(-(mst22*power2(
     DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))) + power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     (mst12 - mst22)*sb) + (2*mu*power2(snt)*power2(log(mw2/Q2))*power3(cb)*
     power3(mw2)*(-(mst22*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(
     mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))) + power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*sb) + (2*cb*mu*power2(log(mw2/Q2))*
     power3(mw2)*(mst22*(-mst12 + mst22 + 2*mt2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 2*mt2*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,
     msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1))) + (mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))
     *(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))
     ))/(msb12*(mst12 - mst22)*sb) + (2*mu*power2(snb)*power2(log(mw2/Q2))*
     power3(cb)*power3(mw2)*(msb22*mst22*(-mst12 + mst22 + 2*mt2)*power2(
     DeltaInv(mst22,mw2,msb12)) + msb12*mst22*(mst12 - mst22 - 2*mt2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 2*mt2*(-(msb22*mst12*power2(DeltaInv(mw2,
     msb12,mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) + msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))) +
     msb12*(mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(2*
     mst24 - mst22*mw2) + 2*msb22*(mst24 + mst22*mw2) - power6(mst2)) + msb22*(
     mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22*(
     mst12 - mst22)*sb) + (2*cb*mu*sb*power2(snb)*power2(log(mw2/Q2))*power3(
     mw2)*(msb22*mst22*(-mst12 + mst22 + mt2)*power2(DeltaInv(mst22,mw2,msb12))
     - msb12*mst22*(-mst12 + mst22 + mt2)*power2(DeltaInv(mst22,mw2,msb22)) +
     mt2*(-(msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))) + msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1)) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1)))) + msb12*(mst12 - mst22 - mt2)*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(2*mst24 - mst22*mw2) + 2*msb22*(mst24 +
     mst22*mw2) - power6(mst2)) + msb22*(mst12 - mst22 - mt2)*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2))))/(msb12*msb22*(mst12 - mst22)) + (cb*mt2*mu*mw2*log(
     mw2/Q2)*(-mst12 + mst22 + mst12*mw2*(-msb12 + 5*mst22 + mw2)*DeltaInv(
     mst22,mw2,msb12) + mst22*(msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,
     mst12) + 6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22
     + mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 6*mst12*mst22*mw2*(-mst14 + (
     14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,
     mst12)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) -
     72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(6.*
     msb12*mst12*(mst12 - mst22)*mst22*sb) + (cb*mu*mw2*sb*log(mw2/Q2)*power2(
     snt)*(-mst12 + mst22 + mst12*mw2*(-msb12 + 5*mst22 + mw2)*DeltaInv(mst22,
     mw2,msb12) + mst22*(msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,mst12) +
     6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb12)) - 6*mst12*mst22*mw2*(-mst14 + (14*mst12
     - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) +
     72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) - 72*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*
     mst12*mst22) + (mu*mw2*log(mw2/Q2)*power2(snt)*power3(cb)*(-mst12 + mst22
     + mst12*mw2*(-msb12 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb12) + mst22*(
     msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,mst12) + 6*mst12*mst22*mw2*(
     -mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(
     mst22,mw2,msb12)) - 6*mst12*mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 +
     msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) + 72*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2)
     - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*power2(mw2)
     *power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(
     mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*mst12*mst22*sb) + (cb*mu*
     mw2*log(mw2/Q2)*power2(snt)*(mst12 - mst22 + mst12*(msb12 - 5*mst22 - mw2)
     *mw2*DeltaInv(mst22,mw2,msb12) + mst22*mw2*(-msb12 + 5*mst12 + mw2)*
     DeltaInv(mw2,msb12,mst12) - 6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*mst12*
     mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb12,mst12)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*
     mw2) + power6(mst1)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(6.*msb12*mst12*mst22*sb) + (mt2*mu*mw2*log(mw2/Q2)*
     power3(cb)*(mst12 - mst22 + mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(
     mst22,mw2,msb12) + mst22*mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(mw2,msb12,
     mst12) - 6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22
     + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*mst12*mst22*mw2*(-mst14 + (
     14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,
     mst12)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) +
     72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(6.*
     msb12*mst12*(mst12 - mst22)*mst22*sb) + (cb*mu*mw2*log(mw2/Q2)*(-((mst12 -
     mst22)*(mst12 - 2*mt2)) + mst12*(mst12 - mst22 - 2*mt2)*mw2*(-msb12 + 5*
     mst22 + mw2)*DeltaInv(mst22,mw2,msb12) + 2*mst22*mt2*mw2*(-msb12 + 5*mst12
     + mw2)*DeltaInv(mw2,msb12,mst12) + 6*mst12*mst22*(mst12 - mst22 - 2*mt2)*
     mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + 12*mst12*mst22*mt2*mw2*(-mst14 + (14*mst12 -
     mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) - 144*
     mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) - 72*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (mu*mw2*log(
     mw2/Q2)*power3(cb)*((mst12 - mst22)*(mst12 - 2*mt2) + mst12*(mst12 - mst22
     - 2*mt2)*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + 2*mst22*
     mt2*(msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,mst12) - 6*mst12*mst22*
     (mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*
     mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 12*mst12*mst22*mt2*mw2*(
     -mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(
     mw2,msb12,mst12)) + 144*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1)) + 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (
     cb*mu*mw2*sb*log(mw2/Q2)*((mst12 - mst22)*(mst12 - mt2) + mst12*(mst12 -
     mst22 - mt2)*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) +
     mst22*mt2*(msb12 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb12,mst12) - 6*mst12*
     mst22*(mst12 - mst22 - mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*
     mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) - 6*mst12*mst22*mt2*mw2*(-
     mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,
     msb12,mst12)) + 72*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1)) + 72*mst12*mst22*(mst12 - mst22 - mt2)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22) - (2*
     cb*mt2*mu*power2(snb)*power2(log(mw2/Q2))*power3(mw2)*(-(msb22*mst22*
     power2(DeltaInv(mst22,mw2,msb12))) + msb12*mst22*power2(DeltaInv(mst22,
     mw2,msb22)) + msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) - msb12*mst12*
     power2(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,
     mst12)) + 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,mst12)) - msb22*
     mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst14*
     power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,
     mst12)) + msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) -
     msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + msb12*power3(
     DeltaInv(mw2,msb22,mst12))*power6(mst1) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22*(
     mst12 - mst22)*sb) + (2*cb*mu*power2(snb)*power2(snt)*power2(log(mw2/Q2))*
     power3(mw2)*(-(msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))) + msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22)) + msb22*mst12*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) + 2*msb12*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mw2*
     power3(DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*mw2*power3(DeltaInv(mw2,
     msb12,mst12)) - msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*
     mst12*mw2*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) + msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     msb12*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(
     msb12*msb22*sb) + (2*mt2*mu*power2(snb)*power2(log(mw2/Q2))*power3(cb)*
     power3(mw2)*(-(msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))) + msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22)) + msb22*mst12*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) + 2*msb12*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mw2*
     power3(DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*mw2*power3(DeltaInv(mw2,
     msb12,mst12)) - msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*
     mst12*mw2*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) + msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     msb12*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(
     msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mu*sb*power2(snb)*power2(snt)*
     power2(log(mw2/Q2))*power3(mw2)*(msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12)) - msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)) - msb22*mst12*
     power2(DeltaInv(mw2,msb12,mst12)) + msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12)) - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*
     msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb22*mst14*mw2*
     power3(DeltaInv(mw2,msb12,mst12)) + msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb22,mst12)) + 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb22,mst12))
     + 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,mst12)) - msb12*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + msb22*power3(DeltaInv(mw2,
     msb12,mst12))*power6(mst1) - msb12*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) - msb22*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) + msb12*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*msb22) + (2*mu*power2(snb)*power2(snt)
     *power2(log(mw2/Q2))*power3(cb)*power3(mw2)*(msb22*mst22*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)) - msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12)) + msb12*mst12*power2(DeltaInv(mw2,
     msb22,mst12)) - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) - 2*
     msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb22*mst14*
     mw2*power3(DeltaInv(mw2,msb12,mst12)) + msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb22,mst12)) + 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb22,mst12))
     + 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,mst12)) - msb12*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + msb22*power3(DeltaInv(mw2,
     msb12,mst12))*power6(mst1) - msb12*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) - msb22*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) + msb12*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*msb22*sb) + (2*cb*mu*power2(snb)*
     power2(log(mw2/Q2))*power3(mw2)*(msb22*mst22*(mst12 - mst22 - 2*mt2)*
     power2(DeltaInv(mst22,mw2,msb12)) + msb12*mst22*(-mst12 + mst22 + 2*mt2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 2*mt2*(msb22*mst12*power2(DeltaInv(
     mw2,msb12,mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14
     + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) - msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))) +
     msb22*(mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(2*
     mst24 - mst22*mw2) + 2*msb12*(mst24 + mst22*mw2) - power6(mst2)) + msb12*(
     mst12 - mst22 - 2*mt2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22*(
     mst12 - mst22)*sb) + (cb*mt2*mu*mw2*log(mw2/Q2)*power2(snb)*(-(msb12*
     mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + msb22*mst12*(msb12 - 5*
     mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*mw2*(-msb22 + 5*
     mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*msb22*mst22*mw2*DeltaInv(
     mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) +
     msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mst22*mw2*
     DeltaInv(mw2,msb22,mst12) + msb22*mst22*DeltaInv(mw2,msb12,mst12)*power2(
     mw2) - msb12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*
     mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*
     msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*
     msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*
     msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) - 6*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     84*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - 6*msb22*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*
     msb12*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*
     msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     ) + 144*msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(
     mst1) + 72*msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     *(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))
     - 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.
     *msb12*msb22*mst12*(mst12 - mst22)*mst22*sb) - (cb*mu*mw2*log(mw2/Q2)*
     power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 -
     msb22*mst22 + msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,
     msb22) - msb12*msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*
     mst22*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,
     msb22,mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*
     msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 +
     mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*
     msb22*mst12*mst22*sb) + (cb*mu*mw2*sb*log(mw2/Q2)*power2(snb)*power2(snt)*
     (-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + msb22*mst12*(
     msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*mw2*(-
     msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*msb22*mst22*mw2*
     DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*DeltaInv(mw2,msb12,
     mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 +
     mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*mw2*(-mst24
     + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,
     msb22)) - 6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))
     + 18*msb12*msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))
     + 6*msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     + 84*msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12))
     - 18*msb12*msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))
     - 6*msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - 84*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) - 6*msb22*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) +
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 144*msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mst22*power2(mw2)
     *power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst22*power2(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*
     mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*
     msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(
     mst1) + 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     *power6(mst1) + 72*msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) - 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) +
     power6(mst2))))/(6.*msb12*msb22*mst12*mst22) - (mt2*mu*mw2*log(mw2/Q2)*
     power2(snb)*power3(cb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 -
     msb22*mst22 + msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,
     msb22) - msb12*msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*
     mst22*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,
     msb22,mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*
     msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 +
     mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*
     msb22*mst12*(mst12 - mst22)*mst22*sb) + (mu*mw2*log(mw2/Q2)*power2(snb)*
     power2(snt)*power3(cb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 -
     msb22*mst22 + msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,
     msb22) - msb12*msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*
     mst22*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,
     msb22,mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*
     mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*
     msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 +
     mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*
     msb22*mst12*mst22*sb) + (mu*mw2*log(mw2/Q2)*power2(snb)*power3(cb)*(msb12*
     mst12*mst22 - msb22*mst12*mst22 + 2*msb12*mst12*mt2 - 2*msb22*mst12*mt2 -
     2*msb12*mst22*mt2 + 2*msb22*mst22*mt2 + msb22*mst12*(mst12 - mst22 - 2*
     mt2)*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*(
     mst12 - mst22 - 2*mt2)*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,
     msb22) + 2*msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 10*msb22*
     mst12*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) - 2*msb12*msb22*mst22*mt2*
     mw2*DeltaInv(mw2,msb22,mst12) + 10*msb12*mst12*mst22*mt2*mw2*DeltaInv(mw2,
     msb22,mst12) - msb12*power2(mst12) + msb22*power2(mst12) - 2*msb22*mst22*
     mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 2*msb12*mst22*mt2*DeltaInv(
     mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*(mst12 - mst22 - 2*mt2)
     *mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*
     mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) + 12*msb22*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) - 36*msb12*msb22*mst22*mt2*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12)) - 12*msb12*msb22*mst12*mst22*mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 168*msb22*mst22*mt2*power2(mst12)
     *power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 12*msb12*mst12*mst14*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 36*msb12*msb22*mst22*
     mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) + 12*msb12*msb22*
     mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 168*msb12*
     mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     12*msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) -
     12*msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) -
     288*msb12*msb22*mst12*mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 288*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 288*msb12*msb22*mst22*mt2*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*msb12*msb22*mst12*
     mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 288*msb12*
     mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 288*
     msb12*msb22*mst22*mt2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + 144*msb22*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 144*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 144*msb22*mst12*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - 144*msb12*mst12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) - 72*msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*msb22*mst12*mst22*(-mst12 +
     mst22)*sb) + (cb*mu*mw2*log(mw2/Q2)*power2(snb)*(-(msb12*mst12*mst22) +
     msb22*mst12*mst22 - 2*msb12*mst12*mt2 + 2*msb22*mst12*mt2 + 2*msb12*mst22*
     mt2 - 2*msb22*mst22*mt2 + msb22*mst12*(mst12 - mst22 - 2*mt2)*mw2*(-msb12
     + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 - mst22 -
     2*mt2)*(msb22 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb22) - 2*msb12*
     msb22*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12) + 10*msb22*mst12*mst22*mt2*
     mw2*DeltaInv(mw2,msb12,mst12) + 2*msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,
     msb22,mst12) - 10*msb12*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) +
     msb12*power2(mst12) - msb22*power2(mst12) + 2*msb22*mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*power2(mw2) - 2*msb12*mst22*mt2*DeltaInv(mw2,msb22,mst12)
     *power2(mw2) + 6*msb22*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (
     14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,
     msb12)) - 6*msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*mw2*(-mst24 + (14*
     mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)
     ) - 12*msb22*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) +
     36*msb12*msb22*mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,
     mst12)) + 12*msb12*msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 168*msb22*mst22*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 12*msb12*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) - 36*msb12*msb22*mst22*mt2*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12)) - 12*msb12*msb22*mst12*mst22*mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 168*msb12*mst22*mt2*power2(mst12)
     *power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 12*msb22*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 12*msb12*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 288*msb12*msb22*mst12*
     mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*msb22*
     mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 288*
     msb12*msb22*mst22*mt2*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 288*msb12*msb22*mst12*mst14*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 288*msb12*mst12*mst14*mst22*mt2*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 288*msb12*msb22*mst22*mt2*power2(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb22*mst22*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*
     msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*msb22*mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power6(mst1) + 144*msb12*mst12*mst22*mt2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power6(mst1) - 72*msb22*mst12*mst22*(mst12 -
     mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) + 72*
     msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*
     mw2) + power6(mst2))))/(6.*msb12*msb22*mst12*mst22*(-mst12 + mst22)*sb) +
     (cb*mu*mw2*sb*log(mw2/Q2)*power2(snb)*(msb12*mst12*mst22 - msb22*mst12*
     mst22 + msb12*mst12*mt2 - msb22*mst12*mt2 - msb12*mst22*mt2 + msb22*mst22*
     mt2 + msb22*mst12*(mst12 - mst22 - mt2)*(msb12 - 5*mst22 - mw2)*mw2*
     DeltaInv(mst22,mw2,msb12) + msb12*mst12*(mst12 - mst22 - mt2)*mw2*(-msb22
     + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mst22*mt2*mw2*
     DeltaInv(mw2,msb12,mst12) - 5*msb22*mst12*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12) - msb12*msb22*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) + 5*
     msb12*mst12*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12) - msb12*power2(mst12)
     + msb22*power2(mst12) - msb22*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(
     mw2) + msb12*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*
     mst12*mst22*(mst12 - mst22 - mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 +
     msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*
     mst22*(mst12 - mst22 - mt2)*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*
     mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst12*mst14*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 18*msb12*msb22*mst22*
     mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*
     mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb22*
     mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*
     msb12*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*
     msb12*msb22*mst22*mt2*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))
     + 6*msb12*msb22*mst12*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) + 84*msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) + 6*msb22*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) - 6*msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 144*msb12*msb22*mst12*mst14*mst22*mt2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb22*mst12*mst14*mst22*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst22*mt2*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst12*mst14*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     + 144*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + 144*msb12*msb22*mst22*mt2*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + 72*msb22*mst22*mt2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb12*mst22*mt2*power2(mst12)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 72*msb22*mst12*mst22*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - 72*msb12*
     mst12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1)
     + 72*msb22*mst12*mst22*(mst12 - mst22 - mt2)*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2)) - 72*msb12*mst12*mst22*(mst12 - mst22 - mt2)*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*
     msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.*msb12*msb22*mst12*mst22*(-
     mst12 + mst22)) + (cb*mu*sb*power2(DeltaInv(mst22,mz2,mst12))*power2(log(
     mz2/Q2))*power3(mz2)*(mst22 - DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 +
     mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/mst12 + (mu*
     power2(DeltaInv(mst22,mz2,mst12))*power2(log(mz2/Q2))*power3(cb)*power3(
     mz2)*(mst22 - DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*
     mst12*(mst24 + mst22*mz2) + power6(mst2))))/(mst12*sb) + (cb*mu*power2(
     DeltaInv(mst22,mz2,mst12))*power2(log(mz2/Q2))*power3(mz2)*(-mst22 +
     DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 +
     mst22*mz2) + power6(mst2))))/(mst12*sb) + (cb*mu*mz2*log(mz2/Q2)*(-1 +
     mz2*(-mst12 + 5*mst22 + mz2)*DeltaInv(mst22,mz2,mst12) + 6*mst22*mz2*(-
     mst24 + (14*mst22 - mz2)*mz2 + mst12*(3*mst22 + mz2))*power2(DeltaInv(
     mst22,mz2,mst12)) - 72*mst22*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))
     *(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))
     ))/(12.*mst12*mst22*sb) + (cb*mu*mz2*sb*log(mz2/Q2)*(1 + (mst12 - 5*mst22
     - mz2)*mz2*DeltaInv(mst22,mz2,mst12) - 6*mst22*mz2*(-mst24 + (14*mst22 -
     mz2)*mz2 + mst12*(3*mst22 + mz2))*power2(DeltaInv(mst22,mz2,mst12)) + 72*
     mst22*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*(mz2*(-2*mst24 +
     mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/(12.*mst12*
     mst22) + (mu*mz2*log(mz2/Q2)*power3(cb)*(1 + (mst12 - 5*mst22 - mz2)*mz2*
     DeltaInv(mst22,mz2,mst12) - 6*mst22*mz2*(-mst24 + (14*mst22 - mz2)*mz2 +
     mst12*(3*mst22 + mz2))*power2(DeltaInv(mst22,mz2,mst12)) + 72*mst22*
     power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*(mz2*(-2*mst24 + mst22*mz2)
     - 2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/(12.*mst12*mst22*sb) + (
     cb*mu*sb*power2(DeltaInv(mst22,mA2,mst12))*power2(log(mA2/Q2))*power3(mA2)
     *(-mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(mst12*mst22
     + mst24) + mst22*power2(mA2) + power6(mst2))))/mst12 + (mu*power2(
     DeltaInv(mst22,mA2,mst12))*power2(log(mA2/Q2))*power3(cb)*power3(mA2)*(-
     mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 +
     mst24) + mst22*power2(mA2) + power6(mst2))))/(mst12*sb) + (cb*mA2*mu*sb*
     log(mA2/Q2)*(-1 + mA2*(mA2 - mst12 + 5*mst22)*DeltaInv(mst22,mA2,mst12) -
     6*mA2*mst22*(-3*mst12*mst22 - mA2*(mst12 + 14*mst22) + mst24 + power2(mA2)
     )*power2(DeltaInv(mst22,mA2,mst12)) - 72*mst22*power2(mA2)*power3(
     DeltaInv(mst22,mA2,mst12))*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 + mst24) +
     mst22*power2(mA2) + power6(mst2))))/(12.*mst12*mst22) + (mA2*mu*log(mA2/
     Q2)*power3(cb)*(-1 + mA2*(mA2 - mst12 + 5*mst22)*DeltaInv(mst22,mA2,mst12)
     - 6*mA2*mst22*(-3*mst12*mst22 - mA2*(mst12 + 14*mst22) + mst24 + power2(
     mA2))*power2(DeltaInv(mst22,mA2,mst12)) - 72*mst22*power2(mA2)*power3(
     DeltaInv(mst22,mA2,mst12))*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 + mst24) +
     mst22*power2(mA2) + power6(mst2))))/(12.*mst12*mst22*sb) + (mC2*mt2*mu*
     log(mC2/Q2)*power3(cb)*(-mst12 + mst22 + mC2*(-mC2 + msb12 - 5*mst12)*
     mst22*DeltaInv(msb12,mst12,mC2) + mC2*mst12*(mC2 - msb12 + 5*mst22)*
     DeltaInv(mst22,msb12,mC2) + 6*mC2*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12
     + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) - 6*
     mC2*mst12*mst22*(-3*msb12*mst22 - mC2*(msb12 + 14*mst22) + mst24 + power2(
     mC2))*power2(DeltaInv(mst22,msb12,mC2)) + 72*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - 72*mst12*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb12*mst12*(mst12 -
     mst22)*mst22*sb) + (cb*mC2*mu*sb*log(mC2/Q2)*power2(snt)*(mst12 - mst22 +
     mC2*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*mst12*(-
     mC2 + msb12 - 5*mst22)*DeltaInv(mst22,msb12,mC2) - 6*mC2*mst12*mst22*(-3*
     msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb12,mst12,mC2)) + 6*mC2*mst12*mst22*(-3*msb12*mst22 - mC2*(
     msb12 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb12,mC2))
     - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     (6.*msb12*mst12*mst22) + (mC2*mu*log(mC2/Q2)*power2(snt)*power3(cb)*(mst12
     - mst22 + mC2*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) +
     mC2*mst12*(-mC2 + msb12 - 5*mst22)*DeltaInv(mst22,msb12,mC2) - 6*mC2*
     mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2)
     )*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*mst12*mst22*(-3*msb12*mst22 -
     mC2*(msb12 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb12,
     mC2)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*
     msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(6.*msb12*mst12*mst22*sb) + (mC2*mu*log(mC2/Q2)*power3(cb)*(-((
     mst12 - mst22)*(mst12 - 2*mt2)) + 2*mC2*(mC2 - msb12 + 5*mst12)*mst22*mt2*
     DeltaInv(msb12,mst12,mC2) + mC2*mst12*(mC2 - msb12 + 5*mst22)*(mst12 -
     mst22 - 2*mt2)*DeltaInv(mst22,msb12,mC2) - 12*mC2*mst12*mst22*mt2*(-3*
     msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb12,mst12,mC2)) - 6*mC2*mst12*mst22*(mst12 - mst22 - 2*mt2)*(-
     3*msb12*mst22 - mC2*(msb12 + 14*mst22) + mst24 + power2(mC2))*power2(
     DeltaInv(mst22,msb12,mC2)) - 144*mst12*mst22*mt2*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)
     *power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb12*
     mst12*(mst12 - mst22)*mst22*sb) + (cb*mC2*mu*sb*log(mC2/Q2)*(-((mst12 -
     mst22)*(mst12 - mt2)) + mC2*(mC2 - msb12 + 5*mst12)*mst22*mt2*DeltaInv(
     msb12,mst12,mC2) - mC2*mst12*(mC2 - msb12 + 5*mst22)*(-mst12 + mst22 +
     mt2)*DeltaInv(mst22,msb12,mC2) - 6*mC2*mst12*mst22*mt2*(-3*msb12*mst12 -
     mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,
     mC2)) + 6*mC2*mst12*mst22*(-mst12 + mst22 + mt2)*(-3*msb12*mst22 - mC2*(
     msb12 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb12,mC2))
     - 72*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + 72*mst12*mst22*(-mst12 + mst22 + mt2)*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22)
     + Fin3(mH2,mst22,mst22,Q2)*((-4*ca*mt2*mu*sa*power2(DeltaInv(mH2,mst22,
     mst22))*(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)))/(mst12 -
     mst22) + (4*cb*mt2*mu*power2(sa)*power2(DeltaInv(mH2,mst22,mst22))*(mst24
     + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)))/((mst12 - mst22)*sb) + 4*
     ca*mu*sa*power2(snt)*power2(DeltaInv(mH2,mst22,mst22))*(mst24 + 4*mH2*
     DeltaInv(mH2,mst22,mst22)*power6(mst2)) - (4*cb*mu*power2(sa)*power2(snt)*
     power2(DeltaInv(mH2,mst22,mst22))*(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)
     *power6(mst2)))/sb - 4*ca*mu*sa*power2(DeltaInv(mH2,mst22,mst22))*power4(
     snt)*(mst24 + 4*mH2*DeltaInv(mH2,mst22,mst22)*power6(mst2)) + (4*cb*mu*
     power2(sa)*power2(DeltaInv(mH2,mst22,mst22))*power4(snt)*(mst24 + 4*mH2*
     DeltaInv(mH2,mst22,mst22)*power6(mst2)))/sb) + Fin3(mst22,mst22,mh2,Q2)*((
     4*ca*mt2*mu*sa*power2(DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*DeltaInv(
     mst22,mst22,mh2)*power6(mst2)))/(mst12 - mst22) + (4*cb*mt2*mu*power2(
     DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)))/((mst12 - mst22)*sb) - (4*cb*mt2*mu*power2(sa)*power2(
     DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)))/((mst12 - mst22)*sb) - 4*ca*mu*sa*power2(snt)*power2(
     DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)) - (4*cb*mu*power2(snt)*power2(DeltaInv(mst22,mst22,mh2))*(
     mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/sb + (4*cb*mu*
     power2(sa)*power2(snt)*power2(DeltaInv(mst22,mst22,mh2))*(mst24 + 4*mh2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/sb + 4*ca*mu*sa*power2(DeltaInv(
     mst22,mst22,mh2))*power4(snt)*(mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)) + (4*cb*mu*power2(DeltaInv(mst22,mst22,mh2))*power4(snt)*(
     mst24 + 4*mh2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/sb - (4*cb*mu*
     power2(sa)*power2(DeltaInv(mst22,mst22,mh2))*power4(snt)*(mst24 + 4*mh2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/sb) + Fin3(mst22,mw2,msb12,Q2)*((
     4*cb*(mst12 - mst22 - 2*mt2)*mu*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(-mst12 + mst22)*sb)
     + (4*cb*(mst12 - mst22 - mt2)*mu*sb*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)) + (
     4*cb*(mst12 - mst22 - 2*mt2)*mu*power2(mw2)*power2(snb)*power2(DeltaInv(
     mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 -
     mst22)*sb) + (4*cb*mt2*mu*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,
     mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2)
     - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb)
     + (4*cb*(mst12 - mst22 - mt2)*mu*sb*power2(mw2)*power2(snb)*power2(
     DeltaInv(mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     (-mst12 + mst22)) + (4*cb*mu*power2(mw2)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*sb) + (
     4*cb*mu*sb*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,
     msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 + (4*(mst12 - mst22 -
     2*mt2)*mu*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*power3(cb)*(mst22
     - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb) + (4*mt2*mu*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*power3(cb)*(mst22 -
     DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb) + (4*(mst12 -
     mst22 - 2*mt2)*mu*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb12)
     )*power3(cb)*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*
     mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*(-mst12 +
     mst22)*sb) + (4*mu*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb12))*power3(cb)*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     sb) + (4*cb*mt2*mu*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 +
     DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb) + (4*cb*mu*sb*
     power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 +
     DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/msb12 + (4*cb*mu*power2(mw2)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 + DeltaInv(mst22,
     mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/(msb12*sb) + (4*mt2*mu*power2(mw2)*power2(snb)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(cb)*(-mst22 + DeltaInv(mst22,mw2,msb12)*
     (mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)))
     )/(msb12*(mst12 - mst22)*sb) + (4*mu*power2(mw2)*power2(snt)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(cb)*(-mst22 + DeltaInv(mst22,mw2,msb12)*
     (mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)))
     )/(msb12*sb)) + Fin3(mst22,mw2,msb22,Q2)*((4*cb*(mst12 - mst22 - 2*mt2)*
     mu*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 -
     DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb22*(-mst12 + mst22)*sb) + (4*cb*(mst12 -
     mst22 - mt2)*mu*sb*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,
     msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*(mst12 - mst22)) + (
     4*cb*mu*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,
     msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*sb) + (4*(mst12 -
     mst22 - 2*mt2)*mu*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,msb22)
     )*power3(cb)*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*
     mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*(mst12 -
     mst22)*sb) + (4*mt2*mu*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,mw2,
     msb22))*power3(cb)*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*(mst12 -
     mst22)*sb) + (4*cb*mt2*mu*power2(mw2)*power2(snb)*power2(DeltaInv(mst22,
     mw2,msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*
     mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*(mst12 -
     mst22)*sb) + (4*cb*mu*sb*power2(mw2)*power2(snb)*power2(snt)*power2(
     DeltaInv(mst22,mw2,msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 +
     (4*mu*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22)
     )*power3(cb)*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*
     mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb22*sb)) + Fin3(
     mst22,mz2,mst12,Q2)*((2*cb*mu*sb*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12))*(mst22 - DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) -
     2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/mst12 + (2*mu*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12))*power3(cb)*(mst22 - DeltaInv(mst22,mz2,
     mst12)*(mz2*(-2*mst24 + mst22*mz2) - 2*mst12*(mst24 + mst22*mz2) + power6(
     mst2))))/(mst12*sb) + (2*cb*mu*power2(mz2)*power2(DeltaInv(mst22,mz2,
     mst12))*(-mst22 + DeltaInv(mst22,mz2,mst12)*(mz2*(-2*mst24 + mst22*mz2) -
     2*mst12*(mst24 + mst22*mz2) + power6(mst2))))/(mst12*sb)) + Fin3(mst22,
     mA2,mst12,Q2)*((2*cb*mu*sb*power2(mA2)*power2(DeltaInv(mst22,mA2,mst12))*(
     -mst22 + DeltaInv(mst22,mA2,mst12)*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 +
     mst24) + mst22*power2(mA2) + power6(mst2))))/mst12 + (2*mu*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12))*power3(cb)*(-mst22 + DeltaInv(mst22,mA2,
     mst12)*(-2*mst12*mst24 - 2*mA2*(mst12*mst22 + mst24) + mst22*power2(mA2) +
     power6(mst2))))/(mst12*sb)) + (2*cb*mu*sb*power2(snt)*power2(log(mC2/Q2))*
     power3(mC2)*(-(mst12*power2(DeltaInv(msb12,mst12,mC2))) + power3(DeltaInv(
     msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb12,mC2))*(mst22 -
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/msb12 + (2*mu*power2(snt)*power2(log(
     mC2/Q2))*power3(cb)*power3(mC2)*(-(mst12*power2(DeltaInv(msb12,mst12,mC2))
     ) + power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12
     + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,
     msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/(msb12*sb) + (
     2*mt2*mu*power2(log(mC2/Q2))*power3(cb)*power3(mC2)*(mst12*power2(
     DeltaInv(msb12,mst12,mC2)) - power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*
     msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2)))))/(msb12*(mst12 - mst22)*sb) + (2*mu*power2(log(mC2/Q2))*power3(
     cb)*power3(mC2)*(-2*mst12*mt2*power2(DeltaInv(msb12,mst12,mC2)) + 2*mt2*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + (mst12 - mst22 - 2*mt2)*
     power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*
     msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2)))))/(msb12*(mst12 - mst22)*sb) + (2*cb*mu*sb*power2(log(mC2/Q2))*
     power3(mC2)*(-(mst12*mt2*power2(DeltaInv(msb12,mst12,mC2))) + mt2*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + (mst12 - mst22 - mt2)*power2(DeltaInv(
     mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*
     mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/(msb12*(
     mst12 - mst22)) + Fin3(mst22,msb12,mC2,Q2)*((4*cb*mu*sb*power2(mC2)*
     power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,
     msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/msb12 + (4*mt2*mu*power2(mC2)*power2(snb)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(cb)*(mst22 - DeltaInv(mst22,msb12,mC2)*(
     -2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(msb12*(mst12 - mst22)*sb) + (4*mu*power2(mC2)*power2(snt)*
     power2(DeltaInv(mst22,msb12,mC2))*power3(cb)*(mst22 - DeltaInv(mst22,
     msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb12*sb) + (4*cb*(mst12 - mst22 - mt2)*mu*sb*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,
     msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb12*(mst12 - mst22)) + (4*cb*(mst12 - mst22 -
     mt2)*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(-
     mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(-mst12 + mst22)) + (
     4*cb*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,
     msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (4*(
     mst12 - mst22 - 2*mt2)*mu*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*
     power3(cb)*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 -
     mst22)*sb) + (4*mt2*mu*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))*
     power3(cb)*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 -
     mst22)*sb) + (4*(mst12 - mst22 - 2*mt2)*mu*power2(mC2)*power2(snb)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(cb)*(-mst22 + DeltaInv(mst22,msb12,mC2)*
     (-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2))))/(msb12*(-mst12 + mst22)*sb) + (4*mu*power2(mC2)*power2(
     snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*power3(cb)*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb12*sb)) + Fin3(mst22,msb22,mC2,Q2)
     *((4*cb*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (4*mu*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*
     power3(cb)*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*sb) + (
     4*cb*(mst12 - mst22 - mt2)*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(
     mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*
     mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(
     mst12 - mst22)) + (4*(mst12 - mst22 - 2*mt2)*mu*power2(mC2)*power2(snb)*
     power2(DeltaInv(mst22,msb22,mC2))*power3(cb)*(-mst22 + DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb22*(mst12 - mst22)*sb) + (4*mt2*mu*power2(mC2)*
     power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*power3(cb)*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22)*sb)) + Fin3(
     mst22,mh2,mst12,Q2)*((2*cb*mu*power2(mh2)*power2(sa)*power2(DeltaInv(
     mst22,mh2,mst12))*(mst22 - DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*
     mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power6(mst2))))/(mst12*sb)
     + (8*ca*mu*sa*power2(mh2)*power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(
     mst22 - DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 +
     mst24) + mst22*power2(mh2) + power6(mst2))))/mst12 + (8*cb*mu*power2(mh2)*
     power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(mst22 - DeltaInv(mst22,mh2,
     mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power6(mst2))))/(mst12*sb) + (8*cb*mu*power2(mh2)*power2(sa)*power2(
     DeltaInv(mst22,mh2,mst12))*power4(snt)*(mst22 - DeltaInv(mst22,mh2,mst12)*
     (-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power6(mst2))))/(mst12*sb) + (2*ca*mu*sa*power2(mh2)*power2(DeltaInv(
     mst22,mh2,mst12))*(-mst22 + DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 - 2*
     mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power6(mst2))))/mst12 + (
     2*cb*mu*power2(mh2)*power2(DeltaInv(mst22,mh2,mst12))*(-mst22 + DeltaInv(
     mst22,mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*
     power2(mh2) + power6(mst2))))/(mst12*sb) + (8*cb*mu*power2(mh2)*power2(sa)
     *power2(snt)*power2(DeltaInv(mst22,mh2,mst12))*(-mst22 + DeltaInv(mst22,
     mh2,mst12)*(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(
     mh2) + power6(mst2))))/(mst12*sb) + (8*ca*mu*sa*power2(mh2)*power2(
     DeltaInv(mst22,mh2,mst12))*power4(snt)*(-mst22 + DeltaInv(mst22,mh2,mst12)
     *(-2*mst12*mst24 - 2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) +
     power6(mst2))))/mst12 + (8*cb*mu*power2(mh2)*power2(DeltaInv(mst22,mh2,
     mst12))*power4(snt)*(-mst22 + DeltaInv(mst22,mh2,mst12)*(-2*mst12*mst24 -
     2*mh2*(mst12*mst22 + mst24) + mst22*power2(mh2) + power6(mst2))))/(mst12*
     sb)) + Fin3(mH2,mst22,mst12,Q2)*((2*ca*mu*sa*power2(mH2)*power2(DeltaInv(
     mH2,mst22,mst12))*(mst22 - DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*
     mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/mst12 + (
     8*cb*mu*power2(mH2)*power2(sa)*power2(snt)*power2(DeltaInv(mH2,mst22,
     mst12))*(mst22 - DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*
     mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/(mst12*sb) + (8*ca*
     mu*sa*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))*power4(snt)*(mst22 -
     DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) +
     mst22*power2(mH2) + power6(mst2))))/mst12 + (2*cb*mu*power2(mH2)*power2(
     sa)*power2(DeltaInv(mH2,mst22,mst12))*(-mst22 + DeltaInv(mH2,mst22,mst12)*
     (-2*mst12*mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) +
     power6(mst2))))/(mst12*sb) + (8*ca*mu*sa*power2(mH2)*power2(snt)*power2(
     DeltaInv(mH2,mst22,mst12))*(-mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*
     mst24 - 2*mH2*(mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/
     mst12 + (8*cb*mu*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,mst22,mst12))*
     power4(snt)*(-mst22 + DeltaInv(mH2,mst22,mst12)*(-2*mst12*mst24 - 2*mH2*(
     mst12*mst22 + mst24) + mst22*power2(mH2) + power6(mst2))))/(mst12*sb)) + (
     2*mu*power2(snb)*power2(log(mC2/Q2))*power3(cb)*power3(mC2)*(2*msb22*
     mst12*mt2*power2(DeltaInv(msb12,mst12,mC2)) - 2*msb12*mst12*mt2*power2(
     DeltaInv(msb22,mst12,mC2)) - 2*msb22*mt2*power3(DeltaInv(msb12,mst12,mC2))
     *(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) + 2*msb12*mt2*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) -
     (mst12 - mst22 - 2*mt2)*(-(msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)))
     + msb22*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*
     mst22 + mst24) + mst22*power2(mC2) + power6(mst2)) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     )/(msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mu*sb*power2(snb)*power2(log(
     mC2/Q2))*power3(mC2)*(msb22*mst12*mt2*power2(DeltaInv(msb12,mst12,mC2)) -
     msb12*mst12*mt2*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mt2*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb12*mt2*power3(DeltaInv(msb22,mst12,
     mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) - (mst12 - mst22 - mt2)*(-(msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2))) + msb22*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)
     *(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2))))))/(msb12*msb22*(mst12 - mst22)) + (cb*mu*sb*log(msb12/Q2)*
     (-(mC2*mst12*mst22) - mC2*mst12*mt2 + mC2*mst22*mt2 + mst12*mst22*mw2 +
     mst12*mt2*mw2 - mst22*mt2*mw2 - mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,
     msb12) + mst12*mst14*mst22*mt2*DeltaInv(mw2,msb12,mst12) + mC2*power2(
     mst12) - mw2*power2(mst12) + mst22*mst24*DeltaInv(mst22,mw2,msb12)*power2(
     mst12) - msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*
     mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) - mst12*mst24*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) + msb12*mst12*mt2*DeltaInv(mst22,
     mw2,msb12)*power2(mst22) - 5*mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12)*
     power2(mst22) - msb12*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(
     mst22) + 5*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mst22) +
     msb12*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) + msb12*mst12*mt2*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*mst12*mst22*mt2*DeltaInv(mst22,
     mw2,msb12)*power2(mw2) - msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(
     mw2) + 5*mst12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*
     DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2) + 5*mst22*DeltaInv(
     mst22,mw2,msb12)*power2(mst12)*power2(mw2) - 5*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mst22)*power2(mw2) - 6*msb12*mst12*mst22*mst24*mt2*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst22*mst24*mw2*power2(mst12)*
     power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst24*mw2*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12)) + 138*mst12*mst22*mst24*mt2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 138*mst22*mst24*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 138*mst12*mst24*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*mst12*mt2*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*power2(
     mst12)*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*
     msb12*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 138*
     mst12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 36*
     msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)
     ) - mst22*mt2*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) -
     (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) +
     mst12*(-mst12 + mst22 + mt2)*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*
     mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) + 6*mC2*mst12*mst22*mt2*power2(DeltaInv(msb12,mst12,mC2))*(-(
     msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) + msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power3(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power3(mst22)
     + 36*msb12*mst12*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*power3(
     mst22) - 6*mC2*mst12*mst22*(-mst12 + mst22 + mt2)*power2(DeltaInv(mst22,
     msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 -
     23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) - mst12*mst22*
     DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst12*mt2*DeltaInv(mst22,mw2,
     msb12)*power3(mw2) + mst22*mt2*DeltaInv(mw2,msb12,mst12)*power3(mw2) +
     DeltaInv(mst22,mw2,msb12)*power2(mst12)*power3(mw2) - 6*msb12*mst12*mst22*
     mt2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*
     mt2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*
     power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     + 6*msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) -
     138*mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2)
     + 138*mst12*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22)*power3(mw2) -
     72*mst12*mst22*mt2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*
     mst22*(-mst12 + mst22 + mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     720*msb12*mst12*mst22*mst24*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 720*msb12*mst22*mst24*power2(mst12)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 720*msb12*mst12*mst24*power2(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 216*msb12*mst22*mt2*power2(mw2)
     *power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 144*mst22*mt2*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*mst12*mt2*mw2*
     power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*mw2*power2(mst12)*
     power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*mst12*mt2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*
     power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     + 144*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mst22) - 144*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 6*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 6*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst22*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) + 144*mst12*mst24*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 216*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 216*msb12*power2(mst12)*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst12*mst14*
     mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 216*msb12*mst22*
     mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*
     mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 144*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) +
     144*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) + 144*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 144*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 144*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power6(mst22)))/(6.*msb12*mst12*(mst12 - mst22)*mst22) + (mu*
     log(msb12/Q2)*power3(cb)*(-(mC2*mst12*mst22) - 2*mC2*mst12*mt2 + 2*mC2*
     mst22*mt2 + mst12*mst22*mw2 + 2*mst12*mt2*mw2 - 2*mst22*mt2*mw2 - 2*mst12*
     mst22*mst24*mt2*DeltaInv(mst22,mw2,msb12) + 2*mst12*mst14*mst22*mt2*
     DeltaInv(mw2,msb12,mst12) + mC2*power2(mst12) - mw2*power2(mst12) + mst22*
     mst24*DeltaInv(mst22,mw2,msb12)*power2(mst12) - 2*msb12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + 10*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) - mst12*mst24*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) + 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - 10*
     mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*DeltaInv(
     mst22,mw2,msb12)*power2(mst12)*power2(mst22) + 5*mw2*DeltaInv(mst22,mw2,
     msb12)*power2(mst12)*power2(mst22) + msb12*mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) + 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(
     mw2) - 10*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 2*msb12*
     mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 10*mst12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*DeltaInv(mst22,mw2,msb12)*
     power2(mst12)*power2(mw2) + 5*mst22*DeltaInv(mst22,mw2,msb12)*power2(
     mst12)*power2(mw2) - 5*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)*
     power2(mw2) - 12*msb12*mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*msb12*mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,
     mw2,msb12)) - 6*msb12*mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12)) + 276*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 138*mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) + 138*mst12*mst24*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) + 72*msb12*mst12*mt2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 12*msb12*mst12*mst14*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 276*mst12*mst14*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 72*msb12*mst22*mt2*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 2*mst22*mt2*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*(-mst12 +
     mst22 + 2*mt2)*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) -
     (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 12*
     mC2*mst12*mst22*mt2*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) +
     mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(
     mC2) + power3(mst12)) + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power3(
     mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power3(mst22) + 36*msb12*
     mst12*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*mC2*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power3(mw2) - 2*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) +
     2*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power3(mw2) + DeltaInv(mst22,mw2,
     msb12)*power2(mst12)*power3(mw2) - 12*msb12*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(mst12)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 276*mst12*mt2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 12*
     msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 276*
     mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) +
     138*mst12*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22)*power3(mw2) -
     144*mst12*mst22*mt2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2))
     + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + 1440*msb12*mst12*mst22*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst22*mst24*power2(mst12)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst24*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 1440*msb12*mst12*
     mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 12*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 432*msb12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 288*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) +
     12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 432*msb12*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) -
     216*msb12*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 288*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) - 144*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) + 288*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))
     *power4(mw2) - 144*mst22*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 144*mst12*mst24*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mw2) + 432*msb12*mst12*mt2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*mst12*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 288*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 432*msb12*
     mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) +
     288*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)
     + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 288*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) +
     144*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 288*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) + 144*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 144*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 288*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power6(mst22)))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (
     mu*log(msb12/Q2)*power2(snb)*power3(cb)*(mC2*mst12*mst22 + 2*mC2*mst12*mt2
     - 2*mC2*mst22*mt2 - mst12*mst22*mw2 - 2*mst12*mt2*mw2 + 2*mst22*mt2*mw2 +
     2*mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,msb12) - 2*mst12*mst14*mst22*
     mt2*DeltaInv(mw2,msb12,mst12) - mC2*power2(mst12) + mw2*power2(mst12) -
     mst22*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst12) + 2*msb12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) - 10*mst22*mt2*mw2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) + mst12*mst24*DeltaInv(mst22,mw2,msb12)*power2(
     mst22) - 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 10*
     mst12*mt2*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + msb12*DeltaInv(
     mst22,mw2,msb12)*power2(mst12)*power2(mst22) - 5*mw2*DeltaInv(mst22,mw2,
     msb12)*power2(mst12)*power2(mst22) - msb12*mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) - 2*msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(
     mw2) + 10*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 2*msb12*
     mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 10*mst12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + msb12*DeltaInv(mst22,mw2,msb12)*
     power2(mst12)*power2(mw2) - 5*mst22*DeltaInv(mst22,mw2,msb12)*power2(
     mst12)*power2(mw2) + 5*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)*
     power2(mw2) + 12*msb12*mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) - 6*msb12*mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*msb12*mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12)) - 276*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 138*mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 138*mst12*mst24*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 72*msb12*mst12*mt2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*power2(mst12)*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 12*msb12*mst12*mst14*
     mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 276*mst12*mst14*mst22*
     mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 72*msb12*mst22*mt2*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 2*mst22*mt2*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*(mst12 -
     mst22 - 2*mt2)*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) -
     (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 12*
     mC2*mst12*mst22*mt2*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) +
     mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(
     mC2) + power3(mst12)) - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power3(
     mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power3(mst22) - 36*msb12*
     mst12*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*mC2*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) + mst12*mst22*DeltaInv(mst22,mw2,
     msb12)*power3(mw2) + 2*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) -
     2*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power3(mw2) - DeltaInv(mst22,mw2,
     msb12)*power2(mst12)*power3(mw2) + 12*msb12*mst12*mst22*mt2*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst22*power2(mst12)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 276*mst12*mt2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 12*
     msb12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 276*
     mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) -
     138*mst12*power2(DeltaInv(mst22,mw2,msb12))*power3(mst22)*power3(mw2) +
     144*mst12*mst22*mt2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2))
     + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*
     mst12*mst22*(mst12 - mst22 - 2*mt2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) - 1440*msb12*mst12*mst22*mst24*mt2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst22*mst24*power2(mst12)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst24*power2(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 1440*msb12*mst12*
     mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 12*mst22*
     mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 432*msb12*mst22*
     mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 288*
     mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) -
     12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 6*mw2*
     power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 432*msb12*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     216*msb12*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 288*mst12*mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power4(mst22) + 144*power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 6*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 6*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 12*mst12*mst22*mt2*power2(DeltaInv(mw2,msb12,mst12))
     *power4(mw2) - 288*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,mw2,msb12))
     *power4(mw2) + 144*mst22*mst24*power2(mst12)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) - 144*mst12*mst24*power2(mst22)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mw2) - 432*msb12*mst12*mt2*power2(mst22)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mst12*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 288*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 432*msb12*
     mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) -
     288*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)
     - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 288*
     mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) -
     144*power2(mst12)*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 288*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 144*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 144*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 288*mst22*mt2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power6(mst22)))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (
     cb*mu*sb*log(msb12/Q2)*power2(snb)*(mC2*mst12*mst22 + mC2*mst12*mt2 - mC2*
     mst22*mt2 - mst12*mst22*mw2 - mst12*mt2*mw2 + mst22*mt2*mw2 + mst12*mst22*
     mst24*mt2*DeltaInv(mst22,mw2,msb12) - mst12*mst14*mst22*mt2*DeltaInv(mw2,
     msb12,mst12) - mC2*power2(mst12) + mw2*power2(mst12) - mst22*mst24*
     DeltaInv(mst22,mw2,msb12)*power2(mst12) + msb12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) - 5*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) + mst12*mst24*DeltaInv(mst22,mw2,msb12)*power2(mst22) -
     msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mt2*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) + msb12*DeltaInv(mst22,mw2,msb12)*
     power2(mst12)*power2(mst22) - 5*mw2*DeltaInv(mst22,mw2,msb12)*power2(
     mst12)*power2(mst22) - msb12*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(
     mw2) - msb12*mst12*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + 5*mst12*
     mst22*mt2*DeltaInv(mst22,mw2,msb12)*power2(mw2) + msb12*mst22*mt2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*mst12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mw2) + msb12*DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power2(mw2) - 5*mst22*DeltaInv(mst22,mw2,msb12)*power2(mst12)*power2(mw2)
     + 5*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22)*power2(mw2) + 6*msb12*
     mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*
     mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12)) - 138*
     mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 138*
     mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     138*mst12*mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - 36*msb12*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 36*msb12*power2(mst12)*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + mst22*mt2*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) - mst12*(-mst12 + mst22 + mt2)*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*mt2*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power3(mst22) + 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power3(mst22) - 36*msb12*mst12*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) + 6*mC2*mst12*mst22*(-
     mst12 + mst22 + mt2)*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) +
     mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + mst12*mst22*DeltaInv(mst22,mw2,msb12)*power3(mw2)
     + mst12*mt2*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*mt2*DeltaInv(
     mw2,msb12,mst12)*power3(mw2) - DeltaInv(mst22,mw2,msb12)*power2(mst12)*
     power3(mw2) + 6*msb12*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12)
     )*power3(mw2) + 6*msb12*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 138*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mw2) + 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) + 138*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst12*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mst22)*power3(mw2) + 72*mst12*mst22*mt2*power2(mC2)*(-(
     (3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*(-mst12 + mst22 + mt2)*power2(
     mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) +
     720*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 6*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12)
     + 216*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) + 144*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) - 6*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 216*msb12*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) +
     216*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 216*msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 144*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 216*msb12*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 144*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power5(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power5(mst22) + 144*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mst22) - 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) - 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power6(mst22)))/(6.*msb12*
     mst12*(mst12 - mst22)*mst22) + (cb*mu*sb*log(msb22/Q2)*power2(snb)*(-(mC2*
     mst12*mst22) - mC2*mst12*mt2 + mC2*mst22*mt2 + mst12*mst22*mw2 + mst12*
     mt2*mw2 - mst22*mt2*mw2 - mst12*mst22*mst24*mt2*DeltaInv(mst22,mw2,msb22)
     + mst12*mst14*mst22*mt2*DeltaInv(mw2,msb22,mst12) + mC2*power2(mst12) -
     mw2*power2(mst12) + mst22*mst24*DeltaInv(mst22,mw2,msb22)*power2(mst12) -
     msb22*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mt2*mw2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) - mst12*mst24*DeltaInv(mst22,mw2,
     msb22)*power2(mst22) + msb22*mst12*mt2*DeltaInv(mst22,mw2,msb22)*power2(
     mst22) - 5*mst12*mt2*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22) - msb22*
     DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(mst22) + 5*mw2*DeltaInv(
     mst22,mw2,msb22)*power2(mst12)*power2(mst22) + msb22*mst12*mst22*DeltaInv(
     mst22,mw2,msb22)*power2(mw2) + msb22*mst12*mt2*DeltaInv(mst22,mw2,msb22)*
     power2(mw2) - 5*mst12*mst22*mt2*DeltaInv(mst22,mw2,msb22)*power2(mw2) -
     msb22*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 5*mst12*mst22*mt2*
     DeltaInv(mw2,msb22,mst12)*power2(mw2) - msb22*DeltaInv(mst22,mw2,msb22)*
     power2(mst12)*power2(mw2) + 5*mst22*DeltaInv(mst22,mw2,msb22)*power2(
     mst12)*power2(mw2) - 5*mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22)*
     power2(mw2) - 6*msb22*mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,
     msb22)) + 6*msb22*mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,
     msb22)) - 6*msb22*mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb22)) + 138*mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) - 138*mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) + 138*mst12*mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 36*msb22*mst12*mt2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 36*msb22*power2(mst12)*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mt2*
     mw2*power2(DeltaInv(mw2,msb22,mst12)) - 138*mst12*mst14*mst22*mt2*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 36*msb22*mst22*mt2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - mst22*mt2*DeltaInv(msb22,
     mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2)
     + 5*mC2*power2(mst12) + power3(mC2)) + mst12*(-mst12 + mst22 + mt2)*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*mt2*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*
     msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power3(mst22) - 5*
     mst12*mw2*DeltaInv(mst22,mw2,msb22)*power3(mst22) + 36*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) - 6*mC2*mst12*mst22*(
     -mst12 + mst22 + mt2)*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) +
     mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     - mst12*mt2*DeltaInv(mst22,mw2,msb22)*power3(mw2) + mst22*mt2*DeltaInv(
     mw2,msb22,mst12)*power3(mw2) + DeltaInv(mst22,mw2,msb22)*power2(mst12)*
     power3(mw2) - 6*msb22*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) + 6*msb22*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22)
     )*power3(mw2) - 6*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mw2) + 138*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb22))*power3(mw2) - 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) - 138*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst12*power2(DeltaInv(mst22,
     mw2,msb22))*power3(mst22)*power3(mw2) - 72*mst12*mst22*mt2*power2(mC2)*(-(
     (3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*(-mst12 + mst22 + mt2)*power2(
     mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*mt2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     720*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 6*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     - 216*msb22*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) - 144*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mst12) + 6*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) + 216*msb22*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 144*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 6*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 216*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) -
     216*msb22*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 216*msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 144*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2) - 216*msb22*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 144*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power5(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power5(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power5(mst22) - 144*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power5(mst22) + 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) - 144*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power6(mst22)))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + (mu*log(msb22/Q2)*power2(snb)*power3(cb)*(-
     (mC2*mst12*mst22) - 2*mC2*mst12*mt2 + 2*mC2*mst22*mt2 + mst12*mst22*mw2 +
     2*mst12*mt2*mw2 - 2*mst22*mt2*mw2 - 2*mst12*mst22*mst24*mt2*DeltaInv(
     mst22,mw2,msb22) + 2*mst12*mst14*mst22*mt2*DeltaInv(mw2,msb22,mst12) +
     mC2*power2(mst12) - mw2*power2(mst12) + mst22*mst24*DeltaInv(mst22,mw2,
     msb22)*power2(mst12) - 2*msb22*mst22*mt2*DeltaInv(mw2,msb22,mst12)*power2(
     mst12) + 10*mst22*mt2*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) - mst12*
     mst24*DeltaInv(mst22,mw2,msb22)*power2(mst22) + 2*msb22*mst12*mt2*
     DeltaInv(mst22,mw2,msb22)*power2(mst22) - 10*mst12*mt2*mw2*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) - msb22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*
     power2(mst22) + 5*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(
     mst22) + msb22*mst12*mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 2*
     msb22*mst12*mt2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 10*mst12*mst22*
     mt2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 2*msb22*mst22*mt2*DeltaInv(
     mw2,msb22,mst12)*power2(mw2) + 10*mst12*mst22*mt2*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) - msb22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(
     mw2) + 5*mst22*DeltaInv(mst22,mw2,msb22)*power2(mst12)*power2(mw2) - 5*
     mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22)*power2(mw2) - 12*msb22*
     mst12*mst22*mst24*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*
     mst22*mst24*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*
     mst12*mst24*mw2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22)) + 276*
     mst12*mst22*mst24*mt2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 138*
     mst22*mst24*power2(mst12)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     138*mst12*mst24*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)
     ) + 72*msb22*mst12*mt2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) - 36*msb22*power2(mst12)*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 12*msb22*mst12*mst14*mst22*mt2*mw2*power2(
     DeltaInv(mw2,msb22,mst12)) - 276*mst12*mst14*mst22*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*mt2*power2(mst12)*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 2*mst22*mt2*DeltaInv(msb22,mst12,mC2)*
     (mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*(-mst12 + mst22 + 2*mt2)*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 12*mC2*mst12*mst22*mt2*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power3(mst22) - 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb22)*power3(mst22) + 36*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*mC2*mst12*mst22*(mst12
     - mst22 - 2*mt2)*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(
     6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - mst12*mst22*DeltaInv(mst22,mw2,msb22)*power3(mw2) - 2*
     mst12*mt2*DeltaInv(mst22,mw2,msb22)*power3(mw2) + 2*mst22*mt2*DeltaInv(
     mw2,msb22,mst12)*power3(mw2) + DeltaInv(mst22,mw2,msb22)*power2(mst12)*
     power3(mw2) - 12*msb22*mst12*mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) + 6*msb22*mst22*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22)
     )*power3(mw2) - 6*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mw2) + 276*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,
     mw2,msb22))*power3(mw2) - 138*power2(mst12)*power2(mst22)*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 12*msb22*mst12*mst22*mt2*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) - 276*mst22*mt2*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst12*power2(DeltaInv(mst22,
     mw2,msb22))*power3(mst22)*power3(mw2) - 144*mst12*mst22*mt2*power2(mC2)*(-
     ((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22
     + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*
     power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 1440*msb22*mst12*mst22*mst24*
     mt2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst22*mst24*
     power2(mst12)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*
     mst12*mst24*power2(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     1440*msb22*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     - 432*msb22*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) - 288*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mst12) + 12*mst12*mt2*mw2*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 6*mw2*power2(mst12)*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) + 432*msb22*mst12*mt2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 288*mst12*mt2*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 144*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 12*mst12*
     mst22*mt2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst22*power2(
     mst12)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 6*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 12*mst12*mst22*mt2*
     power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 288*mst12*mst22*mst24*mt2*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 144*mst22*mst24*power2(
     mst12)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 144*mst12*mst24*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 432*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) -
     216*msb22*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 216*msb22*mst12*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 288*mst12*mst14*mst22*mt2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2) - 432*msb22*mst22*mt2*power2(mst12)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 288*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power5(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power5(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power5(mst22) - 288*mst12*mt2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power5(mst22) + 144*power2(mst12)*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) + 144*mst12*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) - 288*mst12*mt2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*power2(mst12)*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 144*mst12*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 288*mst22*mt2*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power6(mst22)))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22*sb) - (mu*log(mst12/Q2)*power3(cb)*(-(mA2*
     msb12*mst12) + mA2*msb12*mst22 + msb12*mst12*mz2 - msb12*mst22*mz2 - 4*
     mst12*(5*mC2*mst12 - msb12*mst12 + mst14)*mst22*mt2*DeltaInv(msb12,mst12,
     mC2) - msb12*mst12*mst22*mst24*DeltaInv(mst22,mz2,mst12) + 4*mst12*mst14*
     mst22*mt2*DeltaInv(mw2,msb12,mst12) - 4*msb12*mst22*mt2*DeltaInv(mw2,
     msb12,mst12)*power2(mst12) + 20*mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) + msb12*mst24*DeltaInv(mst22,mz2,mst12)*power2(mst22) - 5*
     msb12*mst12*mz2*DeltaInv(mst22,mz2,mst12)*power2(mst22) + msb12*DeltaInv(
     mst22,mz2,mst12)*power2(mst12)*power2(mst22) - 6*msb12*mst12*mst22*
     DeltaInv(mst22,mz2,mst12)*power2(mz2) + msb12*DeltaInv(mst22,mz2,mst12)*
     power2(mst12)*power2(mz2) + 5*msb12*DeltaInv(mst22,mz2,mst12)*power2(
     mst22)*power2(mz2) - 6*msb12*mst22*mst24*mz2*power2(mst12)*power2(
     DeltaInv(mst22,mz2,mst12)) + 6*msb12*mst12*mst24*mz2*power2(mst22)*power2(
     DeltaInv(mst22,mz2,mst12)) + 138*msb12*mst12*mst22*mst24*power2(mz2)*
     power2(DeltaInv(mst22,mz2,mst12)) - 138*msb12*mst24*power2(mst22)*power2(
     mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 36*msb12*power2(mst12)*power2(
     mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) + 24*msb12*mst12*
     mst14*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 336*mst12*mst14*
     mst22*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 72*msb12*mst22*
     mt2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + msb12*(
     mst12 - mst22)*DeltaInv(mst22,mA2,mst12)*(mst22*(-(mst12*mst22) + mst24) -
     (mst12 - 5*mst22)*power2(mA2) + 5*mA2*power2(mst22) + power3(mA2)) + 24*
     mC2*mst12*mst22*mt2*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12
     - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) - msb12*
     mst12*DeltaInv(mst22,mz2,mst12)*power3(mst22) + 5*msb12*mz2*DeltaInv(
     mst22,mz2,mst12)*power3(mst22) - 36*msb12*mst12*power2(mz2)*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mst22) - 6*mA2*msb12*(mst12 - mst22)*
     mst22*power2(DeltaInv(mst22,mA2,mst12))*(-(mst12*mst24) + mA2*(6*mst12*
     mst22 + 23*mst24) - (mst12 - 23*mst22)*power2(mA2) + power3(mA2) + power3(
     mst22)) - 24*mst22*mt2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - msb12*mst12*DeltaInv(mst22,mz2,mst12)*power3(mz2) + msb12*
     mst22*DeltaInv(mst22,mz2,mst12)*power3(mz2) - 6*msb12*mst22*power2(mst12)*
     power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) + 144*msb12*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2) - 138*msb12*power2(
     DeltaInv(mst22,mz2,mst12))*power3(mst22)*power3(mz2) + 288*mst12*mst22*
     mt2*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*
     mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*
     msb12*(mst12 - mst22)*mst22*power2(mA2)*((3*mst12*mst22 + 2*mst24)*power2(
     mA2) - 2*mst22*power3(mA2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mA2*(5*
     mst12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) + 720*
     msb12*mst22*mst24*power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12)) - 720*msb12*mst12*mst24*power2(mst22)*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) + 576*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 24*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 576*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 576*mst22*mt2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*msb12*mst12*mz2*power2(
     DeltaInv(mst22,mz2,mst12))*power4(mst22) + 216*msb12*power2(mst12)*power2(
     mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) + 144*msb12*mst12*
     power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power4(mst22) - 288*mst12*
     mst14*mst22*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 6*msb12*
     mst12*mst22*power2(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 6*msb12*
     power2(mst22)*power2(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 144*msb12*
     mst12*mst22*mst24*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 144*
     msb12*mst24*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) +
     216*msb12*power2(mst12)*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*
     power4(mz2) - 216*msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,mz2,
     mst12))*power4(mz2) - 288*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mst12) - 6*msb12*mz2*power2(DeltaInv(mst22,mz2,mst12))*
     power5(mst22) - 360*msb12*mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power5(mst22) - 144*msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,
     mst12))*power5(mst22) - 144*msb12*mst12*power2(mst22)*power3(DeltaInv(
     mst22,mz2,mst12))*power5(mz2) + 144*msb12*power3(mst22)*power3(DeltaInv(
     mst22,mz2,mst12))*power5(mz2) + 144*msb12*power2(mz2)*power3(DeltaInv(
     mst22,mz2,mst12))*power6(mst22)))/(12.*msb12*mst12*(mst12 - mst22)*mst22*
     sb) - (cb*mu*sb*log(mst12/Q2)*(-(mA2*msb12*mst12) + mA2*msb12*mst22 +
     msb12*mst12*mz2 - msb12*mst22*mz2 - 2*mst12*(5*mC2*mst12 - msb12*mst12 +
     mst14)*mst22*mt2*DeltaInv(msb12,mst12,mC2) - msb12*mst12*mst22*mst24*
     DeltaInv(mst22,mz2,mst12) + 2*mst12*mst14*mst22*mt2*DeltaInv(mw2,msb12,
     mst12) - 2*msb12*mst22*mt2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 10*
     mst22*mt2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst24*
     DeltaInv(mst22,mz2,mst12)*power2(mst22) - 5*msb12*mst12*mz2*DeltaInv(
     mst22,mz2,mst12)*power2(mst22) + msb12*DeltaInv(mst22,mz2,mst12)*power2(
     mst12)*power2(mst22) - 6*msb12*mst12*mst22*DeltaInv(mst22,mz2,mst12)*
     power2(mz2) + msb12*DeltaInv(mst22,mz2,mst12)*power2(mst12)*power2(mz2) +
     5*msb12*DeltaInv(mst22,mz2,mst12)*power2(mst22)*power2(mz2) - 6*msb12*
     mst22*mst24*mz2*power2(mst12)*power2(DeltaInv(mst22,mz2,mst12)) + 6*msb12*
     mst12*mst24*mz2*power2(mst22)*power2(DeltaInv(mst22,mz2,mst12)) + 138*
     msb12*mst12*mst22*mst24*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)) -
     138*msb12*mst24*power2(mst22)*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)
     ) + 36*msb12*power2(mst12)*power2(mst22)*power2(mz2)*power2(DeltaInv(
     mst22,mz2,mst12)) + 12*msb12*mst12*mst14*mst22*mt2*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 168*mst12*mst14*mst22*mt2*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) + 36*msb12*mst22*mt2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + msb12*(mst12 - mst22)*DeltaInv(mst22,mA2,
     mst12)*(mst22*(-(mst12*mst22) + mst24) - (mst12 - 5*mst22)*power2(mA2) +
     5*mA2*power2(mst22) + power3(mA2)) + 12*mC2*mst12*mst22*mt2*power2(
     DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*
     mst14 + mst12*power2(mC2) + power3(mst12)) - msb12*mst12*DeltaInv(mst22,
     mz2,mst12)*power3(mst22) + 5*msb12*mz2*DeltaInv(mst22,mz2,mst12)*power3(
     mst22) - 36*msb12*mst12*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12))*
     power3(mst22) - 6*mA2*msb12*(mst12 - mst22)*mst22*power2(DeltaInv(mst22,
     mA2,mst12))*(-(mst12*mst24) + mA2*(6*mst12*mst22 + 23*mst24) - (mst12 -
     23*mst22)*power2(mA2) + power3(mA2) + power3(mst22)) - 12*mst22*mt2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - msb12*mst12*
     DeltaInv(mst22,mz2,mst12)*power3(mz2) + msb12*mst22*DeltaInv(mst22,mz2,
     mst12)*power3(mz2) - 6*msb12*mst22*power2(mst12)*power2(DeltaInv(mst22,
     mz2,mst12))*power3(mz2) + 144*msb12*mst12*power2(mst22)*power2(DeltaInv(
     mst22,mz2,mst12))*power3(mz2) - 138*msb12*power2(DeltaInv(mst22,mz2,mst12)
     )*power3(mst22)*power3(mz2) + 144*mst12*mst22*mt2*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*(mst12 -
     mst22)*mst22*power2(mA2)*((3*mst12*mst22 + 2*mst24)*power2(mA2) - 2*mst22*
     power3(mA2) + (3*mst12 - 2*mst22)*power3(mst22) + 2*mA2*(5*mst12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,mA2,mst12)) + 720*msb12*mst22*mst24*
     power2(mst12)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) - 720*msb12*
     mst12*mst24*power2(mst22)*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12)) +
     288*msb12*mst12*mst14*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 12*mst22*mt2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12)
     + 288*msb12*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mst12) + 288*mst22*mt2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mst12) + 6*msb12*mst12*mz2*power2(DeltaInv(mst22,mz2,mst12))*
     power4(mst22) + 216*msb12*power2(mst12)*power2(mz2)*power3(DeltaInv(mst22,
     mz2,mst12))*power4(mst22) + 144*msb12*mst12*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12))*power4(mst22) - 144*mst12*mst14*mst22*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) - 6*msb12*power2(mst22)*power2(
     DeltaInv(mst22,mz2,mst12))*power4(mz2) + 144*msb12*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 144*msb12*mst24*power2(
     mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) + 216*msb12*power2(
     mst12)*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - 216*
     msb12*mst12*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) -
     144*mst22*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12)
     - 6*msb12*mz2*power2(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 360*msb12*
     mst12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 144*
     msb12*power3(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power5(mst22) - 144*
     msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) +
     144*msb12*power3(mst22)*power3(DeltaInv(mst22,mz2,mst12))*power5(mz2) +
     144*msb12*power2(mz2)*power3(DeltaInv(mst22,mz2,mst12))*power6(mst22)))/(
     12.*msb12*mst12*(mst12 - mst22)*mst22)) - (56*msb12*power3(DeltaInv(msb12,
     mt2,mu2))*power6(mu2))/mt2 - (56*mst22*power3(DeltaInv(mst22,mt2,mu2))*
     power6(mu2))/mt2 - (56*mst12*power3(DeltaInv(mt2,mu2,mst12))*power6(mu2))/
     mt2 + (power2(Pi)*power2(snb)*(-(msb22*DeltaInv(msb12,mt2,mu2)*power2(
     msb12)*power2(mu2)) + msb12*DeltaInv(mt2,mu2,msb22)*power2(msb22)*power2(
     mu2) + msb22*(3*mC2*mst22 + msb12*mst22 + 3*mst24)*mt2*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*mt2*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*mt2*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 4*msb12*msb22*msb24*mt2*power2(mu2)*
     power2(DeltaInv(mt2,mu2,msb22)) + msb12*msb22*mst12*power2(mt2)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*power2(mt2)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*msb22*mst12*power2(mt2)*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*power2(mt2)*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst22*mt2*mu2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - msb12*msb22*power2(mu2)*
     power2(DeltaInv(msb12,mt2,mu2))*(4*msb14*(mt2 + 4*mu2) + msb12*mu2*(6*mt2
     + 7*mu2) + power3(msb12)) + 16*msb12*msb22*msb24*power2(DeltaInv(mt2,mu2,
     msb22))*power3(mu2) + 6*msb12*mt2*power2(msb22)*power2(DeltaInv(mt2,mu2,
     msb22))*power3(mu2) + 3*msb22*mst12*power2(mt2)*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) - 3*msb12*mst12*power2(mt2)*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 14*msb12*msb22*mst24*mt2*mu2*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*mt2*mu2*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mt2*mu2*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(
     mt2)*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 14*
     msb12*msb22*mst14*power2(mt2)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     ) + 3*msb22*power2(mt2)*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 5*msb12*msb22*power2(mt2)*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst14*power2(mt2)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*power2(mt2)*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*
     msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*
     msb12*mst24*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + msb12*
     power2(mu2)*power2(DeltaInv(mt2,mu2,msb22))*power4(msb22) + 32*msb12*mt2*
     power3(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22) - 3*msb22*
     power2(mt2)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) +
     3*msb12*power2(mt2)*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mst12) + msb22*mt2*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(14*
     mC2*msb12*mst24 + 5*msb12*mst22*power2(mC2) + 3*mst24*power2(mC2) - 3*
     mst22*power3(mC2) + 3*mC2*power3(mst22) + 5*msb12*power3(mst22) - 3*
     power4(mst22)) + 3*msb12*mt2*mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mst22) + 7*msb12*power2(msb22)*power2(DeltaInv(mt2,mu2,msb22)
     )*power4(mu2) + 50*msb12*msb22*msb24*mt2*power3(DeltaInv(mt2,mu2,msb22))*
     power4(mu2) + 18*msb12*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22)*
     power4(mu2) + 5*msb12*msb22*mst12*power2(mt2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) + 3*msb22*mst14*power2(mt2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 5*msb12*msb22*mst12*power2(mt2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mw2) - 3*msb12*mst14*power2(mt2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mw2) + 3*msb12*mst22*mt2*mu2*power3(DeltaInv(mst22,
     msb22,mC2))*power5(mC2) + 2*msb12*msb22*power2(mu2)*power3(DeltaInv(msb12,
     mt2,mu2))*(-(msb14*(25*mt2 + mu2)*power2(mu2)) - mu2*(16*mt2 + 9*mu2)*
     power3(msb12) - (mt2 - 5*mu2)*power4(msb12) + msb12*(-6*mt2*power3(mu2) +
     4*power4(mu2)) + power5(msb12)) + 2*msb12*mt2*power2(mu2)*power3(DeltaInv(
     mt2,mu2,msb22))*power5(msb22) - 10*msb12*power3(mu2)*power3(DeltaInv(mt2,
     mu2,msb22))*power5(msb22) + 2*msb12*msb22*msb24*power3(DeltaInv(mt2,mu2,
     msb22))*power5(mu2) + 12*msb12*mt2*power2(msb22)*power3(DeltaInv(mt2,mu2,
     msb22))*power5(mu2) - 3*msb22*mst12*power2(mt2)*power3(DeltaInv(mw2,msb12,
     mst12))*power5(mw2) + 3*msb12*mst12*power2(mt2)*power3(DeltaInv(mw2,msb22,
     mst12))*power5(mw2) - 2*msb12*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*
     power6(msb22) - 8*msb12*power2(msb22)*power3(DeltaInv(mt2,mu2,msb22))*
     power6(mu2)))/(6.*msb12*msb22*mt2) + (power2(snb)*(msb12*mst22*mt2*mu2 -
     msb22*mst22*mt2*mu2 - msb12*mst22*mt2*mw2 + msb22*mst22*mt2*mw2 - 2*mC2*
     msb12*msb22*mst22*mt2*mu2*DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*msb22*
     mst22*mt2*mu2*DeltaInv(mst22,msb22,mC2) + 2*msb12*msb22*msb24*mst22*mu2*
     DeltaInv(mt2,mu2,msb22) + msb22*mt2*power2(msb12) - msb12*mt2*power2(
     msb22) - 2*msb12*mst22*mt2*mu2*DeltaInv(mt2,mu2,msb22)*power2(msb22) -
     msb12*mt2*power2(mst22) + msb22*mt2*power2(mst22) + msb12*mst22*power2(
     mt2) - msb22*mst22*power2(mt2) - 2*msb12*msb22*mst22*mw2*DeltaInv(mw2,
     msb12,mst12)*power2(mt2) + 2*msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,
     mst12)*power2(mt2) - 6*msb12*msb22*mst22*mt2*DeltaInv(mt2,mu2,msb22)*
     power2(mu2) + 80*msb12*mst22*DeltaInv(mt2,mu2,msb22)*power2(msb22)*power2(
     mu2) - 2*msb12*msb22*mst22*mu2*DeltaInv(msb12,mt2,mu2)*(msb14 - msb12*mt2
     + 40*msb12*mu2 - 3*mt2*mu2 + power2(mu2)) + 84*msb22*mst22*(3*mC2*mst22 +
     msb12*mst22 + 3*mst24)*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) - 252*msb12*mst22*mst24*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) - 84*msb12*msb22*mt2*mu2*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) + 336*msb12*msb22*msb24*mst22*mt2*power2(mu2)*
     power2(DeltaInv(mt2,mu2,msb22)) + 84*msb12*msb22*mst12*mst22*power2(mt2)*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 252*msb22*mst14*mst22*
     power2(mt2)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb12*
     msb22*mst12*mst22*power2(mt2)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) - 252*msb12*mst14*mst22*power2(mt2)*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) - 252*msb12*mt2*mu2*power2(mst22)*power2(DeltaInv(mst22,
     msb22,mC2))*power3(mC2) - 84*msb12*msb22*mst22*power2(mu2)*power2(
     DeltaInv(msb12,mt2,mu2))*(4*msb14*(mt2 + 4*mu2) + msb12*mu2*(6*mt2 + 7*
     mu2) + power3(msb12)) + 2*msb12*msb22*mst22*DeltaInv(mt2,mu2,msb22)*
     power3(mu2) + 1344*msb12*msb22*msb24*mst22*power2(DeltaInv(mt2,mu2,msb22))
     *power3(mu2) + 504*msb12*mst22*mt2*power2(msb22)*power2(DeltaInv(mt2,mu2,
     msb22))*power3(mu2) + 252*msb22*mst12*mst22*power2(mt2)*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) - 252*msb12*mst12*mst22*power2(mt2)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) - 1176*msb12*msb22*mst22*mst24*mt2*
     mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 420*msb12*msb22*mst22*
     power2(mt2)*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     1176*msb12*msb22*mst14*mst22*power2(mt2)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 252*msb22*mst22*power2(mt2)*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 420*msb12*msb22*mst22*power2(mt2)*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 1176*msb12*
     msb22*mst14*mst22*power2(mt2)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     ) - 252*msb12*mst22*power2(mt2)*power3(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12)) - 252*msb12*mst22*mst24*mt2*mu2*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) - 420*msb12*msb22*mt2*mu2*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 84*msb12*mst22*power2(mu2)*
     power2(DeltaInv(mt2,mu2,msb22))*power4(msb22) + 2688*msb12*mst22*mt2*
     power3(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22) - 252*msb22*
     mst22*power2(mt2)*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12) + 252*msb12*mst22*power2(mt2)*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 420*msb12*msb22*mt2*mu2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 252*msb12*mt2*mu2*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 84*msb22*mst22*mt2*mu2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-14*mC2*msb12*mst24 - 5*
     msb12*mst22*power2(mC2) - 3*mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*
     mC2*power3(mst22) - 5*msb12*power3(mst22) + 3*power4(mst22)) + 588*msb12*
     mst22*power2(msb22)*power2(DeltaInv(mt2,mu2,msb22))*power4(mu2) + 4200*
     msb12*msb22*msb24*mst22*mt2*power3(DeltaInv(mt2,mu2,msb22))*power4(mu2) +
     1512*msb12*mst22*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22)*power4(mu2)
     + 420*msb12*msb22*mst12*mst22*power2(mt2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 252*msb22*mst14*mst22*power2(mt2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) - 420*msb12*msb22*mst12*mst22*power2(mt2)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 252*msb12*mst14*mst22*
     power2(mt2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 252*msb12*mt2*
     mu2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 168*
     msb12*msb22*mst22*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*(-(msb14*(
     25*mt2 + mu2)*power2(mu2)) - mu2*(16*mt2 + 9*mu2)*power3(msb12) - (mt2 -
     5*mu2)*power4(msb12) + msb12*(-6*mt2*power3(mu2) + 4*power4(mu2)) +
     power5(msb12)) + 168*msb12*mst22*mt2*power2(mu2)*power3(DeltaInv(mt2,mu2,
     msb22))*power5(msb22) - 840*msb12*mst22*power3(mu2)*power3(DeltaInv(mt2,
     mu2,msb22))*power5(msb22) + 252*msb12*mt2*mu2*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*power5(mst22) + 168*msb12*msb22*msb24*mst22*power3(
     DeltaInv(mt2,mu2,msb22))*power5(mu2) + 1008*msb12*mst22*mt2*power2(msb22)*
     power3(DeltaInv(mt2,mu2,msb22))*power5(mu2) - 252*msb22*mst12*mst22*
     power2(mt2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 252*msb12*
     mst12*mst22*power2(mt2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) -
     168*msb12*mst22*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power6(msb22)
     - 672*msb12*mst22*power2(msb22)*power3(DeltaInv(mt2,mu2,msb22))*power6(
     mu2)))/(12.*msb12*msb22*mst22*mt2) + (power2(log(mt2/Q2))*((-mh2 + mt2)*
     DeltaInv(mh2,mt2,mt2)*power2(mt2) + mz2*DeltaInv(mt2,mt2,mz2)*power2(mt2)
     + 2*msb12*DeltaInv(msb12,mt2,mu2)*power2(mu2) + 2*mst22*DeltaInv(mst22,
     mt2,mu2)*power2(mu2) + 2*mst12*DeltaInv(mt2,mu2,mst12)*power2(mu2) + 2*(3*
     msb14*(mt2 + 4*mu2) + msb12*mu2*(5*mt2 + 4*mu2))*power2(mu2)*power2(
     DeltaInv(msb12,mt2,mu2)) + 6*mst24*mt2*power2(mu2)*power2(DeltaInv(mst22,
     mt2,mu2)) + 6*mst14*mt2*power2(mu2)*power2(DeltaInv(mt2,mu2,mst12)) +
     DeltaInv(mt2,mt2,mz2)*power3(mt2) + 24*mst24*power2(DeltaInv(mst22,mt2,
     mu2))*power3(mu2) + 10*mst22*mt2*power2(DeltaInv(mst22,mt2,mu2))*power3(
     mu2) + 24*mst14*power2(DeltaInv(mt2,mu2,mst12))*power3(mu2) + 10*mst12*
     mt2*power2(DeltaInv(mt2,mu2,mst12))*power3(mu2) + 42*mt2*power3(mst22)*
     power3(mu2)*power3(DeltaInv(mst22,mt2,mu2)) + 42*mt2*power3(mst12)*power3(
     mu2)*power3(DeltaInv(mt2,mu2,mst12)) + 2*mt2*power2(mu2)*power3(DeltaInv(
     mt2,mu2,mst12))*power4(mst12) - 16*power3(mu2)*power3(DeltaInv(mt2,mu2,
     mst12))*power4(mst12) + 2*mt2*power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*
     power4(mst22) - 16*power3(mu2)*power3(DeltaInv(mst22,mt2,mu2))*power4(
     mst22) + 4*mz2*power2(DeltaInv(mt2,mt2,mz2))*power4(mt2) + 8*mst22*power2(
     DeltaInv(mst22,mt2,mu2))*power4(mu2) + 8*mst12*power2(DeltaInv(mt2,mu2,
     mst12))*power4(mu2) + 70*mst24*mt2*power3(DeltaInv(mst22,mt2,mu2))*power4(
     mu2) + 28*power3(mst22)*power3(DeltaInv(mst22,mt2,mu2))*power4(mu2) + 70*
     mst14*mt2*power3(DeltaInv(mt2,mu2,mst12))*power4(mu2) + 28*power3(mst12)*
     power3(DeltaInv(mt2,mu2,mst12))*power4(mu2) - 2*power2(mu2)*power3(
     DeltaInv(msb12,mt2,mu2))*(-35*msb14*mt2*power2(mu2) - 7*mu2*(3*mt2 + 2*
     mu2)*power3(msb12) + msb12*(-7*mt2 + 5*mu2)*power3(mu2) - (mt2 - 8*mu2)*
     power4(msb12) + power5(msb12)) - 2*power2(mu2)*power3(DeltaInv(mt2,mu2,
     mst12))*power5(mst12) - 2*power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*
     power5(mst22) + 14*mst22*mt2*power3(DeltaInv(mst22,mt2,mu2))*power5(mu2) +
     14*mst12*mt2*power3(DeltaInv(mt2,mu2,mst12))*power5(mu2) - 10*mst22*
     power3(DeltaInv(mst22,mt2,mu2))*power6(mu2) - 10*mst12*power3(DeltaInv(
     mt2,mu2,mst12))*power6(mu2)))/(2.*mt2) - (power2(Pi)*(-(mt2*(mh2 + 5*mt2)*
     DeltaInv(mh2,mt2,mt2)) + mt2*mz2*DeltaInv(mt2,mt2,mz2) + 7*DeltaInv(mt2,
     mt2,mz2)*power2(mt2) - (2*msb12*DeltaInv(msb12,mt2,mu2)*power2(mu2))/mt2 -
     (2*mst22*DeltaInv(mst22,mt2,mu2)*power2(mu2))/mt2 - (2*mst12*DeltaInv(mt2,
     mu2,mst12)*power2(mu2))/mt2 + mst22*mu2*power2(mH2)*power2(DeltaInv(mH2,
     mst22,mst12)) + (3*mst24*mu2*power2(mH2)*power2(DeltaInv(mH2,mst22,mst12))
     )/mst12 - 8*msb14*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2)) + mst22*mu2*
     power2(mA2)*power2(DeltaInv(mst22,mA2,mst12)) + (3*mst24*mu2*power2(mA2)*
     power2(DeltaInv(mst22,mA2,mst12)))/mst12 + 2*mst22*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + (6*mst24*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)))/msb12 + 2*mh2*mst24*mt2*power2(DeltaInv(mst22,mst22,
     mh2)) - 8*mst24*power2(mu2)*power2(DeltaInv(mst22,mt2,mu2)) - 8*mst14*
     power2(mu2)*power2(DeltaInv(mt2,mu2,mst12)) + 2*mst12*mt2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + (6*mst14*mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)))/msb12 + (3*mst22*mu2*power2(DeltaInv(mst22,
     mA2,mst12))*power3(mA2))/mst12 + (6*mst22*mu2*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2))/msb12 + (3*mst22*mu2*power2(DeltaInv(mH2,mst22,mst12))*
     power3(mH2))/mst12 - (2*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*
     power3(msb12))/mt2 - (2*power2(mu2)*power2(DeltaInv(mt2,mu2,mst12))*
     power3(mst12))/mt2 + 2*mt2*power2(DeltaInv(mh2,mst12,mst12))*(mh2*mst14 +
     6*power3(mst12)) + 12*mt2*power2(DeltaInv(mst22,mst22,mh2))*power3(mst22)
     - (2*power2(mu2)*power2(DeltaInv(mst22,mt2,mu2))*power3(mst22))/mt2 - 18*
     mh2*power2(DeltaInv(mh2,mt2,mt2))*power3(mt2) + 34*mz2*power2(DeltaInv(
     mt2,mt2,mz2))*power3(mt2) - 12*msb12*power2(DeltaInv(msb12,mt2,mu2))*
     power3(mu2) - (32*msb14*power2(DeltaInv(msb12,mt2,mu2))*power3(mu2))/mt2 -
     12*mst22*power2(DeltaInv(mst22,mt2,mu2))*power3(mu2) - (32*mst24*power2(
     DeltaInv(mst22,mt2,mu2))*power3(mu2))/mt2 - 12*mst12*power2(DeltaInv(mt2,
     mu2,mst12))*power3(mu2) - (32*mst14*power2(DeltaInv(mt2,mu2,mst12))*
     power3(mu2))/mt2 + (6*mst12*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2))/msb12 + 14*mst24*mu2*power3(mH2)*power3(DeltaInv(mH2,mst22,mst12)) +
     5*mu2*power2(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)) + (3*
     mu2*power3(mH2)*power3(mst22)*power3(DeltaInv(mH2,mst22,mst12)))/mst12 -
     64*power3(msb12)*power3(mu2)*power3(DeltaInv(msb12,mt2,mu2)) + 14*mst24*
     mu2*power3(mA2)*power3(DeltaInv(mst22,mA2,mst12)) + 5*mu2*power2(mA2)*
     power3(mst22)*power3(DeltaInv(mst22,mA2,mst12)) + (3*mu2*power3(mA2)*
     power3(mst22)*power3(DeltaInv(mst22,mA2,mst12)))/mst12 + 28*mst24*mu2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 10*mu2*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) + (6*mu2*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)))/msb12 - 64*power3(mst22)*power3(
     mu2)*power3(DeltaInv(mst22,mt2,mu2)) - 64*power3(mst12)*power3(mu2)*
     power3(DeltaInv(mt2,mu2,mst12)) + 10*mt2*power2(mw2)*power3(mst12)*power3(
     DeltaInv(mw2,msb12,mst12)) + 28*mst14*mt2*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + (6*mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)))/msb12 + 5*mst22*mu2*power3(DeltaInv(mst22,mA2,mst12))*
     power4(mA2) + (3*mst24*mu2*power3(DeltaInv(mst22,mA2,mst12))*power4(mA2))/
     mst12 + 10*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + (6*
     mst24*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2))/msb12 + 5*mst22*
     mu2*power3(DeltaInv(mH2,mst22,mst12))*power4(mH2) + (3*mst24*mu2*power3(
     DeltaInv(mH2,mst22,mst12))*power4(mH2))/mst12 - 4*power2(mu2)*power3(
     DeltaInv(msb12,mt2,mu2))*power4(msb12) + (20*power3(mu2)*power3(DeltaInv(
     msb12,mt2,mu2))*power4(msb12))/mt2 + 48*mh2*mt2*power3(DeltaInv(mh2,mst12,
     mst12))*power4(mst12) - 4*power2(mu2)*power3(DeltaInv(mt2,mu2,mst12))*
     power4(mst12) + (20*power3(mu2)*power3(DeltaInv(mt2,mu2,mst12))*power4(
     mst12))/mt2 - (6*mt2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12))/msb12 - (3*mu2*power2(mH2)*power3(DeltaInv(mH2,mst22,mst12))*
     power4(mst22))/mst12 - (3*mu2*power2(mA2)*power3(DeltaInv(mst22,mA2,mst12)
     )*power4(mst22))/mst12 - (6*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22))/msb12 + 48*mh2*mt2*power3(DeltaInv(mst22,mst22,mh2))*
     power4(mst22) - 4*power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*power4(
     mst22) + (20*power3(mu2)*power3(DeltaInv(mst22,mt2,mu2))*power4(mst22))/
     mt2 + 24*power2(DeltaInv(mt2,mt2,mz2))*power4(mt2) - (14*msb12*power2(
     DeltaInv(msb12,mt2,mu2))*power4(mu2))/mt2 - (14*mst22*power2(DeltaInv(
     mst22,mt2,mu2))*power4(mu2))/mt2 - (14*mst12*power2(DeltaInv(mt2,mu2,
     mst12))*power4(mu2))/mt2 - 100*msb14*power3(DeltaInv(msb12,mt2,mu2))*
     power4(mu2) - (36*power3(msb12)*power3(DeltaInv(msb12,mt2,mu2))*power4(
     mu2))/mt2 - 100*mst24*power3(DeltaInv(mst22,mt2,mu2))*power4(mu2) - (36*
     power3(mst22)*power3(DeltaInv(mst22,mt2,mu2))*power4(mu2))/mt2 - 100*
     mst14*power3(DeltaInv(mt2,mu2,mst12))*power4(mu2) - (36*power3(mst12)*
     power3(DeltaInv(mt2,mu2,mst12))*power4(mu2))/mt2 + 10*mst12*mt2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + (6*mst14*mt2*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2))/msb12 - (3*mst22*mu2*power3(DeltaInv(mst22,mA2,
     mst12))*power5(mA2))/mst12 - (6*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2)
     )*power5(mC2))/msb12 - (3*mst22*mu2*power3(DeltaInv(mH2,mst22,mst12))*
     power5(mH2))/mst12 + (4*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*
     power5(msb12))/mt2 + (4*power2(mu2)*power3(DeltaInv(mt2,mu2,mst12))*
     power5(mst12))/mt2 + (4*power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*
     power5(mst22))/mt2 + 96*mz2*power3(DeltaInv(mt2,mt2,mz2))*power5(mt2) -
     24*msb12*power3(DeltaInv(msb12,mt2,mu2))*power5(mu2) - (4*msb14*power3(
     DeltaInv(msb12,mt2,mu2))*power5(mu2))/mt2 - 24*mst22*power3(DeltaInv(
     mst22,mt2,mu2))*power5(mu2) - (4*mst24*power3(DeltaInv(mst22,mt2,mu2))*
     power5(mu2))/mt2 - 24*mst12*power3(DeltaInv(mt2,mu2,mst12))*power5(mu2) -
     (4*mst14*power3(DeltaInv(mt2,mu2,mst12))*power5(mu2))/mt2 - (6*mst12*mt2*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2))/msb12 + (16*msb12*power3(
     DeltaInv(msb12,mt2,mu2))*power6(mu2))/mt2 + (16*mst22*power3(DeltaInv(
     mst22,mt2,mu2))*power6(mu2))/mt2 + (16*mst12*power3(DeltaInv(mt2,mu2,
     mst12))*power6(mu2))/mt2))/12. + (log(mu2/Q2)*(-1 + mst22/(mst22 - mu2) -
     mu2/mst22 + power2(mu2)/(msb12*mt2) + power2(mu2)/(mst12*mt2) + power2(
     mu2)/(mst22*mt2) - 2*DeltaInv(mt2,mu2,mst12)*power2(mu2) - (7*mst12*
     DeltaInv(mt2,mu2,mst12)*power2(mu2))/mt2 - (DeltaInv(msb12,mt2,mu2)*(mu2*(
     -mt2 + mu2) + 2*msb12*(mt2 + 2*mu2) + 7*power2(msb12))*power2(mu2))/(
     msb12*mt2) - (DeltaInv(mst22,mt2,mu2)*(mu2*(-mt2 + mu2) + 2*mst22*(mt2 +
     2*mu2) + 7*power2(mst22))*power2(mu2))/(mst22*mt2) - 6*mst14*power2(mu2)*
     power2(DeltaInv(mt2,mu2,mst12)) + (6*power2(mu2)*power2(DeltaInv(msb12,
     mt2,mu2))*(-(msb14*(mt2 + 7*mu2)) - msb12*mu2*(12*mt2 + 19*mu2) + (-mt2 +
     mu2)*power2(mu2) + power3(msb12)))/mt2 + (6*power2(mu2)*power2(DeltaInv(
     mt2,mu2,mst12))*power3(mst12))/mt2 + (6*power2(mu2)*power2(DeltaInv(mst22,
     mt2,mu2))*(-(mst24*(mt2 + 7*mu2)) - mst22*mu2*(12*mt2 + 19*mu2) + (-mt2 +
     mu2)*power2(mu2) + power3(mst22)))/mt2 + (DeltaInv(mt2,mu2,mst12)*power3(
     mu2))/mst12 - (4*DeltaInv(mt2,mu2,mst12)*power3(mu2))/mt2 - 72*mst12*
     power2(DeltaInv(mt2,mu2,mst12))*power3(mu2) - (42*mst14*power2(DeltaInv(
     mt2,mu2,mst12))*power3(mu2))/mt2 - 36*power3(mst12)*power3(mu2)*power3(
     DeltaInv(mt2,mu2,mst12)) + (36*power3(mu2)*power3(DeltaInv(msb12,mt2,mu2))
     *(-5*msb14*mu2*(2*mt2 + mu2) + msb12*(-5*mt2 + 3*mu2)*power2(mu2) + (-mt2
     + mu2)*power3(msb12) + power4(msb12)))/mt2 + (36*power3(mu2)*power3(
     DeltaInv(mt2,mu2,mst12))*power4(mst12))/mt2 + (36*power3(mu2)*power3(
     DeltaInv(mst22,mt2,mu2))*(-5*mst24*mu2*(2*mt2 + mu2) + mst22*(-5*mt2 + 3*
     mu2)*power2(mu2) + (-mt2 + mu2)*power3(mst22) + power4(mst22)))/mt2 - (
     DeltaInv(mt2,mu2,mst12)*power4(mu2))/(mst12*mt2) - 6*power2(DeltaInv(mt2,
     mu2,mst12))*power4(mu2) - (114*mst12*power2(DeltaInv(mt2,mu2,mst12))*
     power4(mu2))/mt2 - 360*mst14*power3(DeltaInv(mt2,mu2,mst12))*power4(mu2) +
     (36*power3(mst12)*power3(DeltaInv(mt2,mu2,mst12))*power4(mu2))/mt2 + (6*
     power2(DeltaInv(mt2,mu2,mst12))*power5(mu2))/mt2 - 180*mst12*power3(
     DeltaInv(mt2,mu2,mst12))*power5(mu2) - (180*mst14*power3(DeltaInv(mt2,mu2,
     mst12))*power5(mu2))/mt2 + (108*mst12*power3(DeltaInv(mt2,mu2,mst12))*
     power6(mu2))/mt2))/6. - (71*power6(mw2))/(72.*power6(mt2 - mw2)) + (71*
     power7(mw2))/(72.*mt2*power6(mt2 - mw2)) + (power2(cb)*((-18*mC2)/msb12 -
     (9*mA2)/mst12 - (9*mA2)/mst22 - (18*mC2)/mst22 - (35*mC2)/(mC2 - mt2) - (
     35*mC2)/mt2 + (18*mw2)/msb12 + (18*mw2)/mst22 + (35*mw2)/mt2 + (35*mw2)/(-
     mt2 + mw2) + (9*mz2)/mst12 + (9*mz2)/mst22 + 36*mC2*mt2*(1 - (2*mu2)/(
     mst12 - mst22))*DeltaInv(msb12,mst12,mC2) - 18*mA2*mu2*DeltaInv(mst22,mA2,
     mst12) - 36*mC2*mu2*DeltaInv(mst22,msb12,mC2) + (72*mC2*mt2*mu2*DeltaInv(
     mst22,msb12,mC2))/(mst12 - mst22) + 36*mu2*mw2*DeltaInv(mst22,mw2,msb12) -
     (72*mt2*mu2*mw2*DeltaInv(mst22,mw2,msb12))/(mst12 - mst22) + 18*mu2*mz2*
     DeltaInv(mst22,mz2,mst12) - 684*mA2*mt2*DeltaInv(mt2,mt2,mA2) + 684*mt2*
     mz2*DeltaInv(mt2,mt2,mz2) - 36*mt2*mw2*DeltaInv(mw2,msb12,mst12) + (72*
     mt2*mu2*mw2*DeltaInv(mw2,msb12,mst12))/(mst12 - mst22) - (35*power2(mC2))/
     power2(mC2 - mt2) - 5292*DeltaInv(mt2,mt2,mA2)*power2(mt2) + 5292*
     DeltaInv(mt2,mt2,mz2)*power2(mt2) + (35*power2(mw2))/power2(mt2 - mw2) - (
     1512*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*mt2*(mst12 - mst22 - 2*mu2)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)))/(msb12*(mst12 - mst22)) + (
     756*(3*mA2*mst22 + mst12*mst22 + 3*mst24)*mu2*power2(mA2)*power2(DeltaInv(
     mst22,mA2,mst12)))/mst12 + 1512*mst22*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + (4536*mst24*mu2*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)))/msb12 - (3024*mst22*mt2*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)))/(mst12 - mst22) - (9072*mst24*mt2*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)))/(msb12*mst12 - msb12*mst22) - 1512*
     mst22*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - (4536*mst24*mu2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/msb12 + (3024*mst22*mt2*
     mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) + (
     9072*mst24*mt2*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)))/(msb12*
     mst12 - msb12*mst22) - 756*mst22*mu2*power2(mz2)*power2(DeltaInv(mst22,
     mz2,mst12)) - (2268*mst24*mu2*power2(mz2)*power2(DeltaInv(mst22,mz2,mst12)
     ))/mst12 + 1512*mst12*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     (4536*mst14*mt2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/msb12 - (
     3024*mst12*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/(mst12 -
     mst22) - (9072*mst14*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     )/(msb12*mst12 - msb12*mst22) + (45*power3(mC2))/(mt2*power2(mC2 - mt2)) +
     (4536*mst22*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2))/msb12 + (
     9072*mst22*mt2*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2))/(msb12*(
     -mst12 + mst22)) + (10*power3(mC2))/power3(mC2 - mt2) - 25704*mA2*power2(
     DeltaInv(mt2,mt2,mA2))*power3(mt2) + 25704*mz2*power2(DeltaInv(mt2,mt2,
     mz2))*power3(mt2) - (45*power3(mw2))/(mt2*power2(mt2 - mw2)) - (4536*
     mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/msb12 + (9072*
     mst22*mt2*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2))/(msb12*mst12
     - msb12*mst22) + (4536*mst12*mt2*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2))/msb12 - (9072*mst12*mt2*mu2*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2))/(msb12*mst12 - msb12*mst22) + (10*power3(mw2))/power3(mt2 -
     mw2) - (2268*mst22*mu2*power2(DeltaInv(mst22,mz2,mst12))*power3(mz2))/
     mst12 + (1512*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*(-((5*msb12*mst12 +
     3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(
     mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)))/(msb12*(mst12 - mst22)) + 21168*mst24*mu2*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - (42336*mst24*mt2*mu2*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)))/(mst12 - mst22) + 7560*mu2*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) - (15120*mt2*mu2*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)))/(mst12 - mst22) + (4536*
     mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)))/msb12 - (
     9072*mt2*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)))/
     (msb12*mst12 - msb12*mst22) - 7560*mu2*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) + (15120*mt2*mu2*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - 21168*mst24*mu2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + (42336*mst24*mt2*mu2*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/(mst12 - mst22) - (4536*
     mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/msb12 + (
     9072*mt2*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)))/
     (msb12*mst12 - msb12*mst22) - 3780*mu2*power2(mz2)*power3(mst22)*power3(
     DeltaInv(mst22,mz2,mst12)) - 10584*mst24*mu2*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)) - (2268*mu2*power3(mst22)*power3(mz2)*power3(DeltaInv(
     mst22,mz2,mst12)))/mst12 + 7560*mt2*power2(mw2)*power3(mst12)*power3(
     DeltaInv(mw2,msb12,mst12)) - (15120*mt2*mu2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + 21168*mst14*mt2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - (42336*mst14*mt2*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/(mst12 - mst22) + (4536*
     mt2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/msb12 - (
     9072*mt2*mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/
     (msb12*mst12 - msb12*mst22) + 7560*mst22*mu2*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) + (4536*mst24*mu2*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2))/msb12 - (15120*mst22*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2)
     )*power4(mC2))/(mst12 - mst22) - (9072*mst24*mt2*mu2*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2))/(msb12*mst12 - msb12*mst22) - (4536*mt2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12))/msb12 + (
     9072*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12))/
     (msb12*mst12 - msb12*mst22) + (756*mu2*power2(mA2)*power3(DeltaInv(mst22,
     mA2,mst12))*(14*mA2*mst12*mst24 + 5*mst12*mst22*power2(mA2) + 3*mst24*
     power2(mA2) - 3*mst22*power3(mA2) + 3*mA2*power3(mst22) + 5*mst12*power3(
     mst22) - 3*power4(mst22)))/mst12 - (4536*mu2*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22))/msb12 + (9072*mt2*mu2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22))/(msb12*mst12 - msb12*mst22) + (
     4536*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22))/
     msb12 - (9072*mt2*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22))/(msb12*mst12 - msb12*mst22) + (2268*mu2*power2(mz2)*power3(
     DeltaInv(mst22,mz2,mst12))*power4(mst22))/mst12 + (10*power4(mC2))/power4(
     mC2 - mt2) - 18144*power2(DeltaInv(mt2,mt2,mA2))*power4(mt2) + 18144*
     power2(DeltaInv(mt2,mt2,mz2))*power4(mt2) - 7560*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - (4536*mst24*mu2*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2))/msb12 + (15120*mst22*mt2*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2))/(mst12 - mst22) + (9072*mst24*mt2*
     mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2))/(msb12*mst12 - msb12*
     mst22) + 7560*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + (
     4536*mst14*mt2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2))/msb12 - (
     15120*mst12*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2))/(mst12
     - mst22) - (9072*mst14*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2))/(msb12*mst12 - msb12*mst22) - (10*power4(mw2))/power4(mt2 - mw2) -
     3780*mst22*mu2*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2) - (2268*
     mst24*mu2*power3(DeltaInv(mst22,mz2,mst12))*power4(mz2))/mst12 - (4536*
     mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2))/msb12 + (9072*
     mst22*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power5(mC2))/(msb12*mst12
     - msb12*mst22) - (223*power5(mC2))/(mt2*power4(mC2 - mt2)) - (213*power5(
     mC2))/power5(mC2 - mt2) - 72576*mA2*power3(DeltaInv(mt2,mt2,mA2))*power5(
     mt2) + 72576*mz2*power3(DeltaInv(mt2,mt2,mz2))*power5(mt2) + (4536*mst22*
     mu2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2))/msb12 + (9072*mst22*
     mt2*mu2*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2))/(msb12*(-mst12 +
     mst22)) - (4536*mst12*mt2*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2))/
     msb12 + (9072*mst12*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2))
     /(msb12*mst12 - msb12*mst22) + (223*power5(mw2))/(mt2*power4(mt2 - mw2)) +
     (213*power5(mw2))/power5(-mt2 + mw2) + (2268*mst22*mu2*power3(DeltaInv(
     mst22,mz2,mst12))*power5(mz2))/mst12 - (213*power6(mC2))/power6(mC2 - mt2)
     + (213*power6(mw2))/power6(mt2 - mw2) + (213*power7(mC2))/(mt2*power6(mC2
     - mt2)) - (213*power7(mw2))/(mt2*power6(mt2 - mw2))))/216. + (log(mt2/Q2)*
     (-12 + (4*mt2)/mh2 - (6*mt2)/msb12 - (6*mt2)/mst12 - (6*mt2)/mst22 - (6*
     mu2)/msb12 - (6*mu2)/mst12 - (6*mu2)/mst22 + (2*mw2)/mt2 + (14*mw2)/(-mt2
     + mw2) - 2*mt2*(29*mh2 + 100*mt2)*DeltaInv(mh2,mt2,mt2) + 6*mst22*mu2*
     DeltaInv(mst22,mt2,mu2) + 78*mt2*mz2*DeltaInv(mt2,mt2,mz2) + 6*mst12*mu2*
     DeltaInv(mt2,mu2,mst12) + 288*DeltaInv(mt2,mt2,mz2)*power2(mt2) - (6*
     power2(mu2))/(msb12*mt2) - (6*power2(mu2))/(mst12*mt2) - (6*power2(mu2))/(
     mst22*mt2) + 12*DeltaInv(mst22,mt2,mu2)*power2(mu2) - (102*mst22*DeltaInv(
     mst22,mt2,mu2)*power2(mu2))/mt2 + 12*DeltaInv(mt2,mu2,mst12)*power2(mu2) -
     (102*mst12*DeltaInv(mt2,mu2,mst12)*power2(mu2))/mt2 + (6*mu2*DeltaInv(
     msb12,mt2,mu2)*(2*msb12*mu2*(mt2 + 2*mu2) + (mt2 - 17*mu2)*power2(msb12) +
     (-mt2 + mu2)*power2(mu2)))/(msb12*mt2) + (2*power2(mw2))/power2(mt2 - mw2)
     - 468*mst24*power2(mu2)*power2(DeltaInv(mst22,mt2,mu2)) - 468*mst14*
     power2(mu2)*power2(DeltaInv(mt2,mu2,mst12)) - (108*power2(mu2)*power2(
     DeltaInv(mt2,mu2,mst12))*power3(mst12))/mt2 - (108*power2(mu2)*power2(
     DeltaInv(mst22,mt2,mu2))*power3(mst22))/mt2 - 648*mh2*power2(DeltaInv(mh2,
     mt2,mt2))*power3(mt2) + 1368*mz2*power2(DeltaInv(mt2,mt2,mz2))*power3(mt2)
     - (6*DeltaInv(mst22,mt2,mu2)*power3(mu2))/mst22 + (24*DeltaInv(mst22,mt2,
     mu2)*power3(mu2))/mt2 - (6*DeltaInv(mt2,mu2,mst12)*power3(mu2))/mst12 + (
     24*DeltaInv(mt2,mu2,mst12)*power3(mu2))/mt2 - 720*mst22*power2(DeltaInv(
     mst22,mt2,mu2))*power3(mu2) - (2412*mst24*power2(DeltaInv(mst22,mt2,mu2))*
     power3(mu2))/mt2 - 720*mst12*power2(DeltaInv(mt2,mu2,mst12))*power3(mu2) -
     (2412*mst14*power2(DeltaInv(mt2,mu2,mst12))*power3(mu2))/mt2 - (36*power2(
     mu2)*power2(DeltaInv(msb12,mt2,mu2))*(13*msb14*mt2 + 67*msb14*mu2 + 5*
     msb12*mu2*(4*mt2 + 5*mu2) - mt2*power2(mu2) + 3*power3(msb12) + power3(
     mu2)))/mt2 + (2*power3(mw2))/(mt2*power2(mt2 - mw2)) + (4*power3(mw2))/
     power3(-mt2 + mw2) - 4536*power3(mst22)*power3(mu2)*power3(DeltaInv(mst22,
     mt2,mu2)) - 4536*power3(mst12)*power3(mu2)*power3(DeltaInv(mt2,mu2,mst12))
     - 216*power2(mu2)*power3(DeltaInv(mt2,mu2,mst12))*power4(mst12) + (1728*
     power3(mu2)*power3(DeltaInv(mt2,mu2,mst12))*power4(mst12))/mt2 - 216*
     power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*power4(mst22) + (1728*power3(
     mu2)*power3(DeltaInv(mst22,mt2,mu2))*power4(mst22))/mt2 + 864*power2(
     DeltaInv(mt2,mt2,mz2))*power4(mt2) + (6*DeltaInv(mst22,mt2,mu2)*power4(
     mu2))/(mst22*mt2) + (6*DeltaInv(mt2,mu2,mst12)*power4(mu2))/(mst12*mt2) +
     36*power2(DeltaInv(mst22,mt2,mu2))*power4(mu2) - (900*mst22*power2(
     DeltaInv(mst22,mt2,mu2))*power4(mu2))/mt2 + 36*power2(DeltaInv(mt2,mu2,
     mst12))*power4(mu2) - (900*mst12*power2(DeltaInv(mt2,mu2,mst12))*power4(
     mu2))/mt2 - 7560*mst24*power3(DeltaInv(mst22,mt2,mu2))*power4(mu2) - (
     3024*power3(mst22)*power3(DeltaInv(mst22,mt2,mu2))*power4(mu2))/mt2 -
     7560*mst14*power3(DeltaInv(mt2,mu2,mst12))*power4(mu2) - (3024*power3(
     mst12)*power3(DeltaInv(mt2,mu2,mst12))*power4(mu2))/mt2 + (4*power4(mw2))/
     power4(mt2 - mw2) + (216*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*(-35*
     msb14*mt2*power2(mu2) - 7*mu2*(3*mt2 + 2*mu2)*power3(msb12) + msb12*(-7*
     mt2 + 5*mu2)*power3(mu2) - (mt2 - 8*mu2)*power4(msb12) + power5(msb12)))/
     mt2 + (216*power2(mu2)*power3(DeltaInv(mt2,mu2,mst12))*power5(mst12))/mt2
     + (216*power2(mu2)*power3(DeltaInv(mst22,mt2,mu2))*power5(mst22))/mt2 +
     3456*mz2*power3(DeltaInv(mt2,mt2,mz2))*power5(mt2) - (36*power2(DeltaInv(
     mst22,mt2,mu2))*power5(mu2))/mt2 - (36*power2(DeltaInv(mt2,mu2,mst12))*
     power5(mu2))/mt2 - 1512*mst22*power3(DeltaInv(mst22,mt2,mu2))*power5(mu2)
     - 1512*mst12*power3(DeltaInv(mt2,mu2,mst12))*power5(mu2) + (7*power5(mw2))
     /(mt2*power4(mt2 - mw2)) + (11*power5(mw2))/power5(-mt2 + mw2) + (1080*
     mst22*power3(DeltaInv(mst22,mt2,mu2))*power6(mu2))/mt2 + (1080*mst12*
     power3(DeltaInv(mt2,mu2,mst12))*power6(mu2))/mt2 + (11*power6(mw2))/
     power6(mt2 - mw2) - (11*power7(mw2))/(mt2*power6(mt2 - mw2))))/36.;

   return power2(g3) * power2(yt) * result * twoLoop;
}

/// 2-loop O(alpha_b*alpha_s) contributions to Delta alpha_s [arXiv:1009.5455]
Real delta_alpha_s_2loop_ab_as(const Parameters& pars)
{
   using std::log;
   const Real g3    = pars.g3;
   const Real yb    = pars.yb;
   const Real xt    = pars.xt;
   const Real xb    = pars.xb;
   const Real mt    = pars.mt;
   const Real mt2   = power2(pars.mt);
   const Real mb    = pars.mb;
   const Real mst1  = pars.mst1;
   const Real mst12 = power2(pars.mst1);
   const Real mst14 = power4(pars.mst1);
   const Real mst2  = pars.mst2;
   const Real mst22 = power2(pars.mst2);
   const Real mst24 = power4(pars.mst2);
   const Real msb1  = pars.msb1;
   const Real msb12 = power2(pars.msb1);
   const Real msb14 = power4(pars.msb1);
   const Real msb2  = pars.msb2;
   const Real msb22 = power2(pars.msb2);
   const Real msb24 = power4(pars.msb2);
   const Real mw2   = power2(pars.mw);
   const Real mz2   = power2(pars.mz);
   const Real mh2   = power2(pars.mh);
   const Real mH2   = power2(pars.mH);
   const Real mC2   = power2(pars.mC);
   const Real mA2   = power2(pars.mA);
   const Real mu    = pars.mu;
   const Real mu2   = power2(pars.mu);
   const Real tb    = pars.tb;
   const Real sb    = tb / std::sqrt(1. + power2(tb));
   const Real cb    = 1. / std::sqrt(1. + power2(tb));
   const Real Q2    = power2(pars.Q);
   const Real snt   = calc_sin_theta(mt, xt, mst12, mst22);
   const Real snb   = calc_sin_theta(mb, xb, msb12, msb22);
   const Real alpha = calc_alpha(mh2, mH2, tb);
   const Real sa    = std::sin(alpha);
   const Real ca    = std::cos(alpha);

   const Real result =
   ((-6 + mA2*(1/msb12 + 1/msb22))*log(mA2/Q2))/24. + ((-6 + mH2*(1/msb12 + 1/
     msb22))*log(mH2/Q2))/24. - (mt2*(mC2 - mw2)*log(mt2/Q2)*power2(cb))/(3.*(
     mC2 - mt2)*(mt2 - mw2)) - ((mst12/(mst12 - mu2) - mu2/mst12 + mu2/mst22 +
     mst22/(-mst22 + mu2))*log(mu2/Q2)*power2(snt))/6. - (mu2*log(mA2/Q2)*log(
     msb12/Q2)*power2(cb)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))*(3*mA2*
     msb22 + msb12*msb22 + msb24 + DeltaInv(msb12,mA2,msb22)*(5*(msb12*msb22 +
     msb24)*power2(mA2) - 3*msb22*power3(mA2) + (msb12 - msb22)*power3(msb22) -
     mA2*(-10*msb12*msb24 + power3(msb22)))))/(2.*msb12) - (mu2*log(mA2/Q2)*
     log(msb12/Q2)*power2(mA2)*power2(sb)*power2(DeltaInv(msb12,mA2,msb22))*(3*
     mA2*msb22 + msb12*msb22 + msb24 + DeltaInv(msb12,mA2,msb22)*(5*(msb12*
     msb22 + msb24)*power2(mA2) - 3*msb22*power3(mA2) + (msb12 - msb22)*power3(
     msb22) - mA2*(-10*msb12*msb24 + power3(msb22)))))/(2.*msb12) - (mu2*log(
     mA2/Q2)*log(msb12/Q2)*power2(mA2)*power2(sb)*power2(DeltaInv(msb12,mA2,
     msb22))*(3*mA2*msb22 + msb12*msb22 + msb24 + DeltaInv(msb12,mA2,msb22)*(5*
     (msb12*msb22 + msb24)*power2(mA2) - 3*msb22*power3(mA2) + (msb12 - msb22)*
     power3(msb22) - mA2*(-10*msb12*msb24 + power3(msb22)))))/(2.*msb12*power2(
     cb)) - (mu2*log(mh2/Q2)*log(msb12/Q2)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22))*(3*mh2*msb22 + msb12*msb22 + msb24 + DeltaInv(mh2,msb12,
     msb22)*(5*(msb12*msb22 + msb24)*power2(mh2) - 3*msb22*power3(mh2) + (msb12
     - msb22)*power3(msb22) - mh2*(-10*msb12*msb24 + power3(msb22)))))/(2.*
     msb12) - (ca*mu2*sa*sb*log(mh2/Q2)*log(msb12/Q2)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22))*(3*mh2*msb22 + msb12*msb22 + msb24 + DeltaInv(
     mh2,msb12,msb22)*(5*(msb12*msb22 + msb24)*power2(mh2) - 3*msb22*power3(
     mh2) + (msb12 - msb22)*power3(msb22) - mh2*(-10*msb12*msb24 + power3(
     msb22)))))/(cb*msb12) + (mu2*log(mh2/Q2)*log(msb12/Q2)*power2(mh2)*power2(
     sa)*power2(DeltaInv(mh2,msb12,msb22))*(3*mh2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(mh2,msb12,msb22)*(5*(msb12*msb22 + msb24)*power2(mh2) - 3*msb22*
     power3(mh2) + (msb12 - msb22)*power3(msb22) - mh2*(-10*msb12*msb24 +
     power3(msb22)))))/(2.*msb12) - (mu2*log(mh2/Q2)*log(msb12/Q2)*power2(mh2)*
     power2(sa)*power2(sb)*power2(DeltaInv(mh2,msb12,msb22))*(3*mh2*msb22 +
     msb12*msb22 + msb24 + DeltaInv(mh2,msb12,msb22)*(5*(msb12*msb22 + msb24)*
     power2(mh2) - 3*msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) - mh2*(-
     10*msb12*msb24 + power3(msb22)))))/(2.*msb12*power2(cb)) + (ca*mu2*sa*sb*
     log(mH2/Q2)*log(msb12/Q2)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(
     3*mH2*msb22 + msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*(5*(msb12*
     msb22 + msb24)*power2(mH2) - 3*msb22*power3(mH2) + (msb12 - msb22)*power3(
     msb22) - mH2*(-10*msb12*msb24 + power3(msb22)))))/(cb*msb12) - (mu2*log(
     mH2/Q2)*log(msb12/Q2)*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,msb12,
     msb22))*(3*mH2*msb22 + msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*(5*
     (msb12*msb22 + msb24)*power2(mH2) - 3*msb22*power3(mH2) + (msb12 - msb22)*
     power3(msb22) - mH2*(-10*msb12*msb24 + power3(msb22)))))/(2.*msb12) - (
     mu2*log(mH2/Q2)*log(msb12/Q2)*power2(mH2)*power2(sb)*power2(DeltaInv(mH2,
     msb12,msb22))*(3*mH2*msb22 + msb12*msb22 + msb24 + DeltaInv(mH2,msb12,
     msb22)*(5*(msb12*msb22 + msb24)*power2(mH2) - 3*msb22*power3(mH2) + (msb12
     - msb22)*power3(msb22) - mH2*(-10*msb12*msb24 + power3(msb22)))))/(2.*
     msb12*power2(cb)) + (mu2*log(mH2/Q2)*log(msb12/Q2)*power2(mH2)*power2(sa)*
     power2(sb)*power2(DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22 + msb12*msb22 +
     msb24 + DeltaInv(mH2,msb12,msb22)*(5*(msb12*msb22 + msb24)*power2(mH2) -
     3*msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) - mH2*(-10*msb12*msb24
     + power3(msb22)))))/(2.*msb12*power2(cb)) + (2*mu2*log(msb12/Q2)*log(
     msb22/Q2)*power2(mh2)*power2(snb)*power2(DeltaInv(mh2,msb12,msb22))*(mh2*
     msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mh2,msb12,msb22)*((msb12*msb22 -
     msb24)*power2(mh2) - msb22*power3(mh2) + (5*msb12 - 3*msb22)*power3(msb22)
     + 5*mh2*(2*msb12*msb24 + power3(msb22)))))/msb12 + (2*mu2*log(msb12/Q2)*
     log(msb22/Q2)*power2(mH2)*power2(sb)*power2(snb)*power2(DeltaInv(mH2,
     msb12,msb22))*(mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,
     msb22)*((msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (5*msb12 -
     3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))))/(msb12*
     power2(cb)) + (mu2*log(mA2/Q2)*log(msb22/Q2)*power2(cb)*power2(mA2)*
     power2(DeltaInv(msb12,mA2,msb22))*(mA2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(msb12,mA2,msb22)*((msb12*msb22 + msb24)*power2(mA2) - msb22*
     power3(mA2) + (msb12 - msb22)*power3(msb22) + mA2*(6*msb12*msb24 + power3(
     msb22)))))/(2.*msb12) + (mu2*log(mA2/Q2)*log(msb22/Q2)*power2(mA2)*power2(
     sb)*power2(DeltaInv(msb12,mA2,msb22))*(mA2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(msb12,mA2,msb22)*((msb12*msb22 + msb24)*power2(mA2) - msb22*
     power3(mA2) + (msb12 - msb22)*power3(msb22) + mA2*(6*msb12*msb24 + power3(
     msb22)))))/(2.*msb12) + (mu2*log(mA2/Q2)*log(msb22/Q2)*power2(mA2)*power2(
     sb)*power2(DeltaInv(msb12,mA2,msb22))*(mA2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(msb12,mA2,msb22)*((msb12*msb22 + msb24)*power2(mA2) - msb22*
     power3(mA2) + (msb12 - msb22)*power3(msb22) + mA2*(6*msb12*msb24 + power3(
     msb22)))))/(2.*msb12*power2(cb)) + (mu2*log(mh2/Q2)*log(msb22/Q2)*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22))*(mh2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(mh2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mh2) - msb22*
     power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(
     msb22)))))/(2.*msb12) + (ca*mu2*sa*sb*log(mh2/Q2)*log(msb22/Q2)*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22))*(mh2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(mh2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mh2) - msb22*
     power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(
     msb22)))))/(cb*msb12) - (mu2*log(mh2/Q2)*log(msb22/Q2)*power2(mh2)*power2(
     sa)*power2(DeltaInv(mh2,msb12,msb22))*(mh2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(mh2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mh2) - msb22*
     power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(
     msb22)))))/(2.*msb12) + (mu2*log(mh2/Q2)*log(msb22/Q2)*power2(mh2)*power2(
     sa)*power2(sb)*power2(DeltaInv(mh2,msb12,msb22))*(mh2*msb22 + msb12*msb22
     + msb24 + DeltaInv(mh2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mh2) -
     msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 +
     power3(msb22)))))/(2.*msb12*power2(cb)) - (ca*mu2*sa*sb*log(mH2/Q2)*log(
     msb22/Q2)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 +
     msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 + msb24)*
     power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*
     msb12*msb24 + power3(msb22)))))/(cb*msb12) + (mu2*log(mH2/Q2)*log(msb22/
     Q2)*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 +
     msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 + msb24)*
     power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*
     msb12*msb24 + power3(msb22)))))/(2.*msb12) + (mu2*log(mH2/Q2)*log(msb22/
     Q2)*power2(mH2)*power2(sb)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 +
     msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 + msb24)*
     power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*
     msb12*msb24 + power3(msb22)))))/(2.*msb12*power2(cb)) - (mu2*log(mH2/Q2)*
     log(msb22/Q2)*power2(mH2)*power2(sa)*power2(sb)*power2(DeltaInv(mH2,msb12,
     msb22))*(mH2*msb22 + msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*((
     msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*
     power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))))/(2.*msb12*power2(
     cb)) - (mu2*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(sb)*power2(snb)*
     power2(DeltaInv(msb12,mst12,mC2))*(3*mC2*mst12 + msb12*mst12 + mst14 +
     DeltaInv(msb12,mst12,mC2)*(5*(msb12*mst12 + mst14)*power2(mC2) - 3*mst12*
     power3(mC2) + (msb12 - mst12)*power3(mst12) - mC2*(-10*msb12*mst14 +
     power3(mst12)))))/msb12 - (mu2*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*
     power2(sb)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(3*mC2*mst12 +
     msb12*mst12 + mst14 + DeltaInv(msb12,mst12,mC2)*(5*(msb12*mst12 + mst14)*
     power2(mC2) - 3*mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) - mC2*(-
     10*msb12*mst14 + power3(mst12)))))/(msb12*power2(cb)) - (2*mt2*mu2*log(
     msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(
     DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 + 3*mst14 + DeltaInv(
     msb12,mst12,mC2)*((msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) +
     (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))
     ))/(msb12*(mst12 - mst22)) - (mu2*log(msb12/Q2)*log(mst12/Q2)*power2(mC2)*
     power2(sb)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mC2*mst12 +
     msb12*mst12 + 3*mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12 - mst14)*
     power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*
     mC2*(2*msb12*mst14 + power3(mst12)))))/(msb12*power2(cb)) + (mu2*log(
     msb12/Q2)*log(mst12/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*
     power2(DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 + 3*mst14 +
     DeltaInv(msb12,mst12,mC2)*((msb12*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*mst14 +
     power3(mst12)))))/(msb12*power2(cb)) - (mu2*log(mC2/Q2)*log(msb22/Q2)*
     power2(mC2)*power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*
     power2(mC2) - 3*mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-
     10*msb22*mst14 + power3(mst12)))))/msb22 - (mu2*log(mC2/Q2)*log(msb22/Q2)*
     power2(mC2)*power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*
     power2(mC2) - 3*mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-
     10*msb22*mst14 + power3(mst12)))))/(msb22*power2(cb)) + (mu2*log(mC2/Q2)*
     log(msb22/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(DeltaInv(msb22,
     mst12,mC2))*(3*mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)
     *(5*(msb22*mst12 + mst14)*power2(mC2) - 3*mst12*power3(mC2) + (msb22 -
     mst12)*power3(mst12) - mC2*(-10*msb22*mst14 + power3(mst12)))))/msb22 + (
     mu2*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(
     DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(
     msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*power2(mC2) - 3*mst12*power3(
     mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-10*msb22*mst14 + power3(
     mst12)))))/(msb22*power2(cb)) + (2*mt2*mu2*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*power2(log(mst12/Q2))*(mst14 - DeltaInv(msb22,mst12,mC2)
     *(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)) + (2*mt2*mu2*power2(cb)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*(mst14
     - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(msb22*(mst12 -
     mst22)) + (mu2*power2(mC2)*power2(sb)*power2(snt)*power2(DeltaInv(msb22,
     mst12,mC2))*power2(log(mst12/Q2))*(mst14 - DeltaInv(msb22,mst12,mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))))/(msb22*power2(cb)) + (2*mt2*mu2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*(-mst14 + DeltaInv(msb22,
     mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)) + (4*mt2*mu2*
     power2(cb)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mst12/
     Q2))*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(msb22*(
     mst12 - mst22)) + (mu2*power2(mC2)*power2(sb)*power2(DeltaInv(msb22,mst12,
     mC2))*power2(log(mst12/Q2))*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))))/(msb22*power2(cb)) + (mt2*mu2*DeltaInv(msb22,mst12,mC2)*
     log(mst12/Q2)*power2(cb)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*
     DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14
     + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))))/(6.*msb22*(mst12 - mst22)) + (mu2*
     DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)*power2(sb)*(5*mC2*mst12 - msb22*
     mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(12.*msb22*
     power2(cb)) - (mu2*DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)*power2(sb)*
     power2(snt)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,
     mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))))/(12.*msb22*power2(cb)) - (2*mt2*mu2*log(
     msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)
     *((msb22*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*
     mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))))/(msb22*(
     mst12 - mst22)) - (mu2*log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(sb)*
     power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 +
     power3(mst12)))))/(msb22*power2(cb)) + (2*mt2*mu2*log(msb22/Q2)*log(mst12/
     Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*
     mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*
     power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))))/(msb22*(mst12 -
     mst22)) + (mu2*log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(sb)*power2(
     snb)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14
     + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 +
     power3(mst12)))))/(msb22*power2(cb)) + (mu2*log(msb22/Q2)*log(mst12/Q2)*
     power2(mC2)*power2(sb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*
     mst12 + msb22*mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 -
     mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12)
     + 5*mC2*(2*msb22*mst14 + power3(mst12)))))/(msb22*power2(cb)) - (mu2*log(
     msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*
     power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 +
     power3(mst12)))))/(msb22*power2(cb)) - ((mst12 - mst22 - 2*mt2)*mu2*log(
     mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(msb22,mst12,mC2)*(-((
     msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb22 + mst12)*
     power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))))/(msb22*(mst12 -
     mst22)) + ((mt2 + mu2)*log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*
     power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12
     - mst14 + DeltaInv(msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2))
     + mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14
     + power3(mst12)))))/msb22 + (2*mt2*mu2*log(mC2/Q2)*log(mst12/Q2)*power2(
     cb)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12
     + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) -
     mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 +
     power3(mst12)))))/(msb22*(mst12 - mst22)) + (mu2*log(mC2/Q2)*log(mst12/Q2)
     *power2(mC2)*power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*
     msb22*mst14 + power3(mst12)))))/msb22 + (mu2*log(mC2/Q2)*log(mst12/Q2)*
     power2(mC2)*power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*
     msb22*mst14 + power3(mst12)))))/(msb22*power2(cb)) + (mt2*log(mC2/Q2)*log(
     mst12/Q2)*power2(mC2)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*
     mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 +
     mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))))/msb22 - (mu2*log(mC2/Q2)*log(mst12/
     Q2)*power2(mC2)*power2(sb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))))/msb22 - (mu2*log(mC2/Q2)*log(mst12/
     Q2)*power2(mC2)*power2(sb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))))/(msb22*power2(cb)) - (mt2*log(mC2/
     Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2)
     )*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(
     msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))))/msb12 + (mt2*(
     mst12 - mst22 - 2*mu2)*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(-(mC2*mst22) - msb12*mst22
     - 3*mst24 + DeltaInv(mst22,msb12,mC2)*((-(msb12*mst22) + mst24)*power2(
     mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*
     msb12*mst24 + power3(mst22)))))/(msb12*(-mst12 + mst22)) - (mt2*log(msb12/
     Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2)
     )*(mC2*mst22 + msb12*mst22 + 3*mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*
     mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*
     power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))))/msb12 + (2*mt2*
     mu2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(
     DeltaInv(mst22,msb12,mC2))*(mC2*mst22 + msb12*mst22 + 3*mst24 + DeltaInv(
     mst22,msb12,mC2)*((msb12*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) +
     (5*msb12 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))
     ))/(msb12*(mst12 - mst22)) - (mu2*log(msb12/Q2)*log(mst22/Q2)*power2(mC2)*
     power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*
     mst22 + msb12*mst22 + 3*mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 -
     mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22)
     + 5*mC2*(2*msb12*mst24 + power3(mst22)))))/(msb12*power2(cb)) - (mt2*log(
     mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(3*
     mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))))/msb22 + (mt2*log(mC2/Q2)
     *log(msb22/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(
     3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))))/msb22 + (2*mt2*mu2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power2(log(mst22/Q2))*(mst24
     - DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(msb22*(mst12 -
     mst22)) + (4*mt2*mu2*power2(cb)*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*power2(log(mst22/Q2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))))/(msb22*(mst12 - mst22)) + (mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*power2(log(mst22/Q2))*(-mst24 + DeltaInv(mst22,
     msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))))/msb22 + (2*mt2*mu2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*power2(log(mst22/Q2))*(-mst24 + DeltaInv(mst22,
     msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)) - (mt2*(mst12 -
     mst22 - 2*mu2)*power2(cb)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*
     power2(log(mst22/Q2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))))/(msb22*(mst12 - mst22)) + (2*mt2*mu2*power2(cb)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*power2(log(mst22/Q2))*(-mst24 +
     DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(
     mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)) +
     (mu2*power2(mC2)*power2(sb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*
     power2(log(mst22/Q2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))))/(msb22*power2(cb)) + (mt2*DeltaInv(mst22,msb22,mC2)*log(mst22/
     Q2)*(5*mC2*mst22 - msb22*mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*(
     -3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) +
     power3(mst22)) - 72*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))))/(12.*msb22) - (mt2*mu2*DeltaInv(mst22,msb22,mC2)*log(
     mst22/Q2)*power2(cb)*(5*mC2*mst22 - msb22*mst22 + mst24 - 6*mC2*DeltaInv(
     mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) - 72*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))))/(6.*msb22*(mst12 - mst22)) + (mu2*
     DeltaInv(mst22,msb22,mC2)*log(mst22/Q2)*power2(sb)*power2(snt)*(5*mC2*
     mst22 - msb22*mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*
     msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(
     mst22)) - 72*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))))/(12.*msb22*power2(cb)) - (mt2*(mst12 - mst22 - 2*mu2)*DeltaInv(
     mst22,msb22,mC2)*log(mst22/Q2)*power2(cb)*(-5*mC2*mst22 + msb22*mst22 -
     mst24 + 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24
     - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(12.*msb22*(-mst12
     + mst22)) + (mt2*(mst12 - mst22 - 2*mu2)*log(msb22/Q2)*log(mst22/Q2)*
     power2(cb)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) -
     msb22*mst22 - 3*mst24 + DeltaInv(mst22,msb22,mC2)*((-(msb22*mst22) +
     mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(
     mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))))/(msb22*(-mst12 + mst22))
     + (mt2*(mst12 - mst22 - 2*mu2)*log(msb22/Q2)*log(mst22/Q2)*power2(cb)*
     power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) -
     msb22*mst22 - 3*mst24 + DeltaInv(mst22,msb22,mC2)*((-(msb22*mst22) +
     mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(
     mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22))
     - (mt2*log(msb22/Q2)*log(mst22/Q2)*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2))*(mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)
     *((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*
     mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))))/msb22 + (
     2*mt2*mu2*log(msb22/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(
     mst22,msb22,mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) +
     (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))
     ))/(msb22*(mst12 - mst22)) + (mt2*log(msb22/Q2)*log(mst22/Q2)*power2(mC2)*
     power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 +
     3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 - mst24)*power2(mC2) -
     mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*
     mst24 + power3(mst22)))))/msb22 - (2*mt2*mu2*log(msb22/Q2)*log(mst22/Q2)*
     power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*
     mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 -
     mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22)
     + 5*mC2*(2*msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)) - (mu2*
     log(msb22/Q2)*log(mst22/Q2)*power2(mC2)*power2(sb)*power2(snt)*power2(
     DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(
     mst22,msb22,mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) +
     (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))
     ))/(msb22*power2(cb)) + (mu2*log(msb22/Q2)*log(mst22/Q2)*power2(mC2)*
     power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*
     mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 -
     mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22)
     + 5*mC2*(2*msb22*mst24 + power3(mst22)))))/(msb22*power2(cb)) + (mt2*(
     mst12 - mst22 - 2*mu2)*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 +
     DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*
     power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))))/(msb22*(mst12 - mst22)) - ((mt2 + mu2)*log(mC2/Q2)*log(
     mst22/Q2)*power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,
     mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 + DeltaInv(mst22,msb22,mC2)*(-((
     msb22*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb22 + mst22)*
     power3(mst22) - mC2*(6*msb22*mst24 + power3(mst22)))))/msb22 + (mt2*log(
     mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*
     mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 +
     mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) +
     mC2*(6*msb22*mst24 + power3(mst22)))))/msb22 - (2*mt2*mu2*log(mC2/Q2)*log(
     mst22/Q2)*power2(cb)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*
     mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 +
     mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) +
     mC2*(6*msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)) - (mt2*log(
     mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,
     mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((
     msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*
     power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))))/msb22 + (mu2*log(
     mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(sb)*power2(snt)*power2(DeltaInv(
     mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,
     mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 -
     mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))))/msb22 + (
     mu2*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(sb)*power2(snt)*power2(
     DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(
     mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) +
     (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))))/(
     msb22*power2(cb)) + (ca*mu2*sa*sb*log(msb22/Q2)*((5*mh2*msb22 - msb12*
     msb22 + msb24)*DeltaInv(mh2,msb12,msb22) - 6*mh2*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*
     power2(mh2) + power3(msb22)) + DeltaInv(mH2,msb12,msb22)*(-5*mH2*msb22 +
     msb12*msb22 - msb24 + 6*mH2*DeltaInv(mH2,msb12,msb22)*(-3*mH2*msb12*msb22
     - 14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) + power3(msb22)) + 72*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(msb24*power2(mH2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))) - 72*
     power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*
     (msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/(12.*
     cb*msb12) + (mu2*log(msb22/Q2)*power2(sa)*power2(sb)*((5*mh2*msb22 -
     msb12*msb22 + msb24)*DeltaInv(mh2,msb12,msb22) - 6*mh2*power2(DeltaInv(
     mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 +
     msb22*power2(mh2) + power3(msb22)) + DeltaInv(mH2,msb12,msb22)*(-5*mH2*
     msb22 + msb12*msb22 - msb24 + 6*mH2*DeltaInv(mH2,msb12,msb22)*(-3*mH2*
     msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) + power3(
     msb22)) + 72*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(msb24*power2(
     mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(
     msb22)))) - 72*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22))))/(24.*msb12*power2(cb)) + (mu2*power2(sa)*power2(log(msb22/Q2))*(
     msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(-msb24 + DeltaInv(mH2,msb12,msb22)*(msb24*
     power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 +
     power3(msb22)))) - power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22))))/(2.*msb12) + (ca*mu2*sa*sb*power2(log(msb22/Q2))*(-(msb24*
     power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(msb24 - DeltaInv(mH2,msb12,msb22)*(msb24*
     power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 +
     power3(msb22)))) + power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22))))/(cb*msb12) + (mu2*power2(sa)*power2(sb)*power2(log(msb22/
     Q2))*(-(msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)
     *power2(DeltaInv(mH2,msb12,msb22))*(msb24 - DeltaInv(mH2,msb12,msb22)*(
     msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24
     + power3(msb22)))) + power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22))))/(2.*msb12*power2(cb)) + (mu2*log(msb22/Q2)*power2(sa)*(-((
     5*mh2*msb22 - msb12*msb22 + msb24)*DeltaInv(mh2,msb12,msb22)) + 6*mh2*
     power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 -
     msb12*msb24 + msb22*power2(mh2) + power3(msb22)) + DeltaInv(mH2,msb12,
     msb22)*(5*mH2*msb22 - msb12*msb22 + msb24 - 6*mH2*DeltaInv(mH2,msb12,
     msb22)*(-3*mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(
     mH2) + power3(msb22)) - 72*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(
     msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24
     + power3(msb22)))) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 +
     msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(24.*msb12) + (ca*mu2*sa*sb*log(msb12/Q2)*
     log(msb22/Q2)*(-((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)
     )*(mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*
     msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)*
     power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(mh2)*((-(
     msb12*msb22) + msb24)*power2(mh2) + msb22*power3(mh2) + (-5*msb12 + 3*
     msb22)*power3(msb22) - 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(cb*msb12) + (mu2*log(msb12/Q2)*log(msb22/Q2)
     *power2(sa)*power2(sb)*(-((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*(mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,
     msb22)*((msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (5*msb12 -
     3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(
     mh2)*((-(msb12*msb22) + msb24)*power2(mh2) + msb22*power3(mh2) + (-5*msb12
     + 3*msb22)*power3(msb22) - 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(2.*msb12*power2(cb)) + (2*mu2*log(msb12/Q2)*
     log(msb22/Q2)*power2(sa)*power2(snb)*(-((mh2*msb22 + msb12*msb22 + 3*
     msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*msb22 + 3*msb24 +
     DeltaInv(mH2,msb12,msb22)*((msb12*msb22 - msb24)*power2(mH2) - msb22*
     power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 +
     power3(msb22)))) + power2(mh2)*((-(msb12*msb22) + msb24)*power2(mh2) +
     msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - 5*mh2*(2*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/msb12 + (mu2*
     log(msb12/Q2)*log(msb22/Q2)*power2(sa)*((mh2*msb22 + msb12*msb22 + 3*
     msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(-(mH2*msb22) - msb12*msb22 - 3*msb24 +
     DeltaInv(mH2,msb12,msb22)*((-(msb12*msb22) + msb24)*power2(mH2) + msb22*
     power3(mH2) + (-5*msb12 + 3*msb22)*power3(msb22) - 5*mH2*(2*msb12*msb24 +
     power3(msb22)))) + power2(mh2)*((msb12*msb22 - msb24)*power2(mh2) - msb22*
     power3(mh2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mh2*(2*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/(2.*msb12) + (4*ca*
     mu2*sa*sb*log(msb12/Q2)*log(msb22/Q2)*power2(snb)*((mh2*msb22 + msb12*
     msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-(mH2*msb22) - msb12*msb22 - 3*
     msb24 + DeltaInv(mH2,msb12,msb22)*((-(msb12*msb22) + msb24)*power2(mH2) +
     msb22*power3(mH2) + (-5*msb12 + 3*msb22)*power3(msb22) - 5*mH2*(2*msb12*
     msb24 + power3(msb22)))) + power2(mh2)*((msb12*msb22 - msb24)*power2(mh2)
     - msb22*power3(mh2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mh2*(2*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/(cb*msb12) + (
     2*mu2*log(msb12/Q2)*log(msb22/Q2)*power2(sa)*power2(sb)*power2(snb)*((mh2*
     msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-(mH2*msb22) -
     msb12*msb22 - 3*msb24 + DeltaInv(mH2,msb12,msb22)*((-(msb12*msb22) +
     msb24)*power2(mH2) + msb22*power3(mH2) + (-5*msb12 + 3*msb22)*power3(
     msb22) - 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(mh2)*((msb12*
     msb22 - msb24)*power2(mh2) - msb22*power3(mh2) + (5*msb12 - 3*msb22)*
     power3(msb22) + 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     mh2,msb12,msb22))))/(msb12*power2(cb)) + (ca*mu2*sa*sb*power2(log(msb12/
     Q2))*(-((2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(
     mh2,msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*
     msb22 + msb12*msb22 + 2*msb24 + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22
     + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(
     msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))) + power2(mh2)*(-((3*
     msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*
     msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(cb*msb12) + (mu2*power2(sa)*power2(sb)*
     power2(log(msb12/Q2))*(-((2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)
     *power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*(2*mH2*msb22 + msb12*msb22 + 2*msb24 + DeltaInv(mH2,msb12,
     msb22)*((3*msb12*msb22 + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*
     msb12 - 2*msb22)*power3(msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))) +
     power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(
     mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(
     msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/(2.*msb12*power2(cb)) + (mu2*
     power2(sa)*power2(snb)*power2(log(msb12/Q2))*(-(mh2*msb12*msb14*power2(
     DeltaInv(mh2,msb12,msb12))) - 2*(2*mh2*msb22 + msb12*msb22 + 2*msb24)*
     power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mH2*(msb12*msb14*power2(
     DeltaInv(mH2,msb12,msb12)) + 2*mH2*power2(DeltaInv(mH2,msb12,msb22))*(2*
     mH2*msb22 + msb12*msb22 + 2*msb24 + DeltaInv(mH2,msb12,msb22)*((3*msb12*
     msb22 + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*
     power3(msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22))))) + 2*power2(mh2)*(
     -((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12
     + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/msb12 + (mu2*power2(sa)*power2(log(msb12/Q2))
     *((2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-2*mH2*
     msb22 - msb12*msb22 - 2*msb24 + DeltaInv(mH2,msb12,msb22)*(-((3*msb12*
     msb22 + 2*msb24)*power2(mH2)) + 2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)
     *power3(msb22) - 2*mH2*(5*msb12*msb24 + power3(msb22)))) + power2(mh2)*((
     3*msb12*msb22 + 2*msb24)*power2(mh2) - 2*msb22*power3(mh2) + (3*msb12 - 2*
     msb22)*power3(msb22) + 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(2.*msb12) + (2*ca*mu2*sa*sb*power2(snb)*
     power2(log(msb12/Q2))*(mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,msb12)) +
     2*(2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + mH2*(-(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12))) + 2*
     mH2*power2(DeltaInv(mH2,msb12,msb22))*(-2*mH2*msb22 - msb12*msb22 - 2*
     msb24 + DeltaInv(mH2,msb12,msb22)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)
     ) + 2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*
     msb12*msb24 + power3(msb22))))) + 2*power2(mh2)*((3*msb12*msb22 + 2*msb24)
     *power2(mh2) - 2*msb22*power3(mh2) + (3*msb12 - 2*msb22)*power3(msb22) +
     2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))
     /(cb*msb12) + (mu2*power2(sa)*power2(sb)*power2(snb)*power2(log(msb12/Q2))
     *(mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,msb12)) + 2*(2*mh2*msb22 +
     msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) +
     mH2*(-(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12))) + 2*mH2*power2(
     DeltaInv(mH2,msb12,msb22))*(-2*mH2*msb22 - msb12*msb22 - 2*msb24 +
     DeltaInv(mH2,msb12,msb22)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*
     msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22))))) + 2*power2(mh2)*((3*msb12*msb22 + 2*msb24)*
     power2(mh2) - 2*msb22*power3(mh2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*
     mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/(
     msb12*power2(cb)) + (mu2*power2(sb)*power2(snb)*power2(log(msb22/Q2))*(2*
     msb22*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + msb12*(mH2*
     msb22*msb24*power2(DeltaInv(mH2,msb22,msb22)) + power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*(2*mC2*mst12 + msb22*mst12 + 2*mst14 +
     DeltaInv(msb22,mst12,mC2)*((3*msb22*mst12 + 2*mst14)*power2(mC2) - 2*
     mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb22*
     mst14 + power3(mst12))))) - 2*msb22*power2(mH2)*(msb24*power2(mH2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22))))/(msb12*msb22*power2(cb)) - (mu2*log(
     msb12/Q2)*log(msb22/Q2)*power2(sb)*((mH2*msb22 + msb12*msb22 + 3*msb24)*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mA2)*power2(
     DeltaInv(msb12,mA2,msb22))*(mA2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(
     msb12,mA2,msb22)*((msb12*msb22 - msb24)*power2(mA2) - msb22*power3(mA2) +
     (5*msb12 - 3*msb22)*power3(msb22) + 5*mA2*(2*msb12*msb24 + power3(msb22)))
     ) + power2(mH2)*((msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (
     5*msb12 - 3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22))))/(2.*msb12*power2(cb)) + (mu2*log(
     msb12/Q2)*power2(sa)*(-mh2 + mH2 + DeltaInv(mh2,msb12,msb22)*(msb22*(-(
     msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(
     msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*msb22) +
     msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*power2(msb22) + power3(mH2)
     ) - 6*mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(
     6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) +
     power3(msb22)) + 6*mH2*msb22*power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*
     msb24) + mH2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mH2) +
     power3(mH2) + power3(msb22)) + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*
     msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(
     msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22)) - 72*msb22*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) +
     2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))/(24.*msb12*
     msb22) + (ca*mu2*sa*sb*log(msb12/Q2)*(mh2 - mH2 - DeltaInv(mh2,msb12,
     msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) +
     5*mh2*power2(msb22) + power3(mh2)) + DeltaInv(mH2,msb12,msb22)*(msb22*(-(
     msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*power2(
     msb22) + power3(mH2)) + 6*mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(
     msb12*msb24) + mh2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(
     mh2) + power3(mh2) + power3(msb22)) - 6*mH2*msb22*power2(DeltaInv(mH2,
     msb12,msb22))*(-(msb12*msb24) + mH2*(6*msb12*msb22 + 23*msb24) - (msb12 -
     23*msb22)*power2(mH2) + power3(mH2) + power3(msb22)) - 72*msb22*power2(
     mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-
     3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + 72*msb22*power2(mH2)*(-((3*msb12*msb22
     + 2*msb24)*power2(mH2)) + 2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)*
     power3(msb22) - 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     mH2,msb12,msb22))))/(12.*cb*msb12*msb22) + (mu2*log(msb12/Q2)*power2(sa)*
     power2(sb)*(mh2 - mH2 - DeltaInv(mh2,msb12,msb22)*(msb22*(-(msb12*msb22) +
     msb24) - (msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(msb22) + power3(mh2)
     ) + DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 -
     5*msb22)*power2(mH2) + 5*mH2*power2(msb22) + power3(mH2)) + 6*mh2*msb22*
     power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*msb12*msb22 +
     23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) + power3(msb22))
     - 6*mH2*msb22*power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*msb24) + mH2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mH2) + power3(mH2) +
     power3(msb22)) - 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 72*
     msb22*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*msb22*
     power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))/(24.*msb12*msb22*
     power2(cb)) - (mu2*power2(sb)*power2(log(msb12/Q2))*((2*mH2*msb22 + msb12*
     msb22 + 2*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(
     mA2)*power2(DeltaInv(msb12,mA2,msb22))*(2*mA2*msb22 + msb12*msb22 + 2*
     msb24 + DeltaInv(msb12,mA2,msb22)*((3*msb12*msb22 + 2*msb24)*power2(mA2) -
     2*msb22*power3(mA2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*mA2*(5*msb12*
     msb24 + power3(msb22)))) + power2(mH2)*((3*msb12*msb22 + 2*msb24)*power2(
     mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*mH2*(5*
     msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))/(2.*
     msb12*power2(cb)) + (mu2*power2(sb)*power2(snb)*power2(log(msb12/Q2))*(
     mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*(2*mH2*msb22 +
     msb12*msb22 + 2*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))*(-2*mC2*mst12 - msb12*mst12
     - 2*mst14 + DeltaInv(msb12,mst12,mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(
     mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(
     5*msb12*mst14 + power3(mst12)))) + 2*power2(mH2)*((3*msb12*msb22 + 2*
     msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(
     msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,
     msb22))))/(msb12*power2(cb)) + (mu2*power2(sa)*power2(snb)*power2(log(
     msb22/Q2))*(-2*msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) - mh2*
     msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) + 2*power2(mh2)*(msb24*
     power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + mH2*(2*mH2*msb24*
     power2(DeltaInv(mH2,msb12,msb22)) + msb12*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) - 2*mH2*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*
     mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)))))/
     msb12 + (2*ca*mu2*sa*sb*power2(snb)*power2(log(msb22/Q2))*(2*msb24*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(
     mh2,msb22,msb22)) - 2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22)) + mH2*(-2*mH2*msb24*power2(DeltaInv(mH2,msb12,msb22)) -
     msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 2*mH2*(msb24*power2(mH2) +
     (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)))))/(cb*msb12) + (mu2*power2(sa)*power2(
     sb)*power2(snb)*power2(log(msb22/Q2))*(2*msb24*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,
     msb22)) - 2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22)) + mH2*(-2*mH2*msb24*power2(DeltaInv(mH2,msb12,msb22)) - msb12*
     msb24*power2(DeltaInv(mH2,msb22,msb22)) + 2*mH2*(msb24*power2(mH2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)))))/(msb12*power2(cb)) - (mu2*power2(sb)*
     power2(log(msb22/Q2))*(msb22*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + msb22*msb24*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)) + 2*
     msb12*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(2*mC2*mst12 + msb22*
     mst12 + 2*mst14 + DeltaInv(msb22,mst12,mC2)*((3*msb22*mst12 + 2*mst14)*
     power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(mst12) + 2*
     mC2*(5*msb22*mst14 + power3(mst12)))) - msb22*power2(mH2)*(msb24*power2(
     mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(
     msb22)))*power3(DeltaInv(mH2,msb12,msb22)) - msb22*power2(mA2)*(msb24*
     power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22))))/(2.*msb12*msb22*
     power2(cb)) + (mu2*log(msb12/Q2)*power2(sb)*(mA2 + mH2 + 2*msb12 + 2*msb22
     - DeltaInv(msb12,mA2,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*
     msb22)*power2(mA2) + 5*mA2*power2(msb22) + power3(mA2)) - DeltaInv(mH2,
     msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(
     mH2) + 5*mH2*power2(msb22) + power3(mH2)) + 6*mA2*msb22*power2(DeltaInv(
     msb12,mA2,msb22))*(-(msb12*msb24) + mA2*(6*msb12*msb22 + 23*msb24) - (
     msb12 - 23*msb22)*power2(mA2) + power3(mA2) + power3(msb22)) + 6*mH2*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*msb24) + mH2*(6*msb12*
     msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mH2) + power3(mH2) + power3(
     msb22)) - 72*msb22*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) +
     2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) - 72*msb22*
     power2(mA2)*(-((3*msb12*msb22 + 2*msb24)*power2(mA2)) + 2*msb22*power3(
     mA2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mA2*(5*msb12*msb24 + power3(
     msb22)))*power3(DeltaInv(msb12,mA2,msb22))))/(24.*msb12*msb22*power2(cb))
     + (2*mt2*mu2*log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*power2(snb)*
     (-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)))
     + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 +
     DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24 +
     power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12 - mst22)) + (
     mt2*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*power2(snt)*(-((3*
     mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 +
     DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24 +
     power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/msb12 - ((mt2 + mu2)*log(mC2/
     Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(-((3*
     mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 +
     DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24 +
     power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/msb12 - (mu2*log(mC2/Q2)*log(
     msb12/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(-((3*mC2*mst12 +
     msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(
     mst22,msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,
     msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (
     msb12 - mst22)*power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (
     -5*(msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 +
     mst12)*power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/msb12 - (mu2*log(mC2/Q2)*log(msb12/Q2)*
     power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb12*
     mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,
     msb12,mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)
     *(5*(msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 -
     mst22)*power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(
     msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*
     power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(msb12*power2(cb)) + (log(mC2/Q2)*log(msb12/Q2)*
     power2(cb)*power2(mC2)*power2(snb)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*
     (mst12 - mst22 - 2*mt2)*mu2*power2(DeltaInv(msb12,mst12,mC2))) - mt2*(
     mst12 - mst22 - 2*mu2)*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*mst22 -
     msb12*mst22 - mst24 + DeltaInv(mst22,msb12,mC2)*(-5*(msb12*mst22 + mst24)*
     power2(mC2) + 3*mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) + mC2*(
     -10*msb12*mst24 + power3(mst22)))) + (mst12 - mst22 - 2*mt2)*mu2*(-5*(
     msb12*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*
     power3(mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(msb12*(mst12 - mst22)) + (2*mt2*mu2*power2(mC2)*
     power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*power2(DeltaInv(msb12,
     mst12,mC2)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-mst14 + DeltaInv(
     msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))) - msb22*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*
     mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(log(mst12/Q2))*(msb22*
     mst14*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*
     msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) -
     msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*
     msb22*(mst12 - mst22)) + (mu2*power2(mC2)*power2(sb)*power2(snb)*power2(
     snt)*power2(log(mst12/Q2))*(msb22*mst14*power2(DeltaInv(msb12,mst12,mC2))
     + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-mst14 + DeltaInv(msb22,mst12,
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))) - msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*
     power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(msb12*msb22*power2(cb)) + (2*mt2*mu2*power2(mC2)*
     power2(snb)*power2(log(mst12/Q2))*(-(msb22*mst14*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mst14 - DeltaInv(
     msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))) + msb22*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (4*
     mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(log(mst12/Q2))*(-(msb22*
     mst14*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mst14 - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*
     msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) +
     msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*
     msb22*(mst12 - mst22)) + (mu2*power2(mC2)*power2(sb)*power2(snb)*power2(
     log(mst12/Q2))*(-(msb22*mst14*power2(DeltaInv(msb12,mst12,mC2))) + msb12*
     power2(DeltaInv(msb22,mst12,mC2))*(mst14 - DeltaInv(msb22,mst12,mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))) + msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(
     mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22*power2(cb)) + (mt2*mu2*log(mst12/Q2)*power2(snb)*(
     msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - 6*
     mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*
     mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(
     msb22,mst12,mC2)*(-5*mC2*mst12 + msb22*mst12 - mst14 + 6*mC2*DeltaInv(
     msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) + 72*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))) - 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)) + (
     mt2*mu2*log(mst12/Q2)*power2(cb)*power2(snb)*(msb22*(5*mC2*mst12 - msb12*
     mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,mst12,mC2)*(-5*
     mC2*mst12 + msb22*mst12 - mst14 + 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*
     msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(
     mst12)) + 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))) - 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*
     power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(3.*msb12*msb22*(mst12 - mst22)) + (mu2*log(mst12/Q2)*
     power2(sb)*power2(snb)*(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*
     (-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) +
     power3(mst12)) + msb12*DeltaInv(msb22,mst12,mC2)*(-5*mC2*mst12 + msb22*
     mst12 - mst14 + 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 72*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) - 72*msb22*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(12.*
     msb12*msb22*power2(cb)) + (mt2*mu2*log(mst12/Q2)*power2(snb)*(-(msb22*(5*
     mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb22*
     power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 -
     msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,
     mst12,mC2)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,
     mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))) + 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)) + (
     mt2*mu2*log(mst12/Q2)*power2(cb)*power2(snb)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb22*power2(
     DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*
     mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,mst12,
     mC2)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*
     (-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)) + (mu2*log(
     mst12/Q2)*power2(sb)*power2(snb)*power2(snt)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb22*power2(
     DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*
     mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,mst12,
     mC2)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*
     (-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(12.*msb12*msb22*power2(cb)) + (2*mt2*mu2*
     power2(cb)*power2(mC2)*power2(snb)*power2(log(msb12/Q2))*(-((2*mC2*mst12 +
     msb12*mst12 + 2*mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(
     DeltaInv(mst22,msb12,mC2))*(2*mC2*mst22 + msb12*mst22 + 2*mst24 +
     DeltaInv(mst22,msb12,mC2)*((3*msb12*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb12*
     mst24 + power3(mst22)))) + (-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12
     - mst22)) + (mu2*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*power2(
     log(msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(DeltaInv(
     msb12,mst12,mC2)) + power2(DeltaInv(mst22,msb12,mC2))*(-2*mC2*mst22 -
     msb12*mst22 - 2*mst24 + DeltaInv(mst22,msb12,mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))) + ((3*msb12*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2))))/(msb12*power2(cb)) + (2*mt2*mu2*log(mC2/Q2)*log(mst12/Q2)*
     power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*
     power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(
     6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*msb22*(mst12 - mst22)) - ((mst12 - mst22 - 2*mt2)*mu2*log(mC2/Q2)*
     log(mst12/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 +
     msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(
     msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) +
     (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) +
     msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12
     + mst12)*power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*mt2*mu2*
     log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-(msb22*(
     mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (
     mu2*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(sb)*power2(snb)*(-(msb22*
     (mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + ((mt2 + mu2)*log(
     mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(-(
     msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)))
     + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14
     + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (2*mu2*log(mC2/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(-(msb22*(
     mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (mu2*log(mC2/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(-(msb22*(
     mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*power2(cb)) + (2*mt2*
     mu2*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*(msb22*(mC2*mst12 +
     msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(
     msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2)
     + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) +
     msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 -
     mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (4*mt2*mu2*
     log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(snb)*(msb22*(mC2*
     mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*
     power2(DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 +
     DeltaInv(msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*
     power3(mC2) + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 +
     power3(mst12)))) + msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22))
     + (2*mu2*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(sb)*power2(snb)*(
     msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))
     + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 -
     mst14 + DeltaInv(msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) +
     mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 +
     power3(mst12)))) + msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (mu2*log(mC2/
     Q2)*log(mst12/Q2)*power2(mC2)*power2(sb)*power2(snb)*(msb22*(mC2*mst12 +
     msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(
     msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2)
     + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) +
     msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 -
     mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*power2(cb)) + (mt2*log(mC2/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(snb)*power2(snt)*(msb22*(mC2*mst12 +
     msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(
     msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2)
     + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) +
     msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 -
     mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (mu2*log(mC2/Q2)*log(mst12/
     Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(msb22*(mC2*mst12 +
     msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(
     msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2)
     + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) +
     msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 -
     mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (2*mt2*mu2*log(mC2/Q2)*log(
     msb22/Q2)*power2(cb)*power2(mC2)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*
     power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*
     mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12)
     + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     ))/(msb22*(mst12 - mst22)) - (2*mt2*mu2*log(mC2/Q2)*log(msb22/Q2)*power2(
     cb)*power2(mC2)*power2(snb)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*
     mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) -
     mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(
     -10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(
     msb22*(mst12 - mst22)) + (mt2*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*
     power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*
     mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(
     mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*
     msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) +
     3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 - ((mt2
     + mu2)*log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*power2(snt)*(-((3*
     mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*msb22*mst24 +
     power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 - (mu2*log(mC2/Q2)*log(
     msb22/Q2)*power2(mC2)*power2(sb)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12
     + mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,
     mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(
     msb22*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*
     power3(mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12
     + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(
     mst12) + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2))))/msb22 - (mu2*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(
     sb)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(
     msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 +
     msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*
     power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-
     10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2)
     + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*power2(
     cb)) - (mt2*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(snb)*power2(snt)*
     (-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2)))
     + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*msb22*mst24 +
     power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 + ((mt2 + mu2)*log(mC2/
     Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(-((3*
     mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*msb22*mst24 +
     power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 + (mu2*log(mC2/Q2)*log(
     msb22/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(-((3*mC2*mst12 +
     msb22*mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(
     mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,
     msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (
     msb22 - mst22)*power3(mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (
     -5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 +
     mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2))))/msb22 + (mu2*log(mC2/Q2)*log(msb22/Q2)*
     power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb22*
     mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,
     msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)
     *(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 -
     mst22)*power3(mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(
     msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*
     power3(mst12) + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2))))/(msb22*power2(cb)) + (log(mC2/Q2)*log(msb22/Q2)*
     power2(cb)*power2(mC2)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*(mst12 -
     mst22 - 2*mt2)*mu2*power2(DeltaInv(msb22,mst12,mC2))) - mt2*(mst12 - mst22
     - 2*mu2)*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*mst22 - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-5*(msb22*mst22 + mst24)*power2(mC2) +
     3*mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) + mC2*(-10*msb22*
     mst24 + power3(mst22)))) + (mst12 - mst22 - 2*mt2)*mu2*(-5*(msb22*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12)
     + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     ))/(msb22*(mst12 - mst22)) - (log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(
     mC2)*power2(snb)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*(mst12 - mst22 -
     2*mt2)*mu2*power2(DeltaInv(msb22,mst12,mC2))) - mt2*(mst12 - mst22 - 2*
     mu2)*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*mst22 - msb22*mst22 - mst24
     + DeltaInv(mst22,msb22,mC2)*(-5*(msb22*mst22 + mst24)*power2(mC2) + 3*
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) + mC2*(-10*msb22*mst24
     + power3(mst22)))) + (mst12 - mst22 - 2*mt2)*mu2*(-5*(msb22*mst12 + mst14)
     *power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*
     (-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(
     msb22*(mst12 - mst22)) + (2*mt2*mu2*power2(cb)*power2(mC2)*power2(log(
     msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(2*mC2*mst22 + msb22*
     mst22 + 2*mst24 + DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22 + 2*mst24)*
     power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*
     mC2*(5*msb22*mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))
     /(msb22*(mst12 - mst22)) - (2*mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*
     power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(2*mC2*
     mst22 + msb22*mst22 + 2*mst24 + DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22
     + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2))))/(msb22*(mst12 - mst22)) + (mu2*power2(mC2)*power2(sb)*
     power2(snb)*power2(snt)*power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*
     mst12 + 2*mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(
     mst22,msb22,mC2))*(2*mC2*mst22 + msb22*mst22 + 2*mst24 + DeltaInv(mst22,
     msb22,mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) +
     (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))
     ) + (-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2))))/(msb22*power2(cb)) + (mu2*power2(mC2)*
     power2(sb)*power2(snt)*power2(log(msb22/Q2))*((2*mC2*mst12 + msb22*mst12 +
     2*mst14)*power2(DeltaInv(msb22,mst12,mC2)) + power2(DeltaInv(mst22,msb22,
     mC2))*(-2*mC2*mst22 - msb22*mst22 - 2*mst24 + DeltaInv(mst22,msb22,mC2)*(-
     ((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22
     + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))) + ((3*
     msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*
     mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2))))/(msb22*power2(cb)) + (mu2*log(msb22/Q2)*
     power2(sb)*(msb22*(5*mH2*msb22 - msb12*msb22 + msb24)*mst12*DeltaInv(mH2,
     msb12,msb22) + msb22*(5*mA2*msb22 - msb12*msb22 + msb24)*mst12*DeltaInv(
     msb12,mA2,msb22) - 6*mA2*msb22*mst12*power2(DeltaInv(msb12,mA2,msb22))*(-
     3*mA2*msb12*msb22 - 14*mA2*msb24 - msb12*msb24 + msb22*power2(mA2) +
     power3(msb22)) - 6*mH2*msb22*mst12*power2(DeltaInv(mH2,msb12,msb22))*(-3*
     mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) + power3(
     msb22)) - 72*msb22*mst12*power2(mH2)*(msb24*power2(mH2) + (-2*msb12 +
     msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb22*mst12*power2(mA2)*(msb24*power2(mA2)
     + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(msb12,mA2,msb22)) + 2*msb12*(mC2 + msb22 + mst12 -
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + 6*mC2*mst12*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - 72*mst12*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)))))/(24.*msb12*
     msb22*mst12*power2(cb)) + (2*mt2*mu2*power2(mC2)*power2(snb)*power2(log(
     mst22/Q2))*(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))) - msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(msb12*msb22*(mst12 - mst22)) + (mt2*(mst12 - mst22 - 2*mu2)*
     power2(cb)*power2(mC2)*power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*
     power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(DeltaInv(mst22,msb22,mC2)
     )*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) - msb22*(
     mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12
     - mst22)) + (4*mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(log(
     mst22/Q2))*(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))) - msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(msb12*msb22*(mst12 - mst22)) + (mt2*power2(mC2)*power2(snb)*
     power2(log(mst22/Q2))*(-(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)
     *(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*
     mst24 + power3(mst22)))) + msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*
     power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(msb12*msb22) + (2*mt2*mu2*power2(mC2)*power2(snb)*
     power2(log(mst22/Q2))*(-(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)
     *(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*
     mst24 + power3(mst22)))) + msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*
     power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*mt2*mu2*power2(cb)*
     power2(mC2)*power2(snb)*power2(log(mst22/Q2))*(-(msb22*mst24*power2(
     DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(
     mst24 - DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) + msb22*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 -
     mst22)) + (mu2*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*power2(log(
     mst22/Q2))*(-(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2))) + msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(
     mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))) + msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(msb12*msb22*power2(cb)) + (mt2*log(mst22/Q2)*power2(snb)*(msb22*(
     5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - 6*mC2*
     msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24
     - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*DeltaInv(mst22,
     msb22,mC2)*(-5*mC2*mst22 + msb22*mst22 - mst24 + 6*mC2*DeltaInv(mst22,
     msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) + 72*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))) - 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(12.*msb12*msb22) + (mt2*mu2*log(
     mst22/Q2)*power2(snb)*(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(
     mst22,msb12,mC2) - 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*
     msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) + msb12*DeltaInv(mst22,msb22,mC2)*(-5*mC2*mst22 + msb22*mst22 -
     mst24 + 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24
     - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) - 72*msb22*power2(
     mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*
     msb22*(mst12 - mst22)) + (mt2*mu2*log(mst22/Q2)*power2(cb)*power2(snb)*(
     msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - 6*
     mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*
     mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*DeltaInv(
     mst22,msb22,mC2)*(-5*mC2*mst22 + msb22*mst22 - mst24 + 6*mC2*DeltaInv(
     mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) + 72*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))) - 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)) + (
     mu2*log(mst22/Q2)*power2(sb)*power2(snb)*power2(snt)*(msb22*(5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - 6*mC2*msb22*power2(
     DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*
     mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*DeltaInv(mst22,msb22,
     mC2)*(-5*mC2*mst22 + msb22*mst22 - mst24 + 6*mC2*DeltaInv(mst22,msb22,mC2)
     *(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) +
     power3(mst22)) + 72*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))) - 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 +
     mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(12.*msb12*msb22*power2(cb)) + (mt2*mu2*log(
     mst22/Q2)*power2(snb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*
     DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))
     *(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) +
     power3(mst22)) + msb12*DeltaInv(mst22,msb22,mC2)*(5*mC2*mst22 - msb22*
     mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*
     mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) + 72*msb22*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*
     msb12*msb22*(mst12 - mst22)) + (mt2*(mst12 - mst22 - 2*mu2)*log(mst22/Q2)*
     power2(cb)*power2(snb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*
     DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))
     *(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) +
     power3(mst22)) + msb12*DeltaInv(mst22,msb22,mC2)*(5*mC2*mst22 - msb22*
     mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*
     mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) + 72*msb22*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(12.*
     msb12*msb22*(mst12 - mst22)) + (mt2*mu2*log(mst22/Q2)*power2(cb)*power2(
     snb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,
     mC2)) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22
     - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*
     DeltaInv(mst22,msb22,mC2)*(5*mC2*mst22 - msb22*mst22 + mst24 - 6*mC2*
     DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24
     + mst22*power2(mC2) + power3(mst22)) - 72*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))) + 72*msb22*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(3.*msb12*msb22*(mst12
     - mst22)) + (mt2*mu2*log(msb12/Q2)*power2(snb)*((mC2 + msb12)*(mst12 -
     mst22) + mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) -
     (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) -
     mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 -
     5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22) + (mt2*mu2*log(msb12/
     Q2)*power2(cb)*power2(snb)*((mC2 + msb12)*(mst12 - mst22) + mst22*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) +
     mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))
     /(6.*msb12*mst12*(mst12 - mst22)*mst22) + (mu2*log(msb12/Q2)*power2(sb)*
     power2(snb)*power2(snt)*((mC2 + msb12)*(mst12 - mst22) + mst22*DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,
     msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (
     msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*
     msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(12.*msb12*
     mst12*mst22*power2(cb)) + (mt2*mu2*log(msb12/Q2)*power2(snb)*(-((mC2 +
     msb12)*(mst12 - mst22)) - mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*
     mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) +
     power3(mC2)) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) +
     mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)
     ) + 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) +
     mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(
     mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2)
     )*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*
     power2(mC2) + power3(mC2) + power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-
     ((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12
     + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22) + (mt2*mu2*log(
     msb12/Q2)*power2(cb)*power2(snb)*(-((mC2 + msb12)*(mst12 - mst22)) -
     mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2))))/(3.*msb12*mst12*(mst12 - mst22)*mst22) + (2*mt2*mu2*log(mC2/
     Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst22 + msb12*
     mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(
     mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,
     mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 -
     mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*(-((
     msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*
     power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (mt2*(mst12 - mst22 -
     2*mu2)*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24
     + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (4*
     mt2*mu2*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(-(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24
     + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (
     mt2*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(snt)*(-(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24
     + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) - ((mt2 + mu2)*log(
     mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(-(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24
     + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (mu2*log(mC2/Q2)*
     log(mst22/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(-(msb22*(
     mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (mt2*log(mC2/Q2)*
     log(mst22/Q2)*power2(mC2)*power2(snb)*(msb22*(mC2*mst22 + msb12*mst22 +
     mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 + DeltaInv(mst22,msb22,
     mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb22 +
     mst22)*power3(mst22) - mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*((
     msb12*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) + mC2*(6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(msb12*msb22) + (2*mt2*mu2*log(mC2/Q2)*log(mst22/Q2)*
     power2(mC2)*power2(snb)*(msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(
     DeltaInv(mst22,msb12,mC2)) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(
     mC2*mst22) - msb22*mst22 - mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*
     mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb22 + mst22)*power3(
     mst22) - mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*((msb12*mst22 +
     mst24)*power2(mC2) - mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) +
     mC2*(6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(
     msb12*msb22*(mst12 - mst22)) + (2*mt2*mu2*log(mC2/Q2)*log(mst22/Q2)*
     power2(cb)*power2(mC2)*power2(snb)*(msb22*(mC2*mst22 + msb12*mst22 +
     mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 + DeltaInv(mst22,msb22,
     mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb22 +
     mst22)*power3(mst22) - mC2*(6*msb22*mst24 + power3(mst22)))) + msb22*((
     msb12*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) + mC2*(6*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*mu2*log(mC2/Q2)*
     log(mst22/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(msb22*(mC2*
     mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 +
     DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*
     power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (mu2*log(mC2/
     Q2)*log(mst22/Q2)*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*(msb22*(
     mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) +
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*power2(cb)) + (
     mt2*mu2*log(msb22/Q2)*((mC2 + msb22)*(mst12 - mst22) + mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + (mt2*mu2*log(msb22/Q2)*power2(cb)*((mC2 +
     msb22)*(mst12 - mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*
     mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) +
     power3(mC2)) - mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) +
     mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)
     ) - 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) +
     mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(
     mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2)
     )*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*
     power2(mC2) + power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-
     ((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22
     + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22) + (mt2*mu2*log(
     msb22/Q2)*power2(snb)*((mC2 + msb22)*(mst12 - mst22) + mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + (mt2*mu2*log(msb22/Q2)*power2(cb)*power2(
     snb)*((mC2 + msb22)*(mst12 - mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(
     mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(
     msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2)
     )*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(3.*msb22*mst12*(mst12
     - mst22)*mst22) + (mu2*log(msb22/Q2)*power2(sb)*power2(snt)*((mC2 + msb22)
     *(mst12 - mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12)
     + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(
     mC2)) - mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) -
     (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*
     mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(
     6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(
     msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2))))/(12.*msb22*mst12*mst22*power2(cb)) - (mu2*log(msb22/
     Q2)*power2(sb)*power2(snb)*power2(snt)*((mC2 + msb22)*(mst12 - mst22) +
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2))))/(12.*msb22*mst12*mst22*power2(cb)) + (mt2*mu2*log(msb22/Q2)*
     (-((mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(msb22,mst12,mC2)*(
     mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(
     msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2)
     )*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12
     - mst22)*mst22) + (mt2*mu2*log(msb22/Q2)*power2(cb)*(-((mC2 + msb22)*(
     mst12 - mst22)) - mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) +
     mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (
     msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*
     msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(
     msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2))))/(3.*msb22*mst12*(mst12 - mst22)*mst22) + (mt2*mu2*log(
     msb22/Q2)*power2(snb)*(-((mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + (mt2*mu2*log(msb22/Q2)*power2(cb)*power2(
     snb)*(-((mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(msb22,mst12,mC2)*(
     mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(
     msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2)
     )*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12
     - mst22)*mst22) + (2*Fin3(msb12,mt2,mu2,Q2)*power2(mu2)*power2(snb)*
     power2(DeltaInv(msb12,mt2,mu2))*(msb14 + msb12*(mt2 + 3*mu2) + DeltaInv(
     msb12,mt2,mu2)*(5*msb14*mu2*(2*mt2 + mu2) + msb12*(5*mt2 - 3*mu2)*power2(
     mu2) + (mt2 - mu2)*power3(msb12) - power4(msb12))))/mt2 - (DeltaInv(msb12,
     mt2,mu2)*log(msb12/Q2)*log(mu2/Q2)*power2(mu2)*power2(snb)*(msb12 +
     DeltaInv(msb12,mt2,mu2)*(msb12*mu2*(4*mt2 + mu2) + 2*msb14*(mt2 + 4*mu2) -
     power3(msb12)) - 2*mu2*power2(DeltaInv(msb12,mt2,mu2))*(msb14*mu2*(-10*mt2
     + mu2) + msb12*(-mt2 + mu2)*power2(mu2) - 5*(mt2 + mu2)*power3(msb12) + 3*
     power4(msb12))))/mt2 + (ca*mu2*sa*sb*power2(Pi)*(-((3*mh2*msb22 + msb12*
     msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(
     mh2)*(-((5*msb12*msb22 + 3*msb24)*power2(mh2)) + 3*msb22*power3(mh2) + (-
     5*msb12 + 3*msb22)*power3(msb22) - mh2*(14*msb12*msb24 + 3*power3(msb22)))
     *power3(DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*(3*mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,
     msb22)*(14*mH2*msb12*msb24 + 5*msb12*msb22*power2(mH2) + 3*msb24*power2(
     mH2) - 3*msb22*power3(mH2) + 3*mH2*power3(msb22) + 5*msb12*power3(msb22) -
     3*power4(msb22)))))/(6.*cb*msb12) + (mu2*power2(Pi)*power2(sa)*power2(sb)*
     (-((3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22))) + power2(mh2)*(-((5*msb12*msb22 + 3*msb24)*power2(mh2)) +
     3*msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - mh2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22 + msb12*msb22 + 3*msb24 +
     DeltaInv(mH2,msb12,msb22)*(14*mH2*msb12*msb24 + 5*msb12*msb22*power2(mH2)
     + 3*msb24*power2(mH2) - 3*msb22*power3(mH2) + 3*mH2*power3(msb22) + 5*
     msb12*power3(msb22) - 3*power4(msb22)))))/(12.*msb12*power2(cb)) - (mu2*
     log(msb12/Q2)*log(msb22/Q2)*((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22)) + power2(mh2)*((msb12*msb22 -
     msb24)*power2(mh2) - msb22*power3(mh2) + (5*msb12 - 3*msb22)*power3(msb22)
     + 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))
     + power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 + 3*msb24 +
     msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(10*msb12*msb24*mz2 + msb12*msb22*
     power2(mz2) - msb24*power2(mz2) + 5*msb12*power3(msb22) + 5*mz2*power3(
     msb22) - msb22*power3(mz2) - 3*power4(msb22)))))/(2.*msb12) + (mu2*log(
     msb12/Q2)*log(msb22/Q2)*power2(cb)*(-((mA2*msb22 + msb12*msb22 + 3*msb24)*
     power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))) + power2(mA2)*((-(msb12*
     msb22) + msb24)*power2(mA2) + msb22*power3(mA2) + (-5*msb12 + 3*msb22)*
     power3(msb22) - 5*mA2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     msb12,mA2,msb22)) + power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*
     msb22 + 3*msb24 + msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(10*msb12*msb24*
     mz2 + msb12*msb22*power2(mz2) - msb24*power2(mz2) + 5*msb12*power3(msb22)
     + 5*mz2*power3(msb22) - msb22*power3(mz2) - 3*power4(msb22)))))/(2.*msb12)
     + (mu2*log(msb12/Q2)*log(msb22/Q2)*power2(sb)*(-((mA2*msb22 + msb12*msb22
     + 3*msb24)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))) + power2(mA2)*((
     -(msb12*msb22) + msb24)*power2(mA2) + msb22*power3(mA2) + (-5*msb12 + 3*
     msb22)*power3(msb22) - 5*mA2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(msb12,mA2,msb22)) + power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))
     *(msb12*msb22 + 3*msb24 + msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(10*msb12*
     msb24*mz2 + msb12*msb22*power2(mz2) - msb24*power2(mz2) + 5*msb12*power3(
     msb22) + 5*mz2*power3(msb22) - msb22*power3(mz2) - 3*power4(msb22)))))/
     msb12 + (mu2*power2(sb)*power2(log(msb12/Q2))*((2*mA2*msb22 + msb12*msb22
     + 2*msb24)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)) + power2(mA2)*((
     3*msb12*msb22 + 2*msb24)*power2(mA2) - 2*msb22*power3(mA2) + (3*msb12 - 2*
     msb22)*power3(msb22) + 2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(msb12,mA2,msb22)) + power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))
     *(-(msb12*msb22) - 2*(msb24 + msb22*mz2) - DeltaInv(msb12,mz2,msb22)*(10*
     msb12*msb24*mz2 + 3*msb12*msb22*power2(mz2) + 2*msb24*power2(mz2) + 3*
     msb12*power3(msb22) + 2*mz2*power3(msb22) - 2*msb22*power3(mz2) - 2*
     power4(msb22)))))/(2.*msb12) - (mu2*power2(log(msb12/Q2))*((2*mh2*msb22 +
     msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) +
     power2(mh2)*((3*msb12*msb22 + 2*msb24)*power2(mh2) - 2*msb22*power3(mh2) +
     (3*msb12 - 2*msb22)*power3(msb22) + 2*mh2*(5*msb12*msb24 + power3(msb22)))
     *power3(DeltaInv(mh2,msb12,msb22)) + power2(mz2)*power2(DeltaInv(msb12,
     mz2,msb22))*(msb12*msb22 + 2*msb24 + 2*msb22*mz2 + DeltaInv(msb12,mz2,
     msb22)*(10*msb12*msb24*mz2 + 3*msb12*msb22*power2(mz2) + 2*msb24*power2(
     mz2) + 3*msb12*power3(msb22) + 2*mz2*power3(msb22) - 2*msb22*power3(mz2) -
     2*power4(msb22)))))/(2.*msb12) + (mu2*power2(cb)*power2(log(msb12/Q2))*(-(
     (2*mA2*msb22 + msb12*msb22 + 2*msb24)*power2(mA2)*power2(DeltaInv(msb12,
     mA2,msb22))) + power2(mA2)*(-((3*msb12*msb22 + 2*msb24)*power2(mA2)) + 2*
     msb22*power3(mA2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mA2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + power2(mz2)*
     power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 + 2*msb24 + 2*msb22*mz2 +
     DeltaInv(msb12,mz2,msb22)*(10*msb12*msb24*mz2 + 3*msb12*msb22*power2(mz2)
     + 2*msb24*power2(mz2) + 3*msb12*power3(msb22) + 2*mz2*power3(msb22) - 2*
     msb22*power3(mz2) - 2*power4(msb22)))))/(2.*msb12) + (mu2*power2(sb)*
     power2(log(msb12/Q2))*(-((2*mA2*msb22 + msb12*msb22 + 2*msb24)*power2(mA2)
     *power2(DeltaInv(msb12,mA2,msb22))) + power2(mA2)*(-((3*msb12*msb22 + 2*
     msb24)*power2(mA2)) + 2*msb22*power3(mA2) + (-3*msb12 + 2*msb22)*power3(
     msb22) - 2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,
     msb22)) + power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 + 2*
     msb24 + 2*msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(10*msb12*msb24*mz2 + 3*
     msb12*msb22*power2(mz2) + 2*msb24*power2(mz2) + 3*msb12*power3(msb22) + 2*
     mz2*power3(msb22) - 2*msb22*power3(mz2) - 2*power4(msb22)))))/msb12 + (
     power2(snb)*power2(log(mu2/Q2))*power3(mu2)*((msb14 + msb12*(mt2 + 3*mu2))
     *power2(DeltaInv(msb12,mt2,mu2)) + power3(DeltaInv(msb12,mt2,mu2))*(5*
     msb14*mu2*(2*mt2 + mu2) + msb12*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*
     power3(msb12) - power4(msb12)) - power2(DeltaInv(mt2,mu2,msb22))*(msb24 +
     msb22*mt2 + 3*msb22*mu2 + DeltaInv(mt2,mu2,msb22)*(5*msb24*mu2*(2*mt2 +
     mu2) + msb22*(5*mt2 - 3*mu2)*power2(mu2) + (mt2 - mu2)*power3(msb22) -
     power4(msb22)))))/mt2 + Fin3(mt2,mu2,msb22,Q2)*((2*power2(mu2)*power2(
     DeltaInv(mt2,mu2,msb22))*(msb24 + msb22*(mt2 + 3*mu2) + DeltaInv(mt2,mu2,
     msb22)*(5*msb24*mu2*(2*mt2 + mu2) + msb22*(5*mt2 - 3*mu2)*power2(mu2) + (
     mt2 - mu2)*power3(msb22) - power4(msb22))))/mt2 - (2*power2(mu2)*power2(
     snb)*power2(DeltaInv(mt2,mu2,msb22))*(msb24 + msb22*(mt2 + 3*mu2) +
     DeltaInv(mt2,mu2,msb22)*(5*msb24*mu2*(2*mt2 + mu2) + msb22*(5*mt2 - 3*mu2)
     *power2(mu2) + (mt2 - mu2)*power3(msb22) - power4(msb22))))/mt2) + (
     DeltaInv(mt2,mu2,msb22)*log(mt2/Q2)*log(mu2/Q2)*power2(mu2)*(msb22 +
     DeltaInv(mt2,mu2,msb22)*(2*msb24*(mt2 + 5*mu2) + msb22*mu2*(6*mt2 + 7*mu2)
     - power3(msb22)) + 4*mu2*power2(DeltaInv(mt2,mu2,msb22))*(2*msb24*mu2*(5*
     mt2 + mu2) + msb22*(3*mt2 - 2*mu2)*power2(mu2) + (3*mt2 + 2*mu2)*power3(
     msb22) - 2*power4(msb22))))/mt2 + (mu2*log(msb12/Q2)*power2(cb)*(mA2 - mz2
     - DeltaInv(msb12,mA2,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*
     msb22)*power2(mA2) + 5*mA2*power2(msb22) + power3(mA2)) + 6*mA2*msb22*
     power2(DeltaInv(msb12,mA2,msb22))*(-(msb12*msb24) + mA2*(6*msb12*msb22 +
     23*msb24) - (msb12 - 23*msb22)*power2(mA2) + power3(mA2) + power3(msb22))
     + DeltaInv(msb12,mz2,msb22)*(5*mz2*power2(msb22) - msb12*(power2(msb22) +
     power2(mz2)) + msb22*(msb24 + 5*power2(mz2)) + power3(mz2)) - 6*msb22*mz2*
     power2(DeltaInv(msb12,mz2,msb22))*(23*msb24*mz2 + msb22*mz2*(6*msb12 + 23*
     mz2) - msb12*(msb24 + power2(mz2)) + power3(msb22) + power3(mz2)) - 72*
     msb22*power2(mA2)*(-((3*msb12*msb22 + 2*msb24)*power2(mA2)) + 2*msb22*
     power3(mA2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mA2*(5*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) - 72*msb22*power2(mz2)*
     power3(DeltaInv(msb12,mz2,msb22))*(10*msb12*msb24*mz2 + 3*msb12*msb22*
     power2(mz2) + 2*msb24*power2(mz2) + 3*msb12*power3(msb22) + 2*mz2*power3(
     msb22) - 2*msb22*power3(mz2) - 2*power4(msb22))))/(24.*msb12*msb22) + (
     mu2*log(msb12/Q2)*power2(sb)*(mA2 - mz2 - DeltaInv(msb12,mA2,msb22)*(
     msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mA2) + 5*mA2*
     power2(msb22) + power3(mA2)) + 6*mA2*msb22*power2(DeltaInv(msb12,mA2,
     msb22))*(-(msb12*msb24) + mA2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*
     msb22)*power2(mA2) + power3(mA2) + power3(msb22)) + DeltaInv(msb12,mz2,
     msb22)*(5*mz2*power2(msb22) - msb12*(power2(msb22) + power2(mz2)) + msb22*
     (msb24 + 5*power2(mz2)) + power3(mz2)) - 6*msb22*mz2*power2(DeltaInv(
     msb12,mz2,msb22))*(23*msb24*mz2 + msb22*mz2*(6*msb12 + 23*mz2) - msb12*(
     msb24 + power2(mz2)) + power3(msb22) + power3(mz2)) - 72*msb22*power2(mA2)
     *(-((3*msb12*msb22 + 2*msb24)*power2(mA2)) + 2*msb22*power3(mA2) + (-3*
     msb12 + 2*msb22)*power3(msb22) - 2*mA2*(5*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(msb12,mA2,msb22)) - 72*msb22*power2(mz2)*power3(DeltaInv(
     msb12,mz2,msb22))*(10*msb12*msb24*mz2 + 3*msb12*msb22*power2(mz2) + 2*
     msb24*power2(mz2) + 3*msb12*power3(msb22) + 2*mz2*power3(msb22) - 2*msb22*
     power3(mz2) - 2*power4(msb22))))/(12.*msb12*msb22) + (mu2*log(msb12/Q2)*
     power2(sb)*(-mA2 + mz2 + DeltaInv(msb12,mA2,msb22)*(msb22*(-(msb12*msb22)
     + msb24) - (msb12 - 5*msb22)*power2(mA2) + 5*mA2*power2(msb22) + power3(
     mA2)) - 6*mA2*msb22*power2(DeltaInv(msb12,mA2,msb22))*(-(msb12*msb24) +
     mA2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mA2) + power3(
     mA2) + power3(msb22)) - DeltaInv(msb12,mz2,msb22)*(5*mz2*power2(msb22) -
     msb12*(power2(msb22) + power2(mz2)) + msb22*(msb24 + 5*power2(mz2)) +
     power3(mz2)) + 6*msb22*mz2*power2(DeltaInv(msb12,mz2,msb22))*(23*msb24*mz2
     + msb22*mz2*(6*msb12 + 23*mz2) - msb12*(msb24 + power2(mz2)) + power3(
     msb22) + power3(mz2)) + 72*msb22*power2(mA2)*(-((3*msb12*msb22 + 2*msb24)*
     power2(mA2)) + 2*msb22*power3(mA2) + (-3*msb12 + 2*msb22)*power3(msb22) -
     2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) +
     72*msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(10*msb12*msb24*mz2
     + 3*msb12*msb22*power2(mz2) + 2*msb24*power2(mz2) + 3*msb12*power3(msb22)
     + 2*mz2*power3(msb22) - 2*msb22*power3(mz2) - 2*power4(msb22))))/(24.*
     msb12*msb22) + (log(msb12/Q2)*(mh2*msb12*mu2 + 2*msb12*msb22*mu2 + msb12*
     mu2*mz2 - 4*msb22*power2(msb12) - mh2*power2(mu2) - 2*msb12*power2(mu2) -
     2*msb22*power2(mu2) - mz2*power2(mu2) - (msb12 - mu2)*mu2*DeltaInv(mh2,
     msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(
     mh2) + 5*mh2*power2(msb22) + power3(mh2)) + 2*power3(msb12) + 6*mh2*msb22*
     (msb12 - mu2)*mu2*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*
     (6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2)
     + power3(msb22)) + (msb12 - mu2)*mu2*DeltaInv(msb12,mz2,msb22)*(-5*mz2*
     power2(msb22) + msb12*(power2(msb22) + power2(mz2)) - msb22*(msb24 + 5*
     power2(mz2)) - power3(mz2)) + 6*msb22*(msb12 - mu2)*mu2*mz2*power2(
     DeltaInv(msb12,mz2,msb22))*(23*msb24*mz2 + msb22*mz2*(6*msb12 + 23*mz2) -
     msb12*(msb24 + power2(mz2)) + power3(msb22) + power3(mz2)) - 72*msb22*(
     msb12 - mu2)*mu2*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(mh2)) +
     2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 72*msb22*(
     msb12 - mu2)*mu2*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(10*msb12*
     msb24*mz2 + 3*msb12*msb22*power2(mz2) + 2*msb24*power2(mz2) + 3*msb12*
     power3(msb22) + 2*mz2*power3(msb22) - 2*msb22*power3(mz2) - 2*power4(
     msb22))))/(24.*msb12*msb22*(msb12 - mu2)) + (power2(DeltaInv(mt2,mu2,
     msb22))*power2(log(mu2/Q2))*power3(mu2)*(msb24 + msb22*(mt2 + 3*mu2) +
     DeltaInv(mt2,mu2,msb22)*(5*msb24*mu2*(2*mt2 + mu2) + msb22*(5*mt2 - 3*mu2)
     *power2(mu2) + (mt2 - mu2)*power3(msb22) - power4(msb22))))/mt2 - (mu2*
     log(msb12/Q2)*log(mz2/Q2)*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(
     msb12*msb22 + msb24 + 3*msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(5*msb24*
     power2(mz2) - mz2*power3(msb22) + msb12*(10*msb24*mz2 + 5*msb22*power2(
     mz2) + power3(msb22)) - 3*msb22*power3(mz2) - power4(msb22))))/(2.*msb12)
     + (mu2*log(msb12/Q2)*log(mz2/Q2)*power2(cb)*power2(mz2)*power2(DeltaInv(
     msb12,mz2,msb22))*(msb12*msb22 + msb24 + 3*msb22*mz2 + DeltaInv(msb12,mz2,
     msb22)*(5*msb24*power2(mz2) - mz2*power3(msb22) + msb12*(10*msb24*mz2 + 5*
     msb22*power2(mz2) + power3(msb22)) - 3*msb22*power3(mz2) - power4(msb22)))
     )/(2.*msb12) + (mu2*log(msb12/Q2)*log(mz2/Q2)*power2(mz2)*power2(sb)*
     power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 + msb24 + 3*msb22*mz2 +
     DeltaInv(msb12,mz2,msb22)*(5*msb24*power2(mz2) - mz2*power3(msb22) +
     msb12*(10*msb24*mz2 + 5*msb22*power2(mz2) + power3(msb22)) - 3*msb22*
     power3(mz2) - power4(msb22))))/(2.*msb12) + (mu2*log(msb22/Q2)*log(mz2/Q2)
     *power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 + msb24 +
     msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(msb24*power2(mz2) + mz2*power3(
     msb22) + msb12*(6*msb24*mz2 + msb22*power2(mz2) + power3(msb22)) - msb22*
     power3(mz2) - power4(msb22))))/(2.*msb12) - (mu2*log(msb22/Q2)*log(mz2/Q2)
     *power2(cb)*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 +
     msb24 + msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(msb24*power2(mz2) + mz2*
     power3(msb22) + msb12*(6*msb24*mz2 + msb22*power2(mz2) + power3(msb22)) -
     msb22*power3(mz2) - power4(msb22))))/(2.*msb12) - (mu2*log(msb22/Q2)*log(
     mz2/Q2)*power2(mz2)*power2(sb)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*
     msb22 + msb24 + msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(msb24*power2(mz2) +
     mz2*power3(msb22) + msb12*(6*msb24*mz2 + msb22*power2(mz2) + power3(msb22)
     ) - msb22*power3(mz2) - power4(msb22))))/(2.*msb12) + (ca*mu2*sa*sb*log(
     msb22/Q2)*power2(snb)*(-((5*mh2*msb22 - msb12*msb22 + msb24)*DeltaInv(mh2,
     msb12,msb22)) - msb12*(mh2 - 6*msb22)*DeltaInv(mh2,msb22,msb22) + 5*mH2*
     msb22*DeltaInv(mH2,msb12,msb22) - msb12*msb22*DeltaInv(mH2,msb12,msb22) +
     msb24*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) - 6*
     msb12*msb22*DeltaInv(mH2,msb22,msb22) + 6*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb12,msb22)) + 18*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 84*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*
     msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(
     mH2,msb12,msb22))*power3(mH2) - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power3(msb22) - 72*msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) +
     6*mh2*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24
     - msb12*msb24 + msb22*power2(mh2) + power3(msb22)) + 24*msb12*power2(
     DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*
     (msb24*power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*
     power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(
     msb22)*power3(DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,
     msb12,msb22))*power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22)
     )*power4(msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22)))/(
     3.*cb*msb12) - (mu2*log(msb22/Q2)*power2(sa)*power2(snb)*(-((5*mh2*msb22 -
     msb12*msb22 + msb24)*DeltaInv(mh2,msb12,msb22)) - msb12*(mh2 - 6*msb22)*
     DeltaInv(mh2,msb22,msb22) + 5*mH2*msb22*DeltaInv(mH2,msb12,msb22) - msb12*
     msb22*DeltaInv(mH2,msb12,msb22) + msb24*DeltaInv(mH2,msb12,msb22) + mH2*
     msb12*DeltaInv(mH2,msb22,msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,msb22)
     + 6*mH2*msb12*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*msb22*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2)
     - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*
     power2(mh2) + power3(msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     (mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22)))/(6.*msb12)
     + (mu2*log(msb22/Q2)*power2(sa)*power2(sb)*power2(snb)*(-((5*mh2*msb22 -
     msb12*msb22 + msb24)*DeltaInv(mh2,msb12,msb22)) - msb12*(mh2 - 6*msb22)*
     DeltaInv(mh2,msb22,msb22) + 5*mH2*msb22*DeltaInv(mH2,msb12,msb22) - msb12*
     msb22*DeltaInv(mH2,msb12,msb22) + msb24*DeltaInv(mH2,msb12,msb22) + mH2*
     msb12*DeltaInv(mH2,msb22,msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,msb22)
     + 6*mH2*msb12*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*msb22*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2)
     - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*
     power2(mh2) + power3(msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     (mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22)))/(6.*msb12*
     power2(cb)) + (log(mu2/Q2)*power2(mu2)*power2(snb)*(-msb12 + msb22 -
     msb22*DeltaInv(msb12,mt2,mu2)*(mu2*(-mt2 + mu2) + 2*msb12*(mt2 + 2*mu2) +
     7*power2(msb12)) + msb12*DeltaInv(mt2,mu2,msb22)*(mu2*(-mt2 + mu2) + 2*
     msb22*(mt2 + 2*mu2) + 7*power2(msb22)) + 6*msb12*msb22*power2(DeltaInv(
     msb12,mt2,mu2))*(-(msb14*(mt2 + 7*mu2)) - msb12*mu2*(12*mt2 + 19*mu2) + (-
     mt2 + mu2)*power2(mu2) + power3(msb12)) + 6*msb12*msb22*power2(DeltaInv(
     mt2,mu2,msb22))*(msb24*(mt2 + 7*mu2) + msb22*mu2*(12*mt2 + 19*mu2) + (mt2
     - mu2)*power2(mu2) - power3(msb22)) + 36*msb12*msb22*mu2*power3(DeltaInv(
     msb12,mt2,mu2))*(-5*msb14*mu2*(2*mt2 + mu2) + msb12*(-5*mt2 + 3*mu2)*
     power2(mu2) + (-mt2 + mu2)*power3(msb12) + power4(msb12)) - 36*msb12*
     msb22*mu2*power3(DeltaInv(mt2,mu2,msb22))*(-5*msb24*mu2*(2*mt2 + mu2) +
     msb22*(-5*mt2 + 3*mu2)*power2(mu2) + (-mt2 + mu2)*power3(msb22) + power4(
     msb22))))/(6.*msb12*msb22*mt2) + (log(mu2/Q2)*(-3 + msb12/(msb12 - mu2) +
     msb22/(msb22 - mu2) + mst12/(mst12 - mu2) - mu2/msb12 - mu2/msb22 - mu2/
     mst12 + power2(mu2)/(msb22*mt2) - (DeltaInv(mt2,mu2,msb22)*(mu2*(-mt2 +
     mu2) + 2*msb22*(mt2 + 2*mu2) + 7*power2(msb22))*power2(mu2))/(msb22*mt2) +
     (6*power2(mu2)*power2(DeltaInv(mt2,mu2,msb22))*(-(msb24*(mt2 + 7*mu2)) -
     msb22*mu2*(12*mt2 + 19*mu2) + (-mt2 + mu2)*power2(mu2) + power3(msb22)))/
     mt2 + (36*power3(mu2)*power3(DeltaInv(mt2,mu2,msb22))*(-5*msb24*mu2*(2*mt2
     + mu2) + msb22*(-5*mt2 + 3*mu2)*power2(mu2) + (-mt2 + mu2)*power3(msb22) +
     power4(msb22)))/mt2))/6. + (power2(sa)*(-((mh2 - mH2)*(msb12 + msb22)) -
     2*mh2*msb12*msb22*mu2*DeltaInv(mh2,msb12,msb22) + 2*mH2*msb12*msb22*mu2*
     DeltaInv(mH2,msb12,msb22) + 84*msb22*(3*mh2*msb22 + msb12*msb22 + 3*msb24)
     *mu2*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) - 84*msb22*(3*mH2*msb22
     + msb12*msb22 + 3*msb24)*mu2*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))
     - 84*msb22*mu2*power2(mh2)*(-((5*msb12*msb22 + 3*msb24)*power2(mh2)) + 3*
     msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - mh2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 84*msb22*
     mu2*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 -
     5*msb12*msb22*power2(mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*
     mH2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22))))/(24.*msb12*
     msb22) - (DeltaInv(mt2,mu2,msb22)*log(msb22/Q2)*log(mu2/Q2)*power2(mu2)*(
     msb22 + DeltaInv(mt2,mu2,msb22)*(msb22*mu2*(4*mt2 + mu2) + 2*msb24*(mt2 +
     4*mu2) - power3(msb22)) - 2*mu2*power2(DeltaInv(mt2,mu2,msb22))*(msb24*
     mu2*(-10*mt2 + mu2) + msb22*(-mt2 + mu2)*power2(mu2) - 5*(mt2 + mu2)*
     power3(msb22) + 3*power4(msb22))))/mt2 + (DeltaInv(mt2,mu2,msb22)*log(
     msb22/Q2)*log(mu2/Q2)*power2(mu2)*power2(snb)*(msb22 + DeltaInv(mt2,mu2,
     msb22)*(msb22*mu2*(4*mt2 + mu2) + 2*msb24*(mt2 + 4*mu2) - power3(msb22)) -
     2*mu2*power2(DeltaInv(mt2,mu2,msb22))*(msb24*mu2*(-10*mt2 + mu2) + msb22*(
     -mt2 + mu2)*power2(mu2) - 5*(mt2 + mu2)*power3(msb22) + 3*power4(msb22))))
     /mt2 - (2*mu2*log(mh2/Q2)*log(msb22/Q2)*power2(snb)*((mh2*msb22 + msb12*
     msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + power2(mh2)
     *((msb12*msb22 + msb24)*power2(mh2) - msb22*power3(mh2) + (msb12 - msb22)*
     power3(msb22) + mh2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*
     power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22))))/msb12 -
     (4*ca*mu2*sa*sb*log(mh2/Q2)*log(msb22/Q2)*power2(snb)*((mh2*msb22 + msb12*
     msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + power2(mh2)
     *((msb12*msb22 + msb24)*power2(mh2) - msb22*power3(mh2) + (msb12 - msb22)*
     power3(msb22) + mh2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*
     power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22))))/(cb*
     msb12) + (2*mu2*log(mh2/Q2)*log(msb22/Q2)*power2(sa)*power2(snb)*((mh2*
     msb22 + msb12*msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))
     + power2(mh2)*((msb12*msb22 + msb24)*power2(mh2) - msb22*power3(mh2) + (
     msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(DeltaInv(mh2,msb22,msb22)
     )*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(
     msb22))))/msb12 - (2*mu2*log(mh2/Q2)*log(msb22/Q2)*power2(sa)*power2(sb)*
     power2(snb)*((mh2*msb22 + msb12*msb22 + msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + power2(mh2)*((msb12*msb22 + msb24)*power2(
     mh2) - msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(
     DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(
     mh2,msb22,msb22)*power4(msb22))))/(msb12*power2(cb)) + (4*ca*mu2*sa*sb*
     log(mH2/Q2)*log(msb22/Q2)*power2(snb)*((mH2*msb22 + msb12*msb22 + msb24)*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((msb12*msb22
     + msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) +
     mH2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) +
     msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*
     mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))))/(cb*msb12) - (2*mu2*log(
     mH2/Q2)*log(msb22/Q2)*power2(sa)*power2(snb)*((mH2*msb22 + msb12*msb22 +
     msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((
     msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*
     power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,
     msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*
     power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))))/msb12 -
     (2*mu2*log(mH2/Q2)*log(msb22/Q2)*power2(sb)*power2(snb)*((mH2*msb22 +
     msb12*msb22 + msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     power2(mH2)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) + (
     msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,msb22)
     )*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(
     msb22))))/(msb12*power2(cb)) + (2*mu2*log(mH2/Q2)*log(msb22/Q2)*power2(sa)
     *power2(sb)*power2(snb)*((mH2*msb22 + msb12*msb22 + msb24)*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((msb12*msb22 + msb24)*
     power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*
     msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + msb12*
     power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*
     DeltaInv(mH2,msb22,msb22)*power4(msb22))))/(msb12*power2(cb)) + (log(mt2/
     Q2)*log(mu2/Q2)*power2(mu2)*power2(snb)*(msb12*DeltaInv(msb12,mt2,mu2) +
     power2(DeltaInv(msb12,mt2,mu2))*(2*msb14*(mt2 + 5*mu2) + msb12*mu2*(6*mt2
     + 7*mu2) - power3(msb12)) + 4*mu2*power3(DeltaInv(msb12,mt2,mu2))*(2*
     msb14*mu2*(5*mt2 + mu2) + msb12*(3*mt2 - 2*mu2)*power2(mu2) + (3*mt2 + 2*
     mu2)*power3(msb12) - 2*power4(msb12)) + DeltaInv(mt2,mu2,msb22)*(-msb22 +
     DeltaInv(mt2,mu2,msb22)*(-2*msb24*(mt2 + 5*mu2) - msb22*mu2*(6*mt2 + 7*
     mu2) + power3(msb22)) + 4*mu2*power2(DeltaInv(mt2,mu2,msb22))*(-2*msb24*
     mu2*(5*mt2 + mu2) + msb22*(-3*mt2 + 2*mu2)*power2(mu2) - (3*mt2 + 2*mu2)*
     power3(msb22) + 2*power4(msb22)))))/mt2 + (mu2*power2(Pi)*power2(sa)*((3*
     mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + power2(mh2)*((5*msb12*msb22 + 3*msb24)*power2(mh2) - 3*msb22*
     power3(mh2) + (5*msb12 - 3*msb22)*power3(msb22) + mh2*(14*msb12*msb24 + 3*
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - 3*msb24 +
     DeltaInv(mH2,msb12,msb22)*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(mH2)
     - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*power3(msb22) - 5*
     msb12*power3(msb22) + 3*power4(msb22)))))/(12.*msb12) + (ca*mu2*sa*sb*(
     mh2*msb12*DeltaInv(mh2,msb12,msb22) - 42*(3*mh2*msb22 + msb12*msb22 + 3*
     msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 42*power2(mh2)*(-((
     5*msb12*msb22 + 3*msb24)*power2(mh2)) + 3*msb22*power3(mh2) + (-5*msb12 +
     3*msb22)*power3(msb22) - mh2*(14*msb12*msb24 + 3*power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)) + mH2*DeltaInv(mH2,msb12,msb22)*(-msb12 + 42*
     mH2*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*DeltaInv(mH2,msb12,msb22) - 42*
     mH2*power2(DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*msb12*
     msb22*power2(mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*
     power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)))))/(6.*cb*msb12)
     + (mu2*power2(sa)*power2(sb)*(mh2*msb12*DeltaInv(mh2,msb12,msb22) - 42*(3*
     mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + 42*power2(mh2)*(-((5*msb12*msb22 + 3*msb24)*power2(mh2)) + 3*
     msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - mh2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + mH2*
     DeltaInv(mH2,msb12,msb22)*(-msb12 + 42*mH2*(3*mH2*msb22 + msb12*msb22 + 3*
     msb24)*DeltaInv(mH2,msb12,msb22) - 42*mH2*power2(DeltaInv(mH2,msb12,msb22)
     )*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(mH2) - 3*msb24*power2(mH2) +
     3*msb22*power3(mH2) - 3*mH2*power3(msb22) - 5*msb12*power3(msb22) + 3*
     power4(msb22)))))/(12.*msb12*power2(cb)) + (mu2*log(msb12/Q2)*log(msb22/
     Q2)*power2(sb)*((mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*power2(
     DeltaInv(msb12,mA2,msb22)) + power2(mA2)*((msb12*msb22 - msb24)*power2(
     mA2) - msb22*power3(mA2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mA2*(2*
     msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + power2(
     mz2)*power2(DeltaInv(msb12,mz2,msb22))*(-(msb12*msb22) - 3*msb24 - msb22*
     mz2 + DeltaInv(msb12,mz2,msb22)*(msb24*power2(mz2) - 5*mz2*power3(msb22) -
     msb12*(10*msb24*mz2 + msb22*power2(mz2) + 5*power3(msb22)) + msb22*power3(
     mz2) + 3*power4(msb22)))))/(2.*msb12) + (mu2*power2(Pi)*power2(sb)*(msb22*
     (3*mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*power2(DeltaInv(msb12,
     mA2,msb22)) - msb22*(msb12*msb22 + 3*(msb24 + msb22*mz2))*power2(mz2)*
     power2(DeltaInv(msb12,mz2,msb22)) + msb22*power2(mA2)*((5*msb12*msb22 + 3*
     msb24)*power2(mA2) - 3*msb22*power3(mA2) + (5*msb12 - 3*msb22)*power3(
     msb22) + mA2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(msb12,
     mA2,msb22)) - msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(14*
     msb12*msb24*mz2 + 5*msb12*msb22*power2(mz2) + 3*msb24*power2(mz2) + 5*
     msb12*power3(msb22) + 3*mz2*power3(msb22) - 3*msb22*power3(mz2) - 3*
     power4(msb22)) - 2*msb12*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(-((5*msb22*mst12 +
     3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(
     mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*
     msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*
     mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))))))/(12.*msb12*
     msb22) + (mu2*power2(Pi)*power2(sb)*(-(msb22*(3*mA2*msb22 + msb12*msb22 +
     3*msb24)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))) + msb22*(msb12*
     msb22 + 3*(msb24 + msb22*mz2))*power2(mz2)*power2(DeltaInv(msb12,mz2,
     msb22)) + msb22*power2(mA2)*(-((5*msb12*msb22 + 3*msb24)*power2(mA2)) + 3*
     msb22*power3(mA2) + (-5*msb12 + 3*msb22)*power3(msb22) - mA2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + msb22*
     power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(14*msb12*msb24*mz2 + 5*
     msb12*msb22*power2(mz2) + 3*msb24*power2(mz2) + 5*msb12*power3(msb22) + 3*
     mz2*power3(msb22) - 3*msb22*power3(mz2) - 3*power4(msb22)) + 2*msb12*(-((
     3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*
     mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))))))/(6.*msb12*msb22) + (power2(cb)*
     power2(Pi)*(-(msb22*(3*mA2*msb22 + msb12*msb22 + 3*msb24)*(mst12 - mst22)*
     mu2*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))) + msb22*(mst12 - mst22)
     *mu2*(msb12*msb22 + 3*(msb24 + msb22*mz2))*power2(mz2)*power2(DeltaInv(
     msb12,mz2,msb22)) + msb22*(mst12 - mst22)*mu2*power2(mA2)*(-((5*msb12*
     msb22 + 3*msb24)*power2(mA2)) + 3*msb22*power3(mA2) + (-5*msb12 + 3*msb22)
     *power3(msb22) - mA2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(
     msb12,mA2,msb22)) + msb22*(mst12 - mst22)*mu2*power2(mz2)*power3(DeltaInv(
     msb12,mz2,msb22))*(14*msb12*msb24*mz2 + 5*msb12*msb22*power2(mz2) + 3*
     msb24*power2(mz2) + 5*msb12*power3(msb22) + 3*mz2*power3(msb22) - 3*msb22*
     power3(mz2) - 3*power4(msb22)) + 2*msb12*(-((3*mC2*mst12 + msb22*mst12 +
     3*mst14)*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + (3*mC2*mst22 + msb22*mst22 + 3*mst24)*mt2*(mst12 - mst22 -
     2*mu2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + (mst12 - mst22 - 2*
     mt2)*mu2*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + mt2*(-mst12 + mst22
     + 2*mu2)*power2(mC2)*(-((5*msb22*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*
     power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - mC2*(14*msb22*mst24 +
     3*power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + (mst12 - mst22)*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*(mst14 +
     mst12*mw2) + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*
     mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*
     power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))))))/(12.*msb12*
     msb22*(mst12 - mst22)) + (mu2*power2(sb)*(-(mA2*msb12*msb22*DeltaInv(
     msb12,mA2,msb22)) + msb12*msb22*mz2*DeltaInv(msb12,mz2,msb22) + 42*msb22*(
     3*mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*power2(DeltaInv(msb12,
     mA2,msb22)) - 42*msb22*(msb12*msb22 + 3*(msb24 + msb22*mz2))*power2(mz2)*
     power2(DeltaInv(msb12,mz2,msb22)) - 42*msb22*power2(mA2)*(-((5*msb12*msb22
     + 3*msb24)*power2(mA2)) + 3*msb22*power3(mA2) + (-5*msb12 + 3*msb22)*
     power3(msb22) - mA2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(
     msb12,mA2,msb22)) - 42*msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))
     *(14*msb12*msb24*mz2 + 5*msb12*msb22*power2(mz2) + 3*msb24*power2(mz2) +
     5*msb12*power3(msb22) + 3*mz2*power3(msb22) - 3*msb22*power3(mz2) - 3*
     power4(msb22)) + 2*msb12*(-(mC2*msb22*DeltaInv(msb22,mst12,mC2)) + 42*(3*
     mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - 42*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) +
     3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + mw2*
     DeltaInv(mw2,msb22,mst12)*(msb22 - 42*mw2*(msb22*mst12 + 3*mst14 + 3*
     mst12*mw2)*DeltaInv(mw2,msb22,mst12) - 42*mw2*power2(DeltaInv(mw2,msb22,
     mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(
     mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) -
     3*power4(mst12))))))/(12.*msb12*msb22) + (mu2*power2(sb)*(mA2*msb12*msb22*
     DeltaInv(msb12,mA2,msb22) - msb12*msb22*mz2*DeltaInv(msb12,mz2,msb22) -
     42*msb22*(3*mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*power2(
     DeltaInv(msb12,mA2,msb22)) + 42*msb22*(msb12*msb22 + 3*(msb24 + msb22*mz2)
     )*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22)) + 42*msb22*power2(mA2)*(-(
     (5*msb12*msb22 + 3*msb24)*power2(mA2)) + 3*msb22*power3(mA2) + (-5*msb12 +
     3*msb22)*power3(msb22) - mA2*(14*msb12*msb24 + 3*power3(msb22)))*power3(
     DeltaInv(msb12,mA2,msb22)) + 42*msb22*power2(mz2)*power3(DeltaInv(msb12,
     mz2,msb22))*(14*msb12*msb24*mz2 + 5*msb12*msb22*power2(mz2) + 3*msb24*
     power2(mz2) + 5*msb12*power3(msb22) + 3*mz2*power3(msb22) - 3*msb22*
     power3(mz2) - 3*power4(msb22)) + 2*msb12*(mC2*msb22*DeltaInv(msb22,mst12,
     mC2) - 42*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + 42*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) -
     mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     + mw2*DeltaInv(mw2,msb22,mst12)*(-msb22 + 42*mw2*(msb22*mst12 + 3*mst14 +
     3*mst12*mw2)*DeltaInv(mw2,msb22,mst12) + 42*mw2*power2(DeltaInv(mw2,msb22,
     mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(
     mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) -
     3*power4(mst12))))))/(6.*msb12*msb22) + (mu2*power2(sb)*power2(log(msb22/
     Q2))*(msb22*msb24*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)) - msb22*
     msb24*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22)) - msb22*power2(mA2)*(
     msb24*power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + msb22*power2(mz2)*
     power3(DeltaInv(msb12,mz2,msb22))*(msb24*power2(mz2) - 2*mz2*power3(msb22)
     - 2*msb12*(msb24*mz2 + power3(msb22)) + power4(msb22)) + 2*msb12*((2*mC2*
     mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2)) + power2(mC2)*((3*msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*
     power3(mC2) + (3*msb22 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 2*(mst14 + mst12*mw2) -
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))))))/(2.*msb12*msb22) + (mu2*power2(
     sb)*power2(log(msb22/Q2))*(-(msb22*msb24*power2(mA2)*power2(DeltaInv(
     msb12,mA2,msb22))) + msb22*msb24*power2(mz2)*power2(DeltaInv(msb12,mz2,
     msb22)) + msb22*power2(mA2)*(msb24*power2(mA2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mA2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     msb12,mA2,msb22)) - msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(
     msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*msb12*(msb24*mz2 + power3(
     msb22)) + power4(msb22)) + 2*msb12*(-((2*mC2*mst12 + msb22*mst12 + 2*
     mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     *(msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*
     msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))))))/(msb12*msb22) + (power2(cb)*power2(log(msb22/Q2))*(
     msb22*msb24*(-mst12 + mst22)*mu2*power2(mA2)*power2(DeltaInv(msb12,mA2,
     msb22)) + msb22*msb24*(mst12 - mst22)*mu2*power2(mz2)*power2(DeltaInv(
     msb12,mz2,msb22)) + msb22*(mst12 - mst22)*mu2*power2(mA2)*(msb24*power2(
     mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24 + power3(
     msb22)))*power3(DeltaInv(msb12,mA2,msb22)) - msb22*(mst12 - mst22)*mu2*
     power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(msb24*power2(mz2) - 2*mz2*
     power3(msb22) - 2*msb12*(msb24*mz2 + power3(msb22)) + power4(msb22)) + 2*
     msb12*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*(mst12 - mst22 - 2*mt2)*
     mu2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*
     mst22 + 2*mst24)*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + (mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + mt2*(-mst12 + mst22 + 2*mu2)*power2(mC2)*(-((3*msb22*
     mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)
     *power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + (mst12 - mst22)*mu2*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12))*(msb22*mst12 + 2*(mst14 + mst12*mw2) + DeltaInv(mw2,msb22,
     mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(
     mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) -
     2*power4(mst12))))))/(2.*msb12*msb22*(mst12 - mst22)) + (log(msb22/Q2)*
     power2(cb)*(-(msb22*(5*mA2*msb22 - msb12*msb22 + msb24)*mst12*(mst12 -
     mst22)*mst22*mu2*DeltaInv(msb12,mA2,msb22)) - msb22*mst12*(mst12 - mst22)*
     mst22*mu2*(msb12*msb22 - msb24 - 5*msb22*mz2)*DeltaInv(msb12,mz2,msb22) -
     6*msb22*mst12*(mst12 - mst22)*mst22*mu2*mz2*power2(DeltaInv(msb12,mz2,
     msb22))*(msb22*mz2*(-3*msb12 + mz2) - msb24*(msb12 + 14*mz2) + power3(
     msb22)) + 6*mA2*msb22*mst12*(mst12 - mst22)*mst22*mu2*power2(DeltaInv(
     msb12,mA2,msb22))*(-3*mA2*msb12*msb22 - 14*mA2*msb24 - msb12*msb24 +
     msb22*power2(mA2) + power3(msb22)) + 72*msb22*mst12*(mst12 - mst22)*mst22*
     mu2*power2(mA2)*(msb24*power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*
     mA2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) - 72*
     msb22*mst12*(mst12 - mst22)*mst22*mu2*power2(mz2)*power3(DeltaInv(msb12,
     mz2,msb22))*(msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*msb12*(msb24*mz2
     + power3(msb22)) + power4(msb22)) + 2*msb12*(mst22*(mst12 - mst22 - 2*mt2)
     *mu2*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*mt2*(-
     mst12 + mst22 + 2*mu2)*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) +
     mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)
     ) - 6*mC2*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*power2(DeltaInv(msb22,
     mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*
     mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*
     power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*mt2*(mst12 -
     mst22 - 2*mu2)*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + (mst12 -
     mst22)*(mC2*mst12*mt2 + msb22*mst12*mt2 + mst12*mst22*mt2 - mC2*mst22*mu2
     - 2*mC2*mt2*mu2 - 2*msb22*mt2*mu2 + mst22*mu2*mw2 - mst22*mu2*DeltaInv(
     mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) + power2(mw2)
     ) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mst22*mu2*mw2*
     power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*
     mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) + 72*
     mst12*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*
     mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))))))/(24.*msb12*msb22*mst12*mst22*(-mst12 + mst22)) - (mu2*power2(
     Pi)*power2(sb)*(msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + msb22*(3*mA2*msb22 + msb12*msb22 + 3*
     msb24)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)) + msb22*power2(mH2)*(
     (5*msb12*msb22 + 3*msb24)*power2(mH2) - 3*msb22*power3(mH2) + (5*msb12 -
     3*msb22)*power3(msb22) + mH2*(14*msb12*msb24 + 3*power3(msb22)))*power3(
     DeltaInv(mH2,msb12,msb22)) + msb22*power2(mA2)*((5*msb12*msb22 + 3*msb24)*
     power2(mA2) - 3*msb22*power3(mA2) + (5*msb12 - 3*msb22)*power3(msb22) +
     mA2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22))
     + 2*msb12*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)*(14*mC2*msb22*mst14 + 5*
     msb22*mst12*power2(mC2) + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) + 3*
     mC2*power3(mst12) + 5*msb22*power3(mst12) - 3*power4(mst12)))))/(12.*
     msb12*msb22*power2(cb)) + (mu2*power2(Pi)*power2(sb)*power2(snb)*(msb22*(
     3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) + msb22*power2(mC2)*((5*msb12*mst12 + 3*
     mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(
     mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + msb12*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2))
     + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*
     msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(-(msb22*(msb12*mst12 + 3*(mst14 + mst12*mw2))*power2(
     DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*
     mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*
     power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(
     mst12))) - msb22*power3(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*mw2 +
     5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) +
     3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12)))))/(6.*msb12*
     msb22) + (mu2*power2(Pi)*power2(sb)*power2(snb)*(-(msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) + msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(
     mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(
     14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(mC2)*((5*msb22*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(
     mC2) + (5*msb22 - 3*mst12)*power3(mst12) + mC2*(14*msb22*mst14 + 3*power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(msb22*(msb12*
     mst12 + 3*(mst14 + mst12*mw2))*power2(DeltaInv(mw2,msb12,mst12)) - msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(
     mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) -
     3*power4(mst12)))))/(3.*msb12*msb22) - (mu2*power2(sb)*power2(snb)*(-(mC2*
     msb12*msb22*DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(msb22,
     mst12,mC2) + 42*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*mC2*mst12 + msb22*mst12 +
     3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*msb22*power2(
     mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-
     5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))
     *power3(DeltaInv(msb12,mst12,mC2)) + 42*msb12*power2(mC2)*(-((5*msb22*
     mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)
     *power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + mw2*(msb12*msb22*DeltaInv(mw2,msb12,mst12) - 42*msb22*
     mw2*(msb12*mst12 + 3*(mst14 + mst12*mw2))*power2(DeltaInv(mw2,msb12,mst12)
     ) + msb12*DeltaInv(mw2,msb22,mst12)*(-msb22 + 42*mw2*(msb22*mst12 + 3*
     mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb22,mst12) + 42*mw2*power2(DeltaInv(
     mw2,msb22,mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*
     mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*
     power3(mw2) - 3*power4(mst12))) - 42*msb22*mw2*power3(DeltaInv(mw2,msb12,
     mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(
     mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) -
     3*power4(mst12)))))/(6.*msb12*msb22) + (2*mu2*log(msb12/Q2)*log(mst12/Q2)*
     power2(sb)*power2(snb)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)
     *power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/msb12 + (mu2*log(msb12/
     Q2)*log(mst12/Q2)*power2(sb)*power2(snb)*power2(snt)*(-((mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-
     5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(
     10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*
     msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(
     mst12)))))/msb12 + (log(msb12/Q2)*log(mst12/Q2)*power2(snb)*power2(snt)*(-
     ((mC2*mst12 + msb12*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + mt2*power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2)
     + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + mu2*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2
     + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*power2(mw2)
     - mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*
     power3(mw2) - 3*power4(mst12)))))/msb12 + (2*mu2*log(msb22/Q2)*log(mst12/
     Q2)*power2(sb)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*
     power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/msb22 + (mu2*log(msb22/
     Q2)*log(mst12/Q2)*power2(sb)*power2(snb)*(-((mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(
     msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     *(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(
     mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/
     msb22 + (mu2*log(msb22/Q2)*log(mst12/Q2)*power2(sb)*power2(snt)*(-((mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(
     mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) +
     5*msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*
     power4(mst12)))))/msb22 + (2*mu2*log(msb22/Q2)*log(mst12/Q2)*power2(sb)*
     power2(snb)*power2(snt)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)
     *power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(msb22*mst12) +
     mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(
     mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 +
     msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12) + 5*
     mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/msb22 + (log(
     msb22/Q2)*log(mst12/Q2)*power2(snt)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)
     *mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + mt2*power2(mC2)*((-(
     msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*
     msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(
     mst12)))))/msb22 + (mu2*log(msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)
     *(-((mC2*mst12 + msb12*mst12 + 3*mst14)*(mst12 - mst22 - 2*mt2)*power2(
     mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (mst12 - mst22 - 2*mt2)*power2(
     mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + (mst12 - mst22)*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb12*(10*mst14*mw2 +
     mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12)
     ))))/(msb12*(mst12 - mst22)) + (mu2*log(msb22/Q2)*log(mst12/Q2)*power2(cb)
     *(-((mC2*mst12 + msb22*mst12 + 3*mst14)*(mst12 - mst22 - 2*mt2)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (mst12 - mst22 - 2*mt2)*power2(
     mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb22
     + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + (mst12 - mst22)*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb22*(10*mst14*mw2 +
     mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12)
     ))))/(msb22*(mst12 - mst22)) - (mu2*log(msb22/Q2)*log(mst12/Q2)*power2(cb)
     *power2(snb)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*(mst12 - mst22 - 2*
     mt2)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (mst12 - mst22 - 2*
     mt2)*power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2)
     + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + (mst12 - mst22)*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 5*mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) - mst12*power3(
     mw2) - 3*power4(mst12)))))/(msb22*(mst12 - mst22)) + (mu2*power2(sb)*
     power2(snb)*power2(log(msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((3*msb12*
     mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*
     power3(mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(
     msb12*mst12) - 2*(mst14 + mst12*mw2) - DeltaInv(mw2,msb12,mst12)*(10*
     msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12)))))/msb12 + (2*mu2*power2(sb)*power2(snb)*power2(log(msb12/
     Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2))
     + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/msb12 + (power2(cb)*
     power2(snb)*power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)
     *(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))
     ) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*mt2*(mst12 - mst22 - 2*mu2)*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + (mst12 - mst22 - 2*mt2)*
     mu2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + mt2*(-mst12 + mst22 +
     2*mu2)*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + (mst12 - mst22)*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*(mst14 +
     mst12*mw2) + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/(msb12*(mst12 -
     mst22)) + (log(msb12/Q2)*power2(cb)*power2(snb)*(mst22*(mst12 - mst22 - 2*
     mt2)*mu2*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (
     msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     mt2*(-mst12 + mst22 + 2*mu2)*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*
     mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) - 6*mC2*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*power2(
     DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*
     mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) +
     6*mC2*mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(mst22,msb12,
     mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*
     mst22)*power2(mC2) + power3(mC2) + power3(mst22)) + 72*mst12*mst22*(mst12
     - mst22 - 2*mt2)*mu2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2))
     + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*
     mst12*mst22*mt2*(mst12 - mst22 - 2*mu2)*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + (mst12 - mst22)*(mC2*mst12*mt2 + msb12*mst12*mt2 + mst12*
     mst22*mt2 - mC2*mst22*mu2 - 2*mC2*mt2*mu2 - 2*msb12*mt2*mu2 + mst22*mu2*
     mw2 - mst22*mu2*DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(
     power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)
     ) + 6*mst12*mst22*mu2*mw2*power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2
     + mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14 + power2(mw2)) + power3(
     mst12) + power3(mw2)) + 72*mst12*mst22*mu2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*
     mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)))))/(12.*msb12*mst12*mst22*(-mst12 + mst22))
     + (2*mu2*power2(sb)*power2(snb)*power2(log(msb22/Q2))*((2*mC2*mst12 +
     msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) +
     power2(mC2)*((3*msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) +
     (3*msb22 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))
     *power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12))*(-(msb22*mst12) - 2*(mst14 + mst12*mw2) - DeltaInv(mw2,
     msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12)))))/msb22 + (mu2*power2(sb)*power2(snb)*
     power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)
     *power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*
     msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/msb22 - (
     power2(cb)*power2(snb)*power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12
     + 2*mst14)*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*mt2*(mst12 - mst22 -
     2*mu2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + (mst12 - mst22 - 2*
     mt2)*mu2*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + mt2*(-mst12 + mst22 +
     2*mu2)*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + (mst12 - mst22)*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*(mst14 +
     mst12*mw2) + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/(msb22*(mst12 -
     mst22)) + (mu2*log(msb22/Q2)*power2(sb)*(msb22*(5*mA2*msb22 - msb12*msb22
     + msb24)*mst12*DeltaInv(msb12,mA2,msb22) + msb22*mst12*(msb12*msb22 -
     msb24 - 5*msb22*mz2)*DeltaInv(msb12,mz2,msb22) + 6*msb22*mst12*mz2*power2(
     DeltaInv(msb12,mz2,msb22))*(msb22*mz2*(-3*msb12 + mz2) - msb24*(msb12 +
     14*mz2) + power3(msb22)) - 6*mA2*msb22*mst12*power2(DeltaInv(msb12,mA2,
     msb22))*(-3*mA2*msb12*msb22 - 14*mA2*msb24 - msb12*msb24 + msb22*power2(
     mA2) + power3(msb22)) - 72*msb22*mst12*power2(mA2)*(msb24*power2(mA2) + (-
     2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(msb12,mA2,msb22)) + 72*msb22*mst12*power2(mz2)*power3(
     DeltaInv(msb12,mz2,msb22))*(msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*
     msb12*(msb24*mz2 + power3(msb22)) + power4(msb22)) - 2*msb12*(-mC2 + mw2 +
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - 6*mC2*mst12*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mw2*
     power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*
     mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) + 72*
     mst12*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/(12.*msb12*msb22*mst12)
     + (mu2*log(msb22/Q2)*power2(sb)*(-(msb22*(5*mA2*msb22 - msb12*msb22 +
     msb24)*mst12*DeltaInv(msb12,mA2,msb22)) + msb22*mst12*(-(msb12*msb22) +
     msb24 + 5*msb22*mz2)*DeltaInv(msb12,mz2,msb22) - 6*msb22*mst12*mz2*power2(
     DeltaInv(msb12,mz2,msb22))*(msb22*mz2*(-3*msb12 + mz2) - msb24*(msb12 +
     14*mz2) + power3(msb22)) + 6*mA2*msb22*mst12*power2(DeltaInv(msb12,mA2,
     msb22))*(-3*mA2*msb12*msb22 - 14*mA2*msb24 - msb12*msb24 + msb22*power2(
     mA2) + power3(msb22)) + 72*msb22*mst12*power2(mA2)*(msb24*power2(mA2) + (-
     2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(msb12,mA2,msb22)) - 72*msb22*mst12*power2(mz2)*power3(
     DeltaInv(msb12,mz2,msb22))*(msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*
     msb12*(msb24*mz2 + power3(msb22)) + power4(msb22)) + 2*msb12*(-mC2 + mw2 +
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - 6*mC2*mst12*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mw2*
     power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*
     mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) + 72*
     mst12*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/(24.*msb12*msb22*mst12)
     + (log(msb22/Q2)*power2(cb)*power2(snb)*(-(mst22*(mst12 - mst22 - 2*mt2)*
     mu2*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2))) + mst12*mt2*(
     mst12 - mst22 - 2*mu2)*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) +
     mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)
     ) + 6*mC2*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*power2(DeltaInv(msb22,
     mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*
     mt2*(mst12 - mst22 - 2*mu2)*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - 72*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*
     power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*mt2*(mst12 -
     mst22 - 2*mu2)*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) - (mst12 -
     mst22)*(mC2*mst12*mt2 + msb22*mst12*mt2 + mst12*mst22*mt2 - mC2*mst22*mu2
     - 2*mC2*mt2*mu2 - 2*msb22*mt2*mu2 + mst22*mu2*mw2 - mst22*mu2*DeltaInv(
     mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) + power2(mw2)
     ) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mst22*mu2*mw2*
     power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*
     mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) + 72*
     mst12*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*
     mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12)))))/(12.*msb22*mst12*mst22*(-mst12 + mst22)) + (2*mu2*power2(sb)*
     power2(snb)*power2(log(mst12/Q2))*(-(msb22*mst14*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*mst14*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) + msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - msb12*power2(mC2)*(mst14*power2(mC2) + (-2*
     msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12)))))/(msb12*
     msb22) + (mu2*power2(sb)*power2(snb)*power2(snt)*power2(log(mst12/Q2))*(-(
     msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst14*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb22*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - msb12*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(msb22*
     mst14*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*
     power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) +
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*
     power3(mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12)))))/(
     msb12*msb22) + (power2(snb)*power2(snt)*power2(log(mst12/Q2))*(-(msb22*
     mst14*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst14*
     mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb22*mt2*power2(mC2)*
     (mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - msb12*mt2*power2(
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + mu2*power2(
     mw2)*(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(
     mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/(msb12*msb22) + (2*mu2*power2(sb)*power2(log(mst12/Q2))*
     (-(mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/msb22 + (mu2*power2(sb)*power2(snt)*power2(log(mst12/Q2)
     )*(-(mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/msb22 + (power2(snt)*power2(log(mst12/Q2))*(-(mst14*mt2*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + mt2*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(
     mst12)) - power4(mst12)))))/msb22 - (mu2*log(msb12/Q2)*log(mst12/Q2)*
     power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(-(mst14*power2(mw2)) + 5*
     mw2*power3(mst12) + msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(
     mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/msb12 - (mu2*log(msb22/
     Q2)*log(mst12/Q2)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(
     mw2)) + 5*mw2*power3(mst12) + msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*
     power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/msb22 + (mu2*log(
     msb22/Q2)*log(mst12/Q2)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 5*mw2*power3(mst12) + msb22*(10*mst14*mw2 + mst12*
     power2(mw2) + 5*power3(mst12)) - mst12*power3(mw2) - 3*power4(mst12))))/
     msb22 + (mu2*log(msb12/Q2)*power2(sb)*power2(snb)*(mC2 - mw2 - DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + 6*mC2*mst12*power2(
     DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*
     mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) +
     DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) - 6*mst12*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*
     mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) - 72*
     mst12*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*mst12*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))))/(6.*msb12*mst12) + (mu2*
     log(msb12/Q2)*power2(sb)*power2(snb)*(-mC2 + mw2 + DeltaInv(msb12,mst12,
     mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*
     mC2*power2(mst12) + power3(mC2)) - 6*mC2*mst12*power2(DeltaInv(msb12,
     mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - DeltaInv(mw2,msb12,
     mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)) + mst12*
     (mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14
     + power2(mw2)) + power3(mst12) + power3(mw2)) + 72*mst12*power2(mC2)*(-((
     3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 72*mst12*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))))/(12.*msb12*mst12) + (mu2*log(msb22/Q2)*
     power2(sb)*power2(snb)*(mC2 - mw2 - DeltaInv(msb22,mst12,mC2)*(mst12*(-(
     msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) + 6*mC2*mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(
     msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) + DeltaInv(mw2,msb22,mst12)*(5*mw2*
     power2(mst12) - msb22*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*
     power2(mw2)) + power3(mw2)) - 6*mst12*mw2*power2(DeltaInv(mw2,msb22,mst12)
     )*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(
     mw2)) + power3(mst12) + power3(mw2)) - 72*mst12*power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) - 72*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     *(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) +
     3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))))/(12.*msb22*mst12) + (mu2*log(msb22/Q2)*power2(sb)*power2(
     snb)*(-mC2 + mw2 + DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) +
     mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) - 6*mC2*mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(
     6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) - DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(
     power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)
     ) + 6*mst12*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*
     mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) +
     power3(mw2)) + 72*mst12*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(
     mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(
     5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*
     mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 +
     3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) +
     2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))))/(6.*msb22*
     mst12) - (mu2*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))))/msb12 + (mu2*log(msb12/Q2)*log(mw2/Q2)*power2(cb)*power2(
     mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 +
     3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))))/msb12 + (mu2*log(msb12/Q2)*log(mw2/
     Q2)*power2(mw2)*power2(sb)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))))/msb12 - (
     mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     )*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))))/msb22 + (
     mu2*log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     )/msb22 + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(sb)*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))))/msb22 + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))))/msb22 - (mu2*log(msb22/Q2)*log(mw2/
     Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))))/msb22 - (
     mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(sb)*power2(snb)*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*
     mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) -
     power4(mst12))))/msb22 + (mu2*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snb)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/
     (msb12*msb22) - (mu2*log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snb)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/
     (msb12*msb22) - (mu2*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(sb)*
     power2(snb)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/
     (msb12*msb22) - (mu2*log(mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power2(snt)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,
     mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/
     (msb12*msb22) + (mu2*log(mst12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*
     power2(snb)*power2(snt)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*
     (6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))))/(msb12*msb22) + (mu2*log(mst12/Q2)*log(mw2/Q2)*power2(
     mw2)*power2(sb)*power2(snb)*power2(snt)*(msb22*(msb12*mst12 + mst14 +
     mst12*mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,
     msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)
     *(mst14*power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*
     power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))) +
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(
     mst12) + msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*
     power3(mw2) - power4(mst12))))/(msb12*msb22) + (mu2*log(mst12/Q2)*log(mw2/
     Q2)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(
     mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*
     power3(mw2) - power4(mst12))))/msb22 - (mu2*log(mst12/Q2)*log(mw2/Q2)*
     power2(cb)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))))/msb22 - (mu2*log(mst12/Q2)*log(mw2/
     Q2)*power2(mw2)*power2(sb)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12
     + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))))/msb22 - (mu2*log(mst12/Q2)*log(mw2/
     Q2)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12
     + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))))/msb22 + (mu2*log(mst12/Q2)*log(mw2/
     Q2)*power2(cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(
     mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/msb22 + (mu2*log(
     mst12/Q2)*log(mw2/Q2)*power2(mw2)*power2(sb)*power2(snt)*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(mst14*power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 +
     mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))))/
     msb22 + (mu2*power2(mw2)*power2(snb)*power2(log(mst12/Q2))*(-(msb22*mst14*
     power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*
     power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) +
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(
     mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12))))/(msb12*
     msb22) + (mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*power2(log(
     mst12/Q2))*(-mst14 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(mst12))))/
     msb22 + (log(mst12/Q2)*power2(snb)*((-msb12 + msb22)*mst12 + msb22*mu2*(-(
     msb12*mst12) + mst14 + 5*mst12*mw2)*DeltaInv(mw2,msb12,mst12) + msb12*mu2*
     (msb22*mst12 - mst14 - 5*mst12*mw2)*DeltaInv(mw2,msb22,mst12) - 6*msb22*
     mu2*mw2*power2(DeltaInv(mw2,msb12,mst12))*(mst12*mw2*(-3*msb12 + mw2) -
     mst14*(msb12 + 14*mw2) + power3(mst12)) + 6*msb12*mu2*mw2*power2(DeltaInv(
     mw2,msb22,mst12))*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) +
     power3(mst12)) - 72*msb22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)) + 72*msb12*mu2*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*
     mw2 + power3(mst12)) + power4(mst12))))/(12.*msb12*msb22) + (log(mst12/Q2)
     *(mst12*(-2*msb22 + mst12 - mu2) - (mst12 - mu2)*mu2*(msb22*mst12 - mst14
     - 5*mst12*mw2)*DeltaInv(mw2,msb22,mst12) - 6*(mst12 - mu2)*mu2*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 +
     14*mw2) + power3(mst12)) - 72*(mst12 - mu2)*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*
     msb22*(mst14*mw2 + power3(mst12)) + power4(mst12))))/(12.*msb22*(mst12 -
     mu2)) - (log(mst12/Q2)*power2(snt)*(mst12*(-2*msb22 + mst12 - mu2) - (5*
     mC2*mst12 - msb22*mst12 + mst14)*mt2*(mst12 - mu2)*DeltaInv(msb22,mst12,
     mC2) - (mst12 - mu2)*mu2*(msb22*mst12 - mst14 - 5*mst12*mw2)*DeltaInv(mw2,
     msb22,mst12) - 6*(mst12 - mu2)*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12))*(
     mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) + power3(mst12)) + 6*
     mC2*mt2*(mst12 - mu2)*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) +
     72*mt2*(mst12 - mu2)*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) - 72*(mst12 - mu2)*mu2*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*
     mw2 + power3(mst12)) + power4(mst12))))/(12.*msb22*(mst12 - mu2)) + (mu2*
     power2(sb)*power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12)
     - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(msb12*msb22) +
     (2*mu2*power2(sb)*power2(snb)*power2(snt)*power2(log(mst12/Q2))*(msb22*
     mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(msb22*
     mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,
     msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) +
     2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))
     ) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(
     msb12*msb22) + (mu2*power2(cb)*power2(snb)*power2(log(mst12/Q2))*(msb22*
     mst14*(-mst12 + mst22 + 2*mt2)*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2)) + msb12*mst14*(mst12 - mst22 - 2*mt2)*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) + msb22*(mst12 - mst22 - 2*mt2)*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - msb12*(mst12 - mst22 -
     2*mt2)*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - (
     mst12 - mst22)*power2(mw2)*(-(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12)
     )) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,
     mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 +
     power3(mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))
     *(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(msb12*msb22*(mst12 - mst22)) + (power2(cb)*
     power2(snb)*power2(snt)*power2(log(mst12/Q2))*(msb22*mst14*(mt2 + mu2)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*(mt2 + mu2)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb22*(mt2 + mu2)*power2(
     mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + msb12*(mt2 +
     mu2)*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     mu2*power2(mw2)*(-(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(
     mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(msb12*msb22) + (mu2*power2(sb)*power2(log(
     mst12/Q2))*(mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - power2(
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(-mst14 + DeltaInv(mw2,msb22,mst12)*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/msb22 + (2*mu2*power2(sb)*power2(snt)*power2(
     log(mst12/Q2))*(mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-mst14 + DeltaInv(mw2,msb22,mst12)
     *(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/msb22 + (mu2*power2(cb)*power2(log(mst12/Q2))*
     (mst14*(-mst12 + mst22 + 2*mt2)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2)) + (mst12 - mst22 - 2*mt2)*power2(mC2)*(mst14*power2(mC2) + (-2*msb22
     + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - (mst12 - mst22)*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(-mst14 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) -
     2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(mst12))
     )))/(msb22*(mst12 - mst22)) + (power2(cb)*power2(snt)*power2(log(mst12/Q2)
     )*(mst14*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - (mt2
     + mu2)*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-mst14 + DeltaInv(mw2,
     msb22,mst12)*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2
     + power3(mst12)) + power4(mst12)))))/msb22 + (mu2*log(mst12/Q2)*power2(sb)
     *((5*mC2*mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - 6*mC2*
     power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 -
     msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - DeltaInv(mw2,msb22,
     mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,
     mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) + power3(
     mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*power2(
     mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(
     mst12)))))/(6.*msb22) + (mu2*log(mst12/Q2)*power2(sb)*power2(snt)*((5*mC2*
     mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - 6*mC2*power2(
     DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*
     mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - DeltaInv(mw2,msb22,mst12)*(-(
     msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,mst12)*(
     mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) + power3(mst12)) - 72*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(
     12.*msb22) + (mu2*log(mst12/Q2)*power2(sb)*(-((5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2)) + 6*mC2*power2(DeltaInv(msb22,mst12,mC2)
     )*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) + 72*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*
     mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) -
     mst14*(msb22 + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(
     mst14*mw2 + power3(mst12)) + power4(mst12)))))/(12.*msb22) + (mu2*log(
     mst12/Q2)*power2(sb)*power2(snt)*(-((5*mC2*mst12 - msb22*mst12 + mst14)*
     DeltaInv(msb22,mst12,mC2)) + 6*mC2*power2(DeltaInv(msb22,mst12,mC2))*(-3*
     mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(
     mst12)) + 72*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(
     mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,
     mC2)) + DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 -
     6*mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22
     + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 +
     power3(mst12)) + power4(mst12)))))/(6.*msb22) - (mu2*log(mst12/Q2)*power2(
     cb)*(-((5*mC2*mst12 - msb22*mst12 + mst14)*(mst12 - mst22 - 2*mt2)*
     DeltaInv(msb22,mst12,mC2)) + 6*mC2*(mst12 - mst22 - 2*mt2)*power2(
     DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*
     mst14 + mst12*power2(mC2) + power3(mst12)) + 72*(mst12 - mst22 - 2*mt2)*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + (mst12
     - mst22)*DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 -
     6*mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22
     + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 +
     power3(mst12)) + power4(mst12)))))/(12.*msb22*(mst12 - mst22)) - (log(
     mst12/Q2)*power2(cb)*power2(snt)*((5*mC2*mst12 - msb22*mst12 + mst14)*(mt2
     + mu2)*DeltaInv(msb22,mst12,mC2) - 6*mC2*(mt2 + mu2)*power2(DeltaInv(
     msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 72*(mt2 + mu2)*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - mu2*DeltaInv(mw2,
     msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,
     msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) +
     power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*
     power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) +
     power4(mst12)))))/(12.*msb22) + (mu2*log(msb12/Q2)*log(mst12/Q2)*power2(
     sb)*power2(snb)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb12 + (2*mu2*log(msb12/Q2)*log(mst12/Q2)*
     power2(sb)*power2(snb)*power2(snt)*((mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12
     - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(
     mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*
     mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(
     mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*
     power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/msb12 + (log(
     msb12/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*power2(snt)*((mC2*mst12 +
     msb12*mst12 + 3*mst14)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - (mt2 + mu2)*power2(mC2)*((-(msb12*mst12) + mst14)*power2(
     mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 -
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(
     mst12) - msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) +
     mst12*power3(mw2) + 3*power4(mst12)))))/msb12 + (mu2*log(msb22/Q2)*log(
     mst12/Q2)*power2(sb)*((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((msb22*mst12 - mst14)*
     power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*mst14 -
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*power3(
     mst12) - msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) +
     mst12*power3(mw2) + 3*power4(mst12)))))/msb22 + (2*mu2*log(msb22/Q2)*log(
     mst12/Q2)*power2(sb)*power2(snb)*((mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((msb22*mst12
     - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(
     mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*
     mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(
     mw2) - 5*mw2*power3(mst12) - msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*
     power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/msb22 + (2*mu2*
     log(msb22/Q2)*log(mst12/Q2)*power2(sb)*power2(snt)*((mC2*mst12 + msb22*
     mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + power2(
     mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 -
     3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     *(-(msb22*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*
     power2(mw2) - 5*mw2*power3(mst12) - msb22*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/msb22 +
     (mu2*log(msb22/Q2)*log(mst12/Q2)*power2(sb)*power2(snb)*power2(snt)*((mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2)) + power2(mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*power3(mC2)
     + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12))*(-(msb22*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) - msb22*(10*mst14*mw2 +
     mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)
     ))))/msb22 + (log(msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(snt)*((mC2*
     mst12 + msb22*mst12 + 3*mst14)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) - (mt2 + mu2)*power2(mC2)*((-(msb22*mst12) + mst14)*
     power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*
     mst14 - mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*
     power3(mst12) - msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12))
     + mst12*power3(mw2) + 3*power4(mst12)))))/msb22 - (log(msb22/Q2)*log(
     mst12/Q2)*power2(cb)*power2(snb)*power2(snt)*((mC2*mst12 + msb22*mst12 +
     3*mst14)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - (mt2
     + mu2)*power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(
     mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*mst14 - mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb22 + (log(msb22/Q2)*log(mst12/Q2)*power2(
     snb)*power2(snt)*((mC2*mst12 + msb22*mst12 + 3*mst14)*mt2*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) + mt2*power2(mC2)*((msb22*mst12 - mst14)
     *power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*
     mst14 - mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*
     power3(mst12) - msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12))
     + mst12*power3(mw2) + 3*power4(mst12)))))/msb22 - (mu2*log(mst12/Q2)*
     power2(cb)*power2(snb)*(-(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*(mst12
     - mst22 - 2*mt2)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*
     mst12 + mst14)*(mst12 - mst22 - 2*mt2)*DeltaInv(msb22,mst12,mC2) + 6*mC2*
     msb22*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 6*mC2*msb12*(mst12 - mst22 - 2*mt2)*power2(DeltaInv(msb22,mst12,
     mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2)
     + power3(mst12)) + 72*msb22*(mst12 - mst22 - 2*mt2)*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*(mst12 -
     mst22 - 2*mt2)*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(
     mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,
     mC2)) + (mst12 - mst22)*(msb22*(-(msb12*mst12) + mst14 + 5*mst12*mw2)*
     DeltaInv(mw2,msb12,mst12) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))*
     (mst12*mw2*(-3*msb12 + mw2) - mst14*(msb12 + 14*mw2) + power3(mst12)) -
     72*msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2)
     - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(
     mst12)) - msb12*DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*
     mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) -
     mst14*(msb22 + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(
     mst14*mw2 + power3(mst12)) + power4(mst12))))))/(12.*msb12*msb22*(mst12 -
     mst22)) + (log(mst12/Q2)*power2(cb)*power2(snb)*power2(snt)*(-(msb22*(5*
     mC2*mst12 - msb12*mst12 + mst14)*(mt2 + mu2)*DeltaInv(msb12,mst12,mC2)) +
     msb12*(5*mC2*mst12 - msb22*mst12 + mst14)*(mt2 + mu2)*DeltaInv(msb22,
     mst12,mC2) + 6*mC2*msb22*(mt2 + mu2)*power2(DeltaInv(msb12,mst12,mC2))*(-
     3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) +
     power3(mst12)) - 6*mC2*msb12*(mt2 + mu2)*power2(DeltaInv(msb22,mst12,mC2))
     *(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) + 72*msb22*(mt2 + mu2)*power2(mC2)*(mst14*power2(mC2) + (-
     2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*(mt2 + mu2)*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + mu2*(msb22*(-(msb12*
     mst12) + mst14 + 5*mst12*mw2)*DeltaInv(mw2,msb12,mst12) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(mst12*mw2*(-3*msb12 + mw2) - mst14*(
     msb12 + 14*mw2) + power3(mst12)) - 72*msb22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(
     mst14*mw2 + power3(mst12)) + power4(mst12)) - msb12*DeltaInv(mw2,msb22,
     mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,
     mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) + power3(
     mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*power2(
     mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(
     mst12))))))/(12.*msb12*msb22) + (mt2*mu2*power2(cb)*power2(mC2)*power2(Pi)
     *(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)
     )) + (-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-
     5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))
     *power3(DeltaInv(msb22,mst12,mC2)) + power2(DeltaInv(mst22,msb22,mC2))*(3*
     mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*(14*mC2*
     msb22*mst24 + 5*msb22*mst22*power2(mC2) + 3*mst24*power2(mC2) - 3*mst22*
     power3(mC2) + 3*mC2*power3(mst22) + 5*msb22*power3(mst22) - 3*power4(
     mst22)))))/(3.*msb22*(mst12 - mst22)) - (mu2*power2(mC2)*power2(Pi)*
     power2(sb)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     DeltaInv(msb22,mst12,mC2))) + (-((5*msb22*mst12 + 3*mst14)*power2(mC2)) +
     3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(
     DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + 3*mst24 +
     DeltaInv(mst22,msb22,mC2)*(14*mC2*msb22*mst24 + 5*msb22*mst22*power2(mC2)
     + 3*mst24*power2(mC2) - 3*mst22*power3(mC2) + 3*mC2*power3(mst22) + 5*
     msb22*power3(mst22) - 3*power4(mst22)))))/(6.*msb22*power2(cb)) + (mu2*
     power2(Pi)*power2(sb)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)
     *power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (3*mC2*mst22 + msb22*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(
     mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-
     5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))
     *power3(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((5*msb22*mst22 + 3*
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(
     mst22) + mC2*(14*msb22*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*(-((msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*
     mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(
     mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) -
     power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(6.*msb22) + (power2(
     Pi)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*mt2*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))) + (3*mC2*mst22 + msb22*mst22 + 3*mst24)
     *mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + mt2*power2(mC2)*(-((
     5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 +
     3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + mt2*power2(mC2)*((5*msb22*mst22 + 3*mst24)*
     power2(mC2) - 3*mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) +
     mC2*(14*msb22*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))
     + mu2*power2(mw2)*(-((msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*
     mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(
     mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) -
     power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(6.*msb22) + (mu2*
     power2(Pi)*power2(sb)*power2(snt)*((3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - (3*mC2*mst22 + msb22*mst22
     + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((
     5*msb22*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb22 - 3*
     mst12)*power3(mst12) + mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mC2)*(-((5*msb22*mst22 + 3*mst24)*
     power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) -
     mC2*(14*msb22*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))
     + power2(mw2)*((msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*
     mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*
     msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*
     mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2)
     + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*
     mst22*power3(mw2) - 3*power4(mst22)))))/(3.*msb22) + (power2(cb)*power2(
     Pi)*power2(snt)*((3*mC2*mst12 + msb22*mst12 + 3*mst14)*(mt2 + mu2)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) - (3*mC2*mst22 + msb22*mst22 + 3*
     mst24)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - (mt2 +
     mu2)*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + (mt2 + mu2)*power2(
     mC2)*(-((5*msb22*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-
     5*msb22 + 3*mst22)*power3(mst22) - mC2*(14*msb22*mst24 + 3*power3(mst22)))
     *power3(DeltaInv(mst22,msb22,mC2)) + mu2*power2(mw2)*((msb22*mst22 + 3*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,
     msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*
     power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*
     power3(mw2) - 3*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(14*
     msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*
     msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)))))/(6.*msb22) + (mu2*power2(sb)*power2(snt)*(mC2*msb22*
     DeltaInv(msb22,mst12,mC2) - mC2*msb22*DeltaInv(mst22,msb22,mC2) - 42*(3*
     mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) + 42*(3*mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 42*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) -
     mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     - 42*power2(mC2)*(-((5*msb22*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*
     power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - mC2*(14*msb22*mst24 +
     3*power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + mw2*(msb22*DeltaInv(
     mst22,mw2,msb22) - 42*mw2*(msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22)) + DeltaInv(mw2,msb22,mst12)*(-msb22 + 42*mw2*(
     msb22*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb22,mst12) + 42*mw2*
     power2(DeltaInv(mw2,msb22,mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*
     power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(
     mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) - 42*mw2*power3(DeltaInv(
     mst22,mw2,msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*
     mst24*power2(mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*
     power3(mw2) - 3*power4(mst22)))))/(6.*msb22) + (mu2*power2(sb)*power2(snt)
     *(-(mC2*msb22*DeltaInv(msb22,mst12,mC2)) + mC2*msb22*DeltaInv(mst22,msb22,
     mC2) + 42*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - 42*(3*mC2*mst22 + msb22*mst22 + 3*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 42*power2(mC2)*(-((5*
     msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 42*power2(mC2)*(-((5*msb22*mst22 + 3*mst24)*
     power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) -
     mC2*(14*msb22*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))
     + mw2*(-(msb22*DeltaInv(mst22,mw2,msb22)) + 42*mw2*(msb22*mst22 + 3*(mst24
     + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) - DeltaInv(mw2,msb22,
     mst12)*(-msb22 + 42*mw2*(msb22*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(
     mw2,msb22,mst12) + 42*mw2*power2(DeltaInv(mw2,msb22,mst12))*(14*msb22*
     mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*
     power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(
     mst12))) + 42*mw2*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 +
     5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) +
     3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(3.*msb22)
     + (2*mu2*log(msb12/Q2)*log(mst22/Q2)*power2(sb)*power2(snb)*power2(snt)*(-
     ((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2))) + power2(mC2)*((-(msb12*mst22) + mst24)*power2(mC2) + mst22*
     power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(msb12*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb12)*(10*msb12*mst24*mw2 + msb12*mst22*power2(mw2) - mst24*
     power2(mw2) + 5*msb12*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(
     mw2) - 3*power4(mst22)))))/msb12 + (2*mu2*log(msb22/Q2)*log(mst22/Q2)*
     power2(sb)*power2(snt)*(-((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*((-(msb22*mst22) + mst24)
     *power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*mst24*mw2 + msb22*mst22*
     power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(mst22) + 5*mw2*power3(
     mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/msb22 + (mu2*log(msb22/
     Q2)*log(mst22/Q2)*power2(sb)*power2(snb)*power2(snt)*(-((mC2*mst22 +
     msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))) +
     power2(mC2)*((-(msb22*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-
     5*msb22 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(
     10*msb22*mst24*mw2 + msb22*mst22*power2(mw2) - mst24*power2(mw2) + 5*
     msb22*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(
     mst22)))))/msb22 + (log(msb22/Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*(-
     ((mC2*mst22 + msb22*mst22 + 3*mst24)*mt2*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2))) + mt2*power2(mC2)*((-(msb22*mst22) + mst24)*power2(mC2)
     + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + mu2*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2
     + DeltaInv(mst22,mw2,msb22)*(10*msb22*mst24*mw2 + msb22*mst22*power2(mw2)
     - mst24*power2(mw2) + 5*msb22*power3(mst22) + 5*mw2*power3(mst22) - mst22*
     power3(mw2) - 3*power4(mst22)))))/msb22 + (mu2*power2(sb)*power2(snb)*
     power2(snt)*power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)
     *power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*
     mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(
     mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-
     3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((3*msb12*mst22 + 2*mst24)
     *power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) +
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*
     mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22)))))/msb12 + (power2(snb)*power2(snt)*
     power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*mt2*power2(
     mC2)*power2(DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*
     mst24)*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + mt2*power2(mC2)
     *(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + mt2*power2(mC2)*((3*msb12*mst22 + 2*
     mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + mu2*power2(mw2)*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*
     mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) - power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*
     msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*
     mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/msb12 + (2*
     mu2*power2(sb)*power2(snb)*power2(snt)*power2(log(msb12/Q2))*((2*mC2*mst12
     + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - (
     2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*
     mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*(-
     ((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12
     + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*((msb12*mst22 + 2*(mst24 + mst22*
     mw2))*power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12)
     )*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*
     msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 +
     3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) +
     2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/msb12 + (
     power2(cb)*power2(snb)*power2(snt)*power2(log(msb12/Q2))*((2*mC2*mst12 +
     msb12*mst12 + 2*mst14)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - (2*mC2*mst22 + msb12*mst22 + 2*mst24)*(mt2 + mu2)*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) - (mt2 + mu2)*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + (mt2 + mu2)*power2(mC2)*(-((3*msb12*mst22 +
     2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + mu2*power2(mw2)*((msb12*mst22 + 2*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(
     msb12*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*
     mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*
     msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*
     mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/msb12 + (
     mu2*power2(sb)*power2(snt)*power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*
     mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*
     mst22 + msb22*mst22 + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2)) + power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((3*msb22*
     mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*
     power3(mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + power2(mw2)*(-((msb22*mst22 + 2*(mst24 + mst22*mw2))*
     power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*
     power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(
     mst12))) - power3(DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*
     msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*
     mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*power4(mst22)))))/msb22 + (2*
     mu2*power2(sb)*power2(snb)*power2(snt)*power2(log(msb22/Q2))*(-((2*mC2*
     mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) +
     (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))
     *power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-((msb22*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,
     msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*(10*
     msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (power2(cb)*power2(snb)*power2(snt)*power2(log(
     msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*(mt2 + mu2)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*
     mst24)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + (mt2 +
     mu2)*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - (mt2 + mu2)*power2(
     mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + mu2*power2(mw2)*(-((msb22*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,
     msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*(10*
     msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (power2(snt)*power2(log(msb22/Q2))*(-((2*mC2*
     mst12 + msb22*mst12 + 2*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + mt2*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + mt2*power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + mu2*power2(
     mw2)*(-((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,
     mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*
     power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/msb22 + (2*mu2*power2(sb)*power2(snt)*
     power2(log(msb22/Q2))*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 + msb22*mst22 + 2*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((3*msb22*
     mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*
     power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2))
     + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*
     msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(
     mw2)*((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,
     msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,
     mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*
     power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/msb22 + (mu2*power2(sb)*power2(snb)*
     power2(snt)*power2(log(msb22/Q2))*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 + msb22*mst22
     + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((
     3*msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*
     mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     power2(mw2)*((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,
     mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 +
     2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*
     mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/msb22 + (power2(snb)*power2(snt)*power2(
     log(msb22/Q2))*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*mt2*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 + msb22*mst22 + 2*mst24)*
     mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + mt2*power2(mC2)*((3*
     msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*
     mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + mt2*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     mu2*power2(mw2)*((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*
     mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*
     msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb22))*(10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22)))))/msb22 + (power2(cb)*power2(snt)*
     power2(log(msb22/Q2))*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*(mt2 + mu2)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 + msb22*mst22
     + 2*mst24)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - (
     mt2 + mu2)*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + (mt2 + mu2)*
     power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + mu2*power2(mw2)*((msb22*mst22
     + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) - power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (2*mu2*power2(sb)*power2(snb)*power2(snt)*
     power2(log(mst22/Q2))*(-(msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2))) + msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) +
     msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) -
     2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) -
     msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     power2(mw2)*(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(
     mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) -
     power4(mst22)))))/(msb12*msb22) + (power2(cb)*power2(snb)*power2(snt)*
     power2(log(mst22/Q2))*(-(msb22*mst24*(mt2 + mu2)*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))) + msb12*mst24*(mt2 + mu2)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + msb22*(mt2 + mu2)*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - msb12*(mt2 + mu2)*power2(mC2)
     *(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + mu2*power2(
     mw2)*(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(
     mw2)) + 2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) -
     power4(mst22)))))/(msb12*msb22) + (2*mu2*power2(sb)*power2(snt)*power2(
     log(mst22/Q2))*(-(mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))) +
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*
     (-(mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(
     mst22)) - power4(mst22)))))/msb22 + (power2(cb)*power2(snt)*power2(log(
     mst22/Q2))*(-(mst24*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))) + (mt2 + mu2)*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(
     mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2)) + 2*mw2*power3(
     mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(mst22)))))/msb22 + (
     mu2*log(msb12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(
     snt)*((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(5*
     mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb12 + (2*mu2*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(sb)*power2(
     snb)*power2(snt)*((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14
     + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*
     power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12))
     - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12)
     )*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*mst24*mw2 + 5*
     mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22)))
     )/msb12 + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(
     snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)
     ) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(5*
     mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb22 + (2*mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(sb)*power2(
     snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)
     ) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(5*
     mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb22 + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)
     *((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(5*
     mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb22 + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(sb)*power2(snb)*
     power2(snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(
     5*mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb22 + (mu2*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*power2(snt)
     *(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12)
     ) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*
     mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) +
     msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(
     mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*
     power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22) - (
     mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(
     snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(
     mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 + mst22*power2(
     mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/(msb12*msb22)
     - (mu2*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(sb)*power2(snb)*
     power2(snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*
     power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) -
     mst22*power3(mw2) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*(6*mst24*mw2 +
     mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/
     (msb12*msb22) + (mu2*log(mst22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snt)*
     power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 + mst22*mw2 +
     DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(mst22) + msb22*(
     6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/msb22 - (mu2*log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(
     mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + mst24 +
     mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*power3(
     mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*
     power3(mw2) - power4(mst22))))/msb22 - (mu2*log(mst22/Q2)*log(mw2/Q2)*
     power2(mw2)*power2(sb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))))/msb22 + (mt2*mu2*
     power2(mC2)*power2(Pi)*power2(snb)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*
     mst14)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*
     mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 3*mC2*msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2)) - 3*msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + 3*
     mC2*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*
     power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(DeltaInv(mst22,
     msb22,mC2)) + msb22*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*
     power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*mC2*msb12*msb22*
     mst24*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 3*mC2*msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst22*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 3*mC2*msb12*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) + 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*(
     mst12 - mst22)) + (mt2*mu2*power2(cb)*power2(mC2)*power2(Pi)*power2(snb)*(
     msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,
     mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,
     mst12,mC2)) - 3*mC2*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*
     msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(
     DeltaInv(mst22,msb12,mC2)) + 3*mC2*msb12*mst22*power2(DeltaInv(mst22,
     msb22,mC2)) + msb12*msb22*mst22*power2(DeltaInv(mst22,msb22,mC2)) + 3*
     msb12*mst24*power2(DeltaInv(mst22,msb22,mC2)) + msb22*((5*msb12*mst12 + 3*
     mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(
     mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) - 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) -
     5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst22*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*mC2*msb22*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb22,mC2)) + 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 3*msb12*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     - 3*msb12*mst22*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*mC2*
     msb12*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + msb12*power3(DeltaInv(
     msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*
     mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*
     power3(mst12) + 3*power4(mst12)) + 3*msb22*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) - 3*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mst22)))/(3.*msb12*msb22*(mst12 - mst22)) + (mu2*power2(mC2)*power2(Pi)*
     power2(sb)*power2(snb)*power2(snt)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*
     mst14)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*
     mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 3*mC2*msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2)) - 3*msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + 3*
     mC2*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*
     power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(DeltaInv(mst22,
     msb22,mC2)) + msb22*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*
     power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*mC2*msb12*msb22*
     mst24*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 3*mC2*msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst22*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 3*mC2*msb12*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) + 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(6.*msb12*msb22*
     power2(cb)) + (mt2*mu2*power2(mC2)*power2(Pi)*power2(snb)*(-(msb22*(3*mC2*
     mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*
     (3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) +
     3*mC2*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) - 3*mC2*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - msb12*
     msb22*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*power2(
     DeltaInv(mst22,msb22,mC2)) + msb22*(-((5*msb12*mst12 + 3*mst14)*power2(
     mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(
     14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 14*
     mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*
     mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst22*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 3*mC2*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)) - 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,
     mC2)) - 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     - 3*msb12*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*
     mst22*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*mC2*msb12*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2)
     )*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2) + 3*mst14*power2(mC2) -
     3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*msb22*power3(mst12) - 3*
     power4(mst12)) - 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) +
     3*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*
     (mst12 - mst22)) + (2*mt2*mu2*power2(cb)*power2(mC2)*power2(Pi)*power2(
     snb)*(-(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     DeltaInv(msb22,mst12,mC2)) + 3*mC2*msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + 3*
     msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 3*mC2*msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst22*power2(DeltaInv(mst22,
     msb22,mC2)) - 3*msb12*mst24*power2(DeltaInv(mst22,msb22,mC2)) + msb22*(-((
     5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 +
     3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) + 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) + 3*msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     - 3*msb22*mst22*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*mC2*
     msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 14*mC2*msb12*msb22*
     mst24*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - 3*mC2*msb12*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     msb12*power3(DeltaInv(msb22,mst12,mC2))*(14*mC2*msb22*mst14 + 5*msb22*
     mst12*power2(mC2) + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*
     power3(mst12) + 5*msb22*power3(mst12) - 3*power4(mst12)) - 3*msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*(mst12 - mst22)) - (mC2*mt2*
     mu2*power2(cb)*power2(snb)*(-(msb12*msb22*DeltaInv(msb12,mst12,mC2)) +
     msb12*msb22*DeltaInv(msb22,mst12,mC2) + msb12*msb22*DeltaInv(mst22,msb12,
     mC2) - msb12*msb22*DeltaInv(mst22,msb22,mC2) + 42*mC2*msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2)) - 42*mC2*msb12*(
     3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) -
     42*mC2*msb12*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - 126*mC2*
     msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst22*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*mC2*msb12*msb22*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 126*mC2*msb12*mst24*power2(DeltaInv(mst22,
     msb22,mC2)) + 126*msb12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)
     ) - 42*mC2*msb22*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*
     mst22*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*mC2*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*mst22*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*mC2*msb12*msb22*power3(mst22)
     *power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power4(mC2) + 42*mC2*msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) + 126*mC2*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     126*mC2*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*msb12*
     msb22*(mst12 - mst22)) + (mC2*mu2*power2(sb)*power2(snb)*power2(snt)*(-(
     msb12*msb22*DeltaInv(msb12,mst12,mC2)) + msb12*msb22*DeltaInv(msb22,mst12,
     mC2) + msb12*msb22*DeltaInv(mst22,msb12,mC2) - msb12*msb22*DeltaInv(mst22,
     msb22,mC2) + 42*mC2*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(
     DeltaInv(msb12,mst12,mC2)) - 42*mC2*msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 42*mC2*msb12*msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2)) - 126*mC2*msb22*mst24*power2(DeltaInv(
     mst22,msb12,mC2)) - 126*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + 42*mC2*msb12*msb22*mst22*power2(DeltaInv(mst22,msb22,mC2)) +
     126*mC2*msb12*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 42*mC2*msb22*(-((5*msb12*
     mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)
     *power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 588*msb12*msb22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 210*msb12*msb22*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 126*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 210*mC2*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) - 126*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) + 588*msb12*msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 210*msb12*msb22*mst22*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 126*msb12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     ) + 210*mC2*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     126*msb12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     126*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*msb12*
     mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 42*mC2*msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(
     mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) -
     5*msb22*power3(mst12) + 3*power4(mst12)) + 126*mC2*msb22*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) - 126*mC2*msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22)))/(6.*msb12*msb22*power2(cb)) + (log(mst22/Q2)*
     power2(snt)*(mst22*(-2*msb22 + mst22 - mu2) - (5*mC2*mst22 - msb22*mst22 +
     mst24)*mt2*(mst22 - mu2)*DeltaInv(mst22,msb22,mC2) - (mst22 - mu2)*mu2*(
     msb22*mst22 - mst24 - 5*mst22*mw2)*DeltaInv(mst22,mw2,msb22) - 6*(mst22 -
     mu2)*mu2*mw2*power2(DeltaInv(mst22,mw2,msb22))*(mst22*mw2*(-3*msb22 + mw2)
     - mst24*(msb22 + 14*mw2) + power3(mst22)) + 6*mC2*mt2*(mst22 - mu2)*
     power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 -
     msb22*mst24 + mst22*power2(mC2) + power3(mst22)) + 72*mt2*(mst22 - mu2)*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) - 72*(
     mst22 - mu2)*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mst24*
     power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) +
     power4(mst22))))/(12.*msb22*(mst22 - mu2)) + (mu2*log(msb12/Q2)*log(mw2/
     Q2)*power2(mw2)*power2(snb)*power2(snt)*(-((msb12*mst22 + mst24 + 3*mst22*
     mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12)
     )*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*
     power3(mw2) + power4(mst22))))/msb12 + (mu2*log(msb12/Q2)*log(mw2/Q2)*
     power2(mw2)*power2(sb)*power2(snb)*power2(snt)*(-((msb12*mst22 + mst24 +
     3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb12))) + power2(DeltaInv(mw2,
     msb12,mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/msb12 + (mu2*log(msb22/Q2)*log(mw2/
     Q2)*power2(mw2)*power2(snt)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(
     mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) +
     power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) +
     power4(mst22))))/msb22 + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(sb)*power2(snt)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(
     mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) +
     power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) +
     power4(mst22))))/msb22 + (mu2*log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(
     mw2)*power2(snb)*power2(snt)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*
     power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(
     msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*
     power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(
     mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(
     DeltaInv(mst22,mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(mst22) -
     msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*
     power3(mw2) + power4(mst22))))/msb22 + (2*mu2*log(msb22/Q2)*log(mw2/Q2)*
     power2(mw2)*power2(sb)*power2(snb)*power2(snt)*(-((msb22*mst22 + mst24 +
     3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,
     msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     + power3(DeltaInv(mst22,mw2,msb22))*(-5*mst24*power2(mw2) + mw2*power3(
     mst22) - msb22*(10*mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*
     mst22*power3(mw2) + power4(mst22))))/msb22 - (mt2*mu2*power2(cb)*(msb22*(-
     mst12 + mst22) - 2*mC2*msb22*mst12*mst22*DeltaInv(msb22,mst12,mC2) + 2*
     mC2*msb22*mst12*mst22*DeltaInv(mst22,msb22,mC2) + 84*mst12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*mst22*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))
     - 84*mst12*mst22*(3*mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 84*mst12*mst22*power2(mC2)*(-((5*msb22*mst12
     + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*
     power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + 84*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*(-14*mC2*msb22*mst24 - 5*msb22*mst22*power2(mC2) - 3*mst24*
     power2(mC2) + 3*mst22*power3(mC2) - 3*mC2*power3(mst22) - 5*msb22*power3(
     mst22) + 3*power4(mst22))))/(6.*msb22*mst12*(mst12 - mst22)*mst22) + (mu2*
     power2(sb)*power2(snt)*(msb22*(-mst12 + mst22) - 2*mC2*msb22*mst12*mst22*
     DeltaInv(msb22,mst12,mC2) + 2*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) + 84*mst12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mst22*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - 84*mst12*mst22*(3*mC2*mst22 + msb22*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 84*mst12*
     mst22*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 84*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-14*mC2*msb22*mst24 - 5*
     msb22*mst22*power2(mC2) - 3*mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*
     mC2*power3(mst22) - 5*msb22*power3(mst22) + 3*power4(mst22))))/(12.*msb22*
     mst12*mst22*power2(cb)) + (mu2*power2(sb)*power2(snb)*power2(snt)*power2(
     log(mst22/Q2))*(msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     - msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb22*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + msb12*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(
     mw2)*(-(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12))) + msb12*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) -
     power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(msb12*msb22) + (power2(snb)*power2(snt)*power2(log(mst22/Q2))*
     (msb22*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*
     mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb22*mt2*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + msb12*
     mt2*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*
     mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     mu2*power2(mw2)*(-(msb22*mst24*power2(DeltaInv(mst22,mw2,msb12))) + msb12*
     power2(DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(
     mst24*power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(
     mst22)) - power4(mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(
     mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(
     mst22)) + power4(mst22)))))/(msb12*msb22) + (mu2*power2(sb)*power2(snt)*
     power2(log(mst22/Q2))*(mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))
     - power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*
     mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-mst24 + DeltaInv(mst22,
     mw2,msb22)*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 +
     power3(mst22)) + power4(mst22)))))/msb22 + (power2(snt)*power2(log(mst22/
     Q2))*(mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - mt2*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*
     (msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + mu2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-mst24 + DeltaInv(mst22,
     mw2,msb22)*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 +
     power3(mst22)) + power4(mst22)))))/msb22 + (mu2*log(mst22/Q2)*power2(sb)*
     power2(snt)*((5*mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2)
     - 6*mC2*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*
     mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(mC2)*
     (mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) - DeltaInv(mst22,mw2,
     msb22)*(-(msb22*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,
     msb22)*(mst22*mw2*(-3*msb22 + mw2) - mst24*(msb22 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(6.*msb22) + (mu2*log(mst22/Q2)*power2(sb)*power2(snt)*(-((5*
     mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2)) + 6*mC2*
     power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 -
     msb22*mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + DeltaInv(mst22,mw2,
     msb22)*(-(msb22*mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,
     msb22)*(mst22*mw2*(-3*msb22 + mw2) - mst24*(msb22 + 14*mw2) + power3(
     mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(mst24*power2(
     mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(
     mst22)))))/(12.*msb22) + (log(mst22/Q2)*power2(cb)*power2(snt)*((5*mC2*
     mst22 - msb22*mst22 + mst24)*(mt2 + mu2)*DeltaInv(mst22,msb22,mC2) - 6*
     mC2*(mt2 + mu2)*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 -
     14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*(mt2
     + mu2)*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) -
     mu2*DeltaInv(mst22,mw2,msb22)*(-(msb22*mst22) + mst24 + 5*mst22*mw2 - 6*
     mw2*DeltaInv(mst22,mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2) - mst24*(msb22 +
     14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)
     )*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(
     mst22)) + power4(mst22)))))/(12.*msb22) + (mu2*log(msb12/Q2)*log(mst22/Q2)
     *power2(sb)*power2(snb)*power2(snt)*((mC2*mst22 + msb12*mst22 + 3*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((msb12*mst22
     - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(
     mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-(msb12*
     mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(
     mw2) - 5*mw2*power3(mst22) - msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*
     power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/msb12 - (log(
     msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*power2(snt)*((mC2*mst22 +
     msb12*mst22 + 3*mst24)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - (mt2 + mu2)*power2(mC2)*((-(msb12*mst22) + mst24)*power2(
     mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(mst22) - 5*mC2*(2*
     msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + mu2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-(msb12*mst22) - 3*mst24 -
     mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 5*mw2*power3(
     mst22) - msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) +
     mst22*power3(mw2) + 3*power4(mst22)))))/msb12 + (log(msb12/Q2)*log(mst22/
     Q2)*power2(snb)*power2(snt)*((mC2*mst22 + msb12*mst22 + 3*mst24)*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + mt2*power2(mC2)*((msb12*
     mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*
     power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(-(
     msb12*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*
     power2(mw2) - 5*mw2*power3(mst22) - msb12*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/msb12 +
     (mu2*log(msb22/Q2)*log(mst22/Q2)*power2(sb)*power2(snt)*((mC2*mst22 +
     msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) +
     power2(mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*
     msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22))*(-(msb22*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb22)*
     (mst24*power2(mw2) - 5*mw2*power3(mst22) - msb22*(10*mst24*mw2 + mst22*
     power2(mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/
     msb22 + (2*mu2*log(msb22/Q2)*log(mst22/Q2)*power2(sb)*power2(snb)*power2(
     snt)*((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + power2(mC2)*((msb22*mst22 - mst24)*power2(mC2) -
     mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22))*(-(msb22*mst22) - 3*mst24 - mst22*mw2 +
     DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) - 5*mw2*power3(mst22) -
     msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(
     mw2) + 3*power4(mst22)))))/msb22 - (log(msb22/Q2)*log(mst22/Q2)*power2(cb)
     *power2(snt)*((mC2*mst22 + msb22*mst22 + 3*mst24)*(mt2 + mu2)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) - (mt2 + mu2)*power2(mC2)*((-(msb22*
     mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*
     power3(mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-(
     msb22*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*
     power2(mw2) - 5*mw2*power3(mst22) - msb22*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/msb22 +
     (log(msb22/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*power2(snt)*((mC2*
     mst22 + msb22*mst22 + 3*mst24)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) - (mt2 + mu2)*power2(mC2)*((-(msb22*mst22) + mst24)*
     power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - 5*
     mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) +
     mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-(msb22*mst22) - 3*
     mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) - 5*mw2*
     power3(mst22) - msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22))
     + mst22*power3(mw2) + 3*power4(mst22)))))/msb22 + (log(msb22/Q2)*log(
     mst22/Q2)*power2(snt)*((mC2*mst22 + msb22*mst22 + 3*mst24)*mt2*power2(mC2)
     *power2(DeltaInv(mst22,msb22,mC2)) + mt2*power2(mC2)*((msb22*mst22 -
     mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22)
     + 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))
     + mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-(msb22*mst22) - 3*
     mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) - 5*mw2*
     power3(mst22) - msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22))
     + mst22*power3(mw2) + 3*power4(mst22)))))/msb22 + (log(mst22/Q2)*power2(
     cb)*power2(snb)*power2(snt)*(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*(
     mt2 + mu2)*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*mst22 - msb22*mst22 +
     mst24)*(mt2 + mu2)*DeltaInv(mst22,msb22,mC2) - 6*mC2*msb22*(mt2 + mu2)*
     power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 -
     msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + 6*mC2*msb12*(mt2 + mu2)
     *power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 -
     msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*msb22*(mt2 + mu2)*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 72*
     msb12*(mt2 + mu2)*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + mu2*(msb22*(msb12*mst22 - mst24 - 5*mst22*mw2)*
     DeltaInv(mst22,mw2,msb12) + 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*
     (mst22*mw2*(-3*msb12 + mw2) - mst24*(msb12 + 14*mw2) + power3(mst22)) +
     72*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2)
     - 2*mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(
     mst22)) + msb12*DeltaInv(mst22,mw2,msb22)*(-(msb22*mst22) + mst24 + 5*
     mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2) -
     mst24*(msb22 + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(
     mst24*mw2 + power3(mst22)) + power4(mst22))))))/(12.*msb12*msb22) + (mu2*
     log(mst22/Q2)*power2(sb)*power2(snb)*power2(snt)*(msb22*(5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*mst22 -
     msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) + msb12*msb22*mst22*
     DeltaInv(mst22,mw2,msb12) - msb22*mst24*DeltaInv(mst22,mw2,msb12) - 5*
     msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*DeltaInv(
     mst22,mw2,msb22) + msb12*mst24*DeltaInv(mst22,mw2,msb22) + 5*msb12*mst22*
     mw2*DeltaInv(mst22,mw2,msb22) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) - 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 84*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 18*
     msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 84*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mw2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*msb12*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mst22) - 6*mC2*msb22*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) + 6*mC2*msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) + 6*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 6*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)
     *power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + 72*msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb22*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 72*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 72*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 72*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 72*msb12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2)))/(6.*msb12*msb22) + (mu2*log(mst22/Q2)*power2(sb)*
     power2(snb)*power2(snt)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*
     DeltaInv(mst22,msb12,mC2)) + msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*
     DeltaInv(mst22,msb22,mC2) - msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) +
     msb22*mst24*DeltaInv(mst22,mw2,msb12) + 5*msb22*mst22*mw2*DeltaInv(mst22,
     mw2,msb12) + msb12*msb22*mst22*DeltaInv(mst22,mw2,msb22) - msb12*mst24*
     DeltaInv(mst22,mw2,msb22) - 5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) +
     6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 18*msb12*
     msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 84*msb22*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*mst24*
     mw2*power2(DeltaInv(mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mst22) + 6*msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(
     mst22) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22
     - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*mC2*
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24
     - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(12.*msb12*msb22) - (log(mst22/
     Q2)*power2(snb)*power2(snt)*(msb12*mst22 - msb22*mst22 + msb22*(5*mC2*
     mst22 - msb12*mst22 + mst24)*mt2*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*
     mst22 - msb22*mst22 + mst24)*mt2*DeltaInv(mst22,msb22,mC2) + msb12*msb22*
     mst22*mu2*DeltaInv(mst22,mw2,msb12) - msb22*mst24*mu2*DeltaInv(mst22,mw2,
     msb12) - 5*msb22*mst22*mu2*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*
     mst22*mu2*DeltaInv(mst22,mw2,msb22) + msb12*mst24*mu2*DeltaInv(mst22,mw2,
     msb22) + 5*msb12*mst22*mu2*mw2*DeltaInv(mst22,mw2,msb22) - 6*msb12*msb22*
     mst24*mu2*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 18*msb12*msb22*mst22*
     mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 84*msb22*mst24*mu2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*msb22*mst24*mu2*
     mw2*power2(DeltaInv(mst22,mw2,msb22)) + 18*msb12*msb22*mst22*mu2*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 84*msb12*mst24*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mu2*mw2*power2(DeltaInv(mst22,
     mw2,msb12))*power3(mst22) - 6*msb12*mu2*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power3(mst22) - 6*mC2*msb22*mt2*power2(DeltaInv(mst22,msb12,mC2))*
     (-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) +
     power3(mst22)) + 6*mC2*msb12*mt2*power2(DeltaInv(mst22,msb22,mC2))*(-3*
     mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(
     mst22)) + 6*msb22*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     - 6*msb12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 72*
     msb22*mt2*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2)) + 72*msb12*mt2*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) - 144*msb12*msb22*mu2*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*mst24*mu2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb12)) - 144*msb22*mu2*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mu2*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*msb22*mst24*
     mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*mu2*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 72*msb22*mu2*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 72*msb12*
     mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 72*
     msb22*mst24*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 72*msb12*
     mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(12.*msb12*
     msb22) + (mu2*log(mst12/Q2)*power2(sb)*power2(snb)*(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12 -
     msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*
     DeltaInv(mw2,msb12,mst12) - msb22*mst14*DeltaInv(mw2,msb12,mst12) - 5*
     msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) + msb12*mst14*DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) - 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) - 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) - 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) + 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) + 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) - 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) - 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22) + (mu2*log(mst12/Q2)*power2(sb)*
     power2(snb)*power2(snt)*(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12 - msb22*mst12 + mst14)*
     DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*DeltaInv(mw2,msb12,mst12) -
     msb22*mst14*DeltaInv(mw2,msb12,mst12) - 5*msb22*mst12*mw2*DeltaInv(mw2,
     msb12,mst12) - msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) + msb12*mst14*
     DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*mw2*DeltaInv(mw2,msb22,mst12) -
     6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 18*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb22*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst14*
     mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 84*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))
     *power3(mst12) - 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mst12) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12
     - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + 6*mC2*
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14
     - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 6*msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 6*msb12*mst12*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) - 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + 72*msb12*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 144*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*msb22*
     mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 72*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb12*mst14*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(12.*msb12*msb22) + (mu2*log(
     mst12/Q2)*power2(sb)*power2(snb)*(-(msb22*(5*mC2*mst12 - msb12*mst12 +
     mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*DeltaInv(mw2,msb12,
     mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) -
     msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mw2*DeltaInv(mw2,
     msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) +
     18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst14*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) + 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) -
     6*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*msb22*power2(mC2)
     *(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) -
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*
     mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(12.*msb12*msb22) +
     (mu2*log(mst12/Q2)*power2(sb)*power2(snb)*power2(snt)*(-(msb22*(5*mC2*
     mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*
     mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*
     mst12*DeltaInv(mw2,msb12,mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) +
     5*msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22) + (log(mst12/Q2)*power2(snb)*
     power2(snt)*(msb12*mst12 - msb22*mst12 + msb22*(5*mC2*mst12 - msb12*mst12
     + mst14)*mt2*DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12 - msb22*mst12
     + mst14)*mt2*DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*mu2*DeltaInv(
     mw2,msb12,mst12) - msb22*mst14*mu2*DeltaInv(mw2,msb12,mst12) - 5*msb22*
     mst12*mu2*mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*mst12*mu2*DeltaInv(
     mw2,msb22,mst12) + msb12*mst14*mu2*DeltaInv(mw2,msb22,mst12) + 5*msb12*
     mst12*mu2*mw2*DeltaInv(mw2,msb22,mst12) - 6*msb12*msb22*mst14*mu2*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) - 18*msb12*msb22*mst12*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - 84*msb22*mst14*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst14*mu2*mw2*power2(DeltaInv(
     mw2,msb22,mst12)) + 18*msb12*msb22*mst12*mu2*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) + 84*msb12*mst14*mu2*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) + 6*msb22*mu2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mst12) - 6*msb12*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(mst12) -
     6*mC2*msb22*mt2*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 -
     14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + 6*mC2*
     msb12*mt2*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*
     mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 6*msb22*mst12*
     mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 6*msb12*mst12*mu2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*msb22*mt2*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*msb12*mt2*power2(
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 144*msb12*
     msb22*mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst14*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb22*mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     + 144*msb12*msb22*mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,
     mst12)) + 144*msb12*msb22*mst14*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + 144*msb12*mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) + 72*msb22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     *power4(mst12) - 72*msb12*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mst12) + 72*msb22*mst14*mu2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 72*msb12*mst14*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2)))/(12.*msb12*msb22) - (2*mu2*log(msb12/Q2)*log(msb22/Q2)*power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22))*(mh2*msb22 + msb12*msb22 + 3*msb24 +
     DeltaInv(mh2,msb12,msb22)*((msb12*msb22 - msb24)*power2(mh2) - msb22*
     power3(mh2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mh2*(2*msb12*msb24 +
     power3(msb22))))*power4(snb))/msb12 - (2*mu2*log(msb12/Q2)*log(msb22/Q2)*
     power2(mH2)*power2(sb)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 +
     msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 - msb24)*
     power2(mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*
     mH2*(2*msb12*msb24 + power3(msb22))))*power4(snb))/(msb12*power2(cb)) - (
     mh2*mu2*power2(log(msb12/Q2))*(msb12*msb14*power2(DeltaInv(mh2,msb12,
     msb12)) + 2*mh2*power2(DeltaInv(mh2,msb12,msb22))*(2*mh2*msb22 + msb12*
     msb22 + 2*msb24 + DeltaInv(mh2,msb12,msb22)*((3*msb12*msb22 + 2*msb24)*
     power2(mh2) - 2*msb22*power3(mh2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*
     mh2*(5*msb12*msb24 + power3(msb22)))))*power4(snb))/msb12 - (mH2*mu2*
     power2(sb)*power2(log(msb12/Q2))*(msb12*msb14*power2(DeltaInv(mH2,msb12,
     msb12)) + 2*mH2*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*msb22 + msb12*
     msb22 + 2*msb24 + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22 + 2*msb24)*
     power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*
     mH2*(5*msb12*msb24 + power3(msb22)))))*power4(snb))/(msb12*power2(cb)) - (
     mh2*mu2*power2(log(msb22/Q2))*(2*mh2*msb24*power2(DeltaInv(mh2,msb12,
     msb22)) + msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - 2*mh2*(msb24*
     power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)))*power4(snb))/msb12 + (
     4*ca*mu2*sa*sb*log(msb12/Q2)*log(msb22/Q2)*(-((mh2*msb22 + msb12*msb22 +
     3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*msb22 + 3*msb24 +
     DeltaInv(mH2,msb12,msb22)*((msb12*msb22 - msb24)*power2(mH2) - msb22*
     power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 +
     power3(msb22)))) + power2(mh2)*((-(msb12*msb22) + msb24)*power2(mh2) +
     msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - 5*mh2*(2*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)))*power4(snb))/(
     cb*msb12) + (2*mu2*log(msb12/Q2)*log(msb22/Q2)*power2(sa)*power2(sb)*(-((
     mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 +
     msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 - msb24)*
     power2(mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*
     mH2*(2*msb12*msb24 + power3(msb22)))) + power2(mh2)*((-(msb12*msb22) +
     msb24)*power2(mh2) + msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(
     msb22) - 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22)))*power4(snb))/(msb12*power2(cb)) + (2*mu2*log(msb12/Q2)*log(msb22/
     Q2)*power2(sa)*((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))
     *(-(mH2*msb22) - msb12*msb22 - 3*msb24 + DeltaInv(mH2,msb12,msb22)*((-(
     msb12*msb22) + msb24)*power2(mH2) + msb22*power3(mH2) + (-5*msb12 + 3*
     msb22)*power3(msb22) - 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(
     mh2)*((msb12*msb22 - msb24)*power2(mh2) - msb22*power3(mh2) + (5*msb12 -
     3*msb22)*power3(msb22) + 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)))*power4(snb))/msb12 + (2*ca*mu2*sa*sb*power2(
     log(msb12/Q2))*(-(mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,msb12))) - 2*(
     2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + mH2*(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*
     mH2*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*msb22 + msb12*msb22 + 2*msb24
     + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22 + 2*msb24)*power2(mH2) - 2*
     msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*mH2*(5*msb12*
     msb24 + power3(msb22))))) + 2*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*
     power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) -
     2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)))*
     power4(snb))/(cb*msb12) + (mu2*power2(sa)*power2(sb)*power2(log(msb12/Q2))
     *(-(mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,msb12))) - 2*(2*mh2*msb22 +
     msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) +
     mH2*(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*mH2*power2(
     DeltaInv(mH2,msb12,msb22))*(2*mH2*msb22 + msb12*msb22 + 2*msb24 +
     DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22 + 2*msb24)*power2(mH2) - 2*
     msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*mH2*(5*msb12*
     msb24 + power3(msb22))))) + 2*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*
     power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) -
     2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)))*
     power4(snb))/(msb12*power2(cb)) + (mu2*power2(sa)*power2(log(msb12/Q2))*(
     mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,msb12)) + 2*(2*mh2*msb22 +
     msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) +
     mH2*(-(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12))) + 2*mH2*power2(
     DeltaInv(mH2,msb12,msb22))*(-2*mH2*msb22 - msb12*msb22 - 2*msb24 +
     DeltaInv(mH2,msb12,msb22)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*
     msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22))))) + 2*power2(mh2)*((3*msb12*msb22 + 2*msb24)*
     power2(mh2) - 2*msb22*power3(mh2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*
     mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)))*
     power4(snb))/msb12 - (mH2*mu2*power2(sb)*power2(log(msb22/Q2))*(2*mH2*
     msb24*power2(DeltaInv(mH2,msb12,msb22)) + msb12*msb24*power2(DeltaInv(mH2,
     msb22,msb22)) - 2*mH2*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,
     msb22)))*power4(snb))/(msb12*power2(cb)) + (2*ca*mu2*sa*sb*power2(log(
     msb22/Q2))*(-2*msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) - mh2*
     msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) + 2*power2(mh2)*(msb24*
     power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + mH2*(2*mH2*msb24*
     power2(DeltaInv(mH2,msb12,msb22)) + msb12*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) - 2*mH2*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*
     mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))*
     power4(snb))/(cb*msb12) + (mu2*power2(sa)*power2(sb)*power2(log(msb22/Q2))
     *(-2*msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) - mh2*msb12*
     msb24*power2(DeltaInv(mh2,msb22,msb22)) + 2*power2(mh2)*(msb24*power2(mh2)
     + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + mH2*(2*mH2*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) + msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) - 2*mH2*(
     msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))*power4(snb))/(msb12*
     power2(cb)) + (mu2*power2(sa)*power2(log(msb22/Q2))*(2*msb24*power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(mh2,
     msb22,msb22)) - 2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22)) + mH2*(-2*mH2*msb24*power2(DeltaInv(mH2,msb12,msb22)) -
     msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 2*mH2*(msb24*power2(mH2) +
     (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22))))*power4(snb))/msb12 - (log(msb22/Q2)*(
     4*(msb12 - msb22) - (5*mh2*msb22 - msb12*msb22 + msb24)*mu2*DeltaInv(mh2,
     msb12,msb22) - msb12*(mh2 - 6*msb22)*mu2*DeltaInv(mh2,msb22,msb22) + 6*
     mh2*mu2*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*
     msb24 - msb12*msb24 + msb22*power2(mh2) + power3(msb22)) + 24*msb12*mu2*
     power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 3*power3(msb22)) + 72*mu2*
     power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*
     (msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 288*
     mh2*msb12*mu2*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22))*power4(snb)
     )/(6.*msb12) - (ca*mu2*sa*sb*log(msb22/Q2)*(-((5*mh2*msb22 - msb12*msb22 +
     msb24)*DeltaInv(mh2,msb12,msb22)) - msb12*(mh2 - 6*msb22)*DeltaInv(mh2,
     msb22,msb22) + 5*mH2*msb22*DeltaInv(mH2,msb12,msb22) - msb12*msb22*
     DeltaInv(mH2,msb12,msb22) + msb24*DeltaInv(mH2,msb12,msb22) + mH2*msb12*
     DeltaInv(mH2,msb22,msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,msb22) + 6*
     mH2*msb12*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*msb22*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) - 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) - 6*mH2*
     power2(DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*
     power2(mh2) + power3(msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     (mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22))*power4(snb)
     )/(3.*cb*msb12) + (mu2*log(msb22/Q2)*power2(sa)*(-((5*mh2*msb22 - msb12*
     msb22 + msb24)*DeltaInv(mh2,msb12,msb22)) - msb12*(mh2 - 6*msb22)*
     DeltaInv(mh2,msb22,msb22) + 5*mH2*msb22*DeltaInv(mH2,msb12,msb22) - msb12*
     msb22*DeltaInv(mH2,msb12,msb22) + msb24*DeltaInv(mH2,msb12,msb22) + mH2*
     msb12*DeltaInv(mH2,msb22,msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,msb22)
     + 6*mH2*msb12*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*msb22*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2)
     - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*
     power2(mh2) + power3(msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     (mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22))*power4(snb)
     )/(6.*msb12) - (mu2*log(msb22/Q2)*power2(sa)*power2(sb)*(-((5*mh2*msb22 -
     msb12*msb22 + msb24)*DeltaInv(mh2,msb12,msb22)) - msb12*(mh2 - 6*msb22)*
     DeltaInv(mh2,msb22,msb22) + 5*mH2*msb22*DeltaInv(mH2,msb12,msb22) - msb12*
     msb22*DeltaInv(mH2,msb12,msb22) + msb24*DeltaInv(mH2,msb12,msb22) + mH2*
     msb12*DeltaInv(mH2,msb22,msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,msb22)
     + 6*mH2*msb12*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*msb22*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2)
     - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*
     power2(mh2) + power3(msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     (mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22))*power4(snb)
     )/(6.*msb12*power2(cb)) + (2*mu2*log(mh2/Q2)*log(msb22/Q2)*((mh2*msb22 +
     msb12*msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) +
     power2(mh2)*((msb12*msb22 + msb24)*power2(mh2) - msb22*power3(mh2) + (
     msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(DeltaInv(mh2,msb22,msb22)
     )*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(
     msb22)))*power4(snb))/msb12 + (4*ca*mu2*sa*sb*log(mh2/Q2)*log(msb22/Q2)*((
     mh2*msb22 + msb12*msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + power2(mh2)*((msb12*msb22 + msb24)*power2(mh2) - msb22*power3(
     mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(msb22))
     )*power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(DeltaInv(mh2,msb22,
     msb22))*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*
     power4(msb22)))*power4(snb))/(cb*msb12) - (2*mu2*log(mh2/Q2)*log(msb22/Q2)
     *power2(sa)*((mh2*msb22 + msb12*msb22 + msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + power2(mh2)*((msb12*msb22 + msb24)*power2(
     mh2) - msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(
     DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(
     mh2,msb22,msb22)*power4(msb22)))*power4(snb))/msb12 + (2*mu2*log(mh2/Q2)*
     log(msb22/Q2)*power2(sa)*power2(sb)*((mh2*msb22 + msb12*msb22 + msb24)*
     power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + power2(mh2)*((msb12*msb22
     + msb24)*power2(mh2) - msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) +
     mh2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) +
     msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*
     mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)))*power4(snb))/(msb12*power2(
     cb)) - (4*ca*mu2*sa*sb*log(mH2/Q2)*log(msb22/Q2)*((mH2*msb22 + msb12*msb22
     + msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((
     msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*
     power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,
     msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*
     power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)))*power4(
     snb))/(cb*msb12) + (2*mu2*log(mH2/Q2)*log(msb22/Q2)*power2(sa)*((mH2*msb22
     + msb12*msb22 + msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     power2(mH2)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) + (
     msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,msb22)
     )*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(
     msb22)))*power4(snb))/msb12 + (2*mu2*log(mH2/Q2)*log(msb22/Q2)*power2(sb)*
     ((mH2*msb22 + msb12*msb22 + msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + power2(mH2)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(
     mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22))
     )*power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,
     msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*
     power4(msb22)))*power4(snb))/(msb12*power2(cb)) - (2*mu2*log(mH2/Q2)*log(
     msb22/Q2)*power2(sa)*power2(sb)*((mH2*msb22 + msb12*msb22 + msb24)*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((msb12*msb22 +
     msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) +
     mH2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) +
     msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*
     mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)))*power4(snb))/(msb12*power2(
     cb)) - (mu2*log(msb22/Q2)*power2(sb)*(-((5*mH2*msb22 - msb12*msb22 +
     msb24)*DeltaInv(mH2,msb12,msb22)) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22)
     )*(-3*mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) +
     power3(msb22)) + 72*power2(mH2)*(msb24*power2(mH2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,
     msb12,msb22)) + msb12*DeltaInv(mH2,msb22,msb22)*(-mH2 + 6*msb22 + 24*
     DeltaInv(mH2,msb22,msb22)*(mH2*msb24 + 3*power3(msb22)) + 288*mH2*power2(
     DeltaInv(mH2,msb22,msb22))*power4(msb22)))*power4(snb))/(6.*msb12*power2(
     cb)) - (2*mu2*log(mh2/Q2)*log(msb12/Q2)*power2(snb)*(power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 + DeltaInv(
     mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*power3(
     mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 + power3(
     msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(
     msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 -
     (4*ca*mu2*sa*sb*log(mh2/Q2)*log(msb12/Q2)*power2(snb)*(power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 + DeltaInv(
     mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*power3(
     mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 + power3(
     msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(
     msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(cb*
     msb12) + (2*mu2*log(mh2/Q2)*log(msb12/Q2)*power2(sa)*power2(snb)*(power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24
     + DeltaInv(mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*
     msb22*power3(mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24
     + power3(msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 +
     4*power4(msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12))
     )/msb12 - (2*mu2*log(mh2/Q2)*log(msb12/Q2)*power2(sa)*power2(sb)*power2(
     snb)*(power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*
     msb22 - msb24 + DeltaInv(mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*
     power2(mh2) + 3*msb22*power3(mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(
     -10*msb12*msb24 + power3(msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(
     mh2*msb12*msb14 + 4*power4(msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,
     msb12))*power5(msb12)))/(msb12*power2(cb)) + (2*mu2*log(mh2/Q2)*log(msb12/
     Q2)*power4(snb)*(power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*
     msb22 - msb12*msb22 - msb24 + DeltaInv(mh2,msb12,msb22)*(-5*(msb12*msb22 +
     msb24)*power2(mh2) + 3*msb22*power3(mh2) + (-msb12 + msb22)*power3(msb22)
     + mh2*(-10*msb12*msb24 + power3(msb22)))) + power2(DeltaInv(mh2,msb12,
     msb12))*(mh2*msb12*msb14 + 4*power4(msb12)) + 16*mh2*power3(DeltaInv(mh2,
     msb12,msb12))*power5(msb12)))/msb12 + (4*ca*mu2*sa*sb*log(mh2/Q2)*log(
     msb12/Q2)*power4(snb)*(power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))*(-3*
     mh2*msb22 - msb12*msb22 - msb24 + DeltaInv(mh2,msb12,msb22)*(-5*(msb12*
     msb22 + msb24)*power2(mh2) + 3*msb22*power3(mh2) + (-msb12 + msb22)*
     power3(msb22) + mh2*(-10*msb12*msb24 + power3(msb22)))) + power2(DeltaInv(
     mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(msb12)) + 16*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(cb*msb12) - (2*mu2*log(mh2/Q2)
     *log(msb12/Q2)*power2(sa)*power4(snb)*(power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 + DeltaInv(mh2,msb12,
     msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*power3(mh2) + (-
     msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 + power3(msb22)))) +
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(msb12)) +
     16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 + (2*mu2*
     log(mh2/Q2)*log(msb12/Q2)*power2(sa)*power2(sb)*power4(snb)*(power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 +
     DeltaInv(mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*
     power3(mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 +
     power3(msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*
     power4(msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/
     (msb12*power2(cb)) - (mu2*power2(snb)*power2(log(mh2/Q2))*(-2*msb22*
     power2(DeltaInv(mh2,msb12,msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*
     power3(DeltaInv(mh2,msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(
     mh2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mh2,msb22,
     msb22))*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*
     power4(msb22)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/
     msb12 - (2*ca*mu2*sa*sb*power2(snb)*power2(log(mh2/Q2))*(-2*msb22*power2(
     DeltaInv(mh2,msb12,msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 -
     2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(
     DeltaInv(mh2,msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*
     msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*
     msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22))
     + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(cb*msb12) + (
     mu2*power2(sa)*power2(snb)*power2(log(mh2/Q2))*(-2*msb22*power2(DeltaInv(
     mh2,msb12,msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(
     msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(
     mh2,msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 +
     4*power4(msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*
     power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 - (mu2*power2(sa)*
     power2(sb)*power2(snb)*power2(log(mh2/Q2))*(-2*msb22*power2(DeltaInv(mh2,
     msb12,msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*
     msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,
     msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*
     power4(msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*
     power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(msb12*power2(cb)) + (
     mu2*power2(log(mh2/Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(mh2,msb12,
     msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 +
     msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,
     msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(
     msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(
     msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 + (2*ca*mu2*sa*sb*power2(
     log(mh2/Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(mh2,msb12,msb22))*
     power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24)
     + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,msb22)) +
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(msb12)) +
     msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*
     mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*power3(DeltaInv(mh2,
     msb12,msb12))*power5(msb12)))/(cb*msb12) - (mu2*power2(sa)*power2(log(mh2/
     Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(mh2,msb12,msb22))*power3(mh2) +
     2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*
     power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,msb22)) + power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(msb12)) + msb12*
     power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*mh2*
     DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*power3(DeltaInv(mh2,
     msb12,msb12))*power5(msb12)))/msb12 + (mu2*power2(sa)*power2(sb)*power2(
     log(mh2/Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(mh2,msb12,msb22))*
     power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24)
     + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,msb22)) +
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(msb12)) +
     msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*
     mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*power3(DeltaInv(mh2,
     msb12,msb12))*power5(msb12)))/(msb12*power2(cb)) + (mu2*power2(Pi)*power4(
     snb)*(-2*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*
     msb14 + 6*power4(msb12)) + 2*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))
     *(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) +
     3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*
     power4(msb22)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 6*
     power3(msb22) + 24*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 24*mh2*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(6.*msb12) - (log(msb12/
     Q2)*power4(snb)*(4*msb12*msb22 - mh2*mu2 - msb12*mu2 - msb22*mu2 + msb12*(
     -mh2 + 6*msb12)*msb22*mu2*DeltaInv(mh2,msb12,msb12) - 4*power2(msb12) +
     mu2*DeltaInv(mh2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 -
     5*msb22)*power2(mh2) + 5*mh2*power2(msb22) + power3(mh2)) + 24*msb12*
     msb22*mu2*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12))
     - 6*mh2*msb22*mu2*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*
     (6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2)
     + power3(msb22)) + 72*msb22*mu2*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*
     power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) -
     2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) +
     288*mh2*msb22*mu2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(6.*
     msb12*msb22) - (ca*mu2*sa*sb*power2(snb)*(-(mh2*msb12*DeltaInv(mh2,msb12,
     msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,msb22) - mh2*msb12*DeltaInv(mh2,
     msb22,msb22) + mH2*msb12*DeltaInv(mH2,msb12,msb12) - 2*mH2*msb12*DeltaInv(
     mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) - 84*(3*mh2*msb22 +
     msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 42*
     mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - 42*mH2*msb12*msb14*
     power2(DeltaInv(mH2,msb12,msb12)) + 84*msb12*msb22*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 252*msb24*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22)) - 42*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) +
     252*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 252*msb12*
     power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 252*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*msb24*power3(mH2)*
     power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(mH2)*power3(msb22)*
     power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 42*power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 1008*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 252*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(3.*
     cb*msb12) + (mu2*power2(sa)*power2(snb)*(-(mh2*msb12*DeltaInv(mh2,msb12,
     msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,msb22) - mh2*msb12*DeltaInv(mh2,
     msb22,msb22) + mH2*msb12*DeltaInv(mH2,msb12,msb12) - 2*mH2*msb12*DeltaInv(
     mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) - 84*(3*mh2*msb22 +
     msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 42*
     mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - 42*mH2*msb12*msb14*
     power2(DeltaInv(mH2,msb12,msb12)) + 84*msb12*msb22*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 252*msb24*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22)) - 42*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) +
     252*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 252*msb12*
     power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 252*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*msb24*power3(mH2)*
     power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(mH2)*power3(msb22)*
     power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 42*power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 1008*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 252*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*
     msb12) - (mu2*power2(sa)*power2(sb)*power2(snb)*(-(mh2*msb12*DeltaInv(mh2,
     msb12,msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,msb22) - mh2*msb12*
     DeltaInv(mh2,msb22,msb22) + mH2*msb12*DeltaInv(mH2,msb12,msb12) - 2*mH2*
     msb12*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) -
     84*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + 42*mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - 42*
     mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 84*msb12*msb22*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 252*msb24*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) - 42*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) + 252*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 252*
     msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 252*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*msb24*power3(mH2)*
     power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(mH2)*power3(msb22)*
     power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 42*power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 1008*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 252*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*
     msb12*power2(cb)) + (ca*mu2*sa*sb*power4(snb)*(-(mh2*msb12*DeltaInv(mh2,
     msb12,msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,msb22) - mh2*msb12*
     DeltaInv(mh2,msb22,msb22) + mH2*msb12*DeltaInv(mH2,msb12,msb12) - 2*mH2*
     msb12*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) -
     84*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + 42*mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - 42*
     mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 84*msb12*msb22*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 252*msb24*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) - 42*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) + 252*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 252*
     msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 252*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*msb24*power3(mH2)*
     power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(mH2)*power3(msb22)*
     power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 42*power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 1008*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 252*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(3.*
     cb*msb12) - (mu2*power2(sa)*power4(snb)*(-(mh2*msb12*DeltaInv(mh2,msb12,
     msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,msb22) - mh2*msb12*DeltaInv(mh2,
     msb22,msb22) + mH2*msb12*DeltaInv(mH2,msb12,msb12) - 2*mH2*msb12*DeltaInv(
     mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) - 84*(3*mh2*msb22 +
     msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 42*
     mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - 42*mH2*msb12*msb14*
     power2(DeltaInv(mH2,msb12,msb12)) + 84*msb12*msb22*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 252*msb24*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22)) - 42*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) +
     252*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 252*msb12*
     power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 252*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*msb24*power3(mH2)*
     power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(mH2)*power3(msb22)*
     power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 42*power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 1008*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 252*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*
     msb12) + (mu2*power2(sa)*power2(sb)*power4(snb)*(-(mh2*msb12*DeltaInv(mh2,
     msb12,msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,msb22) - mh2*msb12*
     DeltaInv(mh2,msb22,msb22) + mH2*msb12*DeltaInv(mH2,msb12,msb12) - 2*mH2*
     msb12*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) -
     84*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + 42*mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - 42*
     mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 84*msb12*msb22*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 252*msb24*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) - 42*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) + 252*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 252*
     msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 252*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*msb24*power3(mH2)*
     power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(mH2)*power3(msb22)*
     power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 42*power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 1008*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 252*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*
     msb12*power2(cb)) - (ca*mu2*sa*sb*power2(Pi)*power2(snb)*(-2*(3*mh2*msb22
     + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) +
     mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - mH2*msb12*msb14*
     power2(DeltaInv(mH2,msb12,msb12)) + 2*msb12*msb22*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 6*msb24*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22)) - mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 6*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 6*msb12*power2(
     DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*msb12*power2(DeltaInv(mH2,
     msb22,msb22))*power3(msb22) + 28*msb12*msb24*power3(mH2)*power3(DeltaInv(
     mH2,msb12,msb22)) + 10*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(
     mH2,msb12,msb22)) + 6*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 10*msb12*msb22*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) +
     6*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) - 6*power2(DeltaInv(
     mH2,msb12,msb12))*power4(msb12) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*
     msb12*msb14 + 6*power4(msb12)) + 24*mh2*msb12*power3(DeltaInv(mh2,msb22,
     msb22))*power4(msb22) - 6*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) - 24*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(
     msb22) + 2*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*
     msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(
     mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 6*
     msb22*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) + 24*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12) - 24*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/(3.*cb*msb12) + (mu2*power2(Pi)*power2(sa)*
     power2(snb)*(-2*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,
     msb22)) - mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*msb12*
     msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 6*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - mH2*msb12*msb24*power2(DeltaInv(mH2,
     msb22,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 6*
     msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 28*msb12*msb24*power3(mH2)*
     power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*power2(mH2)*power3(msb22)*
     power3(DeltaInv(mH2,msb12,msb22)) + 6*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 10*msb12*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power4(mH2) + 6*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 6*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 24*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 6*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 24*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 6*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 24*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*
     msb12) - (mu2*power2(Pi)*power2(sa)*power2(sb)*power2(snb)*(-2*(3*mh2*
     msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - mH2*msb12*
     msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*msb12*msb22*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + 6*msb24*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) - mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 6*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 6*msb12*power2(
     DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*msb12*power2(DeltaInv(mH2,
     msb22,msb22))*power3(msb22) + 28*msb12*msb24*power3(mH2)*power3(DeltaInv(
     mH2,msb12,msb22)) + 10*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(
     mH2,msb12,msb22)) + 6*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 10*msb12*msb22*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) +
     6*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) - 6*power2(DeltaInv(
     mH2,msb12,msb12))*power4(msb12) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*
     msb12*msb14 + 6*power4(msb12)) + 24*mh2*msb12*power3(DeltaInv(mh2,msb22,
     msb22))*power4(msb22) - 6*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) - 24*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(
     msb22) + 2*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*
     msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(
     mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 6*
     msb22*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) + 24*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12) - 24*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/(6.*msb12*power2(cb)) + (ca*mu2*sa*sb*
     power2(Pi)*power4(snb)*(-2*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(
     mh2,msb22,msb22)) - mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*
     msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 6*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - mH2*msb12*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*
     power3(mH2) + 6*msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) + 28*msb12*msb24*
     power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*power2(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 6*power3(mH2)*power3(
     msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*msb22*power3(DeltaInv(
     mH2,msb12,msb22))*power4(mH2) + 6*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) - 6*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 24*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 6*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 24*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 6*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 24*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(3.*cb*
     msb12) - (mu2*power2(Pi)*power2(sa)*power4(snb)*(-2*(3*mh2*msb22 + msb12*
     msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*
     msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - mH2*msb12*msb14*power2(
     DeltaInv(mH2,msb12,msb12)) + 2*msb12*msb22*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) + 6*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))
     - mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 6*msb22*power2(
     DeltaInv(mH2,msb12,msb22))*power3(mH2) + 6*msb12*power2(DeltaInv(mh2,
     msb22,msb22))*power3(msb22) - 6*msb12*power2(DeltaInv(mH2,msb22,msb22))*
     power3(msb22) + 28*msb12*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,
     msb22)) + 10*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 6*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) +
     10*msb12*msb22*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 6*msb24*
     power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) - 6*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*
     msb12*msb14 + 6*power4(msb12)) + 24*mh2*msb12*power3(DeltaInv(mh2,msb22,
     msb22))*power4(msb22) - 6*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) - 24*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(
     msb22) + 2*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*
     msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(
     mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 6*
     msb22*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) + 24*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12) - 24*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/(6.*msb12) + (mu2*power2(Pi)*power2(sa)*
     power2(sb)*power4(snb)*(-2*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(
     mh2,msb22,msb22)) - mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*
     msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 6*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - mH2*msb12*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*
     power3(mH2) + 6*msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) + 28*msb12*msb24*
     power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*power2(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 6*power3(mH2)*power3(
     msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*msb22*power3(DeltaInv(
     mH2,msb12,msb22))*power4(mH2) + 6*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) - 6*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 24*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 6*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 24*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 6*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 24*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*
     msb12*power2(cb)) + (4*ca*mu2*sa*sb*log(mH2/Q2)*log(msb12/Q2)*power2(snb)*
     (power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22
     - msb24 + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2)
     + 3*msb22*power3(mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*
     msb24 + power3(msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*
     msb14 + 4*power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12)))/(cb*msb12) - (2*mu2*log(mH2/Q2)*log(msb12/Q2)*power2(sa)*
     power2(snb)*(power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 -
     msb12*msb22 - msb24 + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*
     power2(mH2) + 3*msb22*power3(mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(
     -10*msb12*msb24 + power3(msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(
     mH2*msb12*msb14 + 4*power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12)))/msb12 - (2*mu2*log(mH2/Q2)*log(msb12/Q2)*power2(
     sb)*power2(snb)*(power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*
     msb22 - msb12*msb22 - msb24 + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 +
     msb24)*power2(mH2) + 3*msb22*power3(mH2) + (-msb12 + msb22)*power3(msb22)
     + mH2*(-10*msb12*msb24 + power3(msb22)))) + power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/(msb12*power2(cb)) + (2*mu2*log(mH2/Q2)*log(
     msb12/Q2)*power2(sa)*power2(sb)*power2(snb)*(power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(mH2,
     msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*power3(mH2) +
     (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 + power3(msb22)))) +
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) +
     16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(msb12*power2(cb)
     ) - (4*ca*mu2*sa*sb*log(mH2/Q2)*log(msb12/Q2)*power4(snb)*(power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 +
     DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*
     power3(mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 +
     power3(msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*
     power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/
     (cb*msb12) + (2*mu2*log(mH2/Q2)*log(msb12/Q2)*power2(sa)*power4(snb)*(
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22
     - msb24 + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2)
     + 3*msb22*power3(mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*
     msb24 + power3(msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*
     msb14 + 4*power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12)))/msb12 + (2*mu2*log(mH2/Q2)*log(msb12/Q2)*power2(sb)*
     power4(snb)*(power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 -
     msb12*msb22 - msb24 + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*
     power2(mH2) + 3*msb22*power3(mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(
     -10*msb12*msb24 + power3(msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(
     mH2*msb12*msb14 + 4*power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12)))/(msb12*power2(cb)) - (2*mu2*log(mH2/Q2)*log(msb12/
     Q2)*power2(sa)*power2(sb)*power4(snb)*(power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(mH2,msb12,
     msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*power3(mH2) + (-
     msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 + power3(msb22)))) +
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) +
     16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(msb12*power2(cb)
     ) + (2*ca*mu2*sa*sb*power2(snb)*power2(log(mH2/Q2))*(-2*msb22*power2(
     DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*power3(
     DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*
     msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*
     msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))
     + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(cb*msb12) - (
     mu2*power2(sa)*power2(snb)*power2(log(mH2/Q2))*(-2*msb22*power2(DeltaInv(
     mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(
     msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(
     mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 +
     4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*
     power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)) + 16*mH2*
     power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/msb12 - (mu2*power2(sb)*
     power2(snb)*power2(log(mH2/Q2))*(-2*msb22*power2(DeltaInv(mH2,msb12,msb22)
     )*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,
     msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(
     msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(
     msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)) + 16*mH2*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(msb12*power2(cb)) + (mu2*
     power2(sa)*power2(sb)*power2(snb)*power2(log(mH2/Q2))*(-2*msb22*power2(
     DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*power3(
     DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*
     msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*
     msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))
     + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(msb12*power2(
     cb)) - (2*ca*mu2*sa*sb*power2(log(mH2/Q2))*power4(snb)*(-2*msb22*power2(
     DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*power3(
     DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*
     msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*
     msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))
     + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(cb*msb12) + (
     mu2*power2(sa)*power2(log(mH2/Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(
     mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(
     msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(
     mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 +
     4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*
     power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)) + 16*mH2*
     power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/msb12 + (mu2*power2(sb)*
     power2(log(mH2/Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(mH2,msb12,msb22)
     )*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,
     msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(
     msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(
     msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)) + 16*mH2*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(msb12*power2(cb)) - (mu2*
     power2(sa)*power2(sb)*power2(log(mH2/Q2))*power4(snb)*(-2*msb22*power2(
     DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*power3(
     DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*
     msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*
     msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))
     + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(msb12*power2(
     cb)) + (mu2*power2(Pi)*power2(sb)*power4(snb)*(-2*(3*mH2*msb22 + msb12*
     msb22 + 3*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(
     DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 6*power4(msb12)) + 2*power2(
     mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*msb12*
     msb22*power2(mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*
     power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) + msb12*power2(
     DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 6*power3(msb22) + 24*mH2*DeltaInv(
     mH2,msb22,msb22)*power4(msb22)) + 24*mH2*power3(DeltaInv(mH2,msb12,msb12))
     *power5(msb12)))/(6.*msb12*power2(cb)) + (mu2*power2(sb)*power4(snb)*(-(
     mH2*msb12*DeltaInv(mH2,msb12,msb12)) + 2*mH2*msb12*DeltaInv(mH2,msb12,
     msb22) - 84*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 84*power2(mH2)*(-((5*msb12*msb22 + 3*msb24)*
     power2(mH2)) + 3*msb22*power3(mH2) + (-5*msb12 + 3*msb22)*power3(msb22) -
     mH2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))
     + 42*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 6*power4(msb12))
     + msb12*DeltaInv(mH2,msb22,msb22)*(-mH2 + 42*DeltaInv(mH2,msb22,msb22)*(
     mH2*msb24 + 6*power3(msb22)) + 1008*mH2*power2(DeltaInv(mH2,msb22,msb22))*
     power4(msb22)) + 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12))
     )/(6.*msb12*power2(cb)) - (mu2*log(msb12/Q2)*power2(sb)*power4(snb)*(-mH2
     - msb12 - msb22 + msb12*(-mH2 + 6*msb12)*msb22*DeltaInv(mH2,msb12,msb12) +
     DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*
     msb22)*power2(mH2) + 5*mH2*power2(msb22) + power3(mH2)) + 24*msb12*msb22*
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 3*power3(msb12)) - 6*mH2*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*msb24) + mH2*(6*msb12*
     msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mH2) + power3(mH2) + power3(
     msb22)) + 72*msb22*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) +
     2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + 288*mH2*msb22*
     power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*msb12*msb22*power2(
     cb)) + (DeltaInv(msb12,mt2,mu2)*log(msb12/Q2)*log(mt2/Q2)*power2(mu2)*
     power2(snb)*(msb12 + DeltaInv(msb12,mt2,mu2)*(msb12*mu2*(4*mt2 + mu2) + 2*
     msb14*(2*mt2 + 7*mu2) + power3(msb12)) - 2*power2(DeltaInv(msb12,mt2,mu2))
     *(msb14*(-15*mt2 + 4*mu2)*power2(mu2) - 5*mu2*(3*mt2 + 2*mu2)*power3(
     msb12) + msb12*(-mt2 + mu2)*power3(mu2) - (mt2 - 4*mu2)*power4(msb12) +
     power5(msb12))))/mt2 - (power2(Pi)*(msb22*(3*mh2*msb22 + msb12*msb22 + 3*
     msb24)*mt2*mu2*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + msb22*mt2*
     mu2*(msb12*msb22 + 3*(msb24 + msb22*mz2))*power2(mz2)*power2(DeltaInv(
     msb12,mz2,msb22)) + msb22*mt2*mu2*power2(mh2)*((5*msb12*msb22 + 3*msb24)*
     power2(mh2) - 3*msb22*power3(mh2) + (5*msb12 - 3*msb22)*power3(msb22) +
     mh2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))
     + msb22*mt2*mu2*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(14*msb12*
     msb24*mz2 + 5*msb12*msb22*power2(mz2) + 3*msb24*power2(mz2) + 5*msb12*
     power3(msb22) + 3*mz2*power3(msb22) - 3*msb22*power3(mz2) - 3*power4(
     msb22)) - 2*msb12*(-((3*mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*
     power2(mt2)*power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*power2(mt2)*(
     -((5*msb22*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb22
     + 3*mst22)*power3(mst22) - mC2*(14*msb22*mst24 + 3*power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + mu2*(mu2*DeltaInv(mt2,mu2,msb22)*power2(
     msb22) + msb22*mu2*power2(DeltaInv(mt2,mu2,msb22))*(4*msb24*(mt2 + 4*mu2)
     + msb22*mu2*(6*mt2 + 7*mu2) + power3(msb22)) + mt2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*(mst14 + mst12*mw2) -
     DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))) + 2*msb22*mu2*power3(DeltaInv(mt2,
     mu2,msb22))*(msb24*(25*mt2 + mu2)*power2(mu2) + mu2*(16*mt2 + 9*mu2)*
     power3(msb22) + 2*msb22*(3*mt2 - 2*mu2)*power3(mu2) + (mt2 - 5*mu2)*
     power4(msb22) - power5(msb22))))))/(12.*msb12*msb22*mt2) + (DeltaInv(mt2,
     mu2,msb22)*power2(mu2)*power2(log(mt2/Q2))*(msb22 + (3*msb24*(mt2 + 4*mu2)
     + msb22*mu2*(5*mt2 + 4*mu2))*DeltaInv(mt2,mu2,msb22) + power2(DeltaInv(
     mt2,mu2,msb22))*(35*msb24*mt2*power2(mu2) + 7*mu2*(3*mt2 + 2*mu2)*power3(
     msb22) + msb22*(7*mt2 - 5*mu2)*power3(mu2) + (mt2 - 8*mu2)*power4(msb22) -
     power5(msb22))))/mt2 + (mu2*log(mh2/Q2)*power2(snb)*(mh2 + msb12 + msb22 +
     (mh2 - 6*msb12)*msb12*msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*msb22*
     DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,
     msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22) + 6*
     msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 +
     power2(mh2))*power2(DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*
     power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,
     msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(
     msb22))*power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,
     msb22,msb22))*power4(msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,
     msb12))*power5(msb12) + 576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*
     power5(msb22)))/(6.*msb12*msb22) + (ca*mu2*sa*sb*log(mh2/Q2)*power2(snb)*(
     mh2 + msb12 + msb22 + (mh2 - 6*msb12)*msb12*msb22*DeltaInv(mh2,msb12,
     msb12) + mh2*msb12*msb22*DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 + 5*
     msb22)*DeltaInv(mh2,msb12,msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,msb22,
     msb22)*power2(msb22) + 6*msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(msb12 +
     14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,msb22)) + 12*
     mh2*msb12*msb22*msb24*power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*
     msb22*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*
     power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,msb22)) + 144*
     msb12*power2(DeltaInv(mh2,msb22,msb22))*power4(msb22) + 576*mh2*msb22*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) + 576*mh2*msb12*power3(
     DeltaInv(mh2,msb22,msb22))*power5(msb22)))/(3.*cb*msb12*msb22) - (mu2*log(
     mh2/Q2)*power2(sa)*power2(snb)*(mh2 + msb12 + msb22 + (mh2 - 6*msb12)*
     msb12*msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*msb22*DeltaInv(mh2,
     msb22,msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22)*power2(
     mh2) - 6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22) + 6*msb22*power2(
     mh2)*(-3*msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*
     power2(DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*power2(
     DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,msb12,
     msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,msb22,
     msb22))*power4(msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*
     power5(msb12) + 576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power5(
     msb22)))/(6.*msb12*msb22) + (mu2*log(mh2/Q2)*power2(sa)*power2(sb)*power2(
     snb)*(mh2 + msb12 + msb22 + (mh2 - 6*msb12)*msb12*msb22*DeltaInv(mh2,
     msb12,msb12) + mh2*msb12*msb22*DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 +
     5*msb22)*DeltaInv(mh2,msb12,msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,
     msb22,msb22)*power2(msb22) + 6*msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(
     msb12 + 14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,msb22))
     + 12*mh2*msb12*msb22*msb24*power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*
     msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) +
     72*msb22*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) +
     msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,msb22)) +
     144*msb12*power2(DeltaInv(mh2,msb22,msb22))*power4(msb22) + 576*mh2*msb22*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) + 576*mh2*msb12*power3(
     DeltaInv(mh2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22*power2(cb)) - (
     mu2*log(mh2/Q2)*power4(snb)*(mh2 + msb12 + msb22 + (mh2 - 6*msb12)*msb12*
     msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*msb22*DeltaInv(mh2,msb22,
     msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22)*power2(mh2) -
     6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22) + 6*msb22*power2(mh2)*(-3*
     msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*power2(
     DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*power2(DeltaInv(mh2,
     msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*
     msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(
     msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(
     mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) +
     576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power5(msb22)))/(6.*msb12*
     msb22) - (ca*mu2*sa*sb*log(mh2/Q2)*power4(snb)*(mh2 + msb12 + msb22 + (mh2
     - 6*msb12)*msb12*msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*msb22*
     DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,
     msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22) + 6*
     msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 +
     power2(mh2))*power2(DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*
     power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,
     msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(
     msb22))*power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,
     msb22,msb22))*power4(msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,
     msb12))*power5(msb12) + 576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*
     power5(msb22)))/(3.*cb*msb12*msb22) + (mu2*log(mh2/Q2)*power2(sa)*power4(
     snb)*(mh2 + msb12 + msb22 + (mh2 - 6*msb12)*msb12*msb22*DeltaInv(mh2,
     msb12,msb12) + mh2*msb12*msb22*DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 +
     5*msb22)*DeltaInv(mh2,msb12,msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,
     msb22,msb22)*power2(msb22) + 6*msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(
     msb12 + 14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,msb22))
     + 12*mh2*msb12*msb22*msb24*power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*
     msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) +
     72*msb22*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) +
     msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,msb22)) +
     144*msb12*power2(DeltaInv(mh2,msb22,msb22))*power4(msb22) + 576*mh2*msb22*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) + 576*mh2*msb12*power3(
     DeltaInv(mh2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22) - (mu2*log(
     mh2/Q2)*power2(sa)*power2(sb)*power4(snb)*(mh2 + msb12 + msb22 + (mh2 - 6*
     msb12)*msb12*msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*msb22*DeltaInv(
     mh2,msb22,msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22)*
     power2(mh2) - 6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22) + 6*msb22*
     power2(mh2)*(-3*msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 + power2(mh2)
     )*power2(DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*power2(
     DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,msb12,
     msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,msb22,
     msb22))*power4(msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*
     power5(msb12) + 576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power5(
     msb22)))/(6.*msb12*msb22*power2(cb)) + (power4(snb)*(8*msb12*msb22 - mh2*
     msb12*msb22*mu2*DeltaInv(mh2,msb12,msb12) + 2*mh2*msb12*msb22*mu2*
     DeltaInv(mh2,msb12,msb22) - mh2*msb12*msb22*mu2*DeltaInv(mh2,msb22,msb22)
     - 4*power2(msb12) - 4*power2(msb22) - 84*msb22*(3*mh2*msb22 + msb12*msb22
     + 3*msb24)*mu2*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 42*mh2*
     msb12*msb22*msb24*mu2*power2(DeltaInv(mh2,msb22,msb22)) + 42*msb12*msb22*
     mu2*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 6*power3(msb12)) + 252*
     msb12*mu2*power2(DeltaInv(mh2,msb22,msb22))*power4(msb22) + 84*msb22*mu2*
     power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*
     msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*
     mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) + 1008*mh2*
     msb22*mu2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) + 1008*mh2*
     msb12*mu2*power3(DeltaInv(mh2,msb22,msb22))*power5(msb22)))/(6.*msb12*
     msb22) + (ca*mu2*sa*sb*log(msb12/Q2)*power2(snb)*(-mh2 + mH2 + msb12*(-mh2
     + 6*msb12)*msb22*DeltaInv(mh2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb12,msb12) - msb22*msb24*DeltaInv(mH2,msb12,msb22) + msb12*DeltaInv(mH2,
     msb12,msb22)*power2(mH2) - 5*msb22*DeltaInv(mH2,msb12,msb22)*power2(mH2) -
     6*msb22*DeltaInv(mH2,msb12,msb12)*power2(msb12) - 5*mH2*DeltaInv(mH2,
     msb12,msb22)*power2(msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(msb22)
     - 24*mH2*msb12*msb14*msb22*power2(DeltaInv(mH2,msb12,msb12)) - 6*mH2*
     msb12*msb22*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 138*msb22*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 36*msb12*power2(mH2)*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22)) + DeltaInv(mh2,msb12,
     msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) +
     5*mh2*power2(msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*power3(mH2)
     - 6*msb12*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 138*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 24*msb12*
     msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 6*
     mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) +
     power3(msb22)) + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*
     msb12*msb22*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 6*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 144*msb22*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 216*msb12*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 72*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 216*msb12*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 144*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 144*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) +
     288*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 288*mH2*
     msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power5(msb22)))/(3.*cb*msb12*msb22) - (
     mu2*log(msb12/Q2)*power2(sa)*power2(snb)*(-mh2 + mH2 + msb12*(-mh2 + 6*
     msb12)*msb22*DeltaInv(mh2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb12,msb12) - msb22*msb24*DeltaInv(mH2,msb12,msb22) + msb12*DeltaInv(mH2,
     msb12,msb22)*power2(mH2) - 5*msb22*DeltaInv(mH2,msb12,msb22)*power2(mH2) -
     6*msb22*DeltaInv(mH2,msb12,msb12)*power2(msb12) - 5*mH2*DeltaInv(mH2,
     msb12,msb22)*power2(msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(msb22)
     - 24*mH2*msb12*msb14*msb22*power2(DeltaInv(mH2,msb12,msb12)) - 6*mH2*
     msb12*msb22*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 138*msb22*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 36*msb12*power2(mH2)*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22)) + DeltaInv(mh2,msb12,
     msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) +
     5*mh2*power2(msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*power3(mH2)
     - 6*msb12*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 138*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 24*msb12*
     msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 6*
     mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) +
     power3(msb22)) + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*
     msb12*msb22*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 6*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 144*msb22*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 216*msb12*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 72*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 216*msb12*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 144*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 144*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) +
     288*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 288*mH2*
     msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power5(msb22)))/(6.*msb12*msb22) + (mu2*
     log(msb12/Q2)*power2(sa)*power2(sb)*power2(snb)*(-mh2 + mH2 + msb12*(-mh2
     + 6*msb12)*msb22*DeltaInv(mh2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb12,msb12) - msb22*msb24*DeltaInv(mH2,msb12,msb22) + msb12*DeltaInv(mH2,
     msb12,msb22)*power2(mH2) - 5*msb22*DeltaInv(mH2,msb12,msb22)*power2(mH2) -
     6*msb22*DeltaInv(mH2,msb12,msb12)*power2(msb12) - 5*mH2*DeltaInv(mH2,
     msb12,msb22)*power2(msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(msb22)
     - 24*mH2*msb12*msb14*msb22*power2(DeltaInv(mH2,msb12,msb12)) - 6*mH2*
     msb12*msb22*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 138*msb22*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 36*msb12*power2(mH2)*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22)) + DeltaInv(mh2,msb12,
     msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) +
     5*mh2*power2(msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*power3(mH2)
     - 6*msb12*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 138*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 24*msb12*
     msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 6*
     mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) +
     power3(msb22)) + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*
     msb12*msb22*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 6*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 144*msb22*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 216*msb12*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 72*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 216*msb12*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 144*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 144*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) +
     288*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 288*mH2*
     msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power5(msb22)))/(6.*msb12*msb22*power2(
     cb)) - (ca*mu2*sa*sb*log(msb12/Q2)*power4(snb)*(-mh2 + mH2 + msb12*(-mh2 +
     6*msb12)*msb22*DeltaInv(mh2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb12,msb12) - msb22*msb24*DeltaInv(mH2,msb12,msb22) + msb12*DeltaInv(mH2,
     msb12,msb22)*power2(mH2) - 5*msb22*DeltaInv(mH2,msb12,msb22)*power2(mH2) -
     6*msb22*DeltaInv(mH2,msb12,msb12)*power2(msb12) - 5*mH2*DeltaInv(mH2,
     msb12,msb22)*power2(msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(msb22)
     - 24*mH2*msb12*msb14*msb22*power2(DeltaInv(mH2,msb12,msb12)) - 6*mH2*
     msb12*msb22*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 138*msb22*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 36*msb12*power2(mH2)*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22)) + DeltaInv(mh2,msb12,
     msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) +
     5*mh2*power2(msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*power3(mH2)
     - 6*msb12*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 138*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 24*msb12*
     msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 6*
     mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) +
     power3(msb22)) + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*
     msb12*msb22*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 6*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 144*msb22*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 216*msb12*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 72*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 216*msb12*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 144*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 144*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) +
     288*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 288*mH2*
     msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power5(msb22)))/(3.*cb*msb12*msb22) + (
     mu2*log(msb12/Q2)*power2(sa)*power4(snb)*(-mh2 + mH2 + msb12*(-mh2 + 6*
     msb12)*msb22*DeltaInv(mh2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb12,msb12) - msb22*msb24*DeltaInv(mH2,msb12,msb22) + msb12*DeltaInv(mH2,
     msb12,msb22)*power2(mH2) - 5*msb22*DeltaInv(mH2,msb12,msb22)*power2(mH2) -
     6*msb22*DeltaInv(mH2,msb12,msb12)*power2(msb12) - 5*mH2*DeltaInv(mH2,
     msb12,msb22)*power2(msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(msb22)
     - 24*mH2*msb12*msb14*msb22*power2(DeltaInv(mH2,msb12,msb12)) - 6*mH2*
     msb12*msb22*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 138*msb22*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 36*msb12*power2(mH2)*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22)) + DeltaInv(mh2,msb12,
     msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) +
     5*mh2*power2(msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*power3(mH2)
     - 6*msb12*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 138*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 24*msb12*
     msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 6*
     mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) +
     power3(msb22)) + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*
     msb12*msb22*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 6*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 144*msb22*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 216*msb12*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 72*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 216*msb12*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 144*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 144*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) +
     288*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 288*mH2*
     msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power5(msb22)))/(6.*msb12*msb22) - (mu2*
     log(msb12/Q2)*power2(sa)*power2(sb)*power4(snb)*(-mh2 + mH2 + msb12*(-mh2
     + 6*msb12)*msb22*DeltaInv(mh2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb12,msb12) - msb22*msb24*DeltaInv(mH2,msb12,msb22) + msb12*DeltaInv(mH2,
     msb12,msb22)*power2(mH2) - 5*msb22*DeltaInv(mH2,msb12,msb22)*power2(mH2) -
     6*msb22*DeltaInv(mH2,msb12,msb12)*power2(msb12) - 5*mH2*DeltaInv(mH2,
     msb12,msb22)*power2(msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(msb22)
     - 24*mH2*msb12*msb14*msb22*power2(DeltaInv(mH2,msb12,msb12)) - 6*mH2*
     msb12*msb22*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 138*msb22*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 36*msb12*power2(mH2)*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22)) + DeltaInv(mh2,msb12,
     msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) +
     5*mh2*power2(msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*power3(mH2)
     - 6*msb12*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 138*
     power2(msb22)*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 24*msb12*
     msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 6*
     mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) +
     power3(msb22)) + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*
     msb12*msb22*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 6*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 144*msb22*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 216*msb12*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 72*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 216*msb12*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 144*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 144*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) +
     288*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 288*mH2*
     msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power5(msb22)))/(6.*msb12*msb22*power2(
     cb)) - (mu2*power2(Pi)*power2(sb)*power2(snb)*(-2*msb22*(3*mH2*msb22 +
     msb12*msb22 + 3*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22)) + msb12*msb22*
     mst12*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + 3*msb22*mst14*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*msb22*mst12*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 3*msb12*mst14*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + 3*msb22*mst12*power2(DeltaInv(msb12,mst12,
     mC2))*power3(mC2) - 3*msb12*mst12*power2(DeltaInv(msb22,mst12,mC2))*
     power3(mC2) + msb12*msb22*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 +
     6*power3(msb12)) + 14*msb12*msb22*mst14*power3(mC2)*power3(DeltaInv(msb12,
     mst12,mC2)) + 5*msb12*msb22*power2(mC2)*power3(mst12)*power3(DeltaInv(
     msb12,mst12,mC2)) + 3*msb22*power3(mC2)*power3(mst12)*power3(DeltaInv(
     msb12,mst12,mC2)) - 14*msb12*msb22*mst14*power3(mC2)*power3(DeltaInv(
     msb22,mst12,mC2)) - 5*msb12*msb22*power2(mC2)*power3(mst12)*power3(
     DeltaInv(msb22,mst12,mC2)) - 3*msb12*power3(mC2)*power3(mst12)*power3(
     DeltaInv(msb22,mst12,mC2)) + 5*msb12*msb22*mst12*power3(DeltaInv(msb12,
     mst12,mC2))*power4(mC2) + 3*msb22*mst14*power3(DeltaInv(msb12,mst12,mC2))*
     power4(mC2) - 5*msb12*msb22*mst12*power3(DeltaInv(msb22,mst12,mC2))*
     power4(mC2) - 3*msb12*mst14*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2)
     + 6*msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*msb22*
     power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*
     msb12*msb22*power2(mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*
     mH2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 3*msb22*
     power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) + 3*msb12*
     power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*power4(mst12) - 3*msb22*
     mst12*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) + 3*msb12*mst12*
     power3(DeltaInv(msb22,mst12,mC2))*power5(mC2) + 24*mH2*msb22*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12) + 24*mH2*msb12*power3(DeltaInv(
     mH2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22*power2(cb)) - (ca*mu2*
     sa*sb*log(mH2/Q2)*power2(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*
     msb12*msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb22,msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(
     mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(
     mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*
     power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,
     msb22))*power4(msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12) + 576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(
     msb22)))/(3.*cb*msb12*msb22) + (mu2*log(mH2/Q2)*power2(sa)*power2(snb)*(
     mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*DeltaInv(mH2,msb12,
     msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (mH2 - msb12 + 5*
     msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*DeltaInv(mH2,msb22,
     msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*msb22 - mH2*(msb12 +
     14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 12*
     mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*
     msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*
     power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,msb22)) + 144*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*mH2*msb22*
     power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22) + (mu2*log(
     mH2/Q2)*power2(sb)*power2(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*
     msb12*msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb22,msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(
     mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(
     mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*
     power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,
     msb22))*power4(msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12) + 576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(
     msb22)))/(6.*msb12*msb22*power2(cb)) - (mu2*log(mH2/Q2)*power2(sa)*power2(
     sb)*power2(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*
     DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (
     mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*
     DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*
     msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,
     msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22))
     + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(
     msb12)) + 72*msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*
     mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*
     power3(DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22*power2(
     cb)) + (ca*mu2*sa*sb*log(mH2/Q2)*power4(snb)*(mH2 + msb12 + msb22 + (mH2 -
     6*msb12)*msb12*msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(
     mH2,msb22,msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*
     power2(mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*
     power2(mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2)
     )*power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,
     msb22))*power4(msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12) + 576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(
     msb22)))/(3.*cb*msb12*msb22) - (mu2*log(mH2/Q2)*power2(sa)*power4(snb)*(
     mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*DeltaInv(mH2,msb12,
     msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (mH2 - msb12 + 5*
     msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*DeltaInv(mH2,msb22,
     msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*msb22 - mH2*(msb12 +
     14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 12*
     mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*
     msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*
     power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,msb22)) + 144*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*mH2*msb22*
     power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22) - (mu2*log(
     mH2/Q2)*power2(sb)*power4(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*
     msb12*msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb22,msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(
     mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(
     mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*
     power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,
     msb22))*power4(msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12) + 576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(
     msb22)))/(6.*msb12*msb22*power2(cb)) + (mu2*log(mH2/Q2)*power2(sa)*power2(
     sb)*power4(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*
     DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (
     mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*
     DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*
     msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,
     msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22))
     + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(
     msb12)) + 72*msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*
     mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*
     power3(DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22*power2(
     cb)) - (mu2*power2(sb)*power2(snb)*(-msb12 + msb22 - 2*mH2*msb12*msb22*
     DeltaInv(mH2,msb12,msb12) + 4*mH2*msb12*msb22*DeltaInv(mH2,msb12,msb22) -
     2*mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - 2*mC2*msb12*msb22*DeltaInv(
     msb12,mst12,mC2) + 2*mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) - 168*
     msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) + 84*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) + 84*msb12*msb22*mst12*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2)) + 252*msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     84*msb12*msb22*mst12*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 252*
     msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 252*msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2))*power3(mC2) - 252*msb12*mst12*
     power2(DeltaInv(msb22,mst12,mC2))*power3(mC2) + 84*msb12*msb22*power2(
     DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 6*power3(msb12)) + 1176*msb12*
     msb22*mst14*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 420*msb12*
     msb22*power2(mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) + 252*
     msb22*power3(mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) - 1176*
     msb12*msb22*mst14*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2)) - 420*
     msb12*msb22*power2(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,mC2)) -
     252*msb12*power3(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,mC2)) +
     420*msb12*msb22*mst12*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 252*
     msb22*mst14*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) - 420*msb12*
     msb22*mst12*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) - 252*msb12*
     mst14*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) + 504*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 168*msb22*power2(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(
     mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*power3(msb22) -
     5*msb12*power3(msb22) + 3*power4(msb22)) - 252*msb22*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*power4(mst12) + 252*msb12*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*power4(mst12) - 252*msb22*mst12*power3(
     DeltaInv(msb12,mst12,mC2))*power5(mC2) + 252*msb12*mst12*power3(DeltaInv(
     msb22,mst12,mC2))*power5(mC2) + 2016*mH2*msb22*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) + 2016*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*
     power5(msb22)))/(12.*msb12*msb22*power2(cb)) + (log(mt2/Q2)*power2(snb)*((
     msb12 - msb22)*(mt2*mu2 + power2(mt2) + power2(mu2)) + msb22*mu2*DeltaInv(
     msb12,mt2,mu2)*(2*msb12*mu2*(mt2 + 2*mu2) + (mt2 - 17*mu2)*power2(msb12) +
     (-mt2 + mu2)*power2(mu2)) - msb12*mu2*DeltaInv(mt2,mu2,msb22)*(2*msb22*
     mu2*(mt2 + 2*mu2) + (mt2 - 17*mu2)*power2(msb22) + (-mt2 + mu2)*power2(
     mu2)) - 6*msb12*msb22*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*(13*
     msb14*mt2 + 67*msb14*mu2 + 5*msb12*mu2*(4*mt2 + 5*mu2) - mt2*power2(mu2) +
     3*power3(msb12) + power3(mu2)) + 6*msb12*msb22*power2(mu2)*power2(
     DeltaInv(mt2,mu2,msb22))*(13*msb24*mt2 + 67*msb24*mu2 + 5*msb22*mu2*(4*mt2
     + 5*mu2) - mt2*power2(mu2) + 3*power3(msb22) + power3(mu2)) + 36*msb12*
     msb22*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*(-35*msb14*mt2*power2(
     mu2) - 7*mu2*(3*mt2 + 2*mu2)*power3(msb12) + msb12*(-7*mt2 + 5*mu2)*
     power3(mu2) - (mt2 - 8*mu2)*power4(msb12) + power5(msb12)) - 36*msb12*
     msb22*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*(-35*msb24*mt2*power2(
     mu2) - 7*mu2*(3*mt2 + 2*mu2)*power3(msb22) + msb22*(-7*mt2 + 5*mu2)*
     power3(mu2) - (mt2 - 8*mu2)*power4(msb22) + power5(msb22))))/(6.*msb12*
     msb22*mt2) - (log(mt2/Q2)*(mC2*(mt2*mu2 + power2(mt2) + power2(mu2)) -
     mt2*(2*msb22*mt2 + mt2*mu2 + power2(mt2) + power2(mu2)) - (mC2 - mt2)*mu2*
     DeltaInv(mt2,mu2,msb22)*(2*msb22*mu2*(mt2 + 2*mu2) + (mt2 - 17*mu2)*
     power2(msb22) + (-mt2 + mu2)*power2(mu2)) + 6*msb22*(mC2 - mt2)*power2(
     mu2)*power2(DeltaInv(mt2,mu2,msb22))*(13*msb24*mt2 + 67*msb24*mu2 + 5*
     msb22*mu2*(4*mt2 + 5*mu2) - mt2*power2(mu2) + 3*power3(msb22) + power3(
     mu2)) - 36*msb22*(mC2 - mt2)*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*(
     -35*msb24*mt2*power2(mu2) - 7*mu2*(3*mt2 + 2*mu2)*power3(msb22) + msb22*(-
     7*mt2 + 5*mu2)*power3(mu2) - (mt2 - 8*mu2)*power4(msb22) + power5(msb22)))
     )/(6.*msb22*(mC2 - mt2)*mt2) + (DeltaInv(mt2,mu2,msb22)*log(msb22/Q2)*log(
     mt2/Q2)*power2(mu2)*(msb22 + DeltaInv(mt2,mu2,msb22)*(msb22*mu2*(4*mt2 +
     mu2) + 2*msb24*(2*mt2 + 7*mu2) + power3(msb22)) - 2*power2(DeltaInv(mt2,
     mu2,msb22))*(msb24*(-15*mt2 + 4*mu2)*power2(mu2) - 5*mu2*(3*mt2 + 2*mu2)*
     power3(msb22) + msb22*(-mt2 + mu2)*power3(mu2) - (mt2 - 4*mu2)*power4(
     msb22) + power5(msb22))))/mt2 - (DeltaInv(mt2,mu2,msb22)*log(msb22/Q2)*
     log(mt2/Q2)*power2(mu2)*power2(snb)*(msb22 + DeltaInv(mt2,mu2,msb22)*(
     msb22*mu2*(4*mt2 + mu2) + 2*msb24*(2*mt2 + 7*mu2) + power3(msb22)) - 2*
     power2(DeltaInv(mt2,mu2,msb22))*(msb24*(-15*mt2 + 4*mu2)*power2(mu2) - 5*
     mu2*(3*mt2 + 2*mu2)*power3(msb22) + msb22*(-mt2 + mu2)*power3(mu2) - (mt2
     - 4*mu2)*power4(msb22) + power5(msb22))))/mt2 + (mu2*log(msb22/Q2)*power2(
     sb)*power2(snb)*(-2*msb22*(5*mH2*msb22 - msb12*msb22 + msb24)*mst12*
     DeltaInv(mH2,msb12,msb22) + 12*mH2*msb22*mst12*power2(DeltaInv(mH2,msb12,
     msb22))*(-3*mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(
     mH2) + power3(msb22)) + 144*msb22*mst12*power2(mH2)*(msb24*power2(mH2) + (
     -2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)) + msb12*(-mC2 - msb22 - mst12 - 2*(mH2 -
     6*msb22)*msb22*mst12*DeltaInv(mH2,msb22,msb22) + DeltaInv(msb22,mst12,mC2)
     *(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + 48*msb22*mst12*power2(DeltaInv(mH2,msb22,
     msb22))*(mH2*msb24 + 3*power3(msb22)) - 6*mC2*mst12*power2(DeltaInv(msb22,
     mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 72*mst12*power2(
     mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-
     3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + 576*mH2*mst12*power3(DeltaInv(mH2,
     msb22,msb22))*power5(msb22))))/(12.*msb12*msb22*mst12*power2(cb)) + (
     power2(mu2)*power2(snb)*power2(log(mt2/Q2))*(msb12*DeltaInv(msb12,mt2,mu2)
     + (3*msb14*(mt2 + 4*mu2) + msb12*mu2*(5*mt2 + 4*mu2))*power2(DeltaInv(
     msb12,mt2,mu2)) + power3(DeltaInv(msb12,mt2,mu2))*(35*msb14*mt2*power2(
     mu2) + 7*mu2*(3*mt2 + 2*mu2)*power3(msb12) + msb12*(7*mt2 - 5*mu2)*power3(
     mu2) + (mt2 - 8*mu2)*power4(msb12) - power5(msb12)) + DeltaInv(mt2,mu2,
     msb22)*(-msb22 - (3*msb24*(mt2 + 4*mu2) + msb22*mu2*(5*mt2 + 4*mu2))*
     DeltaInv(mt2,mu2,msb22) + power2(DeltaInv(mt2,mu2,msb22))*(-35*msb24*mt2*
     power2(mu2) - 7*mu2*(3*mt2 + 2*mu2)*power3(msb22) + msb22*(-7*mt2 + 5*mu2)
     *power3(mu2) - (mt2 - 8*mu2)*power4(msb22) + power5(msb22)))))/mt2 - (
     power2(log(msb22/Q2))*(msb22*msb24*mt2*mu2*power2(mh2)*power2(DeltaInv(
     mh2,msb12,msb22)) + msb22*msb24*mt2*mu2*power2(mz2)*power2(DeltaInv(msb12,
     mz2,msb22)) - msb22*mt2*mu2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 +
     msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)) - msb22*mt2*mu2*power2(mz2)*power3(DeltaInv(
     msb12,mz2,msb22))*(msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*msb12*(
     msb24*mz2 + power3(msb22)) + power4(msb22)) + 2*msb12*((2*mC2*mst22 +
     msb22*mst22 + 2*mst24)*power2(mC2)*power2(mt2)*power2(DeltaInv(mst22,
     msb22,mC2)) + power2(mC2)*power2(mt2)*((3*msb22*mst22 + 2*mst24)*power2(
     mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*
     msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + mu2*(-(
     msb22*mu2*power2(DeltaInv(mt2,mu2,msb22))*(msb24*(mt2 + 3*mu2) + power3(
     msb22))) + mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12
     + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 +
     3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) +
     2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + msb22*mu2*
     power3(DeltaInv(mt2,mu2,msb22))*(msb24*(-5*mt2 + 3*mu2)*power2(mu2) - 5*
     mu2*(2*mt2 + mu2)*power3(msb22) + (-mt2 + mu2)*power4(msb22) + power5(
     msb22))))))/(2.*msb12*msb22*mt2) + (power2(snb)*power2(log(msb22/Q2))*(2*
     msb22*msb24*mt2*mu2*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) - 2*
     msb22*mt2*mu2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22)) + msb12*(mh2*msb22*msb24*mt2*mu2*power2(DeltaInv(mh2,msb22,msb22))
     + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(mC2)*power2(mt2)*power2(
     DeltaInv(mst22,msb22,mC2)) + power2(mC2)*power2(mt2)*((3*msb22*mst22 + 2*
     mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + mu2*(-(msb22*mu2*power2(DeltaInv(mt2,mu2,msb22))*(msb24*(mt2
     + 3*mu2) + power3(msb22))) + mt2*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))) + msb22*mu2*power3(DeltaInv(mt2,mu2,msb22))*(msb24*(-5*mt2
     + 3*mu2)*power2(mu2) - 5*mu2*(2*mt2 + mu2)*power3(msb22) + (-mt2 + mu2)*
     power4(msb22) + power5(msb22))))))/(msb12*msb22*mt2) + (mu2*log(msb12/Q2)*
     power2(sb)*power2(snb)*(mC2*msb22 + msb12*msb22 - 2*mH2*mst12 - 2*msb12*
     mst12 - msb22*mst12 - 2*(mH2 - 6*msb12)*msb12*msb22*mst12*DeltaInv(mH2,
     msb12,msb12) - msb22*mst12*mst14*DeltaInv(msb12,mst12,mC2) + msb12*msb22*
     DeltaInv(msb12,mst12,mC2)*power2(mC2) - 5*msb22*mst12*DeltaInv(msb12,
     mst12,mC2)*power2(mC2) - 5*mC2*msb22*DeltaInv(msb12,mst12,mC2)*power2(
     mst12) + msb12*msb22*DeltaInv(msb12,mst12,mC2)*power2(mst12) - 6*mC2*
     msb12*msb22*mst12*mst14*power2(DeltaInv(msb12,mst12,mC2)) + 138*msb22*
     mst12*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + 36*msb12*
     msb22*power2(mC2)*power2(mst12)*power2(DeltaInv(msb12,mst12,mC2)) - msb22*
     DeltaInv(msb12,mst12,mC2)*power3(mC2) - 6*msb12*msb22*mst12*power2(
     DeltaInv(msb12,mst12,mC2))*power3(mC2) + 138*msb22*power2(mst12)*power2(
     DeltaInv(msb12,mst12,mC2))*power3(mC2) + 2*mst12*DeltaInv(mH2,msb12,msb22)
     *(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*
     power2(msb22) + power3(mH2)) + 48*msb12*msb22*mst12*power2(DeltaInv(mH2,
     msb12,msb12))*(mH2*msb14 + 3*power3(msb12)) - 12*mH2*msb22*mst12*power2(
     DeltaInv(mH2,msb12,msb22))*(-(msb12*msb24) + mH2*(6*msb12*msb22 + 23*
     msb24) - (msb12 - 23*msb22)*power2(mH2) + power3(mH2) + power3(msb22)) +
     144*msb22*mst12*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*
     msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + 720*msb12*
     msb22*mst12*mst14*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 6*msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 144*msb22*mst12*
     mst14*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 216*msb12*msb22*
     power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 6*mC2*msb22*
     power2(DeltaInv(msb12,mst12,mC2))*power4(mst12) + 216*msb12*msb22*power2(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) + 144*msb22*power3(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) - 144*msb22*power2(
     mst12)*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) + 576*mH2*msb22*
     mst12*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*msb22*power2(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*power5(mst12)))/(12.*msb12*msb22*
     mst12*power2(cb)) - (mu2*power2(sb)*(msb12*msb22 + 2*msb12*mst12 + msb22*
     mst12 - mH2*msb12*msb22*mst12*DeltaInv(mH2,msb12,msb22) - mA2*msb12*msb22*
     mst12*DeltaInv(msb12,mA2,msb22) - 2*mC2*msb12*msb22*mst12*DeltaInv(msb22,
     mst12,mC2) + 42*msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*mst12*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 42*msb22*(3*mA2*msb22 + msb12*
     msb22 + 3*msb24)*mst12*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)) +
     252*msb12*mst12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 84*
     msb12*msb22*power2(mC2)*power2(mst12)*power2(DeltaInv(msb22,mst12,mC2)) +
     252*msb12*power2(mst12)*power2(DeltaInv(msb22,mst12,mC2))*power3(mC2) -
     42*msb22*mst12*power2(mH2)*(-((5*msb12*msb22 + 3*msb24)*power2(mH2)) + 3*
     msb22*power3(mH2) + (-5*msb12 + 3*msb22)*power3(msb22) - mH2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + 1176*msb12*
     msb22*mst12*mst14*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2)) + 252*
     msb12*mst12*mst14*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) + 420*
     msb12*msb22*power2(mst12)*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) -
     42*msb22*mst12*power2(mA2)*power3(DeltaInv(msb12,mA2,msb22))*(-14*mA2*
     msb12*msb24 - 5*msb12*msb22*power2(mA2) - 3*msb24*power2(mA2) + 3*msb22*
     power3(mA2) - 3*mA2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(
     msb22)) + 420*msb12*msb22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*
     power4(mst12) + 252*msb12*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2))*
     power4(mst12) - 252*msb12*power2(mst12)*power3(DeltaInv(msb22,mst12,mC2))*
     power5(mC2) - 252*msb12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*
     power5(mst12)))/(12.*msb12*msb22*mst12*power2(cb)) + (mu2*log(msb12/Q2)*
     power2(sb)*power2(snb)*power2(snt)*(mC2*mst12 - mC2*mst22 - mst12*mw2 +
     mst22*mw2 + mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) - mst12*mst14*
     mst22*DeltaInv(mw2,msb12,mst12) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) -
     msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) +
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*msb12*mst12*mst22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*mst14*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + mst22*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(
     msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2)
     )*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*
     mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) +
     mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*DeltaInv(mw2,msb12,
     mst12)*power3(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) - 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*
     msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 216*msb12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 216*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) + 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) + 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(6.*msb12*
     mst12*mst22) + (mu2*log(msb12/Q2)*power2(sb)*power2(snb)*power2(snt)*(-(
     mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*DeltaInv(mw2,msb12,mst12) -
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*power2(mst22)
     + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 36*
     msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) +
     6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 36*
     msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) -
     mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2)
     + mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 216*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(12.*msb12*mst12*mst22) + (log(msb12/Q2)*power2(cb)*power2(snb)*
     power2(snt)*(mC2*mst12*mt2 + msb12*mst12*mt2 - mC2*mst22*mt2 - msb12*
     mst22*mt2 + mC2*mst12*mu2 - mC2*mst22*mu2 - mst12*mu2*mw2 + mst22*mu2*mw2
     + mst12*mst22*mst24*mu2*DeltaInv(mst22,mw2,msb12) - mst12*mst14*mst22*mu2*
     DeltaInv(mw2,msb12,mst12) + msb12*mst22*mu2*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) - 5*mst22*mu2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) -
     msb12*mst12*mu2*DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mu2*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - msb12*mst12*mu2*DeltaInv(mst22,
     mw2,msb12)*power2(mw2) + 5*mst12*mst22*mu2*DeltaInv(mst22,mw2,msb12)*
     power2(mw2) + msb12*mst22*mu2*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*
     mst12*mst22*mu2*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 6*msb12*mst12*
     mst22*mst24*mu2*mw2*power2(DeltaInv(mst22,mw2,msb12)) - 138*mst12*mst22*
     mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 36*msb12*mst12*
     mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*
     mst12*mst14*mst22*mu2*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 138*mst12*
     mst14*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*
     mst22*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     mst22*(mt2 + mu2)*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) +
     mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) - mst12*(mt2 + mu2)*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) +
     mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)
     ) - 6*mC2*mst12*mst22*(mt2 + mu2)*power2(DeltaInv(msb12,mst12,mC2))*(-(
     msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*(mt2 + mu2)*
     power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 +
     23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     + mst12*mu2*DeltaInv(mst22,mw2,msb12)*power3(mw2) - mst22*mu2*DeltaInv(
     mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*mst22*mu2*power2(DeltaInv(
     mst22,mw2,msb12))*power3(mw2) - 138*mst12*mu2*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*mst22*mu2*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*mu2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*mst12*mst22*(mt2 + mu2)
     *power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*(mt2 + mu2)*
     power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*
     mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst14*
     mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*mu2*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 216*msb12*mst22*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*mst22*
     mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 6*mst12*
     mu2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 216*msb12*mst12*
     mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 144*
     mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 6*
     mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 6*mst12*
     mst22*mu2*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*mst12*mst22*
     mst24*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 216*msb12*mst12*
     mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 144*
     mst12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 216*
     msb12*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 144*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mst22) + 144*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) - 144*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2)))/(12.*msb12*mst12*mst22) + (log(msb12/Q2)*
     power2(snb)*power2(snt)*(-(mC2*mst12*mt2) - msb12*mst12*mt2 + mC2*mst22*
     mt2 + msb12*mst22*mt2 + msb12*mst12*mu2 - msb12*mst22*mu2 + mst12*mu2*mw2
     - mst22*mu2*mw2 - mst12*mst22*mst24*mu2*DeltaInv(mst22,mw2,msb12) + mst12*
     mst14*mst22*mu2*DeltaInv(mw2,msb12,mst12) + mst12*power2(msb12) - mst22*
     power2(msb12) - msb12*mst22*mu2*DeltaInv(mw2,msb12,mst12)*power2(mst12) +
     5*mst22*mu2*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst12*mu2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) - 5*mst12*mu2*mw2*DeltaInv(mst22,
     mw2,msb12)*power2(mst22) + msb12*mst12*mu2*DeltaInv(mst22,mw2,msb12)*
     power2(mw2) - 5*mst12*mst22*mu2*DeltaInv(mst22,mw2,msb12)*power2(mw2) -
     msb12*mst22*mu2*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 5*mst12*mst22*mu2*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - 6*msb12*mst12*mst22*mst24*mu2*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) + 138*mst12*mst22*mst24*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*mst12*mu2*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst14*mst22*
     mu2*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 138*mst12*mst14*mst22*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 36*msb12*mst22*mu2*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - mst22*mt2*DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*mt2*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*mt2*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - 6*mC2*mst12*mst22*mt2*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24)
     + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*mu2*DeltaInv(mst22,mw2,msb12)*power3(
     mw2) + mst22*mu2*DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*
     mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*mu2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*
     mst12*mst22*mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*
     mu2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*
     mst12*mst22*mt2*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*
     mst22*mt2*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 720*msb12*mst12*mst22*
     mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*
     mst14*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*
     mu2*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 216*msb12*mst22*
     mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 144*
     mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*
     mst12*mu2*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) +
     144*mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22)
     + 6*mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*
     mst12*mst22*mu2*power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*mst12*
     mst22*mst24*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*
     mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     144*mst12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) -
     216*msb12*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 144*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mst12) - 144*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12)
     )*power5(mst22) - 144*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 144*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2)))/(12.*msb12*mst12*mst22) + (mu2*power2(sb)*
     power2(snb)*power2(snt)*(-(mC2*msb12*msb22*DeltaInv(msb12,mst12,mC2)) +
     mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*DeltaInv(
     mst22,msb12,mC2) - mC2*msb12*msb22*DeltaInv(mst22,msb22,mC2) - msb12*
     msb22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*mw2*DeltaInv(mst22,mw2,
     msb22) + msb12*msb22*mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*mw2*
     DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)
     *power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - 42*msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*
     msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*msb12*
     msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*msb22*mst22*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*msb12*mst24*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) - 126*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) + 126*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2) + 126*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) -
     126*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 126*msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 126*msb12*mst12*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 42*msb22*power2(mC2)*(-((
     5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 +
     3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 588*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 126*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 588*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 126*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 210*msb12*msb22*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 588*msb12*
     msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*msb12*
     msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 588*
     msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 210*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb12*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*
     mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*
     power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) + (
     mu2*power2(Pi)*power2(sb)*power2(snb)*power2(snt)*(msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) - msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     + msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*
     msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*msb22*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) + msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) + 3*msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) - 3*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 3*msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 3*msb22*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 3*msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb22*mst12*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) + msb22*power2(mC2)*((5*msb12*mst12 + 3*mst14)*power2(
     mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*
     msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*
     msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 14*
     msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 3*
     msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     14*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 5*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) -
     14*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 3*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 5*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     14*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     14*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*
     mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 3*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + msb12*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(
     mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) -
     5*msb22*power3(mst12) + 3*power4(mst12)) + 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) - 3*msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 3*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*
     msb12*mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb22*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) + 3*msb12*mst22*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) + 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 3*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2))
     )/(3.*msb12*msb22) + (mu2*power2(Pi)*power2(sb)*power2(snb)*power2(snt)*(-
     (msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb12*msb22*mst22*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) - msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 3*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     + msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 3*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + msb12*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst22*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) - 3*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) - 3*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     + 3*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 3*msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 3*msb12*mst12*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*power2(mC2)*(-((5*
     msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*
     mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 14*msb12*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 3*msb12*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + msb12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2) + 3*mst14*
     power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*msb22*power3(
     mst12) - 3*power4(mst12)) - 3*msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mst12) + 3*msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) - 3*msb22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) + 3*msb12*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     )*power4(mst22) + 3*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) - 3*msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 3*msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*
     msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 5*msb12*msb22*
     mst12*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 3*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 5*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb12*mst14*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst22*power3(DeltaInv(mst22,msb22,
     mC2))*power5(mC2) + 3*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) - 3*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2)
     - 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) + 3*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) + (
     power2(cb)*power2(snb)*power2(snt)*(-(mC2*msb12*msb22*(mt2 + mu2)*
     DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*(mt2 + mu2)*DeltaInv(msb22,
     mst12,mC2) + mC2*msb12*msb22*mt2*DeltaInv(mst22,msb12,mC2) + mC2*msb12*
     msb22*mu2*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mt2*DeltaInv(mst22,
     msb22,mC2) - mC2*msb12*msb22*mu2*DeltaInv(mst22,msb22,mC2) - msb12*msb22*
     mu2*mw2*DeltaInv(mst22,mw2,msb12) + msb12*msb22*mu2*mw2*DeltaInv(mst22,
     mw2,msb22) + msb12*msb22*mu2*mw2*DeltaInv(mw2,msb12,mst12) - msb12*msb22*
     mu2*mw2*DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*mC2*mst12 + msb12*mst12 +
     3*mst14)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - 42*
     msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*(mt2 + mu2)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - 42*msb12*msb22*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 42*msb12*msb22*mst22*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 42*msb12*msb22*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*mst22*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 42*msb12*msb22*mst22*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 126*msb22*mst24*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 42*msb12*msb22*mst22*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 126*msb12*mst24*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*msb22*mst12*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mst14*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*mst12*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 126*msb22*mst22*mt2*power2(DeltaInv(
     mst22,msb12,mC2))*power3(mC2) - 126*msb22*mst22*mu2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) + 126*msb12*mst22*mt2*power2(DeltaInv(mst22,msb22,
     mC2))*power3(mC2) + 126*msb12*mst22*mu2*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 126*msb22*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) - 126*msb12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 126*msb22*mst12*mu2*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 126*msb12*mst12*mu2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) - 42*msb22*(mt2 + mu2)*power2(mC2)*(-((5*msb12*mst12 + 3*
     mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(
     mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) - 588*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 588*msb12*msb22*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 210*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*mu2*power2(mC2)*power3(mst22)
     *power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mt2*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mu2*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*mst24*
     mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 588*msb12*msb22*mst24*
     mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*
     msb22*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     126*msb12*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2))
     + 126*msb12*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)
     ) + 210*msb12*msb22*mu2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,
     mw2,msb12)) + 588*msb12*msb22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12)) + 126*msb22*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) - 210*msb12*msb22*mu2*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb22)) - 588*msb12*msb22*mst24*mu2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) - 126*msb12*mu2*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 210*msb12*msb22*mu2*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 588*msb12*msb22*mst14*
     mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*msb22*mu2*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*msb12*msb22*
     mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 588*
     msb12*msb22*mst14*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) -
     210*msb12*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) -
     126*msb22*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 210*
     msb12*msb22*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*
     msb22*mst24*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 210*msb12*
     msb22*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*
     mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 210*msb12*msb22*
     mst22*mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*mst24*
     mu2*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb22*mu2*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb12*(
     mt2 + mu2)*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*
     mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(
     mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) +
     126*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22)
     + 126*msb22*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mst22) - 126*msb12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mst22) - 126*msb12*mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     )*power4(mst22) - 126*msb22*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 126*msb12*mu2*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 210*msb12*msb22*mst22*mu2*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 126*msb22*mst24*mu2*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mw2) - 210*msb12*msb22*mst22*mu2*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) - 126*msb12*mst24*mu2*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 210*msb12*msb22*mst12*mu2*power3(DeltaInv(mw2,msb12,
     mst12))*power4(mw2) - 126*msb22*mst14*mu2*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 210*msb12*msb22*mst12*mu2*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mw2) + 126*msb12*mst14*mu2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) + 126*msb22*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 126*msb22*mst22*mu2*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 126*msb12*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 126*msb12*mst22*mu2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 126*msb22*mst22*mu2*power3(DeltaInv(mst22,mw2,msb12))*
     power5(mw2) + 126*msb12*mst22*mu2*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 126*msb22*mst12*mu2*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 126*msb12*mst12*mu2*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mw2)))/(6.*msb12*msb22) + (power2(cb)*power2(Pi)*power2(snb)*
     power2(snt)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*(mt2 + mu2)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*(mt2 + mu2)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 3*msb22*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) - msb12*msb22*mst22*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)
     ) - 3*msb22*mst24*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) +
     msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*
     msb12*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*
     msb22*mst22*mu2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*
     mst24*mu2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*
     mst22*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 3*msb22*mst24*
     mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - msb12*msb22*mst22*mu2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 3*msb12*mst24*mu2*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) - msb12*msb22*mst12*mu2*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + msb12*msb22*mst12*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 3*msb22*mst22*mt2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 3*msb22*mst22*mu2*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) + 3*msb12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 3*msb12*mst22*mu2*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2) + 3*msb22*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) -
     3*msb12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb22*
     mst12*mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*
     mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - msb22*(mt2 + mu2)*
     power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(
     mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*msb12*msb22*mst24*
     mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst24*
     mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*
     msb22*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     14*msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     14*msb12*msb22*mst24*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     5*msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) + 5*msb12*msb22*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) + 3*msb12*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) + 3*msb12*mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) + 5*msb12*msb22*mu2*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb12)) + 14*msb12*msb22*mst24*mu2*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) + 3*msb22*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb12)) - 5*msb12*msb22*mu2*power2(mw2)*power3(mst22)*power3(
     DeltaInv(mst22,mw2,msb22)) - 14*msb12*msb22*mst24*mu2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) - 3*msb12*mu2*power3(mst22)*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) - 5*msb12*msb22*mu2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) - 14*msb12*msb22*mst14*mu2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mu2*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*mu2*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 14*msb12*msb22*mst14*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb12*mu2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*mst22*mt2*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst24*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*mst22*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst24*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 5*msb12*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst24*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 5*msb12*msb22*mst22*mu2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst24*mu2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb22*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 3*msb12*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) + msb12*(mt2 + mu2)*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*
     power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(
     mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 3*msb22*mt2*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb22*mu2*power2(
     mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb12*mt2*power2(
     mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb12*mu2*power2(
     mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 3*msb22*mu2*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 3*msb12*mu2*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 5*msb12*msb22*
     mst22*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 3*msb22*mst24*
     mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 5*msb12*msb22*mst22*
     mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 3*msb12*mst24*mu2*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 5*msb12*msb22*mst12*mu2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*msb22*mst14*mu2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*msb12*msb22*mst12*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb12*mst14*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb22*mst22*mu2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb12*mst22*mu2*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) - 3*msb22*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) + 3*msb12*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) + 3*msb22*mst12*mu2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 3*msb12*mst12*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) + (power2(Pi)*
     power2(snb)*power2(snt)*(-(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*
     mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) +
     msb12*msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 3*
     msb22*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*
     msb22*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*
     mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb12*msb22*
     mst22*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst24*
     mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst22*mu2*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst24*mu2*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + msb12*msb22*mst12*mu2*power2(mw2)
     *power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*msb22*mst12*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 3*msb22*mst22*mt2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 3*msb12*mst22*mt2*power2(DeltaInv(mst22,msb22,
     mC2))*power3(mC2) - 3*msb22*mst22*mu2*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 3*msb12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(
     mw2) + 3*msb22*mst12*mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) -
     3*msb12*mst12*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*
     mt2*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 14*msb12*msb22*
     mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*
     mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*
     mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 14*
     msb12*msb22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 5*
     msb12*msb22*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)
     ) - 3*msb12*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)
     ) - 5*msb12*msb22*mu2*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,
     msb12)) - 14*msb12*msb22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) - 3*msb22*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12)) + 5*msb12*msb22*mu2*power2(mw2)*power3(mst22)*power3(DeltaInv(
     mst22,mw2,msb22)) + 14*msb12*msb22*mst24*mu2*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb22)) + 3*msb12*mu2*power3(mst22)*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb22)) + 5*msb12*msb22*mu2*power2(mw2)*power3(mst12)*power3(
     DeltaInv(mw2,msb12,mst12)) + 14*msb12*msb22*mst14*mu2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 3*msb22*mu2*power3(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 5*msb12*msb22*mu2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst14*mu2*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mu2*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*mst24*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*mst24*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + msb12*mt2*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2)
     + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*
     msb22*power3(mst12) - 3*power4(mst12)) - 3*msb22*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb12*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 3*msb22*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb12*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb22*mu2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*mu2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*msb12*msb22*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*msb22*mst24*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*msb12*mst24*mu2*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2) + 5*msb12*msb22*mst12*mu2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 3*msb22*mst14*mu2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) - 5*msb12*msb22*mst12*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb12*mst14*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst22*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst22*mu2*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mw2) - 3*msb22*mst12*mu2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) + 3*msb12*mst12*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) - (log(msb22/Q2)
     *power2(snb)*(-8 + (8*msb22)/msb12 + msb22/mst12 + mt2/msb22 + mt2/mst22 +
     (mC2*mt2)/(msb22*mst22) + mu2/msb22 + mu2/mst12 + (mu2*mw2)/(msb22*mst12)
     + (2*(5*mh2*msb22 - msb12*msb22 + msb24)*mu2*DeltaInv(mh2,msb12,msb22))/
     msb12 + 2*(mh2 - 6*msb22)*mu2*DeltaInv(mh2,msb22,msb22) + mst22*mt2*
     DeltaInv(mst22,msb22,mC2) - (5*mC2*mst22*mt2*DeltaInv(mst22,msb22,mC2))/
     msb22 - (mst24*mt2*DeltaInv(mst22,msb22,mC2))/msb22 - 2*msb22*mu2*
     DeltaInv(mt2,mu2,msb22) + mst12*mu2*DeltaInv(mw2,msb22,mst12) - (mst14*
     mu2*DeltaInv(mw2,msb22,mst12))/msb22 - (5*mst12*mu2*mw2*DeltaInv(mw2,
     msb22,mst12))/msb22 - (5*mt2*DeltaInv(mst22,msb22,mC2)*power2(mC2))/msb22
     + (mt2*DeltaInv(mst22,msb22,mC2)*power2(mC2))/mst22 - (24*msb22*DeltaInv(
     mt2,mu2,msb22)*power2(mu2))/mt2 - (5*mu2*DeltaInv(mw2,msb22,mst12)*power2(
     mw2))/msb22 + (mu2*DeltaInv(mw2,msb22,mst12)*power2(mw2))/mst12 - 6*mC2*
     mst24*mt2*power2(DeltaInv(mst22,msb22,mC2)) + 36*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + (138*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)))/msb22 - 120*msb24*power2(mu2)*power2(DeltaInv(
     mt2,mu2,msb22)) - 6*mst14*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     mst12*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + (138*mst14*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)))/msb22 - (mt2*DeltaInv(
     mst22,msb22,mC2)*power3(mC2))/(msb22*mst22) - 6*mt2*power2(DeltaInv(mst22,
     msb22,mC2))*power3(mC2) + (138*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))
     *power3(mC2))/msb22 - (48*power2(mu2)*power2(DeltaInv(mt2,mu2,msb22))*
     power3(msb22))/mt2 - (12*mh2*mu2*power2(DeltaInv(mh2,msb12,msb22))*(-3*
     mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*power2(mh2) + power3(
     msb22)))/msb12 - 48*mu2*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 3*
     power3(msb22)) + (6*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mst12))/msb22 + (6*mC2*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22)
     )/msb22 - 48*msb22*power2(DeltaInv(mt2,mu2,msb22))*power3(mu2) - (264*
     msb24*power2(DeltaInv(mt2,mu2,msb22))*power3(mu2))/mt2 - (mu2*DeltaInv(
     mw2,msb22,mst12)*power3(mw2))/(msb22*mst12) - 6*mu2*power2(DeltaInv(mw2,
     msb22,mst12))*power3(mw2) + (138*mst12*mu2*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2))/msb22 - (144*mu2*power2(mh2)*(msb24*power2(mh2) + (-
     2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)))/msb12 + 720*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 216*mt2*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + (144*mt2*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)))/msb22 - 720*power3(msb22)*power3(mu2)*
     power3(DeltaInv(mt2,mu2,msb22)) + 216*mu2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb22,mst12)) + 720*mst14*mu2*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + (144*mu2*power3(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)))/msb22 + (6*mt2*power2(DeltaInv(mst22,msb22,
     mC2))*power4(mC2))/msb22 + 216*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))
     *power4(mC2) + (144*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mC2))/msb22 - 576*mh2*mu2*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22)
     - 72*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22) + (72*
     power3(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22))/mt2 - (144*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12))/msb22 - (144*
     mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22))/msb22 + (
     24*msb22*power2(DeltaInv(mt2,mu2,msb22))*power4(mu2))/mt2 - 360*msb24*
     power3(DeltaInv(mt2,mu2,msb22))*power4(mu2) - (360*power3(msb22)*power3(
     DeltaInv(mt2,mu2,msb22))*power4(mu2))/mt2 + (6*mu2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mw2))/msb22 + 216*mst12*mu2*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mw2) + (144*mst14*mu2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2))/msb22 - (144*mst22*mt2*power3(DeltaInv(mst22,msb22,
     mC2))*power5(mC2))/msb22 + (72*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))
     *power5(msb22))/mt2 + (216*msb24*power3(DeltaInv(mt2,mu2,msb22))*power5(
     mu2))/mt2 - (144*mst12*mu2*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2))/
     msb22))/12. + (mu2*log(msb22/Q2)*power2(sb)*power2(snt)*(mC2*mst12 - mC2*
     mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*mst24*DeltaInv(mst22,mw2,
     msb22) - mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) + msb22*mst22*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*mw2*DeltaInv(mw2,msb22,
     mst12)*power2(mst12) - msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     + 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22) - msb22*mst12*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*mst22*DeltaInv(mst22,mw2,
     msb22)*power2(mw2) + msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 6*msb22*mst12*mst22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) - 138*mst12*mst22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*msb22*mst12*power2(
     mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*
     mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*mst12*mst14*mst22*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*msb22*mst22*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2) - mst22*
     DeltaInv(mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mst12*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*power2(mst12)*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 216*msb22*mst22*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mst22) - 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) -
     216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*mst22) + (mu2*log(msb22/Q2)*power2(sb)*power2(snb)*
     power2(snt)*(mC2*mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*
     mst24*DeltaInv(mst22,mw2,msb22) - mst12*mst14*mst22*DeltaInv(mw2,msb22,
     mst12) + msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*
     mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) - msb22*mst12*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(
     mst22) - msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*
     mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(
     mw2) + 6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) -
     138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     - mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(12.*msb22*mst12*mst22) + (mu2*log(msb22/Q2)*power2(sb)*power2(snt)
     *(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) -
     msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*DeltaInv(mst22,mw2,
     msb22)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     + mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     + 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(12.*msb22*mst12*mst22) + (mu2*log(msb22/Q2)*power2(sb)*power2(snb)
     *power2(snt)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*
     mst22*mst24*DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*DeltaInv(mw2,
     msb22,mst12) - msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*
     mst22*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*DeltaInv(
     mst22,mw2,msb22)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*
     power2(mst22) + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*
     mst12*mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*DeltaInv(
     mw2,msb22,mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*
     power2(mw2) - 6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,
     msb22)) + 138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) + 36*msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,
     mst12)) - 138*mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) - 36*msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) - mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) +
     mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (
     msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*
     msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(
     msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb22)*
     power3(mw2) + mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*power2(
     mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*mst22*
     power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-
     ((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22
     + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     + 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*mst22) + (power2(cb)*power2(Pi)*power2(snb)*(-(
     msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*(mst12 - mst22 - 2*mt2)*mu2*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + msb12*msb22*mst12*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst12*mst24*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst22*mst24*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst22*mt2*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb22*mst24*mt2*mu2*power2(mC2)
     *power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mt2*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst12*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst12*mst24*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*mst24*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst22*mt2*
     mu2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*mst24*mt2*mu2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mt2*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb22*mst12*
     mst14*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*msb22*
     mst12*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 3*msb22*
     mst14*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + msb12*
     msb22*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 3*
     msb12*mst12*mst14*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     msb12*msb22*mst12*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     + 3*msb12*mst14*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     msb12*msb22*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) + 3*msb22*mst12*mst22*mt2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2)
     - 6*msb22*mst22*mt2*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 3*
     msb22*mt2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 3*
     msb12*mst12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 6*
     msb12*mst22*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 3*
     msb12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 3*
     msb22*mst12*mst22*mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*
     msb22*mu2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*
     msb12*mst12*mst22*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 3*
     msb12*mu2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) +
     msb22*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)
     *power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     + 14*msb12*msb22*mst12*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 14*msb12*msb22*mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 28*msb12*msb22*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 5*msb12*msb22*mst12*mt2*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 10*msb12*msb22*mt2*mu2*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst12*mt2*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 6*msb22*mt2*
     mu2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 14*
     msb12*msb22*mst12*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     + 14*msb12*msb22*mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 28*msb12*msb22*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) - 5*msb12*msb22*mst12*mt2*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 10*msb12*msb22*mt2*mu2*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst12*mt2*power3(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 6*msb12*mt2*mu2*power3(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*
     mst22*mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     14*msb12*msb22*mst12*mst14*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 14*msb12*msb22*mst14*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 3*msb22*mst22*mu2*power3(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*mst22*mu2*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst12*mst14*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 14*msb12*msb22*mst14*
     mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst22*
     mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*
     msb22*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*
     msb22*mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*
     msb22*mst22*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 10*
     msb12*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) -
     6*msb22*mst24*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*
     msb12*msb22*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) - 5*msb12*msb22*mst12*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 3*msb12*mst12*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 3*msb12*mst22*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) + 10*msb12*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mC2) + 6*msb12*mst24*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2)
     )*power4(mC2) + 5*msb12*msb22*mt2*power2(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 5*msb12*msb22*mu2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 3*msb22*mst22*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb22*mu2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 5*msb12*msb22*mu2*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 3*msb12*mst22*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 3*msb12*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - msb12*(mst12
     - mst22 - 2*mt2)*mu2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*
     mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*
     mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*
     power4(mst12)) - 5*msb12*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 3*msb22*mst12*mt2*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) + 6*msb22*mt2*mu2*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb22*mt2*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 5*msb12*msb22*mt2*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb12*mst12*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 6*msb12*mt2*
     mu2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb12*
     mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb22*
     mst12*mst14*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 5*msb12*
     msb22*mst12*mst22*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*
     msb22*mst14*mst22*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*
     msb12*msb22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 3*msb12*mst12*mst14*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 5*msb12*msb22*mst12*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) + 3*msb12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) - 5*msb12*msb22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2) - 3*msb22*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,
     mC2))*power5(mC2) + 6*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2)
     )*power5(mC2) + 3*msb22*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     )*power5(mC2) + 3*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 6*msb12*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 3*msb12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 3*msb22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mst12) + 3*msb12*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mst12) + 3*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mst22) - 3*msb12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mst22) + 3*msb22*mst12*mst22*mu2*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 3*msb22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) - 3*msb12*mst12*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mw2) + 3*msb12*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mw2)))/(6.*msb12*msb22*(mst12 - mst22)) + (power2(cb)*power2(snb)*(
     -(mC2*msb12*mst12) + mC2*msb22*mst12 + mC2*msb12*mst22 - mC2*msb22*mst22 -
     msb12*mst12*mt2 + msb22*mst12*mt2 + msb12*mst22*mt2 - msb22*mst22*mt2 +
     msb12*mst12*mw2 - msb22*mst12*mw2 - msb12*mst22*mw2 + msb22*mst22*mw2 + 2*
     mC2*msb12*msb22*(mst12 - mst22 - 2*mt2)*mu2*DeltaInv(msb12,mst12,mC2) + 2*
     mC2*msb12*msb22*(-mst12 + mst22 + 2*mt2)*mu2*DeltaInv(msb22,mst12,mC2) -
     2*mC2*msb12*msb22*mst12*mt2*DeltaInv(mst22,msb12,mC2) + 2*mC2*msb12*msb22*
     mst22*mt2*DeltaInv(mst22,msb12,mC2) + 4*mC2*msb12*msb22*mt2*mu2*DeltaInv(
     mst22,msb12,mC2) + 2*mC2*msb12*msb22*mst12*mt2*DeltaInv(mst22,msb22,mC2) -
     2*mC2*msb12*msb22*mst22*mt2*DeltaInv(mst22,msb22,mC2) - 4*mC2*msb12*msb22*
     mt2*mu2*DeltaInv(mst22,msb22,mC2) - 2*msb12*msb22*mst12*mu2*mw2*DeltaInv(
     mw2,msb12,mst12) + 2*msb12*msb22*mst22*mu2*mw2*DeltaInv(mw2,msb12,mst12) +
     2*msb12*msb22*mst12*mu2*mw2*DeltaInv(mw2,msb22,mst12) - 2*msb12*msb22*
     mst22*mu2*mw2*DeltaInv(mw2,msb22,mst12) - 84*msb22*(3*mC2*mst12 + msb12*
     mst12 + 3*mst14)*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)) + 84*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*(mst12
     - mst22 - 2*mt2)*mu2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 84*
     msb12*msb22*mst12*mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     + 252*msb22*mst12*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     - 252*msb22*mst22*mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     - 168*msb12*msb22*mst22*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) - 504*msb22*mst24*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,
     mC2)) - 84*msb12*msb22*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) - 84*msb12*msb22*mst12*mst22*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 252*msb12*mst12*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 252*msb12*mst22*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 168*msb12*msb22*mst22*mt2*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 504*msb12*mst24*mt2*mu2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*msb22*mt2*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 252*msb22*mst12*mst14*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb12*msb22*mst12*
     mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 252*msb22*mst14*
     mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb12*msb22*
     mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 252*
     msb12*mst12*mst14*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 84*
     msb12*msb22*mst12*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     + 252*msb12*mst14*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - 84*msb12*msb22*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) + 252*msb22*mst12*mst22*mt2*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) - 504*msb22*mst22*mt2*mu2*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) - 252*msb22*mt2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)
     )*power3(mC2) - 252*msb12*mst12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2)
     )*power3(mC2) + 504*msb12*mst22*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 252*msb12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)
     )*power3(mC2) - 252*msb22*mst12*mst22*mu2*power2(DeltaInv(mw2,msb12,mst12)
     )*power3(mw2) + 252*msb22*mu2*power2(mst12)*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 252*msb12*mst12*mst22*mu2*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 252*msb12*mu2*power2(mst12)*power2(DeltaInv(mw2,
     msb22,mst12))*power3(mw2) + 84*msb22*(mst12 - mst22 - 2*mt2)*mu2*power2(
     mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-
     5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))
     *power3(DeltaInv(msb12,mst12,mC2)) + 1176*msb12*msb22*mst12*mst24*mt2*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 1176*msb12*msb22*mst22*
     mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 2352*msb12*
     msb22*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 420*
     msb12*msb22*mst12*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) - 840*msb12*msb22*mt2*mu2*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 252*msb22*mst12*mt2*power3(mC2)*power3(mst22)
     *power3(DeltaInv(mst22,msb12,mC2)) - 504*msb22*mt2*mu2*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 1176*msb12*msb22*mst12*mst24*
     mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 1176*msb12*msb22*
     mst22*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 2352*
     msb12*msb22*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     420*msb12*msb22*mst12*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)) + 840*msb12*msb22*mt2*mu2*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) - 252*msb12*mst12*mt2*power3(mC2)*power3(mst22)
     *power3(DeltaInv(mst22,msb22,mC2)) + 504*msb12*mt2*mu2*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 420*msb12*msb22*mst22*mu2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 1176*msb12*
     msb22*mst12*mst14*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     1176*msb12*msb22*mst14*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) - 252*msb22*mst22*mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) + 420*msb12*msb22*mst22*mu2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb22,mst12)) - 1176*msb12*msb22*mst12*mst14*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 1176*msb12*msb22*mst14*
     mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 252*msb12*mst22*
     mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 420*
     msb12*msb22*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     + 252*msb22*mst12*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     - 252*msb22*mst22*mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2)
     - 840*msb12*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) - 504*msb22*mst24*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mC2) - 420*msb12*msb22*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     *power4(mC2) - 420*msb12*msb22*mst12*mst22*mt2*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) - 252*msb12*mst12*mst24*mt2*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 252*msb12*mst22*mst24*mt2*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) + 840*msb12*msb22*mst22*mt2*mu2*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 504*msb12*mst24*mt2*mu2*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 420*msb12*msb22*mt2*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 420*msb12*msb22*mu2*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 252*msb22*mst22*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 252*msb22*
     mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 420*
     msb12*msb22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mst12) - 252*msb12*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     *power4(mst12) - 252*msb12*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) - 84*msb12*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*
     power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(
     mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) - 420*msb12*msb22*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 252*msb22*
     mst12*mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) +
     504*msb22*mt2*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mst22) - 252*msb22*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mst22) + 420*msb12*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22) + 252*msb12*mst12*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) - 504*msb12*mt2*mu2*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 252*msb12*mt2*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 252*msb22*mst12*
     mst14*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 420*msb12*msb22*
     mst12*mst22*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 252*msb22*
     mst14*mst22*mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 420*msb12*
     msb22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) -
     252*msb12*mst12*mst14*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) +
     420*msb12*msb22*mst12*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 252*msb12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 420*msb12*msb22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))
     *power4(mw2) - 252*msb22*mst12*mst22*mt2*power3(DeltaInv(mst22,msb12,mC2))
     *power5(mC2) + 504*msb22*mst22*mt2*mu2*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 252*msb22*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     )*power5(mC2) + 252*msb12*mst12*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 504*msb12*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 252*msb12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 252*msb22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     )*power5(mst12) + 252*msb12*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power5(mst12) + 252*msb22*mt2*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mst22) - 252*msb12*mt2*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*power5(mst22) + 252*msb22*mst12*mst22*mu2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 252*msb22*mu2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 252*msb12*mst12*mst22*mu2*
     power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) + 252*msb12*mu2*power2(
     mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(12.*msb12*msb22*(
     mst12 - mst22)) + (power2(snt)*(-(mC2*msb22*mst12) + mC2*msb22*mst22 +
     msb22*mst12*mt2 - msb22*mst22*mt2 + msb22*mst12*mu2 - msb22*mst22*mu2 + 2*
     mC2*msb22*mst12*mst22*mt2*DeltaInv(msb22,mst12,mC2) - 2*mC2*msb22*mst12*
     mst22*mt2*DeltaInv(mst22,msb22,mC2) + 2*msb22*mst12*mst22*mu2*mw2*
     DeltaInv(mst22,mw2,msb22) - 2*msb22*mst12*mst22*mu2*mw2*DeltaInv(mw2,
     msb22,mst12) - mst12*power2(msb22) + mst22*power2(msb22) + mst22*power2(
     mst12) - mst12*power2(mst22) - 84*mst12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*mst22*mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 84*mst12*
     mst22*(3*mC2*mst22 + msb22*mst22 + 3*mst24)*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 252*mst12*mst22*mst24*mu2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 84*msb22*mst12*mu2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 252*mst12*mst14*mst22*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 84*msb22*mst22*mu2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 252*mst12*mu2*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 252*mst22*mu2*
     power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 84*mst12*
     mst22*mt2*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 1176*msb22*mst12*
     mst22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 1176*
     msb22*mst12*mst14*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     + 420*msb22*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) + 252*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mst12) - 420*msb22*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 252*mst12*mu2*power3(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power4(mst22) - 84*mst12*mst22*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*(-14*mC2*msb22*mst24 - 5*msb22*mst22*power2(
     mC2) - 3*mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*mC2*power3(mst22) -
     5*msb22*power3(mst22) + 3*power4(mst22)) - 252*mst12*mst22*mst24*mu2*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 420*msb22*mst12*mu2*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 252*mst12*
     mst14*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 420*msb22*
     mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) -
     252*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mst12)
     + 252*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 252*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) - 252*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12)
     )*power5(mw2)))/(12.*msb22*mst12*mst22) + (log(msb22/Q2)*power2(cb)*
     power2(snt)*(mC2*mst12*mt2 + msb22*mst12*mt2 - mC2*mst22*mt2 - msb22*
     mst22*mt2 + mC2*mst12*mu2 - mC2*mst22*mu2 - mst12*mu2*mw2 + mst22*mu2*mw2
     + mst12*mst22*mst24*mu2*DeltaInv(mst22,mw2,msb22) - mst12*mst14*mst22*mu2*
     DeltaInv(mw2,msb22,mst12) + msb22*mst22*mu2*DeltaInv(mw2,msb22,mst12)*
     power2(mst12) - 5*mst22*mu2*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) -
     msb22*mst12*mu2*DeltaInv(mst22,mw2,msb22)*power2(mst22) + 5*mst12*mu2*mw2*
     DeltaInv(mst22,mw2,msb22)*power2(mst22) - msb22*mst12*mu2*DeltaInv(mst22,
     mw2,msb22)*power2(mw2) + 5*mst12*mst22*mu2*DeltaInv(mst22,mw2,msb22)*
     power2(mw2) + msb22*mst22*mu2*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 5*
     mst12*mst22*mu2*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 6*msb22*mst12*
     mst22*mst24*mu2*mw2*power2(DeltaInv(mst22,mw2,msb22)) - 138*mst12*mst22*
     mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*msb22*mst12*
     mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*
     mst12*mst14*mst22*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*mst12*
     mst14*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*msb22*
     mst22*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     mst22*(mt2 + mu2)*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) +
     mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) - mst12*(mt2 + mu2)*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) +
     mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)
     ) - 6*mC2*mst12*mst22*(mt2 + mu2)*power2(DeltaInv(msb22,mst12,mC2))*(-(
     msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*(mt2 + mu2)*
     power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 +
     23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     + mst12*mu2*DeltaInv(mst22,mw2,msb22)*power3(mw2) - mst22*mu2*DeltaInv(
     mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*mu2*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) - 138*mst12*mu2*power2(mst22)*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*mu2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*mu2*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*(mt2 + mu2)
     *power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*(mt2 + mu2)*
     power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*
     mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*
     mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mu2*mw2*
     power2(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 216*msb22*mst22*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 144*mst22*
     mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 6*mst12*
     mu2*mw2*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 216*msb22*mst12*
     mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 144*
     mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 6*
     mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 6*mst12*
     mst22*mu2*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 144*mst12*mst22*
     mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 216*msb22*mst12*
     mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 144*
     mst12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 216*
     msb22*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mst22) + 144*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) - 144*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,
     msb22,mst12))*power5(mw2)))/(12.*msb22*mst12*mst22) + (log(msb22/Q2)*
     power2(snt)*(-(mC2*mst12*mt2) - msb22*mst12*mt2 + mC2*mst22*mt2 + msb22*
     mst22*mt2 + msb22*mst12*mu2 - msb22*mst22*mu2 + mst12*mu2*mw2 - mst22*mu2*
     mw2 - mst12*mst22*mst24*mu2*DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*
     mu2*DeltaInv(mw2,msb22,mst12) + mst12*power2(msb22) - mst22*power2(msb22)
     - msb22*mst22*mu2*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mu2*
     mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*mu2*DeltaInv(
     mst22,mw2,msb22)*power2(mst22) - 5*mst12*mu2*mw2*DeltaInv(mst22,mw2,msb22)
     *power2(mst22) + msb22*mst12*mu2*DeltaInv(mst22,mw2,msb22)*power2(mw2) -
     5*mst12*mst22*mu2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*mu2*
     DeltaInv(mw2,msb22,mst12)*power2(mw2) + 5*mst12*mst22*mu2*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mst24*mu2*mw2*power2(
     DeltaInv(mst22,mw2,msb22)) + 138*mst12*mst22*mst24*mu2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 36*msb22*mst12*mu2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mu2*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 138*mst12*mst14*mst22*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 36*msb22*mst22*mu2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - mst22*mt2*DeltaInv(msb22,
     mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2)
     + 5*mC2*power2(mst12) + power3(mC2)) + mst12*mt2*DeltaInv(mst22,msb22,mC2)
     *(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*
     power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*mt2*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*
     mst12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(
     6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - mst12*mu2*DeltaInv(mst22,mw2,msb22)*power3(mw2) + mst22*
     mu2*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*mst22*mu2*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*mu2*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*
     mst22*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*mu2*
     power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*
     mst22*mt2*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*mt2*
     power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*
     mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*
     mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mu2*mw2*
     power2(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 216*msb22*mst22*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 144*mst22*
     mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 6*mst12*
     mu2*mw2*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 216*msb22*mst12*
     mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 144*
     mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 6*
     mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*mst12*
     mst22*mu2*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*mst12*mst22*
     mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 216*msb22*mst12*
     mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 144*
     mst12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 216*
     msb22*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mst22) - 144*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) + 144*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,
     msb22,mst12))*power5(mw2)))/(12.*msb22*mst12*mst22) - (log(msb22/Q2)*
     power2(snb)*power2(snt)*(-(mC2*mst12*mt2) - msb22*mst12*mt2 + mC2*mst22*
     mt2 + msb22*mst22*mt2 + msb22*mst12*mu2 - msb22*mst22*mu2 + mst12*mu2*mw2
     - mst22*mu2*mw2 - mst12*mst22*mst24*mu2*DeltaInv(mst22,mw2,msb22) + mst12*
     mst14*mst22*mu2*DeltaInv(mw2,msb22,mst12) + mst12*power2(msb22) - mst22*
     power2(msb22) - msb22*mst22*mu2*DeltaInv(mw2,msb22,mst12)*power2(mst12) +
     5*mst22*mu2*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*mu2*
     DeltaInv(mst22,mw2,msb22)*power2(mst22) - 5*mst12*mu2*mw2*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) + msb22*mst12*mu2*DeltaInv(mst22,mw2,msb22)*
     power2(mw2) - 5*mst12*mst22*mu2*DeltaInv(mst22,mw2,msb22)*power2(mw2) -
     msb22*mst22*mu2*DeltaInv(mw2,msb22,mst12)*power2(mw2) + 5*mst12*mst22*mu2*
     DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mst24*mu2*mw2*
     power2(DeltaInv(mst22,mw2,msb22)) + 138*mst12*mst22*mst24*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) + 36*msb22*mst12*mu2*power2(mst22)*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mst12*mst14*mst22*
     mu2*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 138*mst12*mst14*mst22*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 36*msb22*mst22*mu2*power2(
     mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - mst22*mt2*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*mt2*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*mt2*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - 6*mC2*mst12*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24)
     + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*mu2*DeltaInv(mst22,mw2,msb22)*power3(
     mw2) + mst22*mu2*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*
     mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*mu2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*
     mst12*mst22*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*
     mu2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*
     mst12*mst22*mt2*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*
     mst22*mt2*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*
     mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*
     mst14*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*
     mu2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 216*msb22*mst22*
     mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 144*
     mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 6*
     mst12*mu2*mw2*power2(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 216*msb22*
     mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) +
     144*mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22)
     + 6*mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 6*
     mst12*mst22*mu2*power2(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 144*mst12*
     mst22*mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 216*msb22*
     mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) -
     144*mst12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) -
     216*msb22*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) + 144*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mst12) - 144*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22)
     )*power5(mst22) - 144*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) + 144*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,
     msb22,mst12))*power5(mw2)))/(12.*msb22*mst12*mst22) + (log(msb22/Q2)*
     power2(cb)*power2(snb)*power2(snt)*(-(mC2*mst12*mt2) - msb22*mst12*mt2 +
     mC2*mst22*mt2 + msb22*mst22*mt2 - mC2*mst12*mu2 + mC2*mst22*mu2 + mst12*
     mu2*mw2 - mst22*mu2*mw2 - mst12*mst22*mst24*mu2*DeltaInv(mst22,mw2,msb22)
     + mst12*mst14*mst22*mu2*DeltaInv(mw2,msb22,mst12) - msb22*mst22*mu2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mu2*mw2*DeltaInv(mw2,
     msb22,mst12)*power2(mst12) + msb22*mst12*mu2*DeltaInv(mst22,mw2,msb22)*
     power2(mst22) - 5*mst12*mu2*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22) +
     msb22*mst12*mu2*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*mst12*mst22*mu2*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*mu2*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) + 5*mst12*mst22*mu2*DeltaInv(mw2,msb22,mst12)*
     power2(mw2) - 6*msb22*mst12*mst22*mst24*mu2*mw2*power2(DeltaInv(mst22,mw2,
     msb22)) + 138*mst12*mst22*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) + 36*msb22*mst12*mu2*power2(mst22)*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mu2*mw2*power2(DeltaInv(mw2,
     msb22,mst12)) - 138*mst12*mst14*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) - 36*msb22*mst22*mu2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - mst22*(mt2 + mu2)*DeltaInv(msb22,mst12,mC2)*(
     mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*(mt2 + mu2)*DeltaInv(mst22,msb22,mC2)
     *(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*
     power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*(mt2 + mu2)*power2(
     DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*
     mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) -
     6*mC2*mst12*mst22*(mt2 + mu2)*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*mu2*DeltaInv(mst22,mw2,msb22)*power3(
     mw2) + mst22*mu2*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*
     mst22*mu2*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*mu2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*
     mst12*mst22*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*
     mu2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*
     mst12*mst22*(mt2 + mu2)*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(
     mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(
     5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*
     mst12*mst22*(mt2 + mu2)*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(
     mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(
     5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 720*
     msb22*mst12*mst22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22))
     - 720*msb22*mst12*mst14*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 6*mst22*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12))*power4(mst12)
     - 216*msb22*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mst12) - 144*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mst12) + 6*mst12*mu2*mw2*power2(DeltaInv(mst22,mw2,msb22))*
     power4(mst22) + 216*msb22*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 144*mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 6*mst12*mst22*mu2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 6*mst12*mst22*mu2*power2(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) + 144*mst12*mst22*mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 216*msb22*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 144*mst12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2) - 216*msb22*mst22*mu2*power2(mst12)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 144*mst22*mu2*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power5(mst12) - 144*mst12*mu2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power5(mst22) - 144*mst12*mu2*power2(mst22)*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) + 144*mst22*mu2*power2(
     mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(12.*msb22*mst12*
     mst22) + (power2(cb)*power2(snt)*(mC2*msb22*mst12 - mC2*msb22*mst22 -
     msb22*mst12*mt2 + msb22*mst22*mt2 - msb22*mst12*mw2 + msb22*mst22*mw2 - 2*
     mC2*msb22*mst12*mst22*(mt2 + mu2)*DeltaInv(msb22,mst12,mC2) + 2*mC2*msb22*
     mst12*mst22*(mt2 + mu2)*DeltaInv(mst22,msb22,mC2) - 2*msb22*mst12*mst22*
     mu2*mw2*DeltaInv(mst22,mw2,msb22) + 2*msb22*mst12*mst22*mu2*mw2*DeltaInv(
     mw2,msb22,mst12) + 84*mst12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mst22*(
     mt2 + mu2)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 84*mst12*mst22*
     (3*mC2*mst22 + msb22*mst22 + 3*mst24)*(mt2 + mu2)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 252*mst12*mst22*mst24*mu2*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + 84*msb22*mst12*mu2*power2(mst22)*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 252*mst12*mst14*mst22*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 84*msb22*mst22*mu2*power2(mst12)*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 252*mst12*mu2*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 252*mst22*mu2*
     power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 84*mst12*
     mst22*(mt2 + mu2)*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) +
     3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 1176*msb22*
     mst12*mst22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) -
     1176*msb22*mst12*mst14*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 420*msb22*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mst12) - 252*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 420*msb22*mst12*mu2*power2(mw2)*power3(DeltaInv(
     mst22,mw2,msb22))*power4(mst22) + 252*mst12*mu2*power3(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 84*mst12*mst22*(mt2 + mu2)*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-14*mC2*msb22*mst24 - 5*
     msb22*mst22*power2(mC2) - 3*mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*
     mC2*power3(mst22) - 5*msb22*power3(mst22) + 3*power4(mst22)) + 252*mst12*
     mst22*mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 420*msb22*
     mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) -
     252*mst12*mst14*mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) -
     420*msb22*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) + 252*mst22*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*
     power5(mst12) - 252*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22)
     )*power5(mst22) - 252*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) + 252*mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,
     msb22,mst12))*power5(mw2)))/(12.*msb22*mst12*mst22) - (power2(snb)*power2(
     snt)*(-2*mC2*msb12*msb22*mst12*mst22*mt2*DeltaInv(msb12,mst12,mC2) + 2*
     mC2*msb12*msb22*mst12*mst22*mt2*DeltaInv(msb22,mst12,mC2) + 2*mC2*msb12*
     msb22*mst12*mst22*mt2*DeltaInv(mst22,msb12,mC2) - 2*mC2*msb12*msb22*mst12*
     mst22*mt2*DeltaInv(mst22,msb22,mC2) - 2*msb12*msb22*mst12*mst22*mu2*mw2*
     DeltaInv(mst22,mw2,msb12) + 2*msb12*msb22*mst12*mst22*mu2*mw2*DeltaInv(
     mst22,mw2,msb22) + 2*msb12*msb22*mst12*mst22*mu2*mw2*DeltaInv(mw2,msb12,
     mst12) - 2*msb12*msb22*mst12*mst22*mu2*mw2*DeltaInv(mw2,msb22,mst12) +
     msb22*mst12*power2(msb12) - msb22*mst22*power2(msb12) - msb12*mst12*
     power2(msb22) + msb12*mst22*power2(msb22) + msb12*mst22*power2(mst12) -
     msb22*mst22*power2(mst12) - msb12*mst12*power2(mst22) + msb22*mst12*
     power2(mst22) + 84*msb22*mst12*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*
     mst22*mt2*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - 84*msb12*mst12*(
     3*mC2*mst12 + msb22*mst12 + 3*mst14)*mst22*mt2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - 252*msb22*mst12*mst22*mst24*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) - 84*msb12*msb22*mst12*mt2*power2(mC2)*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) + 252*msb12*mst12*mst22*
     mst24*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*msb22*
     mst12*mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) +
     252*msb22*mst12*mst22*mst24*mu2*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 84*msb12*msb22*mst12*mu2*power2(mst22)*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - 252*msb12*mst12*mst22*mst24*mu2*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*msb22*mst12*mu2*power2(mst22)
     *power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 252*msb22*mst12*mst14*
     mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb12*msb22*
     mst22*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     252*msb12*mst12*mst14*mst22*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) + 84*msb12*msb22*mst22*mu2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 252*msb22*mst12*mt2*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) + 252*msb12*mst12*mt2*power2(mst22)
     *power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) + 252*msb22*mst12*mu2*
     power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 252*msb12*
     mst12*mu2*power2(mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) -
     252*msb22*mst22*mu2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) + 252*msb12*mst22*mu2*power2(mst12)*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) - 84*msb22*mst12*mst22*mt2*power2(mC2)*(-((5*msb12*
     mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)
     *power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 1176*msb12*msb22*mst12*mst22*mst24*mt2*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 1176*msb12*msb22*mst12*mst22*mst24*
     mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 1176*msb12*msb22*
     mst12*mst22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) -
     1176*msb12*msb22*mst12*mst22*mst24*mu2*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22)) - 1176*msb12*msb22*mst12*mst14*mst22*mu2*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) + 1176*msb12*msb22*mst12*mst14*mst22*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 252*msb22*mst12*mst22*
     mst24*mt2*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 420*msb12*msb22*
     mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) +
     252*msb12*mst12*mst22*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mC2) + 420*msb12*msb22*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2) - 420*msb12*msb22*mst22*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) - 252*msb22*mst22*mu2*power3(mw2)
     *power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 420*msb12*msb22*mst22*
     mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 252*
     msb12*mst22*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mst12) + 84*msb12*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(msb22,mst12,
     mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(
     mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) +
     3*power4(mst12)) - 420*msb12*msb22*mst12*mt2*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) - 252*msb22*mst12*mt2*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 420*msb12*msb22*mst12*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 252*msb12*
     mst12*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) +
     420*msb12*msb22*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mst22) + 252*msb22*mst12*mu2*power3(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 420*msb12*msb22*mst12*mu2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 252*msb12*mst12*mu2*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 252*msb22*mst12*mst22*
     mst24*mu2*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 420*msb12*msb22*
     mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) -
     252*msb12*mst12*mst22*mst24*mu2*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 420*msb12*msb22*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2) - 252*msb22*mst12*mst14*mst22*mu2*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mw2) - 420*msb12*msb22*mst22*mu2*power2(mst12)*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 252*msb12*mst12*mst14*
     mst22*mu2*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 420*msb12*msb22*
     mst22*mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) +
     252*msb22*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) - 252*msb12*mst12*mt2*power2(mst22)*power3(DeltaInv(mst22,
     msb22,mC2))*power5(mC2) + 252*msb22*mst22*mu2*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power5(mst12) - 252*msb12*mst22*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mst12) + 252*msb22*mst12*mt2*power2(mC2)
     *power3(DeltaInv(mst22,msb12,mC2))*power5(mst22) - 252*msb12*mst12*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power5(mst22) - 252*msb22*
     mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) +
     252*msb12*mst12*mu2*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 252*msb22*mst12*mu2*power2(mst22)*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 252*msb12*mst12*mu2*power2(mst22)*power3(DeltaInv(
     mst22,mw2,msb22))*power5(mw2) + 252*msb22*mst22*mu2*power2(mst12)*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) - 252*msb12*mst22*mu2*power2(mst12)
     *power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(12.*msb12*msb22*mst12*
     mst22) + (log(msb22/Q2)*(((5*mh2*msb22 - msb12*msb22 + msb24)*mu2*
     DeltaInv(mh2,msb12,msb22))/msb12 - (mu2*(msb12*msb22 - msb24 - 5*msb22*
     mz2)*DeltaInv(msb12,mz2,msb22))/msb12 - (6*mu2*mz2*power2(DeltaInv(msb12,
     mz2,msb22))*(msb22*mz2*(-3*msb12 + mz2) - msb24*(msb12 + 14*mz2) + power3(
     msb22)))/msb12 - (6*mh2*mu2*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*
     msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*power2(mh2) + power3(
     msb22)))/msb12 - (72*mu2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 +
     msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)))/msb12 - (72*mu2*power2(mz2)*power3(DeltaInv(
     msb12,mz2,msb22))*(msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*msb12*(
     msb24*mz2 + power3(msb22)) + power4(msb22)))/msb12 + 2*(msb22/msb12 +
     msb22/mst12 + mt2/msb22 + mt2/mst22 + (mC2*mt2)/(msb22*mst22) - (2*msb22)/
     (msb22 - mu2) + mu2/msb22 + mu2/mst12 + (mu2*mw2)/(msb22*mst12) - (2*
     msb22*mu2*(mt2 + 12*mu2)*DeltaInv(mt2,mu2,msb22))/mt2 + mst12*mu2*
     DeltaInv(mw2,msb22,mst12) - (mst14*mu2*DeltaInv(mw2,msb22,mst12))/msb22 -
     (5*mst12*mu2*mw2*DeltaInv(mw2,msb22,mst12))/msb22 - (5*mu2*DeltaInv(mw2,
     msb22,mst12)*power2(mw2))/msb22 + (mu2*DeltaInv(mw2,msb22,mst12)*power2(
     mw2))/mst12 - 6*mst14*mu2*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     mst12*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + (138*mst14*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)))/msb22 - (mt2*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)))/(msb22*mst22) - (24*
     power2(mu2)*power2(DeltaInv(mt2,mu2,msb22))*(msb22*(2*mt2 - mu2)*mu2 +
     msb24*(5*mt2 + 11*mu2) + 2*power3(msb22)))/mt2 + (6*mu2*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12))/msb22 + (6*mC2*mt2*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)))/
     msb22 - (mu2*DeltaInv(mw2,msb22,mst12)*power3(mw2))/(msb22*mst12) - 6*mu2*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + (138*mst12*mu2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2))/msb22 + (72*mt2*power2(mC2)*((3*
     msb22*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*
     mst22)*power3(mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)))/msb22 + 216*mu2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb22,mst12)) + 720*mst14*mu2*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) + (144*mu2*power3(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)))/msb22 - (144*mu2*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12))/msb22 + (6*mu2*power2(DeltaInv(mw2,msb22,
     mst12))*power4(mw2))/msb22 + 216*mst12*mu2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2) + (144*mst14*mu2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2))/msb22 + (72*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*(
     msb24*(-5*mt2 + 3*mu2)*power2(mu2) - 5*mu2*(2*mt2 + mu2)*power3(msb22) + (
     -mt2 + mu2)*power4(msb22) + power5(msb22)))/mt2 - (144*mst12*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2))/msb22)))/24. + Fin3(mh2,msb12,
     msb12,Q2)*(-2*mu2*power2(snb)*power2(DeltaInv(mh2,msb12,msb12))*(msb14 +
     4*mh2*DeltaInv(mh2,msb12,msb12)*power6(msb1)) - (4*ca*mu2*sa*sb*power2(
     snb)*power2(DeltaInv(mh2,msb12,msb12))*(msb14 + 4*mh2*DeltaInv(mh2,msb12,
     msb12)*power6(msb1)))/cb + 2*mu2*power2(sa)*power2(snb)*power2(DeltaInv(
     mh2,msb12,msb12))*(msb14 + 4*mh2*DeltaInv(mh2,msb12,msb12)*power6(msb1)) -
     (2*mu2*power2(sa)*power2(sb)*power2(snb)*power2(DeltaInv(mh2,msb12,msb12))
     *(msb14 + 4*mh2*DeltaInv(mh2,msb12,msb12)*power6(msb1)))/power2(cb) + 2*
     mu2*power2(DeltaInv(mh2,msb12,msb12))*power4(snb)*(msb14 + 4*mh2*DeltaInv(
     mh2,msb12,msb12)*power6(msb1)) + (4*ca*mu2*sa*sb*power2(DeltaInv(mh2,
     msb12,msb12))*power4(snb)*(msb14 + 4*mh2*DeltaInv(mh2,msb12,msb12)*power6(
     msb1)))/cb - 2*mu2*power2(sa)*power2(DeltaInv(mh2,msb12,msb12))*power4(
     snb)*(msb14 + 4*mh2*DeltaInv(mh2,msb12,msb12)*power6(msb1)) + (2*mu2*
     power2(sa)*power2(sb)*power2(DeltaInv(mh2,msb12,msb12))*power4(snb)*(msb14
     + 4*mh2*DeltaInv(mh2,msb12,msb12)*power6(msb1)))/power2(cb)) + Fin3(mH2,
     msb12,msb12,Q2)*((4*ca*mu2*sa*sb*power2(snb)*power2(DeltaInv(mH2,msb12,
     msb12))*(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*power6(msb1)))/cb - 2*
     mu2*power2(sa)*power2(snb)*power2(DeltaInv(mH2,msb12,msb12))*(msb14 + 4*
     mH2*DeltaInv(mH2,msb12,msb12)*power6(msb1)) - (2*mu2*power2(sb)*power2(
     snb)*power2(DeltaInv(mH2,msb12,msb12))*(msb14 + 4*mH2*DeltaInv(mH2,msb12,
     msb12)*power6(msb1)))/power2(cb) + (2*mu2*power2(sa)*power2(sb)*power2(
     snb)*power2(DeltaInv(mH2,msb12,msb12))*(msb14 + 4*mH2*DeltaInv(mH2,msb12,
     msb12)*power6(msb1)))/power2(cb) - (4*ca*mu2*sa*sb*power2(DeltaInv(mH2,
     msb12,msb12))*power4(snb)*(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*power6(
     msb1)))/cb + 2*mu2*power2(sa)*power2(DeltaInv(mH2,msb12,msb12))*power4(
     snb)*(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*power6(msb1)) + (2*mu2*
     power2(sb)*power2(DeltaInv(mH2,msb12,msb12))*power4(snb)*(msb14 + 4*mH2*
     DeltaInv(mH2,msb12,msb12)*power6(msb1)))/power2(cb) - (2*mu2*power2(sa)*
     power2(sb)*power2(DeltaInv(mH2,msb12,msb12))*power4(snb)*(msb14 + 4*mH2*
     DeltaInv(mH2,msb12,msb12)*power6(msb1)))/power2(cb)) - (power2(snb)*
     power2(log(msb12/Q2))*(-(mh2*msb12*msb14*mt2*mu2*power2(DeltaInv(mh2,
     msb12,msb12))) - 2*(2*mh2*msb22 + msb12*msb22 + 2*msb24)*mt2*mu2*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22)) - msb12*msb14*mt2*power2(mu2)*
     power2(DeltaInv(msb12,mt2,mu2)) + msb12*mst22*power2(mC2)*power2(mt2)*
     power2(DeltaInv(mst22,msb12,mC2)) + 2*mst24*power2(mC2)*power2(mt2)*
     power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst12*mt2*mu2*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 2*mst14*mt2*mu2*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 2*mst22*power2(mt2)*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mC2) - 3*msb12*msb14*power2(DeltaInv(msb12,mt2,mu2))*
     power3(mu2) + 2*mst12*mt2*mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2) + 2*mt2*mu2*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(mh2)) +
     2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 10*msb12*
     mst24*power2(mt2)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb12*
     power2(mC2)*power2(mt2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     2*power2(mt2)*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))
     + 3*msb12*mt2*mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,
     mst12)) + 10*msb12*mst14*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 2*mt2*mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 3*msb12*mst22*power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) + 2*mst24*power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*power4(msb12) -
     10*mt2*power3(mu2)*power3(DeltaInv(msb12,mt2,mu2))*power4(msb12) - 2*mt2*
     mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 2*
     power2(mC2)*power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     5*msb12*msb14*mt2*power3(DeltaInv(msb12,mt2,mu2))*power4(mu2) - 5*power3(
     DeltaInv(msb12,mt2,mu2))*power4(msb12)*power4(mu2) + 3*msb12*mst12*mt2*
     mu2*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 2*mst14*mt2*mu2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 2*mst22*power2(mt2)*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) - mt2*power2(mu2)*power3(
     DeltaInv(msb12,mt2,mu2))*power5(msb12) + power3(mu2)*power3(DeltaInv(
     msb12,mt2,mu2))*power5(msb12) + 3*msb12*msb14*power3(DeltaInv(msb12,mt2,
     mu2))*power5(mu2) - 2*mst12*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) + power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*power6(msb12)))/
     (msb12*mt2) + (log(msb12/Q2)*power2(snb)*(8*msb12*msb22*mst12*mst22*mt2 +
     msb12*msb22*mst22*mt2*mu2 - 2*mh2*mst12*mst22*mt2*mu2 - 2*msb12*mst12*
     mst22*mt2*mu2 - msb22*mst12*mst22*mt2*mu2 + msb22*mst22*mt2*mu2*mw2 - 2*(
     mh2 - 6*msb12)*msb12*msb22*mst12*mst22*mt2*mu2*DeltaInv(mh2,msb12,msb12) -
     msb22*mst12*mst14*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12) + msb22*mst22*
     mt2*power2(msb12) - 8*mst12*mst22*mt2*power2(msb12) - 2*msb22*mst12*mst22*
     mt2*mu2*DeltaInv(msb12,mt2,mu2)*power2(msb12) + msb12*msb22*mst22*mt2*mu2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) - 5*msb22*mst22*mt2*mu2*mw2*
     DeltaInv(mw2,msb12,mst12)*power2(mst12) + mC2*msb22*mst12*power2(mt2) +
     msb12*msb22*mst12*power2(mt2) + msb22*mst12*mst22*power2(mt2) - msb22*
     mst12*mst22*mst24*DeltaInv(mst22,msb12,mC2)*power2(mt2) + msb12*msb22*
     mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2)*power2(mt2) - 5*msb22*mst12*
     mst22*DeltaInv(mst22,msb12,mC2)*power2(mC2)*power2(mt2) - 5*mC2*msb22*
     mst12*DeltaInv(mst22,msb12,mC2)*power2(mst22)*power2(mt2) + msb12*msb22*
     mst12*DeltaInv(mst22,msb12,mC2)*power2(mst22)*power2(mt2) - 24*msb22*
     mst12*mst22*DeltaInv(msb12,mt2,mu2)*power2(msb12)*power2(mu2) + msb12*
     msb22*mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*msb22*mst12*
     mst22*mt2*mu2*DeltaInv(mw2,msb12,mst12)*power2(mw2) - 120*msb12*msb14*
     msb22*mst12*mst22*mt2*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2)) - 6*mC2*
     msb12*msb22*mst12*mst22*mst24*power2(mt2)*power2(DeltaInv(mst22,msb12,mC2)
     ) + 138*msb22*mst12*mst22*mst24*power2(mC2)*power2(mt2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 36*msb12*msb22*mst12*power2(mC2)*power2(mst22)*power2(
     mt2)*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst14*mst22*
     mt2*mu2*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 138*msb22*mst12*mst14*
     mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 36*msb12*
     msb22*mst22*mt2*mu2*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mt2)*power3(mC2) -
     6*msb12*msb22*mst12*mst22*power2(mt2)*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) + 138*msb22*mst12*power2(mst22)*power2(mt2)*power2(DeltaInv(
     mst22,msb12,mC2))*power3(mC2) + 2*mst12*mst22*mt2*mu2*DeltaInv(mh2,msb12,
     msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) +
     5*mh2*power2(msb22) + power3(mh2)) + 48*msb12*msb22*mst12*mst22*mt2*mu2*
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 12*mh2*
     msb22*mst12*mst22*mt2*mu2*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*
     msb24) + mh2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) +
     power3(mh2) + power3(msb22)) - 264*msb12*msb14*msb22*mst12*mst22*power2(
     DeltaInv(msb12,mt2,mu2))*power3(mu2) - 48*msb22*mst12*mst22*mt2*power2(
     msb12)*power2(DeltaInv(msb12,mt2,mu2))*power3(mu2) - msb22*mst22*mt2*mu2*
     DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*msb22*mst12*mst22*mt2*mu2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*msb22*mst22*mt2*mu2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 144*msb22*
     mst12*mst22*mt2*mu2*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(mh2))
     + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(5*
     msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*
     msb12*msb22*mst12*mst22*mst24*power2(mt2)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 720*msb12*msb22*mst12*mst14*mst22*mt2*mu2*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 6*msb22*mst12*mst22*power2(mt2)*
     power2(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 144*msb22*mst12*mst22*
     mst24*power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 216*
     msb12*msb22*mst12*power2(mst22)*power2(mt2)*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 48*msb22*mst12*mst22*power2(mu2)*power2(DeltaInv(
     msb12,mt2,mu2))*power4(msb12) - 720*msb22*mst12*mst22*mt2*power3(mu2)*
     power3(DeltaInv(msb12,mt2,mu2))*power4(msb12) + 6*msb22*mst22*mt2*mu2*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 216*msb12*msb22*mst22*
     mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 144*
     msb22*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mst12) + 6*mC2*msb22*mst12*power2(mt2)*power2(DeltaInv(mst22,msb12,mC2))*
     power4(mst22) + 216*msb12*msb22*mst12*power2(mC2)*power2(mt2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 144*msb22*mst12*power2(mt2)*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 24*msb22*
     mst12*mst22*power2(msb12)*power2(DeltaInv(msb12,mt2,mu2))*power4(mu2) -
     360*msb12*msb14*msb22*mst12*mst22*mt2*power3(DeltaInv(msb12,mt2,mu2))*
     power4(mu2) - 360*msb22*mst12*mst22*power3(DeltaInv(msb12,mt2,mu2))*
     power4(msb12)*power4(mu2) + 6*msb22*mst12*mst22*mt2*mu2*power2(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 144*msb22*mst12*mst14*mst22*mt2*mu2*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 216*msb12*msb22*mst22*mt2*
     mu2*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 144*
     msb22*mst12*power2(mst22)*power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 576*mh2*msb22*mst12*mst22*mt2*mu2*power3(DeltaInv(mh2,msb12,
     msb12))*power5(msb12) - 72*msb22*mst12*mst22*mt2*power2(mu2)*power3(
     DeltaInv(msb12,mt2,mu2))*power5(msb12) + 72*msb22*mst12*mst22*power3(mu2)*
     power3(DeltaInv(msb12,mt2,mu2))*power5(msb12) - 144*msb22*mst22*mt2*mu2*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 144*msb22*
     mst12*power2(mC2)*power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mst22) + 216*msb12*msb14*msb22*mst12*mst22*power3(DeltaInv(msb12,mt2,mu2))
     *power5(mu2) - 144*msb22*mst22*mt2*mu2*power2(mst12)*power3(DeltaInv(mw2,
     msb12,mst12))*power5(mw2) + 72*msb22*mst12*mst22*power2(mu2)*power3(
     DeltaInv(msb12,mt2,mu2))*power6(msb12)))/(12.*msb12*msb22*mst12*mst22*mt2)
     + (mu2*power2(cb)*power2(DeltaInv(msb12,mz2,msb22))*power2(log(mz2/Q2))*
     power3(mz2)*(msb22 - DeltaInv(msb12,mz2,msb22)*(mz2*(-2*msb24 + msb22*mz2)
     - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/(2.*msb12) + (mu2*power2(
     sb)*power2(DeltaInv(msb12,mz2,msb22))*power2(log(mz2/Q2))*power3(mz2)*(
     msb22 - DeltaInv(msb12,mz2,msb22)*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(
     msb24 + msb22*mz2) + power6(msb2))))/msb12 + (mu2*power2(DeltaInv(msb12,
     mz2,msb22))*power2(log(mz2/Q2))*power3(mz2)*(-msb22 + DeltaInv(msb12,mz2,
     msb22)*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) + power6(
     msb2))))/(2.*msb12) + (mu2*power2(sb)*power2(DeltaInv(msb12,mz2,msb22))*
     power2(log(mz2/Q2))*power3(mz2)*(-msb22 + DeltaInv(msb12,mz2,msb22)*(mz2*(
     -2*msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/(2.*
     msb12) + (mu2*mz2*log(mz2/Q2)*(-1 + mz2*(-msb12 + 5*msb22 + mz2)*DeltaInv(
     msb12,mz2,msb22) + 6*msb22*mz2*(-msb24 + (14*msb22 - mz2)*mz2 + msb12*(3*
     msb22 + mz2))*power2(DeltaInv(msb12,mz2,msb22)) - 72*msb22*power2(mz2)*
     power3(DeltaInv(msb12,mz2,msb22))*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(
     msb24 + msb22*mz2) + power6(msb2))))/(24.*msb12*msb22) + (mu2*mz2*log(mz2/
     Q2)*power2(sb)*(-1 + mz2*(-msb12 + 5*msb22 + mz2)*DeltaInv(msb12,mz2,
     msb22) + 6*msb22*mz2*(-msb24 + (14*msb22 - mz2)*mz2 + msb12*(3*msb22 +
     mz2))*power2(DeltaInv(msb12,mz2,msb22)) - 72*msb22*power2(mz2)*power3(
     DeltaInv(msb12,mz2,msb22))*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(msb24 +
     msb22*mz2) + power6(msb2))))/(24.*msb12*msb22) + (mu2*mz2*log(mz2/Q2)*
     power2(sb)*(1 + (msb12 - 5*msb22 - mz2)*mz2*DeltaInv(msb12,mz2,msb22) - 6*
     msb22*mz2*(-msb24 + (14*msb22 - mz2)*mz2 + msb12*(3*msb22 + mz2))*power2(
     DeltaInv(msb12,mz2,msb22)) + 72*msb22*power2(mz2)*power3(DeltaInv(msb12,
     mz2,msb22))*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) +
     power6(msb2))))/(12.*msb12*msb22) - (log(mz2/Q2)*power2(cb)*(msb12*(6*
     msb22 - mz2) - (msb22 + mu2)*mz2 + mu2*(-msb12 + 5*msb22 + mz2)*DeltaInv(
     msb12,mz2,msb22)*power2(mz2) + 6*msb22*mu2*(-msb24 + (14*msb22 - mz2)*mz2
     + msb12*(3*msb22 + mz2))*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22)) -
     72*msb22*mu2*power3(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(mz2*(-2*msb24
     + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/(24.*msb12*
     msb22) + (mu2*power2(sb)*power2(DeltaInv(msb12,mA2,msb22))*power2(log(mA2/
     Q2))*power3(mA2)*(msb22 - DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*
     mA2*(msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/(2.*msb12)
     + (mu2*power2(cb)*power2(DeltaInv(msb12,mA2,msb22))*power2(log(mA2/Q2))*
     power3(mA2)*(-msb22 + DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(
     msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/(2.*msb12) + (
     mu2*power2(sb)*power2(DeltaInv(msb12,mA2,msb22))*power2(log(mA2/Q2))*
     power3(mA2)*(-msb22 + DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(
     msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/msb12 + (mu2*
     power2(sb)*power2(DeltaInv(msb12,mA2,msb22))*power2(log(mA2/Q2))*power3(
     mA2)*(-msb22 + DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*
     msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/(2.*msb12*power2(cb))
     + (mA2*mu2*log(mA2/Q2)*power2(sb)*(-1 + mA2*(mA2 - msb12 + 5*msb22)*
     DeltaInv(msb12,mA2,msb22) - 6*mA2*msb22*(-3*msb12*msb22 - mA2*(msb12 + 14*
     msb22) + msb24 + power2(mA2))*power2(DeltaInv(msb12,mA2,msb22)) - 72*
     msb22*power2(mA2)*power3(DeltaInv(msb12,mA2,msb22))*(-2*msb12*msb24 - 2*
     mA2*(msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/(12.*
     msb12*msb22) + (mA2*mu2*log(mA2/Q2)*power2(sb)*(-1 + mA2*(mA2 - msb12 + 5*
     msb22)*DeltaInv(msb12,mA2,msb22) - 6*mA2*msb22*(-3*msb12*msb22 - mA2*(
     msb12 + 14*msb22) + msb24 + power2(mA2))*power2(DeltaInv(msb12,mA2,msb22))
     - 72*msb22*power2(mA2)*power3(DeltaInv(msb12,mA2,msb22))*(-2*msb12*msb24 -
     2*mA2*(msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/(24.*
     msb12*msb22*power2(cb)) + (mA2*mu2*log(mA2/Q2)*power2(sb)*(1 - mA2*(mA2 -
     msb12 + 5*msb22)*DeltaInv(msb12,mA2,msb22) + 6*mA2*msb22*(-3*msb12*msb22 -
     mA2*(msb12 + 14*msb22) + msb24 + power2(mA2))*power2(DeltaInv(msb12,mA2,
     msb22)) + 72*msb22*power2(mA2)*power3(DeltaInv(msb12,mA2,msb22))*(-2*
     msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(mA2) + power6(
     msb2))))/(24.*msb12*msb22) - (log(mA2/Q2)*power2(cb)*(-6*msb12*msb22 +
     mA2*(msb12 + msb22 + mu2) + (-mA2 + msb12 - 5*msb22)*mu2*DeltaInv(msb12,
     mA2,msb22)*power2(mA2) + 6*msb22*mu2*power2(mA2)*(-3*msb12*msb22 - mA2*(
     msb12 + 14*msb22) + msb24 + power2(mA2))*power2(DeltaInv(msb12,mA2,msb22))
     + 72*msb22*mu2*power3(mA2)*power3(DeltaInv(msb12,mA2,msb22))*(-2*msb12*
     msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/
     (24.*msb12*msb22) + (mu2*power2(sa)*power2(DeltaInv(mh2,msb12,msb22))*
     power2(log(mh2/Q2))*power3(mh2)*(msb22 - DeltaInv(mh2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(
     msb2))))/(2.*msb12) + (mu2*power2(DeltaInv(mh2,msb12,msb22))*power2(log(
     mh2/Q2))*power3(mh2)*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 -
     2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/(2.*
     msb12) + (ca*mu2*sa*sb*power2(DeltaInv(mh2,msb12,msb22))*power2(log(mh2/
     Q2))*power3(mh2)*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*
     mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/(cb*msb12)
     + (mu2*power2(sa)*power2(sb)*power2(DeltaInv(mh2,msb12,msb22))*power2(log(
     mh2/Q2))*power3(mh2)*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 -
     2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/(2.*
     msb12*power2(cb)) + (mh2*mu2*log(mh2/Q2)*(-1 + mh2*(mh2 - msb12 + 5*msb22)
     *DeltaInv(mh2,msb12,msb22) - 6*mh2*msb22*(-3*msb12*msb22 - mh2*(msb12 +
     14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,msb22)) - 72*
     msb22*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-2*msb12*msb24 - 2*
     mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/(24.*
     msb12*msb22) + (ca*mh2*mu2*sa*sb*log(mh2/Q2)*(-1 + mh2*(mh2 - msb12 + 5*
     msb22)*DeltaInv(mh2,msb12,msb22) - 6*mh2*msb22*(-3*msb12*msb22 - mh2*(
     msb12 + 14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,msb22))
     - 72*msb22*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-2*msb12*msb24 -
     2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/(12.*cb*
     msb12*msb22) + (mh2*mu2*log(mh2/Q2)*power2(sa)*power2(sb)*(-1 + mh2*(mh2 -
     msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22) - 6*mh2*msb22*(-3*msb12*msb22 -
     mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,
     msb22)) - 72*msb22*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(
     msb2))))/(24.*msb12*msb22*power2(cb)) + (log(mh2/Q2)*power2(sa)*(-6*msb12*
     msb22 + mh2*(msb12 + msb22 + mu2) + (-mh2 + msb12 - 5*msb22)*mu2*DeltaInv(
     mh2,msb12,msb22)*power2(mh2) + 6*msb22*mu2*power2(mh2)*(-3*msb12*msb22 -
     mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,
     msb22)) + 72*msb22*mu2*power3(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(
     msb2))))/(24.*msb12*msb22) + (ca*mu2*sa*sb*power2(DeltaInv(mH2,msb12,
     msb22))*power2(log(mH2/Q2))*power3(mH2)*(msb22 - DeltaInv(mH2,msb12,msb22)
     *(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) +
     power6(msb2))))/(cb*msb12) + (mu2*power2(sa)*power2(sb)*power2(DeltaInv(
     mH2,msb12,msb22))*power2(log(mH2/Q2))*power3(mH2)*(msb22 - DeltaInv(mH2,
     msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(
     mH2) + power6(msb2))))/(2.*msb12*power2(cb)) + (mu2*power2(sa)*power2(
     DeltaInv(mH2,msb12,msb22))*power2(log(mH2/Q2))*power3(mH2)*(-msb22 +
     DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power6(msb2))))/(2.*msb12) + (mu2*power2(sb)*power2(
     DeltaInv(mH2,msb12,msb22))*power2(log(mH2/Q2))*power3(mH2)*(-msb22 +
     DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power6(msb2))))/(2.*msb12*power2(cb)) + (mH2*mu2*log(
     mH2/Q2)*power2(sb)*(-1 + mH2*(mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,
     msb22) - 6*mH2*msb22*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 +
     power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) - 72*msb22*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power6(msb2))))/(24.*msb12*msb22*power2(cb))
     + (ca*mH2*mu2*sa*sb*log(mH2/Q2)*(1 - mH2*(mH2 - msb12 + 5*msb22)*DeltaInv(
     mH2,msb12,msb22) + 6*mH2*msb22*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) +
     msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 72*msb22*power2(
     mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 - 2*mH2*(msb12*
     msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(12.*cb*msb12*msb22)
     + (mH2*mu2*log(mH2/Q2)*power2(sa)*power2(sb)*(1 - mH2*(mH2 - msb12 + 5*
     msb22)*DeltaInv(mH2,msb12,msb22) + 6*mH2*msb22*(-3*msb12*msb22 - mH2*(
     msb12 + 14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22))
     + 72*msb22*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(24.*
     msb12*msb22*power2(cb)) - (log(mH2/Q2)*power2(sa)*(-6*msb12*msb22 + mH2*(
     msb12 + msb22 + mu2) + (-mH2 + msb12 - 5*msb22)*mu2*DeltaInv(mH2,msb12,
     msb22)*power2(mH2) + 6*msb22*mu2*power2(mH2)*(-3*msb12*msb22 - mH2*(msb12
     + 14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 72*
     msb22*mu2*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(24.*
     msb12*msb22) + Fin3(mh2,msb22,msb22,Q2)*(-2*mu2*power2(snb)*power2(
     DeltaInv(mh2,msb22,msb22))*(msb24 + 4*mh2*DeltaInv(mh2,msb22,msb22)*
     power6(msb2)) - (4*ca*mu2*sa*sb*power2(snb)*power2(DeltaInv(mh2,msb22,
     msb22))*(msb24 + 4*mh2*DeltaInv(mh2,msb22,msb22)*power6(msb2)))/cb + 2*
     mu2*power2(sa)*power2(snb)*power2(DeltaInv(mh2,msb22,msb22))*(msb24 + 4*
     mh2*DeltaInv(mh2,msb22,msb22)*power6(msb2)) - (2*mu2*power2(sa)*power2(sb)
     *power2(snb)*power2(DeltaInv(mh2,msb22,msb22))*(msb24 + 4*mh2*DeltaInv(
     mh2,msb22,msb22)*power6(msb2)))/power2(cb) + 2*mu2*power2(DeltaInv(mh2,
     msb22,msb22))*power4(snb)*(msb24 + 4*mh2*DeltaInv(mh2,msb22,msb22)*power6(
     msb2)) + (4*ca*mu2*sa*sb*power2(DeltaInv(mh2,msb22,msb22))*power4(snb)*(
     msb24 + 4*mh2*DeltaInv(mh2,msb22,msb22)*power6(msb2)))/cb - 2*mu2*power2(
     sa)*power2(DeltaInv(mh2,msb22,msb22))*power4(snb)*(msb24 + 4*mh2*DeltaInv(
     mh2,msb22,msb22)*power6(msb2)) + (2*mu2*power2(sa)*power2(sb)*power2(
     DeltaInv(mh2,msb22,msb22))*power4(snb)*(msb24 + 4*mh2*DeltaInv(mh2,msb22,
     msb22)*power6(msb2)))/power2(cb)) + Fin3(mH2,msb22,msb22,Q2)*((4*ca*mu2*
     sa*sb*power2(snb)*power2(DeltaInv(mH2,msb22,msb22))*(msb24 + 4*mH2*
     DeltaInv(mH2,msb22,msb22)*power6(msb2)))/cb - 2*mu2*power2(sa)*power2(snb)
     *power2(DeltaInv(mH2,msb22,msb22))*(msb24 + 4*mH2*DeltaInv(mH2,msb22,
     msb22)*power6(msb2)) - (2*mu2*power2(sb)*power2(snb)*power2(DeltaInv(mH2,
     msb22,msb22))*(msb24 + 4*mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2)))/
     power2(cb) + (2*mu2*power2(sa)*power2(sb)*power2(snb)*power2(DeltaInv(mH2,
     msb22,msb22))*(msb24 + 4*mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2)))/
     power2(cb) - (4*ca*mu2*sa*sb*power2(DeltaInv(mH2,msb22,msb22))*power4(snb)
     *(msb24 + 4*mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2)))/cb + 2*mu2*
     power2(sa)*power2(DeltaInv(mH2,msb22,msb22))*power4(snb)*(msb24 + 4*mH2*
     DeltaInv(mH2,msb22,msb22)*power6(msb2)) + (2*mu2*power2(sb)*power2(
     DeltaInv(mH2,msb22,msb22))*power4(snb)*(msb24 + 4*mH2*DeltaInv(mH2,msb22,
     msb22)*power6(msb2)))/power2(cb) - (2*mu2*power2(sa)*power2(sb)*power2(
     DeltaInv(mH2,msb22,msb22))*power4(snb)*(msb24 + 4*mH2*DeltaInv(mH2,msb22,
     msb22)*power6(msb2)))/power2(cb)) + Fin3(msb12,mz2,msb22,Q2)*((mu2*power2(
     cb)*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb22 - DeltaInv(msb12,
     mz2,msb22)*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) +
     power6(msb2))))/msb12 + (2*mu2*power2(mz2)*power2(sb)*power2(DeltaInv(
     msb12,mz2,msb22))*(msb22 - DeltaInv(msb12,mz2,msb22)*(mz2*(-2*msb24 +
     msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/msb12 + (mu2*
     power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(-msb22 + DeltaInv(msb12,
     mz2,msb22)*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) +
     power6(msb2))))/msb12 + (mu2*power2(mz2)*power2(sb)*power2(DeltaInv(msb12,
     mz2,msb22))*(-msb22 + DeltaInv(msb12,mz2,msb22)*(mz2*(-2*msb24 + msb22*
     mz2) - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/msb12) + Fin3(msb12,
     mA2,msb22,Q2)*((mu2*power2(mA2)*power2(sb)*power2(DeltaInv(msb12,mA2,
     msb22))*(msb22 - DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*
     msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/msb12 + (mu2*power2(
     cb)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))*(-msb22 + DeltaInv(
     msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*
     power2(mA2) + power6(msb2))))/msb12 + (2*mu2*power2(mA2)*power2(sb)*
     power2(DeltaInv(msb12,mA2,msb22))*(-msb22 + DeltaInv(msb12,mA2,msb22)*(-2*
     msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(mA2) + power6(
     msb2))))/msb12 + (mu2*power2(mA2)*power2(sb)*power2(DeltaInv(msb12,mA2,
     msb22))*(-msb22 + DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(
     msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/(msb12*power2(
     cb))) + Fin3(mh2,msb12,msb22,Q2)*((mu2*power2(mh2)*power2(sa)*power2(
     DeltaInv(mh2,msb12,msb22))*(msb22 - DeltaInv(mh2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/
     msb12 + (4*mu2*power2(mh2)*power2(snb)*power2(DeltaInv(mh2,msb12,msb22))*(
     msb22 - DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 +
     msb24) + msb22*power2(mh2) + power6(msb2))))/msb12 + (8*ca*mu2*sa*sb*
     power2(mh2)*power2(snb)*power2(DeltaInv(mh2,msb12,msb22))*(msb22 -
     DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) +
     msb22*power2(mh2) + power6(msb2))))/(cb*msb12) + (4*mu2*power2(mh2)*
     power2(sa)*power2(sb)*power2(snb)*power2(DeltaInv(mh2,msb12,msb22))*(msb22
     - DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24)
     + msb22*power2(mh2) + power6(msb2))))/(msb12*power2(cb)) + (4*mu2*power2(
     mh2)*power2(sa)*power2(DeltaInv(mh2,msb12,msb22))*power4(snb)*(msb22 -
     DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) +
     msb22*power2(mh2) + power6(msb2))))/msb12 + (mu2*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22))*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/
     msb12 + (2*ca*mu2*sa*sb*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))*(-
     msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 +
     msb24) + msb22*power2(mh2) + power6(msb2))))/(cb*msb12) + (mu2*power2(mh2)
     *power2(sa)*power2(sb)*power2(DeltaInv(mh2,msb12,msb22))*(-msb22 +
     DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) +
     msb22*power2(mh2) + power6(msb2))))/(msb12*power2(cb)) + (4*mu2*power2(
     mh2)*power2(sa)*power2(snb)*power2(DeltaInv(mh2,msb12,msb22))*(-msb22 +
     DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) +
     msb22*power2(mh2) + power6(msb2))))/msb12 + (4*mu2*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22))*power4(snb)*(-msb22 + DeltaInv(mh2,msb12,msb22)
     *(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) +
     power6(msb2))))/msb12 + (8*ca*mu2*sa*sb*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22))*power4(snb)*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/
     (cb*msb12) + (4*mu2*power2(mh2)*power2(sa)*power2(sb)*power2(DeltaInv(mh2,
     msb12,msb22))*power4(snb)*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/
     (msb12*power2(cb))) + Fin3(mH2,msb12,msb22,Q2)*((2*ca*mu2*sa*sb*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22))*(msb22 - DeltaInv(mH2,msb12,msb22)*
     (-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) +
     power6(msb2))))/(cb*msb12) + (mu2*power2(mH2)*power2(sa)*power2(sb)*
     power2(DeltaInv(mH2,msb12,msb22))*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(
     msb2))))/(msb12*power2(cb)) + (4*mu2*power2(mH2)*power2(sa)*power2(snb)*
     power2(DeltaInv(mH2,msb12,msb22))*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(
     msb2))))/msb12 + (4*mu2*power2(mH2)*power2(sb)*power2(snb)*power2(
     DeltaInv(mH2,msb12,msb22))*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/
     (msb12*power2(cb)) + (8*ca*mu2*sa*sb*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*power4(snb)*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/
     (cb*msb12) + (4*mu2*power2(mH2)*power2(sa)*power2(sb)*power2(DeltaInv(mH2,
     msb12,msb22))*power4(snb)*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/
     (msb12*power2(cb)) + (mu2*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,
     msb12,msb22))*(-msb22 + DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*
     (msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/msb12 + (mu2*
     power2(mH2)*power2(sb)*power2(DeltaInv(mH2,msb12,msb22))*(-msb22 +
     DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power6(msb2))))/(msb12*power2(cb)) + (8*ca*mu2*sa*sb*
     power2(mH2)*power2(snb)*power2(DeltaInv(mH2,msb12,msb22))*(-msb22 +
     DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power6(msb2))))/(cb*msb12) + (4*mu2*power2(mH2)*
     power2(sa)*power2(sb)*power2(snb)*power2(DeltaInv(mH2,msb12,msb22))*(-
     msb22 + DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power6(msb2))))/(msb12*power2(cb)) + (4*mu2*
     power2(mH2)*power2(sa)*power2(DeltaInv(mH2,msb12,msb22))*power4(snb)*(-
     msb22 + DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power6(msb2))))/msb12 + (4*mu2*power2(mH2)*
     power2(sb)*power2(DeltaInv(mH2,msb12,msb22))*power4(snb)*(-msb22 +
     DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power6(msb2))))/(msb12*power2(cb))) + (mu2*power2(cb)*
     power2(DeltaInv(mw2,msb22,mst12))*power2(log(mw2/Q2))*power3(mw2)*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/msb22 + (2*mu2*power2(sb)*power2(DeltaInv(
     mw2,msb22,mst12))*power2(log(mw2/Q2))*power3(mw2)*(mst12 - DeltaInv(mw2,
     msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) +
     power6(mst1))))/msb22 + (mu2*power2(DeltaInv(mw2,msb22,mst12))*power2(log(
     mw2/Q2))*power3(mw2)*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (mu2*
     power2(sb)*power2(DeltaInv(mw2,msb22,mst12))*power2(log(mw2/Q2))*power3(
     mw2)*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (mu2*mw2*log(mw2/Q2)*(
     -1 + mw2*(-msb22 + 5*mst12 + mw2)*DeltaInv(mw2,msb22,mst12) + 6*mst12*mw2*
     (-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(DeltaInv(
     mw2,msb22,mst12)) - 72*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     *(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))
     ))/(12.*msb22*mst12) + (mu2*mw2*log(mw2/Q2)*power2(sb)*(-1 + mw2*(-msb22 +
     5*mst12 + mw2)*DeltaInv(mw2,msb22,mst12) + 6*mst12*mw2*(-mst14 + (14*mst12
     - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb22,mst12)) -
     72*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(12.*msb22*
     mst12) + (mu2*mw2*log(mw2/Q2)*power2(sb)*(1 + (msb22 - 5*mst12 - mw2)*mw2*
     DeltaInv(mw2,msb22,mst12) - 6*mst12*mw2*(-mst14 + (14*mst12 - mw2)*mw2 +
     msb22*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb22,mst12)) + 72*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2)
     - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(6.*msb22*mst12) + (mu2*
     mw2*log(mw2/Q2)*power2(snb)*(msb12 - msb22 + msb22*mw2*(-msb12 + 5*mst12 +
     mw2)*DeltaInv(mw2,msb12,mst12) + msb12*(msb22 - 5*mst12 - mw2)*mw2*
     DeltaInv(mw2,msb22,mst12) + 6*msb22*mst12*mw2*(-mst14 + (14*mst12 - mw2)*
     mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*
     mst12*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst12*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*
     mw2) + power6(mst1)) + 72*msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) +
     power6(mst1))))/(12.*msb12*msb22*mst12) - (mu2*mw2*log(mw2/Q2)*power2(sb)*
     power2(snb)*(msb12 - msb22 + msb22*mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(
     mw2,msb12,mst12) + msb12*(msb22 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,msb22,
     mst12) + 6*msb22*mst12*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb12*(3*mst12
     + mw2))*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*mst12*mw2*(-mst14 + (
     14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb22,
     mst12)) - 72*msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) +
     72*msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(12.*
     msb12*msb22*mst12) - (mw2*log(mw2/Q2)*power2(cb)*power2(snb)*((msb12 -
     msb22)*(mst12 + mu2) + msb22*mu2*mw2*(-msb12 + 5*mst12 + mw2)*DeltaInv(
     mw2,msb12,mst12) + msb12*mu2*(msb22 - 5*mst12 - mw2)*mw2*DeltaInv(mw2,
     msb22,mst12) + 6*msb22*mst12*mu2*mw2*(-mst14 + (14*mst12 - mw2)*mw2 +
     msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*mst12*
     mu2*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst12*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1)) + 72*msb12*mst12*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/(12.*msb12*msb22*mst12) - (mw2*log(mw2/Q2)*
     power2(cb)*((mst12 + mu2)*(mt2 - mw2) + msb22*(4*mst12 + mt2 - mw2) + mu2*
     (msb22 - 5*mst12 - mw2)*(mt2 - mw2)*mw2*DeltaInv(mw2,msb22,mst12) + 6*
     mst12*mu2*(mt2 - mw2)*mw2*(mst14 + mw2*(-14*mst12 + mw2) - msb22*(3*mst12
     + mw2))*power2(DeltaInv(mw2,msb22,mst12)) + 72*mst12*mu2*(mt2 - mw2)*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2)
     - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(12.*msb22*mst12*(-mt2 +
     mw2)) + (mu2*power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mC2/
     Q2))*power3(mC2)*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (
     2*mu2*power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2))*
     power3(mC2)*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (mu2*
     power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2))*power3(
     mC2)*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*power2(cb)) +
     (mC2*mu2*log(mC2/Q2)*power2(sb)*(-1 + mC2*(mC2 - msb22 + 5*mst12)*
     DeltaInv(msb22,mst12,mC2) - 6*mC2*mst12*(-3*msb22*mst12 - mC2*(msb22 + 14*
     mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 72*
     mst12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(6.*msb22*
     mst12) + (mC2*mu2*log(mC2/Q2)*power2(sb)*(-1 + mC2*(mC2 - msb22 + 5*mst12)
     *DeltaInv(msb22,mst12,mC2) - 6*mC2*mst12*(-3*msb22*mst12 - mC2*(msb22 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 72*
     mst12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(12.*
     msb22*mst12*power2(cb)) + (mC2*mu2*log(mC2/Q2)*power2(sb)*(1 - mC2*(mC2 -
     msb22 + 5*mst12)*DeltaInv(msb22,mst12,mC2) + 6*mC2*mst12*(-3*msb22*mst12 -
     mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,
     mC2)) + 72*mst12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     (12.*msb22*mst12) + (mC2*mu2*log(mC2/Q2)*power2(sb)*power2(snb)*(msb12 -
     msb22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*DeltaInv(msb12,mst12,mC2) + mC2*
     msb12*(-mC2 + msb22 - 5*mst12)*DeltaInv(msb22,mst12,mC2) - 6*mC2*msb22*
     mst12*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*(-3*msb22*mst12 -
     mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,
     mC2)) - 72*msb22*mst12*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + 72*msb12*mst12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(12.*msb12*msb22*mst12) + (mC2*mu2*log(mC2/Q2)*power2(sb)*power2(
     snb)*(msb12 - msb22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*DeltaInv(msb12,
     mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*DeltaInv(msb22,mst12,mC2)
     - 6*mC2*msb22*mst12*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*(-3*
     msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb22,mst12,mC2)) - 72*msb22*mst12*power2(mC2)*power3(DeltaInv(
     msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + 72*msb12*mst12*power2(mC2)*power3(DeltaInv(
     msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(12.*msb12*msb22*mst12*power2(cb)) + Fin3(
     mw2,msb12,mst12,Q2)*((2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(
     DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 +
     (4*mu2*power2(mw2)*power2(sb)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12)
     )*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 + (2*mu2*power2(mw2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/msb12 + (2*mu2*power2(mw2)*power2(sb)*power2(
     snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 - DeltaInv(mw2,
     msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1))))/msb12 + (2*mu2*power2(mw2)*power2(snb)*power2(DeltaInv(
     mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 + (2*mu2*
     power2(mw2)*power2(sb)*power2(snb)*power2(DeltaInv(mw2,msb12,mst12))*(-
     mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(
     mst14 + mst12*mw2) + power6(mst1))))/msb12 + (2*mu2*power2(cb)*power2(mw2)
     *power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(-mst12 +
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/msb12 + (4*mu2*power2(mw2)*power2(sb)*power2(
     snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,
     msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1))))/msb12) + (mu2*power2(cb)*power2(snb)*power2(log(mw2/Q2))*
     power3(mw2)*(msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) - msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1)))))/(msb12*msb22) + (2*mu2*power2(sb)*
     power2(snb)*power2(log(mw2/Q2))*power3(mw2)*(msb22*mst12*power2(DeltaInv(
     mw2,msb12,mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14
     + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) - msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))))/(
     msb12*msb22) + (mu2*power2(snb)*power2(log(mw2/Q2))*power3(mw2)*(-(msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12))) + msb22*power3(DeltaInv(mw2,
     msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) +
     power6(mst1)) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(
     mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2)
     + power6(mst1)))))/(msb12*msb22) + (mu2*power2(sb)*power2(snb)*power2(log(
     mw2/Q2))*power3(mw2)*(-(msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))) +
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb12*(mst14 + mst12*mw2) + power6(mst1)) + msb12*power2(DeltaInv(mw2,
     msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))))/(msb12*msb22) +
     Fin3(mw2,msb22,mst12,Q2)*((2*mu2*power2(cb)*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (4*mu2*
     power2(mw2)*power2(sb)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/msb22 + (2*mu2*power2(mw2)*power2(snb)*
     power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*
     (-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/
     msb22 + (2*mu2*power2(mw2)*power2(sb)*power2(snb)*power2(DeltaInv(mw2,
     msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (2*mu2*
     power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/msb22 + (2*mu2*power2(mw2)*power2(sb)*power2(
     snt)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*
     (mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))
     )/msb22 + (2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(
     DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 +
     (4*mu2*power2(mw2)*power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,
     msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (2*mu2*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,
     msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) +
     power6(mst1))))/msb22 + (2*mu2*power2(mw2)*power2(sb)*power2(DeltaInv(mw2,
     msb22,mst12))*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (2*mu2*
     power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(-
     mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1))))/msb22 + (4*mu2*power2(mw2)*power2(sb)
     *power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,
     msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) +
     power6(mst1))))/msb22 + (2*mu2*power2(cb)*power2(mw2)*power2(snt)*power2(
     DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 +
     (4*mu2*power2(mw2)*power2(sb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12)
     )*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (2*mu2*power2(mw2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 +
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/msb22 + (2*mu2*power2(mw2)*power2(sb)*power2(
     snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,
     msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) +
     power6(mst1))))/msb22) + Fin3(msb12,mst12,mC2,Q2)*((4*mt2*mu2*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(msb12,
     mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (4*mt2*mu2*power2(cb)*
     power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mst12 -
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (2*mu2*
     power2(mC2)*power2(sb)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(
     mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb12 + (4*mu2*power2(mC2)*
     power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(
     mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb12 + (2*mu2*power2(mC2)*
     power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(
     mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*power2(cb)) + (4*mt2*
     mu2*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (2*(mst12 -
     mst22 - 2*mt2)*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(
     msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*(
     mst12 - mst22)) + (8*mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(
     DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     (msb12*(mst12 - mst22)) + (4*mu2*power2(mC2)*power2(sb)*power2(snb)*
     power2(DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb12 + (2*mu2*power2(mC2)*power2(sb)*power2(snb)*power2(
     DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     (msb12*power2(cb)) + (2*mt2*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     msb12 - (2*(mt2 + mu2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*
     power2(DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb12 + (2*mu2*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*
     power2(DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb12) + (2*mu2*power2(sb)*power2(snb)*power2(log(mC2/Q2))*
     power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))) + msb22*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)))))/(msb12*msb22)
     + (mu2*power2(sb)*power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(-(msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2))) + msb22*power3(DeltaInv(msb12,
     mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mst12 -
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)))))/(msb12*msb22*power2(cb)) + (mu2*
     power2(sb)*power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*
     power2(DeltaInv(msb12,mst12,mC2)) - msb22*power3(DeltaInv(msb12,mst12,mC2)
     )*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)))))/(msb12*msb22) + Fin3(msb22,mst12,mC2,
     Q2)*((4*mt2*mu2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 -
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (4*mt2*mu2*
     power2(cb)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 -
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (2*mu2*
     power2(mC2)*power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 -
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (4*mt2*mu2*power2(mC2)*power2(
     snb)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*
     (-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/(msb22*(mst12 - mst22)) + (8*mt2*mu2*power2(cb)*power2(
     mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(
     msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (4*mu2*power2(mC2)
     *power2(sb)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 -
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (2*mu2*power2(mC2)*power2(sb)*
     power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(msb22*power2(cb)) + (4*mu2*power2(mC2)*power2(sb)*
     power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/msb22 + (2*mu2*power2(mC2)*power2(sb)*power2(snt)*
     power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(msb22*power2(cb)) + (2*mt2*power2(mC2)*power2(snb)*power2(snt)*
     power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb22 + (2*mu2*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*
     power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb22 + (4*mt2*mu2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*
     (-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (2*
     (mst12 - mst22 - 2*mt2)*mu2*power2(cb)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22)) + (8*mt2*mu2*power2(cb)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22)) + (4*mu2*power2(mC2)*power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*
     (-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (2*mu2*power2(mC2)*
     power2(sb)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(msb22*power2(cb)) + (4*mt2*mu2*power2(mC2)*power2(
     snb)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)
     *(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/(msb22*(mst12 - mst22)) - (2*(mst12 - mst22 - 2*mt2)*mu2*
     power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(-
     mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (4*
     mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,
     mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22)) + (2*mu2*power2(mC2)*power2(sb)*power2(snb)*power2(DeltaInv(msb22,
     mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (2*mt2*
     power2(mC2)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 - (2*(mt2 + mu2)*power2(cb)*
     power2(mC2)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (2*mu2*power2(mC2)*power2(sb)*
     power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/msb22 + (2*(mt2 + mu2)*power2(cb)*power2(mC2)*
     power2(snb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (4*mu2*power2(mC2)*power2(sb)*
     power2(snb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (2*mu2*power2(mC2)*power2(sb)*
     power2(snb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb22*power2(cb))) + (2*mt2*mu2*
     power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*power2(DeltaInv(
     msb12,mst12,mC2)) - msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) - msb22*
     mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,
     mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - 2*msb12*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) -
     msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - msb22*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb12*power3(DeltaInv(msb22,mst12,mC2)
     )*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) -
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22*(mst12
     - mst22)) + (2*mt2*mu2*power2(cb)*power2(snb)*power2(log(mC2/Q2))*power3(
     mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*power2(
     DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))
     + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))
     + msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) - msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22*(mst12 - mst22)) + ((mt2 + mu2)*power2(cb)*power2(snb)*power2(
     snt)*power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst22*power2(DeltaInv(mst22,
     msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) -
     2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) + msb22*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) +
     2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) - msb12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - msb22*power3(DeltaInv(
     msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - msb12*
     power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22) + (2*mu2*
     power2(sb)*power2(snb)*power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*power2(DeltaInv(
     msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) - msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22) + (mu2*power2(sb)*power2(snb)*power2(snt)*power2(log(mC2/Q2))
     *power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*
     power2(DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) + msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) - msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(msb12*msb22*power2(cb)) + (2*mt2*mu2*power2(
     snb)*power2(log(mC2/Q2))*power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) + msb22*
     mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,
     mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + 2*msb12*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) +
     msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + msb22*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - msb12*power3(DeltaInv(msb22,mst12,mC2)
     )*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) +
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22*(mst12
     - mst22)) + (4*mt2*mu2*power2(cb)*power2(snb)*power2(log(mC2/Q2))*power3(
     mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst12*
     power2(DeltaInv(msb22,mst12,mC2)) + msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) - msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) - msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2)) - 2*msb12*msb22*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + msb12*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) -
     msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) + msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(msb12*msb22*(mst12 - mst22)) + (mt2*power2(
     snb)*power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(-(msb22*mst12*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)
     ) + msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - msb22*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2))
     + msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + msb22*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - msb12*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) + msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*
     msb22) + (mu2*power2(sb)*power2(snb)*power2(snt)*power2(log(mC2/Q2))*
     power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))) + msb12*
     mst12*power2(DeltaInv(msb22,mst12,mC2)) + msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2)) - msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + 2*mC2*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) - msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*
     msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) - 2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb22,mC2)) + msb12*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) - msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) + msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(msb12*msb22) + (mC2*mt2*mu2*log(mC2/Q2)*
     power2(cb)*power2(snb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 -
     msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,
     mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,
     mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*
     msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*
     DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*
     mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*
     mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) -
     6*mC2*msb12*mst12*mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*
     msb12*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) +
     6*msb22*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*
     msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 144*msb22*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*
     power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14
     - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(6.*msb12*msb22*mst12*(mst12 - mst22)*mst22) + (mC2*mt2*
     log(mC2/Q2)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*
     mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(
     msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(
     msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*
     mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*
     DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*
     mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*
     mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) -
     6*mC2*msb12*mst12*mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*
     msb12*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) +
     6*msb22*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*
     msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 144*msb22*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*
     power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14
     - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(12.*msb12*msb22*mst12*mst22) - (mC2*(mt2 + mu2)*log(mC2/
     Q2)*power2(cb)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 +
     msb12*mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*
     DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*
     DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,
     mC2) - 5*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*
     msb22*mst12*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(
     mst22,msb22,mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) +
     msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*
     mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*
     mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) - 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 18*mC2*msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) - 84*msb22*mst12*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*mst22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*
     mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*
     msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     12.*msb12*msb22*mst12*mst22) - (mC2*mu2*log(mC2/Q2)*power2(sb)*power2(snb)
     *power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*
     msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*
     msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*
     DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,
     mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2) - msb22*mst12*
     DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*DeltaInv(mst22,msb22,
     mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*
     mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*
     mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*
     mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*
     mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*
     mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14
     - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(12.*msb12*msb22*mst12*mst22) - (mC2*mu2*log(mC2/Q2)*
     power2(sb)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*
     mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(
     msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(
     msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*
     mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*
     DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*
     mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*
     mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) -
     6*mC2*msb12*mst12*mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*
     msb12*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) +
     6*msb22*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*
     msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 144*msb22*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*
     power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14
     - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(12.*msb12*msb22*mst12*mst22*power2(cb)) + (mC2*log(mC2/
     Q2)*power2(cb)*power2(snb)*(msb12*mst12*mst22*mt2 - msb22*mst12*mst22*mt2
     + msb12*mst12*mst22*mu2 - msb22*mst12*mst22*mu2 + 2*msb12*mst12*mt2*mu2 -
     2*msb22*mst12*mt2*mu2 - 2*msb12*mst22*mt2*mu2 + 2*msb22*mst22*mt2*mu2 +
     mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*(mst12 - mst22 - 2*mt2)*mu2*
     DeltaInv(msb12,mst12,mC2) - mC2*msb12*(mC2 - msb22 + 5*mst12)*mst22*(mst12
     - mst22 - 2*mt2)*mu2*DeltaInv(msb22,mst12,mC2) - mC2*msb12*msb22*mst12*
     mst22*mt2*DeltaInv(mst22,msb12,mC2) - 2*mC2*msb12*msb22*mst12*mt2*mu2*
     DeltaInv(mst22,msb12,mC2) + 10*mC2*msb22*mst12*mst22*mt2*mu2*DeltaInv(
     mst22,msb12,mC2) + mC2*msb12*msb22*mst12*mst22*mt2*DeltaInv(mst22,msb22,
     mC2) + 2*mC2*msb12*msb22*mst12*mt2*mu2*DeltaInv(mst22,msb22,mC2) - 10*mC2*
     msb12*mst12*mst22*mt2*mu2*DeltaInv(mst22,msb22,mC2) + msb22*mst12*mst22*
     mt2*DeltaInv(mst22,msb12,mC2)*power2(mC2) + 2*msb22*mst12*mt2*mu2*
     DeltaInv(mst22,msb12,mC2)*power2(mC2) - msb12*mst12*mst22*mt2*DeltaInv(
     mst22,msb22,mC2)*power2(mC2) - 2*msb12*mst12*mt2*mu2*DeltaInv(mst22,msb22,
     mC2)*power2(mC2) + msb12*mst22*power2(mst12) - msb22*mst22*power2(mst12) -
     msb12*mt2*power2(mst12) + msb22*mt2*power2(mst12) + mC2*msb12*msb22*mt2*
     DeltaInv(mst22,msb12,mC2)*power2(mst12) - 5*mC2*msb22*mst22*mt2*DeltaInv(
     mst22,msb12,mC2)*power2(mst12) - mC2*msb12*msb22*mt2*DeltaInv(mst22,msb22,
     mC2)*power2(mst12) + 5*mC2*msb12*mst22*mt2*DeltaInv(mst22,msb22,mC2)*
     power2(mst12) - msb22*mt2*DeltaInv(mst22,msb12,mC2)*power2(mC2)*power2(
     mst12) + msb12*mt2*DeltaInv(mst22,msb22,mC2)*power2(mC2)*power2(mst12) -
     msb12*mst12*power2(mst22) + msb22*mst12*power2(mst22) - msb12*mu2*power2(
     mst22) + msb22*mu2*power2(mst22) + 5*mC2*msb22*mst12*mt2*DeltaInv(mst22,
     msb12,mC2)*power2(mst22) - 5*mC2*msb12*mst12*mt2*DeltaInv(mst22,msb22,mC2)
     *power2(mst22) - 6*mC2*msb22*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*(-3*
     msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(mst12 - mst22 - 2*
     mt2)*mu2*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb22,mst12,mC2)) - 12*mC2*msb22*mst12*mst22*mst24*mt2*
     mu2*power2(DeltaInv(mst22,msb12,mC2)) + 12*msb12*msb22*mst12*mst22*mt2*
     mu2*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*mst22*
     mst24*mt2*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*
     mst22*mt2*power2(mC2)*power2(mst12)*power2(DeltaInv(mst22,msb12,mC2)) - 6*
     mC2*msb22*mst12*mst24*mt2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))
     + 36*mC2*msb12*msb22*mst12*mt2*mu2*power2(mst22)*power2(DeltaInv(mst22,
     msb12,mC2)) + 6*msb12*msb22*mst12*mt2*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) + 168*msb22*mst12*mt2*mu2*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mt2*power2(
     mst12)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mt2*
     power2(mC2)*power2(mst12)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))
     + 12*mC2*msb12*mst12*mst22*mst24*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))
     - 12*msb12*msb22*mst12*mst22*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) - 6*mC2*msb12*mst22*mst24*mt2*power2(mst12)*power2(DeltaInv(
     mst22,msb22,mC2)) + 6*msb12*msb22*mst22*mt2*power2(mC2)*power2(mst12)*
     power2(DeltaInv(mst22,msb22,mC2)) + 6*mC2*msb12*mst12*mst24*mt2*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 36*mC2*msb12*msb22*mst12*mt2*
     mu2*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 6*msb12*msb22*mst12*
     mt2*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 168*
     msb12*mst12*mt2*mu2*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,
     mC2)) + 18*mC2*msb12*msb22*mt2*power2(mst12)*power2(mst22)*power2(
     DeltaInv(mst22,msb22,mC2)) + 84*msb12*mt2*power2(mC2)*power2(mst12)*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) - 12*msb22*mst12*mst22*
     mt2*mu2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) + 6*msb22*mst22*mt2*
     power2(mst12)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb22*
     mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) +
     12*msb12*mst12*mst22*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 6*msb12*mst22*mt2*power2(mst12)*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 6*msb12*mst12*mt2*power2(mst22)*power2(DeltaInv(mst22,msb22,
     mC2))*power3(mC2) + 18*mC2*msb12*msb22*mst12*mt2*power2(DeltaInv(mst22,
     msb12,mC2))*power3(mst22) + 84*msb22*mst12*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mst22) - 18*mC2*msb12*msb22*mst12*mt2*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mst22) - 84*msb12*mst12*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power3(mst22) + 288*msb12*
     msb22*mst12*mst22*mst24*mt2*mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 144*msb12*msb22*mst22*mst24*mt2*power2(mC2)*power2(mst12)*power3(
     DeltaInv(mst22,msb12,mC2)) + 144*msb12*msb22*mst12*mst24*mt2*power2(mC2)*
     power2(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 288*msb22*mst12*mst22*
     mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst22*mst24*mt2*power2(mst12)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     ) + 144*msb22*mst12*mst24*mt2*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 288*msb12*msb22*mst12*mt2*mu2*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*msb22*mt2*power2(mst12)
     *power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 144*msb12*
     msb22*mst12*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 288*msb12*msb22*mst12*mst22*mst24*mt2*mu2*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*msb22*mst22*mst24*mt2*power2(mC2)*power2(
     mst12)*power3(DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*mst12*mst24*
     mt2*power2(mC2)*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 288*
     msb12*mst12*mst22*mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst22*mst24*mt2*power2(mst12)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 144*msb12*mst12*mst24*mt2*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 288*msb12*msb22*mst12*mt2*
     mu2*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 144*
     msb12*msb22*mt2*power2(mst12)*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - 144*msb12*msb22*mst12*mt2*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) - 144*msb22*mst12*mt2*mu2*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 72*msb22*mt2*power2(mst12)
     *power2(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb22*
     mst12*mt2*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) +
     144*msb12*mst12*mt2*mu2*power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 72*msb12*mt2*power2(mst12)*power2(mst22)*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 72*msb12*mst12*mt2*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*(mst12 -
     mst22 - 2*mt2)*mu2*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + 72*msb12*mst12*mst22*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - 144*msb22*mst12*mst22*mt2*
     mu2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) + 72*msb22*
     mst22*mt2*power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb12,mC2))*
     power6(mst2) - 72*msb22*mst12*mt2*power2(mC2)*power2(mst22)*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) + 144*msb12*mst12*mst22*mt2*mu2*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2) - 72*msb12*
     mst22*mt2*power2(mC2)*power2(mst12)*power3(DeltaInv(mst22,msb22,mC2))*
     power6(mst2) + 72*msb12*mst12*mt2*power2(mC2)*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(12.*msb12*msb22*mst12*(mst12 -
     mst22)*mst22) + (mu2*power2(cb)*power2(snt)*power2(log(mw2/Q2))*power3(
     mw2)*(mst22*power2(DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,
     mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb22*(mst14 + mst12*mw2) + power6(mst1))) - power3(DeltaInv(mst22,mw2,
     msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) +
     power6(mst2))))/msb22 + (2*mu2*power2(sb)*power2(snt)*power2(log(mw2/Q2))*
     power3(mw2)*(mst22*power2(DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(
     mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))) - power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/msb22 + (mu2*power2(snt)*power2(log(mw2/Q2))*
     power3(mw2)*(-(mst22*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(
     mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))) + power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/msb22 + (mu2*power2(sb)*power2(snt)*power2(
     log(mw2/Q2))*power3(mw2)*(-(mst22*power2(DeltaInv(mst22,mw2,msb22))) +
     power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*
     (-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))) +
     power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/msb22 + (mu2*power2(snb)*power2(snt)*
     power2(log(mw2/Q2))*power3(mw2)*(-(msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))) + msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)) + msb22*mst12*
     power2(DeltaInv(mw2,msb12,mst12)) - msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*
     msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*mw2*
     power3(DeltaInv(mw2,msb12,mst12)) - msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb22,mst12)) - 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb22,mst12))
     - 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,mst12)) + msb12*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - msb22*power3(DeltaInv(mw2,
     msb12,mst12))*power6(mst1) + msb12*power3(DeltaInv(mw2,msb22,mst12))*
     power6(mst1) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - msb12*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(msb12*msb22) + (mu2*power2(sb)*power2(snb)*
     power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(-(msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))) + msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     ) + msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) - msb12*mst12*power2(
     DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb12,mst12)) + 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12))
     + 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,mst12)) - msb22*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst14*
     power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,
     mst12)) + msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) -
     msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + msb12*power3(
     DeltaInv(mw2,msb22,mst12))*power6(mst1) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22) +
     (mu2*power2(cb)*power2(snb)*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(
     msb22*mst22*power2(DeltaInv(mst22,mw2,msb12)) - msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22)) - msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))
     + msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb12,mst12)) - 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,
     mst12)) + msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 2*
     msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst12*
     mw2*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*mst14*mw2*power3(DeltaInv(
     mw2,msb22,mst12)) - msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - msb12*
     power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) - msb22*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2)) + msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     msb22) + (2*mu2*power2(sb)*power2(snb)*power2(snt)*power2(log(mw2/Q2))*
     power3(mw2)*(msb22*mst22*power2(DeltaInv(mst22,mw2,msb12)) - msb12*mst22*
     power2(DeltaInv(mst22,mw2,msb22)) - msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12)) + msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*
     mst14*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb12,mst12)) - 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,
     mst12)) + msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 2*
     msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst12*
     mw2*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*mst14*mw2*power3(DeltaInv(
     mw2,msb22,mst12)) - msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - msb12*
     power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) - msb22*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2)) + msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     msb22) + (mu2*mw2*log(mw2/Q2)*power2(snt)*(-mst12 + mst22 + mst12*mw2*(-
     msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) + mst22*(msb22 - 5*mst12
     - mw2)*mw2*DeltaInv(mw2,msb22,mst12) + 6*mst12*mst22*mw2*(-mst24 + (14*
     mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)
     ) - 6*mst12*mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 +
     mw2))*power2(DeltaInv(mw2,msb22,mst12)) + 72*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(12.*msb22*mst12*mst22) + (mu2*mw2*log(mw2/
     Q2)*power2(sb)*power2(snt)*(-mst12 + mst22 + mst12*mw2*(-msb22 + 5*mst22 +
     mw2)*DeltaInv(mst22,mw2,msb22) + mst22*(msb22 - 5*mst12 - mw2)*mw2*
     DeltaInv(mw2,msb22,mst12) + 6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*mst12*
     mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb22,mst12)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*
     mw2) + power6(mst1)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) +
     power6(mst2))))/(12.*msb22*mst12*mst22) + (mu2*mw2*log(mw2/Q2)*power2(sb)*
     power2(snt)*(mst12 - mst22 + mst12*(msb22 - 5*mst22 - mw2)*mw2*DeltaInv(
     mst22,mw2,msb22) + mst22*mw2*(-msb22 + 5*mst12 + mw2)*DeltaInv(mw2,msb22,
     mst12) - 6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22
     + mw2))*power2(DeltaInv(mst22,mw2,msb22)) + 6*mst12*mst22*mw2*(-mst14 + (
     14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb22,
     mst12)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)) +
     72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(6.*
     msb22*mst12*mst22) - (mu2*mw2*log(mw2/Q2)*power2(snb)*power2(snt)*(-(
     msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + msb22*mst12*(
     msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*mst12*mw2*(-
     msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*msb22*mst22*mw2*
     DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*DeltaInv(mw2,msb12,
     mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 +
     mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*mw2*(-mst24
     + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,
     msb22)) - 6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))
     + 18*msb12*msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))
     + 6*msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     + 84*msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12))
     - 18*msb12*msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))
     - 6*msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - 84*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) - 6*msb22*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) +
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 144*msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mst22*power2(mw2)
     *power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst22*power2(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*
     mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*
     msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(
     mst1) + 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))
     *power6(mst1) + 72*msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) - 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) +
     power6(mst2))))/(12.*msb12*msb22*mst12*mst22) + (mu2*mw2*log(mw2/Q2)*
     power2(cb)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*
     mst22 - msb22*mst22 + msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(
     mst22,mw2,msb12) + msb12*mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(
     mst22,mw2,msb22) - msb12*msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*
     msb22*mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*
     DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,
     mst12) + msb22*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*
     DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 +
     (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,
     msb12)) + 6*msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(
     3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*
     power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*
     mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*
     power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*
     mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*
     mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*
     mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*
     mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*
     msb22*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*
     mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*
     msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) +
     144*msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))
     + 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(
     mst1) + 72*msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     *(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))
     - 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(
     12.*msb12*msb22*mst12*mst22) + (mu2*mw2*log(mw2/Q2)*power2(sb)*power2(snb)
     *power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*DeltaInv(mst22,mw2,msb12) + msb12*
     mst12*mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) - msb12*
     msb22*mst22*mw2*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mst22*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst22*mw2*DeltaInv(mw2,msb22,
     mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(mw2,msb22,mst12) + msb22*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - msb12*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) - 6*msb22*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb12*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*
     mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*
     power2(DeltaInv(mst22,mw2,msb22)) - 6*msb22*mst12*mst14*mst22*mw2*power2(
     DeltaInv(mw2,msb12,mst12)) + 18*msb12*msb22*mst22*mw2*power2(mst12)*
     power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst12*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) + 84*msb22*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst22*mw2*power2(mst12)
     *power2(DeltaInv(mw2,msb22,mst12)) - 6*msb12*msb22*mst12*mst22*power2(mw2)
     *power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mst12*mst22*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*mst12*mst22*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*msb12*msb22*mst12*mst14*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*mst12*
     mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*
     msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,
     msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(mst12)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + 72*msb12*mst12*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2)) - 72*msb12*mst12*
     mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(12.*msb12*
     msb22*mst12*mst22) + (mw2*log(mw2/Q2)*power2(cb)*power2(snt)*((mst12 -
     mst22)*(msb22 + mu2) + mst12*mu2*(msb22 - 5*mst22 - mw2)*mw2*DeltaInv(
     mst22,mw2,msb22) + mst22*mu2*mw2*(-msb22 + 5*mst12 + mw2)*DeltaInv(mw2,
     msb22,mst12) - 6*mst12*mst22*mu2*mw2*(-mst24 + (14*mst22 - mw2)*mw2 +
     msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) + 6*mst12*mst22*
     mu2*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb22,mst12)) - 72*mst12*mst22*mu2*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1)) + 72*mst12*mst22*mu2*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/(12.*msb22*mst12*mst22) + (mt2*power2(
     DeltaInv(mst22,msb22,mC2))*power2(log(mC2/Q2))*power3(mC2)*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb22 + (mC2*mt2*mu2*log(mC2/Q2)*(-
     mst12 + mst22 + mC2*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,
     mC2) + mC2*mst12*(mC2 - msb22 + 5*mst22)*DeltaInv(mst22,msb22,mC2) + 6*
     mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*msb22*
     mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(
     mst22,msb22,mC2)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22) + (mC2*mt2*
     mu2*log(mC2/Q2)*power2(cb)*(-mst12 + mst22 + mC2*(-mC2 + msb22 - 5*mst12)*
     mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(mC2 - msb22 + 5*mst22)*
     DeltaInv(mst22,msb22,mC2) + 6*mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22
     + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 6*
     mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(
     mC2))*power2(DeltaInv(mst22,msb22,mC2)) + 72*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - 72*mst12*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb22*mst12*(mst12 -
     mst22)*mst22) + (mC2*mu2*log(mC2/Q2)*power2(sb)*power2(snt)*(-mst12 +
     mst22 + mC2*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) +
     mC2*mst12*(mC2 - msb22 + 5*mst22)*DeltaInv(mst22,msb22,mC2) + 6*mC2*mst12*
     mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*msb22*mst22 -
     mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb22,
     mC2)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(6.*msb22*mst12*mst22) + (mC2*mu2*log(mC2/Q2)*power2(sb)*power2(
     snt)*(-mst12 + mst22 + mC2*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,
     mst12,mC2) + mC2*mst12*(mC2 - msb22 + 5*mst22)*DeltaInv(mst22,msb22,mC2) +
     6*mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*
     msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(
     DeltaInv(mst22,msb22,mC2)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(
     msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(12.*msb22*mst12*mst22*power2(cb)) + (mC2*
     mt2*mu2*log(mC2/Q2)*(mst12 - mst22 + mC2*(mC2 - msb22 + 5*mst12)*mst22*
     DeltaInv(msb22,mst12,mC2) + mC2*mst12*(-mC2 + msb22 - 5*mst22)*DeltaInv(
     mst22,msb22,mC2) - 6*mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*
     mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*
     mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2)
     )*power2(DeltaInv(mst22,msb22,mC2)) - 72*mst12*mst22*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22)
     + (mC2*mt2*mu2*log(mC2/Q2)*power2(cb)*(mst12 - mst22 + mC2*(mC2 - msb22 +
     5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(-mC2 + msb22 - 5*
     mst22)*DeltaInv(mst22,msb22,mC2) - 6*mC2*mst12*mst22*(-3*msb22*mst12 -
     mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,
     mC2)) + 6*mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24
     + power2(mC2))*power2(DeltaInv(mst22,msb22,mC2)) - 72*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(3.*msb22*
     mst12*(mst12 - mst22)*mst22) + (mC2*mu2*log(mC2/Q2)*power2(sb)*power2(snt)
     *(mst12 - mst22 + mC2*(mC2 - msb22 + 5*mst12)*mst22*DeltaInv(msb22,mst12,
     mC2) + mC2*mst12*(-mC2 + msb22 - 5*mst22)*DeltaInv(mst22,msb22,mC2) - 6*
     mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*mst12*mst22*(-3*msb22*
     mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(
     mst22,msb22,mC2)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(12.*msb22*mst12*mst22) + (mC2*log(mC2/Q2)*power2(
     snb)*(-((msb12 - msb22)*(mst22 - mt2)) + mC2*msb22*(mC2 - msb12 + 5*mst22)
     *mt2*DeltaInv(mst22,msb12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst22)*mt2*
     DeltaInv(mst22,msb22,mC2) - 6*mC2*msb22*mst22*mt2*(-3*msb12*mst22 - mC2*(
     msb12 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb12,mC2))
     + 6*mC2*msb12*mst22*mt2*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 +
     power2(mC2))*power2(DeltaInv(mst22,msb22,mC2)) - 72*msb22*mst22*mt2*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)) + 72*msb12*mst22*
     mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*
     (msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(12.*msb12*
     msb22*mst22) + (mC2*log(mC2/Q2)*power2(snt)*((mst12 - mst22)*(msb22 + mt2)
     + mC2*(mC2 - msb22 + 5*mst12)*mst22*mt2*DeltaInv(msb22,mst12,mC2) + mC2*
     mst12*(-mC2 + msb22 - 5*mst22)*mt2*DeltaInv(mst22,msb22,mC2) - 6*mC2*
     mst12*mst22*mt2*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*mst12*mst22*mt2*(-3*msb22*
     mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(
     mst22,msb22,mC2)) - 72*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) + 72*mst12*mst22*mt2*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(12.*msb22*mst12*mst22) - (mC2*log(mC2/Q2)*(
     4*msb22*mst12*mst22 - mC2*(msb22*mst22 + mst12*(mst22 - mt2)) + msb22*
     mst22*mt2 + mst12*mst22*mt2 - mC2*mst12*(mC2 - msb22 + 5*mst22)*(mC2 -
     mt2)*mt2*DeltaInv(mst22,msb22,mC2) - mst12*power2(mt2) + 6*mC2*mst12*
     mst22*(mC2 - mt2)*mt2*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 +
     power2(mC2))*power2(DeltaInv(mst22,msb22,mC2)) + 72*mst12*mst22*(mC2 -
     mt2)*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 -
     2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(12.*
     msb22*mst12*mst22*(mC2 - mt2)) + (mC2*log(mC2/Q2)*power2(cb)*power2(snt)*(
     -((mst12 - mst22)*(msb22 + mt2 + mu2)) - mC2*(mC2 - msb22 + 5*mst12)*
     mst22*(mt2 + mu2)*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(mC2 - msb22 + 5*
     mst22)*(mt2 + mu2)*DeltaInv(mst22,msb22,mC2) + 6*mC2*mst12*mst22*(mt2 +
     mu2)*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(mt2 + mu2)*(-3*
     msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(
     DeltaInv(mst22,msb22,mC2)) + 72*mst12*mst22*(mt2 + mu2)*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - 72*mst12*mst22*(mt2 + mu2)*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(12.*msb22*
     mst12*mst22) + (mC2*log(mC2/Q2)*power2(cb)*((msb22*mst22*(4*mst12 + mt2) +
     mt2*(mst12*mst22 - mst12*mt2 + mst22*mu2 + 2*mt2*mu2) - mC2*(msb22*mst22 +
     mst12*mst22 - mst12*mt2 + mst22*mu2 + 2*mt2*mu2))/(mst12*mst22*(mC2 - mt2)
     ) + (mC2*(mC2 - msb22 + 5*mst12)*(mst12 - mst22 - 2*mt2)*mu2*DeltaInv(
     msb22,mst12,mC2))/(mst12*(mst12 - mst22)) - (mC2*(mC2 - msb22 + 5*mst22)*
     mt2*(-mst12 + mst22 + 2*mu2)*DeltaInv(mst22,msb22,mC2))/(mst22*(-mst12 +
     mst22)) - (6*mC2*(mst12 - mst22 - 2*mt2)*mu2*(-3*msb22*mst12 - mC2*(msb22
     + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)))/(
     mst12 - mst22) - (6*mC2*mt2*(-mst12 + mst22 + 2*mu2)*(-3*msb22*mst22 -
     mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb22,
     mC2)))/(mst12 - mst22) - (72*(mst12 - mst22 - 2*mt2)*mu2*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)))/(mst12 - mst22) - (72*mt2*(-
     mst12 + mst22 + 2*mu2)*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2)))/(mst12 - mst22)))/(12.*msb22) + Fin3(mst22,mw2,msb12,Q2)*((2*mu2*
     power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,
     msb12))*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 + (4*mu2*power2(mw2)*
     power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(
     mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(
     mst24 + mst22*mw2) + power6(mst2))))/msb12 + (2*mu2*power2(mw2)*power2(
     snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 + DeltaInv(
     mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2)
     + power6(mst2))))/msb12 + (2*mu2*power2(mw2)*power2(sb)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 + DeltaInv(mst22,
     mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2))))/msb12) + Fin3(mst22,mw2,msb22,Q2)*((2*mu2*power2(cb)*
     power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 -
     DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/msb22 + (4*mu2*power2(mw2)*power2(sb)*power2(
     snt)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*
     (mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2)))
     )/msb22 + (2*mu2*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 + (2*mu2*
     power2(mw2)*power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,
     msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 + (2*mu2*power2(mw2)*
     power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(-mst22 + DeltaInv(mst22,
     mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) +
     power6(mst2))))/msb22 + (2*mu2*power2(mw2)*power2(sb)*power2(snt)*power2(
     DeltaInv(mst22,mw2,msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 +
     (2*mu2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 + (4*mu2*
     power2(mw2)*power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,
     msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) -
     2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22) + Fin3(mst22,msb12,
     mC2,Q2)*((4*mt2*mu2*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,
     mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*
     mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 -
     mst22)) + (8*mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(
     mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*
     mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(
     mst12 - mst22)) + (2*mt2*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     msb12 + (2*mu2*power2(mC2)*power2(sb)*power2(snb)*power2(snt)*power2(
     DeltaInv(mst22,msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*
     mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     msb12 + (2*mt2*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(
     -mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (4*mt2*mu2*power2(
     mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(
     mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)) + (2*mt2*(mst12 -
     mst22 - 2*mu2)*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,
     msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(-mst12
     + mst22)) + (4*mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(
     mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*
     mC2*(msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(
     mst12 - mst22)) + (2*(mt2 + mu2)*power2(cb)*power2(mC2)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,
     msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/msb12 + (4*mu2*power2(mC2)*power2(sb)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,
     msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/msb12 + (2*mu2*power2(mC2)*power2(sb)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 + DeltaInv(mst22,
     msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb12*power2(cb))) + (2*mt2*mu2*power2(log(mC2/Q2)
     )*power3(mC2)*(-(mst12*power2(DeltaInv(msb22,mst12,mC2))) + power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(
     mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2)))))/(msb22*(mst12 - mst22)) + (
     4*mt2*mu2*power2(cb)*power2(log(mC2/Q2))*power3(mC2)*(-(mst12*power2(
     DeltaInv(msb22,mst12,mC2))) + power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2)))))/(msb22*(mst12 - mst22)) + (mt2*power2(snt)*power2(log(mC2/Q2))*
     power3(mC2)*(-(mst12*power2(DeltaInv(msb22,mst12,mC2))) + power3(DeltaInv(
     msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(mst22 -
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/msb22 - ((mt2 + mu2)*power2(cb)*
     power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(-(mst12*power2(DeltaInv(
     msb22,mst12,mC2))) + power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 -
     2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(
     DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     /msb22 + (mu2*power2(sb)*power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(-(
     mst12*power2(DeltaInv(msb22,mst12,mC2))) + power3(DeltaInv(msb22,mst12,
     mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2)))))/msb22 + (mt2*power2(snb)*power2(log(mC2/Q2))*
     power3(mC2)*(-(msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))) + msb22*
     power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2)) + msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/(msb12*msb22)
     + (2*mt2*mu2*power2(log(mC2/Q2))*power3(mC2)*(mst12*power2(DeltaInv(msb22,
     mst12,mC2)) - power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(
     DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     /(msb22*(mst12 - mst22)) + (2*mt2*mu2*power2(cb)*power2(log(mC2/Q2))*
     power3(mC2)*(mst12*power2(DeltaInv(msb22,mst12,mC2)) - power3(DeltaInv(
     msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/(msb22*(mst12 - mst22)) + (2*mu2*
     power2(sb)*power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(mst12*power2(
     DeltaInv(msb22,mst12,mC2)) - power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2)))))/msb22 + (mu2*power2(sb)*power2(snt)*power2(log(mC2/Q2))*power3(
     mC2)*(mst12*power2(DeltaInv(msb22,mst12,mC2)) - power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/(msb22*power2(cb)) + (power2(cb)*
     power2(log(mC2/Q2))*power3(mC2)*(mst12*(-mst12 + mst22 + 2*mt2)*mu2*
     power2(DeltaInv(msb22,mst12,mC2)) + (mst12 - mst22 - 2*mt2)*mu2*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - mt2*(mst12 - mst22 - 2*mu2)*power2(
     DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     /(msb22*(mst12 - mst22)) + Fin3(mst22,msb22,mC2,Q2)*((4*mt2*mu2*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*
     (-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2))))/(msb22*(mst12 - mst22)) + (8*mt2*mu2*power2(cb)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*
     (-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2))))/(msb22*(mst12 - mst22)) + (2*mt2*power2(mC2)*power2(snb)*
     power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/msb22 + (4*mt2*mu2*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 -
     mst22)) + (4*mt2*mu2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(
     mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*
     mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(
     mst12 - mst22)) + (2*mt2*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (2*mu2*
     power2(mC2)*power2(sb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(
     mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (4*mu2*power2(mC2)*
     power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(
     mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (2*mu2*power2(mC2)*
     power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(
     mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*power2(cb)) + (2*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/msb22 + (4*mt2*mu2*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*
     mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(
     mst12 - mst22)) + (2*mt2*(mst12 - mst22 - 2*mu2)*power2(cb)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(msb22*(-mst12 + mst22)) + (4*mt2*mu2*power2(cb)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(msb22*(mst12 - mst22)) + (4*mt2*mu2*power2(mC2)*power2(snb)*
     power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(msb22*(mst12 - mst22)) + (2*mt2*(mst12 - mst22 - 2*mu2)*power2(
     cb)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22)) + (8*mt2*mu2*
     power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(-
     mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22)) + (2*
     (mt2 + mu2)*power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,
     msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (4*mu2*
     power2(mC2)*power2(sb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-
     mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (2*mu2*power2(mC2)*
     power2(sb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb22*power2(cb)) + (2*mt2*power2(
     mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb22 - (2*(mt2 + mu2)*power2(cb)*
     power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-
     mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (2*mu2*power2(mC2)*
     power2(sb)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-
     mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb22) + (power2(cb)*power2(
     snb)*power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*(-mst12 + mst22 + 2*mt2)
     *mu2*power2(DeltaInv(msb12,mst12,mC2)) + msb12*mst12*(mst12 - mst22 - 2*
     mt2)*mu2*power2(DeltaInv(msb22,mst12,mC2)) + msb22*(mst12 - mst22 - 2*mt2)
     *mu2*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - msb12*(mst12 - mst22
     - 2*mt2)*mu2*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - mt2*(mst12 -
     mst22 - 2*mu2)*(-(msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))) + msb22*
     power3(DeltaInv(mst22,msb12,mC2))*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2)) + msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))))/(msb12*msb22*
     (mst12 - mst22)) + power2(xb)*(-(log(mA2/Q2)*log(msb12/Q2)*power2(mA2)*
     power2(DeltaInv(msb12,mA2,msb22))*(3*mA2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(msb12,mA2,msb22)*(5*(msb12*msb22 + msb24)*power2(mA2) - 3*msb22*
     power3(mA2) + (msb12 - msb22)*power3(msb22) - mA2*(-10*msb12*msb24 +
     power3(msb22)))))/(2.*msb12) + (log(mA2/Q2)*log(msb12/Q2)*power2(cb)*
     power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))*(3*mA2*msb22 + msb12*msb22 +
     msb24 + DeltaInv(msb12,mA2,msb22)*(5*(msb12*msb22 + msb24)*power2(mA2) -
     3*msb22*power3(mA2) + (msb12 - msb22)*power3(msb22) - mA2*(-10*msb12*msb24
     + power3(msb22)))))/(2.*msb12) - (log(mh2/Q2)*log(msb12/Q2)*power2(mh2)*
     power2(sa)*power2(DeltaInv(mh2,msb12,msb22))*(3*mh2*msb22 + msb12*msb22 +
     msb24 + DeltaInv(mh2,msb12,msb22)*(5*(msb12*msb22 + msb24)*power2(mh2) -
     3*msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) - mh2*(-10*msb12*msb24
     + power3(msb22)))))/(2.*msb12) - (log(mH2/Q2)*log(msb12/Q2)*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(mH2,msb12,msb22)*(5*(msb12*msb22 + msb24)*power2(mH2) - 3*msb22*
     power3(mH2) + (msb12 - msb22)*power3(msb22) - mH2*(-10*msb12*msb24 +
     power3(msb22)))))/(2.*msb12) + (log(mH2/Q2)*log(msb12/Q2)*power2(mH2)*
     power2(sa)*power2(DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22 + msb12*msb22 +
     msb24 + DeltaInv(mH2,msb12,msb22)*(5*(msb12*msb22 + msb24)*power2(mH2) -
     3*msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) - mH2*(-10*msb12*msb24
     + power3(msb22)))))/(2.*msb12) + (2*log(msb12/Q2)*log(msb22/Q2)*power2(
     mH2)*power2(snb)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*
     msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 - msb24)*power2(
     mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mH2*(2*
     msb12*msb24 + power3(msb22)))))/msb12 + (log(mA2/Q2)*log(msb22/Q2)*power2(
     mA2)*power2(DeltaInv(msb12,mA2,msb22))*(mA2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(msb12,mA2,msb22)*((msb12*msb22 + msb24)*power2(mA2) - msb22*
     power3(mA2) + (msb12 - msb22)*power3(msb22) + mA2*(6*msb12*msb24 + power3(
     msb22)))))/(2.*msb12) - (log(mA2/Q2)*log(msb22/Q2)*power2(cb)*power2(mA2)*
     power2(DeltaInv(msb12,mA2,msb22))*(mA2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(msb12,mA2,msb22)*((msb12*msb22 + msb24)*power2(mA2) - msb22*
     power3(mA2) + (msb12 - msb22)*power3(msb22) + mA2*(6*msb12*msb24 + power3(
     msb22)))))/(2.*msb12) + (log(mh2/Q2)*log(msb22/Q2)*power2(mh2)*power2(sa)*
     power2(DeltaInv(mh2,msb12,msb22))*(mh2*msb22 + msb12*msb22 + msb24 +
     DeltaInv(mh2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mh2) - msb22*
     power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(
     msb22)))))/(2.*msb12) + (log(mH2/Q2)*log(msb22/Q2)*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*msb22 + msb24 + DeltaInv(
     mH2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) +
     (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))))/(2.
     *msb12) - (log(mH2/Q2)*log(msb22/Q2)*power2(mH2)*power2(sa)*power2(
     DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*msb22 + msb24 + DeltaInv(
     mH2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) +
     (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))))/(2.
     *msb12) - (log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*power2(
     DeltaInv(msb12,mst12,mC2))*(3*mC2*mst12 + msb12*mst12 + mst14 + DeltaInv(
     msb12,mst12,mC2)*(5*(msb12*mst12 + mst14)*power2(mC2) - 3*mst12*power3(
     mC2) + (msb12 - mst12)*power3(mst12) - mC2*(-10*msb12*mst14 + power3(
     mst12)))))/msb12 + (log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(3*mC2*mst12 + msb12*mst12 +
     mst14 + DeltaInv(msb12,mst12,mC2)*(5*(msb12*mst12 + mst14)*power2(mC2) -
     3*mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) - mC2*(-10*msb12*mst14
     + power3(mst12)))))/msb12 - (log(msb12/Q2)*log(mst12/Q2)*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 +
     3*mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12 - mst14)*power2(mC2) -
     mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*
     mst14 + power3(mst12)))))/msb12 + (log(msb12/Q2)*log(mst12/Q2)*power2(mC2)
     *power2(snb)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(mC2*mst12 +
     msb12*mst12 + 3*mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12 - mst14)*
     power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*
     mC2*(2*msb12*mst14 + power3(mst12)))))/msb12 - (log(mC2/Q2)*log(msb22/Q2)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 + msb22*mst12 +
     mst14 + DeltaInv(msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*power2(mC2) -
     3*mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-10*msb22*mst14
     + power3(mst12)))))/msb22 + (log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 + msb22*mst12 + mst14
     + DeltaInv(msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*power2(mC2) - 3*
     mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-10*msb22*mst14 +
     power3(mst12)))))/msb22 + (log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(
     snb)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 + msb22*mst12 + mst14
     + DeltaInv(msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*power2(mC2) - 3*
     mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-10*msb22*mst14 +
     power3(mst12)))))/msb22 - (log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(
     mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 + msb22*
     mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*power2(
     mC2) - 3*mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-10*
     msb22*mst14 + power3(mst12)))))/msb22 + (power2(mC2)*power2(snt)*power2(
     DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*(mst14 - DeltaInv(msb22,
     mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))))/msb22 + (power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*power2(log(mst12/Q2))*(-mst14 + DeltaInv(msb22,mst12,
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))))/msb22 + (DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)
     *(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*
     mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))))/(12.*msb22) - (DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)*power2(
     snt)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*
     (-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))))/(12.*msb22) - (log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 +
     power3(mst12)))))/msb22 + (log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(
     snb)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14
     + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 +
     power3(mst12)))))/msb22 + (log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(
     snt)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14
     + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 +
     power3(mst12)))))/msb22 - (log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(
     snb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*
     mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb22*mst14 + power3(mst12)))))/msb22 + (log(mC2/Q2)*log(mst12/Q2)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))))/msb22 - (log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))))/msb22 - (log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snt)*
     power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))))/msb22 + (log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*
     power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 +
     mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) -
     mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 +
     power3(mst12)))))/msb22 - (log(msb12/Q2)*log(mst22/Q2)*power2(mC2)*power2(
     snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*mst22 + msb12*
     mst22 + 3*mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 - mst24)*power2(
     mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + 5*mC2*(2*
     msb12*mst24 + power3(mst22)))))/msb12 + (power2(mC2)*power2(snt)*power2(
     DeltaInv(mst22,msb22,mC2))*power2(log(mst22/Q2))*(-mst24 + DeltaInv(mst22,
     msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))))/msb22 + (DeltaInv(mst22,msb22,mC2)*log(
     mst22/Q2)*power2(snt)*(5*mC2*mst22 - msb22*mst22 + mst24 - 6*mC2*DeltaInv(
     mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) - 72*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))))/(12.*msb22) - (log(msb22/Q2)*log(mst22/Q2)
     *power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 +
     msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 - mst24)*
     power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*
     mC2*(2*msb22*mst24 + power3(mst22)))))/msb22 + (log(msb22/Q2)*log(mst22/
     Q2)*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*
     (mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*
     mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*
     power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))))/msb22 + (log(mC2/
     Q2)*log(mst22/Q2)*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2)
     )*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*
     mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) + mC2*(6*msb22*mst24 + power3(mst22)))))/msb22 - (log(mC2/Q2)*log(
     mst22/Q2)*power2(cb)*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,
     mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((
     msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*
     power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))))/msb22 + (log(msb22/
     Q2)*power2(sa)*((5*mh2*msb22 - msb12*msb22 + msb24)*DeltaInv(mh2,msb12,
     msb22) - 6*mh2*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*
     mh2*msb24 - msb12*msb24 + msb22*power2(mh2) + power3(msb22)) + DeltaInv(
     mH2,msb12,msb22)*(-5*mH2*msb22 + msb12*msb22 - msb24 + 6*mH2*DeltaInv(mH2,
     msb12,msb22)*(-3*mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*
     power2(mH2) + power3(msb22)) + 72*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22))*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(
     msb12*msb24 + power3(msb22)))) - 72*power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22))))/(24.*msb12) + (power2(sa)*power2(log(
     msb22/Q2))*(-(msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) +
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(msb24 - DeltaInv(mH2,msb12,
     msb22)*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(
     msb12*msb24 + power3(msb22)))) + power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22))))/(2.*msb12) + (log(msb12/Q2)*log(msb22/
     Q2)*power2(sa)*(-((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)
     )*(mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*
     msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)*
     power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(mh2)*((-(
     msb12*msb22) + msb24)*power2(mh2) + msb22*power3(mh2) + (-5*msb12 + 3*
     msb22)*power3(msb22) - 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(2.*msb12) + (2*log(msb12/Q2)*log(msb22/Q2)*
     power2(sa)*power2(snb)*((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22))*(-(mH2*msb22) - msb12*msb22 - 3*msb24 + DeltaInv(mH2,msb12,msb22)*
     ((-(msb12*msb22) + msb24)*power2(mH2) + msb22*power3(mH2) + (-5*msb12 + 3*
     msb22)*power3(msb22) - 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(
     mh2)*((msb12*msb22 - msb24)*power2(mh2) - msb22*power3(mh2) + (5*msb12 -
     3*msb22)*power3(msb22) + 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/msb12 + (power2(sa)*power2(log(msb12/Q2))*(-(
     (2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*
     msb22 + msb12*msb22 + 2*msb24 + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22
     + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(
     msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))) + power2(mh2)*(-((3*
     msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*
     msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(2.*msb12) + (power2(sa)*power2(snb)*power2(
     log(msb12/Q2))*(mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,msb12)) + 2*(2*
     mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + mH2*(-(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12))) + 2*mH2*
     power2(DeltaInv(mH2,msb12,msb22))*(-2*mH2*msb22 - msb12*msb22 - 2*msb24 +
     DeltaInv(mH2,msb12,msb22)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*
     msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22))))) + 2*power2(mh2)*((3*msb12*msb22 + 2*msb24)*
     power2(mh2) - 2*msb22*power3(mh2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*
     mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/
     msb12 + (power2(snb)*power2(log(msb22/Q2))*(2*msb22*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + msb12*(mH2*msb22*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))
     *(2*mC2*mst12 + msb22*mst12 + 2*mst14 + DeltaInv(msb22,mst12,mC2)*((3*
     msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*
     mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12))))) - 2*msb22*
     power2(mH2)*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*
     (msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))/(msb12*
     msb22) - (log(msb12/Q2)*log(msb22/Q2)*((mH2*msb22 + msb12*msb22 + 3*msb24)
     *power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mA2)*power2(
     DeltaInv(msb12,mA2,msb22))*(mA2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(
     msb12,mA2,msb22)*((msb12*msb22 - msb24)*power2(mA2) - msb22*power3(mA2) +
     (5*msb12 - 3*msb22)*power3(msb22) + 5*mA2*(2*msb12*msb24 + power3(msb22)))
     ) + power2(mH2)*((msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (
     5*msb12 - 3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22))))/(2.*msb12) + (log(msb12/Q2)*power2(sa)
     *(mh2 - mH2 - DeltaInv(mh2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) -
     (msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(msb22) + power3(mh2)) +
     DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*
     msb22)*power2(mH2) + 5*mH2*power2(msb22) + power3(mH2)) + 6*mh2*msb22*
     power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*msb12*msb22 +
     23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) + power3(msb22))
     - 6*mH2*msb22*power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*msb24) + mH2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mH2) + power3(mH2) +
     power3(msb22)) - 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 72*
     msb22*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*msb22*
     power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))/(24.*msb12*msb22) - (
     power2(log(msb12/Q2))*((2*mH2*msb22 + msb12*msb22 + 2*msb24)*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + power2(mA2)*power2(DeltaInv(msb12,mA2,
     msb22))*(2*mA2*msb22 + msb12*msb22 + 2*msb24 + DeltaInv(msb12,mA2,msb22)*(
     (3*msb12*msb22 + 2*msb24)*power2(mA2) - 2*msb22*power3(mA2) + (3*msb12 -
     2*msb22)*power3(msb22) + 2*mA2*(5*msb12*msb24 + power3(msb22)))) + power2(
     mH2)*((3*msb12*msb22 + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*
     msb12 - 2*msb22)*power3(msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22))))/(2.*msb12) + (power2(snb)*power2(log(
     msb12/Q2))*(mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*(2*mH2*
     msb22 + msb12*msb22 + 2*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))*(-2*mC2*mst12 -
     msb12*mst12 - 2*mst14 + DeltaInv(msb12,mst12,mC2)*(-((3*msb12*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))) + 2*power2(mH2)*((3*
     msb12*msb22 + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*
     msb22)*power3(msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mH2,msb12,msb22))))/msb12 + (power2(sa)*power2(snb)*power2(log(
     msb22/Q2))*(2*msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*
     msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - 2*power2(mh2)*(msb24*
     power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + mH2*(-2*mH2*msb24*
     power2(DeltaInv(mH2,msb12,msb22)) - msb12*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) + 2*mH2*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*
     mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)))))/
     msb12 - (power2(log(msb22/Q2))*(msb22*msb24*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) + msb22*msb24*power2(mA2)*power2(DeltaInv(msb12,mA2,
     msb22)) + 2*msb12*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(2*mC2*
     mst12 + msb22*mst12 + 2*mst14 + DeltaInv(msb22,mst12,mC2)*((3*msb22*mst12
     + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))) - msb22*power2(mH2)*(
     msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) - msb22*power2(mA2)*(
     msb24*power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22))))/(2.*msb12*msb22) + (
     log(msb12/Q2)*(mA2 + mH2 + 2*msb12 + 2*msb22 - DeltaInv(msb12,mA2,msb22)*(
     msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mA2) + 5*mA2*
     power2(msb22) + power3(mA2)) - DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*
     msb22) + msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*power2(msb22) +
     power3(mH2)) + 6*mA2*msb22*power2(DeltaInv(msb12,mA2,msb22))*(-(msb12*
     msb24) + mA2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mA2) +
     power3(mA2) + power3(msb22)) + 6*mH2*msb22*power2(DeltaInv(mH2,msb12,
     msb22))*(-(msb12*msb24) + mH2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*
     msb22)*power2(mH2) + power3(mH2) + power3(msb22)) - 72*msb22*power2(mH2)*(
     -((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*msb22*power3(mH2) + (-3*msb12
     + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb22*power2(mA2)*(-((3*msb12*msb22 + 2*
     msb24)*power2(mA2)) + 2*msb22*power3(mA2) + (-3*msb12 + 2*msb22)*power3(
     msb22) - 2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,
     msb22))))/(24.*msb12*msb22) - (log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*
     power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*
     mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) -
     mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(
     -10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/
     msb12 + (log(mC2/Q2)*log(msb12/Q2)*power2(cb)*power2(mC2)*power2(snb)*
     power2(snt)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,
     mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*
     mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(
     mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*
     msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) +
     3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/msb12 + (
     power2(mC2)*power2(snb)*power2(snt)*power2(log(mst12/Q2))*(msb22*mst14*
     power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(msb22,mst12,mC2)
     )*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) - msb22*(
     mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (
     power2(mC2)*power2(snb)*power2(log(mst12/Q2))*(-(msb22*mst14*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mst14 - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) + msb22*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (log(
     mst12/Q2)*power2(snb)*(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(
     msb12,mst12,mC2) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) + msb12*DeltaInv(msb22,mst12,mC2)*(-5*mC2*mst12 + msb22*mst12 -
     mst14 + 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14
     - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 72*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) - 72*msb22*power2(
     mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(12.*msb12*
     msb22) + (log(mst12/Q2)*power2(snb)*power2(snt)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb22*power2(
     DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*
     mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,mst12,
     mC2)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*
     (-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(12.*msb12*msb22) + (power2(mC2)*power2(snb)*
     power2(snt)*power2(log(msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*
     power2(DeltaInv(msb12,mst12,mC2)) + power2(DeltaInv(mst22,msb12,mC2))*(-2*
     mC2*mst22 - msb12*mst22 - 2*mst24 + DeltaInv(mst22,msb12,mC2)*(-((3*msb12*
     mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)
     *power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))) + ((3*msb12*mst12
     + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2))))/msb12 + (log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(
     msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*
     msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(
     mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22)
     + (log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*power2(snt)*(-(msb22*
     (mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 +
     DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(
     mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*power3(
     mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (log(mC2/Q2)*log(
     mst12/Q2)*power2(mC2)*power2(snb)*(msb22*(mC2*mst12 + msb12*mst12 + mst14)
     *power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(msb22,mst12,
     mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(msb22,mst12,mC2)*(-((
     msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb22 + mst12)*
     power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*((msb12*
     mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*power3(
     mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22) + (log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*power2(snt)*(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-(
     mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(msb22,mst12,mC2)*(-((msb22*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb22 + mst12)*power3(
     mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*((msb12*mst12 +
     mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) +
     mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*msb22) - (log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(snt)*(-((3*
     mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*msb22*mst24 +
     power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 + (log(mC2/Q2)*log(
     msb22/Q2)*power2(cb)*power2(mC2)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12
     + mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,
     mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(
     msb22*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*
     power3(mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12
     + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(
     mst12) + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2))))/msb22 + (log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*
     mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(
     mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*
     msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) +
     3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 - (log(
     mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*(-((
     3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*msb22*mst24 +
     power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(
     mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/msb22 + (power2(mC2)*power2(
     snb)*power2(snt)*power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*
     mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,
     mC2))*(2*mC2*mst22 + msb22*mst22 + 2*mst24 + DeltaInv(mst22,msb22,mC2)*((
     3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*
     mst22)*power3(mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))) + (-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2))))/msb22 + (power2(mC2)*power2(snt)*power2(log(
     msb22/Q2))*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,
     mst12,mC2)) + power2(DeltaInv(mst22,msb22,mC2))*(-2*mC2*mst22 - msb22*
     mst22 - 2*mst24 + DeltaInv(mst22,msb22,mC2)*(-((3*msb22*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb22*mst24 + power3(mst22)))) + ((3*msb22*mst12 + 2*mst14)*
     power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(mst12) + 2*
     mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/
     msb22 + (log(msb22/Q2)*(msb22*(5*mH2*msb22 - msb12*msb22 + msb24)*mst12*
     DeltaInv(mH2,msb12,msb22) + msb22*(5*mA2*msb22 - msb12*msb22 + msb24)*
     mst12*DeltaInv(msb12,mA2,msb22) - 6*mA2*msb22*mst12*power2(DeltaInv(msb12,
     mA2,msb22))*(-3*mA2*msb12*msb22 - 14*mA2*msb24 - msb12*msb24 + msb22*
     power2(mA2) + power3(msb22)) - 6*mH2*msb22*mst12*power2(DeltaInv(mH2,
     msb12,msb22))*(-3*mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*
     power2(mH2) + power3(msb22)) - 72*msb22*mst12*power2(mH2)*(msb24*power2(
     mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(
     msb22)))*power3(DeltaInv(mH2,msb12,msb22)) - 72*msb22*mst12*power2(mA2)*(
     msb24*power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + 2*msb12*(mC2 + msb22
     + mst12 - DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (
     msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + 6*mC2*
     mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 72*mst12*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)))))/(24.*msb12*
     msb22*mst12) + (power2(mC2)*power2(snb)*power2(snt)*power2(log(mst22/Q2))*
     (-(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(
     mst22,msb22,mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) +
     (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) +
     msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(
     msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*
     msb22) + (log(mst22/Q2)*power2(snb)*power2(snt)*(msb22*(5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - 6*mC2*msb22*power2(
     DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*
     mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*DeltaInv(mst22,msb22,
     mC2)*(-5*mC2*mst22 + msb22*mst22 - mst24 + 6*mC2*DeltaInv(mst22,msb22,mC2)
     *(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) +
     power3(mst22)) + 72*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))) - 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 +
     mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(12.*msb12*msb22) + (log(msb12/Q2)*power2(
     snb)*power2(snt)*((mC2 + msb12)*(mst12 - mst22) + mst22*DeltaInv(msb12,
     mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2)
     + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb12,mC2)*(
     mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*
     power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb12,
     mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*
     power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 +
     23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(12.*msb12*mst12*
     mst22) + (log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*
     power2(snt)*(-(msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(
     mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 +
     msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*
     power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*
     msb22*mst24 + power3(mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(
     mC2)) + mst22*power3(mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22)
     + (log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(snt)*(msb22*(
     mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) +
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (log(msb22/
     Q2)*power2(snt)*((mC2 + msb22)*(mst12 - mst22) + mst22*DeltaInv(msb22,
     mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2)
     + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb22,mC2)*(
     mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*
     power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb22,
     mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 +
     23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(12.*msb22*mst12*
     mst22) - (log(msb22/Q2)*power2(snb)*power2(snt)*((mC2 + msb22)*(mst12 -
     mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) -
     (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) -
     mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 -
     5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2))))/(12.*msb22*mst12*mst22) + (power2(Pi)*power2(sa)*(-((3*mh2*
     msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22))) + power2(mh2)*(-((5*msb12*msb22 + 3*msb24)*power2(mh2)) + 3*
     msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - mh2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22 + msb12*msb22 + 3*msb24 +
     DeltaInv(mH2,msb12,msb22)*(14*mH2*msb12*msb24 + 5*msb12*msb22*power2(mH2)
     + 3*msb24*power2(mH2) - 3*msb22*power3(mH2) + 3*mH2*power3(msb22) + 5*
     msb12*power3(msb22) - 3*power4(msb22)))))/(12.*msb12) + (power2(cb)*
     power2(log(msb12/Q2))*((2*mA2*msb22 + msb12*msb22 + 2*msb24)*power2(mA2)*
     power2(DeltaInv(msb12,mA2,msb22)) + power2(mA2)*((3*msb12*msb22 + 2*msb24)
     *power2(mA2) - 2*msb22*power3(mA2) + (3*msb12 - 2*msb22)*power3(msb22) +
     2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) +
     power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(-(msb12*msb22) - 2*(msb24 +
     msb22*mz2) - DeltaInv(msb12,mz2,msb22)*(10*msb12*msb24*mz2 + 3*msb12*
     msb22*power2(mz2) + 2*msb24*power2(mz2) + 3*msb12*power3(msb22) + 2*mz2*
     power3(msb22) - 2*msb22*power3(mz2) - 2*power4(msb22)))))/(2.*msb12) + (
     log(msb12/Q2)*power2(cb)*(-mA2 + mz2 + DeltaInv(msb12,mA2,msb22)*(msb22*(-
     (msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mA2) + 5*mA2*power2(
     msb22) + power3(mA2)) - 6*mA2*msb22*power2(DeltaInv(msb12,mA2,msb22))*(-(
     msb12*msb24) + mA2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(
     mA2) + power3(mA2) + power3(msb22)) - DeltaInv(msb12,mz2,msb22)*(5*mz2*
     power2(msb22) - msb12*(power2(msb22) + power2(mz2)) + msb22*(msb24 + 5*
     power2(mz2)) + power3(mz2)) + 6*msb22*mz2*power2(DeltaInv(msb12,mz2,msb22)
     )*(23*msb24*mz2 + msb22*mz2*(6*msb12 + 23*mz2) - msb12*(msb24 + power2(
     mz2)) + power3(msb22) + power3(mz2)) + 72*msb22*power2(mA2)*(-((3*msb12*
     msb22 + 2*msb24)*power2(mA2)) + 2*msb22*power3(mA2) + (-3*msb12 + 2*msb22)
     *power3(msb22) - 2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     msb12,mA2,msb22)) + 72*msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))
     *(10*msb12*msb24*mz2 + 3*msb12*msb22*power2(mz2) + 2*msb24*power2(mz2) +
     3*msb12*power3(msb22) + 2*mz2*power3(msb22) - 2*msb22*power3(mz2) - 2*
     power4(msb22))))/(24.*msb12*msb22) - (log(msb12/Q2)*log(mz2/Q2)*power2(cb)
     *power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 + msb24 + 3*
     msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(5*msb24*power2(mz2) - mz2*power3(
     msb22) + msb12*(10*msb24*mz2 + 5*msb22*power2(mz2) + power3(msb22)) - 3*
     msb22*power3(mz2) - power4(msb22))))/(2.*msb12) + (log(msb22/Q2)*log(mz2/
     Q2)*power2(cb)*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22
     + msb24 + msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(msb24*power2(mz2) + mz2*
     power3(msb22) + msb12*(6*msb24*mz2 + msb22*power2(mz2) + power3(msb22)) -
     msb22*power3(mz2) - power4(msb22))))/(2.*msb12) + (log(msb22/Q2)*power2(
     sa)*power2(snb)*(-((5*mh2*msb22 - msb12*msb22 + msb24)*DeltaInv(mh2,msb12,
     msb22)) - msb12*(mh2 - 6*msb22)*DeltaInv(mh2,msb22,msb22) + 5*mH2*msb22*
     DeltaInv(mH2,msb12,msb22) - msb12*msb22*DeltaInv(mH2,msb12,msb22) + msb24*
     DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) - 6*msb12*
     msb22*DeltaInv(mH2,msb22,msb22) + 6*mH2*msb12*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) + 18*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 84*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*
     msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(
     mH2,msb12,msb22))*power3(mH2) - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power3(msb22) - 72*msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) +
     6*mh2*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24
     - msb12*msb24 + msb22*power2(mh2) + power3(msb22)) + 24*msb12*power2(
     DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*
     (msb24*power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*
     power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(
     msb22)*power3(DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,
     msb12,msb22))*power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22)
     )*power4(msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22)))/(
     6.*msb12) - (2*log(mh2/Q2)*log(msb22/Q2)*power2(sa)*power2(snb)*((mh2*
     msb22 + msb12*msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))
     + power2(mh2)*((msb12*msb22 + msb24)*power2(mh2) - msb22*power3(mh2) + (
     msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(DeltaInv(mh2,msb22,msb22)
     )*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(
     msb22))))/msb12 - (2*log(mH2/Q2)*log(msb22/Q2)*power2(snb)*((mH2*msb22 +
     msb12*msb22 + msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     power2(mH2)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) + (
     msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,msb22)
     )*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(
     msb22))))/msb12 + (2*log(mH2/Q2)*log(msb22/Q2)*power2(sa)*power2(snb)*((
     mH2*msb22 + msb12*msb22 + msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + power2(mH2)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(
     mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22))
     )*power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,
     msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*
     power4(msb22))))/msb12 + (power2(sa)*(mh2*msb12*DeltaInv(mh2,msb12,msb22)
     - 42*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(
     mh2,msb12,msb22)) + 42*power2(mh2)*(-((5*msb12*msb22 + 3*msb24)*power2(
     mh2)) + 3*msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - mh2*(
     14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) +
     mH2*DeltaInv(mH2,msb12,msb22)*(-msb12 + 42*mH2*(3*mH2*msb22 + msb12*msb22
     + 3*msb24)*DeltaInv(mH2,msb12,msb22) - 42*mH2*power2(DeltaInv(mH2,msb12,
     msb22))*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(mH2) - 3*msb24*power2(
     mH2) + 3*msb22*power3(mH2) - 3*mH2*power3(msb22) - 5*msb12*power3(msb22) +
     3*power4(msb22)))))/(12.*msb12) + (log(msb12/Q2)*log(msb22/Q2)*power2(cb)*
     ((mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*power2(DeltaInv(msb12,
     mA2,msb22)) + power2(mA2)*((msb12*msb22 - msb24)*power2(mA2) - msb22*
     power3(mA2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mA2*(2*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + power2(mz2)*power2(
     DeltaInv(msb12,mz2,msb22))*(-(msb12*msb22) - 3*msb24 - msb22*mz2 +
     DeltaInv(msb12,mz2,msb22)*(msb24*power2(mz2) - 5*mz2*power3(msb22) -
     msb12*(10*msb24*mz2 + msb22*power2(mz2) + 5*power3(msb22)) + msb22*power3(
     mz2) + 3*power4(msb22)))))/(2.*msb12) + (power2(cb)*power2(Pi)*(msb22*(3*
     mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*power2(DeltaInv(msb12,mA2,
     msb22)) - msb22*(msb12*msb22 + 3*(msb24 + msb22*mz2))*power2(mz2)*power2(
     DeltaInv(msb12,mz2,msb22)) + msb22*power2(mA2)*((5*msb12*msb22 + 3*msb24)*
     power2(mA2) - 3*msb22*power3(mA2) + (5*msb12 - 3*msb22)*power3(msb22) +
     mA2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22))
     - msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(14*msb12*msb24*mz2
     + 5*msb12*msb22*power2(mz2) + 3*msb24*power2(mz2) + 5*msb12*power3(msb22)
     + 3*mz2*power3(msb22) - 3*msb22*power3(mz2) - 3*power4(msb22)) - 2*msb12*(
     -((3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*
     mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))))))/(12.*msb12*msb22) + (power2(cb)*(
     -(mA2*msb12*msb22*DeltaInv(msb12,mA2,msb22)) + msb12*msb22*mz2*DeltaInv(
     msb12,mz2,msb22) + 42*msb22*(3*mA2*msb22 + msb12*msb22 + 3*msb24)*power2(
     mA2)*power2(DeltaInv(msb12,mA2,msb22)) - 42*msb22*(msb12*msb22 + 3*(msb24
     + msb22*mz2))*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22)) - 42*msb22*
     power2(mA2)*(-((5*msb12*msb22 + 3*msb24)*power2(mA2)) + 3*msb22*power3(
     mA2) + (-5*msb12 + 3*msb22)*power3(msb22) - mA2*(14*msb12*msb24 + 3*
     power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) - 42*msb22*power2(mz2)*
     power3(DeltaInv(msb12,mz2,msb22))*(14*msb12*msb24*mz2 + 5*msb12*msb22*
     power2(mz2) + 3*msb24*power2(mz2) + 5*msb12*power3(msb22) + 3*mz2*power3(
     msb22) - 3*msb22*power3(mz2) - 3*power4(msb22)) + 2*msb12*(-(mC2*msb22*
     DeltaInv(msb22,mst12,mC2)) + 42*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*power2(mC2)*(-((5*
     msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + mw2*DeltaInv(mw2,msb22,mst12)*(msb22 - 42*
     mw2*(msb22*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb22,mst12) - 42*
     mw2*power2(DeltaInv(mw2,msb22,mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*
     power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(
     mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))))))/(12.*msb12*msb22) + (
     power2(cb)*power2(log(msb22/Q2))*(msb22*msb24*power2(mA2)*power2(DeltaInv(
     msb12,mA2,msb22)) - msb22*msb24*power2(mz2)*power2(DeltaInv(msb12,mz2,
     msb22)) - msb22*power2(mA2)*(msb24*power2(mA2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mA2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     msb12,mA2,msb22)) + msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(
     msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*msb12*(msb24*mz2 + power3(
     msb22)) + power4(msb22)) + 2*msb12*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((3*msb22*
     mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*
     power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(
     msb22*mst12) - 2*(mst14 + mst12*mw2) - DeltaInv(mw2,msb22,mst12)*(10*
     msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))))))/(2.*msb12*msb22) - (power2(Pi)*(msb22*(3*mH2*msb22 +
     msb12*msb22 + 3*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     msb22*(3*mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*power2(DeltaInv(
     msb12,mA2,msb22)) + msb22*power2(mH2)*((5*msb12*msb22 + 3*msb24)*power2(
     mH2) - 3*msb22*power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + mH2*(14*
     msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + msb22*
     power2(mA2)*((5*msb12*msb22 + 3*msb24)*power2(mA2) - 3*msb22*power3(mA2) +
     (5*msb12 - 3*msb22)*power3(msb22) + mA2*(14*msb12*msb24 + 3*power3(msb22))
     )*power3(DeltaInv(msb12,mA2,msb22)) + 2*msb12*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*(3*mC2*mst12 + msb22*mst12 + 3*mst14 + DeltaInv(msb22,
     mst12,mC2)*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2) + 3*mst14*
     power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*msb22*power3(
     mst12) - 3*power4(mst12)))))/(12.*msb12*msb22) + (power2(cb)*power2(Pi)*
     power2(snb)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb22*power2(mC2)*(
     (5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 -
     3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(-((5*msb22*mst12 + 3*
     mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(
     mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mw2)*(-(msb22*(msb12*mst12 + 3*(mst14 + mst12*mw2))*
     power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*
     msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*
     power4(mst12))) - msb22*power3(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*
     mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(
     mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12)))))/(
     6.*msb12*msb22) + (power2(cb)*power2(snb)*(-(mC2*msb12*msb22*DeltaInv(
     msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) + 42*msb22*(
     3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - 42*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - 42*msb22*power2(mC2)*(-((5*msb12*mst12
     + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*
     power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 42*msb12*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) -
     mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     + mw2*(msb12*msb22*DeltaInv(mw2,msb12,mst12) - 42*msb22*mw2*(msb12*mst12 +
     3*(mst14 + mst12*mw2))*power2(DeltaInv(mw2,msb12,mst12)) + msb12*DeltaInv(
     mw2,msb22,mst12)*(-msb22 + 42*mw2*(msb22*mst12 + 3*mst14 + 3*mst12*mw2)*
     DeltaInv(mw2,msb22,mst12) + 42*mw2*power2(DeltaInv(mw2,msb22,mst12))*(14*
     msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*
     msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*
     power4(mst12))) - 42*msb22*mw2*power3(DeltaInv(mw2,msb12,mst12))*(14*
     msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*
     msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*
     power4(mst12)))))/(6.*msb12*msb22) + (log(msb12/Q2)*log(mst12/Q2)*power2(
     cb)*power2(snb)*power2(snt)*(-((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(
     mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*((-(msb12*mst12) +
     mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(
     mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 +
     msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(mst12) + 5*
     mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/msb12 + (log(
     msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*(-((mC2*mst12 + msb22*mst12
     + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((
     -(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     *(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*
     mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(
     mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/
     msb22 + (log(msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(snt)*(-((mC2*mst12
     + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-
     5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*
     msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(
     mst12)))))/msb22 + (power2(cb)*power2(snb)*power2(log(msb12/Q2))*((2*mC2*
     mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2)) + power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*mst12*
     power3(mC2) + (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 2*(mst14 + mst12*mw2) -
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12)))))/msb12 + (power2(cb)*power2(snb)*
     power2(log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)
     *power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*
     msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/msb22 + (
     log(msb22/Q2)*power2(cb)*(-(msb22*(5*mA2*msb22 - msb12*msb22 + msb24)*
     mst12*DeltaInv(msb12,mA2,msb22)) + msb22*mst12*(-(msb12*msb22) + msb24 +
     5*msb22*mz2)*DeltaInv(msb12,mz2,msb22) - 6*msb22*mst12*mz2*power2(
     DeltaInv(msb12,mz2,msb22))*(msb22*mz2*(-3*msb12 + mz2) - msb24*(msb12 +
     14*mz2) + power3(msb22)) + 6*mA2*msb22*mst12*power2(DeltaInv(msb12,mA2,
     msb22))*(-3*mA2*msb12*msb22 - 14*mA2*msb24 - msb12*msb24 + msb22*power2(
     mA2) + power3(msb22)) + 72*msb22*mst12*power2(mA2)*(msb24*power2(mA2) + (-
     2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(msb12,mA2,msb22)) - 72*msb22*mst12*power2(mz2)*power3(
     DeltaInv(msb12,mz2,msb22))*(msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*
     msb12*(msb24*mz2 + power3(msb22)) + power4(msb22)) + 2*msb12*(-mC2 + mw2 +
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - 6*mC2*mst12*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mw2*
     power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*
     mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) + 72*
     mst12*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*power2(mw2)*
     power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/(24.*msb12*msb22*mst12)
     + (power2(cb)*power2(snb)*power2(snt)*power2(log(mst12/Q2))*(-(msb22*
     mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst14*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb22*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - msb12*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(msb22*
     mst14*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*
     power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) +
     msb22*power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(mw2)) + 2*mw2*
     power3(mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) - power4(mst12)))))/(
     msb12*msb22) + (power2(cb)*power2(snt)*power2(log(mst12/Q2))*(-(mst14*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/msb22 + (log(msb12/Q2)*power2(cb)*power2(snb)*(-mC2 +
     mw2 + DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - 6*mC2*mst12*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*
     mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) + 72*
     mst12*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))))/(12.*msb12*mst12) + (
     log(msb22/Q2)*power2(cb)*power2(snb)*(mC2 - mw2 - DeltaInv(msb22,mst12,
     mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*
     mC2*power2(mst12) + power3(mC2)) + 6*mC2*mst12*power2(DeltaInv(msb22,
     mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + DeltaInv(mw2,msb22,
     mst12)*(5*mw2*power2(mst12) - msb22*(power2(mst12) + power2(mw2)) + mst12*
     (mst14 + 5*power2(mw2)) + power3(mw2)) - 6*mst12*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14
     + power2(mw2)) + power3(mst12) + power3(mw2)) - 72*mst12*power2(mC2)*(-((
     3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))))/(12.*msb22*mst12) - (log(msb12/Q2)*log(
     mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,
     mst12))*(msb12*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*
     mst14*power2(mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*
     power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))))/
     msb12 - (log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*
     mst12*power2(mw2) + power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12)))
     )/msb22 + (log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))))/msb22 + (log(mst12/Q2)*log(mw2/Q2)*power2(
     cb)*power2(mw2)*power2(snb)*(msb22*(msb12*mst12 + mst14 + mst12*mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12)
     )*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*
     power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) +
     power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) + msb12*
     (6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(mw2) -
     power4(mst12))))/(msb12*msb22) - (log(mst12/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*power2(snb)*power2(snt)*(msb22*(msb12*mst12 + mst14 + mst12*
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     mst14*power2(mw2) + mw2*power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(
     mw2) + power3(mst12)) - mst12*power3(mw2) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) + mw2*power3(mst12) +
     msb12*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*power3(
     mw2) - power4(mst12))))/(msb12*msb22) + (log(mst12/Q2)*log(mw2/Q2)*power2(
     cb)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*power3(
     mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) - mst12*
     power3(mw2) - power4(mst12))))/msb22 - (log(mst12/Q2)*log(mw2/Q2)*power2(
     cb)*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12
     + mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) + mw2*
     power3(mst12) + msb22*(6*mst14*mw2 + mst12*power2(mw2) + power3(mst12)) -
     mst12*power3(mw2) - power4(mst12))))/msb22 + (power2(cb)*power2(snb)*
     power2(log(mst12/Q2))*(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) -
     2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(-(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12))) + msb12*
     power2(DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(
     mst14*power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(
     mst12)) - power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(
     mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb12*(mst14*mw2 + power3(
     mst12)) + power4(mst12)))))/(msb12*msb22) + (power2(cb)*power2(log(mst12/
     Q2))*(mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(-mst14 + DeltaInv(mw2,msb22,mst12)*(mst14*
     power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) +
     power4(mst12)))))/msb22 + (log(mst12/Q2)*power2(cb)*power2(snt)*((5*mC2*
     mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - 6*mC2*power2(
     DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*
     mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - DeltaInv(mw2,msb22,mst12)*(-(
     msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,mst12)*(
     mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) + power3(mst12)) - 72*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(
     12.*msb22) + (log(mst12/Q2)*power2(cb)*(-((5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2)) + 6*mC2*power2(DeltaInv(msb22,mst12,mC2)
     )*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) + 72*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*
     mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) -
     mst14*(msb22 + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(
     mst14*mw2 + power3(mst12)) + power4(mst12)))))/(12.*msb22) + (log(msb12/
     Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*((mC2*mst12 + msb12*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((
     msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)
     *power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(
     msb12*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*
     power2(mw2) - 5*mw2*power3(mst12) - msb12*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/msb12 +
     (log(msb22/Q2)*log(mst12/Q2)*power2(cb)*((mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((
     msb22*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)
     *power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(
     msb22*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*
     power2(mw2) - 5*mw2*power3(mst12) - msb22*(10*mst14*mw2 + mst12*power2(
     mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)))))/msb22 +
     (log(msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(snb)*power2(snt)*((mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2)) + power2(mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*power3(mC2)
     + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)
     ))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12))*(-(msb22*mst12) - 3*mst14 - mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) - msb22*(10*mst14*mw2 +
     mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(mw2) + 3*power4(mst12)
     ))))/msb22 - (power2(mC2)*power2(Pi)*power2(snt)*(-((3*mC2*mst12 + msb22*
     mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2))) + (-((5*msb22*mst12 +
     3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(
     mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22
     + 3*mst24 + DeltaInv(mst22,msb22,mC2)*(14*mC2*msb22*mst24 + 5*msb22*mst22*
     power2(mC2) + 3*mst24*power2(mC2) - 3*mst22*power3(mC2) + 3*mC2*power3(
     mst22) + 5*msb22*power3(mst22) - 3*power4(mst22)))))/(6.*msb22) + (power2(
     cb)*power2(Pi)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + (3*mC2*mst22 + msb22*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(
     mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-
     5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))
     *power3(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((5*msb22*mst22 + 3*
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(
     mst22) + mC2*(14*msb22*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2)) + power2(mw2)*(-((msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(
     DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*
     mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(
     mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) -
     power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(6.*msb22) + (power2(
     cb)*power2(snt)*(mC2*msb22*DeltaInv(msb22,mst12,mC2) - mC2*msb22*DeltaInv(
     mst22,msb22,mC2) - 42*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) + 42*(3*mC2*mst22 + msb22*mst22 + 3*
     mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*power2(mC2)*(-((
     5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 +
     3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 42*power2(mC2)*(-((5*msb22*mst22 + 3*mst24)*
     power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) -
     mC2*(14*msb22*mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))
     + mw2*(msb22*DeltaInv(mst22,mw2,msb22) - 42*mw2*(msb22*mst22 + 3*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) + DeltaInv(mw2,msb22,mst12)*
     (-msb22 + 42*mw2*(msb22*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb22,
     mst12) + 42*mw2*power2(DeltaInv(mw2,msb22,mst12))*(14*msb22*mst14*mw2 + 5*
     msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*
     mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) - 42*mw2*
     power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*
     power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(
     mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(6.*msb22) + (log(
     msb22/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*power2(snt)*(-((mC2*mst22 +
     msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))) +
     power2(mC2)*((-(msb22*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-
     5*msb22 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(
     10*msb22*mst24*mw2 + msb22*mst22*power2(mw2) - mst24*power2(mw2) + 5*
     msb22*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(
     mst22)))))/msb22 + (power2(cb)*power2(snb)*power2(snt)*power2(log(msb12/
     Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*(-((3*msb12*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mC2)*((3*msb12*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*(-
     ((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb12)))
     + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb12))*
     (10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb12 + (power2(cb)*power2(snt)*power2(log(msb22/Q2))*(-
     ((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) +
     (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*mst24 + power3(mst22)))
     *power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-((msb22*mst22 + 2*(
     mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22))) + power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,
     msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*(10*
     msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (power2(cb)*power2(snb)*power2(snt)*power2(log(
     msb22/Q2))*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((3*msb22*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*((
     msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) -
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(
     mw2)*power2(snb)*power2(snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(
     mw2) - mw2*power3(mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) +
     power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb22))*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) -
     power4(mst22))))/msb22 + (log(mst22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)
     *power2(snb)*power2(snt)*(msb22*(msb12*mst22 + mst24 + mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(mst22,mw2,msb22))*(
     msb22*mst22 + mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(
     mw2) + mw2*power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) +
     power3(mst22)) - mst22*power3(mw2) - power4(mst22))) + msb22*power3(
     DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) + mw2*power3(mst22) + msb12*
     (6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) - mst22*power3(mw2) -
     power4(mst22))))/(msb12*msb22) + (log(mst22/Q2)*log(mw2/Q2)*power2(cb)*
     power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(msb22*mst22 +
     mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) + mw2*
     power3(mst22) + msb22*(6*mst24*mw2 + mst22*power2(mw2) + power3(mst22)) -
     mst22*power3(mw2) - power4(mst22))))/msb22 + (power2(mC2)*power2(Pi)*
     power2(snb)*power2(snt)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 3*mC2*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) - 3*msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + 3*mC2*
     msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(DeltaInv(mst22,msb22,
     mC2)) + msb22*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2)
     + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*mC2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 3*mC2*msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst22*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 3*mC2*msb12*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) + 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(6.*msb12*msb22) +
     (mC2*power2(snb)*power2(snt)*(-(msb12*msb22*DeltaInv(msb12,mst12,mC2)) +
     msb12*msb22*DeltaInv(msb22,mst12,mC2) + msb12*msb22*DeltaInv(mst22,msb12,
     mC2) - msb12*msb22*DeltaInv(mst22,msb22,mC2) + 42*mC2*msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2)) - 42*mC2*msb12*(
     3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) -
     42*mC2*msb12*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - 126*mC2*
     msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst22*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*mC2*msb12*msb22*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 126*mC2*msb12*mst24*power2(DeltaInv(mst22,
     msb22,mC2)) + 126*msb12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)
     ) - 42*mC2*msb22*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*
     mst22*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*mC2*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*mst22*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*mC2*msb12*msb22*power3(mst22)
     *power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power4(mC2) + 42*mC2*msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) + 126*mC2*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     126*mC2*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(6.*msb12*
     msb22) + (log(msb12/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snb)*
     power2(snt)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb12))*(-
     5*mst24*power2(mw2) + mw2*power3(mst22) - msb12*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb12 + (log(msb22/Q2)*log(mw2/Q2)*power2(cb)*power2(mw2)*power2(snt)*(-((
     msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,mw2,msb22))) +
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(mst12) +
     msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*mst12*
     power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(-5*
     mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb22 + (power2(snt)*(msb22*(-mst12 + mst22) - 2*mC2*msb22*mst12*mst22*
     DeltaInv(msb22,mst12,mC2) + 2*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) + 84*mst12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mst22*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - 84*mst12*mst22*(3*mC2*mst22 + msb22*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 84*mst12*
     mst22*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 84*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-14*mC2*msb22*mst24 - 5*
     msb22*mst22*power2(mC2) - 3*mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*
     mC2*power3(mst22) - 5*msb22*power3(mst22) + 3*power4(mst22))))/(12.*msb22*
     mst12*mst22) + (power2(cb)*power2(snb)*power2(snt)*power2(log(mst22/Q2))*(
     msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb22*power2(mC2)*(mst24*
     power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + msb12*power2(mC2)*(
     mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-(
     msb22*mst24*power2(DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(
     mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2))
     + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(
     mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*
     mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22))))
     )/(msb12*msb22) + (power2(cb)*power2(snt)*power2(log(mst22/Q2))*(mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22))*(-mst24 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) -
     2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(mst22))
     )))/msb22 + (log(mst22/Q2)*power2(cb)*power2(snt)*(-((5*mC2*mst22 - msb22*
     mst22 + mst24)*DeltaInv(mst22,msb22,mC2)) + 6*mC2*power2(DeltaInv(mst22,
     msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) + 72*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + DeltaInv(mst22,mw2,msb22)*(-(msb22*
     mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb22)*(mst22*mw2*
     (-3*msb22 + mw2) - mst24*(msb22 + 14*mw2) + power3(mst22)) - 72*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(12.*
     msb22) + (log(msb12/Q2)*log(mst22/Q2)*power2(cb)*power2(snb)*power2(snt)*(
     (mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + power2(mC2)*((msb12*mst22 - mst24)*power2(mC2) - mst22*
     power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb12))*(-(msb12*mst22) - 3*mst24 - mst22*mw2 +
     DeltaInv(mst22,mw2,msb12)*(mst24*power2(mw2) - 5*mw2*power3(mst22) -
     msb12*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(
     mw2) + 3*power4(mst22)))))/msb12 + (log(msb22/Q2)*log(mst22/Q2)*power2(cb)
     *power2(snt)*((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((msb22*mst22 - mst24)*power2(
     mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*
     msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22))*(-(msb22*mst22) - 3*mst24 - mst22*
     mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) - 5*mw2*power3(mst22) -
     msb22*(10*mst24*mw2 + mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(
     mw2) + 3*power4(mst22)))))/msb22 + (log(mst22/Q2)*power2(cb)*power2(snb)*
     power2(snt)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,
     msb12,mC2)) + msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,
     msb22,mC2) - msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) + msb22*mst24*
     DeltaInv(mst22,mw2,msb12) + 5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) +
     msb12*msb22*mst22*DeltaInv(mst22,mw2,msb22) - msb12*mst24*DeltaInv(mst22,
     mw2,msb22) - 5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) + 6*msb12*msb22*
     mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 18*msb12*msb22*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 84*msb22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*mst24*mw2*power2(
     DeltaInv(mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))*power3(
     mst22) + 6*msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(mst22) + 6*
     mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*
     mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*mC2*msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 -
     msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(12.*msb12*msb22) + (log(mst12/
     Q2)*power2(cb)*power2(snb)*power2(snt)*(msb22*(5*mC2*mst12 - msb12*mst12 +
     mst14)*DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*DeltaInv(mw2,msb12,
     mst12) - msb22*mst14*DeltaInv(mw2,msb12,mst12) - 5*msb22*mst12*mw2*
     DeltaInv(mw2,msb12,mst12) - msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) +
     msb12*mst14*DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*mw2*DeltaInv(mw2,
     msb22,mst12) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) -
     18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 84*msb12*mst14*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 6*msb22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) - 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) + 6*mC2*msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) +
     6*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 6*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*msb22*power2(mC2)
     *(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*msb12*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 72*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb12*
     mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(12.*msb12*msb22) +
     (log(mst12/Q2)*power2(cb)*power2(snb)*(-(msb22*(5*mC2*mst12 - msb12*mst12
     + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*DeltaInv(mw2,msb12,
     mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) -
     msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mw2*DeltaInv(mw2,
     msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) +
     18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst14*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) + 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) -
     6*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*msb22*power2(mC2)
     *(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) -
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*
     mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(12.*msb12*msb22) -
     (2*log(msb12/Q2)*log(msb22/Q2)*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22))*(mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((
     msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)
     *power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22))))*power4(snb))/
     msb12 - (mH2*power2(log(msb12/Q2))*(msb12*msb14*power2(DeltaInv(mH2,msb12,
     msb12)) + 2*mH2*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*msb22 + msb12*
     msb22 + 2*msb24 + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22 + 2*msb24)*
     power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*
     mH2*(5*msb12*msb24 + power3(msb22)))))*power4(snb))/msb12 + (2*log(msb12/
     Q2)*log(msb22/Q2)*power2(sa)*(-((mh2*msb22 + msb12*msb22 + 3*msb24)*
     power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(
     mH2,msb12,msb22)*((msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) +
     (5*msb12 - 3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))
     ) + power2(mh2)*((-(msb12*msb22) + msb24)*power2(mh2) + msb22*power3(mh2)
     + (-5*msb12 + 3*msb22)*power3(msb22) - 5*mh2*(2*msb12*msb24 + power3(
     msb22)))*power3(DeltaInv(mh2,msb12,msb22)))*power4(snb))/msb12 + (power2(
     sa)*power2(log(msb12/Q2))*(-(mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,
     msb12))) - 2*(2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + mH2*(msb12*msb14*power2(DeltaInv(mH2,msb12,
     msb12)) + 2*mH2*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*msb22 + msb12*
     msb22 + 2*msb24 + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22 + 2*msb24)*
     power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*
     mH2*(5*msb12*msb24 + power3(msb22))))) + 2*power2(mh2)*(-((3*msb12*msb22 +
     2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(
     msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22)))*power4(snb))/msb12 - (mH2*power2(log(msb22/Q2))*(2*mH2*msb24*
     power2(DeltaInv(mH2,msb12,msb22)) + msb12*msb24*power2(DeltaInv(mH2,msb22,
     msb22)) - 2*mH2*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*
     mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)))*
     power4(snb))/msb12 + (power2(sa)*power2(log(msb22/Q2))*(-2*msb24*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22)) - mh2*msb12*msb24*power2(DeltaInv(
     mh2,msb22,msb22)) + 2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22)) + mH2*(2*mH2*msb24*power2(DeltaInv(mH2,msb12,msb22)) +
     msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) - 2*mH2*(msb24*power2(mH2) +
     (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22))))*power4(snb))/msb12 - (log(msb22/Q2)*
     power2(sa)*(-((5*mh2*msb22 - msb12*msb22 + msb24)*DeltaInv(mh2,msb12,
     msb22)) - msb12*(mh2 - 6*msb22)*DeltaInv(mh2,msb22,msb22) + 5*mH2*msb22*
     DeltaInv(mH2,msb12,msb22) - msb12*msb22*DeltaInv(mH2,msb12,msb22) + msb24*
     DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22) - 6*msb12*
     msb22*DeltaInv(mH2,msb22,msb22) + 6*mH2*msb12*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) + 18*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 84*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*
     msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(
     mH2,msb12,msb22))*power3(mH2) - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power3(msb22) - 72*msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) +
     6*mh2*power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24
     - msb12*msb24 + msb22*power2(mh2) + power3(msb22)) + 24*msb12*power2(
     DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*
     (msb24*power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*
     power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(
     msb22)*power3(DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,
     msb12,msb22))*power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22)
     )*power4(msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22))*
     power4(snb))/(6.*msb12) + (2*log(mh2/Q2)*log(msb22/Q2)*power2(sa)*((mh2*
     msb22 + msb12*msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))
     + power2(mh2)*((msb12*msb22 + msb24)*power2(mh2) - msb22*power3(mh2) + (
     msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(DeltaInv(mh2,msb22,msb22)
     )*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(
     msb22)))*power4(snb))/msb12 + (2*log(mH2/Q2)*log(msb22/Q2)*((mH2*msb22 +
     msb12*msb22 + msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     power2(mH2)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) + (
     msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,msb22)
     )*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(
     msb22)))*power4(snb))/msb12 - (2*log(mH2/Q2)*log(msb22/Q2)*power2(sa)*((
     mH2*msb22 + msb12*msb22 + msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + power2(mH2)*((msb12*msb22 + msb24)*power2(mH2) - msb22*power3(
     mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22))
     )*power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,
     msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*
     power4(msb22)))*power4(snb))/msb12 - (log(msb22/Q2)*(-((5*mH2*msb22 -
     msb12*msb22 + msb24)*DeltaInv(mH2,msb12,msb22)) + 6*mH2*power2(DeltaInv(
     mH2,msb12,msb22))*(-3*mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 +
     msb22*power2(mH2) + power3(msb22)) + 72*power2(mH2)*(msb24*power2(mH2) + (
     -2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)) + msb12*DeltaInv(mH2,msb22,msb22)*(-mH2
     + 6*msb22 + 24*DeltaInv(mH2,msb22,msb22)*(mH2*msb24 + 3*power3(msb22)) +
     288*mH2*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22)))*power4(snb))/(6.
     *msb12) - (2*log(mh2/Q2)*log(msb12/Q2)*power2(sa)*power2(snb)*(power2(mh2)
     *power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 +
     DeltaInv(mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*
     power3(mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 +
     power3(msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*
     power4(msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/
     msb12 + (2*log(mh2/Q2)*log(msb12/Q2)*power2(sa)*power4(snb)*(power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 +
     DeltaInv(mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*
     power3(mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 +
     power3(msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*
     power4(msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/
     msb12 - (power2(sa)*power2(snb)*power2(log(mh2/Q2))*(-2*msb22*power2(
     DeltaInv(mh2,msb12,msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 -
     2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(
     DeltaInv(mh2,msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*
     msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*
     msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22))
     + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 + (
     power2(sa)*power2(log(mh2/Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(mh2,
     msb12,msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*
     msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,
     msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*
     power4(msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*
     power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 - (power2(sa)*
     power2(snb)*(-(mh2*msb12*DeltaInv(mh2,msb12,msb12)) + 2*mh2*msb12*
     DeltaInv(mh2,msb12,msb22) - mh2*msb12*DeltaInv(mh2,msb22,msb22) + mH2*
     msb12*DeltaInv(mH2,msb12,msb12) - 2*mH2*msb12*DeltaInv(mH2,msb12,msb22) +
     mH2*msb12*DeltaInv(mH2,msb22,msb22) - 84*(3*mh2*msb22 + msb12*msb22 + 3*
     msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 42*mh2*msb12*msb24*
     power2(DeltaInv(mh2,msb22,msb22)) - 42*mH2*msb12*msb14*power2(DeltaInv(
     mH2,msb12,msb12)) + 84*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 252*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - 42*
     mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 252*msb22*power2(
     DeltaInv(mH2,msb12,msb22))*power3(mH2) + 252*msb12*power2(DeltaInv(mh2,
     msb22,msb22))*power3(msb22) - 252*msb12*power2(DeltaInv(mH2,msb22,msb22))*
     power3(msb22) + 1176*msb12*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,
     msb22)) + 420*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 252*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22))
     + 420*msb12*msb22*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 252*
     msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) - 252*power2(DeltaInv(
     mH2,msb12,msb12))*power4(msb12) + 42*power2(DeltaInv(mh2,msb12,msb12))*(
     mh2*msb12*msb14 + 6*power4(msb12)) + 1008*mh2*msb12*power3(DeltaInv(mh2,
     msb22,msb22))*power4(msb22) - 252*power2(mH2)*power3(DeltaInv(mH2,msb12,
     msb22))*power4(msb22) - 1008*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*
     power4(msb22) + 84*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*
     msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*
     power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(
     msb22)) - 252*msb22*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) + 1008*
     mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 1008*mH2*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*msb12) + (power2(sa)*
     power4(snb)*(-(mh2*msb12*DeltaInv(mh2,msb12,msb12)) + 2*mh2*msb12*
     DeltaInv(mh2,msb12,msb22) - mh2*msb12*DeltaInv(mh2,msb22,msb22) + mH2*
     msb12*DeltaInv(mH2,msb12,msb12) - 2*mH2*msb12*DeltaInv(mH2,msb12,msb22) +
     mH2*msb12*DeltaInv(mH2,msb22,msb22) - 84*(3*mh2*msb22 + msb12*msb22 + 3*
     msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 42*mh2*msb12*msb24*
     power2(DeltaInv(mh2,msb22,msb22)) - 42*mH2*msb12*msb14*power2(DeltaInv(
     mH2,msb12,msb12)) + 84*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 252*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - 42*
     mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 252*msb22*power2(
     DeltaInv(mH2,msb12,msb22))*power3(mH2) + 252*msb12*power2(DeltaInv(mh2,
     msb22,msb22))*power3(msb22) - 252*msb12*power2(DeltaInv(mH2,msb22,msb22))*
     power3(msb22) + 1176*msb12*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,
     msb22)) + 420*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 252*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22))
     + 420*msb12*msb22*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 252*
     msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) - 252*power2(DeltaInv(
     mH2,msb12,msb12))*power4(msb12) + 42*power2(DeltaInv(mh2,msb12,msb12))*(
     mh2*msb12*msb14 + 6*power4(msb12)) + 1008*mh2*msb12*power3(DeltaInv(mh2,
     msb22,msb22))*power4(msb22) - 252*power2(mH2)*power3(DeltaInv(mH2,msb12,
     msb22))*power4(msb22) - 1008*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*
     power4(msb22) + 84*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*
     msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*
     power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(
     msb22)) - 252*msb22*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) + 1008*
     mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 1008*mH2*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*msb12) - (power2(Pi)*
     power2(sa)*power2(snb)*(-2*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(
     mh2,msb22,msb22)) - mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*
     msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 6*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - mH2*msb12*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*
     power3(mH2) + 6*msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) + 28*msb12*msb24*
     power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*power2(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 6*power3(mH2)*power3(
     msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*msb22*power3(DeltaInv(
     mH2,msb12,msb22))*power4(mH2) + 6*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) - 6*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 24*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 6*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 24*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 6*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 24*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*
     msb12) + (power2(Pi)*power2(sa)*power4(snb)*(-2*(3*mh2*msb22 + msb12*msb22
     + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*msb12*
     msb24*power2(DeltaInv(mh2,msb22,msb22)) - mH2*msb12*msb14*power2(DeltaInv(
     mH2,msb12,msb12)) + 2*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 6*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - mH2*
     msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 6*msb22*power2(DeltaInv(
     mH2,msb12,msb22))*power3(mH2) + 6*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     power3(msb22) - 6*msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) +
     28*msb12*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*
     power2(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 6*power3(
     mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*msb22*
     power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 6*msb24*power3(DeltaInv(
     mH2,msb12,msb22))*power4(mH2) - 6*power2(DeltaInv(mH2,msb12,msb12))*
     power4(msb12) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*
     power4(msb12)) + 24*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 6*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     24*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*power2(
     mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*
     msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*
     power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 6*msb22*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,msb12,
     msb12))*power5(msb12) - 24*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(
     msb12)))/(6.*msb12) - (2*log(mH2/Q2)*log(msb12/Q2)*power2(snb)*(power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24
     + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*
     msb22*power3(mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24
     + power3(msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 +
     4*power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12))
     )/msb12 + (2*log(mH2/Q2)*log(msb12/Q2)*power2(sa)*power2(snb)*(power2(mH2)
     *power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 +
     DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*
     power3(mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 +
     power3(msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*
     power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/
     msb12 + (2*log(mH2/Q2)*log(msb12/Q2)*power4(snb)*(power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(
     mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*power3(
     mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 + power3(
     msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(
     msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/msb12 -
     (2*log(mH2/Q2)*log(msb12/Q2)*power2(sa)*power4(snb)*(power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(
     mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*power3(
     mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 + power3(
     msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(
     msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/msb12 -
     (power2(snb)*power2(log(mH2/Q2))*(-2*msb22*power2(DeltaInv(mH2,msb12,
     msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,
     msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(
     msb12)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(
     msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)) + 16*mH2*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12)))/msb12 + (power2(sa)*power2(snb)
     *power2(log(mH2/Q2))*(-2*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(
     mH2) + 2*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,msb22)) +
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) +
     msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*
     mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/msb12 + (power2(log(mH2/Q2))*power4(snb)*(-
     2*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(
     msb22))*power3(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,
     msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,
     msb22)*power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(
     msb12)))/msb12 - (power2(sa)*power2(log(mH2/Q2))*power4(snb)*(-2*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(
     mH2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,
     msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*
     power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/
     msb12 + (power2(Pi)*power4(snb)*(-2*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb12*msb14 + 6*power4(msb12)) + 2*power2(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(
     mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*power3(msb22) -
     5*msb12*power3(msb22) + 3*power4(msb22)) + msb12*power2(DeltaInv(mH2,
     msb22,msb22))*(mH2*msb24 + 6*power3(msb22) + 24*mH2*DeltaInv(mH2,msb22,
     msb22)*power4(msb22)) + 24*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(
     msb12)))/(6.*msb12) + (power4(snb)*(-(mH2*msb12*DeltaInv(mH2,msb12,msb12))
     + 2*mH2*msb12*DeltaInv(mH2,msb12,msb22) - 84*(3*mH2*msb22 + msb12*msb22 +
     3*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 84*power2(mH2)*(-
     ((5*msb12*msb22 + 3*msb24)*power2(mH2)) + 3*msb22*power3(mH2) + (-5*msb12
     + 3*msb22)*power3(msb22) - mH2*(14*msb12*msb24 + 3*power3(msb22)))*power3(
     DeltaInv(mH2,msb12,msb22)) + 42*power2(DeltaInv(mH2,msb12,msb12))*(mH2*
     msb12*msb14 + 6*power4(msb12)) + msb12*DeltaInv(mH2,msb22,msb22)*(-mH2 +
     42*DeltaInv(mH2,msb22,msb22)*(mH2*msb24 + 6*power3(msb22)) + 1008*mH2*
     power2(DeltaInv(mH2,msb22,msb22))*power4(msb22)) + 1008*mH2*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(6.*msb12) - (log(msb12/Q2)*
     power4(snb)*(-mH2 - msb12 - msb22 + msb12*(-mH2 + 6*msb12)*msb22*DeltaInv(
     mH2,msb12,msb12) + DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*msb22) +
     msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*power2(msb22) + power3(mH2)
     ) + 24*msb12*msb22*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 3*
     power3(msb12)) - 6*mH2*msb22*power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*
     msb24) + mH2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mH2) +
     power3(mH2) + power3(msb22)) + 72*msb22*power2(mH2)*(-((3*msb12*msb22 + 2*
     msb24)*power2(mH2)) + 2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(
     msb22) - 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,
     msb22)) + 288*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/
     (6.*msb12*msb22) + (log(mh2/Q2)*power2(sa)*power2(snb)*(mh2 + msb12 +
     msb22 + (mh2 - 6*msb12)*msb12*msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*
     msb22*DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,
     msb12,msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22)
     + 6*msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 +
     power2(mh2))*power2(DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*
     power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,
     msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(
     msb22))*power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,
     msb22,msb22))*power4(msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,
     msb12))*power5(msb12) + 576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*
     power5(msb22)))/(6.*msb12*msb22) - (log(mh2/Q2)*power2(sa)*power4(snb)*(
     mh2 + msb12 + msb22 + (mh2 - 6*msb12)*msb12*msb22*DeltaInv(mh2,msb12,
     msb12) + mh2*msb12*msb22*DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 + 5*
     msb22)*DeltaInv(mh2,msb12,msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,msb22,
     msb22)*power2(msb22) + 6*msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(msb12 +
     14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,msb22)) + 12*
     mh2*msb12*msb22*msb24*power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*
     msb22*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*
     power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,msb22)) + 144*
     msb12*power2(DeltaInv(mh2,msb22,msb22))*power4(msb22) + 576*mh2*msb22*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) + 576*mh2*msb12*power3(
     DeltaInv(mh2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22) + (log(msb12/
     Q2)*power2(sa)*power2(snb)*(-mh2 + mH2 + msb12*(-mh2 + 6*msb12)*msb22*
     DeltaInv(mh2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb12,msb12) -
     msb22*msb24*DeltaInv(mH2,msb12,msb22) + msb12*DeltaInv(mH2,msb12,msb22)*
     power2(mH2) - 5*msb22*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb22*
     DeltaInv(mH2,msb12,msb12)*power2(msb12) - 5*mH2*DeltaInv(mH2,msb12,msb22)*
     power2(msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(msb22) - 24*mH2*
     msb12*msb14*msb22*power2(DeltaInv(mH2,msb12,msb12)) - 6*mH2*msb12*msb22*
     msb24*power2(DeltaInv(mH2,msb12,msb22)) + 138*msb22*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + 36*msb12*power2(mH2)*power2(msb22)*
     power2(DeltaInv(mH2,msb12,msb22)) + DeltaInv(mh2,msb12,msb22)*(msb22*(-(
     msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(
     msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*power3(mH2) - 6*msb12*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 138*power2(msb22)*
     power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 24*msb12*msb22*power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 6*mh2*msb22*
     power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*msb12*msb22 +
     23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) + power3(msb22))
     + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*
     msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*msb12*
     msb22*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 6*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 144*msb22*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 216*msb12*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 72*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 216*msb12*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 144*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 144*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) +
     288*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 288*mH2*
     msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power5(msb22)))/(6.*msb12*msb22) - (log(
     msb12/Q2)*power2(sa)*power4(snb)*(-mh2 + mH2 + msb12*(-mh2 + 6*msb12)*
     msb22*DeltaInv(mh2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb12,
     msb12) - msb22*msb24*DeltaInv(mH2,msb12,msb22) + msb12*DeltaInv(mH2,msb12,
     msb22)*power2(mH2) - 5*msb22*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*
     msb22*DeltaInv(mH2,msb12,msb12)*power2(msb12) - 5*mH2*DeltaInv(mH2,msb12,
     msb22)*power2(msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(msb22) - 24*
     mH2*msb12*msb14*msb22*power2(DeltaInv(mH2,msb12,msb12)) - 6*mH2*msb12*
     msb22*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 138*msb22*msb24*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 36*msb12*power2(mH2)*power2(
     msb22)*power2(DeltaInv(mH2,msb12,msb22)) + DeltaInv(mh2,msb12,msb22)*(
     msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) + 5*mh2*
     power2(msb22) + power3(mh2)) - DeltaInv(mH2,msb12,msb22)*power3(mH2) - 6*
     msb12*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 138*power2(
     msb22)*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 24*msb12*msb22*
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 3*power3(msb12)) - 6*mh2*
     msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*msb12*
     msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) + power3(
     msb22)) + 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(mh2)) +
     2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 720*msb12*
     msb22*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 6*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 144*msb22*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 216*msb12*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 72*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*power4(msb12) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 216*msb12*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) + 144*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     msb22) - 144*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) +
     288*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) - 288*mH2*
     msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power5(msb22)))/(6.*msb12*msb22) - (
     power2(Pi)*power2(snb)*(-2*msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + mH2*msb12*msb22*msb24*
     power2(DeltaInv(mH2,msb22,msb22)) + msb12*msb22*mst12*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + 3*msb22*mst14*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2)) - msb12*msb22*mst12*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - 3*msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))
     + 3*msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))*power3(mC2) - 3*msb12*
     mst12*power2(DeltaInv(msb22,mst12,mC2))*power3(mC2) + msb12*msb22*power2(
     DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 6*power3(msb12)) + 14*msb12*msb22*
     mst14*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 5*msb12*msb22*
     power2(mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) + 3*msb22*
     power3(mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) - 14*msb12*
     msb22*mst14*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2)) - 5*msb12*msb22*
     power2(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,mC2)) - 3*msb12*
     power3(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,mC2)) + 5*msb12*
     msb22*mst12*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 3*msb22*mst14*
     power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) - 5*msb12*msb22*mst12*
     power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) - 3*msb12*mst14*power3(
     DeltaInv(msb22,mst12,mC2))*power4(mC2) + 6*msb12*power2(DeltaInv(mH2,
     msb22,msb22))*power4(msb22) + 2*msb22*power2(mH2)*power3(DeltaInv(mH2,
     msb12,msb22))*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(mH2) - 3*msb24*
     power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*power3(msb22) - 5*msb12*power3(
     msb22) + 3*power4(msb22)) - 3*msb22*power2(mC2)*power3(DeltaInv(msb12,
     mst12,mC2))*power4(mst12) + 3*msb12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*power4(mst12) - 3*msb22*mst12*power3(DeltaInv(msb12,mst12,mC2)
     )*power5(mC2) + 3*msb12*mst12*power3(DeltaInv(msb22,mst12,mC2))*power5(
     mC2) + 24*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 24*
     mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(6.*msb12*
     msb22) + (log(mH2/Q2)*power2(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*
     msb12*msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb22,msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(
     mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(
     mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*
     power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,
     msb22))*power4(msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12) + 576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(
     msb22)))/(6.*msb12*msb22) - (log(mH2/Q2)*power2(sa)*power2(snb)*(mH2 +
     msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*DeltaInv(mH2,msb12,msb12) +
     mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (mH2 - msb12 + 5*msb22)*
     DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*
     power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*
     msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*
     msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*
     msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*
     power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,msb22)) + 144*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*mH2*msb22*
     power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22) - (log(mH2/Q2)
     *power4(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*DeltaInv(
     mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (mH2 -
     msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*DeltaInv(
     mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*msb22 -
     mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,
     msb22)) + 12*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 12*
     msb12*msb22*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(
     msb12)) + 72*msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*
     mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*
     power3(DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(6.*msb12*msb22) + (log(
     mH2/Q2)*power2(sa)*power4(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*
     msb12*msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,
     msb22,msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(
     mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(
     mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*
     power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,
     msb22))*power4(msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12) + 576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(
     msb22)))/(6.*msb12*msb22) - (power2(snb)*(-msb12 + msb22 - 2*mH2*msb12*
     msb22*DeltaInv(mH2,msb12,msb12) + 4*mH2*msb12*msb22*DeltaInv(mH2,msb12,
     msb22) - 2*mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - 2*mC2*msb12*msb22*
     DeltaInv(msb12,mst12,mC2) + 2*mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) -
     168*msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 84*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,
     msb22,msb22)) + 84*msb12*msb22*mst12*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) + 252*msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)
     ) - 84*msb12*msb22*mst12*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     252*msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 252*msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2))*power3(mC2) - 252*msb12*mst12*
     power2(DeltaInv(msb22,mst12,mC2))*power3(mC2) + 84*msb12*msb22*power2(
     DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 6*power3(msb12)) + 1176*msb12*
     msb22*mst14*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 420*msb12*
     msb22*power2(mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) + 252*
     msb22*power3(mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) - 1176*
     msb12*msb22*mst14*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2)) - 420*
     msb12*msb22*power2(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,mC2)) -
     252*msb12*power3(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,mC2)) +
     420*msb12*msb22*mst12*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 252*
     msb22*mst14*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) - 420*msb12*
     msb22*mst12*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) - 252*msb12*
     mst14*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) + 504*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 168*msb22*power2(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(
     mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*power3(msb22) -
     5*msb12*power3(msb22) + 3*power4(msb22)) - 252*msb22*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*power4(mst12) + 252*msb12*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*power4(mst12) - 252*msb22*mst12*power3(
     DeltaInv(msb12,mst12,mC2))*power5(mC2) + 252*msb12*mst12*power3(DeltaInv(
     msb22,mst12,mC2))*power5(mC2) + 2016*mH2*msb22*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) + 2016*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*
     power5(msb22)))/(12.*msb12*msb22) + (log(msb22/Q2)*power2(snb)*(-2*msb22*(
     5*mH2*msb22 - msb12*msb22 + msb24)*mst12*DeltaInv(mH2,msb12,msb22) + 12*
     mH2*msb22*mst12*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb12*msb22 -
     14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) + power3(msb22)) + 144*
     msb22*mst12*power2(mH2)*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,
     msb22)) + msb12*(-mC2 - msb22 - mst12 - 2*(mH2 - 6*msb22)*msb22*mst12*
     DeltaInv(mH2,msb22,msb22) + DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*
     mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) +
     power3(mC2)) + 48*msb22*mst12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24
     + 3*power3(msb22)) - 6*mC2*mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(
     msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) + 72*mst12*power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + 576*mH2*mst12*power3(DeltaInv(mH2,msb22,msb22))*
     power5(msb22))))/(12.*msb12*msb22*mst12) + (log(msb12/Q2)*power2(snb)*(
     mC2*msb22 + msb12*msb22 - 2*mH2*mst12 - 2*msb12*mst12 - msb22*mst12 - 2*(
     mH2 - 6*msb12)*msb12*msb22*mst12*DeltaInv(mH2,msb12,msb12) - msb22*mst12*
     mst14*DeltaInv(msb12,mst12,mC2) + msb12*msb22*DeltaInv(msb12,mst12,mC2)*
     power2(mC2) - 5*msb22*mst12*DeltaInv(msb12,mst12,mC2)*power2(mC2) - 5*mC2*
     msb22*DeltaInv(msb12,mst12,mC2)*power2(mst12) + msb12*msb22*DeltaInv(
     msb12,mst12,mC2)*power2(mst12) - 6*mC2*msb12*msb22*mst12*mst14*power2(
     DeltaInv(msb12,mst12,mC2)) + 138*msb22*mst12*mst14*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + 36*msb12*msb22*power2(mC2)*power2(mst12)*
     power2(DeltaInv(msb12,mst12,mC2)) - msb22*DeltaInv(msb12,mst12,mC2)*
     power3(mC2) - 6*msb12*msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))*
     power3(mC2) + 138*msb22*power2(mst12)*power2(DeltaInv(msb12,mst12,mC2))*
     power3(mC2) + 2*mst12*DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*msb22) +
     msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*power2(msb22) + power3(mH2)
     ) + 48*msb12*msb22*mst12*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 3*
     power3(msb12)) - 12*mH2*msb22*mst12*power2(DeltaInv(mH2,msb12,msb22))*(-(
     msb12*msb24) + mH2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(
     mH2) + power3(mH2) + power3(msb22)) + 144*msb22*mst12*power2(mH2)*(-((3*
     msb12*msb22 + 2*msb24)*power2(mH2)) + 2*msb22*power3(mH2) + (-3*msb12 + 2*
     msb22)*power3(msb22) - 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mH2,msb12,msb22)) + 720*msb12*msb22*mst12*mst14*power3(mC2)*
     power3(DeltaInv(msb12,mst12,mC2)) + 6*msb22*mst12*power2(DeltaInv(msb12,
     mst12,mC2))*power4(mC2) + 144*msb22*mst12*mst14*power3(DeltaInv(msb12,
     mst12,mC2))*power4(mC2) + 216*msb12*msb22*power2(mst12)*power3(DeltaInv(
     msb12,mst12,mC2))*power4(mC2) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,
     mC2))*power4(mst12) + 216*msb12*msb22*power2(mC2)*power3(DeltaInv(msb12,
     mst12,mC2))*power4(mst12) + 144*msb22*power3(mC2)*power3(DeltaInv(msb12,
     mst12,mC2))*power4(mst12) - 144*msb22*power2(mst12)*power3(DeltaInv(msb12,
     mst12,mC2))*power5(mC2) + 576*mH2*msb22*mst12*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) - 144*msb22*power2(mC2)*power3(DeltaInv(msb12,mst12,
     mC2))*power5(mst12)))/(12.*msb12*msb22*mst12) - (msb12*msb22 + 2*msb12*
     mst12 + msb22*mst12 - mH2*msb12*msb22*mst12*DeltaInv(mH2,msb12,msb22) -
     mA2*msb12*msb22*mst12*DeltaInv(msb12,mA2,msb22) - 2*mC2*msb12*msb22*mst12*
     DeltaInv(msb22,mst12,mC2) + 42*msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)
     *mst12*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 42*msb22*(3*mA2*
     msb22 + msb12*msb22 + 3*msb24)*mst12*power2(mA2)*power2(DeltaInv(msb12,
     mA2,msb22)) + 252*msb12*mst12*mst14*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) + 84*msb12*msb22*power2(mC2)*power2(mst12)*power2(DeltaInv(
     msb22,mst12,mC2)) + 252*msb12*power2(mst12)*power2(DeltaInv(msb22,mst12,
     mC2))*power3(mC2) - 42*msb22*mst12*power2(mH2)*(-((5*msb12*msb22 + 3*
     msb24)*power2(mH2)) + 3*msb22*power3(mH2) + (-5*msb12 + 3*msb22)*power3(
     msb22) - mH2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(mH2,
     msb12,msb22)) + 1176*msb12*msb22*mst12*mst14*power3(mC2)*power3(DeltaInv(
     msb22,mst12,mC2)) + 252*msb12*mst12*mst14*power3(DeltaInv(msb22,mst12,mC2)
     )*power4(mC2) + 420*msb12*msb22*power2(mst12)*power3(DeltaInv(msb22,mst12,
     mC2))*power4(mC2) - 42*msb22*mst12*power2(mA2)*power3(DeltaInv(msb12,mA2,
     msb22))*(-14*mA2*msb12*msb24 - 5*msb12*msb22*power2(mA2) - 3*msb24*power2(
     mA2) + 3*msb22*power3(mA2) - 3*mA2*power3(msb22) - 5*msb12*power3(msb22) +
     3*power4(msb22)) + 420*msb12*msb22*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*power4(mst12) + 252*msb12*power3(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*power4(mst12) - 252*msb12*power2(mst12)*power3(DeltaInv(msb22,
     mst12,mC2))*power5(mC2) - 252*msb12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*power5(mst12))/(12.*msb12*msb22*mst12) + (log(msb12/Q2)*
     power2(cb)*power2(snb)*power2(snt)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 -
     mst22*mw2 - mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) + mst12*mst14*
     mst22*DeltaInv(mw2,msb12,mst12) - msb12*mst22*DeltaInv(mw2,msb12,mst12)*
     power2(mst12) + 5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) +
     msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mst22) - 5*mst12*mw2*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) + msb12*mst12*DeltaInv(mst22,mw2,
     msb12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) -
     msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) + 5*mst12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - 6*msb12*mst12*mst22*mst24*mw2*
     power2(DeltaInv(mst22,mw2,msb12)) + 138*mst12*mst22*mst24*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb12)) + 36*msb12*mst12*power2(mst22)*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst14*mst22*mw2*
     power2(DeltaInv(mw2,msb12,mst12)) - 138*mst12*mst14*mst22*power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12)) - 36*msb12*mst22*power2(mst12)*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) - mst22*DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(
     msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2)
     )*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*
     mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) -
     mst12*DeltaInv(mst22,mw2,msb12)*power3(mw2) + mst22*DeltaInv(mw2,msb12,
     mst12)*power3(mw2) - 6*msb12*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mw2) + 138*mst12*power2(mst22)*power2(DeltaInv(mst22,mw2,msb12))*
     power3(mw2) + 6*msb12*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 138*mst22*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + 720*msb12*
     mst12*mst22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 720*
     msb12*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*
     mst22*mw2*power2(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 216*msb12*
     mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 144*
     mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 6*
     mst12*mw2*power2(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 216*msb12*
     mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 144*
     mst12*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 6*
     mst12*mst22*power2(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 6*mst12*mst22*
     power2(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*mst12*mst22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 216*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 144*mst12*mst14*
     mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 216*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 144*mst22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(mst12) - 144*mst12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(mst22) - 144*mst12*
     power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) + 144*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)))/(12.*msb12*
     mst12*mst22) - (power2(cb)*power2(snb)*power2(snt)*(-(mC2*msb12*msb22*
     DeltaInv(msb12,mst12,mC2)) + mC2*msb12*msb22*DeltaInv(msb22,mst12,mC2) +
     mC2*msb12*msb22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*DeltaInv(
     mst22,msb22,mC2) - msb12*msb22*mw2*DeltaInv(mst22,mw2,msb12) + msb12*
     msb22*mw2*DeltaInv(mst22,mw2,msb22) + msb12*msb22*mw2*DeltaInv(mw2,msb12,
     mst12) - msb12*msb22*mw2*DeltaInv(mw2,msb22,mst12) + 42*msb22*(3*mC2*mst12
     + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) -
     42*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - 42*msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) + 42*msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + 126*msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) + 42*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 126*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) -
     42*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 42*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 126*msb22*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 42*msb12*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 126*msb12*mst14*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 126*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) + 126*msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2))*power3(mC2) + 126*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 126*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 126*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(
     mw2) + 126*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 42*
     msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*
     mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*
     msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 126*
     msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 210*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) +
     588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 126*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 210*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) -
     588*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 126*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 210*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 126*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 210*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     588*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 126*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 210*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 126*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 126*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 42*msb12*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*
     mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*
     power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 126*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 126*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) + 210*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 126*msb22*
     mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 210*msb12*msb22*
     mst22*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 126*msb12*mst24*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) - 210*msb12*msb22*mst12*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 126*msb22*mst14*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 210*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) + 126*msb12*mst14*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power5(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2) - 126*msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 126*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) +
     126*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2) - 126*msb12*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(6.*msb12*msb22) + (
     power2(cb)*power2(Pi)*power2(snb)*power2(snt)*(-(msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) + msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + 3*msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))
     - msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*
     msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 3*msb22*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*mst22*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) + msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb12,mst12)) - msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) - 3*msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)
     ) + 3*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 3*msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 3*msb22*mst22*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mw2) + 3*msb22*mst12*power2(DeltaInv(
     mw2,msb12,mst12))*power3(mw2) - 3*msb12*mst12*power2(DeltaInv(mw2,msb22,
     mst12))*power3(mw2) + msb22*power2(mC2)*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     + 14*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 5*
     msb12*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     3*msb22*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 14*
     msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*
     msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 3*
     msb12*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 5*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) -
     14*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*
     msb22*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 5*
     msb12*msb22*power2(mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) +
     14*msb12*msb22*mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 3*
     msb12*power3(mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 5*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     14*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 5*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) -
     14*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*
     mst24*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + msb12*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2)
     + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*
     msb22*power3(mst12) - 3*power4(mst12)) - 3*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 3*msb22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb12*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*mst22*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mw2) + 3*msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) + 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) + 3*msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*
     msb12*mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 3*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power5(mC2) + 3*msb12*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*msb22*mst22*power3(DeltaInv(
     mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst22*power3(DeltaInv(mst22,mw2,
     msb22))*power5(mw2) - 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*
     power5(mw2) + 3*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2))
     )/(6.*msb12*msb22) + (log(msb22/Q2)*power2(cb)*power2(snb)*power2(snt)*(
     mC2*mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb22) - mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) +
     msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*mw2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) - msb22*mst12*DeltaInv(mst22,mw2,
     msb22)*power2(mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     - msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) +
     6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) - 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     - mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(12.*msb22*mst12*mst22) + (log(msb22/Q2)*power2(cb)*power2(snt)*(-(
     mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) -
     msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*DeltaInv(mst22,mw2,
     msb22)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     + mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     + 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(12.*msb22*mst12*mst22) + Fin3(mh2,msb12,msb12,Q2)*(-2*power2(sa)*
     power2(snb)*power2(DeltaInv(mh2,msb12,msb12))*(msb14 + 4*mh2*DeltaInv(mh2,
     msb12,msb12)*power6(msb1)) + 2*power2(sa)*power2(DeltaInv(mh2,msb12,msb12)
     )*power4(snb)*(msb14 + 4*mh2*DeltaInv(mh2,msb12,msb12)*power6(msb1))) +
     Fin3(mH2,msb12,msb12,Q2)*(-2*power2(snb)*power2(DeltaInv(mH2,msb12,msb12))
     *(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*power6(msb1)) + 2*power2(sa)*
     power2(snb)*power2(DeltaInv(mH2,msb12,msb12))*(msb14 + 4*mH2*DeltaInv(mH2,
     msb12,msb12)*power6(msb1)) + 2*power2(DeltaInv(mH2,msb12,msb12))*power4(
     snb)*(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*power6(msb1)) - 2*power2(sa)
     *power2(DeltaInv(mH2,msb12,msb12))*power4(snb)*(msb14 + 4*mH2*DeltaInv(
     mH2,msb12,msb12)*power6(msb1))) + (Fin3(msb12,mz2,msb22,Q2)*power2(cb)*
     power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(-msb22 + DeltaInv(msb12,
     mz2,msb22)*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) +
     power6(msb2))))/msb12 + (power2(cb)*power2(DeltaInv(msb12,mz2,msb22))*
     power2(log(mz2/Q2))*power3(mz2)*(-msb22 + DeltaInv(msb12,mz2,msb22)*(mz2*(
     -2*msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/(2.*
     msb12) + (mz2*log(mz2/Q2)*power2(cb)*(-1 + mz2*(-msb12 + 5*msb22 + mz2)*
     DeltaInv(msb12,mz2,msb22) + 6*msb22*mz2*(-msb24 + (14*msb22 - mz2)*mz2 +
     msb12*(3*msb22 + mz2))*power2(DeltaInv(msb12,mz2,msb22)) - 72*msb22*
     power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(mz2*(-2*msb24 + msb22*mz2)
     - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/(24.*msb12*msb22) + (
     power2(cb)*power2(DeltaInv(msb12,mA2,msb22))*power2(log(mA2/Q2))*power3(
     mA2)*(msb22 - DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*
     msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/(2.*msb12) + (power2(
     DeltaInv(msb12,mA2,msb22))*power2(log(mA2/Q2))*power3(mA2)*(-msb22 +
     DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) +
     msb22*power2(mA2) + power6(msb2))))/(2.*msb12) + (mA2*log(mA2/Q2)*(-1 +
     mA2*(mA2 - msb12 + 5*msb22)*DeltaInv(msb12,mA2,msb22) - 6*mA2*msb22*(-3*
     msb12*msb22 - mA2*(msb12 + 14*msb22) + msb24 + power2(mA2))*power2(
     DeltaInv(msb12,mA2,msb22)) - 72*msb22*power2(mA2)*power3(DeltaInv(msb12,
     mA2,msb22))*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(
     mA2) + power6(msb2))))/(24.*msb12*msb22) + (mA2*log(mA2/Q2)*power2(cb)*(1
     - mA2*(mA2 - msb12 + 5*msb22)*DeltaInv(msb12,mA2,msb22) + 6*mA2*msb22*(-3*
     msb12*msb22 - mA2*(msb12 + 14*msb22) + msb24 + power2(mA2))*power2(
     DeltaInv(msb12,mA2,msb22)) + 72*msb22*power2(mA2)*power3(DeltaInv(msb12,
     mA2,msb22))*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(
     mA2) + power6(msb2))))/(24.*msb12*msb22) + (power2(sa)*power2(DeltaInv(
     mh2,msb12,msb22))*power2(log(mh2/Q2))*power3(mh2)*(-msb22 + DeltaInv(mh2,
     msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(
     mh2) + power6(msb2))))/(2.*msb12) + (mh2*log(mh2/Q2)*power2(sa)*(-1 + mh2*
     (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22) - 6*mh2*msb22*(-3*msb12*
     msb22 - mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,
     msb12,msb22)) - 72*msb22*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-
     2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(
     msb2))))/(24.*msb12*msb22) + (power2(sa)*power2(DeltaInv(mH2,msb12,msb22))
     *power2(log(mH2/Q2))*power3(mH2)*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(
     msb2))))/(2.*msb12) + (power2(DeltaInv(mH2,msb12,msb22))*power2(log(mH2/
     Q2))*power3(mH2)*(-msb22 + DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*
     mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(2.*msb12)
     + (mH2*log(mH2/Q2)*(-1 + mH2*(mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,
     msb22) - 6*mH2*msb22*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 +
     power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) - 72*msb22*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power6(msb2))))/(24.*msb12*msb22) + (mH2*log(
     mH2/Q2)*power2(sa)*(1 - mH2*(mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,
     msb22) + 6*mH2*msb22*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 +
     power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 72*msb22*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power6(msb2))))/(24.*msb12*msb22) + Fin3(mh2,
     msb22,msb22,Q2)*(-2*power2(sa)*power2(snb)*power2(DeltaInv(mh2,msb22,
     msb22))*(msb24 + 4*mh2*DeltaInv(mh2,msb22,msb22)*power6(msb2)) + 2*power2(
     sa)*power2(DeltaInv(mh2,msb22,msb22))*power4(snb)*(msb24 + 4*mh2*DeltaInv(
     mh2,msb22,msb22)*power6(msb2))) + Fin3(mH2,msb22,msb22,Q2)*(-2*power2(snb)
     *power2(DeltaInv(mH2,msb22,msb22))*(msb24 + 4*mH2*DeltaInv(mH2,msb22,
     msb22)*power6(msb2)) + 2*power2(sa)*power2(snb)*power2(DeltaInv(mH2,msb22,
     msb22))*(msb24 + 4*mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2)) + 2*power2(
     DeltaInv(mH2,msb22,msb22))*power4(snb)*(msb24 + 4*mH2*DeltaInv(mH2,msb22,
     msb22)*power6(msb2)) - 2*power2(sa)*power2(DeltaInv(mH2,msb22,msb22))*
     power4(snb)*(msb24 + 4*mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2))) +
     Fin3(msb12,mA2,msb22,Q2)*((power2(cb)*power2(mA2)*power2(DeltaInv(msb12,
     mA2,msb22))*(msb22 - DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(
     msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/msb12 + (
     power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))*(-msb22 + DeltaInv(msb12,
     mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(
     mA2) + power6(msb2))))/msb12) + Fin3(mh2,msb12,msb22,Q2)*((4*power2(mh2)*
     power2(sa)*power2(snb)*power2(DeltaInv(mh2,msb12,msb22))*(msb22 -
     DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) +
     msb22*power2(mh2) + power6(msb2))))/msb12 + (power2(mh2)*power2(sa)*
     power2(DeltaInv(mh2,msb12,msb22))*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(
     msb2))))/msb12 + (4*power2(mh2)*power2(sa)*power2(DeltaInv(mh2,msb12,
     msb22))*power4(snb)*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 -
     2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/msb12) +
     Fin3(mH2,msb12,msb22,Q2)*((power2(mH2)*power2(sa)*power2(DeltaInv(mH2,
     msb12,msb22))*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(
     msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/msb12 + (4*
     power2(mH2)*power2(snb)*power2(DeltaInv(mH2,msb12,msb22))*(msb22 -
     DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power6(msb2))))/msb12 + (4*power2(mH2)*power2(sa)*
     power2(DeltaInv(mH2,msb12,msb22))*power4(snb)*(msb22 - DeltaInv(mH2,msb12,
     msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) +
     power6(msb2))))/msb12 + (power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-
     msb22 + DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power6(msb2))))/msb12 + (4*power2(mH2)*
     power2(sa)*power2(snb)*power2(DeltaInv(mH2,msb12,msb22))*(-msb22 +
     DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power6(msb2))))/msb12 + (4*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*power4(snb)*(-msb22 + DeltaInv(mH2,msb12,msb22)
     *(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) +
     power6(msb2))))/msb12) + (power2(cb)*power2(DeltaInv(mw2,msb22,mst12))*
     power2(log(mw2/Q2))*power3(mw2)*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(
     -2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/
     msb22 + (mw2*log(mw2/Q2)*power2(cb)*(-1 + mw2*(-msb22 + 5*mst12 + mw2)*
     DeltaInv(mw2,msb22,mst12) + 6*mst12*mw2*(-mst14 + (14*mst12 - mw2)*mw2 +
     msb22*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb22,mst12)) - 72*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2)
     - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(12.*msb22*mst12) + (mw2*
     log(mw2/Q2)*power2(cb)*power2(snb)*(msb12 - msb22 + msb22*mw2*(-msb12 + 5*
     mst12 + mw2)*DeltaInv(mw2,msb12,mst12) + msb12*(msb22 - 5*mst12 - mw2)*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb22*mst12*mw2*(-mst14 + (14*mst12 -
     mw2)*mw2 + msb12*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb12,mst12)) - 6*
     msb12*mst12*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*
     power2(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst12*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1)) + 72*msb12*mst12*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*
     mw2) + power6(mst1))))/(12.*msb12*msb22*mst12) + (power2(cb)*power2(
     DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2))*power3(mC2)*(mst12 -
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (power2(DeltaInv(msb22,mst12,
     mC2))*power2(log(mC2/Q2))*power3(mC2)*(-mst12 + DeltaInv(msb22,mst12,mC2)*
     (-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/msb22 + (mC2*log(mC2/Q2)*(-1 + mC2*(mC2 - msb22 + 5*mst12)
     *DeltaInv(msb22,mst12,mC2) - 6*mC2*mst12*(-3*msb22*mst12 - mC2*(msb22 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 72*
     mst12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(12.*
     msb22*mst12) + (mC2*log(mC2/Q2)*power2(cb)*(1 - mC2*(mC2 - msb22 + 5*
     mst12)*DeltaInv(msb22,mst12,mC2) + 6*mC2*mst12*(-3*msb22*mst12 - mC2*(
     msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2))
     + 72*mst12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 -
     2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(12.*
     msb22*mst12) + (mC2*log(mC2/Q2)*power2(snb)*(msb12 - msb22 + mC2*msb22*(
     mC2 - msb12 + 5*mst12)*DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22
     - 5*mst12)*DeltaInv(msb22,mst12,mC2) - 6*mC2*msb22*mst12*(-3*msb12*mst12 -
     mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,
     mC2)) + 6*mC2*msb12*mst12*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14
     + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 72*msb22*mst12*power2(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*
     power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(12.*msb12*
     msb22*mst12) - (mC2*log(mC2/Q2)*power2(cb)*power2(snb)*(msb12 - msb22 +
     mC2*msb22*(mC2 - msb12 + 5*mst12)*DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-
     mC2 + msb22 - 5*mst12)*DeltaInv(msb22,mst12,mC2) - 6*mC2*msb22*mst12*(-3*
     msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*(-3*msb22*mst12 - mC2*(
     msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2))
     - 72*msb22*mst12*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*msb12*mst12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     (12.*msb12*msb22*mst12) + Fin3(mw2,msb12,mst12,Q2)*((2*power2(cb)*power2(
     mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/msb12 + (2*power2(cb)*power2(mw2)*power2(snb)
     *power2(DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))
     /msb12) + (power2(cb)*power2(snb)*power2(log(mw2/Q2))*power3(mw2)*(-(
     msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))) + msb22*power3(DeltaInv(
     mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*
     mw2) + power6(mst1)) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst12 -
     DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1)))))/(msb12*msb22) + Fin3(mw2,msb22,mst12,Q2)*((
     2*power2(cb)*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(
     mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1))))/msb22 + (2*power2(cb)*power2(mw2)*
     power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(
     mst1))))/msb22 + (2*power2(cb)*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12))*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (2*power2(cb)*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(-
     mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1))))/msb22) + Fin3(msb12,mst12,mC2,Q2)*((
     2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(
     mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb12 + (2*power2(mC2)*
     power2(snb)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(mst12 -
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb12 + (2*power2(mC2)*power2(snb)*
     power2(DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb12 + (2*power2(cb)*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     msb12) + (power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(-(msb22*mst12*
     power2(DeltaInv(msb12,mst12,mC2))) + msb22*power3(DeltaInv(msb12,mst12,
     mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(
     msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)))))/(msb12*msb22) + (power2(cb)*power2(snb)*
     power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,
     mC2)) - msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(
     msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)))))
     /(msb12*msb22) + Fin3(msb22,mst12,mC2,Q2)*((2*power2(cb)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb22 + (2*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,
     mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (2*power2(
     mC2)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(
     msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/msb22 + (2*power2(cb)*power2(mC2)*power2(
     snb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(
     msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/msb22 + (2*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (
     2*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(-
     mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (2*power2(cb)*power2(
     mC2)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(
     msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/msb22 + (2*power2(mC2)*power2(snb)*power2(
     snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)
     *(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/msb22) + (power2(snb)*power2(snt)*power2(log(mC2/Q2))*
     power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*
     power2(DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) + msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) - msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(msb12*msb22) + (power2(cb)*power2(snb)*power2(
     snt)*power2(log(mC2/Q2))*power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) + msb22*
     mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,
     mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + 2*msb12*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) +
     msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + msb22*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - msb12*power3(DeltaInv(msb22,mst12,mC2)
     )*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) +
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22) - (
     mC2*log(mC2/Q2)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 +
     msb12*mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*
     DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*
     DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,
     mC2) - 5*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*
     msb22*mst12*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(
     mst22,msb22,mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) +
     msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*
     mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*
     mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) - 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 18*mC2*msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) - 84*msb22*mst12*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*mst22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*
     mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*
     msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     12.*msb12*msb22*mst12*mst22) + (mC2*log(mC2/Q2)*power2(cb)*power2(snb)*
     power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*
     msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*
     msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*
     DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,
     mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2) - msb22*mst12*
     DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*DeltaInv(mst22,msb22,
     mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*
     mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*
     mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*
     mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*
     mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*
     mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14
     - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(12.*msb12*msb22*mst12*mst22) + (2*Fin3(mst22,mw2,msb12,
     Q2)*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,
     mw2,msb12))*(-mst22 + DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*
     mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 + (power2(cb)*
     power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(-(mst22*power2(DeltaInv(
     mst22,mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(
     mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2)
     + power6(mst1))) + power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 + (
     power2(cb)*power2(snb)*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(-(
     msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))) + msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22)) + msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))
     - msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12)) + 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb12,mst12)) + 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,
     mst12)) - msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 2*
     msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst12*
     mw2*power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*mw2*power3(DeltaInv(
     mw2,msb22,mst12)) + msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + msb12*
     power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) + msb22*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     msb22) + (mw2*log(mw2/Q2)*power2(cb)*power2(snt)*(-mst12 + mst22 + mst12*
     mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) + mst22*(msb22 - 5*
     mst12 - mw2)*mw2*DeltaInv(mw2,msb22,mst12) + 6*mst12*mst22*mw2*(-mst24 + (
     14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,
     msb22)) - 6*mst12*mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*
     mst12 + mw2))*power2(DeltaInv(mw2,msb22,mst12)) + 72*mst12*mst22*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb22*(mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/(12.*msb22*mst12*mst22) - (mw2*log(
     mw2/Q2)*power2(cb)*power2(snb)*power2(snt)*(-(msb12*mst12) + msb22*mst12 +
     msb12*mst22 - msb22*mst22 + msb22*mst12*(msb12 - 5*mst22 - mw2)*mw2*
     DeltaInv(mst22,mw2,msb12) + msb12*mst12*mw2*(-msb22 + 5*mst22 + mw2)*
     DeltaInv(mst22,mw2,msb22) - msb12*msb22*mst22*mw2*DeltaInv(mw2,msb12,
     mst12) + 5*msb22*mst12*mst22*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*
     mst22*mw2*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mst22*mw2*DeltaInv(
     mw2,msb22,mst12) + msb22*mst22*DeltaInv(mw2,msb12,mst12)*power2(mw2) -
     msb12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) - 6*msb22*mst12*mst22*
     mw2*(-mst24 + (14*mst22 - mw2)*mw2 + msb12*(3*mst22 + mw2))*power2(
     DeltaInv(mst22,mw2,msb12)) + 6*msb12*mst12*mst22*mw2*(-mst24 + (14*mst22 -
     mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) - 6*
     msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,mst12)) + 18*msb12*
     msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*
     msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) +
     6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst22*mw2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12)) - 6*
     msb12*msb22*mst12*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     84*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     - 6*msb22*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*
     msb12*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 144*
     msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)
     ) + 144*msb22*mst12*mst14*mst22*power3(mw2)*power3(DeltaInv(mw2,msb12,
     mst12)) + 144*msb12*msb22*mst22*power2(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) - 144*msb12*msb22*mst12*mst14*mst22*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12)) - 144*msb12*mst12*mst14*mst22*power3(mw2)*
     power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst22*power2(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*mst22*power2(
     mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst22*
     power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 72*msb22*
     mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) +
     72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power6(
     mst1) + 72*msb22*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))
     *(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) + power6(mst2))
     - 72*msb12*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*(mw2*
     (-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(
     12.*msb12*msb22*mst12*mst22) + (mC2*log(mC2/Q2)*power2(snt)*(-mst12 +
     mst22 + mC2*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) +
     mC2*mst12*(mC2 - msb22 + 5*mst22)*DeltaInv(mst22,msb22,mC2) + 6*mC2*mst12*
     mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*msb22*mst22 -
     mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb22,
     mC2)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(12.*msb22*mst12*mst22) + (mC2*log(mC2/Q2)*power2(cb)*power2(snt)
     *(mst12 - mst22 + mC2*(mC2 - msb22 + 5*mst12)*mst22*DeltaInv(msb22,mst12,
     mC2) + mC2*mst12*(-mC2 + msb22 - 5*mst22)*DeltaInv(mst22,msb22,mC2) - 6*
     mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*mst12*mst22*(-3*msb22*
     mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(
     mst22,msb22,mC2)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(12.*msb22*mst12*mst22) + Fin3(mst22,mw2,msb22,Q2)*
     ((2*power2(cb)*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,
     mw2,msb22))*(mst22 - DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2)
     - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 + (2*power2(cb)*
     power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(-mst22 +
     DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/msb22) + Fin3(mst22,msb12,mC2,Q2)*((2*power2(
     cb)*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*
     (mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (2*power2(mC2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb12) + (power2(cb)*power2(snt)*
     power2(log(mC2/Q2))*power3(mC2)*(-(mst12*power2(DeltaInv(msb22,mst12,mC2))
     ) + power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12
     + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/msb22 + (
     power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(mst12*power2(DeltaInv(msb22,
     mst12,mC2)) - power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(
     DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     /msb22 + Fin3(mst22,msb22,mC2,Q2)*((2*power2(cb)*power2(mC2)*power2(snt)*
     power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/msb22 + (2*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*
     mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb22 + (
     2*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb22 + (2*power2(cb)*power2(mC2)*
     power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 +
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb22)) + xb*(-((mu*sb*log(mA2/Q2)*
     log(msb12/Q2)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))*(3*mA2*msb22 +
     msb12*msb22 + msb24 + DeltaInv(msb12,mA2,msb22)*(5*(msb12*msb22 + msb24)*
     power2(mA2) - 3*msb22*power3(mA2) + (msb12 - msb22)*power3(msb22) - mA2*(-
     10*msb12*msb24 + power3(msb22)))))/(cb*msb12)) - (ca*mu*sa*log(mh2/Q2)*
     log(msb12/Q2)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))*(3*mh2*msb22 +
     msb12*msb22 + msb24 + DeltaInv(mh2,msb12,msb22)*(5*(msb12*msb22 + msb24)*
     power2(mh2) - 3*msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) - mh2*(-
     10*msb12*msb24 + power3(msb22)))))/msb12 - (mu*sb*log(mh2/Q2)*log(msb12/
     Q2)*power2(mh2)*power2(sa)*power2(DeltaInv(mh2,msb12,msb22))*(3*mh2*msb22
     + msb12*msb22 + msb24 + DeltaInv(mh2,msb12,msb22)*(5*(msb12*msb22 + msb24)
     *power2(mh2) - 3*msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) - mh2*(
     -10*msb12*msb24 + power3(msb22)))))/(cb*msb12) + (ca*mu*sa*log(mH2/Q2)*
     log(msb12/Q2)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22 +
     msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*(5*(msb12*msb22 + msb24)*
     power2(mH2) - 3*msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) - mH2*(-
     10*msb12*msb24 + power3(msb22)))))/msb12 - (mu*sb*log(mH2/Q2)*log(msb12/
     Q2)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22 + msb12*
     msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*(5*(msb12*msb22 + msb24)*power2(
     mH2) - 3*msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) - mH2*(-10*
     msb12*msb24 + power3(msb22)))))/(cb*msb12) + (mu*sb*log(mH2/Q2)*log(msb12/
     Q2)*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22
     + msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*(5*(msb12*msb22 + msb24)
     *power2(mH2) - 3*msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) - mH2*(
     -10*msb12*msb24 + power3(msb22)))))/(cb*msb12) + (4*mu*sb*log(msb12/Q2)*
     log(msb22/Q2)*power2(mH2)*power2(snb)*power2(DeltaInv(mH2,msb12,msb22))*(
     mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*
     msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)*
     power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))))/(cb*msb12) + (mu*
     sb*log(mA2/Q2)*log(msb22/Q2)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))
     *(mA2*msb22 + msb12*msb22 + msb24 + DeltaInv(msb12,mA2,msb22)*((msb12*
     msb22 + msb24)*power2(mA2) - msb22*power3(mA2) + (msb12 - msb22)*power3(
     msb22) + mA2*(6*msb12*msb24 + power3(msb22)))))/(cb*msb12) + (ca*mu*sa*
     log(mh2/Q2)*log(msb22/Q2)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))*(
     mh2*msb22 + msb12*msb22 + msb24 + DeltaInv(mh2,msb12,msb22)*((msb12*msb22
     + msb24)*power2(mh2) - msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) +
     mh2*(6*msb12*msb24 + power3(msb22)))))/msb12 + (mu*sb*log(mh2/Q2)*log(
     msb22/Q2)*power2(mh2)*power2(sa)*power2(DeltaInv(mh2,msb12,msb22))*(mh2*
     msb22 + msb12*msb22 + msb24 + DeltaInv(mh2,msb12,msb22)*((msb12*msb22 +
     msb24)*power2(mh2) - msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) +
     mh2*(6*msb12*msb24 + power3(msb22)))))/(cb*msb12) - (ca*mu*sa*log(mH2/Q2)*
     log(msb22/Q2)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 +
     msb12*msb22 + msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 + msb24)*
     power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*
     msb12*msb24 + power3(msb22)))))/msb12 + (mu*sb*log(mH2/Q2)*log(msb22/Q2)*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*msb22 +
     msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mH2) -
     msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 +
     power3(msb22)))))/(cb*msb12) - (mu*sb*log(mH2/Q2)*log(msb22/Q2)*power2(
     mH2)*power2(sa)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*msb22
     + msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 + msb24)*power2(mH2) -
     msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*msb24 +
     power3(msb22)))))/(cb*msb12) - (2*mu*sb*log(mC2/Q2)*log(msb12/Q2)*power2(
     mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(3*mC2*mst12 + msb12*
     mst12 + mst14 + DeltaInv(msb12,mst12,mC2)*(5*(msb12*mst12 + mst14)*power2(
     mC2) - 3*mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) - mC2*(-10*
     msb12*mst14 + power3(mst12)))))/(cb*msb12) - (2*mu*sb*log(msb12/Q2)*log(
     mst12/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mC2*
     mst12 + msb12*mst12 + 3*mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12 -
     mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12)
     + 5*mC2*(2*msb12*mst14 + power3(mst12)))))/(cb*msb12) + (2*mu*sb*log(
     msb12/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 + 3*mst14 + DeltaInv(
     msb12,mst12,mC2)*((msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) +
     (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))
     ))/(cb*msb12) - (2*mu*sb*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(
     msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*power2(mC2) - 3*mst12*power3(
     mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-10*msb22*mst14 + power3(
     mst12)))))/(cb*msb22) + (2*mu*sb*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 + msb22*mst12 +
     mst14 + DeltaInv(msb22,mst12,mC2)*(5*(msb22*mst12 + mst14)*power2(mC2) -
     3*mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) - mC2*(-10*msb22*mst14
     + power3(mst12)))))/(cb*msb22) + (2*cb*mt2*mu*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*power2(log(mst12/Q2))*(mst14 - DeltaInv(msb22,mst12,mC2)
     *(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)*sb) + (2*mu*sb*power2(
     mC2)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*(
     mst14 - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(cb*msb22) + (2*
     mt2*mu*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))
     *power3(cb)*(mst14 - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*
     msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(
     msb22*(mst12 - mst22)*sb) + (2*cb*mt2*mu*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*power2(log(mst12/Q2))*(-mst14 + DeltaInv(msb22,mst12,
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)*sb) + (2*mu*sb*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*(-mst14 +
     DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(
     mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(cb*msb22) + (2*mt2*mu*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*
     power3(cb)*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*
     msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(
     msb22*(mst12 - mst22)*sb) + (mu*sb*DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)
     *(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*
     mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))))/(6.*cb*msb22) - (mu*sb*DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)*
     power2(snt)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,
     mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))))/(6.*cb*msb22) - (2*mu*sb*log(msb22/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 +
     msb22*mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*
     power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*
     mC2*(2*msb22*mst14 + power3(mst12)))))/(cb*msb22) + (2*mu*sb*log(msb22/Q2)
     *log(mst12/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*
     mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*
     power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))))/(cb*msb22) + (2*
     mu*sb*log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(snt)*power2(DeltaInv(
     msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14 + DeltaInv(msb22,
     mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*
     msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))))/
     (cb*msb22) - (2*mu*sb*log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 +
     3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) -
     mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*
     mst14 + power3(mst12)))))/(cb*msb22) + (2*mu*sb*log(mC2/Q2)*log(mst12/Q2)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 +
     mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) -
     mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 +
     power3(mst12)))))/(cb*msb22) - (2*mu*sb*log(mC2/Q2)*log(mst12/Q2)*power2(
     mC2)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*
     mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(
     mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*
     mst14 + power3(mst12)))))/(cb*msb22) - (2*mu*sb*log(msb12/Q2)*log(mst22/
     Q2)*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2))*
     (mC2*mst22 + msb12*mst22 + 3*mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*
     mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*
     power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))))/(cb*msb12) + (2*
     cb*mt2*mu*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power2(log(mst22/
     Q2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(msb22*(
     mst12 - mst22)*sb) + (2*mt2*mu*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*power2(log(mst22/Q2))*power3(cb)*(mst24 - DeltaInv(mst22,msb22,mC2)*
     (mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))))/(msb22*(mst12 - mst22)*sb) + (2*cb*mt2*mu*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*power2(log(mst22/Q2))*(-mst24 +
     DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(
     mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)*sb)
     + (2*mu*sb*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*
     power2(log(mst22/Q2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))))/(cb*msb22) + (2*mt2*mu*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*power2(log(mst22/Q2))*power3(cb)*(-mst24 + DeltaInv(mst22,msb22,mC2)
     *(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*
     mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)*sb) + (mu*sb*DeltaInv(
     mst22,msb22,mC2)*log(mst22/Q2)*power2(snt)*(5*mC2*mst22 - msb22*mst22 +
     mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24
     - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(6.*cb*msb22) - (2*
     mu*sb*log(msb22/Q2)*log(mst22/Q2)*power2(mC2)*power2(snt)*power2(DeltaInv(
     mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,
     msb22,mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*
     msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))))/
     (cb*msb22) + (2*mu*sb*log(msb22/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 +
     3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 - mst24)*power2(mC2) -
     mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*
     mst24 + power3(mst22)))))/(cb*msb22) + (2*mu*sb*log(mC2/Q2)*log(mst22/Q2)*
     power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 +
     msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*
     power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*
     msb22*mst24 + power3(mst22)))))/(cb*msb22) + (ca*mu*sa*log(msb22/Q2)*((5*
     mh2*msb22 - msb12*msb22 + msb24)*DeltaInv(mh2,msb12,msb22) - 6*mh2*power2(
     DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*
     msb24 + msb22*power2(mh2) + power3(msb22)) + DeltaInv(mH2,msb12,msb22)*(-
     5*mH2*msb22 + msb12*msb22 - msb24 + 6*mH2*DeltaInv(mH2,msb12,msb22)*(-3*
     mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) + power3(
     msb22)) + 72*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(msb24*power2(
     mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(
     msb22)))) - 72*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22))))/(12.*msb12) + (mu*sb*log(msb22/Q2)*power2(sa)*((5*mh2*msb22 -
     msb12*msb22 + msb24)*DeltaInv(mh2,msb12,msb22) - 6*mh2*power2(DeltaInv(
     mh2,msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 +
     msb22*power2(mh2) + power3(msb22)) + DeltaInv(mH2,msb12,msb22)*(-5*mH2*
     msb22 + msb12*msb22 - msb24 + 6*mH2*DeltaInv(mH2,msb12,msb22)*(-3*mH2*
     msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) + power3(
     msb22)) + 72*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(msb24*power2(
     mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(
     msb22)))) - 72*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22))))/(12.*cb*msb12) + (ca*mu*sa*power2(log(msb22/Q2))*(-(msb24*
     power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(msb24 - DeltaInv(mH2,msb12,msb22)*(msb24*
     power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 +
     power3(msb22)))) + power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22))))/msb12 + (mu*sb*power2(sa)*power2(log(msb22/Q2))*(-(msb24*
     power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(msb24 - DeltaInv(mH2,msb12,msb22)*(msb24*
     power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 +
     power3(msb22)))) + power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22))))/(cb*msb12) + (ca*mu*sa*log(msb12/Q2)*log(msb22/Q2)*(-((
     mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 +
     msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 - msb24)*
     power2(mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*
     mH2*(2*msb12*msb24 + power3(msb22)))) + power2(mh2)*((-(msb12*msb22) +
     msb24)*power2(mh2) + msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(
     msb22) - 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22))))/msb12 + (mu*sb*log(msb12/Q2)*log(msb22/Q2)*power2(sa)*(-((mh2*
     msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 +
     msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 - msb24)*
     power2(mH2) - msb22*power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*
     mH2*(2*msb12*msb24 + power3(msb22)))) + power2(mh2)*((-(msb12*msb22) +
     msb24)*power2(mh2) + msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(
     msb22) - 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22))))/(cb*msb12) + (4*ca*mu*sa*log(msb12/Q2)*log(msb22/Q2)*power2(snb)
     *((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-(mH2*
     msb22) - msb12*msb22 - 3*msb24 + DeltaInv(mH2,msb12,msb22)*((-(msb12*
     msb22) + msb24)*power2(mH2) + msb22*power3(mH2) + (-5*msb12 + 3*msb22)*
     power3(msb22) - 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(mh2)*((
     msb12*msb22 - msb24)*power2(mh2) - msb22*power3(mh2) + (5*msb12 - 3*msb22)
     *power3(msb22) + 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     mh2,msb12,msb22))))/msb12 + (4*mu*sb*log(msb12/Q2)*log(msb22/Q2)*power2(
     sa)*power2(snb)*((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))
     *(-(mH2*msb22) - msb12*msb22 - 3*msb24 + DeltaInv(mH2,msb12,msb22)*((-(
     msb12*msb22) + msb24)*power2(mH2) + msb22*power3(mH2) + (-5*msb12 + 3*
     msb22)*power3(msb22) - 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(
     mh2)*((msb12*msb22 - msb24)*power2(mh2) - msb22*power3(mh2) + (5*msb12 -
     3*msb22)*power3(msb22) + 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(cb*msb12) + (ca*mu*sa*power2(log(msb12/Q2))*
     (-((2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*
     msb22 + msb12*msb22 + 2*msb24 + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22
     + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(
     msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))) + power2(mh2)*(-((3*
     msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*
     msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/msb12 + (mu*sb*power2(sa)*power2(log(msb12/
     Q2))*(-((2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(
     mh2,msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*
     msb22 + msb12*msb22 + 2*msb24 + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22
     + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(
     msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))) + power2(mh2)*(-((3*
     msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*
     msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22))))/(cb*msb12) + (2*ca*mu*sa*power2(snb)*power2(
     log(msb12/Q2))*(mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,msb12)) + 2*(2*
     mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + mH2*(-(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12))) + 2*mH2*
     power2(DeltaInv(mH2,msb12,msb22))*(-2*mH2*msb22 - msb12*msb22 - 2*msb24 +
     DeltaInv(mH2,msb12,msb22)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*
     msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22))))) + 2*power2(mh2)*((3*msb12*msb22 + 2*msb24)*
     power2(mh2) - 2*msb22*power3(mh2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*
     mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/
     msb12 + (2*mu*sb*power2(sa)*power2(snb)*power2(log(msb12/Q2))*(mh2*msb12*
     msb14*power2(DeltaInv(mh2,msb12,msb12)) + 2*(2*mh2*msb22 + msb12*msb22 +
     2*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mH2*(-(msb12*
     msb14*power2(DeltaInv(mH2,msb12,msb12))) + 2*mH2*power2(DeltaInv(mH2,
     msb12,msb22))*(-2*mH2*msb22 - msb12*msb22 - 2*msb24 + DeltaInv(mH2,msb12,
     msb22)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*msb22*power3(mH2) + (
     -3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*msb24 + power3(msb22)))
     )) + 2*power2(mh2)*((3*msb12*msb22 + 2*msb24)*power2(mh2) - 2*msb22*
     power3(mh2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*mh2*(5*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))))/(cb*msb12) + (2*mu*sb*
     power2(snb)*power2(log(msb22/Q2))*(2*msb22*msb24*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + msb12*(mH2*msb22*msb24*power2(DeltaInv(mH2,
     msb22,msb22)) + power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(2*mC2*mst12
     + msb22*mst12 + 2*mst14 + DeltaInv(msb22,mst12,mC2)*((3*msb22*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12))))) - 2*msb22*power2(mH2)*(
     msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))/(cb*msb12*msb22) - (
     mu*sb*log(msb12/Q2)*log(msb22/Q2)*((mH2*msb22 + msb12*msb22 + 3*msb24)*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mA2)*power2(
     DeltaInv(msb12,mA2,msb22))*(mA2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(
     msb12,mA2,msb22)*((msb12*msb22 - msb24)*power2(mA2) - msb22*power3(mA2) +
     (5*msb12 - 3*msb22)*power3(msb22) + 5*mA2*(2*msb12*msb24 + power3(msb22)))
     ) + power2(mH2)*((msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (
     5*msb12 - 3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22))))/(cb*msb12) + (ca*mu*sa*log(msb12/Q2)*(
     mh2 - mH2 - DeltaInv(mh2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (
     msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(msb22) + power3(mh2)) +
     DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*
     msb22)*power2(mH2) + 5*mH2*power2(msb22) + power3(mH2)) + 6*mh2*msb22*
     power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*msb24) + mh2*(6*msb12*msb22 +
     23*msb24) - (msb12 - 23*msb22)*power2(mh2) + power3(mh2) + power3(msb22))
     - 6*mH2*msb22*power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*msb24) + mH2*(6*
     msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mH2) + power3(mH2) +
     power3(msb22)) - 72*msb22*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(
     mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(
     5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + 72*
     msb22*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*msb22*
     power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22))))/(12.*msb12*msb22) + (
     mu*sb*log(msb12/Q2)*power2(sa)*(mh2 - mH2 - DeltaInv(mh2,msb12,msb22)*(
     msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mh2) + 5*mh2*
     power2(msb22) + power3(mh2)) + DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*
     msb22) + msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*power2(msb22) +
     power3(mH2)) + 6*mh2*msb22*power2(DeltaInv(mh2,msb12,msb22))*(-(msb12*
     msb24) + mh2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mh2) +
     power3(mh2) + power3(msb22)) - 6*mH2*msb22*power2(DeltaInv(mH2,msb12,
     msb22))*(-(msb12*msb24) + mH2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*
     msb22)*power2(mH2) + power3(mH2) + power3(msb22)) - 72*msb22*power2(mh2)*(
     -((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12
     + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)) + 72*msb22*power2(mH2)*(-((3*msb12*msb22 + 2*
     msb24)*power2(mH2)) + 2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(
     msb22) - 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,
     msb22))))/(12.*cb*msb12*msb22) - (mu*sb*power2(log(msb12/Q2))*((2*mH2*
     msb22 + msb12*msb22 + 2*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))*(2*mA2*msb22 +
     msb12*msb22 + 2*msb24 + DeltaInv(msb12,mA2,msb22)*((3*msb12*msb22 + 2*
     msb24)*power2(mA2) - 2*msb22*power3(mA2) + (3*msb12 - 2*msb22)*power3(
     msb22) + 2*mA2*(5*msb12*msb24 + power3(msb22)))) + power2(mH2)*((3*msb12*
     msb22 + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*
     power3(msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     mH2,msb12,msb22))))/(cb*msb12) + (2*mu*sb*power2(snb)*power2(log(msb12/Q2)
     )*(mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*(2*mH2*msb22 +
     msb12*msb22 + 2*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))*(-2*mC2*mst12 - msb12*mst12
     - 2*mst14 + DeltaInv(msb12,mst12,mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(
     mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(
     5*msb12*mst14 + power3(mst12)))) + 2*power2(mH2)*((3*msb12*msb22 + 2*
     msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(
     msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,
     msb22))))/(cb*msb12) + (2*ca*mu*sa*power2(snb)*power2(log(msb22/Q2))*(2*
     msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*
     power2(DeltaInv(mh2,msb22,msb22)) - 2*power2(mh2)*(msb24*power2(mh2) + (-
     2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + mH2*(-2*mH2*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) - msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 2*mH2*(
     msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)))))/msb12 + (2*mu*sb*
     power2(sa)*power2(snb)*power2(log(msb22/Q2))*(2*msb24*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,
     msb22)) - 2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22)) + mH2*(-2*mH2*msb24*power2(DeltaInv(mH2,msb12,msb22)) - msb12*
     msb24*power2(DeltaInv(mH2,msb22,msb22)) + 2*mH2*(msb24*power2(mH2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)))))/(cb*msb12) - (mu*sb*power2(log(msb22/
     Q2))*(msb22*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + msb22*
     msb24*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)) + 2*msb12*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))*(2*mC2*mst12 + msb22*mst12 + 2*mst14 +
     DeltaInv(msb22,mst12,mC2)*((3*msb22*mst12 + 2*mst14)*power2(mC2) - 2*
     mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb22*
     mst14 + power3(mst12)))) - msb22*power2(mH2)*(msb24*power2(mH2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22)) - msb22*power2(mA2)*(msb24*power2(mA2) +
     (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(msb12,mA2,msb22))))/(cb*msb12*msb22) + (mu*sb*log(msb12/
     Q2)*(mA2 + mH2 + 2*msb12 + 2*msb22 - DeltaInv(msb12,mA2,msb22)*(msb22*(-(
     msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mA2) + 5*mA2*power2(
     msb22) + power3(mA2)) - DeltaInv(mH2,msb12,msb22)*(msb22*(-(msb12*msb22) +
     msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*power2(msb22) + power3(mH2)
     ) + 6*mA2*msb22*power2(DeltaInv(msb12,mA2,msb22))*(-(msb12*msb24) + mA2*(
     6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mA2) + power3(mA2) +
     power3(msb22)) + 6*mH2*msb22*power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*
     msb24) + mH2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(mH2) +
     power3(mH2) + power3(msb22)) - 72*msb22*power2(mH2)*(-((3*msb12*msb22 + 2*
     msb24)*power2(mH2)) + 2*msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(
     msb22) - 2*mH2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,
     msb22)) - 72*msb22*power2(mA2)*(-((3*msb12*msb22 + 2*msb24)*power2(mA2)) +
     2*msb22*power3(mA2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mA2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22))))/(12.*cb*msb12*
     msb22) - (2*mu*sb*log(mC2/Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,
     mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 + msb12*
     mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*power2(
     mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-10*
     msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2) +
     3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(cb*msb12) + (
     2*cb*mt2*mu*power2(mC2)*power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*
     power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(msb22,mst12,mC2)
     )*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) - msb22*(
     mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12
     - mst22)*sb) + (2*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(log(
     mst12/Q2))*(msb22*mst14*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))) - msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(
     mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(cb*msb12*msb22) + (2*mt2*mu*power2(mC2)*power2(snb)*power2(log(
     mst12/Q2))*power3(cb)*(msb22*mst14*power2(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(-mst14 + DeltaInv(msb22,mst12,
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))) - msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*
     power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mt2*mu*
     power2(mC2)*power2(snb)*power2(log(mst12/Q2))*(-(msb22*mst14*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mst14 - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) + msb22*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 -
     mst22)*sb) + (2*mu*sb*power2(mC2)*power2(snb)*power2(log(mst12/Q2))*(-(
     msb22*mst14*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(
     msb22,mst12,mC2))*(mst14 - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) +
     msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(cb*
     msb12*msb22) + (2*mt2*mu*power2(mC2)*power2(snb)*power2(log(mst12/Q2))*
     power3(cb)*(-(msb22*mst14*power2(DeltaInv(msb12,mst12,mC2))) + msb12*
     power2(DeltaInv(msb22,mst12,mC2))*(mst14 - DeltaInv(msb22,mst12,mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))) + msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(
     mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (cb*mt2*mu*log(mst12/Q2)*
     power2(snb)*(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,
     mst12,mC2) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*
     mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) +
     msb12*DeltaInv(msb22,mst12,mC2)*(-5*mC2*mst12 + msb22*mst12 - mst14 + 6*
     mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*
     mst14 + mst12*power2(mC2) + power3(mst12)) + 72*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(
     mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) - 72*msb22*power2(mC2)*(
     mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14
     + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(6.*msb12*msb22*(
     mst12 - mst22)*sb) + (mu*sb*log(mst12/Q2)*power2(snb)*(msb22*(5*mC2*mst12
     - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - 6*mC2*msb22*power2(
     DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*
     mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,mst12,
     mC2)*(-5*mC2*mst12 + msb22*mst12 - mst14 + 6*mC2*DeltaInv(msb22,mst12,mC2)
     *(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) + 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))) - 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(6.*cb*msb12*msb22) + (mt2*mu*log(mst12/Q2)*
     power2(snb)*power3(cb)*(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*
     (-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) +
     power3(mst12)) + msb12*DeltaInv(msb22,mst12,mC2)*(-5*mC2*mst12 + msb22*
     mst12 - mst14 + 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 72*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) - 72*msb22*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(6.*
     msb12*msb22*(mst12 - mst22)*sb) + (cb*mt2*mu*log(mst12/Q2)*power2(snb)*(-(
     msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + 6*
     mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*
     mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(
     msb22,mst12,mC2)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(
     msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))) + 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)*sb) +
     (mu*sb*log(mst12/Q2)*power2(snb)*power2(snt)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb22*power2(
     DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*
     mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,mst12,
     mC2)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*
     (-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(6.*cb*msb12*msb22) + (mt2*mu*log(mst12/Q2)*
     power2(snb)*power3(cb)*(-(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))
     *(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) +
     power3(mst12)) + msb12*DeltaInv(msb22,mst12,mC2)*(5*mC2*mst12 - msb22*
     mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) + 72*msb22*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(6.*
     msb12*msb22*(mst12 - mst22)*sb) + (2*mu*sb*power2(mC2)*power2(snb)*power2(
     snt)*power2(log(msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(DeltaInv(mst22,msb12,mC2))*(-2*mC2*
     mst22 - msb12*mst22 - 2*mst24 + DeltaInv(mst22,msb12,mC2)*(-((3*msb12*
     mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)
     *power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))) + ((3*msb12*mst12
     + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2))))/(cb*msb12) + (2*cb*mt2*mu*log(mC2/Q2)*log(mst12/Q2)*power2(
     mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*
     power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(
     6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mu*sb*log(mC2/Q2)*log(mst12/Q2)*
     power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12
     + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) +
     mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*
     power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(
     6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(
     msb12*msb22) + (2*mu*sb*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(
     msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*
     msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(
     mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(cb*msb12*
     msb22) + (2*cb*mu*sb*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*
     power2(snt)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(
     msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*
     msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(
     mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22)
     + (2*mt2*mu*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*power3(cb)*(
     -(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)
     )) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 +
     mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) -
     mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 +
     power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(mC2)) + mst12*
     power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 -
     mst22)*sb) + (2*cb*mt2*mu*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(
     snb)*(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,
     mC2)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*
     mst12 - mst14 + DeltaInv(msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(
     mC2)) + mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*
     mst14 + power3(mst12)))) + msb22*((msb12*mst12 + mst14)*power2(mC2) -
     mst12*power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 -
     mst22)*sb) + (2*mu*sb*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*(
     msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2))
     + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 -
     mst14 + DeltaInv(msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) +
     mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 +
     power3(mst12)))) + msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(cb*msb12*msb22) + (2*cb*mu*
     sb*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)*(msb22*(mC2*mst12 +
     msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(
     msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2)
     + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) +
     msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 -
     mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (2*cb*mu*sb*log(mC2/Q2)*log(
     mst12/Q2)*power2(mC2)*power2(snb)*power2(snt)*(msb22*(mC2*mst12 + msb12*
     mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(
     msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(msb22,
     mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-
     msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) +
     msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 -
     mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(msb12*msb22) + (2*mt2*mu*log(mC2/Q2)*log(
     mst12/Q2)*power2(mC2)*power2(snb)*power3(cb)*(msb22*(mC2*mst12 + msb12*
     mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(
     msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(msb22,
     mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-
     msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) +
     msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 -
     mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)*sb) - (2*mu*sb*
     log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*power2(snt)*(-((3*mC2*mst12 + msb22*
     mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,
     msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)
     *(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 -
     mst22)*power3(mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(
     msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*
     power3(mst12) + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2))))/(cb*msb22) + (2*mu*sb*log(mC2/Q2)*log(msb22/Q2)*
     power2(mC2)*power2(snb)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + mst14)
     *power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(
     3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12)
     + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     ))/(cb*msb22) + (2*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(log(
     msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(2*mC2*mst22 + msb22*
     mst22 + 2*mst24 + DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22 + 2*mst24)*
     power2(mC2) - 2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*
     mC2*(5*msb22*mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))
     /(cb*msb22) + (2*mu*sb*power2(mC2)*power2(snt)*power2(log(msb22/Q2))*((2*
     mC2*mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,mst12,mC2)) +
     power2(DeltaInv(mst22,msb22,mC2))*(-2*mC2*mst22 - msb22*mst22 - 2*mst24 +
     DeltaInv(mst22,msb22,mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))) + ((3*msb22*mst12 + 2*mst14)*power2(mC2) - 2*
     mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(cb*msb22) + (
     mu*sb*log(msb22/Q2)*(msb22*(5*mH2*msb22 - msb12*msb22 + msb24)*mst12*
     DeltaInv(mH2,msb12,msb22) + msb22*(5*mA2*msb22 - msb12*msb22 + msb24)*
     mst12*DeltaInv(msb12,mA2,msb22) - 6*mA2*msb22*mst12*power2(DeltaInv(msb12,
     mA2,msb22))*(-3*mA2*msb12*msb22 - 14*mA2*msb24 - msb12*msb24 + msb22*
     power2(mA2) + power3(msb22)) - 6*mH2*msb22*mst12*power2(DeltaInv(mH2,
     msb12,msb22))*(-3*mH2*msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*
     power2(mH2) + power3(msb22)) - 72*msb22*mst12*power2(mH2)*(msb24*power2(
     mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(
     msb22)))*power3(DeltaInv(mH2,msb12,msb22)) - 72*msb22*mst12*power2(mA2)*(
     msb24*power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + 2*msb12*(mC2 + msb22
     + mst12 - DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (
     msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + 6*mC2*
     mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 72*mst12*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)))))/(12.*cb*
     msb12*msb22*mst12) + (2*cb*mt2*mu*power2(mC2)*power2(snb)*power2(log(
     mst22/Q2))*(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))) - msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*mt2*mu*power2(mC2)*power2(
     snb)*power2(log(mst22/Q2))*power3(cb)*(msb22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(-mst24 + DeltaInv(
     mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))) - msb22*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*
     cb*mt2*mu*power2(mC2)*power2(snb)*power2(log(mst22/Q2))*(-(msb22*mst24*
     power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,msb22,
     mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) + msb22*(
     mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12
     - mst22)*sb) + (2*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(log(
     mst22/Q2))*(-(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2))) + msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(
     mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))) + msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(cb*msb12*msb22) + (2*mt2*mu*power2(mC2)*power2(snb)*power2(log(
     mst22/Q2))*power3(cb)*(-(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)
     *(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*
     mst24 + power3(mst22)))) + msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*
     power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (cb*mt2*mu*log(
     mst22/Q2)*power2(snb)*(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(
     mst22,msb12,mC2) - 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*
     msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(
     mst22)) + msb12*DeltaInv(mst22,msb22,mC2)*(-5*mC2*mst22 + msb22*mst22 -
     mst24 + 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24
     - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) - 72*msb22*power2(
     mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*
     msb22*(mst12 - mst22)*sb) + (mu*sb*log(mst22/Q2)*power2(snb)*power2(snt)*(
     msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - 6*
     mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*
     mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*DeltaInv(
     mst22,msb22,mC2)*(-5*mC2*mst22 + msb22*mst22 - mst24 + 6*mC2*DeltaInv(
     mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) + 72*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))) - 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(6.*cb*msb12*msb22) + (mt2*mu*log(
     mst22/Q2)*power2(snb)*power3(cb)*(msb22*(5*mC2*mst22 - msb12*mst22 +
     mst24)*DeltaInv(mst22,msb12,mC2) - 6*mC2*msb22*power2(DeltaInv(mst22,
     msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*
     power2(mC2) + power3(mst22)) + msb12*DeltaInv(mst22,msb22,mC2)*(-5*mC2*
     mst22 + msb22*mst22 - mst24 + 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*
     msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(
     mst22)) + 72*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))) - 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*
     power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)*sb) + (cb*mt2*mu*log(
     mst22/Q2)*power2(snb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*
     DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))
     *(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) +
     power3(mst22)) + msb12*DeltaInv(mst22,msb22,mC2)*(5*mC2*mst22 - msb22*
     mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*
     mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) + 72*msb22*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*
     msb12*msb22*(mst12 - mst22)*sb) + (mt2*mu*log(mst22/Q2)*power2(snb)*
     power3(cb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,
     msb12,mC2)) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*
     mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) +
     msb12*DeltaInv(mst22,msb22,mC2)*(5*mC2*mst22 - msb22*mst22 + mst24 - 6*
     mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*
     mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(
     mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) + 72*msb22*power2(mC2)*(
     mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*msb22*(
     mst12 - mst22)*sb) + (cb*mt2*mu*log(msb12/Q2)*power2(snb)*((mC2 + msb12)*(
     mst12 - mst22) + mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) +
     mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) - mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (
     msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*
     msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (mu*sb*
     log(msb12/Q2)*power2(snb)*power2(snt)*((mC2 + msb12)*(mst12 - mst22) +
     mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2))))/(6.*cb*msb12*mst12*mst22) + (mt2*mu*log(msb12/Q2)*power2(
     snb)*power3(cb)*((mC2 + msb12)*(mst12 - mst22) + mst22*DeltaInv(msb12,
     mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2)
     + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb12,mC2)*(
     mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*
     power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb12,
     mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*
     power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 +
     23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*mst12*(mst12
     - mst22)*mst22*sb) + (cb*mt2*mu*log(msb12/Q2)*power2(snb)*(-((mC2 + msb12)
     *(mst12 - mst22)) - mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12)
     + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(
     mC2)) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) -
     (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*
     mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(
     6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (mt2*mu*
     log(msb12/Q2)*power2(snb)*power3(cb)*(-((mC2 + msb12)*(mst12 - mst22)) -
     mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22*sb) + (2*cb*mt2*mu*
     log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst22 +
     msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(
     mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) +
     (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))) +
     msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12
     + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mu*
     sb*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(snt)*(-(msb22*
     (mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (2*mt2*mu*log(mC2/
     Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power3(cb)*(-(msb22*(mC2*mst22 +
     msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(
     mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) +
     (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))) +
     msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2) + (-msb12
     + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mt2*
     mu*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*(msb22*(mC2*mst22 +
     msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 + DeltaInv(
     mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*power3(mC2)
     + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 + power3(mst22)))) +
     msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) + (msb12 -
     mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)*sb) + (2*mu*sb*
     log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(snt)*(msb22*(mC2*
     mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 +
     DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*
     power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(cb*msb12*msb22) + (2*cb*mu*
     sb*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(snt)*(msb22*(
     mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) +
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22) + (2*mt2*mu*
     log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power3(cb)*(msb22*(mC2*
     mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) + msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 - mst24 +
     DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) + mst22*
     power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)*
     sb) + (cb*mt2*mu*log(msb22/Q2)*((mC2 + msb22)*(mst12 - mst22) + mst22*
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) +
     mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))
     /(6.*msb22*mst12*(mst12 - mst22)*mst22*sb) + (cb*mt2*mu*log(msb22/Q2)*
     power2(snb)*((mC2 + msb22)*(mst12 - mst22) + mst22*DeltaInv(msb22,mst12,
     mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*
     mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb22,mC2)*(mst22*
     (-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2)
     )*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12
     - mst22)*mst22*sb) + (mu*sb*log(msb22/Q2)*power2(snt)*((mC2 + msb22)*(
     mst12 - mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) +
     mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) - mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (
     msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*
     msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(
     msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2))))/(6.*cb*msb22*mst12*mst22) - (mu*sb*log(msb22/Q2)*
     power2(snb)*power2(snt)*((mC2 + msb22)*(mst12 - mst22) + mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*cb*msb22*
     mst12*mst22) + (mt2*mu*log(msb22/Q2)*power3(cb)*((mC2 + msb22)*(mst12 -
     mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) -
     (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) -
     mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 -
     5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22*sb) + (mt2*mu*log(
     msb22/Q2)*power2(snb)*power3(cb)*((mC2 + msb22)*(mst12 - mst22) + mst22*
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) +
     mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))
     /(6.*msb22*mst12*(mst12 - mst22)*mst22*sb) + (cb*mt2*mu*log(msb22/Q2)*(-((
     mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(
     msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*
     mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(
     msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (
     msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(
     mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22*
     sb) + (cb*mt2*mu*log(msb22/Q2)*power2(snb)*(-((mC2 + msb22)*(mst12 -
     mst22)) - mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14)
     - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) +
     mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 -
     5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb22,mC2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22*sb) + (mt2*mu*log(
     msb22/Q2)*power3(cb)*(-((mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22*sb) + (mt2*mu*log(msb22/Q2)*power2(snb)*
     power3(cb)*(-((mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(msb22,mst12,
     mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*
     mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*
     (-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2)
     )*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) -
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12
     - mst22)*mst22*sb) + (ca*mu*sa*power2(Pi)*(-((3*mh2*msb22 + msb12*msb22 +
     3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mh2)*(-((
     5*msb12*msb22 + 3*msb24)*power2(mh2)) + 3*msb22*power3(mh2) + (-5*msb12 +
     3*msb22)*power3(msb22) - mh2*(14*msb12*msb24 + 3*power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))
     *(3*mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,msb22)*(14*mH2*
     msb12*msb24 + 5*msb12*msb22*power2(mH2) + 3*msb24*power2(mH2) - 3*msb22*
     power3(mH2) + 3*mH2*power3(msb22) + 5*msb12*power3(msb22) - 3*power4(
     msb22)))))/(6.*msb12) + (mu*sb*power2(Pi)*power2(sa)*(-((3*mh2*msb22 +
     msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))) +
     power2(mh2)*(-((5*msb12*msb22 + 3*msb24)*power2(mh2)) + 3*msb22*power3(
     mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - mh2*(14*msb12*msb24 + 3*
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(3*mH2*msb22 + msb12*msb22 + 3*msb24 +
     DeltaInv(mH2,msb12,msb22)*(14*mH2*msb12*msb24 + 5*msb12*msb22*power2(mH2)
     + 3*msb24*power2(mH2) - 3*msb22*power3(mH2) + 3*mH2*power3(msb22) + 5*
     msb12*power3(msb22) - 3*power4(msb22)))))/(6.*cb*msb12) + (cb*mu*sb*log(
     msb12/Q2)*log(msb22/Q2)*(-((mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)
     *power2(DeltaInv(msb12,mA2,msb22))) + power2(mA2)*((-(msb12*msb22) +
     msb24)*power2(mA2) + msb22*power3(mA2) + (-5*msb12 + 3*msb22)*power3(
     msb22) - 5*mA2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,
     msb22)) + power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 + 3*
     msb24 + msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(10*msb12*msb24*mz2 + msb12*
     msb22*power2(mz2) - msb24*power2(mz2) + 5*msb12*power3(msb22) + 5*mz2*
     power3(msb22) - msb22*power3(mz2) - 3*power4(msb22)))))/msb12 + (cb*mu*sb*
     power2(log(msb12/Q2))*((2*mA2*msb22 + msb12*msb22 + 2*msb24)*power2(mA2)*
     power2(DeltaInv(msb12,mA2,msb22)) + power2(mA2)*((3*msb12*msb22 + 2*msb24)
     *power2(mA2) - 2*msb22*power3(mA2) + (3*msb12 - 2*msb22)*power3(msb22) +
     2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) +
     power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(-(msb12*msb22) - 2*(msb24 +
     msb22*mz2) - DeltaInv(msb12,mz2,msb22)*(10*msb12*msb24*mz2 + 3*msb12*
     msb22*power2(mz2) + 2*msb24*power2(mz2) + 3*msb12*power3(msb22) + 2*mz2*
     power3(msb22) - 2*msb22*power3(mz2) - 2*power4(msb22)))))/msb12 + (cb*mu*
     sb*power2(log(msb12/Q2))*(-((2*mA2*msb22 + msb12*msb22 + 2*msb24)*power2(
     mA2)*power2(DeltaInv(msb12,mA2,msb22))) + power2(mA2)*(-((3*msb12*msb22 +
     2*msb24)*power2(mA2)) + 2*msb22*power3(mA2) + (-3*msb12 + 2*msb22)*power3(
     msb22) - 2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,
     msb22)) + power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb12*msb22 + 2*
     msb24 + 2*msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(10*msb12*msb24*mz2 + 3*
     msb12*msb22*power2(mz2) + 2*msb24*power2(mz2) + 3*msb12*power3(msb22) + 2*
     mz2*power3(msb22) - 2*msb22*power3(mz2) - 2*power4(msb22)))))/msb12 + (cb*
     mu*sb*log(msb12/Q2)*(mA2 - mz2 - DeltaInv(msb12,mA2,msb22)*(msb22*(-(
     msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mA2) + 5*mA2*power2(
     msb22) + power3(mA2)) + 6*mA2*msb22*power2(DeltaInv(msb12,mA2,msb22))*(-(
     msb12*msb24) + mA2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*msb22)*power2(
     mA2) + power3(mA2) + power3(msb22)) + DeltaInv(msb12,mz2,msb22)*(5*mz2*
     power2(msb22) - msb12*(power2(msb22) + power2(mz2)) + msb22*(msb24 + 5*
     power2(mz2)) + power3(mz2)) - 6*msb22*mz2*power2(DeltaInv(msb12,mz2,msb22)
     )*(23*msb24*mz2 + msb22*mz2*(6*msb12 + 23*mz2) - msb12*(msb24 + power2(
     mz2)) + power3(msb22) + power3(mz2)) - 72*msb22*power2(mA2)*(-((3*msb12*
     msb22 + 2*msb24)*power2(mA2)) + 2*msb22*power3(mA2) + (-3*msb12 + 2*msb22)
     *power3(msb22) - 2*mA2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     msb12,mA2,msb22)) - 72*msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))
     *(10*msb12*msb24*mz2 + 3*msb12*msb22*power2(mz2) + 2*msb24*power2(mz2) +
     3*msb12*power3(msb22) + 2*mz2*power3(msb22) - 2*msb22*power3(mz2) - 2*
     power4(msb22))))/(12.*msb12*msb22) + (cb*mu*sb*log(msb12/Q2)*(-mA2 + mz2 +
     DeltaInv(msb12,mA2,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*
     msb22)*power2(mA2) + 5*mA2*power2(msb22) + power3(mA2)) - 6*mA2*msb22*
     power2(DeltaInv(msb12,mA2,msb22))*(-(msb12*msb24) + mA2*(6*msb12*msb22 +
     23*msb24) - (msb12 - 23*msb22)*power2(mA2) + power3(mA2) + power3(msb22))
     - DeltaInv(msb12,mz2,msb22)*(5*mz2*power2(msb22) - msb12*(power2(msb22) +
     power2(mz2)) + msb22*(msb24 + 5*power2(mz2)) + power3(mz2)) + 6*msb22*mz2*
     power2(DeltaInv(msb12,mz2,msb22))*(23*msb24*mz2 + msb22*mz2*(6*msb12 + 23*
     mz2) - msb12*(msb24 + power2(mz2)) + power3(msb22) + power3(mz2)) + 72*
     msb22*power2(mA2)*(-((3*msb12*msb22 + 2*msb24)*power2(mA2)) + 2*msb22*
     power3(mA2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mA2*(5*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + 72*msb22*power2(mz2)*
     power3(DeltaInv(msb12,mz2,msb22))*(10*msb12*msb24*mz2 + 3*msb12*msb22*
     power2(mz2) + 2*msb24*power2(mz2) + 3*msb12*power3(msb22) + 2*mz2*power3(
     msb22) - 2*msb22*power3(mz2) - 2*power4(msb22))))/(12.*msb12*msb22) + (ca*
     mu*sa*log(msb22/Q2)*power2(snb)*(-((5*mh2*msb22 - msb12*msb22 + msb24)*
     DeltaInv(mh2,msb12,msb22)) - msb12*(mh2 - 6*msb22)*DeltaInv(mh2,msb22,
     msb22) + 5*mH2*msb22*DeltaInv(mH2,msb12,msb22) - msb12*msb22*DeltaInv(mH2,
     msb12,msb22) + msb24*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,
     msb22,msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,msb22) + 6*mH2*msb12*
     msb24*power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*msb22*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) -
     6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) - 6*mH2*power2(
     DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(DeltaInv(mH2,
     msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,msb12,msb22))*(-3*
     mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*power2(mh2) + power3(
     msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 3*
     power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22))
     - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 288*mh2*msb12*
     power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 72*power2(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(msb22) - 288*mH2*msb12*power3(DeltaInv(
     mH2,msb22,msb22))*power4(msb22)))/(3.*msb12) + (mu*sb*log(msb22/Q2)*
     power2(sa)*power2(snb)*(-((5*mh2*msb22 - msb12*msb22 + msb24)*DeltaInv(
     mh2,msb12,msb22)) - msb12*(mh2 - 6*msb22)*DeltaInv(mh2,msb22,msb22) + 5*
     mH2*msb22*DeltaInv(mH2,msb12,msb22) - msb12*msb22*DeltaInv(mH2,msb12,
     msb22) + msb24*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,
     msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,msb22) + 6*mH2*msb12*msb24*
     power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*msb22*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) - 6*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) - 6*mH2*power2(
     DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(DeltaInv(mH2,
     msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,msb12,msb22))*(-3*
     mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*power2(mh2) + power3(
     msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 3*
     power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22))
     - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 288*mh2*msb12*
     power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 72*power2(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(msb22) - 288*mH2*msb12*power3(DeltaInv(
     mH2,msb22,msb22))*power4(msb22)))/(3.*cb*msb12) - (4*ca*mu*sa*log(mh2/Q2)*
     log(msb22/Q2)*power2(snb)*((mh2*msb22 + msb12*msb22 + msb24)*power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22)) + power2(mh2)*((msb12*msb22 + msb24)*
     power2(mh2) - msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*
     msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + msb12*
     power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*mh2*
     DeltaInv(mh2,msb22,msb22)*power4(msb22))))/msb12 - (4*mu*sb*log(mh2/Q2)*
     log(msb22/Q2)*power2(sa)*power2(snb)*((mh2*msb22 + msb12*msb22 + msb24)*
     power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + power2(mh2)*((msb12*msb22
     + msb24)*power2(mh2) - msb22*power3(mh2) + (msb12 - msb22)*power3(msb22) +
     mh2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) +
     msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*
     mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22))))/(cb*msb12) + (4*ca*mu*sa*
     log(mH2/Q2)*log(msb22/Q2)*power2(snb)*((mH2*msb22 + msb12*msb22 + msb24)*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((msb12*msb22
     + msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) +
     mH2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) +
     msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*
     mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))))/msb12 - (4*mu*sb*log(mH2/
     Q2)*log(msb22/Q2)*power2(snb)*((mH2*msb22 + msb12*msb22 + msb24)*power2(
     mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((msb12*msb22 +
     msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) +
     mH2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) +
     msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*
     mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))))/(cb*msb12) + (4*mu*sb*log(
     mH2/Q2)*log(msb22/Q2)*power2(sa)*power2(snb)*((mH2*msb22 + msb12*msb22 +
     msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((
     msb12*msb22 + msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*
     power3(msb22) + mH2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,
     msb12,msb22)) + msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*
     power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22))))/(cb*
     msb12) + (ca*mu*sa*(mh2*msb12*DeltaInv(mh2,msb12,msb22) - 42*(3*mh2*msb22
     + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) +
     42*power2(mh2)*(-((5*msb12*msb22 + 3*msb24)*power2(mh2)) + 3*msb22*power3(
     mh2) + (-5*msb12 + 3*msb22)*power3(msb22) - mh2*(14*msb12*msb24 + 3*
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + mH2*DeltaInv(mH2,
     msb12,msb22)*(-msb12 + 42*mH2*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*
     DeltaInv(mH2,msb12,msb22) - 42*mH2*power2(DeltaInv(mH2,msb12,msb22))*(-14*
     mH2*msb12*msb24 - 5*msb12*msb22*power2(mH2) - 3*msb24*power2(mH2) + 3*
     msb22*power3(mH2) - 3*mH2*power3(msb22) - 5*msb12*power3(msb22) + 3*
     power4(msb22)))))/(6.*msb12) + (mu*sb*power2(sa)*(mh2*msb12*DeltaInv(mh2,
     msb12,msb22) - 42*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22)) + 42*power2(mh2)*(-((5*msb12*msb22 + 3*
     msb24)*power2(mh2)) + 3*msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(
     msb22) - mh2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(mh2,
     msb12,msb22)) + mH2*DeltaInv(mH2,msb12,msb22)*(-msb12 + 42*mH2*(3*mH2*
     msb22 + msb12*msb22 + 3*msb24)*DeltaInv(mH2,msb12,msb22) - 42*mH2*power2(
     DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(
     mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*power3(msb22) -
     5*msb12*power3(msb22) + 3*power4(msb22)))))/(6.*cb*msb12) + (cb*mu*sb*log(
     msb12/Q2)*log(msb22/Q2)*((mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*
     power2(DeltaInv(msb12,mA2,msb22)) + power2(mA2)*((msb12*msb22 - msb24)*
     power2(mA2) - msb22*power3(mA2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*
     mA2*(2*msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) +
     power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(-(msb12*msb22) - 3*msb24 -
     msb22*mz2 + DeltaInv(msb12,mz2,msb22)*(msb24*power2(mz2) - 5*mz2*power3(
     msb22) - msb12*(10*msb24*mz2 + msb22*power2(mz2) + 5*power3(msb22)) +
     msb22*power3(mz2) + 3*power4(msb22)))))/msb12 + (cb*mu*sb*power2(Pi)*(
     msb22*(3*mA2*msb22 + msb12*msb22 + 3*msb24)*power2(mA2)*power2(DeltaInv(
     msb12,mA2,msb22)) - msb22*(msb12*msb22 + 3*(msb24 + msb22*mz2))*power2(
     mz2)*power2(DeltaInv(msb12,mz2,msb22)) + msb22*power2(mA2)*((5*msb12*msb22
     + 3*msb24)*power2(mA2) - 3*msb22*power3(mA2) + (5*msb12 - 3*msb22)*power3(
     msb22) + mA2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(msb12,
     mA2,msb22)) - msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(14*
     msb12*msb24*mz2 + 5*msb12*msb22*power2(mz2) + 3*msb24*power2(mz2) + 5*
     msb12*power3(msb22) + 3*mz2*power3(msb22) - 3*msb22*power3(mz2) - 3*
     power4(msb22)) - 2*msb12*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(-((5*msb22*mst12 +
     3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(
     mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*
     msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*
     mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))))))/(6.*msb12*
     msb22) + (cb*mu*sb*power2(Pi)*(-(msb22*(3*mA2*msb22 + msb12*msb22 + 3*
     msb24)*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))) + msb22*(msb12*msb22
     + 3*(msb24 + msb22*mz2))*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22)) +
     msb22*power2(mA2)*(-((5*msb12*msb22 + 3*msb24)*power2(mA2)) + 3*msb22*
     power3(mA2) + (-5*msb12 + 3*msb22)*power3(msb22) - mA2*(14*msb12*msb24 +
     3*power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + msb22*power2(mz2)*
     power3(DeltaInv(msb12,mz2,msb22))*(14*msb12*msb24*mz2 + 5*msb12*msb22*
     power2(mz2) + 3*msb24*power2(mz2) + 5*msb12*power3(msb22) + 3*mz2*power3(
     msb22) - 3*msb22*power3(mz2) - 3*power4(msb22)) + 2*msb12*(-((3*mC2*mst12
     + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(
     mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))))))/(6.*msb12*msb22) + (cb*mu*sb*(-(
     mA2*msb12*msb22*DeltaInv(msb12,mA2,msb22)) + msb12*msb22*mz2*DeltaInv(
     msb12,mz2,msb22) + 42*msb22*(3*mA2*msb22 + msb12*msb22 + 3*msb24)*power2(
     mA2)*power2(DeltaInv(msb12,mA2,msb22)) - 42*msb22*(msb12*msb22 + 3*(msb24
     + msb22*mz2))*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22)) - 42*msb22*
     power2(mA2)*(-((5*msb12*msb22 + 3*msb24)*power2(mA2)) + 3*msb22*power3(
     mA2) + (-5*msb12 + 3*msb22)*power3(msb22) - mA2*(14*msb12*msb24 + 3*
     power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) - 42*msb22*power2(mz2)*
     power3(DeltaInv(msb12,mz2,msb22))*(14*msb12*msb24*mz2 + 5*msb12*msb22*
     power2(mz2) + 3*msb24*power2(mz2) + 5*msb12*power3(msb22) + 3*mz2*power3(
     msb22) - 3*msb22*power3(mz2) - 3*power4(msb22)) + 2*msb12*(-(mC2*msb22*
     DeltaInv(msb22,mst12,mC2)) + 42*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*power2(mC2)*(-((5*
     msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + mw2*DeltaInv(mw2,msb22,mst12)*(msb22 - 42*
     mw2*(msb22*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb22,mst12) - 42*
     mw2*power2(DeltaInv(mw2,msb22,mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*
     power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(
     mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))))))/(6.*msb12*msb22) + (
     cb*mu*sb*(mA2*msb12*msb22*DeltaInv(msb12,mA2,msb22) - msb12*msb22*mz2*
     DeltaInv(msb12,mz2,msb22) - 42*msb22*(3*mA2*msb22 + msb12*msb22 + 3*msb24)
     *power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)) + 42*msb22*(msb12*msb22 +
     3*(msb24 + msb22*mz2))*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22)) + 42*
     msb22*power2(mA2)*(-((5*msb12*msb22 + 3*msb24)*power2(mA2)) + 3*msb22*
     power3(mA2) + (-5*msb12 + 3*msb22)*power3(msb22) - mA2*(14*msb12*msb24 +
     3*power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + 42*msb22*power2(mz2)
     *power3(DeltaInv(msb12,mz2,msb22))*(14*msb12*msb24*mz2 + 5*msb12*msb22*
     power2(mz2) + 3*msb24*power2(mz2) + 5*msb12*power3(msb22) + 3*mz2*power3(
     msb22) - 3*msb22*power3(mz2) - 3*power4(msb22)) + 2*msb12*(mC2*msb22*
     DeltaInv(msb22,mst12,mC2) - 42*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 42*power2(mC2)*(-((5*
     msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + mw2*DeltaInv(mw2,msb22,mst12)*(-msb22 + 42*
     mw2*(msb22*mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb22,mst12) + 42*
     mw2*power2(DeltaInv(mw2,msb22,mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*
     power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(
     mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))))))/(6.*msb12*msb22) + (
     cb*mu*sb*power2(log(msb22/Q2))*(msb22*msb24*power2(mA2)*power2(DeltaInv(
     msb12,mA2,msb22)) - msb22*msb24*power2(mz2)*power2(DeltaInv(msb12,mz2,
     msb22)) - msb22*power2(mA2)*(msb24*power2(mA2) + (-2*msb12 + msb22)*
     power3(msb22) - 2*mA2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(
     msb12,mA2,msb22)) + msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(
     msb24*power2(mz2) - 2*mz2*power3(msb22) - 2*msb12*(msb24*mz2 + power3(
     msb22)) + power4(msb22)) + 2*msb12*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((3*msb22*
     mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*
     power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(
     msb22*mst12) - 2*(mst14 + mst12*mw2) - DeltaInv(mw2,msb22,mst12)*(10*
     msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))))))/(msb12*msb22) + (cb*mu*sb*power2(log(msb22/Q2))*(-(
     msb22*msb24*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))) + msb22*msb24*
     power2(mz2)*power2(DeltaInv(msb12,mz2,msb22)) + msb22*power2(mA2)*(msb24*
     power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*(msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) - msb22*power2(mz2)*
     power3(DeltaInv(msb12,mz2,msb22))*(msb24*power2(mz2) - 2*mz2*power3(msb22)
     - 2*msb12*(msb24*mz2 + power3(msb22)) + power4(msb22)) + 2*msb12*(-((2*
     mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))))))/(msb12*msb22) - (mu*sb*power2(Pi)
     *(msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) + msb22*(3*mA2*msb22 + msb12*msb22 + 3*msb24)*power2(
     mA2)*power2(DeltaInv(msb12,mA2,msb22)) + msb22*power2(mH2)*((5*msb12*msb22
     + 3*msb24)*power2(mH2) - 3*msb22*power3(mH2) + (5*msb12 - 3*msb22)*power3(
     msb22) + mH2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(mH2,
     msb12,msb22)) + msb22*power2(mA2)*((5*msb12*msb22 + 3*msb24)*power2(mA2) -
     3*msb22*power3(mA2) + (5*msb12 - 3*msb22)*power3(msb22) + mA2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + 2*msb12*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(3*mC2*mst12 + msb22*mst12 +
     3*mst14 + DeltaInv(msb22,mst12,mC2)*(14*mC2*msb22*mst14 + 5*msb22*mst12*
     power2(mC2) + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*power3(
     mst12) + 5*msb22*power3(mst12) - 3*power4(mst12)))))/(6.*cb*msb12*msb22) +
     (cb*mu*sb*power2(Pi)*power2(snb)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*
     mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12
     + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) +
     msb22*power2(mC2)*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(
     mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(-((5*
     msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(msb22*(msb12*mst12 + 3*(mst14
     + mst12*mw2))*power2(DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,
     msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*
     power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*
     power3(mw2) - 3*power4(mst12))) - msb22*power3(DeltaInv(mw2,msb12,mst12))*
     (14*msb12*mst14*mw2 + 5*msb12*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*
     msb12*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*
     power4(mst12)))))/(3.*msb12*msb22) + (cb*mu*sb*power2(Pi)*power2(snb)*(-(
     msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb12,mst12,mC2))) + msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) + msb22*power2(mC2)*(-((5*msb12*
     mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)
     *power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + msb12*power2(mC2)*((5*msb22*mst12 + 3*mst14)*power2(
     mC2) - 3*mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + mC2*(14*
     msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(msb22*(msb12*mst12 + 3*(mst14 + mst12*mw2))*power2(DeltaInv(
     mw2,msb12,mst12)) - msb12*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 +
     3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*
     msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*
     mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(14*msb12*mst14*mw2 + 5*msb12*mst12*
     power2(mw2) + 3*mst14*power2(mw2) + 5*msb12*power3(mst12) + 3*mw2*power3(
     mst12) - 3*mst12*power3(mw2) - 3*power4(mst12)))))/(3.*msb12*msb22) + (2*
     cb*mu*sb*log(msb12/Q2)*log(mst12/Q2)*power2(snb)*(-((mC2*mst12 + msb12*
     mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2))) + power2(
     mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb12
     + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))
     *(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*
     mst14*mw2 + msb12*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb12*power3(
     mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/
     msb12 + (2*cb*mu*sb*log(msb12/Q2)*log(mst12/Q2)*power2(snb)*power2(snt)*(-
     ((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + power2(mC2)*((-(msb12*mst12) + mst14)*power2(mC2) + mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb12,mst12)*(10*msb12*mst14*mw2 + msb12*mst12*power2(mw2) - mst14*
     power2(mw2) + 5*msb12*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(
     mw2) - 3*power4(mst12)))))/msb12 + (2*cb*mu*sb*log(msb22/Q2)*log(mst12/Q2)
     *(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(
     mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*
     power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(
     mw2) - 3*power4(mst12)))))/msb22 + (2*cb*mu*sb*log(msb22/Q2)*log(mst12/Q2)
     *power2(snb)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*
     power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 +
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + msb22*mst12*
     power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12) + 5*mw2*power3(
     mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/msb22 + (2*cb*mu*sb*log(
     msb22/Q2)*log(mst12/Q2)*power2(snt)*(-((mC2*mst12 + msb22*mst12 + 3*mst14)
     *power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*((-(msb22*
     mst12) + mst14)*power2(mC2) + mst12*power3(mC2) + (-5*msb22 + 3*mst12)*
     power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(msb22*
     mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*
     mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) + 5*msb22*power3(mst12)
     + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*power4(mst12)))))/msb22 + (
     2*cb*mu*sb*log(msb22/Q2)*log(mst12/Q2)*power2(snb)*power2(snt)*(-((mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))) + power2(mC2)*((-(msb22*mst12) + mst14)*power2(mC2) + mst12*power3(
     mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - 5*mC2*(2*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + mst12*mw2 + DeltaInv(mw2,msb22,
     mst12)*(10*msb22*mst14*mw2 + msb22*mst12*power2(mw2) - mst14*power2(mw2) +
     5*msb22*power3(mst12) + 5*mw2*power3(mst12) - mst12*power3(mw2) - 3*
     power4(mst12)))))/msb22 + (2*cb*mu*sb*power2(snb)*power2(log(msb12/Q2))*((
     2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) + power2(mC2)*((3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*
     mst12*power3(mC2) + (3*msb12 - 2*mst12)*power3(mst12) + 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(mw2)*
     power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 2*(mst14 + mst12*mw2)
     - DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(
     mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) -
     2*mst12*power3(mw2) - 2*power4(mst12)))))/msb12 + (2*cb*mu*sb*power2(snb)*
     power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)
     *power2(DeltaInv(msb12,mst12,mC2))) + power2(mC2)*(-((3*msb12*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     2*mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12)))))/msb12 + (2*
     cb*mu*sb*power2(snb)*power2(log(msb22/Q2))*((2*mC2*mst12 + msb22*mst12 +
     2*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((3*
     msb22*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*
     mst12)*power3(mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))
     *(-(msb22*mst12) - 2*(mst14 + mst12*mw2) - DeltaInv(mw2,msb22,mst12)*(10*
     msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12)))))/msb22 + (2*cb*mu*sb*power2(snb)*power2(log(msb22/Q2))*(-
     ((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12)))))/msb22 + (cb*mu*sb*log(msb22/Q2)*(
     msb22*(5*mA2*msb22 - msb12*msb22 + msb24)*mst12*DeltaInv(msb12,mA2,msb22)
     + msb22*mst12*(msb12*msb22 - msb24 - 5*msb22*mz2)*DeltaInv(msb12,mz2,
     msb22) + 6*msb22*mst12*mz2*power2(DeltaInv(msb12,mz2,msb22))*(msb22*mz2*(-
     3*msb12 + mz2) - msb24*(msb12 + 14*mz2) + power3(msb22)) - 6*mA2*msb22*
     mst12*power2(DeltaInv(msb12,mA2,msb22))*(-3*mA2*msb12*msb22 - 14*mA2*msb24
     - msb12*msb24 + msb22*power2(mA2) + power3(msb22)) - 72*msb22*mst12*
     power2(mA2)*(msb24*power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*
     (msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) + 72*
     msb22*mst12*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(msb24*power2(
     mz2) - 2*mz2*power3(msb22) - 2*msb12*(msb24*mz2 + power3(msb22)) + power4(
     msb22)) - 2*msb12*(-mC2 + mw2 + DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*
     mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) +
     power3(mC2)) - 6*mC2*mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*
     mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) +
     power3(mC2) + power3(mst12)) - DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(
     mst12) - msb22*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(
     mw2)) + power3(mw2)) + 6*mst12*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*
     mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) +
     power3(mst12) + power3(mw2)) + 72*mst12*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + 72*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*
     msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12)))))/(12.*msb12*msb22*mst12) + (cb*mu*sb*log(msb22/Q2)*(-(
     msb22*(5*mA2*msb22 - msb12*msb22 + msb24)*mst12*DeltaInv(msb12,mA2,msb22))
     + msb22*mst12*(-(msb12*msb22) + msb24 + 5*msb22*mz2)*DeltaInv(msb12,mz2,
     msb22) - 6*msb22*mst12*mz2*power2(DeltaInv(msb12,mz2,msb22))*(msb22*mz2*(-
     3*msb12 + mz2) - msb24*(msb12 + 14*mz2) + power3(msb22)) + 6*mA2*msb22*
     mst12*power2(DeltaInv(msb12,mA2,msb22))*(-3*mA2*msb12*msb22 - 14*mA2*msb24
     - msb12*msb24 + msb22*power2(mA2) + power3(msb22)) + 72*msb22*mst12*
     power2(mA2)*(msb24*power2(mA2) + (-2*msb12 + msb22)*power3(msb22) - 2*mA2*
     (msb12*msb24 + power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)) - 72*
     msb22*mst12*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(msb24*power2(
     mz2) - 2*mz2*power3(msb22) - 2*msb12*(msb24*mz2 + power3(msb22)) + power4(
     msb22)) + 2*msb12*(-mC2 + mw2 + DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*
     mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) +
     power3(mC2)) - 6*mC2*mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*
     mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) +
     power3(mC2) + power3(mst12)) - DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(
     mst12) - msb22*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(
     mw2)) + power3(mw2)) + 6*mst12*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*
     mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) +
     power3(mst12) + power3(mw2)) + 72*mst12*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + 72*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*
     msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12)))))/(12.*msb12*msb22*mst12) + (2*cb*mu*sb*power2(snb)*
     power2(log(mst12/Q2))*(-(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) +
     msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) -
     2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) -
     msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(
     mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/(msb12*msb22) + (2*cb*mu*sb*power2(snb)*power2(snt)*
     power2(log(mst12/Q2))*(-(msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) +
     msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) -
     2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) -
     msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*(msb22*mst14*power2(DeltaInv(mw2,msb12,mst12)) - msb12*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12))) + msb22*power3(DeltaInv(mw2,msb12,mst12))*(-(mst14*power2(
     mw2)) + 2*mw2*power3(mst12) + 2*msb12*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/(msb12*msb22) + (2*cb*mu*sb*power2(log(mst12/Q2))*(-(
     mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/msb22 + (2*cb*mu*sb*power2(snt)*power2(log(mst12/Q2))*(-
     (mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))) + power2(mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12))*(mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*
     power2(mw2)) + 2*mw2*power3(mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) -
     power4(mst12)))))/msb22 + (cb*mu*sb*log(msb12/Q2)*power2(snb)*(mC2 - mw2 -
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + 6*mC2*mst12*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + DeltaInv(mw2,msb12,mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) +
     power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) - 6*mst12*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*
     mw2) - msb12*(mst14 + power2(mw2)) + power3(mst12) + power3(mw2)) - 72*
     mst12*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*mst12*power2(mw2)*
     power3(DeltaInv(mw2,msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*
     power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(
     mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))))/(6.*msb12*mst12) + (cb*
     mu*sb*log(msb12/Q2)*power2(snb)*(-mC2 + mw2 + DeltaInv(msb12,mst12,mC2)*(
     mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) - 6*mC2*mst12*power2(DeltaInv(msb12,mst12,
     mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*
     mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - DeltaInv(mw2,msb12,
     mst12)*(5*mw2*power2(mst12) - msb12*(power2(mst12) + power2(mw2)) + mst12*
     (mst14 + 5*power2(mw2)) + power3(mw2)) + 6*mst12*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*(23*mst14*mw2 + mst12*mw2*(6*msb12 + 23*mw2) - msb12*(mst14
     + power2(mw2)) + power3(mst12) + power3(mw2)) + 72*mst12*power2(mC2)*(-((
     3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 +
     2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 72*mst12*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*(10*msb12*mst14*mw2 + 3*msb12*mst12*power2(mw2) + 2*mst14*
     power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*
     power3(mw2) - 2*power4(mst12))))/(6.*msb12*mst12) + (cb*mu*sb*log(msb22/
     Q2)*power2(snb)*(mC2 - mw2 - DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*
     mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) +
     power3(mC2)) + 6*mC2*mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*
     mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) +
     power3(mC2) + power3(mst12)) + DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(
     mst12) - msb22*(power2(mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(
     mw2)) + power3(mw2)) - 6*mst12*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*
     mst14*mw2 + mst12*mw2*(6*msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) +
     power3(mst12) + power3(mw2)) - 72*mst12*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) - 72*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*
     msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*
     msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*
     power4(mst12))))/(6.*msb22*mst12) + (cb*mu*sb*log(msb22/Q2)*power2(snb)*(-
     mC2 + mw2 + DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (
     msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - 6*mC2*
     mst12*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - DeltaInv(mw2,msb22,mst12)*(5*mw2*power2(mst12) - msb22*(power2(
     mst12) + power2(mw2)) + mst12*(mst14 + 5*power2(mw2)) + power3(mw2)) + 6*
     mst12*mw2*power2(DeltaInv(mw2,msb22,mst12))*(23*mst14*mw2 + mst12*mw2*(6*
     msb22 + 23*mw2) - msb22*(mst14 + power2(mw2)) + power3(mst12) + power3(
     mw2)) + 72*mst12*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(10*msb22*mst14*mw2 + 3*
     msb22*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))))/(6.*msb22*
     mst12) + (2*cb*mu*sb*power2(snb)*power2(log(mst12/Q2))*(msb22*mst14*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12)
     - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(msb12*msb22) +
     (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(mst12/Q2))*(msb22*mst14*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst14*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + msb12*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + power2(mw2)*(-(msb22*mst14*power2(
     DeltaInv(mw2,msb12,mst12))) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst14 + DeltaInv(mw2,msb22,mst12)*(-(mst14*power2(mw2)) + 2*mw2*power3(
     mst12) + 2*msb22*(mst14*mw2 + power3(mst12)) - power4(mst12))) + msb22*
     power3(DeltaInv(mw2,msb12,mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12)
     - 2*msb12*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(msb12*msb22) +
     (2*cb*mu*sb*power2(log(mst12/Q2))*(mst14*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) - power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-mst14
     + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*
     msb22*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/msb22 + (2*cb*mu*sb*
     power2(snt)*power2(log(mst12/Q2))*(mst14*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) - power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-mst14
     + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*
     msb22*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/msb22 + (cb*mu*sb*
     log(mst12/Q2)*((5*mC2*mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,mst12,
     mC2) - 6*mC2*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - DeltaInv(mw2,
     msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,
     msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) +
     power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*
     power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) +
     power4(mst12)))))/(6.*msb22) + (cb*mu*sb*log(mst12/Q2)*power2(snt)*((5*
     mC2*mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - 6*mC2*power2(
     DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*
     mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - DeltaInv(mw2,msb22,mst12)*(-(
     msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,msb22,mst12)*(
     mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) + power3(mst12)) - 72*
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*power2(mw2) - 2*mw2*
     power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) + power4(mst12)))))/(
     6.*msb22) + (cb*mu*sb*log(mst12/Q2)*(-((5*mC2*mst12 - msb22*mst12 + mst14)
     *DeltaInv(msb22,mst12,mC2)) + 6*mC2*power2(DeltaInv(msb22,mst12,mC2))*(-3*
     mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(
     mst12)) + 72*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(
     mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,
     mC2)) + DeltaInv(mw2,msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 -
     6*mw2*DeltaInv(mw2,msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22
     + 14*mw2) + power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12))*(mst14*power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 +
     power3(mst12)) + power4(mst12)))))/(6.*msb22) + (cb*mu*sb*log(mst12/Q2)*
     power2(snt)*(-((5*mC2*mst12 - msb22*mst12 + mst14)*DeltaInv(msb22,mst12,
     mC2)) + 6*mC2*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 72*power2(
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + DeltaInv(mw2,
     msb22,mst12)*(-(msb22*mst12) + mst14 + 5*mst12*mw2 - 6*mw2*DeltaInv(mw2,
     msb22,mst12)*(mst12*mw2*(-3*msb22 + mw2) - mst14*(msb22 + 14*mw2) +
     power3(mst12)) - 72*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(mst14*
     power2(mw2) - 2*mw2*power3(mst12) - 2*msb22*(mst14*mw2 + power3(mst12)) +
     power4(mst12)))))/(6.*msb22) + (2*cb*mu*sb*log(msb12/Q2)*log(mst12/Q2)*
     power2(snb)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb12 + (2*cb*mu*sb*log(msb12/Q2)*log(mst12/Q2)
     *power2(snb)*power2(snt)*((mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((msb12*mst12 - mst14)*
     power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*
     mC2*(2*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12))*(-(msb12*mst12) - 3*mst14 -
     mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(mst14*power2(mw2) - 5*mw2*power3(
     mst12) - msb12*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) +
     mst12*power3(mw2) + 3*power4(mst12)))))/msb12 + (2*cb*mu*sb*log(msb22/Q2)*
     log(mst12/Q2)*((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((msb22*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb22 + (2*cb*mu*sb*log(msb22/Q2)*log(mst12/Q2)
     *power2(snb)*((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((msb22*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb22 + (2*cb*mu*sb*log(msb22/Q2)*log(mst12/Q2)
     *power2(snt)*((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((msb22*mst12 - mst14)*power2(
     mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*
     msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*mst14 - mst12*
     mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*power3(mst12) -
     msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) + mst12*power3(
     mw2) + 3*power4(mst12)))))/msb22 + (2*cb*mu*sb*log(msb22/Q2)*log(mst12/Q2)
     *power2(snb)*power2(snt)*((mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*((msb22*mst12 - mst14)*
     power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*
     mC2*(2*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(-(msb22*mst12) - 3*mst14 -
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(mst14*power2(mw2) - 5*mw2*power3(
     mst12) - msb22*(10*mst14*mw2 + mst12*power2(mw2) + 5*power3(mst12)) +
     mst12*power3(mw2) + 3*power4(mst12)))))/msb22 - (mu*sb*power2(mC2)*power2(
     Pi)*power2(snt)*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(
     msb22,mst12,mC2))) + (-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(DeltaInv(
     mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,
     msb22,mC2)*(14*mC2*msb22*mst24 + 5*msb22*mst22*power2(mC2) + 3*mst24*
     power2(mC2) - 3*mst22*power3(mC2) + 3*mC2*power3(mst22) + 5*msb22*power3(
     mst22) - 3*power4(mst22)))))/(3.*cb*msb22) + (cb*mu*sb*power2(Pi)*power2(
     snt)*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))) + (3*mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((5*msb22*mst12 + 3*
     mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(
     mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mC2)*((5*msb22*mst22 + 3*mst24)*power2(mC2) - 3*
     mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + mC2*(14*msb22*
     mst24 + 3*power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*
     (-((msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22))
     ) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2
     + DeltaInv(mw2,msb22,mst12)*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(
     mw2) + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) -
     3*mst12*power3(mw2) - 3*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22)
     )*(14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) +
     5*msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*
     power4(mst22)))))/(3.*msb22) + (cb*mu*sb*power2(Pi)*power2(snt)*((3*mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2)) - (3*mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + power2(mC2)*((5*msb22*mst12 + 3*mst14)*power2(mC2) -
     3*mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + power2(mC2)*
     (-((5*msb22*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-5*
     msb22 + 3*mst22)*power3(mst22) - mC2*(14*msb22*mst24 + 3*power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*((msb22*mst22 + 3*(mst24 +
     mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) - power2(DeltaInv(mw2,msb22,
     mst12))*(msb22*mst12 + 3*mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(
     14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(mw2) + 5*
     msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) - 3*
     power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*mst24*mw2 +
     5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*power3(mst22) +
     3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(mst22)))))/(3.*msb22)
     + (cb*mu*sb*power2(snt)*(mC2*msb22*DeltaInv(msb22,mst12,mC2) - mC2*msb22*
     DeltaInv(mst22,msb22,mC2) - 42*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 42*(3*mC2*mst22 + msb22*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 42*
     power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(
     mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 42*power2(mC2)*(-((5*
     msb22*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-5*msb22 + 3*
     mst22)*power3(mst22) - mC2*(14*msb22*mst24 + 3*power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + mw2*(msb22*DeltaInv(mst22,mw2,msb22) - 42*
     mw2*(msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)
     ) + DeltaInv(mw2,msb22,mst12)*(-msb22 + 42*mw2*(msb22*mst12 + 3*mst14 + 3*
     mst12*mw2)*DeltaInv(mw2,msb22,mst12) + 42*mw2*power2(DeltaInv(mw2,msb22,
     mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2) + 3*mst14*power2(
     mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*mst12*power3(mw2) -
     3*power4(mst12))) - 42*mw2*power3(DeltaInv(mst22,mw2,msb22))*(14*msb22*
     mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(mw2) + 5*msb22*
     power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) - 3*power4(
     mst22)))))/(3.*msb22) + (cb*mu*sb*power2(snt)*(-(mC2*msb22*DeltaInv(msb22,
     mst12,mC2)) + mC2*msb22*DeltaInv(mst22,msb22,mC2) + 42*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 42*
     (3*mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) - 42*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) +
     3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 42*power2(
     mC2)*(-((5*msb22*mst22 + 3*mst24)*power2(mC2)) + 3*mst22*power3(mC2) + (-
     5*msb22 + 3*mst22)*power3(mst22) - mC2*(14*msb22*mst24 + 3*power3(mst22)))
     *power3(DeltaInv(mst22,msb22,mC2)) + mw2*(-(msb22*DeltaInv(mst22,mw2,
     msb22)) + 42*mw2*(msb22*mst22 + 3*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb22)) - DeltaInv(mw2,msb22,mst12)*(-msb22 + 42*mw2*(msb22*
     mst12 + 3*mst14 + 3*mst12*mw2)*DeltaInv(mw2,msb22,mst12) + 42*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*(14*msb22*mst14*mw2 + 5*msb22*mst12*power2(mw2)
     + 3*mst14*power2(mw2) + 5*msb22*power3(mst12) + 3*mw2*power3(mst12) - 3*
     mst12*power3(mw2) - 3*power4(mst12))) + 42*mw2*power3(DeltaInv(mst22,mw2,
     msb22))*(14*msb22*mst24*mw2 + 5*msb22*mst22*power2(mw2) + 3*mst24*power2(
     mw2) + 5*msb22*power3(mst22) + 3*mw2*power3(mst22) - 3*mst22*power3(mw2) -
     3*power4(mst22)))))/(3.*msb22) + (2*cb*mu*sb*log(msb12/Q2)*log(mst22/Q2)*
     power2(snb)*power2(snt)*(-((mC2*mst22 + msb12*mst22 + 3*mst24)*power2(mC2)
     *power2(DeltaInv(mst22,msb12,mC2))) + power2(mC2)*((-(msb12*mst22) +
     mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb12 + 3*mst22)*power3(
     mst22) - 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))*(msb12*mst22 +
     3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(10*msb12*mst24*mw2 +
     msb12*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb12*power3(mst22) + 5*
     mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/msb12 + (2*cb*
     mu*sb*log(msb22/Q2)*log(mst22/Q2)*power2(snt)*(-((mC2*mst22 + msb22*mst22
     + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*((
     -(msb22*mst22) + mst24)*power2(mC2) + mst22*power3(mC2) + (-5*msb22 + 3*
     mst22)*power3(mst22) - 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))
     *(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(10*msb22*
     mst24*mw2 + msb22*mst22*power2(mw2) - mst24*power2(mw2) + 5*msb22*power3(
     mst22) + 5*mw2*power3(mst22) - mst22*power3(mw2) - 3*power4(mst22)))))/
     msb22 + (2*cb*mu*sb*log(msb22/Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*(-
     ((mC2*mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2))) + power2(mC2)*((-(msb22*mst22) + mst24)*power2(mC2) + mst22*
     power3(mC2) + (-5*msb22 + 3*mst22)*power3(mst22) - 5*mC2*(2*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22))*(msb22*mst22 + 3*mst24 + mst22*mw2 + DeltaInv(
     mst22,mw2,msb22)*(10*msb22*mst24*mw2 + msb22*mst22*power2(mw2) - mst24*
     power2(mw2) + 5*msb22*power3(mst22) + 5*mw2*power3(mst22) - mst22*power3(
     mw2) - 3*power4(mst22)))))/msb22 + (2*cb*mu*sb*power2(snb)*power2(snt)*
     power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(mC2)
     *power2(DeltaInv(msb12,mst12,mC2))) + (2*mC2*mst22 + msb12*mst22 + 2*
     mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*(-((3*
     msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*((3*msb12*mst22 + 2*mst24)*
     power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) + 2*
     mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*(-((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(
     mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*
     mst14 + 2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*
     msb12*mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*
     mw2*power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) - power3(
     DeltaInv(mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2)
     + 2*mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*
     mst22*power3(mw2) - 2*power4(mst22)))))/msb12 + (2*cb*mu*sb*power2(snb)*
     power2(snt)*power2(log(msb12/Q2))*((2*mC2*mst12 + msb12*mst12 + 2*mst14)*
     power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - (2*mC2*mst22 + msb12*mst22
     + 2*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(mC2)*((
     3*msb12*mst12 + 2*mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb12 - 2*
     mst12)*power3(mst12) + 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) +
     power2(mw2)*((msb12*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,
     mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 + 2*mst14 +
     2*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(10*msb12*mst14*mw2 + 3*msb12*
     mst12*power2(mw2) + 2*mst14*power2(mw2) + 3*msb12*power3(mst12) + 2*mw2*
     power3(mst12) - 2*mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb12))*(10*msb12*mst24*mw2 + 3*msb12*mst22*power2(mw2) + 2*
     mst24*power2(mw2) + 3*msb12*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*
     power3(mw2) - 2*power4(mst22)))))/msb12 + (2*cb*mu*sb*power2(snt)*power2(
     log(msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)
     *power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) -
     2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-
     ((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)))
     + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(
     msb22/Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))) + (2*mC2*mst22 + msb22*mst22 + 2*mst24)*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*(-((3*msb22*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2)) + power2(mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) -
     2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-
     ((msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)))
     + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) - power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (2*cb*mu*sb*power2(snt)*power2(log(msb22/Q2))*((
     2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((3*msb22*mst12 + 2*mst14)*
     power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(mst12) + 2*
     mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(
     mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*((msb22*mst22 +
     2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) - power2(
     DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(
     msb22/Q2))*((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) - (2*mC2*mst22 + msb22*mst22 + 2*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(mC2)*((3*msb22*mst12 + 2*
     mst14)*power2(mC2) - 2*mst12*power3(mC2) + (3*msb22 - 2*mst12)*power3(
     mst12) + 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*((
     msb22*mst22 + 2*(mst24 + mst22*mw2))*power2(DeltaInv(mst22,mw2,msb22)) -
     power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + 2*mst14 + 2*mst12*mw2 +
     DeltaInv(mw2,msb22,mst12)*(10*msb22*mst14*mw2 + 3*msb22*mst12*power2(mw2)
     + 2*mst14*power2(mw2) + 3*msb22*power3(mst12) + 2*mw2*power3(mst12) - 2*
     mst12*power3(mw2) - 2*power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*
     (10*msb22*mst24*mw2 + 3*msb22*mst22*power2(mw2) + 2*mst24*power2(mw2) + 3*
     msb22*power3(mst22) + 2*mw2*power3(mst22) - 2*mst22*power3(mw2) - 2*
     power4(mst22)))))/msb22 + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(
     mst22/Q2))*(-(msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + msb22*power2(
     mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) - msb12*power2(
     mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(
     msb22*mst24*power2(DeltaInv(mst22,mw2,msb12)) - msb12*power2(DeltaInv(
     mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2))
     + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(
     mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(-(mst24*power2(mw2)) +
     2*mw2*power3(mst22) + 2*msb12*(mst24*mw2 + power3(mst22)) - power4(mst22))
     )))/(msb12*msb22) + (2*cb*mu*sb*power2(snt)*power2(log(mst22/Q2))*(-(
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))) + power2(mC2)*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*
     power2(mw2)) + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) -
     power4(mst22)))))/msb22 + (2*cb*mu*sb*log(msb12/Q2)*log(mw2/Q2)*power2(
     mw2)*power2(snb)*power2(snt)*((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(
     DeltaInv(mst22,mw2,msb12)) - power2(DeltaInv(mw2,msb12,mst12))*(msb12*
     mst12 + mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(
     mw2) - mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) +
     power3(mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(
     mst22,mw2,msb12))*(5*mst24*power2(mw2) - mw2*power3(mst22) + msb12*(10*
     mst24*mw2 + 5*mst22*power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) -
     power4(mst22))))/msb12 + (2*cb*mu*sb*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)
     *power2(snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(
     5*mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb22 + (2*cb*mu*sb*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power2(snt)*((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) - power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(
     5*mst24*power2(mw2) - mw2*power3(mst22) + msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) - 3*mst22*power3(mw2) - power4(mst22))))/
     msb22 + (cb*mt2*mu*power2(mC2)*power2(Pi)*power2(snb)*(msb22*(3*mC2*mst12
     + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 3*mC2*
     msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(DeltaInv(mst22,msb12,
     mC2)) + 3*mC2*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*
     mst22*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(DeltaInv(
     mst22,msb22,mC2)) + msb22*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*
     mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*mC2*
     msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst22*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 3*mC2*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) +
     5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*
     msb12*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst22*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*mC2*msb12*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*
     mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*
     mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*
     power4(mst12)) + 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     3*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*
     (mst12 - mst22)*sb) + (mu*sb*power2(mC2)*power2(Pi)*power2(snb)*power2(
     snt)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(
     msb22,mst12,mC2)) - 3*mC2*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) -
     msb12*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*
     power2(DeltaInv(mst22,msb12,mC2)) + 3*mC2*msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2)) + msb12*msb22*mst22*power2(DeltaInv(mst22,msb22,mC2)) +
     3*msb12*mst24*power2(DeltaInv(mst22,msb22,mC2)) + msb22*((5*msb12*mst12 +
     3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(
     mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) - 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) -
     5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst22*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*mC2*msb22*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb22,mC2)) + 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 3*msb12*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     - 3*msb12*mst22*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*mC2*
     msb12*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + msb12*power3(DeltaInv(
     msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*
     mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*
     power3(mst12) + 3*power4(mst12)) + 3*msb22*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mst22) - 3*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mst22)))/(3.*cb*msb12*msb22) + (mt2*mu*power2(mC2)*power2(Pi)*power2(snb)*
     power3(cb)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(
     msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     DeltaInv(msb22,mst12,mC2)) - 3*mC2*msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) - msb12*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - 3*
     msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + 3*mC2*msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*power2(DeltaInv(mst22,
     msb22,mC2)) + 3*msb12*mst24*power2(DeltaInv(mst22,msb22,mC2)) + msb22*((5*
     msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*
     mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 14*mC2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) - 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     + 3*msb22*mst22*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*mC2*
     msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 14*mC2*msb12*msb22*
     mst24*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 3*mC2*msb12*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*
     mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*
     power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 3*msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*(mst12 - mst22)*sb) + (cb*mt2*
     mu*power2(mC2)*power2(Pi)*power2(snb)*(-(msb22*(3*mC2*mst12 + msb12*mst12
     + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) + 3*mC2*msb22*
     mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(DeltaInv(mst22,msb12,
     mC2)) - 3*mC2*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - msb12*msb22*
     mst22*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*power2(DeltaInv(
     mst22,msb22,mC2)) + msb22*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*
     mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 14*mC2*
     msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst22*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 3*mC2*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) - 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) -
     5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*
     msb12*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*mC2*msb12*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(14*
     mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2) + 3*mst14*power2(mC2) - 3*
     mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*msb22*power3(mst12) - 3*
     power4(mst12)) - 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) +
     3*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*
     (mst12 - mst22)*sb) + (mt2*mu*power2(mC2)*power2(Pi)*power2(snb)*power3(
     cb)*(-(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     DeltaInv(msb22,mst12,mC2)) + 3*mC2*msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + 3*
     msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 3*mC2*msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst22*power2(DeltaInv(mst22,
     msb22,mC2)) - 3*msb12*mst24*power2(DeltaInv(mst22,msb22,mC2)) + msb22*(-((
     5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 +
     3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) + 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) + 3*msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     - 3*msb22*mst22*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*mC2*
     msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 14*mC2*msb12*msb22*
     mst24*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - 3*mC2*msb12*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     msb12*power3(DeltaInv(msb22,mst12,mC2))*(14*mC2*msb22*mst14 + 5*msb22*
     mst12*power2(mC2) + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*
     power3(mst12) + 5*msb22*power3(mst12) - 3*power4(mst12)) - 3*msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*(mst12 - mst22)*sb) + (mC2*mu*
     sb*power2(snb)*power2(snt)*(-(msb12*msb22*DeltaInv(msb12,mst12,mC2)) +
     msb12*msb22*DeltaInv(msb22,mst12,mC2) + msb12*msb22*DeltaInv(mst22,msb12,
     mC2) - msb12*msb22*DeltaInv(mst22,msb22,mC2) + 42*mC2*msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2)) - 42*mC2*msb12*(
     3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) -
     42*mC2*msb12*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - 126*mC2*
     msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst22*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*mC2*msb12*msb22*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 126*mC2*msb12*mst24*power2(DeltaInv(mst22,
     msb22,mC2)) + 126*msb12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)
     ) - 42*mC2*msb22*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*
     mst22*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*
     power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*mC2*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*mst22*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*mC2*msb12*msb22*power3(mst22)
     *power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + 126*msb22*mst22*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2)
     )*power4(mC2) + 42*mC2*msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) + 126*mC2*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     126*mC2*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*cb*
     msb12*msb22) + (2*cb*mu*sb*log(msb12/Q2)*log(mw2/Q2)*power2(mw2)*power2(
     snb)*power2(snt)*(-((msb12*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(
     mst22,mw2,msb12))) + power2(DeltaInv(mw2,msb12,mst12))*(msb12*mst12 +
     mst14 + 3*mst12*mw2 + DeltaInv(mw2,msb12,mst12)*(5*mst14*power2(mw2) -
     mw2*power3(mst12) + msb12*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(
     mst12)) - 3*mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,
     mw2,msb12))*(-5*mst24*power2(mw2) + mw2*power3(mst22) - msb12*(10*mst24*
     mw2 + 5*mst22*power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(
     mst22))))/msb12 + (2*cb*mu*sb*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*
     power2(snt)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(-
     5*mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb22 + (2*cb*mu*sb*log(msb22/Q2)*log(mw2/Q2)*power2(mw2)*power2(snb)*
     power2(snt)*(-((msb22*mst22 + mst24 + 3*mst22*mw2)*power2(DeltaInv(mst22,
     mw2,msb22))) + power2(DeltaInv(mw2,msb22,mst12))*(msb22*mst12 + mst14 + 3*
     mst12*mw2 + DeltaInv(mw2,msb22,mst12)*(5*mst14*power2(mw2) - mw2*power3(
     mst12) + msb22*(10*mst14*mw2 + 5*mst12*power2(mw2) + power3(mst12)) - 3*
     mst12*power3(mw2) - power4(mst12))) + power3(DeltaInv(mst22,mw2,msb22))*(-
     5*mst24*power2(mw2) + mw2*power3(mst22) - msb22*(10*mst24*mw2 + 5*mst22*
     power2(mw2) + power3(mst22)) + 3*mst22*power3(mw2) + power4(mst22))))/
     msb22 + (mu*sb*power2(snt)*(msb22*(-mst12 + mst22) - 2*mC2*msb22*mst12*
     mst22*DeltaInv(msb22,mst12,mC2) + 2*mC2*msb22*mst12*mst22*DeltaInv(mst22,
     msb22,mC2) + 84*mst12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mst22*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 84*mst12*mst22*(3*mC2*mst22 +
     msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 84*
     mst12*mst22*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*
     mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 84*mst12*
     mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-14*mC2*msb22*mst24 -
     5*msb22*mst22*power2(mC2) - 3*mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*
     mC2*power3(mst22) - 5*msb22*power3(mst22) + 3*power4(mst22))))/(6.*cb*
     msb22*mst12*mst22) + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(mst22/
     Q2))*(msb22*mst24*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*
     mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - msb22*power2(mC2)*(
     mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2)) + msb12*power2(mC2)*(
     mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*(-(
     msb22*mst24*power2(DeltaInv(mst22,mw2,msb12))) + msb12*power2(DeltaInv(
     mst22,mw2,msb22))*(mst24 + DeltaInv(mst22,mw2,msb22)*(-(mst24*power2(mw2))
     + 2*mw2*power3(mst22) + 2*msb22*(mst24*mw2 + power3(mst22)) - power4(
     mst22))) + msb22*power3(DeltaInv(mst22,mw2,msb12))*(mst24*power2(mw2) - 2*
     mw2*power3(mst22) - 2*msb12*(mst24*mw2 + power3(mst22)) + power4(mst22))))
     )/(msb12*msb22) + (2*cb*mu*sb*power2(snt)*power2(log(mst22/Q2))*(mst24*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22))*(-mst24 + DeltaInv(mst22,mw2,msb22)*(mst24*power2(mw2) -
     2*mw2*power3(mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(mst22))
     )))/msb22 + (cb*mu*sb*log(mst22/Q2)*power2(snt)*((5*mC2*mst22 - msb22*
     mst22 + mst24)*DeltaInv(mst22,msb22,mC2) - 6*mC2*power2(DeltaInv(mst22,
     msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) - 72*power2(mC2)*(mst24*power2(mC2) + (-2*
     msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2)) - DeltaInv(mst22,mw2,msb22)*(-(msb22*
     mst22) + mst24 + 5*mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb22)*(mst22*mw2*
     (-3*msb22 + mw2) - mst24*(msb22 + 14*mw2) + power3(mst22)) - 72*power2(
     mw2)*power2(DeltaInv(mst22,mw2,msb22))*(mst24*power2(mw2) - 2*mw2*power3(
     mst22) - 2*msb22*(mst24*mw2 + power3(mst22)) + power4(mst22)))))/(6.*
     msb22) + (cb*mu*sb*log(mst22/Q2)*power2(snt)*(-((5*mC2*mst22 - msb22*mst22
     + mst24)*DeltaInv(mst22,msb22,mC2)) + 6*mC2*power2(DeltaInv(mst22,msb22,
     mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2)
     + power3(mst22)) + 72*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2)) + DeltaInv(mst22,mw2,msb22)*(-(msb22*mst22) + mst24 + 5*
     mst22*mw2 - 6*mw2*DeltaInv(mst22,mw2,msb22)*(mst22*mw2*(-3*msb22 + mw2) -
     mst24*(msb22 + 14*mw2) + power3(mst22)) - 72*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb22))*(mst24*power2(mw2) - 2*mw2*power3(mst22) - 2*msb22*(
     mst24*mw2 + power3(mst22)) + power4(mst22)))))/(6.*msb22) + (2*cb*mu*sb*
     log(msb12/Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*((mC2*mst22 + msb12*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + power2(
     mC2)*((msb12*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb12 -
     3*mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     *(-(msb12*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb12)*(mst24*
     power2(mw2) - 5*mw2*power3(mst22) - msb12*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/msb12 +
     (2*cb*mu*sb*log(msb22/Q2)*log(mst22/Q2)*power2(snt)*((mC2*mst22 + msb22*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + power2(
     mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 -
     3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,mw2,msb22))
     *(-(msb22*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,msb22)*(mst24*
     power2(mw2) - 5*mw2*power3(mst22) - msb22*(10*mst24*mw2 + mst22*power2(
     mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)))))/msb22 +
     (2*cb*mu*sb*log(msb22/Q2)*log(mst22/Q2)*power2(snb)*power2(snt)*((mC2*
     mst22 + msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2)) + power2(mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2)
     + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb22,mC2)) + power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22))*(-(msb22*mst22) - 3*mst24 - mst22*mw2 + DeltaInv(mst22,mw2,
     msb22)*(mst24*power2(mw2) - 5*mw2*power3(mst22) - msb22*(10*mst24*mw2 +
     mst22*power2(mw2) + 5*power3(mst22)) + mst22*power3(mw2) + 3*power4(mst22)
     ))))/msb22 + (cb*mu*sb*log(mst22/Q2)*power2(snb)*power2(snt)*(msb22*(5*
     mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - msb12*(5*mC2*
     mst22 - msb22*mst22 + mst24)*DeltaInv(mst22,msb22,mC2) + msb12*msb22*
     mst22*DeltaInv(mst22,mw2,msb12) - msb22*mst24*DeltaInv(mst22,mw2,msb12) -
     5*msb22*mst22*mw2*DeltaInv(mst22,mw2,msb12) - msb12*msb22*mst22*DeltaInv(
     mst22,mw2,msb22) + msb12*mst24*DeltaInv(mst22,mw2,msb22) + 5*msb12*mst22*
     mw2*DeltaInv(mst22,mw2,msb22) - 6*msb12*msb22*mst24*mw2*power2(DeltaInv(
     mst22,mw2,msb12)) - 18*msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(
     mst22,mw2,msb12)) - 84*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 18*
     msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 84*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 6*msb22*mw2*
     power2(DeltaInv(mst22,mw2,msb12))*power3(mst22) - 6*msb12*mw2*power2(
     DeltaInv(mst22,mw2,msb22))*power3(mst22) - 6*mC2*msb22*power2(DeltaInv(
     mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 +
     mst22*power2(mC2) + power3(mst22)) + 6*mC2*msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) + 6*msb22*mst22*power2(DeltaInv(mst22,mw2,
     msb12))*power3(mw2) - 6*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*
     power3(mw2) - 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)
     *power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb12,mC2)) + 72*msb12*power2(mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 144*msb12*msb22*power2(mw2)*power3(mst22)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 144*msb22*power3(mst22)*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 144*msb12*power3(mst22)*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 72*msb22*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb12))*power4(mst22) - 72*msb12*power2(mw2)*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mst22) + 72*msb22*mst24*power3(DeltaInv(
     mst22,mw2,msb12))*power4(mw2) - 72*msb12*mst24*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mw2)))/(6.*msb12*msb22) + (cb*mu*sb*log(mst22/Q2)*power2(
     snb)*power2(snt)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(
     mst22,msb12,mC2)) + msb12*(5*mC2*mst22 - msb22*mst22 + mst24)*DeltaInv(
     mst22,msb22,mC2) - msb12*msb22*mst22*DeltaInv(mst22,mw2,msb12) + msb22*
     mst24*DeltaInv(mst22,mw2,msb12) + 5*msb22*mst22*mw2*DeltaInv(mst22,mw2,
     msb12) + msb12*msb22*mst22*DeltaInv(mst22,mw2,msb22) - msb12*mst24*
     DeltaInv(mst22,mw2,msb22) - 5*msb12*mst22*mw2*DeltaInv(mst22,mw2,msb22) +
     6*msb12*msb22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb12)) + 18*msb12*
     msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + 84*msb22*
     mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 6*msb12*msb22*mst24*
     mw2*power2(DeltaInv(mst22,mw2,msb22)) - 18*msb12*msb22*mst22*power2(mw2)*
     power2(DeltaInv(mst22,mw2,msb22)) - 84*msb12*mst24*power2(mw2)*power2(
     DeltaInv(mst22,mw2,msb22)) - 6*msb22*mw2*power2(DeltaInv(mst22,mw2,msb12))
     *power3(mst22) + 6*msb12*mw2*power2(DeltaInv(mst22,mw2,msb22))*power3(
     mst22) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22
     - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*mC2*
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(-3*mC2*msb22*mst22 - 14*mC2*mst24
     - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 6*msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst22*power2(DeltaInv(
     mst22,mw2,msb22))*power3(mw2) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2)) - 72*msb12*power2(mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 144*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 144*msb12*msb22*power2(
     mw2)*power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*msb22*
     mst24*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 144*msb12*power3(
     mst22)*power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 72*msb22*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) + 72*msb12*power2(
     mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 72*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 72*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2)))/(6.*msb12*msb22) + (cb*mu*sb*log(
     mst12/Q2)*power2(snb)*(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(
     msb12,mst12,mC2) - msb12*(5*mC2*mst12 - msb22*mst12 + mst14)*DeltaInv(
     msb22,mst12,mC2) + msb12*msb22*mst12*DeltaInv(mw2,msb12,mst12) - msb22*
     mst14*DeltaInv(mw2,msb12,mst12) - 5*msb22*mst12*mw2*DeltaInv(mw2,msb12,
     mst12) - msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) + msb12*mst14*
     DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*mw2*DeltaInv(mw2,msb22,mst12) -
     6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) - 18*msb12*
     msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*msb22*
     mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*mst14*
     mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 84*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 6*msb22*mw2*power2(DeltaInv(mw2,msb12,mst12))
     *power3(mst12) - 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*power3(
     mst12) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12
     - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + 6*mC2*
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14
     - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 6*msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12))*power3(mw2) - 6*msb12*mst12*power2(DeltaInv(
     mw2,msb22,mst12))*power3(mw2) - 72*msb22*power2(mC2)*(mst14*power2(mC2) +
     (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2)) + 72*msb12*power2(mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 144*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*msb22*
     mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*msb12*power3(
     mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 72*msb22*power2(
     mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*msb12*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb12*mst14*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(6.*msb12*msb22) + (cb*mu*sb*log(
     mst12/Q2)*power2(snb)*power2(snt)*(msb22*(5*mC2*mst12 - msb12*mst12 +
     mst14)*DeltaInv(msb12,mst12,mC2) - msb12*(5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2) + msb12*msb22*mst12*DeltaInv(mw2,msb12,
     mst12) - msb22*mst14*DeltaInv(mw2,msb12,mst12) - 5*msb22*mst12*mw2*
     DeltaInv(mw2,msb12,mst12) - msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) +
     msb12*mst14*DeltaInv(mw2,msb22,mst12) + 5*msb12*mst12*mw2*DeltaInv(mw2,
     msb22,mst12) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) -
     18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 84*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 6*msb12*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 18*msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 84*msb12*mst14*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) + 6*msb22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) - 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) + 6*mC2*msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) +
     6*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 6*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*msb22*power2(mC2)
     *(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*msb12*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) -
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) +
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 144*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 72*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) - 72*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) + 72*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 72*msb12*
     mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(6.*msb12*msb22) + (
     cb*mu*sb*log(mst12/Q2)*power2(snb)*(-(msb22*(5*mC2*mst12 - msb12*mst12 +
     mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 - msb22*mst12 +
     mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*DeltaInv(mw2,msb12,
     mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*msb22*mst12*mw2*
     DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(mw2,msb22,mst12) -
     msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*mw2*DeltaInv(mw2,
     msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb12,mst12)) +
     18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 84*
     msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 6*msb12*msb22*
     mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*msb12*mst14*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mst12) + 6*msb12*mw2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mst12) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*
     msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(
     mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,mst12,mC2))*(-3*mC2*msb22*
     mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) -
     6*msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 6*msb12*
     mst12*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*msb22*power2(mC2)
     *(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*msb12*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*
     (msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) +
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 144*
     msb22*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 144*
     msb12*msb22*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) -
     144*msb12*msb22*mst14*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*
     msb12*power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*
     msb22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*
     msb12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 72*msb12*
     mst14*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)))/(6.*msb12*msb22) + (
     cb*mu*sb*log(mst12/Q2)*power2(snb)*power2(snt)*(-(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) + msb12*(5*mC2*mst12 -
     msb22*mst12 + mst14)*DeltaInv(msb22,mst12,mC2) - msb12*msb22*mst12*
     DeltaInv(mw2,msb12,mst12) + msb22*mst14*DeltaInv(mw2,msb12,mst12) + 5*
     msb22*mst12*mw2*DeltaInv(mw2,msb12,mst12) + msb12*msb22*mst12*DeltaInv(
     mw2,msb22,mst12) - msb12*mst14*DeltaInv(mw2,msb22,mst12) - 5*msb12*mst12*
     mw2*DeltaInv(mw2,msb22,mst12) + 6*msb12*msb22*mst14*mw2*power2(DeltaInv(
     mw2,msb12,mst12)) + 18*msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + 84*msb22*mst14*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 6*msb12*msb22*mst14*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 18*
     msb12*msb22*mst12*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 84*
     msb12*mst14*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 6*msb22*mw2*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mst12) + 6*msb12*mw2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mst12) + 6*mC2*msb22*power2(DeltaInv(
     msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 +
     mst12*power2(mC2) + power3(mst12)) - 6*mC2*msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*
     power2(mC2) + power3(mst12)) - 6*msb22*mst12*power2(DeltaInv(mw2,msb12,
     mst12))*power3(mw2) + 6*msb12*mst12*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2) + 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)
     *power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*msb12*power2(mC2)*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 144*msb12*msb22*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb12*msb22*mst14*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) + 144*msb22*power3(mst12)*power3(mw2)*
     power3(DeltaInv(mw2,msb12,mst12)) - 144*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 144*msb12*power3(mst12)*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 72*msb22*power2(mw2)*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mst12) + 72*msb12*power2(mw2)*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mst12) - 72*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) + 72*msb12*mst14*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2)))/(6.*msb12*msb22) - (4*mu*sb*log(msb12/Q2)*log(msb22/
     Q2)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(mH2*msb22 + msb12*msb22
     + 3*msb24 + DeltaInv(mH2,msb12,msb22)*((msb12*msb22 - msb24)*power2(mH2) -
     msb22*power3(mH2) + (5*msb12 - 3*msb22)*power3(msb22) + 5*mH2*(2*msb12*
     msb24 + power3(msb22))))*power4(snb))/(cb*msb12) - (2*mH2*mu*sb*power2(
     log(msb12/Q2))*(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*mH2*
     power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*msb22 + msb12*msb22 + 2*msb24 +
     DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22 + 2*msb24)*power2(mH2) - 2*
     msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*mH2*(5*msb12*
     msb24 + power3(msb22)))))*power4(snb))/(cb*msb12) + (4*ca*mu*sa*log(msb12/
     Q2)*log(msb22/Q2)*(-((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*(mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,
     msb22)*((msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (5*msb12 -
     3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(
     mh2)*((-(msb12*msb22) + msb24)*power2(mh2) + msb22*power3(mh2) + (-5*msb12
     + 3*msb22)*power3(msb22) - 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)))*power4(snb))/msb12 + (4*mu*sb*log(msb12/Q2)*
     log(msb22/Q2)*power2(sa)*(-((mh2*msb22 + msb12*msb22 + 3*msb24)*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22))) + power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*(mH2*msb22 + msb12*msb22 + 3*msb24 + DeltaInv(mH2,msb12,
     msb22)*((msb12*msb22 - msb24)*power2(mH2) - msb22*power3(mH2) + (5*msb12 -
     3*msb22)*power3(msb22) + 5*mH2*(2*msb12*msb24 + power3(msb22)))) + power2(
     mh2)*((-(msb12*msb22) + msb24)*power2(mh2) + msb22*power3(mh2) + (-5*msb12
     + 3*msb22)*power3(msb22) - 5*mh2*(2*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)))*power4(snb))/(cb*msb12) + (2*ca*mu*sa*power2(
     log(msb12/Q2))*(-(mh2*msb12*msb14*power2(DeltaInv(mh2,msb12,msb12))) - 2*(
     2*mh2*msb22 + msb12*msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + mH2*(msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*
     mH2*power2(DeltaInv(mH2,msb12,msb22))*(2*mH2*msb22 + msb12*msb22 + 2*msb24
     + DeltaInv(mH2,msb12,msb22)*((3*msb12*msb22 + 2*msb24)*power2(mH2) - 2*
     msb22*power3(mH2) + (3*msb12 - 2*msb22)*power3(msb22) + 2*mH2*(5*msb12*
     msb24 + power3(msb22))))) + 2*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*
     power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) -
     2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)))*
     power4(snb))/msb12 + (2*mu*sb*power2(sa)*power2(log(msb12/Q2))*(-(mh2*
     msb12*msb14*power2(DeltaInv(mh2,msb12,msb12))) - 2*(2*mh2*msb22 + msb12*
     msb22 + 2*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mH2*(
     msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*mH2*power2(DeltaInv(mH2,
     msb12,msb22))*(2*mH2*msb22 + msb12*msb22 + 2*msb24 + DeltaInv(mH2,msb12,
     msb22)*((3*msb12*msb22 + 2*msb24)*power2(mH2) - 2*msb22*power3(mH2) + (3*
     msb12 - 2*msb22)*power3(msb22) + 2*mH2*(5*msb12*msb24 + power3(msb22)))))
     + 2*power2(mh2)*(-((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*
     power3(mh2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 +
     power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)))*power4(snb))/(cb*msb12)
     - (2*mH2*mu*sb*power2(log(msb22/Q2))*(2*mH2*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) + msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) - 2*mH2*(
     msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24
     + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)))*power4(snb))/(cb*
     msb12) + (2*ca*mu*sa*power2(log(msb22/Q2))*(-2*msb24*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) - mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,
     msb22)) + 2*power2(mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,
     msb22)) + mH2*(2*mH2*msb24*power2(DeltaInv(mH2,msb12,msb22)) + msb12*
     msb24*power2(DeltaInv(mH2,msb22,msb22)) - 2*mH2*(msb24*power2(mH2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mH2,msb12,msb22))))*power4(snb))/msb12 + (2*mu*sb*power2(
     sa)*power2(log(msb22/Q2))*(-2*msb24*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) - mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) + 2*power2(
     mh2)*(msb24*power2(mh2) + (-2*msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + mH2*(2*mH2*
     msb24*power2(DeltaInv(mH2,msb12,msb22)) + msb12*msb24*power2(DeltaInv(mH2,
     msb22,msb22)) - 2*mH2*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(
     msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,
     msb22))))*power4(snb))/(cb*msb12) - (ca*mu*sa*log(msb22/Q2)*(-((5*mh2*
     msb22 - msb12*msb22 + msb24)*DeltaInv(mh2,msb12,msb22)) - msb12*(mh2 - 6*
     msb22)*DeltaInv(mh2,msb22,msb22) + 5*mH2*msb22*DeltaInv(mH2,msb12,msb22) -
     msb12*msb22*DeltaInv(mH2,msb12,msb22) + msb24*DeltaInv(mH2,msb12,msb22) +
     mH2*msb12*DeltaInv(mH2,msb22,msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,
     msb22) + 6*mH2*msb12*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*
     msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)
     *power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2)
     - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*
     power2(mh2) + power3(msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     (mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22))*power4(snb)
     )/(3.*msb12) - (mu*sb*log(msb22/Q2)*power2(sa)*(-((5*mh2*msb22 - msb12*
     msb22 + msb24)*DeltaInv(mh2,msb12,msb22)) - msb12*(mh2 - 6*msb22)*
     DeltaInv(mh2,msb22,msb22) + 5*mH2*msb22*DeltaInv(mH2,msb12,msb22) - msb12*
     msb22*DeltaInv(mH2,msb12,msb22) + msb24*DeltaInv(mH2,msb12,msb22) + mH2*
     msb12*DeltaInv(mH2,msb22,msb22) - 6*msb12*msb22*DeltaInv(mH2,msb22,msb22)
     + 6*mH2*msb12*msb24*power2(DeltaInv(mH2,msb12,msb22)) + 18*msb12*msb22*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 84*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - 24*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb22,msb22)) - 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2)
     - 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*power3(msb22) - 72*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 6*mh2*power2(DeltaInv(mh2,
     msb12,msb22))*(-3*mh2*msb12*msb22 - 14*mh2*msb24 - msb12*msb24 + msb22*
     power2(mh2) + power3(msb22)) + 24*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     (mh2*msb24 + 3*power3(msb22)) + 72*power2(mh2)*(msb24*power2(mh2) + (-2*
     msb12 + msb22)*power3(msb22) - 2*mh2*(msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 144*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) - 72*msb24*power3(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 288*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 72*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     288*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22))*power4(snb)
     )/(3.*cb*msb12) + (4*ca*mu*sa*log(mh2/Q2)*log(msb22/Q2)*((mh2*msb22 +
     msb12*msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) +
     power2(mh2)*((msb12*msb22 + msb24)*power2(mh2) - msb22*power3(mh2) + (
     msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(msb22)))*
     power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(DeltaInv(mh2,msb22,msb22)
     )*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(
     msb22)))*power4(snb))/msb12 + (4*mu*sb*log(mh2/Q2)*log(msb22/Q2)*power2(
     sa)*((mh2*msb22 + msb12*msb22 + msb24)*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)) + power2(mh2)*((msb12*msb22 + msb24)*power2(mh2) - msb22*
     power3(mh2) + (msb12 - msb22)*power3(msb22) + mh2*(6*msb12*msb24 + power3(
     msb22)))*power3(DeltaInv(mh2,msb12,msb22)) + msb12*power2(DeltaInv(mh2,
     msb22,msb22))*(mh2*msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,
     msb22)*power4(msb22)))*power4(snb))/(cb*msb12) - (4*ca*mu*sa*log(mH2/Q2)*
     log(msb22/Q2)*((mH2*msb22 + msb12*msb22 + msb24)*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((msb12*msb22 + msb24)*power2(
     mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(
     DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(
     mH2,msb22,msb22)*power4(msb22)))*power4(snb))/msb12 + (4*mu*sb*log(mH2/Q2)
     *log(msb22/Q2)*((mH2*msb22 + msb12*msb22 + msb24)*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((msb12*msb22 + msb24)*power2(
     mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) + mH2*(6*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + msb12*power2(
     DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(
     mH2,msb22,msb22)*power4(msb22)))*power4(snb))/(cb*msb12) - (4*mu*sb*log(
     mH2/Q2)*log(msb22/Q2)*power2(sa)*((mH2*msb22 + msb12*msb22 + msb24)*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + power2(mH2)*((msb12*msb22
     + msb24)*power2(mH2) - msb22*power3(mH2) + (msb12 - msb22)*power3(msb22) +
     mH2*(6*msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) +
     msb12*power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*
     mH2*DeltaInv(mH2,msb22,msb22)*power4(msb22)))*power4(snb))/(cb*msb12) - (
     mu*sb*log(msb22/Q2)*(-((5*mH2*msb22 - msb12*msb22 + msb24)*DeltaInv(mH2,
     msb12,msb22)) + 6*mH2*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb12*
     msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) + power3(msb22)) +
     72*power2(mH2)*(msb24*power2(mH2) + (-2*msb12 + msb22)*power3(msb22) - 2*
     mH2*(msb12*msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) +
     msb12*DeltaInv(mH2,msb22,msb22)*(-mH2 + 6*msb22 + 24*DeltaInv(mH2,msb22,
     msb22)*(mH2*msb24 + 3*power3(msb22)) + 288*mH2*power2(DeltaInv(mH2,msb22,
     msb22))*power4(msb22)))*power4(snb))/(3.*cb*msb12) - (4*ca*mu*sa*log(mh2/
     Q2)*log(msb12/Q2)*power2(snb)*(power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 + DeltaInv(mh2,msb12,msb22)*(-
     5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*power3(mh2) + (-msb12 +
     msb22)*power3(msb22) + mh2*(-10*msb12*msb24 + power3(msb22)))) + power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(msb12)) + 16*mh2*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 - (4*mu*sb*log(
     mh2/Q2)*log(msb12/Q2)*power2(sa)*power2(snb)*(power2(mh2)*power2(DeltaInv(
     mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 + DeltaInv(mh2,
     msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*power3(mh2) +
     (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 + power3(msb22)))) +
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(msb12)) +
     16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(cb*msb12) + (4*
     ca*mu*sa*log(mh2/Q2)*log(msb12/Q2)*power4(snb)*(power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 + DeltaInv(
     mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*power3(
     mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 + power3(
     msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(
     msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 +
     (4*mu*sb*log(mh2/Q2)*log(msb12/Q2)*power2(sa)*power4(snb)*(power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22))*(-3*mh2*msb22 - msb12*msb22 - msb24 +
     DeltaInv(mh2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mh2) + 3*msb22*
     power3(mh2) + (-msb12 + msb22)*power3(msb22) + mh2*(-10*msb12*msb24 +
     power3(msb22)))) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*
     power4(msb12)) + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/
     (cb*msb12) - (2*ca*mu*sa*power2(snb)*power2(log(mh2/Q2))*(-2*msb22*power2(
     DeltaInv(mh2,msb12,msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 -
     2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(
     DeltaInv(mh2,msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*
     msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*
     msb24 + 4*power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22))
     + 16*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 - (2*mu*
     sb*power2(sa)*power2(snb)*power2(log(mh2/Q2))*(-2*msb22*power2(DeltaInv(
     mh2,msb12,msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(
     msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(
     mh2,msb12,msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 +
     4*power4(msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*
     power3(msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(cb*msb12) + (2*ca*mu*
     sa*power2(log(mh2/Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(mh2,msb12,
     msb22))*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 +
     msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,
     msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(
     msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(
     msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12)))/msb12 + (2*mu*sb*power2(sa)*
     power2(log(mh2/Q2))*power4(snb)*(-2*msb22*power2(DeltaInv(mh2,msb12,msb22)
     )*power3(mh2) + 2*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 +
     msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,
     msb22)) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 4*power4(
     msb12)) + msb12*power2(DeltaInv(mh2,msb22,msb22))*(mh2*msb24 + 4*power3(
     msb22) + 16*mh2*DeltaInv(mh2,msb22,msb22)*power4(msb22)) + 16*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12)))/(cb*msb12) - (ca*mu*sa*power2(
     snb)*(-(mh2*msb12*DeltaInv(mh2,msb12,msb12)) + 2*mh2*msb12*DeltaInv(mh2,
     msb12,msb22) - mh2*msb12*DeltaInv(mh2,msb22,msb22) + mH2*msb12*DeltaInv(
     mH2,msb12,msb12) - 2*mH2*msb12*DeltaInv(mH2,msb12,msb22) + mH2*msb12*
     DeltaInv(mH2,msb22,msb22) - 84*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*
     power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 42*mh2*msb12*msb24*power2(
     DeltaInv(mh2,msb22,msb22)) - 42*mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,
     msb12)) + 84*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     252*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - 42*mH2*msb12*
     msb24*power2(DeltaInv(mH2,msb22,msb22)) + 252*msb22*power2(DeltaInv(mH2,
     msb12,msb22))*power3(mH2) + 252*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     power3(msb22) - 252*msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22)
     + 1176*msb12*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 420*
     msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 252*
     power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*
     msb22*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 252*msb24*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) - 252*power2(DeltaInv(mH2,msb12,
     msb12))*power4(msb12) + 42*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*
     msb14 + 6*power4(msb12)) + 1008*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22)
     )*power4(msb22) - 252*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) - 1008*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(
     msb22) + 84*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*
     msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(
     mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) -
     252*msb22*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) + 1008*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12) - 1008*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/(3.*msb12) - (mu*sb*power2(sa)*power2(snb)*(
     -(mh2*msb12*DeltaInv(mh2,msb12,msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,
     msb22) - mh2*msb12*DeltaInv(mh2,msb22,msb22) + mH2*msb12*DeltaInv(mH2,
     msb12,msb12) - 2*mH2*msb12*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(
     mH2,msb22,msb22) - 84*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*
     power2(DeltaInv(mh2,msb12,msb22)) + 42*mh2*msb12*msb24*power2(DeltaInv(
     mh2,msb22,msb22)) - 42*mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) +
     84*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 252*msb24*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - 42*mH2*msb12*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 252*msb22*power2(DeltaInv(mH2,msb12,msb22))*
     power3(mH2) + 252*msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) -
     252*msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*
     msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(
     mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,
     msb12,msb22))*power4(mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(
     msb12) + 42*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(
     msb12)) + 1008*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) -
     252*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*
     mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)
     *power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*
     power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(
     msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,
     msb12,msb12))*power5(msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12)))/(3.*cb*msb12) + (ca*mu*sa*power4(snb)*(-(mh2*msb12*
     DeltaInv(mh2,msb12,msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,msb22) - mh2*
     msb12*DeltaInv(mh2,msb22,msb22) + mH2*msb12*DeltaInv(mH2,msb12,msb12) - 2*
     mH2*msb12*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22)
     - 84*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(
     mh2,msb12,msb22)) + 42*mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) -
     42*mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 84*msb12*msb22*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 252*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - 42*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb22,msb22)) + 252*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(
     mH2) + 252*msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 252*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*msb24*
     power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*power3(
     msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,
     msb12,msb22))*power4(mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(
     msb12) + 42*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(
     msb12)) + 1008*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) -
     252*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*
     mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)
     *power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*
     power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(
     msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,
     msb12,msb12))*power5(msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12)))/(3.*msb12) + (mu*sb*power2(sa)*power4(snb)*(-(mh2*msb12*
     DeltaInv(mh2,msb12,msb12)) + 2*mh2*msb12*DeltaInv(mh2,msb12,msb22) - mh2*
     msb12*DeltaInv(mh2,msb22,msb22) + mH2*msb12*DeltaInv(mH2,msb12,msb12) - 2*
     mH2*msb12*DeltaInv(mH2,msb12,msb22) + mH2*msb12*DeltaInv(mH2,msb22,msb22)
     - 84*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(
     mh2,msb12,msb22)) + 42*mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) -
     42*mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 84*msb12*msb22*
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 252*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - 42*mH2*msb12*msb24*power2(DeltaInv(
     mH2,msb22,msb22)) + 252*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(
     mH2) + 252*msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 252*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) + 1176*msb12*msb24*
     power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*power2(mH2)*
     power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 252*power3(mH2)*power3(
     msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 420*msb12*msb22*power3(
     DeltaInv(mH2,msb12,msb22))*power4(mH2) + 252*msb24*power3(DeltaInv(mH2,
     msb12,msb22))*power4(mH2) - 252*power2(DeltaInv(mH2,msb12,msb12))*power4(
     msb12) + 42*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(
     msb12)) + 1008*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) -
     252*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 1008*
     mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 84*power2(mh2)
     *power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*
     power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(
     msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 252*msb22*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 1008*mh2*power3(DeltaInv(mh2,
     msb12,msb12))*power5(msb12) - 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12)))/(3.*cb*msb12) - (ca*mu*sa*power2(Pi)*power2(snb)*(-2*(3*
     mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - mH2*msb12*
     msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*msb12*msb22*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + 6*msb24*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) - mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 6*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 6*msb12*power2(
     DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*msb12*power2(DeltaInv(mH2,
     msb22,msb22))*power3(msb22) + 28*msb12*msb24*power3(mH2)*power3(DeltaInv(
     mH2,msb12,msb22)) + 10*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(
     mH2,msb12,msb22)) + 6*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 10*msb12*msb22*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) +
     6*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) - 6*power2(DeltaInv(
     mH2,msb12,msb12))*power4(msb12) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*
     msb12*msb14 + 6*power4(msb12)) + 24*mh2*msb12*power3(DeltaInv(mh2,msb22,
     msb22))*power4(msb22) - 6*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) - 24*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(
     msb22) + 2*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*
     msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(
     mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 6*
     msb22*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) + 24*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12) - 24*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/(3.*msb12) - (mu*sb*power2(Pi)*power2(sa)*
     power2(snb)*(-2*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(
     DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,
     msb22)) - mH2*msb12*msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*msb12*
     msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) + 6*msb24*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) - mH2*msb12*msb24*power2(DeltaInv(mH2,
     msb22,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 6*
     msb12*power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power3(msb22) + 28*msb12*msb24*power3(mH2)*
     power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*power2(mH2)*power3(msb22)*
     power3(DeltaInv(mH2,msb12,msb22)) + 6*power3(mH2)*power3(msb22)*power3(
     DeltaInv(mH2,msb12,msb22)) + 10*msb12*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power4(mH2) + 6*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 6*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + power2(
     DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*power4(msb12)) + 24*mh2*
     msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(msb22) - 6*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) - 24*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*power2(mh2)*power3(DeltaInv(
     mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*msb22*power2(mh2) - 3*
     msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*power3(msb22) - 5*msb12*
     power3(msb22) + 3*power4(msb22)) - 6*msb22*power3(DeltaInv(mH2,msb12,
     msb22))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,msb12,msb12))*power5(
     msb12) - 24*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(3.*cb*
     msb12) + (ca*mu*sa*power2(Pi)*power4(snb)*(-2*(3*mh2*msb22 + msb12*msb22 +
     3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*msb12*msb24*
     power2(DeltaInv(mh2,msb22,msb22)) - mH2*msb12*msb14*power2(DeltaInv(mH2,
     msb12,msb12)) + 2*msb12*msb22*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)
     ) + 6*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) - mH2*msb12*
     msb24*power2(DeltaInv(mH2,msb22,msb22)) + 6*msb22*power2(DeltaInv(mH2,
     msb12,msb22))*power3(mH2) + 6*msb12*power2(DeltaInv(mh2,msb22,msb22))*
     power3(msb22) - 6*msb12*power2(DeltaInv(mH2,msb22,msb22))*power3(msb22) +
     28*msb12*msb24*power3(mH2)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*
     power2(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 6*power3(
     mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,msb22)) + 10*msb12*msb22*
     power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) + 6*msb24*power3(DeltaInv(
     mH2,msb12,msb22))*power4(mH2) - 6*power2(DeltaInv(mH2,msb12,msb12))*
     power4(msb12) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb12*msb14 + 6*
     power4(msb12)) + 24*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) - 6*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) -
     24*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*power2(
     mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*
     msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*
     power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 6*msb22*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 24*mh2*power3(DeltaInv(mh2,msb12,
     msb12))*power5(msb12) - 24*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(
     msb12)))/(3.*msb12) + (mu*sb*power2(Pi)*power2(sa)*power4(snb)*(-2*(3*mh2*
     msb22 + msb12*msb22 + 3*msb24)*power2(mh2)*power2(DeltaInv(mh2,msb12,
     msb22)) + mh2*msb12*msb24*power2(DeltaInv(mh2,msb22,msb22)) - mH2*msb12*
     msb14*power2(DeltaInv(mH2,msb12,msb12)) + 2*msb12*msb22*power2(mH2)*
     power2(DeltaInv(mH2,msb12,msb22)) + 6*msb24*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) - mH2*msb12*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 6*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 6*msb12*power2(
     DeltaInv(mh2,msb22,msb22))*power3(msb22) - 6*msb12*power2(DeltaInv(mH2,
     msb22,msb22))*power3(msb22) + 28*msb12*msb24*power3(mH2)*power3(DeltaInv(
     mH2,msb12,msb22)) + 10*msb12*power2(mH2)*power3(msb22)*power3(DeltaInv(
     mH2,msb12,msb22)) + 6*power3(mH2)*power3(msb22)*power3(DeltaInv(mH2,msb12,
     msb22)) + 10*msb12*msb22*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) +
     6*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(mH2) - 6*power2(DeltaInv(
     mH2,msb12,msb12))*power4(msb12) + power2(DeltaInv(mh2,msb12,msb12))*(mh2*
     msb12*msb14 + 6*power4(msb12)) + 24*mh2*msb12*power3(DeltaInv(mh2,msb22,
     msb22))*power4(msb22) - 6*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power4(msb22) - 24*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power4(
     msb22) + 2*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*
     msb24 - 5*msb12*msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(
     mh2) - 3*mh2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 6*
     msb22*power3(DeltaInv(mH2,msb12,msb22))*power5(mH2) + 24*mh2*power3(
     DeltaInv(mh2,msb12,msb12))*power5(msb12) - 24*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/(3.*cb*msb12) + (4*ca*mu*sa*log(mH2/Q2)*log(
     msb12/Q2)*power2(snb)*(power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-3*
     mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*
     msb22 + msb24)*power2(mH2) + 3*msb22*power3(mH2) + (-msb12 + msb22)*
     power3(msb22) + mH2*(-10*msb12*msb24 + power3(msb22)))) + power2(DeltaInv(
     mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) + 16*mH2*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12)))/msb12 - (4*mu*sb*log(mH2/Q2)*
     log(msb12/Q2)*power2(snb)*(power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(
     -3*mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*
     msb22 + msb24)*power2(mH2) + 3*msb22*power3(mH2) + (-msb12 + msb22)*
     power3(msb22) + mH2*(-10*msb12*msb24 + power3(msb22)))) + power2(DeltaInv(
     mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) + 16*mH2*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(cb*msb12) + (4*mu*sb*log(mH2/
     Q2)*log(msb12/Q2)*power2(sa)*power2(snb)*(power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(mH2,msb12,
     msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*power3(mH2) + (-
     msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 + power3(msb22)))) +
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) +
     16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(cb*msb12) - (4*
     ca*mu*sa*log(mH2/Q2)*log(msb12/Q2)*power4(snb)*(power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(
     mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*power3(
     mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 + power3(
     msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(
     msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/msb12 +
     (4*mu*sb*log(mH2/Q2)*log(msb12/Q2)*power4(snb)*(power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22 - msb24 + DeltaInv(
     mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2) + 3*msb22*power3(
     mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*msb24 + power3(
     msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 4*power4(
     msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(cb*
     msb12) - (4*mu*sb*log(mH2/Q2)*log(msb12/Q2)*power2(sa)*power4(snb)*(
     power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*msb22 - msb12*msb22
     - msb24 + DeltaInv(mH2,msb12,msb22)*(-5*(msb12*msb22 + msb24)*power2(mH2)
     + 3*msb22*power3(mH2) + (-msb12 + msb22)*power3(msb22) + mH2*(-10*msb12*
     msb24 + power3(msb22)))) + power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*
     msb14 + 4*power4(msb12)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12)))/(cb*msb12) + (2*ca*mu*sa*power2(snb)*power2(log(mH2/Q2))*(
     -2*msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-
     2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(
     msb22))*power3(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,
     msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,
     msb22)*power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(
     msb12)))/msb12 - (2*mu*sb*power2(snb)*power2(log(mH2/Q2))*(-2*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(
     mH2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,
     msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*
     power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/
     (cb*msb12) + (2*mu*sb*power2(sa)*power2(snb)*power2(log(mH2/Q2))*(-2*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(
     msb22))*power3(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,
     msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,
     msb22)*power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(
     msb12)))/(cb*msb12) - (2*ca*mu*sa*power2(log(mH2/Q2))*power4(snb)*(-2*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(
     msb22))*power3(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,
     msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,
     msb22)*power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(
     msb12)))/msb12 + (2*mu*sb*power2(log(mH2/Q2))*power4(snb)*(-2*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,msb12))*(
     mH2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,msb22,
     msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,msb22)*
     power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/
     (cb*msb12) - (2*mu*sb*power2(sa)*power2(log(mH2/Q2))*power4(snb)*(-2*
     msb22*power2(DeltaInv(mH2,msb12,msb22))*power3(mH2) + 2*power3(mH2)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(
     msb22))*power3(DeltaInv(mH2,msb12,msb22)) + power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb12*msb14 + 4*power4(msb12)) + msb12*power2(DeltaInv(mH2,
     msb22,msb22))*(mH2*msb24 + 4*power3(msb22) + 16*mH2*DeltaInv(mH2,msb22,
     msb22)*power4(msb22)) + 16*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(
     msb12)))/(cb*msb12) + (mu*sb*power2(Pi)*power4(snb)*(-2*(3*mH2*msb22 +
     msb12*msb22 + 3*msb24)*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22)) +
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 6*power4(msb12)) + 2*
     power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*
     msb12*msb22*power2(mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*
     mH2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) + msb12*
     power2(DeltaInv(mH2,msb22,msb22))*(mH2*msb24 + 6*power3(msb22) + 24*mH2*
     DeltaInv(mH2,msb22,msb22)*power4(msb22)) + 24*mH2*power3(DeltaInv(mH2,
     msb12,msb12))*power5(msb12)))/(3.*cb*msb12) + (mu*sb*power4(snb)*(-(mH2*
     msb12*DeltaInv(mH2,msb12,msb12)) + 2*mH2*msb12*DeltaInv(mH2,msb12,msb22) -
     84*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22)) + 84*power2(mH2)*(-((5*msb12*msb22 + 3*msb24)*power2(mH2)) +
     3*msb22*power3(mH2) + (-5*msb12 + 3*msb22)*power3(msb22) - mH2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + 42*power2(
     DeltaInv(mH2,msb12,msb12))*(mH2*msb12*msb14 + 6*power4(msb12)) + msb12*
     DeltaInv(mH2,msb22,msb22)*(-mH2 + 42*DeltaInv(mH2,msb22,msb22)*(mH2*msb24
     + 6*power3(msb22)) + 1008*mH2*power2(DeltaInv(mH2,msb22,msb22))*power4(
     msb22)) + 1008*mH2*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(3.*
     cb*msb12) - (mu*sb*log(msb12/Q2)*power4(snb)*(-mH2 - msb12 - msb22 +
     msb12*(-mH2 + 6*msb12)*msb22*DeltaInv(mH2,msb12,msb12) + DeltaInv(mH2,
     msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(
     mH2) + 5*mH2*power2(msb22) + power3(mH2)) + 24*msb12*msb22*power2(
     DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 3*power3(msb12)) - 6*mH2*msb22*
     power2(DeltaInv(mH2,msb12,msb22))*(-(msb12*msb24) + mH2*(6*msb12*msb22 +
     23*msb24) - (msb12 - 23*msb22)*power2(mH2) + power3(mH2) + power3(msb22))
     + 72*msb22*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*
     msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + 288*mH2*msb22*
     power3(DeltaInv(mH2,msb12,msb12))*power5(msb12)))/(3.*cb*msb12*msb22) + (
     ca*mu*sa*log(mh2/Q2)*power2(snb)*(mh2 + msb12 + msb22 + (mh2 - 6*msb12)*
     msb12*msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*msb22*DeltaInv(mh2,
     msb22,msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22)*power2(
     mh2) - 6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22) + 6*msb22*power2(
     mh2)*(-3*msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*
     power2(DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*power2(
     DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,msb12,
     msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*
     power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,msb22,
     msb22))*power4(msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*
     power5(msb12) + 576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power5(
     msb22)))/(3.*msb12*msb22) + (mu*sb*log(mh2/Q2)*power2(sa)*power2(snb)*(mh2
     + msb12 + msb22 + (mh2 - 6*msb12)*msb12*msb22*DeltaInv(mh2,msb12,msb12) +
     mh2*msb12*msb22*DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 + 5*msb22)*
     DeltaInv(mh2,msb12,msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,msb22,msb22)*
     power2(msb22) + 6*msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(msb12 + 14*
     msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,msb12,msb22)) + 12*mh2*
     msb12*msb22*msb24*power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*
     msb22*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*
     power2(mh2) + power3(msb22))*power3(DeltaInv(mh2,msb12,msb22)) + 144*
     msb12*power2(DeltaInv(mh2,msb22,msb22))*power4(msb22) + 576*mh2*msb22*
     power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) + 576*mh2*msb12*power3(
     DeltaInv(mh2,msb22,msb22))*power5(msb22)))/(3.*cb*msb12*msb22) - (ca*mu*
     sa*log(mh2/Q2)*power4(snb)*(mh2 + msb12 + msb22 + (mh2 - 6*msb12)*msb12*
     msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*msb22*DeltaInv(mh2,msb22,
     msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22)*power2(mh2) -
     6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22) + 6*msb22*power2(mh2)*(-3*
     msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*power2(
     DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*power2(DeltaInv(mh2,
     msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,msb12,msb12))*(mh2*
     msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*msb12*msb24 - 2*mh2*(
     msb12*msb22 + msb24) + msb22*power2(mh2) + power3(msb22))*power3(DeltaInv(
     mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,msb22,msb22))*power4(
     msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,msb12))*power5(msb12) +
     576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*power5(msb22)))/(3.*msb12*
     msb22) - (mu*sb*log(mh2/Q2)*power2(sa)*power4(snb)*(mh2 + msb12 + msb22 +
     (mh2 - 6*msb12)*msb12*msb22*DeltaInv(mh2,msb12,msb12) + mh2*msb12*msb22*
     DeltaInv(mh2,msb22,msb22) - (mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,
     msb22)*power2(mh2) - 6*msb12*DeltaInv(mh2,msb22,msb22)*power2(msb22) + 6*
     msb22*power2(mh2)*(-3*msb12*msb22 - mh2*(msb12 + 14*msb22) + msb24 +
     power2(mh2))*power2(DeltaInv(mh2,msb12,msb22)) + 12*mh2*msb12*msb22*msb24*
     power2(DeltaInv(mh2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mh2,
     msb12,msb12))*(mh2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mh2)*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power3(
     msb22))*power3(DeltaInv(mh2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mh2,
     msb22,msb22))*power4(msb22) + 576*mh2*msb22*power3(DeltaInv(mh2,msb12,
     msb12))*power5(msb12) + 576*mh2*msb12*power3(DeltaInv(mh2,msb22,msb22))*
     power5(msb22)))/(3.*cb*msb12*msb22) + (ca*mu*sa*log(msb12/Q2)*power2(snb)*
     (-mh2 + mH2 + msb12*(-mh2 + 6*msb12)*msb22*DeltaInv(mh2,msb12,msb12) +
     mH2*msb12*msb22*DeltaInv(mH2,msb12,msb12) - msb22*msb24*DeltaInv(mH2,
     msb12,msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 5*msb22*
     DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb22*DeltaInv(mH2,msb12,msb12)*
     power2(msb12) - 5*mH2*DeltaInv(mH2,msb12,msb22)*power2(msb22) + msb12*
     DeltaInv(mH2,msb12,msb22)*power2(msb22) - 24*mH2*msb12*msb14*msb22*power2(
     DeltaInv(mH2,msb12,msb12)) - 6*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) + 138*msb22*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 36*msb12*power2(mH2)*power2(msb22)*power2(DeltaInv(mH2,msb12,
     msb22)) + DeltaInv(mh2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (
     msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(msb22) + power3(mh2)) -
     DeltaInv(mH2,msb12,msb22)*power3(mH2) - 6*msb12*msb22*power2(DeltaInv(mH2,
     msb12,msb22))*power3(mH2) + 138*power2(msb22)*power2(DeltaInv(mH2,msb12,
     msb22))*power3(mH2) + 24*msb12*msb22*power2(DeltaInv(mh2,msb12,msb12))*(
     mh2*msb14 + 3*power3(msb12)) - 6*mh2*msb22*power2(DeltaInv(mh2,msb12,
     msb22))*(-(msb12*msb24) + mh2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*
     msb22)*power2(mh2) + power3(mh2) + power3(msb22)) + 72*msb22*power2(mh2)*(
     -((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12
     + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)) + 720*msb12*msb22*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 144*msb22*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) + 216*msb12*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 72*msb22*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 6*mH2*
     power2(DeltaInv(mH2,msb12,msb22))*power4(msb22) + 216*msb12*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) + 144*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(msb22) - 144*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 288*mh2*msb22*power3(DeltaInv(
     mh2,msb12,msb12))*power5(msb12) - 288*mH2*msb22*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) - 144*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power5(msb22)))/(3.*msb12*msb22) + (mu*sb*log(msb12/Q2)*power2(sa)*power2(
     snb)*(-mh2 + mH2 + msb12*(-mh2 + 6*msb12)*msb22*DeltaInv(mh2,msb12,msb12)
     + mH2*msb12*msb22*DeltaInv(mH2,msb12,msb12) - msb22*msb24*DeltaInv(mH2,
     msb12,msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 5*msb22*
     DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb22*DeltaInv(mH2,msb12,msb12)*
     power2(msb12) - 5*mH2*DeltaInv(mH2,msb12,msb22)*power2(msb22) + msb12*
     DeltaInv(mH2,msb12,msb22)*power2(msb22) - 24*mH2*msb12*msb14*msb22*power2(
     DeltaInv(mH2,msb12,msb12)) - 6*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) + 138*msb22*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 36*msb12*power2(mH2)*power2(msb22)*power2(DeltaInv(mH2,msb12,
     msb22)) + DeltaInv(mh2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (
     msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(msb22) + power3(mh2)) -
     DeltaInv(mH2,msb12,msb22)*power3(mH2) - 6*msb12*msb22*power2(DeltaInv(mH2,
     msb12,msb22))*power3(mH2) + 138*power2(msb22)*power2(DeltaInv(mH2,msb12,
     msb22))*power3(mH2) + 24*msb12*msb22*power2(DeltaInv(mh2,msb12,msb12))*(
     mh2*msb14 + 3*power3(msb12)) - 6*mh2*msb22*power2(DeltaInv(mh2,msb12,
     msb22))*(-(msb12*msb24) + mh2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*
     msb22)*power2(mh2) + power3(mh2) + power3(msb22)) + 72*msb22*power2(mh2)*(
     -((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12
     + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)) + 720*msb12*msb22*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 144*msb22*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) + 216*msb12*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 72*msb22*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 6*mH2*
     power2(DeltaInv(mH2,msb12,msb22))*power4(msb22) + 216*msb12*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) + 144*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(msb22) - 144*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 288*mh2*msb22*power3(DeltaInv(
     mh2,msb12,msb12))*power5(msb12) - 288*mH2*msb22*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) - 144*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power5(msb22)))/(3.*cb*msb12*msb22) - (ca*mu*sa*log(msb12/Q2)*power4(snb)*
     (-mh2 + mH2 + msb12*(-mh2 + 6*msb12)*msb22*DeltaInv(mh2,msb12,msb12) +
     mH2*msb12*msb22*DeltaInv(mH2,msb12,msb12) - msb22*msb24*DeltaInv(mH2,
     msb12,msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 5*msb22*
     DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb22*DeltaInv(mH2,msb12,msb12)*
     power2(msb12) - 5*mH2*DeltaInv(mH2,msb12,msb22)*power2(msb22) + msb12*
     DeltaInv(mH2,msb12,msb22)*power2(msb22) - 24*mH2*msb12*msb14*msb22*power2(
     DeltaInv(mH2,msb12,msb12)) - 6*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) + 138*msb22*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 36*msb12*power2(mH2)*power2(msb22)*power2(DeltaInv(mH2,msb12,
     msb22)) + DeltaInv(mh2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (
     msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(msb22) + power3(mh2)) -
     DeltaInv(mH2,msb12,msb22)*power3(mH2) - 6*msb12*msb22*power2(DeltaInv(mH2,
     msb12,msb22))*power3(mH2) + 138*power2(msb22)*power2(DeltaInv(mH2,msb12,
     msb22))*power3(mH2) + 24*msb12*msb22*power2(DeltaInv(mh2,msb12,msb12))*(
     mh2*msb14 + 3*power3(msb12)) - 6*mh2*msb22*power2(DeltaInv(mh2,msb12,
     msb22))*(-(msb12*msb24) + mh2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*
     msb22)*power2(mh2) + power3(mh2) + power3(msb22)) + 72*msb22*power2(mh2)*(
     -((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12
     + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)) + 720*msb12*msb22*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 144*msb22*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) + 216*msb12*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 72*msb22*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 6*mH2*
     power2(DeltaInv(mH2,msb12,msb22))*power4(msb22) + 216*msb12*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) + 144*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(msb22) - 144*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 288*mh2*msb22*power3(DeltaInv(
     mh2,msb12,msb12))*power5(msb12) - 288*mH2*msb22*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) - 144*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power5(msb22)))/(3.*msb12*msb22) - (mu*sb*log(msb12/Q2)*power2(sa)*power4(
     snb)*(-mh2 + mH2 + msb12*(-mh2 + 6*msb12)*msb22*DeltaInv(mh2,msb12,msb12)
     + mH2*msb12*msb22*DeltaInv(mH2,msb12,msb12) - msb22*msb24*DeltaInv(mH2,
     msb12,msb22) + msb12*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 5*msb22*
     DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb22*DeltaInv(mH2,msb12,msb12)*
     power2(msb12) - 5*mH2*DeltaInv(mH2,msb12,msb22)*power2(msb22) + msb12*
     DeltaInv(mH2,msb12,msb22)*power2(msb22) - 24*mH2*msb12*msb14*msb22*power2(
     DeltaInv(mH2,msb12,msb12)) - 6*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,
     msb12,msb22)) + 138*msb22*msb24*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22)) + 36*msb12*power2(mH2)*power2(msb22)*power2(DeltaInv(mH2,msb12,
     msb22)) + DeltaInv(mh2,msb12,msb22)*(msb22*(-(msb12*msb22) + msb24) - (
     msb12 - 5*msb22)*power2(mh2) + 5*mh2*power2(msb22) + power3(mh2)) -
     DeltaInv(mH2,msb12,msb22)*power3(mH2) - 6*msb12*msb22*power2(DeltaInv(mH2,
     msb12,msb22))*power3(mH2) + 138*power2(msb22)*power2(DeltaInv(mH2,msb12,
     msb22))*power3(mH2) + 24*msb12*msb22*power2(DeltaInv(mh2,msb12,msb12))*(
     mh2*msb14 + 3*power3(msb12)) - 6*mh2*msb22*power2(DeltaInv(mh2,msb12,
     msb22))*(-(msb12*msb24) + mh2*(6*msb12*msb22 + 23*msb24) - (msb12 - 23*
     msb22)*power2(mh2) + power3(mh2) + power3(msb22)) + 72*msb22*power2(mh2)*(
     -((3*msb12*msb22 + 2*msb24)*power2(mh2)) + 2*msb22*power3(mh2) + (-3*msb12
     + 2*msb22)*power3(msb22) - 2*mh2*(5*msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mh2,msb12,msb22)) + 720*msb12*msb22*msb24*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22)) + 6*msb22*power2(DeltaInv(mH2,msb12,msb22))*
     power4(mH2) + 144*msb22*msb24*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) + 216*msb12*power2(msb22)*power3(DeltaInv(mH2,msb12,msb22))*power4(
     mH2) - 72*msb22*power2(DeltaInv(mH2,msb12,msb12))*power4(msb12) + 6*mH2*
     power2(DeltaInv(mH2,msb12,msb22))*power4(msb22) + 216*msb12*power2(mH2)*
     power3(DeltaInv(mH2,msb12,msb22))*power4(msb22) + 144*power3(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*power4(msb22) - 144*power2(msb22)*power3(
     DeltaInv(mH2,msb12,msb22))*power5(mH2) + 288*mh2*msb22*power3(DeltaInv(
     mh2,msb12,msb12))*power5(msb12) - 288*mH2*msb22*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) - 144*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*
     power5(msb22)))/(3.*cb*msb12*msb22) - (mu*sb*power2(Pi)*power2(snb)*(-2*
     msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*power2(mH2)*power2(DeltaInv(
     mH2,msb12,msb22)) + mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22)
     ) + msb12*msb22*mst12*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + 3*
     msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*msb22*
     mst12*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 3*msb12*mst14*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 3*msb22*mst12*power2(
     DeltaInv(msb12,mst12,mC2))*power3(mC2) - 3*msb12*mst12*power2(DeltaInv(
     msb22,mst12,mC2))*power3(mC2) + msb12*msb22*power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb14 + 6*power3(msb12)) + 14*msb12*msb22*mst14*power3(mC2)*
     power3(DeltaInv(msb12,mst12,mC2)) + 5*msb12*msb22*power2(mC2)*power3(
     mst12)*power3(DeltaInv(msb12,mst12,mC2)) + 3*msb22*power3(mC2)*power3(
     mst12)*power3(DeltaInv(msb12,mst12,mC2)) - 14*msb12*msb22*mst14*power3(
     mC2)*power3(DeltaInv(msb22,mst12,mC2)) - 5*msb12*msb22*power2(mC2)*power3(
     mst12)*power3(DeltaInv(msb22,mst12,mC2)) - 3*msb12*power3(mC2)*power3(
     mst12)*power3(DeltaInv(msb22,mst12,mC2)) + 5*msb12*msb22*mst12*power3(
     DeltaInv(msb12,mst12,mC2))*power4(mC2) + 3*msb22*mst14*power3(DeltaInv(
     msb12,mst12,mC2))*power4(mC2) - 5*msb12*msb22*mst12*power3(DeltaInv(msb22,
     mst12,mC2))*power4(mC2) - 3*msb12*mst14*power3(DeltaInv(msb22,mst12,mC2))*
     power4(mC2) + 6*msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 2*
     msb22*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 -
     5*msb12*msb22*power2(mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*
     mH2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 3*msb22*
     power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) + 3*msb12*
     power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*power4(mst12) - 3*msb22*
     mst12*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) + 3*msb12*mst12*
     power3(DeltaInv(msb22,mst12,mC2))*power5(mC2) + 24*mH2*msb22*power3(
     DeltaInv(mH2,msb12,msb12))*power5(msb12) + 24*mH2*msb12*power3(DeltaInv(
     mH2,msb22,msb22))*power5(msb22)))/(3.*cb*msb12*msb22) - (ca*mu*sa*log(mH2/
     Q2)*power2(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*
     DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (
     mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*
     DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*
     msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,
     msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22))
     + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(
     msb12)) + 72*msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*
     mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*
     power3(DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(3.*msb12*msb22) + (mu*
     sb*log(mH2/Q2)*power2(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*
     msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,
     msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) -
     6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*
     msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*power2(
     DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,
     msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,msb12))*(mH2*
     msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(
     msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(
     mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(
     msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) +
     576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(3.*cb*
     msb12*msb22) - (mu*sb*log(mH2/Q2)*power2(sa)*power2(snb)*(mH2 + msb12 +
     msb22 + (mH2 - 6*msb12)*msb12*msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*
     msb22*DeltaInv(mH2,msb22,msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,
     msb12,msb22)*power2(mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22)
     + 6*msb22*power2(mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 +
     power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*
     power2(DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,
     msb12,msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(
     msb22))*power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,
     msb22,msb22))*power4(msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) + 576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*
     power5(msb22)))/(3.*cb*msb12*msb22) + (ca*mu*sa*log(mH2/Q2)*power4(snb)*(
     mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*DeltaInv(mH2,msb12,
     msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (mH2 - msb12 + 5*
     msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*DeltaInv(mH2,msb22,
     msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*msb22 - mH2*(msb12 +
     14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 12*
     mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*
     power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*
     msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*
     power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,msb22)) + 144*
     msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*mH2*msb22*
     power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*power3(
     DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(3.*msb12*msb22) - (mu*sb*log(
     mH2/Q2)*power4(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*msb12)*msb12*msb22*
     DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - (
     mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*power2(mH2) - 6*msb12*
     DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*power2(mH2)*(-3*msb12*
     msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,
     msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,msb22,msb22))
     + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 12*power3(
     msb12)) + 72*msb22*power3(mH2)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 +
     msb24) + msb22*power2(mH2) + power3(msb22))*power3(DeltaInv(mH2,msb12,
     msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,msb22))*power4(msb22) + 576*
     mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) + 576*mH2*msb12*
     power3(DeltaInv(mH2,msb22,msb22))*power5(msb22)))/(3.*cb*msb12*msb22) + (
     mu*sb*log(mH2/Q2)*power2(sa)*power4(snb)*(mH2 + msb12 + msb22 + (mH2 - 6*
     msb12)*msb12*msb22*DeltaInv(mH2,msb12,msb12) + mH2*msb12*msb22*DeltaInv(
     mH2,msb22,msb22) - (mH2 - msb12 + 5*msb22)*DeltaInv(mH2,msb12,msb22)*
     power2(mH2) - 6*msb12*DeltaInv(mH2,msb22,msb22)*power2(msb22) + 6*msb22*
     power2(mH2)*(-3*msb12*msb22 - mH2*(msb12 + 14*msb22) + msb24 + power2(mH2)
     )*power2(DeltaInv(mH2,msb12,msb22)) + 12*mH2*msb12*msb22*msb24*power2(
     DeltaInv(mH2,msb22,msb22)) + 12*msb12*msb22*power2(DeltaInv(mH2,msb12,
     msb12))*(mH2*msb14 + 12*power3(msb12)) + 72*msb22*power3(mH2)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power3(msb22))*
     power3(DeltaInv(mH2,msb12,msb22)) + 144*msb12*power2(DeltaInv(mH2,msb22,
     msb22))*power4(msb22) + 576*mH2*msb22*power3(DeltaInv(mH2,msb12,msb12))*
     power5(msb12) + 576*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*power5(
     msb22)))/(3.*cb*msb12*msb22) - (mu*sb*power2(snb)*(-msb12 + msb22 - 2*mH2*
     msb12*msb22*DeltaInv(mH2,msb12,msb12) + 4*mH2*msb12*msb22*DeltaInv(mH2,
     msb12,msb22) - 2*mH2*msb12*msb22*DeltaInv(mH2,msb22,msb22) - 2*mC2*msb12*
     msb22*DeltaInv(msb12,mst12,mC2) + 2*mC2*msb12*msb22*DeltaInv(msb22,mst12,
     mC2) - 168*msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 84*mH2*msb12*msb22*msb24*power2(DeltaInv(mH2,
     msb22,msb22)) + 84*msb12*msb22*mst12*power2(mC2)*power2(DeltaInv(msb12,
     mst12,mC2)) + 252*msb22*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)
     ) - 84*msb12*msb22*mst12*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) -
     252*msb12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 252*msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2))*power3(mC2) - 252*msb12*mst12*
     power2(DeltaInv(msb22,mst12,mC2))*power3(mC2) + 84*msb12*msb22*power2(
     DeltaInv(mH2,msb12,msb12))*(mH2*msb14 + 6*power3(msb12)) + 1176*msb12*
     msb22*mst14*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 420*msb12*
     msb22*power2(mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) + 252*
     msb22*power3(mC2)*power3(mst12)*power3(DeltaInv(msb12,mst12,mC2)) - 1176*
     msb12*msb22*mst14*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2)) - 420*
     msb12*msb22*power2(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,mC2)) -
     252*msb12*power3(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,mC2)) +
     420*msb12*msb22*mst12*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 252*
     msb22*mst14*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) - 420*msb12*
     msb22*mst12*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) - 252*msb12*
     mst14*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) + 504*msb12*power2(
     DeltaInv(mH2,msb22,msb22))*power4(msb22) + 168*msb22*power2(mH2)*power3(
     DeltaInv(mH2,msb12,msb22))*(-14*mH2*msb12*msb24 - 5*msb12*msb22*power2(
     mH2) - 3*msb24*power2(mH2) + 3*msb22*power3(mH2) - 3*mH2*power3(msb22) -
     5*msb12*power3(msb22) + 3*power4(msb22)) - 252*msb22*power2(mC2)*power3(
     DeltaInv(msb12,mst12,mC2))*power4(mst12) + 252*msb12*power2(mC2)*power3(
     DeltaInv(msb22,mst12,mC2))*power4(mst12) - 252*msb22*mst12*power3(
     DeltaInv(msb12,mst12,mC2))*power5(mC2) + 252*msb12*mst12*power3(DeltaInv(
     msb22,mst12,mC2))*power5(mC2) + 2016*mH2*msb22*power3(DeltaInv(mH2,msb12,
     msb12))*power5(msb12) + 2016*mH2*msb12*power3(DeltaInv(mH2,msb22,msb22))*
     power5(msb22)))/(6.*cb*msb12*msb22) + (mu*sb*log(msb22/Q2)*power2(snb)*(-
     2*msb22*(5*mH2*msb22 - msb12*msb22 + msb24)*mst12*DeltaInv(mH2,msb12,
     msb22) + 12*mH2*msb22*mst12*power2(DeltaInv(mH2,msb12,msb22))*(-3*mH2*
     msb12*msb22 - 14*mH2*msb24 - msb12*msb24 + msb22*power2(mH2) + power3(
     msb22)) + 144*msb22*mst12*power2(mH2)*(msb24*power2(mH2) + (-2*msb12 +
     msb22)*power3(msb22) - 2*mH2*(msb12*msb24 + power3(msb22)))*power3(
     DeltaInv(mH2,msb12,msb22)) + msb12*(-mC2 - msb22 - mst12 - 2*(mH2 - 6*
     msb22)*msb22*mst12*DeltaInv(mH2,msb22,msb22) + DeltaInv(msb22,mst12,mC2)*(
     mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) + 48*msb22*mst12*power2(DeltaInv(mH2,msb22,
     msb22))*(mH2*msb24 + 3*power3(msb22)) - 6*mC2*mst12*power2(DeltaInv(msb22,
     mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) + 72*mst12*power2(
     mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-
     3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + 576*mH2*mst12*power3(DeltaInv(mH2,
     msb22,msb22))*power5(msb22))))/(6.*cb*msb12*msb22*mst12) + (mu*sb*log(
     msb12/Q2)*power2(snb)*(mC2*msb22 + msb12*msb22 - 2*mH2*mst12 - 2*msb12*
     mst12 - msb22*mst12 - 2*(mH2 - 6*msb12)*msb12*msb22*mst12*DeltaInv(mH2,
     msb12,msb12) - msb22*mst12*mst14*DeltaInv(msb12,mst12,mC2) + msb12*msb22*
     DeltaInv(msb12,mst12,mC2)*power2(mC2) - 5*msb22*mst12*DeltaInv(msb12,
     mst12,mC2)*power2(mC2) - 5*mC2*msb22*DeltaInv(msb12,mst12,mC2)*power2(
     mst12) + msb12*msb22*DeltaInv(msb12,mst12,mC2)*power2(mst12) - 6*mC2*
     msb12*msb22*mst12*mst14*power2(DeltaInv(msb12,mst12,mC2)) + 138*msb22*
     mst12*mst14*power2(mC2)*power2(DeltaInv(msb12,mst12,mC2)) + 36*msb12*
     msb22*power2(mC2)*power2(mst12)*power2(DeltaInv(msb12,mst12,mC2)) - msb22*
     DeltaInv(msb12,mst12,mC2)*power3(mC2) - 6*msb12*msb22*mst12*power2(
     DeltaInv(msb12,mst12,mC2))*power3(mC2) + 138*msb22*power2(mst12)*power2(
     DeltaInv(msb12,mst12,mC2))*power3(mC2) + 2*mst12*DeltaInv(mH2,msb12,msb22)
     *(msb22*(-(msb12*msb22) + msb24) - (msb12 - 5*msb22)*power2(mH2) + 5*mH2*
     power2(msb22) + power3(mH2)) + 48*msb12*msb22*mst12*power2(DeltaInv(mH2,
     msb12,msb12))*(mH2*msb14 + 3*power3(msb12)) - 12*mH2*msb22*mst12*power2(
     DeltaInv(mH2,msb12,msb22))*(-(msb12*msb24) + mH2*(6*msb12*msb22 + 23*
     msb24) - (msb12 - 23*msb22)*power2(mH2) + power3(mH2) + power3(msb22)) +
     144*msb22*mst12*power2(mH2)*(-((3*msb12*msb22 + 2*msb24)*power2(mH2)) + 2*
     msb22*power3(mH2) + (-3*msb12 + 2*msb22)*power3(msb22) - 2*mH2*(5*msb12*
     msb24 + power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + 720*msb12*
     msb22*mst12*mst14*power3(mC2)*power3(DeltaInv(msb12,mst12,mC2)) + 6*msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 144*msb22*mst12*
     mst14*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 216*msb12*msb22*
     power2(mst12)*power3(DeltaInv(msb12,mst12,mC2))*power4(mC2) + 6*mC2*msb22*
     power2(DeltaInv(msb12,mst12,mC2))*power4(mst12) + 216*msb12*msb22*power2(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) + 144*msb22*power3(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*power4(mst12) - 144*msb22*power2(
     mst12)*power3(DeltaInv(msb12,mst12,mC2))*power5(mC2) + 576*mH2*msb22*
     mst12*power3(DeltaInv(mH2,msb12,msb12))*power5(msb12) - 144*msb22*power2(
     mC2)*power3(DeltaInv(msb12,mst12,mC2))*power5(mst12)))/(6.*cb*msb12*msb22*
     mst12) - (mu*sb*(msb12*msb22 + 2*msb12*mst12 + msb22*mst12 - mH2*msb12*
     msb22*mst12*DeltaInv(mH2,msb12,msb22) - mA2*msb12*msb22*mst12*DeltaInv(
     msb12,mA2,msb22) - 2*mC2*msb12*msb22*mst12*DeltaInv(msb22,mst12,mC2) + 42*
     msb22*(3*mH2*msb22 + msb12*msb22 + 3*msb24)*mst12*power2(mH2)*power2(
     DeltaInv(mH2,msb12,msb22)) + 42*msb22*(3*mA2*msb22 + msb12*msb22 + 3*
     msb24)*mst12*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)) + 252*msb12*
     mst12*mst14*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) + 84*msb12*
     msb22*power2(mC2)*power2(mst12)*power2(DeltaInv(msb22,mst12,mC2)) + 252*
     msb12*power2(mst12)*power2(DeltaInv(msb22,mst12,mC2))*power3(mC2) - 42*
     msb22*mst12*power2(mH2)*(-((5*msb12*msb22 + 3*msb24)*power2(mH2)) + 3*
     msb22*power3(mH2) + (-5*msb12 + 3*msb22)*power3(msb22) - mH2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(mH2,msb12,msb22)) + 1176*msb12*
     msb22*mst12*mst14*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2)) + 252*
     msb12*mst12*mst14*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) + 420*
     msb12*msb22*power2(mst12)*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) -
     42*msb22*mst12*power2(mA2)*power3(DeltaInv(msb12,mA2,msb22))*(-14*mA2*
     msb12*msb24 - 5*msb12*msb22*power2(mA2) - 3*msb24*power2(mA2) + 3*msb22*
     power3(mA2) - 3*mA2*power3(msb22) - 5*msb12*power3(msb22) + 3*power4(
     msb22)) + 420*msb12*msb22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*
     power4(mst12) + 252*msb12*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2))*
     power4(mst12) - 252*msb12*power2(mst12)*power3(DeltaInv(msb22,mst12,mC2))*
     power5(mC2) - 252*msb12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*
     power5(mst12)))/(6.*cb*msb12*msb22*mst12) + (cb*mu*sb*log(msb12/Q2)*
     power2(snb)*power2(snt)*(mC2*mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 +
     mst12*mst22*mst24*DeltaInv(mst22,mw2,msb12) - mst12*mst14*mst22*DeltaInv(
     mw2,msb12,mst12) + msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) -
     5*mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) - msb12*mst12*
     DeltaInv(mst22,mw2,msb12)*power2(mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,
     msb12)*power2(mst22) - msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) +
     5*mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) + msb12*mst22*
     DeltaInv(mw2,msb12,mst12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb12,
     mst12)*power2(mw2) + 6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,
     mw2,msb12)) - 138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) - 36*msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) - 6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12)) + 138*mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + 36*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) + mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) +
     mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) - mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (
     msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*
     msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb12)*
     power3(mw2) - mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) + 6*msb12*mst12*
     mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 138*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) - 6*msb12*mst12*
     mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 138*mst22*power2(
     mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 72*mst12*mst22*
     power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-
     ((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12
     + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) - 720*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) + 720*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) + 216*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) - 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     - 216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     + 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*mst22) + (cb*mu*sb*log(msb12/Q2)*power2(snb)*
     power2(snt)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*
     mst22*mst24*DeltaInv(mst22,mw2,msb12) + mst12*mst14*mst22*DeltaInv(mw2,
     msb12,mst12) - msb12*mst22*DeltaInv(mw2,msb12,mst12)*power2(mst12) + 5*
     mst22*mw2*DeltaInv(mw2,msb12,mst12)*power2(mst12) + msb12*mst12*DeltaInv(
     mst22,mw2,msb12)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb12)*
     power2(mst22) + msb12*mst12*DeltaInv(mst22,mw2,msb12)*power2(mw2) - 5*
     mst12*mst22*DeltaInv(mst22,mw2,msb12)*power2(mw2) - msb12*mst22*DeltaInv(
     mw2,msb12,mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb12,mst12)*
     power2(mw2) - 6*msb12*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,
     msb12)) + 138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb12)) + 36*msb12*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 6*msb12*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb12,
     mst12)) - 138*mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) - 36*msb12*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,
     msb12,mst12)) - mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) +
     mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (
     msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*
     msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(
     msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb12)*
     power3(mw2) + mst22*DeltaInv(mw2,msb12,mst12)*power3(mw2) - 6*msb12*mst12*
     mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 138*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 6*msb12*mst12*
     mst22*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 138*mst22*power2(
     mst12)*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 72*mst12*mst22*
     power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-
     ((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12
     + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2)) + 720*msb12*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb12)) - 720*msb12*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 216*msb12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 216*msb12*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb12))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb12)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2)
     + 216*msb12*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2)
     - 216*msb12*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb12))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb12,mst12))*power5(
     mw2)))/(6.*msb12*mst12*mst22) + (cb*mu*sb*power2(Pi)*power2(snb)*power2(
     snt)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2)) - msb12*msb22*mst22*power2(
     mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2)) + msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb12)) + 3*msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12))
     - msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 3*
     msb12*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - msb12*msb22*
     mst12*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - 3*msb22*mst14*
     power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 3*msb12*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 3*msb22*mst22*power2(DeltaInv(mst22,msb12,
     mC2))*power3(mC2) + 3*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2) + 3*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2)
     - 3*msb12*mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 3*msb22*
     mst12*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) + 3*msb12*mst12*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*power2(mC2)*((5*
     msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2) + (5*msb12 - 3*
     mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) - 14*msb12*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb12,mC2)) + 14*msb12*msb22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb12)) + 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) + 3*msb22*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 5*msb12*msb22*power2(mw2)*
     power3(mst22)*power3(DeltaInv(mst22,mw2,msb22)) - 14*msb12*msb22*mst24*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 3*msb12*power3(mst22)*
     power3(mw2)*power3(DeltaInv(mst22,mw2,msb22)) - 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) - 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 3*msb22*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 5*msb12*msb22*power2(mw2)*
     power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) + 14*msb12*msb22*mst14*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 3*msb12*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 3*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) + 5*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) + 3*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 3*msb22*power2(mw2)*power3(DeltaInv(mw2,
     msb12,mst12))*power4(mst12) - 3*msb12*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + msb12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*
     power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(
     mst12) + 3*power4(mst12)) + 3*msb22*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2))*power4(mst22) - 3*msb12*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22) - 3*msb22*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mst22) + 3*msb12*power2(mw2)*power3(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power4(mw2) + 3*msb22*mst24*power3(DeltaInv(mst22,mw2,msb12))*
     power4(mw2) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power4(mw2) - 3*msb12*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) - 3*
     msb22*mst14*power3(DeltaInv(mw2,msb12,mst12))*power4(mw2) + 5*msb12*msb22*
     mst12*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb12*mst14*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 3*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2))*power5(mC2) - 3*msb12*mst22*power3(DeltaInv(
     mst22,msb22,mC2))*power5(mC2) - 3*msb22*mst22*power3(DeltaInv(mst22,mw2,
     msb12))*power5(mw2) + 3*msb12*mst22*power3(DeltaInv(mst22,mw2,msb22))*
     power5(mw2) + 3*msb22*mst12*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)
     - 3*msb12*mst12*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*
     msb22) + (cb*mu*sb*power2(Pi)*power2(snb)*power2(snt)*(-(msb22*(3*mC2*
     mst12 + msb12*mst12 + 3*mst14)*power2(mC2)*power2(DeltaInv(msb12,mst12,
     mC2))) + msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2)) + msb12*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(mC2)*power2(DeltaInv(
     mst22,msb12,mC2)) - msb12*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2)) - 3*msb12*mst24*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))
     - msb12*msb22*mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) - 3*
     msb22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb12)) + msb12*msb22*
     mst22*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 3*msb12*mst24*
     power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + msb12*msb22*mst12*power2(
     mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 3*msb22*mst14*power2(mw2)*power2(
     DeltaInv(mw2,msb12,mst12)) - msb12*msb22*mst12*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) - 3*msb12*mst14*power2(mw2)*power2(DeltaInv(
     mw2,msb22,mst12)) + 3*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))*
     power3(mC2) - 3*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2)
     - 3*msb22*mst22*power2(DeltaInv(mst22,mw2,msb12))*power3(mw2) + 3*msb12*
     mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 3*msb22*mst12*
     power2(DeltaInv(mw2,msb12,mst12))*power3(mw2) - 3*msb12*mst12*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2) + msb22*power2(mC2)*(-((5*msb12*
     mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)
     *power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) + 14*msb12*msb22*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 5*msb12*msb22*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*power3(mC2)*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb12)) - 14*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb12)) - 3*msb22*power3(mst22)*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb12)) + 5*msb12*msb22*power2(mw2)*power3(
     mst22)*power3(DeltaInv(mst22,mw2,msb22)) + 14*msb12*msb22*mst24*power3(
     mw2)*power3(DeltaInv(mst22,mw2,msb22)) + 3*msb12*power3(mst22)*power3(mw2)
     *power3(DeltaInv(mst22,mw2,msb22)) + 5*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 14*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 3*msb22*power3(mst12)*power3(mw2)
     *power3(DeltaInv(mw2,msb12,mst12)) - 5*msb12*msb22*power2(mw2)*power3(
     mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst14*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*power3(mst12)*power3(mw2)
     *power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) + 3*msb22*mst24*power3(DeltaInv(mst22,msb12,
     mC2))*power4(mC2) - 5*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - 3*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2)
     + msb12*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(14*mC2*msb22*mst14
     + 5*msb22*mst12*power2(mC2) + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) +
     3*mC2*power3(mst12) + 5*msb22*power3(mst12) - 3*power4(mst12)) - 3*msb22*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 3*msb22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb12*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 3*msb22*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb12))*power4(mst22) - 3*msb12*
     power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power4(mst22) - 5*msb12*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) - 3*msb22*mst24*
     power3(DeltaInv(mst22,mw2,msb12))*power4(mw2) + 5*msb12*msb22*mst22*
     power3(DeltaInv(mst22,mw2,msb22))*power4(mw2) + 3*msb12*mst24*power3(
     DeltaInv(mst22,mw2,msb22))*power4(mw2) + 5*msb12*msb22*mst12*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + 3*msb22*mst14*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2) - 5*msb12*msb22*mst12*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mw2) - 3*msb12*mst14*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mw2) - 3*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power5(
     mC2) + 3*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power5(mC2) + 3*
     msb22*mst22*power3(DeltaInv(mst22,mw2,msb12))*power5(mw2) - 3*msb12*mst22*
     power3(DeltaInv(mst22,mw2,msb22))*power5(mw2) - 3*msb22*mst12*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2) + 3*msb12*mst12*power3(DeltaInv(
     mw2,msb22,mst12))*power5(mw2)))/(3.*msb12*msb22) + (cb*mu*sb*log(msb22/Q2)
     *power2(snt)*(mC2*mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*
     mst24*DeltaInv(mst22,mw2,msb22) - mst12*mst14*mst22*DeltaInv(mw2,msb22,
     mst12) + msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*
     mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) - msb22*mst12*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(
     mst22) - msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*
     mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(
     mw2) + 6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) -
     138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     - mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*mst22) + (cb*mu*sb*log(msb22/Q2)*power2(snb)*
     power2(snt)*(mC2*mst12 - mC2*mst22 - mst12*mw2 + mst22*mw2 + mst12*mst22*
     mst24*DeltaInv(mst22,mw2,msb22) - mst12*mst14*mst22*DeltaInv(mw2,msb22,
     mst12) + msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) - 5*mst22*
     mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) - msb22*mst12*DeltaInv(mst22,
     mw2,msb22)*power2(mst22) + 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(
     mst22) - msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) + 5*mst12*
     mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) + msb22*mst22*DeltaInv(mw2,
     msb22,mst12)*power2(mw2) - 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(
     mw2) + 6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) -
     138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) - 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) -
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) + 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) +
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     - mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) + 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) - 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) + 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) + 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) - 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) - 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) + 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     - 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) + 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     + 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) - 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) + 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) + 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) - 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*mst22) + (cb*mu*sb*log(msb22/Q2)*power2(snt)*(-(
     mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*mst22*mst24*
     DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*DeltaInv(mw2,msb22,mst12) -
     msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*mst22*mw2*
     DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*DeltaInv(mst22,mw2,
     msb22)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*power2(mst22)
     + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*mst12*mst22*
     DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*DeltaInv(mw2,msb22,
     mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*power2(mw2) -
     6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,msb22)) + 138*
     mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) + 36*
     msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,mw2,msb22)) +
     6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,mst12)) - 138*
     mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 36*
     msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) -
     mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*
     DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*
     mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*
     mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*
     mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb22)*power3(mw2)
     + mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*mst22*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*power2(mst22)*
     power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*mst22*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*power2(mst12)*
     power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*mst22*power2(mC2)
     *(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*
     msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*
     power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*
     mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     + 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*mst22) + (cb*mu*sb*log(msb22/Q2)*power2(snb)*
     power2(snt)*(-(mC2*mst12) + mC2*mst22 + mst12*mw2 - mst22*mw2 - mst12*
     mst22*mst24*DeltaInv(mst22,mw2,msb22) + mst12*mst14*mst22*DeltaInv(mw2,
     msb22,mst12) - msb22*mst22*DeltaInv(mw2,msb22,mst12)*power2(mst12) + 5*
     mst22*mw2*DeltaInv(mw2,msb22,mst12)*power2(mst12) + msb22*mst12*DeltaInv(
     mst22,mw2,msb22)*power2(mst22) - 5*mst12*mw2*DeltaInv(mst22,mw2,msb22)*
     power2(mst22) + msb22*mst12*DeltaInv(mst22,mw2,msb22)*power2(mw2) - 5*
     mst12*mst22*DeltaInv(mst22,mw2,msb22)*power2(mw2) - msb22*mst22*DeltaInv(
     mw2,msb22,mst12)*power2(mw2) + 5*mst12*mst22*DeltaInv(mw2,msb22,mst12)*
     power2(mw2) - 6*msb22*mst12*mst22*mst24*mw2*power2(DeltaInv(mst22,mw2,
     msb22)) + 138*mst12*mst22*mst24*power2(mw2)*power2(DeltaInv(mst22,mw2,
     msb22)) + 36*msb22*mst12*power2(mst22)*power2(mw2)*power2(DeltaInv(mst22,
     mw2,msb22)) + 6*msb22*mst12*mst14*mst22*mw2*power2(DeltaInv(mw2,msb22,
     mst12)) - 138*mst12*mst14*mst22*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) - 36*msb22*mst22*power2(mst12)*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) - mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) +
     mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (
     msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*
     msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(
     msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - mst12*DeltaInv(mst22,mw2,msb22)*
     power3(mw2) + mst22*DeltaInv(mw2,msb22,mst12)*power3(mw2) - 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 138*mst12*power2(
     mst22)*power2(DeltaInv(mst22,mw2,msb22))*power3(mw2) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 138*mst22*power2(
     mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) - 72*mst12*mst22*
     power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(
     mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(
     mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-
     ((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22
     + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb22,mC2)) + 720*msb22*mst12*mst22*mst24*power3(mw2)*
     power3(DeltaInv(mst22,mw2,msb22)) - 720*msb22*mst12*mst14*mst22*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 6*mst22*mw2*power2(DeltaInv(mw2,
     msb22,mst12))*power4(mst12) - 216*msb22*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) - 144*mst22*power3(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mst12) + 6*mst12*mw2*power2(DeltaInv(mst22,mw2,
     msb22))*power4(mst22) + 216*msb22*mst12*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 144*mst12*power3(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*power4(mst22) + 6*mst12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     )*power4(mw2) - 6*mst12*mst22*power2(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst12*mst22*mst24*power3(DeltaInv(mst22,mw2,msb22))*power4(mw2)
     + 216*msb22*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power4(
     mw2) - 144*mst12*mst14*mst22*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2)
     - 216*msb22*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mw2) + 144*mst22*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mst12) - 144*mst12*power2(mw2)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mst22) - 144*mst12*power2(mst22)*power3(DeltaInv(mst22,mw2,msb22))*power5(
     mw2) + 144*mst22*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power5(
     mw2)))/(6.*msb22*mst12*mst22) + Fin3(mh2,msb12,msb12,Q2)*(-4*ca*mu*sa*
     power2(snb)*power2(DeltaInv(mh2,msb12,msb12))*(msb14 + 4*mh2*DeltaInv(mh2,
     msb12,msb12)*power6(msb1)) - (4*mu*sb*power2(sa)*power2(snb)*power2(
     DeltaInv(mh2,msb12,msb12))*(msb14 + 4*mh2*DeltaInv(mh2,msb12,msb12)*
     power6(msb1)))/cb + 4*ca*mu*sa*power2(DeltaInv(mh2,msb12,msb12))*power4(
     snb)*(msb14 + 4*mh2*DeltaInv(mh2,msb12,msb12)*power6(msb1)) + (4*mu*sb*
     power2(sa)*power2(DeltaInv(mh2,msb12,msb12))*power4(snb)*(msb14 + 4*mh2*
     DeltaInv(mh2,msb12,msb12)*power6(msb1)))/cb) + Fin3(mH2,msb12,msb12,Q2)*(
     4*ca*mu*sa*power2(snb)*power2(DeltaInv(mH2,msb12,msb12))*(msb14 + 4*mH2*
     DeltaInv(mH2,msb12,msb12)*power6(msb1)) - (4*mu*sb*power2(snb)*power2(
     DeltaInv(mH2,msb12,msb12))*(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*
     power6(msb1)))/cb + (4*mu*sb*power2(sa)*power2(snb)*power2(DeltaInv(mH2,
     msb12,msb12))*(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*power6(msb1)))/cb -
     4*ca*mu*sa*power2(DeltaInv(mH2,msb12,msb12))*power4(snb)*(msb14 + 4*mH2*
     DeltaInv(mH2,msb12,msb12)*power6(msb1)) + (4*mu*sb*power2(DeltaInv(mH2,
     msb12,msb12))*power4(snb)*(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*power6(
     msb1)))/cb - (4*mu*sb*power2(sa)*power2(DeltaInv(mH2,msb12,msb12))*power4(
     snb)*(msb14 + 4*mH2*DeltaInv(mH2,msb12,msb12)*power6(msb1)))/cb) + (cb*mu*
     sb*power2(DeltaInv(msb12,mz2,msb22))*power2(log(mz2/Q2))*power3(mz2)*(
     msb22 - DeltaInv(msb12,mz2,msb22)*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(
     msb24 + msb22*mz2) + power6(msb2))))/msb12 + (cb*mu*sb*power2(DeltaInv(
     msb12,mz2,msb22))*power2(log(mz2/Q2))*power3(mz2)*(-msb22 + DeltaInv(
     msb12,mz2,msb22)*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2)
     + power6(msb2))))/msb12 + (cb*mu*mz2*sb*log(mz2/Q2)*(-1 + mz2*(-msb12 + 5*
     msb22 + mz2)*DeltaInv(msb12,mz2,msb22) + 6*msb22*mz2*(-msb24 + (14*msb22 -
     mz2)*mz2 + msb12*(3*msb22 + mz2))*power2(DeltaInv(msb12,mz2,msb22)) - 72*
     msb22*power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(mz2*(-2*msb24 +
     msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/(12.*msb12*
     msb22) + (cb*mu*mz2*sb*log(mz2/Q2)*(1 + (msb12 - 5*msb22 - mz2)*mz2*
     DeltaInv(msb12,mz2,msb22) - 6*msb22*mz2*(-msb24 + (14*msb22 - mz2)*mz2 +
     msb12*(3*msb22 + mz2))*power2(DeltaInv(msb12,mz2,msb22)) + 72*msb22*
     power2(mz2)*power3(DeltaInv(msb12,mz2,msb22))*(mz2*(-2*msb24 + msb22*mz2)
     - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/(12.*msb12*msb22) + (cb*
     mu*sb*power2(DeltaInv(msb12,mA2,msb22))*power2(log(mA2/Q2))*power3(mA2)*(
     msb22 - DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 +
     msb24) + msb22*power2(mA2) + power6(msb2))))/msb12 + (mu*sb*power2(
     DeltaInv(msb12,mA2,msb22))*power2(log(mA2/Q2))*power3(mA2)*(-msb22 +
     DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) +
     msb22*power2(mA2) + power6(msb2))))/(cb*msb12) + (cb*mu*sb*power2(
     DeltaInv(msb12,mA2,msb22))*power2(log(mA2/Q2))*power3(mA2)*(-msb22 +
     DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) +
     msb22*power2(mA2) + power6(msb2))))/msb12 + (mA2*mu*sb*log(mA2/Q2)*(-1 +
     mA2*(mA2 - msb12 + 5*msb22)*DeltaInv(msb12,mA2,msb22) - 6*mA2*msb22*(-3*
     msb12*msb22 - mA2*(msb12 + 14*msb22) + msb24 + power2(mA2))*power2(
     DeltaInv(msb12,mA2,msb22)) - 72*msb22*power2(mA2)*power3(DeltaInv(msb12,
     mA2,msb22))*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(
     mA2) + power6(msb2))))/(12.*cb*msb12*msb22) + (cb*mA2*mu*sb*log(mA2/Q2)*(-
     1 + mA2*(mA2 - msb12 + 5*msb22)*DeltaInv(msb12,mA2,msb22) - 6*mA2*msb22*(-
     3*msb12*msb22 - mA2*(msb12 + 14*msb22) + msb24 + power2(mA2))*power2(
     DeltaInv(msb12,mA2,msb22)) - 72*msb22*power2(mA2)*power3(DeltaInv(msb12,
     mA2,msb22))*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(
     mA2) + power6(msb2))))/(12.*msb12*msb22) + (cb*mA2*mu*sb*log(mA2/Q2)*(1 -
     mA2*(mA2 - msb12 + 5*msb22)*DeltaInv(msb12,mA2,msb22) + 6*mA2*msb22*(-3*
     msb12*msb22 - mA2*(msb12 + 14*msb22) + msb24 + power2(mA2))*power2(
     DeltaInv(msb12,mA2,msb22)) + 72*msb22*power2(mA2)*power3(DeltaInv(msb12,
     mA2,msb22))*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(
     mA2) + power6(msb2))))/(12.*msb12*msb22) + (ca*mu*sa*power2(DeltaInv(mh2,
     msb12,msb22))*power2(log(mh2/Q2))*power3(mh2)*(-msb22 + DeltaInv(mh2,
     msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(
     mh2) + power6(msb2))))/msb12 + (mu*sb*power2(sa)*power2(DeltaInv(mh2,
     msb12,msb22))*power2(log(mh2/Q2))*power3(mh2)*(-msb22 + DeltaInv(mh2,
     msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(
     mh2) + power6(msb2))))/(cb*msb12) + (ca*mh2*mu*sa*log(mh2/Q2)*(-1 + mh2*(
     mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22) - 6*mh2*msb22*(-3*msb12*
     msb22 - mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,
     msb12,msb22)) - 72*msb22*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-
     2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(
     msb2))))/(12.*msb12*msb22) + (mh2*mu*sb*log(mh2/Q2)*power2(sa)*(-1 + mh2*(
     mh2 - msb12 + 5*msb22)*DeltaInv(mh2,msb12,msb22) - 6*mh2*msb22*(-3*msb12*
     msb22 - mh2*(msb12 + 14*msb22) + msb24 + power2(mh2))*power2(DeltaInv(mh2,
     msb12,msb22)) - 72*msb22*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-
     2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(
     msb2))))/(12.*cb*msb12*msb22) + (ca*mu*sa*power2(DeltaInv(mH2,msb12,msb22)
     )*power2(log(mH2/Q2))*power3(mH2)*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(
     msb2))))/msb12 + (mu*sb*power2(sa)*power2(DeltaInv(mH2,msb12,msb22))*
     power2(log(mH2/Q2))*power3(mH2)*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(
     msb2))))/(cb*msb12) + (mu*sb*power2(DeltaInv(mH2,msb12,msb22))*power2(log(
     mH2/Q2))*power3(mH2)*(-msb22 + DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(cb*
     msb12) + (mH2*mu*sb*log(mH2/Q2)*(-1 + mH2*(mH2 - msb12 + 5*msb22)*
     DeltaInv(mH2,msb12,msb22) - 6*mH2*msb22*(-3*msb12*msb22 - mH2*(msb12 + 14*
     msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) - 72*
     msb22*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 - 2*
     mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(12.*cb*
     msb12*msb22) + (ca*mH2*mu*sa*log(mH2/Q2)*(1 - mH2*(mH2 - msb12 + 5*msb22)*
     DeltaInv(mH2,msb12,msb22) + 6*mH2*msb22*(-3*msb12*msb22 - mH2*(msb12 + 14*
     msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22)) + 72*
     msb22*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 - 2*
     mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(12.*
     msb12*msb22) + (mH2*mu*sb*log(mH2/Q2)*power2(sa)*(1 - mH2*(mH2 - msb12 +
     5*msb22)*DeltaInv(mH2,msb12,msb22) + 6*mH2*msb22*(-3*msb12*msb22 - mH2*(
     msb12 + 14*msb22) + msb24 + power2(mH2))*power2(DeltaInv(mH2,msb12,msb22))
     + 72*msb22*power2(mH2)*power3(DeltaInv(mH2,msb12,msb22))*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(12.*cb*
     msb12*msb22) + Fin3(mh2,msb22,msb22,Q2)*(-4*ca*mu*sa*power2(snb)*power2(
     DeltaInv(mh2,msb22,msb22))*(msb24 + 4*mh2*DeltaInv(mh2,msb22,msb22)*
     power6(msb2)) - (4*mu*sb*power2(sa)*power2(snb)*power2(DeltaInv(mh2,msb22,
     msb22))*(msb24 + 4*mh2*DeltaInv(mh2,msb22,msb22)*power6(msb2)))/cb + 4*ca*
     mu*sa*power2(DeltaInv(mh2,msb22,msb22))*power4(snb)*(msb24 + 4*mh2*
     DeltaInv(mh2,msb22,msb22)*power6(msb2)) + (4*mu*sb*power2(sa)*power2(
     DeltaInv(mh2,msb22,msb22))*power4(snb)*(msb24 + 4*mh2*DeltaInv(mh2,msb22,
     msb22)*power6(msb2)))/cb) + Fin3(mH2,msb22,msb22,Q2)*(4*ca*mu*sa*power2(
     snb)*power2(DeltaInv(mH2,msb22,msb22))*(msb24 + 4*mH2*DeltaInv(mH2,msb22,
     msb22)*power6(msb2)) - (4*mu*sb*power2(snb)*power2(DeltaInv(mH2,msb22,
     msb22))*(msb24 + 4*mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2)))/cb + (4*
     mu*sb*power2(sa)*power2(snb)*power2(DeltaInv(mH2,msb22,msb22))*(msb24 + 4*
     mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2)))/cb - 4*ca*mu*sa*power2(
     DeltaInv(mH2,msb22,msb22))*power4(snb)*(msb24 + 4*mH2*DeltaInv(mH2,msb22,
     msb22)*power6(msb2)) + (4*mu*sb*power2(DeltaInv(mH2,msb22,msb22))*power4(
     snb)*(msb24 + 4*mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2)))/cb - (4*mu*
     sb*power2(sa)*power2(DeltaInv(mH2,msb22,msb22))*power4(snb)*(msb24 + 4*
     mH2*DeltaInv(mH2,msb22,msb22)*power6(msb2)))/cb) + Fin3(msb12,mz2,msb22,
     Q2)*((2*cb*mu*sb*power2(mz2)*power2(DeltaInv(msb12,mz2,msb22))*(msb22 -
     DeltaInv(msb12,mz2,msb22)*(mz2*(-2*msb24 + msb22*mz2) - 2*msb12*(msb24 +
     msb22*mz2) + power6(msb2))))/msb12 + (2*cb*mu*sb*power2(mz2)*power2(
     DeltaInv(msb12,mz2,msb22))*(-msb22 + DeltaInv(msb12,mz2,msb22)*(mz2*(-2*
     msb24 + msb22*mz2) - 2*msb12*(msb24 + msb22*mz2) + power6(msb2))))/msb12)
     + Fin3(msb12,mA2,msb22,Q2)*((2*cb*mu*sb*power2(mA2)*power2(DeltaInv(msb12,
     mA2,msb22))*(msb22 - DeltaInv(msb12,mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(
     msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/msb12 + (2*mu*
     sb*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22))*(-msb22 + DeltaInv(msb12,
     mA2,msb22)*(-2*msb12*msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(
     mA2) + power6(msb2))))/(cb*msb12) + (2*cb*mu*sb*power2(mA2)*power2(
     DeltaInv(msb12,mA2,msb22))*(-msb22 + DeltaInv(msb12,mA2,msb22)*(-2*msb12*
     msb24 - 2*mA2*(msb12*msb22 + msb24) + msb22*power2(mA2) + power6(msb2))))/
     msb12) + Fin3(mh2,msb12,msb22,Q2)*((8*ca*mu*sa*power2(mh2)*power2(snb)*
     power2(DeltaInv(mh2,msb12,msb22))*(msb22 - DeltaInv(mh2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(
     msb2))))/msb12 + (8*mu*sb*power2(mh2)*power2(sa)*power2(snb)*power2(
     DeltaInv(mh2,msb12,msb22))*(msb22 - DeltaInv(mh2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/
     (cb*msb12) + (2*ca*mu*sa*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22))*(-
     msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 +
     msb24) + msb22*power2(mh2) + power6(msb2))))/msb12 + (2*mu*sb*power2(mh2)*
     power2(sa)*power2(DeltaInv(mh2,msb12,msb22))*(-msb22 + DeltaInv(mh2,msb12,
     msb22)*(-2*msb12*msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) +
     power6(msb2))))/(cb*msb12) + (8*ca*mu*sa*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22))*power4(snb)*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mh2*(msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/
     msb12 + (8*mu*sb*power2(mh2)*power2(sa)*power2(DeltaInv(mh2,msb12,msb22))*
     power4(snb)*(-msb22 + DeltaInv(mh2,msb12,msb22)*(-2*msb12*msb24 - 2*mh2*(
     msb12*msb22 + msb24) + msb22*power2(mh2) + power6(msb2))))/(cb*msb12)) +
     Fin3(mH2,msb12,msb22,Q2)*((2*ca*mu*sa*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(
     msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/msb12 + (2*mu*
     sb*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,msb12,msb22))*(msb22 -
     DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) +
     msb22*power2(mH2) + power6(msb2))))/(cb*msb12) + (8*mu*sb*power2(mH2)*
     power2(snb)*power2(DeltaInv(mH2,msb12,msb22))*(msb22 - DeltaInv(mH2,msb12,
     msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) +
     power6(msb2))))/(cb*msb12) + (8*ca*mu*sa*power2(mH2)*power2(DeltaInv(mH2,
     msb12,msb22))*power4(snb)*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*msb12*
     msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/
     msb12 + (8*mu*sb*power2(mH2)*power2(sa)*power2(DeltaInv(mH2,msb12,msb22))*
     power4(snb)*(msb22 - DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(
     msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(cb*msb12) + (
     2*mu*sb*power2(mH2)*power2(DeltaInv(mH2,msb12,msb22))*(-msb22 + DeltaInv(
     mH2,msb12,msb22)*(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*
     power2(mH2) + power6(msb2))))/(cb*msb12) + (8*ca*mu*sa*power2(mH2)*power2(
     snb)*power2(DeltaInv(mH2,msb12,msb22))*(-msb22 + DeltaInv(mH2,msb12,msb22)
     *(-2*msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) +
     power6(msb2))))/msb12 + (8*mu*sb*power2(mH2)*power2(sa)*power2(snb)*
     power2(DeltaInv(mH2,msb12,msb22))*(-msb22 + DeltaInv(mH2,msb12,msb22)*(-2*
     msb12*msb24 - 2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(
     msb2))))/(cb*msb12) + (8*mu*sb*power2(mH2)*power2(DeltaInv(mH2,msb12,
     msb22))*power4(snb)*(-msb22 + DeltaInv(mH2,msb12,msb22)*(-2*msb12*msb24 -
     2*mH2*(msb12*msb22 + msb24) + msb22*power2(mH2) + power6(msb2))))/(cb*
     msb12)) + (2*cb*mu*sb*power2(DeltaInv(mw2,msb22,mst12))*power2(log(mw2/Q2)
     )*power3(mw2)*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (2*cb*mu*sb*
     power2(DeltaInv(mw2,msb22,mst12))*power2(log(mw2/Q2))*power3(mw2)*(-mst12
     + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 +
     mst12*mw2) + power6(mst1))))/msb22 + (cb*mu*mw2*sb*log(mw2/Q2)*(-1 + mw2*(
     -msb22 + 5*mst12 + mw2)*DeltaInv(mw2,msb22,mst12) + 6*mst12*mw2*(-mst14 +
     (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb22,
     mst12)) - 72*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*
     mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(6.*
     msb22*mst12) + (cb*mu*mw2*sb*log(mw2/Q2)*(1 + (msb22 - 5*mst12 - mw2)*mw2*
     DeltaInv(mw2,msb22,mst12) - 6*mst12*mw2*(-mst14 + (14*mst12 - mw2)*mw2 +
     msb22*(3*mst12 + mw2))*power2(DeltaInv(mw2,msb22,mst12)) + 72*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2)
     - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/(6.*msb22*mst12) + (2*cb*
     mu*sb*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2))*power3(mC2)*(
     mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (2*mu*sb*power2(
     DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2))*power3(mC2)*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(cb*msb22) + (2*cb*mu*sb*power2(
     DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2))*power3(mC2)*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (mC2*mu*sb*log(mC2/Q2)*(-1 +
     mC2*(mC2 - msb22 + 5*mst12)*DeltaInv(msb22,mst12,mC2) - 6*mC2*mst12*(-3*
     msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(6.*cb*msb22*mst12) + (cb*mC2*mu*sb*log(mC2/Q2)*(-1
     + mC2*(mC2 - msb22 + 5*mst12)*DeltaInv(msb22,mst12,mC2) - 6*mC2*mst12*(-3*
     msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(6.*msb22*mst12) + (cb*mC2*mu*sb*log(mC2/Q2)*(1 -
     mC2*(mC2 - msb22 + 5*mst12)*DeltaInv(msb22,mst12,mC2) + 6*mC2*mst12*(-3*
     msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb22,mst12,mC2)) + 72*mst12*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(6.*msb22*mst12) + (mC2*mu*sb*log(mC2/Q2)*power2(
     snb)*(msb12 - msb22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*DeltaInv(msb12,
     mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*DeltaInv(msb22,mst12,mC2)
     - 6*mC2*msb22*mst12*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*(-3*
     msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb22,mst12,mC2)) - 72*msb22*mst12*power2(mC2)*power3(DeltaInv(
     msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + 72*msb12*mst12*power2(mC2)*power3(DeltaInv(
     msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(6.*cb*msb12*msb22*mst12) + Fin3(mw2,msb12,
     mst12,Q2)*((4*cb*mu*sb*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb12,
     mst12))*(mst12 - DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12 + (4*cb*mu*sb*power2(
     mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb12,mst12))*(mst12 -
     DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1))))/msb12 + (4*cb*mu*sb*power2(mw2)*power2(snb)*
     power2(DeltaInv(mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))
     /msb12 + (4*cb*mu*sb*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(
     mw2,msb12,mst12))*(-mst12 + DeltaInv(mw2,msb12,mst12)*(mw2*(-2*mst14 +
     mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1))))/msb12) + (2*cb*
     mu*sb*power2(snb)*power2(log(mw2/Q2))*power3(mw2)*(msb22*mst12*power2(
     DeltaInv(mw2,msb12,mst12)) - msb22*power3(DeltaInv(mw2,msb12,mst12))*(mw2*
     (-2*mst14 + mst12*mw2) - 2*msb12*(mst14 + mst12*mw2) + power6(mst1)) -
     msb12*power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)
     *(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))
     )))/(msb12*msb22) + (2*cb*mu*sb*power2(snb)*power2(log(mw2/Q2))*power3(
     mw2)*(-(msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))) + msb22*power3(
     DeltaInv(mw2,msb12,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb12*(mst14 +
     mst12*mw2) + power6(mst1)) + msb12*power2(DeltaInv(mw2,msb22,mst12))*(
     mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1)))))/(msb12*msb22) + Fin3(mw2,msb22,
     mst12,Q2)*((4*cb*mu*sb*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12))*(
     mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1))))/msb22 + (4*cb*mu*sb*power2(mw2)*
     power2(snb)*power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,
     mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(
     mst1))))/msb22 + (4*cb*mu*sb*power2(mw2)*power2(snt)*power2(DeltaInv(mw2,
     msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*
     mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (4*cb*mu*sb*
     power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(
     mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(
     mst14 + mst12*mw2) + power6(mst1))))/msb22 + (4*cb*mu*sb*power2(mw2)*
     power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,msb22,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))
     /msb22 + (4*cb*mu*sb*power2(mw2)*power2(snb)*power2(DeltaInv(mw2,msb22,
     mst12))*(-mst12 + DeltaInv(mw2,msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) -
     2*msb22*(mst14 + mst12*mw2) + power6(mst1))))/msb22 + (4*cb*mu*sb*power2(
     mw2)*power2(snt)*power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,
     msb22,mst12)*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) +
     power6(mst1))))/msb22 + (4*cb*mu*sb*power2(mw2)*power2(snb)*power2(snt)*
     power2(DeltaInv(mw2,msb22,mst12))*(-mst12 + DeltaInv(mw2,msb22,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))))
     /msb22) + Fin3(msb12,mst12,mC2,Q2)*((4*cb*mt2*mu*power2(mC2)*power2(snb)*
     power2(DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(msb12*(mst12 - mst22)*sb) + (4*cb*mu*sb*power2(mC2)*power2(snb)*
     power2(DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb12 + (4*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     (cb*msb12) + (4*cb*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(msb12,mst12,mC2))*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     msb12 + (4*mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2)
     )*power3(cb)*(mst12 - DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(
     msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 -
     mst22)*sb) + (4*cb*mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,
     mst12,mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(
     msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 -
     mst22)*sb) + (4*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,
     mC2))*(-mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(cb*msb12) + (4*cb*
     mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb12 + (4*cb*mu*sb*power2(mC2)*
     power2(snb)*power2(snt)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb12 + (4*mt2*mu*power2(mC2)*power2(
     snb)*power2(DeltaInv(msb12,mst12,mC2))*power3(cb)*(-mst12 + DeltaInv(
     msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)*sb)) + (2*mu*sb*
     power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(-(msb22*mst12*power2(
     DeltaInv(msb12,mst12,mC2))) + msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)))))/(cb*msb12*msb22) + (2*cb*mu*sb*power2(snb)*power2(
     log(mC2/Q2))*power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)))
     + msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)))))
     /(msb12*msb22) + (2*cb*mu*sb*power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(
     msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb22*power3(DeltaInv(
     msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-
     mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)))))/(msb12*msb22) + Fin3(msb22,
     mst12,mC2,Q2)*((4*cb*mt2*mu*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*
     (mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)*sb) +
     (4*cb*mu*sb*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 -
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (4*cb*mt2*mu*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(msb22*(mst12 - mst22)*sb) + (4*mu*sb*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(cb*msb22) + (4*cb*mu*sb*power2(mC2)*power2(snb)*
     power2(DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/msb22 + (4*mu*sb*power2(mC2)*power2(snt)*power2(DeltaInv(msb22,
     mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(cb*msb22) + (
     4*cb*mu*sb*power2(mC2)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(
     mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (4*cb*mu*sb*power2(
     mC2)*power2(snb)*power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(mst12 -
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (4*mt2*mu*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*power3(cb)*(mst12 - DeltaInv(msb22,mst12,mC2)*(
     -2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(msb22*(mst12 - mst22)*sb) + (4*mt2*mu*power2(mC2)*power2(snb)*
     power2(DeltaInv(msb22,mst12,mC2))*power3(cb)*(mst12 - DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/(msb22*(mst12 - mst22)*sb) + (4*cb*mt2*mu*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)
     *(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/(msb22*(mst12 - mst22)*sb) + (4*mu*sb*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     (cb*msb22) + (4*cb*mu*sb*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(-
     mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/msb22 + (4*cb*mt2*mu*power2(
     mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(
     msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)*sb) + (4*cb*mu*sb*
     power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/msb22 + (4*cb*mu*sb*power2(mC2)*
     power2(snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,
     mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1))))/msb22 + (4*mu*sb*power2(mC2)*power2(snb)*power2(
     snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)
     *(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/(cb*msb22) + (4*cb*mu*sb*power2(mC2)*power2(snb)*power2(
     snt)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)
     *(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1))))/msb22 + (4*mt2*mu*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*power3(cb)*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(
     mst12 - mst22)*sb) + (4*mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(
     msb22,mst12,mC2))*power3(cb)*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1))))/(msb22*(mst12 - mst22)*sb)) + (2*cb*mt2*mu*power2(snb)*power2(
     log(mC2/Q2))*power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) -
     msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2)) + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2))
     - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*
     mst24*power3(DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2)) + msb22*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,
     mC2)) + 2*mC2*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) + 2*msb12*
     msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) - msb12*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) - msb22*power3(DeltaInv(msb12,mst12,mC2)
     )*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 -
     2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb22*
     power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - msb12*power3(DeltaInv(
     mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22*(mst12 - mst22)*sb) + (2*mu*
     sb*power2(snb)*power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*power2(DeltaInv(msb22,
     mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst22*
     power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) - msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     cb*msb12*msb22) + (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(mC2/Q2))*
     power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*
     power2(DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(
     DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) + msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*
     mst24*power3(DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*
     mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) - msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(msb12*msb22) + (2*mt2*mu*power2(snb)*power2(
     log(mC2/Q2))*power3(cb)*power3(mC2)*(msb22*mst12*power2(DeltaInv(msb12,
     mst12,mC2)) - msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) - msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst22*power2(DeltaInv(mst22,
     msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) -
     2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - 2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) + msb22*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) +
     2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) - msb12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - msb22*power3(DeltaInv(
     msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - msb12*
     power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22*(mst12 -
     mst22)*sb) + (2*cb*mt2*mu*power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(-(
     msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst12*power2(
     DeltaInv(msb22,mst12,mC2)) + msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))
     - msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))
     - msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) + msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) + msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22*(mst12 - mst22)*sb) + (2*cb*mu*sb*power2(snb)*power2(snt)*
     power2(log(mC2/Q2))*power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)) + msb22*
     mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst22*power2(DeltaInv(
     mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,
     mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + 2*msb12*
     msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) +
     msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + msb22*power3(
     DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - msb12*power3(DeltaInv(msb22,mst12,mC2)
     )*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) +
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*msb22) + (2*
     mt2*mu*power2(snb)*power2(log(mC2/Q2))*power3(cb)*power3(mC2)*(-(msb22*
     mst12*power2(DeltaInv(msb12,mst12,mC2))) + msb12*mst12*power2(DeltaInv(
     msb22,mst12,mC2)) + msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*
     mst22*power2(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) -
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) + msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) +
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) + msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22*(mst12 - mst22)*sb) - (mC2*mu*sb*log(mC2/Q2)*power2(snb)*
     power2(snt)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 +
     mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*
     msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*
     msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*
     DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,
     mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2) - msb22*mst12*
     DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*DeltaInv(mst22,msb22,
     mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*
     mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*
     mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*
     mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*
     mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*
     mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14
     - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(6.*cb*msb12*msb22*mst12*mst22) + (2*cb*mu*sb*power2(snt)*
     power2(log(mw2/Q2))*power3(mw2)*(mst22*power2(DeltaInv(mst22,mw2,msb22)) -
     power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(mw2*
     (-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1))) -
     power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/msb22 + (2*cb*mu*sb*power2(snt)*
     power2(log(mw2/Q2))*power3(mw2)*(-(mst22*power2(DeltaInv(mst22,mw2,msb22))
     ) + power2(DeltaInv(mw2,msb22,mst12))*(mst12 - DeltaInv(mw2,msb22,mst12)*(
     mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*mw2) + power6(mst1)))
     + power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*
     (mst24 + mst22*mw2) + power6(mst2))))/msb22 + (2*cb*mu*sb*power2(snb)*
     power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(-(msb22*mst22*power2(
     DeltaInv(mst22,mw2,msb12))) + msb12*mst22*power2(DeltaInv(mst22,mw2,msb22)
     ) + msb22*mst12*power2(DeltaInv(mw2,msb12,mst12)) - msb12*mst12*power2(
     DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst14*power3(DeltaInv(mw2,
     msb12,mst12)) + 2*msb12*msb22*mst12*mw2*power3(DeltaInv(mw2,msb12,mst12))
     + 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,mst12)) - msb22*mst12*
     power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst14*
     power3(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb22,mst12)) - 2*msb12*mst14*mw2*power3(DeltaInv(mw2,msb22,
     mst12)) + msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12)) -
     msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) + msb12*power3(
     DeltaInv(mw2,msb22,mst12))*power6(mst1) + msb22*power3(DeltaInv(mst22,mw2,
     msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*mw2) +
     power6(mst2)) - msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*msb22) +
     (2*cb*mu*sb*power2(snb)*power2(snt)*power2(log(mw2/Q2))*power3(mw2)*(
     msb22*mst22*power2(DeltaInv(mst22,mw2,msb12)) - msb12*mst22*power2(
     DeltaInv(mst22,mw2,msb22)) - msb22*mst12*power2(DeltaInv(mw2,msb12,mst12))
     + msb12*mst12*power2(DeltaInv(mw2,msb22,mst12)) - 2*msb12*msb22*mst14*
     power3(DeltaInv(mw2,msb12,mst12)) - 2*msb12*msb22*mst12*mw2*power3(
     DeltaInv(mw2,msb12,mst12)) - 2*msb22*mst14*mw2*power3(DeltaInv(mw2,msb12,
     mst12)) + msb22*mst12*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + 2*
     msb12*msb22*mst14*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*msb22*mst12*
     mw2*power3(DeltaInv(mw2,msb22,mst12)) + 2*msb12*mst14*mw2*power3(DeltaInv(
     mw2,msb22,mst12)) - msb12*mst12*power2(mw2)*power3(DeltaInv(mw2,msb22,
     mst12)) + msb22*power3(DeltaInv(mw2,msb12,mst12))*power6(mst1) - msb12*
     power3(DeltaInv(mw2,msb22,mst12))*power6(mst1) - msb22*power3(DeltaInv(
     mst22,mw2,msb12))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 + mst22*
     mw2) + power6(mst2)) + msb12*power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*
     mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/(msb12*
     msb22) + (cb*mu*mw2*sb*log(mw2/Q2)*power2(snt)*(-mst12 + mst22 + mst12*
     mw2*(-msb22 + 5*mst22 + mw2)*DeltaInv(mst22,mw2,msb22) + mst22*(msb22 - 5*
     mst12 - mw2)*mw2*DeltaInv(mw2,msb22,mst12) + 6*mst12*mst22*mw2*(-mst24 + (
     14*mst22 - mw2)*mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,
     msb22)) - 6*mst12*mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*
     mst12 + mw2))*power2(DeltaInv(mw2,msb22,mst12)) + 72*mst12*mst22*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*
     msb22*(mst14 + mst12*mw2) + power6(mst1)) - 72*mst12*mst22*power2(mw2)*
     power3(DeltaInv(mst22,mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(
     mst24 + mst22*mw2) + power6(mst2))))/(6.*msb22*mst12*mst22) + (cb*mu*mw2*
     sb*log(mw2/Q2)*power2(snt)*(mst12 - mst22 + mst12*(msb22 - 5*mst22 - mw2)*
     mw2*DeltaInv(mst22,mw2,msb22) + mst22*mw2*(-msb22 + 5*mst12 + mw2)*
     DeltaInv(mw2,msb22,mst12) - 6*mst12*mst22*mw2*(-mst24 + (14*mst22 - mw2)*
     mw2 + msb22*(3*mst22 + mw2))*power2(DeltaInv(mst22,mw2,msb22)) + 6*mst12*
     mst22*mw2*(-mst14 + (14*mst12 - mw2)*mw2 + msb22*(3*mst12 + mw2))*power2(
     DeltaInv(mw2,msb22,mst12)) - 72*mst12*mst22*power2(mw2)*power3(DeltaInv(
     mw2,msb22,mst12))*(mw2*(-2*mst14 + mst12*mw2) - 2*msb22*(mst14 + mst12*
     mw2) + power6(mst1)) + 72*mst12*mst22*power2(mw2)*power3(DeltaInv(mst22,
     mw2,msb22))*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) +
     power6(mst2))))/(6.*msb22*mst12*mst22) + (cb*mC2*mt2*mu*log(mC2/Q2)*(-
     mst12 + mst22 + mC2*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,
     mC2) + mC2*mst12*(mC2 - msb22 + 5*mst22)*DeltaInv(mst22,msb22,mC2) + 6*
     mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(
     mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*msb22*
     mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(
     mst22,msb22,mC2)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22*sb) + (mC2*
     mu*sb*log(mC2/Q2)*power2(snt)*(-mst12 + mst22 + mC2*(-mC2 + msb22 - 5*
     mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(mC2 - msb22 + 5*mst22)
     *DeltaInv(mst22,msb22,mC2) + 6*mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(
     msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2))
     - 6*mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 +
     power2(mC2))*power2(DeltaInv(mst22,msb22,mC2)) + 72*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - 72*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*cb*msb22*
     mst12*mst22) + (cb*mC2*mu*sb*log(mC2/Q2)*power2(snt)*(-mst12 + mst22 +
     mC2*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(
     mC2 - msb22 + 5*mst22)*DeltaInv(mst22,msb22,mC2) + 6*mC2*mst12*mst22*(-3*
     msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(
     DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(
     msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb22,mC2))
     + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) -
     72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     (6.*msb22*mst12*mst22) + (mC2*mt2*mu*log(mC2/Q2)*power3(cb)*(-mst12 +
     mst22 + mC2*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) +
     mC2*mst12*(mC2 - msb22 + 5*mst22)*DeltaInv(mst22,msb22,mC2) + 6*mC2*mst12*
     mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*
     power2(DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*msb22*mst22 -
     mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb22,
     mC2)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22*sb) + (cb*mC2*mt2*mu*log(
     mC2/Q2)*(mst12 - mst22 + mC2*(mC2 - msb22 + 5*mst12)*mst22*DeltaInv(msb22,
     mst12,mC2) + mC2*mst12*(-mC2 + msb22 - 5*mst22)*DeltaInv(mst22,msb22,mC2)
     - 6*mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*mst12*mst22*(-3*
     msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(
     DeltaInv(mst22,msb22,mC2)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(
     msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22*sb) +
     (cb*mC2*mu*sb*log(mC2/Q2)*power2(snt)*(mst12 - mst22 + mC2*(mC2 - msb22 +
     5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(-mC2 + msb22 - 5*
     mst22)*DeltaInv(mst22,msb22,mC2) - 6*mC2*mst12*mst22*(-3*msb22*mst12 -
     mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,
     mC2)) + 6*mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24
     + power2(mC2))*power2(DeltaInv(mst22,msb22,mC2)) - 72*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb22*
     mst12*mst22) + (mC2*mt2*mu*log(mC2/Q2)*power3(cb)*(mst12 - mst22 + mC2*(
     mC2 - msb22 + 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(-mC2 +
     msb22 - 5*mst22)*DeltaInv(mst22,msb22,mC2) - 6*mC2*mst12*mst22*(-3*msb22*
     mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb22,mst12,mC2)) + 6*mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*
     mst22) + mst24 + power2(mC2))*power2(DeltaInv(mst22,msb22,mC2)) - 72*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14
     - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24
     - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*
     msb22*mst12*(mst12 - mst22)*mst22*sb) + Fin3(mst22,mw2,msb12,Q2)*((4*cb*
     mu*sb*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12)
     )*(mst22 - DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*
     msb12*(mst24 + mst22*mw2) + power6(mst2))))/msb12 + (4*cb*mu*sb*power2(
     mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb12))*(-mst22 +
     DeltaInv(mst22,mw2,msb12)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb12*(mst24 +
     mst22*mw2) + power6(mst2))))/msb12) + Fin3(mst22,mw2,msb22,Q2)*((4*cb*mu*
     sb*power2(mw2)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 -
     DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 +
     mst22*mw2) + power6(mst2))))/msb22 + (4*cb*mu*sb*power2(mw2)*power2(snb)*
     power2(snt)*power2(DeltaInv(mst22,mw2,msb22))*(mst22 - DeltaInv(mst22,mw2,
     msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(
     mst2))))/msb22 + (4*cb*mu*sb*power2(mw2)*power2(snt)*power2(DeltaInv(
     mst22,mw2,msb22))*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 +
     mst22*mw2) - 2*msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22 + (4*cb*
     mu*sb*power2(mw2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,mw2,msb22)
     )*(-mst22 + DeltaInv(mst22,mw2,msb22)*(mw2*(-2*mst24 + mst22*mw2) - 2*
     msb22*(mst24 + mst22*mw2) + power6(mst2))))/msb22) + Fin3(mst22,msb12,mC2,
     Q2)*((4*cb*mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2)
     )*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22
     + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb)
     + (4*cb*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,
     msb12,mC2))*(mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(
     msb12*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (4*mt2*
     mu*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*power3(cb)*(
     mst22 - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb) +
     (4*cb*mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(-
     mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb) +
     (4*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,
     mC2))*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*
     mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(cb*msb12) + (4*cb*
     mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(DeltaInv(mst22,msb12,mC2)
     )*(-mst22 + DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22
     + mst24) + mst22*power2(mC2) + power6(mst2))))/msb12 + (4*mt2*mu*power2(
     mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*power3(cb)*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)*sb)) + (2*cb*
     mt2*mu*power2(log(mC2/Q2))*power3(mC2)*(-(mst12*power2(DeltaInv(msb22,
     mst12,mC2))) + power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(
     DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     /(msb22*(mst12 - mst22)*sb) + (2*cb*mu*sb*power2(snt)*power2(log(mC2/Q2))*
     power3(mC2)*(-(mst12*power2(DeltaInv(msb22,mst12,mC2))) + power3(DeltaInv(
     msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(mst22 -
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2)))))/msb22 + (2*mt2*mu*power2(log(mC2/Q2))
     *power3(cb)*power3(mC2)*(-(mst12*power2(DeltaInv(msb22,mst12,mC2))) +
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb22,
     mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*
     mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/(msb22*(mst12 -
     mst22)*sb) + (2*cb*mt2*mu*power2(log(mC2/Q2))*power3(mC2)*(mst12*power2(
     DeltaInv(msb22,mst12,mC2)) - power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2)))))/(msb22*(mst12 - mst22)*sb) + (2*mu*sb*power2(snt)*power2(log(
     mC2/Q2))*power3(mC2)*(mst12*power2(DeltaInv(msb22,mst12,mC2)) - power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(-
     mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2)))))/(cb*msb22) + (2*cb*mu*sb*
     power2(snt)*power2(log(mC2/Q2))*power3(mC2)*(mst12*power2(DeltaInv(msb22,
     mst12,mC2)) - power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(
     DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))
     /msb22 + (2*mt2*mu*power2(log(mC2/Q2))*power3(cb)*power3(mC2)*(mst12*
     power2(DeltaInv(msb22,mst12,mC2)) - power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,
     mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2)))))/(msb22*(mst12 - mst22)*sb) + Fin3(mst22,msb22,mC2,Q2)*((
     4*cb*mt2*mu*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 -
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22)*sb) + (4*cb*
     mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 -
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22)*sb) + (4*cb*mu*
     sb*power2(mC2)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 -
     DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/msb22 + (4*mu*sb*power2(mC2)*power2(
     snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(
     mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(cb*msb22) + (4*cb*mu*sb*power2(mC2)*power2(
     snb)*power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(
     mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/msb22 + (4*mt2*mu*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*power3(cb)*(mst22 - DeltaInv(mst22,msb22,mC2)*(
     -2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/(msb22*(mst12 - mst22)*sb) + (4*mt2*mu*power2(mC2)*power2(snb)*
     power2(DeltaInv(mst22,msb22,mC2))*power3(cb)*(mst22 - DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb22*(mst12 - mst22)*sb) + (4*cb*mt2*mu*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)
     *(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2))))/(msb22*(mst12 - mst22)*sb) + (4*cb*mt2*mu*power2(mC2)*
     power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb22*(mst12 - mst22)*sb) + (4*mu*sb*power2(mC2)*
     power2(snt)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(cb*msb22) + (4*cb*mu*sb*power2(mC2)*power2(snt)*
     power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*
     msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(
     mst2))))/msb22 + (4*cb*mu*sb*power2(mC2)*power2(snb)*power2(snt)*power2(
     DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     msb22 + (4*mt2*mu*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power3(cb)
     *(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22
     + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22)*sb)
     + (4*mt2*mu*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*
     power3(cb)*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 -
     mst22)*sb))) + xt*((-2*mt2*mu*sb*log(msb12/Q2)*log(mst12/Q2)*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 +
     3*mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12 - mst14)*power2(mC2) -
     mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*
     mst14 + power3(mst12)))))/(cb*msb12*(mst12 - mst22)) + (2*cb*mt2*mu*sb*
     power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*(mst14
     - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(msb22*(mst12 -
     mst22)) + (2*mt2*mu*sb*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*
     power2(log(mst12/Q2))*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))))/(cb*msb22*(mst12 - mst22)) + (2*cb*mt2*mu*sb*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*(-mst14 +
     DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(
     mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)) +
     (mt2*mu*sb*DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)*(5*mC2*mst12 - msb22*
     mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(6.*cb*
     msb22*(mst12 - mst22)) - (2*mt2*mu*sb*log(msb22/Q2)*log(mst12/Q2)*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14
     + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*
     power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 +
     power3(mst12)))))/(cb*msb22*(mst12 - mst22)) + (2*mt2*mu*sb*log(msb22/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*
     mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*
     power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))))/(cb*msb22*(mst12
     - mst22)) + (2*mt2*mu*sb*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(
     msb22,mst12,mC2)*((msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) +
     (msb22 - mst12)*power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))))/(
     cb*msb22*(mst12 - mst22)) + (2*mt2*mu*sb*log(msb12/Q2)*log(mst22/Q2)*
     power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*mst22 +
     msb12*mst22 + 3*mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 - mst24)*
     power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + 5*
     mC2*(2*msb12*mst24 + power3(mst22)))))/(cb*msb12*(mst12 - mst22)) + (2*
     mt2*mu*sb*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power2(log(mst22/
     Q2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(cb*msb22*(
     mst12 - mst22)) + (2*cb*mt2*mu*sb*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*power2(log(mst22/Q2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))))/(msb22*(mst12 - mst22)) + (2*cb*mt2*mu*sb*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*power2(log(mst22/Q2))*(-mst24 +
     DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(
     mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)) -
     (mt2*mu*sb*DeltaInv(mst22,msb22,mC2)*log(mst22/Q2)*(5*mC2*mst22 - msb22*
     mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*
     mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(6.*cb*
     msb22*(mst12 - mst22)) + (2*mt2*mu*sb*log(msb22/Q2)*log(mst22/Q2)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + 3*mst24
     + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*
     power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 +
     power3(mst22)))))/(cb*msb22*(mst12 - mst22)) - (2*mt2*mu*sb*log(msb22/Q2)*
     log(mst22/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(
     mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*
     mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*
     power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))))/(cb*msb22*(mst12
     - mst22)) - (2*mt2*mu*sb*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(
     mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*power3(mC2) +
     (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(mst22)))))/(
     cb*msb22*(mst12 - mst22)) + (2*mt2*mu*sb*log(mC2/Q2)*log(msb12/Q2)*power2(
     mC2)*power2(snb)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(
     msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*mC2*mst22 +
     msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*mst22 + mst24)*
     power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(mst22) - mC2*(-
     10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 + mst14)*power2(mC2)
     + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) + mC2*(-10*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(cb*msb12*(
     mst12 - mst22)) + (2*cb*mt2*mu*sb*power2(mC2)*power2(snb)*power2(log(
     mst12/Q2))*(msb22*mst14*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(
     DeltaInv(msb22,mst12,mC2))*(-mst14 + DeltaInv(msb22,mst12,mC2)*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))) - msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(
     mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*mt2*mu*sb*power2(mC2)*power2(
     snb)*power2(log(mst12/Q2))*(-(msb22*mst14*power2(DeltaInv(msb12,mst12,mC2)
     )) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mst14 - DeltaInv(msb22,
     mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))) + msb22*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(cb*msb12*msb22*(mst12 - mst22)) + (2*cb*mt2*
     mu*sb*power2(mC2)*power2(snb)*power2(log(mst12/Q2))*(-(msb22*mst14*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(
     mst14 - DeltaInv(msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*
     power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) + msb22*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 -
     mst22)) + (mt2*mu*sb*log(mst12/Q2)*power2(snb)*(msb22*(5*mC2*mst12 -
     msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2) - 6*mC2*msb22*power2(
     DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*
     mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,mst12,
     mC2)*(-5*mC2*mst12 + msb22*mst12 - mst14 + 6*mC2*DeltaInv(msb22,mst12,mC2)
     *(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) +
     power3(mst12)) + 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*
     power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 +
     power3(mst12)))) - 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 +
     mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2))))/(6.*cb*msb12*msb22*(mst12 - mst22)) + (cb*
     mt2*mu*sb*log(mst12/Q2)*power2(snb)*(msb22*(5*mC2*mst12 - msb12*mst12 +
     mst14)*DeltaInv(msb12,mst12,mC2) - 6*mC2*msb22*power2(DeltaInv(msb12,
     mst12,mC2))*(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*
     power2(mC2) + power3(mst12)) + msb12*DeltaInv(msb22,mst12,mC2)*(-5*mC2*
     mst12 + msb22*mst12 - mst14 + 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*
     msb22*mst12 - 14*mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(
     mst12)) + 72*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(
     mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(
     mst12)))) - 72*msb22*power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*
     power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)) + (cb*mt2*mu*sb*log(
     mst12/Q2)*power2(snb)*(-(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))
     *(-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) +
     power3(mst12)) + msb12*DeltaInv(msb22,mst12,mC2)*(5*mC2*mst12 - msb22*
     mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) + 72*msb22*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(6.*
     msb12*msb22*(mst12 - mst22)) + (2*mt2*mu*sb*power2(mC2)*power2(snb)*
     power2(log(msb12/Q2))*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(
     DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(2*mC2*
     mst22 + msb12*mst22 + 2*mst24 + DeltaInv(mst22,msb12,mC2)*((3*msb12*mst22
     + 2*mst24)*power2(mC2) - 2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(
     mst22) + 2*mC2*(5*msb12*mst24 + power3(mst22)))) + (-((3*msb12*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2))))/(cb*msb12*(mst12 - mst22)) + (2*cb*mt2*mu*sb*log(mC2/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*(
     (msb22*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*
     power3(mst12) + mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*
     mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) - mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*mt2*mu*sb*log(mC2/Q2)*log(
     mst12/Q2)*power2(mC2)*power2(snb)*(msb22*(mC2*mst12 + msb12*mst12 + mst14)
     *power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(msb22,mst12,
     mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(msb22,mst12,mC2)*(-((
     msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb22 + mst12)*
     power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*((msb12*
     mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*power3(
     mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,
     mC2))))/(cb*msb12*msb22*(mst12 - mst22)) + (2*cb*mt2*mu*sb*log(mC2/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(snb)*(msb22*(mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2)) + msb12*power2(DeltaInv(msb22,
     mst12,mC2))*(-(mC2*mst12) - msb22*mst12 - mst14 + DeltaInv(msb22,mst12,
     mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) + mst12*power3(mC2) + (-msb22 +
     mst12)*power3(mst12) - mC2*(6*msb22*mst14 + power3(mst12)))) + msb22*((
     msb12*mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb12 - mst12)*
     power3(mst12) + mC2*(6*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*mt2*mu*sb*log(mC2/
     Q2)*log(msb22/Q2)*power2(mC2)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*
     power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*
     mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12)
     + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     ))/(cb*msb22*(mst12 - mst22)) - (2*mt2*mu*sb*log(mC2/Q2)*log(msb22/Q2)*
     power2(mC2)*power2(snb)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(
     DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*
     mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 +
     mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) -
     mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*
     power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(
     -10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(cb*
     msb22*(mst12 - mst22)) + (2*mt2*mu*sb*power2(mC2)*power2(log(msb22/Q2))*(-
     ((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,mst12,mC2)))
     + power2(DeltaInv(mst22,msb22,mC2))*(2*mC2*mst22 + msb22*mst22 + 2*mst24 +
     DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(cb*msb22*(
     mst12 - mst22)) - (2*mt2*mu*sb*power2(mC2)*power2(snb)*power2(log(msb22/
     Q2))*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,mst12,
     mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(2*mC2*mst22 + msb22*mst22 + 2*
     mst24 + DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) -
     2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(cb*msb22*(
     mst12 - mst22)) + (2*mt2*mu*sb*power2(mC2)*power2(snb)*power2(log(mst22/
     Q2))*(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(
     DeltaInv(mst22,msb22,mC2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))) - msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(cb*msb12*msb22*(mst12 - mst22)) + (2*cb*mt2*mu*sb*power2(mC2)*
     power2(snb)*power2(log(mst22/Q2))*(msb22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*power2(DeltaInv(mst22,msb22,mC2))*(-mst24 + DeltaInv(
     mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) -
     2*mC2*(msb22*mst24 + power3(mst22)))) - msb22*(mst24*power2(mC2) + (-2*
     msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*cb*
     mt2*mu*sb*power2(mC2)*power2(snb)*power2(log(mst22/Q2))*(-(msb22*mst24*
     power2(DeltaInv(mst22,msb12,mC2))) + msb12*power2(DeltaInv(mst22,msb22,
     mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) + msb22*(
     mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12
     - mst22)) + (cb*mt2*mu*sb*log(mst22/Q2)*power2(snb)*(msb22*(5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2) - 6*mC2*msb22*power2(
     DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*
     mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*DeltaInv(mst22,msb22,
     mC2)*(-5*mC2*mst22 + msb22*mst22 - mst24 + 6*mC2*DeltaInv(mst22,msb22,mC2)
     *(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) +
     power3(mst22)) + 72*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))) - 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 +
     mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)) + (mt2*mu*
     sb*log(mst22/Q2)*power2(snb)*(-(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*
     DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))
     *(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) +
     power3(mst22)) + msb12*DeltaInv(mst22,msb22,mC2)*(5*mC2*mst22 - msb22*
     mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*
     mC2*mst24 - msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 +
     mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) + 72*msb22*
     power2(mC2)*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*
     (msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*cb*
     msb12*msb22*(mst12 - mst22)) + (cb*mt2*mu*sb*log(mst22/Q2)*power2(snb)*(-(
     msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + 6*
     mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*
     mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*DeltaInv(
     mst22,msb22,mC2)*(5*mC2*mst22 - msb22*mst22 + mst24 - 6*mC2*DeltaInv(
     mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*
     power2(mC2) + power3(mst22)) - 72*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(
     msb22*mst24 + power3(mst22)))) + 72*msb22*power2(mC2)*(mst24*power2(mC2) +
     (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)) + (
     cb*mt2*mu*sb*log(msb12/Q2)*power2(snb)*((mC2 + msb12)*(mst12 - mst22) +
     mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 -
     5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*
     DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*
     mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*
     mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*
     mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(
     mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*
     mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) +
     power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*
     mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)
     *power3(mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*
     mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(
     mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,
     msb12,mC2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22) + (mt2*mu*sb*log(
     msb12/Q2)*power2(snb)*(-((mC2 + msb12)*(mst12 - mst22)) - mst22*DeltaInv(
     msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,
     msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(
     msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (
     msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*
     msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*cb*msb12*
     mst12*(mst12 - mst22)*mst22) + (cb*mt2*mu*sb*log(msb12/Q2)*power2(snb)*(-(
     (mC2 + msb12)*(mst12 - mst22)) - mst22*DeltaInv(msb12,mst12,mC2)*(mst12*(-
     (msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*
     mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(
     msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (
     msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(
     mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22)
     + (2*mt2*mu*sb*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*(-(msb22*
     (mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(cb*msb12*msb22*(mst12 - mst22)) +
     (2*cb*mt2*mu*sb*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*(-(
     msb22*(mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)))
     + msb12*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24
     + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*
     cb*mt2*mu*sb*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*(msb22*(
     mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) +
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22))
     + (cb*mt2*mu*sb*log(msb22/Q2)*((mC2 + msb22)*(mst12 - mst22) + mst22*
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) +
     mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))
     /(6.*msb22*mst12*(mst12 - mst22)*mst22) + (mt2*mu*sb*log(msb22/Q2)*power2(
     snb)*((mC2 + msb22)*(mst12 - mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(
     mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*
     power2(mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(
     msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(
     mst22) + power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2)
     )*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*
     power2(mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*
     mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*cb*msb22*mst12*(
     mst12 - mst22)*mst22) + (cb*mt2*mu*sb*log(msb22/Q2)*power2(snb)*((mC2 +
     msb22)*(mst12 - mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*
     mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) +
     power3(mC2)) - mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) +
     mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)
     ) - 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) +
     mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(
     mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2)
     )*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*
     power2(mC2) + power3(mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-
     ((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22
     + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22) + (mt2*mu*sb*
     log(msb22/Q2)*(-((mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(msb22,
     mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2)
     + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb22,mC2)*(
     mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*
     power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb22,
     mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 -
     23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 +
     23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22))
     - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) +
     2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*
     power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 +
     power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*cb*msb22*mst12*(
     mst12 - mst22)*mst22) + (cb*mt2*mu*sb*log(msb22/Q2)*(-((mC2 + msb22)*(
     mst12 - mst22)) - mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) +
     mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)
     ) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (
     msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*
     mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*
     msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) +
     power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(
     msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(
     mC2) + power3(mC2) + power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*
     msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*
     mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22
     + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*
     power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(
     mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22) + (cb*mt2*mu*
     sb*log(msb22/Q2)*power2(snb)*(-((mC2 + msb22)*(mst12 - mst22)) - mst22*
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     - 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) +
     mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))
     /(6.*msb22*mst12*(mst12 - mst22)*mst22) + (mt2*mu*sb*power2(mC2)*power2(
     Pi)*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,
     mC2))) + (-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) +
     (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)
     ))*power3(DeltaInv(msb22,mst12,mC2)) + power2(DeltaInv(mst22,msb22,mC2))*(
     3*mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*(14*mC2*
     msb22*mst24 + 5*msb22*mst22*power2(mC2) + 3*mst24*power2(mC2) - 3*mst22*
     power3(mC2) + 3*mC2*power3(mst22) + 5*msb22*power3(mst22) - 3*power4(
     mst22)))))/(3.*cb*msb22*(mst12 - mst22)) + (cb*mt2*mu*sb*power2(mC2)*
     power2(Pi)*power2(snb)*(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 3*mC2*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) - 3*msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) + 3*mC2*
     msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(DeltaInv(mst22,msb22,
     mC2)) + msb22*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*mst12*power3(mC2)
     + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*mst14 + 3*power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*mC2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 3*mC2*msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,
     mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) +
     14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst22*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 3*mC2*msb12*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*
     msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*
     power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(
     mst12)) + 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 3*
     msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*(
     mst12 - mst22)) + (mt2*mu*sb*power2(mC2)*power2(Pi)*power2(snb)*(-(msb22*(
     3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,
     mC2)) + 3*mC2*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*msb22*
     mst22*power2(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(DeltaInv(
     mst22,msb12,mC2)) - 3*mC2*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) -
     msb12*msb22*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*
     power2(DeltaInv(mst22,msb22,mC2)) + msb22*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst22*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 3*mC2*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)) - 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,
     mC2)) - 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     - 3*msb12*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*
     mst22*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*mC2*msb12*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2)
     )*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2) + 3*mst14*power2(mC2) -
     3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*msb22*power3(mst12) - 3*
     power4(mst12)) - 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) +
     3*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*cb*msb12*
     msb22*(mst12 - mst22)) + (cb*mt2*mu*sb*power2(mC2)*power2(Pi)*power2(snb)*
     (-(msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     DeltaInv(msb22,mst12,mC2)) + 3*mC2*msb22*mst22*power2(DeltaInv(mst22,
     msb12,mC2)) + msb12*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + 3*
     msb22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 3*mC2*msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) - msb12*msb22*mst22*power2(DeltaInv(mst22,
     msb22,mC2)) - 3*msb12*mst24*power2(DeltaInv(mst22,msb22,mC2)) + msb22*(-((
     5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 +
     3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb12,mst12,mC2)) + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) + 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) + 3*msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))
     - 3*msb22*mst22*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*mC2*
     msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 14*mC2*msb12*msb22*
     mst24*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) - 3*mC2*msb12*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     msb12*power3(DeltaInv(msb22,mst12,mC2))*(14*mC2*msb22*mst14 + 5*msb22*
     mst12*power2(mC2) + 3*mst14*power2(mC2) - 3*mst12*power3(mC2) + 3*mC2*
     power3(mst12) + 5*msb22*power3(mst12) - 3*power4(mst12)) - 3*msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mst22) + 3*msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*(mst12 - mst22)) - (mC2*mt2*
     mu*sb*power2(snb)*(-(msb12*msb22*DeltaInv(msb12,mst12,mC2)) + msb12*msb22*
     DeltaInv(msb22,mst12,mC2) + msb12*msb22*DeltaInv(mst22,msb12,mC2) - msb12*
     msb22*DeltaInv(mst22,msb22,mC2) + 42*mC2*msb22*(3*mC2*mst12 + msb12*mst12
     + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2)) - 42*mC2*msb12*(3*mC2*mst12 +
     msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 42*mC2*msb12*
     msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - 126*mC2*msb22*mst24*
     power2(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb12,mC2)) + 42*mC2*msb12*msb22*mst22*power2(DeltaInv(
     mst22,msb22,mC2)) + 126*mC2*msb12*mst24*power2(DeltaInv(mst22,msb22,mC2))
     + 126*msb12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 42*mC2*
     msb22*(-((5*msb12*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-
     5*msb12 + 3*mst12)*power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))
     *power3(DeltaInv(msb12,mst12,mC2)) - 588*msb12*msb22*mst24*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 210*msb12*msb22*mst22*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 210*mC2*msb12*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) - 126*msb22*power2(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*mst22*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*power3(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 210*mC2*msb12*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) + 126*msb12*power2(mC2)*power3(mst22)*power3(DeltaInv(
     mst22,msb22,mC2)) + 126*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mC2) - 126*msb12*mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mC2) + 42*mC2*msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14
     - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) -
     3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*power4(mst12)) + 126*mC2*
     msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) - 126*mC2*msb12*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*cb*msb12*msb22*(
     mst12 - mst22)) - (mt2*mu*sb*(msb22*(-mst12 + mst22) - 2*mC2*msb22*mst12*
     mst22*DeltaInv(msb22,mst12,mC2) + 2*mC2*msb22*mst12*mst22*DeltaInv(mst22,
     msb22,mC2) + 84*mst12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mst22*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2)) - 84*mst12*mst22*(3*mC2*mst22 +
     msb22*mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 84*
     mst12*mst22*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*
     mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 84*mst12*
     mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-14*mC2*msb22*mst24 -
     5*msb22*mst22*power2(mC2) - 3*mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*
     mC2*power3(mst22) - 5*msb22*power3(mst22) + 3*power4(mst22))))/(6.*cb*
     msb22*mst12*(mst12 - mst22)*mst22) + Fin3(msb12,mst12,mC2,Q2)*((4*cb*mt2*
     mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mst12 -
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (4*mt2*mu*
     sb*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(cb*msb12*(mst12 - mst22)) + (4*cb*
     mt2*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(-
     mst12 + DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22))) +
     Fin3(msb22,mst12,mC2,Q2)*((4*cb*mt2*mu*sb*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(
     mst12 - mst22)) + (4*mt2*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(
     msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*
     mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(cb*msb22*
     (mst12 - mst22)) + (4*cb*mt2*mu*sb*power2(mC2)*power2(snb)*power2(
     DeltaInv(msb22,mst12,mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/
     (msb22*(mst12 - mst22)) + (4*mt2*mu*sb*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(cb*msb22*(
     mst12 - mst22)) + (4*cb*mt2*mu*sb*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22)) + (4*cb*mt2*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,
     mst12,mC2))*(-mst12 + DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22))) + (2*cb*mt2*mu*sb*power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(
     msb22*mst12*power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*power2(
     DeltaInv(msb22,mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,msb12,mC2))
     + msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2))
     + msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) - msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22*(mst12 - mst22)) + (2*mt2*mu*sb*power2(snb)*power2(log(mC2/Q2)
     )*power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))) + msb12*
     mst12*power2(DeltaInv(msb22,mst12,mC2)) + msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2)) - msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + 2*mC2*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) - msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*
     msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) - 2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb22,mC2)) + msb12*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) - msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) + msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(cb*msb12*msb22*(mst12 - mst22)) + (2*cb*mt2*
     mu*sb*power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(-(msb22*mst12*power2(
     DeltaInv(msb12,mst12,mC2))) + msb12*mst12*power2(DeltaInv(msb22,mst12,mC2)
     ) + msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*mst22*power2(
     DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*msb22*mst22*power3(DeltaInv(
     mst22,msb12,mC2)) + 2*mC2*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - msb22*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb12*msb22*mst22*
     power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*mst24*power3(DeltaInv(
     mst22,msb22,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2))
     + msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + msb22*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - msb12*power3(DeltaInv(msb22,
     mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(
     mC2) + power6(mst1)) - msb22*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) + msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(msb12*
     msb22*(mst12 - mst22)) + (mC2*mt2*mu*sb*log(mC2/Q2)*power2(snb)*(-(msb12*
     mst12) + msb22*mst12 + msb12*mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12
     + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*
     mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(
     mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) -
     mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*
     DeltaInv(mst22,msb22,mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(
     mC2) + msb12*mst12*DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*
     mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2)
     )*power2(DeltaInv(msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*
     mst12 - mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,
     msb12,mC2)) - 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) - 18*mC2*msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(
     mst22,msb12,mC2)) - 84*msb22*mst12*power2(mC2)*power2(mst22)*power2(
     DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*mst22*mst24*power2(
     DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*power2(mst22)*
     power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(mC2)*power2(
     mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*mst22*power2(
     DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*power2(
     DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*mst22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*mst12*
     mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb12*
     msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(mst22)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb22*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(mst2) - 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     6.*cb*msb12*msb22*mst12*(mst12 - mst22)*mst22) + (cb*mC2*mt2*mu*sb*log(
     mC2/Q2)*(-mst12 + mst22 + mC2*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(
     msb22,mst12,mC2) + mC2*mst12*(mC2 - msb22 + 5*mst22)*DeltaInv(mst22,msb22,
     mC2) + 6*mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14
     + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 6*mC2*mst12*mst22*(-3*
     msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(mC2))*power2(
     DeltaInv(mst22,msb22,mC2)) + 72*mst12*mst22*power2(mC2)*power3(DeltaInv(
     msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1)) - 72*mst12*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22) + (
     mC2*mt2*mu*sb*log(mC2/Q2)*(mst12 - mst22 + mC2*(mC2 - msb22 + 5*mst12)*
     mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(-mC2 + msb22 - 5*mst22)*
     DeltaInv(mst22,msb22,mC2) - 6*mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22
     + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*
     mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(
     mC2))*power2(DeltaInv(mst22,msb22,mC2)) - 72*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*cb*msb22*mst12*(mst12 -
     mst22)*mst22) + (cb*mC2*mt2*mu*sb*log(mC2/Q2)*(mst12 - mst22 + mC2*(mC2 -
     msb22 + 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(-mC2 + msb22
     - 5*mst22)*DeltaInv(mst22,msb22,mC2) - 6*mC2*mst12*mst22*(-3*msb22*mst12 -
     mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,
     mC2)) + 6*mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24
     + power2(mC2))*power2(DeltaInv(mst22,msb22,mC2)) - 72*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + Fin3(mst22,msb12,mC2,Q2)*((4*mt2*mu*sb*
     power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(mst22 -
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(cb*msb12*(mst12 - mst22)) + (4*cb*
     mt2*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(mst22
     - DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24)
     + mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)) + (4*cb*mt2*
     mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22))) + (2*mt2*mu*
     sb*power2(log(mC2/Q2))*power3(mC2)*(-(mst12*power2(DeltaInv(msb22,mst12,
     mC2))) + power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(
     mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*
     mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/(cb*
     msb22*(mst12 - mst22)) + (2*cb*mt2*mu*sb*power2(log(mC2/Q2))*power3(mC2)*(
     -(mst12*power2(DeltaInv(msb22,mst12,mC2))) + power3(DeltaInv(msb22,mst12,
     mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) +
     power6(mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2)))))/(msb22*(mst12 - mst22)) + (2*cb*mt2*mu*sb*power2(
     log(mC2/Q2))*power3(mC2)*(mst12*power2(DeltaInv(msb22,mst12,mC2)) -
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,msb22,
     mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*
     mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/(msb22*(mst12 -
     mst22)) + Fin3(mst22,msb22,mC2,Q2)*((4*mt2*mu*sb*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*
     mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/
     (cb*msb22*(mst12 - mst22)) + (4*cb*mt2*mu*sb*power2(mC2)*power2(DeltaInv(
     mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*
     mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(
     mst12 - mst22)) + (4*cb*mt2*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(
     mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*
     mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(
     mst12 - mst22)) + (4*cb*mt2*mu*sb*power2(mC2)*power2(DeltaInv(mst22,msb22,
     mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*
     mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(mst12 -
     mst22)) + (4*mt2*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,
     msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(cb*msb22*(
     mst12 - mst22)) + (4*cb*mt2*mu*sb*power2(mC2)*power2(snb)*power2(DeltaInv(
     mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*
     mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(msb22*(
     mst12 - mst22))) + xb*((-2*mt2*log(msb12/Q2)*log(mst12/Q2)*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mC2*mst12 + msb12*mst12 +
     3*mst14 + DeltaInv(msb12,mst12,mC2)*((msb12*mst12 - mst14)*power2(mC2) -
     mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb12*
     mst14 + power3(mst12)))))/(msb12*(mst12 - mst22)) + (2*mt2*log(msb12/Q2)*
     log(mst12/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(DeltaInv(msb12,
     mst12,mC2))*(mC2*mst12 + msb12*mst12 + 3*mst14 + DeltaInv(msb12,mst12,mC2)
     *((msb12*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb12 - 3*
     mst12)*power3(mst12) + 5*mC2*(2*msb12*mst14 + power3(mst12)))))/(msb12*(
     mst12 - mst22)) + (2*mt2*power2(cb)*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2))*power2(log(mst12/Q2))*(mst14 - DeltaInv(msb22,mst12,mC2)*(
     mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*mst14
     + power3(mst12)))))/(msb22*(mst12 - mst22)) + (2*mt2*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*power2(log(mst12/Q2))*(-mst14 + DeltaInv(msb22,
     mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(
     msb22*mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)) + (mt2*DeltaInv(
     msb22,mst12,mC2)*log(mst12/Q2)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*
     DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14
     + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))))/(6.*msb22*(mst12 - mst22)) - (mt2*
     DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)*power2(cb)*(5*mC2*mst12 - msb22*
     mst12 + mst14 - 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) - 72*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))))/(6.*msb22*(
     mst12 - mst22)) - (2*mt2*log(msb22/Q2)*log(mst12/Q2)*power2(mC2)*power2(
     DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 + 3*mst14 + DeltaInv(
     msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) - mst12*power3(mC2) +
     (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))
     ))/(msb22*(mst12 - mst22)) + (2*mt2*log(msb22/Q2)*log(mst12/Q2)*power2(cb)
     *power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 +
     3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) -
     mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*
     mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)) + (2*mt2*log(msb22/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(
     mC2*mst12 + msb22*mst12 + 3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*
     mst12 - mst14)*power2(mC2) - mst12*power3(mC2) + (5*msb22 - 3*mst12)*
     power3(mst12) + 5*mC2*(2*msb22*mst14 + power3(mst12)))))/(msb22*(mst12 -
     mst22)) - (2*mt2*log(msb22/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 + msb22*mst12 +
     3*mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 - mst14)*power2(mC2) -
     mst12*power3(mC2) + (5*msb22 - 3*mst12)*power3(mst12) + 5*mC2*(2*msb22*
     mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)) + (2*mt2*log(mC2/Q2)*
     log(mst12/Q2)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*
     msb22*mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)) - (2*mt2*log(mC2/
     Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))
     *(mC2*mst12 + msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*
     mst12 + mst14)*power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(
     mst12) + mC2*(6*msb22*mst14 + power3(mst12)))))/(msb22*(mst12 - mst22)) +
     (2*mt2*log(msb12/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*power2(
     DeltaInv(mst22,msb12,mC2))*(mC2*mst22 + msb12*mst22 + 3*mst24 + DeltaInv(
     mst22,msb12,mC2)*((msb12*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) +
     (5*msb12 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb12*mst24 + power3(mst22)))
     ))/(msb12*(mst12 - mst22)) - (2*mt2*log(msb12/Q2)*log(mst22/Q2)*power2(cb)
     *power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(mC2*mst22 +
     msb12*mst22 + 3*mst24 + DeltaInv(mst22,msb12,mC2)*((msb12*mst22 - mst24)*
     power2(mC2) - mst22*power3(mC2) + (5*msb12 - 3*mst22)*power3(mst22) + 5*
     mC2*(2*msb12*mst24 + power3(mst22)))))/(msb12*(mst12 - mst22)) + (2*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*power2(log(mst22/Q2))*(mst24
     - DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*msb22 + mst22)*
     power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(msb22*(mst12 -
     mst22)) + (2*mt2*power2(cb)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*
     power2(log(mst22/Q2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*power2(
     mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(
     mst22)))))/(msb22*(mst12 - mst22)) - (mt2*DeltaInv(mst22,msb22,mC2)*log(
     mst22/Q2)*(5*mC2*mst22 - msb22*mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,
     mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2)
     + power3(mst22)) - 72*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(
     mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))))/(6.*msb22*(mst12 - mst22)) + (mt2*DeltaInv(mst22,
     msb22,mC2)*log(mst22/Q2)*power2(cb)*(5*mC2*mst22 - msb22*mst22 + mst24 -
     6*mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 -
     msb22*mst24 + mst22*power2(mC2) + power3(mst22)) - 72*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(
     mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))))/(6.*msb22*(mst12 - mst22))
     + (2*mt2*log(msb22/Q2)*log(mst22/Q2)*power2(mC2)*power2(DeltaInv(mst22,
     msb22,mC2))*(mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)
     *((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*
     mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))))/(msb22*(
     mst12 - mst22)) - (2*mt2*log(msb22/Q2)*log(mst22/Q2)*power2(cb)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + 3*mst24
     + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*
     power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 +
     power3(mst22)))))/(msb22*(mst12 - mst22)) - (2*mt2*log(msb22/Q2)*log(
     mst22/Q2)*power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*
     mst22 + msb22*mst22 + 3*mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 -
     mst24)*power2(mC2) - mst22*power3(mC2) + (5*msb22 - 3*mst22)*power3(mst22)
     + 5*mC2*(2*msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)) + (2*
     mt2*log(msb22/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*power2(
     DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + 3*mst24 + DeltaInv(
     mst22,msb22,mC2)*((msb22*mst22 - mst24)*power2(mC2) - mst22*power3(mC2) +
     (5*msb22 - 3*mst22)*power3(mst22) + 5*mC2*(2*msb22*mst24 + power3(mst22)))
     ))/(msb22*(mst12 - mst22)) - (2*mt2*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))))/(msb22*(mst12 - mst22)) + (2*mt2*log(mC2/Q2)*log(mst22/Q2)*
     power2(cb)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 +
     msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*
     power2(mC2) - mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*
     msb22*mst24 + power3(mst22)))))/(msb22*(mst12 - mst22)) + (2*mt2*log(mC2/
     Q2)*log(msb12/Q2)*power2(mC2)*power2(snb)*(-((3*mC2*mst12 + msb12*mst12 +
     mst14)*power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,
     mC2))*(3*mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(
     msb12*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*
     power3(mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12
     + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(
     mst12) + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2))))/(msb12*(mst12 - mst22)) - (2*mt2*log(mC2/Q2)*log(msb12/Q2)*
     power2(cb)*power2(mC2)*power2(snb)*(-((3*mC2*mst12 + msb12*mst12 + mst14)*
     power2(DeltaInv(msb12,mst12,mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(3*
     mC2*mst22 + msb12*mst22 + mst24 + DeltaInv(mst22,msb12,mC2)*(5*(msb12*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb12 - mst22)*power3(
     mst22) - mC2*(-10*msb12*mst24 + power3(mst22)))) + (-5*(msb12*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12)
     + mC2*(-10*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     ))/(msb12*(mst12 - mst22)) + (2*mt2*power2(cb)*power2(mC2)*power2(snb)*
     power2(log(mst12/Q2))*(msb22*mst14*power2(DeltaInv(msb12,mst12,mC2)) +
     msb12*power2(DeltaInv(msb22,mst12,mC2))*(-mst14 + DeltaInv(msb22,mst12,
     mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) - 2*mC2*(msb22*
     mst14 + power3(mst12)))) - msb22*(mst14*power2(mC2) + (-2*msb12 + mst12)*
     power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*mt2*power2(mC2)*
     power2(snb)*power2(log(mst12/Q2))*(-(msb22*mst14*power2(DeltaInv(msb12,
     mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mst14 - DeltaInv(
     msb22,mst12,mC2)*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))) + msb22*(mst14*power2(mC2) + (-2*
     msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 + power3(mst12)))*
     power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (mt2*
     log(mst12/Q2)*power2(snb)*(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*
     DeltaInv(msb12,mst12,mC2) - 6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*
     (-3*mC2*msb12*mst12 - 14*mC2*mst14 - msb12*mst14 + mst12*power2(mC2) +
     power3(mst12)) + msb12*DeltaInv(msb22,mst12,mC2)*(-5*mC2*mst12 + msb22*
     mst12 - mst14 + 6*mC2*DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*
     mC2*mst14 - msb22*mst14 + mst12*power2(mC2) + power3(mst12)) + 72*power2(
     mC2)*power2(DeltaInv(msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 +
     mst12)*power3(mst12) - 2*mC2*(msb22*mst14 + power3(mst12)))) - 72*msb22*
     power2(mC2)*(mst14*power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*
     (msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(6.*
     msb12*msb22*(mst12 - mst22)) + (mt2*log(mst12/Q2)*power2(cb)*power2(snb)*(
     -(msb22*(5*mC2*mst12 - msb12*mst12 + mst14)*DeltaInv(msb12,mst12,mC2)) +
     6*mC2*msb22*power2(DeltaInv(msb12,mst12,mC2))*(-3*mC2*msb12*mst12 - 14*
     mC2*mst14 - msb12*mst14 + mst12*power2(mC2) + power3(mst12)) + msb12*
     DeltaInv(msb22,mst12,mC2)*(5*mC2*mst12 - msb22*mst12 + mst14 - 6*mC2*
     DeltaInv(msb22,mst12,mC2)*(-3*mC2*msb22*mst12 - 14*mC2*mst14 - msb22*mst14
     + mst12*power2(mC2) + power3(mst12)) - 72*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2))*(mst14*power2(mC2) + (-2*msb22 + mst12)*power3(mst12) -
     2*mC2*(msb22*mst14 + power3(mst12)))) + 72*msb22*power2(mC2)*(mst14*
     power2(mC2) + (-2*msb12 + mst12)*power3(mst12) - 2*mC2*(msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(6.*msb12*msb22*(mst12
     - mst22)) + (2*mt2*power2(mC2)*power2(snb)*power2(log(msb12/Q2))*(-((2*
     mC2*mst12 + msb12*mst12 + 2*mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     power2(DeltaInv(mst22,msb12,mC2))*(2*mC2*mst22 + msb12*mst22 + 2*mst24 +
     DeltaInv(mst22,msb12,mC2)*((3*msb12*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb12*
     mst24 + power3(mst22)))) + (-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12
     - mst22)) - (2*mt2*power2(cb)*power2(mC2)*power2(snb)*power2(log(msb12/Q2)
     )*(-((2*mC2*mst12 + msb12*mst12 + 2*mst14)*power2(DeltaInv(msb12,mst12,
     mC2))) + power2(DeltaInv(mst22,msb12,mC2))*(2*mC2*mst22 + msb12*mst22 + 2*
     mst24 + DeltaInv(mst22,msb12,mC2)*((3*msb12*mst22 + 2*mst24)*power2(mC2) -
     2*mst22*power3(mC2) + (3*msb12 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb12*
     mst24 + power3(mst22)))) + (-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*(mst12
     - mst22)) + (2*mt2*log(mC2/Q2)*log(mst12/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*(-(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(
     msb12,mst12,mC2))) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(mC2*mst12 +
     msb22*mst12 + mst14 + DeltaInv(msb22,mst12,mC2)*((msb22*mst12 + mst14)*
     power2(mC2) - mst12*power3(mC2) + (msb22 - mst12)*power3(mst12) + mC2*(6*
     msb22*mst14 + power3(mst12)))) + msb22*(-((msb12*mst12 + mst14)*power2(
     mC2)) + mst12*power3(mC2) + (-msb12 + mst12)*power3(mst12) - mC2*(6*msb12*
     mst14 + power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(
     mst12 - mst22)) + (2*mt2*log(mC2/Q2)*log(mst12/Q2)*power2(mC2)*power2(snb)
     *(msb22*(mC2*mst12 + msb12*mst12 + mst14)*power2(DeltaInv(msb12,mst12,mC2)
     ) + msb12*power2(DeltaInv(msb22,mst12,mC2))*(-(mC2*mst12) - msb22*mst12 -
     mst14 + DeltaInv(msb22,mst12,mC2)*(-((msb22*mst12 + mst14)*power2(mC2)) +
     mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) - mC2*(6*msb22*mst14 +
     power3(mst12)))) + msb22*((msb12*mst12 + mst14)*power2(mC2) - mst12*
     power3(mC2) + (msb12 - mst12)*power3(mst12) + mC2*(6*msb12*mst14 + power3(
     mst12)))*power3(DeltaInv(msb12,mst12,mC2))))/(msb12*msb22*(mst12 - mst22))
     + (2*mt2*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*(-((3*mC2*mst12 + msb22*
     mst12 + mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,
     msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)
     *(5*(msb22*mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 -
     mst22)*power3(mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(
     msb22*mst12 + mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*
     power3(mst12) + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(
     msb22,mst12,mC2))))/(msb22*(mst12 - mst22)) - (2*mt2*log(mC2/Q2)*log(
     msb22/Q2)*power2(cb)*power2(mC2)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*
     power2(DeltaInv(msb22,mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*
     mC2*mst22 + msb22*mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*
     mst22 + mst24)*power2(mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(
     mst22) - mC2*(-10*msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 +
     mst14)*power2(mC2) + 3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12)
     + mC2*(-10*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     ))/(msb22*(mst12 - mst22)) - (2*mt2*log(mC2/Q2)*log(msb22/Q2)*power2(mC2)*
     power2(snb)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*
     mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(
     mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*
     msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) +
     3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12
     - mst22)) + (2*mt2*log(mC2/Q2)*log(msb22/Q2)*power2(cb)*power2(mC2)*
     power2(snb)*(-((3*mC2*mst12 + msb22*mst12 + mst14)*power2(DeltaInv(msb22,
     mst12,mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*
     mst22 + mst24 + DeltaInv(mst22,msb22,mC2)*(5*(msb22*mst22 + mst24)*power2(
     mC2) - 3*mst22*power3(mC2) + (msb22 - mst22)*power3(mst22) - mC2*(-10*
     msb22*mst24 + power3(mst22)))) + (-5*(msb22*mst12 + mst14)*power2(mC2) +
     3*mst12*power3(mC2) + (-msb22 + mst12)*power3(mst12) + mC2*(-10*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12
     - mst22)) + (2*mt2*power2(mC2)*power2(log(msb22/Q2))*(-((2*mC2*mst12 +
     msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,mst12,mC2))) + power2(
     DeltaInv(mst22,msb22,mC2))*(2*mC2*mst22 + msb22*mst22 + 2*mst24 +
     DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12
     - mst22)) - (2*mt2*power2(cb)*power2(mC2)*power2(log(msb22/Q2))*(-((2*mC2*
     mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(DeltaInv(mst22,msb22,mC2))*(2*mC2*mst22 + msb22*mst22 + 2*mst24 +
     DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12
     - mst22)) - (2*mt2*power2(mC2)*power2(snb)*power2(log(msb22/Q2))*(-((2*
     mC2*mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,mst12,mC2))) +
     power2(DeltaInv(mst22,msb22,mC2))*(2*mC2*mst22 + msb22*mst22 + 2*mst24 +
     DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) - 2*
     mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12
     - mst22)) + (2*mt2*power2(cb)*power2(mC2)*power2(snb)*power2(log(msb22/Q2)
     )*(-((2*mC2*mst12 + msb22*mst12 + 2*mst14)*power2(DeltaInv(msb22,mst12,
     mC2))) + power2(DeltaInv(mst22,msb22,mC2))*(2*mC2*mst22 + msb22*mst22 + 2*
     mst24 + DeltaInv(mst22,msb22,mC2)*((3*msb22*mst22 + 2*mst24)*power2(mC2) -
     2*mst22*power3(mC2) + (3*msb22 - 2*mst22)*power3(mst22) + 2*mC2*(5*msb22*
     mst24 + power3(mst22)))) + (-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*
     mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*
     mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))))/(msb22*(mst12
     - mst22)) + (2*mt2*power2(mC2)*power2(snb)*power2(log(mst22/Q2))*(msb22*
     mst24*power2(DeltaInv(mst22,msb12,mC2)) + msb12*power2(DeltaInv(mst22,
     msb22,mC2))*(-mst24 + DeltaInv(mst22,msb22,mC2)*(mst24*power2(mC2) + (-2*
     msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) -
     msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(
     msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*
     msb22*(mst12 - mst22)) + (2*mt2*power2(cb)*power2(mC2)*power2(snb)*power2(
     log(mst22/Q2))*(-(msb22*mst24*power2(DeltaInv(mst22,msb12,mC2))) + msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(mst24 - DeltaInv(mst22,msb22,mC2)*(
     mst24*power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24
     + power3(mst22)))) + msb22*(mst24*power2(mC2) + (-2*msb12 + mst22)*power3(
     mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,
     mC2))))/(msb12*msb22*(mst12 - mst22)) + (mt2*log(mst22/Q2)*power2(cb)*
     power2(snb)*(msb22*(5*mC2*mst22 - msb12*mst22 + mst24)*DeltaInv(mst22,
     msb12,mC2) - 6*mC2*msb22*power2(DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*
     mst22 - 14*mC2*mst24 - msb12*mst24 + mst22*power2(mC2) + power3(mst22)) +
     msb12*DeltaInv(mst22,msb22,mC2)*(-5*mC2*mst22 + msb22*mst22 - mst24 + 6*
     mC2*DeltaInv(mst22,msb22,mC2)*(-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*
     mst24 + mst22*power2(mC2) + power3(mst22)) + 72*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2))*(mst24*power2(mC2) + (-2*msb22 + mst22)*power3(
     mst22) - 2*mC2*(msb22*mst24 + power3(mst22)))) - 72*msb22*power2(mC2)*(
     mst24*power2(mC2) + (-2*msb12 + mst22)*power3(mst22) - 2*mC2*(msb12*mst24
     + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*msb22*(
     mst12 - mst22)) + (mt2*log(mst22/Q2)*power2(snb)*(-(msb22*(5*mC2*mst22 -
     msb12*mst22 + mst24)*DeltaInv(mst22,msb12,mC2)) + 6*mC2*msb22*power2(
     DeltaInv(mst22,msb12,mC2))*(-3*mC2*msb12*mst22 - 14*mC2*mst24 - msb12*
     mst24 + mst22*power2(mC2) + power3(mst22)) + msb12*DeltaInv(mst22,msb22,
     mC2)*(5*mC2*mst22 - msb22*mst22 + mst24 - 6*mC2*DeltaInv(mst22,msb22,mC2)*
     (-3*mC2*msb22*mst22 - 14*mC2*mst24 - msb22*mst24 + mst22*power2(mC2) +
     power3(mst22)) - 72*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst24*
     power2(mC2) + (-2*msb22 + mst22)*power3(mst22) - 2*mC2*(msb22*mst24 +
     power3(mst22)))) + 72*msb22*power2(mC2)*(mst24*power2(mC2) + (-2*msb12 +
     mst22)*power3(mst22) - 2*mC2*(msb12*mst24 + power3(mst22)))*power3(
     DeltaInv(mst22,msb12,mC2))))/(6.*msb12*msb22*(mst12 - mst22)) + (mt2*log(
     msb12/Q2)*power2(cb)*power2(snb)*((mC2 + msb12)*(mst12 - mst22) + mst22*
     DeltaInv(msb12,mst12,mC2)*(mst12*(-(msb12*mst12) + mst14) - (msb12 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb12,mC2)*(mst22*(-(msb12*mst22) + mst24) - (msb12 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb12,mst12,mC2))*(-(msb12*mst14) + mC2*(6*msb12*mst12 +
     23*mst14) - (msb12 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*(-(msb12*mst24) +
     mC2*(6*msb12*mst22 + 23*mst24) - (msb12 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb12 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb12*mst14 + power3(mst12)))*power3(DeltaInv(msb12,
     mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb12*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb12 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb12*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb12,mC2))))
     /(6.*msb12*mst12*(mst12 - mst22)*mst22) + (mt2*log(msb12/Q2)*power2(snb)*(
     -((mC2 + msb12)*(mst12 - mst22)) - mst22*DeltaInv(msb12,mst12,mC2)*(mst12*
     (-(msb12*mst12) + mst14) - (msb12 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb12,mC2)*(mst22*(-(msb12*
     mst22) + mst24) - (msb12 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb12,mst12,mC2))*(-(
     msb12*mst14) + mC2*(6*msb12*mst12 + 23*mst14) - (msb12 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     mst22,msb12,mC2))*(-(msb12*mst24) + mC2*(6*msb12*mst22 + 23*mst24) - (
     msb12 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb12*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb12 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb12*mst14 +
     power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) + 72*mst12*mst22*power2(
     mC2)*(-((3*msb12*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb12 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb12*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb12,mC2))))/(6.*msb12*mst12*(mst12 - mst22)*mst22)
     + (2*mt2*log(mC2/Q2)*log(mst22/Q2)*power2(mC2)*power2(snb)*(-(msb22*(mC2*
     mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2))) + msb12*
     power2(DeltaInv(mst22,msb22,mC2))*(mC2*mst22 + msb22*mst22 + mst24 +
     DeltaInv(mst22,msb22,mC2)*((msb22*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb22 - mst22)*power3(mst22) + mC2*(6*msb22*mst24 + power3(
     mst22)))) + msb22*(-((msb12*mst22 + mst24)*power2(mC2)) + mst22*power3(
     mC2) + (-msb12 + mst22)*power3(mst22) - mC2*(6*msb12*mst24 + power3(mst22)
     ))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22)) + (2*
     mt2*log(mC2/Q2)*log(mst22/Q2)*power2(cb)*power2(mC2)*power2(snb)*(msb22*(
     mC2*mst22 + msb12*mst22 + mst24)*power2(DeltaInv(mst22,msb12,mC2)) +
     msb12*power2(DeltaInv(mst22,msb22,mC2))*(-(mC2*mst22) - msb22*mst22 -
     mst24 + DeltaInv(mst22,msb22,mC2)*(-((msb22*mst22 + mst24)*power2(mC2)) +
     mst22*power3(mC2) + (-msb22 + mst22)*power3(mst22) - mC2*(6*msb22*mst24 +
     power3(mst22)))) + msb22*((msb12*mst22 + mst24)*power2(mC2) - mst22*
     power3(mC2) + (msb12 - mst22)*power3(mst22) + mC2*(6*msb12*mst24 + power3(
     mst22)))*power3(DeltaInv(mst22,msb12,mC2))))/(msb12*msb22*(mst12 - mst22))
     + (mt2*log(msb22/Q2)*power2(cb)*((mC2 + msb22)*(mst12 - mst22) + mst22*
     DeltaInv(msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*
     mst12)*power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) - mst12*DeltaInv(
     mst22,msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*
     power2(mC2) + 5*mC2*power2(mst22) + power3(mC2)) - 6*mC2*mst12*mst22*
     power2(DeltaInv(msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 +
     23*mst14) - (msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12))
     + 6*mC2*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) +
     mC2*(6*msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(
     mC2) + power3(mst22)) + 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*
     mst14)*power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(
     mst12) - 2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,
     mst12,mC2)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*
     power2(mC2)) + 2*mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) -
     2*mC2*(5*msb22*mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))
     /(6.*msb22*mst12*(mst12 - mst22)*mst22) + (mt2*log(msb22/Q2)*power2(snb)*(
     (mC2 + msb22)*(mst12 - mst22) + mst22*DeltaInv(msb22,mst12,mC2)*(mst12*(-(
     msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) - mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*
     mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) - 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(
     msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) + 6*mC2*mst12*mst22*power2(DeltaInv(
     mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (
     msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) + 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) - 72*mst12*mst22*power2(
     mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22)
     + (mt2*log(msb22/Q2)*(-((mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(
     msb22,mst12,mC2)*(mst12*(-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*
     power2(mC2) + 5*mC2*power2(mst12) + power3(mC2)) + mst12*DeltaInv(mst22,
     msb22,mC2)*(mst22*(-(msb22*mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2)
     + 5*mC2*power2(mst22) + power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(
     msb22,mst12,mC2))*(-(msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (
     msb22 - 23*mst12)*power2(mC2) + power3(mC2) + power3(mst12)) - 6*mC2*
     mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*
     msb22*mst22 + 23*mst24) - (msb22 - 23*mst22)*power2(mC2) + power3(mC2) +
     power3(mst22)) - 72*mst12*mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*
     power2(mC2)) + 2*mst12*power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) -
     2*mC2*(5*msb22*mst14 + power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     72*mst12*mst22*power2(mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*
     mst22*power3(mC2) + (-3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*
     mst24 + power3(mst22)))*power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + (mt2*log(msb22/Q2)*power2(cb)*power2(snb)*(
     -((mC2 + msb22)*(mst12 - mst22)) - mst22*DeltaInv(msb22,mst12,mC2)*(mst12*
     (-(msb22*mst12) + mst14) - (msb22 - 5*mst12)*power2(mC2) + 5*mC2*power2(
     mst12) + power3(mC2)) + mst12*DeltaInv(mst22,msb22,mC2)*(mst22*(-(msb22*
     mst22) + mst24) - (msb22 - 5*mst22)*power2(mC2) + 5*mC2*power2(mst22) +
     power3(mC2)) + 6*mC2*mst12*mst22*power2(DeltaInv(msb22,mst12,mC2))*(-(
     msb22*mst14) + mC2*(6*msb22*mst12 + 23*mst14) - (msb22 - 23*mst12)*power2(
     mC2) + power3(mC2) + power3(mst12)) - 6*mC2*mst12*mst22*power2(DeltaInv(
     mst22,msb22,mC2))*(-(msb22*mst24) + mC2*(6*msb22*mst22 + 23*mst24) - (
     msb22 - 23*mst22)*power2(mC2) + power3(mC2) + power3(mst22)) - 72*mst12*
     mst22*power2(mC2)*(-((3*msb22*mst12 + 2*mst14)*power2(mC2)) + 2*mst12*
     power3(mC2) + (-3*msb22 + 2*mst12)*power3(mst12) - 2*mC2*(5*msb22*mst14 +
     power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 72*mst12*mst22*power2(
     mC2)*(-((3*msb22*mst22 + 2*mst24)*power2(mC2)) + 2*mst22*power3(mC2) + (-
     3*msb22 + 2*mst22)*power3(mst22) - 2*mC2*(5*msb22*mst24 + power3(mst22)))*
     power3(DeltaInv(mst22,msb22,mC2))))/(6.*msb22*mst12*(mst12 - mst22)*mst22)
     + (mt2*power2(mC2)*power2(Pi)*(-((3*mC2*mst12 + msb22*mst12 + 3*mst14)*
     power2(DeltaInv(msb22,mst12,mC2))) + (-((5*msb22*mst12 + 3*mst14)*power2(
     mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(
     14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) +
     power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + 3*mst24 +
     DeltaInv(mst22,msb22,mC2)*(14*mC2*msb22*mst24 + 5*msb22*mst22*power2(mC2)
     + 3*mst24*power2(mC2) - 3*mst22*power3(mC2) + 3*mC2*power3(mst22) + 5*
     msb22*power3(mst22) - 3*power4(mst22)))))/(3.*msb22*(mst12 - mst22)) - (
     mt2*power2(cb)*power2(mC2)*power2(Pi)*(-((3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(DeltaInv(msb22,mst12,mC2))) + (-((5*msb22*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) -
     mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2))
     + power2(DeltaInv(mst22,msb22,mC2))*(3*mC2*mst22 + msb22*mst22 + 3*mst24 +
     DeltaInv(mst22,msb22,mC2)*(14*mC2*msb22*mst24 + 5*msb22*mst22*power2(mC2)
     + 3*mst24*power2(mC2) - 3*mst22*power3(mC2) + 3*mC2*power3(mst22) + 5*
     msb22*power3(mst22) - 3*power4(mst22)))))/(3.*msb22*(mst12 - mst22)) + (
     mt2*power2(cb)*power2(mC2)*power2(Pi)*power2(snb)*(msb22*(3*mC2*mst12 +
     msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2)) - msb12*(3*mC2*
     mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 3*mC2*
     msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) - msb12*msb22*mst22*power2(
     DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(DeltaInv(mst22,msb12,
     mC2)) + 3*mC2*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + msb12*msb22*
     mst22*power2(DeltaInv(mst22,msb22,mC2)) + 3*msb12*mst24*power2(DeltaInv(
     mst22,msb22,mC2)) + msb22*((5*msb12*mst12 + 3*mst14)*power2(mC2) - 3*
     mst12*power3(mC2) + (5*msb12 - 3*mst12)*power3(mst12) + mC2*(14*msb12*
     mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2)) - 14*mC2*
     msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) - 5*msb12*msb22*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst24*power2(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst22*power3(mC2)*power3(
     DeltaInv(mst22,msb12,mC2)) - 3*mC2*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)) - 5*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,mC2)) +
     5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*
     msb12*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst22*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*mC2*msb12*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + 5*msb12*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2))*(-14*
     mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(mC2) + 3*
     mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) + 3*
     power4(mst12)) + 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) -
     3*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*
     (mst12 - mst22)) + (mt2*power2(mC2)*power2(Pi)*power2(snb)*(-(msb22*(3*
     mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,mst12,mC2))) +
     msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(DeltaInv(msb22,mst12,
     mC2)) + 3*mC2*msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*msb22*
     mst22*power2(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*power2(DeltaInv(
     mst22,msb12,mC2)) - 3*mC2*msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) -
     msb12*msb22*mst22*power2(DeltaInv(mst22,msb22,mC2)) - 3*msb12*mst24*
     power2(DeltaInv(mst22,msb22,mC2)) + msb22*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     + 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) + 5*msb12*
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 3*msb22*mst24*
     power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 3*msb22*mst22*power3(mC2)*
     power3(DeltaInv(mst22,msb12,mC2)) + 3*mC2*msb22*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 5*msb12*msb22*power3(mst22)*power3(DeltaInv(
     mst22,msb12,mC2)) - 14*mC2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb22,
     mC2)) - 5*msb12*msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     - 3*msb12*mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 3*msb12*
     mst22*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 3*mC2*msb12*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power3(mst22)*
     power3(DeltaInv(mst22,msb22,mC2)) + msb12*power3(DeltaInv(msb22,mst12,mC2)
     )*(14*mC2*msb22*mst14 + 5*msb22*mst12*power2(mC2) + 3*mst14*power2(mC2) -
     3*mst12*power3(mC2) + 3*mC2*power3(mst12) + 5*msb22*power3(mst12) - 3*
     power4(mst12)) - 3*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(mst22) +
     3*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*
     (mst12 - mst22)) - (mC2*mt2*power2(snb)*(-(msb12*msb22*DeltaInv(msb12,
     mst12,mC2)) + msb12*msb22*DeltaInv(msb22,mst12,mC2) + msb12*msb22*
     DeltaInv(mst22,msb12,mC2) - msb12*msb22*DeltaInv(mst22,msb22,mC2) + 42*
     mC2*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(DeltaInv(msb12,
     mst12,mC2)) - 42*mC2*msb12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*power2(
     DeltaInv(msb22,mst12,mC2)) - 42*mC2*msb12*msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2)) - 126*mC2*msb22*mst24*power2(DeltaInv(mst22,msb12,mC2))
     - 126*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + 42*mC2*
     msb12*msb22*mst22*power2(DeltaInv(mst22,msb22,mC2)) + 126*mC2*msb12*mst24*
     power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst22*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)) - 42*mC2*msb22*(-((5*msb12*mst12 + 3*mst14)*
     power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)*power3(mst12) -
     mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(msb12,mst12,mC2))
     - 588*msb12*msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) -
     210*msb12*msb22*mst22*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 126*
     msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 210*mC2*msb12*
     msb22*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) - 126*msb22*power2(
     mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + 588*msb12*msb22*
     mst24*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*msb12*msb22*
     mst22*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst24*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 210*mC2*msb12*msb22*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb12*power2(mC2)*
     power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) + 126*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*msb12*mst22*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2) + 42*mC2*msb12*power3(DeltaInv(msb22,mst12,
     mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(mC2) - 3*mst14*power2(
     mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) - 5*msb22*power3(mst12) +
     3*power4(mst12)) + 126*mC2*msb22*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mst22) - 126*mC2*msb12*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22)))/(
     3.*msb12*msb22*(mst12 - mst22)) + (mC2*mt2*power2(cb)*power2(snb)*(-(
     msb12*msb22*DeltaInv(msb12,mst12,mC2)) + msb12*msb22*DeltaInv(msb22,mst12,
     mC2) + msb12*msb22*DeltaInv(mst22,msb12,mC2) - msb12*msb22*DeltaInv(mst22,
     msb22,mC2) + 42*mC2*msb22*(3*mC2*mst12 + msb12*mst12 + 3*mst14)*power2(
     DeltaInv(msb12,mst12,mC2)) - 42*mC2*msb12*(3*mC2*mst12 + msb22*mst12 + 3*
     mst14)*power2(DeltaInv(msb22,mst12,mC2)) - 42*mC2*msb12*msb22*mst22*
     power2(DeltaInv(mst22,msb12,mC2)) - 126*mC2*msb22*mst24*power2(DeltaInv(
     mst22,msb12,mC2)) - 126*msb22*mst22*power2(mC2)*power2(DeltaInv(mst22,
     msb12,mC2)) + 42*mC2*msb12*msb22*mst22*power2(DeltaInv(mst22,msb22,mC2)) +
     126*mC2*msb12*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 126*msb12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 42*mC2*msb22*(-((5*msb12*
     mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb12 + 3*mst12)
     *power3(mst12) - mC2*(14*msb12*mst14 + 3*power3(mst12)))*power3(DeltaInv(
     msb12,mst12,mC2)) - 588*msb12*msb22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 210*msb12*msb22*mst22*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) - 126*msb22*mst24*power3(mC2)*power3(DeltaInv(mst22,
     msb12,mC2)) - 210*mC2*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) - 126*msb22*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb12,mC2)) + 588*msb12*msb22*mst24*power2(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 210*msb12*msb22*mst22*power3(mC2)*power3(DeltaInv(mst22,
     msb22,mC2)) + 126*msb12*mst24*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)
     ) + 210*mC2*msb12*msb22*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     126*msb12*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     126*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 126*msb12*
     mst22*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 42*mC2*msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-14*mC2*msb22*mst14 - 5*msb22*mst12*power2(
     mC2) - 3*mst14*power2(mC2) + 3*mst12*power3(mC2) - 3*mC2*power3(mst12) -
     5*msb22*power3(mst12) + 3*power4(mst12)) + 126*mC2*msb22*power3(DeltaInv(
     mst22,msb12,mC2))*power4(mst22) - 126*mC2*msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mst22)))/(3.*msb12*msb22*(mst12 - mst22)) - (mt2*(
     msb22*(-mst12 + mst22) - 2*mC2*msb22*mst12*mst22*DeltaInv(msb22,mst12,mC2)
     + 2*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb22,mC2) + 84*mst12*(3*mC2*
     mst12 + msb22*mst12 + 3*mst14)*mst22*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)) - 84*mst12*mst22*(3*mC2*mst22 + msb22*mst22 + 3*mst24)*power2(
     mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 84*mst12*mst22*power2(mC2)*(-((5*
     msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*power3(mC2) + (-5*msb22 + 3*
     mst12)*power3(mst12) - mC2*(14*msb22*mst14 + 3*power3(mst12)))*power3(
     DeltaInv(msb22,mst12,mC2)) + 84*mst12*mst22*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2))*(-14*mC2*msb22*mst24 - 5*msb22*mst22*power2(mC2) - 3*
     mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*mC2*power3(mst22) - 5*msb22*
     power3(mst22) + 3*power4(mst22))))/(6.*msb22*mst12*(mst12 - mst22)*mst22)
     + (mt2*power2(cb)*(msb22*(-mst12 + mst22) - 2*mC2*msb22*mst12*mst22*
     DeltaInv(msb22,mst12,mC2) + 2*mC2*msb22*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) + 84*mst12*(3*mC2*mst12 + msb22*mst12 + 3*mst14)*mst22*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)) - 84*mst12*mst22*(3*mC2*mst22 + msb22*
     mst22 + 3*mst24)*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - 84*mst12*
     mst22*power2(mC2)*(-((5*msb22*mst12 + 3*mst14)*power2(mC2)) + 3*mst12*
     power3(mC2) + (-5*msb22 + 3*mst12)*power3(mst12) - mC2*(14*msb22*mst14 +
     3*power3(mst12)))*power3(DeltaInv(msb22,mst12,mC2)) + 84*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-14*mC2*msb22*mst24 - 5*
     msb22*mst22*power2(mC2) - 3*mst24*power2(mC2) + 3*mst22*power3(mC2) - 3*
     mC2*power3(mst22) - 5*msb22*power3(mst22) + 3*power4(mst22))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + Fin3(msb12,mst12,mC2,Q2)*((4*mt2*power2(cb)
     *power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(mst12 -
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22)) + (4*mt2*
     power2(mC2)*power2(snb)*power2(DeltaInv(msb12,mst12,mC2))*(-mst12 +
     DeltaInv(msb12,mst12,mC2)*(-2*msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb12*(mst12 - mst22))) + Fin3(msb22,
     mst12,mC2,Q2)*((4*mt2*power2(cb)*power2(mC2)*power2(DeltaInv(msb22,mst12,
     mC2))*(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 -
     mst22)) + (4*mt2*power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))
     *(mst12 - DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (4*
     mt2*power2(mC2)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 + DeltaInv(
     msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*
     power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22)) + (4*mt2*power2(cb)*
     power2(mC2)*power2(snb)*power2(DeltaInv(msb22,mst12,mC2))*(-mst12 +
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1))))/(msb22*(mst12 - mst22))) + (2*mt2*
     power2(cb)*power2(snb)*power2(log(mC2/Q2))*power3(mC2)*(msb22*mst12*
     power2(DeltaInv(msb12,mst12,mC2)) - msb12*mst12*power2(DeltaInv(msb22,
     mst12,mC2)) - msb22*mst22*power2(DeltaInv(mst22,msb12,mC2)) + msb12*mst22*
     power2(DeltaInv(mst22,msb22,mC2)) - 2*mC2*msb12*msb22*mst22*power3(
     DeltaInv(mst22,msb12,mC2)) - 2*mC2*msb22*mst24*power3(DeltaInv(mst22,
     msb12,mC2)) - 2*msb12*msb22*mst24*power3(DeltaInv(mst22,msb12,mC2)) +
     msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb12*
     msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) + 2*mC2*msb12*mst24*power3(
     DeltaInv(mst22,msb22,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(mst22,
     msb22,mC2)) - msb12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2)) -
     msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*mC2*(msb12*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + msb12*power3(
     DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) +
     mst12*power2(mC2) + power6(mst1)) + msb22*power3(DeltaInv(mst22,msb12,mC2)
     )*power6(mst2) - msb12*power3(DeltaInv(mst22,msb22,mC2))*power6(mst2)))/(
     msb12*msb22*(mst12 - mst22)) + (2*mt2*power2(snb)*power2(log(mC2/Q2))*
     power3(mC2)*(-(msb22*mst12*power2(DeltaInv(msb12,mst12,mC2))) + msb12*
     mst12*power2(DeltaInv(msb22,mst12,mC2)) + msb22*mst22*power2(DeltaInv(
     mst22,msb12,mC2)) - msb12*mst22*power2(DeltaInv(mst22,msb22,mC2)) + 2*mC2*
     msb12*msb22*mst22*power3(DeltaInv(mst22,msb12,mC2)) + 2*mC2*msb22*mst24*
     power3(DeltaInv(mst22,msb12,mC2)) + 2*msb12*msb22*mst24*power3(DeltaInv(
     mst22,msb12,mC2)) - msb22*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 2*mC2*msb12*msb22*mst22*power3(DeltaInv(mst22,msb22,mC2)) - 2*mC2*
     msb12*mst24*power3(DeltaInv(mst22,msb22,mC2)) - 2*msb12*msb22*mst24*
     power3(DeltaInv(mst22,msb22,mC2)) + msb12*mst22*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + msb22*power3(DeltaInv(msb12,mst12,mC2))*(-2*
     msb12*mst14 - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) - msb12*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(
     msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) - msb22*power3(
     DeltaInv(mst22,msb12,mC2))*power6(mst2) + msb12*power3(DeltaInv(mst22,
     msb22,mC2))*power6(mst2)))/(msb12*msb22*(mst12 - mst22)) + (mC2*mt2*log(
     mC2/Q2)*power2(snb)*(-(msb12*mst12) + msb22*mst12 + msb12*mst22 - msb22*
     mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(msb12,mst12,mC2)
     + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) +
     mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*mC2*msb22*mst12*mst22*
     DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*DeltaInv(mst22,msb22,
     mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,mC2) - msb22*mst12*
     DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*DeltaInv(mst22,msb22,
     mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*mst12 - mC2*(msb12 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb12,mst12,mC2)) + 6*
     mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 + 14*mst12) + mst14 +
     power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*mC2*msb22*mst12*mst22*
     mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*msb22*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 6*mC2*msb12*mst12*
     mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*msb22*mst12*mst22*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*msb12*msb22*mst12*
     power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*msb12*mst12*power2(
     mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb22*mst12*
     mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*msb12*mst12*mst22*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*msb12*msb22*mst12*
     mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*msb22*
     mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) - 144*
     msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(
     mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*power3(
     mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(mst22)*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*mst12*
     mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14 - 2*
     mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*msb12*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14
     - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(6.*msb12*msb22*mst12*(mst12 - mst22)*mst22) - (mC2*mt2*
     log(mC2/Q2)*power2(cb)*power2(snb)*(-(msb12*mst12) + msb22*mst12 + msb12*
     mst22 - msb22*mst22 + mC2*msb22*(mC2 - msb12 + 5*mst12)*mst22*DeltaInv(
     msb12,mst12,mC2) + mC2*msb12*(-mC2 + msb22 - 5*mst12)*mst22*DeltaInv(
     msb22,mst12,mC2) + mC2*msb12*msb22*mst12*DeltaInv(mst22,msb12,mC2) - 5*
     mC2*msb22*mst12*mst22*DeltaInv(mst22,msb12,mC2) - mC2*msb12*msb22*mst12*
     DeltaInv(mst22,msb22,mC2) + 5*mC2*msb12*mst12*mst22*DeltaInv(mst22,msb22,
     mC2) - msb22*mst12*DeltaInv(mst22,msb12,mC2)*power2(mC2) + msb12*mst12*
     DeltaInv(mst22,msb22,mC2)*power2(mC2) - 6*mC2*msb22*mst12*mst22*(-3*msb12*
     mst12 - mC2*(msb12 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(
     msb12,mst12,mC2)) + 6*mC2*msb12*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22 +
     14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) + 6*
     mC2*msb22*mst12*mst22*mst24*power2(DeltaInv(mst22,msb12,mC2)) - 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) - 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) - 84*
     msb22*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb12,mC2)) -
     6*mC2*msb12*mst12*mst22*mst24*power2(DeltaInv(mst22,msb22,mC2)) + 6*msb12*
     msb22*mst12*mst22*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) + 18*mC2*
     msb12*msb22*mst12*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) + 84*
     msb12*mst12*power2(mC2)*power2(mst22)*power2(DeltaInv(mst22,msb22,mC2)) +
     6*msb22*mst12*mst22*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 6*
     msb12*mst12*mst22*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 144*
     msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2)
     ) - 144*msb22*mst12*mst22*mst24*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) - 144*msb12*msb22*mst12*power2(mst22)*power3(mC2)*power3(DeltaInv(
     mst22,msb12,mC2)) + 144*msb12*msb22*mst12*mst22*mst24*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 144*msb12*mst12*mst22*mst24*power3(mC2)*
     power3(DeltaInv(mst22,msb22,mC2)) + 144*msb12*msb22*mst12*power2(mst22)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + 72*msb22*mst12*power2(
     mst22)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 72*msb12*mst12*
     power2(mst22)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 72*msb22*
     mst12*mst22*power2(mC2)*power3(DeltaInv(msb12,mst12,mC2))*(-2*msb12*mst14
     - 2*mC2*(msb12*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*
     msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*
     mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) +
     72*msb22*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*power6(
     mst2) - 72*msb12*mst12*mst22*power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))
     *power6(mst2)))/(6.*msb12*msb22*mst12*(mst12 - mst22)*mst22) + (mC2*mt2*
     log(mC2/Q2)*power2(cb)*(-mst12 + mst22 + mC2*(-mC2 + msb22 - 5*mst12)*
     mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(mC2 - msb22 + 5*mst22)*
     DeltaInv(mst22,msb22,mC2) + 6*mC2*mst12*mst22*(-3*msb22*mst12 - mC2*(msb22
     + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,mC2)) - 6*
     mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24 + power2(
     mC2))*power2(DeltaInv(mst22,msb22,mC2)) + 72*mst12*mst22*power2(mC2)*
     power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12 +
     mst14) + mst12*power2(mC2) + power6(mst1)) - 72*mst12*mst22*power2(mC2)*
     power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 +
     mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb22*mst12*(mst12 -
     mst22)*mst22) + (mC2*mt2*log(mC2/Q2)*(mst12 - mst22 + mC2*(mC2 - msb22 +
     5*mst12)*mst22*DeltaInv(msb22,mst12,mC2) + mC2*mst12*(-mC2 + msb22 - 5*
     mst22)*DeltaInv(mst22,msb22,mC2) - 6*mC2*mst12*mst22*(-3*msb22*mst12 -
     mC2*(msb22 + 14*mst12) + mst14 + power2(mC2))*power2(DeltaInv(msb22,mst12,
     mC2)) + 6*mC2*mst12*mst22*(-3*msb22*mst22 - mC2*(msb22 + 14*mst22) + mst24
     + power2(mC2))*power2(DeltaInv(mst22,msb22,mC2)) - 72*mst12*mst22*power2(
     mC2)*power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*
     mst12 + mst14) + mst12*power2(mC2) + power6(mst1)) + 72*mst12*mst22*
     power2(mC2)*power3(DeltaInv(mst22,msb22,mC2))*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2))))/(6.*msb22*
     mst12*(mst12 - mst22)*mst22) + Fin3(mst22,msb12,mC2,Q2)*((4*mt2*power2(
     mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(mst22 - DeltaInv(
     mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22)) + (4*mt2*power2(cb)*
     power2(mC2)*power2(snb)*power2(DeltaInv(mst22,msb12,mC2))*(-mst22 +
     DeltaInv(mst22,msb12,mC2)*(-2*msb12*mst24 - 2*mC2*(msb12*mst22 + mst24) +
     mst22*power2(mC2) + power6(mst2))))/(msb12*(mst12 - mst22))) + (2*mt2*
     power2(log(mC2/Q2))*power3(mC2)*(-(mst12*power2(DeltaInv(msb22,mst12,mC2))
     ) + power3(DeltaInv(msb22,mst12,mC2))*(-2*msb22*mst14 - 2*mC2*(msb22*mst12
     + mst14) + mst12*power2(mC2) + power6(mst1)) + power2(DeltaInv(mst22,
     msb22,mC2))*(mst22 - DeltaInv(mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(
     msb22*mst22 + mst24) + mst22*power2(mC2) + power6(mst2)))))/(msb22*(mst12
     - mst22)) + (2*mt2*power2(cb)*power2(log(mC2/Q2))*power3(mC2)*(mst12*
     power2(DeltaInv(msb22,mst12,mC2)) - power3(DeltaInv(msb22,mst12,mC2))*(-2*
     msb22*mst14 - 2*mC2*(msb22*mst12 + mst14) + mst12*power2(mC2) + power6(
     mst1)) + power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,msb22,
     mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(mC2) +
     power6(mst2)))))/(msb22*(mst12 - mst22)) + Fin3(mst22,msb22,mC2,Q2)*((4*
     mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb22*(mst12 - mst22)) + (4*mt2*power2(cb)*power2(
     mC2)*power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(mst22 - DeltaInv(
     mst22,msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*
     power2(mC2) + power6(mst2))))/(msb22*(mst12 - mst22)) + (4*mt2*power2(cb)*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb22*(mst12 - mst22)) + (4*mt2*power2(mC2)*
     power2(snb)*power2(DeltaInv(mst22,msb22,mC2))*(-mst22 + DeltaInv(mst22,
     msb22,mC2)*(-2*msb22*mst24 - 2*mC2*(msb22*mst22 + mst24) + mst22*power2(
     mC2) + power6(mst2))))/(msb22*(mst12 - mst22))))) - (power2(snb)*(16 +
     mC2/msb12 - mC2/msb22 - (8*msb12)/msb22 - (8*msb22)/msb12 + msb12/mst12 -
     msb22/mst12 + mst12/msb12 - mst12/msb22 - mt2/msb12 + mt2/msb22 - mu2/
     msb12 + mu2/msb22 - 2*mh2*mu2*DeltaInv(mh2,msb12,msb12) + 4*mh2*mu2*
     DeltaInv(mh2,msb12,msb22) - 2*mh2*mu2*DeltaInv(mh2,msb22,msb22) + 2*msb12*
     mu2*DeltaInv(msb12,mt2,mu2) - (2*msb14*mu2*DeltaInv(msb12,mt2,mu2))/mt2 -
     2*mC2*mt2*DeltaInv(mst22,msb12,mC2) + 2*mC2*mt2*DeltaInv(mst22,msb22,mC2)
     - 2*msb22*mu2*DeltaInv(mt2,mu2,msb22) + (2*msb24*mu2*DeltaInv(mt2,mu2,
     msb22))/mt2 - 2*mu2*mw2*DeltaInv(mw2,msb12,mst12) + 2*mu2*mw2*DeltaInv(
     mw2,msb22,mst12) + 6*DeltaInv(msb12,mt2,mu2)*power2(mu2) - (80*msb12*
     DeltaInv(msb12,mt2,mu2)*power2(mu2))/mt2 - 6*DeltaInv(mt2,mu2,msb22)*
     power2(mu2) + (80*msb22*DeltaInv(mt2,mu2,msb22)*power2(mu2))/mt2 - (168*(
     3*mh2*msb22 + msb12*msb22 + 3*msb24)*mu2*power2(mh2)*power2(DeltaInv(mh2,
     msb12,msb22)))/msb12 + 84*mh2*msb24*mu2*power2(DeltaInv(mh2,msb22,msb22))
     - 336*msb14*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2)) + 84*mst22*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb12,mC2)) + (252*mst24*mt2*power2(mC2)
     *power2(DeltaInv(mst22,msb12,mC2)))/msb12 - 84*mst22*mt2*power2(mC2)*
     power2(DeltaInv(mst22,msb22,mC2)) - (252*mst24*mt2*power2(mC2)*power2(
     DeltaInv(mst22,msb22,mC2)))/msb22 + 336*msb24*power2(mu2)*power2(DeltaInv(
     mt2,mu2,msb22)) + 84*mst12*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,
     mst12)) + (252*mst14*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)))/
     msb12 - 84*mst12*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - (252*
     mst14*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)))/msb22 + (252*
     mst22*mt2*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2))/msb12 - (252*
     mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2))/msb22 - (84*
     power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*power3(msb12))/mt2 + 84*mu2*
     power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 6*power3(msb12)) + 504*mu2*
     power2(DeltaInv(mh2,msb22,msb22))*power3(msb22) + (84*power2(mu2)*power2(
     DeltaInv(mt2,mu2,msb22))*power3(msb22))/mt2 - (2*DeltaInv(msb12,mt2,mu2)*
     power3(mu2))/mt2 + (2*DeltaInv(mt2,mu2,msb22)*power3(mu2))/mt2 - 504*
     msb12*power2(DeltaInv(msb12,mt2,mu2))*power3(mu2) - (1344*msb14*power2(
     DeltaInv(msb12,mt2,mu2))*power3(mu2))/mt2 + 504*msb22*power2(DeltaInv(mt2,
     mu2,msb22))*power3(mu2) + (1344*msb24*power2(DeltaInv(mt2,mu2,msb22))*
     power3(mu2))/mt2 + (252*mst12*mu2*power2(DeltaInv(mw2,msb12,mst12))*
     power3(mw2))/msb12 - (252*mst12*mu2*power2(DeltaInv(mw2,msb22,mst12))*
     power3(mw2))/msb22 - 2688*power3(msb12)*power3(mu2)*power3(DeltaInv(msb12,
     mt2,mu2)) + 1176*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb12,mC2)) +
     420*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)) + (
     252*mt2*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb12,mC2)))/msb12
     - 1176*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 420*mt2*
     power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - (252*mt2*
     power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)))/msb22 + 2688*
     power3(msb22)*power3(mu2)*power3(DeltaInv(mt2,mu2,msb22)) + 420*mu2*
     power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,mst12)) + 1176*mst14*
     mu2*power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)) + (252*mu2*power3(mst12)
     *power3(mw2)*power3(DeltaInv(mw2,msb12,mst12)))/msb12 - 420*mu2*power2(
     mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) - 1176*mst14*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - (252*mu2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)))/msb22 + 420*mst22*mt2*
     power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + (252*mst24*mt2*power3(
     DeltaInv(mst22,msb12,mC2))*power4(mC2))/msb12 - 420*mst22*mt2*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mC2) - (252*mst24*mt2*power3(DeltaInv(
     mst22,msb22,mC2))*power4(mC2))/msb22 + 2016*mh2*mu2*power3(DeltaInv(mh2,
     msb12,msb12))*power4(msb12) - 168*power2(mu2)*power3(DeltaInv(msb12,mt2,
     mu2))*power4(msb12) + (840*power3(mu2)*power3(DeltaInv(msb12,mt2,mu2))*
     power4(msb12))/mt2 - (168*mu2*power2(mh2)*power3(DeltaInv(mh2,msb12,msb22)
     )*(14*mh2*msb12*msb24 + 5*msb12*msb22*power2(mh2) + 3*msb24*power2(mh2) -
     3*msb22*power3(mh2) + 3*mh2*power3(msb22) + 5*msb12*power3(msb22) - 3*
     power4(msb22)))/msb12 + 2016*mh2*mu2*power3(DeltaInv(mh2,msb22,msb22))*
     power4(msb22) + 168*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power4(
     msb22) - (840*power3(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22))/
     mt2 - (252*mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12)
     )/msb12 + (252*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mst12))/msb22 - (252*mt2*power2(mC2)*power3(DeltaInv(mst22,msb12,mC2))*
     power4(mst22))/msb12 + (252*mt2*power2(mC2)*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mst22))/msb22 - (588*msb12*power2(DeltaInv(msb12,mt2,mu2))*
     power4(mu2))/mt2 + (588*msb22*power2(DeltaInv(mt2,mu2,msb22))*power4(mu2))
     /mt2 - 4200*msb14*power3(DeltaInv(msb12,mt2,mu2))*power4(mu2) - (1512*
     power3(msb12)*power3(DeltaInv(msb12,mt2,mu2))*power4(mu2))/mt2 + 4200*
     msb24*power3(DeltaInv(mt2,mu2,msb22))*power4(mu2) + (1512*power3(msb22)*
     power3(DeltaInv(mt2,mu2,msb22))*power4(mu2))/mt2 + 420*mst12*mu2*power3(
     DeltaInv(mw2,msb12,mst12))*power4(mw2) + (252*mst14*mu2*power3(DeltaInv(
     mw2,msb12,mst12))*power4(mw2))/msb12 - 420*mst12*mu2*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mw2) - (252*mst14*mu2*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mw2))/msb22 - (252*mst22*mt2*power3(DeltaInv(mst22,msb12,
     mC2))*power5(mC2))/msb12 + (252*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2)
     )*power5(mC2))/msb22 + (168*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*
     power5(msb12))/mt2 - (168*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*
     power5(msb22))/mt2 - 1008*msb12*power3(DeltaInv(msb12,mt2,mu2))*power5(
     mu2) - (168*msb14*power3(DeltaInv(msb12,mt2,mu2))*power5(mu2))/mt2 + 1008*
     msb22*power3(DeltaInv(mt2,mu2,msb22))*power5(mu2) + (168*msb24*power3(
     DeltaInv(mt2,mu2,msb22))*power5(mu2))/mt2 - (252*mst12*mu2*power3(
     DeltaInv(mw2,msb12,mst12))*power5(mw2))/msb12 + (252*mst12*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2))/msb22 + (672*msb12*power3(
     DeltaInv(msb12,mt2,mu2))*power6(mu2))/mt2 - (672*msb22*power3(DeltaInv(
     mt2,mu2,msb22))*power6(mu2))/mt2))/12. - (power2(Pi)*power2(snb)*(-(msb22*
     DeltaInv(msb12,mt2,mu2)*power2(msb12)*power2(mu2)) + msb12*DeltaInv(mt2,
     mu2,msb22)*power2(msb22)*power2(mu2) - 2*msb22*(3*mh2*msb22 + msb12*msb22
     + 3*msb24)*mt2*mu2*power2(mh2)*power2(DeltaInv(mh2,msb12,msb22)) + mh2*
     msb12*msb22*msb24*mt2*mu2*power2(DeltaInv(mh2,msb22,msb22)) - 4*msb12*
     msb14*msb22*mt2*power2(mu2)*power2(DeltaInv(msb12,mt2,mu2)) + msb12*msb22*
     mst22*power2(mC2)*power2(mt2)*power2(DeltaInv(mst22,msb12,mC2)) + 3*msb22*
     mst24*power2(mC2)*power2(mt2)*power2(DeltaInv(mst22,msb12,mC2)) - msb12*
     msb22*mst22*power2(mC2)*power2(mt2)*power2(DeltaInv(mst22,msb22,mC2)) - 3*
     msb12*mst24*power2(mC2)*power2(mt2)*power2(DeltaInv(mst22,msb22,mC2)) + 4*
     msb12*msb22*msb24*mt2*power2(mu2)*power2(DeltaInv(mt2,mu2,msb22)) + msb12*
     msb22*mst12*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) + 3*
     msb22*mst14*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb12,mst12)) - msb12*
     msb22*mst12*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) - 3*
     msb12*mst14*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,mst12)) + 3*
     msb22*mst22*power2(mt2)*power2(DeltaInv(mst22,msb12,mC2))*power3(mC2) - 3*
     msb12*mst22*power2(mt2)*power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) +
     msb12*msb22*mt2*mu2*power2(DeltaInv(mh2,msb12,msb12))*(mh2*msb14 + 6*
     power3(msb12)) - 16*msb12*msb14*msb22*power2(DeltaInv(msb12,mt2,mu2))*
     power3(mu2) - 6*msb22*mt2*power2(msb12)*power2(DeltaInv(msb12,mt2,mu2))*
     power3(mu2) + 16*msb12*msb22*msb24*power2(DeltaInv(mt2,mu2,msb22))*power3(
     mu2) + 6*msb12*mt2*power2(msb22)*power2(DeltaInv(mt2,mu2,msb22))*power3(
     mu2) + 3*msb22*mst12*mt2*mu2*power2(DeltaInv(mw2,msb12,mst12))*power3(mw2)
     - 3*msb12*mst12*mt2*mu2*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2) +
     14*msb12*msb22*mst24*power2(mt2)*power3(mC2)*power3(DeltaInv(mst22,msb12,
     mC2)) + 5*msb12*msb22*power2(mC2)*power2(mt2)*power3(mst22)*power3(
     DeltaInv(mst22,msb12,mC2)) + 3*msb22*power2(mt2)*power3(mC2)*power3(mst22)
     *power3(DeltaInv(mst22,msb12,mC2)) - 14*msb12*msb22*mst24*power2(mt2)*
     power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) - 5*msb12*msb22*power2(mC2)*
     power2(mt2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) - 3*msb12*
     power2(mt2)*power3(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,mC2)) +
     5*msb12*msb22*mt2*mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb12,
     mst12)) + 14*msb12*msb22*mst14*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,
     msb12,mst12)) + 3*msb22*mt2*mu2*power3(mst12)*power3(mw2)*power3(DeltaInv(
     mw2,msb12,mst12)) - 5*msb12*msb22*mt2*mu2*power2(mw2)*power3(mst12)*
     power3(DeltaInv(mw2,msb22,mst12)) - 14*msb12*msb22*mst14*mt2*mu2*power3(
     mw2)*power3(DeltaInv(mw2,msb22,mst12)) - 3*msb12*mt2*mu2*power3(mst12)*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 5*msb12*msb22*mst22*
     power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) + 3*msb22*mst24*
     power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*power4(mC2) - 5*msb12*msb22*
     mst22*power2(mt2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - 3*msb12*
     mst24*power2(mt2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) - msb22*
     power2(mu2)*power2(DeltaInv(msb12,mt2,mu2))*power4(msb12) - 32*msb22*mt2*
     power3(mu2)*power3(DeltaInv(msb12,mt2,mu2))*power4(msb12) + 6*msb12*mt2*
     mu2*power2(DeltaInv(mh2,msb22,msb22))*power4(msb22) + msb12*power2(mu2)*
     power2(DeltaInv(mt2,mu2,msb22))*power4(msb22) + 32*msb12*mt2*power3(mu2)*
     power3(DeltaInv(mt2,mu2,msb22))*power4(msb22) + 2*msb22*mt2*mu2*power2(
     mh2)*power3(DeltaInv(mh2,msb12,msb22))*(-14*mh2*msb12*msb24 - 5*msb12*
     msb22*power2(mh2) - 3*msb24*power2(mh2) + 3*msb22*power3(mh2) - 3*mh2*
     power3(msb22) - 5*msb12*power3(msb22) + 3*power4(msb22)) - 3*msb22*mt2*
     mu2*power2(mw2)*power3(DeltaInv(mw2,msb12,mst12))*power4(mst12) + 3*msb12*
     mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(mst12) - 3*
     msb22*power2(mC2)*power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*power4(
     mst22) + 3*msb12*power2(mC2)*power2(mt2)*power3(DeltaInv(mst22,msb22,mC2))
     *power4(mst22) - 7*msb22*power2(msb12)*power2(DeltaInv(msb12,mt2,mu2))*
     power4(mu2) + 7*msb12*power2(msb22)*power2(DeltaInv(mt2,mu2,msb22))*
     power4(mu2) - 50*msb12*msb14*msb22*mt2*power3(DeltaInv(msb12,mt2,mu2))*
     power4(mu2) + 50*msb12*msb22*msb24*mt2*power3(DeltaInv(mt2,mu2,msb22))*
     power4(mu2) - 18*msb22*power3(DeltaInv(msb12,mt2,mu2))*power4(msb12)*
     power4(mu2) + 18*msb12*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22)*
     power4(mu2) + 5*msb12*msb22*mst12*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12)
     )*power4(mw2) + 3*msb22*mst14*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12))*
     power4(mw2) - 5*msb12*msb22*mst12*mt2*mu2*power3(DeltaInv(mw2,msb22,mst12)
     )*power4(mw2) - 3*msb12*mst14*mt2*mu2*power3(DeltaInv(mw2,msb22,mst12))*
     power4(mw2) - 3*msb22*mst22*power2(mt2)*power3(DeltaInv(mst22,msb12,mC2))*
     power5(mC2) + 3*msb12*mst22*power2(mt2)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) + 24*mh2*msb22*mt2*mu2*power3(DeltaInv(mh2,msb12,msb12))*
     power5(msb12) - 2*msb22*mt2*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*
     power5(msb12) + 10*msb22*power3(mu2)*power3(DeltaInv(msb12,mt2,mu2))*
     power5(msb12) + 24*mh2*msb12*mt2*mu2*power3(DeltaInv(mh2,msb22,msb22))*
     power5(msb22) + 2*msb12*mt2*power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*
     power5(msb22) - 10*msb12*power3(mu2)*power3(DeltaInv(mt2,mu2,msb22))*
     power5(msb22) - 2*msb12*msb14*msb22*power3(DeltaInv(msb12,mt2,mu2))*
     power5(mu2) - 12*msb22*mt2*power2(msb12)*power3(DeltaInv(msb12,mt2,mu2))*
     power5(mu2) + 2*msb12*msb22*msb24*power3(DeltaInv(mt2,mu2,msb22))*power5(
     mu2) + 12*msb12*mt2*power2(msb22)*power3(DeltaInv(mt2,mu2,msb22))*power5(
     mu2) - 3*msb22*mst12*mt2*mu2*power3(DeltaInv(mw2,msb12,mst12))*power5(mw2)
     + 3*msb12*mst12*mt2*mu2*power3(DeltaInv(mw2,msb22,mst12))*power5(mw2) + 2*
     msb22*power2(mu2)*power3(DeltaInv(msb12,mt2,mu2))*power6(msb12) - 2*msb12*
     power2(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power6(msb22) + 8*msb22*
     power2(msb12)*power3(DeltaInv(msb12,mt2,mu2))*power6(mu2) - 8*msb12*
     power2(msb22)*power3(DeltaInv(mt2,mu2,msb22))*power6(mu2)))/(6.*msb12*
     msb22*mt2) - (2*mC2*msb12*msb22*mst22*mt2 + mA2*msb12*mst12*mst22*mt2 + 2*
     mC2*msb12*mst12*mst22*mt2 + mH2*msb12*mst12*mst22*mt2 + mA2*msb22*mst12*
     mst22*mt2 + mH2*msb22*mst12*mst22*mt2 - 4*msb12*msb22*mst12*mst22*mt2 + 4*
     msb12*msb22*mst12*mst22*mu2 - 2*msb12*msb22*mst22*mt2*mu2 - 4*msb12*mst12*
     mst22*mt2*mu2 - 2*msb22*mst12*mst22*mt2*mu2 - 2*mh2*msb12*msb22*mst12*
     mst22*mt2*mu2*DeltaInv(mh2,msb12,msb22) - 2*msb12*msb22*mst12*mst22*mt2*
     mu2*mz2*DeltaInv(msb12,mz2,msb22) - 4*msb12*msb22*msb24*mst12*mst22*mu2*
     DeltaInv(mt2,mu2,msb22) - 4*msb12*msb22*mst12*mst22*mt2*mu2*mw2*DeltaInv(
     mw2,msb22,mst12) + 2*mst12*mst22*mt2*power2(msb12) + 2*msb12*mst22*mt2*
     power2(msb22) + 2*mst12*mst22*mt2*power2(msb22) + 4*msb12*mst12*mst22*mt2*
     mu2*DeltaInv(mt2,mu2,msb22)*power2(msb22) + 2*msb12*mst22*mt2*power2(
     mst12) + 2*msb12*msb22*mst12*power2(mt2) - 2*msb12*mst12*mst22*power2(mt2)
     - 4*mC2*msb12*msb22*mst12*mst22*DeltaInv(mst22,msb22,mC2)*power2(mt2) +
     12*msb12*msb22*mst12*mst22*mt2*DeltaInv(mt2,mu2,msb22)*power2(mu2) - 160*
     msb12*mst12*mst22*DeltaInv(mt2,mu2,msb22)*power2(msb22)*power2(mu2) + 84*
     msb22*(3*mh2*msb22 + msb12*msb22 + 3*msb24)*mst12*mst22*mt2*mu2*power2(
     mh2)*power2(DeltaInv(mh2,msb12,msb22)) + 84*msb22*mst12*mst22*mt2*mu2*(
     msb12*msb22 + 3*msb24 + 3*msb22*mz2)*power2(mz2)*power2(DeltaInv(msb12,
     mz2,msb22)) + 504*msb12*mst12*mst22*mst24*power2(mC2)*power2(mt2)*power2(
     DeltaInv(mst22,msb22,mC2)) + 168*msb12*msb22*mst12*power2(mC2)*power2(
     mst22)*power2(mt2)*power2(DeltaInv(mst22,msb22,mC2)) - 672*msb12*msb22*
     msb24*mst12*mst22*mt2*power2(mu2)*power2(DeltaInv(mt2,mu2,msb22)) + 504*
     msb12*mst12*mst14*mst22*mt2*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)) + 168*msb12*msb22*mst22*mt2*mu2*power2(mst12)*power2(mw2)*power2(
     DeltaInv(mw2,msb22,mst12)) + 504*msb12*mst12*power2(mst22)*power2(mt2)*
     power2(DeltaInv(mst22,msb22,mC2))*power3(mC2) - 4*msb12*msb22*mst12*mst22*
     DeltaInv(mt2,mu2,msb22)*power3(mu2) - 2688*msb12*msb22*msb24*mst12*mst22*
     power2(DeltaInv(mt2,mu2,msb22))*power3(mu2) - 1008*msb12*mst12*mst22*mt2*
     power2(msb22)*power2(DeltaInv(mt2,mu2,msb22))*power3(mu2) + 504*msb12*
     mst22*mt2*mu2*power2(mst12)*power2(DeltaInv(mw2,msb22,mst12))*power3(mw2)
     - 84*msb22*mst12*mst22*mt2*mu2*power2(mh2)*(-((5*msb12*msb22 + 3*msb24)*
     power2(mh2)) + 3*msb22*power3(mh2) + (-5*msb12 + 3*msb22)*power3(msb22) -
     mh2*(14*msb12*msb24 + 3*power3(msb22)))*power3(DeltaInv(mh2,msb12,msb22))
     + 2352*msb12*msb22*mst12*mst22*mst24*power2(mt2)*power3(mC2)*power3(
     DeltaInv(mst22,msb22,mC2)) + 2352*msb12*msb22*mst12*mst14*mst22*mt2*mu2*
     power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) + 504*msb12*mst12*mst22*
     mst24*power2(mt2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mC2) + 840*
     msb12*msb22*mst12*power2(mst22)*power2(mt2)*power3(DeltaInv(mst22,msb22,
     mC2))*power4(mC2) + 84*msb22*mst12*mst22*mt2*mu2*power2(mz2)*power3(
     DeltaInv(msb12,mz2,msb22))*(14*msb12*msb24*mz2 + 5*msb12*msb22*power2(mz2)
     + 3*msb24*power2(mz2) + 5*msb12*power3(msb22) + 3*mz2*power3(msb22) - 3*
     msb22*power3(mz2) - 3*power4(msb22)) - 168*msb12*mst12*mst22*power2(mu2)*
     power2(DeltaInv(mt2,mu2,msb22))*power4(msb22) - 5376*msb12*mst12*mst22*
     mt2*power3(mu2)*power3(DeltaInv(mt2,mu2,msb22))*power4(msb22) + 840*msb12*
     msb22*mst22*mt2*mu2*power2(mw2)*power3(DeltaInv(mw2,msb22,mst12))*power4(
     mst12) + 504*msb12*mst22*mt2*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,
     mst12))*power4(mst12) + 840*msb12*msb22*mst12*power2(mC2)*power2(mt2)*
     power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) + 504*msb12*mst12*power2(
     mt2)*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22) - 1176*
     msb12*mst12*mst22*power2(msb22)*power2(DeltaInv(mt2,mu2,msb22))*power4(
     mu2) - 8400*msb12*msb22*msb24*mst12*mst22*mt2*power3(DeltaInv(mt2,mu2,
     msb22))*power4(mu2) - 3024*msb12*mst12*mst22*power3(DeltaInv(mt2,mu2,
     msb22))*power4(msb22)*power4(mu2) + 504*msb12*mst12*mst14*mst22*mt2*mu2*
     power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) + 840*msb12*msb22*mst22*mt2*
     mu2*power2(mst12)*power3(DeltaInv(mw2,msb22,mst12))*power4(mw2) - 504*
     msb12*mst12*power2(mst22)*power2(mt2)*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2) - 336*msb12*mst12*mst22*mt2*power2(mu2)*power3(DeltaInv(mt2,
     mu2,msb22))*power5(msb22) + 1680*msb12*mst12*mst22*power3(mu2)*power3(
     DeltaInv(mt2,mu2,msb22))*power5(msb22) - 504*msb12*mst22*mt2*mu2*power2(
     mw2)*power3(DeltaInv(mw2,msb22,mst12))*power5(mst12) - 504*msb12*mst12*
     power2(mC2)*power2(mt2)*power3(DeltaInv(mst22,msb22,mC2))*power5(mst22) -
     336*msb12*msb22*msb24*mst12*mst22*power3(DeltaInv(mt2,mu2,msb22))*power5(
     mu2) - 2016*msb12*mst12*mst22*mt2*power2(msb22)*power3(DeltaInv(mt2,mu2,
     msb22))*power5(mu2) - 504*msb12*mst22*mt2*mu2*power2(mst12)*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2) + 336*msb12*mst12*mst22*power2(mu2)
     *power3(DeltaInv(mt2,mu2,msb22))*power6(msb22) + 1344*msb12*mst12*mst22*
     power2(msb22)*power3(DeltaInv(mt2,mu2,msb22))*power6(mu2))/(24.*msb12*
     msb22*mst12*mst22*mt2) - (power2(cb)*((-9*mA2)/msb12 - (9*mA2)/msb22 - (
     18*mC2)/msb22 - (18*mC2)/mst12 - (35*mC2)/(mC2 - mt2) - (35*mC2)/mt2 - (
     18*mt2)/msb22 - (18*mt2)/mst22 + (36*mt2*mu2)/((mst12 - mst22)*mst22) + (
     18*mw2)/msb22 + (18*mw2)/mst12 + (35*mw2)/mt2 + (35*mw2)/(-mt2 + mw2) + (
     9*mz2)/msb12 + (9*mz2)/msb22 - 18*mA2*mu2*DeltaInv(msb12,mA2,msb22) + 18*
     mu2*mz2*DeltaInv(msb12,mz2,msb22) - 36*mC2*mu2*DeltaInv(msb22,mst12,mC2) +
     (72*mC2*mt2*mu2*DeltaInv(msb22,mst12,mC2))/(mst12 - mst22) + 36*mC2*mt2*
     DeltaInv(mst22,msb22,mC2) - (72*mC2*mt2*mu2*DeltaInv(mst22,msb22,mC2))/(
     mst12 - mst22) + 36*mu2*mw2*DeltaInv(mw2,msb22,mst12) - (36*mt2*mu2)/(-(
     mst12*mst22) + power2(mst12)) - (35*power2(mC2))/power2(mC2 - mt2) + (35*
     power2(mw2))/power2(mt2 - mw2) + (756*(3*mA2*msb22 + msb12*msb22 + 3*
     msb24)*mu2*power2(mA2)*power2(DeltaInv(msb12,mA2,msb22)))/msb12 - (756*
     mu2*(msb12*msb22 + 3*msb24 + 3*msb22*mz2)*power2(mz2)*power2(DeltaInv(
     msb12,mz2,msb22)))/msb12 + 1512*mst12*mu2*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)) + (4536*mst14*mu2*power2(mC2)*power2(DeltaInv(msb22,
     mst12,mC2)))/msb22 - (3024*mst12*mt2*mu2*power2(mC2)*power2(DeltaInv(
     msb22,mst12,mC2)))/(mst12 - mst22) - (9072*mst14*mt2*mu2*power2(mC2)*
     power2(DeltaInv(msb22,mst12,mC2)))/(msb22*mst12 - msb22*mst22) - 1512*
     mst22*mt2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)) - (4536*mst24*mt2*
     power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)))/msb22 + (3024*mst22*mt2*
     mu2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)))/(mst12 - mst22) + (
     9072*mst24*mt2*mu2*power2(mC2)*power2(DeltaInv(mst22,msb22,mC2)))/(msb22*
     mst12 - msb22*mst22) - 1512*mst12*mu2*power2(mw2)*power2(DeltaInv(mw2,
     msb22,mst12)) - (4536*mst14*mu2*power2(mw2)*power2(DeltaInv(mw2,msb22,
     mst12)))/msb22 + (45*power3(mC2))/(mt2*power2(mC2 - mt2)) + (4536*mst12*
     mu2*power2(DeltaInv(msb22,mst12,mC2))*power3(mC2))/msb22 - (9072*mst12*
     mt2*mu2*power2(DeltaInv(msb22,mst12,mC2))*power3(mC2))/(msb22*mst12 -
     msb22*mst22) - (4536*mst22*mt2*power2(DeltaInv(mst22,msb22,mC2))*power3(
     mC2))/msb22 + (9072*mst22*mt2*mu2*power2(DeltaInv(mst22,msb22,mC2))*
     power3(mC2))/(msb22*mst12 - msb22*mst22) + (10*power3(mC2))/power3(mC2 -
     mt2) - (45*power3(mw2))/(mt2*power2(mt2 - mw2)) - (4536*mst12*mu2*power2(
     DeltaInv(mw2,msb22,mst12))*power3(mw2))/msb22 + (10*power3(mw2))/power3(
     mt2 - mw2) + (756*mu2*power2(mA2)*((5*msb12*msb22 + 3*msb24)*power2(mA2) -
     3*msb22*power3(mA2) + (5*msb12 - 3*msb22)*power3(msb22) + mA2*(14*msb12*
     msb24 + 3*power3(msb22)))*power3(DeltaInv(msb12,mA2,msb22)))/msb12 +
     21168*mst14*mu2*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2)) - (42336*
     mst14*mt2*mu2*power3(mC2)*power3(DeltaInv(msb22,mst12,mC2)))/(mst12 -
     mst22) + 7560*mu2*power2(mC2)*power3(mst12)*power3(DeltaInv(msb22,mst12,
     mC2)) - (15120*mt2*mu2*power2(mC2)*power3(mst12)*power3(DeltaInv(msb22,
     mst12,mC2)))/(mst12 - mst22) + (4536*mu2*power3(mC2)*power3(mst12)*power3(
     DeltaInv(msb22,mst12,mC2)))/msb22 - (9072*mt2*mu2*power3(mC2)*power3(
     mst12)*power3(DeltaInv(msb22,mst12,mC2)))/(msb22*mst12 - msb22*mst22) -
     21168*mst24*mt2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)) + (42336*
     mst24*mt2*mu2*power3(mC2)*power3(DeltaInv(mst22,msb22,mC2)))/(mst12 -
     mst22) - 7560*mt2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,msb22,
     mC2)) + (15120*mt2*mu2*power2(mC2)*power3(mst22)*power3(DeltaInv(mst22,
     msb22,mC2)))/(mst12 - mst22) - (4536*mt2*power3(mC2)*power3(mst22)*power3(
     DeltaInv(mst22,msb22,mC2)))/msb22 + (9072*mt2*mu2*power3(mC2)*power3(
     mst22)*power3(DeltaInv(mst22,msb22,mC2)))/(msb22*mst12 - msb22*mst22) -
     7560*mu2*power2(mw2)*power3(mst12)*power3(DeltaInv(mw2,msb22,mst12)) -
     21168*mst14*mu2*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)) - (4536*mu2*
     power3(mst12)*power3(mw2)*power3(DeltaInv(mw2,msb22,mst12)))/msb22 + 7560*
     mst12*mu2*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2) + (4536*mst14*mu2*
     power3(DeltaInv(msb22,mst12,mC2))*power4(mC2))/msb22 - (15120*mst12*mt2*
     mu2*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2))/(mst12 - mst22) - (
     9072*mst14*mt2*mu2*power3(DeltaInv(msb22,mst12,mC2))*power4(mC2))/(msb22*
     mst12 - msb22*mst22) - 7560*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2) - (4536*mst24*mt2*power3(DeltaInv(mst22,msb22,mC2))*power4(
     mC2))/msb22 + (15120*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))*
     power4(mC2))/(mst12 - mst22) + (9072*mst24*mt2*mu2*power3(DeltaInv(mst22,
     msb22,mC2))*power4(mC2))/(msb22*mst12 - msb22*mst22) - (756*mu2*power2(
     mz2)*power3(DeltaInv(msb12,mz2,msb22))*(14*msb12*msb24*mz2 + 5*msb12*
     msb22*power2(mz2) + 3*msb24*power2(mz2) + 5*msb12*power3(msb22) + 3*mz2*
     power3(msb22) - 3*msb22*power3(mz2) - 3*power4(msb22)))/msb12 - (4536*mu2*
     power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*power4(mst12))/msb22 + (
     9072*mt2*mu2*power2(mC2)*power3(DeltaInv(msb22,mst12,mC2))*power4(mst12))/
     (msb22*mst12 - msb22*mst22) + (4536*mu2*power2(mw2)*power3(DeltaInv(mw2,
     msb22,mst12))*power4(mst12))/msb22 + (4536*mt2*power2(mC2)*power3(
     DeltaInv(mst22,msb22,mC2))*power4(mst22))/msb22 - (9072*mt2*mu2*power2(
     mC2)*power3(DeltaInv(mst22,msb22,mC2))*power4(mst22))/(msb22*mst12 -
     msb22*mst22) + (10*power4(mC2))/power4(mC2 - mt2) - 7560*mst12*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power4(mw2) - (4536*mst14*mu2*power3(DeltaInv(
     mw2,msb22,mst12))*power4(mw2))/msb22 - (10*power4(mw2))/power4(mt2 - mw2)
     - (4536*mst12*mu2*power3(DeltaInv(msb22,mst12,mC2))*power5(mC2))/msb22 + (
     9072*mst12*mt2*mu2*power3(DeltaInv(msb22,mst12,mC2))*power5(mC2))/(msb22*
     mst12 - msb22*mst22) + (4536*mst22*mt2*power3(DeltaInv(mst22,msb22,mC2))*
     power5(mC2))/msb22 + (9072*mst22*mt2*mu2*power3(DeltaInv(mst22,msb22,mC2))
     *power5(mC2))/(msb22*(-mst12 + mst22)) - (223*power5(mC2))/(mt2*power4(mC2
     - mt2)) - (213*power5(mC2))/power5(mC2 - mt2) + (4536*mst12*mu2*power3(
     DeltaInv(mw2,msb22,mst12))*power5(mw2))/msb22 + (223*power5(mw2))/(mt2*
     power4(mt2 - mw2)) + (213*power5(mw2))/power5(-mt2 + mw2) - (213*power6(
     mC2))/power6(mC2 - mt2) + (213*power6(mw2))/power6(mt2 - mw2) + (213*
     power7(mC2))/(mt2*power6(mC2 - mt2)) - (213*power7(mw2))/(mt2*power6(mt2 -
     mw2))))/216.;

   return power2(g3) * power2(yb) * result * twoLoop;
}

std::ostream& operator<<(std::ostream& out, const Parameters& pars)
{
   out <<
      "Delta alpha_s 2L parameters:\n"
      "g3   = " <<  pars.g3   << '\n' <<
      "yt   = " <<  pars.yt   << '\n' <<
      "yb   = " <<  pars.yb   << '\n' <<
      "mt   = " <<  pars.mt   << '\n' <<
      "mb   = " <<  pars.mb   << '\n' <<
      "mg   = " <<  pars.mg   << '\n' <<
      "mst1 = " <<  pars.mst1 << '\n' <<
      "mst2 = " <<  pars.mst2 << '\n' <<
      "msb1 = " <<  pars.msb1 << '\n' <<
      "msb2 = " <<  pars.msb2 << '\n' <<
      "msd1 = " <<  pars.msd1 << '\n' <<
      "msd2 = " <<  pars.msd2 << '\n' <<
      "xt   = " <<  pars.xt   << '\n' <<
      "xb   = " <<  pars.xb   << '\n' <<
      "mw   = " <<  pars.mw   << '\n' <<
      "mz   = " <<  pars.mz   << '\n' <<
      "mh   = " <<  pars.mh   << '\n' <<
      "mH   = " <<  pars.mH   << '\n' <<
      "mC   = " <<  pars.mC   << '\n' <<
      "mA   = " <<  pars.mA   << '\n' <<
      "mu   = " <<  pars.mu   << '\n' <<
      "tb   = " <<  pars.tb   << '\n' <<
      "Q    = " <<  pars.Q    << '\n';

   return out;
}

} // namespace mssm_twoloop_as
} // namespace flexiblesusy
