// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================

// This file has been generated at Sat 22 Jul 2017 11:36:23
// with the script "tau_to_cpp.m".

#include "mssm_twoloop_mtau.h"
#include "numerics.h"
#include <algorithm>
#include <array>
#include <cmath>
#include <complex>
#include <limits>
#include <ostream>

namespace flexiblesusy {
namespace mssm_twoloop_mtau {

namespace {
   const Real Pi = 3.1415926535897932384626433832795l;

   template <typename T> T power2(T x)  { return x*x; }
   template <typename T> T power3(T x)  { return x*x*x; }
   template <typename T> T power4(T x)  { return x*x*x*x; }
   template <typename T> T power5(T x)  { return x*x*x*x*x; }
   template <typename T> T power6(T x)  { return x*x*x*x*x*x; }
   template <typename T> T power7(T x)  { return x*x*x*x*x*x*x; }
   template <typename T> T power8(T x)  { return x*x*x*x*x*x*x*x; }
   template <typename T> T power10(T x) { return x*x*x*x*x * x*x*x*x*x; }
   template <typename T> T power12(T x) { return x*x*x*x*x * x*x*x*x*x * x*x; }
   template <typename T> T power14(T x) { return x*x*x*x*x * x*x*x*x*x * x*x*x*x; }

   const Real oneLoop = 1.l/power2(4*Pi);
   const Real twoLoop = power2(oneLoop);

   template <typename T>
   bool is_zero(T a, T prec = std::numeric_limits<T>::epsilon())
   {
      return std::fabs(a) < prec;
   }

   template <typename T>
   bool is_equal(T a, T b, T prec = std::numeric_limits<T>::epsilon())
   {
      return is_zero(a - b, prec);
   }

   template <typename T>
   bool is_equal_rel(T a, T b, T prec = std::numeric_limits<T>::epsilon())
   {
      if (is_equal(a, b, std::numeric_limits<T>::epsilon()))
         return true;

      if (std::abs(a) < std::numeric_limits<T>::epsilon() ||
          std::abs(b) < std::numeric_limits<T>::epsilon())
         return false;

      return std::abs((a - b)/a) < prec;
   }

   /**
    * Fin20[] function from twoloopbubble.m .
    *
    * @param mm1 squared mass \f$m_1^2\f$
    * @param mm2 squared mass \f$m_2^2\f$
    * @param mmu squared renormalization scale
    *
    * @return Fin20(m12, m22, mmu)
    */
   Real Fin20(Real mm1, Real mm2, Real mmu)
   {
      using std::log;

      return (6*(mm1*log(mm1/mmu) + mm2*log(mm2/mmu)) +
         (-mm1 - mm2)*(7 + power2(Pi)/6.) +
         (mm1 - mm2)*(2*dilog(1 - mm1/mm2) +
            power2(log(mm1/mm2))/2.) +
         ((mm1 + mm2)*power2(log(mm1/mm2)))/2. -
         2*(mm1*power2(log(mm1/mmu)) + mm2*power2(log(mm2/mmu))))/2.;
   }

   Real LambdaSquared(Real x, Real y)
   {
      return power2(1 - x - y) - 4*x*y;
   }

   /// ClausenCl[2,x]
   Real ClausenCl2(Real x)
   {
      using std::exp;
      const std::complex<Real> img(0.l,1.l);

      return std::imag(dilogcl(exp(img*x)));
   }

   /// x < 1 && y < 1, LambdaSquared(x,y) > 0
   Real PhiPos(Real x, Real y)
   {
      const Real lambda = std::sqrt(LambdaSquared(x,y));

      return (-(log(x)*log(y))
              + 2*log((1 - lambda + x - y)/2.)*log((1 - lambda - x + y)/2.)
              - 2*dilog((1 - lambda + x - y)/2.)
              - 2*dilog((1 - lambda - x + y)/2.)
              + power2(Pi)/3.)/lambda;
   }

   /// LambdaSquared(x,y) < 0
   Real PhiNeg(Real x, Real y)
   {
      using std::acos;
      using std::sqrt;
      const Real lambda = std::sqrt(-LambdaSquared(x,y));

      return 2*(+ ClausenCl2(2*acos((1 + x - y)/(2.*sqrt(x))))
                + ClausenCl2(2*acos((1 - x + y)/(2.*sqrt(y))))
                + ClausenCl2(2*acos((-1 + x + y)/(2.*sqrt(x*y)))))/lambda;
   }

   Real Phi(Real x, Real y)
   {
      const Real lambda = LambdaSquared(x,y);

      if (lambda > 0.)
         return PhiPos(x,y);

      return PhiNeg(x,y);
   }

   /**
    * Fin3[] function from twoloopbubble.m .
    *
    * @param mm1 squared mass \f$m_1^2\f$
    * @param mm2 squared mass \f$m_2^2\f$
    * @param mm3 squared mass \f$m_3^2\f$
    * @param mmu squared renormalization scale
    *
    * @return Fin3(m12, m22, m32, mmu)
    */
   Real Fin3(Real mm1, Real mm2, Real mm3, Real mmu)
   {
      using std::log;

      std::array<Real,3> masses = { mm1, mm2, mm3 };
      std::sort(masses.begin(), masses.end());

      const Real mm = masses[2];
      const Real x = masses[0]/mm;
      const Real y = masses[1]/mm;

      const Real lambda = LambdaSquared(x,y);

      if (is_zero(lambda, 1e-10l)) {
         return -(mm*(2*y*(-3 + 2*log(mm/mmu))*log(y)
                      + log(x)*(2*x*(-3 + 2*log(mm/mmu)) + (-1 + x + y)*log(y))
                      + (1 + x + y)*(7 - 6*log(mm/mmu) + power2(Pi)/6. + 2*power2(log(mm/mmu)))
                      + x*power2(log(x)) + y*power2(log(y))))/2.;
      }

      return mm*((-7 + 6*log(mm/mmu) + log(x)*log(y)
                  - lambda*Phi(x,y) - power2(Pi)/6. - 2*power2(log(mm/mmu)))/2.
                 - (x*(7 - 6*log(mm/mmu) + log(x)*(-6 + 4*log(mm/mmu) + log(y))
                       + power2(Pi)/6. + 2*power2(log(mm/mmu)) + power2(log(x))))/2.
                 - (y*(7 - 6*log(mm/mmu) + (
                     -6 + 4*log(mm/mmu) + log(x))*log(y) + power2(Pi)/6.
                       + 2*power2(log(mm/mmu)) + power2(log(y))))/2.);
   }

   /// Delta[m1,m2,m3,-1]
   Real DeltaInv(Real m1, Real m2, Real m3)
   {
      return 1./(power2(m1) + power2(m2) + power2(m3) - 2*(m1*m2 + m1*m3 + m2*m3));
   }

   /// calculates sin(theta)
   Real calc_sin_theta(Real mf, Real xf, Real msf12, Real msf22)
   {
      if (is_zero(mf, 1e-10l) || is_zero(xf, 1e-10l))
         return 0.;

      const Real sin_2theta = 2.0*mf*xf / (msf12 - msf22);
      const Real theta = 0.5*std::asin(sin_2theta);

      return std::sin(theta);
   }

   /// calculates Higgs mixing angle from squarde Higgs masses and tan(beta)
   Real calc_alpha(Real mh2, Real mH2, Real tb)
   {
      const Real beta = std::atan(tb);
      const Real sin_2alpha = -(mH2 + mh2)/(mH2 - mh2) * std::sin(2*beta);

      return 0.5*std::asin(sin_2alpha);
   }

} // anonymous namespace

double delta_mtau_2loop_atau_atau(const Parameters& pars)
{
   using std::log;
   const Real ytau  = pars.ytau;
   const Real xtau  = pars.xtau;
   const Real mstau12 = power2(pars.mstau1);
   const Real mstau2  = pars.mstau2;
   const Real mstau22 = power2(pars.mstau2);
   const Real mstau24 = power4(pars.mstau2);
   const Real msntau2 = power2(pars.msntau);
   const Real mw2   = power2(pars.mw);
   const Real mz2   = power2(pars.mz);
   const Real mh2   = power2(pars.mh);
   const Real mH2   = power2(pars.mH);
   const Real mC2   = power2(pars.mC);
   const Real mA2   = power2(pars.mA);
   const Real mu    = pars.mu;
   const Real mu2   = power2(pars.mu);
   const Real tb    = pars.tb;
   const Real sb    = tb / std::sqrt(1. + power2(tb));
   const Real cb    = 1. / std::sqrt(1. + power2(tb));
   const Real Q2    = power2(pars.Q);
   const Real alpha = calc_alpha(mh2, mH2, tb);
   const Real sa    = std::sin(alpha);
   const Real ca    = std::cos(alpha);

   const double result =
   ((-(mC2*mH2) + mA2*(3*mC2 + mH2))*(mstau12 + mstau22))/(4.*mA2*mC2*mH2) - (
     5*mC2*log(mA2/Q2)*log(mC2/Q2))/(16.*mA2) + ((3*mC2 + 4*mH2)*log(mC2/Q2)*
     log(mH2/Q2))/(16.*mH2) - ((-(1/mA2) + 1/mC2 + 3/mH2)*mstau12*log(mstau12/
     Q2))/4. + (ca*mstau12*mu2*sa*sb*log(mh2/Q2)*log(mstau12/Q2))/(2.*cb*(
     mstau12 - mstau22)*(mstau12 - mu2)) - (ca*mstau12*mu2*sa*sb*log(mH2/Q2)*
     log(mstau12/Q2))/(2.*cb*(mstau12 - mstau22)*(mstau12 - mu2)) + ((1/mA2 -
     1/mC2 - 3/mH2)*mstau22*log(mstau22/Q2))/4. - (ca*mstau22*mu2*sa*sb*log(
     mh2/Q2)*log(mstau22/Q2))/(2.*cb*(mstau12 - mstau22)*(mstau22 - mu2)) + (
     ca*mstau22*mu2*sa*sb*log(mH2/Q2)*log(mstau22/Q2))/(2.*cb*(mstau12 -
     mstau22)*(mstau22 - mu2)) + ((3*mA2 - 5*mC2)*Fin20(mA2,mC2,Q2))/(16.*
     power2(mA2)) + (log(mA2/Q2)*log(mH2/Q2)*(3*power2(mA2) - 5*power2(mH2)))/(
     16.*mA2*mH2) + ((3*mC2 - 5*mH2)*Fin20(mC2,mH2,Q2))/(16.*power2(mH2)) + (3*
     ca*(mh2 - mH2)*mu2*sa*sb*Fin20(mstau22,mstau12,Q2))/(2.*cb*mh2*power2(mH2)
     ) - (log(mA2/Q2)*log(msntau2/Q2)*(4*mA2*mu2 - 4*mA2*mu2*DeltaInv(mA2,
     msntau2,mu2)*(mu2*(-2*msntau2 + mu2) - 2*mA2*(msntau2 + mu2) + power2(mA2)
     ) + 5*power2(msntau2)))/(16.*mA2*msntau2) - (Fin20(msntau2,mu2,Q2)*(8*mA2*
     msntau2*power2(mH2) - 5*power2(mH2)*power2(msntau2 - mu2) + power2(mA2)*(-
     8*mH2*msntau2 + 2*power2(mH2) + 3*power2(msntau2 - mu2))))/(16.*(msntau2 -
     mu2)*power2(mA2)*power2(mH2)) + (3*mstau12*(mstau12 - 2*mu2)*log(mC2/Q2)*
     log(mstau12/Q2))/(16.*power2(mstau12 - mu2)) + (3*msntau2*mstau12*(msntau2
     - 2*mu2)*(mstau12 - 2*mu2)*log(msntau2/Q2)*log(mstau12/Q2))/(16.*power2(
     msntau2 - mu2)*power2(mstau12 - mu2)) - (ca*sa*sb*log(mh2/Q2)*log(mu2/Q2)*
     power2(mu2))/(2.*cb*(mstau12 - mu2)*(-mstau22 + mu2)) + (ca*sa*sb*log(mH2/
     Q2)*log(mu2/Q2)*power2(mu2))/(2.*cb*(mstau12 - mu2)*(-mstau22 + mu2)) - (
     mh2*log(mh2/Q2)*log(mu2/Q2)*power2(mu2)*(4*mstau12*mstau22*mu2 - mstau22*
     power2(mstau12) - mstau12*power2(mstau22) + mstau22*DeltaInv(mstau12,mh2,
     mu2)*(mu2*(-3*mstau12 + mu2) - mh2*(mstau12 + 2*mu2) + power2(mh2))*
     power2(mstau12 - mu2) + mstau12*DeltaInv(mstau22,mh2,mu2)*(mu2*(-3*mstau22
     + mu2) - mh2*(mstau22 + 2*mu2) + power2(mh2))*power2(mstau22 - mu2) -
     mstau12*power2(mu2) - mstau22*power2(mu2)))/(8.*mstau12*mstau22*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (ca*sa*sb*log(mh2/Q2)*log(mu2/Q2)*
     power2(mu2)*(-4*mh2*mstau12*mstau22*mu2 + mh2*mstau22*power2(mstau12) + 2*
     mstau22*mu2*power2(mstau12) + mh2*mstau12*power2(mstau22) + 2*mstau12*mu2*
     power2(mstau22) - 2*power2(mstau12)*power2(mstau22) - mh2*mstau22*
     DeltaInv(mstau12,mh2,mu2)*(mu2*(-3*mstau12 + mu2) - mh2*(mstau12 + 2*mu2)
     + power2(mh2))*power2(mstau12 - mu2) - mh2*mstau12*DeltaInv(mstau22,mh2,
     mu2)*(mu2*(-3*mstau22 + mu2) - mh2*(mstau22 + 2*mu2) + power2(mh2))*
     power2(mstau22 - mu2) + mh2*mstau12*power2(mu2) + mh2*mstau22*power2(mu2)
     - 2*mstau12*mstau22*power2(mu2)))/(4.*cb*mstau12*mstau22*power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) - (ca*sa*sb*log(mH2/Q2)*log(mu2/Q2)*power2(
     mu2)*(-4*mH2*mstau12*mstau22*mu2 + mH2*mstau22*power2(mstau12) + 2*
     mstau22*mu2*power2(mstau12) + mH2*mstau12*power2(mstau22) + 2*mstau12*mu2*
     power2(mstau22) - 2*power2(mstau12)*power2(mstau22) - mH2*mstau22*
     DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mH2*(mstau12 + 2*mu2)
     + power2(mH2))*power2(mstau12 - mu2) - mH2*mstau12*DeltaInv(mstau22,mH2,
     mu2)*(mu2*(-3*mstau22 + mu2) - mH2*(mstau22 + 2*mu2) + power2(mH2))*
     power2(mstau22 - mu2) + mH2*mstau12*power2(mu2) + mH2*mstau22*power2(mu2)
     - 2*mstau12*mstau22*power2(mu2)))/(4.*cb*mstau12*mstau22*power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) - (log(mH2/Q2)*log(msntau2/Q2)*(-4 - (3*
     msntau2)/mH2 + (4*mu2)/msntau2 - (4*mu2*DeltaInv(mH2,msntau2,mu2)*(mu2*(-
     2*msntau2 + mu2) - 2*mH2*(msntau2 + mu2) + power2(mH2)))/msntau2 + (4*
     power2(mu2))/power2(msntau2 - mu2)))/16. + (Fin3(mA2,msntau2,mu2,Q2)*(mA2*
     (3*msntau2 - 2*mu2) + 5*msntau2*(-msntau2 + mu2) + 2*mA2*mu2*DeltaInv(mA2,
     msntau2,mu2)*(-3*mA2*msntau2 - 2*mA2*mu2 - msntau2*mu2 + power2(mA2) +
     power2(mu2))))/(16.*msntau2*power2(mA2)) + (mw2*log(mu2/Q2)*log(mw2/Q2)*
     power2(mu2)*(-4*msntau2*mstau22*mu2 + mstau22*power2(msntau2) + msntau2*
     power2(mstau22) + msntau2*power2(mu2) + mstau22*power2(mu2) + mstau22*
     DeltaInv(msntau2,mw2,mu2)*power2(msntau2 - mu2)*(msntau2*(3*mu2 + mw2) -
     power2(mu2 - mw2)) + msntau2*DeltaInv(mstau22,mw2,mu2)*power2(mstau22 -
     mu2)*(mstau22*(3*mu2 + mw2) - power2(mu2 - mw2))))/(4.*msntau2*mstau22*
     power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (mz2*log(mu2/Q2)*log(mz2/
     Q2)*power2(mu2)*(-4*mstau12*mstau22*mu2 + mstau22*power2(mstau12) +
     mstau12*power2(mstau22) + mstau12*power2(mu2) + mstau22*power2(mu2) +
     mstau22*DeltaInv(mstau12,mz2,mu2)*power2(mstau12 - mu2)*(mstau12*(3*mu2 +
     mz2) - power2(mu2 - mz2)) + mstau12*DeltaInv(mstau22,mz2,mu2)*power2(
     mstau22 - mu2)*(mstau22*(3*mu2 + mz2) - power2(mu2 - mz2))))/(8.*mstau12*
     mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (Fin3(mH2,mstau12,
     mstau12,Q2)*(-1 + power2(sa)))/(2.*(mstau12 - mu2)) + (Fin3(mstau22,
     mstau22,mH2,Q2)*(-1 + power2(sa)))/(2.*(mstau22 - mu2)) - (Fin3(mH2,
     msntau2,mu2,Q2)*(3*msntau2*(msntau2 - mu2) - mH2*(5*msntau2 + 2*mu2) + 2*
     mH2*mu2*DeltaInv(mH2,msntau2,mu2)*(-3*mH2*msntau2 - 2*mH2*mu2 - msntau2*
     mu2 + power2(mH2) + power2(mu2)))*(-1 + power2(sa)))/(16.*msntau2*power2(
     mH2)) - (3*(mh2*(2*mstau12 + 2*mstau22 - mu2) - mH2*mu2)*power2(sa))/(4.*
     mh2*mH2) + (Fin3(mstau12,mstau12,mh2,Q2)*power2(sa))/(2.*(-mstau12 + mu2))
     + (Fin3(mstau22,mstau22,mh2,Q2)*power2(sa))/(2.*(-mstau22 + mu2)) + (3*(
     mstau12 + mstau22 - mu2)*log(mh2/Q2)*power2(sa))/(2.*(mh2 - mH2)) + ((3*
     mC2 + 4*mh2)*log(mC2/Q2)*log(mh2/Q2)*power2(sa))/(16.*mh2) - (3*(mstau12 +
     mstau22 - mu2)*log(mH2/Q2)*power2(sa))/(2.*(mh2 - mH2)) - ((3*mC2 + 4*mH2)
     *log(mC2/Q2)*log(mH2/Q2)*power2(sa))/(16.*mH2) + (3*mstau12*(mh2*(2*
     mstau12 - 2*mstau22 - mu2) - mH2*mu2)*log(mstau12/Q2)*power2(sa))/(4.*mh2*
     mH2*(mstau12 - mstau22)) - (3*mstau12*(mstau12 - mstau22 - mu2)*log(mh2/
     Q2)*log(mstau12/Q2)*power2(sa))/(2.*(mh2 - mH2)*(mstau12 - mstau22)) + (3*
     mstau12*(mstau12 - mstau22 - mu2)*log(mH2/Q2)*log(mstau12/Q2)*power2(sa))/
     (2.*(mh2 - mH2)*(mstau12 - mstau22)) + (3*mstau22*(mH2*mu2 + mh2*(2*
     mstau12 - 2*mstau22 + mu2))*log(mstau22/Q2)*power2(sa))/(4.*mh2*mH2*(
     mstau12 - mstau22)) - (3*mstau22*(mstau12 - mstau22 + mu2)*log(mh2/Q2)*
     log(mstau22/Q2)*power2(sa))/(2.*(mh2 - mH2)*(mstau12 - mstau22)) + (3*
     mstau22*(mstau12 - mstau22 + mu2)*log(mH2/Q2)*log(mstau22/Q2)*power2(sa))/
     (2.*(mh2 - mH2)*(mstau12 - mstau22)) + (log(mA2/Q2)*log(mh2/Q2)*(3*power2(
     mA2) - 5*power2(mh2))*power2(sa))/(16.*mA2*mh2) + ((3*mC2 - 5*mh2)*Fin20(
     mC2,mh2,Q2)*power2(sa))/(16.*power2(mh2)) - (log(mA2/Q2)*log(mH2/Q2)*(3*
     power2(mA2) - 5*power2(mH2))*power2(sa))/(16.*mA2*mH2) - ((3*mC2 - 5*mH2)*
     Fin20(mC2,mH2,Q2)*power2(sa))/(16.*power2(mH2)) + (3*mC2*(mh2 - mH2)*(-4*
     mh2*mH2 + 3*mC2*(mh2 + mH2))*log(mC2/Q2)*power2(sa))/(16.*power2(mh2)*
     power2(mH2)) + (3*(mh2 + mH2)*Fin20(mH2,mh2,Q2)*power2(mh2 - mH2)*power2(
     sa))/(16.*power2(mh2)*power2(mH2)) - (3*mu2*Fin20(mstau22,mstau12,Q2)*
     power2(mh2 - mH2)*power2(sa))/(4.*power2(mh2)*power2(mH2)) + (3*log(mh2/
     Q2)*log(mH2/Q2)*(8*mh2*mH2 + power2(mh2) + power2(mH2))*power2(sa))/(16.*
     mh2*mH2) - (Fin20(msntau2,mu2,Q2)*(-8*mh2*msntau2*power2(mH2) + power2(
     mh2)*(8*mH2*msntau2 - 3*power2(msntau2 - mu2)) + 3*power2(mH2)*power2(
     msntau2 - mu2))*power2(sa))/(16.*(msntau2 - mu2)*power2(mh2)*power2(mH2))
     - (Fin20(mstau12,mu2,Q2)*(8*mh2*(-2*mstau12 + mu2)*power2(mH2) + power2(
     mh2)*(8*mH2*(2*mstau12 - mu2) - 3*power2(mstau12 - mu2)) + 3*power2(mH2)*
     power2(mstau12 - mu2))*power2(sa))/(16.*(mstau12 - mu2)*power2(mh2)*
     power2(mH2)) - (Fin20(mstau22,mu2,Q2)*(8*mh2*(-2*mstau22 + mu2)*power2(
     mH2) + power2(mh2)*(8*mH2*(2*mstau22 - mu2) - 3*power2(mstau22 - mu2)) +
     3*power2(mH2)*power2(mstau22 - mu2))*power2(sa))/(16.*(mstau22 - mu2)*
     power2(mh2)*power2(mH2)) - (log(mh2/Q2)*log(msntau2/Q2)*(-4 - (3*msntau2)/
     mh2 + (4*mu2)/msntau2 - (4*mu2*DeltaInv(msntau2,mh2,mu2)*(mu2*(-2*msntau2
     + mu2) - 2*mh2*(msntau2 + mu2) + power2(mh2)))/msntau2 + (4*power2(mu2))/
     power2(msntau2 - mu2))*power2(sa))/16. + (log(mH2/Q2)*log(msntau2/Q2)*(-4
     - (3*msntau2)/mH2 + (4*mu2)/msntau2 - (4*mu2*DeltaInv(mH2,msntau2,mu2)*(
     mu2*(-2*msntau2 + mu2) - 2*mH2*(msntau2 + mu2) + power2(mH2)))/msntau2 + (
     4*power2(mu2))/power2(msntau2 - mu2))*power2(sa))/16. + (Fin3(msntau2,mh2,
     mu2,Q2)*(3*msntau2*(msntau2 - mu2) - mh2*(5*msntau2 + 2*mu2) + 2*mh2*mu2*
     DeltaInv(msntau2,mh2,mu2)*(-3*mh2*msntau2 - 2*mh2*mu2 - msntau2*mu2 +
     power2(mh2) + power2(mu2)))*power2(sa))/(16.*msntau2*power2(mh2)) + ((
     mstau12 - mu2)*mu2*(mA2 - mz2)*Fin20(mstau12,mu2,Q2)*power2(sb))/(2.*mA2*(
     mstau12 - mstau22)*(mstau22 - mu2)*mz2) + ((mstau12 + mstau22 - 2*mu2)*
     mu2*(mA2 - mz2)*Fin20(mstau22,mstau12,Q2)*power2(sb))/(2.*mA2*(mstau12 -
     mu2)*(-mstau22 + mu2)*mz2) - ((mstau22 - mu2)*mu2*(mA2 - mz2)*Fin20(
     mstau22,mu2,Q2)*power2(sb))/(2.*mA2*(mstau12 - mstau22)*(mstau12 - mu2)*
     mz2) - (mstau12*mu2*log(mA2/Q2)*log(mstau12/Q2)*power2(sb))/(2.*(mstau12 -
     mstau22)*(mstau12 - mu2)) + (mstau22*mu2*log(mA2/Q2)*log(mstau22/Q2)*
     power2(sb))/(2.*(mstau12 - mstau22)*(mstau22 - mu2)) + (mstau12*mu2*log(
     mstau12/Q2)*log(mz2/Q2)*power2(sb))/(2.*(mstau12 - mstau22)*(mstau12 -
     mu2)) - (mstau22*mu2*log(mstau22/Q2)*log(mz2/Q2)*power2(sb))/(2.*(mstau12
     - mstau22)*(mstau22 - mu2)) - ((mstau12 + mstau22 - 2*mu2)*mu2*Fin20(
     mstau22,mstau12,Q2)*power2(sb))/(2.*mA2*(mstau12 - mu2)*(-mstau22 + mu2)*
     power2(cb)) + ((mA2*(mstau12 - mu2) + mH2*(mstau22 - mu2))*mu2*Fin20(
     mstau22,mu2,Q2)*power2(sb))/(2.*mA2*mH2*(mstau12 - mstau22)*(mstau12 -
     mu2)*power2(cb)) + (mu2*(mA2*(-mstau12 + mu2) + mH2*(-mstau22 + mu2))*
     Fin20(mstau22,mu2,Q2)*power2(sb))/(2.*mA2*mH2*(mstau12 - mstau22)*(mstau12
     - mu2)*power2(cb)) + ((mC2*mH2 - mA2*(3*mC2 + mH2))*mstau12*mu2*log(
     mstau12/Q2)*power2(sb))/(4.*mA2*mC2*mH2*(mstau12 - mstau22)*power2(cb)) +
     ((-(mC2*mH2) + mA2*(3*mC2 + mH2))*mstau12*mu2*log(mstau12/Q2)*power2(sb))/
     (4.*mA2*mC2*mH2*(mstau12 - mstau22)*power2(cb)) - (mstau12*mu2*log(mA2/Q2)
     *log(mstau12/Q2)*power2(sb))/(2.*(mstau12 - mstau22)*(mstau12 - mu2)*
     power2(cb)) + (mstau12*mu2*log(mH2/Q2)*log(mstau12/Q2)*power2(sb))/(2.*(
     mstau12 - mstau22)*(mstau12 - mu2)*power2(cb)) + (mstau22*mu2*log(mA2/Q2)*
     log(mstau22/Q2)*power2(sb))/(2.*(mstau12 - mstau22)*(mstau22 - mu2)*
     power2(cb)) - (mstau22*mu2*log(mH2/Q2)*log(mstau22/Q2)*power2(sb))/(2.*(
     mstau12 - mstau22)*(mstau22 - mu2)*power2(cb)) - (mu2*Fin20(mstau22,
     msntau2,Q2)*power2(sb))/(4.*power2(cb)*power2(mC2)) + (mu2*Fin20(mstau22,
     mstau12,Q2)*(3*(mstau12 - mu2)*(mstau22 - mu2)*power2(mA2) + 2*mA2*(
     mstau12 + mstau22 - 2*mu2)*power2(mH2) + (mstau12 - mu2)*(-mstau22 + mu2)*
     power2(mH2))*power2(sb))/(4.*(mstau12 - mu2)*(-mstau22 + mu2)*power2(cb)*
     power2(mA2)*power2(mH2)) + (log(mA2/Q2)*log(mu2/Q2)*power2(mu2)*power2(sb)
     )/(2.*(mstau12 - mu2)*(-mstau22 + mu2)) - (log(mu2/Q2)*log(mz2/Q2)*power2(
     mu2)*power2(sb))/(2.*(mstau12 - mu2)*(-mstau22 + mu2)) + (log(mA2/Q2)*log(
     mu2/Q2)*power2(mu2)*power2(sb))/(2.*(mstau12 - mu2)*(-mstau22 + mu2)*
     power2(cb)) - (log(mH2/Q2)*log(mu2/Q2)*power2(mu2)*power2(sb))/(2.*(
     mstau12 - mu2)*(-mstau22 + mu2)*power2(cb)) - (mC2*log(mC2/Q2)*log(mu2/Q2)
     *power2(mu2)*(4*msntau2*mstau22*mu2 - mstau22*power2(msntau2) - msntau2*
     power2(mstau22) + mstau22*DeltaInv(mC2,msntau2,mu2)*(mu2*(-3*msntau2 +
     mu2) - mC2*(msntau2 + 2*mu2) + power2(mC2))*power2(msntau2 - mu2) +
     msntau2*DeltaInv(mstau22,mC2,mu2)*(mu2*(-3*mstau22 + mu2) - mC2*(mstau22 +
     2*mu2) + power2(mC2))*power2(mstau22 - mu2) - msntau2*power2(mu2) -
     mstau22*power2(mu2))*power2(sb))/(4.*msntau2*mstau22*power2(cb)*power2(
     msntau2 - mu2)*power2(mstau22 - mu2)) + (log(mH2/Q2)*log(mu2/Q2)*power2(
     mu2)*(-4*mH2*mstau12*mstau22*mu2 + mH2*mstau22*power2(mstau12) + 4*
     mstau22*mu2*power2(mstau12) + mH2*mstau12*power2(mstau22) + 4*mstau12*mu2*
     power2(mstau22) - 4*power2(mstau12)*power2(mstau22) - mH2*mstau22*
     DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mH2*(mstau12 + 2*mu2)
     + power2(mH2))*power2(mstau12 - mu2) - mH2*mstau12*DeltaInv(mstau22,mH2,
     mu2)*(mu2*(-3*mstau22 + mu2) - mH2*(mstau22 + 2*mu2) + power2(mH2))*
     power2(mstau22 - mu2) + mH2*mstau12*power2(mu2) + mH2*mstau22*power2(mu2)
     - 4*mstau12*mstau22*power2(mu2))*power2(sb))/(8.*mstau12*mstau22*power2(
     cb)*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (log(mA2/Q2)*log(mu2/
     Q2)*power2(mu2)*(-4*mA2*mstau12*mstau22*mu2 + mA2*mstau22*power2(mstau12)
     - 4*mstau22*mu2*power2(mstau12) + mA2*mstau12*power2(mstau22) - 4*mstau12*
     mu2*power2(mstau22) + 4*power2(mstau12)*power2(mstau22) - mA2*mstau22*
     DeltaInv(mA2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mA2*(mstau12 + 2*mu2)
     + power2(mA2))*power2(mstau12 - mu2) - mA2*mstau12*DeltaInv(mA2,mstau22,
     mu2)*(mu2*(-3*mstau22 + mu2) - mA2*(mstau22 + 2*mu2) + power2(mA2))*
     power2(mstau22 - mu2) + mA2*mstau12*power2(mu2) + mA2*mstau22*power2(mu2)
     + 4*mstau12*mstau22*power2(mu2))*power2(sb))/(8.*mstau12*mstau22*power2(
     cb)*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (mstau12*mu2*log(mh2/
     Q2)*log(mstau12/Q2)*power2(sa)*power2(sb))/(2.*(mstau12 - mstau22)*(
     mstau12 - mu2)*power2(cb)) - (mstau12*mu2*log(mH2/Q2)*log(mstau12/Q2)*
     power2(sa)*power2(sb))/(2.*(mstau12 - mstau22)*(mstau12 - mu2)*power2(cb))
     - (mstau22*mu2*log(mh2/Q2)*log(mstau22/Q2)*power2(sa)*power2(sb))/(2.*(
     mstau12 - mstau22)*(mstau22 - mu2)*power2(cb)) + (mstau22*mu2*log(mH2/Q2)*
     log(mstau22/Q2)*power2(sa)*power2(sb))/(2.*(mstau12 - mstau22)*(mstau22 -
     mu2)*power2(cb)) + (3*(mh2 - mH2)*mu2*Fin20(mstau22,mstau12,Q2)*power2(sa)
     *power2(sb))/(2.*mh2*power2(cb)*power2(mH2)) - (log(mh2/Q2)*log(mu2/Q2)*
     power2(mu2)*power2(sa)*power2(sb))/(2.*(mstau12 - mu2)*(-mstau22 + mu2)*
     power2(cb)) + (log(mH2/Q2)*log(mu2/Q2)*power2(mu2)*power2(sa)*power2(sb))/
     (2.*(mstau12 - mu2)*(-mstau22 + mu2)*power2(cb)) + (log(mh2/Q2)*log(mu2/
     Q2)*power2(mu2)*(-4*mh2*mstau12*mstau22*mu2 + mh2*mstau22*power2(mstau12)
     + 4*mstau22*mu2*power2(mstau12) + mh2*mstau12*power2(mstau22) + 4*mstau12*
     mu2*power2(mstau22) - 4*power2(mstau12)*power2(mstau22) - mh2*mstau22*
     DeltaInv(mstau12,mh2,mu2)*(mu2*(-3*mstau12 + mu2) - mh2*(mstau12 + 2*mu2)
     + power2(mh2))*power2(mstau12 - mu2) - mh2*mstau12*DeltaInv(mstau22,mh2,
     mu2)*(mu2*(-3*mstau22 + mu2) - mh2*(mstau22 + 2*mu2) + power2(mh2))*
     power2(mstau22 - mu2) + mh2*mstau12*power2(mu2) + mh2*mstau22*power2(mu2)
     - 4*mstau12*mstau22*power2(mu2))*power2(sa)*power2(sb))/(8.*mstau12*
     mstau22*power2(cb)*power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (log(
     mH2/Q2)*log(mu2/Q2)*power2(mu2)*(-4*mH2*mstau12*mstau22*mu2 + mH2*mstau22*
     power2(mstau12) + 4*mstau22*mu2*power2(mstau12) + mH2*mstau12*power2(
     mstau22) + 4*mstau12*mu2*power2(mstau22) - 4*power2(mstau12)*power2(
     mstau22) - mH2*mstau22*DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) -
     mH2*(mstau12 + 2*mu2) + power2(mH2))*power2(mstau12 - mu2) - mH2*mstau12*
     DeltaInv(mstau22,mH2,mu2)*(mu2*(-3*mstau22 + mu2) - mH2*(mstau22 + 2*mu2)
     + power2(mH2))*power2(mstau22 - mu2) + mH2*mstau12*power2(mu2) + mH2*
     mstau22*power2(mu2) - 4*mstau12*mstau22*power2(mu2))*power2(sa)*power2(sb)
     )/(8.*mstau12*mstau22*power2(cb)*power2(mstau12 - mu2)*power2(mstau22 -
     mu2)) + (mC2*(mh2 - mH2)*(13*mh2*mH2 - 6*mC2*(mh2 + mH2))*power2(sa)*
     power2(log(mC2/Q2)))/(32.*power2(mh2)*power2(mH2)) - ((mh2 - mH2)*power2(
     sa)*power2(log(mA2/Q2))*(15*mh2*mH2*power2(mA2) + 5*power2(mh2)*power2(
     mH2) + 6*(mh2 + mH2)*power3(mA2)))/(32.*mA2*power2(mh2)*power2(mH2)) + ((
     mh2 - mH2)*log(mA2/Q2)*power2(sa)*(30*mh2*mH2*power2(mA2) + 10*power2(mh2)
     *power2(mH2) + 9*(mh2 + mH2)*power3(mA2)))/(16.*mA2*power2(mh2)*power2(
     mH2)) - (log(msntau2/Q2)*log(mu2/Q2)*power2(mu2)*power2(sb)*(mC2 + msntau2
     + mu2 + DeltaInv(mC2,msntau2,mu2)*(mC2*mu2*(6*msntau2 + mu2) + (msntau2 +
     mu2)*power2(mC2) + (msntau2 - mu2)*power2(mu2) - power3(mC2))))/(4.*
     msntau2*power2(cb)*power2(mstau22 - mu2)) + (power2(log(mC2/Q2))*(11*mA2*
     mstau22*power2(mC2)*power2(mH2) + 4*mC2*mu2*DeltaInv(mstau22,mC2,mu2)*
     power2(mA2)*power2(mH2)*(-3*mC2*mstau22 - 2*mC2*mu2 - mstau22*mu2 +
     power2(mC2) + power2(mu2)) - 10*mstau22*power2(mH2)*power3(mC2) + power2(
     mA2)*(-13*mH2*mstau22*power2(mC2) + mC2*(7*mstau22 - 4*mu2)*power2(mH2) +
     2*mstau22*(-mstau22 + mu2)*power2(mH2) + 6*mstau22*power3(mC2))))/(32.*
     mC2*mstau22*power2(mA2)*power2(mH2)) + (Fin20(mA2,mh2,Q2)*power2(sa)*(9*
     mh2*power2(mA2) + mA2*power2(mh2) + 3*power3(mA2) - 5*power3(mh2)))/(16.*
     power2(mA2)*power2(mh2)) - (3*ca*mu2*sa*sb*log(mh2/Q2)*(2/(-mh2 + mH2) +
     mh2/(mstau12*mstau22 - mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2)
     + (3*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 -
     mu2) - (mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*
     power2(mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))))/(4.*
     cb) - (3*mu2*log(mh2/Q2)*power2(sa)*power2(sb)*(4/(-mh2 + mH2) + mh2/(
     mstau12*mstau22 - mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + (3*
     mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 -
     mu2) - (mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*
     power2(mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))))/(8.*
     power2(cb)) + (ca*mu2*sa*sb*power2(log(mh2/Q2))*(6/(-mh2 + mH2) + mh2/(
     mstau12*mstau22 - mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + (3*
     mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 -
     mu2) - (mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*
     power2(mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))))/(8.*
     cb) + (mu2*power2(sa)*power2(sb)*power2(log(mh2/Q2))*(12/(-mh2 + mH2) +
     mh2/(mstau12*mstau22 - mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2)
     + (3*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 -
     mu2) - (mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*
     power2(mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))))/(16.*
     power2(cb)) - (log(mstau12/Q2)*log(mu2/Q2)*power2(mu2)*power2(sb)*(mA2 +
     mH2 + 2*mstau12 + 2*mu2 + DeltaInv(mA2,mstau12,mu2)*(mA2*mu2*(6*mstau12 +
     mu2) + (mstau12 + mu2)*power2(mA2) + (mstau12 - mu2)*power2(mu2) - power3(
     mA2)) + DeltaInv(mH2,mstau12,mu2)*(mH2*mu2*(6*mstau12 + mu2) + (mstau12 +
     mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2) - power3(mH2))))/(8.*
     mstau12*power2(cb)*power2(mstau22 - mu2)) + (ca*sa*sb*log(mstau12/Q2)*log(
     mu2/Q2)*power2(mu2)*(-mh2 + mH2 + DeltaInv(mstau12,mh2,mu2)*(-(mh2*mu2*(6*
     mstau12 + mu2)) - (mstau12 + mu2)*power2(mh2) + (-mstau12 + mu2)*power2(
     mu2) + power3(mh2)) + DeltaInv(mH2,mstau12,mu2)*(mH2*mu2*(6*mstau12 + mu2)
     + (mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2) - power3(mH2))
     ))/(4.*cb*mstau12*power2(mstau22 - mu2)) + (log(mstau12/Q2)*log(mu2/Q2)*
     power2(mu2)*power2(sa)*power2(sb)*(-mh2 + mH2 + DeltaInv(mstau12,mh2,mu2)*
     (-(mh2*mu2*(6*mstau12 + mu2)) - (mstau12 + mu2)*power2(mh2) + (-mstau12 +
     mu2)*power2(mu2) + power3(mh2)) + DeltaInv(mH2,mstau12,mu2)*(mH2*mu2*(6*
     mstau12 + mu2) + (mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2)
     - power3(mH2))))/(8.*mstau12*power2(cb)*power2(mstau22 - mu2)) + (ca*sa*
     sb*log(mstau22/Q2)*log(mu2/Q2)*power2(mu2)*(-mh2 + mH2 + DeltaInv(mstau22,
     mh2,mu2)*(-(mh2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*power2(mh2) + (-
     mstau22 + mu2)*power2(mu2) + power3(mh2)) + DeltaInv(mstau22,mH2,mu2)*(
     mH2*mu2*(6*mstau22 + mu2) + (mstau22 + mu2)*power2(mH2) + (mstau22 - mu2)*
     power2(mu2) - power3(mH2))))/(4.*cb*mstau22*power2(mstau12 - mu2)) + (log(
     mstau22/Q2)*log(mu2/Q2)*power2(mu2)*power2(sa)*power2(sb)*(-mh2 + mH2 +
     DeltaInv(mstau22,mh2,mu2)*(-(mh2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*
     power2(mh2) + (-mstau22 + mu2)*power2(mu2) + power3(mh2)) + DeltaInv(
     mstau22,mH2,mu2)*(mH2*mu2*(6*mstau22 + mu2) + (mstau22 + mu2)*power2(mH2)
     + (mstau22 - mu2)*power2(mu2) - power3(mH2))))/(8.*mstau22*power2(cb)*
     power2(mstau12 - mu2)) + (Fin20(mA2,mH2,Q2)*(9*mH2*power2(mA2) + mA2*
     power2(mH2) + 3*power3(mA2) - 5*power3(mH2)))/(16.*power2(mA2)*power2(mH2)
     ) - (Fin20(mA2,mH2,Q2)*power2(sa)*(9*mH2*power2(mA2) + mA2*power2(mH2) +
     3*power3(mA2) - 5*power3(mH2)))/(16.*power2(mA2)*power2(mH2)) + (mu2*log(
     msntau2/Q2)*log(mu2/Q2)*power2(sa)*(-((mh2 - mH2)*(3*power2(msntau2) +
     power2(mu2))) + mH2*DeltaInv(msntau2,mh2,mu2)*(-(power2(msntau2 - mu2)*
     power2(mu2)) + mh2*mu2*(5*msntau2*mu2 - 6*power2(msntau2) + power2(mu2)) +
     power2(mh2)*(-power2(msntau2) + power2(mu2)) + (msntau2 - mu2)*power3(mh2)
     ) + mh2*DeltaInv(mH2,msntau2,mu2)*(power2(mH2)*(power2(msntau2) - power2(
     mu2)) + power2(msntau2 - mu2)*power2(mu2) - mH2*mu2*(5*msntau2*mu2 - 6*
     power2(msntau2) + power2(mu2)) + (-msntau2 + mu2)*power3(mH2))))/(8.*mh2*
     mH2*msntau2*(msntau2 - mu2)) - (ca*mu2*sa*sb*log(mh2/Q2)*log(mstau22/Q2)*(
     -2/(mstau12 - mstau22) + (2*mu2)/((mstau12 - mstau22)*(-mstau22 + mu2)) +
     mh2/(mstau12*mstau22 - mstau12*mu2) + (2*mh2*mu2)/(mstau22*power2(mstau12
     - mu2)) - (2*mh2*mu2*DeltaInv(mstau22,mh2,mu2)*(mu2*(-2*mstau22 + mu2) -
     2*mh2*(mstau22 + mu2) + power2(mh2)))/(mstau22*power2(mstau12 - mu2)) - (
     mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (mh2*DeltaInv(mstau22,mstau12,
     mh2)*(-(mh2*(mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mstau12 - mu2))
     + (mstau22 - 2*mu2)*power2(mh2)*power2(mstau12 - mu2) + mstau24*(mstau22 -
     2*mu2)*power2(mu2) - 3*mstau22*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*mu2*(-2*mstau22*mstau24 + 4*mstau24*mu2 + 5*mu2*power2(mstau22) +
     2*mstau22*power2(mu2) - 4*power3(mstau22)) + power2(mstau12)*(mstau22*
     mstau24 - 2*mstau24*mu2 + 2*mu2*power2(mstau22) - 10*mstau22*power2(mu2) +
     2*power3(mstau22))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))
     ))/(4.*cb) - (mu2*log(mh2/Q2)*log(mstau22/Q2)*power2(sa)*power2(sb)*(-4/(
     mstau12 - mstau22) + (4*mu2)/((mstau12 - mstau22)*(-mstau22 + mu2)) + mh2/
     (mstau12*mstau22 - mstau12*mu2) + (2*mh2*mu2)/(mstau22*power2(mstau12 -
     mu2)) - (2*mh2*mu2*DeltaInv(mstau22,mh2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*
     mh2*(mstau22 + mu2) + power2(mh2)))/(mstau22*power2(mstau12 - mu2)) - (
     mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (mh2*DeltaInv(mstau22,mstau12,
     mh2)*(-(mh2*(mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mstau12 - mu2))
     + (mstau22 - 2*mu2)*power2(mh2)*power2(mstau12 - mu2) + mstau24*(mstau22 -
     2*mu2)*power2(mu2) - 3*mstau22*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*mu2*(-2*mstau22*mstau24 + 4*mstau24*mu2 + 5*mu2*power2(mstau22) +
     2*mstau22*power2(mu2) - 4*power3(mstau22)) + power2(mstau12)*(mstau22*
     mstau24 - 2*mstau24*mu2 + 2*mu2*power2(mstau22) - 10*mstau22*power2(mu2) +
     2*power3(mstau22))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))
     ))/(8.*power2(cb)) - (log(mC2/Q2)*log(msntau2/Q2)*(-3 + ((-8*mC2 + 3*
     msntau2 - 3*mu2)*power2(mu2))/power3(msntau2 - mu2)))/16. + (mu2*Fin3(
     mstau22,mw2,mu2,Q2)*(-(msntau2*(mstau22 + mu2)) + mu2*(3*mstau22 + mu2) +
     (msntau2 - mu2)*mu2*DeltaInv(mstau22,mw2,mu2)*(-(mstau22*(mu2 + 3*mw2)) +
     power2(mu2 - mw2))))/(4.*mstau22*power3(msntau2 - mu2)) + (Fin3(mA2,
     mstau22,mu2,Q2)*((-5*(mstau22 - mu2))/power2(mA2) + (2*mu2*DeltaInv(mA2,
     mstau22,mu2)*(-3*mA2*mstau22 - 2*mA2*mu2 - mstau22*mu2 + power2(mA2) +
     power2(mu2))*(power2(cb)*power2(mstau12 - mu2) + mA2*mu2*power2(sb)))/(
     mA2*mstau22*power2(cb)*power2(mstau12 - mu2)) + (3 - (2*mu2)/mstau22 + (8*
     mstau22*mu2*power2(sb))/((mstau12 - mstau22)*(-mstau12 + mu2)) + (8*
     power2(mu2)*power2(sb))/((mstau12 - mstau22)*(mstau12 - mu2)))/mA2 + (2*
     mu2*((-(mstau12*(mstau22 + mu2)) + mu2*(3*mstau22 + mu2))/(mstau22*power2(
     cb)) + (4*power2(mstau12 - mu2))/(mstau12 - mstau22))*power2(sb))/power3(
     mstau12 - mu2)))/16. - (mu2*Fin3(mstau22,mz2,mu2,Q2)*((mstau12 - mstau22)*
     (mstau12 - mu2)*mu2*mz2*DeltaInv(mstau22,mz2,mu2)*(mstau22*(mu2 + 3*mz2) -
     power2(mu2 - mz2)) - mstau12*(mz2*power2(mu2) + power2(mstau22)*(mz2 - 8*
     mu2*power2(sb)) + 4*mstau22*mu2*(mz2 + 2*mu2*power2(sb) + 2*mz2*power2(sb)
     )) + power2(mstau12)*(mu2*mz2 - 4*power2(mstau22)*power2(sb) + mstau22*(
     mz2 + 4*mu2*power2(sb) + 4*mz2*power2(sb))) + mstau22*mu2*(mstau22*(3*mz2
     - 4*mu2*power2(sb)) + mu2*(mz2 + 4*mu2*power2(sb) + 4*mz2*power2(sb)))))/(
     8.*(mstau12 - mstau22)*mstau22*mz2*power3(mstau12 - mu2)) + (Fin3(mA2,
     mstau12,mu2,Q2)*((-5*(mstau12 - mu2))/power2(mA2) + (2*mu2*DeltaInv(mA2,
     mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2 + power2(mA2) +
     power2(mu2))*(power2(cb)*power2(mstau22 - mu2) + mA2*mu2*power2(sb)))/(
     mA2*mstau12*power2(cb)*power2(mstau22 - mu2)) + (3 - (2*mu2)/mstau12 + (8*
     mstau12*mu2*power2(sb))/((mstau12 - mstau22)*(mstau22 - mu2)) + (8*power2(
     mu2)*power2(sb))/((mstau12 - mstau22)*(-mstau22 + mu2)))/mA2 + 2*mu2*
     power2(sb)*(4/((mstau12 - mstau22)*(-mstau22 + mu2)) + (-(mstau12*mstau22)
     + 3*mstau12*mu2 - mstau22*mu2 + power2(mu2))/(mstau12*power2(cb)*power3(
     mstau22 - mu2)))))/16. - (mu2*Fin3(mstau22,mC2,msntau2,Q2)*power2(sb)*(-(
     1/(mC2*msntau2)) + 1/(msntau2*(-mstau22 + mu2)) - 1/power2(mC2) - 2/
     power2(msntau2 - mu2) - 1/power2(mstau22 - mu2) + mu2/(msntau2*power2(
     mstau22 - mu2)) + DeltaInv(mstau22,mC2,msntau2)*(2 + (-3*msntau2*mstau22 +
     mstau24)/(mC2*msntau2) + (2*mu2)/(msntau2 - mu2) + (3*mstau22)/(-msntau2 +
     mu2) + (2*mstau22)/(-mstau22 + mu2) + mstau24/(msntau2*mstau22 - msntau2*
     mu2) + mstau24/power2(msntau2 - mu2) - (mstau22*mu2)/power2(msntau2 - mu2)
     + mC2*(3/(-msntau2 + mu2) + 3/(-mstau22 + mu2) - (2*mstau22)/(msntau2*
     mstau22 - msntau2*mu2) - (2*mstau22)/power2(msntau2 - mu2) - mu2/power2(
     msntau2 - mu2) + (1 + (2*mstau22*mu2)/power2(mstau22 - mu2))/msntau2 + (3*
     mu2)/power2(mstau22 - mu2)) + power2(mC2)*(1/(msntau2*mstau22 - msntau2*
     mu2) + 1/power2(msntau2 - mu2) - mu2/(msntau2*power2(mstau22 - mu2))) - (
     2*mstau22 + (mstau24*mu2)/power2(mstau22 - mu2))/msntau2 + (2*mstau22*mu2)
     /power2(mstau22 - mu2) - power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/
     power3(msntau2 - mu2) + (2*mu2)/power3(mstau22 - mu2)))/(4.*power2(cb)) +
     (mu2*Fin3(msntau2,mw2,mu2,Q2)*(-(msntau2*mstau22) + 3*msntau2*mu2 -
     mstau22*mu2 + power2(mu2) + (mstau22 - mu2)*mu2*DeltaInv(msntau2,mw2,mu2)*
     (-(msntau2*(mu2 + 3*mw2)) + power2(mu2 - mw2))))/(4.*msntau2*power3(
     mstau22 - mu2)) - (mu2*Fin3(mC2,msntau2,mu2,Q2)*(msntau2*(mstau22 - 3*mu2)
     + (mstau22 - mu2)*mu2 - (mstau22 - mu2)*mu2*DeltaInv(mC2,msntau2,mu2)*(-3*
     mC2*msntau2 - 2*mC2*mu2 - msntau2*mu2 + power2(mC2) + power2(mu2)))*
     power2(sb))/(4.*msntau2*power2(cb)*power3(mstau22 - mu2)) + (mu2*Fin3(
     mstau12,mz2,mu2,Q2)*(mstau22*(mstau22 - mu2)*mu2*mz2 - (mstau12 - mstau22)
     *(mstau22 - mu2)*mu2*mz2*DeltaInv(mstau12,mz2,mu2)*(mstau12*(mu2 + 3*mz2)
     - power2(mu2 - mz2)) - power2(mstau12)*(4*power2(mstau22)*power2(sb) +
     mstau22*(mz2 - 8*mu2*power2(sb)) + mu2*(-3*mz2 + 4*mu2*power2(sb))) +
     mstau12*(-4*mstau22*mu2*(mz2 + 2*mu2*power2(sb) + 2*mz2*power2(sb)) +
     power2(mstau22)*(mz2 + 4*mu2*power2(sb) + 4*mz2*power2(sb)) + power2(mu2)*
     (mz2 + 4*mu2*power2(sb) + 4*mz2*power2(sb)))))/(8.*mstau12*(mstau12 -
     mstau22)*mz2*power3(mstau22 - mu2)) + (mu2*mw2*log(msntau2/Q2)*log(mw2/Q2)
     *(-6*msntau2*mstau22*mu2 + 2*mstau22*power2(msntau2) - 6*mu2*power2(
     msntau2) + msntau2*power2(mstau22) + 13*msntau2*power2(mu2) + 2*mstau22*
     power2(mu2) - 2*mu2*DeltaInv(msntau2,mw2,mu2)*power2(msntau2 - mu2)*(2*
     msntau2*(mu2 + mw2) - power2(mu2 - mw2)) + DeltaInv(mstau22,msntau2,mw2)*(
     2*(mstau22 - 2*mu2)*(-mstau24 + (2*mstau22 - mw2)*mw2)*power2(mu2) + 2*(-
     2*mstau22*(mu2 - mw2) - mu2*(mu2 + 4*mw2) + power2(mstau22))*power3(
     msntau2) + power2(msntau2)*((-8*mu2 + 7*mw2)*power2(mstau22) - mstau22*(2*
     mstau24 + 22*mu2*mw2 - 13*power2(mu2) + 2*power2(mw2)) + mu2*(4*mstau24 +
     19*mu2*mw2 + 2*power2(mu2) + 4*power2(mw2)) + power3(mstau22)) + msntau2*(
     -(power2(mstau22)*(mstau24 + 14*mu2*mw2 + power2(mw2))) + 2*mstau22*mu2*(
     3*mstau24 + 13*mu2*mw2 - 3*power2(mu2) + 3*power2(mw2)) - power2(mu2)*(9*
     mstau24 + 10*mu2*mw2 + 9*power2(mw2)) + 2*(mu2 + mw2)*power3(mstau22))) -
     6*power3(mu2)))/(4.*msntau2*power2(msntau2 - mu2)*power2(mstau22 - mu2)) +
     (mu2*mz2*log(mstau12/Q2)*log(mz2/Q2)*(-6*mstau12*mstau22*mu2 + 2*mstau22*
     power2(mstau12) - 6*mu2*power2(mstau12) + mstau12*power2(mstau22) + 13*
     mstau12*power2(mu2) + 2*mstau22*power2(mu2) - 2*mu2*DeltaInv(mstau12,mz2,
     mu2)*power2(mstau12 - mu2)*(2*mstau12*(mu2 + mz2) - power2(mu2 - mz2)) +
     DeltaInv(mstau22,mstau12,mz2)*(2*(mstau22 - 2*mu2)*(-mstau24 + (2*mstau22
     - mz2)*mz2)*power2(mu2) + 2*(-2*mstau22*(mu2 - mz2) - mu2*(mu2 + 4*mz2) +
     power2(mstau22))*power3(mstau12) + power2(mstau12)*((-8*mu2 + 7*mz2)*
     power2(mstau22) - mstau22*(2*mstau24 + 22*mu2*mz2 - 13*power2(mu2) + 2*
     power2(mz2)) + mu2*(4*mstau24 + 19*mu2*mz2 + 2*power2(mu2) + 4*power2(mz2)
     ) + power3(mstau22)) + mstau12*(-(power2(mstau22)*(mstau24 + 14*mu2*mz2 +
     power2(mz2))) + 2*mstau22*mu2*(3*mstau24 + 13*mu2*mz2 - 3*power2(mu2) + 3*
     power2(mz2)) - power2(mu2)*(9*mstau24 + 10*mu2*mz2 + 9*power2(mz2)) + 2*(
     mu2 + mz2)*power3(mstau22))) - 6*power3(mu2)))/(8.*mstau12*power2(mstau12
     - mu2)*power2(mstau22 - mu2)) + (3*ca*sa*sb*log(mu2/Q2)*(mstau22*DeltaInv(
     mstau12,mh2,mu2)*((mstau12 - mu2)*mu2 + mh2*(3*mstau12 + 2*mu2) - power2(
     mh2))*power2(mstau12 - mu2) + mstau22*DeltaInv(mH2,mstau12,mu2)*power2(
     mstau12 - mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)) + mstau12*power2(mstau22 - mu2)*(-(DeltaInv(mstau22,mh2,mu2)*
     (-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 + power2(mh2) + power2(mu2))) +
     DeltaInv(mstau22,mH2,mu2)*(-3*mH2*mstau22 - 2*mH2*mu2 - mstau22*mu2 +
     power2(mH2) + power2(mu2))))*power3(mu2))/(4.*cb*mstau12*mstau22*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (3*log(mu2/Q2)*(mstau22*DeltaInv(
     mstau12,mh2,mu2)*((mstau12 - mu2)*mu2 + mh2*(3*mstau12 + 2*mu2) - power2(
     mh2))*power2(mstau12 - mu2) + mstau22*DeltaInv(mH2,mstau12,mu2)*power2(
     mstau12 - mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)) + mstau12*power2(mstau22 - mu2)*(-(DeltaInv(mstau22,mh2,mu2)*
     (-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 + power2(mh2) + power2(mu2))) +
     DeltaInv(mstau22,mH2,mu2)*(-3*mH2*mstau22 - 2*mH2*mu2 - mstau22*mu2 +
     power2(mH2) + power2(mu2))))*power2(sa)*power2(sb)*power3(mu2))/(8.*
     mstau12*mstau22*power2(cb)*power2(mstau12 - mu2)*power2(mstau22 - mu2)) +
     (ca*sa*sb*(mstau22*DeltaInv(mH2,mstau12,mu2)*((mstau12 - mu2)*mu2 + mH2*(
     3*mstau12 + 2*mu2) - power2(mH2))*power2(mstau12 - mu2) + mstau22*
     DeltaInv(mstau12,mh2,mu2)*power2(mstau12 - mu2)*(-3*mh2*mstau12 - 2*mh2*
     mu2 - mstau12*mu2 + power2(mh2) + power2(mu2)) + mstau12*power2(mstau22 -
     mu2)*(DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2
     + power2(mh2) + power2(mu2)) - DeltaInv(mstau22,mH2,mu2)*(-3*mH2*mstau22 -
     2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(mu2))))*power2(log(mu2/Q2))
     *power3(mu2))/(8.*cb*mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22
     - mu2)) + ((mstau22*DeltaInv(mH2,mstau12,mu2)*((mstau12 - mu2)*mu2 + mH2*(
     3*mstau12 + 2*mu2) - power2(mH2))*power2(mstau12 - mu2) + mstau22*
     DeltaInv(mstau12,mh2,mu2)*power2(mstau12 - mu2)*(-3*mh2*mstau12 - 2*mh2*
     mu2 - mstau12*mu2 + power2(mh2) + power2(mu2)) + mstau12*power2(mstau22 -
     mu2)*(DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2
     + power2(mh2) + power2(mu2)) - DeltaInv(mstau22,mH2,mu2)*(-3*mH2*mstau22 -
     2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(mu2))))*power2(sa)*power2(
     sb)*power2(log(mu2/Q2))*power3(mu2))/(16.*mstau12*mstau22*power2(cb)*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (mu2*mw2*log(mstau22/Q2)*
     log(mw2/Q2)*(2*mstau22*mu2*power2(msntau2) - power2(msntau2)*power2(
     mstau22) - power2(mstau22)*power2(mu2) - 2*msntau2*mu2*DeltaInv(mstau22,
     mw2,mu2)*power2(mstau22 - mu2)*(2*mstau22*(mu2 + mw2) - power2(mu2 - mw2))
     + mstau22*DeltaInv(mstau22,msntau2,mw2)*(-((mstau22 - 2*mu2)*(-mstau24 + (
     2*mstau22 - mw2)*mw2)*power2(mu2)) - (mstau22 - 2*mu2)*(3*mstau22 + mw2)*
     power3(msntau2) + msntau2*mu2*(2*mu2*(2*mstau24 + mw2*(mu2 + 2*mw2)) + (5*
     mu2 + 4*mw2)*power2(mstau22) - mstau22*(2*mstau24 + 9*mu2*mw2 - 2*power2(
     mu2) + 2*power2(mw2)) - 4*power3(mstau22)) + power2(msntau2)*(2*(mu2 -
     mw2)*power2(mstau22) - 2*mu2*(mstau24 + 2*mu2*mw2 + power2(mw2)) +
     mstau22*(mstau24 + 6*mu2*mw2 - 10*power2(mu2) + power2(mw2)) + 2*power3(
     mstau22))) - 2*msntau2*power3(mu2) + 2*mstau22*power3(mu2)))/(4.*msntau2*
     mstau22*power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (mh2*mu2*log(mh2/
     Q2)*log(mstau22/Q2)*(2*mstau22*mu2*power2(mstau12) - power2(mstau12)*
     power2(mstau22) + 2*mstau12*mu2*DeltaInv(mstau22,mh2,mu2)*(mu2*(-2*mstau22
     + mu2) - 2*mh2*(mstau22 + mu2) + power2(mh2))*power2(mstau22 - mu2) -
     power2(mstau22)*power2(mu2) + mstau22*DeltaInv(mstau22,mstau12,mh2)*(-(
     mh2*(mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mstau12 - mu2)) + (
     mstau22 - 2*mu2)*power2(mh2)*power2(mstau12 - mu2) + mstau24*(mstau22 - 2*
     mu2)*power2(mu2) - 3*mstau22*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*
     mu2*(-2*mstau22*mstau24 + 4*mstau24*mu2 + 5*mu2*power2(mstau22) + 2*
     mstau22*power2(mu2) - 4*power3(mstau22)) + power2(mstau12)*(mstau22*
     mstau24 - 2*mstau24*mu2 + 2*mu2*power2(mstau22) - 10*mstau22*power2(mu2) +
     2*power3(mstau22))) - 2*mstau12*power3(mu2) + 2*mstau22*power3(mu2)))/(8.*
     mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (mu2*mz2*
     log(mstau22/Q2)*log(mz2/Q2)*(2*mstau22*mu2*power2(mstau12) - power2(
     mstau12)*power2(mstau22) - power2(mstau22)*power2(mu2) - 2*mstau12*mu2*
     DeltaInv(mstau22,mz2,mu2)*power2(mstau22 - mu2)*(2*mstau22*(mu2 + mz2) -
     power2(mu2 - mz2)) + mstau22*DeltaInv(mstau22,mstau12,mz2)*(-((mstau22 -
     2*mu2)*(-mstau24 + (2*mstau22 - mz2)*mz2)*power2(mu2)) - (mstau22 - 2*mu2)
     *(3*mstau22 + mz2)*power3(mstau12) + mstau12*mu2*(2*mu2*(2*mstau24 + mz2*(
     mu2 + 2*mz2)) + (5*mu2 + 4*mz2)*power2(mstau22) - mstau22*(2*mstau24 + 9*
     mu2*mz2 - 2*power2(mu2) + 2*power2(mz2)) - 4*power3(mstau22)) + power2(
     mstau12)*(2*(mu2 - mz2)*power2(mstau22) - 2*mu2*(mstau24 + 2*mu2*mz2 +
     power2(mz2)) + mstau22*(mstau24 + 6*mu2*mz2 - 10*power2(mu2) + power2(mz2)
     ) + 2*power3(mstau22))) - 2*mstau12*power3(mu2) + 2*mstau22*power3(mu2)))/
     (8.*mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (Fin3(
     mstau22,mC2,mu2,Q2)*(-(1/mC2) - (2*mu2)/(mC2*mstau22) - 6*mu2*DeltaInv(
     mstau22,mC2,mu2) + (2*mC2*mu2*DeltaInv(mstau22,mC2,mu2))/mstau22 -
     mstau22/power2(mC2) + mu2/power2(mC2) - (2*DeltaInv(mstau22,mC2,mu2)*
     power2(mu2))/mC2 - (4*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/mstau22 - (2*
     mu2*(msntau2*(mstau22 + mu2) - mu2*(3*mstau22 + mu2) - (msntau2 - mu2)*
     mu2*DeltaInv(mstau22,mC2,mu2)*(-3*mC2*mstau22 - 2*mC2*mu2 - mstau22*mu2 +
     power2(mC2) + power2(mu2)))*power2(sb))/(mstau22*power2(cb)*power3(msntau2
     - mu2)) + (2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mC2*mstau22)))/8. + (
     log(mH2/Q2)*(3 - (3*mC2)/(mC2 - mH2) - (9*mA2)/mH2 - (9*mC2)/mH2 + (2*mH2)
     /mA2 + (3*mA2)/(-mA2 + mH2) - (6*msntau2)/mH2 - (6*mstau12)/mH2 - (6*
     mstau22)/mH2 + (2*mH2)/(mstau12 - mu2) + (2*mH2)/(mstau22 - mu2) + (18*
     mu2)/mH2 + (6*mu2)/msntau2 + (6*mu2)/mstau12 + (6*mu2)/mstau22 + (4*mu2)/(
     msntau2 - mu2) + (4*mu2)/(mstau12 - mu2) + (4*mu2)/(mstau22 - mu2) + 18*
     mH2*mu2*DeltaInv(mstau22,mH2,mu2) - (3*power2(mA2))/power2(mA2 - mH2) - (
     3*power2(mC2))/power2(mC2 - mH2) - (6*mu2*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2))/mstau22 + (15*power2(mH2))/power2(mA2) - (4*mH2*mu2)/power2(
     mstau12 - mu2) - (4*mH2*mu2)/power2(mstau22 - mu2) + 6*DeltaInv(mstau22,
     mH2,mu2)*power2(mu2) + (12*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/
     mstau22 - (6*mu2*DeltaInv(mH2,msntau2,mu2)*(-3*mH2*msntau2 - 2*mH2*mu2 -
     msntau2*mu2 + power2(mH2) + power2(mu2)))/msntau2 - (6*mu2*DeltaInv(mH2,
     mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)))/mstau12 + (3*power3(mA2))/(mH2*power2(mA2 - mH2)) + (3*
     power3(mC2))/(mH2*power2(mC2 - mH2)) - (6*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/mstau22))/16. + (power2(log(mH2/Q2))*(3*mC2*msntau2*mstau12*
     mstau22*power2(mA2) + 12*mH2*msntau2*mstau12*mstau22*power2(mA2) - 2*mH2*
     msntau2*mstau12*mu2*power2(mA2) - 2*mH2*msntau2*mstau22*mu2*power2(mA2) -
     2*mH2*mstau12*mstau22*mu2*power2(mA2) - 9*msntau2*mstau12*mstau22*mu2*
     power2(mA2) + 7*mA2*msntau2*mstau12*mstau22*power2(mH2) - 6*msntau2*
     mstau12*mstau22*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mA2)*power2(mH2) + 3*
     mstau12*mstau22*power2(mA2)*power2(msntau2) + 3*msntau2*mstau22*power2(
     mA2)*power2(mstau12) + 3*msntau2*mstau12*power2(mA2)*power2(mstau22) - 2*
     mH2*msntau2*mstau12*mstau22*DeltaInv(mstau22,mH2,mu2)*power2(mA2)*power2(
     mu2) - 4*msntau2*mstau12*DeltaInv(mstau22,mH2,mu2)*power2(mA2)*power2(mH2)
     *power2(mu2) + 2*mH2*mstau12*mstau22*mu2*DeltaInv(mH2,msntau2,mu2)*power2(
     mA2)*(-3*mH2*msntau2 - 2*mH2*mu2 - msntau2*mu2 + power2(mH2) + power2(mu2)
     ) + 2*mH2*msntau2*mstau22*mu2*DeltaInv(mH2,mstau12,mu2)*power2(mA2)*(-3*
     mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)) + 3*
     msntau2*mstau12*mstau22*power3(mA2) - 10*msntau2*mstau12*mstau22*power3(
     mH2) + 2*msntau2*mstau12*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mA2)*power3(
     mH2) + 2*mH2*msntau2*mstau12*DeltaInv(mstau22,mH2,mu2)*power2(mA2)*power3(
     mu2)))/(32.*mH2*msntau2*mstau12*mstau22*power2(mA2)) + (power2(log(mA2/Q2)
     )*(15*mH2*msntau2*mstau12*mstau22*power2(mA2) + 4*mA2*msntau2*mstau12*
     mstau22*power2(mH2) - 5*mC2*msntau2*mstau12*mstau22*power2(mH2) - 2*mA2*
     msntau2*mstau12*mu2*power2(mH2) - 2*mA2*msntau2*mstau22*mu2*power2(mH2) -
     2*mA2*mstau12*mstau22*mu2*power2(mH2) + 15*msntau2*mstau12*mstau22*mu2*
     power2(mH2) - 6*msntau2*mstau12*mstau22*mu2*DeltaInv(mA2,mstau22,mu2)*
     power2(mA2)*power2(mH2) - 5*mstau12*mstau22*power2(mH2)*power2(msntau2) -
     5*msntau2*mstau22*power2(mH2)*power2(mstau12) - 5*msntau2*mstau12*power2(
     mH2)*power2(mstau22) - 2*mA2*msntau2*mstau12*mstau22*DeltaInv(mA2,mstau22,
     mu2)*power2(mH2)*power2(mu2) - 4*msntau2*mstau12*DeltaInv(mA2,mstau22,mu2)
     *power2(mA2)*power2(mH2)*power2(mu2) + 2*mA2*mstau12*mstau22*mu2*DeltaInv(
     mA2,msntau2,mu2)*power2(mH2)*(-3*mA2*msntau2 - 2*mA2*mu2 - msntau2*mu2 +
     power2(mA2) + power2(mu2)) + 2*mA2*msntau2*mstau22*mu2*DeltaInv(mA2,
     mstau12,mu2)*power2(mH2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2 +
     power2(mA2) + power2(mu2)) + 6*msntau2*mstau12*mstau22*power3(mA2) + 2*
     msntau2*mstau12*mu2*DeltaInv(mA2,mstau22,mu2)*power2(mH2)*power3(mA2) - 5*
     msntau2*mstau12*mstau22*power3(mH2) + 2*mA2*msntau2*mstau12*DeltaInv(mA2,
     mstau22,mu2)*power2(mH2)*power3(mu2)))/(32.*mA2*msntau2*mstau12*mstau22*
     power2(mH2)) - (3*mu2*log(mH2/Q2)*power2(sb)*(4/mH2 + 6/mstau12 + mH2/(
     mstau12*mstau22 - mstau12*mu2) + mH2/power2(mstau12 - mu2) - (mH2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (3*mu2*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2))/power2(mstau12 - mu2) - (2*mH2*mu2)/(mstau12*power2(mstau22 - mu2))
     - (mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 -
     mstau12*mu2 + power2(mH2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2))
     + (mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 -
     mu2)) + DeltaInv(mstau22,mH2,mstau12)*(18*mstau22 - (6*mstau24)/mstau12 +
     power2(mH2)*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(
     mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(
     mstau12 - mu2) + (-6 - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (
     3*mu2)/power2(mstau22 - mu2)) + mH2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*
     mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/power2(
     mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (12*mstau22 - (
     mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/mstau12 - (2*mstau22*
     mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (-(1/
     power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2)
     ))*power3(mH2)) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2))))/(8.*power2(cb)) + (mu2*power2(sb)*power2(log(mH2/
     Q2))*(6/mH2 + 6/mstau12 + mH2/(mstau12*mstau22 - mstau12*mu2) + mH2/
     power2(mstau12 - mu2) - (mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*
     mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (2*mH2*
     mu2)/(mstau12*power2(mstau22 - mu2)) - (mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2*DeltaInv(mH2,
     mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mH2,
     mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + DeltaInv(mstau22,mH2,
     mstau12)*(18*mstau22 - (6*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 -
     mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) +
     1/power2(mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*
     mu2)/power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) +
     mH2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*
     mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (12*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(
     mstau22 - mu2))/mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(
     mu2)/power2(mstau22 - mu2)) + (-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*
     mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (mH2*DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))))/(16.*
     power2(cb)) - (ca*mu2*sa*sb*power2(log(mH2/Q2))*(6/mH2 + 6/(-mh2 + mH2) +
     6/mstau12 + mH2/(mstau12*mstau22 - mstau12*mu2) + (8*mH2)/(-(mh2*mH2) +
     power2(mh2)) - (8*mH2)/power2(mh2 - mH2) + (2*power2(mH2))/((-mh2 + mH2)*
     power2(mh2)) + (10*power2(mH2))/(mh2*power2(mh2 - mH2)) + mH2/power2(
     mstau12 - mu2) - (mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (2*mH2*mu2)
     /(mstau12*power2(mstau22 - mu2)) - (mH2*DeltaInv(mstau22,mH2,mu2)*power2(
     mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*
     power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2*DeltaInv(mH2,
     mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (2*power3(mH2))/(power2(
     mh2)*power2(mh2 - mH2)) + (mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(18*mstau22
     - (6*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 - mu2) + 3/(mstau22 - mu2)
     + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2))
     + mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*mu2)/power2(mstau22 - mu2))
     /mstau12 - (3*mu2)/power2(mstau22 - mu2)) + mH2*(3 + (3*mstau22)/(mstau12
     - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/
     power2(mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (12*mstau22
     - (mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/mstau12 - (2*mstau22*
     mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (-(1/
     power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2)
     ))*power3(mH2)) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2))))/(8.*cb) - (mu2*power2(sa)*power2(sb)*power2(log(
     mH2/Q2))*(12/mH2 + 12/(-mh2 + mH2) + 12/mstau12 + mH2/(mstau12*mstau22 -
     mstau12*mu2) + (16*mH2)/(-(mh2*mH2) + power2(mh2)) - (16*mH2)/power2(mh2 -
     mH2) + (4*power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (20*power2(mH2))/(mh2*
     power2(mh2 - mH2)) + mH2/power2(mstau12 - mu2) - (mH2*mu2)/(mstau22*
     power2(mstau12 - mu2)) - (3*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/
     power2(mstau12 - mu2) - (2*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (
     mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 -
     mstau12*mu2 + power2(mH2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2))
     - (4*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) + (mu2*DeltaInv(mstau22,
     mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + DeltaInv(mstau22,
     mH2,mstau12)*(36*mstau22 - (12*mstau24)/mstau12 + power2(mH2)*(3/(mstau12
     - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2)
     + 1/power2(mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-12 - (2*
     mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 -
     mu2)) + mH2*(9 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2)
     + (2*mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)
     /power2(mstau12 - mu2) + (24*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(
     mstau22 - mu2))/mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(
     mu2)/power2(mstau22 - mu2)) + (-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*
     mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (mH2*DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))))/(16.*
     power2(cb)) + (ca*mu2*sa*sb*log(mH2/Q2)*(-(1/mh2) + 12/mH2 + 5/(-mh2 +
     mH2) + 18/mstau12 + (3*mH2)/(mstau12*mstau22 - mstau12*mu2) - mH2/power2(
     mh2) + (8*mH2)/(-(mh2*mH2) + power2(mh2)) - (8*mH2)/power2(mh2 - mH2) + (
     4*power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (11*power2(mH2))/(mh2*power2(
     mh2 - mH2)) + (3*mH2)/power2(mstau12 - mu2) - (3*mH2*mu2)/(mstau22*power2(
     mstau12 - mu2)) - (9*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(
     mstau12 - mu2) - (6*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (3*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (6*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (3*mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2
     - mstau12*mu2 + power2(mH2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)
     ) - (3*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) + (3*mu2*DeltaInv(
     mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + 3*
     DeltaInv(mstau22,mH2,mstau12)*(18*mstau22 - (6*mstau24)/mstau12 + power2(
     mH2)*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(mstau12*
     mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(mstau12 -
     mu2) + (-6 - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (3*mu2)/
     power2(mstau22 - mu2)) + mH2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*
     mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/(mstau12*
     mstau22 - mstau12*mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (12*mstau22 + (mstau24*mu2)/power2(mstau22 - mu2))
     /mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(
     mstau22 - mu2)) + (-(1/(mstau12*mstau22 - mstau12*mu2)) - 1/power2(mstau12
     - mu2) + mu2/(mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (3*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))))/(
     4.*cb) + (mu2*log(mH2/Q2)*power2(sa)*power2(sb)*(-2/mh2 + 24/mH2 + 10/(-
     mh2 + mH2) + 36/mstau12 + (3*mH2)/(mstau12*mstau22 - mstau12*mu2) - (2*
     mH2)/power2(mh2) + (16*mH2)/(-(mh2*mH2) + power2(mh2)) - (16*mH2)/power2(
     mh2 - mH2) + (8*power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (22*power2(mH2))
     /(mh2*power2(mh2 - mH2)) + (3*mH2)/power2(mstau12 - mu2) - (3*mH2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (9*mu2*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2))/power2(mstau12 - mu2) - (6*mH2*mu2)/(mstau12*power2(mstau22 - mu2))
     - (3*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (
     6*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(
     mstau12 - mu2)) + (3*mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 -
     2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)))/(mstau12*power2(
     mstau22 - mu2)) - (6*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) + (3*
     mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2))
     + 3*DeltaInv(mstau22,mH2,mstau12)*(36*mstau22 - (12*mstau24)/mstau12 +
     power2(mH2)*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(
     mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(
     mstau12 - mu2) + (-12 - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (
     3*mu2)/power2(mstau22 - mu2)) + mH2*(9 + (3*mstau22)/(mstau12 - mu2) + (2*
     mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/(mstau12*
     mstau22 - mstau12*mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (24*mstau22 + (mstau24*mu2)/power2(mstau22 - mu2))
     /mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(
     mstau22 - mu2)) + (-(1/(mstau12*mstau22 - mstau12*mu2)) - 1/power2(mstau12
     - mu2) + mu2/(mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (3*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))))/(
     8.*power2(cb)) - (3*log(msntau2/Q2)*power2(sa)*(-(mH2*mu2*DeltaInv(
     msntau2,mh2,mu2)*(5*power2(mh2)*(power2(msntau2) - power2(mu2)) + power2(
     msntau2 - mu2)*power2(mu2) + mh2*mu2*(-11*msntau2*mu2 + 10*power2(msntau2)
     + power2(mu2)) - 3*(msntau2 - mu2)*power3(mh2))) + mh2*mu2*DeltaInv(mH2,
     msntau2,mu2)*(5*power2(mH2)*(power2(msntau2) - power2(mu2)) + power2(
     msntau2 - mu2)*power2(mu2) + mH2*mu2*(-11*msntau2*mu2 + 10*power2(msntau2)
     + power2(mu2)) - 3*(msntau2 - mu2)*power3(mH2)) - (mh2 - mH2)*(mu2*power2(
     msntau2) + 2*power3(msntau2) + power3(mu2))))/(8.*mh2*mH2*msntau2*(msntau2
     - mu2)) + (log(mH2/Q2)*log(mstau12/Q2)*((3*mstau12)/mH2 + (4*mu2*DeltaInv(
     mH2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mH2*(mstau12 + mu2) + power2(
     mH2)))/mstau12 + (4*mH2*power2(mu2))/power3(mstau12 - mu2) - (4*(mu2*
     power2(mstau12) - 2*mstau12*power2(mu2) + power3(mstau12) + power3(mu2)))/
     (mstau12*power2(mstau12 - mu2))))/16. + (power2(sa)*power2(log(msntau2/Q2)
     )*(-2*mH2*mu2*DeltaInv(msntau2,mh2,mu2)*(5*power2(mh2)*(power2(msntau2) -
     power2(mu2)) + power2(msntau2 - mu2)*power2(mu2) + mh2*mu2*(-11*msntau2*
     mu2 + 10*power2(msntau2) + power2(mu2)) - 3*(msntau2 - mu2)*power3(mh2)) +
     2*mh2*mu2*DeltaInv(mH2,msntau2,mu2)*(5*power2(mH2)*(power2(msntau2) -
     power2(mu2)) + power2(msntau2 - mu2)*power2(mu2) + mH2*mu2*(-11*msntau2*
     mu2 + 10*power2(msntau2) + power2(mu2)) - 3*(msntau2 - mu2)*power3(mH2)) -
     (mh2 - mH2)*(3*mu2*power2(msntau2) + 3*power3(msntau2) + 2*power3(mu2))))/
     (32.*mh2*mH2*msntau2*(msntau2 - mu2)) + (mh2*mu2*log(mh2/Q2)*log(mstau12/
     Q2)*(-6*mstau12*mstau22*mu2 + 2*mstau22*power2(mstau12) - 6*mu2*power2(
     mstau12) + mstau12*power2(mstau22) + 2*mu2*DeltaInv(mstau12,mh2,mu2)*(mu2*
     (-2*mstau12 + mu2) - 2*mh2*(mstau12 + mu2) + power2(mh2))*power2(mstau12 -
     mu2) + 13*mstau12*power2(mu2) + 2*mstau22*power2(mu2) - 6*power3(mu2) +
     DeltaInv(mstau22,mstau12,mh2)*(2*mstau24*(-mstau22 + 2*mu2)*power2(mu2) -
     power2(mh2)*(2*(mstau22 - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22
     - 3*mu2) + 2*(mstau22 - 2*mu2)*power2(mu2)) + 2*(-2*mstau22*mu2 + power2(
     mstau22) - power2(mu2))*power3(mstau12) + mh2*(4*mstau22*(mstau22 - 2*mu2)
     *power2(mu2) + power2(mstau12)*(-22*mstau22*mu2 + 7*power2(mstau22) + 19*
     power2(mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) + 2*mstau12*(-7*mu2*
     power2(mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) - 5*power3(mu2)
     )) + power2(mstau12)*(-2*mstau22*mstau24 + 4*mstau24*mu2 - 8*mu2*power2(
     mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) + 2*power3(mu2)) +
     mstau12*(6*mstau22*mstau24*mu2 - mstau24*power2(mstau22) - 9*mstau24*
     power2(mu2) + 2*mu2*power3(mstau22) - 6*mstau22*power3(mu2)))))/(8.*
     mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (ca*mu2*sa*sb*log(
     mh2/Q2)*log(mstau12/Q2)*(-2/(mstau12 - mstau22) + (2*mu2)/((mstau12 -
     mstau22)*(-mstau12 + mu2)) + (2*mh2)/(mstau12*mstau22 - mstau12*mu2) +
     mh2/power2(mstau12 - mu2) - (4*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) +
     (2*mh2*mu2*DeltaInv(mstau12,mh2,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mh2*(
     mstau12 + mu2) + power2(mh2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*
     DeltaInv(mstau22,mstau12,mh2)*(2*mstau24*(-mstau22 + 2*mu2)*power2(mu2) -
     power2(mh2)*(2*(mstau22 - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22
     - 3*mu2) + 2*(mstau22 - 2*mu2)*power2(mu2)) + 2*(-2*mstau22*mu2 + power2(
     mstau22) - power2(mu2))*power3(mstau12) + mh2*(4*mstau22*(mstau22 - 2*mu2)
     *power2(mu2) + power2(mstau12)*(-22*mstau22*mu2 + 7*power2(mstau22) + 19*
     power2(mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) + 2*mstau12*(-7*mu2*
     power2(mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) - 5*power3(mu2)
     )) + power2(mstau12)*(-2*mstau22*mstau24 + 4*mstau24*mu2 - 8*mu2*power2(
     mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) + 2*power3(mu2)) +
     mstau12*(6*mstau22*mstau24*mu2 - mstau24*power2(mstau22) - 9*mstau24*
     power2(mu2) + 2*mu2*power3(mstau22) - 6*mstau22*power3(mu2))))/(mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(4.*cb) + (mu2*log(mh2/Q2)*
     log(mstau12/Q2)*power2(sa)*power2(sb)*(-4/(mstau12 - mstau22) + (4*mu2)/((
     mstau12 - mstau22)*(-mstau12 + mu2)) + (2*mh2)/(mstau12*mstau22 - mstau12*
     mu2) + mh2/power2(mstau12 - mu2) - (4*mh2*mu2)/(mstau12*power2(mstau22 -
     mu2)) + (2*mh2*mu2*DeltaInv(mstau12,mh2,mu2)*(mu2*(-2*mstau12 + mu2) - 2*
     mh2*(mstau12 + mu2) + power2(mh2)))/(mstau12*power2(mstau22 - mu2)) + (
     mh2*DeltaInv(mstau22,mstau12,mh2)*(2*mstau24*(-mstau22 + 2*mu2)*power2(
     mu2) - power2(mh2)*(2*(mstau22 - 2*mu2)*power2(mstau12) + mstau12*power2(
     mstau22 - 3*mu2) + 2*(mstau22 - 2*mu2)*power2(mu2)) + 2*(-2*mstau22*mu2 +
     power2(mstau22) - power2(mu2))*power3(mstau12) + mh2*(4*mstau22*(mstau22 -
     2*mu2)*power2(mu2) + power2(mstau12)*(-22*mstau22*mu2 + 7*power2(mstau22)
     + 19*power2(mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) + 2*mstau12*(-7*
     mu2*power2(mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) - 5*power3(
     mu2))) + power2(mstau12)*(-2*mstau22*mstau24 + 4*mstau24*mu2 - 8*mu2*
     power2(mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) + 2*power3(mu2)
     ) + mstau12*(6*mstau22*mstau24*mu2 - mstau24*power2(mstau22) - 9*mstau24*
     power2(mu2) + 2*mu2*power3(mstau22) - 6*mstau22*power3(mu2))))/(mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(8.*power2(cb)) + (Fin3(
     mstau22,mh2,mu2,Q2)*((8*ca*mu2*(mh2 - mstau22 + mu2)*sa*sb)/(cb*mh2*(
     mstau12 - mstau22)*(mstau22 - mu2)) + (8*mu2*(mh2 - mstau22 + mu2)*power2(
     sa)*power2(sb))/(mh2*(mstau12 - mstau22)*(mstau22 - mu2)*power2(cb)) + (2*
     mu2*(-(mstau12*(mstau22 + mu2)) + mu2*(3*mstau22 + mu2) + (mstau12 - mu2)*
     mu2*DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2))))/(mstau22*power3(mstau12 - mu2)) - (4*ca*mu2*
     sa*sb*(-2/(mh2*mstau12 - mh2*mstau22) + 2/((mstau12 - mstau22)*(mstau22 -
     mu2)) + 1/power2(mstau12 - mu2) + mu2/(mstau22*power2(mstau12 - mu2)) + (
     mu2*DeltaInv(mstau22,mh2,mu2)*((mstau22 - mu2)*mu2 + mh2*(3*mstau22 + 2*
     mu2) - power2(mh2)))/(mstau22*power2(mstau12 - mu2)) + (2*mu2)/power3(-
     mstau12 + mu2)))/cb - (2*mu2*power2(sa)*power2(sb)*(-4/(mh2*mstau12 - mh2*
     mstau22) + 4/((mstau12 - mstau22)*(mstau22 - mu2)) + 1/power2(mstau12 -
     mu2) + mu2/(mstau22*power2(mstau12 - mu2)) + (mu2*DeltaInv(mstau22,mh2,
     mu2)*((mstau22 - mu2)*mu2 + mh2*(3*mstau22 + 2*mu2) - power2(mh2)))/(
     mstau22*power2(mstau12 - mu2)) + (2*mu2)/power3(-mstau12 + mu2)))/power2(
     cb) + power2(sa)*(8/(mstau22 - mu2) + (-13 - (2*mu2)/mstau22)/mh2 + (3*(
     mstau22 - mu2))/power2(mh2) + (2*mu2)/power2(mstau12 - mu2) + (2*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mstau22,mh2,mu2)*(
     -2*mstau12*mu2 + mu2*(-mh2 + mu2) + power2(mstau12))*(-3*mh2*mstau22 - 2*
     mh2*mu2 - mstau22*mu2 + power2(mh2) + power2(mu2)))/(mh2*mstau22*power2(
     mstau12 - mu2)) + (4*power2(mu2))/power3(-mstau12 + mu2))))/16. - (log(
     mA2/Q2)*log(mstau12/Q2)*((5*mstau12)/mA2 + (4*mu2)/mstau12 - (4*mu2*
     DeltaInv(mA2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mA2*(mstau12 + mu2)
     + power2(mA2)))/mstau12 + (4*mA2*power2(mu2))/power3(-mstau12 + mu2)))/16.
      + (Fin3(mstau22,mH2,mu2,Q2)*(8/(mstau22 - mu2) + (4*ca*mu2*sa*sb)/(cb*
     power2(mstau12 - mu2)) + (4*ca*sa*sb*power2(mu2))/(cb*mstau22*power2(
     mstau12 - mu2)) + ((13*mstau22 + 2*mu2)*(-1 + power2(sa)))/(mH2*mstau22) -
     (3*(mstau22 - mu2)*(-1 + power2(sa)))/power2(mH2) + (8*power2(sa))/(-
     mstau22 + mu2) - (2*mu2*power2(sa))/power2(mstau12 - mu2) - (2*power2(mu2)
     *power2(sa))/(mstau22*power2(mstau12 - mu2)) - (2*mu2*power2(sb))/(power2(
     cb)*power2(mstau12 - mu2)) - (2*power2(mu2)*power2(sb))/(mstau22*power2(
     cb)*power2(mstau12 - mu2)) + (2*mu2*power2(sa)*power2(sb))/(power2(cb)*
     power2(mstau12 - mu2)) + (2*power2(mu2)*power2(sa)*power2(sb))/(mstau22*
     power2(cb)*power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mstau22,mH2,mu2)*(-3*
     mH2*mstau22 - 2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(mu2))*(-2*ca*
     cb*mH2*mu2*sa*sb + power2(cb)*(2*mstau12*mu2*(-1 + power2(sa)) - power2(
     mstau12)*(-1 + power2(sa)) + mu2*(mu2 + mH2*power2(sa) - mu2*power2(sa)))
     - mH2*mu2*(-1 + power2(sa))*power2(sb)))/(mH2*mstau22*power2(cb)*power2(
     mstau12 - mu2)) + (4*power2(mu2)*power2(sa))/power3(mstau12 - mu2) + (4*
     power2(mu2)*power2(sb))/(power2(cb)*power3(mstau12 - mu2)) + (8*ca*sa*sb*
     power2(mu2))/(cb*power3(-mstau12 + mu2)) + (4*power2(mu2)*power2(sa)*
     power2(sb))/(power2(cb)*power3(-mstau12 + mu2))))/16. + (mu2*log(mC2/Q2)*
     log(mu2/Q2)*(2/mC2 + 4/mstau22 - (4*DeltaInv(mstau22,mC2,mu2)*(mu2*(-3*
     mstau22 + mu2) - mC2*(mstau22 + 2*mu2) + power2(mC2)))/mstau22 + (3*mu2)/
     power2(msntau2 - mu2) + (3*mu2)/power2(mstau12 - mu2) + mu2/power2(mstau22
     - mu2) + 8*mC2*mu2*(1/power3(-msntau2 + mu2) + 1/power3(-mstau22 + mu2))))
     /16. + (Fin3(mstau12,mh2,mu2,Q2)*((-8*ca*mu2*(mh2 - mstau12 + mu2)*sa*sb)/
     (cb*mh2*(mstau12 - mstau22)*(mstau12 - mu2)) - (8*mu2*(mh2 - mstau12 +
     mu2)*power2(sa)*power2(sb))/(mh2*(mstau12 - mstau22)*(mstau12 - mu2)*
     power2(cb)) - power2(sa)*(8/(-mstau12 + mu2) + (13 + (2*mu2)/mstau12)/mh2
     - (3*(mstau12 - mu2))/power2(mh2) - (2*mu2)/power2(mstau22 - mu2) - (2*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (2*mu2*DeltaInv(mstau12,
     mh2,mu2)*(-2*mstau22*mu2 + mu2*(-mh2 + mu2) + power2(mstau22))*(-3*mh2*
     mstau12 - 2*mh2*mu2 - mstau12*mu2 + power2(mh2) + power2(mu2)))/(mh2*
     mstau12*power2(mstau22 - mu2)) + (4*power2(mu2))/power3(mstau22 - mu2)) +
     (2*mu2*(-(mstau12*mstau22) + 3*mstau12*mu2 - mstau22*mu2 + power2(mu2) + (
     mstau22 - mu2)*mu2*DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 -
     mstau12*mu2 + power2(mh2) + power2(mu2))))/(mstau12*power3(mstau22 - mu2))
     - (4*ca*mu2*sa*sb*(2/(mh2*mstau12 - mh2*mstau22) + 2/((mstau12 - mstau22)*
     (-mstau12 + mu2)) + 1/power2(mstau22 - mu2) + mu2/(mstau12*power2(mstau22
     - mu2)) + (mu2*DeltaInv(mstau12,mh2,mu2)*((mstau12 - mu2)*mu2 + mh2*(3*
     mstau12 + 2*mu2) - power2(mh2)))/(mstau12*power2(mstau22 - mu2)) + (2*mu2)
     /power3(-mstau22 + mu2)))/cb - (2*mu2*power2(sa)*power2(sb)*(4/(mh2*
     mstau12 - mh2*mstau22) + 4/((mstau12 - mstau22)*(-mstau12 + mu2)) + 1/
     power2(mstau22 - mu2) + mu2/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mstau12,mh2,mu2)*((mstau12 - mu2)*mu2 + mh2*(3*mstau12 + 2*mu2) -
     power2(mh2)))/(mstau12*power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau22 +
     mu2)))/power2(cb)))/16. + (Fin3(mH2,mstau12,mu2,Q2)*(-13/mH2 + 8/(mstau12
     - mu2) - (2*mu2)/(mH2*mstau12) + (8*ca*mu2*(mH2 - mstau12 + mu2)*sa*sb)/(
     cb*mH2*(mstau12 - mstau22)*(mstau12 - mu2)) - 6*mu2*DeltaInv(mH2,mstau12,
     mu2) + (2*mH2*mu2*DeltaInv(mH2,mstau12,mu2))/mstau12 + (3*mstau12)/power2(
     mH2) - (3*mu2)/power2(mH2) - (2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/mH2
     - (4*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/mstau12 - (8*mu2*(mH2 -
     mstau12 + mu2)*power2(sb))/(mH2*(mstau12 - mstau22)*(mstau12 - mu2)*
     power2(cb)) + (8*mu2*(mH2 - mstau12 + mu2)*power2(sa)*power2(sb))/(mH2*(
     mstau12 - mstau22)*(mstau12 - mu2)*power2(cb)) + (2*DeltaInv(mH2,mstau12,
     mu2)*power3(mu2))/(mH2*mstau12) + (4*ca*mu2*sa*sb*(2/(mH2*mstau12 - mH2*
     mstau22) + 2/((mstau12 - mstau22)*(-mstau12 + mu2)) + 1/power2(mstau22 -
     mu2) + mu2/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mH2,mstau12,
     mu2)*((mstau12 - mu2)*mu2 + mH2*(3*mstau12 + 2*mu2) - power2(mH2)))/(
     mstau12*power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau22 + mu2)))/cb - (2*
     mu2*power2(sb)*(4/(mH2*mstau12 - mH2*mstau22) + 4/((mstau12 - mstau22)*(-
     mstau12 + mu2)) + 1/power2(mstau22 - mu2) + mu2/(mstau12*power2(mstau22 -
     mu2)) + (mu2*DeltaInv(mH2,mstau12,mu2)*((mstau12 - mu2)*mu2 + mH2*(3*
     mstau12 + 2*mu2) - power2(mH2)))/(mstau12*power2(mstau22 - mu2)) + (2*mu2)
     /power3(-mstau22 + mu2)))/power2(cb) + (2*mu2*power2(sa)*power2(sb)*(4/(
     mH2*mstau12 - mH2*mstau22) + 4/((mstau12 - mstau22)*(-mstau12 + mu2)) + 1/
     power2(mstau22 - mu2) + mu2/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mH2,mstau12,mu2)*((mstau12 - mu2)*mu2 + mH2*(3*mstau12 + 2*mu2) -
     power2(mH2)))/(mstau12*power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau22 +
     mu2)))/power2(cb) - power2(sa)*(8/(mstau12 - mu2) + (-13 - (2*mu2)/
     mstau12)/mH2 + (3*(mstau12 - mu2))/power2(mH2) + (2*mu2)/power2(mstau22 -
     mu2) + (2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (2*mu2*DeltaInv(
     mH2,mstau12,mu2)*(-2*mstau22*mu2 + mu2*(-mH2 + mu2) + power2(mstau22))*(-
     3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)))/(
     mH2*mstau12*power2(mstau22 - mu2)) + (4*power2(mu2))/power3(-mstau22 +
     mu2))))/16. + (Fin20(mstau12,mu2,Q2)*(-8/mA2 + (5*(mstau12 - mu2))/power2(
     mA2) + (16*mH2 - 3*mstau12 + 3*mu2 + (2*mH2*(4*mH2*mu2*power3(mstau22 -
     mu2) + mA2*(-4*mu2*power3(mstau22 - mu2) + mH2*(-((mstau22 + mu2)*power2(
     mstau12)) + mu2*power2(mstau22) - 8*mstau22*power2(mu2) + 4*mstau12*(-(
     mstau22*mu2) + power2(mstau22) + power2(mu2)) + power3(mstau22) + 4*
     power3(mu2)))))/(mA2*(mstau12 - mu2)*power3(-mstau22 + mu2)))/power2(mH2))
     )/16. + (mu2*Fin3(mstau22,msntau2,mw2,Q2)*(1/(msntau2*mstau22 - msntau2*
     mu2) + 2/power2(msntau2 - mu2) + 1/power2(mstau22 - mu2) - mu2/(msntau2*
     power2(mstau22 - mu2)) - (DeltaInv(mstau22,msntau2,mw2)*(-((mstau22 - 2*
     mu2)*(-mstau24 + (2*mstau22 - mw2)*mw2)*power2(mu2)) + (2*mu2*(mu2 + 3*
     mw2) - mstau22*(2*mu2 + 3*mw2) + power2(mstau22))*power3(msntau2) +
     power2(msntau2)*((6*mu2 - 5*mw2)*power2(mstau22) + mstau22*(mstau24 + 16*
     mu2*mw2 - 3*power2(mu2) + power2(mw2)) - mu2*(2*mstau24 + 15*mu2*mw2 + 2*
     power2(mu2) + 2*power2(mw2)) - 3*power3(mstau22)) + msntau2*(mstau22*mu2*(
     -4*mstau24 - 17*mu2*mw2 + 4*power2(mu2) - 4*power2(mw2)) + power2(mstau22)
     *(mstau24 + 10*mu2*mw2 - 5*power2(mu2) + power2(mw2)) + power2(mu2)*(5*
     mstau24 + 8*mu2*mw2 + 5*power2(mw2)) + 2*(mu2 - mw2)*power3(mstau22))))/(
     msntau2*power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (2*mu2)/power3(-
     msntau2 + mu2) + (2*mu2)/power3(-mstau22 + mu2)))/4. + (mu2*Fin3(mstau22,
     mstau12,mz2,Q2)*(1/(mstau12*mstau22 - mstau12*mu2) + 2/power2(mstau12 -
     mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 - mu2)) + (4*
     power2(sb))/((mstau12 - mu2)*(-mstau22 + mu2)) - (4*mstau12*power2(sb))/((
     mstau12 - mu2)*(-mstau22 + mu2)*mz2) - (4*mstau22*power2(sb))/((mstau12 -
     mu2)*(-mstau22 + mu2)*mz2) + (8*mu2*power2(sb))/((mstau12 - mu2)*(-mstau22
     + mu2)*mz2) - (DeltaInv(mstau22,mstau12,mz2)*(-((mstau22 - 2*mu2)*(-
     mstau24 + (2*mstau22 - mz2)*mz2)*power2(mu2)) + (2*mu2*(mu2 + 3*mz2) -
     mstau22*(2*mu2 + 3*mz2) + power2(mstau22))*power3(mstau12) + power2(
     mstau12)*((6*mu2 - 5*mz2)*power2(mstau22) + mstau22*(mstau24 + 16*mu2*mz2
     - 3*power2(mu2) + power2(mz2)) - mu2*(2*mstau24 + 15*mu2*mz2 + 2*power2(
     mu2) + 2*power2(mz2)) - 3*power3(mstau22)) + mstau12*(mstau22*mu2*(-4*
     mstau24 - 17*mu2*mz2 + 4*power2(mu2) - 4*power2(mz2)) + power2(mstau22)*(
     mstau24 + 10*mu2*mz2 - 5*power2(mu2) + power2(mz2)) + power2(mu2)*(5*
     mstau24 + 8*mu2*mz2 + 5*power2(mz2)) + 2*(mu2 - mz2)*power3(mstau22))))/(
     mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (2*mu2)/power3(-
     mstau12 + mu2) + (2*mu2)/power3(-mstau22 + mu2)))/8. + (log(mu2/Q2)*
     power2(mu2)*power2(sb)*(4/power2(msntau2 - mu2) + (6*mu2)/(mstau22*power2(
     msntau2 - mu2)) + (18*mC2*mu2*DeltaInv(mstau22,mC2,mu2))/power2(msntau2 -
     mu2) - (6*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/(mstau22*power2(
     msntau2 - mu2)) + 4/power2(mstau12 - mu2) + (6*mu2)/(mstau22*power2(
     mstau12 - mu2)) + (9*mH2*mu2*DeltaInv(mstau22,mH2,mu2))/power2(mstau12 -
     mu2) - (3*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/(mstau22*power2(
     mstau12 - mu2)) + 8/power2(mstau22 - mu2) + (6*mu2)/(msntau2*power2(
     mstau22 - mu2)) + (6*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mC2*mu2*
     DeltaInv(mC2,msntau2,mu2))/power2(mstau22 - mu2) + (9*mH2*mu2*DeltaInv(
     mH2,mstau12,mu2))/power2(mstau22 - mu2) - (6*mu2*DeltaInv(mC2,msntau2,mu2)
     *power2(mC2))/(msntau2*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mH2,
     mstau12,mu2)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (6*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (12*mC2*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) + (3*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (6*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     6*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) + (12*mC2*
     DeltaInv(mC2,msntau2,mu2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (
     3*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2
     + power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*
     DeltaInv(mA2,mstau22,mu2)*(-3*mA2*mstau22 - 2*mA2*mu2 - mstau22*mu2 +
     power2(mA2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) - (3*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     6*DeltaInv(mC2,msntau2,mu2)*power3(mu2))/(msntau2*power2(mstau22 - mu2)) -
     (3*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2))
     + (4*mu2)/power3(-msntau2 + mu2) + (4*mu2)/power3(-mstau12 + mu2) + (8*
     mu2)/power3(-mstau22 + mu2)))/(8.*power2(cb)) - (power2(mu2)*power2(sb)*
     power2(log(mu2/Q2))*(2/power2(msntau2 - mu2) + (2*mu2)/(mstau22*power2(
     msntau2 - mu2)) + (6*mC2*mu2*DeltaInv(mstau22,mC2,mu2))/power2(msntau2 -
     mu2) - (2*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/(mstau22*power2(
     msntau2 - mu2)) + 2/power2(mstau12 - mu2) + (2*mu2)/(mstau22*power2(
     mstau12 - mu2)) + (3*mH2*mu2*DeltaInv(mstau22,mH2,mu2))/power2(mstau12 -
     mu2) - (mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/(mstau22*power2(mstau12
     - mu2)) + 4/power2(mstau22 - mu2) + (2*mu2)/(msntau2*power2(mstau22 - mu2)
     ) + (2*mu2)/(mstau12*power2(mstau22 - mu2)) + (6*mC2*mu2*DeltaInv(mC2,
     msntau2,mu2))/power2(mstau22 - mu2) + (3*mH2*mu2*DeltaInv(mH2,mstau12,mu2)
     )/power2(mstau22 - mu2) + (mu2*DeltaInv(mA2,mstau12,mu2)*((mstau12 - mu2)*
     mu2 + mA2*(3*mstau12 + 2*mu2) - power2(mA2)))/(mstau12*power2(mstau22 -
     mu2)) - (2*mu2*DeltaInv(mC2,msntau2,mu2)*power2(mC2))/(msntau2*power2(
     mstau22 - mu2)) - (mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/(mstau12*
     power2(mstau22 - mu2)) + (mu2*DeltaInv(mA2,mstau22,mu2)*(3*mA2*mstau22 +
     2*mA2*mu2 + mstau22*mu2 - power2(mA2) - power2(mu2)))/(mstau22*power2(
     mstau12 - mu2)) + (2*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2
     - mu2) + (4*mC2*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/(mstau22*power2(
     msntau2 - mu2)) + (DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 -
     mu2) + (2*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/(mstau22*power2(
     mstau12 - mu2)) + (2*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22
     - mu2) + (4*mC2*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/(msntau2*power2(
     mstau22 - mu2)) + (DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 -
     mu2) + (2*mH2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mstau22*
     power2(msntau2 - mu2)) - (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (2*DeltaInv(mC2,msntau2,mu2)*power3(mu2))/(
     msntau2*power2(mstau22 - mu2)) - (DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (4*mu2)/power3(-msntau2 + mu2) + (4*mu2)/
     power3(-mstau12 + mu2) + (8*mu2)/power3(-mstau22 + mu2)))/(16.*power2(cb))
     + (mu2*log(mA2/Q2)*log(mu2/Q2)*(15/mA2 + 2/msntau2 + 2/mstau12 + 2/mstau22
     + 2*mA2*DeltaInv(mA2,mstau22,mu2) + 6*mu2*DeltaInv(mA2,mstau22,mu2) + (4*
     mA2*mu2*DeltaInv(mA2,mstau22,mu2))/mstau22 - (2*DeltaInv(mA2,mstau22,mu2)*
     power2(mA2))/mstau22 - (2*DeltaInv(mA2,msntau2,mu2)*(mu2*(-3*msntau2 +
     mu2) - mA2*(msntau2 + 2*mu2) + power2(mA2)))/msntau2 - (2*DeltaInv(mA2,
     mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mA2*(mstau12 + 2*mu2) + power2(mA2)
     ))/mstau12 - (2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/mstau22 + (4*mA2*
     mu2)/power3(-mstau12 + mu2) + (4*mA2*mu2)/power3(-mstau22 + mu2)))/16. + (
     mu2*log(mh2/Q2)*log(mu2/Q2)*power2(sa)*(-9/mh2 + 2/msntau2 + 2/mstau12 +
     2/mstau22 + 8/(mstau12 - mu2) + 8/(mstau22 - mu2) + 2*mh2*DeltaInv(
     mstau22,mh2,mu2) + 6*mu2*DeltaInv(mstau22,mh2,mu2) + (4*mh2*mu2*DeltaInv(
     mstau22,mh2,mu2))/mstau22 - (2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/
     mstau22 - (2*DeltaInv(msntau2,mh2,mu2)*(mu2*(-3*msntau2 + mu2) - mh2*(
     msntau2 + 2*mu2) + power2(mh2)))/msntau2 + (4*mu2)/power2(msntau2 - mu2) +
     (4*mu2)/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau22*power2(mstau12 - mu2)
     ) - (2*mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/power2(mstau12 - mu2) +
     (4*mu2)/power2(mstau22 - mu2) - (2*mh2*mu2)/(mstau12*power2(mstau22 - mu2)
     ) - (2*DeltaInv(mstau12,mh2,mu2)*(mu2*(-3*mstau12 + mu2) - mh2*(mstau12 +
     2*mu2) + power2(mh2))*(-2*mstau22*mu2 + mu2*(-mh2 + mu2) + power2(mstau22)
     ))/(mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mh2,mu2)*power2(
     mu2))/mstau22 - (6*mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/power2(
     mstau12 - mu2) - (4*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mstau22,mh2,mu2)*power3(
     mh2))/(mstau22*power2(mstau12 - mu2)) + (2*mh2*DeltaInv(mstau22,mh2,mu2)*
     power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (4*mh2*mu2)/power3(-mstau12
     + mu2) + (4*mh2*mu2)/power3(-mstau22 + mu2)))/16. + (mu2*log(mH2/Q2)*log(
     mu2/Q2)*(-9/mH2 + 2/msntau2 + 2/mstau12 + 2/mstau22 + 8/(mstau12 - mu2) +
     8/(mstau22 - mu2) + 2*mH2*DeltaInv(mstau22,mH2,mu2) + 6*mu2*DeltaInv(
     mstau22,mH2,mu2) + (4*mH2*mu2*DeltaInv(mstau22,mH2,mu2))/mstau22 - (2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/mstau22 - (2*DeltaInv(mH2,msntau2,
     mu2)*(mu2*(-3*msntau2 + mu2) - mH2*(msntau2 + 2*mu2) + power2(mH2)))/
     msntau2 - (2*DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mH2*(
     mstau12 + 2*mu2) + power2(mH2)))/mstau12 + (4*mu2)/power2(msntau2 - mu2) +
     (4*mu2)/power2(mstau12 - mu2) + (4*mu2)/power2(mstau22 - mu2) - (2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 + (4*mH2*mu2)/power3(-
     mstau12 + mu2) + (4*mH2*mu2)/power3(-mstau22 + mu2)))/16. - (mu2*log(mH2/
     Q2)*log(mu2/Q2)*power2(sa)*(-9/mH2 + 2/msntau2 + 2/mstau12 + 2/mstau22 +
     8/(mstau12 - mu2) + 8/(mstau22 - mu2) + 2*mH2*DeltaInv(mstau22,mH2,mu2) +
     6*mu2*DeltaInv(mstau22,mH2,mu2) + (4*mH2*mu2*DeltaInv(mstau22,mH2,mu2))/
     mstau22 - (2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/mstau22 - (2*DeltaInv(
     mH2,msntau2,mu2)*(mu2*(-3*msntau2 + mu2) - mH2*(msntau2 + 2*mu2) + power2(
     mH2)))/msntau2 + (4*mu2)/power2(msntau2 - mu2) + (4*mu2)/power2(mstau12 -
     mu2) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (2*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (4*mu2)/power2(
     mstau22 - mu2) - (2*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (2*
     DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mH2*(mstau12 + 2*mu2)
     + power2(mH2))*(-2*mstau22*mu2 + mu2*(-mH2 + mu2) + power2(mstau22)))/(
     mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))
     /mstau22 - (6*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 -
     mu2) - (4*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) + (2*mH2*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (4*mH2*mu2)/power3(-mstau12 + mu2)
     + (4*mH2*mu2)/power3(-mstau22 + mu2)))/16. - (log(mA2/Q2)*log(mstau22/Q2)*
     ((5*mstau22)/mA2 + (4*mu2)/mstau22 - (4*mu2*DeltaInv(mA2,mstau22,mu2)*(
     mu2*(-2*mstau22 + mu2) - 2*mA2*(mstau22 + mu2) + power2(mA2)))/mstau22 + (
     4*mA2*power2(mu2))/power3(-mstau22 + mu2)))/16. - (log(mC2/Q2)*log(
     mstau22/Q2)*(-1 + (2*mstau22)/mC2 + (8*mu2)/mstau22 - (8*mu2*DeltaInv(
     mstau22,mC2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mC2*(mstau22 + mu2) + power2(
     mC2)))/mstau22 + power2(mu2)/power2(mstau22 - mu2) + (8*mC2*power2(mu2))/
     power3(-mstau22 + mu2)))/16. - (log(mH2/Q2)*log(mstau22/Q2)*((-3*mstau22)/
     mH2 - (4*mu2*DeltaInv(mstau22,mH2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mH2*(
     mstau22 + mu2) + power2(mH2)))/mstau22 + (4*(mu2*power2(mstau22) - 2*
     mstau22*power2(mu2) + power3(mstau22) + power3(mu2)))/(mstau22*power2(
     mstau22 - mu2)) + (4*mH2*power2(mu2))/power3(-mstau22 + mu2)))/16. + (3*
     ca*mu2*sb*log(mh2/Q2)*(3*mh2*mH2 - 4*mh2*mstau12 + 2*mH2*mstau12 - 3*
     power2(mh2) + 3*mh2*(mh2 - mH2)*DeltaInv(mstau22,mstau12,mh2)*(-3*mstau12*
     mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 + power2(mh2)))*power3(sa))/
     (2.*cb*mh2*(mh2 - mH2)*mstau12) - (3*ca*mu2*sb*Fin20(mstau22,mstau12,Q2)*
     power2(mh2 - mH2)*power3(sa))/(2.*cb*power2(mh2)*power2(mH2)) + (3*ca*mu2*
     sb*log(mH2/Q2)*(-3*mh2*mH2 - 2*mh2*mstau12 + 4*mH2*mstau12 + 3*power2(mH2)
     + 3*(mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*mstau22 -
     mH2*(mstau12 + 2*mstau22) + mstau24 + power2(mH2)))*power3(sa))/(2.*cb*(
     mh2 - mH2)*mH2*mstau12) - (3*ca*mu2*sb*(mh2*(mH2 - 3*mstau12) + mH2*
     mstau12 - power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(mstau22,mstau12,mh2)*(-3*
     mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 + power2(mh2)))*
     power2(log(mh2/Q2))*power3(sa))/(4.*cb*mh2*(mh2 - mH2)*mstau12) + (3*ca*
     mu2*sb*(-(mh2*(mH2 + mstau12)) + mH2*(mH2 + 3*mstau12) + (mh2 - mH2)*mH2*
     DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*mstau22 - mH2*(mstau12 + 2*
     mstau22) + mstau24 + power2(mH2)))*power2(log(mH2/Q2))*power3(sa))/(4.*cb*
     mH2*(-mh2 + mH2)*mstau12) + (log(mA2/Q2)*(14 + (14*mC2)/mA2 + (6*mC2)/(mA2
     - mC2) - (66*mA2)/mH2 + (20*mH2)/mA2 + (6*mA2)/(-mA2 + mH2) + (20*msntau2)
     /mA2 + (20*mstau12)/mA2 + (20*mstau22)/mA2 + (4*mA2)/(mstau12 - mu2) + (4*
     mA2)/(mstau22 - mu2) - (60*mu2)/mA2 + (12*mu2)/msntau2 + (12*mu2)/mstau12
     + (12*mu2)/mstau22 + 36*mA2*mu2*DeltaInv(mA2,mstau22,mu2) - (12*mu2*
     DeltaInv(mA2,mstau22,mu2)*power2(mA2))/mstau22 - (6*power2(mC2))/power2(
     mA2 - mC2) - (3*power2(mA2))/power2(mA2 - mH2) - (21*power2(mA2))/power2(
     mH2) - (8*mA2*mu2)/power2(mstau12 - mu2) - (8*mA2*mu2)/power2(mstau22 -
     mu2) + 12*DeltaInv(mA2,mstau22,mu2)*power2(mu2) + (24*mA2*DeltaInv(mA2,
     mstau22,mu2)*power2(mu2))/mstau22 - (12*mu2*DeltaInv(mA2,msntau2,mu2)*(-3*
     mA2*msntau2 - 2*mA2*mu2 - msntau2*mu2 + power2(mA2) + power2(mu2)))/
     msntau2 - (12*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 -
     mstau12*mu2 + power2(mA2) + power2(mu2)))/mstau12 + (6*power3(mC2))/(mA2*
     power2(mA2 - mC2)) - (12*DeltaInv(mA2,mstau22,mu2)*power3(mu2))/mstau22 +
     (3*power4(mA2))/(power2(mA2 - mH2)*power2(mH2))))/32. + (log(mC2/Q2)*(-18
     + (3*mC2)/(mA2 - mC2) - (3*mC2)/(mC2 - mH2) + (21*mC2)/mH2 + (8*mstau22)/
     mC2 + (8*mC2)/(msntau2 - mu2) + (8*mC2)/(mstau22 - mu2) - (8*mu2)/mC2 + (
     24*mu2)/mstau22 + (6*mu2)/(msntau2 - mu2) + (6*mu2)/(mstau12 - mu2) + (2*
     mu2)/(mstau22 - mu2) + (3*power2(mC2)*(9 + power2(mC2)/power2(mA2 - mC2)))
     /power2(mA2) - (21*power2(mC2))/power2(mH2) - (16*mC2*mu2)/power2(msntau2
     - mu2) - (16*mC2*mu2)/power2(mstau22 - mu2) - (24*mu2*DeltaInv(mstau22,
     mC2,mu2)*(-3*mC2*mstau22 - 2*mC2*mu2 - mstau22*mu2 + power2(mC2) + power2(
     mu2)))/mstau22 - (3*power3(mC2))/(mH2*power2(mC2 - mH2)) + (-11*mC2 - (3*
     power3(mC2))/power2(mA2 - mC2))/mA2 + (3*power4(mC2))/(power2(mC2 - mH2)*
     power2(mH2))))/32. - (3*mh2*mu2*log(mh2/Q2)*(-3*mstau22*mu2*power2(
     mstau12) + 2*mstau22*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mstau12) + 2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(
     mstau12) - 5*mstau12*mu2*power2(mstau22) + 4*mstau12*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mstau22) + 4*mstau12*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mstau22) + power2(mstau12)*power2(mstau22)
     - mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power2(mstau22) -
     16*mh2*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power2(mstau22) -
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mstau12)*power2(mstau22)
     + 9*mstau12*mstau22*power2(mu2) - 5*mstau12*mstau22*mstau24*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2) - 5*mstau12*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2) + 15*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mstau12)*power2(mu2) + power2(mstau22)*power2(mu2) +
     17*mh2*mstau12*DeltaInv(mstau22,mstau12,mh2)*power2(mstau22)*power2(mu2) -
     mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mstau22)*power2(mu2) -
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mstau22)*power2(mu2) + 3*
     DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power2(mstau22)*power2(mu2)
     + mstau22*mu2*DeltaInv(mstau12,mh2,mu2)*power2(mstau12 - mu2)*(-3*mh2*
     mstau12 - 2*mh2*mu2 - mstau12*mu2 + power2(mh2) + power2(mu2)) + mstau12*
     mu2*DeltaInv(mstau22,mh2,mu2)*power2(mstau22 - mu2)*(-3*mh2*mstau22 - 2*
     mh2*mu2 - mstau22*mu2 + power2(mh2) + power2(mu2)) - 6*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mstau12) + 3*mh2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mstau22)*power3(mstau12) + 2*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mstau22)*power3(mstau12) - 2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2)*power3(mstau12) + mstau12*power3(mstau22) -
     mstau12*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(mstau22) - 10*mh2*
     mstau12*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mstau22) - mstau12*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power3(mstau22) + 5*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power3(mstau22) - 6*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power3(mstau22) + 2*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2)*power3(mstau22) + 5*mstau12*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2)*power3(mstau22) - DeltaInv(
     mstau22,mstau12,mh2)*power3(mstau12)*power3(mstau22) - mstau12*power3(mu2)
     - 3*mstau22*power3(mu2) - 8*mh2*mstau12*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power3(mu2) + 2*mstau22*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2) + 2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power3(mu2) +
     2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power3(mu2) - 4*
     mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mstau22)*power3(mu2) - 4*mstau12*
     DeltaInv(mstau22,mstau12,mh2)*power2(mstau22)*power3(mu2) + 2*mh2*mstau12*
     DeltaInv(mstau22,mstau12,mh2)*power4(mstau22) - 2*mstau12*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power4(mstau22) + 3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mstau12)*power4(mstau22)))/(8.*mstau12*mstau22*power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + (mh2*mu2*power2(log(mh2/Q2))*(-3*mstau22*
     mu2*power2(mstau12) + 2*mstau22*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mstau12) + 2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mstau12) - 5*mstau12*mu2*power2(mstau22) + 4*mstau12*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mstau22) + 4*mstau12*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2)*power2(mstau22) + power2(mstau12)*power2(
     mstau22) - mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power2(
     mstau22) - 16*mh2*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*
     power2(mstau22) - DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(
     mstau12)*power2(mstau22) + 9*mstau12*mstau22*power2(mu2) - 5*mstau12*
     mstau22*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2) - 5*mstau12*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2) + 15*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power2(mu2) +
     power2(mstau22)*power2(mu2) + 17*mh2*mstau12*DeltaInv(mstau22,mstau12,mh2)
     *power2(mstau22)*power2(mu2) - mstau24*DeltaInv(mstau22,mstau12,mh2)*
     power2(mstau22)*power2(mu2) - DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mstau22)*power2(mu2) + 3*DeltaInv(mstau22,mstau12,mh2)*power2(
     mstau12)*power2(mstau22)*power2(mu2) + mstau22*mu2*DeltaInv(mstau12,mh2,
     mu2)*power2(mstau12 - mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)) + mstau12*mu2*DeltaInv(mstau22,mh2,mu2)*power2(
     mstau22 - mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 + power2(mh2) +
     power2(mu2)) - 6*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(
     mstau12) + 3*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mstau22)*power3(
     mstau12) + 2*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mstau22)*power3(
     mstau12) - 2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)*power3(
     mstau12) + mstau12*power3(mstau22) - mstau12*mstau24*DeltaInv(mstau22,
     mstau12,mh2)*power3(mstau22) - 10*mh2*mstau12*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power3(mstau22) - mstau12*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2)*power3(mstau22) + 5*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mstau12)*power3(mstau22) - 6*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mstau12)*power3(mstau22) + 2*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)
     *power3(mstau22) + 5*mstau12*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)*
     power3(mstau22) - DeltaInv(mstau22,mstau12,mh2)*power3(mstau12)*power3(
     mstau22) - mstau12*power3(mu2) - 3*mstau22*power3(mu2) - 8*mh2*mstau12*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2) + 2*mstau22*mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2) + 2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power3(mu2) + 2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mstau12)*power3(mu2) - 4*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mstau22)*power3(mu2) - 4*mstau12*DeltaInv(mstau22,mstau12,mh2)*
     power2(mstau22)*power3(mu2) + 2*mh2*mstau12*DeltaInv(mstau22,mstau12,mh2)*
     power4(mstau22) - 2*mstau12*mu2*DeltaInv(mstau22,mstau12,mh2)*power4(
     mstau22) + 3*DeltaInv(mstau22,mstau12,mh2)*power2(mstau12)*power4(mstau22)
     ))/(16.*mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (
     mu2*power2(sb)*power2(log(mC2/Q2))*(-(1/mC2) - 1/msntau2 + mC2/(msntau2*(-
     mstau22 + mu2)) - mC2/power2(msntau2 - mu2) + (mC2*mu2)/(mstau22*power2(
     msntau2 - mu2)) + (3*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(
     msntau2 - mu2) + (2*mC2*mu2)/(msntau2*power2(mstau22 - mu2)) + (mC2*
     DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (2*
     DeltaInv(mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2
     - mu2)) - (mC2*mu2*DeltaInv(mC2,msntau2,mu2)*(-3*mC2*msntau2 - 2*mC2*mu2 -
     msntau2*mu2 + power2(mC2) + power2(mu2)))/(msntau2*power2(mstau22 - mu2))
     - (mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 -
     mu2)) - (mC2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(
     msntau2 - mu2)) + (DeltaInv(mstau22,mC2,msntau2)*(-((3*msntau2*mstau22 -
     mstau24)*power2(msntau2 - mu2)*power2(mstau22 - mu2)) + ((mstau22 - 2*mu2)
     *power2(msntau2) + (mstau22 - 2*mu2)*power2(mu2) + msntau2*(-4*mstau22*mu2
     + power2(mstau22) + 5*power2(mu2)))*power3(mC2) + power2(mC2)*(power2(mu2)
     *(2*mstau22*mu2 - power2(mstau22) + power2(mu2)) - 2*power2(msntau2)*(-7*
     mstau22*mu2 + 2*power2(mstau22) + 7*power2(mu2)) - 3*(mstau22 - 2*mu2)*
     power3(msntau2) + msntau2*(8*mu2*power2(mstau22) - 13*mstau22*power2(mu2)
     - 2*power3(mstau22) + 6*power3(mu2))) + mC2*(power2(mu2)*power3(msntau2) +
     power2(msntau2)*(-2*mstau24*mu2 + 12*mu2*power2(mstau22) + mstau22*(
     mstau24 - 9*power2(mu2)) - 5*power3(mstau22)) + power2(mu2)*(-2*mstau24*
     mu2 + 4*mu2*power2(mstau22) + mstau22*(mstau24 - 2*power2(mu2)) - 2*
     power3(mstau22)) + msntau2*(power2(mstau22)*(mstau24 - 14*power2(mu2)) +
     5*mstau24*power2(mu2) + 6*mu2*power3(mstau22) + mstau22*(-4*mstau24*mu2 +
     10*power3(mu2)) - power4(mu2)))))/(msntau2*power2(msntau2 - mu2)*power2(
     mstau22 - mu2))))/(8.*power2(cb)) + (mu2*log(mC2/Q2)*power2(sb)*(-2/mC2 -
     3/msntau2 - (3*mC2)/(msntau2*mstau22 - msntau2*mu2) - (3*mC2)/power2(
     msntau2 - mu2) + (3*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (9*mu2*
     DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (6*mC2*mu2)
     /(msntau2*power2(mstau22 - mu2)) + (3*mC2*DeltaInv(mstau22,mC2,mu2)*
     power2(mu2))/power2(msntau2 - mu2) + (6*DeltaInv(mstau22,mC2,mu2)*power2(
     mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (3*mC2*mu2*DeltaInv(
     mC2,msntau2,mu2)*(-3*mC2*msntau2 - 2*mC2*mu2 - msntau2*mu2 + power2(mC2) +
     power2(mu2)))/(msntau2*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,
     mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 - mu2)) - (3*mC2*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) + (3*
     DeltaInv(mstau22,mC2,msntau2)*(-((3*msntau2*mstau22 - mstau24)*power2(
     msntau2 - mu2)*power2(mstau22 - mu2)) + ((mstau22 - 2*mu2)*power2(msntau2)
     + (mstau22 - 2*mu2)*power2(mu2) + msntau2*(-4*mstau22*mu2 + power2(
     mstau22) + 5*power2(mu2)))*power3(mC2) + power2(mC2)*(power2(mu2)*(2*
     mstau22*mu2 - power2(mstau22) + power2(mu2)) - 2*power2(msntau2)*(-7*
     mstau22*mu2 + 2*power2(mstau22) + 7*power2(mu2)) - 3*(mstau22 - 2*mu2)*
     power3(msntau2) + msntau2*(8*mu2*power2(mstau22) - 13*mstau22*power2(mu2)
     - 2*power3(mstau22) + 6*power3(mu2))) + mC2*(power2(mu2)*power3(msntau2) +
     power2(msntau2)*(-2*mstau24*mu2 + 12*mu2*power2(mstau22) + mstau22*(
     mstau24 - 9*power2(mu2)) - 5*power3(mstau22)) + power2(mu2)*(-2*mstau24*
     mu2 + 4*mu2*power2(mstau22) + mstau22*(mstau24 - 2*power2(mu2)) - 2*
     power3(mstau22)) + msntau2*(power2(mstau22)*(mstau24 - 14*power2(mu2)) +
     5*mstau24*power2(mu2) + 6*mu2*power3(mstau22) + mstau22*(-4*mstau24*mu2 +
     10*power3(mu2)) - power4(mu2)))))/(msntau2*power2(msntau2 - mu2)*power2(
     mstau22 - mu2))))/(4.*power2(cb)) + (mu2*log(mstau22/Q2)*log(mu2/Q2)*
     power2(sa)*(mH2*mstau22*mu2*power2(mh2) - mh2*mstau22*mu2*power2(mH2) + 6*
     mh2*mstau12*mu2*power2(mstau22) - 6*mH2*mstau12*mu2*power2(mstau22) - 3*
     mh2*power2(mstau12)*power2(mstau22) + 3*mH2*power2(mstau12)*power2(
     mstau22) - mH2*power2(mh2)*power2(mu2) + mh2*power2(mH2)*power2(mu2) -
     mh2*power2(mstau12)*power2(mu2) + mH2*power2(mstau12)*power2(mu2) - 3*mh2*
     power2(mstau22)*power2(mu2) + 3*mH2*power2(mstau22)*power2(mu2) - mH2*
     DeltaInv(mstau22,mh2,mu2)*(-2*mstau12*mu2 + mu2*(-mh2 + mu2) + power2(
     mstau12))*(power2(mh2)*(power2(mstau22) - power2(mu2)) + power2(mstau22 -
     mu2)*power2(mu2) - mh2*mu2*(5*mstau22*mu2 - 6*power2(mstau22) + power2(
     mu2)) + (-mstau22 + mu2)*power3(mh2)) + mh2*DeltaInv(mstau22,mH2,mu2)*(-2*
     mstau12*mu2 + mu2*(-mH2 + mu2) + power2(mstau12))*(power2(mH2)*(power2(
     mstau22) - power2(mu2)) + power2(mstau22 - mu2)*power2(mu2) - mH2*mu2*(5*
     mstau22*mu2 - 6*power2(mstau22) + power2(mu2)) + (-mstau22 + mu2)*power3(
     mH2)) + 2*mh2*mstau12*power3(mu2) - 2*mH2*mstau12*power3(mu2) - mh2*
     power4(mu2) + mH2*power4(mu2)))/(8.*mh2*mH2*mstau22*(mstau22 - mu2)*
     power2(mstau12 - mu2)) + (mu2*log(mstau12/Q2)*log(mu2/Q2)*power2(sa)*(mH2*
     mstau12*mu2*power2(mh2) - mh2*mstau12*mu2*power2(mH2) + 6*mh2*mstau22*mu2*
     power2(mstau12) - 6*mH2*mstau22*mu2*power2(mstau12) - 3*mh2*power2(
     mstau12)*power2(mstau22) + 3*mH2*power2(mstau12)*power2(mstau22) - mH2*
     power2(mh2)*power2(mu2) + mh2*power2(mH2)*power2(mu2) - 3*mh2*power2(
     mstau12)*power2(mu2) + 3*mH2*power2(mstau12)*power2(mu2) - mh2*power2(
     mstau22)*power2(mu2) + mH2*power2(mstau22)*power2(mu2) - mH2*DeltaInv(
     mstau12,mh2,mu2)*(-2*mstau22*mu2 + mu2*(-mh2 + mu2) + power2(mstau22))*(
     power2(mh2)*(power2(mstau12) - power2(mu2)) + power2(mstau12 - mu2)*
     power2(mu2) - mh2*mu2*(5*mstau12*mu2 - 6*power2(mstau12) + power2(mu2)) +
     (-mstau12 + mu2)*power3(mh2)) + mh2*DeltaInv(mH2,mstau12,mu2)*(-2*mstau22*
     mu2 + mu2*(-mH2 + mu2) + power2(mstau22))*(power2(mH2)*(power2(mstau12) -
     power2(mu2)) + power2(mstau12 - mu2)*power2(mu2) - mH2*mu2*(5*mstau12*mu2
     - 6*power2(mstau12) + power2(mu2)) + (-mstau12 + mu2)*power3(mH2)) + 2*
     mh2*mstau22*power3(mu2) - 2*mH2*mstau22*power3(mu2) - mh2*power4(mu2) +
     mH2*power4(mu2)))/(8.*mh2*mH2*mstau12*(mstau12 - mu2)*power2(mstau22 -
     mu2)) - (log(mh2/Q2)*power2(sa)*(24 + (3*mA2)/(mA2 - mh2) + (3*mC2)/(mC2 -
     mh2) + (9*mA2)/mh2 + (9*mC2)/mh2 - (2*mh2)/mA2 - (6*mh2)/mH2 + (9*mH2)/mh2
     + (45*mH2)/(mh2 - mH2) + (6*msntau2)/mh2 + (6*mstau12)/mh2 + (24*mstau12)/
     (mh2 - mH2) + (6*mstau22)/mh2 + (24*mstau22)/(mh2 - mH2) + (6*mu2)/mh2 + (
     24*mu2)/(mh2 - mH2) - (6*mu2)/msntau2 + (30*mu2)/mstau12 - (6*mu2)/mstau22
     + (4*mu2)/(-msntau2 + mu2) + (2*mh2)/(-mstau12 + mu2) + (4*mu2)/(-mstau12
     + mu2) + (2*mh2)/(-mstau22 + mu2) + (4*mu2)/(-mstau22 + mu2) - (6*mh2*mu2)
     /(mstau12*mstau22 - mstau12*mu2) - 18*mh2*mu2*DeltaInv(mstau22,mh2,mu2) +
     54*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) + 108*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2) + (72*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 - (36*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (18*
     mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (12*mh2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (6*mh2*
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (3*power2(mA2))/power2(mA2 - mh2) + (3*power2(mC2))/power2(mC2 - mh2) +
     (6*mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/mstau22 - (36*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (18*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (18*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (12*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) - (15*
     power2(mh2))/power2(mA2) + (9*power2(mh2))/power2(mH2) + (3*power2(mH2))/
     power2(mh2 - mH2) - (2*mh2*mu2)/power2(mstau12 - mu2) + (6*mh2*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (12*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (4*
     mh2*mu2)/power2(mstau22 - mu2) - 6*DeltaInv(mstau22,mh2,mu2)*power2(mu2) -
     (12*mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/mstau22 + (12*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/(mstau12 - mu2) + (6*mh2*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) - (6*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau12 - mu2) + (18*DeltaInv(mstau22,mh2,mu2)*
     power2(mh2)*power2(mu2))/power2(mstau12 - mu2) - (6*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/power2(mstau12 - mu2) + (12*mh2*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (12*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (6*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (18*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/power2(
     mstau22 - mu2) + (12*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (6*mu2*DeltaInv(msntau2,
     mh2,mu2)*(-3*mh2*msntau2 - 2*mh2*mu2 - msntau2*mu2 + power2(mh2) + power2(
     mu2)))/msntau2 + (6*mu2*DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*
     mu2 - mstau12*mu2 + power2(mh2) + power2(mu2))*(-(mh2*mu2) - 2*mstau22*mu2
     + power2(mstau22) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (3*
     power3(mA2))/(mh2*power2(mA2 - mh2)) - (3*power3(mC2))/(mh2*power2(mC2 -
     mh2)) + (6*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22
     - mstau12*mu2) + (6*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(
     mstau12 - mu2) - (6*DeltaInv(mstau22,mh2,mu2)*power2(mu2)*power3(mh2))/(
     mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(mstau22,mstau12,mh2)*power2(
     mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) - (3*power3(mH2))/(mh2*
     power2(mh2 - mH2)) + (6*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/mstau22 + (
     6*mh2*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (12*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau22*power2(mstau12
     - mu2)) - (6*mh2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22
     - mu2) - (6*mh2*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(mstau22*power2(
     mstau12 - mu2))))/16. + (power2(sa)*power2(log(mh2/Q2))*(-6 + mA2/(mA2 -
     mh2) + mC2/(mC2 - mh2) + (4*mA2)/mh2 + (4*mC2)/mh2 + (7*mh2)/mA2 - (9*mh2)
     /mH2 + (4*mH2)/mh2 + (23*mH2)/(mh2 - mH2) + (3*msntau2)/mh2 + (3*mstau12)/
     mh2 + (3*mstau22)/mh2 + (3*mu2)/mh2 + (24*mu2)/(mh2 - mH2) - (2*mu2)/
     msntau2 + (10*mu2)/mstau12 - (2*mu2)/mstau22 - (2*mh2*mu2)/(mstau12*
     mstau22 - mstau12*mu2) - 6*mh2*mu2*DeltaInv(mstau22,mh2,mu2) + 18*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2) + 36*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2) + (24*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 - (12*
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (4*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (2*mh2*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + power2(
     mA2)/power2(mA2 - mh2) + power2(mC2)/power2(mC2 - mh2) + (2*mu2*DeltaInv(
     mstau22,mh2,mu2)*power2(mh2))/mstau22 - (12*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/mstau12 + (6*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(-mstau12 + mu2) + (6*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))
     /(-mstau22 + mu2) - (4*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau12*mstau22 - mstau12*mu2) - (10*power2(mh2))/power2(mA2) + (6*
     power2(mh2))/power2(mH2) + power2(mH2)/power2(mh2 - mH2) - (2*mh2*mu2)/
     power2(mstau12 - mu2) + (2*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) - (4*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau12 - mu2) - 2*DeltaInv(mstau22,mh2,mu2)*power2(
     mu2) - (4*mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/mstau22 + (4*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12 - mu2) + (2*mh2*
     power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (2*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) + (6*DeltaInv(
     mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/power2(mstau12 - mu2) - (2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/power2(mstau12 -
     mu2) + (4*mh2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (4*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) -
     (2*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (6*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2)
     )/power2(mstau22 - mu2) + (4*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (2*mu2*DeltaInv(
     msntau2,mh2,mu2)*(-3*mh2*msntau2 - 2*mh2*mu2 - msntau2*mu2 + power2(mh2) +
     power2(mu2)))/msntau2 + (2*mu2*DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 -
     2*mh2*mu2 - mstau12*mu2 + power2(mh2) + power2(mu2))*(-(mh2*mu2) - 2*
     mstau22*mu2 + power2(mstau22) + power2(mu2)))/(mstau12*power2(mstau22 -
     mu2)) - power3(mA2)/(mh2*power2(mA2 - mh2)) - power3(mC2)/(mh2*power2(mC2
     - mh2)) + (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*
     mstau22 - mstau12*mu2) + (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))
     /power2(mstau12 - mu2) - (2*DeltaInv(mstau22,mh2,mu2)*power2(mu2)*power3(
     mh2))/(mstau22*power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) - power3(mH2)/(
     mh2*power2(mh2 - mH2)) + (2*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/mstau22
     + (2*mh2*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (
     4*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau22*power2(
     mstau12 - mu2)) - (2*mh2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/
     power2(mstau22 - mu2) - (2*mh2*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2))))/32. + (power2(sa)*power2(log(mH2/Q2))*(-
     54 - mC2/(mC2 - mH2) - (4*mA2)/mH2 - (4*mC2)/mH2 + (3*mh2)/mH2 - (7*mH2)/
     mA2 - (48*mH2)/mh2 + (15*mH2)/(mh2 - mH2) + mA2/(-mA2 + mH2) - (3*msntau2)
     /mH2 - (24*mstau12)/mh2 + (24*mstau12)/(mh2 - mH2) - (3*mstau12)/mH2 - (
     24*mstau22)/mh2 + (24*mstau22)/(mh2 - mH2) - (3*mstau22)/mH2 + (21*mu2)/
     mH2 + (24*mu2)/(-mh2 + mH2) + (2*mu2)/msntau2 + (14*mu2)/mstau12 + (2*mu2)
     /mstau22 + (2*mH2*mu2)/(mstau12*mstau22 - mstau12*mu2) + 6*mH2*mu2*
     DeltaInv(mstau22,mH2,mstau12) + 36*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12) + (24*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (
     12*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (6*mH2*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (4*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau22 - mu2) - (2*mH2*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + 6*mH2*
     mu2*DeltaInv(mstau22,mH2,mu2) + (8*mH2*mstau12)/power2(mh2) + (8*mH2*
     mstau22)/power2(mh2) - (64*mH2*mstau12)/(-(mh2*mH2) + power2(mh2)) - (64*
     mH2*mstau22)/(-(mh2*mH2) + power2(mh2)) + (32*mH2*mu2)/(-(mh2*mH2) +
     power2(mh2)) - power2(mA2)/power2(mA2 - mH2) - power2(mC2)/power2(mC2 -
     mH2) + (32*mH2*mstau12)/power2(mh2 - mH2) + (32*mH2*mstau22)/power2(mh2 -
     mH2) - (32*mH2*mu2)/power2(mh2 - mH2) - (12*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/mstau12 + (6*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12 - mu2) + (6*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau22 - mu2) + (4*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) - (2*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/mstau22 + (10*power2(mH2))/power2(mA2) + (
     18*power2(mH2))/power2(mh2) + (16*mstau12*power2(mH2))/((mh2 - mH2)*
     power2(mh2)) + (16*mstau22*power2(mH2))/((mh2 - mH2)*power2(mh2)) + (8*
     mu2*power2(mH2))/((-mh2 + mH2)*power2(mh2)) - (52*power2(mH2))/(-(mh2*mH2)
     + power2(mh2)) + power2(mH2)/power2(mh2 - mH2) - (40*mstau12*power2(mH2))/
     (mh2*power2(mh2 - mH2)) - (40*mstau22*power2(mH2))/(mh2*power2(mh2 - mH2))
     + (40*mu2*power2(mH2))/(mh2*power2(mh2 - mH2)) + (2*mH2*mu2)/power2(
     mstau12 - mu2) - (2*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(
     mstau12 - mu2) + (4*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))
     /power2(mstau12 - mu2) + (4*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))
     /(-mstau12 + mu2) + 2*DeltaInv(mstau22,mH2,mu2)*power2(mu2) + (4*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 - (2*mH2*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (2*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau12 - mu2) + (2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/power2(mstau12 - mu2) - (6*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/power2(mstau12 - mu2) - (4*mH2*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (4*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (2*mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (6*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/power2(mstau22
     - mu2) - (4*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (2*mu2*DeltaInv(mH2,msntau2,mu2)*(-3*
     mH2*msntau2 - 2*mH2*mu2 - msntau2*mu2 + power2(mH2) + power2(mu2)))/
     msntau2 - (2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 -
     mstau12*mu2 + power2(mH2) + power2(mu2))*(-(mH2*mu2) - 2*mstau22*mu2 +
     power2(mstau22) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + power3(
     mA2)/(mH2*power2(mA2 - mH2)) + power3(mC2)/(mH2*power2(mC2 - mH2)) - (2*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 - mstau12*
     mu2) + (12*power3(mH2))/((mh2 - mH2)*power2(mh2)) - power3(mH2)/(mh2*
     power2(mh2 - mH2)) + (8*mstau12*power3(mH2))/(power2(mh2)*power2(mh2 -
     mH2)) + (8*mstau22*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (8*mu2*
     power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (2*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (2*DeltaInv(mstau22,mH2,
     mu2)*power2(mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + (2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) - (2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mstau22 - (2*
     mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (4*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power3(mu2))/(mstau22*power2(mstau12
     - mu2)) + (2*mH2*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22
     - mu2) + (2*mH2*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(
     mstau12 - mu2))))/32. - (log(mH2/Q2)*power2(sa)*(-36 - (6*mC2)/(mC2 - mH2)
     - (18*mA2)/mH2 - (18*mC2)/mH2 + (12*mh2)/mH2 + (4*mH2)/mA2 - (140*mH2)/mh2
     + (32*mH2)/(mh2 - mH2) + (6*mA2)/(-mA2 + mH2) - (12*msntau2)/mH2 - (88*
     mstau12)/mh2 + (40*mstau12)/(mh2 - mH2) - (12*mstau12)/mH2 - (88*mstau22)/
     mh2 + (40*mstau22)/(mh2 - mH2) - (12*mstau22)/mH2 + (4*mH2)/(mstau12 -
     mu2) + (4*mH2)/(mstau22 - mu2) - (8*mu2)/mh2 + (84*mu2)/mH2 + (40*mu2)/(-
     mh2 + mH2) + (12*mu2)/msntau2 + (84*mu2)/mstau12 + (12*mu2)/mstau22 + (8*
     mu2)/(msntau2 - mu2) + (8*mu2)/(mstau12 - mu2) + (8*mu2)/(mstau22 - mu2) +
     (12*mH2*mu2)/(mstau12*mstau22 - mstau12*mu2) + 36*mH2*mu2*DeltaInv(
     mstau22,mH2,mstau12) + 216*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12) + (
     144*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (72*mstau24*
     mu2*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (36*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (24*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau22 - mu2) - (12*mH2*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + 36*mH2*mu2*
     DeltaInv(mstau22,mH2,mu2) + (40*mH2*mstau12)/power2(mh2) + (40*mH2*
     mstau22)/power2(mh2) - (8*mH2*mu2)/power2(mh2) - (192*mH2*mstau12)/(-(mh2*
     mH2) + power2(mh2)) - (192*mH2*mstau22)/(-(mh2*mH2) + power2(mh2)) + (64*
     mH2*mu2)/(-(mh2*mH2) + power2(mh2)) - (6*power2(mA2))/power2(mA2 - mH2) -
     (6*power2(mC2))/power2(mC2 - mH2) + (64*mH2*mstau12)/power2(mh2 - mH2) + (
     64*mH2*mstau22)/power2(mh2 - mH2) - (64*mH2*mu2)/power2(mh2 - mH2) - (72*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (36*mu2*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (36*mu2*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (24*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2)
     - (12*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/mstau22 + (30*power2(mH2)
     )/power2(mA2) + (65*power2(mH2))/power2(mh2) + (64*mstau12*power2(mH2))/((
     mh2 - mH2)*power2(mh2)) + (64*mstau22*power2(mH2))/((mh2 - mH2)*power2(
     mh2)) + (32*mu2*power2(mH2))/((-mh2 + mH2)*power2(mh2)) - (172*power2(mH2)
     )/(-(mh2*mH2) + power2(mh2)) - (3*power2(mH2))/power2(mh2 - mH2) - (88*
     mstau12*power2(mH2))/(mh2*power2(mh2 - mH2)) - (88*mstau22*power2(mH2))/(
     mh2*power2(mh2 - mH2)) + (88*mu2*power2(mH2))/(mh2*power2(mh2 - mH2)) + (
     4*mH2*mu2)/power2(mstau12 - mu2) - (12*mH2*mstau24*mu2*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau12 - mu2) + (24*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (8*mH2*mu2)/power2(
     mstau22 - mu2) + (24*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-
     mstau12 + mu2) + 12*DeltaInv(mstau22,mH2,mu2)*power2(mu2) + (24*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 - (12*mH2*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (12*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau12 - mu2) + (12*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/power2(mstau12 - mu2) - (36*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/power2(mstau12 - mu2) - (24*mH2*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (24*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (12*mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (36*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/power2(
     mstau22 - mu2) - (24*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (12*mu2*DeltaInv(mH2,
     msntau2,mu2)*(-3*mH2*msntau2 - 2*mH2*mu2 - msntau2*mu2 + power2(mH2) +
     power2(mu2)))/msntau2 - (12*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12
     - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2))*(-(mH2*mu2) - 2*
     mstau22*mu2 + power2(mstau22) + power2(mu2)))/(mstau12*power2(mstau22 -
     mu2)) + (6*power3(mA2))/(mH2*power2(mA2 - mH2)) + (6*power3(mC2))/(mH2*
     power2(mC2 - mH2)) - (12*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(
     mstau12*mstau22 - mstau12*mu2) + (44*power3(mH2))/((mh2 - mH2)*power2(mh2)
     ) + (6*power3(mH2))/(mh2*power2(mh2 - mH2)) + (24*mstau12*power3(mH2))/(
     power2(mh2)*power2(mh2 - mH2)) + (24*mstau22*power3(mH2))/(power2(mh2)*
     power2(mh2 - mH2)) - (24*mu2*power3(mH2))/(power2(mh2)*power2(mh2 - mH2))
     - (12*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2)
     + (12*DeltaInv(mstau22,mH2,mu2)*power2(mu2)*power3(mH2))/(mstau22*power2(
     mstau12 - mu2)) + (12*DeltaInv(mstau22,mH2,mstau12)*power2(mu2)*power3(
     mH2))/(mstau12*power2(mstau22 - mu2)) - (12*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/mstau22 - (12*mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/
     power2(mstau12 - mu2) - (24*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power3(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (12*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mu2))/power2(mstau22 - mu2) - (3*power4(mH2))/(power2(mh2)
     *power2(mh2 - mH2)) + (12*mH2*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2))))/32. + (mu2*log(msntau2/Q2)*log(mu2/Q2)*(-
     10/mA2 + 6/mH2 - 4/msntau2 - (2*mu2)/(mA2*msntau2) - (2*mu2)/(mH2*msntau2)
     + 2/(-msntau2 + mu2) - (8*mu2)/(mA2*msntau2 - mA2*mu2) + (8*mu2)/(mH2*
     msntau2 - mH2*mu2) + (13*mu2)/power2(msntau2 - mu2) + (3*mu2)/power2(
     mstau12 - mu2) - (3*mu2)/power2(mstau22 - mu2) - (4*mu2*mw2)/(msntau2*
     power2(mstau22 - mu2)) - (4*power2(mu2))/(msntau2*power2(mstau22 - mu2)) -
     (24*mw2*DeltaInv(msntau2,mw2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (4*
     mu2*DeltaInv(msntau2,mw2,mu2)*power2(mw2))/power2(mstau22 - mu2) - (4*
     DeltaInv(msntau2,mw2,mu2)*power2(mu2)*power2(mw2))/(msntau2*power2(mstau22
     - mu2)) + (2*DeltaInv(mA2,msntau2,mu2)*(-(mA2*mu2*(6*msntau2 + mu2)) - (
     msntau2 + mu2)*power2(mA2) + (-msntau2 + mu2)*power2(mu2) + power3(mA2)))/
     (mA2*msntau2) + (2*DeltaInv(mH2,msntau2,mu2)*(-(mH2*mu2*(6*msntau2 + mu2))
     - (msntau2 + mu2)*power2(mH2) + (-msntau2 + mu2)*power2(mu2) + power3(mH2)
     ))/(mH2*msntau2) - (3*power3(mu2))/(power2(msntau2 - mu2)*power2(mstau12 -
     mu2)) - (4*DeltaInv(msntau2,mw2,mu2)*power3(mu2))/power2(mstau22 - mu2) -
     (4*mw2*DeltaInv(msntau2,mw2,mu2)*power3(mu2))/(msntau2*power2(mstau22 -
     mu2)) - power3(mu2)/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (4*
     power2(mu2))/power3(-msntau2 + mu2) + (8*msntau2*mu2)/power3(-mstau22 +
     mu2) + (4*mu2*DeltaInv(msntau2,mw2,mu2)*power3(mw2))/(msntau2*power2(
     mstau22 - mu2)) - (3*power3(mu2))/power4(msntau2 - mu2) + (4*DeltaInv(
     msntau2,mw2,mu2)*power4(mu2))/(msntau2*power2(mstau22 - mu2))))/16. - (
     mu2*log(mu2/Q2)*power2(sa)*(9/mh2 - 9/mH2 - (3*mu2)/(mh2*msntau2) + (3*
     mu2)/(mH2*msntau2) - (3*mu2)/(mh2*mstau12) + (3*mu2)/(mH2*mstau12) - (3*
     mu2)/(mh2*mstau22) + (3*mu2)/(mH2*mstau22) + (12*mu2)/(mh2*msntau2 - mh2*
     mu2) + (12*mu2)/(mh2*mstau12 - mh2*mu2) + (12*mu2)/(mh2*mstau22 - mh2*mu2)
     - (12*mu2)/(mH2*msntau2 - mH2*mu2) - (12*mu2)/(mH2*mstau12 - mH2*mu2) - (
     12*mu2)/(mH2*mstau22 - mH2*mu2) - 9*mu2*DeltaInv(msntau2,mh2,mu2) + (3*
     mh2*mu2*DeltaInv(msntau2,mh2,mu2))/msntau2 - 9*mu2*DeltaInv(mstau12,mh2,
     mu2) + (3*mh2*mu2*DeltaInv(mstau12,mh2,mu2))/mstau12 - 9*mu2*DeltaInv(
     mstau22,mh2,mu2) + (3*mh2*mu2*DeltaInv(mstau22,mh2,mu2))/mstau22 + 9*mu2*
     DeltaInv(mstau22,mH2,mu2) - (3*mH2*mu2*DeltaInv(mstau22,mH2,mu2))/mstau22
     - (3*DeltaInv(msntau2,mh2,mu2)*power2(mu2))/mh2 - (6*DeltaInv(msntau2,mh2,
     mu2)*power2(mu2))/msntau2 - (3*DeltaInv(mstau12,mh2,mu2)*power2(mu2))/mh2
     - (6*DeltaInv(mstau12,mh2,mu2)*power2(mu2))/mstau12 - (3*DeltaInv(mstau22,
     mh2,mu2)*power2(mu2))/mh2 - (6*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/
     mstau22 + (3*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mH2 + (6*DeltaInv(
     mstau22,mH2,mu2)*power2(mu2))/mstau22 + (9*mh2*DeltaInv(mstau22,mh2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (9*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (3*DeltaInv(mstau22,mh2,mu2)*power2(
     mh2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (9*
     mh2*DeltaInv(mstau12,mh2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (3*
     DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) - (3*mu2*DeltaInv(mH2,msntau2,mu2)*(-3*mH2*msntau2 - 2*mH2*mu2 -
     msntau2*mu2 + power2(mH2) + power2(mu2)))/(mH2*msntau2) - (3*mu2*DeltaInv(
     mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2))*(-(mH2*mu2) - 2*mstau22*mu2 + power2(mstau22) + power2(mu2)))
     /(mH2*mstau12*power2(mstau22 - mu2)) + (2*mH2*mu2)/power3(mstau12 - mu2) +
     (2*mH2*mu2)/power3(mstau22 - mu2) + (3*DeltaInv(msntau2,mh2,mu2)*power3(
     mu2))/(mh2*msntau2) + (3*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/(mh2*
     mstau12) + (3*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/(mh2*mstau22) - (3*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mH2*mstau22) + (3*DeltaInv(
     mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (6*mh2*DeltaInv(
     mstau22,mh2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (6*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     3*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/power2(mstau22 - mu2) + (6*mh2*
     DeltaInv(mstau12,mh2,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     2*mh2*mu2)/power3(-mstau12 + mu2) + (2*mh2*mu2)/power3(-mstau22 + mu2) - (
     3*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) +
     (3*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2))
     - (3*DeltaInv(mstau12,mh2,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)
     )))/8. + (mu2*power2(sa)*power2(log(mu2/Q2))*(9/mh2 - 9/mH2 - (2*mu2)/(
     mh2*msntau2) + (2*mu2)/(mH2*msntau2) - (2*mu2)/(mh2*mstau12) + (2*mu2)/(
     mH2*mstau12) - (2*mu2)/(mh2*mstau22) + (2*mu2)/(mH2*mstau22) + (8*mu2)/(
     mh2*msntau2 - mh2*mu2) + (8*mu2)/(mh2*mstau12 - mh2*mu2) + (8*mu2)/(mh2*
     mstau22 - mh2*mu2) - (8*mu2)/(mH2*msntau2 - mH2*mu2) - (8*mu2)/(mH2*
     mstau12 - mH2*mu2) - (8*mu2)/(mH2*mstau22 - mH2*mu2) - 6*mu2*DeltaInv(
     msntau2,mh2,mu2) + (2*mh2*mu2*DeltaInv(msntau2,mh2,mu2))/msntau2 - 6*mu2*
     DeltaInv(mstau12,mh2,mu2) + (2*mh2*mu2*DeltaInv(mstau12,mh2,mu2))/mstau12
     - 6*mu2*DeltaInv(mstau22,mh2,mu2) + (2*mh2*mu2*DeltaInv(mstau22,mh2,mu2))/
     mstau22 + 6*mu2*DeltaInv(mstau22,mH2,mu2) - (2*mH2*mu2*DeltaInv(mstau22,
     mH2,mu2))/mstau22 - (2*DeltaInv(msntau2,mh2,mu2)*power2(mu2))/mh2 - (4*
     DeltaInv(msntau2,mh2,mu2)*power2(mu2))/msntau2 - (2*DeltaInv(mstau12,mh2,
     mu2)*power2(mu2))/mh2 - (4*DeltaInv(mstau12,mh2,mu2)*power2(mu2))/mstau12
     - (2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/mh2 - (4*DeltaInv(mstau22,mh2,
     mu2)*power2(mu2))/mstau22 + (2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mH2
     + (4*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 + (6*mh2*DeltaInv(
     mstau22,mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (6*mH2*DeltaInv(
     mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mstau22,
     mh2,mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (6*mh2*DeltaInv(mstau12,mh2,mu2)*power2(mu2))/power2(mstau22 -
     mu2) - (2*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (2*mu2*DeltaInv(mH2,msntau2,mu2)*(-3*mH2*msntau2
     - 2*mH2*mu2 - msntau2*mu2 + power2(mH2) + power2(mu2)))/(mH2*msntau2) - (
     2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2
     + power2(mH2) + power2(mu2))*(-(mH2*mu2) - 2*mstau22*mu2 + power2(mstau22)
     + power2(mu2)))/(mH2*mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(msntau2,
     mh2,mu2)*power3(mu2))/(mh2*msntau2) + (2*DeltaInv(mstau12,mh2,mu2)*power3(
     mu2))/(mh2*mstau12) + (2*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/(mh2*
     mstau22) - (2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mH2*mstau22) + (2*
     DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (4*mh2*
     DeltaInv(mstau22,mh2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (4*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     2*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/power2(mstau22 - mu2) + (4*mh2*
     DeltaInv(mstau12,mh2,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     2*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) +
     (2*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2))
     - (2*DeltaInv(mstau12,mh2,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)
     )))/32. - (mu2*log(mstau12/Q2)*log(mu2/Q2)*(10/mA2 - 6/mH2 + 4/mstau12 +
     2/(mstau12 - mu2) + 8/(mstau22 - mu2) + (2*mu2)/(mA2*mstau12) + (2*mu2)/(
     mH2*mstau12) + (8*mu2)/((-mstau12 + mu2)*(-mstau22 + mu2)) + (8*mu2)/(mA2*
     mstau12 - mA2*mu2) - (8*mu2)/(mH2*mstau12 - mH2*mu2) - (3*mu2)/power2(
     msntau2 - mu2) - (13*mu2)/power2(mstau12 - mu2) - (2*mstau12)/power2(
     mstau22 - mu2) + (3*mu2)/power2(mstau22 - mu2) + (2*mh2*mu2)/(mstau12*
     power2(mstau22 - mu2)) + (2*mu2*mz2)/(mstau12*power2(mstau22 - mu2)) + (2*
     mu2*DeltaInv(mstau12,mh2,mu2)*power2(mh2))/power2(mstau22 - mu2) + (4*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (12*mh2*DeltaInv(mstau12,
     mh2,mu2)*power2(mu2))/power2(mstau22 - mu2) + (12*mz2*DeltaInv(mstau12,
     mz2,mu2)*power2(mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau12,mh2,mu2)
     *power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (2*mu2*
     DeltaInv(mstau12,mz2,mu2)*power2(mz2))/power2(mstau22 - mu2) + (2*
     DeltaInv(mstau12,mz2,mu2)*power2(mu2)*power2(mz2))/(mstau12*power2(mstau22
     - mu2)) - (2*DeltaInv(mA2,mstau12,mu2)*(-(mA2*mu2*(6*mstau12 + mu2)) - (
     mstau12 + mu2)*power2(mA2) + (-mstau12 + mu2)*power2(mu2) + power3(mA2)))/
     (mA2*mstau12) - (2*mu2*DeltaInv(mstau12,mh2,mu2)*power3(mh2))/(mstau12*
     power2(mstau22 - mu2)) - (2*DeltaInv(mH2,mstau12,mu2)*(-(mH2*mu2*(6*
     mstau12 + mu2)) - (mstau12 + mu2)*power2(mH2) + (-mstau12 + mu2)*power2(
     mu2) + power3(mH2)))/(mH2*mstau12) + (4*power2(mu2))/power3(mstau12 - mu2)
     + (4*mstau12*mu2)/power3(mstau22 - mu2) + (3*power3(mu2))/(power2(msntau2
     - mu2)*power2(mstau12 - mu2)) + (2*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/
     power2(mstau22 - mu2) + (2*mh2*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(mstau12,mz2,mu2)*power3(mu2))
     /power2(mstau22 - mu2) + (2*mz2*DeltaInv(mstau12,mz2,mu2)*power3(mu2))/(
     mstau12*power2(mstau22 - mu2)) + power3(mu2)/(power2(mstau12 - mu2)*
     power2(mstau22 - mu2)) - (2*mu2*DeltaInv(mstau12,mz2,mu2)*power3(mz2))/(
     mstau12*power2(mstau22 - mu2)) + (3*power3(mu2))/power4(mstau12 - mu2) - (
     2*DeltaInv(mstau12,mh2,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) -
     (2*DeltaInv(mstau12,mz2,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)))
     )/16. - (mu2*power2(sb)*power2(log(mA2/Q2))*(2/mA2 + 2/mstau12 + mA2/(
     mstau12*(-mstau22 + mu2)) - mA2/power2(mstau12 - mu2) + (mA2*mu2)/(
     mstau22*power2(mstau12 - mu2)) + (3*mu2*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2))/power2(mstau12 - mu2) + (2*mA2*mu2)/(mstau12*power2(mstau22 - mu2))
     + (mA2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/power2(mstau12 - mu2) + (2*
     DeltaInv(mA2,mstau22,mu2)*power2(mA2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (mA2*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 -
     mstau12*mu2 + power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2))
     - (mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*power2(mstau12 -
     mu2)) - (mA2*DeltaInv(mA2,mstau22,mu2)*power3(mu2))/(mstau22*power2(
     mstau12 - mu2)) + (DeltaInv(mA2,mstau22,mstau12)*(2*(3*mstau12*mstau22 -
     mstau24)*power2(mstau12 - mu2)*power2(mstau22 - mu2) + ((mstau22 - 2*mu2)*
     power2(mstau12) + (mstau22 - 2*mu2)*power2(mu2) + mstau12*(-4*mstau22*mu2
     + power2(mstau22) + 5*power2(mu2)))*power3(mA2) - power2(mA2)*(2*power2(
     mu2)*(-4*mstau22*mu2 + 2*power2(mstau22) + power2(mu2)) + power2(mstau12)*
     (-20*mstau22*mu2 + 7*power2(mstau22) + 17*power2(mu2)) + 3*(mstau22 - 2*
     mu2)*power3(mstau12) + mstau12*(-14*mu2*power2(mstau22) + 25*mstau22*
     power2(mu2) + 2*power3(mstau22) - 12*power3(mu2))) + mA2*((-6*mstau22*mu2
     + 3*power2(mstau22) + 4*power2(mu2))*power3(mstau12) + power2(mstau12)*(-
     6*mu2*power2(mstau22) - 2*mu2*(mstau24 + 3*power2(mu2)) + mstau22*(mstau24
     + 9*power2(mu2)) + power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 8*mu2*
     power2(mstau22) + mstau22*(mstau24 + 4*power2(mu2)) + 4*power3(mstau22)) +
     mstau12*(5*mstau24*power2(mu2) - 4*mstau22*mu2*(mstau24 + 2*power2(mu2)) +
     power2(mstau22)*(mstau24 + 13*power2(mu2)) - 6*mu2*power3(mstau22) + 2*
     power4(mu2)))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(
     16.*power2(cb)) + (mu2*log(mA2/Q2)*power2(sb)*(4/mA2 + 6/mstau12 - (3*mA2)
     /(mstau12*mstau22 - mstau12*mu2) - (3*mA2)/power2(mstau12 - mu2) + (3*mA2*
     mu2)/(mstau22*power2(mstau12 - mu2)) + (9*mu2*DeltaInv(mA2,mstau22,mu2)*
     power2(mA2))/power2(mstau12 - mu2) + (6*mA2*mu2)/(mstau12*power2(mstau22 -
     mu2)) + (3*mA2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/power2(mstau12 -
     mu2) + (6*DeltaInv(mA2,mstau22,mu2)*power2(mA2)*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (3*mA2*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*
     mstau12 - 2*mA2*mu2 - mstau12*mu2 + power2(mA2) + power2(mu2)))/(mstau12*
     power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(
     mstau22*power2(mstau12 - mu2)) - (3*mA2*DeltaInv(mA2,mstau22,mu2)*power3(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (3*DeltaInv(mA2,mstau22,mstau12)*(
     2*(3*mstau12*mstau22 - mstau24)*power2(mstau12 - mu2)*power2(mstau22 -
     mu2) + ((mstau22 - 2*mu2)*power2(mstau12) + (mstau22 - 2*mu2)*power2(mu2)
     + mstau12*(-4*mstau22*mu2 + power2(mstau22) + 5*power2(mu2)))*power3(mA2)
     - power2(mA2)*(2*power2(mu2)*(-4*mstau22*mu2 + 2*power2(mstau22) + power2(
     mu2)) + power2(mstau12)*(-20*mstau22*mu2 + 7*power2(mstau22) + 17*power2(
     mu2)) + 3*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*(-14*mu2*power2(
     mstau22) + 25*mstau22*power2(mu2) + 2*power3(mstau22) - 12*power3(mu2))) +
     mA2*((-6*mstau22*mu2 + 3*power2(mstau22) + 4*power2(mu2))*power3(mstau12)
     + power2(mstau12)*(-6*mu2*power2(mstau22) - 2*mu2*(mstau24 + 3*power2(mu2)
     ) + mstau22*(mstau24 + 9*power2(mu2)) + power3(mstau22)) + power2(mu2)*(-
     2*mstau24*mu2 - 8*mu2*power2(mstau22) + mstau22*(mstau24 + 4*power2(mu2))
     + 4*power3(mstau22)) + mstau12*(5*mstau24*power2(mu2) - 4*mstau22*mu2*(
     mstau24 + 2*power2(mu2)) + power2(mstau22)*(mstau24 + 13*power2(mu2)) - 6*
     mu2*power3(mstau22) + 2*power4(mu2)))))/(mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2))))/(8.*power2(cb)) + (mu2*Fin3(mA2,mstau22,mstau12,
     Q2)*power2(sb)*(-2/(mA2*mstau12) + (4*mstau12*(mstau12 + mstau22 - 2*mu2)*
     power2(cb) + mA2*(mu2 - mstau12*(1 + 4*power2(cb))))/(mA2*mstau12*(mstau12
     - mu2)*(-mstau22 + mu2)) - 2/power2(mA2) + 2/power2(mstau12 - mu2) + 1/
     power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 - mu2)) + (2*mu2)/
     power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22 + mu2) - (DeltaInv(mA2,
     mstau22,mstau12)*(2*(3*mstau12*mstau22 - mstau24)*power2(mstau12 - mu2)*
     power2(mstau22 - mu2) + ((mstau22 - 2*mu2)*power2(mstau12) + (mstau22 - 2*
     mu2)*power2(mu2) + mstau12*(-4*mstau22*mu2 + power2(mstau22) + 5*power2(
     mu2)))*power3(mA2) - power2(mA2)*(2*power2(mu2)*(-4*mstau22*mu2 + 2*
     power2(mstau22) + power2(mu2)) + power2(mstau12)*(-20*mstau22*mu2 + 7*
     power2(mstau22) + 17*power2(mu2)) + 3*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*(-14*mu2*power2(mstau22) + 25*mstau22*power2(mu2) + 2*power3(
     mstau22) - 12*power3(mu2))) + mA2*((-6*mstau22*mu2 + 3*power2(mstau22) +
     4*power2(mu2))*power3(mstau12) + power2(mstau12)*(-6*mu2*power2(mstau22) -
     2*mu2*(mstau24 + 3*power2(mu2)) + mstau22*(mstau24 + 9*power2(mu2)) +
     power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 8*mu2*power2(mstau22) +
     mstau22*(mstau24 + 4*power2(mu2)) + 4*power3(mstau22)) + mstau12*(5*
     mstau24*power2(mu2) - 4*mstau22*mu2*(mstau24 + 2*power2(mu2)) + power2(
     mstau22)*(mstau24 + 13*power2(mu2)) - 6*mu2*power3(mstau22) + 2*power4(
     mu2)))))/(mA2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(8.*
     power2(cb)) + (3*(mh2 + mH2)*(mstau12 + mstau22 - mu2)*power4(sa))/(4.*
     mh2*mH2) - (3*(mstau12 + mstau22 - mu2)*log(mh2/Q2)*power4(sa))/(2.*(mh2 -
     mH2)) + (3*(mstau12 + mstau22 - mu2)*log(mH2/Q2)*power4(sa))/(2.*(mh2 -
     mH2)) - (3*(mh2 + mH2)*mstau12*(mstau12 - mstau22 - mu2)*log(mstau12/Q2)*
     power4(sa))/(4.*mh2*mH2*(mstau12 - mstau22)) + (3*mstau12*(mstau12 -
     mstau22 - mu2)*log(mh2/Q2)*log(mstau12/Q2)*power4(sa))/(2.*(mh2 - mH2)*(
     mstau12 - mstau22)) - (3*mstau12*(mstau12 - mstau22 - mu2)*log(mH2/Q2)*
     log(mstau12/Q2)*power4(sa))/(2.*(mh2 - mH2)*(mstau12 - mstau22)) - (3*(mh2
     + mH2)*mstau22*(mstau12 - mstau22 + mu2)*log(mstau22/Q2)*power4(sa))/(4.*
     mh2*mH2*(mstau12 - mstau22)) + (3*mstau22*(mstau12 - mstau22 + mu2)*log(
     mh2/Q2)*log(mstau22/Q2)*power4(sa))/(2.*(mh2 - mH2)*(mstau12 - mstau22)) -
     (3*mstau22*(mstau12 - mstau22 + mu2)*log(mH2/Q2)*log(mstau22/Q2)*power4(
     sa))/(2.*(mh2 - mH2)*(mstau12 - mstau22)) - (3*(mh2 + mH2)*Fin20(mH2,mh2,
     Q2)*power2(mh2 - mH2)*power4(sa))/(16.*power2(mh2)*power2(mH2)) + (3*mu2*
     Fin20(mstau22,mstau12,Q2)*power2(mh2 - mH2)*power4(sa))/(4.*power2(mh2)*
     power2(mH2)) - (3*log(mh2/Q2)*log(mH2/Q2)*(8*mh2*mH2 + power2(mh2) +
     power2(mH2))*power4(sa))/(16.*mh2*mH2) + (3*mu2*log(mh2/Q2)*(3*mh2*mH2 -
     4*mh2*mstau12 + 2*mH2*mstau12 - 3*power2(mh2) + 3*mh2*(mh2 - mH2)*
     DeltaInv(mstau22,mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*
     mstau22) + mstau24 + power2(mh2)))*power2(sb)*power4(sa))/(4.*mh2*(mh2 -
     mH2)*mstau12*power2(cb)) - (3*mu2*Fin20(mstau22,mstau12,Q2)*power2(mh2 -
     mH2)*power2(sb)*power4(sa))/(4.*power2(cb)*power2(mh2)*power2(mH2)) + (3*
     mu2*log(mH2/Q2)*(-3*mh2*mH2 - 2*mh2*mstau12 + 4*mH2*mstau12 + 3*power2(
     mH2) + 3*(mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*mstau22
     - mH2*(mstau12 + 2*mstau22) + mstau24 + power2(mH2)))*power2(sb)*power4(
     sa))/(4.*(mh2 - mH2)*mH2*mstau12*power2(cb)) - (3*mu2*(mh2*(mH2 - 3*
     mstau12) + mH2*mstau12 - power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(mstau22,
     mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 +
     power2(mh2)))*power2(sb)*power2(log(mh2/Q2))*power4(sa))/(8.*mh2*(mh2 -
     mH2)*mstau12*power2(cb)) + (3*mu2*(-(mh2*(mH2 + mstau12)) + mH2*(mH2 + 3*
     mstau12) + (mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*
     mstau22 - mH2*(mstau12 + 2*mstau22) + mstau24 + power2(mH2)))*power2(sb)*
     power2(log(mH2/Q2))*power4(sa))/(8.*mH2*(-mh2 + mH2)*mstau12*power2(cb)) +
     (3*power2(log(mh2/Q2))*(mh2*(-15*mH2*mstau12 + 4*mH2*mu2 - 12*mstau12*mu2)
     *power2(mH2) + (3*mstau12 - 4*mu2)*power2(mh2)*power2(mH2) + 4*mh2*(mh2 -
     mH2)*mu2*DeltaInv(mstau22,mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12
     + 2*mstau22) + mstau24 + power2(mh2))*power2(mH2) + 5*mH2*mstau12*power3(
     mh2) + mstau12*(mH2 + 4*mu2)*power3(mH2) - 2*mstau12*power4(mh2))*power4(
     sa))/(32.*mh2*(mh2 - mH2)*mstau12*power2(mH2)) + (3*log(mh2/Q2)*(mh2*(3*
     mH2*(3*mstau12 - 4*mu2) + 8*mstau12*(mstau12 + mstau22 + 2*mu2))*power2(
     mH2) + (11*mstau12 + 12*mu2)*power2(mh2)*power2(mH2) - 12*mh2*(mh2 - mH2)*
     mu2*DeltaInv(mstau22,mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*
     mstau22) + mstau24 + power2(mh2))*power2(mH2) - 5*mH2*mstau12*power3(mh2)
     - 2*mstau12*(mH2 + 4*mu2)*power3(mH2) + 3*mstau12*power4(mh2))*power4(sa))
     /(16.*mh2*(mh2 - mH2)*mstau12*power2(mH2)) + (3*log(mH2/Q2)*(-(mH2*(8*
     mstau12*(mstau12 + mstau22 + 2*mu2) + mH2*(11*mstau12 + 12*mu2))*power2(
     mh2)) - 12*(mh2 - mH2)*mH2*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mh2)*(
     -3*mstau12*mstau22 - mH2*(mstau12 + 2*mstau22) + mstau24 + power2(mH2)) +
     (-9*mH2*mstau12 + 12*mH2*mu2 + 8*mstau12*mu2)*power3(mh2) + 5*mh2*mstau12*
     power3(mH2) + 2*mstau12*power4(mh2) - 3*mstau12*power4(mH2))*power4(sa))/(
     16.*(mh2 - mH2)*mH2*mstau12*power2(mh2)) - (3*power2(log(mH2/Q2))*(mH2*(3*
     mH2*mstau12 - 4*mH2*mu2 - 12*mstau12*mu2)*power2(mh2) - 4*(mh2 - mH2)*mH2*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mh2)*(-3*mstau12*mstau22 - mH2*(
     mstau12 + 2*mstau22) + mstau24 + power2(mH2)) + (-15*mH2*mstau12 + 4*mH2*
     mu2 + 4*mstau12*mu2)*power3(mh2) + 5*mh2*mstau12*power3(mH2) + mstau12*
     power4(mh2) - 2*mstau12*power4(mH2))*power4(sa))/(32.*(mh2 - mH2)*mH2*
     mstau12*power2(mh2)) + (mu2*Fin3(mstau22,mstau12,mh2,Q2)*(1/(mstau12*
     mstau22 - mstau12*mu2) - 3*DeltaInv(mstau22,mstau12,mh2) + (3*mh2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mh2*DeltaInv(mstau22,mstau12,
     mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22
     - mu2) + (2*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (
     mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12*(-mstau22 + mu2)) + (2*
     mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*(-mstau22 + mu2)) +
     2/power2(mstau12 - mu2) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) - (mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) + (mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) + (mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) -
     (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + 1/
     power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 - mu2)) - (3*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) - (2*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) +
     power2(sa)*(6/(mh2*mstau12) + 1/(mstau12*(-mstau22 + mu2)) + 6/power2(mh2)
     + 12/(-(mh2*mH2) + power2(mh2)) - 2/power2(mstau12 - mu2) - 1/power2(
     mstau22 - mu2) + mu2/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,
     mstau12,mh2)*(9 + (18*mstau22 - (6*mstau24)/mstau12)/mh2 + (2*mu2)/(
     mstau12 - mu2) + (3*mstau22)/(-mstau12 + mu2) + (2*mstau22)/(-mstau22 +
     mu2) + mstau24/(mstau12*mstau22 - mstau12*mu2) + mstau24/power2(mstau12 -
     mu2) - (mstau22*mu2)/power2(mstau12 - mu2) + mh2*(3/(-mstau12 + mu2) + 3/(
     -mstau22 + mu2) - (2*mstau22)/(mstau12*mstau22 - mstau12*mu2) - (2*
     mstau22)/power2(mstau12 - mu2) - mu2/power2(mstau12 - mu2) + (-6 + (2*
     mstau22*mu2)/power2(mstau22 - mu2))/mstau12 + (3*mu2)/power2(mstau22 -
     mu2)) + power2(mh2)*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12
     - mu2) - mu2/(mstau12*power2(mstau22 - mu2))) + (12*mstau22 - (mstau24*
     mu2)/power2(mstau22 - mu2))/mstau12 + (2*mstau22*mu2)/power2(mstau22 -
     mu2) - power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/power3(mstau12 - mu2)
     + (2*mu2)/power3(mstau22 - mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (
     power2(sa)*power2(sb)*(1/(mstau12*mstau22 - mstau12*mu2) - 12/(-(mh2*mH2)
     + power2(mh2)) + 2/power2(mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(
     mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*(mstau24*(
     mstau22 - 2*mu2)*power2(mu2) + power2(mh2)*((mstau22 - 2*mu2)*power2(
     mstau12) + (mstau22 - 2*mu2)*power2(mu2) + mstau12*(-4*mstau22*mu2 +
     power2(mstau22) + 5*power2(mu2))) + (-2*mstau22*mu2 + power2(mstau22) + 2*
     power2(mu2))*power3(mstau12) + power2(mstau12)*(6*mu2*power2(mstau22) +
     mstau22*(mstau24 - 3*power2(mu2)) - 2*mu2*(mstau24 + power2(mu2)) - 3*
     power3(mstau22)) + mstau12*(power2(mstau22)*(mstau24 - 5*power2(mu2)) + 5*
     mstau24*power2(mu2) + 2*mu2*power3(mstau22) + mstau22*(-4*mstau24*mu2 + 4*
     power3(mu2))) + mh2*(power2(mstau12)*(16*mstau22*mu2 - 5*power2(mstau22) -
     15*power2(mu2)) - 2*mstau22*(mstau22 - 2*mu2)*power2(mu2) - 3*(mstau22 -
     2*mu2)*power3(mstau12) + mstau12*(10*mu2*power2(mstau22) - 17*mstau22*
     power2(mu2) - 2*power3(mstau22) + 8*power3(mu2)))))/(mstau12*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (
     2*mu2)/power3(-mstau22 + mu2)))/power2(cb) + (2*ca*sa*sb*(1/(mstau12*
     mstau22 - mstau12*mu2) - 6/(-(mh2*mH2) + power2(mh2)) + 2/power2(mstau12 -
     mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 - mu2)) - (
     DeltaInv(mstau22,mstau12,mh2)*(mstau24*(mstau22 - 2*mu2)*power2(mu2) +
     power2(mh2)*((mstau22 - 2*mu2)*power2(mstau12) + (mstau22 - 2*mu2)*power2(
     mu2) + mstau12*(-4*mstau22*mu2 + power2(mstau22) + 5*power2(mu2))) + (-2*
     mstau22*mu2 + power2(mstau22) + 2*power2(mu2))*power3(mstau12) + power2(
     mstau12)*(6*mu2*power2(mstau22) + mstau22*(mstau24 - 3*power2(mu2)) - 2*
     mu2*(mstau24 + power2(mu2)) - 3*power3(mstau22)) + mstau12*(power2(
     mstau22)*(mstau24 - 5*power2(mu2)) + 5*mstau24*power2(mu2) + 2*mu2*power3(
     mstau22) + mstau22*(-4*mstau24*mu2 + 4*power3(mu2))) + mh2*(power2(
     mstau12)*(16*mstau22*mu2 - 5*power2(mstau22) - 15*power2(mu2)) - 2*
     mstau22*(mstau22 - 2*mu2)*power2(mu2) - 3*(mstau22 - 2*mu2)*power3(
     mstau12) + mstau12*(10*mu2*power2(mstau22) - 17*mstau22*power2(mu2) - 2*
     power3(mstau22) + 8*power3(mu2)))))/(mstau12*power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22
     + mu2)))/cb + (2*mu2)/power3(-mstau22 + mu2) - (12*ca*sb*(mh2*(mH2 - 3*
     mstau12) + mH2*mstau12 - power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(mstau22,
     mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 +
     power2(mh2)))*power3(sa))/(cb*(mh2 - mH2)*mstau12*power2(mh2)) + (6*(mh2*(
     mH2 - 3*mstau12) + mH2*mstau12 - power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(
     mstau22,mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) +
     mstau24 + power2(mh2)))*power4(sa))/((mh2 - mH2)*mstau12*power2(mh2)) - (
     6*(mh2*(mH2 - 3*mstau12) + mH2*mstau12 - power2(mh2) + mh2*(mh2 - mH2)*
     DeltaInv(mstau22,mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*
     mstau22) + mstau24 + power2(mh2)))*power2(sb)*power4(sa))/((mh2 - mH2)*
     mstau12*power2(cb)*power2(mh2))))/8. + (mu2*Fin3(mstau22,mH2,mstau12,Q2)*(
     -(power2(sa)*((12/mh2 - 6/mstau12)/mH2 + 1/(mstau12*(-mstau22 + mu2)) +
     12/(-(mh2*mH2) + power2(mh2)) - 6/power2(mH2) - 2/power2(mstau12 - mu2) -
     1/power2(mstau22 - mu2) + mu2/(mstau12*power2(mstau22 - mu2)) + DeltaInv(
     mstau22,mH2,mstau12)*(-3 + (6*(-3*mstau12*mstau22 + mstau24))/(mH2*
     mstau12) + (2*mu2)/(mstau12 - mu2) + (3*mstau22)/(-mstau12 + mu2) + (2*
     mstau22)/(-mstau22 + mu2) + mstau24/(mstau12*mstau22 - mstau12*mu2) +
     mstau24/power2(mstau12 - mu2) - (mstau22*mu2)/power2(mstau12 - mu2) + mH2*
     (3/(-mstau12 + mu2) + 3/(-mstau22 + mu2) - (2*mstau22)/(mstau12*mstau22 -
     mstau12*mu2) - (2*mstau22)/power2(mstau12 - mu2) - mu2/power2(mstau12 -
     mu2) + (6 + (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 + (3*mu2)/
     power2(mstau22 - mu2)) + power2(mH2)*(1/(mstau12*mstau22 - mstau12*mu2) +
     1/power2(mstau12 - mu2) - mu2/(mstau12*power2(mstau22 - mu2))) - (12*
     mstau22 + (mstau24*mu2)/power2(mstau22 - mu2))/mstau12 + (2*mstau22*mu2)/
     power2(mstau22 - mu2) - power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/
     power3(mstau12 - mu2) + (2*mu2)/power3(mstau22 - mu2))) - (power2(sa)*
     power2(sb)*((12*(mh2 - mstau12))/(mh2*mH2*mstau12) + 1/(mstau12*mstau22 -
     mstau12*mu2) - 12/(-(mh2*mH2) + power2(mh2)) + 12/power2(mH2) + 2/power2(
     mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 -
     mu2)) + DeltaInv(mstau22,mH2,mstau12)*(9 + (36*mstau22 - (12*mstau24)/
     mstau12)/mH2 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) +
     (2*mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (24*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(
     mstau22 - mu2))/mstau12 + mH2*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*
     mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) +
     mu2/power2(mstau12 - mu2) + (-12 - (2*mstau22*mu2)/power2(mstau22 - mu2))/
     mstau12 - (3*mu2)/power2(mstau22 - mu2)) + power2(mH2)*(-(1/power2(mstau12
     - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2))) - (2*
     mstau22*mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) +
     (2*mu2)/power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22 + mu2)))/power2(
     cb) + (power2(sb)*(6/(mH2*mstau12) + 1/(mstau12*mstau22 - mstau12*mu2) +
     6/power2(mH2) + 2/power2(mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(
     mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(3 + (18*
     mstau22 - (6*mstau24)/mstau12)/mH2 + (3*mstau22)/(mstau12 - mu2) + (2*
     mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/power2(
     mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (12*mstau22 - (
     mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/mstau12 + mH2*(3/(
     mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 -
     mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-6
     - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22
     - mu2)) + power2(mH2)*(-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(
     mstau12*power2(mstau22 - mu2))) - (2*mstau22*mu2)/power2(mstau22 - mu2) +
     power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (2*
     mu2)/power3(-mstau22 + mu2)))/power2(cb) - (2*ca*sa*sb*((-6/mh2 + 6/
     mstau12)/mH2 + 1/(mstau12*mstau22 - mstau12*mu2) - 6/(-(mh2*mH2) + power2(
     mh2)) + 6/power2(mH2) + 2/power2(mstau12 - mu2) + 1/power2(mstau22 - mu2)
     - mu2/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(3 +
     (18*mstau22 - (6*mstau24)/mstau12)/mH2 + (3*mstau22)/(mstau12 - mu2) + (2*
     mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/power2(
     mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (12*mstau22 - (
     mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/mstau12 + mH2*(3/(
     mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 -
     mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-6
     - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22
     - mu2)) + power2(mH2)*(-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(
     mstau12*power2(mstau22 - mu2))) - (2*mstau22*mu2)/power2(mstau22 - mu2) +
     power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (2*
     mu2)/power3(-mstau22 + mu2)))/cb - (12*ca*sb*(-(mh2*(mH2 + mstau12)) +
     mH2*(mH2 + 3*mstau12) + (mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,mstau12)*(-3*
     mstau12*mstau22 - mH2*(mstau12 + 2*mstau22) + mstau24 + power2(mH2)))*
     power3(sa))/(cb*(mh2 - mH2)*mstau12*power2(mH2)) + (6*(-(mh2*(mH2 +
     mstau12)) + mH2*(mH2 + 3*mstau12) + (mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,
     mstau12)*(-3*mstau12*mstau22 - mH2*(mstau12 + 2*mstau22) + mstau24 +
     power2(mH2)))*power4(sa))/((mh2 - mH2)*mstau12*power2(mH2)) - (6*(-(mh2*(
     mH2 + mstau12)) + mH2*(mH2 + 3*mstau12) + (mh2 - mH2)*mH2*DeltaInv(
     mstau22,mH2,mstau12)*(-3*mstau12*mstau22 - mH2*(mstau12 + 2*mstau22) +
     mstau24 + power2(mH2)))*power2(sb)*power4(sa))/((mh2 - mH2)*mstau12*
     power2(cb)*power2(mH2))))/8. - (log(msntau2/Q2)*log(mstau22/Q2)*((4*
     msntau2*(mstau22 - mu2)*power2(mu2)*(3*mstau22*mu2*(-2*mu2 + mw2) + mu2*
     power2(mstau22) + 2*(mu2 - 3*mw2)*power2(mu2) + power3(mstau22)) + (mu2*
     power2(mstau22) - 10*mstau22*power2(mu2) + power3(mstau22) - 16*power3(
     mu2))*power4(msntau2) + power3(msntau2)*(mu2*(-21*mu2 + 4*mw2)*power2(
     mstau22) + 4*mstau22*(10*mu2 - 3*mw2)*power2(mu2) + 7*mu2*power3(mstau22)
     + 8*mw2*power3(mu2) - 2*power4(mstau22)) + 2*mu2*power2(msntau2)*(6*mu2*(
     4*mu2 - mw2)*power2(mstau22) + mstau22*(-29*mu2 + 18*mw2)*power2(mu2) - 8*
     mu2*power3(mstau22) + 4*(2*mu2 - 3*mw2)*power3(mu2) + power4(mstau22)) -
     4*(mstau22 + mw2)*(-3*mstau22*mu2 + power2(mstau22) + 2*power2(mu2))*
     power4(mu2) + 8*power2(mu2)*power5(msntau2))/power3(-mstau22 + mu2) + (4*(
     msntau2 - mu2)*mu2*DeltaInv(mstau22,msntau2,mw2)*(-((mstau22 - 2*mu2)*(
     mstau22 + mw2)*(-mstau24 + (2*mstau22 - mw2)*mw2)*power2(mu2)) + power3(
     msntau2)*(-2*(mu2 + 3*mw2)*power2(mstau22) + mstau22*(12*mu2*mw2 + 2*
     power2(mu2) - power2(mw2)) + 2*mu2*power2(mw2) + power3(mstau22)) -
     power2(msntau2)*(power2(mstau22)*(2*mstau24 - 18*mu2*mw2 + power2(mw2)) +
     3*mw2*power3(mstau22) - mstau22*(4*mstau24*mu2 + mstau24*mw2 - 25*mw2*
     power2(mu2) + 4*mu2*power2(mw2) - 2*power3(mu2) + power3(mw2)) + mu2*(3*
     mstau24*mu2 + 2*mstau24*mw2 + 4*mu2*power2(mw2) + 2*power3(mw2))) +
     msntau2*(2*mstau22*mu2*mw2*(mstau24 - 2*mu2*mw2 + 5*power2(mu2) - power2(
     mw2)) + 2*power2(mu2)*(mstau24*(mu2 + mw2) + (mu2 + 2*mw2)*power2(mw2)) +
     mw2*(2*mu2 + mw2)*power3(mstau22) + 2*power2(mstau22)*(-(mstau24*mw2) - 5*
     mw2*power2(mu2) + power3(mu2)) - 2*mu2*power4(mstau22) + power5(mstau22)))
     )/power2(mstau22 - mu2)))/(16.*msntau2*power3(msntau2 - mu2)) + (log(
     mstau12/Q2)*log(mstau22/Q2)*(9 + (2*mstau22)/(mstau12 - mstau22) + (4*mu2)
     /mstau12 - (4*mu2)/(mstau12 - mstau22) + (8*mu2)/(mstau12 - mu2) + (12*
     mu2)/(mstau22 - mu2) + (8*mstau22*mu2)/((mstau12 - mstau22)*(mstau22 -
     mu2)) + (2*mstau22)/(-mstau12 + mu2) + (2*mstau12)/(-mstau22 + mu2) + (2*
     mstau24)/((mstau12 - mstau22)*(-mstau22 + mu2)) + (2*mh2*mu2)/(mstau12*
     mstau22 - mstau12*mu2) + (2*mu2*mz2)/(mstau12*mstau22 - mstau12*mu2) + (2*
     mstau24)/power2(mstau12 - mstau22) - (4*mstau22*mu2)/power2(mstau12 -
     mstau22) + (8*mstau24*mu2)/((mstau22 - mu2)*power2(mstau12 - mstau22)) - (
     2*mstau22*mu2)/power2(mstau12 - mu2) - (6*mstau12*mu2)/power2(mstau22 -
     mu2) - (2*mstau24*mu2)/((mstau12 - mstau22)*power2(mstau22 - mu2)) + (6*
     power2(mu2))/((mstau12 - mstau22)*(-mstau22 + mu2)) + (8*power2(mu2))/((-
     mstau12 + mu2)*(-mstau22 + mu2)) + (6*mstau22*power2(mu2))/((-mstau22 +
     mu2)*power2(mstau12 - mstau22)) - power2(mu2)/power2(mstau12 - mu2) - (5*
     power2(mu2))/power2(mstau22 - mu2) - (2*mh2*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (4*mstau22*power2(mu2))/((mstau12 - mstau22)*power2(
     mstau22 - mu2)) - (2*mz2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     4*mstau24*power2(mu2))/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) +
     (2*power3(mstau22))/((mstau12 - mu2)*power2(mstau12 - mstau22)) + (2*
     power3(mstau22))/((-mstau22 + mu2)*power2(mstau12 - mstau22)) - (2*power3(
     mstau22))/((mstau12 - mstau22)*power2(mstau12 - mu2)) + (2*mu2*power3(
     mstau22))/(power2(mstau12 - mstau22)*power2(mstau12 - mu2)) - (2*mu2*
     power3(mstau22))/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) + (4*
     mstau22*power2(mu2))/power3(mstau12 - mu2) + (4*mstau12*power2(mu2))/
     power3(mstau22 - mu2) - (4*power3(mu2))/(mstau12*power2(mstau22 - mu2)) -
     (2*power3(mu2))/((mstau12 - mstau22)*power2(mstau22 - mu2)) - (2*mstau22*
     power3(mu2))/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) - (2*
     power4(mstau22))/(power2(mstau12 - mstau22)*power2(mstau12 - mu2)) +
     power4(mu2)/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (2*mu2*
     DeltaInv(mstau22,mstau12,mh2)*(mstau22*mstau24*(mstau22 - 2*mu2)*power2(
     mu2) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power3(mh2) + mstau22*(-2*
     mstau22*mu2 + power2(mstau22) + 2*power2(mu2))*power3(mstau12) + mh2*(-((
     mstau22 - 2*mu2)*(-mstau24 + 2*power2(mstau22))*power2(mu2)) - 6*mstau22*(
     mstau22 - 2*mu2)*power3(mstau12) + power2(mstau12)*(-2*mstau24*mu2 + 18*
     mu2*power2(mstau22) + mstau22*(mstau24 - 25*power2(mu2)) - 3*power3(
     mstau22)) + 2*mstau12*(mstau24*power2(mu2) + mstau22*mu2*(mstau24 + 5*
     power2(mu2)) - power2(mstau22)*(mstau24 + 5*power2(mu2)) + mu2*power3(
     mstau22))) - power2(mstau12)*(-4*mstau22*mstau24*mu2 + 2*mstau24*power2(
     mstau22) + 3*mstau24*power2(mu2) + 2*mstau22*power3(mu2)) - power2(mh2)*(
     power2(mstau12)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*
     power2(mu2) + power3(mstau22) + 2*power3(mu2))) + mstau12*(2*mstau24*
     power3(mu2) + 2*power2(mstau22)*power3(mu2) - 2*mu2*power4(mstau22) +
     power5(mstau22))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2)) -
     (2*mu2*DeltaInv(mstau22,mstau12,mz2)*(-((mstau22 - 2*mu2)*(mstau22 + mz2)*
     (-mstau24 + (2*mstau22 - mz2)*mz2)*power2(mu2)) + power3(mstau12)*(-2*(mu2
     + 3*mz2)*power2(mstau22) + mstau22*(12*mu2*mz2 + 2*power2(mu2) - power2(
     mz2)) + 2*mu2*power2(mz2) + power3(mstau22)) - power2(mstau12)*(power2(
     mstau22)*(2*mstau24 - 18*mu2*mz2 + power2(mz2)) + 3*mz2*power3(mstau22) -
     mstau22*(4*mstau24*mu2 + mstau24*mz2 - 25*mz2*power2(mu2) + 4*mu2*power2(
     mz2) - 2*power3(mu2) + power3(mz2)) + mu2*(3*mstau24*mu2 + 2*mstau24*mz2 +
     4*mu2*power2(mz2) + 2*power3(mz2))) + mstau12*(2*mstau22*mu2*mz2*(mstau24
     - 2*mu2*mz2 + 5*power2(mu2) - power2(mz2)) + 2*power2(mu2)*(mstau24*(mu2 +
     mz2) + (mu2 + 2*mz2)*power2(mz2)) + mz2*(2*mu2 + mz2)*power3(mstau22) + 2*
     power2(mstau22)*(-(mstau24*mz2) - 5*mz2*power2(mu2) + power3(mu2)) - 2*
     mu2*power4(mstau22) + power5(mstau22))))/(mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2))))/16. - (log(mstau22/Q2)*log(mu2/Q2)*power2(mu2)*
     power2(sb)*(-2*mA2*msntau2*mu2 - 2*mH2*msntau2*mu2 - 4*mC2*mstau12*mu2 -
     4*msntau2*mstau22*mu2 - 4*mstau12*mstau22*mu2 - 2*msntau2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2) + mA2*power2(msntau2) + mH2*power2(
     msntau2) + 2*mstau22*power2(msntau2) + 2*mu2*power2(msntau2) + 6*mH2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mu2)*power2(msntau2) + mstau22*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2)*power2(msntau2) + mu2*DeltaInv(mstau22,mH2,
     mu2)*power2(mH2)*power2(msntau2) + 2*mC2*power2(mstau12) + 2*mstau22*
     power2(mstau12) + 2*mu2*power2(mstau12) + mA2*power2(mu2) + 2*mC2*power2(
     mu2) + mH2*power2(mu2) - 4*msntau2*power2(mu2) - 4*mstau12*power2(mu2) +
     4*mstau22*power2(mu2) - 12*mH2*msntau2*mstau22*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2) - 2*msntau2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2)
     + mstau22*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2) + mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(msntau2)*power2(mu2) + mstau22*DeltaInv(
     mstau22,mH2,mu2)*power2(msntau2)*power2(mu2) - DeltaInv(mA2,mstau22,mu2)*
     power2(msntau2 - mu2)*(-(mA2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*
     power2(mA2) + (-mstau22 + mu2)*power2(mu2) + power3(mA2)) - 2*DeltaInv(
     mstau22,mC2,mu2)*power2(mstau12 - mu2)*(-(mC2*mu2*(6*mstau22 + mu2)) - (
     mstau22 + mu2)*power2(mC2) + (-mstau22 + mu2)*power2(mu2) + power3(mC2)) +
     2*msntau2*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2) - DeltaInv(mstau22,
     mH2,mu2)*power2(msntau2)*power3(mH2) - DeltaInv(mstau22,mH2,mu2)*power2(
     mu2)*power3(mH2) + 4*power3(mu2) - 2*mH2*msntau2*DeltaInv(mstau22,mH2,mu2)
     *power3(mu2) + 6*mH2*mstau22*DeltaInv(mstau22,mH2,mu2)*power3(mu2) - 2*
     msntau2*mstau22*DeltaInv(mstau22,mH2,mu2)*power3(mu2) + DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power3(mu2) - DeltaInv(mstau22,mH2,mu2)*power2(
     msntau2)*power3(mu2) + mH2*DeltaInv(mstau22,mH2,mu2)*power4(mu2) + 2*
     msntau2*DeltaInv(mstau22,mH2,mu2)*power4(mu2) + mstau22*DeltaInv(mstau22,
     mH2,mu2)*power4(mu2) - DeltaInv(mstau22,mH2,mu2)*power5(mu2)))/(8.*
     mstau22*power2(cb)*power2(msntau2 - mu2)*power2(mstau12 - mu2)) - (3*mu2*
     mw2*log(mw2/Q2)*(-3*mstau22*mu2*power2(msntau2) - 5*msntau2*mu2*power2(
     mstau22) + power2(msntau2)*power2(mstau22) + 9*msntau2*mstau22*power2(mu2)
     + power2(mstau22)*power2(mu2) + 4*msntau2*mw2*DeltaInv(mstau22,mw2,mu2)*
     power2(mstau22)*power2(mu2) - mstau22*mu2*DeltaInv(msntau2,mw2,mu2)*
     power2(msntau2 - mu2)*(msntau2*(mu2 + 3*mw2) - power2(mu2 - mw2)) +
     msntau2*mu2*DeltaInv(mstau22,mw2,mu2)*power2(mstau22)*power2(mw2) - 2*
     msntau2*mstau22*DeltaInv(mstau22,mw2,mu2)*power2(mu2)*power2(mw2) +
     msntau2*power3(mstau22) - 3*msntau2*mu2*mw2*DeltaInv(mstau22,mw2,mu2)*
     power3(mstau22) - msntau2*DeltaInv(mstau22,mw2,mu2)*power2(mu2)*power3(
     mstau22) - mstau22*DeltaInv(mstau22,msntau2,mw2)*(-((mstau22 - 2*mu2)*(-
     mstau24 + (2*mstau22 - mw2)*mw2)*power2(mu2)) + (2*mu2*(mu2 + 3*mw2) -
     mstau22*(2*mu2 + 3*mw2) + power2(mstau22))*power3(msntau2) + power2(
     msntau2)*((6*mu2 - 5*mw2)*power2(mstau22) + mstau22*(mstau24 + 16*mu2*mw2
     - 3*power2(mu2) + power2(mw2)) - mu2*(2*mstau24 + 15*mu2*mw2 + 2*power2(
     mu2) + 2*power2(mw2)) - 3*power3(mstau22)) + msntau2*(mstau22*mu2*(-4*
     mstau24 - 17*mu2*mw2 + 4*power2(mu2) - 4*power2(mw2)) + power2(mstau22)*(
     mstau24 + 10*mu2*mw2 - 5*power2(mu2) + power2(mw2)) + power2(mu2)*(5*
     mstau24 + 8*mu2*mw2 + 5*power2(mw2)) + 2*(mu2 - mw2)*power3(mstau22))) -
     msntau2*power3(mu2) - 3*mstau22*power3(mu2) + msntau2*mstau22*mw2*
     DeltaInv(mstau22,mw2,mu2)*power3(mu2) + 3*msntau2*DeltaInv(mstau22,mw2,
     mu2)*power2(mstau22)*power3(mu2) + msntau2*DeltaInv(mstau22,mw2,mu2)*
     power2(mw2)*power3(mu2) - 3*msntau2*mstau22*DeltaInv(mstau22,mw2,mu2)*
     power4(mu2) - 2*msntau2*mw2*DeltaInv(mstau22,mw2,mu2)*power4(mu2) +
     msntau2*DeltaInv(mstau22,mw2,mu2)*power5(mu2)))/(4.*msntau2*mstau22*
     power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (mu2*mw2*power2(log(mw2/Q2)
     )*(-3*mstau22*mu2*power2(msntau2) - 5*msntau2*mu2*power2(mstau22) +
     power2(msntau2)*power2(mstau22) + 9*msntau2*mstau22*power2(mu2) + power2(
     mstau22)*power2(mu2) + 4*msntau2*mw2*DeltaInv(mstau22,mw2,mu2)*power2(
     mstau22)*power2(mu2) - mstau22*mu2*DeltaInv(msntau2,mw2,mu2)*power2(
     msntau2 - mu2)*(msntau2*(mu2 + 3*mw2) - power2(mu2 - mw2)) + msntau2*mu2*
     DeltaInv(mstau22,mw2,mu2)*power2(mstau22)*power2(mw2) - 2*msntau2*mstau22*
     DeltaInv(mstau22,mw2,mu2)*power2(mu2)*power2(mw2) + msntau2*power3(
     mstau22) - 3*msntau2*mu2*mw2*DeltaInv(mstau22,mw2,mu2)*power3(mstau22) -
     msntau2*DeltaInv(mstau22,mw2,mu2)*power2(mu2)*power3(mstau22) - mstau22*
     DeltaInv(mstau22,msntau2,mw2)*(-((mstau22 - 2*mu2)*(-mstau24 + (2*mstau22
     - mw2)*mw2)*power2(mu2)) + (2*mu2*(mu2 + 3*mw2) - mstau22*(2*mu2 + 3*mw2)
     + power2(mstau22))*power3(msntau2) + power2(msntau2)*((6*mu2 - 5*mw2)*
     power2(mstau22) + mstau22*(mstau24 + 16*mu2*mw2 - 3*power2(mu2) + power2(
     mw2)) - mu2*(2*mstau24 + 15*mu2*mw2 + 2*power2(mu2) + 2*power2(mw2)) - 3*
     power3(mstau22)) + msntau2*(mstau22*mu2*(-4*mstau24 - 17*mu2*mw2 + 4*
     power2(mu2) - 4*power2(mw2)) + power2(mstau22)*(mstau24 + 10*mu2*mw2 - 5*
     power2(mu2) + power2(mw2)) + power2(mu2)*(5*mstau24 + 8*mu2*mw2 + 5*
     power2(mw2)) + 2*(mu2 - mw2)*power3(mstau22))) - msntau2*power3(mu2) - 3*
     mstau22*power3(mu2) + msntau2*mstau22*mw2*DeltaInv(mstau22,mw2,mu2)*
     power3(mu2) + 3*msntau2*DeltaInv(mstau22,mw2,mu2)*power2(mstau22)*power3(
     mu2) + msntau2*DeltaInv(mstau22,mw2,mu2)*power2(mw2)*power3(mu2) - 3*
     msntau2*mstau22*DeltaInv(mstau22,mw2,mu2)*power4(mu2) - 2*msntau2*mw2*
     DeltaInv(mstau22,mw2,mu2)*power4(mu2) + msntau2*DeltaInv(mstau22,mw2,mu2)*
     power5(mu2)))/(8.*msntau2*mstau22*power2(msntau2 - mu2)*power2(mstau22 -
     mu2)) - (3*mu2*mz2*log(mz2/Q2)*(-3*mstau22*mu2*power2(mstau12) - 5*
     mstau12*mu2*power2(mstau22) + power2(mstau12)*power2(mstau22) + 9*mstau12*
     mstau22*power2(mu2) + power2(mstau22)*power2(mu2) + 4*mstau12*mz2*
     DeltaInv(mstau22,mz2,mu2)*power2(mstau22)*power2(mu2) - mstau22*mu2*
     DeltaInv(mstau12,mz2,mu2)*power2(mstau12 - mu2)*(mstau12*(mu2 + 3*mz2) -
     power2(mu2 - mz2)) + mstau12*mu2*DeltaInv(mstau22,mz2,mu2)*power2(mstau22)
     *power2(mz2) - 2*mstau12*mstau22*DeltaInv(mstau22,mz2,mu2)*power2(mu2)*
     power2(mz2) + mstau12*power3(mstau22) - 3*mstau12*mu2*mz2*DeltaInv(
     mstau22,mz2,mu2)*power3(mstau22) - mstau12*DeltaInv(mstau22,mz2,mu2)*
     power2(mu2)*power3(mstau22) - mstau22*DeltaInv(mstau22,mstau12,mz2)*(-((
     mstau22 - 2*mu2)*(-mstau24 + (2*mstau22 - mz2)*mz2)*power2(mu2)) + (2*mu2*
     (mu2 + 3*mz2) - mstau22*(2*mu2 + 3*mz2) + power2(mstau22))*power3(mstau12)
     + power2(mstau12)*((6*mu2 - 5*mz2)*power2(mstau22) + mstau22*(mstau24 +
     16*mu2*mz2 - 3*power2(mu2) + power2(mz2)) - mu2*(2*mstau24 + 15*mu2*mz2 +
     2*power2(mu2) + 2*power2(mz2)) - 3*power3(mstau22)) + mstau12*(mstau22*
     mu2*(-4*mstau24 - 17*mu2*mz2 + 4*power2(mu2) - 4*power2(mz2)) + power2(
     mstau22)*(mstau24 + 10*mu2*mz2 - 5*power2(mu2) + power2(mz2)) + power2(
     mu2)*(5*mstau24 + 8*mu2*mz2 + 5*power2(mz2)) + 2*(mu2 - mz2)*power3(
     mstau22))) - mstau12*power3(mu2) - 3*mstau22*power3(mu2) + mstau12*
     mstau22*mz2*DeltaInv(mstau22,mz2,mu2)*power3(mu2) + 3*mstau12*DeltaInv(
     mstau22,mz2,mu2)*power2(mstau22)*power3(mu2) + mstau12*DeltaInv(mstau22,
     mz2,mu2)*power2(mz2)*power3(mu2) - 3*mstau12*mstau22*DeltaInv(mstau22,mz2,
     mu2)*power4(mu2) - 2*mstau12*mz2*DeltaInv(mstau22,mz2,mu2)*power4(mu2) +
     mstau12*DeltaInv(mstau22,mz2,mu2)*power5(mu2)))/(8.*mstau12*mstau22*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (mu2*mz2*power2(log(mz2/Q2)
     )*(-3*mstau22*mu2*power2(mstau12) - 5*mstau12*mu2*power2(mstau22) +
     power2(mstau12)*power2(mstau22) + 9*mstau12*mstau22*power2(mu2) + power2(
     mstau22)*power2(mu2) + 4*mstau12*mz2*DeltaInv(mstau22,mz2,mu2)*power2(
     mstau22)*power2(mu2) - mstau22*mu2*DeltaInv(mstau12,mz2,mu2)*power2(
     mstau12 - mu2)*(mstau12*(mu2 + 3*mz2) - power2(mu2 - mz2)) + mstau12*mu2*
     DeltaInv(mstau22,mz2,mu2)*power2(mstau22)*power2(mz2) - 2*mstau12*mstau22*
     DeltaInv(mstau22,mz2,mu2)*power2(mu2)*power2(mz2) + mstau12*power3(
     mstau22) - 3*mstau12*mu2*mz2*DeltaInv(mstau22,mz2,mu2)*power3(mstau22) -
     mstau12*DeltaInv(mstau22,mz2,mu2)*power2(mu2)*power3(mstau22) - mstau22*
     DeltaInv(mstau22,mstau12,mz2)*(-((mstau22 - 2*mu2)*(-mstau24 + (2*mstau22
     - mz2)*mz2)*power2(mu2)) + (2*mu2*(mu2 + 3*mz2) - mstau22*(2*mu2 + 3*mz2)
     + power2(mstau22))*power3(mstau12) + power2(mstau12)*((6*mu2 - 5*mz2)*
     power2(mstau22) + mstau22*(mstau24 + 16*mu2*mz2 - 3*power2(mu2) + power2(
     mz2)) - mu2*(2*mstau24 + 15*mu2*mz2 + 2*power2(mu2) + 2*power2(mz2)) - 3*
     power3(mstau22)) + mstau12*(mstau22*mu2*(-4*mstau24 - 17*mu2*mz2 + 4*
     power2(mu2) - 4*power2(mz2)) + power2(mstau22)*(mstau24 + 10*mu2*mz2 - 5*
     power2(mu2) + power2(mz2)) + power2(mu2)*(5*mstau24 + 8*mu2*mz2 + 5*
     power2(mz2)) + 2*(mu2 - mz2)*power3(mstau22))) - mstau12*power3(mu2) - 3*
     mstau22*power3(mu2) + mstau12*mstau22*mz2*DeltaInv(mstau22,mz2,mu2)*
     power3(mu2) + 3*mstau12*DeltaInv(mstau22,mz2,mu2)*power2(mstau22)*power3(
     mu2) + mstau12*DeltaInv(mstau22,mz2,mu2)*power2(mz2)*power3(mu2) - 3*
     mstau12*mstau22*DeltaInv(mstau22,mz2,mu2)*power4(mu2) - 2*mstau12*mz2*
     DeltaInv(mstau22,mz2,mu2)*power4(mu2) + mstau12*DeltaInv(mstau22,mz2,mu2)*
     power5(mu2)))/(16.*mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 -
     mu2)) + (log(mstau22/Q2)*(7 - (2*mA2)/mstau22 - (4*mC2)/mstau22 - (2*mH2)/
     mstau22 - (4*msntau2)/mstau22 - (4*mstau12)/mstau22 + (16*mstau22)/mA2 + (
     12*mstau22)/mC2 - (6*mstau22)/(mstau12 - mstau22) + (9*mstau22)/(msntau2 -
     mu2) + (3*mstau24)/((msntau2 - mstau22)*(msntau2 - mu2)) + (9*mstau22)/(
     mstau12 - mu2) + (3*mstau24)/((mstau12 - mstau22)*(mstau12 - mu2)) + (2*
     mA2)/(mstau22 - mu2) + (4*mC2)/(mstau22 - mu2) + (2*mH2)/(mstau22 - mu2) +
     (4*msntau2)/(mstau22 - mu2) + (8*mstau12)/(mstau22 - mu2) + (6*mstau24)/((
     mstau12 - mstau22)*(mstau22 - mu2)) + (30*mu2)/mA2 + (12*mu2)/mC2 - (18*
     mu2)/mH2 - (12*mu2)/msntau2 - (12*mu2)/mstau12 + (12*mu2)/(mstau12 -
     mstau22) + (80*mu2)/mstau22 + mu2/(msntau2 - mu2) + (15*mu2)/(-mstau12 +
     mu2) + (13*mu2)/(-mstau22 + mu2) + (24*mstau22*mu2)/((mstau12 - mstau22)*(
     -mstau22 + mu2)) + (16*mstau22*mu2)/((-mstau12 + mu2)*(-mstau22 + mu2)) +
     30*mH2*mu2*DeltaInv(mstau22,mH2,mu2) + 12*mstau22*mu2*DeltaInv(mstau22,
     msntau2,mw2) + (12*mstau24*mu2*DeltaInv(mstau22,msntau2,mw2))/msntau2 + (
     36*mstau24*mu2*DeltaInv(mstau22,msntau2,mw2))/(-msntau2 + mu2) - 36*mu2*
     mw2*DeltaInv(mstau22,msntau2,mw2) - (24*mstau22*mu2*mw2*DeltaInv(mstau22,
     msntau2,mw2))/msntau2 + (36*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))
     /(-msntau2 + mu2) - 18*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) + 6*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2) - (12*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (6*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (18*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 +
     mu2) + (18*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) +
     6*mstau22*mu2*DeltaInv(mstau22,mstau12,mz2) + (6*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mz2))/mstau12 + (18*mstau24*mu2*DeltaInv(mstau22,mstau12,
     mz2))/(-mstau12 + mu2) - 18*mu2*mz2*DeltaInv(mstau22,mstau12,mz2) - (12*
     mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/mstau12 + (18*mstau22*mu2*
     mz2*DeltaInv(mstau22,mstau12,mz2))/(-mstau12 + mu2) + (6*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/mstau12 - (18*mu2*DeltaInv(mstau22,mH2,
     mu2)*power2(mH2))/mstau22 - (6*mstau24)/power2(mstau12 - mstau22) + (12*
     mstau22*mu2)/power2(mstau12 - mstau22) + (24*mstau24*mu2)/((-mstau22 +
     mu2)*power2(mstau12 - mstau22)) - (13*mstau22*mu2)/power2(msntau2 - mu2) +
     (3*mstau24*mu2)/((msntau2 - mstau22)*power2(msntau2 - mu2)) - (24*mstau24*
     mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/power2(msntau2 - mu2) - (13*
     mstau22*mu2)/power2(mstau12 - mu2) + (3*mstau24*mu2)/((mstau12 - mstau22)*
     power2(mstau12 - mu2)) - (12*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2)
     )/power2(mstau12 - mu2) - (12*mstau24*mu2*mz2*DeltaInv(mstau22,mstau12,
     mz2))/power2(mstau12 - mu2) + (6*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)
     *power2(mh2))/power2(mstau12 - mu2) - (2*mA2*mu2)/power2(mstau22 - mu2) -
     (4*mC2*mu2)/power2(mstau22 - mu2) - (2*mH2*mu2)/power2(mstau22 - mu2) - (
     4*msntau2*mu2)/power2(mstau22 - mu2) + (8*mstau12*mu2)/power2(mstau22 -
     mu2) + (8*mstau22*mu2)/power2(mstau22 - mu2) + (6*mstau24*mu2)/((mstau12 -
     mstau22)*power2(mstau22 - mu2)) + (6*mstau24*mu2)/((-msntau2 + mu2)*
     power2(mstau22 - mu2)) + (6*mstau24*mu2)/((-mstau12 + mu2)*power2(mstau22
     - mu2)) + (6*power2(mu2))/(mA2*mstau22) + (12*power2(mu2))/(mC2*mstau22) +
     (6*power2(mu2))/(mH2*mstau22) + (18*power2(mu2))/((mstau12 - mstau22)*(
     mstau22 - mu2)) + (16*power2(mu2))/((mstau12 - mu2)*(-mstau22 + mu2)) + (
     24*power2(mu2))/(mA2*mstau22 - mA2*mu2) - (24*power2(mu2))/(mH2*mstau22 -
     mH2*mu2) + 60*DeltaInv(mstau22,mH2,mu2)*power2(mu2) + (30*mH2*DeltaInv(
     mstau22,mH2,mu2)*power2(mu2))/mstau22 + (24*mstau22*DeltaInv(mstau22,
     msntau2,mw2)*power2(mu2))/(msntau2 - mu2) + (12*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12 - mu2) + (12*mstau22*DeltaInv(mstau22,
     mstau12,mz2)*power2(mu2))/(mstau12 - mu2) + (18*mstau22*power2(mu2))/((
     mstau22 - mu2)*power2(mstau12 - mstau22)) + (12*power2(mu2))/power2(
     msntau2 - mu2) + (36*mw2*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (
     12*mstau24*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/power2(msntau2 -
     mu2) - (12*mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/power2(
     msntau2 - mu2) + (12*power2(mu2))/power2(mstau12 - mu2) + (18*mh2*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (18*mz2*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau12 - mu2) - (6*mstau24*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau12 - mu2) - (6*mstau24*DeltaInv(mstau22,
     mstau12,mz2)*power2(mu2))/power2(mstau12 - mu2) - (6*mstau22*mz2*DeltaInv(
     mstau22,mstau12,mz2)*power2(mu2))/power2(mstau12 - mu2) + (30*DeltaInv(
     mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/power2(mstau12 - mu2) - (15*
     power2(mu2))/power2(mstau22 - mu2) + (2*mA2*power2(mu2))/(mstau22*power2(
     mstau22 - mu2)) + (4*mC2*power2(mu2))/(mstau22*power2(mstau22 - mu2)) + (
     2*mH2*power2(mu2))/(mstau22*power2(mstau22 - mu2)) + (4*msntau2*power2(
     mu2))/(mstau22*power2(mstau22 - mu2)) + (4*mstau12*power2(mu2))/(mstau22*
     power2(mstau22 - mu2)) - (12*mstau22*power2(mu2))/((mstau12 - mstau22)*
     power2(mstau22 - mu2)) - (12*mstau24*power2(mu2))/(power2(mstau12 -
     mstau22)*power2(mstau22 - mu2)) - (21*mstau24*power2(mu2))/(power2(msntau2
     - mu2)*power2(mstau22 - mu2)) - (21*mstau24*power2(mu2))/(power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + (12*mu2*DeltaInv(mstau22,msntau2,mw2)*
     power2(mw2))/msntau2 + (12*mstau22*mu2*DeltaInv(mstau22,msntau2,mw2)*
     power2(mw2))/power2(msntau2 - mu2) + (60*DeltaInv(mstau22,mw2,mu2)*power2(
     mu2)*power2(mw2))/power2(msntau2 - mu2) + (6*mu2*DeltaInv(mstau22,mstau12,
     mz2)*power2(mz2))/mstau12 + (6*mstau22*mu2*DeltaInv(mstau22,mstau12,mz2)*
     power2(mz2))/power2(mstau12 - mu2) + (30*DeltaInv(mstau22,mz2,mu2)*power2(
     mu2)*power2(mz2))/power2(mstau12 - mu2) - (6*mu2*DeltaInv(mA2,mstau22,mu2)
     *(mA2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)*power2(mA2) + (-mstau22
     + mu2)*power2(mu2) + 3*power3(mA2)))/(mA2*mstau22) - (12*mu2*DeltaInv(
     mstau22,mC2,mu2)*(mC2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)*power2(
     mC2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mC2)))/(mC2*mstau22) - (18*
     DeltaInv(mstau22,mh2,mu2)*power2(mu2)*power3(mh2))/(mstau22*power2(mstau12
     - mu2)) + (6*power3(mstau22))/((mstau22 - mu2)*power2(mstau12 - mstau22))
     - (3*power3(mstau22))/((msntau2 - mstau22)*power2(msntau2 - mu2)) + (12*
     mu2*DeltaInv(mstau22,msntau2,mw2)*power3(mstau22))/power2(msntau2 - mu2) -
     (3*power3(mstau22))/((mstau12 - mstau22)*power2(mstau12 - mu2)) + (6*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mstau22))/power2(mstau12 - mu2) + (6*
     mu2*DeltaInv(mstau22,mstau12,mz2)*power3(mstau22))/power2(mstau12 - mu2) +
     (6*power3(mstau22))/((msntau2 - mu2)*power2(mstau22 - mu2)) + (6*power3(
     mstau22))/((mstau12 - mu2)*power2(mstau22 - mu2)) + (6*mu2*power3(mstau22)
     )/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) + (15*mu2*power3(
     mstau22))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (15*mu2*power3(
     mstau22))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (24*mstau22*
     power2(mu2))/power3(mstau22 - mu2) + (6*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/mH2 - (6*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mstau22 + (12*
     DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(-mstau22 + mu2) + (6*DeltaInv(
     mstau22,mstau12,mh2)*power3(mu2))/(-mstau22 + mu2) + (6*DeltaInv(mstau22,
     mstau12,mz2)*power3(mu2))/(-mstau22 + mu2) + (12*power3(mu2))/(mstau22*
     power2(msntau2 - mu2)) + (120*mw2*DeltaInv(mstau22,mw2,mu2)*power3(mu2))/
     power2(msntau2 - mu2) + (12*power3(mu2))/(mstau22*power2(mstau12 - mu2)) +
     (60*mh2*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (
     60*mz2*DeltaInv(mstau22,mz2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (30*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau22*power2(mstau12
     - mu2)) + (12*power3(mu2))/(msntau2*power2(mstau22 - mu2)) + (12*power3(
     mu2))/(mstau12*power2(mstau22 - mu2)) + (6*power3(mu2))/((mstau12 -
     mstau22)*power2(mstau22 - mu2)) - (8*power3(mu2))/(mstau22*power2(mstau22
     - mu2)) + power3(mu2)/((-msntau2 + mu2)*power2(mstau22 - mu2)) + power3(
     mu2)/((-mstau12 + mu2)*power2(mstau22 - mu2)) + (24*mstau22*DeltaInv(
     mstau22,msntau2,mw2)*power3(mu2))/power2(mstau22 - mu2) - (12*mstau24*
     DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(msntau2*power2(mstau22 - mu2))
     + (36*mw2*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/power2(mstau22 - mu2)
     + (24*mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(msntau2*
     power2(mstau22 - mu2)) + (18*mh2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2)
     )/power2(mstau22 - mu2) + (12*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power3(mu2))/power2(mstau22 - mu2) + (12*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (6*mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2))
     + (12*mstau22*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/power2(mstau22 -
     mu2) - (6*mstau24*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (18*mz2*DeltaInv(mstau22,mstau12,mz2)*power3(mu2)
     )/power2(mstau22 - mu2) + (12*mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*
     power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (6*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     6*mstau22*power3(mu2))/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) +
     (9*mstau22*power3(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (
     9*mstau22*power3(mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (
     60*DeltaInv(mstau22,mw2,mu2)*power2(mw2)*power3(mu2))/(mstau22*power2(
     msntau2 - mu2)) - (12*DeltaInv(mstau22,msntau2,mw2)*power2(mw2)*power3(
     mu2))/(msntau2*power2(mstau22 - mu2)) + (30*DeltaInv(mstau22,mz2,mu2)*
     power2(mz2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(
     mstau22,mstau12,mz2)*power2(mz2)*power3(mu2))/(mstau12*power2(mstau22 -
     mu2)) + (5*power3(mu2))/power3(mstau22 - mu2) + (4*mA2*power2(mu2))/
     power3(-mstau22 + mu2) + (8*mC2*power2(mu2))/power3(-mstau22 + mu2) + (4*
     mH2*power2(mu2))/power3(-mstau22 + mu2) + (8*msntau2*power2(mu2))/power3(-
     mstau22 + mu2) - (36*DeltaInv(mstau22,mw2,mu2)*power2(mu2)*power3(mw2))/(
     mstau22*power2(msntau2 - mu2)) - (18*DeltaInv(mstau22,mz2,mu2)*power2(mu2)
     *power3(mz2))/(mstau22*power2(mstau12 - mu2)) - (3*power4(mstau22))/(
     power2(msntau2 - mu2)*power2(mstau22 - mu2)) - (3*power4(mstau22))/(
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (6*mstau24*power3(mu2))/((-
     msntau2 + mu2)*power4(mstau22 - mu2)) + (6*mstau24*power3(mu2))/((-mstau12
     + mu2)*power4(mstau22 - mu2)) - (6*mu2*power4(mstau22))/((-msntau2 + mu2)*
     power4(mstau22 - mu2)) - (6*mu2*power4(mstau22))/((-mstau12 + mu2)*power4(
     mstau22 - mu2)) - (6*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mH2*mstau22)
     + (12*DeltaInv(mstau22,mw2,mu2)*power4(mu2))/power2(msntau2 - mu2) - (12*
     mw2*DeltaInv(mstau22,mw2,mu2)*power4(mu2))/(mstau22*power2(msntau2 - mu2))
     + (6*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/power2(mstau12 - mu2) - (6*
     mh2*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2))
     + (6*DeltaInv(mstau22,mz2,mu2)*power4(mu2))/power2(mstau12 - mu2) - (6*
     mz2*DeltaInv(mstau22,mz2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2))
     - (12*DeltaInv(mstau22,msntau2,mw2)*power4(mu2))/power2(mstau22 - mu2) - (
     6*DeltaInv(mstau22,mstau12,mh2)*power4(mu2))/power2(mstau22 - mu2) - (6*
     DeltaInv(mstau22,mstau12,mz2)*power4(mu2))/power2(mstau22 - mu2) - (3*
     mstau22*power4(mu2))/((-msntau2 + mu2)*power4(mstau22 - mu2)) - (3*
     mstau22*power4(mu2))/((-mstau12 + mu2)*power4(mstau22 - mu2)) + (3*power5(
     mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (3*power5(mstau22))/(
     (-mstau12 + mu2)*power4(mstau22 - mu2)) - (12*DeltaInv(mstau22,mw2,mu2)*
     power5(mu2))/(mstau22*power2(msntau2 - mu2)) - (6*DeltaInv(mstau22,mh2,
     mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(mstau22,
     mz2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2))))/16. - (power2(log(
     mstau22/Q2))*(1 + (5*mstau22)/mA2 + (2*mstau22)/mC2 - (3*mstau22)/mH2 - (
     2*mstau22)/(mstau12 - mstau22) + (2*mstau22)/(msntau2 - mu2) + (2*mstau22)
     /(mstau12 - mu2) + (2*mstau12)/(mstau22 - mu2) + (2*mstau24)/((mstau12 -
     mstau22)*(mstau22 - mu2)) + (10*mu2)/mA2 + (4*mu2)/mC2 - (6*mu2)/mH2 - (4*
     mu2)/msntau2 - (4*mu2)/mstau12 + (4*mu2)/(mstau12 - mstau22) + (24*mu2)/
     mstau22 + (8*mu2)/(-mstau12 + mu2) + (2*mu2)/(-mstau22 + mu2) + (8*
     mstau22*mu2)/((mstau12 - mstau22)*(-mstau22 + mu2)) + (8*mstau22*mu2)/((-
     mstau12 + mu2)*(-mstau22 + mu2)) + 10*mH2*mu2*DeltaInv(mstau22,mH2,mu2) +
     4*mstau22*mu2*DeltaInv(mstau22,msntau2,mw2) + (4*mstau24*mu2*DeltaInv(
     mstau22,msntau2,mw2))/msntau2 + (12*mstau24*mu2*DeltaInv(mstau22,msntau2,
     mw2))/(-msntau2 + mu2) - 12*mu2*mw2*DeltaInv(mstau22,msntau2,mw2) - (8*
     mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/msntau2 + (12*mstau22*mu2*
     mw2*DeltaInv(mstau22,msntau2,mw2))/(-msntau2 + mu2) - 6*mh2*mu2*DeltaInv(
     mstau22,mstau12,mh2) + 2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2) - (4*
     mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/(-mstau12 + mu2) + (6*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + 2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mz2) + (2*mstau24*mu2*DeltaInv(mstau22,mstau12,mz2))/mstau12 + (6*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mz2))/(-mstau12 + mu2) - 6*mu2*mz2*DeltaInv(
     mstau22,mstau12,mz2) - (4*mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/
     mstau12 + (6*mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/(-mstau12 +
     mu2) + (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 - (6*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/mstau22 - (2*mstau24)/power2(
     mstau12 - mstau22) + (4*mstau22*mu2)/power2(mstau12 - mstau22) + (8*
     mstau24*mu2)/((-mstau22 + mu2)*power2(mstau12 - mstau22)) - (2*mstau22*
     mu2)/power2(msntau2 - mu2) - (8*mstau24*mu2*mw2*DeltaInv(mstau22,msntau2,
     mw2))/power2(msntau2 - mu2) - (2*mstau22*mu2)/power2(mstau12 - mu2) - (4*
     mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (4*
     mstau24*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/power2(mstau12 - mu2) + (2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 -
     mu2) + (6*mstau12*mu2)/power2(mstau22 - mu2) + (2*mstau24*mu2)/((mstau12 -
     mstau22)*power2(mstau22 - mu2)) + (2*power2(mu2))/(mA2*mstau22) + (4*
     power2(mu2))/(mC2*mstau22) + (2*power2(mu2))/(mH2*mstau22) + (6*power2(
     mu2))/((mstau12 - mstau22)*(mstau22 - mu2)) + (8*power2(mu2))/((mstau12 -
     mu2)*(-mstau22 + mu2)) + (8*power2(mu2))/(mA2*mstau22 - mA2*mu2) - (8*
     power2(mu2))/(mH2*mstau22 - mH2*mu2) + 20*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2) + (10*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 + (8*
     mstau22*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/(msntau2 - mu2) + (4*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12 - mu2) + (4*
     mstau22*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/(mstau12 - mu2) + (6*
     mstau22*power2(mu2))/((mstau22 - mu2)*power2(mstau12 - mstau22)) + (4*
     power2(mu2))/power2(msntau2 - mu2) + (12*mw2*power2(mu2))/(mstau22*power2(
     msntau2 - mu2)) - (4*mstau24*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/
     power2(msntau2 - mu2) - (4*mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*
     power2(mu2))/power2(msntau2 - mu2) + (4*power2(mu2))/power2(mstau12 - mu2)
     + (6*mh2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (6*mz2*power2(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) - (2*mstau24*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) - (2*mstau24*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(mstau12 - mu2) - (2*
     mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(mstau12 -
     mu2) + (10*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/power2(
     mstau12 - mu2) + (4*power2(mu2))/power2(mstau22 - mu2) - (4*mstau22*
     power2(mu2))/((mstau12 - mstau22)*power2(mstau22 - mu2)) + (2*mstau22*
     power2(mu2))/((-msntau2 + mu2)*power2(mstau22 - mu2)) + (2*mstau22*power2(
     mu2))/((-mstau12 + mu2)*power2(mstau22 - mu2)) - (4*mstau24*power2(mu2))/(
     power2(mstau12 - mstau22)*power2(mstau22 - mu2)) - (4*mstau24*power2(mu2))
     /(power2(msntau2 - mu2)*power2(mstau22 - mu2)) - (4*mstau24*power2(mu2))/(
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (4*mu2*DeltaInv(mstau22,
     msntau2,mw2)*power2(mw2))/msntau2 + (4*mstau22*mu2*DeltaInv(mstau22,
     msntau2,mw2)*power2(mw2))/power2(msntau2 - mu2) + (20*DeltaInv(mstau22,
     mw2,mu2)*power2(mu2)*power2(mw2))/power2(msntau2 - mu2) + (2*mu2*DeltaInv(
     mstau22,mstau12,mz2)*power2(mz2))/mstau12 + (2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mz2)*power2(mz2))/power2(mstau12 - mu2) + (10*DeltaInv(
     mstau22,mz2,mu2)*power2(mu2)*power2(mz2))/power2(mstau12 - mu2) - (2*mu2*
     DeltaInv(mA2,mstau22,mu2)*(mA2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)
     *power2(mA2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mA2)))/(mA2*
     mstau22) - (4*mu2*DeltaInv(mstau22,mC2,mu2)*(mC2*mu2*(-10*mstau22 + mu2) -
     5*(mstau22 + mu2)*power2(mC2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(
     mC2)))/(mC2*mstau22) - (6*DeltaInv(mstau22,mh2,mu2)*power2(mu2)*power3(
     mh2))/(mstau22*power2(mstau12 - mu2)) + (2*power3(mstau22))/((-msntau2 +
     mu2)*power2(msntau2 - mstau22)) + (2*power3(mstau22))/((mstau22 - mu2)*
     power2(mstau12 - mstau22)) + (2*power3(mstau22))/((-mstau12 + mu2)*power2(
     mstau12 - mstau22)) + (2*power3(mstau22))/((msntau2 - mstau22)*power2(
     msntau2 - mu2)) + (4*mu2*DeltaInv(mstau22,msntau2,mw2)*power3(mstau22))/
     power2(msntau2 - mu2) - (2*mu2*power3(mstau22))/(power2(msntau2 - mstau22)
     *power2(msntau2 - mu2)) + (2*power3(mstau22))/((mstau12 - mstau22)*power2(
     mstau12 - mu2)) + (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mstau22))/
     power2(mstau12 - mu2) + (2*mu2*DeltaInv(mstau22,mstau12,mz2)*power3(
     mstau22))/power2(mstau12 - mu2) - (2*mu2*power3(mstau22))/(power2(mstau12
     - mstau22)*power2(mstau12 - mu2)) + (2*power3(mstau22))/((msntau2 - mu2)*
     power2(mstau22 - mu2)) + (2*power3(mstau22))/((mstau12 - mu2)*power2(
     mstau22 - mu2)) + (2*mu2*power3(mstau22))/(power2(mstau12 - mstau22)*
     power2(mstau22 - mu2)) + (2*mu2*power3(mstau22))/(power2(msntau2 - mu2)*
     power2(mstau22 - mu2)) + (2*mu2*power3(mstau22))/(power2(mstau12 - mu2)*
     power2(mstau22 - mu2)) + (4*mstau12*power2(mu2))/power3(mstau22 - mu2) + (
     16*mstau22*power2(mu2))/power3(mstau22 - mu2) + (2*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/mH2 - (2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mstau22
     + (4*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(-mstau22 + mu2) + (2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(-mstau22 + mu2) + (2*DeltaInv(
     mstau22,mstau12,mz2)*power3(mu2))/(-mstau22 + mu2) + (4*power3(mu2))/(
     mstau22*power2(msntau2 - mu2)) + (40*mw2*DeltaInv(mstau22,mw2,mu2)*power3(
     mu2))/power2(msntau2 - mu2) + (4*power3(mu2))/(mstau22*power2(mstau12 -
     mu2)) + (20*mh2*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 -
     mu2) + (20*mz2*DeltaInv(mstau22,mz2,mu2)*power3(mu2))/power2(mstau12 -
     mu2) + (10*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2)) + (4*power3(mu2))/(msntau2*power2(mstau22 - mu2)) +
     (4*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (2*power3(mu2))/((
     mstau12 - mstau22)*power2(mstau22 - mu2)) + (8*mstau22*DeltaInv(mstau22,
     msntau2,mw2)*power3(mu2))/power2(mstau22 - mu2) - (4*mstau24*DeltaInv(
     mstau22,msntau2,mw2)*power3(mu2))/(msntau2*power2(mstau22 - mu2)) + (12*
     mw2*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/power2(mstau22 - mu2) + (8*
     mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(msntau2*power2(
     mstau22 - mu2)) + (6*mh2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/
     power2(mstau22 - mu2) + (4*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/power2(mstau22 - mu2) + (4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)
     *power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (2*mstau24*DeltaInv(
     mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (4*
     mstau22*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/power2(mstau22 - mu2) -
     (2*mstau24*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (6*mz2*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/
     power2(mstau22 - mu2) + (4*mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*
     power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     2*mstau22*power3(mu2))/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) +
     (2*mstau22*power3(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (
     2*mstau22*power3(mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (
     20*DeltaInv(mstau22,mw2,mu2)*power2(mw2)*power3(mu2))/(mstau22*power2(
     msntau2 - mu2)) - (4*DeltaInv(mstau22,msntau2,mw2)*power2(mw2)*power3(mu2)
     )/(msntau2*power2(mstau22 - mu2)) + (10*DeltaInv(mstau22,mz2,mu2)*power2(
     mz2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,
     mstau12,mz2)*power2(mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     4*power3(mu2))/power3(mstau22 - mu2) - (12*DeltaInv(mstau22,mw2,mu2)*
     power2(mu2)*power3(mw2))/(mstau22*power2(msntau2 - mu2)) - (6*DeltaInv(
     mstau22,mz2,mu2)*power2(mu2)*power3(mz2))/(mstau22*power2(mstau12 - mu2))
     + (2*power4(mstau22))/(power2(msntau2 - mstau22)*power2(msntau2 - mu2)) +
     (2*power4(mstau22))/(power2(mstau12 - mstau22)*power2(mstau12 - mu2)) + (
     4*mstau24*power3(mu2))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (4*
     mstau24*power3(mu2))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (4*mu2*
     power4(mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) - (4*mu2*power4(
     mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (2*DeltaInv(mstau22,
     mH2,mu2)*power4(mu2))/(mH2*mstau22) + (4*DeltaInv(mstau22,mw2,mu2)*power4(
     mu2))/power2(msntau2 - mu2) - (4*mw2*DeltaInv(mstau22,mw2,mu2)*power4(mu2)
     )/(mstau22*power2(msntau2 - mu2)) + (2*DeltaInv(mstau22,mh2,mu2)*power4(
     mu2))/power2(mstau12 - mu2) - (2*mh2*DeltaInv(mstau22,mh2,mu2)*power4(mu2)
     )/(mstau22*power2(mstau12 - mu2)) + (2*DeltaInv(mstau22,mz2,mu2)*power4(
     mu2))/power2(mstau12 - mu2) - (2*mz2*DeltaInv(mstau22,mz2,mu2)*power4(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (4*DeltaInv(mstau22,msntau2,mw2)*
     power4(mu2))/power2(mstau22 - mu2) - (2*DeltaInv(mstau22,mstau12,mh2)*
     power4(mu2))/power2(mstau22 - mu2) - (2*DeltaInv(mstau22,mstau12,mz2)*
     power4(mu2))/power2(mstau22 - mu2) - (3*power4(mu2))/power4(mstau22 - mu2)
     - (2*mstau22*power4(mu2))/((-msntau2 + mu2)*power4(mstau22 - mu2)) - (2*
     mstau22*power4(mu2))/((-mstau12 + mu2)*power4(mstau22 - mu2)) + (2*power5(
     mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (2*power5(mstau22))/(
     (-mstau12 + mu2)*power4(mstau22 - mu2)) - (4*DeltaInv(mstau22,mw2,mu2)*
     power5(mu2))/(mstau22*power2(msntau2 - mu2)) - (2*DeltaInv(mstau22,mh2,
     mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,
     mz2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2))))/32. - (log(
     mstau22/Q2)*log(mu2/Q2)*((10*mu2)/mA2 + (4*mu2)/mC2 - (6*mu2)/mH2 + (8*
     mu2)/mstau22 + (6*mu2)/(mstau22 - mu2) + (8*mstau22*mu2)/((-mstau12 + mu2)
     *(-mstau22 + mu2)) + 2*mH2*mu2*DeltaInv(mstau22,mH2,mu2) - (2*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/mstau22 - (4*mstau22*mu2)/power2(
     msntau2 - mu2) - (4*mstau22*mu2)/power2(mstau12 - mu2) + (2*power2(mu2))/(
     mA2*mstau22) + (4*power2(mu2))/(mC2*mstau22) + (2*power2(mu2))/(mH2*
     mstau22) + (8*power2(mu2))/(mA2*mstau22 - mA2*mu2) - (8*power2(mu2))/(mH2*
     mstau22 - mH2*mu2) + 12*DeltaInv(mstau22,mH2,mu2)*power2(mu2) + (2*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 + (3*power2(mu2))/power2(
     msntau2 - mu2) + (4*mw2*power2(mu2))/(mstau22*power2(msntau2 - mu2)) + (3*
     power2(mu2))/power2(mstau12 - mu2) + (2*mh2*power2(mu2))/(mstau22*power2(
     mstau12 - mu2)) + (2*mz2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     2*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/power2(mstau12 - mu2)
     - (17*power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*power2(mu2))/((-
     msntau2 + mu2)*power2(mstau22 - mu2)) + (2*mstau22*power2(mu2))/((-mstau12
     + mu2)*power2(mstau22 - mu2)) - (4*mstau24*power2(mu2))/(power2(msntau2 -
     mu2)*power2(mstau22 - mu2)) - (4*mstau24*power2(mu2))/(power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + (4*DeltaInv(mstau22,mw2,mu2)*power2(mu2)*
     power2(mw2))/power2(msntau2 - mu2) + (2*DeltaInv(mstau22,mz2,mu2)*power2(
     mu2)*power2(mz2))/power2(mstau12 - mu2) - (2*mu2*DeltaInv(mA2,mstau22,mu2)
     *(-(mA2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*power2(mA2) + (-mstau22 +
     mu2)*power2(mu2) + power3(mA2)))/(mA2*mstau22) - (4*mu2*DeltaInv(mstau22,
     mC2,mu2)*(-(mC2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*power2(mC2) + (-
     mstau22 + mu2)*power2(mu2) + power3(mC2)))/(mC2*mstau22) - (2*DeltaInv(
     mstau22,mh2,mu2)*power2(mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2))
     + (2*power3(mstau22))/((msntau2 - mu2)*power2(mstau22 - mu2)) + (2*power3(
     mstau22))/((mstau12 - mu2)*power2(mstau22 - mu2)) + (2*mu2*power3(mstau22)
     )/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (2*mu2*power3(mstau22))/
     (power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (4*mstau22*power2(mu2))/
     power3(msntau2 - mu2) + (4*mstau22*power2(mu2))/power3(mstau12 - mu2) + (
     2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mH2 + (2*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/mstau22 + (4*power3(mu2))/(mstau22*power2(msntau2 - mu2)
     ) + (24*mw2*DeltaInv(mstau22,mw2,mu2)*power3(mu2))/power2(msntau2 - mu2) +
     (4*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (12*mh2*DeltaInv(
     mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (12*mz2*DeltaInv(
     mstau22,mz2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (2*DeltaInv(mstau22,
     mh2,mu2)*power2(mh2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (2*
     mstau22*power3(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (2*
     mstau22*power3(mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (4*
     DeltaInv(mstau22,mw2,mu2)*power2(mw2)*power3(mu2))/(mstau22*power2(msntau2
     - mu2)) + (2*DeltaInv(mstau22,mz2,mu2)*power2(mz2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2)) + (12*power3(mu2))/power3(mstau22 - mu2) - (4*
     DeltaInv(mstau22,mw2,mu2)*power2(mu2)*power3(mw2))/(mstau22*power2(msntau2
     - mu2)) - (2*DeltaInv(mstau22,mz2,mu2)*power2(mu2)*power3(mz2))/(mstau22*
     power2(mstau12 - mu2)) + (4*mstau24*power3(mu2))/((-msntau2 + mu2)*power4(
     mstau22 - mu2)) + (4*mstau24*power3(mu2))/((-mstau12 + mu2)*power4(mstau22
     - mu2)) - (4*mu2*power4(mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2))
     - (4*mu2*power4(mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (2*
     DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mH2*mstau22) + (4*DeltaInv(
     mstau22,mw2,mu2)*power4(mu2))/power2(msntau2 - mu2) + (4*mw2*DeltaInv(
     mstau22,mw2,mu2)*power4(mu2))/(mstau22*power2(msntau2 - mu2)) + (2*
     DeltaInv(mstau22,mh2,mu2)*power4(mu2))/power2(mstau12 - mu2) + (2*mh2*
     DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     2*DeltaInv(mstau22,mz2,mu2)*power4(mu2))/power2(mstau12 - mu2) + (2*mz2*
     DeltaInv(mstau22,mz2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) +
     power4(mu2)/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + power4(mu2)/(
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (3*power4(mu2))/power4(
     mstau22 - mu2) - (2*mstau22*power4(mu2))/((-msntau2 + mu2)*power4(mstau22
     - mu2)) - (2*mstau22*power4(mu2))/((-mstau12 + mu2)*power4(mstau22 - mu2))
     + (2*power5(mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (2*
     power5(mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (4*DeltaInv(
     mstau22,mw2,mu2)*power5(mu2))/(mstau22*power2(msntau2 - mu2)) - (2*
     DeltaInv(mstau22,mh2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     2*DeltaInv(mstau22,mz2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2))))
     /16. + (log(msntau2/Q2)*(-3 - (4*mC2)/msntau2 + (20*msntau2)/mA2 + (4*
     msntau2)/mC2 - (12*msntau2)/mH2 - (4*mstau22)/msntau2 + (4*mC2)/(msntau2 -
     mu2) + (8*mstau22)/(msntau2 - mu2) + (4*msntau2)/(mstau22 - mu2) + (30*
     mu2)/mA2 - (18*mu2)/mH2 + (28*mu2)/msntau2 + (7*mu2)/(msntau2 - mu2) + (3*
     mu2)/(mstau12 - mu2) + (11*mu2)/(-mstau22 + mu2) - (36*mu2*mw2)/(msntau2*
     mstau22 - msntau2*mu2) + 12*mstau22*mu2*DeltaInv(mstau22,msntau2,mw2) + (
     12*mstau24*mu2*DeltaInv(mstau22,msntau2,mw2))/msntau2 + (36*mstau24*mu2*
     DeltaInv(mstau22,msntau2,mw2))/(-msntau2 + mu2) - 24*mu2*mw2*DeltaInv(
     mstau22,msntau2,mw2) - (24*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/
     msntau2 + (24*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/(-msntau2 +
     mu2) + (72*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/(-mstau22 + mu2)
     + (36*mstau24*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/(msntau2*mstau22 -
     msntau2*mu2) - (4*mC2*mu2)/power2(msntau2 - mu2) + (4*msntau2*mu2)/power2(
     msntau2 - mu2) - (4*mstau22*mu2)/power2(msntau2 - mu2) - (12*mu2*mw2)/
     power2(msntau2 - mu2) - (12*mstau24*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))
     /power2(msntau2 - mu2) - (8*msntau2*mu2)/power2(mstau22 - mu2) + (6*
     power2(mu2))/(mA2*msntau2) + (6*power2(mu2))/(mH2*msntau2) + (24*power2(
     mu2))/(mA2*msntau2 - mA2*mu2) - (24*power2(mu2))/(mH2*msntau2 - mH2*mu2) +
     (24*mstau22*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/(msntau2 - mu2) + (
     24*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/(msntau2 - mu2) - (13*
     power2(mu2))/power2(msntau2 - mu2) + (4*mC2*power2(mu2))/(msntau2*power2(
     msntau2 - mu2)) + (4*mstau22*power2(mu2))/(msntau2*power2(msntau2 - mu2))
     - (12*mstau24*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/power2(msntau2 -
     mu2) - (72*mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/power2(
     msntau2 - mu2) + (24*power2(mu2))/power2(mstau22 - mu2) + (72*mw2*power2(
     mu2))/(msntau2*power2(mstau22 - mu2)) + (72*mstau22*mw2*DeltaInv(mstau22,
     msntau2,mw2)*power2(mu2))/power2(mstau22 - mu2) - (36*mstau24*mw2*
     DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/(msntau2*power2(mstau22 - mu2))
     + (12*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/msntau2 + (36*mu2*
     DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(-msntau2 + mu2) + (60*mu2*
     DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(-mstau22 + mu2) - (72*mstau22*
     mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(msntau2*mstau22 - msntau2*
     mu2) - (12*mstau22*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/power2(
     msntau2 - mu2) - (12*DeltaInv(mstau22,msntau2,mw2)*power2(mu2)*power2(mw2)
     )/power2(msntau2 - mu2) + (60*DeltaInv(msntau2,mw2,mu2)*power2(mu2)*
     power2(mw2))/power2(mstau22 - mu2) + (60*DeltaInv(mstau22,msntau2,mw2)*
     power2(mu2)*power2(mw2))/power2(mstau22 - mu2) + (72*mstau22*DeltaInv(
     mstau22,msntau2,mw2)*power2(mu2)*power2(mw2))/(msntau2*power2(mstau22 -
     mu2)) - (6*mu2*DeltaInv(mA2,msntau2,mu2)*(mA2*mu2*(-10*msntau2 + mu2) - 5*
     (msntau2 + mu2)*power2(mA2) + (-msntau2 + mu2)*power2(mu2) + 3*power3(mA2)
     ))/(mA2*msntau2) - (6*mu2*DeltaInv(mH2,msntau2,mu2)*(mH2*mu2*(-10*msntau2
     + mu2) - 5*(msntau2 + mu2)*power2(mH2) + (-msntau2 + mu2)*power2(mu2) + 3*
     power3(mH2)))/(mH2*msntau2) + (6*power3(mstau22))/((-msntau2 + mu2)*
     power2(msntau2 - mstau22)) + (6*power3(mstau22))/((msntau2 - mstau22)*
     power2(msntau2 - mu2)) + (12*mu2*DeltaInv(mstau22,msntau2,mw2)*power3(
     mstau22))/power2(msntau2 - mu2) - (6*mu2*power3(mstau22))/(power2(msntau2
     - mstau22)*power2(msntau2 - mu2)) + (12*msntau2*power2(mu2))/power3(
     msntau2 - mu2) + (12*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(-mstau22
     + mu2) - (4*power3(mu2))/(msntau2*power2(msntau2 - mu2)) + (3*power3(mu2))
     /((-mstau12 + mu2)*power2(msntau2 - mu2)) + power3(mu2)/((-mstau22 + mu2)*
     power2(msntau2 - mu2)) + (24*power3(mu2))/(msntau2*power2(mstau22 - mu2))
     + (120*mw2*DeltaInv(msntau2,mw2,mu2)*power3(mu2))/power2(mstau22 - mu2) +
     (24*mstau22*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/power2(mstau22 -
     mu2) - (12*mstau24*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(msntau2*
     power2(mstau22 - mu2)) + (48*mw2*DeltaInv(mstau22,msntau2,mw2)*power3(mu2)
     )/power2(mstau22 - mu2) + (24*mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*
     power3(mu2))/(msntau2*power2(mstau22 - mu2)) + (60*DeltaInv(msntau2,mw2,
     mu2)*power2(mw2)*power3(mu2))/(msntau2*power2(mstau22 - mu2)) - (12*
     DeltaInv(mstau22,msntau2,mw2)*power2(mw2)*power3(mu2))/(msntau2*power2(
     mstau22 - mu2)) + power3(mu2)/power3(msntau2 - mu2) + (8*mC2*power2(mu2))/
     power3(-msntau2 + mu2) + (36*mu2*DeltaInv(mstau22,msntau2,mw2)*power3(mw2)
     )/(msntau2*mstau22 - msntau2*mu2) + (12*mu2*DeltaInv(mstau22,msntau2,mw2)*
     power3(mw2))/power2(msntau2 - mu2) - (36*DeltaInv(msntau2,mw2,mu2)*power2(
     mu2)*power3(mw2))/(msntau2*power2(mstau22 - mu2)) - (36*DeltaInv(mstau22,
     msntau2,mw2)*power2(mu2)*power3(mw2))/(msntau2*power2(mstau22 - mu2)) + (
     6*power4(mstau22))/(power2(msntau2 - mstau22)*power2(msntau2 - mu2)) + (
     12*DeltaInv(msntau2,mw2,mu2)*power4(mu2))/power2(mstau22 - mu2) - (12*mw2*
     DeltaInv(msntau2,mw2,mu2)*power4(mu2))/(msntau2*power2(mstau22 - mu2)) - (
     12*DeltaInv(mstau22,msntau2,mw2)*power4(mu2))/power2(mstau22 - mu2) - (12*
     DeltaInv(msntau2,mw2,mu2)*power5(mu2))/(msntau2*power2(mstau22 - mu2))))/
     16. - (power2(log(msntau2/Q2))*(-3 + (5*msntau2)/mA2 - (3*msntau2)/mH2 + (
     2*mstau22)/(msntau2 - mu2) + (10*mu2)/mA2 - (6*mu2)/mH2 + (8*mu2)/msntau2
     + (2*mu2)/(msntau2 - mu2) + (4*mu2)/(-mstau22 + mu2) - (12*mu2*mw2)/(
     msntau2*mstau22 - msntau2*mu2) + 4*mstau22*mu2*DeltaInv(mstau22,msntau2,
     mw2) + (4*mstau24*mu2*DeltaInv(mstau22,msntau2,mw2))/msntau2 + (12*
     mstau24*mu2*DeltaInv(mstau22,msntau2,mw2))/(-msntau2 + mu2) - 8*mu2*mw2*
     DeltaInv(mstau22,msntau2,mw2) - (8*mstau22*mu2*mw2*DeltaInv(mstau22,
     msntau2,mw2))/msntau2 + (8*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/
     (-msntau2 + mu2) + (24*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/(-
     mstau22 + mu2) + (12*mstau24*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/(
     msntau2*mstau22 - msntau2*mu2) + (2*mstau22*mu2)/power2(msntau2 - mu2) - (
     4*mu2*mw2)/power2(msntau2 - mu2) - (4*mstau24*mu2*mw2*DeltaInv(mstau22,
     msntau2,mw2))/power2(msntau2 - mu2) + (2*power2(mu2))/(mA2*msntau2) + (2*
     power2(mu2))/(mH2*msntau2) + (8*power2(mu2))/(mA2*msntau2 - mA2*mu2) - (8*
     power2(mu2))/(mH2*msntau2 - mH2*mu2) + (8*mstau22*DeltaInv(mstau22,
     msntau2,mw2)*power2(mu2))/(msntau2 - mu2) + (8*mw2*DeltaInv(mstau22,
     msntau2,mw2)*power2(mu2))/(msntau2 - mu2) + (4*power2(mu2))/power2(msntau2
     - mu2) - (4*mstau24*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/power2(
     msntau2 - mu2) - (24*mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(mu2)
     )/power2(msntau2 - mu2) + (8*power2(mu2))/power2(mstau22 - mu2) + (24*mw2*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (24*mstau22*mw2*DeltaInv(
     mstau22,msntau2,mw2)*power2(mu2))/power2(mstau22 - mu2) - (12*mstau24*mw2*
     DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/(msntau2*power2(mstau22 - mu2))
     + (4*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/msntau2 + (12*mu2*
     DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(-msntau2 + mu2) + (20*mu2*
     DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(-mstau22 + mu2) - (24*mstau22*
     mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(msntau2*mstau22 - msntau2*
     mu2) - (4*mstau22*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/power2(
     msntau2 - mu2) - (4*DeltaInv(mstau22,msntau2,mw2)*power2(mu2)*power2(mw2))
     /power2(msntau2 - mu2) + (20*DeltaInv(msntau2,mw2,mu2)*power2(mu2)*power2(
     mw2))/power2(mstau22 - mu2) + (20*DeltaInv(mstau22,msntau2,mw2)*power2(
     mu2)*power2(mw2))/power2(mstau22 - mu2) + (24*mstau22*DeltaInv(mstau22,
     msntau2,mw2)*power2(mu2)*power2(mw2))/(msntau2*power2(mstau22 - mu2)) - (
     2*mu2*DeltaInv(mA2,msntau2,mu2)*(mA2*mu2*(-10*msntau2 + mu2) - 5*(msntau2
     + mu2)*power2(mA2) + (-msntau2 + mu2)*power2(mu2) + 3*power3(mA2)))/(mA2*
     msntau2) - (2*mu2*DeltaInv(mH2,msntau2,mu2)*(mH2*mu2*(-10*msntau2 + mu2) -
     5*(msntau2 + mu2)*power2(mH2) + (-msntau2 + mu2)*power2(mu2) + 3*power3(
     mH2)))/(mH2*msntau2) + (2*power3(mstau22))/((-msntau2 + mu2)*power2(
     msntau2 - mstau22)) + (2*power3(mstau22))/((msntau2 - mstau22)*power2(
     msntau2 - mu2)) + (4*mu2*DeltaInv(mstau22,msntau2,mw2)*power3(mstau22))/
     power2(msntau2 - mu2) - (2*mu2*power3(mstau22))/(power2(msntau2 - mstau22)
     *power2(msntau2 - mu2)) + (8*msntau2*power2(mu2))/power3(msntau2 - mu2) +
     (4*mstau22*power2(mu2))/power3(msntau2 - mu2) + (4*DeltaInv(mstau22,
     msntau2,mw2)*power3(mu2))/(-mstau22 + mu2) + (8*power3(mu2))/(msntau2*
     power2(mstau22 - mu2)) + (40*mw2*DeltaInv(msntau2,mw2,mu2)*power3(mu2))/
     power2(mstau22 - mu2) + (8*mstau22*DeltaInv(mstau22,msntau2,mw2)*power3(
     mu2))/power2(mstau22 - mu2) - (4*mstau24*DeltaInv(mstau22,msntau2,mw2)*
     power3(mu2))/(msntau2*power2(mstau22 - mu2)) + (16*mw2*DeltaInv(mstau22,
     msntau2,mw2)*power3(mu2))/power2(mstau22 - mu2) + (8*mstau22*mw2*DeltaInv(
     mstau22,msntau2,mw2)*power3(mu2))/(msntau2*power2(mstau22 - mu2)) + (20*
     DeltaInv(msntau2,mw2,mu2)*power2(mw2)*power3(mu2))/(msntau2*power2(mstau22
     - mu2)) - (4*DeltaInv(mstau22,msntau2,mw2)*power2(mw2)*power3(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (4*power3(mu2))/power3(msntau2 - mu2) + (
     12*mu2*DeltaInv(mstau22,msntau2,mw2)*power3(mw2))/(msntau2*mstau22 -
     msntau2*mu2) + (4*mu2*DeltaInv(mstau22,msntau2,mw2)*power3(mw2))/power2(
     msntau2 - mu2) - (12*DeltaInv(msntau2,mw2,mu2)*power2(mu2)*power3(mw2))/(
     msntau2*power2(mstau22 - mu2)) - (12*DeltaInv(mstau22,msntau2,mw2)*power2(
     mu2)*power3(mw2))/(msntau2*power2(mstau22 - mu2)) + (2*power4(mstau22))/(
     power2(msntau2 - mstau22)*power2(msntau2 - mu2)) + (4*DeltaInv(msntau2,
     mw2,mu2)*power4(mu2))/power2(mstau22 - mu2) - (4*mw2*DeltaInv(msntau2,mw2,
     mu2)*power4(mu2))/(msntau2*power2(mstau22 - mu2)) - (4*DeltaInv(mstau22,
     msntau2,mw2)*power4(mu2))/power2(mstau22 - mu2) - (3*power4(mu2))/power4(
     msntau2 - mu2) - (4*DeltaInv(msntau2,mw2,mu2)*power5(mu2))/(msntau2*
     power2(mstau22 - mu2))))/32. - (power2(Pi)*(8 - (16*mC2)/mA2 + (2*mC2)/(
     mA2 - mC2) - (2*mC2)/(mC2 - mH2) - (26*mA2)/mH2 + (16*mC2)/mH2 - (8*mH2)/
     mA2 + (2*mA2)/(-mA2 + mH2) + (10*msntau2)/mA2 - (6*msntau2)/mH2 + (10*
     mstau12)/mA2 - (6*mstau12)/mH2 + (10*mstau22)/mA2 + (4*mstau22)/mC2 - (6*
     mstau22)/mH2 - (4*mstau22)/(mstau12 - mstau22) + (4*mstau22)/(msntau2 -
     mu2) + (4*mstau22)/(mstau12 - mu2) + (4*mstau12)/(mstau22 - mu2) + (4*
     mstau24)/((mstau12 - mstau22)*(mstau22 - mu2)) + (30*mu2)/mA2 + (4*mu2)/
     mC2 - (18*mu2)/mH2 + (8*mu2)/msntau2 + (8*mu2)/mstau12 + (8*mu2)/(mstau12
     - mstau22) + (32*mu2)/mstau22 + (4*mu2)/(msntau2 - mu2) + (12*mu2)/(-
     mstau12 + mu2) + (4*mu2)/(-mstau22 + mu2) + (16*mstau22*mu2)/((mstau12 -
     mstau22)*(-mstau22 + mu2)) + (16*mstau22*mu2)/((-mstau12 + mu2)*(-mstau22
     + mu2)) - (8*mh2*mu2)/(mstau12*mstau22 - mstau12*mu2) - (16*mu2*mw2)/(
     msntau2*mstau22 - msntau2*mu2) - (8*mu2*mz2)/(mstau12*mstau22 - mstau12*
     mu2) + 16*mA2*mu2*DeltaInv(mA2,mstau22,mu2) + 16*mH2*mu2*DeltaInv(mH2,
     msntau2,mu2) + 16*mH2*mu2*DeltaInv(mH2,mstau12,mu2) + 32*mC2*mu2*DeltaInv(
     mstau22,mC2,mu2) + 16*mH2*mu2*DeltaInv(mstau22,mH2,mu2) + 8*mstau22*mu2*
     DeltaInv(mstau22,msntau2,mw2) + (8*mstau24*mu2*DeltaInv(mstau22,msntau2,
     mw2))/msntau2 + (24*mstau24*mu2*DeltaInv(mstau22,msntau2,mw2))/(-msntau2 +
     mu2) - 8*mu2*mw2*DeltaInv(mstau22,msntau2,mw2) - (16*mstau22*mu2*mw2*
     DeltaInv(mstau22,msntau2,mw2))/msntau2 + (32*mstau22*mu2*mw2*DeltaInv(
     mstau22,msntau2,mw2))/(-msntau2 + mu2) + (32*mstau22*mu2*mw2*DeltaInv(
     mstau22,msntau2,mw2))/(-mstau22 + mu2) + (16*mstau24*mu2*mw2*DeltaInv(
     mstau22,msntau2,mw2))/(msntau2*mstau22 - msntau2*mu2) - 4*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2) + 4*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2) - (8*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (4*
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (16*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (12*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (16*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (8*mh2*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + 4*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mz2) + (4*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mz2))/mstau12 + (12*mstau24*mu2*DeltaInv(mstau22,mstau12,
     mz2))/(-mstau12 + mu2) - 4*mu2*mz2*DeltaInv(mstau22,mstau12,mz2) - (8*
     mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/mstau12 + (16*mstau22*mu2*
     mz2*DeltaInv(mstau22,mstau12,mz2))/(-mstau12 + mu2) + (16*mstau22*mu2*mz2*
     DeltaInv(mstau22,mstau12,mz2))/(-mstau22 + mu2) + (8*mstau24*mu2*mz2*
     DeltaInv(mstau22,mstau12,mz2))/(mstau12*mstau22 - mstau12*mu2) - (8*mu2*
     DeltaInv(mA2,mstau22,mu2)*power2(mA2))/mstau22 - (16*mu2*DeltaInv(mstau22,
     mC2,mu2)*power2(mC2))/mstau22 + (15*power2(mC2))/power2(mA2) - (2*power2(
     mC2))/power2(mA2 - mC2) + (4*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)
     )/mstau12 + (12*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 +
     mu2) + (16*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau22 + mu2)
     - (16*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*
     mstau22 - mstau12*mu2) - (2*power2(mA2))/power2(mA2 - mH2) - (2*power2(
     mC2))/power2(mC2 - mH2) - (9*power2(mA2))/power2(mH2) - (9*power2(mC2))/
     power2(mH2) - (8*mu2*DeltaInv(mH2,msntau2,mu2)*power2(mH2))/msntau2 - (8*
     mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/mstau12 - (8*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/mstau22 + (15*power2(mH2))/power2(mA2) - (4*
     mstau24)/power2(mstau12 - mstau22) + (8*mstau22*mu2)/power2(mstau12 -
     mstau22) + (16*mstau24*mu2)/((-mstau22 + mu2)*power2(mstau12 - mstau22)) -
     (4*mstau22*mu2)/power2(msntau2 - mu2) - (8*mu2*mw2)/power2(msntau2 - mu2)
     - (8*mstau24*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/power2(msntau2 - mu2)
     - (4*mh2*mu2)/power2(mstau12 - mu2) - (4*mstau22*mu2)/power2(mstau12 -
     mu2) - (4*mu2*mz2)/power2(mstau12 - mu2) - (4*mh2*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (4*mstau24*mu2*mz2*DeltaInv(
     mstau22,mstau12,mz2))/power2(mstau12 - mu2) - (4*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (8*mstau12*mu2)/
     power2(mstau22 - mu2) + (4*mstau24*mu2)/((mstau12 - mstau22)*power2(
     mstau22 - mu2)) + (4*power2(mu2))/(mA2*msntau2) + (4*power2(mu2))/(mH2*
     msntau2) + (4*power2(mu2))/(mA2*mstau12) + (4*power2(mu2))/(mH2*mstau12) +
     (4*power2(mu2))/(mA2*mstau22) + (8*power2(mu2))/(mC2*mstau22) + (4*power2(
     mu2))/(mH2*mstau22) + (12*power2(mu2))/((mstau12 - mstau22)*(mstau22 -
     mu2)) + (16*power2(mu2))/((-mstau12 + mu2)*(-mstau22 + mu2)) + (16*power2(
     mu2))/(mA2*msntau2 - mA2*mu2) + (16*power2(mu2))/(mA2*mstau12 - mA2*mu2) +
     (16*power2(mu2))/(mA2*mstau22 - mA2*mu2) - (16*power2(mu2))/(mH2*msntau2 -
     mH2*mu2) - (16*power2(mu2))/(mH2*mstau12 - mH2*mu2) - (16*power2(mu2))/(
     mH2*mstau22 - mH2*mu2) + 28*DeltaInv(mA2,mstau22,mu2)*power2(mu2) + (12*
     mA2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/mstau22 + 28*DeltaInv(mH2,
     msntau2,mu2)*power2(mu2) + (12*mH2*DeltaInv(mH2,msntau2,mu2)*power2(mu2))/
     msntau2 + 28*DeltaInv(mH2,mstau12,mu2)*power2(mu2) + (12*mH2*DeltaInv(mH2,
     mstau12,mu2)*power2(mu2))/mstau12 + 56*DeltaInv(mstau22,mC2,mu2)*power2(
     mu2) + (24*mC2*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/mstau22 + 28*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2) + (12*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/mstau22 + (16*mstau22*DeltaInv(mstau22,msntau2,mw2)*power2(
     mu2))/(msntau2 - mu2) + (16*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))
     /(msntau2 - mu2) + (8*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12 - mu2) + (8*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12 - mu2) + (8*mstau22*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/(
     mstau12 - mu2) + (8*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/(
     mstau12 - mu2) + (12*mstau22*power2(mu2))/((mstau22 - mu2)*power2(mstau12
     - mstau22)) + (8*power2(mu2))/power2(msntau2 - mu2) + (16*mw2*power2(mu2))
     /(mstau22*power2(msntau2 - mu2)) - (8*mstau24*DeltaInv(mstau22,msntau2,
     mw2)*power2(mu2))/power2(msntau2 - mu2) - (32*mstau22*mw2*DeltaInv(
     mstau22,msntau2,mw2)*power2(mu2))/power2(msntau2 - mu2) + (8*power2(mu2))/
     power2(mstau12 - mu2) + (8*mh2*power2(mu2))/(mstau22*power2(mstau12 - mu2)
     ) + (8*mz2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (16*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) - (4*
     mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) -
     (4*mstau24*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(mstau12 -
     mu2) - (16*mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(
     mstau12 - mu2) + (16*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/
     power2(mstau12 - mu2) - (4*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/power2(mstau12 - mu2) + (32*power2(mu2))/power2(mstau22 -
     mu2) + (16*mh2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (8*mstau22*
     power2(mu2))/((mstau12 - mstau22)*power2(mstau22 - mu2)) + (4*mstau22*
     power2(mu2))/((-msntau2 + mu2)*power2(mstau22 - mu2)) + (4*mstau22*power2(
     mu2))/((-mstau12 + mu2)*power2(mstau22 - mu2)) + (32*mw2*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (16*mz2*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (32*mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(
     mu2))/power2(mstau22 - mu2) - (16*mstau24*mw2*DeltaInv(mstau22,msntau2,
     mw2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (16*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (8*mh2*
     mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (16*mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/
     power2(mstau22 - mu2) - (8*mstau24*mz2*DeltaInv(mstau22,mstau12,mz2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (16*DeltaInv(mstau12,mh2,
     mu2)*power2(mh2)*power2(mu2))/power2(mstau22 - mu2) + (16*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/power2(mstau22 - mu2) + (16*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (8*mstau24*power2(mu2))/(power2(mstau12 -
     mstau22)*power2(mstau22 - mu2)) - (8*mstau24*power2(mu2))/(power2(msntau2
     - mu2)*power2(mstau22 - mu2)) - (8*mstau24*power2(mu2))/(power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + (8*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(
     mw2))/msntau2 + (24*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(-
     msntau2 + mu2) + (32*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(-
     mstau22 + mu2) - (32*mstau22*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2)
     )/(msntau2*mstau22 - msntau2*mu2) - (8*mstau22*mu2*DeltaInv(mstau22,
     msntau2,mw2)*power2(mw2))/power2(msntau2 - mu2) - (8*DeltaInv(mstau22,
     msntau2,mw2)*power2(mu2)*power2(mw2))/power2(msntau2 - mu2) + (32*
     DeltaInv(mstau22,mw2,mu2)*power2(mu2)*power2(mw2))/power2(msntau2 - mu2) +
     (32*DeltaInv(msntau2,mw2,mu2)*power2(mu2)*power2(mw2))/power2(mstau22 -
     mu2) + (32*DeltaInv(mstau22,msntau2,mw2)*power2(mu2)*power2(mw2))/power2(
     mstau22 - mu2) + (32*mstau22*DeltaInv(mstau22,msntau2,mw2)*power2(mu2)*
     power2(mw2))/(msntau2*power2(mstau22 - mu2)) + (4*mu2*DeltaInv(mstau22,
     mstau12,mz2)*power2(mz2))/mstau12 + (12*mu2*DeltaInv(mstau22,mstau12,mz2)*
     power2(mz2))/(-mstau12 + mu2) + (16*mu2*DeltaInv(mstau22,mstau12,mz2)*
     power2(mz2))/(-mstau22 + mu2) - (16*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mz2)*power2(mz2))/(mstau12*mstau22 - mstau12*mu2) - (4*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mz2)*power2(mz2))/power2(mstau12 - mu2) - (4*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*power2(mz2))/power2(mstau12 -
     mu2) + (16*DeltaInv(mstau22,mz2,mu2)*power2(mu2)*power2(mz2))/power2(
     mstau12 - mu2) + (16*DeltaInv(mstau12,mz2,mu2)*power2(mu2)*power2(mz2))/
     power2(mstau22 - mu2) + (16*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*
     power2(mz2))/power2(mstau22 - mu2) + (16*mstau22*DeltaInv(mstau22,mstau12,
     mz2)*power2(mu2)*power2(mz2))/(mstau12*power2(mstau22 - mu2)) + (2*power3(
     mA2))/(mH2*power2(mA2 - mH2)) - (4*mu2*DeltaInv(mA2,msntau2,mu2)*(-7*mA2*
     msntau2*mu2 - (4*msntau2 + 3*mu2)*power2(mA2) + (-msntau2 + mu2)*power2(
     mu2) + 2*power3(mA2)))/(mA2*msntau2) - (4*mu2*DeltaInv(mA2,mstau12,mu2)*(-
     7*mA2*mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mA2) + (-mstau12 + mu2)*
     power2(mu2) + 2*power3(mA2)))/(mA2*mstau12) + (2*power3(mC2))/(mA2*power2(
     mA2 - mC2)) + (2*power3(mC2))/(mH2*power2(mC2 - mH2)) + (8*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2) + (4*
     mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (8*
     DeltaInv(mstau22,mh2,mu2)*power2(mu2)*power3(mh2))/(mstau22*power2(mstau12
     - mu2)) - (8*DeltaInv(mstau12,mh2,mu2)*power2(mu2)*power3(mh2))/(mstau12*
     power2(mstau22 - mu2)) - (8*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)*
     power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (4*power3(mstau22))/((-
     msntau2 + mu2)*power2(msntau2 - mstau22)) + (4*power3(mstau22))/((mstau22
     - mu2)*power2(mstau12 - mstau22)) + (4*power3(mstau22))/((-mstau12 + mu2)*
     power2(mstau12 - mstau22)) + (4*power3(mstau22))/((msntau2 - mstau22)*
     power2(msntau2 - mu2)) + (8*mu2*DeltaInv(mstau22,msntau2,mw2)*power3(
     mstau22))/power2(msntau2 - mu2) - (4*mu2*power3(mstau22))/(power2(msntau2
     - mstau22)*power2(msntau2 - mu2)) + (4*power3(mstau22))/((mstau12 -
     mstau22)*power2(mstau12 - mu2)) + (4*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power3(mstau22))/power2(mstau12 - mu2) + (4*mu2*DeltaInv(mstau22,mstau12,
     mz2)*power3(mstau22))/power2(mstau12 - mu2) - (4*mu2*power3(mstau22))/(
     power2(mstau12 - mstau22)*power2(mstau12 - mu2)) + (4*power3(mstau22))/((
     msntau2 - mu2)*power2(mstau22 - mu2)) + (4*power3(mstau22))/((mstau12 -
     mu2)*power2(mstau22 - mu2)) + (4*mu2*power3(mstau22))/(power2(mstau12 -
     mstau22)*power2(mstau22 - mu2)) + (4*mu2*power3(mstau22))/(power2(msntau2
     - mu2)*power2(mstau22 - mu2)) + (4*mu2*power3(mstau22))/(power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + (4*DeltaInv(mA2,mstau22,mu2)*power3(mu2))/
     mA2 + (4*DeltaInv(mH2,msntau2,mu2)*power3(mu2))/mH2 + (4*DeltaInv(mH2,
     mstau12,mu2)*power3(mu2))/mH2 + (8*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/
     mC2 + (4*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mH2 + (8*DeltaInv(mstau22,
     msntau2,mw2)*power3(mu2))/(-mstau22 + mu2) + (4*DeltaInv(mstau22,mstau12,
     mh2)*power3(mu2))/(-mstau22 + mu2) + (4*DeltaInv(mstau22,mstau12,mz2)*
     power3(mu2))/(-mstau22 + mu2) + (8*power3(mu2))/(mstau22*power2(msntau2 -
     mu2)) + (56*mw2*DeltaInv(mstau22,mw2,mu2)*power3(mu2))/power2(msntau2 -
     mu2) + (8*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (28*mh2*DeltaInv(
     mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (28*mz2*DeltaInv(
     mstau22,mz2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (12*DeltaInv(
     mstau22,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))
     + (16*power3(mu2))/(msntau2*power2(mstau22 - mu2)) + (16*power3(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (4*power3(mu2))/((mstau12 - mstau22)*
     power2(mstau22 - mu2)) + (56*mw2*DeltaInv(msntau2,mw2,mu2)*power3(mu2))/
     power2(mstau22 - mu2) + (28*mh2*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/
     power2(mstau22 - mu2) + (28*mz2*DeltaInv(mstau12,mz2,mu2)*power3(mu2))/
     power2(mstau22 - mu2) + (16*mstau22*DeltaInv(mstau22,msntau2,mw2)*power3(
     mu2))/power2(mstau22 - mu2) - (8*mstau24*DeltaInv(mstau22,msntau2,mw2)*
     power3(mu2))/(msntau2*power2(mstau22 - mu2)) + (24*mw2*DeltaInv(mstau22,
     msntau2,mw2)*power3(mu2))/power2(mstau22 - mu2) + (16*mstau22*mw2*
     DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(msntau2*power2(mstau22 - mu2))
     + (12*mh2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2)
     + (8*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 -
     mu2) + (8*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (4*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/(mstau12*power2(mstau22 - mu2)) + (8*mstau22*DeltaInv(mstau22,
     mstau12,mz2)*power3(mu2))/power2(mstau22 - mu2) - (4*mstau24*DeltaInv(
     mstau22,mstau12,mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (12*
     mz2*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/power2(mstau22 - mu2) + (8*
     mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (12*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power3(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (4*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (4*mstau22*power3(mu2)
     )/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) + (4*mstau22*power3(
     mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (4*mstau22*power3(
     mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (24*DeltaInv(
     mstau22,mw2,mu2)*power2(mw2)*power3(mu2))/(mstau22*power2(msntau2 - mu2))
     + (24*DeltaInv(msntau2,mw2,mu2)*power2(mw2)*power3(mu2))/(msntau2*power2(
     mstau22 - mu2)) - (8*DeltaInv(mstau22,msntau2,mw2)*power2(mw2)*power3(mu2)
     )/(msntau2*power2(mstau22 - mu2)) + (12*DeltaInv(mstau22,mz2,mu2)*power2(
     mz2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (12*DeltaInv(mstau12,
     mz2,mu2)*power2(mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (4*
     DeltaInv(mstau22,mstau12,mz2)*power2(mz2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (16*mu2*DeltaInv(mstau22,msntau2,mw2)*power3(mw2))/(
     msntau2*mstau22 - msntau2*mu2) + (8*mu2*DeltaInv(mstau22,msntau2,mw2)*
     power3(mw2))/power2(msntau2 - mu2) - (16*DeltaInv(mstau22,mw2,mu2)*power2(
     mu2)*power3(mw2))/(mstau22*power2(msntau2 - mu2)) - (16*DeltaInv(msntau2,
     mw2,mu2)*power2(mu2)*power3(mw2))/(msntau2*power2(mstau22 - mu2)) - (16*
     DeltaInv(mstau22,msntau2,mw2)*power2(mu2)*power3(mw2))/(msntau2*power2(
     mstau22 - mu2)) + (8*mu2*DeltaInv(mstau22,mstau12,mz2)*power3(mz2))/(
     mstau12*mstau22 - mstau12*mu2) + (4*mu2*DeltaInv(mstau22,mstau12,mz2)*
     power3(mz2))/power2(mstau12 - mu2) - (8*DeltaInv(mstau22,mz2,mu2)*power2(
     mu2)*power3(mz2))/(mstau22*power2(mstau12 - mu2)) - (8*DeltaInv(mstau12,
     mz2,mu2)*power2(mu2)*power3(mz2))/(mstau12*power2(mstau22 - mu2)) - (8*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*power3(mz2))/(mstau12*power2(
     mstau22 - mu2)) + (4*power4(mstau22))/(power2(msntau2 - mstau22)*power2(
     msntau2 - mu2)) + (4*power4(mstau22))/(power2(mstau12 - mstau22)*power2(
     mstau12 - mu2)) + (8*mstau24*power3(mu2))/((-msntau2 + mu2)*power4(mstau22
     - mu2)) + (8*mstau24*power3(mu2))/((-mstau12 + mu2)*power4(mstau22 - mu2))
     - (8*mu2*power4(mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) - (8*
     mu2*power4(mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (4*
     DeltaInv(mA2,mstau22,mu2)*power4(mu2))/(mA2*mstau22) - (4*DeltaInv(mH2,
     msntau2,mu2)*power4(mu2))/(mH2*msntau2) - (4*DeltaInv(mH2,mstau12,mu2)*
     power4(mu2))/(mH2*mstau12) - (8*DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(
     mC2*mstau22) - (4*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mH2*mstau22) + (
     8*DeltaInv(mstau22,mw2,mu2)*power4(mu2))/power2(msntau2 - mu2) + (4*
     DeltaInv(mstau22,mh2,mu2)*power4(mu2))/power2(mstau12 - mu2) + (4*
     DeltaInv(mstau22,mz2,mu2)*power4(mu2))/power2(mstau12 - mu2) + (8*
     DeltaInv(msntau2,mw2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (4*
     DeltaInv(mstau12,mh2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (4*
     DeltaInv(mstau12,mz2,mu2)*power4(mu2))/power2(mstau22 - mu2) - (8*
     DeltaInv(mstau22,msntau2,mw2)*power4(mu2))/power2(mstau22 - mu2) - (4*
     DeltaInv(mstau22,mstau12,mh2)*power4(mu2))/power2(mstau22 - mu2) - (4*
     DeltaInv(mstau22,mstau12,mz2)*power4(mu2))/power2(mstau22 - mu2) - (4*
     mstau22*power4(mu2))/((-msntau2 + mu2)*power4(mstau22 - mu2)) - (4*
     mstau22*power4(mu2))/((-mstau12 + mu2)*power4(mstau22 - mu2)) + (4*power5(
     mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (4*power5(mstau22))/(
     (-mstau12 + mu2)*power4(mstau22 - mu2)) - (8*DeltaInv(mstau22,mw2,mu2)*
     power5(mu2))/(mstau22*power2(msntau2 - mu2)) - (4*DeltaInv(mstau22,mh2,
     mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)) - (4*DeltaInv(mstau22,
     mz2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)) - (8*DeltaInv(
     msntau2,mw2,mu2)*power5(mu2))/(msntau2*power2(mstau22 - mu2)) - (4*
     DeltaInv(mstau12,mh2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     4*DeltaInv(mstau12,mz2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2))))
     /192. - (power2(log(mu2/Q2))*((15*mu2)/mA2 + (2*mu2)/mC2 - (9*mu2)/mH2 + (
     2*mu2)/(msntau2 - mu2) + (6*mu2)/(mstau22 - mu2) + (6*mu2)/(-mstau12 +
     mu2) + (8*mstau22*mu2)/((-mstau12 + mu2)*(-mstau22 + mu2)) - (4*mstau22*
     mu2)/power2(msntau2 - mu2) - (4*mstau22*mu2)/power2(mstau12 - mu2) - (2*
     mstau12*mu2)/power2(mstau22 - mu2) + (2*power2(mu2))/(mA2*msntau2) + (2*
     power2(mu2))/(mH2*msntau2) + (2*power2(mu2))/(mA2*mstau12) + (2*power2(
     mu2))/(mH2*mstau12) + (2*power2(mu2))/(mA2*mstau22) + (4*power2(mu2))/(
     mC2*mstau22) + (2*power2(mu2))/(mH2*mstau22) + (40*power2(mu2))/((-mstau12
     + mu2)*(-mstau22 + mu2)) + (8*power2(mu2))/(mA2*msntau2 - mA2*mu2) + (8*
     power2(mu2))/(mA2*mstau12 - mA2*mu2) + (8*power2(mu2))/(mA2*mstau22 - mA2*
     mu2) - (8*power2(mu2))/(mH2*msntau2 - mH2*mu2) - (8*power2(mu2))/(mH2*
     mstau12 - mH2*mu2) - (8*power2(mu2))/(mH2*mstau22 - mH2*mu2) + 6*DeltaInv(
     mA2,mstau22,mu2)*power2(mu2) - (2*mA2*DeltaInv(mA2,mstau22,mu2)*power2(
     mu2))/mstau22 + 6*DeltaInv(mH2,msntau2,mu2)*power2(mu2) - (2*mH2*DeltaInv(
     mH2,msntau2,mu2)*power2(mu2))/msntau2 + 6*DeltaInv(mH2,mstau12,mu2)*
     power2(mu2) - (2*mH2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/mstau12 + 12*
     DeltaInv(mstau22,mC2,mu2)*power2(mu2) - (4*mC2*DeltaInv(mstau22,mC2,mu2)*
     power2(mu2))/mstau22 + 6*DeltaInv(mstau22,mH2,mu2)*power2(mu2) - (2*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 + (26*power2(mu2))/power2(
     msntau2 - mu2) + (26*power2(mu2))/power2(mstau12 - mu2) + (46*power2(mu2))
     /power2(mstau22 - mu2) + (2*mstau22*power2(mu2))/((-msntau2 + mu2)*power2(
     mstau22 - mu2)) + (2*mstau22*power2(mu2))/((-mstau12 + mu2)*power2(mstau22
     - mu2)) - (4*mstau24*power2(mu2))/(power2(msntau2 - mu2)*power2(mstau22 -
     mu2)) - (4*mstau24*power2(mu2))/(power2(mstau12 - mu2)*power2(mstau22 -
     mu2)) - (2*DeltaInv(mA2,msntau2,mu2)*power2(mu2)*(-3*mA2*msntau2 - 2*mA2*
     mu2 - msntau2*mu2 + power2(mA2) + power2(mu2)))/(mA2*msntau2) - (2*
     DeltaInv(mA2,mstau12,mu2)*power2(mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 -
     mstau12*mu2 + power2(mA2) + power2(mu2)))/(mA2*mstau12) + (2*power3(
     mstau22))/((msntau2 - mu2)*power2(mstau22 - mu2)) + (2*power3(mstau22))/((
     mstau12 - mu2)*power2(mstau22 - mu2)) + (2*mu2*power3(mstau22))/(power2(
     msntau2 - mu2)*power2(mstau22 - mu2)) + (2*mu2*power3(mstau22))/(power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (2*DeltaInv(mA2,mstau22,mu2)*
     power3(mu2))/mA2 + (4*DeltaInv(mA2,mstau22,mu2)*power3(mu2))/mstau22 + (2*
     DeltaInv(mH2,msntau2,mu2)*power3(mu2))/mH2 + (4*DeltaInv(mH2,msntau2,mu2)*
     power3(mu2))/msntau2 + (2*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/mH2 + (4*
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/mstau12 + (4*DeltaInv(mstau22,mC2,
     mu2)*power3(mu2))/mC2 + (8*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/mstau22
     + (2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mH2 + (4*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/mstau22 + (4*power3(mu2))/(mstau22*power2(msntau2 - mu2)
     ) + (12*mw2*DeltaInv(mstau22,mw2,mu2)*power3(mu2))/power2(msntau2 - mu2) +
     (4*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (6*mh2*DeltaInv(mstau22,
     mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (6*mz2*DeltaInv(mstau22,mz2,
     mu2)*power3(mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mstau22,mh2,mu2)*
     power2(mh2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (4*power3(mu2))
     /(msntau2*power2(mstau22 - mu2)) + (4*power3(mu2))/(mstau12*power2(mstau22
     - mu2)) + (12*mw2*DeltaInv(msntau2,mw2,mu2)*power3(mu2))/power2(mstau22 -
     mu2) + (6*mh2*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/power2(mstau22 - mu2)
     + (6*mz2*DeltaInv(mstau12,mz2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (
     2*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (2*mstau22*power3(mu2))/(power2(msntau2 - mu2)*power2(
     mstau22 - mu2)) + (2*mstau22*power3(mu2))/(power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) - (4*DeltaInv(mstau22,mw2,mu2)*power2(mw2)*power3(mu2))/(
     mstau22*power2(msntau2 - mu2)) - (4*DeltaInv(msntau2,mw2,mu2)*power2(mw2)*
     power3(mu2))/(msntau2*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mz2,
     mu2)*power2(mz2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (2*
     DeltaInv(mstau12,mz2,mu2)*power2(mz2)*power3(mu2))/(mstau12*power2(mstau22
     - mu2)) + (8*msntau2*power2(mu2))/power3(-msntau2 + mu2) + (4*mstau22*
     power2(mu2))/power3(-msntau2 + mu2) + (12*power3(mu2))/power3(-msntau2 +
     mu2) + (8*mstau12*power2(mu2))/power3(-mstau12 + mu2) + (4*mstau22*power2(
     mu2))/power3(-mstau12 + mu2) + (12*power3(mu2))/power3(-mstau12 + mu2) + (
     4*mstau12*power2(mu2))/power3(-mstau22 + mu2) + (16*mstau22*power2(mu2))/
     power3(-mstau22 + mu2) + (28*power3(mu2))/power3(-mstau22 + mu2) + (4*
     mstau24*power3(mu2))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (4*
     mstau24*power3(mu2))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (4*mu2*
     power4(mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) - (4*mu2*power4(
     mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (2*DeltaInv(mA2,
     mstau22,mu2)*power4(mu2))/(mA2*mstau22) - (2*DeltaInv(mH2,msntau2,mu2)*
     power4(mu2))/(mH2*msntau2) - (2*DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(
     mH2*mstau12) - (4*DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(mC2*mstau22) - (
     2*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mH2*mstau22) + (4*DeltaInv(
     mstau22,mw2,mu2)*power4(mu2))/power2(msntau2 - mu2) + (8*mw2*DeltaInv(
     mstau22,mw2,mu2)*power4(mu2))/(mstau22*power2(msntau2 - mu2)) + (2*
     DeltaInv(mstau22,mh2,mu2)*power4(mu2))/power2(mstau12 - mu2) + (4*mh2*
     DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     2*DeltaInv(mstau22,mz2,mu2)*power4(mu2))/power2(mstau12 - mu2) + (4*mz2*
     DeltaInv(mstau22,mz2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     6*power4(mu2))/(power2(msntau2 - mu2)*power2(mstau12 - mu2)) + (4*
     DeltaInv(msntau2,mw2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (8*mw2*
     DeltaInv(msntau2,mw2,mu2)*power4(mu2))/(msntau2*power2(mstau22 - mu2)) + (
     2*DeltaInv(mstau12,mh2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (4*mh2*
     DeltaInv(mstau12,mh2,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     2*DeltaInv(mstau12,mz2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (4*mz2*
     DeltaInv(mstau12,mz2,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     2*power4(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) - (2*power4(
     mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (3*power4(mu2))/
     power4(msntau2 - mu2) - (3*power4(mu2))/power4(mstau12 - mu2) - (3*power4(
     mu2))/power4(mstau22 - mu2) - (2*mstau22*power4(mu2))/((-msntau2 + mu2)*
     power4(mstau22 - mu2)) - (2*mstau22*power4(mu2))/((-mstau12 + mu2)*power4(
     mstau22 - mu2)) + (2*power5(mstau22))/((-msntau2 + mu2)*power4(mstau22 -
     mu2)) + (2*power5(mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (4*
     DeltaInv(mstau22,mw2,mu2)*power5(mu2))/(mstau22*power2(msntau2 - mu2)) - (
     2*DeltaInv(mstau22,mh2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)) -
     (2*DeltaInv(mstau22,mz2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2))
     - (4*DeltaInv(msntau2,mw2,mu2)*power5(mu2))/(msntau2*power2(mstau22 - mu2)
     ) - (2*DeltaInv(mstau12,mh2,mu2)*power5(mu2))/(mstau12*power2(mstau22 -
     mu2)) - (2*DeltaInv(mstau12,mz2,mu2)*power5(mu2))/(mstau12*power2(mstau22
     - mu2))))/32. + (power2(log(mstau12/Q2))*(3 - (5*mstau12)/mA2 + (3*
     mstau12)/mH2 + (2*mstau22)/(mstau12 - mstau22) - (10*mu2)/mA2 + (6*mu2)/
     mH2 - (8*mu2)/mstau12 - (4*mu2)/(mstau12 - mstau22) + (4*mu2)/(mstau22 -
     mu2) + (8*mstau22*mu2)/((mstau12 - mstau22)*(mstau22 - mu2)) + (2*mstau22)
     /(-mstau12 + mu2) + (2*mu2)/(-mstau12 + mu2) + (2*mstau12)/(-mstau22 +
     mu2) + (2*mstau24)/((mstau12 - mstau22)*(-mstau22 + mu2)) + (6*mh2*mu2)/(
     mstau12*mstau22 - mstau12*mu2) + (6*mu2*mz2)/(mstau12*mstau22 - mstau12*
     mu2) + 4*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) - 2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2) + (4*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 - (2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (4*mh2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (6*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (12*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) - (6*mh2*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) - 2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mz2) - (2*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mz2))/mstau12 + (6*mstau24*mu2*DeltaInv(mstau22,mstau12,
     mz2))/(mstau12 - mu2) + 4*mu2*mz2*DeltaInv(mstau22,mstau12,mz2) + (4*
     mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/mstau12 + (4*mstau22*mu2*
     mz2*DeltaInv(mstau22,mstau12,mz2))/(mstau12 - mu2) + (12*mstau22*mu2*mz2*
     DeltaInv(mstau22,mstau12,mz2))/(mstau22 - mu2) - (6*mstau24*mu2*mz2*
     DeltaInv(mstau22,mstau12,mz2))/(mstau12*mstau22 - mstau12*mu2) - (2*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (6*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12 - mu2) + (10*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (12*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (2*mstau24)/power2(mstau12 - mstau22) - (4*mstau22*mu2)/power2(mstau12 -
     mstau22) + (8*mstau24*mu2)/((mstau22 - mu2)*power2(mstau12 - mstau22)) + (
     2*mh2*mu2)/power2(mstau12 - mu2) - (2*mstau22*mu2)/power2(mstau12 - mu2) +
     (2*mu2*mz2)/power2(mstau12 - mu2) + (2*mh2*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau24*mu2*mz2*DeltaInv(mstau22,
     mstau12,mz2))/power2(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (4*mstau12*mu2)/power2(
     mstau22 - mu2) - (2*mstau24*mu2)/((mstau12 - mstau22)*power2(mstau22 -
     mu2)) - (2*power2(mu2))/(mA2*mstau12) - (2*power2(mu2))/(mH2*mstau12) + (
     6*power2(mu2))/((mstau12 - mstau22)*(-mstau22 + mu2)) - (8*power2(mu2))/(
     mA2*mstau12 - mA2*mu2) + (8*power2(mu2))/(mH2*mstau12 - mH2*mu2) + (4*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau12 + mu2) + (4*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau12 + mu2) + (4*mstau22*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/(-mstau12 + mu2) + (4*mz2*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/(-mstau12 + mu2) + (6*mstau22*
     power2(mu2))/((-mstau22 + mu2)*power2(mstau12 - mstau22)) - (4*power2(mu2)
     )/power2(mstau12 - mu2) + (12*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau12 - mu2) + (2*mstau24*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau12 - mu2) + (2*mstau24*DeltaInv(mstau22,
     mstau12,mz2)*power2(mu2))/power2(mstau12 - mu2) + (12*mstau22*mz2*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(mstau12 - mu2) + (2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/power2(mstau12 -
     mu2) - (8*power2(mu2))/power2(mstau22 - mu2) - (12*mh2*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (4*mstau22*power2(mu2))/((mstau12 -
     mstau22)*power2(mstau22 - mu2)) - (12*mz2*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (12*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mu2))/power2(mstau22 - mu2) + (6*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (12*mstau22*mz2*DeltaInv(
     mstau22,mstau12,mz2)*power2(mu2))/power2(mstau22 - mu2) + (6*mstau24*mz2*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (10*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power2(mu2))/power2(mstau22 -
     mu2) - (10*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/power2(
     mstau22 - mu2) - (12*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (4*mstau24*power2(mu2))/(
     power2(mstau12 - mstau22)*power2(mstau22 - mu2)) - (2*mu2*DeltaInv(
     mstau22,mstau12,mz2)*power2(mz2))/mstau12 + (6*mu2*DeltaInv(mstau22,
     mstau12,mz2)*power2(mz2))/(mstau12 - mu2) + (10*mu2*DeltaInv(mstau22,
     mstau12,mz2)*power2(mz2))/(mstau22 - mu2) + (12*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mz2)*power2(mz2))/(mstau12*mstau22 - mstau12*mu2) + (2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mz2)*power2(mz2))/power2(mstau12 -
     mu2) + (2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*power2(mz2))/power2(
     mstau12 - mu2) - (10*DeltaInv(mstau12,mz2,mu2)*power2(mu2)*power2(mz2))/
     power2(mstau22 - mu2) - (10*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*
     power2(mz2))/power2(mstau22 - mu2) - (12*mstau22*DeltaInv(mstau22,mstau12,
     mz2)*power2(mu2)*power2(mz2))/(mstau12*power2(mstau22 - mu2)) + (2*mu2*
     DeltaInv(mA2,mstau12,mu2)*(mA2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)
     *power2(mA2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mA2)))/(mA2*
     mstau12) - (6*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*
     mstau22 - mstau12*mu2) - (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))
     /power2(mstau12 - mu2) + (6*DeltaInv(mstau12,mh2,mu2)*power2(mu2)*power3(
     mh2))/(mstau12*power2(mstau22 - mu2)) + (6*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (2*mu2*
     DeltaInv(mH2,mstau12,mu2)*(mH2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)
     *power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mH2)))/(mH2*
     mstau12) + (2*power3(mstau22))/((mstau12 - mu2)*power2(mstau12 - mstau22))
     + (2*power3(mstau22))/((-mstau22 + mu2)*power2(mstau12 - mstau22)) - (2*
     power3(mstau22))/((mstau12 - mstau22)*power2(mstau12 - mu2)) - (2*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mstau22))/power2(mstau12 - mu2) - (2*
     mu2*DeltaInv(mstau22,mstau12,mz2)*power3(mstau22))/power2(mstau12 - mu2) +
     (2*mu2*power3(mstau22))/(power2(mstau12 - mstau22)*power2(mstau12 - mu2))
     - (2*mu2*power3(mstau22))/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)
     ) + (2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau22 - mu2) + (2*
     DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/(mstau22 - mu2) - (8*power3(
     mu2))/(mstau12*power2(mstau22 - mu2)) - (2*power3(mu2))/((mstau12 -
     mstau22)*power2(mstau22 - mu2)) - (20*mh2*DeltaInv(mstau12,mh2,mu2)*
     power3(mu2))/power2(mstau22 - mu2) - (20*mz2*DeltaInv(mstau12,mz2,mu2)*
     power3(mu2))/power2(mstau22 - mu2) - (8*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power3(mu2))/power2(mstau22 - mu2) - (4*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power3(mu2))/power2(mstau22 - mu2) - (4*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (2*mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2))
     - (4*mstau22*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/power2(mstau22 -
     mu2) + (2*mstau24*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (8*mz2*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))
     /power2(mstau22 - mu2) - (4*mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*
     power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (10*DeltaInv(mstau12,mh2,
     mu2)*power2(mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (2*mstau22*power3(mu2))/(power2(mstau12 - mstau22)*
     power2(mstau22 - mu2)) - (10*DeltaInv(mstau12,mz2,mu2)*power2(mz2)*power3(
     mu2))/(mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mstau12,mz2)*
     power2(mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (8*mstau12*
     power2(mu2))/power3(-mstau12 + mu2) + (4*mstau22*power2(mu2))/power3(-
     mstau12 + mu2) + (4*power3(mu2))/power3(-mstau12 + mu2) - (6*mu2*DeltaInv(
     mstau22,mstau12,mz2)*power3(mz2))/(mstau12*mstau22 - mstau12*mu2) - (2*
     mu2*DeltaInv(mstau22,mstau12,mz2)*power3(mz2))/power2(mstau12 - mu2) + (6*
     DeltaInv(mstau12,mz2,mu2)*power2(mu2)*power3(mz2))/(mstau12*power2(mstau22
     - mu2)) + (6*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*power3(mz2))/(
     mstau12*power2(mstau22 - mu2)) - (2*power4(mstau22))/(power2(mstau12 -
     mstau22)*power2(mstau12 - mu2)) - (2*DeltaInv(mstau12,mh2,mu2)*power4(mu2)
     )/power2(mstau22 - mu2) + (2*mh2*DeltaInv(mstau12,mh2,mu2)*power4(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau12,mz2,mu2)*power4(mu2))
     /power2(mstau22 - mu2) + (2*mz2*DeltaInv(mstau12,mz2,mu2)*power4(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mstau12,mh2)*power4(
     mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mstau12,mz2)*power4(mu2)
     )/power2(mstau22 - mu2) + (3*power4(mu2))/power4(mstau12 - mu2) + (2*
     DeltaInv(mstau12,mh2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     2*DeltaInv(mstau12,mz2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2))))
     /32. - (log(mstau12/Q2)*(3 + (2*mA2)/mstau12 + (2*mH2)/mstau12 - (16*
     mstau12)/mA2 + (4*mstau22)/mstau12 + (3*mstau22)/(mstau12 - mstau22) - (
     30*mu2)/mA2 + (18*mu2)/mH2 - (28*mu2)/mstau12 + (11*mu2)/(mstau22 - mu2) +
     (6*mstau22*mu2)/((mstau12 - mstau22)*(mstau22 - mu2)) + (3*mu2)/(-msntau2
     + mu2) + (2*mA2)/(-mstau12 + mu2) + (2*mH2)/(-mstau12 + mu2) + (8*mstau22)
     /(-mstau12 + mu2) + (7*mu2)/(-mstau12 + mu2) + (12*mstau12)/(-mstau22 +
     mu2) + (3*mstau24)/((mstau12 - mstau22)*(-mstau22 + mu2)) + (18*mh2*mu2)/(
     mstau12*mstau22 - mstau12*mu2) + (18*mu2*mz2)/(mstau12*mstau22 - mstau12*
     mu2) + 12*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) - 6*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2) + (12*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 - (6*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (12*mh2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (18*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (36*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) - (18*mh2*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) - 6*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mz2) - (6*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mz2))/mstau12 + (18*mstau24*mu2*DeltaInv(mstau22,mstau12,
     mz2))/(mstau12 - mu2) + 12*mu2*mz2*DeltaInv(mstau22,mstau12,mz2) + (12*
     mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/mstau12 + (12*mstau22*mu2*
     mz2*DeltaInv(mstau22,mstau12,mz2))/(mstau12 - mu2) + (36*mstau22*mu2*mz2*
     DeltaInv(mstau22,mstau12,mz2))/(mstau22 - mu2) - (18*mstau24*mu2*mz2*
     DeltaInv(mstau22,mstau12,mz2))/(mstau12*mstau22 - mstau12*mu2) - (6*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (18*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12 - mu2) + (30*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (36*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (3*mstau24)/power2(mstau12 - mstau22) + (6*mstau24*mu2)/((mstau22 - mu2)
     *power2(mstau12 - mstau22)) + (2*mA2*mu2)/power2(mstau12 - mu2) + (6*mh2*
     mu2)/power2(mstau12 - mu2) + (2*mH2*mu2)/power2(mstau12 - mu2) - (4*
     mstau12*mu2)/power2(mstau12 - mu2) + (4*mstau22*mu2)/power2(mstau12 - mu2)
     + (6*mu2*mz2)/power2(mstau12 - mu2) + (6*mh2*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (6*mstau24*mu2*mz2*DeltaInv(mstau22,
     mstau12,mz2))/power2(mstau12 - mu2) + (6*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (8*mstau12*mu2)/power2(
     mstau22 - mu2) - (3*mstau24*mu2)/((mstau12 - mstau22)*power2(mstau22 -
     mu2)) - (6*power2(mu2))/(mA2*mstau12) - (6*power2(mu2))/(mH2*mstau12) - (
     24*power2(mu2))/(mA2*mstau12 - mA2*mu2) + (24*power2(mu2))/(mH2*mstau12 -
     mH2*mu2) + (12*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau12 +
     mu2) + (12*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau12 +
     mu2) + (12*mstau22*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/(-mstau12 +
     mu2) + (12*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/(-mstau12 + mu2)
     + (13*power2(mu2))/power2(mstau12 - mu2) - (2*mA2*power2(mu2))/(mstau12*
     power2(mstau12 - mu2)) - (2*mH2*power2(mu2))/(mstau12*power2(mstau12 -
     mu2)) - (4*mstau22*power2(mu2))/(mstau12*power2(mstau12 - mu2)) + (36*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) +
     (6*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 -
     mu2) + (6*mstau24*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(
     mstau12 - mu2) + (36*mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)
     )/power2(mstau12 - mu2) + (6*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/power2(mstau12 - mu2) - (24*power2(mu2))/power2(mstau22 -
     mu2) - (36*mh2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*mstau22*
     power2(mu2))/((mstau12 - mstau22)*power2(mstau22 - mu2)) - (36*mz2*power2(
     mu2))/(mstau12*power2(mstau22 - mu2)) - (36*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (18*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (36*mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(
     mstau22 - mu2) + (18*mstau24*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)
     )/(mstau12*power2(mstau22 - mu2)) - (30*DeltaInv(mstau12,mh2,mu2)*power2(
     mh2)*power2(mu2))/power2(mstau22 - mu2) - (30*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2)*power2(mu2))/power2(mstau22 - mu2) - (36*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (3*mstau24*power2(mu2))/(power2(mstau12 - mstau22)*
     power2(mstau22 - mu2)) - (6*mu2*DeltaInv(mstau22,mstau12,mz2)*power2(mz2))
     /mstau12 + (18*mu2*DeltaInv(mstau22,mstau12,mz2)*power2(mz2))/(mstau12 -
     mu2) + (30*mu2*DeltaInv(mstau22,mstau12,mz2)*power2(mz2))/(mstau22 - mu2)
     + (36*mstau22*mu2*DeltaInv(mstau22,mstau12,mz2)*power2(mz2))/(mstau12*
     mstau22 - mstau12*mu2) + (6*mstau22*mu2*DeltaInv(mstau22,mstau12,mz2)*
     power2(mz2))/power2(mstau12 - mu2) + (6*DeltaInv(mstau22,mstau12,mz2)*
     power2(mu2)*power2(mz2))/power2(mstau12 - mu2) - (30*DeltaInv(mstau12,mz2,
     mu2)*power2(mu2)*power2(mz2))/power2(mstau22 - mu2) - (30*DeltaInv(
     mstau22,mstau12,mz2)*power2(mu2)*power2(mz2))/power2(mstau22 - mu2) - (36*
     mstau22*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*power2(mz2))/(mstau12*
     power2(mstau22 - mu2)) + (6*mu2*DeltaInv(mA2,mstau12,mu2)*(mA2*mu2*(-10*
     mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mA2) + (-mstau12 + mu2)*power2(
     mu2) + 3*power3(mA2)))/(mA2*mstau12) - (18*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (6*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (18*DeltaInv(
     mstau12,mh2,mu2)*power2(mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))
     + (18*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)*power3(mh2))/(mstau12*
     power2(mstau22 - mu2)) + (6*mu2*DeltaInv(mH2,mstau12,mu2)*(mH2*mu2*(-10*
     mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mH2) + (-mstau12 + mu2)*power2(
     mu2) + 3*power3(mH2)))/(mH2*mstau12) + (6*power3(mstau22))/((mstau12 -
     mu2)*power2(mstau12 - mstau22)) + (3*power3(mstau22))/((-mstau22 + mu2)*
     power2(mstau12 - mstau22)) - (6*power3(mstau22))/((mstau12 - mstau22)*
     power2(mstau12 - mu2)) - (6*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(
     mstau22))/power2(mstau12 - mu2) - (6*mu2*DeltaInv(mstau22,mstau12,mz2)*
     power3(mstau22))/power2(mstau12 - mu2) + (6*mu2*power3(mstau22))/(power2(
     mstau12 - mstau22)*power2(mstau12 - mu2)) - (3*mu2*power3(mstau22))/(
     power2(mstau12 - mstau22)*power2(mstau22 - mu2)) + (4*mA2*power2(mu2))/
     power3(mstau12 - mu2) + (4*mH2*power2(mu2))/power3(mstau12 - mu2) + (6*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau22 - mu2) + (6*DeltaInv(
     mstau22,mstau12,mz2)*power3(mu2))/(mstau22 - mu2) + (4*power3(mu2))/(
     mstau12*power2(mstau12 - mu2)) + (3*power3(mu2))/((msntau2 - mu2)*power2(
     mstau12 - mu2)) - power3(mu2)/((-mstau22 + mu2)*power2(mstau12 - mu2)) - (
     24*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (60*mh2*DeltaInv(
     mstau12,mh2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (60*mz2*DeltaInv(
     mstau12,mz2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (24*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) - (12*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) - (12*
     mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (6*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (12*mstau22*DeltaInv(mstau22,mstau12,mz2)
     *power3(mu2))/power2(mstau22 - mu2) + (6*mstau24*DeltaInv(mstau22,mstau12,
     mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (24*mz2*DeltaInv(
     mstau22,mstau12,mz2)*power3(mu2))/power2(mstau22 - mu2) - (12*mstau22*mz2*
     DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2))
     - (30*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (6*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power3(mu2)
     )/(mstau12*power2(mstau22 - mu2)) - (30*DeltaInv(mstau12,mz2,mu2)*power2(
     mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (6*DeltaInv(mstau22,
     mstau12,mz2)*power2(mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     12*mstau12*power2(mu2))/power3(-mstau12 + mu2) + power3(mu2)/power3(-
     mstau12 + mu2) - (18*mu2*DeltaInv(mstau22,mstau12,mz2)*power3(mz2))/(
     mstau12*mstau22 - mstau12*mu2) - (6*mu2*DeltaInv(mstau22,mstau12,mz2)*
     power3(mz2))/power2(mstau12 - mu2) + (18*DeltaInv(mstau12,mz2,mu2)*power2(
     mu2)*power3(mz2))/(mstau12*power2(mstau22 - mu2)) + (18*DeltaInv(mstau22,
     mstau12,mz2)*power2(mu2)*power3(mz2))/(mstau12*power2(mstau22 - mu2)) - (
     6*power4(mstau22))/(power2(mstau12 - mstau22)*power2(mstau12 - mu2)) - (6*
     DeltaInv(mstau12,mh2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (6*mh2*
     DeltaInv(mstau12,mh2,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     6*DeltaInv(mstau12,mz2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (6*mz2*
     DeltaInv(mstau12,mz2,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     6*DeltaInv(mstau22,mstau12,mh2)*power4(mu2))/power2(mstau22 - mu2) + (6*
     DeltaInv(mstau22,mstau12,mz2)*power4(mu2))/power2(mstau22 - mu2) + (6*
     DeltaInv(mstau12,mh2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     6*DeltaInv(mstau12,mz2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2))))
     /16. + (log(mu2/Q2)*((30*mu2)/mA2 + (4*mu2)/mC2 - (18*mu2)/mH2 + mu2/(
     msntau2 - mu2) + (12*mu2)/(mstau22 - mu2) + (15*mu2)/(-mstau12 + mu2) + (
     16*mstau22*mu2)/((-mstau12 + mu2)*(-mstau22 + mu2)) - (4*mstau22*mu2)/
     power2(msntau2 - mu2) - (4*mstau22*mu2)/power2(mstau12 - mu2) - (4*
     mstau12*mu2)/power2(mstau22 - mu2) + (9*mstau24*mu2)/((-msntau2 + mu2)*
     power2(mstau22 - mu2)) + (9*mstau24*mu2)/((-mstau12 + mu2)*power2(mstau22
     - mu2)) + (6*power2(mu2))/(mA2*msntau2) + (6*power2(mu2))/(mH2*msntau2) +
     (6*power2(mu2))/(mA2*mstau12) + (6*power2(mu2))/(mH2*mstau12) + (6*power2(
     mu2))/(mA2*mstau22) + (12*power2(mu2))/(mC2*mstau22) + (6*power2(mu2))/(
     mH2*mstau22) + (80*power2(mu2))/((-mstau12 + mu2)*(-mstau22 + mu2)) + (24*
     power2(mu2))/(mA2*msntau2 - mA2*mu2) + (24*power2(mu2))/(mA2*mstau12 -
     mA2*mu2) + (24*power2(mu2))/(mA2*mstau22 - mA2*mu2) - (24*power2(mu2))/(
     mH2*msntau2 - mH2*mu2) - (24*power2(mu2))/(mH2*mstau12 - mH2*mu2) - (24*
     power2(mu2))/(mH2*mstau22 - mH2*mu2) + 18*DeltaInv(mA2,mstau22,mu2)*
     power2(mu2) - (6*mA2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/mstau22 + 18*
     DeltaInv(mH2,msntau2,mu2)*power2(mu2) - (6*mH2*DeltaInv(mH2,msntau2,mu2)*
     power2(mu2))/msntau2 + 18*DeltaInv(mH2,mstau12,mu2)*power2(mu2) - (6*mH2*
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/mstau12 + 36*DeltaInv(mstau22,mC2,
     mu2)*power2(mu2) - (12*mC2*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/mstau22
     + 18*DeltaInv(mstau22,mH2,mu2)*power2(mu2) - (6*mH2*DeltaInv(mstau22,mH2,
     mu2)*power2(mu2))/mstau22 + (27*power2(mu2))/power2(msntau2 - mu2) + (27*
     power2(mu2))/power2(mstau12 - mu2) + (71*power2(mu2))/power2(mstau22 -
     mu2) - (3*mstau22*power2(mu2))/((-msntau2 + mu2)*power2(mstau22 - mu2)) -
     (3*mstau22*power2(mu2))/((-mstau12 + mu2)*power2(mstau22 - mu2)) + (6*
     mstau24*power2(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (6*
     mstau24*power2(mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (6*
     DeltaInv(mA2,msntau2,mu2)*power2(mu2)*(-3*mA2*msntau2 - 2*mA2*mu2 -
     msntau2*mu2 + power2(mA2) + power2(mu2)))/(mA2*msntau2) - (6*DeltaInv(mA2,
     mstau12,mu2)*power2(mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2 +
     power2(mA2) + power2(mu2)))/(mA2*mstau12) + (7*power3(mstau22))/((msntau2
     - mu2)*power2(mstau22 - mu2)) + (7*power3(mstau22))/((mstau12 - mu2)*
     power2(mstau22 - mu2)) + (8*mC2*power2(mu2))/power3(msntau2 - mu2) + (4*
     mA2*power2(mu2))/power3(mstau12 - mu2) + (4*mH2*power2(mu2))/power3(
     mstau12 - mu2) + (4*mA2*power2(mu2))/power3(mstau22 - mu2) + (8*mC2*
     power2(mu2))/power3(mstau22 - mu2) + (4*mH2*power2(mu2))/power3(mstau22 -
     mu2) + (8*msntau2*power2(mu2))/power3(mstau22 - mu2) + (6*DeltaInv(mA2,
     mstau22,mu2)*power3(mu2))/mA2 + (12*DeltaInv(mA2,mstau22,mu2)*power3(mu2))
     /mstau22 + (6*DeltaInv(mH2,msntau2,mu2)*power3(mu2))/mH2 + (12*DeltaInv(
     mH2,msntau2,mu2)*power3(mu2))/msntau2 + (6*DeltaInv(mH2,mstau12,mu2)*
     power3(mu2))/mH2 + (12*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/mstau12 + (
     12*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/mC2 + (24*DeltaInv(mstau22,mC2,
     mu2)*power3(mu2))/mstau22 + (6*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mH2
     + (12*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mstau22 + (12*power3(mu2))/(
     mstau22*power2(msntau2 - mu2)) - (3*power3(mu2))/((-mstau12 + mu2)*power2(
     msntau2 - mu2)) - power3(mu2)/((-mstau22 + mu2)*power2(msntau2 - mu2)) + (
     36*mw2*DeltaInv(mstau22,mw2,mu2)*power3(mu2))/power2(msntau2 - mu2) + (12*
     power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*power3(mu2))/((msntau2 -
     mu2)*power2(mstau12 - mu2)) - power3(mu2)/((-mstau22 + mu2)*power2(mstau12
     - mu2)) + (18*mh2*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 -
     mu2) + (18*mz2*DeltaInv(mstau22,mz2,mu2)*power3(mu2))/power2(mstau12 -
     mu2) - (6*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2)) + (12*power3(mu2))/(msntau2*power2(mstau22 - mu2))
     + (12*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (36*mw2*DeltaInv(
     msntau2,mw2,mu2)*power3(mu2))/power2(mstau22 - mu2) + (18*mh2*DeltaInv(
     mstau12,mh2,mu2)*power3(mu2))/power2(mstau22 - mu2) + (18*mz2*DeltaInv(
     mstau12,mz2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (6*DeltaInv(mstau12,
     mh2,mu2)*power2(mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (12*
     mstau22*power3(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) - (12*
     mstau22*power3(mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (12*
     DeltaInv(mstau22,mw2,mu2)*power2(mw2)*power3(mu2))/(mstau22*power2(msntau2
     - mu2)) - (12*DeltaInv(msntau2,mw2,mu2)*power2(mw2)*power3(mu2))/(msntau2*
     power2(mstau22 - mu2)) - (6*DeltaInv(mstau22,mz2,mu2)*power2(mz2)*power3(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(mstau12,mz2,mu2)*
     power2(mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (12*msntau2*
     power2(mu2))/power3(-msntau2 + mu2) + power3(mu2)/power3(-msntau2 + mu2) +
     (12*mstau12*power2(mu2))/power3(-mstau12 + mu2) + power3(mu2)/power3(-
     mstau12 + mu2) + (24*mstau22*power2(mu2))/power3(-mstau22 + mu2) + (5*
     power3(mu2))/power3(-mstau22 + mu2) + (40*power2(mu2)*power3(mstau22))/((-
     msntau2 + mu2)*power4(mstau22 - mu2)) + (40*power2(mu2)*power3(mstau22))/(
     (-mstau12 + mu2)*power4(mstau22 - mu2)) + (34*mstau24*power3(mu2))/((
     msntau2 - mu2)*power4(mstau22 - mu2)) + (34*mstau24*power3(mu2))/((mstau12
     - mu2)*power4(mstau22 - mu2)) - (26*mu2*power4(mstau22))/((-msntau2 + mu2)
     *power4(mstau22 - mu2)) - (26*mu2*power4(mstau22))/((-mstau12 + mu2)*
     power4(mstau22 - mu2)) - (6*DeltaInv(mA2,mstau22,mu2)*power4(mu2))/(mA2*
     mstau22) - (6*DeltaInv(mH2,msntau2,mu2)*power4(mu2))/(mH2*msntau2) - (6*
     DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(mH2*mstau12) - (12*DeltaInv(
     mstau22,mC2,mu2)*power4(mu2))/(mC2*mstau22) - (6*DeltaInv(mstau22,mH2,mu2)
     *power4(mu2))/(mH2*mstau22) + (12*DeltaInv(mstau22,mw2,mu2)*power4(mu2))/
     power2(msntau2 - mu2) + (24*mw2*DeltaInv(mstau22,mw2,mu2)*power4(mu2))/(
     mstau22*power2(msntau2 - mu2)) + (6*DeltaInv(mstau22,mh2,mu2)*power4(mu2))
     /power2(mstau12 - mu2) + (12*mh2*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (6*DeltaInv(mstau22,mz2,mu2)*power4(mu2))
     /power2(mstau12 - mu2) + (12*mz2*DeltaInv(mstau22,mz2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (12*DeltaInv(msntau2,mw2,mu2)*power4(mu2)
     )/power2(mstau22 - mu2) + (24*mw2*DeltaInv(msntau2,mw2,mu2)*power4(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (6*DeltaInv(mstau12,mh2,mu2)*power4(mu2))
     /power2(mstau22 - mu2) + (12*mh2*DeltaInv(mstau12,mh2,mu2)*power4(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (6*DeltaInv(mstau12,mz2,mu2)*power4(mu2))
     /power2(mstau22 - mu2) + (12*mz2*DeltaInv(mstau12,mz2,mu2)*power4(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (6*power4(mu2))/(power2(msntau2 - mu2)*
     power2(mstau22 - mu2)) + (6*power4(mu2))/(power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) + (17*mstau22*power4(mu2))/((-msntau2 + mu2)*power4(
     mstau22 - mu2)) + (17*mstau22*power4(mu2))/((-mstau12 + mu2)*power4(
     mstau22 - mu2)) + (7*power5(mstau22))/((-msntau2 + mu2)*power4(mstau22 -
     mu2)) + (7*power5(mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) - (
     12*DeltaInv(mstau22,mw2,mu2)*power5(mu2))/(mstau22*power2(msntau2 - mu2))
     - (6*DeltaInv(mstau22,mh2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)
     ) - (6*DeltaInv(mstau22,mz2,mu2)*power5(mu2))/(mstau22*power2(mstau12 -
     mu2)) - (12*DeltaInv(msntau2,mw2,mu2)*power5(mu2))/(msntau2*power2(mstau22
     - mu2)) - (6*DeltaInv(mstau12,mh2,mu2)*power5(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (6*DeltaInv(mstau12,mz2,mu2)*power5(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (4*power5(mu2))/((msntau2 - mu2)*power4(mstau22 -
     mu2)) + (4*power5(mu2))/((mstau12 - mu2)*power4(mstau22 - mu2))))/16. + (-
     516 + (19*mC2)/mA2 + (35*mC2)/(-mA2 + mC2) + (56*mA2)/(mA2 - mH2) + (35*
     mC2)/(mC2 - mH2) + (368*mA2)/mH2 - (45*mC2)/mH2 - (72*mH2)/mA2 + (16*mC2)/
     msntau2 - (240*msntau2)/mA2 - (32*msntau2)/mC2 + (144*msntau2)/mH2 + (8*
     mA2)/mstau12 + (8*mH2)/mstau12 - (208*mstau12)/mA2 + (48*mstau12)/mH2 + (
     8*mA2)/mstau22 + (16*mC2)/mstau22 + (8*mH2)/mstau22 + (16*msntau2)/mstau22
     + (16*mstau12)/mstau22 - (208*mstau22)/mA2 - (128*mstau22)/mC2 + (48*
     mstau22)/mH2 + (16*mstau22)/msntau2 + (16*mstau22)/mstau12 + (70*mstau22)/
     (mstau12 - mstau22) - (960*mu2)/mA2 - (128*mu2)/mC2 + (576*mu2)/mH2 - (
     240*mu2)/msntau2 - (240*mu2)/mstau12 - (56*mu2)/(mstau12 - mstau22) - (
     928*mu2)/mstau22 + (82*mu2)/(mstau12 - mu2) + (184*mu2)/(mstau22 - mu2) +
     (196*mstau22*mu2)/((mstau12 - mstau22)*(mstau22 - mu2)) + (48*mC2)/(-
     msntau2 + mu2) + (102*mstau22)/(-msntau2 + mu2) + (42*mstau24)/((msntau2 -
     mstau22)*(-msntau2 + mu2)) + (110*mu2)/(-msntau2 + mu2) + (24*mA2)/(-
     mstau12 + mu2) + (24*mH2)/(-mstau12 + mu2) + (102*mstau22)/(-mstau12 +
     mu2) + (42*mstau24)/((mstau12 - mstau22)*(-mstau12 + mu2)) + (192*mstau22*
     mu2)/((mstau22 - mu2)*(-mstau12 + mu2)) + (24*mA2)/(-mstau22 + mu2) + (48*
     mC2)/(-mstau22 + mu2) + (24*mH2)/(-mstau22 + mu2) + (48*msntau2)/(-mstau22
     + mu2) + (144*mstau12)/(-mstau22 + mu2) + (70*mstau24)/((mstau12 -
     mstau22)*(-mstau22 + mu2)) + (224*mh2*mu2)/(mstau12*mstau22 - mstau12*mu2)
     + (448*mu2*mw2)/(msntau2*mstau22 - msntau2*mu2) + (224*mu2*mz2)/(mstau12*
     mstau22 - mstau12*mu2) - 448*mA2*mu2*DeltaInv(mA2,mstau22,mu2) - 448*mH2*
     mu2*DeltaInv(mH2,msntau2,mu2) - 448*mH2*mu2*DeltaInv(mH2,mstau12,mu2) -
     896*mC2*mu2*DeltaInv(mstau22,mC2,mu2) - 448*mH2*mu2*DeltaInv(mstau22,mH2,
     mu2) - 224*mstau22*mu2*DeltaInv(mstau22,msntau2,mw2) - (224*mstau24*mu2*
     DeltaInv(mstau22,msntau2,mw2))/msntau2 + (672*mstau24*mu2*DeltaInv(
     mstau22,msntau2,mw2))/(msntau2 - mu2) + 224*mu2*mw2*DeltaInv(mstau22,
     msntau2,mw2) + (448*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/msntau2
     + (896*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/(msntau2 - mu2) + (
     896*mstau22*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/(mstau22 - mu2) - (448*
     mstau24*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/(msntau2*mstau22 - msntau2*
     mu2) + 112*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) - 112*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2) + (224*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 - (112*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (448*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 -
     mu2) + (336*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (
     448*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) - (224*
     mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*
     mu2) - 112*mstau22*mu2*DeltaInv(mstau22,mstau12,mz2) - (112*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mz2))/mstau12 + (336*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mz2))/(mstau12 - mu2) + 112*mu2*mz2*DeltaInv(mstau22,
     mstau12,mz2) + (224*mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/mstau12
     + (448*mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/(mstau12 - mu2) + (
     448*mstau22*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/(mstau22 - mu2) - (224*
     mstau24*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/(mstau12*mstau22 - mstau12*
     mu2) + (224*mu2*DeltaInv(mA2,mstau22,mu2)*power2(mA2))/mstau22 + (448*mu2*
     DeltaInv(mstau22,mC2,mu2)*power2(mC2))/mstau22 - (119*power2(mC2))/power2(
     mA2) + (14*power2(mC2))/power2(mA2 - mC2) - (112*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/mstau12 + (336*mu2*DeltaInv(mstau22,mstau12,mh2)
     *power2(mh2))/(mstau12 - mu2) + (448*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau22 - mu2) + (448*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + (35*power2(mA2))/
     power2(mA2 - mH2) + (14*power2(mC2))/power2(mC2 - mH2) + (105*power2(mA2))
     /power2(mH2) + (105*power2(mC2))/power2(mH2) + (224*mu2*DeltaInv(mH2,
     msntau2,mu2)*power2(mH2))/msntau2 + (224*mu2*DeltaInv(mH2,mstau12,mu2)*
     power2(mH2))/mstau12 + (224*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/
     mstau22 - (140*power2(mH2))/power2(mA2) - (32*mC2*mu2)/(-(msntau2*mu2) +
     power2(msntau2)) - (32*mstau22*mu2)/(-(msntau2*mu2) + power2(msntau2)) - (
     16*mA2*mu2)/(-(mstau12*mu2) + power2(mstau12)) - (16*mH2*mu2)/(-(mstau12*
     mu2) + power2(mstau12)) - (32*mstau22*mu2)/(-(mstau12*mu2) + power2(
     mstau12)) + (70*mstau24)/power2(mstau12 - mstau22) - (56*mstau22*mu2)/
     power2(mstau12 - mstau22) + (196*mstau24*mu2)/((mstau22 - mu2)*power2(
     mstau12 - mstau22)) - (16*mA2*mu2)/(-(mstau22*mu2) + power2(mstau22)) - (
     32*mC2*mu2)/(-(mstau22*mu2) + power2(mstau22)) - (16*mH2*mu2)/(-(mstau22*
     mu2) + power2(mstau22)) - (32*msntau2*mu2)/(-(mstau22*mu2) + power2(
     mstau22)) - (32*mstau12*mu2)/(-(mstau22*mu2) + power2(mstau22)) + (112*
     mC2*mu2)/power2(msntau2 - mu2) - (48*msntau2*mu2)/power2(msntau2 - mu2) +
     (214*mstau22*mu2)/power2(msntau2 - mu2) - (42*mstau24*mu2)/((msntau2 -
     mstau22)*power2(msntau2 - mu2)) + (224*mu2*mw2)/power2(msntau2 - mu2) + (
     224*mstau24*mu2*mw2*DeltaInv(mstau22,msntau2,mw2))/power2(msntau2 - mu2) +
     (56*mA2*mu2)/power2(mstau12 - mu2) + (112*mh2*mu2)/power2(mstau12 - mu2) +
     (56*mH2*mu2)/power2(mstau12 - mu2) - (48*mstau12*mu2)/power2(mstau12 -
     mu2) + (214*mstau22*mu2)/power2(mstau12 - mu2) - (42*mstau24*mu2)/((
     mstau12 - mstau22)*power2(mstau12 - mu2)) + (112*mu2*mz2)/power2(mstau12 -
     mu2) + (112*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12
     - mu2) + (112*mstau24*mu2*mz2*DeltaInv(mstau22,mstau12,mz2))/power2(
     mstau12 - mu2) + (112*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) + (56*mA2*mu2)/power2(mstau22 - mu2) + (112*
     mC2*mu2)/power2(mstau22 - mu2) + (56*mH2*mu2)/power2(mstau22 - mu2) + (
     112*msntau2*mu2)/power2(mstau22 - mu2) - (80*mstau12*mu2)/power2(mstau22 -
     mu2) - (96*mstau22*mu2)/power2(mstau22 - mu2) - (70*mstau24*mu2)/((mstau12
     - mstau22)*power2(mstau22 - mu2)) + (114*mstau24*mu2)/((msntau2 - mu2)*
     power2(mstau22 - mu2)) + (114*mstau24*mu2)/((mstau12 - mu2)*power2(mstau22
     - mu2)) - (112*power2(mu2))/(mA2*msntau2) - (112*power2(mu2))/(mH2*
     msntau2) - (112*power2(mu2))/(mA2*mstau12) - (112*power2(mu2))/(mH2*
     mstau12) - (112*power2(mu2))/(mA2*mstau22) - (224*power2(mu2))/(mC2*
     mstau22) - (112*power2(mu2))/(mH2*mstau22) + (24*power2(mu2))/((-msntau2 +
     mu2)*(-mstau12 + mu2)) + (84*power2(mu2))/((mstau12 - mstau22)*(-mstau22 +
     mu2)) + (696*power2(mu2))/((mstau12 - mu2)*(-mstau22 + mu2)) + (8*power2(
     mu2))/((-msntau2 + mu2)*(-mstau22 + mu2)) - (448*power2(mu2))/(mA2*msntau2
     - mA2*mu2) - (448*power2(mu2))/(mA2*mstau12 - mA2*mu2) - (448*power2(mu2))
     /(mA2*mstau22 - mA2*mu2) + (448*power2(mu2))/(mH2*msntau2 - mH2*mu2) + (
     448*power2(mu2))/(mH2*mstau12 - mH2*mu2) + (448*power2(mu2))/(mH2*mstau22
     - mH2*mu2) - 784*DeltaInv(mA2,mstau22,mu2)*power2(mu2) - (336*mA2*
     DeltaInv(mA2,mstau22,mu2)*power2(mu2))/mstau22 - 784*DeltaInv(mH2,msntau2,
     mu2)*power2(mu2) - (336*mH2*DeltaInv(mH2,msntau2,mu2)*power2(mu2))/msntau2
     - 784*DeltaInv(mH2,mstau12,mu2)*power2(mu2) - (336*mH2*DeltaInv(mH2,
     mstau12,mu2)*power2(mu2))/mstau12 - 1568*DeltaInv(mstau22,mC2,mu2)*power2(
     mu2) - (672*mC2*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/mstau22 - 784*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2) - (336*mH2*DeltaInv(mstau22,mH2,mu2)
     *power2(mu2))/mstau22 + (448*mstau22*DeltaInv(mstau22,msntau2,mw2)*power2(
     mu2))/(-msntau2 + mu2) + (448*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(
     mu2))/(-msntau2 + mu2) + (224*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mu2))/(-mstau12 + mu2) + (224*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/(-mstau12 + mu2) + (224*mstau22*DeltaInv(mstau22,mstau12,mz2)
     *power2(mu2))/(-mstau12 + mu2) + (224*mz2*DeltaInv(mstau22,mstau12,mz2)*
     power2(mu2))/(-mstau12 + mu2) + (32*power2(mu2))/(-(msntau2*mu2) + power2(
     msntau2)) + (32*power2(mu2))/(-(mstau12*mu2) + power2(mstau12)) + (84*
     mstau22*power2(mu2))/((-mstau22 + mu2)*power2(mstau12 - mstau22)) + (64*
     power2(mu2))/(-(mstau22*mu2) + power2(mstau22)) - (160*power2(mu2))/
     power2(msntau2 - mu2) - (48*mC2*power2(mu2))/(msntau2*power2(msntau2 -
     mu2)) - (48*mstau22*power2(mu2))/(msntau2*power2(msntau2 - mu2)) - (448*
     mw2*power2(mu2))/(mstau22*power2(msntau2 - mu2)) + (224*mstau24*DeltaInv(
     mstau22,msntau2,mw2)*power2(mu2))/power2(msntau2 - mu2) + (896*mstau22*
     mw2*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/power2(msntau2 - mu2) - (
     160*power2(mu2))/power2(mstau12 - mu2) - (24*mA2*power2(mu2))/(mstau12*
     power2(mstau12 - mu2)) - (24*mH2*power2(mu2))/(mstau12*power2(mstau12 -
     mu2)) - (224*mh2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (48*
     mstau22*power2(mu2))/(mstau12*power2(mstau12 - mu2)) - (224*mz2*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (448*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) + (112*mstau24*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) + (112*mstau24*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(mstau12 - mu2) + (448*
     mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(mstau12 -
     mu2) - (448*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/power2(
     mstau12 - mu2) + (112*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(
     mu2))/power2(mstau12 - mu2) - (852*power2(mu2))/power2(mstau22 - mu2) - (
     448*mh2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (24*mA2*power2(mu2)
     )/(mstau22*power2(mstau22 - mu2)) - (48*mC2*power2(mu2))/(mstau22*power2(
     mstau22 - mu2)) - (24*mH2*power2(mu2))/(mstau22*power2(mstau22 - mu2)) - (
     48*msntau2*power2(mu2))/(mstau22*power2(mstau22 - mu2)) - (48*mstau12*
     power2(mu2))/(mstau22*power2(mstau22 - mu2)) + (98*mstau22*power2(mu2))/((
     mstau12 - mstau22)*power2(mstau22 - mu2)) + (2*mstau22*power2(mu2))/((-
     msntau2 + mu2)*power2(mstau22 - mu2)) + (2*mstau22*power2(mu2))/((-mstau12
     + mu2)*power2(mstau22 - mu2)) - (896*mw2*power2(mu2))/(msntau2*power2(
     mstau22 - mu2)) - (448*mz2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) -
     (896*mstau22*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/power2(mstau22
     - mu2) + (448*mstau24*mw2*DeltaInv(mstau22,msntau2,mw2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) - (448*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (224*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (448*mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(mu2))/power2(
     mstau22 - mu2) + (224*mstau24*mz2*DeltaInv(mstau22,mstau12,mz2)*power2(
     mu2))/(mstau12*power2(mstau22 - mu2)) - (448*DeltaInv(mstau12,mh2,mu2)*
     power2(mh2)*power2(mu2))/power2(mstau22 - mu2) - (448*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/power2(mstau22 - mu2) - (448*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (98*mstau24*power2(mu2))/(power2(mstau12 -
     mstau22)*power2(mstau22 - mu2)) + (98*mstau24*power2(mu2))/(power2(msntau2
     - mu2)*power2(mstau22 - mu2)) + (98*mstau24*power2(mu2))/(power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) - (224*mu2*DeltaInv(mstau22,msntau2,mw2)*
     power2(mw2))/msntau2 + (672*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))
     /(msntau2 - mu2) + (896*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(mw2))/(
     mstau22 - mu2) + (896*mstau22*mu2*DeltaInv(mstau22,msntau2,mw2)*power2(
     mw2))/(msntau2*mstau22 - msntau2*mu2) + (224*mstau22*mu2*DeltaInv(mstau22,
     msntau2,mw2)*power2(mw2))/power2(msntau2 - mu2) + (224*DeltaInv(mstau22,
     msntau2,mw2)*power2(mu2)*power2(mw2))/power2(msntau2 - mu2) - (896*
     DeltaInv(mstau22,mw2,mu2)*power2(mu2)*power2(mw2))/power2(msntau2 - mu2) -
     (896*DeltaInv(msntau2,mw2,mu2)*power2(mu2)*power2(mw2))/power2(mstau22 -
     mu2) - (896*DeltaInv(mstau22,msntau2,mw2)*power2(mu2)*power2(mw2))/power2(
     mstau22 - mu2) - (896*mstau22*DeltaInv(mstau22,msntau2,mw2)*power2(mu2)*
     power2(mw2))/(msntau2*power2(mstau22 - mu2)) - (112*mu2*DeltaInv(mstau22,
     mstau12,mz2)*power2(mz2))/mstau12 + (336*mu2*DeltaInv(mstau22,mstau12,mz2)
     *power2(mz2))/(mstau12 - mu2) + (448*mu2*DeltaInv(mstau22,mstau12,mz2)*
     power2(mz2))/(mstau22 - mu2) + (448*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mz2)*power2(mz2))/(mstau12*mstau22 - mstau12*mu2) + (112*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mz2)*power2(mz2))/power2(mstau12 - mu2) + (112*
     DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*power2(mz2))/power2(mstau12 -
     mu2) - (448*DeltaInv(mstau22,mz2,mu2)*power2(mu2)*power2(mz2))/power2(
     mstau12 - mu2) - (448*DeltaInv(mstau12,mz2,mu2)*power2(mu2)*power2(mz2))/
     power2(mstau22 - mu2) - (448*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*
     power2(mz2))/power2(mstau22 - mu2) - (448*mstau22*DeltaInv(mstau22,
     mstau12,mz2)*power2(mu2)*power2(mz2))/(mstau12*power2(mstau22 - mu2)) - (
     14*power3(mA2))/(mH2*power2(mA2 - mH2)) + (112*mu2*DeltaInv(mA2,msntau2,
     mu2)*(-7*mA2*msntau2*mu2 - (4*msntau2 + 3*mu2)*power2(mA2) + (-msntau2 +
     mu2)*power2(mu2) + 2*power3(mA2)))/(mA2*msntau2) + (112*mu2*DeltaInv(mA2,
     mstau12,mu2)*(-7*mA2*mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mA2) + (-
     mstau12 + mu2)*power2(mu2) + 2*power3(mA2)))/(mA2*mstau12) + (7*power3(
     mC2))/(mA2*power2(mA2 - mC2)) + (7*power3(mC2))/(mH2*power2(mC2 - mH2)) -
     (224*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 -
     mstau12*mu2) - (112*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(
     mstau12 - mu2) + (224*DeltaInv(mstau22,mh2,mu2)*power2(mu2)*power3(mh2))/(
     mstau22*power2(mstau12 - mu2)) + (224*DeltaInv(mstau12,mh2,mu2)*power2(
     mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (224*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (
     28*power3(mstau22))/((msntau2 - mu2)*power2(msntau2 - mstau22)) + (28*
     power3(mstau22))/((mstau12 - mu2)*power2(mstau12 - mstau22)) + (70*power3(
     mstau22))/((-mstau22 + mu2)*power2(mstau12 - mstau22)) + (14*power3(
     mstau22))/((msntau2 - mstau22)*power2(msntau2 - mu2)) - (224*mu2*DeltaInv(
     mstau22,msntau2,mw2)*power3(mstau22))/power2(msntau2 - mu2) + (28*mu2*
     power3(mstau22))/(power2(msntau2 - mstau22)*power2(msntau2 - mu2)) + (14*
     power3(mstau22))/((mstau12 - mstau22)*power2(mstau12 - mu2)) - (112*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mstau22))/power2(mstau12 - mu2) - (
     112*mu2*DeltaInv(mstau22,mstau12,mz2)*power3(mstau22))/power2(mstau12 -
     mu2) + (28*mu2*power3(mstau22))/(power2(mstau12 - mstau22)*power2(mstau12
     - mu2)) + (94*power3(mstau22))/((-msntau2 + mu2)*power2(mstau22 - mu2)) +
     (94*power3(mstau22))/((-mstau12 + mu2)*power2(mstau22 - mu2)) - (70*mu2*
     power3(mstau22))/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) - (154*
     mu2*power3(mstau22))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) - (154*
     mu2*power3(mstau22))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (112*
     DeltaInv(mA2,mstau22,mu2)*power3(mu2))/mA2 - (112*DeltaInv(mH2,msntau2,
     mu2)*power3(mu2))/mH2 - (112*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/mH2 -
     (224*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/mC2 - (112*DeltaInv(mstau22,
     mH2,mu2)*power3(mu2))/mH2 + (224*DeltaInv(mstau22,msntau2,mw2)*power3(mu2)
     )/(mstau22 - mu2) + (112*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(
     mstau22 - mu2) + (112*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/(mstau22
     - mu2) + (48*power3(mu2))/(msntau2*power2(msntau2 - mu2)) - (224*power3(
     mu2))/(mstau22*power2(msntau2 - mu2)) - (1568*mw2*DeltaInv(mstau22,mw2,
     mu2)*power3(mu2))/power2(msntau2 - mu2) + (48*power3(mu2))/(mstau12*
     power2(mstau12 - mu2)) - (224*power3(mu2))/(mstau22*power2(mstau12 - mu2))
     - (784*mh2*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) -
     (784*mz2*DeltaInv(mstau22,mz2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (
     336*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau22*power2(
     mstau12 - mu2)) - (448*power3(mu2))/(msntau2*power2(mstau22 - mu2)) - (
     448*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (28*power3(mu2))/((
     mstau12 - mstau22)*power2(mstau22 - mu2)) + (96*power3(mu2))/(mstau22*
     power2(mstau22 - mu2)) + (18*power3(mu2))/((-msntau2 + mu2)*power2(mstau22
     - mu2)) + (18*power3(mu2))/((-mstau12 + mu2)*power2(mstau22 - mu2)) - (
     1568*mw2*DeltaInv(msntau2,mw2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (
     784*mh2*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (
     784*mz2*DeltaInv(mstau12,mz2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (
     448*mstau22*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/power2(mstau22 -
     mu2) + (224*mstau24*DeltaInv(mstau22,msntau2,mw2)*power3(mu2))/(msntau2*
     power2(mstau22 - mu2)) - (672*mw2*DeltaInv(mstau22,msntau2,mw2)*power3(
     mu2))/power2(mstau22 - mu2) - (448*mstau22*mw2*DeltaInv(mstau22,msntau2,
     mw2)*power3(mu2))/(msntau2*power2(mstau22 - mu2)) - (336*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) - (224*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) - (224*
     mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (112*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/
     (mstau12*power2(mstau22 - mu2)) - (224*mstau22*DeltaInv(mstau22,mstau12,
     mz2)*power3(mu2))/power2(mstau22 - mu2) + (112*mstau24*DeltaInv(mstau22,
     mstau12,mz2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (336*mz2*
     DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/power2(mstau22 - mu2) - (224*
     mstau22*mz2*DeltaInv(mstau22,mstau12,mz2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (336*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power3(mu2))/
     (mstau12*power2(mstau22 - mu2)) + (112*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (28*mstau22*
     power3(mu2))/(power2(mstau12 - mstau22)*power2(mstau22 - mu2)) + (98*
     mstau22*power3(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (98*
     mstau22*power3(mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (672*
     DeltaInv(mstau22,mw2,mu2)*power2(mw2)*power3(mu2))/(mstau22*power2(msntau2
     - mu2)) - (672*DeltaInv(msntau2,mw2,mu2)*power2(mw2)*power3(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (224*DeltaInv(mstau22,msntau2,mw2)*
     power2(mw2)*power3(mu2))/(msntau2*power2(mstau22 - mu2)) - (336*DeltaInv(
     mstau22,mz2,mu2)*power2(mz2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))
     - (336*DeltaInv(mstau12,mz2,mu2)*power2(mz2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (112*DeltaInv(mstau22,mstau12,mz2)*power2(mz2)*power3(
     mu2))/(mstau12*power2(mstau22 - mu2)) - (448*mu2*DeltaInv(mstau22,msntau2,
     mw2)*power3(mw2))/(msntau2*mstau22 - msntau2*mu2) - (224*mu2*DeltaInv(
     mstau22,msntau2,mw2)*power3(mw2))/power2(msntau2 - mu2) + (448*DeltaInv(
     mstau22,mw2,mu2)*power2(mu2)*power3(mw2))/(mstau22*power2(msntau2 - mu2))
     + (448*DeltaInv(msntau2,mw2,mu2)*power2(mu2)*power3(mw2))/(msntau2*power2(
     mstau22 - mu2)) + (448*DeltaInv(mstau22,msntau2,mw2)*power2(mu2)*power3(
     mw2))/(msntau2*power2(mstau22 - mu2)) - (224*mu2*DeltaInv(mstau22,mstau12,
     mz2)*power3(mz2))/(mstau12*mstau22 - mstau12*mu2) - (112*mu2*DeltaInv(
     mstau22,mstau12,mz2)*power3(mz2))/power2(mstau12 - mu2) + (224*DeltaInv(
     mstau22,mz2,mu2)*power2(mu2)*power3(mz2))/(mstau22*power2(mstau12 - mu2))
     + (224*DeltaInv(mstau12,mz2,mu2)*power2(mu2)*power3(mz2))/(mstau12*power2(
     mstau22 - mu2)) + (224*DeltaInv(mstau22,mstau12,mz2)*power2(mu2)*power3(
     mz2))/(mstau12*power2(mstau22 - mu2)) - (21*power4(mA2))/(power2(mA2 -
     mH2)*power2(mH2)) - (21*power4(mC2))/(power2(mA2)*power2(mA2 - mC2)) - (
     21*power4(mC2))/(power2(mC2 - mH2)*power2(mH2)) - (28*power4(mstau22))/(
     power2(msntau2 - mstau22)*power2(msntau2 - mu2)) - (28*power4(mstau22))/(
     power2(mstau12 - mstau22)*power2(mstau12 - mu2)) + (42*power4(mstau22))/(
     power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (42*power4(mstau22))/(
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (240*power2(mu2)*power3(
     mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) - (240*power2(mu2)*
     power3(mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) + (184*mstau24*
     power3(mu2))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (184*mstau24*
     power3(mu2))/((-mstau12 + mu2)*power4(mstau22 - mu2)) + (176*mu2*power4(
     mstau22))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (176*mu2*power4(
     mstau22))/((-mstau12 + mu2)*power4(mstau22 - mu2)) + (112*DeltaInv(mA2,
     mstau22,mu2)*power4(mu2))/(mA2*mstau22) + (112*DeltaInv(mH2,msntau2,mu2)*
     power4(mu2))/(mH2*msntau2) + (112*DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(
     mH2*mstau12) + (224*DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(mC2*mstau22) +
     (112*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mH2*mstau22) - (224*DeltaInv(
     mstau22,mw2,mu2)*power4(mu2))/power2(msntau2 - mu2) - (112*DeltaInv(
     mstau22,mh2,mu2)*power4(mu2))/power2(mstau12 - mu2) - (112*DeltaInv(
     mstau22,mz2,mu2)*power4(mu2))/power2(mstau12 - mu2) - (224*DeltaInv(
     msntau2,mw2,mu2)*power4(mu2))/power2(mstau22 - mu2) - (112*DeltaInv(
     mstau12,mh2,mu2)*power4(mu2))/power2(mstau22 - mu2) - (112*DeltaInv(
     mstau12,mz2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (224*DeltaInv(
     mstau22,msntau2,mw2)*power4(mu2))/power2(mstau22 - mu2) + (112*DeltaInv(
     mstau22,mstau12,mh2)*power4(mu2))/power2(mstau22 - mu2) + (112*DeltaInv(
     mstau22,mstau12,mz2)*power4(mu2))/power2(mstau22 - mu2) - (84*power4(mu2))
     /(power2(msntau2 - mu2)*power2(mstau22 - mu2)) - (84*power4(mu2))/(power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) - (92*mstau22*power4(mu2))/((-
     msntau2 + mu2)*power4(mstau22 - mu2)) - (92*mstau22*power4(mu2))/((-
     mstau12 + mu2)*power4(mstau22 - mu2)) + (52*power5(mstau22))/((msntau2 -
     mu2)*power4(mstau22 - mu2)) + (52*power5(mstau22))/((mstau12 - mu2)*
     power4(mstau22 - mu2)) + (224*DeltaInv(mstau22,mw2,mu2)*power5(mu2))/(
     mstau22*power2(msntau2 - mu2)) + (112*DeltaInv(mstau22,mh2,mu2)*power5(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (112*DeltaInv(mstau22,mz2,mu2)*
     power5(mu2))/(mstau22*power2(mstau12 - mu2)) + (224*DeltaInv(msntau2,mw2,
     mu2)*power5(mu2))/(msntau2*power2(mstau22 - mu2)) + (112*DeltaInv(mstau12,
     mh2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2)) + (112*DeltaInv(
     mstau12,mz2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2)) + (24*
     power5(mu2))/((-msntau2 + mu2)*power4(mstau22 - mu2)) + (24*power5(mu2))/(
     (-mstau12 + mu2)*power4(mstau22 - mu2)))/128. + (3*ca*mu2*sb*log(mh2/Q2)*
     log(mstau12/Q2)*power3(sa)*(mh2 + mstau12 + mstau22 + DeltaInv(mstau22,
     mstau12,mh2)*(mstau12*mstau24 + mh2*(6*mstau12*mstau22 + mstau24) + (
     mstau12 + mstau22)*power2(mh2) - power3(mh2) - power6(mstau2))))/(2.*cb*
     mh2*mstau12) + (3*mu2*log(mh2/Q2)*log(mstau12/Q2)*power2(sb)*power4(sa)*(
     mh2 + mstau12 + mstau22 + DeltaInv(mstau22,mstau12,mh2)*(mstau12*mstau24 +
     mh2*(6*mstau12*mstau22 + mstau24) + (mstau12 + mstau22)*power2(mh2) -
     power3(mh2) - power6(mstau2))))/(4.*mh2*mstau12*power2(cb)) + (3*ca*mu2*
     sb*log(mH2/Q2)*log(mstau12/Q2)*power3(sa)*(mH2 + mstau12 + mstau22 +
     DeltaInv(mstau22,mH2,mstau12)*(mstau12*mstau24 + mH2*(6*mstau12*mstau22 +
     mstau24) + (mstau12 + mstau22)*power2(mH2) - power3(mH2) - power6(mstau2))
     ))/(2.*cb*mH2*mstau12) + (3*mu2*log(mH2/Q2)*log(mstau12/Q2)*power2(sb)*
     power4(sa)*(mH2 + mstau12 + mstau22 + DeltaInv(mstau22,mH2,mstau12)*(
     mstau12*mstau24 + mH2*(6*mstau12*mstau22 + mstau24) + (mstau12 + mstau22)*
     power2(mH2) - power3(mH2) - power6(mstau2))))/(4.*mH2*mstau12*power2(cb))
     - (ca*mu2*sa*sb*log(mstau12/Q2)*(-18/mH2 - 18/mstau12 + mstau12/(mh2*(-mh2
     + mH2)) - (54*mstau22)/(mH2*mstau12) + (9*mh2)/(mstau12*mstau22 - mstau12*
     mu2) - (9*mH2)/(mstau12*mstau22 - mstau12*mu2) - 24*mH2*DeltaInv(mstau22,
     mH2,mstau12) - 177*mstau22*DeltaInv(mstau22,mH2,mstau12) + (12*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (90*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/mH2 - (87*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 + (6*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (9*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (18*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau22 + mu2) + (9*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + 6*mh2*DeltaInv(
     mstau22,mstau12,mh2) - 3*mstau22*DeltaInv(mstau22,mstau12,mh2) + (6*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 - (3*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/mstau12 + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12 - mu2) + (9*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12
     - mu2) + (18*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) +
     (6*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (6*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) - (9*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + mstau12/
     power2(mh2) + (mH2*mstau12)/((mh2 - mH2)*power2(mh2)) - (3*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (9*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12 - mu2) + (15*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau22 - mu2) + (18*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + (21*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/mstau12 + (9*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/(-mstau12 + mu2) + (15*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-
     mstau22 + mu2) - (18*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau12*mstau22 - mstau12*mu2) + (3*mh2)/power2(mstau12 - mu2) - (3*mH2)/
     power2(mstau12 - mu2) - (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) - (18*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))
     /power2(mstau12 - mu2) - (3*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) + (3*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (18*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau12 - mu2) + (3*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (3*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) - (3*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (3*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (18*mh2*mu2)/(mstau12*power2(mstau22
     - mu2)) + (18*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mH2*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) - (9*mH2*mstau24*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) - (18*
     mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) + (9*
     mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 -
     mu2)) - (15*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau22
     - mu2) - (18*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(
     mstau12*power2(mstau22 - mu2)) + (15*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau22 - mu2) + (18*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (3*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 + mu2) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau22 - mu2) + (12*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) - (12*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(
     mstau22 - mu2) - (6*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/
     power2(mstau22 - mu2) - (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*mstau24*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2)
     )/(mstau12*power2(mstau22 - mu2)) - (9*DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (9*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (3*mu2*
     DeltaInv(mstau12,mh2,mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)
     *power2(mh2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mh2)))/(mstau12*
     power2(mstau22 - mu2)) + (9*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(
     mstau12*mstau22 - mstau12*mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/power2(mstau12 - mu2) - (9*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mH2,mstau12,mu2)*(
     mH2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mH2) + (-mstau12 +
     mu2)*power2(mu2) + 3*power3(mH2)))/(mstau12*power2(mstau22 - mu2)) - (3*
     DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (3*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (54*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (3*DeltaInv(
     mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) - (3*DeltaInv(
     mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/(4.*cb) - (
     mu2*log(mstau12/Q2)*power2(sa)*power2(sb)*(-36/mH2 - 36/mstau12 - (108*
     mstau22)/(mH2*mstau12) + (9*mh2)/(mstau12*mstau22 - mstau12*mu2) - (9*mH2)
     /(mstau12*mstau22 - mstau12*mu2) - 42*mH2*DeltaInv(mstau22,mH2,mstau12) -
     357*mstau22*DeltaInv(mstau22,mH2,mstau12) + (30*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 - (180*mstau24*DeltaInv(mstau22,mH2,mstau12)
     )/mH2 - (177*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (6*mH2*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) + (18*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau22 + mu2) + (9*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     mstau22 - mstau12*mu2) + 6*mh2*DeltaInv(mstau22,mstau12,mh2) - 3*mstau22*
     DeltaInv(mstau22,mstau12,mh2) + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/mstau12 - (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*
     mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (9*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (18*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (6*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (6*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) - (9*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*mstau22 - mstau12*mu2) + (2*mstau12)/power2(mh2) + (2*mH2*mstau12)
     /((mh2 - mH2)*power2(mh2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))
     /mstau12 + (9*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12 - mu2) +
     (15*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (18*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) - (2*mstau12)/(-(mh2*mH2) + power2(mh2)) + (39*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (9*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(-mstau12 + mu2) + (15*DeltaInv(mstau22,mH2,mstau12)
     *power2(mH2))/(-mstau22 + mu2) - (18*mstau22*DeltaInv(mstau22,mH2,mstau12)
     *power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (3*mh2)/power2(mstau12 -
     mu2) - (3*mH2)/power2(mstau12 - mu2) - (3*mH2*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau12 - mu2) - (18*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (3*mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (18*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (3*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (3*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (3*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (3*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (3*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (18*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mH2*mu2)/(mstau12*
     power2(mstau22 - mu2)) + (18*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/power2(mstau22 - mu2) - (9*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/(mstau12*power2(mstau22 - mu2)) - (18*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau22 - mu2) + (9*mh2*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) - (15*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) - (18*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2))
     + (15*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2)
     + (18*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*
     power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-
     mstau22 + mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau22 -
     mu2) + (12*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 -
     mu2) + (6*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(
     mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (12*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (6*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (6*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (9*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2)
     - (3*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (
     9*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 -
     mu2)) + (3*mu2*DeltaInv(mstau12,mh2,mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*
     (mstau12 + mu2)*power2(mh2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mh2)
     ))/(mstau12*power2(mstau22 - mu2)) + (9*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (3*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/power2(mstau12 - mu2) - (9*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(
     mH2,mstau12,mu2)*(mH2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(
     mH2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mH2)))/(mstau12*power2(
     mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(
     mstau22 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(
     mstau22 - mu2) + (108*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*
     mstau12) + (3*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12
     - mu2) - (3*DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 -
     mu2)))/(8.*power2(cb)) + (ca*mu2*sa*sb*power2(log(mstau12/Q2))*(-6/mH2 -
     6/mstau12 - (18*mstau22)/(mH2*mstau12) + (3*mh2)/(mstau12*mstau22 -
     mstau12*mu2) - (3*mH2)/(mstau12*mstau22 - mstau12*mu2) - 8*mH2*DeltaInv(
     mstau22,mH2,mstau12) - 59*mstau22*DeltaInv(mstau22,mH2,mstau12) + (4*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (30*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/mH2 - (29*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 + (2*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (3*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(-mstau22 + mu2) + (3*mH2*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + 2*mh2*DeltaInv(mstau22,
     mstau12,mh2) - mstau22*DeltaInv(mstau22,mstau12,mh2) + (2*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 - (mstau24*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12 - mu2) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2)
     + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) - (3*mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/mstau12 + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau12 - mu2) + (5*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau22 - mu2) + (6*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau12*mstau22 - mstau12*mu2) + (7*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-
     mstau12 + mu2) + (5*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau22 +
     mu2) - (6*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*
     mstau22 - mstau12*mu2) + mh2/power2(mstau12 - mu2) - mH2/power2(mstau12 -
     mu2) - (mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) -
     (6*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) -
     (mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mh2*
     mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (6*mh2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) +
     (mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (
     6*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) + (6*mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) + (6*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau22 - mu2) - (3*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (mstau12*power2(mstau22 - mu2)) - (6*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau22 - mu2) + (3*mh2*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) - (5*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) - (6*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (5*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (6*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 +
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau22 - mu2) + (4*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) +
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (4*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) - (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/
     (mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau12,mh2,
     mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mh2) + (-
     mstau12 + mu2)*power2(mu2) + 3*power3(mh2)))/(mstau12*power2(mstau22 -
     mu2)) + (mu2*DeltaInv(mH2,mstau12,mu2)*(mH2*(10*mstau12 - mu2)*mu2 + 5*(
     mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2) - 3*power3(mH2)))
     /(mstau12*power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/power2(mstau12 - mu2) - (3*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,
     mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (18*DeltaInv(mstau22,
     mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/power2(mstau12 - mu2) - (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/(8.*cb) + (mu2*
     power2(sa)*power2(sb)*power2(log(mstau12/Q2))*(-12/mH2 - 12/mstau12 - (36*
     mstau22)/(mH2*mstau12) + (3*mh2)/(mstau12*mstau22 - mstau12*mu2) - (3*mH2)
     /(mstau12*mstau22 - mstau12*mu2) - 14*mH2*DeltaInv(mstau22,mH2,mstau12) -
     119*mstau22*DeltaInv(mstau22,mH2,mstau12) + (10*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 - (60*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /mH2 - (59*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (2*mH2*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau22 + mu2) + (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     mstau22 - mstau12*mu2) + 2*mh2*DeltaInv(mstau22,mstau12,mh2) - mstau22*
     DeltaInv(mstau22,mstau12,mh2) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/mstau12 - (mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (6*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) - (3*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/mstau12 + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12 -
     mu2) + (5*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (6*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) + (13*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (
     3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau12 + mu2) + (5*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau22 + mu2) - (6*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2)
     + mh2/power2(mstau12 - mu2) - mH2/power2(mstau12 - mu2) - (mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (6*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (6*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (6*mh2*
     mu2)/(mstau12*power2(mstau22 - mu2)) + (6*mH2*mu2)/(mstau12*power2(mstau22
     - mu2)) + (6*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22
     - mu2) - (3*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     power2(mstau22 - mu2)) - (6*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau22 - mu2) + (3*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))
     /(mstau12*power2(mstau22 - mu2)) - (5*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (6*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (5*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (6*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 +
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau22 - mu2) + (4*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) +
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (4*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) - (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/
     (mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau12,mh2,
     mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mh2) + (-
     mstau12 + mu2)*power2(mu2) + 3*power3(mh2)))/(mstau12*power2(mstau22 -
     mu2)) + (mu2*DeltaInv(mH2,mstau12,mu2)*(mH2*(10*mstau12 - mu2)*mu2 + 5*(
     mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2) - 3*power3(mH2)))
     /(mstau12*power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/power2(mstau12 - mu2) - (3*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,
     mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (36*DeltaInv(mstau22,
     mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/power2(mstau12 - mu2) - (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/(16.*power2(cb)) - (
     mu2*power2(Pi)*power2(sa)*power2(sb)*(12/mH2 + 12/mstau12 + (24*mstau22)/(
     mH2*mstau12) - (2*mh2)/(mstau12*mstau22 - mstau12*mu2) + (2*mH2)/(mstau12*
     mstau22 - mstau12*mu2) + 13*mH2*DeltaInv(mstau22,mH2,mstau12) + 83*
     mstau22*DeltaInv(mstau22,mH2,mstau12) + (2*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/mstau12 + (48*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (
     35*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (4*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (4*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(mstau22 - mu2) + (2*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12
     + mu2) - (2*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 -
     mstau12*mu2) - mh2*DeltaInv(mstau22,mstau12,mh2) + mstau22*DeltaInv(
     mstau22,mstau12,mh2) - (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12 - mu2) + (4*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))
     /(-mstau12 + mu2) + (4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-
     mstau22 + mu2) + (2*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12*
     mstau22 - mstau12*mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/
     mstau12 + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) +
     (4*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (4*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) + (8*mH2)/(-(mh2*mH2) + power2(mh2)) - (8*mH2)/power2(mh2 -
     mH2) - (13*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (3*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (4*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (4*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (2*
     power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (10*power2(mH2))/(mh2*power2(mh2
     - mH2)) - mh2/power2(mstau12 - mu2) + mH2/power2(mstau12 - mu2) + (2*mh2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12
     - mu2)) + (mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (4*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) -
     (mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (4*
     mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (4*mu2*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2))/power2(mstau12 - mu2) - (mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) +
     (mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (
     4*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (4*
     mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (4*mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (4*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau22 - mu2) + (2*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (mstau12*power2(mstau22 - mu2)) + (4*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau22 - mu2) - (2*mh2*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) + (4*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) + (4*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) - (4*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) - (4*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 -
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) + (7*
     mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (7*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (3*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (3*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))
     /power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/power2(mstau22 - mu2) - (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) +
     (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mstau12,mh2,mu2)*(7*mh2*mstau12*mu2 + (4*mstau12 + 3*mu2)*power2(
     mh2) + (mstau12 - mu2)*power2(mu2) - 2*power3(mh2)))/(mstau12*power2(
     mstau22 - mu2)) + (2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*
     mstau22 - mstau12*mu2) - (2*mu2*DeltaInv(mstau22,mh2,mu2)*power3(mh2))/(
     mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/power2(mstau12 - mu2) - (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/(mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) - (2*power3(mH2))/(power2(
     mh2)*power2(mh2 - mH2)) - (DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/
     power2(mstau12 - mu2) + (2*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mH2,mstau12,
     mu2)*(-7*mH2*mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 +
     mu2)*power2(mu2) + 2*power3(mH2)))/(mstau12*power2(mstau22 - mu2)) + (
     DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (DeltaInv(mstau22,
     mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) - (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) - (DeltaInv(mstau22,mh2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mH2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) - (24*DeltaInv(mstau22,
     mH2,mstau12)*power6(mstau2))/(mH2*mstau12) - (DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/(48.*power2(cb)) - (
     ca*mu2*sa*sb*power2(Pi)*(6/mH2 + 6/mstau12 + (12*mstau22)/(mH2*mstau12) -
     (2*mh2)/(mstau12*mstau22 - mstau12*mu2) + (2*mH2)/(mstau12*mstau22 -
     mstau12*mu2) + 7*mH2*DeltaInv(mstau22,mH2,mstau12) + 41*mstau22*DeltaInv(
     mstau22,mH2,mstau12) + (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 + (24*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (17*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (4*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     (mstau12 - mu2) + (4*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau22 -
     mu2) + (2*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) - (2*mH2*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) -
     mh2*DeltaInv(mstau22,mstau12,mh2) + mstau22*DeltaInv(mstau22,mstau12,mh2)
     - (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (mstau24*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12 - mu2) + (4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-
     mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 +
     mu2) + (4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (
     2*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (3*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (4*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (4*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + (4*mH2)/(-(
     mh2*mH2) + power2(mh2)) - (4*mH2)/power2(mh2 - mH2) - (7*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12 - mu2) + (4*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/(mstau22 - mu2) + (4*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/(mstau12*mstau22 - mstau12*mu2) + power2(mH2)/((-mh2 + mH2)*power2(
     mh2)) + (5*power2(mH2))/(mh2*power2(mh2 - mH2)) - mh2/power2(mstau12 -
     mu2) + mH2/power2(mstau12 - mu2) + (2*mh2*mu2)/(mstau22*power2(mstau12 -
     mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (4*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (4*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (4*mu2*DeltaInv(
     mstau22,mh2,mu2)*power2(mh2))/power2(mstau12 - mu2) - (mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) + (mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (4*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (4*mh2*mu2)
     /(mstau12*power2(mstau22 - mu2)) - (4*mH2*mu2)/(mstau12*power2(mstau22 -
     mu2)) - (4*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 -
     mu2) + (2*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(
     mstau22 - mu2)) + (4*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau22 - mu2) - (2*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     (mstau12*power2(mstau22 - mu2)) + (4*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) + (4*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) - (4*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) - (4*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 -
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) + (7*
     mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (7*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (3*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (3*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))
     /power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/power2(mstau22 - mu2) - (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) +
     (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mstau12,mh2,mu2)*(7*mh2*mstau12*mu2 + (4*mstau12 + 3*mu2)*power2(
     mh2) + (mstau12 - mu2)*power2(mu2) - 2*power3(mh2)))/(mstau12*power2(
     mstau22 - mu2)) + (2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*
     mstau22 - mstau12*mu2) - (2*mu2*DeltaInv(mstau22,mh2,mu2)*power3(mh2))/(
     mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/power2(mstau12 - mu2) - (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/(mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) - power3(mH2)/(power2(mh2)*
     power2(mh2 - mH2)) - (DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(
     mstau12 - mu2) + (2*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*
     power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))
     /(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mH2,mstau12,mu2)*(-7*mH2*
     mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 + mu2)*power2(
     mu2) + 2*power3(mH2)))/(mstau12*power2(mstau22 - mu2)) + (DeltaInv(
     mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (DeltaInv(mstau22,
     mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power3(mu2))/power2(mstau22 - mu2) - (DeltaInv(mstau22,mstau12,
     mh2)*power3(mu2))/power2(mstau22 - mu2) - (DeltaInv(mstau22,mh2,mu2)*
     power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mH2,mu2)*
     power4(mu2))/(mstau22*power2(mstau12 - mu2)) - (12*DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/(mH2*mstau12) - (DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/power2(mstau12 - mu2) + (DeltaInv(mstau22,mstau12,mh2)*
     power6(mstau2))/power2(mstau12 - mu2)))/(24.*cb) - (mu2*power2(sa)*power2(
     sb)*(-7/mh2 + 7/(mh2 - mH2) + 120/mH2 + 168/mstau12 + (336*mstau22)/(mH2*
     mstau12) - (28*mh2)/(mstau12*mstau22 - mstau12*mu2) + (28*mH2)/(mstau12*
     mstau22 - mstau12*mu2) + 182*mH2*DeltaInv(mstau22,mH2,mstau12) + 1162*
     mstau22*DeltaInv(mstau22,mH2,mstau12) + (28*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/mstau12 + (672*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 +
     (490*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (56*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (42*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (56*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(mstau22 - mu2) + (28*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) + (28*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau12 + mu2) - (28*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     mstau22 - mstau12*mu2) - 14*mh2*DeltaInv(mstau22,mstau12,mh2) + 14*
     mstau22*DeltaInv(mstau22,mstau12,mh2) - (28*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (14*mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12
     + (28*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (28*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (56*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (42*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (56*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (28*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) - (7*mH2)/
     power2(mh2) - (7*mstau12)/power2(mh2) + (7*mH2*mstau12)/((-mh2 + mH2)*
     power2(mh2)) - (7*mstau22)/power2(mh2) + (7*mH2*mstau22)/((-mh2 + mH2)*
     power2(mh2)) + (14*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (
     42*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (56*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (56*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (32*mH2)/(-(mh2*mH2) + power2(mh2)) + (7*mstau12)/(-(mh2*mH2) + power2(
     mh2)) + (7*mstau22)/(-(mh2*mH2) + power2(mh2)) - (32*mH2)/power2(mh2 -
     mH2) - (182*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (42*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (56*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (56*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (21*
     power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (46*power2(mH2))/(mh2*power2(mh2
     - mH2)) - (14*mh2)/power2(mstau12 - mu2) + (14*mH2)/power2(mstau12 - mu2)
     + (28*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (28*mH2*mu2)/(mstau22*
     power2(mstau12 - mu2)) + (14*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) + (56*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))
     /power2(mstau12 - mu2) + (14*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) - (14*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) - (56*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau12 - mu2) - (14*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (56*mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/
     power2(mstau12 - mu2) - (14*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) - (14*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau12 - mu2) + (14*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/power2(mstau12 - mu2) + (14*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (56*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (56*mh2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (56*mH2*mu2)/(mstau12*power2(mstau22 -
     mu2)) - (56*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22
     - mu2) + (28*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     power2(mstau22 - mu2)) + (56*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)
     )/power2(mstau22 - mu2) - (28*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12*power2(mstau22 - mu2)) + (56*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau22 - mu2) + (56*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) - (56*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) - (
     56*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (14*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22
     - mu2) + (14*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) +
     (98*mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (
     98*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (42*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (42*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (42*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2)
     )/power2(mstau22 - mu2) - (28*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/power2(mstau22 - mu2) - (28*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (14*mstau24*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (42*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2)
     + (28*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 -
     mu2) + (28*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (14*mstau24*DeltaInv(mstau22,mstau12,mh2)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (14*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     14*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mstau12,mh2,mu2)*(7*mh2*mstau12*mu2 + (
     4*mstau12 + 3*mu2)*power2(mh2) + (mstau12 - mu2)*power2(mu2) - 2*power3(
     mh2)))/(mstau12*power2(mstau22 - mu2)) + (28*DeltaInv(mstau22,mstau12,mh2)
     *power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (28*mu2*DeltaInv(mstau22,
     mh2,mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (28*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))
     - (28*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - (14*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (14*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (28*
     mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2))
     + (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mH2,mstau12,mu2)*(-7*mH2*mstau12*mu2 -
     (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 2*power3(
     mH2)))/(mstau12*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mh2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) - (14*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/power2(mstau12 - mu2) + (14*DeltaInv(mstau22,mH2,mstau12)*power3(
     mu2))/power2(mstau22 - mu2) - (14*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/power2(mstau22 - mu2) - (14*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(mstau22,mH2,mu2)*power4(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (336*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/(mH2*mstau12) - (14*DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) + (14*DeltaInv(mstau22,mstau12,mh2)*power6(
     mstau2))/power2(mstau12 - mu2)))/(16.*power2(cb)) - (ca*mu2*sa*sb*(-7/mh2
     + 7/(mh2 - mH2) + 120/mH2 + 168/mstau12 + (336*mstau22)/(mH2*mstau12) - (
     56*mh2)/(mstau12*mstau22 - mstau12*mu2) + (56*mH2)/(mstau12*mstau22 -
     mstau12*mu2) + 196*mH2*DeltaInv(mstau22,mH2,mstau12) + 1148*mstau22*
     DeltaInv(mstau22,mH2,mstau12) + (56*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/mstau12 + (672*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (
     476*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (112*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (84*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (112*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(mstau22 - mu2) + (56*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) + (56*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau12 + mu2) - (56*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     mstau22 - mstau12*mu2) - 28*mh2*DeltaInv(mstau22,mstau12,mh2) + 28*
     mstau22*DeltaInv(mstau22,mstau12,mh2) - (56*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (28*mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12
     + (56*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (56*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (112*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (84*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (112*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (56*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) - (7*mH2)/
     power2(mh2) - (7*mstau12)/power2(mh2) + (7*mH2*mstau12)/((-mh2 + mH2)*
     power2(mh2)) - (7*mstau22)/power2(mh2) + (7*mH2*mstau22)/((-mh2 + mH2)*
     power2(mh2)) + (28*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (
     84*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (112*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (112*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) + (32*mH2)/(-(mh2*mH2) + power2(mh2)) + (7*mstau12)/(-(mh2*
     mH2) + power2(mh2)) + (7*mstau22)/(-(mh2*mH2) + power2(mh2)) - (32*mH2)/
     power2(mh2 - mH2) - (196*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     mstau12 + (84*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) +
     (112*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (112*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (21*power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (46*power2(
     mH2))/(mh2*power2(mh2 - mH2)) - (28*mh2)/power2(mstau12 - mu2) + (28*mH2)/
     power2(mstau12 - mu2) + (56*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (
     56*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (28*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (112*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (28*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (28*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (112*mh2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (28*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (112*mu2*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2))/power2(mstau12 - mu2) - (28*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) -
     (28*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) +
     (28*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 -
     mu2) + (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 -
     mu2) - (112*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 -
     mu2) + (112*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (112*mH2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (112*mH2*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau22 - mu2) + (56*mH2*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (112*mh2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) - (56*mh2*
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2))
     + (112*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau22 -
     mu2) + (112*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(
     mstau12*power2(mstau22 - mu2)) - (112*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau22 - mu2) - (112*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (28*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau22 - mu2) + (28*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) + (196*mh2*DeltaInv(mstau22,
     mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (196*mH2*DeltaInv(mstau22,
     mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (84*DeltaInv(mstau22,mh2,
     mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (84*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (84*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(
     mstau22 - mu2) - (56*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/
     power2(mstau22 - mu2) - (56*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (28*mstau24*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (84*
     mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (
     56*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 -
     mu2) + (56*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (28*mstau24*DeltaInv(mstau22,mstau12,mh2)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (28*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     28*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (28*mu2*DeltaInv(mstau12,mh2,mu2)*(7*mh2*mstau12*mu2 + (
     4*mstau12 + 3*mu2)*power2(mh2) + (mstau12 - mu2)*power2(mu2) - 2*power3(
     mh2)))/(mstau12*power2(mstau22 - mu2)) + (56*DeltaInv(mstau22,mstau12,mh2)
     *power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (56*mu2*DeltaInv(mstau22,
     mh2,mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2)) + (28*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (56*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))
     - (56*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - (14*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (28*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (56*
     mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2))
     + (56*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (28*mu2*DeltaInv(mH2,mstau12,mu2)*(-7*mH2*mstau12*mu2 -
     (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 2*power3(
     mH2)))/(mstau12*power2(mstau22 - mu2)) + (28*DeltaInv(mstau22,mh2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) - (28*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/power2(mstau12 - mu2) + (28*DeltaInv(mstau22,mH2,mstau12)*power3(
     mu2))/power2(mstau22 - mu2) - (28*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/power2(mstau22 - mu2) - (28*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (28*DeltaInv(mstau22,mH2,mu2)*power4(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (336*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/(mH2*mstau12) - (28*DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) + (28*DeltaInv(mstau22,mstau12,mh2)*power6(
     mstau2))/power2(mstau12 - mu2)))/(16.*cb) + (power2(sa)*power2(log(
     mstau12/Q2))*((-21*mstau12)/mh2 + (24*mstau12)/(mh2 - mH2) - (3*mstau12)/
     mH2 + (18*mu2)/mh2 + (6*mu2)/mH2 + (24*mu2)/mstau12 + (36*mstau22*mu2)/(
     mh2*mstau12) + (36*mstau22*mu2)/(mH2*mstau12) - (6*mh2*mu2)/(mstau12*
     mstau22 - mstau12*mu2) + (6*mH2*mu2)/(mstau12*mstau22 - mstau12*mu2) + 16*
     mH2*mu2*DeltaInv(mstau22,mH2,mstau12) + 118*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12) - (8*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/mstau12 +
     (60*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/mH2 + (58*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (4*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mstau24*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12 - mu2) + (12*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(mstau22 - mu2) - (6*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12*mstau22 - mstau12*mu2) + 8*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2) + 122*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2) - (16*mh2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (60*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/mh2 + (62*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (4*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     (-mstau12 + mu2) + (6*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12
     + mu2) + (12*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau22 +
     mu2) + (6*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22
     - mstau12*mu2) + (8*mH2*mstau12)/power2(mh2) - (10*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/mstau12 + (6*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(-mstau12 + mu2) + (10*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(-mstau22 + mu2) - (12*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) - (32*mH2*mstau12)/(-(
     mh2*mH2) + power2(mh2)) - (14*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/mstau12 + (6*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12
     - mu2) + (10*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 -
     mu2) + (12*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau12*mstau22 - mstau12*mu2) + (8*mstau12*power2(mH2))/((mh2 - mH2)*
     power2(mh2)) - (2*mh2*mu2)/power2(mstau12 - mu2) + (2*mH2*mu2)/power2(
     mstau12 - mu2) + (2*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(
     mstau12 - mu2) - (2*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))
     /power2(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (2*power2(mu2))/(mh2*mstau12) + (2*
     power2(mu2))/(mH2*mstau12) + (8*power2(mu2))/(mh2*mstau12 - mh2*mu2) - (8*
     power2(mu2))/(mH2*mstau12 - mH2*mu2) + (4*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(-mstau12 + mu2) + (4*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(-mstau12 + mu2) + (4*mh2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(mstau12 - mu2) + (4*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(mstau12 - mu2) + (12*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau12 - mu2) + (2*mstau24*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau12 - mu2) - (12*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) - (2*
     mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) -
     (2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/power2(mstau12 -
     mu2) + (2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/power2(
     mstau12 - mu2) + (12*mh2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     12*mH2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (12*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (12*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/
     power2(mstau22 - mu2) - (6*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (10*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/power2(mstau22 - mu2) + (12*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (10*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(
     mu2))/power2(mstau22 - mu2) - (12*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (6*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2)
     - (6*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)*power3(mh2))/(mstau12*
     power2(mstau22 - mu2)) + (2*mu2*DeltaInv(mstau12,mh2,mu2)*(-2*mstau22*mu2
     + mu2*(-mh2 + mu2) + power2(mstau22))*(mh2*mu2*(-10*mstau12 + mu2) - 5*(
     mstau12 + mu2)*power2(mh2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mh2))
     )/(mh2*mstau12*power2(mstau22 - mu2)) - (6*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*mstau22 - mstau12*mu2) - (2*mu2*DeltaInv(
     mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (6*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2)*power3(mH2))/(mstau12*power2(mstau22 -
     mu2)) - (2*mu2*DeltaInv(mH2,mstau12,mu2)*(-2*mstau22*mu2 + mu2*(-mH2 +
     mu2) + power2(mstau22))*(mH2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*
     power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mH2)))/(mH2*mstau12*
     power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/(
     mstau22 - mu2) + (2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(-mstau22 +
     mu2) - (8*mH2*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 -
     mu2) - (4*mstau22*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(
     mstau22 - mu2) - (4*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))
     /(mstau12*power2(mstau22 - mu2)) + (2*mstau24*DeltaInv(mstau22,mH2,
     mstau12)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (8*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (4*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (4*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(mstau22
     - mu2)) - (2*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (2*
     DeltaInv(mstau22,mH2,mstau12)*power4(mu2))/power2(mstau22 - mu2) - (2*
     DeltaInv(mstau22,mstau12,mh2)*power4(mu2))/power2(mstau22 - mu2) - (36*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) - (36*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/(mh2*mstau12) - (2*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (2*
     mu2*DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/
     32. - (log(mstau12/Q2)*power2(sa)*(mh2/mstau12 - mH2/mstau12 - (16*
     mstau12)/mh2 + (22*mstau12)/(mh2 - mH2) + (6*mstau12)/mH2 + mH2/(mstau12 -
     mu2) + (27*mu2)/mh2 + (9*mu2)/mH2 + (36*mu2)/mstau12 + (54*mstau22*mu2)/(
     mh2*mstau12) + (54*mstau22*mu2)/(mH2*mstau12) + mh2/(-mstau12 + mu2) - (9*
     mh2*mu2)/(mstau12*mstau22 - mstau12*mu2) + (9*mH2*mu2)/(mstau12*mstau22 -
     mstau12*mu2) + 24*mH2*mu2*DeltaInv(mstau22,mH2,mstau12) + 177*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12) - (12*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12))/mstau12 + (90*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/mH2 + (
     87*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (6*mH2*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (9*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (18*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau22 - mu2) - (9*mH2*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + 12*mh2*
     mu2*DeltaInv(mstau22,mstau12,mh2) + 183*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2) - (24*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12
     + (90*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mh2 + (93*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/(-mstau12 + mu2) + (9*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (18*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau22 + mu2) + (9*mh2*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + (10*mH2*mstau12)/power2(
     mh2) - (2*mstau12*mu2)/power2(mh2) + (2*mH2*mstau12*mu2)/((-mh2 + mH2)*
     power2(mh2)) - (15*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12
     + (9*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (
     15*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (18*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) - (32*mH2*mstau12)/(-(mh2*mH2) + power2(mh2)) + (2*mstau12*
     mu2)/(-(mh2*mH2) + power2(mh2)) - (21*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/mstau12 + (9*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau12 - mu2) + (15*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau22 - mu2) + (18*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)
     )/(mstau12*mstau22 - mstau12*mu2) + (10*mstau12*power2(mH2))/((mh2 - mH2)*
     power2(mh2)) - (2*mh2*mu2)/power2(mstau12 - mu2) + (2*mH2*mu2)/power2(
     mstau12 - mu2) + (3*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(
     mstau12 - mu2) - (3*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) - (3*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))
     /power2(mstau12 - mu2) + (3*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (3*power2(mu2))/(mh2*mstau12) + (3*
     power2(mu2))/(mH2*mstau12) + (12*power2(mu2))/(mh2*mstau12 - mh2*mu2) - (
     12*power2(mu2))/(mH2*mstau12 - mH2*mu2) + (6*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(-mstau12 + mu2) + (6*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(-mstau12 + mu2) + (6*mh2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(mstau12 - mu2) + (6*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(mstau12 - mu2) - (mh2*power2(mu2))/(mstau12*power2(
     mstau12 - mu2)) + (mH2*power2(mu2))/(mstau12*power2(mstau12 - mu2)) + (18*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau12 -
     mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(
     mstau12 - mu2) - (18*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)
     )/power2(mstau12 - mu2) - (3*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(
     mu2))/power2(mstau12 - mu2) - (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)
     *power2(mu2))/power2(mstau12 - mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2)*power2(mu2))/power2(mstau12 - mu2) + (18*mh2*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (18*mH2*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (18*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/power2(mstau22 - mu2) + (9*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (18*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (9*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (15*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/power2(
     mstau22 - mu2) + (18*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (15*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/power2(mstau22 - mu2) - (18*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (9*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(
     mstau12*mstau22 - mstau12*mu2) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/power2(mstau12 - mu2) - (9*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (3*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-2*mstau22*mu2 + mu2*(-mh2 + mu2) + power2(
     mstau22))*(mh2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mh2) + (
     -mstau12 + mu2)*power2(mu2) + 3*power3(mh2)))/(mh2*mstau12*power2(mstau22
     - mu2)) - (9*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*
     mstau22 - mstau12*mu2) - (3*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))
     /power2(mstau12 - mu2) + (9*DeltaInv(mstau22,mH2,mstau12)*power2(mu2)*
     power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mH2,
     mstau12,mu2)*(-2*mstau22*mu2 + mu2*(-mH2 + mu2) + power2(mstau22))*(mH2*
     mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mH2) + (-mstau12 + mu2)
     *power2(mu2) + 3*power3(mH2)))/(mH2*mstau12*power2(mstau22 - mu2)) + (2*
     mh2*power2(mu2))/power3(mstau12 - mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*
     power3(mu2))/(mstau22 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/(-mstau22 + mu2) - (12*mH2*DeltaInv(mstau22,mH2,mstau12)*power3(mu2)
     )/power2(mstau22 - mu2) - (6*mstau22*DeltaInv(mstau22,mH2,mstau12)*power3(
     mu2))/power2(mstau22 - mu2) - (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)
     *power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*mstau24*DeltaInv(
     mstau22,mH2,mstau12)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (12*
     mh2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (6*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) +
     (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     2*mH2*power2(mu2))/power3(-mstau12 + mu2) + (3*DeltaInv(mstau22,mH2,
     mstau12)*power4(mu2))/power2(mstau22 - mu2) - (3*DeltaInv(mstau22,mstau12,
     mh2)*power4(mu2))/power2(mstau22 - mu2) - (54*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/(mH2*mstau12) - (54*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power6(mstau2))/(mh2*mstau12) - (3*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power6(mstau2))/power2(mstau12 - mu2)))/8. + (power2(Pi)*power2(sa)*(
     6 + (2*mA2)/(mA2 - mh2) + (2*mC2)/(mC2 - mh2) + (26*mA2)/mh2 - (16*mC2)/
     mh2 + (8*mh2)/mA2 - (2*mC2)/(mC2 - mH2) - (26*mA2)/mH2 + (16*mC2)/mH2 - (
     12*mh2)/mH2 - (8*mH2)/mA2 - (74*mH2)/mh2 + (62*mH2)/(mh2 - mH2) + (2*mA2)/
     (-mA2 + mH2) + (6*msntau2)/mh2 - (6*msntau2)/mH2 - (42*mstau12)/mh2 + (48*
     mstau12)/(mh2 - mH2) - (6*mstau12)/mH2 - (42*mstau22)/mh2 + (48*mstau22)/(
     mh2 - mH2) - (6*mstau22)/mH2 + (42*mu2)/mh2 + (6*mu2)/mH2 + (48*mu2)/
     mstau12 + (48*mstau22*mu2)/(mh2*mstau12) + (48*mstau22*mu2)/(mH2*mstau12)
     - (8*mh2*mu2)/(mstau12*mstau22 - mstau12*mu2) + (8*mH2*mu2)/(mstau12*
     mstau22 - mstau12*mu2) - 16*mh2*mu2*DeltaInv(msntau2,mh2,mu2) - 16*mh2*
     mu2*DeltaInv(mstau12,mh2,mu2) - 16*mh2*mu2*DeltaInv(mstau22,mh2,mu2) + 28*
     mH2*mu2*DeltaInv(mstau22,mH2,mstau12) + 164*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12) + (8*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/mstau12 +
     (96*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/mH2 + (68*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (16*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (12*mstau24*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12 - mu2) + (16*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(mstau22 - mu2) - (8*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12*mstau22 - mstau12*mu2) + 16*mH2*mu2*DeltaInv(mstau22,
     mH2,mu2) + 20*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) + 172*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2) - (8*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (96*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mh2
     + (76*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (16*mh2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (12*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (16*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (8*mh2*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + (2*
     power2(mA2))/power2(mA2 - mh2) + (2*power2(mC2))/power2(mC2 - mh2) + (16*
     mH2*mstau12)/power2(mh2) + (16*mH2*mstau22)/power2(mh2) + (9*power2(mA2))/
     power2(mh2) + (9*power2(mC2))/power2(mh2) + (8*mu2*DeltaInv(msntau2,mh2,
     mu2)*power2(mh2))/msntau2 + (8*mu2*DeltaInv(mstau12,mh2,mu2)*power2(mh2))/
     mstau12 + (8*mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/mstau22 - (20*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (12*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (16*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (16*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2)
     - (15*power2(mh2))/power2(mA2) - (96*mH2*mstau12)/(-(mh2*mH2) + power2(
     mh2)) - (96*mH2*mstau22)/(-(mh2*mH2) + power2(mh2)) + (32*mH2*mu2)/(-(mh2*
     mH2) + power2(mh2)) - (2*power2(mA2))/power2(mA2 - mH2) - (2*power2(mC2))/
     power2(mC2 - mH2) + (32*mH2*mstau12)/power2(mh2 - mH2) + (32*mH2*mstau22)/
     power2(mh2 - mH2) - (32*mH2*mu2)/power2(mh2 - mH2) - (9*power2(mA2))/
     power2(mH2) - (9*power2(mC2))/power2(mH2) + (9*power2(mh2))/power2(mH2) -
     (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (12*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (16*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (16*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*
     mu2) - (8*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/mstau22 + (15*power2(
     mH2))/power2(mA2) + (29*power2(mH2))/power2(mh2) + (24*mstau12*power2(mH2)
     )/((mh2 - mH2)*power2(mh2)) + (24*mstau22*power2(mH2))/((mh2 - mH2)*
     power2(mh2)) + (8*mu2*power2(mH2))/((-mh2 + mH2)*power2(mh2)) - (84*
     power2(mH2))/(-(mh2*mH2) + power2(mh2)) + (2*power2(mH2))/power2(mh2 -
     mH2) - (40*mstau12*power2(mH2))/(mh2*power2(mh2 - mH2)) - (40*mstau22*
     power2(mH2))/(mh2*power2(mh2 - mH2)) + (40*mu2*power2(mH2))/(mh2*power2(
     mh2 - mH2)) - (4*mh2*mu2)/power2(mstau12 - mu2) + (4*mH2*mu2)/power2(
     mstau12 - mu2) + (4*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(
     mstau12 - mu2) - (4*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) - (4*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))
     /power2(mstau12 - mu2) + (4*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (4*power2(mu2))/(mh2*msntau2) + (4*
     power2(mu2))/(mH2*msntau2) - (4*power2(mu2))/(mh2*mstau12) + (4*power2(
     mu2))/(mH2*mstau12) - (4*power2(mu2))/(mh2*mstau22) + (4*power2(mu2))/(
     mH2*mstau22) + (16*power2(mu2))/(mh2*msntau2 - mh2*mu2) + (16*power2(mu2))
     /(mh2*mstau12 - mh2*mu2) + (16*power2(mu2))/(mh2*mstau22 - mh2*mu2) - (16*
     power2(mu2))/(mH2*msntau2 - mH2*mu2) - (16*power2(mu2))/(mH2*mstau12 -
     mH2*mu2) - (16*power2(mu2))/(mH2*mstau22 - mH2*mu2) - 28*DeltaInv(msntau2,
     mh2,mu2)*power2(mu2) - (12*mh2*DeltaInv(msntau2,mh2,mu2)*power2(mu2))/
     msntau2 - 28*DeltaInv(mstau12,mh2,mu2)*power2(mu2) - (12*mh2*DeltaInv(
     mstau12,mh2,mu2)*power2(mu2))/mstau12 - 28*DeltaInv(mstau22,mh2,mu2)*
     power2(mu2) - (12*mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/mstau22 + (8*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau12 + mu2) + (8*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau12 + mu2) + 28*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2) + (12*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/mstau22 + (8*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12 - mu2) + (8*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12 - mu2) + (8*mh2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (8*
     mH2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (16*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau12 - mu2) + (4*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau12 - mu2) -
     (16*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12
     - mu2) - (4*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(
     mstau12 - mu2) + (16*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/
     power2(mstau12 - mu2) - (4*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/power2(mstau12 - mu2) + (4*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2)*power2(mu2))/power2(mstau12 - mu2) - (16*DeltaInv(mstau22,mH2,
     mu2)*power2(mH2)*power2(mu2))/power2(mstau12 - mu2) + (16*mh2*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (16*mH2*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (16*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/power2(mstau22 - mu2) + (8*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (16*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (8*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (16*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power2(mu2))/power2(mstau22 -
     mu2) + (16*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/power2(
     mstau22 - mu2) + (16*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (16*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/power2(mstau22 - mu2) - (16*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (2*power3(mA2))/(mh2*power2(mA2 - mh2)) + (2*power3(mA2)
     )/(mH2*power2(mA2 - mH2)) - (2*power3(mC2))/(mh2*power2(mC2 - mh2)) + (2*
     power3(mC2))/(mH2*power2(mC2 - mH2)) + (8*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2) + (4*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (8*DeltaInv(
     mstau22,mh2,mu2)*power2(mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2))
     - (8*DeltaInv(mstau12,mh2,mu2)*power2(mu2)*power3(mh2))/(mstau12*power2(
     mstau22 - mu2)) - (8*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)*power3(mh2)
     )/(mstau12*power2(mstau22 - mu2)) - (8*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (20*power3(mH2))/((mh2 -
     mH2)*power2(mh2)) - (2*power3(mH2))/(mh2*power2(mh2 - mH2)) + (8*mstau12*
     power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) + (8*mstau22*power3(mH2))/(
     power2(mh2)*power2(mh2 - mH2)) - (8*mu2*power3(mH2))/(power2(mh2)*power2(
     mh2 - mH2)) - (4*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(
     mstau12 - mu2) + (8*DeltaInv(mstau22,mH2,mu2)*power2(mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) + (8*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (4*mu2*DeltaInv(mH2,
     msntau2,mu2)*(-7*mH2*msntau2*mu2 - (4*msntau2 + 3*mu2)*power2(mH2) + (-
     msntau2 + mu2)*power2(mu2) + 2*power3(mH2)))/(mH2*msntau2) - (4*mu2*
     DeltaInv(mH2,mstau12,mu2)*(-2*mstau22*mu2 + mu2*(-mH2 + mu2) + power2(
     mstau22))*(-7*mH2*mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mH2) + (-
     mstau12 + mu2)*power2(mu2) + 2*power3(mH2)))/(mH2*mstau12*power2(mstau22 -
     mu2)) - (4*DeltaInv(msntau2,mh2,mu2)*power3(mu2))/mh2 - (4*DeltaInv(
     mstau12,mh2,mu2)*power3(mu2))/mh2 - (4*DeltaInv(mstau22,mh2,mu2)*power3(
     mu2))/mh2 + (4*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/(mstau22 - mu2)
     + (4*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mH2 + (4*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/(-mstau22 + mu2) + (28*mh2*DeltaInv(mstau22,mh2,
     mu2)*power3(mu2))/power2(mstau12 - mu2) - (28*mH2*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/power2(mstau12 - mu2) + (12*DeltaInv(mstau22,mh2,mu2)*
     power2(mh2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (12*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))
     + (28*mh2*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (
     12*mH2*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) -
     (8*mstau22*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 -
     mu2) - (8*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (4*mstau24*DeltaInv(mstau22,mH2,mstau12)*power3(
     mu2))/(mstau12*power2(mstau22 - mu2)) + (12*mh2*DeltaInv(mstau22,mstau12,
     mh2)*power3(mu2))/power2(mstau22 - mu2) + (8*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (8*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (4*
     mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(mstau22
     - mu2)) + (12*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power3(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (4*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (4*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (4*
     DeltaInv(msntau2,mh2,mu2)*power4(mu2))/(mh2*msntau2) + (4*DeltaInv(
     mstau12,mh2,mu2)*power4(mu2))/(mh2*mstau12) + (4*DeltaInv(mstau22,mh2,mu2)
     *power4(mu2))/(mh2*mstau22) - (4*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(
     mH2*mstau22) + (4*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/power2(mstau12 -
     mu2) - (4*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/power2(mstau12 - mu2) + (
     4*DeltaInv(mstau12,mh2,mu2)*power4(mu2))/power2(mstau22 - mu2) + (4*
     DeltaInv(mstau22,mH2,mstau12)*power4(mu2))/power2(mstau22 - mu2) - (4*
     DeltaInv(mstau22,mstau12,mh2)*power4(mu2))/power2(mstau22 - mu2) - (4*
     DeltaInv(mstau22,mh2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     4*DeltaInv(mstau22,mH2,mu2)*power5(mu2))/(mstau22*power2(mstau12 - mu2)) -
     (4*DeltaInv(mstau12,mh2,mu2)*power5(mu2))/(mstau12*power2(mstau22 - mu2))
     - (48*mu2*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) - (
     48*mu2*DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/(mh2*mstau12) - (4*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) +
     (4*mu2*DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)
     ))/192. + (power2(sa)*(264 + (56*mA2)/(mA2 - mh2) + (35*mC2)/(mC2 - mh2) +
     (368*mA2)/mh2 - (45*mC2)/mh2 - (72*mh2)/mA2 - (35*mC2)/(mC2 - mH2) - (368*
     mA2)/mH2 + (45*mC2)/mH2 - (24*mh2)/mH2 + (72*mH2)/mA2 - (610*mH2)/mh2 + (
     586*mH2)/(mh2 - mH2) + (56*mA2)/(-mA2 + mH2) + (144*msntau2)/mh2 - (144*
     msntau2)/mH2 + (8*mh2)/mstau12 - (8*mH2)/mstau12 - (352*mstau12)/mh2 + (
     496*mstau12)/(mh2 - mH2) + (48*mstau12)/mH2 + (8*mh2)/mstau22 - (8*mH2)/
     mstau22 - (352*mstau22)/mh2 + (496*mstau22)/(mh2 - mH2) + (48*mstau22)/mH2
     + (24*mH2)/(mstau12 - mu2) + (24*mH2)/(mstau22 - mu2) + (1000*mu2)/mh2 + (
     56*mu2)/(mh2 - mH2) - (96*mu2)/mH2 + (1344*mu2)/mstau12 + (1344*mstau22*
     mu2)/(mh2*mstau12) + (1344*mstau22*mu2)/(mH2*mstau12) + (24*mh2)/(-mstau12
     + mu2) + (24*mh2)/(-mstau22 + mu2) - (224*mh2*mu2)/(mstau12*mstau22 -
     mstau12*mu2) + (224*mH2*mu2)/(mstau12*mstau22 - mstau12*mu2) - 448*mh2*
     mu2*DeltaInv(msntau2,mh2,mu2) - 448*mh2*mu2*DeltaInv(mstau12,mh2,mu2) -
     448*mh2*mu2*DeltaInv(mstau22,mh2,mu2) + 784*mH2*mu2*DeltaInv(mstau22,mH2,
     mstau12) + 4592*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12) + (224*mH2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (2688*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/mH2 + (1904*mstau24*mu2*DeltaInv(mstau22,
     mH2,mstau12))/mstau12 + (448*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/(mstau12 - mu2) + (336*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12 - mu2) + (448*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(
     mstau22 - mu2) - (224*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12*mstau22 - mstau12*mu2) + 448*mH2*mu2*DeltaInv(mstau22,mH2,mu2) +
     560*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) + 4816*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2) - (224*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /mstau12 + (2688*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mh2 + (2128*
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (448*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (336*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (448*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (224*mh2*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + (35*
     power2(mA2))/power2(mA2 - mh2) + (14*power2(mC2))/power2(mC2 - mh2) + (
     272*mH2*mstau12)/power2(mh2) + (272*mH2*mstau22)/power2(mh2) - (56*mH2*
     mu2)/power2(mh2) - (56*mstau12*mu2)/power2(mh2) + (56*mH2*mstau12*mu2)/((-
     mh2 + mH2)*power2(mh2)) - (56*mstau22*mu2)/power2(mh2) + (56*mH2*mstau22*
     mu2)/((-mh2 + mH2)*power2(mh2)) + (105*power2(mA2))/power2(mh2) + (105*
     power2(mC2))/power2(mh2) + (224*mu2*DeltaInv(msntau2,mh2,mu2)*power2(mh2))
     /msntau2 + (224*mu2*DeltaInv(mstau12,mh2,mu2)*power2(mh2))/mstau12 + (224*
     mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/mstau22 - (560*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (336*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (448*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (448*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) - (140*
     power2(mh2))/power2(mA2) - (1024*mH2*mstau12)/(-(mh2*mH2) + power2(mh2)) -
     (1024*mH2*mstau22)/(-(mh2*mH2) + power2(mh2)) + (256*mH2*mu2)/(-(mh2*mH2)
     + power2(mh2)) + (56*mstau12*mu2)/(-(mh2*mH2) + power2(mh2)) + (56*
     mstau22*mu2)/(-(mh2*mH2) + power2(mh2)) - (35*power2(mA2))/power2(mA2 -
     mH2) - (14*power2(mC2))/power2(mC2 - mH2) + (256*mH2*mstau12)/power2(mh2 -
     mH2) + (256*mH2*mstau22)/power2(mh2 - mH2) - (256*mH2*mu2)/power2(mh2 -
     mH2) - (105*power2(mA2))/power2(mH2) - (105*power2(mC2))/power2(mH2) + (
     84*power2(mh2))/power2(mH2) - (784*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/mstau12 + (336*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))
     /(mstau12 - mu2) + (448*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau22 - mu2) + (448*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/(mstau12*mstau22 - mstau12*mu2) - (224*mu2*DeltaInv(mstau22,mH2,mu2)
     *power2(mH2))/mstau22 + (140*power2(mH2))/power2(mA2) + (329*power2(mH2))/
     power2(mh2) + (384*mstau12*power2(mH2))/((mh2 - mH2)*power2(mh2)) + (384*
     mstau22*power2(mH2))/((mh2 - mH2)*power2(mh2)) + (168*mu2*power2(mH2))/((-
     mh2 + mH2)*power2(mh2)) - (824*power2(mH2))/(-(mh2*mH2) + power2(mh2)) - (
     7*power2(mH2))/power2(mh2 - mH2) - (368*mstau12*power2(mH2))/(mh2*power2(
     mh2 - mH2)) - (368*mstau22*power2(mH2))/(mh2*power2(mh2 - mH2)) + (368*
     mu2*power2(mH2))/(mh2*power2(mh2 - mH2)) - (16*mh2*mu2)/(-(mstau12*mu2) +
     power2(mstau12)) + (16*mH2*mu2)/(-(mstau12*mu2) + power2(mstau12)) - (16*
     mh2*mu2)/(-(mstau22*mu2) + power2(mstau22)) + (16*mH2*mu2)/(-(mstau22*mu2)
     + power2(mstau22)) - (56*mh2*mu2)/power2(mstau12 - mu2) + (56*mH2*mu2)/
     power2(mstau12 - mu2) + (112*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/power2(mstau12 - mu2) - (112*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,
     mh2))/power2(mstau12 - mu2) - (112*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) + (112*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) + (56*mh2*mu2)/
     power2(mstau22 - mu2) - (56*mH2*mu2)/power2(mstau22 - mu2) - (112*power2(
     mu2))/(mh2*msntau2) + (112*power2(mu2))/(mH2*msntau2) - (112*power2(mu2))/
     (mh2*mstau12) + (112*power2(mu2))/(mH2*mstau12) - (112*power2(mu2))/(mh2*
     mstau22) + (112*power2(mu2))/(mH2*mstau22) + (448*power2(mu2))/(mh2*
     msntau2 - mh2*mu2) + (448*power2(mu2))/(mh2*mstau12 - mh2*mu2) + (448*
     power2(mu2))/(mh2*mstau22 - mh2*mu2) - (448*power2(mu2))/(mH2*msntau2 -
     mH2*mu2) - (448*power2(mu2))/(mH2*mstau12 - mH2*mu2) - (448*power2(mu2))/(
     mH2*mstau22 - mH2*mu2) - 784*DeltaInv(msntau2,mh2,mu2)*power2(mu2) - (336*
     mh2*DeltaInv(msntau2,mh2,mu2)*power2(mu2))/msntau2 - 784*DeltaInv(mstau12,
     mh2,mu2)*power2(mu2) - (336*mh2*DeltaInv(mstau12,mh2,mu2)*power2(mu2))/
     mstau12 - 784*DeltaInv(mstau22,mh2,mu2)*power2(mu2) - (336*mh2*DeltaInv(
     mstau22,mh2,mu2)*power2(mu2))/mstau22 + (224*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(-mstau12 + mu2) + (224*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/(-mstau12 + mu2) + 784*DeltaInv(mstau22,mH2,mu2)
     *power2(mu2) + (336*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 + (
     224*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12 - mu2) + (224*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12 - mu2) - (24*
     mh2*power2(mu2))/(mstau12*power2(mstau12 - mu2)) + (24*mH2*power2(mu2))/(
     mstau12*power2(mstau12 - mu2)) + (224*mh2*power2(mu2))/(mstau22*power2(
     mstau12 - mu2)) - (224*mH2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) +
     (448*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau12
     - mu2) + (112*mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(
     mstau12 - mu2) - (448*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mu2))/power2(mstau12 - mu2) - (112*mstau24*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau12 - mu2) + (448*DeltaInv(mstau22,mh2,mu2)*
     power2(mh2)*power2(mu2))/power2(mstau12 - mu2) - (112*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/power2(mstau12 - mu2) + (112*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/power2(mstau12 -
     mu2) - (448*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/power2(
     mstau12 - mu2) + (448*mh2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     448*mH2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (24*mh2*power2(mu2)
     )/(mstau22*power2(mstau22 - mu2)) + (24*mH2*power2(mu2))/(mstau22*power2(
     mstau22 - mu2)) - (448*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/power2(mstau22 - mu2) + (224*mH2*mstau24*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (448*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (224*
     mh2*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (448*DeltaInv(mstau12,mh2,mu2)*power2(mh2)*power2(mu2))/
     power2(mstau22 - mu2) + (448*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/power2(mstau22 - mu2) + (448*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     448*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/power2(mstau22
     - mu2) - (448*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(
     mu2))/(mstau12*power2(mstau22 - mu2)) - (14*power3(mA2))/(mh2*power2(mA2 -
     mh2)) + (14*power3(mA2))/(mH2*power2(mA2 - mH2)) + (7*power3(mC2))/(mh2*
     power2(mC2 - mh2)) - (7*power3(mC2))/(mH2*power2(mC2 - mH2)) + (224*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (112*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 -
     mu2) - (224*DeltaInv(mstau22,mh2,mu2)*power2(mu2)*power3(mh2))/(mstau22*
     power2(mstau12 - mu2)) - (224*DeltaInv(mstau12,mh2,mu2)*power2(mu2)*
     power3(mh2))/(mstau12*power2(mstau22 - mu2)) - (224*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) - (
     224*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (224*power3(mH2))/((mh2 - mH2)*power2(mh2)) + (28*power3(
     mH2))/(mh2*power2(mh2 - mH2)) + (112*mstau12*power3(mH2))/(power2(mh2)*
     power2(mh2 - mH2)) + (112*mstau22*power3(mH2))/(power2(mh2)*power2(mh2 -
     mH2)) - (112*mu2*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (112*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (224*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2)*power3(mH2))/(mstau22*power2(mstau12
     - mu2)) + (224*DeltaInv(mstau22,mH2,mstau12)*power2(mu2)*power3(mH2))/(
     mstau12*power2(mstau22 - mu2)) - (112*mu2*DeltaInv(mH2,msntau2,mu2)*(-7*
     mH2*msntau2*mu2 - (4*msntau2 + 3*mu2)*power2(mH2) + (-msntau2 + mu2)*
     power2(mu2) + 2*power3(mH2)))/(mH2*msntau2) - (112*mu2*DeltaInv(mH2,
     mstau12,mu2)*(-2*mstau22*mu2 + mu2*(-mH2 + mu2) + power2(mstau22))*(-7*
     mH2*mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 + mu2)*
     power2(mu2) + 2*power3(mH2)))/(mH2*mstau12*power2(mstau22 - mu2)) - (112*
     DeltaInv(msntau2,mh2,mu2)*power3(mu2))/mh2 - (112*DeltaInv(mstau12,mh2,
     mu2)*power3(mu2))/mh2 - (112*DeltaInv(mstau22,mh2,mu2)*power3(mu2))/mh2 +
     (112*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/(mstau22 - mu2) + (112*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mH2 + (112*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/(-mstau22 + mu2) + (784*mh2*DeltaInv(mstau22,
     mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (784*mH2*DeltaInv(mstau22,
     mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (336*DeltaInv(mstau22,mh2,
     mu2)*power2(mh2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (336*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power3(mu2))/(mstau22*power2(mstau12
     - mu2)) + (784*mh2*DeltaInv(mstau12,mh2,mu2)*power3(mu2))/power2(mstau22 -
     mu2) - (336*mH2*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22
     - mu2) - (224*mstau22*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(
     mstau22 - mu2) - (224*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power3(
     mu2))/(mstau12*power2(mstau22 - mu2)) + (112*mstau24*DeltaInv(mstau22,mH2,
     mstau12)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (336*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (224*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (224*
     mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (112*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/
     (mstau12*power2(mstau22 - mu2)) + (336*DeltaInv(mstau12,mh2,mu2)*power2(
     mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (112*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     112*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power3(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (21*power4(mA2))/(power2(mA2 - mh2)*power2(mh2))
     + (21*power4(mA2))/(power2(mA2 - mH2)*power2(mH2)) - (21*power4(mC2))/(
     power2(mC2 - mh2)*power2(mh2)) + (21*power4(mC2))/(power2(mC2 - mH2)*
     power2(mH2)) - (21*power4(mH2))/(power2(mh2)*power2(mh2 - mH2)) + (112*
     DeltaInv(msntau2,mh2,mu2)*power4(mu2))/(mh2*msntau2) + (112*DeltaInv(
     mstau12,mh2,mu2)*power4(mu2))/(mh2*mstau12) + (112*DeltaInv(mstau22,mh2,
     mu2)*power4(mu2))/(mh2*mstau22) - (112*DeltaInv(mstau22,mH2,mu2)*power4(
     mu2))/(mH2*mstau22) + (112*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/power2(
     mstau12 - mu2) - (112*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/power2(
     mstau12 - mu2) + (112*DeltaInv(mstau12,mh2,mu2)*power4(mu2))/power2(
     mstau22 - mu2) + (112*DeltaInv(mstau22,mH2,mstau12)*power4(mu2))/power2(
     mstau22 - mu2) - (112*DeltaInv(mstau22,mstau12,mh2)*power4(mu2))/power2(
     mstau22 - mu2) - (112*DeltaInv(mstau22,mh2,mu2)*power5(mu2))/(mstau22*
     power2(mstau12 - mu2)) + (112*DeltaInv(mstau22,mH2,mu2)*power5(mu2))/(
     mstau22*power2(mstau12 - mu2)) - (112*DeltaInv(mstau12,mh2,mu2)*power5(
     mu2))/(mstau12*power2(mstau22 - mu2)) - (1344*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/(mH2*mstau12) - (1344*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/(mh2*mstau12) - (112*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (112*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/128. + (3*ca*
     mu2*sb*log(mh2/Q2)*log(mstau22/Q2)*power3(sa)*(-mstau22 + DeltaInv(
     mstau22,mstau12,mh2)*(-3*mh2*mstau12*mstau22 - 2*mh2*mstau24 - mstau12*
     mstau24 + mstau22*power2(mh2) + power6(mstau2))))/(2.*cb*mh2*mstau12) + (
     3*mu2*log(mh2/Q2)*log(mstau22/Q2)*power2(sb)*power4(sa)*(-mstau22 +
     DeltaInv(mstau22,mstau12,mh2)*(-3*mh2*mstau12*mstau22 - 2*mh2*mstau24 -
     mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2))))/(4.*mh2*mstau12*
     power2(cb)) - (3*log(mh2/Q2)*log(mstau22/Q2)*power4(sa)*(mstau22*(2*mh2*
     mstau12 - mh2*mu2 + mH2*mu2) + (mh2 - mH2)*mu2*DeltaInv(mstau22,mstau12,
     mh2)*(-3*mh2*mstau12*mstau22 - 2*mh2*mstau24 - mstau12*mstau24 + mstau22*
     power2(mh2) + power6(mstau2))))/(4.*mh2*(mh2 - mH2)*mstau12) + (9*ca*mu2*
     sb*log(mstau22/Q2)*power3(sa)*(-((mh2 + mH2)*mstau22) + mH2*DeltaInv(
     mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*mh2*mstau24 - 3*mstau12*
     mstau24 + mstau22*power2(mh2) + power6(mstau2)) + mh2*DeltaInv(mstau22,
     mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24 - 3*mstau12*mstau24 +
     mstau22*power2(mH2) + power6(mstau2))))/(2.*cb*mh2*mH2*mstau12) - (3*ca*
     mu2*sb*power2(log(mstau22/Q2))*power3(sa)*(-((mh2 + mH2)*mstau22) + mH2*
     DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*mh2*mstau24 - 3*
     mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)) + mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24 - 3*mstau12*
     mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(4.*cb*mh2*mH2*mstau12)
     + (9*mu2*log(mstau22/Q2)*power2(sb)*power4(sa)*(-((mh2 + mH2)*mstau22) +
     mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*mh2*mstau24
     - 3*mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)) + mh2*
     DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24 - 3*
     mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(4.*mh2*mH2*
     mstau12*power2(cb)) + (3*mu2*power2(log(mstau22/Q2))*power4(sa)*(-((mh2 +
     mH2)*mstau22) + mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22)
     - 2*mh2*mstau24 - 3*mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)
     ) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*
     mstau24 - 3*mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(8.*
     mh2*mH2*mstau12) - (3*mu2*power2(sb)*power2(log(mstau22/Q2))*power4(sa)*(-
     ((mh2 + mH2)*mstau22) + mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*
     mstau22) - 2*mh2*mstau24 - 3*mstau12*mstau24 + mstau22*power2(mh2) +
     power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*
     mstau22) - 2*mH2*mstau24 - 3*mstau12*mstau24 + mstau22*power2(mH2) +
     power6(mstau2))))/(8.*mh2*mH2*mstau12*power2(cb)) - (3*log(mstau22/Q2)*
     power4(sa)*(-((mh2 + mH2)*mstau22*(mstau12 + 3*mu2)) + 3*mH2*mu2*DeltaInv(
     mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*mh2*mstau24 - 3*mstau12*
     mstau24 + mstau22*power2(mh2) + power6(mstau2)) + 3*mh2*mu2*DeltaInv(
     mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24 - 3*mstau12*
     mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(4.*mh2*mH2*mstau12) + (
     3*ca*mu2*sb*log(mH2/Q2)*log(mstau22/Q2)*power3(sa)*(-mstau22 + DeltaInv(
     mstau22,mH2,mstau12)*(-3*mH2*mstau12*mstau22 - 2*mH2*mstau24 - mstau12*
     mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(2.*cb*mH2*mstau12) + (
     3*mu2*log(mH2/Q2)*log(mstau22/Q2)*power2(sb)*power4(sa)*(-mstau22 +
     DeltaInv(mstau22,mH2,mstau12)*(-3*mH2*mstau12*mstau22 - 2*mH2*mstau24 -
     mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(4.*mH2*mstau12*
     power2(cb)) + (3*log(mH2/Q2)*log(mstau22/Q2)*power4(sa)*(mstau22*(2*mH2*
     mstau12 + mh2*mu2 - mH2*mu2) - (mh2 - mH2)*mu2*DeltaInv(mstau22,mH2,
     mstau12)*(-3*mH2*mstau12*mstau22 - 2*mH2*mstau24 - mstau12*mstau24 +
     mstau22*power2(mH2) + power6(mstau2))))/(4.*(mh2 - mH2)*mH2*mstau12) - (3*
     ca*mu2*sb*log(mstau12/Q2)*log(mstau22/Q2)*power3(sa)*(-((mh2 + mH2)*
     mstau22) + mH2*DeltaInv(mstau22,mstau12,mh2)*(-2*mstau12*mstau24 - 2*mh2*(
     mstau12*mstau22 + mstau24) + mstau22*power2(mh2) + power6(mstau2)) + mh2*
     DeltaInv(mstau22,mH2,mstau12)*(-2*mstau12*mstau24 - 2*mH2*(mstau12*mstau22
     + mstau24) + mstau22*power2(mH2) + power6(mstau2))))/(cb*mh2*mH2*mstau12)
     + (3*mu2*log(mstau12/Q2)*log(mstau22/Q2)*power4(sa)*(-((mh2 + mH2)*
     mstau22) + mH2*DeltaInv(mstau22,mstau12,mh2)*(-2*mstau12*mstau24 - 2*mh2*(
     mstau12*mstau22 + mstau24) + mstau22*power2(mh2) + power6(mstau2)) + mh2*
     DeltaInv(mstau22,mH2,mstau12)*(-2*mstau12*mstau24 - 2*mH2*(mstau12*mstau22
     + mstau24) + mstau22*power2(mH2) + power6(mstau2))))/(2.*mh2*mH2*mstau12)
     - (3*mu2*log(mstau12/Q2)*log(mstau22/Q2)*power2(sb)*power4(sa)*(-((mh2 +
     mH2)*mstau22) + mH2*DeltaInv(mstau22,mstau12,mh2)*(-2*mstau12*mstau24 - 2*
     mh2*(mstau12*mstau22 + mstau24) + mstau22*power2(mh2) + power6(mstau2)) +
     mh2*DeltaInv(mstau22,mH2,mstau12)*(-2*mstau12*mstau24 - 2*mH2*(mstau12*
     mstau22 + mstau24) + mstau22*power2(mH2) + power6(mstau2))))/(2.*mh2*mH2*
     mstau12*power2(cb)) + (3*log(mh2/Q2)*log(mstau12/Q2)*power4(sa)*(mH2*(
     mstau12 + mstau22)*mu2 - mu2*power2(mh2) - mh2*(mstau12*mu2 + (-mH2 +
     mstau22)*mu2 + 2*power2(mstau12)) + (mh2 - mH2)*mu2*DeltaInv(mstau22,
     mstau12,mh2)*(-(mstau12*mstau24) - mh2*(6*mstau12*mstau22 + mstau24) - (
     mstau12 + mstau22)*power2(mh2) + power3(mh2) + power6(mstau2))))/(4.*mh2*(
     mh2 - mH2)*mstau12) + (3*log(mH2/Q2)*log(mstau12/Q2)*power4(sa)*(-(mh2*(
     mstau12 + mstau22)*mu2) + mu2*power2(mH2) + mH2*(mstau12*mu2 + (-mh2 +
     mstau22)*mu2 + 2*power2(mstau12)) + (mh2 - mH2)*mu2*DeltaInv(mstau22,mH2,
     mstau12)*(-(mstau12*mstau24) - mH2*(6*mstau12*mstau22 + mstau24) - (
     mstau12 + mstau22)*power2(mH2) + power3(mH2) + power6(mstau2))))/(4.*(mh2
     - mH2)*mH2*mstau12) - (Fin20(mstau22,msntau2,Q2)*(-((mstau24 - 7*mstau22*
     mu2)*power2(msntau2)) - (mstau22 + 3*mu2)*power3(msntau2) + power4(
     msntau2) + msntau2*(2*mstau22*mstau24 + mstau24*mu2 - 6*mu2*power2(
     mstau22) - power6(mstau2)) + mu2*(-2*mstau22*mstau24 + 2*power3(mstau22) +
     power6(mstau2))))/(8.*power2(msntau2 - mstau22)*power3(msntau2 - mu2)) - (
     ca*mu2*sb*power2(Pi)*power3(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 -
     2*mh2*mstau22 - 2*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*mh2*
     mstau12*mstau22 - 3*mh2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2)
     + power3(mh2) + 2*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-7*
     mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(
     mH2) + power3(mH2) + 2*power6(mstau2))))/(4.*cb*mh2*mH2*mstau12) - (mu2*
     power2(Pi)*power2(sb)*power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 -
     2*mh2*mstau22 - 2*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*mh2*
     mstau12*mstau22 - 3*mh2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2)
     + power3(mh2) + 2*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-7*
     mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(
     mH2) + power3(mH2) + 2*power6(mstau2))))/(8.*mh2*mH2*mstau12*power2(cb)) -
     (3*ca*mu2*sb*power3(sa)*(-14*mh2*mH2 - 5*mh2*mstau12 - 5*mH2*mstau12 - 14*
     mh2*mstau22 - 14*mH2*mstau22 + 7*mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*
     mh2*mstau12*mstau22 - 3*mh2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(
     mh2) + power3(mh2) + 2*power6(mstau2)) + 7*mh2*DeltaInv(mstau22,mH2,
     mstau12)*(-7*mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 -
     mstau12*power2(mH2) + power3(mH2) + 2*power6(mstau2))))/(2.*cb*mh2*mH2*
     mstau12) - (3*mu2*power2(sb)*power4(sa)*(-14*mh2*mH2 - 5*mh2*mstau12 - 5*
     mH2*mstau12 - 14*mh2*mstau22 - 14*mH2*mstau22 + 7*mH2*DeltaInv(mstau22,
     mstau12,mh2)*(-7*mh2*mstau12*mstau22 - 3*mh2*mstau24 - 4*mstau12*mstau24 -
     mstau12*power2(mh2) + power3(mh2) + 2*power6(mstau2)) + 7*mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-7*mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*
     mstau24 - mstau12*power2(mH2) + power3(mH2) + 2*power6(mstau2))))/(4.*mh2*
     mH2*mstau12*power2(cb)) - (3*power4(sa)*(8*mH2*mstau12*mstau22*power2(mh2)
     + 40*mH2*mstau12*mu2*power2(mh2) + 112*mH2*mstau22*mu2*power2(mh2) + 8*
     mh2*mstau12*mstau22*power2(mH2) + 40*mh2*mstau12*mu2*power2(mH2) + 112*
     mh2*mstau22*mu2*power2(mH2) + 22*mstau12*power2(mh2)*power2(mH2) + 112*
     mu2*power2(mh2)*power2(mH2) + 8*mH2*power2(mh2)*power2(mstau12) + 8*mh2*
     power2(mH2)*power2(mstau12) - 2*mH2*mstau12*power3(mh2) - 2*mh2*mstau12*
     power3(mH2) + 7*mstau12*power4(mh2) + 7*mstau12*power4(mH2) - 56*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mH2)*(-7*mh2*mstau12*mstau22 - 3*mh2*
     mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2) + power3(mh2) + 2*
     power6(mstau2)) - 56*mH2*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mh2)*(-
     7*mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*
     power2(mH2) + power3(mH2) + 2*power6(mstau2))))/(32.*mstau12*power2(mh2)*
     power2(mH2)) - (power2(Pi)*power4(sa)*(8*mH2*mstau12*mu2*power2(mh2) + 16*
     mH2*mstau22*mu2*power2(mh2) + 8*mh2*mstau12*mu2*power2(mH2) + 16*mh2*
     mstau22*mu2*power2(mH2) + 2*mstau12*power2(mh2)*power2(mH2) + 16*mu2*
     power2(mh2)*power2(mH2) - 4*mH2*mstau12*power3(mh2) - 4*mh2*mstau12*
     power3(mH2) + 3*mstau12*power4(mh2) + 3*mstau12*power4(mH2) - 8*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mH2)*(-7*mh2*mstau12*mstau22 - 3*mh2*
     mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2) + power3(mh2) + 2*
     power6(mstau2)) - 8*mH2*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mh2)*(-7*
     mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(
     mH2) + power3(mH2) + 2*power6(mstau2))))/(64.*mstau12*power2(mh2)*power2(
     mH2)) + (9*ca*mu2*sb*log(mstau12/Q2)*power3(sa)*(-2*mh2*mH2 - mh2*mstau12
     - mH2*mstau12 - 3*mh2*mstau22 - 3*mH2*mstau22 + mH2*DeltaInv(mstau22,
     mstau12,mh2)*(-5*mstau12*mstau24 - 5*mh2*(2*mstau12*mstau22 + mstau24) + (
     -mstau12 + mstau22)*power2(mh2) + power3(mh2) + 3*power6(mstau2)) + mh2*
     DeltaInv(mstau22,mH2,mstau12)*(-5*mstau12*mstau24 - 5*mH2*(2*mstau12*
     mstau22 + mstau24) + (-mstau12 + mstau22)*power2(mH2) + power3(mH2) + 3*
     power6(mstau2))))/(2.*cb*mh2*mH2*mstau12) - (3*ca*mu2*sb*power2(log(
     mstau12/Q2))*power3(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 - 3*mh2*
     mstau22 - 3*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-5*mstau12*
     mstau24 - 5*mh2*(2*mstau12*mstau22 + mstau24) + (-mstau12 + mstau22)*
     power2(mh2) + power3(mh2) + 3*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,
     mstau12)*(-5*mstau12*mstau24 - 5*mH2*(2*mstau12*mstau22 + mstau24) + (-
     mstau12 + mstau22)*power2(mH2) + power3(mH2) + 3*power6(mstau2))))/(4.*cb*
     mh2*mH2*mstau12) + (9*mu2*log(mstau12/Q2)*power2(sb)*power4(sa)*(-2*mh2*
     mH2 - mh2*mstau12 - mH2*mstau12 - 3*mh2*mstau22 - 3*mH2*mstau22 + mH2*
     DeltaInv(mstau22,mstau12,mh2)*(-5*mstau12*mstau24 - 5*mh2*(2*mstau12*
     mstau22 + mstau24) + (-mstau12 + mstau22)*power2(mh2) + power3(mh2) + 3*
     power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-5*mstau12*mstau24 -
     5*mH2*(2*mstau12*mstau22 + mstau24) + (-mstau12 + mstau22)*power2(mH2) +
     power3(mH2) + 3*power6(mstau2))))/(4.*mh2*mH2*mstau12*power2(cb)) + (3*
     mu2*power2(log(mstau12/Q2))*power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*
     mstau12 - 3*mh2*mstau22 - 3*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,
     mh2)*(-5*mstau12*mstau24 - 5*mh2*(2*mstau12*mstau22 + mstau24) + (-mstau12
     + mstau22)*power2(mh2) + power3(mh2) + 3*power6(mstau2)) + mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-5*mstau12*mstau24 - 5*mH2*(2*mstau12*mstau22 +
     mstau24) + (-mstau12 + mstau22)*power2(mH2) + power3(mH2) + 3*power6(
     mstau2))))/(8.*mh2*mH2*mstau12) - (3*mu2*power2(sb)*power2(log(mstau12/Q2)
     )*power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 - 3*mh2*mstau22 - 3*
     mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-5*mstau12*mstau24 - 5*
     mh2*(2*mstau12*mstau22 + mstau24) + (-mstau12 + mstau22)*power2(mh2) +
     power3(mh2) + 3*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-5*
     mstau12*mstau24 - 5*mH2*(2*mstau12*mstau22 + mstau24) + (-mstau12 +
     mstau22)*power2(mH2) + power3(mH2) + 3*power6(mstau2))))/(8.*mh2*mH2*
     mstau12*power2(cb)) - (3*log(mstau12/Q2)*power4(sa)*(-6*mh2*mH2*mu2 - 3*
     mh2*mstau12*mu2 - 3*mH2*mstau12*mu2 - 9*mh2*mstau22*mu2 - 9*mH2*mstau22*
     mu2 - mh2*power2(mstau12) - mH2*power2(mstau12) + 3*mH2*mu2*DeltaInv(
     mstau22,mstau12,mh2)*(-5*mstau12*mstau24 - 5*mh2*(2*mstau12*mstau22 +
     mstau24) + (-mstau12 + mstau22)*power2(mh2) + power3(mh2) + 3*power6(
     mstau2)) + 3*mh2*mu2*DeltaInv(mstau22,mH2,mstau12)*(-5*mstau12*mstau24 -
     5*mH2*(2*mstau12*mstau22 + mstau24) + (-mstau12 + mstau22)*power2(mH2) +
     power3(mH2) + 3*power6(mstau2))))/(4.*mh2*mH2*mstau12) + (Fin20(mstau22,
     mu2,Q2)*(-8/mA2 + 16/mH2 + (5*(mstau22 - mu2))/power2(mA2) - (3*(mstau22 -
     mu2))/power2(mH2) + 2*(1/(msntau2 - mu2) + 1/(mstau12 - mu2) + 3/(-mstau22
     + mu2) + (4*mstau22)/((mstau12 - mu2)*(-mstau22 + mu2)) + (4*mu2)/((
     mstau12 - mu2)*(-mstau22 + mu2)) - (4*mu2)/(mA2*mstau22 - mA2*mu2) + (4*
     mu2)/(mH2*mstau22 - mH2*mu2) + (mstau22 - mu2)/power2(mC2) - mu2/power2(
     msntau2 - mu2) - mu2/power2(mstau12 - mu2) + mstau24/((-msntau2 + mu2)*
     power2(mstau22 - mu2)) - (2*mstau22*mu2)/((-msntau2 + mu2)*power2(mstau22
     - mu2)) + mstau24/((-mstau12 + mu2)*power2(mstau22 - mu2)) - (2*mstau22*
     mu2)/((-mstau12 + mu2)*power2(mstau22 - mu2)) - (4*mstau24*mu2)/(power2(
     msntau2 - mu2)*power2(mstau22 - mu2)) - (4*mstau24*mu2)/(power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + power2(mu2)/((-msntau2 + mu2)*power2(mstau22
     - mu2)) + power2(mu2)/((-mstau12 + mu2)*power2(mstau22 - mu2)) + (5*
     mstau22*power2(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) + (5*
     mstau22*power2(mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (2*
     mstau22*mu2)/power3(msntau2 - mu2) + (2*mstau22*mu2)/power3(mstau12 - mu2)
     - (2*power3(mu2))/(power2(msntau2 - mu2)*power2(mstau22 - mu2)) - (2*
     power3(mu2))/(power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (2*power2(
     mu2))/power3(-msntau2 + mu2) + (2*power2(mu2))/power3(-mstau12 + mu2) + ((
     1/power2(msntau2 - mu2) + 1/power2(mstau12 - mu2))*power6(mstau2))/power2(
     mstau22 - mu2))))/16. - (Fin20(mstau22,mstau12,Q2)*((7*mu2*power2(mstau22)
     + 9*mstau22*power2(mu2) + 4*power3(mstau22) + 10*power3(mu2))*power4(
     mstau12) + power3(mstau12)*(power2(mstau22)*(mstau24 - 2*power2(mu2)) -
     mstau22*mu2*(mstau24 + 24*power2(mu2)) - 15*mu2*power3(mstau22) + 3*
     power4(mstau22) - 2*power4(mu2)) - (5*power2(mstau22) + 7*power2(mu2))*
     power5(mstau12) + power2(mstau12)*(-((mstau24 + 2*power2(mu2))*power3(
     mstau22)) + mstau24*power3(mu2) + 27*power2(mstau22)*power3(mu2) + 7*mu2*
     power4(mstau22) + 2*mstau22*power4(mu2) - 4*power5(mstau22)) + (mstau22 +
     mu2)*power6(mstau12) + mstau12*(-2*mstau22*mstau24*power3(mu2) - power3(
     mstau22)*(5*mstau24*mu2 + 23*power3(mu2)) + 2*(mstau24 + 3*power2(mu2))*
     power4(mstau22) - mstau24*power4(mu2) + 3*power2(mstau22)*(2*mstau24*
     power2(mu2) + power4(mu2)) + 2*mu2*power5(mstau22) - power3(mstau22 - mu2)
     *power6(mstau2)) + mu2*(mstau22*(-7*mstau22*mstau24*power2(mu2) - 2*(
     mstau24 - 7*power2(mu2))*power3(mstau22) + power2(mstau22)*(6*mstau24*mu2
     - 4*power3(mu2)) + 3*mstau24*power3(mu2) - 10*mu2*power4(mstau22) + 2*
     power5(mstau22)) + power3(mstau22 - mu2)*power6(mstau2))))/(8.*power2(
     mstau12 - mstau22)*power3(mstau12 - mu2)*power3(mstau22 - mu2)) + (mu2*
     log(mH2/Q2)*log(mstau12/Q2)*power2(sa)*power2(sb)*(-12/mH2 - 12/mstau12 +
     4/(mstau12 - mstau22) - (12*mstau22)/(mH2*mstau12) + (4*mu2)/((mstau12 -
     mstau22)*(mstau12 - mu2)) - (2*mH2)/(mstau12*mstau22 - mstau12*mu2) - mH2/
     power2(mstau12 - mu2) + (4*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (2*
     mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mH2*(mstau12
     + mu2) + power2(mH2)))/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,
     mH2,mstau12)*((-12*(6*mstau12*mstau22 + mstau24))/mstau12 + power2(mH2)*(
     3/(-mstau12 + mu2) + 4/(-mstau22 + mu2) - (4*mstau22)/(mstau12*mstau22 -
     mstau12*mu2) - (2*mstau22)/power2(mstau12 - mu2) - mu2/power2(mstau12 -
     mu2) + (12 + (4*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 + (4*mu2)/
     power2(mstau22 - mu2)) + mH2*(-10 + (2*mu2)/(mstau12 - mu2) + mstau22/(-
     mstau12 + mu2) + (4*mstau22)/(-mstau22 + mu2) + (2*mstau24)/(mstau12*
     mstau22 - mstau12*mu2) + mstau24/power2(mstau12 - mu2) - (3*mstau22*mu2)/
     power2(mstau12 - mu2) + (-12*mstau22 - (2*mstau24*mu2)/power2(mstau22 -
     mu2))/mstau12 + (4*mstau22*mu2)/power2(mstau22 - mu2)) + (2/(mstau12*
     mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2) - (2*mu2)/(mstau12*
     power2(mstau22 - mu2)))*power3(mH2) + (12*(-(mstau12*mstau24) + power6(
     mstau2)))/(mH2*mstau12))))/(8.*power2(cb)) - (mu2*log(mH2/Q2)*log(mstau22/
     Q2)*power2(sa)*power2(sb)*(4/(mstau12 - mstau22) - (12*mstau22)/(mH2*
     mstau12) + (4*mu2)/((mstau12 - mstau22)*(mstau22 - mu2)) + mH2/(mstau12*(-
     mstau22 + mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mH2*
     mu2*DeltaInv(mstau22,mH2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mH2*(mstau22 +
     mu2) + power2(mH2)))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2)/(mstau12*
     power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(-36*mstau22 - (24*
     mstau24)/mstau12 - ((mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mH2))/(
     mstau12*power2(mstau22 - mu2)) + mH2*(-1 + mstau24/(mstau12*mstau22 -
     mstau12*mu2) + 2*mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2) - mu2/
     power2(mstau12 - mu2) + mu2/power2(mstau22 - mu2)) + (12*mstau22 - (
     mstau24*mu2)/power2(mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 -
     mu2)) + ((mstau22 - 2*mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) +
     (12*(-(mstau12*mstau24) + power6(mstau2)))/(mH2*mstau12))))/(8.*power2(cb)
     ) + (mu2*log(mC2/Q2)*log(msntau2/Q2)*power2(sb)*(1/mC2 + 1/msntau2 +
     mstau22/(mC2*msntau2) + (2*mC2)/(msntau2*mstau22 - msntau2*mu2) + mC2/
     power2(msntau2 - mu2) - (4*mC2*mu2)/(msntau2*power2(mstau22 - mu2)) + (2*
     mC2*mu2*DeltaInv(mC2,msntau2,mu2)*(mu2*(-2*msntau2 + mu2) - 2*mC2*(msntau2
     + mu2) + power2(mC2)))/(msntau2*power2(mstau22 - mu2)) + DeltaInv(mstau22,
     mC2,msntau2)*(6*mstau22 + mstau24/msntau2 + mC2*(-1 + mstau22/(msntau2 -
     mu2) + (4*mstau22)/(mstau22 - mu2) + (2*mu2)/(-msntau2 + mu2) - (2*
     mstau24)/(msntau2*mstau22 - msntau2*mu2) - mstau24/power2(msntau2 - mu2) +
     (3*mstau22*mu2)/power2(msntau2 - mu2) + (mstau22 + (2*mstau24*mu2)/power2(
     mstau22 - mu2))/msntau2 - (4*mstau22*mu2)/power2(mstau22 - mu2)) + power2(
     mC2)*(3/(msntau2 - mu2) + 4/(mstau22 - mu2) + (4*mstau22)/(msntau2*mstau22
     - msntau2*mu2) + (2*mstau22)/power2(msntau2 - mu2) + mu2/power2(msntau2 -
     mu2) - (4*mu2)/power2(mstau22 - mu2) - power2(mstau22 + mu2)/(msntau2*
     power2(mstau22 - mu2))) + (-2/(msntau2*mstau22 - msntau2*mu2) - 1/power2(
     msntau2 - mu2) + (2*mu2)/(msntau2*power2(mstau22 - mu2)))*power3(mC2) + (
     mstau24 - power6(mstau2)/msntau2)/mC2)))/(4.*power2(cb)) - (mu2*log(mC2/
     Q2)*log(mstau22/Q2)*power2(sb)*(mstau22/(mC2*msntau2) + mC2/(msntau2*
     mstau22 - msntau2*mu2) + (2*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) - (2*
     mC2*mu2*DeltaInv(mstau22,mC2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mC2*(mstau22
     + mu2) + power2(mC2)))/(mstau22*power2(msntau2 - mu2)) - (mC2*mu2)/(
     msntau2*power2(mstau22 - mu2)) + DeltaInv(mstau22,mC2,msntau2)*(3*mstau22
     + (2*mstau24)/msntau2 + ((msntau2 + 2*mstau22)*(mstau22 - 2*mu2)*power2(
     mC2))/(msntau2*power2(mstau22 - mu2)) + mC2*(1 + (2*mstau22)/(mstau22 -
     mu2) + (2*mstau22)/(-msntau2 + mu2) + (2*mstau22*mu2)/power2(msntau2 -
     mu2) - (mstau22 + (mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/
     msntau2 - (2*mstau22*mu2)/power2(mstau22 - mu2) - power2(mu2)/power2(
     mstau22 - mu2)) - ((mstau22 - 2*mu2)*power3(mC2))/(msntau2*power2(mstau22
     - mu2)) + (mstau24 - power6(mstau2)/msntau2)/mC2)))/(4.*power2(cb)) - (
     mu2*power2(sa)*power2(sb)*power2(log(mstau22/Q2))*((12*mstau22)/(mH2*
     mstau12) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + mstau22*DeltaInv(mstau22,
     mstau12,mh2) - (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (
     mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))
     /(-mstau12 + mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 +
     (3*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*mH2*mu2)/(mstau22*power2(
     mstau12 - mu2)) - (2*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) - (mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) - (mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) + (mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/
     power2(mstau12 - mu2) - (5*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/
     power2(mstau12 - mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-
     mstau22 + mu2) - (10*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(
     mstau12 - mu2) - (5*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (3*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mh2,mu2)*(mh2*(10*mstau22 - mu2)*
     mu2 + 5*(mstau22 + mu2)*power2(mh2) + (mstau22 - mu2)*power2(mu2) - 3*
     power3(mh2)))/(mstau22*power2(mstau12 - mu2)) + (3*mu2*DeltaInv(mstau22,
     mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) - (DeltaInv(mstau22,
     mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (mH2*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,mH2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2) + DeltaInv(mstau22,mH2,
     mstau12)*((23*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (mstau24*
     mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (mstau24*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(11 + (2*mu2)/(-
     mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 + (3*
     mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (mstau22*
     mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*(-5 - power2(mu2)/power2(mstau22 - mu2)))/mstau12) + power2(mH2)*(
     -(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22 - mu2)
     )/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/power2(
     mstau12 - mu2) + (36*mstau24 - (12*power6(mstau2))/mstau12)/mH2)))/(16.*
     power2(cb)) + (mu2*log(mstau22/Q2)*power2(sa)*power2(sb)*((36*mstau22)/(
     mH2*mstau12) - 9*mh2*DeltaInv(mstau22,mstau12,mh2) + 3*mstau22*DeltaInv(
     mstau22,mstau12,mh2) - (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (9*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(-mstau12 + mu2) - (2*mstau22)/power2(mh2) + (2*mH2*
     mstau22)/((-mh2 + mH2)*power2(mh2)) + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/mstau12 + (2*mstau22)/(-(mh2*mH2) + power2(mh2)) + (9*mh2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (9*mH2*mu2)/(mstau22*power2(mstau12
     - mu2)) - (6*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) - (3*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) - (3*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2)
     + (3*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 -
     mu2) - (15*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 -
     mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) - (
     30*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (15*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (9*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22
     - mu2) + (6*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(
     mstau22 - mu2) + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*mu2*DeltaInv(mstau22,mh2,mu2)*(mh2*mu2*(-10*mstau22 + mu2) - 5*(mstau22
     + mu2)*power2(mh2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mh2)))/(
     mstau22*power2(mstau12 - mu2)) + (9*mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) + (3*mH2*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mH2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2) + 3*DeltaInv(mstau22,
     mH2,mstau12)*((23*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (
     mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (
     mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(11 + (2*
     mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 +
     (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (
     mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2)
     + (2*mstau22*(-5 - power2(mu2)/power2(mstau22 - mu2)))/mstau12) + power2(
     mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22
     - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/
     power2(mstau12 - mu2) + (36*mstau24 - (12*power6(mstau2))/mstau12)/mH2)))/
     (8.*power2(cb)) + (log(mh2/Q2)*log(mstau12/Q2)*power2(sa)*(-4 + (3*
     mstau12)/mh2 + (24*mstau12)/(mh2 - mH2) + (12*mu2)/mh2 + (8*mu2)/mstau12 +
     (12*mstau22*mu2)/(mh2*mstau12) + (8*mu2)/(-mstau12 + mu2) - (4*mh2*mu2)/(
     mstau12*mstau22 - mstau12*mu2) - (2*mh2*mu2)/power2(mstau12 - mu2) + (4*
     mu2*DeltaInv(mstau12,mh2,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mh2*(mstau12 +
     mu2) + power2(mh2))*(-2*mstau22*mu2 + mu2*(-mh2 + mu2) + power2(mstau22)))
     /(mstau12*power2(mstau22 - mu2)) - (4*power2(mu2))/power2(mstau12 - mu2) +
     (8*mh2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (4*mh2*power2(mu2))/
     power3(mstau12 - mu2) + 2*mu2*DeltaInv(mstau22,mstau12,mh2)*((6*(6*
     mstau12*mstau22 + mstau24))/mstau12 + power2(mh2)*(3/(-mstau12 + mu2) + 4/
     (-mstau22 + mu2) - (4*mstau22)/(mstau12*mstau22 - mstau12*mu2) - (2*
     mstau22)/power2(mstau12 - mu2) - mu2/power2(mstau12 - mu2) + (-6 + (4*
     mstau22*mu2)/power2(mstau22 - mu2))/mstau12 + (4*mu2)/power2(mstau22 -
     mu2)) + mh2*(8 + (2*mu2)/(mstau12 - mu2) + mstau22/(-mstau12 + mu2) + (4*
     mstau22)/(-mstau22 + mu2) + (2*mstau24)/(mstau12*mstau22 - mstau12*mu2) +
     mstau24/power2(mstau12 - mu2) - (3*mstau22*mu2)/power2(mstau12 - mu2) + (
     6*mstau22 - (2*mstau24*mu2)/power2(mstau22 - mu2))/mstau12 + (4*mstau22*
     mu2)/power2(mstau22 - mu2)) + (2/(mstau12*mstau22 - mstau12*mu2) + 1/
     power2(mstau12 - mu2) - (2*mu2)/(mstau12*power2(mstau22 - mu2)))*power3(
     mh2) + (6*mstau24 - (6*power6(mstau2))/mstau12)/mh2)))/16. - (log(mh2/Q2)*
     log(mstau22/Q2)*power2(sa)*(4 - (3*mstau22)/mh2 + (24*mstau22)/(-mh2 +
     mH2) + (4*mu2)/mstau22 + (12*mstau22*mu2)/(mh2*mstau12) + (8*mu2)/(mstau22
     - mu2) - (2*mh2*mu2)/(mstau12*mstau22 - mstau12*mu2) - (4*mu2*DeltaInv(
     mstau22,mh2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mh2*(mstau22 + mu2) + power2(
     mh2))*(-2*mstau12*mu2 + mu2*(-mh2 + mu2) + power2(mstau12)))/(mstau22*
     power2(mstau12 - mu2)) - (4*mh2*power2(mu2))/(mstau22*power2(mstau12 -
     mu2)) + (4*power2(mu2))/power2(mstau22 - mu2) + (2*mh2*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (4*mh2*power2(mu2))/power3(-mstau22 +
     mu2) + 2*mu2*DeltaInv(mstau22,mstau12,mh2)*(18*mstau22 + (12*mstau24)/
     mstau12 - ((mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mh2))/(mstau12*
     power2(mstau22 - mu2)) + mh2*(-1 + mstau24/(mstau12*mstau22 - mstau12*mu2)
     + 2*mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2) - mu2/power2(mstau12 -
     mu2) + mu2/power2(mstau22 - mu2)) - (6*mstau22 + (mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 - mu2)) + ((mstau22 -
     2*mu2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (6*mstau24 - (6*
     power6(mstau2))/mstau12)/mh2)))/16. - (log(mH2/Q2)*log(mstau22/Q2)*power2(
     sa)*(-4 + (24*mstau22)/mh2 + (3*mstau22)/mH2 - (4*mu2)/mstau22 + (12*
     mstau22*mu2)/(mH2*mstau12) + (8*mu2)/(-mstau22 + mu2) + (2*mH2*mu2)/(
     mstau12*mstau22 - mstau12*mu2) - (8*mH2*mstau22)/power2(mh2) + (32*mH2*
     mstau22)/(-(mh2*mH2) + power2(mh2)) + (8*mstau22*power2(mH2))/((-mh2 +
     mH2)*power2(mh2)) + (4*mu2*DeltaInv(mstau22,mH2,mu2)*(mu2*(-2*mstau22 +
     mu2) - 2*mH2*(mstau22 + mu2) + power2(mH2))*(-2*mstau12*mu2 + mu2*(-mH2 +
     mu2) + power2(mstau12)))/(mstau22*power2(mstau12 - mu2)) + (4*mH2*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (4*power2(mu2))/power2(mstau22 -
     mu2) - (2*mH2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (4*mH2*
     power2(mu2))/power3(mstau22 - mu2) + 2*mu2*DeltaInv(mstau22,mH2,mstau12)*(
     18*mstau22 + (12*mstau24)/mstau12 + ((mstau12 + 2*mstau22)*(mstau22 - 2*
     mu2)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + mH2*(1 - mstau24/(
     mstau12*mstau22 - mstau12*mu2) + 2*mstau22*(1/(mstau22 - mu2) + 1/(-
     mstau12 + mu2) + mu2/power2(mstau12 - mu2) - mu2/power2(mstau22 - mu2)) +
     (-6*mstau22 + (mstau24*mu2)/power2(mstau22 - mu2))/mstau12 - power2(mu2)/
     power2(mstau22 - mu2)) - ((mstau22 - 2*mu2)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (6*mstau24 - (6*power6(mstau2))/mstau12)/mH2)))/16. + (
     log(mH2/Q2)*log(mstau12/Q2)*power2(sa)*(4 - (24*mstau12)/mh2 - (3*mstau12)
     /mH2 + (12*mu2)/mH2 + (16*mu2)/mstau12 + (12*mstau22*mu2)/(mH2*mstau12) +
     (8*mu2)/(mstau12 - mu2) + (4*mH2*mu2)/(mstau12*mstau22 - mstau12*mu2) + (
     8*mH2*mstau12)/power2(mh2) - (32*mH2*mstau12)/(-(mh2*mH2) + power2(mh2)) +
     (8*mstau12*power2(mH2))/((mh2 - mH2)*power2(mh2)) + (2*mH2*mu2)/power2(
     mstau12 - mu2) - (4*mu2*DeltaInv(mH2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2)
     - 2*mH2*(mstau12 + mu2) + power2(mH2))*(-2*mstau22*mu2 + mu2*(-mH2 + mu2)
     + power2(mstau22)))/(mstau12*power2(mstau22 - mu2)) + (4*power2(mu2))/
     power2(mstau12 - mu2) - (8*mH2*power2(mu2))/(mstau12*power2(mstau22 - mu2)
     ) + (4*mH2*power2(mu2))/power3(-mstau12 + mu2) + 2*mu2*DeltaInv(mstau22,
     mH2,mstau12)*((6*(6*mstau12*mstau22 + mstau24))/mstau12 + mH2*(4 +
     mstau22/(mstau12 - mu2) + (4*mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12
     + mu2) - (2*mstau24)/(mstau12*mstau22 - mstau12*mu2) - mstau24/power2(
     mstau12 - mu2) + (3*mstau22*mu2)/power2(mstau12 - mu2) + (6*mstau22 + (2*
     mstau24*mu2)/power2(mstau22 - mu2))/mstau12 - (4*mstau22*mu2)/power2(
     mstau22 - mu2)) + (-2/(mstau12*mstau22 - mstau12*mu2) - 1/power2(mstau12 -
     mu2) + (2*mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mH2) + (power2(mH2)
     *(-2*power2(mu2)*(-2*mstau22*mu2 + power2(mstau22) + 3*power2(mu2)) +
     power2(mstau12)*(-10*mstau22*mu2 + power2(mstau22) + 13*power2(mu2)) + 4*(
     mstau22 - 2*mu2)*power3(mstau12) + 2*mstau12*(-(mu2*power2(mstau22)) +
     mstau22*power2(mu2) + power3(mstau22) + power3(mu2))))/(mstau12*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (6*mstau24 - (6*power6(mstau2))/
     mstau12)/mH2)))/16. - (ca*mu2*sa*sb*power2(log(mstau22/Q2))*((6*mstau22)/(
     mH2*mstau12) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + mstau22*DeltaInv(
     mstau22,mstau12,mh2) - (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(-mstau12 + mu2) + (DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/mstau12 + (3*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*
     mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (2*mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (mstau24*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (5*mu2*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(-mstau22 + mu2) - (10*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (5*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) - (mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mh2,
     mu2)*(mh2*(10*mstau22 - mu2)*mu2 + 5*(mstau22 + mu2)*power2(mh2) + (
     mstau22 - mu2)*power2(mu2) - 3*power3(mh2)))/(mstau22*power2(mstau12 -
     mu2)) + (3*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(
     mstau12 - mu2)) - (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 -
     mu2) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12
     - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 -
     mu2) + (DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 -
     mu2)) + (DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 -
     mu2) + DeltaInv(mstau22,mH2,mstau12)*((11*mstau24)/mstau12 + (3*mstau24)/(
     mstau12 - mu2) + (mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(
     mstau22 - mu2) + (mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) +
     mstau22*(5 + (2*mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 -
     mu2)) + mH2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12
     - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(
     mstau22 - mu2) + (mstau22*(-4 - (2*power2(mu2))/power2(mstau22 - mu2)))/
     mstau12) + power2(mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(
     mu2)/power2(mstau22 - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) -
     power6(mstau2)/power2(mstau12 - mu2) + (18*mstau24 - (6*power6(mstau2))/
     mstau12)/mH2)))/(8.*cb) + (ca*mu2*sa*sb*log(mstau22/Q2)*((18*mstau22)/(
     mH2*mstau12) - 9*mh2*DeltaInv(mstau22,mstau12,mh2) + 3*mstau22*DeltaInv(
     mstau22,mstau12,mh2) - (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (9*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(-mstau12 + mu2) - mstau22/power2(mh2) + (mH2*
     mstau22)/((-mh2 + mH2)*power2(mh2)) + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/mstau12 + mstau22/(-(mh2*mH2) + power2(mh2)) + (9*mh2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (9*mH2*mu2)/(mstau22*power2(mstau12 -
     mu2)) - (6*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) - (3*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) - (3*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2)
     + (3*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 -
     mu2) - (15*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 -
     mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) - (
     30*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (15*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (9*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22
     - mu2) + (6*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(
     mstau22 - mu2) + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*mu2*DeltaInv(mstau22,mh2,mu2)*(mh2*mu2*(-10*mstau22 + mu2) - 5*(mstau22
     + mu2)*power2(mh2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mh2)))/(
     mstau22*power2(mstau12 - mu2)) + (9*mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) + (3*mH2*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mH2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2) + 3*DeltaInv(mstau22,
     mH2,mstau12)*((11*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (
     mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (
     mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(5 + (2*
     mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 +
     (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (
     mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2)
     + (mstau22*(-4 - (2*power2(mu2))/power2(mstau22 - mu2)))/mstau12) +
     power2(mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(
     mstau22 - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(
     mstau2)/power2(mstau12 - mu2) + (18*mstau24 - (6*power6(mstau2))/mstau12)/
     mH2)))/(4.*cb) + (power2(sa)*power2(log(mstau22/Q2))*((-21*mstau22)/mh2 +
     (24*mstau22)/(mh2 - mH2) - (3*mstau22)/mH2 + (6*mu2)/mh2 - (6*mu2)/mH2 + (
     12*mstau22*mu2)/(mh2*mstau12) + (12*mstau22*mu2)/(mH2*mstau12) + 10*mH2*
     mu2*DeltaInv(mstau22,mH2,mu2) - 6*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) +
     14*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2) - (16*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (36*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/mh2 + (26*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12
     + (6*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (6*
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (8*mH2*
     mstau22)/power2(mh2) + (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/
     mstau12 - (32*mH2*mstau22)/(-(mh2*mH2) + power2(mh2)) - (6*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/mstau22 + (8*mstau22*power2(mH2))/((mh2 -
     mH2)*power2(mh2)) - (4*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau12 - mu2) - (2*power2(mu2))/(mh2*mstau22) + (2*
     power2(mu2))/(mH2*mstau22) + (8*power2(mu2))/(mh2*mstau22 - mh2*mu2) - (8*
     power2(mu2))/(mH2*mstau22 - mH2*mu2) + 20*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2) + (10*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/mstau22 + (4*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12 - mu2) + (6*
     mh2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (6*mH2*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) - (2*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau12 - mu2) - (2*mstau24*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) - (10*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/power2(mstau12 - mu2) + (2*mu2*DeltaInv(
     mstau22,mh2,mu2)*(-2*mstau12*mu2 + mu2*(-mh2 + mu2) + power2(mstau12))*(
     mh2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)*power2(mh2) + (-mstau22 +
     mu2)*power2(mu2) + 3*power3(mh2)))/(mh2*mstau22*power2(mstau12 - mu2)) + (
     6*DeltaInv(mstau22,mH2,mu2)*power2(mu2)*power3(mH2))/(mstau22*power2(
     mstau12 - mu2)) + (2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mH2 - (2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mstau22 + (2*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/(-mstau22 + mu2) - (20*mH2*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/power2(mstau12 - mu2) - (10*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (6*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (4*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) +
     (4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (2*mstau24*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,
     mH2,mu2)*power4(mu2))/(mH2*mstau22) - (2*DeltaInv(mstau22,mH2,mu2)*power4(
     mu2))/power2(mstau12 - mu2) + (2*mH2*DeltaInv(mstau22,mH2,mu2)*power4(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,mstau12,mh2)*
     power4(mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mH2,mu2)*power5(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (12*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power6(mstau2))/(mh2*mstau12) + (2*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power6(mstau2))/power2(mstau12 - mu2) + 2*mu2*DeltaInv(mstau22,mH2,
     mstau12)*((11*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (mstau24*
     mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (mstau24*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(5 + (2*mu2)/(-
     mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 + (3*
     mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (mstau22*
     mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2) + (
     mstau22*(-4 - (2*power2(mu2))/power2(mstau22 - mu2)))/mstau12) + power2(
     mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22
     - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/
     power2(mstau12 - mu2) + (18*mstau24 - (6*power6(mstau2))/mstau12)/mH2)))/
     32. - (log(mstau22/Q2)*power2(sa)*(mh2/mstau22 - mH2/mstau22 - (16*
     mstau22)/mh2 + (22*mstau22)/(mh2 - mH2) + (6*mstau22)/mH2 + mH2/(mstau22 -
     mu2) + (9*mu2)/mh2 - (9*mu2)/mH2 + (18*mstau22*mu2)/(mh2*mstau12) + (18*
     mstau22*mu2)/(mH2*mstau12) + mh2/(-mstau22 + mu2) + 15*mH2*mu2*DeltaInv(
     mstau22,mH2,mu2) - 9*mh2*mu2*DeltaInv(mstau22,mstau12,mh2) + 21*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2) - (24*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (54*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mh2
     + (39*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (9*mh2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (9*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (10*mH2*mstau22)/power2(
     mh2) - (2*mstau22*mu2)/power2(mh2) + (2*mH2*mstau22*mu2)/((-mh2 + mH2)*
     power2(mh2)) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 -
     (32*mH2*mstau22)/(-(mh2*mH2) + power2(mh2)) + (2*mstau22*mu2)/(-(mh2*mH2)
     + power2(mh2)) - (9*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/mstau22 + (
     10*mstau22*power2(mH2))/((mh2 - mH2)*power2(mh2)) - (6*mh2*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (3*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mh2*
     mu2)/power2(mstau22 - mu2) - (mH2*mu2)/power2(mstau22 - mu2) - (3*power2(
     mu2))/(mh2*mstau22) + (3*power2(mu2))/(mH2*mstau22) + (12*power2(mu2))/(
     mh2*mstau22 - mh2*mu2) - (12*power2(mu2))/(mH2*mstau22 - mH2*mu2) + 30*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2) + (15*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/mstau22 + (6*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mu2))/(mstau12 - mu2) + (9*mh2*power2(mu2))/(mstau22*power2(mstau12 - mu2)
     ) - (9*mH2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) - (3*
     mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau12 - mu2) -
     (15*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/power2(mstau12 -
     mu2) - (mh2*power2(mu2))/(mstau22*power2(mstau22 - mu2)) + (mH2*power2(
     mu2))/(mstau22*power2(mstau22 - mu2)) + (3*mu2*DeltaInv(mstau22,mh2,mu2)*(
     -2*mstau12*mu2 + mu2*(-mh2 + mu2) + power2(mstau12))*(mh2*mu2*(-10*mstau22
     + mu2) - 5*(mstau22 + mu2)*power2(mh2) + (-mstau22 + mu2)*power2(mu2) + 3*
     power3(mh2)))/(mh2*mstau22*power2(mstau12 - mu2)) + (9*DeltaInv(mstau22,
     mH2,mu2)*power2(mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + (2*
     mh2*power2(mu2))/power3(mstau22 - mu2) + (3*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/mH2 - (3*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/mstau22 + (3*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/(-mstau22 + mu2) - (30*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (15*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power3(mu2))/(mstau22*power2(mstau12
     - mu2)) + (9*mh2*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22
     - mu2) + (6*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(
     mstau22 - mu2) + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mstau12,
     mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     2*mH2*power2(mu2))/power3(-mstau22 + mu2) - (3*DeltaInv(mstau22,mH2,mu2)*
     power4(mu2))/(mH2*mstau22) - (3*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/
     power2(mstau12 - mu2) + (3*mH2*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power4(
     mu2))/power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mH2,mu2)*power5(mu2))/(
     mstau22*power2(mstau12 - mu2)) - (18*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power6(mstau2))/(mh2*mstau12) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power6(mstau2))/power2(mstau12 - mu2) + 3*mu2*DeltaInv(mstau22,mH2,
     mstau12)*((11*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (mstau24*
     mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (mstau24*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(5 + (2*mu2)/(-
     mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 + (3*
     mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (mstau22*
     mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2) + (
     mstau22*(-4 - (2*power2(mu2))/power2(mstau22 - mu2)))/mstau12) + power2(
     mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22
     - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/
     power2(mstau12 - mu2) + (18*mstau24 - (6*power6(mstau2))/mstau12)/mH2)))/
     8. - (mu2*log(mA2/Q2)*log(mstau12/Q2)*power2(sb)*(2/mA2 + 2/mstau12 - 4/(
     mstau12 - mstau22) + (2*mstau22)/(mA2*mstau12) + (4*mu2)/((mstau12 -
     mstau22)*(-mstau12 + mu2)) - (2*mA2)/(mstau12*mstau22 - mstau12*mu2) -
     mA2/power2(mstau12 - mu2) + (4*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) -
     (2*mA2*mu2*DeltaInv(mA2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mA2*(
     mstau12 + mu2) + power2(mA2)))/(mstau12*power2(mstau22 - mu2)) + DeltaInv(
     mA2,mstau22,mstau12)*((2*(6*mstau12*mstau22 + mstau24))/mstau12 + power2(
     mA2)*(3/(-mstau12 + mu2) + 4/(-mstau22 + mu2) - (4*mstau22)/(mstau12*
     mstau22 - mstau12*mu2) - (2*mstau22)/power2(mstau12 - mu2) - mu2/power2(
     mstau12 - mu2) + (-2 + (4*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 + (
     4*mu2)/power2(mstau22 - mu2)) + mA2*(4 + (2*mu2)/(mstau12 - mu2) +
     mstau22/(-mstau12 + mu2) + (4*mstau22)/(-mstau22 + mu2) + (2*mstau24)/(
     mstau12*mstau22 - mstau12*mu2) + mstau24/power2(mstau12 - mu2) - (3*
     mstau22*mu2)/power2(mstau12 - mu2) + (2*mstau22 - (2*mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 + (4*mstau22*mu2)/power2(mstau22 - mu2)) + (2/(
     mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2) - (2*mu2)/(
     mstau12*power2(mstau22 - mu2)))*power3(mA2) + (2*mstau24 - (2*power6(
     mstau2))/mstau12)/mA2)))/(8.*power2(cb)) + (mu2*log(mA2/Q2)*log(mstau22/
     Q2)*power2(sb)*(-4/(mstau12 - mstau22) + (2*mstau22)/(mA2*mstau12) + mA2/(
     mstau12*(-mstau22 + mu2)) + (4*mu2)/((mstau12 - mstau22)*(-mstau22 + mu2))
     - (2*mA2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mA2*mu2*DeltaInv(mA2,
     mstau22,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mA2*(mstau22 + mu2) + power2(mA2)
     ))/(mstau22*power2(mstau12 - mu2)) + (mA2*mu2)/(mstau12*power2(mstau22 -
     mu2)) + DeltaInv(mA2,mstau22,mstau12)*(6*mstau22 + (4*mstau24)/mstau12 - (
     (mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mA2))/(mstau12*power2(
     mstau22 - mu2)) + ((mstau22 - 2*mu2)*power3(mA2))/(mstau12*power2(mstau22
     - mu2)) + (mA2*(-(mstau12*(mstau22 - 2*mu2)*mu2*(2*mstau24 + 3*mstau22*
     mu2)) + (mstau22 - 2*mu2)*(mstau24 + 6*mstau22*mu2)*power2(mstau12) - 3*
     mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(mu2)*(-2*mstau24*mu2 +
     4*mu2*power2(mstau22) + mstau22*(mstau24 - 2*power2(mu2)) - 2*power3(
     mstau22))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (2*
     mstau24 - (2*power6(mstau2))/mstau12)/mA2)))/(8.*power2(cb)) + (mu2*
     power2(sb)*power2(log(mstau22/Q2))*(2/msntau2 + 2/mstau12 + (2*mstau22)/(
     mC2*msntau2) - (2*mstau22)/(mA2*mstau12) + (6*mstau22)/(mH2*mstau12) + 4/(
     mstau22 - mu2) + 6*mC2*DeltaInv(mstau22,mC2,msntau2) + (2*mC2*mstau22*
     DeltaInv(mstau22,mC2,msntau2))/msntau2 + (6*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/mC2 + (2*mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (6*
     mC2*mstau22*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (6*mstau24*
     DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (4*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2))/(-msntau2 + mu2) + 3*mH2*DeltaInv(mstau22,mH2,
     mstau12) + 5*mstau22*DeltaInv(mstau22,mH2,mstau12) - (4*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (18*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/mH2 + (11*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (3*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(-mstau12 + mu2) - (2*DeltaInv(mstau22,mC2,msntau2)*
     power2(mC2))/msntau2 - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12
     + (2*mstau22)/power2(msntau2 - mu2) - (2*mu2)/power2(msntau2 - mu2) - (6*
     mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (4*mC2*mstau24*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*mC2*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*mstau24*mu2*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) - (2*mstau22*DeltaInv(mstau22,
     mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) - (10*mu2*DeltaInv(
     mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (2*mstau22)/power2(
     mstau12 - mu2) - (2*mu2)/power2(mstau12 - mu2) - (3*mA2*mu2)/(mstau22*
     power2(mstau12 - mu2)) - (3*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*
     mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mH2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (
     mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (5*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (4*
     mu2)/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))
     /(mstau22 - mu2) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 -
     mu2) - (2*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (20*mC2*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (10*DeltaInv(
     mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2))
     - (2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (10*mH2*DeltaInv(
     mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (5*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (2*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (2*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (6*mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))
     /power2(mstau22 - mu2) - (4*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(
     mu2))/power2(mstau22 - mu2) - (4*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)
     *power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (2*mstau24*DeltaInv(
     mstau22,mC2,msntau2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (3*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) -
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mA2,mstau22,mu2)*(mA2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)
     *power2(mA2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mA2)))/(mstau22*
     power2(mstau12 - mu2)) + (6*mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(
     mstau22*power2(msntau2 - mu2)) + (3*mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,mC2,mu2)*
     power3(mu2))/power2(msntau2 - mu2) + (2*mC2*DeltaInv(mstau22,mC2,mu2)*
     power3(mu2))/(mstau22*power2(msntau2 - mu2)) - (DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) + (mH2*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (2*DeltaInv(mstau22,mC2,
     msntau2)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power3(mu2))/power2(mstau22 - mu2) + (8*power2(mu2))/power3(-
     mstau22 + mu2) + (2*DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(mstau22*
     power2(msntau2 - mu2)) + (DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,mC2,msntau2)*power6(mstau2))/
     (mC2*msntau2) - (6*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*
     mstau12) - (2*DeltaInv(mstau22,mC2,msntau2)*power6(mstau2))/power2(msntau2
     - mu2) - (DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 -
     mu2) + DeltaInv(mA2,mstau22,mstau12)*((-5*mstau24)/mstau12 + (3*mstau24)/(
     mstau12 - mu2) + (mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(
     mstau22 - mu2) + (mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) +
     mstau22*(-3 + (2*mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 -
     mu2)) + mA2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12
     - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(
     mstau22 - mu2) + (mstau22*(4 - (2*power2(mu2))/power2(mstau22 - mu2)))/
     mstau12) + power2(mA2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(
     mu2)/power2(mstau22 - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) -
     power6(mstau2)/power2(mstau12 - mu2) + (-6*mstau24 + (2*power6(mstau2))/
     mstau12)/mA2)))/(16.*power2(cb)) - (mu2*log(mstau22/Q2)*power2(sb)*(6/
     msntau2 + 6/mstau12 + 4/mstau22 + (6*mstau22)/(mC2*msntau2) - (6*mstau22)/
     (mA2*mstau12) + (18*mstau22)/(mH2*mstau12) + 4/(mstau22 - mu2) + 18*mC2*
     DeltaInv(mstau22,mC2,msntau2) + (6*mC2*mstau22*DeltaInv(mstau22,mC2,
     msntau2))/msntau2 + (18*mstau24*DeltaInv(mstau22,mC2,msntau2))/mC2 + (6*
     mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (18*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/(msntau2 - mu2) + (18*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/(msntau2 - mu2) + (12*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))
     /(-msntau2 + mu2) + 9*mH2*DeltaInv(mstau22,mH2,mstau12) + 15*mstau22*
     DeltaInv(mstau22,mH2,mstau12) - (12*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/mstau12 + (54*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (33*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (9*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (9*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12 - mu2) + (6*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (-mstau12 + mu2) - (6*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/msntau2 -
     (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (6*mstau22)/
     power2(msntau2 - mu2) - (6*mu2)/power2(msntau2 - mu2) - (18*mC2*mu2)/(
     mstau22*power2(msntau2 - mu2)) + (12*mC2*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/power2(msntau2 - mu2) + (6*mC2*mstau22*mu2*DeltaInv(mstau22,mC2,
     msntau2))/power2(msntau2 - mu2) + (6*mstau24*mu2*DeltaInv(mstau22,mC2,
     msntau2))/power2(msntau2 - mu2) - (6*mstau22*DeltaInv(mstau22,mC2,msntau2)
     *power2(mC2))/power2(msntau2 - mu2) - (30*mu2*DeltaInv(mstau22,mC2,mu2)*
     power2(mC2))/power2(msntau2 - mu2) + (6*mstau22)/power2(mstau12 - mu2) - (
     6*mu2)/power2(mstau12 - mu2) - (9*mA2*mu2)/(mstau22*power2(mstau12 - mu2))
     - (9*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (6*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (3*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (3*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (15*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (4*mu2)/power2(
     mstau22 - mu2) + (6*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(mstau22 -
     mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 - mu2) - (6*
     power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (60*mC2*DeltaInv(mstau22,
     mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (30*DeltaInv(mstau22,mC2,
     mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (6*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (30*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (15*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (6*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) - (6*power2(mu2))/(mstau12*power2(mstau22 -
     mu2)) - (4*power2(mu2))/(mstau22*power2(mstau22 - mu2)) - (18*mC2*
     DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) - (12*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) -
     (12*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(msntau2*
     power2(mstau22 - mu2)) + (6*mstau24*DeltaInv(mstau22,mC2,msntau2)*power2(
     mu2))/(msntau2*power2(mstau22 - mu2)) - (9*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) - (6*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (6*DeltaInv(mstau22,mC2,msntau2)*power2(mC2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*mu2*DeltaInv(mA2,
     mstau22,mu2)*(mA2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)*power2(mA2)
     + (-mstau22 + mu2)*power2(mu2) + 3*power3(mA2)))/(mstau22*power2(mstau12 -
     mu2)) + (18*mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(
     msntau2 - mu2)) + (9*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*
     power2(mstau12 - mu2)) - (6*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/power2(
     msntau2 - mu2) + (6*mC2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mstau22*
     power2(msntau2 - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(
     mstau12 - mu2) + (3*mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2)) + (6*DeltaInv(mstau22,mC2,msntau2)*power3(mu2))/
     power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/
     power2(mstau22 - mu2) + (8*power2(mu2))/power3(-mstau22 + mu2) + (6*
     DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(mstau22*power2(msntau2 - mu2)) + (
     3*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) -
     (6*DeltaInv(mstau22,mC2,msntau2)*power6(mstau2))/(mC2*msntau2) - (18*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) - (6*DeltaInv(
     mstau22,mC2,msntau2)*power6(mstau2))/power2(msntau2 - mu2) - (3*DeltaInv(
     mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + 3*DeltaInv(
     mA2,mstau22,mstau12)*((-5*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) +
     (mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (
     mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(-3 + (2*
     mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mA2*(3 +
     (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (
     mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2)
     + (mstau22*(4 - (2*power2(mu2))/power2(mstau22 - mu2)))/mstau12) + power2(
     mA2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22
     - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/
     power2(mstau12 - mu2) + (-6*mstau24 + (2*power6(mstau2))/mstau12)/mA2)))/(
     8.*power2(cb)) - (ca*mu2*sa*sb*log(mH2/Q2)*log(mstau22/Q2)*(2/(mstau12 -
     mstau22) - (6*mstau22)/(mH2*mstau12) + (2*mu2)/((mstau12 - mstau22)*(
     mstau22 - mu2)) + mH2/(mstau12*(-mstau22 + mu2)) - (2*mH2*mu2)/(mstau22*
     power2(mstau12 - mu2)) + (2*mH2*mu2*DeltaInv(mstau22,mH2,mu2)*(mu2*(-2*
     mstau22 + mu2) - 2*mH2*(mstau22 + mu2) + power2(mH2)))/(mstau22*power2(
     mstau12 - mu2)) + (mH2*mu2)/(mstau12*power2(mstau22 - mu2)) + DeltaInv(
     mstau22,mH2,mstau12)*((-6*(3*mstau12*mstau22 + 2*mstau24))/mstau12 - ((
     mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + mH2*(-1 + mstau24/(mstau12*mstau22 - mstau12*mu2) + 2*
     mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2) - mu2/power2(mstau12 -
     mu2) + mu2/power2(mstau22 - mu2)) + (6*mstau22 - (mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 - mu2)) + ((mstau22 -
     2*mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (-6*mstau24 + (6*
     power6(mstau2))/mstau12)/mH2)))/(4.*cb) + (mu2*log(mH2/Q2)*log(mstau22/Q2)
     *power2(sb)*(4/(mstau12 - mstau22) - (6*mstau22)/(mH2*mstau12) + (4*mu2)/(
     (mstau12 - mstau22)*(mstau22 - mu2)) + mH2/(mstau12*(-mstau22 + mu2)) - (
     2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mH2*mu2*DeltaInv(mstau22,
     mH2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mH2*(mstau22 + mu2) + power2(mH2)))/(
     mstau22*power2(mstau12 - mu2)) + (mH2*mu2)/(mstau12*power2(mstau22 - mu2))
     + DeltaInv(mstau22,mH2,mstau12)*((-6*(3*mstau12*mstau22 + 2*mstau24))/
     mstau12 - ((mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mH2))/(mstau12*
     power2(mstau22 - mu2)) + mH2*(-1 + mstau24/(mstau12*mstau22 - mstau12*mu2)
     + 2*mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2) - mu2/power2(mstau12 -
     mu2) + mu2/power2(mstau22 - mu2)) + (6*mstau22 - (mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 - mu2)) + ((mstau22 -
     2*mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (-6*mstau24 + (6*
     power6(mstau2))/mstau12)/mH2)))/(8.*power2(cb)) + (ca*mu2*sa*sb*log(mH2/
     Q2)*log(mstau12/Q2)*(-6/mH2 - 6/mstau12 + 2/(mstau12 - mstau22) - (6*
     mstau22)/(mH2*mstau12) + (2*mu2)/((mstau12 - mstau22)*(mstau12 - mu2)) - (
     2*mH2)/(mstau12*mstau22 - mstau12*mu2) - mH2/power2(mstau12 - mu2) + (4*
     mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (2*mH2*mu2*DeltaInv(mH2,
     mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mH2*(mstau12 + mu2) + power2(mH2)
     ))/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*((-6*(
     6*mstau12*mstau22 + mstau24))/mstau12 + mH2*(-4 + (2*mu2)/(mstau12 - mu2)
     + mstau22/(-mstau12 + mu2) + (4*mstau22)/(-mstau22 + mu2) + (2*mstau24)/(
     mstau12*mstau22 - mstau12*mu2) + mstau24/power2(mstau12 - mu2) - (3*
     mstau22*mu2)/power2(mstau12 - mu2) + (-6*mstau22 - (2*mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 + (4*mstau22*mu2)/power2(mstau22 - mu2)) + (2/(
     mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2) - (2*mu2)/(
     mstau12*power2(mstau22 - mu2)))*power3(mH2) - (power2(mH2)*(-2*power2(mu2)
     *(-2*mstau22*mu2 + power2(mstau22) + 3*power2(mu2)) + power2(mstau12)*(-
     10*mstau22*mu2 + power2(mstau22) + 13*power2(mu2)) + 4*(mstau22 - 2*mu2)*
     power3(mstau12) + 2*mstau12*(-(mu2*power2(mstau22)) + mstau22*power2(mu2)
     + power3(mstau22) + power3(mu2))))/(mstau12*power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) + (-6*mstau24 + (6*power6(mstau2))/mstau12)/mH2)))/(4.*cb)
     - (mu2*log(mH2/Q2)*log(mstau12/Q2)*power2(sb)*(-6/mH2 - 6/mstau12 + 4/(
     mstau12 - mstau22) - (6*mstau22)/(mH2*mstau12) + (4*mu2)/((mstau12 -
     mstau22)*(mstau12 - mu2)) - (2*mH2)/(mstau12*mstau22 - mstau12*mu2) - mH2/
     power2(mstau12 - mu2) + (4*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (2*
     mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mH2*(mstau12
     + mu2) + power2(mH2)))/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,
     mH2,mstau12)*((-6*(6*mstau12*mstau22 + mstau24))/mstau12 + mH2*(-4 + (2*
     mu2)/(mstau12 - mu2) + mstau22/(-mstau12 + mu2) + (4*mstau22)/(-mstau22 +
     mu2) + (2*mstau24)/(mstau12*mstau22 - mstau12*mu2) + mstau24/power2(
     mstau12 - mu2) - (3*mstau22*mu2)/power2(mstau12 - mu2) + (-6*mstau22 - (2*
     mstau24*mu2)/power2(mstau22 - mu2))/mstau12 + (4*mstau22*mu2)/power2(
     mstau22 - mu2)) + (2/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 -
     mu2) - (2*mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mH2) - (power2(mH2)
     *(-2*power2(mu2)*(-2*mstau22*mu2 + power2(mstau22) + 3*power2(mu2)) +
     power2(mstau12)*(-10*mstau22*mu2 + power2(mstau22) + 13*power2(mu2)) + 4*(
     mstau22 - 2*mu2)*power3(mstau12) + 2*mstau12*(-(mu2*power2(mstau22)) +
     mstau22*power2(mu2) + power3(mstau22) + power3(mu2))))/(mstau12*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (-6*mstau24 + (6*power6(mstau2))/
     mstau12)/mH2)))/(8.*power2(cb)) + (mu2*power2(sb)*power2(log(msntau2/Q2))*
     (1/mC2 + 2/msntau2 + (3*mstau22)/(mC2*msntau2) + 1/(mstau22 - mu2) + (3*
     mC2)/(msntau2*mstau22 - msntau2*mu2) + mC2/power2(msntau2 - mu2) +
     mstau22/power2(msntau2 - mu2) - (2*mu2)/power2(mstau22 - mu2) - (6*mC2*
     mu2)/(msntau2*power2(mstau22 - mu2)) - (2*power2(mu2))/(msntau2*power2(
     mstau22 - mu2)) + (mu2*DeltaInv(mC2,msntau2,mu2)*(mC2*mu2*(-10*msntau2 +
     mu2) - 5*(msntau2 + mu2)*power2(mC2) + (-msntau2 + mu2)*power2(mu2) + 3*
     power3(mC2)))/(msntau2*power2(mstau22 - mu2)) + (2*power2(mu2))/power3(-
     msntau2 + mu2) + (DeltaInv(mstau22,mC2,msntau2)*(power3(mC2)*(-(power2(
     mu2)*(8*mstau22*mu2 - 4*power2(mstau22) + power2(mu2))) + power2(msntau2)*
     (-24*mstau22*mu2 + 7*power2(mstau22) + 22*power2(mu2)) + 5*(mstau22 - 2*
     mu2)*power3(msntau2) + msntau2*(-12*mu2*power2(mstau22) + 26*mstau22*
     power2(mu2) + power3(mstau22) - 10*power3(mu2))) - (3*(mstau22 - 2*mu2)*
     power2(msntau2) + 3*(mstau22 - 2*mu2)*power2(mu2) + msntau2*(-8*mstau22*
     mu2 + power2(mstau22) + 13*power2(mu2)))*power4(mC2) + power2(mC2)*((-18*
     mstau22*mu2 + 9*power2(mstau22) - power2(mu2))*power3(msntau2) + power2(
     mu2)*(6*mstau24*mu2 - 2*mu2*power2(mstau22) - mstau22*(3*mstau24 + power2(
     mu2)) + power3(mstau22)) + power2(msntau2)*(-3*mstau22*mstau24 + 6*
     mstau24*mu2 - 26*mu2*power2(mstau22) + 41*mstau22*power2(mu2) + 3*power3(
     mstau22)) + msntau2*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*
     power2(mu2) + power2(mstau22)*(mstau24 + 7*power2(mu2)) + 2*mu2*power3(
     mstau22) + power4(mu2))) + power2(msntau2 - mu2)*power2(mstau22 - mu2)*(5*
     msntau2*mstau24 - 3*power6(mstau2)) + mC2*(mstau24*power2(mu2)*(-8*
     mstau22*mu2 + 4*power2(mstau22) + 5*power2(mu2)) + mstau22*(-18*mstau22*
     mu2 + 9*power2(mstau22) + 8*power2(mu2))*power3(msntau2) + power2(msntau2)
     *(8*mstau24*power2(mu2) + power2(mstau22)*(7*mstau24 + 40*power2(mu2)) -
     20*mu2*power3(mstau22) - 2*mstau22*(7*mstau24*mu2 + 9*power3(mu2))) -
     msntau2*(mu2*(12*mstau24*power2(mu2) + 2*power2(mstau22)*(5*mstau24 + 11*
     power2(mu2)) - 11*mu2*power3(mstau22) - 10*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mC2*msntau2*
     power2(msntau2 - mu2)*power2(mstau22 - mu2))))/(8.*power2(cb)) - (mu2*log(
     msntau2/Q2)*power2(sb)*(3/mC2 + 7/msntau2 + (9*mstau22)/(mC2*msntau2) + 3/
     (mstau22 - mu2) + 2/(-msntau2 + mu2) + (9*mC2)/(msntau2*mstau22 - msntau2*
     mu2) + (3*mC2)/power2(msntau2 - mu2) + (3*mstau22)/power2(msntau2 - mu2) +
     (2*mu2)/power2(msntau2 - mu2) - (6*mu2)/power2(mstau22 - mu2) - (18*mC2*
     mu2)/(msntau2*power2(mstau22 - mu2)) - power2(mu2)/(msntau2*power2(msntau2
     - mu2)) - (6*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (3*mu2*
     DeltaInv(mC2,msntau2,mu2)*(mC2*mu2*(-10*msntau2 + mu2) - 5*(msntau2 + mu2)
     *power2(mC2) + (-msntau2 + mu2)*power2(mu2) + 3*power3(mC2)))/(msntau2*
     power2(mstau22 - mu2)) + (2*power2(mu2))/power3(-msntau2 + mu2) + (3*
     DeltaInv(mstau22,mC2,msntau2)*(power3(mC2)*(-(power2(mu2)*(8*mstau22*mu2 -
     4*power2(mstau22) + power2(mu2))) + power2(msntau2)*(-24*mstau22*mu2 + 7*
     power2(mstau22) + 22*power2(mu2)) + 5*(mstau22 - 2*mu2)*power3(msntau2) +
     msntau2*(-12*mu2*power2(mstau22) + 26*mstau22*power2(mu2) + power3(
     mstau22) - 10*power3(mu2))) - (3*(mstau22 - 2*mu2)*power2(msntau2) + 3*(
     mstau22 - 2*mu2)*power2(mu2) + msntau2*(-8*mstau22*mu2 + power2(mstau22) +
     13*power2(mu2)))*power4(mC2) + power2(mC2)*((-18*mstau22*mu2 + 9*power2(
     mstau22) - power2(mu2))*power3(msntau2) + power2(mu2)*(6*mstau24*mu2 - 2*
     mu2*power2(mstau22) - mstau22*(3*mstau24 + power2(mu2)) + power3(mstau22))
     + power2(msntau2)*(-3*mstau22*mstau24 + 6*mstau24*mu2 - 26*mu2*power2(
     mstau22) + 41*mstau22*power2(mu2) + 3*power3(mstau22)) + msntau2*(4*
     mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*power2(mu2) + power2(
     mstau22)*(mstau24 + 7*power2(mu2)) + 2*mu2*power3(mstau22) + power4(mu2)))
     + power2(msntau2 - mu2)*power2(mstau22 - mu2)*(5*msntau2*mstau24 - 3*
     power6(mstau2)) + mC2*(mstau24*power2(mu2)*(-8*mstau22*mu2 + 4*power2(
     mstau22) + 5*power2(mu2)) + mstau22*(-18*mstau22*mu2 + 9*power2(mstau22) +
     8*power2(mu2))*power3(msntau2) + power2(msntau2)*(8*mstau24*power2(mu2) +
     power2(mstau22)*(7*mstau24 + 40*power2(mu2)) - 20*mu2*power3(mstau22) - 2*
     mstau22*(7*mstau24*mu2 + 9*power3(mu2))) - msntau2*(mu2*(12*mstau24*
     power2(mu2) + 2*power2(mstau22)*(5*mstau24 + 11*power2(mu2)) - 11*mu2*
     power3(mstau22) - 10*mstau22*(2*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2)))))/(mC2*msntau2*power2(msntau2 - mu2)*
     power2(mstau22 - mu2))))/(4.*power2(cb)) - (mu2*power2(sb)*power2(log(
     mstau12/Q2))*(2/mA2 - 6/mH2 - 6/mstau12 + (6*mstau22)/(mA2*mstau12) - (18*
     mstau22)/(mH2*mstau12) + 2/(-mstau22 + mu2) - (3*mA2)/(mstau12*mstau22 -
     mstau12*mu2) - (3*mH2)/(mstau12*mstau22 - mstau12*mu2) - 8*mH2*DeltaInv(
     mstau22,mH2,mstau12) - 59*mstau22*DeltaInv(mstau22,mH2,mstau12) + (4*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (30*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/mH2 - (29*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 + (2*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (3*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(-mstau22 + mu2) + (3*mH2*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + (7*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/(-mstau12 + mu2) + (5*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-
     mstau22 + mu2) - (6*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau12*mstau22 - mstau12*mu2) - mA2/power2(mstau12 - mu2) - mH2/power2(
     mstau12 - mu2) - (2*mstau22)/power2(mstau12 - mu2) - (mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (6*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (mu2*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) + (4*mu2)/power2(
     mstau22 - mu2) + (6*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) + (6*mH2*mu2)
     /(mstau12*power2(mstau22 - mu2)) + (6*mH2*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau22 - mu2) - (3*mH2*mstau24*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (5*mu2*DeltaInv(mH2,
     mstau12,mu2)*power2(mH2))/power2(mstau22 - mu2) + (5*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (6*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 + mu2) + (4*power2(
     mu2))/(mstau12*power2(mstau22 - mu2)) + (10*mH2*DeltaInv(mH2,mstau12,mu2)*
     power2(mu2))/power2(mstau22 - mu2) + (4*mH2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) + (2*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (5*DeltaInv(mH2,mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mA2,mstau12,
     mu2)*(mA2*(10*mstau12 - mu2)*mu2 + 5*(mstau12 + mu2)*power2(mA2) + (
     mstau12 - mu2)*power2(mu2) - 3*power3(mA2)))/(mstau12*power2(mstau22 -
     mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12
     - mu2) - (3*mu2*DeltaInv(mH2,mstau12,mu2)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(
     mstau12*power2(mstau22 - mu2)) + (4*power2(mu2))/power3(mstau12 - mu2) + (
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/power2(mstau22 - mu2) - (mH2*
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) - (
     DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     18*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (
     DeltaInv(mA2,mstau22,mstau12)*(-(power3(mA2)*(power2(mu2)*(-14*mstau22*mu2
     + 7*power2(mstau22) + 2*power2(mu2)) + 5*power2(mstau12)*(-6*mstau22*mu2 +
     2*power2(mstau22) + 5*power2(mu2)) + 5*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*(-18*mu2*power2(mstau22) + 38*mstau22*power2(mu2) + power3(
     mstau22) - 16*power3(mu2)))) + (3*(mstau22 - 2*mu2)*power2(mstau12) + 3*(
     mstau22 - 2*mu2)*power2(mu2) + mstau12*(-8*mstau22*mu2 + power2(mstau22) +
     13*power2(mu2)))*power4(mA2) - power2(mA2)*(2*(-6*mstau22*mu2 + 3*power2(
     mstau22) - 2*power2(mu2))*power3(mstau12) + power2(mu2)*(-3*mstau22*
     mstau24 + 6*mstau24*mu2 - 8*mu2*power2(mstau22) + 2*mstau22*power2(mu2) +
     4*power3(mstau22)) + power2(mstau12)*(-26*mu2*power2(mstau22) + 6*mu2*(
     mstau24 + power2(mu2)) + mstau22*(-3*mstau24 + 32*power2(mu2)) + 6*power3(
     mstau22)) + mstau12*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*
     power2(mu2) + power2(mstau22)*(mstau24 + 16*power2(mu2)) - 4*mu2*power3(
     mstau22) - 2*power4(mu2))) + 2*power2(mstau12 - mu2)*power2(mstau22 - mu2)
     *(5*mstau12*mstau24 - 3*power6(mstau2)) + mA2*(mstau24*power2(mu2)*(-22*
     mstau22*mu2 + 11*power2(mstau22) + 10*power2(mu2)) + mstau22*(-42*mstau22*
     mu2 + 21*power2(mstau22) + 22*power2(mu2))*power3(mstau12) + power2(
     mstau12)*(7*mstau24*power2(mu2) + 8*power2(mstau22)*(mstau24 + 10*power2(
     mu2)) - 40*mu2*power3(mstau22) - 2*mstau22*(8*mstau24*mu2 + 21*power3(mu2)
     )) + mstau12*(mu2*(-18*mstau24*power2(mu2) - 2*power2(mstau22)*(10*mstau24
     + 19*power2(mu2)) + 19*mu2*power3(mstau22) + 20*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(16.*power2(cb)) + (mu2*
     log(mstau12/Q2)*power2(sb)*(6/mA2 - 18/mH2 - 20/mstau12 + (18*mstau22)/(
     mA2*mstau12) - (54*mstau22)/(mH2*mstau12) + 4/(mstau12 - mu2) + 6/(-
     mstau22 + mu2) - (9*mA2)/(mstau12*mstau22 - mstau12*mu2) - (9*mH2)/(
     mstau12*mstau22 - mstau12*mu2) - 24*mH2*DeltaInv(mstau22,mH2,mstau12) -
     177*mstau22*DeltaInv(mstau22,mH2,mstau12) + (12*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 - (90*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /mH2 - (87*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (6*mH2*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) + (18*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau22 + mu2) + (9*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     mstau22 - mstau12*mu2) + (21*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     mstau12 + (9*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau12 + mu2) +
     (15*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau22 + mu2) - (18*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - (3*mA2)/power2(mstau12 - mu2) - (3*mH2)/power2(mstau12 -
     mu2) - (6*mstau22)/power2(mstau12 - mu2) - (4*mu2)/power2(mstau12 - mu2) -
     (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (18*
     mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*
     mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (3*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) +
     (12*mu2)/power2(mstau22 - mu2) + (18*mA2*mu2)/(mstau12*power2(mstau22 -
     mu2)) + (18*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mH2*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) - (9*mH2*mstau24*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (15*
     mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/power2(mstau22 - mu2) + (15*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (
     18*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22
     + mu2) + (2*power2(mu2))/(mstau12*power2(mstau12 - mu2)) + (12*power2(mu2)
     )/(mstau12*power2(mstau22 - mu2)) + (30*mH2*DeltaInv(mH2,mstau12,mu2)*
     power2(mu2))/power2(mstau22 - mu2) + (12*mH2*DeltaInv(mstau22,mH2,mstau12)
     *power2(mu2))/power2(mstau22 - mu2) + (6*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (15*DeltaInv(mH2,mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mA2,
     mstau12,mu2)*(mA2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mA2)
     + (-mstau12 + mu2)*power2(mu2) + 3*power3(mA2)))/(mstau12*power2(mstau22 -
     mu2)) + (9*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(
     mstau12 - mu2) - (9*mu2*DeltaInv(mH2,mstau12,mu2)*power3(mH2))/(mstau12*
     power2(mstau22 - mu2)) - (9*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))
     /(mstau12*power2(mstau22 - mu2)) + (4*power2(mu2))/power3(mstau12 - mu2) +
     (3*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/power2(mstau22 - mu2) - (3*mH2*
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) - (3*
     DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     54*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (3*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (3*
     DeltaInv(mA2,mstau22,mstau12)*(-(power3(mA2)*(power2(mu2)*(-14*mstau22*mu2
     + 7*power2(mstau22) + 2*power2(mu2)) + 5*power2(mstau12)*(-6*mstau22*mu2 +
     2*power2(mstau22) + 5*power2(mu2)) + 5*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*(-18*mu2*power2(mstau22) + 38*mstau22*power2(mu2) + power3(
     mstau22) - 16*power3(mu2)))) + (3*(mstau22 - 2*mu2)*power2(mstau12) + 3*(
     mstau22 - 2*mu2)*power2(mu2) + mstau12*(-8*mstau22*mu2 + power2(mstau22) +
     13*power2(mu2)))*power4(mA2) - power2(mA2)*(2*(-6*mstau22*mu2 + 3*power2(
     mstau22) - 2*power2(mu2))*power3(mstau12) + power2(mu2)*(-3*mstau22*
     mstau24 + 6*mstau24*mu2 - 8*mu2*power2(mstau22) + 2*mstau22*power2(mu2) +
     4*power3(mstau22)) + power2(mstau12)*(-26*mu2*power2(mstau22) + 6*mu2*(
     mstau24 + power2(mu2)) + mstau22*(-3*mstau24 + 32*power2(mu2)) + 6*power3(
     mstau22)) + mstau12*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*
     power2(mu2) + power2(mstau22)*(mstau24 + 16*power2(mu2)) - 4*mu2*power3(
     mstau22) - 2*power4(mu2))) + 2*power2(mstau12 - mu2)*power2(mstau22 - mu2)
     *(5*mstau12*mstau24 - 3*power6(mstau2)) + mA2*(mstau24*power2(mu2)*(-22*
     mstau22*mu2 + 11*power2(mstau22) + 10*power2(mu2)) + mstau22*(-42*mstau22*
     mu2 + 21*power2(mstau22) + 22*power2(mu2))*power3(mstau12) + power2(
     mstau12)*(7*mstau24*power2(mu2) + 8*power2(mstau22)*(mstau24 + 10*power2(
     mu2)) - 40*mu2*power3(mstau22) - 2*mstau22*(8*mstau24*mu2 + 21*power3(mu2)
     )) + mstau12*(mu2*(-18*mstau24*power2(mu2) - 2*power2(mstau22)*(10*mstau24
     + 19*power2(mu2)) + 19*mu2*power3(mstau22) + 20*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(8.*power2(cb)) + (mu2*log(
     mstau12/Q2)*log(mstau22/Q2)*power2(sa)*power2(sb)*(power2(mstau12 - mu2)*(
     mh2*mH2*(mstau22 - 2*mu2) - (mstau22 - 2*mu2)*power2(mH2) - 24*mstau22*
     power2(mstau22 - mu2)) - mH2*DeltaInv(mstau22,mstau12,mh2)*(mstau22*
     mstau24*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power2(mstau12 -
     mu2)*power3(mh2) + mstau22*(-2*mstau22*mu2 + power2(mstau22) + 2*power2(
     mu2))*power3(mstau12) + mh2*(-((mstau22 - 2*mu2)*(-mstau24 + 2*power2(
     mstau22))*power2(mu2)) - 6*mstau22*(mstau22 - 2*mu2)*power3(mstau12) +
     power2(mstau12)*(-2*mstau24*mu2 + 18*mu2*power2(mstau22) + mstau22*(
     mstau24 - 25*power2(mu2)) - 3*power3(mstau22)) + 2*mstau12*(mstau24*
     power2(mu2) + mstau22*mu2*(mstau24 + 5*power2(mu2)) - power2(mstau22)*(
     mstau24 + 5*power2(mu2)) + mu2*power3(mstau22))) - power2(mstau12)*(-4*
     mstau22*mstau24*mu2 + 2*mstau24*power2(mstau22) + 3*mstau24*power2(mu2) +
     2*mstau22*power3(mu2)) - power2(mh2)*(power2(mstau12)*power2(mstau22 - 2*
     mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power3(
     mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) + 2*power3(
     mu2))) + mstau12*(power2(mu2)*(2*mstau24*mu2 + 2*mu2*power2(mstau22) -
     power3(mstau22)) + power2(mstau22 - mu2)*power6(mstau2))) + DeltaInv(
     mstau22,mH2,mstau12)*(power2(mH2)*(-6*mstau22*(mstau22 - 2*mu2)*power3(
     mstau12) + power2(mstau12)*(-2*mstau24*mu2 - 30*mu2*power2(mstau22) +
     mstau22*(mstau24 - power2(mu2)) + 21*power3(mstau22)) + power2(mu2)*(-2*
     mstau24*mu2 - 44*mu2*power2(mstau22) + mstau22*(mstau24 + 24*power2(mu2))
     + 22*power3(mstau22)) + mstau12*(-2*power2(mstau22)*(mstau24 - 43*power2(
     mu2)) + 2*mstau22*mu2*(mstau24 - 19*power2(mu2)) + 2*mstau24*power2(mu2) -
     46*mu2*power3(mstau22))) - power3(mH2)*(power2(mstau12)*power2(mstau22 -
     2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power3(
     mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) + 2*power3(
     mu2))) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(mH2) + 24*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(mstau2))
     + mH2*(mstau24*(94*mstau22*mu2 - 47*power2(mstau22) - 48*power2(mu2))*
     power2(mu2) + mstau22*(94*mstau22*mu2 - 47*power2(mstau22) - 46*power2(
     mu2))*power3(mstau12) + power2(mstau12)*(-51*mstau24*power2(mu2) - 2*
     power2(mstau22)*(25*mstau24 + 96*power2(mu2)) + 96*mu2*power3(mstau22) +
     2*mstau22*(50*mstau24*mu2 + 47*power3(mu2))) + mstau12*(mu2*(98*mstau24*
     power2(mu2) + power2(mstau22)*(96*mstau24 + 98*power2(mu2)) - 49*mu2*
     power3(mstau22) - 48*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2))))))/(8.*mH2*mstau12*power2(cb)*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (ca*mu2*sa*sb*log(mstau12/Q2)*log(
     mstau22/Q2)*(power2(mstau12 - mu2)*(mh2*mH2*(mstau22 - 2*mu2) - (mstau22 -
     2*mu2)*power2(mH2) - 12*mstau22*power2(mstau22 - mu2)) - mH2*DeltaInv(
     mstau22,mstau12,mh2)*(mstau22*mstau24*(mstau22 - 2*mu2)*power2(mu2) + (
     mstau22 - 2*mu2)*power2(mstau12 - mu2)*power3(mh2) + mstau22*(-2*mstau22*
     mu2 + power2(mstau22) + 2*power2(mu2))*power3(mstau12) + mh2*(-((mstau22 -
     2*mu2)*(-mstau24 + 2*power2(mstau22))*power2(mu2)) - 6*mstau22*(mstau22 -
     2*mu2)*power3(mstau12) + power2(mstau12)*(-2*mstau24*mu2 + 18*mu2*power2(
     mstau22) + mstau22*(mstau24 - 25*power2(mu2)) - 3*power3(mstau22)) + 2*
     mstau12*(mstau24*power2(mu2) + mstau22*mu2*(mstau24 + 5*power2(mu2)) -
     power2(mstau22)*(mstau24 + 5*power2(mu2)) + mu2*power3(mstau22))) -
     power2(mstau12)*(-4*mstau22*mstau24*mu2 + 2*mstau24*power2(mstau22) + 3*
     mstau24*power2(mu2) + 2*mstau22*power3(mu2)) - power2(mh2)*(power2(
     mstau12)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) +
     (mstau22 - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*power2(mu2) +
     power3(mstau22) + 2*power3(mu2))) + mstau12*(power2(mu2)*(2*mstau24*mu2 +
     2*mu2*power2(mstau22) - power3(mstau22)) + power2(mstau22 - mu2)*power6(
     mstau2))) + DeltaInv(mstau22,mH2,mstau12)*(-(power3(mH2)*(power2(mstau12)*
     power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22
     - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(
     mstau22) + 2*power3(mu2)))) + power2(mH2)*(-6*mstau22*(mstau22 - 2*mu2)*
     power3(mstau12) + power2(mstau12)*(-2*mstau24*mu2 - 6*mu2*power2(mstau22)
     + mstau22*(mstau24 - 13*power2(mu2)) + 9*power3(mstau22)) + power2(mu2)*(-
     2*mstau24*mu2 - 20*mu2*power2(mstau22) + mstau22*(mstau24 + 12*power2(mu2)
     ) + 10*power3(mstau22)) - 2*mstau12*(power2(mstau22)*(mstau24 - 19*power2(
     mu2)) - mstau24*power2(mu2) + 11*mu2*power3(mstau22) + mstau22*(-(mstau24*
     mu2) + 7*power3(mu2)))) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(
     mH2) + 12*power2(mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24
     + power6(mstau2)) + mH2*(mstau24*(46*mstau22*mu2 - 23*power2(mstau22) -
     24*power2(mu2))*power2(mu2) + mstau22*(46*mstau22*mu2 - 23*power2(mstau22)
     - 22*power2(mu2))*power3(mstau12) + power2(mstau12)*(-27*mstau24*power2(
     mu2) - 2*power2(mstau22)*(13*mstau24 + 48*power2(mu2)) + 48*mu2*power3(
     mstau22) + mstau22*(52*mstau24*mu2 + 46*power3(mu2))) + mstau12*(mu2*(50*
     mstau24*power2(mu2) + power2(mstau22)*(48*mstau24 + 50*power2(mu2)) - 25*
     mu2*power3(mstau22) - 24*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2))))))/(4.*cb*mH2*mstau12*power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + (mu2*log(msntau2/Q2)*log(mstau22/Q2)*power2(
     sb)*((mstau22 - 2*mu2)*power2(mC2)*power2(msntau2 - mu2) + 2*mstau22*
     power2(msntau2 - mu2)*power2(mstau22 - mu2) + mC2*(power2(msntau2)*power2(
     mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*
     mu2)*power3(msntau2) + msntau2*(-4*mu2*power2(mstau22) + 6*mstau22*power2(
     mu2) + power3(mstau22) - 2*power3(mu2))) - DeltaInv(mstau22,mC2,msntau2)*(
     power2(mC2)*(power2(mu2)*(-2*mstau24*mu2 + mstau22*(mstau24 + 2*power2(
     mu2))) - 6*mstau22*(mstau22 - 2*mu2)*power3(msntau2) - power2(msntau2)*(-(
     mstau22*mstau24) + 2*mstau24*mu2 - 14*mu2*power2(mstau22) + 23*mstau22*
     power2(mu2) + power3(mstau22)) - 2*msntau2*(-(mstau24*power2(mu2)) +
     power2(mstau22)*(mstau24 + power2(mu2)) - mstau22*mu2*(mstau24 + 3*power2(
     mu2)) + mu2*power3(mstau22))) - power3(mC2)*(power2(msntau2)*power2(
     mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*
     mu2)*power3(msntau2) - msntau2*(-4*mstau22*power2(mu2) + power3(mstau22) +
     2*power3(mu2))) + (mstau22 - 2*mu2)*power2(msntau2 - mu2)*power4(mC2) + 2*
     power2(msntau2 - mu2)*power2(mstau22 - mu2)*(-2*msntau2*mstau24 + power6(
     mstau2)) + mC2*(mstau24*(6*mstau22*mu2 - 3*power2(mstau22) - 4*power2(mu2)
     )*power2(mu2) + mstau22*(6*mstau22*mu2 - 3*power2(mstau22) - 2*power2(mu2)
     )*power3(msntau2) + power2(msntau2)*(-7*mstau24*power2(mu2) - 2*power2(
     mstau22)*(3*mstau24 + 8*power2(mu2)) + 8*mu2*power3(mstau22) + 6*mstau22*(
     2*mstau24*mu2 + power3(mu2))) + msntau2*(mu2*(10*mstau24*power2(mu2) + 2*
     power2(mstau22)*(4*mstau24 + 5*power2(mu2)) - 5*mu2*power3(mstau22) - 4*
     mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(
     mstau2))))))/(4.*mC2*msntau2*power2(cb)*power2(msntau2 - mu2)*power2(
     mstau22 - mu2)) - (mu2*log(mstau12/Q2)*log(mstau22/Q2)*power2(sb)*(-2/
     mstau12 + (4*mstau22)/(mA2*mstau12) - (12*mstau22)/(mH2*mstau12) + 2/(-
     mstau22 + mu2) + mA2/(mstau12*(-mstau22 + mu2)) + mH2/(mstau12*(-mstau22 +
     mu2)) - (2*mstau22)/power2(mstau12 - mu2) + (2*mu2)/power2(mstau22 - mu2)
     + (mA2*mu2)/(mstau12*power2(mstau22 - mu2)) + (mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) + (2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     DeltaInv(mstau22,mH2,mstau12)*(-(power3(mH2)*(power2(mstau12)*power2(
     mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*
     mu2)*power3(mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) +
     2*power3(mu2)))) + power2(mH2)*(-6*mstau22*(mstau22 - 2*mu2)*power3(
     mstau12) + power2(mstau12)*(-2*mstau24*mu2 - 6*mu2*power2(mstau22) +
     mstau22*(mstau24 - 13*power2(mu2)) + 9*power3(mstau22)) + power2(mu2)*(-2*
     mstau24*mu2 - 20*mu2*power2(mstau22) + mstau22*(mstau24 + 12*power2(mu2))
     + 10*power3(mstau22)) - 2*mstau12*(power2(mstau22)*(mstau24 - 19*power2(
     mu2)) - mstau24*power2(mu2) + 11*mu2*power3(mstau22) + mstau22*(-(mstau24*
     mu2) + 7*power3(mu2)))) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(
     mH2) + 12*power2(mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24
     + power6(mstau2)) + mH2*(mstau24*(46*mstau22*mu2 - 23*power2(mstau22) -
     24*power2(mu2))*power2(mu2) + mstau22*(46*mstau22*mu2 - 23*power2(mstau22)
     - 22*power2(mu2))*power3(mstau12) + power2(mstau12)*(-27*mstau24*power2(
     mu2) - 2*power2(mstau22)*(13*mstau24 + 48*power2(mu2)) + 48*mu2*power3(
     mstau22) + mstau22*(52*mstau24*mu2 + 46*power3(mu2))) + mstau12*(mu2*(50*
     mstau24*power2(mu2) + power2(mstau22)*(48*mstau24 + 50*power2(mu2)) - 25*
     mu2*power3(mstau22) - 24*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2)))))/(mH2*mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2)) + (DeltaInv(mA2,mstau22,mstau12)*(power2(mA2)*(-6*
     mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(mstau12)*(-2*mstau24*
     mu2 + 26*mu2*power2(mstau22) + mstau22*(mstau24 - 29*power2(mu2)) - 7*
     power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 + 12*mu2*power2(mstau22) +
     mstau22*(mstau24 - 4*power2(mu2)) - 6*power3(mstau22)) + 2*mstau12*(
     mstau24*power2(mu2) + mstau22*mu2*(mstau24 + 9*power2(mu2)) - power2(
     mstau22)*(mstau24 + 13*power2(mu2)) + 5*mu2*power3(mstau22))) - power3(
     mA2)*(power2(mstau12)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*
     power2(mu2) + power3(mstau22) + 2*power3(mu2))) + (mstau22 - 2*mu2)*
     power2(mstau12 - mu2)*power4(mA2) - 4*power2(mstau12 - mu2)*power2(mstau22
     - mu2)*(-2*mstau12*mstau24 + power6(mstau2)) + mA2*(mstau24*power2(mu2)*(-
     18*mstau22*mu2 + 9*power2(mstau22) + 8*power2(mu2)) + mstau22*(-18*
     mstau22*mu2 + 9*power2(mstau22) + 10*power2(mu2))*power3(mstau12) +
     power2(mstau12)*(5*mstau24*power2(mu2) + power2(mstau22)*(6*mstau24 + 32*
     power2(mu2)) - 16*mu2*power3(mstau22) - 6*mstau22*(2*mstau24*mu2 + 3*
     power3(mu2))) + mstau12*(mu2*(-14*mstau24*power2(mu2) - 2*power2(mstau22)*
     (8*mstau24 + 7*power2(mu2)) + 7*mu2*power3(mstau22) + 8*mstau22*(4*
     mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(
     mA2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(8.*power2(cb))
     + (mu2*log(mstau12/Q2)*log(mstau22/Q2)*power2(sa)*((12*mstau22)/(mh2*
     mstau12) + (12*mstau22)/(mH2*mstau12) + mh2/(mstau12*(-mstau22 + mu2)) +
     mH2/(mstau12*mstau22 - mstau12*mu2) + (mh2*mu2)/(mstau12*power2(mstau22 -
     mu2)) - (mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,mH2,
     mstau12)*(-(power3(mH2)*(power2(mstau12)*power2(mstau22 - 2*mu2) +
     mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power3(mstau12)
     - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) + 2*power3(mu2)))) +
     power2(mH2)*(-6*mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(
     mstau12)*(-2*mstau24*mu2 - 6*mu2*power2(mstau22) + mstau22*(mstau24 - 13*
     power2(mu2)) + 9*power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 20*mu2*
     power2(mstau22) + mstau22*(mstau24 + 12*power2(mu2)) + 10*power3(mstau22))
     - 2*mstau12*(power2(mstau22)*(mstau24 - 19*power2(mu2)) - mstau24*power2(
     mu2) + 11*mu2*power3(mstau22) + mstau22*(-(mstau24*mu2) + 7*power3(mu2))))
     + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(mH2) + 12*power2(mstau12
     - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(mstau2)) + mH2*(
     mstau24*(46*mstau22*mu2 - 23*power2(mstau22) - 24*power2(mu2))*power2(mu2)
     + mstau22*(46*mstau22*mu2 - 23*power2(mstau22) - 22*power2(mu2))*power3(
     mstau12) + power2(mstau12)*(-27*mstau24*power2(mu2) - 2*power2(mstau22)*(
     13*mstau24 + 48*power2(mu2)) + 48*mu2*power3(mstau22) + mstau22*(52*
     mstau24*mu2 + 46*power3(mu2))) + mstau12*(mu2*(50*mstau24*power2(mu2) +
     power2(mstau22)*(48*mstau24 + 50*power2(mu2)) - 25*mu2*power3(mstau22) -
     24*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(
     mstau2)))))/(mH2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (
     DeltaInv(mstau22,mstau12,mh2)*(power2(mh2)*(-6*mstau22*(mstau22 - 2*mu2)*
     power3(mstau12) + power2(mstau12)*(-2*mstau24*mu2 + 42*mu2*power2(mstau22)
     + mstau22*(mstau24 - 37*power2(mu2)) - 15*power3(mstau22)) + power2(mu2)*(
     -2*mstau24*mu2 + 28*mu2*power2(mstau22) + mstau22*(mstau24 - 12*power2(
     mu2)) - 14*power3(mstau22)) + 2*mstau12*(mstau24*power2(mu2) + mstau22*
     mu2*(mstau24 + 17*power2(mu2)) - power2(mstau22)*(mstau24 + 29*power2(mu2)
     ) + 13*mu2*power3(mstau22))) - power3(mh2)*(power2(mstau12)*power2(mstau22
     - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*
     power3(mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) + 2*
     power3(mu2))) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(mh2) - 12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(
     mstau2)) + mh2*(mstau24*power2(mu2)*(-50*mstau22*mu2 + 25*power2(mstau22)
     + 24*power2(mu2)) + mstau22*(-50*mstau22*mu2 + 25*power2(mstau22) + 26*
     power2(mu2))*power3(mstau12) + power2(mstau12)*(21*mstau24*power2(mu2) +
     power2(mstau22)*(22*mstau24 + 96*power2(mu2)) - 48*mu2*power3(mstau22) -
     2*mstau22*(22*mstau24*mu2 + 25*power3(mu2))) + mstau12*(mu2*(-46*mstau24*
     power2(mu2) - 2*power2(mstau22)*(24*mstau24 + 23*power2(mu2)) + 23*mu2*
     power3(mstau22) + 24*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2)))))/(mh2*mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2))))/8. + (mu2*power2(sb)*(-10/mA2 + 10/mC2 + 30/mH2 +
     29/msntau2 + 43/mstau12 + 2/mstau22 + (28*mstau22)/(mC2*msntau2) - (28*
     mstau22)/(mA2*mstau12) + (84*mstau22)/(mH2*mstau12) + 18/(mstau22 - mu2) +
     5/(-msntau2 + mu2) + 5/(-mstau12 + mu2) + (28*mC2)/(msntau2*mstau22 -
     msntau2*mu2) + (14*mA2)/(mstau12*mstau22 - mstau12*mu2) + (14*mH2)/(
     mstau12*mstau22 - mstau12*mu2) + 28*mC2*DeltaInv(mstau22,mC2,msntau2) +
     84*mstau22*DeltaInv(mstau22,mC2,msntau2) + (28*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/msntau2 + (56*mstau24*DeltaInv(mstau22,mC2,msntau2))
     /mC2 + (28*mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (56*mC2*
     mstau22*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (42*mstau24*
     DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (56*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/(mstau22 - mu2) + (28*mC2*mu2*DeltaInv(mstau22,mC2,
     msntau2))/(-msntau2 + mu2) + (28*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2)
     )/(-msntau2 + mu2) - (28*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/(
     msntau2*mstau22 - msntau2*mu2) + 49*mH2*DeltaInv(mstau22,mH2,mstau12) +
     287*mstau22*DeltaInv(mstau22,mH2,mstau12) + (14*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 + (168*mstau24*DeltaInv(mstau22,mH2,mstau12)
     )/mH2 + (119*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (28*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (21*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (28*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(mstau22 - mu2) + (14*mH2*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(-mstau12 + mu2) + (14*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/(-mstau12 + mu2) - (14*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12*mstau22 - mstau12*mu2) - (28*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2))/msntau2 + (42*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2 -
     mu2) + (56*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(mstau22 - mu2) + (
     56*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2*mstau22 -
     msntau2*mu2) - (49*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (
     21*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (28*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (28*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2)
     - (2*mu2)/(-(msntau2*mu2) + power2(msntau2)) - (2*mu2)/(-(mstau12*mu2) +
     power2(mstau12)) - (4*mu2)/(-(mstau22*mu2) + power2(mstau22)) + (14*mC2)/
     power2(msntau2 - mu2) + (14*mstau22)/power2(msntau2 - mu2) - (3*mu2)/
     power2(msntau2 - mu2) - (28*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (
     14*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (56*
     mC2*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (
     14*mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (14*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) +
     (14*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) -
     (56*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (7*
     mA2)/power2(mstau12 - mu2) + (7*mH2)/power2(mstau12 - mu2) + (14*mstau22)/
     power2(mstau12 - mu2) - (3*mu2)/power2(mstau12 - mu2) - (14*mA2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (14*mH2*mu2)/(mstau22*power2(mstau12 -
     mu2)) + (7*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (28*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (7*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2)
     - (28*mu2*DeltaInv(mA2,mstau22,mu2)*power2(mA2))/power2(mstau12 - mu2) + (
     7*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2)
     + (7*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2)
     - (28*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (
     34*mu2)/power2(mstau22 - mu2) - (56*mC2*mu2)/(msntau2*power2(mstau22 -
     mu2)) - (28*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) - (28*mH2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (56*mC2*mstau22*mu2*DeltaInv(mstau22,mC2,
     msntau2))/power2(mstau22 - mu2) + (28*mC2*mstau24*mu2*DeltaInv(mstau22,
     mC2,msntau2))/(msntau2*power2(mstau22 - mu2)) - (28*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) + (14*mH2*mstau24*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) - (56*
     mu2*DeltaInv(mC2,msntau2,mu2)*power2(mC2))/power2(mstau22 - mu2) - (56*
     mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(mstau22 - mu2) - (
     56*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2*power2(
     mstau22 - mu2)) - (28*mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/power2(
     mstau22 - mu2) - (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     power2(mstau22 - mu2) - (28*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mC2,
     msntau2)*power2(mu2))/(mstau22 - mu2) + (7*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/(mstau22 - mu2) - (3*power2(mu2))/(msntau2*power2(msntau2 -
     mu2)) - (14*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (98*mC2*
     DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (42*
     DeltaInv(mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2
     - mu2)) - (3*power2(mu2))/(mstau12*power2(mstau12 - mu2)) - (14*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (49*mA2*DeltaInv(mA2,mstau22,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (49*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (21*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (21*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (28*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (28*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (6*power2(mu2))/(mstau22*power2(mstau22 - mu2)) -
     (98*mC2*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (
     49*mH2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) - (42*
     mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) - (
     28*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 -
     mu2) - (28*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (14*mstau24*DeltaInv(mstau22,mC2,msntau2)
     *power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (21*mH2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (14*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (14*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (7*mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (42*DeltaInv(mC2,msntau2,mu2)*power2(mC2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (21*DeltaInv(mH2,
     mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     7*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*
     power2(mstau12 - mu2)) + (7*mu2*DeltaInv(mA2,mstau12,mu2)*(-7*mA2*mstau12*
     mu2 - (4*mstau12 + 3*mu2)*power2(mA2) + (-mstau12 + mu2)*power2(mu2) + 2*
     power3(mA2)))/(mstau12*power2(mstau22 - mu2)) - (28*DeltaInv(mstau22,mC2,
     msntau2)*power3(mC2))/(msntau2*mstau22 - msntau2*mu2) - (14*DeltaInv(
     mstau22,mC2,msntau2)*power3(mC2))/power2(msntau2 - mu2) + (28*mu2*
     DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 - mu2)) + (
     28*mu2*DeltaInv(mC2,msntau2,mu2)*power3(mC2))/(msntau2*power2(mstau22 -
     mu2)) + (28*mu2*DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/(msntau2*
     power2(mstau22 - mu2)) - (14*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(
     mstau12*mstau22 - mstau12*mu2) - (7*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/power2(mstau12 - mu2) + (14*mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) + (14*mu2*DeltaInv(mH2,mstau12,mu2)*
     power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (14*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (14*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/power2(msntau2 - mu2) - (7*DeltaInv(mA2,
     mstau22,mu2)*power3(mu2))/power2(mstau12 - mu2) - (7*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/power2(mstau12 - mu2) - (14*DeltaInv(mC2,msntau2,mu2)*
     power3(mu2))/power2(mstau22 - mu2) - (7*DeltaInv(mH2,mstau12,mu2)*power3(
     mu2))/power2(mstau22 - mu2) + (14*DeltaInv(mstau22,mC2,msntau2)*power3(
     mu2))/power2(mstau22 - mu2) + (7*DeltaInv(mstau22,mH2,mstau12)*power3(mu2)
     )/power2(mstau22 - mu2) + (14*DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(
     mstau22*power2(msntau2 - mu2)) + (7*DeltaInv(mA2,mstau22,mu2)*power4(mu2))
     /(mstau22*power2(mstau12 - mu2)) + (7*DeltaInv(mstau22,mH2,mu2)*power4(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(mC2,msntau2,mu2)*
     power4(mu2))/(msntau2*power2(mstau22 - mu2)) + (7*DeltaInv(mH2,mstau12,
     mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) - (28*DeltaInv(mstau22,
     mC2,msntau2)*power6(mstau2))/(mC2*msntau2) - (84*DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/(mH2*mstau12) - (14*DeltaInv(mstau22,mC2,msntau2)
     *power6(mstau2))/power2(msntau2 - mu2) - (7*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/power2(mstau12 - mu2) - (7*DeltaInv(mA2,mstau22,mstau12)*(
     -(power3(mA2)*(power2(mu2)*(-10*mstau22*mu2 + 5*power2(mstau22) + 2*
     power2(mu2)) + power2(mstau12)*(-24*mstau22*mu2 + 8*power2(mstau22) + 21*
     power2(mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*(-14*mu2*
     power2(mstau22) + 29*mstau22*power2(mu2) + power3(mstau22) - 14*power3(
     mu2)))) + (2*(mstau22 - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22 -
     3*mu2) + 2*(mstau22 - 2*mu2)*power2(mu2))*power4(mA2) + power2(mA2)*(-2*(
     mstau22 - 2*mu2)*(-mstau24 + power2(mstau22))*power2(mu2) + (6*mstau22*mu2
     - 3*power2(mstau22) + 4*power2(mu2))*power3(mstau12) - 2*power2(mstau12)*(
     -(mstau22*mstau24) + 2*mstau24*mu2 - 10*mu2*power2(mstau22) + 10*mstau22*
     power2(mu2) + 3*power3(mstau22) + 3*power3(mu2)) + mstau12*(-2*mstau22*
     mu2*(mstau24 - 5*power2(mu2)) + 7*mstau24*power2(mu2) - power2(mstau22)*(
     mstau24 + 13*power2(mu2)) + 4*mu2*power3(mstau22) + 2*power4(mu2))) - 4*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(
     mstau2)) + mA2*(mstau24*power2(mu2)*(-14*mstau22*mu2 + 7*power2(mstau22) +
     6*power2(mu2)) + mstau22*(-30*mstau22*mu2 + 15*power2(mstau22) + 16*
     power2(mu2))*power3(mstau12) + power2(mstau12)*(3*mstau24*power2(mu2) + 4*
     power2(mstau22)*(mstau24 + 14*power2(mu2)) - 28*mu2*power3(mstau22) - 2*
     mstau22*(4*mstau24*mu2 + 15*power3(mu2))) + mstau12*(mu2*(-10*mstau24*
     power2(mu2) - 2*power2(mstau22)*(6*mstau24 + 13*power2(mu2)) + 13*mu2*
     power3(mstau22) + 2*mstau22*(12*mstau24*mu2 + 7*power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2))))/(8.*power2(cb)) + (mu2*power2(Pi)*power2(sb)*(-2/
     mA2 + 2/mC2 + 6/mH2 + 4/msntau2 + 6/mstau12 + (4*mstau22)/(mC2*msntau2) -
     (4*mstau22)/(mA2*mstau12) + (12*mstau22)/(mH2*mstau12) + 4/(mstau22 - mu2)
     + (4*mC2)/(msntau2*mstau22 - msntau2*mu2) + (2*mA2)/(mstau12*mstau22 -
     mstau12*mu2) + (2*mH2)/(mstau12*mstau22 - mstau12*mu2) + 4*mC2*DeltaInv(
     mstau22,mC2,msntau2) + 12*mstau22*DeltaInv(mstau22,mC2,msntau2) + (4*mC2*
     mstau22*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (8*mstau24*DeltaInv(
     mstau22,mC2,msntau2))/mC2 + (4*mstau24*DeltaInv(mstau22,mC2,msntau2))/
     msntau2 + (8*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) +
     (6*mstau24*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (8*mC2*
     mstau22*DeltaInv(mstau22,mC2,msntau2))/(mstau22 - mu2) + (4*mC2*mu2*
     DeltaInv(mstau22,mC2,msntau2))/(-msntau2 + mu2) + (4*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2))/(-msntau2 + mu2) - (4*mC2*mstau24*DeltaInv(mstau22,
     mC2,msntau2))/(msntau2*mstau22 - msntau2*mu2) + 7*mH2*DeltaInv(mstau22,
     mH2,mstau12) + 41*mstau22*DeltaInv(mstau22,mH2,mstau12) + (2*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (24*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/mH2 + (17*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (4*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (4*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(mstau22 - mu2) + (2*mH2*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) - (2*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12*mstau22 - mstau12*mu2) - (4*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2))/msntau2 + (6*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2 -
     mu2) + (8*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(mstau22 - mu2) + (8*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2*mstau22 -
     msntau2*mu2) - (7*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (3*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (4*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (4*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (2*
     mC2)/power2(msntau2 - mu2) + (2*mstau22)/power2(msntau2 - mu2) - (2*mu2)/
     power2(msntau2 - mu2) - (4*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (2*
     mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (8*mC2*
     mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*
     mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) +
     (2*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) -
     (8*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + mA2/
     power2(mstau12 - mu2) + mH2/power2(mstau12 - mu2) + (2*mstau22)/power2(
     mstau12 - mu2) - (2*mu2)/power2(mstau12 - mu2) - (2*mA2*mu2)/(mstau22*
     power2(mstau12 - mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (
     mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (4*mH2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (
     mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (4*mu2*
     DeltaInv(mA2,mstau22,mu2)*power2(mA2))/power2(mstau12 - mu2) + (mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) + (mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (4*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (8*mu2)/
     power2(mstau22 - mu2) - (8*mC2*mu2)/(msntau2*power2(mstau22 - mu2)) - (4*
     mA2*mu2)/(mstau12*power2(mstau22 - mu2)) - (4*mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (8*mC2*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/
     power2(mstau22 - mu2) + (4*mC2*mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/
     (msntau2*power2(mstau22 - mu2)) - (4*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau22 - mu2) + (2*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12*power2(mstau22 - mu2)) - (8*mu2*DeltaInv(mC2,msntau2,
     mu2)*power2(mC2))/power2(mstau22 - mu2) - (8*mu2*DeltaInv(mstau22,mC2,
     msntau2)*power2(mC2))/power2(mstau22 - mu2) - (8*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2)*power2(mC2))/(msntau2*power2(mstau22 - mu2)) - (4*
     mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/power2(mstau22 - mu2) - (4*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) - (4*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(mstau22 -
     mu2) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 - mu2) - (2*
     power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (14*mC2*DeltaInv(mstau22,
     mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (6*DeltaInv(mstau22,mC2,mu2)
     *power2(mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (2*power2(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (7*mA2*DeltaInv(mA2,mstau22,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (7*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (3*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (4*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (4*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (14*mC2*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/
     power2(mstau22 - mu2) - (7*mH2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/
     power2(mstau22 - mu2) - (6*mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/
     power2(mstau22 - mu2) - (4*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(
     mu2))/power2(mstau22 - mu2) - (4*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)
     *power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (2*mstau24*DeltaInv(
     mstau22,mC2,msntau2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (3*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) -
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (6*DeltaInv(mC2,msntau2,mu2)*power2(mC2)*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mC2,
     msntau2)*power2(mC2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (3*
     DeltaInv(mH2,mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (2*mu2*DeltaInv(mA2,mstau22,mu2)*power3(
     mA2))/(mstau22*power2(mstau12 - mu2)) + (mu2*DeltaInv(mA2,mstau12,mu2)*(-
     7*mA2*mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mA2) + (-mstau12 + mu2)*
     power2(mu2) + 2*power3(mA2)))/(mstau12*power2(mstau22 - mu2)) - (4*
     DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/(msntau2*mstau22 - msntau2*mu2)
     - (2*DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/power2(msntau2 - mu2) + (
     4*mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 -
     mu2)) + (4*mu2*DeltaInv(mC2,msntau2,mu2)*power3(mC2))/(msntau2*power2(
     mstau22 - mu2)) + (4*mu2*DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/(
     msntau2*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/power2(mstau12 - mu2) + (2*mu2*DeltaInv(mstau22,mH2,mu2)*
     power3(mH2))/(mstau22*power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mH2,
     mstau12,mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (2*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2))
     - (2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/power2(msntau2 - mu2) - (
     DeltaInv(mA2,mstau22,mu2)*power3(mu2))/power2(mstau12 - mu2) - (DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mC2,
     msntau2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (DeltaInv(mH2,mstau12,
     mu2)*power3(mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mC2,msntau2)
     *power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,mH2,mstau12)*
     power3(mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mC2,mu2)*power4(
     mu2))/(mstau22*power2(msntau2 - mu2)) + (DeltaInv(mA2,mstau22,mu2)*power4(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mH2,mu2)*power4(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (2*DeltaInv(mC2,msntau2,mu2)*
     power4(mu2))/(msntau2*power2(mstau22 - mu2)) + (DeltaInv(mH2,mstau12,mu2)*
     power4(mu2))/(mstau12*power2(mstau22 - mu2)) - (4*DeltaInv(mstau22,mC2,
     msntau2)*power6(mstau2))/(mC2*msntau2) - (12*DeltaInv(mstau22,mH2,mstau12)
     *power6(mstau2))/(mH2*mstau12) - (2*DeltaInv(mstau22,mC2,msntau2)*power6(
     mstau2))/power2(msntau2 - mu2) - (DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) - (DeltaInv(mA2,mstau22,mstau12)*(-(power3(
     mA2)*(power2(mu2)*(-10*mstau22*mu2 + 5*power2(mstau22) + 2*power2(mu2)) +
     power2(mstau12)*(-24*mstau22*mu2 + 8*power2(mstau22) + 21*power2(mu2)) +
     4*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*(-14*mu2*power2(mstau22) +
     29*mstau22*power2(mu2) + power3(mstau22) - 14*power3(mu2)))) + (2*(mstau22
     - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22 - 3*mu2) + 2*(mstau22 -
     2*mu2)*power2(mu2))*power4(mA2) + power2(mA2)*(-2*(mstau22 - 2*mu2)*(-
     mstau24 + power2(mstau22))*power2(mu2) + (6*mstau22*mu2 - 3*power2(
     mstau22) + 4*power2(mu2))*power3(mstau12) - 2*power2(mstau12)*(-(mstau22*
     mstau24) + 2*mstau24*mu2 - 10*mu2*power2(mstau22) + 10*mstau22*power2(mu2)
     + 3*power3(mstau22) + 3*power3(mu2)) + mstau12*(-2*mstau22*mu2*(mstau24 -
     5*power2(mu2)) + 7*mstau24*power2(mu2) - power2(mstau22)*(mstau24 + 13*
     power2(mu2)) + 4*mu2*power3(mstau22) + 2*power4(mu2))) - 4*power2(mstau12
     - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(mstau2)) + mA2*(
     mstau24*power2(mu2)*(-14*mstau22*mu2 + 7*power2(mstau22) + 6*power2(mu2))
     + mstau22*(-30*mstau22*mu2 + 15*power2(mstau22) + 16*power2(mu2))*power3(
     mstau12) + power2(mstau12)*(3*mstau24*power2(mu2) + 4*power2(mstau22)*(
     mstau24 + 14*power2(mu2)) - 28*mu2*power3(mstau22) - 2*mstau22*(4*mstau24*
     mu2 + 15*power3(mu2))) + mstau12*(mu2*(-10*mstau24*power2(mu2) - 2*power2(
     mstau22)*(6*mstau24 + 13*power2(mu2)) + 13*mu2*power3(mstau22) + 2*
     mstau22*(12*mstau24*mu2 + 7*power3(mu2))) + power2(mstau22 - mu2)*power6(
     mstau2)))))/(mA2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(
     48.*power2(cb)) + xtau*(((-(1/mA2) + 1/mC2 + 3/mH2)*mu*sb)/(4.*cb) - (ca*(
     mh2 - mH2)*mu*sa*Fin20(mstau12,mu2,Q2))/(2.*mh2*mH2*(mstau12 - mstau22)) +
     (mu*(mH2*(mstau12 - mu2) + mA2*(mstau22 - mu2))*sb*Fin20(mstau12,mu2,Q2))/
     (2.*cb*mA2*mH2*(mstau12 - mstau22)*(mstau22 - mu2)) - (mu*(mstau12 +
     mstau22 - 2*mu2)*sb*Fin20(mstau22,mstau12,Q2))/(2.*cb*mA2*(mstau12 - mu2)*
     (-mstau22 + mu2)) + (ca*(mh2 - mH2)*mu*sa*Fin20(mstau22,mu2,Q2))/(2.*mh2*
     mH2*(mstau12 - mstau22)) - (mu*(mA2*(mstau12 - mu2) + mH2*(mstau22 - mu2))
     *sb*Fin20(mstau22,mu2,Q2))/(2.*cb*mA2*mH2*(mstau12 - mstau22)*(mstau12 -
     mu2)) - (mu*sa*(ca*cb + sa*sb)*Fin3(mstau12,mstau12,mh2,Q2))/(2.*cb*(
     mstau12 - mstau22)*(mstau12 - mu2)) + (mu*sa*(ca*cb + sa*sb)*Fin3(mstau22,
     mstau22,mh2,Q2))/(2.*cb*(mstau12 - mstau22)*(mstau22 - mu2)) + ((mC2*mH2 -
     mA2*(3*mC2 + mH2))*mstau12*mu*sb*log(mstau12/Q2))/(2.*cb*mA2*mC2*mH2*(
     mstau12 - mstau22)) + ((-(mC2*mH2) + mA2*(3*mC2 + mH2))*mstau12*mu*sb*log(
     mstau12/Q2))/(4.*cb*mA2*mC2*mH2*(mstau12 - mstau22)) - (mstau12*mu*sb*log(
     mA2/Q2)*log(mstau12/Q2))/(2.*cb*(mstau12 - mstau22)*(mstau12 - mu2)) + (
     mstau12*mu*sb*log(mH2/Q2)*log(mstau12/Q2))/(2.*cb*(mstau12 - mstau22)*(
     mstau12 - mu2)) + ((-(mC2*mH2) + mA2*(3*mC2 + mH2))*mstau22*mu*sb*log(
     mstau22/Q2))/(4.*cb*mA2*mC2*mH2*(mstau12 - mstau22)) + (mstau22*mu*sb*log(
     mA2/Q2)*log(mstau22/Q2))/(2.*cb*(mstau12 - mstau22)*(mstau22 - mu2)) - (
     mstau22*mu*sb*log(mH2/Q2)*log(mstau22/Q2))/(2.*cb*(mstau12 - mstau22)*(
     mstau22 - mu2)) + (mu*mu2*sb*log(mA2/Q2)*log(mu2/Q2))/(2.*cb*(mstau12 -
     mu2)*(-mstau22 + mu2)) - (mu*mu2*sb*log(mH2/Q2)*log(mu2/Q2))/(2.*cb*(
     mstau12 - mu2)*(-mstau22 + mu2)) - (mu*sb*Fin20(mstau22,msntau2,Q2))/(2.*
     cb*power2(mC2)) + (3*ca*(mh2 - mH2)*mu*sa*Fin20(mstau22,mstau12,Q2))/(2.*
     mh2*power2(mH2)) + (mu*sb*Fin20(mstau22,mstau12,Q2)*(3*(mstau12 - mu2)*(
     mstau22 - mu2)*power2(mA2) + 2*mA2*(mstau12 + mstau22 - 2*mu2)*power2(mH2)
     + (mstau12 - mu2)*(-mstau22 + mu2)*power2(mH2)))/(2.*cb*(mstau12 - mu2)*(-
     mstau22 + mu2)*power2(mA2)*power2(mH2)) - (mC2*mu*mu2*sb*log(mC2/Q2)*log(
     mu2/Q2)*(4*msntau2*mstau22*mu2 - mstau22*power2(msntau2) - msntau2*power2(
     mstau22) + mstau22*DeltaInv(mC2,msntau2,mu2)*(mu2*(-3*msntau2 + mu2) -
     mC2*(msntau2 + 2*mu2) + power2(mC2))*power2(msntau2 - mu2) + msntau2*
     DeltaInv(mstau22,mC2,mu2)*(mu2*(-3*mstau22 + mu2) - mC2*(mstau22 + 2*mu2)
     + power2(mC2))*power2(mstau22 - mu2) - msntau2*power2(mu2) - mstau22*
     power2(mu2)))/(2.*cb*msntau2*mstau22*power2(msntau2 - mu2)*power2(mstau22
     - mu2)) + (mu*mu2*sb*log(mH2/Q2)*log(mu2/Q2)*(-4*mH2*mstau12*mstau22*mu2 +
     mH2*mstau22*power2(mstau12) + 4*mstau22*mu2*power2(mstau12) + mH2*mstau12*
     power2(mstau22) + 4*mstau12*mu2*power2(mstau22) - 4*power2(mstau12)*
     power2(mstau22) - mH2*mstau22*DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*mstau12 +
     mu2) - mH2*(mstau12 + 2*mu2) + power2(mH2))*power2(mstau12 - mu2) - mH2*
     mstau12*DeltaInv(mstau22,mH2,mu2)*(mu2*(-3*mstau22 + mu2) - mH2*(mstau22 +
     2*mu2) + power2(mH2))*power2(mstau22 - mu2) + mH2*mstau12*power2(mu2) +
     mH2*mstau22*power2(mu2) - 4*mstau12*mstau22*power2(mu2)))/(4.*cb*mstau12*
     mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (ca*mu*mu2*sa*log(
     mh2/Q2)*log(mu2/Q2)*(-4*mh2*mstau12*mstau22*mu2 + mh2*mstau22*power2(
     mstau12) + 2*mstau22*mu2*power2(mstau12) + mh2*mstau12*power2(mstau22) +
     2*mstau12*mu2*power2(mstau22) - 2*power2(mstau12)*power2(mstau22) - mh2*
     mstau22*DeltaInv(mstau12,mh2,mu2)*(mu2*(-3*mstau12 + mu2) - mh2*(mstau12 +
     2*mu2) + power2(mh2))*power2(mstau12 - mu2) - mh2*mstau12*DeltaInv(
     mstau22,mh2,mu2)*(mu2*(-3*mstau22 + mu2) - mh2*(mstau22 + 2*mu2) + power2(
     mh2))*power2(mstau22 - mu2) + mh2*mstau12*power2(mu2) + mh2*mstau22*
     power2(mu2) - 2*mstau12*mstau22*power2(mu2)))/(4.*mstau12*mstau22*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) - (ca*mu*mu2*sa*log(mH2/Q2)*log(mu2/
     Q2)*(-4*mH2*mstau12*mstau22*mu2 + mH2*mstau22*power2(mstau12) + 2*mstau22*
     mu2*power2(mstau12) + mH2*mstau12*power2(mstau22) + 2*mstau12*mu2*power2(
     mstau22) - 2*power2(mstau12)*power2(mstau22) - mH2*mstau22*DeltaInv(mH2,
     mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mH2*(mstau12 + 2*mu2) + power2(mH2)
     )*power2(mstau12 - mu2) - mH2*mstau12*DeltaInv(mstau22,mH2,mu2)*(mu2*(-3*
     mstau22 + mu2) - mH2*(mstau22 + 2*mu2) + power2(mH2))*power2(mstau22 -
     mu2) + mH2*mstau12*power2(mu2) + mH2*mstau22*power2(mu2) - 2*mstau12*
     mstau22*power2(mu2)))/(4.*mstau12*mstau22*power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) + (mu*mu2*sb*log(mA2/Q2)*log(mu2/Q2)*(-4*mA2*mstau12*
     mstau22*mu2 + mA2*mstau22*power2(mstau12) - 4*mstau22*mu2*power2(mstau12)
     + mA2*mstau12*power2(mstau22) - 4*mstau12*mu2*power2(mstau22) + 4*power2(
     mstau12)*power2(mstau22) - mA2*mstau22*DeltaInv(mA2,mstau12,mu2)*(mu2*(-3*
     mstau12 + mu2) - mA2*(mstau12 + 2*mu2) + power2(mA2))*power2(mstau12 -
     mu2) - mA2*mstau12*DeltaInv(mA2,mstau22,mu2)*(mu2*(-3*mstau22 + mu2) -
     mA2*(mstau22 + 2*mu2) + power2(mA2))*power2(mstau22 - mu2) + mA2*mstau12*
     power2(mu2) + mA2*mstau22*power2(mu2) + 4*mstau12*mstau22*power2(mu2)))/(
     4.*cb*mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (3*
     ca*mu*sa*log(mu2/Q2)*power2(mu2)*(mstau22*DeltaInv(mstau12,mh2,mu2)*((
     mstau12 - mu2)*mu2 + mh2*(3*mstau12 + 2*mu2) - power2(mh2))*power2(mstau12
     - mu2) + mstau22*DeltaInv(mH2,mstau12,mu2)*power2(mstau12 - mu2)*(-3*mH2*
     mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)) + mstau12*
     power2(mstau22 - mu2)*(-(DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*
     mh2*mu2 - mstau22*mu2 + power2(mh2) + power2(mu2))) + DeltaInv(mstau22,
     mH2,mu2)*(-3*mH2*mstau22 - 2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(
     mu2)))))/(4.*mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2))
     + (mu*Fin3(mH2,mstau12,mstau12,Q2)*(ca*cb*sa + sb*(-1 + power2(sa))))/(2.*
     cb*(mstau12 - mstau22)*(mstau12 - mu2)) - (mu*Fin3(mstau22,mstau22,mH2,Q2)
     *(ca*cb*sa + sb*(-1 + power2(sa))))/(2.*cb*(mstau12 - mstau22)*(mstau22 -
     mu2)) - (3*mu*sb*power2(sa))/(2.*cb*mH2) - ((mh2 - mH2)*mu*sb*Fin20(
     mstau12,mu2,Q2)*power2(sa))/(2.*cb*mh2*mH2*(mstau12 - mstau22)) + ((mh2 -
     mH2)*mu*sb*Fin20(mstau22,mu2,Q2)*power2(sa))/(2.*cb*mh2*mH2*(mstau12 -
     mstau22)) + (3*mu*sb*log(mh2/Q2)*power2(sa))/(2*cb*mh2 - 2*cb*mH2) - (3*
     mu*sb*log(mH2/Q2)*power2(sa))/(2*cb*mh2 - 2*cb*mH2) + (3*mstau12*mu*sb*
     log(mstau12/Q2)*power2(sa))/(2*cb*mH2*mstau12 - 2*cb*mH2*mstau22) - (3*
     mstau12*mu*sb*log(mh2/Q2)*log(mstau12/Q2)*power2(sa))/(2.*cb*(mh2 - mH2)*(
     mstau12 - mstau22)) + (mstau12*mu*sb*log(mh2/Q2)*log(mstau12/Q2)*power2(
     sa))/(2.*cb*(mstau12 - mstau22)*(mstau12 - mu2)) + (3*mstau12*mu*sb*log(
     mH2/Q2)*log(mstau12/Q2)*power2(sa))/(2.*cb*(mh2 - mH2)*(mstau12 - mstau22)
     ) - (mstau12*mu*sb*log(mH2/Q2)*log(mstau12/Q2)*power2(sa))/(2.*cb*(mstau12
     - mstau22)*(mstau12 - mu2)) - (3*mstau22*mu*sb*log(mstau22/Q2)*power2(sa))
     /(2*cb*mH2*mstau12 - 2*cb*mH2*mstau22) + (3*mstau22*mu*sb*log(mh2/Q2)*log(
     mstau22/Q2)*power2(sa))/(2.*cb*(mh2 - mH2)*(mstau12 - mstau22)) - (
     mstau22*mu*sb*log(mh2/Q2)*log(mstau22/Q2)*power2(sa))/(2.*cb*(mstau12 -
     mstau22)*(mstau22 - mu2)) - (3*mstau22*mu*sb*log(mH2/Q2)*log(mstau22/Q2)*
     power2(sa))/(2.*cb*(mh2 - mH2)*(mstau12 - mstau22)) + (mstau22*mu*sb*log(
     mH2/Q2)*log(mstau22/Q2)*power2(sa))/(2.*cb*(mstau12 - mstau22)*(mstau22 -
     mu2)) - (mu*mu2*sb*log(mh2/Q2)*log(mu2/Q2)*power2(sa))/(2.*cb*(mstau12 -
     mu2)*(-mstau22 + mu2)) + (mu*mu2*sb*log(mH2/Q2)*log(mu2/Q2)*power2(sa))/(
     2.*cb*(mstau12 - mu2)*(-mstau22 + mu2)) + (3*(mh2 - mH2)*mu*sb*Fin20(
     mstau22,mstau12,Q2)*power2(sa))/(cb*mh2*power2(mH2)) + (mu*mu2*sb*log(mh2/
     Q2)*log(mu2/Q2)*(-4*mh2*mstau12*mstau22*mu2 + mh2*mstau22*power2(mstau12)
     + 4*mstau22*mu2*power2(mstau12) + mh2*mstau12*power2(mstau22) + 4*mstau12*
     mu2*power2(mstau22) - 4*power2(mstau12)*power2(mstau22) - mh2*mstau22*
     DeltaInv(mstau12,mh2,mu2)*(mu2*(-3*mstau12 + mu2) - mh2*(mstau12 + 2*mu2)
     + power2(mh2))*power2(mstau12 - mu2) - mh2*mstau12*DeltaInv(mstau22,mh2,
     mu2)*(mu2*(-3*mstau22 + mu2) - mh2*(mstau22 + 2*mu2) + power2(mh2))*
     power2(mstau22 - mu2) + mh2*mstau12*power2(mu2) + mh2*mstau22*power2(mu2)
     - 4*mstau12*mstau22*power2(mu2))*power2(sa))/(4.*cb*mstau12*mstau22*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (mu*mu2*sb*log(mH2/Q2)*log(
     mu2/Q2)*(-4*mH2*mstau12*mstau22*mu2 + mH2*mstau22*power2(mstau12) + 4*
     mstau22*mu2*power2(mstau12) + mH2*mstau12*power2(mstau22) + 4*mstau12*mu2*
     power2(mstau22) - 4*power2(mstau12)*power2(mstau22) - mH2*mstau22*
     DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mH2*(mstau12 + 2*mu2)
     + power2(mH2))*power2(mstau12 - mu2) - mH2*mstau12*DeltaInv(mstau22,mH2,
     mu2)*(mu2*(-3*mstau22 + mu2) - mH2*(mstau22 + 2*mu2) + power2(mH2))*
     power2(mstau22 - mu2) + mH2*mstau12*power2(mu2) + mH2*mstau22*power2(mu2)
     - 4*mstau12*mstau22*power2(mu2))*power2(sa))/(4.*cb*mstau12*mstau22*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (3*mu*sb*log(mu2/Q2)*
     power2(mu2)*(mstau22*DeltaInv(mstau12,mh2,mu2)*((mstau12 - mu2)*mu2 + mh2*
     (3*mstau12 + 2*mu2) - power2(mh2))*power2(mstau12 - mu2) + mstau22*
     DeltaInv(mH2,mstau12,mu2)*power2(mstau12 - mu2)*(-3*mH2*mstau12 - 2*mH2*
     mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)) + mstau12*power2(mstau22 -
     mu2)*(-(DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*
     mu2 + power2(mh2) + power2(mu2))) + DeltaInv(mstau22,mH2,mu2)*(-3*mH2*
     mstau22 - 2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(mu2))))*power2(
     sa))/(4.*cb*mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) -
     (ca*mu*sa*power2(mu2)*(mstau22*DeltaInv(mstau12,mh2,mu2)*((mstau12 - mu2)*
     mu2 + mh2*(3*mstau12 + 2*mu2) - power2(mh2))*power2(mstau12 - mu2) +
     mstau22*DeltaInv(mH2,mstau12,mu2)*power2(mstau12 - mu2)*(-3*mH2*mstau12 -
     2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)) + mstau12*power2(
     mstau22 - mu2)*(-(DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 -
     mstau22*mu2 + power2(mh2) + power2(mu2))) + DeltaInv(mstau22,mH2,mu2)*(-3*
     mH2*mstau22 - 2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(mu2))))*
     power2(log(mu2/Q2)))/(8.*mstau12*mstau22*power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) - (mu*sb*power2(mu2)*(mstau22*DeltaInv(mstau12,mh2,mu2)*((
     mstau12 - mu2)*mu2 + mh2*(3*mstau12 + 2*mu2) - power2(mh2))*power2(mstau12
     - mu2) + mstau22*DeltaInv(mH2,mstau12,mu2)*power2(mstau12 - mu2)*(-3*mH2*
     mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)) + mstau12*
     power2(mstau22 - mu2)*(-(DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*
     mh2*mu2 - mstau22*mu2 + power2(mh2) + power2(mu2))) + DeltaInv(mstau22,
     mH2,mu2)*(-3*mH2*mstau22 - 2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(
     mu2))))*power2(sa)*power2(log(mu2/Q2)))/(8.*cb*mstau12*mstau22*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) - (mu*mu2*sb*log(msntau2/Q2)*log(
     mu2/Q2)*(mC2 + msntau2 + mu2 + DeltaInv(mC2,msntau2,mu2)*(mC2*mu2*(6*
     msntau2 + mu2) + (msntau2 + mu2)*power2(mC2) + (msntau2 - mu2)*power2(mu2)
     - power3(mC2))))/(2.*cb*msntau2*power2(mstau22 - mu2)) - (3*ca*mu*sa*log(
     mh2/Q2)*(2/(-mh2 + mH2) + mh2/(mstau12*mstau22 - mstau12*mu2) - 3*mh2*
     DeltaInv(mstau22,mstau12,mh2) + (3*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12 - mu2) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(
     mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 +
     mu2) + (mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12*(-mstau22 +
     mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12 - mu2) + (
     3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (2*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2)
     + mh2/power2(mstau12 - mu2) - (mh2*mu2)/(mstau22*power2(mstau12 - mu2)) -
     (mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mh2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) +
     (mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (
     2*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2))
     + (mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) +
     (mh2*mu2*DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*
     mu2 + power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*
     mu2*DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))))/4. - (
     3*mu*sb*log(mh2/Q2)*power2(sa)*(4/(-mh2 + mH2) + mh2/(mstau12*mstau22 -
     mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + (3*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 - mu2) - (mh2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))))/(4.*
     cb) + (ca*mu*sa*power2(log(mh2/Q2))*(6/(-mh2 + mH2) + mh2/(mstau12*mstau22
     - mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + (3*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 - mu2) - (mh2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))))/8. + (
     mu*sb*power2(sa)*power2(log(mh2/Q2))*(12/(-mh2 + mH2) + mh2/(mstau12*
     mstau22 - mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + (3*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(
     mstau22,mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 - mu2) -
     (mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))))/(8.*
     cb) - (mu*mu2*sb*log(mstau12/Q2)*log(mu2/Q2)*(mA2 + mH2 + 2*mstau12 + 2*
     mu2 + DeltaInv(mA2,mstau12,mu2)*(mA2*mu2*(6*mstau12 + mu2) + (mstau12 +
     mu2)*power2(mA2) + (mstau12 - mu2)*power2(mu2) - power3(mA2)) + DeltaInv(
     mH2,mstau12,mu2)*(mH2*mu2*(6*mstau12 + mu2) + (mstau12 + mu2)*power2(mH2)
     + (mstau12 - mu2)*power2(mu2) - power3(mH2))))/(4.*cb*mstau12*power2(
     mstau22 - mu2)) + (ca*mu*mu2*sa*log(mstau12/Q2)*log(mu2/Q2)*(-mh2 + mH2 +
     DeltaInv(mstau12,mh2,mu2)*(-(mh2*mu2*(6*mstau12 + mu2)) - (mstau12 + mu2)*
     power2(mh2) + (-mstau12 + mu2)*power2(mu2) + power3(mh2)) + DeltaInv(mH2,
     mstau12,mu2)*(mH2*mu2*(6*mstau12 + mu2) + (mstau12 + mu2)*power2(mH2) + (
     mstau12 - mu2)*power2(mu2) - power3(mH2))))/(4.*mstau12*power2(mstau22 -
     mu2)) + (mu*mu2*sb*log(mstau12/Q2)*log(mu2/Q2)*power2(sa)*(-mh2 + mH2 +
     DeltaInv(mstau12,mh2,mu2)*(-(mh2*mu2*(6*mstau12 + mu2)) - (mstau12 + mu2)*
     power2(mh2) + (-mstau12 + mu2)*power2(mu2) + power3(mh2)) + DeltaInv(mH2,
     mstau12,mu2)*(mH2*mu2*(6*mstau12 + mu2) + (mstau12 + mu2)*power2(mH2) + (
     mstau12 - mu2)*power2(mu2) - power3(mH2))))/(4.*cb*mstau12*power2(mstau22
     - mu2)) + (ca*mu*mu2*sa*log(mstau22/Q2)*log(mu2/Q2)*(-mh2 + mH2 +
     DeltaInv(mstau22,mh2,mu2)*(-(mh2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*
     power2(mh2) + (-mstau22 + mu2)*power2(mu2) + power3(mh2)) + DeltaInv(
     mstau22,mH2,mu2)*(mH2*mu2*(6*mstau22 + mu2) + (mstau22 + mu2)*power2(mH2)
     + (mstau22 - mu2)*power2(mu2) - power3(mH2))))/(4.*mstau22*power2(mstau12
     - mu2)) + (mu*mu2*sb*log(mstau22/Q2)*log(mu2/Q2)*power2(sa)*(-mh2 + mH2 +
     DeltaInv(mstau22,mh2,mu2)*(-(mh2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*
     power2(mh2) + (-mstau22 + mu2)*power2(mu2) + power3(mh2)) + DeltaInv(
     mstau22,mH2,mu2)*(mH2*mu2*(6*mstau22 + mu2) + (mstau22 + mu2)*power2(mH2)
     + (mstau22 - mu2)*power2(mu2) - power3(mH2))))/(4.*cb*mstau22*power2(
     mstau12 - mu2)) - (ca*mu*sa*log(mh2/Q2)*log(mstau22/Q2)*(-2/(mstau12 -
     mstau22) + (2*mu2)/((mstau12 - mstau22)*(-mstau22 + mu2)) + mh2/(mstau12*
     mstau22 - mstau12*mu2) + (2*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (2*
     mh2*mu2*DeltaInv(mstau22,mh2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mh2*(mstau22
     + mu2) + power2(mh2)))/(mstau22*power2(mstau12 - mu2)) - (mh2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (mh2*DeltaInv(mstau22,mstau12,mh2)*(-(
     mh2*(mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mstau12 - mu2)) + (
     mstau22 - 2*mu2)*power2(mh2)*power2(mstau12 - mu2) + mstau24*(mstau22 - 2*
     mu2)*power2(mu2) - 3*mstau22*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*
     mu2*(-2*mstau22*mstau24 + 4*mstau24*mu2 + 5*mu2*power2(mstau22) + 2*
     mstau22*power2(mu2) - 4*power3(mstau22)) + power2(mstau12)*(mstau22*
     mstau24 - 2*mstau24*mu2 + 2*mu2*power2(mstau22) - 10*mstau22*power2(mu2) +
     2*power3(mstau22))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))
     ))/4. - (mu*sb*log(mh2/Q2)*log(mstau22/Q2)*power2(sa)*(-4/(mstau12 -
     mstau22) + (4*mu2)/((mstau12 - mstau22)*(-mstau22 + mu2)) + mh2/(mstau12*
     mstau22 - mstau12*mu2) + (2*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (2*
     mh2*mu2*DeltaInv(mstau22,mh2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mh2*(mstau22
     + mu2) + power2(mh2)))/(mstau22*power2(mstau12 - mu2)) - (mh2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (mh2*DeltaInv(mstau22,mstau12,mh2)*(-(
     mh2*(mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mstau12 - mu2)) + (
     mstau22 - 2*mu2)*power2(mh2)*power2(mstau12 - mu2) + mstau24*(mstau22 - 2*
     mu2)*power2(mu2) - 3*mstau22*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*
     mu2*(-2*mstau22*mstau24 + 4*mstau24*mu2 + 5*mu2*power2(mstau22) + 2*
     mstau22*power2(mu2) - 4*power3(mstau22)) + power2(mstau12)*(mstau22*
     mstau24 - 2*mstau24*mu2 + 2*mu2*power2(mstau22) - 10*mstau22*power2(mu2) +
     2*power3(mstau22))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))
     ))/(4.*cb) - (mu*sb*Fin3(mstau22,mC2,mu2,Q2)*(msntau2*(mstau22 + mu2) -
     mu2*(3*mstau22 + mu2) - (msntau2 - mu2)*mu2*DeltaInv(mstau22,mC2,mu2)*(-3*
     mC2*mstau22 - 2*mC2*mu2 - mstau22*mu2 + power2(mC2) + power2(mu2))))/(2.*
     cb*mstau22*power3(msntau2 - mu2)) + (mu*sb*Fin3(mA2,mstau22,mu2,Q2)*(2*
     mstau22*(mstau22 - mu2)*power2(mstau12 - mu2) + mA2*(mstau12 - mstau22)*(
     mstau12 - mu2)*mu2*DeltaInv(mA2,mstau22,mu2)*(-3*mA2*mstau22 - 2*mA2*mu2 -
     mstau22*mu2 + power2(mA2) + power2(mu2)) + mA2*(-3*mstau22*mu2*(mstau22 +
     mu2) - (3*mstau22 + mu2)*power2(mstau12) + mstau12*(8*mstau22*mu2 +
     power2(mstau22) + power2(mu2)))))/(4.*cb*mA2*(mstau12 - mstau22)*mstau22*
     power3(mstau12 - mu2)) - (mu*sb*Fin3(mstau22,mC2,msntau2,Q2)*(-(1/(mC2*
     msntau2)) + 1/(msntau2*(-mstau22 + mu2)) - 1/power2(mC2) - 2/power2(
     msntau2 - mu2) - 1/power2(mstau22 - mu2) + mu2/(msntau2*power2(mstau22 -
     mu2)) + DeltaInv(mstau22,mC2,msntau2)*(2 + (-3*msntau2*mstau22 + mstau24)/
     (mC2*msntau2) + (2*mu2)/(msntau2 - mu2) + (3*mstau22)/(-msntau2 + mu2) + (
     2*mstau22)/(-mstau22 + mu2) + mstau24/(msntau2*mstau22 - msntau2*mu2) +
     mstau24/power2(msntau2 - mu2) - (mstau22*mu2)/power2(msntau2 - mu2) + mC2*
     (3/(-msntau2 + mu2) + 3/(-mstau22 + mu2) - (2*mstau22)/(msntau2*mstau22 -
     msntau2*mu2) - (2*mstau22)/power2(msntau2 - mu2) - mu2/power2(msntau2 -
     mu2) + (1 + (2*mstau22*mu2)/power2(mstau22 - mu2))/msntau2 + (3*mu2)/
     power2(mstau22 - mu2)) + power2(mC2)*(1/(msntau2*mstau22 - msntau2*mu2) +
     1/power2(msntau2 - mu2) - mu2/(msntau2*power2(mstau22 - mu2))) - (2*
     mstau22 + (mstau24*mu2)/power2(mstau22 - mu2))/msntau2 + (2*mstau22*mu2)/
     power2(mstau22 - mu2) - power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/
     power3(msntau2 - mu2) + (2*mu2)/power3(mstau22 - mu2)))/(2.*cb) - (mu*sb*
     Fin3(mC2,msntau2,mu2,Q2)*(msntau2*(mstau22 - 3*mu2) + (mstau22 - mu2)*mu2
     - (mstau22 - mu2)*mu2*DeltaInv(mC2,msntau2,mu2)*(-3*mC2*msntau2 - 2*mC2*
     mu2 - msntau2*mu2 + power2(mC2) + power2(mu2))))/(2.*cb*msntau2*power3(
     mstau22 - mu2)) + (mu*sb*Fin3(mA2,mstau12,mu2,Q2)*(-2*mstau12*(mstau12 -
     mu2)*power2(mstau22 - mu2) + mA2*(mstau12 - mstau22)*(mstau22 - mu2)*mu2*
     DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2 +
     power2(mA2) + power2(mu2)) + mA2*(mstau22*(mstau22 - mu2)*mu2 - (mstau22 -
     3*mu2)*power2(mstau12) + mstau12*(-8*mstau22*mu2 + 3*power2(mstau22) + 3*
     power2(mu2)))))/(4.*cb*mA2*mstau12*(mstau12 - mstau22)*power3(mstau22 -
     mu2)) + (mu*sa*(ca*cb + sa*sb)*Fin3(mstau22,mh2,mu2,Q2)*(mh2*(mstau12 -
     mstau22)*(mstau12 - mu2)*(mstau22 - mu2)*mu2*DeltaInv(mstau22,mh2,mu2)*(-
     3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 + power2(mh2) + power2(mu2)) + 2*
     mstau22*(mstau22 - mu2)*power3(mstau12 - mu2) + mh2*(power2(mstau12)*(6*
     mstau22*mu2 - power2(mstau22) + power2(mu2)) + mstau22*mu2*(2*mstau22*mu2
     - 3*power2(mstau22) + 3*power2(mu2)) - 2*mstau22*power3(mstau12) +
     mstau12*(3*mu2*power2(mstau22) - 9*mstau22*power2(mu2) + power3(mstau22) -
     power3(mu2)))))/(4.*cb*mh2*(mstau12 - mstau22)*mstau22*(mstau22 - mu2)*
     power3(mstau12 - mu2)) - (mu*Fin3(mstau22,mH2,mu2,Q2)*(ca*cb*sa + sb*(-1 +
     power2(sa)))*(mH2*(mstau12 - mstau22)*(mstau12 - mu2)*(mstau22 - mu2)*mu2*
     DeltaInv(mstau22,mH2,mu2)*(-3*mH2*mstau22 - 2*mH2*mu2 - mstau22*mu2 +
     power2(mH2) + power2(mu2)) + 2*mstau22*(mstau22 - mu2)*power3(mstau12 -
     mu2) + mH2*(power2(mstau12)*(6*mstau22*mu2 - power2(mstau22) + power2(mu2)
     ) + mstau22*mu2*(2*mstau22*mu2 - 3*power2(mstau22) + 3*power2(mu2)) - 2*
     mstau22*power3(mstau12) + mstau12*(3*mu2*power2(mstau22) - 9*mstau22*
     power2(mu2) + power3(mstau22) - power3(mu2)))))/(4.*cb*mH2*(mstau12 -
     mstau22)*mstau22*(mstau22 - mu2)*power3(mstau12 - mu2)) - (3*mu*sb*log(
     mH2/Q2)*(4/mH2 + 6/mstau12 + mH2/(mstau12*mstau22 - mstau12*mu2) + mH2/
     power2(mstau12 - mu2) - (mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*
     mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (2*mH2*
     mu2)/(mstau12*power2(mstau22 - mu2)) - (mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2*DeltaInv(mH2,
     mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mH2,
     mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + DeltaInv(mstau22,mH2,
     mstau12)*(18*mstau22 - (6*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 -
     mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) +
     1/power2(mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*
     mu2)/power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) +
     mH2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*
     mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (12*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(
     mstau22 - mu2))/mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(
     mu2)/power2(mstau22 - mu2)) + (-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*
     mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (mH2*DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))))/(4.*cb) +
     (mu*sb*power2(log(mH2/Q2))*(6/mH2 + 6/mstau12 + mH2/(mstau12*mstau22 -
     mstau12*mu2) + mH2/power2(mstau12 - mu2) - (mH2*mu2)/(mstau22*power2(
     mstau12 - mu2)) - (3*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(
     mstau12 - mu2) - (2*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 -
     mstau12*mu2 + power2(mH2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2))
     + (mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 -
     mu2)) + DeltaInv(mstau22,mH2,mstau12)*(18*mstau22 - (6*mstau24)/mstau12 +
     power2(mH2)*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(
     mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(
     mstau12 - mu2) + (-6 - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (
     3*mu2)/power2(mstau22 - mu2)) + mH2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*
     mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/power2(
     mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (12*mstau22 - (
     mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/mstau12 - (2*mstau22*
     mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (-(1/
     power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2)
     ))*power3(mH2)) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2))))/(8.*cb) - (ca*mu*sa*power2(log(mH2/Q2))*(6/mH2 +
     6/(-mh2 + mH2) + 6/mstau12 + mH2/(mstau12*mstau22 - mstau12*mu2) + (8*mH2)
     /(-(mh2*mH2) + power2(mh2)) - (8*mH2)/power2(mh2 - mH2) + (2*power2(mH2))/
     ((-mh2 + mH2)*power2(mh2)) + (10*power2(mH2))/(mh2*power2(mh2 - mH2)) +
     mH2/power2(mstau12 - mu2) - (mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (
     3*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (2*
     mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2*DeltaInv(mH2,
     mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (2*power3(mH2))/(power2(
     mh2)*power2(mh2 - mH2)) + (mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(18*mstau22
     - (6*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 - mu2) + 3/(mstau22 - mu2)
     + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2))
     + mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*mu2)/power2(mstau22 - mu2))
     /mstau12 - (3*mu2)/power2(mstau22 - mu2)) + mH2*(3 + (3*mstau22)/(mstau12
     - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/
     power2(mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (12*mstau22
     - (mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/mstau12 - (2*mstau22*
     mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (-(1/
     power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2)
     ))*power3(mH2)) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2))))/8. - (mu*sb*power2(sa)*power2(log(mH2/Q2))*(12/
     mH2 + 12/(-mh2 + mH2) + 12/mstau12 + mH2/(mstau12*mstau22 - mstau12*mu2) +
     (16*mH2)/(-(mh2*mH2) + power2(mh2)) - (16*mH2)/power2(mh2 - mH2) + (4*
     power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (20*power2(mH2))/(mh2*power2(mh2
     - mH2)) + mH2/power2(mstau12 - mu2) - (mH2*mu2)/(mstau22*power2(mstau12 -
     mu2)) - (3*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 -
     mu2) - (2*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (mH2*DeltaInv(
     mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (mH2*
     mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 +
     power2(mH2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (4*power3(
     mH2))/(power2(mh2)*power2(mh2 - mH2)) + (mu2*DeltaInv(mstau22,mH2,mu2)*
     power3(mH2))/(mstau22*power2(mstau12 - mu2)) + DeltaInv(mstau22,mH2,
     mstau12)*(36*mstau22 - (12*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 -
     mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) +
     1/power2(mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-12 - (2*mstau22*
     mu2)/power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) +
     mH2*(9 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*
     mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (24*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(
     mstau22 - mu2))/mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(
     mu2)/power2(mstau22 - mu2)) + (-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*
     mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (mH2*DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))))/(8.*cb) +
     (ca*mu*sa*log(mH2/Q2)*(-(1/mh2) + 12/mH2 + 5/(-mh2 + mH2) + 18/mstau12 + (
     3*mH2)/(mstau12*mstau22 - mstau12*mu2) - mH2/power2(mh2) + (8*mH2)/(-(mh2*
     mH2) + power2(mh2)) - (8*mH2)/power2(mh2 - mH2) + (4*power2(mH2))/((-mh2 +
     mH2)*power2(mh2)) + (11*power2(mH2))/(mh2*power2(mh2 - mH2)) + (3*mH2)/
     power2(mstau12 - mu2) - (3*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (9*
     mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (6*mH2*
     mu2)/(mstau12*power2(mstau22 - mu2)) - (3*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (6*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*mH2*mu2*DeltaInv(
     mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (3*power3(mH2))/(power2(
     mh2)*power2(mh2 - mH2)) + (3*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) + 3*DeltaInv(mstau22,mH2,mstau12)*(18*
     mstau22 - (6*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 - mu2) + 3/(
     mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(
     mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*mu2)/
     power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) + mH2*(3 +
     (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-
     mstau12 + mu2) - mstau24/(mstau12*mstau22 - mstau12*mu2) - mstau24/power2(
     mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (12*mstau22 + (
     mstau24*mu2)/power2(mstau22 - mu2))/mstau12 - (2*mstau22*mu2)/power2(
     mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (-(1/(mstau12*
     mstau22 - mstau12*mu2)) - 1/power2(mstau12 - mu2) + mu2/(mstau12*power2(
     mstau22 - mu2)))*power3(mH2)) + (3*mH2*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/(mstau22*power2(mstau12 - mu2))))/4. + (mu*sb*log(mH2/Q2)*power2(sa)
     *(-2/mh2 + 24/mH2 + 10/(-mh2 + mH2) + 36/mstau12 + (3*mH2)/(mstau12*
     mstau22 - mstau12*mu2) - (2*mH2)/power2(mh2) + (16*mH2)/(-(mh2*mH2) +
     power2(mh2)) - (16*mH2)/power2(mh2 - mH2) + (8*power2(mH2))/((-mh2 + mH2)*
     power2(mh2)) + (22*power2(mH2))/(mh2*power2(mh2 - mH2)) + (3*mH2)/power2(
     mstau12 - mu2) - (3*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (9*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (6*mH2*mu2)
     /(mstau12*power2(mstau22 - mu2)) - (3*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (6*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*mH2*mu2*DeltaInv(
     mH2,mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (6*power3(mH2))/(power2(
     mh2)*power2(mh2 - mH2)) + (3*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) + 3*DeltaInv(mstau22,mH2,mstau12)*(36*
     mstau22 - (12*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 - mu2) + 3/(
     mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(
     mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-12 - (2*mstau22*mu2)/
     power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) + mH2*(9 +
     (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-
     mstau12 + mu2) - mstau24/(mstau12*mstau22 - mstau12*mu2) - mstau24/power2(
     mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (24*mstau22 + (
     mstau24*mu2)/power2(mstau22 - mu2))/mstau12 - (2*mstau22*mu2)/power2(
     mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (-(1/(mstau12*
     mstau22 - mstau12*mu2)) - 1/power2(mstau12 - mu2) + mu2/(mstau12*power2(
     mstau22 - mu2)))*power3(mH2)) + (3*mH2*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/(mstau22*power2(mstau12 - mu2))))/(4.*cb) + (ca*mu*sa*log(mh2/Q2)*
     log(mstau12/Q2)*(-2/(mstau12 - mstau22) + (2*mu2)/((mstau12 - mstau22)*(-
     mstau12 + mu2)) + (2*mh2)/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(
     mstau12 - mu2) - (4*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) + (2*mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mh2*(mstau12 + mu2)
     + power2(mh2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(mstau22,
     mstau12,mh2)*(2*mstau24*(-mstau22 + 2*mu2)*power2(mu2) - power2(mh2)*(2*(
     mstau22 - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22 - 3*mu2) + 2*(
     mstau22 - 2*mu2)*power2(mu2)) + 2*(-2*mstau22*mu2 + power2(mstau22) -
     power2(mu2))*power3(mstau12) + mh2*(4*mstau22*(mstau22 - 2*mu2)*power2(
     mu2) + power2(mstau12)*(-22*mstau22*mu2 + 7*power2(mstau22) + 19*power2(
     mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) + 2*mstau12*(-7*mu2*power2(
     mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) - 5*power3(mu2))) +
     power2(mstau12)*(-2*mstau22*mstau24 + 4*mstau24*mu2 - 8*mu2*power2(
     mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) + 2*power3(mu2)) +
     mstau12*(6*mstau22*mstau24*mu2 - mstau24*power2(mstau22) - 9*mstau24*
     power2(mu2) + 2*mu2*power3(mstau22) - 6*mstau22*power3(mu2))))/(mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/4. + (mu*sb*log(mh2/Q2)*
     log(mstau12/Q2)*power2(sa)*(-4/(mstau12 - mstau22) + (4*mu2)/((mstau12 -
     mstau22)*(-mstau12 + mu2)) + (2*mh2)/(mstau12*mstau22 - mstau12*mu2) +
     mh2/power2(mstau12 - mu2) - (4*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) +
     (2*mh2*mu2*DeltaInv(mstau12,mh2,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mh2*(
     mstau12 + mu2) + power2(mh2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*
     DeltaInv(mstau22,mstau12,mh2)*(2*mstau24*(-mstau22 + 2*mu2)*power2(mu2) -
     power2(mh2)*(2*(mstau22 - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22
     - 3*mu2) + 2*(mstau22 - 2*mu2)*power2(mu2)) + 2*(-2*mstau22*mu2 + power2(
     mstau22) - power2(mu2))*power3(mstau12) + mh2*(4*mstau22*(mstau22 - 2*mu2)
     *power2(mu2) + power2(mstau12)*(-22*mstau22*mu2 + 7*power2(mstau22) + 19*
     power2(mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) + 2*mstau12*(-7*mu2*
     power2(mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) - 5*power3(mu2)
     )) + power2(mstau12)*(-2*mstau22*mstau24 + 4*mstau24*mu2 - 8*mu2*power2(
     mstau22) + 13*mstau22*power2(mu2) + power3(mstau22) + 2*power3(mu2)) +
     mstau12*(6*mstau22*mstau24*mu2 - mstau24*power2(mstau22) - 9*mstau24*
     power2(mu2) + 2*mu2*power3(mstau22) - 6*mstau22*power3(mu2))))/(mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(4.*cb) + (mu*sb*Fin3(mA2,
     mstau22,mstau12,Q2)*(2/((mstau12 - mstau22)*(mstau12 - mu2)) + 2/((mstau12
     - mstau22)*(-mstau22 + mu2)) + 1/(mstau12*mstau22 - mstau12*mu2) - 2/
     power2(mA2) - (2*(-(mstau12*mu2) + (mstau22 - mu2)*mu2 + power2(mstau12)))
     /(mA2*mstau12*(mstau12 - mu2)*(-mstau22 + mu2)) + 2/power2(mstau12 - mu2)
     + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 - mu2)) +
     DeltaInv(mA2,mstau22,mstau12)*(-5 + (-6*mstau22 + (2*mstau24)/mstau12)/mA2
     + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-
     mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/power2(
     mstau12 - mu2) - (4*mstau22 + (mstau24*(mstau22 - 2*mu2))/power2(mstau22 -
     mu2))/mstau12 + mA2*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/
     (mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(
     mstau12 - mu2) + (2 - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (3*
     mu2)/power2(mstau22 - mu2)) + power2(mA2)*(-(1/power2(mstau12 - mu2)) + (-
     mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2))) - (2*mstau22*mu2)/
     power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/
     power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22 + mu2)))/(4.*cb) + (mu*
     mu2*sb*log(mu2/Q2)*(4/power2(msntau2 - mu2) + (6*mu2)/(mstau22*power2(
     msntau2 - mu2)) + (18*mC2*mu2*DeltaInv(mstau22,mC2,mu2))/power2(msntau2 -
     mu2) - (6*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/(mstau22*power2(
     msntau2 - mu2)) + 4/power2(mstau12 - mu2) + (6*mu2)/(mstau22*power2(
     mstau12 - mu2)) + (9*mH2*mu2*DeltaInv(mstau22,mH2,mu2))/power2(mstau12 -
     mu2) - (3*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/(mstau22*power2(
     mstau12 - mu2)) + 8/power2(mstau22 - mu2) + (6*mu2)/(msntau2*power2(
     mstau22 - mu2)) + (6*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mC2*mu2*
     DeltaInv(mC2,msntau2,mu2))/power2(mstau22 - mu2) + (9*mH2*mu2*DeltaInv(
     mH2,mstau12,mu2))/power2(mstau22 - mu2) - (6*mu2*DeltaInv(mC2,msntau2,mu2)
     *power2(mC2))/(msntau2*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mH2,
     mstau12,mu2)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (6*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (12*mC2*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) + (3*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (6*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     6*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) + (12*mC2*
     DeltaInv(mC2,msntau2,mu2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (
     3*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2
     + power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*
     DeltaInv(mA2,mstau22,mu2)*(-3*mA2*mstau22 - 2*mA2*mu2 - mstau22*mu2 +
     power2(mA2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) - (3*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     6*DeltaInv(mC2,msntau2,mu2)*power3(mu2))/(msntau2*power2(mstau22 - mu2)) -
     (3*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2))
     + (4*mu2)/power3(-msntau2 + mu2) + (4*mu2)/power3(-mstau12 + mu2) + (8*
     mu2)/power3(-mstau22 + mu2)))/(4.*cb) - (mu*mu2*sb*power2(log(mu2/Q2))*(2/
     power2(msntau2 - mu2) + (2*mu2)/(mstau22*power2(msntau2 - mu2)) + (6*mC2*
     mu2*DeltaInv(mstau22,mC2,mu2))/power2(msntau2 - mu2) - (2*mu2*DeltaInv(
     mstau22,mC2,mu2)*power2(mC2))/(mstau22*power2(msntau2 - mu2)) + 2/power2(
     mstau12 - mu2) + (2*mu2)/(mstau22*power2(mstau12 - mu2)) + (3*mH2*mu2*
     DeltaInv(mstau22,mH2,mu2))/power2(mstau12 - mu2) - (mu2*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2))/(mstau22*power2(mstau12 - mu2)) + 4/power2(mstau22 -
     mu2) + (2*mu2)/(msntau2*power2(mstau22 - mu2)) + (2*mu2)/(mstau12*power2(
     mstau22 - mu2)) + (6*mC2*mu2*DeltaInv(mC2,msntau2,mu2))/power2(mstau22 -
     mu2) + (3*mH2*mu2*DeltaInv(mH2,mstau12,mu2))/power2(mstau22 - mu2) + (mu2*
     DeltaInv(mA2,mstau12,mu2)*((mstau12 - mu2)*mu2 + mA2*(3*mstau12 + 2*mu2) -
     power2(mA2)))/(mstau12*power2(mstau22 - mu2)) - (2*mu2*DeltaInv(mC2,
     msntau2,mu2)*power2(mC2))/(msntau2*power2(mstau22 - mu2)) - (mu2*DeltaInv(
     mH2,mstau12,mu2)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mA2,mstau22,mu2)*(3*mA2*mstau22 + 2*mA2*mu2 + mstau22*mu2 -
     power2(mA2) - power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (2*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (4*mC2*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) + (DeltaInv(
     mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (2*mH2*DeltaInv(
     mstau22,mH2,mu2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (2*
     DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) + (4*mC2*
     DeltaInv(mC2,msntau2,mu2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) + (2*mH2*
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) -
     (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) -
     (2*DeltaInv(mC2,msntau2,mu2)*power3(mu2))/(msntau2*power2(mstau22 - mu2))
     - (DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2))
     + (4*mu2)/power3(-msntau2 + mu2) + (4*mu2)/power3(-mstau12 + mu2) + (8*
     mu2)/power3(-mstau22 + mu2)))/(8.*cb) + (mu*sa*(ca*cb + sa*sb)*Fin3(
     mstau12,mh2,mu2,Q2)*(mh2*(mstau12 - mstau22)*(mstau12 - mu2)*(mstau22 -
     mu2)*mu2*DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*
     mu2 + power2(mh2) + power2(mu2)) + mh2*(power2(mstau12)*(-3*mstau22*mu2 +
     power2(mstau22) - 2*power2(mu2)) + mstau22*(-mstau22 + mu2)*power2(mu2) -
     (mstau22 - 3*mu2)*power3(mstau12) + mstau12*(-6*mu2*power2(mstau22) + 9*
     mstau22*power2(mu2) + 2*power3(mstau22) - 3*power3(mu2))) + 2*mstau12*(
     mstau12 - mu2)*power3(-mstau22 + mu2)))/(4.*cb*mh2*mstau12*(mstau12 -
     mstau22)*(mstau12 - mu2)*power3(mstau22 - mu2)) - (mu*Fin3(mH2,mstau12,
     mu2,Q2)*(ca*cb*sa + sb*(-1 + power2(sa)))*(mH2*(mstau12 - mstau22)*(
     mstau12 - mu2)*(mstau22 - mu2)*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*
     mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)) + mH2*(
     power2(mstau12)*(-3*mstau22*mu2 + power2(mstau22) - 2*power2(mu2)) +
     mstau22*(-mstau22 + mu2)*power2(mu2) - (mstau22 - 3*mu2)*power3(mstau12) +
     mstau12*(-6*mu2*power2(mstau22) + 9*mstau22*power2(mu2) + 2*power3(
     mstau22) - 3*power3(mu2))) + 2*mstau12*(mstau12 - mu2)*power3(-mstau22 +
     mu2)))/(4.*cb*mH2*mstau12*(mstau12 - mstau22)*(mstau12 - mu2)*power3(
     mstau22 - mu2)) + (3*ca*mu*log(mh2/Q2)*(3*mh2*mH2 - 4*mh2*mstau12 + 2*mH2*
     mstau12 - 3*power2(mh2) + 3*mh2*(mh2 - mH2)*DeltaInv(mstau22,mstau12,mh2)*
     (-3*mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 + power2(mh2)))*
     power3(sa))/(2.*mh2*(mh2 - mH2)*mstau12) - (3*ca*mu*Fin20(mstau22,mstau12,
     Q2)*power2(mh2 - mH2)*power3(sa))/(2.*power2(mh2)*power2(mH2)) + (3*ca*mu*
     log(mH2/Q2)*(-3*mh2*mH2 - 2*mh2*mstau12 + 4*mH2*mstau12 + 3*power2(mH2) +
     3*(mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*mstau22 - mH2*
     (mstau12 + 2*mstau22) + mstau24 + power2(mH2)))*power3(sa))/(2.*(mh2 -
     mH2)*mH2*mstau12) - (3*ca*mu*(mh2*(mH2 - 3*mstau12) + mH2*mstau12 -
     power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(mstau22,mstau12,mh2)*(-3*mstau12*
     mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 + power2(mh2)))*power2(log(
     mh2/Q2))*power3(sa))/(4.*mh2*(mh2 - mH2)*mstau12) + (3*ca*mu*(-(mh2*(mH2 +
     mstau12)) + mH2*(mH2 + 3*mstau12) + (mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,
     mstau12)*(-3*mstau12*mstau22 - mH2*(mstau12 + 2*mstau22) + mstau24 +
     power2(mH2)))*power2(log(mH2/Q2))*power3(sa))/(4.*mH2*(-mh2 + mH2)*
     mstau12) + (mu*sa*Fin3(mstau22,mstau12,mh2,Q2)*((-6*ca*(mh2*(mH2 - 3*
     mstau12) + mH2*mstau12 - power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(mstau22,
     mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 +
     power2(mh2)))*power2(sa))/((mh2 - mH2)*mstau12*power2(mh2)) + (sa*sb*(1/(
     mstau12*mstau22 - mstau12*mu2) - 12/(-(mh2*mH2) + power2(mh2)) + 2/power2(
     mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 -
     mu2)) - (DeltaInv(mstau22,mstau12,mh2)*(mstau24*(mstau22 - 2*mu2)*power2(
     mu2) + power2(mh2)*((mstau22 - 2*mu2)*power2(mstau12) + (mstau22 - 2*mu2)*
     power2(mu2) + mstau12*(-4*mstau22*mu2 + power2(mstau22) + 5*power2(mu2)))
     + (-2*mstau22*mu2 + power2(mstau22) + 2*power2(mu2))*power3(mstau12) +
     power2(mstau12)*(6*mu2*power2(mstau22) + mstau22*(mstau24 - 3*power2(mu2))
     - 2*mu2*(mstau24 + power2(mu2)) - 3*power3(mstau22)) + mstau12*(power2(
     mstau22)*(mstau24 - 5*power2(mu2)) + 5*mstau24*power2(mu2) + 2*mu2*power3(
     mstau22) + mstau22*(-4*mstau24*mu2 + 4*power3(mu2))) + mh2*(power2(
     mstau12)*(16*mstau22*mu2 - 5*power2(mstau22) - 15*power2(mu2)) - 2*
     mstau22*(mstau22 - 2*mu2)*power2(mu2) - 3*(mstau22 - 2*mu2)*power3(
     mstau12) + mstau12*(10*mu2*power2(mstau22) - 17*mstau22*power2(mu2) - 2*
     power3(mstau22) + 8*power3(mu2)))))/(mstau12*power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22
     + mu2)))/cb + ca*(1/(mstau12*mstau22 - mstau12*mu2) - 6/(-(mh2*mH2) +
     power2(mh2)) + 2/power2(mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(
     mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*(mstau24*(
     mstau22 - 2*mu2)*power2(mu2) + power2(mh2)*((mstau22 - 2*mu2)*power2(
     mstau12) + (mstau22 - 2*mu2)*power2(mu2) + mstau12*(-4*mstau22*mu2 +
     power2(mstau22) + 5*power2(mu2))) + (-2*mstau22*mu2 + power2(mstau22) + 2*
     power2(mu2))*power3(mstau12) + power2(mstau12)*(6*mu2*power2(mstau22) +
     mstau22*(mstau24 - 3*power2(mu2)) - 2*mu2*(mstau24 + power2(mu2)) - 3*
     power3(mstau22)) + mstau12*(power2(mstau22)*(mstau24 - 5*power2(mu2)) + 5*
     mstau24*power2(mu2) + 2*mu2*power3(mstau22) + mstau22*(-4*mstau24*mu2 + 4*
     power3(mu2))) + mh2*(power2(mstau12)*(16*mstau22*mu2 - 5*power2(mstau22) -
     15*power2(mu2)) - 2*mstau22*(mstau22 - 2*mu2)*power2(mu2) - 3*(mstau22 -
     2*mu2)*power3(mstau12) + mstau12*(10*mu2*power2(mstau22) - 17*mstau22*
     power2(mu2) - 2*power3(mstau22) + 8*power3(mu2)))))/(mstau12*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (
     2*mu2)/power3(-mstau22 + mu2)) - (6*sb*(mh2*(mH2 - 3*mstau12) + mH2*
     mstau12 - power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(mstau22,mstau12,mh2)*(-3*
     mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 + power2(mh2)))*
     power3(sa))/(cb*(mh2 - mH2)*mstau12*power2(mh2))))/4. - (mu*sb*power2(log(
     mC2/Q2))*(-(1/mC2) - 1/msntau2 + mC2/(msntau2*(-mstau22 + mu2)) - mC2/
     power2(msntau2 - mu2) + (mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (3*
     mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (2*mC2*
     mu2)/(msntau2*power2(mstau22 - mu2)) + (mC2*DeltaInv(mstau22,mC2,mu2)*
     power2(mu2))/power2(msntau2 - mu2) + (2*DeltaInv(mstau22,mC2,mu2)*power2(
     mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (mC2*mu2*DeltaInv(mC2,
     msntau2,mu2)*(-3*mC2*msntau2 - 2*mC2*mu2 - msntau2*mu2 + power2(mC2) +
     power2(mu2)))/(msntau2*power2(mstau22 - mu2)) - (mu2*DeltaInv(mstau22,mC2,
     mu2)*power3(mC2))/(mstau22*power2(msntau2 - mu2)) - (mC2*DeltaInv(mstau22,
     mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) + (DeltaInv(mstau22,
     mC2,msntau2)*(-((3*msntau2*mstau22 - mstau24)*power2(msntau2 - mu2)*
     power2(mstau22 - mu2)) + ((mstau22 - 2*mu2)*power2(msntau2) + (mstau22 -
     2*mu2)*power2(mu2) + msntau2*(-4*mstau22*mu2 + power2(mstau22) + 5*power2(
     mu2)))*power3(mC2) + power2(mC2)*(power2(mu2)*(2*mstau22*mu2 - power2(
     mstau22) + power2(mu2)) - 2*power2(msntau2)*(-7*mstau22*mu2 + 2*power2(
     mstau22) + 7*power2(mu2)) - 3*(mstau22 - 2*mu2)*power3(msntau2) + msntau2*
     (8*mu2*power2(mstau22) - 13*mstau22*power2(mu2) - 2*power3(mstau22) + 6*
     power3(mu2))) + mC2*(power2(mu2)*power3(msntau2) + power2(msntau2)*(-2*
     mstau24*mu2 + 12*mu2*power2(mstau22) + mstau22*(mstau24 - 9*power2(mu2)) -
     5*power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 + 4*mu2*power2(mstau22) +
     mstau22*(mstau24 - 2*power2(mu2)) - 2*power3(mstau22)) + msntau2*(power2(
     mstau22)*(mstau24 - 14*power2(mu2)) + 5*mstau24*power2(mu2) + 6*mu2*
     power3(mstau22) + mstau22*(-4*mstau24*mu2 + 10*power3(mu2)) - power4(mu2))
     )))/(msntau2*power2(msntau2 - mu2)*power2(mstau22 - mu2))))/(4.*cb) + (mu*
     sb*log(mC2/Q2)*(-2/mC2 - 3/msntau2 - (3*mC2)/(msntau2*mstau22 - msntau2*
     mu2) - (3*mC2)/power2(msntau2 - mu2) + (3*mC2*mu2)/(mstau22*power2(msntau2
     - mu2)) + (9*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 -
     mu2) + (6*mC2*mu2)/(msntau2*power2(mstau22 - mu2)) + (3*mC2*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (6*DeltaInv(mstau22,
     mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (3*
     mC2*mu2*DeltaInv(mC2,msntau2,mu2)*(-3*mC2*msntau2 - 2*mC2*mu2 - msntau2*
     mu2 + power2(mC2) + power2(mu2)))/(msntau2*power2(mstau22 - mu2)) - (3*
     mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 - mu2))
     - (3*mC2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 -
     mu2)) + (3*DeltaInv(mstau22,mC2,msntau2)*(-((3*msntau2*mstau22 - mstau24)*
     power2(msntau2 - mu2)*power2(mstau22 - mu2)) + ((mstau22 - 2*mu2)*power2(
     msntau2) + (mstau22 - 2*mu2)*power2(mu2) + msntau2*(-4*mstau22*mu2 +
     power2(mstau22) + 5*power2(mu2)))*power3(mC2) + power2(mC2)*(power2(mu2)*(
     2*mstau22*mu2 - power2(mstau22) + power2(mu2)) - 2*power2(msntau2)*(-7*
     mstau22*mu2 + 2*power2(mstau22) + 7*power2(mu2)) - 3*(mstau22 - 2*mu2)*
     power3(msntau2) + msntau2*(8*mu2*power2(mstau22) - 13*mstau22*power2(mu2)
     - 2*power3(mstau22) + 6*power3(mu2))) + mC2*(power2(mu2)*power3(msntau2) +
     power2(msntau2)*(-2*mstau24*mu2 + 12*mu2*power2(mstau22) + mstau22*(
     mstau24 - 9*power2(mu2)) - 5*power3(mstau22)) + power2(mu2)*(-2*mstau24*
     mu2 + 4*mu2*power2(mstau22) + mstau22*(mstau24 - 2*power2(mu2)) - 2*
     power3(mstau22)) + msntau2*(power2(mstau22)*(mstau24 - 14*power2(mu2)) +
     5*mstau24*power2(mu2) + 6*mu2*power3(mstau22) + mstau22*(-4*mstau24*mu2 +
     10*power3(mu2)) - power4(mu2)))))/(msntau2*power2(msntau2 - mu2)*power2(
     mstau22 - mu2))))/(2.*cb) - (mu*sb*power2(log(mA2/Q2))*(2/mA2 + 2/mstau12
     + mA2/(mstau12*(-mstau22 + mu2)) - mA2/power2(mstau12 - mu2) + (mA2*mu2)/(
     mstau22*power2(mstau12 - mu2)) + (3*mu2*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2))/power2(mstau12 - mu2) + (2*mA2*mu2)/(mstau12*power2(mstau22 - mu2))
     + (mA2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/power2(mstau12 - mu2) + (2*
     DeltaInv(mA2,mstau22,mu2)*power2(mA2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (mA2*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 -
     mstau12*mu2 + power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2))
     - (mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*power2(mstau12 -
     mu2)) - (mA2*DeltaInv(mA2,mstau22,mu2)*power3(mu2))/(mstau22*power2(
     mstau12 - mu2)) + (DeltaInv(mA2,mstau22,mstau12)*(2*(3*mstau12*mstau22 -
     mstau24)*power2(mstau12 - mu2)*power2(mstau22 - mu2) + ((mstau22 - 2*mu2)*
     power2(mstau12) + (mstau22 - 2*mu2)*power2(mu2) + mstau12*(-4*mstau22*mu2
     + power2(mstau22) + 5*power2(mu2)))*power3(mA2) - power2(mA2)*(2*power2(
     mu2)*(-4*mstau22*mu2 + 2*power2(mstau22) + power2(mu2)) + power2(mstau12)*
     (-20*mstau22*mu2 + 7*power2(mstau22) + 17*power2(mu2)) + 3*(mstau22 - 2*
     mu2)*power3(mstau12) + mstau12*(-14*mu2*power2(mstau22) + 25*mstau22*
     power2(mu2) + 2*power3(mstau22) - 12*power3(mu2))) + mA2*((-6*mstau22*mu2
     + 3*power2(mstau22) + 4*power2(mu2))*power3(mstau12) + power2(mstau12)*(-
     6*mu2*power2(mstau22) - 2*mu2*(mstau24 + 3*power2(mu2)) + mstau22*(mstau24
     + 9*power2(mu2)) + power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 8*mu2*
     power2(mstau22) + mstau22*(mstau24 + 4*power2(mu2)) + 4*power3(mstau22)) +
     mstau12*(5*mstau24*power2(mu2) - 4*mstau22*mu2*(mstau24 + 2*power2(mu2)) +
     power2(mstau22)*(mstau24 + 13*power2(mu2)) - 6*mu2*power3(mstau22) + 2*
     power4(mu2)))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(
     8.*cb) + (mu*sb*log(mA2/Q2)*(4/mA2 + 6/mstau12 - (3*mA2)/(mstau12*mstau22
     - mstau12*mu2) - (3*mA2)/power2(mstau12 - mu2) + (3*mA2*mu2)/(mstau22*
     power2(mstau12 - mu2)) + (9*mu2*DeltaInv(mA2,mstau22,mu2)*power2(mA2))/
     power2(mstau12 - mu2) + (6*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) + (3*
     mA2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/power2(mstau12 - mu2) + (6*
     DeltaInv(mA2,mstau22,mu2)*power2(mA2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (3*mA2*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2
     - mstau12*mu2 + power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)
     ) - (3*mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*power2(mstau12
     - mu2)) - (3*mA2*DeltaInv(mA2,mstau22,mu2)*power3(mu2))/(mstau22*power2(
     mstau12 - mu2)) + (3*DeltaInv(mA2,mstau22,mstau12)*(2*(3*mstau12*mstau22 -
     mstau24)*power2(mstau12 - mu2)*power2(mstau22 - mu2) + ((mstau22 - 2*mu2)*
     power2(mstau12) + (mstau22 - 2*mu2)*power2(mu2) + mstau12*(-4*mstau22*mu2
     + power2(mstau22) + 5*power2(mu2)))*power3(mA2) - power2(mA2)*(2*power2(
     mu2)*(-4*mstau22*mu2 + 2*power2(mstau22) + power2(mu2)) + power2(mstau12)*
     (-20*mstau22*mu2 + 7*power2(mstau22) + 17*power2(mu2)) + 3*(mstau22 - 2*
     mu2)*power3(mstau12) + mstau12*(-14*mu2*power2(mstau22) + 25*mstau22*
     power2(mu2) + 2*power3(mstau22) - 12*power3(mu2))) + mA2*((-6*mstau22*mu2
     + 3*power2(mstau22) + 4*power2(mu2))*power3(mstau12) + power2(mstau12)*(-
     6*mu2*power2(mstau22) - 2*mu2*(mstau24 + 3*power2(mu2)) + mstau22*(mstau24
     + 9*power2(mu2)) + power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 8*mu2*
     power2(mstau22) + mstau22*(mstau24 + 4*power2(mu2)) + 4*power3(mstau22)) +
     mstau12*(5*mstau24*power2(mu2) - 4*mstau22*mu2*(mstau24 + 2*power2(mu2)) +
     power2(mstau22)*(mstau24 + 13*power2(mu2)) - 6*mu2*power3(mstau22) + 2*
     power4(mu2)))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(
     4.*cb) + (3*(mh2 + mH2)*mu*sb*power4(sa))/(4.*cb*mh2*mH2) - (3*mu*sb*log(
     mh2/Q2)*power4(sa))/(2*cb*mh2 - 2*cb*mH2) - (3*mu*sb*log(mH2/Q2)*power4(
     sa))/(2*cb*mh2 - 2*cb*mH2) + (3*mu*sb*log(mH2/Q2)*power4(sa))/(cb*mh2 -
     cb*mH2) - (3*(mh2 + mH2)*mstau12*mu*sb*log(mstau12/Q2)*power4(sa))/(4.*cb*
     mh2*mH2*(mstau12 - mstau22)) + (3*mstau12*mu*sb*log(mh2/Q2)*log(mstau12/
     Q2)*power4(sa))/(2.*cb*(mh2 - mH2)*(mstau12 - mstau22)) - (3*mstau12*mu*
     sb*log(mH2/Q2)*log(mstau12/Q2)*power4(sa))/(2.*cb*(mh2 - mH2)*(mstau12 -
     mstau22)) + (3*(mh2 + mH2)*mstau22*mu*sb*log(mstau22/Q2)*power4(sa))/(4.*
     cb*mh2*mH2*(mstau12 - mstau22)) + (3*mstau22*mu*sb*log(mh2/Q2)*log(
     mstau22/Q2)*power4(sa))/(2.*cb*(-mh2 + mH2)*(mstau12 - mstau22)) + (3*
     mstau22*mu*sb*log(mH2/Q2)*log(mstau22/Q2)*power4(sa))/(2.*cb*(mh2 - mH2)*(
     mstau12 - mstau22)) + (3*mu*sb*log(mh2/Q2)*(3*mh2*mH2 - 4*mh2*mstau12 + 2*
     mH2*mstau12 - 3*power2(mh2) + 3*mh2*(mh2 - mH2)*DeltaInv(mstau22,mstau12,
     mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 + power2(
     mh2)))*power4(sa))/(2.*cb*mh2*(mh2 - mH2)*mstau12) - (3*mu*sb*Fin20(
     mstau22,mstau12,Q2)*power2(mh2 - mH2)*power4(sa))/(2.*cb*power2(mh2)*
     power2(mH2)) + (3*mu*sb*log(mH2/Q2)*(-3*mh2*mH2 - 2*mh2*mstau12 + 4*mH2*
     mstau12 + 3*power2(mH2) + 3*(mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,mstau12)*
     (-3*mstau12*mstau22 - mH2*(mstau12 + 2*mstau22) + mstau24 + power2(mH2)))*
     power4(sa))/(2.*cb*(mh2 - mH2)*mH2*mstau12) - (3*mu*sb*(mh2*(mH2 - 3*
     mstau12) + mH2*mstau12 - power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(mstau22,
     mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 +
     power2(mh2)))*power2(log(mh2/Q2))*power4(sa))/(4.*cb*mh2*(mh2 - mH2)*
     mstau12) + (3*mu*sb*(-(mh2*(mH2 + mstau12)) + mH2*(mH2 + 3*mstau12) + (mh2
     - mH2)*mH2*DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*mstau22 - mH2*(
     mstau12 + 2*mstau22) + mstau24 + power2(mH2)))*power2(log(mH2/Q2))*power4(
     sa))/(4.*cb*mH2*(-mh2 + mH2)*mstau12) + (mu*Fin3(mstau22,mH2,mstau12,Q2)*(
     -((sb*power2(sa)*((12*(mh2 - mstau12))/(mh2*mH2*mstau12) + 1/(mstau12*
     mstau22 - mstau12*mu2) - 12/(-(mh2*mH2) + power2(mh2)) + 12/power2(mH2) +
     2/power2(mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(
     mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(9 + (36*mstau22 - (12*
     mstau24)/mstau12)/mH2 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22
     - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (
     mstau22*mu2)/power2(mstau12 - mu2) + (24*mstau22 - (mstau24*(mstau22 - 2*
     mu2))/power2(mstau22 - mu2))/mstau12 + mH2*(3/(mstau12 - mu2) + 3/(mstau22
     - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 -
     mu2)) + mu2/power2(mstau12 - mu2) + (-12 - (2*mstau22*mu2)/power2(mstau22
     - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) + power2(mH2)*(-(1/
     power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2)
     )) - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 -
     mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22 + mu2)))/
     cb) + (sb*(6/(mH2*mstau12) + 1/(mstau12*mstau22 - mstau12*mu2) + 6/power2(
     mH2) + 2/power2(mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*
     power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(3 + (18*mstau22 -
     (6*mstau24)/mstau12)/mH2 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(
     mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2)
     + (mstau22*mu2)/power2(mstau12 - mu2) + (12*mstau22 - (mstau24*(mstau22 -
     2*mu2))/power2(mstau22 - mu2))/mstau12 + mH2*(3/(mstau12 - mu2) + 3/(
     mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(
     mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*mu2)/
     power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) + power2(
     mH2)*(-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(
     mstau22 - mu2))) - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(mu2)/
     power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau12 + mu2) + (2*mu2)/power3(-
     mstau22 + mu2)))/cb - ca*sa*((-6/mh2 + 6/mstau12)/mH2 + 1/(mstau12*mstau22
     - mstau12*mu2) - 6/(-(mh2*mH2) + power2(mh2)) + 6/power2(mH2) + 2/power2(
     mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 -
     mu2)) + DeltaInv(mstau22,mH2,mstau12)*(3 + (18*mstau22 - (6*mstau24)/
     mstau12)/mH2 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) +
     (2*mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (12*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(
     mstau22 - mu2))/mstau12 + mH2*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*
     mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) +
     mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*mu2)/power2(mstau22 - mu2))/
     mstau12 - (3*mu2)/power2(mstau22 - mu2)) + power2(mH2)*(-(1/power2(mstau12
     - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2))) - (2*
     mstau22*mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) +
     (2*mu2)/power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22 + mu2)) - (6*ca*(
     -(mh2*(mH2 + mstau12)) + mH2*(mH2 + 3*mstau12) + (mh2 - mH2)*mH2*DeltaInv(
     mstau22,mH2,mstau12)*(-3*mstau12*mstau22 - mH2*(mstau12 + 2*mstau22) +
     mstau24 + power2(mH2)))*power3(sa))/((mh2 - mH2)*mstau12*power2(mH2)) - (
     6*sb*(-(mh2*(mH2 + mstau12)) + mH2*(mH2 + 3*mstau12) + (mh2 - mH2)*mH2*
     DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*mstau22 - mH2*(mstau12 + 2*
     mstau22) + mstau24 + power2(mH2)))*power4(sa))/(cb*(mh2 - mH2)*mstau12*
     power2(mH2))))/4. - (mu*mu2*sb*log(mstau22/Q2)*log(mu2/Q2)*(-2*mA2*
     msntau2*mu2 - 2*mH2*msntau2*mu2 - 4*mC2*mstau12*mu2 - 4*msntau2*mstau22*
     mu2 - 4*mstau12*mstau22*mu2 - 2*msntau2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mu2)*power2(mH2) + mA2*power2(msntau2) + mH2*power2(msntau2) + 2*mstau22*
     power2(msntau2) + 2*mu2*power2(msntau2) + 6*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(msntau2) + mstau22*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2)*power2(msntau2) + mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*
     power2(msntau2) + 2*mC2*power2(mstau12) + 2*mstau22*power2(mstau12) + 2*
     mu2*power2(mstau12) + mA2*power2(mu2) + 2*mC2*power2(mu2) + mH2*power2(
     mu2) - 4*msntau2*power2(mu2) - 4*mstau12*power2(mu2) + 4*mstau22*power2(
     mu2) - 12*mH2*msntau2*mstau22*DeltaInv(mstau22,mH2,mu2)*power2(mu2) - 2*
     msntau2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2) + mstau22*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2) + mH2*DeltaInv(mstau22,
     mH2,mu2)*power2(msntau2)*power2(mu2) + mstau22*DeltaInv(mstau22,mH2,mu2)*
     power2(msntau2)*power2(mu2) - DeltaInv(mA2,mstau22,mu2)*power2(msntau2 -
     mu2)*(-(mA2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*power2(mA2) + (-
     mstau22 + mu2)*power2(mu2) + power3(mA2)) - 2*DeltaInv(mstau22,mC2,mu2)*
     power2(mstau12 - mu2)*(-(mC2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*
     power2(mC2) + (-mstau22 + mu2)*power2(mu2) + power3(mC2)) + 2*msntau2*mu2*
     DeltaInv(mstau22,mH2,mu2)*power3(mH2) - DeltaInv(mstau22,mH2,mu2)*power2(
     msntau2)*power3(mH2) - DeltaInv(mstau22,mH2,mu2)*power2(mu2)*power3(mH2) +
     4*power3(mu2) - 2*mH2*msntau2*DeltaInv(mstau22,mH2,mu2)*power3(mu2) + 6*
     mH2*mstau22*DeltaInv(mstau22,mH2,mu2)*power3(mu2) - 2*msntau2*mstau22*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2) + DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power3(mu2) - DeltaInv(mstau22,mH2,mu2)*power2(msntau2)*power3(mu2) +
     mH2*DeltaInv(mstau22,mH2,mu2)*power4(mu2) + 2*msntau2*DeltaInv(mstau22,
     mH2,mu2)*power4(mu2) + mstau22*DeltaInv(mstau22,mH2,mu2)*power4(mu2) -
     DeltaInv(mstau22,mH2,mu2)*power5(mu2)))/(4.*cb*mstau22*power2(msntau2 -
     mu2)*power2(mstau12 - mu2)) + (3*ca*mu*log(mh2/Q2)*log(mstau12/Q2)*power3(
     sa)*(mh2 + mstau12 + mstau22 + DeltaInv(mstau22,mstau12,mh2)*(mstau12*
     mstau24 + mh2*(6*mstau12*mstau22 + mstau24) + (mstau12 + mstau22)*power2(
     mh2) - power3(mh2) - power6(mstau2))))/(2.*mh2*mstau12) + (3*mu*sb*log(
     mh2/Q2)*log(mstau12/Q2)*power4(sa)*(mh2 + mstau12 + mstau22 + DeltaInv(
     mstau22,mstau12,mh2)*(mstau12*mstau24 + mh2*(6*mstau12*mstau22 + mstau24)
     + (mstau12 + mstau22)*power2(mh2) - power3(mh2) - power6(mstau2))))/(2.*
     cb*mh2*mstau12) + (3*ca*mu*log(mH2/Q2)*log(mstau12/Q2)*power3(sa)*(mH2 +
     mstau12 + mstau22 + DeltaInv(mstau22,mH2,mstau12)*(mstau12*mstau24 + mH2*(
     6*mstau12*mstau22 + mstau24) + (mstau12 + mstau22)*power2(mH2) - power3(
     mH2) - power6(mstau2))))/(2.*mH2*mstau12) + (3*mu*sb*log(mH2/Q2)*log(
     mstau12/Q2)*power4(sa)*(mH2 + mstau12 + mstau22 + DeltaInv(mstau22,mH2,
     mstau12)*(mstau12*mstau24 + mH2*(6*mstau12*mstau22 + mstau24) + (mstau12 +
     mstau22)*power2(mH2) - power3(mH2) - power6(mstau2))))/(2.*cb*mH2*mstau12)
     - (ca*mu*sa*log(mstau12/Q2)*(-18/mH2 - 18/mstau12 + mstau12/(mh2*(-mh2 +
     mH2)) - (54*mstau22)/(mH2*mstau12) + (9*mh2)/(mstau12*mstau22 - mstau12*
     mu2) - (9*mH2)/(mstau12*mstau22 - mstau12*mu2) - 24*mH2*DeltaInv(mstau22,
     mH2,mstau12) - 177*mstau22*DeltaInv(mstau22,mH2,mstau12) + (12*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (90*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/mH2 - (87*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 + (6*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (9*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (18*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau22 + mu2) + (9*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + 6*mh2*DeltaInv(
     mstau22,mstau12,mh2) - 3*mstau22*DeltaInv(mstau22,mstau12,mh2) + (6*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 - (3*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/mstau12 + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12 - mu2) + (9*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12
     - mu2) + (18*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) +
     (6*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (6*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) - (9*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + mstau12/
     power2(mh2) + (mH2*mstau12)/((mh2 - mH2)*power2(mh2)) - (3*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (9*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12 - mu2) + (15*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau22 - mu2) + (18*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + (21*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/mstau12 + (9*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/(-mstau12 + mu2) + (15*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-
     mstau22 + mu2) - (18*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau12*mstau22 - mstau12*mu2) + (3*mh2)/power2(mstau12 - mu2) - (3*mH2)/
     power2(mstau12 - mu2) - (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) - (18*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))
     /power2(mstau12 - mu2) - (3*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) + (3*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (18*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau12 - mu2) + (3*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (3*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) - (3*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (3*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (18*mh2*mu2)/(mstau12*power2(mstau22
     - mu2)) + (18*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mH2*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) - (9*mH2*mstau24*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) - (18*
     mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) + (9*
     mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 -
     mu2)) - (15*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau22
     - mu2) - (18*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(
     mstau12*power2(mstau22 - mu2)) + (15*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau22 - mu2) + (18*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (3*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 + mu2) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau22 - mu2) + (12*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) - (12*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(
     mstau22 - mu2) - (6*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/
     power2(mstau22 - mu2) - (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*mstau24*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2)
     )/(mstau12*power2(mstau22 - mu2)) - (9*DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (9*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (3*mu2*
     DeltaInv(mstau12,mh2,mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)
     *power2(mh2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mh2)))/(mstau12*
     power2(mstau22 - mu2)) + (9*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(
     mstau12*mstau22 - mstau12*mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/power2(mstau12 - mu2) - (9*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mH2,mstau12,mu2)*(
     mH2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mH2) + (-mstau12 +
     mu2)*power2(mu2) + 3*power3(mH2)))/(mstau12*power2(mstau22 - mu2)) - (3*
     DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (3*
     DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (54*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (3*DeltaInv(
     mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) - (3*DeltaInv(
     mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/4. - (mu*sb*
     log(mstau12/Q2)*power2(sa)*(-36/mH2 - 36/mstau12 - (108*mstau22)/(mH2*
     mstau12) + (9*mh2)/(mstau12*mstau22 - mstau12*mu2) - (9*mH2)/(mstau12*
     mstau22 - mstau12*mu2) - 42*mH2*DeltaInv(mstau22,mH2,mstau12) - 357*
     mstau22*DeltaInv(mstau22,mH2,mstau12) + (30*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/mstau12 - (180*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 -
     (177*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (6*mH2*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12 - mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau12 + mu2) + (18*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau22
     + mu2) + (9*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 -
     mstau12*mu2) + 6*mh2*DeltaInv(mstau22,mstau12,mh2) - 3*mstau22*DeltaInv(
     mstau22,mstau12,mh2) + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 - (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (9*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (18*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (6*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (6*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) - (9*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*mstau22 - mstau12*mu2) + (2*mstau12)/power2(mh2) + (2*mH2*mstau12)
     /((mh2 - mH2)*power2(mh2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))
     /mstau12 + (9*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12 - mu2) +
     (15*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (18*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) - (2*mstau12)/(-(mh2*mH2) + power2(mh2)) + (39*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (9*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(-mstau12 + mu2) + (15*DeltaInv(mstau22,mH2,mstau12)
     *power2(mH2))/(-mstau22 + mu2) - (18*mstau22*DeltaInv(mstau22,mH2,mstau12)
     *power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (3*mh2)/power2(mstau12 -
     mu2) - (3*mH2)/power2(mstau12 - mu2) - (3*mH2*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau12 - mu2) - (18*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (3*mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (18*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (3*mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (3*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (3*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (3*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (3*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (18*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mH2*mu2)/(mstau12*
     power2(mstau22 - mu2)) + (18*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/power2(mstau22 - mu2) - (9*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/(mstau12*power2(mstau22 - mu2)) - (18*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau22 - mu2) + (9*mh2*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) - (15*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) - (18*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2))
     + (15*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2)
     + (18*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*
     power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-
     mstau22 + mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau22 -
     mu2) + (12*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 -
     mu2) + (6*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(
     mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (12*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (6*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (6*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (9*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2)
     - (3*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (
     9*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 -
     mu2)) + (3*mu2*DeltaInv(mstau12,mh2,mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*
     (mstau12 + mu2)*power2(mh2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mh2)
     ))/(mstau12*power2(mstau22 - mu2)) + (9*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (3*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/power2(mstau12 - mu2) - (9*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(
     mH2,mstau12,mu2)*(mH2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(
     mH2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mH2)))/(mstau12*power2(
     mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(
     mstau22 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(
     mstau22 - mu2) + (108*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*
     mstau12) + (3*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12
     - mu2) - (3*DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 -
     mu2)))/(4.*cb) + (ca*mu*sa*power2(log(mstau12/Q2))*(-6/mH2 - 6/mstau12 - (
     18*mstau22)/(mH2*mstau12) + (3*mh2)/(mstau12*mstau22 - mstau12*mu2) - (3*
     mH2)/(mstau12*mstau22 - mstau12*mu2) - 8*mH2*DeltaInv(mstau22,mH2,mstau12)
     - 59*mstau22*DeltaInv(mstau22,mH2,mstau12) + (4*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 - (30*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /mH2 - (29*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (2*mH2*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau22 + mu2) + (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     mstau22 - mstau12*mu2) + 2*mh2*DeltaInv(mstau22,mstau12,mh2) - mstau22*
     DeltaInv(mstau22,mstau12,mh2) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/mstau12 - (mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (6*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) - (3*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/mstau12 + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12 -
     mu2) + (5*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (6*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) + (7*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (3*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau12 + mu2) + (5*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(-mstau22 + mu2) - (6*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + mh2/
     power2(mstau12 - mu2) - mH2/power2(mstau12 - mu2) - (mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (6*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (6*mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau24*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (mu2*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (6*mh2*mu2)/(
     mstau12*power2(mstau22 - mu2)) + (6*mH2*mu2)/(mstau12*power2(mstau22 -
     mu2)) + (6*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 -
     mu2) - (3*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(
     mstau22 - mu2)) - (6*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau22 - mu2) + (3*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     (mstau12*power2(mstau22 - mu2)) - (5*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (6*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (5*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (6*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 +
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau22 - mu2) + (4*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) +
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (4*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) - (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/
     (mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau12,mh2,
     mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mh2) + (-
     mstau12 + mu2)*power2(mu2) + 3*power3(mh2)))/(mstau12*power2(mstau22 -
     mu2)) + (mu2*DeltaInv(mH2,mstau12,mu2)*(mH2*(10*mstau12 - mu2)*mu2 + 5*(
     mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2) - 3*power3(mH2)))
     /(mstau12*power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/power2(mstau12 - mu2) - (3*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,
     mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (18*DeltaInv(mstau22,
     mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/power2(mstau12 - mu2) - (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/8. + (mu*sb*power2(
     sa)*power2(log(mstau12/Q2))*(-12/mH2 - 12/mstau12 - (36*mstau22)/(mH2*
     mstau12) + (3*mh2)/(mstau12*mstau22 - mstau12*mu2) - (3*mH2)/(mstau12*
     mstau22 - mstau12*mu2) - 14*mH2*DeltaInv(mstau22,mH2,mstau12) - 119*
     mstau22*DeltaInv(mstau22,mH2,mstau12) + (10*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/mstau12 - (60*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 - (
     59*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (2*mH2*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12 - mu2) + (2*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau12 + mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau22 +
     mu2) + (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 -
     mstau12*mu2) + 2*mh2*DeltaInv(mstau22,mstau12,mh2) - mstau22*DeltaInv(
     mstau22,mstau12,mh2) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 - (mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (6*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) - (3*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/mstau12 + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12 -
     mu2) + (5*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (6*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) + (13*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (
     3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau12 + mu2) + (5*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau22 + mu2) - (6*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2)
     + mh2/power2(mstau12 - mu2) - mH2/power2(mstau12 - mu2) - (mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (6*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (6*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (6*mh2*
     mu2)/(mstau12*power2(mstau22 - mu2)) + (6*mH2*mu2)/(mstau12*power2(mstau22
     - mu2)) + (6*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22
     - mu2) - (3*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     power2(mstau22 - mu2)) - (6*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau22 - mu2) + (3*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))
     /(mstau12*power2(mstau22 - mu2)) - (5*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (6*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (5*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (6*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 +
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau22 - mu2) + (4*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) +
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (4*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) - (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/
     (mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau12,mh2,
     mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mh2) + (-
     mstau12 + mu2)*power2(mu2) + 3*power3(mh2)))/(mstau12*power2(mstau22 -
     mu2)) + (mu2*DeltaInv(mH2,mstau12,mu2)*(mH2*(10*mstau12 - mu2)*mu2 + 5*(
     mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2) - 3*power3(mH2)))
     /(mstau12*power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/power2(mstau12 - mu2) - (3*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,
     mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (36*DeltaInv(mstau22,
     mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/power2(mstau12 - mu2) - (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/(8.*cb) - (mu*sb*
     power2(Pi)*power2(sa)*(12/mH2 + 12/mstau12 + (24*mstau22)/(mH2*mstau12) -
     (2*mh2)/(mstau12*mstau22 - mstau12*mu2) + (2*mH2)/(mstau12*mstau22 -
     mstau12*mu2) + 13*mH2*DeltaInv(mstau22,mH2,mstau12) + 83*mstau22*DeltaInv(
     mstau22,mH2,mstau12) + (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 + (48*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (35*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (4*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     (mstau12 - mu2) + (4*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau22 -
     mu2) + (2*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) - (2*mH2*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) -
     mh2*DeltaInv(mstau22,mstau12,mh2) + mstau22*DeltaInv(mstau22,mstau12,mh2)
     - (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (mstau24*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12 - mu2) + (4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-
     mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 +
     mu2) + (4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (
     2*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (3*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (4*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (4*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + (8*mH2)/(-(
     mh2*mH2) + power2(mh2)) - (8*mH2)/power2(mh2 - mH2) - (13*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(mstau12 - mu2) + (4*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau22 - mu2) + (4*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (2*power2(mH2))/((-mh2 +
     mH2)*power2(mh2)) + (10*power2(mH2))/(mh2*power2(mh2 - mH2)) - mh2/power2(
     mstau12 - mu2) + mH2/power2(mstau12 - mu2) + (2*mh2*mu2)/(mstau22*power2(
     mstau12 - mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (mH2*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (4*mH2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (
     mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mh2*
     mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (4*mh2*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (4*mu2*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2))/power2(mstau12 - mu2) - (mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) +
     (mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (
     4*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (4*
     mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (4*mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (4*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau22 - mu2) + (2*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (mstau12*power2(mstau22 - mu2)) + (4*mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau22 - mu2) - (2*mh2*mstau24*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) + (4*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) + (4*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) - (4*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) - (4*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 -
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) + (7*
     mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (7*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (3*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (3*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))
     /power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/power2(mstau22 - mu2) - (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (3*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) +
     (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mstau12,mh2,mu2)*(7*mh2*mstau12*mu2 + (4*mstau12 + 3*mu2)*power2(
     mh2) + (mstau12 - mu2)*power2(mu2) - 2*power3(mh2)))/(mstau12*power2(
     mstau22 - mu2)) + (2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*
     mstau22 - mstau12*mu2) - (2*mu2*DeltaInv(mstau22,mh2,mu2)*power3(mh2))/(
     mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/power2(mstau12 - mu2) - (2*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/(mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) - (2*power3(mH2))/(power2(
     mh2)*power2(mh2 - mH2)) - (DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/
     power2(mstau12 - mu2) + (2*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mH2,mstau12,
     mu2)*(-7*mH2*mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 +
     mu2)*power2(mu2) + 2*power3(mH2)))/(mstau12*power2(mstau22 - mu2)) + (
     DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (DeltaInv(mstau22,
     mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) - (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) - (DeltaInv(mstau22,mh2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mH2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) - (24*DeltaInv(mstau22,
     mH2,mstau12)*power6(mstau2))/(mH2*mstau12) - (DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/(24.*cb) - (ca*mu*sa*
     power2(Pi)*(6/mH2 + 6/mstau12 + (12*mstau22)/(mH2*mstau12) - (2*mh2)/(
     mstau12*mstau22 - mstau12*mu2) + (2*mH2)/(mstau12*mstau22 - mstau12*mu2) +
     7*mH2*DeltaInv(mstau22,mH2,mstau12) + 41*mstau22*DeltaInv(mstau22,mH2,
     mstau12) + (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (24*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (17*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/mstau12 + (4*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12 - mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2)
     + (4*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau22 - mu2) + (2*mH2*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) - (2*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) - mh2*DeltaInv(
     mstau22,mstau12,mh2) + mstau22*DeltaInv(mstau22,mstau12,mh2) - (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (mstau24*DeltaInv(
     mstau22,mstau12,mh2))/mstau12 + (2*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/
     (mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 -
     mu2) + (4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (
     3*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (4*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (2*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + (
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (4*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(-mstau22 + mu2) - (4*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) + (4*mH2)/(-(mh2*mH2) +
     power2(mh2)) - (4*mH2)/power2(mh2 - mH2) - (7*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/(mstau12 - mu2) + (4*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau22 - mu2) + (4*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau12*mstau22 - mstau12*mu2) + power2(mH2)/((-mh2 + mH2)*power2(mh2)) +
     (5*power2(mH2))/(mh2*power2(mh2 - mH2)) - mh2/power2(mstau12 - mu2) + mH2/
     power2(mstau12 - mu2) + (2*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (2*
     mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (mH2*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau12 - mu2) + (4*mH2*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau12 - mu2) + (mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) - (mh2*mstau24*DeltaInv(mstau22,mstau12,
     mh2))/power2(mstau12 - mu2) - (4*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/power2(mstau12 - mu2) - (mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (4*mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/
     power2(mstau12 - mu2) - (mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)
     )/power2(mstau12 - mu2) - (mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/
     power2(mstau12 - mu2) + (mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)
     )/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     power2(mstau12 - mu2) - (4*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/
     power2(mstau12 - mu2) + (4*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (4*
     mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (4*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau22 - mu2) + (2*mH2*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (4*mh2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) - (2*mh2*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) + (4*
     mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) + (4*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(
     mstau22 - mu2)) - (4*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     power2(mstau22 - mu2) - (4*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau22 - mu2) + (DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/(-mstau22 + mu2) + (7*mh2*DeltaInv(mstau22,mh2,mu2)*power2(
     mu2))/power2(mstau12 - mu2) - (7*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2)
     )/power2(mstau12 - mu2) + (3*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*mH2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) -
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (3*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/
     (mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau12,mh2,mu2)*(7*mh2*
     mstau12*mu2 + (4*mstau12 + 3*mu2)*power2(mh2) + (mstau12 - mu2)*power2(
     mu2) - 2*power3(mh2)))/(mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (2*
     mu2*DeltaInv(mstau22,mh2,mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (2*
     mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 -
     mu2)) - (2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - power3(mH2)/(power2(mh2)*power2(mh2 - mH2)) - (DeltaInv(
     mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (2*mu2*DeltaInv(
     mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + (2*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2))
     + (mu2*DeltaInv(mH2,mstau12,mu2)*(-7*mH2*mstau12*mu2 - (4*mstau12 + 3*mu2)
     *power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 2*power3(mH2)))/(mstau12*
     power2(mstau22 - mu2)) + (DeltaInv(mstau22,mh2,mu2)*power3(mu2))/power2(
     mstau12 - mu2) - (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 -
     mu2) + (DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) -
     (DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) - (
     DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     12*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) - (
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (
     DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)))/24.
      - (mu*sb*power2(sa)*(-7/mh2 + 7/(mh2 - mH2) + 120/mH2 + 168/mstau12 + (
     336*mstau22)/(mH2*mstau12) - (28*mh2)/(mstau12*mstau22 - mstau12*mu2) + (
     28*mH2)/(mstau12*mstau22 - mstau12*mu2) + 182*mH2*DeltaInv(mstau22,mH2,
     mstau12) + 1162*mstau22*DeltaInv(mstau22,mH2,mstau12) + (28*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (672*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/mH2 + (490*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 +
     (56*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (42*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (56*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(mstau22 - mu2) + (28*mH2*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(-mstau12 + mu2) + (28*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(-mstau12 + mu2) - (28*mH2*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12*mstau22 - mstau12*mu2) - 14*mh2*DeltaInv(mstau22,
     mstau12,mh2) + 14*mstau22*DeltaInv(mstau22,mstau12,mh2) - (28*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (14*mstau24*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (28*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12 - mu2) + (28*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 -
     mu2) + (56*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (
     42*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (56*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (28*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) - (7*mH2)/
     power2(mh2) - (7*mstau12)/power2(mh2) + (7*mH2*mstau12)/((-mh2 + mH2)*
     power2(mh2)) - (7*mstau22)/power2(mh2) + (7*mH2*mstau22)/((-mh2 + mH2)*
     power2(mh2)) + (14*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (
     42*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (56*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (56*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (32*mH2)/(-(mh2*mH2) + power2(mh2)) + (7*mstau12)/(-(mh2*mH2) + power2(
     mh2)) + (7*mstau22)/(-(mh2*mH2) + power2(mh2)) - (32*mH2)/power2(mh2 -
     mH2) - (182*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (42*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (56*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (56*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (21*
     power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (46*power2(mH2))/(mh2*power2(mh2
     - mH2)) - (14*mh2)/power2(mstau12 - mu2) + (14*mH2)/power2(mstau12 - mu2)
     + (28*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (28*mH2*mu2)/(mstau22*
     power2(mstau12 - mu2)) + (14*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) + (56*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))
     /power2(mstau12 - mu2) + (14*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) - (14*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) - (56*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau12 - mu2) - (14*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (56*mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/
     power2(mstau12 - mu2) - (14*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) - (14*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau12 - mu2) + (14*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/power2(mstau12 - mu2) + (14*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (56*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (56*mh2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (56*mH2*mu2)/(mstau12*power2(mstau22 -
     mu2)) - (56*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22
     - mu2) + (28*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     power2(mstau22 - mu2)) + (56*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)
     )/power2(mstau22 - mu2) - (28*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12*power2(mstau22 - mu2)) + (56*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau22 - mu2) + (56*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) - (56*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) - (
     56*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (14*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22
     - mu2) + (14*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) +
     (98*mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (
     98*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (42*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (42*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (42*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2)
     )/power2(mstau22 - mu2) - (28*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/power2(mstau22 - mu2) - (28*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (14*mstau24*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (42*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2)
     + (28*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 -
     mu2) + (28*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (14*mstau24*DeltaInv(mstau22,mstau12,mh2)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (14*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     14*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mstau12,mh2,mu2)*(7*mh2*mstau12*mu2 + (
     4*mstau12 + 3*mu2)*power2(mh2) + (mstau12 - mu2)*power2(mu2) - 2*power3(
     mh2)))/(mstau12*power2(mstau22 - mu2)) + (28*DeltaInv(mstau22,mstau12,mh2)
     *power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (28*mu2*DeltaInv(mstau22,
     mh2,mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (28*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))
     - (28*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - (14*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (14*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (28*
     mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2))
     + (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mH2,mstau12,mu2)*(-7*mH2*mstau12*mu2 -
     (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 2*power3(
     mH2)))/(mstau12*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mh2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) - (14*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/power2(mstau12 - mu2) + (14*DeltaInv(mstau22,mH2,mstau12)*power3(
     mu2))/power2(mstau22 - mu2) - (14*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/power2(mstau22 - mu2) - (14*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(mstau22,mH2,mu2)*power4(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (336*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/(mH2*mstau12) - (14*DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) + (14*DeltaInv(mstau22,mstau12,mh2)*power6(
     mstau2))/power2(mstau12 - mu2)))/(8.*cb) - (ca*mu*sa*(-7/mh2 + 7/(mh2 -
     mH2) + 120/mH2 + 168/mstau12 + (336*mstau22)/(mH2*mstau12) - (56*mh2)/(
     mstau12*mstau22 - mstau12*mu2) + (56*mH2)/(mstau12*mstau22 - mstau12*mu2)
     + 196*mH2*DeltaInv(mstau22,mH2,mstau12) + 1148*mstau22*DeltaInv(mstau22,
     mH2,mstau12) + (56*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (
     672*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (476*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 + (112*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12 - mu2) + (84*mstau24*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12 - mu2) + (112*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau22
     - mu2) + (56*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (
     56*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) - (56*mH2*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) -
     28*mh2*DeltaInv(mstau22,mstau12,mh2) + 28*mstau22*DeltaInv(mstau22,
     mstau12,mh2) - (56*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (
     28*mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (56*mh2*mu2*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12 - mu2) + (56*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12 - mu2) + (112*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) + (84*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-
     mstau12 + mu2) + (112*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau22
     + mu2) + (56*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 -
     mstau12*mu2) - (7*mH2)/power2(mh2) - (7*mstau12)/power2(mh2) + (7*mH2*
     mstau12)/((-mh2 + mH2)*power2(mh2)) - (7*mstau22)/power2(mh2) + (7*mH2*
     mstau22)/((-mh2 + mH2)*power2(mh2)) + (28*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/mstau12 + (84*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-
     mstau12 + mu2) + (112*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau22
     + mu2) - (112*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*
     mstau22 - mstau12*mu2) + (32*mH2)/(-(mh2*mH2) + power2(mh2)) + (7*mstau12)
     /(-(mh2*mH2) + power2(mh2)) + (7*mstau22)/(-(mh2*mH2) + power2(mh2)) - (
     32*mH2)/power2(mh2 - mH2) - (196*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)
     )/mstau12 + (84*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2)
     + (112*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (112*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (21*power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (46*power2(
     mH2))/(mh2*power2(mh2 - mH2)) - (28*mh2)/power2(mstau12 - mu2) + (28*mH2)/
     power2(mstau12 - mu2) + (56*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (
     56*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (28*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (112*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (28*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (28*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (112*mh2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (28*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (112*mu2*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2))/power2(mstau12 - mu2) - (28*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) -
     (28*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) +
     (28*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 -
     mu2) + (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 -
     mu2) - (112*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 -
     mu2) + (112*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (112*mH2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (112*mH2*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau22 - mu2) + (56*mH2*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (112*mh2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) - (56*mh2*
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2))
     + (112*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau22 -
     mu2) + (112*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(
     mstau12*power2(mstau22 - mu2)) - (112*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau22 - mu2) - (112*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (28*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau22 - mu2) + (28*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) + (196*mh2*DeltaInv(mstau22,
     mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (196*mH2*DeltaInv(mstau22,
     mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (84*DeltaInv(mstau22,mh2,
     mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (84*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (84*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(
     mstau22 - mu2) - (56*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/
     power2(mstau22 - mu2) - (56*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (28*mstau24*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (84*
     mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (
     56*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 -
     mu2) + (56*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (28*mstau24*DeltaInv(mstau22,mstau12,mh2)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (28*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     28*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (28*mu2*DeltaInv(mstau12,mh2,mu2)*(7*mh2*mstau12*mu2 + (
     4*mstau12 + 3*mu2)*power2(mh2) + (mstau12 - mu2)*power2(mu2) - 2*power3(
     mh2)))/(mstau12*power2(mstau22 - mu2)) + (56*DeltaInv(mstau22,mstau12,mh2)
     *power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (56*mu2*DeltaInv(mstau22,
     mh2,mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2)) + (28*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (56*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))
     - (56*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - (14*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (28*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (56*
     mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2))
     + (56*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (28*mu2*DeltaInv(mH2,mstau12,mu2)*(-7*mH2*mstau12*mu2 -
     (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 2*power3(
     mH2)))/(mstau12*power2(mstau22 - mu2)) + (28*DeltaInv(mstau22,mh2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) - (28*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/power2(mstau12 - mu2) + (28*DeltaInv(mstau22,mH2,mstau12)*power3(
     mu2))/power2(mstau22 - mu2) - (28*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/power2(mstau22 - mu2) - (28*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (28*DeltaInv(mstau22,mH2,mu2)*power4(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (336*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/(mH2*mstau12) - (28*DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) + (28*DeltaInv(mstau22,mstau12,mh2)*power6(
     mstau2))/power2(mstau12 - mu2)))/16. + (3*ca*mu*log(mh2/Q2)*log(mstau22/
     Q2)*power3(sa)*(-mstau22 + DeltaInv(mstau22,mstau12,mh2)*(-3*mh2*mstau12*
     mstau22 - 2*mh2*mstau24 - mstau12*mstau24 + mstau22*power2(mh2) + power6(
     mstau2))))/(2.*mh2*mstau12) + (3*mu*sb*log(mh2/Q2)*log(mstau22/Q2)*power4(
     sa)*(-mstau22 + DeltaInv(mstau22,mstau12,mh2)*(-3*mh2*mstau12*mstau22 - 2*
     mh2*mstau24 - mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2))))/(
     2.*cb*mh2*mstau12) + (9*ca*mu*log(mstau22/Q2)*power3(sa)*(-((mh2 + mH2)*
     mstau22) + mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*
     mh2*mstau24 - 3*mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)) +
     mh2*DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24
     - 3*mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(2.*mh2*mH2*
     mstau12) - (3*ca*mu*power2(log(mstau22/Q2))*power3(sa)*(-((mh2 + mH2)*
     mstau22) + mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*
     mh2*mstau24 - 3*mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)) +
     mh2*DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24
     - 3*mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(4.*mh2*mH2*
     mstau12) + (9*mu*sb*log(mstau22/Q2)*power4(sa)*(-((mh2 + mH2)*mstau22) +
     mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*mh2*mstau24
     - 3*mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)) + mh2*
     DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24 - 3*
     mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(2.*cb*mh2*mH2*
     mstau12) - (3*mu*sb*power2(log(mstau22/Q2))*power4(sa)*(-((mh2 + mH2)*
     mstau22) + mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*
     mh2*mstau24 - 3*mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)) +
     mh2*DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24
     - 3*mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(4.*cb*mh2*
     mH2*mstau12) + (3*ca*mu*log(mH2/Q2)*log(mstau22/Q2)*power3(sa)*(-mstau22 +
     DeltaInv(mstau22,mH2,mstau12)*(-3*mH2*mstau12*mstau22 - 2*mH2*mstau24 -
     mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(2.*mH2*mstau12)
     + (3*mu*sb*log(mH2/Q2)*log(mstau22/Q2)*power4(sa)*(-mstau22 + DeltaInv(
     mstau22,mH2,mstau12)*(-3*mH2*mstau12*mstau22 - 2*mH2*mstau24 - mstau12*
     mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(2.*cb*mH2*mstau12) - (
     3*ca*mu*log(mstau12/Q2)*log(mstau22/Q2)*power3(sa)*(-((mh2 + mH2)*mstau22)
     + mH2*DeltaInv(mstau22,mstau12,mh2)*(-2*mstau12*mstau24 - 2*mh2*(mstau12*
     mstau22 + mstau24) + mstau22*power2(mh2) + power6(mstau2)) + mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-2*mstau12*mstau24 - 2*mH2*(mstau12*mstau22 +
     mstau24) + mstau22*power2(mH2) + power6(mstau2))))/(mh2*mH2*mstau12) - (3*
     mu*sb*log(mstau12/Q2)*log(mstau22/Q2)*power4(sa)*(-((mh2 + mH2)*mstau22) +
     mH2*DeltaInv(mstau22,mstau12,mh2)*(-2*mstau12*mstau24 - 2*mh2*(mstau12*
     mstau22 + mstau24) + mstau22*power2(mh2) + power6(mstau2)) + mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-2*mstau12*mstau24 - 2*mH2*(mstau12*mstau22 +
     mstau24) + mstau22*power2(mH2) + power6(mstau2))))/(cb*mh2*mH2*mstau12) -
     (ca*mu*power2(Pi)*power3(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 - 2*
     mh2*mstau22 - 2*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*mh2*
     mstau12*mstau22 - 3*mh2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2)
     + power3(mh2) + 2*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-7*
     mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(
     mH2) + power3(mH2) + 2*power6(mstau2))))/(4.*mh2*mH2*mstau12) - (mu*sb*
     power2(Pi)*power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 - 2*mh2*
     mstau22 - 2*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*mh2*
     mstau12*mstau22 - 3*mh2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2)
     + power3(mh2) + 2*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-7*
     mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(
     mH2) + power3(mH2) + 2*power6(mstau2))))/(4.*cb*mh2*mH2*mstau12) - (3*ca*
     mu*power3(sa)*(-14*mh2*mH2 - 5*mh2*mstau12 - 5*mH2*mstau12 - 14*mh2*
     mstau22 - 14*mH2*mstau22 + 7*mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*mh2*
     mstau12*mstau22 - 3*mh2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2)
     + power3(mh2) + 2*power6(mstau2)) + 7*mh2*DeltaInv(mstau22,mH2,mstau12)*(-
     7*mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*
     power2(mH2) + power3(mH2) + 2*power6(mstau2))))/(2.*mh2*mH2*mstau12) - (3*
     mu*sb*power4(sa)*(-14*mh2*mH2 - 5*mh2*mstau12 - 5*mH2*mstau12 - 14*mh2*
     mstau22 - 14*mH2*mstau22 + 7*mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*mh2*
     mstau12*mstau22 - 3*mh2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2)
     + power3(mh2) + 2*power6(mstau2)) + 7*mh2*DeltaInv(mstau22,mH2,mstau12)*(-
     7*mH2*mstau12*mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*
     power2(mH2) + power3(mH2) + 2*power6(mstau2))))/(2.*cb*mh2*mH2*mstau12) +
     (9*ca*mu*log(mstau12/Q2)*power3(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*
     mstau12 - 3*mh2*mstau22 - 3*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,
     mh2)*(-5*mstau12*mstau24 - 5*mh2*(2*mstau12*mstau22 + mstau24) + (-mstau12
     + mstau22)*power2(mh2) + power3(mh2) + 3*power6(mstau2)) + mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-5*mstau12*mstau24 - 5*mH2*(2*mstau12*mstau22 +
     mstau24) + (-mstau12 + mstau22)*power2(mH2) + power3(mH2) + 3*power6(
     mstau2))))/(2.*mh2*mH2*mstau12) - (3*ca*mu*power2(log(mstau12/Q2))*power3(
     sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 - 3*mh2*mstau22 - 3*mH2*
     mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-5*mstau12*mstau24 - 5*mh2*(
     2*mstau12*mstau22 + mstau24) + (-mstau12 + mstau22)*power2(mh2) + power3(
     mh2) + 3*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-5*mstau12*
     mstau24 - 5*mH2*(2*mstau12*mstau22 + mstau24) + (-mstau12 + mstau22)*
     power2(mH2) + power3(mH2) + 3*power6(mstau2))))/(4.*mh2*mH2*mstau12) + (9*
     mu*sb*log(mstau12/Q2)*power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 -
     3*mh2*mstau22 - 3*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-5*
     mstau12*mstau24 - 5*mh2*(2*mstau12*mstau22 + mstau24) + (-mstau12 +
     mstau22)*power2(mh2) + power3(mh2) + 3*power6(mstau2)) + mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-5*mstau12*mstau24 - 5*mH2*(2*mstau12*mstau22 +
     mstau24) + (-mstau12 + mstau22)*power2(mH2) + power3(mH2) + 3*power6(
     mstau2))))/(2.*cb*mh2*mH2*mstau12) - (3*mu*sb*power2(log(mstau12/Q2))*
     power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 - 3*mh2*mstau22 - 3*
     mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-5*mstau12*mstau24 - 5*
     mh2*(2*mstau12*mstau22 + mstau24) + (-mstau12 + mstau22)*power2(mh2) +
     power3(mh2) + 3*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-5*
     mstau12*mstau24 - 5*mH2*(2*mstau12*mstau22 + mstau24) + (-mstau12 +
     mstau22)*power2(mH2) + power3(mH2) + 3*power6(mstau2))))/(4.*cb*mh2*mH2*
     mstau12) + (mu*sb*log(mH2/Q2)*log(mstau12/Q2)*power2(sa)*(-12/mH2 - 12/
     mstau12 + 4/(mstau12 - mstau22) - (12*mstau22)/(mH2*mstau12) + (4*mu2)/((
     mstau12 - mstau22)*(mstau12 - mu2)) - (2*mH2)/(mstau12*mstau22 - mstau12*
     mu2) - mH2/power2(mstau12 - mu2) + (4*mH2*mu2)/(mstau12*power2(mstau22 -
     mu2)) - (2*mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*
     mH2*(mstau12 + mu2) + power2(mH2)))/(mstau12*power2(mstau22 - mu2)) +
     DeltaInv(mstau22,mH2,mstau12)*((-12*(6*mstau12*mstau22 + mstau24))/mstau12
     + power2(mH2)*(3/(-mstau12 + mu2) + 4/(-mstau22 + mu2) - (4*mstau22)/(
     mstau12*mstau22 - mstau12*mu2) - (2*mstau22)/power2(mstau12 - mu2) - mu2/
     power2(mstau12 - mu2) + (12 + (4*mstau22*mu2)/power2(mstau22 - mu2))/
     mstau12 + (4*mu2)/power2(mstau22 - mu2)) + mH2*(-10 + (2*mu2)/(mstau12 -
     mu2) + mstau22/(-mstau12 + mu2) + (4*mstau22)/(-mstau22 + mu2) + (2*
     mstau24)/(mstau12*mstau22 - mstau12*mu2) + mstau24/power2(mstau12 - mu2) -
     (3*mstau22*mu2)/power2(mstau12 - mu2) + (-12*mstau22 - (2*mstau24*mu2)/
     power2(mstau22 - mu2))/mstau12 + (4*mstau22*mu2)/power2(mstau22 - mu2)) +
     (2/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2) - (2*mu2)/(
     mstau12*power2(mstau22 - mu2)))*power3(mH2) + (12*(-(mstau12*mstau24) +
     power6(mstau2)))/(mH2*mstau12))))/(4.*cb) - (mu*sb*log(mH2/Q2)*log(
     mstau22/Q2)*power2(sa)*(4/(mstau12 - mstau22) - (12*mstau22)/(mH2*mstau12)
     + (4*mu2)/((mstau12 - mstau22)*(mstau22 - mu2)) + mH2/(mstau12*(-mstau22 +
     mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mH2*mu2*DeltaInv(
     mstau22,mH2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mH2*(mstau22 + mu2) + power2(
     mH2)))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2)/(mstau12*power2(mstau22
     - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(-36*mstau22 - (24*mstau24)/
     mstau12 - ((mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mH2))/(mstau12*
     power2(mstau22 - mu2)) + mH2*(-1 + mstau24/(mstau12*mstau22 - mstau12*mu2)
     + 2*mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2) - mu2/power2(mstau12 -
     mu2) + mu2/power2(mstau22 - mu2)) + (12*mstau22 - (mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 - mu2)) + ((mstau22 -
     2*mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (12*(-(mstau12*
     mstau24) + power6(mstau2)))/(mH2*mstau12))))/(4.*cb) + (mu*sb*log(mC2/Q2)*
     log(msntau2/Q2)*(1/mC2 + 1/msntau2 + mstau22/(mC2*msntau2) + (2*mC2)/(
     msntau2*mstau22 - msntau2*mu2) + mC2/power2(msntau2 - mu2) - (4*mC2*mu2)/(
     msntau2*power2(mstau22 - mu2)) + (2*mC2*mu2*DeltaInv(mC2,msntau2,mu2)*(
     mu2*(-2*msntau2 + mu2) - 2*mC2*(msntau2 + mu2) + power2(mC2)))/(msntau2*
     power2(mstau22 - mu2)) + DeltaInv(mstau22,mC2,msntau2)*(6*mstau22 +
     mstau24/msntau2 + mC2*(-1 + mstau22/(msntau2 - mu2) + (4*mstau22)/(mstau22
     - mu2) + (2*mu2)/(-msntau2 + mu2) - (2*mstau24)/(msntau2*mstau22 -
     msntau2*mu2) - mstau24/power2(msntau2 - mu2) + (3*mstau22*mu2)/power2(
     msntau2 - mu2) + (mstau22 + (2*mstau24*mu2)/power2(mstau22 - mu2))/msntau2
     - (4*mstau22*mu2)/power2(mstau22 - mu2)) + power2(mC2)*(3/(msntau2 - mu2)
     + 4/(mstau22 - mu2) + (4*mstau22)/(msntau2*mstau22 - msntau2*mu2) + (2*
     mstau22)/power2(msntau2 - mu2) + mu2/power2(msntau2 - mu2) - (4*mu2)/
     power2(mstau22 - mu2) - power2(mstau22 + mu2)/(msntau2*power2(mstau22 -
     mu2))) + (-2/(msntau2*mstau22 - msntau2*mu2) - 1/power2(msntau2 - mu2) + (
     2*mu2)/(msntau2*power2(mstau22 - mu2)))*power3(mC2) + (mstau24 - power6(
     mstau2)/msntau2)/mC2)))/(2.*cb) - (mu*sb*log(mC2/Q2)*log(mstau22/Q2)*(
     mstau22/(mC2*msntau2) + mC2/(msntau2*mstau22 - msntau2*mu2) + (2*mC2*mu2)/
     (mstau22*power2(msntau2 - mu2)) - (2*mC2*mu2*DeltaInv(mstau22,mC2,mu2)*(
     mu2*(-2*mstau22 + mu2) - 2*mC2*(mstau22 + mu2) + power2(mC2)))/(mstau22*
     power2(msntau2 - mu2)) - (mC2*mu2)/(msntau2*power2(mstau22 - mu2)) +
     DeltaInv(mstau22,mC2,msntau2)*(3*mstau22 + (2*mstau24)/msntau2 + ((msntau2
     + 2*mstau22)*(mstau22 - 2*mu2)*power2(mC2))/(msntau2*power2(mstau22 - mu2)
     ) + mC2*(1 + (2*mstau22)/(mstau22 - mu2) + (2*mstau22)/(-msntau2 + mu2) +
     (2*mstau22*mu2)/power2(msntau2 - mu2) - (mstau22 + (mstau24*(mstau22 - 2*
     mu2))/power2(mstau22 - mu2))/msntau2 - (2*mstau22*mu2)/power2(mstau22 -
     mu2) - power2(mu2)/power2(mstau22 - mu2)) - ((mstau22 - 2*mu2)*power3(mC2)
     )/(msntau2*power2(mstau22 - mu2)) + (mstau24 - power6(mstau2)/msntau2)/
     mC2)))/(2.*cb) - (mu*sb*power2(sa)*power2(log(mstau22/Q2))*((12*mstau22)/(
     mH2*mstau12) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + mstau22*DeltaInv(
     mstau22,mstau12,mh2) - (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(-mstau12 + mu2) + (DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/mstau12 + (3*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*
     mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (2*mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (mstau24*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (5*mu2*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(-mstau22 + mu2) - (10*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (5*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) - (mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mh2,
     mu2)*(mh2*(10*mstau22 - mu2)*mu2 + 5*(mstau22 + mu2)*power2(mh2) + (
     mstau22 - mu2)*power2(mu2) - 3*power3(mh2)))/(mstau22*power2(mstau12 -
     mu2)) + (3*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(
     mstau12 - mu2)) - (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 -
     mu2) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12
     - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 -
     mu2) + (DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 -
     mu2)) + (DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 -
     mu2) + DeltaInv(mstau22,mH2,mstau12)*((23*mstau24)/mstau12 + (3*mstau24)/(
     mstau12 - mu2) + (mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(
     mstau22 - mu2) + (mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) +
     mstau22*(11 + (2*mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 -
     mu2)) + mH2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12
     - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(
     mstau22 - mu2) + (2*mstau22*(-5 - power2(mu2)/power2(mstau22 - mu2)))/
     mstau12) + power2(mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(
     mu2)/power2(mstau22 - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) -
     power6(mstau2)/power2(mstau12 - mu2) + (36*mstau24 - (12*power6(mstau2))/
     mstau12)/mH2)))/(8.*cb) + (mu*sb*log(mstau22/Q2)*power2(sa)*((36*mstau22)/
     (mH2*mstau12) - 9*mh2*DeltaInv(mstau22,mstau12,mh2) + 3*mstau22*DeltaInv(
     mstau22,mstau12,mh2) - (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (9*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(-mstau12 + mu2) - (2*mstau22)/power2(mh2) + (2*mH2*
     mstau22)/((-mh2 + mH2)*power2(mh2)) + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/mstau12 + (2*mstau22)/(-(mh2*mH2) + power2(mh2)) + (9*mh2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (9*mH2*mu2)/(mstau22*power2(mstau12
     - mu2)) - (6*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) - (3*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) - (3*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2)
     + (3*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 -
     mu2) - (15*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 -
     mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) - (
     30*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (15*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (9*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22
     - mu2) + (6*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(
     mstau22 - mu2) + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*mu2*DeltaInv(mstau22,mh2,mu2)*(mh2*mu2*(-10*mstau22 + mu2) - 5*(mstau22
     + mu2)*power2(mh2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mh2)))/(
     mstau22*power2(mstau12 - mu2)) + (9*mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) + (3*mH2*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mH2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2) + 3*DeltaInv(mstau22,
     mH2,mstau12)*((23*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (
     mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (
     mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(11 + (2*
     mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 +
     (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (
     mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2)
     + (2*mstau22*(-5 - power2(mu2)/power2(mstau22 - mu2)))/mstau12) + power2(
     mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22
     - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/
     power2(mstau12 - mu2) + (36*mstau24 - (12*power6(mstau2))/mstau12)/mH2)))/
     (4.*cb) - (ca*mu*sa*power2(log(mstau22/Q2))*((6*mstau22)/(mH2*mstau12) -
     3*mh2*DeltaInv(mstau22,mstau12,mh2) + mstau22*DeltaInv(mstau22,mstau12,
     mh2) - (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (mstau24*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12 - mu2) + (3*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-
     mstau12 + mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (3*
     mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*mH2*mu2)/(mstau22*power2(
     mstau12 - mu2)) - (2*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) - (mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) - (mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) + (mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/
     power2(mstau12 - mu2) - (5*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/
     power2(mstau12 - mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-
     mstau22 + mu2) - (10*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(
     mstau12 - mu2) - (5*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (3*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mh2,mu2)*(mh2*(10*mstau22 - mu2)*
     mu2 + 5*(mstau22 + mu2)*power2(mh2) + (mstau22 - mu2)*power2(mu2) - 3*
     power3(mh2)))/(mstau22*power2(mstau12 - mu2)) + (3*mu2*DeltaInv(mstau22,
     mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) - (DeltaInv(mstau22,
     mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (mH2*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,mH2,
     mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2) + DeltaInv(mstau22,mH2,
     mstau12)*((11*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (mstau24*
     mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (mstau24*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(5 + (2*mu2)/(-
     mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 + (3*
     mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (mstau22*
     mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2) + (
     mstau22*(-4 - (2*power2(mu2))/power2(mstau22 - mu2)))/mstau12) + power2(
     mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22
     - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/
     power2(mstau12 - mu2) + (18*mstau24 - (6*power6(mstau2))/mstau12)/mH2)))/
     8. + (ca*mu*sa*log(mstau22/Q2)*((18*mstau22)/(mH2*mstau12) - 9*mh2*
     DeltaInv(mstau22,mstau12,mh2) + 3*mstau22*DeltaInv(mstau22,mstau12,mh2) -
     (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (3*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (6*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(mstau12 - mu2) + (9*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-
     mstau12 + mu2) - mstau22/power2(mh2) + (mH2*mstau22)/((-mh2 + mH2)*power2(
     mh2)) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + mstau22/(-
     (mh2*mH2) + power2(mh2)) + (9*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (
     9*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (6*mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (3*mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) - (3*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (3*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (15*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (3*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) - (30*mH2*DeltaInv(mstau22,mH2,
     mu2)*power2(mu2))/power2(mstau12 - mu2) - (15*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (9*mh2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (6*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) +
     (6*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(
     mstau22,mh2,mu2)*(mh2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)*power2(
     mh2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mh2)))/(mstau22*power2(
     mstau12 - mu2)) + (9*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*
     power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(
     mstau12 - mu2) + (3*mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/
     power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*power6(
     mstau2))/power2(mstau12 - mu2) + 3*DeltaInv(mstau22,mH2,mstau12)*((11*
     mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (mstau24*mu2)/power2(
     mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (mstau24*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + mstau22*(5 + (2*mu2)/(-mstau12 + mu2) - (
     2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 + (3*mstau22)/(mstau12 -
     mu2) + (2*mstau24)/power2(mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 -
     mu2) - (3*power2(mu2))/power2(mstau22 - mu2) + (mstau22*(-4 - (2*power2(
     mu2))/power2(mstau22 - mu2)))/mstau12) + power2(mH2)*(-(mstau22/power2(
     mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22 - mu2))/mstau12) +
     power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/power2(mstau12 - mu2) +
     (18*mstau24 - (6*power6(mstau2))/mstau12)/mH2)))/4. - (mu*sb*log(mA2/Q2)*
     log(mstau12/Q2)*(2/mA2 + 2/mstau12 - 4/(mstau12 - mstau22) + (2*mstau22)/(
     mA2*mstau12) + (4*mu2)/((mstau12 - mstau22)*(-mstau12 + mu2)) - (2*mA2)/(
     mstau12*mstau22 - mstau12*mu2) - mA2/power2(mstau12 - mu2) + (4*mA2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (2*mA2*mu2*DeltaInv(mA2,mstau12,mu2)*(
     mu2*(-2*mstau12 + mu2) - 2*mA2*(mstau12 + mu2) + power2(mA2)))/(mstau12*
     power2(mstau22 - mu2)) + DeltaInv(mA2,mstau22,mstau12)*((2*(6*mstau12*
     mstau22 + mstau24))/mstau12 + power2(mA2)*(3/(-mstau12 + mu2) + 4/(-
     mstau22 + mu2) - (4*mstau22)/(mstau12*mstau22 - mstau12*mu2) - (2*mstau22)
     /power2(mstau12 - mu2) - mu2/power2(mstau12 - mu2) + (-2 + (4*mstau22*mu2)
     /power2(mstau22 - mu2))/mstau12 + (4*mu2)/power2(mstau22 - mu2)) + mA2*(4
     + (2*mu2)/(mstau12 - mu2) + mstau22/(-mstau12 + mu2) + (4*mstau22)/(-
     mstau22 + mu2) + (2*mstau24)/(mstau12*mstau22 - mstau12*mu2) + mstau24/
     power2(mstau12 - mu2) - (3*mstau22*mu2)/power2(mstau12 - mu2) + (2*mstau22
     - (2*mstau24*mu2)/power2(mstau22 - mu2))/mstau12 + (4*mstau22*mu2)/power2(
     mstau22 - mu2)) + (2/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 -
     mu2) - (2*mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mA2) + (2*mstau24 -
     (2*power6(mstau2))/mstau12)/mA2)))/(4.*cb) + (mu*sb*log(mA2/Q2)*log(
     mstau22/Q2)*(-4/(mstau12 - mstau22) + (2*mstau22)/(mA2*mstau12) + mA2/(
     mstau12*(-mstau22 + mu2)) + (4*mu2)/((mstau12 - mstau22)*(-mstau22 + mu2))
     - (2*mA2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mA2*mu2*DeltaInv(mA2,
     mstau22,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mA2*(mstau22 + mu2) + power2(mA2)
     ))/(mstau22*power2(mstau12 - mu2)) + (mA2*mu2)/(mstau12*power2(mstau22 -
     mu2)) + DeltaInv(mA2,mstau22,mstau12)*(6*mstau22 + (4*mstau24)/mstau12 - (
     (mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mA2))/(mstau12*power2(
     mstau22 - mu2)) + ((mstau22 - 2*mu2)*power3(mA2))/(mstau12*power2(mstau22
     - mu2)) + (mA2*(-(mstau12*(mstau22 - 2*mu2)*mu2*(2*mstau24 + 3*mstau22*
     mu2)) + (mstau22 - 2*mu2)*(mstau24 + 6*mstau22*mu2)*power2(mstau12) - 3*
     mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(mu2)*(-2*mstau24*mu2 +
     4*mu2*power2(mstau22) + mstau22*(mstau24 - 2*power2(mu2)) - 2*power3(
     mstau22))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (2*
     mstau24 - (2*power6(mstau2))/mstau12)/mA2)))/(4.*cb) + (mu*sb*power2(log(
     mstau22/Q2))*(2/msntau2 + 2/mstau12 + (2*mstau22)/(mC2*msntau2) - (2*
     mstau22)/(mA2*mstau12) + (6*mstau22)/(mH2*mstau12) + 4/(mstau22 - mu2) +
     6*mC2*DeltaInv(mstau22,mC2,msntau2) + (2*mC2*mstau22*DeltaInv(mstau22,mC2,
     msntau2))/msntau2 + (6*mstau24*DeltaInv(mstau22,mC2,msntau2))/mC2 + (2*
     mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (6*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/(msntau2 - mu2) + (6*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/(msntau2 - mu2) + (4*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/
     (-msntau2 + mu2) + 3*mH2*DeltaInv(mstau22,mH2,mstau12) + 5*mstau22*
     DeltaInv(mstau22,mH2,mstau12) - (4*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/mstau12 + (18*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (11*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (3*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (-mstau12 + mu2) - (2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/msntau2 -
     (DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (2*mstau22)/power2(
     msntau2 - mu2) - (2*mu2)/power2(msntau2 - mu2) - (6*mC2*mu2)/(mstau22*
     power2(msntau2 - mu2)) + (4*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/
     power2(msntau2 - mu2) + (2*mC2*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/
     power2(msntau2 - mu2) + (2*mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/
     power2(msntau2 - mu2) - (2*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2))/power2(msntau2 - mu2) - (10*mu2*DeltaInv(mstau22,mC2,mu2)*power2(
     mC2))/power2(msntau2 - mu2) + (2*mstau22)/power2(mstau12 - mu2) - (2*mu2)/
     power2(mstau12 - mu2) - (3*mA2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*
     mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (5*mu2*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (4*mu2)/power2(mstau22 -
     mu2) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(mstau22 - mu2) + (
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 - mu2) - (2*power2(
     mu2))/(mstau22*power2(msntau2 - mu2)) - (20*mC2*DeltaInv(mstau22,mC2,mu2)*
     power2(mu2))/power2(msntau2 - mu2) - (10*DeltaInv(mstau22,mC2,mu2)*power2(
     mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (2*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) - (10*mH2*DeltaInv(mstau22,mH2,mu2)*power2(
     mu2))/power2(mstau12 - mu2) - (5*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*
     power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (2*power2(mu2))/(msntau2*
     power2(mstau22 - mu2)) - (2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) -
     (6*mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) -
     (4*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 -
     mu2) - (4*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(msntau2*
     power2(mstau22 - mu2)) + (2*mstau24*DeltaInv(mstau22,mC2,msntau2)*power2(
     mu2))/(msntau2*power2(mstau22 - mu2)) - (3*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mA2,
     mstau22,mu2)*(mA2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)*power2(mA2)
     + (-mstau22 + mu2)*power2(mu2) + 3*power3(mA2)))/(mstau22*power2(mstau12 -
     mu2)) + (6*mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(
     msntau2 - mu2)) + (3*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*
     power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/power2(
     msntau2 - mu2) + (2*mC2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mstau22*
     power2(msntau2 - mu2)) - (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(
     mstau12 - mu2) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2)) + (2*DeltaInv(mstau22,mC2,msntau2)*power3(mu2))/
     power2(mstau22 - mu2) + (DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/
     power2(mstau22 - mu2) + (8*power2(mu2))/power3(-mstau22 + mu2) + (2*
     DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(mstau22*power2(msntau2 - mu2)) + (
     DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     2*DeltaInv(mstau22,mC2,msntau2)*power6(mstau2))/(mC2*msntau2) - (6*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) - (2*DeltaInv(
     mstau22,mC2,msntau2)*power6(mstau2))/power2(msntau2 - mu2) - (DeltaInv(
     mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + DeltaInv(mA2,
     mstau22,mstau12)*((-5*mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (
     mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (
     mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(-3 + (2*
     mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mA2*(3 +
     (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (
     mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2)
     + (mstau22*(4 - (2*power2(mu2))/power2(mstau22 - mu2)))/mstau12) + power2(
     mA2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22
     - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/
     power2(mstau12 - mu2) + (-6*mstau24 + (2*power6(mstau2))/mstau12)/mA2)))/(
     8.*cb) - (mu*sb*log(mstau22/Q2)*(6/msntau2 + 6/mstau12 + 4/mstau22 + (6*
     mstau22)/(mC2*msntau2) - (6*mstau22)/(mA2*mstau12) + (18*mstau22)/(mH2*
     mstau12) + 4/(mstau22 - mu2) + 18*mC2*DeltaInv(mstau22,mC2,msntau2) + (6*
     mC2*mstau22*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (18*mstau24*DeltaInv(
     mstau22,mC2,msntau2))/mC2 + (6*mstau24*DeltaInv(mstau22,mC2,msntau2))/
     msntau2 + (18*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) +
     (18*mstau24*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (12*mstau22*
     mu2*DeltaInv(mstau22,mC2,msntau2))/(-msntau2 + mu2) + 9*mH2*DeltaInv(
     mstau22,mH2,mstau12) + 15*mstau22*DeltaInv(mstau22,mH2,mstau12) - (12*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (54*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/mH2 + (33*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 + (9*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) +
     (9*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) - (6*DeltaInv(mstau22,
     mC2,msntau2)*power2(mC2))/msntau2 - (3*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/mstau12 + (6*mstau22)/power2(msntau2 - mu2) - (6*mu2)/power2(
     msntau2 - mu2) - (18*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (12*mC2*
     mstau24*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (6*mC2*
     mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (6*
     mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) - (6*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) -
     (30*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (6*
     mstau22)/power2(mstau12 - mu2) - (6*mu2)/power2(mstau12 - mu2) - (9*mA2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (9*mH2*mu2)/(mstau22*power2(mstau12
     - mu2)) + (6*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (3*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (3*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2)
     - (3*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 -
     mu2) - (15*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 -
     mu2) - (4*mu2)/power2(mstau22 - mu2) + (6*DeltaInv(mstau22,mC2,msntau2)*
     power2(mu2))/(mstau22 - mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/(mstau22 - mu2) - (6*power2(mu2))/(mstau22*power2(msntau2 - mu2)) -
     (60*mC2*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (
     30*DeltaInv(mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(
     msntau2 - mu2)) - (6*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (30*
     mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (15*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (6*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (6*power2(mu2)
     )/(mstau12*power2(mstau22 - mu2)) - (4*power2(mu2))/(mstau22*power2(
     mstau22 - mu2)) - (18*mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/
     power2(mstau22 - mu2) - (12*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(
     mu2))/power2(mstau22 - mu2) - (12*mC2*mstau22*DeltaInv(mstau22,mC2,
     msntau2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (6*mstau24*
     DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(msntau2*power2(mstau22 - mu2))
     - (9*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2)
     - (6*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 -
     mu2) - (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/(mstau12*power2(mstau22 - mu2)) + (6*DeltaInv(mstau22,mC2,msntau2)*
     power2(mC2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (3*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 -
     mu2)) + (3*mu2*DeltaInv(mA2,mstau22,mu2)*(mA2*mu2*(-10*mstau22 + mu2) - 5*
     (mstau22 + mu2)*power2(mA2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mA2)
     ))/(mstau22*power2(mstau12 - mu2)) + (18*mu2*DeltaInv(mstau22,mC2,mu2)*
     power3(mC2))/(mstau22*power2(msntau2 - mu2)) + (9*mu2*DeltaInv(mstau22,
     mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/power2(msntau2 - mu2) + (6*mC2*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) - (3*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (3*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     6*DeltaInv(mstau22,mC2,msntau2)*power3(mu2))/power2(mstau22 - mu2) + (3*
     DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (8*
     power2(mu2))/power3(-mstau22 + mu2) + (6*DeltaInv(mstau22,mC2,mu2)*power4(
     mu2))/(mstau22*power2(msntau2 - mu2)) + (3*DeltaInv(mstau22,mH2,mu2)*
     power4(mu2))/(mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(mstau22,mC2,
     msntau2)*power6(mstau2))/(mC2*msntau2) - (18*DeltaInv(mstau22,mH2,mstau12)
     *power6(mstau2))/(mH2*mstau12) - (6*DeltaInv(mstau22,mC2,msntau2)*power6(
     mstau2))/power2(msntau2 - mu2) - (3*DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) + 3*DeltaInv(mA2,mstau22,mstau12)*((-5*
     mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (mstau24*mu2)/power2(
     mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (mstau24*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + mstau22*(-3 + (2*mu2)/(-mstau12 + mu2) -
     (2*power2(mu2))/power2(mstau22 - mu2)) + mA2*(3 + (3*mstau22)/(mstau12 -
     mu2) + (2*mstau24)/power2(mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 -
     mu2) - (3*power2(mu2))/power2(mstau22 - mu2) + (mstau22*(4 - (2*power2(
     mu2))/power2(mstau22 - mu2)))/mstau12) + power2(mA2)*(-(mstau22/power2(
     mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22 - mu2))/mstau12) +
     power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/power2(mstau12 - mu2) +
     (-6*mstau24 + (2*power6(mstau2))/mstau12)/mA2)))/(4.*cb) - (ca*mu*sa*log(
     mH2/Q2)*log(mstau22/Q2)*(2/(mstau12 - mstau22) - (6*mstau22)/(mH2*mstau12)
     + (2*mu2)/((mstau12 - mstau22)*(mstau22 - mu2)) + mH2/(mstau12*(-mstau22 +
     mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mH2*mu2*DeltaInv(
     mstau22,mH2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mH2*(mstau22 + mu2) + power2(
     mH2)))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2)/(mstau12*power2(mstau22
     - mu2)) + DeltaInv(mstau22,mH2,mstau12)*((-6*(3*mstau12*mstau22 + 2*
     mstau24))/mstau12 - ((mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mH2))/
     (mstau12*power2(mstau22 - mu2)) + mH2*(-1 + mstau24/(mstau12*mstau22 -
     mstau12*mu2) + 2*mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2) - mu2/
     power2(mstau12 - mu2) + mu2/power2(mstau22 - mu2)) + (6*mstau22 - (
     mstau24*mu2)/power2(mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 -
     mu2)) + ((mstau22 - 2*mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) +
     (-6*mstau24 + (6*power6(mstau2))/mstau12)/mH2)))/4. + (mu*sb*log(mH2/Q2)*
     log(mstau22/Q2)*(4/(mstau12 - mstau22) - (6*mstau22)/(mH2*mstau12) + (4*
     mu2)/((mstau12 - mstau22)*(mstau22 - mu2)) + mH2/(mstau12*(-mstau22 + mu2)
     ) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mH2*mu2*DeltaInv(
     mstau22,mH2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mH2*(mstau22 + mu2) + power2(
     mH2)))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2)/(mstau12*power2(mstau22
     - mu2)) + DeltaInv(mstau22,mH2,mstau12)*((-6*(3*mstau12*mstau22 + 2*
     mstau24))/mstau12 - ((mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mH2))/
     (mstau12*power2(mstau22 - mu2)) + mH2*(-1 + mstau24/(mstau12*mstau22 -
     mstau12*mu2) + 2*mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2) - mu2/
     power2(mstau12 - mu2) + mu2/power2(mstau22 - mu2)) + (6*mstau22 - (
     mstau24*mu2)/power2(mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 -
     mu2)) + ((mstau22 - 2*mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) +
     (-6*mstau24 + (6*power6(mstau2))/mstau12)/mH2)))/(4.*cb) + (ca*mu*sa*log(
     mH2/Q2)*log(mstau12/Q2)*(-6/mH2 - 6/mstau12 + 2/(mstau12 - mstau22) - (6*
     mstau22)/(mH2*mstau12) + (2*mu2)/((mstau12 - mstau22)*(mstau12 - mu2)) - (
     2*mH2)/(mstau12*mstau22 - mstau12*mu2) - mH2/power2(mstau12 - mu2) + (4*
     mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (2*mH2*mu2*DeltaInv(mH2,
     mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mH2*(mstau12 + mu2) + power2(mH2)
     ))/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*((-6*(
     6*mstau12*mstau22 + mstau24))/mstau12 + mH2*(-4 + (2*mu2)/(mstau12 - mu2)
     + mstau22/(-mstau12 + mu2) + (4*mstau22)/(-mstau22 + mu2) + (2*mstau24)/(
     mstau12*mstau22 - mstau12*mu2) + mstau24/power2(mstau12 - mu2) - (3*
     mstau22*mu2)/power2(mstau12 - mu2) + (-6*mstau22 - (2*mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 + (4*mstau22*mu2)/power2(mstau22 - mu2)) + (2/(
     mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2) - (2*mu2)/(
     mstau12*power2(mstau22 - mu2)))*power3(mH2) - (power2(mH2)*(-2*power2(mu2)
     *(-2*mstau22*mu2 + power2(mstau22) + 3*power2(mu2)) + power2(mstau12)*(-
     10*mstau22*mu2 + power2(mstau22) + 13*power2(mu2)) + 4*(mstau22 - 2*mu2)*
     power3(mstau12) + 2*mstau12*(-(mu2*power2(mstau22)) + mstau22*power2(mu2)
     + power3(mstau22) + power3(mu2))))/(mstau12*power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) + (-6*mstau24 + (6*power6(mstau2))/mstau12)/mH2)))/4. - (
     mu*sb*log(mH2/Q2)*log(mstau12/Q2)*(-6/mH2 - 6/mstau12 + 4/(mstau12 -
     mstau22) - (6*mstau22)/(mH2*mstau12) + (4*mu2)/((mstau12 - mstau22)*(
     mstau12 - mu2)) - (2*mH2)/(mstau12*mstau22 - mstau12*mu2) - mH2/power2(
     mstau12 - mu2) + (4*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (2*mH2*mu2*
     DeltaInv(mH2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mH2*(mstau12 + mu2)
     + power2(mH2)))/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,
     mstau12)*((-6*(6*mstau12*mstau22 + mstau24))/mstau12 + mH2*(-4 + (2*mu2)/(
     mstau12 - mu2) + mstau22/(-mstau12 + mu2) + (4*mstau22)/(-mstau22 + mu2) +
     (2*mstau24)/(mstau12*mstau22 - mstau12*mu2) + mstau24/power2(mstau12 -
     mu2) - (3*mstau22*mu2)/power2(mstau12 - mu2) + (-6*mstau22 - (2*mstau24*
     mu2)/power2(mstau22 - mu2))/mstau12 + (4*mstau22*mu2)/power2(mstau22 -
     mu2)) + (2/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2) - (2*
     mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mH2) - (power2(mH2)*(-2*
     power2(mu2)*(-2*mstau22*mu2 + power2(mstau22) + 3*power2(mu2)) + power2(
     mstau12)*(-10*mstau22*mu2 + power2(mstau22) + 13*power2(mu2)) + 4*(mstau22
     - 2*mu2)*power3(mstau12) + 2*mstau12*(-(mu2*power2(mstau22)) + mstau22*
     power2(mu2) + power3(mstau22) + power3(mu2))))/(mstau12*power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + (-6*mstau24 + (6*power6(mstau2))/mstau12)/
     mH2)))/(4.*cb) + (mu*sb*power2(log(msntau2/Q2))*(1/mC2 + 2/msntau2 + (3*
     mstau22)/(mC2*msntau2) + 1/(mstau22 - mu2) + (3*mC2)/(msntau2*mstau22 -
     msntau2*mu2) + mC2/power2(msntau2 - mu2) + mstau22/power2(msntau2 - mu2) -
     (2*mu2)/power2(mstau22 - mu2) - (6*mC2*mu2)/(msntau2*power2(mstau22 - mu2)
     ) - (2*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (mu2*DeltaInv(mC2,
     msntau2,mu2)*(mC2*mu2*(-10*msntau2 + mu2) - 5*(msntau2 + mu2)*power2(mC2)
     + (-msntau2 + mu2)*power2(mu2) + 3*power3(mC2)))/(msntau2*power2(mstau22 -
     mu2)) + (2*power2(mu2))/power3(-msntau2 + mu2) + (DeltaInv(mstau22,mC2,
     msntau2)*(power3(mC2)*(-(power2(mu2)*(8*mstau22*mu2 - 4*power2(mstau22) +
     power2(mu2))) + power2(msntau2)*(-24*mstau22*mu2 + 7*power2(mstau22) + 22*
     power2(mu2)) + 5*(mstau22 - 2*mu2)*power3(msntau2) + msntau2*(-12*mu2*
     power2(mstau22) + 26*mstau22*power2(mu2) + power3(mstau22) - 10*power3(
     mu2))) - (3*(mstau22 - 2*mu2)*power2(msntau2) + 3*(mstau22 - 2*mu2)*
     power2(mu2) + msntau2*(-8*mstau22*mu2 + power2(mstau22) + 13*power2(mu2)))
     *power4(mC2) + power2(mC2)*((-18*mstau22*mu2 + 9*power2(mstau22) - power2(
     mu2))*power3(msntau2) + power2(mu2)*(6*mstau24*mu2 - 2*mu2*power2(mstau22)
     - mstau22*(3*mstau24 + power2(mu2)) + power3(mstau22)) + power2(msntau2)*(
     -3*mstau22*mstau24 + 6*mstau24*mu2 - 26*mu2*power2(mstau22) + 41*mstau22*
     power2(mu2) + 3*power3(mstau22)) + msntau2*(4*mstau22*mu2*(mstau24 - 4*
     power2(mu2)) - 11*mstau24*power2(mu2) + power2(mstau22)*(mstau24 + 7*
     power2(mu2)) + 2*mu2*power3(mstau22) + power4(mu2))) + power2(msntau2 -
     mu2)*power2(mstau22 - mu2)*(5*msntau2*mstau24 - 3*power6(mstau2)) + mC2*(
     mstau24*power2(mu2)*(-8*mstau22*mu2 + 4*power2(mstau22) + 5*power2(mu2)) +
     mstau22*(-18*mstau22*mu2 + 9*power2(mstau22) + 8*power2(mu2))*power3(
     msntau2) + power2(msntau2)*(8*mstau24*power2(mu2) + power2(mstau22)*(7*
     mstau24 + 40*power2(mu2)) - 20*mu2*power3(mstau22) - 2*mstau22*(7*mstau24*
     mu2 + 9*power3(mu2))) - msntau2*(mu2*(12*mstau24*power2(mu2) + 2*power2(
     mstau22)*(5*mstau24 + 11*power2(mu2)) - 11*mu2*power3(mstau22) - 10*
     mstau22*(2*mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(
     mstau2)))))/(mC2*msntau2*power2(msntau2 - mu2)*power2(mstau22 - mu2))))/(
     4.*cb) - (mu*sb*log(msntau2/Q2)*(3/mC2 + 7/msntau2 + (9*mstau22)/(mC2*
     msntau2) + 3/(mstau22 - mu2) + 2/(-msntau2 + mu2) + (9*mC2)/(msntau2*
     mstau22 - msntau2*mu2) + (3*mC2)/power2(msntau2 - mu2) + (3*mstau22)/
     power2(msntau2 - mu2) + (2*mu2)/power2(msntau2 - mu2) - (6*mu2)/power2(
     mstau22 - mu2) - (18*mC2*mu2)/(msntau2*power2(mstau22 - mu2)) - power2(
     mu2)/(msntau2*power2(msntau2 - mu2)) - (6*power2(mu2))/(msntau2*power2(
     mstau22 - mu2)) + (3*mu2*DeltaInv(mC2,msntau2,mu2)*(mC2*mu2*(-10*msntau2 +
     mu2) - 5*(msntau2 + mu2)*power2(mC2) + (-msntau2 + mu2)*power2(mu2) + 3*
     power3(mC2)))/(msntau2*power2(mstau22 - mu2)) + (2*power2(mu2))/power3(-
     msntau2 + mu2) + (3*DeltaInv(mstau22,mC2,msntau2)*(power3(mC2)*(-(power2(
     mu2)*(8*mstau22*mu2 - 4*power2(mstau22) + power2(mu2))) + power2(msntau2)*
     (-24*mstau22*mu2 + 7*power2(mstau22) + 22*power2(mu2)) + 5*(mstau22 - 2*
     mu2)*power3(msntau2) + msntau2*(-12*mu2*power2(mstau22) + 26*mstau22*
     power2(mu2) + power3(mstau22) - 10*power3(mu2))) - (3*(mstau22 - 2*mu2)*
     power2(msntau2) + 3*(mstau22 - 2*mu2)*power2(mu2) + msntau2*(-8*mstau22*
     mu2 + power2(mstau22) + 13*power2(mu2)))*power4(mC2) + power2(mC2)*((-18*
     mstau22*mu2 + 9*power2(mstau22) - power2(mu2))*power3(msntau2) + power2(
     mu2)*(6*mstau24*mu2 - 2*mu2*power2(mstau22) - mstau22*(3*mstau24 + power2(
     mu2)) + power3(mstau22)) + power2(msntau2)*(-3*mstau22*mstau24 + 6*
     mstau24*mu2 - 26*mu2*power2(mstau22) + 41*mstau22*power2(mu2) + 3*power3(
     mstau22)) + msntau2*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*
     power2(mu2) + power2(mstau22)*(mstau24 + 7*power2(mu2)) + 2*mu2*power3(
     mstau22) + power4(mu2))) + power2(msntau2 - mu2)*power2(mstau22 - mu2)*(5*
     msntau2*mstau24 - 3*power6(mstau2)) + mC2*(mstau24*power2(mu2)*(-8*
     mstau22*mu2 + 4*power2(mstau22) + 5*power2(mu2)) + mstau22*(-18*mstau22*
     mu2 + 9*power2(mstau22) + 8*power2(mu2))*power3(msntau2) + power2(msntau2)
     *(8*mstau24*power2(mu2) + power2(mstau22)*(7*mstau24 + 40*power2(mu2)) -
     20*mu2*power3(mstau22) - 2*mstau22*(7*mstau24*mu2 + 9*power3(mu2))) -
     msntau2*(mu2*(12*mstau24*power2(mu2) + 2*power2(mstau22)*(5*mstau24 + 11*
     power2(mu2)) - 11*mu2*power3(mstau22) - 10*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mC2*msntau2*
     power2(msntau2 - mu2)*power2(mstau22 - mu2))))/(2.*cb) - (mu*sb*power2(
     log(mstau12/Q2))*(2/mA2 - 6/mH2 - 6/mstau12 + (6*mstau22)/(mA2*mstau12) -
     (18*mstau22)/(mH2*mstau12) + 2/(-mstau22 + mu2) - (3*mA2)/(mstau12*mstau22
     - mstau12*mu2) - (3*mH2)/(mstau12*mstau22 - mstau12*mu2) - 8*mH2*DeltaInv(
     mstau22,mH2,mstau12) - 59*mstau22*DeltaInv(mstau22,mH2,mstau12) + (4*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (30*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/mH2 - (29*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 + (2*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (3*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(-mstau22 + mu2) + (3*mH2*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + (7*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/(-mstau12 + mu2) + (5*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-
     mstau22 + mu2) - (6*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau12*mstau22 - mstau12*mu2) - mA2/power2(mstau12 - mu2) - mH2/power2(
     mstau12 - mu2) - (2*mstau22)/power2(mstau12 - mu2) - (mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (6*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (mu2*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) + (4*mu2)/power2(
     mstau22 - mu2) + (6*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) + (6*mH2*mu2)
     /(mstau12*power2(mstau22 - mu2)) + (6*mH2*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau22 - mu2) - (3*mH2*mstau24*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (5*mu2*DeltaInv(mH2,
     mstau12,mu2)*power2(mH2))/power2(mstau22 - mu2) + (5*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (6*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 + mu2) + (4*power2(
     mu2))/(mstau12*power2(mstau22 - mu2)) + (10*mH2*DeltaInv(mH2,mstau12,mu2)*
     power2(mu2))/power2(mstau22 - mu2) + (4*mH2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) + (2*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (5*DeltaInv(mH2,mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mA2,mstau12,
     mu2)*(mA2*(10*mstau12 - mu2)*mu2 + 5*(mstau12 + mu2)*power2(mA2) + (
     mstau12 - mu2)*power2(mu2) - 3*power3(mA2)))/(mstau12*power2(mstau22 -
     mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12
     - mu2) - (3*mu2*DeltaInv(mH2,mstau12,mu2)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(
     mstau12*power2(mstau22 - mu2)) + (4*power2(mu2))/power3(mstau12 - mu2) + (
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/power2(mstau22 - mu2) - (mH2*
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) - (
     DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     18*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (
     DeltaInv(mA2,mstau22,mstau12)*(-(power3(mA2)*(power2(mu2)*(-14*mstau22*mu2
     + 7*power2(mstau22) + 2*power2(mu2)) + 5*power2(mstau12)*(-6*mstau22*mu2 +
     2*power2(mstau22) + 5*power2(mu2)) + 5*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*(-18*mu2*power2(mstau22) + 38*mstau22*power2(mu2) + power3(
     mstau22) - 16*power3(mu2)))) + (3*(mstau22 - 2*mu2)*power2(mstau12) + 3*(
     mstau22 - 2*mu2)*power2(mu2) + mstau12*(-8*mstau22*mu2 + power2(mstau22) +
     13*power2(mu2)))*power4(mA2) - power2(mA2)*(2*(-6*mstau22*mu2 + 3*power2(
     mstau22) - 2*power2(mu2))*power3(mstau12) + power2(mu2)*(-3*mstau22*
     mstau24 + 6*mstau24*mu2 - 8*mu2*power2(mstau22) + 2*mstau22*power2(mu2) +
     4*power3(mstau22)) + power2(mstau12)*(-26*mu2*power2(mstau22) + 6*mu2*(
     mstau24 + power2(mu2)) + mstau22*(-3*mstau24 + 32*power2(mu2)) + 6*power3(
     mstau22)) + mstau12*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*
     power2(mu2) + power2(mstau22)*(mstau24 + 16*power2(mu2)) - 4*mu2*power3(
     mstau22) - 2*power4(mu2))) + 2*power2(mstau12 - mu2)*power2(mstau22 - mu2)
     *(5*mstau12*mstau24 - 3*power6(mstau2)) + mA2*(mstau24*power2(mu2)*(-22*
     mstau22*mu2 + 11*power2(mstau22) + 10*power2(mu2)) + mstau22*(-42*mstau22*
     mu2 + 21*power2(mstau22) + 22*power2(mu2))*power3(mstau12) + power2(
     mstau12)*(7*mstau24*power2(mu2) + 8*power2(mstau22)*(mstau24 + 10*power2(
     mu2)) - 40*mu2*power3(mstau22) - 2*mstau22*(8*mstau24*mu2 + 21*power3(mu2)
     )) + mstau12*(mu2*(-18*mstau24*power2(mu2) - 2*power2(mstau22)*(10*mstau24
     + 19*power2(mu2)) + 19*mu2*power3(mstau22) + 20*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(8.*cb) + (mu*sb*log(
     mstau12/Q2)*(6/mA2 - 18/mH2 - 20/mstau12 + (18*mstau22)/(mA2*mstau12) - (
     54*mstau22)/(mH2*mstau12) + 4/(mstau12 - mu2) + 6/(-mstau22 + mu2) - (9*
     mA2)/(mstau12*mstau22 - mstau12*mu2) - (9*mH2)/(mstau12*mstau22 - mstau12*
     mu2) - 24*mH2*DeltaInv(mstau22,mH2,mstau12) - 177*mstau22*DeltaInv(
     mstau22,mH2,mstau12) + (12*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/
     mstau12 - (90*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 - (87*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (6*mH2*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12 - mu2) + (6*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (mstau12 - mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau12
     + mu2) + (9*mstau24*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (18*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau22 + mu2) + (9*mH2*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + (
     21*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (9*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(-mstau12 + mu2) + (15*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/(-mstau22 + mu2) - (18*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) - (3*mA2)/
     power2(mstau12 - mu2) - (3*mH2)/power2(mstau12 - mu2) - (6*mstau22)/
     power2(mstau12 - mu2) - (4*mu2)/power2(mstau12 - mu2) - (3*mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (18*mH2*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (3*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) + (12*
     mu2)/power2(mstau22 - mu2) + (18*mA2*mu2)/(mstau12*power2(mstau22 - mu2))
     + (18*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) - (9*mH2*mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (15*mu2*
     DeltaInv(mH2,mstau12,mu2)*power2(mH2))/power2(mstau22 - mu2) + (15*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (18*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22
     + mu2) + (2*power2(mu2))/(mstau12*power2(mstau12 - mu2)) + (12*power2(mu2)
     )/(mstau12*power2(mstau22 - mu2)) + (30*mH2*DeltaInv(mH2,mstau12,mu2)*
     power2(mu2))/power2(mstau22 - mu2) + (12*mH2*DeltaInv(mstau22,mH2,mstau12)
     *power2(mu2))/power2(mstau22 - mu2) + (6*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (15*DeltaInv(mH2,mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mA2,
     mstau12,mu2)*(mA2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mA2)
     + (-mstau12 + mu2)*power2(mu2) + 3*power3(mA2)))/(mstau12*power2(mstau22 -
     mu2)) + (9*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(
     mstau12 - mu2) - (9*mu2*DeltaInv(mH2,mstau12,mu2)*power3(mH2))/(mstau12*
     power2(mstau22 - mu2)) - (9*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))
     /(mstau12*power2(mstau22 - mu2)) + (4*power2(mu2))/power3(mstau12 - mu2) +
     (3*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/power2(mstau22 - mu2) - (3*mH2*
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) - (3*
     DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     54*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (3*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (3*
     DeltaInv(mA2,mstau22,mstau12)*(-(power3(mA2)*(power2(mu2)*(-14*mstau22*mu2
     + 7*power2(mstau22) + 2*power2(mu2)) + 5*power2(mstau12)*(-6*mstau22*mu2 +
     2*power2(mstau22) + 5*power2(mu2)) + 5*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*(-18*mu2*power2(mstau22) + 38*mstau22*power2(mu2) + power3(
     mstau22) - 16*power3(mu2)))) + (3*(mstau22 - 2*mu2)*power2(mstau12) + 3*(
     mstau22 - 2*mu2)*power2(mu2) + mstau12*(-8*mstau22*mu2 + power2(mstau22) +
     13*power2(mu2)))*power4(mA2) - power2(mA2)*(2*(-6*mstau22*mu2 + 3*power2(
     mstau22) - 2*power2(mu2))*power3(mstau12) + power2(mu2)*(-3*mstau22*
     mstau24 + 6*mstau24*mu2 - 8*mu2*power2(mstau22) + 2*mstau22*power2(mu2) +
     4*power3(mstau22)) + power2(mstau12)*(-26*mu2*power2(mstau22) + 6*mu2*(
     mstau24 + power2(mu2)) + mstau22*(-3*mstau24 + 32*power2(mu2)) + 6*power3(
     mstau22)) + mstau12*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*
     power2(mu2) + power2(mstau22)*(mstau24 + 16*power2(mu2)) - 4*mu2*power3(
     mstau22) - 2*power4(mu2))) + 2*power2(mstau12 - mu2)*power2(mstau22 - mu2)
     *(5*mstau12*mstau24 - 3*power6(mstau2)) + mA2*(mstau24*power2(mu2)*(-22*
     mstau22*mu2 + 11*power2(mstau22) + 10*power2(mu2)) + mstau22*(-42*mstau22*
     mu2 + 21*power2(mstau22) + 22*power2(mu2))*power3(mstau12) + power2(
     mstau12)*(7*mstau24*power2(mu2) + 8*power2(mstau22)*(mstau24 + 10*power2(
     mu2)) - 40*mu2*power3(mstau22) - 2*mstau22*(8*mstau24*mu2 + 21*power3(mu2)
     )) + mstau12*(mu2*(-18*mstau24*power2(mu2) - 2*power2(mstau22)*(10*mstau24
     + 19*power2(mu2)) + 19*mu2*power3(mstau22) + 20*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(4.*cb) - (mu*sb*log(
     mstau12/Q2)*log(mstau22/Q2)*power2(sa)*(power2(mstau12 - mu2)*(-(mh2*mH2*(
     mstau22 - 2*mu2)) + (mstau22 - 2*mu2)*power2(mH2) + 24*mstau22*power2(
     mstau22 - mu2)) + mH2*DeltaInv(mstau22,mstau12,mh2)*(mstau22*mstau24*(
     mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*
     power3(mh2) + mstau22*(-2*mstau22*mu2 + power2(mstau22) + 2*power2(mu2))*
     power3(mstau12) + mh2*(-((mstau22 - 2*mu2)*(-mstau24 + 2*power2(mstau22))*
     power2(mu2)) - 6*mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(
     mstau12)*(-2*mstau24*mu2 + 18*mu2*power2(mstau22) + mstau22*(mstau24 - 25*
     power2(mu2)) - 3*power3(mstau22)) + 2*mstau12*(mstau24*power2(mu2) +
     mstau22*mu2*(mstau24 + 5*power2(mu2)) - power2(mstau22)*(mstau24 + 5*
     power2(mu2)) + mu2*power3(mstau22))) - power2(mstau12)*(-4*mstau22*
     mstau24*mu2 + 2*mstau24*power2(mstau22) + 3*mstau24*power2(mu2) + 2*
     mstau22*power3(mu2)) - power2(mh2)*(power2(mstau12)*power2(mstau22 - 2*
     mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power3(
     mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) + 2*power3(
     mu2))) + mstau12*(power2(mu2)*(2*mstau24*mu2 + 2*mu2*power2(mstau22) -
     power3(mstau22)) + power2(mstau22 - mu2)*power6(mstau2))) - DeltaInv(
     mstau22,mH2,mstau12)*(power2(mH2)*(-6*mstau22*(mstau22 - 2*mu2)*power3(
     mstau12) + power2(mstau12)*(-2*mstau24*mu2 - 30*mu2*power2(mstau22) +
     mstau22*(mstau24 - power2(mu2)) + 21*power3(mstau22)) + power2(mu2)*(-2*
     mstau24*mu2 - 44*mu2*power2(mstau22) + mstau22*(mstau24 + 24*power2(mu2))
     + 22*power3(mstau22)) + mstau12*(-2*power2(mstau22)*(mstau24 - 43*power2(
     mu2)) + 2*mstau22*mu2*(mstau24 - 19*power2(mu2)) + 2*mstau24*power2(mu2) -
     46*mu2*power3(mstau22))) - power3(mH2)*(power2(mstau12)*power2(mstau22 -
     2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power3(
     mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) + 2*power3(
     mu2))) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(mH2) + 24*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(mstau2))
     + mH2*(mstau24*(94*mstau22*mu2 - 47*power2(mstau22) - 48*power2(mu2))*
     power2(mu2) + mstau22*(94*mstau22*mu2 - 47*power2(mstau22) - 46*power2(
     mu2))*power3(mstau12) + power2(mstau12)*(-51*mstau24*power2(mu2) - 2*
     power2(mstau22)*(25*mstau24 + 96*power2(mu2)) + 96*mu2*power3(mstau22) +
     2*mstau22*(50*mstau24*mu2 + 47*power3(mu2))) + mstau12*(mu2*(98*mstau24*
     power2(mu2) + power2(mstau22)*(96*mstau24 + 98*power2(mu2)) - 49*mu2*
     power3(mstau22) - 48*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2))))))/(4.*cb*mH2*mstau12*power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) - (ca*mu*sa*log(mstau12/Q2)*log(mstau22/Q2)*(
     power2(mstau12 - mu2)*(-(mh2*mH2*(mstau22 - 2*mu2)) + (mstau22 - 2*mu2)*
     power2(mH2) + 12*mstau22*power2(mstau22 - mu2)) + mH2*DeltaInv(mstau22,
     mstau12,mh2)*(mstau22*mstau24*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 -
     2*mu2)*power2(mstau12 - mu2)*power3(mh2) + mstau22*(-2*mstau22*mu2 +
     power2(mstau22) + 2*power2(mu2))*power3(mstau12) + mh2*(-((mstau22 - 2*
     mu2)*(-mstau24 + 2*power2(mstau22))*power2(mu2)) - 6*mstau22*(mstau22 - 2*
     mu2)*power3(mstau12) + power2(mstau12)*(-2*mstau24*mu2 + 18*mu2*power2(
     mstau22) + mstau22*(mstau24 - 25*power2(mu2)) - 3*power3(mstau22)) + 2*
     mstau12*(mstau24*power2(mu2) + mstau22*mu2*(mstau24 + 5*power2(mu2)) -
     power2(mstau22)*(mstau24 + 5*power2(mu2)) + mu2*power3(mstau22))) -
     power2(mstau12)*(-4*mstau22*mstau24*mu2 + 2*mstau24*power2(mstau22) + 3*
     mstau24*power2(mu2) + 2*mstau22*power3(mu2)) - power2(mh2)*(power2(
     mstau12)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) +
     (mstau22 - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*power2(mu2) +
     power3(mstau22) + 2*power3(mu2))) + mstau12*(power2(mu2)*(2*mstau24*mu2 +
     2*mu2*power2(mstau22) - power3(mstau22)) + power2(mstau22 - mu2)*power6(
     mstau2))) - DeltaInv(mstau22,mH2,mstau12)*(-(power3(mH2)*(power2(mstau12)*
     power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22
     - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(
     mstau22) + 2*power3(mu2)))) + power2(mH2)*(-6*mstau22*(mstau22 - 2*mu2)*
     power3(mstau12) + power2(mstau12)*(-2*mstau24*mu2 - 6*mu2*power2(mstau22)
     + mstau22*(mstau24 - 13*power2(mu2)) + 9*power3(mstau22)) + power2(mu2)*(-
     2*mstau24*mu2 - 20*mu2*power2(mstau22) + mstau22*(mstau24 + 12*power2(mu2)
     ) + 10*power3(mstau22)) - 2*mstau12*(power2(mstau22)*(mstau24 - 19*power2(
     mu2)) - mstau24*power2(mu2) + 11*mu2*power3(mstau22) + mstau22*(-(mstau24*
     mu2) + 7*power3(mu2)))) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(
     mH2) + 12*power2(mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24
     + power6(mstau2)) + mH2*(mstau24*(46*mstau22*mu2 - 23*power2(mstau22) -
     24*power2(mu2))*power2(mu2) + mstau22*(46*mstau22*mu2 - 23*power2(mstau22)
     - 22*power2(mu2))*power3(mstau12) + power2(mstau12)*(-27*mstau24*power2(
     mu2) - 2*power2(mstau22)*(13*mstau24 + 48*power2(mu2)) + 48*mu2*power3(
     mstau22) + mstau22*(52*mstau24*mu2 + 46*power3(mu2))) + mstau12*(mu2*(50*
     mstau24*power2(mu2) + power2(mstau22)*(48*mstau24 + 50*power2(mu2)) - 25*
     mu2*power3(mstau22) - 24*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2))))))/(4.*mH2*mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2)) + (mu*sb*log(msntau2/Q2)*log(mstau22/Q2)*((mstau22
     - 2*mu2)*power2(mC2)*power2(msntau2 - mu2) + 2*mstau22*power2(msntau2 -
     mu2)*power2(mstau22 - mu2) + mC2*(power2(msntau2)*power2(mstau22 - 2*mu2)
     + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power3(
     msntau2) + msntau2*(-4*mu2*power2(mstau22) + 6*mstau22*power2(mu2) +
     power3(mstau22) - 2*power3(mu2))) - DeltaInv(mstau22,mC2,msntau2)*(power2(
     mC2)*(power2(mu2)*(-2*mstau24*mu2 + mstau22*(mstau24 + 2*power2(mu2))) -
     6*mstau22*(mstau22 - 2*mu2)*power3(msntau2) - power2(msntau2)*(-(mstau22*
     mstau24) + 2*mstau24*mu2 - 14*mu2*power2(mstau22) + 23*mstau22*power2(mu2)
     + power3(mstau22)) - 2*msntau2*(-(mstau24*power2(mu2)) + power2(mstau22)*(
     mstau24 + power2(mu2)) - mstau22*mu2*(mstau24 + 3*power2(mu2)) + mu2*
     power3(mstau22))) - power3(mC2)*(power2(msntau2)*power2(mstau22 - 2*mu2) +
     mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power3(msntau2)
     - msntau2*(-4*mstau22*power2(mu2) + power3(mstau22) + 2*power3(mu2))) + (
     mstau22 - 2*mu2)*power2(msntau2 - mu2)*power4(mC2) + 2*power2(msntau2 -
     mu2)*power2(mstau22 - mu2)*(-2*msntau2*mstau24 + power6(mstau2)) + mC2*(
     mstau24*(6*mstau22*mu2 - 3*power2(mstau22) - 4*power2(mu2))*power2(mu2) +
     mstau22*(6*mstau22*mu2 - 3*power2(mstau22) - 2*power2(mu2))*power3(
     msntau2) + power2(msntau2)*(-7*mstau24*power2(mu2) - 2*power2(mstau22)*(3*
     mstau24 + 8*power2(mu2)) + 8*mu2*power3(mstau22) + 6*mstau22*(2*mstau24*
     mu2 + power3(mu2))) + msntau2*(mu2*(10*mstau24*power2(mu2) + 2*power2(
     mstau22)*(4*mstau24 + 5*power2(mu2)) - 5*mu2*power3(mstau22) - 4*mstau22*(
     4*mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2))))))/
     (2.*cb*mC2*msntau2*power2(msntau2 - mu2)*power2(mstau22 - mu2)) - (mu*sb*
     log(mstau12/Q2)*log(mstau22/Q2)*(-2/mstau12 + (4*mstau22)/(mA2*mstau12) -
     (12*mstau22)/(mH2*mstau12) + 2/(-mstau22 + mu2) + mA2/(mstau12*(-mstau22 +
     mu2)) + mH2/(mstau12*(-mstau22 + mu2)) - (2*mstau22)/power2(mstau12 - mu2)
     + (2*mu2)/power2(mstau22 - mu2) + (mA2*mu2)/(mstau12*power2(mstau22 - mu2)
     ) + (mH2*mu2)/(mstau12*power2(mstau22 - mu2)) + (2*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*(-(power3(mH2)*(
     power2(mstau12)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*
     power2(mu2) + power3(mstau22) + 2*power3(mu2)))) + power2(mH2)*(-6*
     mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(mstau12)*(-2*mstau24*
     mu2 - 6*mu2*power2(mstau22) + mstau22*(mstau24 - 13*power2(mu2)) + 9*
     power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 20*mu2*power2(mstau22) +
     mstau22*(mstau24 + 12*power2(mu2)) + 10*power3(mstau22)) - 2*mstau12*(
     power2(mstau22)*(mstau24 - 19*power2(mu2)) - mstau24*power2(mu2) + 11*mu2*
     power3(mstau22) + mstau22*(-(mstau24*mu2) + 7*power3(mu2)))) + (mstau22 -
     2*mu2)*power2(mstau12 - mu2)*power4(mH2) + 12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(mstau2)) + mH2*(
     mstau24*(46*mstau22*mu2 - 23*power2(mstau22) - 24*power2(mu2))*power2(mu2)
     + mstau22*(46*mstau22*mu2 - 23*power2(mstau22) - 22*power2(mu2))*power3(
     mstau12) + power2(mstau12)*(-27*mstau24*power2(mu2) - 2*power2(mstau22)*(
     13*mstau24 + 48*power2(mu2)) + 48*mu2*power3(mstau22) + mstau22*(52*
     mstau24*mu2 + 46*power3(mu2))) + mstau12*(mu2*(50*mstau24*power2(mu2) +
     power2(mstau22)*(48*mstau24 + 50*power2(mu2)) - 25*mu2*power3(mstau22) -
     24*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(
     mstau2)))))/(mH2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (
     DeltaInv(mA2,mstau22,mstau12)*(power2(mA2)*(-6*mstau22*(mstau22 - 2*mu2)*
     power3(mstau12) + power2(mstau12)*(-2*mstau24*mu2 + 26*mu2*power2(mstau22)
     + mstau22*(mstau24 - 29*power2(mu2)) - 7*power3(mstau22)) + power2(mu2)*(-
     2*mstau24*mu2 + 12*mu2*power2(mstau22) + mstau22*(mstau24 - 4*power2(mu2))
     - 6*power3(mstau22)) + 2*mstau12*(mstau24*power2(mu2) + mstau22*mu2*(
     mstau24 + 9*power2(mu2)) - power2(mstau22)*(mstau24 + 13*power2(mu2)) + 5*
     mu2*power3(mstau22))) - power3(mA2)*(power2(mstau12)*power2(mstau22 - 2*
     mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*mu2)*power3(
     mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) + 2*power3(
     mu2))) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(mA2) - 4*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(mstau2))
     + mA2*(mstau24*power2(mu2)*(-18*mstau22*mu2 + 9*power2(mstau22) + 8*
     power2(mu2)) + mstau22*(-18*mstau22*mu2 + 9*power2(mstau22) + 10*power2(
     mu2))*power3(mstau12) + power2(mstau12)*(5*mstau24*power2(mu2) + power2(
     mstau22)*(6*mstau24 + 32*power2(mu2)) - 16*mu2*power3(mstau22) - 6*
     mstau22*(2*mstau24*mu2 + 3*power3(mu2))) + mstau12*(mu2*(-14*mstau24*
     power2(mu2) - 2*power2(mstau22)*(8*mstau24 + 7*power2(mu2)) + 7*mu2*
     power3(mstau22) + 8*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2))))/(4.*cb) + (mu*sb*(-10/mA2 + 10/mC2 + 30/mH2 + 29/
     msntau2 + 43/mstau12 + 2/mstau22 + (28*mstau22)/(mC2*msntau2) - (28*
     mstau22)/(mA2*mstau12) + (84*mstau22)/(mH2*mstau12) + 18/(mstau22 - mu2) +
     5/(-msntau2 + mu2) + 5/(-mstau12 + mu2) + (28*mC2)/(msntau2*mstau22 -
     msntau2*mu2) + (14*mA2)/(mstau12*mstau22 - mstau12*mu2) + (14*mH2)/(
     mstau12*mstau22 - mstau12*mu2) + 28*mC2*DeltaInv(mstau22,mC2,msntau2) +
     84*mstau22*DeltaInv(mstau22,mC2,msntau2) + (28*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/msntau2 + (56*mstau24*DeltaInv(mstau22,mC2,msntau2))
     /mC2 + (28*mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (56*mC2*
     mstau22*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (42*mstau24*
     DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (56*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/(mstau22 - mu2) + (28*mC2*mu2*DeltaInv(mstau22,mC2,
     msntau2))/(-msntau2 + mu2) + (28*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2)
     )/(-msntau2 + mu2) - (28*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/(
     msntau2*mstau22 - msntau2*mu2) + 49*mH2*DeltaInv(mstau22,mH2,mstau12) +
     287*mstau22*DeltaInv(mstau22,mH2,mstau12) + (14*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 + (168*mstau24*DeltaInv(mstau22,mH2,mstau12)
     )/mH2 + (119*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (28*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (21*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (28*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(mstau22 - mu2) + (14*mH2*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(-mstau12 + mu2) + (14*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/(-mstau12 + mu2) - (14*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12*mstau22 - mstau12*mu2) - (28*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2))/msntau2 + (42*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2 -
     mu2) + (56*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(mstau22 - mu2) + (
     56*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2*mstau22 -
     msntau2*mu2) - (49*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (
     21*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (28*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (28*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2)
     - (2*mu2)/(-(msntau2*mu2) + power2(msntau2)) - (2*mu2)/(-(mstau12*mu2) +
     power2(mstau12)) - (4*mu2)/(-(mstau22*mu2) + power2(mstau22)) + (14*mC2)/
     power2(msntau2 - mu2) + (14*mstau22)/power2(msntau2 - mu2) - (3*mu2)/
     power2(msntau2 - mu2) - (28*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (
     14*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (56*
     mC2*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (
     14*mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (14*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) +
     (14*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) -
     (56*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (7*
     mA2)/power2(mstau12 - mu2) + (7*mH2)/power2(mstau12 - mu2) + (14*mstau22)/
     power2(mstau12 - mu2) - (3*mu2)/power2(mstau12 - mu2) - (14*mA2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (14*mH2*mu2)/(mstau22*power2(mstau12 -
     mu2)) + (7*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (28*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (7*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2)
     - (28*mu2*DeltaInv(mA2,mstau22,mu2)*power2(mA2))/power2(mstau12 - mu2) + (
     7*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2)
     + (7*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2)
     - (28*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (
     34*mu2)/power2(mstau22 - mu2) - (56*mC2*mu2)/(msntau2*power2(mstau22 -
     mu2)) - (28*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) - (28*mH2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (56*mC2*mstau22*mu2*DeltaInv(mstau22,mC2,
     msntau2))/power2(mstau22 - mu2) + (28*mC2*mstau24*mu2*DeltaInv(mstau22,
     mC2,msntau2))/(msntau2*power2(mstau22 - mu2)) - (28*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) + (14*mH2*mstau24*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) - (56*
     mu2*DeltaInv(mC2,msntau2,mu2)*power2(mC2))/power2(mstau22 - mu2) - (56*
     mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(mstau22 - mu2) - (
     56*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2*power2(
     mstau22 - mu2)) - (28*mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/power2(
     mstau22 - mu2) - (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     power2(mstau22 - mu2) - (28*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mC2,
     msntau2)*power2(mu2))/(mstau22 - mu2) + (7*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/(mstau22 - mu2) - (3*power2(mu2))/(msntau2*power2(msntau2 -
     mu2)) - (14*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (98*mC2*
     DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (42*
     DeltaInv(mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2
     - mu2)) - (3*power2(mu2))/(mstau12*power2(mstau12 - mu2)) - (14*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (49*mA2*DeltaInv(mA2,mstau22,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (49*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (21*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (21*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (28*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (28*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (6*power2(mu2))/(mstau22*power2(mstau22 - mu2)) -
     (98*mC2*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (
     49*mH2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) - (42*
     mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) - (
     28*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 -
     mu2) - (28*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (14*mstau24*DeltaInv(mstau22,mC2,msntau2)
     *power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (21*mH2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (14*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (14*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (7*mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (42*DeltaInv(mC2,msntau2,mu2)*power2(mC2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (21*DeltaInv(mH2,
     mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     7*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*
     power2(mstau12 - mu2)) + (7*mu2*DeltaInv(mA2,mstau12,mu2)*(-7*mA2*mstau12*
     mu2 - (4*mstau12 + 3*mu2)*power2(mA2) + (-mstau12 + mu2)*power2(mu2) + 2*
     power3(mA2)))/(mstau12*power2(mstau22 - mu2)) - (28*DeltaInv(mstau22,mC2,
     msntau2)*power3(mC2))/(msntau2*mstau22 - msntau2*mu2) - (14*DeltaInv(
     mstau22,mC2,msntau2)*power3(mC2))/power2(msntau2 - mu2) + (28*mu2*
     DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 - mu2)) + (
     28*mu2*DeltaInv(mC2,msntau2,mu2)*power3(mC2))/(msntau2*power2(mstau22 -
     mu2)) + (28*mu2*DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/(msntau2*
     power2(mstau22 - mu2)) - (14*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(
     mstau12*mstau22 - mstau12*mu2) - (7*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/power2(mstau12 - mu2) + (14*mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) + (14*mu2*DeltaInv(mH2,mstau12,mu2)*
     power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (14*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (14*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/power2(msntau2 - mu2) - (7*DeltaInv(mA2,
     mstau22,mu2)*power3(mu2))/power2(mstau12 - mu2) - (7*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/power2(mstau12 - mu2) - (14*DeltaInv(mC2,msntau2,mu2)*
     power3(mu2))/power2(mstau22 - mu2) - (7*DeltaInv(mH2,mstau12,mu2)*power3(
     mu2))/power2(mstau22 - mu2) + (14*DeltaInv(mstau22,mC2,msntau2)*power3(
     mu2))/power2(mstau22 - mu2) + (7*DeltaInv(mstau22,mH2,mstau12)*power3(mu2)
     )/power2(mstau22 - mu2) + (14*DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(
     mstau22*power2(msntau2 - mu2)) + (7*DeltaInv(mA2,mstau22,mu2)*power4(mu2))
     /(mstau22*power2(mstau12 - mu2)) + (7*DeltaInv(mstau22,mH2,mu2)*power4(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(mC2,msntau2,mu2)*
     power4(mu2))/(msntau2*power2(mstau22 - mu2)) + (7*DeltaInv(mH2,mstau12,
     mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) - (28*DeltaInv(mstau22,
     mC2,msntau2)*power6(mstau2))/(mC2*msntau2) - (84*DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/(mH2*mstau12) - (14*DeltaInv(mstau22,mC2,msntau2)
     *power6(mstau2))/power2(msntau2 - mu2) - (7*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/power2(mstau12 - mu2) - (7*DeltaInv(mA2,mstau22,mstau12)*(
     -(power3(mA2)*(power2(mu2)*(-10*mstau22*mu2 + 5*power2(mstau22) + 2*
     power2(mu2)) + power2(mstau12)*(-24*mstau22*mu2 + 8*power2(mstau22) + 21*
     power2(mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*(-14*mu2*
     power2(mstau22) + 29*mstau22*power2(mu2) + power3(mstau22) - 14*power3(
     mu2)))) + (2*(mstau22 - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22 -
     3*mu2) + 2*(mstau22 - 2*mu2)*power2(mu2))*power4(mA2) + power2(mA2)*(-2*(
     mstau22 - 2*mu2)*(-mstau24 + power2(mstau22))*power2(mu2) + (6*mstau22*mu2
     - 3*power2(mstau22) + 4*power2(mu2))*power3(mstau12) - 2*power2(mstau12)*(
     -(mstau22*mstau24) + 2*mstau24*mu2 - 10*mu2*power2(mstau22) + 10*mstau22*
     power2(mu2) + 3*power3(mstau22) + 3*power3(mu2)) + mstau12*(-2*mstau22*
     mu2*(mstau24 - 5*power2(mu2)) + 7*mstau24*power2(mu2) - power2(mstau22)*(
     mstau24 + 13*power2(mu2)) + 4*mu2*power3(mstau22) + 2*power4(mu2))) - 4*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(
     mstau2)) + mA2*(mstau24*power2(mu2)*(-14*mstau22*mu2 + 7*power2(mstau22) +
     6*power2(mu2)) + mstau22*(-30*mstau22*mu2 + 15*power2(mstau22) + 16*
     power2(mu2))*power3(mstau12) + power2(mstau12)*(3*mstau24*power2(mu2) + 4*
     power2(mstau22)*(mstau24 + 14*power2(mu2)) - 28*mu2*power3(mstau22) - 2*
     mstau22*(4*mstau24*mu2 + 15*power3(mu2))) + mstau12*(mu2*(-10*mstau24*
     power2(mu2) - 2*power2(mstau22)*(6*mstau24 + 13*power2(mu2)) + 13*mu2*
     power3(mstau22) + 2*mstau22*(12*mstau24*mu2 + 7*power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2))))/(4.*cb) + (mu*sb*power2(Pi)*(-2/mA2 + 2/mC2 + 6/
     mH2 + 4/msntau2 + 6/mstau12 + (4*mstau22)/(mC2*msntau2) - (4*mstau22)/(
     mA2*mstau12) + (12*mstau22)/(mH2*mstau12) + 4/(mstau22 - mu2) + (4*mC2)/(
     msntau2*mstau22 - msntau2*mu2) + (2*mA2)/(mstau12*mstau22 - mstau12*mu2) +
     (2*mH2)/(mstau12*mstau22 - mstau12*mu2) + 4*mC2*DeltaInv(mstau22,mC2,
     msntau2) + 12*mstau22*DeltaInv(mstau22,mC2,msntau2) + (4*mC2*mstau22*
     DeltaInv(mstau22,mC2,msntau2))/msntau2 + (8*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/mC2 + (4*mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (8*
     mC2*mstau22*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (6*mstau24*
     DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (8*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/(mstau22 - mu2) + (4*mC2*mu2*DeltaInv(mstau22,mC2,
     msntau2))/(-msntau2 + mu2) + (4*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))
     /(-msntau2 + mu2) - (4*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/(
     msntau2*mstau22 - msntau2*mu2) + 7*mH2*DeltaInv(mstau22,mH2,mstau12) + 41*
     mstau22*DeltaInv(mstau22,mH2,mstau12) + (2*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/mstau12 + (24*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (
     17*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (4*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (4*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(mstau22 - mu2) + (2*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12
     + mu2) - (2*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 -
     mstau12*mu2) - (4*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/msntau2 + (6*
     DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2 - mu2) + (8*DeltaInv(
     mstau22,mC2,msntau2)*power2(mC2))/(mstau22 - mu2) + (8*mstau22*DeltaInv(
     mstau22,mC2,msntau2)*power2(mC2))/(msntau2*mstau22 - msntau2*mu2) - (7*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (3*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (4*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(mstau22 - mu2) + (4*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (2*mC2)/power2(
     msntau2 - mu2) + (2*mstau22)/power2(msntau2 - mu2) - (2*mu2)/power2(
     msntau2 - mu2) - (4*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (2*mC2*
     mstau24*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (8*mC2*
     mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*
     mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) +
     (2*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) -
     (8*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + mA2/
     power2(mstau12 - mu2) + mH2/power2(mstau12 - mu2) + (2*mstau22)/power2(
     mstau12 - mu2) - (2*mu2)/power2(mstau12 - mu2) - (2*mA2*mu2)/(mstau22*
     power2(mstau12 - mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (
     mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (4*mH2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (
     mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (4*mu2*
     DeltaInv(mA2,mstau22,mu2)*power2(mA2))/power2(mstau12 - mu2) + (mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) + (mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (4*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (8*mu2)/
     power2(mstau22 - mu2) - (8*mC2*mu2)/(msntau2*power2(mstau22 - mu2)) - (4*
     mA2*mu2)/(mstau12*power2(mstau22 - mu2)) - (4*mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (8*mC2*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/
     power2(mstau22 - mu2) + (4*mC2*mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/
     (msntau2*power2(mstau22 - mu2)) - (4*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau22 - mu2) + (2*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12*power2(mstau22 - mu2)) - (8*mu2*DeltaInv(mC2,msntau2,
     mu2)*power2(mC2))/power2(mstau22 - mu2) - (8*mu2*DeltaInv(mstau22,mC2,
     msntau2)*power2(mC2))/power2(mstau22 - mu2) - (8*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2)*power2(mC2))/(msntau2*power2(mstau22 - mu2)) - (4*
     mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/power2(mstau22 - mu2) - (4*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) - (4*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(mstau22 -
     mu2) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 - mu2) - (2*
     power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (14*mC2*DeltaInv(mstau22,
     mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (6*DeltaInv(mstau22,mC2,mu2)
     *power2(mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (2*power2(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (7*mA2*DeltaInv(mA2,mstau22,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (7*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (3*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (4*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (4*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (14*mC2*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/
     power2(mstau22 - mu2) - (7*mH2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/
     power2(mstau22 - mu2) - (6*mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/
     power2(mstau22 - mu2) - (4*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(
     mu2))/power2(mstau22 - mu2) - (4*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)
     *power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (2*mstau24*DeltaInv(
     mstau22,mC2,msntau2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (3*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) -
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (6*DeltaInv(mC2,msntau2,mu2)*power2(mC2)*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mC2,
     msntau2)*power2(mC2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (3*
     DeltaInv(mH2,mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (2*mu2*DeltaInv(mA2,mstau22,mu2)*power3(
     mA2))/(mstau22*power2(mstau12 - mu2)) + (mu2*DeltaInv(mA2,mstau12,mu2)*(-
     7*mA2*mstau12*mu2 - (4*mstau12 + 3*mu2)*power2(mA2) + (-mstau12 + mu2)*
     power2(mu2) + 2*power3(mA2)))/(mstau12*power2(mstau22 - mu2)) - (4*
     DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/(msntau2*mstau22 - msntau2*mu2)
     - (2*DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/power2(msntau2 - mu2) + (
     4*mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 -
     mu2)) + (4*mu2*DeltaInv(mC2,msntau2,mu2)*power3(mC2))/(msntau2*power2(
     mstau22 - mu2)) + (4*mu2*DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/(
     msntau2*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/power2(mstau12 - mu2) + (2*mu2*DeltaInv(mstau22,mH2,mu2)*
     power3(mH2))/(mstau22*power2(mstau12 - mu2)) + (2*mu2*DeltaInv(mH2,
     mstau12,mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (2*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2))
     - (2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/power2(msntau2 - mu2) - (
     DeltaInv(mA2,mstau22,mu2)*power3(mu2))/power2(mstau12 - mu2) - (DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mC2,
     msntau2,mu2)*power3(mu2))/power2(mstau22 - mu2) - (DeltaInv(mH2,mstau12,
     mu2)*power3(mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mC2,msntau2)
     *power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,mH2,mstau12)*
     power3(mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mC2,mu2)*power4(
     mu2))/(mstau22*power2(msntau2 - mu2)) + (DeltaInv(mA2,mstau22,mu2)*power4(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mH2,mu2)*power4(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (2*DeltaInv(mC2,msntau2,mu2)*
     power4(mu2))/(msntau2*power2(mstau22 - mu2)) + (DeltaInv(mH2,mstau12,mu2)*
     power4(mu2))/(mstau12*power2(mstau22 - mu2)) - (4*DeltaInv(mstau22,mC2,
     msntau2)*power6(mstau2))/(mC2*msntau2) - (12*DeltaInv(mstau22,mH2,mstau12)
     *power6(mstau2))/(mH2*mstau12) - (2*DeltaInv(mstau22,mC2,msntau2)*power6(
     mstau2))/power2(msntau2 - mu2) - (DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) - (DeltaInv(mA2,mstau22,mstau12)*(-(power3(
     mA2)*(power2(mu2)*(-10*mstau22*mu2 + 5*power2(mstau22) + 2*power2(mu2)) +
     power2(mstau12)*(-24*mstau22*mu2 + 8*power2(mstau22) + 21*power2(mu2)) +
     4*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*(-14*mu2*power2(mstau22) +
     29*mstau22*power2(mu2) + power3(mstau22) - 14*power3(mu2)))) + (2*(mstau22
     - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22 - 3*mu2) + 2*(mstau22 -
     2*mu2)*power2(mu2))*power4(mA2) + power2(mA2)*(-2*(mstau22 - 2*mu2)*(-
     mstau24 + power2(mstau22))*power2(mu2) + (6*mstau22*mu2 - 3*power2(
     mstau22) + 4*power2(mu2))*power3(mstau12) - 2*power2(mstau12)*(-(mstau22*
     mstau24) + 2*mstau24*mu2 - 10*mu2*power2(mstau22) + 10*mstau22*power2(mu2)
     + 3*power3(mstau22) + 3*power3(mu2)) + mstau12*(-2*mstau22*mu2*(mstau24 -
     5*power2(mu2)) + 7*mstau24*power2(mu2) - power2(mstau22)*(mstau24 + 13*
     power2(mu2)) + 4*mu2*power3(mstau22) + 2*power4(mu2))) - 4*power2(mstau12
     - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(mstau2)) + mA2*(
     mstau24*power2(mu2)*(-14*mstau22*mu2 + 7*power2(mstau22) + 6*power2(mu2))
     + mstau22*(-30*mstau22*mu2 + 15*power2(mstau22) + 16*power2(mu2))*power3(
     mstau12) + power2(mstau12)*(3*mstau24*power2(mu2) + 4*power2(mstau22)*(
     mstau24 + 14*power2(mu2)) - 28*mu2*power3(mstau22) - 2*mstau22*(4*mstau24*
     mu2 + 15*power3(mu2))) + mstau12*(mu2*(-10*mstau24*power2(mu2) - 2*power2(
     mstau22)*(6*mstau24 + 13*power2(mu2)) + 13*mu2*power3(mstau22) + 2*
     mstau22*(12*mstau24*mu2 + 7*power3(mu2))) + power2(mstau22 - mu2)*power6(
     mstau2)))))/(mA2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))))/(
     24.*cb)) + power2(xtau)*((Fin3(mH2,mstau12,mstau12,Q2)*(-1 + power2(sa)))/
     (2.*(mstau12 - mstau22)*(mstau12 - mu2)) - (Fin3(mstau22,mstau22,mH2,Q2)*(
     -1 + power2(sa)))/(2.*(mstau12 - mstau22)*(mstau22 - mu2)) + (Fin3(
     mstau12,mstau12,mh2,Q2)*power2(sa))/(2.*(mstau12 - mstau22)*(-mstau12 +
     mu2)) + (Fin3(mstau22,mstau22,mh2,Q2)*power2(sa))/(2.*(mstau12 - mstau22)*
     (mstau22 - mu2)) + (Fin3(mstau22,mC2,mu2,Q2)*(-(msntau2*(mstau22 + mu2)) +
     mu2*(3*mstau22 + mu2) + (msntau2 - mu2)*mu2*DeltaInv(mstau22,mC2,mu2)*(-3*
     mC2*mstau22 - 2*mC2*mu2 - mstau22*mu2 + power2(mC2) + power2(mu2))))/(4.*
     mstau22*power3(msntau2 - mu2)) + (Fin3(mA2,mstau22,mu2,Q2)*(4*mstau22*(
     mstau22 - mu2)*power2(mstau12 - mu2) + mA2*(mstau12 - mstau22)*(mstau12 -
     mu2)*mu2*DeltaInv(mA2,mstau22,mu2)*(-3*mA2*mstau22 - 2*mA2*mu2 - mstau22*
     mu2 + power2(mA2) + power2(mu2)) + mA2*(-(mstau22*mu2*(3*mstau22 + 5*mu2))
     - (5*mstau22 + mu2)*power2(mstau12) + mstau12*(12*mstau22*mu2 + power2(
     mstau22) + power2(mu2)))))/(8.*mA2*(mstau12 - mstau22)*mstau22*power3(
     mstau12 - mu2)) + (Fin3(mC2,msntau2,mu2,Q2)*(-(msntau2*mstau22) + 3*
     msntau2*mu2 - mstau22*mu2 + power2(mu2) + (mstau22 - mu2)*mu2*DeltaInv(
     mC2,msntau2,mu2)*(-3*mC2*msntau2 - 2*mC2*mu2 - msntau2*mu2 + power2(mC2) +
     power2(mu2))))/(4.*msntau2*power3(mstau22 - mu2)) + (Fin3(mA2,mstau12,mu2,
     Q2)*(-4*mstau12*(mstau12 - mu2)*power2(mstau22 - mu2) + mA2*(mstau12 -
     mstau22)*(mstau22 - mu2)*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 -
     2*mA2*mu2 - mstau12*mu2 + power2(mA2) + power2(mu2)) + mA2*(mstau22*(
     mstau22 - mu2)*mu2 - (mstau22 - 3*mu2)*power2(mstau12) + mstau12*(-12*
     mstau22*mu2 + 5*power2(mstau22) + 5*power2(mu2)))))/(8.*mA2*mstau12*(
     mstau12 - mstau22)*power3(mstau22 - mu2)) - (Fin3(mstau22,mH2,mu2,Q2)*(-1
     + power2(sa))*(mH2*(mstau12 - mstau22)*(mstau12 - mu2)*(mstau22 - mu2)*
     mu2*DeltaInv(mstau22,mH2,mu2)*(-3*mH2*mstau22 - 2*mH2*mu2 - mstau22*mu2 +
     power2(mH2) + power2(mu2)) + 4*mstau22*(mstau22 - mu2)*power3(mstau12 -
     mu2) + mH2*(power2(mstau12)*(12*mstau22*mu2 - power2(mstau22) + power2(
     mu2)) + mstau22*mu2*(2*mstau22*mu2 - 3*power2(mstau22) + 5*power2(mu2)) -
     4*mstau22*power3(mstau12) + mstau12*(3*mu2*power2(mstau22) - 15*mstau22*
     power2(mu2) + power3(mstau22) - power3(mu2)))))/(8.*mH2*(mstau12 -
     mstau22)*mstau22*(mstau22 - mu2)*power3(mstau12 - mu2)) - (Fin3(mstau22,
     mh2,mu2,Q2)*power2(sa)*(-4/(mh2*mstau12 - mh2*mstau22) + 4/((mstau12 -
     mstau22)*(mstau22 - mu2)) + 1/power2(mstau12 - mu2) + mu2/(mstau22*power2(
     mstau12 - mu2)) + (mu2*DeltaInv(mstau22,mh2,mu2)*((mstau22 - mu2)*mu2 +
     mh2*(3*mstau22 + 2*mu2) - power2(mh2)))/(mstau22*power2(mstau12 - mu2)) +
     (2*mu2)/power3(-mstau12 + mu2)))/8. - (Fin3(mstau12,mh2,mu2,Q2)*power2(sa)
     *(4/(mh2*mstau12 - mh2*mstau22) + 4/((mstau12 - mstau22)*(-mstau12 + mu2))
     + 1/power2(mstau22 - mu2) + mu2/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mstau12,mh2,mu2)*((mstau12 - mu2)*mu2 + mh2*(3*mstau12 + 2*mu2) -
     power2(mh2)))/(mstau12*power2(mstau22 - mu2)) + (2*mu2)/power3(-mstau22 +
     mu2)))/8. + (Fin3(mstau22,mC2,msntau2,Q2)*(1/(mC2*msntau2) + 1/(msntau2*
     mstau22 - msntau2*mu2) + 1/power2(mC2) + 2/power2(msntau2 - mu2) + 1/
     power2(mstau22 - mu2) - mu2/(msntau2*power2(mstau22 - mu2)) + DeltaInv(
     mstau22,mC2,msntau2)*(-2 + (3*mstau22 - mstau24/msntau2)/mC2 + (3*mstau22)
     /(msntau2 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-msntau2 + mu2)
     - mstau24/power2(msntau2 - mu2) + (mstau22*mu2)/power2(msntau2 - mu2) + (
     2*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/msntau2 +
     mC2*(3/(msntau2 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(msntau2*mstau22
     - msntau2*mu2) + 1/power2(msntau2 - mu2)) + mu2/power2(msntau2 - mu2) + (-
     1 - (2*mstau22*mu2)/power2(mstau22 - mu2))/msntau2 - (3*mu2)/power2(
     mstau22 - mu2)) + power2(mC2)*(-(1/power2(msntau2 - mu2)) + (-mstau22 + 2*
     mu2)/(msntau2*power2(mstau22 - mu2))) - (2*mstau22*mu2)/power2(mstau22 -
     mu2) + power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/power3(-msntau2 + mu2)
     + (2*mu2)/power3(-mstau22 + mu2)))/4. + (Fin3(mA2,mstau22,mstau12,Q2)*(4/(
     (mstau12 - mstau22)*(mstau12 - mu2)) + 4/((mstau12 - mstau22)*(-mstau22 +
     mu2)) + 1/(mstau12*mstau22 - mstau12*mu2) - 2/power2(mA2) - (2*(mstau12*(
     mstau22 - 3*mu2) + (mstau22 - mu2)*mu2 + 2*power2(mstau12)))/(mA2*mstau12*
     (mstau12 - mu2)*(-mstau22 + mu2)) + 2/power2(mstau12 - mu2) + 1/power2(
     mstau22 - mu2) - mu2/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mA2,
     mstau22,mstau12)*(-5 + (-6*mstau22 + (2*mstau24)/mstau12)/mA2 + (3*
     mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12
     + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 -
     mu2) - (4*mstau22 + (mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/
     mstau12 + mA2*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(
     mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(
     mstau12 - mu2) + (2 - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (3*
     mu2)/power2(mstau22 - mu2)) + power2(mA2)*(-(1/power2(mstau12 - mu2)) + (-
     mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2))) - (2*mstau22*mu2)/
     power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/
     power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22 + mu2)))/8. + (Fin3(
     mstau22,mstau12,mh2,Q2)*power2(sa)*(1/(mstau12*mstau22 - mstau12*mu2) - 3*
     DeltaInv(mstau22,mstau12,mh2) + (3*mh2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12 - mu2) + (3*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2)
     + (3*mh2*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (mstau24*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*(-mstau22 + mu2)) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     (mstau12*mstau22 - mstau12*mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau12*(-mstau22 + mu2)) - 12/(-(mh2*mH2) + power2(mh2)) + 2/
     power2(mstau12 - mu2) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) - (mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(
     mstau12 - mu2) + (mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) + (mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) -
     (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + 1/
     power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 - mu2)) - (3*mh2*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) - (2*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (6*(
     mh2*(mH2 - 3*mstau12) + mH2*mstau12 - power2(mh2) + mh2*(mh2 - mH2)*
     DeltaInv(mstau22,mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*
     mstau22) + mstau24 + power2(mh2)))*power2(sa))/((mh2 - mH2)*mstau12*
     power2(mh2)) + (2*mu2)/power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22 +
     mu2)))/8. - (Fin3(mH2,mstau12,mu2,Q2)*(-1 + power2(sa))*(mH2*(mstau12 -
     mstau22)*(mstau12 - mu2)*(mstau22 - mu2)*mu2*DeltaInv(mH2,mstau12,mu2)*(-
     3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)) +
     mH2*(power2(mstau12)*(-3*mstau22*mu2 + power2(mstau22) - 2*power2(mu2)) +
     mstau22*(-mstau22 + mu2)*power2(mu2) - (mstau22 - 3*mu2)*power3(mstau12) +
     mstau12*(-12*mu2*power2(mstau22) + 15*mstau22*power2(mu2) + 4*power3(
     mstau22) - 5*power3(mu2))) + 4*mstau12*(mstau12 - mu2)*power3(-mstau22 +
     mu2)))/(8.*mH2*mstau12*(mstau12 - mstau22)*(mstau12 - mu2)*power3(mstau22
     - mu2)) + (Fin3(mstau22,mH2,mstau12,Q2)*(6/(mH2*mstau12) + 1/(mstau12*
     mstau22 - mstau12*mu2) + 3*DeltaInv(mstau22,mH2,mstau12) - (6*mH2*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (18*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/mH2 + (12*mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (6*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/(mH2*mstau12) + (3*mH2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (3*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12 - mu2) + (3*mH2*DeltaInv(mstau22,mH2,mstau12))/(mstau22
     - mu2) + (2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau22 - mu2) + (2*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*(-mstau22 + mu2)) + (2*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + 6/power2(
     mH2) + (DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*(-mstau22 +
     mu2)) + 2/power2(mstau12 - mu2) + (2*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) - (mstau24*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) + (mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(
     mstau12 - mu2) + (mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(
     mstau12 - mu2) - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(
     mstau12 - mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 -
     mu2)) - (3*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) -
     (2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) - (2*
     mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 -
     mu2)) + (mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(
     mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(
     mstau12*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(
     mu2))/power2(mstau22 - mu2) + (2*mu2)/power3(-mstau12 + mu2) - power2(sa)*
     ((12*(mh2 - mstau12))/(mh2*mH2*mstau12) + 1/(mstau12*mstau22 - mstau12*
     mu2) - 12/(-(mh2*mH2) + power2(mh2)) + 12/power2(mH2) + 2/power2(mstau12 -
     mu2) + 1/power2(mstau22 - mu2) - mu2/(mstau12*power2(mstau22 - mu2)) +
     DeltaInv(mstau22,mH2,mstau12)*(9 + (36*mstau22 - (12*mstau24)/mstau12)/mH2
     + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-
     mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/power2(
     mstau12 - mu2) + (24*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(mstau22
     - mu2))/mstau12 + mH2*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(
     1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(
     mstau12 - mu2) + (-12 - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (
     3*mu2)/power2(mstau22 - mu2)) + power2(mH2)*(-(1/power2(mstau12 - mu2)) +
     (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2))) - (2*mstau22*mu2)/
     power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (2*mu2)/
     power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau22 + mu2)) + (2*mu2)/power3(
     -mstau22 + mu2) - (6*(-(mh2*(mH2 + mstau12)) + mH2*(mH2 + 3*mstau12) + (
     mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*mstau22 - mH2*(
     mstau12 + 2*mstau22) + mstau24 + power2(mH2)))*power4(sa))/((mh2 - mH2)*
     mstau12*power2(mH2))))/8. + (12*(-(1/mA2) + 1/mC2 + 3/mH2) + (24*(mH2*(
     mstau12 - mu2) + mA2*(mstau22 - mu2))*Fin20(mstau12,mu2,Q2))/(mA2*mH2*(
     mstau12 - mstau22)*(mstau22 - mu2)) - (24*(mA2*(mstau12 - mu2) + mH2*(
     mstau22 - mu2))*Fin20(mstau22,mu2,Q2))/(mA2*mH2*(mstau12 - mstau22)*(
     mstau12 - mu2)) + (12*(mC2*mH2 - mA2*(3*mC2 + mH2))*mstau12*log(mstau12/
     Q2))/(mA2*mC2*mH2*(mstau12 - mstau22)) + (12*(-(mC2*mH2) + mA2*(3*mC2 +
     mH2))*mstau22*log(mstau22/Q2))/(mA2*mC2*mH2*(mstau12 - mstau22)) - (12*
     Fin20(mstau22,msntau2,Q2))/power2(mC2) + (12*Fin20(mstau22,mstau12,Q2)*(3*
     (mstau12 - mu2)*(mstau22 - mu2)*power2(mA2) + 2*mA2*(mstau12 + mstau22 -
     2*mu2)*power2(mH2) + (mstau12 - mu2)*(-mstau22 + mu2)*power2(mH2)))/((
     mstau12 - mu2)*(-mstau22 + mu2)*power2(mA2)*power2(mH2)) - (12*mC2*mu2*
     log(mC2/Q2)*log(mu2/Q2)*(4*msntau2*mstau22*mu2 - mstau22*power2(msntau2) -
     msntau2*power2(mstau22) + mstau22*DeltaInv(mC2,msntau2,mu2)*(mu2*(-3*
     msntau2 + mu2) - mC2*(msntau2 + 2*mu2) + power2(mC2))*power2(msntau2 -
     mu2) + msntau2*DeltaInv(mstau22,mC2,mu2)*(mu2*(-3*mstau22 + mu2) - mC2*(
     mstau22 + 2*mu2) + power2(mC2))*power2(mstau22 - mu2) - msntau2*power2(
     mu2) - mstau22*power2(mu2)))/(msntau2*mstau22*power2(msntau2 - mu2)*
     power2(mstau22 - mu2)) + (6*mu2*log(mH2/Q2)*log(mu2/Q2)*(-4*mH2*mstau12*
     mstau22*mu2 + mH2*mstau22*power2(mstau12) + 4*mstau22*mu2*power2(mstau12)
     + mH2*mstau12*power2(mstau22) + 4*mstau12*mu2*power2(mstau22) - 4*power2(
     mstau12)*power2(mstau22) - mH2*mstau22*DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*
     mstau12 + mu2) - mH2*(mstau12 + 2*mu2) + power2(mH2))*power2(mstau12 -
     mu2) - mH2*mstau12*DeltaInv(mstau22,mH2,mu2)*(mu2*(-3*mstau22 + mu2) -
     mH2*(mstau22 + 2*mu2) + power2(mH2))*power2(mstau22 - mu2) + mH2*mstau12*
     power2(mu2) + mH2*mstau22*power2(mu2) - 4*mstau12*mstau22*power2(mu2)))/(
     mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (6*mu2*log(
     mA2/Q2)*log(mu2/Q2)*(-4*mA2*mstau12*mstau22*mu2 + mA2*mstau22*power2(
     mstau12) - 4*mstau22*mu2*power2(mstau12) + mA2*mstau12*power2(mstau22) -
     4*mstau12*mu2*power2(mstau22) + 4*power2(mstau12)*power2(mstau22) - mA2*
     mstau22*DeltaInv(mA2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mA2*(mstau12 +
     2*mu2) + power2(mA2))*power2(mstau12 - mu2) - mA2*mstau12*DeltaInv(mA2,
     mstau22,mu2)*(mu2*(-3*mstau22 + mu2) - mA2*(mstau22 + 2*mu2) + power2(mA2)
     )*power2(mstau22 - mu2) + mA2*mstau12*power2(mu2) + mA2*mstau22*power2(
     mu2) + 4*mstau12*mstau22*power2(mu2)))/(mstau12*mstau22*power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) - (72*power2(sa))/mH2 - (24*(mh2 - mH2)*Fin20(
     mstau12,mu2,Q2)*power2(sa))/(mh2*mH2*(mstau12 - mstau22)) + (24*(mh2 -
     mH2)*Fin20(mstau22,mu2,Q2)*power2(sa))/(mh2*mH2*(mstau12 - mstau22)) + (
     72*log(mh2/Q2)*power2(sa))/(mh2 - mH2) - (72*log(mH2/Q2)*power2(sa))/(mh2
     - mH2) + (72*mstau12*log(mstau12/Q2)*power2(sa))/(mH2*mstau12 - mH2*
     mstau22) - (72*mstau12*log(mh2/Q2)*log(mstau12/Q2)*power2(sa))/((mh2 -
     mH2)*(mstau12 - mstau22)) + (72*mstau12*log(mH2/Q2)*log(mstau12/Q2)*
     power2(sa))/((mh2 - mH2)*(mstau12 - mstau22)) - (72*mstau22*log(mstau22/
     Q2)*power2(sa))/(mH2*mstau12 - mH2*mstau22) + (72*mstau22*log(mh2/Q2)*log(
     mstau22/Q2)*power2(sa))/((mh2 - mH2)*(mstau12 - mstau22)) - (72*mstau22*
     log(mH2/Q2)*log(mstau22/Q2)*power2(sa))/((mh2 - mH2)*(mstau12 - mstau22))
     + (72*(mh2 - mH2)*Fin20(mstau22,mstau12,Q2)*power2(sa))/(mh2*power2(mH2))
     + (6*mu2*log(mh2/Q2)*log(mu2/Q2)*(-4*mh2*mstau12*mstau22*mu2 + mh2*
     mstau22*power2(mstau12) + 4*mstau22*mu2*power2(mstau12) + mh2*mstau12*
     power2(mstau22) + 4*mstau12*mu2*power2(mstau22) - 4*power2(mstau12)*
     power2(mstau22) - mh2*mstau22*DeltaInv(mstau12,mh2,mu2)*(mu2*(-3*mstau12 +
     mu2) - mh2*(mstau12 + 2*mu2) + power2(mh2))*power2(mstau12 - mu2) - mh2*
     mstau12*DeltaInv(mstau22,mh2,mu2)*(mu2*(-3*mstau22 + mu2) - mh2*(mstau22 +
     2*mu2) + power2(mh2))*power2(mstau22 - mu2) + mh2*mstau12*power2(mu2) +
     mh2*mstau22*power2(mu2) - 4*mstau12*mstau22*power2(mu2))*power2(sa))/(
     mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (6*mu2*log(
     mH2/Q2)*log(mu2/Q2)*(-4*mH2*mstau12*mstau22*mu2 + mH2*mstau22*power2(
     mstau12) + 4*mstau22*mu2*power2(mstau12) + mH2*mstau12*power2(mstau22) +
     4*mstau12*mu2*power2(mstau22) - 4*power2(mstau12)*power2(mstau22) - mH2*
     mstau22*DeltaInv(mH2,mstau12,mu2)*(mu2*(-3*mstau12 + mu2) - mH2*(mstau12 +
     2*mu2) + power2(mH2))*power2(mstau12 - mu2) - mH2*mstau12*DeltaInv(
     mstau22,mH2,mu2)*(mu2*(-3*mstau22 + mu2) - mH2*(mstau22 + 2*mu2) + power2(
     mH2))*power2(mstau22 - mu2) + mH2*mstau12*power2(mu2) + mH2*mstau22*
     power2(mu2) - 4*mstau12*mstau22*power2(mu2))*power2(sa))/(mstau12*mstau22*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (18*log(mu2/Q2)*power2(mu2)
     *(mstau22*DeltaInv(mstau12,mh2,mu2)*((mstau12 - mu2)*mu2 + mh2*(3*mstau12
     + 2*mu2) - power2(mh2))*power2(mstau12 - mu2) + mstau22*DeltaInv(mH2,
     mstau12,mu2)*power2(mstau12 - mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*
     mu2 + power2(mH2) + power2(mu2)) + mstau12*power2(mstau22 - mu2)*(-(
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2))) + DeltaInv(mstau22,mH2,mu2)*(-3*mH2*mstau22 -
     2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(mu2))))*power2(sa))/(
     mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (3*power2(
     mu2)*(mstau22*DeltaInv(mstau12,mh2,mu2)*((mstau12 - mu2)*mu2 + mh2*(3*
     mstau12 + 2*mu2) - power2(mh2))*power2(mstau12 - mu2) + mstau22*DeltaInv(
     mH2,mstau12,mu2)*power2(mstau12 - mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 -
     mstau12*mu2 + power2(mH2) + power2(mu2)) + mstau12*power2(mstau22 - mu2)*(
     -(DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2))) + DeltaInv(mstau22,mH2,mu2)*(-3*mH2*mstau22 -
     2*mH2*mu2 - mstau22*mu2 + power2(mH2) + power2(mu2))))*power2(sa)*power2(
     log(mu2/Q2)))/(mstau12*mstau22*power2(mstau12 - mu2)*power2(mstau22 - mu2)
     ) - (12*mu2*log(msntau2/Q2)*log(mu2/Q2)*(mC2 + msntau2 + mu2 + DeltaInv(
     mC2,msntau2,mu2)*(mC2*mu2*(6*msntau2 + mu2) + (msntau2 + mu2)*power2(mC2)
     + (msntau2 - mu2)*power2(mu2) - power3(mC2))))/(msntau2*power2(mstau22 -
     mu2)) - 18*log(mh2/Q2)*power2(sa)*(4/(-mh2 + mH2) + mh2/(mstau12*mstau22 -
     mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + (3*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 - mu2) - (mh2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))) + 3*
     power2(sa)*power2(log(mh2/Q2))*(12/(-mh2 + mH2) + mh2/(mstau12*mstau22 -
     mstau12*mu2) - 3*mh2*DeltaInv(mstau22,mstau12,mh2) + (3*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (2*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (mh2*mstau24*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12*(-mstau22 + mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 - mu2) - (mh2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (mh2*mstau24*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (mh2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2))/power2(mstau12 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau12 - mu2) - (2*mh2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (2*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau22 - mu2) + (mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (2*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (mh2*mu2*
     DeltaInv(mstau12,mh2,mu2)*(-3*mh2*mstau12 - 2*mh2*mu2 - mstau12*mu2 +
     power2(mh2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mh2*mu2*
     DeltaInv(mstau22,mh2,mu2)*(-3*mh2*mstau22 - 2*mh2*mu2 - mstau22*mu2 +
     power2(mh2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*(-mstau22 + mu2)) - (DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))) - (6*
     mu2*log(mstau12/Q2)*log(mu2/Q2)*(mA2 + mH2 + 2*mstau12 + 2*mu2 + DeltaInv(
     mA2,mstau12,mu2)*(mA2*mu2*(6*mstau12 + mu2) + (mstau12 + mu2)*power2(mA2)
     + (mstau12 - mu2)*power2(mu2) - power3(mA2)) + DeltaInv(mH2,mstau12,mu2)*(
     mH2*mu2*(6*mstau12 + mu2) + (mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*
     power2(mu2) - power3(mH2))))/(mstau12*power2(mstau22 - mu2)) + (6*mu2*log(
     mstau12/Q2)*log(mu2/Q2)*power2(sa)*(-mh2 + mH2 + DeltaInv(mstau12,mh2,mu2)
     *(-(mh2*mu2*(6*mstau12 + mu2)) - (mstau12 + mu2)*power2(mh2) + (-mstau12 +
     mu2)*power2(mu2) + power3(mh2)) + DeltaInv(mH2,mstau12,mu2)*(mH2*mu2*(6*
     mstau12 + mu2) + (mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2)
     - power3(mH2))))/(mstau12*power2(mstau22 - mu2)) + (6*mu2*log(mstau22/Q2)*
     log(mu2/Q2)*power2(sa)*(-mh2 + mH2 + DeltaInv(mstau22,mh2,mu2)*(-(mh2*mu2*
     (6*mstau22 + mu2)) - (mstau22 + mu2)*power2(mh2) + (-mstau22 + mu2)*
     power2(mu2) + power3(mh2)) + DeltaInv(mstau22,mH2,mu2)*(mH2*mu2*(6*mstau22
     + mu2) + (mstau22 + mu2)*power2(mH2) + (mstau22 - mu2)*power2(mu2) -
     power3(mH2))))/(mstau22*power2(mstau12 - mu2)) - 6*log(mh2/Q2)*log(
     mstau22/Q2)*power2(sa)*(-4/(mstau12 - mstau22) + (4*mu2)/((mstau12 -
     mstau22)*(-mstau22 + mu2)) + mh2/(mstau12*mstau22 - mstau12*mu2) + (2*mh2*
     mu2)/(mstau22*power2(mstau12 - mu2)) - (2*mh2*mu2*DeltaInv(mstau22,mh2,
     mu2)*(mu2*(-2*mstau22 + mu2) - 2*mh2*(mstau22 + mu2) + power2(mh2)))/(
     mstau22*power2(mstau12 - mu2)) - (mh2*mu2)/(mstau12*power2(mstau22 - mu2))
     - (mh2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*(mstau12 + 2*mstau22)*(mstau22
     - 2*mu2)*power2(mstau12 - mu2)) + (mstau22 - 2*mu2)*power2(mh2)*power2(
     mstau12 - mu2) + mstau24*(mstau22 - 2*mu2)*power2(mu2) - 3*mstau22*(
     mstau22 - 2*mu2)*power3(mstau12) + mstau12*mu2*(-2*mstau22*mstau24 + 4*
     mstau24*mu2 + 5*mu2*power2(mstau22) + 2*mstau22*power2(mu2) - 4*power3(
     mstau22)) + power2(mstau12)*(mstau22*mstau24 - 2*mstau24*mu2 + 2*mu2*
     power2(mstau22) - 10*mstau22*power2(mu2) + 2*power3(mstau22))))/(mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))) - 18*log(mH2/Q2)*(4/mH2 + 6/
     mstau12 + mH2/(mstau12*mstau22 - mstau12*mu2) + mH2/power2(mstau12 - mu2)
     - (mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*mu2*DeltaInv(mstau22,mH2,
     mu2)*power2(mH2))/power2(mstau12 - mu2) - (2*mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(
     mstau12 - mu2) - (2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*
     mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)))/(
     mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(18*
     mstau22 - (6*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 - mu2) + 3/(
     mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(
     mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*mu2)/
     power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) + mH2*(3 +
     (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-
     mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/power2(
     mstau12 - mu2) + (12*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(mstau22
     - mu2))/mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(mu2)/
     power2(mstau22 - mu2)) + (-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/
     (mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (mH2*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))) + 3*power2(log(mH2/Q2))
     *(6/mH2 + 6/mstau12 + mH2/(mstau12*mstau22 - mstau12*mu2) + mH2/power2(
     mstau12 - mu2) - (mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*mu2*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (2*mH2*mu2)
     /(mstau12*power2(mstau22 - mu2)) - (mH2*DeltaInv(mstau22,mH2,mu2)*power2(
     mu2))/power2(mstau12 - mu2) - (2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*
     power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2*DeltaInv(mH2,
     mstau12,mu2)*(-3*mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) +
     power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mH2,
     mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + DeltaInv(mstau22,mH2,
     mstau12)*(18*mstau22 - (6*mstau24)/mstau12 + power2(mH2)*(3/(mstau12 -
     mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(mstau12*mstau22 - mstau12*mu2) +
     1/power2(mstau12 - mu2)) + mu2/power2(mstau12 - mu2) + (-6 - (2*mstau22*
     mu2)/power2(mstau22 - mu2))/mstau12 - (3*mu2)/power2(mstau22 - mu2)) +
     mH2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*mstau22)/(mstau22 - mu2) + (2*
     mu2)/(-mstau12 + mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (12*mstau22 - (mstau24*(mstau22 - 2*mu2))/power2(
     mstau22 - mu2))/mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(
     mu2)/power2(mstau22 - mu2)) + (-(1/power2(mstau12 - mu2)) + (-mstau22 + 2*
     mu2)/(mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (mH2*DeltaInv(
     mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))) - 3*power2(
     sa)*power2(log(mH2/Q2))*(12/mH2 + 12/(-mh2 + mH2) + 12/mstau12 + mH2/(
     mstau12*mstau22 - mstau12*mu2) + (16*mH2)/(-(mh2*mH2) + power2(mh2)) - (
     16*mH2)/power2(mh2 - mH2) + (4*power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (
     20*power2(mH2))/(mh2*power2(mh2 - mH2)) + mH2/power2(mstau12 - mu2) - (
     mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (3*mu2*DeltaInv(mstau22,mH2,
     mu2)*power2(mH2))/power2(mstau12 - mu2) - (2*mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(
     mstau12 - mu2) - (2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*
     mH2*mstau12 - 2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)))/(
     mstau12*power2(mstau22 - mu2)) - (4*power3(mH2))/(power2(mh2)*power2(mh2 -
     mH2)) + (mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(
     mstau12 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*(36*mstau22 - (12*mstau24)
     /mstau12 + power2(mH2)*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*
     (1/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/
     power2(mstau12 - mu2) + (-12 - (2*mstau22*mu2)/power2(mstau22 - mu2))/
     mstau12 - (3*mu2)/power2(mstau22 - mu2)) + mH2*(9 + (3*mstau22)/(mstau12 -
     mu2) + (2*mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/
     power2(mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) + (24*mstau22
     - (mstau24*(mstau22 - 2*mu2))/power2(mstau22 - mu2))/mstau12 - (2*mstau22*
     mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(mstau22 - mu2)) + (-(1/
     power2(mstau12 - mu2)) + (-mstau22 + 2*mu2)/(mstau12*power2(mstau22 - mu2)
     ))*power3(mH2)) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2))) + 6*log(mH2/Q2)*power2(sa)*(-2/mh2 + 24/mH2 + 10/(
     -mh2 + mH2) + 36/mstau12 + (3*mH2)/(mstau12*mstau22 - mstau12*mu2) - (2*
     mH2)/power2(mh2) + (16*mH2)/(-(mh2*mH2) + power2(mh2)) - (16*mH2)/power2(
     mh2 - mH2) + (8*power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (22*power2(mH2))
     /(mh2*power2(mh2 - mH2)) + (3*mH2)/power2(mstau12 - mu2) - (3*mH2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (9*mu2*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2))/power2(mstau12 - mu2) - (6*mH2*mu2)/(mstau12*power2(mstau22 - mu2))
     - (3*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (
     6*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(
     mstau12 - mu2)) + (3*mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(-3*mH2*mstau12 -
     2*mH2*mu2 - mstau12*mu2 + power2(mH2) + power2(mu2)))/(mstau12*power2(
     mstau22 - mu2)) - (6*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) + (3*
     mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2))
     + 3*DeltaInv(mstau22,mH2,mstau12)*(36*mstau22 - (12*mstau24)/mstau12 +
     power2(mH2)*(3/(mstau12 - mu2) + 3/(mstau22 - mu2) + 2*mstau22*(1/(
     mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2)) + mu2/power2(
     mstau12 - mu2) + (-12 - (2*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (
     3*mu2)/power2(mstau22 - mu2)) + mH2*(9 + (3*mstau22)/(mstau12 - mu2) + (2*
     mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - mstau24/(mstau12*
     mstau22 - mstau12*mu2) - mstau24/power2(mstau12 - mu2) + (mstau22*mu2)/
     power2(mstau12 - mu2) + (24*mstau22 + (mstau24*mu2)/power2(mstau22 - mu2))
     /mstau12 - (2*mstau22*mu2)/power2(mstau22 - mu2) + power2(mu2)/power2(
     mstau22 - mu2)) + (-(1/(mstau12*mstau22 - mstau12*mu2)) - 1/power2(mstau12
     - mu2) + mu2/(mstau12*power2(mstau22 - mu2)))*power3(mH2)) + (3*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2))) +
     3*mu2*power2(log(mu2/Q2))*(-2/power2(msntau2 - mu2) - (2*mu2)/(mstau22*
     power2(msntau2 - mu2)) - (6*mC2*mu2*DeltaInv(mstau22,mC2,mu2))/power2(
     msntau2 - mu2) + (2*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/(mstau22*
     power2(msntau2 - mu2)) - 2/power2(mstau12 - mu2) - (2*mu2)/(mstau22*
     power2(mstau12 - mu2)) - (3*mH2*mu2*DeltaInv(mstau22,mH2,mu2))/power2(
     mstau12 - mu2) + (mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/(mstau22*
     power2(mstau12 - mu2)) - 4/power2(mstau22 - mu2) - (2*mu2)/(msntau2*
     power2(mstau22 - mu2)) - (2*mu2)/(mstau12*power2(mstau22 - mu2)) - (6*mC2*
     mu2*DeltaInv(mC2,msntau2,mu2))/power2(mstau22 - mu2) - (3*mH2*mu2*
     DeltaInv(mH2,mstau12,mu2))/power2(mstau22 - mu2) + (2*mu2*DeltaInv(mC2,
     msntau2,mu2)*power2(mC2))/(msntau2*power2(mstau22 - mu2)) + (mu2*DeltaInv(
     mH2,mstau12,mu2)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) - (2*
     DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (4*mC2*
     DeltaInv(mstau22,mC2,mu2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (2*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     2*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (4*mC2*
     DeltaInv(mC2,msntau2,mu2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) - (2*mH2*
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2 +
     power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mA2,mstau22,mu2)*(-3*mA2*mstau22 - 2*mA2*mu2 - mstau22*mu2 +
     power2(mA2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) + (4*mu2)/
     power3(msntau2 - mu2) + (4*mu2)/power3(mstau12 - mu2) + (8*mu2)/power3(
     mstau22 - mu2) + (2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mstau22*
     power2(msntau2 - mu2)) + (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*
     power2(mstau12 - mu2)) + (2*DeltaInv(mC2,msntau2,mu2)*power3(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(
     mstau12*power2(mstau22 - mu2))) + 6*log(mh2/Q2)*log(mstau12/Q2)*power2(sa)
     *(-4/(mstau12 - mstau22) + (4*mu2)/((mstau12 - mstau22)*(-mstau12 + mu2))
     + (2*mh2)/(mstau12*mstau22 - mstau12*mu2) + mh2/power2(mstau12 - mu2) - (
     4*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) + (2*mh2*mu2*DeltaInv(mstau12,
     mh2,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mh2*(mstau12 + mu2) + power2(mh2)))/(
     mstau12*power2(mstau22 - mu2)) + (mh2*DeltaInv(mstau22,mstau12,mh2)*(2*
     mstau24*(-mstau22 + 2*mu2)*power2(mu2) - power2(mh2)*(2*(mstau22 - 2*mu2)*
     power2(mstau12) + mstau12*power2(mstau22 - 3*mu2) + 2*(mstau22 - 2*mu2)*
     power2(mu2)) + 2*(-2*mstau22*mu2 + power2(mstau22) - power2(mu2))*power3(
     mstau12) + mh2*(4*mstau22*(mstau22 - 2*mu2)*power2(mu2) + power2(mstau12)*
     (-22*mstau22*mu2 + 7*power2(mstau22) + 19*power2(mu2)) + 4*(mstau22 - 2*
     mu2)*power3(mstau12) + 2*mstau12*(-7*mu2*power2(mstau22) + 13*mstau22*
     power2(mu2) + power3(mstau22) - 5*power3(mu2))) + power2(mstau12)*(-2*
     mstau22*mstau24 + 4*mstau24*mu2 - 8*mu2*power2(mstau22) + 13*mstau22*
     power2(mu2) + power3(mstau22) + 2*power3(mu2)) + mstau12*(6*mstau22*
     mstau24*mu2 - mstau24*power2(mstau22) - 9*mstau24*power2(mu2) + 2*mu2*
     power3(mstau22) - 6*mstau22*power3(mu2))))/(mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2))) + 6*mu2*log(mu2/Q2)*(4/power2(msntau2 - mu2) + (6*
     mu2)/(mstau22*power2(msntau2 - mu2)) + (18*mC2*mu2*DeltaInv(mstau22,mC2,
     mu2))/power2(msntau2 - mu2) - (6*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2)
     )/(mstau22*power2(msntau2 - mu2)) + 4/power2(mstau12 - mu2) + (6*mu2)/(
     mstau22*power2(mstau12 - mu2)) + (9*mH2*mu2*DeltaInv(mstau22,mH2,mu2))/
     power2(mstau12 - mu2) - (3*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/(
     mstau22*power2(mstau12 - mu2)) + 8/power2(mstau22 - mu2) + (6*mu2)/(
     msntau2*power2(mstau22 - mu2)) + (6*mu2)/(mstau12*power2(mstau22 - mu2)) +
     (18*mC2*mu2*DeltaInv(mC2,msntau2,mu2))/power2(mstau22 - mu2) + (9*mH2*mu2*
     DeltaInv(mH2,mstau12,mu2))/power2(mstau22 - mu2) - (6*mu2*DeltaInv(mC2,
     msntau2,mu2)*power2(mC2))/(msntau2*power2(mstau22 - mu2)) - (3*mu2*
     DeltaInv(mH2,mstau12,mu2)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (
     6*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (12*mC2*
     DeltaInv(mstau22,mC2,mu2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) + (
     3*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (6*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     6*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) + (12*mC2*
     DeltaInv(mC2,msntau2,mu2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (
     3*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2
     + power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*
     DeltaInv(mA2,mstau22,mu2)*(-3*mA2*mstau22 - 2*mA2*mu2 - mstau22*mu2 +
     power2(mA2) + power2(mu2)))/(mstau22*power2(mstau12 - mu2)) - (6*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) - (3*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     6*DeltaInv(mC2,msntau2,mu2)*power3(mu2))/(msntau2*power2(mstau22 - mu2)) -
     (3*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2))
     + (4*mu2)/power3(-msntau2 + mu2) + (4*mu2)/power3(-mstau12 + mu2) + (8*
     mu2)/power3(-mstau22 + mu2)) + 6*power2(log(mC2/Q2))*(1/mC2 + 1/msntau2 +
     mC2/(msntau2*mstau22 - msntau2*mu2) + mC2/power2(msntau2 - mu2) - (mC2*
     mu2)/(mstau22*power2(msntau2 - mu2)) - (3*mu2*DeltaInv(mstau22,mC2,mu2)*
     power2(mC2))/power2(msntau2 - mu2) - (2*mC2*mu2)/(msntau2*power2(mstau22 -
     mu2)) - (mC2*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2)
     - (2*DeltaInv(mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(
     msntau2 - mu2)) + (mC2*mu2*DeltaInv(mC2,msntau2,mu2)*(-3*mC2*msntau2 - 2*
     mC2*mu2 - msntau2*mu2 + power2(mC2) + power2(mu2)))/(msntau2*power2(
     mstau22 - mu2)) + (mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*
     power2(msntau2 - mu2)) + (mC2*DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(
     mstau22*power2(msntau2 - mu2)) - (DeltaInv(mstau22,mC2,msntau2)*(-((3*
     msntau2*mstau22 - mstau24)*power2(msntau2 - mu2)*power2(mstau22 - mu2)) +
     ((mstau22 - 2*mu2)*power2(msntau2) + (mstau22 - 2*mu2)*power2(mu2) +
     msntau2*(-4*mstau22*mu2 + power2(mstau22) + 5*power2(mu2)))*power3(mC2) +
     power2(mC2)*(power2(mu2)*(2*mstau22*mu2 - power2(mstau22) + power2(mu2)) -
     2*power2(msntau2)*(-7*mstau22*mu2 + 2*power2(mstau22) + 7*power2(mu2)) -
     3*(mstau22 - 2*mu2)*power3(msntau2) + msntau2*(8*mu2*power2(mstau22) - 13*
     mstau22*power2(mu2) - 2*power3(mstau22) + 6*power3(mu2))) + mC2*(power2(
     mu2)*power3(msntau2) + power2(msntau2)*(-2*mstau24*mu2 + 12*mu2*power2(
     mstau22) + mstau22*(mstau24 - 9*power2(mu2)) - 5*power3(mstau22)) +
     power2(mu2)*(-2*mstau24*mu2 + 4*mu2*power2(mstau22) + mstau22*(mstau24 -
     2*power2(mu2)) - 2*power3(mstau22)) + msntau2*(power2(mstau22)*(mstau24 -
     14*power2(mu2)) + 5*mstau24*power2(mu2) + 6*mu2*power3(mstau22) + mstau22*
     (-4*mstau24*mu2 + 10*power3(mu2)) - power4(mu2)))))/(msntau2*power2(
     msntau2 - mu2)*power2(mstau22 - mu2))) + 12*log(mC2/Q2)*(-2/mC2 - 3/
     msntau2 - (3*mC2)/(msntau2*mstau22 - msntau2*mu2) - (3*mC2)/power2(msntau2
     - mu2) + (3*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (9*mu2*DeltaInv(
     mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (6*mC2*mu2)/(
     msntau2*power2(mstau22 - mu2)) + (3*mC2*DeltaInv(mstau22,mC2,mu2)*power2(
     mu2))/power2(msntau2 - mu2) + (6*DeltaInv(mstau22,mC2,mu2)*power2(mC2)*
     power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (3*mC2*mu2*DeltaInv(mC2,
     msntau2,mu2)*(-3*mC2*msntau2 - 2*mC2*mu2 - msntau2*mu2 + power2(mC2) +
     power2(mu2)))/(msntau2*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,
     mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 - mu2)) - (3*mC2*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) + (3*
     DeltaInv(mstau22,mC2,msntau2)*(-((3*msntau2*mstau22 - mstau24)*power2(
     msntau2 - mu2)*power2(mstau22 - mu2)) + ((mstau22 - 2*mu2)*power2(msntau2)
     + (mstau22 - 2*mu2)*power2(mu2) + msntau2*(-4*mstau22*mu2 + power2(
     mstau22) + 5*power2(mu2)))*power3(mC2) + power2(mC2)*(power2(mu2)*(2*
     mstau22*mu2 - power2(mstau22) + power2(mu2)) - 2*power2(msntau2)*(-7*
     mstau22*mu2 + 2*power2(mstau22) + 7*power2(mu2)) - 3*(mstau22 - 2*mu2)*
     power3(msntau2) + msntau2*(8*mu2*power2(mstau22) - 13*mstau22*power2(mu2)
     - 2*power3(mstau22) + 6*power3(mu2))) + mC2*(power2(mu2)*power3(msntau2) +
     power2(msntau2)*(-2*mstau24*mu2 + 12*mu2*power2(mstau22) + mstau22*(
     mstau24 - 9*power2(mu2)) - 5*power3(mstau22)) + power2(mu2)*(-2*mstau24*
     mu2 + 4*mu2*power2(mstau22) + mstau22*(mstau24 - 2*power2(mu2)) - 2*
     power3(mstau22)) + msntau2*(power2(mstau22)*(mstau24 - 14*power2(mu2)) +
     5*mstau24*power2(mu2) + 6*mu2*power3(mstau22) + mstau22*(-4*mstau24*mu2 +
     10*power3(mu2)) - power4(mu2)))))/(msntau2*power2(msntau2 - mu2)*power2(
     mstau22 - mu2))) + 3*power2(log(mA2/Q2))*(-2/mA2 - 2/mstau12 + mA2/(
     mstau12*mstau22 - mstau12*mu2) + mA2/power2(mstau12 - mu2) - (mA2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (3*mu2*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2))/power2(mstau12 - mu2) - (2*mA2*mu2)/(mstau12*power2(mstau22 - mu2))
     - (mA2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/power2(mstau12 - mu2) - (2*
     DeltaInv(mA2,mstau22,mu2)*power2(mA2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (mA2*mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 -
     mstau12*mu2 + power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2))
     + (mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*power2(mstau12 -
     mu2)) + (mA2*DeltaInv(mA2,mstau22,mu2)*power3(mu2))/(mstau22*power2(
     mstau12 - mu2)) - (DeltaInv(mA2,mstau22,mstau12)*(2*(3*mstau12*mstau22 -
     mstau24)*power2(mstau12 - mu2)*power2(mstau22 - mu2) + ((mstau22 - 2*mu2)*
     power2(mstau12) + (mstau22 - 2*mu2)*power2(mu2) + mstau12*(-4*mstau22*mu2
     + power2(mstau22) + 5*power2(mu2)))*power3(mA2) - power2(mA2)*(2*power2(
     mu2)*(-4*mstau22*mu2 + 2*power2(mstau22) + power2(mu2)) + power2(mstau12)*
     (-20*mstau22*mu2 + 7*power2(mstau22) + 17*power2(mu2)) + 3*(mstau22 - 2*
     mu2)*power3(mstau12) + mstau12*(-14*mu2*power2(mstau22) + 25*mstau22*
     power2(mu2) + 2*power3(mstau22) - 12*power3(mu2))) + mA2*((-6*mstau22*mu2
     + 3*power2(mstau22) + 4*power2(mu2))*power3(mstau12) + power2(mstau12)*(-
     6*mu2*power2(mstau22) - 2*mu2*(mstau24 + 3*power2(mu2)) + mstau22*(mstau24
     + 9*power2(mu2)) + power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 8*mu2*
     power2(mstau22) + mstau22*(mstau24 + 4*power2(mu2)) + 4*power3(mstau22)) +
     mstau12*(5*mstau24*power2(mu2) - 4*mstau22*mu2*(mstau24 + 2*power2(mu2)) +
     power2(mstau22)*(mstau24 + 13*power2(mu2)) - 6*mu2*power3(mstau22) + 2*
     power4(mu2)))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))) +
     6*log(mA2/Q2)*(4/mA2 + 6/mstau12 - (3*mA2)/(mstau12*mstau22 - mstau12*mu2)
     - (3*mA2)/power2(mstau12 - mu2) + (3*mA2*mu2)/(mstau22*power2(mstau12 -
     mu2)) + (9*mu2*DeltaInv(mA2,mstau22,mu2)*power2(mA2))/power2(mstau12 -
     mu2) + (6*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) + (3*mA2*DeltaInv(mA2,
     mstau22,mu2)*power2(mu2))/power2(mstau12 - mu2) + (6*DeltaInv(mA2,mstau22,
     mu2)*power2(mA2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*mA2*
     mu2*DeltaInv(mA2,mstau12,mu2)*(-3*mA2*mstau12 - 2*mA2*mu2 - mstau12*mu2 +
     power2(mA2) + power2(mu2)))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*
     DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*power2(mstau12 - mu2)) - (
     3*mA2*DeltaInv(mA2,mstau22,mu2)*power3(mu2))/(mstau22*power2(mstau12 -
     mu2)) + (3*DeltaInv(mA2,mstau22,mstau12)*(2*(3*mstau12*mstau22 - mstau24)*
     power2(mstau12 - mu2)*power2(mstau22 - mu2) + ((mstau22 - 2*mu2)*power2(
     mstau12) + (mstau22 - 2*mu2)*power2(mu2) + mstau12*(-4*mstau22*mu2 +
     power2(mstau22) + 5*power2(mu2)))*power3(mA2) - power2(mA2)*(2*power2(mu2)
     *(-4*mstau22*mu2 + 2*power2(mstau22) + power2(mu2)) + power2(mstau12)*(-
     20*mstau22*mu2 + 7*power2(mstau22) + 17*power2(mu2)) + 3*(mstau22 - 2*mu2)
     *power3(mstau12) + mstau12*(-14*mu2*power2(mstau22) + 25*mstau22*power2(
     mu2) + 2*power3(mstau22) - 12*power3(mu2))) + mA2*((-6*mstau22*mu2 + 3*
     power2(mstau22) + 4*power2(mu2))*power3(mstau12) + power2(mstau12)*(-6*
     mu2*power2(mstau22) - 2*mu2*(mstau24 + 3*power2(mu2)) + mstau22*(mstau24 +
     9*power2(mu2)) + power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 8*mu2*
     power2(mstau22) + mstau22*(mstau24 + 4*power2(mu2)) + 4*power3(mstau22)) +
     mstau12*(5*mstau24*power2(mu2) - 4*mstau22*mu2*(mstau24 + 2*power2(mu2)) +
     power2(mstau22)*(mstau24 + 13*power2(mu2)) - 6*mu2*power3(mstau22) + 2*
     power4(mu2)))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))) + (
     36*(mh2 + mH2)*power4(sa))/(mh2*mH2) + (72*log(mh2/Q2)*power4(sa))/(-mh2 +
     mH2) + (72*log(mH2/Q2)*power4(sa))/(mh2 - mH2) - (36*(mh2 + mH2)*mstau12*
     log(mstau12/Q2)*power4(sa))/(mh2*mH2*(mstau12 - mstau22)) + (72*mstau12*
     log(mh2/Q2)*log(mstau12/Q2)*power4(sa))/((mh2 - mH2)*(mstau12 - mstau22))
     - (72*mstau12*log(mH2/Q2)*log(mstau12/Q2)*power4(sa))/((mh2 - mH2)*(
     mstau12 - mstau22)) + (36*(mh2 + mH2)*mstau22*log(mstau22/Q2)*power4(sa))/
     (mh2*mH2*(mstau12 - mstau22)) + (72*mstau22*log(mh2/Q2)*log(mstau22/Q2)*
     power4(sa))/((-mh2 + mH2)*(mstau12 - mstau22)) + (72*mstau22*log(mH2/Q2)*
     log(mstau22/Q2)*power4(sa))/((mh2 - mH2)*(mstau12 - mstau22)) + (36*log(
     mh2/Q2)*(3*mh2*mH2 - 4*mh2*mstau12 + 2*mH2*mstau12 - 3*power2(mh2) + 3*
     mh2*(mh2 - mH2)*DeltaInv(mstau22,mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(
     mstau12 + 2*mstau22) + mstau24 + power2(mh2)))*power4(sa))/(mh2*(mh2 -
     mH2)*mstau12) - (36*Fin20(mstau22,mstau12,Q2)*power2(mh2 - mH2)*power4(sa)
     )/(power2(mh2)*power2(mH2)) + (36*log(mH2/Q2)*(-3*mh2*mH2 - 2*mh2*mstau12
     + 4*mH2*mstau12 + 3*power2(mH2) + 3*(mh2 - mH2)*mH2*DeltaInv(mstau22,mH2,
     mstau12)*(-3*mstau12*mstau22 - mH2*(mstau12 + 2*mstau22) + mstau24 +
     power2(mH2)))*power4(sa))/((mh2 - mH2)*mH2*mstau12) - (18*(mh2*(mH2 - 3*
     mstau12) + mH2*mstau12 - power2(mh2) + mh2*(mh2 - mH2)*DeltaInv(mstau22,
     mstau12,mh2)*(-3*mstau12*mstau22 - mh2*(mstau12 + 2*mstau22) + mstau24 +
     power2(mh2)))*power2(log(mh2/Q2))*power4(sa))/(mh2*(mh2 - mH2)*mstau12) +
     (18*(-(mh2*(mH2 + mstau12)) + mH2*(mH2 + 3*mstau12) + (mh2 - mH2)*mH2*
     DeltaInv(mstau22,mH2,mstau12)*(-3*mstau12*mstau22 - mH2*(mstau12 + 2*
     mstau22) + mstau24 + power2(mH2)))*power2(log(mH2/Q2))*power4(sa))/(mH2*(-
     mh2 + mH2)*mstau12) - (6*mu2*log(mstau22/Q2)*log(mu2/Q2)*(-2*mA2*msntau2*
     mu2 - 2*mH2*msntau2*mu2 - 4*mC2*mstau12*mu2 - 4*msntau2*mstau22*mu2 - 4*
     mstau12*mstau22*mu2 - 2*msntau2*mstau22*mu2*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2) + mA2*power2(msntau2) + mH2*power2(msntau2) + 2*mstau22*
     power2(msntau2) + 2*mu2*power2(msntau2) + 6*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(msntau2) + mstau22*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2)*power2(msntau2) + mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*
     power2(msntau2) + 2*mC2*power2(mstau12) + 2*mstau22*power2(mstau12) + 2*
     mu2*power2(mstau12) + mA2*power2(mu2) + 2*mC2*power2(mu2) + mH2*power2(
     mu2) - 4*msntau2*power2(mu2) - 4*mstau12*power2(mu2) + 4*mstau22*power2(
     mu2) - 12*mH2*msntau2*mstau22*DeltaInv(mstau22,mH2,mu2)*power2(mu2) - 2*
     msntau2*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2) + mstau22*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2) + mH2*DeltaInv(mstau22,
     mH2,mu2)*power2(msntau2)*power2(mu2) + mstau22*DeltaInv(mstau22,mH2,mu2)*
     power2(msntau2)*power2(mu2) - DeltaInv(mA2,mstau22,mu2)*power2(msntau2 -
     mu2)*(-(mA2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*power2(mA2) + (-
     mstau22 + mu2)*power2(mu2) + power3(mA2)) - 2*DeltaInv(mstau22,mC2,mu2)*
     power2(mstau12 - mu2)*(-(mC2*mu2*(6*mstau22 + mu2)) - (mstau22 + mu2)*
     power2(mC2) + (-mstau22 + mu2)*power2(mu2) + power3(mC2)) + 2*msntau2*mu2*
     DeltaInv(mstau22,mH2,mu2)*power3(mH2) - DeltaInv(mstau22,mH2,mu2)*power2(
     msntau2)*power3(mH2) - DeltaInv(mstau22,mH2,mu2)*power2(mu2)*power3(mH2) +
     4*power3(mu2) - 2*mH2*msntau2*DeltaInv(mstau22,mH2,mu2)*power3(mu2) + 6*
     mH2*mstau22*DeltaInv(mstau22,mH2,mu2)*power3(mu2) - 2*msntau2*mstau22*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2) + DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power3(mu2) - DeltaInv(mstau22,mH2,mu2)*power2(msntau2)*power3(mu2) +
     mH2*DeltaInv(mstau22,mH2,mu2)*power4(mu2) + 2*msntau2*DeltaInv(mstau22,
     mH2,mu2)*power4(mu2) + mstau22*DeltaInv(mstau22,mH2,mu2)*power4(mu2) -
     DeltaInv(mstau22,mH2,mu2)*power5(mu2)))/(mstau22*power2(msntau2 - mu2)*
     power2(mstau12 - mu2)) - 6*log(mstau12/Q2)*power2(sa)*(-36/mH2 - 36/
     mstau12 - (108*mstau22)/(mH2*mstau12) + (9*mh2)/(mstau12*mstau22 -
     mstau12*mu2) - (9*mH2)/(mstau12*mstau22 - mstau12*mu2) - 42*mH2*DeltaInv(
     mstau22,mH2,mstau12) - 357*mstau22*DeltaInv(mstau22,mH2,mstau12) + (30*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 - (180*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/mH2 - (177*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/mstau12 + (6*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 -
     mu2) + (6*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (9*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (18*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau22 + mu2) + (9*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) + 6*mh2*DeltaInv(
     mstau22,mstau12,mh2) - 3*mstau22*DeltaInv(mstau22,mstau12,mh2) + (6*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 - (3*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/mstau12 + (6*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12 - mu2) + (9*mstau24*DeltaInv(mstau22,mstau12,mh2))/(mstau12
     - mu2) + (18*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau22 - mu2) +
     (6*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (6*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) - (9*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + (2*
     mstau12)/power2(mh2) + (2*mH2*mstau12)/((mh2 - mH2)*power2(mh2)) - (3*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (9*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12 - mu2) + (15*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau22 - mu2) + (18*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2) - (2*mstau12)/(-(mh2*
     mH2) + power2(mh2)) + (39*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     mstau12 + (9*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau12 + mu2) +
     (15*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau22 + mu2) - (18*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (3*mh2)/power2(mstau12 - mu2) - (3*mH2)/power2(mstau12 -
     mu2) - (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2)
     - (18*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2)
     - (3*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (
     3*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (18*
     mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (3*
     mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (3*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) +
     (3*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) -
     (3*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 -
     mu2) - (3*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 -
     mu2) - (18*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mH2*mu2)/(
     mstau12*power2(mstau22 - mu2)) + (18*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau22 - mu2) - (9*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12*power2(mstau22 - mu2)) - (18*mh2*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2))/power2(mstau22 - mu2) + (9*mh2*mstau24*mu2*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) - (15*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) - (18*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2))
     + (15*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2)
     + (18*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*
     power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-
     mstau22 + mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau22 -
     mu2) + (12*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 -
     mu2) + (6*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(
     mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (3*mstau24*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (12*mh2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (6*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) - (6*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (9*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2)
     - (3*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) + (
     9*mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 -
     mu2)) + (3*mu2*DeltaInv(mstau12,mh2,mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*
     (mstau12 + mu2)*power2(mh2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mh2)
     ))/(mstau12*power2(mstau22 - mu2)) + (9*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (3*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/power2(mstau12 - mu2) - (9*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(
     mH2,mstau12,mu2)*(mH2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(
     mH2) + (-mstau12 + mu2)*power2(mu2) + 3*power3(mH2)))/(mstau12*power2(
     mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(
     mstau22 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(
     mstau22 - mu2) + (108*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*
     mstau12) + (3*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12
     - mu2) - (3*DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 -
     mu2)) + 3*power2(sa)*power2(log(mstau12/Q2))*(-12/mH2 - 12/mstau12 - (36*
     mstau22)/(mH2*mstau12) + (3*mh2)/(mstau12*mstau22 - mstau12*mu2) - (3*mH2)
     /(mstau12*mstau22 - mstau12*mu2) - 14*mH2*DeltaInv(mstau22,mH2,mstau12) -
     119*mstau22*DeltaInv(mstau22,mH2,mstau12) + (10*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 - (60*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /mH2 - (59*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (2*mH2*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(-mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau22 + mu2) + (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     mstau22 - mstau12*mu2) + 2*mh2*DeltaInv(mstau22,mstau12,mh2) - mstau22*
     DeltaInv(mstau22,mstau12,mh2) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,
     mh2))/mstau12 - (mstau24*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (3*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (6*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/(mstau22 - mu2) + (2*mh2*mu2*DeltaInv(mstau22,
     mstau12,mh2))/(-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(-mstau12 + mu2) - (3*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/mstau12 + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12 -
     mu2) + (5*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau22 - mu2) + (6*
     mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 -
     mstau12*mu2) + (13*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (
     3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau12 + mu2) + (5*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau22 + mu2) - (6*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2)
     + mh2/power2(mstau12 - mu2) - mH2/power2(mstau12 - mu2) - (mH2*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (6*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (mstau24*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (6*mh2*mstau22*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau24*mu2*
     DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) + (mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) + (mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2) - (mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (6*mh2*
     mu2)/(mstau12*power2(mstau22 - mu2)) + (6*mH2*mu2)/(mstau12*power2(mstau22
     - mu2)) + (6*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22
     - mu2) - (3*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     power2(mstau22 - mu2)) - (6*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau22 - mu2) + (3*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))
     /(mstau12*power2(mstau22 - mu2)) - (5*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau22 - mu2) - (6*mstau22*mu2*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) + (5*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (6*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 +
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau22 - mu2) + (4*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) +
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (4*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) - (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/
     (mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power3(
     mh2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power3(mh2))/(mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau12,mh2,
     mu2)*(mh2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mh2) + (-
     mstau12 + mu2)*power2(mu2) + 3*power3(mh2)))/(mstau12*power2(mstau22 -
     mu2)) + (mu2*DeltaInv(mH2,mstau12,mu2)*(mH2*(10*mstau12 - mu2)*mu2 + 5*(
     mstau12 + mu2)*power2(mH2) + (mstau12 - mu2)*power2(mu2) - 3*power3(mH2)))
     /(mstau12*power2(mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*
     power3(mH2))/(mstau12*mstau22 - mstau12*mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/power2(mstau12 - mu2) - (3*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,
     mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,
     mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (36*DeltaInv(mstau22,
     mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/power2(mstau12 - mu2) - (DeltaInv(mstau22,
     mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2)) - power2(Pi)*power2(
     sa)*(12/mH2 + 12/mstau12 + (24*mstau22)/(mH2*mstau12) - (2*mh2)/(mstau12*
     mstau22 - mstau12*mu2) + (2*mH2)/(mstau12*mstau22 - mstau12*mu2) + 13*mH2*
     DeltaInv(mstau22,mH2,mstau12) + 83*mstau22*DeltaInv(mstau22,mH2,mstau12) +
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (48*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/mH2 + (35*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/mstau12 + (4*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12
     - mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (4*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau22 - mu2) + (2*mH2*mu2*
     DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(-mstau12 + mu2) - (2*mH2*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) - mh2*DeltaInv(mstau22,
     mstau12,mh2) + mstau22*DeltaInv(mstau22,mstau12,mh2) - (2*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (mstau24*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (2*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12
     - mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) + (
     4*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (3*
     mstau24*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (4*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (2*mh2*mstau24*DeltaInv(
     mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) + (DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2))/mstau12 + (3*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/(-mstau12 + mu2) + (4*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(-mstau22 + mu2) - (4*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/(mstau12*mstau22 - mstau12*mu2) + (8*mH2)/(-(mh2*mH2) + power2(mh2))
     - (8*mH2)/power2(mh2 - mH2) - (13*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 -
     mu2) + (4*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (4*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (2*power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (10*power2(
     mH2))/(mh2*power2(mh2 - mH2)) - mh2/power2(mstau12 - mu2) + mH2/power2(
     mstau12 - mu2) + (2*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (2*mH2*mu2)
     /(mstau22*power2(mstau12 - mu2)) + (mH2*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) + (4*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) + (mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) - (mh2*mstau24*DeltaInv(mstau22,mstau12,
     mh2))/power2(mstau12 - mu2) - (4*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/power2(mstau12 - mu2) - (mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (4*mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/
     power2(mstau12 - mu2) - (mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)
     )/power2(mstau12 - mu2) - (mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/
     power2(mstau12 - mu2) + (mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)
     )/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     power2(mstau12 - mu2) - (4*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/
     power2(mstau12 - mu2) + (4*mh2*mu2)/(mstau12*power2(mstau22 - mu2)) - (4*
     mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (4*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau22 - mu2) + (2*mH2*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (4*mh2*mstau22*
     mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau22 - mu2) - (2*mh2*mstau24*
     mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12*power2(mstau22 - mu2)) + (4*
     mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau22 - mu2) + (4*
     mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(
     mstau22 - mu2)) - (4*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     power2(mstau22 - mu2) - (4*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau22 - mu2) + (DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/(-mstau22 + mu2) + (7*mh2*DeltaInv(mstau22,mh2,mu2)*power2(
     mu2))/power2(mstau12 - mu2) - (7*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2)
     )/power2(mstau12 - mu2) + (3*DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (3*mH2*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) -
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (3*mh2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) + (2*mh2*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (mstau24*
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     - (DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/
     (mstau12*power2(mstau22 - mu2)) + (mu2*DeltaInv(mstau12,mh2,mu2)*(7*mh2*
     mstau12*mu2 + (4*mstau12 + 3*mu2)*power2(mh2) + (mstau12 - mu2)*power2(
     mu2) - 2*power3(mh2)))/(mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (2*
     mu2*DeltaInv(mstau22,mh2,mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2))
     + (DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (2*
     mu2*DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 -
     mu2)) - (2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - (2*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (2*mu2*
     DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) + (
     2*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(mstau22 -
     mu2)) + (mu2*DeltaInv(mH2,mstau12,mu2)*(-7*mH2*mstau12*mu2 - (4*mstau12 +
     3*mu2)*power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 2*power3(mH2)))/(
     mstau12*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mh2,mu2)*power3(mu2))/
     power2(mstau12 - mu2) - (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(
     mstau12 - mu2) + (DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(
     mstau22 - mu2) - (DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(
     mstau22 - mu2) - (DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(mstau22*power2(
     mstau12 - mu2)) + (DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(
     mstau12 - mu2)) - (24*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*
     mstau12) - (DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 -
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 -
     mu2)) - 3*power2(sa)*(-7/mh2 + 7/(mh2 - mH2) + 120/mH2 + 168/mstau12 + (
     336*mstau22)/(mH2*mstau12) - (28*mh2)/(mstau12*mstau22 - mstau12*mu2) + (
     28*mH2)/(mstau12*mstau22 - mstau12*mu2) + 182*mH2*DeltaInv(mstau22,mH2,
     mstau12) + 1162*mstau22*DeltaInv(mstau22,mH2,mstau12) + (28*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (672*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/mH2 + (490*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 +
     (56*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (42*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (56*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/(mstau22 - mu2) + (28*mH2*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(-mstau12 + mu2) + (28*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(-mstau12 + mu2) - (28*mH2*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12*mstau22 - mstau12*mu2) - 14*mh2*DeltaInv(mstau22,
     mstau12,mh2) + 14*mstau22*DeltaInv(mstau22,mstau12,mh2) - (28*mh2*mstau22*
     DeltaInv(mstau22,mstau12,mh2))/mstau12 + (14*mstau24*DeltaInv(mstau22,
     mstau12,mh2))/mstau12 + (28*mh2*mu2*DeltaInv(mstau22,mstau12,mh2))/(
     mstau12 - mu2) + (28*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 -
     mu2) + (56*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (
     42*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (56*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau22 + mu2) + (28*mh2*mstau24*
     DeltaInv(mstau22,mstau12,mh2))/(mstau12*mstau22 - mstau12*mu2) - (7*mH2)/
     power2(mh2) - (7*mstau12)/power2(mh2) + (7*mH2*mstau12)/((-mh2 + mH2)*
     power2(mh2)) - (7*mstau22)/power2(mh2) + (7*mH2*mstau22)/((-mh2 + mH2)*
     power2(mh2)) + (14*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (
     42*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau12 + mu2) + (56*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(-mstau22 + mu2) - (56*mstau22*
     DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/(mstau12*mstau22 - mstau12*mu2)
     + (32*mH2)/(-(mh2*mH2) + power2(mh2)) + (7*mstau12)/(-(mh2*mH2) + power2(
     mh2)) + (7*mstau22)/(-(mh2*mH2) + power2(mh2)) - (32*mH2)/power2(mh2 -
     mH2) - (182*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (42*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (56*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (56*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (21*
     power2(mH2))/((-mh2 + mH2)*power2(mh2)) + (46*power2(mH2))/(mh2*power2(mh2
     - mH2)) - (14*mh2)/power2(mstau12 - mu2) + (14*mH2)/power2(mstau12 - mu2)
     + (28*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (28*mH2*mu2)/(mstau22*
     power2(mstau12 - mu2)) + (14*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) + (56*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))
     /power2(mstau12 - mu2) + (14*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau12 - mu2) - (14*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) - (56*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))
     /power2(mstau12 - mu2) - (14*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/
     power2(mstau12 - mu2) + (56*mu2*DeltaInv(mstau22,mh2,mu2)*power2(mh2))/
     power2(mstau12 - mu2) - (14*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) - (14*mu2*DeltaInv(mstau22,mstau12,mh2)*
     power2(mh2))/power2(mstau12 - mu2) + (14*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/power2(mstau12 - mu2) + (14*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) - (56*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (56*mh2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (56*mH2*mu2)/(mstau12*power2(mstau22 -
     mu2)) - (56*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22
     - mu2) + (28*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*
     power2(mstau22 - mu2)) + (56*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2)
     )/power2(mstau22 - mu2) - (28*mh2*mstau24*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12*power2(mstau22 - mu2)) + (56*mu2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2))/power2(mstau22 - mu2) + (56*mstau22*mu2*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/(mstau12*power2(mstau22 - mu2)) - (56*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) - (
     56*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (14*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22
     - mu2) + (14*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) +
     (98*mh2*DeltaInv(mstau22,mh2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (
     98*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) + (42*
     DeltaInv(mstau22,mh2,mu2)*power2(mh2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) - (42*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (42*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2)
     )/power2(mstau22 - mu2) - (28*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/power2(mstau22 - mu2) - (28*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (14*mstau24*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (42*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2)
     + (28*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22 -
     mu2) + (28*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (14*mstau24*DeltaInv(mstau22,mstau12,mh2)
     *power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (14*DeltaInv(mstau22,
     mstau12,mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     14*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mstau12,mh2,mu2)*(7*mh2*mstau12*mu2 + (
     4*mstau12 + 3*mu2)*power2(mh2) + (mstau12 - mu2)*power2(mu2) - 2*power3(
     mh2)))/(mstau12*power2(mstau22 - mu2)) + (28*DeltaInv(mstau22,mstau12,mh2)
     *power3(mh2))/(mstau12*mstau22 - mstau12*mu2) - (28*mu2*DeltaInv(mstau22,
     mh2,mu2)*power3(mh2))/(mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(
     mstau22,mstau12,mh2)*power3(mh2))/power2(mstau12 - mu2) - (28*mu2*
     DeltaInv(mstau22,mstau12,mh2)*power3(mh2))/(mstau12*power2(mstau22 - mu2))
     - (28*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - (14*power3(mH2))/(power2(mh2)*power2(mh2 - mH2)) - (14*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (28*
     mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2))
     + (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mH2,mstau12,mu2)*(-7*mH2*mstau12*mu2 -
     (4*mstau12 + 3*mu2)*power2(mH2) + (-mstau12 + mu2)*power2(mu2) + 2*power3(
     mH2)))/(mstau12*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mh2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) - (14*DeltaInv(mstau22,mH2,mu2)*power3(
     mu2))/power2(mstau12 - mu2) + (14*DeltaInv(mstau22,mH2,mstau12)*power3(
     mu2))/power2(mstau22 - mu2) - (14*DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/power2(mstau22 - mu2) - (14*DeltaInv(mstau22,mh2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(mstau22,mH2,mu2)*power4(mu2)
     )/(mstau22*power2(mstau12 - mu2)) - (336*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/(mH2*mstau12) - (14*DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) + (14*DeltaInv(mstau22,mstau12,mh2)*power6(
     mstau2))/power2(mstau12 - mu2)) + (36*log(mh2/Q2)*log(mstau22/Q2)*power4(
     sa)*(-mstau22 + DeltaInv(mstau22,mstau12,mh2)*(-3*mh2*mstau12*mstau22 - 2*
     mh2*mstau24 - mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2))))/(
     mh2*mstau12) + (108*log(mstau22/Q2)*power4(sa)*(-((mh2 + mH2)*mstau22) +
     mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*mh2*mstau24
     - 3*mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)) + mh2*
     DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24 - 3*
     mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(mh2*mH2*
     mstau12) - (18*power2(log(mstau22/Q2))*power4(sa)*(-((mh2 + mH2)*mstau22)
     + mH2*DeltaInv(mstau22,mstau12,mh2)*(-(mh2*mstau12*mstau22) - 2*mh2*
     mstau24 - 3*mstau12*mstau24 + mstau22*power2(mh2) + power6(mstau2)) + mh2*
     DeltaInv(mstau22,mH2,mstau12)*(-(mH2*mstau12*mstau22) - 2*mH2*mstau24 - 3*
     mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(mh2*mH2*
     mstau12) + (36*log(mH2/Q2)*log(mstau22/Q2)*power4(sa)*(-mstau22 +
     DeltaInv(mstau22,mH2,mstau12)*(-3*mH2*mstau12*mstau22 - 2*mH2*mstau24 -
     mstau12*mstau24 + mstau22*power2(mH2) + power6(mstau2))))/(mH2*mstau12) -
     (72*log(mstau12/Q2)*log(mstau22/Q2)*power4(sa)*(-((mh2 + mH2)*mstau22) +
     mH2*DeltaInv(mstau22,mstau12,mh2)*(-2*mstau12*mstau24 - 2*mh2*(mstau12*
     mstau22 + mstau24) + mstau22*power2(mh2) + power6(mstau2)) + mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-2*mstau12*mstau24 - 2*mH2*(mstau12*mstau22 +
     mstau24) + mstau22*power2(mH2) + power6(mstau2))))/(mh2*mH2*mstau12) - (
     36*log(mh2/Q2)*log(mstau12/Q2)*power4(sa)*(-mh2 - mstau12 - mstau22 +
     DeltaInv(mstau22,mstau12,mh2)*(-(mstau12*mstau24) - mh2*(6*mstau12*mstau22
     + mstau24) - (mstau12 + mstau22)*power2(mh2) + power3(mh2) + power6(
     mstau2))))/(mh2*mstau12) - (36*log(mH2/Q2)*log(mstau12/Q2)*power4(sa)*(-
     mH2 - mstau12 - mstau22 + DeltaInv(mstau22,mH2,mstau12)*(-(mstau12*
     mstau24) - mH2*(6*mstau12*mstau22 + mstau24) - (mstau12 + mstau22)*power2(
     mH2) + power3(mH2) + power6(mstau2))))/(mH2*mstau12) - (6*power2(Pi)*
     power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 - 2*mh2*mstau22 - 2*
     mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*mh2*mstau12*mstau22 -
     3*mh2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2) + power3(mh2) + 2*
     power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-7*mH2*mstau12*
     mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mH2) +
     power3(mH2) + 2*power6(mstau2))))/(mh2*mH2*mstau12) - (36*power4(sa)*(-14*
     mh2*mH2 - 5*mh2*mstau12 - 5*mH2*mstau12 - 14*mh2*mstau22 - 14*mH2*mstau22
     + 7*mH2*DeltaInv(mstau22,mstau12,mh2)*(-7*mh2*mstau12*mstau22 - 3*mh2*
     mstau24 - 4*mstau12*mstau24 - mstau12*power2(mh2) + power3(mh2) + 2*
     power6(mstau2)) + 7*mh2*DeltaInv(mstau22,mH2,mstau12)*(-7*mH2*mstau12*
     mstau22 - 3*mH2*mstau24 - 4*mstau12*mstau24 - mstau12*power2(mH2) +
     power3(mH2) + 2*power6(mstau2))))/(mh2*mH2*mstau12) + (108*log(mstau12/Q2)
     *power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*mstau12 - 3*mh2*mstau22 - 3*
     mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,mh2)*(-5*mstau12*mstau24 - 5*
     mh2*(2*mstau12*mstau22 + mstau24) + (-mstau12 + mstau22)*power2(mh2) +
     power3(mh2) + 3*power6(mstau2)) + mh2*DeltaInv(mstau22,mH2,mstau12)*(-5*
     mstau12*mstau24 - 5*mH2*(2*mstau12*mstau22 + mstau24) + (-mstau12 +
     mstau22)*power2(mH2) + power3(mH2) + 3*power6(mstau2))))/(mh2*mH2*mstau12)
     - (18*power2(log(mstau12/Q2))*power4(sa)*(-2*mh2*mH2 - mh2*mstau12 - mH2*
     mstau12 - 3*mh2*mstau22 - 3*mH2*mstau22 + mH2*DeltaInv(mstau22,mstau12,
     mh2)*(-5*mstau12*mstau24 - 5*mh2*(2*mstau12*mstau22 + mstau24) + (-mstau12
     + mstau22)*power2(mh2) + power3(mh2) + 3*power6(mstau2)) + mh2*DeltaInv(
     mstau22,mH2,mstau12)*(-5*mstau12*mstau24 - 5*mH2*(2*mstau12*mstau22 +
     mstau24) + (-mstau12 + mstau22)*power2(mH2) + power3(mH2) + 3*power6(
     mstau2))))/(mh2*mH2*mstau12) + 12*log(mC2/Q2)*log(mstau22/Q2)*(-(mstau22/(
     mC2*msntau2)) + mC2/(msntau2*(-mstau22 + mu2)) - (2*mC2*mu2)/(mstau22*
     power2(msntau2 - mu2)) + (2*mC2*mu2*DeltaInv(mstau22,mC2,mu2)*(mu2*(-2*
     mstau22 + mu2) - 2*mC2*(mstau22 + mu2) + power2(mC2)))/(mstau22*power2(
     msntau2 - mu2)) + (mC2*mu2)/(msntau2*power2(mstau22 - mu2)) + DeltaInv(
     mstau22,mC2,msntau2)*(-3*mstau22 - (2*mstau24)/msntau2 - ((msntau2 + 2*
     mstau22)*(mstau22 - 2*mu2)*power2(mC2))/(msntau2*power2(mstau22 - mu2)) +
     mC2*(-1 + mstau24/(msntau2*mstau22 - msntau2*mu2) + 2*mstau22*(1/(msntau2
     - mu2) + 1/(-mstau22 + mu2) - mu2/power2(msntau2 - mu2) + mu2/power2(
     mstau22 - mu2)) + (mstau22 - (mstau24*mu2)/power2(mstau22 - mu2))/msntau2
     + power2(mu2)/power2(mstau22 - mu2)) + ((mstau22 - 2*mu2)*power3(mC2))/(
     msntau2*power2(mstau22 - mu2)) + (-(msntau2*mstau24) + power6(mstau2))/(
     mC2*msntau2))) + 6*log(mH2/Q2)*log(mstau12/Q2)*power2(sa)*(-12/mH2 - 12/
     mstau12 + 4/(mstau12 - mstau22) - (12*mstau22)/(mH2*mstau12) + (4*mu2)/((
     mstau12 - mstau22)*(mstau12 - mu2)) - (2*mH2)/(mstau12*mstau22 - mstau12*
     mu2) - mH2/power2(mstau12 - mu2) + (4*mH2*mu2)/(mstau12*power2(mstau22 -
     mu2)) - (2*mH2*mu2*DeltaInv(mH2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*
     mH2*(mstau12 + mu2) + power2(mH2)))/(mstau12*power2(mstau22 - mu2)) +
     DeltaInv(mstau22,mH2,mstau12)*((-12*(6*mstau12*mstau22 + mstau24))/mstau12
     + power2(mH2)*(3/(-mstau12 + mu2) + 4/(-mstau22 + mu2) - (4*mstau22)/(
     mstau12*mstau22 - mstau12*mu2) - (2*mstau22)/power2(mstau12 - mu2) - mu2/
     power2(mstau12 - mu2) + (12 + (4*mstau22*mu2)/power2(mstau22 - mu2))/
     mstau12 + (4*mu2)/power2(mstau22 - mu2)) + mH2*(-10 + (2*mu2)/(mstau12 -
     mu2) + mstau22/(-mstau12 + mu2) + (4*mstau22)/(-mstau22 + mu2) + (2*
     mstau24)/(mstau12*mstau22 - mstau12*mu2) + mstau24/power2(mstau12 - mu2) -
     (3*mstau22*mu2)/power2(mstau12 - mu2) + (-12*mstau22 - (2*mstau24*mu2)/
     power2(mstau22 - mu2))/mstau12 + (4*mstau22*mu2)/power2(mstau22 - mu2)) +
     (2/(mstau12*mstau22 - mstau12*mu2) + 1/power2(mstau12 - mu2) - (2*mu2)/(
     mstau12*power2(mstau22 - mu2)))*power3(mH2) + (12*(-(mstau12*mstau24) +
     power6(mstau2)))/(mH2*mstau12))) - 6*log(mH2/Q2)*log(mstau22/Q2)*power2(
     sa)*(4/(mstau12 - mstau22) - (12*mstau22)/(mH2*mstau12) + (4*mu2)/((
     mstau12 - mstau22)*(mstau22 - mu2)) + mH2/(mstau12*(-mstau22 + mu2)) - (2*
     mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mH2*mu2*DeltaInv(mstau22,
     mH2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mH2*(mstau22 + mu2) + power2(mH2)))/(
     mstau22*power2(mstau12 - mu2)) + (mH2*mu2)/(mstau12*power2(mstau22 - mu2))
     + DeltaInv(mstau22,mH2,mstau12)*(-36*mstau22 - (24*mstau24)/mstau12 - ((
     mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + mH2*(-1 + mstau24/(mstau12*mstau22 - mstau12*mu2) + 2*
     mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2) - mu2/power2(mstau12 -
     mu2) + mu2/power2(mstau22 - mu2)) + (12*mstau22 - (mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 - mu2)) + ((mstau22 -
     2*mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (12*(-(mstau12*
     mstau24) + power6(mstau2)))/(mH2*mstau12))) + 12*log(mC2/Q2)*log(msntau2/
     Q2)*(1/mC2 + 1/msntau2 + mstau22/(mC2*msntau2) + (2*mC2)/(msntau2*mstau22
     - msntau2*mu2) + mC2/power2(msntau2 - mu2) - (4*mC2*mu2)/(msntau2*power2(
     mstau22 - mu2)) + (2*mC2*mu2*DeltaInv(mC2,msntau2,mu2)*(mu2*(-2*msntau2 +
     mu2) - 2*mC2*(msntau2 + mu2) + power2(mC2)))/(msntau2*power2(mstau22 -
     mu2)) + DeltaInv(mstau22,mC2,msntau2)*(6*mstau22 + mstau24/msntau2 + mC2*(
     -1 + mstau22/(msntau2 - mu2) + (4*mstau22)/(mstau22 - mu2) + (2*mu2)/(-
     msntau2 + mu2) - (2*mstau24)/(msntau2*mstau22 - msntau2*mu2) - mstau24/
     power2(msntau2 - mu2) + (3*mstau22*mu2)/power2(msntau2 - mu2) + (mstau22 +
     (2*mstau24*mu2)/power2(mstau22 - mu2))/msntau2 - (4*mstau22*mu2)/power2(
     mstau22 - mu2)) + power2(mC2)*(3/(msntau2 - mu2) + 4/(mstau22 - mu2) + (4*
     mstau22)/(msntau2*mstau22 - msntau2*mu2) + (2*mstau22)/power2(msntau2 -
     mu2) + mu2/power2(msntau2 - mu2) - (4*mu2)/power2(mstau22 - mu2) - power2(
     mstau22 + mu2)/(msntau2*power2(mstau22 - mu2))) + (-2/(msntau2*mstau22 -
     msntau2*mu2) - 1/power2(msntau2 - mu2) + (2*mu2)/(msntau2*power2(mstau22 -
     mu2)))*power3(mC2) + (mstau24 - power6(mstau2)/msntau2)/mC2)) - 3*power2(
     sa)*power2(log(mstau22/Q2))*((12*mstau22)/(mH2*mstau12) - 3*mh2*DeltaInv(
     mstau22,mstau12,mh2) + mstau22*DeltaInv(mstau22,mstau12,mh2) - (2*mh2*
     mstau22*DeltaInv(mstau22,mstau12,mh2))/mstau12 + (mstau24*DeltaInv(
     mstau22,mstau12,mh2))/mstau12 + (2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/(mstau12 - mu2) + (3*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-
     mstau12 + mu2) + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 +
     mu2) + (DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (3*mh2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (3*mH2*mu2)/(mstau22*power2(mstau12 -
     mu2)) - (2*mh2*mstau24*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) - (mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 -
     mu2) - (mstau24*mu2*DeltaInv(mstau22,mstau12,mh2))/power2(mstau12 - mu2) +
     (mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mh2))/power2(mstau12 - mu2)
     - (5*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (
     DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(-mstau22 + mu2) - (10*mH2*
     DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (5*
     DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12
     - mu2)) + (3*mh2*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(mstau22
     - mu2) + (2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/power2(
     mstau22 - mu2) + (2*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))
     /(mstau12*power2(mstau22 - mu2)) - (mstau24*DeltaInv(mstau22,mstau12,mh2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (DeltaInv(mstau22,mstau12,
     mh2)*power2(mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mstau22,mh2,mu2)*(mh2*(10*mstau22 - mu2)*mu2 + 5*(mstau22 + mu2)*
     power2(mh2) + (mstau22 - mu2)*power2(mu2) - 3*power3(mh2)))/(mstau22*
     power2(mstau12 - mu2)) + (3*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(
     mstau22*power2(mstau12 - mu2)) - (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/
     power2(mstau12 - mu2) + (mH2*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(
     mstau22*power2(mstau12 - mu2)) - (DeltaInv(mstau22,mstau12,mh2)*power3(
     mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(
     mstau22*power2(mstau12 - mu2)) + (DeltaInv(mstau22,mstau12,mh2)*power6(
     mstau2))/power2(mstau12 - mu2) + DeltaInv(mstau22,mH2,mstau12)*((23*
     mstau24)/mstau12 + (3*mstau24)/(mstau12 - mu2) + (mstau24*mu2)/power2(
     mstau12 - mu2) + power2(mu2)/(mstau22 - mu2) + (mstau24*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + mstau22*(11 + (2*mu2)/(-mstau12 + mu2) -
     (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3 + (3*mstau22)/(mstau12 -
     mu2) + (2*mstau24)/power2(mstau12 - mu2) + (mstau22*mu2)/power2(mstau12 -
     mu2) - (3*power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*(-5 - power2(
     mu2)/power2(mstau22 - mu2)))/mstau12) + power2(mH2)*(-(mstau22/power2(
     mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22 - mu2))/mstau12) +
     power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/power2(mstau12 - mu2) +
     (36*mstau24 - (12*power6(mstau2))/mstau12)/mH2)) + 6*log(mstau22/Q2)*
     power2(sa)*((36*mstau22)/(mH2*mstau12) - 9*mh2*DeltaInv(mstau22,mstau12,
     mh2) + 3*mstau22*DeltaInv(mstau22,mstau12,mh2) - (6*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2))/mstau12 + (3*mstau24*DeltaInv(mstau22,mstau12,mh2))/
     mstau12 + (6*mstau22*mu2*DeltaInv(mstau22,mstau12,mh2))/(mstau12 - mu2) +
     (9*mh2*mstau22*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) + (9*
     mstau24*DeltaInv(mstau22,mstau12,mh2))/(-mstau12 + mu2) - (2*mstau22)/
     power2(mh2) + (2*mH2*mstau22)/((-mh2 + mH2)*power2(mh2)) + (3*DeltaInv(
     mstau22,mstau12,mh2)*power2(mh2))/mstau12 + (2*mstau22)/(-(mh2*mH2) +
     power2(mh2)) + (9*mh2*mu2)/(mstau22*power2(mstau12 - mu2)) - (9*mH2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (6*mh2*mstau24*DeltaInv(mstau22,mstau12,
     mh2))/power2(mstau12 - mu2) - (3*mh2*mstau22*mu2*DeltaInv(mstau22,mstau12,
     mh2))/power2(mstau12 - mu2) - (3*mstau24*mu2*DeltaInv(mstau22,mstau12,mh2)
     )/power2(mstau12 - mu2) + (3*mstau22*DeltaInv(mstau22,mstau12,mh2)*power2(
     mh2))/power2(mstau12 - mu2) - (15*mu2*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2))/power2(mstau12 - mu2) + (3*DeltaInv(mstau22,mstau12,mh2)*power2(mu2)
     )/(-mstau22 + mu2) - (30*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/
     power2(mstau12 - mu2) - (15*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (9*mh2*DeltaInv(mstau22,mstau12,
     mh2)*power2(mu2))/power2(mstau22 - mu2) + (6*mstau22*DeltaInv(mstau22,
     mstau12,mh2)*power2(mu2))/power2(mstau22 - mu2) + (6*mh2*mstau22*DeltaInv(
     mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*
     mstau24*DeltaInv(mstau22,mstau12,mh2)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) - (3*DeltaInv(mstau22,mstau12,mh2)*power2(mh2)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mstau22,mh2,mu2)*(mh2*
     mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)*power2(mh2) + (-mstau22 + mu2)
     *power2(mu2) + 3*power3(mh2)))/(mstau22*power2(mstau12 - mu2)) + (9*mu2*
     DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(mstau12 - mu2)) - (
     3*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (3*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     3*DeltaInv(mstau22,mstau12,mh2)*power3(mu2))/power2(mstau22 - mu2) + (3*
     DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     3*DeltaInv(mstau22,mstau12,mh2)*power6(mstau2))/power2(mstau12 - mu2) + 3*
     DeltaInv(mstau22,mH2,mstau12)*((23*mstau24)/mstau12 + (3*mstau24)/(mstau12
     - mu2) + (mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(mstau22 - mu2)
     + (mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + mstau22*(11 + (
     2*mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 - mu2)) + mH2*(3
     + (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12 - mu2) + (
     mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(mstau22 - mu2)
     + (2*mstau22*(-5 - power2(mu2)/power2(mstau22 - mu2)))/mstau12) + power2(
     mH2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(mu2)/power2(mstau22
     - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) - power6(mstau2)/
     power2(mstau12 - mu2) + (36*mstau24 - (12*power6(mstau2))/mstau12)/mH2)) +
     6*log(mH2/Q2)*log(mstau12/Q2)*(6/mH2 + 6/mstau12 - 4/(mstau12 - mstau22) +
     (6*mstau22)/(mH2*mstau12) + (4*mu2)/((mstau12 - mstau22)*(-mstau12 + mu2))
     + (2*mH2)/(mstau12*mstau22 - mstau12*mu2) + mH2/power2(mstau12 - mu2) - (
     4*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) + (2*mH2*mu2*DeltaInv(mH2,
     mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mH2*(mstau12 + mu2) + power2(mH2)
     ))/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*((6*(6*
     mstau12*mstau22 + mstau24))/mstau12 + mH2*(4 + mstau22/(mstau12 - mu2) + (
     4*mstau22)/(mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - (2*mstau24)/(
     mstau12*mstau22 - mstau12*mu2) - mstau24/power2(mstau12 - mu2) + (3*
     mstau22*mu2)/power2(mstau12 - mu2) + (6*mstau22 + (2*mstau24*mu2)/power2(
     mstau22 - mu2))/mstau12 - (4*mstau22*mu2)/power2(mstau22 - mu2)) + (-2/(
     mstau12*mstau22 - mstau12*mu2) - 1/power2(mstau12 - mu2) + (2*mu2)/(
     mstau12*power2(mstau22 - mu2)))*power3(mH2) + (power2(mH2)*(-2*power2(mu2)
     *(-2*mstau22*mu2 + power2(mstau22) + 3*power2(mu2)) + power2(mstau12)*(-
     10*mstau22*mu2 + power2(mstau22) + 13*power2(mu2)) + 4*(mstau22 - 2*mu2)*
     power3(mstau12) + 2*mstau12*(-(mu2*power2(mstau22)) + mstau22*power2(mu2)
     + power3(mstau22) + power3(mu2))))/(mstau12*power2(mstau12 - mu2)*power2(
     mstau22 - mu2)) + (6*mstau24 - (6*power6(mstau2))/mstau12)/mH2)) + 6*log(
     mA2/Q2)*log(mstau22/Q2)*(-4/(mstau12 - mstau22) + (2*mstau22)/(mA2*
     mstau12) + mA2/(mstau12*(-mstau22 + mu2)) + (4*mu2)/((mstau12 - mstau22)*(
     -mstau22 + mu2)) - (2*mA2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mA2*
     mu2*DeltaInv(mA2,mstau22,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mA2*(mstau22 +
     mu2) + power2(mA2)))/(mstau22*power2(mstau12 - mu2)) + (mA2*mu2)/(mstau12*
     power2(mstau22 - mu2)) + DeltaInv(mA2,mstau22,mstau12)*(6*mstau22 + (4*
     mstau24)/mstau12 - ((mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*power2(mA2))/(
     mstau12*power2(mstau22 - mu2)) + ((mstau22 - 2*mu2)*power3(mA2))/(mstau12*
     power2(mstau22 - mu2)) + (mA2*(-(mstau12*(mstau22 - 2*mu2)*mu2*(2*mstau24
     + 3*mstau22*mu2)) + (mstau22 - 2*mu2)*(mstau24 + 6*mstau22*mu2)*power2(
     mstau12) - 3*mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(mu2)*(-2*
     mstau24*mu2 + 4*mu2*power2(mstau22) + mstau22*(mstau24 - 2*power2(mu2)) -
     2*power3(mstau22))))/(mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))
     + (2*mstau24 - (2*power6(mstau2))/mstau12)/mA2)) + 6*log(mstau22/Q2)*(-6/
     msntau2 - 6/mstau12 - 4/mstau22 - (6*mstau22)/(mC2*msntau2) + (6*mstau22)/
     (mA2*mstau12) - (18*mstau22)/(mH2*mstau12) + 4/(-mstau22 + mu2) - 18*mC2*
     DeltaInv(mstau22,mC2,msntau2) - (6*mC2*mstau22*DeltaInv(mstau22,mC2,
     msntau2))/msntau2 - (18*mstau24*DeltaInv(mstau22,mC2,msntau2))/mC2 - (6*
     mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (12*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2))/(msntau2 - mu2) + (18*mC2*mstau22*DeltaInv(mstau22,
     mC2,msntau2))/(-msntau2 + mu2) + (18*mstau24*DeltaInv(mstau22,mC2,msntau2)
     )/(-msntau2 + mu2) - 9*mH2*DeltaInv(mstau22,mH2,mstau12) - 15*mstau22*
     DeltaInv(mstau22,mH2,mstau12) + (12*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/mstau12 - (54*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 - (33*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (6*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (9*mH2*mstau22*DeltaInv(mstau22,
     mH2,mstau12))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) + (6*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/msntau2
     + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 - (6*mstau22)/
     power2(msntau2 - mu2) + (6*mu2)/power2(msntau2 - mu2) + (18*mC2*mu2)/(
     mstau22*power2(msntau2 - mu2)) - (12*mC2*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/power2(msntau2 - mu2) - (6*mC2*mstau22*mu2*DeltaInv(mstau22,mC2,
     msntau2))/power2(msntau2 - mu2) - (6*mstau24*mu2*DeltaInv(mstau22,mC2,
     msntau2))/power2(msntau2 - mu2) + (6*mstau22*DeltaInv(mstau22,mC2,msntau2)
     *power2(mC2))/power2(msntau2 - mu2) + (30*mu2*DeltaInv(mstau22,mC2,mu2)*
     power2(mC2))/power2(msntau2 - mu2) - (6*mstau22)/power2(mstau12 - mu2) + (
     6*mu2)/power2(mstau12 - mu2) + (9*mA2*mu2)/(mstau22*power2(mstau12 - mu2))
     + (9*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) - (6*mH2*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*mH2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*mstau24*mu2*DeltaInv(
     mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (3*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) + (15*mu2*DeltaInv(
     mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) + (4*mu2)/power2(
     mstau22 - mu2) + (6*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(-mstau22 +
     mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22 + mu2) + (
     6*power2(mu2))/(mstau22*power2(msntau2 - mu2)) + (60*mC2*DeltaInv(mstau22,
     mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (30*DeltaInv(mstau22,mC2,
     mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2)) + (6*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (30*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) + (15*DeltaInv(mstau22,mH2,mu2)*power2(
     mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) + (6*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (6*power2(mu2))/(mstau12*power2(mstau22 -
     mu2)) + (4*power2(mu2))/(mstau22*power2(mstau22 - mu2)) + (18*mC2*
     DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) + (12*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) +
     (12*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(msntau2*
     power2(mstau22 - mu2)) - (6*mstau24*DeltaInv(mstau22,mC2,msntau2)*power2(
     mu2))/(msntau2*power2(mstau22 - mu2)) + (9*mH2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mstau22*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) - (6*DeltaInv(mstau22,mC2,msntau2)*power2(mC2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mA2,
     mstau22,mu2)*(mA2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)*power2(mA2)
     + (-mstau22 + mu2)*power2(mu2) + 3*power3(mA2)))/(mstau22*power2(mstau12 -
     mu2)) - (18*mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(
     msntau2 - mu2)) - (9*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*
     power2(mstau12 - mu2)) + (8*power2(mu2))/power3(mstau22 - mu2) + (6*
     DeltaInv(mstau22,mC2,mu2)*power3(mu2))/power2(msntau2 - mu2) - (6*mC2*
     DeltaInv(mstau22,mC2,mu2)*power3(mu2))/(mstau22*power2(msntau2 - mu2)) + (
     3*DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(mstau12 - mu2) - (3*mH2*
     DeltaInv(mstau22,mH2,mu2)*power3(mu2))/(mstau22*power2(mstau12 - mu2)) - (
     6*DeltaInv(mstau22,mC2,msntau2)*power3(mu2))/power2(mstau22 - mu2) - (3*
     DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) - (6*
     DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(mstau22*power2(msntau2 - mu2)) - (
     3*DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) +
     (6*DeltaInv(mstau22,mC2,msntau2)*power6(mstau2))/(mC2*msntau2) + (18*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (6*DeltaInv(
     mstau22,mC2,msntau2)*power6(mstau2))/power2(msntau2 - mu2) + (3*DeltaInv(
     mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + 3*DeltaInv(
     mA2,mstau22,mstau12)*((5*mstau24)/mstau12 + (3*mstau24)/(-mstau12 + mu2) -
     (mstau24*mu2)/power2(mstau12 - mu2) + mstau22*power2(mA2)*(1/power2(
     mstau12 - mu2) + (mstau22 - 2*mu2)/(mstau12*power2(mstau22 - mu2))) +
     power2(mu2)/(-mstau22 + mu2) - (mstau24*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + mstau22*(3 + (2*mu2)/(mstau12 - mu2) + (2*power2(mu2))/
     power2(mstau22 - mu2)) + mA2*(-3 + (3*mstau22)/(-mstau12 + mu2) - (2*
     mstau24)/power2(mstau12 - mu2) - (mstau22*mu2)/power2(mstau12 - mu2) + (3*
     power2(mu2))/power2(mstau22 - mu2) + (2*mstau22*(-2 + power2(mu2)/power2(
     mstau22 - mu2)))/mstau12) - power3(mu2)/power2(mstau22 - mu2) + power6(
     mstau2)/power2(mstau12 - mu2) + (6*mstau24 - (2*power6(mstau2))/mstau12)/
     mA2)) + 3*power2(log(mstau22/Q2))*(2/msntau2 + 2/mstau12 + (2*mstau22)/(
     mC2*msntau2) - (2*mstau22)/(mA2*mstau12) + (6*mstau22)/(mH2*mstau12) + 4/(
     mstau22 - mu2) + 6*mC2*DeltaInv(mstau22,mC2,msntau2) + (2*mC2*mstau22*
     DeltaInv(mstau22,mC2,msntau2))/msntau2 + (6*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/mC2 + (2*mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (6*
     mC2*mstau22*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (6*mstau24*
     DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (4*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2))/(-msntau2 + mu2) + 3*mH2*DeltaInv(mstau22,mH2,
     mstau12) + 5*mstau22*DeltaInv(mstau22,mH2,mstau12) - (4*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (18*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/mH2 + (11*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (3*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (2*mstau22*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(-mstau12 + mu2) - (2*DeltaInv(mstau22,mC2,msntau2)*
     power2(mC2))/msntau2 - (DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12
     + (2*mstau22)/power2(msntau2 - mu2) - (2*mu2)/power2(msntau2 - mu2) - (6*
     mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (4*mC2*mstau24*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*mC2*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*mstau24*mu2*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) - (2*mstau22*DeltaInv(mstau22,
     mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) - (10*mu2*DeltaInv(
     mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (2*mstau22)/power2(
     mstau12 - mu2) - (2*mu2)/power2(mstau12 - mu2) - (3*mA2*mu2)/(mstau22*
     power2(mstau12 - mu2)) - (3*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*
     mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (mH2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) + (
     mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (5*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (4*
     mu2)/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))
     /(mstau22 - mu2) + (DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau22 -
     mu2) - (2*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (20*mC2*DeltaInv(
     mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (10*DeltaInv(
     mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2 - mu2))
     - (2*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (10*mH2*DeltaInv(
     mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (5*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (2*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (2*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (6*mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))
     /power2(mstau22 - mu2) - (4*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(
     mu2))/power2(mstau22 - mu2) - (4*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)
     *power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (2*mstau24*DeltaInv(
     mstau22,mC2,msntau2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (3*
     mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) -
     (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(
     mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mu2*
     DeltaInv(mA2,mstau22,mu2)*(mA2*mu2*(-10*mstau22 + mu2) - 5*(mstau22 + mu2)
     *power2(mA2) + (-mstau22 + mu2)*power2(mu2) + 3*power3(mA2)))/(mstau22*
     power2(mstau12 - mu2)) + (6*mu2*DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(
     mstau22*power2(msntau2 - mu2)) + (3*mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,mC2,mu2)*
     power3(mu2))/power2(msntau2 - mu2) + (2*mC2*DeltaInv(mstau22,mC2,mu2)*
     power3(mu2))/(mstau22*power2(msntau2 - mu2)) - (DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/power2(mstau12 - mu2) + (mH2*DeltaInv(mstau22,mH2,mu2)*
     power3(mu2))/(mstau22*power2(mstau12 - mu2)) + (2*DeltaInv(mstau22,mC2,
     msntau2)*power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power3(mu2))/power2(mstau22 - mu2) + (8*power2(mu2))/power3(-
     mstau22 + mu2) + (2*DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(mstau22*
     power2(msntau2 - mu2)) + (DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*
     power2(mstau12 - mu2)) - (2*DeltaInv(mstau22,mC2,msntau2)*power6(mstau2))/
     (mC2*msntau2) - (6*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*
     mstau12) - (2*DeltaInv(mstau22,mC2,msntau2)*power6(mstau2))/power2(msntau2
     - mu2) - (DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 -
     mu2) + DeltaInv(mA2,mstau22,mstau12)*((-5*mstau24)/mstau12 + (3*mstau24)/(
     mstau12 - mu2) + (mstau24*mu2)/power2(mstau12 - mu2) + power2(mu2)/(
     mstau22 - mu2) + (mstau24*power2(mu2))/(mstau12*power2(mstau22 - mu2)) +
     mstau22*(-3 + (2*mu2)/(-mstau12 + mu2) - (2*power2(mu2))/power2(mstau22 -
     mu2)) + mA2*(3 + (3*mstau22)/(mstau12 - mu2) + (2*mstau24)/power2(mstau12
     - mu2) + (mstau22*mu2)/power2(mstau12 - mu2) - (3*power2(mu2))/power2(
     mstau22 - mu2) + (mstau22*(4 - (2*power2(mu2))/power2(mstau22 - mu2)))/
     mstau12) + power2(mA2)*(-(mstau22/power2(mstau12 - mu2)) + (-1 + power2(
     mu2)/power2(mstau22 - mu2))/mstau12) + power3(mu2)/power2(mstau22 - mu2) -
     power6(mstau2)/power2(mstau12 - mu2) + (-6*mstau24 + (2*power6(mstau2))/
     mstau12)/mA2)) + 6*log(mA2/Q2)*log(mstau12/Q2)*(-2/mA2 - 2/mstau12 + 4/(
     mstau12 - mstau22) - (2*mstau22)/(mA2*mstau12) + (4*mu2)/((mstau12 -
     mstau22)*(mstau12 - mu2)) + (2*mA2)/(mstau12*mstau22 - mstau12*mu2) + mA2/
     power2(mstau12 - mu2) - (4*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) + (2*
     mA2*mu2*DeltaInv(mA2,mstau12,mu2)*(mu2*(-2*mstau12 + mu2) - 2*mA2*(mstau12
     + mu2) + power2(mA2)))/(mstau12*power2(mstau22 - mu2)) + DeltaInv(mA2,
     mstau22,mstau12)*((-2*(6*mstau12*mstau22 + mstau24))/mstau12 + power2(mA2)
     *(3/(mstau12 - mu2) + 4/(mstau22 - mu2) + (4*mstau22)/(mstau12*mstau22 -
     mstau12*mu2) + (2*mstau22)/power2(mstau12 - mu2) + mu2/power2(mstau12 -
     mu2) + (2 - (4*mstau22*mu2)/power2(mstau22 - mu2))/mstau12 - (4*mu2)/
     power2(mstau22 - mu2)) + mA2*(-4 + mstau22/(mstau12 - mu2) + (4*mstau22)/(
     mstau22 - mu2) + (2*mu2)/(-mstau12 + mu2) - (2*mstau24)/(mstau12*mstau22 -
     mstau12*mu2) - mstau24/power2(mstau12 - mu2) + (3*mstau22*mu2)/power2(
     mstau12 - mu2) + (-2*mstau22 + (2*mstau24*mu2)/power2(mstau22 - mu2))/
     mstau12 - (4*mstau22*mu2)/power2(mstau22 - mu2)) + (-2/(mstau12*mstau22 -
     mstau12*mu2) - 1/power2(mstau12 - mu2) + (2*mu2)/(mstau12*power2(mstau22 -
     mu2)))*power3(mA2) + (-2*mstau24 + (2*power6(mstau2))/mstau12)/mA2)) + 6*
     log(mH2/Q2)*log(mstau22/Q2)*(4/(mstau12 - mstau22) - (6*mstau22)/(mH2*
     mstau12) + (4*mu2)/((mstau12 - mstau22)*(mstau22 - mu2)) + mH2/(mstau12*(-
     mstau22 + mu2)) - (2*mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (2*mH2*
     mu2*DeltaInv(mstau22,mH2,mu2)*(mu2*(-2*mstau22 + mu2) - 2*mH2*(mstau22 +
     mu2) + power2(mH2)))/(mstau22*power2(mstau12 - mu2)) + (mH2*mu2)/(mstau12*
     power2(mstau22 - mu2)) + DeltaInv(mstau22,mH2,mstau12)*((-6*(3*mstau12*
     mstau22 + 2*mstau24))/mstau12 - ((mstau12 + 2*mstau22)*(mstau22 - 2*mu2)*
     power2(mH2))/(mstau12*power2(mstau22 - mu2)) + mH2*(-1 + mstau24/(mstau12*
     mstau22 - mstau12*mu2) + 2*mstau22*(1/(mstau12 - mu2) + 1/(-mstau22 + mu2)
     - mu2/power2(mstau12 - mu2) + mu2/power2(mstau22 - mu2)) + (6*mstau22 - (
     mstau24*mu2)/power2(mstau22 - mu2))/mstau12 + power2(mu2)/power2(mstau22 -
     mu2)) + ((mstau22 - 2*mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) +
     (-6*mstau24 + (6*power6(mstau2))/mstau12)/mH2)) + 6*power2(log(msntau2/Q2)
     )*(1/mC2 + 2/msntau2 + (3*mstau22)/(mC2*msntau2) + 1/(mstau22 - mu2) + (3*
     mC2)/(msntau2*mstau22 - msntau2*mu2) + mC2/power2(msntau2 - mu2) +
     mstau22/power2(msntau2 - mu2) - (2*mu2)/power2(mstau22 - mu2) - (6*mC2*
     mu2)/(msntau2*power2(mstau22 - mu2)) - (2*power2(mu2))/(msntau2*power2(
     mstau22 - mu2)) + (mu2*DeltaInv(mC2,msntau2,mu2)*(mC2*mu2*(-10*msntau2 +
     mu2) - 5*(msntau2 + mu2)*power2(mC2) + (-msntau2 + mu2)*power2(mu2) + 3*
     power3(mC2)))/(msntau2*power2(mstau22 - mu2)) + (2*power2(mu2))/power3(-
     msntau2 + mu2) + (DeltaInv(mstau22,mC2,msntau2)*(power3(mC2)*(-(power2(
     mu2)*(8*mstau22*mu2 - 4*power2(mstau22) + power2(mu2))) + power2(msntau2)*
     (-24*mstau22*mu2 + 7*power2(mstau22) + 22*power2(mu2)) + 5*(mstau22 - 2*
     mu2)*power3(msntau2) + msntau2*(-12*mu2*power2(mstau22) + 26*mstau22*
     power2(mu2) + power3(mstau22) - 10*power3(mu2))) - (3*(mstau22 - 2*mu2)*
     power2(msntau2) + 3*(mstau22 - 2*mu2)*power2(mu2) + msntau2*(-8*mstau22*
     mu2 + power2(mstau22) + 13*power2(mu2)))*power4(mC2) + power2(mC2)*((-18*
     mstau22*mu2 + 9*power2(mstau22) - power2(mu2))*power3(msntau2) + power2(
     mu2)*(6*mstau24*mu2 - 2*mu2*power2(mstau22) - mstau22*(3*mstau24 + power2(
     mu2)) + power3(mstau22)) + power2(msntau2)*(-3*mstau22*mstau24 + 6*
     mstau24*mu2 - 26*mu2*power2(mstau22) + 41*mstau22*power2(mu2) + 3*power3(
     mstau22)) + msntau2*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*
     power2(mu2) + power2(mstau22)*(mstau24 + 7*power2(mu2)) + 2*mu2*power3(
     mstau22) + power4(mu2))) + power2(msntau2 - mu2)*power2(mstau22 - mu2)*(5*
     msntau2*mstau24 - 3*power6(mstau2)) + mC2*(mstau24*power2(mu2)*(-8*
     mstau22*mu2 + 4*power2(mstau22) + 5*power2(mu2)) + mstau22*(-18*mstau22*
     mu2 + 9*power2(mstau22) + 8*power2(mu2))*power3(msntau2) + power2(msntau2)
     *(8*mstau24*power2(mu2) + power2(mstau22)*(7*mstau24 + 40*power2(mu2)) -
     20*mu2*power3(mstau22) - 2*mstau22*(7*mstau24*mu2 + 9*power3(mu2))) -
     msntau2*(mu2*(12*mstau24*power2(mu2) + 2*power2(mstau22)*(5*mstau24 + 11*
     power2(mu2)) - 11*mu2*power3(mstau22) - 10*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mC2*msntau2*
     power2(msntau2 - mu2)*power2(mstau22 - mu2))) + 12*log(msntau2/Q2)*(-3/mC2
     - 7/msntau2 - (9*mstau22)/(mC2*msntau2) + 2/(msntau2 - mu2) + 3/(-mstau22
     + mu2) - (9*mC2)/(msntau2*mstau22 - msntau2*mu2) - (3*mC2)/power2(msntau2
     - mu2) - (3*mstau22)/power2(msntau2 - mu2) - (2*mu2)/power2(msntau2 - mu2)
     + (6*mu2)/power2(mstau22 - mu2) + (18*mC2*mu2)/(msntau2*power2(mstau22 -
     mu2)) + power2(mu2)/(msntau2*power2(msntau2 - mu2)) + (6*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mC2,msntau2,mu2)*(mC2*
     mu2*(-10*msntau2 + mu2) - 5*(msntau2 + mu2)*power2(mC2) + (-msntau2 + mu2)
     *power2(mu2) + 3*power3(mC2)))/(msntau2*power2(mstau22 - mu2)) + (2*
     power2(mu2))/power3(msntau2 - mu2) + (3*DeltaInv(mstau22,mC2,msntau2)*(
     power3(mC2)*(power2(msntau2)*(24*mstau22*mu2 - 7*power2(mstau22) - 22*
     power2(mu2)) + power2(mu2)*(8*mstau22*mu2 - 4*power2(mstau22) + power2(
     mu2)) - 5*(mstau22 - 2*mu2)*power3(msntau2) - msntau2*(-12*mu2*power2(
     mstau22) + 26*mstau22*power2(mu2) + power3(mstau22) - 10*power3(mu2))) + (
     3*(mstau22 - 2*mu2)*power2(msntau2) + 3*(mstau22 - 2*mu2)*power2(mu2) +
     msntau2*(-8*mstau22*mu2 + power2(mstau22) + 13*power2(mu2)))*power4(mC2) -
     power2(mC2)*((-18*mstau22*mu2 + 9*power2(mstau22) - power2(mu2))*power3(
     msntau2) + power2(mu2)*(6*mstau24*mu2 - 2*mu2*power2(mstau22) - mstau22*(
     3*mstau24 + power2(mu2)) + power3(mstau22)) + power2(msntau2)*(-3*mstau22*
     mstau24 + 6*mstau24*mu2 - 26*mu2*power2(mstau22) + 41*mstau22*power2(mu2)
     + 3*power3(mstau22)) + msntau2*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) -
     11*mstau24*power2(mu2) + power2(mstau22)*(mstau24 + 7*power2(mu2)) + 2*
     mu2*power3(mstau22) + power4(mu2))) + power2(msntau2 - mu2)*power2(mstau22
     - mu2)*(-5*msntau2*mstau24 + 3*power6(mstau2)) + mC2*(mstau24*(8*mstau22*
     mu2 - 4*power2(mstau22) - 5*power2(mu2))*power2(mu2) + mstau22*(18*
     mstau22*mu2 - 9*power2(mstau22) - 8*power2(mu2))*power3(msntau2) + power2(
     msntau2)*(-8*mstau24*power2(mu2) - power2(mstau22)*(7*mstau24 + 40*power2(
     mu2)) + 20*mu2*power3(mstau22) + 2*mstau22*(7*mstau24*mu2 + 9*power3(mu2))
     ) + msntau2*(mu2*(12*mstau24*power2(mu2) + 2*power2(mstau22)*(5*mstau24 +
     11*power2(mu2)) - 11*mu2*power3(mstau22) - 10*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mC2*msntau2*
     power2(msntau2 - mu2)*power2(mstau22 - mu2))) + 3*power2(log(mstau12/Q2))*
     (-2/mA2 + 6/mH2 + 6/mstau12 - (6*mstau22)/(mA2*mstau12) + (18*mstau22)/(
     mH2*mstau12) + 2/(mstau22 - mu2) + (3*mA2)/(mstau12*mstau22 - mstau12*mu2)
     + (3*mH2)/(mstau12*mstau22 - mstau12*mu2) + 8*mH2*DeltaInv(mstau22,mH2,
     mstau12) + 59*mstau22*DeltaInv(mstau22,mH2,mstau12) - (4*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12))/mstau12 + (30*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/mH2 + (29*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (2*
     mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (3*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(mstau22 - mu2) + (2*mH2*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(-mstau12 + mu2) + (2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))
     /(-mstau12 + mu2) - (3*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12*mstau22 - mstau12*mu2) - (7*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 -
     mu2) + (5*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (6*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + mA2/power2(mstau12 - mu2) + mH2/power2(mstau12 - mu2) + (2*
     mstau22)/power2(mstau12 - mu2) + (mH2*mstau24*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) + (6*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) + (mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) + (mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (4*mu2)/power2(mstau22 - mu2) - (6*
     mA2*mu2)/(mstau12*power2(mstau22 - mu2)) - (6*mH2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (6*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/
     power2(mstau22 - mu2) + (3*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/
     (mstau12*power2(mstau22 - mu2)) - (5*mu2*DeltaInv(mH2,mstau12,mu2)*power2(
     mH2))/power2(mstau22 - mu2) - (5*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(
     mH2))/power2(mstau22 - mu2) - (6*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)
     *power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau22 - mu2) - (4*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (10*mH2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(
     mstau22 - mu2) - (4*mH2*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/power2(
     mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/
     power2(mstau22 - mu2) - (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (mstau24*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (5*DeltaInv(
     mH2,mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (mu2*DeltaInv(mA2,mstau12,mu2)*(mA2*mu2*(-10*mstau12 +
     mu2) - 5*(mstau12 + mu2)*power2(mA2) + (-mstau12 + mu2)*power2(mu2) + 3*
     power3(mA2)))/(mstau12*power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,
     mstau12)*power3(mH2))/(mstau12*mstau22 - mstau12*mu2) - (DeltaInv(mstau22,
     mH2,mstau12)*power3(mH2))/power2(mstau12 - mu2) + (3*mu2*DeltaInv(mH2,
     mstau12,mu2)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (3*mu2*
     DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2))
     - (DeltaInv(mH2,mstau12,mu2)*power3(mu2))/power2(mstau22 - mu2) + (mH2*
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (4*
     power2(mu2))/power3(-mstau12 + mu2) + (DeltaInv(mH2,mstau12,mu2)*power4(
     mu2))/(mstau12*power2(mstau22 - mu2)) - (18*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/(mH2*mstau12) - (DeltaInv(mstau22,mH2,mstau12)*power6(
     mstau2))/power2(mstau12 - mu2) - (DeltaInv(mA2,mstau22,mstau12)*(-(power3(
     mA2)*(power2(mu2)*(-14*mstau22*mu2 + 7*power2(mstau22) + 2*power2(mu2)) +
     5*power2(mstau12)*(-6*mstau22*mu2 + 2*power2(mstau22) + 5*power2(mu2)) +
     5*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*(-18*mu2*power2(mstau22) +
     38*mstau22*power2(mu2) + power3(mstau22) - 16*power3(mu2)))) + (3*(mstau22
     - 2*mu2)*power2(mstau12) + 3*(mstau22 - 2*mu2)*power2(mu2) + mstau12*(-8*
     mstau22*mu2 + power2(mstau22) + 13*power2(mu2)))*power4(mA2) - power2(mA2)
     *(2*(-6*mstau22*mu2 + 3*power2(mstau22) - 2*power2(mu2))*power3(mstau12) +
     power2(mu2)*(-3*mstau22*mstau24 + 6*mstau24*mu2 - 8*mu2*power2(mstau22) +
     2*mstau22*power2(mu2) + 4*power3(mstau22)) + power2(mstau12)*(-26*mu2*
     power2(mstau22) + 6*mu2*(mstau24 + power2(mu2)) + mstau22*(-3*mstau24 +
     32*power2(mu2)) + 6*power3(mstau22)) + mstau12*(4*mstau22*mu2*(mstau24 -
     4*power2(mu2)) - 11*mstau24*power2(mu2) + power2(mstau22)*(mstau24 + 16*
     power2(mu2)) - 4*mu2*power3(mstau22) - 2*power4(mu2))) + 2*power2(mstau12
     - mu2)*power2(mstau22 - mu2)*(5*mstau12*mstau24 - 3*power6(mstau2)) + mA2*
     (mstau24*power2(mu2)*(-22*mstau22*mu2 + 11*power2(mstau22) + 10*power2(
     mu2)) + mstau22*(-42*mstau22*mu2 + 21*power2(mstau22) + 22*power2(mu2))*
     power3(mstau12) + power2(mstau12)*(7*mstau24*power2(mu2) + 8*power2(
     mstau22)*(mstau24 + 10*power2(mu2)) - 40*mu2*power3(mstau22) - 2*mstau22*(
     8*mstau24*mu2 + 21*power3(mu2))) + mstau12*(mu2*(-18*mstau24*power2(mu2) -
     2*power2(mstau22)*(10*mstau24 + 19*power2(mu2)) + 19*mu2*power3(mstau22) +
     20*mstau22*(2*mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(
     mstau2)))))/(mA2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))) +
     6*log(mstau12/Q2)*(6/mA2 - 18/mH2 - 20/mstau12 + (18*mstau22)/(mA2*
     mstau12) - (54*mstau22)/(mH2*mstau12) + 4/(mstau12 - mu2) + 6/(-mstau22 +
     mu2) - (9*mA2)/(mstau12*mstau22 - mstau12*mu2) - (9*mH2)/(mstau12*mstau22
     - mstau12*mu2) - 24*mH2*DeltaInv(mstau22,mH2,mstau12) - 177*mstau22*
     DeltaInv(mstau22,mH2,mstau12) + (12*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/mstau12 - (90*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 - (87*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (6*mH2*mu2*DeltaInv(
     mstau22,mH2,mstau12))/(mstau12 - mu2) + (6*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/(mstau12 - mu2) + (6*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/(-mstau12 + mu2) + (9*mstau24*DeltaInv(mstau22,mH2,mstau12))/(-
     mstau12 + mu2) + (18*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(-mstau22
     + mu2) + (9*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 -
     mstau12*mu2) + (21*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (
     9*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau12 + mu2) + (15*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(-mstau22 + mu2) - (18*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2)
     - (3*mA2)/power2(mstau12 - mu2) - (3*mH2)/power2(mstau12 - mu2) - (6*
     mstau22)/power2(mstau12 - mu2) - (4*mu2)/power2(mstau12 - mu2) - (3*mH2*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (18*mH2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*
     mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2) - (3*
     mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) -
     (3*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2) +
     (12*mu2)/power2(mstau22 - mu2) + (18*mA2*mu2)/(mstau12*power2(mstau22 -
     mu2)) + (18*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) + (18*mH2*mstau22*
     mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) - (9*mH2*mstau24*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) + (15*
     mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/power2(mstau22 - mu2) + (15*
     mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau22 - mu2) + (
     18*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*power2(
     mstau22 - mu2)) + (3*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(-mstau22
     + mu2) + (2*power2(mu2))/(mstau12*power2(mstau12 - mu2)) + (12*power2(mu2)
     )/(mstau12*power2(mstau22 - mu2)) + (30*mH2*DeltaInv(mH2,mstau12,mu2)*
     power2(mu2))/power2(mstau22 - mu2) + (12*mH2*DeltaInv(mstau22,mH2,mstau12)
     *power2(mu2))/power2(mstau22 - mu2) + (6*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) + (6*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) + (15*DeltaInv(mH2,mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (3*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*
     power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (3*mu2*DeltaInv(mA2,
     mstau12,mu2)*(mA2*mu2*(-10*mstau12 + mu2) - 5*(mstau12 + mu2)*power2(mA2)
     + (-mstau12 + mu2)*power2(mu2) + 3*power3(mA2)))/(mstau12*power2(mstau22 -
     mu2)) + (9*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) + (3*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(
     mstau12 - mu2) - (9*mu2*DeltaInv(mH2,mstau12,mu2)*power3(mH2))/(mstau12*
     power2(mstau22 - mu2)) - (9*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))
     /(mstau12*power2(mstau22 - mu2)) + (4*power2(mu2))/power3(mstau12 - mu2) +
     (3*DeltaInv(mH2,mstau12,mu2)*power3(mu2))/power2(mstau22 - mu2) - (3*mH2*
     DeltaInv(mH2,mstau12,mu2)*power3(mu2))/(mstau12*power2(mstau22 - mu2)) - (
     3*DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) - (3*
     DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     54*DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) + (3*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) + (3*
     DeltaInv(mA2,mstau22,mstau12)*(-(power3(mA2)*(power2(mu2)*(-14*mstau22*mu2
     + 7*power2(mstau22) + 2*power2(mu2)) + 5*power2(mstau12)*(-6*mstau22*mu2 +
     2*power2(mstau22) + 5*power2(mu2)) + 5*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*(-18*mu2*power2(mstau22) + 38*mstau22*power2(mu2) + power3(
     mstau22) - 16*power3(mu2)))) + (3*(mstau22 - 2*mu2)*power2(mstau12) + 3*(
     mstau22 - 2*mu2)*power2(mu2) + mstau12*(-8*mstau22*mu2 + power2(mstau22) +
     13*power2(mu2)))*power4(mA2) - power2(mA2)*(2*(-6*mstau22*mu2 + 3*power2(
     mstau22) - 2*power2(mu2))*power3(mstau12) + power2(mu2)*(-3*mstau22*
     mstau24 + 6*mstau24*mu2 - 8*mu2*power2(mstau22) + 2*mstau22*power2(mu2) +
     4*power3(mstau22)) + power2(mstau12)*(-26*mu2*power2(mstau22) + 6*mu2*(
     mstau24 + power2(mu2)) + mstau22*(-3*mstau24 + 32*power2(mu2)) + 6*power3(
     mstau22)) + mstau12*(4*mstau22*mu2*(mstau24 - 4*power2(mu2)) - 11*mstau24*
     power2(mu2) + power2(mstau22)*(mstau24 + 16*power2(mu2)) - 4*mu2*power3(
     mstau22) - 2*power4(mu2))) + 2*power2(mstau12 - mu2)*power2(mstau22 - mu2)
     *(5*mstau12*mstau24 - 3*power6(mstau2)) + mA2*(mstau24*power2(mu2)*(-22*
     mstau22*mu2 + 11*power2(mstau22) + 10*power2(mu2)) + mstau22*(-42*mstau22*
     mu2 + 21*power2(mstau22) + 22*power2(mu2))*power3(mstau12) + power2(
     mstau12)*(7*mstau24*power2(mu2) + 8*power2(mstau22)*(mstau24 + 10*power2(
     mu2)) - 40*mu2*power3(mstau22) - 2*mstau22*(8*mstau24*mu2 + 21*power3(mu2)
     )) + mstau12*(mu2*(-18*mstau24*power2(mu2) - 2*power2(mstau22)*(10*mstau24
     + 19*power2(mu2)) + 19*mu2*power3(mstau22) + 20*mstau22*(2*mstau24*mu2 +
     power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))) - (6*log(mstau12/Q2)*log(
     mstau22/Q2)*power2(sa)*(power2(mstau12 - mu2)*(-(mh2*mH2*(mstau22 - 2*mu2)
     ) + (mstau22 - 2*mu2)*power2(mH2) + 24*mstau22*power2(mstau22 - mu2)) +
     mH2*DeltaInv(mstau22,mstau12,mh2)*(mstau22*mstau24*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power3(mh2) +
     mstau22*(-2*mstau22*mu2 + power2(mstau22) + 2*power2(mu2))*power3(mstau12)
     + mh2*(-((mstau22 - 2*mu2)*(-mstau24 + 2*power2(mstau22))*power2(mu2)) -
     6*mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(mstau12)*(-2*mstau24*
     mu2 + 18*mu2*power2(mstau22) + mstau22*(mstau24 - 25*power2(mu2)) - 3*
     power3(mstau22)) + 2*mstau12*(mstau24*power2(mu2) + mstau22*mu2*(mstau24 +
     5*power2(mu2)) - power2(mstau22)*(mstau24 + 5*power2(mu2)) + mu2*power3(
     mstau22))) - power2(mstau12)*(-4*mstau22*mstau24*mu2 + 2*mstau24*power2(
     mstau22) + 3*mstau24*power2(mu2) + 2*mstau22*power3(mu2)) - power2(mh2)*(
     power2(mstau12)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*
     power2(mu2) + power3(mstau22) + 2*power3(mu2))) + mstau12*(power2(mu2)*(2*
     mstau24*mu2 + 2*mu2*power2(mstau22) - power3(mstau22)) + power2(mstau22 -
     mu2)*power6(mstau2))) - DeltaInv(mstau22,mH2,mstau12)*(power2(mH2)*(-6*
     mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(mstau12)*(-2*mstau24*
     mu2 - 30*mu2*power2(mstau22) + mstau22*(mstau24 - power2(mu2)) + 21*
     power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 - 44*mu2*power2(mstau22) +
     mstau22*(mstau24 + 24*power2(mu2)) + 22*power3(mstau22)) + mstau12*(-2*
     power2(mstau22)*(mstau24 - 43*power2(mu2)) + 2*mstau22*mu2*(mstau24 - 19*
     power2(mu2)) + 2*mstau24*power2(mu2) - 46*mu2*power3(mstau22))) - power3(
     mH2)*(power2(mstau12)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*
     power2(mu2) + power3(mstau22) + 2*power3(mu2))) + (mstau22 - 2*mu2)*
     power2(mstau12 - mu2)*power4(mH2) + 24*power2(mstau12 - mu2)*power2(
     mstau22 - mu2)*(-2*mstau12*mstau24 + power6(mstau2)) + mH2*(mstau24*(94*
     mstau22*mu2 - 47*power2(mstau22) - 48*power2(mu2))*power2(mu2) + mstau22*(
     94*mstau22*mu2 - 47*power2(mstau22) - 46*power2(mu2))*power3(mstau12) +
     power2(mstau12)*(-51*mstau24*power2(mu2) - 2*power2(mstau22)*(25*mstau24 +
     96*power2(mu2)) + 96*mu2*power3(mstau22) + 2*mstau22*(50*mstau24*mu2 + 47*
     power3(mu2))) + mstau12*(mu2*(98*mstau24*power2(mu2) + power2(mstau22)*(
     96*mstau24 + 98*power2(mu2)) - 49*mu2*power3(mstau22) - 48*mstau22*(4*
     mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2))))))/(
     mH2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (12*log(
     msntau2/Q2)*log(mstau22/Q2)*((mstau22 - 2*mu2)*power2(mC2)*power2(msntau2
     - mu2) + 2*mstau22*power2(msntau2 - mu2)*power2(mstau22 - mu2) + mC2*(
     power2(msntau2)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power3(msntau2) + msntau2*(-4*mu2*power2(
     mstau22) + 6*mstau22*power2(mu2) + power3(mstau22) - 2*power3(mu2))) -
     DeltaInv(mstau22,mC2,msntau2)*(power2(mC2)*(power2(mu2)*(-2*mstau24*mu2 +
     mstau22*(mstau24 + 2*power2(mu2))) - 6*mstau22*(mstau22 - 2*mu2)*power3(
     msntau2) - power2(msntau2)*(-(mstau22*mstau24) + 2*mstau24*mu2 - 14*mu2*
     power2(mstau22) + 23*mstau22*power2(mu2) + power3(mstau22)) - 2*msntau2*(-
     (mstau24*power2(mu2)) + power2(mstau22)*(mstau24 + power2(mu2)) - mstau22*
     mu2*(mstau24 + 3*power2(mu2)) + mu2*power3(mstau22))) - power3(mC2)*(
     power2(msntau2)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power3(msntau2) - msntau2*(-4*mstau22*
     power2(mu2) + power3(mstau22) + 2*power3(mu2))) + (mstau22 - 2*mu2)*
     power2(msntau2 - mu2)*power4(mC2) + 2*power2(msntau2 - mu2)*power2(mstau22
     - mu2)*(-2*msntau2*mstau24 + power6(mstau2)) + mC2*(mstau24*(6*mstau22*mu2
     - 3*power2(mstau22) - 4*power2(mu2))*power2(mu2) + mstau22*(6*mstau22*mu2
     - 3*power2(mstau22) - 2*power2(mu2))*power3(msntau2) + power2(msntau2)*(-
     7*mstau24*power2(mu2) - 2*power2(mstau22)*(3*mstau24 + 8*power2(mu2)) + 8*
     mu2*power3(mstau22) + 6*mstau22*(2*mstau24*mu2 + power3(mu2))) + msntau2*(
     mu2*(10*mstau24*power2(mu2) + 2*power2(mstau22)*(4*mstau24 + 5*power2(mu2)
     ) - 5*mu2*power3(mstau22) - 4*mstau22*(4*mstau24*mu2 + power3(mu2))) +
     power2(mstau22 - mu2)*power6(mstau2))))))/(mC2*msntau2*power2(msntau2 -
     mu2)*power2(mstau22 - mu2)) + 6*log(mstau12/Q2)*log(mstau22/Q2)*(2/mstau12
     - (4*mstau22)/(mA2*mstau12) + (12*mstau22)/(mH2*mstau12) + 2/(mstau22 -
     mu2) + mA2/(mstau12*mstau22 - mstau12*mu2) + mH2/(mstau12*mstau22 -
     mstau12*mu2) + (2*mstau22)/power2(mstau12 - mu2) - (2*mu2)/power2(mstau22
     - mu2) - (mA2*mu2)/(mstau12*power2(mstau22 - mu2)) - (mH2*mu2)/(mstau12*
     power2(mstau22 - mu2)) - (2*power2(mu2))/(mstau12*power2(mstau22 - mu2)) -
     (DeltaInv(mstau22,mH2,mstau12)*(-(power3(mH2)*(power2(mstau12)*power2(
     mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*power2(mu2) + (mstau22 - 2*
     mu2)*power3(mstau12) - mstau12*(-4*mstau22*power2(mu2) + power3(mstau22) +
     2*power3(mu2)))) + power2(mH2)*(-6*mstau22*(mstau22 - 2*mu2)*power3(
     mstau12) + power2(mstau12)*(-2*mstau24*mu2 - 6*mu2*power2(mstau22) +
     mstau22*(mstau24 - 13*power2(mu2)) + 9*power3(mstau22)) + power2(mu2)*(-2*
     mstau24*mu2 - 20*mu2*power2(mstau22) + mstau22*(mstau24 + 12*power2(mu2))
     + 10*power3(mstau22)) - 2*mstau12*(power2(mstau22)*(mstau24 - 19*power2(
     mu2)) - mstau24*power2(mu2) + 11*mu2*power3(mstau22) + mstau22*(-(mstau24*
     mu2) + 7*power3(mu2)))) + (mstau22 - 2*mu2)*power2(mstau12 - mu2)*power4(
     mH2) + 12*power2(mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24
     + power6(mstau2)) + mH2*(mstau24*(46*mstau22*mu2 - 23*power2(mstau22) -
     24*power2(mu2))*power2(mu2) + mstau22*(46*mstau22*mu2 - 23*power2(mstau22)
     - 22*power2(mu2))*power3(mstau12) + power2(mstau12)*(-27*mstau24*power2(
     mu2) - 2*power2(mstau22)*(13*mstau24 + 48*power2(mu2)) + 48*mu2*power3(
     mstau22) + mstau22*(52*mstau24*mu2 + 46*power3(mu2))) + mstau12*(mu2*(50*
     mstau24*power2(mu2) + power2(mstau22)*(48*mstau24 + 50*power2(mu2)) - 25*
     mu2*power3(mstau22) - 24*mstau22*(4*mstau24*mu2 + power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2)))))/(mH2*mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2)) - (DeltaInv(mA2,mstau22,mstau12)*(power2(mA2)*(-6*
     mstau22*(mstau22 - 2*mu2)*power3(mstau12) + power2(mstau12)*(-2*mstau24*
     mu2 + 26*mu2*power2(mstau22) + mstau22*(mstau24 - 29*power2(mu2)) - 7*
     power3(mstau22)) + power2(mu2)*(-2*mstau24*mu2 + 12*mu2*power2(mstau22) +
     mstau22*(mstau24 - 4*power2(mu2)) - 6*power3(mstau22)) + 2*mstau12*(
     mstau24*power2(mu2) + mstau22*mu2*(mstau24 + 9*power2(mu2)) - power2(
     mstau22)*(mstau24 + 13*power2(mu2)) + 5*mu2*power3(mstau22))) - power3(
     mA2)*(power2(mstau12)*power2(mstau22 - 2*mu2) + mstau22*(mstau22 - 2*mu2)*
     power2(mu2) + (mstau22 - 2*mu2)*power3(mstau12) - mstau12*(-4*mstau22*
     power2(mu2) + power3(mstau22) + 2*power3(mu2))) + (mstau22 - 2*mu2)*
     power2(mstau12 - mu2)*power4(mA2) - 4*power2(mstau12 - mu2)*power2(mstau22
     - mu2)*(-2*mstau12*mstau24 + power6(mstau2)) + mA2*(mstau24*power2(mu2)*(-
     18*mstau22*mu2 + 9*power2(mstau22) + 8*power2(mu2)) + mstau22*(-18*
     mstau22*mu2 + 9*power2(mstau22) + 10*power2(mu2))*power3(mstau12) +
     power2(mstau12)*(5*mstau24*power2(mu2) + power2(mstau22)*(6*mstau24 + 32*
     power2(mu2)) - 16*mu2*power3(mstau22) - 6*mstau22*(2*mstau24*mu2 + 3*
     power3(mu2))) + mstau12*(mu2*(-14*mstau24*power2(mu2) - 2*power2(mstau22)*
     (8*mstau24 + 7*power2(mu2)) + 7*mu2*power3(mstau22) + 8*mstau22*(4*
     mstau24*mu2 + power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(
     mA2*mstau12*power2(mstau12 - mu2)*power2(mstau22 - mu2))) + 6*(-10/mA2 +
     10/mC2 + 30/mH2 + 29/msntau2 + 43/mstau12 + 2/mstau22 + (28*mstau22)/(mC2*
     msntau2) - (28*mstau22)/(mA2*mstau12) + (84*mstau22)/(mH2*mstau12) + 18/(
     mstau22 - mu2) + 5/(-msntau2 + mu2) + 5/(-mstau12 + mu2) + (28*mC2)/(
     msntau2*mstau22 - msntau2*mu2) + (14*mA2)/(mstau12*mstau22 - mstau12*mu2)
     + (14*mH2)/(mstau12*mstau22 - mstau12*mu2) + 28*mC2*DeltaInv(mstau22,mC2,
     msntau2) + 84*mstau22*DeltaInv(mstau22,mC2,msntau2) + (28*mC2*mstau22*
     DeltaInv(mstau22,mC2,msntau2))/msntau2 + (56*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/mC2 + (28*mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (56*
     mC2*mstau22*DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (42*mstau24*
     DeltaInv(mstau22,mC2,msntau2))/(msntau2 - mu2) + (56*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/(mstau22 - mu2) + (28*mC2*mu2*DeltaInv(mstau22,mC2,
     msntau2))/(-msntau2 + mu2) + (28*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2)
     )/(-msntau2 + mu2) - (28*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/(
     msntau2*mstau22 - msntau2*mu2) + 49*mH2*DeltaInv(mstau22,mH2,mstau12) +
     287*mstau22*DeltaInv(mstau22,mH2,mstau12) + (14*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 + (168*mstau24*DeltaInv(mstau22,mH2,mstau12)
     )/mH2 + (119*mstau24*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (28*mH2*
     mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (21*mstau24*
     DeltaInv(mstau22,mH2,mstau12))/(mstau12 - mu2) + (28*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12))/(mstau22 - mu2) + (14*mH2*mu2*DeltaInv(mstau22,mH2,
     mstau12))/(-mstau12 + mu2) + (14*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)
     )/(-mstau12 + mu2) - (14*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12*mstau22 - mstau12*mu2) - (28*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2))/msntau2 + (42*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2 -
     mu2) + (56*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(mstau22 - mu2) + (
     56*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2*mstau22 -
     msntau2*mu2) - (49*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (
     21*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12 - mu2) + (28*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau22 - mu2) + (28*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/(mstau12*mstau22 - mstau12*mu2)
     - (2*mu2)/(-(msntau2*mu2) + power2(msntau2)) - (2*mu2)/(-(mstau12*mu2) +
     power2(mstau12)) - (4*mu2)/(-(mstau22*mu2) + power2(mstau22)) + (14*mC2)/
     power2(msntau2 - mu2) + (14*mstau22)/power2(msntau2 - mu2) - (3*mu2)/
     power2(msntau2 - mu2) - (28*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (
     14*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (56*
     mC2*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (
     14*mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (14*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) +
     (14*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) -
     (56*mu2*DeltaInv(mstau22,mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + (7*
     mA2)/power2(mstau12 - mu2) + (7*mH2)/power2(mstau12 - mu2) + (14*mstau22)/
     power2(mstau12 - mu2) - (3*mu2)/power2(mstau12 - mu2) - (14*mA2*mu2)/(
     mstau22*power2(mstau12 - mu2)) - (14*mH2*mu2)/(mstau22*power2(mstau12 -
     mu2)) + (7*mH2*mstau24*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (28*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 -
     mu2) + (7*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau12 - mu2)
     - (28*mu2*DeltaInv(mA2,mstau22,mu2)*power2(mA2))/power2(mstau12 - mu2) + (
     7*mstau22*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2)
     + (7*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/power2(mstau12 - mu2)
     - (28*mu2*DeltaInv(mstau22,mH2,mu2)*power2(mH2))/power2(mstau12 - mu2) - (
     34*mu2)/power2(mstau22 - mu2) - (56*mC2*mu2)/(msntau2*power2(mstau22 -
     mu2)) - (28*mA2*mu2)/(mstau12*power2(mstau22 - mu2)) - (28*mH2*mu2)/(
     mstau12*power2(mstau22 - mu2)) - (56*mC2*mstau22*mu2*DeltaInv(mstau22,mC2,
     msntau2))/power2(mstau22 - mu2) + (28*mC2*mstau24*mu2*DeltaInv(mstau22,
     mC2,msntau2))/(msntau2*power2(mstau22 - mu2)) - (28*mH2*mstau22*mu2*
     DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2) + (14*mH2*mstau24*
     mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(mstau22 - mu2)) - (56*
     mu2*DeltaInv(mC2,msntau2,mu2)*power2(mC2))/power2(mstau22 - mu2) - (56*
     mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(mstau22 - mu2) - (
     56*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/(msntau2*power2(
     mstau22 - mu2)) - (28*mu2*DeltaInv(mH2,mstau12,mu2)*power2(mH2))/power2(
     mstau22 - mu2) - (28*mu2*DeltaInv(mstau22,mH2,mstau12)*power2(mH2))/
     power2(mstau22 - mu2) - (28*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mC2,
     msntau2)*power2(mu2))/(mstau22 - mu2) + (7*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/(mstau22 - mu2) - (3*power2(mu2))/(msntau2*power2(msntau2 -
     mu2)) - (14*power2(mu2))/(mstau22*power2(msntau2 - mu2)) - (98*mC2*
     DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (42*
     DeltaInv(mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(mstau22*power2(msntau2
     - mu2)) - (3*power2(mu2))/(mstau12*power2(mstau12 - mu2)) - (14*power2(
     mu2))/(mstau22*power2(mstau12 - mu2)) - (49*mA2*DeltaInv(mA2,mstau22,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (49*mH2*DeltaInv(mstau22,mH2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (21*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (21*DeltaInv(mstau22,
     mH2,mu2)*power2(mH2)*power2(mu2))/(mstau22*power2(mstau12 - mu2)) - (28*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (28*power2(mu2))/(mstau12*
     power2(mstau22 - mu2)) - (6*power2(mu2))/(mstau22*power2(mstau22 - mu2)) -
     (98*mC2*DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (
     49*mH2*DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) - (42*
     mC2*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) - (
     28*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 -
     mu2) - (28*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (14*mstau24*DeltaInv(mstau22,mC2,msntau2)
     *power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (21*mH2*DeltaInv(mstau22,
     mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (14*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/power2(mstau22 - mu2) - (14*mH2*mstau22*
     DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2))
     + (7*mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) - (42*DeltaInv(mC2,msntau2,mu2)*power2(mC2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) + (14*DeltaInv(mstau22,mC2,msntau2)*power2(
     mC2)*power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (21*DeltaInv(mH2,
     mstau12,mu2)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     7*DeltaInv(mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(
     mstau22 - mu2)) + (14*mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*
     power2(mstau12 - mu2)) + (7*mu2*DeltaInv(mA2,mstau12,mu2)*(-7*mA2*mstau12*
     mu2 - (4*mstau12 + 3*mu2)*power2(mA2) + (-mstau12 + mu2)*power2(mu2) + 2*
     power3(mA2)))/(mstau12*power2(mstau22 - mu2)) - (28*DeltaInv(mstau22,mC2,
     msntau2)*power3(mC2))/(msntau2*mstau22 - msntau2*mu2) - (14*DeltaInv(
     mstau22,mC2,msntau2)*power3(mC2))/power2(msntau2 - mu2) + (28*mu2*
     DeltaInv(mstau22,mC2,mu2)*power3(mC2))/(mstau22*power2(msntau2 - mu2)) + (
     28*mu2*DeltaInv(mC2,msntau2,mu2)*power3(mC2))/(msntau2*power2(mstau22 -
     mu2)) + (28*mu2*DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/(msntau2*
     power2(mstau22 - mu2)) - (14*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(
     mstau12*mstau22 - mstau12*mu2) - (7*DeltaInv(mstau22,mH2,mstau12)*power3(
     mH2))/power2(mstau12 - mu2) + (14*mu2*DeltaInv(mstau22,mH2,mu2)*power3(
     mH2))/(mstau22*power2(mstau12 - mu2)) + (14*mu2*DeltaInv(mH2,mstau12,mu2)*
     power3(mH2))/(mstau12*power2(mstau22 - mu2)) + (14*mu2*DeltaInv(mstau22,
     mH2,mstau12)*power3(mH2))/(mstau12*power2(mstau22 - mu2)) - (14*DeltaInv(
     mstau22,mC2,mu2)*power3(mu2))/power2(msntau2 - mu2) - (7*DeltaInv(mA2,
     mstau22,mu2)*power3(mu2))/power2(mstau12 - mu2) - (7*DeltaInv(mstau22,mH2,
     mu2)*power3(mu2))/power2(mstau12 - mu2) - (14*DeltaInv(mC2,msntau2,mu2)*
     power3(mu2))/power2(mstau22 - mu2) - (7*DeltaInv(mH2,mstau12,mu2)*power3(
     mu2))/power2(mstau22 - mu2) + (14*DeltaInv(mstau22,mC2,msntau2)*power3(
     mu2))/power2(mstau22 - mu2) + (7*DeltaInv(mstau22,mH2,mstau12)*power3(mu2)
     )/power2(mstau22 - mu2) + (14*DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(
     mstau22*power2(msntau2 - mu2)) + (7*DeltaInv(mA2,mstau22,mu2)*power4(mu2))
     /(mstau22*power2(mstau12 - mu2)) + (7*DeltaInv(mstau22,mH2,mu2)*power4(
     mu2))/(mstau22*power2(mstau12 - mu2)) + (14*DeltaInv(mC2,msntau2,mu2)*
     power4(mu2))/(msntau2*power2(mstau22 - mu2)) + (7*DeltaInv(mH2,mstau12,
     mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) - (28*DeltaInv(mstau22,
     mC2,msntau2)*power6(mstau2))/(mC2*msntau2) - (84*DeltaInv(mstau22,mH2,
     mstau12)*power6(mstau2))/(mH2*mstau12) - (14*DeltaInv(mstau22,mC2,msntau2)
     *power6(mstau2))/power2(msntau2 - mu2) - (7*DeltaInv(mstau22,mH2,mstau12)*
     power6(mstau2))/power2(mstau12 - mu2) - (7*DeltaInv(mA2,mstau22,mstau12)*(
     -(power3(mA2)*(power2(mu2)*(-10*mstau22*mu2 + 5*power2(mstau22) + 2*
     power2(mu2)) + power2(mstau12)*(-24*mstau22*mu2 + 8*power2(mstau22) + 21*
     power2(mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) + mstau12*(-14*mu2*
     power2(mstau22) + 29*mstau22*power2(mu2) + power3(mstau22) - 14*power3(
     mu2)))) + (2*(mstau22 - 2*mu2)*power2(mstau12) + mstau12*power2(mstau22 -
     3*mu2) + 2*(mstau22 - 2*mu2)*power2(mu2))*power4(mA2) + power2(mA2)*(-2*(
     mstau22 - 2*mu2)*(-mstau24 + power2(mstau22))*power2(mu2) + (6*mstau22*mu2
     - 3*power2(mstau22) + 4*power2(mu2))*power3(mstau12) - 2*power2(mstau12)*(
     -(mstau22*mstau24) + 2*mstau24*mu2 - 10*mu2*power2(mstau22) + 10*mstau22*
     power2(mu2) + 3*power3(mstau22) + 3*power3(mu2)) + mstau12*(-2*mstau22*
     mu2*(mstau24 - 5*power2(mu2)) + 7*mstau24*power2(mu2) - power2(mstau22)*(
     mstau24 + 13*power2(mu2)) + 4*mu2*power3(mstau22) + 2*power4(mu2))) - 4*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)*(-2*mstau12*mstau24 + power6(
     mstau2)) + mA2*(mstau24*power2(mu2)*(-14*mstau22*mu2 + 7*power2(mstau22) +
     6*power2(mu2)) + mstau22*(-30*mstau22*mu2 + 15*power2(mstau22) + 16*
     power2(mu2))*power3(mstau12) + power2(mstau12)*(3*mstau24*power2(mu2) + 4*
     power2(mstau22)*(mstau24 + 14*power2(mu2)) - 28*mu2*power3(mstau22) - 2*
     mstau22*(4*mstau24*mu2 + 15*power3(mu2))) + mstau12*(mu2*(-10*mstau24*
     power2(mu2) - 2*power2(mstau22)*(6*mstau24 + 13*power2(mu2)) + 13*mu2*
     power3(mstau22) + 2*mstau22*(12*mstau24*mu2 + 7*power3(mu2))) + power2(
     mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*power2(mstau12 - mu2)*
     power2(mstau22 - mu2))) + power2(Pi)*(-2/mA2 + 2/mC2 + 6/mH2 + 4/msntau2 +
     6/mstau12 + (4*mstau22)/(mC2*msntau2) - (4*mstau22)/(mA2*mstau12) + (12*
     mstau22)/(mH2*mstau12) + 4/(mstau22 - mu2) + (4*mC2)/(msntau2*mstau22 -
     msntau2*mu2) + (2*mA2)/(mstau12*mstau22 - mstau12*mu2) + (2*mH2)/(mstau12*
     mstau22 - mstau12*mu2) + 4*mC2*DeltaInv(mstau22,mC2,msntau2) + 12*mstau22*
     DeltaInv(mstau22,mC2,msntau2) + (4*mC2*mstau22*DeltaInv(mstau22,mC2,
     msntau2))/msntau2 + (8*mstau24*DeltaInv(mstau22,mC2,msntau2))/mC2 + (4*
     mstau24*DeltaInv(mstau22,mC2,msntau2))/msntau2 + (8*mC2*mstau22*DeltaInv(
     mstau22,mC2,msntau2))/(msntau2 - mu2) + (6*mstau24*DeltaInv(mstau22,mC2,
     msntau2))/(msntau2 - mu2) + (8*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2))/
     (mstau22 - mu2) + (4*mC2*mu2*DeltaInv(mstau22,mC2,msntau2))/(-msntau2 +
     mu2) + (4*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/(-msntau2 + mu2) - (
     4*mC2*mstau24*DeltaInv(mstau22,mC2,msntau2))/(msntau2*mstau22 - msntau2*
     mu2) + 7*mH2*DeltaInv(mstau22,mH2,mstau12) + 41*mstau22*DeltaInv(mstau22,
     mH2,mstau12) + (2*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/mstau12 + (
     24*mstau24*DeltaInv(mstau22,mH2,mstau12))/mH2 + (17*mstau24*DeltaInv(
     mstau22,mH2,mstau12))/mstau12 + (4*mH2*mstau22*DeltaInv(mstau22,mH2,
     mstau12))/(mstau12 - mu2) + (3*mstau24*DeltaInv(mstau22,mH2,mstau12))/(
     mstau12 - mu2) + (4*mH2*mstau22*DeltaInv(mstau22,mH2,mstau12))/(mstau22 -
     mu2) + (2*mH2*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) + (2*
     mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/(-mstau12 + mu2) - (2*mH2*
     mstau24*DeltaInv(mstau22,mH2,mstau12))/(mstau12*mstau22 - mstau12*mu2) - (
     4*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/msntau2 + (6*DeltaInv(
     mstau22,mC2,msntau2)*power2(mC2))/(msntau2 - mu2) + (8*DeltaInv(mstau22,
     mC2,msntau2)*power2(mC2))/(mstau22 - mu2) + (8*mstau22*DeltaInv(mstau22,
     mC2,msntau2)*power2(mC2))/(msntau2*mstau22 - msntau2*mu2) - (7*DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2))/mstau12 + (3*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(mstau12 - mu2) + (4*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau22 - mu2) + (4*mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/(mstau12*mstau22 - mstau12*mu2) + (2*mC2)/power2(msntau2 -
     mu2) + (2*mstau22)/power2(msntau2 - mu2) - (2*mu2)/power2(msntau2 - mu2) -
     (4*mC2*mu2)/(mstau22*power2(msntau2 - mu2)) + (2*mC2*mstau24*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (8*mC2*mstau22*mu2*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*mstau24*mu2*DeltaInv(
     mstau22,mC2,msntau2))/power2(msntau2 - mu2) + (2*mstau22*DeltaInv(mstau22,
     mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) + (2*mu2*DeltaInv(mstau22,
     mC2,msntau2)*power2(mC2))/power2(msntau2 - mu2) - (8*mu2*DeltaInv(mstau22,
     mC2,mu2)*power2(mC2))/power2(msntau2 - mu2) + mA2/power2(mstau12 - mu2) +
     mH2/power2(mstau12 - mu2) + (2*mstau22)/power2(mstau12 - mu2) - (2*mu2)/
     power2(mstau12 - mu2) - (2*mA2*mu2)/(mstau22*power2(mstau12 - mu2)) - (2*
     mH2*mu2)/(mstau22*power2(mstau12 - mu2)) + (mH2*mstau24*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau12 - mu2) + (4*mH2*mstau22*mu2*DeltaInv(mstau22,
     mH2,mstau12))/power2(mstau12 - mu2) + (mstau24*mu2*DeltaInv(mstau22,mH2,
     mstau12))/power2(mstau12 - mu2) - (4*mu2*DeltaInv(mA2,mstau22,mu2)*power2(
     mA2))/power2(mstau12 - mu2) + (mstau22*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) + (mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau12 - mu2) - (4*mu2*DeltaInv(mstau22,mH2,mu2)*
     power2(mH2))/power2(mstau12 - mu2) - (8*mu2)/power2(mstau22 - mu2) - (8*
     mC2*mu2)/(msntau2*power2(mstau22 - mu2)) - (4*mA2*mu2)/(mstau12*power2(
     mstau22 - mu2)) - (4*mH2*mu2)/(mstau12*power2(mstau22 - mu2)) - (8*mC2*
     mstau22*mu2*DeltaInv(mstau22,mC2,msntau2))/power2(mstau22 - mu2) + (4*mC2*
     mstau24*mu2*DeltaInv(mstau22,mC2,msntau2))/(msntau2*power2(mstau22 - mu2))
     - (4*mH2*mstau22*mu2*DeltaInv(mstau22,mH2,mstau12))/power2(mstau22 - mu2)
     + (2*mH2*mstau24*mu2*DeltaInv(mstau22,mH2,mstau12))/(mstau12*power2(
     mstau22 - mu2)) - (8*mu2*DeltaInv(mC2,msntau2,mu2)*power2(mC2))/power2(
     mstau22 - mu2) - (8*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))/power2(
     mstau22 - mu2) - (8*mstau22*mu2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2))
     /(msntau2*power2(mstau22 - mu2)) - (4*mu2*DeltaInv(mH2,mstau12,mu2)*
     power2(mH2))/power2(mstau22 - mu2) - (4*mu2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mH2))/power2(mstau22 - mu2) - (4*mstau22*mu2*DeltaInv(mstau22,mH2,
     mstau12)*power2(mH2))/(mstau12*power2(mstau22 - mu2)) + (2*DeltaInv(
     mstau22,mC2,msntau2)*power2(mu2))/(mstau22 - mu2) + (DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/(mstau22 - mu2) - (2*power2(mu2))/(mstau22*power2(
     msntau2 - mu2)) - (14*mC2*DeltaInv(mstau22,mC2,mu2)*power2(mu2))/power2(
     msntau2 - mu2) - (6*DeltaInv(mstau22,mC2,mu2)*power2(mC2)*power2(mu2))/(
     mstau22*power2(msntau2 - mu2)) - (2*power2(mu2))/(mstau22*power2(mstau12 -
     mu2)) - (7*mA2*DeltaInv(mA2,mstau22,mu2)*power2(mu2))/power2(mstau12 -
     mu2) - (7*mH2*DeltaInv(mstau22,mH2,mu2)*power2(mu2))/power2(mstau12 - mu2)
     - (3*DeltaInv(mA2,mstau22,mu2)*power2(mA2)*power2(mu2))/(mstau22*power2(
     mstau12 - mu2)) - (3*DeltaInv(mstau22,mH2,mu2)*power2(mH2)*power2(mu2))/(
     mstau22*power2(mstau12 - mu2)) - (4*power2(mu2))/(msntau2*power2(mstau22 -
     mu2)) - (4*power2(mu2))/(mstau12*power2(mstau22 - mu2)) - (14*mC2*
     DeltaInv(mC2,msntau2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (7*mH2*
     DeltaInv(mH2,mstau12,mu2)*power2(mu2))/power2(mstau22 - mu2) - (6*mC2*
     DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) - (4*
     mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/power2(mstau22 - mu2) -
     (4*mC2*mstau22*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(msntau2*power2(
     mstau22 - mu2)) + (2*mstau24*DeltaInv(mstau22,mC2,msntau2)*power2(mu2))/(
     msntau2*power2(mstau22 - mu2)) - (3*mH2*DeltaInv(mstau22,mH2,mstau12)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mstau22*DeltaInv(mstau22,mH2,
     mstau12)*power2(mu2))/power2(mstau22 - mu2) - (2*mH2*mstau22*DeltaInv(
     mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (
     mstau24*DeltaInv(mstau22,mH2,mstau12)*power2(mu2))/(mstau12*power2(mstau22
     - mu2)) - (6*DeltaInv(mC2,msntau2,mu2)*power2(mC2)*power2(mu2))/(msntau2*
     power2(mstau22 - mu2)) + (2*DeltaInv(mstau22,mC2,msntau2)*power2(mC2)*
     power2(mu2))/(msntau2*power2(mstau22 - mu2)) - (3*DeltaInv(mH2,mstau12,
     mu2)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 - mu2)) + (DeltaInv(
     mstau22,mH2,mstau12)*power2(mH2)*power2(mu2))/(mstau12*power2(mstau22 -
     mu2)) + (2*mu2*DeltaInv(mA2,mstau22,mu2)*power3(mA2))/(mstau22*power2(
     mstau12 - mu2)) + (mu2*DeltaInv(mA2,mstau12,mu2)*(-7*mA2*mstau12*mu2 - (4*
     mstau12 + 3*mu2)*power2(mA2) + (-mstau12 + mu2)*power2(mu2) + 2*power3(
     mA2)))/(mstau12*power2(mstau22 - mu2)) - (4*DeltaInv(mstau22,mC2,msntau2)*
     power3(mC2))/(msntau2*mstau22 - msntau2*mu2) - (2*DeltaInv(mstau22,mC2,
     msntau2)*power3(mC2))/power2(msntau2 - mu2) + (4*mu2*DeltaInv(mstau22,mC2,
     mu2)*power3(mC2))/(mstau22*power2(msntau2 - mu2)) + (4*mu2*DeltaInv(mC2,
     msntau2,mu2)*power3(mC2))/(msntau2*power2(mstau22 - mu2)) + (4*mu2*
     DeltaInv(mstau22,mC2,msntau2)*power3(mC2))/(msntau2*power2(mstau22 - mu2))
     - (2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/(mstau12*mstau22 -
     mstau12*mu2) - (DeltaInv(mstau22,mH2,mstau12)*power3(mH2))/power2(mstau12
     - mu2) + (2*mu2*DeltaInv(mstau22,mH2,mu2)*power3(mH2))/(mstau22*power2(
     mstau12 - mu2)) + (2*mu2*DeltaInv(mH2,mstau12,mu2)*power3(mH2))/(mstau12*
     power2(mstau22 - mu2)) + (2*mu2*DeltaInv(mstau22,mH2,mstau12)*power3(mH2))
     /(mstau12*power2(mstau22 - mu2)) - (2*DeltaInv(mstau22,mC2,mu2)*power3(
     mu2))/power2(msntau2 - mu2) - (DeltaInv(mA2,mstau22,mu2)*power3(mu2))/
     power2(mstau12 - mu2) - (DeltaInv(mstau22,mH2,mu2)*power3(mu2))/power2(
     mstau12 - mu2) - (2*DeltaInv(mC2,msntau2,mu2)*power3(mu2))/power2(mstau22
     - mu2) - (DeltaInv(mH2,mstau12,mu2)*power3(mu2))/power2(mstau22 - mu2) + (
     2*DeltaInv(mstau22,mC2,msntau2)*power3(mu2))/power2(mstau22 - mu2) + (
     DeltaInv(mstau22,mH2,mstau12)*power3(mu2))/power2(mstau22 - mu2) + (2*
     DeltaInv(mstau22,mC2,mu2)*power4(mu2))/(mstau22*power2(msntau2 - mu2)) + (
     DeltaInv(mA2,mstau22,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     DeltaInv(mstau22,mH2,mu2)*power4(mu2))/(mstau22*power2(mstau12 - mu2)) + (
     2*DeltaInv(mC2,msntau2,mu2)*power4(mu2))/(msntau2*power2(mstau22 - mu2)) +
     (DeltaInv(mH2,mstau12,mu2)*power4(mu2))/(mstau12*power2(mstau22 - mu2)) -
     (4*DeltaInv(mstau22,mC2,msntau2)*power6(mstau2))/(mC2*msntau2) - (12*
     DeltaInv(mstau22,mH2,mstau12)*power6(mstau2))/(mH2*mstau12) - (2*DeltaInv(
     mstau22,mC2,msntau2)*power6(mstau2))/power2(msntau2 - mu2) - (DeltaInv(
     mstau22,mH2,mstau12)*power6(mstau2))/power2(mstau12 - mu2) - (DeltaInv(
     mA2,mstau22,mstau12)*(-(power3(mA2)*(power2(mu2)*(-10*mstau22*mu2 + 5*
     power2(mstau22) + 2*power2(mu2)) + power2(mstau12)*(-24*mstau22*mu2 + 8*
     power2(mstau22) + 21*power2(mu2)) + 4*(mstau22 - 2*mu2)*power3(mstau12) +
     mstau12*(-14*mu2*power2(mstau22) + 29*mstau22*power2(mu2) + power3(
     mstau22) - 14*power3(mu2)))) + (2*(mstau22 - 2*mu2)*power2(mstau12) +
     mstau12*power2(mstau22 - 3*mu2) + 2*(mstau22 - 2*mu2)*power2(mu2))*power4(
     mA2) + power2(mA2)*(-2*(mstau22 - 2*mu2)*(-mstau24 + power2(mstau22))*
     power2(mu2) + (6*mstau22*mu2 - 3*power2(mstau22) + 4*power2(mu2))*power3(
     mstau12) - 2*power2(mstau12)*(-(mstau22*mstau24) + 2*mstau24*mu2 - 10*mu2*
     power2(mstau22) + 10*mstau22*power2(mu2) + 3*power3(mstau22) + 3*power3(
     mu2)) + mstau12*(-2*mstau22*mu2*(mstau24 - 5*power2(mu2)) + 7*mstau24*
     power2(mu2) - power2(mstau22)*(mstau24 + 13*power2(mu2)) + 4*mu2*power3(
     mstau22) + 2*power4(mu2))) - 4*power2(mstau12 - mu2)*power2(mstau22 - mu2)
     *(-2*mstau12*mstau24 + power6(mstau2)) + mA2*(mstau24*power2(mu2)*(-14*
     mstau22*mu2 + 7*power2(mstau22) + 6*power2(mu2)) + mstau22*(-30*mstau22*
     mu2 + 15*power2(mstau22) + 16*power2(mu2))*power3(mstau12) + power2(
     mstau12)*(3*mstau24*power2(mu2) + 4*power2(mstau22)*(mstau24 + 14*power2(
     mu2)) - 28*mu2*power3(mstau22) - 2*mstau22*(4*mstau24*mu2 + 15*power3(mu2)
     )) + mstau12*(mu2*(-10*mstau24*power2(mu2) - 2*power2(mstau22)*(6*mstau24
     + 13*power2(mu2)) + 13*mu2*power3(mstau22) + 2*mstau22*(12*mstau24*mu2 +
     7*power3(mu2))) + power2(mstau22 - mu2)*power6(mstau2)))))/(mA2*mstau12*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/48.);

   return result * power4(ytau) * twoLoop;
}

double delta_mtau_2loop_atau_at(const Parameters& pars)
{
   using std::log;
   const Real ytau  = pars.ytau;
   const Real yt    = pars.yt;
   const Real xt    = pars.xt;
   const Real xtau  = pars.xtau;
   const Real mt    = pars.mt;
   const Real mt2   = power2(pars.mt);
   const Real mst1  = pars.mst1;
   const Real mst12 = power2(pars.mst1);
   const Real mst14 = power4(pars.mst1);
   const Real mst2  = pars.mst2;
   const Real mst22 = power2(pars.mst2);
   const Real mst24 = power4(pars.mst2);
   const Real msb1  = pars.msb1;
   const Real msb12 = power2(pars.msb1);
   const Real msb14 = power4(pars.msb1);
   const Real mstau12 = power2(pars.mstau1);
   const Real mstau22 = power2(pars.mstau2);
   const Real msntau2 = power2(pars.msntau);
   const Real mh2   = power2(pars.mh);
   const Real mH2   = power2(pars.mH);
   const Real mC2   = power2(pars.mC);
   const Real mA2   = power2(pars.mA);
   const Real mu    = pars.mu;
   const Real mu2   = power2(pars.mu);
   const Real tb    = pars.tb;
   const Real sb    = tb / std::sqrt(1. + power2(tb));
   const Real cb    = 1. / std::sqrt(1. + power2(tb));
   const Real Q2    = power2(pars.Q);
   const Real snt   = calc_sin_theta(mt, xt, mst12, mst22);
   const Real alpha = calc_alpha(mh2, mH2, tb);
   const Real sa    = std::sin(alpha);
   const Real ca    = std::cos(alpha);

   const double result =
   (-3*mu2*(-2 + 3*mA2*(mA2 - mst12 - mst22)*DeltaInv(mA2,mst12,mst22))*log(
     mA2/Q2))/(4.*mA2) + (3*mu2*(-2 + 3*mC2*(mC2 - msb12 - mst22)*DeltaInv(mC2,
     mst22,msb12))*log(mC2/Q2))/(4.*mC2) + (9*ca*cb*mu2*sa*log(mh2/Q2))/(2*mh2*
     sb - 2*mH2*sb) + (9*mu2*(-2 + 3*mH2*(mH2 - mst12 - mst22)*DeltaInv(mst12,
     mH2,mst22))*log(mH2/Q2))/(4.*mH2) + (3*(mC2*mH2 - mA2*(3*mC2 + mH2))*
     mst12*mu2*log(mst12/Q2))/(4.*mA2*mC2*mH2*(mst12 - mst22)) + (3*(-(mC2*mH2)
     + mA2*(3*mC2 + mH2))*mst12*mu2*log(mst12/Q2))/(4.*mA2*mC2*mH2*(mst12 -
     mst22)) + (3*(mC2*mH2 - mA2*(3*mC2 + mH2))*mst22*mu2*log(mst22/Q2))/(4.*
     mA2*mC2*mH2*(mst12 - mst22)) + (3*(-(mC2*mH2) + mA2*(3*mC2 + mH2))*mst22*
     mu2*log(mst22/Q2))/(4.*mA2*mC2*mH2*(mst12 - mst22)) + (3*(mA2*mst22 +
     mst12*mst22 - mst24)*mu2*DeltaInv(mA2,mst12,mst22)*log(mA2/Q2)*log(mst22/
     Q2))/(4.*mA2) - (3*(mC2*mst22 + msb12*mst22 - mst24)*mu2*DeltaInv(mC2,
     mst22,msb12)*log(mC2/Q2)*log(mst22/Q2))/(4.*mC2) - (9*(mH2*mst22 + mst12*
     mst22 - mst24)*mu2*DeltaInv(mst12,mH2,mst22)*log(mH2/Q2)*log(mst22/Q2))/(
     4.*mH2) + (3*msb12*mst22*mu2*DeltaInv(mC2,mst22,msb12)*log(msb12/Q2)*log(
     mst22/Q2))/(2.*mC2) - (9*ca*cb*mst12*mst22*mu2*sa*DeltaInv(mst12,mH2,
     mst22)*log(mst12/Q2)*log(mst22/Q2))/(mH2*sb) + (3*mst12*mst22*mu2*(-(mH2*
     DeltaInv(mA2,mst12,mst22)) + 3*mA2*DeltaInv(mst12,mH2,mst22))*log(mst12/
     Q2)*log(mst22/Q2))/(2.*mA2*mH2) + (3*mu2*(-1 + mA2*(mA2 - mst12 - mst22)*
     DeltaInv(mA2,mst12,mst22))*Fin3(mA2,mst12,mst22,Q2))/(4.*power2(mA2)) + (
     3*mu2*log(mA2/Q2)*log(mst12/Q2)*(-1 + DeltaInv(mA2,mst12,mst22)*(-(mst12*
     mst22) - mA2*(mst12 + 2*mst22) + mst24 + power2(mA2))))/(4.*mA2) - (3*mu2*
     Fin20(mst22,msb12,Q2))/(4.*power2(mC2)) - (3*mu2*log(mC2/Q2)*log(msb12/Q2)
     *(-1 + DeltaInv(mC2,mst22,msb12)*(-(msb12*mst22) - mC2*(msb12 + 2*mst22) +
     mst24 + power2(mC2))))/(4.*mC2) - (3*Fin20(mst12,mst22,Q2)*(-(mu2/power2(
     mA2)) + (3*mu2)/power2(mH2)))/4. + (9*ca*cb*(mh2 - mH2)*mu2*sa*Fin20(
     mst12,mst22,Q2))/(2.*mh2*sb*power2(mH2)) - (9*ca*cb*mu2*sa*(4*mst22*mt2 +
     16*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - mH2*(mst12 - mst22)*(mH2*
     mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(
     mst22/Q2))/(2.*(mst12 - mst22)*sb*power2(mH2)) - (9*ca*cb*mu2*sa*log(mH2/
     Q2)*(-2*mh2*mH2*mst12 + 2*mh2*mH2*mst22 + 12*mh2*mst12*mt2 - 12*mH2*mst12*
     mt2 - 12*mh2*mst22*mt2 + 12*mH2*mst22*mt2 - 48*(mh2 - mH2)*mH2*mst24*mt2*
     DeltaInv(mH2,mst22,mst22) + 48*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)
     + 3*mst12*power2(mH2) - 3*mst22*power2(mH2) + 3*(mh2 - mH2)*(mH2 - mst12 -
     mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) - 48*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2)))/(2.*(mh2 - mH2)*(mst12 -
     mst22)*sb*power2(mH2)) - (9*mu2*log(mH2/Q2)*log(mst12/Q2)*(-1 + DeltaInv(
     mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(
     mH2))))/(4.*mH2) + (9*ca*cb*mu2*sa*log(mH2/Q2)*log(mst12/Q2)*(-(mH2*mst12)
     + mH2*mst22 + 4*mst12*mt2 + 16*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) +
     mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12
     + 2*mst22) + mst24 + power2(mH2))))/(2.*(mst12 - mst22)*sb*power2(mH2)) +
     (9*mu2*power2(sa))/(2.*mH2) + (9*(mH2*(mst12 + mst22 - 2*mt2) + mh2*(mst12
     + mst22 - 2*(mt2 + mu2)))*power2(sa))/(4.*mh2*mH2) - (9*(mst12 + mst22 -
     2*mt2 - mu2)*log(mh2/Q2)*power2(sa))/(2.*(mh2 - mH2)) + (9*mu2*log(mh2/Q2)
     *power2(sa))/(2.*(-mh2 + mH2)) + (9*(mst12 + mst22 - 2*mt2 - mu2)*log(mH2/
     Q2)*power2(sa))/(2.*(mh2 - mH2)) + (9*mu2*log(mH2/Q2)*power2(sa))/(2*mh2 -
     2*mH2) - (9*mst12*(mH2*(mst12 - mst22) + mh2*(mst12 - mst22 - 2*mu2))*log(
     mst12/Q2)*power2(sa))/(4.*mh2*mH2*(mst12 - mst22)) - (9*mst12*mu2*log(
     mst12/Q2)*power2(sa))/(2*mH2*mst12 - 2*mH2*mst22) + (9*mst12*(mst12 -
     mst22 - mu2)*log(mh2/Q2)*log(mst12/Q2)*power2(sa))/(2.*(mh2 - mH2)*(mst12
     - mst22)) + (9*mst12*mu2*log(mh2/Q2)*log(mst12/Q2)*power2(sa))/(2.*(mh2 -
     mH2)*(mst12 - mst22)) - (9*mst12*(mst12 - mst22 - mu2)*log(mH2/Q2)*log(
     mst12/Q2)*power2(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) - (9*mst12*mu2*log(
     mH2/Q2)*log(mst12/Q2)*power2(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) + (9*
     mst22*(mH2*(-mst12 + mst22) + mh2*(-mst12 + mst22 - 2*mu2))*log(mst22/Q2)*
     power2(sa))/(4.*mh2*mH2*(mst12 - mst22)) + (9*mst22*mu2*log(mst22/Q2)*
     power2(sa))/(2*mH2*mst12 - 2*mH2*mst22) - (9*mst22*mu2*log(mh2/Q2)*log(
     mst22/Q2)*power2(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) + (9*mst22*(mst12 -
     mst22 + mu2)*log(mh2/Q2)*log(mst22/Q2)*power2(sa))/(2.*(mh2 - mH2)*(mst12
     - mst22)) + (9*mst22*mu2*log(mH2/Q2)*log(mst22/Q2)*power2(sa))/(2.*(mh2 -
     mH2)*(mst12 - mst22)) - (9*mst22*(mst12 - mst22 + mu2)*log(mH2/Q2)*log(
     mst22/Q2)*power2(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) - (9*mst12*mst22*
     mu2*DeltaInv(mst12,mH2,mst22)*log(mst12/Q2)*log(mst22/Q2)*power2(sa))/mH2
     + (9*(mh2 + mH2)*mt2*log(mt2/Q2)*power2(sa))/(2.*mh2*mH2) + (9*mt2*log(
     mh2/Q2)*log(mt2/Q2)*power2(sa))/(-mh2 + mH2) + (9*(mh2 + mH2)*mt2*log(mh2/
     Q2)*log(mt2/Q2)*power2(sa))/(2.*mh2*(mh2 - mH2)) + (9*mt2*log(mH2/Q2)*log(
     mt2/Q2)*power2(sa))/(mh2 - mH2) - (9*(mh2 + mH2)*mt2*log(mH2/Q2)*log(mt2/
     Q2)*power2(sa))/(2.*(mh2 - mH2)*mH2) - (9*log(mh2/Q2)*log(mst12/Q2)*((8*
     mst14*mt2*DeltaInv(mst12,mst12,mh2))/mh2 + mst12*(1/(mh2 - mH2) + (2*mt2)/
     power2(mh2)))*power2(sa))/2. - (9*log(mh2/Q2)*log(mst22/Q2)*((8*mst24*mt2*
     DeltaInv(mst22,mst22,mh2))/mh2 + mst22*(1/(mh2 - mH2) + (2*mt2)/power2(
     mh2)))*power2(sa))/2. - (9*log(mh2/Q2)*(3*mh2*mH2*mt2 - 6*mh2*mst12*mt2 +
     6*mH2*mst12*mt2 - 6*mh2*mst22*mt2 + 6*mH2*mst22*mt2 - 24*mh2*(mh2 - mH2)*
     mst14*mt2*DeltaInv(mst12,mst12,mh2) - 24*mh2*(mh2 - mH2)*mst24*mt2*
     DeltaInv(mst22,mst22,mh2) + mH2*power2(mh2) - mst12*power2(mh2) - mst22*
     power2(mh2) - mt2*power2(mh2) - mu2*power2(mh2))*power2(sa))/(2.*(mh2 -
     mH2)*power2(mh2)) + (9*(mh2 - mH2)*mu2*Fin20(mst12,mst22,Q2)*power2(sa))/(
     2.*mh2*power2(mH2)) - (9*log(mH2/Q2)*(-3*mh2*mH2*mt2 - 6*mh2*mst12*mt2 +
     6*mH2*mst12*mt2 - 6*mh2*mst22*mt2 + 6*mH2*mst22*mt2 - 2*mh2*mH2*mu2 - 24*(
     mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - 24*mh2*mH2*mst14*mt2*
     DeltaInv(mst12,mst12,mH2) - mh2*power2(mH2) + mst12*power2(mH2) + mst22*
     power2(mH2) + mt2*power2(mH2) + 3*mu2*power2(mH2) + 3*(mh2 - mH2)*(mH2 -
     mst12 - mst22)*mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2) + 24*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power2(mH2))*power2(sa))/(2.*(mh2 - mH2)*power2(
     mH2)) + (9*log(mH2/Q2)*log(mst12/Q2)*(-2*mh2*mst12*mt2 + 2*mH2*mst12*mt2 -
     mh2*mH2*mu2 + 8*mH2*(-mh2 + mH2)*mst14*mt2*DeltaInv(mst12,mst12,mH2) +
     mst12*power2(mH2) + mu2*power2(mH2) + (mh2 - mH2)*mH2*mu2*DeltaInv(mst12,
     mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))
     *power2(sa))/(2.*(mh2 - mH2)*power2(mH2)) + (9*log(mH2/Q2)*log(mst22/Q2)*(
     8*mH2*(-mh2 + mH2)*mst24*mt2*DeltaInv(mH2,mst22,mst22) + (mh2 - mH2)*mH2*(
     mH2*mst22 + mst12*mst22 - mst24)*mu2*DeltaInv(mst12,mH2,mst22) + mst22*(-
     2*mh2*mt2 + 2*mH2*mt2 + power2(mH2)))*power2(sa))/(2.*(mh2 - mH2)*power2(
     mH2)) + (9*(mH2*(mH2 - 10*mt2) + mh2*(mH2 + 2*mt2))*Fin3(mH2,mt2,mt2,Q2)*(
     -1 + power2(sa))*power2(sa))/(4.*(mh2 - mH2)*power2(mH2)) + (3*Fin3(mh2,
     mt2,mt2,Q2)*(-3*mh2*(mH2 - 10*mt2) - 2*mt2*(3*mH2 + 8*mt2) - 3*power2(mh2)
     + 16*mh2*(mh2 - 4*mt2)*DeltaInv(mh2,mt2,mt2)*power2(mt2))*(-1 + power2(sa)
     )*power2(sa))/(4.*(mh2 - mH2)*power2(mh2)) + (9*(mh2 + mH2)*mu2*power2(cb)
     *power2(sa))/(4.*mh2*mH2*power2(sb)) + (9*mu2*log(mh2/Q2)*power2(cb)*
     power2(sa))/(2.*(-mh2 + mH2)*power2(sb)) + (9*mu2*log(mH2/Q2)*power2(cb)*
     power2(sa))/(2.*(mh2 - mH2)*power2(sb)) - (9*(mh2 + mH2)*mst12*mu2*log(
     mst12/Q2)*power2(cb)*power2(sa))/(4.*mh2*mH2*(mst12 - mst22)*power2(sb)) +
     (9*mst12*mu2*log(mh2/Q2)*log(mst12/Q2)*power2(cb)*power2(sa))/(2.*(mh2 -
     mH2)*(mst12 - mst22)*power2(sb)) - (9*mst12*mu2*log(mH2/Q2)*log(mst12/Q2)*
     power2(cb)*power2(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)*power2(sb)) + (9*(
     mh2 + mH2)*mst22*mu2*log(mst22/Q2)*power2(cb)*power2(sa))/(4.*mh2*mH2*(
     mst12 - mst22)*power2(sb)) - (9*mst22*mu2*log(mh2/Q2)*log(mst22/Q2)*
     power2(cb)*power2(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)*power2(sb)) + (9*
     mst22*mu2*log(mH2/Q2)*log(mst22/Q2)*power2(cb)*power2(sa))/(2.*(mh2 - mH2)
     *(mst12 - mst22)*power2(sb)) + (9*mst12*mst22*mu2*(mh2*DeltaInv(mst12,mH2,
     mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*
     power2(cb)*power2(sa))/(2.*mh2*mH2*power2(sb)) + (9*mu2*(-(mh2*(mst12 -
     mst22)*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)) + 8*
     mt2*(mst22 + 4*mh2*mst24*DeltaInv(mst22,mst22,mh2)))*log(mh2/Q2)*log(
     mst22/Q2)*power2(cb)*power2(sa))/(4.*(mst12 - mst22)*power2(mh2)*power2(
     sb)) - (9*mu2*log(mh2/Q2)*log(mst12/Q2)*power2(cb)*(-(mh2*mst12) + mh2*
     mst22 + 8*mst12*mt2 + 32*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2) + mh2*(
     mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*
     mst22) + mst24 + power2(mh2)))*power2(sa))/(4.*(mst12 - mst22)*power2(mh2)
     *power2(sb)) - (9*mu2*log(mh2/Q2)*power2(cb)*((-96*mst14*mt2*DeltaInv(
     mst12,mst12,mh2))/(mh2*mst12 - mh2*mst22) + 3*(-mh2 + mst12 + mst22)*
     DeltaInv(mst12,mst22,mh2) + (96*mh2*(mh2 - mH2)*mst24*mt2*DeltaInv(mst22,
     mst22,mh2) + 2*(mst12 - mst22)*(12*mH2*mt2 - mh2*(mH2 + 12*mt2) + 2*
     power2(mh2)))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)))*power2(sa))/(4.*
     power2(sb)) + (9*mu2*(8*mst22*mt2 + 32*mH2*mst24*mt2*DeltaInv(mH2,mst22,
     mst22) - mH2*(mst12 - mst22)*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(
     mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(cb)*power2(sa))/(4.*(
     mst12 - mst22)*power2(mH2)*power2(sb)) - (9*mu2*Fin20(mst12,mst22,Q2)*
     power2(cb)*power2(mh2 - mH2)*power2(sa))/(4.*power2(mh2)*power2(mH2)*
     power2(sb)) + (9*mu2*log(mH2/Q2)*power2(cb)*(-2*mh2*mH2*mst12 + 2*mh2*mH2*
     mst22 + 24*mh2*mst12*mt2 - 24*mH2*mst12*mt2 - 24*mh2*mst22*mt2 + 24*mH2*
     mst22*mt2 - 96*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) + 96*
     mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + 4*mst12*power2(mH2) - 4*
     mst22*power2(mH2) + 3*(mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mH2) - 96*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power2(mH2))*power2(sa))/(4.*(mh2 - mH2)*(mst12 - mst22)*power2(mH2)*
     power2(sb)) - (9*mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*(-(mH2*mst12) +
     mH2*mst22 + 8*mst12*mt2 + 32*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) +
     mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12
     + 2*mst22) + mst24 + power2(mH2)))*power2(sa))/(4.*(mst12 - mst22)*power2(
     mH2)*power2(sb)) + (3*mu2*(-1 + mC2*(mC2 - msb12 - mst22)*DeltaInv(mC2,
     mst22,msb12))*Fin3(mC2,mst22,msb12,Q2)*(-1 + power2(snt)))/(4.*power2(mC2)
     ) - (9*mu2*((mC2 - msb12 - mst22)*DeltaInv(mC2,mst22,msb12) + (-mC2 +
     msb12 + mst12)*DeltaInv(mst12,mC2,msb12))*log(mC2/Q2)*power2(snt))/4. - (
     3*(mC2*mst12 + msb12*mst12 - mst14)*mu2*DeltaInv(mst12,mC2,msb12)*log(mC2/
     Q2)*log(mst12/Q2)*power2(snt))/(4.*mC2) + (3*msb12*mst12*mu2*DeltaInv(
     mst12,mC2,msb12)*log(msb12/Q2)*log(mst12/Q2)*power2(snt))/(2.*mC2) + (3*(
     mC2*mst22 + msb12*mst22 - mst24)*mu2*DeltaInv(mC2,mst22,msb12)*log(mC2/Q2)
     *log(mst22/Q2)*power2(snt))/(4.*mC2) - (3*msb12*mst22*mu2*DeltaInv(mC2,
     mst22,msb12)*log(msb12/Q2)*log(mst22/Q2)*power2(snt))/(2.*mC2) - (18*
     mst12*mst22*mu2*DeltaInv(mst12,mH2,mst22)*log(mst12/Q2)*log(mst22/Q2)*
     power2(snt))/mH2 + (36*ca*cb*mst12*mst22*mu2*sa*DeltaInv(mst12,mH2,mst22)*
     log(mst12/Q2)*log(mst22/Q2)*power2(snt))/(mH2*sb) - (3*mu2*Fin20(mst12,
     msb12,Q2)*power2(snt))/(4.*power2(mC2)) + (3*mu2*Fin20(mst22,msb12,Q2)*
     power2(snt))/(4.*power2(mC2)) - (3*mu2*(-1 + mC2*(mC2 - msb12 - mst12)*
     DeltaInv(mst12,mC2,msb12))*Fin3(mst12,mC2,msb12,Q2)*power2(snt))/(4.*
     power2(mC2)) - (9*mu2*log(msb12/Q2)*(-(DeltaInv(mst12,mC2,msb12)*(-3*
     msb12*mst12 - mC2*(msb12 + 2*mst12) + mst14 + power2(mC2))) + DeltaInv(
     mC2,mst22,msb12)*(-3*msb12*mst22 - mC2*(msb12 + 2*mst22) + mst24 + power2(
     mC2)))*power2(snt))/(4.*mC2) + (3*mu2*log(mC2/Q2)*log(msb12/Q2)*(DeltaInv(
     mst12,mC2,msb12)*(msb12*mst12 + mC2*(msb12 + 2*mst12) - mst14 - power2(
     mC2)) + DeltaInv(mC2,mst22,msb12)*(-(msb12*mst22) - mC2*(msb12 + 2*mst22)
     + mst24 + power2(mC2)))*power2(snt))/(4.*mC2) + (9*mu2*Fin20(mst12,mst22,
     Q2)*power2(snt))/power2(mH2) + (18*ca*cb*(-mh2 + mH2)*mu2*sa*Fin20(mst12,
     mst22,Q2)*power2(snt))/(mh2*sb*power2(mH2)) + (9*mu2*(-mst22 - 4*mH2*
     mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(snt))/power2(
     mH2) - (18*ca*cb*mu2*sa*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) +
     mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/
     Q2)*log(mst22/Q2)*power2(snt))/(sb*power2(mH2)) - (27*mu2*log(mH2/Q2)*(-
     mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(snt))/power2(mH2) + (54*ca*cb*mu2*sa*log(mH2/
     Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*
     mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,
     mH2,mst22)*power2(mH2))*power2(snt))/(sb*power2(mH2)) + (9*mu2*log(mH2/Q2)
     *log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) +
     mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)))*power2(snt))/power2(mH2) - (18*ca*cb*mu2*sa*log(mH2/
     Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) +
     mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)))*power2(snt))/(sb*power2(mH2)) - (3*mu2*log(mst12/Q2)
     *(-144/mH2 + (3*mst12)/((msntau2 - mst12)*(-msntau2 + mu2)) - (12*(mC2*
     mst12 - msb12*mst12 - mst14)*DeltaInv(mst12,mC2,msb12))/mC2 - (288*mst14*
     DeltaInv(mst12,mst12,mH2))/mH2 + (144*DeltaInv(mst12,mH2,mst22)*(-3*mst12*
     mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (13*mst12)/
     power2(msntau2 - mu2) + (3*mst14)/((msntau2 - mst12)*power2(msntau2 - mu2)
     ) - (3*mst12*mu2)/((msntau2 - mst12)*power2(msntau2 - mu2)))*power2(snt))/
     16. + (3*mu2*log(mst22/Q2)*((-3*mst22)/((msntau2 - mst22)*(msntau2 - mu2))
     - (12*(mC2*mst22 - msb12*mst22 - mst24)*DeltaInv(mC2,mst22,msb12))/mC2 + (
     288*mst24*DeltaInv(mH2,mst22,mst22))/mH2 - 144*mst22*DeltaInv(mst12,mH2,
     mst22) + (144*mst12*mst22*DeltaInv(mst12,mH2,mst22))/mH2 + (144*mst24*
     DeltaInv(mst12,mH2,mst22))/mH2 - (13*mst22)/power2(msntau2 - mu2) + (3*
     mst24)/((msntau2 - mst22)*power2(msntau2 - mu2)) + (3*mst22*mu2)/((-
     msntau2 + mst22)*power2(msntau2 - mu2)))*power2(snt))/16. + (36*mst12*
     mst22*mu2*DeltaInv(mst12,mH2,mst22)*log(mst12/Q2)*log(mst22/Q2)*power2(sa)
     *power2(snt))/mH2 + (18*(-mh2 + mH2)*mu2*Fin20(mst12,mst22,Q2)*power2(sa)*
     power2(snt))/(mh2*power2(mH2)) - (18*mu2*(-mst22 - 4*mH2*mst24*DeltaInv(
     mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,
     mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(sa)*power2(snt))/power2(mH2)
     + (54*mu2*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,
     mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(snt))/
     power2(mH2) - (18*mu2*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*
     mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*
     mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(sa)*power2(
     snt))/power2(mH2) - (18*mst12*mst22*mu2*(mh2*DeltaInv(mst12,mH2,mst22) +
     mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power2(cb)*
     power2(sa)*power2(snt))/(mh2*mH2*power2(sb)) + (9*mu2*(-mst22 + mh2*(mh2*
     mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)
     *power2(snt))/(power2(mh2)*power2(sb)) - (27*mu2*log(mh2/Q2)*power2(cb)*(-
     mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,
     mh2)*power2(mh2))*power2(sa)*power2(snt))/(power2(mh2)*power2(sb)) + (9*
     mu2*log(mh2/Q2)*log(mst12/Q2)*power2(cb)*(-mh2 - mst12 - 4*mh2*mst14*
     DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22)
     - mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))*power2(sa)*power2(snt))/(
     power2(mh2)*power2(sb)) + (9*mu2*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*
     log(mH2/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)*power2(snt))/(power2(mH2)*
     power2(sb)) + (9*mu2*Fin20(mst12,mst22,Q2)*power2(cb)*power2(mh2 - mH2)*
     power2(sa)*power2(snt))/(power2(mh2)*power2(mH2)*power2(sb)) - (27*mu2*
     log(mH2/Q2)*power2(cb)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,
     mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(snt))/(
     power2(mH2)*power2(sb)) + (9*mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*(-
     mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,
     mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))
     *power2(sa)*power2(snt))/(power2(mH2)*power2(sb)) - (9*Fin3(mH2,mst22,
     mst22,Q2)*(sa*(ca*cb*mu2*sb*(-1 + 2*power2(sa))*(mt2 + (mst12 - mst22)*(-1
     + power2(snt))*power2(snt)) - mu2*sa*power2(cb)*(-1 + power2(sa))*(2*mt2 +
     (mst12 - mst22)*(-1 + power2(snt))*power2(snt)) + (mst12 - mst22)*sa*(-1 +
     power2(sa))*power2(sb)*(mt2 + mu2*(-1 + power2(snt))*power2(snt))) - (mh2
     - mH2)*mst22*DeltaInv(mH2,mst22,mst22)*(-1 + power2(sa))*(-2*ca*cb*mu2*sa*
     sb*(mt2 + (mst12 - mst22)*(-1 + power2(snt))*power2(snt)) + mu2*power2(cb)
     *power2(sa)*(2*mt2 + (mst12 - mst22)*(-1 + power2(snt))*power2(snt)) - (
     mst12 - mst22)*power2(sb)*(mt2*power2(sa) + mu2*(-1 + power2(sa))*(-1 +
     power2(snt))*power2(snt)))))/(mH2*(-mh2 + mH2)*(mst12 - mst22)*power2(sb))
     - (9*Fin3(mst12,mst12,mH2,Q2)*(sa*(-(ca*cb*mu2*sb*(-1 + 2*power2(sa))*(mt2
     - (mst12 - mst22)*(-1 + power2(snt))*power2(snt))) + mu2*sa*power2(cb)*(-1
     + power2(sa))*(2*mt2 - (mst12 - mst22)*(-1 + power2(snt))*power2(snt)) + (
     mst12 - mst22)*sa*(-1 + power2(sa))*power2(sb)*(mt2 + mu2*(-1 + power2(
     snt))*power2(snt))) + (mh2 - mH2)*mst12*DeltaInv(mst12,mst12,mH2)*(-1 +
     power2(sa))*(-2*ca*cb*mu2*sa*sb*(mt2 - (mst12 - mst22)*(-1 + power2(snt))*
     power2(snt)) + mu2*power2(cb)*power2(sa)*(2*mt2 - (mst12 - mst22)*(-1 +
     power2(snt))*power2(snt)) + (mst12 - mst22)*power2(sb)*(mt2*power2(sa) +
     mu2*(-1 + power2(sa))*(-1 + power2(snt))*power2(snt)))))/(mH2*(-mh2 + mH2)
     *(mst12 - mst22)*power2(sb)) + (3*sa*Fin3(mst12,mst12,mh2,Q2)*((3*mh2 - 2*
     mst12)*(ca*cb*mu2*sb*(-1 + 2*power2(sa))*(mt2 - (mst12 - mst22)*(-1 +
     power2(snt))*power2(snt)) - mu2*sa*power2(cb)*(-1 + power2(sa))*(2*mt2 - (
     mst12 - mst22)*(-1 + power2(snt))*power2(snt)) - (mst12 - mst22)*sa*(-1 +
     power2(sa))*power2(sb)*(mt2 + mu2*(-1 + power2(snt))*power2(snt))) + mh2*
     DeltaInv(mst12,mst12,mh2)*(-((mh2*mst12 - 3*mH2*mst12 + 8*mst14)*mu2*sa*
     power2(cb)*(-1 + power2(sa))*(-2*mt2 + (mst12 - mst22)*(-1 + power2(snt))*
     power2(snt))) + 2*ca*cb*mu2*sb*(-3*mH2*mst12*power2(sa) + mh2*mst12*(1 +
     power2(sa)) + mst14*(-4 + 8*power2(sa)))*(-mt2 + (mst12 - mst22)*(-1 +
     power2(snt))*power2(snt)) + (mst12 - mst22)*sa*power2(sb)*(8*mst14*(-1 +
     power2(sa))*(mt2 + mu2*(-1 + power2(snt))*power2(snt)) - 3*mH2*mst12*(mt2*
     (-1 + power2(sa)) + mu2*power2(sa)*(-1 + power2(snt))*power2(snt)) + mh2*
     mst12*(mt2*(-1 + power2(sa)) + mu2*(2 + power2(sa))*(-1 + power2(snt))*
     power2(snt))))))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)*power2(sb)) + (
     3*sa*Fin3(mst22,mst22,mh2,Q2)*((3*mh2 - 2*mst22)*(-(ca*cb*mu2*sb*(-1 + 2*
     power2(sa))*(mt2 + (mst12 - mst22)*(-1 + power2(snt))*power2(snt))) + mu2*
     sa*power2(cb)*(-1 + power2(sa))*(2*mt2 + (mst12 - mst22)*(-1 + power2(snt)
     )*power2(snt)) - (mst12 - mst22)*sa*(-1 + power2(sa))*power2(sb)*(mt2 +
     mu2*(-1 + power2(snt))*power2(snt))) + mh2*DeltaInv(mst22,mst22,mh2)*((
     mh2*mst22 - 3*mH2*mst22 + 8*mst24)*mu2*sa*power2(cb)*(-1 + power2(sa))*(-
     2*mt2 - (mst12 - mst22)*(-1 + power2(snt))*power2(snt)) - 2*ca*cb*mu2*sb*(
     -3*mH2*mst22*power2(sa) + mh2*mst22*(1 + power2(sa)) + mst24*(-4 + 8*
     power2(sa)))*(-mt2 - (mst12 - mst22)*(-1 + power2(snt))*power2(snt)) + (
     mst12 - mst22)*sa*power2(sb)*(8*mst24*(-1 + power2(sa))*(mt2 + mu2*(-1 +
     power2(snt))*power2(snt)) - 3*mH2*mst22*(mt2*(-1 + power2(sa)) + mu2*
     power2(sa)*(-1 + power2(snt))*power2(snt)) + mh2*mst22*(mt2*(-1 + power2(
     sa)) + mu2*(2 + power2(sa))*(-1 + power2(snt))*power2(snt))))))/((mh2 -
     mH2)*(mst12 - mst22)*power2(mh2)*power2(sb)) + (3*mu2*(-1 + mA2*(mA2 -
     mst12 - mst22)*DeltaInv(mA2,mst12,mst22))*power2(log(mA2/Q2)))/(8.*mA2) -
     (3*mu2*(-1 + mC2*(mC2 - msb12 - mst22)*DeltaInv(mC2,mst22,msb12))*power2(
     log(mC2/Q2)))/(8.*mC2) + (3*mu2*((mC2 - msb12 - mst22)*DeltaInv(mC2,mst22,
     msb12) + (-mC2 + msb12 + mst12)*DeltaInv(mst12,mC2,msb12))*power2(snt)*
     power2(log(mC2/Q2)))/8. - (9*ca*cb*mu2*sa*power2(log(mh2/Q2)))/(4*mh2*sb -
     4*mH2*sb) - (9*(-1 + (2*mH2)/(-mh2 + mH2) + (2*mt2)/mh2 + (2*mu2)/(mh2 -
     mH2) + (16*mst14*mt2*DeltaInv(mst12,mst12,mh2))/mh2 + (16*mst24*mt2*
     DeltaInv(mst22,mst22,mh2))/mh2 + (4*mst12*mt2)/power2(mh2) + (4*mst22*mt2)
     /power2(mh2))*power2(sa)*power2(log(mh2/Q2)))/8. + (9*mu2*power2(cb)*(1/
     mh2 + 2/(mh2 - mH2) - (32*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*mst12
     - mh2*mst22) + (-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) + (32*
     mst24*mt2*DeltaInv(mst22,mst22,mh2))/(mh2*mst12 - mh2*mst22) + (8*mst22*
     mt2)/((mst12 - mst22)*power2(mh2)) + (8*mst12*mt2)/((-mst12 + mst22)*
     power2(mh2)))*power2(sa)*power2(log(mh2/Q2)))/(8.*power2(sb)) + (9*mu2*
     power2(cb)*(-mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) -
     4*mh2*mst24*DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(
     mst12,mst22,mh2)*power2(mh2))*power2(sa)*power2(snt)*power2(log(mh2/Q2)))/
     (2.*power2(mh2)*power2(sb)) - (9*mu2*(-1 + mH2*(mH2 - mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22))*power2(log(mH2/Q2)))/(8.*mH2) + (9*ca*cb*mu2*
     sa*(mh2*mH2*mst12 - mh2*mH2*mst22 - 4*mh2*mst12*mt2 + 4*mH2*mst12*mt2 + 4*
     mh2*mst22*mt2 - 4*mH2*mst22*mt2 + 16*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(
     mH2,mst22,mst22) - 16*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) - 2*
     mst12*power2(mH2) + 2*mst22*power2(mH2) - (mh2 - mH2)*(mH2 - mst12 -
     mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) + 16*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2))*power2(log(mH2/Q2)))/(4.*(-mh2
     + mH2)*(mst12 - mst22)*sb*power2(mH2)) + (9*mu2*power2(cb)*(-(mh2*mH2*
     mst12) + mh2*mH2*mst22 + 8*mh2*mst12*mt2 - 8*mH2*mst12*mt2 - 8*mh2*mst22*
     mt2 + 8*mH2*mst22*mt2 - 32*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,
     mst22) + 32*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + 3*mst12*power2(
     mH2) - 3*mst22*power2(mH2) + (mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) - 32*mst14*mt2*DeltaInv(
     mst12,mst12,mH2)*power2(mH2))*power2(sa)*power2(log(mH2/Q2)))/(8.*(-mh2 +
     mH2)*(mst12 - mst22)*power2(mH2)*power2(sb)) + (9*mu2*(-mH2 - mst12 -
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power2(snt)*power2(log(mH2/Q2)))/(2.*power2(mH2)) - (9*ca*cb*mu2*sa*
     (-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*
     mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,
     mH2,mst22)*power2(mH2))*power2(snt)*power2(log(mH2/Q2)))/(sb*power2(mH2))
     - (9*mu2*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) -
     4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(snt)*power2(log(mH2/Q2)))/
     power2(mH2) + (9*mu2*power2(cb)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*
     DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 -
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(
     snt)*power2(log(mH2/Q2)))/(2.*power2(mH2)*power2(sb)) + (3*mu2*(-(
     DeltaInv(mst12,mC2,msb12)*(-3*msb12*mst12 - mC2*(msb12 + 2*mst12) + mst14
     + power2(mC2))) + DeltaInv(mC2,mst22,msb12)*(-3*msb12*mst22 - mC2*(msb12 +
     2*mst22) + mst24 + power2(mC2)))*power2(snt)*power2(log(msb12/Q2)))/(8.*
     mC2) + (9*mu2*Fin3(mst12,mH2,mst22,Q2)*(2*ca*cb*sa*sb*(mH2*(2 - 3*power2(
     sa)) + mh2*(-1 + power2(sa))) - (mh2 - 3*mH2)*power2(cb)*(-1 + power2(sa))
     *power2(sa) + (-1 + power2(sa))*(mH2 + mh2*(-1 + power2(sa)) - 3*mH2*
     power2(sa))*power2(sb) + (mh2 - mH2)*mH2*(mH2 - mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*(-1 + power2(sa))*(-2*ca*cb*sa*sb + power2(cb)*power2(sa)
     - (-1 + power2(sa))*power2(sb)))*power2(1 - 2*power2(snt)))/(4.*(mh2 -
     mH2)*power2(mH2)*power2(sb)) + (9*mu2*sa*Fin3(mst12,mst22,mh2,Q2)*(-((3*
     mh2 - mH2)*sa*power2(cb)*(-1 + power2(sa))) - 2*ca*cb*sb*(mh2 - 3*mh2*
     power2(sa) + mH2*power2(sa)) - sa*(mh2*(2 - 3*power2(sa)) + mH2*power2(sa)
     )*power2(sb) + mh2*(mh2 - mH2)*(mh2 - mst12 - mst22)*sa*DeltaInv(mst12,
     mst22,mh2)*(-2*ca*cb*sa*sb + power2(cb)*(-1 + power2(sa)) - power2(sa)*
     power2(sb)))*power2(1 - 2*power2(snt)))/(4.*(mh2 - mH2)*power2(mh2)*
     power2(sb)) - (9*mt2*log(mt2/Q2)*power2(sa)*(15*mH2*mt2*power2(mh2) + mh2*
     (mH2 - 15*mt2)*power2(mH2) + mt2*(3*mH2 + 8*mt2)*power2(mH2) - 8*mh2*(mh2
     - 4*mt2)*DeltaInv(mh2,mt2,mt2)*power2(mH2)*power2(mt2) - (mH2 + 3*mt2)*
     power3(mh2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)) + (3*mt2*power2(sa)*
     power2(log(mt2/Q2))*(60*mH2*mt2*power2(mh2) + 3*mh2*(mH2 - 20*mt2)*power2(
     mH2) + 4*mt2*(3*mH2 + 8*mt2)*power2(mH2) - 32*mh2*(mh2 - 4*mt2)*DeltaInv(
     mh2,mt2,mt2)*power2(mH2)*power2(mt2) - 3*(mH2 + 4*mt2)*power3(mh2)))/(4.*(
     mh2 - mH2)*power2(mh2)*power2(mH2)) + (9*power2(sa)*power2(log(mH2/Q2))*(-
     2*mh2*mH2*mt2 - 4*mh2*mst12*mt2 + 4*mH2*mst12*mt2 - 4*mh2*mst22*mt2 + 4*
     mH2*mst22*mt2 - 2*mh2*mH2*mu2 - 16*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,
     mst22,mst22) - 16*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) - mh2*
     power2(mH2) + 2*mt2*power2(mH2) + 4*mu2*power2(mH2) + 2*(mh2 - mH2)*(mH2 -
     mst12 - mst22)*mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2) + 16*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power2(mH2) - power3(mH2)))/(8.*(mh2 - mH2)*
     power2(mH2)) - (3*(msntau2 - mst22)*mu2*Fin20(mst22,mu2,Q2))/(4.*power3(
     msntau2 - mu2)) - (3*mu2*(-msb12 + mt2 + mu2 + (msntau2 - mu2)*(msb14 +
     mu2*(-mt2 + mu2) - msb12*(mt2 + 2*mu2))*DeltaInv(msb12,mt2,mu2))*Fin3(
     msb12,mt2,mu2,Q2))/(4.*power3(msntau2 - mu2)) + (3*mu2*(-msb12 + msntau2 +
     mt2 - (msb14 + msntau2*(msntau2 - mt2) - msb12*(2*msntau2 + mt2))*(msntau2
     - mu2)*DeltaInv(msntau2,msb12,mt2))*Fin3(msntau2,msb12,mt2,Q2))/(4.*
     power3(msntau2 - mu2)) - (3*msb12*msntau2*mu2*(1 + 2*mt2*(msntau2 - mu2)*
     DeltaInv(msntau2,msb12,mt2))*log(msb12/Q2)*log(msntau2/Q2))/(4.*power3(
     msntau2 - mu2)) - (3*msntau2*mu2*log(msntau2/Q2)*log(mt2/Q2)*(-msntau2 -
     mt2 + mu2 + (msntau2 - mu2)*DeltaInv(msntau2,msb12,mt2)*(msb14 - 2*msb12*
     msntau2 - 3*msb12*mt2 - msntau2*mt2 + power2(msntau2))))/(4.*power3(
     msntau2 - mu2)) + (3*msb12*(1 + 2*mt2*(-msntau2 + mu2)*DeltaInv(msb12,mt2,
     mu2))*log(msb12/Q2)*log(mu2/Q2)*power2(mu2))/(4.*power3(msntau2 - mu2)) -
     (3*(msntau2 - mst12)*mu2*Fin20(mst12,mu2,Q2)*power2(snt))/(4.*power3(
     msntau2 - mu2)) + (3*(msntau2 - mst22)*mu2*Fin20(mst22,mu2,Q2)*power2(snt)
     )/(4.*power3(msntau2 - mu2)) + (3*mst12*log(mst12/Q2)*log(mu2/Q2)*power2(
     mu2)*power2(snt))/(4.*power3(msntau2 - mu2)) + (3*(mst12 - mst22)*power2(
     mu2)*power2(snt)*power2(log(mu2/Q2)))/(8.*power3(msntau2 - mu2)) - (3*mu2*
     Fin20(mst12,msntau2,Q2)*power2(snt)*(mst14*mu2 + 2*(mst12 - mu2)*power2(
     msntau2) - 3*mu2*power2(mst12) - msntau2*(mst14 - 4*mst12*mu2 + 3*power2(
     mst12)) + 2*power3(mst12)))/(8.*power2(msntau2 - mst12)*power3(msntau2 -
     mu2)) - (3*mu2*Fin20(msntau2,mst22,Q2)*(mst24*mu2 + 2*(mst22 - mu2)*
     power2(msntau2) - 3*mu2*power2(mst22) - msntau2*(mst24 - 4*mst22*mu2 + 3*
     power2(mst22)) + 2*power3(mst22)))/(8.*power2(msntau2 - mst22)*power3(
     msntau2 - mu2)) + (3*mu2*Fin20(msntau2,mst22,Q2)*power2(snt)*(mst24*mu2 +
     2*(mst22 - mu2)*power2(msntau2) - 3*mu2*power2(mst22) - msntau2*(mst24 -
     4*mst22*mu2 + 3*power2(mst22)) + 2*power3(mst22)))/(8.*power2(msntau2 -
     mst22)*power3(msntau2 - mu2)) + (3*mu2*(-mst12 + mstau12 + mt2*(1 + (8*(
     mstau12 - mu2)*mu2)/((-mst12 + mst22)*(mstau12 - mstau22))) - (mst14 +
     mstau12*(mstau12 - mt2) - mst12*(2*mstau12 + mt2))*(mstau12 - mu2)*
     DeltaInv(mst12,mstau12,mt2))*Fin3(mst12,mstau12,mt2,Q2))/(4.*power3(
     mstau12 - mu2)) + (3*mstau12*mu2*(2*mst12 + 2*mst22 - 5*mstau12 - 4*mt2 +
     3*mu2 + 3*(mst14 + mstau12*(mstau12 - mt2) - mst12*(2*mstau12 + mt2))*(
     mstau12 - mu2)*DeltaInv(mst12,mstau12,mt2) + 3*(mst24 + mstau12*(mstau12 -
     mt2) - mst22*(2*mstau12 + mt2))*(mstau12 - mu2)*DeltaInv(mst22,mstau12,
     mt2))*log(mstau12/Q2))/(4.*power3(mstau12 - mu2)) - (3*mst12*mstau12*mu2*(
     1 + 2*mt2*(mstau12 - mu2)*DeltaInv(mst12,mstau12,mt2))*log(mst12/Q2)*log(
     mstau12/Q2))/(4.*power3(mstau12 - mu2)) - (3*mst22*mstau12*mu2*(1 + 2*mt2*
     (mstau12 - mu2)*DeltaInv(mst22,mstau12,mt2))*log(mst22/Q2)*log(mstau12/Q2)
     )/(4.*power3(mstau12 - mu2)) - (3*mstau12*mu2*log(mstau12/Q2)*log(mt2/Q2)*
     (-2*(mstau12 + mt2 - mu2) + (mstau12 - mu2)*DeltaInv(mst12,mstau12,mt2)*(
     mst14 - 2*mst12*mstau12 - 3*mst12*mt2 - mstau12*mt2 + power2(mstau12)) + (
     mstau12 - mu2)*DeltaInv(mst22,mstau12,mt2)*(mst24 - 2*mst22*mstau12 - 3*
     mst22*mt2 - mstau12*mt2 + power2(mstau12))))/(4.*power3(mstau12 - mu2)) -
     (3*mstau12*mu2*(mst12 + mst22 - 2*mstau12 - 2*mt2 + (mst14 + mstau12*(
     mstau12 - mt2) - mst12*(2*mstau12 + mt2))*(mstau12 - mu2)*DeltaInv(mst12,
     mstau12,mt2) + (mst24 + mstau12*(mstau12 - mt2) - mst22*(2*mstau12 + mt2))
     *(mstau12 - mu2)*DeltaInv(mst22,mstau12,mt2))*power2(log(mstau12/Q2)))/(8.
     *power3(mstau12 - mu2)) + (3*mu2*(-mst12 + mstau22 + mt2*(1 + (8*(mstau22
     - mu2)*mu2)/((mst12 - mst22)*(mstau12 - mstau22))) - (mst14 + mstau22*(
     mstau22 - mt2) - mst12*(2*mstau22 + mt2))*(mstau22 - mu2)*DeltaInv(
     mstau22,mst12,mt2))*Fin3(mstau22,mst12,mt2,Q2))/(4.*power3(mstau22 - mu2))
     + (3*mstau22*mu2*(2*mst12 + 2*mst22 - 5*mstau22 - 4*mt2 + 3*mu2 + 3*(mst14
     + mstau22*(mstau22 - mt2) - mst12*(2*mstau22 + mt2))*(mstau22 - mu2)*
     DeltaInv(mstau22,mst12,mt2) + 3*(mst24 + mstau22*(mstau22 - mt2) - mst22*(
     2*mstau22 + mt2))*(mstau22 - mu2)*DeltaInv(mstau22,mst22,mt2))*log(
     mstau22/Q2))/(4.*power3(mstau22 - mu2)) - (3*mst12*mstau22*mu2*(1 + 2*mt2*
     (mstau22 - mu2)*DeltaInv(mstau22,mst12,mt2))*log(mst12/Q2)*log(mstau22/Q2)
     )/(4.*power3(mstau22 - mu2)) - (3*mst22*mstau22*mu2*(1 + 2*mt2*(mstau22 -
     mu2)*DeltaInv(mstau22,mst22,mt2))*log(mst22/Q2)*log(mstau22/Q2))/(4.*
     power3(mstau22 - mu2)) - (3*mstau22*mu2*log(mstau22/Q2)*log(mt2/Q2)*(-2*(
     mstau22 + mt2 - mu2) + (mstau22 - mu2)*DeltaInv(mstau22,mst12,mt2)*(mst14
     - 2*mst12*mstau22 - 3*mst12*mt2 - mstau22*mt2 + power2(mstau22)) + (
     mstau22 - mu2)*DeltaInv(mstau22,mst22,mt2)*(mst24 - 2*mst22*mstau22 - 3*
     mst22*mt2 - mstau22*mt2 + power2(mstau22))))/(4.*power3(mstau22 - mu2)) -
     (3*mstau22*mu2*(mst12 + mst22 - 2*mstau22 - 2*mt2 + (mst14 + mstau22*(
     mstau22 - mt2) - mst12*(2*mstau22 + mt2))*(mstau22 - mu2)*DeltaInv(
     mstau22,mst12,mt2) + (mst24 + mstau22*(mstau22 - mt2) - mst22*(2*mstau22 +
     mt2))*(mstau22 - mu2)*DeltaInv(mstau22,mst22,mt2))*power2(log(mstau22/Q2))
     )/(8.*power3(mstau22 - mu2)) + (3*(mst12 - mst22)*log(mu2/Q2)*power2(mu2)*
     power2(snt))/(2.*power3(-msntau2 + mu2)) + (3*mst22*log(mst22/Q2)*log(mu2/
     Q2)*power2(mu2)*power2(snt))/(4.*power3(-msntau2 + mu2)) - (3*mu2*Fin3(
     mst22,mstau12,mt2,Q2)*(-(1/power2(mstau12 - mu2)) + (8*mt2*mu2)/((-mst12 +
     mst22)*(mstau12 - mstau22)*power2(mstau12 - mu2)) + ((mst24 + mstau12*(
     mstau12 - mt2) - mst22*(2*mstau12 + mt2))*DeltaInv(mst22,mstau12,mt2))/
     power2(mstau12 - mu2) + mst22/power3(mstau12 - mu2) + mt2/power3(-mstau12
     + mu2) + mu2/power3(-mstau12 + mu2)))/4. - (3*mu2*Fin3(mst22,mt2,mu2,Q2)*(
     (-8*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (8*mu2)/(
     (mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (DeltaInv(mst22,
     mt2,mu2)*(8*(mstau12 - mu2)*mu2*(-mstau22 + mu2)*(mst24 + mu2*(-mt2 + mu2)
     ) + mst12*(mst24 + mu2*(-mt2 + mu2) - mst22*(mt2 + 2*mu2))*(-2*mstau12*mu2
     - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)) + (
     mt2 + 2*mu2)*power2(mst22)*(-2*mstau12*mu2 - 2*mstau22*mu2 + power2(
     mstau12) + power2(mstau22) + 2*power2(mu2)) - mst22*(mst24*(-2*mstau12*mu2
     - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)) +
     mu2*(2*mstau22*mu2*(13*mt2 + 7*mu2) + (-mt2 + mu2)*power2(mstau12) + (-mt2
     + mu2)*power2(mstau22) - 2*mstau12*(12*mstau22*mt2 + 8*mstau22*mu2 - 13*
     mt2*mu2 - 7*power2(mu2)) - 2*(13*mt2 + 7*mu2)*power2(mu2)))))/((mst12 -
     mst22)*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + mu2/power3(mstau12 -
     mu2) + mt2*((-8*(mstau12 + mstau22 - 2*mu2)*mu2)/((mst12 - mst22)*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + 1/power3(mstau12 - mu2) + 1/
     power3(mstau22 - mu2)) + mu2/power3(mstau22 - mu2) + mst22/power3(-mstau12
     + mu2) + mst22/power3(-mstau22 + mu2)))/4. - (3*mu2*Fin3(mstau22,mst22,
     mt2,Q2)*(-(1/power2(mstau22 - mu2)) + (8*mt2*mu2)/((mst12 - mst22)*(
     mstau12 - mstau22)*power2(mstau22 - mu2)) + ((mst24 + mstau22*(mstau22 -
     mt2) - mst22*(2*mstau22 + mt2))*DeltaInv(mstau22,mst22,mt2))/power2(
     mstau22 - mu2) + mst22/power3(mstau22 - mu2) + mt2/power3(-mstau22 + mu2)
     + mu2/power3(-mstau22 + mu2)))/4. + (3*mu2*Fin3(mst12,mt2,mu2,Q2)*((-8*
     mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (8*mu2)/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (8*mt2*mu2)/((mst12
     - mst22)*(mstau12 - mstau22)*power2(mstau12 - mu2)) + (8*mt2*mu2)/((-mst12
     + mst22)*(mstau12 - mstau22)*power2(mstau22 - mu2)) + (DeltaInv(mst12,mt2,
     mu2)*((mt2 + 2*mu2)*power2(mst12)*(-2*mstau12*mu2 - 2*mstau22*mu2 +
     power2(mstau12) + power2(mstau22) + 2*power2(mu2)) + (mst14 + mu2*(-mt2 +
     mu2))*(8*(mstau12 - mu2)*mu2*(-mstau22 + mu2) + mst22*(-2*mstau12*mu2 - 2*
     mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2))) - mst12*
     (mst14*(-2*mstau12*mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22)
     + 2*power2(mu2)) + mst22*(mt2 + 2*mu2)*(-2*mstau12*mu2 - 2*mstau22*mu2 +
     power2(mstau12) + power2(mstau22) + 2*power2(mu2)) + mu2*(2*mstau22*mu2*(
     13*mt2 + 7*mu2) + (-mt2 + mu2)*power2(mstau12) + (-mt2 + mu2)*power2(
     mstau22) - 2*mstau12*(12*mstau22*mt2 + 8*mstau22*mu2 - 13*mt2*mu2 - 7*
     power2(mu2)) - 2*(13*mt2 + 7*mu2)*power2(mu2)))))/((mst12 - mst22)*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + mst12*(1/power3(mstau12 - mu2) +
     1/power3(mstau22 - mu2)) + mt2/power3(-mstau12 + mu2) + mu2/power3(-
     mstau12 + mu2) + mt2/power3(-mstau22 + mu2) + mu2/power3(-mstau22 + mu2)))
     /4. + (3*power2(mu2)*power2(log(mu2/Q2))*((-8*mst24*mu2*DeltaInv(mst22,
     mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (24*
     mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) + (8*mst24*mu2*DeltaInv(mst22,mt2,mu2))/((mst12
     - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (24*mst22*mt2*mu2*
     DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 -
     mu2)) - 1/power2(msntau2 - mu2) - ((msb14 + mu2*(-mt2 + mu2) - msb12*(mt2
     + 2*mu2))*DeltaInv(msb12,mt2,mu2))/power2(msntau2 - mu2) - (mst24*
     DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (mst22*mt2*DeltaInv(
     mst22,mt2,mu2))/power2(mstau12 - mu2) + (2*mst22*mu2*DeltaInv(mst22,mt2,
     mu2))/power2(mstau12 - mu2) + (mt2*mu2*DeltaInv(mst22,mt2,mu2))/power2(
     mstau12 - mu2) - (mst24*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (
     mst22*mt2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (2*mst22*mu2*
     DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (mt2*mu2*DeltaInv(mst22,
     mt2,mu2))/power2(mstau22 - mu2) + (16*mst22*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (8*
     mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) - (16*mst22*DeltaInv(mst22,mt2,mu2)*power2(mu2))
     /((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (8*mt2*DeltaInv(
     mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) - (DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) + (DeltaInv(
     mst12,mt2,mu2)*((mt2 + 2*mu2)*power2(mst12)*(-2*mstau12*mu2 - 2*mstau22*
     mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)) + (mst14 + mu2*(-
     mt2 + mu2))*(8*(mstau12 - mu2)*mu2*(-mstau22 + mu2) + mst22*(-2*mstau12*
     mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)))
     - mst12*(mst14*(-2*mstau12*mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(
     mstau22) + 2*power2(mu2)) + mst22*(mt2 + 2*mu2)*(-2*mstau12*mu2 - 2*
     mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)) + mu2*(2*
     mstau22*mu2*(13*mt2 + 7*mu2) + (-mt2 + mu2)*power2(mstau12) + (-mt2 + mu2)
     *power2(mstau22) - 2*mstau12*(12*mstau22*mt2 + 8*mstau22*mu2 - 13*mt2*mu2
     - 7*power2(mu2)) - 2*(13*mt2 + 7*mu2)*power2(mu2)))))/((mst12 - mst22)*
     power2(mstau12 - mu2)*power2(mstau22 - mu2)) + msb12/power3(msntau2 - mu2)
     + mst22/power3(msntau2 - mu2) + mst12/power3(mstau12 - mu2) + mst22/
     power3(mstau12 - mu2) + mst12/power3(mstau22 - mu2) + mst22/power3(mstau22
     - mu2) - (8*DeltaInv(mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12
     - mstau22)*(mstau12 - mu2)) + (8*DeltaInv(mst22,mt2,mu2)*power3(mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + mt2/power3(-msntau2
     + mu2) + (2*mu2)/power3(-msntau2 + mu2) + (2*mt2)/power3(-mstau12 + mu2) +
     (2*mu2)/power3(-mstau12 + mu2) + (2*mt2)/power3(-mstau22 + mu2) + (2*mu2)/
     power3(-mstau22 + mu2)))/8. - (3*log(mu2/Q2)*power2(mu2)*((-24*mst24*mu2*
     DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 -
     mu2)) + (72*mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau12 - mu2)) + (24*mst24*mu2*DeltaInv(mst22,mt2,
     mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (72*mst22*
     mt2*mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(
     mstau22 - mu2)) - (3*(msb14 + mu2*(-mt2 + mu2) - msb12*(mt2 + 2*mu2))*
     DeltaInv(msb12,mt2,mu2))/power2(msntau2 - mu2) + 3/power2(mstau12 - mu2) -
     (3*mst24*DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (3*mst22*mt2*
     DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (6*mst22*mu2*DeltaInv(
     mst22,mt2,mu2))/power2(mstau12 - mu2) + (3*mt2*mu2*DeltaInv(mst22,mt2,mu2)
     )/power2(mstau12 - mu2) + 3/power2(mstau22 - mu2) - (3*mst24*DeltaInv(
     mst22,mt2,mu2))/power2(mstau22 - mu2) + (3*mst22*mt2*DeltaInv(mst22,mt2,
     mu2))/power2(mstau22 - mu2) + (6*mst22*mu2*DeltaInv(mst22,mt2,mu2))/
     power2(mstau22 - mu2) + (3*mt2*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau22
     - mu2) + (48*mst22*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau12 - mu2)) + (24*mt2*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (48*
     mst22*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) - (24*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/(
     (mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (3*DeltaInv(mst22,
     mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (3*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/power2(mstau22 - mu2) + (3*DeltaInv(mst12,mt2,mu2)*((mt2 + 2*
     mu2)*power2(mst12)*(-2*mstau12*mu2 - 2*mstau22*mu2 + power2(mstau12) +
     power2(mstau22) + 2*power2(mu2)) + (mst14 + mu2*(-mt2 + mu2))*(8*(mstau12
     - mu2)*mu2*(-mstau22 + mu2) + mst22*(-2*mstau12*mu2 - 2*mstau22*mu2 +
     power2(mstau12) + power2(mstau22) + 2*power2(mu2))) - mst12*(mst14*(-2*
     mstau12*mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*
     power2(mu2)) + mst22*(mt2 + 2*mu2)*(-2*mstau12*mu2 - 2*mstau22*mu2 +
     power2(mstau12) + power2(mstau22) + 2*power2(mu2)) + mu2*(2*mstau22*mu2*(
     13*mt2 + 7*mu2) + (-mt2 + mu2)*power2(mstau12) + (-mt2 + mu2)*power2(
     mstau22) - 2*mstau12*(12*mstau22*mt2 + 8*mstau22*mu2 - 13*mt2*mu2 - 7*
     power2(mu2)) - 2*(13*mt2 + 7*mu2)*power2(mu2)))))/((mst12 - mst22)*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (2*msb12)/power3(msntau2 - mu2) +
     (2*mst22)/power3(msntau2 - mu2) + (2*mst12)/power3(mstau12 - mu2) + (2*
     mst22)/power3(mstau12 - mu2) + (2*mst12)/power3(mstau22 - mu2) + (2*mst22)
     /power3(mstau22 - mu2) - (24*DeltaInv(mst22,mt2,mu2)*power3(mu2))/((mst12
     - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (24*DeltaInv(mst22,mt2,
     mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) +
     (2*mt2)/power3(-msntau2 + mu2) + (2*mu2)/power3(-msntau2 + mu2) + (4*mt2)/
     power3(-mstau12 + mu2) + (2*mu2)/power3(-mstau12 + mu2) + (4*mt2)/power3(-
     mstau22 + mu2) + (2*mu2)/power3(-mstau22 + mu2)))/4. + (9*ca*cb*mst12*
     mst22*mu2*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*
     log(mst12/Q2)*log(mst22/Q2)*power3(sa))/(mh2*mH2*sb) + (9*ca*cb*mu2*(-(
     mh2*(mst12 - mst22)*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,
     mst22,mh2)) + 4*mt2*(mst22 + 4*mh2*mst24*DeltaInv(mst22,mst22,mh2)))*log(
     mh2/Q2)*log(mst22/Q2)*power3(sa))/(2.*(mst12 - mst22)*sb*power2(mh2)) - (
     9*ca*cb*mu2*log(mh2/Q2)*log(mst12/Q2)*(-(mh2*mst12) + mh2*mst22 + 4*mst12*
     mt2 + 16*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2) + mh2*(mst12 - mst22)*
     DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24
     + power2(mh2)))*power3(sa))/(2.*(mst12 - mst22)*sb*power2(mh2)) + (9*ca*
     cb*mu2*log(mh2/Q2)*(48*mh2*(mh2 - mH2)*mst14*mt2*DeltaInv(mst12,mst12,mh2)
     + 3*(mh2 - mH2)*(mh2 - mst12 - mst22)*(mst12 - mst22)*DeltaInv(mst12,
     mst22,mh2)*power2(mh2) - 2*(24*mh2*(mh2 - mH2)*mst24*mt2*DeltaInv(mst22,
     mst22,mh2) + (mst12 - mst22)*(6*mH2*mt2 - mh2*(mH2 + 6*mt2) + 2*power2(
     mh2))))*power3(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)*sb*power2(mh2)) + (9*
     ca*cb*mu2*(4*mst22*mt2 + 16*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - mH2*
     (mst12 - mst22)*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22))*log(mH2/Q2)*log(mst22/Q2)*power3(sa))/(2.*(mst12 - mst22)*sb*
     power2(mH2)) - (9*ca*cb*mu2*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*
     power3(sa))/(2.*sb*power2(mh2)*power2(mH2)) + (9*ca*cb*mu2*log(mH2/Q2)*(-
     48*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - 2*((mst12 -
     mst22)*(mh2*(mH2 - 6*mt2) - 2*mH2*(mH2 - 3*mt2)) - 24*(mh2 - mH2)*mH2*
     mst14*mt2*DeltaInv(mst12,mst12,mH2)) + 3*(mh2 - mH2)*(mH2 - mst12 - mst22)
     *(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power3(sa))/(2.*(
     mh2 - mH2)*(mst12 - mst22)*sb*power2(mH2)) - (9*ca*cb*mu2*log(mH2/Q2)*log(
     mst12/Q2)*(-(mH2*mst12) + mH2*mst22 + 4*mst12*mt2 + 16*mH2*mst14*mt2*
     DeltaInv(mst12,mst12,mH2) + mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*
     (-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power3(sa)
     )/(2.*(mst12 - mst22)*sb*power2(mH2)) - (36*ca*cb*mst12*mst22*mu2*(mh2*
     DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*
     log(mst22/Q2)*power2(snt)*power3(sa))/(mh2*mH2*sb) + (18*ca*cb*mu2*(-mst22
     + mh2*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*
     mst24*DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power2(snt)*
     power3(sa))/(sb*power2(mh2)) - (54*ca*cb*mu2*log(mh2/Q2)*(-mh2 - mst12 -
     mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(
     mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(
     mh2))*power2(snt)*power3(sa))/(sb*power2(mh2)) + (18*ca*cb*mu2*log(mh2/Q2)
     *log(mst12/Q2)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) +
     mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) +
     mst24 + power2(mh2)))*power2(snt)*power3(sa))/(sb*power2(mh2)) + (18*ca*
     cb*mu2*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*
     power2(snt)*power3(sa))/(sb*power2(mH2)) + (18*ca*cb*mu2*Fin20(mst12,
     mst22,Q2)*power2(mh2 - mH2)*power2(snt)*power3(sa))/(sb*power2(mh2)*
     power2(mH2)) - (54*ca*cb*mu2*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*
     mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) +
     (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(snt)*
     power3(sa))/(sb*power2(mH2)) + (18*ca*cb*mu2*log(mH2/Q2)*log(mst12/Q2)*(-
     mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,
     mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))
     *power2(snt)*power3(sa))/(sb*power2(mH2)) + (9*ca*cb*mu2*(1/mh2 + 2/(mh2 -
     mH2) - (16*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*mst12 - mh2*mst22) +
     (-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) + (16*mst24*mt2*DeltaInv(
     mst22,mst22,mh2))/(mh2*mst12 - mh2*mst22) + (4*mst22*mt2)/((mst12 - mst22)
     *power2(mh2)) + (4*mst12*mt2)/((-mst12 + mst22)*power2(mh2)))*power2(log(
     mh2/Q2))*power3(sa))/(4.*sb) + (9*ca*cb*mu2*(-mh2 - mst12 - mst22 - 4*mh2*
     mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) +
     (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(snt)*
     power2(log(mh2/Q2))*power3(sa))/(sb*power2(mh2)) + (9*ca*cb*mu2*(-(mh2*
     mH2*mst12) + mh2*mH2*mst22 + 4*mh2*mst12*mt2 - 4*mH2*mst12*mt2 - 4*mh2*
     mst22*mt2 + 4*mH2*mst22*mt2 - 16*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,
     mst22,mst22) + 16*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + 3*mst12*
     power2(mH2) - 3*mst22*power2(mH2) + (mh2 - mH2)*(mH2 - mst12 - mst22)*(
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) - 16*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power2(mH2))*power2(log(mH2/Q2))*power3(sa))/(4.
     *(-mh2 + mH2)*(mst12 - mst22)*sb*power2(mH2)) + (9*ca*cb*mu2*(-mH2 - mst12
     - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power2(snt)*power2(log(mH2/Q2))*power3(sa))/(sb*power2(mH2)) - (9*(
     mh2 + mH2)*(mst12 + mst22 - 2*mt2 - mu2)*power4(sa))/(4.*mh2*mH2) - (9*(
     mh2 + mH2)*mu2*power4(sa))/(4.*mh2*mH2) + (9*(mst12 + mst22 - 2*mt2 - mu2)
     *log(mh2/Q2)*power4(sa))/(2.*(mh2 - mH2)) + (9*mu2*log(mh2/Q2)*power4(sa))
     /(2*mh2 - 2*mH2) - (9*(mst12 + mst22 - 2*mt2 - mu2)*log(mH2/Q2)*power4(sa)
     )/(2.*(mh2 - mH2)) - (9*mu2*log(mH2/Q2)*power4(sa))/(2*mh2 - 2*mH2) + (9*(
     mh2 + mH2)*mst12*(mst12 - mst22 - mu2)*log(mst12/Q2)*power4(sa))/(4.*mh2*
     mH2*(mst12 - mst22)) + (9*(mh2 + mH2)*mst12*mu2*log(mst12/Q2)*power4(sa))/
     (4.*mh2*mH2*(mst12 - mst22)) - (9*mst12*(mst12 - mst22 - mu2)*log(mh2/Q2)*
     log(mst12/Q2)*power4(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) - (9*mst12*mu2*
     log(mh2/Q2)*log(mst12/Q2)*power4(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) + (
     9*mst12*(mst12 - mst22 - mu2)*log(mH2/Q2)*log(mst12/Q2)*power4(sa))/(2.*(
     mh2 - mH2)*(mst12 - mst22)) + (9*mst12*mu2*log(mH2/Q2)*log(mst12/Q2)*
     power4(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) - (9*(mh2 + mH2)*mst22*mu2*
     log(mst22/Q2)*power4(sa))/(4.*mh2*mH2*(mst12 - mst22)) + (9*(mh2 + mH2)*
     mst22*(mst12 - mst22 + mu2)*log(mst22/Q2)*power4(sa))/(4.*mh2*mH2*(mst12 -
     mst22)) + (9*mst22*mu2*log(mh2/Q2)*log(mst22/Q2)*power4(sa))/(2.*(mh2 -
     mH2)*(mst12 - mst22)) - (9*mst22*(mst12 - mst22 + mu2)*log(mh2/Q2)*log(
     mst22/Q2)*power4(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) - (9*mst22*mu2*log(
     mH2/Q2)*log(mst22/Q2)*power4(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)) + (9*
     mst22*(mst12 - mst22 + mu2)*log(mH2/Q2)*log(mst22/Q2)*power4(sa))/(2.*(mh2
     - mH2)*(mst12 - mst22)) + (9*mst12*mst22*mu2*(mh2*DeltaInv(mst12,mH2,
     mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*
     power4(sa))/(2.*mh2*mH2) - (9*(mh2 + mH2)*mt2*log(mt2/Q2)*power4(sa))/(2.*
     mh2*mH2) + (9*mt2*log(mh2/Q2)*log(mt2/Q2)*power4(sa))/(mh2 - mH2) - (9*(
     mh2 + mH2)*mt2*log(mh2/Q2)*log(mt2/Q2)*power4(sa))/(2.*mh2*(mh2 - mH2)) -
     (9*mt2*log(mH2/Q2)*log(mt2/Q2)*power4(sa))/(mh2 - mH2) + (9*(mh2 + mH2)*
     mt2*log(mH2/Q2)*log(mt2/Q2)*power4(sa))/(2.*(mh2 - mH2)*mH2) - (9*log(mh2/
     Q2)*log(mst22/Q2)*(((mh2*mst22 + mst12*mst22 - mst24)*mu2*DeltaInv(mst12,
     mst22,mh2))/mh2 - (16*mst24*mt2*DeltaInv(mst22,mst22,mh2))/mh2 + 2*mst22*(
     1/(-mh2 + mH2) - (2*mt2)/power2(mh2)))*power4(sa))/4. + (9*log(mh2/Q2)*
     log(mst12/Q2)*((2*mst12)/(mh2 - mH2) + mu2/mh2 + (16*mst14*mt2*DeltaInv(
     mst12,mst12,mh2))/mh2 + (4*mst12*mt2)/power2(mh2) - (mu2*DeltaInv(mst12,
     mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))
     /mh2)*power4(sa))/4. + (9*log(mh2/Q2)*(-48*mh2*(mh2 - mH2)*mst14*mt2*
     DeltaInv(mst12,mst12,mh2) + 3*(mh2 - mH2)*(mh2 - mst12 - mst22)*mu2*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) + 2*(6*mH2*(mst12 + mst22)*mt2 +
     mh2*(-6*(mst12 + mst22)*mt2 + mH2*(3*mt2 + mu2)) - 24*mh2*(mh2 - mH2)*
     mst24*mt2*DeltaInv(mst22,mst22,mh2) + (mH2 - mst12 - mst22 - mt2 - 2*mu2)*
     power2(mh2)))*power4(sa))/(4.*(mh2 - mH2)*power2(mh2)) - (9*mu2*Fin20(
     mst12,mst22,Q2)*power2(mh2 - mH2)*power4(sa))/(4.*power2(mh2)*power2(mH2))
     + (9*log(mH2/Q2)*log(mst12/Q2)*(4*mh2*mst12*mt2 - 4*mH2*mst12*mt2 + mh2*
     mH2*mu2 + 16*(mh2 - mH2)*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) - 2*
     mst12*power2(mH2) - mu2*power2(mH2) - (mh2 - mH2)*mH2*mu2*DeltaInv(mst12,
     mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))
     *power4(sa))/(4.*(mh2 - mH2)*power2(mH2)) + (9*log(mH2/Q2)*log(mst22/Q2)*(
     16*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - (mh2 - mH2)*mH2*(
     mH2*mst22 + mst12*mst22 - mst24)*mu2*DeltaInv(mst12,mH2,mst22) - 2*mst22*(
     -2*mh2*mt2 + 2*mH2*mt2 + power2(mH2)))*power4(sa))/(4.*(mh2 - mH2)*power2(
     mH2)) + (9*log(mH2/Q2)*(-48*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,
     mst22) + 3*(mh2 - mH2)*(mH2 - mst12 - mst22)*mu2*DeltaInv(mst12,mH2,mst22)
     *power2(mH2) - 2*(-(mH2*(6*(mst12 + mst22)*mt2 + mH2*(mst12 + mst22 + mt2
     + 2*mu2))) + 24*(mh2 - mH2)*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + mh2*
     (6*(mst12 + mst22)*mt2 + mH2*(3*mt2 + mu2) + power2(mH2))))*power4(sa))/(
     4.*(mh2 - mH2)*power2(mH2)) - (9*(mh2 + mH2)*mu2*power2(cb)*power4(sa))/(
     4.*mh2*mH2*power2(sb)) + (9*mu2*log(mh2/Q2)*power2(cb)*power4(sa))/(2.*(
     mh2 - mH2)*power2(sb)) - (9*mu2*log(mH2/Q2)*power2(cb)*power4(sa))/(2.*(
     mh2 - mH2)*power2(sb)) + (9*(mh2 + mH2)*mst12*mu2*log(mst12/Q2)*power2(cb)
     *power4(sa))/(4.*mh2*mH2*(mst12 - mst22)*power2(sb)) - (9*mst12*mu2*log(
     mh2/Q2)*log(mst12/Q2)*power2(cb)*power4(sa))/(2.*(mh2 - mH2)*(mst12 -
     mst22)*power2(sb)) + (9*mst12*mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*
     power4(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)*power2(sb)) - (9*(mh2 + mH2)*
     mst22*mu2*log(mst22/Q2)*power2(cb)*power4(sa))/(4.*mh2*mH2*(mst12 - mst22)
     *power2(sb)) + (9*mst22*mu2*log(mh2/Q2)*log(mst22/Q2)*power2(cb)*power4(
     sa))/(2.*(mh2 - mH2)*(mst12 - mst22)*power2(sb)) - (9*mst22*mu2*log(mH2/
     Q2)*log(mst22/Q2)*power2(cb)*power4(sa))/(2.*(mh2 - mH2)*(mst12 - mst22)*
     power2(sb)) - (9*mst12*mst22*mu2*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*
     DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power2(cb)*power4(
     sa))/(2.*mh2*mH2*power2(sb)) - (9*mu2*(-(mh2*(mst12 - mst22)*(mh2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)) + 8*mt2*(mst22 + 4*mh2*
     mst24*DeltaInv(mst22,mst22,mh2)))*log(mh2/Q2)*log(mst22/Q2)*power2(cb)*
     power4(sa))/(4.*(mst12 - mst22)*power2(mh2)*power2(sb)) + (9*mu2*log(mh2/
     Q2)*log(mst12/Q2)*power2(cb)*(-(mh2*mst12) + mh2*mst22 + 8*mst12*mt2 + 32*
     mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2) + mh2*(mst12 - mst22)*DeltaInv(
     mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(
     mh2)))*power4(sa))/(4.*(mst12 - mst22)*power2(mh2)*power2(sb)) + (9*mu2*
     log(mh2/Q2)*power2(cb)*((-96*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*
     mst12 - mh2*mst22) + 3*(-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) +
     (96*mh2*(mh2 - mH2)*mst24*mt2*DeltaInv(mst22,mst22,mh2) + 2*(mst12 -
     mst22)*(12*mH2*mt2 - mh2*(mH2 + 12*mt2) + 2*power2(mh2)))/((mh2 - mH2)*(
     mst12 - mst22)*power2(mh2)))*power4(sa))/(4.*power2(sb)) - (9*mu2*(8*
     mst22*mt2 + 32*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - mH2*(mst12 -
     mst22)*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(
     mH2/Q2)*log(mst22/Q2)*power2(cb)*power4(sa))/(4.*(mst12 - mst22)*power2(
     mH2)*power2(sb)) + (9*mu2*Fin20(mst12,mst22,Q2)*power2(cb)*power2(mh2 -
     mH2)*power4(sa))/(4.*power2(mh2)*power2(mH2)*power2(sb)) - (9*mu2*log(mH2/
     Q2)*power2(cb)*(-2*mh2*mH2*mst12 + 2*mh2*mH2*mst22 + 24*mh2*mst12*mt2 -
     24*mH2*mst12*mt2 - 24*mh2*mst22*mt2 + 24*mH2*mst22*mt2 - 96*(mh2 - mH2)*
     mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) + 96*mh2*mH2*mst14*mt2*DeltaInv(
     mst12,mst12,mH2) + 4*mst12*power2(mH2) - 4*mst22*power2(mH2) + 3*(mh2 -
     mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*
     power2(mH2) - 96*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2))*power4(
     sa))/(4.*(mh2 - mH2)*(mst12 - mst22)*power2(mH2)*power2(sb)) + (9*mu2*log(
     mH2/Q2)*log(mst12/Q2)*power2(cb)*(-(mH2*mst12) + mH2*mst22 + 8*mst12*mt2 +
     32*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + mH2*(mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(
     mH2)))*power4(sa))/(4.*(mst12 - mst22)*power2(mH2)*power2(sb)) - (18*
     mst12*mst22*mu2*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,
     mh2))*log(mst12/Q2)*log(mst22/Q2)*power2(snt)*power4(sa))/(mh2*mH2) + (9*
     mu2*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,
     mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*
     power2(snt)*power4(sa))/power2(mh2) - (27*mu2*log(mh2/Q2)*(-mh2 - mst12 -
     mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(
     mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(
     mh2))*power2(snt)*power4(sa))/power2(mh2) + (9*mu2*log(mh2/Q2)*log(mst12/
     Q2)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(
     mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(
     mh2)))*power2(snt)*power4(sa))/power2(mh2) + (9*mu2*(-mst22 - 4*mH2*mst24*
     DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(snt)*power4(
     sa))/power2(mH2) + (9*mu2*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*power2(
     snt)*power4(sa))/(power2(mh2)*power2(mH2)) - (27*mu2*log(mH2/Q2)*(-mH2 -
     mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(snt)*power4(sa))/power2(mH2) + (9*mu2*log(mH2/
     Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) +
     mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)))*power2(snt)*power4(sa))/power2(mH2) + (18*mst12*
     mst22*mu2*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*
     log(mst12/Q2)*log(mst22/Q2)*power2(cb)*power2(snt)*power4(sa))/(mh2*mH2*
     power2(sb)) - (9*mu2*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*log(
     mh2/Q2)*log(mst22/Q2)*power2(cb)*power2(snt)*power4(sa))/(power2(mh2)*
     power2(sb)) + (27*mu2*log(mh2/Q2)*power2(cb)*(-mh2 - mst12 - mst22 - 4*
     mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power2(snt)*power4(sa))/(power2(mh2)*power2(sb)) - (9*mu2*log(mh2/Q2)*log(
     mst12/Q2)*power2(cb)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     + mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) +
     mst24 + power2(mh2)))*power2(snt)*power4(sa))/(power2(mh2)*power2(sb)) - (
     9*mu2*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*
     power2(cb)*power2(snt)*power4(sa))/(power2(mH2)*power2(sb)) - (9*mu2*
     Fin20(mst12,mst22,Q2)*power2(cb)*power2(mh2 - mH2)*power2(snt)*power4(sa))
     /(power2(mh2)*power2(mH2)*power2(sb)) + (27*mu2*log(mH2/Q2)*power2(cb)*(-
     mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(snt)*power4(sa))/(power2(mH2)*power2(sb)) - (9*
     mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*(-mH2 - mst12 - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22)
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(snt)*power4(sa))/(
     power2(mH2)*power2(sb)) + (9*(-1 + (2*mH2)/(-mh2 + mH2) + (2*mt2)/mh2 +
     mu2/mh2 + (2*mu2)/(mh2 - mH2) + (16*mst14*mt2*DeltaInv(mst12,mst12,mh2))/
     mh2 + (-mh2 + mst12 + mst22)*mu2*DeltaInv(mst12,mst22,mh2) + (16*mst24*
     mt2*DeltaInv(mst22,mst22,mh2))/mh2 + (4*mst12*mt2)/power2(mh2) + (4*mst22*
     mt2)/power2(mh2))*power2(log(mh2/Q2))*power4(sa))/8. - (9*mu2*power2(cb)*(
     1/mh2 + 2/(mh2 - mH2) - (32*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*
     mst12 - mh2*mst22) + (-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) + (
     32*mst24*mt2*DeltaInv(mst22,mst22,mh2))/(mh2*mst12 - mh2*mst22) + (8*
     mst22*mt2)/((mst12 - mst22)*power2(mh2)) + (8*mst12*mt2)/((-mst12 + mst22)
     *power2(mh2)))*power2(log(mh2/Q2))*power4(sa))/(8.*power2(sb)) + (9*mu2*(-
     mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,
     mh2)*power2(mh2))*power2(snt)*power2(log(mh2/Q2))*power4(sa))/(2.*power2(
     mh2)) - (9*mu2*power2(cb)*(-mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(
     mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 -
     mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(snt)*power2(log(mh2/
     Q2))*power4(sa))/(2.*power2(mh2)*power2(sb)) + (9*mu2*power2(cb)*(mh2*mH2*
     mst12 - mh2*mH2*mst22 - 8*mh2*mst12*mt2 + 8*mH2*mst12*mt2 + 8*mh2*mst22*
     mt2 - 8*mH2*mst22*mt2 + 32*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,
     mst22) - 32*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) - 3*mst12*power2(
     mH2) + 3*mst22*power2(mH2) - (mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) + 32*mst14*mt2*DeltaInv(
     mst12,mst12,mH2)*power2(mH2))*power2(log(mH2/Q2))*power4(sa))/(8.*(-mh2 +
     mH2)*(mst12 - mst22)*power2(mH2)*power2(sb)) + (9*mu2*(-mH2 - mst12 -
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power2(snt)*power2(log(mH2/Q2))*power4(sa))/(2.*power2(mH2)) - (9*
     mu2*power2(cb)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(snt)*power2(log(mH2/Q2))*
     power4(sa))/(2.*power2(mH2)*power2(sb)) + (9*mt2*log(mt2/Q2)*(15*mH2*mt2*
     power2(mh2) + mh2*(mH2 - 15*mt2)*power2(mH2) + mt2*(3*mH2 + 8*mt2)*power2(
     mH2) - 8*mh2*(mh2 - 4*mt2)*DeltaInv(mh2,mt2,mt2)*power2(mH2)*power2(mt2) -
     (mH2 + 3*mt2)*power3(mh2))*power4(sa))/((mh2 - mH2)*power2(mh2)*power2(
     mH2)) + (3*mt2*power2(log(mt2/Q2))*(-60*mH2*mt2*power2(mh2) - 3*mh2*(mH2 -
     20*mt2)*power2(mH2) - 4*mt2*(3*mH2 + 8*mt2)*power2(mH2) + 32*mh2*(mh2 - 4*
     mt2)*DeltaInv(mh2,mt2,mt2)*power2(mH2)*power2(mt2) + 3*(mH2 + 4*mt2)*
     power3(mh2))*power4(sa))/(4.*(mh2 - mH2)*power2(mh2)*power2(mH2)) - (9*
     power2(log(mH2/Q2))*(-2*mh2*mH2*mt2 - 4*mh2*mst12*mt2 + 4*mH2*mst12*mt2 -
     4*mh2*mst22*mt2 + 4*mH2*mst22*mt2 - mh2*mH2*mu2 - 16*(mh2 - mH2)*mH2*
     mst24*mt2*DeltaInv(mH2,mst22,mst22) - 16*mh2*mH2*mst14*mt2*DeltaInv(mst12,
     mst12,mH2) - mh2*power2(mH2) + 2*mt2*power2(mH2) + 3*mu2*power2(mH2) + (
     mh2 - mH2)*(mH2 - mst12 - mst22)*mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2)
     + 16*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2) - power3(mH2))*
     power4(sa))/(8.*(mh2 - mH2)*power2(mH2)) - (27*mu2*(2*mst24*DeltaInv(mH2,
     mst22,mst22) + (-(mH2*mst22) + mst12*mst22 + mst24)*DeltaInv(mst12,mH2,
     mst22))*log(mst22/Q2)*power4(snt))/mH2 + (18*mst12*mst22*mu2*DeltaInv(
     mst12,mH2,mst22)*log(mst12/Q2)*log(mst22/Q2)*power4(snt))/mH2 - (36*ca*cb*
     mst12*mst22*mu2*sa*DeltaInv(mst12,mH2,mst22)*log(mst12/Q2)*log(mst22/Q2)*
     power4(snt))/(mH2*sb) - (9*mu2*Fin20(mst12,mst22,Q2)*power4(snt))/power2(
     mH2) + (18*ca*cb*(mh2 - mH2)*mu2*sa*Fin20(mst12,mst22,Q2)*power4(snt))/(
     mh2*sb*power2(mH2)) - (9*mu2*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*
     log(mH2/Q2)*log(mst22/Q2)*power4(snt))/power2(mH2) + (18*ca*cb*mu2*sa*(-
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*
     power4(snt))/(sb*power2(mH2)) + (27*mu2*log(mH2/Q2)*(-mH2 - mst12 - mst22
     - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,
     mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*
     power4(snt))/power2(mH2) - (54*ca*cb*mu2*sa*log(mH2/Q2)*(-mH2 - mst12 -
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power4(snt))/(sb*power2(mH2)) - (63*mu2*(-2*mH2 - mst12 - mst22 - 6*
     mH2*mst24*DeltaInv(mH2,mst22,mst22) - 6*mH2*mst14*DeltaInv(mst12,mst12,
     mH2) + 2*mH2*DeltaInv(mst12,mH2,mst22)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)))*power4(snt))/(2.*power2(mH2)) + (27*mu2*log(mst12/
     Q2)*(-1 - 2*mst14*DeltaInv(mst12,mst12,mH2) + DeltaInv(mst12,mH2,mst22)*(-
     3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power4(snt))
     /mH2 - (9*mu2*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22)
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power4(snt))/power2(mH2) +
     (18*ca*cb*mu2*sa*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22)
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power4(snt))/(sb*power2(
     mH2)) - (3*mu2*(-2*mH2 - mst12 - mst22 - 6*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) - 6*mH2*mst14*DeltaInv(mst12,mst12,mH2) + 2*mH2*DeltaInv(mst12,mH2,
     mst22)*(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)))*power2(Pi)*
     power4(snt))/(4.*power2(mH2)) - (36*mst12*mst22*mu2*DeltaInv(mst12,mH2,
     mst22)*log(mst12/Q2)*log(mst22/Q2)*power2(sa)*power4(snt))/mH2 + (18*(mh2
     - mH2)*mu2*Fin20(mst12,mst22,Q2)*power2(sa)*power4(snt))/(mh2*power2(mH2))
     + (18*mu2*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22
     + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/
     Q2)*power2(sa)*power4(snt))/power2(mH2) - (54*mu2*log(mH2/Q2)*(-mH2 -
     mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(sa)*power4(snt))/power2(mH2) + (18*mu2*log(mH2/
     Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) +
     mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)))*power2(sa)*power4(snt))/power2(mH2) + (18*mst12*
     mst22*mu2*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*
     log(mst12/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)*power4(snt))/(mh2*mH2*
     power2(sb)) - (9*mu2*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*log(
     mh2/Q2)*log(mst22/Q2)*power2(cb)*power2(sa)*power4(snt))/(power2(mh2)*
     power2(sb)) + (27*mu2*log(mh2/Q2)*power2(cb)*(-mh2 - mst12 - mst22 - 4*
     mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power2(sa)*power4(snt))/(power2(mh2)*power2(sb)) - (9*mu2*log(mh2/Q2)*log(
     mst12/Q2)*power2(cb)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     + mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) +
     mst24 + power2(mh2)))*power2(sa)*power4(snt))/(power2(mh2)*power2(sb)) - (
     9*mu2*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*
     power2(cb)*power2(sa)*power4(snt))/(power2(mH2)*power2(sb)) - (9*mu2*
     Fin20(mst12,mst22,Q2)*power2(cb)*power2(mh2 - mH2)*power2(sa)*power4(snt))
     /(power2(mh2)*power2(mH2)*power2(sb)) + (27*mu2*log(mH2/Q2)*power2(cb)*(-
     mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(sa)*power4(snt))/(power2(mH2)*power2(sb)) - (9*
     mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*(-mH2 - mst12 - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22)
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(sa)*power4(snt))/(
     power2(mH2)*power2(sb)) - (9*mu2*power2(cb)*(-mh2 - mst12 - mst22 - 4*mh2*
     mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) +
     (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(sa)*
     power2(log(mh2/Q2))*power4(snt))/(2.*power2(mh2)*power2(sb)) - (9*mu2*(-
     mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(log(mH2/Q2))*power4(snt))/(2.*power2(mH2)) + (
     9*ca*cb*mu2*sa*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(log(mH2/Q2))*power4(snt))/(
     sb*power2(mH2)) + (9*mu2*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,
     mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(log(mH2/
     Q2))*power4(snt))/power2(mH2) - (9*mu2*power2(cb)*(-mH2 - mst12 - mst22 -
     4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,
     mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*
     power2(sa)*power2(log(mH2/Q2))*power4(snt))/(2.*power2(mH2)*power2(sb)) -
     (9*mu2*(-mH2 + mst12 + mH2*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(log(mst12/Q2))*
     power4(snt))/(2.*power2(mH2)) - (9*mu2*(mst22 + mH2*(mH2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*power2(log(mst22/Q2))*power4(
     snt))/(2.*power2(mH2)) + (36*ca*cb*mst12*mst22*mu2*(mh2*DeltaInv(mst12,
     mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*
     power3(sa)*power4(snt))/(mh2*mH2*sb) - (18*ca*cb*mu2*(-mst22 + mh2*(mh2*
     mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power3(sa)*power4(
     snt))/(sb*power2(mh2)) + (54*ca*cb*mu2*log(mh2/Q2)*(-mh2 - mst12 - mst22 -
     4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power3(sa)*power4(snt))/(sb*power2(mh2)) - (18*ca*cb*mu2*log(mh2/Q2)*log(
     mst12/Q2)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*
     DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24
     + power2(mh2)))*power3(sa)*power4(snt))/(sb*power2(mh2)) - (18*ca*cb*mu2*(
     -mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*
     power3(sa)*power4(snt))/(sb*power2(mH2)) - (18*ca*cb*mu2*Fin20(mst12,
     mst22,Q2)*power2(mh2 - mH2)*power3(sa)*power4(snt))/(sb*power2(mh2)*
     power2(mH2)) + (54*ca*cb*mu2*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*
     mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) +
     (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power3(sa)*
     power4(snt))/(sb*power2(mH2)) - (18*ca*cb*mu2*log(mH2/Q2)*log(mst12/Q2)*(-
     mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,
     mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))
     *power3(sa)*power4(snt))/(sb*power2(mH2)) - (9*ca*cb*mu2*(-mh2 - mst12 -
     mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(
     mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(
     mh2))*power2(log(mh2/Q2))*power3(sa)*power4(snt))/(sb*power2(mh2)) - (9*
     ca*cb*mu2*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) -
     4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*power2(mH2))*power2(log(mH2/Q2))*power3(sa)*power4(snt))/
     (sb*power2(mH2)) + (18*mst12*mst22*mu2*(mh2*DeltaInv(mst12,mH2,mst22) +
     mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power4(sa)*
     power4(snt))/(mh2*mH2) - (9*mu2*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 -
     mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*
     log(mh2/Q2)*log(mst22/Q2)*power4(sa)*power4(snt))/power2(mh2) + (27*mu2*
     log(mh2/Q2)*(-mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(
     mst12,mst22,mh2)*power2(mh2))*power4(sa)*power4(snt))/power2(mh2) - (9*
     mu2*log(mh2/Q2)*log(mst12/Q2)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,
     mst12,mh2) + mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 +
     2*mst22) + mst24 + power2(mh2)))*power4(sa)*power4(snt))/power2(mh2) - (9*
     mu2*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*
     power4(sa)*power4(snt))/power2(mH2) - (9*mu2*Fin20(mst12,mst22,Q2)*power2(
     mh2 - mH2)*power4(sa)*power4(snt))/(power2(mh2)*power2(mH2)) + (27*mu2*
     log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22)
     - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*power2(mH2))*power4(sa)*power4(snt))/power2(mH2) - (9*
     mu2*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,
     mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 +
     2*mst22) + mst24 + power2(mH2)))*power4(sa)*power4(snt))/power2(mH2) - (
     18*mst12*mst22*mu2*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,
     mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power2(cb)*power4(sa)*power4(snt))
     /(mh2*mH2*power2(sb)) + (9*mu2*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 -
     mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*
     log(mh2/Q2)*log(mst22/Q2)*power2(cb)*power4(sa)*power4(snt))/(power2(mh2)*
     power2(sb)) - (27*mu2*log(mh2/Q2)*power2(cb)*(-mh2 - mst12 - mst22 - 4*
     mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power4(sa)*power4(snt))/(power2(mh2)*power2(sb)) + (9*mu2*log(mh2/Q2)*log(
     mst12/Q2)*power2(cb)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     + mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) +
     mst24 + power2(mh2)))*power4(sa)*power4(snt))/(power2(mh2)*power2(sb)) + (
     9*mu2*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*
     power2(cb)*power4(sa)*power4(snt))/(power2(mH2)*power2(sb)) + (9*mu2*
     Fin20(mst12,mst22,Q2)*power2(cb)*power2(mh2 - mH2)*power4(sa)*power4(snt))
     /(power2(mh2)*power2(mH2)*power2(sb)) - (27*mu2*log(mH2/Q2)*power2(cb)*(-
     mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power4(sa)*power4(snt))/(power2(mH2)*power2(sb)) + (9*
     mu2*log(mH2/Q2)*log(mst12/Q2)*power2(cb)*(-mH2 - mst12 - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22)
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power4(sa)*power4(snt))/(
     power2(mH2)*power2(sb)) - (9*mu2*(-mh2 - mst12 - mst22 - 4*mh2*mst14*
     DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) + (mh2 -
     mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(log(mh2/Q2))*
     power4(sa)*power4(snt))/(2.*power2(mh2)) + (9*mu2*power2(cb)*(-mh2 - mst12
     - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(
     mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(
     mh2))*power2(log(mh2/Q2))*power4(sa)*power4(snt))/(2.*power2(mh2)*power2(
     sb)) - (9*mu2*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(log(mH2/Q2))*power4(sa)*
     power4(snt))/(2.*power2(mH2)) + (9*mu2*power2(cb)*(-mH2 - mst12 - mst22 -
     4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,
     mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*
     power2(log(mH2/Q2))*power4(sa)*power4(snt))/(2.*power2(mH2)*power2(sb)) -
     (3*mu2*log(msb12/Q2)*log(mt2/Q2)*(-2*msb12 + DeltaInv(msntau2,msb12,mt2)*(
     msb12*msntau2*(msntau2 - 3*mt2) - msb14*(2*msntau2 + mt2) + power6(msb1))
     + DeltaInv(msb12,mt2,mu2)*(msb12*mu2*(-3*mt2 + mu2) - msb14*(mt2 + 2*mu2)
     + power6(msb1))))/(4.*power2(msntau2 - mu2)) - (3*mu2*power2(log(msb12/Q2)
     )*(-(mC2*msb12) + 2*msntau2*mu2 - 2*mC2*msb14*msntau2*DeltaInv(msntau2,
     msb12,mt2) - mC2*msb14*mt2*DeltaInv(msntau2,msb12,mt2) - mC2*msb12*
     msntau2*mt2*DeltaInv(msntau2,msb12,mt2) - power2(msntau2) + mC2*msb12*
     DeltaInv(msntau2,msb12,mt2)*power2(msntau2) + DeltaInv(mC2,mst22,msb12)*(-
     3*msb12*mst22 - mC2*(msb12 + 2*mst22) + mst24 + power2(mC2))*power2(
     msntau2 - mu2) - power2(mu2) + mC2*DeltaInv(msntau2,msb12,mt2)*power6(
     msb1) + mC2*DeltaInv(msb12,mt2,mu2)*(msb12*mu2*(-mt2 + mu2) - msb14*(mt2 +
     2*mu2) + power6(msb1))))/(8.*mC2*power2(msntau2 - mu2)) + (3*mu2*log(
     msb12/Q2)*(-2*mC2*msb12 + 6*msntau2*mu2 - 6*mC2*msb14*msntau2*DeltaInv(
     msntau2,msb12,mt2) - 3*mC2*msb14*mt2*DeltaInv(msntau2,msb12,mt2) - 3*mC2*
     msb12*msntau2*mt2*DeltaInv(msntau2,msb12,mt2) - 3*power2(msntau2) + 3*mC2*
     msb12*DeltaInv(msntau2,msb12,mt2)*power2(msntau2) + 3*DeltaInv(mC2,mst22,
     msb12)*(-3*msb12*mst22 - mC2*(msb12 + 2*mst22) + mst24 + power2(mC2))*
     power2(msntau2 - mu2) - 3*power2(mu2) + 3*mC2*DeltaInv(msntau2,msb12,mt2)*
     power6(msb1) + 3*mC2*DeltaInv(msb12,mt2,mu2)*(msb12*mu2*(-mt2 + mu2) -
     msb14*(mt2 + 2*mu2) + power6(msb1))))/(4.*mC2*power2(msntau2 - mu2)) - (3*
     mu2*power2(cb)*power2(sa)*power2(snt)*power2(log(mst12/Q2))*(3/mh2 + 3/mH2
     + (12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*
     mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))
     /mh2 - (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) +
     (8*mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2))
     - (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))))/(2.*power2(
     sb)) - (3*ca*cb*mu2*power2(snt)*power2(log(mst12/Q2))*power3(sa)*(3/mh2 +
     3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*
     mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))
     /mh2 - (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) +
     (8*mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2))
     - (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))))/sb + (3*mu2*
     power2(cb)*power2(snt)*power2(log(mst12/Q2))*power4(sa)*(3/mh2 + 3/mH2 + (
     12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(
     mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*
     DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 -
     (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (8*
     mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2)) -
     (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))))/(2.*power2(
     sb)) + (3*mu2*power2(cb)*power2(sa)*power2(log(mst12/Q2))*power4(snt)*(3/
     mh2 + 3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*
     mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*
     mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,
     mst22,mh2))/mh2 - (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*
     power2(mh2)) + (8*mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2)
     + power2(mh2)) - (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*
     mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (8*
     DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2)))
     )/(2.*power2(sb)) + (3*ca*cb*mu2*power2(log(mst12/Q2))*power3(sa)*power4(
     snt)*(3/mh2 + 3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,
     mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(
     mst12,mst22,mh2))/mh2 - (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (8*mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(
     mh2*mH2) + power2(mh2)) - (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,
     mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2
     - (8*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 -
     mH2))))/sb - (3*mu2*power2(cb)*power2(log(mst12/Q2))*power4(sa)*power4(
     snt)*(3/mh2 + 3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,
     mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(
     mst12,mst22,mh2))/mh2 - (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (8*mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(
     mh2*mH2) + power2(mh2)) - (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,
     mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2
     - (8*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 -
     mH2))))/(2.*power2(sb)) + (18*ca*cb*mu2*sa*log(mst12/Q2)*power2(snt)*(3*
     mh2*mH2*mst12 - 2*mH2*mst14 + 3*mH2*power2(mh2) - 3*mst12*power2(mh2) + 6*
     mH2*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2) + 3*(mh2 - mH2)*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)) - 3*power3(mh2) - 6*mst14*DeltaInv(mst12,mst12,mH2)*
     power3(mh2) + 2*mh2*mH2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(
     mst1))))/((mh2 - mH2)*mH2*sb*power2(mh2)) + (18*mu2*log(mst12/Q2)*power2(
     sa)*power2(snt)*(3*mh2*mH2*mst12 - 2*mH2*mst14 + 3*mH2*power2(mh2) - 3*
     mst12*power2(mh2) + 6*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2) + 3*
     (mh2 - mH2)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)) - 3*power3(mh2) - 6*mst14*
     DeltaInv(mst12,mst12,mH2)*power3(mh2) + 2*mh2*mH2*DeltaInv(mst12,mst12,
     mh2)*(mh2*mst14 - 4*power6(mst1))))/((mh2 - mH2)*mH2*power2(mh2)) + (18*
     ca*cb*mu2*sa*log(mst12/Q2)*power4(snt)*(3*mh2*mH2*mst12 - 2*mH2*mst14 + 3*
     mH2*power2(mh2) - 3*mst12*power2(mh2) + 6*mH2*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2) + 3*(mh2 - mH2)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-
     3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)) - 3*power3(
     mh2) - 6*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 2*mh2*mH2*DeltaInv(
     mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1))))/(mH2*(-mh2 + mH2)*sb*
     power2(mh2)) + (18*mu2*log(mst12/Q2)*power2(sa)*power4(snt)*(3*mh2*mH2*
     mst12 - 2*mH2*mst14 + 3*mH2*power2(mh2) - 3*mst12*power2(mh2) + 6*mH2*
     mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2) + 3*(mh2 - mH2)*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)) - 3*power3(mh2) - 6*mst14*DeltaInv(mst12,mst12,mH2)*
     power3(mh2) + 2*mh2*mH2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(
     mst1))))/(mH2*(-mh2 + mH2)*power2(mh2)) - (9*ca*cb*mu2*sa*log(mst12/Q2)*(
     12*mh2*mH2*mst12*mt2 - 8*mH2*mst14*mt2 - 3*mH2*mst12*power2(mh2) + 3*mH2*
     mst22*power2(mh2) - 12*mst12*mt2*power2(mh2) + 24*mH2*mst14*mt2*DeltaInv(
     mst12,mst12,mH2)*power2(mh2) - 3*(mh2 - mH2)*(mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)) + 3*mst12*power3(mh2) - 3*mst22*power3(mh2) - 24*
     mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 8*mh2*mH2*mt2*DeltaInv(
     mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1))))/(2.*mH2*(-mh2 + mH2)*(
     mst12 - mst22)*sb*power2(mh2)) + (3*ca*cb*mu2*power2(log(mst12/Q2))*
     power3(sa)*(3/mh2 + 3/mH2 - (8*mst12*mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)
     ) - (48*mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) - 3*mh2*DeltaInv(mst12,
     mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,
     mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 + (20*mst12*mt2)/((mst12 - mst22)*power2(
     mh2)) + (8*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) - (32*
     mst14*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + (12*mst12*mt2)/((
     mst12 - mst22)*power2(mH2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (32*mt2*DeltaInv(
     mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 -
     mst22))))/(4.*sb) + (3*mu2*power2(cb)*power2(sa)*power2(log(mst12/Q2))*(3/
     mh2 + 3/mH2 - (16*mst12*mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)) - (96*
     mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) - 3*mh2*DeltaInv(mst12,mst22,
     mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(
     mst12,mst22,mh2))/mh2 + (40*mst12*mt2)/((mst12 - mst22)*power2(mh2)) + (
     16*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) - (64*mst14*
     mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + (24*mst12*mt2)/((mst12 -
     mst22)*power2(mH2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (64*mt2*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))
     ))/(8.*power2(sb)) - (3*mu2*power2(cb)*power2(log(mst12/Q2))*power4(sa)*(
     3/mh2 + 3/mH2 - (16*mst12*mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)) - (96*
     mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) - 3*mh2*DeltaInv(mst12,mst22,
     mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(
     mst12,mst22,mh2))/mh2 + (40*mst12*mt2)/((mst12 - mst22)*power2(mh2)) + (
     16*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) - (64*mst14*
     mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + (24*mst12*mt2)/((mst12 -
     mst22)*power2(mH2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (64*mt2*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))
     ))/(8.*power2(sb)) + (3*ca*cb*mu2*sa*power2(snt)*power2(log(mst12/Q2))*(-
     9*mH2*mst12*power2(mh2) + 6*mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) +
     3*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2))
     - 3*mH2*power3(mh2) + 3*mst12*power3(mh2) + 4*mh2*DeltaInv(mst12,mst12,
     mh2)*power2(mH2)*(mh2*mst14 - 4*power6(mst1))))/((-mh2 + mH2)*sb*power2(
     mh2)*power2(mH2)) + (3*mu2*power2(sa)*power2(snt)*power2(log(mst12/Q2))*(-
     9*mH2*mst12*power2(mh2) + 6*mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) +
     3*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2))
     - 3*mH2*power3(mh2) + 3*mst12*power3(mh2) + 4*mh2*DeltaInv(mst12,mst12,
     mh2)*power2(mH2)*(mh2*mst14 - 4*power6(mst1))))/((-mh2 + mH2)*power2(mh2)*
     power2(mH2)) + (3*ca*cb*mu2*sa*power2(log(mst12/Q2))*power4(snt)*(-9*mH2*
     mst12*power2(mh2) + 6*mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) + 3*
     power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2))
     - 3*mH2*power3(mh2) + 3*mst12*power3(mh2) + 4*mh2*DeltaInv(mst12,mst12,
     mh2)*power2(mH2)*(mh2*mst14 - 4*power6(mst1))))/((mh2 - mH2)*sb*power2(
     mh2)*power2(mH2)) + (3*mu2*power2(sa)*power2(log(mst12/Q2))*power4(snt)*(-
     9*mH2*mst12*power2(mh2) + 6*mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) +
     3*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2))
     - 3*mH2*power3(mh2) + 3*mst12*power3(mh2) + 4*mh2*DeltaInv(mst12,mst12,
     mh2)*power2(mH2)*(mh2*mst14 - 4*power6(mst1))))/((mh2 - mH2)*power2(mh2)*
     power2(mH2)) + (3*mu2*power2(snt)*power2(log(mst12/Q2))*power4(sa)*(-15*
     mH2*mst12*power2(mh2) + 15*mh2*mst12*power2(mH2) - 8*mst14*power2(mH2) -
     9*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*
     mst22) + mst24 + power2(mH2)) - 3*mH2*power3(mh2) + 3*mst12*power3(mh2) -
     3*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 3*mh2*power3(mH2) - 3*
     mst12*power3(mH2) + 9*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(
     mH2) - 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*power3(mH2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 3*DeltaInv(mst12,mst22,mh2)*
     power3(mh2)*power3(mH2) + 3*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(
     mh2) + 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*(mh2*mst14 - 4*power6(
     mst1))))/(2.*(mh2 - mH2)*power2(mh2)*power2(mH2)) - (3*mu2*power2(log(
     mst12/Q2))*power4(sa)*power4(snt)*(-15*mH2*mst12*power2(mh2) + 15*mh2*
     mst12*power2(mH2) - 8*mst14*power2(mH2) - 9*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2) + 3*mst24*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2))
     - 3*mH2*power3(mh2) + 3*mst12*power3(mh2) - 3*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power3(mh2) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power3(mh2) + 3*mh2*power3(mH2) - 3*mst12*power3(mH2) + 9*mh2*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) - 3*mh2*mst24*DeltaInv(mst12,
     mst22,mh2)*power3(mH2) + 3*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power3(mH2) + 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) -
     3*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 3*DeltaInv(mst12,
     mst22,mh2)*power2(mH2)*power4(mh2) + 8*mh2*DeltaInv(mst12,mst12,mh2)*
     power2(mH2)*(mh2*mst14 - 4*power6(mst1))))/(2.*(mh2 - mH2)*power2(mh2)*
     power2(mH2)) + (3*ca*cb*mu2*sa*power2(log(mst12/Q2))*(-36*mH2*mst12*mt2*
     power2(mh2) + 24*mh2*mst12*mt2*power2(mH2) - 16*mst14*mt2*power2(mH2) - 3*
     mst12*power2(mh2)*power2(mH2) + 3*mst22*power2(mh2)*power2(mH2) - 3*(mh2 -
     mH2)*mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*
     mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)) + 3*mH2*mst12*power3(
     mh2) - 3*mH2*mst22*power3(mh2) + 12*mst12*mt2*power3(mh2) + 16*mh2*mt2*
     DeltaInv(mst12,mst12,mh2)*power2(mH2)*(mh2*mst14 - 4*power6(mst1))))/(4.*(
     -mh2 + mH2)*(mst12 - mst22)*sb*power2(mh2)*power2(mH2)) + (3*power2(sa)*
     power2(log(mst12/Q2))*(-30*mH2*mst12*mt2*power2(mh2) + 30*mh2*mst12*mt2*
     power2(mH2) - 16*mst14*mt2*power2(mH2) + 3*mu2*power2(mh2)*power2(mH2) +
     3*(mh2 - mH2)*mH2*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*
     mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)) + 6*mst12*mt2*power3(
     mh2) - 3*mH2*mu2*power3(mh2) - 6*mst12*mt2*power3(mH2) + 16*mh2*mt2*
     DeltaInv(mst12,mst12,mh2)*power2(mH2)*(mh2*mst14 - 4*power6(mst1))))/(4.*(
     mh2 - mH2)*power2(mh2)*power2(mH2)) - (3*power2(log(mst12/Q2))*power4(sa)*
     (-60*mH2*mst12*mt2*power2(mh2) + 60*mh2*mst12*mt2*power2(mH2) - 32*mst14*
     mt2*power2(mH2) - 9*mst12*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2) + 3*mst24*mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*(mh2 - mH2)*mH2*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*
     mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)) + 12*mst12*mt2*
     power3(mh2) - 3*mH2*mu2*power3(mh2) - 3*mst12*mu2*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power3(mh2) - 6*mst22*mu2*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power3(mh2) - 12*mst12*mt2*power3(mH2) + 3*mh2*mu2*power3(mH2)
     + 9*mh2*mst12*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power3(mH2) - 3*mh2*
     mst24*mu2*DeltaInv(mst12,mst22,mh2)*power3(mH2) + 3*mst12*mu2*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*mu2*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power3(mH2) - 3*mu2*DeltaInv(mst12,mst22,mh2)*
     power3(mh2)*power3(mH2) + 3*mu2*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power4(mh2) + 32*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*(mh2*mst14
     - 4*power6(mst1))))/(8.*(mh2 - mH2)*power2(mh2)*power2(mH2)) + (3*mu2*
     power2(snt)*power2(log(mst12/Q2))*(-24/mH2 - (2*(mC2*mst12 - msb12*mst12 -
     mst14)*DeltaInv(mst12,mC2,msb12))/mC2 + (24*mst12)/power2(mH2) + (24*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 + mst14/((msntau2 - mu2)*power2(msntau2 - mst12)) - (
     2*mst12)/power2(msntau2 - mu2) - mst14/((msntau2 - mst12)*power2(msntau2 -
     mu2)) + (mst14*mu2)/(power2(msntau2 - mst12)*power2(msntau2 - mu2)) -
     power6(mst1)/(power2(msntau2 - mst12)*power2(msntau2 - mu2))))/16. - (3*
     mu2*power2(log(mst12/Q2))*(1/mA2 - 3/mH2 + (8*mst12*mu2)/((mst12 - mst22)*
     (mstau12 - mstau22)*(mstau12 - mu2)) - (8*mst12*mu2)/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) + (-mA2 + mst12 + 2*mst22 + (3*mst12*
     mst22 - mst24)/mA2)*DeltaInv(mA2,mst12,mst22) + mst12*DeltaInv(mst12,
     mstau12,mt2) + (2*mst12*mu2*DeltaInv(mst12,mstau12,mt2))/(mstau12 - mu2) +
     (2*mst14*DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (mst12*mt2*
     DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (24*mst14*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 -
     mu2)) - (24*mst14*mt2*mu2*DeltaInv(mst12,mt2,mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) + mst12*DeltaInv(mstau22,mst12,mt2) +
     (2*mst12*mu2*DeltaInv(mstau22,mst12,mt2))/(mstau22 - mu2) + (2*mst14*
     DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (mst12*mt2*DeltaInv(
     mstau22,mst12,mt2))/(-mstau22 + mu2) + (3*DeltaInv(mst12,mH2,mst22)*(-3*
     mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - mst12/
     power2(mstau12 - mu2) - (mst14*mt2*DeltaInv(mst12,mstau12,mt2))/power2(
     mstau12 - mu2) - (2*mst14*mu2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12
     - mu2) - (mst12*mt2*mu2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2)
     - (mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (2*mst14*
     mu2*DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (mst12*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - mst12/power2(mstau22 -
     mu2) - (mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (2*
     mst14*mu2*DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (mst12*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (mst14*mt2*DeltaInv(
     mstau22,mst12,mt2))/power2(mstau22 - mu2) - (2*mst14*mu2*DeltaInv(mstau22,
     mst12,mt2))/power2(mstau22 - mu2) - (mst12*mt2*mu2*DeltaInv(mstau22,mst12,
     mt2))/power2(mstau22 - mu2) + (16*mst14*DeltaInv(mst12,mt2,mu2)*power2(
     mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (8*mst12*
     mt2*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) - (16*mst14*DeltaInv(mst12,mt2,mu2)*power2(mu2))
     /((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (8*mst12*mt2*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau22 - mu2)) + (mst12*DeltaInv(mst12,mstau12,mt2)*power2(mu2))/power2(
     mstau12 - mu2) + (mst12*DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(
     mstau12 - mu2) + (mst12*DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(
     mstau22 - mu2) + (mst12*DeltaInv(mstau22,mst12,mt2)*power2(mu2))/power2(
     mstau22 - mu2) - (8*mst12*DeltaInv(mst12,mt2,mu2)*power3(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (8*mst12*DeltaInv(mst12,mt2,
     mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) -
     (8*mu2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) + (8*mu2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/(
     (mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (DeltaInv(mst12,
     mstau12,mt2)*power6(mst1))/power2(mstau12 - mu2) + (DeltaInv(mst12,mt2,
     mu2)*power6(mst1))/power2(mstau12 - mu2) + (DeltaInv(mst12,mt2,mu2)*
     power6(mst1))/power2(mstau22 - mu2) + (DeltaInv(mstau22,mst12,mt2)*power6(
     mst1))/power2(mstau22 - mu2)))/8. + (3*mu2*log(mst12/Q2)*(3/mA2 - 9/mH2 +
     (24*mst12*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (
     24*mst12*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + 3*
     mst12*DeltaInv(mst12,mstau12,mt2) + (6*mst12*mu2*DeltaInv(mst12,mstau12,
     mt2))/(mstau12 - mu2) + (6*mst14*DeltaInv(mst12,mstau12,mt2))/(-mstau12 +
     mu2) + (3*mst12*mt2*DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (72*
     mst14*mt2*mu2*DeltaInv(mst12,mt2,mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) - (72*mst14*mt2*mu2*DeltaInv(mst12,mt2,mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + 3*mst12*DeltaInv(
     mstau22,mst12,mt2) + (6*mst12*mu2*DeltaInv(mstau22,mst12,mt2))/(mstau22 -
     mu2) + (6*mst14*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (3*mst12*
     mt2*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) - (3*DeltaInv(mA2,mst12,
     mst22)*(-3*mst12*mst22 - mA2*(mst12 + 2*mst22) + mst24 + power2(mA2)))/mA2
     + (9*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)))/mH2 - (2*mst12)/power2(mstau12 - mu2) - (3*mst14*
     mt2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (6*mst14*mu2*
     DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (3*mst12*mt2*mu2*
     DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (3*mst14*mt2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (6*mst14*mu2*DeltaInv(
     mst12,mt2,mu2))/power2(mstau12 - mu2) - (3*mst12*mt2*mu2*DeltaInv(mst12,
     mt2,mu2))/power2(mstau12 - mu2) - (2*mst12)/power2(mstau22 - mu2) - (3*
     mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (6*mst14*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (3*mst12*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (3*mst14*mt2*DeltaInv(
     mstau22,mst12,mt2))/power2(mstau22 - mu2) - (6*mst14*mu2*DeltaInv(mstau22,
     mst12,mt2))/power2(mstau22 - mu2) - (3*mst12*mt2*mu2*DeltaInv(mstau22,
     mst12,mt2))/power2(mstau22 - mu2) + (48*mst14*DeltaInv(mst12,mt2,mu2)*
     power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (24*
     mst12*mt2*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) - (48*mst14*DeltaInv(mst12,mt2,mu2)*power2(mu2))
     /((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (24*mst12*mt2*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau22 - mu2)) + (3*mst12*DeltaInv(mst12,mstau12,mt2)*power2(mu2))/
     power2(mstau12 - mu2) + (3*mst12*DeltaInv(mst12,mt2,mu2)*power2(mu2))/
     power2(mstau12 - mu2) + (3*mst12*DeltaInv(mst12,mt2,mu2)*power2(mu2))/
     power2(mstau22 - mu2) + (3*mst12*DeltaInv(mstau22,mst12,mt2)*power2(mu2))/
     power2(mstau22 - mu2) - (24*mst12*DeltaInv(mst12,mt2,mu2)*power3(mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (24*mst12*DeltaInv(
     mst12,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) - (24*mu2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau12 - mu2)) + (24*mu2*DeltaInv(mst12,mt2,mu2)*
     power6(mst1))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (3*
     DeltaInv(mst12,mstau12,mt2)*power6(mst1))/power2(mstau12 - mu2) + (3*
     DeltaInv(mst12,mt2,mu2)*power6(mst1))/power2(mstau12 - mu2) + (3*DeltaInv(
     mst12,mt2,mu2)*power6(mst1))/power2(mstau22 - mu2) + (3*DeltaInv(mstau22,
     mst12,mt2)*power6(mst1))/power2(mstau22 - mu2)))/4. - (3*mu2*log(msntau2/
     Q2)*log(mst12/Q2)*power2(snt)*(mst12*mst14*mu2 - 4*power2(msntau2)*power2(
     mst12) + 2*mst12*power3(msntau2) - mu2*power6(mst1) + msntau2*(-(mst12*
     mst14) + 2*power3(mst12) + power6(mst1))))/(8.*power2(msntau2 - mst12)*
     power3(msntau2 - mu2)) + (9*mu2*log(mst12/Q2)*power2(cb)*power2(sa)*
     power2(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(mh2*mH2) + (6*mst14*DeltaInv(
     mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*
     mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))
     /mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (4*
     mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (3*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*
     mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))/power2(sb) + (18*ca*cb*mu2*
     log(mst12/Q2)*power2(snt)*power3(sa)*(3/mh2 + 3/mH2 + (6*mst12)/(mh2*mH2)
     + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,mst22,
     mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(
     mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*
     power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*mH2) +
     power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 +
     2*mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,mh2)*(mh2*
     mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))/sb - (9*mu2*
     log(mst12/Q2)*power2(cb)*power2(snt)*power4(sa)*(3/mh2 + 3/mH2 + (6*mst12)
     /(mh2*mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(
     mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*
     mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-
     mh2 + mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(
     mh2*mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))
     /power2(sb) - (9*mu2*log(mst12/Q2)*power2(cb)*power2(sa)*power4(snt)*(3/
     mh2 + 3/mH2 + (6*mst12)/(mh2*mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/
     mH2 - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2)
     + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,
     mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) +
     (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(
     mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-
     3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*
     DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(
     mh2*(mh2 - mH2))))/power2(sb) - (18*ca*cb*mu2*log(mst12/Q2)*power3(sa)*
     power4(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(mh2*mH2) + (6*mst14*DeltaInv(
     mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*
     mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))
     /mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (4*
     mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (3*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*
     mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))/sb + (9*mu2*log(mst12/Q2)*
     power2(cb)*power4(sa)*power4(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(mh2*mH2) + (
     6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,mst22,mh2) +
     3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (
     9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,
     mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(
     mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*mH2) + power2(
     mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*
     mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,mh2)*(mh2*
     mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))/power2(sb) - (
     9*mu2*log(mst12/Q2)*power2(snt)*power4(sa)*(6*mh2*mH2*mst12 - 4*mH2*mst14
     - 6*mst12*power2(mh2) + 6*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)
     - 9*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2) + 3*mH2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) + 3*mh2*power2(mH2) + 9*mh2*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2) - 3*mh2*mst24*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) + 3*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) +
     3*(mh2 - mH2)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*
     (mst12 + 2*mst22) + mst24 + power2(mH2)) - 3*power3(mh2) - 6*mst14*
     DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*mH2*mst12*DeltaInv(mst12,mst22,
     mh2)*power3(mh2) - 6*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) - 3*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 3*mH2*DeltaInv(mst12,
     mst22,mh2)*power4(mh2) - 2*mh2*mH2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 -
     3*mH2*mst14 + 8*power6(mst1))))/((mh2 - mH2)*mH2*power2(mh2)) + (9*mu2*
     log(mst12/Q2)*power4(sa)*power4(snt)*(6*mh2*mH2*mst12 - 4*mH2*mst14 - 6*
     mst12*power2(mh2) + 6*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2) - 9*
     mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2) + 3*mH2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) + 3*mh2*power2(mH2) + 9*mh2*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2) - 3*mh2*mst24*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) + 3*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) +
     3*(mh2 - mH2)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*
     (mst12 + 2*mst22) + mst24 + power2(mH2)) - 3*power3(mh2) - 6*mst14*
     DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*mH2*mst12*DeltaInv(mst12,mst22,
     mh2)*power3(mh2) - 6*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) - 3*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 3*mH2*DeltaInv(mst12,
     mst22,mh2)*power4(mh2) - 2*mh2*mH2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 -
     3*mH2*mst14 + 8*power6(mst1))))/((mh2 - mH2)*mH2*power2(mh2)) + (9*log(
     mst12/Q2)*power4(sa)*(24*mh2*mH2*mst12*mt2 - 16*mH2*mst14*mt2 - 24*mst12*
     mt2*power2(mh2) + 24*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2) -
     9*mH2*mst12*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2) + 3*mH2*mst24*
     mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2) + mh2*mst12*power2(mH2) + 3*mh2*
     mu2*power2(mH2) + 9*mh2*mst12*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(
     mH2) - 3*mh2*mst24*mu2*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mst12*
     mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 6*mst22*mu2*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mu2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*
     mst22) + mst24 + power2(mH2)) - mst12*power3(mh2) - 3*mu2*power3(mh2) -
     24*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*mH2*mst12*mu2*
     DeltaInv(mst12,mst22,mh2)*power3(mh2) - 6*mH2*mst22*mu2*DeltaInv(mst12,
     mst22,mh2)*power3(mh2) - 3*mu2*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) + 3*mH2*mu2*DeltaInv(mst12,mst22,mh2)*power4(mh2) - 8*mh2*mH2*
     mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1))))
     /(4.*(mh2 - mH2)*mH2*power2(mh2)) + (9*log(mst12/Q2)*power2(sa)*(-24*mh2*
     mH2*mst12*mt2 + 16*mH2*mst14*mt2 + 24*mst12*mt2*power2(mh2) - 6*mH2*mu2*
     power2(mh2) - 24*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2) -
     mh2*mst12*power2(mH2) - 6*(mh2 - mH2)*mu2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2))
     + mst12*power3(mh2) + 6*mu2*power3(mh2) + 24*mst14*mt2*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) + 8*mh2*mH2*mt2*DeltaInv(mst12,mst12,mh2)*(mh2*
     mst14 - 3*mH2*mst14 + 8*power6(mst1))))/(4.*(mh2 - mH2)*mH2*power2(mh2)) -
     (3*mu2*log(mst12/Q2)*power2(cb)*power2(sa)*(9/mh2 + 9/mH2 - (16*mst12*mt2)
     /(mh2*(mh2 - mH2)*(mst12 - mst22)) - (144*mst12*mt2)/(mh2*mH2*mst12 - mh2*
     mH2*mst22) - (144*mst14*mt2*DeltaInv(mst12,mst12,mH2))/(mH2*mst12 - mH2*
     mst22) - 9*mh2*DeltaInv(mst12,mst22,mh2) + 9*mst12*DeltaInv(mst12,mst22,
     mh2) + 18*mst22*DeltaInv(mst12,mst22,mh2) + (27*mst12*mst22*DeltaInv(
     mst12,mst22,mh2))/mh2 - (9*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/
     power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (16*mst12*mt2)/((
     mst12 - mst22)*power2(mh2)) + (16*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 -
     mst22)*power2(mh2)) - (96*mst14*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(
     mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (9*DeltaInv(mst12,mH2,mst22)*(-
     3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (48*
     mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/
     (mh2*(mh2 - mH2)*(mst12 - mst22))))/(4.*power2(sb)) + (3*mu2*log(mst12/Q2)
     *power2(cb)*power4(sa)*(9/mh2 + 9/mH2 - (16*mst12*mt2)/(mh2*(mh2 - mH2)*(
     mst12 - mst22)) - (144*mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) - (144*
     mst14*mt2*DeltaInv(mst12,mst12,mH2))/(mH2*mst12 - mH2*mst22) - 9*mh2*
     DeltaInv(mst12,mst22,mh2) + 9*mst12*DeltaInv(mst12,mst22,mh2) + 18*mst22*
     DeltaInv(mst12,mst22,mh2) + (27*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2
     - (9*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*
     mst12)/((-mh2 + mH2)*power2(mh2)) + (16*mst12*mt2)/((mst12 - mst22)*
     power2(mh2)) + (16*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)
     ) - (96*mst14*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + mst12/(-(
     mh2*mH2) + power2(mh2)) - (9*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (48*mt2*DeltaInv(
     mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 -
     mH2)*(mst12 - mst22))))/(4.*power2(sb)) - (3*ca*cb*mu2*log(mst12/Q2)*
     power3(sa)*(9/mh2 + 9/mH2 - (8*mst12*mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)
     ) - (72*mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) - (72*mst14*mt2*
     DeltaInv(mst12,mst12,mH2))/(mH2*mst12 - mH2*mst22) - 9*mh2*DeltaInv(mst12,
     mst22,mh2) + 9*mst12*DeltaInv(mst12,mst22,mh2) + 18*mst22*DeltaInv(mst12,
     mst22,mh2) + (27*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (9*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (8*mst12*mt2)/((mst12 - mst22)*power2(mh2)) + (8*mH2*
     mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) - (48*mst14*mt2)/((
     mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + mst12/(-(mh2*mH2) + power2(mh2))
     - (9*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)))/mH2 - (24*mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 -
     3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))))/(2.*sb)
     - (3*log(mst12/Q2)*log(mu2/Q2)*power2(mu2)*(mst12*(-8/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau12 - mu2)) + 8/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) + 1/power3(-mstau12 + mu2) + 1/power3(-mstau22 +
     mu2)) + (2*DeltaInv(mst12,mt2,mu2)*(4*mst14*(mstau12 - mu2)*(mstau22 -
     mu2)*(mt2 + 2*mu2) + mt2*power2(mst12)*(-2*mstau12*mu2 - 2*mstau22*mu2 +
     power2(mstau12) + power2(mstau22) + 2*power2(mu2)) - mst12*(-4*(mstau12 -
     mu2)*mu2*(-mstau22 + mu2)*(-3*mt2 + mu2) + mst22*mt2*(-2*mstau12*mu2 - 2*
     mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2))) + 4*(
     mstau12 - mu2)*(-mstau22 + mu2)*power6(mst1)))/((mst12 - mst22)*power2(
     mstau12 - mu2)*power2(mstau22 - mu2))))/4. - (3*mu2*log(mst12/Q2)*log(mt2/
     Q2)*(-4*mst12*mst22*mstau12*mu2 - 4*mst12*mst22*mstau22*mu2 - 16*mst12*
     mstau12*mstau22*mu2 + 4*mst12*mst14*mstau12*mstau22*mu2*DeltaInv(mstau22,
     mst12,mt2) - 4*mst14*mst22*mstau12*mstau22*mu2*DeltaInv(mstau22,mst12,mt2)
     + 2*mst12*mst14*mstau12*mt2*mu2*DeltaInv(mstau22,mst12,mt2) - 2*mst14*
     mst22*mstau12*mt2*mu2*DeltaInv(mstau22,mst12,mt2) - 6*mst12*mst22*mstau12*
     mstau22*mt2*mu2*DeltaInv(mstau22,mst12,mt2) + 4*mstau12*mu2*power2(mst12)
     + 4*mstau22*mu2*power2(mst12) + 6*mstau12*mstau22*mt2*mu2*DeltaInv(
     mstau22,mst12,mt2)*power2(mst12) + 2*mst12*mst22*power2(mstau12) - 2*
     mst12*mst14*mstau22*DeltaInv(mstau22,mst12,mt2)*power2(mstau12) + 2*mst14*
     mst22*mstau22*DeltaInv(mstau22,mst12,mt2)*power2(mstau12) - mst12*mst14*
     mt2*DeltaInv(mstau22,mst12,mt2)*power2(mstau12) + mst14*mst22*mt2*
     DeltaInv(mstau22,mst12,mt2)*power2(mstau12) + 3*mst12*mst22*mstau22*mt2*
     DeltaInv(mstau22,mst12,mt2)*power2(mstau12) - 2*power2(mst12)*power2(
     mstau12) - 3*mstau22*mt2*DeltaInv(mstau22,mst12,mt2)*power2(mst12)*power2(
     mstau12) + 2*mst12*mst22*power2(mstau22) + 2*mst12*mst22*mstau12*mu2*
     DeltaInv(mstau22,mst12,mt2)*power2(mstau22) - 2*power2(mst12)*power2(
     mstau22) - 2*mstau12*mu2*DeltaInv(mstau22,mst12,mt2)*power2(mst12)*power2(
     mstau22) - mst12*mst22*DeltaInv(mstau22,mst12,mt2)*power2(mstau12)*power2(
     mstau22) + DeltaInv(mstau22,mst12,mt2)*power2(mst12)*power2(mstau12)*
     power2(mstau22) + 4*mst12*mst22*power2(mu2) + 16*mst12*mstau12*power2(mu2)
     + 16*mst12*mstau22*power2(mu2) - 2*mst12*mst14*mstau22*DeltaInv(mstau22,
     mst12,mt2)*power2(mu2) + 2*mst14*mst22*mstau22*DeltaInv(mstau22,mst12,mt2)
     *power2(mu2) - mst12*mst14*mt2*DeltaInv(mstau22,mst12,mt2)*power2(mu2) +
     mst14*mst22*mt2*DeltaInv(mstau22,mst12,mt2)*power2(mu2) + 3*mst12*mst22*
     mstau22*mt2*DeltaInv(mstau22,mst12,mt2)*power2(mu2) - 4*power2(mst12)*
     power2(mu2) - 3*mstau22*mt2*DeltaInv(mstau22,mst12,mt2)*power2(mst12)*
     power2(mu2) - mst12*mst22*DeltaInv(mstau22,mst12,mt2)*power2(mstau22)*
     power2(mu2) + DeltaInv(mstau22,mst12,mt2)*power2(mst12)*power2(mstau22)*
     power2(mu2) - 16*mst12*power3(mu2) - 2*mst12*mstau12*mu2*DeltaInv(mstau22,
     mst12,mt2)*power6(mst1) + 2*mst22*mstau12*mu2*DeltaInv(mstau22,mst12,mt2)*
     power6(mst1) + mst12*DeltaInv(mstau22,mst12,mt2)*power2(mstau12)*power6(
     mst1) - mst22*DeltaInv(mstau22,mst12,mt2)*power2(mstau12)*power6(mst1) +
     mst12*DeltaInv(mstau22,mst12,mt2)*power2(mu2)*power6(mst1) - mst22*
     DeltaInv(mstau22,mst12,mt2)*power2(mu2)*power6(mst1) + (mst12 - mst22)*
     DeltaInv(mst12,mstau12,mt2)*power2(mstau22 - mu2)*(mst12*mstau12*(mstau12
     - 3*mt2) - mst14*(2*mstau12 + mt2) + power6(mst1)) + DeltaInv(mst12,mt2,
     mu2)*(-((3*mt2 - mu2)*mu2*power2(mst12)*(-2*mstau12*mu2 - 2*mstau22*mu2 +
     power2(mstau12) + power2(mstau22) + 2*power2(mu2))) + mst14*(32*(mstau12 -
     mu2)*mu2*(-mstau22 + mu2)*(mt2 + mu2) + mst22*(mt2 + 2*mu2)*(-2*mstau12*
     mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)))
     - mst12*(mst14*(mt2 + 2*mu2)*(-2*mstau12*mu2 - 2*mstau22*mu2 + power2(
     mstau12) + power2(mstau22) + 2*power2(mu2)) + mu2*(16*(mstau12 - mu2)*mu2*
     (-mstau22 + mu2)*(-2*mt2 + mu2) - mst22*(3*mt2 - mu2)*(-2*mstau12*mu2 - 2*
     mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)))) + (16*(
     mstau12 - mu2)*(mstau22 - mu2)*mu2 + mst12*(-2*mstau12*mu2 - 2*mstau22*mu2
     + power2(mstau12) + power2(mstau22) + 2*power2(mu2)) - mst22*(-2*mstau12*
     mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)))*
     power6(mst1))))/(4.*(mst12 - mst22)*power2(mstau12 - mu2)*power2(mstau22 -
     mu2)) + (9*ca*cb*mu2*sa*log(mst22/Q2)*(24*(mh2 - mH2)*mst24*mt2*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) - 3*(mh2 - mH2)*(mst12 - mst22)*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 4*mt2*(-3*
     mh2*mH2*mst22 + 2*mH2*mst24 + 3*mst22*power2(mh2) - 2*mh2*mH2*DeltaInv(
     mst22,mst22,mh2)*(mh2*mst24 - 4*power6(mst2)))))/(2.*(mh2 - mH2)*mH2*(
     mst12 - mst22)*sb*power2(mh2)) + (3*ca*cb*mu2*sa*power2(log(mst22/Q2))*(-
     3*(mh2 - mH2)*mH2*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) - 4*mt2*(-9*mH2*mst22*power2(mh2) +
     6*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) + 3*mst22*power3(mh2) + 4*
     mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/
     (4.*(mh2 - mH2)*(-mst12 + mst22)*sb*power2(mh2)*power2(mH2)) - (3*mu2*
     power2(cb)*power2(sa)*power2(log(mst22/Q2))*(-3*(mh2 - mH2)*mH2*(mst12 -
     mst22)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(
     mh2) - 3*mh2*(mh2 - mH2)*(mst12 - mst22)*(mh2*mst22 - mst12*mst22 - mst24)
     *DeltaInv(mst12,mst22,mh2)*power2(mH2) - 8*mt2*(-15*mH2*mst22*power2(mh2)
     + 15*mh2*mst22*power2(mH2) - (3*mH2*mst22 + 8*mst24)*power2(mH2) + 3*
     mst22*power3(mh2) + 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24
     - 4*power6(mst2)))))/(8.*(mh2 - mH2)*(-mst12 + mst22)*power2(mh2)*power2(
     mH2)*power2(sb)) + (3*mu2*power2(cb)*power2(log(mst22/Q2))*power4(sa)*(-3*
     (mh2 - mH2)*mH2*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mh2*(mh2 - mH2)*(mst12 - mst22)*
     (mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) -
     8*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*power2(mH2) - (3*mH2*mst22
     + 8*mst24)*power2(mH2) + 3*mst22*power3(mh2) + 8*mh2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/(8.*(mh2 - mH2)*(-mst12 +
     mst22)*power2(mh2)*power2(mH2)*power2(sb)) - (3*ca*cb*mu2*power2(log(
     mst22/Q2))*power3(sa)*(-3*(mh2 - mH2)*mH2*(mst12 - mst22)*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mh2*(mh2 -
     mH2)*(mst12 - mst22)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) - 4*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*
     power2(mH2) - (3*mH2*mst22 + 8*mst24)*power2(mH2) + 3*mst22*power3(mh2) +
     8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2))))
     )/(4.*(mh2 - mH2)*(-mst12 + mst22)*sb*power2(mh2)*power2(mH2)) - (3*
     power2(log(mst22/Q2))*power4(sa)*(3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*
     mst22 - mst24)*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 3*mh2*(mh2 -
     mH2)*(mh2*mst22 - mst12*mst22 - mst24)*mu2*DeltaInv(mst12,mst22,mh2)*
     power2(mH2) + 4*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*power2(mH2)
     - (3*mH2*mst22 + 8*mst24)*power2(mH2) + 3*mst22*power3(mh2) + 8*mh2*
     DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/(8.*
     (mh2 - mH2)*power2(mh2)*power2(mH2)) + (3*power2(sa)*power2(log(mst22/Q2))
     *(3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*mu2*DeltaInv(mst12,
     mH2,mst22)*power2(mh2) + 2*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*
     power2(mH2) - 8*mst24*power2(mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(
     mH2) + 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*power6(
     mst2)))))/(4.*(mh2 - mH2)*power2(mh2)*power2(mH2)) - (3*ca*cb*mu2*sa*
     power2(snt)*power2(log(mst22/Q2))*(-9*mH2*mst22*power2(mh2) + 3*(mh2 -
     mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2) + 6*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) + 3*mst22*
     power3(mh2) + 4*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*
     power6(mst2))))/((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) - (3*mu2*power2(
     sa)*power2(snt)*power2(log(mst22/Q2))*(-9*mH2*mst22*power2(mh2) + 3*(mh2 -
     mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2) + 6*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) + 3*mst22*
     power3(mh2) + 4*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*
     power6(mst2))))/((mh2 - mH2)*power2(mh2)*power2(mH2)) + (3*ca*cb*mu2*sa*
     power2(log(mst22/Q2))*power4(snt)*(-9*mH2*mst22*power2(mh2) + 3*(mh2 -
     mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2) + 6*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) + 3*mst22*
     power3(mh2) + 4*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*
     power6(mst2))))/((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) + (3*mu2*power2(
     sa)*power2(log(mst22/Q2))*power4(snt)*(-9*mH2*mst22*power2(mh2) + 3*(mh2 -
     mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2) + 6*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) + 3*mst22*
     power3(mh2) + 4*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*
     power6(mst2))))/((mh2 - mH2)*power2(mh2)*power2(mH2)) - (3*mu2*power2(log(
     msntau2/Q2))*(msntau2*mst22*mst24 - mst22*mst24*mu2 - 4*mst22*mt2*power2(
     msntau2) - 4*mst22*mu2*power2(msntau2) - 2*msb12*msntau2*power2(msntau2 -
     mst22) - 2*msntau2*(msb14 + msntau2*(msntau2 - mt2) - msb12*(2*msntau2 +
     mt2))*(msntau2 - mu2)*DeltaInv(msntau2,msb12,mt2)*power2(msntau2 - mst22)
     + 2*msntau2*mt2*power2(mst22) + 2*msntau2*mu2*power2(mst22) + 6*power2(
     msntau2)*power2(mst22) - 6*mst22*power3(msntau2) + 2*mt2*power3(msntau2) +
     2*mu2*power3(msntau2) - 2*msntau2*power3(mst22) + 2*power4(msntau2) -
     msntau2*power6(mst2) + mu2*power6(mst2)))/(16.*power2(msntau2 - mst22)*
     power3(-msntau2 + mu2)) + (3*mu2*log(msntau2/Q2)*(3*msntau2*mst22*mst24 -
     3*mst22*mst24*mu2 - 8*mst22*mt2*power2(msntau2) - 4*msb12*msntau2*power2(
     msntau2 - mst22) - 6*msntau2*(msb14 + msntau2*(msntau2 - mt2) - msb12*(2*
     msntau2 + mt2))*(msntau2 - mu2)*DeltaInv(msntau2,msb12,mt2)*power2(msntau2
     - mst22) + 4*msntau2*mt2*power2(mst22) + 12*power2(msntau2)*power2(mst22)
     - 12*mst22*power3(msntau2) + 4*mt2*power3(msntau2) - 4*msntau2*power3(
     mst22) + 4*power4(msntau2) - 3*msntau2*power6(mst2) + 3*mu2*power6(mst2)))
     /(8.*power2(msntau2 - mst22)*power3(-msntau2 + mu2)) - (9*mu2*log(mst22/
     Q2)*power2(cb)*power2(sa)*power2(snt)*(6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*
     mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2)
     + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,
     mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)
     ))/((mh2 - mH2)*mH2*power2(mh2)*power2(sb)) - (18*ca*cb*mu2*log(mst22/Q2)*
     power2(snt)*power3(sa)*(6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(
     mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2
     - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(
     mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*
     mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,
     mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*
     mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)
     *power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 -
     mH2)*mH2*sb*power2(mh2)) - (9*mu2*log(mst22/Q2)*power2(snt)*power4(sa)*(6*
     mh2*mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,
     mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*
     mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)) + (
     9*mu2*log(mst22/Q2)*power2(cb)*power2(snt)*power4(sa)*(6*mh2*mH2*mst22 -
     4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,
     mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)
     *DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2)
     - 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,
     mh2)*power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*
     DeltaInv(mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,
     mh2)*power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)*power2(sb)) + (9*mu2*log(
     mst22/Q2)*power2(cb)*power2(sa)*power4(snt)*(6*mh2*mH2*mst22 - 4*mH2*mst24
     - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*
     power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(
     mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)
     *power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*
     mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*
     power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)*power2(sb)) + (18*ca*cb*mu2*
     log(mst22/Q2)*power3(sa)*power4(snt)*(6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*
     mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2)
     + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,
     mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)
     ))/((mh2 - mH2)*mH2*sb*power2(mh2)) + (9*mu2*log(mst22/Q2)*power4(sa)*
     power4(snt)*(6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2
     + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*
     mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*
     mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)
     + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(
     mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*
     mH2*power2(mh2)) - (9*mu2*log(mst22/Q2)*power2(cb)*power4(sa)*power4(snt)*
     (6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)*
     mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,
     mst22,mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)
     + 3*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)*
     power2(sb)) + (18*ca*cb*mu2*sa*log(mst22/Q2)*power2(snt)*(3*mh2*mH2*mst22
     - 2*mH2*mst24 - 3*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,
     mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)
     *DeltaInv(mst12,mH2,mst22)*power2(mh2) + 2*mH2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mh2) - 8*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((
     mh2 - mH2)*mH2*sb*power2(mh2)) + (18*mu2*log(mst22/Q2)*power2(sa)*power2(
     snt)*(3*mh2*mH2*mst22 - 2*mH2*mst24 - 3*mst22*power2(mh2) + 6*(-mh2 + mH2)
     *mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 2*mH2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2) - 8*mh2*mH2*DeltaInv(mst22,mst22,
     mh2)*power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)) - (18*ca*cb*mu2*sa*log(
     mst22/Q2)*power4(snt)*(3*mh2*mH2*mst22 - 2*mH2*mst24 - 3*mst22*power2(mh2)
     + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 -
     mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(
     mh2) + 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) - 8*mh2*mH2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*sb*power2(mh2))
     - (18*mu2*log(mst22/Q2)*power2(sa)*power4(snt)*(3*mh2*mH2*mst22 - 2*mH2*
     mst24 - 3*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,
     mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) + 2*mH2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mh2) - 8*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((
     mh2 - mH2)*mH2*power2(mh2)) - (9*log(mst22/Q2)*power2(sa)*(24*mh2*mH2*
     mst22*mt2 - 16*mH2*mst24*mt2 - 24*mst22*mt2*power2(mh2) - 24*(mh2 - mH2)*
     mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 6*(mh2 - mH2)*(mH2*mst22
     - mst12*mst22 - mst24)*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 8*mH2*
     mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2) + mh2*mst22*power2(mH2) +
     24*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2) - mst22*power3(mh2)
     - 64*mh2*mH2*mt2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/(4.*(mh2 - mH2)*
     mH2*power2(mh2)) + (9*log(mst22/Q2)*power4(sa)*(24*mh2*mH2*mst22*mt2 - 16*
     mH2*mst24*mt2 - 24*mst22*mt2*power2(mh2) - 24*(mh2 - mH2)*mst24*mt2*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*
     mst22 - mst24)*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*
     mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*mu2*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) - 8*mH2*mst24*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mh2) + mh2*mst22*power2(mH2) + 3*mh2*mst12*mst22*mu2*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*mu2*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) + 24*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*
     power2(mH2) - 3*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mH2) - mst22*power3(mh2) + 3*mH2*mst22*mu2*DeltaInv(mst12,mst22,mh2)*
     power3(mh2) - 64*mh2*mH2*mt2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/(4.*
     (mh2 - mH2)*mH2*power2(mh2)) + (9*ca*cb*mu2*log(mst22/Q2)*power3(sa)*(-24*
     mh2*mH2*mst22*mt2 + 16*mH2*mst24*mt2 + 24*mst22*mt2*power2(mh2) + 24*(mh2
     - mH2)*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*(mh2 - mH2)*(
     mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)
     *power2(mh2) + 3*mH2*mst12*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) -
     3*mH2*mst22*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) + 8*mH2*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mh2) - 3*mh2*mst12*mst24*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) + 3*mh2*mst22*mst24*DeltaInv(mst12,mst22,mh2)*
     power2(mH2) - 24*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2) + 3*
     mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst12) - 3*mh2*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12) - 3*mH2*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22) + 3*mh2*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22) - 3*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power3(mh2) + 3*mH2*DeltaInv(mst12,mst22,mh2)*
     power2(mst22)*power3(mh2) + 64*mh2*mH2*mt2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)))/(2.*(mh2 - mH2)*mH2*(-mst12 + mst22)*sb*power2(mh2)) + (9*
     mu2*log(mst22/Q2)*power2(cb)*power2(sa)*(-48*mh2*mH2*mst22*mt2 + 32*mH2*
     mst24*mt2 + 48*mst22*mt2*power2(mh2) + 48*(mh2 - mH2)*mst24*mt2*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) - 3*(mh2 - mH2)*(mst12 - mst22)*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 3*mH2*mst12*
     mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst22*mst24*DeltaInv(
     mst12,mst22,mh2)*power2(mh2) + 16*mH2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*
     power2(mh2) - 3*mh2*mst12*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*
     mh2*mst22*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) - 48*mh2*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mH2) + 3*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)
     *power2(mh2)*power2(mst12) - 3*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power2(mst12) - 3*mH2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst22) + 3*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(
     mst22) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22)
     - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) + 3*mH2*
     DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mh2) + 128*mh2*mH2*mt2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/(4.*(mh2 - mH2)*mH2*(-mst12 +
     mst22)*power2(mh2)*power2(sb)) + (9*mu2*log(mst22/Q2)*power2(cb)*power4(
     sa)*(-48*mh2*mH2*mst22*mt2 + 32*mH2*mst24*mt2 + 48*mst22*mt2*power2(mh2) +
     48*(mh2 - mH2)*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*(mh2 -
     mH2)*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22)*power2(mh2) + 3*mH2*mst12*mst24*DeltaInv(mst12,mst22,mh2)*power2(
     mh2) - 3*mH2*mst22*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) + 16*mH2*
     mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2) - 3*mh2*mst12*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst22*mst24*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) - 48*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*
     power2(mH2) + 3*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst12) -
     3*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12) - 3*mH2*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22) + 3*mh2*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22) - 3*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power3(mh2) + 3*mH2*DeltaInv(mst12,mst22,mh2)*
     power2(mst22)*power3(mh2) + 128*mh2*mH2*mt2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)))/(4.*(mh2 - mH2)*mH2*(mst12 - mst22)*power2(mh2)*power2(sb))
     + (3*log(mt2/Q2)*log(mu2/Q2)*power2(mu2)*((-8*mst12)/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau12 - mu2)) + (8*mst22)/((mst12 - mst22)*(mstau12
     - mstau22)*(mstau12 - mu2)) + (8*mst12)/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) - (8*mst22)/((mst12 - mst22)*(mstau12 - mstau22)
     *(mstau22 - mu2)) + (8*mst24*mt2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)
     *(mstau12 - mstau22)*(mstau12 - mu2)) - (8*mst24*mt2*DeltaInv(mst22,mt2,
     mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (8*mst24*
     mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12
     - mu2)) + (48*mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau12 - mu2)) - (8*mst24*mu2*DeltaInv(mst22,mt2,mu2)
     )/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (48*mst22*mt2*
     mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) + 1/power2(msntau2 - mu2) + 2/power2(mstau12 - mu2) - (mst24*
     DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (3*mst22*mt2*DeltaInv(
     mst22,mt2,mu2))/power2(mstau12 - mu2) + (2*mst22*mu2*DeltaInv(mst22,mt2,
     mu2))/power2(mstau12 - mu2) + (mt2*mu2*DeltaInv(mst22,mt2,mu2))/power2(
     mstau12 - mu2) + 2/power2(mstau22 - mu2) - (mst24*DeltaInv(mst22,mt2,mu2))
     /power2(mstau22 - mu2) + (3*mst22*mt2*DeltaInv(mst22,mt2,mu2))/power2(
     mstau22 - mu2) + (2*mst22*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 -
     mu2) + (mt2*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (8*mst22*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau12 - mu2)) + (8*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (8*mst22*DeltaInv(mst22,mt2,
     mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) -
     (8*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) - (DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(
     mstau12 - mu2) - (DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau22 -
     mu2) - (DeltaInv(msb12,mt2,mu2)*(msb14 - 3*msb12*mt2 - 2*msb12*mu2 - mt2*
     mu2 + power2(mu2)))/power2(msntau2 - mu2) - (8*DeltaInv(mst22,mt2,mu2)*
     power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (8*
     DeltaInv(mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau22 - mu2)) + mt2/power3(-msntau2 + mu2) + (2*mt2)/power3(-mstau12 +
     mu2) + (2*mt2)/power3(-mstau22 + mu2) + (DeltaInv(mst12,mt2,mu2)*(-(mst22*
     mt2*mu2*power2(mstau12)) - mst22*mt2*mu2*power2(mstau22) + 2*mst22*
     mstau12*mt2*power2(mu2) + 2*mst22*mstau22*mt2*power2(mu2) + 8*mstau12*
     mstau22*mt2*power2(mu2) + mst22*power2(mstau12)*power2(mu2) + mst22*
     power2(mstau22)*power2(mu2) + (3*mt2 + 2*mu2)*power2(mst12)*(-2*mstau12*
     mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)) +
     mst14*(8*(mstau12 - mu2)*(mstau22 - mu2)*(mt2 + mu2) + mst22*(-2*mstau12*
     mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)))
     - mst12*(mst14*(-2*mstau12*mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(
     mstau22) + 2*power2(mu2)) + mst22*(3*mt2 + 2*mu2)*(-2*mstau12*mu2 - 2*
     mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2)) + mu2*(2*
     mstau22*mu2*(25*mt2 + 3*mu2) + mstau12*(-8*mstau22*(6*mt2 + mu2) + 2*mu2*(
     25*mt2 + 3*mu2)) + (-mt2 + mu2)*power2(mstau12) + (-mt2 + mu2)*power2(
     mstau22) - 2*(25*mt2 + 3*mu2)*power2(mu2))) - 2*mst22*mstau12*power3(mu2)
     - 2*mst22*mstau22*power3(mu2) - 8*mstau12*mstau22*power3(mu2) - 2*mst22*
     mt2*power3(mu2) - 8*mstau12*mt2*power3(mu2) - 8*mstau22*mt2*power3(mu2) +
     2*mst22*power4(mu2) + 8*mstau12*power4(mu2) + 8*mstau22*power4(mu2) + 8*
     mt2*power4(mu2) - 8*power5(mu2) - 8*mstau12*mstau22*power6(mst1) + 8*
     mstau12*mu2*power6(mst1) + 8*mstau22*mu2*power6(mst1) - 8*power2(mu2)*
     power6(mst1)))/((mst12 - mst22)*power2(mstau12 - mu2)*power2(mstau22 -
     mu2)) - (8*DeltaInv(mst22,mt2,mu2)*power6(mst2))/((mst12 - mst22)*(mstau12
     - mstau22)*(mstau12 - mu2)) + (8*DeltaInv(mst22,mt2,mu2)*power6(mst2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2))))/4. + (3*mu2*power2(
     cb)*power2(sa)*power2(snt)*power2(log(mst22/Q2))*(-15*mH2*mst22*power2(
     mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,
     mH2,mst22)*power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) +
     3*mh2*(mh2 - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,
     mh2)*power2(mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(mh2 - mH2)*power2(mh2)*power2(
     mH2)*power2(sb)) + (3*ca*cb*mu2*power2(snt)*power2(log(mst22/Q2))*power3(
     sa)*(-15*mH2*mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 15*mh2*mst22*
     power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2 - mH2)*(mh2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 8*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mst22*power3(mh2) - 3*mst22*
     power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     ((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) + (3*mu2*power2(snt)*power2(log(
     mst22/Q2))*power4(sa)*(-15*mH2*mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*
     mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 15*
     mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2 - mH2)*(mh2*mst22
     - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 8*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mst22*power3(mh2) -
     3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(
     mst2)))/(2.*(mh2 - mH2)*power2(mh2)*power2(mH2)) - (3*mu2*power2(cb)*
     power2(snt)*power2(log(mst22/Q2))*power4(sa)*(-15*mH2*mst22*power2(mh2) +
     3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22)*power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*
     mh2*(mh2 - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,
     mh2)*power2(mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(mh2 - mH2)*power2(mh2)*power2(
     mH2)*power2(sb)) - (3*mu2*power2(cb)*power2(sa)*power2(log(mst22/Q2))*
     power4(snt)*(-15*mH2*mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 15*mh2*mst22*
     power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2 - mH2)*(mh2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 8*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mst22*power3(mh2) - 3*mst22*
     power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     (2.*(mh2 - mH2)*power2(mh2)*power2(mH2)*power2(sb)) - (3*ca*cb*mu2*power2(
     log(mst22/Q2))*power3(sa)*power4(snt)*(-15*mH2*mst22*power2(mh2) + 3*(mh2
     - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2
     - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(
     mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*
     mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)
     *power2(mH2)*power6(mst2)))/((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) - (3*
     mu2*power2(log(mst22/Q2))*power4(sa)*power4(snt)*(-15*mH2*mst22*power2(
     mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,
     mH2,mst22)*power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) +
     3*mh2*(mh2 - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,
     mh2)*power2(mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(mh2 - mH2)*power2(mh2)*power2(
     mH2)) + (3*mu2*power2(cb)*power2(log(mst22/Q2))*power4(sa)*power4(snt)*(-
     15*mH2*mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 -
     mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 15*mh2*mst22*power2(mH2) -
     8*mst24*power2(mH2) + 3*mh2*(mh2 - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*
     power2(mh2)*power2(mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*
     mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(mh2 - mH2)*
     power2(mh2)*power2(mH2)*power2(sb)) + (21*mu2*power2(cb)*power2(sa)*
     power2(snt)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2
     - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*
     power2(mH2) - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*
     power2(mH2) - 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) +
     18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*
     power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*
     mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(
     mh2 - mH2)*power2(mh2)*power2(mH2)*power2(sb)) + (mu2*power2(cb)*power2(
     Pi)*power2(sa)*power2(snt)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(
     mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*
     mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*
     mst24*power2(mH2) - 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(
     mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*
     mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*
     power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*
     mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(
     mh2 - mH2)*power2(mh2)*power2(mH2)*power2(sb)) + (21*ca*cb*mu2*power2(snt)
     *power3(sa)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2
     - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*
     power2(mH2) - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*
     power2(mH2) - 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) +
     18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*
     power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*
     mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2
     - mH2)*sb*power2(mh2)*power2(mH2)) + (ca*cb*mu2*power2(Pi)*power2(snt)*
     power3(sa)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 -
     mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(
     mH2) - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2)
     - 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*power3(
     mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*mst12*
     power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) +
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(mh2 - mH2)*sb*power2(mh2)
     *power2(mH2)) + (21*mu2*power2(snt)*power4(sa)*(-3*mH2*mst12*power2(mh2) -
     3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,
     mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) + 3*
     mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mh2)*power2(mH2) - 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*
     mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*
     power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)
     *power3(mh2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) -
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(
     mH2) + 3*mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(
     mst12,mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,
     mh2)*power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(
     mh2 - mH2)*power2(mh2)*power2(mH2)) + (mu2*power2(Pi)*power2(snt)*power4(
     sa)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*
     mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2)
     - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) -
     14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*power3(
     mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*mst12*
     power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) +
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 - mH2)*power2(mh2)*
     power2(mH2)) - (21*mu2*power4(sa)*power4(snt)*(-3*mH2*mst12*power2(mh2) -
     3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,
     mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) + 3*
     mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mh2)*power2(mH2) - 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*
     mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*
     power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)
     *power3(mh2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) -
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(
     mH2) + 3*mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(
     mst12,mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,
     mh2)*power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(
     mh2 - mH2)*power2(mh2)*power2(mH2)) - (mu2*power2(Pi)*power4(sa)*power4(
     snt)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*
     mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2)
     - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) -
     14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*power3(
     mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*mst12*
     power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) +
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 - mH2)*power2(mh2)*
     power2(mH2)) + (21*mu2*power2(cb)*power4(sa)*power4(snt)*(-3*mH2*mst12*
     power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(
     mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) +
     6*mh2*power3(mH2) + 3*mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*
     mst14*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*
     power3(mH2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) +
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))
     /(2.*(mh2 - mH2)*power2(mh2)*power2(mH2)*power2(sb)) + (mu2*power2(cb)*
     power2(Pi)*power4(sa)*power4(snt)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*
     power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(
     mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(
     mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(
     mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) -
     14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)
     *mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*
     power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*
     mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(
     mh2 - mH2)*power2(mh2)*power2(mH2)*power2(sb)) - (21*ca*cb*mu2*sa*power2(
     snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*
     mh2*mst12*power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 2*
     mst24*power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     ((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) - (ca*cb*mu2*sa*power2(Pi)*
     power2(snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) + 3*mh2*mst12*power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(
     mH2) - 2*mst24*power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(
     mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)
     *(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2)
     - 3*mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     (2.*(mh2 - mH2)*sb*power2(mh2)*power2(mH2)) - (21*mu2*power2(sa)*power2(
     snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*
     mh2*mst12*power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 2*
     mst24*power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     ((mh2 - mH2)*power2(mh2)*power2(mH2)) - (mu2*power2(Pi)*power2(sa)*power2(
     snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*
     mh2*mst12*power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 2*
     mst24*power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     (2.*(mh2 - mH2)*power2(mh2)*power2(mH2)) + (21*ca*cb*mu2*sa*power4(snt)*(-
     18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*
     mst12*power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 2*
     mst24*power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     ((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) + (ca*cb*mu2*sa*power2(Pi)*
     power4(snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) + 3*mh2*mst12*power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(
     mH2) - 2*mst24*power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(
     mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)
     *(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2)
     - 3*mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     (2.*(mh2 - mH2)*sb*power2(mh2)*power2(mH2)) + (21*mu2*power2(sa)*power4(
     snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*
     mh2*mst12*power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 2*
     mst24*power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     ((mh2 - mH2)*power2(mh2)*power2(mH2)) + (mu2*power2(Pi)*power2(sa)*power4(
     snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*
     mh2*mst12*power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 2*
     mst24*power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     (2.*(mh2 - mH2)*power2(mh2)*power2(mH2)) + (21*mu2*power2(cb)*power2(snt)*
     power4(sa)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 -
     mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(
     mH2) + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2)
     + 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(
     mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*
     power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) -
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(mh2 - mH2)*power2(mh2)*
     power2(mH2)*power2(sb)) + (mu2*power2(cb)*power2(Pi)*power2(snt)*power4(
     sa)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*
     mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2)
     + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) +
     14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(
     mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*
     power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) -
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 - mH2)*power2(mh2)*
     power2(mH2)*power2(sb)) + (21*mu2*power2(cb)*power2(sa)*power4(snt)*(3*
     mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*mH2*
     mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2) + 4*
     mst14*power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) + 14*
     mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(
     mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*
     power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) -
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(mh2 - mH2)*power2(mh2)*
     power2(mH2)*power2(sb)) + (mu2*power2(cb)*power2(Pi)*power2(sa)*power4(
     snt)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*
     mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2)
     + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) +
     14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(
     mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*
     power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) -
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 - mH2)*power2(mh2)*
     power2(mH2)*power2(sb)) + (21*ca*cb*mu2*power3(sa)*power4(snt)*(3*mH2*
     mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*mH2*mst24*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2) + 4*mst14*
     power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) + 14*mst14*
     DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*DeltaInv(
     mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(mst22,mst22,mh2)*
     power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) + 6*mH2*
     power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(mh2) + 18*mH2*mst14*
     DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*power3(mH2) - 3*mst22*power3(
     mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)*power3(mH2) - 12*mh2*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) - 18*mh2*mst24*DeltaInv(mst22,
     mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) +
     6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power2(mH2)*power4(mh2) + 16*mh2*DeltaInv(mst12,mst12,mh2)*
     power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*
     power6(mst2)))/((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) + (ca*cb*mu2*
     power2(Pi)*power3(sa)*power4(snt)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*
     power2(mh2) + 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) - 3*mh2*mst12*power2(mH2) + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(
     mH2) + 4*mst24*power2(mH2) + 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(
     mh2)*power2(mH2) - 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(
     mH2) + 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) +
     14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)
     *mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*
     power3(mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*
     mst12*power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) - 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(
     mh2 - mH2)*sb*power2(mh2)*power2(mH2)) - (3*mu2*power2(cb)*power2(sa)*(84*
     mH2*mst12*mt2*power2(mh2) - 84*mH2*mst22*mt2*power2(mh2) - 504*(mh2 - mH2)
     *mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 84*mh2*mst12*mt2*
     power2(mH2) + 112*mst14*mt2*power2(mH2) + 84*mh2*mst22*mt2*power2(mH2) -
     112*mst24*mt2*power2(mH2) + 392*mst14*mt2*DeltaInv(mst12,mst12,mh2)*
     power2(mh2)*power2(mH2) - 504*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(
     mh2)*power2(mH2) - 392*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 21*(mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)
     *power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 42*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst12) +
     42*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) -
     15*mH2*mst12*power3(mh2) + 15*mH2*mst22*power3(mh2) + 84*mst12*mt2*power3(
     mh2) - 84*mst22*mt2*power3(mh2) + 504*mH2*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power3(mh2) - 21*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12)*
     power3(mh2) + 21*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22)*
     power3(mh2) + 15*mh2*mst12*power3(mH2) - 15*mh2*mst22*power3(mH2) - 84*
     mst12*mt2*power3(mH2) + 84*mst22*mt2*power3(mH2) - 504*mh2*mst14*mt2*
     DeltaInv(mst12,mst12,mh2)*power3(mH2) + 504*mh2*mst24*mt2*DeltaInv(mst22,
     mst22,mh2)*power3(mH2) + 42*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mst12)*power3(mH2) + 21*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mst12)*power3(mH2) - 42*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mst22)*
     power3(mH2) - 21*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22)*
     power3(mH2) - 21*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) +
     21*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 21*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 448*mh2*mt2*DeltaInv(mst12,
     mst12,mh2)*power2(mH2)*power6(mst1) - 448*mh2*mt2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 - mH2)*(mst12 - mst22)*power2(
     mh2)*power2(mH2)*power2(sb)) + (3*mu2*power2(cb)*power4(sa)*(84*mH2*mst12*
     mt2*power2(mh2) - 84*mH2*mst22*mt2*power2(mh2) - 504*(mh2 - mH2)*mH2*
     mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 84*mh2*mst12*mt2*power2(
     mH2) + 112*mst14*mt2*power2(mH2) + 84*mh2*mst22*mt2*power2(mH2) - 112*
     mst24*mt2*power2(mH2) + 392*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(
     mh2)*power2(mH2) - 504*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*
     power2(mH2) - 392*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 21*(mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 42*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst12) +
     42*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) -
     15*mH2*mst12*power3(mh2) + 15*mH2*mst22*power3(mh2) + 84*mst12*mt2*power3(
     mh2) - 84*mst22*mt2*power3(mh2) + 504*mH2*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power3(mh2) - 21*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12)*
     power3(mh2) + 21*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22)*
     power3(mh2) + 15*mh2*mst12*power3(mH2) - 15*mh2*mst22*power3(mH2) - 84*
     mst12*mt2*power3(mH2) + 84*mst22*mt2*power3(mH2) - 504*mh2*mst14*mt2*
     DeltaInv(mst12,mst12,mh2)*power3(mH2) + 504*mh2*mst24*mt2*DeltaInv(mst22,
     mst22,mh2)*power3(mH2) + 42*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mst12)*power3(mH2) + 21*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mst12)*power3(mH2) - 42*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mst22)*
     power3(mH2) - 21*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22)*
     power3(mH2) - 21*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) +
     21*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 21*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 448*mh2*mt2*DeltaInv(mst12,
     mst12,mh2)*power2(mH2)*power6(mst1) - 448*mh2*mt2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 - mH2)*(mst12 - mst22)*power2(
     mh2)*power2(mH2)*power2(sb)) + (3*power2(sa)*(-42*mH2*mst12*mt2*power2(
     mh2) - 42*mH2*mst22*mt2*power2(mh2) - 252*(mh2 - mH2)*mH2*mst24*mt2*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) + 42*mh2*mst12*mt2*power2(mH2) - 56*
     mst14*mt2*power2(mH2) + 42*mh2*mst22*mt2*power2(mH2) - 56*mst24*mt2*
     power2(mH2) + 30*mu2*power2(mh2)*power2(mH2) + 84*mst12*mst22*mu2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*power2(mH2) - 196*mst14*mt2*
     DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 252*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 196*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 210*mH2*power2(mh2)*
     power2(mt2) - 210*mh2*power2(mH2)*power2(mt2) - 3*mH2*mst12*power3(mh2) -
     3*mH2*mst22*power3(mh2) - 36*mH2*mt2*power3(mh2) - 42*mst12*mt2*power3(
     mh2) - 42*mst22*mt2*power3(mh2) - 30*mH2*mu2*power3(mh2) - 84*mH2*mst12*
     mst22*mu2*DeltaInv(mst12,mH2,mst22)*power3(mh2) - 252*mH2*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power3(mh2) - 12*power2(mH2)*power3(mh2) - 42*
     mst12*mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2)*power3(mh2) - 42*mst22*
     mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2)*power3(mh2) - 42*power2(mt2)*
     power3(mh2) + 3*mh2*mst12*power3(mH2) + 3*mh2*mst22*power3(mH2) + 36*mh2*
     mt2*power3(mH2) + 42*mst12*mt2*power3(mH2) + 42*mst22*mt2*power3(mH2) +
     252*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 252*mh2*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 12*power2(mh2)*power3(mH2) +
     42*mst12*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*power3(mH2) + 42*mst22*
     mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*power3(mH2) + 42*power2(mt2)*
     power3(mH2) + 42*mu2*DeltaInv(mst12,mH2,mst22)*power3(mh2)*power3(mH2) +
     112*power2(mH2)*power3(mt2) - 112*mh2*(mh2 - 4*mt2)*DeltaInv(mh2,mt2,mt2)*
     power2(mH2)*power3(mt2) - 42*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*
     power4(mH2) - 224*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 224*mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(
     4.*(mh2 - mH2)*power2(mh2)*power2(mH2)) - (3*power4(sa)*(-42*mH2*mst12*
     mt2*power2(mh2) - 42*mH2*mst22*mt2*power2(mh2) - 252*(mh2 - mH2)*mH2*
     mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 42*mh2*mst12*mt2*power2(
     mH2) - 56*mst14*mt2*power2(mH2) + 42*mh2*mst22*mt2*power2(mH2) - 56*mst24*
     mt2*power2(mH2) + 42*mst12*mst22*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)
     *power2(mH2) - 196*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(
     mH2) + 252*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) -
     42*mst12*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) -
     196*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 210*mH2*
     power2(mh2)*power2(mt2) - 210*mh2*power2(mH2)*power2(mt2) - 3*mH2*mst12*
     power3(mh2) - 3*mH2*mst22*power3(mh2) - 36*mH2*mt2*power3(mh2) - 42*mst12*
     mt2*power3(mh2) - 42*mst22*mt2*power3(mh2) - 15*mH2*mu2*power3(mh2) - 42*
     mH2*mst12*mst22*mu2*DeltaInv(mst12,mH2,mst22)*power3(mh2) - 252*mH2*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 12*power2(mH2)*power3(mh2) -
     21*mst12*mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2)*power3(mh2) - 21*mst22*
     mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2)*power3(mh2) - 21*mst12*mu2*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 21*mst22*mu2*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 42*power2(mt2)*power3(mh2) + 3*
     mh2*mst12*power3(mH2) + 3*mh2*mst22*power3(mH2) + 36*mh2*mt2*power3(mH2) +
     42*mst12*mt2*power3(mH2) + 42*mst22*mt2*power3(mH2) + 15*mh2*mu2*power3(
     mH2) + 252*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 42*mh2*
     mst12*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power3(mH2) + 252*mh2*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power3(mH2) + 12*power2(mh2)*power3(mH2) + 21*
     mst12*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*power3(mH2) + 21*mst22*
     mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*power3(mH2) + 21*mst12*mu2*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 21*mst22*mu2*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 42*power2(mt2)*power3(mH2) +
     21*mu2*DeltaInv(mst12,mH2,mst22)*power3(mh2)*power3(mH2) - 21*mu2*
     DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 112*power2(mH2)*
     power3(mt2) - 112*mh2*(mh2 - 4*mt2)*DeltaInv(mh2,mt2,mt2)*power2(mH2)*
     power3(mt2) + 21*mu2*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) -
     21*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*power4(mH2) - 224*mh2*mt2*
     DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 224*mh2*mt2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 - mH2)*power2(mh2)*
     power2(mH2)) - (3*ca*cb*mu2*power3(sa)*(42*mH2*mst12*mt2*power2(mh2) - 42*
     mH2*mst22*mt2*power2(mh2) - 252*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,
     mst22,mst22)*power2(mh2) - 42*mh2*mst12*mt2*power2(mH2) + 56*mst14*mt2*
     power2(mH2) + 42*mh2*mst22*mt2*power2(mH2) - 56*mst24*mt2*power2(mH2) +
     196*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 252*
     mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 196*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 21*(mh2 - mH2)*
     mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22
     - mH2*(mst12 + mst22) + power2(mH2)) - 42*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst12) + 42*mst12*DeltaInv(mst12,mst22,mh2)
     *power2(mh2)*power2(mH2)*power2(mst22) - 15*mH2*mst12*power3(mh2) + 15*
     mH2*mst22*power3(mh2) + 42*mst12*mt2*power3(mh2) - 42*mst22*mt2*power3(
     mh2) + 252*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 21*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) + 21*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 15*mh2*
     mst12*power3(mH2) - 15*mh2*mst22*power3(mH2) - 42*mst12*mt2*power3(mH2) +
     42*mst22*mt2*power3(mH2) - 252*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*
     power3(mH2) + 252*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) +
     42*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mst12)*power3(mH2) + 21*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst12)*power3(mH2) - 42*mh2*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mH2) - 21*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mst22)*power3(mH2) - 21*mst12*
     DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power4(mh2) - 21*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 224*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 224*mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(
     mst2)))/(2.*(mh2 - mH2)*(mst12 - mst22)*sb*power2(mh2)*power2(mH2)) + (3*
     ca*cb*mu2*sa*(-252*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*
     power2(mh2) - 42*mh2*mst12*mt2*power2(mH2) + 28*mst14*mt2*power2(mH2) +
     42*mh2*mst22*mt2*power2(mH2) - 28*mst24*mt2*power2(mH2) + 15*mst12*power2(
     mh2)*power2(mH2) - 15*mst22*power2(mh2)*power2(mH2) - 28*mst14*mt2*
     DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 252*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 28*mst24*mt2*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 21*(mh2 - mH2)*mH2*(mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12
     + mst22) + power2(mH2)) - 15*mH2*mst12*power3(mh2) + 15*mH2*mst22*power3(
     mh2) + 42*mst12*mt2*power3(mh2) - 42*mst22*mt2*power3(mh2) + 252*mH2*
     mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 112*mh2*mt2*DeltaInv(
     mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 112*mh2*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/(2.*(mh2 - mH2)*(mst12 - mst22)*sb*
     power2(mh2)*power2(mH2)) - (mu2*power2(cb)*power2(Pi)*power2(sa)*(12*mH2*
     mst12*mt2*power2(mh2) - 12*mH2*mst22*mt2*power2(mh2) - 72*(mh2 - mH2)*mH2*
     mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 12*mh2*mst12*mt2*power2(
     mH2) + 16*mst14*mt2*power2(mH2) + 12*mh2*mst22*mt2*power2(mH2) - 16*mst24*
     mt2*power2(mH2) + 56*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*
     power2(mH2) - 72*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(
     mH2) - 56*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*
     (mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst12) + 6*mst12*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*mst12*power3(mh2)
     + 3*mH2*mst22*power3(mh2) + 12*mst12*mt2*power3(mh2) - 12*mst22*mt2*
     power3(mh2) + 72*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) + 3*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 3*mh2*
     mst12*power3(mH2) - 3*mh2*mst22*power3(mH2) - 12*mst12*mt2*power3(mH2) +
     12*mst22*mt2*power3(mH2) - 72*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*
     power3(mH2) + 72*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mst12)*power3(mH2) + 3*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst12)*power3(mH2) - 6*mh2*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mH2) - 3*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mst22)*power3(mH2) - 3*mst12*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 3*mst22*DeltaInv(mst12,mst22,
     mh2)*power3(mh2)*power3(mH2) + 3*mst12*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(
     mh2) + 64*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 64*
     mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(8.*(mh2 -
     mH2)*(mst12 - mst22)*power2(mh2)*power2(mH2)*power2(sb)) + (mu2*power2(cb)
     *power2(Pi)*power4(sa)*(12*mH2*mst12*mt2*power2(mh2) - 12*mH2*mst22*mt2*
     power2(mh2) - 72*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*
     power2(mh2) - 12*mh2*mst12*mt2*power2(mH2) + 16*mst14*mt2*power2(mH2) +
     12*mh2*mst22*mt2*power2(mH2) - 16*mst24*mt2*power2(mH2) + 56*mst14*mt2*
     DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 72*mst14*mt2*DeltaInv(
     mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 56*mst24*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*(mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2)*power2(mst12) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2)*power2(mst22) - 3*mH2*mst12*power3(mh2) + 3*mH2*mst22*power3(
     mh2) + 12*mst12*mt2*power3(mh2) - 12*mst22*mt2*power3(mh2) + 72*mH2*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst12)*power3(mh2) + 3*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst22)*power3(mh2) + 3*mh2*mst12*power3(mH2) - 3*mh2*
     mst22*power3(mH2) - 12*mst12*mt2*power3(mH2) + 12*mst22*mt2*power3(mH2) -
     72*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 72*mh2*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mh2*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mst12)*power3(mH2) + 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst12)*power3(mH2) - 6*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(
     mst22)*power3(mH2) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22)
     *power3(mH2) - 3*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) +
     3*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 3*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 64*mh2*mt2*DeltaInv(mst12,
     mst12,mh2)*power2(mH2)*power6(mst1) - 64*mh2*mt2*DeltaInv(mst22,mst22,mh2)
     *power2(mH2)*power6(mst2)))/(8.*(mh2 - mH2)*(mst12 - mst22)*power2(mh2)*
     power2(mH2)*power2(sb)) - (power2(Pi)*power4(sa)*(-6*mH2*mst12*mt2*power2(
     mh2) - 6*mH2*mst22*mt2*power2(mh2) - 36*(mh2 - mH2)*mH2*mst24*mt2*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) + 6*mh2*mst12*mt2*power2(mH2) - 8*
     mst14*mt2*power2(mH2) + 6*mh2*mst22*mt2*power2(mH2) - 8*mst24*mt2*power2(
     mH2) + 6*mst12*mst22*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*power2(mH2)
     - 28*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 36*
     mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 6*mst12*
     mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 28*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 30*mH2*power2(mh2)
     *power2(mt2) - 30*mh2*power2(mH2)*power2(mt2) - 6*mH2*mt2*power3(mh2) - 6*
     mst12*mt2*power3(mh2) - 6*mst22*mt2*power3(mh2) - 3*mH2*mu2*power3(mh2) -
     6*mH2*mst12*mst22*mu2*DeltaInv(mst12,mH2,mst22)*power3(mh2) - 36*mH2*
     mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*mst12*mu2*DeltaInv(
     mst12,mH2,mst22)*power2(mH2)*power3(mh2) - 3*mst22*mu2*DeltaInv(mst12,mH2,
     mst22)*power2(mH2)*power3(mh2) - 3*mst12*mu2*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power3(mh2) - 3*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power3(mh2) - 6*power2(mt2)*power3(mh2) + 6*mh2*mt2*power3(mH2) + 6*
     mst12*mt2*power3(mH2) + 6*mst22*mt2*power3(mH2) + 3*mh2*mu2*power3(mH2) +
     36*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 6*mh2*mst12*
     mst22*mu2*DeltaInv(mst12,mst22,mh2)*power3(mH2) + 36*mh2*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power3(mH2) + 3*mst12*mu2*DeltaInv(mst12,mH2,
     mst22)*power2(mh2)*power3(mH2) + 3*mst22*mu2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*power3(mH2) + 3*mst12*mu2*DeltaInv(mst12,mst22,mh2)*power2(
     mh2)*power3(mH2) + 3*mst22*mu2*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power3(mH2) + 6*power2(mt2)*power3(mH2) + 3*mu2*DeltaInv(mst12,mH2,mst22)*
     power3(mh2)*power3(mH2) - 3*mu2*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 16*power2(mH2)*power3(mt2) - 16*mh2*(mh2 - 4*mt2)*DeltaInv(
     mh2,mt2,mt2)*power2(mH2)*power3(mt2) + 3*mu2*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power4(mh2) - 3*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*
     power4(mH2) - 32*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 32*mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(
     8.*(mh2 - mH2)*power2(mh2)*power2(mH2)) - (ca*cb*mu2*power2(Pi)*power3(sa)
     *(6*mH2*mst12*mt2*power2(mh2) - 6*mH2*mst22*mt2*power2(mh2) - 36*(mh2 -
     mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 6*mh2*mst12*
     mt2*power2(mH2) + 8*mst14*mt2*power2(mH2) + 6*mh2*mst22*mt2*power2(mH2) -
     8*mst24*mt2*power2(mH2) + 28*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(
     mh2)*power2(mH2) - 36*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*
     power2(mH2) - 28*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*(mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst12) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*
     mst12*power3(mh2) + 3*mH2*mst22*power3(mh2) + 6*mst12*mt2*power3(mh2) - 6*
     mst22*mt2*power3(mh2) + 36*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(
     mh2) - 3*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) +
     3*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 3*mh2*
     mst12*power3(mH2) - 3*mh2*mst22*power3(mH2) - 6*mst12*mt2*power3(mH2) + 6*
     mst22*mt2*power3(mH2) - 36*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(
     mH2) + 36*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mh2*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mst12)*power3(mH2) + 3*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mst12)*power3(mH2) - 6*mh2*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mH2) - 3*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mst22)*power3(mH2) - 3*mst12*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 3*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mh2)*power3(mH2) + 3*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power4(mh2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) +
     32*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 32*mh2*
     mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 - mH2)*(
     mst12 - mst22)*sb*power2(mh2)*power2(mH2)) + (power2(Pi)*power2(sa)*(-3*
     mH2*mst12*mt2*power2(mh2) - 3*mH2*mst22*mt2*power2(mh2) - 18*(mh2 - mH2)*
     mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*mt2*
     power2(mH2) - 4*mst14*mt2*power2(mH2) + 3*mh2*mst22*mt2*power2(mH2) - 4*
     mst24*mt2*power2(mH2) + 3*mu2*power2(mh2)*power2(mH2) + 6*mst12*mst22*mu2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*power2(mH2) - 14*mst14*mt2*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*mt2*DeltaInv(mst12,
     mst12,mH2)*power2(mh2)*power2(mH2) - 14*mst24*mt2*DeltaInv(mst22,mst22,
     mh2)*power2(mh2)*power2(mH2) + 15*mH2*power2(mh2)*power2(mt2) - 15*mh2*
     power2(mH2)*power2(mt2) - 3*mH2*mt2*power3(mh2) - 3*mst12*mt2*power3(mh2)
     - 3*mst22*mt2*power3(mh2) - 3*mH2*mu2*power3(mh2) - 6*mH2*mst12*mst22*mu2*
     DeltaInv(mst12,mH2,mst22)*power3(mh2) - 18*mH2*mst14*mt2*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 3*mst12*mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2)
     *power3(mh2) - 3*mst22*mu2*DeltaInv(mst12,mH2,mst22)*power2(mH2)*power3(
     mh2) - 3*power2(mt2)*power3(mh2) + 3*mh2*mt2*power3(mH2) + 3*mst12*mt2*
     power3(mH2) + 3*mst22*mt2*power3(mH2) + 18*mh2*mst14*mt2*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) + 18*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*
     power3(mH2) + 3*mst12*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*power3(
     mH2) + 3*mst22*mu2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*power3(mH2) + 3*
     power2(mt2)*power3(mH2) + 3*mu2*DeltaInv(mst12,mH2,mst22)*power3(mh2)*
     power3(mH2) + 8*power2(mH2)*power3(mt2) - 8*mh2*(mh2 - 4*mt2)*DeltaInv(
     mh2,mt2,mt2)*power2(mH2)*power3(mt2) - 3*mu2*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*power4(mH2) - 16*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)
     *power6(mst1) - 16*mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(
     mst2)))/(4.*(mh2 - mH2)*power2(mh2)*power2(mH2)) + (ca*cb*mu2*sa*power2(
     Pi)*(-36*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) -
     6*mh2*mst12*mt2*power2(mH2) + 4*mst14*mt2*power2(mH2) + 6*mh2*mst22*mt2*
     power2(mH2) - 4*mst24*mt2*power2(mH2) + 3*mst12*power2(mh2)*power2(mH2) -
     3*mst22*power2(mh2)*power2(mH2) - 4*mst14*mt2*DeltaInv(mst12,mst12,mh2)*
     power2(mh2)*power2(mH2) - 36*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(
     mh2)*power2(mH2) + 4*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 3*(mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 3*mH2*
     mst12*power3(mh2) + 3*mH2*mst22*power3(mh2) + 6*mst12*mt2*power3(mh2) - 6*
     mst22*mt2*power3(mh2) + 36*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(
     mh2) + 16*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 16*
     mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/(4.*(mh2 -
     mH2)*(mst12 - mst22)*sb*power2(mh2)*power2(mH2)) - (3*mu2*power2(snt)*
     power2(log(mst22/Q2))*((-2*(mC2*mst22 - msb12*mst22 - mst24)*DeltaInv(mC2,
     mst22,msb12))/mC2 - (24*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,
     mH2,mst22))/mH2 - (24*mst22)/power2(mH2) + mst24/((msntau2 - mu2)*power2(
     msntau2 - mst22)) - (2*mst22)/power2(msntau2 - mu2) + mst24/((-msntau2 +
     mst22)*power2(msntau2 - mu2)) + (mst24*mu2)/(power2(msntau2 - mst22)*
     power2(msntau2 - mu2)) - power6(mst2)/(power2(msntau2 - mst22)*power2(
     msntau2 - mu2))))/16. + (mu2*power2(Pi)*power2(snt)*(-24/mH2 + (2*mC2 - (
     4*msb12*mst22)/mC2 - 2*(msb12 + mst22))*DeltaInv(mC2,mst22,msb12) - (72*
     mst24*DeltaInv(mH2,mst22,mst22))/mH2 - 2*mC2*DeltaInv(mst12,mC2,msb12) +
     2*msb12*DeltaInv(mst12,mC2,msb12) + 2*mst12*DeltaInv(mst12,mC2,msb12) + (
     4*msb12*mst12*DeltaInv(mst12,mC2,msb12))/mC2 + 24*mH2*DeltaInv(mst12,mH2,
     mst22) - 24*mst12*DeltaInv(mst12,mH2,mst22) - 24*mst22*DeltaInv(mst12,mH2,
     mst22) - (48*mst12*mst22*DeltaInv(mst12,mH2,mst22))/mH2 - (72*mst14*
     DeltaInv(mst12,mst12,mH2))/mH2 - (12*mst12)/power2(mH2) - (12*mst22)/
     power2(mH2) + mst14/((msntau2 - mu2)*power2(msntau2 - mst12)) + mst24/((-
     msntau2 + mu2)*power2(msntau2 - mst22)) - (2*mst12)/power2(msntau2 - mu2)
     - mst14/((msntau2 - mst12)*power2(msntau2 - mu2)) + (2*mst22)/power2(
     msntau2 - mu2) + mst24/((msntau2 - mst22)*power2(msntau2 - mu2)) + (mst14*
     mu2)/(power2(msntau2 - mst12)*power2(msntau2 - mu2)) - (mst24*mu2)/(
     power2(msntau2 - mst22)*power2(msntau2 - mu2)) - power6(mst1)/(power2(
     msntau2 - mst12)*power2(msntau2 - mu2)) + power6(mst2)/(power2(msntau2 -
     mst22)*power2(msntau2 - mu2))))/16. + (3*mu2*power2(snt)*(-1344/mH2 + (21*
     mst22)/((msntau2 - mst22)*(msntau2 - mu2)) + (21*mst12)/((msntau2 - mst12)
     *(-msntau2 + mu2)) - (4032*mst24*DeltaInv(mH2,mst22,mst22))/mH2 - 112*mC2*
     DeltaInv(mst12,mC2,msb12) + 112*msb12*DeltaInv(mst12,mC2,msb12) + 112*
     mst12*DeltaInv(mst12,mC2,msb12) + (224*msb12*mst12*DeltaInv(mst12,mC2,
     msb12))/mC2 + 1344*mH2*DeltaInv(mst12,mH2,mst22) - 1344*mst12*DeltaInv(
     mst12,mH2,mst22) - 1344*mst22*DeltaInv(mst12,mH2,mst22) - (2688*mst12*
     mst22*DeltaInv(mst12,mH2,mst22))/mH2 - (4032*mst14*DeltaInv(mst12,mst12,
     mH2))/mH2 + (112*DeltaInv(mC2,mst22,msb12)*(-2*msb12*mst22 - mC2*(msb12 +
     mst22) + power2(mC2)))/mC2 - (672*mst12)/power2(mH2) - (672*mst22)/power2(
     mH2) + (14*mst14)/((msntau2 - mu2)*power2(msntau2 - mst12)) + (14*mst24)/(
     (-msntau2 + mu2)*power2(msntau2 - mst22)) - (75*mst12)/power2(msntau2 -
     mu2) + (7*mst14)/((msntau2 - mst12)*power2(msntau2 - mu2)) + (75*mst22)/
     power2(msntau2 - mu2) + (7*mst24)/((-msntau2 + mst22)*power2(msntau2 -
     mu2)) - (21*mst12*mu2)/((msntau2 - mst12)*power2(msntau2 - mu2)) + (21*
     mst22*mu2)/((msntau2 - mst22)*power2(msntau2 - mu2)) + (14*mst14*mu2)/(
     power2(msntau2 - mst12)*power2(msntau2 - mu2)) - (14*mst24*mu2)/(power2(
     msntau2 - mst22)*power2(msntau2 - mu2)) - (14*power6(mst1))/(power2(
     msntau2 - mst12)*power2(msntau2 - mu2)) + (14*power6(mst2))/(power2(
     msntau2 - mst22)*power2(msntau2 - mu2))))/64. - (3*mu2*log(mst22/Q2)*((-3*
     mst22)/((msntau2 - mst22)*(msntau2 - mu2)) + (96*mst22*mu2)/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (96*mst22*mu2)/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (12*(mA2*mst22 - mst12*mst22
     - mst24)*DeltaInv(mA2,mst12,mst22))/mA2 - (12*(mC2*mst22 - msb12*mst22 -
     mst24)*DeltaInv(mC2,mst22,msb12))/mC2 - 36*mst22*DeltaInv(mst12,mH2,mst22)
     + (36*mst12*mst22*DeltaInv(mst12,mH2,mst22))/mH2 + (36*mst24*DeltaInv(
     mst12,mH2,mst22))/mH2 - 12*mst22*DeltaInv(mst22,mstau12,mt2) + (24*mst24*
     DeltaInv(mst22,mstau12,mt2))/(mstau12 - mu2) + (12*mst22*mt2*DeltaInv(
     mst22,mstau12,mt2))/(mstau12 - mu2) + (24*mst22*mu2*DeltaInv(mst22,
     mstau12,mt2))/(-mstau12 + mu2) + (288*mst24*mt2*mu2*DeltaInv(mst22,mt2,
     mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (288*mst24*
     mt2*mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(
     mstau22 - mu2)) - 12*mst22*DeltaInv(mstau22,mst22,mt2) + (24*mst24*
     DeltaInv(mstau22,mst22,mt2))/(mstau22 - mu2) + (12*mst22*mt2*DeltaInv(
     mstau22,mst22,mt2))/(mstau22 - mu2) + (24*mst22*mu2*DeltaInv(mstau22,
     mst22,mt2))/(-mstau22 + mu2) - (13*mst22)/power2(msntau2 - mu2) + (3*
     mst24)/((msntau2 - mst22)*power2(msntau2 - mu2)) + (3*mst22*mu2)/((-
     msntau2 + mst22)*power2(msntau2 - mu2)) + (8*mst22)/power2(mstau12 - mu2)
     + (12*mst24*mt2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) + (24*
     mst24*mu2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) + (12*mst22*
     mt2*mu2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) + (12*mst24*
     mt2*DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (24*mst24*mu2*
     DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (12*mst22*mt2*mu2*
     DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (8*mst22)/power2(mstau22
     - mu2) + (12*mst24*mt2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (
     24*mst24*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (12*mst22*
     mt2*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (12*mst24*mt2*
     DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (24*mst24*mu2*
     DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (12*mst22*mt2*mu2*
     DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (192*mst24*DeltaInv(
     mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12
     - mu2)) + (96*mst22*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (192*mst24*DeltaInv(mst22,
     mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)
     ) - (96*mst22*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) - (12*mst22*DeltaInv(mst22,mstau12,
     mt2)*power2(mu2))/power2(mstau12 - mu2) - (12*mst22*DeltaInv(mst22,mt2,
     mu2)*power2(mu2))/power2(mstau12 - mu2) - (12*mst22*DeltaInv(mst22,mt2,
     mu2)*power2(mu2))/power2(mstau22 - mu2) - (12*mst22*DeltaInv(mstau22,
     mst22,mt2)*power2(mu2))/power2(mstau22 - mu2) - (96*mst22*DeltaInv(mst22,
     mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)
     ) + (96*mst22*DeltaInv(mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) - (96*mu2*DeltaInv(mst22,mt2,mu2)*
     power6(mst2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (96*
     mu2*DeltaInv(mst22,mt2,mu2)*power6(mst2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) - (12*DeltaInv(mst22,mstau12,mt2)*power6(mst2))/
     power2(mstau12 - mu2) - (12*DeltaInv(mst22,mt2,mu2)*power6(mst2))/power2(
     mstau12 - mu2) - (12*DeltaInv(mst22,mt2,mu2)*power6(mst2))/power2(mstau22
     - mu2) - (12*DeltaInv(mstau22,mst22,mt2)*power6(mst2))/power2(mstau22 -
     mu2)))/16. + (3*mu2*power2(log(mst22/Q2))*((16*mst22*mu2)/((mst12 - mst22)
     *(mstau12 - mstau22)*(mstau12 - mu2)) - (16*mst22*mu2)/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) + (2*(mA2*mst22 - mst12*mst22 - mst24)
     *DeltaInv(mA2,mst12,mst22))/mA2 - (2*(mC2*mst22 - msb12*mst22 - mst24)*
     DeltaInv(mC2,mst22,msb12))/mC2 - 6*mst22*DeltaInv(mst12,mH2,mst22) + (6*
     mst12*mst22*DeltaInv(mst12,mH2,mst22))/mH2 + (6*mst24*DeltaInv(mst12,mH2,
     mst22))/mH2 - 2*mst22*DeltaInv(mst22,mstau12,mt2) + (4*mst24*DeltaInv(
     mst22,mstau12,mt2))/(mstau12 - mu2) + (2*mst22*mt2*DeltaInv(mst22,mstau12,
     mt2))/(mstau12 - mu2) + (4*mst22*mu2*DeltaInv(mst22,mstau12,mt2))/(-
     mstau12 + mu2) + (48*mst24*mt2*mu2*DeltaInv(mst22,mt2,mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (48*mst24*mt2*mu2*DeltaInv(
     mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - 2*
     mst22*DeltaInv(mstau22,mst22,mt2) + (4*mst24*DeltaInv(mstau22,mst22,mt2))/
     (mstau22 - mu2) + (2*mst22*mt2*DeltaInv(mstau22,mst22,mt2))/(mstau22 -
     mu2) + (4*mst22*mu2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) + mst24/
     ((msntau2 - mu2)*power2(msntau2 - mst22)) - (2*mst22)/power2(msntau2 -
     mu2) + mst24/((-msntau2 + mst22)*power2(msntau2 - mu2)) + (mst24*mu2)/(
     power2(msntau2 - mst22)*power2(msntau2 - mu2)) + (2*mst22)/power2(mstau12
     - mu2) + (2*mst24*mt2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) +
     (4*mst24*mu2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) + (2*
     mst22*mt2*mu2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) + (2*
     mst24*mt2*DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (4*mst24*mu2*
     DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (2*mst22*mt2*mu2*
     DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) + (2*mst22)/power2(mstau22
     - mu2) + (2*mst24*mt2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (4*
     mst24*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (2*mst22*mt2*
     mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) + (2*mst24*mt2*
     DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (4*mst24*mu2*
     DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (2*mst22*mt2*mu2*
     DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (32*mst24*DeltaInv(
     mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12
     - mu2)) + (16*mst22*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (32*mst24*DeltaInv(mst22,
     mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)
     ) - (16*mst22*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) - (2*mst22*DeltaInv(mst22,mstau12,mt2)
     *power2(mu2))/power2(mstau12 - mu2) - (2*mst22*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*mst22*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mst22*DeltaInv(mstau22,mst22,mt2)*
     power2(mu2))/power2(mstau22 - mu2) - (16*mst22*DeltaInv(mst22,mt2,mu2)*
     power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (16*
     mst22*DeltaInv(mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) - (16*mu2*DeltaInv(mst22,mt2,mu2)*power6(mst2))/
     ((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (16*mu2*DeltaInv(
     mst22,mt2,mu2)*power6(mst2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) - power6(mst2)/(power2(msntau2 - mst22)*power2(msntau2 - mu2)) - (
     2*DeltaInv(mst22,mstau12,mt2)*power6(mst2))/power2(mstau12 - mu2) - (2*
     DeltaInv(mst22,mt2,mu2)*power6(mst2))/power2(mstau12 - mu2) - (2*DeltaInv(
     mst22,mt2,mu2)*power6(mst2))/power2(mstau22 - mu2) - (2*DeltaInv(mstau22,
     mst22,mt2)*power6(mst2))/power2(mstau22 - mu2)))/16. - (3*mu2*power2(log(
     mt2/Q2))*((24*mst12*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 -
     mu2)) - (24*mst22*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 -
     mu2)) - (24*mst12*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 -
     mu2)) + (24*mst22*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 -
     mu2)) + 1/(-msntau2 + mu2) + 2/(-mstau12 + mu2) + 2/(-mstau22 + mu2) -
     mst12*DeltaInv(mst12,mstau12,mt2) + mstau12*DeltaInv(mst12,mstau12,mt2) -
     mt2*DeltaInv(mst12,mstau12,mt2) + 2*mu2*DeltaInv(mst12,mstau12,mt2) + (
     mst14*DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (6*mst12*mt2*
     DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (2*mst12*mu2*DeltaInv(
     mst12,mstau12,mt2))/(-mstau12 + mu2) + (2*mt2*mu2*DeltaInv(mst12,mstau12,
     mt2))/(-mstau12 + mu2) + (40*mst14*mt2*mu2*DeltaInv(mst12,mt2,mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (40*mst14*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 -
     mu2)) - mst22*DeltaInv(mst22,mstau12,mt2) + mstau12*DeltaInv(mst22,
     mstau12,mt2) - mt2*DeltaInv(mst22,mstau12,mt2) + 2*mu2*DeltaInv(mst22,
     mstau12,mt2) + (mst24*DeltaInv(mst22,mstau12,mt2))/(-mstau12 + mu2) + (6*
     mst22*mt2*DeltaInv(mst22,mstau12,mt2))/(-mstau12 + mu2) + (2*mst22*mu2*
     DeltaInv(mst22,mstau12,mt2))/(-mstau12 + mu2) + (2*mt2*mu2*DeltaInv(mst22,
     mstau12,mt2))/(-mstau12 + mu2) - (40*mst24*mt2*mu2*DeltaInv(mst22,mt2,mu2)
     )/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (40*mst24*mt2*
     mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) - mst12*DeltaInv(mstau22,mst12,mt2) + mstau22*DeltaInv(mstau22,
     mst12,mt2) - mt2*DeltaInv(mstau22,mst12,mt2) + 2*mu2*DeltaInv(mstau22,
     mst12,mt2) + (mst14*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (6*
     mst12*mt2*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (2*mst12*mu2*
     DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (2*mt2*mu2*DeltaInv(
     mstau22,mst12,mt2))/(-mstau22 + mu2) - mst22*DeltaInv(mstau22,mst22,mt2) +
     mstau22*DeltaInv(mstau22,mst22,mt2) - mt2*DeltaInv(mstau22,mst22,mt2) + 2*
     mu2*DeltaInv(mstau22,mst22,mt2) + (mst24*DeltaInv(mstau22,mst22,mt2))/(-
     mstau22 + mu2) + (6*mst22*mt2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 +
     mu2) + (2*mst22*mu2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) + (2*
     mt2*mu2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) - (2*msb12)/power2(
     msntau2 - mu2) - mt2/power2(msntau2 - mu2) - (2*mu2)/power2(msntau2 - mu2)
     - (2*mst12)/power2(mstau12 - mu2) - (2*mst22)/power2(mstau12 - mu2) - (2*
     mt2)/power2(mstau12 - mu2) - (4*mu2)/power2(mstau12 - mu2) - (mst14*mt2*
     DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (mst14*mu2*DeltaInv(
     mst12,mstau12,mt2))/power2(mstau12 - mu2) - (6*mst12*mt2*mu2*DeltaInv(
     mst12,mstau12,mt2))/power2(mstau12 - mu2) - (mst14*mt2*DeltaInv(mst12,mt2,
     mu2))/power2(mstau12 - mu2) - (mst14*mu2*DeltaInv(mst12,mt2,mu2))/power2(
     mstau12 - mu2) - (6*mst12*mt2*mu2*DeltaInv(mst12,mt2,mu2))/power2(mstau12
     - mu2) - (mst24*mt2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) - (
     mst24*mu2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) - (6*mst22*
     mt2*mu2*DeltaInv(mst22,mstau12,mt2))/power2(mstau12 - mu2) - (mst24*mt2*
     DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) - (mst24*mu2*DeltaInv(
     mst22,mt2,mu2))/power2(mstau12 - mu2) - (6*mst22*mt2*mu2*DeltaInv(mst22,
     mt2,mu2))/power2(mstau12 - mu2) - (2*mst12)/power2(mstau22 - mu2) - (2*
     mst22)/power2(mstau22 - mu2) - (2*mt2)/power2(mstau22 - mu2) - (4*mu2)/
     power2(mstau22 - mu2) - (mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau22
     - mu2) - (mst14*mu2*DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (6*
     mst12*mt2*mu2*DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (mst24*mt2*
     DeltaInv(mst22,mt2,mu2))/power2(mstau22 - mu2) - (mst24*mu2*DeltaInv(
     mst22,mt2,mu2))/power2(mstau22 - mu2) - (6*mst22*mt2*mu2*DeltaInv(mst22,
     mt2,mu2))/power2(mstau22 - mu2) - (mst14*mt2*DeltaInv(mstau22,mst12,mt2))/
     power2(mstau22 - mu2) - (mst14*mu2*DeltaInv(mstau22,mst12,mt2))/power2(
     mstau22 - mu2) - (6*mst12*mt2*mu2*DeltaInv(mstau22,mst12,mt2))/power2(
     mstau22 - mu2) - (mst24*mt2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 -
     mu2) - (mst24*mu2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) - (6*
     mst22*mt2*mu2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (3*
     DeltaInv(mst12,mstau12,mt2)*power2(mu2))/(mstau12 - mu2) + (40*mst14*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau12 - mu2)) + (80*mst12*mt2*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (40*mst14*DeltaInv(
     mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) - (80*mst12*mt2*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (3*DeltaInv(mst22,mstau12,
     mt2)*power2(mu2))/(mstau12 - mu2) - (40*mst24*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (80*
     mst22*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) + (40*mst24*DeltaInv(mst22,mt2,mu2)*power2(mu2))
     /((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (80*mst22*mt2*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau22 - mu2)) + (3*DeltaInv(mstau22,mst12,mt2)*power2(mu2))/(mstau22 -
     mu2) + (3*DeltaInv(mstau22,mst22,mt2)*power2(mu2))/(mstau22 - mu2) - (
     mst12*DeltaInv(mst12,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (
     mt2*DeltaInv(mst12,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (
     mst12*DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (mt2*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (mst22*
     DeltaInv(mst22,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (mt2*
     DeltaInv(mst22,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (mst22*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (mt2*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (mst12*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (mt2*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (mst22*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (mt2*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (mst12*
     DeltaInv(mstau22,mst12,mt2)*power2(mu2))/power2(mstau22 - mu2) - (mt2*
     DeltaInv(mstau22,mst12,mt2)*power2(mu2))/power2(mstau22 - mu2) - (mst22*
     DeltaInv(mstau22,mst22,mt2)*power2(mu2))/power2(mstau22 - mu2) - (mt2*
     DeltaInv(mstau22,mst22,mt2)*power2(mu2))/power2(mstau22 - mu2) - (8*mst12*
     DeltaInv(mst12,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau12 - mu2)) + (8*mt2*DeltaInv(mst12,mt2,mu2)*power3(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (8*mst12*DeltaInv(mst12,mt2,
     mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) -
     (8*mt2*DeltaInv(mst12,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) + (8*mst22*DeltaInv(mst22,mt2,mu2)*power3(mu2))/
     ((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (8*mt2*DeltaInv(
     mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12
     - mu2)) - (8*mst22*DeltaInv(mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) + (8*mt2*DeltaInv(mst22,mt2,mu2)*
     power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (
     DeltaInv(mst12,mstau12,mt2)*power3(mu2))/power2(mstau12 - mu2) + (
     DeltaInv(mst12,mt2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (DeltaInv(
     mst22,mstau12,mt2)*power3(mu2))/power2(mstau12 - mu2) + (DeltaInv(mst22,
     mt2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (DeltaInv(mst12,mt2,mu2)*
     power3(mu2))/power2(mstau22 - mu2) + (DeltaInv(mst22,mt2,mu2)*power3(mu2))
     /power2(mstau22 - mu2) + (DeltaInv(mstau22,mst12,mt2)*power3(mu2))/power2(
     mstau22 - mu2) + (DeltaInv(mstau22,mst22,mt2)*power3(mu2))/power2(mstau22
     - mu2) - (8*DeltaInv(mst12,mt2,mu2)*power4(mu2))/((mst12 - mst22)*(mstau12
     - mstau22)*(mstau12 - mu2)) + (8*DeltaInv(mst12,mt2,mu2)*power4(mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (8*DeltaInv(mst22,
     mt2,mu2)*power4(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)
     ) - (8*DeltaInv(mst22,mt2,mu2)*power4(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) + (DeltaInv(msntau2,msb12,mt2)*(-(msb14*(msntau2
     + mt2)) + msntau2*(msntau2*(msntau2 - mt2) - msb12*(msntau2 + 6*mt2)) +
     power6(msb1)))/power2(msntau2 - mu2) + (DeltaInv(msb12,mt2,mu2)*(-(msb14*(
     mt2 + mu2)) + mu2*(mu2*(-mt2 + mu2) - msb12*(6*mt2 + mu2)) + power6(msb1))
     )/power2(msntau2 - mu2) - (24*mu2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (24*mu2*DeltaInv(
     mst12,mt2,mu2)*power6(mst1))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) + (DeltaInv(mst12,mstau12,mt2)*power6(mst1))/power2(mstau12 - mu2)
     + (DeltaInv(mst12,mt2,mu2)*power6(mst1))/power2(mstau12 - mu2) + (
     DeltaInv(mst12,mt2,mu2)*power6(mst1))/power2(mstau22 - mu2) + (DeltaInv(
     mstau22,mst12,mt2)*power6(mst1))/power2(mstau22 - mu2) + (24*mu2*DeltaInv(
     mst22,mt2,mu2)*power6(mst2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12
     - mu2)) - (24*mu2*DeltaInv(mst22,mt2,mu2)*power6(mst2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) + (DeltaInv(mst22,mstau12,mt2)*power6(
     mst2))/power2(mstau12 - mu2) + (DeltaInv(mst22,mt2,mu2)*power6(mst2))/
     power2(mstau12 - mu2) + (DeltaInv(mst22,mt2,mu2)*power6(mst2))/power2(
     mstau22 - mu2) + (DeltaInv(mstau22,mst22,mt2)*power6(mst2))/power2(mstau22
     - mu2)))/8. - (mu2*power2(Pi)*(2/mA2 - 2/mC2 - 6/mH2 + (32*mst12*mu2)/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (32*mst22*mu2)/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (32*mst12*mu2)/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (32*mst22*mu2)/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + 2/(-msntau2 + mu2) +
     4/(-mstau12 + mu2) + 4/(-mstau22 + mu2) + 2*(-mA2 + mst12 + mst22 + (2*
     mst12*mst22)/mA2)*DeltaInv(mA2,mst12,mst22) + (2*mC2 - (4*msb12*mst22)/mC2
     - 2*(msb12 + mst22))*DeltaInv(mC2,mst22,msb12) - 2*msb12*DeltaInv(msntau2,
     msb12,mt2) + 2*msntau2*DeltaInv(msntau2,msb12,mt2) - 2*mt2*DeltaInv(
     msntau2,msb12,mt2) + 4*mu2*DeltaInv(msntau2,msb12,mt2) + (2*msb14*
     DeltaInv(msntau2,msb12,mt2))/(-msntau2 + mu2) + (8*msb12*mt2*DeltaInv(
     msntau2,msb12,mt2))/(-msntau2 + mu2) + (4*msb12*mu2*DeltaInv(msntau2,
     msb12,mt2))/(-msntau2 + mu2) + (4*mt2*mu2*DeltaInv(msntau2,msb12,mt2))/(-
     msntau2 + mu2) + 6*mH2*DeltaInv(mst12,mH2,mst22) - 6*mst12*DeltaInv(mst12,
     mH2,mst22) - 6*mst22*DeltaInv(mst12,mH2,mst22) - (12*mst12*mst22*DeltaInv(
     mst12,mH2,mst22))/mH2 - 2*mst12*DeltaInv(mst12,mstau12,mt2) + 2*mstau12*
     DeltaInv(mst12,mstau12,mt2) - 2*mt2*DeltaInv(mst12,mstau12,mt2) + 4*mu2*
     DeltaInv(mst12,mstau12,mt2) + (2*mst14*DeltaInv(mst12,mstau12,mt2))/(-
     mstau12 + mu2) + (8*mst12*mt2*DeltaInv(mst12,mstau12,mt2))/(-mstau12 +
     mu2) + (4*mst12*mu2*DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (4*
     mt2*mu2*DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (64*mst14*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 -
     mu2)) - (64*mst14*mt2*mu2*DeltaInv(mst12,mt2,mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) - 2*mst22*DeltaInv(mst22,mstau12,mt2)
     + 2*mstau12*DeltaInv(mst22,mstau12,mt2) - 2*mt2*DeltaInv(mst22,mstau12,
     mt2) + 4*mu2*DeltaInv(mst22,mstau12,mt2) + (2*mst24*DeltaInv(mst22,
     mstau12,mt2))/(-mstau12 + mu2) + (8*mst22*mt2*DeltaInv(mst22,mstau12,mt2))
     /(-mstau12 + mu2) + (4*mst22*mu2*DeltaInv(mst22,mstau12,mt2))/(-mstau12 +
     mu2) + (4*mt2*mu2*DeltaInv(mst22,mstau12,mt2))/(-mstau12 + mu2) - (64*
     mst24*mt2*mu2*DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) + (64*mst24*mt2*mu2*DeltaInv(mst22,mt2,mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - 2*mst12*DeltaInv(
     mstau22,mst12,mt2) + 2*mstau22*DeltaInv(mstau22,mst12,mt2) - 2*mt2*
     DeltaInv(mstau22,mst12,mt2) + 4*mu2*DeltaInv(mstau22,mst12,mt2) + (2*
     mst14*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (8*mst12*mt2*
     DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (4*mst12*mu2*DeltaInv(
     mstau22,mst12,mt2))/(-mstau22 + mu2) + (4*mt2*mu2*DeltaInv(mstau22,mst12,
     mt2))/(-mstau22 + mu2) - 2*mst22*DeltaInv(mstau22,mst22,mt2) + 2*mstau22*
     DeltaInv(mstau22,mst22,mt2) - 2*mt2*DeltaInv(mstau22,mst22,mt2) + 4*mu2*
     DeltaInv(mstau22,mst22,mt2) + (2*mst24*DeltaInv(mstau22,mst22,mt2))/(-
     mstau22 + mu2) + (8*mst22*mt2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 +
     mu2) + (4*mst22*mu2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) + (4*
     mt2*mu2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) + mst24/((-msntau2 +
     mu2)*power2(msntau2 - mst22)) - (2*msb12)/power2(msntau2 - mu2) + (2*
     mst22)/power2(msntau2 - mu2) + mst24/((msntau2 - mst22)*power2(msntau2 -
     mu2)) - (2*mt2)/power2(msntau2 - mu2) - (4*mu2)/power2(msntau2 - mu2) - (
     2*msb14*mt2*DeltaInv(msb12,mt2,mu2))/power2(msntau2 - mu2) - (2*msb14*mu2*
     DeltaInv(msb12,mt2,mu2))/power2(msntau2 - mu2) - (8*msb12*mt2*mu2*
     DeltaInv(msb12,mt2,mu2))/power2(msntau2 - mu2) - (2*msb14*mt2*DeltaInv(
     msntau2,msb12,mt2))/power2(msntau2 - mu2) - (2*msb14*mu2*DeltaInv(msntau2,
     msb12,mt2))/power2(msntau2 - mu2) - (8*msb12*mt2*mu2*DeltaInv(msntau2,
     msb12,mt2))/power2(msntau2 - mu2) - (mst24*mu2)/(power2(msntau2 - mst22)*
     power2(msntau2 - mu2)) - (2*mst12)/power2(mstau12 - mu2) - (2*mst22)/
     power2(mstau12 - mu2) - (4*mt2)/power2(mstau12 - mu2) - (8*mu2)/power2(
     mstau12 - mu2) - (2*mst14*mt2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12
     - mu2) - (2*mst14*mu2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) -
     (8*mst12*mt2*mu2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (2*
     mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (2*mst14*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (8*mst12*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (2*mst24*mt2*DeltaInv(
     mst22,mstau12,mt2))/power2(mstau12 - mu2) - (2*mst24*mu2*DeltaInv(mst22,
     mstau12,mt2))/power2(mstau12 - mu2) - (8*mst22*mt2*mu2*DeltaInv(mst22,
     mstau12,mt2))/power2(mstau12 - mu2) - (2*mst24*mt2*DeltaInv(mst22,mt2,mu2)
     )/power2(mstau12 - mu2) - (2*mst24*mu2*DeltaInv(mst22,mt2,mu2))/power2(
     mstau12 - mu2) - (8*mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau12
     - mu2) - (2*mst12)/power2(mstau22 - mu2) - (2*mst22)/power2(mstau22 - mu2)
     - (4*mt2)/power2(mstau22 - mu2) - (8*mu2)/power2(mstau22 - mu2) - (2*
     mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (2*mst14*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (8*mst12*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (2*mst24*mt2*DeltaInv(
     mst22,mt2,mu2))/power2(mstau22 - mu2) - (2*mst24*mu2*DeltaInv(mst22,mt2,
     mu2))/power2(mstau22 - mu2) - (8*mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/
     power2(mstau22 - mu2) - (2*mst14*mt2*DeltaInv(mstau22,mst12,mt2))/power2(
     mstau22 - mu2) - (2*mst14*mu2*DeltaInv(mstau22,mst12,mt2))/power2(mstau22
     - mu2) - (8*mst12*mt2*mu2*DeltaInv(mstau22,mst12,mt2))/power2(mstau22 -
     mu2) - (2*mst24*mt2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) - (
     2*mst24*mu2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) - (8*mst22*
     mt2*mu2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (6*DeltaInv(
     msntau2,msb12,mt2)*power2(mu2))/(msntau2 - mu2) + (6*DeltaInv(mst12,
     mstau12,mt2)*power2(mu2))/(mstau12 - mu2) + (48*mst14*DeltaInv(mst12,mt2,
     mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) +
     (112*mst12*mt2*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau12 - mu2)) - (48*mst14*DeltaInv(mst12,mt2,mu2)*
     power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (112*
     mst12*mt2*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) + (6*DeltaInv(mst22,mstau12,mt2)*power2(mu2))/(
     mstau12 - mu2) - (48*mst24*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (112*mst22*mt2*DeltaInv(
     mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12
     - mu2)) + (48*mst24*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*
     (mstau12 - mstau22)*(mstau22 - mu2)) + (112*mst22*mt2*DeltaInv(mst22,mt2,
     mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) +
     (6*DeltaInv(mstau22,mst12,mt2)*power2(mu2))/(mstau22 - mu2) + (6*DeltaInv(
     mstau22,mst22,mt2)*power2(mu2))/(mstau22 - mu2) - (2*msb12*DeltaInv(msb12,
     mt2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (2*mt2*DeltaInv(msb12,mt2,
     mu2)*power2(mu2))/power2(msntau2 - mu2) - (2*msb12*DeltaInv(msntau2,msb12,
     mt2)*power2(mu2))/power2(msntau2 - mu2) - (2*mt2*DeltaInv(msntau2,msb12,
     mt2)*power2(mu2))/power2(msntau2 - mu2) - (2*mst12*DeltaInv(mst12,mstau12,
     mt2)*power2(mu2))/power2(mstau12 - mu2) - (2*mt2*DeltaInv(mst12,mstau12,
     mt2)*power2(mu2))/power2(mstau12 - mu2) - (2*mst12*DeltaInv(mst12,mt2,mu2)
     *power2(mu2))/power2(mstau12 - mu2) - (2*mt2*DeltaInv(mst12,mt2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*mst22*DeltaInv(mst22,mstau12,mt2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*mt2*DeltaInv(mst22,mstau12,mt2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*mst22*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*mt2*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/power2(mstau12 - mu2) - (2*mst12*DeltaInv(mst12,mt2,mu2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mt2*DeltaInv(mst12,mt2,mu2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mst22*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mt2*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mst12*DeltaInv(mstau22,mst12,mt2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mt2*DeltaInv(mstau22,mst12,mt2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mst22*DeltaInv(mstau22,mst22,mt2)*
     power2(mu2))/power2(mstau22 - mu2) - (2*mt2*DeltaInv(mstau22,mst22,mt2)*
     power2(mu2))/power2(mstau22 - mu2) + (16*mt2*DeltaInv(mst12,mt2,mu2)*
     power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (16*
     mt2*DeltaInv(mst12,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) - (16*mt2*DeltaInv(mst22,mt2,mu2)*power3(mu2))/(
     (mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (16*mt2*DeltaInv(
     mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) + (2*DeltaInv(msb12,mt2,mu2)*power3(mu2))/power2(msntau2 - mu2) +
     (2*DeltaInv(msntau2,msb12,mt2)*power3(mu2))/power2(msntau2 - mu2) + (2*
     DeltaInv(mst12,mstau12,mt2)*power3(mu2))/power2(mstau12 - mu2) + (2*
     DeltaInv(mst12,mt2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (2*DeltaInv(
     mst22,mstau12,mt2)*power3(mu2))/power2(mstau12 - mu2) + (2*DeltaInv(mst22,
     mt2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (2*DeltaInv(mst12,mt2,mu2)*
     power3(mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mst22,mt2,mu2)*power3(
     mu2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mst12,mt2)*power3(mu2))/
     power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mst22,mt2)*power3(mu2))/
     power2(mstau22 - mu2) - (16*DeltaInv(mst12,mt2,mu2)*power4(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (16*DeltaInv(mst12,mt2,mu2)*
     power4(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (16*
     DeltaInv(mst22,mt2,mu2)*power4(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau12 - mu2)) - (16*DeltaInv(mst22,mt2,mu2)*power4(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (2*DeltaInv(msb12,mt2,mu2)*
     power6(msb1))/power2(msntau2 - mu2) + (2*DeltaInv(msntau2,msb12,mt2)*
     power6(msb1))/power2(msntau2 - mu2) - (32*mu2*DeltaInv(mst12,mt2,mu2)*
     power6(mst1))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (32*
     mu2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) + (2*DeltaInv(mst12,mstau12,mt2)*power6(mst1))/
     power2(mstau12 - mu2) + (2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/power2(
     mstau12 - mu2) + (2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/power2(mstau22 -
     mu2) + (2*DeltaInv(mstau22,mst12,mt2)*power6(mst1))/power2(mstau22 - mu2)
     + (32*mu2*DeltaInv(mst22,mt2,mu2)*power6(mst2))/((mst12 - mst22)*(mstau12
     - mstau22)*(mstau12 - mu2)) - (32*mu2*DeltaInv(mst22,mt2,mu2)*power6(mst2)
     )/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + power6(mst2)/(
     power2(msntau2 - mst22)*power2(msntau2 - mu2)) + (2*DeltaInv(mst22,
     mstau12,mt2)*power6(mst2))/power2(mstau12 - mu2) + (2*DeltaInv(mst22,mt2,
     mu2)*power6(mst2))/power2(mstau12 - mu2) + (2*DeltaInv(mst22,mt2,mu2)*
     power6(mst2))/power2(mstau22 - mu2) + (2*DeltaInv(mstau22,mst22,mt2)*
     power6(mst2))/power2(mstau22 - mu2)))/16. + (3*mu2*log(mt2/Q2)*((72*mst12*
     mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (72*mst22*
     mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (72*mst12*
     mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (72*mst22*
     mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + 3/(-msntau2 +
     mu2) + 6/(-mstau12 + mu2) + 6/(-mstau22 + mu2) - 3*mst12*DeltaInv(mst12,
     mstau12,mt2) + 3*mstau12*DeltaInv(mst12,mstau12,mt2) - 3*mt2*DeltaInv(
     mst12,mstau12,mt2) + 6*mu2*DeltaInv(mst12,mstau12,mt2) + (3*mst14*
     DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (18*mst12*mt2*DeltaInv(
     mst12,mstau12,mt2))/(-mstau12 + mu2) + (6*mst12*mu2*DeltaInv(mst12,
     mstau12,mt2))/(-mstau12 + mu2) + (6*mt2*mu2*DeltaInv(mst12,mstau12,mt2))/(
     -mstau12 + mu2) + (120*mst14*mt2*mu2*DeltaInv(mst12,mt2,mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (120*mst14*mt2*mu2*DeltaInv(
     mst12,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - 3*
     mst22*DeltaInv(mst22,mstau12,mt2) + 3*mstau12*DeltaInv(mst22,mstau12,mt2)
     - 3*mt2*DeltaInv(mst22,mstau12,mt2) + 6*mu2*DeltaInv(mst22,mstau12,mt2) +
     (3*mst24*DeltaInv(mst22,mstau12,mt2))/(-mstau12 + mu2) + (18*mst22*mt2*
     DeltaInv(mst22,mstau12,mt2))/(-mstau12 + mu2) + (6*mst22*mu2*DeltaInv(
     mst22,mstau12,mt2))/(-mstau12 + mu2) + (6*mt2*mu2*DeltaInv(mst22,mstau12,
     mt2))/(-mstau12 + mu2) - (120*mst24*mt2*mu2*DeltaInv(mst22,mt2,mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (120*mst24*mt2*mu2*
     DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 -
     mu2)) - 3*mst12*DeltaInv(mstau22,mst12,mt2) + 3*mstau22*DeltaInv(mstau22,
     mst12,mt2) - 3*mt2*DeltaInv(mstau22,mst12,mt2) + 6*mu2*DeltaInv(mstau22,
     mst12,mt2) + (3*mst14*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (18*
     mst12*mt2*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (6*mst12*mu2*
     DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (6*mt2*mu2*DeltaInv(
     mstau22,mst12,mt2))/(-mstau22 + mu2) - 3*mst22*DeltaInv(mstau22,mst22,mt2)
     + 3*mstau22*DeltaInv(mstau22,mst22,mt2) - 3*mt2*DeltaInv(mstau22,mst22,
     mt2) + 6*mu2*DeltaInv(mstau22,mst22,mt2) + (3*mst24*DeltaInv(mstau22,
     mst22,mt2))/(-mstau22 + mu2) + (18*mst22*mt2*DeltaInv(mstau22,mst22,mt2))/
     (-mstau22 + mu2) + (6*mst22*mu2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 +
     mu2) + (6*mt2*mu2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) - (6*
     msb12)/power2(msntau2 - mu2) - (4*mt2)/power2(msntau2 - mu2) - (6*mu2)/
     power2(msntau2 - mu2) - (6*mst12)/power2(mstau12 - mu2) - (6*mst22)/
     power2(mstau12 - mu2) - (8*mt2)/power2(mstau12 - mu2) - (12*mu2)/power2(
     mstau12 - mu2) - (3*mst14*mt2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12
     - mu2) - (3*mst14*mu2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) -
     (18*mst12*mt2*mu2*DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (3*
     mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (3*mst14*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (18*mst12*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (3*mst24*mt2*DeltaInv(
     mst22,mstau12,mt2))/power2(mstau12 - mu2) - (3*mst24*mu2*DeltaInv(mst22,
     mstau12,mt2))/power2(mstau12 - mu2) - (18*mst22*mt2*mu2*DeltaInv(mst22,
     mstau12,mt2))/power2(mstau12 - mu2) - (3*mst24*mt2*DeltaInv(mst22,mt2,mu2)
     )/power2(mstau12 - mu2) - (3*mst24*mu2*DeltaInv(mst22,mt2,mu2))/power2(
     mstau12 - mu2) - (18*mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau12
     - mu2) - (6*mst12)/power2(mstau22 - mu2) - (6*mst22)/power2(mstau22 - mu2)
     - (8*mt2)/power2(mstau22 - mu2) - (12*mu2)/power2(mstau22 - mu2) - (3*
     mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (3*mst14*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (18*mst12*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (3*mst24*mt2*DeltaInv(
     mst22,mt2,mu2))/power2(mstau22 - mu2) - (3*mst24*mu2*DeltaInv(mst22,mt2,
     mu2))/power2(mstau22 - mu2) - (18*mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/
     power2(mstau22 - mu2) - (3*mst14*mt2*DeltaInv(mstau22,mst12,mt2))/power2(
     mstau22 - mu2) - (3*mst14*mu2*DeltaInv(mstau22,mst12,mt2))/power2(mstau22
     - mu2) - (18*mst12*mt2*mu2*DeltaInv(mstau22,mst12,mt2))/power2(mstau22 -
     mu2) - (3*mst24*mt2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) - (
     3*mst24*mu2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) - (18*
     mst22*mt2*mu2*DeltaInv(mstau22,mst22,mt2))/power2(mstau22 - mu2) + (9*
     DeltaInv(mst12,mstau12,mt2)*power2(mu2))/(mstau12 - mu2) + (120*mst14*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau12 - mu2)) + (240*mst12*mt2*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (120*mst14*DeltaInv(
     mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) - (240*mst12*mt2*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (9*DeltaInv(mst22,mstau12,
     mt2)*power2(mu2))/(mstau12 - mu2) - (120*mst24*DeltaInv(mst22,mt2,mu2)*
     power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (240*
     mst22*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) + (120*mst24*DeltaInv(mst22,mt2,mu2)*power2(mu2)
     )/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (240*mst22*mt2*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau22 - mu2)) + (9*DeltaInv(mstau22,mst12,mt2)*power2(mu2))/(mstau22 -
     mu2) + (9*DeltaInv(mstau22,mst22,mt2)*power2(mu2))/(mstau22 - mu2) - (3*
     mst12*DeltaInv(mst12,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (3*
     mt2*DeltaInv(mst12,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (3*
     mst12*DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (3*mt2*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (3*mst22*
     DeltaInv(mst22,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (3*mt2*
     DeltaInv(mst22,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (3*mst22*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (3*mt2*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (3*mst12*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (3*mt2*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (3*mst22*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (3*mt2*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (3*mst12*
     DeltaInv(mstau22,mst12,mt2)*power2(mu2))/power2(mstau22 - mu2) - (3*mt2*
     DeltaInv(mstau22,mst12,mt2)*power2(mu2))/power2(mstau22 - mu2) - (3*mst22*
     DeltaInv(mstau22,mst22,mt2)*power2(mu2))/power2(mstau22 - mu2) - (3*mt2*
     DeltaInv(mstau22,mst22,mt2)*power2(mu2))/power2(mstau22 - mu2) - (24*
     mst12*DeltaInv(mst12,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) + (24*mt2*DeltaInv(mst12,mt2,mu2)*power3(mu2))/(
     (mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (24*mst12*DeltaInv(
     mst12,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) - (24*mt2*DeltaInv(mst12,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) + (24*mst22*DeltaInv(mst22,mt2,mu2)*
     power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (24*
     mt2*DeltaInv(mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) - (24*mst22*DeltaInv(mst22,mt2,mu2)*power3(mu2))
     /((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (24*mt2*DeltaInv(
     mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) + (3*DeltaInv(mst12,mstau12,mt2)*power3(mu2))/power2(mstau12 -
     mu2) + (3*DeltaInv(mst12,mt2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (3*
     DeltaInv(mst22,mstau12,mt2)*power3(mu2))/power2(mstau12 - mu2) + (3*
     DeltaInv(mst22,mt2,mu2)*power3(mu2))/power2(mstau12 - mu2) + (3*DeltaInv(
     mst12,mt2,mu2)*power3(mu2))/power2(mstau22 - mu2) + (3*DeltaInv(mst22,mt2,
     mu2)*power3(mu2))/power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mst12,mt2)*
     power3(mu2))/power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mst22,mt2)*
     power3(mu2))/power2(mstau22 - mu2) - (24*DeltaInv(mst12,mt2,mu2)*power4(
     mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (24*
     DeltaInv(mst12,mt2,mu2)*power4(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau22 - mu2)) + (24*DeltaInv(mst22,mt2,mu2)*power4(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (24*DeltaInv(mst22,mt2,mu2)*
     power4(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (3*
     DeltaInv(msntau2,msb12,mt2)*(-(msb14*(msntau2 + mt2)) + msntau2*(msntau2*(
     msntau2 - mt2) - msb12*(msntau2 + 6*mt2)) + power6(msb1)))/power2(msntau2
     - mu2) + (3*DeltaInv(msb12,mt2,mu2)*(-(msb14*(mt2 + mu2)) + mu2*(mu2*(-mt2
     + mu2) - msb12*(6*mt2 + mu2)) + power6(msb1)))/power2(msntau2 - mu2) - (
     72*mu2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) + (72*mu2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/
     ((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (3*DeltaInv(mst12,
     mstau12,mt2)*power6(mst1))/power2(mstau12 - mu2) + (3*DeltaInv(mst12,mt2,
     mu2)*power6(mst1))/power2(mstau12 - mu2) + (3*DeltaInv(mst12,mt2,mu2)*
     power6(mst1))/power2(mstau22 - mu2) + (3*DeltaInv(mstau22,mst12,mt2)*
     power6(mst1))/power2(mstau22 - mu2) + (72*mu2*DeltaInv(mst22,mt2,mu2)*
     power6(mst2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (72*
     mu2*DeltaInv(mst22,mt2,mu2)*power6(mst2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) + (3*DeltaInv(mst22,mstau12,mt2)*power6(mst2))/
     power2(mstau12 - mu2) + (3*DeltaInv(mst22,mt2,mu2)*power6(mst2))/power2(
     mstau12 - mu2) + (3*DeltaInv(mst22,mt2,mu2)*power6(mst2))/power2(mstau22 -
     mu2) + (3*DeltaInv(mstau22,mst22,mt2)*power6(mst2))/power2(mstau22 - mu2))
     )/4. - (3*mu2*(80/mA2 - 80/mC2 - 240/mH2 + (21*mst22)/((msntau2 - mst22)*(
     msntau2 - mu2)) + (1792*mst12*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(
     mstau12 - mu2)) - (1792*mst22*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(
     mstau12 - mu2)) - (1792*mst12*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(
     mstau22 - mu2)) + (1792*mst22*mu2)/((mst12 - mst22)*(mstau12 - mstau22)*(
     mstau22 - mu2)) + 80/(-msntau2 + mu2) + 192/(-mstau12 + mu2) + 192/(-
     mstau22 + mu2) - 112*msb12*DeltaInv(msntau2,msb12,mt2) + 112*msntau2*
     DeltaInv(msntau2,msb12,mt2) - 112*mt2*DeltaInv(msntau2,msb12,mt2) + 224*
     mu2*DeltaInv(msntau2,msb12,mt2) + (112*msb14*DeltaInv(msntau2,msb12,mt2))/
     (-msntau2 + mu2) + (448*msb12*mt2*DeltaInv(msntau2,msb12,mt2))/(-msntau2 +
     mu2) + (224*msb12*mu2*DeltaInv(msntau2,msb12,mt2))/(-msntau2 + mu2) + (
     224*mt2*mu2*DeltaInv(msntau2,msb12,mt2))/(-msntau2 + mu2) + 336*mH2*
     DeltaInv(mst12,mH2,mst22) - 336*mst12*DeltaInv(mst12,mH2,mst22) - 336*
     mst22*DeltaInv(mst12,mH2,mst22) - (672*mst12*mst22*DeltaInv(mst12,mH2,
     mst22))/mH2 - 112*mst12*DeltaInv(mst12,mstau12,mt2) + 112*mstau12*
     DeltaInv(mst12,mstau12,mt2) - 112*mt2*DeltaInv(mst12,mstau12,mt2) + 224*
     mu2*DeltaInv(mst12,mstau12,mt2) + (112*mst14*DeltaInv(mst12,mstau12,mt2))/
     (-mstau12 + mu2) + (448*mst12*mt2*DeltaInv(mst12,mstau12,mt2))/(-mstau12 +
     mu2) + (224*mst12*mu2*DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (
     224*mt2*mu2*DeltaInv(mst12,mstau12,mt2))/(-mstau12 + mu2) + (3584*mst14*
     mt2*mu2*DeltaInv(mst12,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(
     mstau12 - mu2)) - (3584*mst14*mt2*mu2*DeltaInv(mst12,mt2,mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - 112*mst22*DeltaInv(mst22,
     mstau12,mt2) + 112*mstau12*DeltaInv(mst22,mstau12,mt2) - 112*mt2*DeltaInv(
     mst22,mstau12,mt2) + 224*mu2*DeltaInv(mst22,mstau12,mt2) + (112*mst24*
     DeltaInv(mst22,mstau12,mt2))/(-mstau12 + mu2) + (448*mst22*mt2*DeltaInv(
     mst22,mstau12,mt2))/(-mstau12 + mu2) + (224*mst22*mu2*DeltaInv(mst22,
     mstau12,mt2))/(-mstau12 + mu2) + (224*mt2*mu2*DeltaInv(mst22,mstau12,mt2))
     /(-mstau12 + mu2) - (3584*mst24*mt2*mu2*DeltaInv(mst22,mt2,mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (3584*mst24*mt2*mu2*
     DeltaInv(mst22,mt2,mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 -
     mu2)) - 112*mst12*DeltaInv(mstau22,mst12,mt2) + 112*mstau22*DeltaInv(
     mstau22,mst12,mt2) - 112*mt2*DeltaInv(mstau22,mst12,mt2) + 224*mu2*
     DeltaInv(mstau22,mst12,mt2) + (112*mst14*DeltaInv(mstau22,mst12,mt2))/(-
     mstau22 + mu2) + (448*mst12*mt2*DeltaInv(mstau22,mst12,mt2))/(-mstau22 +
     mu2) + (224*mst12*mu2*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) + (
     224*mt2*mu2*DeltaInv(mstau22,mst12,mt2))/(-mstau22 + mu2) - 112*mst22*
     DeltaInv(mstau22,mst22,mt2) + 112*mstau22*DeltaInv(mstau22,mst22,mt2) -
     112*mt2*DeltaInv(mstau22,mst22,mt2) + 224*mu2*DeltaInv(mstau22,mst22,mt2)
     + (112*mst24*DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) + (448*mst22*
     mt2*DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) + (224*mst22*mu2*
     DeltaInv(mstau22,mst22,mt2))/(-mstau22 + mu2) + (224*mt2*mu2*DeltaInv(
     mstau22,mst22,mt2))/(-mstau22 + mu2) - (112*DeltaInv(mA2,mst12,mst22)*(-2*
     mst12*mst22 - mA2*(mst12 + mst22) + power2(mA2)))/mA2 + (112*DeltaInv(mC2,
     mst22,msb12)*(-2*msb12*mst22 - mC2*(msb12 + mst22) + power2(mC2)))/mC2 + (
     14*mst24)/((-msntau2 + mu2)*power2(msntau2 - mst22)) - (128*msb12)/power2(
     msntau2 - mu2) + (75*mst22)/power2(msntau2 - mu2) + (7*mst24)/((-msntau2 +
     mst22)*power2(msntau2 - mu2)) - (96*mt2)/power2(msntau2 - mu2) - (160*mu2)
     /power2(msntau2 - mu2) + (21*mst22*mu2)/((msntau2 - mst22)*power2(msntau2
     - mu2)) - (112*msb14*mt2*DeltaInv(msb12,mt2,mu2))/power2(msntau2 - mu2) -
     (112*msb14*mu2*DeltaInv(msb12,mt2,mu2))/power2(msntau2 - mu2) - (448*
     msb12*mt2*mu2*DeltaInv(msb12,mt2,mu2))/power2(msntau2 - mu2) - (112*msb14*
     mt2*DeltaInv(msntau2,msb12,mt2))/power2(msntau2 - mu2) - (112*msb14*mu2*
     DeltaInv(msntau2,msb12,mt2))/power2(msntau2 - mu2) - (448*msb12*mt2*mu2*
     DeltaInv(msntau2,msb12,mt2))/power2(msntau2 - mu2) - (14*mst24*mu2)/(
     power2(msntau2 - mst22)*power2(msntau2 - mu2)) - (128*mst12)/power2(
     mstau12 - mu2) - (128*mst22)/power2(mstau12 - mu2) - (192*mt2)/power2(
     mstau12 - mu2) - (384*mu2)/power2(mstau12 - mu2) - (112*mst14*mt2*
     DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (112*mst14*mu2*
     DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (448*mst12*mt2*mu2*
     DeltaInv(mst12,mstau12,mt2))/power2(mstau12 - mu2) - (112*mst14*mt2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau12 - mu2) - (112*mst14*mu2*DeltaInv(
     mst12,mt2,mu2))/power2(mstau12 - mu2) - (448*mst12*mt2*mu2*DeltaInv(mst12,
     mt2,mu2))/power2(mstau12 - mu2) - (112*mst24*mt2*DeltaInv(mst22,mstau12,
     mt2))/power2(mstau12 - mu2) - (112*mst24*mu2*DeltaInv(mst22,mstau12,mt2))/
     power2(mstau12 - mu2) - (448*mst22*mt2*mu2*DeltaInv(mst22,mstau12,mt2))/
     power2(mstau12 - mu2) - (112*mst24*mt2*DeltaInv(mst22,mt2,mu2))/power2(
     mstau12 - mu2) - (112*mst24*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau12 -
     mu2) - (448*mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/power2(mstau12 - mu2) -
     (128*mst12)/power2(mstau22 - mu2) - (128*mst22)/power2(mstau22 - mu2) - (
     192*mt2)/power2(mstau22 - mu2) - (384*mu2)/power2(mstau22 - mu2) - (112*
     mst14*mt2*DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (112*mst14*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (448*mst12*mt2*mu2*
     DeltaInv(mst12,mt2,mu2))/power2(mstau22 - mu2) - (112*mst24*mt2*DeltaInv(
     mst22,mt2,mu2))/power2(mstau22 - mu2) - (112*mst24*mu2*DeltaInv(mst22,mt2,
     mu2))/power2(mstau22 - mu2) - (448*mst22*mt2*mu2*DeltaInv(mst22,mt2,mu2))/
     power2(mstau22 - mu2) - (112*mst14*mt2*DeltaInv(mstau22,mst12,mt2))/
     power2(mstau22 - mu2) - (112*mst14*mu2*DeltaInv(mstau22,mst12,mt2))/
     power2(mstau22 - mu2) - (448*mst12*mt2*mu2*DeltaInv(mstau22,mst12,mt2))/
     power2(mstau22 - mu2) - (112*mst24*mt2*DeltaInv(mstau22,mst22,mt2))/
     power2(mstau22 - mu2) - (112*mst24*mu2*DeltaInv(mstau22,mst22,mt2))/
     power2(mstau22 - mu2) - (448*mst22*mt2*mu2*DeltaInv(mstau22,mst22,mt2))/
     power2(mstau22 - mu2) + (336*DeltaInv(msntau2,msb12,mt2)*power2(mu2))/(
     msntau2 - mu2) + (336*DeltaInv(mst12,mstau12,mt2)*power2(mu2))/(mstau12 -
     mu2) + (2688*mst14*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau12 - mu2)) + (6272*mst12*mt2*DeltaInv(mst12,mt2,
     mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) -
     (2688*mst14*DeltaInv(mst12,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12
     - mstau22)*(mstau22 - mu2)) - (6272*mst12*mt2*DeltaInv(mst12,mt2,mu2)*
     power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (336*
     DeltaInv(mst22,mstau12,mt2)*power2(mu2))/(mstau12 - mu2) - (2688*mst24*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau12 - mu2)) - (6272*mst22*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (2688*mst24*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau22 - mu2)) + (6272*mst22*mt2*DeltaInv(mst22,mt2,mu2)*power2(mu2))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (336*DeltaInv(
     mstau22,mst12,mt2)*power2(mu2))/(mstau22 - mu2) + (336*DeltaInv(mstau22,
     mst22,mt2)*power2(mu2))/(mstau22 - mu2) - (112*msb12*DeltaInv(msb12,mt2,
     mu2)*power2(mu2))/power2(msntau2 - mu2) - (112*mt2*DeltaInv(msb12,mt2,mu2)
     *power2(mu2))/power2(msntau2 - mu2) - (112*msb12*DeltaInv(msntau2,msb12,
     mt2)*power2(mu2))/power2(msntau2 - mu2) - (112*mt2*DeltaInv(msntau2,msb12,
     mt2)*power2(mu2))/power2(msntau2 - mu2) - (112*mst12*DeltaInv(mst12,
     mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (112*mt2*DeltaInv(mst12,
     mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (112*mst12*DeltaInv(
     mst12,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (112*mt2*DeltaInv(
     mst12,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (112*mst22*DeltaInv(
     mst22,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (112*mt2*DeltaInv(
     mst22,mstau12,mt2)*power2(mu2))/power2(mstau12 - mu2) - (112*mst22*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (112*mt2*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau12 - mu2) - (112*mst12*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (112*mt2*
     DeltaInv(mst12,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (112*mst22*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (112*mt2*
     DeltaInv(mst22,mt2,mu2)*power2(mu2))/power2(mstau22 - mu2) - (112*mst12*
     DeltaInv(mstau22,mst12,mt2)*power2(mu2))/power2(mstau22 - mu2) - (112*mt2*
     DeltaInv(mstau22,mst12,mt2)*power2(mu2))/power2(mstau22 - mu2) - (112*
     mst22*DeltaInv(mstau22,mst22,mt2)*power2(mu2))/power2(mstau22 - mu2) - (
     112*mt2*DeltaInv(mstau22,mst22,mt2)*power2(mu2))/power2(mstau22 - mu2) + (
     896*mt2*DeltaInv(mst12,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) - (896*mt2*DeltaInv(mst12,mt2,mu2)*power3(mu2))/
     ((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) - (896*mt2*DeltaInv(
     mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12
     - mu2)) + (896*mt2*DeltaInv(mst22,mt2,mu2)*power3(mu2))/((mst12 - mst22)*(
     mstau12 - mstau22)*(mstau22 - mu2)) + (112*DeltaInv(msb12,mt2,mu2)*power3(
     mu2))/power2(msntau2 - mu2) + (112*DeltaInv(msntau2,msb12,mt2)*power3(mu2)
     )/power2(msntau2 - mu2) + (112*DeltaInv(mst12,mstau12,mt2)*power3(mu2))/
     power2(mstau12 - mu2) + (112*DeltaInv(mst12,mt2,mu2)*power3(mu2))/power2(
     mstau12 - mu2) + (112*DeltaInv(mst22,mstau12,mt2)*power3(mu2))/power2(
     mstau12 - mu2) + (112*DeltaInv(mst22,mt2,mu2)*power3(mu2))/power2(mstau12
     - mu2) + (112*DeltaInv(mst12,mt2,mu2)*power3(mu2))/power2(mstau22 - mu2) +
     (112*DeltaInv(mst22,mt2,mu2)*power3(mu2))/power2(mstau22 - mu2) + (112*
     DeltaInv(mstau22,mst12,mt2)*power3(mu2))/power2(mstau22 - mu2) + (112*
     DeltaInv(mstau22,mst22,mt2)*power3(mu2))/power2(mstau22 - mu2) - (896*
     DeltaInv(mst12,mt2,mu2)*power4(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*
     (mstau12 - mu2)) + (896*DeltaInv(mst12,mt2,mu2)*power4(mu2))/((mst12 -
     mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (896*DeltaInv(mst22,mt2,mu2)
     *power4(mu2))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) - (
     896*DeltaInv(mst22,mt2,mu2)*power4(mu2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau22 - mu2)) + (112*DeltaInv(msb12,mt2,mu2)*power6(msb1))/
     power2(msntau2 - mu2) + (112*DeltaInv(msntau2,msb12,mt2)*power6(msb1))/
     power2(msntau2 - mu2) - (1792*mu2*DeltaInv(mst12,mt2,mu2)*power6(mst1))/((
     mst12 - mst22)*(mstau12 - mstau22)*(mstau12 - mu2)) + (1792*mu2*DeltaInv(
     mst12,mt2,mu2)*power6(mst1))/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22
     - mu2)) + (112*DeltaInv(mst12,mstau12,mt2)*power6(mst1))/power2(mstau12 -
     mu2) + (112*DeltaInv(mst12,mt2,mu2)*power6(mst1))/power2(mstau12 - mu2) +
     (112*DeltaInv(mst12,mt2,mu2)*power6(mst1))/power2(mstau22 - mu2) + (112*
     DeltaInv(mstau22,mst12,mt2)*power6(mst1))/power2(mstau22 - mu2) + (1792*
     mu2*DeltaInv(mst22,mt2,mu2)*power6(mst2))/((mst12 - mst22)*(mstau12 -
     mstau22)*(mstau12 - mu2)) - (1792*mu2*DeltaInv(mst22,mt2,mu2)*power6(mst2)
     )/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2)) + (14*power6(mst2)
     )/(power2(msntau2 - mst22)*power2(msntau2 - mu2)) + (112*DeltaInv(mst22,
     mstau12,mt2)*power6(mst2))/power2(mstau12 - mu2) + (112*DeltaInv(mst22,
     mt2,mu2)*power6(mst2))/power2(mstau12 - mu2) + (112*DeltaInv(mst22,mt2,
     mu2)*power6(mst2))/power2(mstau22 - mu2) + (112*DeltaInv(mstau22,mst22,
     mt2)*power6(mst2))/power2(mstau22 - mu2)))/64. - (3*mu2*log(msntau2/Q2)*
     log(mst22/Q2)*(mst22*mst24*mu2 - 4*power2(msntau2)*power2(mst22) + 2*
     mst22*power3(msntau2) - mu2*power6(mst2) + msntau2*(-(mst22*mst24) + 2*
     power3(mst22) + power6(mst2))))/(8.*power2(msntau2 - mst22)*power3(msntau2
     - mu2)) + (3*mu2*log(msntau2/Q2)*log(mst22/Q2)*power2(snt)*(mst22*mst24*
     mu2 - 4*power2(msntau2)*power2(mst22) + 2*mst22*power3(msntau2) - mu2*
     power6(mst2) + msntau2*(-(mst22*mst24) + 2*power3(mst22) + power6(mst2))))
     /(8.*power2(msntau2 - mst22)*power3(msntau2 - mu2)) - (3*log(mst22/Q2)*
     log(mu2/Q2)*power2(mu2)*(mst22*(8/((mst12 - mst22)*(mstau12 - mstau22)*(
     mstau12 - mu2)) - 8/((mst12 - mst22)*(mstau12 - mstau22)*(mstau22 - mu2))
     + 1/power3(-msntau2 + mu2) + 1/power3(-mstau12 + mu2) + 1/power3(-mstau22
     + mu2)) + (2*DeltaInv(mst22,mt2,mu2)*(-4*mst24*mstau12*mstau22*mt2 - 8*
     mst24*mstau12*mstau22*mu2 + 4*mst24*mstau12*mt2*mu2 + 4*mst24*mstau22*mt2*
     mu2 - 12*mst22*mstau12*mstau22*mt2*mu2 + 2*mstau12*mt2*mu2*power2(mst22) +
     2*mstau22*mt2*mu2*power2(mst22) - mt2*power2(mst22)*power2(mstau12) - mt2*
     power2(mst22)*power2(mstau22) + 8*mst24*mstau12*power2(mu2) + 8*mst24*
     mstau22*power2(mu2) + 4*mst22*mstau12*mstau22*power2(mu2) - 4*mst24*mt2*
     power2(mu2) + 12*mst22*mstau12*mt2*power2(mu2) + 12*mst22*mstau22*mt2*
     power2(mu2) - 2*mt2*power2(mst22)*power2(mu2) + mst12*mst22*mt2*(-2*
     mstau12*mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*
     power2(mu2)) - 8*mst24*power3(mu2) - 4*mst22*mstau12*power3(mu2) - 4*
     mst22*mstau22*power3(mu2) - 12*mst22*mt2*power3(mu2) + 4*mst22*power4(mu2)
     + 4*(mstau12 - mu2)*(mstau22 - mu2)*power6(mst2)))/((mst12 - mst22)*
     power2(mstau12 - mu2)*power2(mstau22 - mu2))))/4. + (3*mu2*power2(snt)*
     power2(log(msntau2/Q2))*(power2(msntau2)*(-(mst14/power2(msntau2 - mst12))
     + mst24/power2(msntau2 - mst22)) + msntau2*(2*mst12 + mst14/(msntau2 -
     mst12) - 2*mst22 + mst24/(-msntau2 + mst22) + (mst14*mu2)/power2(msntau2 -
     mst12) + power6(mst1)/power2(msntau2 - mst12) - power6(mst2)/power2(
     msntau2 - mst22)) + mu2*(-(mst14/(msntau2 - mst12)) - power6(mst1)/power2(
     msntau2 - mst12) + (-(mst22*mst24) + power6(mst2))/power2(msntau2 - mst22)
     )))/(16.*power3(-msntau2 + mu2)) - (3*mu2*log(msntau2/Q2)*power2(snt)*(
     power2(msntau2)*((-3*mst14)/power2(msntau2 - mst12) + (3*mst24)/power2(
     msntau2 - mst22)) + msntau2*(4*mst12 + (3*mst14)/(msntau2 - mst12) - 4*
     mst22 + (3*mst24)/(-msntau2 + mst22) + (3*mst14*mu2)/power2(msntau2 -
     mst12) + (3*power6(mst1))/power2(msntau2 - mst12) - (3*power6(mst2))/
     power2(msntau2 - mst22)) + 3*mu2*(-(mst14/(msntau2 - mst12)) - power6(
     mst1)/power2(msntau2 - mst12) + (-(mst22*mst24) + power6(mst2))/power2(
     msntau2 - mst22))))/(8.*power3(-msntau2 + mu2)) + (3*mu2*log(mst22/Q2)*
     log(mt2/Q2)*(4*mst12*mst22*mstau12*mu2 + 4*mst12*mst22*mstau22*mu2 + 16*
     mst22*mstau12*mstau22*mu2 + 4*mst12*mst24*mstau12*mstau22*mu2*DeltaInv(
     mstau22,mst22,mt2) - 4*mst22*mst24*mstau12*mstau22*mu2*DeltaInv(mstau22,
     mst22,mt2) + 2*mst12*mst24*mstau12*mt2*mu2*DeltaInv(mstau22,mst22,mt2) -
     2*mst22*mst24*mstau12*mt2*mu2*DeltaInv(mstau22,mst22,mt2) + 6*mst12*mst22*
     mstau12*mstau22*mt2*mu2*DeltaInv(mstau22,mst22,mt2) - 4*mstau12*mu2*
     power2(mst22) - 4*mstau22*mu2*power2(mst22) - 6*mstau12*mstau22*mt2*mu2*
     DeltaInv(mstau22,mst22,mt2)*power2(mst22) - 2*mst12*mst22*power2(mstau12)
     - 2*mst12*mst24*mstau22*DeltaInv(mstau22,mst22,mt2)*power2(mstau12) + 2*
     mst22*mst24*mstau22*DeltaInv(mstau22,mst22,mt2)*power2(mstau12) - mst12*
     mst24*mt2*DeltaInv(mstau22,mst22,mt2)*power2(mstau12) + mst22*mst24*mt2*
     DeltaInv(mstau22,mst22,mt2)*power2(mstau12) - 3*mst12*mst22*mstau22*mt2*
     DeltaInv(mstau22,mst22,mt2)*power2(mstau12) + 2*power2(mst22)*power2(
     mstau12) + 3*mstau22*mt2*DeltaInv(mstau22,mst22,mt2)*power2(mst22)*power2(
     mstau12) - 2*mst12*mst22*power2(mstau22) - 2*mst12*mst22*mstau12*mu2*
     DeltaInv(mstau22,mst22,mt2)*power2(mstau22) + 2*power2(mst22)*power2(
     mstau22) + 2*mstau12*mu2*DeltaInv(mstau22,mst22,mt2)*power2(mst22)*power2(
     mstau22) + mst12*mst22*DeltaInv(mstau22,mst22,mt2)*power2(mstau12)*power2(
     mstau22) - DeltaInv(mstau22,mst22,mt2)*power2(mst22)*power2(mstau12)*
     power2(mstau22) - 4*mst12*mst22*power2(mu2) - 16*mst22*mstau12*power2(mu2)
     - 16*mst22*mstau22*power2(mu2) - 2*mst12*mst24*mstau22*DeltaInv(mstau22,
     mst22,mt2)*power2(mu2) + 2*mst22*mst24*mstau22*DeltaInv(mstau22,mst22,mt2)
     *power2(mu2) - mst12*mst24*mt2*DeltaInv(mstau22,mst22,mt2)*power2(mu2) +
     mst22*mst24*mt2*DeltaInv(mstau22,mst22,mt2)*power2(mu2) - 3*mst12*mst22*
     mstau22*mt2*DeltaInv(mstau22,mst22,mt2)*power2(mu2) + 4*power2(mst22)*
     power2(mu2) + 3*mstau22*mt2*DeltaInv(mstau22,mst22,mt2)*power2(mst22)*
     power2(mu2) + mst12*mst22*DeltaInv(mstau22,mst22,mt2)*power2(mstau22)*
     power2(mu2) - DeltaInv(mstau22,mst22,mt2)*power2(mst22)*power2(mstau22)*
     power2(mu2) + 16*mst22*power3(mu2) - 2*mst12*mstau12*mu2*DeltaInv(mstau22,
     mst22,mt2)*power6(mst2) + 2*mst22*mstau12*mu2*DeltaInv(mstau22,mst22,mt2)*
     power6(mst2) + mst12*DeltaInv(mstau22,mst22,mt2)*power2(mstau12)*power6(
     mst2) - mst22*DeltaInv(mstau22,mst22,mt2)*power2(mstau12)*power6(mst2) +
     mst12*DeltaInv(mstau22,mst22,mt2)*power2(mu2)*power6(mst2) - mst22*
     DeltaInv(mstau22,mst22,mt2)*power2(mu2)*power6(mst2) + (mst12 - mst22)*
     DeltaInv(mst22,mstau12,mt2)*power2(mstau22 - mu2)*(mst22*mstau12*(mstau12
     - 3*mt2) - mst24*(2*mstau12 + mt2) + power6(mst2)) + DeltaInv(mst22,mt2,
     mu2)*(-2*mst22*mst24*mstau12*mt2*mu2 - 2*mst22*mst24*mstau22*mt2*mu2 + 32*
     mst24*mstau12*mstau22*mt2*mu2 + mst22*mst24*mt2*power2(mstau12) + 2*mst22*
     mst24*mu2*power2(mstau12) + 3*mt2*mu2*power2(mst22)*power2(mstau12) +
     mst22*mst24*mt2*power2(mstau22) + 2*mst22*mst24*mu2*power2(mstau22) + 3*
     mt2*mu2*power2(mst22)*power2(mstau22) - 4*mst22*mst24*mstau12*power2(mu2)
     - 4*mst22*mst24*mstau22*power2(mu2) + 32*mst24*mstau12*mstau22*power2(mu2)
     + 2*mst22*mst24*mt2*power2(mu2) - 32*mst24*mstau12*mt2*power2(mu2) - 32*
     mst24*mstau22*mt2*power2(mu2) + 32*mst22*mstau12*mstau22*mt2*power2(mu2) -
     6*mstau12*mt2*power2(mst22)*power2(mu2) - 6*mstau22*mt2*power2(mst22)*
     power2(mu2) - power2(mst22)*power2(mstau12)*power2(mu2) - power2(mst22)*
     power2(mstau22)*power2(mu2) + 4*mst22*mst24*power3(mu2) - 32*mst24*
     mstau12*power3(mu2) - 32*mst24*mstau22*power3(mu2) - 16*mst22*mstau12*
     mstau22*power3(mu2) + 32*mst24*mt2*power3(mu2) - 32*mst22*mstau12*mt2*
     power3(mu2) - 32*mst22*mstau22*mt2*power3(mu2) + 2*mstau12*power2(mst22)*
     power3(mu2) + 2*mstau22*power2(mst22)*power3(mu2) + 6*mt2*power2(mst22)*
     power3(mu2) + 32*mst24*power4(mu2) + 16*mst22*mstau12*power4(mu2) + 16*
     mst22*mstau22*power4(mu2) + 32*mst22*mt2*power4(mu2) - 2*power2(mst22)*
     power4(mu2) - 16*mst22*power5(mu2) - (16*(mstau12 - mu2)*(mstau22 - mu2)*
     mu2 + mst22*(-2*mstau12*mu2 - 2*mstau22*mu2 + power2(mstau12) + power2(
     mstau22) + 2*power2(mu2)))*power6(mst2) + mst12*(-2*mstau12*mu2 - 2*
     mstau22*mu2 + power2(mstau12) + power2(mstau22) + 2*power2(mu2))*(mst22*
     mu2*(-3*mt2 + mu2) - mst24*(mt2 + 2*mu2) + power6(mst2)))))/(4.*(-mst12 +
     mst22)*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + power2(xt)*((9*(1 +
     (mh2 - mH2)*mst12*DeltaInv(mst12,mst12,mH2))*Fin3(mst12,mst12,mH2,Q2)*(-1
     + power2(sa))*power2(sa)*(2*mt2 - (mst12 - mst22)*(-1 + power2(snt))*
     power2(snt)))/((mh2 - mH2)*mH2*(mst12 - mst22)) + (3*(-3*mh2 + 2*mst12 +
     mh2*(mh2*mst12 - 3*mH2*mst12 + 8*mst14)*DeltaInv(mst12,mst12,mh2))*Fin3(
     mst12,mst12,mh2,Q2)*(-1 + power2(sa))*power2(sa)*(2*mt2 - (mst12 - mst22)*
     (-1 + power2(snt))*power2(snt)))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2))
     - (9*(1 + (mh2 - mH2)*mst22*DeltaInv(mH2,mst22,mst22))*Fin3(mH2,mst22,
     mst22,Q2)*(-1 + power2(sa))*power2(sa)*(2*mt2 + (mst12 - mst22)*(-1 +
     power2(snt))*power2(snt)))/((mh2 - mH2)*mH2*(mst12 - mst22)) - (3*(-3*mh2
     + 2*mst22 + mh2*(mh2*mst22 - 3*mH2*mst22 + 8*mst24)*DeltaInv(mst22,mst22,
     mh2))*Fin3(mst22,mst22,mh2,Q2)*(-1 + power2(sa))*power2(sa)*(2*mt2 + (
     mst12 - mst22)*(-1 + power2(snt))*power2(snt)))/((mh2 - mH2)*(mst12 -
     mst22)*power2(mh2)) + (9*(-3*mh2 + mH2 + mh2*(mh2 - mH2)*(mh2 - mst12 -
     mst22)*DeltaInv(mst12,mst22,mh2))*Fin3(mst12,mst22,mh2,Q2)*(-1 + power2(
     sa))*power2(sa - 2*sa*power2(snt)))/(4.*(mh2 - mH2)*power2(mh2)) + (9*(-
     mh2 + 3*mH2 + (mh2 - mH2)*mH2*(mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22))*Fin3(mst12,mH2,mst22,Q2)*(-1 + power2(sa))*power2(sa - 2*sa*
     power2(snt)))/(4.*(mh2 - mH2)*power2(mH2)) + (power2(sa)*((18*(mh2 + mH2))
     /(mh2*mH2) + (36*log(mh2/Q2))/(-mh2 + mH2) + (36*log(mH2/Q2))/(mh2 - mH2)
     - (18*(mh2 + mH2)*mst12*log(mst12/Q2))/(mh2*mH2*(mst12 - mst22)) + (36*
     mst12*log(mh2/Q2)*log(mst12/Q2))/((mh2 - mH2)*(mst12 - mst22)) - (36*
     mst12*log(mH2/Q2)*log(mst12/Q2))/((mh2 - mH2)*(mst12 - mst22)) + (18*(mh2
     + mH2)*mst22*log(mst22/Q2))/(mh2*mH2*(mst12 - mst22)) - (36*mst22*log(mh2/
     Q2)*log(mst22/Q2))/((mh2 - mH2)*(mst12 - mst22)) + (36*mst22*log(mH2/Q2)*
     log(mst22/Q2))/((mh2 - mH2)*(mst12 - mst22)) + (36*mst12*mst22*(mh2*
     DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*
     log(mst22/Q2))/(mh2*mH2) + (18*(-(mh2*(mst12 - mst22)*(mh2*mst22 + mst12*
     mst22 - mst24)*DeltaInv(mst12,mst22,mh2)) + 8*mt2*(mst22 + 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2)))*log(mh2/Q2)*log(mst22/Q2))/((mst12 - mst22)*
     power2(mh2)) - (18*log(mh2/Q2)*log(mst12/Q2)*(-(mh2*mst12) + mh2*mst22 +
     8*mst12*mt2 + 32*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2) + mh2*(mst12 -
     mst22)*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) +
     mst24 + power2(mh2))))/((mst12 - mst22)*power2(mh2)) - 18*log(mh2/Q2)*((-
     96*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*mst12 - mh2*mst22) + 3*(-mh2
     + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) + (96*mh2*(mh2 - mH2)*mst24*
     mt2*DeltaInv(mst22,mst22,mh2) + 2*(mst12 - mst22)*(12*mH2*mt2 - mh2*(mH2 +
     12*mt2) + 2*power2(mh2)))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2))) + (
     18*(8*mst22*mt2 + 32*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - mH2*(mst12
     - mst22)*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(
     mH2/Q2)*log(mst22/Q2))/((mst12 - mst22)*power2(mH2)) - (18*Fin20(mst12,
     mst22,Q2)*power2(mh2 - mH2))/(power2(mh2)*power2(mH2)) + (18*log(mH2/Q2)*(
     -2*mh2*mH2*mst12 + 2*mh2*mH2*mst22 + 24*mh2*mst12*mt2 - 24*mH2*mst12*mt2 -
     24*mh2*mst22*mt2 + 24*mH2*mst22*mt2 - 96*(mh2 - mH2)*mH2*mst24*mt2*
     DeltaInv(mH2,mst22,mst22) + 96*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)
     + 4*mst12*power2(mH2) - 4*mst22*power2(mH2) + 3*(mh2 - mH2)*(mH2 - mst12 -
     mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) - 96*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2)))/((mh2 - mH2)*(mst12 - mst22)*
     power2(mH2)) - (18*log(mH2/Q2)*log(mst12/Q2)*(-(mH2*mst12) + mH2*mst22 +
     8*mst12*mt2 + 32*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + mH2*(mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2))))/((mst12 - mst22)*power2(mH2)) - (18*(mh2 + mH2)*
     power2(sa))/(mh2*mH2) + (36*log(mh2/Q2)*power2(sa))/(mh2 - mH2) - (36*log(
     mH2/Q2)*power2(sa))/(mh2 - mH2) + (18*(mh2 + mH2)*mst12*log(mst12/Q2)*
     power2(sa))/(mh2*mH2*(mst12 - mst22)) - (36*mst12*log(mh2/Q2)*log(mst12/
     Q2)*power2(sa))/((mh2 - mH2)*(mst12 - mst22)) + (36*mst12*log(mH2/Q2)*log(
     mst12/Q2)*power2(sa))/((mh2 - mH2)*(mst12 - mst22)) - (18*(mh2 + mH2)*
     mst22*log(mst22/Q2)*power2(sa))/(mh2*mH2*(mst12 - mst22)) + (36*mst22*log(
     mh2/Q2)*log(mst22/Q2)*power2(sa))/((mh2 - mH2)*(mst12 - mst22)) - (36*
     mst22*log(mH2/Q2)*log(mst22/Q2)*power2(sa))/((mh2 - mH2)*(mst12 - mst22))
     - (36*mst12*mst22*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,
     mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power2(sa))/(mh2*mH2) - (18*(-(
     mh2*(mst12 - mst22)*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,
     mst22,mh2)) + 8*mt2*(mst22 + 4*mh2*mst24*DeltaInv(mst22,mst22,mh2)))*log(
     mh2/Q2)*log(mst22/Q2)*power2(sa))/((mst12 - mst22)*power2(mh2)) + (18*log(
     mh2/Q2)*log(mst12/Q2)*(-(mh2*mst12) + mh2*mst22 + 8*mst12*mt2 + 32*mh2*
     mst14*mt2*DeltaInv(mst12,mst12,mh2) + mh2*(mst12 - mst22)*DeltaInv(mst12,
     mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))
     *power2(sa))/((mst12 - mst22)*power2(mh2)) + 18*log(mh2/Q2)*((-96*mst14*
     mt2*DeltaInv(mst12,mst12,mh2))/(mh2*mst12 - mh2*mst22) + 3*(-mh2 + mst12 +
     mst22)*DeltaInv(mst12,mst22,mh2) + (96*mh2*(mh2 - mH2)*mst24*mt2*DeltaInv(
     mst22,mst22,mh2) + 2*(mst12 - mst22)*(12*mH2*mt2 - mh2*(mH2 + 12*mt2) + 2*
     power2(mh2)))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)))*power2(sa) - (18*
     (8*mst22*mt2 + 32*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - mH2*(mst12 -
     mst22)*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(
     mH2/Q2)*log(mst22/Q2)*power2(sa))/((mst12 - mst22)*power2(mH2)) + (18*
     Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*power2(sa))/(power2(mh2)*power2(
     mH2)) - (18*log(mH2/Q2)*(-2*mh2*mH2*mst12 + 2*mh2*mH2*mst22 + 24*mh2*
     mst12*mt2 - 24*mH2*mst12*mt2 - 24*mh2*mst22*mt2 + 24*mH2*mst22*mt2 - 96*(
     mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) + 96*mh2*mH2*mst14*mt2*
     DeltaInv(mst12,mst12,mH2) + 4*mst12*power2(mH2) - 4*mst22*power2(mH2) + 3*
     (mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2) - 96*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2))*
     power2(sa))/((mh2 - mH2)*(mst12 - mst22)*power2(mH2)) + (18*log(mH2/Q2)*
     log(mst12/Q2)*(-(mH2*mst12) + mH2*mst22 + 8*mst12*mt2 + 32*mH2*mst14*mt2*
     DeltaInv(mst12,mst12,mH2) + mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*
     (-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(sa)
     )/((mst12 - mst22)*power2(mH2)) - (144*mst12*mst22*(mh2*DeltaInv(mst12,
     mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*
     power2(snt))/(mh2*mH2) + (72*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 -
     mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*
     log(mh2/Q2)*log(mst22/Q2)*power2(snt))/power2(mh2) - (216*log(mh2/Q2)*(-
     mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,
     mh2)*power2(mh2))*power2(snt))/power2(mh2) + (72*log(mh2/Q2)*log(mst12/Q2)
     *(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(
     mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(
     mh2)))*power2(snt))/power2(mh2) + (72*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,
     mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(snt))/power2(mH2) + (72*Fin20(
     mst12,mst22,Q2)*power2(mh2 - mH2)*power2(snt))/(power2(mh2)*power2(mH2)) -
     (216*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(snt))/power2(mH2) + (72*log(
     mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,
     mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*
     mst22) + mst24 + power2(mH2)))*power2(snt))/power2(mH2) + (144*mst12*
     mst22*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(
     mst12/Q2)*log(mst22/Q2)*power2(sa)*power2(snt))/(mh2*mH2) - (72*(-mst22 +
     mh2*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*
     mst24*DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power2(sa)*
     power2(snt))/power2(mh2) + (216*log(mh2/Q2)*(-mh2 - mst12 - mst22 - 4*mh2*
     mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) +
     (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(sa)*
     power2(snt))/power2(mh2) - (72*log(mh2/Q2)*log(mst12/Q2)*(-mh2 - mst12 -
     4*mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(mst12,mst22,mh2)*(-(
     mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))*power2(sa)*
     power2(snt))/power2(mh2) - (72*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*
     log(mH2/Q2)*log(mst22/Q2)*power2(sa)*power2(snt))/power2(mH2) - (72*Fin20(
     mst12,mst22,Q2)*power2(mh2 - mH2)*power2(sa)*power2(snt))/(power2(mh2)*
     power2(mH2)) + (216*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*
     DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 -
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(
     snt))/power2(mH2) - (72*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*
     mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*
     mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(sa)*power2(
     snt))/power2(mH2) + 9*(1/mh2 + 2/(mh2 - mH2) - (32*mst14*mt2*DeltaInv(
     mst12,mst12,mh2))/(mh2*mst12 - mh2*mst22) + (-mh2 + mst12 + mst22)*
     DeltaInv(mst12,mst22,mh2) + (32*mst24*mt2*DeltaInv(mst22,mst22,mh2))/(mh2*
     mst12 - mh2*mst22) + (8*mst22*mt2)/((mst12 - mst22)*power2(mh2)) + (8*
     mst12*mt2)/((-mst12 + mst22)*power2(mh2)))*power2(log(mh2/Q2)) - 9*(1/mh2
     + 2/(mh2 - mH2) - (32*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*mst12 -
     mh2*mst22) + (-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) + (32*mst24*
     mt2*DeltaInv(mst22,mst22,mh2))/(mh2*mst12 - mh2*mst22) + (8*mst22*mt2)/((
     mst12 - mst22)*power2(mh2)) + (8*mst12*mt2)/((-mst12 + mst22)*power2(mh2))
     )*power2(sa)*power2(log(mh2/Q2)) + (36*(-mh2 - mst12 - mst22 - 4*mh2*
     mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) +
     (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(snt)*
     power2(log(mh2/Q2)))/power2(mh2) - (36*(-mh2 - mst12 - mst22 - 4*mh2*
     mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) +
     (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(sa)*
     power2(snt)*power2(log(mh2/Q2)))/power2(mh2) + (9*(-(mh2*mH2*mst12) + mh2*
     mH2*mst22 + 8*mh2*mst12*mt2 - 8*mH2*mst12*mt2 - 8*mh2*mst22*mt2 + 8*mH2*
     mst22*mt2 - 32*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) + 32*
     mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + 3*mst12*power2(mH2) - 3*
     mst22*power2(mH2) + (mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mH2) - 32*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power2(mH2))*power2(log(mH2/Q2)))/((-mh2 + mH2)*(mst12 - mst22)*
     power2(mH2)) + (9*(mh2*mH2*mst12 - mh2*mH2*mst22 - 8*mh2*mst12*mt2 + 8*
     mH2*mst12*mt2 + 8*mh2*mst22*mt2 - 8*mH2*mst22*mt2 + 32*(mh2 - mH2)*mH2*
     mst24*mt2*DeltaInv(mH2,mst22,mst22) - 32*mh2*mH2*mst14*mt2*DeltaInv(mst12,
     mst12,mH2) - 3*mst12*power2(mH2) + 3*mst22*power2(mH2) - (mh2 - mH2)*(mH2
     - mst12 - mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) +
     32*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2))*power2(sa)*power2(log(
     mH2/Q2)))/((-mh2 + mH2)*(mst12 - mst22)*power2(mH2)) + (36*(-mH2 - mst12 -
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power2(snt)*power2(log(mH2/Q2)))/power2(mH2) - (36*(-mH2 - mst12 -
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power2(sa)*power2(snt)*power2(log(mH2/Q2)))/power2(mH2) + (144*
     mst12*mst22*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2)
     )*log(mst12/Q2)*log(mst22/Q2)*power4(snt))/(mh2*mH2) - (72*(-mst22 + mh2*(
     mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power4(snt))/power2(
     mh2) + (216*log(mh2/Q2)*(-mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(
     mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 -
     mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power4(snt))/power2(mh2) - (
     72*log(mh2/Q2)*log(mst12/Q2)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,
     mst12,mh2) + mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 +
     2*mst22) + mst24 + power2(mh2)))*power4(snt))/power2(mh2) - (72*(-mst22 -
     4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 -
     mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power4(snt))/
     power2(mH2) - (72*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*power4(snt))/(
     power2(mh2)*power2(mH2)) + (216*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*
     mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) +
     (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power4(snt))/
     power2(mH2) - (72*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22)
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power4(snt))/power2(mH2) -
     (144*mst12*mst22*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,
     mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power2(sa)*power4(snt))/(mh2*mH2)
     + (72*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,
     mst22,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/
     Q2)*power2(sa)*power4(snt))/power2(mh2) - (216*log(mh2/Q2)*(-mh2 - mst12 -
     mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(
     mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(
     mh2))*power2(sa)*power4(snt))/power2(mh2) + (72*log(mh2/Q2)*log(mst12/Q2)*
     (-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(
     mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(
     mh2)))*power2(sa)*power4(snt))/power2(mh2) + (72*(-mst22 - 4*mH2*mst24*
     DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(sa)*power4(
     snt))/power2(mH2) + (72*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*power2(sa)
     *power4(snt))/(power2(mh2)*power2(mH2)) - (216*log(mH2/Q2)*(-mH2 - mst12 -
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power2(sa)*power4(snt))/power2(mH2) + (72*log(mH2/Q2)*log(mst12/Q2)*
     (-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(
     mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(
     mH2)))*power2(sa)*power4(snt))/power2(mH2) - (36*(-mh2 - mst12 - mst22 -
     4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power2(log(mh2/Q2))*power4(snt))/power2(mh2) + (36*(-mh2 - mst12 - mst22 -
     4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power2(sa)*power2(log(mh2/Q2))*power4(snt))/power2(mh2) - (36*(-mH2 -
     mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(log(mH2/Q2))*power4(snt))/power2(mH2) + (36*(-
     mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(sa)*power2(log(mH2/Q2))*power4(snt))/power2(
     mH2) - 12*power2(snt)*power2(log(mst12/Q2))*(3/mh2 + 3/mH2 + (12*mst12)/(
     mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,
     mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,
     mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - (5*mst12)/
     power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (8*mst14)/((mh2 -
     mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2)) - (3*mst12)/
     power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 +
     2*mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,mst12,mh2)*(mh2*
     mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))) + 12*power2(sa)*power2(snt)*
     power2(log(mst12/Q2))*(3/mh2 + 3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*
     DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2
     - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - (5*mst12)/power2(mh2) + (2*
     mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (8*mst14)/((mh2 - mH2)*power2(mh2)
     ) + (2*mst12)/(-(mh2*mH2) + power2(mh2)) - (3*mst12)/power2(mH2) - (3*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 - (8*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(
     mst1)))/(mh2*(mh2 - mH2))) + 12*power2(log(mst12/Q2))*power4(snt)*(3/mh2 +
     3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*
     mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))
     /mh2 - (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) +
     (8*mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2))
     - (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))) - 12*power2(
     sa)*power2(log(mst12/Q2))*power4(snt)*(3/mh2 + 3/mH2 + (12*mst12)/(mh2*
     mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2)
     + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,
     mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - (5*mst12)/power2(
     mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (8*mst14)/((mh2 - mH2)*
     power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2)) - (3*mst12)/power2(
     mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*
     mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,mst12,mh2)*(mh2*
     mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))) + 3*power2(log(mst12/Q2))*(3/
     mh2 + 3/mH2 - (16*mst12*mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)) - (96*
     mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) - 3*mh2*DeltaInv(mst12,mst22,
     mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(
     mst12,mst22,mh2))/mh2 + (40*mst12*mt2)/((mst12 - mst22)*power2(mh2)) + (
     16*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) - (64*mst14*
     mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + (24*mst12*mt2)/((mst12 -
     mst22)*power2(mH2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (64*mt2*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))
     ) - 3*power2(sa)*power2(log(mst12/Q2))*(3/mh2 + 3/mH2 - (16*mst12*mt2)/(
     mh2*(mh2 - mH2)*(mst12 - mst22)) - (96*mst12*mt2)/(mh2*mH2*mst12 - mh2*
     mH2*mst22) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,
     mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(
     mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 + (40*
     mst12*mt2)/((mst12 - mst22)*power2(mh2)) + (16*mH2*mst12*mt2)/((mh2 - mH2)
     *(mst12 - mst22)*power2(mh2)) - (64*mst14*mt2)/((mh2 - mH2)*(mst12 -
     mst22)*power2(mh2)) + (24*mst12*mt2)/((mst12 - mst22)*power2(mH2)) - (3*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 + (64*mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*
     power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))) + 72*log(mst12/Q2)*
     power2(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(mh2*mH2) + (6*mst14*DeltaInv(
     mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*
     mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))
     /mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (4*
     mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (3*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*
     mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))) - 72*log(mst12/Q2)*power2(sa)*
     power2(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(mh2*mH2) + (6*mst14*DeltaInv(
     mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*
     mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))
     /mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (4*
     mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (3*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*
     mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))) - 72*log(mst12/Q2)*power4(snt)
     *(3/mh2 + 3/mH2 + (6*mst12)/(mh2*mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2)
     )/mH2 - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,
     mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,
     mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(
     mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*
     power2(mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,
     mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2
     + (2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))
     /(mh2*(mh2 - mH2))) + 72*log(mst12/Q2)*power2(sa)*power4(snt)*(3/mh2 + 3/
     mH2 + (6*mst12)/(mh2*mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*
     mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*
     mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2)
     )/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (
     mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)
     ) + mst12/(-(mh2*mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*
     mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*
     DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(
     mh2*(mh2 - mH2))) - 6*log(mst12/Q2)*(9/mh2 + 9/mH2 - (16*mst12*mt2)/(mh2*(
     mh2 - mH2)*(mst12 - mst22)) - (144*mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*
     mst22) - (144*mst14*mt2*DeltaInv(mst12,mst12,mH2))/(mH2*mst12 - mH2*mst22)
     - 9*mh2*DeltaInv(mst12,mst22,mh2) + 9*mst12*DeltaInv(mst12,mst22,mh2) +
     18*mst22*DeltaInv(mst12,mst22,mh2) + (27*mst12*mst22*DeltaInv(mst12,mst22,
     mh2))/mh2 - (9*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) +
     (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (16*mst12*mt2)/((mst12 - mst22)*
     power2(mh2)) + (16*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)
     ) - (96*mst14*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + mst12/(-(
     mh2*mH2) + power2(mh2)) - (9*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (48*mt2*DeltaInv(
     mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 -
     mH2)*(mst12 - mst22))) + 6*log(mst12/Q2)*power2(sa)*(9/mh2 + 9/mH2 - (16*
     mst12*mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)) - (144*mst12*mt2)/(mh2*mH2*
     mst12 - mh2*mH2*mst22) - (144*mst14*mt2*DeltaInv(mst12,mst12,mH2))/(mH2*
     mst12 - mH2*mst22) - 9*mh2*DeltaInv(mst12,mst22,mh2) + 9*mst12*DeltaInv(
     mst12,mst22,mh2) + 18*mst22*DeltaInv(mst12,mst22,mh2) + (27*mst12*mst22*
     DeltaInv(mst12,mst22,mh2))/mh2 - (9*mst24*DeltaInv(mst12,mst22,mh2))/mh2 -
     mst12/power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (16*mst12*
     mt2)/((mst12 - mst22)*power2(mh2)) + (16*mH2*mst12*mt2)/((mh2 - mH2)*(
     mst12 - mst22)*power2(mh2)) - (96*mst14*mt2)/((mh2 - mH2)*(mst12 - mst22)*
     power2(mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (9*DeltaInv(mst12,mH2,
     mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2
     - (48*mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(
     mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))) - (3*power2(log(mst22/Q2))*(-3*
     (mh2 - mH2)*mH2*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mh2*(mh2 - mH2)*(mst12 - mst22)*
     (mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) -
     8*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*power2(mH2) - (3*mH2*mst22
     + 8*mst24)*power2(mH2) + 3*mst22*power3(mh2) + 8*mh2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/((mh2 - mH2)*(-mst12 +
     mst22)*power2(mh2)*power2(mH2)) + (3*power2(sa)*power2(log(mst22/Q2))*(-3*
     (mh2 - mH2)*mH2*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mh2*(mh2 - mH2)*(mst12 - mst22)*
     (mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) -
     8*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*power2(mH2) - (3*mH2*mst22
     + 8*mst24)*power2(mH2) + 3*mst22*power3(mh2) + 8*mh2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/((mh2 - mH2)*(-mst12 +
     mst22)*power2(mh2)*power2(mH2)) - (72*log(mst22/Q2)*power2(snt)*(6*mh2*
     mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,
     mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*
     mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)) + (
     72*log(mst22/Q2)*power2(sa)*power2(snt)*(6*mh2*mH2*mst22 - 4*mH2*mst24 -
     6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*
     power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(
     mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)
     *power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*
     mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*
     power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)) + (72*log(mst22/Q2)*power4(
     snt)*(6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)
     *mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,
     mst22,mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)
     + 3*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)) - (
     72*log(mst22/Q2)*power2(sa)*power4(snt)*(6*mh2*mH2*mst22 - 4*mH2*mst24 -
     6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*
     power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(
     mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)
     *power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*
     mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*
     power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)) + (18*log(mst22/Q2)*(-48*mh2*
     mH2*mst22*mt2 + 32*mH2*mst24*mt2 + 48*mst22*mt2*power2(mh2) + 48*(mh2 -
     mH2)*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*(mh2 - mH2)*(
     mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)
     *power2(mh2) + 3*mH2*mst12*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) -
     3*mH2*mst22*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) + 16*mH2*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2) - 3*mh2*mst12*mst24*DeltaInv(
     mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst22*mst24*DeltaInv(mst12,mst22,mh2)
     *power2(mH2) - 48*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2) + 3*
     mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst12) - 3*mh2*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12) - 3*mH2*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22) + 3*mh2*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22) - 3*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power3(mh2) + 3*mH2*DeltaInv(mst12,mst22,mh2)*
     power2(mst22)*power3(mh2) + 128*mh2*mH2*mt2*DeltaInv(mst22,mst22,mh2)*
     power6(mst2)))/((mh2 - mH2)*mH2*(-mst12 + mst22)*power2(mh2)) + (18*log(
     mst22/Q2)*power2(sa)*(-48*mh2*mH2*mst22*mt2 + 32*mH2*mst24*mt2 + 48*mst22*
     mt2*power2(mh2) + 48*(mh2 - mH2)*mst24*mt2*DeltaInv(mH2,mst22,mst22)*
     power2(mh2) - 3*(mh2 - mH2)*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 -
     mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 3*mH2*mst12*mst24*DeltaInv(
     mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst22*mst24*DeltaInv(mst12,mst22,mh2)
     *power2(mh2) + 16*mH2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2) - 3*
     mh2*mst12*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst22*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) - 48*mh2*mst24*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2) + 3*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst12) - 3*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(
     mst12) - 3*mH2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22) +
     3*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22) - 3*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*
     mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) + 3*mH2*DeltaInv(mst12,
     mst22,mh2)*power2(mst22)*power3(mh2) + 128*mh2*mH2*mt2*DeltaInv(mst22,
     mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*(mst12 - mst22)*power2(mh2)) +
     (12*power2(snt)*power2(log(mst22/Q2))*(-15*mH2*mst22*power2(mh2) + 3*(mh2
     - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2
     - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(
     mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*
     mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)
     *power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)) - (12*
     power2(sa)*power2(snt)*power2(log(mst22/Q2))*(-15*mH2*mst22*power2(mh2) +
     3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22)*power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*
     mh2*(mh2 - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,
     mh2)*power2(mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)
     ) - (12*power2(log(mst22/Q2))*power4(snt)*(-15*mH2*mst22*power2(mh2) + 3*(
     mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)
     *power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2
     - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(
     mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*
     mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)
     *power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)) + (12*
     power2(sa)*power2(log(mst22/Q2))*power4(snt)*(-15*mH2*mst22*power2(mh2) +
     3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22)*power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*
     mh2*(mh2 - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,
     mh2)*power2(mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)
     ) + (84*power2(snt)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) -
     18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*
     mst12*power2(mH2) - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*
     mst24*power2(mH2) - 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(
     mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*
     mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*
     power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*
     mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2
     - mH2)*power2(mh2)*power2(mH2)) + (2*power2(Pi)*power2(snt)*(-3*mH2*mst12*
     power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(
     mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) +
     6*mh2*power3(mH2) + 3*mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*
     mst14*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*
     power3(mH2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) +
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))
     /((mh2 - mH2)*power2(mh2)*power2(mH2)) + (84*power2(sa)*power4(snt)*(-3*
     mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*
     mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*
     mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*
     mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*power3(
     mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*mst12*
     power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) +
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*
     power2(mH2)) + (2*power2(Pi)*power2(sa)*power4(snt)*(-3*mH2*mst12*power2(
     mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,
     mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(mH2) +
     3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mh2)*power2(mH2) - 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*
     mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*
     power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)
     *power3(mh2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) -
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(
     mH2) + 3*mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(
     mst12,mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,
     mh2)*power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2
     - mH2)*power2(mh2)*power2(mH2)) + (84*power2(sa)*power2(snt)*(3*mH2*mst12*
     power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2) + 4*mst14*power2(
     mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) + 14*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) + 6*mH2*power3(mh2) + 3*
     mst12*power3(mh2) + 3*mst22*power3(mh2) + 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) -
     6*mh2*power3(mH2) - 3*mst12*power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*
     mst14*DeltaInv(mst12,mst12,mh2)*power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mH2) - 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*
     power3(mH2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) -
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power4(mh2) + 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))
     /((mh2 - mH2)*power2(mh2)*power2(mH2)) + (2*power2(Pi)*power2(sa)*power2(
     snt)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*
     mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2)
     + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) +
     14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(
     mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*
     power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) -
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*
     power2(mH2)) + (84*power4(snt)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*
     power2(mh2) + 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) - 3*mh2*mst12*power2(mH2) + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(
     mH2) + 4*mst24*power2(mH2) + 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(
     mh2)*power2(mH2) - 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(
     mH2) + 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) +
     14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)
     *mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*
     power3(mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*
     mst12*power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) - 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2
     - mH2)*power2(mh2)*power2(mH2)) + (2*power2(Pi)*power4(snt)*(3*mH2*mst12*
     power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2) + 4*mst14*power2(
     mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) + 14*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) + 6*mH2*power3(mh2) + 3*
     mst12*power3(mh2) + 3*mst22*power3(mh2) + 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) -
     6*mh2*power3(mH2) - 3*mst12*power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*
     mst14*DeltaInv(mst12,mst12,mh2)*power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mH2) - 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*
     power3(mH2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) -
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power4(mh2) + 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))
     /((mh2 - mH2)*power2(mh2)*power2(mH2)) - (6*(84*mH2*mst12*mt2*power2(mh2)
     - 84*mH2*mst22*mt2*power2(mh2) - 504*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) - 84*mh2*mst12*mt2*power2(mH2) + 112*mst14*
     mt2*power2(mH2) + 84*mh2*mst22*mt2*power2(mH2) - 112*mst24*mt2*power2(mH2)
     + 392*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 504*
     mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 392*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 21*(mh2 - mH2)*
     mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22
     - mH2*(mst12 + mst22) + power2(mH2)) - 42*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst12) + 42*mst12*DeltaInv(mst12,mst22,mh2)
     *power2(mh2)*power2(mH2)*power2(mst22) - 15*mH2*mst12*power3(mh2) + 15*
     mH2*mst22*power3(mh2) + 84*mst12*mt2*power3(mh2) - 84*mst22*mt2*power3(
     mh2) + 504*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 21*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) + 21*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 15*mh2*
     mst12*power3(mH2) - 15*mh2*mst22*power3(mH2) - 84*mst12*mt2*power3(mH2) +
     84*mst22*mt2*power3(mH2) - 504*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*
     power3(mH2) + 504*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) +
     42*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mst12)*power3(mH2) + 21*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst12)*power3(mH2) - 42*mh2*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mH2) - 21*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mst22)*power3(mH2) - 21*mst12*
     DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power4(mh2) - 21*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 448*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 448*mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(
     mst2)))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)*power2(mH2)) + (6*power2(
     sa)*(84*mH2*mst12*mt2*power2(mh2) - 84*mH2*mst22*mt2*power2(mh2) - 504*(
     mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 84*mh2*
     mst12*mt2*power2(mH2) + 112*mst14*mt2*power2(mH2) + 84*mh2*mst22*mt2*
     power2(mH2) - 112*mst24*mt2*power2(mH2) + 392*mst14*mt2*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) - 504*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) - 392*mst24*mt2*DeltaInv(mst22,mst22,mh2)*
     power2(mh2)*power2(mH2) + 21*(mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) - 42*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*
     power2(mst12) + 42*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)
     *power2(mst22) - 15*mH2*mst12*power3(mh2) + 15*mH2*mst22*power3(mh2) + 84*
     mst12*mt2*power3(mh2) - 84*mst22*mt2*power3(mh2) + 504*mH2*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power3(mh2) - 21*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst12)*power3(mh2) + 21*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst22)*power3(mh2) + 15*mh2*mst12*power3(mH2) - 15*mh2*
     mst22*power3(mH2) - 84*mst12*mt2*power3(mH2) + 84*mst22*mt2*power3(mH2) -
     504*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 504*mh2*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 42*mh2*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mst12)*power3(mH2) + 21*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mst12)*power3(mH2) - 42*mh2*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mst22)*power3(mH2) - 21*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst22)*power3(mH2) - 21*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)
     *power3(mH2) + 21*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2)
     + 21*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 21*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 448*mh2*mt2*DeltaInv(
     mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 448*mh2*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*(mst12 - mst22)*power2(
     mh2)*power2(mH2)) - (power2(Pi)*(12*mH2*mst12*mt2*power2(mh2) - 12*mH2*
     mst22*mt2*power2(mh2) - 72*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,
     mst22)*power2(mh2) - 12*mh2*mst12*mt2*power2(mH2) + 16*mst14*mt2*power2(
     mH2) + 12*mh2*mst22*mt2*power2(mH2) - 16*mst24*mt2*power2(mH2) + 56*mst14*
     mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 72*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 56*mst24*mt2*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*(mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12
     + mst22) + power2(mH2)) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2)*power2(mst12) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2)*power2(mst22) - 3*mH2*mst12*power3(mh2) + 3*mH2*mst22*power3(
     mh2) + 12*mst12*mt2*power3(mh2) - 12*mst22*mt2*power3(mh2) + 72*mH2*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst12)*power3(mh2) + 3*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst22)*power3(mh2) + 3*mh2*mst12*power3(mH2) - 3*mh2*
     mst22*power3(mH2) - 12*mst12*mt2*power3(mH2) + 12*mst22*mt2*power3(mH2) -
     72*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 72*mh2*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mh2*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mst12)*power3(mH2) + 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst12)*power3(mH2) - 6*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(
     mst22)*power3(mH2) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22)
     *power3(mH2) - 3*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) +
     3*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 3*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 64*mh2*mt2*DeltaInv(mst12,
     mst12,mh2)*power2(mH2)*power6(mst1) - 64*mh2*mt2*DeltaInv(mst22,mst22,mh2)
     *power2(mH2)*power6(mst2)))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)*
     power2(mH2)) + (power2(Pi)*power2(sa)*(12*mH2*mst12*mt2*power2(mh2) - 12*
     mH2*mst22*mt2*power2(mh2) - 72*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,
     mst22,mst22)*power2(mh2) - 12*mh2*mst12*mt2*power2(mH2) + 16*mst14*mt2*
     power2(mH2) + 12*mh2*mst22*mt2*power2(mH2) - 16*mst24*mt2*power2(mH2) +
     56*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 72*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 56*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*(
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 -
     mH2*(mst12 + mst22) + power2(mH2)) - 6*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst12) + 6*mst12*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*mst12*power3(mh2) + 3*mH2*
     mst22*power3(mh2) + 12*mst12*mt2*power3(mh2) - 12*mst22*mt2*power3(mh2) +
     72*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*DeltaInv(mst12,
     mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) + 3*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 3*mh2*mst12*power3(mH2) - 3*
     mh2*mst22*power3(mH2) - 12*mst12*mt2*power3(mH2) + 12*mst22*mt2*power3(
     mH2) - 72*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 72*mh2*
     mst24*mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mh2*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mst12)*power3(mH2) + 3*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mst12)*power3(mH2) - 6*mh2*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mst22)*power3(mH2) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst22)*power3(mH2) - 3*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 3*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) +
     3*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 3*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 64*mh2*mt2*DeltaInv(
     mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 64*mh2*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*(mst12 - mst22)*power2(
     mh2)*power2(mH2))))/8.) + xtau*((6*cb*mt2*mu*mu2*Fin3(mst12,mstau12,mt2,
     Q2))/((-mst12 + mst22)*(mstau12 - mstau22)*sb*power2(mstau12 - mu2)) + (6*
     cb*mt2*mu*mu2*Fin3(mst22,mstau12,mt2,Q2))/((mst12 - mst22)*(mstau12 -
     mstau22)*sb*power2(mstau12 - mu2)) + (6*cb*mt2*mu*mu2*Fin3(mstau22,mst12,
     mt2,Q2))/((mst12 - mst22)*(mstau12 - mstau22)*sb*power2(mstau22 - mu2)) +
     (6*cb*mt2*mu*mu2*Fin3(mstau22,mst22,mt2,Q2))/((-mst12 + mst22)*(mstau12 -
     mstau22)*sb*power2(mstau22 - mu2)) - (6*cb*mu*mu2*Fin3(mst12,mt2,mu2,Q2)*(
     mstau22*mt2 + mstau22*mu2 - 2*mt2*mu2 + mstau12*(-mstau22 + mt2 + mu2) -
     power2(mu2) + (mstau12 - mu2)*(mstau22 - mu2)*DeltaInv(mst12,mt2,mu2)*(
     mst14 - 3*mst12*mt2 - 2*mst12*mu2 - mt2*mu2 + power2(mu2))))/((mst12 -
     mst22)*sb*power2(mstau12 - mu2)*power2(mstau22 - mu2)) + (6*cb*mu*mu2*
     Fin3(mst22,mt2,mu2,Q2)*(mstau22*mt2 + mstau22*mu2 - 2*mt2*mu2 + mstau12*(-
     mstau22 + mt2 + mu2) - power2(mu2) + (mstau12 - mu2)*(mstau22 - mu2)*
     DeltaInv(mst22,mt2,mu2)*(mst24 - 3*mst22*mt2 - 2*mst22*mu2 - mt2*mu2 +
     power2(mu2))))/((mst12 - mst22)*sb*power2(mstau12 - mu2)*power2(mstau22 -
     mu2)) + (cb*mu*mu2*(-84*mst12 + 84*mst22 + 18*mst12*log(mst12/Q2) - 18*
     mst22*log(mst22/Q2) + 54*mst12*log(mt2/Q2) - 54*mst22*log(mt2/Q2) - 12*
     mst12*log(mst12/Q2)*log(mt2/Q2) + 12*mst22*log(mst22/Q2)*log(mt2/Q2) + 6*
     mst12*log(mst12/Q2)*log(mu2/Q2) - 6*mst22*log(mst22/Q2)*log(mu2/Q2) - 6*
     mst12*log(mt2/Q2)*log(mu2/Q2) + 6*mst22*log(mt2/Q2)*log(mu2/Q2) - 2*mst12*
     power2(Pi) + 2*mst22*power2(Pi) - 3*mst12*power2(log(mst12/Q2)) + 3*mst22*
     power2(log(mst22/Q2)) - 9*mst12*power2(log(mt2/Q2)) + 9*mst22*power2(log(
     mt2/Q2)) + DeltaInv(mst12,mt2,mu2)*(-168*mst14*mt2 - 126*mst14*mu2 - 294*
     mst12*mt2*mu2 - 18*mst14*mu2*log(mu2/Q2) + 54*mst12*mt2*mu2*log(mu2/Q2) -
     42*mt2*power2(mu2) + 36*mst12*log(mu2/Q2)*power2(mu2) + 18*mt2*log(mu2/Q2)
     *power2(mu2) - 4*mst14*mt2*power2(Pi) - 3*mst14*mu2*power2(Pi) - 7*mst12*
     mt2*mu2*power2(Pi) - mt2*power2(mu2)*power2(Pi) + 3*mst14*mu2*power2(log(
     mu2/Q2)) - 9*mst12*mt2*mu2*power2(log(mu2/Q2)) - 6*mst12*power2(mu2)*
     power2(log(mu2/Q2)) - 3*mt2*power2(mu2)*power2(log(mu2/Q2)) + 42*power3(
     mu2) - 18*log(mu2/Q2)*power3(mu2) + power2(Pi)*power3(mu2) + 3*power2(log(
     mu2/Q2))*power3(mu2) + 84*power6(mst1) + 2*power2(Pi)*power6(mst1) + 3*
     power2(log(mst12/Q2))*(mst12*mu2*(-mt2 + mu2) - mst14*(3*mt2 + 2*mu2) +
     power6(mst1)) + 3*power2(log(mt2/Q2))*(-5*mst14*(mt2 + mu2) + mu2*(-10*
     mst12*mt2 + mst12*mu2 - mt2*mu2 + power2(mu2)) + 3*power6(mst1)) + 6*log(
     mst12/Q2)*(9*mst14*mt2 + 6*mst14*mu2 + 3*mst12*mt2*mu2 - 3*mst12*power2(
     mu2) + log(mu2/Q2)*(mst12*(3*mt2 - mu2)*mu2 + mst14*(mt2 + 2*mu2) -
     power6(mst1)) - 3*power6(mst1) + 2*log(mt2/Q2)*(mst12*mu2*(-2*mt2 + mu2) -
     2*mst14*(mt2 + mu2) + power6(mst1))) + 6*log(mt2/Q2)*(log(mu2/Q2)*(-(
     mst14*(mt2 + mu2)) + mu2*(mu2*(-mt2 + mu2) - mst12*(6*mt2 + mu2)) +
     power6(mst1)) - 3*(-5*mst14*(mt2 + mu2) + mu2*(-10*mst12*mt2 + mst12*mu2 -
     mt2*mu2 + power2(mu2)) + 3*power6(mst1)))) + DeltaInv(mst22,mt2,mu2)*(168*
     mst24*mt2 + 126*mst24*mu2 + 294*mst22*mt2*mu2 + 18*mst24*mu2*log(mu2/Q2) -
     54*mst22*mt2*mu2*log(mu2/Q2) + 42*mt2*power2(mu2) - 36*mst22*log(mu2/Q2)*
     power2(mu2) - 18*mt2*log(mu2/Q2)*power2(mu2) + 4*mst24*mt2*power2(Pi) + 3*
     mst24*mu2*power2(Pi) + 7*mst22*mt2*mu2*power2(Pi) + mt2*power2(mu2)*
     power2(Pi) - 3*mst24*mu2*power2(log(mu2/Q2)) + 9*mst22*mt2*mu2*power2(log(
     mu2/Q2)) + 6*mst22*power2(mu2)*power2(log(mu2/Q2)) + 3*mt2*power2(mu2)*
     power2(log(mu2/Q2)) - 42*power3(mu2) + 18*log(mu2/Q2)*power3(mu2) -
     power2(Pi)*power3(mu2) - 3*power2(log(mu2/Q2))*power3(mu2) - 84*power6(
     mst2) - 2*power2(Pi)*power6(mst2) - 3*power2(log(mst22/Q2))*(mst22*mu2*(-
     mt2 + mu2) - mst24*(3*mt2 + 2*mu2) + power6(mst2)) - 3*power2(log(mt2/Q2))
     *(-5*mst24*(mt2 + mu2) + mu2*(-10*mst22*mt2 + mst22*mu2 - mt2*mu2 +
     power2(mu2)) + 3*power6(mst2)) - 6*log(mst22/Q2)*(9*mst24*mt2 + 6*mst24*
     mu2 + 3*mst22*mt2*mu2 - 3*mst22*power2(mu2) + log(mu2/Q2)*(mst22*(3*mt2 -
     mu2)*mu2 + mst24*(mt2 + 2*mu2) - power6(mst2)) - 3*power6(mst2) + 2*log(
     mt2/Q2)*(mst22*mu2*(-2*mt2 + mu2) - 2*mst24*(mt2 + mu2) + power6(mst2))) -
     6*log(mt2/Q2)*(log(mu2/Q2)*(-(mst24*(mt2 + mu2)) + mu2*(mu2*(-mt2 + mu2) -
     mst22*(6*mt2 + mu2)) + power6(mst2)) - 3*(-5*mst24*(mt2 + mu2) + mu2*(-10*
     mst22*mt2 + mst22*mu2 - mt2*mu2 + power2(mu2)) + 3*power6(mst2))))))/((
     mst12 - mst22)*(mstau12 - mu2)*(-mstau22 + mu2)*sb)) + xt*((9*mu*sa*Fin3(
     mst12,mst12,mH2,Q2)*(-(ca*sb*(-1 + 2*power2(sa))*(mt2 - (mst12 - mst22)*(-
     1 + power2(snt))*power2(snt))) + 2*cb*sa*(-1 + power2(sa))*(2*mt2 - (mst12
     - mst22)*(-1 + power2(snt))*power2(snt)) + 2*(mh2 - mH2)*mst12*DeltaInv(
     mst12,mst12,mH2)*(-1 + power2(sa))*(-(ca*sb*(mt2 - (mst12 - mst22)*(-1 +
     power2(snt))*power2(snt))) + cb*sa*(2*mt2 - (mst12 - mst22)*(-1 + power2(
     snt))*power2(snt)))))/((mh2 - mH2)*mH2*(mst12 - mst22)*sb) + (3*mu*sa*
     Fin3(mst12,mst12,mh2,Q2)*((3*mh2 - 2*mst12)*(ca*sb*(-1 + 2*power2(sa))*(
     mt2 - (mst12 - mst22)*(-1 + power2(snt))*power2(snt)) - 2*cb*sa*(-1 +
     power2(sa))*(2*mt2 - (mst12 - mst22)*(-1 + power2(snt))*power2(snt))) - 2*
     mh2*DeltaInv(mst12,mst12,mh2)*(cb*(mh2*mst12 - 3*mH2*mst12 + 8*mst14)*sa*(
     -1 + power2(sa))*(-2*mt2 + (mst12 - mst22)*(-1 + power2(snt))*power2(snt))
     - ca*sb*(-3*mH2*mst12*power2(sa) + mh2*mst12*(1 + power2(sa)) + mst14*(-4
     + 8*power2(sa)))*(-mt2 + (mst12 - mst22)*(-1 + power2(snt))*power2(snt))))
     )/((mh2 - mH2)*(mst12 - mst22)*sb*power2(mh2)) - (9*mu*sa*Fin3(mH2,mst22,
     mst22,Q2)*(-(ca*sb*(-1 + 2*power2(sa))*(mt2 + (mst12 - mst22)*(-1 +
     power2(snt))*power2(snt))) + 2*cb*sa*(-1 + power2(sa))*(2*mt2 + (mst12 -
     mst22)*(-1 + power2(snt))*power2(snt)) + 2*(mh2 - mH2)*mst22*DeltaInv(mH2,
     mst22,mst22)*(-1 + power2(sa))*(-(ca*sb*(mt2 + (mst12 - mst22)*(-1 +
     power2(snt))*power2(snt))) + cb*sa*(2*mt2 + (mst12 - mst22)*(-1 + power2(
     snt))*power2(snt)))))/((mh2 - mH2)*mH2*(mst12 - mst22)*sb) + (3*mu*sa*
     Fin3(mst22,mst22,mh2,Q2)*(2*mh2*DeltaInv(mst22,mst22,mh2)*(cb*(mh2*mst22 -
     3*mH2*mst22 + 8*mst24)*sa*(-1 + power2(sa))*(-2*mt2 - (mst12 - mst22)*(-1
     + power2(snt))*power2(snt)) - ca*sb*(-3*mH2*mst22*power2(sa) + mh2*mst22*(
     1 + power2(sa)) + mst24*(-4 + 8*power2(sa)))*(-mt2 - (mst12 - mst22)*(-1 +
     power2(snt))*power2(snt))) + (3*mh2 - 2*mst22)*(-(ca*sb*(-1 + 2*power2(sa)
     )*(mt2 + (mst12 - mst22)*(-1 + power2(snt))*power2(snt))) + 2*cb*sa*(-1 +
     power2(sa))*(2*mt2 + (mst12 - mst22)*(-1 + power2(snt))*power2(snt)))))/((
     mh2 - mH2)*(mst12 - mst22)*sb*power2(mh2)) + (9*mu*sa*Fin3(mst12,mH2,
     mst22,Q2)*(ca*sb*(mH2*(2 - 3*power2(sa)) + mh2*(-1 + power2(sa))) - cb*(
     mh2 - 3*mH2)*sa*(-1 + power2(sa)) + (mh2 - mH2)*mH2*(mH2 - mst12 - mst22)*
     (cb*sa - ca*sb)*DeltaInv(mst12,mH2,mst22)*(-1 + power2(sa)))*power2(1 - 2*
     power2(snt)))/(2.*(mh2 - mH2)*sb*power2(mH2)) + (9*mu*sa*Fin3(mst12,mst22,
     mh2,Q2)*(mh2*(mh2 - mH2)*(mh2 - mst12 - mst22)*sa*DeltaInv(mst12,mst22,
     mh2)*(-(ca*sa*sb) + cb*(-1 + power2(sa))) - cb*(3*mh2 - mH2)*sa*(-1 +
     power2(sa)) - ca*sb*(mh2 - 3*mh2*power2(sa) + mH2*power2(sa)))*power2(1 -
     2*power2(snt)))/(2.*(mh2 - mH2)*sb*power2(mh2)) + (mu*((-3*(-(1/mA2) + 1/
     mC2 + 3/mH2)*sb)/cb + (18*ca*sa*log(mh2/Q2))/(mh2 - mH2) + (3*(-(mC2*mH2)
     + mA2*(3*mC2 + mH2))*mst12*sb*log(mst12/Q2))/(cb*mA2*mC2*mH2*(mst12 -
     mst22)) + (3*(mC2*mH2 - mA2*(3*mC2 + mH2))*mst22*sb*log(mst22/Q2))/(cb*
     mA2*mC2*mH2*(mst12 - mst22)) - (36*ca*mst12*mst22*sa*DeltaInv(mst12,mH2,
     mst22)*log(mst12/Q2)*log(mst22/Q2))/mH2 + (18*ca*(mh2 - mH2)*sa*Fin20(
     mst12,mst22,Q2))/(mh2*power2(mH2)) - (18*ca*sa*(4*mst22*mt2 + 16*mH2*
     mst24*mt2*DeltaInv(mH2,mst22,mst22) - mH2*(mst12 - mst22)*(mH2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2))
     /((mst12 - mst22)*power2(mH2)) - (18*ca*sa*log(mH2/Q2)*(-2*mh2*mH2*mst12 +
     2*mh2*mH2*mst22 + 12*mh2*mst12*mt2 - 12*mH2*mst12*mt2 - 12*mh2*mst22*mt2 +
     12*mH2*mst22*mt2 - 48*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)
     + 48*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + 3*mst12*power2(mH2) -
     3*mst22*power2(mH2) + 3*(mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mH2) - 48*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power2(mH2)))/((mh2 - mH2)*(mst12 - mst22)*power2(mH2)) + (18*ca*sa*
     log(mH2/Q2)*log(mst12/Q2)*(-(mH2*mst12) + mH2*mst22 + 4*mst12*mt2 + 16*
     mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + mH2*(mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(
     mH2))))/((mst12 - mst22)*power2(mH2)) + (18*cb*(mh2 + mH2)*power2(sa))/(
     mh2*mH2*sb) + (18*sb*power2(sa))/(cb*mH2) - (18*sb*log(mh2/Q2)*power2(sa))
     /(cb*mh2 - cb*mH2) - (36*cb*log(mh2/Q2)*power2(sa))/(mh2*sb - mH2*sb) + (
     18*sb*log(mH2/Q2)*power2(sa))/(cb*mh2 - cb*mH2) + (36*cb*log(mH2/Q2)*
     power2(sa))/(mh2*sb - mH2*sb) - (18*cb*(mh2 + mH2)*mst12*log(mst12/Q2)*
     power2(sa))/(mh2*mH2*(mst12 - mst22)*sb) - (18*mst12*sb*log(mst12/Q2)*
     power2(sa))/(cb*mH2*mst12 - cb*mH2*mst22) + (36*cb*mst12*log(mh2/Q2)*log(
     mst12/Q2)*power2(sa))/((mh2 - mH2)*(mst12 - mst22)*sb) + (18*mst12*sb*log(
     mh2/Q2)*log(mst12/Q2)*power2(sa))/(cb*(mh2 - mH2)*(mst12 - mst22)) - (36*
     cb*mst12*log(mH2/Q2)*log(mst12/Q2)*power2(sa))/((mh2 - mH2)*(mst12 -
     mst22)*sb) - (18*mst12*sb*log(mH2/Q2)*log(mst12/Q2)*power2(sa))/(cb*(mh2 -
     mH2)*(mst12 - mst22)) + (18*cb*(mh2 + mH2)*mst22*log(mst22/Q2)*power2(sa))
     /(mh2*mH2*(mst12 - mst22)*sb) + (18*mst22*sb*log(mst22/Q2)*power2(sa))/(
     cb*mH2*mst12 - cb*mH2*mst22) - (36*cb*mst22*log(mh2/Q2)*log(mst22/Q2)*
     power2(sa))/((mh2 - mH2)*(mst12 - mst22)*sb) - (18*mst22*sb*log(mh2/Q2)*
     log(mst22/Q2)*power2(sa))/(cb*(mh2 - mH2)*(mst12 - mst22)) + (36*cb*mst22*
     log(mH2/Q2)*log(mst22/Q2)*power2(sa))/((mh2 - mH2)*(mst12 - mst22)*sb) + (
     18*mst22*sb*log(mH2/Q2)*log(mst22/Q2)*power2(sa))/(cb*(mh2 - mH2)*(mst12 -
     mst22)) + (36*cb*mst12*mst22*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*
     DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power2(sa))/(mh2*
     mH2*sb) + (18*cb*(-(mh2*(mst12 - mst22)*(mh2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mst22,mh2)) + 8*mt2*(mst22 + 4*mh2*mst24*DeltaInv(mst22,
     mst22,mh2)))*log(mh2/Q2)*log(mst22/Q2)*power2(sa))/((mst12 - mst22)*sb*
     power2(mh2)) - (18*cb*log(mh2/Q2)*log(mst12/Q2)*(-(mh2*mst12) + mh2*mst22
     + 8*mst12*mt2 + 32*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2) + mh2*(mst12 -
     mst22)*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) +
     mst24 + power2(mh2)))*power2(sa))/((mst12 - mst22)*sb*power2(mh2)) - (18*
     cb*log(mh2/Q2)*((-96*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*mst12 -
     mh2*mst22) + 3*(-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) + (96*mh2*
     (mh2 - mH2)*mst24*mt2*DeltaInv(mst22,mst22,mh2) + 2*(mst12 - mst22)*(12*
     mH2*mt2 - mh2*(mH2 + 12*mt2) + 2*power2(mh2)))/((mh2 - mH2)*(mst12 -
     mst22)*power2(mh2)))*power2(sa))/sb + (18*cb*(8*mst22*mt2 + 32*mH2*mst24*
     mt2*DeltaInv(mH2,mst22,mst22) - mH2*(mst12 - mst22)*(mH2*mst22 + mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*
     power2(sa))/((mst12 - mst22)*sb*power2(mH2)) - (18*cb*Fin20(mst12,mst22,
     Q2)*power2(mh2 - mH2)*power2(sa))/(sb*power2(mh2)*power2(mH2)) + (18*cb*
     log(mH2/Q2)*(-2*mh2*mH2*mst12 + 2*mh2*mH2*mst22 + 24*mh2*mst12*mt2 - 24*
     mH2*mst12*mt2 - 24*mh2*mst22*mt2 + 24*mH2*mst22*mt2 - 96*(mh2 - mH2)*mH2*
     mst24*mt2*DeltaInv(mH2,mst22,mst22) + 96*mh2*mH2*mst14*mt2*DeltaInv(mst12,
     mst12,mH2) + 4*mst12*power2(mH2) - 4*mst22*power2(mH2) + 3*(mh2 - mH2)*(
     mH2 - mst12 - mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2)
     - 96*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2))*power2(sa))/((mh2 -
     mH2)*(mst12 - mst22)*sb*power2(mH2)) - (18*cb*log(mH2/Q2)*log(mst12/Q2)*(-
     (mH2*mst12) + mH2*mst22 + 8*mst12*mt2 + 32*mH2*mst14*mt2*DeltaInv(mst12,
     mst12,mH2) + mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22)
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(sa))/((mst12 -
     mst22)*sb*power2(mH2)) + (144*ca*mst12*mst22*sa*DeltaInv(mst12,mH2,mst22)*
     log(mst12/Q2)*log(mst22/Q2)*power2(snt))/mH2 + (72*ca*(-mh2 + mH2)*sa*
     Fin20(mst12,mst22,Q2)*power2(snt))/(mh2*power2(mH2)) - (72*ca*sa*(-mst22 -
     4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 -
     mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(snt))/
     power2(mH2) + (216*ca*sa*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*
     DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 -
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(snt))/power2(
     mH2) - (72*ca*sa*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22)
     - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(snt))/power2(mH2) -
     (144*cb*mst12*mst22*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,
     mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power2(sa)*power2(snt))/(mh2*mH2*
     sb) + (72*cb*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(
     mst12,mst22,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(
     mst22/Q2)*power2(sa)*power2(snt))/(sb*power2(mh2)) - (216*cb*log(mh2/Q2)*(
     -mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*
     mst24*DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,
     mst22,mh2)*power2(mh2))*power2(sa)*power2(snt))/(sb*power2(mh2)) + (72*cb*
     log(mh2/Q2)*log(mst12/Q2)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,
     mst12,mh2) + mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 +
     2*mst22) + mst24 + power2(mh2)))*power2(sa)*power2(snt))/(sb*power2(mh2))
     + (72*cb*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22
     + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/
     Q2)*power2(sa)*power2(snt))/(sb*power2(mH2)) + (72*cb*Fin20(mst12,mst22,
     Q2)*power2(mh2 - mH2)*power2(sa)*power2(snt))/(sb*power2(mh2)*power2(mH2))
     - (216*cb*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,
     mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(snt))/(sb*
     power2(mH2)) + (72*cb*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*
     mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*
     mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power2(sa)*power2(
     snt))/(sb*power2(mH2)) + (9*ca*sa*power2(log(mh2/Q2)))/(-mh2 + mH2) + (9*
     cb*(1/mh2 + 2/(mh2 - mH2) - (32*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*
     mst12 - mh2*mst22) + (-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) + (
     32*mst24*mt2*DeltaInv(mst22,mst22,mh2))/(mh2*mst12 - mh2*mst22) + (8*
     mst22*mt2)/((mst12 - mst22)*power2(mh2)) + (8*mst12*mt2)/((-mst12 + mst22)
     *power2(mh2)))*power2(sa)*power2(log(mh2/Q2)))/sb + (36*cb*(-mh2 - mst12 -
     mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(
     mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(
     mh2))*power2(sa)*power2(snt)*power2(log(mh2/Q2)))/(sb*power2(mh2)) + (9*
     ca*sa*(mh2*mH2*mst12 - mh2*mH2*mst22 - 4*mh2*mst12*mt2 + 4*mH2*mst12*mt2 +
     4*mh2*mst22*mt2 - 4*mH2*mst22*mt2 + 16*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(
     mH2,mst22,mst22) - 16*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) - 2*
     mst12*power2(mH2) + 2*mst22*power2(mH2) - (mh2 - mH2)*(mH2 - mst12 -
     mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) + 16*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2))*power2(log(mH2/Q2)))/((-mh2 +
     mH2)*(mst12 - mst22)*power2(mH2)) + (9*cb*(-(mh2*mH2*mst12) + mh2*mH2*
     mst22 + 8*mh2*mst12*mt2 - 8*mH2*mst12*mt2 - 8*mh2*mst22*mt2 + 8*mH2*mst22*
     mt2 - 32*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) + 32*mh2*mH2*
     mst14*mt2*DeltaInv(mst12,mst12,mH2) + 3*mst12*power2(mH2) - 3*mst22*
     power2(mH2) + (mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*power2(mH2) - 32*mst14*mt2*DeltaInv(mst12,mst12,mH2)*
     power2(mH2))*power2(sa)*power2(log(mH2/Q2)))/((-mh2 + mH2)*(mst12 - mst22)
     *sb*power2(mH2)) - (36*ca*sa*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(
     mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(snt)*power2(log(mH2/
     Q2)))/power2(mH2) + (36*cb*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(
     mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(snt)*
     power2(log(mH2/Q2)))/(sb*power2(mH2)) + (36*ca*mst12*mst22*(mh2*DeltaInv(
     mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/
     Q2)*power3(sa))/(mh2*mH2) + (18*ca*(-(mh2*(mst12 - mst22)*(mh2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)) + 4*mt2*(mst22 + 4*mh2*
     mst24*DeltaInv(mst22,mst22,mh2)))*log(mh2/Q2)*log(mst22/Q2)*power3(sa))/((
     mst12 - mst22)*power2(mh2)) - (18*ca*log(mh2/Q2)*log(mst12/Q2)*(-(mh2*
     mst12) + mh2*mst22 + 4*mst12*mt2 + 16*mh2*mst14*mt2*DeltaInv(mst12,mst12,
     mh2) + mh2*(mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) -
     mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))*power3(sa))/((mst12 - mst22)
     *power2(mh2)) + (18*ca*log(mh2/Q2)*(48*mh2*(mh2 - mH2)*mst14*mt2*DeltaInv(
     mst12,mst12,mh2) + 3*(mh2 - mH2)*(mh2 - mst12 - mst22)*(mst12 - mst22)*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*(24*mh2*(mh2 - mH2)*mst24*mt2*
     DeltaInv(mst22,mst22,mh2) + (mst12 - mst22)*(6*mH2*mt2 - mh2*(mH2 + 6*mt2)
     + 2*power2(mh2))))*power3(sa))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) +
     (18*ca*(4*mst22*mt2 + 16*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - mH2*(
     mst12 - mst22)*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)
     )*log(mH2/Q2)*log(mst22/Q2)*power3(sa))/((mst12 - mst22)*power2(mH2)) - (
     18*ca*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*power3(sa))/(power2(mh2)*
     power2(mH2)) + (18*ca*log(mH2/Q2)*(-48*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(
     mH2,mst22,mst22) - 2*((mst12 - mst22)*(mh2*(mH2 - 6*mt2) - 2*mH2*(mH2 - 3*
     mt2)) - 24*(mh2 - mH2)*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)) + 3*(mh2 -
     mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*
     power2(mH2))*power3(sa))/((mh2 - mH2)*(mst12 - mst22)*power2(mH2)) - (18*
     ca*log(mH2/Q2)*log(mst12/Q2)*(-(mH2*mst12) + mH2*mst22 + 4*mst12*mt2 + 16*
     mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + mH2*(mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(
     mH2)))*power3(sa))/((mst12 - mst22)*power2(mH2)) - (144*ca*mst12*mst22*(
     mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/
     Q2)*log(mst22/Q2)*power2(snt)*power3(sa))/(mh2*mH2) + (72*ca*(-mst22 +
     mh2*(mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*
     mst24*DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power2(snt)*
     power3(sa))/power2(mh2) - (216*ca*log(mh2/Q2)*(-mh2 - mst12 - mst22 - 4*
     mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power2(snt)*power3(sa))/power2(mh2) + (72*ca*log(mh2/Q2)*log(mst12/Q2)*(-
     mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(mst12,
     mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))
     *power2(snt)*power3(sa))/power2(mh2) + (72*ca*(-mst22 - 4*mH2*mst24*
     DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(snt)*power3(
     sa))/power2(mH2) + (72*ca*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*power2(
     snt)*power3(sa))/(power2(mh2)*power2(mH2)) - (216*ca*log(mH2/Q2)*(-mH2 -
     mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(snt)*power3(sa))/power2(mH2) + (72*ca*log(mH2/
     Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) +
     mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) +
     mst24 + power2(mH2)))*power2(snt)*power3(sa))/power2(mH2) + 9*ca*(1/mh2 +
     2/(mh2 - mH2) - (16*mst14*mt2*DeltaInv(mst12,mst12,mh2))/(mh2*mst12 - mh2*
     mst22) + (-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2) + (16*mst24*mt2*
     DeltaInv(mst22,mst22,mh2))/(mh2*mst12 - mh2*mst22) + (4*mst22*mt2)/((mst12
     - mst22)*power2(mh2)) + (4*mst12*mt2)/((-mst12 + mst22)*power2(mh2)))*
     power2(log(mh2/Q2))*power3(sa) + (36*ca*(-mh2 - mst12 - mst22 - 4*mh2*
     mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) +
     (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(snt)*
     power2(log(mh2/Q2))*power3(sa))/power2(mh2) + (9*ca*(-(mh2*mH2*mst12) +
     mh2*mH2*mst22 + 4*mh2*mst12*mt2 - 4*mH2*mst12*mt2 - 4*mh2*mst22*mt2 + 4*
     mH2*mst22*mt2 - 16*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) +
     16*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) + 3*mst12*power2(mH2) - 3*
     mst22*power2(mH2) + (mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mH2) - 16*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power2(mH2))*power2(log(mH2/Q2))*power3(sa))/((-mh2 + mH2)*(mst12 -
     mst22)*power2(mH2)) + (36*ca*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(
     mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(snt)*power2(log(mH2/
     Q2))*power3(sa))/power2(mH2) - (18*cb*(mh2 + mH2)*power4(sa))/(mh2*mH2*sb)
     - (9*(mh2 + mH2)*sb*power4(sa))/(cb*mh2*mH2) + (18*sb*log(mh2/Q2)*power4(
     sa))/(cb*mh2 - cb*mH2) + (36*cb*log(mh2/Q2)*power4(sa))/(mh2*sb - mH2*sb)
     - (18*sb*log(mH2/Q2)*power4(sa))/(cb*mh2 - cb*mH2) - (36*cb*log(mH2/Q2)*
     power4(sa))/(mh2*sb - mH2*sb) + (18*cb*(mh2 + mH2)*mst12*log(mst12/Q2)*
     power4(sa))/(mh2*mH2*(mst12 - mst22)*sb) + (9*(mh2 + mH2)*mst12*sb*log(
     mst12/Q2)*power4(sa))/(cb*mh2*mH2*(mst12 - mst22)) - (36*cb*mst12*log(mh2/
     Q2)*log(mst12/Q2)*power4(sa))/((mh2 - mH2)*(mst12 - mst22)*sb) - (18*
     mst12*sb*log(mh2/Q2)*log(mst12/Q2)*power4(sa))/(cb*(mh2 - mH2)*(mst12 -
     mst22)) + (36*cb*mst12*log(mH2/Q2)*log(mst12/Q2)*power4(sa))/((mh2 - mH2)*
     (mst12 - mst22)*sb) + (18*mst12*sb*log(mH2/Q2)*log(mst12/Q2)*power4(sa))/(
     cb*(mh2 - mH2)*(mst12 - mst22)) - (18*cb*(mh2 + mH2)*mst22*log(mst22/Q2)*
     power4(sa))/(mh2*mH2*(mst12 - mst22)*sb) - (9*(mh2 + mH2)*mst22*sb*log(
     mst22/Q2)*power4(sa))/(cb*mh2*mH2*(mst12 - mst22)) + (36*cb*mst22*log(mh2/
     Q2)*log(mst22/Q2)*power4(sa))/((mh2 - mH2)*(mst12 - mst22)*sb) + (18*
     mst22*sb*log(mh2/Q2)*log(mst22/Q2)*power4(sa))/(cb*(mh2 - mH2)*(mst12 -
     mst22)) - (36*cb*mst22*log(mH2/Q2)*log(mst22/Q2)*power4(sa))/((mh2 - mH2)*
     (mst12 - mst22)*sb) - (18*mst22*sb*log(mH2/Q2)*log(mst22/Q2)*power4(sa))/(
     cb*(mh2 - mH2)*(mst12 - mst22)) - (36*cb*mst12*mst22*(mh2*DeltaInv(mst12,
     mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*
     power4(sa))/(mh2*mH2*sb) - (18*cb*(-(mh2*(mst12 - mst22)*(mh2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)) + 8*mt2*(mst22 + 4*mh2*
     mst24*DeltaInv(mst22,mst22,mh2)))*log(mh2/Q2)*log(mst22/Q2)*power4(sa))/((
     mst12 - mst22)*sb*power2(mh2)) + (18*cb*log(mh2/Q2)*log(mst12/Q2)*(-(mh2*
     mst12) + mh2*mst22 + 8*mst12*mt2 + 32*mh2*mst14*mt2*DeltaInv(mst12,mst12,
     mh2) + mh2*(mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) -
     mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))*power4(sa))/((mst12 - mst22)
     *sb*power2(mh2)) + (18*cb*log(mh2/Q2)*((-96*mst14*mt2*DeltaInv(mst12,
     mst12,mh2))/(mh2*mst12 - mh2*mst22) + 3*(-mh2 + mst12 + mst22)*DeltaInv(
     mst12,mst22,mh2) + (96*mh2*(mh2 - mH2)*mst24*mt2*DeltaInv(mst22,mst22,mh2)
     + 2*(mst12 - mst22)*(12*mH2*mt2 - mh2*(mH2 + 12*mt2) + 2*power2(mh2)))/((
     mh2 - mH2)*(mst12 - mst22)*power2(mh2)))*power4(sa))/sb - (18*cb*(8*mst22*
     mt2 + 32*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) - mH2*(mst12 - mst22)*(
     mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*
     log(mst22/Q2)*power4(sa))/((mst12 - mst22)*sb*power2(mH2)) + (18*cb*Fin20(
     mst12,mst22,Q2)*power2(mh2 - mH2)*power4(sa))/(sb*power2(mh2)*power2(mH2))
     - (18*cb*log(mH2/Q2)*(-2*mh2*mH2*mst12 + 2*mh2*mH2*mst22 + 24*mh2*mst12*
     mt2 - 24*mH2*mst12*mt2 - 24*mh2*mst22*mt2 + 24*mH2*mst22*mt2 - 96*(mh2 -
     mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22) + 96*mh2*mH2*mst14*mt2*
     DeltaInv(mst12,mst12,mH2) + 4*mst12*power2(mH2) - 4*mst22*power2(mH2) + 3*
     (mh2 - mH2)*(mH2 - mst12 - mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2) - 96*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2))*
     power4(sa))/((mh2 - mH2)*(mst12 - mst22)*sb*power2(mH2)) + (18*cb*log(mH2/
     Q2)*log(mst12/Q2)*(-(mH2*mst12) + mH2*mst22 + 8*mst12*mt2 + 32*mH2*mst14*
     mt2*DeltaInv(mst12,mst12,mH2) + mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*
     power4(sa))/((mst12 - mst22)*sb*power2(mH2)) + (144*cb*mst12*mst22*(mh2*
     DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*
     log(mst22/Q2)*power2(snt)*power4(sa))/(mh2*mH2*sb) - (72*cb*(-mst22 + mh2*
     (mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power2(snt)*power4(
     sa))/(sb*power2(mh2)) + (216*cb*log(mh2/Q2)*(-mh2 - mst12 - mst22 - 4*mh2*
     mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) +
     (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(snt)*
     power4(sa))/(sb*power2(mh2)) - (72*cb*log(mh2/Q2)*log(mst12/Q2)*(-mh2 -
     mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(mst12,mst22,
     mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))*
     power2(snt)*power4(sa))/(sb*power2(mh2)) - (72*cb*(-mst22 - 4*mH2*mst24*
     DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(snt)*power4(
     sa))/(sb*power2(mH2)) - (72*cb*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*
     power2(snt)*power4(sa))/(sb*power2(mh2)*power2(mH2)) + (216*cb*log(mH2/Q2)
     *(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*
     mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,
     mH2,mst22)*power2(mH2))*power2(snt)*power4(sa))/(sb*power2(mH2)) - (72*cb*
     log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,
     mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 +
     2*mst22) + mst24 + power2(mH2)))*power2(snt)*power4(sa))/(sb*power2(mH2))
     - (9*cb*(1/mh2 + 2/(mh2 - mH2) - (32*mst14*mt2*DeltaInv(mst12,mst12,mh2))/
     (mh2*mst12 - mh2*mst22) + (-mh2 + mst12 + mst22)*DeltaInv(mst12,mst22,mh2)
     + (32*mst24*mt2*DeltaInv(mst22,mst22,mh2))/(mh2*mst12 - mh2*mst22) + (8*
     mst22*mt2)/((mst12 - mst22)*power2(mh2)) + (8*mst12*mt2)/((-mst12 + mst22)
     *power2(mh2)))*power2(log(mh2/Q2))*power4(sa))/sb - (36*cb*(-mh2 - mst12 -
     mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(
     mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(
     mh2))*power2(snt)*power2(log(mh2/Q2))*power4(sa))/(sb*power2(mh2)) + (9*
     cb*(mh2*mH2*mst12 - mh2*mH2*mst22 - 8*mh2*mst12*mt2 + 8*mH2*mst12*mt2 + 8*
     mh2*mst22*mt2 - 8*mH2*mst22*mt2 + 32*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(
     mH2,mst22,mst22) - 32*mh2*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2) - 3*
     mst12*power2(mH2) + 3*mst22*power2(mH2) - (mh2 - mH2)*(mH2 - mst12 -
     mst22)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2) + 32*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mH2))*power2(log(mH2/Q2))*power4(sa))
     /((-mh2 + mH2)*(mst12 - mst22)*sb*power2(mH2)) - (36*cb*(-mH2 - mst12 -
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power2(snt)*power2(log(mH2/Q2))*power4(sa))/(sb*power2(mH2)) - (144*
     ca*mst12*mst22*sa*DeltaInv(mst12,mH2,mst22)*log(mst12/Q2)*log(mst22/Q2)*
     power4(snt))/mH2 + (72*ca*(mh2 - mH2)*sa*Fin20(mst12,mst22,Q2)*power4(snt)
     )/(mh2*power2(mH2)) + (72*ca*sa*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,
     mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*
     log(mH2/Q2)*log(mst22/Q2)*power4(snt))/power2(mH2) - (216*ca*sa*log(mH2/
     Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*
     mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,
     mH2,mst22)*power2(mH2))*power4(snt))/power2(mH2) + (72*ca*sa*log(mH2/Q2)*
     log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + mH2*
     DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))*power4(snt))/power2(mH2) + (144*cb*mst12*mst22*(mh2*
     DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*
     log(mst22/Q2)*power2(sa)*power4(snt))/(mh2*mH2*sb) - (72*cb*(-mst22 + mh2*
     (mh2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*
     DeltaInv(mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power2(sa)*power4(
     snt))/(sb*power2(mh2)) + (216*cb*log(mh2/Q2)*(-mh2 - mst12 - mst22 - 4*
     mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power2(sa)*power4(snt))/(sb*power2(mh2)) - (72*cb*log(mh2/Q2)*log(mst12/
     Q2)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(
     mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(
     mh2)))*power2(sa)*power4(snt))/(sb*power2(mh2)) - (72*cb*(-mst22 - 4*mH2*
     mst24*DeltaInv(mH2,mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22))*log(mH2/Q2)*log(mst22/Q2)*power2(sa)*power4(
     snt))/(sb*power2(mH2)) - (72*cb*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*
     power2(sa)*power4(snt))/(sb*power2(mh2)*power2(mH2)) + (216*cb*log(mH2/Q2)
     *(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*
     mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,
     mH2,mst22)*power2(mH2))*power2(sa)*power4(snt))/(sb*power2(mH2)) - (72*cb*
     log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,
     mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 +
     2*mst22) + mst24 + power2(mH2)))*power2(sa)*power4(snt))/(sb*power2(mH2))
     - (36*cb*(-mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2) -
     4*mh2*mst24*DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(
     mst12,mst22,mh2)*power2(mh2))*power2(sa)*power2(log(mh2/Q2))*power4(snt))/
     (sb*power2(mh2)) + (36*ca*sa*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(
     mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(log(mH2/Q2))*power4(
     snt))/power2(mH2) - (36*cb*(-mH2 - mst12 - mst22 - 4*mH2*mst24*DeltaInv(
     mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 -
     mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power2(sa)*power2(log(mH2/
     Q2))*power4(snt))/(sb*power2(mH2)) + (144*ca*mst12*mst22*(mh2*DeltaInv(
     mst12,mH2,mst22) + mH2*DeltaInv(mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/
     Q2)*power3(sa)*power4(snt))/(mh2*mH2) - (72*ca*(-mst22 + mh2*(mh2*mst22 +
     mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*DeltaInv(
     mst22,mst22,mh2))*log(mh2/Q2)*log(mst22/Q2)*power3(sa)*power4(snt))/
     power2(mh2) + (216*ca*log(mh2/Q2)*(-mh2 - mst12 - mst22 - 4*mh2*mst14*
     DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) + (mh2 -
     mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power3(sa)*power4(
     snt))/power2(mh2) - (72*ca*log(mh2/Q2)*log(mst12/Q2)*(-mh2 - mst12 - 4*
     mh2*mst14*DeltaInv(mst12,mst12,mh2) + mh2*DeltaInv(mst12,mst22,mh2)*(-(
     mst12*mst22) - mh2*(mst12 + 2*mst22) + mst24 + power2(mh2)))*power3(sa)*
     power4(snt))/power2(mh2) - (72*ca*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,
     mst22,mst22) + mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22))*log(mH2/Q2)*log(mst22/Q2)*power3(sa)*power4(snt))/power2(mH2) - (
     72*ca*Fin20(mst12,mst22,Q2)*power2(mh2 - mH2)*power3(sa)*power4(snt))/(
     power2(mh2)*power2(mH2)) + (216*ca*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*
     mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,
     mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*
     power3(sa)*power4(snt))/power2(mH2) - (72*ca*log(mH2/Q2)*log(mst12/Q2)*(-
     mH2 - mst12 - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,
     mH2,mst22)*(-(mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))
     *power3(sa)*power4(snt))/power2(mH2) - (36*ca*(-mh2 - mst12 - mst22 - 4*
     mh2*mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,
     mh2) + (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*
     power2(log(mh2/Q2))*power3(sa)*power4(snt))/power2(mh2) - (36*ca*(-mH2 -
     mst12 - mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*
     DeltaInv(mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mH2))*power2(log(mH2/Q2))*power3(sa)*power4(snt))/power2(
     mH2) - (144*cb*mst12*mst22*(mh2*DeltaInv(mst12,mH2,mst22) + mH2*DeltaInv(
     mst12,mst22,mh2))*log(mst12/Q2)*log(mst22/Q2)*power4(sa)*power4(snt))/(
     mh2*mH2*sb) + (72*cb*(-mst22 + mh2*(mh2*mst22 + mst12*mst22 - mst24)*
     DeltaInv(mst12,mst22,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2))*log(
     mh2/Q2)*log(mst22/Q2)*power4(sa)*power4(snt))/(sb*power2(mh2)) - (216*cb*
     log(mh2/Q2)*(-mh2 - mst12 - mst22 - 4*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) + (mh2 - mst12 - mst22)*DeltaInv(
     mst12,mst22,mh2)*power2(mh2))*power4(sa)*power4(snt))/(sb*power2(mh2)) + (
     72*cb*log(mh2/Q2)*log(mst12/Q2)*(-mh2 - mst12 - 4*mh2*mst14*DeltaInv(
     mst12,mst12,mh2) + mh2*DeltaInv(mst12,mst22,mh2)*(-(mst12*mst22) - mh2*(
     mst12 + 2*mst22) + mst24 + power2(mh2)))*power4(sa)*power4(snt))/(sb*
     power2(mh2)) + (72*cb*(-mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) +
     mH2*(mH2*mst22 + mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22))*log(mH2/
     Q2)*log(mst22/Q2)*power4(sa)*power4(snt))/(sb*power2(mH2)) + (72*cb*Fin20(
     mst12,mst22,Q2)*power2(mh2 - mH2)*power4(sa)*power4(snt))/(sb*power2(mh2)*
     power2(mH2)) - (216*cb*log(mH2/Q2)*(-mH2 - mst12 - mst22 - 4*mH2*mst24*
     DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + (mH2 -
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mH2))*power4(sa)*power4(
     snt))/(sb*power2(mH2)) + (72*cb*log(mH2/Q2)*log(mst12/Q2)*(-mH2 - mst12 -
     4*mH2*mst14*DeltaInv(mst12,mst12,mH2) + mH2*DeltaInv(mst12,mH2,mst22)*(-(
     mst12*mst22) - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))*power4(sa)*
     power4(snt))/(sb*power2(mH2)) + (36*cb*(-mh2 - mst12 - mst22 - 4*mh2*
     mst14*DeltaInv(mst12,mst12,mh2) - 4*mh2*mst24*DeltaInv(mst22,mst22,mh2) +
     (mh2 - mst12 - mst22)*DeltaInv(mst12,mst22,mh2)*power2(mh2))*power2(log(
     mh2/Q2))*power4(sa)*power4(snt))/(sb*power2(mh2)) + (36*cb*(-mH2 - mst12 -
     mst22 - 4*mH2*mst24*DeltaInv(mH2,mst22,mst22) - 4*mH2*mst14*DeltaInv(
     mst12,mst12,mH2) + (mH2 - mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(
     mH2))*power2(log(mH2/Q2))*power4(sa)*power4(snt))/(sb*power2(mH2)) - (12*
     cb*power2(sa)*power2(snt)*power2(log(mst12/Q2))*(3/mh2 + 3/mH2 + (12*
     mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(
     mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*
     DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 -
     (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (8*
     mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2)) -
     (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))))/sb - 12*ca*
     power2(snt)*power2(log(mst12/Q2))*power3(sa)*(3/mh2 + 3/mH2 + (12*mst12)/(
     mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,
     mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,
     mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - (5*mst12)/
     power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (8*mst14)/((mh2 -
     mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2)) - (3*mst12)/
     power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 +
     2*mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,mst12,mh2)*(mh2*
     mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))) + (12*cb*power2(snt)*power2(
     log(mst12/Q2))*power4(sa)*(3/mh2 + 3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*
     DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2
     - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - (5*mst12)/power2(mh2) + (2*
     mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (8*mst14)/((mh2 - mH2)*power2(mh2)
     ) + (2*mst12)/(-(mh2*mH2) + power2(mh2)) - (3*mst12)/power2(mH2) - (3*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 - (8*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(
     mst1)))/(mh2*(mh2 - mH2))))/sb + (12*cb*power2(sa)*power2(log(mst12/Q2))*
     power4(snt)*(3/mh2 + 3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,
     mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,
     mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 - (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-
     mh2 + mH2)*power2(mh2)) + (8*mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/
     (-(mh2*mH2) + power2(mh2)) - (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,
     mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))
     /mH2 - (8*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(
     mh2 - mH2))))/sb + 12*ca*power2(log(mst12/Q2))*power3(sa)*power4(snt)*(3/
     mh2 + 3/mH2 + (12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*
     mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*
     mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,
     mst22,mh2))/mh2 - (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*
     power2(mh2)) + (8*mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2)
     + power2(mh2)) - (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*
     mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (8*
     DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2)))
     - (12*cb*power2(log(mst12/Q2))*power4(sa)*power4(snt)*(3/mh2 + 3/mH2 + (
     12*mst12)/(mh2*mH2) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(
     mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*
     DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 -
     (5*mst12)/power2(mh2) + (2*mH2*mst12)/((-mh2 + mH2)*power2(mh2)) + (8*
     mst14)/((mh2 - mH2)*power2(mh2)) + (2*mst12)/(-(mh2*mH2) + power2(mh2)) -
     (3*mst12)/power2(mH2) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (8*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2))))/sb + (72*ca*
     sa*log(mst12/Q2)*power2(snt)*(3*mh2*mH2*mst12 - 2*mH2*mst14 + 3*mH2*
     power2(mh2) - 3*mst12*power2(mh2) + 6*mH2*mst14*DeltaInv(mst12,mst12,mH2)*
     power2(mh2) + 3*(mh2 - mH2)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*
     mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)) - 3*power3(mh2)
     - 6*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 2*mh2*mH2*DeltaInv(
     mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1))))/((mh2 - mH2)*mH2*power2(
     mh2)) + (72*ca*sa*log(mst12/Q2)*power4(snt)*(3*mh2*mH2*mst12 - 2*mH2*mst14
     + 3*mH2*power2(mh2) - 3*mst12*power2(mh2) + 6*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power2(mh2) + 3*(mh2 - mH2)*DeltaInv(mst12,mH2,mst22)*power2(
     mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)) - 3*
     power3(mh2) - 6*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 2*mh2*mH2*
     DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1))))/(mH2*(-mh2 + mH2)
     *power2(mh2)) - (18*ca*sa*log(mst12/Q2)*(12*mh2*mH2*mst12*mt2 - 8*mH2*
     mst14*mt2 - 3*mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) - 12*mst12*
     mt2*power2(mh2) + 24*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2) -
     3*(mh2 - mH2)*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*
     mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)) + 3*mst12*
     power3(mh2) - 3*mst22*power3(mh2) - 24*mst14*mt2*DeltaInv(mst12,mst12,mH2)
     *power3(mh2) + 8*mh2*mH2*mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*
     power6(mst1))))/(mH2*(-mh2 + mH2)*(mst12 - mst22)*power2(mh2)) + 3*ca*
     power2(log(mst12/Q2))*power3(sa)*(3/mh2 + 3/mH2 - (8*mst12*mt2)/(mh2*(mh2
     - mH2)*(mst12 - mst22)) - (48*mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) -
     3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*
     mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2)
     )/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 + (20*mst12*mt2)/((mst12 -
     mst22)*power2(mh2)) + (8*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*
     power2(mh2)) - (32*mst14*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) +
     (12*mst12*mt2)/((mst12 - mst22)*power2(mH2)) - (3*DeltaInv(mst12,mH2,
     mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2
     + (32*mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(
     mh2 - mH2)*(mst12 - mst22))) + (3*cb*power2(sa)*power2(log(mst12/Q2))*(3/
     mh2 + 3/mH2 - (16*mst12*mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)) - (96*
     mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) - 3*mh2*DeltaInv(mst12,mst22,
     mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(
     mst12,mst22,mh2))/mh2 + (40*mst12*mt2)/((mst12 - mst22)*power2(mh2)) + (
     16*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) - (64*mst14*
     mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + (24*mst12*mt2)/((mst12 -
     mst22)*power2(mH2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (64*mt2*DeltaInv(mst12,
     mst12,mh2)*(mh2*mst14 - 4*power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))
     ))/sb - (3*cb*power2(log(mst12/Q2))*power4(sa)*(3/mh2 + 3/mH2 - (16*mst12*
     mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)) - (96*mst12*mt2)/(mh2*mH2*mst12 -
     mh2*mH2*mst22) - 3*mh2*DeltaInv(mst12,mst22,mh2) + 3*mst12*DeltaInv(mst12,
     mst22,mh2) + 6*mst22*DeltaInv(mst12,mst22,mh2) + (9*mst12*mst22*DeltaInv(
     mst12,mst22,mh2))/mh2 - (3*mst24*DeltaInv(mst12,mst22,mh2))/mh2 + (40*
     mst12*mt2)/((mst12 - mst22)*power2(mh2)) + (16*mH2*mst12*mt2)/((mh2 - mH2)
     *(mst12 - mst22)*power2(mh2)) - (64*mst14*mt2)/((mh2 - mH2)*(mst12 -
     mst22)*power2(mh2)) + (24*mst12*mt2)/((mst12 - mst22)*power2(mH2)) - (3*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 + (64*mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 4*
     power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))))/sb + (12*ca*sa*power2(
     snt)*power2(log(mst12/Q2))*(-9*mH2*mst12*power2(mh2) + 6*mh2*mst12*power2(
     mH2) - 4*mst14*power2(mH2) + 3*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*
     mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 +
     2*mst22) + mst24 + power2(mH2)) - 3*mH2*power3(mh2) + 3*mst12*power3(mh2)
     + 4*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*(mh2*mst14 - 4*power6(mst1))
     ))/((-mh2 + mH2)*power2(mh2)*power2(mH2)) + (12*ca*sa*power2(log(mst12/Q2)
     )*power4(snt)*(-9*mH2*mst12*power2(mh2) + 6*mh2*mst12*power2(mH2) - 4*
     mst14*power2(mH2) + 3*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*
     mst22) + mst24 + power2(mH2)) - 3*mH2*power3(mh2) + 3*mst12*power3(mh2) +
     4*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*(mh2*mst14 - 4*power6(mst1))))
     /((mh2 - mH2)*power2(mh2)*power2(mH2)) + (3*ca*sa*power2(log(mst12/Q2))*(-
     36*mH2*mst12*mt2*power2(mh2) + 24*mh2*mst12*mt2*power2(mH2) - 16*mst14*
     mt2*power2(mH2) - 3*mst12*power2(mh2)*power2(mH2) + 3*mst22*power2(mh2)*
     power2(mH2) - 3*(mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24 + power2(mH2))
     + 3*mH2*mst12*power3(mh2) - 3*mH2*mst22*power3(mh2) + 12*mst12*mt2*power3(
     mh2) + 16*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*(mh2*mst14 - 4*
     power6(mst1))))/((-mh2 + mH2)*(mst12 - mst22)*power2(mh2)*power2(mH2)) + (
     72*cb*log(mst12/Q2)*power2(sa)*power2(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(
     mh2*mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,
     mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,
     mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*
     mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,
     mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))/sb +
     72*ca*log(mst12/Q2)*power2(snt)*power3(sa)*(3/mh2 + 3/mH2 + (6*mst12)/(
     mh2*mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,
     mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,
     mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*
     mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,
     mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))) - (72*
     cb*log(mst12/Q2)*power2(snt)*power4(sa)*(3/mh2 + 3/mH2 + (6*mst12)/(mh2*
     mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,
     mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,
     mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*
     mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,
     mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))/sb -
     (72*cb*log(mst12/Q2)*power2(sa)*power4(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(
     mh2*mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,
     mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,
     mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*
     mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,
     mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))/sb -
     72*ca*log(mst12/Q2)*power3(sa)*power4(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(
     mh2*mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,
     mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,
     mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*
     mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,
     mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))) + (72*
     cb*log(mst12/Q2)*power4(sa)*power4(snt)*(3/mh2 + 3/mH2 + (6*mst12)/(mh2*
     mH2) + (6*mst14*DeltaInv(mst12,mst12,mH2))/mH2 - 3*mh2*DeltaInv(mst12,
     mst22,mh2) + 3*mst12*DeltaInv(mst12,mst22,mh2) + 6*mst22*DeltaInv(mst12,
     mst22,mh2) + (9*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (3*mst24*
     DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 +
     mH2)*power2(mh2)) + (4*mst14)/((mh2 - mH2)*power2(mh2)) + mst12/(-(mh2*
     mH2) + power2(mh2)) - (3*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(
     mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 + (2*DeltaInv(mst12,mst12,
     mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2))))/sb -
     (6*cb*log(mst12/Q2)*power2(sa)*(9/mh2 + 9/mH2 - (16*mst12*mt2)/(mh2*(mh2 -
     mH2)*(mst12 - mst22)) - (144*mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) -
     (144*mst14*mt2*DeltaInv(mst12,mst12,mH2))/(mH2*mst12 - mH2*mst22) - 9*mh2*
     DeltaInv(mst12,mst22,mh2) + 9*mst12*DeltaInv(mst12,mst22,mh2) + 18*mst22*
     DeltaInv(mst12,mst22,mh2) + (27*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2
     - (9*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*
     mst12)/((-mh2 + mH2)*power2(mh2)) + (16*mst12*mt2)/((mst12 - mst22)*
     power2(mh2)) + (16*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)
     ) - (96*mst14*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + mst12/(-(
     mh2*mH2) + power2(mh2)) - (9*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 -
     mH2*(mst12 + 2*mst22) + mst24 + power2(mH2)))/mH2 - (48*mt2*DeltaInv(
     mst12,mst12,mh2)*(mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 -
     mH2)*(mst12 - mst22))))/sb + (6*cb*log(mst12/Q2)*power4(sa)*(9/mh2 + 9/mH2
     - (16*mst12*mt2)/(mh2*(mh2 - mH2)*(mst12 - mst22)) - (144*mst12*mt2)/(mh2*
     mH2*mst12 - mh2*mH2*mst22) - (144*mst14*mt2*DeltaInv(mst12,mst12,mH2))/(
     mH2*mst12 - mH2*mst22) - 9*mh2*DeltaInv(mst12,mst22,mh2) + 9*mst12*
     DeltaInv(mst12,mst22,mh2) + 18*mst22*DeltaInv(mst12,mst22,mh2) + (27*
     mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2 - (9*mst24*DeltaInv(mst12,
     mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*mst12)/((-mh2 + mH2)*power2(
     mh2)) + (16*mst12*mt2)/((mst12 - mst22)*power2(mh2)) + (16*mH2*mst12*mt2)/
     ((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) - (96*mst14*mt2)/((mh2 - mH2)*(
     mst12 - mst22)*power2(mh2)) + mst12/(-(mh2*mH2) + power2(mh2)) - (9*
     DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 + 2*mst22) + mst24
     + power2(mH2)))/mH2 - (48*mt2*DeltaInv(mst12,mst12,mh2)*(mh2*mst14 - 3*
     mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 - mst22))))/sb - 6*
     ca*log(mst12/Q2)*power3(sa)*(9/mh2 + 9/mH2 - (8*mst12*mt2)/(mh2*(mh2 -
     mH2)*(mst12 - mst22)) - (72*mst12*mt2)/(mh2*mH2*mst12 - mh2*mH2*mst22) - (
     72*mst14*mt2*DeltaInv(mst12,mst12,mH2))/(mH2*mst12 - mH2*mst22) - 9*mh2*
     DeltaInv(mst12,mst22,mh2) + 9*mst12*DeltaInv(mst12,mst22,mh2) + 18*mst22*
     DeltaInv(mst12,mst22,mh2) + (27*mst12*mst22*DeltaInv(mst12,mst22,mh2))/mh2
     - (9*mst24*DeltaInv(mst12,mst22,mh2))/mh2 - mst12/power2(mh2) + (mH2*
     mst12)/((-mh2 + mH2)*power2(mh2)) + (8*mst12*mt2)/((mst12 - mst22)*power2(
     mh2)) + (8*mH2*mst12*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) - (48*
     mst14*mt2)/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)) + mst12/(-(mh2*mH2) +
     power2(mh2)) - (9*DeltaInv(mst12,mH2,mst22)*(-3*mst12*mst22 - mH2*(mst12 +
     2*mst22) + mst24 + power2(mH2)))/mH2 - (24*mt2*DeltaInv(mst12,mst12,mh2)*(
     mh2*mst14 - 3*mH2*mst14 + 8*power6(mst1)))/(mh2*(mh2 - mH2)*(mst12 -
     mst22))) + (18*ca*sa*log(mst22/Q2)*(24*(mh2 - mH2)*mst24*mt2*DeltaInv(mH2,
     mst22,mst22)*power2(mh2) - 3*(mh2 - mH2)*(mst12 - mst22)*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 4*mt2*(-3*
     mh2*mH2*mst22 + 2*mH2*mst24 + 3*mst22*power2(mh2) - 2*mh2*mH2*DeltaInv(
     mst22,mst22,mh2)*(mh2*mst24 - 4*power6(mst2)))))/((mh2 - mH2)*mH2*(mst12 -
     mst22)*power2(mh2)) + (3*ca*sa*power2(log(mst22/Q2))*(-3*(mh2 - mH2)*mH2*(
     mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)
     *power2(mh2) - 4*mt2*(-9*mH2*mst22*power2(mh2) + 6*mh2*mst22*power2(mH2) -
     4*mst24*power2(mH2) + 3*mst22*power3(mh2) + 4*mh2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/((mh2 - mH2)*(-mst12 +
     mst22)*power2(mh2)*power2(mH2)) - (3*cb*power2(sa)*power2(log(mst22/Q2))*(
     -3*(mh2 - mH2)*mH2*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mh2*(mh2 - mH2)*(mst12 - mst22)*
     (mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) -
     8*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*power2(mH2) - (3*mH2*mst22
     + 8*mst24)*power2(mH2) + 3*mst22*power3(mh2) + 8*mh2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/((mh2 - mH2)*(-mst12 +
     mst22)*sb*power2(mh2)*power2(mH2)) + (3*cb*power2(log(mst22/Q2))*power4(
     sa)*(-3*(mh2 - mH2)*mH2*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mh2*(mh2 - mH2)*(mst12 - mst22)*
     (mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) -
     8*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*power2(mH2) - (3*mH2*mst22
     + 8*mst24)*power2(mH2) + 3*mst22*power3(mh2) + 8*mh2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/((mh2 - mH2)*(-mst12 +
     mst22)*sb*power2(mh2)*power2(mH2)) - (3*ca*power2(log(mst22/Q2))*power3(
     sa)*(-3*(mh2 - mH2)*mH2*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mh2*(mh2 - mH2)*(mst12 - mst22)*
     (mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) -
     4*mt2*(-15*mH2*mst22*power2(mh2) + 15*mh2*mst22*power2(mH2) - (3*mH2*mst22
     + 8*mst24)*power2(mH2) + 3*mst22*power3(mh2) + 8*mh2*DeltaInv(mst22,mst22,
     mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2)))))/((mh2 - mH2)*(-mst12 +
     mst22)*power2(mh2)*power2(mH2)) - (12*ca*sa*power2(snt)*power2(log(mst22/
     Q2))*(-9*mH2*mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 6*mh2*mst22*power2(
     mH2) - 4*mst24*power2(mH2) + 3*mst22*power3(mh2) + 4*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*(mh2*mst24 - 4*power6(mst2))))/((mh2 - mH2)*power2(
     mh2)*power2(mH2)) + (12*ca*sa*power2(log(mst22/Q2))*power4(snt)*(-9*mH2*
     mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) + 6*mh2*mst22*power2(mH2) - 4*mst24*
     power2(mH2) + 3*mst22*power3(mh2) + 4*mh2*DeltaInv(mst22,mst22,mh2)*
     power2(mH2)*(mh2*mst24 - 4*power6(mst2))))/((mh2 - mH2)*power2(mh2)*
     power2(mH2)) - (72*cb*log(mst22/Q2)*power2(sa)*power2(snt)*(6*mh2*mH2*
     mst22 - 4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 -
     mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*
     power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*
     mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(
     mst12,mst22,mh2)*power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*
     power2(mH2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) +
     3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(
     mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*sb*power2(mh2)) - (72*ca*
     log(mst22/Q2)*power2(snt)*power3(sa)*(6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*
     mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,
     mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2)
     + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,
     mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)
     ))/((mh2 - mH2)*mH2*power2(mh2)) + (72*cb*log(mst22/Q2)*power2(snt)*
     power4(sa)*(6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2
     + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*
     mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*
     mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)
     + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(
     mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*
     mH2*sb*power2(mh2)) + (72*cb*log(mst22/Q2)*power2(sa)*power4(snt)*(6*mh2*
     mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,
     mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*
     mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,
     mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*sb*power2(mh2))
     + (72*ca*log(mst22/Q2)*power3(sa)*power4(snt)*(6*mh2*mH2*mst22 - 4*mH2*
     mst24 - 6*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,
     mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2) - 3*mH2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2)
     - 2*mH2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,
     mh2)*power2(mH2) + 6*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*
     DeltaInv(mst12,mst22,mh2)*power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,
     mh2)*power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)) - (72*cb*log(mst22/Q2)*
     power4(sa)*power4(snt)*(6*mh2*mH2*mst22 - 4*mH2*mst24 - 6*mst22*power2(
     mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2
     - mH2)*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(
     mh2) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*
     mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 2*mH2*mst24*DeltaInv(mst22,
     mst22,mh2)*power2(mh2) + 3*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mH2) + 3*mh2*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 6*mh2*
     mst24*DeltaInv(mst22,mst22,mh2)*power2(mH2) - 3*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)
     *power3(mh2) - 16*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 -
     mH2)*mH2*sb*power2(mh2)) + (72*ca*sa*log(mst22/Q2)*power2(snt)*(3*mh2*mH2*
     mst22 - 2*mH2*mst24 - 3*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*mst22 -
     mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 2*mH2*mst24*DeltaInv(mst22,
     mst22,mh2)*power2(mh2) - 8*mh2*mH2*DeltaInv(mst22,mst22,mh2)*power6(mst2))
     )/((mh2 - mH2)*mH2*power2(mh2)) - (72*ca*sa*log(mst22/Q2)*power4(snt)*(3*
     mh2*mH2*mst22 - 2*mH2*mst24 - 3*mst22*power2(mh2) + 6*(-mh2 + mH2)*mst24*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*(mh2 - mH2)*(mH2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 2*mH2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2) - 8*mh2*mH2*DeltaInv(mst22,mst22,
     mh2)*power6(mst2)))/((mh2 - mH2)*mH2*power2(mh2)) + (18*ca*log(mst22/Q2)*
     power3(sa)*(-24*mh2*mH2*mst22*mt2 + 16*mH2*mst24*mt2 + 24*mst22*mt2*
     power2(mh2) + 24*(mh2 - mH2)*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) - 3*(mh2 - mH2)*(mst12 - mst22)*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) + 3*mH2*mst12*mst24*DeltaInv(mst12,
     mst22,mh2)*power2(mh2) - 3*mH2*mst22*mst24*DeltaInv(mst12,mst22,mh2)*
     power2(mh2) + 8*mH2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2) - 3*
     mh2*mst12*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*mh2*mst22*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) - 24*mh2*mst24*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2) + 3*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst12) - 3*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(
     mst12) - 3*mH2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22) +
     3*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22) - 3*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*
     mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) + 3*mH2*DeltaInv(mst12,
     mst22,mh2)*power2(mst22)*power3(mh2) + 64*mh2*mH2*mt2*DeltaInv(mst22,
     mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*(-mst12 + mst22)*power2(mh2)) +
     (18*cb*log(mst22/Q2)*power2(sa)*(-48*mh2*mH2*mst22*mt2 + 32*mH2*mst24*mt2
     + 48*mst22*mt2*power2(mh2) + 48*(mh2 - mH2)*mst24*mt2*DeltaInv(mH2,mst22,
     mst22)*power2(mh2) - 3*(mh2 - mH2)*(mst12 - mst22)*(mH2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 3*mH2*mst12*mst24*
     DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst22*mst24*DeltaInv(mst12,
     mst22,mh2)*power2(mh2) + 16*mH2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*
     power2(mh2) - 3*mh2*mst12*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 3*
     mh2*mst22*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) - 48*mh2*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mH2) + 3*mst12*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(mst12,mst22,mh2)
     *power2(mh2)*power2(mst12) - 3*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power2(mst12) - 3*mH2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst22) + 3*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(
     mst22) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*power2(mst22)
     - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2) + 3*mH2*
     DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mh2) + 128*mh2*mH2*mt2*
     DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*(-mst12 + mst22)
     *sb*power2(mh2)) + (18*cb*log(mst22/Q2)*power4(sa)*(-48*mh2*mH2*mst22*mt2
     + 32*mH2*mst24*mt2 + 48*mst22*mt2*power2(mh2) + 48*(mh2 - mH2)*mst24*mt2*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*(mh2 - mH2)*(mst12 - mst22)*(
     mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) +
     3*mH2*mst12*mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) - 3*mH2*mst22*
     mst24*DeltaInv(mst12,mst22,mh2)*power2(mh2) + 16*mH2*mst24*mt2*DeltaInv(
     mst22,mst22,mh2)*power2(mh2) - 3*mh2*mst12*mst24*DeltaInv(mst12,mst22,mh2)
     *power2(mH2) + 3*mh2*mst22*mst24*DeltaInv(mst12,mst22,mh2)*power2(mH2) -
     48*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2) + 3*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mH2*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mst12) - 3*mh2*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mH2)*power2(mst12) - 3*mH2*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power2(mst22) + 3*mh2*mst12*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst22) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2)*power2(mst22) - 3*mH2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mh2) + 3*mH2*DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mh2) +
     128*mh2*mH2*mt2*DeltaInv(mst22,mst22,mh2)*power6(mst2)))/((mh2 - mH2)*mH2*
     (mst12 - mst22)*sb*power2(mh2)) + (12*cb*power2(sa)*power2(snt)*power2(
     log(mst22/Q2))*(-15*mH2*mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 -
     mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 15*mh2*mst22*
     power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2 - mH2)*(mh2*mst22 - mst12*
     mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 8*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mst22*power3(mh2) - 3*mst22*
     power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/
     ((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) + (12*ca*power2(snt)*power2(log(
     mst22/Q2))*power3(sa)*(-15*mH2*mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*
     mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) + 15*
     mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2 - mH2)*(mh2*mst22
     - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 8*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mst22*power3(mh2) -
     3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(
     mst2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)) - (12*cb*power2(snt)*power2(
     log(mst22/Q2))*power4(sa)*(-15*mH2*mst22*power2(mh2) + 3*(mh2 - mH2)*mH2*(
     mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*power2(mh2) +
     15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2 - mH2)*(mh2*
     mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(mH2) + 8*
     mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*mst22*power3(
     mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*
     power6(mst2)))/((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) - (12*cb*power2(
     sa)*power2(log(mst22/Q2))*power4(snt)*(-15*mH2*mst22*power2(mh2) + 3*(mh2
     - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mH2,mst22)*
     power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) + 3*mh2*(mh2
     - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,mh2)*power2(
     mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*
     mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,mst22,mh2)
     *power2(mH2)*power6(mst2)))/((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) - (
     12*ca*power2(log(mst22/Q2))*power3(sa)*power4(snt)*(-15*mH2*mst22*power2(
     mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,
     mH2,mst22)*power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*mst24*power2(mH2) +
     3*mh2*(mh2 - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*DeltaInv(mst12,mst22,
     mh2)*power2(mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)
     ) + (12*cb*power2(log(mst22/Q2))*power4(sa)*power4(snt)*(-15*mH2*mst22*
     power2(mh2) + 3*(mh2 - mH2)*mH2*(mH2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2) + 15*mh2*mst22*power2(mH2) - 8*
     mst24*power2(mH2) + 3*mh2*(mh2 - mH2)*(mh2*mst22 - mst12*mst22 - mst24)*
     DeltaInv(mst12,mst22,mh2)*power2(mH2) + 8*mst24*DeltaInv(mst22,mst22,mh2)*
     power2(mh2)*power2(mH2) + 3*mst22*power3(mh2) - 3*mst22*power3(mH2) - 32*
     mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*sb*
     power2(mh2)*power2(mH2)) + (84*cb*power2(sa)*power2(snt)*(-3*mH2*mst12*
     power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(
     mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) +
     6*mh2*power3(mH2) + 3*mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*
     mst14*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*
     power3(mH2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) +
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))
     /((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) + (2*cb*power2(Pi)*power2(sa)*
     power2(snt)*(-3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2
     - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*
     power2(mH2) - 4*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*
     power2(mH2) - 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) +
     18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*
     mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*
     power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*
     mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2
     - mH2)*sb*power2(mh2)*power2(mH2)) + (84*ca*power2(snt)*power3(sa)*(-3*
     mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*
     mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*
     mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*
     mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*power3(
     mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*mst12*
     power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) +
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*
     power2(mH2)) + (2*ca*power2(Pi)*power2(snt)*power3(sa)*(-3*mH2*mst12*
     power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(
     mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) +
     6*mh2*power3(mH2) + 3*mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*
     mst14*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*
     power3(mH2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) +
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))
     /((mh2 - mH2)*power2(mh2)*power2(mH2)) + (84*cb*power4(sa)*power4(snt)*(-
     3*mH2*mst12*power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*
     mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*
     mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*
     mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*power3(
     mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mh2*power3(mH2) + 3*mst12*
     power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) +
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*sb*power2(mh2)*
     power2(mH2)) + (2*cb*power2(Pi)*power4(sa)*power4(snt)*(-3*mH2*mst12*
     power2(mh2) - 3*mH2*mst22*power2(mh2) - 18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 4*mst14*power2(
     mH2) + 3*mh2*mst22*power2(mH2) - 4*mst24*power2(mH2) - 14*mst14*DeltaInv(
     mst12,mst12,mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) - 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2) - 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*
     power2(mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*
     mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*
     mst12*power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,
     mst12,mH2)*power3(mh2) - 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*
     power3(mh2) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) +
     6*mh2*power3(mH2) + 3*mst12*power3(mH2) + 3*mst22*power3(mH2) + 18*mh2*
     mst14*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 12*mh2*mst12*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mH2) + 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*
     power3(mH2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) +
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power4(mh2) - 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))
     /((mh2 - mH2)*sb*power2(mh2)*power2(mH2)) - (84*ca*sa*power2(snt)*(-18*(
     mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*
     power2(mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 2*mst24*
     power2(mH2) + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(mst12,mst12,
     mh2)*power2(mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(
     mh2)*power2(mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(
     mH2) + 6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*
     mst22 - mH2*(mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*
     power3(mh2) - 3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)
     *power3(mh2) - 8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) -
     8*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*
     power2(mh2)*power2(mH2)) - (2*ca*sa*power2(Pi)*power2(snt)*(-18*(mh2 -
     mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(
     mH2) - 2*mst14*power2(mH2) + 3*mh2*mst22*power2(mH2) - 2*mst24*power2(mH2)
     + 6*power2(mh2)*power2(mH2) + 2*mst14*DeltaInv(mst12,mst12,mh2)*power2(
     mh2)*power2(mH2) + 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(
     mH2) + 2*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2
     - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(
     mst12 + mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) -
     3*mst22*power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) -
     8*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 8*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*
     power2(mH2)) + (84*ca*sa*power4(snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 2*mst14*power2(
     mH2) + 3*mh2*mst22*power2(mH2) - 2*mst24*power2(mH2) + 6*power2(mh2)*
     power2(mH2) + 2*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) +
     18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*
     power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 8*mh2*
     DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 8*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)
     ) + (2*ca*sa*power2(Pi)*power4(snt)*(-18*(mh2 - mH2)*mH2*mst24*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) + 3*mh2*mst12*power2(mH2) - 2*mst14*power2(
     mH2) + 3*mh2*mst22*power2(mH2) - 2*mst24*power2(mH2) + 6*power2(mh2)*
     power2(mH2) + 2*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) +
     18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 2*mst24*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 6*(mh2 - mH2)*mH2*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mH2*power3(mh2) - 3*mst12*power3(mh2) - 3*mst22*
     power3(mh2) - 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 8*mh2*
     DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 8*mh2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*power2(mH2)
     ) + (84*cb*power2(snt)*power4(sa)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*
     power2(mh2) + 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(
     mh2) - 3*mh2*mst12*power2(mH2) + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(
     mH2) + 4*mst24*power2(mH2) + 14*mst14*DeltaInv(mst12,mst12,mh2)*power2(
     mh2)*power2(mH2) - 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(
     mH2) + 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) +
     14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)
     *mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*
     power3(mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*
     mst12*power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,
     mst12,mh2)*power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*
     power3(mH2) - 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2
     - mH2)*sb*power2(mh2)*power2(mH2)) + (2*cb*power2(Pi)*power2(snt)*power4(
     sa)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*
     mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2)
     + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) +
     14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(
     mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*
     power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) -
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*sb*power2(mh2)*
     power2(mH2)) + (84*cb*power2(sa)*power4(snt)*(3*mH2*mst12*power2(mh2) + 3*
     mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)
     *power2(mh2) - 3*mh2*mst12*power2(mH2) + 4*mst14*power2(mH2) - 3*mh2*
     mst22*power2(mH2) + 4*mst24*power2(mH2) + 14*mst14*DeltaInv(mst12,mst12,
     mh2)*power2(mh2)*power2(mH2) - 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(
     mh2)*power2(mH2) + 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2) + 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) -
     6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 -
     mH2*(mst12 + mst22) + power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(
     mh2) + 3*mst22*power3(mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*
     power3(mh2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) +
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(
     mH2) - 3*mst12*power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(
     mst12,mst12,mh2)*power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,
     mh2)*power3(mH2) - 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2
     - mH2)*sb*power2(mh2)*power2(mH2)) + (2*cb*power2(Pi)*power2(sa)*power4(
     snt)*(3*mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*
     mH2*mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2)
     + 4*mst14*power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) +
     14*mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(
     mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*
     power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) -
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*sb*power2(mh2)*
     power2(mH2)) + (84*ca*power3(sa)*power4(snt)*(3*mH2*mst12*power2(mh2) + 3*
     mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*mH2*mst24*DeltaInv(mH2,mst22,mst22)
     *power2(mh2) - 3*mh2*mst12*power2(mH2) + 4*mst14*power2(mH2) - 3*mh2*
     mst22*power2(mH2) + 4*mst24*power2(mH2) + 14*mst14*DeltaInv(mst12,mst12,
     mh2)*power2(mh2)*power2(mH2) - 18*mst14*DeltaInv(mst12,mst12,mH2)*power2(
     mh2)*power2(mH2) + 12*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2) + 14*mst24*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) -
     6*(mh2 - mH2)*mH2*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 -
     mH2*(mst12 + mst22) + power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(
     mh2) + 3*mst22*power3(mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*
     power3(mh2) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) +
     6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(
     mH2) - 3*mst12*power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(
     mst12,mst12,mh2)*power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,
     mh2)*power3(mH2) - 18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,
     mst22,mh2)*power3(mh2)*power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 16*mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(
     mst1) + 16*mh2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2
     - mH2)*power2(mh2)*power2(mH2)) + (2*ca*power2(Pi)*power3(sa)*power4(snt)*
     (3*mH2*mst12*power2(mh2) + 3*mH2*mst22*power2(mh2) + 18*(mh2 - mH2)*mH2*
     mst24*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 3*mh2*mst12*power2(mH2) + 4*
     mst14*power2(mH2) - 3*mh2*mst22*power2(mH2) + 4*mst24*power2(mH2) + 14*
     mst14*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 18*mst14*
     DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 12*mst12*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2) + 14*mst24*DeltaInv(
     mst22,mst22,mh2)*power2(mh2)*power2(mH2) - 6*(mh2 - mH2)*mH2*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) + 6*mH2*power3(mh2) + 3*mst12*power3(mh2) + 3*mst22*power3(
     mh2) + 18*mH2*mst14*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 6*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power3(mh2) + 6*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power3(mh2) - 6*mh2*power3(mH2) - 3*mst12*
     power3(mH2) - 3*mst22*power3(mH2) - 18*mh2*mst14*DeltaInv(mst12,mst12,mh2)
     *power3(mH2) - 12*mh2*mst12*mst22*DeltaInv(mst12,mst22,mh2)*power3(mH2) -
     18*mh2*mst24*DeltaInv(mst22,mst22,mh2)*power3(mH2) - 6*mst12*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power3(mH2) - 6*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mh2)*power3(mH2) + 6*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) - 6*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 16*
     mh2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) + 16*mh2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*power2(mh2)*
     power2(mH2)) - (6*cb*power2(sa)*(84*mH2*mst12*mt2*power2(mh2) - 84*mH2*
     mst22*mt2*power2(mh2) - 504*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,
     mst22)*power2(mh2) - 84*mh2*mst12*mt2*power2(mH2) + 112*mst14*mt2*power2(
     mH2) + 84*mh2*mst22*mt2*power2(mH2) - 112*mst24*mt2*power2(mH2) + 392*
     mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 504*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 392*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 21*(mh2 - mH2)*mH2*(
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 -
     mH2*(mst12 + mst22) + power2(mH2)) - 42*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst12) + 42*mst12*DeltaInv(mst12,mst22,mh2)
     *power2(mh2)*power2(mH2)*power2(mst22) - 15*mH2*mst12*power3(mh2) + 15*
     mH2*mst22*power3(mh2) + 84*mst12*mt2*power3(mh2) - 84*mst22*mt2*power3(
     mh2) + 504*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 21*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) + 21*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 15*mh2*
     mst12*power3(mH2) - 15*mh2*mst22*power3(mH2) - 84*mst12*mt2*power3(mH2) +
     84*mst22*mt2*power3(mH2) - 504*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*
     power3(mH2) + 504*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) +
     42*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mst12)*power3(mH2) + 21*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst12)*power3(mH2) - 42*mh2*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mH2) - 21*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mst22)*power3(mH2) - 21*mst12*
     DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power4(mh2) - 21*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 448*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 448*mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(
     mst2)))/((mh2 - mH2)*(mst12 - mst22)*sb*power2(mh2)*power2(mH2)) + (6*cb*
     power4(sa)*(84*mH2*mst12*mt2*power2(mh2) - 84*mH2*mst22*mt2*power2(mh2) -
     504*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 84*
     mh2*mst12*mt2*power2(mH2) + 112*mst14*mt2*power2(mH2) + 84*mh2*mst22*mt2*
     power2(mH2) - 112*mst24*mt2*power2(mH2) + 392*mst14*mt2*DeltaInv(mst12,
     mst12,mh2)*power2(mh2)*power2(mH2) - 504*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power2(mh2)*power2(mH2) - 392*mst24*mt2*DeltaInv(mst22,mst22,mh2)*
     power2(mh2)*power2(mH2) + 21*(mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(
     mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) +
     power2(mH2)) - 42*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)*
     power2(mst12) + 42*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mH2)
     *power2(mst22) - 15*mH2*mst12*power3(mh2) + 15*mH2*mst22*power3(mh2) + 84*
     mst12*mt2*power3(mh2) - 84*mst22*mt2*power3(mh2) + 504*mH2*mst14*mt2*
     DeltaInv(mst12,mst12,mH2)*power3(mh2) - 21*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst12)*power3(mh2) + 21*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst22)*power3(mh2) + 15*mh2*mst12*power3(mH2) - 15*mh2*
     mst22*power3(mH2) - 84*mst12*mt2*power3(mH2) + 84*mst22*mt2*power3(mH2) -
     504*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 504*mh2*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 42*mh2*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mst12)*power3(mH2) + 21*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mst12)*power3(mH2) - 42*mh2*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mst22)*power3(mH2) - 21*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst22)*power3(mH2) - 21*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)
     *power3(mH2) + 21*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2)
     + 21*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 21*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 448*mh2*mt2*DeltaInv(
     mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 448*mh2*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*(mst12 - mst22)*sb*
     power2(mh2)*power2(mH2)) - (6*ca*power3(sa)*(42*mH2*mst12*mt2*power2(mh2)
     - 42*mH2*mst22*mt2*power2(mh2) - 252*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(
     mH2,mst22,mst22)*power2(mh2) - 42*mh2*mst12*mt2*power2(mH2) + 56*mst14*
     mt2*power2(mH2) + 42*mh2*mst22*mt2*power2(mH2) - 56*mst24*mt2*power2(mH2)
     + 196*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 252*
     mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 196*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 21*(mh2 - mH2)*
     mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22
     - mH2*(mst12 + mst22) + power2(mH2)) - 42*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst12) + 42*mst12*DeltaInv(mst12,mst22,mh2)
     *power2(mh2)*power2(mH2)*power2(mst22) - 15*mH2*mst12*power3(mh2) + 15*
     mH2*mst22*power3(mh2) + 42*mst12*mt2*power3(mh2) - 42*mst22*mt2*power3(
     mh2) + 252*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 21*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) + 21*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 15*mh2*
     mst12*power3(mH2) - 15*mh2*mst22*power3(mH2) - 42*mst12*mt2*power3(mH2) +
     42*mst22*mt2*power3(mH2) - 252*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*
     power3(mH2) + 252*mh2*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) +
     42*mh2*mst22*DeltaInv(mst12,mst22,mh2)*power2(mst12)*power3(mH2) + 21*
     DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst12)*power3(mH2) - 42*mh2*
     mst12*DeltaInv(mst12,mst22,mh2)*power2(mst22)*power3(mH2) - 21*DeltaInv(
     mst12,mst22,mh2)*power2(mh2)*power2(mst22)*power3(mH2) - 21*mst12*
     DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst22*DeltaInv(
     mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 21*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power4(mh2) - 21*mst22*DeltaInv(mst12,mst22,mh2)*power2(
     mH2)*power4(mh2) + 224*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 224*mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(
     mst2)))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)*power2(mH2)) + (6*ca*sa*(
     -252*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*power2(mh2) - 42*
     mh2*mst12*mt2*power2(mH2) + 28*mst14*mt2*power2(mH2) + 42*mh2*mst22*mt2*
     power2(mH2) - 28*mst24*mt2*power2(mH2) + 15*mst12*power2(mh2)*power2(mH2)
     - 15*mst22*power2(mh2)*power2(mH2) - 28*mst14*mt2*DeltaInv(mst12,mst12,
     mh2)*power2(mh2)*power2(mH2) - 252*mst14*mt2*DeltaInv(mst12,mst12,mH2)*
     power2(mh2)*power2(mH2) + 28*mst24*mt2*DeltaInv(mst22,mst22,mh2)*power2(
     mh2)*power2(mH2) + 21*(mh2 - mH2)*mH2*(mst12 - mst22)*DeltaInv(mst12,mH2,
     mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 + mst22) + power2(mH2)) -
     15*mH2*mst12*power3(mh2) + 15*mH2*mst22*power3(mh2) + 42*mst12*mt2*power3(
     mh2) - 42*mst22*mt2*power3(mh2) + 252*mH2*mst14*mt2*DeltaInv(mst12,mst12,
     mH2)*power3(mh2) + 112*mh2*mt2*DeltaInv(mst12,mst12,mh2)*power2(mH2)*
     power6(mst1) - 112*mh2*mt2*DeltaInv(mst22,mst22,mh2)*power2(mH2)*power6(
     mst2)))/((mh2 - mH2)*(mst12 - mst22)*power2(mh2)*power2(mH2)) - (cb*
     power2(Pi)*power2(sa)*(12*mH2*mst12*mt2*power2(mh2) - 12*mH2*mst22*mt2*
     power2(mh2) - 72*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,mst22,mst22)*
     power2(mh2) - 12*mh2*mst12*mt2*power2(mH2) + 16*mst14*mt2*power2(mH2) +
     12*mh2*mst22*mt2*power2(mH2) - 16*mst24*mt2*power2(mH2) + 56*mst14*mt2*
     DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 72*mst14*mt2*DeltaInv(
     mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 56*mst24*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*(mst12 - mst22)*
     DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 - mH2*(mst12 +
     mst22) + power2(mH2)) - 6*mst22*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2)*power2(mst12) + 6*mst12*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mH2)*power2(mst22) - 3*mH2*mst12*power3(mh2) + 3*mH2*mst22*power3(
     mh2) + 12*mst12*mt2*power3(mh2) - 12*mst22*mt2*power3(mh2) + 72*mH2*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst12)*power3(mh2) + 3*DeltaInv(mst12,mst22,mh2)*
     power2(mH2)*power2(mst22)*power3(mh2) + 3*mh2*mst12*power3(mH2) - 3*mh2*
     mst22*power3(mH2) - 12*mst12*mt2*power3(mH2) + 12*mst22*mt2*power3(mH2) -
     72*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 72*mh2*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mh2*mst22*DeltaInv(mst12,mst22,
     mh2)*power2(mst12)*power3(mH2) + 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst12)*power3(mH2) - 6*mh2*mst12*DeltaInv(mst12,mst22,mh2)*power2(
     mst22)*power3(mH2) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*power2(mst22)
     *power3(mH2) - 3*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) +
     3*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) + 3*mst12*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 3*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 64*mh2*mt2*DeltaInv(mst12,
     mst12,mh2)*power2(mH2)*power6(mst1) - 64*mh2*mt2*DeltaInv(mst22,mst22,mh2)
     *power2(mH2)*power6(mst2)))/((mh2 - mH2)*(mst12 - mst22)*sb*power2(mh2)*
     power2(mH2)) + (cb*power2(Pi)*power4(sa)*(12*mH2*mst12*mt2*power2(mh2) -
     12*mH2*mst22*mt2*power2(mh2) - 72*(mh2 - mH2)*mH2*mst24*mt2*DeltaInv(mH2,
     mst22,mst22)*power2(mh2) - 12*mh2*mst12*mt2*power2(mH2) + 16*mst14*mt2*
     power2(mH2) + 12*mh2*mst22*mt2*power2(mH2) - 16*mst24*mt2*power2(mH2) +
     56*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 72*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 56*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*(
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 -
     mH2*(mst12 + mst22) + power2(mH2)) - 6*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst12) + 6*mst12*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*mst12*power3(mh2) + 3*mH2*
     mst22*power3(mh2) + 12*mst12*mt2*power3(mh2) - 12*mst22*mt2*power3(mh2) +
     72*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*DeltaInv(mst12,
     mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) + 3*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 3*mh2*mst12*power3(mH2) - 3*
     mh2*mst22*power3(mH2) - 12*mst12*mt2*power3(mH2) + 12*mst22*mt2*power3(
     mH2) - 72*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 72*mh2*
     mst24*mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mh2*mst22*DeltaInv(
     mst12,mst22,mh2)*power2(mst12)*power3(mH2) + 3*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mst12)*power3(mH2) - 6*mh2*mst12*DeltaInv(mst12,mst22,
     mh2)*power2(mst22)*power3(mH2) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst22)*power3(mH2) - 3*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 3*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) +
     3*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 3*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 64*mh2*mt2*DeltaInv(
     mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 64*mh2*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*(mst12 - mst22)*sb*
     power2(mh2)*power2(mH2)) - (ca*power2(Pi)*power3(sa)*(6*mH2*mst12*mt2*
     power2(mh2) - 6*mH2*mst22*mt2*power2(mh2) - 36*(mh2 - mH2)*mH2*mst24*mt2*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) - 6*mh2*mst12*mt2*power2(mH2) + 8*
     mst14*mt2*power2(mH2) + 6*mh2*mst22*mt2*power2(mH2) - 8*mst24*mt2*power2(
     mH2) + 28*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) -
     36*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) - 28*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*
     (mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 -
     mH2*(mst12 + mst22) + power2(mH2)) - 6*mst22*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst12) + 6*mst12*DeltaInv(mst12,mst22,mh2)*
     power2(mh2)*power2(mH2)*power2(mst22) - 3*mH2*mst12*power3(mh2) + 3*mH2*
     mst22*power3(mh2) + 6*mst12*mt2*power3(mh2) - 6*mst22*mt2*power3(mh2) +
     36*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) - 3*DeltaInv(mst12,
     mst22,mh2)*power2(mH2)*power2(mst12)*power3(mh2) + 3*DeltaInv(mst12,mst22,
     mh2)*power2(mH2)*power2(mst22)*power3(mh2) + 3*mh2*mst12*power3(mH2) - 3*
     mh2*mst22*power3(mH2) - 6*mst12*mt2*power3(mH2) + 6*mst22*mt2*power3(mH2)
     - 36*mh2*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power3(mH2) + 36*mh2*mst24*
     mt2*DeltaInv(mst22,mst22,mh2)*power3(mH2) + 6*mh2*mst22*DeltaInv(mst12,
     mst22,mh2)*power2(mst12)*power3(mH2) + 3*DeltaInv(mst12,mst22,mh2)*power2(
     mh2)*power2(mst12)*power3(mH2) - 6*mh2*mst12*DeltaInv(mst12,mst22,mh2)*
     power2(mst22)*power3(mH2) - 3*DeltaInv(mst12,mst22,mh2)*power2(mh2)*
     power2(mst22)*power3(mH2) - 3*mst12*DeltaInv(mst12,mst22,mh2)*power3(mh2)*
     power3(mH2) + 3*mst22*DeltaInv(mst12,mst22,mh2)*power3(mh2)*power3(mH2) +
     3*mst12*DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) - 3*mst22*
     DeltaInv(mst12,mst22,mh2)*power2(mH2)*power4(mh2) + 32*mh2*mt2*DeltaInv(
     mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 32*mh2*mt2*DeltaInv(mst22,
     mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*(mst12 - mst22)*power2(
     mh2)*power2(mH2)) + (ca*sa*power2(Pi)*(-36*(mh2 - mH2)*mH2*mst24*mt2*
     DeltaInv(mH2,mst22,mst22)*power2(mh2) - 6*mh2*mst12*mt2*power2(mH2) + 4*
     mst14*mt2*power2(mH2) + 6*mh2*mst22*mt2*power2(mH2) - 4*mst24*mt2*power2(
     mH2) + 3*mst12*power2(mh2)*power2(mH2) - 3*mst22*power2(mh2)*power2(mH2) -
     4*mst14*mt2*DeltaInv(mst12,mst12,mh2)*power2(mh2)*power2(mH2) - 36*mst14*
     mt2*DeltaInv(mst12,mst12,mH2)*power2(mh2)*power2(mH2) + 4*mst24*mt2*
     DeltaInv(mst22,mst22,mh2)*power2(mh2)*power2(mH2) + 3*(mh2 - mH2)*mH2*(
     mst12 - mst22)*DeltaInv(mst12,mH2,mst22)*power2(mh2)*(-2*mst12*mst22 -
     mH2*(mst12 + mst22) + power2(mH2)) - 3*mH2*mst12*power3(mh2) + 3*mH2*
     mst22*power3(mh2) + 6*mst12*mt2*power3(mh2) - 6*mst22*mt2*power3(mh2) +
     36*mH2*mst14*mt2*DeltaInv(mst12,mst12,mH2)*power3(mh2) + 16*mh2*mt2*
     DeltaInv(mst12,mst12,mh2)*power2(mH2)*power6(mst1) - 16*mh2*mt2*DeltaInv(
     mst22,mst22,mh2)*power2(mH2)*power6(mst2)))/((mh2 - mH2)*(mst12 - mst22)*
     power2(mh2)*power2(mH2))))/4. + xtau*((6*mt2*mu2*Fin3(mst12,mstau12,mt2,
     Q2))/((-mst12 + mst22)*(mstau12 - mstau22)*power2(mstau12 - mu2)) + (6*
     mt2*mu2*Fin3(mst22,mstau12,mt2,Q2))/((mst12 - mst22)*(mstau12 - mstau22)*
     power2(mstau12 - mu2)) + (6*mt2*mu2*Fin3(mstau22,mst12,mt2,Q2))/((mst12 -
     mst22)*(mstau12 - mstau22)*power2(mstau22 - mu2)) + (6*mt2*mu2*Fin3(
     mstau22,mst22,mt2,Q2))/((-mst12 + mst22)*(mstau12 - mstau22)*power2(
     mstau22 - mu2)) - (6*mu2*Fin3(mst12,mt2,mu2,Q2)*(mstau22*mt2 + mstau22*mu2
     - 2*mt2*mu2 + mstau12*(-mstau22 + mt2 + mu2) - power2(mu2) + (mstau12 -
     mu2)*(mstau22 - mu2)*DeltaInv(mst12,mt2,mu2)*(mst14 - 3*mst12*mt2 - 2*
     mst12*mu2 - mt2*mu2 + power2(mu2))))/((mst12 - mst22)*power2(mstau12 -
     mu2)*power2(mstau22 - mu2)) + (6*mu2*Fin3(mst22,mt2,mu2,Q2)*(mstau22*mt2 +
     mstau22*mu2 - 2*mt2*mu2 + mstau12*(-mstau22 + mt2 + mu2) - power2(mu2) + (
     mstau12 - mu2)*(mstau22 - mu2)*DeltaInv(mst22,mt2,mu2)*(mst24 - 3*mst22*
     mt2 - 2*mst22*mu2 - mt2*mu2 + power2(mu2))))/((mst12 - mst22)*power2(
     mstau12 - mu2)*power2(mstau22 - mu2)) + (mu2*(-84*mst12 + 84*mst22 + 18*
     mst12*log(mst12/Q2) - 18*mst22*log(mst22/Q2) + 54*mst12*log(mt2/Q2) - 54*
     mst22*log(mt2/Q2) - 12*mst12*log(mst12/Q2)*log(mt2/Q2) + 12*mst22*log(
     mst22/Q2)*log(mt2/Q2) + 6*mst12*log(mst12/Q2)*log(mu2/Q2) - 6*mst22*log(
     mst22/Q2)*log(mu2/Q2) - 6*mst12*log(mt2/Q2)*log(mu2/Q2) + 6*mst22*log(mt2/
     Q2)*log(mu2/Q2) - 2*mst12*power2(Pi) + 2*mst22*power2(Pi) - 3*mst12*
     power2(log(mst12/Q2)) + 3*mst22*power2(log(mst22/Q2)) - 9*mst12*power2(
     log(mt2/Q2)) + 9*mst22*power2(log(mt2/Q2)) + DeltaInv(mst12,mt2,mu2)*(-
     168*mst14*mt2 - 126*mst14*mu2 - 294*mst12*mt2*mu2 - 18*mst14*mu2*log(mu2/
     Q2) + 54*mst12*mt2*mu2*log(mu2/Q2) - 42*mt2*power2(mu2) + 36*mst12*log(
     mu2/Q2)*power2(mu2) + 18*mt2*log(mu2/Q2)*power2(mu2) - 4*mst14*mt2*power2(
     Pi) - 3*mst14*mu2*power2(Pi) - 7*mst12*mt2*mu2*power2(Pi) - mt2*power2(
     mu2)*power2(Pi) + 3*mst14*mu2*power2(log(mu2/Q2)) - 9*mst12*mt2*mu2*
     power2(log(mu2/Q2)) - 6*mst12*power2(mu2)*power2(log(mu2/Q2)) - 3*mt2*
     power2(mu2)*power2(log(mu2/Q2)) + 42*power3(mu2) - 18*log(mu2/Q2)*power3(
     mu2) + power2(Pi)*power3(mu2) + 3*power2(log(mu2/Q2))*power3(mu2) + 84*
     power6(mst1) + 2*power2(Pi)*power6(mst1) + 3*power2(log(mst12/Q2))*(mst12*
     mu2*(-mt2 + mu2) - mst14*(3*mt2 + 2*mu2) + power6(mst1)) + 3*power2(log(
     mt2/Q2))*(-5*mst14*(mt2 + mu2) + mu2*(-10*mst12*mt2 + mst12*mu2 - mt2*mu2
     + power2(mu2)) + 3*power6(mst1)) + 6*log(mst12/Q2)*(9*mst14*mt2 + 6*mst14*
     mu2 + 3*mst12*mt2*mu2 - 3*mst12*power2(mu2) + log(mu2/Q2)*(mst12*(3*mt2 -
     mu2)*mu2 + mst14*(mt2 + 2*mu2) - power6(mst1)) - 3*power6(mst1) + 2*log(
     mt2/Q2)*(mst12*mu2*(-2*mt2 + mu2) - 2*mst14*(mt2 + mu2) + power6(mst1))) +
     6*log(mt2/Q2)*(log(mu2/Q2)*(-(mst14*(mt2 + mu2)) + mu2*(mu2*(-mt2 + mu2) -
     mst12*(6*mt2 + mu2)) + power6(mst1)) - 3*(-5*mst14*(mt2 + mu2) + mu2*(-10*
     mst12*mt2 + mst12*mu2 - mt2*mu2 + power2(mu2)) + 3*power6(mst1)))) +
     DeltaInv(mst22,mt2,mu2)*(168*mst24*mt2 + 126*mst24*mu2 + 294*mst22*mt2*mu2
     + 18*mst24*mu2*log(mu2/Q2) - 54*mst22*mt2*mu2*log(mu2/Q2) + 42*mt2*power2(
     mu2) - 36*mst22*log(mu2/Q2)*power2(mu2) - 18*mt2*log(mu2/Q2)*power2(mu2) +
     4*mst24*mt2*power2(Pi) + 3*mst24*mu2*power2(Pi) + 7*mst22*mt2*mu2*power2(
     Pi) + mt2*power2(mu2)*power2(Pi) - 3*mst24*mu2*power2(log(mu2/Q2)) + 9*
     mst22*mt2*mu2*power2(log(mu2/Q2)) + 6*mst22*power2(mu2)*power2(log(mu2/Q2)
     ) + 3*mt2*power2(mu2)*power2(log(mu2/Q2)) - 42*power3(mu2) + 18*log(mu2/
     Q2)*power3(mu2) - power2(Pi)*power3(mu2) - 3*power2(log(mu2/Q2))*power3(
     mu2) - 84*power6(mst2) - 2*power2(Pi)*power6(mst2) - 3*power2(log(mst22/
     Q2))*(mst22*mu2*(-mt2 + mu2) - mst24*(3*mt2 + 2*mu2) + power6(mst2)) - 3*
     power2(log(mt2/Q2))*(-5*mst24*(mt2 + mu2) + mu2*(-10*mst22*mt2 + mst22*mu2
     - mt2*mu2 + power2(mu2)) + 3*power6(mst2)) - 6*log(mst22/Q2)*(9*mst24*mt2
     + 6*mst24*mu2 + 3*mst22*mt2*mu2 - 3*mst22*power2(mu2) + log(mu2/Q2)*(
     mst22*(3*mt2 - mu2)*mu2 + mst24*(mt2 + 2*mu2) - power6(mst2)) - 3*power6(
     mst2) + 2*log(mt2/Q2)*(mst22*mu2*(-2*mt2 + mu2) - 2*mst24*(mt2 + mu2) +
     power6(mst2))) - 6*log(mt2/Q2)*(log(mu2/Q2)*(-(mst24*(mt2 + mu2)) + mu2*(
     mu2*(-mt2 + mu2) - mst22*(6*mt2 + mu2)) + power6(mst2)) - 3*(-5*mst24*(mt2
     + mu2) + mu2*(-10*mst22*mt2 + mst22*mu2 - mt2*mu2 + power2(mu2)) + 3*
     power6(mst2))))))/((mst12 - mst22)*(mstau12 - mu2)*(-mstau22 + mu2))));

   return result * power2(ytau) * power2(yt) * twoLoop;
}

double delta_mtau_2loop_atau_ab(const Parameters& pars)
{
   using std::log;
   const Real ytau  = pars.ytau;
   const Real yb    = pars.yb;
   const Real xt    = pars.xt;
   const Real xb    = pars.xb;
   const Real mt    = pars.mt;
   const Real mt2   = power2(pars.mt);
   const Real mst1  = pars.mst1;
   const Real mst12 = power2(pars.mst1);
   const Real mst14 = power4(pars.mst1);
   const Real mst2  = pars.mst2;
   const Real mst22 = power2(pars.mst2);
   const Real mst24 = power4(pars.mst2);
   const Real msb1  = pars.msb1;
   const Real msb12 = power2(pars.msb1);
   const Real msb14 = power4(pars.msb1);
   const Real msb2  = pars.msb2;
   const Real msb22 = power2(pars.msb2);
   const Real msb24 = power4(pars.msb2);
   const Real mstau12 = power2(pars.mstau1);
   const Real mstau22 = power2(pars.mstau2);
   const Real msntau2 = power2(pars.msntau);
   const Real msntau4 = power4(pars.msntau);
   const Real mh2   = power2(pars.mh);
   const Real mH2   = power2(pars.mH);
   const Real mC2   = power2(pars.mC);
   const Real mA2   = power2(pars.mA);
   const Real mu    = pars.mu;
   const Real mu2   = power2(pars.mu);
   const Real tb    = pars.tb;
   const Real sb    = tb / std::sqrt(1. + power2(tb));
   const Real cb    = 1. / std::sqrt(1. + power2(tb));
   const Real Q2    = power2(pars.Q);
   const Real snt   = calc_sin_theta(mt, xt, mst12, mst22);
   const Real alpha = calc_alpha(mh2, mH2, tb);
   const Real sa    = std::sin(alpha);
   const Real ca    = std::cos(alpha);

   const double result =
   (3*(-(mC2*mH2) + mA2*(3*mC2 + mH2))*(msb12 + msb22))/(4.*mA2*mC2*mH2) + (3*
     log(mA2/Q2))/4. + (3*(2 + 3*(mC2 - msb22 - mst22)*mt2*DeltaInv(msb22,mC2,
     mst22))*log(mC2/Q2))/4. - (9*mt2*mu2*((mC2 - msb22 - mst22)*DeltaInv(
     msb22,mC2,mst22) + (-mC2 + msb22 + mst12)*DeltaInv(msb22,mst12,mC2))*log(
     mC2/Q2))/(2.*(mst12 - mst22)) + (9*ca*mu2*sa*sb*log(mh2/Q2))/(2*cb*mh2 -
     2*cb*mH2) - (9*log(mH2/Q2))/4. - (9*ca*mu2*sa*sb*(-2*mh2 + 3*mH2 + 3*(mh2
     - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*log(mH2/Q2))/(
     2.*cb*(mh2 - mH2)*mH2) + (3*(1/mA2 - 1/mC2 - 3/mH2)*msb12*log(msb12/Q2))/
     4. + (3*(1/mA2 - 1/mC2 - 3/mH2)*msb22*log(msb22/Q2))/4. - (27*ca*(mH2*
     msb22 - msb12*msb22 - msb24)*mu2*sa*sb*DeltaInv(msb22,mH2,msb12)*log(
     msb22/Q2))/(2.*cb*mH2) + (9*ca*(mH2*msb22 + msb12*msb22 - msb24)*mu2*sa*
     sb*DeltaInv(msb22,mH2,msb12)*log(mH2/Q2)*log(msb22/Q2))/(2.*cb*mH2) - (9*
     ca*msb12*msb22*mu2*sa*sb*DeltaInv(msb22,mH2,msb12)*log(msb12/Q2)*log(
     msb22/Q2))/(cb*mH2) + (9*(mC2*mst12 - msb22*mst12 - mst14)*mt2*mu2*
     DeltaInv(msb22,mst12,mC2)*log(mst12/Q2))/(2.*mC2*(mst12 - mst22)) - (3*(
     mC2*mst12 + msb22*mst12 - mst14)*mt2*mu2*DeltaInv(msb22,mst12,mC2)*log(
     mC2/Q2)*log(mst12/Q2))/(2.*mC2*(mst12 - mst22)) + (3*msb22*mst12*mt2*mu2*
     DeltaInv(msb22,mst12,mC2)*log(msb22/Q2)*log(mst12/Q2))/(mC2*mst12 - mC2*
     mst22) + (9*(mC2*mst22 - msb22*mst22 - mst24)*mt2*DeltaInv(msb22,mC2,
     mst22)*log(mst22/Q2))/(4.*mC2) - (9*(mC2*mst22 - msb22*mst22 - mst24)*mt2*
     mu2*DeltaInv(msb22,mC2,mst22)*log(mst22/Q2))/(2.*mC2*(mst12 - mst22)) - (
     3*(mC2*mst22 + msb22*mst22 - mst24)*mt2*DeltaInv(msb22,mC2,mst22)*log(mC2/
     Q2)*log(mst22/Q2))/(4.*mC2) + (3*(mC2*mst22 + msb22*mst22 - mst24)*mt2*
     mu2*DeltaInv(msb22,mC2,mst22)*log(mC2/Q2)*log(mst22/Q2))/(2.*mC2*(mst12 -
     mst22)) + (3*msb22*mst22*mt2*DeltaInv(msb22,mC2,mst22)*log(msb22/Q2)*log(
     mst22/Q2))/(2.*mC2) - (3*msb22*mst22*mt2*mu2*DeltaInv(msb22,mC2,mst22)*
     log(msb22/Q2)*log(mst22/Q2))/(mC2*mst12 - mC2*mst22) - (3*mt2*log(mC2/Q2)*
     log(mt2/Q2))/(4.*mC2) - (3*(mC2 + mt2)*Fin20(mC2,mt2,Q2))/(4.*power2(mC2))
     + (3*mt2*mu2*Fin20(msb22,mst12,Q2))/(2.*(-mst12 + mst22)*power2(mC2)) - (
     3*mt2*Fin20(msb22,mst22,Q2))/(4.*power2(mC2)) + (3*mt2*mu2*Fin20(msb22,
     mst22,Q2))/(2.*(mst12 - mst22)*power2(mC2)) + (21*mt2*mu2*(DeltaInv(msb22,
     mst12,mC2)*(2*msb22*mst12 + mC2*(msb22 + mst12) - power2(mC2)) + DeltaInv(
     msb22,mC2,mst22)*(-2*msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2))))/(
     2.*mC2*(mst12 - mst22)) - (9*mt2*mu2*log(msb22/Q2)*(-(DeltaInv(msb22,
     mst12,mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2)))
     + DeltaInv(msb22,mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) +
     mst24 + power2(mC2))))/(2.*mC2*(mst12 - mst22)) - (3*mt2*log(mC2/Q2)*log(
     msb22/Q2)*(-1 + DeltaInv(msb22,mC2,mst22)*(-(msb22*mst22) - mC2*(msb22 +
     2*mst22) + mst24 + power2(mC2))))/(4.*mC2) + (3*mt2*mu2*log(mC2/Q2)*log(
     msb22/Q2)*(DeltaInv(msb22,mst12,mC2)*(msb22*mst12 + mC2*(msb22 + 2*mst12)
     - mst14 - power2(mC2)) + DeltaInv(msb22,mC2,mst22)*(-(msb22*mst22) - mC2*(
     msb22 + 2*mst22) + mst24 + power2(mC2))))/(2.*mC2*(mst12 - mst22)) + (9*
     ca*(mh2 - mH2)*mu2*sa*sb*Fin20(msb22,msb12,Q2))/(2.*cb*mh2*power2(mH2)) +
     (9*ca*mu2*sa*sb*(-5 + 7*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(
     msb12 + msb22) + power2(mH2))))/(2.*cb*mH2) - (27*ca*mu2*sa*sb*log(msb12/
     Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*
     msb22) + msb24 + power2(mH2))))/(2.*cb*mH2) + (9*ca*mu2*sa*sb*log(mH2/Q2)*
     log(msb12/Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12
     + 2*msb22) + msb24 + power2(mH2))))/(2.*cb*mH2) + (mt2*mu2*(DeltaInv(
     msb22,mst12,mC2)*(2*msb22*mst12 + mC2*(msb22 + mst12) - power2(mC2)) +
     DeltaInv(msb22,mC2,mst22)*(-2*msb22*mst22 - mC2*(msb22 + mst22) + power2(
     mC2)))*power2(Pi))/(4.*mC2*(mst12 - mst22)) + (3*ca*mu2*sa*sb*(-1 +
     DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(
     mH2)))*power2(Pi))/(4.*cb*mH2) - (9*(mh2*(2*msb12 + 2*msb22 - mu2) - mH2*
     mu2)*power2(sa))/(4.*mh2*mH2) + (9*(msb12 + msb22 - mu2)*log(mh2/Q2)*
     power2(sa))/(2.*(mh2 - mH2)) - (9*(msb12 + msb22 - mu2)*log(mH2/Q2)*
     power2(sa))/(2.*(mh2 - mH2)) + (9*(2*(mh2*(mH2 - mu2) + mH2*(mH2 + msb12 +
     msb22 + 2*mu2)) + 3*(mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*mu2*DeltaInv(
     msb22,mH2,msb12))*log(mH2/Q2)*power2(sa))/(4.*(mh2 - mH2)*mH2) + (9*msb12*
     (mh2*(2*msb12 - 2*msb22 - mu2) - mH2*mu2)*log(msb12/Q2)*power2(sa))/(4.*
     mh2*mH2*(msb12 - msb22)) - (9*msb12*(msb12 - msb22 - mu2)*log(mh2/Q2)*log(
     msb12/Q2)*power2(sa))/(2.*(mh2 - mH2)*(msb12 - msb22)) + (9*msb12*(msb12 -
     msb22 - mu2)*log(mH2/Q2)*log(msb12/Q2)*power2(sa))/(2.*(mh2 - mH2)*(msb12
     - msb22)) + (9*msb22*(mH2*mu2 + mh2*(2*msb12 - 2*msb22 + mu2))*log(msb22/
     Q2)*power2(sa))/(4.*mh2*mH2*(msb12 - msb22)) + (9*(3*mH2*(mh2*msb22 -
     msb12*msb22 - msb24)*mu2*DeltaInv(msb22,mh2,msb12) + mh2*(-2*msb22 + 3*(
     mH2*msb22 - msb12*msb22 - msb24)*mu2*DeltaInv(msb22,mH2,msb12)))*log(
     msb22/Q2)*power2(sa))/(4.*mh2*mH2) - (9*msb22*(msb12 - msb22 + mu2)*log(
     mh2/Q2)*log(msb22/Q2)*power2(sa))/(2.*(mh2 - mH2)*(msb12 - msb22)) - (9*(-
     2*mh2*msb22 + (mh2 - mH2)*(mh2*msb22 + msb12*msb22 - msb24)*mu2*DeltaInv(
     msb22,mh2,msb12))*log(mh2/Q2)*log(msb22/Q2)*power2(sa))/(4.*mh2*(mh2 -
     mH2)) + (9*msb22*(msb12 - msb22 + mu2)*log(mH2/Q2)*log(msb22/Q2)*power2(
     sa))/(2.*(mh2 - mH2)*(msb12 - msb22)) + (9*(-2*mH2*msb22 - (mh2 - mH2)*(
     mH2*msb22 + msb12*msb22 - msb24)*mu2*DeltaInv(msb22,mH2,msb12))*log(mH2/
     Q2)*log(msb22/Q2)*power2(sa))/(4.*(mh2 - mH2)*mH2) + (9*msb12*msb22*mu2*(
     mH2*DeltaInv(msb22,mh2,msb12) + mh2*DeltaInv(msb22,mH2,msb12))*log(msb12/
     Q2)*log(msb22/Q2)*power2(sa))/(2.*mh2*mH2) - (9*log(mh2/Q2)*log(msb12/Q2)*
     (mH2*mu2 - mh2*(2*msb12 + mu2) + (mh2 - mH2)*mu2*DeltaInv(msb22,mh2,msb12)
     *(-(msb12*msb22) - mh2*(msb12 + 2*msb22) + msb24 + power2(mh2)))*power2(
     sa))/(4.*mh2*(mh2 - mH2)) + (9*log(mh2/Q2)*(3*mh2*(mh2 - mH2)*(mh2 - msb12
     - msb22)*mu2*DeltaInv(msb22,mh2,msb12) - 2*(-(mH2*mu2) + mh2*(msb12 +
     msb22 + 2*mu2) + 2*power2(mh2)))*power2(sa))/(4.*mh2*(mh2 - mH2)) - (9*
     mu2*Fin20(msb22,msb12,Q2)*power2(mh2 - mH2)*power2(sa))/(4.*power2(mh2)*
     power2(mH2)) - (9*(-4*mh2*mH2 - 2*mh2*msb12 - 2*mh2*msb22 - 5*mh2*mu2 - 5*
     mH2*mu2 + 7*mH2*mu2*DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 - mh2*(msb12
     + msb22) + power2(mh2)) + 7*mh2*mu2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*
     msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(sa))/(4.*mh2*mH2) + (9*
     log(msb12/Q2)*(-2*mh2*msb12 - 3*mh2*mu2 - 3*mH2*mu2 + 3*mH2*mu2*DeltaInv(
     msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12 + 2*msb22) + msb24 + power2(
     mh2)) + 3*mh2*mu2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 +
     2*msb22) + msb24 + power2(mH2)))*power2(sa))/(4.*mh2*mH2) + (9*log(mH2/Q2)
     *log(msb12/Q2)*(mh2*mu2 - mH2*(2*msb12 + mu2) - (mh2 - mH2)*mu2*DeltaInv(
     msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12 + 2*msb22) + msb24 + power2(
     mH2)))*power2(sa))/(4.*(mh2 - mH2)*mH2) - (3*mu2*(-mh2 - mH2 + mH2*
     DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 - mh2*(msb12 + msb22) + power2(
     mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 +
     msb22) + power2(mH2)))*power2(Pi)*power2(sa))/(8.*mh2*mH2) - (3*mu2*(-2 +
     3*mA2*(mA2 - msb12 - msb22)*DeltaInv(mA2,msb22,msb12))*log(mA2/Q2)*power2(
     sb))/(4.*mA2*power2(cb)) + (3*mu2*(-2 + 3*mC2*(mC2 - msb22 - mst12)*
     DeltaInv(msb22,mst12,mC2))*log(mC2/Q2)*power2(sb))/(4.*mC2*power2(cb)) + (
     9*mu2*(-2 + 3*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*log(
     mH2/Q2)*power2(sb))/(4.*mH2*power2(cb)) + (3*(mA2*msb22 + msb12*msb22 -
     msb24)*mu2*DeltaInv(mA2,msb22,msb12)*log(mA2/Q2)*log(msb22/Q2)*power2(sb))
     /(4.*mA2*power2(cb)) - (9*(mH2*msb22 + msb12*msb22 - msb24)*mu2*DeltaInv(
     msb22,mH2,msb12)*log(mH2/Q2)*log(msb22/Q2)*power2(sb))/(4.*mH2*power2(cb))
     + (3*msb12*msb22*mu2*(-(mH2*DeltaInv(mA2,msb22,msb12)) + 3*mA2*DeltaInv(
     msb22,mH2,msb12))*log(msb12/Q2)*log(msb22/Q2)*power2(sb))/(2.*mA2*mH2*
     power2(cb)) + (9*(mC2*mst12 - msb22*mst12 - mst14)*mu2*DeltaInv(msb22,
     mst12,mC2)*log(mst12/Q2)*power2(sb))/(4.*mC2*power2(cb)) - (3*(mC2*mst12 +
     msb22*mst12 - mst14)*mu2*DeltaInv(msb22,mst12,mC2)*log(mC2/Q2)*log(mst12/
     Q2)*power2(sb))/(4.*mC2*power2(cb)) + (3*msb22*mst12*mu2*DeltaInv(msb22,
     mst12,mC2)*log(msb22/Q2)*log(mst12/Q2)*power2(sb))/(2.*mC2*power2(cb)) + (
     3*mu2*(-1 + mA2*(mA2 - msb12 - msb22)*DeltaInv(mA2,msb22,msb12))*Fin3(mA2,
     msb22,msb12,Q2)*power2(sb))/(4.*power2(cb)*power2(mA2)) + (3*mu2*log(mA2/
     Q2)*log(msb12/Q2)*(-1 + DeltaInv(mA2,msb22,msb12)*(-(msb12*msb22) - mA2*(
     msb12 + 2*msb22) + msb24 + power2(mA2)))*power2(sb))/(4.*mA2*power2(cb)) -
     (3*mu2*Fin20(msb22,mst12,Q2)*power2(sb))/(4.*power2(cb)*power2(mC2)) - (3*
     mu2*log(mC2/Q2)*log(msb22/Q2)*(-1 + DeltaInv(msb22,mst12,mC2)*(-(msb22*
     mst12) - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2)))*power2(sb))/(4.*
     mC2*power2(cb)) + (9*mu2*log(msb22/Q2)*(mC2*mH2*(-(mA2*msb22) + msb12*
     msb22 + msb24)*DeltaInv(mA2,msb22,msb12) + mA2*(3*mC2*(mH2*msb22 - msb12*
     msb22 - msb24)*DeltaInv(msb22,mH2,msb12) + mH2*(-1 + DeltaInv(msb22,mst12,
     mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2)))))*
     power2(sb))/(4.*mA2*mC2*mH2*power2(cb)) - (3*mu2*Fin20(msb22,msb12,Q2)*(-(
     1/power2(mA2)) + 3/power2(mH2))*power2(sb))/(4.*power2(cb)) + (3*mu2*(15*
     mA2*mC2 + 5*mA2*mH2 - 5*mC2*mH2 + 7*mA2*mC2*mH2*msb22*DeltaInv(msb22,
     mst12,mC2) + 7*mA2*mC2*mH2*mst12*DeltaInv(msb22,mst12,mC2) + 14*mA2*mH2*
     msb22*mst12*DeltaInv(msb22,mst12,mC2) + 7*mC2*mH2*DeltaInv(mA2,msb22,
     msb12)*(-2*msb12*msb22 - mA2*(msb12 + msb22) + power2(mA2)) - 7*mA2*mH2*
     DeltaInv(msb22,mst12,mC2)*power2(mC2) - 21*mA2*mC2*DeltaInv(msb22,mH2,
     msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(sb))/(
     4.*mA2*mC2*mH2*power2(cb)) - (9*mu2*log(msb12/Q2)*(3*mA2 - mH2 + mH2*
     DeltaInv(mA2,msb22,msb12)*(-3*msb12*msb22 - mA2*(msb12 + 2*msb22) + msb24
     + power2(mA2)) - 3*mA2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(
     msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(sb))/(4.*mA2*mH2*power2(
     cb)) - (9*mu2*log(mH2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-
     (msb12*msb22) - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(sb))/
     (4.*mH2*power2(cb)) + (mu2*(3*mA2*mC2 + mA2*mH2 - mC2*mH2 + mA2*mC2*mH2*
     msb22*DeltaInv(msb22,mst12,mC2) + mA2*mC2*mH2*mst12*DeltaInv(msb22,mst12,
     mC2) + 2*mA2*mH2*msb22*mst12*DeltaInv(msb22,mst12,mC2) + mC2*mH2*DeltaInv(
     mA2,msb22,msb12)*(-2*msb12*msb22 - mA2*(msb12 + msb22) + power2(mA2)) -
     mA2*mH2*DeltaInv(msb22,mst12,mC2)*power2(mC2) + 3*mA2*mC2*DeltaInv(msb22,
     mH2,msb12)*(2*msb12*msb22 + mH2*(msb12 + msb22) - power2(mH2)))*power2(Pi)
     *power2(sb))/(8.*mA2*mC2*mH2*power2(cb)) + (9*mu2*log(mh2/Q2)*power2(sa)*
     power2(sb))/(2.*(mh2 - mH2)*power2(cb)) - (9*mu2*(-2*mh2 + 3*mH2 + 3*(mh2
     - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*log(mH2/Q2)*
     power2(sa)*power2(sb))/(2.*(mh2 - mH2)*mH2*power2(cb)) - (27*(mH2*msb22 -
     msb12*msb22 - msb24)*mu2*DeltaInv(msb22,mH2,msb12)*log(msb22/Q2)*power2(
     sa)*power2(sb))/(2.*mH2*power2(cb)) + (9*(mH2*msb22 + msb12*msb22 - msb24)
     *mu2*DeltaInv(msb22,mH2,msb12)*log(mH2/Q2)*log(msb22/Q2)*power2(sa)*
     power2(sb))/(2.*mH2*power2(cb)) - (9*msb12*msb22*mu2*DeltaInv(msb22,mH2,
     msb12)*log(msb12/Q2)*log(msb22/Q2)*power2(sa)*power2(sb))/(mH2*power2(cb))
     + (9*(mh2 - mH2)*mu2*Fin20(msb22,msb12,Q2)*power2(sa)*power2(sb))/(2.*mh2*
     power2(cb)*power2(mH2)) + (9*mu2*(-5 + 7*DeltaInv(msb22,mH2,msb12)*(-2*
     msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(sa)*power2(sb))/(
     2.*mH2*power2(cb)) - (27*mu2*log(msb12/Q2)*(-1 + DeltaInv(msb22,mH2,msb12)
     *(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(
     sa)*power2(sb))/(2.*mH2*power2(cb)) + (9*mu2*log(mH2/Q2)*log(msb12/Q2)*(-1
     + DeltaInv(msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12 + 2*msb22) +
     msb24 + power2(mH2)))*power2(sa)*power2(sb))/(2.*mH2*power2(cb)) + (3*mu2*
     (-1 + DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) +
     power2(mH2)))*power2(Pi)*power2(sa)*power2(sb))/(4.*mH2*power2(cb)) - (9*
     mu2*Fin3(msb22,mH2,msb12,Q2)*((mh2 - 3*mH2)*power2(cb)*(-1 + power2(sa))*
     power2(sa) + 2*ca*cb*sa*sb*(mh2 - mh2*power2(sa) + mH2*(-2 + 3*power2(sa))
     ) + (-1 + power2(sa))*(mh2 - mh2*power2(sa) + mH2*(-1 + 3*power2(sa)))*
     power2(sb) - (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,
     msb12)*(-1 + power2(sa))*(-2*ca*cb*sa*sb + power2(cb)*power2(sa) - (-1 +
     power2(sa))*power2(sb))))/(4.*(mh2 - mH2)*power2(cb)*power2(mH2)) - (9*
     mu2*sa*Fin3(msb22,mh2,msb12,Q2)*((3*mh2 - mH2)*sa*power2(cb)*(-1 + power2(
     sa)) + 2*ca*cb*sb*(mh2 - 3*mh2*power2(sa) + mH2*power2(sa)) + sa*(mh2*(2 -
     3*power2(sa)) + mH2*power2(sa))*power2(sb) - mh2*(mh2 - mH2)*(mh2 - msb12
     - msb22)*sa*DeltaInv(msb22,mh2,msb12)*(-2*ca*cb*sa*sb + power2(cb)*(-1 +
     power2(sa)) - power2(sa)*power2(sb))))/(4.*(mh2 - mH2)*power2(cb)*power2(
     mh2)) - (9*mt2*((mC2 - msb22 - mst22)*DeltaInv(msb22,mC2,mst22) + (-mC2 +
     msb22 + mst12)*DeltaInv(msb22,mst12,mC2))*log(mC2/Q2)*power2(snt))/4. - (
     3*(mC2*mst12 + msb22*mst12 - mst14)*mt2*DeltaInv(msb22,mst12,mC2)*log(mC2/
     Q2)*log(mst12/Q2)*power2(snt))/(4.*mC2) + (3*msb22*mst12*mt2*DeltaInv(
     msb22,mst12,mC2)*log(msb22/Q2)*log(mst12/Q2)*power2(snt))/(2.*mC2) + (3*(
     mC2*mst22 + msb22*mst22 - mst24)*mt2*DeltaInv(msb22,mC2,mst22)*log(mC2/Q2)
     *log(mst22/Q2)*power2(snt))/(4.*mC2) - (3*msb22*mst22*mt2*DeltaInv(msb22,
     mC2,mst22)*log(msb22/Q2)*log(mst22/Q2)*power2(snt))/(2.*mC2) - (3*mt2*
     Fin20(msb22,mst12,Q2)*power2(snt))/(4.*power2(mC2)) + (3*mt2*Fin20(msb22,
     mst22,Q2)*power2(snt))/(4.*power2(mC2)) - (9*mt2*log(msb22/Q2)*(-(
     DeltaInv(msb22,mst12,mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14
     + power2(mC2))) + DeltaInv(msb22,mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 +
     2*mst22) + mst24 + power2(mC2)))*power2(snt))/(4.*mC2) + (3*mt2*log(mC2/
     Q2)*log(msb22/Q2)*(DeltaInv(msb22,mst12,mC2)*(msb22*mst12 + mC2*(msb22 +
     2*mst12) - mst14 - power2(mC2)) + DeltaInv(msb22,mC2,mst22)*(-(msb22*
     mst22) - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2)))*power2(snt))/(4.*
     mC2) + ((2*mt2*DeltaInv(msb22,mC2,mst22)*(-2*msb22*mst22 - mC2*(msb22 +
     mst22) + power2(mC2)) + (mC2*(mst12 - mst22)*(msntau2 + mu2) - 2*mt2*
     DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst12 - mC2*(msb22 + mst12) + power2(
     mC2))*power2(msntau2 - mu2))/power2(msntau2 - mu2))*power2(Pi)*power2(snt)
     )/(16.*mC2) + (9*mu2*((mC2 - msb22 - mst22)*DeltaInv(msb22,mC2,mst22) + (-
     mC2 + msb22 + mst12)*DeltaInv(msb22,mst12,mC2))*log(mC2/Q2)*power2(sb)*
     power2(snt))/(4.*power2(cb)) - (9*(mC2*mst12 - msb22*mst12 - mst14)*mu2*
     DeltaInv(msb22,mst12,mC2)*log(mst12/Q2)*power2(sb)*power2(snt))/(4.*mC2*
     power2(cb)) + (3*(mC2*mst12 + msb22*mst12 - mst14)*mu2*DeltaInv(msb22,
     mst12,mC2)*log(mC2/Q2)*log(mst12/Q2)*power2(sb)*power2(snt))/(4.*mC2*
     power2(cb)) - (3*msb22*mst12*mu2*DeltaInv(msb22,mst12,mC2)*log(msb22/Q2)*
     log(mst12/Q2)*power2(sb)*power2(snt))/(2.*mC2*power2(cb)) + (9*(mC2*mst22
     - msb22*mst22 - mst24)*mu2*DeltaInv(msb22,mC2,mst22)*log(mst22/Q2)*power2(
     sb)*power2(snt))/(4.*mC2*power2(cb)) - (3*(mC2*mst22 + msb22*mst22 -
     mst24)*mu2*DeltaInv(msb22,mC2,mst22)*log(mC2/Q2)*log(mst22/Q2)*power2(sb)*
     power2(snt))/(4.*mC2*power2(cb)) + (3*msb22*mst22*mu2*DeltaInv(msb22,mC2,
     mst22)*log(msb22/Q2)*log(mst22/Q2)*power2(sb)*power2(snt))/(2.*mC2*power2(
     cb)) + (3*mu2*Fin20(msb22,mst12,Q2)*power2(sb)*power2(snt))/(4.*power2(cb)
     *power2(mC2)) - (3*mu2*Fin20(msb22,mst22,Q2)*power2(sb)*power2(snt))/(4.*
     power2(cb)*power2(mC2)) - (21*mu2*(DeltaInv(msb22,mst12,mC2)*(2*msb22*
     mst12 + mC2*(msb22 + mst12) - power2(mC2)) + DeltaInv(msb22,mC2,mst22)*(-
     2*msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2)))*power2(sb)*power2(snt)
     )/(4.*mC2*power2(cb)) + (9*mu2*log(msb22/Q2)*(-(DeltaInv(msb22,mst12,mC2)*
     (-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))) +
     DeltaInv(msb22,mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24
     + power2(mC2)))*power2(sb)*power2(snt))/(4.*mC2*power2(cb)) - (3*mu2*log(
     mC2/Q2)*log(msb22/Q2)*(DeltaInv(msb22,mst12,mC2)*(msb22*mst12 + mC2*(msb22
     + 2*mst12) - mst14 - power2(mC2)) + DeltaInv(msb22,mC2,mst22)*(-(msb22*
     mst22) - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2)))*power2(sb)*power2(
     snt))/(4.*mC2*power2(cb)) - (mu2*(DeltaInv(msb22,mst12,mC2)*(2*msb22*mst12
     + mC2*(msb22 + mst12) - power2(mC2)) + DeltaInv(msb22,mC2,mst22)*(-2*
     msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2)))*power2(Pi)*power2(sb)*
     power2(snt))/(8.*mC2*power2(cb)) + (3*(1 + mC2*(-mC2 + msb22 + mst12)*
     DeltaInv(msb22,mst12,mC2))*Fin3(msb22,mst12,mC2,Q2)*(-((mst12 - mst22)*
     mu2*power2(sb)*(-1 + power2(snt))) + mt2*power2(cb)*(2*mu2 + (mst12 -
     mst22)*power2(snt))))/(4.*(mst12 - mst22)*power2(cb)*power2(mC2)) - (3*(1
     + mC2*(-mC2 + msb22 + mst22)*DeltaInv(msb22,mC2,mst22))*Fin3(msb22,mC2,
     mst22,Q2)*((-mst12 + mst22)*mu2*power2(sb)*power2(snt) + mt2*power2(cb)*(
     mst22 + 2*mu2 + mst12*(-1 + power2(snt)) - mst22*power2(snt))))/(4.*(mst12
     - mst22)*power2(cb)*power2(mC2)) - (3*power2(log(mA2/Q2)))/8. + (3*mu2*(-1
     + mA2*(mA2 - msb12 - msb22)*DeltaInv(mA2,msb22,msb12))*power2(sb)*power2(
     log(mA2/Q2)))/(8.*mA2*power2(cb)) - (3*(1 + (mC2 - msb22 - mst22)*mt2*
     DeltaInv(msb22,mC2,mst22))*power2(log(mC2/Q2)))/8. + (3*mt2*mu2*((mC2 -
     msb22 - mst22)*DeltaInv(msb22,mC2,mst22) + (-mC2 + msb22 + mst12)*
     DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2)))/(4.*(mst12 - mst22)) - (3*
     mu2*(-1 + mC2*(mC2 - msb22 - mst12)*DeltaInv(msb22,mst12,mC2))*power2(sb)*
     power2(log(mC2/Q2)))/(8.*mC2*power2(cb)) + (3*mt2*((mC2 - msb22 - mst22)*
     DeltaInv(msb22,mC2,mst22) + (-mC2 + msb22 + mst12)*DeltaInv(msb22,mst12,
     mC2))*power2(snt)*power2(log(mC2/Q2)))/8. - (3*mu2*((mC2 - msb22 - mst22)*
     DeltaInv(msb22,mC2,mst22) + (-mC2 + msb22 + mst12)*DeltaInv(msb22,mst12,
     mC2))*power2(sb)*power2(snt)*power2(log(mC2/Q2)))/(8.*power2(cb)) - (9*ca*
     mu2*sa*sb*power2(log(mh2/Q2)))/(4*cb*mh2 - 4*cb*mH2) - (9*(-3*mh2*mu2 +
     mH2*mu2 + mh2*(mh2 - mH2)*(mh2 - msb12 - msb22)*mu2*DeltaInv(msb22,mh2,
     msb12) - 2*power2(mh2))*power2(sa)*power2(log(mh2/Q2)))/(8.*mh2*(mh2 -
     mH2)) + (9*mu2*power2(sa)*power2(sb)*power2(log(mh2/Q2)))/(4.*(-mh2 + mH2)
     *power2(cb)) + (9*power2(log(mH2/Q2)))/8. + (9*ca*mu2*sa*sb*(mh2 - 2*mH2 -
     (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*power2(
     log(mH2/Q2)))/(4.*cb*mH2*(-mh2 + mH2)) - (9*(2*mh2*mH2 - mh2*mu2 + 3*mH2*
     mu2 + (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*mu2*DeltaInv(msb22,mH2,msb12))
     *power2(sa)*power2(log(mH2/Q2)))/(8.*(mh2 - mH2)*mH2) - (9*mu2*(-1 + mH2*(
     mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*power2(sb)*power2(log(mH2/
     Q2)))/(8.*mH2*power2(cb)) + (9*mu2*(mh2 - 2*mH2 - (mh2 - mH2)*mH2*(mH2 -
     msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*power2(sa)*power2(sb)*power2(
     log(mH2/Q2)))/(4.*mH2*(-mh2 + mH2)*power2(cb)) + (9*ca*mu2*sa*sb*(-1 +
     DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24
     + power2(mH2)))*power2(log(msb12/Q2)))/(4.*cb*mH2) - (3*msb12*((mstau22 +
     mu2)*power2(mstau12) + mstau12*(-4*mstau22*mu2 + power2(mstau22) - power2(
     mu2)) + mu2*(-(mstau22*mu2) + power2(mstau22) + 2*power2(mu2)))*power2(
     log(msb12/Q2)))/(16.*power2(mstau12 - mu2)*power2(mstau22 - mu2)) - (9*
     mu2*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(
     msb12 + 2*msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-
     3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(sa)*
     power2(log(msb12/Q2)))/(8.*mh2*mH2) + (3*mu2*(3*mA2 - mH2 + mH2*DeltaInv(
     mA2,msb22,msb12)*(-3*msb12*msb22 - mA2*(msb12 + 2*msb22) + msb24 + power2(
     mA2)) - 3*mA2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*
     msb22) + msb24 + power2(mH2)))*power2(sb)*power2(log(msb12/Q2)))/(8.*mA2*
     mH2*power2(cb)) + (9*mu2*(-1 + DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 -
     mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(sa)*power2(sb)*
     power2(log(msb12/Q2)))/(4.*mH2*power2(cb)) + (9*ca*(mH2*msb22 - msb12*
     msb22 - msb24)*mu2*sa*sb*DeltaInv(msb22,mH2,msb12)*power2(log(msb22/Q2)))/
     (4.*cb*mH2) + (3*mt2*mu2*(-(DeltaInv(msb22,mst12,mC2)*(-3*msb22*mst12 -
     mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))) + DeltaInv(msb22,mC2,mst22)*
     (-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2)))*power2(
     log(msb22/Q2)))/(4.*mC2*(mst12 - mst22)) - (9*mu2*(mH2*(mh2*msb22 - msb12*
     msb22 - msb24)*DeltaInv(msb22,mh2,msb12) + mh2*(mH2*msb22 - msb12*msb22 -
     msb24)*DeltaInv(msb22,mH2,msb12))*power2(sa)*power2(log(msb22/Q2)))/(8.*
     mh2*mH2) - (3*mu2*(mC2*mH2*(-(mA2*msb22) + msb12*msb22 + msb24)*DeltaInv(
     mA2,msb22,msb12) + mA2*(3*mC2*(mH2*msb22 - msb12*msb22 - msb24)*DeltaInv(
     msb22,mH2,msb12) + mH2*(-1 + DeltaInv(msb22,mst12,mC2)*(-3*msb22*mst12 -
     mC2*(msb22 + 2*mst12) + mst14 + power2(mC2)))))*power2(sb)*power2(log(
     msb22/Q2)))/(8.*mA2*mC2*mH2*power2(cb)) + (9*(mH2*msb22 - msb12*msb22 -
     msb24)*mu2*DeltaInv(msb22,mH2,msb12)*power2(sa)*power2(sb)*power2(log(
     msb22/Q2)))/(4.*mH2*power2(cb)) + (3*mt2*(-(DeltaInv(msb22,mst12,mC2)*(-3*
     msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))) + DeltaInv(
     msb22,mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24 + power2(
     mC2)))*power2(snt)*power2(log(msb22/Q2)))/(8.*mC2) - (3*mu2*(-(DeltaInv(
     msb22,mst12,mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(
     mC2))) + DeltaInv(msb22,mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22)
     + mst24 + power2(mC2)))*power2(sb)*power2(snt)*power2(log(msb22/Q2)))/(8.*
     mC2*power2(cb)) - (3*(mC2*mst12 - msb22*mst12 - mst14)*mt2*mu2*DeltaInv(
     msb22,mst12,mC2)*power2(log(mst12/Q2)))/(4.*mC2*(mst12 - mst22)) - (3*
     mst12*(msntau2 + mu2)*power2(log(mst12/Q2)))/(16.*power2(msntau2 - mu2)) -
     (3*(mC2*mst12 - msb22*mst12 - mst14)*mu2*DeltaInv(msb22,mst12,mC2)*power2(
     sb)*power2(log(mst12/Q2)))/(8.*mC2*power2(cb)) + (3*((-2*(mC2*mst12 -
     msb22*mst12 - mst14)*mt2*DeltaInv(msb22,mst12,mC2))/mC2 + (mst12*(msntau2
     + mu2))/power2(msntau2 - mu2))*power2(snt)*power2(log(mst12/Q2)))/16. + (
     3*(mC2*mst12 - msb22*mst12 - mst14)*mu2*DeltaInv(msb22,mst12,mC2)*power2(
     sb)*power2(snt)*power2(log(mst12/Q2)))/(8.*mC2*power2(cb)) - (3*(mC2*mst22
     - msb22*mst22 - mst24)*mt2*DeltaInv(msb22,mC2,mst22)*power2(log(mst22/Q2))
     )/(8.*mC2) + (3*(mC2*mst22 - msb22*mst22 - mst24)*mt2*mu2*DeltaInv(msb22,
     mC2,mst22)*power2(log(mst22/Q2)))/(4.*mC2*(mst12 - mst22)) - (3*((-2*(mC2*
     mst22 - msb22*mst22 - mst24)*mt2*DeltaInv(msb22,mC2,mst22))/mC2 + (mst22*(
     msntau2 + mu2))/power2(msntau2 - mu2))*power2(snt)*power2(log(mst22/Q2)))/
     16. - (3*(mC2*mst22 - msb22*mst22 - mst24)*mu2*DeltaInv(msb22,mC2,mst22)*
     power2(sb)*power2(snt)*power2(log(mst22/Q2)))/(8.*mC2*power2(cb)) + xt*((
     3*mt2*mu*sb*(-1 + mC2*(mC2 - msb22 - mst22)*DeltaInv(msb22,mC2,mst22))*
     Fin3(msb22,mC2,mst22,Q2))/(2.*cb*(mst12 - mst22)*power2(mC2)) - (3*mt2*mu*
     sb*(-1 + mC2*(mC2 - msb22 - mst12)*DeltaInv(msb22,mst12,mC2))*Fin3(msb22,
     mst12,mC2,Q2))/(2.*cb*(mst12 - mst22)*power2(mC2)) + (mt2*mu*sb*(-6*Fin20(
     msb22,mst12,Q2) + 6*Fin20(msb22,mst22,Q2) + mC2*DeltaInv(msb22,mst12,mC2)*
     (42*mC2*msb22 + 42*mC2*mst12 + 84*msb22*mst12 + 18*mC2*mst12*log(mst12/Q2)
     - 18*msb22*mst12*log(mst12/Q2) - 18*mst14*log(mst12/Q2) - 42*power2(mC2) +
     6*log(msb22/Q2)*(2*msb22*mst12*log(mst12/Q2) + 3*(-3*msb22*mst12 - mC2*(
     msb22 + 2*mst12) + mst14 + power2(mC2))) - 6*log(mC2/Q2)*(3*mC2*(-mC2 +
     msb22 + mst12) + (mC2*mst12 + msb22*mst12 - mst14)*log(mst12/Q2) + log(
     msb22/Q2)*(-(msb22*mst12) - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2)))
     + mC2*msb22*power2(Pi) + mC2*mst12*power2(Pi) + 2*msb22*mst12*power2(Pi) -
     power2(mC2)*power2(Pi) + 3*mC2*(-mC2 + msb22 + mst12)*power2(log(mC2/Q2))
     - 3*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))*power2(
     log(msb22/Q2)) - 3*mC2*mst12*power2(log(mst12/Q2)) + 3*msb22*mst12*power2(
     log(mst12/Q2)) + 3*mst14*power2(log(mst12/Q2))) + mC2*DeltaInv(msb22,mC2,
     mst22)*(-42*mC2*msb22 - 42*mC2*mst22 - 84*msb22*mst22 - 18*mC2*mst22*log(
     mst22/Q2) + 18*msb22*mst22*log(mst22/Q2) + 18*mst24*log(mst22/Q2) + 42*
     power2(mC2) - 6*log(msb22/Q2)*(2*msb22*mst22*log(mst22/Q2) + 3*(-3*msb22*
     mst22 - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2))) + 6*log(mC2/Q2)*(3*
     mC2*(-mC2 + msb22 + mst22) + (mC2*mst22 + msb22*mst22 - mst24)*log(mst22/
     Q2) + log(msb22/Q2)*(-(msb22*mst22) - mC2*(msb22 + 2*mst22) + mst24 +
     power2(mC2))) - mC2*msb22*power2(Pi) - mC2*mst22*power2(Pi) - 2*msb22*
     mst22*power2(Pi) + power2(mC2)*power2(Pi) + 3*mC2*(mC2 - msb22 - mst22)*
     power2(log(mC2/Q2)) + 3*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24 +
     power2(mC2))*power2(log(msb22/Q2)) + 3*mC2*mst22*power2(log(mst22/Q2)) -
     3*msb22*mst22*power2(log(mst22/Q2)) - 3*mst24*power2(log(mst22/Q2)))))/(4.
     *cb*(mst12 - mst22)*power2(mC2)) + xb*((3*mt2*(-1 + mC2*(mC2 - msb22 -
     mst22)*DeltaInv(msb22,mC2,mst22))*Fin3(msb22,mC2,mst22,Q2))/(2.*(mst12 -
     mst22)*power2(mC2)) + (3*mt2*(1 + mC2*(-mC2 + msb22 + mst12)*DeltaInv(
     msb22,mst12,mC2))*Fin3(msb22,mst12,mC2,Q2))/(2.*(mst12 - mst22)*power2(
     mC2)) + (mt2*(-6*Fin20(msb22,mst12,Q2) + 6*Fin20(msb22,mst22,Q2) + mC2*
     DeltaInv(msb22,mst12,mC2)*(42*mC2*msb22 + 42*mC2*mst12 + 84*msb22*mst12 +
     18*mC2*mst12*log(mst12/Q2) - 18*msb22*mst12*log(mst12/Q2) - 18*mst14*log(
     mst12/Q2) - 42*power2(mC2) + 6*log(msb22/Q2)*(2*msb22*mst12*log(mst12/Q2)
     + 3*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))) - 6*
     log(mC2/Q2)*(3*mC2*(-mC2 + msb22 + mst12) + (mC2*mst12 + msb22*mst12 -
     mst14)*log(mst12/Q2) + log(msb22/Q2)*(-(msb22*mst12) - mC2*(msb22 + 2*
     mst12) + mst14 + power2(mC2))) + mC2*msb22*power2(Pi) + mC2*mst12*power2(
     Pi) + 2*msb22*mst12*power2(Pi) - power2(mC2)*power2(Pi) + 3*mC2*(-mC2 +
     msb22 + mst12)*power2(log(mC2/Q2)) - 3*(-3*msb22*mst12 - mC2*(msb22 + 2*
     mst12) + mst14 + power2(mC2))*power2(log(msb22/Q2)) - 3*mC2*mst12*power2(
     log(mst12/Q2)) + 3*msb22*mst12*power2(log(mst12/Q2)) + 3*mst14*power2(log(
     mst12/Q2))) + mC2*DeltaInv(msb22,mC2,mst22)*(-42*mC2*msb22 - 42*mC2*mst22
     - 84*msb22*mst22 - 18*mC2*mst22*log(mst22/Q2) + 18*msb22*mst22*log(mst22/
     Q2) + 18*mst24*log(mst22/Q2) + 42*power2(mC2) - 6*log(msb22/Q2)*(2*msb22*
     mst22*log(mst22/Q2) + 3*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24 +
     power2(mC2))) + 6*log(mC2/Q2)*(3*mC2*(-mC2 + msb22 + mst22) + (mC2*mst22 +
     msb22*mst22 - mst24)*log(mst22/Q2) + log(msb22/Q2)*(-(msb22*mst22) - mC2*(
     msb22 + 2*mst22) + mst24 + power2(mC2))) - mC2*msb22*power2(Pi) - mC2*
     mst22*power2(Pi) - 2*msb22*mst22*power2(Pi) + power2(mC2)*power2(Pi) + 3*
     mC2*(mC2 - msb22 - mst22)*power2(log(mC2/Q2)) + 3*(-3*msb22*mst22 - mC2*(
     msb22 + 2*mst22) + mst24 + power2(mC2))*power2(log(msb22/Q2)) + 3*mC2*
     mst22*power2(log(mst22/Q2)) - 3*msb22*mst22*power2(log(mst22/Q2)) - 3*
     mst24*power2(log(mst22/Q2)))))/(4.*(mst12 - mst22)*power2(mC2)))) + (3*
     power2(snt)*(-112*mt2*DeltaInv(msb22,mst12,mC2)*(-2*msb22*mst12 - mC2*(
     msb22 + mst12) + power2(mC2)) + 112*mt2*DeltaInv(msb22,mC2,mst22)*(-2*
     msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2)) + (16*(msntau2 - mst12)*(
     msntau2 - mst22)*(mst12 - mst22)*power2(msntau2 - mu2) + 3*mC2*(-16*(mst12
     - mst22)*(mst12 + mst22 - mu2)*power2(msntau2) + mst12*mst22*(-7*mst14 +
     7*mst24 + 16*mst12*mu2 - 16*mst22*mu2 + 7*power2(mst12) - 7*power2(mst22))
     + 16*(mst12 - mst22)*power3(msntau2) + msntau2*(16*(mst22 - mu2)*power2(
     mst12) + mst12*(7*mst14 - 16*power2(mst22)) + mst22*(-7*mst24 + 16*mst22*
     mu2 + 7*power2(mst22)) - 7*power3(mst12))))/((msntau2 - mst12)*(msntau2 -
     mst22)*power2(msntau2 - mu2))))/(64.*mC2) + (3*(msntau2*(mst12 - 3*mu2) +
     mu2*(mst12 + mu2))*Fin20(mst12,mu2,Q2))/(8.*power3(msntau2 - mu2)) + (3*(-
     ((msb22 - msntau2 - mt2)*(msntau2 + mu2)) - 2*msntau2*(msb24 + msntau2*(
     msntau2 - mt2) - msb22*(2*msntau2 + mt2))*(msntau2 - mu2)*DeltaInv(msb22,
     msntau2,mt2))*Fin3(msb22,msntau2,mt2,Q2))/(8.*power3(msntau2 - mu2)) - (3*
     (-((msb22 - mt2 - mu2)*(msntau2 + mu2)) + 2*(msntau2 - mu2)*mu2*(msb24 +
     mu2*(-mt2 + mu2) - msb22*(mt2 + 2*mu2))*DeltaInv(msb22,mt2,mu2))*Fin3(
     msb22,mt2,mu2,Q2))/(8.*power3(msntau2 - mu2)) - (3*msb22*msntau2*(msntau2
     + mu2 + 4*msntau2*mt2*(msntau2 - mu2)*DeltaInv(msb22,msntau2,mt2))*log(
     msb22/Q2)*log(msntau2/Q2))/(8.*power3(msntau2 - mu2)) - (3*msntau2*mst12*(
     msntau2 + mu2)*log(msntau2/Q2)*log(mst12/Q2))/(8.*power3(msntau2 - mu2)) +
     (3*mst12*mu2*(msntau2 + mu2)*log(mst12/Q2)*log(mu2/Q2))/(8.*power3(msntau2
     - mu2)) - (3*mu2*log(mt2/Q2)*log(mu2/Q2)*(msntau2*(mt2 - 2*mu2) + mu2*(mt2
     + 2*mu2) + 2*(msntau2 - mu2)*mu2*DeltaInv(msb22,mt2,mu2)*(msb24 - 3*msb22*
     mt2 - 2*msb22*mu2 - mt2*mu2 + power2(mu2))))/(8.*power3(msntau2 - mu2)) -
     (3*(msntau2*(mst12 - 3*mu2) + mu2*(mst12 + mu2))*Fin20(mst12,mu2,Q2)*
     power2(snt))/(8.*power3(msntau2 - mu2)) + (3*(msntau2*(mst22 - 3*mu2) +
     mu2*(mst22 + mu2))*Fin20(mst22,mu2,Q2)*power2(snt))/(8.*power3(msntau2 -
     mu2)) - (3*msntau2*(mst12 - mst22)*(msntau2 + mu2)*log(msntau2/Q2)*power2(
     snt))/(4.*power3(msntau2 - mu2)) + (3*msntau2*mst12*(msntau2 + mu2)*log(
     msntau2/Q2)*log(mst12/Q2)*power2(snt))/(8.*power3(msntau2 - mu2)) - (3*
     msntau2*mst22*(msntau2 + mu2)*log(msntau2/Q2)*log(mst22/Q2)*power2(snt))/(
     8.*power3(msntau2 - mu2)) + (3*(mst12 - mst22)*mu2*(msntau2 + mu2)*log(
     mu2/Q2)*power2(snt))/(4.*power3(msntau2 - mu2)) + (3*mst22*mu2*(msntau2 +
     mu2)*log(mst22/Q2)*log(mu2/Q2)*power2(snt))/(8.*power3(msntau2 - mu2)) + (
     3*msntau2*(mst12 - mst22)*(msntau2 + mu2)*power2(snt)*power2(log(msntau2/
     Q2)))/(16.*power3(msntau2 - mu2)) - (3*power2(log(msntau2/Q2))*(msntau2*(
     msntau2*(mst12 - mt2 - 4*mu2) + (mst12 - mt2)*mu2 + msb22*(msntau2 + mu2))
     + 2*(msntau2 - mu2)*DeltaInv(msb22,msntau2,mt2)*(-2*msntau2*msntau4*mu2 +
     msb24*power2(msntau2) + msntau4*power2(msntau2) - msb22*(2*msntau2 + mt2)*
     power2(msntau2) + msntau4*power2(mu2) - power2(msntau2)*power2(mu2) - mt2*
     power3(msntau2) + 2*mu2*power3(msntau2))))/(16.*power3(msntau2 - mu2)) + (
     3*log(msntau2/Q2)*log(mt2/Q2)*(msntau2*(msntau2*(mt2 - 2*mu2) + mt2*mu2 +
     2*power2(msntau2)) - 2*(msntau2 - mu2)*DeltaInv(msb22,msntau2,mt2)*(-2*
     msntau2*msntau4*mu2 + msb24*power2(msntau2) + msntau4*power2(msntau2) -
     msb22*(2*msntau2 + 3*mt2)*power2(msntau2) + msntau4*power2(mu2) - power2(
     msntau2)*power2(mu2) - mt2*power3(msntau2) + 2*mu2*power3(msntau2))))/(8.*
     power3(msntau2 - mu2)) + (3*mstau12*(2*mstau12*(mstau12 - 2*mu2) + msb12*(
     mstau12 + mu2) + msb22*(mstau12 + mu2))*log(mstau12/Q2))/(4.*power3(
     mstau12 - mu2)) - (3*msb12*mstau12*(mstau12 + mu2)*log(msb12/Q2)*log(
     mstau12/Q2))/(8.*power3(mstau12 - mu2)) - (3*msb22*mstau12*(mstau12 + mu2)
     *log(msb22/Q2)*log(mstau12/Q2))/(8.*power3(mstau12 - mu2)) - (3*mstau12*(
     2*mstau12*(mstau12 - 3*mu2) + msb12*(mstau12 + mu2) + msb22*(mstau12 +
     mu2))*power2(log(mstau12/Q2)))/(16.*power3(mstau12 - mu2)) + (3*mstau22*(
     2*mstau22*(mstau22 - 2*mu2) + msb12*(mstau22 + mu2) + msb22*(mstau22 +
     mu2))*log(mstau22/Q2))/(4.*power3(mstau22 - mu2)) - (3*msb12*mstau22*(
     mstau22 + mu2)*log(msb12/Q2)*log(mstau22/Q2))/(8.*power3(mstau22 - mu2)) -
     (3*msb22*mstau22*(mstau22 + mu2)*log(msb22/Q2)*log(mstau22/Q2))/(8.*
     power3(mstau22 - mu2)) - (3*mstau22*(2*mstau22*(mstau22 - 3*mu2) + msb12*(
     mstau22 + mu2) + msb22*(mstau22 + mu2))*power2(log(mstau22/Q2)))/(16.*
     power3(mstau22 - mu2)) + (3*mst12*mu2*(msntau2 + mu2)*log(mst12/Q2)*log(
     mu2/Q2)*power2(snt))/(8.*power3(-msntau2 + mu2)) + (3*(mst12 - mst22)*mu2*
     (msntau2 + mu2)*power2(snt)*power2(log(mu2/Q2)))/(16.*power3(-msntau2 +
     mu2)) + (3*log(msntau2/Q2)*(msntau2*(-msb22 + msntau2 - mst12 + mt2)*(
     msntau2 + mu2) - 3*(msntau2 - mu2)*DeltaInv(msb22,msntau2,mt2)*(-2*
     msntau2*msntau4*mu2 + msb24*power2(msntau2) + msntau4*power2(msntau2) -
     msb22*(2*msntau2 + mt2)*power2(msntau2) + msntau4*power2(mu2) - power2(
     msntau2)*power2(mu2) - mt2*power3(msntau2) + 2*mu2*power3(msntau2))))/(4.*
     power3(-msntau2 + mu2)) - (3*Fin20(msb12,mu2,Q2)*(mu2*(3*(1/power2(mstau12
     - mu2) + 1/power2(mstau22 - mu2)) + 2*mu2*(1/power3(mstau12 - mu2) + 1/
     power3(mstau22 - mu2))) + msb12*(-(1/power2(mstau12 - mu2)) - 1/power2(
     mstau22 - mu2) + 2*mu2*(1/power3(-mstau12 + mu2) + 1/power3(-mstau22 +
     mu2)))))/8. - (3*Fin20(msb22,mu2,Q2)*(mu2*(3*(1/power2(mstau12 - mu2) + 1/
     power2(mstau22 - mu2)) + 2*mu2*(1/power3(mstau12 - mu2) + 1/power3(mstau22
     - mu2))) + msb22*(-(1/power2(mstau12 - mu2)) - 1/power2(mstau22 - mu2) +
     2*mu2*(1/power3(-mstau12 + mu2) + 1/power3(-mstau22 + mu2)))))/8. - (3*
     msb12*mu2*log(msb12/Q2)*log(mu2/Q2)*(-(1/power2(mstau12 - mu2)) - 1/
     power2(mstau22 - mu2) + 2*mu2*(1/power3(-mstau12 + mu2) + 1/power3(-
     mstau22 + mu2))))/8. - (3*msb22*mu2*log(msb22/Q2)*log(mu2/Q2)*(-(1/power2(
     msntau2 - mu2)) + (4*mt2*mu2*DeltaInv(msb22,mt2,mu2))/power2(msntau2 -
     mu2) - 1/power2(mstau12 - mu2) - 1/power2(mstau22 - mu2) + 2*mu2*(1/
     power3(-msntau2 + mu2) + 1/power3(-mstau12 + mu2) + 1/power3(-mstau22 +
     mu2))))/8. - (3*mu2*log(mu2/Q2)*(mst12/power2(msntau2 - mu2) - mt2/power2(
     msntau2 - mu2) - mu2/power2(msntau2 - mu2) - (3*mu2*(msb24 + mu2*(-mt2 +
     mu2) - msb22*(mt2 + 2*mu2))*DeltaInv(msb22,mt2,mu2))/power2(msntau2 - mu2)
     + msb12/power2(mstau12 - mu2) - (4*mu2)/power2(mstau12 - mu2) + msb12/
     power2(mstau22 - mu2) - (4*mu2)/power2(mstau22 - mu2) + (2*mst12*mu2)/
     power3(msntau2 - mu2) + (2*msb12*mu2)/power3(mstau12 - mu2) + msb22*(1/
     power2(msntau2 - mu2) + 1/power2(mstau12 - mu2) + 1/power2(mstau22 - mu2)
     + 2*mu2*(1/power3(msntau2 - mu2) + 1/power3(mstau12 - mu2) + 1/power3(
     mstau22 - mu2))) + (2*msb12*mu2)/power3(mstau22 - mu2) + (2*mt2*mu2)/
     power3(-msntau2 + mu2) + (2*power2(mu2))/power3(-msntau2 + mu2) + (2*
     power2(mu2))/power3(-mstau12 + mu2) + (2*power2(mu2))/power3(-mstau22 +
     mu2)))/4. + (3*mu2*power2(log(mu2/Q2))*(mst12/power2(msntau2 - mu2) - mt2/
     power2(msntau2 - mu2) - (4*mu2)/power2(msntau2 - mu2) - (2*mu2*(msb24 +
     mu2*(-mt2 + mu2) - msb22*(mt2 + 2*mu2))*DeltaInv(msb22,mt2,mu2))/power2(
     msntau2 - mu2) + msb12/power2(mstau12 - mu2) - (6*mu2)/power2(mstau12 -
     mu2) + msb12/power2(mstau22 - mu2) - (6*mu2)/power2(mstau22 - mu2) + (2*
     mst12*mu2)/power3(msntau2 - mu2) + (2*msb12*mu2)/power3(mstau12 - mu2) +
     msb22*(1/power2(msntau2 - mu2) + 1/power2(mstau12 - mu2) + 1/power2(
     mstau22 - mu2) + 2*mu2*(1/power3(msntau2 - mu2) + 1/power3(mstau12 - mu2)
     + 1/power3(mstau22 - mu2))) + (2*msb12*mu2)/power3(mstau22 - mu2) + (2*
     mt2*mu2)/power3(-msntau2 + mu2) + (4*power2(mu2))/power3(-msntau2 + mu2) +
     (4*power2(mu2))/power3(-mstau12 + mu2) + (4*power2(mu2))/power3(-mstau22 +
     mu2)))/16. + (9*ca*mu2*sb*(-2/mh2 + 2/(-mh2 + mH2) + 3*(mh2 - msb12 -
     msb22)*DeltaInv(msb22,mh2,msb12))*log(mh2/Q2)*power3(sa))/(2.*cb) + (9*ca*
     mu2*sb*(-2*mh2 + 4*mH2 + 3*(mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(
     msb22,mH2,msb12))*log(mH2/Q2)*power3(sa))/(2.*cb*(mh2 - mH2)*mH2) + (27*
     ca*mu2*sb*(mH2*(mh2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mh2,msb12)
     + mh2*(mH2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12))*log(
     msb22/Q2)*power3(sa))/(2.*cb*mh2*mH2) - (9*ca*(mh2*msb22 + msb12*msb22 -
     msb24)*mu2*sb*DeltaInv(msb22,mh2,msb12)*log(mh2/Q2)*log(msb22/Q2)*power3(
     sa))/(2.*cb*mh2) - (9*ca*(mH2*msb22 + msb12*msb22 - msb24)*mu2*sb*
     DeltaInv(msb22,mH2,msb12)*log(mH2/Q2)*log(msb22/Q2)*power3(sa))/(2.*cb*
     mH2) + (9*ca*msb12*msb22*mu2*sb*(mH2*DeltaInv(msb22,mh2,msb12) + mh2*
     DeltaInv(msb22,mH2,msb12))*log(msb12/Q2)*log(msb22/Q2)*power3(sa))/(cb*
     mh2*mH2) - (9*ca*mu2*sb*log(mh2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(msb22,
     mh2,msb12)*(-(msb12*msb22) - mh2*(msb12 + 2*msb22) + msb24 + power2(mh2)))
     *power3(sa))/(2.*cb*mh2) - (9*ca*mu2*sb*Fin20(msb22,msb12,Q2)*power2(mh2 -
     mH2)*power3(sa))/(2.*cb*power2(mh2)*power2(mH2)) - (9*ca*mu2*sb*(-5*(mh2 +
     mH2) + 7*mH2*DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 - mh2*(msb12 +
     msb22) + power2(mh2)) + 7*mh2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 -
     mH2*(msb12 + msb22) + power2(mH2)))*power3(sa))/(2.*cb*mh2*mH2) + (27*ca*
     mu2*sb*log(msb12/Q2)*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*
     msb12*msb22 - mh2*(msb12 + 2*msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(
     msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(
     mH2)))*power3(sa))/(2.*cb*mh2*mH2) - (9*ca*mu2*sb*log(mH2/Q2)*log(msb12/
     Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12 + 2*
     msb22) + msb24 + power2(mH2)))*power3(sa))/(2.*cb*mH2) - (3*ca*mu2*sb*(-
     mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 - mh2*(msb12 +
     msb22) + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 -
     mH2*(msb12 + msb22) + power2(mH2)))*power2(Pi)*power3(sa))/(4.*cb*mh2*mH2)
     - (9*ca*mu2*sb*(-(1/mh2) + 2/(-mh2 + mH2) + (mh2 - msb12 - msb22)*
     DeltaInv(msb22,mh2,msb12))*power2(log(mh2/Q2))*power3(sa))/(4.*cb) - (9*
     ca*mu2*sb*(mh2 - 3*mH2 - (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(
     msb22,mH2,msb12))*power2(log(mH2/Q2))*power3(sa))/(4.*cb*mH2*(-mh2 + mH2))
     - (9*ca*mu2*sb*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22
     - mh2*(msb12 + 2*msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,
     msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*
     power2(log(msb12/Q2))*power3(sa))/(4.*cb*mh2*mH2) - (9*ca*mu2*sb*(mH2*(
     mh2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mh2,msb12) + mh2*(mH2*
     msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12))*power2(log(msb22/
     Q2))*power3(sa))/(4.*cb*mh2*mH2) + (3*power2(log(msb22/Q2))*((2*mt2)/mC2 +
     msb22/(msntau2 - mu2) + msb22/(-mstau12 + mu2) + msb22/(-mstau22 + mu2) -
     (2*mt2*DeltaInv(msb22,mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) +
     mst24 + power2(mC2)))/mC2 + (2*msb22*mu2)/power2(msntau2 - mu2) + (2*
     msb24*mt2*mu2*DeltaInv(msb22,mt2,mu2))/power2(msntau2 - mu2) - (2*msb22*
     mu2)/power2(mstau12 - mu2) - (2*msb22*mu2)/power2(mstau22 - mu2) + (4*
     msb24*DeltaInv(msb22,mt2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (2*
     msb22*mt2*DeltaInv(msb22,mt2,mu2)*power2(mu2))/power2(msntau2 - mu2) +
     power3(msb22)/((mstau12 - mu2)*power2(msb22 - mstau12)) + power3(msb22)/((
     mstau22 - mu2)*power2(msb22 - mstau22)) - (2*mu2*DeltaInv(msb22,mt2,mu2)*
     power3(msb22))/power2(msntau2 - mu2) + power3(msb22)/((msb22 - mstau12)*
     power2(mstau12 - mu2)) + (mu2*power3(msb22))/(power2(msb22 - mstau12)*
     power2(mstau12 - mu2)) + power3(msb22)/((msb22 - mstau22)*power2(mstau22 -
     mu2)) + (mu2*power3(msb22))/(power2(msb22 - mstau22)*power2(mstau22 - mu2)
     ) - (2*msntau2*DeltaInv(msb22,msntau2,mt2)*(msb22*msntau2*(msntau2 - mt2)
     - msb24*(2*msntau2 + mt2) + power3(msb22)))/power2(msntau2 - mu2) - (2*
     msb22*DeltaInv(msb22,mt2,mu2)*power3(mu2))/power2(msntau2 - mu2) - power4(
     msb22)/(power2(msb22 - mstau12)*power2(mstau12 - mu2)) - power4(msb22)/(
     power2(msb22 - mstau22)*power2(mstau22 - mu2))))/16. + (3*(-64 + (16*
     msb12)/mA2 - (48*msb12)/mH2 + (16*msb22)/mA2 - (16*msb22)/mC2 - (48*msb22)
     /mH2 - (16*mst12)/mC2 + (16*mt2)/mC2 + (64*msb22)/(msntau2 - mu2) + (48*
     mt2)/(msntau2 - mu2) + (21*msb14)/((msb12 - mstau12)*(mstau12 - mu2)) + (
     21*msb24)/((msb22 - mstau12)*(mstau12 - mu2)) + (21*msb14)/((msb12 -
     mstau22)*(mstau22 - mu2)) + (21*msb24)/((msb22 - mstau22)*(mstau22 - mu2))
     + (160*mu2)/(msntau2 - mu2) + (48*mst12)/(-msntau2 + mu2) + (21*mst14)/((
     msntau2 - mst12)*(-msntau2 + mu2)) + (48*msb12)/(-mstau12 + mu2) + (48*
     msb22)/(-mstau12 + mu2) + (64*mu2)/(-mstau12 + mu2) + (48*msb12)/(-mstau22
     + mu2) + (48*msb22)/(-mstau22 + mu2) + (64*mu2)/(-mstau22 + mu2) - (112*
     mt2*DeltaInv(msb22,mC2,mst22)*(-2*msb22*mst22 - mC2*(msb22 + mst22) +
     power2(mC2)))/mC2 - (56*power2(mt2))/power2(mC2) + (21*mst14)/power2(
     msntau2 - mu2) + (128*msb22*mu2)/power2(msntau2 - mu2) - (96*mst12*mu2)/
     power2(msntau2 - mu2) - (21*mst14*mu2)/((msntau2 - mst12)*power2(msntau2 -
     mu2)) + (96*mt2*mu2)/power2(msntau2 - mu2) + (112*msb24*mt2*mu2*DeltaInv(
     msb22,mt2,mu2))/power2(msntau2 - mu2) + (21*msb14)/power2(mstau12 - mu2) +
     (21*msb24)/power2(mstau12 - mu2) - (96*msb12*mu2)/power2(mstau12 - mu2) -
     (96*msb22*mu2)/power2(mstau12 - mu2) + (21*msb14*mu2)/((msb12 - mstau12)*
     power2(mstau12 - mu2)) + (21*msb24*mu2)/((msb22 - mstau12)*power2(mstau12
     - mu2)) + (21*msb14)/power2(mstau22 - mu2) + (21*msb24)/power2(mstau22 -
     mu2) - (96*msb12*mu2)/power2(mstau22 - mu2) - (96*msb22*mu2)/power2(
     mstau22 - mu2) + (21*msb14*mu2)/((msb12 - mstau22)*power2(mstau22 - mu2))
     + (21*msb24*mu2)/((msb22 - mstau22)*power2(mstau22 - mu2)) + (160*power2(
     mu2))/power2(msntau2 - mu2) + (112*msb24*DeltaInv(msb22,mt2,mu2)*power2(
     mu2))/power2(msntau2 - mu2) + (448*msb22*mt2*DeltaInv(msb22,mt2,mu2)*
     power2(mu2))/power2(msntau2 - mu2) - (64*power2(mu2))/power2(mstau12 -
     mu2) - (64*power2(mu2))/power2(mstau22 - mu2) + (14*power3(msb12))/((
     mstau12 - mu2)*power2(msb12 - mstau12)) + (14*power3(msb12))/((mstau22 -
     mu2)*power2(msb12 - mstau22)) - (7*power3(msb12))/((msb12 - mstau12)*
     power2(mstau12 - mu2)) + (14*mu2*power3(msb12))/(power2(msb12 - mstau12)*
     power2(mstau12 - mu2)) - (7*power3(msb12))/((msb12 - mstau22)*power2(
     mstau22 - mu2)) + (14*mu2*power3(msb12))/(power2(msb12 - mstau22)*power2(
     mstau22 - mu2)) + (14*power3(msb22))/((mstau12 - mu2)*power2(msb22 -
     mstau12)) + (14*power3(msb22))/((mstau22 - mu2)*power2(msb22 - mstau22)) -
     (112*mu2*DeltaInv(msb22,mt2,mu2)*power3(msb22))/power2(msntau2 - mu2) + (
     7*power3(msb22))/((-msb22 + mstau12)*power2(mstau12 - mu2)) + (14*mu2*
     power3(msb22))/(power2(msb22 - mstau12)*power2(mstau12 - mu2)) + (7*
     power3(msb22))/((-msb22 + mstau22)*power2(mstau22 - mu2)) + (14*mu2*
     power3(msb22))/(power2(msb22 - mstau22)*power2(mstau22 - mu2)) + (112*
     DeltaInv(msb22,msntau2,mt2)*(msb24*msntau2*(msntau2 + mt2) + 2*msntau2*
     msntau4*mu2 - msntau4*power2(msntau2) + msb22*(msntau2 + 4*mt2)*power2(
     msntau2) - msntau4*power2(mu2) + power2(msntau2)*power2(mu2) - msntau2*
     power3(msb22) + mt2*power3(msntau2) - 2*mu2*power3(msntau2)))/power2(
     msntau2 - mu2) + (14*power3(mst12))/((msntau2 - mu2)*power2(msntau2 -
     mst12)) + (7*power3(mst12))/((msntau2 - mst12)*power2(msntau2 - mu2)) + (
     14*mu2*power3(mst12))/(power2(msntau2 - mst12)*power2(msntau2 - mu2)) + (
     112*msb22*DeltaInv(msb22,mt2,mu2)*power3(mu2))/power2(msntau2 - mu2) + (
     112*mt2*DeltaInv(msb22,mt2,mu2)*power3(mu2))/power2(msntau2 - mu2) - (14*
     power4(msb12))/(power2(msb12 - mstau12)*power2(mstau12 - mu2)) - (14*
     power4(msb12))/(power2(msb12 - mstau22)*power2(mstau22 - mu2)) - (14*
     power4(msb22))/(power2(msb22 - mstau12)*power2(mstau12 - mu2)) - (14*
     power4(msb22))/(power2(msb22 - mstau22)*power2(mstau22 - mu2)) - (14*
     power4(mst12))/(power2(msntau2 - mst12)*power2(msntau2 - mu2)) - (112*
     DeltaInv(msb22,mt2,mu2)*power4(mu2))/power2(msntau2 - mu2)))/64. + (
     power2(Pi)*(2 + msb22/(msntau2 - mu2) + mt2/(msntau2 - mu2) + (4*mu2)/(
     msntau2 - mu2) + mst12/(-msntau2 + mu2) + msb12/(-mstau12 + mu2) + msb22/(
     -mstau12 + mu2) + msb12/(-mstau22 + mu2) + msb22/(-mstau22 + mu2) + 2*(-
     mC2 + msb22 + mst22 + (2*msb22*mst22)/mC2)*mt2*DeltaInv(msb22,mC2,mst22) -
     (3*power2(mt2))/power2(mC2) + (2*msb22*mu2)/power2(msntau2 - mu2) - (2*
     mst12*mu2)/power2(msntau2 - mu2) + (2*mt2*mu2)/power2(msntau2 - mu2) + (2*
     msb24*mt2*mu2*DeltaInv(msb22,mt2,mu2))/power2(msntau2 - mu2) - (2*msb12*
     mu2)/power2(mstau12 - mu2) - (2*msb22*mu2)/power2(mstau12 - mu2) - (2*
     msb12*mu2)/power2(mstau22 - mu2) - (2*msb22*mu2)/power2(mstau22 - mu2) + (
     4*power2(mu2))/power2(msntau2 - mu2) + (2*msb24*DeltaInv(msb22,mt2,mu2)*
     power2(mu2))/power2(msntau2 - mu2) + (8*msb22*mt2*DeltaInv(msb22,mt2,mu2)*
     power2(mu2))/power2(msntau2 - mu2) + power3(msb12)/((mstau12 - mu2)*
     power2(msb12 - mstau12)) + power3(msb12)/((mstau22 - mu2)*power2(msb12 -
     mstau22)) + power3(msb12)/((msb12 - mstau12)*power2(mstau12 - mu2)) + (
     mu2*power3(msb12))/(power2(msb12 - mstau12)*power2(mstau12 - mu2)) +
     power3(msb12)/((msb12 - mstau22)*power2(mstau22 - mu2)) + (mu2*power3(
     msb12))/(power2(msb12 - mstau22)*power2(mstau22 - mu2)) + power3(msb22)/((
     mstau12 - mu2)*power2(msb22 - mstau12)) + power3(msb22)/((mstau22 - mu2)*
     power2(msb22 - mstau22)) - (2*mu2*DeltaInv(msb22,mt2,mu2)*power3(msb22))/
     power2(msntau2 - mu2) + power3(msb22)/((msb22 - mstau12)*power2(mstau12 -
     mu2)) + (mu2*power3(msb22))/(power2(msb22 - mstau12)*power2(mstau12 - mu2)
     ) + power3(msb22)/((msb22 - mstau22)*power2(mstau22 - mu2)) + (mu2*power3(
     msb22))/(power2(msb22 - mstau22)*power2(mstau22 - mu2)) + (2*DeltaInv(
     msb22,msntau2,mt2)*(msb24*msntau2*(msntau2 + mt2) + 2*msntau2*msntau4*mu2
     - msntau4*power2(msntau2) + msb22*(msntau2 + 4*mt2)*power2(msntau2) -
     msntau4*power2(mu2) + power2(msntau2)*power2(mu2) - msntau2*power3(msb22)
     + mt2*power3(msntau2) - 2*mu2*power3(msntau2)))/power2(msntau2 - mu2) +
     power3(mst12)/((msntau2 - mu2)*power2(msntau2 - mst12)) - power3(mst12)/((
     msntau2 - mst12)*power2(msntau2 - mu2)) + (mu2*power3(mst12))/(power2(
     msntau2 - mst12)*power2(msntau2 - mu2)) + (2*msb22*DeltaInv(msb22,mt2,mu2)
     *power3(mu2))/power2(msntau2 - mu2) + (2*mt2*DeltaInv(msb22,mt2,mu2)*
     power3(mu2))/power2(msntau2 - mu2) - power4(msb12)/(power2(msb12 -
     mstau12)*power2(mstau12 - mu2)) - power4(msb12)/(power2(msb12 - mstau22)*
     power2(mstau22 - mu2)) - power4(msb22)/(power2(msb22 - mstau12)*power2(
     mstau12 - mu2)) - power4(msb22)/(power2(msb22 - mstau22)*power2(mstau22 -
     mu2)) - power4(mst12)/(power2(msntau2 - mst12)*power2(msntau2 - mu2)) - (
     2*DeltaInv(msb22,mt2,mu2)*power4(mu2))/power2(msntau2 - mu2)))/16. + (9*(
     mh2 + mH2)*(msb12 + msb22 - mu2)*power4(sa))/(4.*mh2*mH2) - (9*(msb12 +
     msb22 - mu2)*log(mh2/Q2)*power4(sa))/(2.*(mh2 - mH2)) + (9*(msb12 + msb22
     - mu2)*log(mH2/Q2)*power4(sa))/(2.*(mh2 - mH2)) - (9*(mh2*(mH2 - 2*mu2) +
     mH2*(3*mH2 + 2*(msb12 + msb22 + 2*mu2)) + 3*(mh2 - mH2)*mH2*(mH2 - msb12 -
     msb22)*mu2*DeltaInv(msb22,mH2,msb12))*log(mH2/Q2)*power4(sa))/(4.*(mh2 -
     mH2)*mH2) - (9*(mh2 + mH2)*msb12*(msb12 - msb22 - mu2)*log(msb12/Q2)*
     power4(sa))/(4.*mh2*mH2*(msb12 - msb22)) + (9*msb12*(msb12 - msb22 - mu2)*
     log(mh2/Q2)*log(msb12/Q2)*power4(sa))/(2.*(mh2 - mH2)*(msb12 - msb22)) - (
     9*msb12*(msb12 - msb22 - mu2)*log(mH2/Q2)*log(msb12/Q2)*power4(sa))/(2.*(
     mh2 - mH2)*(msb12 - msb22)) - (9*(mh2 + mH2)*msb22*(msb12 - msb22 + mu2)*
     log(msb22/Q2)*power4(sa))/(4.*mh2*mH2*(msb12 - msb22)) - (9*(-((mh2 + mH2)
     *msb22) + 3*mH2*(mh2*msb22 - msb12*msb22 - msb24)*mu2*DeltaInv(msb22,mh2,
     msb12) + 3*mh2*(mH2*msb22 - msb12*msb22 - msb24)*mu2*DeltaInv(msb22,mH2,
     msb12))*log(msb22/Q2)*power4(sa))/(4.*mh2*mH2) + (9*msb22*(msb12 - msb22 +
     mu2)*log(mh2/Q2)*log(msb22/Q2)*power4(sa))/(2.*(mh2 - mH2)*(msb12 - msb22)
     ) + (9*(-2*mh2*msb22 + (mh2 - mH2)*(mh2*msb22 + msb12*msb22 - msb24)*mu2*
     DeltaInv(msb22,mh2,msb12))*log(mh2/Q2)*log(msb22/Q2)*power4(sa))/(4.*mh2*(
     mh2 - mH2)) - (9*msb22*(msb12 - msb22 + mu2)*log(mH2/Q2)*log(msb22/Q2)*
     power4(sa))/(2.*(mh2 - mH2)*(msb12 - msb22)) + (9*(2*mH2*msb22 + (mh2 -
     mH2)*(mH2*msb22 + msb12*msb22 - msb24)*mu2*DeltaInv(msb22,mH2,msb12))*log(
     mH2/Q2)*log(msb22/Q2)*power4(sa))/(4.*(mh2 - mH2)*mH2) - (9*msb12*msb22*
     mu2*(mH2*DeltaInv(msb22,mh2,msb12) + mh2*DeltaInv(msb22,mH2,msb12))*log(
     msb12/Q2)*log(msb22/Q2)*power4(sa))/(2.*mh2*mH2) + (9*log(mh2/Q2)*(-2*mH2*
     mu2 + mh2*(mH2 + 2*(msb12 + msb22 + 2*mu2)) - 3*mh2*(mh2 - mH2)*(mh2 -
     msb12 - msb22)*mu2*DeltaInv(msb22,mh2,msb12) + 3*power2(mh2))*power4(sa))/
     (4.*mh2*(mh2 - mH2)) + (9*log(mh2/Q2)*log(msb12/Q2)*(mH2*mu2 - mh2*(2*
     msb12 + mu2) + (mh2 - mH2)*mu2*DeltaInv(msb22,mh2,msb12)*(-(msb12*msb22) -
     mh2*(msb12 + 2*msb22) + msb24 + power2(mh2)))*power4(sa))/(4.*mh2*(mh2 -
     mH2)) + (9*mu2*Fin20(msb22,msb12,Q2)*power2(mh2 - mH2)*power4(sa))/(4.*
     power2(mh2)*power2(mH2)) + (9*(-4*mh2*mH2 - mh2*msb12 - mH2*msb12 - mh2*
     msb22 - mH2*msb22 - 5*mh2*mu2 - 5*mH2*mu2 + 7*mH2*mu2*DeltaInv(msb22,mh2,
     msb12)*(-2*msb12*msb22 - mh2*(msb12 + msb22) + power2(mh2)) + 7*mh2*mu2*
     DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(
     mH2)))*power4(sa))/(4.*mh2*mH2) - (9*log(msb12/Q2)*(-((mh2 + mH2)*(msb12 +
     3*mu2)) + 3*mH2*mu2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12
     + 2*msb22) + msb24 + power2(mh2)) + 3*mh2*mu2*DeltaInv(msb22,mH2,msb12)*(-
     3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power4(sa))/
     (4.*mh2*mH2) + (9*log(mH2/Q2)*log(msb12/Q2)*(-(mh2*mu2) + mH2*(2*msb12 +
     mu2) + (mh2 - mH2)*mu2*DeltaInv(msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(
     msb12 + 2*msb22) + msb24 + power2(mH2)))*power4(sa))/(4.*(mh2 - mH2)*mH2)
     + (3*mu2*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 -
     mh2*(msb12 + msb22) + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-2*
     msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(Pi)*power4(sa))/(
     8.*mh2*mH2) + (9*mu2*(-2/mh2 + 2/(-mh2 + mH2) + 3*(mh2 - msb12 - msb22)*
     DeltaInv(msb22,mh2,msb12))*log(mh2/Q2)*power2(sb)*power4(sa))/(4.*power2(
     cb)) + (9*mu2*(-2*mh2 + 4*mH2 + 3*(mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*
     DeltaInv(msb22,mH2,msb12))*log(mH2/Q2)*power2(sb)*power4(sa))/(4.*(mh2 -
     mH2)*mH2*power2(cb)) + (27*mu2*(mH2*(mh2*msb22 - msb12*msb22 - msb24)*
     DeltaInv(msb22,mh2,msb12) + mh2*(mH2*msb22 - msb12*msb22 - msb24)*
     DeltaInv(msb22,mH2,msb12))*log(msb22/Q2)*power2(sb)*power4(sa))/(4.*mh2*
     mH2*power2(cb)) - (9*(mh2*msb22 + msb12*msb22 - msb24)*mu2*DeltaInv(msb22,
     mh2,msb12)*log(mh2/Q2)*log(msb22/Q2)*power2(sb)*power4(sa))/(4.*mh2*
     power2(cb)) - (9*(mH2*msb22 + msb12*msb22 - msb24)*mu2*DeltaInv(msb22,mH2,
     msb12)*log(mH2/Q2)*log(msb22/Q2)*power2(sb)*power4(sa))/(4.*mH2*power2(cb)
     ) + (9*msb12*msb22*mu2*(mH2*DeltaInv(msb22,mh2,msb12) + mh2*DeltaInv(
     msb22,mH2,msb12))*log(msb12/Q2)*log(msb22/Q2)*power2(sb)*power4(sa))/(2.*
     mh2*mH2*power2(cb)) - (9*mu2*log(mh2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(
     msb22,mh2,msb12)*(-(msb12*msb22) - mh2*(msb12 + 2*msb22) + msb24 + power2(
     mh2)))*power2(sb)*power4(sa))/(4.*mh2*power2(cb)) - (9*mu2*Fin20(msb22,
     msb12,Q2)*power2(mh2 - mH2)*power2(sb)*power4(sa))/(4.*power2(cb)*power2(
     mh2)*power2(mH2)) - (9*mu2*(-5*(mh2 + mH2) + 7*mH2*DeltaInv(msb22,mh2,
     msb12)*(-2*msb12*msb22 - mh2*(msb12 + msb22) + power2(mh2)) + 7*mh2*
     DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(
     mH2)))*power2(sb)*power4(sa))/(4.*mh2*mH2*power2(cb)) + (27*mu2*log(msb12/
     Q2)*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(
     msb12 + 2*msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-
     3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(sb)*
     power4(sa))/(4.*mh2*mH2*power2(cb)) - (9*mu2*log(mH2/Q2)*log(msb12/Q2)*(-1
     + DeltaInv(msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12 + 2*msb22) +
     msb24 + power2(mH2)))*power2(sb)*power4(sa))/(4.*mH2*power2(cb)) - (3*mu2*
     (-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 - mh2*(msb12 +
     msb22) + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 -
     mH2*(msb12 + msb22) + power2(mH2)))*power2(Pi)*power2(sb)*power4(sa))/(8.*
     mh2*mH2*power2(cb)) - (9*(-(mH2*mu2) + mh2*(mH2 + 3*mu2) - mh2*(mh2 - mH2)
     *(mh2 - msb12 - msb22)*mu2*DeltaInv(msb22,mh2,msb12) + power2(mh2))*
     power2(log(mh2/Q2))*power4(sa))/(8.*mh2*(mh2 - mH2)) - (9*mu2*(-(1/mh2) +
     2/(-mh2 + mH2) + (mh2 - msb12 - msb22)*DeltaInv(msb22,mh2,msb12))*power2(
     sb)*power2(log(mh2/Q2))*power4(sa))/(8.*power2(cb)) + (9*(mh2*(mH2 - mu2)
     + mH2*(mH2 + 3*mu2) + (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*mu2*DeltaInv(
     msb22,mH2,msb12))*power2(log(mH2/Q2))*power4(sa))/(8.*(mh2 - mH2)*mH2) - (
     9*mu2*(mh2 - 3*mH2 - (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,
     mH2,msb12))*power2(sb)*power2(log(mH2/Q2))*power4(sa))/(8.*mH2*(-mh2 +
     mH2)*power2(cb)) + (9*mu2*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*
     msb12*msb22 - mh2*(msb12 + 2*msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(
     msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(
     mH2)))*power2(log(msb12/Q2))*power4(sa))/(8.*mh2*mH2) - (9*mu2*(-mh2 - mH2
     + mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12 + 2*msb22) +
     msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 -
     mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(sb)*power2(log(msb12/
     Q2))*power4(sa))/(8.*mh2*mH2*power2(cb)) + (9*mu2*(mH2*(mh2*msb22 - msb12*
     msb22 - msb24)*DeltaInv(msb22,mh2,msb12) + mh2*(mH2*msb22 - msb12*msb22 -
     msb24)*DeltaInv(msb22,mH2,msb12))*power2(log(msb22/Q2))*power4(sa))/(8.*
     mh2*mH2) - (9*mu2*(mH2*(mh2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,
     mh2,msb12) + mh2*(mH2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,
     msb12))*power2(sb)*power2(log(msb22/Q2))*power4(sa))/(8.*mh2*mH2*power2(
     cb)) + power2(xb)*(((-3 + 3*mA2*(mA2 - msb12 - msb22)*DeltaInv(mA2,msb22,
     msb12))*Fin3(mA2,msb22,msb12,Q2))/(4.*power2(mA2)) - (9*Fin3(msb22,mH2,
     msb12,Q2)*(-1 + power2(sa))*(mh2 + (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*
     DeltaInv(msb22,mH2,msb12)*(-1 + power2(sa)) - mh2*power2(sa) + mH2*(-1 +
     3*power2(sa))))/(4.*(mh2 - mH2)*power2(mH2)) + (3*(-1 + mC2*(mC2 - msb22 -
     mst12)*DeltaInv(msb22,mst12,mC2))*Fin3(msb22,mst12,mC2,Q2)*(-1 + power2(
     snt)))/(4.*power2(mC2)) - (3*(-1 + mC2*(mC2 - msb22 - mst22)*DeltaInv(
     msb22,mC2,mst22))*Fin3(msb22,mC2,mst22,Q2)*power2(snt))/(4.*power2(mC2)) -
     (9*Fin3(msb22,mh2,msb12,Q2)*(2*mh2*power2(sa) - 3*mh2*power4(sa) + mH2*
     power4(sa) + mh2*(mh2 - mH2)*(mh2 - msb12 - msb22)*DeltaInv(msb22,mh2,
     msb12)*power4(sa)))/(4.*(mh2 - mH2)*power2(mh2)) + (6*(-(1/mA2) + 1/mC2 +
     3/mH2) - (6*(-2 + 3*mA2*(mA2 - msb12 - msb22)*DeltaInv(mA2,msb22,msb12))*
     log(mA2/Q2))/mA2 + (6*(-2 + 3*mC2*(mC2 - msb22 - mst12)*DeltaInv(msb22,
     mst12,mC2))*log(mC2/Q2))/mC2 + (18*(-2 + 3*mH2*(mH2 - msb12 - msb22)*
     DeltaInv(msb22,mH2,msb12))*log(mH2/Q2))/mH2 + (6*(mC2*mH2 - mA2*(3*mC2 +
     mH2))*msb12*log(msb12/Q2))/(mA2*mC2*mH2*(msb12 - msb22)) + (6*(-(mC2*mH2)
     + mA2*(3*mC2 + mH2))*msb22*log(msb22/Q2))/(mA2*mC2*mH2*(msb12 - msb22)) +
     (6*(mA2*msb22 + msb12*msb22 - msb24)*DeltaInv(mA2,msb22,msb12)*log(mA2/Q2)
     *log(msb22/Q2))/mA2 - (18*(mH2*msb22 + msb12*msb22 - msb24)*DeltaInv(
     msb22,mH2,msb12)*log(mH2/Q2)*log(msb22/Q2))/mH2 + (12*msb12*msb22*(-(mH2*
     DeltaInv(mA2,msb22,msb12)) + 3*mA2*DeltaInv(msb22,mH2,msb12))*log(msb12/
     Q2)*log(msb22/Q2))/(mA2*mH2) + (18*(mC2*mst12 - msb22*mst12 - mst14)*
     DeltaInv(msb22,mst12,mC2)*log(mst12/Q2))/mC2 - (6*(mC2*mst12 + msb22*mst12
     - mst14)*DeltaInv(msb22,mst12,mC2)*log(mC2/Q2)*log(mst12/Q2))/mC2 + (12*
     msb22*mst12*DeltaInv(msb22,mst12,mC2)*log(msb22/Q2)*log(mst12/Q2))/mC2 + (
     6*log(mA2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(mA2,msb22,msb12)*(-(msb12*
     msb22) - mA2*(msb12 + 2*msb22) + msb24 + power2(mA2))))/mA2 - (6*Fin20(
     msb22,mst12,Q2))/power2(mC2) - (6*log(mC2/Q2)*log(msb22/Q2)*(-1 +
     DeltaInv(msb22,mst12,mC2)*(-(msb22*mst12) - mC2*(msb22 + 2*mst12) + mst14
     + power2(mC2))))/mC2 + (18*log(msb22/Q2)*(mC2*mH2*(-(mA2*msb22) + msb12*
     msb22 + msb24)*DeltaInv(mA2,msb22,msb12) + mA2*(3*mC2*(mH2*msb22 - msb12*
     msb22 - msb24)*DeltaInv(msb22,mH2,msb12) + mH2*(-1 + DeltaInv(msb22,mst12,
     mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))))))/(
     mA2*mC2*mH2) - 6*Fin20(msb22,msb12,Q2)*(-(1/power2(mA2)) + 3/power2(mH2))
     + (6*(15*mA2*mC2 + 5*mA2*mH2 - 5*mC2*mH2 + 7*mA2*mC2*mH2*msb22*DeltaInv(
     msb22,mst12,mC2) + 7*mA2*mC2*mH2*mst12*DeltaInv(msb22,mst12,mC2) + 14*mA2*
     mH2*msb22*mst12*DeltaInv(msb22,mst12,mC2) + 7*mC2*mH2*DeltaInv(mA2,msb22,
     msb12)*(-2*msb12*msb22 - mA2*(msb12 + msb22) + power2(mA2)) - 7*mA2*mH2*
     DeltaInv(msb22,mst12,mC2)*power2(mC2) - 21*mA2*mC2*DeltaInv(msb22,mH2,
     msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2))))/(mA2*mC2*
     mH2) - (18*log(msb12/Q2)*(3*mA2 - mH2 + mH2*DeltaInv(mA2,msb22,msb12)*(-3*
     msb12*msb22 - mA2*(msb12 + 2*msb22) + msb24 + power2(mA2)) - 3*mA2*
     DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24
     + power2(mH2))))/(mA2*mH2) - (18*log(mH2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(
     msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12 + 2*msb22) + msb24 + power2(
     mH2))))/mH2 + ((3*mA2*mC2 + mA2*mH2 - mC2*mH2 + mA2*mC2*mH2*msb22*
     DeltaInv(msb22,mst12,mC2) + mA2*mC2*mH2*mst12*DeltaInv(msb22,mst12,mC2) +
     2*mA2*mH2*msb22*mst12*DeltaInv(msb22,mst12,mC2) + mC2*mH2*DeltaInv(mA2,
     msb22,msb12)*(-2*msb12*msb22 - mA2*(msb12 + msb22) + power2(mA2)) - mA2*
     mH2*DeltaInv(msb22,mst12,mC2)*power2(mC2) + 3*mA2*mC2*DeltaInv(msb22,mH2,
     msb12)*(2*msb12*msb22 + mH2*(msb12 + msb22) - power2(mH2)))*power2(Pi))/(
     mA2*mC2*mH2) - (36*power2(sa))/mH2 + (72*log(mh2/Q2)*power2(sa))/(mh2 -
     mH2) - (36*log(mH2/Q2)*power2(sa))/(mh2 - mH2) - (36*(-2*mh2 + 3*mH2 + 3*(
     mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*log(mH2/
     Q2)*power2(sa))/((mh2 - mH2)*mH2) + (36*msb12*log(msb12/Q2)*power2(sa))/(
     mH2*msb12 - mH2*msb22) - (36*msb12*log(mh2/Q2)*log(msb12/Q2)*power2(sa))/(
     (mh2 - mH2)*(msb12 - msb22)) + (36*msb12*log(mH2/Q2)*log(msb12/Q2)*power2(
     sa))/((mh2 - mH2)*(msb12 - msb22)) - (36*msb22*log(msb22/Q2)*power2(sa))/(
     mH2*msb12 - mH2*msb22) - (108*(mH2*msb22 - msb12*msb22 - msb24)*DeltaInv(
     msb22,mH2,msb12)*log(msb22/Q2)*power2(sa))/mH2 + (36*msb22*log(mh2/Q2)*
     log(msb22/Q2)*power2(sa))/((mh2 - mH2)*(msb12 - msb22)) - (36*msb22*log(
     mH2/Q2)*log(msb22/Q2)*power2(sa))/((mh2 - mH2)*(msb12 - msb22)) + (36*(
     mH2*msb22 + msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12)*log(mH2/Q2)*
     log(msb22/Q2)*power2(sa))/mH2 - (72*msb12*msb22*DeltaInv(msb22,mH2,msb12)*
     log(msb12/Q2)*log(msb22/Q2)*power2(sa))/mH2 + (36*(mh2 - mH2)*Fin20(msb22,
     msb12,Q2)*power2(sa))/(mh2*power2(mH2)) + (36*(-5 + 7*DeltaInv(msb22,mH2,
     msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(sa))/
     mH2 - (108*log(msb12/Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 -
     mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(sa))/mH2 + (36*log(
     mH2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-(msb12*msb22) -
     mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(sa))/mH2 + (6*(-1 +
     DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(
     mH2)))*power2(Pi)*power2(sa))/mH2 + 18*((mC2 - msb22 - mst22)*DeltaInv(
     msb22,mC2,mst22) + (-mC2 + msb22 + mst12)*DeltaInv(msb22,mst12,mC2))*log(
     mC2/Q2)*power2(snt) - (18*(mC2*mst12 - msb22*mst12 - mst14)*DeltaInv(
     msb22,mst12,mC2)*log(mst12/Q2)*power2(snt))/mC2 + (6*(mC2*mst12 + msb22*
     mst12 - mst14)*DeltaInv(msb22,mst12,mC2)*log(mC2/Q2)*log(mst12/Q2)*power2(
     snt))/mC2 - (12*msb22*mst12*DeltaInv(msb22,mst12,mC2)*log(msb22/Q2)*log(
     mst12/Q2)*power2(snt))/mC2 + (18*(mC2*mst22 - msb22*mst22 - mst24)*
     DeltaInv(msb22,mC2,mst22)*log(mst22/Q2)*power2(snt))/mC2 - (6*(mC2*mst22 +
     msb22*mst22 - mst24)*DeltaInv(msb22,mC2,mst22)*log(mC2/Q2)*log(mst22/Q2)*
     power2(snt))/mC2 + (12*msb22*mst22*DeltaInv(msb22,mC2,mst22)*log(msb22/Q2)
     *log(mst22/Q2)*power2(snt))/mC2 + (6*Fin20(msb22,mst12,Q2)*power2(snt))/
     power2(mC2) - (6*Fin20(msb22,mst22,Q2)*power2(snt))/power2(mC2) - (42*(
     DeltaInv(msb22,mst12,mC2)*(2*msb22*mst12 + mC2*(msb22 + mst12) - power2(
     mC2)) + DeltaInv(msb22,mC2,mst22)*(-2*msb22*mst22 - mC2*(msb22 + mst22) +
     power2(mC2)))*power2(snt))/mC2 + (18*log(msb22/Q2)*(-(DeltaInv(msb22,
     mst12,mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2)))
     + DeltaInv(msb22,mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) +
     mst24 + power2(mC2)))*power2(snt))/mC2 - (6*log(mC2/Q2)*log(msb22/Q2)*(
     DeltaInv(msb22,mst12,mC2)*(msb22*mst12 + mC2*(msb22 + 2*mst12) - mst14 -
     power2(mC2)) + DeltaInv(msb22,mC2,mst22)*(-(msb22*mst22) - mC2*(msb22 + 2*
     mst22) + mst24 + power2(mC2)))*power2(snt))/mC2 - ((DeltaInv(msb22,mst12,
     mC2)*(2*msb22*mst12 + mC2*(msb22 + mst12) - power2(mC2)) + DeltaInv(msb22,
     mC2,mst22)*(-2*msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2)))*power2(
     Pi)*power2(snt))/mC2 + (3*(-1 + mA2*(mA2 - msb12 - msb22)*DeltaInv(mA2,
     msb22,msb12))*power2(log(mA2/Q2)))/mA2 - (3*(-1 + mC2*(mC2 - msb22 -
     mst12)*DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2)))/mC2 + 3*((-mC2 +
     msb22 + mst22)*DeltaInv(msb22,mC2,mst22) + (mC2 - msb22 - mst12)*DeltaInv(
     msb22,mst12,mC2))*power2(snt)*power2(log(mC2/Q2)) + (18*power2(sa)*power2(
     log(mh2/Q2)))/(-mh2 + mH2) - (9*(-1 + mH2*(mH2 - msb12 - msb22)*DeltaInv(
     msb22,mH2,msb12))*power2(log(mH2/Q2)))/mH2 + (18*(mh2 - 2*mH2 - (mh2 -
     mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*power2(sa)*
     power2(log(mH2/Q2)))/(mH2*(-mh2 + mH2)) + (3*(3*mA2 - mH2 + mH2*DeltaInv(
     mA2,msb22,msb12)*(-3*msb12*msb22 - mA2*(msb12 + 2*msb22) + msb24 + power2(
     mA2)) - 3*mA2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*
     msb22) + msb24 + power2(mH2)))*power2(log(msb12/Q2)))/(mA2*mH2) + (18*(-1
     + DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) +
     msb24 + power2(mH2)))*power2(sa)*power2(log(msb12/Q2)))/mH2 - (3*(mC2*mH2*
     (-(mA2*msb22) + msb12*msb22 + msb24)*DeltaInv(mA2,msb22,msb12) + mA2*(3*
     mC2*(mH2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12) + mH2*(-1
     + DeltaInv(msb22,mst12,mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) +
     mst14 + power2(mC2)))))*power2(log(msb22/Q2)))/(mA2*mC2*mH2) + (18*(mH2*
     msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12)*power2(sa)*power2(
     log(msb22/Q2)))/mH2 - (3*(-(DeltaInv(msb22,mst12,mC2)*(-3*msb22*mst12 -
     mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))) + DeltaInv(msb22,mC2,mst22)*
     (-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2)))*power2(
     snt)*power2(log(msb22/Q2)))/mC2 - (3*(mC2*mst12 - msb22*mst12 - mst14)*
     DeltaInv(msb22,mst12,mC2)*power2(log(mst12/Q2)))/mC2 + (3*(mC2*mst12 -
     msb22*mst12 - mst14)*DeltaInv(msb22,mst12,mC2)*power2(snt)*power2(log(
     mst12/Q2)))/mC2 - (3*(mC2*mst22 - msb22*mst22 - mst24)*DeltaInv(msb22,mC2,
     mst22)*power2(snt)*power2(log(mst22/Q2)))/mC2 + (18*(mh2 + mH2)*power4(sa)
     )/(mh2*mH2) + (36*log(mh2/Q2)*power4(sa))/(-mh2 + mH2) + 18*(-2/mh2 + 2/(-
     mh2 + mH2) + 3*(mh2 - msb12 - msb22)*DeltaInv(msb22,mh2,msb12))*log(mh2/
     Q2)*power4(sa) + (36*log(mH2/Q2)*power4(sa))/(mh2 - mH2) + (18*(-2*mh2 +
     4*mH2 + 3*(mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))
     *log(mH2/Q2)*power4(sa))/((mh2 - mH2)*mH2) - (18*(mh2 + mH2)*msb12*log(
     msb12/Q2)*power4(sa))/(mh2*mH2*(msb12 - msb22)) + (36*msb12*log(mh2/Q2)*
     log(msb12/Q2)*power4(sa))/((mh2 - mH2)*(msb12 - msb22)) - (36*msb12*log(
     mH2/Q2)*log(msb12/Q2)*power4(sa))/((mh2 - mH2)*(msb12 - msb22)) + (18*(mh2
     + mH2)*msb22*log(msb22/Q2)*power4(sa))/(mh2*mH2*(msb12 - msb22)) + (54*(
     mH2*(mh2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mh2,msb12) + mh2*(
     mH2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12))*log(msb22/Q2)*
     power4(sa))/(mh2*mH2) + (36*msb22*log(mh2/Q2)*log(msb22/Q2)*power4(sa))/((
     -mh2 + mH2)*(msb12 - msb22)) - (18*(mh2*msb22 + msb12*msb22 - msb24)*
     DeltaInv(msb22,mh2,msb12)*log(mh2/Q2)*log(msb22/Q2)*power4(sa))/mh2 + (36*
     msb22*log(mH2/Q2)*log(msb22/Q2)*power4(sa))/((mh2 - mH2)*(msb12 - msb22))
     - (18*(mH2*msb22 + msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12)*log(mH2/
     Q2)*log(msb22/Q2)*power4(sa))/mH2 + (36*msb12*msb22*(mH2*DeltaInv(msb22,
     mh2,msb12) + mh2*DeltaInv(msb22,mH2,msb12))*log(msb12/Q2)*log(msb22/Q2)*
     power4(sa))/(mh2*mH2) - (18*log(mh2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(
     msb22,mh2,msb12)*(-(msb12*msb22) - mh2*(msb12 + 2*msb22) + msb24 + power2(
     mh2)))*power4(sa))/mh2 - (18*Fin20(msb22,msb12,Q2)*power2(mh2 - mH2)*
     power4(sa))/(power2(mh2)*power2(mH2)) - (18*(-5*(mh2 + mH2) + 7*mH2*
     DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 - mh2*(msb12 + msb22) + power2(
     mh2)) + 7*mh2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 +
     msb22) + power2(mH2)))*power4(sa))/(mh2*mH2) + (54*log(msb12/Q2)*(-mh2 -
     mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12 + 2*
     msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*
     msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power4(sa))/(mh2*
     mH2) - (18*log(mH2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-(
     msb12*msb22) - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power4(sa))/
     mH2 - (3*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 -
     mh2*(msb12 + msb22) + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-2*
     msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(Pi)*power4(sa))/(
     mh2*mH2) - 9*(-(1/mh2) + 2/(-mh2 + mH2) + (mh2 - msb12 - msb22)*DeltaInv(
     msb22,mh2,msb12))*power2(log(mh2/Q2))*power4(sa) - (9*(mh2 - 3*mH2 - (mh2
     - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*power2(log(
     mH2/Q2))*power4(sa))/(mH2*(-mh2 + mH2)) - (9*(-mh2 - mH2 + mH2*DeltaInv(
     msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12 + 2*msb22) + msb24 + power2(
     mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*
     msb22) + msb24 + power2(mH2)))*power2(log(msb12/Q2))*power4(sa))/(mh2*mH2)
     - (9*(mH2*(mh2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mh2,msb12) +
     mh2*(mH2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12))*power2(
     log(msb22/Q2))*power4(sa))/(mh2*mH2))/8.) + xb*((3*mu*sb*(-1 + mA2*(mA2 -
     msb12 - msb22)*DeltaInv(mA2,msb22,msb12))*Fin3(mA2,msb22,msb12,Q2))/(2.*
     cb*power2(mA2)) - (9*mu*sa*Fin3(msb22,mh2,msb12,Q2)*(mh2*(mh2 - mH2)*(mh2
     - msb12 - msb22)*(ca*cb + sa*sb)*DeltaInv(msb22,mh2,msb12)*power2(sa) +
     sa*sb*(mh2*(2 - 3*power2(sa)) + mH2*power2(sa)) + ca*cb*(mh2 - 3*mh2*
     power2(sa) + mH2*power2(sa))))/(2.*cb*(mh2 - mH2)*power2(mh2)) - (9*mu*
     Fin3(msb22,mH2,msb12,Q2)*((mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(
     msb22,mH2,msb12)*(ca*cb*sa + sb*(-1 + power2(sa)))*(-1 + power2(sa)) + ca*
     cb*sa*(mh2 - mh2*power2(sa) + mH2*(-2 + 3*power2(sa))) + sb*(-1 + power2(
     sa))*(mh2 - mh2*power2(sa) + mH2*(-1 + 3*power2(sa)))))/(2.*cb*(mh2 - mH2)
     *power2(mH2)) + (3*mu*(1 + mC2*(-mC2 + msb22 + mst12)*DeltaInv(msb22,
     mst12,mC2))*Fin3(msb22,mst12,mC2,Q2)*(mt2*power2(cb) - (mst12 - mst22)*
     power2(sb)*(-1 + power2(snt))))/(2.*cb*(mst12 - mst22)*sb*power2(mC2)) + (
     3*mu*(1 + mC2*(-mC2 + msb22 + mst22)*DeltaInv(msb22,mC2,mst22))*Fin3(
     msb22,mC2,mst22,Q2)*(mt2*power2(cb) + (-mst12 + mst22)*power2(sb)*power2(
     snt)))/(2.*cb*(-mst12 + mst22)*sb*power2(mC2)) + (mu*((3*(-(1/mA2) + 1/mC2
     + 3/mH2)*sb)/cb - (6*sb*(-2 + 3*mA2*(mA2 - msb12 - msb22)*DeltaInv(mA2,
     msb22,msb12))*log(mA2/Q2))/(cb*mA2) + (6*sb*(-2 + 3*mC2*(mC2 - msb22 -
     mst12)*DeltaInv(msb22,mst12,mC2))*log(mC2/Q2))/(cb*mC2) - (18*cb*mt2*((mC2
     - msb22 - mst22)*DeltaInv(msb22,mC2,mst22) + (-mC2 + msb22 + mst12)*
     DeltaInv(msb22,mst12,mC2))*log(mC2/Q2))/((mst12 - mst22)*sb) + (18*ca*sa*
     log(mh2/Q2))/(mh2 - mH2) + (18*sb*(-2 + 3*mH2*(mH2 - msb12 - msb22)*
     DeltaInv(msb22,mH2,msb12))*log(mH2/Q2))/(cb*mH2) - (18*ca*sa*(-2*mh2 + 3*
     mH2 + 3*(mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*
     log(mH2/Q2))/((mh2 - mH2)*mH2) + (3*(mC2*mH2 - mA2*(3*mC2 + mH2))*msb12*
     sb*log(msb12/Q2))/(cb*mA2*mC2*mH2*(msb12 - msb22)) + (3*(-(mC2*mH2) + mA2*
     (3*mC2 + mH2))*msb22*sb*log(msb22/Q2))/(cb*mA2*mC2*mH2*(msb12 - msb22)) -
     (54*ca*(mH2*msb22 - msb12*msb22 - msb24)*sa*DeltaInv(msb22,mH2,msb12)*log(
     msb22/Q2))/mH2 + (6*(mA2*msb22 + msb12*msb22 - msb24)*sb*DeltaInv(mA2,
     msb22,msb12)*log(mA2/Q2)*log(msb22/Q2))/(cb*mA2) + (18*ca*(mH2*msb22 +
     msb12*msb22 - msb24)*sa*DeltaInv(msb22,mH2,msb12)*log(mH2/Q2)*log(msb22/
     Q2))/mH2 - (18*(mH2*msb22 + msb12*msb22 - msb24)*sb*DeltaInv(msb22,mH2,
     msb12)*log(mH2/Q2)*log(msb22/Q2))/(cb*mH2) - (36*ca*msb12*msb22*sa*
     DeltaInv(msb22,mH2,msb12)*log(msb12/Q2)*log(msb22/Q2))/mH2 + (12*msb12*
     msb22*sb*(-(mH2*DeltaInv(mA2,msb22,msb12)) + 3*mA2*DeltaInv(msb22,mH2,
     msb12))*log(msb12/Q2)*log(msb22/Q2))/(cb*mA2*mH2) + (18*cb*(mC2*mst12 -
     msb22*mst12 - mst14)*mt2*DeltaInv(msb22,mst12,mC2)*log(mst12/Q2))/(mC2*(
     mst12 - mst22)*sb) + (18*(mC2*mst12 - msb22*mst12 - mst14)*sb*DeltaInv(
     msb22,mst12,mC2)*log(mst12/Q2))/(cb*mC2) - (6*cb*(mC2*mst12 + msb22*mst12
     - mst14)*mt2*DeltaInv(msb22,mst12,mC2)*log(mC2/Q2)*log(mst12/Q2))/(mC2*(
     mst12 - mst22)*sb) - (6*(mC2*mst12 + msb22*mst12 - mst14)*sb*DeltaInv(
     msb22,mst12,mC2)*log(mC2/Q2)*log(mst12/Q2))/(cb*mC2) + (12*msb22*mst12*sb*
     DeltaInv(msb22,mst12,mC2)*log(msb22/Q2)*log(mst12/Q2))/(cb*mC2) + (12*cb*
     msb22*mst12*mt2*DeltaInv(msb22,mst12,mC2)*log(msb22/Q2)*log(mst12/Q2))/(
     mC2*mst12*sb - mC2*mst22*sb) - (18*cb*(mC2*mst22 - msb22*mst22 - mst24)*
     mt2*DeltaInv(msb22,mC2,mst22)*log(mst22/Q2))/(mC2*(mst12 - mst22)*sb) + (
     6*cb*(mC2*mst22 + msb22*mst22 - mst24)*mt2*DeltaInv(msb22,mC2,mst22)*log(
     mC2/Q2)*log(mst22/Q2))/(mC2*(mst12 - mst22)*sb) - (12*cb*msb22*mst22*mt2*
     DeltaInv(msb22,mC2,mst22)*log(msb22/Q2)*log(mst22/Q2))/(mC2*mst12*sb -
     mC2*mst22*sb) + (6*sb*log(mA2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(mA2,msb22,
     msb12)*(-(msb12*msb22) - mA2*(msb12 + 2*msb22) + msb24 + power2(mA2))))/(
     cb*mA2) + (6*cb*mt2*Fin20(msb22,mst12,Q2))/((-mst12 + mst22)*sb*power2(
     mC2)) - (6*sb*Fin20(msb22,mst12,Q2))/(cb*power2(mC2)) + (6*cb*mt2*Fin20(
     msb22,mst22,Q2))/((mst12 - mst22)*sb*power2(mC2)) - (6*sb*log(mC2/Q2)*log(
     msb22/Q2)*(-1 + DeltaInv(msb22,mst12,mC2)*(-(msb22*mst12) - mC2*(msb22 +
     2*mst12) + mst14 + power2(mC2))))/(cb*mC2) + (42*cb*mt2*(DeltaInv(msb22,
     mst12,mC2)*(2*msb22*mst12 + mC2*(msb22 + mst12) - power2(mC2)) + DeltaInv(
     msb22,mC2,mst22)*(-2*msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2))))/(
     mC2*(mst12 - mst22)*sb) - (18*cb*mt2*log(msb22/Q2)*(-(DeltaInv(msb22,
     mst12,mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2)))
     + DeltaInv(msb22,mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) +
     mst24 + power2(mC2))))/(mC2*(mst12 - mst22)*sb) + (6*cb*mt2*log(mC2/Q2)*
     log(msb22/Q2)*(DeltaInv(msb22,mst12,mC2)*(msb22*mst12 + mC2*(msb22 + 2*
     mst12) - mst14 - power2(mC2)) + DeltaInv(msb22,mC2,mst22)*(-(msb22*mst22)
     - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2))))/(mC2*(mst12 - mst22)*sb)
     + (18*sb*log(msb22/Q2)*(mC2*mH2*(-(mA2*msb22) + msb12*msb22 + msb24)*
     DeltaInv(mA2,msb22,msb12) + mA2*(3*mC2*(mH2*msb22 - msb12*msb22 - msb24)*
     DeltaInv(msb22,mH2,msb12) + mH2*(-1 + DeltaInv(msb22,mst12,mC2)*(-3*msb22*
     mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))))))/(cb*mA2*mC2*mH2)
     - (6*sb*Fin20(msb22,msb12,Q2)*(-(1/power2(mA2)) + 3/power2(mH2)))/cb + (
     18*ca*(mh2 - mH2)*sa*Fin20(msb22,msb12,Q2))/(mh2*power2(mH2)) + (18*ca*sa*
     (-5 + 7*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) +
     power2(mH2))))/mH2 + (6*sb*(15*mA2*mC2 + 5*mA2*mH2 - 5*mC2*mH2 + 7*mA2*
     mC2*mH2*msb22*DeltaInv(msb22,mst12,mC2) + 7*mA2*mC2*mH2*mst12*DeltaInv(
     msb22,mst12,mC2) + 14*mA2*mH2*msb22*mst12*DeltaInv(msb22,mst12,mC2) + 7*
     mC2*mH2*DeltaInv(mA2,msb22,msb12)*(-2*msb12*msb22 - mA2*(msb12 + msb22) +
     power2(mA2)) - 7*mA2*mH2*DeltaInv(msb22,mst12,mC2)*power2(mC2) - 21*mA2*
     mC2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) +
     power2(mH2))))/(cb*mA2*mC2*mH2) - (54*ca*sa*log(msb12/Q2)*(-1 + DeltaInv(
     msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(
     mH2))))/mH2 - (18*sb*log(msb12/Q2)*(3*mA2 - mH2 + mH2*DeltaInv(mA2,msb22,
     msb12)*(-3*msb12*msb22 - mA2*(msb12 + 2*msb22) + msb24 + power2(mA2)) - 3*
     mA2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) +
     msb24 + power2(mH2))))/(cb*mA2*mH2) + (18*ca*sa*log(mH2/Q2)*log(msb12/Q2)*
     (-1 + DeltaInv(msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12 + 2*msb22) +
     msb24 + power2(mH2))))/mH2 - (18*sb*log(mH2/Q2)*log(msb12/Q2)*(-1 +
     DeltaInv(msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12 + 2*msb22) + msb24
     + power2(mH2))))/(cb*mH2) + (cb*mt2*(DeltaInv(msb22,mst12,mC2)*(2*msb22*
     mst12 + mC2*(msb22 + mst12) - power2(mC2)) + DeltaInv(msb22,mC2,mst22)*(-
     2*msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2)))*power2(Pi))/(mC2*(
     mst12 - mst22)*sb) + (sb*(3*mA2*mC2 + mA2*mH2 - mC2*mH2 + mA2*mC2*mH2*
     msb22*DeltaInv(msb22,mst12,mC2) + mA2*mC2*mH2*mst12*DeltaInv(msb22,mst12,
     mC2) + 2*mA2*mH2*msb22*mst12*DeltaInv(msb22,mst12,mC2) + mC2*mH2*DeltaInv(
     mA2,msb22,msb12)*(-2*msb12*msb22 - mA2*(msb12 + msb22) + power2(mA2)) -
     mA2*mH2*DeltaInv(msb22,mst12,mC2)*power2(mC2) + 3*mA2*mC2*DeltaInv(msb22,
     mH2,msb12)*(2*msb12*msb22 + mH2*(msb12 + msb22) - power2(mH2)))*power2(Pi)
     )/(cb*mA2*mC2*mH2) + (3*ca*sa*(-1 + DeltaInv(msb22,mH2,msb12)*(-2*msb12*
     msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(Pi))/mH2 - (18*sb*
     power2(sa))/(cb*mH2) + (54*sb*log(mh2/Q2)*power2(sa))/(cb*mh2 - cb*mH2) -
     (18*sb*log(mH2/Q2)*power2(sa))/(cb*mh2 - cb*mH2) - (36*sb*(-2*mh2 + 3*mH2
     + 3*(mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*log(
     mH2/Q2)*power2(sa))/(cb*(mh2 - mH2)*mH2) + (18*msb12*sb*log(msb12/Q2)*
     power2(sa))/(cb*mH2*msb12 - cb*mH2*msb22) - (18*msb12*sb*log(mh2/Q2)*log(
     msb12/Q2)*power2(sa))/(cb*(mh2 - mH2)*(msb12 - msb22)) + (18*msb12*sb*log(
     mH2/Q2)*log(msb12/Q2)*power2(sa))/(cb*(mh2 - mH2)*(msb12 - msb22)) - (18*
     msb22*sb*log(msb22/Q2)*power2(sa))/(cb*mH2*msb12 - cb*mH2*msb22) - (108*(
     mH2*msb22 - msb12*msb22 - msb24)*sb*DeltaInv(msb22,mH2,msb12)*log(msb22/
     Q2)*power2(sa))/(cb*mH2) + (18*msb22*sb*log(mh2/Q2)*log(msb22/Q2)*power2(
     sa))/(cb*(mh2 - mH2)*(msb12 - msb22)) - (18*msb22*sb*log(mH2/Q2)*log(
     msb22/Q2)*power2(sa))/(cb*(mh2 - mH2)*(msb12 - msb22)) + (36*(mH2*msb22 +
     msb12*msb22 - msb24)*sb*DeltaInv(msb22,mH2,msb12)*log(mH2/Q2)*log(msb22/
     Q2)*power2(sa))/(cb*mH2) - (72*msb12*msb22*sb*DeltaInv(msb22,mH2,msb12)*
     log(msb12/Q2)*log(msb22/Q2)*power2(sa))/(cb*mH2) + (36*(mh2 - mH2)*sb*
     Fin20(msb22,msb12,Q2)*power2(sa))/(cb*mh2*power2(mH2)) + (36*sb*(-5 + 7*
     DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(
     mH2)))*power2(sa))/(cb*mH2) - (108*sb*log(msb12/Q2)*(-1 + DeltaInv(msb22,
     mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))
     *power2(sa))/(cb*mH2) + (36*sb*log(mH2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(
     msb22,mH2,msb12)*(-(msb12*msb22) - mH2*(msb12 + 2*msb22) + msb24 + power2(
     mH2)))*power2(sa))/(cb*mH2) + (6*sb*(-1 + DeltaInv(msb22,mH2,msb12)*(-2*
     msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(Pi)*power2(sa))/(
     cb*mH2) + (18*sb*((mC2 - msb22 - mst22)*DeltaInv(msb22,mC2,mst22) + (-mC2
     + msb22 + mst12)*DeltaInv(msb22,mst12,mC2))*log(mC2/Q2)*power2(snt))/cb -
     (18*(mC2*mst12 - msb22*mst12 - mst14)*sb*DeltaInv(msb22,mst12,mC2)*log(
     mst12/Q2)*power2(snt))/(cb*mC2) + (6*(mC2*mst12 + msb22*mst12 - mst14)*sb*
     DeltaInv(msb22,mst12,mC2)*log(mC2/Q2)*log(mst12/Q2)*power2(snt))/(cb*mC2)
     - (12*msb22*mst12*sb*DeltaInv(msb22,mst12,mC2)*log(msb22/Q2)*log(mst12/Q2)
     *power2(snt))/(cb*mC2) + (18*(mC2*mst22 - msb22*mst22 - mst24)*sb*
     DeltaInv(msb22,mC2,mst22)*log(mst22/Q2)*power2(snt))/(cb*mC2) - (6*(mC2*
     mst22 + msb22*mst22 - mst24)*sb*DeltaInv(msb22,mC2,mst22)*log(mC2/Q2)*log(
     mst22/Q2)*power2(snt))/(cb*mC2) + (12*msb22*mst22*sb*DeltaInv(msb22,mC2,
     mst22)*log(msb22/Q2)*log(mst22/Q2)*power2(snt))/(cb*mC2) + (6*sb*Fin20(
     msb22,mst12,Q2)*power2(snt))/(cb*power2(mC2)) - (6*sb*Fin20(msb22,mst22,
     Q2)*power2(snt))/(cb*power2(mC2)) - (42*sb*(DeltaInv(msb22,mst12,mC2)*(2*
     msb22*mst12 + mC2*(msb22 + mst12) - power2(mC2)) + DeltaInv(msb22,mC2,
     mst22)*(-2*msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2)))*power2(snt))/
     (cb*mC2) + (18*sb*log(msb22/Q2)*(-(DeltaInv(msb22,mst12,mC2)*(-3*msb22*
     mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))) + DeltaInv(msb22,
     mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2)))
     *power2(snt))/(cb*mC2) - (6*sb*log(mC2/Q2)*log(msb22/Q2)*(DeltaInv(msb22,
     mst12,mC2)*(msb22*mst12 + mC2*(msb22 + 2*mst12) - mst14 - power2(mC2)) +
     DeltaInv(msb22,mC2,mst22)*(-(msb22*mst22) - mC2*(msb22 + 2*mst22) + mst24
     + power2(mC2)))*power2(snt))/(cb*mC2) - (sb*(DeltaInv(msb22,mst12,mC2)*(2*
     msb22*mst12 + mC2*(msb22 + mst12) - power2(mC2)) + DeltaInv(msb22,mC2,
     mst22)*(-2*msb22*mst22 - mC2*(msb22 + mst22) + power2(mC2)))*power2(Pi)*
     power2(snt))/(cb*mC2) + (3*sb*(-1 + mA2*(mA2 - msb12 - msb22)*DeltaInv(
     mA2,msb22,msb12))*power2(log(mA2/Q2)))/(cb*mA2) - (3*sb*(-1 + mC2*(mC2 -
     msb22 - mst12)*DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2)))/(cb*mC2) +
     (3*cb*mt2*((mC2 - msb22 - mst22)*DeltaInv(msb22,mC2,mst22) + (-mC2 + msb22
     + mst12)*DeltaInv(msb22,mst12,mC2))*power2(log(mC2/Q2)))/((mst12 - mst22)*
     sb) - (3*sb*((mC2 - msb22 - mst22)*DeltaInv(msb22,mC2,mst22) + (-mC2 +
     msb22 + mst12)*DeltaInv(msb22,mst12,mC2))*power2(snt)*power2(log(mC2/Q2)))
     /cb + (9*ca*sa*power2(log(mh2/Q2)))/(-mh2 + mH2) - (18*sb*power2(sa)*
     power2(log(mh2/Q2)))/(cb*mh2 - cb*mH2) - (9*sb*(-1 + mH2*(mH2 - msb12 -
     msb22)*DeltaInv(msb22,mH2,msb12))*power2(log(mH2/Q2)))/(cb*mH2) + (9*ca*
     sa*(mh2 - 2*mH2 - (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,
     mH2,msb12))*power2(log(mH2/Q2)))/(mH2*(-mh2 + mH2)) + (18*sb*(mh2 - 2*mH2
     - (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*power2(
     sa)*power2(log(mH2/Q2)))/(cb*mH2*(-mh2 + mH2)) + (9*ca*sa*(-1 + DeltaInv(
     msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(
     mH2)))*power2(log(msb12/Q2)))/mH2 + (3*sb*(3*mA2 - mH2 + mH2*DeltaInv(mA2,
     msb22,msb12)*(-3*msb12*msb22 - mA2*(msb12 + 2*msb22) + msb24 + power2(mA2)
     ) - 3*mA2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*
     msb22) + msb24 + power2(mH2)))*power2(log(msb12/Q2)))/(cb*mA2*mH2) + (18*
     sb*(-1 + DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 - mH2*(msb12 + 2*msb22)
     + msb24 + power2(mH2)))*power2(sa)*power2(log(msb12/Q2)))/(cb*mH2) + (9*
     ca*(mH2*msb22 - msb12*msb22 - msb24)*sa*DeltaInv(msb22,mH2,msb12)*power2(
     log(msb22/Q2)))/mH2 + (3*cb*mt2*(-(DeltaInv(msb22,mst12,mC2)*(-3*msb22*
     mst12 - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))) + DeltaInv(msb22,
     mC2,mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2)))
     *power2(log(msb22/Q2)))/(mC2*(mst12 - mst22)*sb) - (3*sb*(mC2*mH2*(-(mA2*
     msb22) + msb12*msb22 + msb24)*DeltaInv(mA2,msb22,msb12) + mA2*(3*mC2*(mH2*
     msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12) + mH2*(-1 +
     DeltaInv(msb22,mst12,mC2)*(-3*msb22*mst12 - mC2*(msb22 + 2*mst12) + mst14
     + power2(mC2)))))*power2(log(msb22/Q2)))/(cb*mA2*mC2*mH2) + (18*(mH2*msb22
     - msb12*msb22 - msb24)*sb*DeltaInv(msb22,mH2,msb12)*power2(sa)*power2(log(
     msb22/Q2)))/(cb*mH2) - (3*sb*(-(DeltaInv(msb22,mst12,mC2)*(-3*msb22*mst12
     - mC2*(msb22 + 2*mst12) + mst14 + power2(mC2))) + DeltaInv(msb22,mC2,
     mst22)*(-3*msb22*mst22 - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2)))*
     power2(snt)*power2(log(msb22/Q2)))/(cb*mC2) - (3*cb*(mC2*mst12 - msb22*
     mst12 - mst14)*mt2*DeltaInv(msb22,mst12,mC2)*power2(log(mst12/Q2)))/(mC2*(
     mst12 - mst22)*sb) - (3*(mC2*mst12 - msb22*mst12 - mst14)*sb*DeltaInv(
     msb22,mst12,mC2)*power2(log(mst12/Q2)))/(cb*mC2) + (3*(mC2*mst12 - msb22*
     mst12 - mst14)*sb*DeltaInv(msb22,mst12,mC2)*power2(snt)*power2(log(mst12/
     Q2)))/(cb*mC2) + (3*cb*(mC2*mst22 - msb22*mst22 - mst24)*mt2*DeltaInv(
     msb22,mC2,mst22)*power2(log(mst22/Q2)))/(mC2*(mst12 - mst22)*sb) - (3*(
     mC2*mst22 - msb22*mst22 - mst24)*sb*DeltaInv(msb22,mC2,mst22)*power2(snt)*
     power2(log(mst22/Q2)))/(cb*mC2) + 18*ca*(-2/mh2 + 2/(-mh2 + mH2) + 3*(mh2
     - msb12 - msb22)*DeltaInv(msb22,mh2,msb12))*log(mh2/Q2)*power3(sa) + (18*
     ca*(-2*mh2 + 4*mH2 + 3*(mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(
     msb22,mH2,msb12))*log(mH2/Q2)*power3(sa))/((mh2 - mH2)*mH2) + (54*ca*(mH2*
     (mh2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mh2,msb12) + mh2*(mH2*
     msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12))*log(msb22/Q2)*
     power3(sa))/(mh2*mH2) - (18*ca*(mh2*msb22 + msb12*msb22 - msb24)*DeltaInv(
     msb22,mh2,msb12)*log(mh2/Q2)*log(msb22/Q2)*power3(sa))/mh2 - (18*ca*(mH2*
     msb22 + msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12)*log(mH2/Q2)*log(
     msb22/Q2)*power3(sa))/mH2 + (36*ca*msb12*msb22*(mH2*DeltaInv(msb22,mh2,
     msb12) + mh2*DeltaInv(msb22,mH2,msb12))*log(msb12/Q2)*log(msb22/Q2)*
     power3(sa))/(mh2*mH2) - (18*ca*log(mh2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(
     msb22,mh2,msb12)*(-(msb12*msb22) - mh2*(msb12 + 2*msb22) + msb24 + power2(
     mh2)))*power3(sa))/mh2 - (18*ca*Fin20(msb22,msb12,Q2)*power2(mh2 - mH2)*
     power3(sa))/(power2(mh2)*power2(mH2)) - (18*ca*(-5*(mh2 + mH2) + 7*mH2*
     DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 - mh2*(msb12 + msb22) + power2(
     mh2)) + 7*mh2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 +
     msb22) + power2(mH2)))*power3(sa))/(mh2*mH2) + (54*ca*log(msb12/Q2)*(-mh2
     - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12 + 2*
     msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*
     msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power3(sa))/(mh2*
     mH2) - (18*ca*log(mH2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-
     (msb12*msb22) - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power3(sa))/
     mH2 - (3*ca*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 -
     mh2*(msb12 + msb22) + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-2*
     msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(Pi)*power3(sa))/(
     mh2*mH2) - 9*ca*(-(1/mh2) + 2/(-mh2 + mH2) + (mh2 - msb12 - msb22)*
     DeltaInv(msb22,mh2,msb12))*power2(log(mh2/Q2))*power3(sa) - (9*ca*(mh2 -
     3*mH2 - (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*
     power2(log(mH2/Q2))*power3(sa))/(mH2*(-mh2 + mH2)) - (9*ca*(-mh2 - mH2 +
     mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12 + 2*msb22) +
     msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*msb22 -
     mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(log(msb12/Q2))*
     power3(sa))/(mh2*mH2) - (9*ca*(mH2*(mh2*msb22 - msb12*msb22 - msb24)*
     DeltaInv(msb22,mh2,msb12) + mh2*(mH2*msb22 - msb12*msb22 - msb24)*
     DeltaInv(msb22,mH2,msb12))*power2(log(msb22/Q2))*power3(sa))/(mh2*mH2) + (
     9*(mh2 + mH2)*sb*power4(sa))/(cb*mh2*mH2) - (18*sb*log(mh2/Q2)*power4(sa))
     /(cb*mh2 - cb*mH2) + (18*sb*(-2/mh2 + 2/(-mh2 + mH2) + 3*(mh2 - msb12 -
     msb22)*DeltaInv(msb22,mh2,msb12))*log(mh2/Q2)*power4(sa))/cb + (18*sb*log(
     mH2/Q2)*power4(sa))/(cb*mh2 - cb*mH2) + (18*sb*(-2*mh2 + 4*mH2 + 3*(mh2 -
     mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,mH2,msb12))*log(mH2/Q2)*
     power4(sa))/(cb*(mh2 - mH2)*mH2) - (9*(mh2 + mH2)*msb12*sb*log(msb12/Q2)*
     power4(sa))/(cb*mh2*mH2*(msb12 - msb22)) + (18*msb12*sb*log(mh2/Q2)*log(
     msb12/Q2)*power4(sa))/(cb*(mh2 - mH2)*(msb12 - msb22)) - (18*msb12*sb*log(
     mH2/Q2)*log(msb12/Q2)*power4(sa))/(cb*(mh2 - mH2)*(msb12 - msb22)) + (9*(
     mh2 + mH2)*msb22*sb*log(msb22/Q2)*power4(sa))/(cb*mh2*mH2*(msb12 - msb22))
     + (54*sb*(mH2*(mh2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mh2,msb12)
     + mh2*(mH2*msb22 - msb12*msb22 - msb24)*DeltaInv(msb22,mH2,msb12))*log(
     msb22/Q2)*power4(sa))/(cb*mh2*mH2) + (18*msb22*sb*log(mh2/Q2)*log(msb22/
     Q2)*power4(sa))/(cb*(-mh2 + mH2)*(msb12 - msb22)) - (18*(mh2*msb22 +
     msb12*msb22 - msb24)*sb*DeltaInv(msb22,mh2,msb12)*log(mh2/Q2)*log(msb22/
     Q2)*power4(sa))/(cb*mh2) + (18*msb22*sb*log(mH2/Q2)*log(msb22/Q2)*power4(
     sa))/(cb*(mh2 - mH2)*(msb12 - msb22)) - (18*(mH2*msb22 + msb12*msb22 -
     msb24)*sb*DeltaInv(msb22,mH2,msb12)*log(mH2/Q2)*log(msb22/Q2)*power4(sa))/
     (cb*mH2) + (36*msb12*msb22*sb*(mH2*DeltaInv(msb22,mh2,msb12) + mh2*
     DeltaInv(msb22,mH2,msb12))*log(msb12/Q2)*log(msb22/Q2)*power4(sa))/(cb*
     mh2*mH2) - (18*sb*log(mh2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(msb22,mh2,
     msb12)*(-(msb12*msb22) - mh2*(msb12 + 2*msb22) + msb24 + power2(mh2)))*
     power4(sa))/(cb*mh2) - (18*sb*Fin20(msb22,msb12,Q2)*power2(mh2 - mH2)*
     power4(sa))/(cb*power2(mh2)*power2(mH2)) - (18*sb*(-5*(mh2 + mH2) + 7*mH2*
     DeltaInv(msb22,mh2,msb12)*(-2*msb12*msb22 - mh2*(msb12 + msb22) + power2(
     mh2)) + 7*mh2*DeltaInv(msb22,mH2,msb12)*(-2*msb12*msb22 - mH2*(msb12 +
     msb22) + power2(mH2)))*power4(sa))/(cb*mh2*mH2) + (54*sb*log(msb12/Q2)*(-
     mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12 +
     2*msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*
     msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power4(sa))/(cb*mh2*
     mH2) - (18*sb*log(mH2/Q2)*log(msb12/Q2)*(-1 + DeltaInv(msb22,mH2,msb12)*(-
     (msb12*msb22) - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power4(sa))/
     (cb*mH2) - (3*sb*(-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-2*msb12*
     msb22 - mh2*(msb12 + msb22) + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)
     *(-2*msb12*msb22 - mH2*(msb12 + msb22) + power2(mH2)))*power2(Pi)*power4(
     sa))/(cb*mh2*mH2) - (9*sb*(-(1/mh2) + 2/(-mh2 + mH2) + (mh2 - msb12 -
     msb22)*DeltaInv(msb22,mh2,msb12))*power2(log(mh2/Q2))*power4(sa))/cb - (9*
     sb*(mh2 - 3*mH2 - (mh2 - mH2)*mH2*(mH2 - msb12 - msb22)*DeltaInv(msb22,
     mH2,msb12))*power2(log(mH2/Q2))*power4(sa))/(cb*mH2*(-mh2 + mH2)) - (9*sb*
     (-mh2 - mH2 + mH2*DeltaInv(msb22,mh2,msb12)*(-3*msb12*msb22 - mh2*(msb12 +
     2*msb22) + msb24 + power2(mh2)) + mh2*DeltaInv(msb22,mH2,msb12)*(-3*msb12*
     msb22 - mH2*(msb12 + 2*msb22) + msb24 + power2(mH2)))*power2(log(msb12/Q2)
     )*power4(sa))/(cb*mh2*mH2) - (9*sb*(mH2*(mh2*msb22 - msb12*msb22 - msb24)*
     DeltaInv(msb22,mh2,msb12) + mh2*(mH2*msb22 - msb12*msb22 - msb24)*
     DeltaInv(msb22,mH2,msb12))*power2(log(msb22/Q2))*power4(sa))/(cb*mh2*mH2))
     )/4.) + (3*Fin20(mstau12,msb12,Q2)*(6*mstau12*mu2*power2(msb12) - 2*mu2*
     power3(msb12) + msb12*(-2*msb14*mstau12 + 2*msb14*mu2 - 7*mu2*power2(
     mstau12) + power3(mstau12)) - mstau12*(-(msb14*mstau12) + msb14*mu2 - 3*
     mu2*power2(mstau12) + power3(mstau12)) + (mstau12 - mu2)*power6(msb1)))/(
     8.*power2(msb12 - mstau12)*power3(mstau12 - mu2)) + (3*Fin20(mstau22,
     msb12,Q2)*(6*mstau22*mu2*power2(msb12) - 2*mu2*power3(msb12) + msb12*(-2*
     msb14*mstau22 + 2*msb14*mu2 - 7*mu2*power2(mstau22) + power3(mstau22)) -
     mstau22*(-(msb14*mstau22) + msb14*mu2 - 3*mu2*power2(mstau22) + power3(
     mstau22)) + (mstau22 - mu2)*power6(msb1)))/(8.*power2(msb12 - mstau22)*
     power3(mstau22 - mu2)) - (3*log(msb12/Q2)*((4*msb12)/mA2 - (12*msb12)/mH2
     + (3*msb14)/((msb12 - mstau12)*(mstau12 - mu2)) + (3*msb14)/((msb12 -
     mstau22)*(mstau22 - mu2)) + (8*msb12)/(-mstau12 + mu2) + (8*msb12)/(-
     mstau22 + mu2) + (3*msb14)/power2(mstau12 - mu2) - (16*msb12*mu2)/power2(
     mstau12 - mu2) + (3*msb14*mu2)/((msb12 - mstau12)*power2(mstau12 - mu2)) +
     (3*msb14)/power2(mstau22 - mu2) - (16*msb12*mu2)/power2(mstau22 - mu2) + (
     3*msb14*mu2)/((msb12 - mstau22)*power2(mstau22 - mu2)) - (3*power6(msb1))/
     ((msb12 - mstau12)*power2(mstau12 - mu2)) - (3*power6(msb1))/((msb12 -
     mstau22)*power2(mstau22 - mu2))))/16. + (3*Fin20(msb22,mstau12,Q2)*(6*
     mstau12*mu2*power2(msb22) - 2*mu2*power3(msb22) + msb22*(-2*msb24*mstau12
     + 2*msb24*mu2 - 7*mu2*power2(mstau12) + power3(mstau12)) - mstau12*(-(
     msb24*mstau12) + msb24*mu2 - 3*mu2*power2(mstau12) + power3(mstau12)) + (
     mstau12 - mu2)*power6(msb2)))/(8.*power2(msb22 - mstau12)*power3(mstau12 -
     mu2)) + (3*Fin20(mstau22,msb22,Q2)*(6*mstau22*mu2*power2(msb22) - 2*mu2*
     power3(msb22) + msb22*(-2*msb24*mstau22 + 2*msb24*mu2 - 7*mu2*power2(
     mstau22) + power3(mstau22)) - mstau22*(-(msb24*mstau22) + msb24*mu2 - 3*
     mu2*power2(mstau22) + power3(mstau22)) + (mstau22 - mu2)*power6(msb2)))/(
     8.*power2(msb22 - mstau22)*power3(mstau22 - mu2)) - (3*log(msb22/Q2)*((4*
     msb22)/mA2 - (4*msb22)/mC2 - (12*msb22)/mH2 + (12*mt2)/mC2 + (4*msb22)/(
     msntau2 - mu2) + (3*msb24)/((msb22 - mstau12)*(mstau12 - mu2)) + (3*msb24)
     /((msb22 - mstau22)*(mstau22 - mu2)) + (8*msb22)/(-mstau12 + mu2) + (8*
     msb22)/(-mstau22 + mu2) - (12*mt2*DeltaInv(msb22,mC2,mst22)*(-3*msb22*
     mst22 - mC2*(msb22 + 2*mst22) + mst24 + power2(mC2)))/mC2 + (8*msb22*mu2)/
     power2(msntau2 - mu2) + (12*msb24*mt2*mu2*DeltaInv(msb22,mt2,mu2))/power2(
     msntau2 - mu2) + (3*msb24)/power2(mstau12 - mu2) - (16*msb22*mu2)/power2(
     mstau12 - mu2) + (3*msb24*mu2)/((msb22 - mstau12)*power2(mstau12 - mu2)) +
     (3*msb24)/power2(mstau22 - mu2) - (16*msb22*mu2)/power2(mstau22 - mu2) + (
     3*msb24*mu2)/((msb22 - mstau22)*power2(mstau22 - mu2)) + (24*msb24*
     DeltaInv(msb22,mt2,mu2)*power2(mu2))/power2(msntau2 - mu2) + (12*msb22*
     mt2*DeltaInv(msb22,mt2,mu2)*power2(mu2))/power2(msntau2 - mu2) - (12*
     msb22*DeltaInv(msb22,mt2,mu2)*power3(mu2))/power2(msntau2 - mu2) - (12*
     mu2*DeltaInv(msb22,mt2,mu2)*power6(msb2))/power2(msntau2 - mu2) + (3*
     power6(msb2))/((-msb22 + mstau12)*power2(mstau12 - mu2)) + (3*power6(msb2)
     )/((-msb22 + mstau22)*power2(mstau22 - mu2)) - (12*msntau2*DeltaInv(msb22,
     msntau2,mt2)*(msb22*msntau2*(msntau2 - mt2) - msb24*(2*msntau2 + mt2) +
     power6(msb2)))/power2(msntau2 - mu2)))/16. - (3*log(msb22/Q2)*log(mt2/Q2)*
     (-(msb22*(msntau2 + mu2)) + msntau2*DeltaInv(msb22,msntau2,mt2)*(msb22*
     msntau2*(msntau2 - 3*mt2) - msb24*(2*msntau2 + mt2) + power6(msb2)) + mu2*
     DeltaInv(msb22,mt2,mu2)*(msb22*mu2*(-3*mt2 + mu2) - msb24*(mt2 + 2*mu2) +
     power6(msb2))))/(4.*power2(msntau2 - mu2)) + (3*power2(log(mt2/Q2))*(4*
     mC2*msntau2*mt2*mu2 + 2*msb22*msntau2*power2(mC2) + msntau2*mt2*power2(
     mC2) + 2*msb22*mu2*power2(mC2) + mt2*mu2*power2(mC2) - 2*mC2*mt2*power2(
     msntau2) + 2*power2(mC2)*power2(msntau2) + 8*msntau2*mu2*power2(mt2) - 4*
     power2(msntau2)*power2(mt2) - 2*mC2*mt2*power2(mu2) + 2*power2(mC2)*
     power2(mu2) - 4*power2(mt2)*power2(mu2) + 2*mu2*DeltaInv(msb22,mt2,mu2)*
     power2(mC2)*(msb24*(mt2 + mu2) + mu2*((mt2 - mu2)*mu2 + msb22*(6*mt2 +
     mu2)) - power6(msb2)) + 2*DeltaInv(msb22,msntau2,mt2)*power2(mC2)*(msb24*
     msntau2*(msntau2 + mt2) + 2*msntau2*msntau4*mu2 - msntau4*power2(msntau2)
     + 6*msb22*mt2*power2(msntau2) - msntau4*power2(mu2) + power2(msntau2)*
     power2(mu2) + msb22*power3(msntau2) + mt2*power3(msntau2) - 2*mu2*power3(
     msntau2) - msntau2*power6(msb2))))/(16.*power2(mC2)*power2(msntau2 - mu2))
     - (3*log(mt2/Q2)*(4*mC2*msntau2*mt2*mu2 + 3*msb22*msntau2*power2(mC2) + 2*
     msntau2*mt2*power2(mC2) + 3*msb22*mu2*power2(mC2) + 2*mt2*mu2*power2(mC2)
     - 2*mC2*mt2*power2(msntau2) + 3*power2(mC2)*power2(msntau2) + 6*msntau2*
     mu2*power2(mt2) - 3*power2(msntau2)*power2(mt2) - 2*mC2*mt2*power2(mu2) +
     3*power2(mC2)*power2(mu2) - 3*power2(mt2)*power2(mu2) + 3*mu2*DeltaInv(
     msb22,mt2,mu2)*power2(mC2)*(msb24*(mt2 + mu2) + mu2*((mt2 - mu2)*mu2 +
     msb22*(6*mt2 + mu2)) - power6(msb2)) + 3*DeltaInv(msb22,msntau2,mt2)*
     power2(mC2)*(msb24*msntau2*(msntau2 + mt2) + 2*msntau2*msntau4*mu2 -
     msntau4*power2(msntau2) + 6*msb22*mt2*power2(msntau2) - msntau4*power2(
     mu2) + power2(msntau2)*power2(mu2) + msb22*power3(msntau2) + mt2*power3(
     msntau2) - 2*mu2*power3(msntau2) - msntau2*power6(msb2))))/(4.*power2(mC2)
     *power2(msntau2 - mu2)) + (3*log(mst12/Q2)*(4*(msntau2 - mst12)*mst12*
     power2(msntau2 - mu2) + mC2*(mst12*(-8*msntau2*mst12 + 3*mst14 + 8*
     msntau2*mu2 - 8*mst12*mu2 + 8*power2(msntau2)) - 3*power6(mst1))))/(16.*
     mC2*(msntau2 - mst12)*power2(msntau2 - mu2)) - (3*Fin20(mst12,msntau2,Q2)*
     (-((mst14 - 7*mst12*mu2)*power2(msntau2)) - (mst12 + 3*mu2)*power3(
     msntau2) + power4(msntau2) + msntau2*(2*mst12*mst14 + mst14*mu2 - 6*mu2*
     power2(mst12) - power6(mst1)) + mu2*(-2*mst12*mst14 + 2*power3(mst12) +
     power6(mst1))))/(8.*power2(msntau2 - mst12)*power3(msntau2 - mu2)) + (3*
     Fin20(mst12,msntau2,Q2)*power2(snt)*(-((mst14 - 7*mst12*mu2)*power2(
     msntau2)) - (mst12 + 3*mu2)*power3(msntau2) + power4(msntau2) + msntau2*(
     2*mst12*mst14 + mst14*mu2 - 6*mu2*power2(mst12) - power6(mst1)) + mu2*(-2*
     mst12*mst14 + 2*power3(mst12) + power6(mst1))))/(8.*power2(msntau2 -
     mst12)*power3(msntau2 - mu2)) + (3*log(mst12/Q2)*power2(snt)*(12*(mC2*
     mst12 - msb22*mst12 - mst14)*mt2*DeltaInv(msb22,mst12,mC2) + (-4*(msntau2
     - mst12)*mst12*power2(msntau2 - mu2) + mC2*(mst12*(8*msntau2*mst12 - 3*
     mst14 - 8*msntau2*mu2 + 8*mst12*mu2 - 8*power2(msntau2)) + 3*power6(mst1))
     )/((msntau2 - mst12)*power2(msntau2 - mu2))))/(16.*mC2) + (3*log(mst22/Q2)
     *power2(snt)*(-12*(mC2*mst22 - msb22*mst22 - mst24)*mt2*DeltaInv(msb22,
     mC2,mst22) + (4*(msntau2 - mst22)*mst22*power2(msntau2 - mu2) + mC2*(
     mst22*(-8*msntau2*mst22 + 3*mst24 + 8*msntau2*mu2 - 8*mst22*mu2 + 8*
     power2(msntau2)) - 3*power6(mst2)))/((msntau2 - mst22)*power2(msntau2 -
     mu2))))/(16.*mC2) - (3*Fin20(msntau2,mst22,Q2)*power2(snt)*(-((mst24 - 7*
     mst22*mu2)*power2(msntau2)) - (mst22 + 3*mu2)*power3(msntau2) + power4(
     msntau2) + msntau2*(2*mst22*mst24 + mst24*mu2 - 6*mu2*power2(mst22) -
     power6(mst2)) + mu2*(-2*mst22*mst24 + 2*power3(mst22) + power6(mst2))))/(
     8.*power2(msntau2 - mst22)*power3(msntau2 - mu2));

   return result * power2(ytau) * power2(yb) * twoLoop;
}

std::ostream& operator<<(std::ostream& out, const Parameters& pars)
{
   out <<
      "Delta m_tau 2L parameters:\n"
      "yt     = " << pars.yt     << '\n' <<
      "yb     = " << pars.yb     << '\n' <<
      "ytau   = " << pars.ytau   << '\n' <<
      "mt     = " << pars.mt     << '\n' <<
      "mb     = " << pars.mb     << '\n' <<
      "mtau   = " << pars.mtau   << '\n' <<
      "mst1   = " << pars.mst1   << '\n' <<
      "mst2   = " << pars.mst2   << '\n' <<
      "msb1   = " << pars.msb1   << '\n' <<
      "msb2   = " << pars.msb2   << '\n' <<
      "mstau1 = " << pars.mstau1 << '\n' <<
      "mstau2 = " << pars.mstau2 << '\n' <<
      "msntau = " << pars.msntau << '\n' <<
      "xt     = " << pars.xt     << '\n' <<
      "xb     = " << pars.xb     << '\n' <<
      "xtau   = " << pars.xtau   << '\n' <<
      "mw     = " << pars.mw     << '\n' <<
      "mz     = " << pars.mz     << '\n' <<
      "mh     = " << pars.mh     << '\n' <<
      "mH     = " << pars.mH     << '\n' <<
      "mC     = " << pars.mC     << '\n' <<
      "mA     = " << pars.mA     << '\n' <<
      "mu     = " << pars.mu     << '\n' <<
      "tb     = " << pars.tb     << '\n' <<
      "Q      = " << pars.Q      << '\n';

   return out;
}

} // namespace mssm_twoloop_mtau
} // namespace flexiblesusy
